package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2017
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.*;
import com.polytechnik.realtimebook.Trade; // (time,price,shares) container
import com.polytechnik.realtimebook.TraderInterface; // for TIME_SCALE=1000000000L
import com.polytechnik.trading.ImpactQVMMuse;
import com.polytechnik.trading.ImpactQVMMuseLegendreShifted;
import com.polytechnik.trading.ImpactQVMMuseLaguerre;
import com.polytechnik.trading.ImpactQVMMuseMonomials;
import com.polytechnik.trading.ImpactQVMMuse_pi;

/** A reference implementation of dynamic equation non--stationary solution.
 * @version $Id: CallAMuseOfCashFlowAndLiquidityDeficit.java,v 1.66 2022/09/14 00:57:03 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class CallAMuseOfCashFlowAndLiquidityDeficit {

static Skewness getSKshifted(final double offset,final Skewness s)
{
    return new Skewness(s.xa+offset,s.xm+offset,s.sk,s.x1+offset,s.w1,s.x2+offset,s.w2);
}
static String getSkewnessHeader(final String prefix,final int num_start)
{
    return // 6 fields
	"\t"+(num_start)+":"+prefix+".xa"
	+"\t"+(num_start+1)+":"+prefix+".x1"
	+"\t"+(num_start+2)+":"+prefix+".w1"
	+"\t"+(num_start+3)+":"+prefix+".x2"
	+"\t"+(num_start+4)+":"+prefix+".w2"
	+"\t"+(num_start+5)+":"+prefix+".sk";
}
static String getSkewnessData(final Skewness s)
{
    return // 6 fields
	"\t"+(s.xa)
	+"\t"+(s.x1)
	+"\t"+s.w1
	+"\t"+(s.x2)
	+"\t"+s.w2
	+"\t"+s.sk;
}
static String getProbabilityCorrelationHeader(final String prefix,final int num_start)
{
    return // 10 fields
	"\t"+(num_start)+":"+prefix+".pL"
	+"\t"+(num_start+1)+":"+prefix+".pH"
	+"\t"+(num_start+2)+":"+prefix+".iL"
	+"\t"+(num_start+3)+":"+prefix+".iH"
	+"\t"+(num_start+4)+":"+prefix+".corr"
	+"\t"+(num_start+5)+":"+prefix+".w_pL_iL"
	+"\t"+(num_start+6)+":"+prefix+".w_pL_iH"
	+"\t"+(num_start+7)+":"+prefix+".w_pH_iL"
	+"\t"+(num_start+8)+":"+prefix+".w_pH_iH"
	+"\t"+(num_start+9)+":"+prefix+".W"
	;
}
static String getProbabilityCorrelationData(final double p_offset,final ProbabilityCorrelation s)
{
    return // 10 fields
	"\t"+(s.pL+p_offset)
	+"\t"+(s.pH+p_offset)
	+"\t"+s.iL
	+"\t"+s.iH
	+"\t"+s.corr
	+"\t"+s.w_pL_iL
	+"\t"+s.w_pL_iH
	+"\t"+s.w_pH_iL
	+"\t"+s.w_pH_iH
	+"\t"+s.W
	;
}
static int ImpactQVMMuse_outputDataHeader(final java.io.PrintWriter out)
{
    out.println("|1:T\t2:shares\t3:P_read"
		+PrintFields.getEVXDataHeader("I",4)
		+PrintFields.getEVXDataHeader("P",10)
		+getSkewnessHeader("SK_P_IH",16)

		+getSkewnessHeader("pnlss.SK_P_average",22)
		+getSkewnessHeader("pnlss.gSK_P_average",28)
		+getSkewnessHeader("pnlss.gSK_I_average",34)

		+getSkewnessHeader("pnldidsk.SK_spur__nodI",40)
		+getSkewnessHeader("pnldidsk.SK_movaver__nodI",46)
		+getSkewnessHeader("pnldidsk.SKs_W0pi.getSkewnessAtPlast",52)
		+getSkewnessHeader("pnldidsk.SKs_IHf.getSkewnessAtPlast",58)
		+"\t64:pnldidsk.Pf_from_pt_true_pi"
		+"\t65:pnldidsk.deg_from_pt_true_pi"

		+PrintFields.getEVXDataHeader("pnlfutureSk.I",66)
		+"\t72:pnlfutureSk.p_IH"
		+"\t73:pnlfutureSk.P0"
		+"\t74:pnlfutureSk.Pf_SameGammaFuture"
		+"\t75:pnlfutureSk.GammaPast"
		+"\t76:pnlfutureSk.GammaFuture"
		+"\t77:pnlfutureSk.degeneracy"

		);
    return 1;
}

static <RES extends ImpactQVMMuse<?>> int ImpactQVMMuse_outputData(final java.io.PrintWriter out,final long currenttime,final Trade t,final RES r)
{
    out.println(String.valueOf(currenttime)+"\t"+t.shares+"\t"+t.price
		+PrintFields.getEVXDataData(r.I)
		+PrintFields.getEVXDataDataShifted(r.p_offset,r.P)
		+getSkewnessData(getSKshifted(r.p_offset,r.SK_P_IH))

		// simple n=2 case from pnlss
		+getSkewnessData(getSKshifted(r.pnlss.p_offset,r.pnlss.SK_P_average))
		+getSkewnessData(getSKshifted(r.pnlss.p_offset,r.pnlss.gSK_P_average))
		+getSkewnessData(getSKshifted(r.pnlss.p_offset,r.pnlss.gSK_I_average))

		// pnldidsk
		+getSkewnessData(getSKshifted(r.pnldidsk.p_offset,r.pnldidsk.SK_spur__nodI))
		+getSkewnessData(getSKshifted(r.pnldidsk.p_offset,r.pnldidsk.SK_movaver__nodI))
		+getSkewnessData(getSKshifted(r.pnldidsk.p_offset,r.pnldidsk.SKs_W0pi.getSkewnessAtPlast(r.pnldidsk)))
		+getSkewnessData(getSKshifted(r.pnldidsk.p_offset,r.pnldidsk.SKs[r.pnldidsk.I.neq-1].getSkewnessAtPlast(r.pnldidsk)))
		+"\t"+(r.pnldidsk.p_offset+r.pnldidsk.Pf_from_pt_true_pi)
		+"\t"+(r.pnldidsk.deg_from_pt_true_pi)

		// pnlfutureSk
		+PrintFields.getEVXDataData(r.pnlfutureSk.I)
		+"\t"+(r.pnlfutureSk.p_offset+r.pnlfutureSk.p_IH)
		+"\t"+(r.pnlfutureSk.p_offset+r.pnlfutureSk.P0)
		+"\t"+(r.pnlfutureSk.p_offset+r.pnlfutureSk.Pf_SameGammaFuture)
		+"\t"+r.pnlfutureSk.GammaPast
		+"\t"+r.pnlfutureSk.GammaFuture
		+"\t"+r.pnlfutureSk.degeneracy

		);
    return 1;
}

public static class TradeDataProcessorLegendreShiftedBasis extends TradeDataProcessor<ImpactQVMMuseLegendreShifted> {
    public TradeDataProcessorLegendreShiftedBasis(final MuseConfig c){super(c);}
    ImpactQVMMuseLegendreShifted ref=null;
    ImpactQVMMuseLegendreShifted calculateData(final long currenttime,final Trade t)
    {
	// actual trades has >0 shares
	if(t.shares>0) addTradeToBuffer(t);
	final ImpactQVMMuseLegendreShifted r=
	    ImpactQVMMuseLegendreShifted.calculate(c.n,currenttime,
						   c.tau*TraderInterface.TIME_SCALE,
						   buf,n_trades_in_buffer,this.ref);
	this.ref=r;
	return r;
    }
    public int outputDataHeader(final java.io.PrintWriter out) throws java.io.IOException {return ImpactQVMMuse_outputDataHeader(out);}

    public int processOneReadLine(final java.io.PrintWriter out,final long currenttime,final Trade t) throws java.io.IOException{
	return ImpactQVMMuse_outputData(out,currenttime,t,calculateData(currenttime,t));
    }
} // END OF class TradeDataProcessorLegendreShiftedBasis


public static class TradeDataProcessorLaguerreBasis extends TradeDataProcessor<ImpactQVMMuseLaguerre> {
    public TradeDataProcessorLaguerreBasis(final MuseConfig c){super(c);}
    ImpactQVMMuseLaguerre ref=null;
    ImpactQVMMuseLaguerre calculateData(final long currenttime,final Trade t)
    {
	// actual trades has >0 shares
	if(t.shares>0) addTradeToBuffer(t);
	final ImpactQVMMuseLaguerre r=
	    ImpactQVMMuseLaguerre.calculate(c.n,currenttime,
					c.tau*TraderInterface.TIME_SCALE,
					buf,n_trades_in_buffer,this.ref);
	this.ref=r;
	return r;
    }
    public int outputDataHeader(final java.io.PrintWriter out) throws java.io.IOException {return ImpactQVMMuse_outputDataHeader(out);}

    public int processOneReadLine(final java.io.PrintWriter out,final long currenttime,final Trade t) throws java.io.IOException{
	return ImpactQVMMuse_outputData(out,currenttime,t,calculateData(currenttime,t));
    }
} // END OF class TradeDataProcessorLaguerreBasis

public static class TradeDataProcessorMonomialsBasis extends TradeDataProcessor<ImpactQVMMuseMonomials> {
    public TradeDataProcessorMonomialsBasis(final MuseConfig c){super(c);}
    ImpactQVMMuseMonomials ref=null;
    ImpactQVMMuseMonomials calculateData(final long currenttime,final Trade t)
    {
	// actual trades has >0 shares
	if(t.shares>0) addTradeToBuffer(t);
	final ImpactQVMMuseMonomials r=
	    ImpactQVMMuseMonomials.calculate(c.n,currenttime,
					c.tau*TraderInterface.TIME_SCALE,
					buf,n_trades_in_buffer,this.ref);
	this.ref=r;
	return r;
    }
    public int outputDataHeader(final java.io.PrintWriter out) throws java.io.IOException {return ImpactQVMMuse_outputDataHeader(out);}

    public int processOneReadLine(final java.io.PrintWriter out,final long currenttime,final Trade t) throws java.io.IOException{
	return ImpactQVMMuse_outputData(out,currenttime,t,calculateData(currenttime,t));
    }
} // END OF class TradeDataProcessorMonomialsBasis
public static class TradeDataProcessorPiBasis extends TradeDataProcessor<ImpactQVMMuse_pi> {
    public TradeDataProcessorPiBasis(final MuseConfig c){super(c);}
    ImpactQVMMuse_pi ref=null;
    ImpactQVMMuse_pi calculateData(final long currenttime,final Trade t)
    {
	// actual trades has >0 shares
	if(t.shares>0) addTradeToBuffer(t);
	final ImpactQVMMuse_pi r=
	    ImpactQVMMuse_pi.calculate(c.n,currenttime,
				       c.tau*TraderInterface.TIME_SCALE,
				       buf,n_trades_in_buffer,this.ref);
	this.ref=r;
	return r;
    }
    public int outputDataHeader(final java.io.PrintWriter out) throws java.io.IOException {return ImpactQVMMuse_outputDataHeader(out);}
    public int processOneReadLine(final java.io.PrintWriter out,final long currenttime,final Trade t) throws java.io.IOException{
	return ImpactQVMMuse_outputData(out,currenttime,t,calculateData(currenttime,t));
    }
} // END OF class TradeDataProcessorPiBasis

public static void main(final String [] args) throws java.io.IOException
{
    final MuseConfig c=new MuseConfig(args);
    final java.util.List<String> errs=c.validateOptions();
    if(errs.size()>0)
    {
	System.err.println("Invalid or not set arguments:\n"+String.join("\n",errs));
	System.exit(1);
    }
    else if(c.measure.equals("ImpactQVMMuseLegendreShifted"))
    {
	new TradeDataProcessorLegendreShiftedBasis(c).processFiles();
    }
    else if(c.measure.equals("ImpactQVMMuseLaguerre"))
    {
	new TradeDataProcessorLaguerreBasis(c).processFiles();
    }
    else if(c.measure.equals("ImpactQVMMuseMonomials"))
    {
	new TradeDataProcessorMonomialsBasis(c).processFiles();
    }
    else if(c.measure.equals("ImpactQVMMuse_pi"))
    {
	new TradeDataProcessorPiBasis(c).processFiles();
    }
    else throw new RuntimeException("The measure "+c.measure+" is not supported, allowed values:{ImpactQVMMuseLegendreShifted|ImpactQVMMuseLaguerre|ImpactQVMMuseMonomials|ImpactQVMMuse_pi}");
}

} // END OF class CallAMuseOfCashFlowAndLiquidityDeficit
