package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2020
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.Trade; // (time,price,shares) container
import com.polytechnik.realtimebook.TraderInterface; // for TIME_SCALE=1000000000L
import com.polytechnik.trading.PnLILowIHighTPVFactory;
import com.polytechnik.trading.PnLILowIHighTPVFactoryLegendreShifted;
import com.polytechnik.trading.PnLILowIHighTPVFactoryLaguerre;
import com.polytechnik.trading.PnLILowIHighTPVFactoryMonomials;


/** A reference implementation of dynamic equation non--stationary solution.
 * @version $Id: CallAMuseOfCashFlowAndLiquidityDeficitWithPnL.java,v 1.24 2022/09/14 00:57:03 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class CallAMuseOfCashFlowAndLiquidityDeficitWithPnL {

static int PnLILowIHigh_outputDataHeader(final java.io.PrintWriter out)
{
    out.println(
	"|1:T\t2:shares\t3:P_read"
	+"\t4:"+"pi_average"
	+PrintFields.getEVXDataHeader("I",5)
	+"\t11:"+"p_IH"
	+"\t12:"+"sst.getSumPcDIH()"
	+"\t13:"+"DP_since_dIH_H"
	+"\t14:"+"AbsDP_since_dIH_H"

	+"\t15:"+"P_at_dIH_H"
	+"\t16:"+"DP_at_dIH_H"
	+"\t17:"+"AbsDP_at_dIH_H"
	+PrintFields.getEVXDataHeader("dIHpnl",18)
	+"\t24:"+"dP"
	+"\t25:"+"dIH"
		);
    return 1;
}

static int PnLILowIHigh_outputData(final java.io.PrintWriter out,final long currenttime,final Trade t,final PnLILowIHighTPVFactory.PnLILowIHighTPV r)
{
    out.println(String.valueOf(currenttime)+"\t"+t.shares+"\t"+t.price
		+"\t"+r.pi_average
		+PrintFields.getEVXDataData(r.I)
		+"\t"+r.p_IH
		+"\t"+r.sst.getSumPcDIH()
		+"\t"+r.DP_since_dIH_H
		+"\t"+r.AbsDP_since_dIH_H

		+"\t"+r.P_at_dIH_H
		+"\t"+r.DP_at_dIH_H
		+"\t"+r.AbsDP_at_dIH_H
		+PrintFields.getEVXDataData(r.dIHpnl)
		+"\t"+r.dP
		+"\t"+r.dIH
		);
    return 1;
}

static final boolean flag_imitate_zero_shares_call_before_execution=false;

public static class TradeDataProcessorTPV extends TradeDataProcessor<PnLILowIHighTPVFactory.PnLILowIHighTPV> {
    final PnLILowIHighTPVFactory factory;
    PnLILowIHighTPVFactory.PnLILowIHighTPV ref=null;

    public TradeDataProcessorTPV(final PnLILowIHighTPVFactory factory,
				 final MuseConfig c){
	super(c);
	this.factory=factory;
    }
    public int outputDataHeader(final java.io.PrintWriter out) throws java.io.IOException {return PnLILowIHigh_outputDataHeader(out);}
    public int outputDataFooter(final java.io.PrintWriter out) throws java.io.IOException {
	out.println("| calculations done");
	return 1;
    }

    @Override public int processOneReadLine(final java.io.PrintWriter out,final long currenttime,final Trade t) throws java.io.IOException{
	int nl=0;
	if(t.shares>0 && flag_imitate_zero_shares_call_before_execution && this.ref!=null){
	    // an imitation of book event, try to catch liquidity deficit
	    final var tt=createTradeFromTimePriceSharesTraded(currenttime,ref.sst.P,0);
	    nl+=PnLILowIHigh_outputData(out,currenttime,tt,calculateData(currenttime,tt));
	}
	nl+=PnLILowIHigh_outputData(out,currenttime,t,calculateData(currenttime,t));
	return nl;
    }

    PnLILowIHighTPVFactory.PnLILowIHighTPV calculateData(final long currenttime,final Trade t){
	// actual trades has >0 shares
	// for this version no buffer needed (a factory is used) // if(t.shares>0) addTradeToBuffer(t);
	return this.ref=
	    this.factory.getNext(currenttime,t.price,t.shares,this.ref!=null?this.ref.sst:null);
    }
} // END OF class TradeDataProcessorTPV


public static void main(final String [] args) throws java.io.IOException
{
    final MuseConfig c=new MuseConfig(args);
    final java.util.List<String> errs=c.validateOptions();
    if(errs.size()>0)
    {
	System.err.println("Invalid or not set arguments:\n"+String.join("\n",errs));
	System.exit(1);
    }
    else if(c.measure.equals("PnLILowIHighTPVFactoryLegendreShifted"))
    {
	new TradeDataProcessorTPV(
	    new PnLILowIHighTPVFactoryLegendreShifted(c.n,c.tau*TraderInterface.TIME_SCALE),c).processFiles();
    }
    else if(c.measure.equals("PnLILowIHighTPVFactoryLaguerre"))
    {
	new TradeDataProcessorTPV(
	    new PnLILowIHighTPVFactoryLaguerre(c.n,c.tau*TraderInterface.TIME_SCALE),c).processFiles();
    }
    else if(c.measure.equals("PnLILowIHighTPVFactoryMonomials"))
    {
	new TradeDataProcessorTPV(
	    new PnLILowIHighTPVFactoryMonomials(c.n,c.tau*TraderInterface.TIME_SCALE),c).processFiles();
    }
    else throw new RuntimeException("The measure "+c.measure+" is not supported, allowed values:{PnLILowIHighTPVFactoryLegendreShifted,PnLILowIHighTPVFactoryLaguerre,PnLILowIHighTPVFactoryMonomials}");
}

} // END OF class CallAMuseOfCashFlowAndLiquidityDeficitWithPnL
