package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.Trade; // (time,price,shares) container
import com.polytechnik.realtimebook.TraderInterface; // for TIME_SCALE=1000000000L
import com.polytechnik.trading.ScalpedMaxIProjection;
import com.polytechnik.trading.ScalpedMaxIProjectionLegendreShifted;
import com.polytechnik.trading.ScalpedMaxIProjectionLaguerre;
import com.polytechnik.trading.ScalpedMaxIProjectionMonomials;


/** A reference implementation of dynamic equation non--stationary solution.
 * @version $Id: CallAMuseOfCashFlowAndLiquidityDeficitWithScalp.java,v 1.71 2022/09/14 00:57:03 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class CallAMuseOfCashFlowAndLiquidityDeficitWithScalp {

static int ScalpedMaxIProjection_outputDataHeader(final java.io.PrintWriter out)
{
    out.println(
	"|1:T\t2:shares\t3:P_read"
	+"\t4:p_last"
	+"\t5:"+"pi_average"
	+"\t6:"+"pt_average"
	+PrintFields.getEVXDataHeader("I",7)
	+"\t13:"+"dpdt_0"
	+"\t14:"+"p_0"
	+"\t15:"+"pt_0"
	+"\t16:"+"p_IH"
	+"\t17:"+"pt_IH"
	+"\t18:"+"pV_IH"
	+"\t19:"+"pT_IH"
	+"\t20:"+"dpdt_IH"
	+"\t21:"+"var1pI_IH"
	+"\t22:"+"var1pI_IH_00"
	+"\t23:"+"pmin_0_IH"
	+"\t24:"+"pmax_0_IH"
	+"\t25:"+"Skewness_0_IH"
	+"\t26:"+"ProbabilityCorrelation_0_IH"
	+"\t27:"+"getFlFromRegularMoments()"
	+"\t28:"+"Fdt"
	+"\t29:"+"dIH"
	+"\t30:"+"dp_IH"
	+"\t31:"+"getSumFdt()"
	+"\t32:"+"DIR"
	+"\t33:"+"DIRa"
	+"\t34:"+"F_IH"
	+"\t35:"+"Fa_IH"
		);
    return 1;
}

static <RES extends ScalpedMaxIProjection<?>> int ScalpedMaxIProjection_outputData(final java.io.PrintWriter out,final long currenttime,final Trade t,final RES r)
{
    out.println(String.valueOf(currenttime)+"\t"+t.shares+"\t"+t.price
		+"\t"+(r.p_last+r.p_offset)
		+"\t"+(r.pi_average+r.p_offset)
		+"\t"+(r.pt_average+r.p_offset)
		+PrintFields.getEVXDataData(r.I)
		+"\t"+r.dpdt_0
		+"\t"+(r.p_0+r.p_offset)
		+"\t"+(r.pt_0+r.p_offset)
		+"\t"+(r.p_IH+r.p_offset)
		+"\t"+(r.pt_IH+r.p_offset)
		+"\t"+(r.pV_IH+r.p_offset)
		+"\t"+(r.pT_IH+r.p_offset)
		+"\t"+r.dpdt_IH
		+"\t"+r.var1pI_IH
		+"\t"+r.var1pI_IH_00
		+"\t"+(r.pmin_0_IH+r.p_offset)
		+"\t"+(r.pmax_0_IH+r.p_offset)
		+"\t"+r.Skewness_0_IH
		+"\t"+r.ProbabilityCorrelation_0_IH
		+"\t"+r.getFlFromRegularMoments()
		+"\t"+r.Fdt
		+"\t"+r.dIH
		+"\t"+r.dp_IH
		+"\t"+r.sst.getSumFdt()
		+"\t"+r.DIR
		+"\t"+r.DIRa
		+"\t"+r.F_IH
		+"\t"+r.Fa_IH
		);
    return 1;
}


public static abstract class TradeDataProcessorScalpedMaxIProjection<RES extends ScalpedMaxIProjection<?>> extends TradeDataProcessor<RES> {
    public TradeDataProcessorScalpedMaxIProjection(final MuseConfig c){super(c);}
    private long t_start=0;
    RES ref=null;
    public int outputDataHeader(final java.io.PrintWriter out) throws java.io.IOException {t_start=System.currentTimeMillis();return ScalpedMaxIProjection_outputDataHeader(out);}
    public int outputDataFooter(final java.io.PrintWriter out) throws java.io.IOException {
	out.println("| calculations done ref.dp_to_use="+(ref!=null?String.valueOf(ref.dp_to_use):"??")+" in "+(System.currentTimeMillis()-t_start)+" milliseconds");
	return 1;
    }
    public int processOneReadLine(final java.io.PrintWriter out,final long currenttime,final Trade t) throws java.io.IOException{
	return ScalpedMaxIProjection_outputData(out,currenttime,t,calculateData(currenttime,t));
    }

     abstract RES calculateData(final long currenttime,final Trade t);

} // END OF class TradeDataProcessorScalpedMaxIProjection


public static void main(final String [] args) throws java.io.IOException
{
    final MuseConfig c=new MuseConfig(args);
    final java.util.List<String> errs=c.validateOptions();
    if(errs.size()>0)
    {
	System.err.println("Invalid or not set arguments:\n"+String.join("\n",errs));
	System.exit(1);
    }
    else if(c.measure.equals("ScalpedMaxIProjectionLegendreShifted"))
    {
	new TradeDataProcessorScalpedMaxIProjection<ScalpedMaxIProjectionLegendreShifted>(c){
	    ScalpedMaxIProjectionLegendreShifted calculateData(final long currenttime,final Trade t){
		// actual trades has >0 shares
		if(t.shares>0) addTradeToBuffer(t);
		return this.ref=
		    ScalpedMaxIProjectionLegendreShifted.calculate(c.n,currenttime,c.tau*TraderInterface.TIME_SCALE,buf,n_trades_in_buffer,this.ref);
	    }
	}.processFiles();
    }
    else if(c.measure.equals("ScalpedMaxIProjectionLaguerre"))
    {
	new TradeDataProcessorScalpedMaxIProjection<ScalpedMaxIProjectionLaguerre>(c){
	    ScalpedMaxIProjectionLaguerre calculateData(final long currenttime,final Trade t){
		// actual trades has >0 shares
		if(t.shares>0) addTradeToBuffer(t);
		return this.ref=
		    ScalpedMaxIProjectionLaguerre.calculate(c.n,currenttime,c.tau*TraderInterface.TIME_SCALE,buf,n_trades_in_buffer,this.ref);
	    }
	}.processFiles();
    }
    else if(c.measure.equals("ScalpedMaxIProjectionMonomials"))
    {
	new TradeDataProcessorScalpedMaxIProjection<ScalpedMaxIProjectionMonomials>(c){
	    ScalpedMaxIProjectionMonomials calculateData(final long currenttime,final Trade t){
		// actual trades has >0 shares
		if(t.shares>0) addTradeToBuffer(t);
		return this.ref=
		    ScalpedMaxIProjectionMonomials.calculate(c.n,currenttime,c.tau*TraderInterface.TIME_SCALE,buf,n_trades_in_buffer,this.ref);
	    }
	}.processFiles();
    }
    else throw new RuntimeException("The measure "+c.measure+" is not supported, allowed values:{ScalpedMaxIProjectionLegendreShifted|ScalpedMaxIProjectionLaguerre|ScalpedMaxIProjectionMonomials}");
}

} // END OF class CallAMuseOfCashFlowAndLiquidityDeficitWithScalp
