package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.*;

/** A class to store read data.
  * @version $Id: DataReadObservationXF.java,v 1.18 2022/06/28 07:36:58 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
 */
public class DataReadObservationXF {
    public final java.util.ArrayList<ObservationXF> data;
    public final double xmin,xmax,scale,offset;
    public final double BMIN=-1,BMAX=1;
    public final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> Q;

    public DataReadObservationXF(final java.util.ArrayList<ObservationXF> data,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> Q)
    {
	this.data=data;
	this.Q=Q;
	double dxmin=Double.NaN,dxmax=Double.NaN;
	int n=0;
	for(ObservationXF o:data)
	{
	    if(n<=0 || o.x<=dxmin) dxmin=o.x;
	    if(n<=0 || o.x>=dxmax) dxmax=o.x;
	    n++;
	}
	this.xmin=dxmin;
	this.xmax=dxmax;
	this.scale=(BMAX-BMIN)/(xmax-xmin);
	this.offset=BMAX-scale*xmax;
    }
    public double convertOrigToB(final double xorig){return xorig*scale+offset;}
    public double convertBToOrig(final double xB){return (xB-offset)/scale;}

    public static class MomentsRead {
	public final double [] xmoments_analytical,xmoments_sample,dfmoments,fmoments,dfmoments_bypartsfromfmoments;
	MomentsRead(final double [] xmoments_analytical,
		    final double [] xmoments_sample,
		    final double [] dfmoments,
		    final double [] fmoments,
		    final double [] dfmoments_bypartsfromfmoments)
	{
	    this.xmoments_analytical=xmoments_analytical;
	    this.xmoments_sample=xmoments_sample;
	    this.dfmoments=dfmoments;
	    this.fmoments=fmoments;
	    this.dfmoments_bypartsfromfmoments=dfmoments_bypartsfromfmoments;
	}
    }
    public MomentsRead getMoments(final int nmoments)
    {
	if(data.size()<=0) return null;
	final double [] xmoments_sample=new double [nmoments],
	    dfmoments=new double [nmoments],fmoments=new double [nmoments];

	ObservationXF oprev=data.get(0),oxmin=oprev;
	for(int d=data.size(),k=1;k<d;k++)
	{
	    final ObservationXF o=data.get(k);
	    final double xBprev=convertOrigToB(oprev.x),xB=convertOrigToB(o.x);
	    if(xBprev>xB) throw new RuntimeException("Data is not sorted: k="+k+" x="+o.x+" oprev.x="+oprev.x);
	    final double dx=xB-xBprev,xBc=0.5*(xB+xBprev),
		df=(o.f-oprev.f),fc=0.5*(o.f+oprev.f);
	    final double [] Qm=Q.B.getX0Moments(nmoments,xBc);
	    for(int m=0;m<nmoments;m++)
	    {
		xmoments_sample[m]+=Qm[m]*dx;
		dfmoments[m]+=Qm[m]*df;
		fmoments[m]+=Qm[m]*fc*dx;
	    }
	    oprev=o;
	}
	final ObservationXF oxmax=oprev;

	final double Bxmin=convertOrigToB(oxmin.x),Bxmax=convertOrigToB(oxmax.x),fxmin=oxmin.f,fxmax=oxmax.f;
	final double [] xmoments_analytical=new double [nmoments],
	    dfmoments_bypartsfromfmoments=new double [nmoments];
	for(int k=0;k<nmoments;k++)
	{
	    final double [] qk=new double [k+1];qk[k]=1;
	    final double [] iqk=Q.B.getIntegrated(qk),dqk=Q.B.getDifferentiated(qk);
	    xmoments_analytical[k]=Q.B.getD0(iqk,Bxmax)-Q.B.getD0(iqk,Bxmin);
	    dfmoments_bypartsfromfmoments[k]=(fxmax*Q.B.getD0(qk,Bxmax)-fxmin*Q.B.getD0(qk,Bxmin))-TMatr.mulVVn1(dqk,fmoments);
	}

	if(false)
	{
	    System.err.println("xmoments_analytical="+java.util.Arrays.toString(xmoments_analytical));
	    System.err.println("xmoments_sample="+java.util.Arrays.toString(xmoments_sample));
	    System.err.println("dfmoments="+java.util.Arrays.toString(dfmoments));
	    System.err.println("dfmoments_bypartsfromfmoments="+java.util.Arrays.toString(dfmoments_bypartsfromfmoments));
	}
	return new MomentsRead(xmoments_analytical,xmoments_sample,dfmoments,fmoments,dfmoments_bypartsfromfmoments);
    }

    public XXMatrs getSampledMoments(final int nmoments,final java.util.function.DoubleUnaryOperator getFactorToDivide)
    {
	if(data.size()<=0) return null;
	final double [] moments_sample=new double [nmoments],fmoments_sample=new double [nmoments];

	ObservationXF oprev=data.get(0);
	for(int d=data.size(),k=1;k<d;k++)
	{
	    final ObservationXF o=data.get(k);
	    final double xBprev=convertOrigToB(oprev.x),xB=convertOrigToB(o.x);
	    if(xBprev>xB) throw new RuntimeException("Data is not sorted: k="+k+" x="+o.x+" oprev.x="+oprev.x);
	    final double dx=xB-xBprev,xBc=0.5*(xB+xBprev),fc=0.5*(o.f+oprev.f);
	    final double [] Qm=Q.B.getX0Moments(nmoments,xBc);
	    final double divto=getFactorToDivide.applyAsDouble(xBc),sdx=dx/divto;
	    for(int m=0;m<nmoments;m++)
	    {
		final double ws=Qm[m]*sdx;
		moments_sample[m]+=ws;
		fmoments_sample[m]+=ws*fc;
	    }
	    oprev=o;
	}
	return new XXMatrs(1,nmoments,fmoments_sample,moments_sample);
    }

    public void printData()
    {
	for(ObservationXF o:data)
	{
	    System.err.println("x="+o.x+" xscaled="+convertOrigToB(o.x)+" back="+convertBToOrig(convertOrigToB(o.x))+" f="+o.f);
	}
    }

     public static java.util.ArrayList<ObservationXF> getObservations(final String filename)
	throws java.io.IOException
    {
	return getObservations(filename,2,0,1);
    }

    public static java.util.ArrayList<ObservationXF> getObservations(final String filename,final int columns_number,final int indcol_x,final int indcol_f)
	throws java.io.IOException
    {
	final java.util.ArrayList<ObservationXF> data=
	    new java.util.ArrayList<ObservationXF>();
	System.err.print("Opening file="+filename);
	final java.io.BufferedReader br=
            new java.io.BufferedReader(new java.io.FileReader(filename));
        final java.io.StreamTokenizer tk=ReadWriteUtils.getTokenizer("\t ",br);
        tk.quoteChar('\"');
	int n=0;
	for(;;)
	{
	    final String [] s=ReadWriteUtils.readOneLine(tk);
	    if(s==null) break; // end of file
	    n++;
	    //System.err.println("n="+n+" s.length="+s.length+" s="+java.util.Arrays.toString(s));
	    if(s.length<=0||s[0].startsWith("|")) continue; // comments
	    if(s.length!=columns_number) throw new RuntimeException("File="+filename+" line="+n+" records number="+s.length+" is not "+columns_number+" from string="+java.util.Arrays.toString(s));
	    data.add(new ObservationXF(Double.valueOf(s[indcol_x]),Double.valueOf(s[indcol_f])));
	}
	br.close();
	System.err.println(" read "+n+" lines , nrecords="+data.size());
	return data;
    }

} // END OF class DataReadObservationXF
