package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.*;
import com.polytechnik.kgo.*;

/** An interpolation of an image with KGO.
 * @version $Id: ExampleImageInterpolationKGO.java,v 1.19 2024/05/06 16:03:10 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
class ExampleImageInterpolationKGO {


    public static void main(String [] args) throws java.io.IOException,java.net.URISyntaxException {
	if(args.length!=4)
	{
	    System.err.println("usage: ExampleImageInterpolationKGO {file:path/|http:host/}file.png nX nY nF");
	    System.exit(1);
	}
	final int nX=java.lang.Integer.valueOf(args[1]),
	    nY=java.lang.Integer.valueOf(args[2]),
	    nF=java.lang.Integer.valueOf(args[3]);
	final String imageurl=args[0];
	final ImageInterpolationStub E=new ImageInterpolationStub(imageurl);
	E.saveImage(E.getImageFromRaster(E.origraster),(fn)->fn+".same");

	System.err.println("Creating nX="+nX+" nY="+nY+" sample nF="+nF);
	final DataReadObservationVectorXVectorF sample=E.getImageNxNySample(nX,nY,nF);
	System.err.println("Created basis X size="+sample.xHeader.size()+" F size="+sample.fHeader.size());
	for(var SKtype:java.util.List.of(
//		SKFromSampleCalculationMethod.CHRISTOFFEL,
//		SKFromSampleCalculationMethod.CHRISTOFFEL_DEGREESOFFREEDOMADJUSTED,
//		SKFromSampleCalculationMethod.FXFX,
		SKFromSampleCalculationMethod.FXFX_F_CHRISTOFFEL)){
	    final var aList=switch(SKtype) {
	         case CHRISTOFFEL,
	              CHRISTOFFEL_DEGREESOFFREEDOMADJUSTED,
	              FXFX
		   ->java.util.List.of(
		       KGOOptimizationResult.Approximation.MAXEV_NOADJ,
		       KGOOptimizationResult.Approximation.MAXEV_EVADJ,
		       KGOOptimizationResult.Approximation.ITERATIONS_KGOIterationalLinearConstraintsE);
		case FXFX_F_CHRISTOFFEL
		   ->java.util.List.of(
		       KGOOptimizationResult.Approximation.LSQ_NOADJ,
		       KGOOptimizationResult.Approximation.LSQ_EVADJ,
		       KGOOptimizationResult.Approximation.MAXEV_NOADJ,
		       KGOOptimizationResult.Approximation.MAXEV_EVADJ,
		       KGOOptimizationResult.Approximation.ITERATIONS_KGOIterationalLinearConstraintsE,
		       KGOOptimizationResult.Approximation.ITERATIONS_KGOIterationalSimpleOptimizationU,
		       KGOOptimizationResult.Approximation.LISTEV_EVADJ);
	        default -> throw new IllegalArgumentException("SKtype="+SKtype);
	    };
	    for(var approximation:aList){
		System.err.println("Creating approximation="+approximation+" for "+SKtype);
		final var kgo=KGOSolutionVectorXVectorF.createFromSample(sample,approximation,SKtype);
		System.err.println("Output "+approximation+" with SKtype="+SKtype+" for nX="+nX+" x nY="+nY+" basis size="+kgo.nX+" nF="+kgo.nF);
		// print value
		E.saveImage(E.getImageKGOProcessed(kgo,
						   sample,
						   ImageInterpolationStub::getFforEvaluation),
			    (fn)->fn+"."+approximation+"."+SKtype+"."+nX+"."+nY+".bs"+kgo.nX+"nf"+kgo.nF);
		// print correctness probability
		E.saveImage(E.getImageKGOProcessed(kgo,
						   sample,
						   ImageInterpolationStub::getPforEvaluation),
			    (fn)->fn+".P."+approximation+"."+SKtype+"."+nX+"."+nY+".bs"+kgo.nX+"nf"+kgo.nF);
	    }
	}
    }
}
