package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.*;

/** A simple example calculating Radon-Nikodym
 * interpolation for f and Df.
  * @version $Id: ExampleRadonNikodym_F_and_DF.java,v 1.32 2022/06/28 07:40:50 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  */
public class ExampleRadonNikodym_F_and_DF {
    final int order;
    final boolean flag_use_sample_dx;
    ExampleRadonNikodym_F_and_DF(final int order,final boolean flag_use_sample_dx){
	this.order=order;
	this.flag_use_sample_dx=flag_use_sample_dx;
    }

    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> Q=
	new OrthogonalPolynomialsChebyshevBasis()
	//new OrthogonalPolynomialsLegendreBasis()
	//new OrthogonalPolynomialsMonomialsBasis()
	;

    public static String getFPrefix(final String fname)
    {
	return fname.replaceAll("/+","_").replaceAll("[:;+\\s\\\\]+","_")+".";
    }

    static final int MATR_F=101,MATR_DF=102,MATR_DF_BYPARTS=103,MATR_DF_F=104;

    QQMatrices  QQmatr=null;

    /** A class to store QQ matrices, calculated from the data read.
     */
    class QQMatrices {
	final int dim;
	final DataReadObservationXF dr;
	final double [] QQ,xQQ,IQQ,QQf,QQdf,QQdf_byparts,QKQ,QfKQ;
	final double [] Q_moments,f_moments,df_moments,df_moments_byparts;
	QQMatrices(final DataReadObservationXF dr)
	{
	    this.dr=dr;
	    this.dim=order;
	    final DataReadObservationXF.MomentsRead m=dr.getMoments(1+2*dim-1);
	    final double [] xmom=flag_use_sample_dx?m.xmoments_sample:m.xmoments_analytical,
		dfmom=m.dfmoments,fmom=m.fmoments;
	    // QQ=<Q_i(x)Q_j(x)>
	    this.QQ=Q.B.getQQMatr(dim,xmom);
	    // IQQ=(QQ)^-1
	    this.IQQ=Linsystems.getInvertedMatrix(dim,dim,QQ);
	    // xQQ=<xQ_i(x)Q_j(x)>
	    this.xQQ=Q.B.getQQMatr(dim,Q.B.getXQkMomentsFromQkMoments(xmom));
	    // scale xQQ matrix to original x
	    for(int k=0;k<xQQ.length;k++) xQQ[k]=(xQQ[k]-dr.offset*QQ[k])/dr.scale;
	    // QQf=<f Q_i(x)Q_j(x)>
	    this.QQf=Q.B.getQQMatr(dim,fmom);
	    // QQdf=<df/dx Q_i(x)Q_j(x)>
	    this.QQdf=TMatr.scaleVectorInPlace(Q.B.getQQMatr(dim,dfmom),dr.scale); // convert d/dx to proper basis, y=scale*x+offset
	    this.QQdf_byparts=TMatr.scaleVectorInPlace(Q.B.getQQMatr(dim,m.dfmoments_bypartsfromfmoments),dr.scale);
	    this.Q_moments=xmom;
	    this.f_moments=fmom;
	    this.df_moments=TMatr.getVectorScaled(dfmom,dr.scale);
	    this.df_moments_byparts=TMatr.getVectorScaled(m.dfmoments_bypartsfromfmoments,dr.scale);
	    // Christoffel function moments
	    QKQ=Q.B.getQQMatr(dim,dr.getSampledMoments(m.xmoments_sample.length,(xb)->TMatr.getAMatrA(Q.B.getX0Moments(dim,xb),IQQ,dim)).XX);
	    final double [] IQQKf=Linsystems.getInvertedMatrix(dim,dim,QQf);
	    QfKQ=Q.B.getQQMatr(dim,dr.getSampledMoments(m.xmoments_sample.length,(xb)->TMatr.getAMatrA(Q.B.getX0Moments(dim,xb),IQQKf,dim)).XfX);
	}
	void printDataInterpolated(final String filename) throws java.io.IOException
	{
	    System.err.print("Writing test points to "+filename);
	    final java.io.PrintWriter out=new java.io.PrintWriter(filename);
	    out.println("|x\tf_orig\tf_RN\tf_LS\tdf_RN\tdf_LS\tdf_RN_byparts\tdf_LS_byparts");
	    for(ObservationXF o:dr.data)
	    {
		final double [] Q_at_x=Q.B.getX0Moments(dim,dr.convertOrigToB(o.x));
		final double
		    f_RN=RNgetInterporated(Q_at_x,QQf,QQ,IQQ),
		    df_RN=RNgetInterporated(Q_at_x,QQdf,QQ,IQQ),
		    df_RN_byparts=RNgetInterporated(Q_at_x,QQdf_byparts,QQ,IQQ),
		    f_LS=LSgetInterporated(Q_at_x,IQQ,f_moments),
		    df_LS=LSgetInterporated(Q_at_x,IQQ,df_moments),
		    df_LS_byparts=LSgetInterporated(Q_at_x,IQQ,df_moments_byparts);
		out.println(String.valueOf(o.x)+"\t"+o.f+"\t"+f_RN+"\t"+f_LS+"\t"+
			    df_RN+"\t"+df_LS+"\t"+df_RN_byparts+"\t"+df_LS_byparts);
	    }
	    out.close();
	    System.err.println(" DONE");
	}
	void EV_printDataInterpolated(final String filename) throws java.io.IOException
	{
	    System.err.print("Writing test points to "+filename);
	    final double [] ev_QQf_QQ=Q.EV.getGEV_Generalized(dim,QQf,QQ,true),
		ev_QQdf_QQ=Q.EV.getGEV_Generalized(dim,QQdf,QQ,true),
		ev_QQdf_byparts_QQ=Q.EV.getGEV_Generalized(dim,QQdf_byparts,QQ,true);

	    final java.io.PrintWriter out=new java.io.PrintWriter(filename);
	    out.println("|x\tf_orig\tf_RN\tf_LS\tdf_RN\tdf_LS\tdf_RN_byparts\tdf_LS_byparts");
	    for(ObservationXF o:dr.data)
	    {
		final double [] Q_at_x=Q.B.getX0Moments(dim,dr.convertOrigToB(o.x));
		final double f_RN=EV_RNgetInterporated(Q_at_x,ev_QQf_QQ),
		    df_RN=EV_RNgetInterporated(Q_at_x,ev_QQdf_QQ),
		    df_RN_byparts=EV_RNgetInterporated(Q_at_x,ev_QQdf_byparts_QQ),
		    f_LS=EV_LSgetInterporated(Q_at_x,ev_QQf_QQ,Q_moments),
		    df_LS=EV_LSgetInterporated(Q_at_x,ev_QQdf_QQ,Q_moments),
		    df_LS_byparts=EV_LSgetInterporated(Q_at_x,ev_QQdf_byparts_QQ,Q_moments);
		out.println(String.valueOf(o.x)+"\t"+o.f+"\t"+f_RN+"\t"+f_LS+"\t"+
			    df_RN+"\t"+df_LS+"\t"+df_RN_byparts+"\t"+df_LS_byparts);
	    }
	    out.close();
	    System.err.println(" DONE");
	}
	void printSpectrum(final String filename,final int whatmatr) throws java.io.IOException
	{
	    final double [] QQg,QQs,QQK;
	    if(whatmatr==MATR_F)
	    {
		QQg=QQf;QQs=QQ;QQK=QKQ;
	    }
	    else if(whatmatr==MATR_DF)
	    {
		QQg=QQdf;QQs=QQ;QQK=QKQ;
	    }
	    else if(whatmatr==MATR_DF_BYPARTS)
	    {
		QQg=QQdf_byparts;QQs=QQ;QQK=QKQ;
	    }
	    else if(whatmatr==MATR_DF_F)
	    {
		QQg=QQdf;QQs=QQf;QQK=QfKQ;
	    }
	    else throw new IllegalArgumentException("whatmatr="+whatmatr);

	    System.err.print("Writing test points to "+filename);
	    final java.io.PrintWriter out=new java.io.PrintWriter(filename);
	    final double [] ONE=Q.B.getONE(),ev=Q.EV.getGEV_Generalized(dim,QQg,QQs,true);
	    out.println("|n\tev\tx\tw=<psi>^2\tChristoffelWeight");
	    out.println("|sumW\t"+(QQs[0]/dr.scale));
	    out.println("|n\t"+(dim));
	    double sumW=0,sumWk=0;
	    for(int k=0;k<dim;k++)
	    {
		final double xk_orig=TMatr.getAMatrA(dim,ev,dim+k*dim,xQQ,dim)/
		    (QQs==QQ?1:TMatr.getAMatrA(dim,ev,dim+k*dim,QQ,dim)),
		    psiaverOrig=(1/Math.sqrt(dr.scale))*TMatr.getAMatrB(ONE.length,ONE,0,dim,ev,dim+k*dim,QQs,dim),
		    ChristoffelWeight=TMatr.getAMatrA(dim,ev,dim+k*dim,QQK,dim)/dr.scale,
		    Weight=Math.pow(psiaverOrig,2);
		    out.println(String.valueOf(k)+"\t"+ev[k]+"\t"+xk_orig+"\t"+Weight+"\t"+ChristoffelWeight);
		    sumW+=Weight;
		    sumWk+=ChristoffelWeight;
	    }
	    out.println("|SUM\tN/A\tN/A\t"+sumW+"\t"+sumWk);
	    out.close();
	    System.err.println(" DONE");
	}
    } // END OF class QQMatrices

    static double LSgetInterporated(final double [] Q_at_x,final double [] IQQ,final double [] f_moments)
    {
	final int n=Q_at_x.length;
	final double [] qx=TMatr.getMatr_x_A(n,IQQ,Q_at_x);
	return TMatr.mulVVn1(qx,f_moments);
	//return TMatr.getAMatrB(n,Q_at_x,f_moments,IQQ,n);
    }
    static double RNgetInterporated(final double [] Q_at_x,final double [] QQf,final double [] QQ,final double [] IQQ)
    {
	final int n=Q_at_x.length;
	final double [] qx=TMatr.getMatr_x_A(n,IQQ,Q_at_x);
	return TMatr.getAMatrA(qx,QQf,n)/TMatr.getAMatrA(qx,QQ,n);
    }
    /** Calculates sum(psi_k(x)^2 \lambda_k)/sum(psi_k(x)^2)
     */
    static double EV_LSgetInterporated(final double [] Q_at_x,final double [] ev,final double [] Q_moments)
    {
	final int n=Q_at_x.length;
	double s=0;
	for(int k=0;k<n;k++)
	{
	    final double [] psik=EVXData.getPsiFromBasis(k,ev,n);
	    final double psi_at_x=TMatr.mulVV(psik,Q_at_x),
		psi_aver=TMatr.mulVVn1(psik,Q_moments);
	    s+=ev[k]*psi_at_x*psi_aver;
	}
	return s;
    }
    /** Calculates sum(psi_k(x)^2 \lambda_k)/sum(psi_k(x)^2)
     */
    static double EV_RNgetInterporated(final double [] Q_at_x,final double [] ev)
    {
	final int n=Q_at_x.length;
	double snom=0,sdenom=0;
	for(int k=0;k<n;k++)
	{
	    final double [] psik=EVXData.getPsiFromBasis(k,ev,n);
	    final double psi_at_x_squared=Math.pow(TMatr.mulVV(psik,Q_at_x),2);
	    snom+=ev[k]*psi_at_x_squared;
	    sdenom+=psi_at_x_squared;
	}
	return snom/sdenom;
    }
    void readDataFromFile(final String filename)
	throws java.io.IOException
    {
	this.QQmatr=new QQMatrices(new DataReadObservationXF(DataReadObservationXF.getObservations(filename),Q));
    }

    public static void main(final String [] args)
	throws java.io.IOException
    {
	if(args.length!=3)
	{
	    System.err.println("Usage: ExampleRadonNikodym_F_and_DF datafile order {sampleDX|analyticalDX}");
	    System.exit(1);
	}
	final String filename=args[0];
	final int order=Integer.valueOf(args[1]);
	boolean flag_use_sample_dx=false;
	if(args[2].equals("sampleDX")) flag_use_sample_dx=true;
	else if(args[2].equals("analyticalDX")) flag_use_sample_dx=false;
	else throw new RuntimeException("unknown Xsample parameter="+args[2]);

	final String fprefix=getFPrefix(filename);

	final ExampleRadonNikodym_F_and_DF r=new ExampleRadonNikodym_F_and_DF(order,flag_use_sample_dx);
	r.readDataFromFile(filename);
	//r.QQmatr.dr.printData();
	r.QQmatr.printDataInterpolated(fprefix+"RN_interpolated.dat");
	r.QQmatr.EV_printDataInterpolated(fprefix+"EV_RN_interpolated.dat");
	r.QQmatr.printSpectrum(fprefix+"QQf_QQ_spectrum.dat",MATR_F);
	r.QQmatr.printSpectrum(fprefix+"QQdf_QQ_spectrum.dat",MATR_DF);
	r.QQmatr.printSpectrum(fprefix+"QQdfbyparts_QQ_spectrum.dat",MATR_DF_BYPARTS);
	r.QQmatr.printSpectrum(fprefix+"QQdf_QQf_spectrum.dat",MATR_DF_F);
    }


}
