package com.polytechnik.algorithms;
import com.polytechnik.utils.OrthogonalPolynomialsChebyshevBasis;

/** Histogram distribution.
  * @version $Id: HistogramDistribution.java,v 1.8 2018/11/17 22:56:54 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  */
public class HistogramDistribution {
    static double [] minmaxF(final DataReadObservationXF d)
    {
	double min=Double.POSITIVE_INFINITY,max=Double.NEGATIVE_INFINITY;
	for(ObservationXF o: d.data)
	{
	    if(min>=o.f) min=o.f;
	    if(max<=o.f) max=o.f;
	}
	return new double []{min,max};
    }

    static double countF(final double minF,final double maxF,final DataReadObservationXF d,final boolean xcol_is_weight)
    {
	double n=0;
	for(ObservationXF o: d.data)
	{
	    if(minF<=o.f && o.f<maxF){
		if(xcol_is_weight) n+=o.x;
		else n++;
	    }
	}
	//System.err.println("minF="+minF+" maxF="+maxF+" n="+n);
	return n;
    }

    public static void main(String [] args) throws java.io.IOException
    {
	if(args.length!=3)
	{
	    System.err.println("HistogramDistribution inputfile columns_number:indcol_f:indcol_weight(-1 to ignore weight, columns index is base 0) bins_number \n");
	    System.exit(1);
	}
	final String [] cols=com.polytechnik.utils.ReadWriteUtils.extractnames(args[1],":");
	//System.err.println("cols="+java.util.Arrays.toString(cols));
	if(cols.length!=3) throw new RuntimeException("cols="+java.util.Arrays.toString(cols));
	final int columns_number=Integer.valueOf(cols[0]),indcol_f=Integer.valueOf(cols[1]),indcol_weight=Integer.valueOf(cols[2]);
	System.out.println("|"+HistogramDistribution.class.getName()+" "+args[0]+" "+args[1]+" "+args[2]);
	final int indcol_x_weightordummy=indcol_weight>=0?indcol_weight:indcol_f;
	final DataReadObservationXF s=
	    new DataReadObservationXF(DataReadObservationXF.getObservations(args[0],columns_number,indcol_x_weightordummy,indcol_f),new OrthogonalPolynomialsChebyshevBasis());
	final int n=Integer.valueOf(args[2]);
	final double [] r=minmaxF(s);
	double minF=r[0],maxF=r[1],dF=(maxF-minF)/n;
	for(int j=0;j<=n;j++)
	{
	    final double f=minF+j*dF,c=countF(f-dF/2,f+dF/2,s,indcol_weight>=0);
	    System.out.println(String.valueOf(f)+"\t"+c);
	}

    }

}
