package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */

/** A class to read config.
 * @version $Id: MuseConfig.java,v 1.4 2018/11/17 23:28:37 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class MuseConfig {
    public final int indcol_T,indcol_P,indcol_V,columns_number,n;
    public final long tau;
    public final String musein_file,museout_file,measure;

    protected static String getVal(final String a,final String key)
    {
	if(a.startsWith(key))
	{
	    return a.substring(key.length());
	}
	return null;
    }
    protected final java.util.List<String> opts_remained_unprocessed=new java.util.ArrayList<String>();
    public MuseConfig(final String [] args)
    {
	int columns_number=-1,indcol_T=-1,indcol_P=-1,indcol_V=-1,n=-1;
	long tau=-1;
	String musein_file=null,museout_file=null,measure=null;
	for(String a:args)
	{
	    String v=null;
	    if(null!=(v=getVal(a,"--musein_file="))) musein_file=v;
	    else if(null!=(v=getVal(a,"--museout_file="))) museout_file=v;
	    else if(null!=(v=getVal(a,"--measure="))) measure=v;
	    else if(null!=(v=getVal(a,"--musein_cols="))){
		final String [] k=com.polytechnik.utils.ReadWriteUtils.extractnames(v,":");
		if(k.length!=4) throw new RuntimeException("(numcols:T:P:V) colums index="+v+" is invalid. The format is like 15:1:4:5");
		columns_number=Integer.valueOf(k[0]);
		indcol_T=Integer.valueOf(k[1]);
		indcol_P=Integer.valueOf(k[2]);
		indcol_V=Integer.valueOf(k[3]);
	    }
	    else if(null!=(v=getVal(a,"--n="))) n=Integer.valueOf(v);
	    else if(null!=(v=getVal(a,"--tau="))) tau=Long.valueOf(v);
	    else{
		opts_remained_unprocessed.add(a);
	    }
	}
	this.columns_number=columns_number;
	this.indcol_T=indcol_T;
	this.indcol_P=indcol_P;
	this.indcol_V=indcol_V;
	this.n=n;
	this.tau=tau;
	this.musein_file=musein_file;
	this.museout_file=museout_file;
	this.measure=measure;
    }
    public java.util.List<String> validateOptions()
    {
	final java.util.List<String> errs=new java.util.ArrayList<String>();
	if(measure==null) errs.add("no --measure= specified");
	if(musein_file==null) errs.add("no --musein_file= input file specified");
	if(columns_number<=0||indcol_T<0||indcol_P<0||indcol_V<0) errs.add("no --musein_cols=numcols:T:P:V columns numbers specified for the file --musein_file=");
	if(museout_file==null) errs.add("no --museout_file= output file specified");
	if(tau<=0) errs.add("no --tau= is set");
	if(n<=0) errs.add("no --n= is set");
	if(opts_remained_unprocessed.size()>0)
	    errs.add("There are unknown options found:"+opts_remained_unprocessed.toString());
	return errs;
    }
    public String getDescription()
    {
	return "#BEGIN{MuseConfig}\n"+
	    "n="+n+" # basis dimension\n"+
	    "tau="+tau+" # tau in seconds\n"+
	    "musein_file="+musein_file+" # input file name\n"+
	    "musein_cols="+columns_number+":"+indcol_T+":"+indcol_P+":"+indcol_V+" # (numcols:indT:indP:indV) input file="+musein_file+" columns description\n"+
	    "museout_file="+museout_file+" # output file name\n"+
	    "measure="+measure+" # measure (class name)\n"+
	    "#END{MuseConfig}\n";
    }
} // END OF class MuseConfig
