package com.polytechnik.algorithms;

/*
 * (C) Vladislav Malyshkin 2010
 * This file is under GPL version 3.
 *
 */

import java.io.StreamTokenizer;
import com.polytechnik.utils.ReadWriteUtils;
import com.polytechnik.utils.Linsystems;

/** A class for simle optimization tests.
 * @author Vladislav Malyshkin mal@gromco.com
 * @version $Id: OptimizationSeries.java,v 1.39 2022/07/28 21:03:02 mal Exp $
 */
public class OptimizationSeries {
    private static final boolean SIGN_CORR=false;

    int n_points_read=0;
    long [] timepoints=null,timepoints_consequent=null;
    double [][] antecedents=null;
    double [] consequents=null;
    long consequent_shiftahead=0;


    int [] antecedents_indexes;
    int consequent_index=-1,timepoints_index=-1,timepoints_consequent_index=-1;


    String filename;
    int pstart=-1,npoints=-1;
    String [] antecedent_header_fields;
    String consequent_header,timepoints_header,timepoints_consequent_header;
    boolean flag_consequent_separatetimepoints=false;


    final java.util.function.Predicate<String []> HEADER_PROCESSOR=new java.util.function.Predicate<String []>() {
	public boolean test(final String [] b)
	{
	    timepoints_index=ReadWriteUtils.elementIndex(b,timepoints_header);
	    if(timepoints_index<0)
	    {
		System.err.println("No timepoints_header="+timepoints_header+" in header.");
		return false;
	    }

	    if(flag_consequent_separatetimepoints)
	    {
		timepoints_consequent_index=ReadWriteUtils.elementIndex(b,timepoints_consequent_header);
		if(timepoints_consequent_index<0)
		{
		    System.err.println("No timepoints_consequent_header="+timepoints_consequent_header+" in header.");
		    return false;
		}
	    }

	    consequent_index=ReadWriteUtils.elementIndex(b,consequent_header);
	    if(consequent_index<0)
	    {
		System.err.println("No consequent_header="+consequent_header+" in header.");
		return false;
	    }
	    antecedents_indexes=new int [antecedent_header_fields.length];
	    antecedents=new double [antecedents_indexes.length][];
	    for(int i=0;i<antecedents_indexes.length;i++)
	    {
		if(0>(antecedents_indexes[i]=ReadWriteUtils.elementIndex(b,antecedent_header_fields[i])))
		{
		    System.err.println("No header="+antecedent_header_fields[i]+" in header.");
		    return false;
		}
	    }
	    return true;
	}
    };


    final java.util.function.Predicate<String []> BODY_PROCESSOR=new java.util.function.Predicate<String []>() {
	private void checkArraySizes()
	{
	    if(timepoints==null||n_points_read>=timepoints.length)
	    {
		// increase array sizes
		final int nn=10+2*n_points_read;
		final long [] ntp=new long [nn];
		if(timepoints!=null)
		{
		    System.arraycopy(timepoints,0,ntp,0,n_points_read);
		}
		timepoints=ntp;
		if(flag_consequent_separatetimepoints)
		{
		    final long [] ntpc=new long [nn];
		    if(timepoints_consequent!=null)
		    {
			System.arraycopy(timepoints_consequent,0,ntpc,0,n_points_read);
		    }
		    timepoints_consequent=ntpc;
		}
		final double [] nsnsq=new double [nn];
		if(consequents!=null)
		{
		    System.arraycopy(consequents,0,nsnsq,0,n_points_read);
		}
		consequents=nsnsq;

		for(int i=0;i<antecedents.length;i++)
		{
		    final double [] na=new double [nn];
		    if(antecedents[i]!=null)
		    {
			System.arraycopy(antecedents[i],0,na,0,n_points_read);
		    }
		    antecedents[i]=na;
		}
	    }
	}

	public boolean test(final String [] b)
	{
	    checkArraySizes();
	    final int p=n_points_read;
	    timepoints[p]=Long.valueOf(b[timepoints_index]);
	    if(p>0 && timepoints[p]<timepoints[p-1])
	    {
		System.err.println("Timepoints are not sorted");
		return false;
	    }
	    if(flag_consequent_separatetimepoints)
	    {
		timepoints_consequent[p]=Long.valueOf(b[timepoints_consequent_index]);
		//timepoints_consequent[p]=(long)Double.parseDouble(b[timepoints_consequent_index]);
		if(p>0 && timepoints_consequent[p]<timepoints_consequent[p-1])
		{
		    System.err.println("Consequent timepoints are not sorted");
		    return false;
		}
	    }
	    consequents[p]=Double.valueOf(b[consequent_index]);
	    for(int i=0;i<antecedents.length;i++)
	    {
		antecedents[i][p]=Double.valueOf(b[antecedents_indexes[i]]);
	    }
	    n_points_read++;
	    return true;
	}
    };



    void usage()
    {
	System.err.println("OptimizationSeries --filename=/tmp/k.dat --pstart=0 --npoints=900 --timeserieheader='T' {--consshiftahead=64000000000|--consequenttimeserieheader='CT'} --consheader='AAA' F1 F2 F3");
	System.exit(1);
    }


    long getLongfield(final String name,final String a)
    {
	if(a.startsWith(name))
	{
	    return Long.valueOf(a.substring(name.length()));
	}
	throw new RuntimeException("The string "+a+" does not starts with "+name);
    }
    int getIntfield(final String name,final String a)
    {
	if(a.startsWith(name))
	{
	    return Integer.valueOf(a.substring(name.length()));
	}
	throw new RuntimeException("The string "+a+" does not starts with "+name);
    }
    String getStringfield(final String name,final String a)
    {
	if(a.startsWith(name))
	{
	    return a.substring(name.length());
	}
	throw new RuntimeException("The string "+a+" does not starts with "+name);
    }



    void processArguments(final String [] args)
    {
	if(args.length<7)
	{
	    usage();
	    return;
	}

	filename=getStringfield("--filename=",args[0]);
	pstart=getIntfield("--pstart=",args[1]);
	npoints=getIntfield("--npoints=",args[2]);
	timepoints_header=getStringfield("--timeserieheader=",args[3]);
	if(args[4].startsWith("--consshiftahead="))
	{
	    consequent_shiftahead=getLongfield("--consshiftahead=",args[4]);
	    flag_consequent_separatetimepoints=false;
	}
	else
	{
	    timepoints_consequent_header=getStringfield("--consequenttimeserieheader=",args[4]);
	    flag_consequent_separatetimepoints=true;
	}
	consequent_header=getStringfield("--consheader=",args[5]);
	antecedent_header_fields=new String [args.length-6];
	for(int j=antecedent_header_fields.length,i=args.length-1;j-->0;i--)
	{
	    antecedent_header_fields[j]=args[i];
	}

	System.err.println("Args:");
	System.err.println("filename="+filename);
	System.err.println("pstart="+pstart);
	System.err.println("npoints="+npoints);
	System.err.println("timepoints_header="+timepoints_header);
	if(flag_consequent_separatetimepoints)
	{
	    System.err.println("timepoints_consequent_header="+timepoints_consequent_header);
	}
	else
	{
	    System.err.println("consequent_shiftahead="+consequent_shiftahead);
	}
	System.err.println("consequent_header="+consequent_header);
	System.err.println("antecedent_header_fields="+java.util.Arrays.toString(antecedent_header_fields));

    }

    void readData() throws java.io.IOException
    {
	// prepare stream
	final java.io.BufferedReader br=
	    new java.io.BufferedReader(new java.io.FileReader(filename));
	final StreamTokenizer tk=ReadWriteUtils.getTokenizer(",",br);
	tk.quoteChar('\"');

	ReadWriteUtils.processStreamByLines(
	    filename,
	    tk,
	    HEADER_PROCESSOR,
	    BODY_PROCESSOR);
	br.close();
	System.err.println("Read "+n_points_read+" records from "+filename);
    }


    public static void main(final String [] args) throws java.io.IOException
    {

	final OptimizationSeries o=new OptimizationSeries();
	o.processArguments(args);
	o.readData();
	o.linRegrInterpolation();
	o.signInterpolation(1000000L);
    }


    int findPtrCons(final int i0,final int icons0)
    {
	int icons=icons0<i0?i0:icons0;
	boolean cons_found=false;
	if(flag_consequent_separatetimepoints)
	{
	    final long tc=timepoints[i0];
	    for(;icons<n_points_read;)
	    {
		if(timepoints_consequent[icons]>=tc)
		{
		    cons_found=true;
		    break;
		}
		icons++;
	    }
	}
	else
	{
	    final long tc=timepoints[i0]+consequent_shiftahead;
	    for(;icons<n_points_read;)
	    {
		if(timepoints[icons]>=tc)
		{
		    cons_found=true;
		    break;
		}
		icons++;
	    }
	}
	if(!cons_found)
	{
	    throw new RuntimeException("Consequent not found, adjust pstart/npoints");
	}
	return icons;
    }




    void printCorrs(final double [] coefs)
    {
	// print normalized & unnormalized coefs
	System.err.println("-----printCorrs coefs:");
	double an=0,sumcoefs=0;
	for(int j=0;j<coefs.length;j++)
	{
	    sumcoefs+=coefs[j];
	    an+=Math.abs(coefs[j]);
	}
	for(int j=0;j<coefs.length;j++)
	{
	    System.err.println(antecedent_header_fields[j]+"="+(coefs[j]/an)+" (or="+coefs[j]+")");
	}
	// calculate & print correlations
	final int npe=pstart+npoints>n_points_read?n_points_read:pstart+npoints;
	double c_a_cov=0,aver_c=0,aver_a=0,aver_c_var2=0,aver_a_var2=0,n=0,
	    sign_c_a_cov=0,aver_sign_a=0,aver_sign_c=0,aver_sign_a_var2=0,aver_sign_c_var2=0;
	for(int i=pstart,icons=i;i<npe;i++)
	{
	    // find consequent index
	    icons=findPtrCons(i,icons);
	    //System.err.println("I="+i+" icons="+icons+" dt="+((timepoints[icons]-timepoints[i])/1e9));
	    final double c=SIGN_CORR?Math.signum(consequents[icons]):consequents[icons];
	    double a=0;
	    for(int j=0;j<coefs.length;j++)
	    {
		a+=coefs[j]*antecedents[j][i];
	    }
	    c_a_cov+=a*c;
	    aver_c+=c;
	    aver_a+=a;
	    aver_c_var2+=c*c;
	    aver_a_var2+=a*a;

	    final double sa=Math.signum(a),sc=Math.signum(c);
	    sign_c_a_cov+=sa*sc;
	    aver_sign_c+=sc;
	    aver_sign_a+=sa;
	    aver_sign_c_var2+=sc*sc;
	    aver_sign_a_var2+=sc*sc;
	    n++;
	}
	c_a_cov/=n;
	aver_c/=n;
	aver_a/=n;
	aver_c_var2/=n;
	aver_c_var2=Math.sqrt(aver_c_var2-aver_c*aver_c);
	aver_a_var2/=n;
	aver_a_var2=Math.sqrt(aver_a_var2-aver_a*aver_a);
	final double c_a_corr=(c_a_cov-aver_a*aver_c)/(aver_c_var2*aver_a_var2);

	sign_c_a_cov/=n;
	aver_sign_c/=n;
	aver_sign_a/=n;
	aver_sign_c_var2/=n; // =1
	aver_sign_c_var2=Math.sqrt(aver_sign_c_var2-aver_sign_c*aver_sign_c);
	aver_sign_a_var2/=n; // =1
	aver_sign_a_var2=Math.sqrt(aver_sign_a_var2-aver_sign_a*aver_sign_a);
	final double c_a_sign_corr=(sign_c_a_cov-aver_sign_a*aver_sign_c)/(aver_sign_c_var2*aver_sign_a_var2);


	System.err.println("-----printCorrs N_POINTS="+n+" sumcoefs="+sumcoefs+" sumabscoefs="+an);

	System.err.println("c_a_correlation="+c_a_corr);
	System.err.println("\taver_c="+aver_c);
	System.err.println("\taver_a="+aver_a);

	System.err.println("c_a_sign_correlation="+c_a_sign_corr+" sign_noaver="+sign_c_a_cov);
	System.err.println("\taver_sign_c="+aver_sign_c);
	System.err.println("\taver_sign_a="+aver_sign_a);
    }





    void linRegrInterpolation()
    {
	System.err.println("================ linRegrInterpolation() $Id: OptimizationSeries.java,v 1.39 2022/07/28 21:03:02 mal Exp $");

	final int nt=antecedents_indexes.length;
	final double [][] matr=new double [nt][nt];
	final double [] rightpart=new double [nt];
	final int npe=pstart+npoints>n_points_read?n_points_read:pstart+npoints;

	for(int i=pstart,icons=i;i<npe;i++)
	{
	    // find consequent index
	    icons=findPtrCons(i,icons);
	    //System.err.println("Cons="+icons+" antcd="+i);

	    final double c=SIGN_CORR?Math.signum(consequents[icons]):consequents[icons];

	    for(int j1=0;j1<nt;j1++)
	    {
		final double [] a=antecedents[j1];
		rightpart[j1]+=c*a[i];
		for(int j2=0;j2<nt;j2++)
		{
		    matr[j1][j2]+=a[i]*antecedents[j2][i];
		}
	    }
	}


	// solve linear system
	final int [] ipvt=new int [nt];
	final int info=Linsystems.dgefa(matr,nt,ipvt);
	if(info!=0)
	{
	    throw new RuntimeException("INFO="+info);
	}
	Linsystems.dgesl(matr,nt,ipvt,rightpart,1);
	// linear system solution is now saved in rightpart

	printCorrs(rightpart);
    }



    double get_signA_signC(final double [] coefs)
    {
	final int npe=pstart+npoints>n_points_read?n_points_read:pstart+npoints;
	double sa_sign=0,n=0;
	for(int i=pstart,icons=i;i<npe;i++)
	{
	    // find consequent index
	    icons=findPtrCons(i,icons);

	    final double c=consequents[icons];
	    double a=0;
	    for(int j=0;j<coefs.length;j++)
	    {
		a+=coefs[j]*antecedents[j][i];
	    }
	    sa_sign+=Math.signum(a)*Math.signum(c);
	    n++;
	}
	return sa_sign/n;
    }


   /** Create random vector with sum(coefs)=0, sum(abs(coefs))=1.
    */
    void setRandomValues(final double [] coefs,
			 final java.util.Random r)
    {
	final int nt=coefs.length;
	double s=0,as=0;
	for(int j=1;j<nt;j++)
	{
	    coefs[j]=2*r.nextDouble()-1;
	    s+=coefs[j];
	    as+=Math.abs(coefs[j]);
	}
	coefs[0]=-s;
	as+=Math.abs(s);
	for(int j=0;j<nt;j++)
	{
	    coefs[j]/=as;
	}
    }




    void signInterpolation(final long n_trials)
    {
	System.err.println("================ signInterpolation() $Id: OptimizationSeries.java,v 1.39 2022/07/28 21:03:02 mal Exp $");
	final long t0=System.currentTimeMillis();
	final int nt=antecedents.length;
	if(nt<=2)
	{
	    throw new RuntimeException("More than 2 antecedents required.");
	}
	final java.util.Random r=new java.util.Random(-179091);
	final double [] sol=new double [nt],sc=new double [nt];
	double m=Double.NaN;
	for(long i=0;i<n_trials;i++)
	{

	    // create random vector.
	    setRandomValues(sc,r);

	    // get sign match values
	    final double v=get_signA_signC(sc);
	    if(!(v<m))
	    { // save sc as the solution sol
		m=v;
		System.arraycopy(sc,0,sol,0,nt);
	    }
	}
	// Output the solution
	System.err.println("---- trials="+n_trials+" m="+m);

	printCorrs(sol);
	final long t1=System.currentTimeMillis();
	System.err.println("signInterpolation() "+n_trials+" trials done in "+(t1-t0)+" milliseconds.");
    }


}
