package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2024
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.*;
import java.io.File;

/** Print a maping of polynomial.
  * @version $Id: PrintChebyshevToLegendreMapping.java,v 1.3 2024/05/08 04:53:33 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  */
public class PrintChebyshevToLegendreMapping {


    final int maxChebyshev,maxLegendre;
    final int nPoints;

    final BasisPolynomials CB=new Chebyshev();
    final BasisPolynomials PB=new Legendre();

    PrintChebyshevToLegendreMapping(final int maxChebyshev,
				    final int maxLegendre,
				    final int nPoints){
	this.maxChebyshev=maxChebyshev;
	this.maxLegendre=maxLegendre;
	this.nPoints=nPoints;
    }


    public void printSeq(final String filename,
			 final java.util.Random r) throws java.io.IOException {
	final var out=new java.io.PrintWriter(filename);
	final var s=DataReadObservationVectorXVectorF.SEPARATOR;
	System.err.println("Writing "+nPoints+" points to "+filename);

	out.println("| Output Chebyshev & Legendre on "+nPoints+" steps");
	out.println("| maxChebyshev:"+maxChebyshev);
	out.println("| maxLegendre:"+maxLegendre);
	out.println("|");
	out.println("| Header:");
	int o=0;
	out.print("|N:"+(o++));
	out.print(s+"X:"+(o++));
	for(int i=0;i<=maxChebyshev;i++){
	    out.print(s+"T"+i+":"+(o++));
	}
	out.print(s+"Trandom_sign:"+(o++));
	for(int i=0;i<=maxLegendre;i++){
	    out.print(s+"P"+i+":"+(o++));
	}
	out.println(s+"Prandom_sign:"+(o++));
	//out.println(s+"W:"+(o++));

	for(int l=0;l<=nPoints;l++){
	    final double x=-1+(2.0*l)/nPoints;
	    out.print(String.valueOf(l));
	    out.print(s+x);
	    final double sT=r.nextBoolean()?1:-1,
		sP=r.nextBoolean()?1:-1;
	    final double [] T=TMatr.getVectorScaled(CB.getX0Moments(1+maxChebyshev,x),sT),
		P=TMatr.getVectorScaled(PB.getX0Moments(1+maxLegendre,x),sP);
	    for(var v:T){
		out.print(s+v);
	    }
	    out.print(s+sT);
	    for(var v:P){
		out.print(s+v);
	    }
	    out.print(s+sP);
	    out.println();
	}
	out.close();
    }

    public static void main(String [] args) throws java.io.IOException {

	// for random signs
	final var r=new java.util.Random(123);
	new PrintChebyshevToLegendreMapping
	    (10,//maxChebyshev,
	     5,//maxLegendre,
	     500//nPoints
	     ).printSeq("/tmp/ChebyshevLegendre.csv",r);
    }


}
