package com.polytechnik.algorithms;

import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.*;


/** Visually check Runge effect on edges.
 * $Id: PrintChristoffel.java,v 1.12 2022/06/27 07:19:10 mal Exp $
 */
class PrintChristoffel {

    static final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable>
	T=new OrthogonalPolynomialsChebyshevBasis();

    public static void main(String [] args)
    {
	if(args.length<=0)
	{
	    System.err.println("usage: PrintChristoffel 7");
	    System.exit(1);
	}
	final int n=Integer.valueOf(args[0]);
	final double [] moms_P=new double [2*n-1],moms_T=new double [2*n-1],moms_U=new double [2*n-1];
	moms_T[0]=Math.PI;
	for(int k=0;k<moms_U.length;k++){
	    final double [] tk=new double [k+1];tk[k]=1;
	    final double [] itk=T.B.getIntegrated(tk);
	    // \int_{-1}^{1}T_k(x)sqrt(1-x*x)dx
	    moms_U[k]=T.B.getaxbP(itk,1,0)[0]*moms_T[0];
	    // \int_{-1}^{1}T_k(x)dx
	    moms_P[k]=T.B.getD0(itk,1)-T.B.getD0(itk,-1);
	}
	final double [] OK_P=T.getChristoffelOFunctionFromMoments(n,moms_P),
	    OK_T=T.getChristoffelOFunctionFromMoments(n,moms_T),
	    OK_U=T.getChristoffelOFunctionFromMoments(n,moms_U);
	if(OK_P.length!=1+2*(n-1) || OK_P.length!=OK_T.length || OK_P.length!=OK_U.length) throw new IllegalArgumentException();
	if(true){
	    // diag
	    System.err.println("moms_U="+java.util.Arrays.toString(moms_U));
	    final Quadrature.GaussQuadratureWithEVData qu=T.getGaussQuadratureFromMoments(n-1,moms_U),
		qt=T.getGaussQuadratureFromMoments(n-1,moms_T);
	    System.err.println("U sum w="+TMatr.sumElements(qu.w));
	    System.err.println("Tx="+java.util.Arrays.toString(qt.x)+" w="+java.util.Arrays.toString(qt.w)+" K(x)="+java.util.Arrays.stream(qt.x).mapToObj(x->String.valueOf(1/T.B.getD0(OK_T,x))).toList());
	    final double [][] p=T.getOrthogonalPolynomialsFirstKind(n,moms_U);
	    for(int k=0;k<p.length;k++){
		System.err.println("U"+k+"="+java.util.Arrays.toString(TMatr.scaleVectorInPlace(T.B.convertFunctionTo_Monomials_from_Basis(p[k]),Math.sqrt(Math.PI/2))));
	    }
	}

	System.out.println("|x,KP,KT,KU");
	for(int t=0,nt=10000;t<=nt;t++){
	    final double x=-1+(2.0*t)/nt;
	    System.out.println(String.valueOf(x)+","+(1/T.B.getD0(OK_P,x))+","+(1/T.B.getD0(OK_T,x))+","+(1/T.B.getD0(OK_U,x)));
	}
    }
}
