package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.Trade; // (time,price,shares) container
import com.polytechnik.realtimebook.TraderInterface; // for TIME_SCALE=1000000000L
import com.polytechnik.trading.IntervalSincePsiM;
import com.polytechnik.trading.PsiMSelection;
import com.polytechnik.trading.DaDv;
import com.polytechnik.trading.TwoTopI;
import com.polytechnik.trading.SplitdIHdt;
import com.polytechnik.trading.ExperimentaldIHdt;
import com.polytechnik.utils.EVXData;
import com.polytechnik.freemoney.PFuture;


/** A print fields.
 * @version $Id: PrintFields.java,v 1.43 2022/09/30 10:26:13 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class PrintFields {
static String getExperimentaldIHdtHeader(final String prefix,final int num_start)
{
    int n=num_start;
    return // 12 fields
	 "\t"+(n++)+":"+prefix+"dPIHdt_spurM"
	 +"\t"+(n++)+":"+prefix+"dpdt_spurM"
	 +"\t"+(n++)+":"+prefix+"dIHdt_spurM"
	 +"\t"+(n++)+":"+prefix+"pv_dI"
	 +"\t"+(n++)+":"+prefix+"pv_maxdIcontrib"
	 +"\t"+(n++)+":"+prefix+"pv_mindIcontrib"
	 +"\t"+(n++)+":"+prefix+"Tv_maxdIcontrib"
	 +"\t"+(n++)+":"+prefix+"Tv_mindIcontrib"
	 +"\t"+(n++)+":"+prefix+"pv_posdI"
	 +"\t"+(n++)+":"+prefix+"pv_negdI"
	 +"\t"+(n++)+":"+prefix+"dpdt_posdI_spurM"
	 +"\t"+(n++)+":"+prefix+"dpdt_negdI_spurM"
	;
}
static String getExperimentaldIHdtData(final ExperimentaldIHdt r){
    return
    	"\t"+r.dPIHdt_spurM
	+"\t"+r.dpdt_spurM
	+"\t"+r.dIHdt_spurM
	+"\t"+r.pv_dI
	+"\t"+r.pv_maxdIcontrib
	+"\t"+r.pv_mindIcontrib
	+"\t"+r.Tv_maxdIcontrib
	+"\t"+r.Tv_mindIcontrib
	+"\t"+r.pv_posdI
	+"\t"+r.pv_negdI
	+"\t"+r.dpdt_posdI_spurM
	+"\t"+r.dpdt_negdI_spurM
	;
}

static String getDaDvHeader(final String prefix,final int num_start)
{
    int n=num_start;
    return // 12 fields
	getEVXDataHeader(prefix+"I",n)
	+"\t"+((n+=6+1)-1)+":"+prefix+"pv_L"
	 +"\t"+(n++)+":"+prefix+"Tv_L"
	 +"\t"+(n++)+":"+prefix+"pa_L"
	 +"\t"+(n++)+":"+prefix+"pv_H"
	 +"\t"+(n++)+":"+prefix+"Tv_H"
	 +"\t"+(n++)+":"+prefix+"pa_H"
	;
}

static String getDaDvData(final DaDv r){
    return
	getEVXDataData(r.I)
	+"\t"+r.pv_L
	+"\t"+r.Tv_L
	+"\t"+r.pa_L
	+"\t"+r.pv_H
	+"\t"+r.Tv_H
	+"\t"+r.pa_H
	;
    }

/** Print fields in header.
 */
static String getIntervalSincePsiMHeader(final String prefix,final int num_start)
{
    int n=num_start;
    // 74 fields
    return
	"\t"+(n++)+":"+prefix+"I_M"
	+"\t"+(n++)+":"+prefix+"Tv_M"
	+"\t"+(n++)+":"+prefix+"T_M"
	+"\t"+(n++)+":"+prefix+"V_M"
	+"\t"+(n++)+":"+prefix+"Tv_spurM"
	+"\t"+(n++)+":"+prefix+"pv_M"
	+"\t"+(n++)+":"+prefix+"pt_M"
	+"\t"+(n++)+":"+prefix+"dpdt_M"
	+"\t"+(n++)+":"+prefix+"dpdtPI_M"
	+"\t"+(n++)+":"+prefix+"p_M"
	+"\t"+(n++)+":"+prefix+"VdivT_M"
	+"\t"+(n++)+":"+prefix+"covPTi_M"

	+"\t"+(n++)+":"+prefix+"pv_average"
	+"\t"+(n++)+":"+prefix+"Tv_average"

	+"\t"+(n++)+":"+prefix+"pv_spurM"
	+"\t"+(n++)+":"+prefix+"pt_spurM"
	+"\t"+(n++)+":"+prefix+"dpdt_spurM"
	+"\t"+(n++)+":"+prefix+"Idpdt_spurM"
	+"\t"+(n++)+":"+prefix+"covPTi_spurM"
	+"\t"+(n++)+":"+prefix+"dpdtDivI_spurM"

	+"\t"+(n++)+":"+prefix+"dpdtPI_spurM"
	+"\t"+(n++)+":"+prefix+"IdpdtPI_spurM"
	+"\t"+(n++)+":"+prefix+"I2dpdtPI_spurM"
	+"\t"+(n++)+":"+prefix+"dpdtDivIPI_spurM"


	+"\t"+(n++)+":"+prefix+"dPIdtF_spurM"
	+"\t"+(n++)+":"+prefix+"pFPI_posdI_spurM"
	+"\t"+(n++)+":"+prefix+"pFPI_negdI_spurM"
	+"\t"+(n++)+":"+prefix+"dIdt_posdI_spurM"
	+"\t"+(n++)+":"+prefix+"dIdt_negdI_spurM"
	+"\t"+(n++)+":"+prefix+"proj0_posdI_spurM"
	+"\t"+(n++)+":"+prefix+"proj0_negdI_spurM"
	+"\t"+(n++)+":"+prefix+"dpdt_posdI_spurM"
	+"\t"+(n++)+":"+prefix+"dpdt_negdI_spurM"
	+"\t"+(n++)+":"+prefix+"dpdtPI_posdI_spurM"
	+"\t"+(n++)+":"+prefix+"dpdtPI_negdI_spurM"

	+getExperimentaldIHdtHeader(prefix+"dIHdtSplitI0.",n)
	+getExperimentaldIHdtHeader(prefix+"dIHdtSplitIeq0.",(n+=12))
	+getExperimentaldIHdtHeader(prefix+"dIHdtSplitF.",(n+=12))

	+"\t"+((n+=12+1)-1)+":"+prefix+"pdI_spurM"
	+"\t"+(n++)+":"+prefix+"pdIPI_spurM"
	+"\t"+(n++)+":"+prefix+"dpdI_spurM"
	;
}
static String  getIntervalSincePsiMData(final IntervalSincePsiM r)
{
    return
    	"\t"+r.I_M
	+"\t"+r.Tv_M
	+"\t"+r.T_M
	+"\t"+r.V_M
	+"\t"+r.Tv_spurM
	+"\t"+r.pv_M
	+"\t"+r.pt_M
	+"\t"+r.dpdt_M
	+"\t"+r.dpdtPI_M
	+"\t"+r.p_M
	+"\t"+r.VdivT_M
	+"\t"+r.covPTi_M

	+"\t"+r.pv_average
	+"\t"+r.Tv_average

	+"\t"+r.pv_spurM
	+"\t"+r.pt_spurM
	+"\t"+r.dpdt_spurM
	+"\t"+r.Idpdt_spurM
	+"\t"+r.covPTi_spurM
	+"\t"+r.dpdtDivI_spurM

	+"\t"+r.dpdtPI_spurM
	+"\t"+r.IdpdtPI_spurM
	+"\t"+r.I2dpdtPI_spurM
	+"\t"+r.dpdtDivIPI_spurM

	+"\t"+r.dPIdtF_spurM
	+"\t"+r.pFPI_posdI_spurM
	+"\t"+r.pFPI_negdI_spurM
	+"\t"+r.dIdt_posdI_spurM
	+"\t"+r.dIdt_negdI_spurM
	+"\t"+r.proj0_posdI_spurM
	+"\t"+r.proj0_negdI_spurM
	+"\t"+r.dpdt_posdI_spurM
	+"\t"+r.dpdt_negdI_spurM
	+"\t"+r.dpdtPI_posdI_spurM
	+"\t"+r.dpdtPI_negdI_spurM

	+getExperimentaldIHdtData(r.dIHdtSplitI0)
	+getExperimentaldIHdtData(r.dIHdtSplitIeq0)
	+getExperimentaldIHdtData(r.dIHdtSplitF)

	+"\t"+r.pdI_spurM
	+"\t"+r.pdIPI_spurM
	+"\t"+r.dpdI_spurM
	;
}

static String getPsiMSelectionHeader(final String prefix,final int num_start)
{
    int n=num_start;
      return
	  "\t"+(n++)+":"+prefix+"I_M"
	  +"\t"+(n++)+":"+prefix+"Tv_M"
	  +"\t"+(n++)+":"+prefix+"T_M"
	  +"\t"+(n++)+":"+prefix+"V_M"
	  +"\t"+(n++)+":"+prefix+"pv_M"
	  +"\t"+(n++)+":"+prefix+"pt_M"
	  +"\t"+(n++)+":"+prefix+"VdivT_M"
	  +"\t"+(n++)+":"+prefix+"Tv_spurM"
	  +"\t"+(n++)+":"+prefix+"pv_spurM"
	  +"\t"+(n++)+":"+prefix+"pt_spurM"
	  +"\t"+(n++)+":"+prefix+"dpdt_M"
	  +"\t"+(n++)+":"+prefix+"dpdtPI_M"
	  +"\t"+(n++)+":"+prefix+"dpdt_spurM"
	  +"\t"+(n++)+":"+prefix+"Idpdt_spurM"
	  +"\t"+(n++)+":"+prefix+"dpdtPI_spurM"
	  +"\t"+(n++)+":"+prefix+"IdpdtPI_spurM"
	  +"\t"+(n++)+":"+prefix+"Tv_dpMin"
	  +"\t"+(n++)+":"+prefix+"pv_dpMin"
	  +"\t"+(n++)+":"+prefix+"dp_dpMin"
	  +"\t"+(n++)+":"+prefix+"Tv_dpMax"
	  +"\t"+(n++)+":"+prefix+"pv_dpMax"
	  +"\t"+(n++)+":"+prefix+"dp_dpMax"
	  ;
}

static String  getPsiMSelectionData(final PsiMSelection r)
{
    return
	"\t"+r.I_M
	+"\t"+r.Tv_M
	+"\t"+r.T_M
	+"\t"+r.V_M
	+"\t"+r.pv_M
	+"\t"+r.pt_M
	+"\t"+r.VdivT_M
	+"\t"+r.Tv_spurM
	+"\t"+r.pv_spurM
	+"\t"+r.pt_spurM
	+"\t"+r.dpdt_M
	+"\t"+r.dpdtPI_M
	+"\t"+r.dpdt_spurM
	+"\t"+r.Idpdt_spurM
	+"\t"+r.dpdtPI_spurM
	+"\t"+r.IdpdtPI_spurM
	+"\t"+r.Tv_dpMin
	+"\t"+r.pv_dpMin
	+"\t"+r.dp_dpMin
	+"\t"+r.Tv_dpMax
	+"\t"+r.pv_dpMax
	+"\t"+r.dp_dpMax
	;
}


static String getTwoTopIHeader(final String prefix,final int num_start)
{
    int n=num_start;
    return // 14 fields
	"\t"+(n++)+":"+prefix+"I_IlH"
	+"\t"+(n++)+":"+prefix+"pv_IlH"
	+"\t"+(n++)+":"+prefix+"Tv_IlH"
	+"\t"+(n++)+":"+prefix+"V_IlH"
	+"\t"+(n++)+":"+prefix+"dp_IlH"
	+"\t"+(n++)+":"+prefix+"I_IlL"
	+"\t"+(n++)+":"+prefix+"pv_IlL"
	+"\t"+(n++)+":"+prefix+"Tv_IlL"
	+"\t"+(n++)+":"+prefix+"V_IlL"
	+"\t"+(n++)+":"+prefix+"dp_IlL"
	+"\t"+(n++)+":"+prefix+"pv_sel"
	+"\t"+(n++)+":"+prefix+"Tv_sel"
	+"\t"+(n++)+":"+prefix+"V_sel"
	+"\t"+(n++)+":"+prefix+"dp_sel"
	;
}
static String getTwoTopIData(final TwoTopI r){
    return
	"\t"+r.I_IlH
	+"\t"+r.pv_IlH
	+"\t"+r.Tv_IlH
	+"\t"+r.V_IlH
	+"\t"+r.dp_IlH
	+"\t"+r.I_IlL
	+"\t"+r.pv_IlL
	+"\t"+r.Tv_IlL
	+"\t"+r.V_IlL
	+"\t"+r.dp_IlL
	+"\t"+r.pv_sel
	+"\t"+r.Tv_sel
	+"\t"+r.V_sel
	+"\t"+r.dp_sel
	;
}


/** Print fields in header.
 */
static String getSplitdIHdtHeader(final String prefix,final int num_start)
{
    int n=num_start;
    // 19 records
    return
	PrintFields.getEVXDataHeader(prefix+"IH",n)
	+"\t"+((n+=(6+1)) - 1)+":"+prefix+"Tv_M"
	+"\t"+(n++)+":"+prefix+"T_M"
	+"\t"+(n++)+":"+prefix+"I_M"
	+"\t"+(n++)+":"+prefix+"pv_M"
	+"\t"+(n++)+":"+prefix+"dIHdt_spurM"

	+"\t"+(n++)+":"+prefix+"PdIHdt_spurM"
	+"\t"+(n++)+":"+prefix+"dIHdt_posdIH_spurM"
	+"\t"+(n++)+":"+prefix+"dIHdt_negdIH_spurM"
	+"\t"+(n++)+":"+prefix+"pF_posdIH_spurM"
	+"\t"+(n++)+":"+prefix+"pF_negdIH_spurM"
	+"\t"+(n++)+":"+prefix+"dpdt_spurM"
	+"\t"+(n++)+":"+prefix+"dpdt_posdIH_spurM"
	+"\t"+(n++)+":"+prefix+"dpdt_negdIH_spurM"
	;
}
static String  getSplitdIHdtData(final SplitdIHdt r)
{
    return
	PrintFields.getEVXDataData(r.IH)
    	+"\t"+r.Tv_M
    	+"\t"+r.T_M
    	+"\t"+r.I_M
    	+"\t"+r.pv_M
    	+"\t"+r.dIHdt_spurM
	+"\t"+r.PdIHdt_spurM

	+"\t"+r.dIHdt_posdIH_spurM
	+"\t"+r.dIHdt_negdIH_spurM
	+"\t"+r.pF_posdIH_spurM
	+"\t"+r.pF_negdIH_spurM
	+"\t"+r.dpdt_spurM
	+"\t"+r.dpdt_posdIH_spurM
	+"\t"+r.dpdt_negdIH_spurM
	;
}


static String getEVXDataHeader(final String prefix,final int num_start)
{
    return // 6 fields
	"\t"+(num_start)+":"+prefix+".s0"
	+"\t"+(num_start+1)+":"+prefix+".sL"
	+"\t"+(num_start+2)+":"+prefix+".wL_squared"
	+"\t"+(num_start+3)+":"+prefix+".sH"
	+"\t"+(num_start+4)+":"+prefix+".wH_squared"
	+"\t"+(num_start+5)+":"+prefix+".Gamma0";
}
static String getEVXDataData(final EVXData I){return getEVXDataDataShifted(0,I);}
static String getEVXDataDataShifted(final double offset,final EVXData I)
{
    return // 6 fields
	"\t"+(I.s0+offset)
	+"\t"+(I.sL+offset)
	+"\t"+Math.pow(I.wL,2)
	+"\t"+(I.sH+offset)
	+"\t"+Math.pow(I.wH,2)
	+"\t"+((2*I.s0-I.sL-I.sH)/(I.sL-I.sH))
	;
}



/** Print fields in header.
 */
static String getPFutureHeader(final String prefix,final int num_start)
{
    int n=num_start;
    // 42 records
    return
	getEVXDataHeader(prefix+"I",(n+=6)-6)
	+"\t"+(n++)+":"+prefix+"pv_average"
	+"\t"+(n++)+":"+prefix+"Tv_average"
	+"\t"+(n++)+":"+prefix+"totalVolume"

	+"\t"+(n++)+":"+prefix+"I_M"
	+"\t"+(n++)+":"+prefix+"Volume_M"
	+"\t"+(n++)+":"+prefix+"VdivT_M"
	+"\t"+(n++)+":"+prefix+"Tv_M"
	+"\t"+(n++)+":"+prefix+"dp_M"
	+"\t"+(n++)+":"+prefix+"pv_M"
	+"\t"+(n++)+":"+prefix+"Tv_spurM"
	+"\t"+(n++)+":"+prefix+"pv_spurM"
	+"\t"+(n++)+":"+prefix+"dp_spurM"
	+"\t"+(n++)+":"+prefix+"I_spurM"

	+"\t"+(n++)+":"+prefix+"PnL_tradeI_spurM"
	+"\t"+(n++)+":"+prefix+"Vol_tradeI_spurM"
	+"\t"+(n++)+":"+prefix+"P_posImVT"
	+"\t"+(n++)+":"+prefix+"P_negImVT"
	+"\t"+(n++)+":"+prefix+"accessible_Volume_ratio"

	+"\t"+(n++)+":"+prefix+"VdP_M"
	+"\t"+(n++)+":"+prefix+"VdP_spurM"
	+"\t"+(n++)+":"+prefix+"DtVdP_M"
	+"\t"+(n++)+":"+prefix+"dPIdt_M"
	+"\t"+(n++)+":"+prefix+"dIdt_spurM"
	+"\t"+(n++)+":"+prefix+"dPIdt_spurM"

	+"\t"+(n++)+":"+prefix+"PEQV_from_M"
	+"\t"+(n++)+":"+prefix+"PEQV_from_spurM"


	// experiments and tests
	+"\t"+(n++)+":"+prefix+"PEQ_I_fromDirectdpI"
	+"\t"+(n++)+":"+prefix+"PEQ_I_fromDPI"
	+"\t"+(n++)+":"+prefix+"PEQ_I_fromI2DtpDivI"
	+"\t"+(n++)+":"+prefix+"PEQ_JJ_FromDPI"
	+"\t"+(n++)+":"+prefix+"PEQtotalL_JJ_FromDPI"

	+"\t"+(n++)+":"+prefix+"Idpdt_spurM_from_dp_I"
	+"\t"+(n++)+":"+prefix+"Idpdt_spurM_from_DP_I"
	+"\t"+(n++)+":"+prefix+"Idpdt_spurM_from_sqI_DP_sqI"

	+"\t"+(n++)+":"+prefix+"DtpDivI_spurM"
	+"\t"+(n++)+":"+prefix+"I2DtpDivI_spurM"

	;
}
static String  getPFutureData(final PFuture r)
{
    return
	getEVXDataData(r.ianddm.I)
    	+"\t"+r.pv_average
    	+"\t"+r.Tv_average
    	+"\t"+r.totalVolume

    	+"\t"+r.I_M
    	+"\t"+r.Volume_M
	+"\t"+r.VdivT_M
	+"\t"+r.Tv_M
	+"\t"+r.dp_M
	+"\t"+r.pv_M
	+"\t"+r.Tv_spurM
	+"\t"+r.pv_spurM
	+"\t"+r.dp_spurM
	+"\t"+r.I_spurM

	+"\t"+r.PnL_tradeI_spurM
	+"\t"+r.Vol_tradeI_spurM
	+"\t"+r.P_posImVT
	+"\t"+r.P_negImVT
	+"\t"+r.accessible_Volume_ratio

	+"\t"+r.VdP_M
	+"\t"+r.VdP_spurM
	+"\t"+r.DtVdP_M
	+"\t"+r.dPIdt_M
	+"\t"+r.dIdt_spurM
	+"\t"+r.dPIdt_spurM

	+"\t"+r.PEQV_from_M
	+"\t"+r.PEQV_from_spurM


	// experiments and tests
	+"\t"+r.PEQ_I_fromDirectdpI
	+"\t"+r.PEQ_I_fromDPI
	+"\t"+r.PEQ_I_fromI2DtpDivI
	+"\t"+r.PEQ_JJ_FromDPI
	+"\t"+r.PEQtotalL_JJ_FromDPI

	+"\t"+r.Idpdt_spurM_from_dp_I
	+"\t"+r.Idpdt_spurM_from_DP_I
	+"\t"+r.Idpdt_spurM_from_sqI_DP_sqI

	+"\t"+r.DtpDivI_spurM
	+"\t"+r.I2DtpDivI_spurM

	;
}


}
