package com.polytechnik.algorithms;

import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.*;


/** Fourier basis checks.
 * $Id: PrintFourierBasis.java,v 1.8 2020/02/22 00:16:03 mal Exp $
 */
class PrintFourierBasis {

    static class PrintFunctionInFourierBasis {
	final int nharmonics,npoints;
	final java.util.function.DoubleFunction<double []> fun;
	PrintFunctionInFourierBasis(final int nharmonics,final int npoints,
				    final java.util.function.DoubleFunction<double []> fun){
	    System.err.println("PrintFunctionInFourierBasis: nharmonics="+nharmonics+" npoints="+npoints);
	    this.nharmonics=nharmonics;
	    this.fun=fun;
	    this.npoints=npoints;
	}
	void printHeader(java.io.PrintWriter out){
	    int ne=-1;
	    out.print("|x:"+"ind="+(++ne));
	    for(int k=0;k<=nharmonics;k++){
		if(k<=0) out.print(","+"C:"+"ind="+(++ne));
		else {
		    out.print(","+"sin("+k+"*x*pi):"+"ind="+(++ne));
		    out.print(","+"cos("+k+"*x*pi):"+"ind="+(++ne));
		}
	    }
	    final double [] f=fun.apply(0);
	    for(int k=0;k<f.length;k++){
		out.print(",fun_"+k+":"+"ind="+(++ne));
	    }
	    out.print(","+"weight:"+"ind="+(++ne));
	    out.println();
	}
	void printDataRow(java.io.PrintWriter out,double x,double weight){
	    out.print(String.valueOf(x));
	    for(int k=0;k<=nharmonics;k++){
		if(k<=0) out.print(","+(Math.sqrt(2)/2));
		else {
		    out.print(","+Math.sin(k*x*Math.PI));
		    out.print(","+Math.cos(k*x*Math.PI));
		}
	    }
	    final double [] f=fun.apply(x);
	    for(int k=0;k<f.length;k++){
		out.print(","+f[k]);
	    }
	    out.print(","+weight);
	    out.println();
	}

	void printData(java.io.PrintWriter out){
	    final double npd=Double.valueOf(npoints),w=2/npd;
	    printHeader(out);
	    for(int t=0;t<=npoints;t++){
		double x=-1+2*t/npd;
		printDataRow(out,x,t==0||t==npoints?w/2:w);
	    }
	    out.flush();
	}
    } // END OF class PrintFunctionInFourierBasis

    static final java.util.function.DoubleFunction<double []> functionstocalculate=
	(x)->new double []{
	    Math.sin(x*Math.PI), // sin
	    x<0?-1:1, // meander
	    Math.abs(x)<0.125?1:-1, // an impules
	    Math.abs(x)<0.125?1:(0.90<=x && x<=0.901?1e6:-1), // with a spike
	    1/(1+25*Math.pow(x,2))
    };

    public static void main(String [] args)
    {
	if(args.length<=0)
	{
	    System.err.println("usage: PrintFourierBasis 7 # the number of basis functions is 2*n+1");
	    System.exit(1);
	}
	final int n=Integer.valueOf(args[0]);
	new PrintFunctionInFourierBasis(n,10000,functionstocalculate).printData(new java.io.PrintWriter(System.out));
    }
}
