package com.polytechnik.algorithms;
import com.polytechnik.utils.DiffSkewness;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.TMatr;

/** Print Gamma(P) for several distributions.
 * @author: $Id: PrintGammaP.java,v 1.18 2018/08/16 08:56:36 mal Exp $
 */
class PrintGammaP {

    final DiffSkewness [] sk;
    PrintGammaP(final java.util.ArrayList<double []> moms)
    {
	this.sk=new DiffSkewness [moms.size()];
	for(int k=0;k<sk.length;k++)
	{
	    final double [] m=moms.get(k);
	    sk[k]=DiffSkewness.getDiffSkewness_FromMonomials(m[0],m[1],m[2],m[3],m[4]); // m[4] is di
	}
    }
    void printGammaP(java.io.PrintWriter out,final double xfrom,final double xto,final double xstep)
    {
	for(double x=xfrom;x<xto;x+=xstep)
	{
	    out.print(String.valueOf(x));
	    for(int k=0;k<sk.length;k++)
	    {
		out.print("\t"+sk[k].getGamma().getF(x)+"\t"+sk[k].getGamma1(x));
	    }
	    out.println();
	}
    }
    void printVolatilityP(java.io.PrintWriter out,final double xfrom,final double xto,final double xstep)
    {
	for(double x=xfrom;x<xto;x+=xstep)
	{
	    out.print(String.valueOf(x));
	    for(int k=0;k<sk.length;k++)
	    {
		final double l2=sk[k].getL2Volatility().getF(x),
		    l4=sk[k].getL4Volatility().getF(x);
		out.print("\t"+l4+"\t"+l2);
	    }
	    out.println();
	}
    }
    void printMidpointP(java.io.PrintWriter out,final double xfrom,final double xto,final double xstep)
    {
	for(double x=xfrom;x<xto;x+=xstep)
	{
	    out.print(String.valueOf(x));
	    for(int k=0;k<sk.length;k++)
	    {
		final double mm2=sk[k].getX1X2midpoint().getF(x),
		    mm1=(x+sk[k].sknodi.xa)/2;
		out.print("\t"+mm2+"\t"+mm1);
	    }
	    out.println();
	}
    }
    void printD2D1MidpointP(java.io.PrintWriter out,final double xfrom,final double xto,final double xstep)
    {
	for(double x=xfrom;x<xto;x+=xstep)
	{
	    out.print(String.valueOf(x));
	    for(int k=0;k<sk.length;k++)
	    {
		final double mm2=sk[k].getX1X2midpoint().getD2F(x),
		    mm1=sk[k].getX1X2midpoint().getDF(x);
		out.print("\t"+mm2+"\t"+mm1);
	    }
	    out.println();
	}
    }
    void printD0D1Pd(java.io.PrintWriter out,final double xfrom,final double xto,final double xstep)
    {
	for(double x=xfrom;x<xto;x+=xstep)
	{
	    out.print(String.valueOf(x));
	    for(int k=0;k<sk.length;k++)
	    {
		final double pd1=sk[k].getX2minusX1().getDF(x),
		    pd0=sk[k].getX2minusX1().getF(x);
		out.print("\t"+pd0+"\t"+pd1);
	    }
	    out.println();
	}
    }
    void printP1P2(java.io.PrintWriter out,final double xfrom,final double xto,final double xstep)
    {
	for(double x=xfrom;x<xto;x+=xstep)
	{
	    out.print(String.valueOf(x));
	    for(int k=0;k<sk.length;k++)
	    {
		final double p1=sk[k].getX1X2()[0].getF(x),
		    p2=sk[k].getX1X2()[1].getF(x);
		out.print("\t"+p1+"\t"+p2);
	    }
	    out.println();
	}
    }
    void printW1W2(java.io.PrintWriter out,final double xfrom,final double xto,final double xstep)
    {
	for(double x=xfrom;x<xto;x+=xstep)
	{
	    out.print(String.valueOf(x));
	    for(int k=0;k<sk.length;k++)
	    {
		final Skewness s=sk[k].getSkewnessAt(x);
		out.print("\t"+s.w1+"\t"+s.w2);
	    }
	    out.println();
	}
    }
    static double [] getMoments(final double [][] xw,
				final double di)
    {
	final double [] m=new double [5];
	for(int k=0;k<xw.length;k++)
	{
	    TMatr.sumP1(m,DiffSkewness.MB.B.getX0Moments(4,xw[k][0]),xw[k][1]);
	}
	m[4]=di;
	return m;
    }

    public static void main(String []args) throws java.io.IOException
    {
	final java.util.ArrayList<double []>a=new java.util.ArrayList<double []>();
	a.add(getMoments(new double [][] {{0.1,1},{0.3,0.2}},0.4));
	a.add(getMoments(new double [][] {{-1,1},{-1.2,0.2}},1.4));
	a.add(getMoments(new double [][] {{0.6,0.2},{0.3,1.2}},1.4));
	a.add(getMoments(new double [][] {{0.8,0.5},{1,1.2}},1.6));
	a.add(getMoments(new double [][] {{0.8,0.5},{0.8001,0.5001}},1.6));
	a.add(getMoments(new double [][] {{0.1,1},{0.3,0.2}},0.4*10e6));
	a.add(getMoments(new double [][] {{-0.1,1},{-0.3,0.2},{-0.5,0.2},{-0.6,0.2},{0.2,0.2},{0.3,0.5}},1.4));
	final PrintGammaP g=new PrintGammaP(a);
	final String fgamma="GammaP.dat",fvolatility="VolatilityP.dat",
	    fmidpoint="MidpointP.dat",fd2d1midpoint="D2D1MidpointP.dat",fpd01="PdD0D1P.dat",fp1p2="P1P2.dat",fw1w2="W1W2.dat";

	System.err.println("writing "+fgamma);
	java.io.PrintWriter out=new java.io.PrintWriter(fgamma);
	g.printGammaP(out,-5,5,0.01);
	out.close();

	System.err.println("writing "+fvolatility);
	out=new java.io.PrintWriter(fvolatility);
	g.printVolatilityP(out,-5,5,0.01);
	out.close();

	System.err.println("writing "+fmidpoint);
	out=new java.io.PrintWriter(fmidpoint);
	g.printMidpointP(out,-5,5,0.01);
	out.close();

	System.err.println("writing "+fd2d1midpoint);
	out=new java.io.PrintWriter(fd2d1midpoint);
	g.printD2D1MidpointP(out,-5,5,0.01);
	out.close();

	System.err.println("writing "+fpd01);
	out=new java.io.PrintWriter(fpd01);
	g.printD0D1Pd(out,-5,5,0.01);
	out.close();


	System.err.println("writing "+fp1p2);
	out=new java.io.PrintWriter(fp1p2);
	g.printP1P2(out,-5,5,0.01);
	out.close();

	System.err.println("writing "+fw1w2);
	out=new java.io.PrintWriter(fw1w2);
	g.printW1W2(out,-5,5,0.01);
	out.close();
    }
}
