package com.polytechnik.algorithms;

import com.polytechnik.utils.*;

class PrintMatr {


    static final long fact(final int n)
    {
	if(n<=1)
	{
	    return 1;
	}
	return n*fact(n-1);
    }

    static final long cnk(final int n,final int k)
    {
	return fact(n)/fact(k)/fact(n-k);
    }


    static double [][] getMatrEG(final int n)
    {
	final double [][] matr =new double [n][n];
	for(int i=0;i<n;i++)
	{
	    double v=1;
	    for(int j=0;j<n;j++)
	    {
		matr[i][j]=v;
		v*=(1+j)*1.0/(1.0+i);
	    }
	}
	return matr;
    }

    static double [][] getMatrE(final int n)
    {
	final double [][] matr =new double [n][n];
	for(int i=0;i<n;i++)
	{
	    double v=1;
	    for(int j=0;j<n;j++)
	    {
		matr[i][j]=v;
		v*=(1+i)*(1+j);
	    }
	}
	return matr;
    }

    static double [][] getMatrET(final int n)
    {
	final double [][] matr =new double [n][n];
	for(int i=0;i<n;i++)
	{
	    double v=1;
	    for(int j=0;j<n;j++)
	    {
		matr[i][j]=v;
		v*=(1+i+j);
	    }
	}
	return matr;
    }

    static double [][] getMatrDeltas(final int n)
    {
	final double [][] matr =new double [n][n];
	for(int i=0;i<n;i++)
	{
	    double i1=i+1;
	    for(int j=0;j<n;j++)
	    {
		matr[i][j]=Math.exp(-j/i1)/i1;
	    }
	}
	return matr;
    }

    static double [][] getMatrGammaTau(final int n)
    {
	final double [][] matr =new double [n][n];
	final double g0tau0=1;
	for(int i=0;i<n;i++)
	{
	    double i1=i+1;
	    for(int j=0;j<n;j++)
	    {
		matr[i][j]=Math.exp(-i1*g0tau0*j)*i1;
	    }
	}
	return matr;
    }

    static double [][] getMatrCn(final int n)
    {
	final double [][] matr =new double [n][n];
	for(int i=0;i<n;i++)
	{
	    for(int j=0;j<n;j++)
	    {
		matr[i][j]=cnk(j+i,j);
	    }
	}
	return matr;
    }

    static double [][] getMatrExpF(final int n)
    {
	final double [][] matr =new double [n][n];
	for(int i=0;i<n;i++)
	{
	    //final double l=1.0/(i+1);
	    final double l=(i+1);
	    for(int j=0;j<n;j++)
	    {
		//matr[i][j]=l/(l+1.0/(1+j));
		matr[i][j]=l/(l+(/*1*/+j));
	    }
	}
	return matr;
    }
    static void printCoefsSum(final double [][] matr)
    {
	final int n=matr.length;
	final int [] ipvt=new int [n];
	final int info=Linsystems.dgefa(matr,n,ipvt);
	if(info!=0)
	{
	    throw new RuntimeException("INFO="+info);
	}
	final double [] r=new double [n];
	System.err.println("Order="+n+" ==============");
	for(int i=0;i<n;i++)
	{
	    java.util.Arrays.fill(r,0);
	    r[i]=1;
	    Linsystems.dgesl(matr,n,ipvt,r,1);
	    double s=0;
	    for(int j=0;j<n;j++)
	    {
		s+=r[j];
	    }
	    System.err.println("k"+i+"="+s);
	}
    }


    static void printCoefs(final double [][] matr)
    {
	final int n=matr.length;
	final int [] ipvt=new int [n];
	final int info=Linsystems.dgefa(matr,n,ipvt);
	if(info!=0)
	{
	    throw new RuntimeException("INFO="+info);
	}
	final double [] r=new double [n];
	System.err.println("Order="+n+" ==============");
	for(int i=0;i<n;i++)
	{
	    java.util.Arrays.fill(r,0);
	    r[i]=1;
	    Linsystems.dgesl(matr,n,ipvt,r,1);
	    System.err.println("k"+i+"="+r[0]+" dk"+i+"="+r[1]);
	}
    }

    public static void main(String [] args)
    {
	/*
	System.err.println("Delta matr");
	printCoefs(getMatrDeltas(2));
	printCoefs(getMatrDeltas(3));
	printCoefs(getMatrDeltas(5));
	System.err.println("GammaTau matr");
	printCoefs(getMatrGammaTau(2));
	printCoefs(getMatrGammaTau(3));
	printCoefs(getMatrGammaTau(5));

	System.err.println("EXPF matr");
	printCoefsSum(getMatrExpF(1));
	printCoefsSum(getMatrExpF(2));
	printCoefsSum(getMatrExpF(3));
	printCoefsSum(getMatrExpF(5));
	printCoefsSum(getMatrExpF(6));
	printCoefsSum(getMatrExpF(7));
	*/


	System.err.println("ET matr");
	printCoefs(getMatrET(2));
	printCoefs(getMatrET(3));
	printCoefs(getMatrET(5));

	System.err.println("E matr");
	printCoefs(getMatrE(2));
	printCoefs(getMatrE(3));
	printCoefs(getMatrE(5));
	printCoefs(getMatrE(7));

	System.err.println("EG matr");
	printCoefs(getMatrEG(2));
	printCoefs(getMatrEG(3));
	printCoefs(getMatrEG(5));
	printCoefs(getMatrEG(7));


	System.err.println("Cn matr");
	printCoefs(getMatrCn(2));
	printCoefs(getMatrCn(3));
	printCoefs(getMatrCn(4));
	printCoefs(getMatrCn(5));
	printCoefs(getMatrCn(7));
    }
}

