package com.polytechnik.algorithms;

import com.polytechnik.utils.*;

/** Print orthogonal polynomials.
 * $Id: PrintOPol.java,v 1.23 2022/06/28 07:36:58 mal Exp $
 */
class PrintOPol {

static final OrthogonalPolynomialsLegendreBasis O=new OrthogonalPolynomialsLegendreBasis();
static final BasisFunctionsCalculatable B=new Legendre();
static double [] getMoments(final int n)
{
    final double [] m=new double [2*n+1];
    m[0]=1;
    return m;
}

static class PrintLocalizedPsi {
    final double [] y;
    final double [][] Qy;
    final int n;
    final double [] QQ,IQQ;
    PrintLocalizedPsi(final double [] y,final int n)
    {
	this.y=y;
	this.n=n;
	final double [] moms=getMoments(n);
	this.QQ=O.B.getQQMatr(n,moms);
	this.IQQ=Linsystems.getInvertedMatrix(n,n,QQ);
	this.Qy=new double [y.length][];
	for(int k=0;k<Qy.length;k++)
	{
	    Qy[k]=O.B.getX0Moments(n,y[k]);
	}
    }
    void printPoints(final java.io.PrintStream out,final double x0,final double xend,final int npoints)
    {
	out.print("|x");
	for(int k=0;k<y.length;k++) out.print("\ty="+y[k]);
	out.println();


	double xt=x0,dx=(xend-x0)/(npoints-1);
	for(int t=0;t<npoints;t++)
	{
	    out.print(String.valueOf(xt));
	    final double [] Qxt=O.B.getX0Moments(n,xt);
	    for(int k=0;k<Qy.length;k++)
	    {
		final double nom=Math.pow(TMatr.getAMatrB(Qxt,Qy[k],IQQ,n),2),
		    denom=TMatr.getAMatrB(Qy[k],Qy[k],IQQ,n);
		out.print("\t"+(nom/denom));
	    }
	    out.println();
	    xt+=dx;
	}
    }
    public static void main(String [] args)
    {
	new PrintLocalizedPsi(new double []{-0.3,0.1},12).
	    printPoints(System.out,-1,1,10000);

    }
}

    static class PP {
	final int n;
	final double [] moms,x,w,nodes;
	final double [][] L,DL,sDL;
	PP(final int n)
	{
	    this.n=n;
	    this.moms=getMoments(n);
	    final Quadrature.GaussQuadratureWithNodesPolynomialData nr=O.getGaussQuadratureFromPolynomials(n,O.getOrthogonalPolynomialsFirstKind(n+1,moms));
	    this.x=nr.x;
	    this.w=nr.w;
	    if(x.length!=n) throw new IllegalArgumentException();
	    this.nodes=nr.nodespolynomial;
	    this.L=new double [n][];
	    this.DL=new double [n][];
	    this.sDL=new double [n][];
	    for(int k=0;k<n;k++)
	    {
		final double xk=x[k];
		final double [] psi=O.B.sdiv1(nodes,xk).q;
		final double fk=O.B.getD0(psi,xk),fdk=O.B.getD1(psi,xk);
		for(int j=0;j<psi.length;j++){psi[j]/=fk;}
		L[k]=B.mult2Pol(psi,psi);
		final double [] dpsi=B.getDifferentiated(psi);
		DL[k]=java.util.Arrays.copyOf(B.mult2Pol(psi,dpsi),L[k].length);
		TMatr.sumP1(DL[k],L[k],-(fdk/fk));
		sDL[k]=B.sdiv1(DL[k],xk).q;
	    }
	}
	void print(final double xb,final double xe,final int np)
	{
	    final double eps=1e-6;
	    final double [] xmoms=O.B.getXQkMomentsFromQkMoments(moms),x2moms=O.B.getXQkMomentsFromQkMoments(xmoms);
	    final double dx=(xe-xb)/(np);
	    System.err.print("#|x\tP"+n+"(x)\tP(x)P'(x)");
	    for(int k=0;k<n;k++)
	    {
		UnitTests.checkDoubleRel(TMatr.mulVVn1(DL[k],moms),"<DL[k]|moms>",0,"0",eps);
		UnitTests.checkDoubleRel(TMatr.mulVVn1(DL[k],xmoms),"<DL[k]|xmoms>",0,"0",eps);
		UnitTests.checkDoubleRel(TMatr.mulVVn1(L[k],xmoms)/TMatr.mulVVn1(L[k],moms),"<L[k]|xmoms>/<L[k]|moms>",x[k],"x[k]",eps);
		System.err.print("\tL(x"+k+"="+x[k]+")\tDL(x"+k+"="+x[k]+")\tsDL(x"+k+"="+x[k]+")");
	    }
	    System.err.println();
	    for(int t=0;t<=np;t++)
	    {
		final double x=xb+dx*t;
		System.err.print(x+"\t"+O.B.getD0(nodes,x)+"\t"+(O.B.getD0(nodes,x)*O.B.getD1(nodes,x)));
		for(int k=0;k<n;k++)
		{
		    System.err.print("\t"+O.B.getD0(L[k],x)+"\t"+O.B.getD0(DL[k],x)+"\t"+O.B.getD0(sDL[k],x));
		}
		System.err.println();
	    }
	}
    } // END OF class PP

    public static void main(final String [] args)
    {
	if(args.length!=1)
	{
	    System.err.println("usage: PrintOPol n");
	    System.exit(1);
	}
	final PP pp=new PP(Integer.valueOf(args[0]));
	pp.print(-1,1,1000);
    }

}
