package com.polytechnik.algorithms;

import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.*;


/** Visually check Runge effect on edges.
 * $Id: PrintPsi0.java,v 1.2 2022/10/22 12:27:49 mal Exp $
 */
class PrintPsi0 {

    static final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable>
	T=new OrthogonalPolynomialsChebyshevBasis();

    public static void main(String [] args)
    {
	if(args.length<=0)
	{
	    System.err.println("usage: PrintChristoffel 7");
	    System.exit(1);
	}
	final int n=Integer.valueOf(args[0]);
	final double [] moms_P=new double [2*n-1];
	for(int k=0;k<moms_P.length;k++){
	    final double [] tk=new double [k+1];tk[k]=1;
	    final double [] itk=T.B.getIntegrated(tk);
	    // \int_{-1}^{1}T_k(x)dx
	    moms_P[k]=T.B.getD0(itk,1)-T.B.getD0(itk,-1);
	}
	final double [] QQ=T.B.getQQMatr(n,moms_P),
	    IQQ=Linsystems.getInvertedMatrix(n,n,QQ);
	final double [] x0=T.B.getX0Moments(n,0),
	    x04=T.B.getX0Moments(n,0.4),x06=T.B.getX0Moments(n,-0.6);

	System.out.println("|n="+n);
	System.out.println("|x,psi0,psi04,psi06");
	double s0=0,s04=0,s06=0;
	for(int t=0,nt=10000;t<=nt;t++){
	    final double x=-1+(2.0*t)/nt;
	    final double [] X=T.B.getX0Moments(n,x);
	    final double
		psi0_2=Math.pow(TMatr.getAMatrB(X,x0,IQQ,n),2)/TMatr.getAMatrA(x0,IQQ,n),
		psi04_2=Math.pow(TMatr.getAMatrB(X,x04,IQQ,n),2)/TMatr.getAMatrA(x04,IQQ,n),
		psi06_2=Math.pow(TMatr.getAMatrB(X,x06,IQQ,n),2)/TMatr.getAMatrA(x06,IQQ,n);
	    final double w=(t==0||t==nt)?1.0/nt:2.0/nt;
	    s0+=w*psi0_2;
	    s04+=w*psi04_2;
	    s06+=w*psi06_2;

	    System.out.println(String.valueOf(x)+","+psi0_2+","+psi04_2+","+psi06_2);
	}
	System.err.println("s0="+s0+" s04="+s04+" s06="+s06+" all must be =1");
    }
}
