package com.polytechnik.algorithms;

import com.polytechnik.utils.*;


/** Visually check Runge effect on edges.
 * $Id: PrintTestRungeEffectOnEdges.java,v 1.20 2022/08/08 20:10:45 mal Exp $
 */
class PrintTestRungeEffectOnEdges {

    static double f0(final double x)
    {
	return 1/(1+25*Math.pow(2*x-1,2));
	//return Math.cos(3*(x*(2*Math.PI)))/(1+25*Math.pow(2*x-1,2));
    }

static class Integr implements com.polytechnik.utils.QUANC8.DoubleFunction {
    final int order;
    final double [] pol;
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
    Integr(final int order,
	   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	this.order=order;
	this.M=M;
	pol=new double [order+1];
	pol[order]=1;

    }
    public double f(final double x)
    {
	final double f=M.B.getD0(pol,x);
	return f*f0(x);
    }

    static double [] getMoments(final int n,
				final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	final double [] mom=new double [n];
	for(int k=0;k<n;k++)
	{
	    final com.polytechnik.utils.QUANC8 i=
		com.polytechnik.utils.QUANC8.integrate(new Integr(k,M),
							    0,
							    1,
							    0,
							    1e-5,
							    1000000);
	    if(!(i.flag==0))
	    {
		throw new RuntimeException("flag="+i.flag);
	    }
	    mom[k]=i.integral;
	}
	return mom;
    }
}

static final double EPS=1e-6;

static void test_approx(final int n)
{
    final com.polytechnik.trading.DeprecatedHelperQVMDataPDirectAccessWithBasis access=
	new com.polytechnik.trading.DeprecatedHelperQVMDataPDirectAccessWithBasis();
    final int nmoms=2*n-1;
    final var M=
	new OrthogonalPolynomialsLegendreShiftedBasis();
    final double [] momf=Integr.getMoments(nmoms,M);
    final double [] momt=new double [nmoms];
    access.setMT(momt,nmoms);
    final com.polytechnik.utils.PolynomialsRatio pr=com.polytechnik.utils.NevaiOperator.getNevaiOperator_FromMoments(n,momt,momf,M.B);
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{momt,momf});
    final double [] QQt=QQ[0],QQf=QQ[1];
    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	Z=TMatr.mulMMM(n,IQQt,QQf,IQQt);
    final double [] lsq=access.getLSQ(n,QQt,new double [][]{momf})[0];
    final double [] ev=M.EV.getGEV_Generalized(n,QQf,QQt,true);
    final double lmin=ev[0],lmax=ev[n-1];

    final double [][] kkk=access.getKK(n,new double [][]{IQQt,Z});
    final double [] kkt=kkk[0],kkf=kkk[1];

    System.err.println("|x_"+n+"\tf(x)\tIf(x)\tBIf(x)\tlmin\tlmax\tChristoffel\t");
    for(double x=0;x<=1;x+=1e-5)
    {
	final double f=(M.B.getD0(kkf,x)/M.B.getD0(kkt,x)),xm0[]=M.B.getX0Moments(n,x);
	System.err.println(String.valueOf(x)+"\t"+f0(x)+"\t"+M.B.getD0(lsq,x)+"\t"+
			   f+"\t"+
			   lmin+"\t"+lmax+"\t"+(1/TMatr.getAMatrA(xm0,IQQt,n))+"\t");
	UnitTests.checkDoubleRel(f,"f",pr.getF(x),"pr.getF(x)",EPS);
    }

}

    public static void main(String [] args)
    {
	if(args.length<=0)
	{
	    System.err.println("usage: PrintTestRungeEffectOnEdges 7");
	    System.exit(1);
	}
	test_approx(Integer.valueOf(args[0]));
    }
}
