package com.polytechnik.algorithms;

import com.polytechnik.utils.*;

/** Print orthogonal polynomials.
 * $Id: PrintThetaInterpolation.java,v 1.14 2022/08/08 20:10:45 mal Exp $
 */
class PrintThetaInterpolation {

static final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> O=new OrthogonalPolynomialsLegendreShiftedBasis();
static final BasisFunctionsCalculatable B=new LegendreShifted();
    static final double XSTART=0,XEND=1;
static double [] getTMoments(final int nmoms)
{
    final double [] m=new double [nmoms];
    m[0]=1;
    return m;
}
static double [] getThetaMoments(final int nmoms,final double x0)
{
    final double [] m=new double [nmoms];
    for(int k=m.length;--k>=0;)
    {
	final double [] tmp=new double [k+1];tmp[k]=1;
	final double [] in=B.getIntegrated(tmp);
	m[k]=O.B.getD0(in,x0)-O.B.getD0(in,XSTART);
    }
    return m;
}
static double [] getFx0(final int n,final double x0)
{
    final double [] f=new double [n],r=new double [n];
    for(int k=f.length;--k>=0;)
    {
	r[k]=1;
	f[k]=O.B.getD0(r,x0);
	r[k]=0;
    }
    return f;
}

    static class PP {
	final int n;
	final double x0;
	final double [] tmoms,thetamoms;

	final double [] pol;
	final double [] knom,kdenom;
	final double [] idelta,idelta2,idelta4;
	PP(final int n,final double x0)
	{
	    this.n=n;
	    this.x0=x0;
	    final int nmoms=2*n-1+1;
	    this.tmoms=getTMoments(nmoms);
	    this.thetamoms=getThetaMoments(nmoms,x0);
	    final double [] QQt=O.B.getQQMatr(n,tmoms),QQtheta=O.B.getQQMatr(n,thetamoms),
		IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
	    this.pol=TMatr.getMatr_x_A(n,IQQt,java.util.Arrays.copyOf(this.thetamoms,n));
	    this.knom=O.B.getKK(n,TMatr.mulMMM(n,IQQt,QQtheta,IQQt));
	    this.kdenom=O.B.getKK(n,IQQt);
	    final double [] delta=TMatr.getMatr_x_A(n,IQQt,getFx0(n,x0));
	    this.idelta=B.getIntegrated(delta);
	    this.idelta[0]-=O.B.getD0(idelta,XEND);
	    TMatr.scaleVectorInPlace(this.idelta,-1);
	    final double [] delta2=B.mult2Pol(delta,delta);
	    final double nr=TMatr.getAMatrA(delta,QQt,n);
	    TMatr.scaleVectorInPlace(delta2,1/nr);
	    this.idelta2=B.getIntegrated(delta2);
	    this.idelta2[0]-=O.B.getD0(idelta2,XEND);
	    TMatr.scaleVectorInPlace(this.idelta2,-1);

	    final double [] delta4=B.mult2Pol(delta2,delta2);
	    final double nrr=TMatr.mulVVn1(delta4,getTMoments(delta4.length));
	    TMatr.scaleVectorInPlace(delta4,1/nrr);
	    this.idelta4=B.getIntegrated(delta4);
	    this.idelta4[0]-=O.B.getD0(idelta4,XEND);
	    TMatr.scaleVectorInPlace(this.idelta4,-1);
	}

	void print(final double xb,final double xe,final int np)
	{
	    final double dx=(xe-xb)/(np);
	    System.err.println("#|x(x0="+x0+",n="+n+")\tPol\tIdelta\tKK\tIdelta2\tIdelta4");
	    for(int t=0;t<=np;t++)
	    {
		final double x=xb+dx*t;
		System.err.println(x+"\t"+O.B.getD0(pol,x)+"\t"+O.B.getD0(idelta,x)+
				   "\t"+(O.B.getD0(knom,x)/O.B.getD0(kdenom,x))+
				   "\t"+O.B.getD0(idelta2,x)+"\t"+O.B.getD0(idelta4,x));
	    }
	}
    } // END OF class PP

    public static void main(final String [] args)
    {
	if(args.length!=2)
	{
	    System.err.println("PrintThetaInterpolation n x0");
	    System.exit(1);
	}
	final PP pp=new PP(Integer.valueOf(args[0]),Double.valueOf(args[1]));
	pp.print(0,1,1000);
    }

}
