package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
/** Run all java code we have.
  * Any PROGRAM.main(new String[]{"a1","a2"}) can be run from a command line as
  *  java PROGRAM "a1" "a2"
  * here we call them all at once.
  * @version $Id: RunAllJavaPrograms.java,v 1.5 2024/04/21 19:51:20 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  */
class RunAllJavaPrograms {

  static void runTimePriceShares_programs() throws java.io.IOException {

    // https://arxiv.org/abs/1709.06759 . Skewness & probability correlation.
    com.polytechnik.algorithms.CallAMuseOfCashFlowAndLiquidityDeficit.main(new String[]{
      "--musein_file=dataexamples/aapl_old.csv.gz",
      "--musein_cols=9:1:2:3",
      "--n=12",
      "--tau=256",
      "--measure=ImpactQVMMuseLegendreShifted",
      "--museout_file=museoutSkewnessProbabilityCorrelation.dat"});

    // The https://arxiv.org/abs/1903.11530 results with scalp--function.
    com.polytechnik.algorithms.CallAMuseOfCashFlowAndLiquidityDeficitWithScalp.main(new String[]{
      "--musein_file=dataexamples/aapl_old.csv.gz",
      "--musein_cols=9:1:2:3",
      "--n=12",
      "--tau=256",
      "--measure=ScalpedMaxIProjectionLegendreShifted",
      "--museout_file=museoutScalp.dat"});

    // Some tries with price support--function.
    com.polytechnik.algorithms.CallAMuseOfCashFlowAndLiquidityDeficitWithSupportPrice.main(new String[]{
      "--musein_file=dataexamples/aapl_old.csv.gz",
      "--musein_cols=9:1:2:3",
      "--n=12",
      "--tau=256",
      "--measure=SupportPriceLegendreShifted",
      "--museout_file=museoutSupportPrice.dat"});

    // The same thing
    com.polytechnik.algorithms.CallAMuseOfCashFlowAndLiquidityDeficitWithSimpleSupportPrice.main(new String[]{
      "--musein_file=dataexamples/aapl_old.csv.gz",
      "--musein_cols=9:1:2:3",
      "--n=12",
      "--tau=256",
      "--measure=SimpleSupportPriceLegendreShifted",
      "--museout_file=museoutSimpleSupportPrice.dat"});

    com.polytechnik.algorithms.CallAMuseOfCashFlowAndLiquidityDeficitWithPnL.main(new String[]{
      "--musein_file=dataexamples/aapl_old.csv.gz",
      "--musein_cols=9:1:2:3",
      "--n=12",
      "--tau=256",
      "--measure=PnLILowIHighTPVFactoryLegendreShifted",
      "--museout_file=museoutPnLILowIHigh.dat"});

    com.polytechnik.algorithms.TestCall_PnLInPsiHstate.main(new String[]{
      "--musein_file=dataexamples/aapl_old.csv.gz",
      "--musein_cols=9:1:2:3",
      "--n=12",
      "--tau=256",
      "--measure=PnLInPsiHstateLegendreShifted",
      "--museout_file=museoutPnLInPsiHstate.dat"});

    com.polytechnik.algorithms.TestCall_FreeMoneyForAll.main(new String[]{
      "--musein_file=dataexamples/aapl_old.csv.gz",
      "--musein_cols=9:1:2:3",
      "--n=12",
      "--tau=256",
      "--measure=CommonlyUsedMomentsLegendreShifted",
      "--museout_file=museoutFreeMoneyForAll.dat"});
  }


public static void main(String [] args) throws Exception {

    runTimePriceShares_programs();

    // https://arxiv.org/abs/1906.00460
    PrintChristoffel.main(new String []{"25"});
    ExampleImageInterpolation2.main(new String []{"file:dataexamples/lena512.bmp","25","25","34"});

    // Test Radon Nikodym. Small part of https://arxiv.org/abs/1611.07386 and https://arxiv.org/abs/1807.06007
    final String fname_data="slope_800_200.csv";
    // generate a two--stage file
    PrintFunTwoLinearStages.main(new String[]{fname_data,"10000"/*points*/,"1000"/*length*/,"800"/*cut*/,"1e-4"/*slope1*/,"5e-4"/*slope2*/,"0"/*randomness*/});
    // Apply least squares and Radon-Nikodym interploations, also obtain the spectrum. https://arxiv.org/abs/1611.07386
    ExampleRadonNikodym_F_and_DF.main(new String[]{fname_data,"50","sampleDX"});
    // https://arxiv.org/abs/1807.06007 Fig. 1e, a histogram with eigenvalues number as the weight.
    HistogramDistribution.main(new String[]{ExampleRadonNikodym_F_and_DF.getFPrefix(fname_data)+"QQdf_QQ_spectrum.dat","5:1:-1","25"});
    // https://arxiv.org/abs/1807.06007 Fig. 1f, a histogram with Lebesgue quadrature weight.
    // (column with index 3)
    HistogramDistribution.main(new String[]{ExampleRadonNikodym_F_and_DF.getFPrefix(fname_data)+"QQdf_QQ_spectrum.dat","5:1:3","25"});
    // Christoffel Weight (column with index 4)
    HistogramDistribution.main(new String[]{ExampleRadonNikodym_F_and_DF.getFPrefix(fname_data)+"QQdf_QQ_spectrum.dat","5:1:4","25"});
    // Radon--Nikodym With Optimal Clustering: https://arxiv.org/abs/1906.00460
    com.polytechnik.utils.RN.main(new String[]{"--data_cols=9:0,6:7:8:1","--data_file_to_build_model_from=dataexamples/runge_function.csv","--clusters_number=3"});

  }
}
