package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.Trade; // (time,price,shares) container
import com.polytechnik.realtimebook.TraderInterface; // for TIME_SCALE=1000000000L
import com.polytechnik.trading.PnLInPsiHstate;
import com.polytechnik.trading.PnLInPsiHstateLegendreShifted;
import com.polytechnik.trading.PnLInPsiHstateLaguerre;
import com.polytechnik.trading.PnLInPsiHstateMonomials;
import com.polytechnik.trading.IntervalSincePsiM;
import com.polytechnik.trading.PsiMSelection;
import com.polytechnik.trading.DaDv;
import com.polytechnik.trading.ExperimentaldIHdt;


/** A test.
 * @version $Id: TestCall_PnLInPsiHstate.java,v 1.192 2022/09/30 10:26:13 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class TestCall_PnLInPsiHstate {

private static int _outputDataHeader(final java.io.PrintWriter out)
{
    int n=0;
    out.println(
	"|1:T\t2:shares\t3:P_read"
	+"\t4:"+"pv_average"
	+"\t5:"+"pt_average"
	+"\t6:"+"Tv_average"
	+PrintFields.getEVXDataHeader("I",7)
	+"\t13:"+"pi_IH"
	+PrintFields.getPFutureHeader("pFV.",(n=14+42)-42)

	//+PrintFields.getTwoTopIHeader("twoI.",44)
	+PrintFields.getIntervalSincePsiMHeader("SM.",(n+=74)-74)
	+PrintFields.getIntervalSincePsiMHeader("SMVT.",(n+=74)-74)
	//+PrintFields.getDaDvHeader("dadv.",14)
	//+PrintFields.getDaDvHeader("dadt.",26)
	// other fields
	//+PrintFields.getIntervalSincePsiMHeader("SM.",38)
	//+PrintFields.getIntervalSincePsiMHeader("SMVT.",53)
	//+PrintFields.getPsiMSelectionHeader("PSIM.",53)
	//+PrintFields.getIntervalSincePsiMHeader("SMTimVT_H.",57)
	//+PrintFields.getIntervalSincePsiMHeader("SMTimVT_L.",103)

		);
    return 1;
}

private static int _outputData(final java.io.PrintWriter out,final long currenttime,final Trade t,final PnLInPsiHstate<?> r)
{
    out.println(String.valueOf(currenttime)+"\t"+t.shares+"\t"+t.price
		+"\t"+r.pv_average
		+"\t"+r.pt_average
		+"\t"+r.Tv_average
		+PrintFields.getEVXDataData(r.I)
		+"\t"+r.pi_IH
		+PrintFields.getPFutureData(r.pFV)

		//+PrintFields.getTwoTopIData(r.twoI)
		+PrintFields.getIntervalSincePsiMData(r.SM)
		+PrintFields.getIntervalSincePsiMData(r.SMVT)

		//+PrintFields.getIntervalSincePsiMData(r.SM)
		//+PrintFields.getDaDvData(r.dadv)
		//+PrintFields.getDaDvData(r.dadt)
		// other fields

		//+PrintFields.getPsiMSelectionData(r.PSIM)
		//+PrintFields.getIntervalSincePsiMData(r.SMTimVT_H)
		//+PrintFields.getIntervalSincePsiMData(r.SMTimVT_L)

		);
    return 1;
}


@FunctionalInterface interface CalculateOnePoint<RES> {
    RES calculate(final int neq,
		  final long currenttime,
		  final long tau,
		  final Trade [] tb,
		  final int tsize,
		  final RES ref);
}

public static class TradeDataProcessorTPV<RES extends PnLInPsiHstate<?>> extends TradeDataProcessor<RES> {
    private RES ref=null;
    private final CalculateOnePoint<RES> f;

    public TradeDataProcessorTPV(
	final MuseConfig c,
	final CalculateOnePoint<RES> f){
	super(c);
	this.f=f;
    }
    @Override public int outputDataHeader(final java.io.PrintWriter out) throws java.io.IOException {return _outputDataHeader(out);}
    @Override public int outputDataFooter(final java.io.PrintWriter out) throws java.io.IOException {
	out.println("| calculations done");
	return 1;
    }

    @Override public int processOneReadLine(final java.io.PrintWriter out,final long currenttime,final Trade t) throws java.io.IOException{
	int nl=0;
	try {
	    nl+=_outputData(out,currenttime,t,calculateData(currenttime,t));
	} catch(Exception e) {
	    out.close();
	    throw new RuntimeException(e);
	}
	return nl;
    }

    public RES calculateData(final long currenttime,final Trade t){
	// actual trades has >0 shares
	if(t.shares>0) addTradeToBuffer(t);
	final RES r=f.calculate(c.n,currenttime,
				c.tau*TraderInterface.TIME_SCALE,
				this.buf,this.n_trades_in_buffer,this.ref);
	this.ref=r;
	return r;
    }
} // END OF class TradeDataProcessorTPV


public static void main(final String [] args) throws java.io.IOException
{
    final MuseConfig c=new MuseConfig(args);
    final java.util.List<String> errs=c.validateOptions();
    if(errs.size()>0)
    {
	System.err.println("Invalid or not set arguments:\n"+String.join("\n",errs));
	System.exit(1);
    }
    else if(c.measure.equals("PnLInPsiHstateLegendreShifted"))
    {
	new TradeDataProcessorTPV<PnLInPsiHstateLegendreShifted>(c,PnLInPsiHstateLegendreShifted::calculate).processFiles();
    }
    else if(c.measure.equals("PnLInPsiHstateLaguerre"))
    {
	new TradeDataProcessorTPV<PnLInPsiHstateLaguerre>(c,PnLInPsiHstateLaguerre::calculate).processFiles();
    }
    else if(c.measure.equals("PnLInPsiHstateMonomials"))
    {
	new TradeDataProcessorTPV<PnLInPsiHstateMonomials>(c,PnLInPsiHstateMonomials::calculate).processFiles();
    }
    else throw new RuntimeException("The measure "+c.measure+" is not supported, allowed values:{PnLInPsiHstateLegendreShifted,PnLInPsiHstateLaguerre,PnLInPsiHstateMonomials}");
}

} // END OF class TestCall_PnLInPsiHstate
