package com.polytechnik.algorithms;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.Trade; // (time,price,shares) container

/** The class actually performs calculations
 * and outputs the results.
 * @version $Id: TradeDataProcessor.java,v 1.17 2024/02/24 12:22:44 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class TradeDataProcessor<RES> {
    public final MuseConfig c;
    public TradeDataProcessor(final MuseConfig c){this.c=c;}
    final int BUFSIZE=500,MINTRADES=20;
    protected final Trade [] buf=new Trade [BUFSIZE];
    protected int n_trades_in_buffer=0;
    protected void addTradeToBuffer(final Trade t)
    {
	if(n_trades_in_buffer>=BUFSIZE)
	{
	    final int ntomove1=MINTRADES;
	    final Trade tb=buf[n_trades_in_buffer-ntomove1],te=buf[n_trades_in_buffer-1];
	    System.arraycopy(buf,n_trades_in_buffer-ntomove1,buf,0,ntomove1);
	    java.util.Arrays.fill(buf,ntomove1,n_trades_in_buffer,null);
	    n_trades_in_buffer=ntomove1;
	    if(tb!=buf[0]||te!=buf[n_trades_in_buffer-1]) throw new RuntimeException("Broken Invariant");
	}
	if(buf[n_trades_in_buffer]!=null) throw new RuntimeException("Broken Invariant");
	buf[n_trades_in_buffer++]=t;
    }
    public int outputDataFooter(final java.io.PrintWriter out) throws java.io.IOException {return 0;}
    public int outputDataHeader(final java.io.PrintWriter out) throws java.io.IOException {return 0;}

    public static Trade createTradeFromTimePriceSharesTraded(final long time,final double price,final int shares){
	return Trade.getDummyTrade(price,shares,Trade.BOOK_UNKNOWN,time);
    }

    public abstract int processOneReadLine(final java.io.PrintWriter out,final long currenttime,final Trade t) throws java.io.IOException;

    /** Reads tab--separated file.
     * For each line read perform calculations and output the results.
     */
    public void processFiles() throws java.io.IOException
    {
	final long t0=System.currentTimeMillis();
	System.err.println(getClass().getName()+": using config\n"+c.getDescription());
	System.err.println("Reading from file="+c.musein_file);
	final java.io.BufferedReader in=new java.io.BufferedReader(
	    c.musein_file.toLowerCase().endsWith(".gz")?
	    new java.io.InputStreamReader(new java.util.zip.GZIPInputStream(new java.io.FileInputStream(c.musein_file),1024*128))
	    :
	    new java.io.FileReader(c.musein_file));
	System.err.println("Writing to file="+c.museout_file);
	final java.io.PrintWriter out=new java.io.PrintWriter(c.museout_file);
	long n_lines_read=0,n_lines_printed=0,prevtime=-1;
	n_lines_printed+=outputDataHeader(out);
	final java.io.StreamTokenizer tk=com.polytechnik.utils.ReadWriteUtils.getTokenizer("\t",in);
        tk.quoteChar('\"');
	for(;;)
	{
	    final String [] s=com.polytechnik.utils.ReadWriteUtils.readOneLine(tk);
	    if(s==null) break; // end of file
	    n_lines_read++;
	    //System.err.println("n_lines_read="+n_lines_read+" s.length="+s.length+" s="+java.util.Arrays.toString(s)+" c.indcol_T="+c.indcol_T+" c.indcol_P="+c.indcol_P+" c.indcol_V="+c.indcol_V);
	    if(s.length<=0||s[0].startsWith("|")) continue; // comments
	    if(s.length!=c.columns_number) throw new RuntimeException("File="+c.musein_file+" line#="+n_lines_read+" the number of cols="+s.length+" is not "+c.columns_number+" records from string="+java.util.Arrays.toString(s));
	    // Read time,price,shares traded from a tab-separated file
	    final long time=Long.valueOf(s[c.indcol_T]);
	    final double price=Double.valueOf(s[c.indcol_P]);
	    final int shares=Integer.valueOf(s[c.indcol_V]);
	    if(time<prevtime) throw new RuntimeException("File="+c.musein_file+" line#="+n_lines_read+" the data is not ordered: time="+Trade.getTimeDiag(time)+" prevtime="+Trade.getTimeDiag(prevtime)+" records from string="+java.util.Arrays.toString(s));
	    prevtime=time;
	    // Obtain Trade object with time,price,shares traded data only.
	    final Trade t=createTradeFromTimePriceSharesTraded(time,price,shares);
	    // Perform the calculations for this new trade and output the result to a file.
	    n_lines_printed+=processOneReadLine(out,time,t);
	    // Diag output to stderr every 1000 lines read.
	    if(n_lines_read/1000*1000==n_lines_read) System.err.println(
		"Processed "+n_lines_read+" lines of "+c.musein_file+
		" itch_time="+Trade.getTimeDiag(time)+" computer_time="+new java.util.Date());
	}
	n_lines_printed+=outputDataFooter(out);
	System.err.println("Closing output file="+c.museout_file+" ; Saved "+n_lines_printed+" lines.");
	out.close();
	System.err.println("Closing input file="+c.musein_file+" ; Read "+n_lines_read+" lines.");
	in.close();
	System.err.println("Processed musein_file="+c.musein_file+" "+n_lines_read+" lines. Saved to museout_file="+c.museout_file+" "+n_lines_printed+" lines. "+
                           "Finished in "+((System.currentTimeMillis()-t0)/1000L)+
                           " seconds.\nProcessed by $Id: TradeDataProcessor.java,v 1.17 2024/02/24 12:22:44 mal Exp $");
    }
} // END OF class TradeDataProcessor
