package com.polytechnik.algorithms;


/** Calculate R(gamma_1,gamma_2) for our solar paper.
 */
class with_V_M_Makarov_PrintRgammas {

    static final int N=20;
    static final double q=0.0000001;

    static double getFx(final double r,final int N,final double gamma1,final double gamma2)
    {
	final double er=Math.exp(-r),en1r=Math.exp(-r*(N+1)),
	    eq=Math.exp(-q),en1q=Math.exp(-q*(N+1));
//	System.err.println("rs="+(gamma1*(er-en1r)/(1-er))+" er="+er+" en1r="+en1r+
//			   " N="+N+
//			   " gamma1="+gamma1+" r="+r+" gamma2="+gamma2);
//	System.err.println("ee="+((eq-en1q)/(1-eq)));
	return gamma1*(er-en1r)/(1-er)-gamma2*(eq-en1q)/(1-eq);
    }

    static class BS extends com.polytechnik.outils.BissectionRootFinder {
	final double gamma1,gamma2;
	final int n;
	BS(final double gamma1,final double gamma2,final int n)
	{
	    this.gamma1=gamma1;
	    this.gamma2=gamma2;
	    this.n=n;
	}
	public double getF(final double r)
	{
//	    System.err.println("r="+r+" f="+(getFx(r,n,gamma1,gamma2)-1));
	    return getFx(r,n,gamma1,gamma2)-1;
	}
    }

    static class BSGamma extends com.polytechnik.outils.BissectionRootFinder {
	final double gamma1,r;
	final int n;
	BSGamma(final double gamma1,final double r,final int n)
	{
	    this.gamma1=gamma1;
	    this.r=r;
	    this.n=n;
	}
	public double getF(final double gamma2)
	{
//	    System.err.println("r="+r+" f="+(getFx(r,n,gamma1,gamma2)-1));
	    return getFx(r,n,gamma1,gamma2)-1;
	}
    }

    static class BSR extends com.polytechnik.outils.BissectionRootFinder {
	final double gamma1,gamma2;
	final int n;
	BSR(final double gamma1,final double gamma2,final int n)
	{
	    this.gamma1=gamma1;
	    this.gamma2=gamma2;
	    this.n=n;
	}
	public double getF(final double r)
	{
//	    System.err.println("r="+r+" f="+(getFx(r,n,gamma1,gamma2)-1));
	    return getFx(r,n,gamma1,gamma2)-1;
	}
	public static void main(String [] args)
	{

	    final double gamma1=Double.valueOf(args[0]),
		gamma2=Double.valueOf(args[1]);
	    final double r=new BSR(gamma1,gamma2,N).doBissection(-1,1);
	    System.out.println("gamma1="+gamma1+" gamma2="+gamma2+" r="+r);
	}
    }


    public static void main(String [] args)
    {
	final double [] rs=new double []{0.00000001,0.01,0.02,0.05,0.1,0.15,0.3};
	System.out.print("|gamma1");
	for(double r:rs) System.out.print("\t|r_"+r);
	System.out.println();

	for(double gamma1=0;gamma1<=0.2;gamma1+=0.001)
	{
	    System.out.print(""+gamma1);
	    for(double r:rs)
	    {
		final double gamma2=new BSGamma(gamma1,r,N).doBissection(-10,10);
		System.out.print("\t"+gamma2);
	    }
	    System.out.println();
	}
    }
}

