package com.polytechnik.broker;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.BookItem;
import com.polytechnik.realtimebook.OrdersFlow;

/** A class for a order for execution. Instances are created by us.
 *  @version $Id: AOrder.java,v 1.42 2012/01/24 19:08:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class AOrder extends BOrder {

    private static final boolean PRINT_DATA_DEBUG=true;
    static final int SET_ADDED=100,SET_CANCELLED=200,SET_REJECTED=300;

    static final int
	STATUS_UNSET=-1,
	STATUS_MESSAGEPOSTED=31,
	STATUS_CONFIRMED=41;

    /** Number of shares.
     */
    final int original_shares;
    final long original_time;
    public final OrderAttributes attributes;
    private final AOrderCancellationCheck checkcancellation;

    private int flag_added,flag_cancelled,flag_rejected;

    public AOrder(final long time,
		  final AOrderCancellationCheck checkcancellation,
		  final double price,
		  final int shares,
		  final int side,
		  final OrderAttributes attributes)
    {
	super(-1,time,price,shares,side);
	this.checkcancellation=checkcancellation;
	this.original_shares=shares;
	this.original_time=time;
	this.attributes=attributes;
	this.flag_added=this.flag_cancelled=this.flag_rejected=STATUS_UNSET;
	//if(this.checkcancellation!=null) this.checkcancellation.setCallbackReference(this);
    }

    public boolean isCancelled()
    {
	return
	    this.flag_cancelled==AOrder.STATUS_CONFIRMED ||
	    this.flag_rejected==AOrder.STATUS_CONFIRMED ||
	    getShares()==0
	    ;
    }

    boolean isAlive(final long t,final double p_buy,final double p_sell)
    {
	if(!isCancelled() &&
	   this.flag_cancelled!=AOrder.STATUS_MESSAGEPOSTED &&
	   this.flag_rejected!=AOrder.STATUS_MESSAGEPOSTED &&
	   (this.checkcancellation==null ||
	    this.checkcancellation.shouldOrderBeAlive(this,t,p_buy,p_sell)))
	{
	    return true;
	}
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println(getClass().getName()+":isAlive:"+
			       " cancelled="+isCancelled()+
			       " shouldOrderBeAlive="+(this.checkcancellation==null?"N/A":
						       "C="+this.checkcancellation.shouldOrderBeAlive(this,t,p_buy,p_sell))+
			       " getDescription()="+getDescription());
	}
	return false;
    }


    String getDescription()
    {
	return getClass().getName()+" "+this.attributes.ticker+" id="+getID()+" price="+this.price+
	    " shares_signed="+(getSharesSign()*getShares())+
	    " shares="+getShares()+" original_shares="+this.original_shares+
	    " time="+BookItem.getTimeDiag(getTime())+
	    " original_time="+BookItem.getTimeDiag(this.original_time)+
	    " side="+this.side+" sign="+getSharesSign()+
	    " cancelled="+isCancelled()+
	    " flag_added="+this.flag_added+" flag_cancelled="+this.flag_cancelled+
	    " flag_rejected="+this.flag_rejected+
	    " attributes="+this.attributes.getDescription();
    }


    void setAttrib(final int attribname,final int value)
    {
	if(attribname==SET_ADDED)
	{
	    if(this.flag_added<value)
	    {
		this.flag_added=value;
	    }
	}
	else if(attribname==SET_CANCELLED)
	{
	    if(this.flag_cancelled<value)
	    {
		this.flag_cancelled=value;
	    }
	}
	else if(attribname==SET_REJECTED)
	{
	    if(this.flag_rejected<value)
	    {
		this.flag_rejected=value;
	    }
	}
	else
	{
	    throw new IllegalArgumentException();
	}
    }
}
