package com.polytechnik.broker;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** Similar to ITCH RealtimeBook.
 *  @version $Id: AOrderBook.java,v 1.28 2022/07/28 22:30:47 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class AOrderBook extends com.polytechnik.utils.LongHashMap<AOrder> {

  public static class OrderInfo {
      public final int shares_buys,n_buys,shares_sells,n_sells;
      public final long [] orders_active,orders_to_die;
      OrderInfo(final OrderStats os)
      {
	  this.shares_buys=os.shares_buy;
	  this.n_buys=os.n_buy;
	  this.shares_sells=os.shares_sell;
	  this.n_sells=os.n_sell;
	  this.orders_active=os.n_active_orders>0?java.util.Arrays.copyOf(os.onotcancelled,os.n_active_orders):null;
	  this.orders_to_die=os.n_dead>0?java.util.Arrays.copyOf(os.odead,os.n_dead):null;
      }
  }

  AOrderBook()
  {
      super(29,0.541f);
  }

  public OrderInfo getInfo(final long time,
			   final double p_buy,
			   final double p_sell)
  {
      final OrderStats os=new OrderStats(size(),time,p_buy,p_sell);
      forEachEntry(os);
      return new OrderInfo(os);
    }


private static class OrderStats implements java.util.function.Predicate<AOrder> {

    final long time;
    final double p_buy,p_sell;
    final long [] odead,onotcancelled;

    OrderStats(final int size,
	       final long time,
	       final double p_buy,
	       final double p_sell)
    {
	this.odead=new long [size];
	this.onotcancelled=new long [size];
	this.time=time;
	this.p_buy=p_buy;
	this.p_sell=p_sell;
    }

    int shares_buy=0,
	n_buy=0,
	shares_sell=0,
	n_sell=0,
	n_active_orders=0,
	n_dead=0; //(expired, cancelled but not confirmed, etc) orders.
    public boolean test(final AOrder b)
    {
	if(b.isCancelled())
	{
	    return true;
	}
	onotcancelled[n_active_orders++]=b.getID();
	final int s=b.getShares();
	if(s<=0)
	{
	    System.err.println("Something is terrible wrong, shares="+s+" <0");
	    return true;
	}
	if(b.getSharesSign()>=0)
	{
	    shares_buy+=s; // buy
	    n_buy++;
	}
	else
	{
	    shares_sell+=s; // sell
	    n_sell++;
	}
	if(!b.isAlive(time,p_buy,p_sell))
	{
	    odead[n_dead++]=b.getID();
	}
	return true;
    }
}

}
