package com.polytechnik.broker;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** A trade (returned by broker).
 *  We do not have separate class used internally, as it is
 *  with AOrder and BOrder.
 *  @version $Id: ATrade.java,v 1.24 2012/01/18 10:18:54 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ATrade implements com.polytechnik.utils.CircularList.TimeAccess {


    public final /* long */ String id;
    private final long time;
    final long order_id;
    private AOrder order;
    private final String ticker;


    public final double price;

    /** Number of shares.
     */
    public final int shares;

    private final int side;

    final String extraproperties;

    public ATrade(final /* long */ String id,
		  final long time,
		  final long order_id,

		  final double price,

		  final int shares,
		  final int side,
		  final String extraproperties,
		  final String ticker)
    {
	this.id=id;
	this.time=time;
	this.order_id=order_id;
	this.price=price;

	this.shares=shares;
	this.side=side;
	this.extraproperties=extraproperties;
	this.ticker=ticker;
    }

    public long getTime()
    {
	return this.time;
    }

    public AOrder getOrder()
    {
	return this.order;
    }
    void setOrder(final AOrder order)
    {
	if(this.order!=null || this.order_id!=order.getID() || this.side!=order.side)
	{
	    throw new RuntimeException("Trade ticker="+this.ticker+" id="+this.id+" cannot set order="+order+
				       " id="+order.getID()+
				       " already set or mismatch from required order_id="+this.order_id+
				       " trade side="+this.side+" order side="+order.side+
				       " trade="+getDescription());
	}
	this.order=order;
    }


    String getDescription()
    {
	final AOrder ao=getOrder();
	final double pdiff;
	if(ao!=null)
	{
	    pdiff=ao.getSharesSign()>0?this.price-ao.price:ao.price-this.price;
	}
	else
	{
	    pdiff=Double.NaN;
	}
	final String ticker=this.ticker==null?this.ticker:"N/A";
	return getClass().getName()+" "+ticker+" id="+this.id+" price="+this.price+
	    " shares="+this.shares+" order_id="+this.order_id+
	    " time="+getTime()+" side="+this.side+" pdiff="+pdiff+
	    " extraproperties="+this.extraproperties+
	    (ao!=null?ao.getDescription():"order=null");
    }

}
