package com.polytechnik.broker;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** A circular buffer to store actual trades (confirmed by broker).
 *  @version $Id: ATradeList.java,v 1.18 2012/01/18 17:34:34 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ATradeList extends java.util.ArrayList<ATrade> {


    private final Positions pref;
    ATradeList(final Positions pref)
    {
	super(20);
	this.pref=pref;
    }

    public static class TradeInfo {
	public final double money_p_and_l;
	public final double unrealized_p_and_l;
	public final int shares;
	public final double current_money_p_and_l;
	public final double current_unrealized_p_and_l;
	public final int current_shares;
	TradeInfo(final double money_p_and_l,
		  final double unrealized_p_and_l,
		  final int shares,
		  final double current_money_p_and_l,
		  final double current_unrealized_p_and_l,
		  final int current_shares)
	{
	    this.money_p_and_l=money_p_and_l;
	    this.unrealized_p_and_l=unrealized_p_and_l;
	    this.shares=shares;
	    this.current_money_p_and_l=current_money_p_and_l;
	    this.current_unrealized_p_and_l=current_unrealized_p_and_l;
	    this.current_shares=current_shares;
	}
    }


    void initiateNewStrategy(final long time)
    {
	add(new ATrade("SEPARATOR",
		       time,
		       -1,
		       Double.NaN,
		       -1,
		       -1,
		       null,null));
    }
    static boolean isSeparator(final ATrade b)
    {
	return b.order_id==-1 &&
	    b.shares==-1 &&
	    Double.isNaN(b.price) &&
	    b.getOrder()==null;
    }


    public TradeInfo getInfo(final long time,
			     final double p_buy,
			     final double p_sell)
    {
	int total_shares=0,last_strategy_shares=0;
	double total_p_l=0,last_strategy_p_l=0;
	for(ATrade b : this)
	{
	    if(isSeparator(b))
	    { // separator
		if(last_strategy_shares!=0)
		{
		    throw new IllegalStateException("Marked as closed a strategy is not actually closed shares="+last_strategy_shares+"\n"+this.pref.dumpData());
		}
		last_strategy_shares=0;
		last_strategy_p_l=0;
	    }
	    else
	    { // actual trade
		final int s=b.shares*b.getOrder().getSharesSign();
		final double v=s*b.price;
		total_shares+=s;
		last_strategy_shares+=s;
		total_p_l-=v;
		last_strategy_p_l-=v;
	    }
	}
	if(total_shares!=last_strategy_shares)
	{
	    throw new IllegalStateException(
		"Marked as closed a strategy"+
		" is not actually closed shares_1="+total_shares+
		" shares_2="+last_strategy_shares+"\n"+this.pref.dumpData());
	}
	final double curassetsprice=(total_shares>0?total_shares*p_buy:(total_shares<0?total_shares*p_sell:0.0));
	if(false)
	{
	    System.err.println(" total_p_l="+total_p_l+
			       " curassetsprice="+curassetsprice+
			       " total_shares="+total_shares+
			       " p_buy="+p_buy+
			       " p_sell="+p_sell);
	}
	return new TradeInfo(total_p_l,
			     total_p_l+curassetsprice,
			     total_shares,
			     last_strategy_p_l,
			     last_strategy_p_l+curassetsprice,
			     last_strategy_shares);
    }

}
