package com.polytechnik.broker;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** A class for a order for execution. Instances are created by broker.
 *  @version $Id: BOrder.java,v 1.12 2010/08/30 12:06:08 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class BOrder {

    public static final int
	SIDE_BUY=75100,
	SIDE_BUY_COVER=75200,
	SIDE_SELL=75300,
	SIDE_SELL_SHORT=75400;

    private long id;
    private long time;
    public final double price;

    /** Number of shares.
     */
    private int shares;
    public final int side;


    public BOrder(final long id,
		  final long time,
		  final double price,
		  final int shares,
		  final int side)
    {
	this.id=id;
	this.time=time;
	this.price=price;
	this.shares=shares;
	if(shares<0)
	{
	    throw new IllegalArgumentException("shares="+shares);
	}
	this.side=side;
	getSharesSign(); // check side valid values
    }

    public long getID()
    {
	return this.id;
    }
    void setID(final long id)
    {
	if(this.id>=0)
	{
	    throw new RuntimeException("Order id="+this.id+" cannot be reset to"+id);
	}
	this.id=id;
    }



    public int getShares()
    {
	if(this.shares<0)
	{
	    throw new IllegalStateException();
	}
	return this.shares;
    }
    void setShares(final int new_shares)
    {
	this.shares=new_shares;
    }

    public long getTime()
    {
	return this.time;
    }
    void setTime(final long time)
    {
	this.time=time;
    }

    public int getSharesSign()
    {
	if(this.side==BOrder.SIDE_BUY||this.side==BOrder.SIDE_BUY_COVER)
	{
	    return 1;
	}
	else if(this.side==BOrder.SIDE_SELL||this.side==BOrder.SIDE_SELL_SHORT)
	{
	    return -1;
	}
	throw new RuntimeException("Invalid side="+this.side+
				   " for order="+this.id+
				   " shares="+this.shares);
    }
}
