package com.polytechnik.broker;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.TraderInterface;
import com.polytechnik.realtimebook.BookItem;

/** Money manager.
 *  @version $Id: MoneyManager.java,v 1.80 2012/02/04 13:06:37 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class MoneyManager {

    /** A fraction to calculate time when next predictor is allowed.
     *  t_allowed=t0+tau_minpredictor/frac_nextpredictor_allowed.
     */
    public final long frac_nextpredictor_allowed=2;

    public final double fraction_signal_normalexit=0.06;
    public final double fraction_signal_speedexit=0.04;

    private TraderContainer sd=null;

    private int round_lot_shares=100;
    private boolean round_lot_only=false;
    private String market=null;

    final int N_INCREASE_POSITION=1;

    private final int MAX_LONG_SHARES,MAX_SHORT_SHARES,
	INCREMENT_LONG,
	INCREMENT_SHORT;

    private long T_MIDNIGHT_MILLISECONDS=-1;

    public MoneyManager(final int MAX_LONG_SHARES,
			final int MAX_SHORT_SHARES)
    {
	this.MAX_LONG_SHARES=MAX_LONG_SHARES;
	this.MAX_SHORT_SHARES=MAX_SHORT_SHARES;
	this.INCREMENT_LONG=MAX_LONG_SHARES/N_INCREASE_POSITION;
	this.INCREMENT_SHORT=MAX_SHORT_SHARES/N_INCREASE_POSITION;
    }

    void setTraderContainer(final TraderContainer sd)
    {
	this.sd=sd;
	if(this.sd.flag_realtime)
	{
	    // midnight time in milliseconds
	    T_MIDNIGHT_MILLISECONDS=BookItem.getSystemTimeMidnight00Milliseconds();
	}
	else
	{
	    T_MIDNIGHT_MILLISECONDS=0;
	}
    }

    private Positions.PositionsInfo pinfo=null;

    public void notifyPositionsInfo(final Positions.PositionsInfo pinfo)
    {
	if(this.pinfo!=null && pinfo!=null)
	{
	    throw new IllegalStateException("Race is somewhere for ticker="+this.sd.ticker+
					    " money manager.");
	}
	this.pinfo=pinfo;
    }


    public void setRountLotSizes(final int round_lot_shares,
				 final boolean round_lot_only,
				 final String market)
    {
	this.round_lot_shares=round_lot_shares;
	this.round_lot_only=round_lot_only;
	this.market=market;
    }


    public int getAvailableLong(final long time,
				final double p_buy,
				final double p_sell,
				final int estimate_avail_shares_buy,
				final int estimate_avail_shares_sell)
    {
	final int s=this.pinfo.getEffShares();
	final int ss=s+INCREMENT_LONG;
	if(ss<=MAX_LONG_SHARES)
	{
	    return INCREMENT_LONG;
	}
	final int sa=MAX_LONG_SHARES-ss;
	return sa>0?sa:0;
    }

    public int getAvailableShort(final long time,
				 final double p_buy,
				 final double p_sell,
				 final int estimate_avail_shares_buy,
				 final int estimate_avail_shares_sell)
    {
	final int s=-this.pinfo.getEffShares();
	final int ss=s+INCREMENT_SHORT;
	if(ss<=MAX_SHORT_SHARES)
	{
	    return INCREMENT_SHORT;
	}
	final int sa=MAX_SHORT_SHARES-ss;
	return sa>0?sa:0;
    }

    public double getMaxLoss()
    {
	return Double.NaN; // disable stoploss for now 0.996; // 0.4%
    }

    public long getTimeOrderToDie_ClosePostion(final long time)
    {
	// if an order to close trade was not executed in 60 seconds - cancel it and try again.
	return time+300*TraderInterface.TIME_SCALE;
    }

    public long getTimeOrderToDie_Opportunity(final long time)
    {
	// if an opportunity trade was not executed in 60 seconds - cancel it.
	return time+60*TraderInterface.TIME_SCALE;
    }

    public long getTimeNextCheck_OpenPostition(final long time)
    {
	// once we have a position - check it every 0.1 seconds
	return time+TraderInterface.TIME_SCALE/10; //* 10 */; // 1 second
    }

    public long getTimeNextCheck_WatchOpportunity(final long time)
    {
	return time+TraderInterface.TIME_SCALE/10; // 1 seconds
    }

    private long getTimeToStart()
    {
	return TraderInterface.TIME_SCALE*(9*3600+30*60+10); // 9:30:10
    }


    private long getTimeToEnd()
    {
	return TraderInterface.TIME_SCALE*(15*3600+53*60); // 15:53
	//return TraderInterface.TIME_SCALE*(10*3600+1*60); // 10:01
    }

    static final long DIFF_WITH_CLOCK=1*TraderInterface.TIME_SCALE; // 1 sec difference max

    static long FREQUENCY_REPORTING=30*TraderInterface.TIME_SCALE; // dump trades & orders every 30 sec
    private long trepout=0,trepin=0;
    public boolean isTimeOK(final long time)
    {
	if(this.sd.flag_realtime)
	{
	    // convert miliseconds to the same scale as internal time
	    final long t00=(System.currentTimeMillis()-T_MIDNIGHT_MILLISECONDS)*
		(TraderInterface.TIME_SCALE/1000);
	    final long d=time-t00;
	    if(Math.abs(d)>DIFF_WITH_CLOCK)
	    {
		if(time>=trepout)
		{
		    System.err.println(
			getClass().getName()+" ticker="+this.sd.ticker+
			" clock OUT OF SYNC:"+
			" system time="+BookItem.getTimeDiag(t00)+
			" ITCH time="+BookItem.getTimeDiag(time)+
			" diff="+d);
		    trepout=time+FREQUENCY_REPORTING;
		}
		return false;
	    }
	    else
	    {
		if(time>=trepin)
		{
		     System.err.println(
			getClass().getName()+" ticker="+this.sd.ticker+
			" clock IN SYNC:"+
			" system time="+BookItem.getTimeDiag(t00)+
			" ITCH time="+BookItem.getTimeDiag(time)+
			" diff="+d);
		     trepin=time+FREQUENCY_REPORTING;
		}
	    }
	}
	return getTimeToStart()<time &&  time<getTimeToEnd();
    }

}
