package com.polytechnik.broker;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.realtimebook.BookItem;
import com.polytechnik.realtimebook.TraderInterface;

/** To hold current positions.
 *  @version $Id: Positions.java,v 1.83 2020/02/08 10:42:26 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class Positions {

    public static class PositionsInfo {
	public final long time;
	public final double p_buy,p_sell;
	public final long avail_buy,avail_sell;
	public final AOrderBook.OrderInfo order_info;
	public final ATradeList.TradeInfo trade_info;
	public final String ticker;

	private PositionsInfo(final long time,
			      final double p_buy,
			      final double p_sell,
			      final long avail_buy,
			      final long avail_sell,

			      final AOrderBook.OrderInfo order_info,
			      final ATradeList.TradeInfo trade_info,
			      final String ticker)
	{
	    this.time=time;
	    this.p_buy=p_buy;
	    this.p_sell=p_sell;
	    this.avail_buy=avail_buy;
	    this.avail_sell=avail_sell;
	    this.order_info=order_info;
	    this.trade_info=trade_info;
	    this.ticker=ticker;
	}
	public int getEffShares()
	{
	    return
		this.trade_info.current_shares+
		(this.order_info.shares_buys-this.order_info.shares_sells);
	}

	/*
	public boolean hasPendingOrders()
	{
	    return this.order_info.orders_active!=null;
	}
	*/

	public String getMarketInfo()
	{
	    return "MARKET:ticker="+this.ticker+" time="+BookItem.getTimeDiag(this.time)+
		" p_buy="+this.p_buy+
		" p_sell="+this.p_sell+
		" MONEY: spent="+this.trade_info.current_money_p_and_l+
		" unrel P&L="+this.trade_info.current_unrealized_p_and_l+
		" effshares="+this.getEffShares()+
		" trades_shares="+this.trade_info.current_shares+
		" ORDERS: shares_buys="+this.order_info.shares_buys+"("+this.order_info.n_buys+")"+
		" shares_sells="+this.order_info.shares_sells+"("+this.order_info.n_sells+")"+
		" BOOK: av_vbuy="+this.avail_buy+
		" av_vsell="+this.avail_sell;
	}
    }




    private final String ticker;
    final AOrderBook book=new AOrderBook();
    final ATradeList trades=new ATradeList(this);
    public Positions(final String ticker)
    {
	this.ticker=ticker;
    }

    public PositionsInfo getInfo(final long time,
				 final double p_buy,
				 final double p_sell,
				 final long avail_buy,
				 final long avail_sell)
    {

	return new PositionsInfo(time,
				 p_buy,
				 p_sell,
				 avail_buy,
				 avail_sell,
				 this.book.getInfo(time,p_buy,p_sell),
				 this.trades.getInfo(time,p_buy,p_sell),
				 this.ticker);
    }

    public void initiateNewStrategy(final long time)
    {
	this.trades.initiateNewStrategy(time);
    }

    public String dumpData()
    {
	final StringBuilder buf=new StringBuilder();
	buf.append("TICKER:"+this.ticker+"\n");
	buf.append("ORDERS:"+this.book.size()+"\n");
	this.book.forEachEntry(new java.util.function.Predicate<AOrder>(){
		public boolean test(final AOrder b)
		{
		    buf.append(b.getDescription());
		    buf.append("\n");
		    return true;
		}
	    });
	buf.append("TRADES:"+this.trades.size()+"\n");
	for(ATrade b : this.trades)
	{
	    buf.append(b.getDescription());
	    buf.append("\n");
	}
	return buf.toString();
    }

    private static final java.util.Comparator<AOrder> CMP_ORDERS_BY_TIME=
	new java.util.Comparator<AOrder>(){
	public int compare(final AOrder o1,final AOrder o2)
	{
	    final long t1=o1.original_time,t2=o2.original_time;
	    if(t1<t2)
	    {
		return -1;
	    }
	    if(t2>t2)
	    {
		return 1;
	    }
	    final long id1=o1.getID(),id2=o2.getID();
	    if(id1<id2)
	    {
		return -1;
	    }
	    if(id1>id2)
	    {
		return 1;
	    }
	    return 0;
	}
    };

    public void dumpOrdersCSV(final java.io.PrintWriter out,final long t,final double p_buy,final double p_sell)
    {
	final AOrderBook.OrderInfo oi=this.book.getInfo(t,p_buy,p_sell);
	out.println("Ticker\t"+this.ticker);
	out.println("Date\t"+BookItem.getTimeDiag(t));
	out.println("SystemDate\t"+new java.util.Date());
	out.println("Shares buy\t"+oi.shares_buys);
	out.println("Shares sell\t"+oi.shares_sells);
	final AOrder [] orders=new AOrder [this.book.size()];
	this.book.copyValues(orders);
	com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(orders,0,orders.length-1,CMP_ORDERS_BY_TIME); // earliest at 0

	out.println("TIME\tID\tSHARES\tSIGN\tPRICE\tthours\t\tORIG_TIME\tORIG_SHARES\t\tTXT\t");
	for(AOrder o:orders)
	{
	    final int s=o.getShares()*o.getSharesSign();
	    out.println(
		BookItem.getTimeHHMMSS(o.getTime())+"\t"+
		o.getID()+"\t"+
		s+"\t"+
		o.getSharesSign()+"\t"+
		o.price+"\t"+
		(o.getTime()/3600.0/TraderInterface.TIME_SCALE)+"\t\t"+
		BookItem.getTimeHHMMSS(o.original_time)+"\t"+
		o.original_shares+"\t\t"+
		o.getDescription()+"\t");
	}
    }

    public void dumpTradesCSV(final java.io.PrintWriter out,final long t,final double p_buy,final double p_sell)
    {
	final ATradeList.TradeInfo ti=this.trades.getInfo(t,p_buy,p_sell);
	out.println("Ticker\t"+this.ticker);
	out.println("Date\t"+BookItem.getTimeDiag(t));
	out.println("SystemDate\t"+new java.util.Date());
	out.println("Shares\t"+ti.shares);
	out.println("Money\t"+ti.money_p_and_l);
	out.println("unrel P&L\t"+ti.unrealized_p_and_l);

	final StringBuilder sout=new StringBuilder();
	sout.append("TIME\tSHARES\tPRICE\tthours\tSTRATEGY P&L\tORDER COMMENTS\tTRADE COMMENTS\n");

	int n_plus=0,n_minus=0,shares_abs=0,total_shares=0,shares_strategy=0,shares_abs_strategy=0;
	double pl_plus=0,pl_minus=0,pl_strategy=0,pl=0;
	for(ATrade b : this.trades)
	{
	    if(ATradeList.isSeparator(b))
	    {
		if(shares_abs_strategy>0)
		{
		    if(pl_strategy>=0)
		    {
			n_plus++;
			pl_plus+=pl_strategy;
		    }
		    else
		    {
			n_minus++;
			pl_minus+=pl_strategy;
		    }
		}
		sout.append("\t\t\t\t"+pl_strategy+"\t"+shares_strategy+"\t\n");
		shares_strategy=0;
		shares_abs_strategy=0;
		pl_strategy=0;
	    }
	    else
	    { // actual trade
		final int s=b.shares*b.getOrder().getSharesSign();
		final double v=s*b.price;
		shares_abs_strategy+=b.shares;
		shares_abs+=b.shares;
		shares_strategy+=s;
		total_shares+=s;
		pl_strategy-=v;
		pl-=v;
		sout.append(BookItem.getTimeHHMMSS(b.getTime())+"\t"+
			    s+"\t"+
			    b.price+"\t"+
			    (b.getTime()/3600.0/TraderInterface.TIME_SCALE)+"\t\t"+
			    b.getOrder().getDescription()+"\t"+
			    b.extraproperties+
			    "\n");
	    }
	}

	final double curassetsprice=(total_shares>0?total_shares*p_buy:(total_shares<0?total_shares*p_sell:0.0));

	out.println("Approx Unrel P&L\t"+(pl+curassetsprice));
	out.println("Shares traded\t"+shares_abs);
	out.println("N+\t"+n_plus);
	out.println("N-\t"+n_minus);
	out.println("Nr\t"+(n_plus/((double)(n_plus+n_minus))));
	out.println("P+\t"+pl_plus);
	out.println("P-\t"+pl_minus);
	out.println("Pr\t"+(ti.unrealized_p_and_l/(Math.abs(pl_plus)+Math.abs(pl_minus))));
	out.println(sout);
	out.println();

	out.println("END");
    }


}
