package com.polytechnik.freemoney;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;

/** An integrator with interal state {@link WIntegratorLaguerre}
 * to calculate commonly used moments in Laguerre basis.
 * @version $Id: CommonlyUsedMomentsLaguerre.java,v 1.23 2022/09/05 02:14:06 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class CommonlyUsedMomentsLaguerre extends CommonlyUsedMoments {

    public CommonlyUsedMomentsLaguerre(final int n,final double atau){
	super(n,atau);
    }

    
    /** An adapter to {@link WIntegratorLaguerre#getDtMoments}.
     */
    @Override public double [] getdtMoments(final int nmoms){
	return WIntegratorLaguerre.getDtMoments(nmoms);
    }
    /** Calculate \(\psi/2-d\psi/dx\),
     * an adapter to {@link WIntegratorLaguerre#getEDPsi}.
     * @see CommonlyUsedMomentsMonomials#getEDPsi
     */
    @Override public double [] getEDPsi(final double [] psi){
	return WIntegratorLaguerre.getEDPsi(psi);
    }
    /** Integrate \( \int_x^{\infty} P(x) \exp(-x) dx = R(x) \exp(-x) \),
     * then return \( R(x) \),
     * an adapter to {@link WIntegratorLaguerre#getPolynomialWIntegratedDt}.
     * @see CommonlyUsedMomentsMonomials#getPolynomialWIntegratedDt
     */
    @Override public double [] getPolynomialWIntegratedDt(final double [] polynomial){
	return WIntegratorLaguerre.getPolynomialWIntegratedDt(polynomial);
    }

    @Override public OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}

    @Override public WIntegratorLaguerre getEmptyWIntegrator(final int ntm,final double atau,final int nobservables){
	return new WIntegratorLaguerre(ntm,atau,nobservables);
    }

    @Override public double [] getDFMomentsFromF(final double [] Fmoments){
	return WIntegratorLaguerre.B.getL1FromL0(Fmoments);
    }

}
