package com.polytechnik.freemoney;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.Polynomials;

/** An integrator with interal state {@link WIntegratorMonomials}
 * to calculate commonly used moments in monomials basis.
 * @version $Id: CommonlyUsedMomentsMonomials.java,v 1.25 2022/09/05 02:14:06 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class CommonlyUsedMomentsMonomials extends CommonlyUsedMoments {

    public CommonlyUsedMomentsMonomials(final int n,final double atau){
	super(n,atau);
    }

    /** An adapter to {@link WIntegratorMonomials#getDtMoments}.
     */
    @Override public double [] getdtMoments(final int nmoms){
	return WIntegratorMonomials.getDtMoments(nmoms);
    }
    /** Calculate \(\psi/2+d\psi/dx\),
     * an adapter to {@link WIntegratorMonomials#getEDPsi}.
     * @see CommonlyUsedMomentsLaguerre#getEDPsi
     */
    @Override public double [] getEDPsi(final double [] psi){
	return WIntegratorMonomials.getEDPsi(psi);
    }
    /**  Integrate \( \int_{-\infty}^x P(x) \exp(x) dx = R(x) \exp(x) \),
     * then return \( R(x) \),
     * an adapter to {@link  WIntegratorMonomials#getPolynomialWIntegratedDt}.
     * @see CommonlyUsedMomentsLaguerre#getPolynomialWIntegratedDt
     */
    @Override public double [] getPolynomialWIntegratedDt(final double [] polynomial){
	return WIntegratorMonomials.getPolynomialWIntegratedDt(polynomial);
    }

    @Override public OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis(){
	    /** Multiply by -x, Radau root at 0, x is negative on measure support.
	     */
	    @Override public double [] getRadauMultiplied(final double [] nodes0)
	    {
		return TMatr.scaleVectorInPlace(Polynomials.getxP(nodes0),-1);
	    }
	};
    }

    @Override public WIntegratorMonomials getEmptyWIntegrator(final int ntm,final double atau,final int nobservables){
	return new WIntegratorMonomials(ntm,atau,nobservables);
    }

    @Override public double [] getDFMomentsFromF(final double [] Fmoments){
	final double [] dFmoments=new double [Fmoments.length];
	for(int k=0;k<Fmoments.length;k++){dFmoments[k]=((k<=0?0:k*Fmoments[k-1])+Fmoments[k]);}
	return dFmoments;
    }
}
