package com.polytechnik.freemoney;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.EVXData;

/**
 * Furure price.
 * @version $Id: PFuture.java,v 1.85 2022/09/30 16:19:34 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
*/
public class PFuture {

    /** Eigenproblem {@literal \( \left|I|\psi^{[i]}\right>=\lambda^{[i]}\left|\psi^{[i]}\right> \)}
     * solution in original basis \(Q_k\)
     * along with corresponding density matrix obtained from
     * \(\psi=\left|\psi^{[IH]}\right\rangle\)
     * as "since \(\psi^2\) till now".
     */
    public final IandDM ianddm;

    public final double I_M,Volume_M,Tv_M,Tv_spurM;

    /** The value of aggregated execution flow
	{@code V/T}=\(
	\frac{\mathrm{Spur}\left\|I\middle|\rho_{IH}\right\|}
	{\mathrm{Spur}\left\|\rho_{IH}\right\|}
	\)={@link #I_spurM}/{@link #one_spurM}.
    */
    public final double VdivT_M;


    /** The value of \(\left\langle\psi\middle|pI\middle|\psi\right\rangle\)/{@link #I_M}.
     */
    public final double pv_M;

    /** The value of \(\mathrm{Spur}\left\|pI\middle|\rho_{IH}\right\|\)/{@link #I_spurM}.
     */
    public final double pv_spurM;
    
    /** The value of \(
	\mathrm{Spur}\left\|I\middle|\rho_{IH}\right\|
	= 0-\left\langle\psi\middle|V\middle|\psi\right\rangle
     \)
     * = {@link #Volume_M}.
     */
    public final double I_spurM;

    /** The value of \(
	\mathrm{Spur}\left\|\rho_{IH}\right\|=
	0-\left\langle\psi\middle|T\middle|\psi\right\rangle
	\).
     */
    public final double one_spurM;

    
    /** The value of \(\left\langle\psi\middle|\frac{dp}{dt}\middle|\psi\right\rangle\).
     */
    public final double dp_M;

    /** The value of \(\mathrm{Spur}\left\|\frac{dp}{dt}\middle|\rho_{IH}\right\|\)=
     * \(P^{last}-\left\langle\psi\middle|p\middle|\psi\right\rangle\).
     */
    public final double dp_spurM;
    
    public final double P_last,pv_average,Tv_average,totalVolume;


    /** The value of
     *	\(\left\langle\psi\middle|V\frac{dp}{dt}\middle|\psi\right\rangle\).
     */
    public final double VdP_M;

    /** The value of \(\mathrm{Spur}\left\|V\frac{dp}{dt}\middle|\rho_{IH}\right\|\).
     */
    public final double VdP_spurM;

    /** The value of
     *	\(\left\langle\psi\middle|\frac{d}{dt}V\frac{dp}{dt}\middle|\psi\right\rangle\).
     */
    public final double DtVdP_M;

    public final double
    	PnL_tradeI_spurM,
	Vol_tradeI_spurM,
	P_posImVT,
	P_negImVT,
	accessible_Volume_ratio;
    

    /** The value of \(\mathrm{Spur}\left\|\frac{d}{dt}V\frac{dp}{dt}\middle|\rho_{IH}\right\|\)=
     *  -{@link #VdP_M}.
     */
    public final double DtVdP_spurM;

    /**
     * The value of \(\mathrm{Spur}\left\|\frac{dI}{dt}\middle|\rho_{IH}\right\|\)=
     *  {@code 0}.
     */
    public final double dIdt_spurM;

    /** The value of \(\mathrm{Spur}\left\|\frac{dpI}{dt}\middle|\rho_{IH}\right\|\)
     * = {@link #I_M}*({@link #P_last}-{@link #pv_M}).
     */
    public final double dPIdt_spurM;

    /** The value of 
	\(
	\left\langle\psi\middle|\frac{dpI}{dt}\middle|\psi\right\rangle
	\).
    */
    public final double dPIdt_M;

    /** \(P^{EQ}\) from volume-driven dynamics.
     */
    public final double PEQV_from_M,PEQV_from_spurM;

    /** An approximation for \(\mathrm{Spur}\left\|I\frac{dp}{dt}\middle|\rho_{IH}\right\|\).
     */
    public final double
	Idpdt_spurM_from_dp_I,
	Idpdt_spurM_from_DP_I,
	Idpdt_spurM_from_sqI_DP_sqI
	;
    public final double DtpDivI_spurM,I2DtpDivI_spurM,IdpmpdI_spurM;


    /** A try for the future price.
     */
    public final double PEQ_I_fromDirectdpI,PEQ_I_fromDPI,PEQ_I_fromI2DtpDivI;
    public final double PEQ_JJ_FromDPI,PEQtotalL_JJ_FromDPI;

    
    public PFuture(
	final IandDM ianddm,
	final double p_offset,
	final double p_last,
	final double [] QQt,
	final double [] QQi,
	final double [] QQpi,
	final double [] QQTi,
	final double [] QQV,
	final double [] QQdp,
	final double [] QQVdp,
	final double totalVolume,
	// basis
	final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	final java.util.function.Function<double [],double []> getPolynomialWIntegratedDt,
	final java.util.function.Function<double [],double []> getEDPsi,

	// for unit tests
	final double eps){

	this.ianddm=ianddm;
	final EVXData I=ianddm.I;
	final int n=I.neq,ind_IH=n-1; // maximal eigenvalue

	P_last=p_offset+p_last;
	// regular moving average
	pv_average=p_offset+QQpi[0]/QQi[0];
	Tv_average= -QQTi[0]/QQi[0];
	this.totalVolume=totalVolume;


	I_M=I.getFPsi(ind_IH,QQi);
	pv_M=p_offset+I.getFPsi(ind_IH,QQpi)/I_M;
	Tv_M= -I.getFPsi(ind_IH,QQTi)/I_M;
	Volume_M= -I.getFPsi(ind_IH,QQV);
	dp_M=I.getFPsi(ind_IH,QQdp);

	// the matrices in b-basis
	final double []
	    bpsiM=EVXData.convertPsiFromQtoB_Basis(n,ianddm.psiM,I.ev,QQt),
	    bQQDensityMatrix=EVXData.convertMatrFromQtoB_Basis(n,ianddm.QQDensityMatrix,I.ev),
	    bQQi=TMatr.getDiagMatrix(I.getEValues()),
	    bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	    bQQdp=EVXData.convertMatrFromQtoB_Basis(n,QQdp,I.ev),
	    bQQTi=EVXData.convertMatrFromQtoB_Basis(n,QQTi,I.ev),
	    bQQVdp=EVXData.convertMatrFromQtoB_Basis(n,QQVdp,I.ev),
	    bQQdiffVdp=MatricesFromPI.getbQQdpdtFromQQp_withLastP(QQVdp,I,getEDPsi,QQt,0);


	// averages "since psiM till now"
	I_spurM= TMatr.getSpurMM(n,bQQi,bQQDensityMatrix); // =Volume_M
	Tv_spurM= TMatr.getSpurMM(n,bQQTi,bQQDensityMatrix)/I_spurM;
	pv_spurM= p_offset+TMatr.getSpurMM(n,bQQpi,bQQDensityMatrix)/I_spurM;
	dp_spurM= TMatr.getSpurMM(n,bQQdp,bQQDensityMatrix);
	final double T_M=TMatr.getSpurM(n,bQQDensityMatrix);
	VdivT_M=I_spurM/T_M;
	one_spurM=T_M;

	// simple P&L projection
	final double [] bQQProjPnL_posImVT=new double [n*n],bQQProjPnL_negImVT=new double [n*n];
	for(int k=0;k<n;k++){
	    final int diagk=k*n+k;
	    final double d=1-VdivT_M/I.ev[k];
	    if(d>0){
		bQQProjPnL_posImVT[diagk]=d;
	    } else if(d<=0){
		bQQProjPnL_negImVT[diagk]=d;
	    }
	}
	final double [] bQQProjPnL=TMatr.sumAcB(bQQProjPnL_posImVT,1,bQQProjPnL_negImVT);
	PnL_tradeI_spurM=TMatr.getSpurMMM(n,bQQpi,bQQProjPnL,bQQDensityMatrix);
	Vol_tradeI_spurM=TMatr.getSpurMMM(n,bQQi,bQQProjPnL_posImVT,bQQDensityMatrix);
	final double negVol_tradeI_spurM=TMatr.getSpurMMM(n,bQQi,bQQProjPnL_negImVT,bQQDensityMatrix);
	P_posImVT=p_offset+TMatr.getSpurMMM(n,bQQpi,bQQProjPnL_posImVT,bQQDensityMatrix)/Vol_tradeI_spurM;
	P_negImVT=p_offset+TMatr.getSpurMMM(n,bQQpi,bQQProjPnL_negImVT,bQQDensityMatrix)/negVol_tradeI_spurM;
	accessible_Volume_ratio=(Vol_tradeI_spurM-negVol_tradeI_spurM)/Volume_M;

	if(eps>0){
	    // simple test
	    UnitTests.checkDoubleRel1(TMatr.getSpurMMM(n,bQQi,bQQProjPnL,bQQDensityMatrix),"Spur(bQQi*bQQProjPnL*bQQDensityMatrix)",0,"0",eps);
	    UnitTests.checkDoubleRel1(Vol_tradeI_spurM+negVol_tradeI_spurM,"Vol_tradeI_spurM+negVol_tradeI_spurM",0,"0",eps);
	    UnitTests.checkDoubleRel1(P_posImVT*Vol_tradeI_spurM+P_negImVT*negVol_tradeI_spurM,"P_posImVT*Vol_tradeI_spurM+P_negImVT*negVol_tradeI_spurM",PnL_tradeI_spurM,"PnL_tradeI_spurM",eps);
	    System.err.println("PFuture:PnL:Volume_M="+Volume_M+" Vol_tradeI_spurM="+Vol_tradeI_spurM+" accessible_Volume_ratio="+accessible_Volume_ratio+" n="+n);
	}

	// Vdp term
	VdP_M=TMatr.getAMatrA(bpsiM,bQQVdp,n);
	VdP_spurM=TMatr.getSpurMM(n,bQQVdp,bQQDensityMatrix);
	DtVdP_M=TMatr.getAMatrA(bpsiM,bQQdiffVdp,n);
	// equals to 0-VdP_M
	DtVdP_spurM=TMatr.getSpurMM(n,bQQdiffVdp,bQQDensityMatrix);

	PEQV_from_M=pv_M-(VdP_M/I_M);
	PEQV_from_spurM=pv_spurM-(VdP_spurM/I_spurM);

	
	// Future value of execution flow. The value is equal to the maximal eigenvalue I.sH
	// The Impact from the future is then: I.sH-I.s0
	// However, as we sometimes use as |psiM> not the |psiH>,
	// the eigenvector of maximal eigenvalue of |I|psi>=lambda|psi>,
	// here we set Ifuture=<psiM|I|psiM>. Again, proper |psiM> is |psiH> and hence Ifuture=I.sH
	final double Ifuture=I_M,I0=I.s0,dIF=I_M-I0;
	final double []
	    bQQdIdtF=MatricesFromPI.getbQQdpdtFromQQp_withLastP(QQi,I,getEDPsi,QQt,Ifuture),
	    bQQdIdt0=MatricesFromPI.getbQQdpdtFromQQp_withLastP(QQi,I,getEDPsi,QQt,0),
	    bQQdPIdtF=MatricesFromPI.getbQQdpdtFromQQp_withLastP(QQpi,I,getEDPsi,QQt,p_last*Ifuture)	    
	    ;

	// Basic version
	dIdt_spurM=TMatr.getSpurMM(n,bQQdIdtF,bQQDensityMatrix); // should be =0
	// (pI)' gives exact result: IFuture*(P^last - pv_M)
	dPIdt_spurM=TMatr.getSpurMM(n,bQQdPIdtF,bQQDensityMatrix);
	dPIdt_M=TMatr.getAMatrA(bpsiM,bQQdPIdtF,n);

	// other versions of Idpdt_spurM
	final double []
	    // Corresponds to DP_{jk} matrix of the article.
	    bQQ_DP=MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(0,QQpi,I,getEDPsi,QQi,p_last),
	    bQQIdpdt_from_sqI_DP_sqI=MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(1,QQpi,I,getEDPsi,QQi,p_last),
	    // due to V factor -- the boundary condition=0 is OK
	    bQQd2pdt2=MatricesFromPI.getbQQdpdtFromQQp_withLastP(QQdp,I,getEDPsi,QQt,0)
	    ;

	// Other approximations for Idpdt_spurM

	// Directly. Possible spurious spikes as both bQQdp & bQQi have spikes.
	// Also, possible problems to combine it with d(pI)/dt as they are sampled differently: from pI & dp.
	Idpdt_spurM_from_dp_I=TMatr.getSpurMMM(n,bQQdp,bQQi,bQQDensityMatrix);
	// Surrogate I dp/dt, with dp/dt calculated from pI.
	// Seens to be the best one to combine with d(pI)/dt.
	Idpdt_spurM_from_DP_I=TMatr.getSpurMMM(n,bQQ_DP,bQQi,bQQDensityMatrix);
	// Surrogate I dp/dt, with I*dp/dt calculated from pI.
	// Not as good as Idpdt_spurM_from_DP_I. Typically lower.
	Idpdt_spurM_from_sqI_DP_sqI=TMatr.getSpurMM(n,bQQIdpdt_from_sqI_DP_sqI,bQQDensityMatrix);


	PEQ_I_fromDirectdpI=p_offset+p_last-(2*Idpdt_spurM_from_dp_I-dPIdt_spurM)/I_M;

	// This is our attempt to obtain Idp/dt-pdI/dt from d/dt p/I
	final double []
	    bQQDtpDivI=MatricesFromPI.getbQQDtpDivIFromQQpi_withLastP(QQpi,I,getEDPsi,p_last,Ifuture),
	    // we assume omega(x0)=1
            bQQ0=TMatr.getQQaa(I.psik_at_x0);
	final double IfutureAdjusted=
	    -TMatr.getSpurMM(n,TMatr.mulMMM(n,bQQi,bQQ0,bQQi),bQQDensityMatrix)/
	    TMatr.getSpurMM(n,TMatr.mulMMM(n,bQQi,MatricesFromPI.getbQQDtpDivIFromQQpi_withLastP(QQi,I,getEDPsi,0,Ifuture),bQQi),bQQDensityMatrix);
	final double []
	    bQQI2DtpDivI= // |I d/dt (p/I) I| with adjusted Ifuture
	    TMatr.mulMMM(n,bQQi,MatricesFromPI.getbQQDtpDivIFromQQpi_withLastP(QQpi,I,getEDPsi,p_last,IfutureAdjusted),bQQi),
	    // this non-Hermitian matrix should produduce almost the same result as bQQI2DtpDivI
	    // 2*bQQ_DP*bQQi-bQQdPIdtF
	    bQQNH_2IdpmdPI_fromDP=TMatr.sumaAbB(2,TMatr.mulMM(n,bQQ_DP,bQQi),-1,bQQdPIdtF),
	    // 4*bQQ_DB*bQQi - (bQQdPIdtF+bQQdiffVdp)
	    bQQNH_TotalL=TMatr.sumAcBdE(TMatr.scaleVectorInPlace(TMatr.mulMM(n,bQQ_DP,bQQi),4),-1,bQQdPIdtF,-1,bQQdiffVdp)
	    ;

	QQDP_toDump=EVXData.convertMatrFromBtoQ_Basis(n,bQQ_DP,I.ev,QQt);
	QQIdp_toDump=EVXData.convertMatrFromBtoQ_Basis(n,TMatr.mulMM(n,bQQ_DP,bQQi),I.ev,QQt);
	QQdIdt_toDump=EVXData.convertMatrFromBtoQ_Basis(n,TMatr.mulMM(n,bQQdIdtF,bQQi),I.ev,QQt);

	DtpDivI_spurM=TMatr.getSpurMM(n,bQQDtpDivI,bQQDensityMatrix);
	I2DtpDivI_spurM=TMatr.getSpurMM(n,bQQI2DtpDivI,bQQDensityMatrix);
	IdpmpdI_spurM=TMatr.getSpurMM(n,bQQNH_2IdpmdPI_fromDP,bQQDensityMatrix);

	if(eps>0){
	    UnitTests.checkDoubleRel1(TMatr.getSpurMM(n,TMatr.mulMatrices(n,new double [][]{ianddm.IQQt,QQIdp_toDump,ianddm.IQQt}),ianddm.QQDensityMatrix),"QQIdp_toDump",Idpdt_spurM_from_DP_I,"Idpdt_spurM_from_DP_I",eps);
	    final var diagElements=TMatr.getMatrixDiagElements(n,TMatr.mulMM(n,bQQdIdtF,bQQDensityMatrix));
	    System.err.println("PFuture:Diag:bQQdIdtF*bQQDensityMathix="+java.util.Arrays.toString(diagElements)+" sum="+TMatr.sumElements(diagElements)+" n="+n);
	}

	PEQ_I_fromI2DtpDivI=p_offset+p_last-(I2DtpDivI_spurM/I_M);
	PEQ_I_fromDPI=p_offset+p_last-(IdpmpdI_spurM/I_M);

	// With bQQDensityMatrix2 (integrate twice).
	final double VToCompensate_ImVT=Ifuture*T_M-I_spurM;
	final double [] bQQDensityMatrix2=EVXData.convertMatrFromQtoB_Basis(n,ianddm.QQDensityMatrix2,I.ev);
	PEQ_JJ_FromDPI=p_offset-(TMatr.getSpurMM(n,bQQNH_2IdpmdPI_fromDP,bQQDensityMatrix2)/VToCompensate_ImVT);
	PEQtotalL_JJ_FromDPI=p_offset-(TMatr.getSpurMM(n,bQQNH_TotalL,bQQDensityMatrix2)/VToCompensate_ImVT);

	//System.err.println("bQQI2DtpDivI="+java.util.Arrays.toString(bQQI2DtpDivI));	

	// A try with split based on dI/dt. Currently only a unit test.
	if(eps>0){
	    final double [] bQQsplitBase=bQQdIdtF; //bQQdIdt0

	    final var splitBydI=new SplitdIdt(n,bQQsplitBase,M.EV,eps);
	    final var split_dIdtF=splitBydI.getSplitted(bQQdIdtF,bQQDensityMatrix);
	    final var split_2IdpmdPI=splitBydI.getSplitted(bQQNH_2IdpmdPI_fromDP,bQQDensityMatrix);
	    final double PEQ_I_posdI=p_offset-split_2IdpmdPI.f_posdI_spurM/split_dIdtF.f_posdI_spurM,
		PEQ_I_negdI=p_offset-split_2IdpmdPI.f_negdI_spurM/split_dIdtF.f_negdI_spurM,
		PEQ_I_pn=p_offset-(split_2IdpmdPI.f_posdI_spurM-split_2IdpmdPI.f_negdI_spurM)/(split_dIdtF.f_posdI_spurM-split_dIdtF.f_negdI_spurM);
	    UnitTests.checkDoubleRel1(IdpmpdI_spurM,"IdpmpdI_spurM",split_2IdpmdPI.f_spurM,"split_2IdpmdPI.f_spurM",eps); 
	    // check Ifuture
	    if(bQQsplitBase==bQQdIdtF){
		// for bQQdIdtF
		UnitTests.checkDoubleRel1(0,"0",split_dIdtF.f_spurM,"split_dIdtF.f_spurM",eps);
	    } else if(bQQsplitBase==bQQdIdt0){
		// for bQQdIdt0
		UnitTests.checkDoubleRel1(0-I_M,"0-I_M",split_dIdtF.f_spurM,"split_dIdtF.f_spurM",eps);
	    } else throw new RuntimeException();

	}
	
	if(eps>0){
	    // Check integration by parts match
	    UnitTests.checkDoubleRel(Volume_M,"Volume_M",I_spurM,"I_spurM",eps);
	    UnitTests.checkDoubleRel1(1,"1",TMatr.getAMatrA(M.B.getX0Moments(n,I.x0),TMatr.mulMMM(n,ianddm.IQQt,ianddm.QQDensityMatrix,ianddm.IQQt),n),"<psi0|QQDensityMatrix|psi0>",eps);

	    
	    final VolMatrix VOLATILITY_EDPsi=getEDPsi::apply;
	    final double Plast_m_pv_M_Ifuture=(P_last-pv_M)*Ifuture;
	    UnitTests.checkDoubleRel1(dPIdt_spurM,"dPIdt_spurM",Plast_m_pv_M_Ifuture,"Plast_m_pv_M_Ifuture",eps);
	    UnitTests.checkDoubleRel1(DtVdP_spurM,"DtVdP_spurM",0-VdP_M,"0-VdP_M",eps);
	    UnitTests.checkDoubleRel1(TMatr.getSpurMM(n,bQQd2pdt2,bQQDensityMatrix),"spur(bQQd2pdt2*bQQDensityMatrix)",0-dp_M,"0-dp_M",eps);

	    // check double integration match
	    UnitTests.checkDoubleRel1(TMatr.getSpurMM(n,bQQdIdt0,bQQDensityMatrix2),"spur(bQQdIdt0*bQQDensityMatrix2)",-I_spurM,"-I_spurM",eps);
	    UnitTests.checkDoubleRel1(TMatr.getSpurMM(n,bQQdIdtF,bQQDensityMatrix2),"spur(bQQdIdtF*bQQDensityMatrix2)",VToCompensate_ImVT,"VToCompensate_ImVT",eps);
	    UnitTests.checkDoubleRel1(TMatr.getSpurMM(n,bQQdPIdtF,bQQDensityMatrix2),"spur(bQQdPIdtF*bQQDensityMatrix2)",Ifuture*T_M*p_last-TMatr.getSpurMM(n,bQQpi,bQQDensityMatrix),"Ifuture*T_M*p_last-spur(bQQpi*bQQDensityMatrix)",eps);

	    UnitTests.checkDoubleRel1(DtVdP_M,"DtVdP_M",bQQdiffVdp[ind_IH*n+ind_IH],"bQQdiffVdp[ind_IH*n+ind_IH]",eps);

	    
	    // Check matrices
	    final double []
		_bQQpPI=MatricesFromPI.getbQQpFrombQQpi(bQQpi,I),
		QQ0=TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),
		_bQQdIdtF=EVXData.convertMatrFromQtoB_Basis(
		    n,
		    TMatr.sumaAbB(Ifuture,QQ0,-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQi)),
		    I.ev),
		_bQQdPIdtF=EVXData.convertMatrFromQtoB_Basis(
		    n,
		    TMatr.sumaAbB(p_last*Ifuture,QQ0,-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQpi)),
		    I.ev),
		_bQQd2pdt2=EVXData.convertMatrFromQtoB_Basis(
		    n,
		    TMatr.sumaAbB(0,QQ0,-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQdp)),
		    I.ev);
	    UnitTests.checkDoubleArrayRel1(bQQdIdtF,"bQQdIdtF",_bQQdIdtF,"_bQQdIdtF",eps);
	    UnitTests.checkDoubleArrayRel1(bQQdPIdtF,"bQQdPIdtF",_bQQdPIdtF,"_bQQdPIdtF",eps);
	    UnitTests.checkDoubleArrayRel1(bQQd2pdt2,"bQQd2pdt2",_bQQd2pdt2,"_bQQd2pdt2",eps);
	    UnitTests.checkDoubleRel1(_bQQdPIdtF[ind_IH*n+ind_IH],"_bQQdPIdtF[ind_IH*n+ind_IH]",dPIdt_M,"dPIdt_M",eps);
	    UnitTests.checkDoubleRel1(pv_M,"pv_M",p_offset+_bQQpPI[ind_IH*n+ind_IH],"p_offset+_bQQpPI[ind_IH*n+ind_IH]",eps);


	    System.err.println("PFuture: test OK n="+n+" eps="+eps);
	}

	if(eps>0){
	    final double [] bQQDt1DivI=MatricesFromPI.getbQQDtpDivIFromQQpi_withLastP(QQi,I,getEDPsi,1,Ifuture),
		bQQDtpDivI_offsetP=MatricesFromPI.getbQQDtpDivIFromQQpi_withLastP(TMatr.sumAcB(QQpi,1.2,QQi),I,getEDPsi,p_last+1.2,Ifuture),
		bQQI2DtpDivI_offsetP=TMatr.mulMMM(n,bQQi,MatricesFromPI.getbQQDtpDivIFromQQpi_withLastP(TMatr.sumAcB(QQpi,1.4,QQi),I,getEDPsi,p_last+1.4,IfutureAdjusted),bQQi),
		_bQQ1DivI=new double [n*n]
		;
	    for(int k=0;k<n;k++) _bQQ1DivI[k*n+k]=1/I.ev[k];
	    final double [] QQ1DivI=EVXData.convertMatrFromBtoQ_Basis(n,_bQQ1DivI,I.ev,QQt),
		_bQQDt1DivI=MatricesFromPI.getbQQdpdtFromQQp_withLastP(QQ1DivI,I,getEDPsi,QQt,1/Ifuture),
		_bQQDtIDivI=MatricesFromPI.getbQQDtpDivIFromQQpi_withLastP(TMatr.mulMMM(n,QQi,ianddm.IQQt,QQi),I,getEDPsi,Ifuture,Ifuture);

	    UnitTests.checkDoubleArrayRel1(bQQDt1DivI,"bQQDt1DivI",_bQQDt1DivI,"_bQQDt1DivI",eps);
	    UnitTests.checkDoubleArrayRel1(_bQQDtIDivI,"_bQQDtIDivI",new double [n*n],"0",eps);

	    // sometimes fails due to dumerical instability
	    if(false){
		UnitTests.checkDoubleRel1(0,"0",TMatr.getSpurMM(n,bQQDt1DivI,bQQDensityMatrix),"spur(bQQDt1DivI*bQQDensityMatrix)",eps);
		UnitTests.checkDoubleRel1(DtpDivI_spurM,"DtpDivI_spurM",TMatr.getSpurMM(n,bQQDtpDivI_offsetP,bQQDensityMatrix),"Spur(bQQDtpDivI_offsetP*bQQDensityMatrix)",eps);
	    }
	    final double [] bQQI2Dt1DivI=
		TMatr.mulMMM(n,bQQi,MatricesFromPI.getbQQDtpDivIFromQQpi_withLastP(QQi,I,getEDPsi,1,IfutureAdjusted),bQQi);
	    System.err.println("PFuture: Ifuture="+Ifuture+" IfutureAdjusted="+IfutureAdjusted+" n="+n+" eps="+eps);
	    UnitTests.checkDoubleRel1(0,"0",TMatr.getSpurMM(n,bQQI2Dt1DivI,bQQDensityMatrix),"spur(bQQI2Dt1DivI*bQQDensityMatrix)",eps);
	    UnitTests.checkDoubleRel1(I2DtpDivI_spurM,"I2DtpDivI_spurM",TMatr.getSpurMM(n,bQQI2DtpDivI_offsetP,bQQDensityMatrix),"Spur(bQQI2DtpDivI_offsetP*bQQDensityMatrix)",eps);

	    final double _2IdpmdPI=2*Idpdt_spurM_from_DP_I-dPIdt_spurM,
		_DtotalL=4*Idpdt_spurM_from_DP_I-(dPIdt_spurM+DtVdP_spurM);
	    		
	    UnitTests.checkDoubleRel1(_DtotalL,"_DtotalL",TMatr.getSpurMM(n,bQQNH_TotalL,bQQDensityMatrix),"spur(bQQNH_TotalL*bQQDensityMatrix)",eps);
	    
	    // Make sure the same thing
	    UnitTests.checkDoubleRel1(_2IdpmdPI,"_2IdpmdPI",IdpmpdI_spurM,"IdpmpdI_spurM",eps);
	    // Amost match ??? May fail!!!
	    UnitTests.checkDoubleRel1(I2DtpDivI_spurM,"I2DtpDivI_spurM",IdpmpdI_spurM,"IdpmpdI_spurM",eps);

	    // Does not hold !!! Requires adjusted IfutureAdjusted!
	    //UnitTests.checkDoubleRel1(0,"0",TMatr.getSpurMM(n,TMatr.mulMMM(n,bQQi,bQQi,bQQDt1DivI),bQQDensityMatrix),"spur(bQQi*bQQi*bQQDt1DivI*bQQDensityMatrix)",eps);
	}
    }

    /** To dump the state in debug.
     */
    public final double [] QQDP_toDump,QQIdp_toDump,QQdIdt_toDump;

public static void match(final PFuture a,final PFuture b,final double eps){
    EVXData.matchEVXData(a.ianddm.I,b.ianddm.I,false,eps);

    UnitTests.checkDoubleRel(a.I_M,"a.I_M",b.I_M,"b.I_M",eps);
    UnitTests.checkDoubleRel(a.Volume_M,"a.Volume_M",b.Volume_M,"b.Volume_M",eps);
    UnitTests.checkDoubleRel(a.VdivT_M,"a.VdivT_M",b.VdivT_M,"b.VdivT_M",eps);
    UnitTests.checkDoubleRel(a.Tv_M,"a.Tv_M",b.Tv_M,"b.Tv_M",eps);
    UnitTests.checkDoubleRel(a.dp_M,"a.dp_M",b.dp_M,"b.dp_M",eps);
    UnitTests.checkDoubleRel(a.pv_M,"a.pv_M",b.pv_M,"b.pv_M",eps);
    UnitTests.checkDoubleRel(a.I_spurM,"a.I_spurM",b.I_spurM,"b.I_spurM",eps);
    UnitTests.checkDoubleRel(a.Tv_spurM,"a.Tv_spurM",b.Tv_spurM,"b.Tv_spurM",eps);
    UnitTests.checkDoubleRel(a.pv_spurM,"a.pv_spurM",b.pv_spurM,"b.pv_spurM",eps);
    UnitTests.checkDoubleRel(a.dp_spurM,"a.dp_spurM",b.dp_spurM,"b.dp_spurM",eps);
    UnitTests.checkDoubleRel(a.I_spurM,"a.I_spurM",b.I_spurM,"b.I_spurM",eps);


    UnitTests.checkDoubleRel(a.PnL_tradeI_spurM,"a.PnL_tradeI_spurM",b.PnL_tradeI_spurM,"b.PnL_tradeI_spurM",eps);
    UnitTests.checkDoubleRel(a.Vol_tradeI_spurM,"a.Vol_tradeI_spurM",b.Vol_tradeI_spurM,"b.Vol_tradeI_spurM",eps);
    UnitTests.checkDoubleRel(a.P_posImVT,"a.P_posImVT",b.P_posImVT,"b.P_posImVT",eps);
    UnitTests.checkDoubleRel(a.P_negImVT,"a.P_negImVT",b.P_negImVT,"b.P_negImVT",eps);
    UnitTests.checkDoubleRel(a.accessible_Volume_ratio,"a.accessible_Volume_ratio",b.accessible_Volume_ratio,"b.accessible_Volume_ratio",eps);

    
    UnitTests.checkDoubleRel(a.VdP_M,"a.VdP_M",b.VdP_M,"b.VdP_M",eps);
    UnitTests.checkDoubleRel(a.VdP_spurM,"a.VdP_spurM",b.VdP_spurM,"b.VdP_spurM",eps);
    UnitTests.checkDoubleRel(a.DtVdP_M,"a.DtVdP_M",b.DtVdP_M,"b.DtVdP_M",eps);
    UnitTests.checkDoubleRel(a.DtVdP_spurM,"a.DtVdP_spurM",b.DtVdP_spurM,"b.DtVdP_spurM",eps);
    UnitTests.checkDoubleRel1(a.dPIdt_M,"a.dPIdt_M",b.dPIdt_M,"b.dPIdt_M",eps);
    UnitTests.checkDoubleRel1(a.dPIdt_spurM,"a.dPIdt_spurM",b.dPIdt_spurM,"b.dPIdt_spurM",eps);
    UnitTests.checkDoubleRel1(a.dIdt_spurM,"a.dIdt_spurM",b.dIdt_spurM,"b.dIdt_spurM",eps);


    UnitTests.checkDoubleRel(a.PEQV_from_M,"a.PEQV_from_M",b.PEQV_from_M,"b.PEQV_from_M",eps);
    UnitTests.checkDoubleRel(a.PEQV_from_spurM,"a.PEQV_from_spurM",b.PEQV_from_spurM,"b.PEQV_from_spurM",eps);

    
    UnitTests.checkDoubleRel(a.PEQ_I_fromDirectdpI,"a.PEQ_I_fromDirectdpI",b.PEQ_I_fromDirectdpI,"b.PEQ_I_fromDirectdpI",eps);
    UnitTests.checkDoubleRel(a.PEQ_I_fromDPI,"a.PEQ_I_fromDPI",b.PEQ_I_fromDPI,"b.PEQ_I_fromDPI",eps);
    UnitTests.checkDoubleRel(a.PEQ_I_fromI2DtpDivI,"a.PEQ_I_fromI2DtpDivI",b.PEQ_I_fromI2DtpDivI,"b.PEQ_I_fromI2DtpDivI",eps);
    UnitTests.checkDoubleRel(a.PEQ_JJ_FromDPI,"a.PEQ_JJ_FromDPI",b.PEQ_JJ_FromDPI,"b.PEQ_JJ_FromDPI",eps);
    UnitTests.checkDoubleRel(a.PEQtotalL_JJ_FromDPI,"a.PEQtotalL_JJ_FromDPI",b.PEQtotalL_JJ_FromDPI,"b.PEQtotalL_JJ_FromDPI",eps);

    UnitTests.checkDoubleRel1(a.Idpdt_spurM_from_dp_I,"a.Idpdt_spurM_from_dp_I",b.Idpdt_spurM_from_dp_I,"b.Idpdt_spurM_from_dp_I",eps);
    UnitTests.checkDoubleRel1(a.Idpdt_spurM_from_DP_I,"a.Idpdt_spurM_from_DP_I",b.Idpdt_spurM_from_DP_I,"b.Idpdt_spurM_from_DP_I",eps);
    UnitTests.checkDoubleRel1(a.Idpdt_spurM_from_sqI_DP_sqI,"a.Idpdt_spurM_from_sqI_DP_sqI",b.Idpdt_spurM_from_sqI_DP_sqI,"b.Idpdt_spurM_from_sqI_DP_sqI",eps);

    
    UnitTests.checkDoubleRel1(a.DtpDivI_spurM,"a.DtpDivI_spurM",b.DtpDivI_spurM,"b.DtpDivI_spurM",eps);
    UnitTests.checkDoubleRel1(a.I2DtpDivI_spurM,"a.I2DtpDivI_spurM",b.I2DtpDivI_spurM,"b.I2DtpDivI_spurM",eps);
	
    System.err.println("PFuture.match test OK n="+a.ianddm.I.neq+" eps="+eps);
}


}
