package com.polytechnik.freemoney;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
/** Laguerre basis.
 *  @see com.polytechnik.trading.QVMDataL
 *  @version $Id: WIntegratorLaguerre.java,v 1.42 2022/09/03 19:02:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class WIntegratorLaguerre extends WIntegrator {

    public WIntegratorLaguerre(final int ntm,final double atau,final int nobservables){
	super(ntm,atau,nobservables);
    }
    public static final com.polytechnik.utils.Laguerre B=new com.polytechnik.utils.Laguerre();

    @Override public void updateSavedMomentsToNewTimeReference(final long currenttime){
	if(currenttime<this.currenttime_previous) throw new IllegalArgumentException();
	if(currenttime_previous<=Long.MIN_VALUE){
	    this.currenttime_previous=currenttime;
	    return; // first call no need to update
	}
	final double [] b=new double [ntm],quot=new double [ntm],
	     next_integration_moments[]=new double [nobservables][ntm];
	final double ref_delay=getXFromTime(currenttime,currenttime_previous),exp_ref_delay=Math.exp(-ref_delay);
	for(int k=0;k<ntm;++k)
	{
	    // Simplified QVMDataL.java version of the recurrence.
	    // 3 term recurrence applied directly to arrays
	    B.setNewtonBinomialLikeCoefs(k,b,quot,1,ref_delay);
	    for(int j=0;j<nobservables;j++){
		next_integration_moments[j][k]=exp_ref_delay*TMatr.mulVVn(k+1,b,integration_moments[j]);
	    }
	}
	System.arraycopy(next_integration_moments,0,integration_moments,0,nobservables);
	this.currenttime_previous=currenttime;
    }


    /** Basis function argument {@code x=(currenttime-t)/atau}.
     */
    @Override public double getXFromTime(final long currenttime,final long t){
	if(t<=Long.MIN_VALUE) return Double.POSITIVE_INFINITY;
	return (currenttime-t)/atau;
    }

    @Override public void addSingleObservationToSomeObservables(final long currenttime,long t,final double [] Fdt,final int staringSerieIndex){
	if(currenttime!=currenttime_previous || currenttime_previous<=Long.MIN_VALUE) throw new IllegalStateException("The internal reference probably was not updated by updateSavedMomentsToNewTimeReference");
	if(t>currenttime || staringSerieIndex<0 || staringSerieIndex+Fdt.length>nobservables) throw new IllegalArgumentException();

	final double etau=getXFromTime(currenttime,t),ex=Math.exp(-etau),
	    Q[]=B.getX0Moments(ntm,etau);
	for(int j=0;j<Fdt.length;j++){
	    TMatr.sumP1(integration_moments[staringSerieIndex+j],Q,ex*Fdt[j]);
	    // simple "sum"
	    sum[staringSerieIndex+j]+=Fdt[j];
	}
    }
    
/** Square, then integrate by parts (with the weight) a wavefunction.
 * A helper function for Laguerre basis.
 */
public static double [] getPsi2WIntegratedDt(final double [] psi)
{
    return getPolynomialWIntegratedDt(B.mult2Pol(psi,psi));
}

/** Integrate by parts (with the weight) a polynomial.
 * A helper function for Laguerre basis.
 */
public static double [] getPolynomialWIntegratedDt(final double [] polynomial)
{
    return B.getLaguerreExpFIntegrated(polynomial);
}

/**
 * Infinitesimal time--shift.
 * Calculates psi/2-psi'.
 * A helper function for Laguerre basis.
 */
public static double [] getEDPsi(final double [] psi)
{
    return TMatr.sumaAbB(0.5,psi,-1,java.util.Arrays.copyOf(B.getDifferentiated(psi),psi.length));
}


/** The moments of the measure \int Q(x)w(x)dt.
 */
public static double [] getDtMoments(final int nmoms){
    final double [] m=new double [nmoms];
    // analytic solution is know
    m[0]=1;
    return m;
}



}
