package com.polytechnik.freemoney;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
/** A simple accumulator with integration in shifted Legendre basis.
 *  @see com.polytechnik.trading.QVMDataP
 *  @version $Id: WIntegratorLegendreShifted.java,v 1.44 2022/09/03 19:02:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class WIntegratorLegendreShifted extends WIntegrator {

    public WIntegratorLegendreShifted(final int ntm,final double atau,final int nobservables){
	super(ntm,atau,nobservables);
    }

    public static final com.polytechnik.utils.LegendreShifted B=new com.polytechnik.utils.LegendreShifted();

    /** Recurrently update saved moments to new currenttime.
     */
    @Override public void updateSavedMomentsToNewTimeReference(final long currenttime){
	if(currenttime<this.currenttime_previous) throw new IllegalArgumentException();
	if(currenttime_previous<=Long.MIN_VALUE){
	    this.currenttime_previous=currenttime;
	    return; // first call no need to update
	}
	/* Temporary arrays.
	 */
	final double [] b=new double [ntm],quot=new double [ntm],
	    next_integration_moments[]=new double [nobservables][ntm];
	final double exp_ref_delay=getXFromTime(currenttime,currenttime_previous);
	for(int k=0;k<ntm;++k)
	{
	    // Simplified QVMDataP.java version of the recurrence.
	    // 3 term recurrence applied directly to arrays
	    B.setNewtonBinomialLikeCoefs(k,b,quot,exp_ref_delay,0);
	    for(int j=0;j<nobservables;j++){
		next_integration_moments[j][k]=exp_ref_delay*TMatr.mulVVn(k+1,b,integration_moments[j]);
	    }
	}
	System.arraycopy(next_integration_moments,0,integration_moments,0,nobservables);
	this.currenttime_previous=currenttime;
    }


    /** Basis function argument {@code x=exp(-(currenttime-t)/atau)}.
     */
    @Override public double getXFromTime(final long currenttime,final long t){
	if(t<=Long.MIN_VALUE) return 0;
	return Math.exp(-(currenttime-t)/atau);
    }

    @Override public void addSingleObservationToSomeObservables(final long currenttime,long t,final double [] Fdt,final int staringSerieIndex){
	if(currenttime!=currenttime_previous || currenttime_previous<=Long.MIN_VALUE) throw new IllegalStateException("The internal reference probably was not updated by updateSavedMomentsToNewTimeReference");
	if(t>currenttime || staringSerieIndex<0 || staringSerieIndex+Fdt.length>nobservables) throw new IllegalArgumentException();

	final double ex=getXFromTime(currenttime,t),Q[]=B.getX0Moments(ntm,ex);
	for(int j=0;j<Fdt.length;j++){
	    TMatr.sumP1(integration_moments[staringSerieIndex+j],Q,ex*Fdt[j]);
	    // simple "sum"
	    sum[staringSerieIndex+j]+=Fdt[j];
	}
    }


/** Square, then integrate by parts (with the weight) a wavefunction.
 * A helper function for shifted Legendre basis.
 */
public static double [] getPsi2WIntegratedDt(final double [] psi)
{
    return getPolynomialWIntegratedDt(B.mult2Pol(psi,psi));
}
/** Integrate by parts (with the weight) a polynomial.
 * A helper function for shifted Legendre basis.
 */
public static double [] getPolynomialWIntegratedDt(final double [] polynomial)
{
    // dt=dx/x
    final double [] pp=B.getIntegrated(polynomial);
    if(polynomial.length!=pp.length-1) throw new RuntimeException();
    return B.sdiv1(pp,0).q;
}

/** Infinitesimal time--shift.
 * Calculates psi/2+x*psi'.
 * A helper function for shifted Legendre basis.
 */
public static double [] getEDPsi(final double [] psi)
{
    double [] d=B.getDifferentiated(psi),xd;
    if(d.length>0) xd=B.getaxbP(d,1,0);
    else xd=new double [d.length+1];
    TMatr.sumP1(xd,psi,0.5);
    return xd;
}

/** The moments of the measure \int Q(x)w(x)dt.
 */
public static double [] getDtMoments(final int nmoms){
    final double [] m=new double [nmoms];
    // analytic solution is know
    m[0]=1;
    return m;
}



}
