package com.polytechnik.freemoney;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
/** Monomials basis.
 *  @see com.polytechnik.trading.QVMData
 *  @version $Id: WIntegratorMonomials.java,v 1.47 2022/09/03 19:02:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class WIntegratorMonomials extends WIntegrator {

    public WIntegratorMonomials(final int ntm,final double atau,final int nobservables){
	super(ntm,atau,nobservables);
    }

    public static final com.polytechnik.utils.Monomials B=new com.polytechnik.utils.Monomials();

    @Override public void updateSavedMomentsToNewTimeReference(final long currenttime){
	if(currenttime<this.currenttime_previous) throw new IllegalArgumentException();
	if(currenttime_previous<=Long.MIN_VALUE){
	    this.currenttime_previous=currenttime;
	    return; // first call no need to update
	}
	final double [] b=new double [ntm],quot=new double [ntm],
	    next_integration_moments[]=new double [nobservables][ntm];
	final double ref_delay=getXFromTime(currenttime,currenttime_previous),exp_ref_delay=Math.exp(ref_delay); // ref_delay is negative
	for(int k=0;k<ntm;++k)
	{
	    // Simplified QVMData.java version of the recurrence.
	    // 3 term recurrence applied directly to arrays
	    B.setNewtonBinomialLikeCoefs(k,b,quot,1,ref_delay);
	    for(int j=0;j<nobservables;j++){
		next_integration_moments[j][k]=exp_ref_delay*TMatr.mulVVn(k+1,b,integration_moments[j]);
	    }
	}
	System.arraycopy(next_integration_moments,0,integration_moments,0,nobservables);
	this.currenttime_previous=currenttime;
    }


    /** Basis function argument {@code x=(t-currenttime)/atau}.
     */
    @Override public double getXFromTime(final long currenttime,final long t){
	if(t<=Long.MIN_VALUE) return Double.NEGATIVE_INFINITY;
	return (t-currenttime)/atau;
    }

    @Override public void addSingleObservationToSomeObservables(final long currenttime,long t,final double [] Fdt,final int staringSerieIndex){
	if(currenttime!=currenttime_previous || currenttime_previous<=Long.MIN_VALUE) throw new IllegalStateException("The internal reference probably was not updated by updateSavedMomentsToNewTimeReference");
	if(t>currenttime || staringSerieIndex<0 || staringSerieIndex+Fdt.length>nobservables) throw new IllegalArgumentException();

	final double etau=getXFromTime(currenttime,t),ex=Math.exp(etau), // etau is negative
	    Q[]=B.getX0Moments(ntm,etau);
	for(int j=0;j<Fdt.length;j++){
	    TMatr.sumP1(integration_moments[staringSerieIndex+j],Q,ex*Fdt[j]);
	    // simple "sum"
	    sum[staringSerieIndex+j]+=Fdt[j];
	}
    }

/** Integrate by parts (with the weight) a polynomial.
 * A helper function for monomials basis.
 */
public static double [] getPolynomialWIntegratedDt(final double [] polynomial)
{
    final double [] pp=new double [polynomial.length];
    inverseOddCoefs(pp.length,pp,polynomial);
    final double [] ppp=com.polytechnik.utils.Polynomials.getPolynomialExpFIntegrated(pp);
    inverseOddCoefs(ppp.length,ppp,ppp);
    return ppp;
}

/** Square, then integrate by parts (with the weight) a wavefunction.
 * A helper function for monomials basis.
 */
public static double [] getPsi2WIntegratedDt(final double [] psi)
{
    return getPolynomialWIntegratedDt(B.mult2Pol(psi,psi));
}

/**
 * Infinitesimal time--shift.
 * Calculates psi/2+psi'.
 * A helper function for monomials basis.
 */
public static double [] getEDPsi(final double [] psi)
{
    return TMatr.sumaAbB(0.5,psi,1,java.util.Arrays.copyOf(B.getDifferentiated(psi),psi.length));
}

/** Inverts the sign of the odd coefs ([1]:x,[3]:x^3,[5]:x^5, etc.) of a polynomial.
 */
public static void inverseOddCoefs(final int n,final double [] res,final double [] l)
{
    boolean flag=false;
    for(int k=0;k<n;k++,flag=!flag)
    {
	if(flag)
	    res[k]=-l[k];
	else
	    res[k]=l[k];
    }
}

/** The moments of the measure \int Q(x)w(x)dt.
 */
public static double [] getDtMoments(final int nmoms){
    final double [] m=new double [nmoms];
    // analytic solution is know
    double f=1;for(int k=0;k<nmoms;++k,f*=(-k))m[k]=f;
    return m;
}


}
