package com.polytechnik.kgo;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVSolver;

/** A class to store SVD-adjusted solution.
 * @version $Id: AdjustedStateToUnitaryWithSVD.java,v 1.21 2024/04/04 17:07:53 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class AdjustedStateToUnitaryWithSVD implements UAdjustment {
	final EVSolver.SVDResult svd_orig;
	final EVSolver.SVDResult svd_adj;
	private final double [] u_orig,u_adj;
	AdjustedStateToUnitaryWithSVD(
	    final EVSolver.SVDResult svd_orig,
	    final double [] u_orig,
	    final EVSolver.SVDResult svd_adj,
	    final double [] u_adj){
	    this.svd_orig=svd_orig;
	    this.u_orig=u_orig;
	    this.svd_adj=svd_adj;
	    this.u_adj=u_adj;
	}
    @Override public double [] getUOrig(){return u_orig;}
    @Override public double [] getUAdj(){return u_adj;}
    @Override public AdjustedStateToUnitaryWithSVD getSVDForTesting(){return this;}

    /* // No longer used.
    double [] getLagrangeMultipliersInSigmaBasis_QuadraticConstraint(final double [] SK){
	final double [] SKinS=convertFunctionalFromOriginalToSigmaBasis(svd_adj,SK);
	final int nC=svd_adj.m;
	final double [] lagrangeMultipliersSigmaBasis=
	    TMatr.mul_MA_x_MB(nC,SKinS,nC,svd_adj.S,1);
	for(int s=0;s<nC;s++){
	    lagrangeMultipliersSigmaBasis[s]/=svd_adj.S[s];
	}
	return lagrangeMultipliersSigmaBasis;
    }

    double [] getLagrangeMultipliersInOriginalBasis_QuadraticConstraint(final double [] SK){
	final double [] lagrangeMultipliersSigmaBasis=getLagrangeMultipliersInSigmaBasis_QuadraticConstraint(SK);
	final int nC=svd_adj.m;
	// convert from Sigma-basis to original
	final double [] lagrangeMultipliers=new double [nC*nC];
	for(int j=0;j<nC;j++){
	    for(int jp=0;jp<nC;jp++){
		double sum=0;
		for(int s=0;s<nC;s++){
		    sum+=svd_adj.U[j*nC+s]*svd_adj.U[jp*nC+s]*lagrangeMultipliersSigmaBasis[s];
		}
		lagrangeMultipliers[j*nC+jp]=sum;
	    }}
	return lagrangeMultipliers;
    }
    */

    /** Reconstruct the matrix using adjusted singular values.
     * For unit test.
     */
    static double [] getMatrixConstructedWithAdjustedSingularValues(
	final EVSolver.SVDResult svd,
	final double [] S_adj){
	if(S_adj.length!=svd.S.length) throw new IllegalArgumentException();

	final int nC=svd.m,nX=svd.n;
	final double [] u_adj=new double [nC*nX];
	for(int j=0;j<nC;j++){
	for(int k=0;k<nX;k++){
	    double sumAdjusted=0;
//	    double sumOrig=0;
	    for(int l=0,lE=Math.min(nC,nX);l<lE;l++){
		sumAdjusted+=svd.U[j*nC+l]*S_adj[l]*svd.VT[l*nX+k];
//		sumOrig+=svd.U[j*nC+l]*svd.S[l]*svd.VT[l*nX+k];
	    }
	    u_adj[j*nX+k]=sumAdjusted;
//	    if(eps>0){
//		// check SVD expansion is OK
//		UnitTests.checkDoubleRel1(sumOrig,"SVD:SumOrig["+j+","+k+"]",u_orig[j*nX+k],"u_orig",eps);
//	    }
	}}
	return u_adj;
    }


    /**
     * Partial sum over "coordinate" indexes, only singular index
     * is not summed.
     * Convert the objective functional matrix from original
     * basis to operator's singular values basis.
     * @return A matrix of the dimension svd.S.length x svd.S.length.
     */
    static double [] convertFunctionalFromOriginalToSigmaBasis(final EVSolver.SVDResult svd,final double [] SK){
	final int nC=svd.m,nX=svd.n,n=nC*nX,nS=svd.S.length;
	if(SK.length!=n*n) throw new IllegalArgumentException();
	final double [] SKinS=new double [nS*nS];
	for(int s=0;s<nS;s++){
	for(int sp=0;sp<nS;sp++){
	    double sum=0;
	    for(int j=0;j<nC;j++){
	    for(int k=0;k<nX;k++){
	    final double tmp=svd.U[j*nC+s]*svd.VT[s*nX+k];
	    for(int jp=0;jp<nC;jp++){
	    for(int kp=0;kp<nX;kp++){
		sum+=tmp
		    *SK[(j*nX+k)*n+(jp*nX+kp)]
		    *svd.U[jp*nC+sp]*svd.VT[sp*nX+kp];
	    }}}}
	    SKinS[s*nS+sp]=sum;
	}}

	return SKinS;
    }


}
