package com.polytechnik.kgo;
/*
 * (C) Vladislav Malyshkin 2024
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.UnitTests;

/** Calculate Gram matrix and possibly apply a function to it.
 * @version $Id: ApplyFunToGramMatrix.java,v 1.8 2024/04/10 23:11:36 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
class ApplyFunToGramMatrix extends ApplyFunToMatrix {
    final int nX;
    final double [] u_orig;
    ApplyFunToGramMatrix(final int nC,final int nX,final double [] u_orig){
	super(nC,TMatr.mul_MA_x_MBTrans(nC,u_orig,nX,u_orig,nC));
	this.nX=nX;
	this.u_orig=u_orig;
	if(nC*nX!=u_orig.length) throw new IllegalArgumentException();

	// unit test
	final double eps=TestKGO.EPS_selftest;
	if(eps>0){

	    // Test |u>=lambda |G|u>
	    final double [] ev_IG=KGOSolutionVectorXVectorF.EV.getGEV_Generalized(nC,TMatr.getUnitMatrix(nC),m,true),
		transform_IG_from_u_orig=java.util.Arrays.copyOfRange(ev_IG,nC,ev_IG.length),
		unew_IG=TMatr.mul_MA_x_MB(nC,transform_IG_from_u_orig,nC,u_orig,nX);
	    TestKGO.testOrthogonality("Adj unew_IG G-basis ",nC,unew_IG,nX,eps);
	    // create the states "averaged" in ev- basis
	    final double [] sumS=new double [nC];
	    for(int j=0;j<nC;j++){
		double sum=0;
		for(int s=0;s<nC;s++) sum+=ev_IG[s*nC+j+nC];
		sumS[j]=sum;
	    }
	    // denominator spur
	    UnitTests.checkDoubleRel1(nC,"nC",TMatr.getAMatrA(sumS,m,nC),"<sumS|G|sumS>",eps);
	    // nominator spur
	    UnitTests.checkDoubleRel1(TMatr.sumElements(java.util.Arrays.copyOf(ev_IG,nC)),"sum Lambdas",TMatr.mulVV(sumS,sumS),"<sumS|Unit|sumS>",eps);

	    // More unit tests.
	    // convert back to original basis
	    final double [] transform_u_adj_From_u_orig=applyFun((ind,lam)->1/Math.sqrt(lam)),
		u_adj=TMatr.mul_MATrans_x_MB(nC,transform_u_adj_From_u_orig,nC,u_orig,nX);

	    System.err.println("ApplyFunToGramMatrix: Gram EV="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_m,nC)));
	    TestKGO.testOrthogonality("Adj EV orig-basis ",nC,u_adj,nX,eps);
	    UnitTests.checkDoubleArrayRel1(u_adj,"u_adj",TMatr.mul_MA_x_MB(nC,transform_u_adj_From_u_orig,nC,u_orig,nX),"transform",eps);
	    // match to SVD
	    final var svd_orig=KGOSolutionVectorXVectorF.EV.getSVD(nC,nX,u_orig);
	    final double [] sigma2=TMatr.squareArrayElements(svd_orig.S);
	    java.util.Arrays.sort(sigma2);
	    UnitTests.checkDoubleArrayRel1(java.util.Arrays.copyOf(ev_m,nC),"ev_m",sigma2,"sigma2",eps);
	    UnitTests.checkDoubleRel1(nC,"nC",TMatr.sumElements(sigma2),"sum sigma2",eps);

	    final double [] Sadj=new double [nC];
	    java.util.Arrays.fill(Sadj,1);
	    final double [] u_adj_svd=AdjustedStateToUnitaryWithSVD.getMatrixConstructedWithAdjustedSingularValues(svd_orig,Sadj);
	    UnitTests.checkDoubleRel1(Math.abs(TMatr.mulVV(u_adj,u_adj_svd)),"<u_adj|u_adj_svd>",nC,"nC",eps);
	    //System.err.println("sum_k u_ik *u_jk="+java.util.Arrays.toString(TMatr.mul_MA_x_MBTrans(nC,u_adj,nX,u_adj,nC))+" nC="+nC+" nX="+nX);
	    //System.err.println("sum_j u_jk *u_jq="+java.util.Arrays.toString(TMatr.mul_MATrans_x_MB(nX,u_adj,nC,u_adj,nX))+" nC="+nC+" nX="+nX);
	    System.err.println("ApplyFunToGramMatrix: SVD match OK nC="+nC+" Gram EV="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_m,nC))+" Sigma^2="+java.util.Arrays.toString(sigma2));

	    //System.err.println("transform_IG_from_u_orig="+java.util.Arrays.toString(transform_IG_from_u_orig));
	    //System.err.println("transform_u_adj_From_u_orig="+java.util.Arrays.toString(transform_u_adj_From_u_orig));
	}
    }

}
