package com.polytechnik.kgo;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import static com.polytechnik.utils.RNConfig.getVal;



/** Command line options for {@link KGO},
 * a class to read config.
 * @version $Id: KGOConfig.java,v 1.10 2024/05/01 22:47:57 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class KGOConfig {
    public final int indcol_x_start,indcol_x_end;
    public final int indcol_f_start,indcol_f_end;
    public final int indcol_weight,indcol_label,columns_number;
    public final int flag_print_verbosity;

    public final int max_multiindex_x,max_multiindex_f;
    public final KGOOptimizationResult.Approximation approximation;
    public final SKFromSampleCalculationMethod SKtype;

    /** In/Out file related info.
     */
    public final String output_files_prefix,data_file_to_build_model_from,data_file_evaluation[];

    protected final java.util.List<String> opts_remained_unprocessed=new java.util.ArrayList<String>();
    public KGOConfig(final String [] args)
    {
	int indcol_x_start=-1,indcol_x_end=-1,
	    indcol_f_start=-1,indcol_f_end=-1,
	    indcol_weight=-1,indcol_label=-1,columns_number=-1,
	    max_multiindex_x=-1,max_multiindex_f=-1;
	boolean flagAdjustTheNumberOfDegreesOfFreedom=true;
	String approximation=KGOOptimizationResult.Approximation.DEFAULT.toString();
	String SKtype=SKFromSampleCalculationMethod.DEFAULT.toString();

	String data_file_to_build_model_from=null,
	    output_files_prefix="";

	int flag_print_verbosity=2;
	final java.util.ArrayList<String> data_file_evaluation=new java.util.ArrayList<>();
	for(String a:args)
	{
	    String v=null;
	    if(null!=(v=getVal(a,"--data_file_to_build_model_from="))) data_file_to_build_model_from=v;
	    else if(null!=(v=getVal(a,"--data_file_evaluation="))) data_file_evaluation.add(v);
	    else if(null!=(v=getVal(a,"--output_files_prefix="))) output_files_prefix=v;
	    else if(null!=(v=getVal(a,"--flag_print_verbosity="))) flag_print_verbosity=Integer.valueOf(v);
	    else if(null!=(v=getVal(a,"--max_multiindex_x="))) max_multiindex_x=Integer.valueOf(v);
	    else if(null!=(v=getVal(a,"--max_multiindex_f="))) max_multiindex_f=Integer.valueOf(v);
	    else if(null!=(v=getVal(a,"--approximation="))) approximation=v;
	    else if(null!=(v=getVal(a,"--SKtype="))) SKtype=v;
	    else if(null!=(v=getVal(a,"--data_cols="))){
		final String [] k=com.polytechnik.utils.ReadWriteUtils.extractnames(v,":");
		if(k.length!=5) throw new RuntimeException("(numcols:xstart,xend:fstart,fend:w:label) colums index="+v+" is invalid. The format is like 15:0,3:4,4:5:0");
		columns_number=Integer.valueOf(k[0]);
		final String [] kx=com.polytechnik.utils.ReadWriteUtils.extractnames(k[1],",");
		final String [] kf=com.polytechnik.utils.ReadWriteUtils.extractnames(k[2],",");
		indcol_weight=Integer.valueOf(k[3]);
		indcol_label=Integer.valueOf(k[4]);
		if(kx.length!=2||kf.length!=2) throw new RuntimeException("(numcols:xstart,xend:fstart,fend:w:label) colums index="+v+" is invalid. The format is like 15:0,3:4,4:5:0");
		indcol_x_start=Integer.valueOf(kx[0]);
		indcol_x_end=Integer.valueOf(kx[1]);
		indcol_f_start=Integer.valueOf(kf[0]);
		indcol_f_end=Integer.valueOf(kf[1]);
	    }
	    else{
		opts_remained_unprocessed.add(a);
	    }
	}
	this.columns_number=columns_number;
	this.max_multiindex_x=max_multiindex_x;
	this.max_multiindex_f=max_multiindex_f;
	this.approximation=KGOOptimizationResult.Approximation.valueOf(approximation);
	this.SKtype=SKFromSampleCalculationMethod.valueOf(SKtype);
	this.indcol_x_start=indcol_x_start;
	this.indcol_x_end=indcol_x_end;
	this.indcol_f_start=indcol_f_start;
	this.indcol_f_end=indcol_f_end;
	this.indcol_weight=indcol_weight;
	this.indcol_label=indcol_label;

	this.data_file_to_build_model_from=data_file_to_build_model_from;
	this.data_file_evaluation=data_file_evaluation.toArray(String[]::new);
	this.output_files_prefix=output_files_prefix;
	this.flag_print_verbosity=flag_print_verbosity;
    }
    public java.util.List<String> validateOptions()
    {
	final java.util.List<String> errs=new java.util.ArrayList<String>();
	if(data_file_to_build_model_from==null) errs.add("no --data_file_to_build_model_from= specified");
	if(columns_number<=0||
	   indcol_x_start<0||indcol_x_end<0||
	   indcol_x_start>indcol_x_end||indcol_x_end>=columns_number||
	   indcol_f_start<0||indcol_f_end<0||
	   indcol_f_start>indcol_f_end||indcol_f_end>=columns_number||
	   indcol_weight>=columns_number||
	   indcol_label >=columns_number ) errs.add("no --data_cols=numcols:xstart,xend:fstart,fend:w:label columns numbers specified for the file --data_file_to_build_model_from=");
	if(opts_remained_unprocessed.size()>0)
	    errs.add("There are unknown options found:"+opts_remained_unprocessed.toString());
	return errs;
    }
    public int [] getXIndexesSequence(){
	final int [] inX=new int [indcol_x_end-indcol_x_start+1];
	for(int k=indcol_x_start,ind=0;k<=indcol_x_end;k++){
	    inX[ind++]=k;
	}
	return inX;
    }
    public int [] getFIndexesSequence(){
	final int [] inF=new int [indcol_f_end-indcol_f_start+1];
	for(int k=indcol_f_start,ind=0;k<=indcol_f_end;k++){
	    inF[ind++]=k;
	}
	return inF;
    }
    public String getDescription()
    {
	return "#BEGIN{KGOConfig}\n"+
	    "data_file_to_build_model_from="+data_file_to_build_model_from+" # input file name\n"+
	    "data_file_evaluation="+java.util.Arrays.toString(data_file_evaluation)+" # data file(s) to evaluate the model\n"+
	    "output_files_prefix="+output_files_prefix+"\n"+
	    "data_cols="+columns_number+":"+indcol_x_start+","+indcol_x_end+":"+indcol_f_start+","+indcol_f_end+":"+indcol_weight+":"+indcol_label+" # (numcols:xstart,xend:fstart,fend:w:label) the file="+data_file_to_build_model_from+" columns index (base 0)\n"+
	    "flag_print_verbosity="+flag_print_verbosity+" # (2 standard output (default), 0 skip X in output)\n"+
	    "max_multiindex_x="+max_multiindex_x+" # if set then a x-polynomial of this multi-index is constructed\n"+
	    "max_multiindex_f="+max_multiindex_f+" # if set then a f-polynomial of this multi-index is constructed\n"+
	    "approximation="+approximation+" # possible values "+java.util.Arrays.toString(KGOOptimizationResult.Approximation.values())+"\n"+
	    "SKtype="+SKtype+" # possible values "+java.util.Arrays.toString(SKFromSampleCalculationMethod.values())+"\n"+
	    "#END{KGOConfig}\n";
    }
} // END OF class KGOConfig
