package com.polytechnik.kgo;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.EVXData;

/**
 * A class to find KGO operator using linear constraints
 * and solving eigenproblem with {@code nC} extra degreed of freedom.
 * Not very good, worse that {@link KGOIterationalLinearConstraints}.
 * @version $Id: KGOIterationalLinearConstraintsExtraDegreesOfFreedom.java,v 1.76 2024/04/26 17:41:21 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class KGOIterationalLinearConstraintsExtraDegreesOfFreedom {
    private static final int N_iterations=80;
    private static final boolean FORCE_DIAG_DUMP=true;

    public final KGOEVSelection evSelected;
    public final UAdjustment aep;

    KGOIterationalLinearConstraintsExtraDegreesOfFreedom(
	final int nC,final int nX,
	final double [] SK,
	final double eps)
{
    final int n=nC*nX,na=n+nC;
    if(SK.length!=n*n) throw new IllegalArgumentException();

    final double [] SKwithLambdaLin=new double [na*na];
    final java.util.function.Function<LambdaLin,double []> adjSKWithLambdaLin=curLambdaLin->{
	// construct SK adjusted with lambdaLin (nC more degrees of freedom)
	for(int i=0,iSK=na-nC;i<na;i++){
	    if(i<iSK){
		System.arraycopy(SK,i*n,SKwithLambdaLin,i*na,n);
		for(int j=0;j<nC;j++){
		    SKwithLambdaLin[i*na+iSK+j]= -curLambdaLin.b[j][i];
		}
	    } else {
		for(int k=0;k<n;k++){
		    SKwithLambdaLin[i*na+k]= -curLambdaLin.b[i-iSK][k];
		}
		for(int j=0;j<nC;j++){
		    SKwithLambdaLin[i*na+iSK+j]=(j==i-iSK)?curLambdaLin.c[j]:0;
		}
	    }
	}
	if(eps>0){
	    UnitTests.checkDoubleArrayRel1(TMatr.selectAntiHermitianPartInPlace(na,SKwithLambdaLin.clone()),"AntiHerm SKwithLambdaLin",new double [na*na],"0",eps);
	}
	return SKwithLambdaLin;
    };

    LambdaLin lambdaLin=new LambdaLin(nC,nX);
    KGOEVSelectionSeparatelyExtraElements result_evsel=null;
    UAdjustment result_aep=null;
    double itmaxSK=0;
    for(int it=0;it<N_iterations;it++){
	// solve regular eigenproblem
	final double [] evSKLambda=KGOSolutionVectorXVectorF.EV.getEV(na,adjSKWithLambdaLin.apply(lambdaLin),true);

	// BEGIN select the eigenvector among several
	double maxSKadj=0;
	KGOEVSelectionSeparatelyExtraElements evsel=null;
	AdjustedStateToUnitaryWithEigenproblem aepsel=null;
	// select eigenvalue
	for(var e:KGOEVSelectionSeparatelyExtraElements.selectTopEVs(na, /* na -- try all ; otherwise try top 3*/evSKLambda,nC,nX)){
	    if(it<=0 || true || evsel==null){
		if(e.originalUNorm<=0) continue;
		final var s=e.getEVAdjustedTo1();
		final double curSKadj=TMatr.getAMatrA(s.getUAdj(),SK,n);
		if(evsel==null || !Double.isFinite(maxSKadj) || curSKadj>=maxSKadj){
		    evsel=e;
		    aepsel=s;
		    maxSKadj=curSKadj;
		}
		if(eps>0){
		    final double curSKorig=TMatr.getAMatrA(s.getUOrig(),SK,n);

		    TestKGO.testOrthogonality("Adj SVD ",nC,s.getUAdj(),nX,eps);
		    System.err.println("it="+it+" e.ind="+e.ind+" e.eLambda="+e.eLambda+" Forig="+curSKorig+" Fadj="+curSKadj+" e.eVectorExtraElements="+java.util.Arrays.toString(e.eVectorExtraElements));

		    // Check lambda--adjusted test.
		    final var ll=LambdaLin.fromAdjustedState(nC,nX,SK,s.getUAdj(),eps);
		    final double []
			u1=getU_addExtraDegreesOfFreedom1(s.getUAdj(),nC),
			testSK=adjSKWithLambdaLin.apply(ll),
			testSK_u1=TMatr.mul_MA_x_MB(na,testSK,na,u1,1);
		    UnitTests.checkDoubleRel1(TMatr.mulVV(u1,u1),"<u1|u1>",2*nC,"2*nC",eps);
		    //System.err.println("testSK="+java.util.Arrays.toString(testSK));
		    UnitTests.checkDoubleRel1(TMatr.getAMatrA(u1,testSK,na),"<u1|testSK|u1>",0,"0",eps);
		    //UnitTests.checkDoubleArrayRel1(testSK_u1,"testSK_u1",new double [na],"0",eps);
		    UnitTests.checkDoubleRel1(TMatr.mulVV(testSK_u1,u1),"<testSK_u1|u1>",0,"0",eps);
		}
		if(false){
		    final double [] b=TMatr.mul_MA_x_MB(n,SK,n,s.getUAdj(),1);
		    TMatr.scaleVectorInPlace(b,1/Math.sqrt(TMatr.mulVV(b,b)));
		    final var eb=new KGOEVSelectionSeparatelyExtraElements(-e.ind,java.util.Arrays.copyOf(b,na),TMatr.getAMatrA(b,SK,n),nC,nX);
		    final var sb=eb.getEVAdjustedTo1();
		    final double bcurSKadj=TMatr.getAMatrA(sb.getUAdj(),SK,n);
		    if(evsel==null|| !Double.isFinite(maxSKadj) || bcurSKadj>=maxSKadj){
			evsel=eb;
			aepsel=sb;
			maxSKadj=bcurSKadj;
		    }
		    if(eps>0){
			final double curSKorig=TMatr.getAMatrA(s.getUOrig(),SK,n);
			System.err.println("it="+it+" e.ind="+e.ind+" e.eLambda="+e.eLambda+" Forig="+curSKorig+" Fadj="+curSKadj+" borig="+TMatr.getAMatrA(sb.getUOrig(),SK,n)+" sbadj="+TMatr.getAMatrA(sb.getUAdj(),SK,n));
		    }
		}
	    }
	}
	// END select the eigenvector among several

	// BEGIN calculate new values of Lagrange multipliers
	final double Fadj=TMatr.getAMatrA(aepsel.getUAdj(),SK,n);
	lambdaLin= LambdaLin.fromAdjustedState(nC,nX,SK,aepsel.getUAdj(),eps);
	// END calculate new values of Lagrange multipliers

	if(FORCE_DIAG_DUMP||eps>0){
	    System.err.println("it="+it+" Selected ind="+evsel.ind+" ("+n+")  Forig="+TMatr.getAMatrA(aepsel.getUOrig(),SK,n)+" Fadj="+Fadj+" evsel.eVectorExtraElements="+java.util.Arrays.toString(evsel.eVectorExtraElements));
	}

	if(result_evsel==null || !Double.isFinite(itmaxSK) || Fadj>=itmaxSK) {
	    result_evsel=evsel;
	    result_aep=aepsel;
	    itmaxSK=Fadj;
	}
    }
    evSelected=result_evsel;
    aep=result_aep;
    if(FORCE_DIAG_DUMP||eps>0||KGOOptimizationResult.FORCE_DIAG_RESULT){
	System.err.println("KGOIterationalLinearConstraintsExtraDegreesOfFreedom: RESULT: Selected ind="+evSelected.ind+" ("+n+")  Forig="+TMatr.getAMatrA(aep.getUOrig(),SK,n)+" Fadj="+TMatr.getAMatrA(aep.getUAdj(),SK,n)+" N_iterations="+N_iterations);
    }
}


/** For unit test.
 */
private static double [] getU_addExtraDegreesOfFreedom1(final double [] u,
							final int nC){
    final double [] u1=java.util.Arrays.copyOf(u,u.length+nC);
    java.util.Arrays.fill(u1,u.length,u1.length,1);
    return u1;
}


private static class KGOEVSelectionSeparatelyExtraElements extends KGOEVSelection {
    final double originalUNorm;
    final double [] eVectorExtraElements,eVector_original;
    KGOEVSelectionSeparatelyExtraElements(final int ind,
				 final double [] eVector_original,
				 final double eLambda,
				 final int nC,
				 final int nX){
	super(ind,new double [nC*nX],eLambda,nC,nX);
	this.eVector_original=eVector_original;
	final int n=nC*nX,na=n+nC;
	if(eVector_original.length!=na) throw new IllegalArgumentException();
	System.arraycopy(eVector_original,0,eVector,0,n);
	this.eVectorExtraElements=java.util.Arrays.copyOfRange(eVector_original,n,na);
	this.originalUNorm=Math.sqrt(TMatr.mulVV(eVector,eVector));
	TMatr.scaleVectorInPlace(eVector,1/originalUNorm);
    }

    static java.util.List<KGOEVSelectionSeparatelyExtraElements> selectTopEVs(
	final int ntop,final double [] ev,final int nC,final int nX){
	final int na=nC*nX+nC;
	if(ev.length!=na+na*na) throw new IllegalArgumentException();
	// for now -- try max ntop eigenvalues
	final var a=new java.util.ArrayList<KGOEVSelectionSeparatelyExtraElements>();
	for(int k=na,ne=(na-ntop);--k>=ne;){
	    if(k>=0){
		a.add(
		    new KGOEVSelectionSeparatelyExtraElements(k,EVXData.getPsiFromBasis(k,ev,na),ev[k],nC,nX)
		      );
	    }
	}
	return a;
    }
} // END OF class KGOEVSelectionSeparatelyExtraElements


    private static class LambdaLin {
	final double [][] b;
	final double [] c;
	LambdaLin(final double [][] b,
		  final double [] c){
	    this.b=b;
	    this.c=c;
	}
	LambdaLin(final int nC,final int nX){
	    this(new double [nC][nX*nC],new double [nC]);
	}
	static LambdaLin fromAdjustedState(final int nC,final int nX,
					   final double [] SK,
					   final double [] u_adj,
					   final double eps){
	    final int n=nC*nX;
	    if(u_adj.length!=n) throw new IllegalArgumentException();
	    if(eps>0){
		TestKGO.testOrthogonality("Test u_adj ",nC,u_adj,nX,eps);
	    }
	    final double [] SKu=TMatr.mul_MATrans_x_MB(n,SK,n,u_adj,1);
	    final double [][] b=new double [nC][n];
	    // Construct the coefficients with extra degrees of freedom
	    // The coefficients with these degres is one
	    final double [] c=new double [nC];
	    if(false){
		// produces good enough result, the same as KGOIterationalLinearConstraints
		final double Fadj=TMatr.mulVV(u_adj,SKu);
		c[0]=Fadj;
		if(true){
		    // better
		    System.arraycopy(SKu,0,b[0],0,n);
		} else if(true){
		    // with lambda
		    final double [] lambda=TMatr.selectHermitianPartInPlace(
			nC,
			TMatr.mul_MA_x_MBTrans(
			    nC,u_adj,nX,TMatr.mul_MATrans_x_MB(n,SK,n,u_adj,1),nC)
									    );
		    final double [] lambda_u_adj=TMatr.mul_MA_x_MB(nC,lambda,nC,u_adj,nX);
		    //System.arraycopy(SKu,0,b[0],0,n);
		    TMatr.sumP1(b[0],lambda_u_adj,1);
		} else {
		    // worse
		    System.arraycopy(u_adj,0,b[0],0,n);
		    TMatr.scaleVectorInPlace(b[0],Fadj/nC);
		}
		if(eps>0){
		    UnitTests.checkDoubleRel1(TMatr.mulVV(u_adj,b[0]),"<u_adj|b[0]>",Fadj,"Fadj",eps);
		}
	    } else {
		// not that good
		for(int j=0;j<nC;j++){
		    double sum=0;
		    for(int k=0;k<nX;k++){
			// other elements remain 0
			sum+=u_adj[j*nX+k]*SKu[j*nX+k];
		    }
		    c[j]=sum;
		}
		//System.err.println("c="+java.util.Arrays.toString(c));
		for(int j=0;j<nC;j++){
		    final double [] bfill=b[j];
		    for(int k=0;k<nX;k++){
			// other elements remain 0
			bfill[j*nX+k]=c[j]*u_adj[j*nX+k];
		    }
		}
	    }
	    return new LambdaLin(b,c);
	}
    } // END OF class LambdaLin

}
