package com.polytechnik.kgo;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.EVXData;

/**
 * A Knowledge Generalized Operator iteration optimization, directly in u_orig basis.
 * Currently almost identical to {@link KGOIterationalSimpleOptimizationU}.
 * @version $Id: KGOIterationalMultipleTransforms.java,v 1.75 2024/04/21 14:53:27 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class KGOIterationalMultipleTransforms {
    private static final int N_iterations=40;

    public final KGOEVSelection evSelected;
    public final UAdjustment aep;
    public KGOIterationalMultipleTransforms(final int nC,final int nX,
					    final double [] SK,
					    final double eps)
{
    final int n=nC*nX;
    if(SK.length!=n*n) throw new IllegalArgumentException();
    //if(true) throw new RuntimeException();


    double [] SKtransformed=SK,
	mu=new double [nC],
	lambdaB=new double [nC*nC],
	basisTransform=TMatr.getUnitMatrix(nC);
    java.util.Arrays.fill(mu,1);
    final double [] muOne=mu.clone();
    KGOEVSelection result_evsel=null;
    UAdjustment result_aep=null;
    double itmaxSK=0;
    int selectedItNum=-1;
    for(int it=0;it<N_iterations;it++){

	// solve regular eigenproblem
	final double [] evSKmu=KGOSolutionVectorXVectorF.EV.getEV(
	    n,
	    MultipleTransformHelpers.scaleSKwithMuAndAdjLambdaB(muOne /*mu*/,lambdaB,nC,nX,SKtransformed),
	    true);

	// BEGIN select the eigenvector among several
	double maxSKadj=0;
	KGOEVSelection evsel=null;
	AdjustedStateToUnitaryWithEigenproblem aepsel=null;
	// select eigenvalue
	for(var e:KGOEVSelection.selectTopEV(n, /* n -- try all ; otherwise try top 3*/evSKmu,nC,nX)){
	    final var s=e.getEVAdjustedTo1();
	    final double curSKadj=TMatr.getAMatrA(s.getUAdj(),SKtransformed,n);
	    if(evsel==null || !Double.isFinite(maxSKadj) || curSKadj>=maxSKadj){
		evsel=e;
		aepsel=s;
		maxSKadj=curSKadj;
	    }
	    if(eps>0){
		TestKGO.testOrthogonality("Adj EV ",nC,s.getUAdj(),nX,eps);
/*		final double
		    qform=AdjustedStateToUnitaryWithEigenproblem.evaluateQuadraticForm(mu,e.eVector,SKtransformed),
		    linfact=AdjustedStateToUnitaryWithEigenproblem.getVectorNorm2WithScaleAndFadj(mu,lambdaB,e.eVector),
		    nom=qform-linfact,
		    denom=AdjustedStateToUnitaryWithEigenproblem.getVectorNorm2WithScale(mu,e.eVector);
		UnitTests.checkDoubleRel1(nom,"nom",e.eLambda,"e.eLambda",eps);
*/
		final double
		    sk_orig=TMatr.getAMatrA(s.getUOrig(),SKtransformed,n),
		    sk_adj=TMatr.getAMatrA(s.getUAdj(),SKtransformed,n);
		UnitTests.checkDoubleRel1(curSKadj,"curSKadj",sk_adj,"sk_adj",eps);

		System.err.println("e.ind="+e.ind+" e.eLambda="+e.eLambda+" curSKadj="+curSKadj+" sk_orig="+sk_orig+" sum Gram diag="+TMatr.getSpurM(nC,s.gram.m)+" sum Gram All="+TMatr.sumElements(s.gram.m));
	    }
	}
	// END select the eigenvector among several


	if(eps>0){
	    // test basis invariance
	    final double []
		evOrig=KGOSolutionVectorXVectorF.EV.getEV(n,SKtransformed,false),
		lG=MultipleTransformHelpers.scaleSKwithMuAndAdjLambdaB(
		    mu,new double [nC*nC],nC,nX,SKtransformed),
		rG=MultipleTransformHelpers.createSKDiagGramDiagK(nC,nX,mu),
		evOrigG=KGOSolutionVectorXVectorF.EV.getGEV_Generalized(n,lG,rG,false);
	    //System.err.println("testevSKEV ="+java.util.Arrays.toString(evOrig));
	    //System.err.println("testevSKGEV="+java.util.Arrays.toString(evOrigG));
	    UnitTests.checkDoubleArrayRel1(evOrig,"evOrig",evOrigG,"evOrigG",eps);

	    System.err.println("it="+it+":Selected ind="+evsel.ind+" of ("+n+") Forig="+TMatr.getAMatrA(aepsel.getUOrig(),SKtransformed,n)+" Fadj="+maxSKadj);
	    if(false){
		// try local, does it help?
		final double [] SKlocal=MultipleTransformHelpers.createLocalSK(nC,nX,SKtransformed,aepsel.getUAdj());
		UnitTests.checkDoubleRel1(TMatr.getAMatrA(TMatr.getUnitMatrix(nC),SKlocal,nC*nC),"init Local",maxSKadj,"maxSKadj",eps);
		final var r=new KGOIterationalSimpleOptimizationU(nC,nC,SKlocal,eps);
		System.err.println("it="+it+" Selected ind="+evsel.ind+" Init Fadj="+maxSKadj+" Local res="+TMatr.getAMatrA(r.aep.getUAdj(),SKlocal,nC*nC));
	    }
	}

	final double [] prev_SKtransformed=SKtransformed; // for unit test
	// Calculate Lagrange multiplers and scaling factors mu.
	// Calculations in new basis
	final double [] thisTransform=java.util.Arrays.copyOfRange(aepsel.gram.ev_m,nC,aepsel.gram.ev_m.length);

	// to new basis:
	basisTransform=TMatr.mul_MA_x_MB(nC,thisTransform,nC,basisTransform,nC);
	// The SK in new basis
	SKtransformed=AdjustedStateToUnitaryWithEigenproblem.convertSKtoNewBasis(nC,nX,SK,basisTransform);

	// New mu
	mu=new double [nC];
	for(int j=0;j<nC;j++){
	    mu[j]=1/Math.sqrt(aepsel.gram.ev_m[j]);
	}
	// The original and adjusted states in new basis
	double [] u_orig=TMatr.mul_MA_x_MB(nC,thisTransform,nC,aepsel.getUOrig(),nX),
	    u_adj=MultipleTransformHelpers.scaleVector(mu,u_orig);
	final double []
	    // Lagrange multipliers in new basis.
	    HermLambdaB=LagrangeMultipliersPartialSubspace.calculateRegularLambda(nC,nX,SKtransformed,u_adj);
	// lambdaB in new basis
	lambdaB=HermLambdaB;

	if(eps>0){
	    TestKGO.testOrthogonality("u_adj ",nC,u_adj,nX,eps);
	    final double []
		ev_HermLambdaB=KGOSolutionVectorXVectorF.EV.getEV(nC,HermLambdaB,true),
		basisForLambdaB=java.util.Arrays.copyOfRange(ev_HermLambdaB,nC,ev_HermLambdaB.length),
		_HermLambdaB=TMatr.mulMMM(nC,TMatr.getMatrTrans(nC,basisForLambdaB),TMatr.getDiagMatrix(java.util.Arrays.copyOf(ev_HermLambdaB,nC)),basisForLambdaB),
		diagLambdaB=TMatr.mulMMM(nC,basisForLambdaB,HermLambdaB,TMatr.getMatrTrans(nC,basisForLambdaB));

	    final double n2=MultipleTransformHelpers.getVectorNorm2WithScale(mu,u_orig);
	    UnitTests.checkDoubleRel1(n2,"n2",nC,"nC",eps);
	    UnitTests.checkDoubleArrayRel1(HermLambdaB,"HermLambdaB",_HermLambdaB,"_HermLambdaB",eps);
	    UnitTests.checkDoubleArrayRel1(TMatr.getMatrixDiagElements(nC,diagLambdaB),"diag diagLambdaB",java.util.Arrays.copyOf(ev_HermLambdaB,nC),"EV HermLambdaB",eps);
	    System.err.println("EV:_lambdaB="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_HermLambdaB,nC)));
	}
	if(false){
	    // same thing, the problem is invariant with mu=1
	    // one more transform
	    final double [] ev_HermLambdaB=KGOSolutionVectorXVectorF.EV.getEV(nC,HermLambdaB,true),
		thisTransformToHermLambdaB=java.util.Arrays.copyOfRange(ev_HermLambdaB,nC,ev_HermLambdaB.length);
	    lambdaB=TMatr.getDiagMatrix(java.util.Arrays.copyOf(ev_HermLambdaB,nC));
	    // One more unitary transform (for lambdaB stability).
	    // Transform once again: to the basis of diagonal lambdaBAll Hermitian part
	    basisTransform=TMatr.mul_MA_x_MB(nC,thisTransformToHermLambdaB,nC,basisTransform,nC);
	    // The SK in new basis
	    SKtransformed=AdjustedStateToUnitaryWithEigenproblem.convertSKtoNewBasis(nC,nX,SK,basisTransform);

	    // u_orig & u_adj in new basis
	    // The original and adjusted states in new basis
	    u_orig=TMatr.mul_MA_x_MB(nC,thisTransformToHermLambdaB,nC,u_orig,nX);
	    u_adj=TMatr.mul_MA_x_MB(nC,thisTransformToHermLambdaB,nC,u_adj,nX);
	} else {
	    if(eps>0){
		// this is correct only when no extra transforms
		final double []  SKalam=
		    MultipleTransformHelpers.scaleSKwithMuAndAdjLambdaB(mu,lambdaB,nC,nX,SKtransformed);
		final double FSKlam_orig=TMatr.getAMatrA(u_orig,SKalam,n);
		UnitTests.checkDoubleRel1(FSKlam_orig,"FSKlam_orig",0,"0",eps);
	    }
	}
	if(eps>0){

	    final double []
		u_orig_prevBasis=aepsel.getUOrig(),
		u_adj_prevBasis=aepsel.getUAdj(),
		u_adj_InitBasis=TMatr.mul_MATrans_x_MB(nC,basisTransform,nC,u_adj,nX),
		u_orig_InitBasis=TMatr.mul_MATrans_x_MB(nC,basisTransform,nC,u_orig,nX)
		;
	    final double
		Forig_prevBasis=TMatr.getAMatrA(u_orig_prevBasis,prev_SKtransformed,n),
		Forig=TMatr.getAMatrA(u_orig,SKtransformed,n),
		Forig_InitBasis=TMatr.getAMatrA(u_orig_InitBasis,SK,n),

		Fadj_prevBasis=TMatr.getAMatrA(u_adj_prevBasis,prev_SKtransformed,n),
		Fadj=TMatr.getAMatrA(u_adj,SKtransformed,n),
		Fadj_InitBasis=TMatr.getAMatrA(u_adj_InitBasis,SK,n);

	    TestKGO.testOrthogonality("u_adj ",nC,u_adj,nX,eps);
	    TestKGO.testOrthogonality("u_adj_prevBasis ",nC,u_adj_prevBasis,nX,eps);
	    TestKGO.testOrthogonality("u_adj_InitBasis ",nC,u_adj_InitBasis,nX,eps);
	    UnitTests.checkDoubleRel1(Forig,"Forig",Forig_InitBasis,"Forig_InitBasis",eps);
	    UnitTests.checkDoubleRel1(Forig,"Forig",Forig_prevBasis,"Forig_prevBasis",eps);
	    UnitTests.checkDoubleRel1(Fadj,"Fadj",Fadj_InitBasis,"Fadj_InitBasis",eps);
	    UnitTests.checkDoubleRel1(Fadj,"Fadj",Fadj_prevBasis,"Fadj_prevBasis",eps);
	    UnitTests.checkDoubleRel1(Fadj,"Fadj",TMatr.getSpurM(nC,lambdaB),"spur lambdaB",eps);

	    UnitTests.checkDoubleRel1(TMatr.mulVV(aepsel.getUOrig(),aepsel.getUOrig()),"norm2 orig",nC,"nC",eps);


	    System.err.println("KGOIterationalMultipleTransforms: it="+it+" Forig="+Forig+" Fadj="+Fadj+" Gram EV="+java.util.Arrays.toString(java.util.Arrays.copyOf(aepsel.gram.ev_m,nC))+" Sum Gram EV="+TMatr.sumElements(java.util.Arrays.copyOf(aepsel.gram.ev_m,nC))+" ev.eLambda*nC="+(evsel.eLambda*nC));
	}

	final double Fadj=TMatr.getAMatrA(u_adj,SKtransformed,n);
	if(result_evsel==null || !Double.isFinite(itmaxSK) || Fadj>=itmaxSK) {
	    final double []
		u_adj_InitBasis=TMatr.mul_MATrans_x_MB(nC,basisTransform,nC,u_adj,nX),
		u_orig_InitBasis=TMatr.mul_MATrans_x_MB(nC,basisTransform,nC,u_orig,nX);
	    result_evsel=new KGOEVSelection(evsel.ind,
					    TMatr.getVectorScaled(u_orig_InitBasis,1/Math.sqrt(nC)),
					    evsel.eLambda,nC,nX);
	    result_aep=new UAdjustment(){
		    @Override public double [] getUOrig(){return u_orig_InitBasis;}
		    @Override public double [] getUAdj(){return u_adj_InitBasis;}
		    @Override public AdjustedStateToUnitaryWithSVD getSVDForTesting(){
			return new AdjustedStateToUnitaryWithSVD(
			    KGOSolutionVectorXVectorF.EV.getSVD(nC,nX,getUOrig()),
			    getUOrig(),
			    KGOSolutionVectorXVectorF.EV.getSVD(nC,nX,getUAdj()),
			    getUAdj());
		    }
		};
	    itmaxSK=Fadj;
	    selectedItNum=it;
	}
    }
    evSelected=result_evsel;
    aep=result_aep;
    if(eps>0){
	final var s=evSelected.getEVAdjustedTo1();
	UnitTests.checkDoubleArrayRel1(EVXData.getMSign(aep.getUOrig()),"aep.getUOrig()",EVXData.getMSign(s.getUOrig()),"s.getUOrig()",eps);
	UnitTests.checkDoubleArrayRel1(EVXData.getMSign(aep.getUAdj()),"aep.getUAdj()",EVXData.getMSign(s.getUAdj()),"s.getUAdj()",eps);
	System.err.println("KGOIterationalMultipleTransforms: RESULT:Selected ind="+evSelected.ind+" ("+n+")  Forig="+TMatr.getAMatrA(aep.getUOrig(),SK,n)+" Fadj="+TMatr.getAMatrA(aep.getUAdj(),SK,n)+" at iteration="+selectedItNum+" N_iterations="+N_iterations);
    }
}



}
