package com.polytechnik.kgo;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;

/** A helper class for unit tests and obsolete functions used in
 * {@link KGOIterationalMultipleTransforms}.
 * @version $Id: MultipleTransformHelpers.java,v 1.1 2024/04/11 20:44:10 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
class MultipleTransformHelpers {

    /** Given S_jkjpkp calculate mu_j S_jkjpkp mu_jp - delta_{kkp} lambdaB[j,jp] mu_j * mu_jp.
     * To remove along with {@link KGOIterationalMultipleTransforms}.
     */
    static double [] scaleSKwithMuAndAdjLambdaB(final double [] mu_scale,final double [] lambdaB,final int nC,final int nX,final double [] SK){
	final int n=nC*nX;
	if(mu_scale.length!=nC||
	   lambdaB.length!=nC*nC||
	   SK.length!=n*n) throw new IllegalArgumentException();
	final double [] SKmu=new double [n*n];
	for(int j=0;j<nC;j++){
	for(int jp=0;jp<nC;jp++){
	    final double muj_mujp=mu_scale[j]*mu_scale[jp],lam_muj_mujp=lambdaB[j*nC+jp]*muj_mujp;
	    for(int k=0;k<nX;k++){
	    SKmu[(j*nX+k)*n+(jp*nX+k)] -= lam_muj_mujp;
	    for(int kp=0;kp<nX;kp++){
		SKmu[(j*nX+k)*n+(jp*nX+kp)]+=muj_mujp*SK[(j*nX+k)*n+(jp*nX+kp)];
	    }}
	}}
	return SKmu;
    }


    /**
     * To remove along with {@link KGOIterationalMultipleTransforms}.
     */
    static double [] createLocalSK(final int nC,final int nX,final double [] SK, final double [] u_orig){
	final int n=nC*nX,nXL=nC,nL=nC*nXL;
	if(SK.length!=n*n || u_orig.length!=n) throw new IllegalArgumentException();
	final double [] SKnew=new double [nL*nL];
	for(int s=0;s<nC;s++){
	for(int sp=0;sp<nC;sp++){
	for(int i=0;i<nC;i++){
	for(int ip=0;ip<nC;ip++){
	    double sum=0;
	    for(int k=0;k<nX;k++){
	    for(int kp=0;kp<nX;kp++){
		sum+=u_orig[i*nX+k]*SK[(s*nX+k)*n+(sp*nX+kp)]*u_orig[ip*nX+kp];
	    }}
	    SKnew[(s*nXL+i)*nL+(sp*nXL+ip)]=sum;
	}}}}
	return SKnew;
    }


    /** To remove along with {@link KGOIterationalMultipleTransforms}.
     * For unit test.
     */
    static double [] createSKDiagGramDiagK(final int nC,final int nX,final double [] mu_scale){
	return createSKBasisDiagK(nC,nX,mu_scale,TMatr.getUnitMatrix(nC));
    }
    /** To remove along with {@link KGOIterationalMultipleTransforms}.
     * For unit test.
     */
    private static double [] createSKBasisDiagK(final int nC,final int nX,final double [] mu_scale,final double [] basis_of_AdjF){
	final int n=nC*nX;
	if(mu_scale.length!=nC||basis_of_AdjF.length!=nC*nC) throw new IllegalArgumentException();
	final double [] SKmu=new double [n*n];
	for(int j=0;j<nC;j++){
	for(int jp=0;jp<nC;jp++){
	    final double muj_mujp=mu_scale[j]*mu_scale[jp];
	    for(int s=0;s<nC;s++){
	    final double fact=muj_mujp*basis_of_AdjF[s*nC+j]*basis_of_AdjF[s*nC+jp];
	    for(int k=0;k<nX;k++){
		SKmu[(j*nX+k)*n+(jp*nX+k)]+=fact;
	    }}
	}}
	return SKmu;
    }


    /** Evaluate quadratic form in vec state with some scale in first index of {@code vec}.
     * To remove along with {@link KGOIterationalMultipleTransforms}.
     */
    static double evaluateQuadraticForm(final double [] mu_scale,final double [] vec,final double [] SK){
	return evaluateQuadraticForm(mu_scale,vec,mu_scale,vec,SK);
    }

    /**
     * To remove along with {@link KGOIterationalMultipleTransforms}.
     */
    static double evaluateQuadraticForm(final double [] mu1_scale,final double [] vec1,final double [] mu2_scale,final double [] vec2,final double [] SK){
	final int nC=mu1_scale.length,nX=vec1.length/nC,n=nC*nX;
	if(vec1.length!=n || vec2.length!=n || mu2_scale.length!=nC || SK.length!=n*n) throw new IllegalArgumentException();

	double sum=0;
	for(int j=0;j<nC;j++){
	for(int k=0;k<nX;k++){
	    final int j_nX_k=j*nX+k,nj_nX_k=j_nX_k*n;
	    final double ujk=vec1[j_nX_k]*mu1_scale[j];
	    for(int jp=0;jp<nC;jp++){
	    for(int kp=0;kp<nX;kp++){
		sum+=ujk*SK[nj_nX_k+(jp*nX+kp)]*vec2[jp*nX+kp]*mu2_scale[jp];
	    }}
	}}
	return sum;
    }

    /** Scale the vector in first index.
     */
    static double [] scaleVector(final double [] mu_scale,final double [] vec){
        final int nC=mu_scale.length,nX=vec.length/nC;
        if(vec.length!=nC*nX) throw new IllegalArgumentException();

        final double [] v=new double [nC*nX];
        for(int j=0;j<nC;j++){
        final double fact=mu_scale[j];
        for(int k=0,j_nX=j*nX;k<nX;k++){
            v[j_nX+k]=vec[j_nX+k]*fact;
        }}
        return v;
    }


    /**
     * To remove along with {@link KGOIterationalMultipleTransforms}.
     */
    static double getVectorNorm2WithScale(final double [] mu_scale,final double [] vec){
	return getVectorScalProductWithScale(mu_scale,vec,mu_scale,vec);
    }

    private static double getVectorScalProductWithScale(final double [] mu1_scale,final double [] vec1,final double [] mu2_scale,final double [] vec2){
	final int nC=mu1_scale.length,nX=vec1.length/nC,n=nX*nC;
	if(mu2_scale.length!=nC || vec1.length!=n || vec2.length!=n) throw new IllegalArgumentException();

	double sum=0;
	for(int j=0;j<nC;j++){
	final double fact=mu1_scale[j]*mu2_scale[j];
	for(int k=0,j_nX=j*nX;k<nX;k++){
	    sum+=fact*vec1[j_nX+k]*vec2[j_nX+k];
	}}
	return sum;
    }

}
