package com.polytechnik.kgo;
/*
 * (C) Vladislav Malyshkin 2024
 * This file is under GPL version 3.
 *
 */
/** The list of implemented options to calculate \( S_{jk;j^{\prime}k^{\prime}} \)
 * from original data sample \( (\mathbf{x}^{(l)},\mathbf{f}^{(l)}) \),
 * \( l=1\dots M\).
 * @version $Id: SKFromSampleCalculationMethod.java,v 1.4 2024/05/01 23:01:00 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public enum SKFromSampleCalculationMethod {

    /** The \( \left\langle  F_j X_k F_{j^{\prime}} X_{k^{\prime}}  K(\mathbf{F})K(\mathbf{X}) \right\rangle  \)
     * is obtained from original sample \( (\mathbf{x},\mathbf{f}) \) after regularization.
     */
    CHRISTOFFEL,
    /** The \( \left\langle  F_j X_k F_{j^{\prime}} X_{k^{\prime}}  K(\mathbf{F})\widetilde{K}(\mathbf{X}) \right\rangle  \)
     * where \( \widetilde{K}(\mathbf{X}) \) is reduced size dimension adjusted Christoffel function.
     */
    CHRISTOFFEL_DEGREESOFFREEDOMADJUSTED,
    /** The \( \left\langle F_j X_k F_{j^{\prime}} X_{k^{\prime}} K(\mathbf{F}) \right\rangle  \)
     * is obtained from original sample \( (\mathbf{x},\mathbf{f}) \) after regularization.
     */
    FXFX_F_CHRISTOFFEL,

    /** Just plain \( \left\langle F_j X_k F_{j^{\prime}} X_{k^{\prime}} \right\rangle  \)
     * in regularized \( (\mathbf{X},\mathbf{F}) \),
     * that are obtained from \( (\mathbf{x},\mathbf{f}) \) original sample.
     */
    FXFX
    ;

    public static final SKFromSampleCalculationMethod DEFAULT=FXFX_F_CHRISTOFFEL;

    private static final java.util.List<SKFromSampleCalculationMethod> BROKENLIST=
    java.util.List.of(
	//FXFX
			  );
    public static java.util.List<SKFromSampleCalculationMethod> valuesSkipBroken(){
	return java.util.Arrays.stream(values()).filter(e -> !BROKENLIST.contains(e)).toList();
    }

}
