package com.polytechnik.kgo;
/*
 * (C) Vladislav Malyshkin 2021
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.ObservationVectorXVectorF;
import com.polytechnik.utils.DataReadObservationVectorXVectorF;
import com.polytechnik.utils.VectorXVectorFPointAccess;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.XXMatrs;

/** A class for unit test.
  * @version $Id: TestDirectProjection.java,v 1.70 2024/04/02 01:29:10 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
 */
public class TestDirectProjection {
    final DataReadObservationVectorXVectorF ro;
    final DirectProjectionSolutionVectorXVectorF directProjectionTestSolutionVectorXVectorF;

    TestDirectProjection(final DataReadObservationVectorXVectorF ro){
	this.ro=ro;
	this.directProjectionTestSolutionVectorXVectorF=new DirectProjectionSolutionVectorXVectorF(ro.xHeader.size(),ro.fHeader.size(),getFF(),getXX(),getXF());
    }


    double [] getXX(){return getXX(o->1);}
    double [] getXX(final java.util.function.ToDoubleFunction<ObservationVectorXVectorF> funo){
	final int n=ro.xHeader.size();
	final double [] m=new double [n*n];
	for(ObservationVectorXVectorF o:ro.data){
	    final double factor=funo.applyAsDouble(o)*o.weight;
	    for(int k=0;k<n;k++){
		for(int j=0;j<n;j++){
		    m[k*n+j]+=o.x[k]*o.x[j]*factor;
		}
	    }
	}
	return m;
    }
    double [] getFF(){return getFF(o->1);}
    double [] getFF(final java.util.function.ToDoubleFunction<ObservationVectorXVectorF> funo){
	final int n=ro.fHeader.size();
	final double [] m=new double [n*n];
	for(ObservationVectorXVectorF o:ro.data){
	    //System.err.println("o.f="+java.util.Arrays.toString(o.f));
	    final double factor=funo.applyAsDouble(o)*o.weight;
	    for(int k=0;k<n;k++){
		for(int j=0;j<n;j++){
		    m[k*n+j]+=o.f[k]*o.f[j]*factor;
		}
	    }
	}
	return m;
    }
    double [] getXF(){return getXF(o->1);}
    double [] getXF(final java.util.function.ToDoubleFunction<ObservationVectorXVectorF> funo){
	final int nx=ro.xHeader.size(),nf=ro.fHeader.size();
	final double [] m=new double [nx*nf];
	for(ObservationVectorXVectorF o:ro.data){
	    final double factor=funo.applyAsDouble(o)*o.weight;
	    for(int k=0;k<nx;k++){
		for(int j=0;j<nf;j++){
		    m[k*nf+j]+=o.x[k]*o.f[j]*factor;
		}
	    }
	}
	return m;
    }


    double getTotalWeight(){
	return ro.data.stream().mapToDouble(o -> o.weight).sum();
    }

    double getAveraged(final java.util.function.ToDoubleFunction<ObservationVectorXVectorF> funo){
	double sumfw=0,sumw=0;
	for(ObservationVectorXVectorF o:ro.data){
	    sumfw+=o.weight*funo.applyAsDouble(o);
	    sumw+=o.weight;
	}
	return sumfw/sumw;
    }



    XXMatrs getXXandXKftoxX(){
	final int nx=ro.xHeader.size(),nf=ro.fHeader.size();
	final double [] xf=directProjectionTestSolutionVectorXVectorF.XF,
	    Gfm1=directProjectionTestSolutionVectorXVectorF.Gfm1,
	    Gfm1fKfGfm1=TMatr.mulMMM(nf,Gfm1,getFF(o->1/TMatr.getAMatrA(o.f,Gfm1,nf)),Gfm1),
	    tmp=TMatr.mul_MA_x_MB(nx,xf,nf,Gfm1fKfGfm1,nf),
	    Kftox=TMatr.mul_MA_x_MBTrans(nx,tmp,nf,xf,nx);
	return new XXMatrs(nx,nx,Kftox,directProjectionTestSolutionVectorXVectorF.XX);
    }

    XXMatrs getXXandXKfX(){
	final int nx=ro.xHeader.size(),nf=ro.fHeader.size();
	final double [] Gfm1=directProjectionTestSolutionVectorXVectorF.Gfm1;
	return new XXMatrs(nx,nx,getXX(o->1/TMatr.getAMatrA(o.f,Gfm1,nf)),directProjectionTestSolutionVectorXVectorF.XX);
    }
    XXMatrs getXXandXKX(){
	final int n=ro.xHeader.size();
	final double [] Gxm1=directProjectionTestSolutionVectorXVectorF.Gxm1;
	return new XXMatrs(n,n,getXX(o->1/TMatr.getAMatrA(o.x,Gxm1,n)),directProjectionTestSolutionVectorXVectorF.XX);
    }
    XXMatrs getFFandFKF(){
	final int n=ro.fHeader.size();
	final double [] Gfm1=directProjectionTestSolutionVectorXVectorF.Gfm1;
	return new XXMatrs(n,n,getFF(o->1/TMatr.getAMatrA(o.f,Gfm1,n)),directProjectionTestSolutionVectorXVectorF.FF);
    }

    /** Local weight; if the Error=0 then it is=1, corresponds to Eq. (88) for \varpi(g).
     */
    java.util.function.ToDoubleFunction<double []> getFtoXProjectedWeight(){
	final XXMatrs m=directProjectionTestSolutionVectorXVectorF.rankMatrices;
	final double [] Gfm1=directProjectionTestSolutionVectorXVectorF.Gfm1,
	    Mnominator=TMatr.mulMMM(m.dim1,Gfm1,m.XfX,Gfm1);
	return of -> TMatr.getAMatrA(of,Mnominator,m.dim1)/TMatr.getAMatrA(of,Gfm1,m.dim1);
    }

    /**
     *	Total covered weight, corresponds to the sum in Error expression, Eq. (89):
     *	\sum \varpi(x^{(l)}) * \omega^{(l)}.
     */
    double getProjectedTotalCoverageWithoutSpectrum(){
	final java.util.function.ToDoubleFunction<double []> projectedWeight=getFtoXProjectedWeight();
	return ro.data.stream().mapToDouble(o -> o.weight*projectedWeight.applyAsDouble(o.f)).sum();
    }

    /** The same as getProjectedTotalCoverageWithoutSpectrum.
     */
    double getProjectedTotalCoverageWithoutSpectrum2(){
	final XXMatrs m=getXXandXKftoxX();
	final double [] Gxm1=directProjectionTestSolutionVectorXVectorF.Gxm1;
	return TMatr.getSpurMM(m.dim1,m.XfX,Gxm1);
    }

    /**
     *	\sum \varpi(x^{(l)}) * \omega^{(l)},
     * the \varpi is calculated in an orthogonal basis using Eq. (91)
     */
    double getProjectedTotalCoverageWithoutSpectrum3(double [] evXbasis){
	final int nx=ro.xHeader.size(),nf=ro.fHeader.size();
	if(evXbasis.length!=nx+nx*nx) throw new IllegalArgumentException();
	final double [] Gfm1=directProjectionTestSolutionVectorXVectorF.Gfm1,
	    XfGfm1=directProjectionTestSolutionVectorXVectorF.XfGfm1,
	    XX=directProjectionTestSolutionVectorXVectorF.XX;
	final java.util.function.ToDoubleFunction<double []> projectedWeight= f -> {
	    final double [] xproj=TMatr.mul_MA_x_MB(nx,XfGfm1,nf,f,1);
	    double s=0,fl=TMatr.getAMatrA(f,Gfm1,nf);
	    for(int j=0;j<nx;j++){
		s+=Math.pow(TMatr.mulVV(EVXData.getPsiFromBasis(j,evXbasis,nx),xproj),2);
	    }
	    return s/fl;
	};
	return ro.data.stream().mapToDouble(o -> o.weight*projectedWeight.applyAsDouble(o.f)).sum();
    }


    /** Total covered weight, corresponds to the sum in Error expression, Eq. (100):
     *	\sum Prob(f^{(l)}|x^{(l)}) * \omega^{(l)}.
     */
    double getWeightFromJointProbabilityLSQpredictor(){
	return ro.data.stream().mapToDouble( o -> o.weight*directProjectionTestSolutionVectorXVectorF.evaluateAt(o.x).getFProbabilityAtX(o.f)).sum();
    }



    static void testApproachVectorXVectorF_Simple(final TestDirectProjection ts,final boolean flag_exact,final double eps){

	final XXMatrs m=ts.getXXandXKftoxX();
	final double [] basis=KGOSolutionVectorXVectorF.EV.getGEV_Generalized(m.dim1,m.XfX,m.XX,true),
	    eigenvalues=java.util.Arrays.copyOf​(basis,m.dim1);
	System.err.println("testApproachVectorXVectorF_Simple: EV="+java.util.Arrays.toString(eigenvalues)+" SUM(EV)="+TMatr.sumElements(eigenvalues)+" Total weight="+ts.getTotalWeight()+" ts.getWeightFromJointProbabilityLSQpredictor()="+ts.getWeightFromJointProbabilityLSQpredictor()+" m.dim1="+m.dim1+" flag_exact="+flag_exact);

	UnitTests.checkDoubleRel(TMatr.sumElements(eigenvalues),"sum(EV)",ts.getProjectedTotalCoverageWithoutSpectrum(),"ts.getProjectedTotalCoverageWithoutSpectrum()",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(eigenvalues),"sum(EV)",ts.getProjectedTotalCoverageWithoutSpectrum2(),"ts.getProjectedTotalCoverageWithoutSpectrum2()",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(eigenvalues),"sum(EV)",ts.getProjectedTotalCoverageWithoutSpectrum3(basis),"ts.getProjectedTotalCoverageWithoutSpectrum3(basis)",eps);

	if(flag_exact){
	    UnitTests.checkDoubleRel(TMatr.sumElements(eigenvalues),"sum(EV)",ts.getTotalWeight(),"ts.getTotalWeight()",eps);
	    UnitTests.checkDoubleRel(ts.getWeightFromJointProbabilityLSQpredictor(),"ts.getWeightFromJointProbabilityLSQpredictor()",ts.getTotalWeight(),"ts.getTotalWeight()",eps);
	} else {
	    // they are in different bases (x and f), the condition
	    // CoverageWidetildeLSQ<CoverageBestEstimator
	    // does not hold true
	    /*
	    final double CoverageWidetildeLSQ=ts.getWeightFromJointProbabilityLSQpredictor(),
		CoverageBestEstimator=ts.getProjectedTotalCoverageWithoutSpectrum();
	    if(CoverageWidetildeLSQ<CoverageBestEstimator){
		throw new RuntimeException("Coverages do not relate: CoverageWidetildeLSQ="+CoverageWidetildeLSQ+" CoverageBestEstimator="+CoverageBestEstimator);
	    }
	    */
	}

	final XXMatrs mf=ts.directProjectionTestSolutionVectorXVectorF.rankMatrices;
	final double [] basisf=KGOSolutionVectorXVectorF.EV.getGEV_Generalized(mf.dim1,mf.XfX,mf.XX,true),
	    eigenvaluesf=java.util.Arrays.copyOf​(basisf,mf.dim1);
	System.err.println("testApproachVectorXVectorF_Simple: EVF="+java.util.Arrays.toString(eigenvaluesf)+" SUM(EVF)="+TMatr.sumElements(eigenvaluesf)+" full rank="+mf.dim1);
	UnitTests.checkDoubleRel(TMatr.sumElements(eigenvaluesf),"sum(EVF)",ts.directProjectionTestSolutionVectorXVectorF.getProjectedTotalRankWithoutSpectrum(),"ts.directProjectionTestSolutionVectorXVectorF.getProjectedTotalRankWithoutSpectrum()",eps);

	if(flag_exact) {
	    UnitTests.checkDoubleRel(TMatr.sumElements(eigenvaluesf),"sum(EVF)",mf.dim1,"mf.dim1",eps);
	}


	final double srankN=ts.ro.data.stream().mapToDouble(o -> o.weight*ts.directProjectionTestSolutionVectorXVectorF.evaluateAt(o.x).getChristoffelOatX()).sum();
	UnitTests.checkDoubleRel(srankN,"srankN",ts.directProjectionTestSolutionVectorXVectorF.getProjectedTotalRankWithoutSpectrum(),"ts.directProjectionTestSolutionVectorXVectorF.getProjectedTotalRankWithoutSpectrum()",eps);


	final double srank=ts.ro.data.stream().mapToDouble(
	    o -> o.weight*TMatr.mulVV(o.f,ts.directProjectionTestSolutionVectorXVectorF.evaluateAt(o.x).getPsiFNonNormalized())
							).sum();
	UnitTests.checkDoubleRel(srank,"srank",ts.directProjectionTestSolutionVectorXVectorF.getProjectedTotalRankWithoutSpectrum(),"ts.directProjectionTestSolutionVectorXVectorF.getProjectedTotalRankWithoutSpectrum()",eps);

	// test normalizing
	for(ObservationVectorXVectorF oxtest : ts.ro.data){
	    final VectorXVectorFPointAccess a=ts.directProjectionTestSolutionVectorXVectorF.evaluateAt(oxtest.x);
	    final double [] psif=TMatr.getVectorScaled(a.getPsiFNonNormalized(),1/Math.sqrt(a.getChristoffelOatX()));
	    // test normalizing <psi^2>=1
	    final double norm=ts.ro.data.stream().mapToDouble(o -> o.weight*Math.pow(TMatr.mulVV(o.f,psif),2)).sum();
	    UnitTests.checkDoubleRel(norm,"norm",1,"1",eps);
	}

	System.err.println("testApproachVectorXVectorF_Simple(): OK eps="+eps);
    }

    static void testApproachVectorXVectorF_SpectralTest(final TestDirectProjection ts,final boolean flag_exact,final double eps){

	final int nF=ts.directProjectionTestSolutionVectorXVectorF.nF;
	for(ObservationVectorXVectorF oxtest : ts.ro.data){
	    final VectorXVectorFPointAccess a=ts.directProjectionTestSolutionVectorXVectorF.evaluateAt(oxtest.x);
	    final double [] b=TMatr.getVectorScaled(a.getFLeastSquaresX(),1/Math.sqrt(a.getChristoffelOatX()));

	    UnitTests.checkDoubleRel(a.getMaximalProbability(),"a.getMaximalProbability()",1,"1",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(b,ts.directProjectionTestSolutionVectorXVectorF.Gfm1,nF),"bGfm1b",1,"1",eps);
	    UnitTests.checkDoubleRel(a.getFProbabilityAtX(b),"a.getFProbabilityAtX(b)",1,"1",eps);
	    UnitTests.checkDoubleRel(a.getFProbabilityAtX(a.getFLeastSquaresX()),"a.getFProbabilityAtX(a.getFLeastSquaresX())",1,"1",eps);
	    // for non--mangled basis the 0-th element is a constant=1, obtain exactly
	    if(!flag_exact){
		UnitTests.checkDoubleRel(a.getFLeastSquaresX()[0],"getFLeastSquaresX()[0]",1,"1",eps);
	    }


	    if(flag_exact){
		UnitTests.checkDoubleRel(a.getFProbabilityAtX(oxtest.f),"a.getFProbabilityAtX(oxtest.f)",1,"1",eps);
	    }

	    for(double [][] mm:new double [][][]{
		    {  TMatr.getVectorScaled(TMatr.getQQaa(a.getPsiFNonNormalized()),1/a.getChristoffelOatX()),
		       ts.directProjectionTestSolutionVectorXVectorF.Gfm1
		    },
		    {  TMatr.getQQaa(b),
		       ts.directProjectionTestSolutionVectorXVectorF.FF
		    }}){
		final double [] L=mm[0],R=mm[1];
		final double [] basisf=KGOSolutionVectorXVectorF.EV.getGEV_Generalized(nF,L,R,true),
		    eigenvaluesf=java.util.Arrays.copyOf​(basisf,nF);
		if(Double.isFinite(eigenvaluesf[nF-1])) {
		    // a kludge for problematic evsolver
		    UnitTests.checkDoubleRel(eigenvaluesf[nF-1],"eigenvaluesf[nF-1]",a.getMaximalProbability(),"a.getMaximalProbability()=1",eps);
		    for(int k=0;k<nF-1;k++){
			UnitTests.checkDoubleRel(eigenvaluesf[k],"eigenvaluesf[k]",0,"0",eps);
		    }
		}
		System.err.println("a.getMaximalProbability()="+a.getMaximalProbability()+" conditional probability="+a.getFProbabilityAtX(oxtest.f)+" ev="+java.util.Arrays.toString(eigenvaluesf));
	    }
	}
    }

    static void testApproachVectorXVectorF_TestCorrelation(final TestDirectProjection ts,final double eps){

	final XXMatrs mf=ts.directProjectionTestSolutionVectorXVectorF.rankMatrices,mx=ts.getXXandXKftoxX();
	if(mf.dim1!=2) throw new RuntimeException("dim f!=2");
	if(mx.dim1!=2) throw new RuntimeException("dim x!=2");
	final double [] basisf=KGOSolutionVectorXVectorF.EV.getGEV_Generalized(mf.dim1,mf.XfX,mf.XX,true),
	    eigenvaluesf=java.util.Arrays.copyOf​(basisf,mf.dim1);

	// Correlation. The 0-th element of x and f is 1.
	final double xa=ts.getAveraged(o->o.x[1]),fa=ts.getAveraged(o->o.f[1]),
	    corr=ts.getAveraged(o->(o.x[1]-xa)*(o.f[1]-fa))/
	    Math.sqrt(ts.getAveraged(o->Math.pow(o.x[1]-xa,2))*ts.getAveraged(o->Math.pow(o.f[1]-fa,2)));
	// minimal eigenvalues is correlation squared
	UnitTests.checkDoubleRel(eigenvaluesf[0],"eigenvaluesf[0]",Math.pow(corr,2),"corrr^2",eps);
	// maximal eigenvalue is 1 because a const present bot in x and f basis
	UnitTests.checkDoubleRel(eigenvaluesf[1],"eigenvaluesf[1]",1,"1",eps);

	System.err.println("testApproachVectorXVectorF_TestCorrelation: EVF="+java.util.Arrays.toString(eigenvaluesf)+" SUM(EVF)="+TMatr.sumElements(eigenvaluesf)+" full rank="+mf.dim1+" corr="+corr+" corr2="+Math.pow(corr,2));
    }


    static void testApproachVectorXVectorF_Christoffel(final TestDirectProjection ts,final boolean flag_exact,final double eps){
	final int nf=ts.ro.fHeader.size();
	final XXMatrs m=ts.getXXandXKfX();
	final double [] basis=KGOSolutionVectorXVectorF.EV.getGEV_Generalized(m.dim1,m.XfX,m.XX,true),
	    eigenvalues=java.util.Arrays.copyOf​(basis,m.dim1),
	    eigenvaluesMaxNf=java.util.Arrays.copyOfRange​(eigenvalues,eigenvalues.length-nf,eigenvalues.length);
	System.err.println("EVs="+java.util.Arrays.toString(eigenvalues)+" W="+ts.getTotalWeight()+" ratio="+(TMatr.sumElements(eigenvalues)/ts.getTotalWeight())+" xKfx[0,0]/xx[0,0]="+(m.XfX[0]/m.XX[0]));
	//if(flag_exact) UnitTests.checkDoubleRel(TMatr.sumElements(eigenvaluesMaxNf),"sum(eigenvaluesMaxNf)",ts.getTotalWeight(),"ts.getTotalWeight()",eps);

	if(true){
	    // test Christoffel function in XX and FF bases
	    for(XXMatrs mm : java.util.List.of(ts.getXXandXKX(),ts.getFFandFKF())){
		    final double [] basismm=KGOSolutionVectorXVectorF.EV.getGEV_Generalized(mm.dim1,mm.XfX,mm.XX,true),
			eigenvaluesmm=java.util.Arrays.copyOf​(basismm,mm.dim1);
		    UnitTests.checkDoubleRel(TMatr.sumElements(eigenvaluesmm),"sum(eigenvaluesmm)",ts.getTotalWeight(),"ts.getTotalWeight()",eps);
		    System.err.println("Christoffel: EV="+java.util.Arrays.toString(eigenvaluesmm)+" SUM(EV)="+TMatr.sumElements(eigenvaluesmm)+" W="+ts.getTotalWeight());
	    }
	}
	System.err.println("testApproachVectorXVectorF_TestXFDegeneracy(): OK eps="+eps+" flag_exact="+flag_exact);
    }



    /** Just to test a concept: try many to many.
     */
    static void testApproachVectorXVectorF_Not_Ready_Yet(final int nx,final int nf,final java.util.Random r,final int size,final double eps){

	testApproachVectorXVectorF_TestCorrelation(new TestDirectProjection(TestObservationSequenceFromSampleVectorXVectorF.getNoNDegeneratedVectorXVectorF(2,2,r,size)),eps);
	boolean flag_exact=false;
	for(TestDirectProjection ts:java.util.List.of(
		new TestDirectProjection(TestObservationSequenceFromSampleVectorXVectorF.getNoNDegeneratedVectorXVectorF(nx,nf,r,size)),
		new TestDirectProjection(TestObservationSequenceFromSampleVectorXVectorF.getNoNDegeneratedVectorXVectorF_CombinedXandF(nx,nf,r,size))
								   )){
	    testApproachVectorXVectorF_Simple(ts,flag_exact,eps);
	    testApproachVectorXVectorF_SpectralTest(ts,flag_exact,eps);
	    testApproachVectorXVectorF_Christoffel(ts,flag_exact,eps);
	    flag_exact=!flag_exact;
	}

	System.err.println("testApproachVectorXVectorF_Not_Ready_Yet(): OK eps="+eps);
    }


    static void runAllTests(final java.util.Random r,final double eps){
	System.err.println("TestDirectProjection:starting tests; ver=$Id: TestDirectProjection.java,v 1.70 2024/04/02 01:29:10 mal Exp $");
	final long t0=System.currentTimeMillis();

	for(int stest=0;stest<10;stest++){
	    final int size=1300+r.nextInt(1000);

	    for(int nx: java.util.List.of(2,9,3,4,16)){
		for(int nf: java.util.List.of(2,3,4,9)){
		    // just a concept
		    if(nx>nf){ // this version does not work otherwise
			testApproachVectorXVectorF_Not_Ready_Yet(nx,nf,r,size,eps);
		    }
		}
	    }
	}

	final long t1=System.currentTimeMillis();
	System.err.println("TestDirectProjection: tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: TestDirectProjection.java,v 1.70 2024/04/02 01:29:10 mal Exp $");
    }



    public static void main(String [] args){
	final double eps=1e-5;
	// force a selftest
	TestKGO.EPS_selftest=eps;
	// run a selftest
	runAllTests(new java.util.Random(12),eps);
	// disable selftest
	TestKGO.EPS_selftest=0;

    }

}

