package com.polytechnik.kgo;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.RegularizedVectorXVectorF;
import com.polytechnik.utils.DataReadObservationVectorXVectorF;
import com.polytechnik.utils.RandomSample;
import com.polytechnik.utils.ObservationVectorXVectorF;
import com.polytechnik.utils.DegAndNDeg;

/** A class with the data for vector-to-vector models unit tests.
 * @see TestDirectProjection
 * @see TestKGO
 * @version $Id: TestObservationSequenceFromSampleVectorXVectorF.java,v 1.23 2024/04/02 01:29:10 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
class TestObservationSequenceFromSampleVectorXVectorF {
    private final DataReadObservationVectorXVectorF ro;

    private TestObservationSequenceFromSampleVectorXVectorF(RandomSample [] xin,RandomSample [] fin,RandomSample weight){
	final var data=new java.util.ArrayList<ObservationVectorXVectorF>(weight.sample.length);
	for(int k=0;k<weight.sample.length;k++){
	    final double [] x=new double [xin.length];
	    for(int m=0;m<xin.length;m++){
		x[m]=xin[m].sample[k];
	    }
	    final double [] f=new double [fin.length];
	    for(int m=0;m<fin.length;m++){
		f[m]=fin[m].sample[k];
	    }
	    data.add(new ObservationVectorXVectorF(String.valueOf(k),x,f,weight.sample[k]));
	}
	this.ro=new DataReadObservationVectorXVectorF(
	    data,
	    java.util.stream.IntStream.range(0,xin.length).mapToObj(k->"X:"+k).toList(),
	    java.util.stream.IntStream.range(0,fin.length).mapToObj(k->"F:"+k).toList()
						     );
    }


    static DataReadObservationVectorXVectorF getNoNDegeneratedVectorXVectorF(
	final int nx,
	final int nf,
	final java.util.Random r,
	final int size){
	final RandomSample [] fin=new RandomSample[nf];
	for(int m=0;m<fin.length;m++){
	    if(m<=0) fin[m]=new RandomSample(1,1,size,r); // const
	    else fin[m]=new RandomSample(-3*r.nextDouble(),3*r.nextDouble(),size,r);
	}
	final RandomSample [] xin=new RandomSample[nx];
	for(int m=0;m<xin.length;m++){
	    if(m<=0) xin[m]=new RandomSample(1,1,size,r); // const
	    else xin[m]=new RandomSample(-3*r.nextDouble(),3*r.nextDouble(),size,r);
	}
	return new TestObservationSequenceFromSampleVectorXVectorF(xin,fin,new RandomSample(0.01,3,size,r)).ro;
    }

    static DataReadObservationVectorXVectorF getNoNDegeneratedVectorXVectorF_CombinedXandF(
	final int nx,
	final int nf,
	final java.util.Random r,final int size){
	final RandomSample [] fin=new RandomSample[nf];
	for(int m=0;m<fin.length;m++){
	    if(m<=0) fin[m]=new RandomSample(1,1,size,r); // const
	    else fin[m]=new RandomSample(-3*r.nextDouble(),3*r.nextDouble(),size,r);
	}
	final RandomSample [] xin=new RandomSample[nx+nf];
	for(int m=0;m<nx;m++){
	    xin[m]=new RandomSample(-3*r.nextDouble(),3*r.nextDouble(),size,r);
	}
	// Add f to x atributes; randomize to test invariance
	// This makes the Spur maximal:
	//    Generalized eigenproblem with directProjectionTestSolutionVectorXVectorF.rankMatrices matices, Spur=rank.
	//    Generalized eigenproblem with getXXandXKftoxX() matices, Spur=total weight.
	// In general case the error is the difference between max value and current value.
	for(int m=nx,j=0;m<xin.length;m++,j++){
	    xin[m]=fin[j];
	    // randomize to test invariance
	    TMatr.sumP1(xin[m].sample,xin[r.nextInt(m+1)].sample,0.4*(1-2*r.nextDouble()));
	    TMatr.sumP1(xin[m].sample,xin[r.nextInt(m+1)].sample,0.4*(1-2*r.nextDouble()));
	}
	return new TestObservationSequenceFromSampleVectorXVectorF(xin,fin,new RandomSample(0.01,3,size,r)).ro;
    }

    static DataReadObservationVectorXVectorF makeSampleDegenerate(
	final DataReadObservationVectorXVectorF ro,
	final int nToAddX,
	final int nSumX,
	final int nToAddF,
	final int nSumF,
	final java.util.Random r){

    final int [][] xInd=new int [nToAddX][nSumX],fInd=new int [nToAddF][nSumF];
    final double [][] xCoef=new double [nToAddX][nSumX],fCoef=new double [nToAddF][nSumF];
    final java.util.ArrayList<String>
	xHeader=new java.util.ArrayList<>(),fHeader=new java.util.ArrayList<>();
    xHeader.addAll(ro.xHeader);
    fHeader.addAll(ro.fHeader);
    final int nX=ro.xHeader.size(),nF=ro.fHeader.size();
    for(int j=0;j<xInd.length;j++){
	String s="";
	for(int k=0;k<nSumX;k++){
	    xInd[j][k]=r.nextInt(nX);
	    xCoef[j][k]=5.2*(1-2*r.nextDouble());
	    s+=(j>0?"+":"")+"x["+xInd[j][k]+"]*"+xCoef[j][k];
	}
	xHeader.add(s);
    }
    for(int j=0;j<fInd.length;j++){
	String s="";
	for(int k=0;k<nSumF;k++){
	    fInd[j][k]=r.nextInt(nF);
	    fCoef[j][k]=5.1*(1-2*r.nextDouble());
	    s+=(j>0?"+":"")+"f["+fInd[j][k]+"]*"+fCoef[j][k];
	}
	fHeader.add(s);
    }

    final var data=new java.util.ArrayList<ObservationVectorXVectorF>(10+ro.data.size());
    for(var o:ro.data){
	final double [] x=new double [nX+xInd.length],f=new double [nF+fInd.length];
	System.arraycopy(o.x,0,x,0,nX);
	for(int ns=nX,j=0;j<xInd.length;j++,ns++){
	    for(int k=0;k<nSumX;k++){
		x[ns]+=x[xInd[j][k]]*xCoef[j][k];
	    }
	}
	System.arraycopy(o.f,0,f,0,nF);
	for(int ns=nF,j=0;j<fInd.length;j++,ns++){
	    for(int k=0;k<nSumF;k++){
		f[ns]+=f[fInd[j][k]]*fCoef[j][k];
	    }
	}
	data.add(new ObservationVectorXVectorF(o.label,x,f,o.weight));
    }
    return new DataReadObservationVectorXVectorF(data,xHeader,fHeader);
}


    static DegAndNDeg<DataReadObservationVectorXVectorF>
	getDegeneratedXandF_VectorXVectorF(final int nx,final int nf,final java.util.Random r,final int size){

	final DataReadObservationVectorXVectorF ro=getNoNDegeneratedVectorXVectorF(nx,nf,r,size),
	    ro_deg=makeSampleDegenerate(ro,6,14,5,15,r);
	return new DegAndNDeg<DataReadObservationVectorXVectorF>(ro,ro_deg);
    }

    static class RegularizationWithSample {
	final java.util.List<RegularizedVectorXVectorF> regList;
	final DataReadObservationVectorXVectorF ro;
	RegularizationWithSample(final java.util.List<RegularizedVectorXVectorF> regList,
				 final DataReadObservationVectorXVectorF ro){
	    this.regList=regList;
	    this.ro=ro;
	}
    }


    static void matchFunctionOnTwoSamples(
	final String txt,
	final DataReadObservationVectorXVectorF ro1,
	final java.util.function.ToDoubleFunction<ObservationVectorXVectorF> fun1,
	final DataReadObservationVectorXVectorF ro2,
	final java.util.function.ToDoubleFunction<ObservationVectorXVectorF> fun2,
	final double eps){
	if(ro1.data.size()!=ro2.data.size())
	    throw new RuntimeException("Size mismatch :"+ro1.data.size()+
				       " and "+ro2.data.size());
	final var it2=ro2.data.iterator();
	for(var o1:ro1.data){
	    final var o2=it2.next();
	    UnitTests.checkDoubleRel1(fun1.applyAsDouble(o1),"fun1(o1)",fun2.applyAsDouble(o2),"fun2(o2)",eps);
	}
	if(txt!=null)System.err.println(txt+":matchFunctionOnTwoSamples(): ro1.nx="+ro1.xHeader.size()+" ro2.nx="+ro2.xHeader.size()+" ro1.nf="+ro1.fHeader.size()+" ro2.nf="+ro2.fHeader.size()+" OK size="+ro2.data.size()+" eps="+eps);
    }

    static void matchArrayFunctionOnTwoSamples(
	final String txt,
	final DataReadObservationVectorXVectorF ro1,
	final java.util.function.Function<ObservationVectorXVectorF,double []> fun1,
	final DataReadObservationVectorXVectorF ro2,
	final java.util.function.Function<ObservationVectorXVectorF,double []> fun2,
	final double eps){
	if(ro1.data.size()!=ro2.data.size())
	    throw new RuntimeException("Size mismatch :"+ro1.data.size()+
				       " and "+ro2.data.size());
	final var it2=ro2.data.iterator();
	for(var o1:ro1.data){
	    final var o2=it2.next();
	    UnitTests.checkDoubleArrayRel1(fun1.apply(o1),"fun1(o1)",fun2.apply(o2),"fun2(o2)",eps);
	}
	if(txt!=null)System.err.println(txt+":matchArrayFunctionOnTwoSamples(): OK size="+ro2.data.size()+" eps="+eps);
    }


    static void testBasisConversion(final java.util.List<RegularizationWithSample> dList,
				    final boolean fStartsTheSame,
				    final double eps){

	// Test different regularizations produce the same results.
	for(var dL1: dList){
	    final var ro1=dL1.ro;
	    for(var reg1:dL1.regList){
		// check matrices
		final double [] XX=ro1.getFun1Fun2Averaged(o->reg1.X.getRegularizedBasisXFromOriginalX(o.x),
							   o->reg1.X.getRegularizedBasisXFromOriginalX(o.x),
							   o->1);
		UnitTests.checkDoubleArrayRel1(XX,"XX",TMatr.getUnitMatrix(reg1.X.nX),"1",eps);
		final double [] FF=ro1.getFun1Fun2Averaged(o->reg1.F.getRegularizedBasisXFromOriginalX(o.f),
							   o->reg1.F.getRegularizedBasisXFromOriginalX(o.f),
							   o->1);
		UnitTests.checkDoubleArrayRel1(FF,"FF",TMatr.getUnitMatrix(reg1.F.nX),"1",eps);

		// x-expansion in X should be exact
		final var xX=ro1.getFun1Fun2Averaged(
		    o->o.x,o->reg1.X.getRegularizedBasisXFromOriginalX(o.x),o->1);
		matchArrayFunctionOnTwoSamples(
		    null/*"LeastSquares x in X"*/,
		    ro1,o->KGOSolutionVectorXVectorF.leastSquaresO(xX,reg1.X,o.x),
		    ro1,o->o.x,eps);
		// f-expansion in F should be exact
		final var fF=ro1.getFun1Fun2Averaged(
		    o->o.f,o->reg1.F.getRegularizedBasisXFromOriginalX(o.f),o->1);
		matchArrayFunctionOnTwoSamples(
		    null/*"LeastSquares f in F"*/,
		    ro1,o->KGOSolutionVectorXVectorF.leastSquaresO(fF,reg1.F,o.f),
		    ro1,o->o.f,eps);

		// Pair reg1, reg2 matching. Should give the same value.

		// test original f, not regularized.
		final var fX1=ro1.getFun1Fun2Averaged(
		    o->o.f,o->reg1.X.getRegularizedBasisXFromOriginalX(o.x),o->1);
		// check some values match
		for(var dL2: dList){
		    final var ro2=dL2.ro;
		    for(var reg2:dL2.regList){
			// test original f, not regularized.
			final var fX2=ro2.getFun1Fun2Averaged(
			    o->o.f,o->reg2.X.getRegularizedBasisXFromOriginalX(o.x),o->1);

			matchFunctionOnTwoSamples(
			    null,//"ChristoffelX",
			    ro1,o->KGOSolutionVectorXVectorF.christoffel(reg1.X,o.x),
			    ro2,o->KGOSolutionVectorXVectorF.christoffel(reg2.X,o.x),
			    eps);
			matchFunctionOnTwoSamples(
			    null,//"ChristoffelF",
			    ro1,o->KGOSolutionVectorXVectorF.christoffel(reg1.F,o.f),
			    ro2,o->KGOSolutionVectorXVectorF.christoffel(reg2.F,o.f),
			    eps);

			// we use degenerated f-data, the f-size may be different
			if(fStartsTheSame){
			    final int nFToCheck=Math.min(reg1.F.noriginal,reg2.F.noriginal);
			    matchArrayFunctionOnTwoSamples(
			    null/*"LeastSquares f in X"*/,
			    ro1,o->java.util.Arrays.copyOf(KGOSolutionVectorXVectorF.leastSquaresO(fX1,reg1.X,o.x),nFToCheck),
			    ro2,o->java.util.Arrays.copyOf(KGOSolutionVectorXVectorF.leastSquaresO(fX2,reg2.X,o.x),nFToCheck),
			    eps);
			}
		    }
		}
	    }
	}
	System.err.println("testBasisConversion(): OK eps="+eps);
    }

}

