package com.polytechnik.kgo;
/*
 * (C) Vladislav Malyshkin 2021
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.VectorXVectorFPointEvaluatable;
import com.polytechnik.utils.RegularizedVectorX;
import com.polytechnik.utils.VectorXVectorFPointAccess;

/** A helper class for common vector type of evaluation.
 * @version $Id: VectorXVectorFPointEvaluatableCommon.java,v 1.10 2024/05/02 12:33:53 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class VectorXVectorFPointEvaluatableCommon implements VectorXVectorFPointEvaluatable {

    /** Some extra data.
     */
    public static interface VectorXVectorFPointAccessWithDimensionAdjusted extends VectorXVectorFPointAccess {
	double getChristoffelOatXDimensionAdjusted();
    }

    private final RegularizedVectorX regX;
    private final RegularizedVectorX regF;
    /** Contributing subspace.
     */
    private final double [] CX;
    private final UAdjustment uadj;
    private final java.util.function.Function<double[],double []> forigFromF;
    private final double [] FX;
    private final SKFromSampleCalculationMethod SKtype;

    VectorXVectorFPointEvaluatableCommon(
	final UAdjustment uadj,
        final RegularizedVectorX regX,
	final RegularizedVectorX regF,
	final double [] CX,
	final java.util.function.Function<double[],double []> forigFromF,
	final double [] FX,
	final SKFromSampleCalculationMethod SKtype){

	this.uadj=uadj;
	this.regX=regX;
	this.regF=regF;
	this.CX=CX;
	this.forigFromF=forigFromF;
	this.FX=FX;
	this.SKtype=SKtype;
    }

    public VectorXVectorFPointAccessWithDimensionAdjusted evaluateAt(final double [] xorig){
	final int nX=regX.nX,nF=regF.nX;
	final double [] X=regX.getRegularizedBasisXFromOriginalX(xorig);
	if(X.length!=nX||xorig.length!=regX.noriginal) throw new IllegalArgumentException();
	final double [] forig_leastSquares=forigFromF.apply(KGOSolutionVectorXVectorF.leastSquaresO(FX,regX,xorig));
	// we have a normalized basis
	final double _ChristoffelOatX=TMatr.mulVV(X,X);
	final double [] C=TMatr.mul_MA_x_MB(CX.length/X.length,CX,X.length,X,1);
	final double _ChristoffelOatXDimensionAdjusted=TMatr.mulVV(C,C);

	// The only coefficients required to obtain all the results
	final double [] alphaFCoefs=TMatr.mul_MA_x_MBTrans(nF,uadj.getUAdj(),nX,X,1);


	return new VectorXVectorFPointAccessWithDimensionAdjusted() {
	    public double getFProbabilityAtX(final double [] forig){
		final double [] F=regF.getRegularizedBasisXFromOriginalX(forig);
		if(F.length!=nF||forig.length!=regF.noriginal) throw new IllegalArgumentException();
		final double denomfact=switch(SKtype) {
		case CHRISTOFFEL_DEGREESOFFREEDOMADJUSTED,FXFX_F_CHRISTOFFEL,FXFX -> TMatr.mulVV(alphaFCoefs,alphaFCoefs);
		case CHRISTOFFEL ->TMatr.mulVV(X,X);
		default -> throw new IllegalArgumentException();
		};

		return Math.pow(TMatr.mulVV(F,alphaFCoefs),2)/
		    (TMatr.mulVV(F,F)*denomfact)
		    ;
	    }
	    public double getChristoffelOatX(){return _ChristoffelOatX;}
	    public double getChristoffelOatXDimensionAdjusted(){return _ChristoffelOatXDimensionAdjusted;}
	    public double getChristoffelOatF(final double [] forig){
		final double [] F=regF.getRegularizedBasisXFromOriginalX(forig);
		if(F.length!=nF||forig.length!=regF.noriginal) throw new IllegalArgumentException();
		// we have a normalized basis
		return TMatr.mulVV(F,F);
	    }
	    /** Equals to {@code 1} for adjusted number
	     * of the degrees of freedom, otherwise not.
	     */
	    public double getMaximalProbability(){
		return switch(SKtype) {
		case CHRISTOFFEL_DEGREESOFFREEDOMADJUSTED,FXFX_F_CHRISTOFFEL,FXFX -> 1;
		case CHRISTOFFEL ->TMatr.mulVV(alphaFCoefs,alphaFCoefs)/TMatr.mulVV(X,X);
		default -> throw new IllegalArgumentException();
		};
	    }
	    public double [] getPsiFNonNormalized(){return alphaFCoefs.clone();}
	    public double [] getFLeastSquaresX(){return forig_leastSquares.clone();}
	    public double [] getFofMaximalProbability(){
		return forigFromF.apply(TMatr.getVectorScaled(alphaFCoefs,regF.normalizedconstbasisfunction/alphaFCoefs[regF.indexConst]));
	    }
	};
    }
}
