#include <lapacke.h>
#include "com/polytechnik/lapack/com_polytechnik_lapack_Eigenvalues_JNI_lapacke.h"
#include <math.h>
#include <string.h>

/** C code implementations for Eigenvalues_JNI_lapacke.java
 *  @version $Id: Eigenvalues_JNI_lapacke.c,v 1.10 2022/10/20 08:47:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
jdoubleArray getEVArrayFrom2(JNIEnv *env,jint n, double w[],double z[],_Bool flag_NaN,_Bool flag_ev)
{
    if(flag_NaN)
    {
	for(int k=0;k<n;k++) w[k]=NAN;
	if(flag_ev) for(int k=n*n;--k>=0;) z[k]=NAN;
    }
    jdoubleArray ev = (*env)->NewDoubleArray(env, flag_ev?n+n*n:n);
    (*env)->SetDoubleArrayRegion(env, ev, 0 , n, w);
    if(flag_ev) (*env)->SetDoubleArrayRegion(env, ev, n , n*n, z);
    return ev;
}


JNIEXPORT jdoubleArray JNICALL Java_com_polytechnik_lapack_Eigenvalues_1JNI_1lapacke_getEV_1dsyevx
    (JNIEnv *env, jclass _class, jint n, jdoubleArray matr_j, jint lda_matr, jboolean flag_ev)
{
    lapack_int info=-1,nevfound=0;
    char jobz=flag_ev?'V':'N';
    double *w=malloc(n*sizeof(double)),*z=flag_ev?malloc((n*n)*sizeof(double)):NULL;
    if(n!=lda_matr || n*n>(*env)->GetArrayLength(env, matr_j)) goto errx;
    lapack_int *ifail=malloc(n*sizeof(lapack_int));
    jdouble *matr = (*env)->GetDoubleArrayElements(env, matr_j, NULL),
	*_matr=malloc((n*n)*sizeof(jdouble));
    memcpy(_matr,matr,(n*n)*sizeof(jdouble)); // lda_matr=n
    
    info=LAPACKE_dsyevx(LAPACK_COL_MAJOR,
			jobz,
			'A',
			'U',
			n,
			_matr,
			lda_matr,
			0,
			0,
			0,
			0,
			0.0,
			&nevfound,
			w,z,
			n,ifail);
    (*env)->ReleaseDoubleArrayElements(env, matr_j, matr, JNI_ABORT);
    free(ifail);free(_matr);
 errx:;
    jdoubleArray ev=getEVArrayFrom2(env,n,w,z,info!=0 || nevfound!=n,flag_ev);
    free(w);free(z);
    return ev;
}



JNIEXPORT jdoubleArray JNICALL Java_com_polytechnik_lapack_Eigenvalues_1JNI_1lapacke_getEV3_1dstemr
  (JNIEnv *env, jclass _class, jint n, jdoubleArray diag_j, jint diag_off, jdoubleArray offdiag_j, jint offdiag_off, jboolean flag_ev)
{
    if(n<=0) return getEVArrayFrom2(env,0,NULL,NULL,JNI_FALSE,JNI_FALSE);
    lapack_int info=-1,nevfound=0;
    char jobz=flag_ev?'V':'N';
    double *w=malloc(n*sizeof(double)),*z=flag_ev?malloc((n*n)*sizeof(double)):NULL;

    if(n>(*env)->GetArrayLength(env, diag_j)+diag_off||
       n>(*env)->GetArrayLength(env, offdiag_j)+offdiag_off) goto errx;
    lapack_int *isupz=malloc(2*n*sizeof(lapack_int));
    lapack_logical tryrac=JNI_FALSE;
    jdouble *diag = (*env)->GetDoubleArrayElements(env, diag_j, NULL),
	*offdiag=(*env)->GetDoubleArrayElements(env,offdiag_j, NULL),
	*_diag=malloc(n*sizeof(jdouble)),*_offdiag=malloc(n*sizeof(jdouble));
    //fprintf(stderr,"n=%d diag_off=%d offdiag_off=%d lendiag=%d lenoffdiag=%d\n",n,diag_off,offdiag_off,(*env)->GetArrayLength(env, diag_j),(*env)->GetArrayLength(env, offdiag_j));    
    memcpy(_diag,&diag[diag_off],n*sizeof(jdouble));
    memcpy(_offdiag,&offdiag[offdiag_off],(n-1)*sizeof(jdouble));
    
    info=LAPACKE_dstemr(LAPACK_COL_MAJOR,
			jobz,
			'A',
			n,
			_diag,
			_offdiag,
			0,0,-1,-1,
			&nevfound,
			w,z,n,
			n,
			isupz,
			&tryrac);
    
    (*env)->ReleaseDoubleArrayElements(env, diag_j, diag, JNI_ABORT);
    (*env)->ReleaseDoubleArrayElements(env, offdiag_j, offdiag, JNI_ABORT);
    free(isupz);free(_diag);free(_offdiag);
 errx:;
    jdoubleArray ev=getEVArrayFrom2(env,n,w,z,info!=0 || nevfound!=n,flag_ev);
    free(w);free(z);
    return ev;
}

JNIEXPORT jdoubleArray JNICALL Java_com_polytechnik_lapack_Eigenvalues_1JNI_1lapacke_getGEV_1dsygvx
    (JNIEnv *env, jclass _class, jint n, jdoubleArray A_j, jint lda_A, jdoubleArray B_j, jint lda_B, jboolean flag_ev)
{
    if(n<=0) return getEVArrayFrom2(env,0,NULL,NULL,JNI_FALSE,JNI_FALSE);
    lapack_int info=-1,nevfound=0;
    char jobz=flag_ev?'V':'N';
    double *w=malloc(n*sizeof(double)),*z=flag_ev?malloc((n*n)*sizeof(double)):NULL;
    if(n!=lda_A || n!=lda_B) goto errx;
    lapack_int *ifail=malloc(n*sizeof(lapack_int));
    jdouble *A = (*env)->GetDoubleArrayElements(env, A_j, NULL),
	*B=(*env)->GetDoubleArrayElements(env,B_j, NULL),
	*_A=malloc((n*n)*sizeof(jdouble)),*_B=malloc((n*n)*sizeof(jdouble));
    memcpy(_A,A,(n*n)*sizeof(jdouble));
    memcpy(_B,B,(n*n)*sizeof(jdouble));
        
    info=LAPACKE_dsygvx (LAPACK_COL_MAJOR,
			 1,
			 jobz,
			 'A',
			 'U',
			 n,
			 _A,lda_A,
			 _B,lda_B,
			 0,0,0,0,0.0,
			 &nevfound,
			 w,z,n,ifail);
    free(ifail);free(_A);free(_B);
    (*env)->ReleaseDoubleArrayElements(env, A_j, A, JNI_ABORT);
    (*env)->ReleaseDoubleArrayElements(env, B_j, B, JNI_ABORT);

 errx:;
    jdoubleArray ev=getEVArrayFrom2(env,n,w,z,info!=0 || nevfound!=n,flag_ev);
    free(w);free(z);
    return ev;
}

JNIEXPORT jint JNICALL Java_com_polytechnik_lapack_Eigenvalues_1JNI_1lapacke_getNonSymEV_1dgees
  (JNIEnv *env, jclass _class, jint n, jdoubleArray matr_j, jint lda_matr, jdoubleArray wr_j, jdoubleArray wi_j)
{
    lapack_int sdim=0,info=-1;
    if(n==0) return 0;
    if(n!=lda_matr||
       n!=(*env)->GetArrayLength(env, wr_j)||
       n!=(*env)->GetArrayLength(env, wi_j)||
       n*n>(*env)->GetArrayLength(env, matr_j)) return info;
    jdouble *matr = (*env)->GetDoubleArrayElements(env, matr_j, NULL),
	*wr = (*env)->GetDoubleArrayElements(env, wr_j, NULL),
	*wi = (*env)->GetDoubleArrayElements(env, wi_j, NULL),
	*_matr=malloc((n*n)*sizeof(jdouble));
    memcpy(_matr,matr,(n*n)*sizeof(jdouble));
    double *vs=malloc((n*n)*sizeof(double));
   
    info=LAPACKE_dgees(LAPACK_COL_MAJOR,
		       'N','N',
		       NULL,
		       n,
		       _matr,
		       lda_matr,
		       &sdim,
		       wr,
		       wi,
		       vs,n);
    
    (*env)->ReleaseDoubleArrayElements(env, matr_j, matr, JNI_ABORT);
    (*env)->ReleaseDoubleArrayElements(env, wr_j, wr, 0);
    (*env)->ReleaseDoubleArrayElements(env, wi_j, wi, 0);
    free(vs);free(_matr);
    return info;
}

