package com.polytechnik.lapack;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TestEigenvalues;

/** Eigenvectors problem. Direct JNI C-calls to lapacke.
 * To compile:
 *
 *<pre>
{@code
  javac -g -h com/polytechnik/lapack/  com/polytechnik/lapack/Eigenvalues_JNI_lapacke.java
  x="`java -XshowSettings:properties -version 2>&1| sed -nE 's/^\s*java.home = //p'`/"
  gcc -Wall -fPIC -I.  -I"$x/include" -I"$x/include/linux" -shared -o com/polytechnik/lapack/libEigenvalues_JNI_lapacke.so  com/polytechnik/lapack/Eigenvalues_JNI_lapacke.c -llapacke
  java -Djava.library.path=com/polytechnik/lapack/   com/polytechnik/lapack/Eigenvalues_JNI_lapacke
}</pre>
 * 
 *
 *  @version $Id: Eigenvalues_JNI_lapacke.java,v 1.12 2022/10/20 18:31:14 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class Eigenvalues_JNI_lapacke {

static {
    System.loadLibrary("Eigenvalues_JNI_lapacke"); // load shared library
}


public static native double [] getEV_dsyevx(final int n,final double [] matr,final int lda_matr,final boolean flag_ev);
public static native double [] getEV3_dstemr(final int n,final double [] diag,final int diag_off,final double [] offdiag,final int offdiag_off,final boolean flag_ev);
public static native double [] getGEV_dsygvx(final int n,final double [] A,final int lda_A,final double [] B,final int lda_B,final boolean flag_ev);
public static native int getNonSymEV_dgees(final int n,final double [] matr,final int lda_matr,final double [] wr,final double [] wi);

public static void main(String [] args)
{
    final double eps=1e-13;
    final java.util.Random r=new java.util.Random(1167);
    final int n_tests=1000;
    for(int n=1;n<50;n++)
    {
	TestEigenvalues.test_getEV(Eigenvalues_JNI_lapacke::getEV_dsyevx,n_tests,n,r,eps);
	//fails //TestEigenvalues.test_getEV3(Eigenvalues_JNI_lapacke::getEV3_dstemr,n_tests,n,r,eps);
	TestEigenvalues.test_getGEV(Eigenvalues_JNI_lapacke::getGEV_dsygvx,n_tests,n,r,eps);
    }
       
}

}
