/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dbdsqr.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dbdsqr {

// *> \brief \b DBDSQR
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DBDSQR + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DBDSQR( UPLO, N, NCVT, NRU, NCC, D, E, VT, LDVT, U,
// *                          LDU, C, LDC, WORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            INFO, LDC, LDU, LDVT, N, NCC, NCVT, NRU
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   C( LDC, * ), D( * ), E( * ), U( LDU, * ),
// *      $                   VT( LDVT, * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DBDSQR computes the singular values and, optionally, the right and/or
// *> left singular vectors from the singular value decomposition (SVD) of

// *> a real N-by-N (upper or lower) bidiagonal matrix B using the implicit
// *> zero-shift QR algorithm.  The SVD of B has the form
// *>
// *>    B = Q * S * P**T
// *>
// *> where S is the diagonal matrix of singular values, Q is an orthogonal
// *> matrix of left singular vectors, and P is an orthogonal matrix of
// *> right singular vectors.  If left singular vectors are requested, this
// *> subroutine actually returns U*Q instead of Q, and, if right singular

// *> vectors are requested, this subroutine returns P**T*VT instead of
// *> P**T, for given real input matrices U and VT.  When U and VT are the

// *> orthogonal matrices that reduce a general matrix A to bidiagonal
// *> form:  A = U*B*VT, as computed by DGEBRD, then
// *>
// *>    A = (U*Q) * S * (P**T*VT)
// *>
// *> is the SVD of A.  Optionally, the subroutine may also compute Q**T*C

// *> for a given real input matrix C.
// *>
// *> See "Computing  Small Singular Values of Bidiagonal Matrices With
// *> Guaranteed High Relative Accuracy," by J. Demmel and W. Kahan,
// *> LAPACK Working Note #3 (or SIAM J. Sci. Statist. Comput. vol. 11,
// *> no. 5, pp. 873-912, Sept 1990) and
// *> "Accurate singular values and differential qd algorithms," by
// *> B. Parlett and V. Fernando, Technical Report CPAM-554, Mathematics
// *> Department, University of California at Berkeley, July 1992
// *> for a detailed description of the algorithm.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          = 'U':  B is upper bidiagonal;
// *>          = 'L':  B is lower bidiagonal.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix B.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] NCVT
// *> \verbatim
// *>          NCVT is INTEGER
// *>          The number of columns of the matrix VT. NCVT >= 0.
// *> \endverbatim
// *>
// *> \param[in] NRU
// *> \verbatim
// *>          NRU is INTEGER
// *>          The number of rows of the matrix U. NRU >= 0.
// *> \endverbatim
// *>
// *> \param[in] NCC
// *> \verbatim
// *>          NCC is INTEGER
// *>          The number of columns of the matrix C. NCC >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the n diagonal elements of the bidiagonal matrix B
// *>          On exit, if INFO=0, the singular values of B in decreasing
// *>          order.
// *> \endverbatim
// *>
// *> \param[in,out] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N-1)
// *>          On entry, the N-1 offdiagonal elements of the bidiagonal
// *>          matrix B.
// *>          On exit, if INFO = 0, E is destroyed; if INFO > 0, D and E
// *>          will contain the diagonal and superdiagonal elements of a
// *>          bidiagonal matrix orthogonally equivalent to the one given
// *>          as input.
// *> \endverbatim
// *>
// *> \param[in,out] VT
// *> \verbatim
// *>          VT is DOUBLE PRECISION array, dimension (LDVT, NCVT)
// *>          On entry, an N-by-NCVT matrix VT.
// *>          On exit, VT is overwritten by P**T * VT.
// *>          Not referenced if NCVT = 0.
// *> \endverbatim
// *>
// *> \param[in] LDVT
// *> \verbatim
// *>          LDVT is INTEGER
// *>          The leading dimension of the array VT.
// *>          LDVT >= max(1,N) if NCVT > 0; LDVT >= 1 if NCVT = 0.
// *> \endverbatim
// *>
// *> \param[in,out] U
// *> \verbatim
// *>          U is DOUBLE PRECISION array, dimension (LDU, N)
// *>          On entry, an NRU-by-N matrix U.
// *>          On exit, U is overwritten by U * Q.
// *>          Not referenced if NRU = 0.
// *> \endverbatim
// *>
// *> \param[in] LDU
// *> \verbatim
// *>          LDU is INTEGER
// *>          The leading dimension of the array U.  LDU >= max(1,NRU).
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension (LDC, NCC)
// *>          On entry, an N-by-NCC matrix C.
// *>          On exit, C is overwritten by Q**T * C.
// *>          Not referenced if NCC = 0.
// *> \endverbatim
// *>
// *> \param[in] LDC
// *> \verbatim
// *>          LDC is INTEGER
// *>          The leading dimension of the array C.
// *>          LDC >= max(1,N) if NCC > 0; LDC >=1 if NCC = 0.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (4*(N-1))
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  If INFO = -i, the i-th argument had an illegal value
// *>          > 0:
// *>             if NCVT = NRU = NCC = 0,
// *>                = 1, a split was marked by a positive value in E
// *>                = 2, current block of Z not diagonalized after 30*N
// *>                     iterations (in inner while loop)
// *>                = 3, termination criterion of outer while loop not met
// *>                     (program created more than N unreduced blocks)
// *>             else NCVT = NRU = NCC = 0,
// *>                   the algorithm did not converge; D and E contain the
// *>                   elements of a bidiagonal matrix which is orthogonal
// *>                   similar to the input matrix B;  if INFO = i, i
// *>                   elements of E have not converged to zero.
// *> \endverbatim
// *
// *> \par Internal Parameters:
// *  =========================
// *>
// *> \verbatim
// *>  TOLMUL  DOUBLE PRECISION, default = max(10,min(100,EPS**(-1/8)))
// *>          TOLMUL controls the convergence criterion of the QR loop.
// *>          If it is positive, TOLMUL*EPS is the desired relative
// *>             precision in the computed singular values.
// *>          If it is negative, abs(TOLMUL*EPS*sigma_max) is the
// *>             desired absolute accuracy in the computed singular
// *>             values (corresponds to relative accuracy
// *>             abs(TOLMUL*EPS) in the largest singular value.
// *>          abs(TOLMUL) should be between 1 and 1/EPS, and preferably
// *>             between 10 (for fast convergence) and .1/EPS
// *>             (for there to be some accuracy in the results).
// *>          Default is to lose at either one eighth or 2 of the
// *>             available decimal digits in each computed singular value

// *>             (whichever is smaller).
// *>
// *>  MAXITR  INTEGER, default = 6
// *>          MAXITR controls the maximum number of passes of the
// *>          algorithm through its inner loop. The algorithms stops
// *>          (and so fails to converge) if the number of passes
// *>          through the inner loop exceeds MAXITR*N**2.
// *>
// *> \endverbatim
// *
// *> \par Note:
// *  ===========
// *>
// *> \verbatim
// *>  Bug report from Cezary Dendek.
// *>  On March 23rd 2017, the INTEGER variable MAXIT = MAXITR*N**2 is
// *>  removed since it can overflow pretty easily (for N larger or equal
// *>  than 18,919). We instead use MAXITDIVN = MAXITR*N.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dbdsqr (String uplo,
int n,
int ncvt,
int nru,
int ncc,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] vt, int _vt_offset,
int ldvt,
double [] u, int _u_offset,
int ldu,
double [] c, int _c_offset,
int ldc,
double [] work, int _work_offset,
intW info)  {

label_999999:{

boolean lower= false;
boolean rotate= false;
int i= 0;
int idir= 0;
int isub= 0;
int iter= 0;
int iterdivn= 0;
int j= 0;
int ll= 0;
int lll= 0;
int m= 0;
int maxitdivn= 0;
int nm1= 0;
int nm12= 0;
int nm13= 0;
int oldll= 0;
int oldm= 0;
double abse= 0.0d;
double abss= 0.0d;
doubleW cosl= new doubleW(0.0d);
doubleW cosr= new doubleW(0.0d);
doubleW cs= new doubleW(0.0d);
double eps= 0.0d;
double f= 0.0d;
double g= 0.0d;
double h= 0.0d;
double mu= 0.0d;
doubleW oldcs= new doubleW(0.0d);
doubleW oldsn= new doubleW(0.0d);
doubleW r= new doubleW(0.0d);
doubleW shift= new doubleW(0.0d);
doubleW sigmn= new doubleW(0.0d);
doubleW sigmx= new doubleW(0.0d);
doubleW sinl= new doubleW(0.0d);
doubleW sinr= new doubleW(0.0d);
double sll= 0.0d;
double smax= 0.0d;
double smin= 0.0d;
double sminl= 0.0d;
double sminoa= 0.0d;
doubleW sn= new doubleW(0.0d);
double thresh= 0.0d;
double tol= 0.0d;
double tolmul= 0.0d;
double unfl= 0.0d;
uplo = uplo.substring((1)-1,1);
info.val = 0;
lower = com.polytechnik.lapack310.Lsame.lsame(uplo,"L");
if (((!com.polytechnik.lapack310.Lsame.lsame(uplo,"U")) && (!lower)))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((ncvt < 0))  {
    info.val = -3;
}              // Close else if()
else if ((nru < 0))  {
    info.val = -4;
}              // Close else if()
else if ((ncc < 0))  {
    info.val = -5;
}              // Close else if()
else if (((((ncvt == 0) && (ldvt < 1))) || (((ncvt > 0) && (ldvt < Math.max(1, n))))))  {
    info.val = -9;
}              // Close else if()
else if ((ldu < Math.max(1, nru)))  {
    info.val = -11;
}              // Close else if()
else if (((((ncc == 0) && (ldc < 1))) || (((ncc > 0) && (ldc < Math.max(1, n))))))  {
    info.val = -13;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DBDSQR",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",999999);
}
if ((n == 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",999999);
}
    /***
*
*     ROTATE is true if any singular vectors desired, false otherwise
*
***/
label200:{
label160:{
if ((n == 1)) {
    break label160;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",160);
}
    /***
*
*     If no singular vectors desired, use qd algorithm
*
***/
rotate = ((((ncvt > 0)) || ((nru > 0))) || ((ncc > 0)));
if ((!rotate))  {
    /***
*
*     If INFO equals 2, dqds didn't finish, try to finish
*
***/
com.polytechnik.lapack310.Dlasq1.dlasq1(n,d,_d_offset,e,_e_offset,work,_work_offset,info);
if ((info.val != 2)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",999999);
}
    info.val = 0;
// *
}
nm1 = (n-1);
nm12 = (nm1+nm1);
nm13 = (nm12+nm1);
/***
*
*     Get machine constants
*
***/
idir = 0;
eps = com.polytechnik.lapack310.Dlamch.dlamch("Epsilon");
/***
*
*     If matrix lower bidiagonal, rotate to be upper bidiagonal
*     by applying Givens rotations on the left
*
***/
unfl = com.polytechnik.lapack310.Dlamch.dlamch("Safe minimum");
if (lower)  {
    {
for (i = 1; i <= (n-1); i++) {
com.polytechnik.lapack310.Dlartg.dlartg(d[(i-(1))+ _d_offset],e[(i-(1))+ _e_offset],cs,sn,r);
d[(i-(1))+ _d_offset] = r.val;
e[(i-(1))+ _e_offset] = (sn.val*d[((i+1)-(1))+ _d_offset]);
d[((i+1)-(1))+ _d_offset] = (cs.val*d[((i+1)-(1))+ _d_offset]);
work[(i-(1))+ _work_offset] = cs.val;
work[((nm1+i)-(1))+ _work_offset] = sn.val;
/***
*
*        Update singular vectors if desired
*
***/
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",10);
}              //  Close for() loop.
}
if ((nru > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("R","V","F",nru,n,work,(1-(1))+ _work_offset,work,(n-(1))+ _work_offset,u,_u_offset,ldu);
}
    if ((ncc > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("L","V","F",n,ncc,work,(1-(1))+ _work_offset,work,(n-(1))+ _work_offset,c,_c_offset,ldc);
}
    /***
*
*     Compute singular values to relative accuracy TOL
*     (By setting TOL to be negative, algorithm will compute
*     singular values to absolute accuracy ABS(TOL)*norm(input matrix))
*
***/
}
tolmul = Math.max(10.0, Math.min(100.0, ( Math.pow(eps, -0.125))));
/***
*
*     Compute approximate maximum, minimum singular values
*
***/
tol = (tolmul*eps);
smax = 0.0;
{
for (i = 1; i <= n; i++) {
smax = Math.max(smax, Math.abs(d[(i-(1))+ _d_offset]));
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",20);
}              //  Close for() loop.
}
{
for (i = 1; i <= (n-1); i++) {
smax = Math.max(smax, Math.abs(e[(i-(1))+ _e_offset]));
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",30);
}              //  Close for() loop.
}
sminl = 0.0;
/***
*
*        Relative accuracy desired
*
***/
if ((tol >= 0.0))  {
    sminoa = Math.abs(d[(1-(1))+ _d_offset]);
if ((sminoa == 0.0)) {
    //Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",50);
} else /* label50 */ {
    mu = sminoa;
{
label40:for (i = 2; i <= n; i++) {
mu = (Math.abs(d[(i-(1))+ _d_offset])*((mu/((mu+Math.abs(e[((i-1)-(1))+ _e_offset]))))));
sminoa = Math.min(sminoa, mu);
if ((sminoa == 0.0)) {
    break label40;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",50);
}
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",40);
}              //  Close for() loop.
}
}// END label50:
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",50);
sminoa = (sminoa/Math.sqrt((double)(n)));
thresh = Math.max((tol*sminoa), (6*((n*((n*unfl))))));
/***
*
*        Absolute accuracy desired
*
***/
}
else  {
  thresh = Math.max((Math.abs(tol)*smax), (6*((n*((n*unfl))))));
/***
*
*     Prepare for main iteration loop for the singular values
*     (MAXIT is the maximum number of passes through the inner
*     loop permitted before nonconvergence signalled.)
*
***/
}              //  Close else.
maxitdivn = (6*n);
iterdivn = 0;
iter = -1;
oldll = -1;
/***
*
*     M points to last element of unconverged part of matrix
*
***/
oldm = -1;
/***
*
*     Begin main iteration loop
*
***/
m = n;
/***
*
*     Check for convergence or exceeding iteration count
*
***/
label60:for(;;){
    //Dummy.label("com/polytechnik/lapack310/Dbdsqr",60);
// *
if ((m <= 1)) {
    break label160;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",160);
}
    if ((iter >= n))  {
    iter = (iter-n);
iterdivn = (iterdivn+1);
if ((iterdivn >= maxitdivn)) {
    break label200;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",200);
}
    /***
*
*     Find diagonal block of matrix to work on
*
***/
}
if (((tol < 0.0) && (Math.abs(d[(m-(1))+ _d_offset]) <= thresh))) {
    d[(m-(1))+ _d_offset] = 0.0;
}
    smax = Math.abs(d[(m-(1))+ _d_offset]);
smin = smax;
label90:{
label80:{
{
for (lll = 1; lll <= (m-1); lll++) {
ll = (m-lll);
abss = Math.abs(d[(ll-(1))+ _d_offset]);
abse = Math.abs(e[(ll-(1))+ _e_offset]);
if (((tol < 0.0) && (abss <= thresh))) {
    d[(ll-(1))+ _d_offset] = 0.0;
}
    if ((abse <= thresh)) {
	break label80;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",80);
}
    smin = Math.min(smin, abss);
smax = Util.max(smax, abss, abse);
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",70);
}              //  Close for() loop.
}
ll = 0;
break label90;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",90);
}// END label80:
    //Dummy.label("com/polytechnik/lapack310/Dbdsqr",80);
/***
*
*     Matrix splits since E(LL) = 0
*
***/
e[(ll-(1))+ _e_offset] = 0.0;
/***
*
*        Convergence of bottom singular value, return to top of loop
*
***/
if ((ll == (m-1)))  {
    m = (m-1);
continue label60;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",60);
}
}// END label90:
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",90);
/***
*
*     E(LL) through E(M-1) are nonzero, E(LL-1) is zero
*
***/
ll = (ll+1);
/***
*
*        2 by 2 block, handle separately
*
***/
if ((ll == (m-1)))  {
    com.polytechnik.lapack310.Dlasv2.dlasv2(d[((m-1)-(1))+ _d_offset],e[((m-1)-(1))+ _e_offset],d[(m-(1))+ _d_offset],sigmn,sigmx,sinr,cosr,sinl,cosl);
d[((m-1)-(1))+ _d_offset] = sigmx.val;
e[((m-1)-(1))+ _e_offset] = 0.0;
/***
*
*        Compute singular vectors, if desired
*
***/
d[(m-(1))+ _d_offset] = sigmn.val;
if ((ncvt > 0)) {
    com.polytechnik.lapack310.Drot.drot(ncvt,vt,((m-1)-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt,vt,(m-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt,cosr.val,sinr.val);
}
    if ((nru > 0)) {
    com.polytechnik.lapack310.Drot.drot(nru,u,(1-(1))+((m-1)-(1)) * (ldu)+ _u_offset,1,u,(1-(1))+(m-(1)) * (ldu)+ _u_offset,1,cosl.val,sinl.val);
}
    if ((ncc > 0)) {
    com.polytechnik.lapack310.Drot.drot(ncc,c,((m-1)-(1))+(1-(1)) * (ldc)+ _c_offset,ldc,c,(m-(1))+(1-(1)) * (ldc)+ _c_offset,ldc,cosl.val,sinl.val);
}
    m = (m-2);
continue label60;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",60);
/***
*
*     If working on new submatrix, choose shift direction
*     (from larger end diagonal element towards smaller)
*
***/
}
if (((ll > oldm) || (m < oldll)))  {
    /***
*
*           Chase bulge from top (big end) to bottom (small end)
*
***/
if ((Math.abs(d[(ll-(1))+ _d_offset]) >= Math.abs(d[(m-(1))+ _d_offset])))  {
    idir = 1;
/***
*
*           Chase bulge from bottom (big end) to top (small end)
*
***/
}
else  {
  idir = 2;
}              //  Close else.
/***
*
*     Apply convergence tests
*
***/
}
/***
*
*        Run convergence test in forward direction
*        First apply standard test to bottom of matrix
*
***/
if ((idir == 1))  {
    if (((Math.abs(e[((m-1)-(1))+ _e_offset]) <= (Math.abs(tol)*Math.abs(d[(m-(1))+ _d_offset]))) || (((tol < 0.0) && (Math.abs(e[((m-1)-(1))+ _e_offset]) <= thresh)))))  {
    e[((m-1)-(1))+ _e_offset] = 0.0;
continue label60;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",60);
// *
}
/***
*
*           If relative accuracy desired,
*           apply convergence criterion forward
*
***/
if ((tol >= 0.0))  {
    mu = Math.abs(d[(ll-(1))+ _d_offset]);
sminl = mu;
{
for (lll = ll; lll <= (m-1); lll++) {
if ((Math.abs(e[(lll-(1))+ _e_offset]) <= (tol*mu)))  {
    e[(lll-(1))+ _e_offset] = 0.0;
continue label60;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",60);
}
mu = (Math.abs(d[((lll+1)-(1))+ _d_offset])*((mu/((mu+Math.abs(e[(lll-(1))+ _e_offset]))))));
sminl = Math.min(sminl, mu);
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",100);
}              //  Close for() loop.
}
// *
}
/***
*
*        Run convergence test in backward direction
*        First apply standard test to top of matrix
*
***/
}
else  {
  if (((Math.abs(e[(ll-(1))+ _e_offset]) <= (Math.abs(tol)*Math.abs(d[(ll-(1))+ _d_offset]))) || (((tol < 0.0) && (Math.abs(e[(ll-(1))+ _e_offset]) <= thresh)))))  {
    e[(ll-(1))+ _e_offset] = 0.0;
continue label60;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",60);
// *
}
/***
*
*           If relative accuracy desired,
*           apply convergence criterion backward
*
***/
if ((tol >= 0.0))  {
    mu = Math.abs(d[(m-(1))+ _d_offset]);
sminl = mu;
{
int _lll_inc = -1;
for (lll = (m-1); lll >= ll; lll += _lll_inc) {
if ((Math.abs(e[(lll-(1))+ _e_offset]) <= (tol*mu)))  {
    e[(lll-(1))+ _e_offset] = 0.0;
continue label60;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",60);
}
mu = (Math.abs(d[(lll-(1))+ _d_offset])*((mu/((mu+Math.abs(e[(lll-(1))+ _e_offset]))))));
sminl = Math.min(sminl, mu);
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",110);
}              //  Close for() loop.
}
}
}              //  Close else.
oldll = ll;
/***
*
*     Compute shift.  First, test if shifting would ruin relative
*     accuracy, and if so set the shift to zero.
*
***/
oldm = m;
/***
*
*        Use a zero shift to avoid loss of relative accuracy
*
***/
if (((tol >= 0.0) && (((n*tol)*((sminl/smax))) <= Math.max(eps, (0.01000000000000000020816681711721685132943*tol)))))  {
    shift.val = 0.0;
/***
*
*        Compute the shift from 2-by-2 block at end of matrix
*
***/
}
else  {
  if ((idir == 1))  {
    sll = Math.abs(d[(ll-(1))+ _d_offset]);
com.polytechnik.lapack310.Dlas2.dlas2(d[((m-1)-(1))+ _d_offset],e[((m-1)-(1))+ _e_offset],d[(m-(1))+ _d_offset],shift,r);
}
else  {
  sll = Math.abs(d[(m-(1))+ _d_offset]);
com.polytechnik.lapack310.Dlas2.dlas2(d[(ll-(1))+ _d_offset],e[(ll-(1))+ _e_offset],d[((ll+1)-(1))+ _d_offset],shift,r);
/***
*
*        Test if shift negligible, and if so set to zero
*
***/
}              //  Close else.
if ((sll > 0.0))  {
    if ((( Math.pow(((shift.val/sll)), 2)) < eps)) {
    shift.val = 0.0;
}
    }
/***
*
*     Increment iteration count
*
***/
}              //  Close else.
/***
*
*     If SHIFT = 0, do simplified QR iteration
*
***/
iter = ((iter+m)-ll);
if ((shift.val == 0.0))  {
    /***
*
*           Chase bulge from top to bottom
*           Save cosines and sines for later singular vector updates
*
***/
if ((idir == 1))  {
    cs.val = 1.0;
oldcs.val = 1.0;
{
for (i = ll; i <= (m-1); i++) {
com.polytechnik.lapack310.Dlartg.dlartg((d[(i-(1))+ _d_offset]*cs.val),e[(i-(1))+ _e_offset],cs,sn,r);
if ((i > ll)) {
    e[((i-1)-(1))+ _e_offset] = (oldsn.val*r.val);
}
    dlartg_adapter((oldcs.val*r.val),(d[((i+1)-(1))+ _d_offset]*sn.val),oldcs,oldsn,d,(i-(1))+ _d_offset);
work[(((i-ll)+1)-(1))+ _work_offset] = cs.val;
work[((((i-ll)+1)+nm1)-(1))+ _work_offset] = sn.val;
work[((((i-ll)+1)+nm12)-(1))+ _work_offset] = oldcs.val;
work[((((i-ll)+1)+nm13)-(1))+ _work_offset] = oldsn.val;
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",120);
}              //  Close for() loop.
}
h = (d[(m-(1))+ _d_offset]*cs.val);
d[(m-(1))+ _d_offset] = (h*oldcs.val);
/***
*
*           Update singular vectors
*
***/
e[((m-1)-(1))+ _e_offset] = (h*oldsn.val);
if ((ncvt > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("L","V","F",((m-ll)+1),ncvt,work,(1-(1))+ _work_offset,work,(n-(1))+ _work_offset,vt,(ll-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt);
}
    if ((nru > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("R","V","F",nru,((m-ll)+1),work,((nm12+1)-(1))+ _work_offset,work,((nm13+1)-(1))+ _work_offset,u,(1-(1))+(ll-(1)) * (ldu)+ _u_offset,ldu);
}
    /***
*
*           Test convergence
*
***/
if ((ncc > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("L","V","F",((m-ll)+1),ncc,work,((nm12+1)-(1))+ _work_offset,work,((nm13+1)-(1))+ _work_offset,c,(ll-(1))+(1-(1)) * (ldc)+ _c_offset,ldc);
}
    // *
if ((Math.abs(e[((m-1)-(1))+ _e_offset]) <= thresh)) {
    e[((m-1)-(1))+ _e_offset] = 0.0;
}
    /***
*
*           Chase bulge from bottom to top
*           Save cosines and sines for later singular vector updates
*
***/
}
else  {
  cs.val = 1.0;
oldcs.val = 1.0;
{
int _i_inc = -1;
for (i = m; i >= (ll+1); i += _i_inc) {
com.polytechnik.lapack310.Dlartg.dlartg((d[(i-(1))+ _d_offset]*cs.val),e[((i-1)-(1))+ _e_offset],cs,sn,r);
if ((i < m)) {
    e[(i-(1))+ _e_offset] = (oldsn.val*r.val);
}
    dlartg_adapter((oldcs.val*r.val),(d[((i-1)-(1))+ _d_offset]*sn.val),oldcs,oldsn,d,(i-(1))+ _d_offset);
work[((i-ll)-(1))+ _work_offset] = cs.val;
work[(((i-ll)+nm1)-(1))+ _work_offset] = (-(sn.val));
work[(((i-ll)+nm12)-(1))+ _work_offset] = oldcs.val;
work[(((i-ll)+nm13)-(1))+ _work_offset] = (-(oldsn.val));
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",130);
}              //  Close for() loop.
}
h = (d[(ll-(1))+ _d_offset]*cs.val);
d[(ll-(1))+ _d_offset] = (h*oldcs.val);
/***
*
*           Update singular vectors
*
***/
e[(ll-(1))+ _e_offset] = (h*oldsn.val);
if ((ncvt > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("L","V","B",((m-ll)+1),ncvt,work,((nm12+1)-(1))+ _work_offset,work,((nm13+1)-(1))+ _work_offset,vt,(ll-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt);
}
    if ((nru > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("R","V","B",nru,((m-ll)+1),work,(1-(1))+ _work_offset,work,(n-(1))+ _work_offset,u,(1-(1))+(ll-(1)) * (ldu)+ _u_offset,ldu);
}
    /***
*
*           Test convergence
*
***/
if ((ncc > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("L","V","B",((m-ll)+1),ncc,work,(1-(1))+ _work_offset,work,(n-(1))+ _work_offset,c,(ll-(1))+(1-(1)) * (ldc)+ _c_offset,ldc);
}
    if ((Math.abs(e[(ll-(1))+ _e_offset]) <= thresh)) {
    e[(ll-(1))+ _e_offset] = 0.0;
}
    }              //  Close else.
/***
*
*        Use nonzero shift
*
***/
}
else  {
  /***
*
*           Chase bulge from top to bottom
*           Save cosines and sines for later singular vector updates
*
***/
if ((idir == 1))  {
    f = (((Math.abs(d[(ll-(1))+ _d_offset])-shift.val))*((Util.dsign(1.0,d[(ll-(1))+ _d_offset])+(shift.val/d[(ll-(1))+ _d_offset]))));
g = e[(ll-(1))+ _e_offset];
{
for (i = ll; i <= (m-1); i++) {
com.polytechnik.lapack310.Dlartg.dlartg(f,g,cosr,sinr,r);
if ((i > ll)) {
    e[((i-1)-(1))+ _e_offset] = r.val;
}
    f = ((cosr.val*d[(i-(1))+ _d_offset])+(sinr.val*e[(i-(1))+ _e_offset]));
e[(i-(1))+ _e_offset] = ((cosr.val*e[(i-(1))+ _e_offset])-(sinr.val*d[(i-(1))+ _d_offset]));
g = (sinr.val*d[((i+1)-(1))+ _d_offset]);
d[((i+1)-(1))+ _d_offset] = (cosr.val*d[((i+1)-(1))+ _d_offset]);
com.polytechnik.lapack310.Dlartg.dlartg(f,g,cosl,sinl,r);
d[(i-(1))+ _d_offset] = r.val;
f = ((cosl.val*e[(i-(1))+ _e_offset])+(sinl.val*d[((i+1)-(1))+ _d_offset]));
d[((i+1)-(1))+ _d_offset] = ((cosl.val*d[((i+1)-(1))+ _d_offset])-(sinl.val*e[(i-(1))+ _e_offset]));
if ((i < (m-1)))  {
    g = (sinl.val*e[((i+1)-(1))+ _e_offset]);
e[((i+1)-(1))+ _e_offset] = (cosl.val*e[((i+1)-(1))+ _e_offset]);
}
work[(((i-ll)+1)-(1))+ _work_offset] = cosr.val;
work[((((i-ll)+1)+nm1)-(1))+ _work_offset] = sinr.val;
work[((((i-ll)+1)+nm12)-(1))+ _work_offset] = cosl.val;
work[((((i-ll)+1)+nm13)-(1))+ _work_offset] = sinl.val;
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",140);
}              //  Close for() loop.
}
/***
*
*           Update singular vectors
*
***/
e[((m-1)-(1))+ _e_offset] = f;
if ((ncvt > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("L","V","F",((m-ll)+1),ncvt,work,(1-(1))+ _work_offset,work,(n-(1))+ _work_offset,vt,(ll-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt);
}
    if ((nru > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("R","V","F",nru,((m-ll)+1),work,((nm12+1)-(1))+ _work_offset,work,((nm13+1)-(1))+ _work_offset,u,(1-(1))+(ll-(1)) * (ldu)+ _u_offset,ldu);
}
    /***
*
*           Test convergence
*
***/
if ((ncc > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("L","V","F",((m-ll)+1),ncc,work,((nm12+1)-(1))+ _work_offset,work,((nm13+1)-(1))+ _work_offset,c,(ll-(1))+(1-(1)) * (ldc)+ _c_offset,ldc);
}
    // *
if ((Math.abs(e[((m-1)-(1))+ _e_offset]) <= thresh)) {
    e[((m-1)-(1))+ _e_offset] = 0.0;
}
    /***
*
*           Chase bulge from bottom to top
*           Save cosines and sines for later singular vector updates
*
***/
}
else  {
  f = (((Math.abs(d[(m-(1))+ _d_offset])-shift.val))*((Util.dsign(1.0,d[(m-(1))+ _d_offset])+(shift.val/d[(m-(1))+ _d_offset]))));
g = e[((m-1)-(1))+ _e_offset];
{
int _i_inc = -1;
for (i = m; i >= (ll+1); i += _i_inc) {
com.polytechnik.lapack310.Dlartg.dlartg(f,g,cosr,sinr,r);
if ((i < m)) {
    e[(i-(1))+ _e_offset] = r.val;
}
    f = ((cosr.val*d[(i-(1))+ _d_offset])+(sinr.val*e[((i-1)-(1))+ _e_offset]));
e[((i-1)-(1))+ _e_offset] = ((cosr.val*e[((i-1)-(1))+ _e_offset])-(sinr.val*d[(i-(1))+ _d_offset]));
g = (sinr.val*d[((i-1)-(1))+ _d_offset]);
d[((i-1)-(1))+ _d_offset] = (cosr.val*d[((i-1)-(1))+ _d_offset]);
com.polytechnik.lapack310.Dlartg.dlartg(f,g,cosl,sinl,r);
d[(i-(1))+ _d_offset] = r.val;
f = ((cosl.val*e[((i-1)-(1))+ _e_offset])+(sinl.val*d[((i-1)-(1))+ _d_offset]));
d[((i-1)-(1))+ _d_offset] = ((cosl.val*d[((i-1)-(1))+ _d_offset])-(sinl.val*e[((i-1)-(1))+ _e_offset]));
if ((i > (ll+1)))  {
    g = (sinl.val*e[((i-2)-(1))+ _e_offset]);
e[((i-2)-(1))+ _e_offset] = (cosl.val*e[((i-2)-(1))+ _e_offset]);
}
work[((i-ll)-(1))+ _work_offset] = cosr.val;
work[(((i-ll)+nm1)-(1))+ _work_offset] = (-(sinr.val));
work[(((i-ll)+nm12)-(1))+ _work_offset] = cosl.val;
work[(((i-ll)+nm13)-(1))+ _work_offset] = (-(sinl.val));
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",150);
}              //  Close for() loop.
}
/***
*
*           Test convergence
*
***/
e[(ll-(1))+ _e_offset] = f;
/***
*
*           Update singular vectors if desired
*
***/
if ((Math.abs(e[(ll-(1))+ _e_offset]) <= thresh)) {
    e[(ll-(1))+ _e_offset] = 0.0;
}
    if ((ncvt > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("L","V","B",((m-ll)+1),ncvt,work,((nm12+1)-(1))+ _work_offset,work,((nm13+1)-(1))+ _work_offset,vt,(ll-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt);
}
    if ((nru > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("R","V","B",nru,((m-ll)+1),work,(1-(1))+ _work_offset,work,(n-(1))+ _work_offset,u,(1-(1))+(ll-(1)) * (ldu)+ _u_offset,ldu);
}
    if ((ncc > 0)) {
    com.polytechnik.lapack310.Dlasr.dlasr("L","V","B",((m-ll)+1),ncc,work,(1-(1))+ _work_offset,work,(n-(1))+ _work_offset,c,(ll-(1))+(1-(1)) * (ldc)+ _c_offset,ldc);
}
    }              //  Close else.
/***
*
*     QR iteration finished, go back and check convergence
*
***/
}              //  Close else.
/***
*
*     All singular values converged, so make them positive
*
***/
continue label60;}// END label60
//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",60);
}// END label160
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",160);
{
for (i = 1; i <= n; i++) {
if ((d[(i-(1))+ _d_offset] < 0.0))  {
    /***
*
*           Change sign of singular vectors, if desired
*
***/
d[(i-(1))+ _d_offset] = (-(d[(i-(1))+ _d_offset]));
if ((ncvt > 0)) {
    com.polytechnik.lapack310.Dscal.dscal(ncvt,-1.0,vt,(i-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt);
}
    }
/***
*
*     Sort the singular values into decreasing order (insertion sort on
*     singular values, but only one transposition per singular vector)
*
***/
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",170);
}              //  Close for() loop.
}
/***
*
*        Scan for smallest D(I)
*
***/
{
for (i = 1; i <= (n-1); i++) {
isub = 1;
smin = d[(1-(1))+ _d_offset];
{
for (j = 2; j <= ((n+1)-i); j++) {
if ((d[(j-(1))+ _d_offset] <= smin))  {
    isub = j;
smin = d[(j-(1))+ _d_offset];
}
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",180);
}              //  Close for() loop.
}
/***
*
*           Swap singular values and vectors
*
***/
if ((isub != ((n+1)-i)))  {
    d[(isub-(1))+ _d_offset] = d[(((n+1)-i)-(1))+ _d_offset];
d[(((n+1)-i)-(1))+ _d_offset] = smin;
if ((ncvt > 0)) {
    com.polytechnik.lapack310.Dswap.dswap(ncvt,vt,(isub-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt,vt,(((n+1)-i)-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt);
}
    if ((nru > 0)) {
    com.polytechnik.lapack310.Dswap.dswap(nru,u,(1-(1))+(isub-(1)) * (ldu)+ _u_offset,1,u,(1-(1))+(((n+1)-i)-(1)) * (ldu)+ _u_offset,1);
}
    if ((ncc > 0)) {
    com.polytechnik.lapack310.Dswap.dswap(ncc,c,(isub-(1))+(1-(1)) * (ldc)+ _c_offset,ldc,c,(((n+1)-i)-(1))+(1-(1)) * (ldc)+ _c_offset,ldc);
}
    }
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",190);
}              //  Close for() loop.
}
/***
*
*     Maximum number of iterations exceeded, failure to converge
*
***/
break label_999999;
//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",220);
}// END label200:
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",200);
info.val = 0;
{
for (i = 1; i <= (n-1); i++) {
if ((e[(i-(1))+ _e_offset] != 0.0)) {
    info.val = (info.val+1);
}
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",210);
}              //  Close for() loop.
}
//label220:
//Dummy.label("com/polytechnik/lapack310/Dbdsqr",220);
/***
*
*     End of DBDSQR
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dbdsqr",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dbdsqr",999999);
return;
   }
// adapter for dlartg(DDLorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;Lorg/netlib/util/doubleW;)V
private static void dlartg_adapter(double arg0 ,double arg1 ,doubleW arg2 ,doubleW arg3 ,double [] arg4 , int arg4_offset )
{
doubleW _f2j_tmp4 = new doubleW(arg4[arg4_offset]);

com.polytechnik.lapack310.Dlartg.dlartg(arg0,arg1,arg2,arg3,_f2j_tmp4);

arg4[arg4_offset] = _f2j_tmp4.val;
}

} // End class.
