/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgebrd.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dgebrd {

// *> \brief \b DGEBRD
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DGEBRD + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGEBRD( M, N, A, LDA, D, E, TAUQ, TAUP, WORK, LWORK,

// *                          INFO )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, LDA, LWORK, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), D( * ), E( * ), TAUP( * ),
// *      $                   TAUQ( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGEBRD reduces a general real M-by-N matrix A to upper or lower
// *> bidiagonal form B by an orthogonal transformation: Q**T * A * P = B.

// *>
// *> If m >= n, B is upper bidiagonal; if m < n, B is lower bidiagonal.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows in the matrix A.  M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns in the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the M-by-N general matrix to be reduced.
// *>          On exit,
// *>          if m >= n, the diagonal and the first superdiagonal are
// *>            overwritten with the upper bidiagonal matrix B; the
// *>            elements below the diagonal, with the array TAUQ, represen
// *>            the orthogonal matrix Q as a product of elementary
// *>            reflectors, and the elements above the first superdiagonal
// *>            with the array TAUP, represent the orthogonal matrix P as

// *>            a product of elementary reflectors;
// *>          if m < n, the diagonal and the first subdiagonal are
// *>            overwritten with the lower bidiagonal matrix B; the
// *>            elements below the first subdiagonal, with the array TAUQ,
// *>            represent the orthogonal matrix Q as a product of
// *>            elementary reflectors, and the elements above the diagonal
// *>            with the array TAUP, represent the orthogonal matrix P as

// *>            a product of elementary reflectors.
// *>          See Further Details.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,M).
// *> \endverbatim
// *>
// *> \param[out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (min(M,N))
// *>          The diagonal elements of the bidiagonal matrix B:
// *>          D(i) = A(i,i).
// *> \endverbatim
// *>
// *> \param[out] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (min(M,N)-1)
// *>          The off-diagonal elements of the bidiagonal matrix B:
// *>          if m >= n, E(i) = A(i,i+1) for i = 1,2,...,n-1;
// *>          if m < n, E(i) = A(i+1,i) for i = 1,2,...,m-1.
// *> \endverbatim
// *>
// *> \param[out] TAUQ
// *> \verbatim
// *>          TAUQ is DOUBLE PRECISION array, dimension (min(M,N))
// *>          The scalar factors of the elementary reflectors which
// *>          represent the orthogonal matrix Q. See Further Details.
// *> \endverbatim
// *>
// *> \param[out] TAUP
// *> \verbatim
// *>          TAUP is DOUBLE PRECISION array, dimension (min(M,N))
// *>          The scalar factors of the elementary reflectors which
// *>          represent the orthogonal matrix P. See Further Details.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The length of the array WORK.  LWORK >= max(1,M,N).
// *>          For optimum performance LWORK >= (M+N)*NB, where NB
// *>          is the optimal blocksize.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value.

// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleGEcomputational
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  The matrices Q and P are represented as products of elementary
// *>  reflectors:
// *>
// *>  If m >= n,
// *>
// *>     Q = H(1) H(2) . . . H(n)  and  P = G(1) G(2) . . . G(n-1)
// *>
// *>  Each H(i) and G(i) has the form:
// *>
// *>     H(i) = I - tauq * v * v**T  and G(i) = I - taup * u * u**T
// *>
// *>  where tauq and taup are real scalars, and v and u are real vectors;

// *>  v(1:i-1) = 0, v(i) = 1, and v(i+1:m) is stored on exit in A(i+1:m,i)
// *>  u(1:i) = 0, u(i+1) = 1, and u(i+2:n) is stored on exit in A(i,i+2:n)
// *>  tauq is stored in TAUQ(i) and taup in TAUP(i).
// *>
// *>  If m < n,
// *>
// *>     Q = H(1) H(2) . . . H(m-1)  and  P = G(1) G(2) . . . G(m)
// *>
// *>  Each H(i) and G(i) has the form:
// *>
// *>     H(i) = I - tauq * v * v**T  and G(i) = I - taup * u * u**T
// *>
// *>  where tauq and taup are real scalars, and v and u are real vectors;

// *>  v(1:i) = 0, v(i+1) = 1, and v(i+2:m) is stored on exit in A(i+2:m,i)
// *>  u(1:i-1) = 0, u(i) = 1, and u(i+1:n) is stored on exit in A(i,i+1:n)
// *>  tauq is stored in TAUQ(i) and taup in TAUP(i).
// *>
// *>  The contents of A on exit are illustrated by the following examples:
// *>
// *>  m = 6 and n = 5 (m > n):          m = 5 and n = 6 (m < n):
// *>
// *>    (  d   e   u1  u1  u1 )           (  d   u1  u1  u1  u1  u1 )
// *>    (  v1  d   e   u2  u2 )           (  e   d   u2  u2  u2  u2 )
// *>    (  v1  v2  d   e   u3 )           (  v1  e   d   u3  u3  u3 )
// *>    (  v1  v2  v3  d   e  )           (  v1  v2  e   d   u4  u4 )
// *>    (  v1  v2  v3  v4  d  )           (  v1  v2  v3  e   d   u5 )
// *>    (  v1  v2  v3  v4  v5 )
// *>
// *>  where d and e denote diagonal and off-diagonal elements of B, vi
// *>  denotes an element of the vector defining H(i), and ui an element of
// *>  the vector defining G(i).
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters
*
***/

public static void dgebrd (int m,
int n,
double [] a, int _a_offset,
int lda,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] tauq, int _tauq_offset,
double [] taup, int _taup_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

label_999999:{

boolean lquery= false;
int i= 0;
intW iinfo= new intW(0);
int j= 0;
int ldwrkx= 0;
int ldwrky= 0;
int lwkopt= 0;
int minmn= 0;
int nb= 0;
int nbmin= 0;
int nx= 0;
int ws= 0;
info.val = 0;
nb = Math.max(1, 1/*com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DGEBRD"," ",m,n,-1,-1)*/);
lwkopt = (((m+n))*nb);
work[(1-(1))+ _work_offset] = (double)(lwkopt);
lquery = ((lwork == -1));
if ((m < 0))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info.val = -4;
}              // Close else if()
else if (((lwork < Util.max(1, m, n)) && (!lquery)))  {
    info.val = -10;
}              // Close else if()
if ((info.val < 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DGEBRD",(-(info.val)));
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgebrd",999999);
}
else if (lquery)  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgebrd",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
minmn = Math.min(m, n);
if ((minmn == 0))  {
    work[(1-(1))+ _work_offset] = (double)(1);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgebrd",999999);
// *
}
ws = Math.max(m, n);
ldwrkx = m;
// *
ldwrky = n;
/***
*
*        Set the crossover point NX.
*
***/
if (((nb > 1) && (nb < minmn)))  {
    /***
*
*        Determine when to switch from blocked to unblocked code.
*
***/
    nx = Math.max(nb, 0/*com.polytechnik.lapack310.Ilaenv.ilaenv(3,"DGEBRD"," ",m,n,-1,-1)*/);
if ((nx < minmn))  {
    ws = (((m+n))*nb);
/***
*
*              Not enough work space for the optimal NB, consider using
*              a smaller block size.
*
***/
if ((lwork < ws))  {
    nbmin = 2;/*com.polytechnik.lapack310.Ilaenv.ilaenv(2,"DGEBRD"," ",m,n,-1,-1);*/
if ((lwork >= (((m+n))*nbmin)))  {
    nb = (lwork/((m+n)));
}
else  {
  nb = 1;
nx = minmn;
}              //  Close else.
}
}
}
else  {
  nx = minmn;
// *
}              //  Close else.
/***
*
*        Reduce rows and columns i:i+nb-1 to bidiagonal form and return
*        the matrices X and Y which are needed to update the unreduced
*        part of the matrix
*
***/
{
int _i_inc = nb;
for (i = 1; (_i_inc < 0) ? i >= (minmn-nx) : i <= (minmn-nx); i += _i_inc) {
/***
*
*        Update the trailing submatrix A(i+nb:m,i+nb:n), using an update
*        of the form  A := A - V*Y**T - X*U**T
*
***/
com.polytechnik.lapack310.Dlabrd.dlabrd(((m-i)+1),((n-i)+1),nb,a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,d,(i-(1))+ _d_offset,e,(i-(1))+ _e_offset,tauq,(i-(1))+ _tauq_offset,taup,(i-(1))+ _taup_offset,work,_work_offset,ldwrkx,work,(((ldwrkx*nb)+1)-(1))+ _work_offset,ldwrky);
com.polytechnik.lapack310.Dgemm.dgemm("No transpose","Transpose",(((m-i)-nb)+1),(((n-i)-nb)+1),nb,-1.0,a,((i+nb)-(1))+(i-(1)) * (lda)+ _a_offset,lda,work,((((ldwrkx*nb)+nb)+1)-(1))+ _work_offset,ldwrky,1.0,a,((i+nb)-(1))+((i+nb)-(1)) * (lda)+ _a_offset,lda);
/***
*
*        Copy diagonal and off-diagonal elements of B back into A
*
***/
com.polytechnik.lapack310.Dgemm.dgemm("No transpose","No transpose",(((m-i)-nb)+1),(((n-i)-nb)+1),nb,-1.0,work,((nb+1)-(1))+ _work_offset,ldwrkx,a,(i-(1))+((i+nb)-(1)) * (lda)+ _a_offset,lda,1.0,a,((i+nb)-(1))+((i+nb)-(1)) * (lda)+ _a_offset,lda);
if ((m >= n))  {
    {
for (j = i; j <= ((i+nb)-1); j++) {
a[(j-(1))+(j-(1)) * (lda)+ _a_offset] = d[(j-(1))+ _d_offset];
a[(j-(1))+((j+1)-(1)) * (lda)+ _a_offset] = e[(j-(1))+ _e_offset];
//Dummy.label("com/polytechnik/lapack310/Dgebrd",10);
}              //  Close for() loop.
}
}
else  {
  {
for (j = i; j <= ((i+nb)-1); j++) {
a[(j-(1))+(j-(1)) * (lda)+ _a_offset] = d[(j-(1))+ _d_offset];
a[((j+1)-(1))+(j-(1)) * (lda)+ _a_offset] = e[(j-(1))+ _e_offset];
//Dummy.label("com/polytechnik/lapack310/Dgebrd",20);
}              //  Close for() loop.
}
}              //  Close else.
/***
*
*     Use unblocked code to reduce the remainder of the matrix
*
***/
//Dummy.label("com/polytechnik/lapack310/Dgebrd",30);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dgebd2.dgebd2(((m-i)+1),((n-i)+1),a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,d,(i-(1))+ _d_offset,e,(i-(1))+ _e_offset,tauq,(i-(1))+ _tauq_offset,taup,(i-(1))+ _taup_offset,work,_work_offset,iinfo);
work[(1-(1))+ _work_offset] = (double)(ws);
/***
*
*     End of DGEBRD
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgebrd",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dgebrd",999999);
return;
   }
} // End class.
