/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgehrd.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dgehrd {

// *> \brief \b DGEHRD
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DGEHRD + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGEHRD( N, ILO, IHI, A, LDA, TAU, WORK, LWORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            IHI, ILO, INFO, LDA, LWORK, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION  A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGEHRD reduces a real general matrix A to upper Hessenberg form H by

// *> an orthogonal similarity transformation:  Q**T * A * Q = H .
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] ILO
// *> \verbatim
// *>          ILO is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IHI
// *> \verbatim
// *>          IHI is INTEGER
// *>
// *>          It is assumed that A is already upper triangular in rows
// *>          and columns 1:ILO-1 and IHI+1:N. ILO and IHI are normally
// *>          set by a previous call to DGEBAL; otherwise they should be
// *>          set to 1 and N respectively. See Further Details.
// *>          1 <= ILO <= IHI <= N, if N > 0; ILO=1 and IHI=0, if N=0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the N-by-N general matrix to be reduced.
// *>          On exit, the upper triangle and the first subdiagonal of A
// *>          are overwritten with the upper Hessenberg matrix H, and the

// *>          elements below the first subdiagonal, with the array TAU,
// *>          represent the orthogonal matrix Q as a product of elementary
// *>          reflectors. See Further Details.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (N-1)
// *>          The scalar factors of the elementary reflectors (see Further
// *>          Details). Elements 1:ILO-1 and IHI:N-1 of TAU are set to
// *>          zero.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (LWORK)
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The length of the array WORK.  LWORK >= max(1,N).
// *>          For good performance, LWORK should generally be larger.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value.

// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleGEcomputational
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  The matrix Q is represented as a product of (ihi-ilo) elementary
// *>  reflectors
// *>
// *>     Q = H(ilo) H(ilo+1) . . . H(ihi-1).
// *>
// *>  Each H(i) has the form
// *>
// *>     H(i) = I - tau * v * v**T
// *>
// *>  where tau is a real scalar, and v is a real vector with
// *>  v(1:i) = 0, v(i+1) = 1 and v(ihi+1:n) = 0; v(i+2:ihi) is stored on
// *>  exit in A(i+2:ihi,i), and tau in TAU(i).
// *>
// *>  The contents of A are illustrated by the following example, with
// *>  n = 7, ilo = 2 and ihi = 6:
// *>
// *>  on entry,                        on exit,
// *>
// *>  ( a   a   a   a   a   a   a )    (  a   a   h   h   h   h   a )
// *>  (     a   a   a   a   a   a )    (      a   h   h   h   h   a )
// *>  (     a   a   a   a   a   a )    (      h   h   h   h   h   h )
// *>  (     a   a   a   a   a   a )    (      v2  h   h   h   h   h )
// *>  (     a   a   a   a   a   a )    (      v2  v3  h   h   h   h )
// *>  (     a   a   a   a   a   a )    (      v2  v3  v4  h   h   h )
// *>  (                         a )    (                          a )
// *>
// *>  where a denotes an element of the original matrix A, h denotes a
// *>  modified element of the upper Hessenberg matrix H, and vi denotes an
// *>  element of the vector defining H(i).
// *>
// *>  This file is a slight modification of LAPACK-3.0's DGEHRD
// *>  subroutine incorporating improvements proposed by Quintana-Orti and

// *>  Van de Geijn (2006). (See DLAHR2.)
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters
*
***/

public static void dgehrd (int n,
int ilo,
int ihi,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

label_999999:{

boolean lquery= false;
int i= 0;
int ib= 0;
intW iinfo= new intW(0);
int iwt= 0;
int j= 0;
int ldwork= 0;
int lwkopt= 0;
int nb= 0;
int nbmin= 0;
int nh= 0;
int nx= 0;
double ei= 0.0d;
info.val = 0;
lquery = ((lwork == -1));
if ((n < 0))  {
    info.val = -1;
}
else if (((ilo < 1) || (ilo > Math.max(1, n))))  {
    info.val = -2;
}              // Close else if()
else if (((ihi < Math.min(ilo, n)) || (ihi > n)))  {
    info.val = -3;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -5;
}              // Close else if()
else if (((lwork < Math.max(1, n)) && (!lquery)))  {
    info.val = -8;
// *
}              // Close else if()
/***
*
*        Compute the workspace requirements
*
***/
if ((info.val == 0))  {
    nb = Math.min(64, 1/*com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DGEHRD"," ",n,ilo,ihi,-1)*/);
lwkopt = ((n*nb)+4160);
work[(1-(1))+ _work_offset] = (double)(lwkopt);
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DGEHRD",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgehrd",999999);
}
else if (lquery)  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgehrd",999999);
/***
*
*     Set elements 1:ILO-1 and IHI:N-1 of TAU to zero
*
***/
}              // Close else if()
{
for (i = 1; i <= (ilo-1); i++) {
tau[(i-(1))+ _tau_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dgehrd",10);
}              //  Close for() loop.
}
{
for (i = Math.max(1, ihi); i <= (n-1); i++) {
tau[(i-(1))+ _tau_offset] = 0.0;
/***
*
*     Quick return if possible
*
***/
//Dummy.label("com/polytechnik/lapack310/Dgehrd",20);
}              //  Close for() loop.
}
nh = ((ihi-ilo)+1);
if ((nh <= 1))  {
    work[(1-(1))+ _work_offset] = (double)(1);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgehrd",999999);
/***
*
*     Determine the block size
*
***/
}
nb = Math.min(64, 1/*com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DGEHRD"," ",n,ilo,ihi,-1)*/);
nbmin = 2;
/***
*
*        Determine when to cross over from blocked to unblocked code
*        (last block is always handled by unblocked code)
*
***/
if (((nb > 1) && (nb < nh)))  {
    nx = Math.max(nb, 1/*com.polytechnik.lapack310.Ilaenv.ilaenv(3,"DGEHRD"," ",n,ilo,ihi,-1)*/);
/***
*
*           Determine if workspace is large enough for blocked code
*
***/
if ((nx < nh))  {
    /***
*
*              Not enough workspace to use optimal NB:  determine the
*              minimum value of NB, and reduce NB or force use of
*              unblocked code
*
***/
if ((lwork < ((n*nb)+4160)))  {
    nbmin = Math.max(2, 2/*com.polytechnik.lapack310.Ilaenv.ilaenv(2,"DGEHRD"," ",n,ilo,ihi,-1)*/);
if ((lwork >= (((n*nbmin)+4160))))  {
    nb = (((lwork-4160))/n);
}
else  {
  nb = 1;
}              //  Close else.
}
}
}
// *
ldwork = n;
/***
*
*        Use unblocked code below
*
***/
if (((nb < nbmin) || (nb >= nh)))  {
    // *
i = ilo;
/***
*
*        Use blocked code
*
***/
}
else  {
  iwt = (1+(n*nb));
{
int _i_inc = nb;
for (i = ilo; (_i_inc < 0) ? i >= ((ihi-1)-nx) : i <= ((ihi-1)-nx); i += _i_inc) {
/***
*
*           Reduce columns i:i+ib-1 to Hessenberg form, returning the
*           matrices V and T of the block reflector H = I - V*T*V**T
*           which performs the reduction, and also the matrix Y = A*V*T
*
***/
ib = Math.min(nb, (ihi-i));
/***
*
*           Apply the block reflector H to A(1:ihi,i+ib:ihi) from the
*           right, computing  A := A - Y * V**T. V(i+ib,ib-1) must be set
*           to 1
*
***/
com.polytechnik.lapack310.Dlahr2.dlahr2(ihi,i,ib,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,lda,tau,(i-(1))+ _tau_offset,work,(iwt-(1))+ _work_offset,65,work,_work_offset,ldwork);
ei = a[((i+ib)-(1))+(((i+ib)-1)-(1)) * (lda)+ _a_offset];
a[((i+ib)-(1))+(((i+ib)-1)-(1)) * (lda)+ _a_offset] = 1.0;
com.polytechnik.lapack310.Dgemm.dgemm("No transpose","Transpose",ihi,(((ihi-i)-ib)+1),ib,-1.0,work,_work_offset,ldwork,a,((i+ib)-(1))+(i-(1)) * (lda)+ _a_offset,lda,1.0,a,(1-(1))+((i+ib)-(1)) * (lda)+ _a_offset,lda);
/***
*
*           Apply the block reflector H to A(1:i,i+1:i+ib-1) from the
*           right
*
***/
a[((i+ib)-(1))+(((i+ib)-1)-(1)) * (lda)+ _a_offset] = ei;
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower","Transpose","Unit",i,(ib-1),1.0,a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,lda,work,_work_offset,ldwork);
{
for (j = 0; j <= (ib-2); j++) {
com.polytechnik.lapack310.Daxpy.daxpy(i,-1.0,work,(((ldwork*j)+1)-(1))+ _work_offset,1,a,(1-(1))+(((i+j)+1)-(1)) * (lda)+ _a_offset,1);
/***
*
*           Apply the block reflector H to A(i+1:ihi,i+ib:n) from the
*           left
*
***/
//Dummy.label("com/polytechnik/lapack310/Dgehrd",30);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dlarfb.dlarfb("Left","Transpose","Forward","Columnwise",(ihi-i),(((n-i)-ib)+1),ib,a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,lda,work,(iwt-(1))+ _work_offset,65,a,((i+1)-(1))+((i+ib)-(1)) * (lda)+ _a_offset,lda,work,_work_offset,ldwork);
//Dummy.label("com/polytechnik/lapack310/Dgehrd",40);
}              //  Close for() loop.
}
/***
*
*     Use unblocked code to reduce the rest of the matrix
*
***/
}              //  Close else.
com.polytechnik.lapack310.Dgehd2.dgehd2(n,i,ihi,a,_a_offset,lda,tau,_tau_offset,work,_work_offset,iinfo);
// *
work[(1-(1))+ _work_offset] = (double)(lwkopt);
/***
*
*     End of DGEHRD
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgehrd",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dgehrd",999999);
return;
   }
} // End class.
