/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgeqr2.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dgeqr2 {

// *> \brief \b DGEQR2 computes the QR factorization of a general rectangul
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DGEQR2 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGEQR2( M, N, A, LDA, TAU, WORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, LDA, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGEQR2 computes a QR factorization of a real m-by-n matrix A:
// *>
// *>    A = Q * ( R ),
// *>            ( 0 )
// *>
// *> where:
// *>
// *>    Q is a m-by-m orthogonal matrix;
// *>    R is an upper-triangular n-by-n matrix;
// *>    0 is a (m-n)-by-n zero matrix, if m > n.
// *>
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix A.  M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the m by n matrix A.
// *>          On exit, the elements on and above the diagonal of the array
// *>          contain the min(m,n) by n upper trapezoidal matrix R (R is
// *>          upper triangular if m >= n); the elements below the diagonal
// *>          with the array TAU, represent the orthogonal matrix Q as a
// *>          product of elementary reflectors (see Further Details).
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,M).
// *> \endverbatim
// *>
// *> \param[out] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (min(M,N))
// *>          The scalar factors of the elementary reflectors (see Further
// *>          Details).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (N)
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0: successful exit
// *>          < 0: if INFO = -i, the i-th argument had an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleGEcomputational
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  The matrix Q is represented as a product of elementary reflectors
// *>
// *>     Q = H(1) H(2) . . . H(k), where k = min(m,n).
// *>
// *>  Each H(i) has the form
// *>
// *>     H(i) = I - tau * v * v**T
// *>
// *>  where tau is a real scalar, and v is a real vector with
// *>  v(1:i-1) = 0 and v(i) = 1; v(i+1:m) is stored on exit in A(i+1:m,i),
// *>  and tau in TAU(i).
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dgeqr2 (int m,
int n,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
intW info)  {

label_999999:{

int i= 0;
int k= 0;
double aii= 0.0d;
info.val = 0;
if ((m < 0))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info.val = -4;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DGEQR2",(-(info.val)));
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgeqr2",999999);
// *
}
// *
k = Math.min(m, n);
/***
*
*        Generate elementary reflector H(i) to annihilate A(i+1:m,i)
*
***/
{
for (i = 1; i <= k; i++) {
dlarfg_adapter(((m-i)+1),a,(i-(1))+(i-(1)) * (lda)+ _a_offset,a,(Math.min((i+1), m)-(1))+(i-(1)) * (lda)+ _a_offset,1,tau,(i-(1))+ _tau_offset);
/***
*
*           Apply H(i) to A(i:m,i+1:n) from the left
*
***/
if ((i < n))  {
    aii = a[(i-(1))+(i-(1)) * (lda)+ _a_offset];
a[(i-(1))+(i-(1)) * (lda)+ _a_offset] = 1.0;
com.polytechnik.lapack310.Dlarf.dlarf("Left",((m-i)+1),(n-i),a,(i-(1))+(i-(1)) * (lda)+ _a_offset,1,tau[(i-(1))+ _tau_offset],a,(i-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda,work,_work_offset);
a[(i-(1))+(i-(1)) * (lda)+ _a_offset] = aii;
}
//Dummy.label("com/polytechnik/lapack310/Dgeqr2",10);
}              //  Close for() loop.
}
/***
*
*     End of DGEQR2
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgeqr2",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dgeqr2",999999);
return;
   }
// adapter for dlarfg(ILorg/netlib/util/doubleW;[DIILorg/netlib/util/doubleW;)V
private static void dlarfg_adapter(int arg0 ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,int arg3 ,double [] arg4 , int arg4_offset )
{
doubleW _f2j_tmp1 = new doubleW(arg1[arg1_offset]);
doubleW _f2j_tmp4 = new doubleW(arg4[arg4_offset]);

com.polytechnik.lapack310.Dlarfg.dlarfg(arg0,_f2j_tmp1,arg2, arg2_offset,arg3,_f2j_tmp4);

arg1[arg1_offset] = _f2j_tmp1.val;
arg4[arg4_offset] = _f2j_tmp4.val;
}

} // End class.
