/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgeqrf.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dgeqrf {

// *> \brief \b DGEQRF
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DGEQRF + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGEQRF( M, N, A, LDA, TAU, WORK, LWORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, LDA, LWORK, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGEQRF computes a QR factorization of a real M-by-N matrix A:
// *>
// *>    A = Q * ( R ),
// *>            ( 0 )
// *>
// *> where:
// *>
// *>    Q is a M-by-M orthogonal matrix;
// *>    R is an upper-triangular N-by-N matrix;
// *>    0 is a (M-N)-by-N zero matrix, if M > N.
// *>
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix A.  M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the M-by-N matrix A.
// *>          On exit, the elements on and above the diagonal of the array
// *>          contain the min(M,N)-by-N upper trapezoidal matrix R (R is
// *>          upper triangular if m >= n); the elements below the diagonal
// *>          with the array TAU, represent the orthogonal matrix Q as a
// *>          product of min(m,n) elementary reflectors (see Further
// *>          Details).
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,M).
// *> \endverbatim
// *>
// *> \param[out] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (min(M,N))
// *>          The scalar factors of the elementary reflectors (see Further
// *>          Details).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK.
// *>          LWORK >= 1, if MIN(M,N) = 0, and LWORK >= N, otherwise.
// *>          For optimum performance LWORK >= N*NB, where NB is
// *>          the optimal blocksize.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleGEcomputational
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  The matrix Q is represented as a product of elementary reflectors
// *>
// *>     Q = H(1) H(2) . . . H(k), where k = min(m,n).
// *>
// *>  Each H(i) has the form
// *>
// *>     H(i) = I - tau * v * v**T
// *>
// *>  where tau is a real scalar, and v is a real vector with
// *>  v(1:i-1) = 0 and v(i) = 1; v(i+1:m) is stored on exit in A(i+1:m,i),
// *>  and tau in TAU(i).
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dgeqrf (int m,
int n,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

label_999999:{

boolean lquery= false;
int i= 0;
int ib= 0;
intW iinfo= new intW(0);
int iws= 0;
int k= 0;
int ldwork= 0;
int lwkopt= 0;
int nb= 0;
int nbmin= 0;
int nx= 0;
k = Math.min(m, n);
info.val = 0;
nb = 1;//com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DGEQRF"," ",m,n,-1,-1);
lquery = ((lwork == -1));
if ((m < 0))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info.val = -4;
}              // Close else if()
else if ((!lquery))  {
    if (((lwork <= 0) || (((m > 0) && (lwork < Math.max(1, n)))))) {
    info.val = -7;
}
    }              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DGEQRF",(-(info.val)));
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgeqrf",999999);
}
else if (lquery)  {
    if ((k == 0))  {
    lwkopt = 1;
}
else  {
  lwkopt = (n*nb);
}              //  Close else.
work[(1-(1))+ _work_offset] = (double)(lwkopt);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgeqrf",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
if ((k == 0))  {
    work[(1-(1))+ _work_offset] = (double)(1);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgeqrf",999999);
// *
}
nbmin = 2;
nx = 0;
iws = n;
/***
*
*        Determine when to cross over from blocked to unblocked code.
*
***/
if (((nb > 1) && (nb < k)))  {
    nx = Math.max(0, 0/*com.polytechnik.lapack310.Ilaenv.ilaenv(3,"DGEQRF"," ",m,n,-1,-1)*/);
/***
*
*           Determine if workspace is large enough for blocked code.
*
***/
if ((nx < k))  {
    ldwork = n;
iws = (ldwork*nb);
/***
*
*              Not enough workspace to use optimal NB:  reduce NB and
*              determine the minimum value of NB.
*
***/
if ((lwork < iws))  {
    nb = (lwork/ldwork);
    nbmin = Math.max(2, 2/*com.polytechnik.lapack310.Ilaenv.ilaenv(2,"DGEQRF"," ",m,n,-1,-1)*/);
}
}
// *
}
/***
*
*        Use blocked code initially
*
***/
if ((((nb >= nbmin) && (nb < k)) && (nx < k)))  {
    {
int _i_inc = nb;
for (i = 1; (_i_inc < 0) ? i >= (k-nx) : i <= (k-nx); i += _i_inc) {
/***
*
*           Compute the QR factorization of the current block
*           A(i:m,i:i+ib-1)
*
***/
ib = Math.min(((k-i)+1), nb);
com.polytechnik.lapack310.Dgeqr2.dgeqr2(((m-i)+1),ib,a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,tau,(i-(1))+ _tau_offset,work,_work_offset,iinfo);
/***
*
*              Form the triangular factor of the block reflector
*              H = H(i) H(i+1) . . . H(i+ib-1)
*
***/
if (((i+ib) <= n))  {
    /***
*
*              Apply H**T to A(i:m,i+ib:n) from the left
*
***/
com.polytechnik.lapack310.Dlarft.dlarft("Forward","Columnwise",((m-i)+1),ib,a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,tau,(i-(1))+ _tau_offset,work,_work_offset,ldwork);
com.polytechnik.lapack310.Dlarfb.dlarfb("Left","Transpose","Forward","Columnwise",((m-i)+1),(((n-i)-ib)+1),ib,a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,work,_work_offset,ldwork,a,(i-(1))+((i+ib)-(1)) * (lda)+ _a_offset,lda,work,((ib+1)-(1))+ _work_offset,ldwork);
}
//Dummy.label("com/polytechnik/lapack310/Dgeqrf",10);
}              //  Close for() loop.
}
}
else  {
  i = 1;
/***
*
*     Use unblocked code to factor the last or only block.
*
***/
}              //  Close else.
// *
if ((i <= k)) {
    com.polytechnik.lapack310.Dgeqr2.dgeqr2(((m-i)+1),((n-i)+1),a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,tau,(i-(1))+ _tau_offset,work,_work_offset,iinfo);
}
    work[(1-(1))+ _work_offset] = (double)(iws);
/***
*
*     End of DGEQRF
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgeqrf",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dgeqrf",999999);
return;
   }
} // End class.
