/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgesvd.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dgesvd {

// *> \brief <b> DGESVD computes the singular value decomposition (SVD) for
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DGESVD + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGESVD( JOBU, JOBVT, M, N, A, LDA, S, U, LDU, VT, LDV
// *                          WORK, LWORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          JOBU, JOBVT
// *       INTEGER            INFO, LDA, LDU, LDVT, LWORK, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), S( * ), U( LDU, * ),
// *      $                   VT( LDVT, * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGESVD computes the singular value decomposition (SVD) of a real
// *> M-by-N matrix A, optionally computing the left and/or right singular

// *> vectors. The SVD is written
// *>
// *>      A = U * SIGMA * transpose(V)
// *>
// *> where SIGMA is an M-by-N matrix which is zero except for its
// *> min(m,n) diagonal elements, U is an M-by-M orthogonal matrix, and
// *> V is an N-by-N orthogonal matrix.  The diagonal elements of SIGMA
// *> are the singular values of A; they are real and non-negative, and
// *> are returned in descending order.  The first min(m,n) columns of
// *> U and V are the left and right singular vectors of A.
// *>
// *> Note that the routine returns V**T, not V.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] JOBU
// *> \verbatim
// *>          JOBU is CHARACTER*1
// *>          Specifies options for computing all or part of the matrix U:
// *>          = 'A':  all M columns of U are returned in array U:
// *>          = 'S':  the first min(m,n) columns of U (the left singular
// *>                  vectors) are returned in the array U;
// *>          = 'O':  the first min(m,n) columns of U (the left singular
// *>                  vectors) are overwritten on the array A;
// *>          = 'N':  no columns of U (no left singular vectors) are
// *>                  computed.
// *> \endverbatim
// *>
// *> \param[in] JOBVT
// *> \verbatim
// *>          JOBVT is CHARACTER*1
// *>          Specifies options for computing all or part of the matrix
// *>          V**T:
// *>          = 'A':  all N rows of V**T are returned in the array VT;
// *>          = 'S':  the first min(m,n) rows of V**T (the right singular

// *>                  vectors) are returned in the array VT;
// *>          = 'O':  the first min(m,n) rows of V**T (the right singular

// *>                  vectors) are overwritten on the array A;
// *>          = 'N':  no rows of V**T (no right singular vectors) are
// *>                  computed.
// *>
// *>          JOBVT and JOBU cannot both be 'O'.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the input matrix A.  M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the input matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the M-by-N matrix A.
// *>          On exit,
// *>          if JOBU = 'O',  A is overwritten with the first min(m,n)
// *>                          columns of U (the left singular vectors,
// *>                          stored columnwise);
// *>          if JOBVT = 'O', A is overwritten with the first min(m,n)
// *>                          rows of V**T (the right singular vectors,
// *>                          stored rowwise);
// *>          if JOBU .ne. 'O' and JOBVT .ne. 'O', the contents of A
// *>                          are destroyed.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,M).
// *> \endverbatim
// *>
// *> \param[out] S
// *> \verbatim
// *>          S is DOUBLE PRECISION array, dimension (min(M,N))
// *>          The singular values of A, sorted so that S(i) >= S(i+1).
// *> \endverbatim
// *>
// *> \param[out] U
// *> \verbatim
// *>          U is DOUBLE PRECISION array, dimension (LDU,UCOL)
// *>          (LDU,M) if JOBU = 'A' or (LDU,min(M,N)) if JOBU = 'S'.
// *>          If JOBU = 'A', U contains the M-by-M orthogonal matrix U;
// *>          if JOBU = 'S', U contains the first min(m,n) columns of U
// *>          (the left singular vectors, stored columnwise);
// *>          if JOBU = 'N' or 'O', U is not referenced.
// *> \endverbatim
// *>
// *> \param[in] LDU
// *> \verbatim
// *>          LDU is INTEGER
// *>          The leading dimension of the array U.  LDU >= 1; if
// *>          JOBU = 'S' or 'A', LDU >= M.
// *> \endverbatim
// *>
// *> \param[out] VT
// *> \verbatim
// *>          VT is DOUBLE PRECISION array, dimension (LDVT,N)
// *>          If JOBVT = 'A', VT contains the N-by-N orthogonal matrix
// *>          V**T;
// *>          if JOBVT = 'S', VT contains the first min(m,n) rows of
// *>          V**T (the right singular vectors, stored rowwise);
// *>          if JOBVT = 'N' or 'O', VT is not referenced.
// *> \endverbatim
// *>
// *> \param[in] LDVT
// *> \verbatim
// *>          LDVT is INTEGER
// *>          The leading dimension of the array VT.  LDVT >= 1; if
// *>          JOBVT = 'A', LDVT >= N; if JOBVT = 'S', LDVT >= min(M,N).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK;
// *>          if INFO > 0, WORK(2:MIN(M,N)) contains the unconverged
// *>          superdiagonal elements of an upper bidiagonal matrix B
// *>          whose diagonal is in S (not necessarily sorted). B
// *>          satisfies A = U * B * VT, so it has the same singular values
// *>          as A, and singular vectors related by U and VT.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK.
// *>          LWORK >= MAX(1,5*MIN(M,N)) for the paths (see comments insid
// *>             - PATH 1  (M much larger than N, JOBU='N')
// *>             - PATH 1t (N much larger than M, JOBVT='N')
// *>          LWORK >= MAX(1,3*MIN(M,N) + MAX(M,N),5*MIN(M,N)) for the oth
// *>          For good performance, LWORK should generally be larger.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit.
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value.

// *>          > 0:  if DBDSQR did not converge, INFO specifies how many
// *>                superdiagonals of an intermediate bidiagonal form B
// *>                did not converge to zero. See the description of WORK

// *>                above for details.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleGEsing
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dgesvd (String jobu,
String jobvt,
int m,
int n,
double [] a, int _a_offset,
int lda,
double [] s, int _s_offset,
double [] u, int _u_offset,
int ldu,
double [] vt, int _vt_offset,
int ldvt,
double [] work, int _work_offset,
int lwork,
intW info)  {

label_999999:{

boolean lquery= false;
boolean wntua= false;
boolean wntuas= false;
boolean wntun= false;
boolean wntuo= false;
boolean wntus= false;
boolean wntva= false;
boolean wntvas= false;
boolean wntvn= false;
boolean wntvo= false;
boolean wntvs= false;
int bdspac= 0;
int blk= 0;
int chunk= 0;
int i= 0;
int ie= 0;
intW ierr= new intW(0);
int ir= 0;
int iscl= 0;
int itau= 0;
int itaup= 0;
int itauq= 0;
int iu= 0;
int iwork= 0;
int ldwrkr= 0;
int ldwrku= 0;
int maxwrk= 0;
int minmn= 0;
int minwrk= 0;
int mnthr= 0;
int ncu= 0;
int ncvt= 0;
int nru= 0;
int nrvt= 0;
int wrkbl= 0;
int lwork_dgeqrf= 0;
int lwork_dorgqr_n= 0;
int lwork_dorgqr_m= 0;
int lwork_dgebrd= 0;
int lwork_dorgbr_p= 0;
int lwork_dorgbr_q= 0;
int lwork_dgelqf= 0;
int lwork_dorglq_n= 0;
int lwork_dorglq_m= 0;
double anrm= 0.0d;
double bignum= 0.0d;
double eps= 0.0d;
double smlnum= 0.0d;
double [] dum= new double[(1)];
jobu = jobu.substring((1)-1,1);
jobvt = jobvt.substring((1)-1,1);
info.val = 0;
minmn = Math.min(m, n);
wntua = com.polytechnik.lapack310.Lsame.lsame(jobu,"A");
wntus = com.polytechnik.lapack310.Lsame.lsame(jobu,"S");
wntuas = (wntua || wntus);
wntuo = com.polytechnik.lapack310.Lsame.lsame(jobu,"O");
wntun = com.polytechnik.lapack310.Lsame.lsame(jobu,"N");
wntva = com.polytechnik.lapack310.Lsame.lsame(jobvt,"A");
wntvs = com.polytechnik.lapack310.Lsame.lsame(jobvt,"S");
wntvas = (wntva || wntvs);
wntvo = com.polytechnik.lapack310.Lsame.lsame(jobvt,"O");
wntvn = com.polytechnik.lapack310.Lsame.lsame(jobvt,"N");
// *
lquery = ((lwork == -1));
if ((!((((wntua || wntus) || wntuo) || wntun))))  {
    info.val = -1;
}
else if (((!((((wntva || wntvs) || wntvo) || wntvn))) || ((wntvo && wntuo))))  {
    info.val = -2;
}              // Close else if()
else if ((m < 0))  {
    info.val = -3;
}              // Close else if()
else if ((n < 0))  {
    info.val = -4;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info.val = -6;
}              // Close else if()
else if (((ldu < 1) || ((wntuas && (ldu < m)))))  {
    info.val = -9;
}              // Close else if()
else if ((((ldvt < 1) || ((wntva && (ldvt < n)))) || ((wntvs && (ldvt < minmn)))))  {
    info.val = -11;
/***
*
*     Compute workspace
*      (Note: Comments in the code beginning "Workspace:" describe the
*       minimal amount of workspace needed at that point in the code,
*       as well as the preferred amount for good performance.
*       NB refers to the optimal block size for the immediately
*       following subroutine, as returned by ILAENV.)
*
***/
}              // Close else if()
if ((info.val == 0))  {
    minwrk = 1;
maxwrk = 1;
/***
*
*           Compute space needed for DBDSQR
*
***/
if (((m >= n) && (minmn > 0)))  {
    mnthr = 1+(int)(1.6*Math.min(m,n));//com.polytechnik.lapack310.Ilaenv.ilaenv(6,"DGESVD",(jobu+jobvt),m,n,0,0);
// *           Compute space needed for DGEQRF
bdspac = (5*n);
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,dum,(1-(1)),dum,(1-(1)),-1,ierr);
// *           Compute space needed for DORGQR
lwork_dgeqrf = (int)(dum[(1-(1))]);
com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,n,a,_a_offset,lda,dum,(1-(1)),dum,(1-(1)),-1,ierr);
lwork_dorgqr_n = (int)(dum[(1-(1))]);
com.polytechnik.lapack310.Dorgqr.dorgqr(m,m,n,a,_a_offset,lda,dum,(1-(1)),dum,(1-(1)),-1,ierr);
// *           Compute space needed for DGEBRD
lwork_dorgqr_m = (int)(dum[(1-(1))]);
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,a,_a_offset,lda,s,_s_offset,dum,(1-(1)),dum,(1-(1)),dum,(1-(1)),dum,(1-(1)),-1,ierr);
// *           Compute space needed for DORGBR P
lwork_dgebrd = (int)(dum[(1-(1))]);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,a,_a_offset,lda,dum,(1-(1)),dum,(1-(1)),-1,ierr);
// *           Compute space needed for DORGBR Q
lwork_dorgbr_p = (int)(dum[(1-(1))]);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",n,n,n,a,_a_offset,lda,dum,(1-(1)),dum,(1-(1)),-1,ierr);
// *
lwork_dorgbr_q = (int)(dum[(1-(1))]);
if ((m >= mnthr))  {
    /***
*
*                 Path 1 (M much larger than N, JOBU='N')
*
***/
if (wntun)  {
    maxwrk = (n+lwork_dgeqrf);
maxwrk = Math.max(maxwrk, ((3*n)+lwork_dgebrd));
if ((wntvo || wntvas)) {
    maxwrk = Math.max(maxwrk, ((3*n)+lwork_dorgbr_p));
}
    maxwrk = Math.max(maxwrk, bdspac);
minwrk = Math.max((4*n), bdspac);
/***
*
*                 Path 2 (M much larger than N, JOBU='O', JOBVT='N')
*
***/
}
else if ((wntuo && wntvn))  {
    wrkbl = (n+lwork_dgeqrf);
wrkbl = Math.max(wrkbl, (n+lwork_dorgqr_n));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = Math.max(((n*n)+wrkbl), (((n*n)+(m*n))+n));
minwrk = Math.max(((3*n)+m), bdspac);
/***
*
*                 Path 3 (M much larger than N, JOBU='O', JOBVT='S' or
*                 'A')
*
***/
}              // Close else if()
else if ((wntuo && wntvas))  {
    wrkbl = (n+lwork_dgeqrf);
wrkbl = Math.max(wrkbl, (n+lwork_dorgqr_n));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = Math.max(((n*n)+wrkbl), (((n*n)+(m*n))+n));
minwrk = Math.max(((3*n)+m), bdspac);
/***
*
*                 Path 4 (M much larger than N, JOBU='S', JOBVT='N')
*
***/
}              // Close else if()
else if ((wntus && wntvn))  {
    wrkbl = (n+lwork_dgeqrf);
wrkbl = Math.max(wrkbl, (n+lwork_dorgqr_n));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = ((n*n)+wrkbl);
minwrk = Math.max(((3*n)+m), bdspac);
/***
*
*                 Path 5 (M much larger than N, JOBU='S', JOBVT='O')
*
***/
}              // Close else if()
else if ((wntus && wntvo))  {
    wrkbl = (n+lwork_dgeqrf);
wrkbl = Math.max(wrkbl, (n+lwork_dorgqr_n));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = (((2*n)*n)+wrkbl);
minwrk = Math.max(((3*n)+m), bdspac);
/***
*
*                 Path 6 (M much larger than N, JOBU='S', JOBVT='S' or
*                 'A')
*
***/
}              // Close else if()
else if ((wntus && wntvas))  {
    wrkbl = (n+lwork_dgeqrf);
wrkbl = Math.max(wrkbl, (n+lwork_dorgqr_n));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = ((n*n)+wrkbl);
minwrk = Math.max(((3*n)+m), bdspac);
/***
*
*                 Path 7 (M much larger than N, JOBU='A', JOBVT='N')
*
***/
}              // Close else if()
else if ((wntua && wntvn))  {
    wrkbl = (n+lwork_dgeqrf);
wrkbl = Math.max(wrkbl, (n+lwork_dorgqr_m));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = ((n*n)+wrkbl);
minwrk = Math.max(((3*n)+m), bdspac);
/***
*
*                 Path 8 (M much larger than N, JOBU='A', JOBVT='O')
*
***/
}              // Close else if()
else if ((wntua && wntvo))  {
    wrkbl = (n+lwork_dgeqrf);
wrkbl = Math.max(wrkbl, (n+lwork_dorgqr_m));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = (((2*n)*n)+wrkbl);
minwrk = Math.max(((3*n)+m), bdspac);
/***
*
*                 Path 9 (M much larger than N, JOBU='A', JOBVT='S' or
*                 'A')
*
***/
}              // Close else if()
else if ((wntua && wntvas))  {
    wrkbl = (n+lwork_dgeqrf);
wrkbl = Math.max(wrkbl, (n+lwork_dorgqr_m));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, ((3*n)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = ((n*n)+wrkbl);
minwrk = Math.max(((3*n)+m), bdspac);
}              // Close else if()
/***
*
*              Path 10 (M at least N, but not much larger)
*
***/
}
else  {
  com.polytechnik.lapack310.Dgebrd.dgebrd(m,n,a,_a_offset,lda,s,_s_offset,dum,(1-(1)),dum,(1-(1)),dum,(1-(1)),dum,(1-(1)),-1,ierr);
lwork_dgebrd = (int)(dum[(1-(1))]);
maxwrk = ((3*n)+lwork_dgebrd);
if ((wntus || wntuo))  {
    com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,n,n,a,_a_offset,lda,dum,(1-(1)),dum,(1-(1)),-1,ierr);
lwork_dorgbr_q = (int)(dum[(1-(1))]);
maxwrk = Math.max(maxwrk, ((3*n)+lwork_dorgbr_q));
}
if (wntua)  {
    com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,n,a,_a_offset,lda,dum,(1-(1)),dum,(1-(1)),-1,ierr);
lwork_dorgbr_q = (int)(dum[(1-(1))]);
maxwrk = Math.max(maxwrk, ((3*n)+lwork_dorgbr_q));
}
if ((!wntvn))  {
    maxwrk = Math.max(maxwrk, ((3*n)+lwork_dorgbr_p));
}
maxwrk = Math.max(maxwrk, bdspac);
minwrk = Math.max(((3*n)+m), bdspac);
}              //  Close else.
/***
*
*           Compute space needed for DBDSQR
*
***/
}
else if ((minmn > 0))  {
    mnthr = 1+(int)(1.6*Math.min(m,n));//com.polytechnik.lapack310.Ilaenv.ilaenv(6,"DGESVD",(jobu+jobvt),m,n,0,0);
// *           Compute space needed for DGELQF
bdspac = (5*m);
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,dum,(1-(1)),dum,(1-(1)),-1,ierr);
// *           Compute space needed for DORGLQ
lwork_dgelqf = (int)(dum[(1-(1))]);
com.polytechnik.lapack310.Dorglq.dorglq(n,n,m,dum,(1-(1)),n,dum,(1-(1)),dum,(1-(1)),-1,ierr);
lwork_dorglq_n = (int)(dum[(1-(1))]);
com.polytechnik.lapack310.Dorglq.dorglq(m,n,m,a,_a_offset,lda,dum,(1-(1)),dum,(1-(1)),-1,ierr);
// *           Compute space needed for DGEBRD
lwork_dorglq_m = (int)(dum[(1-(1))]);
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,a,_a_offset,lda,s,_s_offset,dum,(1-(1)),dum,(1-(1)),dum,(1-(1)),dum,(1-(1)),-1,ierr);
// *            Compute space needed for DORGBR P
lwork_dgebrd = (int)(dum[(1-(1))]);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,m,m,a,_a_offset,n,dum,(1-(1)),dum,(1-(1)),-1,ierr);
// *           Compute space needed for DORGBR Q
lwork_dorgbr_p = (int)(dum[(1-(1))]);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,a,_a_offset,n,dum,(1-(1)),dum,(1-(1)),-1,ierr);
lwork_dorgbr_q = (int)(dum[(1-(1))]);
if ((n >= mnthr))  {
    /***
*
*                 Path 1t(N much larger than M, JOBVT='N')
*
***/
if (wntvn)  {
    maxwrk = (m+lwork_dgelqf);
maxwrk = Math.max(maxwrk, ((3*m)+lwork_dgebrd));
if ((wntuo || wntuas)) {
    maxwrk = Math.max(maxwrk, ((3*m)+lwork_dorgbr_q));
}
    maxwrk = Math.max(maxwrk, bdspac);
minwrk = Math.max((4*m), bdspac);
/***
*
*                 Path 2t(N much larger than M, JOBU='N', JOBVT='O')
*
***/
}
else if ((wntvo && wntun))  {
    wrkbl = (m+lwork_dgelqf);
wrkbl = Math.max(wrkbl, (m+lwork_dorglq_m));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = Math.max(((m*m)+wrkbl), (((m*m)+(m*n))+m));
minwrk = Math.max(((3*m)+n), bdspac);
/***
*
*                 Path 3t(N much larger than M, JOBU='S' or 'A',
*                 JOBVT='O')
*
***/
}              // Close else if()
else if ((wntvo && wntuas))  {
    wrkbl = (m+lwork_dgelqf);
wrkbl = Math.max(wrkbl, (m+lwork_dorglq_m));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = Math.max(((m*m)+wrkbl), (((m*m)+(m*n))+m));
minwrk = Math.max(((3*m)+n), bdspac);
/***
*
*                 Path 4t(N much larger than M, JOBU='N', JOBVT='S')
*
***/
}              // Close else if()
else if ((wntvs && wntun))  {
    wrkbl = (m+lwork_dgelqf);
wrkbl = Math.max(wrkbl, (m+lwork_dorglq_m));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = ((m*m)+wrkbl);
minwrk = Math.max(((3*m)+n), bdspac);
/***
*
*                 Path 5t(N much larger than M, JOBU='O', JOBVT='S')
*
***/
}              // Close else if()
else if ((wntvs && wntuo))  {
    wrkbl = (m+lwork_dgelqf);
wrkbl = Math.max(wrkbl, (m+lwork_dorglq_m));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = (((2*m)*m)+wrkbl);
minwrk = Math.max(((3*m)+n), bdspac);
/***
*
*                 Path 6t(N much larger than M, JOBU='S' or 'A',
*                 JOBVT='S')
*
***/
}              // Close else if()
else if ((wntvs && wntuas))  {
    wrkbl = (m+lwork_dgelqf);
wrkbl = Math.max(wrkbl, (m+lwork_dorglq_m));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = ((m*m)+wrkbl);
minwrk = Math.max(((3*m)+n), bdspac);
/***
*
*                 Path 7t(N much larger than M, JOBU='N', JOBVT='A')
*
***/
}              // Close else if()
else if ((wntva && wntun))  {
    wrkbl = (m+lwork_dgelqf);
wrkbl = Math.max(wrkbl, (m+lwork_dorglq_n));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = ((m*m)+wrkbl);
minwrk = Math.max(((3*m)+n), bdspac);
/***
*
*                 Path 8t(N much larger than M, JOBU='O', JOBVT='A')
*
***/
}              // Close else if()
else if ((wntva && wntuo))  {
    wrkbl = (m+lwork_dgelqf);
wrkbl = Math.max(wrkbl, (m+lwork_dorglq_n));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = (((2*m)*m)+wrkbl);
minwrk = Math.max(((3*m)+n), bdspac);
/***
*
*                 Path 9t(N much larger than M, JOBU='S' or 'A',
*                 JOBVT='A')
*
***/
}              // Close else if()
else if ((wntva && wntuas))  {
    wrkbl = (m+lwork_dgelqf);
wrkbl = Math.max(wrkbl, (m+lwork_dorglq_n));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dgebrd));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_p));
wrkbl = Math.max(wrkbl, ((3*m)+lwork_dorgbr_q));
wrkbl = Math.max(wrkbl, bdspac);
maxwrk = ((m*m)+wrkbl);
minwrk = Math.max(((3*m)+n), bdspac);
}              // Close else if()
/***
*
*              Path 10t(N greater than M, but not much larger)
*
***/
}
else  {
  com.polytechnik.lapack310.Dgebrd.dgebrd(m,n,a,_a_offset,lda,s,_s_offset,dum,(1-(1)),dum,(1-(1)),dum,(1-(1)),dum,(1-(1)),-1,ierr);
lwork_dgebrd = (int)(dum[(1-(1))]);
maxwrk = ((3*m)+lwork_dgebrd);
// *                Compute space needed for DORGBR P
if ((wntvs || wntvo))  {
    com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,n,m,a,_a_offset,n,dum,(1-(1)),dum,(1-(1)),-1,ierr);
lwork_dorgbr_p = (int)(dum[(1-(1))]);
maxwrk = Math.max(maxwrk, ((3*m)+lwork_dorgbr_p));
}
if (wntva)  {
    com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,m,a,_a_offset,n,dum,(1-(1)),dum,(1-(1)),-1,ierr);
lwork_dorgbr_p = (int)(dum[(1-(1))]);
maxwrk = Math.max(maxwrk, ((3*m)+lwork_dorgbr_p));
}
if ((!wntun))  {
    maxwrk = Math.max(maxwrk, ((3*m)+lwork_dorgbr_q));
}
maxwrk = Math.max(maxwrk, bdspac);
minwrk = Math.max(((3*m)+n), bdspac);
}              //  Close else.
}              // Close else if()
maxwrk = Math.max(maxwrk, minwrk);
// *
work[(1-(1))+ _work_offset] = (double)(maxwrk);
if (((lwork < minwrk) && (!lquery)))  {
    info.val = -13;
}
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DGESVD",(-(info.val)));
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgesvd",999999);
}
else if (lquery)  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgesvd",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
if (((m == 0) || (n == 0)))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgesvd",999999);
/***
*
*     Get machine constants
*
***/
}
eps = com.polytechnik.lapack310.Dlamch.dlamch("P");
smlnum = (Math.sqrt(com.polytechnik.lapack310.Dlamch.dlamch("S"))/eps);
/***
*
*     Scale A if max element outside range [SMLNUM,BIGNUM]
*
***/
bignum = (1.0/smlnum);
anrm = com.polytechnik.lapack310.Dlange.dlange("M",m,n,a,_a_offset,lda,dum,0);
iscl = 0;
if (((anrm > 0.0) && (anrm < smlnum)))  {
    iscl = 1;
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,anrm,smlnum,m,n,a,_a_offset,lda,ierr);
}
else if ((anrm > bignum))  {
    iscl = 1;
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,anrm,bignum,m,n,a,_a_offset,lda,ierr);
// *
}              // Close else if()
/***
*
*        A has at least as many rows as columns. If A has sufficiently
*        more rows than columns, first reduce using the QR
*        decomposition (if sufficient workspace available)
*
***/
if ((m >= n))  {
    // *
if ((m >= mnthr))  {
    /***
*
*              Path 1 (M much larger than N, JOBU='N')
*              No left singular vectors to be computed
*
***/
if (wntun)  {
    itau = 1;
/***
*
*              Compute A=Q*R
*              (Workspace: need 2*N, prefer N + N*NB)
*
***/
iwork = (itau+n);
/***
*
*              Zero out below R
*
***/
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
if ((n > 1))  {
    com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,a,(2-(1))+(1-(1)) * (lda)+ _a_offset,lda);
}
ie = 1;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*              Bidiagonalize R in A
*              (Workspace: need 4*N, prefer 3*N + 2*N*NB)
*
***/
iwork = (itaup+n);
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ncvt = 0;
/***
*
*                 If right singular vectors desired, generate P'.
*                 (Workspace: need 4*N-1, prefer 3*N + (N-1)*NB)
*
***/
if ((wntvo || wntvas))  {
    com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,a,_a_offset,lda,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ncvt = n;
}
/***
*
*              Perform bidiagonal QR iteration, computing right
*              singular vectors of A in A if desired
*              (Workspace: need BDSPAC)
*
***/
iwork = (ie+n);
/***
*
*              If right singular vectors desired in VT, copy them there
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,ncvt,0,0,s,_s_offset,work,(ie-(1))+ _work_offset,a,_a_offset,lda,dum,0,1,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
if (wntvas) {
    com.polytechnik.lapack310.Dlacpy.dlacpy("F",n,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
}
    /***
*
*              Path 2 (M much larger than N, JOBU='O', JOBVT='N')
*              N left singular vectors to be overwritten on A and
*              no right singular vectors to be computed
*
***/
}
else if ((wntuo && wntvn))  {
    /***
*
*                 Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((n*n)+Math.max((4*n), bdspac))))  {
    ir = 1;
/***
*
*                    WORK(IU) is LDA by N, WORK(IR) is LDA by N
*
***/
if ((lwork >= (Math.max(wrkbl, ((lda*n)+n))+(lda*n))))  {
    ldwrku = lda;
ldwrkr = lda;
/***
*
*                    WORK(IU) is LDA by N, WORK(IR) is N by N
*
***/
}
else if ((lwork >= (Math.max(wrkbl, ((lda*n)+n))+(n*n))))  {
    ldwrku = lda;
ldwrkr = n;
/***
*
*                    WORK(IU) is LDWRKU by N, WORK(IR) is N by N
*
***/
}              // Close else if()
else  {
  ldwrku = ((((lwork-(n*n))-n))/n);
ldwrkr = n;
}              //  Close else.
itau = (ir+(ldwrkr*n));
/***
*
*                 Compute A=Q*R
*                 (Workspace: need N*N + 2*N, prefer N*N + N + N*NB)
*
***/
iwork = (itau+n);
/***
*
*                 Copy R to WORK(IR) and zero out below it
*
***/
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                 Generate Q in A
*                 (Workspace: need N*N + 2*N, prefer N*N + N + N*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,work,((ir+1)-(1))+ _work_offset,ldwrkr);
com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                 Bidiagonalize R in WORK(IR)
*                 (Workspace: need N*N + 4*N, prefer N*N + 3*N + 2*N*NB)
*
***/
iwork = (itaup+n);
/***
*
*                 Generate left vectors bidiagonalizing R
*                 (Workspace: need N*N + 4*N, prefer N*N + 3*N + N*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,work,(ir-(1))+ _work_offset,ldwrkr,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",n,n,n,work,(ir-(1))+ _work_offset,ldwrkr,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Perform bidiagonal QR iteration, computing left
*                 singular vectors of R in WORK(IR)
*                 (Workspace: need N*N + BDSPAC)
*
***/
iwork = (ie+n);
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,0,n,0,s,_s_offset,work,(ie-(1))+ _work_offset,dum,0,1,work,(ir-(1))+ _work_offset,ldwrkr,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                 Multiply Q in A by left singular vectors of R in
*                 WORK(IR), storing result in WORK(IU) and copying to A
*                 (Workspace: need N*N + 2*N, prefer N*N + M*N + N)
*
***/
iu = (ie+n);
{
int _i_inc = ldwrku;
for (i = 1; (_i_inc < 0) ? i >= m : i <= m; i += _i_inc) {
chunk = Math.min(((m-i)+1), ldwrku);
com.polytechnik.lapack310.Dgemm.dgemm("N","N",chunk,n,n,1.0,a,(i-(1))+(1-(1)) * (lda)+ _a_offset,lda,work,(ir-(1))+ _work_offset,ldwrkr,0.0,work,(iu-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dlacpy.dlacpy("F",chunk,n,work,(iu-(1))+ _work_offset,ldwrku,a,(i-(1))+(1-(1)) * (lda)+ _a_offset,lda);
// *
//Dummy.label("com/polytechnik/lapack310/Dgesvd",10);
}              //  Close for() loop.
}
/***
*
*                 Insufficient workspace for a fast algorithm
*
***/
}
else  {
  ie = 1;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                 Bidiagonalize A
*                 (Workspace: need 3*N + M, prefer 3*N + (M + N)*NB)
*
***/
iwork = (itaup+n);
/***
*
*                 Generate left vectors bidiagonalizing A
*                 (Workspace: need 4*N, prefer 3*N + N*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,n,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,n,n,a,_a_offset,lda,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Perform bidiagonal QR iteration, computing left
*                 singular vectors of A in A
*                 (Workspace: need BDSPAC)
*
***/
iwork = (ie+n);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,0,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,dum,0,1,a,_a_offset,lda,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
/***
*
*              Path 3 (M much larger than N, JOBU='O', JOBVT='S' or 'A')
*              N left singular vectors to be overwritten on A and
*              N right singular vectors to be computed in VT
*
***/
}              // Close else if()
else if ((wntuo && wntvas))  {
    /***
*
*                 Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((n*n)+Math.max((4*n), bdspac))))  {
    ir = 1;
/***
*
*                    WORK(IU) is LDA by N and WORK(IR) is LDA by N
*
***/
if ((lwork >= (Math.max(wrkbl, ((lda*n)+n))+(lda*n))))  {
    ldwrku = lda;
ldwrkr = lda;
/***
*
*                    WORK(IU) is LDA by N and WORK(IR) is N by N
*
***/
}
else if ((lwork >= (Math.max(wrkbl, ((lda*n)+n))+(n*n))))  {
    ldwrku = lda;
ldwrkr = n;
/***
*
*                    WORK(IU) is LDWRKU by N and WORK(IR) is N by N
*
***/
}              // Close else if()
else  {
  ldwrku = ((((lwork-(n*n))-n))/n);
ldwrkr = n;
}              //  Close else.
itau = (ir+(ldwrkr*n));
/***
*
*                 Compute A=Q*R
*                 (Workspace: need N*N + 2*N, prefer N*N + N + N*NB)
*
***/
iwork = (itau+n);
/***
*
*                 Copy R to VT, zeroing out below it
*
***/
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
/***
*
*                 Generate Q in A
*                 (Workspace: need N*N + 2*N, prefer N*N + N + N*NB)
*
***/
if ((n > 1)) {
    com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,vt,(2-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt);
}
    com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                 Bidiagonalize R in VT, copying result to WORK(IR)
*                 (Workspace: need N*N + 4*N, prefer N*N + 3*N + 2*N*NB)
*
***/
iwork = (itaup+n);
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,vt,_vt_offset,ldvt,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Generate left vectors bidiagonalizing R in WORK(IR)
*                 (Workspace: need N*N + 4*N, prefer N*N + 3*N + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",n,n,vt,_vt_offset,ldvt,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                 Generate right vectors bidiagonalizing R in VT
*                 (Workspace: need N*N + 4*N-1, prefer N*N + 3*N + (N-1)*NB)
*
***/
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",n,n,n,work,(ir-(1))+ _work_offset,ldwrkr,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,vt,_vt_offset,ldvt,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Perform bidiagonal QR iteration, computing left
*                 singular vectors of R in WORK(IR) and computing right
*                 singular vectors of R in VT
*                 (Workspace: need N*N + BDSPAC)
*
***/
iwork = (ie+n);
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,n,n,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,work,(ir-(1))+ _work_offset,ldwrkr,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                 Multiply Q in A by left singular vectors of R in
*                 WORK(IR), storing result in WORK(IU) and copying to A
*                 (Workspace: need N*N + 2*N, prefer N*N + M*N + N)
*
***/
iu = (ie+n);
{
int _i_inc = ldwrku;
for (i = 1; (_i_inc < 0) ? i >= m : i <= m; i += _i_inc) {
chunk = Math.min(((m-i)+1), ldwrku);
com.polytechnik.lapack310.Dgemm.dgemm("N","N",chunk,n,n,1.0,a,(i-(1))+(1-(1)) * (lda)+ _a_offset,lda,work,(ir-(1))+ _work_offset,ldwrkr,0.0,work,(iu-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dlacpy.dlacpy("F",chunk,n,work,(iu-(1))+ _work_offset,ldwrku,a,(i-(1))+(1-(1)) * (lda)+ _a_offset,lda);
// *
//Dummy.label("com/polytechnik/lapack310/Dgesvd",20);
}              //  Close for() loop.
}
/***
*
*                 Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                 Compute A=Q*R
*                 (Workspace: need 2*N, prefer N + N*NB)
*
***/
iwork = (itau+n);
/***
*
*                 Copy R to VT, zeroing out below it
*
***/
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
/***
*
*                 Generate Q in A
*                 (Workspace: need 2*N, prefer N + N*NB)
*
***/
if ((n > 1)) {
    com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,vt,(2-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt);
}
    com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                 Bidiagonalize R in VT
*                 (Workspace: need 4*N, prefer 3*N + 2*N*NB)
*
***/
iwork = (itaup+n);
/***
*
*                 Multiply Q in A by left vectors bidiagonalizing R
*                 (Workspace: need 3*N + M, prefer 3*N + M*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,vt,_vt_offset,ldvt,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Generate right vectors bidiagonalizing R in VT
*                 (Workspace: need 4*N-1, prefer 3*N + (N-1)*NB)
*
***/
com.polytechnik.lapack310.Dormbr.dormbr("Q","R","N",m,n,n,vt,_vt_offset,ldvt,work,(itauq-(1))+ _work_offset,a,_a_offset,lda,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,vt,_vt_offset,ldvt,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Perform bidiagonal QR iteration, computing left
*                 singular vectors of A in A and computing right
*                 singular vectors of A in VT
*                 (Workspace: need BDSPAC)
*
***/
iwork = (ie+n);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,n,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,a,_a_offset,lda,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
// *
}              // Close else if()
else if (wntus)  {
    /***
*
*                 Path 4 (M much larger than N, JOBU='S', JOBVT='N')
*                 N left singular vectors to be computed in U and
*                 no right singular vectors to be computed
*
***/
if (wntvn)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((n*n)+Math.max((4*n), bdspac))))  {
    ir = 1;
/***
*
*                       WORK(IR) is LDA by N
*
***/
if ((lwork >= (wrkbl+(lda*n))))  {
    ldwrkr = lda;
/***
*
*                       WORK(IR) is N by N
*
***/
}
else  {
  ldwrkr = n;
}              //  Close else.
itau = (ir+(ldwrkr*n));
/***
*
*                    Compute A=Q*R
*                    (Workspace: need N*N + 2*N, prefer N*N + N + N*NB)
*
***/
iwork = (itau+n);
/***
*
*                    Copy R to WORK(IR), zeroing out below it
*
***/
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                    Generate Q in A
*                    (Workspace: need N*N + 2*N, prefer N*N + N + N*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,work,((ir+1)-(1))+ _work_offset,ldwrkr);
com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Bidiagonalize R in WORK(IR)
*                    (Workspace: need N*N + 4*N, prefer N*N + 3*N + 2*N*NB)
*
***/
iwork = (itaup+n);
/***
*
*                    Generate left vectors bidiagonalizing R in WORK(IR)
*                    (Workspace: need N*N + 4*N, prefer N*N + 3*N + N*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,work,(ir-(1))+ _work_offset,ldwrkr,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",n,n,n,work,(ir-(1))+ _work_offset,ldwrkr,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of R in WORK(IR)
*                    (Workspace: need N*N + BDSPAC)
*
***/
iwork = (ie+n);
/***
*
*                    Multiply Q in A by left singular vectors of R in
*                    WORK(IR), storing result in U
*                    (Workspace: need N*N)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,0,n,0,s,_s_offset,work,(ie-(1))+ _work_offset,dum,0,1,work,(ir-(1))+ _work_offset,ldwrkr,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,n,1.0,a,_a_offset,lda,work,(ir-(1))+ _work_offset,ldwrkr,0.0,u,_u_offset,ldu);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=Q*R, copying result to U
*                    (Workspace: need 2*N, prefer N + N*NB)
*
***/
iwork = (itau+n);
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in U
*                    (Workspace: need 2*N, prefer N + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,n,a,_a_offset,lda,u,_u_offset,ldu);
com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,n,u,_u_offset,ldu,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Zero out below R in A
*
***/
iwork = (itaup+n);
if ((n > 1))  {
    com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,a,(2-(1))+(1-(1)) * (lda)+ _a_offset,lda);
/***
*
*                    Bidiagonalize R in A
*                    (Workspace: need 4*N, prefer 3*N + 2*N*NB)
*
***/
}
/***
*
*                    Multiply Q in U by left vectors bidiagonalizing R
*                    (Workspace: need 3*N + M, prefer 3*N + M*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dormbr.dormbr("Q","R","N",m,n,n,a,_a_offset,lda,work,(itauq-(1))+ _work_offset,u,_u_offset,ldu,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of A in U
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+n);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,0,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,dum,0,1,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
/***
*
*                 Path 5 (M much larger than N, JOBU='S', JOBVT='O')
*                 N left singular vectors to be computed in U and
*                 N right singular vectors to be overwritten on A
*
***/
}
else if (wntvo)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= (((2*n)*n)+Math.max((4*n), bdspac))))  {
    iu = 1;
/***
*
*                       WORK(IU) is LDA by N and WORK(IR) is LDA by N
*
***/
if ((lwork >= (wrkbl+((2*lda)*n))))  {
    ldwrku = lda;
ir = (iu+(ldwrku*n));
ldwrkr = lda;
/***
*
*                       WORK(IU) is LDA by N and WORK(IR) is N by N
*
***/
}
else if ((lwork >= (wrkbl+(((lda+n))*n))))  {
    ldwrku = lda;
ir = (iu+(ldwrku*n));
ldwrkr = n;
/***
*
*                       WORK(IU) is N by N and WORK(IR) is N by N
*
***/
}              // Close else if()
else  {
  ldwrku = n;
ir = (iu+(ldwrku*n));
ldwrkr = n;
}              //  Close else.
itau = (ir+(ldwrkr*n));
/***
*
*                    Compute A=Q*R
*                    (Workspace: need 2*N*N + 2*N, prefer 2*N*N + N + N*NB)
*
***/
iwork = (itau+n);
/***
*
*                    Copy R to WORK(IU), zeroing out below it
*
***/
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,work,(iu-(1))+ _work_offset,ldwrku);
/***
*
*                    Generate Q in A
*                    (Workspace: need 2*N*N + 2*N, prefer 2*N*N + N + N*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,work,((iu+1)-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Bidiagonalize R in WORK(IU), copying result to
*                    WORK(IR)
*                    (Workspace: need 2*N*N + 4*N,
*                                prefer 2*N*N+3*N+2*N*NB)
*
***/
iwork = (itaup+n);
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,work,(iu-(1))+ _work_offset,ldwrku,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate left bidiagonalizing vectors in WORK(IU)
*                    (Workspace: need 2*N*N + 4*N, prefer 2*N*N + 3*N + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,work,(iu-(1))+ _work_offset,ldwrku,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                    Generate right bidiagonalizing vectors in WORK(IR)
*                    (Workspace: need 2*N*N + 4*N-1,
*                                prefer 2*N*N+3*N+(N-1)*NB)
*
***/
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",n,n,n,work,(iu-(1))+ _work_offset,ldwrku,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,work,(ir-(1))+ _work_offset,ldwrkr,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of R in WORK(IU) and computing
*                    right singular vectors of R in WORK(IR)
*                    (Workspace: need 2*N*N + BDSPAC)
*
***/
iwork = (ie+n);
/***
*
*                    Multiply Q in A by left singular vectors of R in
*                    WORK(IU), storing result in U
*                    (Workspace: need N*N)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,n,n,0,s,_s_offset,work,(ie-(1))+ _work_offset,work,(ir-(1))+ _work_offset,ldwrkr,work,(iu-(1))+ _work_offset,ldwrku,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                    Copy right singular vectors of R to A
*                    (Workspace: need N*N)
*
***/
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,n,1.0,a,_a_offset,lda,work,(iu-(1))+ _work_offset,ldwrku,0.0,u,_u_offset,ldu);
// *
com.polytechnik.lapack310.Dlacpy.dlacpy("F",n,n,work,(ir-(1))+ _work_offset,ldwrkr,a,_a_offset,lda);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=Q*R, copying result to U
*                    (Workspace: need 2*N, prefer N + N*NB)
*
***/
iwork = (itau+n);
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in U
*                    (Workspace: need 2*N, prefer N + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,n,a,_a_offset,lda,u,_u_offset,ldu);
com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,n,u,_u_offset,ldu,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Zero out below R in A
*
***/
iwork = (itaup+n);
if ((n > 1))  {
    com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,a,(2-(1))+(1-(1)) * (lda)+ _a_offset,lda);
/***
*
*                    Bidiagonalize R in A
*                    (Workspace: need 4*N, prefer 3*N + 2*N*NB)
*
***/
}
/***
*
*                    Multiply Q in U by left vectors bidiagonalizing R
*                    (Workspace: need 3*N + M, prefer 3*N + M*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate right vectors bidiagonalizing R in A
*                    (Workspace: need 4*N-1, prefer 3*N + (N-1)*NB)
*
***/
com.polytechnik.lapack310.Dormbr.dormbr("Q","R","N",m,n,n,a,_a_offset,lda,work,(itauq-(1))+ _work_offset,u,_u_offset,ldu,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,a,_a_offset,lda,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of A in U and computing right
*                    singular vectors of A in A
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+n);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,n,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,a,_a_offset,lda,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
/***
*
*                 Path 6 (M much larger than N, JOBU='S', JOBVT='S'
*                         or 'A')
*                 N left singular vectors to be computed in U and
*                 N right singular vectors to be computed in VT
*
***/
}              // Close else if()
else if (wntvas)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((n*n)+Math.max((4*n), bdspac))))  {
    iu = 1;
/***
*
*                       WORK(IU) is LDA by N
*
***/
if ((lwork >= (wrkbl+(lda*n))))  {
    ldwrku = lda;
/***
*
*                       WORK(IU) is N by N
*
***/
}
else  {
  ldwrku = n;
}              //  Close else.
itau = (iu+(ldwrku*n));
/***
*
*                    Compute A=Q*R
*                    (Workspace: need N*N + 2*N, prefer N*N + N + N*NB)
*
***/
iwork = (itau+n);
/***
*
*                    Copy R to WORK(IU), zeroing out below it
*
***/
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,work,(iu-(1))+ _work_offset,ldwrku);
/***
*
*                    Generate Q in A
*                    (Workspace: need N*N + 2*N, prefer N*N + N + N*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,work,((iu+1)-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Bidiagonalize R in WORK(IU), copying result to VT
*                    (Workspace: need N*N + 4*N, prefer N*N + 3*N + 2*N*NB)
*
***/
iwork = (itaup+n);
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,work,(iu-(1))+ _work_offset,ldwrku,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate left bidiagonalizing vectors in WORK(IU)
*                    (Workspace: need N*N + 4*N, prefer N*N + 3*N + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,work,(iu-(1))+ _work_offset,ldwrku,vt,_vt_offset,ldvt);
/***
*
*                    Generate right bidiagonalizing vectors in VT
*                    (Workspace: need N*N + 4*N-1,
*                                prefer N*N+3*N+(N-1)*NB)
*
***/
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",n,n,n,work,(iu-(1))+ _work_offset,ldwrku,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,vt,_vt_offset,ldvt,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of R in WORK(IU) and computing
*                    right singular vectors of R in VT
*                    (Workspace: need N*N + BDSPAC)
*
***/
iwork = (ie+n);
/***
*
*                    Multiply Q in A by left singular vectors of R in
*                    WORK(IU), storing result in U
*                    (Workspace: need N*N)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,n,n,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,work,(iu-(1))+ _work_offset,ldwrku,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,n,1.0,a,_a_offset,lda,work,(iu-(1))+ _work_offset,ldwrku,0.0,u,_u_offset,ldu);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=Q*R, copying result to U
*                    (Workspace: need 2*N, prefer N + N*NB)
*
***/
iwork = (itau+n);
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in U
*                    (Workspace: need 2*N, prefer N + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,n,a,_a_offset,lda,u,_u_offset,ldu);
/***
*
*                    Copy R to VT, zeroing out below it
*
***/
com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,n,u,_u_offset,ldu,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
if ((n > 1)) {
    com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,vt,(2-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt);
}
    ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Bidiagonalize R in VT
*                    (Workspace: need 4*N, prefer 3*N + 2*N*NB)
*
***/
iwork = (itaup+n);
/***
*
*                    Multiply Q in U by left bidiagonalizing vectors
*                    in VT
*                    (Workspace: need 3*N + M, prefer 3*N + M*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,vt,_vt_offset,ldvt,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate right bidiagonalizing vectors in VT
*                    (Workspace: need 4*N-1, prefer 3*N + (N-1)*NB)
*
***/
com.polytechnik.lapack310.Dormbr.dormbr("Q","R","N",m,n,n,vt,_vt_offset,ldvt,work,(itauq-(1))+ _work_offset,u,_u_offset,ldu,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,vt,_vt_offset,ldvt,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of A in U and computing right
*                    singular vectors of A in VT
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+n);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,n,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
// *
}              // Close else if()
// *
}              // Close else if()
else if (wntua)  {
    /***
*
*                 Path 7 (M much larger than N, JOBU='A', JOBVT='N')
*                 M left singular vectors to be computed in U and
*                 no right singular vectors to be computed
*
***/
if (wntvn)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((n*n)+Util.max((n+m), (4*n), bdspac))))  {
    ir = 1;
/***
*
*                       WORK(IR) is LDA by N
*
***/
if ((lwork >= (wrkbl+(lda*n))))  {
    ldwrkr = lda;
/***
*
*                       WORK(IR) is N by N
*
***/
}
else  {
  ldwrkr = n;
}              //  Close else.
itau = (ir+(ldwrkr*n));
/***
*
*                    Compute A=Q*R, copying result to U
*                    (Workspace: need N*N + 2*N, prefer N*N + N + N*NB)
*
***/
iwork = (itau+n);
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Copy R to WORK(IR), zeroing out below it
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,n,a,_a_offset,lda,u,_u_offset,ldu);
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                    Generate Q in U
*                    (Workspace: need N*N + N + M, prefer N*N + N + M*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,work,((ir+1)-(1))+ _work_offset,ldwrkr);
com.polytechnik.lapack310.Dorgqr.dorgqr(m,m,n,u,_u_offset,ldu,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Bidiagonalize R in WORK(IR)
*                    (Workspace: need N*N + 4*N, prefer N*N + 3*N + 2*N*NB)
*
***/
iwork = (itaup+n);
/***
*
*                    Generate left bidiagonalizing vectors in WORK(IR)
*                    (Workspace: need N*N + 4*N, prefer N*N + 3*N + N*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,work,(ir-(1))+ _work_offset,ldwrkr,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",n,n,n,work,(ir-(1))+ _work_offset,ldwrkr,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of R in WORK(IR)
*                    (Workspace: need N*N + BDSPAC)
*
***/
iwork = (ie+n);
/***
*
*                    Multiply Q in U by left singular vectors of R in
*                    WORK(IR), storing result in A
*                    (Workspace: need N*N)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,0,n,0,s,_s_offset,work,(ie-(1))+ _work_offset,dum,0,1,work,(ir-(1))+ _work_offset,ldwrkr,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                    Copy left singular vectors of A from A to U
*
***/
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,n,1.0,u,_u_offset,ldu,work,(ir-(1))+ _work_offset,ldwrkr,0.0,a,_a_offset,lda);
// *
com.polytechnik.lapack310.Dlacpy.dlacpy("F",m,n,a,_a_offset,lda,u,_u_offset,ldu);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=Q*R, copying result to U
*                    (Workspace: need 2*N, prefer N + N*NB)
*
***/
iwork = (itau+n);
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in U
*                    (Workspace: need N + M, prefer N + M*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,n,a,_a_offset,lda,u,_u_offset,ldu);
com.polytechnik.lapack310.Dorgqr.dorgqr(m,m,n,u,_u_offset,ldu,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Zero out below R in A
*
***/
iwork = (itaup+n);
if ((n > 1))  {
    com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,a,(2-(1))+(1-(1)) * (lda)+ _a_offset,lda);
/***
*
*                    Bidiagonalize R in A
*                    (Workspace: need 4*N, prefer 3*N + 2*N*NB)
*
***/
}
/***
*
*                    Multiply Q in U by left bidiagonalizing vectors
*                    in A
*                    (Workspace: need 3*N + M, prefer 3*N + M*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dormbr.dormbr("Q","R","N",m,n,n,a,_a_offset,lda,work,(itauq-(1))+ _work_offset,u,_u_offset,ldu,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of A in U
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+n);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,0,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,dum,0,1,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
/***
*
*                 Path 8 (M much larger than N, JOBU='A', JOBVT='O')
*                 M left singular vectors to be computed in U and
*                 N right singular vectors to be overwritten on A
*
***/
}
else if (wntvo)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= (((2*n)*n)+Util.max((n+m), (4*n), bdspac))))  {
    iu = 1;
/***
*
*                       WORK(IU) is LDA by N and WORK(IR) is LDA by N
*
***/
if ((lwork >= (wrkbl+((2*lda)*n))))  {
    ldwrku = lda;
ir = (iu+(ldwrku*n));
ldwrkr = lda;
/***
*
*                       WORK(IU) is LDA by N and WORK(IR) is N by N
*
***/
}
else if ((lwork >= (wrkbl+(((lda+n))*n))))  {
    ldwrku = lda;
ir = (iu+(ldwrku*n));
ldwrkr = n;
/***
*
*                       WORK(IU) is N by N and WORK(IR) is N by N
*
***/
}              // Close else if()
else  {
  ldwrku = n;
ir = (iu+(ldwrku*n));
ldwrkr = n;
}              //  Close else.
itau = (ir+(ldwrkr*n));
/***
*
*                    Compute A=Q*R, copying result to U
*                    (Workspace: need 2*N*N + 2*N, prefer 2*N*N + N + N*NB)
*
***/
iwork = (itau+n);
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in U
*                    (Workspace: need 2*N*N + N + M, prefer 2*N*N + N + M*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,n,a,_a_offset,lda,u,_u_offset,ldu);
/***
*
*                    Copy R to WORK(IU), zeroing out below it
*
***/
com.polytechnik.lapack310.Dorgqr.dorgqr(m,m,n,u,_u_offset,ldu,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,work,(iu-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,work,((iu+1)-(1))+ _work_offset,ldwrku);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Bidiagonalize R in WORK(IU), copying result to
*                    WORK(IR)
*                    (Workspace: need 2*N*N + 4*N,
*                                prefer 2*N*N+3*N+2*N*NB)
*
***/
iwork = (itaup+n);
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,work,(iu-(1))+ _work_offset,ldwrku,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate left bidiagonalizing vectors in WORK(IU)
*                    (Workspace: need 2*N*N + 4*N, prefer 2*N*N + 3*N + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,work,(iu-(1))+ _work_offset,ldwrku,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                    Generate right bidiagonalizing vectors in WORK(IR)
*                    (Workspace: need 2*N*N + 4*N-1,
*                                prefer 2*N*N+3*N+(N-1)*NB)
*
***/
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",n,n,n,work,(iu-(1))+ _work_offset,ldwrku,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,work,(ir-(1))+ _work_offset,ldwrkr,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of R in WORK(IU) and computing
*                    right singular vectors of R in WORK(IR)
*                    (Workspace: need 2*N*N + BDSPAC)
*
***/
iwork = (ie+n);
/***
*
*                    Multiply Q in U by left singular vectors of R in
*                    WORK(IU), storing result in A
*                    (Workspace: need N*N)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,n,n,0,s,_s_offset,work,(ie-(1))+ _work_offset,work,(ir-(1))+ _work_offset,ldwrkr,work,(iu-(1))+ _work_offset,ldwrku,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                    Copy left singular vectors of A from A to U
*
***/
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,n,1.0,u,_u_offset,ldu,work,(iu-(1))+ _work_offset,ldwrku,0.0,a,_a_offset,lda);
/***
*
*                    Copy right singular vectors of R from WORK(IR) to A
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("F",m,n,a,_a_offset,lda,u,_u_offset,ldu);
// *
com.polytechnik.lapack310.Dlacpy.dlacpy("F",n,n,work,(ir-(1))+ _work_offset,ldwrkr,a,_a_offset,lda);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=Q*R, copying result to U
*                    (Workspace: need 2*N, prefer N + N*NB)
*
***/
iwork = (itau+n);
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in U
*                    (Workspace: need N + M, prefer N + M*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,n,a,_a_offset,lda,u,_u_offset,ldu);
com.polytechnik.lapack310.Dorgqr.dorgqr(m,m,n,u,_u_offset,ldu,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Zero out below R in A
*
***/
iwork = (itaup+n);
if ((n > 1))  {
    com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,a,(2-(1))+(1-(1)) * (lda)+ _a_offset,lda);
/***
*
*                    Bidiagonalize R in A
*                    (Workspace: need 4*N, prefer 3*N + 2*N*NB)
*
***/
}
/***
*
*                    Multiply Q in U by left bidiagonalizing vectors
*                    in A
*                    (Workspace: need 3*N + M, prefer 3*N + M*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate right bidiagonalizing vectors in A
*                    (Workspace: need 4*N-1, prefer 3*N + (N-1)*NB)
*
***/
com.polytechnik.lapack310.Dormbr.dormbr("Q","R","N",m,n,n,a,_a_offset,lda,work,(itauq-(1))+ _work_offset,u,_u_offset,ldu,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,a,_a_offset,lda,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of A in U and computing right
*                    singular vectors of A in A
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+n);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,n,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,a,_a_offset,lda,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
/***
*
*                 Path 9 (M much larger than N, JOBU='A', JOBVT='S'
*                         or 'A')
*                 M left singular vectors to be computed in U and
*                 N right singular vectors to be computed in VT
*
***/
}              // Close else if()
else if (wntvas)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((n*n)+Util.max((n+m), (4*n), bdspac))))  {
    iu = 1;
/***
*
*                       WORK(IU) is LDA by N
*
***/
if ((lwork >= (wrkbl+(lda*n))))  {
    ldwrku = lda;
/***
*
*                       WORK(IU) is N by N
*
***/
}
else  {
  ldwrku = n;
}              //  Close else.
itau = (iu+(ldwrku*n));
/***
*
*                    Compute A=Q*R, copying result to U
*                    (Workspace: need N*N + 2*N, prefer N*N + N + N*NB)
*
***/
iwork = (itau+n);
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in U
*                    (Workspace: need N*N + N + M, prefer N*N + N + M*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,n,a,_a_offset,lda,u,_u_offset,ldu);
/***
*
*                    Copy R to WORK(IU), zeroing out below it
*
***/
com.polytechnik.lapack310.Dorgqr.dorgqr(m,m,n,u,_u_offset,ldu,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,work,(iu-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,work,((iu+1)-(1))+ _work_offset,ldwrku);
ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Bidiagonalize R in WORK(IU), copying result to VT
*                    (Workspace: need N*N + 4*N, prefer N*N + 3*N + 2*N*NB)
*
***/
iwork = (itaup+n);
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,work,(iu-(1))+ _work_offset,ldwrku,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate left bidiagonalizing vectors in WORK(IU)
*                    (Workspace: need N*N + 4*N, prefer N*N + 3*N + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,work,(iu-(1))+ _work_offset,ldwrku,vt,_vt_offset,ldvt);
/***
*
*                    Generate right bidiagonalizing vectors in VT
*                    (Workspace: need N*N + 4*N-1,
*                                prefer N*N+3*N+(N-1)*NB)
*
***/
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",n,n,n,work,(iu-(1))+ _work_offset,ldwrku,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,vt,_vt_offset,ldvt,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of R in WORK(IU) and computing
*                    right singular vectors of R in VT
*                    (Workspace: need N*N + BDSPAC)
*
***/
iwork = (ie+n);
/***
*
*                    Multiply Q in U by left singular vectors of R in
*                    WORK(IU), storing result in A
*                    (Workspace: need N*N)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,n,n,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,work,(iu-(1))+ _work_offset,ldwrku,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                    Copy left singular vectors of A from A to U
*
***/
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,n,1.0,u,_u_offset,ldu,work,(iu-(1))+ _work_offset,ldwrku,0.0,a,_a_offset,lda);
// *
com.polytechnik.lapack310.Dlacpy.dlacpy("F",m,n,a,_a_offset,lda,u,_u_offset,ldu);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=Q*R, copying result to U
*                    (Workspace: need 2*N, prefer N + N*NB)
*
***/
iwork = (itau+n);
com.polytechnik.lapack310.Dgeqrf.dgeqrf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in U
*                    (Workspace: need N + M, prefer N + M*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,n,a,_a_offset,lda,u,_u_offset,ldu);
/***
*
*                    Copy R from A to VT, zeroing out below it
*
***/
com.polytechnik.lapack310.Dorgqr.dorgqr(m,m,n,u,_u_offset,ldu,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
if ((n > 1)) {
    com.polytechnik.lapack310.Dlaset.dlaset("L",(n-1),(n-1),0.0,0.0,vt,(2-(1))+(1-(1)) * (ldvt)+ _vt_offset,ldvt);
}
    ie = itau;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*                    Bidiagonalize R in VT
*                    (Workspace: need 4*N, prefer 3*N + 2*N*NB)
*
***/
iwork = (itaup+n);
/***
*
*                    Multiply Q in U by left bidiagonalizing vectors
*                    in VT
*                    (Workspace: need 3*N + M, prefer 3*N + M*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(n,n,vt,_vt_offset,ldvt,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate right bidiagonalizing vectors in VT
*                    (Workspace: need 4*N-1, prefer 3*N + (N-1)*NB)
*
***/
com.polytechnik.lapack310.Dormbr.dormbr("Q","R","N",m,n,n,vt,_vt_offset,ldvt,work,(itauq-(1))+ _work_offset,u,_u_offset,ldu,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,vt,_vt_offset,ldvt,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of A in U and computing right
*                    singular vectors of A in VT
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+n);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,n,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
// *
}              // Close else if()
// *
}              // Close else if()
/***
*
*           M .LT. MNTHR
*
*           Path 10 (M at least N, but not much larger)
*           Reduce to bidiagonal form without QR decomposition
*
***/
}
else  {
  ie = 1;
itauq = (ie+n);
itaup = (itauq+n);
/***
*
*           Bidiagonalize A
*           (Workspace: need 3*N + M, prefer 3*N + (M + N)*NB)
*
***/
iwork = (itaup+n);
com.polytechnik.lapack310.Dgebrd.dgebrd(m,n,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*              If left singular vectors desired in U, copy result to U
*              and generate left bidiagonalizing vectors in U
*              (Workspace: need 3*N + NCU, prefer 3*N + NCU*NB)
*
***/
if (wntuas)  {
    com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,n,a,_a_offset,lda,u,_u_offset,ldu);
if (wntus) {
    ncu = n;
}
    if (wntua) {
    ncu = m;
}
    com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,ncu,n,u,_u_offset,ldu,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
}
/***
*
*              If right singular vectors desired in VT, copy result to
*              VT and generate right bidiagonalizing vectors in VT
*              (Workspace: need 4*N-1, prefer 3*N + (N-1)*NB)
*
***/
if (wntvas)  {
    com.polytechnik.lapack310.Dlacpy.dlacpy("U",n,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,vt,_vt_offset,ldvt,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
}
/***
*
*              If left singular vectors desired in A, generate left
*              bidiagonalizing vectors in A
*              (Workspace: need 4*N, prefer 3*N + N*NB)
*
***/
if (wntuo)  {
    com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,n,n,a,_a_offset,lda,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
}
/***
*
*              If right singular vectors desired in A, generate right
*              bidiagonalizing vectors in A
*              (Workspace: need 4*N-1, prefer 3*N + (N-1)*NB)
*
***/
if (wntvo)  {
    com.polytechnik.lapack310.Dorgbr.dorgbr("P",n,n,n,a,_a_offset,lda,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
}
iwork = (ie+n);
if ((wntuas || wntuo)) {
    nru = m;
}
    if (wntun) {
    nru = 0;
}
    if ((wntvas || wntvo)) {
    ncvt = n;
}
    if (wntvn) {
    ncvt = 0;
}
    /***
*
*              Perform bidiagonal QR iteration, if desired, computing
*              left singular vectors in U and computing right singular
*              vectors in VT
*              (Workspace: need BDSPAC)
*
***/
if ((((!wntuo)) && ((!wntvo))))  {
    com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,ncvt,nru,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*              Perform bidiagonal QR iteration, if desired, computing
*              left singular vectors in U and computing right singular
*              vectors in A
*              (Workspace: need BDSPAC)
*
***/
}
else if ((((!wntuo)) && wntvo))  {
    com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,ncvt,nru,0,s,_s_offset,work,(ie-(1))+ _work_offset,a,_a_offset,lda,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*              Perform bidiagonal QR iteration, if desired, computing
*              left singular vectors in A and computing right singular
*              vectors in VT
*              (Workspace: need BDSPAC)
*
***/
}              // Close else if()
else  {
  com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",n,ncvt,nru,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,a,_a_offset,lda,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
// *
}              //  Close else.
/***
*
*        A has more columns than rows. If A has sufficiently more
*        columns than rows, first reduce using the LQ decomposition (if
*        sufficient workspace available)
*
***/
}
else  {
  // *
if ((n >= mnthr))  {
    /***
*
*              Path 1t(N much larger than M, JOBVT='N')
*              No right singular vectors to be computed
*
***/
if (wntvn)  {
    itau = 1;
/***
*
*              Compute A=L*Q
*              (Workspace: need 2*M, prefer M + M*NB)
*
***/
iwork = (itau+m);
/***
*
*              Zero out above L
*
***/
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,a,(1-(1))+(2-(1)) * (lda)+ _a_offset,lda);
ie = 1;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*              Bidiagonalize L in A
*              (Workspace: need 4*M, prefer 3*M + 2*M*NB)
*
***/
iwork = (itaup+m);
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 If left singular vectors desired, generate Q
*                 (Workspace: need 4*M, prefer 3*M + M*NB)
*
***/
if ((wntuo || wntuas))  {
    com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,a,_a_offset,lda,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
}
iwork = (ie+m);
nru = 0;
/***
*
*              Perform bidiagonal QR iteration, computing left singular
*              vectors of A in A if desired
*              (Workspace: need BDSPAC)
*
***/
if ((wntuo || wntuas)) {
    nru = m;
}
    /***
*
*              If left singular vectors desired in U, copy them there
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,0,nru,0,s,_s_offset,work,(ie-(1))+ _work_offset,dum,0,1,a,_a_offset,lda,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
if (wntuas) {
    com.polytechnik.lapack310.Dlacpy.dlacpy("F",m,m,a,_a_offset,lda,u,_u_offset,ldu);
}
    /***
*
*              Path 2t(N much larger than M, JOBU='N', JOBVT='O')
*              M right singular vectors to be overwritten on A and
*              no left singular vectors to be computed
*
***/
}
else if ((wntvo && wntun))  {
    /***
*
*                 Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((m*m)+Math.max((4*m), bdspac))))  {
    ir = 1;
/***
*
*                    WORK(IU) is LDA by N and WORK(IR) is LDA by M
*
***/
if ((lwork >= (Math.max(wrkbl, ((lda*n)+m))+(lda*m))))  {
    ldwrku = lda;
chunk = n;
ldwrkr = lda;
/***
*
*                    WORK(IU) is LDA by N and WORK(IR) is M by M
*
***/
}
else if ((lwork >= (Math.max(wrkbl, ((lda*n)+m))+(m*m))))  {
    ldwrku = lda;
chunk = n;
ldwrkr = m;
/***
*
*                    WORK(IU) is M by CHUNK and WORK(IR) is M by M
*
***/
}              // Close else if()
else  {
  ldwrku = m;
chunk = ((((lwork-(m*m))-m))/m);
ldwrkr = m;
}              //  Close else.
itau = (ir+(ldwrkr*m));
/***
*
*                 Compute A=L*Q
*                 (Workspace: need M*M + 2*M, prefer M*M + M + M*NB)
*
***/
iwork = (itau+m);
/***
*
*                 Copy L to WORK(IR) and zero out above it
*
***/
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                 Generate Q in A
*                 (Workspace: need M*M + 2*M, prefer M*M + M + M*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,work,((ir+ldwrkr)-(1))+ _work_offset,ldwrkr);
com.polytechnik.lapack310.Dorglq.dorglq(m,n,m,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                 Bidiagonalize L in WORK(IR)
*                 (Workspace: need M*M + 4*M, prefer M*M + 3*M + 2*M*NB)
*
***/
iwork = (itaup+m);
/***
*
*                 Generate right vectors bidiagonalizing L
*                 (Workspace: need M*M + 4*M-1, prefer M*M + 3*M + (M-1)*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,work,(ir-(1))+ _work_offset,ldwrkr,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,m,m,work,(ir-(1))+ _work_offset,ldwrkr,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Perform bidiagonal QR iteration, computing right
*                 singular vectors of L in WORK(IR)
*                 (Workspace: need M*M + BDSPAC)
*
***/
iwork = (ie+m);
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,m,0,0,s,_s_offset,work,(ie-(1))+ _work_offset,work,(ir-(1))+ _work_offset,ldwrkr,dum,0,1,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                 Multiply right singular vectors of L in WORK(IR) by Q
*                 in A, storing result in WORK(IU) and copying to A
*                 (Workspace: need M*M + 2*M, prefer M*M + M*N + M)
*
***/
iu = (ie+m);
{
int _i_inc = chunk;
for (i = 1; (_i_inc < 0) ? i >= n : i <= n; i += _i_inc) {
blk = Math.min(((n-i)+1), chunk);
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,blk,m,1.0,work,(ir-(1))+ _work_offset,ldwrkr,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,lda,0.0,work,(iu-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dlacpy.dlacpy("F",m,blk,work,(iu-(1))+ _work_offset,ldwrku,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,lda);
// *
//Dummy.label("com/polytechnik/lapack310/Dgesvd",30);
}              //  Close for() loop.
}
/***
*
*                 Insufficient workspace for a fast algorithm
*
***/
}
else  {
  ie = 1;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                 Bidiagonalize A
*                 (Workspace: need 3*M + N, prefer 3*M + (M + N)*NB)
*
***/
iwork = (itaup+m);
/***
*
*                 Generate right vectors bidiagonalizing A
*                 (Workspace: need 4*M, prefer 3*M + M*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,n,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,n,m,a,_a_offset,lda,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Perform bidiagonal QR iteration, computing right
*                 singular vectors of A in A
*                 (Workspace: need BDSPAC)
*
***/
iwork = (ie+m);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("L",m,n,0,0,s,_s_offset,work,(ie-(1))+ _work_offset,a,_a_offset,lda,dum,0,1,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
/***
*
*              Path 3t(N much larger than M, JOBU='S' or 'A', JOBVT='O')
*              M right singular vectors to be overwritten on A and
*              M left singular vectors to be computed in U
*
***/
}              // Close else if()
else if ((wntvo && wntuas))  {
    /***
*
*                 Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((m*m)+Math.max((4*m), bdspac))))  {
    ir = 1;
/***
*
*                    WORK(IU) is LDA by N and WORK(IR) is LDA by M
*
***/
if ((lwork >= (Math.max(wrkbl, ((lda*n)+m))+(lda*m))))  {
    ldwrku = lda;
chunk = n;
ldwrkr = lda;
/***
*
*                    WORK(IU) is LDA by N and WORK(IR) is M by M
*
***/
}
else if ((lwork >= (Math.max(wrkbl, ((lda*n)+m))+(m*m))))  {
    ldwrku = lda;
chunk = n;
ldwrkr = m;
/***
*
*                    WORK(IU) is M by CHUNK and WORK(IR) is M by M
*
***/
}              // Close else if()
else  {
  ldwrku = m;
chunk = ((((lwork-(m*m))-m))/m);
ldwrkr = m;
}              //  Close else.
itau = (ir+(ldwrkr*m));
/***
*
*                 Compute A=L*Q
*                 (Workspace: need M*M + 2*M, prefer M*M + M + M*NB)
*
***/
iwork = (itau+m);
/***
*
*                 Copy L to U, zeroing about above it
*
***/
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,u,_u_offset,ldu);
/***
*
*                 Generate Q in A
*                 (Workspace: need M*M + 2*M, prefer M*M + M + M*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,u,(1-(1))+(2-(1)) * (ldu)+ _u_offset,ldu);
com.polytechnik.lapack310.Dorglq.dorglq(m,n,m,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                 Bidiagonalize L in U, copying result to WORK(IR)
*                 (Workspace: need M*M + 4*M, prefer M*M + 3*M + 2*M*NB)
*
***/
iwork = (itaup+m);
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,u,_u_offset,ldu,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Generate right vectors bidiagonalizing L in WORK(IR)
*                 (Workspace: need M*M + 4*M-1, prefer M*M + 3*M + (M-1)*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",m,m,u,_u_offset,ldu,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                 Generate left vectors bidiagonalizing L in U
*                 (Workspace: need M*M + 4*M, prefer M*M + 3*M + M*NB)
*
***/
com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,m,m,work,(ir-(1))+ _work_offset,ldwrkr,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,u,_u_offset,ldu,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Perform bidiagonal QR iteration, computing left
*                 singular vectors of L in U, and computing right
*                 singular vectors of L in WORK(IR)
*                 (Workspace: need M*M + BDSPAC)
*
***/
iwork = (ie+m);
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,m,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,work,(ir-(1))+ _work_offset,ldwrkr,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                 Multiply right singular vectors of L in WORK(IR) by Q
*                 in A, storing result in WORK(IU) and copying to A
*                 (Workspace: need M*M + 2*M, prefer M*M + M*N + M))
*
***/
iu = (ie+m);
{
int _i_inc = chunk;
for (i = 1; (_i_inc < 0) ? i >= n : i <= n; i += _i_inc) {
blk = Math.min(((n-i)+1), chunk);
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,blk,m,1.0,work,(ir-(1))+ _work_offset,ldwrkr,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,lda,0.0,work,(iu-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dlacpy.dlacpy("F",m,blk,work,(iu-(1))+ _work_offset,ldwrku,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,lda);
// *
//Dummy.label("com/polytechnik/lapack310/Dgesvd",40);
}              //  Close for() loop.
}
/***
*
*                 Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                 Compute A=L*Q
*                 (Workspace: need 2*M, prefer M + M*NB)
*
***/
iwork = (itau+m);
/***
*
*                 Copy L to U, zeroing out above it
*
***/
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,u,_u_offset,ldu);
/***
*
*                 Generate Q in A
*                 (Workspace: need 2*M, prefer M + M*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,u,(1-(1))+(2-(1)) * (ldu)+ _u_offset,ldu);
com.polytechnik.lapack310.Dorglq.dorglq(m,n,m,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                 Bidiagonalize L in U
*                 (Workspace: need 4*M, prefer 3*M + 2*M*NB)
*
***/
iwork = (itaup+m);
/***
*
*                 Multiply right vectors bidiagonalizing L by Q in A
*                 (Workspace: need 3*M + N, prefer 3*M + N*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,u,_u_offset,ldu,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Generate left vectors bidiagonalizing L in U
*                 (Workspace: need 4*M, prefer 3*M + M*NB)
*
***/
com.polytechnik.lapack310.Dormbr.dormbr("P","L","T",m,n,m,u,_u_offset,ldu,work,(itaup-(1))+ _work_offset,a,_a_offset,lda,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,u,_u_offset,ldu,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                 Perform bidiagonal QR iteration, computing left
*                 singular vectors of A in U and computing right
*                 singular vectors of A in A
*                 (Workspace: need BDSPAC)
*
***/
iwork = (ie+m);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,n,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,a,_a_offset,lda,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
// *
}              // Close else if()
else if (wntvs)  {
    /***
*
*                 Path 4t(N much larger than M, JOBU='N', JOBVT='S')
*                 M right singular vectors to be computed in VT and
*                 no left singular vectors to be computed
*
***/
if (wntun)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((m*m)+Math.max((4*m), bdspac))))  {
    ir = 1;
/***
*
*                       WORK(IR) is LDA by M
*
***/
if ((lwork >= (wrkbl+(lda*m))))  {
    ldwrkr = lda;
/***
*
*                       WORK(IR) is M by M
*
***/
}
else  {
  ldwrkr = m;
}              //  Close else.
itau = (ir+(ldwrkr*m));
/***
*
*                    Compute A=L*Q
*                    (Workspace: need M*M + 2*M, prefer M*M + M + M*NB)
*
***/
iwork = (itau+m);
/***
*
*                    Copy L to WORK(IR), zeroing out above it
*
***/
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                    Generate Q in A
*                    (Workspace: need M*M + 2*M, prefer M*M + M + M*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,work,((ir+ldwrkr)-(1))+ _work_offset,ldwrkr);
com.polytechnik.lapack310.Dorglq.dorglq(m,n,m,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Bidiagonalize L in WORK(IR)
*                    (Workspace: need M*M + 4*M, prefer M*M + 3*M + 2*M*NB)
*
***/
iwork = (itaup+m);
/***
*
*                    Generate right vectors bidiagonalizing L in
*                    WORK(IR)
*                    (Workspace: need M*M + 4*M, prefer M*M + 3*M + (M-1)*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,work,(ir-(1))+ _work_offset,ldwrkr,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,m,m,work,(ir-(1))+ _work_offset,ldwrkr,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing right
*                    singular vectors of L in WORK(IR)
*                    (Workspace: need M*M + BDSPAC)
*
***/
iwork = (ie+m);
/***
*
*                    Multiply right singular vectors of L in WORK(IR) by
*                    Q in A, storing result in VT
*                    (Workspace: need M*M)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,m,0,0,s,_s_offset,work,(ie-(1))+ _work_offset,work,(ir-(1))+ _work_offset,ldwrkr,dum,0,1,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,m,1.0,work,(ir-(1))+ _work_offset,ldwrkr,a,_a_offset,lda,0.0,vt,_vt_offset,ldvt);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=L*Q
*                    (Workspace: need 2*M, prefer M + M*NB)
*
***/
iwork = (itau+m);
/***
*
*                    Copy result to VT
*
***/
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in VT
*                    (Workspace: need 2*M, prefer M + M*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
com.polytechnik.lapack310.Dorglq.dorglq(m,n,m,vt,_vt_offset,ldvt,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Zero out above L in A
*
***/
iwork = (itaup+m);
/***
*
*                    Bidiagonalize L in A
*                    (Workspace: need 4*M, prefer 3*M + 2*M*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,a,(1-(1))+(2-(1)) * (lda)+ _a_offset,lda);
/***
*
*                    Multiply right vectors bidiagonalizing L by Q in VT
*                    (Workspace: need 3*M + N, prefer 3*M + N*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dormbr.dormbr("P","L","T",m,n,m,a,_a_offset,lda,work,(itaup-(1))+ _work_offset,vt,_vt_offset,ldvt,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing right
*                    singular vectors of A in VT
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+m);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,n,0,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,dum,0,1,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
/***
*
*                 Path 5t(N much larger than M, JOBU='O', JOBVT='S')
*                 M right singular vectors to be computed in VT and
*                 M left singular vectors to be overwritten on A
*
***/
}
else if (wntuo)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= (((2*m)*m)+Math.max((4*m), bdspac))))  {
    iu = 1;
/***
*
*                       WORK(IU) is LDA by M and WORK(IR) is LDA by M
*
***/
if ((lwork >= (wrkbl+((2*lda)*m))))  {
    ldwrku = lda;
ir = (iu+(ldwrku*m));
ldwrkr = lda;
/***
*
*                       WORK(IU) is LDA by M and WORK(IR) is M by M
*
***/
}
else if ((lwork >= (wrkbl+(((lda+m))*m))))  {
    ldwrku = lda;
ir = (iu+(ldwrku*m));
ldwrkr = m;
/***
*
*                       WORK(IU) is M by M and WORK(IR) is M by M
*
***/
}              // Close else if()
else  {
  ldwrku = m;
ir = (iu+(ldwrku*m));
ldwrkr = m;
}              //  Close else.
itau = (ir+(ldwrkr*m));
/***
*
*                    Compute A=L*Q
*                    (Workspace: need 2*M*M + 2*M, prefer 2*M*M + M + M*NB)
*
***/
iwork = (itau+m);
/***
*
*                    Copy L to WORK(IU), zeroing out below it
*
***/
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,work,(iu-(1))+ _work_offset,ldwrku);
/***
*
*                    Generate Q in A
*                    (Workspace: need 2*M*M + 2*M, prefer 2*M*M + M + M*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,work,((iu+ldwrku)-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dorglq.dorglq(m,n,m,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Bidiagonalize L in WORK(IU), copying result to
*                    WORK(IR)
*                    (Workspace: need 2*M*M + 4*M,
*                                prefer 2*M*M+3*M+2*M*NB)
*
***/
iwork = (itaup+m);
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,work,(iu-(1))+ _work_offset,ldwrku,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate right bidiagonalizing vectors in WORK(IU)
*                    (Workspace: need 2*M*M + 4*M-1,
*                                prefer 2*M*M+3*M+(M-1)*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,work,(iu-(1))+ _work_offset,ldwrku,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                    Generate left bidiagonalizing vectors in WORK(IR)
*                    (Workspace: need 2*M*M + 4*M, prefer 2*M*M + 3*M + M*NB)
*
***/
com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,m,m,work,(iu-(1))+ _work_offset,ldwrku,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,work,(ir-(1))+ _work_offset,ldwrkr,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of L in WORK(IR) and computing
*                    right singular vectors of L in WORK(IU)
*                    (Workspace: need 2*M*M + BDSPAC)
*
***/
iwork = (ie+m);
/***
*
*                    Multiply right singular vectors of L in WORK(IU) by
*                    Q in A, storing result in VT
*                    (Workspace: need M*M)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,m,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,work,(iu-(1))+ _work_offset,ldwrku,work,(ir-(1))+ _work_offset,ldwrkr,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                    Copy left singular vectors of L to A
*                    (Workspace: need M*M)
*
***/
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,m,1.0,work,(iu-(1))+ _work_offset,ldwrku,a,_a_offset,lda,0.0,vt,_vt_offset,ldvt);
// *
com.polytechnik.lapack310.Dlacpy.dlacpy("F",m,m,work,(ir-(1))+ _work_offset,ldwrkr,a,_a_offset,lda);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=L*Q, copying result to VT
*                    (Workspace: need 2*M, prefer M + M*NB)
*
***/
iwork = (itau+m);
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in VT
*                    (Workspace: need 2*M, prefer M + M*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
com.polytechnik.lapack310.Dorglq.dorglq(m,n,m,vt,_vt_offset,ldvt,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Zero out above L in A
*
***/
iwork = (itaup+m);
/***
*
*                    Bidiagonalize L in A
*                    (Workspace: need 4*M, prefer 3*M + 2*M*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,a,(1-(1))+(2-(1)) * (lda)+ _a_offset,lda);
/***
*
*                    Multiply right vectors bidiagonalizing L by Q in VT
*                    (Workspace: need 3*M + N, prefer 3*M + N*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate left bidiagonalizing vectors of L in A
*                    (Workspace: need 4*M, prefer 3*M + M*NB)
*
***/
com.polytechnik.lapack310.Dormbr.dormbr("P","L","T",m,n,m,a,_a_offset,lda,work,(itaup-(1))+ _work_offset,vt,_vt_offset,ldvt,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,a,_a_offset,lda,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, compute left
*                    singular vectors of A in A and compute right
*                    singular vectors of A in VT
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+m);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,n,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,a,_a_offset,lda,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
/***
*
*                 Path 6t(N much larger than M, JOBU='S' or 'A',
*                         JOBVT='S')
*                 M right singular vectors to be computed in VT and
*                 M left singular vectors to be computed in U
*
***/
}              // Close else if()
else if (wntuas)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((m*m)+Math.max((4*m), bdspac))))  {
    iu = 1;
/***
*
*                       WORK(IU) is LDA by N
*
***/
if ((lwork >= (wrkbl+(lda*m))))  {
    ldwrku = lda;
/***
*
*                       WORK(IU) is LDA by M
*
***/
}
else  {
  ldwrku = m;
}              //  Close else.
itau = (iu+(ldwrku*m));
/***
*
*                    Compute A=L*Q
*                    (Workspace: need M*M + 2*M, prefer M*M + M + M*NB)
*
***/
iwork = (itau+m);
/***
*
*                    Copy L to WORK(IU), zeroing out above it
*
***/
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,work,(iu-(1))+ _work_offset,ldwrku);
/***
*
*                    Generate Q in A
*                    (Workspace: need M*M + 2*M, prefer M*M + M + M*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,work,((iu+ldwrku)-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dorglq.dorglq(m,n,m,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Bidiagonalize L in WORK(IU), copying result to U
*                    (Workspace: need M*M + 4*M, prefer M*M + 3*M + 2*M*NB)
*
***/
iwork = (itaup+m);
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,work,(iu-(1))+ _work_offset,ldwrku,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate right bidiagonalizing vectors in WORK(IU)
*                    (Workspace: need M*M + 4*M-1,
*                                prefer M*M+3*M+(M-1)*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,work,(iu-(1))+ _work_offset,ldwrku,u,_u_offset,ldu);
/***
*
*                    Generate left bidiagonalizing vectors in U
*                    (Workspace: need M*M + 4*M, prefer M*M + 3*M + M*NB)
*
***/
com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,m,m,work,(iu-(1))+ _work_offset,ldwrku,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,u,_u_offset,ldu,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of L in U and computing right
*                    singular vectors of L in WORK(IU)
*                    (Workspace: need M*M + BDSPAC)
*
***/
iwork = (ie+m);
/***
*
*                    Multiply right singular vectors of L in WORK(IU) by
*                    Q in A, storing result in VT
*                    (Workspace: need M*M)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,m,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,work,(iu-(1))+ _work_offset,ldwrku,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,m,1.0,work,(iu-(1))+ _work_offset,ldwrku,a,_a_offset,lda,0.0,vt,_vt_offset,ldvt);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=L*Q, copying result to VT
*                    (Workspace: need 2*M, prefer M + M*NB)
*
***/
iwork = (itau+m);
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in VT
*                    (Workspace: need 2*M, prefer M + M*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
/***
*
*                    Copy L to U, zeroing out above it
*
***/
com.polytechnik.lapack310.Dorglq.dorglq(m,n,m,vt,_vt_offset,ldvt,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,u,_u_offset,ldu);
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,u,(1-(1))+(2-(1)) * (ldu)+ _u_offset,ldu);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Bidiagonalize L in U
*                    (Workspace: need 4*M, prefer 3*M + 2*M*NB)
*
***/
iwork = (itaup+m);
/***
*
*                    Multiply right bidiagonalizing vectors in U by Q
*                    in VT
*                    (Workspace: need 3*M + N, prefer 3*M + N*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,u,_u_offset,ldu,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate left bidiagonalizing vectors in U
*                    (Workspace: need 4*M, prefer 3*M + M*NB)
*
***/
com.polytechnik.lapack310.Dormbr.dormbr("P","L","T",m,n,m,u,_u_offset,ldu,work,(itaup-(1))+ _work_offset,vt,_vt_offset,ldvt,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,u,_u_offset,ldu,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of A in U and computing right
*                    singular vectors of A in VT
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+m);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,n,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
// *
}              // Close else if()
// *
}              // Close else if()
else if (wntva)  {
    /***
*
*                 Path 7t(N much larger than M, JOBU='N', JOBVT='A')
*                 N right singular vectors to be computed in VT and
*                 no left singular vectors to be computed
*
***/
if (wntun)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((m*m)+Util.max((n+m), (4*m), bdspac))))  {
    ir = 1;
/***
*
*                       WORK(IR) is LDA by M
*
***/
if ((lwork >= (wrkbl+(lda*m))))  {
    ldwrkr = lda;
/***
*
*                       WORK(IR) is M by M
*
***/
}
else  {
  ldwrkr = m;
}              //  Close else.
itau = (ir+(ldwrkr*m));
/***
*
*                    Compute A=L*Q, copying result to VT
*                    (Workspace: need M*M + 2*M, prefer M*M + M + M*NB)
*
***/
iwork = (itau+m);
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Copy L to WORK(IR), zeroing out above it
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                    Generate Q in VT
*                    (Workspace: need M*M + M + N, prefer M*M + M + N*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,work,((ir+ldwrkr)-(1))+ _work_offset,ldwrkr);
com.polytechnik.lapack310.Dorglq.dorglq(n,n,m,vt,_vt_offset,ldvt,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Bidiagonalize L in WORK(IR)
*                    (Workspace: need M*M + 4*M, prefer M*M + 3*M + 2*M*NB)
*
***/
iwork = (itaup+m);
/***
*
*                    Generate right bidiagonalizing vectors in WORK(IR)
*                    (Workspace: need M*M + 4*M-1,
*                                prefer M*M+3*M+(M-1)*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,work,(ir-(1))+ _work_offset,ldwrkr,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,m,m,work,(ir-(1))+ _work_offset,ldwrkr,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing right
*                    singular vectors of L in WORK(IR)
*                    (Workspace: need M*M + BDSPAC)
*
***/
iwork = (ie+m);
/***
*
*                    Multiply right singular vectors of L in WORK(IR) by
*                    Q in VT, storing result in A
*                    (Workspace: need M*M)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,m,0,0,s,_s_offset,work,(ie-(1))+ _work_offset,work,(ir-(1))+ _work_offset,ldwrkr,dum,0,1,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                    Copy right singular vectors of A from A to VT
*
***/
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,m,1.0,work,(ir-(1))+ _work_offset,ldwrkr,vt,_vt_offset,ldvt,0.0,a,_a_offset,lda);
// *
com.polytechnik.lapack310.Dlacpy.dlacpy("F",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=L*Q, copying result to VT
*                    (Workspace: need 2*M, prefer M + M*NB)
*
***/
iwork = (itau+m);
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in VT
*                    (Workspace: need M + N, prefer M + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
com.polytechnik.lapack310.Dorglq.dorglq(n,n,m,vt,_vt_offset,ldvt,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Zero out above L in A
*
***/
iwork = (itaup+m);
/***
*
*                    Bidiagonalize L in A
*                    (Workspace: need 4*M, prefer 3*M + 2*M*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,a,(1-(1))+(2-(1)) * (lda)+ _a_offset,lda);
/***
*
*                    Multiply right bidiagonalizing vectors in A by Q
*                    in VT
*                    (Workspace: need 3*M + N, prefer 3*M + N*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dormbr.dormbr("P","L","T",m,n,m,a,_a_offset,lda,work,(itaup-(1))+ _work_offset,vt,_vt_offset,ldvt,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing right
*                    singular vectors of A in VT
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+m);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,n,0,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,dum,0,1,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
/***
*
*                 Path 8t(N much larger than M, JOBU='O', JOBVT='A')
*                 N right singular vectors to be computed in VT and
*                 M left singular vectors to be overwritten on A
*
***/
}
else if (wntuo)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= (((2*m)*m)+Util.max((n+m), (4*m), bdspac))))  {
    iu = 1;
/***
*
*                       WORK(IU) is LDA by M and WORK(IR) is LDA by M
*
***/
if ((lwork >= (wrkbl+((2*lda)*m))))  {
    ldwrku = lda;
ir = (iu+(ldwrku*m));
ldwrkr = lda;
/***
*
*                       WORK(IU) is LDA by M and WORK(IR) is M by M
*
***/
}
else if ((lwork >= (wrkbl+(((lda+m))*m))))  {
    ldwrku = lda;
ir = (iu+(ldwrku*m));
ldwrkr = m;
/***
*
*                       WORK(IU) is M by M and WORK(IR) is M by M
*
***/
}              // Close else if()
else  {
  ldwrku = m;
ir = (iu+(ldwrku*m));
ldwrkr = m;
}              //  Close else.
itau = (ir+(ldwrkr*m));
/***
*
*                    Compute A=L*Q, copying result to VT
*                    (Workspace: need 2*M*M + 2*M, prefer 2*M*M + M + M*NB)
*
***/
iwork = (itau+m);
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in VT
*                    (Workspace: need 2*M*M + M + N, prefer 2*M*M + M + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
/***
*
*                    Copy L to WORK(IU), zeroing out above it
*
***/
com.polytechnik.lapack310.Dorglq.dorglq(n,n,m,vt,_vt_offset,ldvt,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,work,(iu-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,work,((iu+ldwrku)-(1))+ _work_offset,ldwrku);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Bidiagonalize L in WORK(IU), copying result to
*                    WORK(IR)
*                    (Workspace: need 2*M*M + 4*M,
*                                prefer 2*M*M+3*M+2*M*NB)
*
***/
iwork = (itaup+m);
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,work,(iu-(1))+ _work_offset,ldwrku,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate right bidiagonalizing vectors in WORK(IU)
*                    (Workspace: need 2*M*M + 4*M-1,
*                                prefer 2*M*M+3*M+(M-1)*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,work,(iu-(1))+ _work_offset,ldwrku,work,(ir-(1))+ _work_offset,ldwrkr);
/***
*
*                    Generate left bidiagonalizing vectors in WORK(IR)
*                    (Workspace: need 2*M*M + 4*M, prefer 2*M*M + 3*M + M*NB)
*
***/
com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,m,m,work,(iu-(1))+ _work_offset,ldwrku,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,work,(ir-(1))+ _work_offset,ldwrkr,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of L in WORK(IR) and computing
*                    right singular vectors of L in WORK(IU)
*                    (Workspace: need 2*M*M + BDSPAC)
*
***/
iwork = (ie+m);
/***
*
*                    Multiply right singular vectors of L in WORK(IU) by
*                    Q in VT, storing result in A
*                    (Workspace: need M*M)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,m,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,work,(iu-(1))+ _work_offset,ldwrku,work,(ir-(1))+ _work_offset,ldwrkr,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                    Copy right singular vectors of A from A to VT
*
***/
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,m,1.0,work,(iu-(1))+ _work_offset,ldwrku,vt,_vt_offset,ldvt,0.0,a,_a_offset,lda);
/***
*
*                    Copy left singular vectors of A from WORK(IR) to A
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("F",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
// *
com.polytechnik.lapack310.Dlacpy.dlacpy("F",m,m,work,(ir-(1))+ _work_offset,ldwrkr,a,_a_offset,lda);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=L*Q, copying result to VT
*                    (Workspace: need 2*M, prefer M + M*NB)
*
***/
iwork = (itau+m);
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in VT
*                    (Workspace: need M + N, prefer M + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
com.polytechnik.lapack310.Dorglq.dorglq(n,n,m,vt,_vt_offset,ldvt,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Zero out above L in A
*
***/
iwork = (itaup+m);
/***
*
*                    Bidiagonalize L in A
*                    (Workspace: need 4*M, prefer 3*M + 2*M*NB)
*
***/
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,a,(1-(1))+(2-(1)) * (lda)+ _a_offset,lda);
/***
*
*                    Multiply right bidiagonalizing vectors in A by Q
*                    in VT
*                    (Workspace: need 3*M + N, prefer 3*M + N*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate left bidiagonalizing vectors in A
*                    (Workspace: need 4*M, prefer 3*M + M*NB)
*
***/
com.polytechnik.lapack310.Dormbr.dormbr("P","L","T",m,n,m,a,_a_offset,lda,work,(itaup-(1))+ _work_offset,vt,_vt_offset,ldvt,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,a,_a_offset,lda,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of A in A and computing right
*                    singular vectors of A in VT
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+m);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,n,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,a,_a_offset,lda,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
/***
*
*                 Path 9t(N much larger than M, JOBU='S' or 'A',
*                         JOBVT='A')
*                 N right singular vectors to be computed in VT and
*                 M left singular vectors to be computed in U
*
***/
}              // Close else if()
else if (wntuas)  {
    /***
*
*                    Sufficient workspace for a fast algorithm
*
***/
if ((lwork >= ((m*m)+Util.max((n+m), (4*m), bdspac))))  {
    iu = 1;
/***
*
*                       WORK(IU) is LDA by M
*
***/
if ((lwork >= (wrkbl+(lda*m))))  {
    ldwrku = lda;
/***
*
*                       WORK(IU) is M by M
*
***/
}
else  {
  ldwrku = m;
}              //  Close else.
itau = (iu+(ldwrku*m));
/***
*
*                    Compute A=L*Q, copying result to VT
*                    (Workspace: need M*M + 2*M, prefer M*M + M + M*NB)
*
***/
iwork = (itau+m);
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in VT
*                    (Workspace: need M*M + M + N, prefer M*M + M + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
/***
*
*                    Copy L to WORK(IU), zeroing out above it
*
***/
com.polytechnik.lapack310.Dorglq.dorglq(n,n,m,vt,_vt_offset,ldvt,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,work,(iu-(1))+ _work_offset,ldwrku);
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,work,((iu+ldwrku)-(1))+ _work_offset,ldwrku);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Bidiagonalize L in WORK(IU), copying result to U
*                    (Workspace: need M*M + 4*M, prefer M*M + 3*M + 2*M*NB)
*
***/
iwork = (itaup+m);
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,work,(iu-(1))+ _work_offset,ldwrku,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate right bidiagonalizing vectors in WORK(IU)
*                    (Workspace: need M*M + 4*M, prefer M*M + 3*M + (M-1)*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,work,(iu-(1))+ _work_offset,ldwrku,u,_u_offset,ldu);
/***
*
*                    Generate left bidiagonalizing vectors in U
*                    (Workspace: need M*M + 4*M, prefer M*M + 3*M + M*NB)
*
***/
com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,m,m,work,(iu-(1))+ _work_offset,ldwrku,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,u,_u_offset,ldu,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of L in U and computing right
*                    singular vectors of L in WORK(IU)
*                    (Workspace: need M*M + BDSPAC)
*
***/
iwork = (ie+m);
/***
*
*                    Multiply right singular vectors of L in WORK(IU) by
*                    Q in VT, storing result in A
*                    (Workspace: need M*M)
*
***/
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,m,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,work,(iu-(1))+ _work_offset,ldwrku,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*                    Copy right singular vectors of A from A to VT
*
***/
com.polytechnik.lapack310.Dgemm.dgemm("N","N",m,n,m,1.0,work,(iu-(1))+ _work_offset,ldwrku,vt,_vt_offset,ldvt,0.0,a,_a_offset,lda);
// *
com.polytechnik.lapack310.Dlacpy.dlacpy("F",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
/***
*
*                    Insufficient workspace for a fast algorithm
*
***/
}
else  {
  itau = 1;
/***
*
*                    Compute A=L*Q, copying result to VT
*                    (Workspace: need 2*M, prefer M + M*NB)
*
***/
iwork = (itau+m);
com.polytechnik.lapack310.Dgelqf.dgelqf(m,n,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate Q in VT
*                    (Workspace: need M + N, prefer M + N*NB)
*
***/
com.polytechnik.lapack310.Dlacpy.dlacpy("U",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
/***
*
*                    Copy L to U, zeroing out above it
*
***/
com.polytechnik.lapack310.Dorglq.dorglq(n,n,m,vt,_vt_offset,ldvt,work,(itau-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,u,_u_offset,ldu);
com.polytechnik.lapack310.Dlaset.dlaset("U",(m-1),(m-1),0.0,0.0,u,(1-(1))+(2-(1)) * (ldu)+ _u_offset,ldu);
ie = itau;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*                    Bidiagonalize L in U
*                    (Workspace: need 4*M, prefer 3*M + 2*M*NB)
*
***/
iwork = (itaup+m);
/***
*
*                    Multiply right bidiagonalizing vectors in U by Q
*                    in VT
*                    (Workspace: need 3*M + N, prefer 3*M + N*NB)
*
***/
com.polytechnik.lapack310.Dgebrd.dgebrd(m,m,u,_u_offset,ldu,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Generate left bidiagonalizing vectors in U
*                    (Workspace: need 4*M, prefer 3*M + M*NB)
*
***/
com.polytechnik.lapack310.Dormbr.dormbr("P","L","T",m,n,m,u,_u_offset,ldu,work,(itaup-(1))+ _work_offset,vt,_vt_offset,ldvt,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,m,u,_u_offset,ldu,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*                    Perform bidiagonal QR iteration, computing left
*                    singular vectors of A in U and computing right
*                    singular vectors of A in VT
*                    (Workspace: need BDSPAC)
*
***/
iwork = (ie+m);
// *
com.polytechnik.lapack310.Dbdsqr.dbdsqr("U",m,n,m,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
// *
}              // Close else if()
// *
}              // Close else if()
/***
*
*           N .LT. MNTHR
*
*           Path 10t(N greater than M, but not much larger)
*           Reduce to bidiagonal form without LQ decomposition
*
***/
}
else  {
  ie = 1;
itauq = (ie+m);
itaup = (itauq+m);
/***
*
*           Bidiagonalize A
*           (Workspace: need 3*M + N, prefer 3*M + (M + N)*NB)
*
***/
iwork = (itaup+m);
com.polytechnik.lapack310.Dgebrd.dgebrd(m,n,a,_a_offset,lda,s,_s_offset,work,(ie-(1))+ _work_offset,work,(itauq-(1))+ _work_offset,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
/***
*
*              If left singular vectors desired in U, copy result to U
*              and generate left bidiagonalizing vectors in U
*              (Workspace: need 4*M-1, prefer 3*M + (M-1)*NB)
*
***/
if (wntuas)  {
    com.polytechnik.lapack310.Dlacpy.dlacpy("L",m,m,a,_a_offset,lda,u,_u_offset,ldu);
com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,n,u,_u_offset,ldu,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
}
/***
*
*              If right singular vectors desired in VT, copy result to
*              VT and generate right bidiagonalizing vectors in VT
*              (Workspace: need 3*M + NRVT, prefer 3*M + NRVT*NB)
*
***/
if (wntvas)  {
    com.polytechnik.lapack310.Dlacpy.dlacpy("U",m,n,a,_a_offset,lda,vt,_vt_offset,ldvt);
if (wntva) {
    nrvt = n;
}
    if (wntvs) {
    nrvt = m;
}
    com.polytechnik.lapack310.Dorgbr.dorgbr("P",nrvt,n,m,vt,_vt_offset,ldvt,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
}
/***
*
*              If left singular vectors desired in A, generate left
*              bidiagonalizing vectors in A
*              (Workspace: need 4*M-1, prefer 3*M + (M-1)*NB)
*
***/
if (wntuo)  {
    com.polytechnik.lapack310.Dorgbr.dorgbr("Q",m,m,n,a,_a_offset,lda,work,(itauq-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
}
/***
*
*              If right singular vectors desired in A, generate right
*              bidiagonalizing vectors in A
*              (Workspace: need 4*M, prefer 3*M + M*NB)
*
***/
if (wntvo)  {
    com.polytechnik.lapack310.Dorgbr.dorgbr("P",m,n,m,a,_a_offset,lda,work,(itaup-(1))+ _work_offset,work,(iwork-(1))+ _work_offset,((lwork-iwork)+1),ierr);
}
iwork = (ie+m);
if ((wntuas || wntuo)) {
    nru = m;
}
    if (wntun) {
    nru = 0;
}
    if ((wntvas || wntvo)) {
    ncvt = n;
}
    if (wntvn) {
    ncvt = 0;
}
    /***
*
*              Perform bidiagonal QR iteration, if desired, computing
*              left singular vectors in U and computing right singular
*              vectors in VT
*              (Workspace: need BDSPAC)
*
***/
if ((((!wntuo)) && ((!wntvo))))  {
    com.polytechnik.lapack310.Dbdsqr.dbdsqr("L",m,ncvt,nru,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*              Perform bidiagonal QR iteration, if desired, computing
*              left singular vectors in U and computing right singular
*              vectors in A
*              (Workspace: need BDSPAC)
*
***/
}
else if ((((!wntuo)) && wntvo))  {
    com.polytechnik.lapack310.Dbdsqr.dbdsqr("L",m,ncvt,nru,0,s,_s_offset,work,(ie-(1))+ _work_offset,a,_a_offset,lda,u,_u_offset,ldu,dum,0,1,work,(iwork-(1))+ _work_offset,info);
/***
*
*              Perform bidiagonal QR iteration, if desired, computing
*              left singular vectors in A and computing right singular
*              vectors in VT
*              (Workspace: need BDSPAC)
*
***/
}              // Close else if()
else  {
  com.polytechnik.lapack310.Dbdsqr.dbdsqr("L",m,ncvt,nru,0,s,_s_offset,work,(ie-(1))+ _work_offset,vt,_vt_offset,ldvt,a,_a_offset,lda,dum,0,1,work,(iwork-(1))+ _work_offset,info);
// *
}              //  Close else.
// *
}              //  Close else.
/***
*
*     If DBDSQR failed to converge, copy unconverged superdiagonals
*     to WORK( 2:MINMN )
*
***/
}              //  Close else.
if ((info.val != 0))  {
    if ((ie > 2))  {
    {
for (i = 1; i <= (minmn-1); i++) {
work[((i+1)-(1))+ _work_offset] = work[(((i+ie)-1)-(1))+ _work_offset];
//Dummy.label("com/polytechnik/lapack310/Dgesvd",50);
}              //  Close for() loop.
}
}
if ((ie < 2))  {
    {
int _i_inc = -1;
for (i = (minmn-1); i >= 1; i += _i_inc) {
work[((i+1)-(1))+ _work_offset] = work[(((i+ie)-1)-(1))+ _work_offset];
//Dummy.label("com/polytechnik/lapack310/Dgesvd",60);
}              //  Close for() loop.
}
}
/***
*
*     Undo scaling if necessary
*
***/
}
if ((iscl == 1))  {
    if ((anrm > bignum)) {
    com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,bignum,anrm,minmn,1,s,_s_offset,minmn,ierr);
}
    if (((info.val != 0) && (anrm > bignum))) {
    com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,bignum,anrm,(minmn-1),1,work,(2-(1))+ _work_offset,minmn,ierr);
}
    if ((anrm < smlnum)) {
    com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,smlnum,anrm,minmn,1,s,_s_offset,minmn,ierr);
}
    if (((info.val != 0) && (anrm < smlnum))) {
    com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,smlnum,anrm,(minmn-1),1,work,(2-(1))+ _work_offset,minmn,ierr);
}
    /***
*
*     Return optimal workspace in WORK(1)
*
***/
}
// *
work[(1-(1))+ _work_offset] = (double)(maxwrk);
/***
*
*     End of DGESVD
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dgesvd",999999);w
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dgesvd",999999);
return;
   }
} // End class.
