/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlaed2.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlaed2 {

// *> \brief \b DLAED2 used by DSTEDC. Merges eigenvalues and deflates secu
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLAED2 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAED2( K, N, N1, D, Q, LDQ, INDXQ, RHO, Z, DLAMDA, W
// *                          Q2, INDX, INDXC, INDXP, COLTYP, INFO )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, K, LDQ, N, N1
// *       DOUBLE PRECISION   RHO
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            COLTYP( * ), INDX( * ), INDXC( * ), INDXP( *
// *      $                   INDXQ( * )
// *       DOUBLE PRECISION   D( * ), DLAMDA( * ), Q( LDQ, * ), Q2( * ),
// *      $                   W( * ), Z( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLAED2 merges the two sets of eigenvalues together into a single
// *> sorted set.  Then it tries to deflate the size of the problem.
// *> There are two ways in which deflation can occur:  when two or more
// *> eigenvalues are close together or if there is a tiny entry in the
// *> Z vector.  For each such occurrence the order of the related secular

// *> equation problem is reduced by one.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[out] K
// *> \verbatim
// *>          K is INTEGER
// *>         The number of non-deflated eigenvalues, and the order of the

// *>         related secular equation. 0 <= K <=N.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>         The dimension of the symmetric tridiagonal matrix.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] N1
// *> \verbatim
// *>          N1 is INTEGER
// *>         The location of the last eigenvalue in the leading sub-matrix
// *>         min(1,N) <= N1 <= N/2.
// *> \endverbatim
// *>
// *> \param[in,out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>         On entry, D contains the eigenvalues of the two submatrices t
// *>         be combined.
// *>         On exit, D contains the trailing (N-K) updated eigenvalues
// *>         (those which were deflated) sorted into increasing order.
// *> \endverbatim
// *>
// *> \param[in,out] Q
// *> \verbatim
// *>          Q is DOUBLE PRECISION array, dimension (LDQ, N)
// *>         On entry, Q contains the eigenvectors of two submatrices in
// *>         the two square blocks with corners at (1,1), (N1,N1)
// *>         and (N1+1, N1+1), (N,N).
// *>         On exit, Q contains the trailing (N-K) updated eigenvectors
// *>         (those which were deflated) in its last N-K columns.
// *> \endverbatim
// *>
// *> \param[in] LDQ
// *> \verbatim
// *>          LDQ is INTEGER
// *>         The leading dimension of the array Q.  LDQ >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in,out] INDXQ
// *> \verbatim
// *>          INDXQ is INTEGER array, dimension (N)
// *>         The permutation which separately sorts the two sub-problems
// *>         in D into ascending order.  Note that elements in the second

// *>         half of this permutation must first have N1 added to their
// *>         values. Destroyed on exit.
// *> \endverbatim
// *>
// *> \param[in,out] RHO
// *> \verbatim
// *>          RHO is DOUBLE PRECISION
// *>         On entry, the off-diagonal element associated with the rank-1
// *>         cut which originally split the two submatrices which are now

// *>         being recombined.
// *>         On exit, RHO has been modified to the value required by
// *>         DLAED3.
// *> \endverbatim
// *>
// *> \param[in] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (N)
// *>         On entry, Z contains the updating vector (the last
// *>         row of the first sub-eigenvector matrix and the first row of

// *>         the second sub-eigenvector matrix).
// *>         On exit, the contents of Z have been destroyed by the updatin
// *>         process.
// *> \endverbatim
// *>
// *> \param[out] DLAMDA
// *> \verbatim
// *>          DLAMDA is DOUBLE PRECISION array, dimension (N)
// *>         A copy of the first K eigenvalues which will be used by
// *>         DLAED3 to form the secular equation.
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>         The first k values of the final deflation-altered z-vector
// *>         which will be passed to DLAED3.
// *> \endverbatim
// *>
// *> \param[out] Q2
// *> \verbatim
// *>          Q2 is DOUBLE PRECISION array, dimension (N1**2+(N-N1)**2)
// *>         A copy of the first K eigenvectors which will be used by
// *>         DLAED3 in a matrix multiply (DGEMM) to solve for the new
// *>         eigenvectors.
// *> \endverbatim
// *>
// *> \param[out] INDX
// *> \verbatim
// *>          INDX is INTEGER array, dimension (N)
// *>         The permutation used to sort the contents of DLAMDA into
// *>         ascending order.
// *> \endverbatim
// *>
// *> \param[out] INDXC
// *> \verbatim
// *>          INDXC is INTEGER array, dimension (N)
// *>         The permutation used to arrange the columns of the deflated
// *>         Q matrix into three groups:  the first group contains non-zer
// *>         elements only at and above N1, the second contains
// *>         non-zero elements only below N1, and the third is dense.
// *> \endverbatim
// *>
// *> \param[out] INDXP
// *> \verbatim
// *>          INDXP is INTEGER array, dimension (N)
// *>         The permutation used to place deflated values of D at the end
// *>         of the array.  INDXP(1:K) points to the nondeflated D-values

// *>         and INDXP(K+1:N) points to the deflated eigenvalues.
// *> \endverbatim
// *>
// *> \param[out] COLTYP
// *> \verbatim
// *>          COLTYP is INTEGER array, dimension (N)
// *>         During execution, a label which will indicate which of the
// *>         following types a column in the Q2 matrix is:
// *>         1 : non-zero in the upper half only;
// *>         2 : dense;
// *>         3 : non-zero in the lower half only;
// *>         4 : deflated.
// *>         On exit, COLTYP(i) is the number of columns of type i,
// *>         for i=1 to 4 only.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit.
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value.

// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Jeff Rutter, Computer Science Division, University of California
// *> at Berkeley, USA \n
// *>  Modified by Francoise Tisseur, University of Tennessee
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/
// *

public static void dlaed2 (intW k,
int n,
int n1,
double [] d, int _d_offset,
double [] q, int _q_offset,
int ldq,
int [] indxq, int _indxq_offset,
doubleW rho,
double [] z, int _z_offset,
double [] dlamda, int _dlamda_offset,
double [] w, int _w_offset,
double [] q2, int _q2_offset,
int [] indx, int _indx_offset,
int [] indxc, int _indxc_offset,
int [] indxp, int _indxp_offset,
int [] coltyp, int _coltyp_offset,
intW info)  {

label_999999:{

int [] ctot= new int[(4)];
int [] psm= new int[(4)];
int ct= 0;
int i= 0;
int imax= 0;
int iq1= 0;
int iq2= 0;
int j= 0;
int jmax= 0;
int js= 0;
int k2= 0;
int n1p1= 0;
int n2= 0;
int nj= 0;
int pj= 0;
double c= 0.0d;
double eps= 0.0d;
double s= 0.0d;
double t= 0.0d;
double tau= 0.0d;
double tol= 0.0d;
info.val = 0;
if ((n < 0))  {
    info.val = -2;
}
else if ((ldq < Math.max(1, n)))  {
    info.val = -6;
}              // Close else if()
else if (((Math.min(1, ((n/2))) > n1) || (((n/2)) < n1)))  {
    info.val = -3;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DLAED2",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlaed2",999999);
/***
*
*     Quick return if possible
*
***/
}
// *
if ((n == 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlaed2",999999);
}
    n2 = (n-n1);
// *
n1p1 = (n1+1);
if ((rho.val < 0.0))  {
    com.polytechnik.lapack310.Dscal.dscal(n2,-1.0,z,(n1p1-(1))+ _z_offset,1);
/***
*
*     Normalize z so that norm(z) = 1.  Since z is the concatenation of
*     two normalized vectors, norm2(z) = sqrt(2).
*
***/
}
t = (1.0/Math.sqrt(2.0));
/***
*
*     RHO = ABS( norm(z)**2 * RHO )
*
***/
com.polytechnik.lapack310.Dscal.dscal(n,t,z,_z_offset,1);
/***
*
*     Sort the eigenvalues into increasing order
*
***/
rho.val = Math.abs((2.0*rho.val));
{
for (i = n1p1; i <= n; i++) {
indxq[(i-(1))+ _indxq_offset] = (indxq[(i-(1))+ _indxq_offset]+n1);
/***
*
*     re-integrate the deflated parts from the last pass
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaed2",10);
}              //  Close for() loop.
}
{
for (i = 1; i <= n; i++) {
dlamda[(i-(1))+ _dlamda_offset] = d[(indxq[(i-(1))+ _indxq_offset]-(1))+ _d_offset];
//Dummy.label("com/polytechnik/lapack310/Dlaed2",20);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dlamrg.dlamrg(n1,n2,dlamda,_dlamda_offset,1,1,indxc,_indxc_offset);
{
for (i = 1; i <= n; i++) {
indx[(i-(1))+ _indx_offset] = indxq[(indxc[(i-(1))+ _indxc_offset]-(1))+ _indxq_offset];
/***
*
*     Calculate the allowable deflation tolerance
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaed2",30);
}              //  Close for() loop.
}
imax = com.polytechnik.lapack310.Idamax.idamax(n,z,_z_offset,1);
jmax = com.polytechnik.lapack310.Idamax.idamax(n,d,_d_offset,1);
eps = com.polytechnik.lapack310.Dlamch.dlamch("Epsilon");
/***
*
*     If the rank-1 modifier is small enough, no more needs to be done
*     except to reorganize Q so that its columns correspond with the
*     elements in D.
*
***/
tol = ((8.0*eps)*Math.max(Math.abs(d[(jmax-(1))+ _d_offset]), Math.abs(z[(imax-(1))+ _z_offset])));
if (((rho.val*Math.abs(z[(imax-(1))+ _z_offset])) <= tol))  {
    k.val = 0;
iq2 = 1;
{
for (j = 1; j <= n; j++) {
i = indx[(j-(1))+ _indx_offset];
com.polytechnik.lapack310.Dcopy.dcopy(n,q,(1-(1))+(i-(1)) * (ldq)+ _q_offset,1,q2,(iq2-(1))+ _q2_offset,1);
dlamda[(j-(1))+ _dlamda_offset] = d[(i-(1))+ _d_offset];
iq2 = (iq2+n);
//Dummy.label("com/polytechnik/lapack310/Dlaed2",40);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dlacpy.dlacpy("A",n,n,q2,_q2_offset,n,q,_q_offset,ldq);
com.polytechnik.lapack310.Dcopy.dcopy(n,dlamda,_dlamda_offset,1,d,_d_offset,1);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlaed2",190);
/***
*
*     If there are multiple eigenvalues then the problem deflates.  Here
*     the number of equal eigenvalues are found.  As each equal
*     eigenvalue is found, an elementary reflector is computed to rotate
*     the corresponding eigensubspace so that the corresponding
*     components of Z are zero in this new basis.
*
***/
}
{
for (i = 1; i <= n1; i++) {
coltyp[(i-(1))+ _coltyp_offset] = 1;
//Dummy.label("com/polytechnik/lapack310/Dlaed2",50);
}              //  Close for() loop.
}
{
for (i = n1p1; i <= n; i++) {
coltyp[(i-(1))+ _coltyp_offset] = 3;
/***
*
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaed2",60);
}              //  Close for() loop.
}
k.val = 0;
k2 = (n+1);
label100:{
{
label70:for (j = 1; j <= n; j++) {
nj = indx[(j-(1))+ _indx_offset];
/***
*
*           Deflate due to small z component.
*
***/
if (((rho.val*Math.abs(z[(nj-(1))+ _z_offset])) <= tol))  {
    k2 = (k2-1);
coltyp[(nj-(1))+ _coltyp_offset] = 4;
indxp[(k2-(1))+ _indxp_offset] = nj;
if ((j == n)) {
    break label100;//Dummy.go_to("com/polytechnik/lapack310/Dlaed2",100);
}
    }
else  {
  pj = nj;
  break label70;//Dummy.go_to("com/polytechnik/lapack310/Dlaed2",80);
}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dlaed2",70);
}              //  Close for() loop.
}
label80:for(;;){
//   Dummy.label("com/polytechnik/lapack310/Dlaed2",80);
j = (j+1);
nj = indx[(j-(1))+ _indx_offset];
if ((j > n)) {
    break label100;//Dummy.go_to("com/polytechnik/lapack310/Dlaed2",100);
}
    /***
*
*        Deflate due to small z component.
*
***/
if (((rho.val*Math.abs(z[(nj-(1))+ _z_offset])) <= tol))  {
    k2 = (k2-1);
coltyp[(nj-(1))+ _coltyp_offset] = 4;
indxp[(k2-(1))+ _indxp_offset] = nj;
/***
*
*        Check if eigenvalues are close enough to allow deflation.
*
***/
}
else  {
  s = z[(pj-(1))+ _z_offset];
/***
*
*        Find sqrt(a**2+b**2) without overflow or
*        destructive underflow.
*
***/
c = z[(nj-(1))+ _z_offset];
tau = com.polytechnik.lapack310.Dlapy2.dlapy2(c,s);
t = (d[(nj-(1))+ _d_offset]-d[(pj-(1))+ _d_offset]);
c = (c/tau);
s = (-((s/tau)));
/***
*
*           Deflation is possible.
*
***/
if ((Math.abs(((t*c)*s)) <= tol))  {
    z[(nj-(1))+ _z_offset] = tau;
z[(pj-(1))+ _z_offset] = 0.0;
if ((coltyp[(nj-(1))+ _coltyp_offset] != coltyp[(pj-(1))+ _coltyp_offset])) {
    coltyp[(nj-(1))+ _coltyp_offset] = 2;
}
    coltyp[(pj-(1))+ _coltyp_offset] = 4;
com.polytechnik.lapack310.Drot.drot(n,q,(1-(1))+(pj-(1)) * (ldq)+ _q_offset,1,q,(1-(1))+(nj-(1)) * (ldq)+ _q_offset,1,c,s);
t = ((d[(pj-(1))+ _d_offset]*( Math.pow(c, 2)))+(d[(nj-(1))+ _d_offset]*( Math.pow(s, 2))));
d[(nj-(1))+ _d_offset] = ((d[(pj-(1))+ _d_offset]*( Math.pow(s, 2)))+(d[(nj-(1))+ _d_offset]*( Math.pow(c, 2))));
d[(pj-(1))+ _d_offset] = t;
k2 = (k2-1);
i = 1;
label90:for(;;){
    //Dummy.label("com/polytechnik/lapack310/Dlaed2",90);
if (((k2+i) <= n))  {
    if ((d[(pj-(1))+ _d_offset] < d[(indxp[((k2+i)-(1))+ _indxp_offset]-(1))+ _d_offset]))  {
    indxp[(((k2+i)-1)-(1))+ _indxp_offset] = indxp[((k2+i)-(1))+ _indxp_offset];
indxp[((k2+i)-(1))+ _indxp_offset] = pj;
i = (i+1);
continue label90;//Dummy.go_to("com/polytechnik/lapack310/Dlaed2",90);
}
else  {
  indxp[(((k2+i)-1)-(1))+ _indxp_offset] = pj;
}              //  Close else.
}
else  {
  indxp[(((k2+i)-1)-(1))+ _indxp_offset] = pj;
}              //  Close else.
break label90;}// END label90
pj = nj;
}
else  {
  k.val = (k.val+1);
dlamda[(k.val-(1))+ _dlamda_offset] = d[(pj-(1))+ _d_offset];
w[(k.val-(1))+ _w_offset] = z[(pj-(1))+ _z_offset];
indxp[(k.val-(1))+ _indxp_offset] = pj;
pj = nj;
}              //  Close else.
}              //  Close else.
continue label80;//Dummy.go_to("com/polytechnik/lapack310/Dlaed2",80);
}// END label80
/***
*
*     Record the last eigenvalue.
*
***/
}// END label100
//Dummy.label("com/polytechnik/lapack310/Dlaed2",100);
k.val = (k.val+1);
dlamda[(k.val-(1))+ _dlamda_offset] = d[(pj-(1))+ _d_offset];
w[(k.val-(1))+ _w_offset] = z[(pj-(1))+ _z_offset];
/***
*
*     Count up the total number of the various types of columns, then
*     form a permutation which positions the four column types into
*     four uniform groups (although one or more of these groups may be
*     empty).
*
***/
indxp[(k.val-(1))+ _indxp_offset] = pj;
{
for (j = 1; j <= 4; j++) {
ctot[(j-(1))] = 0;
//Dummy.label("com/polytechnik/lapack310/Dlaed2",110);
}              //  Close for() loop.
}
{
for (j = 1; j <= n; j++) {
ct = coltyp[(j-(1))+ _coltyp_offset];
ctot[(ct-(1))] = (ctot[(ct-(1))]+1);
/***
*
*     PSM(*) = Position in SubMatrix (of types 1 through 4)
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaed2",120);
}              //  Close for() loop.
}
psm[(1-(1))] = 1;
psm[(2-(1))] = (1+ctot[(1-(1))]);
psm[(3-(1))] = (psm[(2-(1))]+ctot[(2-(1))]);
psm[(4-(1))] = (psm[(3-(1))]+ctot[(3-(1))]);
/***
*
*     Fill out the INDXC array so that the permutation which it induces
*     will place all type-1 columns first, all type-2 columns next,
*     then all type-3's, and finally all type-4's.
*
***/
k.val = (n-ctot[(4-(1))]);
{
for (j = 1; j <= n; j++) {
js = indxp[(j-(1))+ _indxp_offset];
ct = coltyp[(js-(1))+ _coltyp_offset];
indx[(psm[(ct-(1))]-(1))+ _indx_offset] = js;
indxc[(psm[(ct-(1))]-(1))+ _indxc_offset] = j;
psm[(ct-(1))] = (psm[(ct-(1))]+1);
/***
*
*     Sort the eigenvalues and corresponding eigenvectors into DLAMDA
*     and Q2 respectively.  The eigenvalues/vectors which were not
*     deflated go into the first K slots of DLAMDA and Q2 respectively,
*     while those which were deflated go into the last N - K slots.
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaed2",130);
}              //  Close for() loop.
}
i = 1;
iq1 = 1;
iq2 = (1+(((ctot[(1-(1))]+ctot[(2-(1))]))*n1));
{
for (j = 1; j <= ctot[(1-(1))]; j++) {
js = indx[(i-(1))+ _indx_offset];
com.polytechnik.lapack310.Dcopy.dcopy(n1,q,(1-(1))+(js-(1)) * (ldq)+ _q_offset,1,q2,(iq1-(1))+ _q2_offset,1);
z[(i-(1))+ _z_offset] = d[(js-(1))+ _d_offset];
i = (i+1);
iq1 = (iq1+n1);
// *
//Dummy.label("com/polytechnik/lapack310/Dlaed2",140);
}              //  Close for() loop.
}
{
for (j = 1; j <= ctot[(2-(1))]; j++) {
js = indx[(i-(1))+ _indx_offset];
com.polytechnik.lapack310.Dcopy.dcopy(n1,q,(1-(1))+(js-(1)) * (ldq)+ _q_offset,1,q2,(iq1-(1))+ _q2_offset,1);
com.polytechnik.lapack310.Dcopy.dcopy(n2,q,((n1+1)-(1))+(js-(1)) * (ldq)+ _q_offset,1,q2,(iq2-(1))+ _q2_offset,1);
z[(i-(1))+ _z_offset] = d[(js-(1))+ _d_offset];
i = (i+1);
iq1 = (iq1+n1);
iq2 = (iq2+n2);
// *
//Dummy.label("com/polytechnik/lapack310/Dlaed2",150);
}              //  Close for() loop.
}
{
for (j = 1; j <= ctot[(3-(1))]; j++) {
js = indx[(i-(1))+ _indx_offset];
com.polytechnik.lapack310.Dcopy.dcopy(n2,q,((n1+1)-(1))+(js-(1)) * (ldq)+ _q_offset,1,q2,(iq2-(1))+ _q2_offset,1);
z[(i-(1))+ _z_offset] = d[(js-(1))+ _d_offset];
i = (i+1);
iq2 = (iq2+n2);
// *
//Dummy.label("com/polytechnik/lapack310/Dlaed2",160);
}              //  Close for() loop.
}
iq1 = iq2;
{
for (j = 1; j <= ctot[(4-(1))]; j++) {
js = indx[(i-(1))+ _indx_offset];
com.polytechnik.lapack310.Dcopy.dcopy(n,q,(1-(1))+(js-(1)) * (ldq)+ _q_offset,1,q2,(iq2-(1))+ _q2_offset,1);
iq2 = (iq2+n);
z[(i-(1))+ _z_offset] = d[(js-(1))+ _d_offset];
i = (i+1);
/***
*
*     The deflated eigenvalues and their corresponding vectors go back
*     into the last N - K slots of D and Q respectively.
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaed2",170);
}              //  Close for() loop.
}
if ((k.val < n))  {
    com.polytechnik.lapack310.Dlacpy.dlacpy("A",n,ctot[(4-(1))],q2,(iq1-(1))+ _q2_offset,n,q,(1-(1))+((k.val+1)-(1)) * (ldq)+ _q_offset,ldq);
com.polytechnik.lapack310.Dcopy.dcopy((n-k.val),z,((k.val+1)-(1))+ _z_offset,1,d,((k.val+1)-(1))+ _d_offset,1);
/***
*
*     Copy CTOT into COLTYP for referencing in DLAED3.
*
***/
}
{
for (j = 1; j <= 4; j++) {
coltyp[(j-(1))+ _coltyp_offset] = ctot[(j-(1))];
// *
//Dummy.label("com/polytechnik/lapack310/Dlaed2",180);
}              //  Close for() loop.
}
//label190:
//   Dummy.label("com/polytechnik/lapack310/Dlaed2",190);
/***
*
*     End of DLAED2
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlaed2",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlaed2",999999);
return;
   }
} // End class.
