/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlahr2.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlahr2 {

// *> \brief \b DLAHR2 reduces the specified number of first columns of a g
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLAHR2 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAHR2( N, K, NB, A, LDA, TAU, T, LDT, Y, LDY )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            K, LDA, LDT, LDY, N, NB
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION  A( LDA, * ), T( LDT, NB ), TAU( NB ),
// *      $                   Y( LDY, NB )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLAHR2 reduces the first NB columns of A real general n-BY-(n-k+1)
// *> matrix A so that elements below the k-th subdiagonal are zero. The
// *> reduction is performed by an orthogonal similarity transformation
// *> Q**T * A * Q. The routine returns the matrices V and T which determin
// *> Q as a block reflector I - V*T*V**T, and also the matrix Y = A * V *
// *>
// *> This is an auxiliary routine called by DGEHRD.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>          The offset for the reduction. Elements below the k-th
// *>          subdiagonal in the first NB columns are reduced to zero.
// *>          K < N.
// *> \endverbatim
// *>
// *> \param[in] NB
// *> \verbatim
// *>          NB is INTEGER
// *>          The number of columns to be reduced.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N-K+1)
// *>          On entry, the n-by-(n-k+1) general matrix A.
// *>          On exit, the elements on and above the k-th subdiagonal in
// *>          the first NB columns are overwritten with the corresponding

// *>          elements of the reduced matrix; the elements below the k-th

// *>          subdiagonal, with the array TAU, represent the matrix Q as a
// *>          product of elementary reflectors. The other columns of A are
// *>          unchanged. See Further Details.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (NB)
// *>          The scalar factors of the elementary reflectors. See Further
// *>          Details.
// *> \endverbatim
// *>
// *> \param[out] T
// *> \verbatim
// *>          T is DOUBLE PRECISION array, dimension (LDT,NB)
// *>          The upper triangular matrix T.
// *> \endverbatim
// *>
// *> \param[in] LDT
// *> \verbatim
// *>          LDT is INTEGER
// *>          The leading dimension of the array T.  LDT >= NB.
// *> \endverbatim
// *>
// *> \param[out] Y
// *> \verbatim
// *>          Y is DOUBLE PRECISION array, dimension (LDY,NB)
// *>          The n-by-nb matrix Y.
// *> \endverbatim
// *>
// *> \param[in] LDY
// *> \verbatim
// *>          LDY is INTEGER
// *>          The leading dimension of the array Y. LDY >= N.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  The matrix Q is represented as a product of nb elementary reflectors
// *>
// *>     Q = H(1) H(2) . . . H(nb).
// *>
// *>  Each H(i) has the form
// *>
// *>     H(i) = I - tau * v * v**T
// *>
// *>  where tau is a real scalar, and v is a real vector with
// *>  v(1:i+k-1) = 0, v(i+k) = 1; v(i+k+1:n) is stored on exit in
// *>  A(i+k+1:n,i), and tau in TAU(i).
// *>
// *>  The elements of the vectors v together form the (n-k+1)-by-nb matrix
// *>  V which is needed, with T and Y, to apply the transformation to the

// *>  unreduced part of the matrix, using an update of the form:
// *>  A := (I - V*T*V**T) * (A - Y*V**T).
// *>
// *>  The contents of A on exit are illustrated by the following example
// *>  with n = 7, k = 3 and nb = 2:
// *>
// *>     ( a   a   a   a   a )
// *>     ( a   a   a   a   a )
// *>     ( a   a   a   a   a )
// *>     ( h   h   a   a   a )
// *>     ( v1  h   a   a   a )
// *>     ( v1  v2  a   a   a )
// *>     ( v1  v2  a   a   a )
// *>
// *>  where a denotes an element of the original matrix A, h denotes a
// *>  modified element of the upper Hessenberg matrix H, and vi denotes an
// *>  element of the vector defining H(i).
// *>
// *>  This subroutine is a slight modification of LAPACK-3.0's DLAHRD
// *>  incorporating improvements proposed by Quintana-Orti and Van de
// *>  Gejin. Note that the entries of A(1:K,2:NB) differ from those
// *>  returned by the original LAPACK-3.0's DLAHRD routine. (This
// *>  subroutine is not backward compatible with LAPACK-3.0's DLAHRD.)
// *> \endverbatim
// *
// *> \par References:
// *  ================
// *>
// *>  Gregorio Quintana-Orti and Robert van de Geijn, "Improving the
// *>  performance of reduction to Hessenberg form," ACM Transactions on
// *>  Mathematical Software, 32(2):180-194, June 2006.
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Quick return if possible
*
***/
// *

public static void dlahr2 (int n,
int k,
int nb,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] t, int _t_offset,
int ldt,
double [] y, int _y_offset,
int ldy)  {

label_999999:{

int i= 0;
double ei= 0.0d;
if ((n <= 1)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlahr2",999999);
}
    {
for (i = 1; i <= nb; i++) {
/***
*
*           Update A(K+1:N,I)
*
*           Update I-th column of A - Y * V**T
*
***/
if ((i > 1))  {
    /***
*
*           Apply I - V * T**T * V**T to this column (call it b) from the
*           left, using the last column of T as workspace
*
*           Let  V = ( V1 )   and   b = ( b1 )   (first I-1 rows)
*                    ( V2 )             ( b2 )
*
*           where V1 is unit lower triangular
*
*           w := V1**T * b1
*
***/
com.polytechnik.lapack310.Dgemv.dgemv("NO TRANSPOSE",(n-k),(i-1),-1.0,y,((k+1)-(1))+(1-(1)) * (ldy)+ _y_offset,ldy,a,(((k+i)-1)-(1))+(1-(1)) * (lda)+ _a_offset,lda,1.0,a,((k+1)-(1))+(i-(1)) * (lda)+ _a_offset,1);
com.polytechnik.lapack310.Dcopy.dcopy((i-1),a,((k+1)-(1))+(i-(1)) * (lda)+ _a_offset,1,t,(1-(1))+(nb-(1)) * (ldt)+ _t_offset,1);
/***
*
*           w := w + V2**T * b2
*
***/
com.polytechnik.lapack310.Dtrmv.dtrmv("Lower","Transpose","UNIT",(i-1),a,((k+1)-(1))+(1-(1)) * (lda)+ _a_offset,lda,t,(1-(1))+(nb-(1)) * (ldt)+ _t_offset,1);
/***
*
*           w := T**T * w
*
***/
com.polytechnik.lapack310.Dgemv.dgemv("Transpose",(((n-k)-i)+1),(i-1),1.0,a,((k+i)-(1))+(1-(1)) * (lda)+ _a_offset,lda,a,((k+i)-(1))+(i-(1)) * (lda)+ _a_offset,1,1.0,t,(1-(1))+(nb-(1)) * (ldt)+ _t_offset,1);
/***
*
*           b2 := b2 - V2*w
*
***/
com.polytechnik.lapack310.Dtrmv.dtrmv("Upper","Transpose","NON-UNIT",(i-1),t,_t_offset,ldt,t,(1-(1))+(nb-(1)) * (ldt)+ _t_offset,1);
/***
*
*           b1 := b1 - V1*w
*
***/
com.polytechnik.lapack310.Dgemv.dgemv("NO TRANSPOSE",(((n-k)-i)+1),(i-1),-1.0,a,((k+i)-(1))+(1-(1)) * (lda)+ _a_offset,lda,t,(1-(1))+(nb-(1)) * (ldt)+ _t_offset,1,1.0,a,((k+i)-(1))+(i-(1)) * (lda)+ _a_offset,1);
com.polytechnik.lapack310.Dtrmv.dtrmv("Lower","NO TRANSPOSE","UNIT",(i-1),a,((k+1)-(1))+(1-(1)) * (lda)+ _a_offset,lda,t,(1-(1))+(nb-(1)) * (ldt)+ _t_offset,1);
// *
com.polytechnik.lapack310.Daxpy.daxpy((i-1),-1.0,t,(1-(1))+(nb-(1)) * (ldt)+ _t_offset,1,a,((k+1)-(1))+(i-(1)) * (lda)+ _a_offset,1);
a[(((k+i)-1)-(1))+((i-1)-(1)) * (lda)+ _a_offset] = ei;
/***
*
*        Generate the elementary reflector H(I) to annihilate
*        A(K+I+1:N,I)
*
***/
}
dlarfg_adapter((((n-k)-i)+1),a,((k+i)-(1))+(i-(1)) * (lda)+ _a_offset,a,(Math.min(((k+i)+1), n)-(1))+(i-(1)) * (lda)+ _a_offset,1,tau,(i-(1))+ _tau_offset);
ei = a[((k+i)-(1))+(i-(1)) * (lda)+ _a_offset];
/***
*
*        Compute  Y(K+1:N,I)
*
***/
a[((k+i)-(1))+(i-(1)) * (lda)+ _a_offset] = 1.0;
com.polytechnik.lapack310.Dgemv.dgemv("NO TRANSPOSE",(n-k),(((n-k)-i)+1),1.0,a,((k+1)-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda,a,((k+i)-(1))+(i-(1)) * (lda)+ _a_offset,1,0.0,y,((k+1)-(1))+(i-(1)) * (ldy)+ _y_offset,1);
com.polytechnik.lapack310.Dgemv.dgemv("Transpose",(((n-k)-i)+1),(i-1),1.0,a,((k+i)-(1))+(1-(1)) * (lda)+ _a_offset,lda,a,((k+i)-(1))+(i-(1)) * (lda)+ _a_offset,1,0.0,t,(1-(1))+(i-(1)) * (ldt)+ _t_offset,1);
com.polytechnik.lapack310.Dgemv.dgemv("NO TRANSPOSE",(n-k),(i-1),-1.0,y,((k+1)-(1))+(1-(1)) * (ldy)+ _y_offset,ldy,t,(1-(1))+(i-(1)) * (ldt)+ _t_offset,1,1.0,y,((k+1)-(1))+(i-(1)) * (ldy)+ _y_offset,1);
/***
*
*        Compute T(1:I,I)
*
***/
com.polytechnik.lapack310.Dscal.dscal((n-k),tau[(i-(1))+ _tau_offset],y,((k+1)-(1))+(i-(1)) * (ldy)+ _y_offset,1);
com.polytechnik.lapack310.Dscal.dscal((i-1),(-(tau[(i-(1))+ _tau_offset])),t,(1-(1))+(i-(1)) * (ldt)+ _t_offset,1);
com.polytechnik.lapack310.Dtrmv.dtrmv("Upper","No Transpose","NON-UNIT",(i-1),t,_t_offset,ldt,t,(1-(1))+(i-(1)) * (ldt)+ _t_offset,1);
// *
t[(i-(1))+(i-(1)) * (ldt)+ _t_offset] = tau[(i-(1))+ _tau_offset];
//Dummy.label("com/polytechnik/lapack310/Dlahr2",10);
}              //  Close for() loop.
}
/***
*
*     Compute Y(1:K,1:NB)
*
***/
a[((k+nb)-(1))+(nb-(1)) * (lda)+ _a_offset] = ei;
com.polytechnik.lapack310.Dlacpy.dlacpy("ALL",k,nb,a,(1-(1))+(2-(1)) * (lda)+ _a_offset,lda,y,_y_offset,ldy);
com.polytechnik.lapack310.Dtrmm.dtrmm("RIGHT","Lower","NO TRANSPOSE","UNIT",k,nb,1.0,a,((k+1)-(1))+(1-(1)) * (lda)+ _a_offset,lda,y,_y_offset,ldy);
if ((n > (k+nb))) {
    com.polytechnik.lapack310.Dgemm.dgemm("NO TRANSPOSE","NO TRANSPOSE",k,nb,((n-k)-nb),1.0,a,(1-(1))+((2+nb)-(1)) * (lda)+ _a_offset,lda,a,(((k+1)+nb)-(1))+(1-(1)) * (lda)+ _a_offset,lda,1.0,y,_y_offset,ldy);
}
    // *
com.polytechnik.lapack310.Dtrmm.dtrmm("RIGHT","Upper","NO TRANSPOSE","NON-UNIT",k,nb,1.0,t,_t_offset,ldt,y,_y_offset,ldy);
/***
*
*     End of DLAHR2
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlahr2",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlahr2",999999);
return;
   }
// adapter for dlarfg(ILorg/netlib/util/doubleW;[DIILorg/netlib/util/doubleW;)V
private static void dlarfg_adapter(int arg0 ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,int arg3 ,double [] arg4 , int arg4_offset )
{
doubleW _f2j_tmp1 = new doubleW(arg1[arg1_offset]);
doubleW _f2j_tmp4 = new doubleW(arg4[arg4_offset]);

com.polytechnik.lapack310.Dlarfg.dlarfg(arg0,_f2j_tmp1,arg2, arg2_offset,arg3,_f2j_tmp4);

arg1[arg1_offset] = _f2j_tmp1.val;
arg4[arg4_offset] = _f2j_tmp4.val;
}

} // End class.
