package com.polytechnik.lapack310;


/*
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlamch.java,v 1.2 2022/10/19 20:33:08 mal Exp $
 */
class Dlamch {


    private static final double eps=0.5*Math.ulp(1.0),sfmin=Double.MIN_NORMAL,base=2,emax=Double.MAX_EXPONENT+1,rmax=Math.pow(2,Double.MAX_EXPONENT)*(1-0.5*Math.ulp(1.0))*2;


    /**
<pre>{@code
*>          Specifies the value to be returned by DLAMCH:
*>          = 'E' or 'e',   DLAMCH := eps
*>          = 'S' or 's ,   DLAMCH := sfmin
*>          = 'B' or 'b',   DLAMCH := base
*>          = 'P' or 'p',   DLAMCH := eps*base
*>          = 'N' or 'n',   DLAMCH := t
*>          = 'R' or 'r',   DLAMCH := rnd
*>          = 'M' or 'm',   DLAMCH := emin
*>          = 'U' or 'u',   DLAMCH := rmin
*>          = 'L' or 'l',   DLAMCH := emax
*>          = 'O' or 'o',   DLAMCH := rmax
*>          where
*>          eps   = relative machine precision
*>          sfmin = safe minimum, such that 1/sfmin does not overflow
*>          base  = base of the machine
*>          prec  = eps*base
*>          t     = number of (base) digits in the mantissa
*>          rnd   = 1.0 when rounding occurs in addition, 0.0 otherwise
*>          emin  = minimum exponent before (gradual) underflow
*>          rmin  = underflow threshold - base**(emin-1)
*>          emax  = largest exponent before overflow
*>          rmax  = overflow threshold  - (base**emax)*(1-eps)
}</pre>
*/

    public static double dlamch(final String s)
    {
	if(Lsame.lsame("E",s)) return eps;
	if(Lsame.lsame("S",s)) return sfmin;
	if(Lsame.lsame("B",s)) return base;
	if(Lsame.lsame("P",s)) return eps*base;
	if(Lsame.lsame("U",s)) return sfmin;
	if(Lsame.lsame("L",s)) return emax;
	if(Lsame.lsame("O",s)) return rmax;
	if(true)
	    throw new RuntimeException("s="+s);
	return Double.NaN;
    }

    public static void main(String [] args)
    {
	for(var s:new String[]{"E","S","B","P","U","L","O"})
	{
	    System.err.println(s+"="+dlamch(s));
	}
    }

}
