/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlamrg.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlamrg {

// *> \brief \b DLAMRG creates a permutation list to merge the entries of t
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLAMRG + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAMRG( N1, N2, A, DTRD1, DTRD2, INDEX )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            DTRD1, DTRD2, N1, N2
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            INDEX( * )
// *       DOUBLE PRECISION   A( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLAMRG will create a permutation list which will merge the elements
// *> of A (which is composed of two independently sorted sets) into a
// *> single set which is sorted in ascending order.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N1
// *> \verbatim
// *>          N1 is INTEGER
// *> \endverbatim
// *>
// *> \param[in] N2
// *> \verbatim
// *>          N2 is INTEGER
// *>         These arguments contain the respective lengths of the two
// *>         sorted lists to be merged.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (N1+N2)
// *>         The first N1 elements of A contain a list of numbers which
// *>         are sorted in either ascending or descending order.  Likewise
// *>         for the final N2 elements.
// *> \endverbatim
// *>
// *> \param[in] DTRD1
// *> \verbatim
// *>          DTRD1 is INTEGER
// *> \endverbatim
// *>
// *> \param[in] DTRD2
// *> \verbatim
// *>          DTRD2 is INTEGER
// *>         These are the strides to be taken through the array A.
// *>         Allowable strides are 1 and -1.  They indicate whether a
// *>         subset of A is sorted in ascending (DTRDx = 1) or descending

// *>         (DTRDx = -1) order.
// *> \endverbatim
// *>
// *> \param[out] INDEX
// *> \verbatim
// *>          INDEX is INTEGER array, dimension (N1+N2)
// *>         On exit this array will contain a permutation such that
// *>         if B( I ) = A( INDEX( I ) ) for I=1,N1+N2, then B will be
// *>         sorted in ascending order.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlamrg (int n1,
int n2,
double [] a, int _a_offset,
int dtrd1,
int dtrd2,
int [] index, int _index_offset)  {

label_999999:{

int i= 0;
int ind1= 0;
int ind2= 0;
int n1sv= 0;
int n2sv= 0;
n1sv = n1;
n2sv = n2;
if ((dtrd1 > 0))  {
    ind1 = 1;
}
else  {
  ind1 = n1;
}              //  Close else.
if ((dtrd2 > 0))  {
    ind2 = (1+n1);
}
else  {
  ind2 = (n1+n2);
}              //  Close else.
// *     while ( (N1SV > 0) & (N2SV > 0) )
i = 1;
label10:for(int jjx=0;jjx<Util.ITMAX;jjx++){
    //Dummy.label("com/polytechnik/lapack310/Dlamrg",10);
if (((n1sv > 0) && (n2sv > 0)))  {
    if ((a[(ind1-(1))+ _a_offset] <= a[(ind2-(1))+ _a_offset]))  {
    index[(i-(1))+ _index_offset] = ind1;
i = (i+1);
ind1 = (ind1+dtrd1);
n1sv = (n1sv-1);
}
else  {
  index[(i-(1))+ _index_offset] = ind2;
i = (i+1);
ind2 = (ind2+dtrd2);
n2sv = (n2sv-1);
}              //  Close else.
    continue label10;//Dummy.go_to("com/polytechnik/lapack310/Dlamrg",10);
// *     end while
}
break label10;} // END label10
if ((n1sv == 0))  {
    {
for (n1sv = 1; n1sv <= n2sv; n1sv++) {
index[(i-(1))+ _index_offset] = ind2;
i = (i+1);
ind2 = (ind2+dtrd2);
//Dummy.label("com/polytechnik/lapack310/Dlamrg",20);
}              //  Close for() loop.
}
// *     N2SV .EQ. 0
}
else  {
  {
for (n2sv = 1; n2sv <= n1sv; n2sv++) {
index[(i-(1))+ _index_offset] = ind1;
i = (i+1);
ind1 = (ind1+dtrd1);
//Dummy.label("com/polytechnik/lapack310/Dlamrg",30);
}              //  Close for() loop.
}
// *
}              //  Close else.
/***
*
*     End of DLAMRG
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlamrg",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlamrg",999999);
return;
   }
} // End class.
