/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlanv2.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlanv2 {

// *> \brief \b DLANV2 computes the Schur factorization of a real 2-by-2 no
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLANV2 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLANV2( A, B, C, D, RT1R, RT1I, RT2R, RT2I, CS, SN )

// *
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION   A, B, C, CS, D, RT1I, RT1R, RT2I, RT2R, SN
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLANV2 computes the Schur factorization of a real 2-by-2 nonsymmetric
// *> matrix in standard form:
// *>
// *>      [ A  B ] = [ CS -SN ] [ AA  BB ] [ CS  SN ]
// *>      [ C  D ]   [ SN  CS ] [ CC  DD ] [-SN  CS ]
// *>
// *> where either
// *> 1) CC = 0 so that AA and DD are real eigenvalues of the matrix, or
// *> 2) AA = DD and BB*CC < 0, so that AA + or - sqrt(BB*CC) are complex
// *> conjugate eigenvalues.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in,out] B
// *> \verbatim
// *>          B is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in,out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION
// *>          On entry, the elements of the input matrix.
// *>          On exit, they are overwritten by the elements of the
// *>          standardised Schur form.
// *> \endverbatim
// *>
// *> \param[out] RT1R
// *> \verbatim
// *>          RT1R is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[out] RT1I
// *> \verbatim
// *>          RT1I is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[out] RT2R
// *> \verbatim
// *>          RT2R is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[out] RT2I
// *> \verbatim
// *>          RT2I is DOUBLE PRECISION
// *>          The real and imaginary parts of the eigenvalues. If the
// *>          eigenvalues are a complex conjugate pair, RT1I > 0.
// *> \endverbatim
// *>
// *> \param[out] CS
// *> \verbatim
// *>          CS is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[out] SN
// *> \verbatim
// *>          SN is DOUBLE PRECISION
// *>          Parameters of the rotation matrix.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Modified by V. Sima, Research Institute for Informatics, Bucharest,

// *>  Romania, to reduce the risk of cancellation errors,
// *>  when computing real eigenvalues, and to ensure, if possible, that
// *>  abs(RT1R) >= abs(RT2R).
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlanv2 (doubleW a,
doubleW b,
doubleW c,
doubleW d,
doubleW rt1r,
doubleW rt1i,
doubleW rt2r,
doubleW rt2i,
doubleW cs,
doubleW sn)  {

label_999999:{

double aa= 0.0d;
double bb= 0.0d;
double bcmax= 0.0d;
double bcmis= 0.0d;
double cc= 0.0d;
double cs1= 0.0d;
double dd= 0.0d;
double eps= 0.0d;
double p= 0.0d;
double sab= 0.0d;
double sac= 0.0d;
double scale= 0.0d;
double sigma= 0.0d;
double sn1= 0.0d;
double tau= 0.0d;
double temp= 0.0d;
double z= 0.0d;
double safmin= 0.0d;
double safmn2= 0.0d;
double safmx2= 0.0d;
int count= 0;
safmin = com.polytechnik.lapack310.Dlamch.dlamch("S");
eps = com.polytechnik.lapack310.Dlamch.dlamch("P");
safmn2 = ( Math.pow(com.polytechnik.lapack310.Dlamch.dlamch("B"), (int)(((Math.log((safmin/eps))/Math.log(com.polytechnik.lapack310.Dlamch.dlamch("B")))/2.0))));
safmx2 = (1.0/safmn2);
if ((c.val == 0.0))  {
    cs.val = 1.0;
// *
sn.val = 0.0;
/***
*
*        Swap rows and columns
*
***/
}
else if ((b.val == 0.0))  {
    cs.val = 0.0;
sn.val = 1.0;
temp = d.val;
d.val = a.val;
a.val = temp;
b.val = (-(c.val));
// *
c.val = 0.0;
}              // Close else if()
else if (((((a.val-d.val)) == 0.0) && (Util.dsign(1.0,b.val) != Util.dsign(1.0,c.val))))  {
    cs.val = 1.0;
// *
sn.val = 0.0;
// *
}              // Close else if()
else  {
  temp = (a.val-d.val);
p = (0.5*temp);
bcmax = Math.max(Math.abs(b.val), Math.abs(c.val));
bcmis = ((Math.min(Math.abs(b.val), Math.abs(c.val))*Util.dsign(1.0,b.val))*Util.dsign(1.0,c.val));
scale = Math.max(Math.abs(p), bcmax);
/***
*
*        If Z is of the order of the machine accuracy, postpone the
*        decision on the nature of eigenvalues
*
***/
z = ((((p/scale))*p)+(((bcmax/scale))*bcmis));
/***
*
*           Real eigenvalues. Compute A and D.
*
***/
if ((z >= (4.0*eps)))  {
    z = (p+Util.dsign((Math.sqrt(scale)*Math.sqrt(z)),p));
a.val = (d.val+z);
/***
*
*           Compute B and the rotation matrix
*
***/
d.val = (d.val-(((bcmax/z))*bcmis));
tau = com.polytechnik.lapack310.Dlapy2.dlapy2(c.val,z);
cs.val = (z/tau);
sn.val = (c.val/tau);
b.val = (b.val-c.val);
// *
c.val = 0.0;
/***
*
*           Complex eigenvalues, or real (almost) equal eigenvalues.
*           Make diagonal elements equal.
*
***/
}
else  {
  count = 0;
sigma = (b.val+c.val);
label10:for(int jjx=0;jjx<Util.ITMAX;jjx++){
    //Dummy.label("com/polytechnik/lapack310/Dlanv2",10);
count = (count+1);
scale = Math.max(Math.abs(temp), Math.abs(sigma));
if ((scale >= safmx2))  {
    sigma = (sigma*safmn2);
temp = (temp*safmn2);
if ((count <= 20)) {
    continue label10;//Dummy.go_to("com/polytechnik/lapack310/Dlanv2",10);
}
    }
if ((scale <= safmn2))  {
    sigma = (sigma*safmx2);
temp = (temp*safmx2);
if ((count <= 20)) {
    continue label10;//Dummy.go_to("com/polytechnik/lapack310/Dlanv2",10);
}
    }
break label10;}// END label10
p = (0.5*temp);
tau = com.polytechnik.lapack310.Dlapy2.dlapy2(sigma,temp);
cs.val = Math.sqrt((0.5*((1.0+(Math.abs(sigma)/tau)))));
/***
*
*           Compute [ AA  BB ] = [ A  B ] [ CS -SN ]
*                   [ CC  DD ]   [ C  D ] [ SN  CS ]
*
***/
sn.val = (-((((p/((tau*cs.val))))*Util.dsign(1.0,sigma))));
aa = ((a.val*cs.val)+(b.val*sn.val));
bb = ((-((a.val*sn.val)))+(b.val*cs.val));
cc = ((c.val*cs.val)+(d.val*sn.val));
/***
*
*           Compute [ A  B ] = [ CS  SN ] [ AA  BB ]
*                   [ C  D ]   [-SN  CS ] [ CC  DD ]
*
***/
dd = ((-((c.val*sn.val)))+(d.val*cs.val));
a.val = ((aa*cs.val)+(cc*sn.val));
b.val = ((bb*cs.val)+(dd*sn.val));
c.val = ((-((aa*sn.val)))+(cc*cs.val));
// *
d.val = ((-((bb*sn.val)))+(dd*cs.val));
temp = (0.5*((a.val+d.val)));
a.val = temp;
// *
d.val = temp;
if ((c.val != 0.0))  {
    if ((b.val != 0.0))  {
    /***
*
*                    Real eigenvalues: reduce to upper triangular form
*
***/
if ((Util.dsign(1.0,b.val) == Util.dsign(1.0,c.val)))  {
    sab = Math.sqrt(Math.abs(b.val));
sac = Math.sqrt(Math.abs(c.val));
p = Util.dsign((sab*sac),c.val);
tau = (1.0/Math.sqrt(Math.abs((b.val+c.val))));
a.val = (temp+p);
d.val = (temp-p);
b.val = (b.val-c.val);
c.val = 0.0;
cs1 = (sab*tau);
sn1 = (sac*tau);
temp = ((cs.val*cs1)-(sn.val*sn1));
sn.val = ((cs.val*sn1)+(sn.val*cs1));
cs.val = temp;
}
}
else  {
  b.val = (-(c.val));
c.val = 0.0;
temp = cs.val;
cs.val = (-(sn.val));
sn.val = temp;
}              //  Close else.
}
// *
}              //  Close else.
/***
*
*     Store eigenvalues in (RT1R,RT1I) and (RT2R,RT2I).
*
***/
}              //  Close else.
rt1r.val = a.val;
rt2r.val = d.val;
if ((c.val == 0.0))  {
    rt1i.val = 0.0;
rt2i.val = 0.0;
}
else  {
  rt1i.val = (Math.sqrt(Math.abs(b.val))*Math.sqrt(Math.abs(c.val)));
rt2i.val = (-(rt1i.val));
}              //  Close else.
/***
*
*     End of DLANV2
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlanv2",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlanv2",999999);
return;
   }
} // End class.
