/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlaqr1.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlaqr1 {

// *> \brief \b DLAQR1 sets a scalar multiple of the first column of the pr
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLAQR1 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAQR1( N, H, LDH, SR1, SI1, SR2, SI2, V )
// *
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION   SI1, SI2, SR1, SR2
// *       INTEGER            LDH, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   H( LDH, * ), V( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *>      Given a 2-by-2 or 3-by-3 matrix H, DLAQR1 sets v to a
// *>      scalar multiple of the first column of the product
// *>
// *>      (*)  K = (H - (sr1 + i*si1)*I)*(H - (sr2 + i*si2)*I)
// *>
// *>      scaling to avoid overflows and most underflows. It
// *>      is assumed that either
// *>
// *>              1) sr1 = sr2 and si1 = -si2
// *>          or
// *>              2) si1 = si2 = 0.
// *>
// *>      This is useful for starting double implicit shift bulges
// *>      in the QR algorithm.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>              Order of the matrix H. N must be either 2 or 3.
// *> \endverbatim
// *>
// *> \param[in] H
// *> \verbatim
// *>          H is DOUBLE PRECISION array, dimension (LDH,N)
// *>              The 2-by-2 or 3-by-3 matrix H in (*).
// *> \endverbatim
// *>
// *> \param[in] LDH
// *> \verbatim
// *>          LDH is INTEGER
// *>              The leading dimension of H as declared in
// *>              the calling procedure.  LDH >= N
// *> \endverbatim
// *>
// *> \param[in] SR1
// *> \verbatim
// *>          SR1 is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] SI1
// *> \verbatim
// *>          SI1 is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] SR2
// *> \verbatim
// *>          SR2 is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] SI2
// *> \verbatim
// *>          SI2 is DOUBLE PRECISION
// *>              The shifts in (*).
// *> \endverbatim
// *>
// *> \param[out] V
// *> \verbatim
// *>          V is DOUBLE PRECISION array, dimension (N)
// *>              A scalar multiple of the first column of the
// *>              matrix K in (*).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *> \par Contributors:
// *  ==================
// *>
// *>       Karen Braman and Ralph Byers, Department of Mathematics,
// *>       University of Kansas, USA
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  ================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Quick return if possible
*
***/

public static void dlaqr1 (int n,
double [] h, int _h_offset,
int ldh,
double sr1,
double si1,
double sr2,
double si2,
double [] v, int _v_offset)  {

label_999999:{

double h21s= 0.0d;
double h31s= 0.0d;
double s= 0.0d;
if (((n != 2) && (n != 3)))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlaqr1",999999);
// *
}
if ((n == 2))  {
    s = ((Math.abs((h[(1-(1))+(1-(1)) * (ldh)+ _h_offset]-sr2))+Math.abs(si2))+Math.abs(h[(2-(1))+(1-(1)) * (ldh)+ _h_offset]));
if ((s == 0.0))  {
    v[(1-(1))+ _v_offset] = 0.0;
v[(2-(1))+ _v_offset] = 0.0;
}
else  {
  h21s = (h[(2-(1))+(1-(1)) * (ldh)+ _h_offset]/s);
v[(1-(1))+ _v_offset] = (((h21s*h[(1-(1))+(2-(1)) * (ldh)+ _h_offset])+(((h[(1-(1))+(1-(1)) * (ldh)+ _h_offset]-sr1))*((((h[(1-(1))+(1-(1)) * (ldh)+ _h_offset]-sr2))/s))))-(si1*((si2/s))));
v[(2-(1))+ _v_offset] = (h21s*((((h[(1-(1))+(1-(1)) * (ldh)+ _h_offset]+h[(2-(1))+(2-(1)) * (ldh)+ _h_offset])-sr1)-sr2)));
}              //  Close else.
}
else  {
  s = (((Math.abs((h[(1-(1))+(1-(1)) * (ldh)+ _h_offset]-sr2))+Math.abs(si2))+Math.abs(h[(2-(1))+(1-(1)) * (ldh)+ _h_offset]))+Math.abs(h[(3-(1))+(1-(1)) * (ldh)+ _h_offset]));
if ((s == 0.0))  {
    v[(1-(1))+ _v_offset] = 0.0;
v[(2-(1))+ _v_offset] = 0.0;
v[(3-(1))+ _v_offset] = 0.0;
}
else  {
  h21s = (h[(2-(1))+(1-(1)) * (ldh)+ _h_offset]/s);
h31s = (h[(3-(1))+(1-(1)) * (ldh)+ _h_offset]/s);
v[(1-(1))+ _v_offset] = ((((((h[(1-(1))+(1-(1)) * (ldh)+ _h_offset]-sr1))*((((h[(1-(1))+(1-(1)) * (ldh)+ _h_offset]-sr2))/s)))-(si1*((si2/s))))+(h[(1-(1))+(2-(1)) * (ldh)+ _h_offset]*h21s))+(h[(1-(1))+(3-(1)) * (ldh)+ _h_offset]*h31s));
v[(2-(1))+ _v_offset] = ((h21s*((((h[(1-(1))+(1-(1)) * (ldh)+ _h_offset]+h[(2-(1))+(2-(1)) * (ldh)+ _h_offset])-sr1)-sr2)))+(h[(2-(1))+(3-(1)) * (ldh)+ _h_offset]*h31s));
v[(3-(1))+ _v_offset] = ((h31s*((((h[(1-(1))+(1-(1)) * (ldh)+ _h_offset]+h[(3-(1))+(3-(1)) * (ldh)+ _h_offset])-sr1)-sr2)))+(h21s*h[(3-(1))+(2-(1)) * (ldh)+ _h_offset]));
}              //  Close else.
}              //  Close else.
// dummy stmt
break label_999999;}//Dummy.label("com/polytechnik/lapack310/Dlaqr1",999999);
return;
   }
} // End class.
