/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarf.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlarf {

// *> \brief \b DLARF applies an elementary reflector to a general rectangu
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLARF + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARF( SIDE, M, N, V, INCV, TAU, C, LDC, WORK )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          SIDE
// *       INTEGER            INCV, LDC, M, N
// *       DOUBLE PRECISION   TAU
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   C( LDC, * ), V( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARF applies a real elementary reflector H to a real m by n matrix
// *> C, from either the left or the right. H is represented in the form
// *>
// *>       H = I - tau * v * v**T
// *>
// *> where tau is a real scalar and v is a real vector.
// *>
// *> If tau = 0, then H is taken to be the unit matrix.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] SIDE
// *> \verbatim
// *>          SIDE is CHARACTER*1
// *>          = 'L': form  H * C
// *>          = 'R': form  C * H
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix C.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix C.
// *> \endverbatim
// *>
// *> \param[in] V
// *> \verbatim
// *>          V is DOUBLE PRECISION array, dimension
// *>                     (1 + (M-1)*abs(INCV)) if SIDE = 'L'
// *>                  or (1 + (N-1)*abs(INCV)) if SIDE = 'R'
// *>          The vector v in the representation of H. V is not used if
// *>          TAU = 0.
// *> \endverbatim
// *>
// *> \param[in] INCV
// *> \verbatim
// *>          INCV is INTEGER
// *>          The increment between elements of v. INCV <> 0.
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION
// *>          The value tau in the representation of H.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension (LDC,N)
// *>          On entry, the m by n matrix C.
// *>          On exit, C is overwritten by the matrix H * C if SIDE = 'L',
// *>          or C * H if SIDE = 'R'.
// *> \endverbatim
// *>
// *> \param[in] LDC
// *> \verbatim
// *>          LDC is INTEGER
// *>          The leading dimension of the array C. LDC >= max(1,M).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension
// *>                         (N) if SIDE = 'L'
// *>                      or (M) if SIDE = 'R'
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlarf (String side,
int m,
int n,
double [] v, int _v_offset,
int incv,
double tau,
double [] c, int _c_offset,
int ldc,
double [] work, int _work_offset)  {

label_999999:{

boolean applyleft= false;
int i= 0;
int lastv= 0;
int lastc= 0;
side = side.substring((1)-1,1);
applyleft = com.polytechnik.lapack310.Lsame.lsame(side,"L");
lastv = 0;
lastc = 0;
/***
!     Set up variables for scanning V.  LASTV begins pointing to the end
!     of V.
***/
if ((tau != 0.0))  {
    if (applyleft)  {
    lastv = m;
}
else  {
  lastv = n;
}              //  Close else.
if ((incv > 0))  {
    i = (1+(((lastv-1))*incv));
}
else  {
  i = 1;
// !     Look for the last non-zero row in V.
}              //  Close else.
{
while (((lastv > 0) && (v[(i-(1))+ _v_offset] == 0.0)))  {
    lastv = (lastv-1);
i = (i-incv);
//}
//else  {
//  Dummy.go_to("com/polytechnik/lapack310/Dlarf",1774);
//}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dlarf",1000000);
}              //  Close for() loop.
}
//label1774:
//   Dummy.label("com/polytechnik/lapack310/Dlarf",1774);
// !     Scan for the last non-zero column in C(1:lastv,:).
if (applyleft)  {
    lastc = com.polytechnik.lapack310.Iladlc.iladlc(lastv,n,c,_c_offset,ldc);
// !     Scan for the last non-zero row in C(:,1:lastv).
}
else  {
  lastc = com.polytechnik.lapack310.Iladlr.iladlr(m,lastv,c,_c_offset,ldc);
}              //  Close else.
/***
!     Note that lastc.eq.0 renders the BLAS operations null; no special
!     case is needed at this level.
***/
}
/***
*
*        Form  H * C
*
***/
if (applyleft)  {
    /***
*
*           w(1:lastc,1) := C(1:lastv,1:lastc)**T * v(1:lastv,1)
*
***/
if ((lastv > 0))  {
    /***
*
*           C(1:lastv,1:lastc) := C(...) - v(1:lastv,1) * w(1:lastc,1)**T
*
***/
com.polytechnik.lapack310.Dgemv.dgemv("Transpose",lastv,lastc,1.0,c,_c_offset,ldc,v,_v_offset,incv,0.0,work,_work_offset,1);
com.polytechnik.lapack310.Dger.dger(lastv,lastc,(-(tau)),v,_v_offset,incv,work,_work_offset,1,c,_c_offset,ldc);
}
/***
*
*        Form  C * H
*
***/
}
else  {
  /***
*
*           w(1:lastc,1) := C(1:lastc,1:lastv) * v(1:lastv,1)
*
***/
if ((lastv > 0))  {
    /***
*
*           C(1:lastc,1:lastv) := C(...) - w(1:lastc,1) * v(1:lastv,1)**T
*
***/
com.polytechnik.lapack310.Dgemv.dgemv("No transpose",lastc,lastv,1.0,c,_c_offset,ldc,v,_v_offset,incv,0.0,work,_work_offset,1);
com.polytechnik.lapack310.Dger.dger(lastc,lastv,(-(tau)),work,_work_offset,1,v,_v_offset,incv,c,_c_offset,ldc);
}
}              //  Close else.
/***
*
*     End of DLARF
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlarf",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlarf",999999);
return;
   }
} // End class.
