/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarfb.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlarfb {

// *> \brief \b DLARFB applies a block reflector or its transpose to a gene
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLARFB + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARFB( SIDE, TRANS, DIRECT, STOREV, M, N, K, V, LDV,
// *                          T, LDT, C, LDC, WORK, LDWORK )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          DIRECT, SIDE, STOREV, TRANS
// *       INTEGER            K, LDC, LDT, LDV, LDWORK, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   C( LDC, * ), T( LDT, * ), V( LDV, * ),
// *      $                   WORK( LDWORK, * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARFB applies a real block reflector H or its transpose H**T to a
// *> real m by n matrix C, from either the left or the right.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] SIDE
// *> \verbatim
// *>          SIDE is CHARACTER*1
// *>          = 'L': apply H or H**T from the Left
// *>          = 'R': apply H or H**T from the Right
// *> \endverbatim
// *>
// *> \param[in] TRANS
// *> \verbatim
// *>          TRANS is CHARACTER*1
// *>          = 'N': apply H (No transpose)
// *>          = 'T': apply H**T (Transpose)
// *> \endverbatim
// *>
// *> \param[in] DIRECT
// *> \verbatim
// *>          DIRECT is CHARACTER*1
// *>          Indicates how H is formed from a product of elementary
// *>          reflectors
// *>          = 'F': H = H(1) H(2) . . . H(k) (Forward)
// *>          = 'B': H = H(k) . . . H(2) H(1) (Backward)
// *> \endverbatim
// *>
// *> \param[in] STOREV
// *> \verbatim
// *>          STOREV is CHARACTER*1
// *>          Indicates how the vectors which define the elementary
// *>          reflectors are stored:
// *>          = 'C': Columnwise
// *>          = 'R': Rowwise
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix C.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix C.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>          The order of the matrix T (= the number of elementary
// *>          reflectors whose product defines the block reflector).
// *>          If SIDE = 'L', M >= K >= 0;
// *>          if SIDE = 'R', N >= K >= 0.
// *> \endverbatim
// *>
// *> \param[in] V
// *> \verbatim
// *>          V is DOUBLE PRECISION array, dimension
// *>                                (LDV,K) if STOREV = 'C'
// *>                                (LDV,M) if STOREV = 'R' and SIDE = 'L'
// *>                                (LDV,N) if STOREV = 'R' and SIDE = 'R'
// *>          The matrix V. See Further Details.
// *> \endverbatim
// *>
// *> \param[in] LDV
// *> \verbatim
// *>          LDV is INTEGER
// *>          The leading dimension of the array V.
// *>          If STOREV = 'C' and SIDE = 'L', LDV >= max(1,M);
// *>          if STOREV = 'C' and SIDE = 'R', LDV >= max(1,N);
// *>          if STOREV = 'R', LDV >= K.
// *> \endverbatim
// *>
// *> \param[in] T
// *> \verbatim
// *>          T is DOUBLE PRECISION array, dimension (LDT,K)
// *>          The triangular k by k matrix T in the representation of the

// *>          block reflector.
// *> \endverbatim
// *>
// *> \param[in] LDT
// *> \verbatim
// *>          LDT is INTEGER
// *>          The leading dimension of the array T. LDT >= K.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension (LDC,N)
// *>          On entry, the m by n matrix C.
// *>          On exit, C is overwritten by H*C or H**T*C or C*H or C*H**T.
// *> \endverbatim
// *>
// *> \param[in] LDC
// *> \verbatim
// *>          LDC is INTEGER
// *>          The leading dimension of the array C. LDC >= max(1,M).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (LDWORK,K)
// *> \endverbatim
// *>
// *> \param[in] LDWORK
// *> \verbatim
// *>          LDWORK is INTEGER
// *>          The leading dimension of the array WORK.
// *>          If SIDE = 'L', LDWORK >= max(1,N);
// *>          if SIDE = 'R', LDWORK >= max(1,M).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  The shape of the matrix V and the storage of the vectors which defin
// *>  the H(i) is best illustrated by the following example with n = 5 and
// *>  k = 3. The elements equal to 1 are not stored; the corresponding
// *>  array elements are modified but restored on exit. The rest of the
// *>  array is not used.
// *>
// *>  DIRECT = 'F' and STOREV = 'C':         DIRECT = 'F' and STOREV = 'R'
// *>
// *>               V = (  1       )                 V = (  1 v1 v1 v1 v1 )
// *>                   ( v1  1    )                     (     1 v2 v2 v2 )
// *>                   ( v1 v2  1 )                     (        1 v3 v3 )
// *>                   ( v1 v2 v3 )
// *>                   ( v1 v2 v3 )
// *>
// *>  DIRECT = 'B' and STOREV = 'C':         DIRECT = 'B' and STOREV = 'R'
// *>
// *>               V = ( v1 v2 v3 )                 V = ( v1 v1  1       )
// *>                   ( v1 v2 v3 )                     ( v2 v2 v2  1    )
// *>                   (  1 v2 v3 )                     ( v3 v3 v3 v3  1 )
// *>                   (     1 v3 )
// *>                   (        1 )
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Quick return if possible
*
***/
// *

public static void dlarfb (String side,
String trans,
String direct,
String storev,
int m,
int n,
int k,
double [] v, int _v_offset,
int ldv,
double [] t, int _t_offset,
int ldt,
double [] c, int _c_offset,
int ldc,
double [] work, int _work_offset,
int ldwork)  {

label_999999:{

String transt= new String(" ");
int i= 0;
int j= 0;
side = side.substring((1)-1,1);
trans = trans.substring((1)-1,1);
direct = direct.substring((1)-1,1);
storev = storev.substring((1)-1,1);
if (((m <= 0) || (n <= 0))) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlarfb",999999);
}
    if (com.polytechnik.lapack310.Lsame.lsame(trans,"N"))  {
    transt = "T";
}
else  {
  transt = "N";
// *
}              //  Close else.
// *
if (com.polytechnik.lapack310.Lsame.lsame(storev,"C"))  {
    /***
*
*           Let  V =  ( V1 )    (first K rows)
*                     ( V2 )
*           where  V1  is unit lower triangular.
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(direct,"F"))  {
    /***
*
*              Form  H * C  or  H**T * C  where  C = ( C1 )
*                                                    ( C2 )
*
*              W := C**T * V  =  (C1**T * V1 + C2**T * V2)  (stored in WORK)
*
*              W := C1**T
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(side,"L"))  {
    {
for (j = 1; j <= k; j++) {
com.polytechnik.lapack310.Dcopy.dcopy(n,c,(j-(1))+(1-(1)) * (ldc)+ _c_offset,ldc,work,(1-(1))+(j-(1)) * (ldwork)+ _work_offset,1);
/***
*
*              W := W * V1
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlarfb",10);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower","No transpose","Unit",n,k,1.0,v,_v_offset,ldv,work,_work_offset,ldwork);
/***
*
*                 W := W + C2**T * V2
*
***/
if ((m > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("Transpose","No transpose",n,k,(m-k),1.0,c,((k+1)-(1))+(1-(1)) * (ldc)+ _c_offset,ldc,v,((k+1)-(1))+(1-(1)) * (ldv)+ _v_offset,ldv,1.0,work,_work_offset,ldwork);
/***
*
*              W := W * T**T  or  W * T
*
***/
}
/***
*
*              C := C - V * W**T
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper",transt,"Non-unit",n,k,1.0,t,_t_offset,ldt,work,_work_offset,ldwork);
/***
*
*                 C2 := C2 - V2 * W**T
*
***/
if ((m > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("No transpose","Transpose",(m-k),n,k,-1.0,v,((k+1)-(1))+(1-(1)) * (ldv)+ _v_offset,ldv,work,_work_offset,ldwork,1.0,c,((k+1)-(1))+(1-(1)) * (ldc)+ _c_offset,ldc);
/***
*
*              W := W * V1**T
*
***/
}
/***
*
*              C1 := C1 - W**T
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower","Transpose","Unit",n,k,1.0,v,_v_offset,ldv,work,_work_offset,ldwork);
{
for (j = 1; j <= k; j++) {
{
for (i = 1; i <= n; i++) {
c[(j-(1))+(i-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(i-(1)) * (ldc)+ _c_offset]-work[(i-(1))+(j-(1)) * (ldwork)+ _work_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlarfb",20);
}              //  Close for() loop.
}
// *
//Dummy.label("com/polytechnik/lapack310/Dlarfb",30);
}              //  Close for() loop.
}
/***
*
*              Form  C * H  or  C * H**T  where  C = ( C1  C2 )
*
*              W := C * V  =  (C1*V1 + C2*V2)  (stored in WORK)
*
*              W := C1
*
***/
}
else if (com.polytechnik.lapack310.Lsame.lsame(side,"R"))  {
    {
for (j = 1; j <= k; j++) {
com.polytechnik.lapack310.Dcopy.dcopy(m,c,(1-(1))+(j-(1)) * (ldc)+ _c_offset,1,work,(1-(1))+(j-(1)) * (ldwork)+ _work_offset,1);
/***
*
*              W := W * V1
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlarfb",40);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower","No transpose","Unit",m,k,1.0,v,_v_offset,ldv,work,_work_offset,ldwork);
/***
*
*                 W := W + C2 * V2
*
***/
if ((n > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("No transpose","No transpose",m,k,(n-k),1.0,c,(1-(1))+((k+1)-(1)) * (ldc)+ _c_offset,ldc,v,((k+1)-(1))+(1-(1)) * (ldv)+ _v_offset,ldv,1.0,work,_work_offset,ldwork);
/***
*
*              W := W * T  or  W * T**T
*
***/
}
/***
*
*              C := C - W * V**T
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper",trans,"Non-unit",m,k,1.0,t,_t_offset,ldt,work,_work_offset,ldwork);
/***
*
*                 C2 := C2 - W * V2**T
*
***/
if ((n > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("No transpose","Transpose",m,(n-k),k,-1.0,work,_work_offset,ldwork,v,((k+1)-(1))+(1-(1)) * (ldv)+ _v_offset,ldv,1.0,c,(1-(1))+((k+1)-(1)) * (ldc)+ _c_offset,ldc);
/***
*
*              W := W * V1**T
*
***/
}
/***
*
*              C1 := C1 - W
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower","Transpose","Unit",m,k,1.0,v,_v_offset,ldv,work,_work_offset,ldwork);
{
for (j = 1; j <= k; j++) {
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]-work[(i-(1))+(j-(1)) * (ldwork)+ _work_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlarfb",50);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dlarfb",60);
}              //  Close for() loop.
}
// *
}              // Close else if()
/***
*
*           Let  V =  ( V1 )
*                     ( V2 )    (last K rows)
*           where  V2  is unit upper triangular.
*
***/
}
else  {
  /***
*
*              Form  H * C  or  H**T * C  where  C = ( C1 )
*                                                    ( C2 )
*
*              W := C**T * V  =  (C1**T * V1 + C2**T * V2)  (stored in WORK)
*
*              W := C2**T
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(side,"L"))  {
    {
for (j = 1; j <= k; j++) {
com.polytechnik.lapack310.Dcopy.dcopy(n,c,(((m-k)+j)-(1))+(1-(1)) * (ldc)+ _c_offset,ldc,work,(1-(1))+(j-(1)) * (ldwork)+ _work_offset,1);
/***
*
*              W := W * V2
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlarfb",70);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper","No transpose","Unit",n,k,1.0,v,(((m-k)+1)-(1))+(1-(1)) * (ldv)+ _v_offset,ldv,work,_work_offset,ldwork);
/***
*
*                 W := W + C1**T * V1
*
***/
if ((m > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("Transpose","No transpose",n,k,(m-k),1.0,c,_c_offset,ldc,v,_v_offset,ldv,1.0,work,_work_offset,ldwork);
/***
*
*              W := W * T**T  or  W * T
*
***/
}
/***
*
*              C := C - V * W**T
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower",transt,"Non-unit",n,k,1.0,t,_t_offset,ldt,work,_work_offset,ldwork);
/***
*
*                 C1 := C1 - V1 * W**T
*
***/
if ((m > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("No transpose","Transpose",(m-k),n,k,-1.0,v,_v_offset,ldv,work,_work_offset,ldwork,1.0,c,_c_offset,ldc);
/***
*
*              W := W * V2**T
*
***/
}
/***
*
*              C2 := C2 - W**T
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper","Transpose","Unit",n,k,1.0,v,(((m-k)+1)-(1))+(1-(1)) * (ldv)+ _v_offset,ldv,work,_work_offset,ldwork);
{
for (j = 1; j <= k; j++) {
{
for (i = 1; i <= n; i++) {
c[(((m-k)+j)-(1))+(i-(1)) * (ldc)+ _c_offset] = (c[(((m-k)+j)-(1))+(i-(1)) * (ldc)+ _c_offset]-work[(i-(1))+(j-(1)) * (ldwork)+ _work_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlarfb",80);
}              //  Close for() loop.
}
// *
//Dummy.label("com/polytechnik/lapack310/Dlarfb",90);
}              //  Close for() loop.
}
/***
*
*              Form  C * H  or  C * H**T  where  C = ( C1  C2 )
*
*              W := C * V  =  (C1*V1 + C2*V2)  (stored in WORK)
*
*              W := C2
*
***/
}
else if (com.polytechnik.lapack310.Lsame.lsame(side,"R"))  {
    {
for (j = 1; j <= k; j++) {
com.polytechnik.lapack310.Dcopy.dcopy(m,c,(1-(1))+(((n-k)+j)-(1)) * (ldc)+ _c_offset,1,work,(1-(1))+(j-(1)) * (ldwork)+ _work_offset,1);
/***
*
*              W := W * V2
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlarfb",100);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper","No transpose","Unit",m,k,1.0,v,(((n-k)+1)-(1))+(1-(1)) * (ldv)+ _v_offset,ldv,work,_work_offset,ldwork);
/***
*
*                 W := W + C1 * V1
*
***/
if ((n > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("No transpose","No transpose",m,k,(n-k),1.0,c,_c_offset,ldc,v,_v_offset,ldv,1.0,work,_work_offset,ldwork);
/***
*
*              W := W * T  or  W * T**T
*
***/
}
/***
*
*              C := C - W * V**T
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower",trans,"Non-unit",m,k,1.0,t,_t_offset,ldt,work,_work_offset,ldwork);
/***
*
*                 C1 := C1 - W * V1**T
*
***/
if ((n > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("No transpose","Transpose",m,(n-k),k,-1.0,work,_work_offset,ldwork,v,_v_offset,ldv,1.0,c,_c_offset,ldc);
/***
*
*              W := W * V2**T
*
***/
}
/***
*
*              C2 := C2 - W
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper","Transpose","Unit",m,k,1.0,v,(((n-k)+1)-(1))+(1-(1)) * (ldv)+ _v_offset,ldv,work,_work_offset,ldwork);
{
for (j = 1; j <= k; j++) {
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(((n-k)+j)-(1)) * (ldc)+ _c_offset] = (c[(i-(1))+(((n-k)+j)-(1)) * (ldc)+ _c_offset]-work[(i-(1))+(j-(1)) * (ldwork)+ _work_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlarfb",110);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dlarfb",120);
}              //  Close for() loop.
}
}              // Close else if()
// *
}              //  Close else.
// *
}
else if (com.polytechnik.lapack310.Lsame.lsame(storev,"R"))  {
    /***
*
*           Let  V =  ( V1  V2 )    (V1: first K columns)
*           where  V1  is unit upper triangular.
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(direct,"F"))  {
    /***
*
*              Form  H * C  or  H**T * C  where  C = ( C1 )
*                                                    ( C2 )
*
*              W := C**T * V**T  =  (C1**T * V1**T + C2**T * V2**T) (stored in WORK)
*
*              W := C1**T
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(side,"L"))  {
    {
for (j = 1; j <= k; j++) {
com.polytechnik.lapack310.Dcopy.dcopy(n,c,(j-(1))+(1-(1)) * (ldc)+ _c_offset,ldc,work,(1-(1))+(j-(1)) * (ldwork)+ _work_offset,1);
/***
*
*              W := W * V1**T
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlarfb",130);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper","Transpose","Unit",n,k,1.0,v,_v_offset,ldv,work,_work_offset,ldwork);
/***
*
*                 W := W + C2**T * V2**T
*
***/
if ((m > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("Transpose","Transpose",n,k,(m-k),1.0,c,((k+1)-(1))+(1-(1)) * (ldc)+ _c_offset,ldc,v,(1-(1))+((k+1)-(1)) * (ldv)+ _v_offset,ldv,1.0,work,_work_offset,ldwork);
/***
*
*              W := W * T**T  or  W * T
*
***/
}
/***
*
*              C := C - V**T * W**T
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper",transt,"Non-unit",n,k,1.0,t,_t_offset,ldt,work,_work_offset,ldwork);
/***
*
*                 C2 := C2 - V2**T * W**T
*
***/
if ((m > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("Transpose","Transpose",(m-k),n,k,-1.0,v,(1-(1))+((k+1)-(1)) * (ldv)+ _v_offset,ldv,work,_work_offset,ldwork,1.0,c,((k+1)-(1))+(1-(1)) * (ldc)+ _c_offset,ldc);
/***
*
*              W := W * V1
*
***/
}
/***
*
*              C1 := C1 - W**T
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper","No transpose","Unit",n,k,1.0,v,_v_offset,ldv,work,_work_offset,ldwork);
{
for (j = 1; j <= k; j++) {
{
for (i = 1; i <= n; i++) {
c[(j-(1))+(i-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(i-(1)) * (ldc)+ _c_offset]-work[(i-(1))+(j-(1)) * (ldwork)+ _work_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlarfb",140);
}              //  Close for() loop.
}
// *
//Dummy.label("com/polytechnik/lapack310/Dlarfb",150);
}              //  Close for() loop.
}
/***
*
*              Form  C * H  or  C * H**T  where  C = ( C1  C2 )
*
*              W := C * V**T  =  (C1*V1**T + C2*V2**T)  (stored in WORK)
*
*              W := C1
*
***/
}
else if (com.polytechnik.lapack310.Lsame.lsame(side,"R"))  {
    {
for (j = 1; j <= k; j++) {
com.polytechnik.lapack310.Dcopy.dcopy(m,c,(1-(1))+(j-(1)) * (ldc)+ _c_offset,1,work,(1-(1))+(j-(1)) * (ldwork)+ _work_offset,1);
/***
*
*              W := W * V1**T
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlarfb",160);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper","Transpose","Unit",m,k,1.0,v,_v_offset,ldv,work,_work_offset,ldwork);
/***
*
*                 W := W + C2 * V2**T
*
***/
if ((n > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("No transpose","Transpose",m,k,(n-k),1.0,c,(1-(1))+((k+1)-(1)) * (ldc)+ _c_offset,ldc,v,(1-(1))+((k+1)-(1)) * (ldv)+ _v_offset,ldv,1.0,work,_work_offset,ldwork);
/***
*
*              W := W * T  or  W * T**T
*
***/
}
/***
*
*              C := C - W * V
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper",trans,"Non-unit",m,k,1.0,t,_t_offset,ldt,work,_work_offset,ldwork);
/***
*
*                 C2 := C2 - W * V2
*
***/
if ((n > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("No transpose","No transpose",m,(n-k),k,-1.0,work,_work_offset,ldwork,v,(1-(1))+((k+1)-(1)) * (ldv)+ _v_offset,ldv,1.0,c,(1-(1))+((k+1)-(1)) * (ldc)+ _c_offset,ldc);
/***
*
*              W := W * V1
*
***/
}
/***
*
*              C1 := C1 - W
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Upper","No transpose","Unit",m,k,1.0,v,_v_offset,ldv,work,_work_offset,ldwork);
{
for (j = 1; j <= k; j++) {
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]-work[(i-(1))+(j-(1)) * (ldwork)+ _work_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlarfb",170);
}              //  Close for() loop.
}
// *
//Dummy.label("com/polytechnik/lapack310/Dlarfb",180);
}              //  Close for() loop.
}
// *
}              // Close else if()
/***
*
*           Let  V =  ( V1  V2 )    (V2: last K columns)
*           where  V2  is unit lower triangular.
*
***/
}
else  {
  /***
*
*              Form  H * C  or  H**T * C  where  C = ( C1 )
*                                                    ( C2 )
*
*              W := C**T * V**T  =  (C1**T * V1**T + C2**T * V2**T) (stored in WORK)
*
*              W := C2**T
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(side,"L"))  {
    {
for (j = 1; j <= k; j++) {
com.polytechnik.lapack310.Dcopy.dcopy(n,c,(((m-k)+j)-(1))+(1-(1)) * (ldc)+ _c_offset,ldc,work,(1-(1))+(j-(1)) * (ldwork)+ _work_offset,1);
/***
*
*              W := W * V2**T
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlarfb",190);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower","Transpose","Unit",n,k,1.0,v,(1-(1))+(((m-k)+1)-(1)) * (ldv)+ _v_offset,ldv,work,_work_offset,ldwork);
/***
*
*                 W := W + C1**T * V1**T
*
***/
if ((m > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("Transpose","Transpose",n,k,(m-k),1.0,c,_c_offset,ldc,v,_v_offset,ldv,1.0,work,_work_offset,ldwork);
/***
*
*              W := W * T**T  or  W * T
*
***/
}
/***
*
*              C := C - V**T * W**T
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower",transt,"Non-unit",n,k,1.0,t,_t_offset,ldt,work,_work_offset,ldwork);
/***
*
*                 C1 := C1 - V1**T * W**T
*
***/
if ((m > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("Transpose","Transpose",(m-k),n,k,-1.0,v,_v_offset,ldv,work,_work_offset,ldwork,1.0,c,_c_offset,ldc);
/***
*
*              W := W * V2
*
***/
}
/***
*
*              C2 := C2 - W**T
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower","No transpose","Unit",n,k,1.0,v,(1-(1))+(((m-k)+1)-(1)) * (ldv)+ _v_offset,ldv,work,_work_offset,ldwork);
{
for (j = 1; j <= k; j++) {
{
for (i = 1; i <= n; i++) {
c[(((m-k)+j)-(1))+(i-(1)) * (ldc)+ _c_offset] = (c[(((m-k)+j)-(1))+(i-(1)) * (ldc)+ _c_offset]-work[(i-(1))+(j-(1)) * (ldwork)+ _work_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlarfb",200);
}              //  Close for() loop.
}
// *
//Dummy.label("com/polytechnik/lapack310/Dlarfb",210);
}              //  Close for() loop.
}
/***
*
*              Form  C * H  or  C * H'  where  C = ( C1  C2 )
*
*              W := C * V**T  =  (C1*V1**T + C2*V2**T)  (stored in WORK)
*
*              W := C2
*
***/
}
else if (com.polytechnik.lapack310.Lsame.lsame(side,"R"))  {
    {
for (j = 1; j <= k; j++) {
com.polytechnik.lapack310.Dcopy.dcopy(m,c,(1-(1))+(((n-k)+j)-(1)) * (ldc)+ _c_offset,1,work,(1-(1))+(j-(1)) * (ldwork)+ _work_offset,1);
/***
*
*              W := W * V2**T
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlarfb",220);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower","Transpose","Unit",m,k,1.0,v,(1-(1))+(((n-k)+1)-(1)) * (ldv)+ _v_offset,ldv,work,_work_offset,ldwork);
/***
*
*                 W := W + C1 * V1**T
*
***/
if ((n > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("No transpose","Transpose",m,k,(n-k),1.0,c,_c_offset,ldc,v,_v_offset,ldv,1.0,work,_work_offset,ldwork);
/***
*
*              W := W * T  or  W * T**T
*
***/
}
/***
*
*              C := C - W * V
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower",trans,"Non-unit",m,k,1.0,t,_t_offset,ldt,work,_work_offset,ldwork);
/***
*
*                 C1 := C1 - W * V1
*
***/
if ((n > k))  {
    com.polytechnik.lapack310.Dgemm.dgemm("No transpose","No transpose",m,(n-k),k,-1.0,work,_work_offset,ldwork,v,_v_offset,ldv,1.0,c,_c_offset,ldc);
/***
*
*              W := W * V2
*
***/
}
/***
*
*              C1 := C1 - W
*
***/
com.polytechnik.lapack310.Dtrmm.dtrmm("Right","Lower","No transpose","Unit",m,k,1.0,v,(1-(1))+(((n-k)+1)-(1)) * (ldv)+ _v_offset,ldv,work,_work_offset,ldwork);
{
for (j = 1; j <= k; j++) {
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(((n-k)+j)-(1)) * (ldc)+ _c_offset] = (c[(i-(1))+(((n-k)+j)-(1)) * (ldc)+ _c_offset]-work[(i-(1))+(j-(1)) * (ldwork)+ _work_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlarfb",230);
}              //  Close for() loop.
}
// *
//Dummy.label("com/polytechnik/lapack310/Dlarfb",240);
}              //  Close for() loop.
}
// *
}              // Close else if()
}              //  Close else.
// *
}              // Close else if()
/***
*
*     End of DLARFB
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlarfb",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlarfb",999999);
return;
   }
} // End class.
