/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarft.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlarft {

// *> \brief \b DLARFT forms the triangular factor T of a block reflector H
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLARFT + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARFT( DIRECT, STOREV, N, K, V, LDV, TAU, T, LDT )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          DIRECT, STOREV
// *       INTEGER            K, LDT, LDV, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   T( LDT, * ), TAU( * ), V( LDV, * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARFT forms the triangular factor T of a real block reflector H
// *> of order n, which is defined as a product of k elementary reflectors.
// *>
// *> If DIRECT = 'F', H = H(1) H(2) . . . H(k) and T is upper triangular;

// *>
// *> If DIRECT = 'B', H = H(k) . . . H(2) H(1) and T is lower triangular.

// *>
// *> If STOREV = 'C', the vector which defines the elementary reflector
// *> H(i) is stored in the i-th column of the array V, and
// *>
// *>    H  =  I - V * T * V**T
// *>
// *> If STOREV = 'R', the vector which defines the elementary reflector
// *> H(i) is stored in the i-th row of the array V, and
// *>
// *>    H  =  I - V**T * T * V
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] DIRECT
// *> \verbatim
// *>          DIRECT is CHARACTER*1
// *>          Specifies the order in which the elementary reflectors are
// *>          multiplied to form the block reflector:
// *>          = 'F': H = H(1) H(2) . . . H(k) (Forward)
// *>          = 'B': H = H(k) . . . H(2) H(1) (Backward)
// *> \endverbatim
// *>
// *> \param[in] STOREV
// *> \verbatim
// *>          STOREV is CHARACTER*1
// *>          Specifies how the vectors which define the elementary
// *>          reflectors are stored (see also Further Details):
// *>          = 'C': columnwise
// *>          = 'R': rowwise
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the block reflector H. N >= 0.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>          The order of the triangular factor T (= the number of
// *>          elementary reflectors). K >= 1.
// *> \endverbatim
// *>
// *> \param[in] V
// *> \verbatim
// *>          V is DOUBLE PRECISION array, dimension
// *>                               (LDV,K) if STOREV = 'C'
// *>                               (LDV,N) if STOREV = 'R'
// *>          The matrix V. See further details.
// *> \endverbatim
// *>
// *> \param[in] LDV
// *> \verbatim
// *>          LDV is INTEGER
// *>          The leading dimension of the array V.
// *>          If STOREV = 'C', LDV >= max(1,N); if STOREV = 'R', LDV >= K.
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (K)
// *>          TAU(i) must contain the scalar factor of the elementary
// *>          reflector H(i).
// *> \endverbatim
// *>
// *> \param[out] T
// *> \verbatim
// *>          T is DOUBLE PRECISION array, dimension (LDT,K)
// *>          The k by k triangular factor T of the block reflector.
// *>          If DIRECT = 'F', T is upper triangular; if DIRECT = 'B', T i
// *>          lower triangular. The rest of the array is not used.
// *> \endverbatim
// *>
// *> \param[in] LDT
// *> \verbatim
// *>          LDT is INTEGER
// *>          The leading dimension of the array T. LDT >= K.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  The shape of the matrix V and the storage of the vectors which defin
// *>  the H(i) is best illustrated by the following example with n = 5 and
// *>  k = 3. The elements equal to 1 are not stored.
// *>
// *>  DIRECT = 'F' and STOREV = 'C':         DIRECT = 'F' and STOREV = 'R'
// *>
// *>               V = (  1       )                 V = (  1 v1 v1 v1 v1 )
// *>                   ( v1  1    )                     (     1 v2 v2 v2 )
// *>                   ( v1 v2  1 )                     (        1 v3 v3 )
// *>                   ( v1 v2 v3 )
// *>                   ( v1 v2 v3 )
// *>
// *>  DIRECT = 'B' and STOREV = 'C':         DIRECT = 'B' and STOREV = 'R'
// *>
// *>               V = ( v1 v2 v3 )                 V = ( v1 v1  1       )
// *>                   ( v1 v2 v3 )                     ( v2 v2 v2  1    )
// *>                   (  1 v2 v3 )                     ( v3 v3 v3 v3  1 )
// *>                   (     1 v3 )
// *>                   (        1 )
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Quick return if possible
*
***/
// *

public static void dlarft (String direct,
String storev,
int n,
int k,
double [] v, int _v_offset,
int ldv,
double [] tau, int _tau_offset,
double [] t, int _t_offset,
int ldt)  {

label_999999:{

int i= 0;
int j= 0;
int prevlastv= 0;
int lastv= 0;
direct = direct.substring((1)-1,1);
storev = storev.substring((1)-1,1);
if ((n == 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlarft",999999);
}
    if (com.polytechnik.lapack310.Lsame.lsame(direct,"F"))  {
    prevlastv = n;
{
for (i = 1; i <= k; i++) {
prevlastv = Math.max(i, prevlastv);
/***
*
*              H(i)  =  I
*
***/
if ((tau[(i-(1))+ _tau_offset] == 0.0))  {
    {
for (j = 1; j <= i; j++) {
t[(j-(1))+(i-(1)) * (ldt)+ _t_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000001);
}              //  Close for() loop.
}
/***
*
*              general case
*
***/
}
else  {
  // *                 Skip any trailing zeros.
if (com.polytechnik.lapack310.Lsame.lsame(storev,"C"))  {
    // *                     IF( V( LASTV, I ).NE.ZERO ) EXIT
{
int _lastv_inc = -1;
for (lastv = n; lastv >= (i+1); lastv += _lastv_inc) {
if ((v[(lastv-(1))+(i-(1)) * (ldv)+ _v_offset] != 0.0)) {
    break;//Dummy.go_to("com/polytechnik/lapack310/Dlarft",1777);
}
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000002);
}              //  Close for() loop.
}
//label1777:
//   Dummy.label("com/polytechnik/lapack310/Dlarft",1777);
{
for (j = 1; j <= (i-1); j++) {
t[(j-(1))+(i-(1)) * (ldt)+ _t_offset] = (-((tau[(i-(1))+ _tau_offset]*v[(i-(1))+(j-(1)) * (ldv)+ _v_offset])));
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000003);
}              //  Close for() loop.
}
/***
*
*                 T(1:i-1,i) := - tau(i) * V(i:j,1:i-1)**T * V(i:j,i)
*
***/
j = Math.min(lastv, prevlastv);
com.polytechnik.lapack310.Dgemv.dgemv("Transpose",(j-i),(i-1),(-(tau[(i-(1))+ _tau_offset])),v,((i+1)-(1))+(1-(1)) * (ldv)+ _v_offset,ldv,v,((i+1)-(1))+(i-(1)) * (ldv)+ _v_offset,1,1.0,t,(1-(1))+(i-(1)) * (ldt)+ _t_offset,1);
// *                 Skip any trailing zeros.
}
else  {
  // *                     IF( V( I, LASTV ).NE.ZERO ) EXIT
{
int _lastv_inc = -1;
for (lastv = n; lastv >= (i+1); lastv += _lastv_inc) {
if ((v[(i-(1))+(lastv-(1)) * (ldv)+ _v_offset] != 0.0)) {
    break;//Dummy.go_to("com/polytechnik/lapack310/Dlarft",1778);
}
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000004);
}              //  Close for() loop.
}
//label1778:
//   Dummy.label("com/polytechnik/lapack310/Dlarft",1778);
{
for (j = 1; j <= (i-1); j++) {
t[(j-(1))+(i-(1)) * (ldt)+ _t_offset] = (-((tau[(i-(1))+ _tau_offset]*v[(j-(1))+(i-(1)) * (ldv)+ _v_offset])));
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000005);
}              //  Close for() loop.
}
/***
*
*                 T(1:i-1,i) := - tau(i) * V(1:i-1,i:j) * V(i,i:j)**T
*
***/
j = Math.min(lastv, prevlastv);
com.polytechnik.lapack310.Dgemv.dgemv("No transpose",(i-1),(j-i),(-(tau[(i-(1))+ _tau_offset])),v,(1-(1))+((i+1)-(1)) * (ldv)+ _v_offset,ldv,v,(i-(1))+((i+1)-(1)) * (ldv)+ _v_offset,ldv,1.0,t,(1-(1))+(i-(1)) * (ldt)+ _t_offset,1);
/***
*
*              T(1:i-1,i) := T(1:i-1,1:i-1) * T(1:i-1,i)
*
***/
}              //  Close else.
com.polytechnik.lapack310.Dtrmv.dtrmv("Upper","No transpose","Non-unit",(i-1),t,_t_offset,ldt,t,(1-(1))+(i-(1)) * (ldt)+ _t_offset,1);
t[(i-(1))+(i-(1)) * (ldt)+ _t_offset] = tau[(i-(1))+ _tau_offset];
if ((i > 1))  {
    prevlastv = Math.max(prevlastv, lastv);
}
else  {
  prevlastv = lastv;
}              //  Close else.
}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000000);
}              //  Close for() loop.
}
}
else  {
  prevlastv = 1;
{
int _i_inc = -1;
for (i = k; i >= 1; i += _i_inc) {
/***
*
*              H(i)  =  I
*
***/
if ((tau[(i-(1))+ _tau_offset] == 0.0))  {
    {
for (j = i; j <= k; j++) {
t[(j-(1))+(i-(1)) * (ldt)+ _t_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000007);
}              //  Close for() loop.
}
/***
*
*              general case
*
***/
}
else  {
  if ((i < k))  {
    // *                    Skip any leading zeros.
if (com.polytechnik.lapack310.Lsame.lsame(storev,"C"))  {
    // *                        IF( V( LASTV, I ).NE.ZERO ) EXIT
{
for (lastv = 1; lastv <= (i-1); lastv++) {
if ((v[(lastv-(1))+(i-(1)) * (ldv)+ _v_offset] != 0.0)) {
    break;//Dummy.go_to("com/polytechnik/lapack310/Dlarft",1779);
}
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000008);
}              //  Close for() loop.
}
//label1779:
//   Dummy.label("com/polytechnik/lapack310/Dlarft",1779);
{
for (j = (i+1); j <= k; j++) {
t[(j-(1))+(i-(1)) * (ldt)+ _t_offset] = (-((tau[(i-(1))+ _tau_offset]*v[(((n-k)+i)-(1))+(j-(1)) * (ldv)+ _v_offset])));
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000009);
}              //  Close for() loop.
}
/***
*
*                    T(i+1:k,i) = -tau(i) * V(j:n-k+i,i+1:k)**T * V(j:n-k+i,i)
*
***/
j = Math.max(lastv, prevlastv);
com.polytechnik.lapack310.Dgemv.dgemv("Transpose",(((n-k)+i)-j),(k-i),(-(tau[(i-(1))+ _tau_offset])),v,(j-(1))+((i+1)-(1)) * (ldv)+ _v_offset,ldv,v,(j-(1))+(i-(1)) * (ldv)+ _v_offset,1,1.0,t,((i+1)-(1))+(i-(1)) * (ldt)+ _t_offset,1);
// *                    Skip any leading zeros.
}
else  {
  // *                        IF( V( I, LASTV ).NE.ZERO ) EXIT
{
for (lastv = 1; lastv <= (i-1); lastv++) {
if ((v[(i-(1))+(lastv-(1)) * (ldv)+ _v_offset] != 0.0)) {
    break;//Dummy.go_to("com/polytechnik/lapack310/Dlarft",1780);
}
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000010);
}              //  Close for() loop.
}
//label1780:
//   Dummy.label("com/polytechnik/lapack310/Dlarft",1780);
{
for (j = (i+1); j <= k; j++) {
t[(j-(1))+(i-(1)) * (ldt)+ _t_offset] = (-((tau[(i-(1))+ _tau_offset]*v[(j-(1))+(((n-k)+i)-(1)) * (ldv)+ _v_offset])));
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000011);
}              //  Close for() loop.
}
/***
*
*                    T(i+1:k,i) = -tau(i) * V(i+1:k,j:n-k+i) * V(i,j:n-k+i)**T
*
***/
j = Math.max(lastv, prevlastv);
com.polytechnik.lapack310.Dgemv.dgemv("No transpose",(k-i),(((n-k)+i)-j),(-(tau[(i-(1))+ _tau_offset])),v,((i+1)-(1))+(j-(1)) * (ldv)+ _v_offset,ldv,v,(i-(1))+(j-(1)) * (ldv)+ _v_offset,ldv,1.0,t,((i+1)-(1))+(i-(1)) * (ldt)+ _t_offset,1);
/***
*
*                 T(i+1:k,i) := T(i+1:k,i+1:k) * T(i+1:k,i)
*
***/
}              //  Close else.
com.polytechnik.lapack310.Dtrmv.dtrmv("Lower","No transpose","Non-unit",(k-i),t,((i+1)-(1))+((i+1)-(1)) * (ldt)+ _t_offset,ldt,t,((i+1)-(1))+(i-(1)) * (ldt)+ _t_offset,1);
if ((i > 1))  {
    prevlastv = Math.min(prevlastv, lastv);
}
else  {
  prevlastv = lastv;
}              //  Close else.
}
t[(i-(1))+(i-(1)) * (ldt)+ _t_offset] = tau[(i-(1))+ _tau_offset];
}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dlarft",1000006);
}              //  Close for() loop.
}
}              //  Close else.
/***
*
*     End of DLARFT
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlarft",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlarft",999999);
return;
   }
} // End class.
