/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarfx.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlarfx {

// *> \brief \b DLARFX applies an elementary reflector to a general rectang
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLARFX + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARFX( SIDE, M, N, V, TAU, C, LDC, WORK )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          SIDE
// *       INTEGER            LDC, M, N
// *       DOUBLE PRECISION   TAU
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   C( LDC, * ), V( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARFX applies a real elementary reflector H to a real m by n
// *> matrix C, from either the left or the right. H is represented in the

// *> form
// *>
// *>       H = I - tau * v * v**T
// *>
// *> where tau is a real scalar and v is a real vector.
// *>
// *> If tau = 0, then H is taken to be the unit matrix
// *>
// *> This version uses inline code if H has order < 11.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] SIDE
// *> \verbatim
// *>          SIDE is CHARACTER*1
// *>          = 'L': form  H * C
// *>          = 'R': form  C * H
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix C.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix C.
// *> \endverbatim
// *>
// *> \param[in] V
// *> \verbatim
// *>          V is DOUBLE PRECISION array, dimension (M) if SIDE = 'L'
// *>                                     or (N) if SIDE = 'R'
// *>          The vector v in the representation of H.
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION
// *>          The value tau in the representation of H.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension (LDC,N)
// *>          On entry, the m by n matrix C.
// *>          On exit, C is overwritten by the matrix H * C if SIDE = 'L',
// *>          or C * H if SIDE = 'R'.
// *> \endverbatim
// *>
// *> \param[in] LDC
// *> \verbatim
// *>          LDC is INTEGER
// *>          The leading dimension of the array C. LDC >= (1,M).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension
// *>                      (N) if SIDE = 'L'
// *>                      or (M) if SIDE = 'R'
// *>          WORK is not referenced if H has order < 11.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlarfx (String side,
int m,
int n,
double [] v, int _v_offset,
double tau,
double [] c, int _c_offset,
int ldc,
double [] work, int _work_offset)  {

label_999999:{

int j= 0;
double sum= 0.0d;
double t1= 0.0d;
double t10= 0.0d;
double t2= 0.0d;
double t3= 0.0d;
double t4= 0.0d;
double t5= 0.0d;
double t6= 0.0d;
double t7= 0.0d;
double t8= 0.0d;
double t9= 0.0d;
double v1= 0.0d;
double v10= 0.0d;
double v2= 0.0d;
double v3= 0.0d;
double v4= 0.0d;
double v5= 0.0d;
double v6= 0.0d;
double v7= 0.0d;
double v8= 0.0d;
double v9= 0.0d;
side = side.substring((1)-1,1);
if ((tau == 0.0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",999999);
}

label410:{
/***
*
*        Form  H * C, where H has order m.
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(side,"L"))  {
    /***
*
*        Code for general M
*
***/
label190:{
label170:{
label150:{
label130:{
label110:{
label90:{
label70:{
label50:{
label30:{
label10:{
{
  int _cg_tmp = m;
if (_cg_tmp == 1)
    break label10;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",10);
else if (_cg_tmp == 2)
    break label30;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",30);
else if (_cg_tmp == 3)
    break label50;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",50);
else if (_cg_tmp == 4)
    break label70;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",70);
else if (_cg_tmp == 5)
    break label90;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",90);
else if (_cg_tmp == 6)
    break label110;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",110);
else if (_cg_tmp == 7)
    break label130;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",130);
else if (_cg_tmp == 8)
    break label150;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",150);
else if (_cg_tmp == 9)
    break label170;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",170);
else if (_cg_tmp == 10)
    break label190;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",190);
}
com.polytechnik.lapack310.Dlarf.dlarf(side,m,n,v,_v_offset,1,tau,c,_c_offset,ldc,work,_work_offset);
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 1 x 1 Householder
*
***/
}// END label10
    //Dummy.label("com/polytechnik/lapack310/Dlarfx",10);
t1 = (1.0-((tau*v[(1-(1))+ _v_offset])*v[(1-(1))+ _v_offset]));
{
for (j = 1; j <= n; j++) {
c[(1-(1))+(j-(1)) * (ldc)+ _c_offset] = (t1*c[(1-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlarfx",20);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 2 x 2 Householder
*
***/
}// END label30
    //Dummy.label("com/polytechnik/lapack310/Dlarfx",30);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
{
for (j = 1; j <= n; j++) {
sum = ((v1*c[(1-(1))+(j-(1)) * (ldc)+ _c_offset])+(v2*c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]));
c[(1-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(1-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(2-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t2));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",40);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 3 x 3 Householder
*
***/
}// END label50
//Dummy.label("com/polytechnik/lapack310/Dlarfx",50);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
{
for (j = 1; j <= n; j++) {
sum = (((v1*c[(1-(1))+(j-(1)) * (ldc)+ _c_offset])+(v2*c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v3*c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]));
c[(1-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(1-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(2-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(3-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t3));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",60);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 4 x 4 Householder
*
***/
}// END label70
//Dummy.label("com/polytechnik/lapack310/Dlarfx",70);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
{
for (j = 1; j <= n; j++) {
sum = ((((v1*c[(1-(1))+(j-(1)) * (ldc)+ _c_offset])+(v2*c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v3*c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v4*c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]));
c[(1-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(1-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(2-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(3-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(4-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t4));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",80);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 5 x 5 Householder
*
***/
}// END label90
//Dummy.label("com/polytechnik/lapack310/Dlarfx",90);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
{
for (j = 1; j <= n; j++) {
sum = (((((v1*c[(1-(1))+(j-(1)) * (ldc)+ _c_offset])+(v2*c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v3*c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v4*c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v5*c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]));
c[(1-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(1-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(2-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(3-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(4-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(5-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t5));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",100);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 6 x 6 Householder
*
***/
}// END label110
//Dummy.label("com/polytechnik/lapack310/Dlarfx",110);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
v6 = v[(6-(1))+ _v_offset];
t6 = (tau*v6);
{
for (j = 1; j <= n; j++) {
sum = ((((((v1*c[(1-(1))+(j-(1)) * (ldc)+ _c_offset])+(v2*c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v3*c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v4*c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v5*c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v6*c[(6-(1))+(j-(1)) * (ldc)+ _c_offset]));
c[(1-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(1-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(2-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(3-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(4-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(5-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t5));
c[(6-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(6-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t6));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",120);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 7 x 7 Householder
*
***/
}// END label130
//Dummy.label("com/polytechnik/lapack310/Dlarfx",130);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
v6 = v[(6-(1))+ _v_offset];
t6 = (tau*v6);
v7 = v[(7-(1))+ _v_offset];
t7 = (tau*v7);
{
for (j = 1; j <= n; j++) {
sum = (((((((v1*c[(1-(1))+(j-(1)) * (ldc)+ _c_offset])+(v2*c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v3*c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v4*c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v5*c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v6*c[(6-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v7*c[(7-(1))+(j-(1)) * (ldc)+ _c_offset]));
c[(1-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(1-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(2-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(3-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(4-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(5-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t5));
c[(6-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(6-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t6));
c[(7-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(7-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t7));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",140);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 8 x 8 Householder
*
***/
}// END label150
//Dummy.label("com/polytechnik/lapack310/Dlarfx",150);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
v6 = v[(6-(1))+ _v_offset];
t6 = (tau*v6);
v7 = v[(7-(1))+ _v_offset];
t7 = (tau*v7);
v8 = v[(8-(1))+ _v_offset];
t8 = (tau*v8);
{
for (j = 1; j <= n; j++) {
sum = ((((((((v1*c[(1-(1))+(j-(1)) * (ldc)+ _c_offset])+(v2*c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v3*c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v4*c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v5*c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v6*c[(6-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v7*c[(7-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v8*c[(8-(1))+(j-(1)) * (ldc)+ _c_offset]));
c[(1-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(1-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(2-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(3-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(4-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(5-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t5));
c[(6-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(6-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t6));
c[(7-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(7-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t7));
c[(8-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(8-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t8));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",160);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 9 x 9 Householder
*
***/
}// END label170
//Dummy.label("com/polytechnik/lapack310/Dlarfx",170);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
v6 = v[(6-(1))+ _v_offset];
t6 = (tau*v6);
v7 = v[(7-(1))+ _v_offset];
t7 = (tau*v7);
v8 = v[(8-(1))+ _v_offset];
t8 = (tau*v8);
v9 = v[(9-(1))+ _v_offset];
t9 = (tau*v9);
{
for (j = 1; j <= n; j++) {
sum = (((((((((v1*c[(1-(1))+(j-(1)) * (ldc)+ _c_offset])+(v2*c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v3*c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v4*c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v5*c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v6*c[(6-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v7*c[(7-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v8*c[(8-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v9*c[(9-(1))+(j-(1)) * (ldc)+ _c_offset]));
c[(1-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(1-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(2-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(3-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(4-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(5-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t5));
c[(6-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(6-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t6));
c[(7-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(7-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t7));
c[(8-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(8-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t8));
c[(9-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(9-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t9));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",180);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 10 x 10 Householder
*
***/
}// END label190
//Dummy.label("com/polytechnik/lapack310/Dlarfx",190);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
v6 = v[(6-(1))+ _v_offset];
t6 = (tau*v6);
v7 = v[(7-(1))+ _v_offset];
t7 = (tau*v7);
v8 = v[(8-(1))+ _v_offset];
t8 = (tau*v8);
v9 = v[(9-(1))+ _v_offset];
t9 = (tau*v9);
v10 = v[(10-(1))+ _v_offset];
t10 = (tau*v10);
{
for (j = 1; j <= n; j++) {
sum = ((((((((((v1*c[(1-(1))+(j-(1)) * (ldc)+ _c_offset])+(v2*c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v3*c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v4*c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v5*c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v6*c[(6-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v7*c[(7-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v8*c[(8-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v9*c[(9-(1))+(j-(1)) * (ldc)+ _c_offset]))+(v10*c[(10-(1))+(j-(1)) * (ldc)+ _c_offset]));
c[(1-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(1-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(2-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(2-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(3-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(3-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(4-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(4-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(5-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(5-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t5));
c[(6-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(6-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t6));
c[(7-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(7-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t7));
c[(8-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(8-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t8));
c[(9-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(9-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t9));
c[(10-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(10-(1))+(j-(1)) * (ldc)+ _c_offset]-(sum*t10));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",200);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Form  C * H, where H has order n.
*
***/
}
else  {
  /***
*
*        Code for general N
*
***/
label390:{
label370:{
label350:{
label330:{
label310:{
label290:{
label270:{
label250:{
label230:{
label210:{


{
  int _cg_tmp = n;
if (_cg_tmp == 1)
    break label210;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",210);
else if (_cg_tmp == 2)
    break label230;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",230);
else if (_cg_tmp == 3)
    break label250;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",250);
else if (_cg_tmp == 4)
    break label270;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",270);
else if (_cg_tmp == 5)
    break label290;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",290);
else if (_cg_tmp == 6)
    break label310;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",310);
else if (_cg_tmp == 7)
    break label330;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",330);
else if (_cg_tmp == 8)
    break label350;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",350);
else if (_cg_tmp == 9)
    break label370;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",370);
else if (_cg_tmp == 10)
    break label390;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",390);
}
com.polytechnik.lapack310.Dlarf.dlarf(side,m,n,v,_v_offset,1,tau,c,_c_offset,ldc,work,_work_offset);
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 1 x 1 Householder
*
***/
}// END label210
    //Dummy.label("com/polytechnik/lapack310/Dlarfx",210);
t1 = (1.0-((tau*v[(1-(1))+ _v_offset])*v[(1-(1))+ _v_offset]));
{
for (j = 1; j <= m; j++) {
c[(j-(1))+(1-(1)) * (ldc)+ _c_offset] = (t1*c[(j-(1))+(1-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlarfx",220);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 2 x 2 Householder
*
***/
}// END label230
//Dummy.label("com/polytechnik/lapack310/Dlarfx",230);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
{
for (j = 1; j <= m; j++) {
sum = ((v1*c[(j-(1))+(1-(1)) * (ldc)+ _c_offset])+(v2*c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]));
c[(j-(1))+(1-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(1-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(j-(1))+(2-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]-(sum*t2));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",240);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 3 x 3 Householder
*
***/
}// END label250
    //Dummy.label("com/polytechnik/lapack310/Dlarfx",250);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
{
for (j = 1; j <= m; j++) {
sum = (((v1*c[(j-(1))+(1-(1)) * (ldc)+ _c_offset])+(v2*c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]))+(v3*c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]));
c[(j-(1))+(1-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(1-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(j-(1))+(2-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(j-(1))+(3-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]-(sum*t3));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",260);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 4 x 4 Householder
*
***/
}// END label270
//Dummy.label("com/polytechnik/lapack310/Dlarfx",270);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
{
for (j = 1; j <= m; j++) {
sum = ((((v1*c[(j-(1))+(1-(1)) * (ldc)+ _c_offset])+(v2*c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]))+(v3*c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]))+(v4*c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]));
c[(j-(1))+(1-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(1-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(j-(1))+(2-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(j-(1))+(3-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(j-(1))+(4-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]-(sum*t4));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",280);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 5 x 5 Householder
*
***/
}//END label290:
//Dummy.label("com/polytechnik/lapack310/Dlarfx",290);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
{
for (j = 1; j <= m; j++) {
sum = (((((v1*c[(j-(1))+(1-(1)) * (ldc)+ _c_offset])+(v2*c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]))+(v3*c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]))+(v4*c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]))+(v5*c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]));
c[(j-(1))+(1-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(1-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(j-(1))+(2-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(j-(1))+(3-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(j-(1))+(4-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(j-(1))+(5-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]-(sum*t5));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",300);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 6 x 6 Householder
*
***/
}// END label310:
 //Dummy.label("com/polytechnik/lapack310/Dlarfx",310);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
v6 = v[(6-(1))+ _v_offset];
t6 = (tau*v6);
{
for (j = 1; j <= m; j++) {
sum = ((((((v1*c[(j-(1))+(1-(1)) * (ldc)+ _c_offset])+(v2*c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]))+(v3*c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]))+(v4*c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]))+(v5*c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]))+(v6*c[(j-(1))+(6-(1)) * (ldc)+ _c_offset]));
c[(j-(1))+(1-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(1-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(j-(1))+(2-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(j-(1))+(3-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(j-(1))+(4-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(j-(1))+(5-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]-(sum*t5));
c[(j-(1))+(6-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(6-(1)) * (ldc)+ _c_offset]-(sum*t6));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",320);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 7 x 7 Householder
*
***/
}// END label330:
//Dummy.label("com/polytechnik/lapack310/Dlarfx",330);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
v6 = v[(6-(1))+ _v_offset];
t6 = (tau*v6);
v7 = v[(7-(1))+ _v_offset];
t7 = (tau*v7);
{
for (j = 1; j <= m; j++) {
sum = (((((((v1*c[(j-(1))+(1-(1)) * (ldc)+ _c_offset])+(v2*c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]))+(v3*c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]))+(v4*c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]))+(v5*c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]))+(v6*c[(j-(1))+(6-(1)) * (ldc)+ _c_offset]))+(v7*c[(j-(1))+(7-(1)) * (ldc)+ _c_offset]));
c[(j-(1))+(1-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(1-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(j-(1))+(2-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(j-(1))+(3-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(j-(1))+(4-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(j-(1))+(5-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]-(sum*t5));
c[(j-(1))+(6-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(6-(1)) * (ldc)+ _c_offset]-(sum*t6));
c[(j-(1))+(7-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(7-(1)) * (ldc)+ _c_offset]-(sum*t7));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",340);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 8 x 8 Householder
*
***/
}// END label350:
//Dummy.label("com/polytechnik/lapack310/Dlarfx",350);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
v6 = v[(6-(1))+ _v_offset];
t6 = (tau*v6);
v7 = v[(7-(1))+ _v_offset];
t7 = (tau*v7);
v8 = v[(8-(1))+ _v_offset];
t8 = (tau*v8);
{
for (j = 1; j <= m; j++) {
sum = ((((((((v1*c[(j-(1))+(1-(1)) * (ldc)+ _c_offset])+(v2*c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]))+(v3*c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]))+(v4*c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]))+(v5*c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]))+(v6*c[(j-(1))+(6-(1)) * (ldc)+ _c_offset]))+(v7*c[(j-(1))+(7-(1)) * (ldc)+ _c_offset]))+(v8*c[(j-(1))+(8-(1)) * (ldc)+ _c_offset]));
c[(j-(1))+(1-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(1-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(j-(1))+(2-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(j-(1))+(3-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(j-(1))+(4-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(j-(1))+(5-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]-(sum*t5));
c[(j-(1))+(6-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(6-(1)) * (ldc)+ _c_offset]-(sum*t6));
c[(j-(1))+(7-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(7-(1)) * (ldc)+ _c_offset]-(sum*t7));
c[(j-(1))+(8-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(8-(1)) * (ldc)+ _c_offset]-(sum*t8));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",360);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 9 x 9 Householder
*
***/
}// END label370:
//Dummy.label("com/polytechnik/lapack310/Dlarfx",370);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
v6 = v[(6-(1))+ _v_offset];
t6 = (tau*v6);
v7 = v[(7-(1))+ _v_offset];
t7 = (tau*v7);
v8 = v[(8-(1))+ _v_offset];
t8 = (tau*v8);
v9 = v[(9-(1))+ _v_offset];
t9 = (tau*v9);
{
for (j = 1; j <= m; j++) {
sum = (((((((((v1*c[(j-(1))+(1-(1)) * (ldc)+ _c_offset])+(v2*c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]))+(v3*c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]))+(v4*c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]))+(v5*c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]))+(v6*c[(j-(1))+(6-(1)) * (ldc)+ _c_offset]))+(v7*c[(j-(1))+(7-(1)) * (ldc)+ _c_offset]))+(v8*c[(j-(1))+(8-(1)) * (ldc)+ _c_offset]))+(v9*c[(j-(1))+(9-(1)) * (ldc)+ _c_offset]));
c[(j-(1))+(1-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(1-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(j-(1))+(2-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(j-(1))+(3-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(j-(1))+(4-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(j-(1))+(5-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]-(sum*t5));
c[(j-(1))+(6-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(6-(1)) * (ldc)+ _c_offset]-(sum*t6));
c[(j-(1))+(7-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(7-(1)) * (ldc)+ _c_offset]-(sum*t7));
c[(j-(1))+(8-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(8-(1)) * (ldc)+ _c_offset]-(sum*t8));
c[(j-(1))+(9-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(9-(1)) * (ldc)+ _c_offset]-(sum*t9));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",380);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*        Special code for 10 x 10 Householder
*
***/
}// END label390:
    //Dummy.label("com/polytechnik/lapack310/Dlarfx",390);
v1 = v[(1-(1))+ _v_offset];
t1 = (tau*v1);
v2 = v[(2-(1))+ _v_offset];
t2 = (tau*v2);
v3 = v[(3-(1))+ _v_offset];
t3 = (tau*v3);
v4 = v[(4-(1))+ _v_offset];
t4 = (tau*v4);
v5 = v[(5-(1))+ _v_offset];
t5 = (tau*v5);
v6 = v[(6-(1))+ _v_offset];
t6 = (tau*v6);
v7 = v[(7-(1))+ _v_offset];
t7 = (tau*v7);
v8 = v[(8-(1))+ _v_offset];
t8 = (tau*v8);
v9 = v[(9-(1))+ _v_offset];
t9 = (tau*v9);
v10 = v[(10-(1))+ _v_offset];
t10 = (tau*v10);
{
for (j = 1; j <= m; j++) {
sum = ((((((((((v1*c[(j-(1))+(1-(1)) * (ldc)+ _c_offset])+(v2*c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]))+(v3*c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]))+(v4*c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]))+(v5*c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]))+(v6*c[(j-(1))+(6-(1)) * (ldc)+ _c_offset]))+(v7*c[(j-(1))+(7-(1)) * (ldc)+ _c_offset]))+(v8*c[(j-(1))+(8-(1)) * (ldc)+ _c_offset]))+(v9*c[(j-(1))+(9-(1)) * (ldc)+ _c_offset]))+(v10*c[(j-(1))+(10-(1)) * (ldc)+ _c_offset]));
c[(j-(1))+(1-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(1-(1)) * (ldc)+ _c_offset]-(sum*t1));
c[(j-(1))+(2-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(2-(1)) * (ldc)+ _c_offset]-(sum*t2));
c[(j-(1))+(3-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(3-(1)) * (ldc)+ _c_offset]-(sum*t3));
c[(j-(1))+(4-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(4-(1)) * (ldc)+ _c_offset]-(sum*t4));
c[(j-(1))+(5-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(5-(1)) * (ldc)+ _c_offset]-(sum*t5));
c[(j-(1))+(6-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(6-(1)) * (ldc)+ _c_offset]-(sum*t6));
c[(j-(1))+(7-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(7-(1)) * (ldc)+ _c_offset]-(sum*t7));
c[(j-(1))+(8-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(8-(1)) * (ldc)+ _c_offset]-(sum*t8));
c[(j-(1))+(9-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(9-(1)) * (ldc)+ _c_offset]-(sum*t9));
c[(j-(1))+(10-(1)) * (ldc)+ _c_offset] = (c[(j-(1))+(10-(1)) * (ldc)+ _c_offset]-(sum*t10));
//Dummy.label("com/polytechnik/lapack310/Dlarfx",400);
}              //  Close for() loop.
}
break label410;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",410);
}              //  Close else.
} // END label410
//Dummy.label("com/polytechnik/lapack310/Dlarfx",410);
/***
*
*     End of DLARFX
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlarfx",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlarfx",999999);
return;
   }
} // End class.
