/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarnv.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlarnv {

// *> \brief \b DLARNV returns a vector of random numbers from a uniform or
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLARNV + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARNV( IDIST, ISEED, N, X )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            IDIST, N
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            ISEED( 4 )
// *       DOUBLE PRECISION   X( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARNV returns a vector of n random real numbers from a uniform or
// *> normal distribution.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] IDIST
// *> \verbatim
// *>          IDIST is INTEGER
// *>          Specifies the distribution of the random numbers:
// *>          = 1:  uniform (0,1)
// *>          = 2:  uniform (-1,1)
// *>          = 3:  normal (0,1)
// *> \endverbatim
// *>
// *> \param[in,out] ISEED
// *> \verbatim
// *>          ISEED is INTEGER array, dimension (4)
// *>          On entry, the seed of the random number generator; the array
// *>          elements must be between 0 and 4095, and ISEED(4) must be
// *>          odd.
// *>          On exit, the seed is updated.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of random numbers to be generated.
// *> \endverbatim
// *>
// *> \param[out] X
// *> \verbatim
// *>          X is DOUBLE PRECISION array, dimension (N)
// *>          The generated random numbers.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup OTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  This routine calls the auxiliary routine DLARUV to generate random
// *>  real numbers from a uniform (0,1) distribution, in batches of up to

// *>  128 using vectorisable code. The Box-Muller method is used to
// *>  transform numbers from a uniform to a normal distribution.
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlarnv (int idist,
int [] iseed, int _iseed_offset,
int n,
double [] x, int _x_offset)  {

label_999999:{
final double TWOPI = 2*Math.PI;

int i= 0;
int il= 0;
int il2= 0;
int iv= 0;
double [] u= new double[(128)];
{
int _iv_inc = (128/2);
for (iv = 1; (_iv_inc < 0) ? iv >= n : iv <= n; iv += _iv_inc) {
il = Math.min((128/2), ((n-iv)+1));
if ((idist == 3))  {
    il2 = (2*il);
}
else  {
  il2 = il;
/***
*
*        Call DLARUV to generate IL2 numbers from a uniform (0,1)
*        distribution (IL2 <= LV)
*
***/
}              //  Close else.
// *
com.polytechnik.lapack310.Dlaruv.dlaruv(iseed,_iseed_offset,il2,u,0);
/***
*
*           Copy generated numbers
*
***/
if ((idist == 1))  {
    {
for (i = 1; i <= il; i++) {
x[(((iv+i)-1)-(1))+ _x_offset] = u[(i-(1))];
//Dummy.label("com/polytechnik/lapack310/Dlarnv",10);
}              //  Close for() loop.
}
/***
*
*           Convert generated numbers to uniform (-1,1) distribution
*
***/
}
else if ((idist == 2))  {
    {
for (i = 1; i <= il; i++) {
x[(((iv+i)-1)-(1))+ _x_offset] = ((2.0*u[(i-(1))])-1.0);
//Dummy.label("com/polytechnik/lapack310/Dlarnv",20);
}              //  Close for() loop.
}
/***
*
*           Convert generated numbers to normal (0,1) distribution
*
***/
}              // Close else if()
else if ((idist == 3))  {
    {
for (i = 1; i <= il; i++) {
x[(((iv+i)-1)-(1))+ _x_offset] = (Math.sqrt((-((2.0*Math.log(u[(((2*i)-1)-(1))])))))*Math.cos((TWOPI*u[((2*i)-(1))])));
//Dummy.label("com/polytechnik/lapack310/Dlarnv",30);
}              //  Close for() loop.
}
}              // Close else if()
//Dummy.label("com/polytechnik/lapack310/Dlarnv",40);
}              //  Close for() loop.
}
/***
*
*     End of DLARNV
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlarnv",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlarnv",999999);
return;
   }
} // End class.
