/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlartg.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlartg {

// *> \brief \b DLARTG generates a plane rotation with real cosine and real
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLARTG + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARTG( F, G, CS, SN, R )
// *
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION   CS, F, G, R, SN
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARTG generate a plane rotation so that
// *>
// *>    [  CS  SN  ]  .  [ F ]  =  [ R ]   where CS**2 + SN**2 = 1.
// *>    [ -SN  CS  ]     [ G ]     [ 0 ]
// *>
// *> This is a slower, more accurate version of the BLAS1 routine DROTG,
// *> with the following other differences:
// *>    F and G are unchanged on return.
// *>    If G=0, then CS=1 and SN=0.
// *>    If F=0 and (G .ne. 0), then CS=0 and SN=1 without doing any
// *>       floating point operations (saves work in DBDSQR when
// *>       there are zeros on the diagonal).
// *>
// *> If F exceeds G in magnitude, CS will be positive.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] F
// *> \verbatim
// *>          F is DOUBLE PRECISION
// *>          The first component of vector to be rotated.
// *> \endverbatim
// *>
// *> \param[in] G
// *> \verbatim
// *>          G is DOUBLE PRECISION
// *>          The second component of vector to be rotated.
// *> \endverbatim
// *>
// *> \param[out] CS
// *> \verbatim
// *>          CS is DOUBLE PRECISION
// *>          The cosine of the rotation.
// *> \endverbatim
// *>
// *> \param[out] SN
// *> \verbatim
// *>          SN is DOUBLE PRECISION
// *>          The sine of the rotation.
// *> \endverbatim
// *>
// *> \param[out] R
// *> \verbatim
// *>          R is DOUBLE PRECISION
// *>          The nonzero component of the rotated vector.
// *>
// *>  This version has a few statements commented out for thread safety
// *>  (machine parameters are computed on each entry). 10 feb 03, SJH.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup OTHERauxiliary
// *
// *  =====================================================================
//
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
*     LOGICAL            FIRST
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Save statement ..
*     SAVE               FIRST, SAFMX2, SAFMIN, SAFMN2
*     ..
*     .. Data statements ..
*     DATA               FIRST / .TRUE. /
*     ..
*     .. Executable Statements ..
*
*     IF( FIRST ) THEN
***/

public static void dlartg (double f,
double g,
doubleW cs,
doubleW sn,
doubleW r)  {

label_999999:{

int count= 0;
int i= 0;
double eps= 0.0d;
double f1= 0.0d;
double g1= 0.0d;
double safmin= 0.0d;
double safmn2= 0.0d;
double safmx2= 0.0d;
double scale= 0.0d;
safmin = com.polytechnik.lapack310.Dlamch.dlamch("S");
eps = com.polytechnik.lapack310.Dlamch.dlamch("E");
safmn2 = ( Math.pow(com.polytechnik.lapack310.Dlamch.dlamch("B"), (int)(((Math.log((safmin/eps))/Math.log(com.polytechnik.lapack310.Dlamch.dlamch("B")))/2.0))));
/***
*        FIRST = .FALSE.
*     END IF
***/
safmx2 = (1.0/safmn2);
if ((g == 0.0))  {
    cs.val = 1.0;
sn.val = 0.0;
r.val = f;
}
else if ((f == 0.0))  {
    cs.val = 0.0;
sn.val = 1.0;
r.val = g;
}              // Close else if()
else  {
  f1 = f;
g1 = g;
scale = Math.max(Math.abs(f1), Math.abs(g1));
if ((scale >= safmx2))  {
    count = 0;
label10:for(int jjx=0;jjx<Util.ITMAX;jjx++){
	//Dummy.label("com/polytechnik/lapack310/Dlartg",10);
count = (count+1);
f1 = (f1*safmn2);
g1 = (g1*safmn2);
scale = Math.max(Math.abs(f1), Math.abs(g1));
if (((scale >= safmx2) && (count < 20))) {
    continue label10;//Dummy.go_to("com/polytechnik/lapack310/Dlartg",10);
}
break label10;}
    r.val = Math.sqrt((( Math.pow(f1, 2))+( Math.pow(g1, 2))));
cs.val = (f1/r.val);
sn.val = (g1/r.val);
{
for (i = 1; i <= count; i++) {
r.val = (r.val*safmx2);
//Dummy.label("com/polytechnik/lapack310/Dlartg",20);
}              //  Close for() loop.
}
}
else if ((scale <= safmn2))  {
    count = 0;
label30:for(int jjx=0;jjx<Util.ITMAX;jjx++){
	//Dummy.label("com/polytechnik/lapack310/Dlartg",30);
count = (count+1);
f1 = (f1*safmx2);
g1 = (g1*safmx2);
scale = Math.max(Math.abs(f1), Math.abs(g1));
if ((scale <= safmn2)) {
    continue label30;//Dummy.go_to("com/polytechnik/lapack310/Dlartg",30);
}
break label30;}

    r.val = Math.sqrt((( Math.pow(f1, 2))+( Math.pow(g1, 2))));
cs.val = (f1/r.val);
sn.val = (g1/r.val);
{
for (i = 1; i <= count; i++) {
r.val = (r.val*safmn2);
//Dummy.label("com/polytechnik/lapack310/Dlartg",40);
}              //  Close for() loop.
}
}              // Close else if()
else  {
  r.val = Math.sqrt((( Math.pow(f1, 2))+( Math.pow(g1, 2))));
cs.val = (f1/r.val);
sn.val = (g1/r.val);
}              //  Close else.
if (((Math.abs(f) > Math.abs(g)) && (cs.val < 0.0)))  {
    cs.val = (-(cs.val));
sn.val = (-(sn.val));
r.val = (-(r.val));
}
}              //  Close else.
/***
*
*     End of DLARTG
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlartg",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlartg",999999);
return;
   }
} // End class.
