/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlaruv.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlaruv {

// *> \brief \b DLARUV returns a vector of n random real numbers from a uni
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLARUV + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARUV( ISEED, N, X )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            N
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            ISEED( 4 )
// *       DOUBLE PRECISION   X( N )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARUV returns a vector of n random real numbers from a uniform (0,1)
// *> distribution (n <= 128).
// *>
// *> This is an auxiliary routine called by DLARNV and ZLARNV.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in,out] ISEED
// *> \verbatim
// *>          ISEED is INTEGER array, dimension (4)
// *>          On entry, the seed of the random number generator; the array
// *>          elements must be between 0 and 4095, and ISEED(4) must be
// *>          odd.
// *>          On exit, the seed is updated.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of random numbers to be generated. N <= 128.
// *> \endverbatim
// *>
// *> \param[out] X
// *> \verbatim
// *>          X is DOUBLE PRECISION array, dimension (N)
// *>          The generated random numbers.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup OTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  This routine uses a multiplicative congruential method with modulus

// *>  2**48 and multiplier 33952834046453 (see G.S.Fishman,
// *>  'Multiplicative congruential random number generators with modulus
// *>  2**b: an exhaustive analysis for b = 32 and a partial analysis for
// *>  b = 48', Math. Comp. 189, pp 331-344, 1990).
// *>
// *>  48-bit integers are stored in 4 integer array elements with 12 bits

// *>  per element. Hence the routine is portable across machines with
// *>  integers of 32 bits or more.
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
public static int [] mm= new int[(128) * (4)];
/***
*     ..
*     .. Data statements ..
***/
static {
mm[(1-(1))+( 1 -(1)) * (128)] = 494;
mm[(1-(1))+( 2 -(1)) * (128)] = 322;
mm[(1-(1))+( 3 -(1)) * (128)] = 2508;
mm[(1-(1))+( 4 -(1)) * (128)] = 2549;
}
static {
mm[(2-(1))+( 1 -(1)) * (128)] = 2637;
mm[(2-(1))+( 2 -(1)) * (128)] = 789;
mm[(2-(1))+( 3 -(1)) * (128)] = 3754;
mm[(2-(1))+( 4 -(1)) * (128)] = 1145;
}
static {
mm[(3-(1))+( 1 -(1)) * (128)] = 255;
mm[(3-(1))+( 2 -(1)) * (128)] = 1440;
mm[(3-(1))+( 3 -(1)) * (128)] = 1766;
mm[(3-(1))+( 4 -(1)) * (128)] = 2253;
}
static {
mm[(4-(1))+( 1 -(1)) * (128)] = 2008;
mm[(4-(1))+( 2 -(1)) * (128)] = 752;
mm[(4-(1))+( 3 -(1)) * (128)] = 3572;
mm[(4-(1))+( 4 -(1)) * (128)] = 305;
}
static {
mm[(5-(1))+( 1 -(1)) * (128)] = 1253;
mm[(5-(1))+( 2 -(1)) * (128)] = 2859;
mm[(5-(1))+( 3 -(1)) * (128)] = 2893;
mm[(5-(1))+( 4 -(1)) * (128)] = 3301;
}
static {
mm[(6-(1))+( 1 -(1)) * (128)] = 3344;
mm[(6-(1))+( 2 -(1)) * (128)] = 123;
mm[(6-(1))+( 3 -(1)) * (128)] = 307;
mm[(6-(1))+( 4 -(1)) * (128)] = 1065;
}
static {
mm[(7-(1))+( 1 -(1)) * (128)] = 4084;
mm[(7-(1))+( 2 -(1)) * (128)] = 1848;
mm[(7-(1))+( 3 -(1)) * (128)] = 1297;
mm[(7-(1))+( 4 -(1)) * (128)] = 3133;
}
static {
mm[(8-(1))+( 1 -(1)) * (128)] = 1739;
mm[(8-(1))+( 2 -(1)) * (128)] = 643;
mm[(8-(1))+( 3 -(1)) * (128)] = 3966;
mm[(8-(1))+( 4 -(1)) * (128)] = 2913;
}
static {
mm[(9-(1))+( 1 -(1)) * (128)] = 3143;
mm[(9-(1))+( 2 -(1)) * (128)] = 2405;
mm[(9-(1))+( 3 -(1)) * (128)] = 758;
mm[(9-(1))+( 4 -(1)) * (128)] = 3285;
}
static {
mm[(10-(1))+( 1 -(1)) * (128)] = 3468;
mm[(10-(1))+( 2 -(1)) * (128)] = 2638;
mm[(10-(1))+( 3 -(1)) * (128)] = 2598;
mm[(10-(1))+( 4 -(1)) * (128)] = 1241;
}
static {
mm[(11-(1))+( 1 -(1)) * (128)] = 688;
mm[(11-(1))+( 2 -(1)) * (128)] = 2344;
mm[(11-(1))+( 3 -(1)) * (128)] = 3406;
mm[(11-(1))+( 4 -(1)) * (128)] = 1197;
}
static {
mm[(12-(1))+( 1 -(1)) * (128)] = 1657;
mm[(12-(1))+( 2 -(1)) * (128)] = 46;
mm[(12-(1))+( 3 -(1)) * (128)] = 2922;
mm[(12-(1))+( 4 -(1)) * (128)] = 3729;
}
static {
mm[(13-(1))+( 1 -(1)) * (128)] = 1238;
mm[(13-(1))+( 2 -(1)) * (128)] = 3814;
mm[(13-(1))+( 3 -(1)) * (128)] = 1038;
mm[(13-(1))+( 4 -(1)) * (128)] = 2501;
}
static {
mm[(14-(1))+( 1 -(1)) * (128)] = 3166;
mm[(14-(1))+( 2 -(1)) * (128)] = 913;
mm[(14-(1))+( 3 -(1)) * (128)] = 2934;
mm[(14-(1))+( 4 -(1)) * (128)] = 1673;
}
static {
mm[(15-(1))+( 1 -(1)) * (128)] = 1292;
mm[(15-(1))+( 2 -(1)) * (128)] = 3649;
mm[(15-(1))+( 3 -(1)) * (128)] = 2091;
mm[(15-(1))+( 4 -(1)) * (128)] = 541;
}
static {
mm[(16-(1))+( 1 -(1)) * (128)] = 3422;
mm[(16-(1))+( 2 -(1)) * (128)] = 339;
mm[(16-(1))+( 3 -(1)) * (128)] = 2451;
mm[(16-(1))+( 4 -(1)) * (128)] = 2753;
}
static {
mm[(17-(1))+( 1 -(1)) * (128)] = 1270;
mm[(17-(1))+( 2 -(1)) * (128)] = 3808;
mm[(17-(1))+( 3 -(1)) * (128)] = 1580;
mm[(17-(1))+( 4 -(1)) * (128)] = 949;
}
static {
mm[(18-(1))+( 1 -(1)) * (128)] = 2016;
mm[(18-(1))+( 2 -(1)) * (128)] = 822;
mm[(18-(1))+( 3 -(1)) * (128)] = 1958;
mm[(18-(1))+( 4 -(1)) * (128)] = 2361;
}
static {
mm[(19-(1))+( 1 -(1)) * (128)] = 154;
mm[(19-(1))+( 2 -(1)) * (128)] = 2832;
mm[(19-(1))+( 3 -(1)) * (128)] = 2055;
mm[(19-(1))+( 4 -(1)) * (128)] = 1165;
}
static {
mm[(20-(1))+( 1 -(1)) * (128)] = 2862;
mm[(20-(1))+( 2 -(1)) * (128)] = 3078;
mm[(20-(1))+( 3 -(1)) * (128)] = 1507;
mm[(20-(1))+( 4 -(1)) * (128)] = 4081;
}
static {
mm[(21-(1))+( 1 -(1)) * (128)] = 697;
mm[(21-(1))+( 2 -(1)) * (128)] = 3633;
mm[(21-(1))+( 3 -(1)) * (128)] = 1078;
mm[(21-(1))+( 4 -(1)) * (128)] = 2725;
}
static {
mm[(22-(1))+( 1 -(1)) * (128)] = 1706;
mm[(22-(1))+( 2 -(1)) * (128)] = 2970;
mm[(22-(1))+( 3 -(1)) * (128)] = 3273;
mm[(22-(1))+( 4 -(1)) * (128)] = 3305;
}
static {
mm[(23-(1))+( 1 -(1)) * (128)] = 491;
mm[(23-(1))+( 2 -(1)) * (128)] = 637;
mm[(23-(1))+( 3 -(1)) * (128)] = 17;
mm[(23-(1))+( 4 -(1)) * (128)] = 3069;
}
static {
mm[(24-(1))+( 1 -(1)) * (128)] = 931;
mm[(24-(1))+( 2 -(1)) * (128)] = 2249;
mm[(24-(1))+( 3 -(1)) * (128)] = 854;
mm[(24-(1))+( 4 -(1)) * (128)] = 3617;
}
static {
mm[(25-(1))+( 1 -(1)) * (128)] = 1444;
mm[(25-(1))+( 2 -(1)) * (128)] = 2081;
mm[(25-(1))+( 3 -(1)) * (128)] = 2916;
mm[(25-(1))+( 4 -(1)) * (128)] = 3733;
}
static {
mm[(26-(1))+( 1 -(1)) * (128)] = 444;
mm[(26-(1))+( 2 -(1)) * (128)] = 4019;
mm[(26-(1))+( 3 -(1)) * (128)] = 3971;
mm[(26-(1))+( 4 -(1)) * (128)] = 409;
}
static {
mm[(27-(1))+( 1 -(1)) * (128)] = 3577;
mm[(27-(1))+( 2 -(1)) * (128)] = 1478;
mm[(27-(1))+( 3 -(1)) * (128)] = 2889;
mm[(27-(1))+( 4 -(1)) * (128)] = 2157;
}
static {
mm[(28-(1))+( 1 -(1)) * (128)] = 3944;
mm[(28-(1))+( 2 -(1)) * (128)] = 242;
mm[(28-(1))+( 3 -(1)) * (128)] = 3831;
mm[(28-(1))+( 4 -(1)) * (128)] = 1361;
}
static {
mm[(29-(1))+( 1 -(1)) * (128)] = 2184;
mm[(29-(1))+( 2 -(1)) * (128)] = 481;
mm[(29-(1))+( 3 -(1)) * (128)] = 2621;
mm[(29-(1))+( 4 -(1)) * (128)] = 3973;
}
static {
mm[(30-(1))+( 1 -(1)) * (128)] = 1661;
mm[(30-(1))+( 2 -(1)) * (128)] = 2075;
mm[(30-(1))+( 3 -(1)) * (128)] = 1541;
mm[(30-(1))+( 4 -(1)) * (128)] = 1865;
}
static {
mm[(31-(1))+( 1 -(1)) * (128)] = 3482;
mm[(31-(1))+( 2 -(1)) * (128)] = 4058;
mm[(31-(1))+( 3 -(1)) * (128)] = 893;
mm[(31-(1))+( 4 -(1)) * (128)] = 2525;
}
static {
mm[(32-(1))+( 1 -(1)) * (128)] = 657;
mm[(32-(1))+( 2 -(1)) * (128)] = 622;
mm[(32-(1))+( 3 -(1)) * (128)] = 736;
mm[(32-(1))+( 4 -(1)) * (128)] = 1409;
}
static {
mm[(33-(1))+( 1 -(1)) * (128)] = 3023;
mm[(33-(1))+( 2 -(1)) * (128)] = 3376;
mm[(33-(1))+( 3 -(1)) * (128)] = 3992;
mm[(33-(1))+( 4 -(1)) * (128)] = 3445;
}
static {
mm[(34-(1))+( 1 -(1)) * (128)] = 3618;
mm[(34-(1))+( 2 -(1)) * (128)] = 812;
mm[(34-(1))+( 3 -(1)) * (128)] = 787;
mm[(34-(1))+( 4 -(1)) * (128)] = 3577;
}
static {
mm[(35-(1))+( 1 -(1)) * (128)] = 1267;
mm[(35-(1))+( 2 -(1)) * (128)] = 234;
mm[(35-(1))+( 3 -(1)) * (128)] = 2125;
mm[(35-(1))+( 4 -(1)) * (128)] = 77;
}
static {
mm[(36-(1))+( 1 -(1)) * (128)] = 1828;
mm[(36-(1))+( 2 -(1)) * (128)] = 641;
mm[(36-(1))+( 3 -(1)) * (128)] = 2364;
mm[(36-(1))+( 4 -(1)) * (128)] = 3761;
}
static {
mm[(37-(1))+( 1 -(1)) * (128)] = 164;
mm[(37-(1))+( 2 -(1)) * (128)] = 4005;
mm[(37-(1))+( 3 -(1)) * (128)] = 2460;
mm[(37-(1))+( 4 -(1)) * (128)] = 2149;
}
static {
mm[(38-(1))+( 1 -(1)) * (128)] = 3798;
mm[(38-(1))+( 2 -(1)) * (128)] = 1122;
mm[(38-(1))+( 3 -(1)) * (128)] = 257;
mm[(38-(1))+( 4 -(1)) * (128)] = 1449;
}
static {
mm[(39-(1))+( 1 -(1)) * (128)] = 3087;
mm[(39-(1))+( 2 -(1)) * (128)] = 3135;
mm[(39-(1))+( 3 -(1)) * (128)] = 1574;
mm[(39-(1))+( 4 -(1)) * (128)] = 3005;
}
static {
mm[(40-(1))+( 1 -(1)) * (128)] = 2400;
mm[(40-(1))+( 2 -(1)) * (128)] = 2640;
mm[(40-(1))+( 3 -(1)) * (128)] = 3912;
mm[(40-(1))+( 4 -(1)) * (128)] = 225;
}
static {
mm[(41-(1))+( 1 -(1)) * (128)] = 2870;
mm[(41-(1))+( 2 -(1)) * (128)] = 2302;
mm[(41-(1))+( 3 -(1)) * (128)] = 1216;
mm[(41-(1))+( 4 -(1)) * (128)] = 85;
}
static {
mm[(42-(1))+( 1 -(1)) * (128)] = 3876;
mm[(42-(1))+( 2 -(1)) * (128)] = 40;
mm[(42-(1))+( 3 -(1)) * (128)] = 3248;
mm[(42-(1))+( 4 -(1)) * (128)] = 3673;
}
static {
mm[(43-(1))+( 1 -(1)) * (128)] = 1905;
mm[(43-(1))+( 2 -(1)) * (128)] = 1832;
mm[(43-(1))+( 3 -(1)) * (128)] = 3401;
mm[(43-(1))+( 4 -(1)) * (128)] = 3117;
}
static {
mm[(44-(1))+( 1 -(1)) * (128)] = 1593;
mm[(44-(1))+( 2 -(1)) * (128)] = 2247;
mm[(44-(1))+( 3 -(1)) * (128)] = 2124;
mm[(44-(1))+( 4 -(1)) * (128)] = 3089;
}
static {
mm[(45-(1))+( 1 -(1)) * (128)] = 1797;
mm[(45-(1))+( 2 -(1)) * (128)] = 2034;
mm[(45-(1))+( 3 -(1)) * (128)] = 2762;
mm[(45-(1))+( 4 -(1)) * (128)] = 1349;
}
static {
mm[(46-(1))+( 1 -(1)) * (128)] = 1234;
mm[(46-(1))+( 2 -(1)) * (128)] = 2637;
mm[(46-(1))+( 3 -(1)) * (128)] = 149;
mm[(46-(1))+( 4 -(1)) * (128)] = 2057;
}
static {
mm[(47-(1))+( 1 -(1)) * (128)] = 3460;
mm[(47-(1))+( 2 -(1)) * (128)] = 1287;
mm[(47-(1))+( 3 -(1)) * (128)] = 2245;
mm[(47-(1))+( 4 -(1)) * (128)] = 413;
}
static {
mm[(48-(1))+( 1 -(1)) * (128)] = 328;
mm[(48-(1))+( 2 -(1)) * (128)] = 1691;
mm[(48-(1))+( 3 -(1)) * (128)] = 166;
mm[(48-(1))+( 4 -(1)) * (128)] = 65;
}
static {
mm[(49-(1))+( 1 -(1)) * (128)] = 2861;
mm[(49-(1))+( 2 -(1)) * (128)] = 496;
mm[(49-(1))+( 3 -(1)) * (128)] = 466;
mm[(49-(1))+( 4 -(1)) * (128)] = 1845;
}
static {
mm[(50-(1))+( 1 -(1)) * (128)] = 1950;
mm[(50-(1))+( 2 -(1)) * (128)] = 1597;
mm[(50-(1))+( 3 -(1)) * (128)] = 4018;
mm[(50-(1))+( 4 -(1)) * (128)] = 697;
}
static {
mm[(51-(1))+( 1 -(1)) * (128)] = 617;
mm[(51-(1))+( 2 -(1)) * (128)] = 2394;
mm[(51-(1))+( 3 -(1)) * (128)] = 1399;
mm[(51-(1))+( 4 -(1)) * (128)] = 3085;
}
static {
mm[(52-(1))+( 1 -(1)) * (128)] = 2070;
mm[(52-(1))+( 2 -(1)) * (128)] = 2584;
mm[(52-(1))+( 3 -(1)) * (128)] = 190;
mm[(52-(1))+( 4 -(1)) * (128)] = 3441;
}
static {
mm[(53-(1))+( 1 -(1)) * (128)] = 3331;
mm[(53-(1))+( 2 -(1)) * (128)] = 1843;
mm[(53-(1))+( 3 -(1)) * (128)] = 2879;
mm[(53-(1))+( 4 -(1)) * (128)] = 1573;
}
static {
mm[(54-(1))+( 1 -(1)) * (128)] = 769;
mm[(54-(1))+( 2 -(1)) * (128)] = 336;
mm[(54-(1))+( 3 -(1)) * (128)] = 153;
mm[(54-(1))+( 4 -(1)) * (128)] = 3689;
}
static {
mm[(55-(1))+( 1 -(1)) * (128)] = 1558;
mm[(55-(1))+( 2 -(1)) * (128)] = 1472;
mm[(55-(1))+( 3 -(1)) * (128)] = 2320;
mm[(55-(1))+( 4 -(1)) * (128)] = 2941;
}
static {
mm[(56-(1))+( 1 -(1)) * (128)] = 2412;
mm[(56-(1))+( 2 -(1)) * (128)] = 2407;
mm[(56-(1))+( 3 -(1)) * (128)] = 18;
mm[(56-(1))+( 4 -(1)) * (128)] = 929;
}
static {
mm[(57-(1))+( 1 -(1)) * (128)] = 2800;
mm[(57-(1))+( 2 -(1)) * (128)] = 433;
mm[(57-(1))+( 3 -(1)) * (128)] = 712;
mm[(57-(1))+( 4 -(1)) * (128)] = 533;
}
static {
mm[(58-(1))+( 1 -(1)) * (128)] = 189;
mm[(58-(1))+( 2 -(1)) * (128)] = 2096;
mm[(58-(1))+( 3 -(1)) * (128)] = 2159;
mm[(58-(1))+( 4 -(1)) * (128)] = 2841;
}
static {
mm[(59-(1))+( 1 -(1)) * (128)] = 287;
mm[(59-(1))+( 2 -(1)) * (128)] = 1761;
mm[(59-(1))+( 3 -(1)) * (128)] = 2318;
mm[(59-(1))+( 4 -(1)) * (128)] = 4077;
}
static {
mm[(60-(1))+( 1 -(1)) * (128)] = 2045;
mm[(60-(1))+( 2 -(1)) * (128)] = 2810;
mm[(60-(1))+( 3 -(1)) * (128)] = 2091;
mm[(60-(1))+( 4 -(1)) * (128)] = 721;
}
static {
mm[(61-(1))+( 1 -(1)) * (128)] = 1227;
mm[(61-(1))+( 2 -(1)) * (128)] = 566;
mm[(61-(1))+( 3 -(1)) * (128)] = 3443;
mm[(61-(1))+( 4 -(1)) * (128)] = 2821;
}
static {
mm[(62-(1))+( 1 -(1)) * (128)] = 2838;
mm[(62-(1))+( 2 -(1)) * (128)] = 442;
mm[(62-(1))+( 3 -(1)) * (128)] = 1510;
mm[(62-(1))+( 4 -(1)) * (128)] = 2249;
}
static {
mm[(63-(1))+( 1 -(1)) * (128)] = 209;
mm[(63-(1))+( 2 -(1)) * (128)] = 41;
mm[(63-(1))+( 3 -(1)) * (128)] = 449;
mm[(63-(1))+( 4 -(1)) * (128)] = 2397;
}
static {
mm[(64-(1))+( 1 -(1)) * (128)] = 2770;
mm[(64-(1))+( 2 -(1)) * (128)] = 1238;
mm[(64-(1))+( 3 -(1)) * (128)] = 1956;
mm[(64-(1))+( 4 -(1)) * (128)] = 2817;
}
static {
mm[(65-(1))+( 1 -(1)) * (128)] = 3654;
mm[(65-(1))+( 2 -(1)) * (128)] = 1086;
mm[(65-(1))+( 3 -(1)) * (128)] = 2201;
mm[(65-(1))+( 4 -(1)) * (128)] = 245;
}
static {
mm[(66-(1))+( 1 -(1)) * (128)] = 3993;
mm[(66-(1))+( 2 -(1)) * (128)] = 603;
mm[(66-(1))+( 3 -(1)) * (128)] = 3137;
mm[(66-(1))+( 4 -(1)) * (128)] = 1913;
}
static {
mm[(67-(1))+( 1 -(1)) * (128)] = 192;
mm[(67-(1))+( 2 -(1)) * (128)] = 840;
mm[(67-(1))+( 3 -(1)) * (128)] = 3399;
mm[(67-(1))+( 4 -(1)) * (128)] = 1997;
}
static {
mm[(68-(1))+( 1 -(1)) * (128)] = 2253;
mm[(68-(1))+( 2 -(1)) * (128)] = 3168;
mm[(68-(1))+( 3 -(1)) * (128)] = 1321;
mm[(68-(1))+( 4 -(1)) * (128)] = 3121;
}
static {
mm[(69-(1))+( 1 -(1)) * (128)] = 3491;
mm[(69-(1))+( 2 -(1)) * (128)] = 1499;
mm[(69-(1))+( 3 -(1)) * (128)] = 2271;
mm[(69-(1))+( 4 -(1)) * (128)] = 997;
}
static {
mm[(70-(1))+( 1 -(1)) * (128)] = 2889;
mm[(70-(1))+( 2 -(1)) * (128)] = 1084;
mm[(70-(1))+( 3 -(1)) * (128)] = 3667;
mm[(70-(1))+( 4 -(1)) * (128)] = 1833;
}
static {
mm[(71-(1))+( 1 -(1)) * (128)] = 2857;
mm[(71-(1))+( 2 -(1)) * (128)] = 3438;
mm[(71-(1))+( 3 -(1)) * (128)] = 2703;
mm[(71-(1))+( 4 -(1)) * (128)] = 2877;
}
static {
mm[(72-(1))+( 1 -(1)) * (128)] = 2094;
mm[(72-(1))+( 2 -(1)) * (128)] = 2408;
mm[(72-(1))+( 3 -(1)) * (128)] = 629;
mm[(72-(1))+( 4 -(1)) * (128)] = 1633;
}
static {
mm[(73-(1))+( 1 -(1)) * (128)] = 1818;
mm[(73-(1))+( 2 -(1)) * (128)] = 1589;
mm[(73-(1))+( 3 -(1)) * (128)] = 2365;
mm[(73-(1))+( 4 -(1)) * (128)] = 981;
}
static {
mm[(74-(1))+( 1 -(1)) * (128)] = 688;
mm[(74-(1))+( 2 -(1)) * (128)] = 2391;
mm[(74-(1))+( 3 -(1)) * (128)] = 2431;
mm[(74-(1))+( 4 -(1)) * (128)] = 2009;
}
static {
mm[(75-(1))+( 1 -(1)) * (128)] = 1407;
mm[(75-(1))+( 2 -(1)) * (128)] = 288;
mm[(75-(1))+( 3 -(1)) * (128)] = 1113;
mm[(75-(1))+( 4 -(1)) * (128)] = 941;
}
static {
mm[(76-(1))+( 1 -(1)) * (128)] = 634;
mm[(76-(1))+( 2 -(1)) * (128)] = 26;
mm[(76-(1))+( 3 -(1)) * (128)] = 3922;
mm[(76-(1))+( 4 -(1)) * (128)] = 2449;
}
static {
mm[(77-(1))+( 1 -(1)) * (128)] = 3231;
mm[(77-(1))+( 2 -(1)) * (128)] = 512;
mm[(77-(1))+( 3 -(1)) * (128)] = 2554;
mm[(77-(1))+( 4 -(1)) * (128)] = 197;
}
static {
mm[(78-(1))+( 1 -(1)) * (128)] = 815;
mm[(78-(1))+( 2 -(1)) * (128)] = 1456;
mm[(78-(1))+( 3 -(1)) * (128)] = 184;
mm[(78-(1))+( 4 -(1)) * (128)] = 2441;
}
static {
mm[(79-(1))+( 1 -(1)) * (128)] = 3524;
mm[(79-(1))+( 2 -(1)) * (128)] = 171;
mm[(79-(1))+( 3 -(1)) * (128)] = 2099;
mm[(79-(1))+( 4 -(1)) * (128)] = 285;
}
static {
mm[(80-(1))+( 1 -(1)) * (128)] = 1914;
mm[(80-(1))+( 2 -(1)) * (128)] = 1677;
mm[(80-(1))+( 3 -(1)) * (128)] = 3228;
mm[(80-(1))+( 4 -(1)) * (128)] = 1473;
}
static {
mm[(81-(1))+( 1 -(1)) * (128)] = 516;
mm[(81-(1))+( 2 -(1)) * (128)] = 2657;
mm[(81-(1))+( 3 -(1)) * (128)] = 4012;
mm[(81-(1))+( 4 -(1)) * (128)] = 2741;
}
static {
mm[(82-(1))+( 1 -(1)) * (128)] = 164;
mm[(82-(1))+( 2 -(1)) * (128)] = 2270;
mm[(82-(1))+( 3 -(1)) * (128)] = 1921;
mm[(82-(1))+( 4 -(1)) * (128)] = 3129;
}
static {
mm[(83-(1))+( 1 -(1)) * (128)] = 303;
mm[(83-(1))+( 2 -(1)) * (128)] = 2587;
mm[(83-(1))+( 3 -(1)) * (128)] = 3452;
mm[(83-(1))+( 4 -(1)) * (128)] = 909;
}
static {
mm[(84-(1))+( 1 -(1)) * (128)] = 2144;
mm[(84-(1))+( 2 -(1)) * (128)] = 2961;
mm[(84-(1))+( 3 -(1)) * (128)] = 3901;
mm[(84-(1))+( 4 -(1)) * (128)] = 2801;
}
static {
mm[(85-(1))+( 1 -(1)) * (128)] = 3480;
mm[(85-(1))+( 2 -(1)) * (128)] = 1970;
mm[(85-(1))+( 3 -(1)) * (128)] = 572;
mm[(85-(1))+( 4 -(1)) * (128)] = 421;
}
static {
mm[(86-(1))+( 1 -(1)) * (128)] = 119;
mm[(86-(1))+( 2 -(1)) * (128)] = 1817;
mm[(86-(1))+( 3 -(1)) * (128)] = 3309;
mm[(86-(1))+( 4 -(1)) * (128)] = 4073;
}
static {
mm[(87-(1))+( 1 -(1)) * (128)] = 3357;
mm[(87-(1))+( 2 -(1)) * (128)] = 676;
mm[(87-(1))+( 3 -(1)) * (128)] = 3171;
mm[(87-(1))+( 4 -(1)) * (128)] = 2813;
}
static {
mm[(88-(1))+( 1 -(1)) * (128)] = 837;
mm[(88-(1))+( 2 -(1)) * (128)] = 1410;
mm[(88-(1))+( 3 -(1)) * (128)] = 817;
mm[(88-(1))+( 4 -(1)) * (128)] = 2337;
}
static {
mm[(89-(1))+( 1 -(1)) * (128)] = 2826;
mm[(89-(1))+( 2 -(1)) * (128)] = 3723;
mm[(89-(1))+( 3 -(1)) * (128)] = 3039;
mm[(89-(1))+( 4 -(1)) * (128)] = 1429;
}
static {
mm[(90-(1))+( 1 -(1)) * (128)] = 2332;
mm[(90-(1))+( 2 -(1)) * (128)] = 2803;
mm[(90-(1))+( 3 -(1)) * (128)] = 1696;
mm[(90-(1))+( 4 -(1)) * (128)] = 1177;
}
static {
mm[(91-(1))+( 1 -(1)) * (128)] = 2089;
mm[(91-(1))+( 2 -(1)) * (128)] = 3185;
mm[(91-(1))+( 3 -(1)) * (128)] = 1256;
mm[(91-(1))+( 4 -(1)) * (128)] = 1901;
}
static {
mm[(92-(1))+( 1 -(1)) * (128)] = 3780;
mm[(92-(1))+( 2 -(1)) * (128)] = 184;
mm[(92-(1))+( 3 -(1)) * (128)] = 3715;
mm[(92-(1))+( 4 -(1)) * (128)] = 81;
}
static {
mm[(93-(1))+( 1 -(1)) * (128)] = 1700;
mm[(93-(1))+( 2 -(1)) * (128)] = 663;
mm[(93-(1))+( 3 -(1)) * (128)] = 2077;
mm[(93-(1))+( 4 -(1)) * (128)] = 1669;
}
static {
mm[(94-(1))+( 1 -(1)) * (128)] = 3712;
mm[(94-(1))+( 2 -(1)) * (128)] = 499;
mm[(94-(1))+( 3 -(1)) * (128)] = 3019;
mm[(94-(1))+( 4 -(1)) * (128)] = 2633;
}
static {
mm[(95-(1))+( 1 -(1)) * (128)] = 150;
mm[(95-(1))+( 2 -(1)) * (128)] = 3784;
mm[(95-(1))+( 3 -(1)) * (128)] = 1497;
mm[(95-(1))+( 4 -(1)) * (128)] = 2269;
}
static {
mm[(96-(1))+( 1 -(1)) * (128)] = 2000;
mm[(96-(1))+( 2 -(1)) * (128)] = 1631;
mm[(96-(1))+( 3 -(1)) * (128)] = 1101;
mm[(96-(1))+( 4 -(1)) * (128)] = 129;
}
static {
mm[(97-(1))+( 1 -(1)) * (128)] = 3375;
mm[(97-(1))+( 2 -(1)) * (128)] = 1925;
mm[(97-(1))+( 3 -(1)) * (128)] = 717;
mm[(97-(1))+( 4 -(1)) * (128)] = 1141;
}
static {
mm[(98-(1))+( 1 -(1)) * (128)] = 1621;
mm[(98-(1))+( 2 -(1)) * (128)] = 3912;
mm[(98-(1))+( 3 -(1)) * (128)] = 51;
mm[(98-(1))+( 4 -(1)) * (128)] = 249;
}
static {
mm[(99-(1))+( 1 -(1)) * (128)] = 3090;
mm[(99-(1))+( 2 -(1)) * (128)] = 1398;
mm[(99-(1))+( 3 -(1)) * (128)] = 981;
mm[(99-(1))+( 4 -(1)) * (128)] = 3917;
}
static {
mm[(100-(1))+( 1 -(1)) * (128)] = 3765;
mm[(100-(1))+( 2 -(1)) * (128)] = 1349;
mm[(100-(1))+( 3 -(1)) * (128)] = 1978;
mm[(100-(1))+( 4 -(1)) * (128)] = 2481;
}
static {
mm[(101-(1))+( 1 -(1)) * (128)] = 1149;
mm[(101-(1))+( 2 -(1)) * (128)] = 1441;
mm[(101-(1))+( 3 -(1)) * (128)] = 1813;
mm[(101-(1))+( 4 -(1)) * (128)] = 3941;
}
static {
mm[(102-(1))+( 1 -(1)) * (128)] = 3146;
mm[(102-(1))+( 2 -(1)) * (128)] = 2224;
mm[(102-(1))+( 3 -(1)) * (128)] = 3881;
mm[(102-(1))+( 4 -(1)) * (128)] = 2217;
}
static {
mm[(103-(1))+( 1 -(1)) * (128)] = 33;
mm[(103-(1))+( 2 -(1)) * (128)] = 2411;
mm[(103-(1))+( 3 -(1)) * (128)] = 76;
mm[(103-(1))+( 4 -(1)) * (128)] = 2749;
}
static {
mm[(104-(1))+( 1 -(1)) * (128)] = 3082;
mm[(104-(1))+( 2 -(1)) * (128)] = 1907;
mm[(104-(1))+( 3 -(1)) * (128)] = 3846;
mm[(104-(1))+( 4 -(1)) * (128)] = 3041;
}
static {
mm[(105-(1))+( 1 -(1)) * (128)] = 2741;
mm[(105-(1))+( 2 -(1)) * (128)] = 3192;
mm[(105-(1))+( 3 -(1)) * (128)] = 3694;
mm[(105-(1))+( 4 -(1)) * (128)] = 1877;
}
static {
mm[(106-(1))+( 1 -(1)) * (128)] = 359;
mm[(106-(1))+( 2 -(1)) * (128)] = 2786;
mm[(106-(1))+( 3 -(1)) * (128)] = 1682;
mm[(106-(1))+( 4 -(1)) * (128)] = 345;
}
static {
mm[(107-(1))+( 1 -(1)) * (128)] = 3316;
mm[(107-(1))+( 2 -(1)) * (128)] = 382;
mm[(107-(1))+( 3 -(1)) * (128)] = 124;
mm[(107-(1))+( 4 -(1)) * (128)] = 2861;
}
static {
mm[(108-(1))+( 1 -(1)) * (128)] = 1749;
mm[(108-(1))+( 2 -(1)) * (128)] = 37;
mm[(108-(1))+( 3 -(1)) * (128)] = 1660;
mm[(108-(1))+( 4 -(1)) * (128)] = 1809;
}
static {
mm[(109-(1))+( 1 -(1)) * (128)] = 185;
mm[(109-(1))+( 2 -(1)) * (128)] = 759;
mm[(109-(1))+( 3 -(1)) * (128)] = 3997;
mm[(109-(1))+( 4 -(1)) * (128)] = 3141;
}
static {
mm[(110-(1))+( 1 -(1)) * (128)] = 2784;
mm[(110-(1))+( 2 -(1)) * (128)] = 2948;
mm[(110-(1))+( 3 -(1)) * (128)] = 479;
mm[(110-(1))+( 4 -(1)) * (128)] = 2825;
}
static {
mm[(111-(1))+( 1 -(1)) * (128)] = 2202;
mm[(111-(1))+( 2 -(1)) * (128)] = 1862;
mm[(111-(1))+( 3 -(1)) * (128)] = 1141;
mm[(111-(1))+( 4 -(1)) * (128)] = 157;
}
static {
mm[(112-(1))+( 1 -(1)) * (128)] = 2199;
mm[(112-(1))+( 2 -(1)) * (128)] = 3802;
mm[(112-(1))+( 3 -(1)) * (128)] = 886;
mm[(112-(1))+( 4 -(1)) * (128)] = 2881;
}
static {
mm[(113-(1))+( 1 -(1)) * (128)] = 1364;
mm[(113-(1))+( 2 -(1)) * (128)] = 2423;
mm[(113-(1))+( 3 -(1)) * (128)] = 3514;
mm[(113-(1))+( 4 -(1)) * (128)] = 3637;
}
static {
mm[(114-(1))+( 1 -(1)) * (128)] = 1244;
mm[(114-(1))+( 2 -(1)) * (128)] = 2051;
mm[(114-(1))+( 3 -(1)) * (128)] = 1301;
mm[(114-(1))+( 4 -(1)) * (128)] = 1465;
}
static {
mm[(115-(1))+( 1 -(1)) * (128)] = 2020;
mm[(115-(1))+( 2 -(1)) * (128)] = 2295;
mm[(115-(1))+( 3 -(1)) * (128)] = 3604;
mm[(115-(1))+( 4 -(1)) * (128)] = 2829;
}
static {
mm[(116-(1))+( 1 -(1)) * (128)] = 3160;
mm[(116-(1))+( 2 -(1)) * (128)] = 1332;
mm[(116-(1))+( 3 -(1)) * (128)] = 1888;
mm[(116-(1))+( 4 -(1)) * (128)] = 2161;
}
static {
mm[(117-(1))+( 1 -(1)) * (128)] = 2785;
mm[(117-(1))+( 2 -(1)) * (128)] = 1832;
mm[(117-(1))+( 3 -(1)) * (128)] = 1836;
mm[(117-(1))+( 4 -(1)) * (128)] = 3365;
}
static {
mm[(118-(1))+( 1 -(1)) * (128)] = 2772;
mm[(118-(1))+( 2 -(1)) * (128)] = 2405;
mm[(118-(1))+( 3 -(1)) * (128)] = 1990;
mm[(118-(1))+( 4 -(1)) * (128)] = 361;
}
static {
mm[(119-(1))+( 1 -(1)) * (128)] = 1217;
mm[(119-(1))+( 2 -(1)) * (128)] = 3638;
mm[(119-(1))+( 3 -(1)) * (128)] = 2058;
mm[(119-(1))+( 4 -(1)) * (128)] = 2685;
}
static {
mm[(120-(1))+( 1 -(1)) * (128)] = 1822;
mm[(120-(1))+( 2 -(1)) * (128)] = 3661;
mm[(120-(1))+( 3 -(1)) * (128)] = 692;
mm[(120-(1))+( 4 -(1)) * (128)] = 3745;
}
static {
mm[(121-(1))+( 1 -(1)) * (128)] = 1245;
mm[(121-(1))+( 2 -(1)) * (128)] = 327;
mm[(121-(1))+( 3 -(1)) * (128)] = 1194;
mm[(121-(1))+( 4 -(1)) * (128)] = 2325;
}
static {
mm[(122-(1))+( 1 -(1)) * (128)] = 2252;
mm[(122-(1))+( 2 -(1)) * (128)] = 3660;
mm[(122-(1))+( 3 -(1)) * (128)] = 20;
mm[(122-(1))+( 4 -(1)) * (128)] = 3609;
}
static {
mm[(123-(1))+( 1 -(1)) * (128)] = 3904;
mm[(123-(1))+( 2 -(1)) * (128)] = 716;
mm[(123-(1))+( 3 -(1)) * (128)] = 3285;
mm[(123-(1))+( 4 -(1)) * (128)] = 3821;
}
static {
mm[(124-(1))+( 1 -(1)) * (128)] = 2774;
mm[(124-(1))+( 2 -(1)) * (128)] = 1842;
mm[(124-(1))+( 3 -(1)) * (128)] = 2046;
mm[(124-(1))+( 4 -(1)) * (128)] = 3537;
}
static {
mm[(125-(1))+( 1 -(1)) * (128)] = 997;
mm[(125-(1))+( 2 -(1)) * (128)] = 3987;
mm[(125-(1))+( 3 -(1)) * (128)] = 2107;
mm[(125-(1))+( 4 -(1)) * (128)] = 517;
}
static {
mm[(126-(1))+( 1 -(1)) * (128)] = 2573;
mm[(126-(1))+( 2 -(1)) * (128)] = 1368;
mm[(126-(1))+( 3 -(1)) * (128)] = 3508;
mm[(126-(1))+( 4 -(1)) * (128)] = 3017;
}
static {
mm[(127-(1))+( 1 -(1)) * (128)] = 1148;
mm[(127-(1))+( 2 -(1)) * (128)] = 1848;
mm[(127-(1))+( 3 -(1)) * (128)] = 3525;
mm[(127-(1))+( 4 -(1)) * (128)] = 2141;
}
/***
*     ..
*     .. Executable Statements ..
*
***/
static {
mm[(128-(1))+( 1 -(1)) * (128)] = 545;
mm[(128-(1))+( 2 -(1)) * (128)] = 2366;
mm[(128-(1))+( 3 -(1)) * (128)] = 3801;
mm[(128-(1))+( 4 -(1)) * (128)] = 1537;
}

public static void dlaruv (int [] iseed, int _iseed_offset,
int n,
double [] x, int _x_offset)  {

label_999999:{

int i= 0;
int i1= 0;
int i2= 0;
int i3= 0;
int i4= 0;
int it1= 0;
int it2= 0;
int it3= 0;
int it4= 0;
int j= 0;
i1 = iseed[(1-(1))+ _iseed_offset];
i2 = iseed[(2-(1))+ _iseed_offset];
i3 = iseed[(3-(1))+ _iseed_offset];
// *
i4 = iseed[(4-(1))+ _iseed_offset];
// *
{
for (i = 1; i <= Math.min(n, 128); i++) {
/***
*
*        Multiply the seed by i-th power of the multiplier modulo 2**48
*
***/
label20:for(;;){
	//Dummy.label("com/polytechnik/lapack310/Dlaruv",20);
it4 = (i4*mm[(i-(1))+(4-(1)) * (128)]);
it3 = (it4/4096);
it4 = (it4-(4096*it3));
it3 = ((it3+(i3*mm[(i-(1))+(4-(1)) * (128)]))+(i4*mm[(i-(1))+(3-(1)) * (128)]));
it2 = (it3/4096);
it3 = (it3-(4096*it2));
it2 = (((it2+(i2*mm[(i-(1))+(4-(1)) * (128)]))+(i3*mm[(i-(1))+(3-(1)) * (128)]))+(i4*mm[(i-(1))+(2-(1)) * (128)]));
it1 = (it2/4096);
it2 = (it2-(4096*it1));
it1 = ((((it1+(i1*mm[(i-(1))+(4-(1)) * (128)]))+(i2*mm[(i-(1))+(3-(1)) * (128)]))+(i3*mm[(i-(1))+(2-(1)) * (128)]))+(i4*mm[(i-(1))+(1-(1)) * (128)]));
/***
*
*        Convert 48-bit integer to a real number in the interval (0,1)
*
***/
it1 = (it1)%(4096);
// *
x[(i-(1))+ _x_offset] = (0.000244140625*(((double)(it1)+(0.000244140625*(((double)(it2)+(0.000244140625*(((double)(it3)+(0.000244140625*(double)(it4)))))))))));
/***
*           If a real number has n bits of precision, and the first
*           n bits of the 48-bit integer above happen to be all 1 (which
*           will occur about once every 2**n calls), then X( I ) will
*           be rounded to exactly 1.0.
*           Since X( I ) is not supposed to return exactly 0.0 or 1.0,
*           the statistically correct thing to do in this situation is
*           simply to iterate again.
*           N.B. the case X( I ) = 0.0 should not be possible.
***/
if ((x[(i-(1))+ _x_offset] == 1.0e0))  {
    i1 = (i1+2);
i2 = (i2+2);
i3 = (i3+2);
i4 = (i4+2);
continue label20;//Dummy.go_to("com/polytechnik/lapack310/Dlaruv",20);
// *
}
break label20;}
/***
*
*     Return final value of seed
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaruv",10);
}              //  Close for() loop.
}
iseed[(1-(1))+ _iseed_offset] = it1;
iseed[(2-(1))+ _iseed_offset] = it2;
iseed[(3-(1))+ _iseed_offset] = it3;
iseed[(4-(1))+ _iseed_offset] = it4;
/***
*
*     End of DLARUV
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlaruv",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlaruv",999999);
return;
   }
} // End class.
