/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlas2.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlas2 {

// *> \brief \b DLAS2 computes singular values of a 2-by-2 triangular matri
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLAS2 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAS2( F, G, H, SSMIN, SSMAX )
// *
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION   F, G, H, SSMAX, SSMIN
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLAS2  computes the singular values of the 2-by-2 matrix
// *>    [  F   G  ]
// *>    [  0   H  ].
// *> On return, SSMIN is the smaller singular value and SSMAX is the
// *> larger singular value.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] F
// *> \verbatim
// *>          F is DOUBLE PRECISION
// *>          The (1,1) element of the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[in] G
// *> \verbatim
// *>          G is DOUBLE PRECISION
// *>          The (1,2) element of the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[in] H
// *> \verbatim
// *>          H is DOUBLE PRECISION
// *>          The (2,2) element of the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[out] SSMIN
// *> \verbatim
// *>          SSMIN is DOUBLE PRECISION
// *>          The smaller singular value.
// *> \endverbatim
// *>
// *> \param[out] SSMAX
// *> \verbatim
// *>          SSMAX is DOUBLE PRECISION
// *>          The larger singular value.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup OTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Barring over/underflow, all output quantities are correct to within

// *>  a few units in the last place (ulps), even in the absence of a guard
// *>  digit in addition/subtraction.
// *>
// *>  In IEEE arithmetic, the code works correctly if one matrix element i
// *>  infinite.
// *>
// *>  Overflow will not occur unless the largest singular value itself
// *>  overflows, or is within a few ulps of overflow. (On machines with
// *>  partial overflow, like the Cray, overflow may occur if the largest
// *>  singular value is within a factor of 2 of overflow.)
// *>
// *>  Underflow is harmless if underflow is gradual. Otherwise, results
// *>  may correspond to a matrix modified by perturbations of size near
// *>  the underflow threshold.
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*
*  ====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlas2 (double f,
double g,
double h,
doubleW ssmin,
doubleW ssmax)  {

label_999999:{

double as= 0.0d;
double at= 0.0d;
double au= 0.0d;
double c= 0.0d;
double fa= 0.0d;
double fhmn= 0.0d;
double fhmx= 0.0d;
double ga= 0.0d;
double ha= 0.0d;
fa = Math.abs(f);
ga = Math.abs(g);
ha = Math.abs(h);
fhmn = Math.min(fa, ha);
fhmx = Math.max(fa, ha);
if ((fhmn == 0.0))  {
    ssmin.val = 0.0;
if ((fhmx == 0.0))  {
    ssmax.val = ga;
}
else  {
  ssmax.val = (Math.max(fhmx, ga)*Math.sqrt((1.0+( Math.pow(((Math.min(fhmx, ga)/Math.max(fhmx, ga))), 2)))));
}              //  Close else.
}
else  {
  if ((ga < fhmx))  {
    as = (1.0+(fhmn/fhmx));
at = (((fhmx-fhmn))/fhmx);
au = ( Math.pow(((ga/fhmx)), 2));
c = (2.0/((Math.sqrt(((as*as)+au))+Math.sqrt(((at*at)+au)))));
ssmin.val = (fhmn*c);
ssmax.val = (fhmx/c);
}
else  {
  au = (fhmx/ga);
/***
*
*              Avoid possible harmful underflow if exponent range
*              asymmetric (true SSMIN may not underflow even if
*              AU underflows)
*
***/
if ((au == 0.0))  {
    ssmin.val = (((fhmn*fhmx))/ga);
ssmax.val = ga;
}
else  {
  as = (1.0+(fhmn/fhmx));
at = (((fhmx-fhmn))/fhmx);
c = (1.0/((Math.sqrt((1.0+( Math.pow(((as*au)), 2))))+Math.sqrt((1.0+( Math.pow(((at*au)), 2)))))));
ssmin.val = (((fhmn*c))*au);
ssmin.val = (ssmin.val+ssmin.val);
ssmax.val = (ga/((c+c)));
}              //  Close else.
}              //  Close else.
}              //  Close else.
/***
*
*     End of DLAS2
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlas2",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlas2",999999);
return;
   }
} // End class.
