/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlasq1.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlasq1 {

// *> \brief \b DLASQ1 computes the singular values of a real square bidiag
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLASQ1 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASQ1( N, D, E, WORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   D( * ), E( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASQ1 computes the singular values of a real N-by-N bidiagonal
// *> matrix with diagonal D and off-diagonal E. The singular values
// *> are computed to high relative accuracy, in the absence of
// *> denormalization, underflow and overflow. The algorithm was first
// *> presented in
// *>
// *> "Accurate singular values and differential qd algorithms" by K. V.
// *> Fernando and B. N. Parlett, Numer. Math., Vol-67, No. 2, pp. 191-230,
// *> 1994,
// *>
// *> and the present implementation is described in "An implementation of

// *> the dqds Algorithm (Positive Case)", LAPACK Working Note.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>        The number of rows and columns in the matrix. N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>        On entry, D contains the diagonal elements of the
// *>        bidiagonal matrix whose SVD is desired. On normal exit,
// *>        D contains the singular values in decreasing order.
// *> \endverbatim
// *>
// *> \param[in,out] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N)
// *>        On entry, elements E(1:N-1) contain the off-diagonal elements

// *>        of the bidiagonal matrix whose SVD is desired.
// *>        On exit, E is overwritten.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (4*N)
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>        = 0: successful exit
// *>        < 0: if INFO = -i, the i-th argument had an illegal value
// *>        > 0: the algorithm failed
// *>             = 1, a split was marked by a positive value in E
// *>             = 2, current block of Z not diagonalized after 100*N
// *>                  iterations (in inner while loop)  On exit D and E
// *>                  represent a matrix with the same singular values
// *>                  which the calling subroutine could use to finish the
// *>                  computation, or even feed back into DLASQ1
// *>             = 3, termination criterion of outer while loop not met
// *>                  (program created more than N unreduced blocks)
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlasq1 (int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] work, int _work_offset,
intW info)  {

label_999999:{

int i= 0;
intW iinfo= new intW(0);
double eps= 0.0d;
double scale= 0.0d;
double safmin= 0.0d;
doubleW sigmn= new doubleW(0.0d);
doubleW sigmx= new doubleW(0.0d);
info.val = 0;
if ((n < 0))  {
    info.val = -1;
com.polytechnik.lapack310.Xerbla.xerbla("DLASQ1",(-(info.val)));
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq1",999999);
}
else if ((n == 0))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq1",999999);
}              // Close else if()
else if ((n == 1))  {
    d[(1-(1))+ _d_offset] = Math.abs(d[(1-(1))+ _d_offset]);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq1",999999);
}              // Close else if()
else if ((n == 2))  {
    com.polytechnik.lapack310.Dlas2.dlas2(d[(1-(1))+ _d_offset],e[(1-(1))+ _e_offset],d[(2-(1))+ _d_offset],sigmn,sigmx);
d[(1-(1))+ _d_offset] = sigmx.val;
d[(2-(1))+ _d_offset] = sigmn.val;
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq1",999999);
/***
*
*     Estimate the largest singular value.
*
***/
}              // Close else if()
sigmx.val = 0.0;
{
for (i = 1; i <= (n-1); i++) {
d[(i-(1))+ _d_offset] = Math.abs(d[(i-(1))+ _d_offset]);
sigmx.val = Math.max(sigmx.val, Math.abs(e[(i-(1))+ _e_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlasq1",10);
}              //  Close for() loop.
}
/***
*
*     Early return if SIGMX is zero (matrix is already diagonal).
*
***/
d[(n-(1))+ _d_offset] = Math.abs(d[(n-(1))+ _d_offset]);
if ((sigmx.val == 0.0))  {
    com.polytechnik.lapack310.Dlasrt.dlasrt("D",n,d,_d_offset,iinfo);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq1",999999);
// *
}
{
for (i = 1; i <= n; i++) {
sigmx.val = Math.max(sigmx.val, d[(i-(1))+ _d_offset]);
/***
*
*     Copy D and E into WORK (in the Z format) and scale (squaring the
*     input data makes scaling by a power of the radix pointless).
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlasq1",20);
}              //  Close for() loop.
}
eps = com.polytechnik.lapack310.Dlamch.dlamch("Precision");
safmin = com.polytechnik.lapack310.Dlamch.dlamch("Safe minimum");
scale = Math.sqrt((eps/safmin));
com.polytechnik.lapack310.Dcopy.dcopy(n,d,_d_offset,1,work,(1-(1))+ _work_offset,2);
com.polytechnik.lapack310.Dcopy.dcopy((n-1),e,_e_offset,1,work,(2-(1))+ _work_offset,2);
/***
*
*     Compute the q's and e's.
*
***/
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,sigmx.val,scale,((2*n)-1),1,work,_work_offset,((2*n)-1),iinfo);
{
for (i = 1; i <= ((2*n)-1); i++) {
work[(i-(1))+ _work_offset] = ( Math.pow(work[(i-(1))+ _work_offset], 2));
//Dummy.label("com/polytechnik/lapack310/Dlasq1",30);
}              //  Close for() loop.
}
// *
work[((2*n)-(1))+ _work_offset] = 0.0;
// *
com.polytechnik.lapack310.Dlasq2.dlasq2(n,work,_work_offset,info);
if ((info.val == 0))  {
    {
for (i = 1; i <= n; i++) {
d[(i-(1))+ _d_offset] = Math.sqrt(work[(i-(1))+ _work_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlasq1",40);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,scale,sigmx.val,n,1,d,_d_offset,n,iinfo);
/***
*
*     Maximum number of iterations exceeded.  Move data from WORK
*     into D and E so the calling subroutine can try to finish
*
***/
}
else if ((info.val == 2))  {
    {
for (i = 1; i <= n; i++) {
d[(i-(1))+ _d_offset] = Math.sqrt(work[(((2*i)-1)-(1))+ _work_offset]);
e[(i-(1))+ _e_offset] = Math.sqrt(work[((2*i)-(1))+ _work_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlasq1",1000000);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,scale,sigmx.val,n,1,d,_d_offset,n,iinfo);
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,scale,sigmx.val,n,1,e,_e_offset,n,iinfo);
// *
}              // Close else if()
/***
*
*     End of DLASQ1
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq1",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlasq1",999999);
return;
   }
} // End class.
