/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlasq2.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlasq2 {

// *> \brief \b DLASQ2 computes all the eigenvalues of the symmetric positi
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLASQ2 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASQ2( N, Z, INFO )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   Z( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASQ2 computes all the eigenvalues of the symmetric positive
// *> definite tridiagonal matrix associated with the qd array Z to high
// *> relative accuracy are computed to high relative accuracy, in the
// *> absence of denormalization, underflow and overflow.
// *>
// *> To see the relation of Z to the tridiagonal matrix, let L be a
// *> unit lower bidiagonal matrix with subdiagonals Z(2,4,6,,..) and
// *> let U be an upper bidiagonal matrix with 1's above and diagonal
// *> Z(1,3,5,,..). The tridiagonal is L*U or, if you prefer, the
// *> symmetric tridiagonal to which it is similar.
// *>
// *> Note : DLASQ2 defines a logical variable, IEEE, which is true
// *> on machines which follow ieee-754 floating-point standard in their
// *> handling of infinities and NaNs, and false otherwise. This variable
// *> is passed to DLASQ3.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>        The number of rows and columns in the matrix. N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension ( 4*N )
// *>        On entry Z holds the qd array. On exit, entries 1 to N hold
// *>        the eigenvalues in decreasing order, Z( 2*N+1 ) holds the
// *>        trace, and Z( 2*N+2 ) holds the sum of the eigenvalues. If
// *>        N > 2, then Z( 2*N+3 ) holds the iteration count, Z( 2*N+4 )
// *>        holds NDIVS/NIN^2, and Z( 2*N+5 ) holds the percentage of
// *>        shifts that failed.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>        = 0: successful exit
// *>        < 0: if the i-th argument is a scalar and had an illegal
// *>             value, then INFO = -i, if the i-th argument is an
// *>             array and the j-entry had an illegal value, then
// *>             INFO = -(i*100+j)
// *>        > 0: the algorithm failed
// *>              = 1, a split was marked by a positive value in E
// *>              = 2, current block of Z not diagonalized after 100*N
// *>                   iterations (in inner while loop).  On exit Z holds

// *>                   a qd array with the same eigenvalues as the given Z
// *>              = 3, termination criterion of outer while loop not met
// *>                   (program created more than N unreduced blocks)
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Local Variables: I0:N0 defines a current unreduced segment of Z.
// *>  The shifts are accumulated in SIGMA. Iteration count is in ITER.
// *>  Ping-pong is controlled by PP (alternates between 0 and 1).
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments.
*     (in case DLASQ2 is not called by DLASQ1)
*
***/

public static void dlasq2 (int n,
double [] z, int _z_offset,
intW info)  {

label_999999:{

boolean ieee= false;
int i0= 0;
int i1= 0;
int i4= 0;
intW iinfo= new intW(0);
int ipn4= 0;
intW iter= new intW(0);
int iwhila= 0;
int iwhilb= 0;
int k= 0;
int kmin= 0;
intW n0= new intW(0);
int n1= 0;
int nbig= 0;
intW ndiv= new intW(0);
intW nfail= new intW(0);
intW pp= new intW(0);
int splt= 0;
intW ttype= new intW(0);
double d= 0.0d;
double dee= 0.0d;
double deemin= 0.0d;
doubleW desig= new doubleW(0.0d);
doubleW dmin= new doubleW(0.0d);
doubleW dmin1= new doubleW(0.0d);
doubleW dmin2= new doubleW(0.0d);
doubleW dn= new doubleW(0.0d);
doubleW dn1= new doubleW(0.0d);
doubleW dn2= new doubleW(0.0d);
double e= 0.0d;
double emax= 0.0d;
double emin= 0.0d;
double eps= 0.0d;
doubleW g= new doubleW(0.0d);
double oldemn= 0.0d;
doubleW qmax= new doubleW(0.0d);
double qmin= 0.0d;
double s= 0.0d;
double safmin= 0.0d;
doubleW sigma= new doubleW(0.0d);
double t= 0.0d;
doubleW tau= new doubleW(0.0d);
double temp= 0.0d;
double tol= 0.0d;
double tol2= 0.0d;
double trace= 0.0d;
double zmax= 0.0d;
double tempe= 0.0d;
double tempq= 0.0d;
info.val = 0;
eps = com.polytechnik.lapack310.Dlamch.dlamch("Precision");
safmin = com.polytechnik.lapack310.Dlamch.dlamch("Safe minimum");
tol = (eps*100.0);
// *
tol2 = ( Math.pow(tol, 2));
if ((n < 0))  {
    info.val = -1;
com.polytechnik.lapack310.Xerbla.xerbla("DLASQ2",1);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
}
else if ((n == 0))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
/***
*
*        1-by-1 case.
*
***/
}              // Close else if()
else if ((n == 1))  {
    if ((z[(1-(1))+ _z_offset] < 0.0))  {
    info.val = -201;
com.polytechnik.lapack310.Xerbla.xerbla("DLASQ2",2);
}
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
/***
*
*        2-by-2 case.
*
***/
}              // Close else if()
else if ((n == 2))  {
    if ((z[(1-(1))+ _z_offset] < 0.0))  {
    info.val = -201;
com.polytechnik.lapack310.Xerbla.xerbla("DLASQ2",2);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
}
else if ((z[(2-(1))+ _z_offset] < 0.0))  {
    info.val = -202;
com.polytechnik.lapack310.Xerbla.xerbla("DLASQ2",2);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
}              // Close else if()
else if ((z[(3-(1))+ _z_offset] < 0.0))  {
    info.val = -203;
com.polytechnik.lapack310.Xerbla.xerbla("DLASQ2",2);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
}              // Close else if()
else if ((z[(3-(1))+ _z_offset] > z[(1-(1))+ _z_offset]))  {
    d = z[(3-(1))+ _z_offset];
z[(3-(1))+ _z_offset] = z[(1-(1))+ _z_offset];
z[(1-(1))+ _z_offset] = d;
}              // Close else if()
z[(5-(1))+ _z_offset] = ((z[(1-(1))+ _z_offset]+z[(2-(1))+ _z_offset])+z[(3-(1))+ _z_offset]);
if ((z[(2-(1))+ _z_offset] > (z[(3-(1))+ _z_offset]*tol2)))  {
    t = (0.5*((((z[(1-(1))+ _z_offset]-z[(3-(1))+ _z_offset]))+z[(2-(1))+ _z_offset])));
s = (z[(3-(1))+ _z_offset]*((z[(2-(1))+ _z_offset]/t)));
if ((s <= t))  {
    s = (z[(3-(1))+ _z_offset]*((z[(2-(1))+ _z_offset]/((t*((1.0+Math.sqrt((1.0+(s/t))))))))));
}
else  {
  s = (z[(3-(1))+ _z_offset]*((z[(2-(1))+ _z_offset]/((t+(Math.sqrt(t)*Math.sqrt((t+s))))))));
}              //  Close else.
t = (z[(1-(1))+ _z_offset]+((s+z[(2-(1))+ _z_offset])));
z[(3-(1))+ _z_offset] = (z[(3-(1))+ _z_offset]*((z[(1-(1))+ _z_offset]/t)));
z[(1-(1))+ _z_offset] = t;
}
z[(2-(1))+ _z_offset] = z[(3-(1))+ _z_offset];
z[(6-(1))+ _z_offset] = (z[(2-(1))+ _z_offset]+z[(1-(1))+ _z_offset]);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
/***
*
*     Check for negative data and compute sums of q's and e's.
*
***/
}              // Close else if()
z[((2*n)-(1))+ _z_offset] = 0.0;
emin = z[(2-(1))+ _z_offset];
qmax.val = 0.0;
zmax = 0.0;
d = 0.0;
// *
e = 0.0;
{
int _k_inc = 2;
for (k = 1; k <= (2*((n-1))); k += _k_inc) {
if ((z[(k-(1))+ _z_offset] < 0.0))  {
    info.val = (-(((200+k))));
com.polytechnik.lapack310.Xerbla.xerbla("DLASQ2",2);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
}
else if ((z[((k+1)-(1))+ _z_offset] < 0.0))  {
    info.val = (-((((200+k)+1))));
com.polytechnik.lapack310.Xerbla.xerbla("DLASQ2",2);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
}              // Close else if()
d = (d+z[(k-(1))+ _z_offset]);
e = (e+z[((k+1)-(1))+ _z_offset]);
qmax.val = Math.max(qmax.val, z[(k-(1))+ _z_offset]);
emin = Math.min(emin, z[((k+1)-(1))+ _z_offset]);
zmax = Util.max(qmax.val, zmax, z[((k+1)-(1))+ _z_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlasq2",10);
}              //  Close for() loop.
}
if ((z[(((2*n)-1)-(1))+ _z_offset] < 0.0))  {
    info.val = (-((((200+(2*n))-1))));
com.polytechnik.lapack310.Xerbla.xerbla("DLASQ2",2);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
}
d = (d+z[(((2*n)-1)-(1))+ _z_offset]);
qmax.val = Math.max(qmax.val, z[(((2*n)-1)-(1))+ _z_offset]);
/***
*
*     Check for diagonality.
*
***/
zmax = Math.max(qmax.val, zmax);
if ((e == 0.0))  {
    {
for (k = 2; k <= n; k++) {
z[(k-(1))+ _z_offset] = z[(((2*k)-1)-(1))+ _z_offset];
//Dummy.label("com/polytechnik/lapack310/Dlasq2",20);
}              //  Close for() loop.
}
com.polytechnik.lapack310.Dlasrt.dlasrt("D",n,z,_z_offset,iinfo);
z[(((2*n)-1)-(1))+ _z_offset] = d;
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
// *
}
/***
*
*     Check for zero data.
*
***/
trace = (d+e);
if ((trace == 0.0))  {
    z[(((2*n)-1)-(1))+ _z_offset] = 0.0;
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
/***
*
*     Check whether the machine is IEEE conformable.
*
***/
}
/***
*
*     Rearrange data for locality: Z=(q1,qq1,e1,ee1,q2,qq2,e2,ee2,...).
*
***/
ieee = true;//((com.polytechnik.lapack310.Ilaenv.ilaenv(10,"DLASQ2","N",1,2,3,4) == 1));
{
int _k_inc = -2;
for (k = (2*n); k >= 2; k += _k_inc) {
z[((2*k)-(1))+ _z_offset] = 0.0;
z[(((2*k)-1)-(1))+ _z_offset] = z[(k-(1))+ _z_offset];
z[(((2*k)-2)-(1))+ _z_offset] = 0.0;
z[(((2*k)-3)-(1))+ _z_offset] = z[((k-1)-(1))+ _z_offset];
// *
//Dummy.label("com/polytechnik/lapack310/Dlasq2",30);
}              //  Close for() loop.
}
i0 = 1;
/***
*
*     Reverse the qd-array, if warranted.
*
***/
n0.val = n;
if (((1.5*z[(((4*i0)-3)-(1))+ _z_offset]) < z[(((4*n0.val)-3)-(1))+ _z_offset]))  {
    ipn4 = (4*((i0+n0.val)));
{
int _i4_inc = 4;
for (i4 = (4*i0); i4 <= (2*(((i0+n0.val)-1))); i4 += _i4_inc) {
temp = z[((i4-3)-(1))+ _z_offset];
z[((i4-3)-(1))+ _z_offset] = z[(((ipn4-i4)-3)-(1))+ _z_offset];
z[(((ipn4-i4)-3)-(1))+ _z_offset] = temp;
temp = z[((i4-1)-(1))+ _z_offset];
z[((i4-1)-(1))+ _z_offset] = z[(((ipn4-i4)-5)-(1))+ _z_offset];
z[(((ipn4-i4)-5)-(1))+ _z_offset] = temp;
//Dummy.label("com/polytechnik/lapack310/Dlasq2",40);
}              //  Close for() loop.
}
/***
*
*     Initial split checking via dqd and Li's test.
*
***/
}
// *
pp.val = 0;
// *
{
for (k = 1; k <= 2; k++) {
d = z[((((4*n0.val)+pp.val)-3)-(1))+ _z_offset];
{
int _i4_inc = -4;
for (i4 = ((4*((n0.val-1)))+pp.val); i4 >= ((4*i0)+pp.val); i4 += _i4_inc) {
if ((z[((i4-1)-(1))+ _z_offset] <= (tol2*d)))  {
    z[((i4-1)-(1))+ _z_offset] = -0.0;
d = z[((i4-3)-(1))+ _z_offset];
}
else  {
  d = (z[((i4-3)-(1))+ _z_offset]*((d/((d+z[((i4-1)-(1))+ _z_offset])))));
}              //  Close else.
/***
*
*        dqd maps Z to ZZ plus Li's test.
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlasq2",50);
}              //  Close for() loop.
}
emin = z[((((4*i0)+pp.val)+1)-(1))+ _z_offset];
d = z[((((4*i0)+pp.val)-3)-(1))+ _z_offset];
{
int _i4_inc = 4;
for (i4 = ((4*i0)+pp.val); i4 <= ((4*((n0.val-1)))+pp.val); i4 += _i4_inc) {
z[(((i4-(2*pp.val))-2)-(1))+ _z_offset] = (d+z[((i4-1)-(1))+ _z_offset]);
if ((z[((i4-1)-(1))+ _z_offset] <= (tol2*d)))  {
    z[((i4-1)-(1))+ _z_offset] = -0.0;
z[(((i4-(2*pp.val))-2)-(1))+ _z_offset] = d;
z[((i4-(2*pp.val))-(1))+ _z_offset] = 0.0;
d = z[((i4+1)-(1))+ _z_offset];
}
else if ((((safmin*z[((i4+1)-(1))+ _z_offset]) < z[(((i4-(2*pp.val))-2)-(1))+ _z_offset]) && ((safmin*z[(((i4-(2*pp.val))-2)-(1))+ _z_offset]) < z[((i4+1)-(1))+ _z_offset])))  {
    temp = (z[((i4+1)-(1))+ _z_offset]/z[(((i4-(2*pp.val))-2)-(1))+ _z_offset]);
z[((i4-(2*pp.val))-(1))+ _z_offset] = (z[((i4-1)-(1))+ _z_offset]*temp);
d = (d*temp);
}              // Close else if()
else  {
  z[((i4-(2*pp.val))-(1))+ _z_offset] = (z[((i4+1)-(1))+ _z_offset]*((z[((i4-1)-(1))+ _z_offset]/z[(((i4-(2*pp.val))-2)-(1))+ _z_offset])));
d = (z[((i4+1)-(1))+ _z_offset]*((d/z[(((i4-(2*pp.val))-2)-(1))+ _z_offset])));
}              //  Close else.
emin = Math.min(emin, z[((i4-(2*pp.val))-(1))+ _z_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlasq2",60);
}              //  Close for() loop.
}
/***
*
*        Now find qmax.
*
***/
z[((((4*n0.val)-pp.val)-2)-(1))+ _z_offset] = d;
qmax.val = z[((((4*i0)-pp.val)-2)-(1))+ _z_offset];
{
int _i4_inc = 4;
for (i4 = (((4*i0)-pp.val)+2); i4 <= (((4*n0.val)-pp.val)-2); i4 += _i4_inc) {
qmax.val = Math.max(qmax.val, z[(i4-(1))+ _z_offset]);
/***
*
*        Prepare for the next iteration on K.
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlasq2",70);
}              //  Close for() loop.
}
pp.val = (1-pp.val);
/***
*
*     Initialise variables to pass to DLASQ3.
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlasq2",80);
}              //  Close for() loop.
}
ttype.val = 0;
dmin1.val = 0.0;
dmin2.val = 0.0;
dn.val = 0.0;
dn1.val = 0.0;
dn2.val = 0.0;
g.val = 0.0;
// *
tau.val = 0.0;
iter.val = 2;
nfail.val = 0;
// *
ndiv.val = (2*((n0.val-i0)));
label170:{
{
label160:for (iwhila = 1; iwhila <= (n+1); iwhila++) {
/***
*
*        While array unfinished do
*
*        E(N0) holds the value of SIGMA when submatrix in I0:N0
*        splits from the rest of the array, but is negated.
*
***/
if ((n0.val < 1)) {
    break label170;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",170);
}
    desig.val = 0.0;
if ((n0.val == n))  {
    sigma.val = 0.0;
}
else  {
  sigma.val = (-(z[(((4*n0.val)-1)-(1))+ _z_offset]));
}              //  Close else.
if ((sigma.val < 0.0))  {
    info.val = 1;
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
/***
*
*        Find last unreduced submatrix's top index I0, find QMAX and
*        EMIN. Find Gershgorin-type bound if Q's much greater than E's.
*
***/
}
emax = 0.0;
if ((n0.val > i0))  {
    emin = Math.abs(z[(((4*n0.val)-5)-(1))+ _z_offset]);
}
else  {
  emin = 0.0;
}              //  Close else.
qmin = z[(((4*n0.val)-3)-(1))+ _z_offset];
qmax.val = qmin;
label100:{
{
int _i4_inc = -4;
for (i4 = (4*n0.val); i4 >= 8; i4 += _i4_inc) {
if ((z[((i4-5)-(1))+ _z_offset] <= 0.0)) {
    break label100;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",100);
}
    if ((qmin >= (4.0*emax)))  {
    qmin = Math.min(qmin, z[((i4-3)-(1))+ _z_offset]);
emax = Math.max(emax, z[((i4-5)-(1))+ _z_offset]);
}
qmax.val = Math.max(qmax.val, (z[((i4-7)-(1))+ _z_offset]+z[((i4-5)-(1))+ _z_offset]));
emin = Math.min(emin, z[((i4-5)-(1))+ _z_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlasq2",90);
}              //  Close for() loop.
}
// *
i4 = 4;
break label100;}
//Dummy.label("com/polytechnik/lapack310/Dlasq2",100);
i0 = (i4/4);
// *
pp.val = 0;
if (((n0.val-i0) > 1))  {
    dee = z[(((4*i0)-3)-(1))+ _z_offset];
deemin = dee;
kmin = i0;
{
int _i4_inc = 4;
for (i4 = ((4*i0)+1); i4 <= ((4*n0.val)-3); i4 += _i4_inc) {
dee = (z[(i4-(1))+ _z_offset]*((dee/((dee+z[((i4-2)-(1))+ _z_offset])))));
if ((dee <= deemin))  {
    deemin = dee;
kmin = (((i4+3))/4);
}
//Dummy.label("com/polytechnik/lapack310/Dlasq2",110);
}              //  Close for() loop.
}
if ((((((kmin-i0))*2) < (n0.val-kmin)) && (deemin <= (0.5*z[(((4*n0.val)-3)-(1))+ _z_offset]))))  {
    ipn4 = (4*((i0+n0.val)));
pp.val = 2;
{
int _i4_inc = 4;
for (i4 = (4*i0); i4 <= (2*(((i0+n0.val)-1))); i4 += _i4_inc) {
temp = z[((i4-3)-(1))+ _z_offset];
z[((i4-3)-(1))+ _z_offset] = z[(((ipn4-i4)-3)-(1))+ _z_offset];
z[(((ipn4-i4)-3)-(1))+ _z_offset] = temp;
temp = z[((i4-2)-(1))+ _z_offset];
z[((i4-2)-(1))+ _z_offset] = z[(((ipn4-i4)-2)-(1))+ _z_offset];
z[(((ipn4-i4)-2)-(1))+ _z_offset] = temp;
temp = z[((i4-1)-(1))+ _z_offset];
z[((i4-1)-(1))+ _z_offset] = z[(((ipn4-i4)-5)-(1))+ _z_offset];
z[(((ipn4-i4)-5)-(1))+ _z_offset] = temp;
temp = z[(i4-(1))+ _z_offset];
z[(i4-(1))+ _z_offset] = z[(((ipn4-i4)-4)-(1))+ _z_offset];
z[(((ipn4-i4)-4)-(1))+ _z_offset] = temp;
//Dummy.label("com/polytechnik/lapack310/Dlasq2",120);
}              //  Close for() loop.
}
}
/***
*
*        Put -(initial shift) into DMIN.
*
***/
}
/***
*
*        Now I0:N0 is unreduced.
*        PP = 0 for ping, PP = 1 for pong.
*        PP = 2 indicates that flipping was applied to the Z array and
*               and that the tests for deflation upon entry in DLASQ3
*               should not be performed.
*
***/
dmin.val = (-(Math.max(0.0, (qmin-((2.0*Math.sqrt(qmin))*Math.sqrt(emax))))));
nbig = (100*(((n0.val-i0)+1)));
{
for (iwhilb = 1; iwhilb <= nbig; iwhilb++) {
/***
*
*           While submatrix unfinished take a good dqds step.
*
***/
if ((i0 > n0.val)) {
    continue label160;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",150);
}
    // *
com.polytechnik.lapack310.Dlasq3.dlasq3(i0,n0,z,_z_offset,pp,dmin,sigma,desig,qmax,nfail,iter,ndiv,ieee,ttype,dmin1,dmin2,dn,dn1,dn2,g,tau);
/***
*
*           When EMIN is very small check for splits.
*
***/
pp.val = (1-pp.val);
if (((pp.val == 0) && ((n0.val-i0) >= 3)))  {
    if (((z[((4*n0.val)-(1))+ _z_offset] <= (tol2*qmax.val)) || (z[(((4*n0.val)-1)-(1))+ _z_offset] <= (tol2*sigma.val))))  {
    splt = (i0-1);
qmax.val = z[(((4*i0)-3)-(1))+ _z_offset];
emin = z[(((4*i0)-1)-(1))+ _z_offset];
oldemn = z[((4*i0)-(1))+ _z_offset];
{
int _i4_inc = 4;
for (i4 = (4*i0); i4 <= (4*((n0.val-3))); i4 += _i4_inc) {
if (((z[(i4-(1))+ _z_offset] <= (tol2*z[((i4-3)-(1))+ _z_offset])) || (z[((i4-1)-(1))+ _z_offset] <= (tol2*sigma.val))))  {
    z[((i4-1)-(1))+ _z_offset] = (-(sigma.val));
splt = (i4/4);
qmax.val = 0.0;
emin = z[((i4+3)-(1))+ _z_offset];
oldemn = z[((i4+4)-(1))+ _z_offset];
}
else  {
  qmax.val = Math.max(qmax.val, z[((i4+1)-(1))+ _z_offset]);
emin = Math.min(emin, z[((i4-1)-(1))+ _z_offset]);
oldemn = Math.min(oldemn, z[(i4-(1))+ _z_offset]);
}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dlasq2",130);
}              //  Close for() loop.
}
z[(((4*n0.val)-1)-(1))+ _z_offset] = emin;
z[((4*n0.val)-(1))+ _z_offset] = oldemn;
i0 = (splt+1);
}
// *
}
// *
//Dummy.label("com/polytechnik/lapack310/Dlasq2",140);
}              //  Close for() loop.
}
/***
*
*        Maximum number of iterations exceeded, restore the shift
*        SIGMA and place the new d's and e's in a qd array.
*        This might need to be done for several blocks
*
***/
info.val = 2;
i1 = i0;
n1 = n0.val;
label145:for(;;){
    //Dummy.label("com/polytechnik/lapack310/Dlasq2",145);
tempq = z[(((4*i0)-3)-(1))+ _z_offset];
z[(((4*i0)-3)-(1))+ _z_offset] = (z[(((4*i0)-3)-(1))+ _z_offset]+sigma.val);
{
for (k = (i0+1); k <= n0.val; k++) {
tempe = z[(((4*k)-5)-(1))+ _z_offset];
z[(((4*k)-5)-(1))+ _z_offset] = (z[(((4*k)-5)-(1))+ _z_offset]*((tempq/z[(((4*k)-7)-(1))+ _z_offset])));
tempq = z[(((4*k)-3)-(1))+ _z_offset];
z[(((4*k)-3)-(1))+ _z_offset] = (((z[(((4*k)-3)-(1))+ _z_offset]+sigma.val)+tempe)-z[(((4*k)-5)-(1))+ _z_offset]);
/***
*
*        Prepare to do this on the previous block if there is one
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlasq2",1000000);
}              //  Close for() loop.
}
if ((i1 > 1))  {
    n1 = (i1-1);
{
//for (jjjjj = 1; jjjjj <= 1000000; jjjjj++) {
while ((((i1 >= 2)) && ((z[(((4*i1)-5)-(1))+ _z_offset] >= 0.0))))  {
    i1 = (i1-1);
//else  {
//  Dummy.go_to("com/polytechnik/lapack310/Dlasq2",1774);
//}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dlasq2",1000001);
}              //  Close for() loop.
}
//label1774:
//   Dummy.label("com/polytechnik/lapack310/Dlasq2",1774);
sigma.val = (-(z[(((4*n1)-1)-(1))+ _z_offset]));
continue label145;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",145);
//
}
break label145;}
{
for (k = 1; k <= n; k++) {
/***
*
*        Only the block 1..N0 is unfinished.  The rest of the e's
*        must be essentially zero, although sometimes other data
*        has been stored in them.
*
***/
z[(((2*k)-1)-(1))+ _z_offset] = z[(((4*k)-3)-(1))+ _z_offset];
if ((k < n0.val))  {
    z[((2*k)-(1))+ _z_offset] = z[(((4*k)-1)-(1))+ _z_offset];
}
else  {
  z[((2*k)-(1))+ _z_offset] = (double)(0);
}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dlasq2",1000002);
}              //  Close for() loop.
}
/***
*
*        end IWHILB
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
// *
//Dummy.label("com/polytechnik/lapack310/Dlasq2",150);
// *
//Dummy.label("com/polytechnik/lapack310/Dlasq2",160);
}              //  Close for() loop.
}
info.val = 3;
/***
*
*     end IWHILA
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
/***
*
*     Move q's to the front.
*
***/
}//break label170;
//   Dummy.label("com/polytechnik/lapack310/Dlasq2",170);
{
for (k = 2; k <= n; k++) {
z[(k-(1))+ _z_offset] = z[(((4*k)-3)-(1))+ _z_offset];
/***
*
*     Sort and compute sum of eigenvalues.
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlasq2",180);
}              //  Close for() loop.
}
// *
com.polytechnik.lapack310.Dlasrt.dlasrt("D",n,z,_z_offset,iinfo);
e = 0.0;
{
int _k_inc = -1;
for (k = n; k >= 1; k += _k_inc) {
e = (e+z[(k-(1))+ _z_offset]);
/***
*
*     Store trace, sum(eigenvalues) and information on performance.
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlasq2",190);
}              //  Close for() loop.
}
z[(((2*n)+1)-(1))+ _z_offset] = trace;
z[(((2*n)+2)-(1))+ _z_offset] = e;
z[(((2*n)+3)-(1))+ _z_offset] = (double)(iter.val);
z[(((2*n)+4)-(1))+ _z_offset] = ((double)(ndiv.val)/(double)(((int)Math.pow(n, 2))));
z[(((2*n)+5)-(1))+ _z_offset] = ((100.0*nfail.val)/(double)(iter.val));
/***
*
*     End of DLASQ2
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq2",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlasq2",999999);
return;
   }
} // End class.
