/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlasq3.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlasq3 {

// *> \brief \b DLASQ3 checks for deflation, computes a shift and calls dqd
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLASQ3 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASQ3( I0, N0, Z, PP, DMIN, SIGMA, DESIG, QMAX, NFAI
// *                          ITER, NDIV, IEEE, TTYPE, DMIN1, DMIN2, DN, DN
// *                          DN2, G, TAU )
// *
// *       .. Scalar Arguments ..
// *       LOGICAL            IEEE
// *       INTEGER            I0, ITER, N0, NDIV, NFAIL, PP
// *       DOUBLE PRECISION   DESIG, DMIN, DMIN1, DMIN2, DN, DN1, DN2, G,
// *      $                   QMAX, SIGMA, TAU
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   Z( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASQ3 checks for deflation, computes a shift (TAU) and calls dqds.
// *> In case of failure it changes shifts, and tries again until output
// *> is positive.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] I0
// *> \verbatim
// *>          I0 is INTEGER
// *>         First index.
// *> \endverbatim
// *>
// *> \param[in,out] N0
// *> \verbatim
// *>          N0 is INTEGER
// *>         Last index.
// *> \endverbatim
// *>
// *> \param[in,out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension ( 4*N0 )
// *>         Z holds the qd array.
// *> \endverbatim
// *>
// *> \param[in,out] PP
// *> \verbatim
// *>          PP is INTEGER
// *>         PP=0 for ping, PP=1 for pong.
// *>         PP=2 indicates that flipping was applied to the Z array
// *>         and that the initial tests for deflation should not be
// *>         performed.
// *> \endverbatim
// *>
// *> \param[out] DMIN
// *> \verbatim
// *>          DMIN is DOUBLE PRECISION
// *>         Minimum value of d.
// *> \endverbatim
// *>
// *> \param[out] SIGMA
// *> \verbatim
// *>          SIGMA is DOUBLE PRECISION
// *>         Sum of shifts used in current segment.
// *> \endverbatim
// *>
// *> \param[in,out] DESIG
// *> \verbatim
// *>          DESIG is DOUBLE PRECISION
// *>         Lower order part of SIGMA
// *> \endverbatim
// *>
// *> \param[in] QMAX
// *> \verbatim
// *>          QMAX is DOUBLE PRECISION
// *>         Maximum value of q.
// *> \endverbatim
// *>
// *> \param[in,out] NFAIL
// *> \verbatim
// *>          NFAIL is INTEGER
// *>         Increment NFAIL by 1 each time the shift was too big.
// *> \endverbatim
// *>
// *> \param[in,out] ITER
// *> \verbatim
// *>          ITER is INTEGER
// *>         Increment ITER by 1 for each iteration.
// *> \endverbatim
// *>
// *> \param[in,out] NDIV
// *> \verbatim
// *>          NDIV is INTEGER
// *>         Increment NDIV by 1 for each division.
// *> \endverbatim
// *>
// *> \param[in] IEEE
// *> \verbatim
// *>          IEEE is LOGICAL
// *>         Flag for IEEE or non IEEE arithmetic (passed to DLASQ5).
// *> \endverbatim
// *>
// *> \param[in,out] TTYPE
// *> \verbatim
// *>          TTYPE is INTEGER
// *>         Shift type.
// *> \endverbatim
// *>
// *> \param[in,out] DMIN1
// *> \verbatim
// *>          DMIN1 is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in,out] DMIN2
// *> \verbatim
// *>          DMIN2 is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in,out] DN
// *> \verbatim
// *>          DN is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in,out] DN1
// *> \verbatim
// *>          DN1 is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in,out] DN2
// *> \verbatim
// *>          DN2 is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in,out] G
// *> \verbatim
// *>          G is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in,out] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION
// *>
// *>         These are passed as arguments in order to save their values
// *>         between calls to DLASQ3.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Function ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlasq3 (int i0,
intW n0,
double [] z, int _z_offset,
intW pp,
doubleW dmin,
doubleW sigma,
doubleW desig,
doubleW qmax,
intW nfail,
intW iter,
intW ndiv,
boolean ieee,
intW ttype,
doubleW dmin1,
doubleW dmin2,
doubleW dn,
doubleW dn1,
doubleW dn2,
doubleW g,
doubleW tau)  {

label_999999:{

int ipn4= 0;
int j4= 0;
int n0in= 0;
int nn= 0;
double eps= 0.0d;
double s= 0.0d;
double t= 0.0d;
double temp= 0.0d;
double tol= 0.0d;
double tol2= 0.0d;
n0in = n0.val;
eps = com.polytechnik.lapack310.Dlamch.dlamch("Precision");
tol = (eps*100.0);
/***
*
*     Check for deflation.
*
***/
tol2 = ( Math.pow(tol, 2));
// *
label50:{
label10:for(;;){
   //Dummy.label("com/polytechnik/lapack310/Dlasq3",10);
label40:{
label30:{
label20:{
if ((n0.val < i0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",999999);
}
    if ((n0.val == i0)) {
    break label20;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",20);
}
    nn = ((4*n0.val)+pp.val);
/***
*
*     Check whether E(N0-1) is negligible, 1 eigenvalue.
*
***/
if ((n0.val == ((i0+1)))) {
    break label40;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",40);
}
    // *
if (((z[((nn-5)-(1))+ _z_offset] > (tol2*((sigma.val+z[((nn-3)-(1))+ _z_offset])))) && (z[(((nn-(2*pp.val))-4)-(1))+ _z_offset] > (tol2*z[((nn-7)-(1))+ _z_offset])))) {
    break label30;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",30);
}
    // *
break label20;}
   //Dummy.label("com/polytechnik/lapack310/Dlasq3",20);
z[(((4*n0.val)-3)-(1))+ _z_offset] = (z[((((4*n0.val)+pp.val)-3)-(1))+ _z_offset]+sigma.val);
n0.val = (n0.val-1);
/***
*
*     Check  whether E(N0-2) is negligible, 2 eigenvalues.
*
***/
continue label10;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",10);
// *
}//break label30;
   //Dummy.label("com/polytechnik/lapack310/Dlasq3",30);
// *
if (((z[((nn-9)-(1))+ _z_offset] > (tol2*sigma.val)) && (z[(((nn-(2*pp.val))-8)-(1))+ _z_offset] > (tol2*z[((nn-11)-(1))+ _z_offset])))) {
    break label50;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",50);
}
    // *
break label40;}
   //Dummy.label("com/polytechnik/lapack310/Dlasq3",40);
if ((z[((nn-3)-(1))+ _z_offset] > z[((nn-7)-(1))+ _z_offset]))  {
    s = z[((nn-3)-(1))+ _z_offset];
z[((nn-3)-(1))+ _z_offset] = z[((nn-7)-(1))+ _z_offset];
z[((nn-7)-(1))+ _z_offset] = s;
}
t = (0.5*((((z[((nn-7)-(1))+ _z_offset]-z[((nn-3)-(1))+ _z_offset]))+z[((nn-5)-(1))+ _z_offset])));
if (((z[((nn-5)-(1))+ _z_offset] > (z[((nn-3)-(1))+ _z_offset]*tol2)) && (t != 0.0)))  {
    s = (z[((nn-3)-(1))+ _z_offset]*((z[((nn-5)-(1))+ _z_offset]/t)));
if ((s <= t))  {
    s = (z[((nn-3)-(1))+ _z_offset]*((z[((nn-5)-(1))+ _z_offset]/((t*((1.0+Math.sqrt((1.0+(s/t))))))))));
}
else  {
  s = (z[((nn-3)-(1))+ _z_offset]*((z[((nn-5)-(1))+ _z_offset]/((t+(Math.sqrt(t)*Math.sqrt((t+s))))))));
}              //  Close else.
t = (z[((nn-7)-(1))+ _z_offset]+((s+z[((nn-5)-(1))+ _z_offset])));
z[((nn-3)-(1))+ _z_offset] = (z[((nn-3)-(1))+ _z_offset]*((z[((nn-7)-(1))+ _z_offset]/t)));
z[((nn-7)-(1))+ _z_offset] = t;
}
z[(((4*n0.val)-7)-(1))+ _z_offset] = (z[((nn-7)-(1))+ _z_offset]+sigma.val);
z[(((4*n0.val)-3)-(1))+ _z_offset] = (z[((nn-3)-(1))+ _z_offset]+sigma.val);
n0.val = (n0.val-2);
// *
continue label10;}//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",10);
}//break label50;
   //Dummy.label("com/polytechnik/lapack310/Dlasq3",50);
/***
*
*     Reverse the qd-array, if warranted.
*
***/
if ((pp.val == 2)) {
    pp.val = 0;
}
    if (((dmin.val <= 0.0) || (n0.val < n0in)))  {
    if (((1.5*z[((((4*i0)+pp.val)-3)-(1))+ _z_offset]) < z[((((4*n0.val)+pp.val)-3)-(1))+ _z_offset]))  {
    ipn4 = (4*((i0+n0.val)));
{
int _j4_inc = 4;
for (j4 = (4*i0); j4 <= (2*(((i0+n0.val)-1))); j4 += _j4_inc) {
temp = z[((j4-3)-(1))+ _z_offset];
z[((j4-3)-(1))+ _z_offset] = z[(((ipn4-j4)-3)-(1))+ _z_offset];
z[(((ipn4-j4)-3)-(1))+ _z_offset] = temp;
temp = z[((j4-2)-(1))+ _z_offset];
z[((j4-2)-(1))+ _z_offset] = z[(((ipn4-j4)-2)-(1))+ _z_offset];
z[(((ipn4-j4)-2)-(1))+ _z_offset] = temp;
temp = z[((j4-1)-(1))+ _z_offset];
z[((j4-1)-(1))+ _z_offset] = z[(((ipn4-j4)-5)-(1))+ _z_offset];
z[(((ipn4-j4)-5)-(1))+ _z_offset] = temp;
temp = z[(j4-(1))+ _z_offset];
z[(j4-(1))+ _z_offset] = z[(((ipn4-j4)-4)-(1))+ _z_offset];
z[(((ipn4-j4)-4)-(1))+ _z_offset] = temp;
//Dummy.label("com/polytechnik/lapack310/Dlasq3",60);
}              //  Close for() loop.
}
if (((n0.val-i0) <= 4))  {
    z[((((4*n0.val)+pp.val)-1)-(1))+ _z_offset] = z[((((4*i0)+pp.val)-1)-(1))+ _z_offset];
z[(((4*n0.val)-pp.val)-(1))+ _z_offset] = z[(((4*i0)-pp.val)-(1))+ _z_offset];
}
dmin2.val = Math.min(dmin2.val, z[((((4*n0.val)+pp.val)-1)-(1))+ _z_offset]);
z[((((4*n0.val)+pp.val)-1)-(1))+ _z_offset] = Util.min(z[((((4*n0.val)+pp.val)-1)-(1))+ _z_offset], z[((((4*i0)+pp.val)-1)-(1))+ _z_offset], z[((((4*i0)+pp.val)+3)-(1))+ _z_offset]);
z[(((4*n0.val)-pp.val)-(1))+ _z_offset] = Util.min(z[(((4*n0.val)-pp.val)-(1))+ _z_offset], z[(((4*i0)-pp.val)-(1))+ _z_offset], z[((((4*i0)-pp.val)+4)-(1))+ _z_offset]);
qmax.val = Util.max(qmax.val, z[((((4*i0)+pp.val)-3)-(1))+ _z_offset], z[((((4*i0)+pp.val)+1)-(1))+ _z_offset]);
dmin.val = -0.0;
}
/***
*
*     Choose a shift.
*
***/
}
/***
*
*     Call dqds until DMIN > 0.
*
***/
com.polytechnik.lapack310.Dlasq4.dlasq4(i0,n0.val,z,_z_offset,pp.val,n0in,dmin.val,dmin1.val,dmin2.val,dn.val,dn1.val,dn2.val,tau,ttype,g);
// *
label90:{
label70:for(;;){
   //Dummy.label("com/polytechnik/lapack310/Dlasq3",70);
// *
com.polytechnik.lapack310.Dlasq5.dlasq5(i0,n0.val,z,_z_offset,pp.val,tau,sigma.val,dmin,dmin1,dmin2,dn,dn1,dn2,ieee,eps);
ndiv.val = (ndiv.val+(((n0.val-i0)+2)));
/***
*
*     Check status.
*
***/
iter.val = (iter.val+1);
/***
*
*        Success.
*
***/
if (((dmin.val >= 0.0) && (dmin1.val >= 0.0)))  {
    // *
break label90;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",90);
/***
*
*        Convergence hidden by negative DN.
*
***/
}
else if (((((dmin.val < 0.0) && (dmin1.val > 0.0)) && (z[(((4*((n0.val-1)))-pp.val)-(1))+ _z_offset] < (tol*((sigma.val+dn1.val))))) && (Math.abs(dn.val) < (tol*sigma.val))))  {
    z[((((4*((n0.val-1)))-pp.val)+2)-(1))+ _z_offset] = 0.0;
dmin.val = 0.0;
break label90;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",90);
/***
*
*        TAU too big. Select new TAU and try again.
*
***/
}              // Close else if()
else if ((dmin.val < 0.0))  {
    nfail.val = (nfail.val+1);
/***
*
*           Failed twice. Play it safe.
*
***/
if ((ttype.val < -22))  {
    tau.val = 0.0;
/***
*
*           Late failure. Gives excellent shift.
*
***/
}
else if ((dmin1.val > 0.0))  {
    tau.val = (((tau.val+dmin.val))*((1.0-(2.0*eps))));
ttype.val = (ttype.val-11);
/***
*
*           Early failure. Divide by 4.
*
***/
}              // Close else if()
else  {
  tau.val = (0.25*tau.val);
ttype.val = (ttype.val-12);
}              //  Close else.
continue label70;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",70);
/***
*
*        NaN.
*
***/
}              // Close else if()
else if (com.polytechnik.lapack310.Disnan.disnan(dmin.val))  {
    if ((tau.val == 0.0))  {
    break label70;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",80);
}
else  {
  tau.val = 0.0;
continue label70;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",70);
}              //  Close else.
/***
*
*        Possible underflow. Play it safe.
*
***/
}              // Close else if()
else  {
  break label70;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",80);
/***
*
*     Risk of underflow.
*
***/
}              //  Close else.
}//break label70;
//label80:
//   Dummy.label("com/polytechnik/lapack310/Dlasq3",80);
com.polytechnik.lapack310.Dlasq6.dlasq6(i0,n0.val,z,_z_offset,pp.val,dmin,dmin1,dmin2,dn,dn1,dn2);
ndiv.val = (ndiv.val+(((n0.val-i0)+2)));
iter.val = (iter.val+1);
// *
tau.val = 0.0;
break label90;}
   //Dummy.label("com/polytechnik/lapack310/Dlasq3",90);
if ((tau.val < sigma.val))  {
    desig.val = (desig.val+tau.val);
t = (sigma.val+desig.val);
desig.val = (desig.val-((t-sigma.val)));
}
else  {
  t = (sigma.val+tau.val);
desig.val = ((sigma.val-((t-tau.val)))+desig.val);
}              //  Close else.
// *
sigma.val = t;
/***
*
*     End of DLASQ3
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq3",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlasq3",999999);
return;
   }
} // End class.
