/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlasq4.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlasq4 {

// *> \brief \b DLASQ4 computes an approximation to the smallest eigenvalue
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLASQ4 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASQ4( I0, N0, Z, PP, N0IN, DMIN, DMIN1, DMIN2, DN,

// *                          DN1, DN2, TAU, TTYPE, G )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            I0, N0, N0IN, PP, TTYPE
// *       DOUBLE PRECISION   DMIN, DMIN1, DMIN2, DN, DN1, DN2, G, TAU
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   Z( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASQ4 computes an approximation TAU to the smallest eigenvalue
// *> using values of d from the previous transform.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] I0
// *> \verbatim
// *>          I0 is INTEGER
// *>        First index.
// *> \endverbatim
// *>
// *> \param[in] N0
// *> \verbatim
// *>          N0 is INTEGER
// *>        Last index.
// *> \endverbatim
// *>
// *> \param[in] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension ( 4*N0 )
// *>        Z holds the qd array.
// *> \endverbatim
// *>
// *> \param[in] PP
// *> \verbatim
// *>          PP is INTEGER
// *>        PP=0 for ping, PP=1 for pong.
// *> \endverbatim
// *>
// *> \param[in] N0IN
// *> \verbatim
// *>          N0IN is INTEGER
// *>        The value of N0 at start of EIGTEST.
// *> \endverbatim
// *>
// *> \param[in] DMIN
// *> \verbatim
// *>          DMIN is DOUBLE PRECISION
// *>        Minimum value of d.
// *> \endverbatim
// *>
// *> \param[in] DMIN1
// *> \verbatim
// *>          DMIN1 is DOUBLE PRECISION
// *>        Minimum value of d, excluding D( N0 ).
// *> \endverbatim
// *>
// *> \param[in] DMIN2
// *> \verbatim
// *>          DMIN2 is DOUBLE PRECISION
// *>        Minimum value of d, excluding D( N0 ) and D( N0-1 ).
// *> \endverbatim
// *>
// *> \param[in] DN
// *> \verbatim
// *>          DN is DOUBLE PRECISION
// *>        d(N)
// *> \endverbatim
// *>
// *> \param[in] DN1
// *> \verbatim
// *>          DN1 is DOUBLE PRECISION
// *>        d(N-1)
// *> \endverbatim
// *>
// *> \param[in] DN2
// *> \verbatim
// *>          DN2 is DOUBLE PRECISION
// *>        d(N-2)
// *> \endverbatim
// *>
// *> \param[out] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION
// *>        This is the shift.
// *> \endverbatim
// *>
// *> \param[out] TTYPE
// *> \verbatim
// *>          TTYPE is INTEGER
// *>        Shift type.
// *> \endverbatim
// *>
// *> \param[in,out] G
// *> \verbatim
// *>          G is DOUBLE PRECISION
// *>        G is passed as an argument in order to save its value between

// *>        calls to DLASQ4.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  CNST1 = 9/16
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     A negative DMIN forces the shift to take that absolute value
*     TTYPE records the type of shift.
*
***/

public static void dlasq4 (int i0,
int n0,
double [] z, int _z_offset,
int pp,
int n0in,
double dmin,
double dmin1,
double dmin2,
double dn,
double dn1,
double dn2,
doubleW tau,
intW ttype,
doubleW g)  {

label_999999:{
final double CNST1 = 0.5630E0, CNST2 = 1.010E0, CNST3 = 1.050E0,THIRD = 0.3330E0;


int i4= 0;
int nn= 0;
int np= 0;
double a2= 0.0d;
double b1= 0.0d;
double b2= 0.0d;
double gam= 0.0d;
double gap1= 0.0d;
double gap2= 0.0d;
double s= 0.0d;
if ((dmin <= 0.0))  {
    tau.val = (-(dmin));
ttype.val = -1;
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
// *
}
nn = ((4*n0)+pp);
/***
*
*        No eigenvalues deflated.
*
***/
if ((n0in == n0))  {
    // *
if (((dmin == dn) || (dmin == dn1)))  {
    b1 = (Math.sqrt(z[((nn-3)-(1))+ _z_offset])*Math.sqrt(z[((nn-5)-(1))+ _z_offset]));
b2 = (Math.sqrt(z[((nn-7)-(1))+ _z_offset])*Math.sqrt(z[((nn-9)-(1))+ _z_offset]));
/***
*
*           Cases 2 and 3.
*
***/
a2 = (z[((nn-7)-(1))+ _z_offset]+z[((nn-5)-(1))+ _z_offset]);
if (((dmin == dn) && (dmin1 == dn1)))  {
    gap2 = ((dmin2-a2)-(dmin2*0.25));
if (((gap2 > 0.0) && (gap2 > b2)))  {
    gap1 = ((a2-dn)-(((b2/gap2))*b2));
}
else  {
  gap1 = ((a2-dn)-((b1+b2)));
}              //  Close else.
if (((gap1 > 0.0) && (gap1 > b1)))  {
    s = Math.max((dn-(((b1/gap1))*b1)), (0.5*dmin));
ttype.val = -2;
}
else  {
  s = 0.0;
if ((dn > b1)) {
    s = (dn-b1);
}
    if ((a2 > ((b1+b2)))) {
    s = Math.min(s, (a2-((b1+b2))));
}
    s = Math.max(s, (THIRD*dmin));
ttype.val = -3;
}              //  Close else.
/***
*
*              Case 4.
*
***/
}
else  {
  ttype.val = -4;
s = (0.25*dmin);
if ((dmin == dn))  {
    gam = dn;
a2 = 0.0;
if ((z[((nn-5)-(1))+ _z_offset] > z[((nn-7)-(1))+ _z_offset])) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
}
    b2 = (z[((nn-5)-(1))+ _z_offset]/z[((nn-7)-(1))+ _z_offset]);
np = (nn-9);
}
else  {
  np = (nn-(2*pp));
gam = dn1;
if ((z[((np-4)-(1))+ _z_offset] > z[((np-2)-(1))+ _z_offset])) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
}
    a2 = (z[((np-4)-(1))+ _z_offset]/z[((np-2)-(1))+ _z_offset]);
if ((z[((nn-9)-(1))+ _z_offset] > z[((nn-11)-(1))+ _z_offset])) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
}
    b2 = (z[((nn-9)-(1))+ _z_offset]/z[((nn-11)-(1))+ _z_offset]);
np = (nn-13);
/***
*
*              Approximate contribution to norm squared from I < NN-1.
*
***/
}              //  Close else.
a2 = (a2+b2);
{
int _i4_inc = -4;
label10:for (i4 = np; i4 >= (((4*i0)-1)+pp); i4 += _i4_inc) {
if ((b2 == 0.0)) {
    break label10;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",20);
}
    b1 = b2;
if ((z[(i4-(1))+ _z_offset] > z[((i4-2)-(1))+ _z_offset])) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
}
    b2 = (b2*((z[(i4-(1))+ _z_offset]/z[((i4-2)-(1))+ _z_offset])));
a2 = (a2+b2);
if ((((100.0*Math.max(b2, b1)) < a2) || (CNST1 < a2))) {
    break label10;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",20);
}
//Dummy.label("com/polytechnik/lapack310/Dlasq4",10);
}              //  Close for() loop.
}
//label20:
//   Dummy.label("com/polytechnik/lapack310/Dlasq4",20);
/***
*
*              Rayleigh quotient residual bound.
*
***/
a2 = (CNST3*a2);
if ((a2 < CNST1)) {
    s = ((gam*((1.0-Math.sqrt(a2))))/((1.0+a2)));
}
    }              //  Close else.
/***
*
*           Case 5.
*
***/
}
else if ((dmin == dn2))  {
    ttype.val = -5;
/***
*
*           Compute contribution to norm squared from I > NN-2.
*
***/
s = (0.25*dmin);
np = (nn-(2*pp));
b1 = z[((np-2)-(1))+ _z_offset];
b2 = z[((np-6)-(1))+ _z_offset];
gam = dn2;
if (((z[((np-8)-(1))+ _z_offset] > b2) || (z[((np-4)-(1))+ _z_offset] > b1))) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
}
    /***
*
*           Approximate contribution to norm squared from I < NN-2.
*
***/
a2 = (((z[((np-8)-(1))+ _z_offset]/b2))*((1.0+(z[((np-4)-(1))+ _z_offset]/b1))));
if (((n0-i0) > 2))  {
    b2 = (z[((nn-13)-(1))+ _z_offset]/z[((nn-15)-(1))+ _z_offset]);
a2 = (a2+b2);
{
int _i4_inc = -4;
label30:for (i4 = (nn-17); i4 >= (((4*i0)-1)+pp); i4 += _i4_inc) {
if ((b2 == 0.0)) {
    break label30;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",40);
}
    b1 = b2;
if ((z[(i4-(1))+ _z_offset] > z[((i4-2)-(1))+ _z_offset])) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
}
    b2 = (b2*((z[(i4-(1))+ _z_offset]/z[((i4-2)-(1))+ _z_offset])));
a2 = (a2+b2);
if ((((100.0*Math.max(b2, b1)) < a2) || (CNST1 < a2))) {
    break label30;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",40);
}
//Dummy.label("com/polytechnik/lapack310/Dlasq4",30);
}              //  Close for() loop.
}
//label40:
//   Dummy.label("com/polytechnik/lapack310/Dlasq4",40);
a2 = (CNST3*a2);
// *
}
if ((a2 < CNST1)) {
    s = ((gam*((1.0-Math.sqrt(a2))))/((1.0+a2)));
}
    /***
*
*           Case 6, no information to guide us.
*
***/
}              // Close else if()
else  {
  if ((ttype.val == -6))  {
    g.val = (g.val+(THIRD*((1.0-g.val))));
}
else if ((ttype.val == -18))  {
    g.val = (0.25*THIRD);
}              // Close else if()
else  {
  g.val = 0.25;
}              //  Close else.
s = (g.val*dmin);
ttype.val = -6;
// *
}              //  Close else.
/***
*
*        One eigenvalue just deflated. Use DMIN1, DN1 for DMIN and DN.
*
***/
}
else if ((n0in == ((n0+1))))  {
    /***
*
*           Cases 7 and 8.
*
***/
if (((dmin1 == dn1) && (dmin2 == dn2)))  {
    ttype.val = -7;
s = (THIRD*dmin1);
if ((z[((nn-5)-(1))+ _z_offset] > z[((nn-7)-(1))+ _z_offset])) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
}
    b1 = (z[((nn-5)-(1))+ _z_offset]/z[((nn-7)-(1))+ _z_offset]);
b2 = b1;
if ((b2 == 0.0)) {
    //Dummy.go_to("com/polytechnik/lapack310/Dlasq4",60);
} else
    {
int _i4_inc = -4;
label50:for (i4 = (((4*n0)-9)+pp); i4 >= (((4*i0)-1)+pp); i4 += _i4_inc) {
a2 = b1;
if ((z[(i4-(1))+ _z_offset] > z[((i4-2)-(1))+ _z_offset])) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
}
    b1 = (b1*((z[(i4-(1))+ _z_offset]/z[((i4-2)-(1))+ _z_offset])));
b2 = (b2+b1);
if (((100.0*Math.max(b1, a2)) < b2)) {
    break label50;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",60);
}
//Dummy.label("com/polytechnik/lapack310/Dlasq4",50);
}              //  Close for() loop.
}
//label60:
//   Dummy.label("com/polytechnik/lapack310/Dlasq4",60);
b2 = Math.sqrt((CNST3*b2));
a2 = (dmin1/((1.0+( Math.pow(b2, 2)))));
gap2 = ((0.5*dmin2)-a2);
if (((gap2 > 0.0) && (gap2 > (b2*a2))))  {
    s = Math.max(s, (a2*((1.0-(((CNST2*a2)*((b2/gap2)))*b2)))));
}
else  {
  s = Math.max(s, (a2*((1.0-(CNST2*b2)))));
ttype.val = -8;
}              //  Close else.
/***
*
*           Case 9.
*
***/
}
else  {
  s = (0.25*dmin1);
if ((dmin1 == dn1)) {
    s = (0.5*dmin1);
}
    ttype.val = -9;
// *
}              //  Close else.
/***
*
*        Two eigenvalues deflated. Use DMIN2, DN2 for DMIN and DN.
*
*        Cases 10 and 11.
*
***/
}              // Close else if()
else if ((n0in == ((n0+2))))  {
    if (((dmin2 == dn2) && ((2.0*z[((nn-5)-(1))+ _z_offset]) < z[((nn-7)-(1))+ _z_offset])))  {
    ttype.val = -10;
s = (THIRD*dmin2);
if ((z[((nn-5)-(1))+ _z_offset] > z[((nn-7)-(1))+ _z_offset])) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
}
    b1 = (z[((nn-5)-(1))+ _z_offset]/z[((nn-7)-(1))+ _z_offset]);
b2 = b1;
if ((b2 == 0.0)) {
    //Dummy.go_to("com/polytechnik/lapack310/Dlasq4",80);
} else
    {
int _i4_inc = -4;
label70:for (i4 = (((4*n0)-9)+pp); i4 >= (((4*i0)-1)+pp); i4 += _i4_inc) {
if ((z[(i4-(1))+ _z_offset] > z[((i4-2)-(1))+ _z_offset])) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
}
    b1 = (b1*((z[(i4-(1))+ _z_offset]/z[((i4-2)-(1))+ _z_offset])));
b2 = (b2+b1);
if (((100.0*b1) < b2)) {
    break label70;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",80);
}
//Dummy.label("com/polytechnik/lapack310/Dlasq4",70);
}              //  Close for() loop.
}
//label80:
//   Dummy.label("com/polytechnik/lapack310/Dlasq4",80);
b2 = Math.sqrt((CNST3*b2));
a2 = (dmin2/((1.0+( Math.pow(b2, 2)))));
gap2 = (((z[((nn-7)-(1))+ _z_offset]+z[((nn-9)-(1))+ _z_offset])-(Math.sqrt(z[((nn-11)-(1))+ _z_offset])*Math.sqrt(z[((nn-9)-(1))+ _z_offset])))-a2);
if (((gap2 > 0.0) && (gap2 > (b2*a2))))  {
    s = Math.max(s, (a2*((1.0-(((CNST2*a2)*((b2/gap2)))*b2)))));
}
else  {
  s = Math.max(s, (a2*((1.0-(CNST2*b2)))));
}              //  Close else.
}
else  {
  s = (0.25*dmin2);
ttype.val = -11;
}              //  Close else.
/***
*
*        Case 12, more than two eigenvalues deflated. No information.
*
***/
}              // Close else if()
else if ((n0in > ((n0+2))))  {
    s = 0.0;
ttype.val = -12;
// *
}              // Close else if()
tau.val = s;
/***
*
*     End of DLASQ4
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq4",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlasq4",999999);
return;
   }
} // End class.
