/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlasq5.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlasq5 {

// *> \brief \b DLASQ5 computes one dqds transform in ping-pong form. Used
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLASQ5 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASQ5( I0, N0, Z, PP, TAU, SIGMA, DMIN, DMIN1, DMIN2
// *                          DNM1, DNM2, IEEE, EPS )
// *
// *       .. Scalar Arguments ..
// *       LOGICAL            IEEE
// *       INTEGER            I0, N0, PP
// *       DOUBLE PRECISION   DMIN, DMIN1, DMIN2, DN, DNM1, DNM2, TAU, SIGM
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   Z( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASQ5 computes one dqds transform in ping-pong form, one
// *> version for IEEE machines another for non IEEE machines.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] I0
// *> \verbatim
// *>          I0 is INTEGER
// *>        First index.
// *> \endverbatim
// *>
// *> \param[in] N0
// *> \verbatim
// *>          N0 is INTEGER
// *>        Last index.
// *> \endverbatim
// *>
// *> \param[in] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension ( 4*N )
// *>        Z holds the qd array. EMIN is stored in Z(4*N0) to avoid
// *>        an extra argument.
// *> \endverbatim
// *>
// *> \param[in] PP
// *> \verbatim
// *>          PP is INTEGER
// *>        PP=0 for ping, PP=1 for pong.
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION
// *>        This is the shift.
// *> \endverbatim
// *>
// *> \param[in] SIGMA
// *> \verbatim
// *>          SIGMA is DOUBLE PRECISION
// *>        This is the accumulated shift up to this step.
// *> \endverbatim
// *>
// *> \param[out] DMIN
// *> \verbatim
// *>          DMIN is DOUBLE PRECISION
// *>        Minimum value of d.
// *> \endverbatim
// *>
// *> \param[out] DMIN1
// *> \verbatim
// *>          DMIN1 is DOUBLE PRECISION
// *>        Minimum value of d, excluding D( N0 ).
// *> \endverbatim
// *>
// *> \param[out] DMIN2
// *> \verbatim
// *>          DMIN2 is DOUBLE PRECISION
// *>        Minimum value of d, excluding D( N0 ) and D( N0-1 ).
// *> \endverbatim
// *>
// *> \param[out] DN
// *> \verbatim
// *>          DN is DOUBLE PRECISION
// *>        d(N0), the last value of d.
// *> \endverbatim
// *>
// *> \param[out] DNM1
// *> \verbatim
// *>          DNM1 is DOUBLE PRECISION
// *>        d(N0-1).
// *> \endverbatim
// *>
// *> \param[out] DNM2
// *> \verbatim
// *>          DNM2 is DOUBLE PRECISION
// *>        d(N0-2).
// *> \endverbatim
// *>
// *> \param[in] IEEE
// *> \verbatim
// *>          IEEE is LOGICAL
// *>        Flag for IEEE or non IEEE arithmetic.
// *> \endverbatim
// *>
// *> \param[in] EPS
// *> \verbatim
// *>          EPS is DOUBLE PRECISION
// *>        This is the value of epsilon used.
// *> \endverbatim
// *>
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameter ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/
// *

public static void dlasq5 (int i0,
int n0,
double [] z, int _z_offset,
int pp,
doubleW tau,
double sigma,
doubleW dmin,
doubleW dmin1,
doubleW dmin2,
doubleW dn,
doubleW dnm1,
doubleW dnm2,
boolean ieee,
double eps)  {

label_999999:{

int j4= 0;
int j4p2= 0;
double d= 0.0d;
double emin= 0.0d;
double temp= 0.0d;
double dthresh= 0.0d;
if (((((n0-i0)-1)) <= 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq5",999999);
}
    dthresh = (eps*((sigma+tau.val)));
if ((tau.val < (dthresh*0.5))) {
    tau.val = 0.0;
}
    if ((tau.val != 0.0))  {
    j4 = (((4*i0)+pp)-3);
emin = z[((j4+4)-(1))+ _z_offset];
d = (z[(j4-(1))+ _z_offset]-tau.val);
dmin.val = d;
// *
dmin1.val = (-(z[(j4-(1))+ _z_offset]));
/***
*
*        Code for IEEE arithmetic.
*
***/
if (ieee)  {
    if ((pp == 0))  {
    {
int _j4_inc = 4;
for (j4 = (4*i0); j4 <= (4*((n0-3))); j4 += _j4_inc) {
z[((j4-2)-(1))+ _z_offset] = (d+z[((j4-1)-(1))+ _z_offset]);
temp = (z[((j4+1)-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset]);
d = ((d*temp)-tau.val);
dmin.val = Math.min(dmin.val, d);
z[(j4-(1))+ _z_offset] = (z[((j4-1)-(1))+ _z_offset]*temp);
emin = Math.min(z[(j4-(1))+ _z_offset], emin);
//Dummy.label("com/polytechnik/lapack310/Dlasq5",10);
}              //  Close for() loop.
}
}
else  {
  {
int _j4_inc = 4;
for (j4 = (4*i0); j4 <= (4*((n0-3))); j4 += _j4_inc) {
z[((j4-3)-(1))+ _z_offset] = (d+z[(j4-(1))+ _z_offset]);
temp = (z[((j4+2)-(1))+ _z_offset]/z[((j4-3)-(1))+ _z_offset]);
d = ((d*temp)-tau.val);
dmin.val = Math.min(dmin.val, d);
z[((j4-1)-(1))+ _z_offset] = (z[(j4-(1))+ _z_offset]*temp);
emin = Math.min(z[((j4-1)-(1))+ _z_offset], emin);
//Dummy.label("com/polytechnik/lapack310/Dlasq5",20);
}              //  Close for() loop.
}
/***
*
*        Unroll last two steps.
*
***/
}              //  Close else.
dnm2.val = d;
dmin2.val = dmin.val;
j4 = ((4*((n0-2)))-pp);
j4p2 = ((j4+(2*pp))-1);
z[((j4-2)-(1))+ _z_offset] = (dnm2.val+z[(j4p2-(1))+ _z_offset]);
z[(j4-(1))+ _z_offset] = (z[((j4p2+2)-(1))+ _z_offset]*((z[(j4p2-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
dnm1.val = ((z[((j4p2+2)-(1))+ _z_offset]*((dnm2.val/z[((j4-2)-(1))+ _z_offset])))-tau.val);
// *
dmin.val = Math.min(dmin.val, dnm1.val);
dmin1.val = dmin.val;
j4 = (j4+4);
j4p2 = ((j4+(2*pp))-1);
z[((j4-2)-(1))+ _z_offset] = (dnm1.val+z[(j4p2-(1))+ _z_offset]);
z[(j4-(1))+ _z_offset] = (z[((j4p2+2)-(1))+ _z_offset]*((z[(j4p2-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
dn.val = ((z[((j4p2+2)-(1))+ _z_offset]*((dnm1.val/z[((j4-2)-(1))+ _z_offset])))-tau.val);
// *
dmin.val = Math.min(dmin.val, dn.val);
/***
*
*        Code for non IEEE arithmetic.
*
***/
}
else  {
  if ((pp == 0))  {
    {
int _j4_inc = 4;
for (j4 = (4*i0); j4 <= (4*((n0-3))); j4 += _j4_inc) {
z[((j4-2)-(1))+ _z_offset] = (d+z[((j4-1)-(1))+ _z_offset]);
if ((d < 0.0))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq5",999999);
}
else  {
  z[(j4-(1))+ _z_offset] = (z[((j4+1)-(1))+ _z_offset]*((z[((j4-1)-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
d = ((z[((j4+1)-(1))+ _z_offset]*((d/z[((j4-2)-(1))+ _z_offset])))-tau.val);
}              //  Close else.
dmin.val = Math.min(dmin.val, d);
emin = Math.min(emin, z[(j4-(1))+ _z_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlasq5",30);
}              //  Close for() loop.
}
}
else  {
  {
int _j4_inc = 4;
for (j4 = (4*i0); j4 <= (4*((n0-3))); j4 += _j4_inc) {
z[((j4-3)-(1))+ _z_offset] = (d+z[(j4-(1))+ _z_offset]);
if ((d < 0.0))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq5",999999);
}
else  {
  z[((j4-1)-(1))+ _z_offset] = (z[((j4+2)-(1))+ _z_offset]*((z[(j4-(1))+ _z_offset]/z[((j4-3)-(1))+ _z_offset])));
d = ((z[((j4+2)-(1))+ _z_offset]*((d/z[((j4-3)-(1))+ _z_offset])))-tau.val);
}              //  Close else.
dmin.val = Math.min(dmin.val, d);
emin = Math.min(emin, z[((j4-1)-(1))+ _z_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlasq5",40);
}              //  Close for() loop.
}
/***
*
*        Unroll last two steps.
*
***/
}              //  Close else.
dnm2.val = d;
dmin2.val = dmin.val;
j4 = ((4*((n0-2)))-pp);
j4p2 = ((j4+(2*pp))-1);
z[((j4-2)-(1))+ _z_offset] = (dnm2.val+z[(j4p2-(1))+ _z_offset]);
if ((dnm2.val < 0.0))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq5",999999);
}
else  {
  z[(j4-(1))+ _z_offset] = (z[((j4p2+2)-(1))+ _z_offset]*((z[(j4p2-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
dnm1.val = ((z[((j4p2+2)-(1))+ _z_offset]*((dnm2.val/z[((j4-2)-(1))+ _z_offset])))-tau.val);
}              //  Close else.
// *
dmin.val = Math.min(dmin.val, dnm1.val);
dmin1.val = dmin.val;
j4 = (j4+4);
j4p2 = ((j4+(2*pp))-1);
z[((j4-2)-(1))+ _z_offset] = (dnm1.val+z[(j4p2-(1))+ _z_offset]);
if ((dnm1.val < 0.0))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq5",999999);
}
else  {
  z[(j4-(1))+ _z_offset] = (z[((j4p2+2)-(1))+ _z_offset]*((z[(j4p2-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
dn.val = ((z[((j4p2+2)-(1))+ _z_offset]*((dnm1.val/z[((j4-2)-(1))+ _z_offset])))-tau.val);
}              //  Close else.
// *
dmin.val = Math.min(dmin.val, dn.val);
}              //  Close else.
// *     This is the version that sets d's to zero if they are small enough
}
else  {
  j4 = (((4*i0)+pp)-3);
emin = z[((j4+4)-(1))+ _z_offset];
d = (z[(j4-(1))+ _z_offset]-tau.val);
dmin.val = d;
dmin1.val = (-(z[(j4-(1))+ _z_offset]));
/***
*
*     Code for IEEE arithmetic.
*
***/
if (ieee)  {
    if ((pp == 0))  {
    {
int _j4_inc = 4;
for (j4 = (4*i0); j4 <= (4*((n0-3))); j4 += _j4_inc) {
z[((j4-2)-(1))+ _z_offset] = (d+z[((j4-1)-(1))+ _z_offset]);
temp = (z[((j4+1)-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset]);
d = ((d*temp)-tau.val);
if ((d < dthresh)) {
    d = 0.0;
}
    dmin.val = Math.min(dmin.val, d);
z[(j4-(1))+ _z_offset] = (z[((j4-1)-(1))+ _z_offset]*temp);
emin = Math.min(z[(j4-(1))+ _z_offset], emin);
//Dummy.label("com/polytechnik/lapack310/Dlasq5",50);
}              //  Close for() loop.
}
}
else  {
  {
int _j4_inc = 4;
for (j4 = (4*i0); j4 <= (4*((n0-3))); j4 += _j4_inc) {
z[((j4-3)-(1))+ _z_offset] = (d+z[(j4-(1))+ _z_offset]);
temp = (z[((j4+2)-(1))+ _z_offset]/z[((j4-3)-(1))+ _z_offset]);
d = ((d*temp)-tau.val);
if ((d < dthresh)) {
    d = 0.0;
}
    dmin.val = Math.min(dmin.val, d);
z[((j4-1)-(1))+ _z_offset] = (z[(j4-(1))+ _z_offset]*temp);
emin = Math.min(z[((j4-1)-(1))+ _z_offset], emin);
//Dummy.label("com/polytechnik/lapack310/Dlasq5",60);
}              //  Close for() loop.
}
/***
*
*     Unroll last two steps.
*
***/
}              //  Close else.
dnm2.val = d;
dmin2.val = dmin.val;
j4 = ((4*((n0-2)))-pp);
j4p2 = ((j4+(2*pp))-1);
z[((j4-2)-(1))+ _z_offset] = (dnm2.val+z[(j4p2-(1))+ _z_offset]);
z[(j4-(1))+ _z_offset] = (z[((j4p2+2)-(1))+ _z_offset]*((z[(j4p2-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
dnm1.val = ((z[((j4p2+2)-(1))+ _z_offset]*((dnm2.val/z[((j4-2)-(1))+ _z_offset])))-tau.val);
// *
dmin.val = Math.min(dmin.val, dnm1.val);
dmin1.val = dmin.val;
j4 = (j4+4);
j4p2 = ((j4+(2*pp))-1);
z[((j4-2)-(1))+ _z_offset] = (dnm1.val+z[(j4p2-(1))+ _z_offset]);
z[(j4-(1))+ _z_offset] = (z[((j4p2+2)-(1))+ _z_offset]*((z[(j4p2-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
dn.val = ((z[((j4p2+2)-(1))+ _z_offset]*((dnm1.val/z[((j4-2)-(1))+ _z_offset])))-tau.val);
// *
dmin.val = Math.min(dmin.val, dn.val);
/***
*
*     Code for non IEEE arithmetic.
*
***/
}
else  {
  if ((pp == 0))  {
    {
int _j4_inc = 4;
for (j4 = (4*i0); j4 <= (4*((n0-3))); j4 += _j4_inc) {
z[((j4-2)-(1))+ _z_offset] = (d+z[((j4-1)-(1))+ _z_offset]);
if ((d < 0.0))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq5",999999);
}
else  {
  z[(j4-(1))+ _z_offset] = (z[((j4+1)-(1))+ _z_offset]*((z[((j4-1)-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
d = ((z[((j4+1)-(1))+ _z_offset]*((d/z[((j4-2)-(1))+ _z_offset])))-tau.val);
}              //  Close else.
if ((d < dthresh)) {
    d = 0.0;
}
    dmin.val = Math.min(dmin.val, d);
emin = Math.min(emin, z[(j4-(1))+ _z_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlasq5",70);
}              //  Close for() loop.
}
}
else  {
  {
int _j4_inc = 4;
for (j4 = (4*i0); j4 <= (4*((n0-3))); j4 += _j4_inc) {
z[((j4-3)-(1))+ _z_offset] = (d+z[(j4-(1))+ _z_offset]);
if ((d < 0.0))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq5",999999);
}
else  {
  z[((j4-1)-(1))+ _z_offset] = (z[((j4+2)-(1))+ _z_offset]*((z[(j4-(1))+ _z_offset]/z[((j4-3)-(1))+ _z_offset])));
d = ((z[((j4+2)-(1))+ _z_offset]*((d/z[((j4-3)-(1))+ _z_offset])))-tau.val);
}              //  Close else.
if ((d < dthresh)) {
    d = 0.0;
}
    dmin.val = Math.min(dmin.val, d);
emin = Math.min(emin, z[((j4-1)-(1))+ _z_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlasq5",80);
}              //  Close for() loop.
}
/***
*
*     Unroll last two steps.
*
***/
}              //  Close else.
dnm2.val = d;
dmin2.val = dmin.val;
j4 = ((4*((n0-2)))-pp);
j4p2 = ((j4+(2*pp))-1);
z[((j4-2)-(1))+ _z_offset] = (dnm2.val+z[(j4p2-(1))+ _z_offset]);
if ((dnm2.val < 0.0))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq5",999999);
}
else  {
  z[(j4-(1))+ _z_offset] = (z[((j4p2+2)-(1))+ _z_offset]*((z[(j4p2-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
dnm1.val = ((z[((j4p2+2)-(1))+ _z_offset]*((dnm2.val/z[((j4-2)-(1))+ _z_offset])))-tau.val);
}              //  Close else.
// *
dmin.val = Math.min(dmin.val, dnm1.val);
dmin1.val = dmin.val;
j4 = (j4+4);
j4p2 = ((j4+(2*pp))-1);
z[((j4-2)-(1))+ _z_offset] = (dnm1.val+z[(j4p2-(1))+ _z_offset]);
if ((dnm1.val < 0.0))  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq5",999999);
}
else  {
  z[(j4-(1))+ _z_offset] = (z[((j4p2+2)-(1))+ _z_offset]*((z[(j4p2-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
dn.val = ((z[((j4p2+2)-(1))+ _z_offset]*((dnm1.val/z[((j4-2)-(1))+ _z_offset])))-tau.val);
}              //  Close else.
// *
dmin.val = Math.min(dmin.val, dn.val);
}              //  Close else.
// *
}              //  Close else.
z[((j4+2)-(1))+ _z_offset] = dn.val;
z[(((4*n0)-pp)-(1))+ _z_offset] = emin;
/***
*
*     End of DLASQ5
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq5",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlasq5",999999);
return;
   }
} // End class.
