/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlasq6.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlasq6 {

// *> \brief \b DLASQ6 computes one dqd transform in ping-pong form. Used b
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLASQ6 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASQ6( I0, N0, Z, PP, DMIN, DMIN1, DMIN2, DN,
// *                          DNM1, DNM2 )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            I0, N0, PP
// *       DOUBLE PRECISION   DMIN, DMIN1, DMIN2, DN, DNM1, DNM2
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   Z( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASQ6 computes one dqd (shift equal to zero) transform in
// *> ping-pong form, with protection against underflow and overflow.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] I0
// *> \verbatim
// *>          I0 is INTEGER
// *>        First index.
// *> \endverbatim
// *>
// *> \param[in] N0
// *> \verbatim
// *>          N0 is INTEGER
// *>        Last index.
// *> \endverbatim
// *>
// *> \param[in] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension ( 4*N )
// *>        Z holds the qd array. EMIN is stored in Z(4*N0) to avoid
// *>        an extra argument.
// *> \endverbatim
// *>
// *> \param[in] PP
// *> \verbatim
// *>          PP is INTEGER
// *>        PP=0 for ping, PP=1 for pong.
// *> \endverbatim
// *>
// *> \param[out] DMIN
// *> \verbatim
// *>          DMIN is DOUBLE PRECISION
// *>        Minimum value of d.
// *> \endverbatim
// *>
// *> \param[out] DMIN1
// *> \verbatim
// *>          DMIN1 is DOUBLE PRECISION
// *>        Minimum value of d, excluding D( N0 ).
// *> \endverbatim
// *>
// *> \param[out] DMIN2
// *> \verbatim
// *>          DMIN2 is DOUBLE PRECISION
// *>        Minimum value of d, excluding D( N0 ) and D( N0-1 ).
// *> \endverbatim
// *>
// *> \param[out] DN
// *> \verbatim
// *>          DN is DOUBLE PRECISION
// *>        d(N0), the last value of d.
// *> \endverbatim
// *>
// *> \param[out] DNM1
// *> \verbatim
// *>          DNM1 is DOUBLE PRECISION
// *>        d(N0-1).
// *> \endverbatim
// *>
// *> \param[out] DNM2
// *> \verbatim
// *>          DNM2 is DOUBLE PRECISION
// *>        d(N0-2).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameter ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Function ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/
// *

public static void dlasq6 (int i0,
int n0,
double [] z, int _z_offset,
int pp,
doubleW dmin,
doubleW dmin1,
doubleW dmin2,
doubleW dn,
doubleW dnm1,
doubleW dnm2)  {

label_999999:{

int j4= 0;
int j4p2= 0;
double d= 0.0d;
double emin= 0.0d;
double safmin= 0.0d;
double temp= 0.0d;
if (((((n0-i0)-1)) <= 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq6",999999);
}
    safmin = com.polytechnik.lapack310.Dlamch.dlamch("Safe minimum");
j4 = (((4*i0)+pp)-3);
emin = z[((j4+4)-(1))+ _z_offset];
d = z[(j4-(1))+ _z_offset];
// *
dmin.val = d;
if ((pp == 0))  {
    {
int _j4_inc = 4;
for (j4 = (4*i0); j4 <= (4*((n0-3))); j4 += _j4_inc) {
z[((j4-2)-(1))+ _z_offset] = (d+z[((j4-1)-(1))+ _z_offset]);
if ((z[((j4-2)-(1))+ _z_offset] == 0.0))  {
    z[(j4-(1))+ _z_offset] = 0.0;
d = z[((j4+1)-(1))+ _z_offset];
dmin.val = d;
emin = 0.0;
}
else if ((((safmin*z[((j4+1)-(1))+ _z_offset]) < z[((j4-2)-(1))+ _z_offset]) && ((safmin*z[((j4-2)-(1))+ _z_offset]) < z[((j4+1)-(1))+ _z_offset])))  {
    temp = (z[((j4+1)-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset]);
z[(j4-(1))+ _z_offset] = (z[((j4-1)-(1))+ _z_offset]*temp);
d = (d*temp);
}              // Close else if()
else  {
  z[(j4-(1))+ _z_offset] = (z[((j4+1)-(1))+ _z_offset]*((z[((j4-1)-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
d = (z[((j4+1)-(1))+ _z_offset]*((d/z[((j4-2)-(1))+ _z_offset])));
}              //  Close else.
dmin.val = Math.min(dmin.val, d);
emin = Math.min(emin, z[(j4-(1))+ _z_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlasq6",10);
}              //  Close for() loop.
}
}
else  {
  {
int _j4_inc = 4;
for (j4 = (4*i0); j4 <= (4*((n0-3))); j4 += _j4_inc) {
z[((j4-3)-(1))+ _z_offset] = (d+z[(j4-(1))+ _z_offset]);
if ((z[((j4-3)-(1))+ _z_offset] == 0.0))  {
    z[((j4-1)-(1))+ _z_offset] = 0.0;
d = z[((j4+2)-(1))+ _z_offset];
dmin.val = d;
emin = 0.0;
}
else if ((((safmin*z[((j4+2)-(1))+ _z_offset]) < z[((j4-3)-(1))+ _z_offset]) && ((safmin*z[((j4-3)-(1))+ _z_offset]) < z[((j4+2)-(1))+ _z_offset])))  {
    temp = (z[((j4+2)-(1))+ _z_offset]/z[((j4-3)-(1))+ _z_offset]);
z[((j4-1)-(1))+ _z_offset] = (z[(j4-(1))+ _z_offset]*temp);
d = (d*temp);
}              // Close else if()
else  {
  z[((j4-1)-(1))+ _z_offset] = (z[((j4+2)-(1))+ _z_offset]*((z[(j4-(1))+ _z_offset]/z[((j4-3)-(1))+ _z_offset])));
d = (z[((j4+2)-(1))+ _z_offset]*((d/z[((j4-3)-(1))+ _z_offset])));
}              //  Close else.
dmin.val = Math.min(dmin.val, d);
emin = Math.min(emin, z[((j4-1)-(1))+ _z_offset]);
//Dummy.label("com/polytechnik/lapack310/Dlasq6",20);
}              //  Close for() loop.
}
/***
*
*     Unroll last two steps.
*
***/
}              //  Close else.
dnm2.val = d;
dmin2.val = dmin.val;
j4 = ((4*((n0-2)))-pp);
j4p2 = ((j4+(2*pp))-1);
z[((j4-2)-(1))+ _z_offset] = (dnm2.val+z[(j4p2-(1))+ _z_offset]);
if ((z[((j4-2)-(1))+ _z_offset] == 0.0))  {
    z[(j4-(1))+ _z_offset] = 0.0;
dnm1.val = z[((j4p2+2)-(1))+ _z_offset];
dmin.val = dnm1.val;
emin = 0.0;
}
else if ((((safmin*z[((j4p2+2)-(1))+ _z_offset]) < z[((j4-2)-(1))+ _z_offset]) && ((safmin*z[((j4-2)-(1))+ _z_offset]) < z[((j4p2+2)-(1))+ _z_offset])))  {
    temp = (z[((j4p2+2)-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset]);
z[(j4-(1))+ _z_offset] = (z[(j4p2-(1))+ _z_offset]*temp);
dnm1.val = (dnm2.val*temp);
}              // Close else if()
else  {
  z[(j4-(1))+ _z_offset] = (z[((j4p2+2)-(1))+ _z_offset]*((z[(j4p2-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
dnm1.val = (z[((j4p2+2)-(1))+ _z_offset]*((dnm2.val/z[((j4-2)-(1))+ _z_offset])));
}              //  Close else.
// *
dmin.val = Math.min(dmin.val, dnm1.val);
dmin1.val = dmin.val;
j4 = (j4+4);
j4p2 = ((j4+(2*pp))-1);
z[((j4-2)-(1))+ _z_offset] = (dnm1.val+z[(j4p2-(1))+ _z_offset]);
if ((z[((j4-2)-(1))+ _z_offset] == 0.0))  {
    z[(j4-(1))+ _z_offset] = 0.0;
dn.val = z[((j4p2+2)-(1))+ _z_offset];
dmin.val = dn.val;
emin = 0.0;
}
else if ((((safmin*z[((j4p2+2)-(1))+ _z_offset]) < z[((j4-2)-(1))+ _z_offset]) && ((safmin*z[((j4-2)-(1))+ _z_offset]) < z[((j4p2+2)-(1))+ _z_offset])))  {
    temp = (z[((j4p2+2)-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset]);
z[(j4-(1))+ _z_offset] = (z[(j4p2-(1))+ _z_offset]*temp);
dn.val = (dnm1.val*temp);
}              // Close else if()
else  {
  z[(j4-(1))+ _z_offset] = (z[((j4p2+2)-(1))+ _z_offset]*((z[(j4p2-(1))+ _z_offset]/z[((j4-2)-(1))+ _z_offset])));
dn.val = (z[((j4p2+2)-(1))+ _z_offset]*((dnm1.val/z[((j4-2)-(1))+ _z_offset])));
}              //  Close else.
// *
dmin.val = Math.min(dmin.val, dn.val);
z[((j4+2)-(1))+ _z_offset] = dn.val;
z[(((4*n0)-pp)-(1))+ _z_offset] = emin;
/***
*
*     End of DLASQ6
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasq6",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlasq6",999999);
return;
   }
} // End class.
