/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlasr.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlasr {

// *> \brief \b DLASR applies a sequence of plane rotations to a general re
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLASR + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASR( SIDE, PIVOT, DIRECT, M, N, C, S, A, LDA )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          DIRECT, PIVOT, SIDE
// *       INTEGER            LDA, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), C( * ), S( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASR applies a sequence of plane rotations to a real matrix A,
// *> from either the left or the right.
// *>
// *> When SIDE = 'L', the transformation takes the form
// *>
// *>    A := P*A
// *>
// *> and when SIDE = 'R', the transformation takes the form
// *>
// *>    A := A*P**T
// *>
// *> where P is an orthogonal matrix consisting of a sequence of z plane
// *> rotations, with z = M when SIDE = 'L' and z = N when SIDE = 'R',
// *> and P**T is the transpose of P.
// *>
// *> When DIRECT = 'F' (Forward sequence), then
// *>
// *>    P = P(z-1) * ... * P(2) * P(1)
// *>
// *> and when DIRECT = 'B' (Backward sequence), then
// *>
// *>    P = P(1) * P(2) * ... * P(z-1)
// *>
// *> where P(k) is a plane rotation matrix defined by the 2-by-2 rotation

// *>
// *>    R(k) = (  c(k)  s(k) )
// *>         = ( -s(k)  c(k) ).
// *>
// *> When PIVOT = 'V' (Variable pivot), the rotation is performed
// *> for the plane (k,k+1), i.e., P(k) has the form
// *>
// *>    P(k) = (  1                                            )
// *>           (       ...                                     )
// *>           (              1                                )
// *>           (                   c(k)  s(k)                  )
// *>           (                  -s(k)  c(k)                  )
// *>           (                                1              )
// *>           (                                     ...       )
// *>           (                                            1  )
// *>
// *> where R(k) appears as a rank-2 modification to the identity matrix in
// *> rows and columns k and k+1.
// *>
// *> When PIVOT = 'T' (Top pivot), the rotation is performed for the
// *> plane (1,k+1), so P(k) has the form
// *>
// *>    P(k) = (  c(k)                    s(k)                 )
// *>           (         1                                     )
// *>           (              ...                              )
// *>           (                     1                         )
// *>           ( -s(k)                    c(k)                 )
// *>           (                                 1             )
// *>           (                                      ...      )
// *>           (                                             1 )
// *>
// *> where R(k) appears in rows and columns 1 and k+1.
// *>
// *> Similarly, when PIVOT = 'B' (Bottom pivot), the rotation is
// *> performed for the plane (k,z), giving P(k) the form
// *>
// *>    P(k) = ( 1                                             )
// *>           (      ...                                      )
// *>           (             1                                 )
// *>           (                  c(k)                    s(k) )
// *>           (                         1                     )
// *>           (                              ...              )
// *>           (                                     1         )
// *>           (                 -s(k)                    c(k) )
// *>
// *> where R(k) appears in rows and columns k and z.  The rotations are
// *> performed without ever forming P(k) explicitly.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] SIDE
// *> \verbatim
// *>          SIDE is CHARACTER*1
// *>          Specifies whether the plane rotation matrix P is applied to

// *>          A on the left or the right.
// *>          = 'L':  Left, compute A := P*A
// *>          = 'R':  Right, compute A:= A*P**T
// *> \endverbatim
// *>
// *> \param[in] PIVOT
// *> \verbatim
// *>          PIVOT is CHARACTER*1
// *>          Specifies the plane for which P(k) is a plane rotation
// *>          matrix.
// *>          = 'V':  Variable pivot, the plane (k,k+1)
// *>          = 'T':  Top pivot, the plane (1,k+1)
// *>          = 'B':  Bottom pivot, the plane (k,z)
// *> \endverbatim
// *>
// *> \param[in] DIRECT
// *> \verbatim
// *>          DIRECT is CHARACTER*1
// *>          Specifies whether P is a forward or backward sequence of
// *>          plane rotations.
// *>          = 'F':  Forward, P = P(z-1)*...*P(2)*P(1)
// *>          = 'B':  Backward, P = P(1)*P(2)*...*P(z-1)
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix A.  If m <= 1, an immediate
// *>          return is effected.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix A.  If n <= 1, an
// *>          immediate return is effected.
// *> \endverbatim
// *>
// *> \param[in] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension
// *>                  (M-1) if SIDE = 'L'
// *>                  (N-1) if SIDE = 'R'
// *>          The cosines c(k) of the plane rotations.
// *> \endverbatim
// *>
// *> \param[in] S
// *> \verbatim
// *>          S is DOUBLE PRECISION array, dimension
// *>                  (M-1) if SIDE = 'L'
// *>                  (N-1) if SIDE = 'R'
// *>          The sines s(k) of the plane rotations.  The 2-by-2 plane
// *>          rotation part of the matrix P(k), R(k), has the form
// *>          R(k) = (  c(k)  s(k) )
// *>                 ( -s(k)  c(k) ).
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          The M-by-N matrix A.  On exit, A is overwritten by P*A if
// *>          SIDE = 'L' or by A*P**T if SIDE = 'R'.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,M).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup OTHERauxiliary
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters
*
***/

public static void dlasr (String side,
String pivot,
String direct,
int m,
int n,
double [] c, int _c_offset,
double [] s, int _s_offset,
double [] a, int _a_offset,
int lda)  {

label_999999:{

int i= 0;
int info= 0;
int j= 0;
double ctemp= 0.0d;
double stemp= 0.0d;
double temp= 0.0d;
side = side.substring((1)-1,1);
pivot = pivot.substring((1)-1,1);
direct = direct.substring((1)-1,1);
info = 0;
if ((!((com.polytechnik.lapack310.Lsame.lsame(side,"L") || com.polytechnik.lapack310.Lsame.lsame(side,"R")))))  {
    info = 1;
}
else if ((!(((com.polytechnik.lapack310.Lsame.lsame(pivot,"V") || com.polytechnik.lapack310.Lsame.lsame(pivot,"T")) || com.polytechnik.lapack310.Lsame.lsame(pivot,"B")))))  {
    info = 2;
}              // Close else if()
else if ((!((com.polytechnik.lapack310.Lsame.lsame(direct,"F") || com.polytechnik.lapack310.Lsame.lsame(direct,"B")))))  {
    info = 3;
}              // Close else if()
else if ((m < 0))  {
    info = 4;
}              // Close else if()
else if ((n < 0))  {
    info = 5;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info = 9;
}              // Close else if()
if ((info != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DLASR ",info);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasr",999999);
/***
*
*     Quick return if possible
*
***/
}
if ((((m == 0)) || ((n == 0)))) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasr",999999);
}
    /***
*
*        Form  P * A
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(side,"L"))  {
    if (com.polytechnik.lapack310.Lsame.lsame(pivot,"V"))  {
    if (com.polytechnik.lapack310.Lsame.lsame(direct,"F"))  {
    {
for (j = 1; j <= (m-1); j++) {
ctemp = c[(j-(1))+ _c_offset];
stemp = s[(j-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= n; i++) {
temp = a[((j+1)-(1))+(i-(1)) * (lda)+ _a_offset];
a[((j+1)-(1))+(i-(1)) * (lda)+ _a_offset] = ((ctemp*temp)-(stemp*a[(j-(1))+(i-(1)) * (lda)+ _a_offset]));
a[(j-(1))+(i-(1)) * (lda)+ _a_offset] = ((stemp*temp)+(ctemp*a[(j-(1))+(i-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlasr",10);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",20);
}              //  Close for() loop.
}
}
else if (com.polytechnik.lapack310.Lsame.lsame(direct,"B"))  {
    {
int _j_inc = -1;
for (j = (m-1); j >= 1; j += _j_inc) {
ctemp = c[(j-(1))+ _c_offset];
stemp = s[(j-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= n; i++) {
temp = a[((j+1)-(1))+(i-(1)) * (lda)+ _a_offset];
a[((j+1)-(1))+(i-(1)) * (lda)+ _a_offset] = ((ctemp*temp)-(stemp*a[(j-(1))+(i-(1)) * (lda)+ _a_offset]));
a[(j-(1))+(i-(1)) * (lda)+ _a_offset] = ((stemp*temp)+(ctemp*a[(j-(1))+(i-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlasr",30);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",40);
}              //  Close for() loop.
}
}              // Close else if()
}
else if (com.polytechnik.lapack310.Lsame.lsame(pivot,"T"))  {
    if (com.polytechnik.lapack310.Lsame.lsame(direct,"F"))  {
    {
for (j = 2; j <= m; j++) {
ctemp = c[((j-1)-(1))+ _c_offset];
stemp = s[((j-1)-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= n; i++) {
temp = a[(j-(1))+(i-(1)) * (lda)+ _a_offset];
a[(j-(1))+(i-(1)) * (lda)+ _a_offset] = ((ctemp*temp)-(stemp*a[(1-(1))+(i-(1)) * (lda)+ _a_offset]));
a[(1-(1))+(i-(1)) * (lda)+ _a_offset] = ((stemp*temp)+(ctemp*a[(1-(1))+(i-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlasr",50);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",60);
}              //  Close for() loop.
}
}
else if (com.polytechnik.lapack310.Lsame.lsame(direct,"B"))  {
    {
int _j_inc = -1;
for (j = m; j >= 2; j += _j_inc) {
ctemp = c[((j-1)-(1))+ _c_offset];
stemp = s[((j-1)-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= n; i++) {
temp = a[(j-(1))+(i-(1)) * (lda)+ _a_offset];
a[(j-(1))+(i-(1)) * (lda)+ _a_offset] = ((ctemp*temp)-(stemp*a[(1-(1))+(i-(1)) * (lda)+ _a_offset]));
a[(1-(1))+(i-(1)) * (lda)+ _a_offset] = ((stemp*temp)+(ctemp*a[(1-(1))+(i-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlasr",70);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",80);
}              //  Close for() loop.
}
}              // Close else if()
}              // Close else if()
else if (com.polytechnik.lapack310.Lsame.lsame(pivot,"B"))  {
    if (com.polytechnik.lapack310.Lsame.lsame(direct,"F"))  {
    {
for (j = 1; j <= (m-1); j++) {
ctemp = c[(j-(1))+ _c_offset];
stemp = s[(j-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= n; i++) {
temp = a[(j-(1))+(i-(1)) * (lda)+ _a_offset];
a[(j-(1))+(i-(1)) * (lda)+ _a_offset] = ((stemp*a[(m-(1))+(i-(1)) * (lda)+ _a_offset])+(ctemp*temp));
a[(m-(1))+(i-(1)) * (lda)+ _a_offset] = ((ctemp*a[(m-(1))+(i-(1)) * (lda)+ _a_offset])-(stemp*temp));
//Dummy.label("com/polytechnik/lapack310/Dlasr",90);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",100);
}              //  Close for() loop.
}
}
else if (com.polytechnik.lapack310.Lsame.lsame(direct,"B"))  {
    {
int _j_inc = -1;
for (j = (m-1); j >= 1; j += _j_inc) {
ctemp = c[(j-(1))+ _c_offset];
stemp = s[(j-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= n; i++) {
temp = a[(j-(1))+(i-(1)) * (lda)+ _a_offset];
a[(j-(1))+(i-(1)) * (lda)+ _a_offset] = ((stemp*a[(m-(1))+(i-(1)) * (lda)+ _a_offset])+(ctemp*temp));
a[(m-(1))+(i-(1)) * (lda)+ _a_offset] = ((ctemp*a[(m-(1))+(i-(1)) * (lda)+ _a_offset])-(stemp*temp));
//Dummy.label("com/polytechnik/lapack310/Dlasr",110);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",120);
}              //  Close for() loop.
}
}              // Close else if()
}              // Close else if()
/***
*
*        Form A * P**T
*
***/
}
else if (com.polytechnik.lapack310.Lsame.lsame(side,"R"))  {
    if (com.polytechnik.lapack310.Lsame.lsame(pivot,"V"))  {
    if (com.polytechnik.lapack310.Lsame.lsame(direct,"F"))  {
    {
for (j = 1; j <= (n-1); j++) {
ctemp = c[(j-(1))+ _c_offset];
stemp = s[(j-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= m; i++) {
temp = a[(i-(1))+((j+1)-(1)) * (lda)+ _a_offset];
a[(i-(1))+((j+1)-(1)) * (lda)+ _a_offset] = ((ctemp*temp)-(stemp*a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = ((stemp*temp)+(ctemp*a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlasr",130);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",140);
}              //  Close for() loop.
}
}
else if (com.polytechnik.lapack310.Lsame.lsame(direct,"B"))  {
    {
int _j_inc = -1;
for (j = (n-1); j >= 1; j += _j_inc) {
ctemp = c[(j-(1))+ _c_offset];
stemp = s[(j-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= m; i++) {
temp = a[(i-(1))+((j+1)-(1)) * (lda)+ _a_offset];
a[(i-(1))+((j+1)-(1)) * (lda)+ _a_offset] = ((ctemp*temp)-(stemp*a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = ((stemp*temp)+(ctemp*a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlasr",150);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",160);
}              //  Close for() loop.
}
}              // Close else if()
}
else if (com.polytechnik.lapack310.Lsame.lsame(pivot,"T"))  {
    if (com.polytechnik.lapack310.Lsame.lsame(direct,"F"))  {
    {
for (j = 2; j <= n; j++) {
ctemp = c[((j-1)-(1))+ _c_offset];
stemp = s[((j-1)-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= m; i++) {
temp = a[(i-(1))+(j-(1)) * (lda)+ _a_offset];
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = ((ctemp*temp)-(stemp*a[(i-(1))+(1-(1)) * (lda)+ _a_offset]));
a[(i-(1))+(1-(1)) * (lda)+ _a_offset] = ((stemp*temp)+(ctemp*a[(i-(1))+(1-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlasr",170);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",180);
}              //  Close for() loop.
}
}
else if (com.polytechnik.lapack310.Lsame.lsame(direct,"B"))  {
    {
int _j_inc = -1;
for (j = n; j >= 2; j += _j_inc) {
ctemp = c[((j-1)-(1))+ _c_offset];
stemp = s[((j-1)-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= m; i++) {
temp = a[(i-(1))+(j-(1)) * (lda)+ _a_offset];
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = ((ctemp*temp)-(stemp*a[(i-(1))+(1-(1)) * (lda)+ _a_offset]));
a[(i-(1))+(1-(1)) * (lda)+ _a_offset] = ((stemp*temp)+(ctemp*a[(i-(1))+(1-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlasr",190);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",200);
}              //  Close for() loop.
}
}              // Close else if()
}              // Close else if()
else if (com.polytechnik.lapack310.Lsame.lsame(pivot,"B"))  {
    if (com.polytechnik.lapack310.Lsame.lsame(direct,"F"))  {
    {
for (j = 1; j <= (n-1); j++) {
ctemp = c[(j-(1))+ _c_offset];
stemp = s[(j-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= m; i++) {
temp = a[(i-(1))+(j-(1)) * (lda)+ _a_offset];
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = ((stemp*a[(i-(1))+(n-(1)) * (lda)+ _a_offset])+(ctemp*temp));
a[(i-(1))+(n-(1)) * (lda)+ _a_offset] = ((ctemp*a[(i-(1))+(n-(1)) * (lda)+ _a_offset])-(stemp*temp));
//Dummy.label("com/polytechnik/lapack310/Dlasr",210);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",220);
}              //  Close for() loop.
}
}
else if (com.polytechnik.lapack310.Lsame.lsame(direct,"B"))  {
    {
int _j_inc = -1;
for (j = (n-1); j >= 1; j += _j_inc) {
ctemp = c[(j-(1))+ _c_offset];
stemp = s[(j-(1))+ _s_offset];
if ((((ctemp != 1.0)) || ((stemp != 0.0))))  {
    {
for (i = 1; i <= m; i++) {
temp = a[(i-(1))+(j-(1)) * (lda)+ _a_offset];
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = ((stemp*a[(i-(1))+(n-(1)) * (lda)+ _a_offset])+(ctemp*temp));
a[(i-(1))+(n-(1)) * (lda)+ _a_offset] = ((ctemp*a[(i-(1))+(n-(1)) * (lda)+ _a_offset])-(stemp*temp));
//Dummy.label("com/polytechnik/lapack310/Dlasr",230);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dlasr",240);
}              //  Close for() loop.
}
}              // Close else if()
}              // Close else if()
// *
}              // Close else if()
/***
*
*     End of DLASR
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasr",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlasr",999999);
return;
   }
} // End class.
