/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlasrt.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlasrt {

// *> \brief \b DLASRT sorts numbers in increasing or decreasing order.
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLASRT + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASRT( ID, N, D, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          ID
// *       INTEGER            INFO, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   D( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> Sort the numbers in D in increasing order (if ID = 'I') or
// *> in decreasing order (if ID = 'D' ).
// *>
// *> Use Quick Sort, reverting to Insertion sort on arrays of
// *> size <= 20. Dimension of STACK limits N to about 2**32.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] ID
// *> \verbatim
// *>          ID is CHARACTER*1
// *>          = 'I': sort D in increasing order;
// *>          = 'D': sort D in decreasing order.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The length of the array D.
// *> \endverbatim
// *>
// *> \param[in,out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the array to be sorted.
// *>          On exit, D has been sorted into increasing order
// *>          (D(1) <= ... <= D(N) ) or into decreasing order
// *>          (D(1) >= ... >= D(N) ), depending on ID.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dlasrt (String id,
int n,
double [] d, int _d_offset,
intW info)  {

label_999999:{

int dir= 0;
int endd= 0;
int i= 0;
int j= 0;
int start= 0;
int stkpnt= 0;
double d1= 0.0d;
double d2= 0.0d;
double d3= 0.0d;
double dmnmx= 0.0d;
double tmp= 0.0d;
int [] stack= new int[(2) * (32)];
id = id.substring((1)-1,1);
info.val = 0;
dir = -1;
if (com.polytechnik.lapack310.Lsame.lsame(id,"D"))  {
    dir = 0;
}
else if (com.polytechnik.lapack310.Lsame.lsame(id,"I"))  {
    dir = 1;
}              // Close else if()
if ((dir == -1))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DLASRT",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",999999);
/***
*
*     Quick return if possible
*
***/
}
// *
if ((n <= 1)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",999999);
}
    stkpnt = 1;
stack[(1-(1))+(1-(1)) * (2)] = 1;
stack[(2-(1))+(1-(1)) * (2)] = n;
label10: for(;;){
    //Dummy.label("com/polytechnik/lapack310/Dlasrt",10);
start = stack[(1-(1))+(stkpnt-(1)) * (2)];
endd = stack[(2-(1))+(stkpnt-(1)) * (2)];
stkpnt = (stkpnt-1);
/***
*
*        Do Insertion sort on D( START:ENDD )
*
***/
if ((((endd-start) <= 20) && ((endd-start) > 0)))  {
    /***
*
*           Sort into decreasing order
*
***/
if ((dir == 0))  {
    {
label_30:for (i = (start+1); i <= endd; i++) {
{
int _j_inc = -1;
for (j = i; j >= (start+1); j += _j_inc) {
if ((d[(j-(1))+ _d_offset] > d[((j-1)-(1))+ _d_offset]))  {
    dmnmx = d[(j-(1))+ _d_offset];
d[(j-(1))+ _d_offset] = d[((j-1)-(1))+ _d_offset];
d[((j-1)-(1))+ _d_offset] = dmnmx;
}
else  {
  continue label_30;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",30);
}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dlasrt",20);
}              //  Close for() loop.
}
// *
//Dummy.label("com/polytechnik/lapack310/Dlasrt",30);
}              //  Close for() loop.
}
/***
*
*           Sort into increasing order
*
***/
}
else  {
  {
label_50:for (i = (start+1); i <= endd; i++) {
{
int _j_inc = -1;
for (j = i; j >= (start+1); j += _j_inc) {
if ((d[(j-(1))+ _d_offset] < d[((j-1)-(1))+ _d_offset]))  {
    dmnmx = d[(j-(1))+ _d_offset];
d[(j-(1))+ _d_offset] = d[((j-1)-(1))+ _d_offset];
d[((j-1)-(1))+ _d_offset] = dmnmx;
}
else  {
  continue label_50;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",50);
}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dlasrt",40);
}              //  Close for() loop.
}
// *
//Dummy.label("com/polytechnik/lapack310/Dlasrt",50);
}              //  Close for() loop.
}
// *
}              //  Close else.
/***
*
*        Partition D( START:ENDD ) and stack parts, largest one first
*
*        Choose partition entry as median of 3
*
***/
}
else if (((endd-start) > 20))  {
    d1 = d[(start-(1))+ _d_offset];
d2 = d[(endd-(1))+ _d_offset];
i = (((start+endd))/2);
d3 = d[(i-(1))+ _d_offset];
if ((d1 < d2))  {
    if ((d3 < d1))  {
    dmnmx = d1;
}
else if ((d3 < d2))  {
    dmnmx = d3;
}              // Close else if()
else  {
  dmnmx = d2;
}              //  Close else.
}
else  {
  if ((d3 < d2))  {
    dmnmx = d2;
}
else if ((d3 < d1))  {
    dmnmx = d3;
}              // Close else if()
else  {
  dmnmx = d1;
}              //  Close else.
// *
}              //  Close else.
/***
*
*           Sort into decreasing order
*
***/
if ((dir == 0))  {
    i = (start-1);
j = (endd+1);
label60:for(;;){
   //Dummy.label("com/polytechnik/lapack310/Dlasrt",60);
label70:for(;;){
   //Dummy.label("com/polytechnik/lapack310/Dlasrt",70);
j = (j-1);
if ((d[(j-(1))+ _d_offset] < dmnmx)) {
    continue label70;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",70);
}
    label80:for(;;){
   //Dummy.label("com/polytechnik/lapack310/Dlasrt",80);
i = (i+1);
if ((d[(i-(1))+ _d_offset] > dmnmx)) {
    continue label80;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",80);
}
    if ((i < j))  {
    tmp = d[(i-(1))+ _d_offset];
d[(i-(1))+ _d_offset] = d[(j-(1))+ _d_offset];
d[(j-(1))+ _d_offset] = tmp;
continue label60;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",60);
}
if (((j-start) > ((endd-j)-1)))  {
    stkpnt = (stkpnt+1);
stack[(1-(1))+(stkpnt-(1)) * (2)] = start;
stack[(2-(1))+(stkpnt-(1)) * (2)] = j;
stkpnt = (stkpnt+1);
stack[(1-(1))+(stkpnt-(1)) * (2)] = (j+1);
stack[(2-(1))+(stkpnt-(1)) * (2)] = endd;
}
else  {
  stkpnt = (stkpnt+1);
stack[(1-(1))+(stkpnt-(1)) * (2)] = (j+1);
stack[(2-(1))+(stkpnt-(1)) * (2)] = endd;
stkpnt = (stkpnt+1);
stack[(1-(1))+(stkpnt-(1)) * (2)] = start;
stack[(2-(1))+(stkpnt-(1)) * (2)] = j;
}              //  Close else.
break label80;}
break label70;}
break label60;}
/***
*
*           Sort into increasing order
*
***/
}
else  {
  i = (start-1);
j = (endd+1);
label90:for(;;){
   //Dummy.label("com/polytechnik/lapack310/Dlasrt",90);
label100:for(;;){
   //Dummy.label("com/polytechnik/lapack310/Dlasrt",100);
j = (j-1);
if ((d[(j-(1))+ _d_offset] > dmnmx)) {
    continue label100;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",100);
}
    label110:for(;;){
   //Dummy.label("com/polytechnik/lapack310/Dlasrt",110);
i = (i+1);
if ((d[(i-(1))+ _d_offset] < dmnmx)) {
    continue label110;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",110);
}
    if ((i < j))  {
    tmp = d[(i-(1))+ _d_offset];
d[(i-(1))+ _d_offset] = d[(j-(1))+ _d_offset];
d[(j-(1))+ _d_offset] = tmp;
continue label90;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",90);
}
if (((j-start) > ((endd-j)-1)))  {
    stkpnt = (stkpnt+1);
stack[(1-(1))+(stkpnt-(1)) * (2)] = start;
stack[(2-(1))+(stkpnt-(1)) * (2)] = j;
stkpnt = (stkpnt+1);
stack[(1-(1))+(stkpnt-(1)) * (2)] = (j+1);
stack[(2-(1))+(stkpnt-(1)) * (2)] = endd;
}
else  {
  stkpnt = (stkpnt+1);
stack[(1-(1))+(stkpnt-(1)) * (2)] = (j+1);
stack[(2-(1))+(stkpnt-(1)) * (2)] = endd;
stkpnt = (stkpnt+1);
stack[(1-(1))+(stkpnt-(1)) * (2)] = start;
stack[(2-(1))+(stkpnt-(1)) * (2)] = j;
}              //  Close else.
break label110;}
break label100;}
break label90;}
}              //  Close else.
}              // Close else if()
if ((stkpnt > 0)) {
    continue label10;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",10);
}
break; } // end label10
    /***
*
*     End of DLASRT
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasrt",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlasrt",999999);
return;
   }
} // End class.
