/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlasv2.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlasv2 {

// *> \brief \b DLASV2 computes the singular value decomposition of a 2-by-
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLASV2 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASV2( F, G, H, SSMIN, SSMAX, SNR, CSR, SNL, CSL )
// *
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION   CSL, CSR, F, G, H, SNL, SNR, SSMAX, SSMIN
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASV2 computes the singular value decomposition of a 2-by-2
// *> triangular matrix
// *>    [  F   G  ]
// *>    [  0   H  ].
// *> On return, abs(SSMAX) is the larger singular value, abs(SSMIN) is the
// *> smaller singular value, and (CSL,SNL) and (CSR,SNR) are the left and

// *> right singular vectors for abs(SSMAX), giving the decomposition
// *>
// *>    [ CSL  SNL ] [  F   G  ] [ CSR -SNR ]  =  [ SSMAX   0   ]
// *>    [-SNL  CSL ] [  0   H  ] [ SNR  CSR ]     [  0    SSMIN ].
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] F
// *> \verbatim
// *>          F is DOUBLE PRECISION
// *>          The (1,1) element of the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[in] G
// *> \verbatim
// *>          G is DOUBLE PRECISION
// *>          The (1,2) element of the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[in] H
// *> \verbatim
// *>          H is DOUBLE PRECISION
// *>          The (2,2) element of the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[out] SSMIN
// *> \verbatim
// *>          SSMIN is DOUBLE PRECISION
// *>          abs(SSMIN) is the smaller singular value.
// *> \endverbatim
// *>
// *> \param[out] SSMAX
// *> \verbatim
// *>          SSMAX is DOUBLE PRECISION
// *>          abs(SSMAX) is the larger singular value.
// *> \endverbatim
// *>
// *> \param[out] SNL
// *> \verbatim
// *>          SNL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[out] CSL
// *> \verbatim
// *>          CSL is DOUBLE PRECISION
// *>          The vector (CSL, SNL) is a unit left singular vector for the
// *>          singular value abs(SSMAX).
// *> \endverbatim
// *>
// *> \param[out] SNR
// *> \verbatim
// *>          SNR is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[out] CSR
// *> \verbatim
// *>          CSR is DOUBLE PRECISION
// *>          The vector (CSR, SNR) is a unit right singular vector for th
// *>          singular value abs(SSMAX).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup OTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Any input parameter may be aliased with any output parameter.
// *>
// *>  Barring over/underflow and assuming a guard digit in subtraction, al
// *>  output quantities are correct to within a few units in the last
// *>  place (ulps).
// *>
// *>  In IEEE arithmetic, the code works correctly if one matrix element i
// *>  infinite.
// *>
// *>  Overflow will not occur unless the largest singular value itself
// *>  overflows or is within a few ulps of overflow. (On machines with
// *>  partial overflow, like the Cray, overflow may occur if the largest
// *>  singular value is within a factor of 2 of overflow.)
// *>
// *>  Underflow is harmless if underflow is gradual. Otherwise, results
// *>  may correspond to a matrix modified by perturbations of size near
// *>  the underflow threshold.
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*
* =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlasv2 (final double f,
final double g,
final double h,
doubleW ssmin,
doubleW ssmax,
doubleW snr,
doubleW csr,
doubleW snl,
doubleW csl)  {

label_999999:{

boolean gasmal= false;
boolean swap= false;
int pmax= 0;
double a= 0.0d;
double clt= 0.0d;
double crt= 0.0d;
double d= 0.0d;
double fa= 0.0d;
double ft= 0.0d;
double ga= 0.0d;
double gt= 0.0d;
double ha= 0.0d;
double ht= 0.0d;
double l= 0.0d;
double m= 0.0d;
double mm= 0.0d;
double r= 0.0d;
double s= 0.0d;
double slt= 0.0d;
double srt= 0.0d;
double t= 0.0d;
double temp= 0.0d;
double tsign= 0.0d;
double tt= 0.0d;
ft = f;
fa = Math.abs(ft);
ht = h;
/***
*
*     PMAX points to the maximum absolute element of matrix
*       PMAX = 1 if F largest in absolute values
*       PMAX = 2 if G largest in absolute values
*       PMAX = 3 if H largest in absolute values
*
***/
ha = Math.abs(h);
pmax = 1;
swap = ((ha > fa));
if (swap)  {
    pmax = 3;
temp = ft;
ft = ht;
ht = temp;
temp = fa;
fa = ha;
/***
*
*        Now FA .ge. HA
*
***/
ha = temp;
}
gt = g;
ga = Math.abs(gt);
/***
*
*        Diagonal matrix
*
***/
if ((ga == 0.0))  {
    ssmin.val = ha;
ssmax.val = fa;
clt = 1.0;
crt = 1.0;
slt = 0.0;
srt = 0.0;
}
else  {
  gasmal = true;
if ((ga > fa))  {
    pmax = 2;
/***
*
*              Case of very large GA
*
***/
if ((((fa/ga)) < com.polytechnik.lapack310.Dlamch.dlamch("EPS")))  {
    gasmal = false;
ssmax.val = ga;
if ((ha > 1.0))  {
    ssmin.val = (fa/((ga/ha)));
}
else  {
  ssmin.val = (((fa/ga))*ha);
}              //  Close else.
clt = 1.0;
slt = (ht/gt);
srt = 1.0;
crt = (ft/gt);
}
}
/***
*
*           Normal case
*
***/
if (gasmal)  {
    d = (fa-ha);
/***
*
*              Copes with infinite F or H
*
***/
if ((d == fa))  {
    l = 1.0;
}
else  {
  l = (d/fa);
/***
*
*           Note that 0 .le. L .le. 1
*
***/
}              //  Close else.
/***
*
*           Note that abs(M) .le. 1/macheps
*
***/
m = (gt/ft);
/***
*
*           Note that T .ge. 1
*
***/
t = (2.0-l);
mm = (m*m);
tt = (t*t);
/***
*
*           Note that 1 .le. S .le. 1 + 1/macheps
*
***/
s = Math.sqrt((tt+mm));
if ((l == 0.0))  {
    r = Math.abs(m);
}
else  {
  r = Math.sqrt(((l*l)+mm));
/***
*
*           Note that 0 .le. R .le. 1 + 1/macheps
*
***/
}              //  Close else.
/***
*
*           Note that 1 .le. A .le. 1 + abs(M)
*
***/
a = (0.5*((s+r)));
ssmin.val = (ha/a);
ssmax.val = (fa*a);
/***
*
*              Note that M is very tiny
*
***/
if ((mm == 0.0))  {
    if ((l == 0.0))  {
    t = (Util.dsign(2.0,ft)*Util.dsign(1.0,gt));
}
else  {
  t = ((gt/Util.dsign(d,ft))+(m/t));
}              //  Close else.
}
else  {
  t = ((((m/((s+t)))+(m/((r+l)))))*((1.0+a)));
}              //  Close else.
l = Math.sqrt(((t*t)+4.0));
crt = (2.0/l);
srt = (t/l);
clt = (((crt+(srt*m)))/a);
slt = ((((ht/ft))*srt)/a);
}
}              //  Close else.
if (swap)  {
    csl.val = srt;
snl.val = crt;
csr.val = slt;
snr.val = clt;
}
else  {
  csl.val = clt;
snl.val = slt;
csr.val = crt;
snr.val = srt;
/***
*
*     Correct signs of SSMAX and SSMIN
*
***/
}              //  Close else.
if ((pmax == 1)) {
    tsign = ((Util.dsign(1.0,csr.val)*Util.dsign(1.0,csl.val))*Util.dsign(1.0,f));
}
    if ((pmax == 2)) {
    tsign = ((Util.dsign(1.0,snr.val)*Util.dsign(1.0,csl.val))*Util.dsign(1.0,g));
}
    if ((pmax == 3)) {
    tsign = ((Util.dsign(1.0,snr.val)*Util.dsign(1.0,snl.val))*Util.dsign(1.0,h));
}
    ssmax.val = Util.dsign(ssmax.val,tsign);
ssmin.val = Util.dsign(ssmin.val,((tsign*Util.dsign(1.0,f))*Util.dsign(1.0,h)));
/***
*
*     End of DLASV2
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasv2",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlasv2",999999);
return;
   }
} // End class.
