/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlasy2.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlasy2 {

// *> \brief \b DLASY2 solves the Sylvester matrix equation where the matri
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLASY2 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASY2( LTRANL, LTRANR, ISGN, N1, N2, TL, LDTL, TR,
// *                          LDTR, B, LDB, SCALE, X, LDX, XNORM, INFO )
// *
// *       .. Scalar Arguments ..
// *       LOGICAL            LTRANL, LTRANR
// *       INTEGER            INFO, ISGN, LDB, LDTL, LDTR, LDX, N1, N2
// *       DOUBLE PRECISION   SCALE, XNORM
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   B( LDB, * ), TL( LDTL, * ), TR( LDTR, * ),
// *      $                   X( LDX, * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASY2 solves for the N1 by N2 matrix X, 1 <= N1,N2 <= 2, in
// *>
// *>        op(TL)*X + ISGN*X*op(TR) = SCALE*B,
// *>
// *> where TL is N1 by N1, TR is N2 by N2, B is N1 by N2, and ISGN = 1 or

// *> -1.  op(T) = T or T**T, where T**T denotes the transpose of T.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] LTRANL
// *> \verbatim
// *>          LTRANL is LOGICAL
// *>          On entry, LTRANL specifies the op(TL):
// *>             = .FALSE., op(TL) = TL,
// *>             = .TRUE., op(TL) = TL**T.
// *> \endverbatim
// *>
// *> \param[in] LTRANR
// *> \verbatim
// *>          LTRANR is LOGICAL
// *>          On entry, LTRANR specifies the op(TR):
// *>            = .FALSE., op(TR) = TR,
// *>            = .TRUE., op(TR) = TR**T.
// *> \endverbatim
// *>
// *> \param[in] ISGN
// *> \verbatim
// *>          ISGN is INTEGER
// *>          On entry, ISGN specifies the sign of the equation
// *>          as described before. ISGN may only be 1 or -1.
// *> \endverbatim
// *>
// *> \param[in] N1
// *> \verbatim
// *>          N1 is INTEGER
// *>          On entry, N1 specifies the order of matrix TL.
// *>          N1 may only be 0, 1 or 2.
// *> \endverbatim
// *>
// *> \param[in] N2
// *> \verbatim
// *>          N2 is INTEGER
// *>          On entry, N2 specifies the order of matrix TR.
// *>          N2 may only be 0, 1 or 2.
// *> \endverbatim
// *>
// *> \param[in] TL
// *> \verbatim
// *>          TL is DOUBLE PRECISION array, dimension (LDTL,2)
// *>          On entry, TL contains an N1 by N1 matrix.
// *> \endverbatim
// *>
// *> \param[in] LDTL
// *> \verbatim
// *>          LDTL is INTEGER
// *>          The leading dimension of the matrix TL. LDTL >= max(1,N1).
// *> \endverbatim
// *>
// *> \param[in] TR
// *> \verbatim
// *>          TR is DOUBLE PRECISION array, dimension (LDTR,2)
// *>          On entry, TR contains an N2 by N2 matrix.
// *> \endverbatim
// *>
// *> \param[in] LDTR
// *> \verbatim
// *>          LDTR is INTEGER
// *>          The leading dimension of the matrix TR. LDTR >= max(1,N2).
// *> \endverbatim
// *>
// *> \param[in] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array, dimension (LDB,2)
// *>          On entry, the N1 by N2 matrix B contains the right-hand
// *>          side of the equation.
// *> \endverbatim
// *>
// *> \param[in] LDB
// *> \verbatim
// *>          LDB is INTEGER
// *>          The leading dimension of the matrix B. LDB >= max(1,N1).
// *> \endverbatim
// *>
// *> \param[out] SCALE
// *> \verbatim
// *>          SCALE is DOUBLE PRECISION
// *>          On exit, SCALE contains the scale factor. SCALE is chosen
// *>          less than or equal to 1 to prevent the solution overflowing.
// *> \endverbatim
// *>
// *> \param[out] X
// *> \verbatim
// *>          X is DOUBLE PRECISION array, dimension (LDX,2)
// *>          On exit, X contains the N1 by N2 solution.
// *> \endverbatim
// *>
// *> \param[in] LDX
// *> \verbatim
// *>          LDX is INTEGER
// *>          The leading dimension of the matrix X. LDX >= max(1,N1).
// *> \endverbatim
// *>
// *> \param[out] XNORM
// *> \verbatim
// *>          XNORM is DOUBLE PRECISION
// *>          On exit, XNORM is the infinity-norm of the solution.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          On exit, INFO is set to
// *>             0: successful exit.
// *>             1: TL and TR have too close eigenvalues, so TL or
// *>                TR is perturbed to get a nonsingular equation.
// *>          NOTE: In the interests of speed, this routine does not
// *>                check the inputs for errors.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleSYauxiliary
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
* =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Data statements ..
***/
private static final int [] locu22 = {
4 , 3 , 2 , 1
};
private static final int [] locl21 = {
2 , 1 , 4 , 3
};
private static final int [] locu12 = {
3 , 4 , 1 , 2
};
private static final boolean [] xswpiv = {
false , false , true , true
};
/***
*     ..
*     .. Executable Statements ..
*
*     Do not check the input parameters for errors
*
***/
private static final boolean [] bswpiv = {
false , true , false , true
};
/***
*
*     Quick return if possible
*
***/

public static void dlasy2 (boolean ltranl,
boolean ltranr,
int isgn,
int n1,
int n2,
double [] tl, int _tl_offset,
int ldtl,
double [] tr, int _tr_offset,
int ldtr,
double [] b, int _b_offset,
int ldb,
doubleW scale,
double [] x, int _x_offset,
int ldx,
doubleW xnorm,
intW info)  {

label_999999:{

boolean bswap= false;
boolean xswap= false;
int i= 0;
int ip= 0;
int ipiv= 0;
int ipsv= 0;
int j= 0;
int jp= 0;
int jpsv= 0;
int k= 0;
double bet= 0.0d;
double eps= 0.0d;
double gam= 0.0d;
double l21= 0.0d;
double sgn= 0.0d;
double smin= 0.0d;
double smlnum= 0.0d;
double tau1= 0.0d;
double temp= 0.0d;
double u11= 0.0d;
double u12= 0.0d;
double u22= 0.0d;
double xmax= 0.0d;
int [] jpiv= new int[(4)];
double [] btmp= new double[(4)];
double [] t16= new double[(4) * (4)];
double [] tmp= new double[(4)];
double [] x2= new double[(2)];
info.val = 0;
/***
*
*     Set constants to control overflow
*
***/
if (((n1 == 0) || (n2 == 0))) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasy2",999999);
}
    eps = com.polytechnik.lapack310.Dlamch.dlamch("P");
smlnum = (com.polytechnik.lapack310.Dlamch.dlamch("S")/eps);
// *
sgn = (double)(isgn);
k = (((n1+n1)+n2)-2);
/***
*
*     1 by 1: TL11*X + SGN*X*TR11 = B11
*
***/
label50:{
label40:{
label30:{
label20:{
label10:{
{
  int _cg_tmp = k;
if (_cg_tmp == 1)
    break label10;//Dummy.go_to("com/polytechnik/lapack310/Dlasy2",10);
else if (_cg_tmp == 2)
    break label20;//Dummy.go_to("com/polytechnik/lapack310/Dlasy2",20);
else if (_cg_tmp == 3)
    break label30;//Dummy.go_to("com/polytechnik/lapack310/Dlasy2",30);
else if (_cg_tmp == 4)
    break label50;//Dummy.go_to("com/polytechnik/lapack310/Dlasy2",50);
}
}// END label10:
//Dummy.label("com/polytechnik/lapack310/Dlasy2",10);
tau1 = (tl[(1-(1))+(1-(1)) * (ldtl)+ _tl_offset]+(sgn*tr[(1-(1))+(1-(1)) * (ldtr)+ _tr_offset]));
bet = Math.abs(tau1);
if ((bet <= smlnum))  {
    tau1 = smlnum;
bet = smlnum;
info.val = 1;
// *
}
scale.val = 1.0;
gam = Math.abs(b[(1-(1))+(1-(1)) * (ldb)+ _b_offset]);
// *
if (((smlnum*gam) > bet)) {
    scale.val = (1.0/gam);
}
    x[(1-(1))+(1-(1)) * (ldx)+ _x_offset] = (((b[(1-(1))+(1-(1)) * (ldb)+ _b_offset]*scale.val))/tau1);
xnorm.val = Math.abs(x[(1-(1))+(1-(1)) * (ldx)+ _x_offset]);
/***
*
*     1 by 2:
*     TL11*[X11 X12] + ISGN*[X11 X12]*op[TR11 TR12]  = [B11 B12]
*                                       [TR21 TR22]
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasy2",999999);
// *
}// END label20:
//Dummy.label("com/polytechnik/lapack310/Dlasy2",20);
smin = Math.max((eps*Math.max(Math.max(Util.max(Math.abs(tl[(1-(1))+(1-(1)) * (ldtl)+ _tl_offset]), Math.abs(tr[(1-(1))+(1-(1)) * (ldtr)+ _tr_offset]), Math.abs(tr[(1-(1))+(2-(1)) * (ldtr)+ _tr_offset])), Math.abs(tr[(2-(1))+(1-(1)) * (ldtr)+ _tr_offset])), Math.abs(tr[(2-(1))+(2-(1)) * (ldtr)+ _tr_offset])) ), smlnum);
tmp[(1-(1))] = (tl[(1-(1))+(1-(1)) * (ldtl)+ _tl_offset]+(sgn*tr[(1-(1))+(1-(1)) * (ldtr)+ _tr_offset]));
tmp[(4-(1))] = (tl[(1-(1))+(1-(1)) * (ldtl)+ _tl_offset]+(sgn*tr[(2-(1))+(2-(1)) * (ldtr)+ _tr_offset]));
if (ltranr)  {
    tmp[(2-(1))] = (sgn*tr[(2-(1))+(1-(1)) * (ldtr)+ _tr_offset]);
tmp[(3-(1))] = (sgn*tr[(1-(1))+(2-(1)) * (ldtr)+ _tr_offset]);
}
else  {
  tmp[(2-(1))] = (sgn*tr[(1-(1))+(2-(1)) * (ldtr)+ _tr_offset]);
tmp[(3-(1))] = (sgn*tr[(2-(1))+(1-(1)) * (ldtr)+ _tr_offset]);
}              //  Close else.
btmp[(1-(1))] = b[(1-(1))+(1-(1)) * (ldb)+ _b_offset];
btmp[(2-(1))] = b[(1-(1))+(2-(1)) * (ldb)+ _b_offset];
/***
*
*     2 by 1:
*          op[TL11 TL12]*[X11] + ISGN* [X11]*TR11  = [B11]
*            [TL21 TL22] [X21]         [X21]         [B21]
*
***/
break label40;//Dummy.go_to("com/polytechnik/lapack310/Dlasy2",40);
}// END label30:
//Dummy.label("com/polytechnik/lapack310/Dlasy2",30);
smin = Math.max((eps*Math.max(Math.max(Util.max(Math.abs(tr[(1-(1))+(1-(1)) * (ldtr)+ _tr_offset]), Math.abs(tl[(1-(1))+(1-(1)) * (ldtl)+ _tl_offset]), Math.abs(tl[(1-(1))+(2-(1)) * (ldtl)+ _tl_offset])), Math.abs(tl[(2-(1))+(1-(1)) * (ldtl)+ _tl_offset])), Math.abs(tl[(2-(1))+(2-(1)) * (ldtl)+ _tl_offset])) ), smlnum);
tmp[(1-(1))] = (tl[(1-(1))+(1-(1)) * (ldtl)+ _tl_offset]+(sgn*tr[(1-(1))+(1-(1)) * (ldtr)+ _tr_offset]));
tmp[(4-(1))] = (tl[(2-(1))+(2-(1)) * (ldtl)+ _tl_offset]+(sgn*tr[(1-(1))+(1-(1)) * (ldtr)+ _tr_offset]));
if (ltranl)  {
    tmp[(2-(1))] = tl[(1-(1))+(2-(1)) * (ldtl)+ _tl_offset];
tmp[(3-(1))] = tl[(2-(1))+(1-(1)) * (ldtl)+ _tl_offset];
}
else  {
  tmp[(2-(1))] = tl[(2-(1))+(1-(1)) * (ldtl)+ _tl_offset];
tmp[(3-(1))] = tl[(1-(1))+(2-(1)) * (ldtl)+ _tl_offset];
}              //  Close else.
btmp[(1-(1))] = b[(1-(1))+(1-(1)) * (ldb)+ _b_offset];
btmp[(2-(1))] = b[(2-(1))+(1-(1)) * (ldb)+ _b_offset];
/***
*
*     Solve 2 by 2 system using complete pivoting.
*     Set pivots less than SMIN to SMIN.
*
***/
}// END label40:
//Dummy.label("com/polytechnik/lapack310/Dlasy2",40);
ipiv = com.polytechnik.lapack310.Idamax.idamax(4,tmp,0,1);
u11 = tmp[(ipiv-(1))];
if ((Math.abs(u11) <= smin))  {
    info.val = 1;
u11 = smin;
}
u12 = tmp[(locu12[(ipiv-(1))]-(1))];
l21 = (tmp[(locl21[(ipiv-(1))]-(1))]/u11);
u22 = (tmp[(locu22[(ipiv-(1))]-(1))]-(u12*l21));
xswap = xswpiv[(ipiv-(1))];
bswap = bswpiv[(ipiv-(1))];
if ((Math.abs(u22) <= smin))  {
    info.val = 1;
u22 = smin;
}
if (bswap)  {
    temp = btmp[(2-(1))];
btmp[(2-(1))] = (btmp[(1-(1))]-(l21*temp));
btmp[(1-(1))] = temp;
}
else  {
  btmp[(2-(1))] = (btmp[(2-(1))]-(l21*btmp[(1-(1))]));
}              //  Close else.
scale.val = 1.0;
if ((((((2.0*smlnum))*Math.abs(btmp[(2-(1))])) > Math.abs(u22)) || ((((2.0*smlnum))*Math.abs(btmp[(1-(1))])) > Math.abs(u11))))  {
    scale.val = (0.5/Math.max(Math.abs(btmp[(1-(1))]), Math.abs(btmp[(2-(1))])));
btmp[(1-(1))] = (btmp[(1-(1))]*scale.val);
btmp[(2-(1))] = (btmp[(2-(1))]*scale.val);
}
x2[(2-(1))] = (btmp[(2-(1))]/u22);
x2[(1-(1))] = ((btmp[(1-(1))]/u11)-(((u12/u11))*x2[(2-(1))]));
if (xswap)  {
    temp = x2[(2-(1))];
x2[(2-(1))] = x2[(1-(1))];
x2[(1-(1))] = temp;
}
x[(1-(1))+(1-(1)) * (ldx)+ _x_offset] = x2[(1-(1))];
if ((n1 == 1))  {
    x[(1-(1))+(2-(1)) * (ldx)+ _x_offset] = x2[(2-(1))];
xnorm.val = (Math.abs(x[(1-(1))+(1-(1)) * (ldx)+ _x_offset])+Math.abs(x[(1-(1))+(2-(1)) * (ldx)+ _x_offset]));
}
else  {
  x[(2-(1))+(1-(1)) * (ldx)+ _x_offset] = x2[(2-(1))];
xnorm.val = Math.max(Math.abs(x[(1-(1))+(1-(1)) * (ldx)+ _x_offset]), Math.abs(x[(2-(1))+(1-(1)) * (ldx)+ _x_offset]));
}              //  Close else.
/***
*
*     2 by 2:
*     op[TL11 TL12]*[X11 X12] +ISGN* [X11 X12]*op[TR11 TR12] = [B11 B12]
*       [TL21 TL22] [X21 X22]        [X21 X22]   [TR21 TR22]   [B21 B22]
*
*     Solve equivalent 4 by 4 system using complete pivoting.
*     Set pivots less than SMIN to SMIN.
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasy2",999999);
}// END label50:
//Dummy.label("com/polytechnik/lapack310/Dlasy2",50);
smin = Math.max(Util.max(Math.abs(tr[(1-(1))+(1-(1)) * (ldtr)+ _tr_offset]), Math.abs(tr[(1-(1))+(2-(1)) * (ldtr)+ _tr_offset]), Math.abs(tr[(2-(1))+(1-(1)) * (ldtr)+ _tr_offset])), Math.abs(tr[(2-(1))+(2-(1)) * (ldtr)+ _tr_offset])) ;
smin = Math.max(Math.max(Util.max(smin, Math.abs(tl[(1-(1))+(1-(1)) * (ldtl)+ _tl_offset]), Math.abs(tl[(1-(1))+(2-(1)) * (ldtl)+ _tl_offset])), Math.abs(tl[(2-(1))+(1-(1)) * (ldtl)+ _tl_offset])), Math.abs(tl[(2-(1))+(2-(1)) * (ldtl)+ _tl_offset])) ;
smin = Math.max((eps*smin), smlnum);
btmp[(1-(1))] = 0.0;
com.polytechnik.lapack310.Dcopy.dcopy(16,btmp,0,0,t16,0,1);
t16[(1-(1))+(1-(1)) * (4)] = (tl[(1-(1))+(1-(1)) * (ldtl)+ _tl_offset]+(sgn*tr[(1-(1))+(1-(1)) * (ldtr)+ _tr_offset]));
t16[(2-(1))+(2-(1)) * (4)] = (tl[(2-(1))+(2-(1)) * (ldtl)+ _tl_offset]+(sgn*tr[(1-(1))+(1-(1)) * (ldtr)+ _tr_offset]));
t16[(3-(1))+(3-(1)) * (4)] = (tl[(1-(1))+(1-(1)) * (ldtl)+ _tl_offset]+(sgn*tr[(2-(1))+(2-(1)) * (ldtr)+ _tr_offset]));
t16[(4-(1))+(4-(1)) * (4)] = (tl[(2-(1))+(2-(1)) * (ldtl)+ _tl_offset]+(sgn*tr[(2-(1))+(2-(1)) * (ldtr)+ _tr_offset]));
if (ltranl)  {
    t16[(1-(1))+(2-(1)) * (4)] = tl[(2-(1))+(1-(1)) * (ldtl)+ _tl_offset];
t16[(2-(1))+(1-(1)) * (4)] = tl[(1-(1))+(2-(1)) * (ldtl)+ _tl_offset];
t16[(3-(1))+(4-(1)) * (4)] = tl[(2-(1))+(1-(1)) * (ldtl)+ _tl_offset];
t16[(4-(1))+(3-(1)) * (4)] = tl[(1-(1))+(2-(1)) * (ldtl)+ _tl_offset];
}
else  {
  t16[(1-(1))+(2-(1)) * (4)] = tl[(1-(1))+(2-(1)) * (ldtl)+ _tl_offset];
t16[(2-(1))+(1-(1)) * (4)] = tl[(2-(1))+(1-(1)) * (ldtl)+ _tl_offset];
t16[(3-(1))+(4-(1)) * (4)] = tl[(1-(1))+(2-(1)) * (ldtl)+ _tl_offset];
t16[(4-(1))+(3-(1)) * (4)] = tl[(2-(1))+(1-(1)) * (ldtl)+ _tl_offset];
}              //  Close else.
if (ltranr)  {
    t16[(1-(1))+(3-(1)) * (4)] = (sgn*tr[(1-(1))+(2-(1)) * (ldtr)+ _tr_offset]);
t16[(2-(1))+(4-(1)) * (4)] = (sgn*tr[(1-(1))+(2-(1)) * (ldtr)+ _tr_offset]);
t16[(3-(1))+(1-(1)) * (4)] = (sgn*tr[(2-(1))+(1-(1)) * (ldtr)+ _tr_offset]);
t16[(4-(1))+(2-(1)) * (4)] = (sgn*tr[(2-(1))+(1-(1)) * (ldtr)+ _tr_offset]);
}
else  {
  t16[(1-(1))+(3-(1)) * (4)] = (sgn*tr[(2-(1))+(1-(1)) * (ldtr)+ _tr_offset]);
t16[(2-(1))+(4-(1)) * (4)] = (sgn*tr[(2-(1))+(1-(1)) * (ldtr)+ _tr_offset]);
t16[(3-(1))+(1-(1)) * (4)] = (sgn*tr[(1-(1))+(2-(1)) * (ldtr)+ _tr_offset]);
t16[(4-(1))+(2-(1)) * (4)] = (sgn*tr[(1-(1))+(2-(1)) * (ldtr)+ _tr_offset]);
}              //  Close else.
btmp[(1-(1))] = b[(1-(1))+(1-(1)) * (ldb)+ _b_offset];
btmp[(2-(1))] = b[(2-(1))+(1-(1)) * (ldb)+ _b_offset];
btmp[(3-(1))] = b[(1-(1))+(2-(1)) * (ldb)+ _b_offset];
/***
*
*     Perform elimination
*
***/
btmp[(4-(1))] = b[(2-(1))+(2-(1)) * (ldb)+ _b_offset];
{
for (i = 1; i <= 3; i++) {
xmax = 0.0;
{
for (ip = i; ip <= 4; ip++) {
{
for (jp = i; jp <= 4; jp++) {
if ((Math.abs(t16[(ip-(1))+(jp-(1)) * (4)]) >= xmax))  {
    xmax = Math.abs(t16[(ip-(1))+(jp-(1)) * (4)]);
ipsv = ip;
jpsv = jp;
}
//Dummy.label("com/polytechnik/lapack310/Dlasy2",60);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dlasy2",70);
}              //  Close for() loop.
}
if ((ipsv != i))  {
    com.polytechnik.lapack310.Dswap.dswap(4,t16,(ipsv-(1))+(1-(1)) * (4),4,t16,(i-(1))+(1-(1)) * (4),4);
temp = btmp[(i-(1))];
btmp[(i-(1))] = btmp[(ipsv-(1))];
btmp[(ipsv-(1))] = temp;
}
if ((jpsv != i)) {
    com.polytechnik.lapack310.Dswap.dswap(4,t16,(1-(1))+(jpsv-(1)) * (4),1,t16,(1-(1))+(i-(1)) * (4),1);
}
    jpiv[(i-(1))] = jpsv;
if ((Math.abs(t16[(i-(1))+(i-(1)) * (4)]) < smin))  {
    info.val = 1;
t16[(i-(1))+(i-(1)) * (4)] = smin;
}
{
for (j = (i+1); j <= 4; j++) {
t16[(j-(1))+(i-(1)) * (4)] = (t16[(j-(1))+(i-(1)) * (4)]/t16[(i-(1))+(i-(1)) * (4)]);
btmp[(j-(1))] = (btmp[(j-(1))]-(t16[(j-(1))+(i-(1)) * (4)]*btmp[(i-(1))]));
{
for (k = (i+1); k <= 4; k++) {
t16[(j-(1))+(k-(1)) * (4)] = (t16[(j-(1))+(k-(1)) * (4)]-(t16[(j-(1))+(i-(1)) * (4)]*t16[(i-(1))+(k-(1)) * (4)]));
//Dummy.label("com/polytechnik/lapack310/Dlasy2",80);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dlasy2",90);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dlasy2",100);
}              //  Close for() loop.
}
if ((Math.abs(t16[(4-(1))+(4-(1)) * (4)]) < smin))  {
    info.val = 1;
t16[(4-(1))+(4-(1)) * (4)] = smin;
}
scale.val = 1.0;
if ((((((((8.0*smlnum))*Math.abs(btmp[(1-(1))])) > Math.abs(t16[(1-(1))+(1-(1)) * (4)])) || ((((8.0*smlnum))*Math.abs(btmp[(2-(1))])) > Math.abs(t16[(2-(1))+(2-(1)) * (4)]))) || ((((8.0*smlnum))*Math.abs(btmp[(3-(1))])) > Math.abs(t16[(3-(1))+(3-(1)) * (4)]))) || ((((8.0*smlnum))*Math.abs(btmp[(4-(1))])) > Math.abs(t16[(4-(1))+(4-(1)) * (4)]))))  {
    scale.val = (((1.0/8.0))/Math.max(Util.max(Math.abs(btmp[(1-(1))]), Math.abs(btmp[(2-(1))]), Math.abs(btmp[(3-(1))])), Math.abs(btmp[(4-(1))])) );
btmp[(1-(1))] = (btmp[(1-(1))]*scale.val);
btmp[(2-(1))] = (btmp[(2-(1))]*scale.val);
btmp[(3-(1))] = (btmp[(3-(1))]*scale.val);
btmp[(4-(1))] = (btmp[(4-(1))]*scale.val);
}
{
for (i = 1; i <= 4; i++) {
k = (5-i);
temp = (1.0/t16[(k-(1))+(k-(1)) * (4)]);
tmp[(k-(1))] = (btmp[(k-(1))]*temp);
{
for (j = (k+1); j <= 4; j++) {
tmp[(k-(1))] = (tmp[(k-(1))]-(((temp*t16[(k-(1))+(j-(1)) * (4)]))*tmp[(j-(1))]));
//Dummy.label("com/polytechnik/lapack310/Dlasy2",110);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dlasy2",120);
}              //  Close for() loop.
}
{
for (i = 1; i <= 3; i++) {
if ((jpiv[((4-i)-(1))] != (4-i)))  {
    temp = tmp[((4-i)-(1))];
tmp[((4-i)-(1))] = tmp[(jpiv[((4-i)-(1))]-(1))];
tmp[(jpiv[((4-i)-(1))]-(1))] = temp;
}
//Dummy.label("com/polytechnik/lapack310/Dlasy2",130);
}              //  Close for() loop.
}
x[(1-(1))+(1-(1)) * (ldx)+ _x_offset] = tmp[(1-(1))];
x[(2-(1))+(1-(1)) * (ldx)+ _x_offset] = tmp[(2-(1))];
x[(1-(1))+(2-(1)) * (ldx)+ _x_offset] = tmp[(3-(1))];
x[(2-(1))+(2-(1)) * (ldx)+ _x_offset] = tmp[(4-(1))];
xnorm.val = Math.max((Math.abs(tmp[(1-(1))])+Math.abs(tmp[(3-(1))])), (Math.abs(tmp[(2-(1))])+Math.abs(tmp[(4-(1))])));
/***
*
*     End of DLASY2
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlasy2",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dlasy2",999999);
return;
   }
} // End class.
