/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dorgbr.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dorgbr {

// *> \brief \b DORGBR
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DORGBR + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DORGBR( VECT, M, N, K, A, LDA, TAU, WORK, LWORK, INFO
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          VECT
// *       INTEGER            INFO, K, LDA, LWORK, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DORGBR generates one of the real orthogonal matrices Q or P**T
// *> determined by DGEBRD when reducing a real matrix A to bidiagonal
// *> form: A = Q * B * P**T.  Q and P**T are defined as products of
// *> elementary reflectors H(i) or G(i) respectively.
// *>
// *> If VECT = 'Q', A is assumed to have been an M-by-K matrix, and Q
// *> is of order M:
// *> if m >= k, Q = H(1) H(2) . . . H(k) and DORGBR returns the first n
// *> columns of Q, where m >= n >= k;
// *> if m < k, Q = H(1) H(2) . . . H(m-1) and DORGBR returns Q as an
// *> M-by-M matrix.
// *>
// *> If VECT = 'P', A is assumed to have been a K-by-N matrix, and P**T
// *> is of order N:
// *> if k < n, P**T = G(k) . . . G(2) G(1) and DORGBR returns the first m

// *> rows of P**T, where n >= m >= k;
// *> if k >= n, P**T = G(n-1) . . . G(2) G(1) and DORGBR returns P**T as
// *> an N-by-N matrix.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] VECT
// *> \verbatim
// *>          VECT is CHARACTER*1
// *>          Specifies whether the matrix Q or the matrix P**T is
// *>          required, as defined in the transformation applied by DGEBRD
// *>          = 'Q':  generate Q;
// *>          = 'P':  generate P**T.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix Q or P**T to be returned.
// *>          M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix Q or P**T to be returned
// *>          N >= 0.
// *>          If VECT = 'Q', M >= N >= min(M,K);
// *>          if VECT = 'P', N >= M >= min(N,K).
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>          If VECT = 'Q', the number of columns in the original M-by-K

// *>          matrix reduced by DGEBRD.
// *>          If VECT = 'P', the number of rows in the original K-by-N
// *>          matrix reduced by DGEBRD.
// *>          K >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the vectors which define the elementary reflectors
// *>          as returned by DGEBRD.
// *>          On exit, the M-by-N matrix Q or P**T.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A. LDA >= max(1,M).
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension
// *>                                (min(M,K)) if VECT = 'Q'
// *>                                (min(N,K)) if VECT = 'P'
// *>          TAU(i) must contain the scalar factor of the elementary
// *>          reflector H(i) or G(i), which determines Q or P**T, as
// *>          returned by DGEBRD in its array argument TAUQ or TAUP.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK. LWORK >= max(1,min(M,N)).
// *>          For optimum performance LWORK >= min(M,N)*NB, where NB
// *>          is the optimal blocksize.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleGBcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dorgbr (String vect,
int m,
int n,
int k,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

label_999999:{

boolean lquery= false;
boolean wantq= false;
int i= 0;
intW iinfo= new intW(0);
int j= 0;
int lwkopt= 0;
int mn= 0;
vect = vect.substring((1)-1,1);
info.val = 0;
wantq = com.polytechnik.lapack310.Lsame.lsame(vect,"Q");
mn = Math.min(m, n);
lquery = ((lwork == -1));
if (((!wantq) && (!com.polytechnik.lapack310.Lsame.lsame(vect,"P"))))  {
    info.val = -1;
}
else if ((m < 0))  {
    info.val = -2;
}              // Close else if()
else if ((((n < 0) || ((wantq && (((n > m) || (n < Math.min(m, k))))))) || (((!wantq) && (((m > n) || (m < Math.min(n, k))))))))  {
    info.val = -3;
}              // Close else if()
else if ((k < 0))  {
    info.val = -4;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info.val = -6;
}              // Close else if()
else if (((lwork < Math.max(1, mn)) && (!lquery)))  {
    info.val = -9;
// *
}              // Close else if()
if ((info.val == 0))  {
    work[(1-(1))+ _work_offset] = (double)(1);
if (wantq)  {
    if ((m >= k))  {
    com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,k,a,_a_offset,lda,tau,_tau_offset,work,_work_offset,-1,iinfo);
}
else  {
  if ((m > 1))  {
    com.polytechnik.lapack310.Dorgqr.dorgqr((m-1),(m-1),(m-1),a,_a_offset,lda,tau,_tau_offset,work,_work_offset,-1,iinfo);
}
}              //  Close else.
}
else  {
  if ((k < n))  {
    com.polytechnik.lapack310.Dorglq.dorglq(m,n,k,a,_a_offset,lda,tau,_tau_offset,work,_work_offset,-1,iinfo);
}
else  {
  if ((n > 1))  {
    com.polytechnik.lapack310.Dorglq.dorglq((n-1),(n-1),(n-1),a,_a_offset,lda,tau,_tau_offset,work,_work_offset,-1,iinfo);
}
}              //  Close else.
}              //  Close else.
lwkopt = (int)(work[(1-(1))+ _work_offset]);
lwkopt = Math.max(lwkopt, mn);
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DORGBR",(-(info.val)));
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorgbr",999999);
}
else if (lquery)  {
    work[(1-(1))+ _work_offset] = (double)(lwkopt);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorgbr",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
if (((m == 0) || (n == 0)))  {
    work[(1-(1))+ _work_offset] = (double)(1);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorgbr",999999);
// *
}
/***
*
*        Form Q, determined by a call to DGEBRD to reduce an m-by-k
*        matrix
*
***/
if (wantq)  {
    /***
*
*           If m >= k, assume m >= n >= k
*
***/
if ((m >= k))  {
    // *
com.polytechnik.lapack310.Dorgqr.dorgqr(m,n,k,a,_a_offset,lda,tau,_tau_offset,work,_work_offset,lwork,iinfo);
/***
*
*           If m < k, assume m = n
*
*           Shift the vectors which define the elementary reflectors one
*           column to the right, and set the first row and column of Q
*           to those of the unit matrix
*
***/
}
else  {
  {
int _j_inc = -1;
for (j = m; j >= 2; j += _j_inc) {
a[(1-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
{
for (i = (j+1); i <= m; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = a[(i-(1))+((j-1)-(1)) * (lda)+ _a_offset];
//Dummy.label("com/polytechnik/lapack310/Dorgbr",10);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dorgbr",20);
}              //  Close for() loop.
}
a[(1-(1))+(1-(1)) * (lda)+ _a_offset] = 1.0;
{
for (i = 2; i <= m; i++) {
a[(i-(1))+(1-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dorgbr",30);
}              //  Close for() loop.
}
/***
*
*              Form Q(2:m,2:m)
*
***/
if ((m > 1))  {
    com.polytechnik.lapack310.Dorgqr.dorgqr((m-1),(m-1),(m-1),a,(2-(1))+(2-(1)) * (lda)+ _a_offset,lda,tau,_tau_offset,work,_work_offset,lwork,iinfo);
}
}              //  Close else.
/***
*
*        Form P**T, determined by a call to DGEBRD to reduce a k-by-n
*        matrix
*
***/
}
else  {
  /***
*
*           If k < n, assume k <= m <= n
*
***/
if ((k < n))  {
    // *
com.polytechnik.lapack310.Dorglq.dorglq(m,n,k,a,_a_offset,lda,tau,_tau_offset,work,_work_offset,lwork,iinfo);
/***
*
*           If k >= n, assume m = n
*
*           Shift the vectors which define the elementary reflectors one
*           row downward, and set the first row and column of P**T to
*           those of the unit matrix
*
***/
}
else  {
  a[(1-(1))+(1-(1)) * (lda)+ _a_offset] = 1.0;
{
for (i = 2; i <= n; i++) {
a[(i-(1))+(1-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dorgbr",40);
}              //  Close for() loop.
}
{
for (j = 2; j <= n; j++) {
{
int _i_inc = -1;
for (i = (j-1); i >= 2; i += _i_inc) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = a[((i-1)-(1))+(j-(1)) * (lda)+ _a_offset];
//Dummy.label("com/polytechnik/lapack310/Dorgbr",50);
}              //  Close for() loop.
}
a[(1-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dorgbr",60);
}              //  Close for() loop.
}
/***
*
*              Form P**T(2:n,2:n)
*
***/
if ((n > 1))  {
    com.polytechnik.lapack310.Dorglq.dorglq((n-1),(n-1),(n-1),a,(2-(1))+(2-(1)) * (lda)+ _a_offset,lda,tau,_tau_offset,work,_work_offset,lwork,iinfo);
}
}              //  Close else.
}              //  Close else.
work[(1-(1))+ _work_offset] = (double)(lwkopt);
/***
*
*     End of DORGBR
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorgbr",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dorgbr",999999);
return;
   }
} // End class.
