/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dorgl2.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dorgl2 {

// *> \brief \b DORGL2
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DORGL2 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DORGL2( M, N, K, A, LDA, TAU, WORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, K, LDA, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DORGL2 generates an m by n real matrix Q with orthonormal rows,
// *> which is defined as the first m rows of a product of k elementary
// *> reflectors of order n
// *>
// *>       Q  =  H(k) . . . H(2) H(1)
// *>
// *> as returned by DGELQF.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix Q. M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix Q. N >= M.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>          The number of elementary reflectors whose product defines th
// *>          matrix Q. M >= K >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the i-th row must contain the vector which defines
// *>          the elementary reflector H(i), for i = 1,2,...,k, as returne
// *>          by DGELQF in the first k rows of its array argument A.
// *>          On exit, the m-by-n matrix Q.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The first dimension of the array A. LDA >= max(1,M).
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (K)
// *>          TAU(i) must contain the scalar factor of the elementary
// *>          reflector H(i), as returned by DGELQF.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (M)
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0: successful exit
// *>          < 0: if INFO = -i, the i-th argument has an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dorgl2 (int m,
int n,
int k,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
intW info)  {

label_999999:{

int i= 0;
int j= 0;
int l= 0;
info.val = 0;
if ((m < 0))  {
    info.val = -1;
}
else if ((n < m))  {
    info.val = -2;
}              // Close else if()
else if (((k < 0) || (k > m)))  {
    info.val = -3;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info.val = -5;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DORGL2",(-(info.val)));
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorgl2",999999);
/***
*
*     Quick return if possible
*
***/
}
// *
if ((m <= 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorgl2",999999);
}
    /***
*
*        Initialise rows k+1:m to rows of the unit matrix
*
***/
if ((k < m))  {
    {
for (j = 1; j <= n; j++) {
{
for (l = (k+1); l <= m; l++) {
a[(l-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dorgl2",10);
}              //  Close for() loop.
}
if (((j > k) && (j <= m))) {
    a[(j-(1))+(j-(1)) * (lda)+ _a_offset] = 1.0;
}
//Dummy.label("com/polytechnik/lapack310/Dorgl2",20);
}              //  Close for() loop.
}
// *
}
/***
*
*        Apply H(i) to A(i:m,i:n) from the right
*
***/
{
int _i_inc = -1;
for (i = k; i >= 1; i += _i_inc) {
if ((i < n))  {
    if ((i < m))  {
    a[(i-(1))+(i-(1)) * (lda)+ _a_offset] = 1.0;
com.polytechnik.lapack310.Dlarf.dlarf("Right",(m-i),((n-i)+1),a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,tau[(i-(1))+ _tau_offset],a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,lda,work,_work_offset);
}
com.polytechnik.lapack310.Dscal.dscal((n-i),(-(tau[(i-(1))+ _tau_offset])),a,(i-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda);
}
/***
*
*        Set A(i,1:i-1) to zero
*
***/
a[(i-(1))+(i-(1)) * (lda)+ _a_offset] = (1.0-tau[(i-(1))+ _tau_offset]);
{
for (l = 1; l <= (i-1); l++) {
a[(i-(1))+(l-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dorgl2",30);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dorgl2",40);
}              //  Close for() loop.
}
/***
*
*     End of DORGL2
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorgl2",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dorgl2",999999);
return;
   }
} // End class.
