/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dorglq.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dorglq {

// *> \brief \b DORGLQ
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DORGLQ + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DORGLQ( M, N, K, A, LDA, TAU, WORK, LWORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, K, LDA, LWORK, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DORGLQ generates an M-by-N real matrix Q with orthonormal rows,
// *> which is defined as the first M rows of a product of K elementary
// *> reflectors of order N
// *>
// *>       Q  =  H(k) . . . H(2) H(1)
// *>
// *> as returned by DGELQF.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix Q. M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix Q. N >= M.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>          The number of elementary reflectors whose product defines th
// *>          matrix Q. M >= K >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the i-th row must contain the vector which defines
// *>          the elementary reflector H(i), for i = 1,2,...,k, as returne
// *>          by DGELQF in the first k rows of its array argument A.
// *>          On exit, the M-by-N matrix Q.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The first dimension of the array A. LDA >= max(1,M).
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (K)
// *>          TAU(i) must contain the scalar factor of the elementary
// *>          reflector H(i), as returned by DGELQF.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK. LWORK >= max(1,M).
// *>          For optimum performance LWORK >= M*NB, where NB is
// *>          the optimal blocksize.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument has an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dorglq (int m,
int n,
int k,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

label_999999:{

boolean lquery= false;
int i= 0;
int ib= 0;
intW iinfo= new intW(0);
int iws= 0;
int j= 0;
int ki= 0;
int kk= 0;
int l= 0;
int ldwork= 0;
int lwkopt= 0;
int nb= 0;
int nbmin= 0;
int nx= 0;
info.val = 0;
nb = 1;//com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DORGLQ"," ",m,n,k,-1);
lwkopt = (Math.max(1, m)*nb);
work[(1-(1))+ _work_offset] = (double)(lwkopt);
lquery = ((lwork == -1));
if ((m < 0))  {
    info.val = -1;
}
else if ((n < m))  {
    info.val = -2;
}              // Close else if()
else if (((k < 0) || (k > m)))  {
    info.val = -3;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info.val = -5;
}              // Close else if()
else if (((lwork < Math.max(1, m)) && (!lquery)))  {
    info.val = -8;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DORGLQ",(-(info.val)));
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorglq",999999);
}
else if (lquery)  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorglq",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
if ((m <= 0))  {
    work[(1-(1))+ _work_offset] = (double)(1);
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorglq",999999);
// *
}
nbmin = 2;
nx = 0;
iws = m;
/***
*
*        Determine when to cross over from blocked to unblocked code.
*
***/
if (((nb > 1) && (nb < k)))  {
    nx = Math.max(0, 0/*com.polytechnik.lapack310.Ilaenv.ilaenv(3,"DORGLQ"," ",m,n,k,-1)*/);
/***
*
*           Determine if workspace is large enough for blocked code.
*
***/
if ((nx < k))  {
    ldwork = m;
iws = (ldwork*nb);
/***
*
*              Not enough workspace to use optimal NB:  reduce NB and
*              determine the minimum value of NB.
*
***/
if ((lwork < iws))  {
    nb = (lwork/ldwork);
    nbmin = Math.max(2, 2/*com.polytechnik.lapack310.Ilaenv.ilaenv(2,"DORGLQ"," ",m,n,k,-1)*/);
}
}
// *
}
/***
*
*        Use blocked code after the last block.
*        The first kk rows are handled by the block method.
*
***/
if ((((nb >= nbmin) && (nb < k)) && (nx < k)))  {
    ki = ((((((k-nx)-1))/nb))*nb);
/***
*
*        Set A(kk+1:m,1:kk) to zero.
*
***/
kk = Math.min(k, (ki+nb));
{
for (j = 1; j <= kk; j++) {
{
for (i = (kk+1); i <= m; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dorglq",10);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dorglq",20);
}              //  Close for() loop.
}
}
else  {
  kk = 0;
/***
*
*     Use unblocked code for the last or only block.
*
***/
}              //  Close else.
// *
if ((kk < m)) {
    com.polytechnik.lapack310.Dorgl2.dorgl2((m-kk),(n-kk),(k-kk),a,((kk+1)-(1))+((kk+1)-(1)) * (lda)+ _a_offset,lda,tau,((kk+1)-(1))+ _tau_offset,work,_work_offset,iinfo);
}
    /***
*
*        Use blocked code
*
***/
if ((kk > 0))  {
    {
int _i_inc = (-(nb));
for (i = (ki+1); (_i_inc < 0) ? i >= 1 : i <= 1; i += _i_inc) {
ib = Math.min(nb, ((k-i)+1));
/***
*
*              Form the triangular factor of the block reflector
*              H = H(i) H(i+1) . . . H(i+ib-1)
*
***/
if (((i+ib) <= m))  {
    /***
*
*              Apply H**T to A(i+ib:m,i:n) from the right
*
***/
com.polytechnik.lapack310.Dlarft.dlarft("Forward","Rowwise",((n-i)+1),ib,a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,tau,(i-(1))+ _tau_offset,work,_work_offset,ldwork);
com.polytechnik.lapack310.Dlarfb.dlarfb("Right","Transpose","Forward","Rowwise",(((m-i)-ib)+1),((n-i)+1),ib,a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,work,_work_offset,ldwork,a,((i+ib)-(1))+(i-(1)) * (lda)+ _a_offset,lda,work,((ib+1)-(1))+ _work_offset,ldwork);
/***
*
*           Apply H**T to columns i:n of current block
*
***/
}
/***
*
*           Set columns 1:i-1 of current block to zero
*
***/
com.polytechnik.lapack310.Dorgl2.dorgl2(ib,((n-i)+1),ib,a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,tau,(i-(1))+ _tau_offset,work,_work_offset,iinfo);
{
for (j = 1; j <= (i-1); j++) {
{
for (l = i; l <= ((i+ib)-1); l++) {
a[(l-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dorglq",30);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dorglq",40);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dorglq",50);
}              //  Close for() loop.
}
// *
}
work[(1-(1))+ _work_offset] = (double)(iws);
/***
*
*     End of DORGLQ
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorglq",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dorglq",999999);
return;
   }
} // End class.
