/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dorgtr.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dorgtr {

// *> \brief \b DORGTR
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DORGTR + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DORGTR( UPLO, N, A, LDA, TAU, WORK, LWORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            INFO, LDA, LWORK, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DORGTR generates a real orthogonal matrix Q which is defined as the
// *> product of n-1 elementary reflectors of order N, as returned by
// *> DSYTRD:
// *>
// *> if UPLO = 'U', Q = H(n-1) . . . H(2) H(1),
// *>
// *> if UPLO = 'L', Q = H(1) H(2) . . . H(n-1).
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          = 'U': Upper triangle of A contains elementary reflectors
// *>                 from DSYTRD;
// *>          = 'L': Lower triangle of A contains elementary reflectors
// *>                 from DSYTRD.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix Q. N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the vectors which define the elementary reflectors
// *>          as returned by DSYTRD.
// *>          On exit, the N-by-N orthogonal matrix Q.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A. LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (N-1)
// *>          TAU(i) must contain the scalar factor of the elementary
// *>          reflector H(i), as returned by DSYTRD.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK. LWORK >= max(1,N-1).
// *>          For optimum performance LWORK >= (N-1)*NB, where NB is
// *>          the optimal blocksize.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dorgtr (String uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

label_999999:{

boolean lquery= false;
boolean upper= false;
int i= 0;
intW iinfo= new intW(0);
int j= 0;
int lwkopt= 0;
int nb= 0;
uplo = uplo.substring((1)-1,1);
info.val = 0;
lquery = ((lwork == -1));
upper = com.polytechnik.lapack310.Lsame.lsame(uplo,"U");
if (((!upper) && (!com.polytechnik.lapack310.Lsame.lsame(uplo,"L"))))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -4;
}              // Close else if()
else if (((lwork < Math.max(1, (n-1))) && (!lquery)))  {
    info.val = -7;
// *
}              // Close else if()
if ((info.val == 0))  {
    if (upper)  {
	nb = 1;//com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DORGQL"," ",(n-1),(n-1),(n-1),-1);
}
else  {
    nb = 1;//com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DORGQR"," ",(n-1),(n-1),(n-1),-1);
}              //  Close else.
lwkopt = (Math.max(1, (n-1))*nb);
work[(1-(1))+ _work_offset] = (double)(lwkopt);
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DORGTR",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorgtr",999999);
}
else if (lquery)  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorgtr",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
if ((n == 0))  {
    work[(1-(1))+ _work_offset] = (double)(1);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorgtr",999999);
// *
}
/***
*
*        Q was determined by a call to DSYTRD with UPLO = 'U'
*
*        Shift the vectors which define the elementary reflectors one
*        column to the left, and set the last row and column of Q to
*        those of the unit matrix
*
***/
if (upper)  {
    {
for (j = 1; j <= (n-1); j++) {
{
for (i = 1; i <= (j-1); i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = a[(i-(1))+((j+1)-(1)) * (lda)+ _a_offset];
//Dummy.label("com/polytechnik/lapack310/Dorgtr",10);
}              //  Close for() loop.
}
a[(n-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dorgtr",20);
}              //  Close for() loop.
}
{
for (i = 1; i <= (n-1); i++) {
a[(i-(1))+(n-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dorgtr",30);
}              //  Close for() loop.
}
/***
*
*        Generate Q(1:n-1,1:n-1)
*
***/
a[(n-(1))+(n-(1)) * (lda)+ _a_offset] = 1.0;
// *
com.polytechnik.lapack310.Dorgql.dorgql((n-1),(n-1),(n-1),a,_a_offset,lda,tau,_tau_offset,work,_work_offset,lwork,iinfo);
/***
*
*        Q was determined by a call to DSYTRD with UPLO = 'L'.
*
*        Shift the vectors which define the elementary reflectors one
*        column to the right, and set the first row and column of Q to
*        those of the unit matrix
*
***/
}
else  {
  {
int _j_inc = -1;
for (j = n; j >= 2; j += _j_inc) {
a[(1-(1))+(j-(1)) * (lda)+ _a_offset] = 0.0;
{
for (i = (j+1); i <= n; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = a[(i-(1))+((j-1)-(1)) * (lda)+ _a_offset];
//Dummy.label("com/polytechnik/lapack310/Dorgtr",40);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dorgtr",50);
}              //  Close for() loop.
}
a[(1-(1))+(1-(1)) * (lda)+ _a_offset] = 1.0;
{
for (i = 2; i <= n; i++) {
a[(i-(1))+(1-(1)) * (lda)+ _a_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dorgtr",60);
}              //  Close for() loop.
}
/***
*
*           Generate Q(2:n,2:n)
*
***/
if ((n > 1))  {
    com.polytechnik.lapack310.Dorgqr.dorgqr((n-1),(n-1),(n-1),a,(2-(1))+(2-(1)) * (lda)+ _a_offset,lda,tau,_tau_offset,work,_work_offset,lwork,iinfo);
}
}              //  Close else.
work[(1-(1))+ _work_offset] = (double)(lwkopt);
/***
*
*     End of DORGTR
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dorgtr",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dorgtr",999999);
return;
   }
} // End class.
