/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dormbr.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dormbr {

// *> \brief \b DORMBR
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DORMBR + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DORMBR( VECT, SIDE, TRANS, M, N, K, A, LDA, TAU, C,
// *                          LDC, WORK, LWORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          SIDE, TRANS, VECT
// *       INTEGER            INFO, K, LDA, LDC, LWORK, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), C( LDC, * ), TAU( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> If VECT = 'Q', DORMBR overwrites the general real M-by-N matrix C
// *> with
// *>                 SIDE = 'L'     SIDE = 'R'
// *> TRANS = 'N':      Q * C          C * Q
// *> TRANS = 'T':      Q**T * C       C * Q**T
// *>
// *> If VECT = 'P', DORMBR overwrites the general real M-by-N matrix C
// *> with
// *>                 SIDE = 'L'     SIDE = 'R'
// *> TRANS = 'N':      P * C          C * P
// *> TRANS = 'T':      P**T * C       C * P**T
// *>
// *> Here Q and P**T are the orthogonal matrices determined by DGEBRD when
// *> reducing a real matrix A to bidiagonal form: A = Q * B * P**T. Q and

// *> P**T are defined as products of elementary reflectors H(i) and G(i)
// *> respectively.
// *>
// *> Let nq = m if SIDE = 'L' and nq = n if SIDE = 'R'. Thus nq is the
// *> order of the orthogonal matrix Q or P**T that is applied.
// *>
// *> If VECT = 'Q', A is assumed to have been an NQ-by-K matrix:
// *> if nq >= k, Q = H(1) H(2) . . . H(k);
// *> if nq < k, Q = H(1) H(2) . . . H(nq-1).
// *>
// *> If VECT = 'P', A is assumed to have been a K-by-NQ matrix:
// *> if k < nq, P = G(1) G(2) . . . G(k);
// *> if k >= nq, P = G(1) G(2) . . . G(nq-1).
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] VECT
// *> \verbatim
// *>          VECT is CHARACTER*1
// *>          = 'Q': apply Q or Q**T;
// *>          = 'P': apply P or P**T.
// *> \endverbatim
// *>
// *> \param[in] SIDE
// *> \verbatim
// *>          SIDE is CHARACTER*1
// *>          = 'L': apply Q, Q**T, P or P**T from the Left;
// *>          = 'R': apply Q, Q**T, P or P**T from the Right.
// *> \endverbatim
// *>
// *> \param[in] TRANS
// *> \verbatim
// *>          TRANS is CHARACTER*1
// *>          = 'N':  No transpose, apply Q  or P;
// *>          = 'T':  Transpose, apply Q**T or P**T.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix C. M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix C. N >= 0.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>          If VECT = 'Q', the number of columns in the original
// *>          matrix reduced by DGEBRD.
// *>          If VECT = 'P', the number of rows in the original
// *>          matrix reduced by DGEBRD.
// *>          K >= 0.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension
// *>                                (LDA,min(nq,K)) if VECT = 'Q'
// *>                                (LDA,nq)        if VECT = 'P'
// *>          The vectors which define the elementary reflectors H(i) and

// *>          G(i), whose products determine the matrices Q and P, as
// *>          returned by DGEBRD.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.
// *>          If VECT = 'Q', LDA >= max(1,nq);
// *>          if VECT = 'P', LDA >= max(1,min(nq,K)).
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (min(nq,K))
// *>          TAU(i) must contain the scalar factor of the elementary
// *>          reflector H(i) or G(i) which determines Q or P, as returned

// *>          by DGEBRD in the array argument TAUQ or TAUP.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension (LDC,N)
// *>          On entry, the M-by-N matrix C.
// *>          On exit, C is overwritten by Q*C or Q**T*C or C*Q**T or C*Q

// *>          or P*C or P**T*C or C*P or C*P**T.
// *> \endverbatim
// *>
// *> \param[in] LDC
// *> \verbatim
// *>          LDC is INTEGER
// *>          The leading dimension of the array C. LDC >= max(1,M).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK.
// *>          If SIDE = 'L', LWORK >= max(1,N);
// *>          if SIDE = 'R', LWORK >= max(1,M).
// *>          For optimum performance LWORK >= N*NB if SIDE = 'L', and
// *>          LWORK >= M*NB if SIDE = 'R', where NB is the optimal
// *>          blocksize.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dormbr (String vect,
String side,
String trans,
int m,
int n,
int k,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] c, int _c_offset,
int ldc,
double [] work, int _work_offset,
int lwork,
intW info)  {

label_999999:{

boolean applyq= false;
boolean left= false;
boolean lquery= false;
boolean notran= false;
String transt= new String(" ");
int i1= 0;
int i2= 0;
intW iinfo= new intW(0);
int lwkopt= 0;
int mi= 0;
int nb= 0;
int ni= 0;
int nq= 0;
int nw= 0;
vect = vect.substring((1)-1,1);
side = side.substring((1)-1,1);
trans = trans.substring((1)-1,1);
info.val = 0;
applyq = com.polytechnik.lapack310.Lsame.lsame(vect,"Q");
left = com.polytechnik.lapack310.Lsame.lsame(side,"L");
notran = com.polytechnik.lapack310.Lsame.lsame(trans,"N");
/***
*
*     NQ is the order of Q or P and NW is the minimum dimension of WORK
*
***/
lquery = ((lwork == -1));
if (left)  {
    nq = m;
nw = Math.max(1, n);
}
else  {
  nq = n;
nw = Math.max(1, m);
}              //  Close else.
if (((!applyq) && (!com.polytechnik.lapack310.Lsame.lsame(vect,"P"))))  {
    info.val = -1;
}
else if (((!left) && (!com.polytechnik.lapack310.Lsame.lsame(side,"R"))))  {
    info.val = -2;
}              // Close else if()
else if (((!notran) && (!com.polytechnik.lapack310.Lsame.lsame(trans,"T"))))  {
    info.val = -3;
}              // Close else if()
else if ((m < 0))  {
    info.val = -4;
}              // Close else if()
else if ((n < 0))  {
    info.val = -5;
}              // Close else if()
else if ((k < 0))  {
    info.val = -6;
}              // Close else if()
else if ((((applyq && (lda < Math.max(1, nq)))) || (((!applyq) && (lda < Math.max(1, Math.min(nq, k)))))))  {
    info.val = -8;
}              // Close else if()
else if ((ldc < Math.max(1, m)))  {
    info.val = -11;
}              // Close else if()
else if (((lwork < nw) && (!lquery)))  {
    info.val = -13;
// *
}              // Close else if()
if ((info.val == 0))  {
    if (applyq)  {
    if (left)  {
	nb = 1;//com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DORMQR",(side+trans),(m-1),n,(m-1),-1);
}
else  {
    nb = 1;//com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DORMQR",(side+trans),m,(n-1),(n-1),-1);
}              //  Close else.
}
else  {
  if (left)  {
      nb = 1;//com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DORMLQ",(side+trans),(m-1),n,(m-1),-1);
}
else  {
    nb = 1;//com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DORMLQ",(side+trans),m,(n-1),(n-1),-1);
}              //  Close else.
}              //  Close else.
lwkopt = (nw*nb);
work[(1-(1))+ _work_offset] = (double)(lwkopt);
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DORMBR",(-(info.val)));
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dormbr",999999);
}
else if (lquery)  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dormbr",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
work[(1-(1))+ _work_offset] = (double)(1);
// *
if (((m == 0) || (n == 0))) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dormbr",999999);
}
    /***
*
*        Apply Q
*
***/
if (applyq)  {
    /***
*
*           Q was determined by a call to DGEBRD with nq >= k
*
***/
if ((nq >= k))  {
    com.polytechnik.lapack310.Dormqr.dormqr(side,trans,m,n,k,a,_a_offset,lda,tau,_tau_offset,c,_c_offset,ldc,work,_work_offset,lwork,iinfo);
/***
*
*           Q was determined by a call to DGEBRD with nq < k
*
***/
}
else if ((nq > 1))  {
    if (left)  {
    mi = (m-1);
ni = n;
i1 = 2;
i2 = 1;
}
else  {
  mi = m;
ni = (n-1);
i1 = 1;
i2 = 2;
}              //  Close else.
com.polytechnik.lapack310.Dormqr.dormqr(side,trans,mi,ni,(nq-1),a,(2-(1))+(1-(1)) * (lda)+ _a_offset,lda,tau,_tau_offset,c,(i1-(1))+(i2-(1)) * (ldc)+ _c_offset,ldc,work,_work_offset,lwork,iinfo);
}              // Close else if()
/***
*
*        Apply P
*
***/
}
else  {
  if (notran)  {
    transt = "T";
}
else  {
  transt = "N";
}              //  Close else.
/***
*
*           P was determined by a call to DGEBRD with nq > k
*
***/
if ((nq > k))  {
    com.polytechnik.lapack310.Dormlq.dormlq(side,transt,m,n,k,a,_a_offset,lda,tau,_tau_offset,c,_c_offset,ldc,work,_work_offset,lwork,iinfo);
/***
*
*           P was determined by a call to DGEBRD with nq <= k
*
***/
}
else if ((nq > 1))  {
    if (left)  {
    mi = (m-1);
ni = n;
i1 = 2;
i2 = 1;
}
else  {
  mi = m;
ni = (n-1);
i1 = 1;
i2 = 2;
}              //  Close else.
com.polytechnik.lapack310.Dormlq.dormlq(side,transt,mi,ni,(nq-1),a,(1-(1))+(2-(1)) * (lda)+ _a_offset,lda,tau,_tau_offset,c,(i1-(1))+(i2-(1)) * (ldc)+ _c_offset,ldc,work,_work_offset,lwork,iinfo);
}              // Close else if()
}              //  Close else.
work[(1-(1))+ _work_offset] = (double)(lwkopt);
/***
*
*     End of DORMBR
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dormbr",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dormbr",999999);
return;
   }
} // End class.
