/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dormqr.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dormqr {

// *> \brief \b DORMQR
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DORMQR + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DORMQR( SIDE, TRANS, M, N, K, A, LDA, TAU, C, LDC,
// *                          WORK, LWORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          SIDE, TRANS
// *       INTEGER            INFO, K, LDA, LDC, LWORK, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), C( LDC, * ), TAU( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DORMQR overwrites the general real M-by-N matrix C with
// *>
// *>                 SIDE = 'L'     SIDE = 'R'
// *> TRANS = 'N':      Q * C          C * Q
// *> TRANS = 'T':      Q**T * C       C * Q**T
// *>
// *> where Q is a real orthogonal matrix defined as the product of k
// *> elementary reflectors
// *>
// *>       Q = H(1) H(2) . . . H(k)
// *>
// *> as returned by DGEQRF. Q is of order M if SIDE = 'L' and of order N
// *> if SIDE = 'R'.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] SIDE
// *> \verbatim
// *>          SIDE is CHARACTER*1
// *>          = 'L': apply Q or Q**T from the Left;
// *>          = 'R': apply Q or Q**T from the Right.
// *> \endverbatim
// *>
// *> \param[in] TRANS
// *> \verbatim
// *>          TRANS is CHARACTER*1
// *>          = 'N':  No transpose, apply Q;
// *>          = 'T':  Transpose, apply Q**T.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix C. M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix C. N >= 0.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>          The number of elementary reflectors whose product defines
// *>          the matrix Q.
// *>          If SIDE = 'L', M >= K >= 0;
// *>          if SIDE = 'R', N >= K >= 0.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,K)
// *>          The i-th column must contain the vector which defines the
// *>          elementary reflector H(i), for i = 1,2,...,k, as returned by
// *>          DGEQRF in the first k columns of its array argument A.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.
// *>          If SIDE = 'L', LDA >= max(1,M);
// *>          if SIDE = 'R', LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (K)
// *>          TAU(i) must contain the scalar factor of the elementary
// *>          reflector H(i), as returned by DGEQRF.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension (LDC,N)
// *>          On entry, the M-by-N matrix C.
// *>          On exit, C is overwritten by Q*C or Q**T*C or C*Q**T or C*Q.
// *> \endverbatim
// *>
// *> \param[in] LDC
// *> \verbatim
// *>          LDC is INTEGER
// *>          The leading dimension of the array C. LDC >= max(1,M).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK.
// *>          If SIDE = 'L', LWORK >= max(1,N);
// *>          if SIDE = 'R', LWORK >= max(1,M).
// *>          For good performance, LWORK should generally be larger.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dormqr (String side,
String trans,
int m,
int n,
int k,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] c, int _c_offset,
int ldc,
double [] work, int _work_offset,
int lwork,
intW info)  {

label_999999:{

boolean left= false;
boolean lquery= false;
boolean notran= false;
int i= 0;
int i1= 0;
int i2= 0;
int i3= 0;
int ib= 0;
int ic= 0;
intW iinfo= new intW(0);
int iwt= 0;
int jc= 0;
int ldwork= 0;
int lwkopt= 0;
int mi= 0;
int nb= 0;
int nbmin= 0;
int ni= 0;
int nq= 0;
int nw= 0;
side = side.substring((1)-1,1);
trans = trans.substring((1)-1,1);
info.val = 0;
left = com.polytechnik.lapack310.Lsame.lsame(side,"L");
notran = com.polytechnik.lapack310.Lsame.lsame(trans,"N");
/***
*
*     NQ is the order of Q and NW is the minimum dimension of WORK
*
***/
lquery = ((lwork == -1));
if (left)  {
    nq = m;
nw = Math.max(1, n);
}
else  {
  nq = n;
nw = Math.max(1, m);
}              //  Close else.
if (((!left) && (!com.polytechnik.lapack310.Lsame.lsame(side,"R"))))  {
    info.val = -1;
}
else if (((!notran) && (!com.polytechnik.lapack310.Lsame.lsame(trans,"T"))))  {
    info.val = -2;
}              // Close else if()
else if ((m < 0))  {
    info.val = -3;
}              // Close else if()
else if ((n < 0))  {
    info.val = -4;
}              // Close else if()
else if (((k < 0) || (k > nq)))  {
    info.val = -5;
}              // Close else if()
else if ((lda < Math.max(1, nq)))  {
    info.val = -7;
}              // Close else if()
else if ((ldc < Math.max(1, m)))  {
    info.val = -10;
}              // Close else if()
else if (((lwork < nw) && (!lquery)))  {
    info.val = -12;
// *
}              // Close else if()
/***
*
*        Compute the workspace requirements
*
***/
if ((info.val == 0))  {
    nb = Math.min(64, 1/*com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DORMQR",(side+trans),m,n,k,-1)*/);
lwkopt = ((nw*nb)+4160);
work[(1-(1))+ _work_offset] = (double)(lwkopt);
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DORMQR",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dormqr",999999);
}
else if (lquery)  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dormqr",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
if ((((m == 0) || (n == 0)) || (k == 0)))  {
    work[(1-(1))+ _work_offset] = (double)(1);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dormqr",999999);
// *
}
nbmin = 2;
ldwork = nw;
if (((nb > 1) && (nb < k)))  {
    if ((lwork < lwkopt))  {
    nb = (((lwork-4160))/ldwork);
    nbmin = Math.max(2, 2/*com.polytechnik.lapack310.Ilaenv.ilaenv(2,"DORMQR",(side+trans),m,n,k,-1)*/);
}
// *
}
/***
*
*        Use unblocked code
*
***/
if (((nb < nbmin) || (nb >= k)))  {
    com.polytechnik.lapack310.Dorm2r.dorm2r(side,trans,m,n,k,a,_a_offset,lda,tau,_tau_offset,c,_c_offset,ldc,work,_work_offset,iinfo);
/***
*
*        Use blocked code
*
***/
}
else  {
  iwt = (1+(nw*nb));
if ((((left && (!notran))) || (((!left) && notran))))  {
    i1 = 1;
i2 = k;
i3 = nb;
}
else  {
  i1 = ((((((k-1))/nb))*nb)+1);
i2 = 1;
i3 = (-(nb));
// *
}              //  Close else.
if (left)  {
    ni = n;
jc = 1;
}
else  {
  mi = m;
ic = 1;
// *
}              //  Close else.
{
int _i_inc = i3;
for (i = i1; (_i_inc < 0) ? i >= i2 : i <= i2; i += _i_inc) {
/***
*
*           Form the triangular factor of the block reflector
*           H = H(i) H(i+1) . . . H(i+ib-1)
*
***/
ib = Math.min(nb, ((k-i)+1));
com.polytechnik.lapack310.Dlarft.dlarft("Forward","Columnwise",((nq-i)+1),ib,a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,tau,(i-(1))+ _tau_offset,work,(iwt-(1))+ _work_offset,65);
/***
*
*              H or H**T is applied to C(i:m,1:n)
*
***/
if (left)  {
    mi = ((m-i)+1);
ic = i;
/***
*
*              H or H**T is applied to C(1:m,i:n)
*
***/
}
else  {
  ni = ((n-i)+1);
jc = i;
/***
*
*           Apply H or H**T
*
***/
}              //  Close else.
com.polytechnik.lapack310.Dlarfb.dlarfb(side,trans,"Forward","Columnwise",mi,ni,ib,a,(i-(1))+(i-(1)) * (lda)+ _a_offset,lda,work,(iwt-(1))+ _work_offset,65,c,(ic-(1))+(jc-(1)) * (ldc)+ _c_offset,ldc,work,_work_offset,ldwork);
//Dummy.label("com/polytechnik/lapack310/Dormqr",10);
}              //  Close for() loop.
}
}              //  Close else.
work[(1-(1))+ _work_offset] = (double)(lwkopt);
/***
*
*     End of DORMQR
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dormqr",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dormqr",999999);
return;
   }
} // End class.
