/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dpotrf.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dpotrf {

// *> \brief \b DPOTRF
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DPOTRF + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DPOTRF( UPLO, N, A, LDA, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            INFO, LDA, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DPOTRF computes the Cholesky factorization of a real symmetric
// *> positive definite matrix A.
// *>
// *> The factorization has the form
// *>    A = U**T * U,  if UPLO = 'U', or
// *>    A = L  * L**T,  if UPLO = 'L',
// *> where U is an upper triangular matrix and L is lower triangular.
// *>
// *> This is the block version of the algorithm, calling Level 3 BLAS.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          = 'U':  Upper triangle of A is stored;
// *>          = 'L':  Lower triangle of A is stored.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the leadin
// *>          N-by-N upper triangular part of A contains the upper
// *>          triangular part of the matrix A, and the strictly lower
// *>          triangular part of A is not referenced.  If UPLO = 'L', the

// *>          leading N-by-N lower triangular part of A contains the lower
// *>          triangular part of the matrix A, and the strictly upper
// *>          triangular part of A is not referenced.
// *>
// *>          On exit, if INFO = 0, the factor U or L from the Cholesky
// *>          factorization A = U**T*U or A = L*L**T.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  if INFO = i, the leading minor of order i is not
// *>                positive definite, and the factorization could not be

// *>                completed.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doublePOcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dpotrf (String uplo,
int n,
double [] a, int _a_offset,
int lda,
intW info)  {

label_999999:{

boolean upper= false;
int j= 0;
int jb= 0;
int nb= 0;
uplo = uplo.substring((1)-1,1);
info.val = 0;
upper = com.polytechnik.lapack310.Lsame.lsame(uplo,"U");
if (((!upper) && (!com.polytechnik.lapack310.Lsame.lsame(uplo,"L"))))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -4;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DPOTRF",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf",999999);
/***
*
*     Quick return if possible
*
***/
}
/***
*
*     Determine the block size for this environment.
*
***/
if ((n == 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf",999999);
}
nb = 1;//com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DPOTRF",uplo,n,-1,-1,-1);
label40:{
label30:{
/***
*
*        Use unblocked code.
*
***/
if (((nb <= 1) || (nb >= n)))  {
    com.polytechnik.lapack310.Dpotrf2.dpotrf2(uplo,n,a,_a_offset,lda,info);
/***
*
*        Use blocked code.
*
***/
}
else  {
  /***
*
*           Compute the Cholesky factorization A = U**T*U.
*
***/
if (upper)  {
    /***
*
*              Update and factorize the current diagonal block and test
*              for non-positive-definiteness.
*
***/
{
int _j_inc = nb;
for (j = 1; (_j_inc < 0) ? j >= n : j <= n; j += _j_inc) {
jb = Math.min(nb, ((n-j)+1));
com.polytechnik.lapack310.Dsyrk.dsyrk("Upper","Transpose",jb,(j-1),-1.0,a,(1-(1))+(j-(1)) * (lda)+ _a_offset,lda,1.0,a,(j-(1))+(j-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dpotrf2.dpotrf2("Upper",jb,a,(j-(1))+(j-(1)) * (lda)+ _a_offset,lda,info);
if ((info.val != 0)) {
    break label30;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf",30);
}
    /***
*
*                 Compute the current block row.
*
***/
if (((j+jb) <= n))  {
    com.polytechnik.lapack310.Dgemm.dgemm("Transpose","No transpose",jb,(((n-j)-jb)+1),(j-1),-1.0,a,(1-(1))+(j-(1)) * (lda)+ _a_offset,lda,a,(1-(1))+((j+jb)-(1)) * (lda)+ _a_offset,lda,1.0,a,(j-(1))+((j+jb)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dtrsm.dtrsm("Left","Upper","Transpose","Non-unit",jb,(((n-j)-jb)+1),1.0,a,(j-(1))+(j-(1)) * (lda)+ _a_offset,lda,a,(j-(1))+((j+jb)-(1)) * (lda)+ _a_offset,lda);
}
// *
//Dummy.label("com/polytechnik/lapack310/Dpotrf",10);
}              //  Close for() loop.
}
/***
*
*           Compute the Cholesky factorization A = L*L**T.
*
***/
}
else  {
  /***
*
*              Update and factorize the current diagonal block and test
*              for non-positive-definiteness.
*
***/
{
int _j_inc = nb;
for (j = 1; (_j_inc < 0) ? j >= n : j <= n; j += _j_inc) {
jb = Math.min(nb, ((n-j)+1));
com.polytechnik.lapack310.Dsyrk.dsyrk("Lower","No transpose",jb,(j-1),-1.0,a,(j-(1))+(1-(1)) * (lda)+ _a_offset,lda,1.0,a,(j-(1))+(j-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dpotrf2.dpotrf2("Lower",jb,a,(j-(1))+(j-(1)) * (lda)+ _a_offset,lda,info);
if ((info.val != 0)) {
    break label30;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf",30);
}
    /***
*
*                 Compute the current block column.
*
***/
if (((j+jb) <= n))  {
    com.polytechnik.lapack310.Dgemm.dgemm("No transpose","Transpose",(((n-j)-jb)+1),jb,(j-1),-1.0,a,((j+jb)-(1))+(1-(1)) * (lda)+ _a_offset,lda,a,(j-(1))+(1-(1)) * (lda)+ _a_offset,lda,1.0,a,((j+jb)-(1))+(j-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dtrsm.dtrsm("Right","Lower","Transpose","Non-unit",(((n-j)-jb)+1),jb,1.0,a,(j-(1))+(j-(1)) * (lda)+ _a_offset,lda,a,((j+jb)-(1))+(j-(1)) * (lda)+ _a_offset,lda);
}
//Dummy.label("com/polytechnik/lapack310/Dpotrf",20);
}              //  Close for() loop.
}
}              //  Close else.
}              //  Close else.
// *
break label40;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf",40);
} // END label30
    //Dummy.label("com/polytechnik/lapack310/Dpotrf",30);
// *
info.val = ((info.val+j)-1);
break label40;}// END label40
//Dummy.label("com/polytechnik/lapack310/Dpotrf",40);
/***
*
*     End of DPOTRF
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dpotrf",999999);
return;
   }
} // End class.
