/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dpotrf2.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dpotrf2 {

// *> \brief \b DPOTRF2
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *  Definition:
// *  ===========
// *
// *       RECURSIVE SUBROUTINE DPOTRF2( UPLO, N, A, LDA, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            INFO, LDA, N
// *       ..
// *       .. Array Arguments ..
// *       REAL               A( LDA, * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DPOTRF2 computes the Cholesky factorization of a real symmetric
// *> positive definite matrix A using the recursive algorithm.
// *>
// *> The factorization has the form
// *>    A = U**T * U,  if UPLO = 'U', or
// *>    A = L  * L**T,  if UPLO = 'L',
// *> where U is an upper triangular matrix and L is lower triangular.
// *>
// *> This is the recursive version of the algorithm. It divides
// *> the matrix into four submatrices:
// *>
// *>        [  A11 | A12  ]  where A11 is n1 by n1 and A22 is n2 by n2
// *>    A = [ -----|----- ]  with n1 = n/2
// *>        [  A21 | A22  ]       n2 = n-n1
// *>
// *> The subroutine calls itself to factor A11. Update and scale A21
// *> or A12, update A22 then calls itself to factor A22.
// *>
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          = 'U':  Upper triangle of A is stored;
// *>          = 'L':  Lower triangle of A is stored.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the leadin
// *>          N-by-N upper triangular part of A contains the upper
// *>          triangular part of the matrix A, and the strictly lower
// *>          triangular part of A is not referenced.  If UPLO = 'L', the

// *>          leading N-by-N lower triangular part of A contains the lower
// *>          triangular part of the matrix A, and the strictly upper
// *>          triangular part of A is not referenced.
// *>
// *>          On exit, if INFO = 0, the factor U or L from the Cholesky
// *>          factorization A = U**T*U or A = L*L**T.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  if INFO = i, the leading minor of order i is not
// *>                positive definite, and the factorization could not be

// *>                completed.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doublePOcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters
*
***/

public static void dpotrf2 (String uplo,
int n,
double [] a, int _a_offset,
int lda,
intW info)  {

label_999999:{

boolean upper= false;
int n1= 0;
int n2= 0;
intW iinfo= new intW(0);
uplo = uplo.substring((1)-1,1);
info.val = 0;
upper = com.polytechnik.lapack310.Lsame.lsame(uplo,"U");
if (((!upper) && (!com.polytechnik.lapack310.Lsame.lsame(uplo,"L"))))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -4;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DPOTRF2",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf2",999999);
/***
*
*     Quick return if possible
*
***/
}
/***
*
*     N=1 case
*
***/
if ((n == 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf2",999999);
}
    /***
*
*        Test for non-positive-definiteness
*
***/
if ((n == 1))  {
    if (((a[(1-(1))+(1-(1)) * (lda)+ _a_offset] <= 0.0) || com.polytechnik.lapack310.Disnan.disnan(a[(1-(1))+(1-(1)) * (lda)+ _a_offset])))  {
    info.val = 1;
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf2",999999);
/***
*
*        Factor
*
***/
}
/***
*
*     Use recursive code
*
***/
a[(1-(1))+(1-(1)) * (lda)+ _a_offset] = Math.sqrt(a[(1-(1))+(1-(1)) * (lda)+ _a_offset]);
}
else  {
  n1 = (n/2);
/***
*
*        Factor A11
*
***/
n2 = (n-n1);
com.polytechnik.lapack310.Dpotrf2.dpotrf2(uplo,n1,a,(1-(1))+(1-(1)) * (lda)+ _a_offset,lda,iinfo);
if ((iinfo.val != 0))  {
    info.val = iinfo.val;
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf2",999999);
/***
*
*        Compute the Cholesky factorization A = U**T*U
*
***/
}
/***
*
*           Update and scale A12
*
***/
if (upper)  {
    /***
*
*           Update and factor A22
*
***/
com.polytechnik.lapack310.Dtrsm.dtrsm("L","U","T","N",n1,n2,1.0,a,(1-(1))+(1-(1)) * (lda)+ _a_offset,lda,a,(1-(1))+((n1+1)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsyrk.dsyrk(uplo,"T",n2,n1,-1.0,a,(1-(1))+((n1+1)-(1)) * (lda)+ _a_offset,lda,1.0,a,((n1+1)-(1))+((n1+1)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dpotrf2.dpotrf2(uplo,n2,a,((n1+1)-(1))+((n1+1)-(1)) * (lda)+ _a_offset,lda,iinfo);
if ((iinfo.val != 0))  {
    info.val = (iinfo.val+n1);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf2",999999);
/***
*
*        Compute the Cholesky factorization A = L*L**T
*
***/
}
/***
*
*           Update and scale A21
*
***/
}
else  {
  /***
*
*           Update and factor A22
*
***/
com.polytechnik.lapack310.Dtrsm.dtrsm("R","L","T","N",n2,n1,1.0,a,(1-(1))+(1-(1)) * (lda)+ _a_offset,lda,a,((n1+1)-(1))+(1-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsyrk.dsyrk(uplo,"N",n2,n1,-1.0,a,((n1+1)-(1))+(1-(1)) * (lda)+ _a_offset,lda,1.0,a,((n1+1)-(1))+((n1+1)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dpotrf2.dpotrf2(uplo,n2,a,((n1+1)-(1))+((n1+1)-(1)) * (lda)+ _a_offset,lda,iinfo);
if ((iinfo.val != 0))  {
    info.val = (iinfo.val+n1);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf2",999999);
}
}              //  Close else.
}              //  Close else.
/***
*
*     End of DPOTRF2
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dpotrf2",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dpotrf2",999999);
return;
   }
} // End class.
