/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dscal.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dscal {

// *> \brief \b DSCAL
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSCAL(N,DA,DX,INCX)
// *
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION DA
// *       INTEGER INCX,N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION DX(*)
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *>    DSCAL scales a vector by a constant.
// *>    uses unrolled loops for increment equal to 1.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>         number of elements in input vector(s)
// *> \endverbatim
// *>
// *> \param[in] DA
// *> \verbatim
// *>          DA is DOUBLE PRECISION
// *>           On entry, DA specifies the scalar alpha.
// *> \endverbatim
// *>
// *> \param[in,out] DX
// *> \verbatim
// *>          DX is DOUBLE PRECISION array, dimension ( 1 + ( N - 1 )*abs(
// *> \endverbatim
// *>
// *> \param[in] INCX
// *> \verbatim
// *>          INCX is INTEGER
// *>         storage spacing between elements of DX
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup double_blas_level1
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>     jack dongarra, linpack, 3/11/78.
// *>     modified 3/93 to return if incx .le. 0.
// *>     modified 12/3/93, array(1) declarations changed to array(*)
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
// *     ..

public static void dscal (int n,
double da,
double [] dx, int _dx_offset,
int incx)  {

label_999999:{

int i= 0;
int m= 0;
int mp1= 0;
int nincx= 0;
if (((n <= 0) || (incx <= 0))) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dscal",999999);
}
    /***
*
*        code for increment equal to 1
*
*
*        clean-up loop
*
***/
if ((incx == 1))  {
    m = (n)%(5);
if ((m != 0))  {
    {
for (i = 1; i <= m; i++) {
dx[(i-(1))+ _dx_offset] = (da*dx[(i-(1))+ _dx_offset]);
//Dummy.label("com/polytechnik/lapack310/Dscal",1000000);
}              //  Close for() loop.
}
if ((n < 5)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dscal",999999);
}
    }
mp1 = (m+1);
{
int _i_inc = 5;
for (i = mp1; i <= n; i += _i_inc) {
dx[(i-(1))+ _dx_offset] = (da*dx[(i-(1))+ _dx_offset]);
dx[((i+1)-(1))+ _dx_offset] = (da*dx[((i+1)-(1))+ _dx_offset]);
dx[((i+2)-(1))+ _dx_offset] = (da*dx[((i+2)-(1))+ _dx_offset]);
dx[((i+3)-(1))+ _dx_offset] = (da*dx[((i+3)-(1))+ _dx_offset]);
dx[((i+4)-(1))+ _dx_offset] = (da*dx[((i+4)-(1))+ _dx_offset]);
//Dummy.label("com/polytechnik/lapack310/Dscal",1000001);
}              //  Close for() loop.
}
/***
*
*        code for increment not equal to 1
*
***/
}
else  {
  nincx = (n*incx);
{
int _i_inc = incx;
for (i = 1; (_i_inc < 0) ? i >= nincx : i <= nincx; i += _i_inc) {
dx[(i-(1))+ _dx_offset] = (da*dx[(i-(1))+ _dx_offset]);
//Dummy.label("com/polytechnik/lapack310/Dscal",1000002);
}              //  Close for() loop.
}
}              //  Close else.
/***
*
*     End of DSCAL
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dscal",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dscal",999999);
return;
   }
} // End class.
