/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dstedc.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dstedc {

// *> \brief \b DSTEDC
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DSTEDC + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSTEDC( COMPZ, N, D, E, Z, LDZ, WORK, LWORK, IWORK,
// *                          LIWORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          COMPZ
// *       INTEGER            INFO, LDZ, LIWORK, LWORK, N
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IWORK( * )
// *       DOUBLE PRECISION   D( * ), E( * ), WORK( * ), Z( LDZ, * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSTEDC computes all eigenvalues and, optionally, eigenvectors of a
// *> symmetric tridiagonal matrix using the divide and conquer method.
// *> The eigenvectors of a full or band real symmetric matrix can also be

// *> found if DSYTRD or DSPTRD or DSBTRD has been used to reduce this
// *> matrix to tridiagonal form.
// *>
// *> This code makes very mild assumptions about floating point
// *> arithmetic. It will work on machines with a guard digit in
// *> add/subtract, or on those binary machines without guard digits
// *> which subtract like the Cray X-MP, Cray Y-MP, Cray C-90, or Cray-2.
// *> It could conceivably fail on hexadecimal or decimal machines
// *> without guard digits, but we know of none.  See DLAED3 for details.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] COMPZ
// *> \verbatim
// *>          COMPZ is CHARACTER*1
// *>          = 'N':  Compute eigenvalues only.
// *>          = 'I':  Compute eigenvectors of tridiagonal matrix also.
// *>          = 'V':  Compute eigenvectors of original dense symmetric
// *>                  matrix also.  On entry, Z contains the orthogonal
// *>                  matrix used to reduce the original matrix to
// *>                  tridiagonal form.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The dimension of the symmetric tridiagonal matrix.  N >= 0.

// *> \endverbatim
// *>
// *> \param[in,out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the diagonal elements of the tridiagonal matrix.
// *>          On exit, if INFO = 0, the eigenvalues in ascending order.
// *> \endverbatim
// *>
// *> \param[in,out] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N-1)
// *>          On entry, the subdiagonal elements of the tridiagonal matrix
// *>          On exit, E has been destroyed.
// *> \endverbatim
// *>
// *> \param[in,out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (LDZ,N)
// *>          On entry, if COMPZ = 'V', then Z contains the orthogonal
// *>          matrix used in the reduction to tridiagonal form.
// *>          On exit, if INFO = 0, then if COMPZ = 'V', Z contains the
// *>          orthonormal eigenvectors of the original symmetric matrix,
// *>          and if COMPZ = 'I', Z contains the orthonormal eigenvectors

// *>          of the symmetric tridiagonal matrix.
// *>          If  COMPZ = 'N', then Z is not referenced.
// *> \endverbatim
// *>
// *> \param[in] LDZ
// *> \verbatim
// *>          LDZ is INTEGER
// *>          The leading dimension of the array Z.  LDZ >= 1.
// *>          If eigenvectors are desired, then LDZ >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK.
// *>          If COMPZ = 'N' or N <= 1 then LWORK must be at least 1.
// *>          If COMPZ = 'V' and N > 1 then LWORK must be at least
// *>                         ( 1 + 3*N + 2*N*lg N + 4*N**2 ),
// *>                         where lg( N ) = smallest integer k such
// *>                         that 2**k >= N.
// *>          If COMPZ = 'I' and N > 1 then LWORK must be at least
// *>                         ( 1 + 4*N + N**2 ).
// *>          Note that for COMPZ = 'I' or 'V', then if N is less than or

// *>          equal to the minimum divide size, usually 25, then LWORK nee
// *>          only be max(1,2*(N-1)).
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (MAX(1,LIWORK))
// *>          On exit, if INFO = 0, IWORK(1) returns the optimal LIWORK.
// *> \endverbatim
// *>
// *> \param[in] LIWORK
// *> \verbatim
// *>          LIWORK is INTEGER
// *>          The dimension of the array IWORK.
// *>          If COMPZ = 'N' or N <= 1 then LIWORK must be at least 1.
// *>          If COMPZ = 'V' and N > 1 then LIWORK must be at least
// *>                         ( 6 + 6*N + 5*N*lg N ).
// *>          If COMPZ = 'I' and N > 1 then LIWORK must be at least
// *>                         ( 3 + 5*N ).
// *>          Note that for COMPZ = 'I' or 'V', then if N is less than or

// *>          equal to the minimum divide size, usually 25, then LIWORK
// *>          need only be 1.
// *>
// *>          If LIWORK = -1, then a workspace query is assumed; the
// *>          routine only calculates the optimal size of the IWORK array,
// *>          returns this value as the first entry of the IWORK array, an
// *>          no error message related to LIWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit.
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value.

// *>          > 0:  The algorithm failed to compute an eigenvalue while
// *>                working on the submatrix lying in rows and columns
// *>                INFO/(N+1) through mod(INFO,N+1).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Jeff Rutter, Computer Science Division, University of California
// *> at Berkeley, USA \n
// *>  Modified by Francoise Tisseur, University of Tennessee
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dstedc (String compz,
int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
int lwork,
int [] iwork, int _iwork_offset,
int liwork,
intW info)  {

label_999999:{

boolean lquery= false;
int finish= 0;
int i= 0;
int icompz= 0;
int ii= 0;
int j= 0;
int k= 0;
int lgn= 0;
int liwmin= 0;
int lwmin= 0;
int m= 0;
int smlsiz= 0;
int start= 0;
int storez= 0;
int strtrw= 0;
double eps= 0.0d;
double orgnrm= 0.0d;
double p= 0.0d;
double tiny= 0.0d;
compz = compz.substring((1)-1,1);
info.val = 0;
// *
lquery = (((lwork == -1) || (liwork == -1)));
if (com.polytechnik.lapack310.Lsame.lsame(compz,"N"))  {
    icompz = 0;
}
else if (com.polytechnik.lapack310.Lsame.lsame(compz,"V"))  {
    icompz = 1;
}              // Close else if()
else if (com.polytechnik.lapack310.Lsame.lsame(compz,"I"))  {
    icompz = 2;
}              // Close else if()
else  {
  icompz = -1;
}              //  Close else.
if ((icompz < 0))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((((ldz < 1)) || (((icompz > 0) && (ldz < Math.max(1, n))))))  {
    info.val = -6;
// *
}              // Close else if()
/***
*
*        Compute the workspace requirements
*
***/
if ((info.val == 0))  {
    smlsiz = 25;//com.polytechnik.lapack310.Ilaenv.ilaenv(9,"DSTEDC"," ",0,0,0,0);
if (((n <= 1) || (icompz == 0)))  {
    liwmin = 1;
lwmin = 1;
}
else if ((n <= smlsiz))  {
    liwmin = 1;
lwmin = (2*((n-1)));
}              // Close else if()
else  {
  lgn = (int)((Math.log((double)(n))/Math.log(2.0)));
if ((((int)Math.pow(2, lgn)) < n)) {
    lgn = (lgn+1);
}
    if ((((int)Math.pow(2, lgn)) < n)) {
    lgn = (lgn+1);
}
    if ((icompz == 1))  {
    lwmin = (((1+(3*n))+((2*n)*lgn))+(4*((int)Math.pow(n, 2))));
liwmin = ((6+(6*n))+((5*n)*lgn));
}
else if ((icompz == 2))  {
    lwmin = ((1+(4*n))+((int)Math.pow(n, 2)));
liwmin = (3+(5*n));
}              // Close else if()
}              //  Close else.
work[(1-(1))+ _work_offset] = (double)(lwmin);
// *
iwork[(1-(1))+ _iwork_offset] = liwmin;
if (((lwork < lwmin) && (!lquery)))  {
    info.val = -8;
}
else if (((liwork < liwmin) && (!lquery)))  {
    info.val = -10;
}              // Close else if()
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DSTEDC",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",999999);
}
else if (lquery)  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
if ((n == 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",999999);
}
    if ((n == 1))  {
    if ((icompz != 0)) {
    z[(1-(1))+(1-(1)) * (ldz)+ _z_offset] = 1.0;
}
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",999999);
/***
*
*     If the following conditional clause is removed, then the routine
*     will use the Divide and Conquer routine to compute only the
*     eigenvalues, which requires (3N + 3N**2) real workspace and
*     (2 + 5N + 2N lg(N)) integer workspace.
*     Since on many architectures DSTERF is much faster than any other
*     algorithm for finding eigenvalues only, it is used here
*     as the default. If the conditional clause is removed, then
*     information on the size of workspace needs to be changed.
*
*     If COMPZ = 'N', use DSTERF to compute the eigenvalues.
*
***/
}
label50:{
if ((icompz == 0))  {
    com.polytechnik.lapack310.Dsterf.dsterf(n,d,_d_offset,e,_e_offset,info);
    break label50;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",50);
/***
*
*     If N is smaller than the minimum divide size (SMLSIZ+1), then
*     solve the problem with another solver.
*
***/
}
// *
if ((n <= smlsiz))  {
    // *
com.polytechnik.lapack310.Dsteqr.dsteqr(compz,n,d,_d_offset,e,_e_offset,z,_z_offset,ldz,work,_work_offset,info);
/***
*
*        If COMPZ = 'V', the Z matrix must be stored elsewhere for later
*        use.
*
***/
}
else  {
  if ((icompz == 1))  {
    storez = (1+(n*n));
}
else  {
  storez = 1;
// *
}              //  Close else.
if ((icompz == 2))  {
    com.polytechnik.lapack310.Dlaset.dlaset("Full",n,n,0.0,1.0,z,_z_offset,ldz);
/***
*
*        Scale.
*
***/
}
orgnrm = com.polytechnik.lapack310.Dlanst.dlanst("M",n,d,_d_offset,e,_e_offset);
// *
if ((orgnrm == 0.0)) {
    break label50;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",50);
}
    // *
eps = com.polytechnik.lapack310.Dlamch.dlamch("Epsilon");
/***
*
*        while ( START <= N )
*
***/
start = 1;
label10:for(int jjx=0;jjx<Util.ITMAX;jjx++){
    //Dummy.label("com/polytechnik/lapack310/Dstedc",10);
/***
*
*           Let FINISH be the position of the next subdiagonal entry
*           such that E( FINISH ) <= TINY or FINISH = N if no such
*           subdiagonal exists.  The matrix identified by the elements
*           between START and FINISH constitutes an independent
*           sub-problem.
*
***/
if ((start <= n))  {
    finish = start;
label20:for(int jjxx=0;jjxx<Util.ITMAX;jjxx++){
	//Dummy.label("com/polytechnik/lapack310/Dstedc",20);
if ((finish < n))  {
    tiny = ((eps*Math.sqrt(Math.abs(d[(finish-(1))+ _d_offset])))*Math.sqrt(Math.abs(d[((finish+1)-(1))+ _d_offset])));
if ((Math.abs(e[(finish-(1))+ _e_offset]) > tiny))  {
    finish = (finish+1);
    continue label20;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",20);
}
/***
*
*           (Sub) Problem determined.  Compute its size and solve it.
*
***/
}
break label20;}// END label20
m = ((finish-start)+1);
if ((m == 1))  {
    start = (finish+1);
    continue label10;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",10);
}
/***
*
*              Scale.
*
***/
if ((m > smlsiz))  {
    orgnrm = com.polytechnik.lapack310.Dlanst.dlanst("M",m,d,(start-(1))+ _d_offset,e,(start-(1))+ _e_offset);
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,orgnrm,1.0,m,1,d,(start-(1))+ _d_offset,m,info);
// *
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,orgnrm,1.0,(m-1),1,e,(start-(1))+ _e_offset,(m-1),info);
if ((icompz == 1))  {
    strtrw = 1;
}
else  {
  strtrw = start;
}              //  Close else.
com.polytechnik.lapack310.Dlaed0.dlaed0(icompz,n,m,d,(start-(1))+ _d_offset,e,(start-(1))+ _e_offset,z,(strtrw-(1))+(start-(1)) * (ldz)+ _z_offset,ldz,work,(1-(1))+ _work_offset,n,work,(storez-(1))+ _work_offset,iwork,_iwork_offset,info);
if ((info.val != 0))  {
    info.val = ((((((((info.val/((m+1)))+start)-1))*((n+1)))+(info.val)%(((m+1))))+start)-1);
break label50;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",50);
/***
*
*              Scale back.
*
***/
}
// *
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,1.0,orgnrm,m,1,d,(start-(1))+ _d_offset,m,info);
}
else  {
  /***
*
*                 Since QR won't update a Z matrix which is larger than
*                 the length of D, we must solve the sub-problem in a
*                 workspace and then multiply back into Z.
*
***/
if ((icompz == 1))  {
    com.polytechnik.lapack310.Dsteqr.dsteqr("I",m,d,(start-(1))+ _d_offset,e,(start-(1))+ _e_offset,work,_work_offset,m,work,(((m*m)+1)-(1))+ _work_offset,info);
com.polytechnik.lapack310.Dlacpy.dlacpy("A",n,m,z,(1-(1))+(start-(1)) * (ldz)+ _z_offset,ldz,work,(storez-(1))+ _work_offset,n);
com.polytechnik.lapack310.Dgemm.dgemm("N","N",n,m,m,1.0,work,(storez-(1))+ _work_offset,n,work,_work_offset,m,0.0,z,(1-(1))+(start-(1)) * (ldz)+ _z_offset,ldz);
}
else if ((icompz == 2))  {
    com.polytechnik.lapack310.Dsteqr.dsteqr("I",m,d,(start-(1))+ _d_offset,e,(start-(1))+ _e_offset,z,(start-(1))+(start-(1)) * (ldz)+ _z_offset,ldz,work,_work_offset,info);
}              // Close else if()
else  {
  com.polytechnik.lapack310.Dsterf.dsterf(m,d,(start-(1))+ _d_offset,e,(start-(1))+ _e_offset,info);
}              //  Close else.
if ((info.val != 0))  {
    info.val = ((start*((n+1)))+finish);
break label50;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",50);
}
// *
}              //  Close else.
start = (finish+1);
continue label10;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",10);
/***
*
*        endwhile
*
***/
}
break label10;}// END label10
/***
*
*          Use Quick Sort
*
***/
if ((icompz == 0))  {
    // *
com.polytechnik.lapack310.Dlasrt.dlasrt("I",n,d,_d_offset,info);
/***
*
*          Use Selection Sort to minimize swaps of eigenvectors
*
***/
}
else  {
  {
for (ii = 2; ii <= n; ii++) {
i = (ii-1);
k = i;
p = d[(i-(1))+ _d_offset];
{
for (j = ii; j <= n; j++) {
if ((d[(j-(1))+ _d_offset] < p))  {
    k = j;
p = d[(j-(1))+ _d_offset];
}
//Dummy.label("com/polytechnik/lapack310/Dstedc",30);
}              //  Close for() loop.
}
if ((k != i))  {
    d[(k-(1))+ _d_offset] = d[(i-(1))+ _d_offset];
d[(i-(1))+ _d_offset] = p;
com.polytechnik.lapack310.Dswap.dswap(n,z,(1-(1))+(i-(1)) * (ldz)+ _z_offset,1,z,(1-(1))+(k-(1)) * (ldz)+ _z_offset,1);
}
//Dummy.label("com/polytechnik/lapack310/Dstedc",40);
}              //  Close for() loop.
}
}              //  Close else.
// *
}              //  Close else.
break label50;}// END label50
    //Dummy.label("com/polytechnik/lapack310/Dstedc",50);
work[(1-(1))+ _work_offset] = (double)(lwmin);
// *
iwork[(1-(1))+ _iwork_offset] = liwmin;
/***
*
*     End of DSTEDC
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dstedc",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dstedc",999999);
return;
   }
} // End class.
