/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsteqr.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dsteqr {

// *> \brief \b DSTEQR
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DSTEQR + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSTEQR( COMPZ, N, D, E, Z, LDZ, WORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          COMPZ
// *       INTEGER            INFO, LDZ, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   D( * ), E( * ), WORK( * ), Z( LDZ, * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSTEQR computes all eigenvalues and, optionally, eigenvectors of a
// *> symmetric tridiagonal matrix using the implicit QL or QR method.
// *> The eigenvectors of a full or band symmetric matrix can also be found
// *> if DSYTRD or DSPTRD or DSBTRD has been used to reduce this matrix to

// *> tridiagonal form.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] COMPZ
// *> \verbatim
// *>          COMPZ is CHARACTER*1
// *>          = 'N':  Compute eigenvalues only.
// *>          = 'V':  Compute eigenvalues and eigenvectors of the original
// *>                  symmetric matrix.  On entry, Z must contain the
// *>                  orthogonal matrix used to reduce the original matrix
// *>                  to tridiagonal form.
// *>          = 'I':  Compute eigenvalues and eigenvectors of the
// *>                  tridiagonal matrix.  Z is initialized to the identit
// *>                  matrix.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          On entry, the diagonal elements of the tridiagonal matrix.
// *>          On exit, if INFO = 0, the eigenvalues in ascending order.
// *> \endverbatim
// *>
// *> \param[in,out] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N-1)
// *>          On entry, the (n-1) subdiagonal elements of the tridiagonal

// *>          matrix.
// *>          On exit, E has been destroyed.
// *> \endverbatim
// *>
// *> \param[in,out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (LDZ, N)
// *>          On entry, if  COMPZ = 'V', then Z contains the orthogonal
// *>          matrix used in the reduction to tridiagonal form.
// *>          On exit, if INFO = 0, then if  COMPZ = 'V', Z contains the
// *>          orthonormal eigenvectors of the original symmetric matrix,
// *>          and if COMPZ = 'I', Z contains the orthonormal eigenvectors

// *>          of the symmetric tridiagonal matrix.
// *>          If COMPZ = 'N', then Z is not referenced.
// *> \endverbatim
// *>
// *> \param[in] LDZ
// *> \verbatim
// *>          LDZ is INTEGER
// *>          The leading dimension of the array Z.  LDZ >= 1, and if
// *>          eigenvectors are desired, then  LDZ >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (max(1,2*N-2))
// *>          If COMPZ = 'N', then WORK is not referenced.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  the algorithm has failed to find all the eigenvalues i
// *>                a total of 30*N iterations; if INFO = i, then i
// *>                elements of E have not converged to zero; on exit, D
// *>                and E contain the elements of a symmetric tridiagonal

// *>                matrix which is orthogonally similar to the original
// *>                matrix.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup auxOTHERcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/
// *

public static void dsteqr (String compz,
int n,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
intW info)  {

label_999999:{

int i= 0;
int icompz= 0;
int ii= 0;
int iscale= 0;
int j= 0;
int jtot= 0;
int k= 0;
int l= 0;
int l1= 0;
int lend= 0;
int lendm1= 0;
int lendp1= 0;
int lendsv= 0;
int lm1= 0;
int lsv= 0;
int m= 0;
int mm= 0;
int mm1= 0;
int nm1= 0;
int nmaxit= 0;
double anorm= 0.0d;
double b= 0.0d;
doubleW c= new doubleW(0.0d);
double eps= 0.0d;
double eps2= 0.0d;
double f= 0.0d;
double g= 0.0d;
double p= 0.0d;
doubleW r= new doubleW(0.0d);
doubleW rt1= new doubleW(0.0d);
doubleW rt2= new doubleW(0.0d);
doubleW s= new doubleW(0.0d);
double safmax= 0.0d;
double safmin= 0.0d;
double ssfmax= 0.0d;
double ssfmin= 0.0d;
double tst= 0.0d;
compz = compz.substring((1)-1,1);
info.val = 0;
if (com.polytechnik.lapack310.Lsame.lsame(compz,"N"))  {
    icompz = 0;
}
else if (com.polytechnik.lapack310.Lsame.lsame(compz,"V"))  {
    icompz = 1;
}              // Close else if()
else if (com.polytechnik.lapack310.Lsame.lsame(compz,"I"))  {
    icompz = 2;
}              // Close else if()
else  {
  icompz = -1;
}              //  Close else.
if ((icompz < 0))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((((ldz < 1)) || (((icompz > 0) && (ldz < Math.max(1, n))))))  {
    info.val = -6;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DSTEQR",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",999999);
/***
*
*     Quick return if possible
*
***/
}
// *
if ((n == 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",999999);
}
    if ((n == 1))  {
    if ((icompz == 2)) {
    z[(1-(1))+(1-(1)) * (ldz)+ _z_offset] = 1.0;
}
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",999999);
/***
*
*     Determine the unit roundoff and over/underflow thresholds.
*
***/
}
eps = com.polytechnik.lapack310.Dlamch.dlamch("E");
eps2 = ( Math.pow(eps, 2));
safmin = com.polytechnik.lapack310.Dlamch.dlamch("S");
safmax = (1.0/safmin);
ssfmax = (Math.sqrt(safmax)/3.0);
/***
*
*     Compute the eigenvalues and eigenvectors of the tridiagonal
*     matrix.
*
***/
ssfmin = (Math.sqrt(safmin)/eps2);
// *
if ((icompz == 2)) {
    com.polytechnik.lapack310.Dlaset.dlaset("Full",n,n,0.0,1.0,z,_z_offset,ldz);
}
    nmaxit = (n*30);
/***
*
*     Determine where the matrix splits and choose QL or QR iteration
*     for each block, according to whether top or bottom diagonal
*     element is smaller.
*
***/
jtot = 0;
l1 = 1;
// *
nm1 = (n-1);
label190:{
label160:{
label10:for(int jjx=0;jjx<Util.ITMAX;jjx++){
label140:{
label30:{
	    //Dummy.label("com/polytechnik/lapack310/Dsteqr",10);
if ((l1 > n)) {
    break label160;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",160);
}
    if ((l1 > 1)) {
    e[((l1-1)-(1))+ _e_offset] = 0.0;
}
    if ((l1 <= nm1))  {
    {
for (m = l1; m <= nm1; m++) {
tst = Math.abs(e[(m-(1))+ _e_offset]);
if ((tst == 0.0)) {
    break label30;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",30);
}
    if ((tst <= (((Math.sqrt(Math.abs(d[(m-(1))+ _d_offset]))*Math.sqrt(Math.abs(d[((m+1)-(1))+ _d_offset]))))*eps)))  {
    e[(m-(1))+ _e_offset] = 0.0;
    break label30;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",30);
}
//Dummy.label("com/polytechnik/lapack310/Dsteqr",20);
}              //  Close for() loop.
}
}
// *
m = n;
break label30;}//END label30
//Dummy.label("com/polytechnik/lapack310/Dsteqr",30);
l = l1;
lsv = l;
lend = m;
lendsv = lend;
l1 = (m+1);
/***
*
*     Scale submatrix in rows and columns L to LEND
*
***/
if ((lend == l)) {
    continue label10;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",10);
}
    anorm = com.polytechnik.lapack310.Dlanst.dlanst("M",((lend-l)+1),d,(l-(1))+ _d_offset,e,(l-(1))+ _e_offset);
iscale = 0;
if ((anorm == 0.0)) {
    continue label10;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",10);
}
    if ((anorm > ssfmax))  {
    iscale = 1;
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,anorm,ssfmax,((lend-l)+1),1,d,(l-(1))+ _d_offset,n,info);
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,anorm,ssfmax,(lend-l),1,e,(l-(1))+ _e_offset,n,info);
}
else if ((anorm < ssfmin))  {
    iscale = 2;
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,anorm,ssfmin,((lend-l)+1),1,d,(l-(1))+ _d_offset,n,info);
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,anorm,ssfmin,(lend-l),1,e,(l-(1))+ _e_offset,n,info);
/***
*
*     Choose between QL and QR iteration
*
***/
}              // Close else if()
if ((Math.abs(d[(lend-(1))+ _d_offset]) < Math.abs(d[(l-(1))+ _d_offset])))  {
    lend = lsv;
l = lendsv;
// *
}
/***
*
*        QL Iteration
*
*        Look for small subdiagonal element.
*
***/
if ((lend > l))  {
label40:for(int jjxxx=0;jjxxx<Util.ITMAX;jjxxx++){
	//Dummy.label("com/polytechnik/lapack310/Dsteqr",40);
label80:{
label60:{
if ((l != lend))  {
    lendm1 = (lend-1);
{
for (m = l; m <= lendm1; m++) {
tst = ( Math.pow(Math.abs(e[(m-(1))+ _e_offset]), 2));
if ((tst <= ((((eps2*Math.abs(d[(m-(1))+ _d_offset])))*Math.abs(d[((m+1)-(1))+ _d_offset]))+safmin))) {
    break label60;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",60);
}
//Dummy.label("com/polytechnik/lapack310/Dsteqr",50);
}              //  Close for() loop.
}
// *
}
// *
m = lend;
break label60;}// END label60
	//Dummy.label("com/polytechnik/lapack310/Dsteqr",60);
if ((m < lend)) {
    e[(m-(1))+ _e_offset] = 0.0;
}
    p = d[(l-(1))+ _d_offset];
/***
*
*        If remaining matrix is 2-by-2, use DLAE2 or SLAEV2
*        to compute its eigensystem.
*
***/
if ((m == l)) {
    break label80;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",80);
}
    if ((m == (l+1)))  {
    if ((icompz > 0))  {
    com.polytechnik.lapack310.Dlaev2.dlaev2(d[(l-(1))+ _d_offset],e[(l-(1))+ _e_offset],d[((l+1)-(1))+ _d_offset],rt1,rt2,c,s);
work[(l-(1))+ _work_offset] = c.val;
work[(((n-1)+l)-(1))+ _work_offset] = s.val;
com.polytechnik.lapack310.Dlasr.dlasr("R","V","B",n,2,work,(l-(1))+ _work_offset,work,(((n-1)+l)-(1))+ _work_offset,z,(1-(1))+(l-(1)) * (ldz)+ _z_offset,ldz);
}
else  {
  com.polytechnik.lapack310.Dlae2.dlae2(d[(l-(1))+ _d_offset],e[(l-(1))+ _e_offset],d[((l+1)-(1))+ _d_offset],rt1,rt2);
}              //  Close else.
d[(l-(1))+ _d_offset] = rt1.val;
d[((l+1)-(1))+ _d_offset] = rt2.val;
e[(l-(1))+ _e_offset] = 0.0;
l = (l+2);
if ((l <= lend)) {
    continue label40;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",40);
}
break label140;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",140);
// *
}
if ((jtot == nmaxit)) {
    break label140;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",140);
}
    /***
*
*        Form shift.
*
***/
jtot = (jtot+1);
g = (((d[((l+1)-(1))+ _d_offset]-p))/((2.0*e[(l-(1))+ _e_offset])));
r.val = com.polytechnik.lapack310.Dlapy2.dlapy2(g,1.0);
// *
g = ((d[(m-(1))+ _d_offset]-p)+((e[(l-(1))+ _e_offset]/((g+Util.dsign(r.val,g))))));
s.val = 1.0;
c.val = 1.0;
/***
*
*        Inner loop
*
***/
p = 0.0;
mm1 = (m-1);
{
int _i_inc = -1;
for (i = mm1; i >= l; i += _i_inc) {
f = (s.val*e[(i-(1))+ _e_offset]);
b = (c.val*e[(i-(1))+ _e_offset]);
com.polytechnik.lapack310.Dlartg.dlartg(g,f,c,s,r);
if ((i != (m-1))) {
    e[((i+1)-(1))+ _e_offset] = r.val;
}
    g = (d[((i+1)-(1))+ _d_offset]-p);
r.val = ((((d[(i-(1))+ _d_offset]-g))*s.val)+((2.0*c.val)*b));
p = (s.val*r.val);
d[((i+1)-(1))+ _d_offset] = (g+p);
/***
*
*           If eigenvectors are desired, then save rotations.
*
***/
g = ((c.val*r.val)-b);
if ((icompz > 0))  {
    work[(i-(1))+ _work_offset] = c.val;
work[(((n-1)+i)-(1))+ _work_offset] = (-(s.val));
// *
}
/***
*
*        If eigenvectors are desired, then apply saved rotations.
*
***/
//Dummy.label("com/polytechnik/lapack310/Dsteqr",70);
}              //  Close for() loop.
}
if ((icompz > 0))  {
    mm = ((m-l)+1);
com.polytechnik.lapack310.Dlasr.dlasr("R","V","B",n,mm,work,(l-(1))+ _work_offset,work,(((n-1)+l)-(1))+ _work_offset,z,(1-(1))+(l-(1)) * (ldz)+ _z_offset,ldz);
// *
}
d[(l-(1))+ _d_offset] = (d[(l-(1))+ _d_offset]-p);
e[(l-(1))+ _e_offset] = g;
/***
*
*        Eigenvalue found.
*
***/
continue label40;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",40);
}// END label80
//Dummy.label("com/polytechnik/lapack310/Dsteqr",80);
// *
d[(l-(1))+ _d_offset] = p;
l = (l+1);
if ((l <= lend)) {
    continue label40;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",40);
}
    // *
break label140;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",140);
    }// END label40
/***
*
*        QR Iteration
*
*        Look for small superdiagonal element.
*
***/
}
else  {
label90:for(int jjxx=0;jjxx<Util.ITMAX;jjxx++){
label130:{
label110:{
	//Dummy.label("com/polytechnik/lapack310/Dsteqr",90);
if ((l != lend))  {
    lendp1 = (lend+1);
{
int _m_inc = -1;
for (m = l; m >= lendp1; m += _m_inc) {
tst = ( Math.pow(Math.abs(e[((m-1)-(1))+ _e_offset]), 2));
if ((tst <= ((((eps2*Math.abs(d[(m-(1))+ _d_offset])))*Math.abs(d[((m-1)-(1))+ _d_offset]))+safmin))) {
    break label110;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",110);
}
//Dummy.label("com/polytechnik/lapack310/Dsteqr",100);
}              //  Close for() loop.
}
// *
}
// *
m = lend;
break label110;}// END label110
	//Dummy.label("com/polytechnik/lapack310/Dsteqr",110);
if ((m > lend)) {
    e[((m-1)-(1))+ _e_offset] = 0.0;
}
    p = d[(l-(1))+ _d_offset];
/***
*
*        If remaining matrix is 2-by-2, use DLAE2 or SLAEV2
*        to compute its eigensystem.
*
***/
if ((m == l)) {
    break label130;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",130);
}
    if ((m == (l-1)))  {
    if ((icompz > 0))  {
    com.polytechnik.lapack310.Dlaev2.dlaev2(d[((l-1)-(1))+ _d_offset],e[((l-1)-(1))+ _e_offset],d[(l-(1))+ _d_offset],rt1,rt2,c,s);
work[(m-(1))+ _work_offset] = c.val;
work[(((n-1)+m)-(1))+ _work_offset] = s.val;
com.polytechnik.lapack310.Dlasr.dlasr("R","V","F",n,2,work,(m-(1))+ _work_offset,work,(((n-1)+m)-(1))+ _work_offset,z,(1-(1))+((l-1)-(1)) * (ldz)+ _z_offset,ldz);
}
else  {
  com.polytechnik.lapack310.Dlae2.dlae2(d[((l-1)-(1))+ _d_offset],e[((l-1)-(1))+ _e_offset],d[(l-(1))+ _d_offset],rt1,rt2);
}              //  Close else.
d[((l-1)-(1))+ _d_offset] = rt1.val;
d[(l-(1))+ _d_offset] = rt2.val;
e[((l-1)-(1))+ _e_offset] = 0.0;
l = (l-2);
if ((l >= lend)) {
    continue label90;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",90);
}
    break label140;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",140);
// *
}
if ((jtot == nmaxit)) {
    break label140;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",140);
}
    /***
*
*        Form shift.
*
***/
jtot = (jtot+1);
g = (((d[((l-1)-(1))+ _d_offset]-p))/((2.0*e[((l-1)-(1))+ _e_offset])));
r.val = com.polytechnik.lapack310.Dlapy2.dlapy2(g,1.0);
// *
g = ((d[(m-(1))+ _d_offset]-p)+((e[((l-1)-(1))+ _e_offset]/((g+Util.dsign(r.val,g))))));
s.val = 1.0;
c.val = 1.0;
/***
*
*        Inner loop
*
***/
p = 0.0;
lm1 = (l-1);
{
for (i = m; i <= lm1; i++) {
f = (s.val*e[(i-(1))+ _e_offset]);
b = (c.val*e[(i-(1))+ _e_offset]);
com.polytechnik.lapack310.Dlartg.dlartg(g,f,c,s,r);
if ((i != m)) {
    e[((i-1)-(1))+ _e_offset] = r.val;
}
    g = (d[(i-(1))+ _d_offset]-p);
r.val = ((((d[((i+1)-(1))+ _d_offset]-g))*s.val)+((2.0*c.val)*b));
p = (s.val*r.val);
d[(i-(1))+ _d_offset] = (g+p);
/***
*
*           If eigenvectors are desired, then save rotations.
*
***/
g = ((c.val*r.val)-b);
if ((icompz > 0))  {
    work[(i-(1))+ _work_offset] = c.val;
work[(((n-1)+i)-(1))+ _work_offset] = s.val;
// *
}
/***
*
*        If eigenvectors are desired, then apply saved rotations.
*
***/
//Dummy.label("com/polytechnik/lapack310/Dsteqr",120);
}              //  Close for() loop.
}
if ((icompz > 0))  {
    mm = ((l-m)+1);
com.polytechnik.lapack310.Dlasr.dlasr("R","V","F",n,mm,work,(m-(1))+ _work_offset,work,(((n-1)+m)-(1))+ _work_offset,z,(1-(1))+(m-(1)) * (ldz)+ _z_offset,ldz);
// *
}
d[(l-(1))+ _d_offset] = (d[(l-(1))+ _d_offset]-p);
e[(lm1-(1))+ _e_offset] = g;
/***
*
*        Eigenvalue found.
*
***/
continue label90;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",90);
}// END label130
	//Dummy.label("com/polytechnik/lapack310/Dsteqr",130);
// *
d[(l-(1))+ _d_offset] = p;
l = (l-1);
if ((l >= lend)) {
    continue label90;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",90);
}
    // *
break label140;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",140);
}// END label90;
/***
*
*     Undo scaling if necessary
*
***/
}              //  Close else.
break label140;}// END label140
//Dummy.label("com/polytechnik/lapack310/Dsteqr",140);
if ((iscale == 1))  {
    com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,ssfmax,anorm,((lendsv-lsv)+1),1,d,(lsv-(1))+ _d_offset,n,info);
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,ssfmax,anorm,(lendsv-lsv),1,e,(lsv-(1))+ _e_offset,n,info);
}
else if ((iscale == 2))  {
    com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,ssfmin,anorm,((lendsv-lsv)+1),1,d,(lsv-(1))+ _d_offset,n,info);
com.polytechnik.lapack310.Dlascl.dlascl("G",0,0,ssfmin,anorm,(lendsv-lsv),1,e,(lsv-(1))+ _e_offset,n,info);
/***
*
*     Check for no convergence to an eigenvalue after a total
*     of N*MAXIT iterations.
*
***/
}              // Close else if()
if ((jtot < nmaxit)) {
    continue label10;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",10);
}
    {
for (i = 1; i <= (n-1); i++) {
if ((e[(i-(1))+ _e_offset] != 0.0)) {
    info.val = (info.val+1);
}
//Dummy.label("com/polytechnik/lapack310/Dsteqr",150);
}              //  Close for() loop.
}
/***
*
*     Order eigenvalues and eigenvectors.
*
***/
break label190;
}//break label10;
//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",190);
break label160;}
//Dummy.label("com/polytechnik/lapack310/Dsteqr",160);
/***
*
*        Use Quick Sort
*
***/
if ((icompz == 0))  {
    // *
com.polytechnik.lapack310.Dlasrt.dlasrt("I",n,d,_d_offset,info);
/***
*
*        Use Selection Sort to minimize swaps of eigenvectors
*
***/
}
else  {
  {
for (ii = 2; ii <= n; ii++) {
i = (ii-1);
k = i;
p = d[(i-(1))+ _d_offset];
{
for (j = ii; j <= n; j++) {
if ((d[(j-(1))+ _d_offset] < p))  {
    k = j;
p = d[(j-(1))+ _d_offset];
}
//Dummy.label("com/polytechnik/lapack310/Dsteqr",170);
}              //  Close for() loop.
}
if ((k != i))  {
    d[(k-(1))+ _d_offset] = d[(i-(1))+ _d_offset];
d[(i-(1))+ _d_offset] = p;
com.polytechnik.lapack310.Dswap.dswap(n,z,(1-(1))+(i-(1)) * (ldz)+ _z_offset,1,z,(1-(1))+(k-(1)) * (ldz)+ _z_offset,1);
}
//Dummy.label("com/polytechnik/lapack310/Dsteqr",180);
}              //  Close for() loop.
}
// *
}              //  Close else.
break label190;}
//Dummy.label("com/polytechnik/lapack310/Dsteqr",190);
/***
*
*     End of DSTEQR
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsteqr",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dsteqr",999999);
return;
   }
} // End class.
