/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dswap.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dswap {

// *> \brief \b DSWAP
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSWAP(N,DX,INCX,DY,INCY)
// *
// *       .. Scalar Arguments ..
// *       INTEGER INCX,INCY,N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION DX(*),DY(*)
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *>    DSWAP interchanges two vectors.
// *>    uses unrolled loops for increments equal to 1.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>         number of elements in input vector(s)
// *> \endverbatim
// *>
// *> \param[in,out] DX
// *> \verbatim
// *>          DX is DOUBLE PRECISION array, dimension ( 1 + ( N - 1 )*abs(
// *> \endverbatim
// *>
// *> \param[in] INCX
// *> \verbatim
// *>          INCX is INTEGER
// *>         storage spacing between elements of DX
// *> \endverbatim
// *>
// *> \param[in,out] DY
// *> \verbatim
// *>          DY is DOUBLE PRECISION array, dimension ( 1 + ( N - 1 )*abs(
// *> \endverbatim
// *>
// *> \param[in] INCY
// *> \verbatim
// *>          INCY is INTEGER
// *>         storage spacing between elements of DY
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup double_blas_level1
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>     jack dongarra, linpack, 3/11/78.
// *>     modified 12/3/93, array(1) declarations changed to array(*)
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
// *     ..

public static void dswap (int n,
double [] dx, int _dx_offset,
int incx,
double [] dy, int _dy_offset,
int incy)  {

label_999999:{

double dtemp= 0.0d;
int i= 0;
int ix= 0;
int iy= 0;
int m= 0;
int mp1= 0;
if ((n <= 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dswap",999999);
}
    /***
*
*       code for both increments equal to 1
*
*
*       clean-up loop
*
***/
if (((incx == 1) && (incy == 1)))  {
    m = (n)%(3);
if ((m != 0))  {
    {
for (i = 1; i <= m; i++) {
dtemp = dx[(i-(1))+ _dx_offset];
dx[(i-(1))+ _dx_offset] = dy[(i-(1))+ _dy_offset];
dy[(i-(1))+ _dy_offset] = dtemp;
//Dummy.label("com/polytechnik/lapack310/Dswap",1000000);
}              //  Close for() loop.
}
if ((n < 3)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dswap",999999);
}
    }
mp1 = (m+1);
{
int _i_inc = 3;
for (i = mp1; i <= n; i += _i_inc) {
dtemp = dx[(i-(1))+ _dx_offset];
dx[(i-(1))+ _dx_offset] = dy[(i-(1))+ _dy_offset];
dy[(i-(1))+ _dy_offset] = dtemp;
dtemp = dx[((i+1)-(1))+ _dx_offset];
dx[((i+1)-(1))+ _dx_offset] = dy[((i+1)-(1))+ _dy_offset];
dy[((i+1)-(1))+ _dy_offset] = dtemp;
dtemp = dx[((i+2)-(1))+ _dx_offset];
dx[((i+2)-(1))+ _dx_offset] = dy[((i+2)-(1))+ _dy_offset];
dy[((i+2)-(1))+ _dy_offset] = dtemp;
//Dummy.label("com/polytechnik/lapack310/Dswap",1000001);
}              //  Close for() loop.
}
/***
*
*       code for unequal increments or equal increments not equal
*         to 1
*
***/
}
else  {
  ix = 1;
iy = 1;
if ((incx < 0)) {
    ix = (((((-(n))+1))*incx)+1);
}
    if ((incy < 0)) {
    iy = (((((-(n))+1))*incy)+1);
}
    {
for (i = 1; i <= n; i++) {
dtemp = dx[(ix-(1))+ _dx_offset];
dx[(ix-(1))+ _dx_offset] = dy[(iy-(1))+ _dy_offset];
dy[(iy-(1))+ _dy_offset] = dtemp;
ix = (ix+incx);
iy = (iy+incy);
//Dummy.label("com/polytechnik/lapack310/Dswap",1000002);
}              //  Close for() loop.
}
}              //  Close else.
/***
*
*     End of DSWAP
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dswap",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dswap",999999);
return;
   }
} // End class.
