/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsyevd.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dsyevd {

// *> \brief <b> DSYEVD computes the eigenvalues and, optionally, the left
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DSYEVD + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYEVD( JOBZ, UPLO, N, A, LDA, W, WORK, LWORK, IWORK,
// *                          LIWORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          JOBZ, UPLO
// *       INTEGER            INFO, LDA, LIWORK, LWORK, N
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IWORK( * )
// *       DOUBLE PRECISION   A( LDA, * ), W( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYEVD computes all eigenvalues and, optionally, eigenvectors of a
// *> real symmetric matrix A. If eigenvectors are desired, it uses a
// *> divide and conquer algorithm.
// *>
// *> The divide and conquer algorithm makes very mild assumptions about
// *> floating point arithmetic. It will work on machines with a guard
// *> digit in add/subtract, or on those binary machines without guard
// *> digits which subtract like the Cray X-MP, Cray Y-MP, Cray C-90, or
// *> Cray-2. It could conceivably fail on hexadecimal or decimal machines

// *> without guard digits, but we know of none.
// *>
// *> Because of large use of BLAS of level 3, DSYEVD needs N**2 more
// *> workspace than DSYEVX.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] JOBZ
// *> \verbatim
// *>          JOBZ is CHARACTER*1
// *>          = 'N':  Compute eigenvalues only;
// *>          = 'V':  Compute eigenvalues and eigenvectors.
// *> \endverbatim
// *>
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          = 'U':  Upper triangle of A is stored;
// *>          = 'L':  Lower triangle of A is stored.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA, N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the
// *>          leading N-by-N upper triangular part of A contains the
// *>          upper triangular part of the matrix A.  If UPLO = 'L',
// *>          the leading N-by-N lower triangular part of A contains
// *>          the lower triangular part of the matrix A.
// *>          On exit, if JOBZ = 'V', then if INFO = 0, A contains the
// *>          orthonormal eigenvectors of the matrix A.
// *>          If JOBZ = 'N', then on exit the lower triangle (if UPLO='L')
// *>          or the upper triangle (if UPLO='U') of A, including the
// *>          diagonal, is destroyed.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          If INFO = 0, the eigenvalues in ascending order.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array,
// *>                                         dimension (LWORK)
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK.
// *>          If N <= 1,               LWORK must be at least 1.
// *>          If JOBZ = 'N' and N > 1, LWORK must be at least 2*N+1.
// *>          If JOBZ = 'V' and N > 1, LWORK must be at least
// *>                                                1 + 6*N + 2*N**2.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal sizes of the WORK and IWORK
// *>          arrays, returns these values as the first entries of the WOR
// *>          and IWORK arrays, and no error message related to LWORK or
// *>          LIWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (MAX(1,LIWORK))
// *>          On exit, if INFO = 0, IWORK(1) returns the optimal LIWORK.
// *> \endverbatim
// *>
// *> \param[in] LIWORK
// *> \verbatim
// *>          LIWORK is INTEGER
// *>          The dimension of the array IWORK.
// *>          If N <= 1,                LIWORK must be at least 1.
// *>          If JOBZ  = 'N' and N > 1, LIWORK must be at least 1.
// *>          If JOBZ  = 'V' and N > 1, LIWORK must be at least 3 + 5*N.
// *>
// *>          If LIWORK = -1, then a workspace query is assumed; the
// *>          routine only calculates the optimal sizes of the WORK and
// *>          IWORK arrays, returns these values as the first entries of
// *>          the WORK and IWORK arrays, and no error message related to
// *>          LWORK or LIWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  if INFO = i and JOBZ = 'N', then the algorithm failed

// *>                to converge; i off-diagonal elements of an intermediat
// *>                tridiagonal form did not converge to zero;
// *>                if INFO = i and JOBZ = 'V', then the algorithm failed

// *>                to compute an eigenvalue while working on the submatri
// *>                lying in rows and columns INFO/(N+1) through
// *>                mod(INFO,N+1).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleSYeigen
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Jeff Rutter, Computer Science Division, University of California
// *> at Berkeley, USA \n
// *>  Modified by Francoise Tisseur, University of Tennessee \n
// *>  Modified description of INFO. Sven, 16 Feb 05. \n
//
//
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
*
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dsyevd (String jobz,
String uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] w, int _w_offset,
double [] work, int _work_offset,
int lwork,
int [] iwork, int _iwork_offset,
int liwork,
intW info)  {

label_999999:{

boolean lower= false;
boolean lquery= false;
boolean wantz= false;
intW iinfo= new intW(0);
int inde= 0;
int indtau= 0;
int indwk2= 0;
int indwrk= 0;
int iscale= 0;
int liopt= 0;
int liwmin= 0;
int llwork= 0;
int llwrk2= 0;
int lopt= 0;
int lwmin= 0;
double anrm= 0.0d;
double bignum= 0.0d;
double eps= 0.0d;
double rmax= 0.0d;
double rmin= 0.0d;
double safmin= 0.0d;
double sigma= 0.0d;
double smlnum= 0.0d;
jobz = jobz.substring((1)-1,1);
uplo = uplo.substring((1)-1,1);
wantz = com.polytechnik.lapack310.Lsame.lsame(jobz,"V");
lower = com.polytechnik.lapack310.Lsame.lsame(uplo,"L");
// *
lquery = (((lwork == -1) || (liwork == -1)));
info.val = 0;
if ((!((wantz || com.polytechnik.lapack310.Lsame.lsame(jobz,"N")))))  {
    info.val = -1;
}
else if ((!((lower || com.polytechnik.lapack310.Lsame.lsame(uplo,"U")))))  {
    info.val = -2;
}              // Close else if()
else if ((n < 0))  {
    info.val = -3;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -5;
// *
}              // Close else if()
if ((info.val == 0))  {
    if ((n <= 1))  {
    liwmin = 1;
lwmin = 1;
lopt = lwmin;
liopt = liwmin;
}
else  {
  if (wantz)  {
    liwmin = (3+(5*n));
lwmin = ((1+(6*n))+(2*((int)Math.pow(n, 2))));
}
else  {
  liwmin = 1;
lwmin = ((2*n)+1);
}              //  Close else.
  lopt = Math.max(lwmin, ((2*n)+1/*com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DSYTRD",uplo,n,-1,-1,-1)*/));
liopt = liwmin;
}              //  Close else.
work[(1-(1))+ _work_offset] = (double)(lopt);
// *
iwork[(1-(1))+ _iwork_offset] = liopt;
if (((lwork < lwmin) && (!lquery)))  {
    info.val = -8;
}
else if (((liwork < liwmin) && (!lquery)))  {
    info.val = -10;
}              // Close else if()
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DSYEVD",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsyevd",999999);
}
else if (lquery)  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsyevd",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
// *
if ((n == 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsyevd",999999);
}
    if ((n == 1))  {
    w[(1-(1))+ _w_offset] = a[(1-(1))+(1-(1)) * (lda)+ _a_offset];
if (wantz) {
    a[(1-(1))+(1-(1)) * (lda)+ _a_offset] = 1.0;
}
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsyevd",999999);
/***
*
*     Get machine constants.
*
***/
}
safmin = com.polytechnik.lapack310.Dlamch.dlamch("Safe minimum");
eps = com.polytechnik.lapack310.Dlamch.dlamch("Precision");
smlnum = (safmin/eps);
bignum = (1.0/smlnum);
rmin = Math.sqrt(smlnum);
/***
*
*     Scale matrix to allowable range, if necessary.
*
***/
rmax = Math.sqrt(bignum);
anrm = com.polytechnik.lapack310.Dlansy.dlansy("M",uplo,n,a,_a_offset,lda,work,_work_offset);
iscale = 0;
if (((anrm > 0.0) && (anrm < rmin)))  {
    iscale = 1;
sigma = (rmin/anrm);
}
else if ((anrm > rmax))  {
    iscale = 1;
sigma = (rmax/anrm);
}              // Close else if()
/***
*
*     Call DSYTRD to reduce symmetric matrix to tridiagonal form.
*
***/
if ((iscale == 1)) {
    com.polytechnik.lapack310.Dlascl.dlascl(uplo,0,0,1.0,sigma,n,n,a,_a_offset,lda,info);
}
    inde = 1;
indtau = (inde+n);
indwrk = (indtau+n);
llwork = ((lwork-indwrk)+1);
indwk2 = (indwrk+(n*n));
// *
llwrk2 = ((lwork-indwk2)+1);
/***
*
*     For eigenvalues only, call DSTERF.  For eigenvectors, first call
*     DSTEDC to generate the eigenvector matrix, WORK(INDWRK), of the
*     tridiagonal matrix, then call DORMTR to multiply it by the
*     Householder transformations stored in A.
*
***/
com.polytechnik.lapack310.Dsytrd.dsytrd(uplo,n,a,_a_offset,lda,w,_w_offset,work,(inde-(1))+ _work_offset,work,(indtau-(1))+ _work_offset,work,(indwrk-(1))+ _work_offset,llwork,iinfo);
if ((!wantz))  {
    com.polytechnik.lapack310.Dsterf.dsterf(n,w,_w_offset,work,(inde-(1))+ _work_offset,info);
}
else  {
  com.polytechnik.lapack310.Dstedc.dstedc("I",n,w,_w_offset,work,(inde-(1))+ _work_offset,work,(indwrk-(1))+ _work_offset,n,work,(indwk2-(1))+ _work_offset,llwrk2,iwork,_iwork_offset,liwork,info);
com.polytechnik.lapack310.Dormtr.dormtr("L",uplo,"N",n,n,a,_a_offset,lda,work,(indtau-(1))+ _work_offset,work,(indwrk-(1))+ _work_offset,n,work,(indwk2-(1))+ _work_offset,llwrk2,iinfo);
com.polytechnik.lapack310.Dlacpy.dlacpy("A",n,n,work,(indwrk-(1))+ _work_offset,n,a,_a_offset,lda);
/***
*
*     If matrix was scaled, then rescale eigenvalues appropriately.
*
***/
}              //  Close else.
// *
if ((iscale == 1)) {
    com.polytechnik.lapack310.Dscal.dscal(n,(1.0/sigma),w,_w_offset,1);
}
    work[(1-(1))+ _work_offset] = (double)(lopt);
// *
iwork[(1-(1))+ _iwork_offset] = liopt;
/***
*
*     End of DSYEVD
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsyevd",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dsyevd",999999);
return;
   }
} // End class.
