/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsygst.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dsygst {

// *> \brief \b DSYGST
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DSYGST + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYGST( ITYPE, UPLO, N, A, LDA, B, LDB, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            INFO, ITYPE, LDA, LDB, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), B( LDB, * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYGST reduces a real symmetric-definite generalized eigenproblem
// *> to standard form.
// *>
// *> If ITYPE = 1, the problem is A*x = lambda*B*x,
// *> and A is overwritten by inv(U**T)*A*inv(U) or inv(L)*A*inv(L**T)
// *>
// *> If ITYPE = 2 or 3, the problem is A*B*x = lambda*x or
// *> B*A*x = lambda*x, and A is overwritten by U*A*U**T or L**T*A*L.
// *>
// *> B must have been previously factorized as U**T*U or L*L**T by DPOTRF.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] ITYPE
// *> \verbatim
// *>          ITYPE is INTEGER
// *>          = 1: compute inv(U**T)*A*inv(U) or inv(L)*A*inv(L**T);
// *>          = 2 or 3: compute U*A*U**T or L**T*A*L.
// *> \endverbatim
// *>
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          = 'U':  Upper triangle of A is stored and B is factored as
// *>                  U**T*U;
// *>          = 'L':  Lower triangle of A is stored and B is factored as
// *>                  L*L**T.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrices A and B.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the leadin
// *>          N-by-N upper triangular part of A contains the upper
// *>          triangular part of the matrix A, and the strictly lower
// *>          triangular part of A is not referenced.  If UPLO = 'L', the

// *>          leading N-by-N lower triangular part of A contains the lower
// *>          triangular part of the matrix A, and the strictly upper
// *>          triangular part of A is not referenced.
// *>
// *>          On exit, if INFO = 0, the transformed matrix, stored in the

// *>          same format as A.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array, dimension (LDB,N)
// *>          The triangular factor from the Cholesky factorization of B,

// *>          as returned by DPOTRF.
// *> \endverbatim
// *>
// *> \param[in] LDB
// *> \verbatim
// *>          LDB is INTEGER
// *>          The leading dimension of the array B.  LDB >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleSYcomputational
// *
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dsygst (int itype,
String uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
intW info)  {

label_999999:{

boolean upper= false;
int k= 0;
int kb= 0;
int nb= 0;
uplo = uplo.substring((1)-1,1);
info.val = 0;
upper = com.polytechnik.lapack310.Lsame.lsame(uplo,"U");
if (((itype < 1) || (itype > 3)))  {
    info.val = -1;
}
else if (((!upper) && (!com.polytechnik.lapack310.Lsame.lsame(uplo,"L"))))  {
    info.val = -2;
}              // Close else if()
else if ((n < 0))  {
    info.val = -3;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -5;
}              // Close else if()
else if ((ldb < Math.max(1, n)))  {
    info.val = -7;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DSYGST",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsygst",999999);
/***
*
*     Quick return if possible
*
***/
}
/***
*
*     Determine the block size for this environment.
*
***/
if ((n == 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsygst",999999);
}
    // *
nb = 1;//com.polytechnik.lapack310.Ilaenv.ilaenv(1,"DSYGST",uplo,n,-1,-1,-1);
/***
*
*        Use unblocked code
*
***/
if (((nb <= 1) || (nb >= n)))  {
    com.polytechnik.lapack310.Dsygs2.dsygs2(itype,uplo,n,a,_a_offset,lda,b,_b_offset,ldb,info);
/***
*
*        Use blocked code
*
***/
}
else  {
  if ((itype == 1))  {
    /***
*
*              Compute inv(U**T)*A*inv(U)
*
***/
if (upper)  {
    {
int _k_inc = nb;
for (k = 1; (_k_inc < 0) ? k >= n : k <= n; k += _k_inc) {
/***
*
*                 Update the upper triangle of A(k:n,k:n)
*
***/
kb = Math.min(((n-k)+1), nb);
com.polytechnik.lapack310.Dsygs2.dsygs2(itype,uplo,kb,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,info);
if (((k+kb) <= n))  {
    com.polytechnik.lapack310.Dtrsm.dtrsm("Left",uplo,"Transpose","Non-unit",kb,(((n-k)-kb)+1),1.0,b,(k-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,a,(k-(1))+((k+kb)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsymm.dsymm("Left",uplo,kb,(((n-k)-kb)+1),-0.5,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+((k+kb)-(1)) * (ldb)+ _b_offset,ldb,1.0,a,(k-(1))+((k+kb)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsyr2k.dsyr2k(uplo,"Transpose",(((n-k)-kb)+1),kb,-1.0,a,(k-(1))+((k+kb)-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+((k+kb)-(1)) * (ldb)+ _b_offset,ldb,1.0,a,((k+kb)-(1))+((k+kb)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsymm.dsymm("Left",uplo,kb,(((n-k)-kb)+1),-0.5,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+((k+kb)-(1)) * (ldb)+ _b_offset,ldb,1.0,a,(k-(1))+((k+kb)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dtrsm.dtrsm("Right",uplo,"No transpose","Non-unit",kb,(((n-k)-kb)+1),1.0,b,((k+kb)-(1))+((k+kb)-(1)) * (ldb)+ _b_offset,ldb,a,(k-(1))+((k+kb)-(1)) * (lda)+ _a_offset,lda);
}
//Dummy.label("com/polytechnik/lapack310/Dsygst",10);
}              //  Close for() loop.
}
/***
*
*              Compute inv(L)*A*inv(L**T)
*
***/
}
else  {
  {
int _k_inc = nb;
for (k = 1; (_k_inc < 0) ? k >= n : k <= n; k += _k_inc) {
/***
*
*                 Update the lower triangle of A(k:n,k:n)
*
***/
kb = Math.min(((n-k)+1), nb);
com.polytechnik.lapack310.Dsygs2.dsygs2(itype,uplo,kb,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,info);
if (((k+kb) <= n))  {
    com.polytechnik.lapack310.Dtrsm.dtrsm("Right",uplo,"Transpose","Non-unit",(((n-k)-kb)+1),kb,1.0,b,(k-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,a,((k+kb)-(1))+(k-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsymm.dsymm("Right",uplo,(((n-k)-kb)+1),kb,-0.5,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,((k+kb)-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,1.0,a,((k+kb)-(1))+(k-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsyr2k.dsyr2k(uplo,"No transpose",(((n-k)-kb)+1),kb,-1.0,a,((k+kb)-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,((k+kb)-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,1.0,a,((k+kb)-(1))+((k+kb)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsymm.dsymm("Right",uplo,(((n-k)-kb)+1),kb,-0.5,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,((k+kb)-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,1.0,a,((k+kb)-(1))+(k-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dtrsm.dtrsm("Left",uplo,"No transpose","Non-unit",(((n-k)-kb)+1),kb,1.0,b,((k+kb)-(1))+((k+kb)-(1)) * (ldb)+ _b_offset,ldb,a,((k+kb)-(1))+(k-(1)) * (lda)+ _a_offset,lda);
}
//Dummy.label("com/polytechnik/lapack310/Dsygst",20);
}              //  Close for() loop.
}
}              //  Close else.
}
else  {
  /***
*
*              Compute U*A*U**T
*
***/
if (upper)  {
    {
int _k_inc = nb;
for (k = 1; (_k_inc < 0) ? k >= n : k <= n; k += _k_inc) {
/***
*
*                 Update the upper triangle of A(1:k+kb-1,1:k+kb-1)
*
***/
kb = Math.min(((n-k)+1), nb);
com.polytechnik.lapack310.Dtrmm.dtrmm("Left",uplo,"No transpose","Non-unit",(k-1),kb,1.0,b,_b_offset,ldb,a,(1-(1))+(k-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsymm.dsymm("Right",uplo,(k-1),kb,0.5,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,(1-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,1.0,a,(1-(1))+(k-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsyr2k.dsyr2k(uplo,"No transpose",(k-1),kb,1.0,a,(1-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,(1-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,1.0,a,_a_offset,lda);
com.polytechnik.lapack310.Dsymm.dsymm("Right",uplo,(k-1),kb,0.5,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,(1-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,1.0,a,(1-(1))+(k-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dtrmm.dtrmm("Right",uplo,"Transpose","Non-unit",(k-1),kb,1.0,b,(k-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,a,(1-(1))+(k-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsygs2.dsygs2(itype,uplo,kb,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,info);
//Dummy.label("com/polytechnik/lapack310/Dsygst",30);
}              //  Close for() loop.
}
/***
*
*              Compute L**T*A*L
*
***/
}
else  {
  {
int _k_inc = nb;
for (k = 1; (_k_inc < 0) ? k >= n : k <= n; k += _k_inc) {
/***
*
*                 Update the lower triangle of A(1:k+kb-1,1:k+kb-1)
*
***/
kb = Math.min(((n-k)+1), nb);
com.polytechnik.lapack310.Dtrmm.dtrmm("Right",uplo,"No transpose","Non-unit",kb,(k-1),1.0,b,_b_offset,ldb,a,(k-(1))+(1-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsymm.dsymm("Left",uplo,kb,(k-1),0.5,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+(1-(1)) * (ldb)+ _b_offset,ldb,1.0,a,(k-(1))+(1-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsyr2k.dsyr2k(uplo,"Transpose",(k-1),kb,1.0,a,(k-(1))+(1-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+(1-(1)) * (ldb)+ _b_offset,ldb,1.0,a,_a_offset,lda);
com.polytechnik.lapack310.Dsymm.dsymm("Left",uplo,kb,(k-1),0.5,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+(1-(1)) * (ldb)+ _b_offset,ldb,1.0,a,(k-(1))+(1-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dtrmm.dtrmm("Left",uplo,"Transpose","Non-unit",kb,(k-1),1.0,b,(k-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,a,(k-(1))+(1-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack310.Dsygs2.dsygs2(itype,uplo,kb,a,(k-(1))+(k-(1)) * (lda)+ _a_offset,lda,b,(k-(1))+(k-(1)) * (ldb)+ _b_offset,ldb,info);
//Dummy.label("com/polytechnik/lapack310/Dsygst",40);
}              //  Close for() loop.
}
}              //  Close else.
}              //  Close else.
}              //  Close else.
/***
*
*     End of DSYGST
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsygst",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dsygst",999999);
return;
   }
} // End class.
