/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsygvd.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dsygvd {

// *> \brief \b DSYGVD
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DSYGVD + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYGVD( ITYPE, JOBZ, UPLO, N, A, LDA, B, LDB, W, WORK
// *                          LWORK, IWORK, LIWORK, INFO )
// *
// *       .. Scalar Arguments ..
// *       CHARACTER          JOBZ, UPLO
// *       INTEGER            INFO, ITYPE, LDA, LDB, LIWORK, LWORK, N
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IWORK( * )
// *       DOUBLE PRECISION   A( LDA, * ), B( LDB, * ), W( * ), WORK( * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYGVD computes all the eigenvalues, and optionally, the eigenvectors
// *> of a real generalized symmetric-definite eigenproblem, of the form
// *> A*x=(lambda)*B*x,  A*Bx=(lambda)*x,  or B*A*x=(lambda)*x.  Here A and
// *> B are assumed to be symmetric and B is also positive definite.
// *> If eigenvectors are desired, it uses a divide and conquer algorithm.

// *>
// *> The divide and conquer algorithm makes very mild assumptions about
// *> floating point arithmetic. It will work on machines with a guard
// *> digit in add/subtract, or on those binary machines without guard
// *> digits which subtract like the Cray X-MP, Cray Y-MP, Cray C-90, or
// *> Cray-2. It could conceivably fail on hexadecimal or decimal machines

// *> without guard digits, but we know of none.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] ITYPE
// *> \verbatim
// *>          ITYPE is INTEGER
// *>          Specifies the problem type to be solved:
// *>          = 1:  A*x = (lambda)*B*x
// *>          = 2:  A*B*x = (lambda)*x
// *>          = 3:  B*A*x = (lambda)*x
// *> \endverbatim
// *>
// *> \param[in] JOBZ
// *> \verbatim
// *>          JOBZ is CHARACTER*1
// *>          = 'N':  Compute eigenvalues only;
// *>          = 'V':  Compute eigenvalues and eigenvectors.
// *> \endverbatim
// *>
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          = 'U':  Upper triangles of A and B are stored;
// *>          = 'L':  Lower triangles of A and B are stored.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrices A and B.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA, N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the
// *>          leading N-by-N upper triangular part of A contains the
// *>          upper triangular part of the matrix A.  If UPLO = 'L',
// *>          the leading N-by-N lower triangular part of A contains
// *>          the lower triangular part of the matrix A.
// *>
// *>          On exit, if JOBZ = 'V', then if INFO = 0, A contains the
// *>          matrix Z of eigenvectors.  The eigenvectors are normalized
// *>          as follows:
// *>          if ITYPE = 1 or 2, Z**T*B*Z = I;
// *>          if ITYPE = 3, Z**T*inv(B)*Z = I.
// *>          If JOBZ = 'N', then on exit the upper triangle (if UPLO='U')
// *>          or the lower triangle (if UPLO='L') of A, including the
// *>          diagonal, is destroyed.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in,out] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array, dimension (LDB, N)
// *>          On entry, the symmetric matrix B.  If UPLO = 'U', the
// *>          leading N-by-N upper triangular part of B contains the
// *>          upper triangular part of the matrix B.  If UPLO = 'L',
// *>          the leading N-by-N lower triangular part of B contains
// *>          the lower triangular part of the matrix B.
// *>
// *>          On exit, if INFO <= N, the part of B containing the matrix i
// *>          overwritten by the triangular factor U or L from the Cholesk
// *>          factorization B = U**T*U or B = L*L**T.
// *> \endverbatim
// *>
// *> \param[in] LDB
// *> \verbatim
// *>          LDB is INTEGER
// *>          The leading dimension of the array B.  LDB >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          If INFO = 0, the eigenvalues in ascending order.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK.
// *>          If N <= 1,               LWORK >= 1.
// *>          If JOBZ = 'N' and N > 1, LWORK >= 2*N+1.
// *>          If JOBZ = 'V' and N > 1, LWORK >= 1 + 6*N + 2*N**2.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal sizes of the WORK and IWORK
// *>          arrays, returns these values as the first entries of the WOR
// *>          and IWORK arrays, and no error message related to LWORK or
// *>          LIWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (MAX(1,LIWORK))
// *>          On exit, if INFO = 0, IWORK(1) returns the optimal LIWORK.
// *> \endverbatim
// *>
// *> \param[in] LIWORK
// *> \verbatim
// *>          LIWORK is INTEGER
// *>          The dimension of the array IWORK.
// *>          If N <= 1,                LIWORK >= 1.
// *>          If JOBZ  = 'N' and N > 1, LIWORK >= 1.
// *>          If JOBZ  = 'V' and N > 1, LIWORK >= 3 + 5*N.
// *>
// *>          If LIWORK = -1, then a workspace query is assumed; the
// *>          routine only calculates the optimal sizes of the WORK and
// *>          IWORK arrays, returns these values as the first entries of
// *>          the WORK and IWORK arrays, and no error message related to
// *>          LWORK or LIWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  DPOTRF or DSYEVD returned an error code:
// *>             <= N:  if INFO = i and JOBZ = 'N', then the algorithm
// *>                    failed to converge; i off-diagonal elements of an

// *>                    intermediate tridiagonal form did not converge to

// *>                    zero;
// *>                    if INFO = i and JOBZ = 'V', then the algorithm
// *>                    failed to compute an eigenvalue while working on
// *>                    the submatrix lying in rows and columns INFO/(N+1)
// *>                    through mod(INFO,N+1);
// *>             > N:   if INFO = N + i, for 1 <= i <= N, then the leading
// *>                    minor of order i of B is not positive definite.
// *>                    The factorization of B could not be completed and

// *>                    no eigenvalues or eigenvectors were computed.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleSYeigen
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Modified so that no backsubstitution is performed if DSYEVD fails to
// *>  converge (NEIG in old code could be greater than N causing out of
// *>  bounds reference to A - reported by Ralf Meyer).  Also corrected the
// *>  description of INFO and the test on ITYPE. Sven, 16 Feb 05.
// *> \endverbatim
// *
// *> \par Contributors:
// *  ==================
// *>
// *>     Mark Fahey, Department of Mathematics, Univ. of Kentucky, USA
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dsygvd (int itype,
String jobz,
String uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
double [] w, int _w_offset,
double [] work, int _work_offset,
int lwork,
int [] iwork, int _iwork_offset,
int liwork,
intW info)  {

label_999999:{

boolean lquery= false;
boolean upper= false;
boolean wantz= false;
String trans= new String(" ");
int liopt= 0;
int liwmin= 0;
int lopt= 0;
int lwmin= 0;
jobz = jobz.substring((1)-1,1);
uplo = uplo.substring((1)-1,1);
wantz = com.polytechnik.lapack310.Lsame.lsame(jobz,"V");
upper = com.polytechnik.lapack310.Lsame.lsame(uplo,"U");
// *
lquery = (((lwork == -1) || (liwork == -1)));
info.val = 0;
if ((n <= 1))  {
    liwmin = 1;
lwmin = 1;
}
else if (wantz)  {
    liwmin = (3+(5*n));
lwmin = ((1+(6*n))+(2*((int)Math.pow(n, 2))));
}              // Close else if()
else  {
  liwmin = 1;
lwmin = ((2*n)+1);
}              //  Close else.
lopt = lwmin;
liopt = liwmin;
if (((itype < 1) || (itype > 3)))  {
    info.val = -1;
}
else if ((!((wantz || com.polytechnik.lapack310.Lsame.lsame(jobz,"N")))))  {
    info.val = -2;
}              // Close else if()
else if ((!((upper || com.polytechnik.lapack310.Lsame.lsame(uplo,"L")))))  {
    info.val = -3;
}              // Close else if()
else if ((n < 0))  {
    info.val = -4;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -6;
}              // Close else if()
else if ((ldb < Math.max(1, n)))  {
    info.val = -8;
// *
}              // Close else if()
if ((info.val == 0))  {
    work[(1-(1))+ _work_offset] = (double)(lopt);
// *
iwork[(1-(1))+ _iwork_offset] = liopt;
if (((lwork < lwmin) && (!lquery)))  {
    info.val = -11;
}
else if (((liwork < liwmin) && (!lquery)))  {
    info.val = -13;
}              // Close else if()
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DSYGVD",(-(info.val)));
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsygvd",999999);
}
else if (lquery)  {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsygvd",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
/***
*
*     Form a Cholesky factorization of B.
*
***/
if ((n == 0)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsygvd",999999);
}
    com.polytechnik.lapack310.Dpotrf.dpotrf(uplo,n,b,_b_offset,ldb,info);
if ((info.val != 0))  {
    info.val = (n+info.val);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsygvd",999999);
/***
*
*     Transform problem to standard eigenvalue problem and solve.
*
***/
}
com.polytechnik.lapack310.Dsygst.dsygst(itype,uplo,n,a,_a_offset,lda,b,_b_offset,ldb,info);
com.polytechnik.lapack310.Dsyevd.dsyevd(jobz,uplo,n,a,_a_offset,lda,w,_w_offset,work,_work_offset,lwork,iwork,_iwork_offset,liwork,info);
lopt = (int)(Math.max((double)(lopt), (double)(work[(1-(1))+ _work_offset])));
// *
liopt = (int)(Math.max((double)(liopt), (double)(iwork[(1-(1))+ _iwork_offset])));
/***
*
*        Backtransform eigenvectors to the original problem.
*
***/
if ((wantz && (info.val == 0)))  {
    /***
*
*           For A*x=(lambda)*B*x and A*B*x=(lambda)*x;
*           backtransform eigenvectors: x = inv(L)**T*y or inv(U)*y
*
***/
if (((itype == 1) || (itype == 2)))  {
    if (upper)  {
    trans = "N";
}
else  {
  trans = "T";
// *
}              //  Close else.
// *
com.polytechnik.lapack310.Dtrsm.dtrsm("Left",uplo,trans,"Non-unit",n,n,1.0,b,_b_offset,ldb,a,_a_offset,lda);
/***
*
*           For B*A*x=(lambda)*x;
*           backtransform eigenvectors: x = L*y or U**T*y
*
***/
}
else if ((itype == 3))  {
    if (upper)  {
    trans = "T";
}
else  {
  trans = "N";
// *
}              //  Close else.
com.polytechnik.lapack310.Dtrmm.dtrmm("Left",uplo,trans,"Non-unit",n,n,1.0,b,_b_offset,ldb,a,_a_offset,lda);
}              // Close else if()
// *
}
work[(1-(1))+ _work_offset] = (double)(lopt);
// *
iwork[(1-(1))+ _iwork_offset] = liopt;
/***
*
*     End of DSYGVD
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsygvd",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dsygvd",999999);
return;
   }
} // End class.
