/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsymm.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dsymm {

// *> \brief \b DSYMM
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYMM(SIDE,UPLO,M,N,ALPHA,A,LDA,B,LDB,BETA,C,LDC)
// *
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION ALPHA,BETA
// *       INTEGER LDA,LDB,LDC,M,N
// *       CHARACTER SIDE,UPLO
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION A(LDA,*),B(LDB,*),C(LDC,*)
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYMM  performs one of the matrix-matrix operations
// *>
// *>    C := alpha*A*B + beta*C,
// *>
// *> or
// *>
// *>    C := alpha*B*A + beta*C,
// *>
// *> where alpha and beta are scalars,  A is a symmetric matrix and  B and
// *> C are  m by n matrices.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] SIDE
// *> \verbatim
// *>          SIDE is CHARACTER*1
// *>           On entry,  SIDE  specifies whether  the  symmetric matrix
// *>           appears on the  left or right  in the  operation as follows
// *>
// *>              SIDE = 'L' or 'l'   C := alpha*A*B + beta*C,
// *>
// *>              SIDE = 'R' or 'r'   C := alpha*B*A + beta*C,
// *> \endverbatim
// *>
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>           On  entry,   UPLO  specifies  whether  the  upper  or  lowe
// *>           triangular  part  of  the  symmetric  matrix   A  is  to  b
// *>           referenced as follows:
// *>
// *>              UPLO = 'U' or 'u'   Only the upper triangular part of th
// *>                                  symmetric matrix is to be referenced
// *>
// *>              UPLO = 'L' or 'l'   Only the lower triangular part of th
// *>                                  symmetric matrix is to be referenced
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>           On entry,  M  specifies the number of rows of the matrix  C
// *>           M  must be at least zero.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>           On entry, N specifies the number of columns of the matrix C
// *>           N  must be at least zero.
// *> \endverbatim
// *>
// *> \param[in] ALPHA
// *> \verbatim
// *>          ALPHA is DOUBLE PRECISION.
// *>           On entry, ALPHA specifies the scalar alpha.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension ( LDA, ka ), where ka
// *>           m  when  SIDE = 'L' or 'l'  and is  n otherwise.
// *>           Before entry  with  SIDE = 'L' or 'l',  the  m by m  part o
// *>           the array  A  must contain the  symmetric matrix,  such tha
// *>           when  UPLO = 'U' or 'u', the leading m by m upper triangula
// *>           part of the array  A  must contain the upper triangular par
// *>           of the  symmetric matrix and the  strictly  lower triangula
// *>           part of  A  is not referenced,  and when  UPLO = 'L' or 'l'
// *>           the leading  m by m  lower triangular part  of the  array
// *>           must  contain  the  lower triangular part  of the  symmetri
// *>           matrix and the  strictly upper triangular part of  A  is no
// *>           referenced.
// *>           Before entry  with  SIDE = 'R' or 'r',  the  n by n  part o
// *>           the array  A  must contain the  symmetric matrix,  such tha
// *>           when  UPLO = 'U' or 'u', the leading n by n upper triangula
// *>           part of the array  A  must contain the upper triangular par
// *>           of the  symmetric matrix and the  strictly  lower triangula
// *>           part of  A  is not referenced,  and when  UPLO = 'L' or 'l'
// *>           the leading  n by n  lower triangular part  of the  array
// *>           must  contain  the  lower triangular part  of the  symmetri
// *>           matrix and the  strictly upper triangular part of  A  is no
// *>           referenced.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>           On entry, LDA specifies the first dimension of A as declare
// *>           in the calling (sub) program.  When  SIDE = 'L' or 'l'  the
// *>           LDA must be at least  max( 1, m ), otherwise  LDA must be a
// *>           least  max( 1, n ).
// *> \endverbatim
// *>
// *> \param[in] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array, dimension ( LDB, N )
// *>           Before entry, the leading  m by n part of the array  B  mus
// *>           contain the matrix B.
// *> \endverbatim
// *>
// *> \param[in] LDB
// *> \verbatim
// *>          LDB is INTEGER
// *>           On entry, LDB specifies the first dimension of B as declare
// *>           in  the  calling  (sub)  program.   LDB  must  be  at  leas
// *>           max( 1, m ).
// *> \endverbatim
// *>
// *> \param[in] BETA
// *> \verbatim
// *>          BETA is DOUBLE PRECISION.
// *>           On entry,  BETA  specifies the scalar  beta.  When  BETA  i
// *>           supplied as zero then C need not be set on input.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension ( LDC, N )
// *>           Before entry, the leading  m by n  part of the array  C mus
// *>           contain the matrix  C,  except when  beta  is zero, in whic
// *>           case C need not be set on entry.
// *>           On exit, the array  C  is overwritten by the  m by n update
// *>           matrix.
// *> \endverbatim
// *>
// *> \param[in] LDC
// *> \verbatim
// *>          LDC is INTEGER
// *>           On entry, LDC specifies the first dimension of C as declare
// *>           in  the  calling  (sub)  program.   LDC  must  be  at  leas
// *>           max( 1, m ).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup double_blas_level3
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Level 3 Blas routine.
// *>
// *>  -- Written on 8-February-1989.
// *>     Jack Dongarra, Argonne National Laboratory.
// *>     Iain Duff, AERE Harwell.
// *>     Jeremy Du Croz, Numerical Algorithms Group Ltd.
// *>     Sven Hammarling, Numerical Algorithms Group Ltd.
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Parameters ..
***/
/***
*     ..
*
*     Set NROWA as the number of rows of A.
*
***/

public static void dsymm (String side,
String uplo,
int m,
int n,
double alpha,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
double beta,
double [] c, int _c_offset,
int ldc)  {

label_999999:{

double temp1= 0.0d;
double temp2= 0.0d;
int i= 0;
int info= 0;
int j= 0;
int k= 0;
int nrowa= 0;
boolean upper= false;
side = side.substring((1)-1,1);
uplo = uplo.substring((1)-1,1);
if (com.polytechnik.lapack310.Lsame.lsame(side,"L"))  {
    nrowa = m;
}
else  {
  nrowa = n;
}              //  Close else.
/***
*
*     Test the input parameters.
*
***/
upper = com.polytechnik.lapack310.Lsame.lsame(uplo,"U");
info = 0;
if ((((!com.polytechnik.lapack310.Lsame.lsame(side,"L"))) && ((!com.polytechnik.lapack310.Lsame.lsame(side,"R")))))  {
    info = 1;
}
else if ((((!upper)) && ((!com.polytechnik.lapack310.Lsame.lsame(uplo,"L")))))  {
    info = 2;
}              // Close else if()
else if ((m < 0))  {
    info = 3;
}              // Close else if()
else if ((n < 0))  {
    info = 4;
}              // Close else if()
else if ((lda < Math.max(1, nrowa)))  {
    info = 7;
}              // Close else if()
else if ((ldb < Math.max(1, m)))  {
    info = 9;
}              // Close else if()
else if ((ldc < Math.max(1, m)))  {
    info = 12;
}              // Close else if()
if ((info != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DSYMM ",info);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsymm",999999);
/***
*
*     Quick return if possible.
*
***/
}
/***
*
*     And when  alpha.eq.zero.
*
***/
if (((((m == 0)) || ((n == 0))) || ((((alpha == 0.0)) && ((beta == 1.0)))))) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsymm",999999);
}
    if ((alpha == 0.0))  {
    if ((beta == 0.0))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dsymm",10);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsymm",20);
}              //  Close for() loop.
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack310/Dsymm",30);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsymm",40);
}              //  Close for() loop.
}
}              //  Close else.
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsymm",999999);
/***
*
*     Start the operations.
*
***/
}
/***
*
*        Form  C := alpha*A*B + beta*C.
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(side,"L"))  {
    if (upper)  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
temp1 = (alpha*b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]);
temp2 = 0.0;
{
for (k = 1; k <= (i-1); k++) {
c[(k-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(k-(1))+(j-(1)) * (ldc)+ _c_offset]+(temp1*a[(k-(1))+(i-(1)) * (lda)+ _a_offset]));
temp2 = (temp2+(b[(k-(1))+(j-(1)) * (ldb)+ _b_offset]*a[(k-(1))+(i-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dsymm",50);
}              //  Close for() loop.
}
if ((beta == 0.0))  {
    c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = ((temp1*a[(i-(1))+(i-(1)) * (lda)+ _a_offset])+(alpha*temp2));
}
else  {
  c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (((beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset])+(temp1*a[(i-(1))+(i-(1)) * (lda)+ _a_offset]))+(alpha*temp2));
}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dsymm",60);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsymm",70);
}              //  Close for() loop.
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
int _i_inc = -1;
for (i = m; i >= 1; i += _i_inc) {
temp1 = (alpha*b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]);
temp2 = 0.0;
{
for (k = (i+1); k <= m; k++) {
c[(k-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(k-(1))+(j-(1)) * (ldc)+ _c_offset]+(temp1*a[(k-(1))+(i-(1)) * (lda)+ _a_offset]));
temp2 = (temp2+(b[(k-(1))+(j-(1)) * (ldb)+ _b_offset]*a[(k-(1))+(i-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dsymm",80);
}              //  Close for() loop.
}
if ((beta == 0.0))  {
    c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = ((temp1*a[(i-(1))+(i-(1)) * (lda)+ _a_offset])+(alpha*temp2));
}
else  {
  c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (((beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset])+(temp1*a[(i-(1))+(i-(1)) * (lda)+ _a_offset]))+(alpha*temp2));
}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dsymm",90);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsymm",100);
}              //  Close for() loop.
}
}              //  Close else.
/***
*
*        Form  C := alpha*B*A + beta*C.
*
***/
}
else  {
  {
for (j = 1; j <= n; j++) {
temp1 = (alpha*a[(j-(1))+(j-(1)) * (lda)+ _a_offset]);
if ((beta == 0.0))  {
    {
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (temp1*b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]);
//Dummy.label("com/polytechnik/lapack310/Dsymm",110);
}              //  Close for() loop.
}
}
else  {
  {
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = ((beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset])+(temp1*b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]));
//Dummy.label("com/polytechnik/lapack310/Dsymm",120);
}              //  Close for() loop.
}
}              //  Close else.
{
for (k = 1; k <= (j-1); k++) {
if (upper)  {
    temp1 = (alpha*a[(k-(1))+(j-(1)) * (lda)+ _a_offset]);
}
else  {
  temp1 = (alpha*a[(j-(1))+(k-(1)) * (lda)+ _a_offset]);
}              //  Close else.
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]+(temp1*b[(i-(1))+(k-(1)) * (ldb)+ _b_offset]));
//Dummy.label("com/polytechnik/lapack310/Dsymm",130);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsymm",140);
}              //  Close for() loop.
}
{
for (k = (j+1); k <= n; k++) {
if (upper)  {
    temp1 = (alpha*a[(j-(1))+(k-(1)) * (lda)+ _a_offset]);
}
else  {
  temp1 = (alpha*a[(k-(1))+(j-(1)) * (lda)+ _a_offset]);
}              //  Close else.
{
for (i = 1; i <= m; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]+(temp1*b[(i-(1))+(k-(1)) * (ldb)+ _b_offset]));
//Dummy.label("com/polytechnik/lapack310/Dsymm",150);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsymm",160);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsymm",170);
}              //  Close for() loop.
}
// *
}              //  Close else.
/***
*
*     End of DSYMM
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsymm",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dsymm",999999);
return;
   }
} // End class.
