/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsyrk.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dsyrk {

// *> \brief \b DSYRK
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYRK(UPLO,TRANS,N,K,ALPHA,A,LDA,BETA,C,LDC)
// *
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION ALPHA,BETA
// *       INTEGER K,LDA,LDC,N
// *       CHARACTER TRANS,UPLO
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION A(LDA,*),C(LDC,*)
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYRK  performs one of the symmetric rank k operations
// *>
// *>    C := alpha*A*A**T + beta*C,
// *>
// *> or
// *>
// *>    C := alpha*A**T*A + beta*C,
// *>
// *> where  alpha and beta  are scalars, C is an  n by n  symmetric matrix
// *> and  A  is an  n by k  matrix in the first case and a  k by n  matrix
// *> in the second case.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>           On  entry,   UPLO  specifies  whether  the  upper  or  lowe
// *>           triangular  part  of the  array  C  is to be  referenced  a
// *>           follows:
// *>
// *>              UPLO = 'U' or 'u'   Only the  upper triangular part of
// *>                                  is to be referenced.
// *>
// *>              UPLO = 'L' or 'l'   Only the  lower triangular part of
// *>                                  is to be referenced.
// *> \endverbatim
// *>
// *> \param[in] TRANS
// *> \verbatim
// *>          TRANS is CHARACTER*1
// *>           On entry,  TRANS  specifies the operation to be performed a
// *>           follows:
// *>
// *>              TRANS = 'N' or 'n'   C := alpha*A*A**T + beta*C.
// *>
// *>              TRANS = 'T' or 't'   C := alpha*A**T*A + beta*C.
// *>
// *>              TRANS = 'C' or 'c'   C := alpha*A**T*A + beta*C.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>           On entry,  N specifies the order of the matrix C.  N must b
// *>           at least zero.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>           On entry with  TRANS = 'N' or 'n',  K  specifies  the numbe
// *>           of  columns   of  the   matrix   A,   and  on   entry   wit
// *>           TRANS = 'T' or 't' or 'C' or 'c',  K  specifies  the  numbe
// *>           of rows of the matrix  A.  K must be at least zero.
// *> \endverbatim
// *>
// *> \param[in] ALPHA
// *> \verbatim
// *>          ALPHA is DOUBLE PRECISION.
// *>           On entry, ALPHA specifies the scalar alpha.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension ( LDA, ka ), where ka
// *>           k  when  TRANS = 'N' or 'n',  and is  n  otherwise.
// *>           Before entry with  TRANS = 'N' or 'n',  the  leading  n by
// *>           part of the array  A  must contain the matrix  A,  otherwis
// *>           the leading  k by n  part of the array  A  must contain  th
// *>           matrix A.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>           On entry, LDA specifies the first dimension of A as declare
// *>           in  the  calling  (sub)  program.   When  TRANS = 'N' or 'n
// *>           then  LDA must be at least  max( 1, n ), otherwise  LDA mus
// *>           be at least  max( 1, k ).
// *> \endverbatim
// *>
// *> \param[in] BETA
// *> \verbatim
// *>          BETA is DOUBLE PRECISION.
// *>           On entry, BETA specifies the scalar beta.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension ( LDC, N )
// *>           Before entry  with  UPLO = 'U' or 'u',  the leading  n by n
// *>           upper triangular part of the array C must contain the upper
// *>           triangular part  of the  symmetric matrix  and the strictly
// *>           lower triangular part of C is not referenced.  On exit, the
// *>           upper triangular part of the array  C is overwritten by the
// *>           upper triangular part of the updated matrix.
// *>           Before entry  with  UPLO = 'L' or 'l',  the leading  n by n
// *>           lower triangular part of the array C must contain the lower
// *>           triangular part  of the  symmetric matrix  and the strictly
// *>           upper triangular part of C is not referenced.  On exit, the
// *>           lower triangular part of the array  C is overwritten by the
// *>           lower triangular part of the updated matrix.
// *> \endverbatim
// *>
// *> \param[in] LDC
// *> \verbatim
// *>          LDC is INTEGER
// *>           On entry, LDC specifies the first dimension of C as declare
// *>           in  the  calling  (sub)  program.   LDC  must  be  at  leas
// *>           max( 1, n ).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup double_blas_level3
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Level 3 Blas routine.
// *>
// *>  -- Written on 8-February-1989.
// *>     Jack Dongarra, Argonne National Laboratory.
// *>     Iain Duff, AERE Harwell.
// *>     Jeremy Du Croz, Numerical Algorithms Group Ltd.
// *>     Sven Hammarling, Numerical Algorithms Group Ltd.
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Parameters ..
***/
/***
*     ..
*
*     Test the input parameters.
*
***/

public static void dsyrk (String uplo,
String trans,
int n,
int k,
double alpha,
double [] a, int _a_offset,
int lda,
double beta,
double [] c, int _c_offset,
int ldc)  {

label_999999:{

double temp= 0.0d;
int i= 0;
int info= 0;
int j= 0;
int l= 0;
int nrowa= 0;
boolean upper= false;
uplo = uplo.substring((1)-1,1);
trans = trans.substring((1)-1,1);
if (com.polytechnik.lapack310.Lsame.lsame(trans,"N"))  {
    nrowa = n;
}
else  {
  nrowa = k;
}              //  Close else.
// *
upper = com.polytechnik.lapack310.Lsame.lsame(uplo,"U");
info = 0;
if ((((!upper)) && ((!com.polytechnik.lapack310.Lsame.lsame(uplo,"L")))))  {
    info = 1;
}
else if (((((!com.polytechnik.lapack310.Lsame.lsame(trans,"N"))) && ((!com.polytechnik.lapack310.Lsame.lsame(trans,"T")))) && ((!com.polytechnik.lapack310.Lsame.lsame(trans,"C")))))  {
    info = 2;
}              // Close else if()
else if ((n < 0))  {
    info = 3;
}              // Close else if()
else if ((k < 0))  {
    info = 4;
}              // Close else if()
else if ((lda < Math.max(1, nrowa)))  {
    info = 7;
}              // Close else if()
else if ((ldc < Math.max(1, n)))  {
    info = 10;
}              // Close else if()
if ((info != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DSYRK ",info);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsyrk",999999);
/***
*
*     Quick return if possible.
*
***/
}
/***
*
*     And when  alpha.eq.zero.
*
***/
if ((((n == 0)) || ((((((alpha == 0.0)) || ((k == 0)))) && ((beta == 1.0)))))) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsyrk",999999);
}
    if ((alpha == 0.0))  {
    if (upper)  {
    if ((beta == 0.0))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= j; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dsyrk",10);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsyrk",20);
}              //  Close for() loop.
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= j; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack310/Dsyrk",30);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsyrk",40);
}              //  Close for() loop.
}
}              //  Close else.
}
else  {
  if ((beta == 0.0))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = j; i <= n; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dsyrk",50);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsyrk",60);
}              //  Close for() loop.
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = j; i <= n; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack310/Dsyrk",70);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsyrk",80);
}              //  Close for() loop.
}
}              //  Close else.
}              //  Close else.
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsyrk",999999);
/***
*
*     Start the operations.
*
***/
}
/***
*
*        Form  C := alpha*A*A**T + beta*C.
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(trans,"N"))  {
    if (upper)  {
    {
for (j = 1; j <= n; j++) {
if ((beta == 0.0))  {
    {
for (i = 1; i <= j; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dsyrk",90);
}              //  Close for() loop.
}
}
else if ((beta != 1.0))  {
    {
for (i = 1; i <= j; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack310/Dsyrk",100);
}              //  Close for() loop.
}
}              // Close else if()
{
for (l = 1; l <= k; l++) {
if ((a[(j-(1))+(l-(1)) * (lda)+ _a_offset] != 0.0))  {
    temp = (alpha*a[(j-(1))+(l-(1)) * (lda)+ _a_offset]);
{
for (i = 1; i <= j; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]+(temp*a[(i-(1))+(l-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dsyrk",110);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dsyrk",120);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsyrk",130);
}              //  Close for() loop.
}
}
else  {
  {
for (j = 1; j <= n; j++) {
if ((beta == 0.0))  {
    {
for (i = j; i <= n; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dsyrk",140);
}              //  Close for() loop.
}
}
else if ((beta != 1.0))  {
    {
for (i = j; i <= n; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]);
//Dummy.label("com/polytechnik/lapack310/Dsyrk",150);
}              //  Close for() loop.
}
}              // Close else if()
{
for (l = 1; l <= k; l++) {
if ((a[(j-(1))+(l-(1)) * (lda)+ _a_offset] != 0.0))  {
    temp = (alpha*a[(j-(1))+(l-(1)) * (lda)+ _a_offset]);
{
for (i = j; i <= n; i++) {
c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]+(temp*a[(i-(1))+(l-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dsyrk",160);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dsyrk",170);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsyrk",180);
}              //  Close for() loop.
}
}              //  Close else.
/***
*
*        Form  C := alpha*A**T*A + beta*C.
*
***/
}
else  {
  if (upper)  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= j; i++) {
temp = 0.0;
{
for (l = 1; l <= k; l++) {
temp = (temp+(a[(l-(1))+(i-(1)) * (lda)+ _a_offset]*a[(l-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dsyrk",190);
}              //  Close for() loop.
}
if ((beta == 0.0))  {
    c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (alpha*temp);
}
else  {
  c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = ((alpha*temp)+(beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]));
}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dsyrk",200);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsyrk",210);
}              //  Close for() loop.
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = j; i <= n; i++) {
temp = 0.0;
{
for (l = 1; l <= k; l++) {
temp = (temp+(a[(l-(1))+(i-(1)) * (lda)+ _a_offset]*a[(l-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dsyrk",220);
}              //  Close for() loop.
}
if ((beta == 0.0))  {
    c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = (alpha*temp);
}
else  {
  c[(i-(1))+(j-(1)) * (ldc)+ _c_offset] = ((alpha*temp)+(beta*c[(i-(1))+(j-(1)) * (ldc)+ _c_offset]));
}              //  Close else.
//Dummy.label("com/polytechnik/lapack310/Dsyrk",230);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dsyrk",240);
}              //  Close for() loop.
}
}              //  Close else.
// *
}              //  Close else.
/***
*
*     End of DSYRK
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dsyrk",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dsyrk",999999);
return;
   }
} // End class.
