/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dtrmm.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dtrmm {

// *> \brief \b DTRMM
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DTRMM(SIDE,UPLO,TRANSA,DIAG,M,N,ALPHA,A,LDA,B,LDB)
// *
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION ALPHA
// *       INTEGER LDA,LDB,M,N
// *       CHARACTER DIAG,SIDE,TRANSA,UPLO
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION A(LDA,*),B(LDB,*)
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DTRMM  performs one of the matrix-matrix operations
// *>
// *>    B := alpha*op( A )*B,   or   B := alpha*B*op( A ),
// *>
// *> where  alpha  is a scalar,  B  is an m by n matrix,  A  is a unit, or
// *> non-unit,  upper or lower triangular matrix  and  op( A )  is one  of
// *>
// *>    op( A ) = A   or   op( A ) = A**T.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] SIDE
// *> \verbatim
// *>          SIDE is CHARACTER*1
// *>           On entry,  SIDE specifies whether  op( A ) multiplies B fro
// *>           the left or right as follows:
// *>
// *>              SIDE = 'L' or 'l'   B := alpha*op( A )*B.
// *>
// *>              SIDE = 'R' or 'r'   B := alpha*B*op( A ).
// *> \endverbatim
// *>
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>           On entry, UPLO specifies whether the matrix A is an upper o
// *>           lower triangular matrix as follows:
// *>
// *>              UPLO = 'U' or 'u'   A is an upper triangular matrix.
// *>
// *>              UPLO = 'L' or 'l'   A is a lower triangular matrix.
// *> \endverbatim
// *>
// *> \param[in] TRANSA
// *> \verbatim
// *>          TRANSA is CHARACTER*1
// *>           On entry, TRANSA specifies the form of op( A ) to be used i
// *>           the matrix multiplication as follows:
// *>
// *>              TRANSA = 'N' or 'n'   op( A ) = A.
// *>
// *>              TRANSA = 'T' or 't'   op( A ) = A**T.
// *>
// *>              TRANSA = 'C' or 'c'   op( A ) = A**T.
// *> \endverbatim
// *>
// *> \param[in] DIAG
// *> \verbatim
// *>          DIAG is CHARACTER*1
// *>           On entry, DIAG specifies whether or not A is unit triangula
// *>           as follows:
// *>
// *>              DIAG = 'U' or 'u'   A is assumed to be unit triangular.

// *>
// *>              DIAG = 'N' or 'n'   A is not assumed to be unit
// *>                                  triangular.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>           On entry, M specifies the number of rows of B. M must be at
// *>           least zero.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>           On entry, N specifies the number of columns of B.  N must b
// *>           at least zero.
// *> \endverbatim
// *>
// *> \param[in] ALPHA
// *> \verbatim
// *>          ALPHA is DOUBLE PRECISION.
// *>           On entry,  ALPHA specifies the scalar  alpha. When  alpha i
// *>           zero then  A is not referenced and  B need not be set befor
// *>           entry.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>           A is DOUBLE PRECISION array, dimension ( LDA, k ), where k
// *>           when  SIDE = 'L' or 'l'  and is  n  when  SIDE = 'R' or 'r'
// *>           Before entry  with  UPLO = 'U' or 'u',  the  leading  k by
// *>           upper triangular part of the array  A must contain the uppe
// *>           triangular matrix  and the strictly lower triangular part o
// *>           A is not referenced.
// *>           Before entry  with  UPLO = 'L' or 'l',  the  leading  k by
// *>           lower triangular part of the array  A must contain the lowe
// *>           triangular matrix  and the strictly upper triangular part o
// *>           A is not referenced.
// *>           Note that when  DIAG = 'U' or 'u',  the diagonal elements o
// *>           A  are not referenced either,  but are assumed to be  unity
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>           On entry, LDA specifies the first dimension of A as declare
// *>           in the calling (sub) program.  When  SIDE = 'L' or 'l'  the
// *>           LDA  must be at least  max( 1, m ),  when  SIDE = 'R' or 'r
// *>           then LDA must be at least max( 1, n ).
// *> \endverbatim
// *>
// *> \param[in,out] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array, dimension ( LDB, N )
// *>           Before entry,  the leading  m by n part of the array  B mus
// *>           contain the matrix  B,  and  on exit  is overwritten  by th
// *>           transformed matrix.
// *> \endverbatim
// *>
// *> \param[in] LDB
// *> \verbatim
// *>          LDB is INTEGER
// *>           On entry, LDB specifies the first dimension of B as declare
// *>           in  the  calling  (sub)  program.   LDB  must  be  at  leas
// *>           max( 1, m ).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup double_blas_level3
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Level 3 Blas routine.
// *>
// *>  -- Written on 8-February-1989.
// *>     Jack Dongarra, Argonne National Laboratory.
// *>     Iain Duff, AERE Harwell.
// *>     Jeremy Du Croz, Numerical Algorithms Group Ltd.
// *>     Sven Hammarling, Numerical Algorithms Group Ltd.
// *> \endverbatim
// *>
// *  =====================================================================
//
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Parameters ..
***/
/***
*     ..
*
*     Test the input parameters.
*
***/

public static void dtrmm (String side,
String uplo,
String transa,
String diag,
int m,
int n,
double alpha,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb)  {

label_999999:{

double temp= 0.0d;
int i= 0;
int info= 0;
int j= 0;
int k= 0;
int nrowa= 0;
boolean lside= false;
boolean nounit= false;
boolean upper= false;
side = side.substring((1)-1,1);
uplo = uplo.substring((1)-1,1);
transa = transa.substring((1)-1,1);
diag = diag.substring((1)-1,1);
lside = com.polytechnik.lapack310.Lsame.lsame(side,"L");
if (lside)  {
    nrowa = m;
}
else  {
  nrowa = n;
}              //  Close else.
nounit = com.polytechnik.lapack310.Lsame.lsame(diag,"N");
// *
upper = com.polytechnik.lapack310.Lsame.lsame(uplo,"U");
info = 0;
if ((((!lside)) && ((!com.polytechnik.lapack310.Lsame.lsame(side,"R")))))  {
    info = 1;
}
else if ((((!upper)) && ((!com.polytechnik.lapack310.Lsame.lsame(uplo,"L")))))  {
    info = 2;
}              // Close else if()
else if (((((!com.polytechnik.lapack310.Lsame.lsame(transa,"N"))) && ((!com.polytechnik.lapack310.Lsame.lsame(transa,"T")))) && ((!com.polytechnik.lapack310.Lsame.lsame(transa,"C")))))  {
    info = 3;
}              // Close else if()
else if ((((!com.polytechnik.lapack310.Lsame.lsame(diag,"U"))) && ((!com.polytechnik.lapack310.Lsame.lsame(diag,"N")))))  {
    info = 4;
}              // Close else if()
else if ((m < 0))  {
    info = 5;
}              // Close else if()
else if ((n < 0))  {
    info = 6;
}              // Close else if()
else if ((lda < Math.max(1, nrowa)))  {
    info = 9;
}              // Close else if()
else if ((ldb < Math.max(1, m)))  {
    info = 11;
}              // Close else if()
if ((info != 0))  {
    com.polytechnik.lapack310.Xerbla.xerbla("DTRMM ",info);
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dtrmm",999999);
/***
*
*     Quick return if possible.
*
***/
}
/***
*
*     And when  alpha.eq.zero.
*
***/
if (((m == 0) || (n == 0))) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dtrmm",999999);
}
    if ((alpha == 0.0))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack310/Dtrmm",10);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",20);
}              //  Close for() loop.
}
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dtrmm",999999);
/***
*
*     Start the operations.
*
***/
}
if (lside)  {
    /***
*
*           Form  B := alpha*A*B.
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(transa,"N"))  {
    if (upper)  {
    {
for (j = 1; j <= n; j++) {
{
for (k = 1; k <= m; k++) {
if ((b[(k-(1))+(j-(1)) * (ldb)+ _b_offset] != 0.0))  {
    temp = (alpha*b[(k-(1))+(j-(1)) * (ldb)+ _b_offset]);
{
for (i = 1; i <= (k-1); i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = (b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]+(temp*a[(i-(1))+(k-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dtrmm",30);
}              //  Close for() loop.
}
if (nounit) {
    temp = (temp*a[(k-(1))+(k-(1)) * (lda)+ _a_offset]);
}
    b[(k-(1))+(j-(1)) * (ldb)+ _b_offset] = temp;
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",40);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",50);
}              //  Close for() loop.
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
int _k_inc = -1;
for (k = m; k >= 1; k += _k_inc) {
if ((b[(k-(1))+(j-(1)) * (ldb)+ _b_offset] != 0.0))  {
    temp = (alpha*b[(k-(1))+(j-(1)) * (ldb)+ _b_offset]);
b[(k-(1))+(j-(1)) * (ldb)+ _b_offset] = temp;
if (nounit) {
    b[(k-(1))+(j-(1)) * (ldb)+ _b_offset] = (b[(k-(1))+(j-(1)) * (ldb)+ _b_offset]*a[(k-(1))+(k-(1)) * (lda)+ _a_offset]);
}
    {
for (i = (k+1); i <= m; i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = (b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]+(temp*a[(i-(1))+(k-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack310/Dtrmm",60);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",70);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",80);
}              //  Close for() loop.
}
}              //  Close else.
/***
*
*           Form  B := alpha*A**T*B.
*
***/
}
else  {
  if (upper)  {
    {
for (j = 1; j <= n; j++) {
{
int _i_inc = -1;
for (i = m; i >= 1; i += _i_inc) {
temp = b[(i-(1))+(j-(1)) * (ldb)+ _b_offset];
if (nounit) {
    temp = (temp*a[(i-(1))+(i-(1)) * (lda)+ _a_offset]);
}
    {
for (k = 1; k <= (i-1); k++) {
temp = (temp+(a[(k-(1))+(i-(1)) * (lda)+ _a_offset]*b[(k-(1))+(j-(1)) * (ldb)+ _b_offset]));
//Dummy.label("com/polytechnik/lapack310/Dtrmm",90);
}              //  Close for() loop.
}
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = (alpha*temp);
//Dummy.label("com/polytechnik/lapack310/Dtrmm",100);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",110);
}              //  Close for() loop.
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
temp = b[(i-(1))+(j-(1)) * (ldb)+ _b_offset];
if (nounit) {
    temp = (temp*a[(i-(1))+(i-(1)) * (lda)+ _a_offset]);
}
    {
for (k = (i+1); k <= m; k++) {
temp = (temp+(a[(k-(1))+(i-(1)) * (lda)+ _a_offset]*b[(k-(1))+(j-(1)) * (ldb)+ _b_offset]));
//Dummy.label("com/polytechnik/lapack310/Dtrmm",120);
}              //  Close for() loop.
}
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = (alpha*temp);
//Dummy.label("com/polytechnik/lapack310/Dtrmm",130);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",140);
}              //  Close for() loop.
}
}              //  Close else.
}              //  Close else.
}
else  {
  /***
*
*           Form  B := alpha*B*A.
*
***/
if (com.polytechnik.lapack310.Lsame.lsame(transa,"N"))  {
    if (upper)  {
    {
int _j_inc = -1;
for (j = n; j >= 1; j += _j_inc) {
temp = alpha;
if (nounit) {
    temp = (temp*a[(j-(1))+(j-(1)) * (lda)+ _a_offset]);
}
    {
for (i = 1; i <= m; i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = (temp*b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]);
//Dummy.label("com/polytechnik/lapack310/Dtrmm",150);
}              //  Close for() loop.
}
{
for (k = 1; k <= (j-1); k++) {
if ((a[(k-(1))+(j-(1)) * (lda)+ _a_offset] != 0.0))  {
    temp = (alpha*a[(k-(1))+(j-(1)) * (lda)+ _a_offset]);
{
for (i = 1; i <= m; i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = (b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]+(temp*b[(i-(1))+(k-(1)) * (ldb)+ _b_offset]));
//Dummy.label("com/polytechnik/lapack310/Dtrmm",160);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",170);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",180);
}              //  Close for() loop.
}
}
else  {
  {
for (j = 1; j <= n; j++) {
temp = alpha;
if (nounit) {
    temp = (temp*a[(j-(1))+(j-(1)) * (lda)+ _a_offset]);
}
    {
for (i = 1; i <= m; i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = (temp*b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]);
//Dummy.label("com/polytechnik/lapack310/Dtrmm",190);
}              //  Close for() loop.
}
{
for (k = (j+1); k <= n; k++) {
if ((a[(k-(1))+(j-(1)) * (lda)+ _a_offset] != 0.0))  {
    temp = (alpha*a[(k-(1))+(j-(1)) * (lda)+ _a_offset]);
{
for (i = 1; i <= m; i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = (b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]+(temp*b[(i-(1))+(k-(1)) * (ldb)+ _b_offset]));
//Dummy.label("com/polytechnik/lapack310/Dtrmm",200);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",210);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",220);
}              //  Close for() loop.
}
}              //  Close else.
/***
*
*           Form  B := alpha*B*A**T.
*
***/
}
else  {
  if (upper)  {
    {
for (k = 1; k <= n; k++) {
{
for (j = 1; j <= (k-1); j++) {
if ((a[(j-(1))+(k-(1)) * (lda)+ _a_offset] != 0.0))  {
    temp = (alpha*a[(j-(1))+(k-(1)) * (lda)+ _a_offset]);
{
for (i = 1; i <= m; i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = (b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]+(temp*b[(i-(1))+(k-(1)) * (ldb)+ _b_offset]));
//Dummy.label("com/polytechnik/lapack310/Dtrmm",230);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",240);
}              //  Close for() loop.
}
temp = alpha;
if (nounit) {
    temp = (temp*a[(k-(1))+(k-(1)) * (lda)+ _a_offset]);
}
    if ((temp != 1.0))  {
    {
for (i = 1; i <= m; i++) {
b[(i-(1))+(k-(1)) * (ldb)+ _b_offset] = (temp*b[(i-(1))+(k-(1)) * (ldb)+ _b_offset]);
//Dummy.label("com/polytechnik/lapack310/Dtrmm",250);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",260);
}              //  Close for() loop.
}
}
else  {
  {
int _k_inc = -1;
for (k = n; k >= 1; k += _k_inc) {
{
for (j = (k+1); j <= n; j++) {
if ((a[(j-(1))+(k-(1)) * (lda)+ _a_offset] != 0.0))  {
    temp = (alpha*a[(j-(1))+(k-(1)) * (lda)+ _a_offset]);
{
for (i = 1; i <= m; i++) {
b[(i-(1))+(j-(1)) * (ldb)+ _b_offset] = (b[(i-(1))+(j-(1)) * (ldb)+ _b_offset]+(temp*b[(i-(1))+(k-(1)) * (ldb)+ _b_offset]));
//Dummy.label("com/polytechnik/lapack310/Dtrmm",270);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",280);
}              //  Close for() loop.
}
temp = alpha;
if (nounit) {
    temp = (temp*a[(k-(1))+(k-(1)) * (lda)+ _a_offset]);
}
    if ((temp != 1.0))  {
    {
for (i = 1; i <= m; i++) {
b[(i-(1))+(k-(1)) * (ldb)+ _b_offset] = (temp*b[(i-(1))+(k-(1)) * (ldb)+ _b_offset]);
//Dummy.label("com/polytechnik/lapack310/Dtrmm",290);
}              //  Close for() loop.
}
}
//Dummy.label("com/polytechnik/lapack310/Dtrmm",300);
}              //  Close for() loop.
}
}              //  Close else.
}              //  Close else.
// *
}              //  Close else.
/***
*
*     End of DTRMM
*
***/
break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dtrmm",999999);
// dummy stmt
}//Dummy.label("com/polytechnik/lapack310/Dtrmm",999999);
return;
   }
} // End class.
