package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;

/** QVM calculations.
 *  @version $Id: AQVM.java,v 1.69 2022/06/29 08:04:53 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class AQVM {

final double p_eq,dp_eq,sigma2_eq,osp,pi_eq,sigma2pi_eq,dpi_eq,scale_io,scale_io0,scale_dj,scale_dj0,jo,io,scale_dip0,di,dip,dj,djp,scaleo_io,scaleo_dp_eq;

private final DeprecatedHelperBasisInterface obasis;
private final DeprecatedHelperBasisInterface [] ibasis;
AQVM(final DeprecatedHelperBasisInterface obasis,final DeprecatedHelperBasisInterface [] ibasis)
{
    this.obasis=obasis;
    this.ibasis=ibasis;
    final int neq_t=obasis.get_ntm(),neq_p=2+ibasis.length;
    if(neq_p>neq_t) throw new RuntimeException("neq_t="+neq_t+" < neq_p="+neq_p+"=("+ibasis.length+"+2)");
    final double [] i=new double [neq_t],j=new double [neq_t];
    obasis.setI(i);
    obasis.setIBS(j);
    final double [] dpk=new double [neq_t],dpl=new double [neq_t];
    final double [] matr=new double [neq_p*neq_p],rp_im=new double [neq_p],rps_im=new double [neq_p],
	rp_jm=new double [neq_p],rps_jm=new double [neq_p];
    double dp_o=Double.NaN,dp_dp=Double.NaN,i_dp=Double.NaN;
    for(int k=0;k<neq_p;k++)
    {
	if(k<=0) obasis.setDPn(dpk,0);
	else if(k<=1){
	    obasis.setDPn(dpk,1);
	    dp_o=obasis.get_f_o(dpk);
	    dp_dp=obasis.get_f_g(dpk,dpk);
	    i_dp=obasis.get_f_g(i,dpk);
	}
	else{
	    final DeprecatedHelperBasisInterface b=ibasis[k-2];
	    b.setDPn(dpk,1);
	    if(b.get_ntm()!=neq_t||b.get_npm()!=obasis.get_npm()) throw new RuntimeException(
		    " obasis.{ntm="+obasis.get_ntm()+",npm="+obasis.get_npm()+"}"+
		    " ibasis["+(k-2)+"].{ntm="+b.get_ntm()+",npm="+b.get_npm()+"}");
	}
	for(int l=0;l<=k;l++)
	{
	    if(l<=0) obasis.setDPn(dpl,0);
	    else if(l<=1) obasis.setDPn(dpl,1);
	    else ibasis[l-2].setDPn(dpl,1);
	    matr[k+l*neq_p]=matr[l+k*neq_p]=obasis.get_f_g(dpk,dpl);
	}
	rp_im[k]=rps_im[k]=obasis.get_f_g(i,dpk);
	rp_jm[k]=rps_jm[k]=obasis.get_f_g(j,dpk);
    }
    final double o_o=obasis.get_o_o(),fdp_o=dp_o/o_o,x_dp_dp=dp_dp-dp_o*fdp_o;
    this.p_eq=obasis.getPOffset()+fdp_o;
    this.sigma2_eq=x_dp_dp/o_o;
    final int [] ipvt=new int [neq_p];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,neq_p,neq_p,ipvt);
    if(pinfo!=0)
    {
	java.util.Arrays.fill(rp_im,Double.NaN);
	java.util.Arrays.fill(rp_jm,Double.NaN);
    }
    else
    {
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq_p,neq_p,ipvt,rp_im,1);
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq_p,neq_p,ipvt,rp_jm,1);
    }
    final double i_o=obasis.get_f_o(i),j_o=obasis.get_f_o(j),fi_o=i_o/o_o,fj_o=j_o/o_o,
	x_i_dp=i_dp-fdp_o*i_o,
	i_i=obasis.get_f_g(i,i),j_j=obasis.get_f_g(j,j),
	x_i_i=i_i-i_o*fi_o,sigma_i_i=i_i-TMatr.mulVV(rps_im,rp_im),
	x_j_j=j_j-j_o*fj_o,sigma_j_j=j_j-TMatr.mulVV(rps_jm,rp_jm);
    this.osp=x_i_dp/Math.sqrt(x_i_i*x_dp_dp);
    final double dpi=(x_i_dp/x_i_i)*fi_o;
    this.pi_eq=p_eq-dpi;
    this.sigma2pi_eq=this.sigma2_eq-(x_i_dp*x_i_dp/x_i_i)/o_o;
    final double dplast=obasis.getPLast()-obasis.getPOffset();
    this.dpi_eq=(osp*osp)*(dplast-fdp_o)+dpi;
    this.dp_eq=dplast*o_o-dp_o;

    this.jo=j_o;
    this.io=i_o;
    this.scale_io=Math.sqrt(o_o*sigma_i_i);
    this.scale_io0=Math.sqrt(o_o*x_i_i);
    this.scaleo_io=Math.sqrt(o_o*i_i);
    this.scaleo_dp_eq=Math.sqrt(this.dp_eq*this.dp_eq+o_o*x_dp_dp);
    this.scale_dip0=Math.sqrt(x_i_i/x_dp_dp);
    this.scale_dj=Math.sqrt(o_o*sigma_j_j);
    this.scale_dj0=Math.sqrt(o_o*x_j_j);

    double si=0,sj=0;
    for(int k=0;k<neq_p;k++)
    { // need to inverse sign when using our definition of dpk.
	rp_im[k]=-o_o*rp_im[k];
	rp_jm[k]=-o_o*rp_jm[k];
	final double dp;
	if(k<=0) dp=1;
	else if(k<=1) dp=obasis.getPLast()-obasis.getPOffset();
	else dp=ibasis[k-2].getPLast()-ibasis[k-2].getPOffset();
	si+=dp*rp_im[k];
	sj+=dp*rp_jm[k];
    }
    this.di=si;
    this.dip=rp_im[1]/o_o;
    this.dj=sj;
    this.djp=rp_jm[1]/o_o;
}



private static DeprecatedHelperBasisInterface swapI_IBS(final DeprecatedHelperBasisInterface odata)
{
    return new DeprecatedHelperBasisClone(odata){
	public void setIPn(final double [] i,final int n){throw new RuntimeException("Should not be used.");}
	public void setIBS(final double [] i){o.setI(i);}
	public void setI(final double [] i){o.setIBS(i);}
    };
}
private static DeprecatedHelperBasisInterface feed_ibasis_as_PPowers(final DeprecatedHelperBasisInterface odata,final DeprecatedHelperBasisInterface [] idata)
{
    return new DeprecatedHelperBasisClone(odata){
	public void setIPn(final double [] i,final int n){throw new RuntimeException("Should not be used.");}
	public int get_npm(){return 2+idata.length;}
	public void setDPn(final double [] pt,final int n){if(n<2) o.setDPn(pt,n); else idata[n-2].setDPn(pt,1);}
    };
}

static abstract class AQVMTest {
final String name;
final double eps,epssum;
AQVMTest(final String name,final double eps,final double epssum)
{
    this.name=name;
    this.eps=eps;
    this.epssum=epssum;
}

abstract AQVM getAQVM(final int tsize,final int ntm,final int neq_i,final java.util.Random r);

private static void matchQVMDPI(final QVMD q,final double p_eq,final double sigma2_eq,final double osp,final double pi_eq,final double sigma2pi_eq,final double dpi_eq,final double eps)
{
    UnitTests.checkDoubleRel(q.p_eq,"q.p_eq",p_eq,"p_eq",eps);
    UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",sigma2_eq,"sigma2_eq",eps);
    UnitTests.checkDoubleRel(q.osp,"q.osp",osp,"osp",eps);
    UnitTests.checkDoubleRel(q.pi_eq,"q.pi_eq",pi_eq,"pi_eq",eps);
    UnitTests.checkDoubleRel(q.sigma2pi_eq,"q.sigma2pi_eq",sigma2pi_eq,"sigma2pi_eq",eps);
    UnitTests.checkDoubleRel(q.dpi_eq,"q.dpi_eq",dpi_eq,"dpi_eq",eps);
}
private static void matchQVMD(final QVMD q,final double di,final double dip,final double io,final double scale_io,final double scale_io0,final double eps)
{
    UnitTests.checkDoubleRel(q.io,"q.io",io,"io",eps);
    UnitTests.checkDoubleRel(q.di,"q.di",di,"di",eps);
    UnitTests.checkDoubleRel(q.dip,"q.dip",dip,"dip",eps);
    UnitTests.checkDoubleRel(q.scale_io,"q.scale_io",scale_io,"scale_io",eps);
    UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",scale_io0,"scale_io0",eps);
}
private static void matchAQVM(final AQVM q,final AQVM s,final double eps,final double epssum)
{
    UnitTests.checkDoubleRel(q.p_eq,"q.p_eq",s.p_eq,"s.p_eq",eps);
    UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",s.sigma2_eq,"s.sigma2_eq",eps);
    UnitTests.checkDoubleRel(q.di,"q.di",s.di,"s.di",eps);
    UnitTests.checkDoubleRel(q.dip,"q.dip",s.dip,"s.dip",eps);
    UnitTests.checkDoubleRel(q.scale_io,"q.scale_io",s.scale_io,"s.scale_io",epssum);
    UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",s.scale_io0,"s.scale_io0",eps);
    UnitTests.checkDoubleRel(q.dj,"q.dj",s.dj,"s.dj",eps);
    UnitTests.checkDoubleRel(q.scale_dj,"q.scale_dj",s.scale_dj,"s.scale_dj",epssum);
    UnitTests.checkDoubleRel(q.scale_dj0,"q.scale_dj0",s.scale_dj0,"s.scale_dj0",eps);
}

private void checkQVMD(final int tsize,final java.util.Random r,final int neq_t)
{
    System.err.println(name+":AQVMTest:checkQVMD(): neq_t="+neq_t);
    AQVM q=getAQVM(tsize,neq_t,0,r);
    final QVMD qvmd_i=new QVMD(q.obasis),qvmd_j=new QVMD(swapI_IBS(q.obasis));
    matchQVMDPI(qvmd_i,q.p_eq,q.sigma2_eq,q.osp,q.pi_eq,q.sigma2pi_eq,q.dpi_eq,eps);
    matchQVMD(qvmd_i,q.di,q.dip,q.io,q.scale_io,q.scale_io0,eps);
    matchQVMD(qvmd_j,q.dj,q.djp,q.jo,q.scale_dj,q.scale_dj0,eps);
    q=new AQVM(swapI_IBS(q.obasis),q.ibasis);
    matchQVMD(qvmd_j,q.di,q.dip,q.io,q.scale_io,q.scale_io0,eps);
    matchQVMDPI(qvmd_j,q.p_eq,q.sigma2_eq,q.osp,q.pi_eq,q.sigma2pi_eq,q.dpi_eq,eps);
    matchQVMD(qvmd_i,q.dj,q.djp,q.jo,q.scale_dj,q.scale_dj0,eps);
    System.err.println(name+":AQVMTest:checkQVMD(): TEST OK neq_t="+neq_t);
}
private void checkSPIMin(final int tsize,final java.util.Random r,final int neq_t,final int neq_i)
{
    System.err.println(name+":AQVMTest:checkSPIMin(): neq_t="+neq_t+" neq_i="+neq_i);
    final AQVM q=getAQVM(tsize,neq_t,neq_i,r);
    SPIMin.SPIMinData xx=new SPIMin(neq_t,2+neq_i,feed_ibasis_as_PPowers(q.obasis,q.ibasis)).getI0P();
    UnitTests.checkDoubleRel(q.scale_io,"q.scale_io",xx.scale_io,"xx.scale_io",eps);
    UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",xx.scale_io0,"xx.scale_io0",eps);
    double s=0;
    for(int k=0;k<xx.inter.length;k++)
    {
	final double dp=k<=0?1:(k<=1?q.obasis.getPLast()-q.obasis.getPOffset():q.ibasis[k-2].getPLast()-q.ibasis[k-2].getPOffset());
	s+=xx.inter[k]*dp;
    }
    UnitTests.checkDoubleRel(q.di,"q.di",s,"sum i",eps);
    xx=new SPIMin(neq_t,2+neq_i,swapI_IBS(feed_ibasis_as_PPowers(q.obasis,q.ibasis))).getI0P();
    UnitTests.checkDoubleRel(q.scale_dj,"q.scale_dj",xx.scale_io,"xx.scale_io",eps);
    UnitTests.checkDoubleRel(q.scale_dj0,"q.scale_dj0",xx.scale_io0,"xx.scale_io0",eps);
    s=0;
    for(int k=0;k<xx.inter.length;k++)
    {
	final double dp=k<=0?1:(k<=1?q.obasis.getPLast()-q.obasis.getPOffset():q.ibasis[k-2].getPLast()-q.ibasis[k-2].getPOffset());
	s+=xx.inter[k]*dp;
    }
    UnitTests.checkDoubleRel(q.dj,"q.dj",s,"sum j",eps);
    System.err.println(name+":AQVMTest:checkSPIMin(): TEST OK neq_t="+neq_t+" neq_i="+neq_i);
}
void checkSwap(final int tsize,final java.util.Random r,final int neq_t,final int neq_i)
{
    System.err.println(name+":AQVMTest:checkSwap(): neq_t="+neq_t+" neq_i="+neq_i);
    if(neq_i<1)
    {
	System.err.println(name+":AQVMTest:checkSwap(): neq_i="+neq_i+" skipping test");
	return;
    }
    final AQVM q=getAQVM(tsize,neq_t,neq_i,r);
    final DeprecatedHelperBasisInterface [] ibasis=q.ibasis.clone();
    for(int t=0;t<20;t++)
    {
	for(int m=neq_i;--m>=1;)
	{
	    final int n=r.nextInt(m+1);
	    final DeprecatedHelperBasisInterface b=ibasis[n];ibasis[n]=ibasis[m];ibasis[m]=b;
	}
	matchAQVM(q,new AQVM(q.obasis,ibasis),eps,epssum);
    }
    final int is=r.nextInt(neq_i);
    final DeprecatedHelperBasisInterface b=ibasis[is];ibasis[is]=q.obasis;
    final AQVM s=new AQVM(new DeprecatedHelperBasisClone(b){
	    public void setIBS(final double [] i){q.obasis.setIBS(i);}
	    public void setI(final double [] i){q.obasis.setI(i);}},ibasis);
    UnitTests.checkDoubleRel(q.di,"q.di",s.di,"s.di",eps);
    UnitTests.checkDoubleRel(q.scale_io,"q.scale_io",s.scale_io,"s.scale_io",epssum);
    UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",s.scale_io0,"s.scale_io0",eps);
    UnitTests.checkDoubleRel(q.dj,"q.dj",s.dj,"s.dj",eps);
    UnitTests.checkDoubleRel(q.scale_dj,"q.scale_dj",s.scale_dj,"s.scale_dj",epssum);
    UnitTests.checkDoubleRel(q.scale_dj0,"q.scale_dj0",s.scale_dj0,"s.scale_dj0",eps);
    System.err.println(name+":AQVMTest:checkSwap(): TEST OK neq_t="+neq_t+" neq_i="+neq_i);
}
void checkExternal(final int tsize,final java.util.Random r,final int neq_t,final int neq_i){}

void checkCalculations(final int tsize,final java.util.Random r,final int neq_t,final int neq_i)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":AQVMTest:checkCalculations(): test with N points="+tsize+
		       " neq_t="+neq_t+" neq_i="+neq_i+" eps="+eps+" epssum="+epssum);
    checkQVMD(tsize,r,neq_t);
    checkSPIMin(tsize,r,neq_t,neq_i);
    checkSwap(tsize,r,neq_t,neq_i);
    checkExternal(tsize,r,neq_t,neq_i);
    System.err.println(name+":AQVMTest:checkCalculations(): test OK with N points="+tsize+
		       " neq_t="+neq_t+" neq_i="+neq_i+" eps="+eps+" epssum="+epssum);
}
void runTests()
{
    System.err.println(name+":AQVMTest: starting tests; ver=$Id: AQVM.java,v 1.69 2022/06/29 08:04:53 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79091);
    for(int neq_t=1;neq_t<=12;neq_t++)
    {
	for(int neq_i=0;neq_i<=4;neq_i++)
	{
	    if(neq_i+2>neq_t) continue;
	    if(neq_i+2>neq_t-3) continue; // give extra dimensions to avoid degeneracy
	    for(int i=0;i<3;i++)
	    {
		checkCalculations(103210+r.nextInt(103210),r,neq_t,neq_i);
	    }
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":AQVMTest: tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: AQVM.java,v 1.69 2022/06/29 08:04:53 mal Exp $");
}
} // END OF class AQVMTest

} // END OF class AQVM
