package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.utils.UnitTests;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: AQVMLaguerre.java,v 1.3 2019/01/06 23:50:30 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class AQVMLaguerre extends AQVM {

private final DeprecatedHelperLaguerreBasis odata;
private final DeprecatedHelperLaguerreBasis [] idata;
AQVMLaguerre(final DeprecatedHelperLaguerreBasis obasis,final DeprecatedHelperLaguerreBasis [] ibasis)
{
    super(obasis,ibasis);
    this.odata=obasis;
    this.idata=ibasis;
}

public static AQVMLaguerre calculate(final int neq_t,
					    final int neq_i,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final Trade [][] itb,
					    final int [] itbsize,
					    final AQVMLaguerre ref)
{
    final int npm=3;
    final DeprecatedHelperLaguerreBasis o=
	DeprecatedHelperLaguerreBasis.calculateBasis(neq_t,
				     npm,
				     currenttime,
				     tau,
				     tb,
				     tsize,
				     ref!=null?ref.odata:null);
    final DeprecatedHelperLaguerreBasis [] i=new DeprecatedHelperLaguerreBasis [neq_i];
    for(int k=0;k<neq_i;k++)
    {
	i[k]=DeprecatedHelperLaguerreBasis.calculateBasis(neq_t,
					  npm,
					  currenttime,
					  tau,
					  itb[k],
					  itbsize[k],
					  ref!=null?ref.idata[k]:null);
    }
    return new AQVMLaguerre(o,i);
}


public static void main(String [] args)
{
    new AQVMTest("AQVMLaguerre:",1e-6,1e-3){
	AQVMLaguerre getAQVM(final int tsize,final int ntm,final int neq_i,final java.util.Random r)
	{
	    final Trade [] o_tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
	    long currenttime=o_tb[o_tb.length-1].time+1000;
	    final long tau=(o_tb[o_tb.length-1].time-o_tb[0].time)/24;
	    final Trade [][] i_tb=new Trade [neq_i][];
	    final int [] i_tb_size=new int [neq_i];
	    for(int i=0;i<neq_i;i++)
	    {
		final Trade [] tb=i_tb[i]=CalculateKendall.TradeEMoments.getRandomTrades(tsize+r.nextInt(tsize+1),r);
		i_tb_size[i]=tb.length;
		if(currenttime<tb[tb.length-1].time)
		{
		    currenttime=tb[tb.length-1].time;
		}
	    }
	    return calculate(ntm,neq_i,currenttime,tau,o_tb,tsize,i_tb,i_tb_size,null);
	}}.runTests();
}

}
