package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.Skewness;
import com.polytechnik.freemoney.MatricesFromPI;

/**
 * Try to predict using basis transfrom.
 * @version $Id: ATestForBasisUnitaryTransforms.java,v 1.34 2022/08/15 01:40:16 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
*/
public class ATestForBasisUnitaryTransforms {
    final EVXData I;
    public final double p_IH,p_IH_diagI,p_IHvar_diagI,p_0,p_0_diagI,
	PextAtImax,p0w,p0Iw,dp0ModStates,dp0ModStatespi;

    ATestForBasisUnitaryTransforms(final int n,
				    final double p_offset,
				    final double [] QQt,
				    final double [] QQi,
				    final double [] QQpi,
				    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				    final java.util.function.Function<double [],double []> getEDPsi,
				    final double eps){
	final int ixH=n-1;
	this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	this.p_IH=p_offset+I.getFPsi(ixH,QQpi)/I.sH;

	final double [] bpsi0=EVXData.convertPsiFromQtoB_Basis(n,I.psi0,I.ev,QQt),
	    bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	    bQQp=MatricesFromPI.getbQQpFrombQQpi(bQQpi,I),
	    bQQp_diagI=TMatr.getDiagMatrix(TMatr.getMatrixDiagElements(n,bQQp)),
	    psiH=I.getPsi(ixH),DpsiH=getEDPsi.apply(psiH),
	    bpsiH=EVXData.convertPsiFromQtoB_Basis(n,psiH,I.ev,QQt),
	    bDpsiH=EVXData.convertPsiFromQtoB_Basis(n,DpsiH,I.ev,QQt),
	    bDpsiH_ort=TMatr.sumAcB(bDpsiH,-TMatr.mulVV(bDpsiH,bpsiH),bpsiH);
	p_IH_diagI=p_offset+TMatr.getAMatrA(bpsiH,bQQp_diagI,n);
	p_IHvar_diagI=p_offset+TMatr.getAMatrA(bDpsiH_ort,bQQp_diagI,n)/TMatr.mulVV(bDpsiH_ort,bDpsiH_ort);
	p_0=p_offset+TMatr.getAMatrA(bpsi0,bQQp,n);
	p_0_diagI=p_offset+TMatr.getAMatrA(bpsi0,bQQp_diagI,n);

	final double [] ev_bQQp=M.EV.getEV(n,bQQp,true),
	    UNIT=TMatr.getUnitMatrix(n),
	    PeValues=java.util.Arrays.copyOf(ev_bQQp,n),
	    bbpsi0=EVXData.convertPsiFromQtoB_Basis(n,bpsi0,ev_bQQp,UNIT),
	    bQQi=TMatr.getDiagMatrix(I.getEValues()),
	    PatI=TMatr.getMatrixDiagElements(n,bQQp),
	    IatP=TMatr.getMatrixDiagElementsInBasis(n,ev_bQQp,bQQi),
	    w0P=new double [n];
	int indP_imax=0;
	for(int k=0;k<n;k++){
	    if(IatP[k]>=IatP[indP_imax]){
		indP_imax=k;
	    }
	    final double [] bpsik=EVXData.getPsiFromBasis(k,ev_bQQp,n);
	    w0P[k]=Math.pow(TMatr.mulVV(bpsi0,bpsik),2);
	}
	PextAtImax=p_offset+ev_bQQp[indP_imax];
	p0w=p_offset+TMatr.mulVV(w0P,PeValues)/TMatr.sumElements(w0P);
	p0Iw=p_offset+TMatr.mulVVV(w0P,PeValues,IatP)/TMatr.mulVV(w0P,IatP);

	if(eps>0){
	    UnitTests.checkDoubleRel(p_0,"p_0",p0w,"p0w",eps);
	    UnitTests.checkDoubleRel(TMatr.sumElements(w0P),"sum(w0P)",1,"1",eps);
	    // check bpsi0 in new basis
	    UnitTests.checkDoubleRel(I.s0,"I.s0",TMatr.mulV2V(bpsi0,I.getEValues()),"<bpsi0|bQQi|bpsi0>",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psi0,QQpi,n),"pI0",TMatr.getAMatrA(bpsi0,bQQpi,n),"<bpsi0|bQQpi|bpsi0>",eps);
	    // test the matrix in |I|psi>=lambda|psi> basis
	    for(int j=0;j<n;j++){
		for(int k=0;k<n;k++){
		    UnitTests.checkDoubleRel(I.getFPsiPsi(j,k,QQpi)/(Math.sqrt(I.ev[j])*Math.sqrt(I.ev[k])),"p_jk",bQQp[j*n+k],"bQQp[j,k]",eps);
		}
	    }
	    // check matrix transform and state invariance
	    final double []
		IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
		bbQQi=EVXData.convertMatrFromQtoB_Basis(n,bQQi,ev_bQQp),
		bbQQp=TMatr.getDiagMatrix(java.util.Arrays.copyOf(ev_bQQp,n)),
		_bQQp=EVXData.convertMatrFromBtoQ_Basis(n,bbQQp,ev_bQQp,UNIT),
		QQp=EVXData.convertMatrFromBtoQ_Basis(n,_bQQp,I.ev,QQt),
		QQp_diagI=EVXData.convertMatrFromBtoQ_Basis(n,bQQp_diagI,I.ev,QQt);
	    UnitTests.checkDoubleArrayRel1(bQQp,"bQQp",_bQQp,"_bQQp",eps);
	    UnitTests.checkDoubleArrayRel1(TMatr.getMatrixDiagElements(n,bQQp),"diag bQQp",PatI,"PatI",eps);
	    UnitTests.checkDoubleArrayRel1(TMatr.getMatrixDiagElements(n,bbQQi),"diag bbQQi",IatP,"IatP",eps);

	    // spur invariance
	    UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,QQp),"spur IQQt*QQp",TMatr.sumElements(PatI),"sum(PatI)",eps);
	    UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,QQi),"spur IQQt*QQi",TMatr.sumElements(IatP),"sum(IatP)",eps);
	    UnitTests.checkDoubleRel(TMatr.mulVV(PatI,I.getEValues()),"spur PI",TMatr.mulVV(IatP,java.util.Arrays.copyOf(ev_bQQp,n)),"spur IP",eps);

	    // know states
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(bbpsi0,bbQQp,n),"<bbpsi0|bbQQp|bbpsi0>",TMatr.getAMatrA(I.psi0,QQp,n),"<psi0|QQp|psi0>",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQp,n),"<bpsi0|bQQp|bpsi0>",TMatr.getAMatrA(I.psi0,QQp,n),"<psi0|QQp|psi0>",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQp_diagI,n),"<bpsi0|bQQp_diagI|bpsi0>",TMatr.getAMatrA(I.psi0,QQp_diagI,n),"<psi0|QQp_diagI|psi0>",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(bbpsi0,bbQQi,n),"<bbpsi0|bbQQi|bbpsi0>",I.s0,"I.s0",eps);

	    // I-basis check
	    for(int k=0;k<n;k++){
		final double [] bpsi=new double [n];bpsi[k]=0.35;
		for(int l=0;l<n;l++){
		    bpsi[l]=1; // for k=l match |I| states, otherwise some random wavefunction
		    final double [] bbpsi=EVXData.convertPsiFromQtoB_Basis(n,bpsi,ev_bQQp,UNIT);
		    UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi,bQQp,n),"<bpsi|bQQp|bpsi>",TMatr.getAMatrA(bbpsi,bbQQp,n),"<bbpsi|bbQQp|bbpsi>",eps);
		    UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi,bQQp,n),"<bpsi|bQQp|bpsi>",TMatr.mulV2V(bbpsi,java.util.Arrays.copyOf(ev_bQQp,n)),"<bbpsi|bbQQp|bbpsi>",eps);
		    UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi,bQQi,n),"<bpsi|bQQi|bpsi>",TMatr.getAMatrA(bbpsi,bbQQi,n),"<bbpsi|bbQQi|bbpsi>",eps);
		}
	    }
	    final double []
		rqI=RayleighQuotient.getRQVariationValues(psiH,DpsiH,null,QQi,QQt,n),
		rqp=RayleighQuotient.getRQVariationValues(psiH,DpsiH,null,QQp,QQt,n),
		brqI=RayleighQuotient.getRQVariationValues(bpsiH,bDpsiH,null,TMatr.getDiagMatrix(I.getEValues()),UNIT,n),
		brqp=RayleighQuotient.getRQVariationValues(bpsiH,bDpsiH,null,bQQp,UNIT,n),
		brqp_diagI=RayleighQuotient.getRQVariationValues(bpsiH,bDpsiH,null,bQQp_diagI,UNIT,n);
	    UnitTests.checkDoubleArrayRel1(rqI,"rqI",brqI,"brqI",eps);
	    UnitTests.checkDoubleArrayRel1(rqp,"rqp",brqp,"brqp",eps);
	    UnitTests.checkDoubleRel(rqI[1],"rqI[1]",0,"0",eps);
	    UnitTests.checkDoubleRel(brqp_diagI[1],"brqp_diagI[1]",0,"0",eps);
	    final double [] DpsiH_ort=TMatr.sumAcB(DpsiH,-TMatr.getAMatrB(psiH,DpsiH,QQt,n),psiH);
	    UnitTests.checkDoubleRel(p_IHvar_diagI-p_offset,"p_IHvar_diagI-p_offset",TMatr.getAMatrA(DpsiH_ort,QQp_diagI,n)/TMatr.getAMatrA(DpsiH_ort,QQt,n),"p at DpsiH_ort",eps);
	    UnitTests.checkDoubleRel((p_IHvar_diagI-p_IH_diagI)*TMatr.getAMatrA(DpsiH_ort,QQt,n),"(p_IHvar_diagI-p_IH_diagI)*norm2",0.5*brqp_diagI[2],"0.5*brqp_diagI[2]",eps);

	    // diag
	    System.err.println("ATestForBasisUnitaryTransforms: test OK n="+n+" eps="+eps);
	}

	final double [] bQQp_offdiag=bQQp.clone(),bQQpi_offdiag=bQQpi.clone();
	for(int k=0;k<n;k++) {
	    bQQp_offdiag[k+n*k]=0;
	    bQQpi_offdiag[k+n*k]=0;
	}
	final double [] ev_bQQp_offdiag=M.EV.getEV(n,bQQp_offdiag,true),
	    IatP_offdiag=TMatr.getMatrixDiagElementsInBasis(n,ev_bQQp_offdiag,bQQi),
	    eValued_bQQp_offdiag=java.util.Arrays.copyOf(ev_bQQp_offdiag,n);
	dp0ModStates=TMatr.getAMatrA(bpsi0,bQQp_offdiag,n);

	final double [] ev_bQQpi_offdiag=M.EV.getEV(n,bQQpi_offdiag,true),
	    IatPi_offdiag=TMatr.getMatrixDiagElementsInBasis(n,ev_bQQpi_offdiag,bQQi),
	    eValued_bQQpi_offdiag=java.util.Arrays.copyOf(ev_bQQpi_offdiag,n);
	dp0ModStatespi=TMatr.getAMatrA(bpsi0,bQQpi_offdiag,n);

	if(eps>0){
	    UnitTests.checkDoubleRel(TMatr.sumElements(eValued_bQQp_offdiag),"sum eValued_bQQp_offdiag",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.sumElements(IatP_offdiag),"sum IatP_offdiag",TMatr.sumElements(IatP),"sum IatP",eps);

	    System.err.println("ATestForBasisUnitaryTransforms: eValued_bQQp_offdiag="+java.util.Arrays.toString(eValued_bQQp_offdiag)+" sum="+TMatr.sumElements(eValued_bQQp_offdiag)+" sum pi offdiag="+TMatr.sumElements(eValued_bQQpi_offdiag)+" n="+n+" eps="+eps);
	}

    }
}
