package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.outils.*;
import com.polytechnik.realtimebook.*;

/** BQVM and Rurik measure (continued).
 *  @version $Id: BQVM.java,v 1.227 2022/11/20 08:48:26 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class BQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;

/** Rurik evolution. */
final double I0,I0sam,K00t,A0,SA0,sigma2T_SA0,sigma2T_A0,K11t,A1,SA1,dS,dSe,dSf,sigma2T_dS,XM,AM,SAM,AE,AEt,SAE,SBE,AR,ARt,SAR,SBR,AEl,AElt,SAEl,SBEl,ARl,ARlt,SARl,SBRl;
final double SArvsL,SArvsH,SARrvsL,SARrvsH;
final double [] kkt,kka,kkrvs;
final double [] ev,evcut,sQQv,rve,rvecut,o,psiIE,psiIv,o0t,o0v,psiL,psiH,psi_p_L,psi_p_H,sQQt,sQQMrvs,sQQMts,sQQMdp,sIQQt,sQQMvs,pev,sQQMpv,sQQMrts,sQQx,sQQMt_sam,sQQva,sQQva_sam,sQQMp2t,sQQMp1t,sQQMp0t; // for unit test
final double o_o,o_o_v,wL,wH,wIE,IE,wPL,sPL,wPH,sPH;
final double ptE,pt0,pvE,pv0,sigma2_ptE,sigma2_pt0,sigma2_pvE,sigma2_pv0;
final EVXData RVS,RTS,DPS,DPt0,D2Pt0,RVSV,RTST,RTSTsam,IT,ITshifted,ITsam,DS,DSsam,DSsamV,IT0R,DSva,DSta,DSva_sam,DSta_sam,ITinv,DStaI,DStaI0,TS,TSsam,VS,VSsam,RVSsam,VSV,TSV;
final double sIE,pH0,sH0,ppv0,ppt0,ppvH,pptH,pf0,pfL,pfLt,peamL,peamH,pev2oamL,pev2oamH,pev2oDamL,pev2oDamH;

final EVXData DSI,DSImI,DSIpI,DSImI_testV,DSIpI_testV,P2VOL,PVOL,PX2VOL,PXVOL;
final EVXData DVSta,DRVSta,DSmu,DSRmu,DSmu_sam,DSRmu_sam,P_sam,EIsell,EIbuy,EAM,EV2oAM,EV2oDAM,EAMM;
final M2Data mdp,mS;
final MXData mpv,mDpv;
final double DPt0_f0,D2Pt0_f0;
final TwoCloseRootsFinder.RootsData r_mdp,r_mvdp;
final TwoRootsIFX rm0_kkdp_kkt,rm0_kkvdp_kkt,rm0_kkdp_kkiv,rm0_kkdp_kka;
final double c_dp_i_x0;
final TwoRootsIFD rms_kkdp_kka,rms_kkvdp_kkt;

BQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final int nmoms=2*neq-1;

    final double [] vmom=new double [nmoms],tmom=new double [nmoms],rvsmom=new double [nmoms],rtsmom=new double [nmoms],vsmom=new double [nmoms],tsmom=new double [nmoms],pvmom=new double [nmoms],p2vmom=new double [nmoms],ptmom=new double [nmoms],p2tmom=new double [nmoms],dpmom=new double [nmoms],tmom_sam=new double [nmoms],dvsmom=new double [nmoms],isell=new double [nmoms],ibuy=new double [nmoms],amoms=new double [nmoms];
    access.setM(vmom,0,access.M_I,odata,nmoms);
    access.setM(amoms,0,access.M_A,odata,nmoms);
    access.setMT(tmom,nmoms);
    access.setM(tmom_sam,0,access.M_T,odata,nmoms);
    access.setM(rvsmom,0,access.M_RVS,odata,nmoms);
    access.setM(rtsmom,0,access.M_RTS,odata,nmoms);
    access.setM(vsmom,0,access.M_VS,odata,nmoms);
    access.setM(tsmom,0,access.M_TS,odata,nmoms);
    access.setM(pvmom,1,access.M_I,odata,nmoms);
    access.setM(p2vmom,2,access.M_I,odata,nmoms);
    access.setM(ptmom,1,access.M_T,odata,nmoms);
    access.setM(p2tmom,2,access.M_T,odata,nmoms);
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setM(isell,0,access.M_ISELL,odata,nmoms);
    access.setM(ibuy,0,access.M_IBUY,odata,nmoms);
    final double [][] QQ=access.getQQ_Matrs(neq,1,new double [][]{vmom,tmom,rvsmom,rtsmom,vsmom,tsmom,pvmom,p2vmom,ptmom,p2tmom,tmom_sam,amoms,dpmom});
    final double [] QQMv=QQ[0],QQMt=QQ[1],QQMrvs=QQ[2],QQMrts=QQ[3],QQMvs=QQ[4],QQMts=QQ[5],QQMpv=QQ[6],QQMp2v=QQ[7],QQMpt=QQ[8],QQMp2t=QQ[9],QQMt_sam=QQ[10],QQMa=QQ[11],QQMdp=QQ[12],
	IQQt=getDiagInverted(neq,QQMt),
	QQrvs=getIQQdiag_F_IQQdiag(neq,IQQt,QQMrvs),
	QQa=getIQQdiag_F_IQQdiag(neq,IQQt,QQMv),
	IQQv=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQMv),
	KQQiGim1dp=getIQQdiag_F_IQQdiag(neq,IQQt,TMatr.mulMMM(neq,QQMv,IQQt,QQMdp)),
	KQQdp=getIQQdiag_F_IQQdiag(neq,IQQt,QQMdp),
	KQQdpGtm1dp=getIQQdiag_F_IQQdiag(neq,IQQt,TMatr.mulMMM(neq,QQMdp,IQQt,QQMdp)),
	KQQIGtm1I=getIQQdiag_F_IQQdiag(neq,IQQt,TMatr.mulMMM(neq,QQMv,IQQt,QQMv));

    final double [][] kkk=access.getKK(neq,new double [][] {IQQt,QQa,QQrvs,KQQiGim1dp,KQQdp,IQQv,KQQdpGtm1dp,KQQIGtm1I});
    kkt=kkk[0];
    kka=kkk[1];
    kkrvs=kkk[2];

    final double x0=getX0();

    final double [] kkdp=kkk[4],kkvdp=kkk[3],kkiv=kkk[5],kkdp2=kkk[6],kkI2=kkk[7];
    r_mdp=getTRD(kkt,kkdp);
    r_mvdp=getTRD(kkt,kkvdp);
    rm0_kkdp_kkt=getTRIFX(kkt,kkdp);
    rm0_kkvdp_kkt=getTRIFX(kkt,kkvdp);
    rm0_kkdp_kkiv=getTRIFX(kkiv,kkdp);
    rm0_kkdp_kka=getTRIFX(kka,kkdp);
    this.c_dp_i_x0=access.fx(kkvdp,x0)/Math.sqrt(access.fx(kkdp2,x0)*access.fx(kkI2,x0));


    rms_kkdp_kka=getTRIFD(kka,kkdp);
    rms_kkvdp_kkt=getTRIFD(kkt,kkvdp);


    K00t=access.fx(kkt,x0);
    A0=access.fx(kka,x0)/K00t;
    SA0=access.fx(kkrvs,x0)/K00t;
    sigma2T_SA0=TMatr.getSpurMM(neq,QQrvs,QQMrvs)-SA0*SA0;
    sigma2T_A0=TMatr.getSpurMM(neq,QQa,QQMv)-A0*A0;

    final double [] xall=getXAll(neq,IQQt);
    final double x1=xall.length>=2?xall[1]:Double.NaN;
    K11t=access.fx(kkt,x1);
    A1=access.fx(kka,x1)/K11t;
    SA1=access.fx(kkrvs,x1)/K11t;

    dS=(SA1)*(A1/A0-1);
    dSe=(SA1)*(1-A0/A1);
    dSf=(SA1)*(A1/A0)-(SA0);
    sigma2T_dS=sigma2T_SA0;

    double Imax=Double.NEGATIVE_INFINITY;int ind_Imax=-1;
    for(int k=1;k<xall.length;k++) // start with k=1, k=0 is Radau edge node
    {
	final double I=access.fx(kka,xall[k])/access.fx(kkt,xall[k]);
	if(I>=Imax){Imax=I;ind_Imax=k;}
    }
    if(ind_Imax>0)
    {
	XM=xall[ind_Imax];
	final double K=access.fx(kkt,XM);
	AM=access.fx(kka,XM)/K;
	SAM=access.fx(kkrvs,XM)/K;
    }
    else XM=AM=SAM=Double.NaN;


    this.o=getOKDiagT(neq,QQMt);
    this.o_o=access.fx(o,x0);
    final double [] r=new double [neq];
    o0t=new double [neq];
    for(int k=0;k<neq;k++) {o0t[k]=1;r[k]=access.fx(o0t,x0);o0t[k]=0;}
    o0v=TMatr.getMatr_x_A(neq,IQQv,r);
    this.o_o_v=access.fx(o0v,x0);
    this.I0=K00t/o_o_v;
    final double sqrt_o_o=Math.sqrt(o_o),sqrt_o_o_v=Math.sqrt(o_o_v);
    for(int k=0;k<neq;k++) {o0t[k]=o[k]/sqrt_o_o;o0v[k]/=sqrt_o_o_v;}
    psiIE=new double [neq];
    ev=hh_getSymEVDiagT(neq,QQMv,QQMt);
    System.arraycopy(ev,neq+(0)*neq,psiIE,0,neq);
    psiIv=new double [neq];
    final double p_offset=getPOffset(),imax=ev[0],sqrtimax=Math.sqrt(imax);
    IE=imax;
    wIE=access.fx(psiIE,x0)/sqrt_o_o;
    for(int k=0;k<neq;k++) psiIv[k]=psiIE[k]/sqrtimax;
    final double dptE=TMatr.getAMatrA(psiIE,QQMpt,neq),
	dpt0=TMatr.getAMatrA(o0t,QQMpt,neq),
	dpvE=TMatr.getAMatrA(psiIv,QQMpv,neq),
	dpv0=TMatr.getAMatrA(o0v,QQMpv,neq);
    ptE=p_offset+dptE;
    sigma2_ptE=TMatr.getAMatrA(psiIE,QQMp2t,neq)-dptE*dptE;
    pt0=p_offset+dpt0;
    sigma2_pt0=TMatr.getAMatrA(o0t,QQMp2t,neq)-dpt0*dpt0;
    pvE=p_offset+dpvE;
    sigma2_pvE=TMatr.getAMatrA(psiIv,QQMp2v,neq)-dpvE*dpvE;
    pv0=p_offset+dpv0;
    sigma2_pv0=TMatr.getAMatrA(o0v,QQMp2v,neq)-dpv0*dpv0;


    AE=imax;
    AEt=TMatr.getAMatrB(neq,ev,0+neq,ev,0+neq,QQMv,neq);
    SAE=TMatr.getAMatrB(neq,ev,0+neq,ev,0+neq,QQMrvs,neq);
    SBE=TMatr.getAMatrB(neq,ev,0+neq,ev,0+neq,QQMrts,neq);
    AEl=ev[neq-1];
    AElt=TMatr.getAMatrB(neq,ev,(neq-1)*neq+neq,ev,(neq-1)*neq+neq,QQMv,neq);
    SAEl=TMatr.getAMatrB(neq,ev,(neq-1)*neq+neq,ev,(neq-1)*neq+neq,QQMrvs,neq);
    SBEl=TMatr.getAMatrB(neq,ev,(neq-1)*neq+neq,ev,(neq-1)*neq+neq,QQMrts,neq);

    sQQv=QQMv;
    evcut=getSymQ0RemovedA(neq,QQMv,QQMt);
    if(neq>1)
    {
	AR=evcut[0];
	ARt=TMatr.getAMatrB(neq,evcut,0+neq,evcut,0+neq,QQMv,neq);
	SAR=TMatr.getAMatrB(neq,evcut,0+neq,evcut,0+neq,QQMrvs,neq);
	SBR=TMatr.getAMatrB(neq,evcut,0+neq,evcut,0+neq,QQMrts,neq);
	final int nl=neq-1;
	ARl=evcut[nl-1];
	ARlt=TMatr.getAMatrB(neq,evcut,(nl-1)*neq+neq,evcut,(nl-1)*neq+neq,QQMv,neq);
	SARl=TMatr.getAMatrB(neq,evcut,(nl-1)*neq+neq,evcut,(nl-1)*neq+neq,QQMrvs,neq);
	SBRl=TMatr.getAMatrB(neq,evcut,(nl-1)*neq+neq,evcut,(nl-1)*neq+neq,QQMrts,neq);
    } else AR=ARt=SAR=SBR=ARl=ARlt=SARl=SBRl=0;


    sQQt=QQMt;
    sQQMrvs=QQMrvs;
    rve=hh_getSymEVDiagT(neq,QQMrvs,QQMt);
    SArvsL=rve[neq-1];
    SArvsH=rve[0];
    psiL=new double [neq];
    System.arraycopy(rve,neq+(neq-1)*neq,psiL,0,neq);
    wL=access.fx(psiL,x0)/sqrt_o_o;
    psiH=new double [neq];
    System.arraycopy(rve,neq+(0)*neq,psiH,0,neq);
    wH=access.fx(psiH,x0)/sqrt_o_o;


    sQQMts=QQMts;
    sQQMvs=QQMvs;
    sQQMpv=QQMpv;
    pev=hh_getSymEVDiagT(neq,QQMpv,QQMt);
    sPL=pev[neq-1];
    sPH=pev[0];
    psi_p_L=new double [neq];
    System.arraycopy(pev,neq+(neq-1)*neq,psi_p_L,0,neq);
    wPL=access.fx(psi_p_L,x0)/sqrt_o_o;
    psi_p_H=new double [neq];
    System.arraycopy(pev,neq+(0)*neq,psi_p_H,0,neq);
    wPH=access.fx(psi_p_H,x0)/sqrt_o_o;


    rvecut=getSymQ0RemovedA(neq,QQMrvs,QQMt);
    if(neq>1) {SARrvsL=rvecut[neq-2];SARrvsH=rvecut[0];}
    else SARrvsL=SARrvsH=0;


    VS=getEVXData(neq,vsmom,tmom);
    VSsam=getEVXData(neq,vsmom,tmom_sam);
    TS=getEVXData(neq,tsmom,tmom);
    TSsam=getEVXData(neq,tsmom,tmom_sam);
    VSV=getEVXData(neq,vsmom,vmom);
    TSV=getEVXData(neq,tsmom,vmom);

    sQQMrts=QQMrts;
    RVS=getEVXData(neq,rvsmom,tmom);
    RTS=getEVXData(neq,rtsmom,vmom);
    final double [] xmom=new double [nmoms];

    final double [] temp=new double [IQQt.length];
    DPS=getEVXData(neq,dpmom,vmom);
    final double [] dpmom0=getMomPsi0Removed(neq,dpmom,tmom,access,x0),dp2mom0=new double [nmoms];
    access.setDFMoments(dp2mom0,dpmom0,nmoms);
    DPt0=getEVXData(neq,dpmom0,tmom);
    D2Pt0=getEVXData(neq,dp2mom0,tmom);
    if(true)
    {
	final double [] DQQ=ZQVM.getIQQ_F_IQQ(neq,IQQt,access.getQQ_Matrs(neq,1,new double [][]{dpmom0})[0],temp);
	final double [][] kkkk=access.getKK(neq,new double [][] {IQQt,DQQ});
	final double [] kkkt=kkkk[0],kkkdp=kkkk[1];
	this.DPt0_f0=access.fx(kkkdp,x0)/access.fx(kkkt,x0);
	this.D2Pt0_f0=access.Dfx(kkkdp,x0)/access.fx(kkkt,x0);

    }
    RVSV=getEVXData(neq,rvsmom,vmom);
    RTST=getEVXData(neq,rtsmom,tmom);
    RTSTsam=getEVXData(neq,rtsmom,tmom_sam);
    RVSsam=getEVXData(neq,rvsmom,tmom_sam);

    PVOL=getEVXData(neq,ptmom,vmom); // p_offset==p_last // FIXME
    P2VOL=getEVXData(neq,p2tmom,vmom); // p_offset==p_last // FIXME
    PX2VOL=getEVXData(neq,p2vmom,tmom); // p_offset==p_last // FIXME
    PXVOL=getEVXData(neq,pvmom,tmom); // p_offset==p_last // FIXME
    pf0=p_offset+TMatr.getAMatrA(P2VOL.psi0,QQMpv,neq);
    pfL=p_offset+TMatr.getAMatrA(P2VOL.psiL,QQMpv,neq);
    pfLt=p_offset+TMatr.getAMatrA(P2VOL.psiL,QQMpt,neq)/TMatr.getAMatrA(P2VOL.psiL,QQMt_sam,neq);
    EIsell=getEVXData(neq,isell,tmom);
    EIbuy=getEVXData(neq,ibuy,tmom);
    EAM=getEVXData(neq,amoms,vmom);
    EV2oAM=getEVXDataFromQQ(neq,ZQVM.getIQQ_F_IQQ(neq,QQMa,IQQt,temp),QQMv);
    EV2oDAM=getEVXDataFromQQ(neq,ZQVM.getIQQ_F_IQQ(neq,QQMdp,IQQt,temp),QQMv);
    EAMM=getEVXDataFromQQ(neq,QQMa,QQMv);
    mdp=M2Data.getM2Data(neq,QQMdp,QQMv,IQQt,access,x0);
    mS=M2Data.getM2Data(neq,QQMts,QQMv,IQQt,access,x0);
    sQQMdp=QQMdp;
    sIQQt=IQQt;


    IT=getEVXData(neq,vmom,tmom);
    ITinv=getEVXData(neq,tmom,vmom);
    final double ix=TMatr.getAMatrA(IT.psiH,QQMv,neq);
    for(int k=0;k<nmoms;k++){xmom[k]=tmom[k]*ix-vmom[k];}
    ITshifted=getEVXData(neq,xmom,tmom);
    sIE=TMatr.getAMatrA(IT.psiH,QQMrvs,neq);

    ITsam=getEVXData(neq,vmom,tmom_sam);
    I0sam=ITsam.o_o/ITinv.o_o;
    for(int k=0;k<nmoms;k++){xmom[k]=ITsam.sH*rtsmom[k]-rvsmom[k];}
    DS=getEVXData(neq,xmom,tmom);
    DSsam=getEVXData(neq,xmom,tmom_sam);
    DSsamV=getEVXData(neq,xmom,vmom);
    sQQx=access.getQQ_Matrs(neq,1,new double [][]{xmom})[0];
    sQQMt_sam=QQMt_sam;

    for(int k=0;k<nmoms;k++){xmom[k]=rvsmom[k]+(IT.sH-IT.s0)*rtsmom[k];}
    DSva=getEVXData(neq,xmom,tmom);
    DSta=getEVXData(neq,xmom,vmom);
    sQQva=access.getQQ_Matrs(neq,1,new double [][]{xmom})[0];
    for(int k=0;k<nmoms;k++){xmom[k]=rvsmom[k]+(ITsam.sH-ITsam.s0)*rtsmom[k];}
    DSva_sam=getEVXData(neq,xmom,tmom_sam);
    DSta_sam=getEVXData(neq,xmom,vmom);
    sQQva_sam=access.getQQ_Matrs(neq,1,new double [][]{xmom})[0];

    for(int k=0;k<nmoms;k++){xmom[k]=rvsmom[k]+(1/ITinv.sL-1/ITinv.s0)*rtsmom[k];}
    DStaI=getEVXData(neq,xmom,vmom);
    for(int k=0;k<nmoms;k++){xmom[k]=rvsmom[k]+(IT.sH-I0)*rtsmom[k];}
    DStaI0=getEVXData(neq,xmom,vmom);

    IT0R=getEVXData0Removed(neq,vmom,tmom,access,x0);
    pH0=p_offset+getPAW(neq,QQMpv,QQMv,IT.psiH,IT.psi0);
    sH0=getPAW(neq,QQMrvs,QQMv,IT.psiH,IT.psi0);
    ppv0=p_offset+TMatr.getAMatrA(IT.psi0,QQMpv,neq)/IT.s0;
    ppt0=p_offset+TMatr.getAMatrA(IT.psi0,QQMpt,neq);
    ppvH=p_offset+TMatr.getAMatrA(IT.psiH,QQMpv,neq)/IT.sH;
    pptH=p_offset+TMatr.getAMatrA(IT.psiH,QQMpt,neq);
    peamL=p_offset+TMatr.getAMatrA(EAM.psiL,QQMpv,neq);
    peamH=p_offset+TMatr.getAMatrA(EAM.psiH,QQMpv,neq);
    pev2oamL=p_offset+TMatr.getAMatrA(EV2oAM.psiL,QQMpv,neq);
    pev2oamH=p_offset+TMatr.getAMatrA(EV2oAM.psiH,QQMpv,neq);
    pev2oDamL=p_offset+TMatr.getAMatrA(EV2oDAM.psiL,QQMpv,neq);
    pev2oDamH=p_offset+TMatr.getAMatrA(EV2oDAM.psiH,QQMpv,neq);


    DSI=getEVXDataFromQQ(neq,getQQSpectrCTtransformed(neq,QQMrts,ev,0,1),QQMt);
    DSImI=getEVXDataFromQQ(neq,getQQSpectrCTtransformed(neq,QQMrts,ev,IT.sH,-1),QQMt);
    final double [] QQIpI=getQQSpectrCTtransformed(neq,QQMrts,ev,IT.sH,-1);
    TMatr.sumP1(QQIpI,QQMrvs,1);
    DSIpI=getEVXDataFromQQ(neq,QQIpI,QQMt);
    final double [] QQIpI_test=getQQSpectrCTtransformed(neq,QQMt,ev,IT.sH,-1);
    DSImI_testV=getEVXDataFromQQ(neq,QQIpI_test,QQMt);
    TMatr.sumP1(QQIpI_test,QQMv,1);
    DSIpI_testV=getEVXDataFromQQ(neq,QQIpI_test,QQMt);



    for(int k=0;k<nmoms;k++){xmom[k]=rvsmom[k]+IT.sH*rtsmom[k];}
    DRVSta=getEVXData(neq,xmom,vmom);
    for(int k=0;k<nmoms;k++){xmom[k]=vsmom[k]+IT.sH*tsmom[k];}
    DVSta=getEVXData(neq,xmom,vmom);

    final double [] mumom=new double [nmoms];
    {
    final double alfa=I0sam/ITsam.sH,talfa=ITsam.sH*(1-alfa);
    for(int k=0;k<nmoms;k++){
	xmom[k]=alfa*rvsmom[k]+talfa*rtsmom[k];
	mumom[k]=alfa*vmom[k]+talfa*tmom_sam[k];
    }
    DSRmu_sam=getEVXData(neq,xmom,mumom);
    for(int k=0;k<nmoms;k++){
	xmom[k]=alfa*vsmom[k]+talfa*tsmom[k];
	mumom[k]=alfa*vmom[k]+talfa*tmom_sam[k];
    }
    DSmu_sam=getEVXData(neq,xmom,mumom);
    for(int k=0;k<nmoms;k++){
	xmom[k]=alfa*pvmom[k]+talfa*ptmom[k];
	mumom[k]=alfa*vmom[k]+talfa*tmom_sam[k];
    }
    P_sam=getEVXData(neq,xmom,mumom);
    }

    {
    final double alfa=I0/IT.sH,talfa=IT.sH*(1-alfa);
    for(int k=0;k<nmoms;k++){
	xmom[k]=alfa*rvsmom[k]+talfa*rtsmom[k];
	mumom[k]=alfa*vmom[k]+talfa*tmom[k];
    }
    DSRmu=getEVXData(neq,xmom,mumom);
    for(int k=0;k<nmoms;k++){
	xmom[k]=alfa*vsmom[k]+talfa*tsmom[k];
	mumom[k]=alfa*vmom[k]+talfa*tmom[k];
    }
    DSmu=getEVXData(neq,xmom,mumom);
    }

    sQQMp2t=QQMp2t;
    sQQMp1t=QQMpt;
    sQQMp0t=QQMt_sam;
    this.mpv=MXData.getMXData(neq,QQMp2t,QQMpt,QQMt_sam,QQMv,QQMt,p_offset);
    if(true)
    {
	final int n=neq;
	final double [] f0=new double [n],xr=new double [n];
	for(int k=0;k<n;k++)
	{
	    xr[k]=1;
	    f0[k]=access.fx(xr,x0);
	    xr[k]=0;
	}
	final double [] S=new double [n*n];
	for(int k=0;k<n;k++)
	{
	    for(int l=0;l<n;l++)
	    {
		S[k+l*n]=f0[k]*f0[l];
	    }
	}
	final double dpl=0; // FIXME, assume here p_offset=p_last
	final double [] Dpo=new double [n*n];
	for(int k=S.length;--k>=0;)
	{
	    Dpo[k]=dpl*S[k]-QQMdp[k];
	}
	final double [] Dp0=TMatr.mulMMM(n,S,IQQt,S),
	    Dp1=TMatr.mulMMM(n,Dpo,IQQt,S),
	    Dp2=TMatr.mulMMM(n,Dpo,IQQt,Dpo);
	this.sDp0=Dp0;this.sDp1=Dp1;this.sDp2=Dp2;
	this.mDpv=MXData.getMXData(neq,Dp2,Dp1,Dp0,QQMv,QQMt,p_offset);
    }
}
final double [] sDp0,sDp1,sDp2;


double getPAW(final int n,final double [] QQvf,final double [] QQv,final double [] psiH,final double [] psi0)
{
    return (TMatr.getAMatrA(psiH,QQvf,n)-TMatr.getAMatrA(psi0,QQvf,n))/
	(TMatr.getAMatrA(psiH,QQv,n)-TMatr.getAMatrA(psi0,QQv,n));
}

double [] getIQQdiag_F_IQQdiag(final int n,final double [] IQQt,final double [] F)
{
    final double [] R=new double [n*n];
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<n;l++)
	{
	    R[k*n+l]=IQQt[n*k+k]*F[n*k+l]*IQQt[n*l+l];
	}
    }
    return R;
}
double [] getDiagInverted(final int n,final double [] QQ)
{
    final double [] R=new double [n*n];
    for(int k=0;k<n;k++)
    {
	R[k*n+k]=1/QQ[k*n+k];
    }
    return R;
}
double [] getOKDiagT(final int n,final double [] QQt)
{
    final double x0=getX0();
    final double [] r=new double [n],o=new double [n];
    for(int k=0;k<n;k++)
    {
	r[k]=1;o[k]=access.fx(r,x0);r[k]=0;
	o[k]/=QQt[k*n+k]; // assume QQt is diagonal
    }
    return o;
}

static double [] getMomPsi0Removed(final int n,final double [] momf,final double [] momt,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{momt,momf});
    final double [] QQdp=QQ[1],QQt=QQ[0],psi0=access.getOK(n,QQt,x0);
    final double delta=-TMatr.getAMatrA(psi0,QQdp,n)/Math.pow(access.fx(psi0,x0),2);
    final double [] momf0=new double [momf.length],r=new double [momf.length];
    for(int k=momf0.length;--k>=0;)
    {
	r[k]=1;
	final double fk0=access.fx(r,x0);
	r[k]=0;
	momf0[k]=momf[k]+delta*fk0;
    }
    //final double [] xQQ=access.getQQ_Matrs(n,1,new double [][]{momf0})[0];
    //System.err.println("val f0="+TMatr.getAMatrA(psi0,xQQ,n));
    return momf0;
}

double [] hh_getSymEVDiagT(final int n,final double [] QQv,final double [] QQt)
{
    final double [] ev=EVXData.getSymEVDiagT(n,QQv,QQt,DeprecatedHelperEVXData.EV);
    // Make highest lamda on top. Inverse order.
    UQVM.inverseOrderOfEV(n,ev);
    return ev;
}
double [] getSymQ0RemovedA(final int nn,final double [] QQv,final double [] QQt)
{
    return getSymQ0RemovedA(nn,QQv,QQt,access,getX0(),true);
}

static double [][] getQQ0RemovedA(final int nn,final double [][] QQ,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
{
    final int n=nn-1;
    final double [] qx0=new double [nn],r=new double [nn];
    for(int k=0;k<nn;k++)
    {
	r[k]=1;qx0[k]=access.fx(r,x0);r[k]=0;
    }
    final double q0=qx0[0]; // here we assume that 0-th element in basis is a constant
    final double [][] qq=new double [QQ.length][n*n];
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<=k;l++)
	{
	    final int kk=k+1,ll=l+1;
	    for(int j=qq.length;--j>=0;)
	    {
		qq[j][k*n+l]=qq[j][l*n+k]=QQ[j][kk*nn+ll]-qx0[kk]*(QQ[j][0+ll]/q0)-qx0[ll]*(QQ[j][0+kk]/q0)+qx0[kk]*qx0[ll]*(QQ[j][0+0]/(q0*q0));
	    }
	}
    }
    return qq;
}

/** Convert matrices to (Q_s(x)-Q_s(x0)) basis, assuming Q_0 is a constant,
 * <Q_i Q_j> is input (i,j=0..n-1), <(Q_i-Q_i(x0))(Q_j-Q_j(x0))> (i,j=1..n-1) is output.
 */
static double [][] getQQ0RemovedA(final int nn,final double [][] QQ,final OrthogonalPolynomialsBasisFunctionsCalculatable<?extends BasisFunctionsCalculatable> M,final double x0)
{
    final int n=nn-1;
    final double [] qx0=M.B.getX0Moments(nn,x0);
    final double q0=qx0[0]; // here we assume that 0-th element in basis is a constant
    final double [][] qq=new double [QQ.length][n*n];
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<=k;l++)
	{
	    final int kk=k+1,ll=l+1;
	    for(int j=qq.length;--j>=0;)
	    {
		qq[j][k*n+l]=qq[j][l*n+k]=QQ[j][kk*nn+ll]-qx0[kk]*(QQ[j][0+ll]/q0)-qx0[ll]*(QQ[j][0+kk]/q0)+qx0[kk]*qx0[ll]*(QQ[j][0+0]/(q0*q0));
	    }
	}
    }
    return qq;
}


static double [] getSymQ0RemovedA(final int nn,final double [] QQv,final double [] QQt,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0,final boolean flag_inverse_ev)
{
    final int n=nn-1;
    final double [][] QQ=getQQ0RemovedA(nn,new double [][]{QQv,QQt},access,x0);
    final double [] qqv=QQ[0],qqt=QQ[1];
    final double [] ev=DeprecatedHelperEVXData.solveGeneralizedEV(n,qqv,qqt);
    // Make highest lamda on top. Inverse order.
    if(flag_inverse_ev) UQVM.inverseOrderOfEV(n,ev);
    // convert back to the Q basis
    final double [] evorigbasis=new double [nn+n*nn];
    final double [] qx0=new double [nn],r=new double [nn];
    for(int k=0;k<nn;k++)
    {
	r[k]=1;qx0[k]=access.fx(r,x0);r[k]=0;
    }
    final double q0=qx0[0];
    for(int k=0;k<n;k++)
    {
	evorigbasis[k]=ev[k];
	double s0=0;final int kk=k+1,oo=n*kk,on=nn*kk;
	for(int l=0;l<n;l++)
	{
	    final double vk=evorigbasis[on+(l+1)]=ev[oo+l];
	    s0-=vk*(qx0[kk]/q0);
	}
	evorigbasis[on+0]=s0;
    }
    return evorigbasis;
}


static double [] getQQSpectrCTtransformed(final int n,final double [] QQ,final double [] ev,final double i0,final double ci)
{
    final double [] temp=new double [n*n],bQQ=new double [n*n];
    // convert QQ to the basis of ev eigenvectors
    DeprecatedHelperTMatr.multiply_Matr_x_MatrTrans(n,temp,QQ,n,0,ev,n,n+0*n);
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,bQQ,ev,n,n+0*n,temp,n,0);
    // OK, now bQQ is QQ in the basis of ev eigenvectors.

    // build from bQQ a TC transformed matrix.
    for(int k=0;k<n;k++)
    {
	final double dk=i0+ci*ev[k],fk=Math.sqrt(dk>0?dk:0);
	for(int l=0;l<n;l++)
	{
	    final double dl=i0+ci*ev[l],fl=Math.sqrt(dl>0?dl:0);
	    bQQ[k*n+l]*=fl*fk;
	}
    }
    // for unit test calculate inverted matrix directly,
    // instead of using orthogonality property with scalar product determined by QQt.
    System.arraycopy(ev,n+0*n,temp,0,n*n);
    final double [] inv=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,temp);
    // convert back to QQ basis
    DeprecatedHelperTMatr.multiply_Matr_x_MatrTrans(n,temp,bQQ,n,0,inv,n,0);
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,bQQ,inv,n,0,temp,n,0);
    return bQQ;
}

static class MXData {
    static final int NITER=15;
    final double [] psi;
    final double lam,p0,sigma2_p0,lamE;
    MXData(final double [] psi,final double lam,final double p0,final double sigma2_p0,final double lamE)
    {
	this.psi=psi;
	this.lam=lam;
	this.lamE=lamE;
	this.p0=p0;
	this.sigma2_p0=sigma2_p0;
    }
    static MXData getMXData_test(final int n,final double [] QQp2,final double [] QQp1,final double [] QQp0,final double [] QQv,final double p_offset,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0,final double eps)
    {
	double dp0=0,dp1=dp0,lam=Double.NaN;
	final double [] QQx=new double [n*n];
	EVXData ev=null;
	int ind_min=0;
	for(int i=0;i<NITER;i++)
	{
	    for(int k=QQx.length;--k>=0;)
	    {
		QQx[k]=QQp2[k]+dp0*(-2*QQp1[k]+dp0*QQp0[k]);
	    }
	    ev=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQx,QQv,access,x0);
	    ind_min=0;
	    double lammin=Double.POSITIVE_INFINITY;
	    for(int e=0;e<n;e++)
	    {
		final double m1=TMatr.getAMatrB(n,ev.ev,n+e*n,ev.ev,n+e*n,QQp1,n),
		    m0=TMatr.getAMatrB(n,ev.ev,n+e*n,ev.ev,n+e*n,QQp0,n);
		final double dpe=m1/m0;
		for(int k=QQx.length;--k>=0;){QQx[k]=QQp2[k]+dpe*(-2*QQp1[k]+dpe*QQp0[k]);}
		final double lame=TMatr.getAMatrB(n,ev.ev,n+e*n,ev.ev,n+e*n,QQx,n);
		final double lame1=ev.ev[e]+(dpe-dp0)*(-2*m1+(dpe+dp0)*m0);
		UnitTests.checkDoubleRel(lame,"lame",lame1,"lame1",eps);
		if(i<=1 && !(lame<=ev.ev[e]))
		{
		    System.err.println("lame="+lame+" lame1="+lame1+" ev.ev["+e+"]="+ev.ev[e]);
		    throw new RuntimeException(" i="+i+" lame="+lame+" ev.ev["+e+"]="+ev.ev[e]);
		}
		UnitTests.checkDoubleRel(1,"1",TMatr.getAMatrB(n,ev.ev,n+(e)*n,ev.ev,n+(e)*n,QQv,n),"<psi|QQv|psi>",eps);
		if(lame<=lammin)
		{
		    lammin=lame;
		    ind_min=e;
		    dp1=dpe;
		}
	    }
	    dp0=dp1;
	    lam=lammin;
	    System.err.println("test:i="+i+" ind_min="+ind_min+" lammin="+lammin+" ev.sL="+ev.sL);
	}
	return new MXData(ev.psiL,ev.sL,dp0+p_offset,ev.sL/TMatr.getAMatrA(ev.psiL,QQp0,n),lam);
    }
static MXData getMXData(final int n,final double [] QQp2,final double [] QQp1,final double [] QQp0,final double [] QQv,final double [] diag_QQt,final double p_offset)
    {
	final double [] evI=EVXData.getSymEVDiagT(n,QQv,diag_QQt,DeprecatedHelperEVXData.EV);
	final double [] BBx=new double [n*n],QQvv=new double [n*n],
	    BBp2=new double [n*n],BBp1=new double [n*n],BBp0=new double [n*n];
	// Convert to the basis of evI eigenvectors. QQvv is diagonal in this basis.
	DeprecatedHelperTMatr.multiply_Matr_x_MatrTrans(n,BBx,QQp0,n,0,evI,n,n+0*n);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,BBp0,evI,n,n+0*n,BBx,n,0);
	DeprecatedHelperTMatr.multiply_Matr_x_MatrTrans(n,BBx,QQp1,n,0,evI,n,n+0*n);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,BBp1,evI,n,n+0*n,BBx,n,0);
	DeprecatedHelperTMatr.multiply_Matr_x_MatrTrans(n,BBx,QQp2,n,0,evI,n,n+0*n);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,BBp2,evI,n,n+0*n,BBx,n,0);
	for(int k=0;k<n;k++) QQvv[n*k+k]=evI[k];
	final int ind_max=n-1;
	double //dp0=TMatr.getAMatrB(n,evI,n+(ind_max)*n,evI,n+(ind_max)*n,QQp1,n)/TMatr.getAMatrB(n,evI,n+(ind_max)*n,evI,n+(ind_max)*n,QQp0,n),
	    dp0=0,
	    dp1=dp0,lam=Double.NaN;
	int ind_min=0;
	double [] evS=null;
	for(int i=0;i<NITER;i++)
	{
	    dp0=dp1;
	    for(int k=BBx.length;--k>=0;)
	    {
		BBx[k]=BBp2[k]+dp0*(-2*BBp1[k]+dp0*BBp0[k]);
	    }
	    evS=EVXData.getSymEVDiagT(n,BBx,QQvv,DeprecatedHelperEVXData.EV);
	    double lammin=Double.POSITIVE_INFINITY;
	    ind_min=0;
	    for(int e=0;e<n;e++)
	    {
		// e-th eigenvalue
		final double m1=TMatr.getAMatrB(n,evS,n+(e)*n,evS,n+(e)*n,BBp1,n),
		    m0=TMatr.getAMatrB(n,evS,n+(e)*n,evS,n+(e)*n,BBp0,n);
		final double dpe=m1/m0,da=dpe-dp0;
		final double lame=evS[e]+(dpe-dp0)*(-2*m1+(dpe+dp0)*m0);
		if(false)
		{
		    final double eps=1e-5;
		    for(int k=BBx.length;--k>=0;){BBx[k]=BBp2[k]+dpe*(-2*BBp1[k]+dpe*BBp0[k]);}
		    UnitTests.checkDoubleRel(lame,"lame",TMatr.getAMatrB(n,evS,n+(e)*n,evS,n+(e)*n,BBx,n),"<psi|BBxe|psi>",eps);
		    if(i<=1 && !(lame<=evS[e]))
		    {
			throw new RuntimeException("lame="+lame+" evS["+e+"]="+evS[e]);
		    }
		    //System.err.println("evS["+e+"]="+evS[e]+" <psi|BBx|psi>="+TMatr.getAMatrB(n,evS,n+(e)*n,evS,n+(e)*n,BBx,n)+" lame="+lame+" m1="+m1+" m0="+m0);
		    //for(int k=BBx.length;--k>=0;){BBx[k]=BBp2[k]+dpe*(-2*BBp1[k]+dpe*BBp0[k]);}System.err.println("DPE <psi|BBx|psi>="+TMatr.getAMatrB(n,evS,n+(e)*n,evS,n+(e)*n,BBx,n));
		}
		if(lame<=lammin)
		{
		    lammin=lame;
		    dp1=dpe;
		    ind_min=e;
		}
	    }
	    lam=lammin;
	    //System.err.println("i="+i+" ind_min="+ind_min+" evS="+evS[ind_min]+" lam="+lam);
	}
	final double [] psir=new double [n];
	for(int k=n;--k>=0;)
	{
	    DeprecatedHelperTMatr.sumP1(psir,evI,n+k*n,n,evS[k+n+ind_min*n]);
	}
	final double sigma2=evS[ind_min]/TMatr.getAMatrB(n,evS,n+(ind_min)*n,evS,n+(ind_min)*n,BBp0,n);
	return new MXData(psir,evS[ind_min],dp1+p_offset,sigma2,lam);
    }
    static void test_MXData(final MXData a,final MXData b,final double eps)
    {
	UnitTests.checkDoubleRel(a.psi.length,"a.psi.length",b.psi.length,"b.psi.length",eps);
	UnitTests.checkDoubleRel(a.lam,"a.lam",b.lam,"b.lam",eps);
	UnitTests.checkDoubleArrayRel(a.psi,"a.psi",EVXData.getMSign(b.psi,a.psi[a.psi.length-1]),"b.psi",eps);
	UnitTests.checkDoubleRel(a.p0,"a.p0",b.p0,"b.p0",eps);
	UnitTests.checkDoubleRel(a.sigma2_p0,"a.sigma2_p0",b.sigma2_p0,"b.sigma2_p0",eps);

	System.err.println("MXData:test_MXData test OK n="+a.psi.length+" eps="+eps);
    }
}

static class M2Data {
    final EVXData evx;
    final double lam0,lam1,D,dp;
    M2Data(EVXData evx,final double lam0,final double lam1,final double dp)
    {
	this.evx=evx;
	this.lam0=lam0;
	this.lam1=lam1;
	this.dp=dp;
	this.D=((lam0-lam1)/lam0)*Math.signum(dp);
    }
    static M2Data getM2Data(final int n,final double [] Dp,final double [] QQv,final double [] IQQt,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
    {
	final double [] f0=new double [n],r=new double [n];
	for(int k=0;k<n;k++)
	{
	    r[k]=1;
	    f0[k]=access.fx(r,x0);
	    r[k]=0;
	}
	final double [] IQQt_Dp=TMatr.mulMM(n,IQQt,Dp);
	final double [] Dp_IQQt_Dp=TMatr.mulMM(n,Dp,IQQt_Dp);
	final EVXData evx=DeprecatedHelperEVXData.getEVXDataFromQQ(n,Dp_IQQt_Dp,QQv,access,x0);
	final double [] psi=evx.psiH;
	final double psif0=TMatr.mulVVn1(psi,f0),o_o=TMatr.getAMatrB(f0,f0,IQQt,n),dp_o=TMatr.getAMatrB(f0,psi,IQQt_Dp,n);
	final double dp=-dp_o/(o_o*psif0);
	final double lam0=TMatr.getAMatrB(psi,psi,Dp_IQQt_Dp,n),
	    lam1=lam0+dp*psif0*(2*dp_o+dp*psif0*o_o);
	return new M2Data(evx,lam0,lam1,dp);
    }
    static void test_M2Data(final M2Data m,final double [] Dp,final double [] QQv,final double [] IQQt,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0,final double eps)
    {
	final int n=m.evx.neq;
	final double [] f0=new double [n],r=new double [n];
	for(int k=0;k<n;k++)
	{
	    r[k]=1;
	    f0[k]=access.fx(r,x0);
	    r[k]=0;
	}
	final double [] psi=m.evx.psiH;

	final double [] Dp_a=new double [n*n];
	for(int k=0;k<n;k++)
	{
	    for(int l=0;l<n;l++)
	    {
		Dp_a[k+l*n]=Dp[k+l*n]+m.dp*f0[k]*f0[l];
	    }
	}
	final double [] Dp_IQQt_Dp=TMatr.mulMMM(n,Dp,IQQt,Dp),
	    Dpa_IQQt_Dpa=TMatr.mulMMM(n,Dp_a,IQQt,Dp_a);
	if(n>1)
	{
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,Dp_IQQt_Dp,n),"<psi|Dp_IQQt_Dp|psi>",m.lam0,"m.lam0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,Dpa_IQQt_Dpa,n),"<psi|Dpa_IQQt_Dpa|psi>",m.lam1,"m.lam1",eps);
	if(m.lam1>m.lam0) throw new RuntimeException("m.lam1="+m.lam1+" m.lam0="+m.lam0);
	}
	System.err.println("M2Data:test_M2Data test OK n="+n+" eps="+eps);
    }
}



EVXData getEVXData(final int n,final double [] momf,final double [] mom)
{
    return DeprecatedHelperEVXData.getEVXData(n,momf,mom,access,getX0());
}

EVXData getEVXDataFromQQ(final int n,final double [] QQf,final double [] QQa)
{
    return DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQf,QQa,access,getX0());
}


static EVXData getEVXData0Removed(final int n,final double [] momf,final double [] mom,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{momf,mom});
    final double [] QQf=QQ[0],QQa=QQ[1];
    final double [] psiL=new double [n],psiH=new double [n],psi0=access.getOK(n,QQa,x0);
    final double o_o=access.fx(psi0,x0),sqrt_o_o=Math.sqrt(o_o),
	s0=TMatr.getAMatrA(psi0,QQf,n);
    for(int k=0;k<n;k++) {psi0[k]/=sqrt_o_o;}
    if(n>=2){
	final double [] ev=getSymQ0RemovedA(n,QQf,QQa,access,x0,false);
	final double sL=ev[0],sH=ev[n-2];
	System.arraycopy(ev,n+(0)*n,psiL,0,n);
	System.arraycopy(ev,n+(n-2)*n,psiH,0,n);
	final double wL=access.fx(psiL,x0)/sqrt_o_o,wH=access.fx(psiH,x0)/sqrt_o_o;
	return new EVXData(n-1,wL,sL,psiL,wH,sH,psiH,psi0,ev,null,s0,o_o,x0);
    }
    else
    {
	final double [] ev=new double [n-1+(n-1)*n];
	java.util.Arrays.fill(psiL,Double.NaN);
	java.util.Arrays.fill(psiH,Double.NaN);
	return new EVXData(n-1,0,0,psiL,0,0,psiH,psi0,ev,null,s0,o_o,x0);
    }
}

static EVXData getEVXData0Removed_STD(final int n,final double [] momf,final double [] mom,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
{
    final double [][] QQ=getQQ0RemovedA(n,access.getQQ_Matrs(n,1,new double [][]{momf,mom}),access,x0);
    return DeprecatedHelperEVXData.getEVXDataFromQQ(n-1,QQ[0],QQ[1],access,x0);
}



abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getX0();
abstract double getPOffset();
abstract TwoCloseRootsFinder.RootsData getTRD(final double [] kkt,final double [] kkf);
abstract TwoRootsIFX getTRIFX(final double [] kkt,final double [] kkf);
abstract TwoRootsIFD getTRIFD(final double [] kkt,final double [] kkf);


/** Calculate the roots of the polynomial Q_j(x) IQQ_jk Q_k(x0).
 * This (n-1)-th order polynomial has exactly n-1 roots.
 * return n-1 found roots and x0, totally n roots.
 * x0 is at 0.
 */
abstract double [] getXAll(final int n,final double [] IQQ);

static abstract class BQVMTest<DTYPE> extends QTest {


abstract BQVM<DTYPE> getBQVM();
abstract ZQVM<DTYPE> getZQVM();
abstract UQVM<DTYPE> getUQVM();
abstract double [][] getMonomialMoments(final DTYPE o);

BQVMTest(final String name,
	 final java.util.Random r,final double eps)
{
    super(name,r,eps);
}
void test_Common()
{
    final BQVM<DTYPE> q=getBQVM();
    UnitTests.checkDoubleRel(q.AE,"q.AE",q.AEt,"q.AEt",eps);
    UnitTests.checkDoubleRel(q.AEl,"q.AEl",q.AElt,"q.AElt",eps);
    UnitTests.checkDoubleRel(q.AR,"q.AR",q.ARt,"q.ARt",eps);
    UnitTests.checkDoubleRel(q.ARl,"q.ARl",q.ARlt,"q.ARlt",eps);
    UnitTests.checkDoubleRel(q.sIE,"q.sIE",q.SAE,"q.SAE",eps);
    for(int k=0;k<neq-1;k++)
    {
	final double AEV=q.evcut[k];
	final double AV=TMatr.getAMatrB(neq,q.evcut,neq+k*neq,q.evcut,neq+k*neq,q.sQQv,neq);
	System.err.println("evcut #"+k+" AEV="+AEV+" AV="+AV);
	UnitTests.checkDoubleRel(AEV,"AEV",AV,"AV",eps);
    }
    UnitTests.checkDoubleArrayRel(q.o0v,"q.o0v",EVXData.getMSign(q.DPS.psi0,q.o0v[neq-1]),"q.DPS.psi0",eps);

    UnitTests.checkDoubleRel(q.A0,"q.A0",q.IT.s0,"q.IT.s0",eps);
    UnitTests.checkDoubleRel(q.IE,"q.IE",q.IT.sH,"q.IT.sH",eps);
    UnitTests.checkDoubleRel(Math.abs(q.wIE),"q.wIE",Math.abs(q.IT.wH),"q.IT.wH",eps);
    UnitTests.checkDoubleArrayRel(q.psiIE,"q.psiIE",EVXData.getMSign(q.IT.psiH,q.psiIE[neq-1]),"q.IT.psiH",eps);
    UnitTests.checkDoubleArrayRel(q.o0t,"q.o0t",EVXData.getMSign(q.IT.psi0,q.o0t[neq-1]),"q.IT.psi0",eps);
    System.err.println("IT.sH="+q.IT.sH+" ITsam.sH="+q.ITsam.sH);
    System.err.println("IT.ev="+java.util.Arrays.toString(q.IT.ev));
    System.err.println("ITshifted.ev="+java.util.Arrays.toString(q.ITshifted.ev));
    UnitTests.checkDoubleRel(q.ITshifted.sL,"q.ITshifted.sL",0,"0",eps);
    final double shift=q.IT.sH;
    // check q.ITshifted.ev spectr is equal to shift-q.IT.ev spectr.
    for(int k=0;k<neq;k++)
    {
	UnitTests.checkDoubleRel(shift-q.IT.ev[k],"shift-q.IT.ev[k]",q.ITshifted.ev[neq-1-k],"q.ITshifted.ev[neq-1-k]",eps);
    }
    UnitTests.checkDoubleArrayRel(q.IT.psiL,"q.IT.psiL",EVXData.getMSign(q.ITshifted.psiH,q.IT.psiL[neq-1]),"q.ITshifted.psiH",eps);
    UnitTests.checkDoubleArrayRel(q.IT.psiH,"q.IT.psiH",EVXData.getMSign(q.ITshifted.psiL,q.IT.psiH[neq-1]),"q.ITshifted.psiL",eps);
    System.err.println("DS.ev="+java.util.Arrays.toString(q.DS.ev));


    UnitTests.checkDoubleRel(q.SA0,"q.SA0",q.RVS.s0,"q.RVS.s0",eps);
    UnitTests.checkDoubleRel(q.SArvsL,"q.SArvsL",q.RVS.sL,"q.RVS.sL",eps);
    UnitTests.checkDoubleRel(Math.abs(q.wL),"q.wL",Math.abs(q.RVS.wL),"q.RVS.wL",eps);
    UnitTests.checkDoubleRel(q.SArvsH,"q.SArvsH",q.RVS.sH,"q.RVS.sH",eps);
    UnitTests.checkDoubleRel(Math.abs(q.wH),"q.wH",Math.abs(q.RVS.wH),"q.RVS.wH",eps);


    UnitTests.checkDoubleRel(q.AR,"q.AR",q.IT0R.sH,"q.IT0R.sH",eps);
    UnitTests.checkDoubleRel(q.ARl,"q.ARl",q.IT0R.sL,"q.IT0R.sL",eps);
    UnitTests.checkDoubleRel(q.pv0,"q.pv0",q.pf0,"q.pf0",eps);
    //UnitTests.checkDoubleRel(q.getPOffset(),"q.getPOffset()",q.pfLt,"q.pfLt",eps);

    System.err.println("q.IT.ev="+java.util.Arrays.toString(q.IT.ev));
    System.err.println("q.DSIpI_testV.ev="+java.util.Arrays.toString(q.DSIpI_testV.ev));
    for(int k=0;k<neq;k++)
    {
	UnitTests.checkDoubleRel(q.IT.sH,"q.IT.sH",q.DSIpI_testV.ev[neq-1-k],"q.DSIpI_testV.ev[neq-1-k]",eps);
	UnitTests.checkDoubleRel(q.IT.sH-q.IT.ev[k],"q.IT.sH-q.IT.ev[k]",q.DSImI_testV.ev[neq-1-k],"q.DSImI_testV.ev[neq-1-k]",eps);
    }

    UnitTests.checkDoubleRel(q.EIsell.s0+q.EIbuy.s0,"q.EIsell.s0+q.EIbuy.s0",q.A0,"q.A0",eps);
    EVXData.matchEVXData(q.EAM,q.EAMM,eps);


    M2Data.test_M2Data(q.mdp,q.sQQMdp,q.sQQv,q.sIQQt,q.access,q.getX0(),eps);
    M2Data.test_M2Data(q.mS,q.sQQMts,q.sQQv,q.sIQQt,q.access,q.getX0(),eps);
    EVXData.matchEVXData(q.EV2oDAM,q.mdp.evx,eps);

    if(neq>3)
    {
	MXData.test_MXData(q.mpv,MXData.getMXData_test(neq,q.sQQMp2t,q.sQQMp1t,q.sQQMp0t,q.sQQv,q.getPOffset(),q.access,q.getX0(),eps),eps);
	//MXData.test_MXData(q.mDpv,MXData.getMXData_test(neq,q.sDp2,q.sDp1,q.sDp0,q.sQQv,q.getPOffset(),q.access,q.getX0(),eps),eps);
    }


    UnitTests.checkDoubleRel(q.DPt0_f0,"q.DPt0_f0",0,"0",eps);
    test_f0(q.DPt0);

    System.err.println(name+":BQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}

void test_f0(final EVXData g)
{
    int n0=0;
    for(int k=0;k<g.neq;k++)
    {
	System.err.println("g.ev["+k+"]="+g.ev[k]+" g.s0="+g.s0);
	if(Math.abs(g.ev[k])<eps)
	{
	    n0++;
	    final double [] psi=new double [g.neq];
	    System.arraycopy(g.ev,g.neq+k*g.neq,psi,0,g.neq);
	    final double [] psi0=EVXData.getMSign(psi,Math.signum(g.psi0[g.neq-1]));
	    //UnitTests.checkDoubleArrayRel(psi0,"psi0",g.psi0,"g.psi0",eps);
	}
    }
    if(n0!=1)
    {
	//throw new RuntimeException("test_f0: n0="+n0);
    }
    System.err.println(name+":BQVMTest:test_f0(): test OK neq="+g.neq+" eps="+eps);
}


void test_psi(final double [] psi,final double [] QQv,final double [] QQt,final BQVM<DTYPE> q,final double w,final double A)
{
    UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,QQt,neq),"<psi|psi>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,QQv,neq),"<psi|f|psi>",A,"A",eps);
    UnitTests.checkDoubleArrayRel(TMatr.getMatr_x_A(neq,QQv,psi),"QQv psi",TMatr.getVectorScaled(TMatr.getMatr_x_A(neq,QQt,psi),A),"A*QQt psi",eps);

    final double [] IQQ=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQt);
    final double [] r=new double [neq],f=new double [neq];
    for(int k=0;k<neq;k++){f[k]=1;r[k]=q.access.fx(f,q.getX0());f[k]=0;}
    final double [] o=TMatr.getMatr_x_A(neq,IQQ,r);
    final double o_o=q.access.fx(o,q.getX0());
    UnitTests.checkDoubleRel(TMatr.getAMatrB(o,o,QQt,neq),"<o|o>",o_o,"o_o",eps);
    System.err.println("test_psi: neq="+neq+" o_o="+o_o+" o="+java.util.Arrays.toString(o));
    System.err.println("test_psi: w="+w+" A="+A);
    for(int k=0;k<neq;k++) o[k]/=Math.sqrt(o_o);
    UnitTests.checkDoubleRel(Math.abs(q.access.fx(psi,q.getX0())/Math.sqrt(o_o)),"|<psi|o>|",Math.abs(w),"|w|",eps);
    UnitTests.checkDoubleRel(Math.abs(TMatr.getAMatrB(psi,o,QQt,neq)),"|<psi|o>|",Math.abs(w),"|w|",eps);

    System.arraycopy(psi,0,r,0,neq);
    TMatr.sumP1(r,o,-w);
    UnitTests.checkDoubleRel(q.access.fx(r,q.getX0()),"<psi~|o>",0,"0",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(r,o,QQt,neq),"<psi~|o>",0,"0",eps);
    final double wr=TMatr.getAMatrA(r,QQt,neq);
    if(neq>1) UnitTests.checkDoubleRel(wr,"wr",1-w*w,"1-w*w",eps);

    System.err.println(name+":BQVMTest:test_psi(): test OK neq="+neq+" eps="+eps);
}

void test_psiL_psiH()
{
    final BQVM<DTYPE> q=getBQVM();

    test_psi(q.psiL,q.sQQMrvs,q.sQQt,q,q.wL,q.SArvsL);
    test_psi(q.psiH,q.sQQMrvs,q.sQQt,q,q.wH,q.SArvsH);
    test_psi(q.TS.psiL,q.sQQMts,q.sQQt,q,q.TS.wL,q.TS.sL);
    test_psi(q.TS.psiH,q.sQQMts,q.sQQt,q,q.TS.wH,q.TS.sH);
    test_psi(q.VS.psiL,q.sQQMvs,q.sQQt,q,q.VS.wL,q.VS.sL);
    test_psi(q.VS.psiH,q.sQQMvs,q.sQQt,q,q.VS.wH,q.VS.sH);
    test_psi(q.psiIE,q.sQQv,q.sQQt,q,q.wIE,q.AE);

    test_psi(q.psi_p_L,q.sQQMpv,q.sQQt,q,q.wPL,q.sPL);
    test_psi(q.psi_p_H,q.sQQMpv,q.sQQt,q,q.wPH,q.sPH);

    test_psi(q.RTS.psiL,q.sQQMrts,q.sQQv,q,q.RTS.wL,q.RTS.sL);
    test_psi(q.RTS.psiH,q.sQQMrts,q.sQQv,q,q.RTS.wH,q.RTS.sH);

    test_psi(q.RTST.psiL,q.sQQMrts,q.sQQt,q,q.RTST.wL,q.RTST.sL);
    test_psi(q.RTST.psiH,q.sQQMrts,q.sQQt,q,q.RTST.wH,q.RTST.sH);

    test_psi(q.RVSV.psiL,q.sQQMrvs,q.sQQv,q,q.RVSV.wL,q.RVSV.sL);
    test_psi(q.RVSV.psiH,q.sQQMrvs,q.sQQv,q,q.RVSV.wH,q.RVSV.sH);

    test_psi(q.IT.psiL,q.sQQv,q.sQQt,q,q.IT.wL,q.IT.sL);
    test_psi(q.IT.psiH,q.sQQv,q.sQQt,q,q.IT.wH,q.IT.sH);

    test_psi(q.ITsam.psiL,q.sQQv,q.sQQMt_sam,q,q.ITsam.wL,q.ITsam.sL);
    test_psi(q.ITsam.psiH,q.sQQv,q.sQQMt_sam,q,q.ITsam.wH,q.ITsam.sH);

    test_psi(q.DS.psiL,q.sQQx,q.sQQt,q,q.DS.wL,q.DS.sL);
    test_psi(q.DS.psiH,q.sQQx,q.sQQt,q,q.DS.wH,q.DS.sH);


    System.err.println(name+":BQVMTest:test_psiL_psiH(): test OK neq="+neq+" eps="+eps);
}

void test_TwoRootsIFX()
{
    final BQVM<DTYPE> q=getBQVM();

    TwoRootsIFX.test_F(q.rm0_kkdp_kkt,eps);
    TwoRootsIFX.test_F(q.rm0_kkvdp_kkt,eps);
    TwoRootsIFX.test_F(q.rm0_kkdp_kkiv,eps);
    TwoRootsIFX.test_F(q.rm0_kkdp_kka,eps);

    TwoRootsIFD.test_F(q.rms_kkdp_kka,eps);
    TwoRootsIFD.test_F(q.rms_kkvdp_kkt,eps);

    System.err.println(name+":BQVMTest:test_TwoRootsIFX(): test OK neq="+neq+" eps="+eps);
}

void test_ZQVM()
{
    final BQVM<DTYPE> q=getBQVM();
    final ZQVM<DTYPE> s=getZQVM();

    UnitTests.checkDoubleArrayRel(q.kkt,"q.kkt",s.kk,"s.kk",eps);
    UnitTests.checkDoubleArrayRel(q.kkrvs,"q.kkrvs",s.kkr,"s.kkr",eps);
    UnitTests.checkDoubleArrayRel(q.kka,"q.kka",s.kka,"s.kka",eps);

    UnitTests.checkDoubleRel(q.K00t,"q.K00t",s.K00,"s.K00",eps);
    UnitTests.checkDoubleRel(q.SA0,"q.SA0",s.ntrvs0,"s.ntrvs0",eps);
    UnitTests.checkDoubleRel(q.sigma2T_SA0,"q.sigma2T_SA0",s.sigma2T_ntrvs0,"s.sigma2T_ntrvs0",eps);
    UnitTests.checkDoubleRel(q.A0,"q.A0",s.nA0,"s.nA0",eps);
    UnitTests.checkDoubleRel(q.sigma2T_A0,"q.sigma2T_A0",s.sigma2T_nA0,"s.sigma2T_nA0",eps);

    System.err.println(name+":BQVMTest:test_ZQVM(): test OK neq="+neq+" eps="+eps);
}

void test_UQVM()
{
    final BQVM<DTYPE> q=getBQVM();
    final UQVM<DTYPE> s=getUQVM();

    final double [] qlam=new double [neq],slam=new double [neq],qpsi0=new double [neq];
    for(int k=0;k<neq;k++)
    {
	qlam[k]=q.ev[k];
	slam[k]=s.bv.ev[k]*s.w00;
	qpsi0[k]=q.ev[neq+k];
    }

    UnitTests.checkDoubleArrayRel(qlam,"qlam",slam,"slam",eps);
    UnitTests.checkDoubleRel(q.AE,"q.AE",q.AEt,"q.AEt",eps);
    UnitTests.checkDoubleArrayRel(EVXData.getMSign(qpsi0),"qpsi0",EVXData.getMSign(s.bv.psi0),"s.bv.psi0",eps);

    {
    UnitTests.checkDoubleArrayRel(EVXData.getMSign(q.o0t),"q.o0t",EVXData.getMSign(s.pv.psiok),"s.pv.psiok",eps);
    final double [] s_psi0t=new double [neq];System.arraycopy(s.pv.ev,neq+0*neq,s_psi0t,0,neq);
    UnitTests.checkDoubleArrayRel(EVXData.getMSign(q.psiIE),"q.psiIE",EVXData.getMSign(s_psi0t),"s_psi0t",eps);
    UnitTests.checkDoubleRel(q.ptE,"q.ptE",s.pv.pR[0],"s.pv.pR",eps);
    UnitTests.checkDoubleRel(q.sigma2_ptE,"q.sigma2_ptE",s.pv.stdev2_pR[0],"s.pv.stdev2_pR",eps);
    UnitTests.checkDoubleRel(q.pt0,"q.pt0",s.pv.pRok,"s.pv.pRok",eps);
    UnitTests.checkDoubleRel(q.sigma2_pt0,"q.sigma2_pt0",s.pv.stdev2_pRok,"s.pv.stdev2_pRok",eps);
   }
    {
    UnitTests.checkDoubleArrayRel(EVXData.getMSign(q.o0v,s.pt.psiok[neq-1]),"q.o0v",TMatr.getVectorScaled(s.pt.psiok,1/Math.sqrt(s.w00)),"s.pt.psiok",eps);
    final double [] s_psi0v=new double [neq];System.arraycopy(s.pt.ev,neq+0*neq,s_psi0v,0,neq);
    UnitTests.checkDoubleArrayRel(EVXData.getMSign(q.psiIv,s_psi0v[neq-1]),"q.psiIv",TMatr.getVectorScaled(s_psi0v,1/Math.sqrt(s.w00)),"s_psi0v",eps);
    UnitTests.checkDoubleRel(q.pvE,"q.pvE",s.pt.pR[0],"s.pt.pR",eps);
    UnitTests.checkDoubleRel(q.sigma2_pvE,"q.sigma2_pvE",s.pt.stdev2_pR[0],"s.pt.stdev2_pR",eps);
    UnitTests.checkDoubleRel(q.pv0,"q.pv0",s.pt.pRok,"s.pt.pRok",eps);
    UnitTests.checkDoubleRel(q.sigma2_pv0,"q.sigma2_pv0",s.pt.stdev2_pRok,"s.pt.stdev2_pRok",eps);
    }
    UnitTests.checkDoubleRel(q.o_o_v,"q.o_o_v",s.K00v/s.w00,"s.K00v/s.w00",eps);

    UnitTests.checkDoubleRel(q.pH0-s.getPOffset(),"q.pH0-s.getPOffset()",(s.bv.dP_I0-s.bv.dP_Iok)/(s.bv.I0-s.bv.Iok),"(s.bv.dP_I0-s.bv.dP_Iok)/(s.bv.I0-s.bv.Iok)",eps);

    UnitTests.checkDoubleRel(q.ppvH,"q.ppvH",s.pv.pL[0],"s.pv.pL[0]",eps);
    UnitTests.checkDoubleRel(q.pptH,"q.pptH",s.pv.pR[0],"s.pv.pR[0]",eps);
    UnitTests.checkDoubleRel(q.ppv0,"q.ppv0",s.pv.pLok,"s.pv.pLok",eps);
    UnitTests.checkDoubleRel(q.ppt0,"q.ppt0",s.pv.pRok,"s.pv.pRok",eps);

    System.err.println(name+":BQVMTest:test_UQVM(): test OK neq="+neq+" eps="+eps);
}


void test_Lastpoint()
{
    final BQVM<DTYPE> q=getBQVM();
    final Trade [] tb=new Trade [this.tb.length+1];
    System.arraycopy(this.tb,0,tb,0,this.tb.length);
    final double pl=130.0+2*r.nextDouble();
    final int vadded=10000;
    tb[this.tb.length]=new Trade(pl,
				 vadded,
				 null,
				 r.nextInt(2)==1?BookItem.BOOK_BUY:BookItem.BOOK_SELL,
				 currenttime,
				 -1,-1,-1,
				 null,Trade.TRADE_TYPE_REGULAR,
				 Trade.getDummyTrade(pl-10*r.nextDouble(),100,BookItem.BOOK_BUY,currenttime),
				 Trade.getDummyTrade(pl+10*r.nextDouble(),100,BookItem.BOOK_SELL,currenttime),
				 -1);
    this.tb=tb;
    final BQVM<DTYPE> s=getBQVM();
    System.err.println("q.dS="+q.dS+" s.dS="+s.dS);
    System.err.println("q.dSe="+q.dSe+" s.dSe="+s.dSe);
    System.err.println("q.dSf="+q.dSf+" s.dSf="+s.dSf);
    System.err.println("q.SA0="+q.SA0+" s.SA0="+s.SA0);
    System.err.println("q.SA1="+q.SA1+" s.SA1="+s.SA1);
    System.err.println("q.sigma2T_SA0="+q.sigma2T_SA0+" s.sigma2T_SA0="+s.sigma2T_SA0);
    System.err.println("q.A0="+q.A0+" s.A0="+s.A0+" vadded*o_o="+(vadded*s.o_o));
    System.err.println("q.sigma2T_A0="+q.sigma2T_A0+" s.sigma2T_A0="+s.sigma2T_A0);

    UnitTests.checkDoubleRel(s.A0,"s.A0",q.A0+vadded*q.o_o,"q.A0+vadded*q.o_o",eps);
    UnitTests.checkDoubleRel(q.K00t,"q.K00t",s.K00t,"s.K00t",eps);
    UnitTests.checkDoubleRel(q.sigma2T_SA0,"q.sigma2T_SA0",s.sigma2T_SA0,"s.sigma2T_SA0",eps);
    UnitTests.checkDoubleRel(q.sigma2T_A0,"q.sigma2T_A0",s.sigma2T_A0,"s.sigma2T_A0",eps);
    UnitTests.checkDoubleRel(q.SA1,"q.SA1",s.SA1,"s.SA1",eps);
    UnitTests.checkDoubleRel(q.A1,"q.A1",s.A1,"s.A1",eps);
    UnitTests.checkDoubleRel(q.K11t,"q.K11t",s.K11t,"s.K11t",eps);
    UnitTests.checkDoubleRel(q.SAM,"q.SAM",s.SAM,"s.SAM",eps);
    UnitTests.checkDoubleRel(q.AM,"q.AM",s.AM,"s.AM",eps);
    UnitTests.checkDoubleRel(q.XM,"q.XM",s.XM,"s.XM",eps);
    UnitTests.checkDoubleRel(q.SAR,"q.SAR",s.SAR,"s.SAR",eps);
    UnitTests.checkDoubleRel(q.AR,"q.AR",s.AR,"s.AR",eps);
    UnitTests.checkDoubleRel(q.SARrvsL,"q.SARrvsL",s.SARrvsL,"s.SARrvsL",eps);
    UnitTests.checkDoubleRel(q.SARrvsH,"q.SARrvsH",s.SARrvsH,"s.SARrvsH",eps);

    System.err.println(name+":BQVMTest:test_Lastpoint(): test OK neq="+neq+" eps="+eps);
}

static double [] getEVBasisCut(final int n,final int nb,final double [][] basis,final double [] QQv, final int nm_QQv,final double [] QQt,final int nm_QQt)
{
    final double [] qqv=new double [n*n],qqt=new double [n*n];
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<=k;l++)
	{
	    qqt[n*k+l]=qqt[n*l+k]=TMatr.getAMatrB(nb,basis[k],basis[l],QQt,nm_QQt);
	    qqv[n*k+l]=qqv[n*l+k]=TMatr.getAMatrB(nb,basis[k],basis[l],QQv,nm_QQv);
	}
    }
    final double [] ev=DeprecatedHelperEVXData.solveGeneralizedEV(n,qqv,qqt);
    // Make highest lamda on top. Inverse order.
    UQVM.inverseOrderOfEV(n,ev);
    return ev;
}

void test_EV(final double x0,final double [] qev,final double [] mom_v,final double [] mom_t,final boolean flag_cut,final double lmin,final double lmax,final boolean flag_inverse)
{

    final double [] QQMv=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_v,1),
	QQMt=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_t,1);
    final double [] o=NQVM.NQVMTest.getOK(neq,QQMt,x0);

    final double [][] rbasis=XQVM.XQVMTest.getRadauBasis(neq,o,x0,QQMt)[0];
    final int n;
    if(flag_cut) n=neq-1;
    else n=neq;
    if(neq+n*neq!=qev.length) throw new RuntimeException("Dimension mismatch: neq+n*neq="+(neq+n*neq)+" qev.length="+qev.length);

    final double [] ev=getEVBasisCut(n,neq,rbasis,QQMv,neq,QQMt,neq);
    if(flag_inverse) UQVM.inverseOrderOfEV(neq,ev);
    System.err.println("qev="+java.util.Arrays.toString(qev));
    System.err.println("ev="+java.util.Arrays.toString(ev));
    if(n+n*n!=ev.length) throw new RuntimeException("Dimension mismatch: n+n*n="+(n+n*n)+" ev.length="+ev.length);
    UnitTests.checkDoubleArrayRel(java.util.Arrays.copyOf(qev,n),"qev",java.util.Arrays.copyOf(ev,n),"ev",eps);

    final double [] qpsi=new double [neq],psi=new double [neq];
    double ee=flag_inverse?Double.NEGATIVE_INFINITY:Double.POSITIVE_INFINITY,tlmax=Double.NaN,tlmin=Double.NaN;
    for(int k=0;k<n;k++)
    {
	// match eigenvalue
	final double e=ev[k];
	UnitTests.checkDoubleRel(qev[k],"qev[k]",e,"ev[k]",eps);
	if(flag_inverse)
	{
	    if(!(e>=ee)) throw new RuntimeException("e="+e+" ee="+ee);
	}
	else
	{
	    if(!(e<=ee)) throw new RuntimeException("e="+e+" ee="+ee);
	}
	ee=e;
	if(flag_inverse?k==(n-1):k==0){UnitTests.checkDoubleRel(lmax,"lmax",e,"max e",eps);}
	else if(flag_inverse?k==0:k==(n-1)){UnitTests.checkDoubleRel(lmin,"lmin",e,"min e",eps);}

	// match eigenvector
	System.arraycopy(qev,neq+k*neq,qpsi,0,neq);
	final double [] qp=convertToMonomials(qpsi);
	java.util.Arrays.fill(psi,0);
	for(int l=0;l<n;l++)
	{
	    TMatr.sumP1(psi,rbasis[l],ev[n*(k+1)+l]);
	}
	if(Math.signum(psi[psi.length-1])!=Math.signum(qp[qp.length-1]))
	{
	    for(int l=qp.length;--l>=0;) qp[l]=-qp[l];
	}
	UnitTests.checkDoubleArrayRel(qp,"qp",psi,"psi",eps);
    }
    System.err.println(name+":BQVMTest:test_EV(): test OK neq="+neq+" eps="+eps);
}


void test_Monomials_BasisExpansion()
{
    final BQVM<DTYPE> q=getBQVM();
    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] mom_v=amoms[0],mom_t=amoms[1],mom_rts=amoms[2],mom_rvs=amoms[3],mom_vs=amoms[4],mom_ts=amoms[5],mom_t_sam=amoms[6],mom_pv=amoms[7];


    test_EV(q.getX0(),q.ev,mom_v,mom_t,false,q.AEl,q.AE,false);
    test_EV(q.getX0(),q.evcut,mom_v,mom_t,true,q.ARl,q.AR,false);
    test_EV(q.getX0(),q.rve,mom_rvs,mom_t,false,q.SArvsL,q.SArvsH,false);
    test_EV(q.getX0(),q.rvecut,mom_rvs,mom_t,true,q.SARrvsL,q.SARrvsH,false);
    test_EV(q.getX0(),q.pev,mom_pv,mom_t,false,q.sPL,q.sPH,false);
    // Differently sorted eigenvalues, last argument=true.
    test_EV(q.getX0(),q.TS.ev,mom_ts,mom_t,false,q.TS.sL,q.TS.sH,true);
    test_EV(q.getX0(),q.TSsam.ev,mom_ts,mom_t_sam,false,q.TSsam.sL,q.TSsam.sH,true);
    test_EV(q.getX0(),q.VS.ev,mom_vs,mom_t,false,q.VS.sL,q.VS.sH,true);
    test_EV(q.getX0(),q.VSsam.ev,mom_vs,mom_t_sam,false,q.VSsam.sL,q.VSsam.sH,true);
    test_EV(q.getX0(),q.RTS.ev,mom_rts,mom_v,false,q.RTS.sL,q.RTS.sH,true);
    test_EV(q.getX0(),q.RTST.ev,mom_rts,mom_t,false,q.RTST.sL,q.RTST.sH,true);
    test_EV(q.getX0(),q.RVSV.ev,mom_rvs,mom_v,false,q.RVSV.sL,q.RVSV.sH,true);
    test_EV(q.getX0(),q.IT.ev,mom_v,mom_t,false,q.IT.sL,q.IT.sH,true);

    System.err.println(name+":BQVMTest:test_Monomials_BasisExpansion(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":BQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_psiL_psiH();
    test_ZQVM();
    test_UQVM();
    test_Monomials_BasisExpansion();
    test_TwoRootsIFX();
    if(neq>1) test_Lastpoint();

    System.err.println(name+":BQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":BQVMTest: starting tests; ver=$Id: BQVM.java,v 1.227 2022/11/20 08:48:26 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":BQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: BQVM.java,v 1.227 2022/11/20 08:48:26 mal Exp $");
}
} // END OF class BQVMTest

}// END OF class BQVM
