package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.PolynomialsRatio;
import com.polytechnik.outils.*;
import com.polytechnik.realtimebook.*;

public class BQVMLaguerre extends BQVM<QVMDataL> {

static final Laguerre B=new Laguerre();

BQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static BQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final BQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new BQVMLaguerre(neq,o);
}
TwoCloseRootsFinder.RootsData getTRD(final double [] kkt,final double [] kkf)
    {return TwoCloseRootsFinder.findRootsData(getX0(),2.1,3000,new PolynomialsRatio(kkt,kkf,getABasis().B));}
TwoRootsIFX getTRIFX(final double [] kkt,final double [] kkf)
{
    return new TwoRootsIFX(kkt,kkf,access){
	TwoCloseRootsFinder.RootsData getTRD0(final double [] kkt,final double [] kkf)
	{
	    return TwoCloseRootsFinder.findRootsData(getX0(),2.1,3000,new TwoRootsNoDiv0(kkt,kkf,getABasis()));
	}
	double [] integrateP(final double [] x)
	{
	    return B.getIntegrated(x);
	}
    };
}
TwoRootsIFD getTRIFD(final double [] kkt,final double [] kkf)
{
    return new TwoRootsIFD(kkt,kkf,access){
	TwoCloseRootsFinder.RootsData getTRD0(final double [] kkt,final double [] kkf)
	{
	    return TwoCloseRootsFinder.findRootsData(getX0(),2.1,3000,new TwoRootsNoDiv0(kkt,kkf,getABasis()));
	}
    };
}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
double [] getXAll(final int n,final double [] IQQ)
{
    if(n<2) return new double []{getX0()};
    if(n==2) return new double []{getX0(),2.0}; // exact value of Radau nodes.
    final double [] x=B.getRadauRoots(n);
    com.polytechnik.utils.SortArrays.sortArrayWithLoad(x.length,x,null);
    x[0]=getX0(); // reduce rounding errors;
    return x;
}

static class BQVMLaguerreTest extends BQVMTest<QVMDataL> {
BQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6873L),1e-4);
}
long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
BQVMLaguerre getBQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
ZQVMLaguerre getZQVM(){return ZQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
UQVMLaguerre getUQVM(){return UQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData getPData(final int nmoms)
{
    final QVMData qdata=QVMData.calculateQVMData(1,nmoms,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
    return qdata;
}
double [][] getMonomialMoments(final QVMDataL odata)
{
    final int nmoms=2*neq-1;
    final QVMData qd=getPData(nmoms);
    return new double [][]{
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomT(nmoms),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomRTS(nmoms,qd),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomRVS(nmoms,qd),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomVS(nmoms,qd),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomTS(nmoms,qd),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dT,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_dV,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_dV,odata.p_offset)
    };
}


} // END of class BQVMLaguerreTest

public static void main(String [] args)
{
    new BQVMLaguerreTest("BQVMLaguerre").runTests(5);
} // END main



} // END OF class BQVMLaguerre
