package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.PolynomialsRatio;
import com.polytechnik.outils.*;
import com.polytechnik.realtimebook.*;

public class BQVMLegendreShifted extends BQVM<QVMDataP> {

static final LegendreShifted B=new LegendreShifted();

BQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static BQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final BQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new BQVMLegendreShifted(neq,o);
}
TwoCloseRootsFinder.RootsData getTRD(final double [] kkt,final double [] kkf)
    {return TwoCloseRootsFinder.findRootsData(getX0(),0.5,3000,new PolynomialsRatio(kkt,kkf,getABasis().B));}
TwoRootsIFX getTRIFX(final double [] kkt,final double [] kkf)
{
    return new TwoRootsIFX(kkt,kkf,access){
	TwoCloseRootsFinder.RootsData getTRD0(final double [] kkt,final double [] kkf)
	{
	    return TwoCloseRootsFinder.findRootsData(getX0(),0.5,3000,new TwoRootsNoDiv0(kkt,kkf,getABasis()));
	}
	double [] integrateP(final double [] x)
	{
	    return B.getIntegrated(x);
	}
    };
}
TwoRootsIFD getTRIFD(final double [] kkt,final double [] kkf)
{
    return new TwoRootsIFD(kkt,kkf,access){
	TwoCloseRootsFinder.RootsData getTRD0(final double [] kkt,final double [] kkf)
	{
	    return TwoCloseRootsFinder.findRootsData(getX0(),0.5,3000,new TwoRootsNoDiv0(kkt,kkf,getABasis()));
	}
    };
}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
double getX0(){return 1;}
double getPOffset(){return odata.p_offset;}
double [] getXAll(final int n,final double [] IQQ){
    if(n<2) return new double []{getX0()};
    if(n==2) return new double []{getX0(),1.0/3.0}; // exact value of Radau nodes.
    final double [] x=B.getRadauRoots(n);
    com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(x.length,x,null);
    x[0]=getX0(); // reduce rounding errors;
    return x;
}

static class BQVMLegendreShiftedTest extends BQVMTest<QVMDataP> {
BQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
BQVMLegendreShifted getBQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
ZQVMLegendreShifted getZQVM(){return ZQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
UQVMLegendreShifted getUQVM(){return UQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData [] getPData(final int nmoms)
{
    final QVMData [] qdata=new QVMData [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}
double [][] getMonomialMoments(final QVMDataP odata)
{
    final int nmoms=2*neq-1;
    final QVMData [] qd=getPData(nmoms);
    return new double [][]{
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomT(nmoms),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomRTS(nmoms,qd),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomRVS(nmoms,qd),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomVS(nmoms,qd),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomTS(nmoms,qd),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dT,true,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_dV,false,odata.p_offset)
    };
}

} // END of class BQVMLegendreShiftedTest

public static void main(String [] args)
{
    new BQVMLegendreShiftedTest("BQVMLegendreShifted").runTests(5);
} // END main



} // END OF class BQVMLegendreShifted
