package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.outils.*;
import com.polytechnik.realtimebook.*;

/** BQVM and Rurik measure (continued).
 *  @version $Id: BiQVM.java,v 1.160 2022/08/08 20:10:45 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class BiQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;

/** Rurik evolution. */
final EVXData I,TI,pI,Vt,VT,Pt,pt,P2t,P2i,p2ti,p2t,p2v,PV,pV,P2V,p2V,Ct,C2t,tsI;
final DPDVm d,D,dm,Dm,C,Cm;

final EVXData Cx,C2x;

final EVXData dAt,dAv,dRTSa,dRTSt,dRTSv,dPa,dPt,dPv;
final EVXData Rx,Px;
final EVXData dp2I;
final double dst_i,dsi_t,pvL_dp2I;
final QQDX J;
final EVXData dVSt;
final TwoCloseRootsFinder.RootsData rv;
final double rv_vs_x0,rv_vs_x1,rv_vs_x2,rv_Pt_x0,rv_Pt_x1,rv_Pt_x2,rv_pt_x0,rv_pt_x1,rv_pt_x2,
    rv_DdpI_x0,
    rv_DdpI_x1,
    rv_DdpI_x2,
    rv_DvsI_x0,
    rv_DvsI_x1,
    rv_DvsI_x2,
    rv_Dpv_x0,rv_Dpv_x1,rv_Dpv_x2,
    rv_DDpI_x0,rv_DDpI_x1,rv_DDpI_x2,
    rv_DSI_x0,rv_DSI_x1,rv_DSI_x2;
final double psiH_vs_psiH,psiL_vs_psiL,psi0_vs_psi0,
    psi0_vs_psiH,psi0_vs_psiL,psi0_psiH,psi0_psiL,
    psiH_rvs_psiH,psiL_rvs_psiL,psi0_rvs_psi0;
final double pt0,ptL,ptH,Pt0,PtL,PtH,pv0,pvL,pvH,
    VS0,VSL,VSH,RVS0,RVSL,RVSH;
final double psiH_C1_psiH,
    psiH_VS_psiH,
    psiH_RVS_psiH,
    psiH_P_psiH,
    psiH_pt_psiH,
    psiH_pv_psiH,
    psiH_dI_psiH;
final double idp_0,idp_L,idp_H,ivs_0,ivs_L,ivs_H,
Didp_0,Didp_L,Didp_H,Divs_0,Divs_L,Divs_H,
    sigma2_i_0,sigma2_dp_0,sigma2_vs_0,
    DdpI_0,
    DdpI_L,
    DdpI_H,
    DvsI_0,
    DvsI_L,
    DvsI_H;
final double DpsiH_dp_psiH,
    DpsiL_dp_psiL,
    Dpsi0_dp_psi0,
    DpsiH_vs_psiH,
    DpsiL_vs_psiL,
    Dpsi0_vs_psi0;
final EVXData DdPt,DIt,dxt,Ddxt;
final double ptQ0,ptQL,ptQH,PtQ0,PtQL,PtQH,
    DptQ0,DptQL,DptQH,DPtQ0,DPtQL,DPtQH,
    dp_i_QL,dp_dp_QL,i_i_QL,
    vsQL,rvsQL,arvsQL,c_Dpt_i_QL,c_vs_i_QL,c_Dpt_c_QL,c_vs_c_QL,c_dp_i_x0,
    cv_Dpt_i_QL,cv_vs_i_QL,cv_Dpt_c_QL,cv_vs_c_QL;
final double dpP2QL,adpP2QL,sigma2_dpP2QL;
final double DP_P2t_psiL,DP2_P2i_psiL,DP1_P2i_psiL,DP0_P2i_psiL,jm;

BiQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final double p_offset=getPOffset();

    final int nmoms=2*neq-1;
    final double [] imom=new double [nmoms],tmom=new double [nmoms],
	ptmom=new double [nmoms],p2tmom=new double [nmoms],
	pvmom=new double [nmoms],p2vmom=new double [nmoms],
	dpmom=new double [nmoms],dp2mom=new double [nmoms],
	damom=new double [nmoms],
	tsmom=new double [nmoms],vsmom=new double [nmoms],
	rtsmom=new double [nmoms],rvsmom=new double [nmoms],
	Pmom=new double [nmoms],P2mom=new double [nmoms],
	C1mom=new double [nmoms],C2mom=new double [nmoms],
	Vmom=new double [nmoms],
	Tmom=new double [nmoms],
	VSmom=new double [nmoms],RVSmom=new double [nmoms];
    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(ptmom,1,access.M_T,odata,nmoms);
    access.setM(p2tmom,2,access.M_T,odata,nmoms);
    access.setM(pvmom,1,access.M_I,odata,nmoms);
    access.setM(p2vmom,2,access.M_I,odata,nmoms);
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setM(dp2mom,2,access.M_DP,odata,nmoms);
    access.setM(damom,0,access.M_A,odata,nmoms);
    access.setM(tsmom,0,access.M_TS,odata,nmoms);
    access.setM(vsmom,0,access.M_VS,odata,nmoms);
    access.setM(rtsmom,0,access.M_RTS,odata,nmoms);
    access.setM(rvsmom,0,access.M_RVS,odata,nmoms);
    access.setFMoments(Vmom,imom,nmoms);
    access.setFMoments(Tmom,tmom,nmoms);
    access.setFMoments(Pmom,dpmom,nmoms);
    access.setFMoments(P2mom,dp2mom,nmoms);
    access.setFMoments(C1mom,pvmom,nmoms);
    access.setFMoments(C2mom,p2vmom,nmoms);
    access.setFMoments(VSmom,vsmom,nmoms);
    access.setFMoments(RVSmom,rvsmom,nmoms);
    for(int k=nmoms;--k>=0;) {Vmom[k]=-Vmom[k];Tmom[k]=-Tmom[k];C1mom[k]=-C1mom[k];C2mom[k]=-C2mom[k];}
    final double x0=getX0(),rho_x0=getRho(x0);

    final double [] Ddpmom=getDIMoms(neq,dpmom,tmom,access,x0,rho_x0),
	Dvsmom=getDIMoms(neq,vsmom,tmom,access,x0,rho_x0),
	Dimom=getDIMoms(neq,imom,tmom,access,x0,rho_x0);


    this.I=DeprecatedHelperEVXData.getEVXData(neq,imom,tmom,access,x0);
    this.TI=DeprecatedHelperEVXData.getEVXData(neq,tmom,imom,access,x0);
    this.pI=DeprecatedHelperEVXData.getEVXData(neq,ptmom,imom,access,x0);
    this.Vt=DeprecatedHelperEVXData.getEVXData(neq,Vmom,tmom,access,x0);
    this.VT=DeprecatedHelperEVXData.getEVXData(neq,Vmom,Tmom,access,x0);
    this.Pt=DeprecatedHelperEVXData.getEVXData(neq,Pmom,tmom,access,x0);
    this.pt=DeprecatedHelperEVXData.getEVXData(neq,ptmom,tmom,access,x0);
    this.P2t=DeprecatedHelperEVXData.getEVXData(neq,P2mom,tmom,access,x0);
    this.P2i=DeprecatedHelperEVXData.getEVXData(neq,P2mom,imom,access,x0);
    this.p2ti=DeprecatedHelperEVXData.getEVXData(neq,p2tmom,imom,access,x0);
    this.p2t=DeprecatedHelperEVXData.getEVXData(neq,p2tmom,tmom,access,x0);
    this.p2v=DeprecatedHelperEVXData.getEVXData(neq,p2vmom,tmom,access,x0);
    this.PV=DeprecatedHelperEVXData.getEVXData(neq,Pmom,Vmom,access,x0);
    this.pV=DeprecatedHelperEVXData.getEVXData(neq,ptmom,Vmom,access,x0);
    this.P2V=DeprecatedHelperEVXData.getEVXData(neq,P2mom,Vmom,access,x0);
    this.p2V=DeprecatedHelperEVXData.getEVXData(neq,p2tmom,Vmom,access,x0);
    this.Ct=DeprecatedHelperEVXData.getEVXData(neq,C1mom,tmom,access,x0);
    this.C2t=DeprecatedHelperEVXData.getEVXData(neq,C2mom,tmom,access,x0);
    this.D=DPDVm.getDPDVm(neq,Vmom,tmom,Pmom,P2mom,access,true);
    this.d=DPDVm.getDPDVm(neq,Vmom,tmom,ptmom,p2tmom,access,true);
    this.Dm=DPDVm.getDPDVm(neq,Vmom,tmom,Pmom,P2mom,access,false);
    this.dm=DPDVm.getDPDVm(neq,Vmom,tmom,ptmom,p2tmom,access,false);
    this.C=DPDVm.getDPDVm(neq,tmom,Vmom,C1mom,C2mom,access,true);
    this.Cm=DPDVm.getDPDVm(neq,tmom,Vmom,C1mom,C2mom,access,false);
    this.dAt=DeprecatedHelperEVXData.getEVXData(neq,damom,tmom,access,x0);
    this.dAv=DeprecatedHelperEVXData.getEVXData(neq,damom,imom,access,x0);
    this.dRTSa=DeprecatedHelperEVXData.getEVXData(neq,rtsmom,damom,access,x0);
    this.dRTSt=DeprecatedHelperEVXData.getEVXData(neq,rtsmom,tmom,access,x0);
    this.dRTSv=DeprecatedHelperEVXData.getEVXData(neq,rtsmom,imom,access,x0);
    this.tsI=DeprecatedHelperEVXData.getEVXData(neq,tsmom,imom,access,x0);
    this.dPa=DeprecatedHelperEVXData.getEVXData(neq,dpmom,damom,access,x0);
    this.dPt=DeprecatedHelperEVXData.getEVXData(neq,dpmom,tmom,access,x0);
    this.dPv=DeprecatedHelperEVXData.getEVXData(neq,dpmom,imom,access,x0);
    this.dVSt=DeprecatedHelperEVXData.getEVXData(neq,vsmom,tmom,access,x0);

    this.DdPt=DeprecatedHelperEVXData.getEVXData(neq,Ddpmom,tmom,access,x0);
    this.DIt=DeprecatedHelperEVXData.getEVXData(neq,Dimom,tmom,access,x0);
    final double [] dxmom=new double [nmoms];
    for(int k=0;k<dxmom.length;k++) dxmom[k]=dpmom[k]-imom[k]*(dPt.s0/I.s0);
    final double [] Ddxmom=getDIMoms(neq,dxmom,tmom,access,x0,rho_x0);
    this.dxt=DeprecatedHelperEVXData.getEVXData(neq,dxmom,tmom,access,x0);
    this.Ddxt=DeprecatedHelperEVXData.getEVXData(neq,Ddxmom,tmom,access,x0);




    this.J=new QQDX(neq,imom,tmom,dpmom,access,x0,rho_x0);
    if(!(J.rho_x0==1)) throw new RuntimeException("J.rho_x0="+J.rho_x0);
    final double [] tmp=new double [neq*neq];


    final double [][] QQ=access.getQQ_Matrs(neq,1,new double [][]{tmom,imom,vsmom,rvsmom,ptmom,Pmom,pvmom,VSmom,RVSmom,C1mom,dpmom,J.tDimom,Ddpmom,Dvsmom,damom,P2mom});
    final double [] QQt=QQ[0],QQv=QQ[1],QQvs=QQ[2],QQrvs=QQ[3],
	QQpt=QQ[4],QQPt=QQ[5],QQpv=QQ[6],QQVS=QQ[7],QQRVS=QQ[8],QQC1=QQ[9],QQdp=QQ[10],QQDi=QQ[11],QQDdp=QQ[12],QQDvs=QQ[13],QQadp=QQ[14],QQP2t=QQ[15],
	IQQv=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQv),
	IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQt),
	QQdpIQQtQQdp=ZQVM.getIQQ_F_IQQ(neq,QQdp,IQQt,tmp);
    dp2I=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQdpIQQtQQdp,QQv,access,x0);
    pvL_dp2I=p_offset+TMatr.getAMatrA(dp2I.psiL,QQpv,neq);

    psiH_C1_psiH=TMatr.getAMatrB(I.psiH,I.psiH,QQC1,neq);
    psiH_VS_psiH=TMatr.getAMatrB(I.psiH,I.psiH,QQVS,neq);
    psiH_RVS_psiH=TMatr.getAMatrB(I.psiH,I.psiH,QQRVS,neq);
    psiH_P_psiH=TMatr.getAMatrB(I.psiH,I.psiH,QQPt,neq);
    psiH_pt_psiH=TMatr.getAMatrB(I.psiH,I.psiH,QQpt,neq);
    psiH_pv_psiH=TMatr.getAMatrB(I.psiH,I.psiH,QQpv,neq);
    psiH_dI_psiH=TMatr.getAMatrB(I.psiH,I.psiH,QQDi,neq);
    this.psiH_vs_psiH=TMatr.getAMatrB(I.psiH,I.psiH,QQvs,neq);
    this.psiL_vs_psiL=TMatr.getAMatrB(I.psiL,I.psiL,QQvs,neq);
    this.psi0_vs_psi0=TMatr.getAMatrB(I.psi0,I.psi0,QQvs,neq);
    psi0_vs_psiH=TMatr.getAMatrB(I.psi0,I.psiH,QQvs,neq);
    psi0_vs_psiL=TMatr.getAMatrB(I.psi0,I.psiL,QQvs,neq);
    psi0_psiH=TMatr.getAMatrB(I.psi0,I.psiH,QQt,neq);
    psi0_psiL=TMatr.getAMatrB(I.psi0,I.psiL,QQt,neq);

    idp_0=getACORR(neq,QQdp,QQv,IQQt,I.psi0);
    idp_L=getACORR(neq,QQdp,QQv,IQQt,I.psiL);
    idp_H=getACORR(neq,QQdp,QQv,IQQt,I.psiH);
    ivs_0=getACORR(neq,QQvs,QQv,IQQt,I.psi0);
    ivs_L=getACORR(neq,QQvs,QQv,IQQt,I.psiL);
    ivs_H=getACORR(neq,QQvs,QQv,IQQt,I.psiH);
    sigma2_i_0=geAM(neq,QQv,IQQt,QQv,I.psi0);
    sigma2_dp_0=geAM(neq,QQdp,IQQt,QQdp,I.psi0);
    sigma2_vs_0=geAM(neq,QQvs,IQQt,QQvs,I.psi0);
    Didp_0=getACORR(neq,QQdp,QQDi,IQQt,I.psi0);
    Didp_L=getACORR(neq,QQdp,QQDi,IQQt,I.psiL);
    Didp_H=getACORR(neq,QQdp,QQDi,IQQt,I.psiH);
    Divs_0=getACORR(neq,QQvs,QQDi,IQQt,I.psi0);
    Divs_L=getACORR(neq,QQvs,QQDi,IQQt,I.psiL);
    Divs_H=getACORR(neq,QQvs,QQDi,IQQt,I.psiH);


    DdpI_0=getACORR(neq,QQDdp,QQv,IQQt,I.psi0);
    t0_DdpI_0=geAM(neq,QQDdp,IQQt,QQv,I.psi0);
    DdpI_L=getACORR(neq,QQDdp,QQv,IQQt,I.psiL);
    DdpI_H=getACORR(neq,QQDdp,QQv,IQQt,I.psiH);
    DvsI_0=getACORR(neq,QQDvs,QQv,IQQt,I.psi0);
    t0_DvsI_0=geAM(neq,QQDvs,IQQt,QQv,I.psi0);
    DvsI_L=getACORR(neq,QQDvs,QQv,IQQt,I.psiL);
    DvsI_H=getACORR(neq,QQDvs,QQv,IQQt,I.psiH);



    this.psiH_rvs_psiH=TMatr.getAMatrB(I.psiH,I.psiH,QQrvs,neq);
    this.psiL_rvs_psiL=TMatr.getAMatrB(I.psiL,I.psiL,QQrvs,neq);
    this.psi0_rvs_psi0=TMatr.getAMatrB(I.psi0,I.psi0,QQrvs,neq);

    ptH=p_offset+TMatr.getAMatrA(I.psiH,QQpt,neq);
    ptL=p_offset+TMatr.getAMatrA(I.psiL,QQpt,neq);
    pt0=p_offset+TMatr.getAMatrA(I.psi0,QQpt,neq);
    PtH=p_offset+TMatr.getAMatrA(I.psiH,QQPt,neq);
    PtL=p_offset+TMatr.getAMatrA(I.psiL,QQPt,neq);
    Pt0=p_offset+TMatr.getAMatrA(I.psi0,QQPt,neq);


    ptQH=p_offset+TMatr.getAMatrA(p2t.psiH,QQpt,neq);
    ptQL=p_offset+TMatr.getAMatrA(p2t.psiL,QQpt,neq);
    ptQ0=p_offset+TMatr.getAMatrA(p2t.psi0,QQpt,neq);
    PtQH=p_offset+TMatr.getAMatrA(P2t.psiH,QQPt,neq);
    PtQL=p_offset+TMatr.getAMatrA(P2t.psiL,QQPt,neq);
    PtQ0=p_offset+TMatr.getAMatrA(P2t.psi0,QQPt,neq);

    DptQH=TMatr.getAMatrA(p2t.psiH,QQdp,neq);
    DptQL=TMatr.getAMatrA(p2t.psiL,QQdp,neq);
    DptQ0=TMatr.getAMatrA(p2t.psi0,QQdp,neq);
    DPtQH=TMatr.getAMatrA(P2t.psiH,QQdp,neq);
    DPtQL=TMatr.getAMatrA(P2t.psiL,QQdp,neq);
    DPtQ0=TMatr.getAMatrA(P2t.psi0,QQdp,neq);

    dp_i_QL=geAM(neq,QQdp,IQQt,QQv,P2i.psiL);
    //System.err.println("dpQL="+TMatr.getAMatrA(P2i.psiL,TMatr.mulMMM(neq,QQdp,IQQt,QQv),neq));
    dp_dp_QL=geAM(neq,QQdp,IQQt,QQdp,P2i.psiL);
    i_i_QL=geAM(neq,QQv,IQQt,QQv,P2i.psiL);

    vsQL=TMatr.getAMatrA(P2i.psiL,QQvs,neq);

    rvsQL=TMatr.getAMatrA(P2i.psiL,QQrvs,neq);
    arvsQL=TMatr.getAMatrA(P2i.psiL,QQv,neq);


    dpP2QL=TMatr.getAMatrA(P2t.psiL,QQdp,neq);
    adpP2QL=TMatr.getAMatrA(P2t.psiL,QQadp,neq);
    sigma2_dpP2QL=geAM(neq,QQdp,IQQt,QQdp,P2t.psiL);
    DP_P2t_psiL=TMatr.getAMatrA(P2t.psiL,QQPt,neq);

    DP2_P2i_psiL=TMatr.getAMatrA(P2i.psiL,QQP2t,neq);
    DP1_P2i_psiL=TMatr.getAMatrA(P2i.psiL,QQPt,neq);
    DP0_P2i_psiL=TMatr.getAMatrA(P2i.psiL,QQt,neq);
    jm=DP2_P2i_psiL-Math.pow(DP1_P2i_psiL,2)/DP0_P2i_psiL;


    c_dp_i_x0=getACORR(neq,QQdp,QQv,IQQt,P2t.psi0);
    c_Dpt_i_QL=getACORR(neq,QQdp,QQv,IQQt,P2t.psiL);
    c_vs_i_QL=getACORR(neq,QQvs,QQv,IQQt,P2t.psiL);
    c_Dpt_c_QL=getACORR(neq,QQdp,QQt,IQQt,P2t.psiL);
    c_vs_c_QL=getACORR(neq,QQvs,QQt,IQQt,P2t.psiL);

    cv_Dpt_i_QL=getACORR(neq,QQdp,QQv,IQQv,P2i.psiL);
    cv_vs_i_QL=getACORR(neq,QQvs,QQv,IQQv,P2i.psiL);
    cv_Dpt_c_QL=getACORR(neq,QQdp,QQt,IQQv,P2i.psiL);
    cv_vs_c_QL=getACORR(neq,QQvs,QQt,IQQv,P2i.psiL);

    pvH=p_offset+TMatr.getAMatrB(I.psiH,I.psiH,QQpv,neq)/I.sH;
    pvL=p_offset+TMatr.getAMatrB(I.psiL,I.psiL,QQpv,neq)/I.sL;
    pv0=p_offset+TMatr.getAMatrB(I.psi0,I.psi0,QQpv,neq)/I.s0;

    VSH=TMatr.getAMatrB(I.psiH,I.psiH,QQVS,neq);
    VSL=TMatr.getAMatrB(I.psiL,I.psiL,QQVS,neq);
    VS0=TMatr.getAMatrB(I.psi0,I.psi0,QQVS,neq);
    RVSH=TMatr.getAMatrB(I.psiH,I.psiH,QQRVS,neq);
    RVSL=TMatr.getAMatrB(I.psiL,I.psiL,QQRVS,neq);
    RVS0=TMatr.getAMatrB(I.psi0,I.psi0,QQRVS,neq);

    if(true)
    {

	sdCmomx=TMatr.getMatr_x_A(neq,QQt,TMatr.getMatr_x_A(neq,IQQv,java.util.Arrays.copyOf(dpmom,neq)));

	this.Cx=getEX(neq,tmom,dpmom,imom);
	this.C2x=getEX(neq,tmom,dp2mom,imom);
	this.dst_i=TMatr.getAMatrB(neq,rtsmom,imom,IQQt,neq);
	this.dsi_t=TMatr.getAMatrB(neq,rvsmom,tmom,IQQt,neq);
    }



    this.Rx=getEX(neq,tmom,rtsmom,imom);
    this.Px=getEX(neq,tmom,dpmom,imom);



    if(true)
    {
	final double []
	    MQQv=ZQVM.getIQQ_F_IQQ(neq,IQQt,QQv,tmp),
	    MQQvs=ZQVM.getIQQ_F_IQQ(neq,IQQt,QQvs,tmp),
	    MQQPt=ZQVM.getIQQ_F_IQQ(neq,IQQt,QQPt,tmp),
	    MQQpt=ZQVM.getIQQ_F_IQQ(neq,IQQt,QQpt,tmp),
	    MQQpv=ZQVM.getIQQ_F_IQQ(neq,IQQt,QQpt,tmp),
	    MQQDpI=new double [neq*neq],MQQSI=new double [neq*neq],
	    MQQDdpI=new double [neq*neq],MQQDvsI=new double [neq*neq];
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,tmp,QQdp,neq,MQQv,neq);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,MQQDpI,IQQt,neq,tmp,neq);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,tmp,QQvs,neq,MQQv,neq);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,MQQSI,IQQt,neq,tmp,neq);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,tmp,QQDdp,neq,MQQv,neq);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,MQQDdpI,IQQt,neq,tmp,neq);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,tmp,QQDvs,neq,MQQv,neq);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,MQQDvsI,IQQt,neq,tmp,neq);

	final double [][] kks=access.getKK(neq,new double [][] {IQQt,MQQv,MQQvs,MQQPt,MQQpt,MQQpv,MQQDpI,MQQSI,MQQDdpI,MQQDvsI});
	final double [] kkt=kks[0],kkv=kks[1],kkvs=kks[2],kkPt=kks[3],kkpt=kks[4],kkpv=kks[5],kkDpI=kks[6],kkSI=kks[7],kkDdpI=kks[8],kkDvsI=kks[9];
	this.rv=getTR(kkt,kks[1]);
	rv_vs_x0=access.fx(kkvs,rv.x0)/access.fx(kkt,rv.x0);
	rv_vs_x1=access.fx(kkvs,rv.x1)/access.fx(kkt,rv.x1);
	rv_vs_x2=access.fx(kkvs,rv.x2)/access.fx(kkt,rv.x2);
	rv_Pt_x0=access.fx(kkPt,rv.x0)/access.fx(kkt,rv.x0);
	rv_Pt_x1=access.fx(kkPt,rv.x1)/access.fx(kkt,rv.x1);
	rv_Pt_x2=access.fx(kkPt,rv.x2)/access.fx(kkt,rv.x2);
	rv_pt_x0=access.fx(kkpt,rv.x0)/access.fx(kkt,rv.x0);
	rv_pt_x1=access.fx(kkpt,rv.x1)/access.fx(kkt,rv.x1);
	rv_pt_x2=access.fx(kkpt,rv.x2)/access.fx(kkt,rv.x2);

	rv_DdpI_x0=access.fx(kkDdpI,rv.x0)/access.fx(kkt,rv.x0);
	rv_DdpI_x1=access.fx(kkDdpI,rv.x1)/access.fx(kkt,rv.x1);
	rv_DdpI_x2=access.fx(kkDdpI,rv.x2)/access.fx(kkt,rv.x2);
	rv_DvsI_x0=access.fx(kkDvsI,rv.x0)/access.fx(kkt,rv.x0);
	rv_DvsI_x1=access.fx(kkDvsI,rv.x1)/access.fx(kkt,rv.x1);
	rv_DvsI_x2=access.fx(kkDvsI,rv.x2)/access.fx(kkt,rv.x2);

	rv_Dpv_x0=getDD(kkpv,kkt,access,rv.x0);
	rv_Dpv_x1=getDD(kkpv,kkt,access,rv.x1);
	rv_Dpv_x2=getDD(kkpv,kkt,access,rv.x2);
	rv_DDpI_x0=getDD(kkDpI,kkt,access,rv.x0);
	rv_DDpI_x1=getDD(kkDpI,kkt,access,rv.x1);
	rv_DDpI_x2=getDD(kkDpI,kkt,access,rv.x2);
	rv_DSI_x0=getDD(kkSI,kkt,access,rv.x0);
	rv_DSI_x1=getDD(kkSI,kkt,access,rv.x1);
	rv_DSI_x2=getDD(kkSI,kkt,access,rv.x2);
    }

    if(true)
    {
	DpsiH_dp_psiH=getPsiDQQPsi(neq,I.psiH,dpmom,QQt);
	DpsiL_dp_psiL=getPsiDQQPsi(neq,I.psiL,dpmom,QQt);
	Dpsi0_dp_psi0=getPsiDQQPsi(neq,I.psi0,dpmom,QQt);
	DpsiH_vs_psiH=getPsiDQQPsi(neq,I.psiH,vsmom,QQt);
	DpsiL_vs_psiL=getPsiDQQPsi(neq,I.psiL,vsmom,QQt);
	Dpsi0_vs_psi0=getPsiDQQPsi(neq,I.psi0,vsmom,QQt);
    }

    if(true)
    {
	sVmom=Vmom;
	sPmom=Pmom;
	sP2mom=P2mom;
	stmom=tmom;
	sdpmom=dpmom;
	simom=imom;
	sQQt=QQt;
    }
}

/** For unit tests. */
final double t0_DdpI_0,t0_DvsI_0;
final double [] sVmom,sPmom,sP2mom,stmom,sdpmom,simom,sdCmomx,sQQt;

static double getDD(final double [] nom,final double [] denom,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x)
{
    final double d=access.fx(denom,x),Dd=access.Dfx(denom,x);
    return access.Dfx(nom,x)/d-(Dd/d)*(access.fx(nom,x)/d);
}

double getPsiDQQPsi(final int n,final double [] Psi,final double [] mom,final double [] QQt)
{
    final double [] psi=getPsiORemoved(n,Psi,QQt);
    final double [] DQQ=getDQQ_Matrs(n,1,new double [][]{mom})[0];
    return TMatr.getAMatrA(psi,DQQ,n);
}


abstract int setDF(final int n,final double [] res,final double [] f);
double [][] getDQQ_Matrs(final int n,final double scale,final double [][] moms)
{
    final double [][] r=new double [moms.length][n*n];
    for(int k=0;k<n;k++)
    {
	final int ok=n*k;
	for(int l=0;l<=k;l++)
	{
	    final double [] tmp=new double [l+k+1];
	    final int na=access.setQQCoefs(l,k,tmp);
	    final int ne=setDF(na,tmp,tmp);
	    for(int j=r.length,kl=ok+l,lk=n*l+k;--j>=0;)
	    {
		r[j][kl]=r[j][lk]=TMatr.mulVVn(ne,tmp,moms[j])/scale;
	    }
	}
    }
    //  <Q_i|Q_j> are now generated for all moms
    return r;
}
double [] getPsiORemoved(final int n,final double [] psi,final double [] QQt)
{
    final double x0=getX0();
    final double [] ok=access.getOK(n,QQt,x0);
    final double o_o=access.fx(ok,x0);
    final double psi_o=TMatr.getAMatrB(psi,ok,QQt,n)/o_o;
    final double [] psio=new double [n];
    for(int k=0;k<n;k++)
    {
	psio[k]=psi[k]-psi_o*ok[k];
    }
    return psio;
}

abstract double getRho(final double x);
abstract TwoCloseRootsFinder.RootsData getTR(final double [] kkdenom,final double [] kknom);


/** Calculates psi^T QQa QQb QQc psi
 */
static double geAM(final int n,final double [] QQa,final double [] QQb,final double [] QQc,final double [] psi)
{
    final double [] tmp=new double [n*n];
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,QQb,n,QQc,n);
    final double [] x=TMatr.mulMM(n,QQa,tmp);
    return TMatr.getAMatrA(psi,x,n);
}

static double getACORR(final int n,final double [] QQdp,final double [] QQv,final double [] IQQt,final double [] psi)
{
    return geAM(n,QQdp,IQQt,QQv,psi)/Math.sqrt(geAM(n,QQdp,IQQt,QQdp,psi)*geAM(n,QQv,IQQt,QQv,psi));
}

static double [] getDIMoms(final int n,final double [] imom,final double [] tmom,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0,final double rho_x0)
{
    return getDIMoms(n,imom,tmom,access,x0,rho_x0,false);
}

static double [] getDIMoms(final int n,final double [] imom,final double [] tmom,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0,final double rho_x0,final boolean flag_0_at_x0)
{
    final int nmoms=2*n-1;
    final double [] Dimom=new double [nmoms];
    access.setDFMoments(Dimom,imom,nmoms);
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{tmom,flag_0_at_x0?Dimom:imom});
    final double [] QQt=QQ[0],QQi=QQ[1],ok=access.getOK(n,QQt,x0);

    final double o_o=access.fx(ok,x0),I0=TMatr.getAMatrA(ok,QQi,n)/(flag_0_at_x0?o_o*o_o:o_o);

    final double [] rr=new double [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	rr[k]=1;
	Dimom[k]=I0*rho_x0*access.fx(rr,x0)-Dimom[k];
	rr[k]=0;
    }
    return Dimom;
}



static class QQDX {
final DeprecatedHelperDataDirectAccessWithBasis<?> access;
final EVXData IT,DIT;
final double osp0;
final double I0,o_o,x0,rho_x0;
final double [] QQi,QQt,QQd,tQQd,IQQt;
final double dp_dp0,dp_di0,di_di0,P0;
final double tdp_di0,tdi_di0;
final double tdp_dp,tdp_di,tdi_di,tP;
final double [] tDimom,imom,tmom,dpmom;
QQDX(final int n,final double [] imom,final double [] tmom,final double [] dpmom,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0,final double rho_x0)
{
    this.access=access;
    this.x0=x0;
    this.rho_x0=rho_x0;
    final double [] Dimom=new double [2*n-1];
    access.setDFMoments(Dimom,imom,Dimom.length);
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,Dimom});
    this.QQi=QQ[0];
    this.QQt=QQ[1];
    this.QQd=QQ[2];
    this.IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
    this.IT=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);
    final double [] ok=access.getOK(n,QQt,x0);
    this.o_o=access.fx(ok,x0);
    this.I0=TMatr.getAMatrA(ok,QQi,n)/o_o;
    final double [] r=new double [n],Qx0=new double [n];
    for(int k=0;k<n;k++)
    {
	r[k]=1;Qx0[k]=access.fx(r,x0);r[k]=0;
    }
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<n;l++)
	{
	    QQd[k*n+l]=I0*rho_x0*Qx0[k]*Qx0[l]-QQd[k*n+l];
	}
    }
    this.DIT=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQd,QQt,access,x0);
    final double dp_o=TMatr.mulVVn1(ok,dpmom),di_o=TMatr.mulVVn1(ok,Dimom),
	i_o=TMatr.mulVVn1(ok,imom);

    this.dp_di0=-(TMatr.getAMatrB(n,dpmom,Dimom,IQQt,n)-(dp_o*di_o/o_o));
    this.dp_dp0=TMatr.getAMatrB(n,dpmom,dpmom,IQQt,n)-(dp_o*dp_o/o_o);
    this.di_di0=TMatr.getAMatrB(n,Dimom,Dimom,IQQt,n)-(di_o*di_o/o_o);
    this.P0=dp_di0/Math.sqrt(dp_dp0*di_di0);

    this.osp0=-(TMatr.getAMatrB(n,dpmom,imom,IQQt,n)-(dp_o*i_o/o_o))
	/Math.sqrt(this.dp_dp0*(TMatr.getAMatrB(n,imom,imom,IQQt,n)-(i_o*i_o/o_o)));

    this.tDimom=getDIMoms(n,imom,tmom,access,x0,rho_x0);

    this.imom=imom;
    this.tmom=tmom;
    this.dpmom=dpmom;

    //System.err.println("tDimom="+java.util.Arrays.toString(tDimom)+"\nDimom="+java.util.Arrays.toString(Dimom)+"\nI0="+I0);
    // for unit test
    tQQd=access.getQQ_Matrs(n,1,new double [][]{tDimom})[0];
    this.tdp_di=TMatr.getAMatrB(n,dpmom,tDimom,IQQt,n);
    this.tdp_di0=tdp_di-(dp_o*TMatr.mulVVn1(ok,tDimom)/o_o);
    this.tdi_di=TMatr.getAMatrB(n,tDimom,tDimom,IQQt,n);
    this.tdi_di0=tdi_di-(Math.pow(TMatr.mulVVn1(ok,tDimom),2)/o_o);
    this.tdp_dp=TMatr.getAMatrB(n,dpmom,dpmom,IQQt,n);
    this.tP=this.tdp_di/Math.sqrt(this.tdi_di*this.tdp_dp);
}

static void matchQQDX(final QQDX a,final QQDX b,final double eps)
{
    EVXData.matchEVXData(a.IT,b.IT,false,eps);
    EVXData.matchEVXData(a.DIT,b.DIT,false,eps);
    UnitTests.checkDoubleRel(a.o_o,"a.o_o",b.o_o,"b.o_o",eps);
    UnitTests.checkDoubleRel(a.I0,"a.I0",b.I0,"b.I0",eps);
    UnitTests.checkDoubleRel(a.x0,"a.x0",b.x0,"b.x0",eps);
    UnitTests.checkDoubleRel(a.dp_di0,"a.dp_di0",b.dp_di0,"b.dp_di0",eps);
    UnitTests.checkDoubleRel(a.dp_dp0,"a.dp_dp0",b.dp_dp0,"b.dp_dp0",eps);
    UnitTests.checkDoubleRel(a.di_di0,"a.di_di0",b.di_di0,"b.di_di0",eps);
    UnitTests.checkDoubleRel(a.tdp_di,"a.tdp_di",b.tdp_di,"b.tdp_di",eps);
    UnitTests.checkDoubleRel(a.tdp_dp,"a.tdp_dp",b.tdp_dp,"b.tdp_dp",eps);
    UnitTests.checkDoubleRel(a.tdi_di,"a.tdi_di",b.tdi_di,"b.tdi_di",eps);
    UnitTests.checkDoubleRel(a.tdp_di0,"a.tdp_di0",b.tdp_di0,"b.tdp_di0",eps);
    UnitTests.checkDoubleRel(a.tdi_di0,"a.tdi_di0",b.tdi_di0,"b.tdi_di0",eps);

    for(QQDX x:new QQDX[]{a,b})
    {
	UnitTests.checkDoubleArrayRel(x.QQd,"x.QQd",x.tQQd,"x.tQQd",eps);
	UnitTests.checkDoubleRel(x.dp_di0,"x.dp_di0",x.tdp_di0,"x.tdp_di0",eps);
	UnitTests.checkDoubleRel(x.di_di0,"x.di_di0",x.tdi_di0,"x.tdi_di0",eps);

	/*
	final double [] QQd0=new QQDX(x.IT.neq,TMatr.getVectorScaled(x.tmom,2.73),x.tmom,x.dpmom,x.access,x.x0,x.rho_x0).QQd;
	final double [] QQ0=new double [QQd0.length];
	UnitTests.checkDoubleArrayRel(QQd0,"QQd0",QQ0,"QQ0",eps);
	*/
    }

    System.err.println("QQDX:matchQQDX test OK neq="+a.IT.neq+" eps="+eps);
}


}// END OF class QQDX



EVXData getEX(final int n,final double [] tmom,final double [] dEmom,final double [] dAmom)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{tmom,dAmom});
    final double [] QQt=QQ[0],QQa=QQ[1],
	IQQa=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQa);
    final double []
	dEmomx=TMatr.getMatr_x_A(n,QQt,TMatr.getMatr_x_A(n,IQQa,java.util.Arrays.copyOf(dEmom,n))),
	Emomx=new double [n];
    access.setFMoments(Emomx,dEmomx,n);
    return DeprecatedHelperEVXData.getEVXData((n+1)/2,Emomx,tmom,access,getX0());
}


static class DPDVm {
    final double [] ev;
    final double lambda,lambdamin,dp;
    final double mP0,mP1,mP2,mP2_test;
    final boolean flag_max;
    DPDVm(final double mP0,
	  final double mP1,
	  final double mP2,
	  final double [] ev,
	  final double mP2_test,
	  final boolean flag_max)
    {
	this.lambda=mP2;
	this.dp=-mP1/mP0;
	this.lambdamin=this.lambda+dp*(2*mP1+dp*mP0);
	this.ev=ev;

	this.mP0=mP0;
	this.mP1=mP1;
	this.mP2=mP2;
	this.mP2_test=mP2_test;
	this.flag_max=flag_max;
    }
    static DPDVm getDPDVm(final int n,final double [] Vmom,final double [] P0mom,final double [] P1mom,final double [] P2mom,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final boolean flag_max)

    {
	final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{Vmom,P0mom,P1mom,P2mom});
	final double [] QQv=QQ[0],QQp0=QQ[1],QQp1=QQ[2],QQp2=QQ[3];
	final double [] ev=DeprecatedHelperEVXData.solveGeneralizedEV(n,QQp2,QQv);
	final int ind=flag_max?(n-1):0;
	final double mP2=ev[ind],
	    mP0=TMatr.getAMatrB(n,ev,(ind)*n+n,ev,(ind)*n+n,QQp0,n),
	    mP1=TMatr.getAMatrB(n,ev,(ind)*n+n,ev,(ind)*n+n,QQp1,n),
	    mP2_test=TMatr.getAMatrB(n,ev,(ind)*n+n,ev,(ind)*n+n,QQp2,n);
	return new DPDVm(mP0,mP1,mP2,ev,mP2_test,flag_max);
    }

static void test_EVXData(final DPDVm q,final EVXData s,final double eps)
{
    final int n=s.neq;
    UnitTests.checkDoubleRel(q.mP2,"q.mP2",q.mP2_test,"q.mP2_test",eps);
    UnitTests.checkDoubleRel(q.mP2,"q.mP2",q.flag_max?s.sH:s.sL,q.flag_max?"s.sH":"s.sL",eps);
    final double [] psi=new double [n];
    System.arraycopy(q.ev,n+n*(q.flag_max?(n-1):0),psi,0,n);
    UnitTests.checkDoubleArrayRel(psi,"psi",EVXData.getMSign(q.flag_max?s.psiH:s.psiL,psi[psi.length-1]),"psiH/L",eps);
    System.err.println("q.lambda="+q.lambda+" q.lambdamin="+q.lambdamin);
    System.err.println("n="+n+" q.ev="+java.util.Arrays.toString(java.util.Arrays.copyOf(q.ev,n)));
    if(q.lambda<q.lambdamin)
    {
	throw new RuntimeException("q.lambda="+q.lambda+" q.lambdamin="+q.lambdamin+" n="+n);
    }
    System.err.println("DPDVm:test_EVXData test OK n="+n+" eps="+eps);
}
} // END OF class DPDVm

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getX0();
abstract double getPOffset();


static abstract class BiQVMTest<DTYPE> extends QTest {

abstract BiQVM<DTYPE> getBiQVM();
abstract BQVM<DTYPE> getBQVM();
abstract ZVQVM<DTYPE> getZVQVM();
abstract VQVM<DTYPE> getVQVM();
abstract LQVM<DTYPE> getLQVM();
abstract double [][] getMonomialMoments(final DTYPE o);
abstract DeprecatedHelperDataDirectAccessWithBasis<?> getPolAccess();

abstract double [] getF_from_dF_byParts(final double [] dF);


BiQVMTest(final String name,final java.util.Random r,final double eps)
{
    super(name,r,eps);
}
void print_Diag()
{
    final BiQVM<DTYPE> q=getBiQVM();
    final double [][] QQ=q.access.getQQ_Matrs(neq,1,new double [][]{q.stmom,q.simom,q.sdpmom});
    final double [] IQQv=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQ[1]),
	QQt=QQ[0],QQdp=QQ[2];
    final double [] M=TMatr.mulMMM(neq,QQdp,IQQv,QQt);
    System.err.println("n="+neq+"\tDpGv^{-1}Gt="+java.util.Arrays.toString(M));
    System.err.println("n="+neq+"\npsi0_I_psi0="+q.I.s0+"\tpsiH_dI_psiH="+q.psiH_dI_psiH);
}

void test_setFDF()
{
    final BiQVM<DTYPE> q=getBiQVM();
    final int nmoms=q.stmom.length;
    final double [] v1=new double [nmoms],v2=new double [nmoms];
    for(double [] v:new double [][]{q.stmom,q.sdpmom,q.simom,q.sVmom,q.sPmom,q.sP2mom})
    {
	q.access.setFMoments(v1,v,nmoms);
	// historivcally DF has opposite sign
	q.access.setDFMoments(v2,v1,nmoms);
	UnitTests.checkDoubleArrayRel(v,"v",TMatr.getVectorScaled(v2,-1),"-v2",eps);
	q.access.setDFMoments(v1,v,nmoms);
	q.access.setFMoments(v2,v1,nmoms);
	UnitTests.checkDoubleArrayRel(v,"v",TMatr.getVectorScaled(v2,-1),"-v2",eps);
    }
    System.err.println(name+":BiQVMTest:test_setFDF(): test OK neq="+neq+" eps="+eps);
}

void test_getDQQ()
{
    final BiQVM<DTYPE> q=getBiQVM();
    final LQVM<DTYPE> l=getLQVM();
    final double [] DQQ=q.getDQQ_Matrs(neq,1,new double [][] {q.stmom})[0];
    for(double [] psi: new double [][] {q.I.psiL,q.I.psiH})
    {
	final double a=TMatr.getAMatrA(neq,psi,DQQ,neq),
	    b=Math.pow(q.access.fx(psi,q.getX0()),2)-1;
	final double [] psio=q.getPsiORemoved(neq,psi,q.sQQt);
	UnitTests.checkDoubleRel(q.access.fx(psio,q.getX0()),"psio(0)",0,"0",eps);
	final double psi_psi=TMatr.getAMatrA(psi,q.sQQt,neq);
	UnitTests.checkDoubleRel(psi_psi,"psi_psi",1,"1",eps);
	final double psio_psio=TMatr.getAMatrA(psio,q.sQQt,neq);
	UnitTests.checkDoubleRel(psio_psio,"psio_psio",1-Math.pow(q.access.fx(psi,q.getX0()),2)*l.wttnodes[0],"1-psi(0)^2*wtt",eps);
	final double pp=q.getPsiDQQPsi(neq,psi,q.stmom,q.sQQt);
	UnitTests.checkDoubleRel(psio_psio,"psio_psio",-pp,"-pp",eps);
    }
    System.err.println(name+":BiQVMTest:test_getDQQ(): test OK neq="+neq+" eps="+eps);
}


void test_Extra(){}

void test_Common()
{
    final BiQVM<DTYPE> q=getBiQVM();
    DPDVm.test_EVXData(q.D,q.P2V,eps);
    DPDVm.test_EVXData(q.d,q.p2V,eps);
    DPDVm.test_EVXData(q.Dm,q.P2V,eps);
    DPDVm.test_EVXData(q.dm,q.p2V,eps);

    // diag
    System.err.println("n="+neq+" q.Vt.ev="+java.util.Arrays.toString(q.Vt.getEValues()));
    System.err.println("n="+neq+" q.P2t.ev="+java.util.Arrays.toString(q.P2t.getEValues()));
    System.err.println("n="+neq+" q.p2t.ev="+java.util.Arrays.toString(q.p2t.getEValues()));
    System.err.println("n="+neq+" q.Pt.ev="+java.util.Arrays.toString(q.Pt.getEValues()));
    System.err.println("n="+neq+" q.pt.ev="+java.util.Arrays.toString(q.pt.getEValues()));


    UnitTests.checkDoubleRel(q.dxt.s0,"q.dxt.s0",0,"0",eps);

    EVXData.matchEVXData(q.J.IT,q.I,true,eps);
    UnitTests.checkDoubleRel(q.J.I0,"q.J.I0",q.J.IT.s0,"q.J.IT.s0",eps);
    final double [] QQd0=new QQDX(neq,TMatr.getVectorScaled(q.stmom,2+r.nextDouble()),q.stmom,q.sdpmom,q.access,q.getX0(),q.getRho(q.getX0())).QQd;
    System.err.println("n="+neq+" test Jconst="+java.util.Arrays.toString(QQd0));
    final double [] QQ0=new double [neq*neq];
    UnitTests.checkDoubleArrayRel(QQd0,"QQd0",QQ0,"QQ0",eps);
    UnitTests.checkDoubleArrayRel(q.J.QQd,"q.J.QQd",q.J.tQQd,"q.J.tQQd",eps);
    UnitTests.checkDoubleRel(q.J.dp_di0,"q.J.dp_di0",q.J.tdp_di0,"q.J.tdp_di0",eps);
    UnitTests.checkDoubleRel(q.J.di_di0,"q.J.di_di0",q.J.tdi_di0,"q.J.tdi_di0",eps);

    UnitTests.checkDoubleRel(Math.exp(q.rv.F0),"exp(q.rv.F0)",q.I.s0,"q.I.s0",eps);
    UnitTests.checkDoubleRel(q.rv_vs_x0,"q.rv_vs_x0",q.psi0_vs_psi0,"q.psi0_vs_psi0",eps);

    UnitTests.checkDoubleRel(q.rv_DdpI_x0,"q.rv_DdpI_x0",q.t0_DdpI_0,"q.t0_DdpI_0",eps);
    UnitTests.checkDoubleRel(q.rv_DvsI_x0,"q.rv_DvsI_x0",q.t0_DvsI_0,"q.t0_DvsI_0",eps);

    System.err.println(name+":BiQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}
abstract double [] getX0ToTest();
void test_QQDX()
{
    final BiQVM<DTYPE> q=getBiQVM();

    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] tmom=amoms[0],imom=amoms[1],dpmom=amoms[6];
    final DeprecatedHelperDataDirectAccessWithBasis<?> paccess=getPolAccess();
    final double [] x0s=getX0ToTest();
    for(int k=0;k<=x0s.length;k++)
    {
	final double x0=k<=0?q.getX0():x0s[k-1];
	final double rho0=q.getRho(x0);
	final QQDX
	    a=new QQDX(neq,q.J.imom,q.J.tmom,q.J.dpmom,q.access,x0,rho0),
	    b=new QQDX(neq,imom,tmom,dpmom,paccess,x0,rho0);
	QQDX.matchQQDX(a,b,eps);
    }

    System.err.println(name+":BiQVMTest:test_QQDX(): test OK neq="+neq+" eps="+eps);
}

void test_EX(final double [] tmom,final double [] dEmom,final double [] dAmom,
	     final DeprecatedHelperDataDirectAccessWithBasis<?> paccess,final double x0,final EVXData basis_Ex)
{
    final double [] QQMa=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,dAmom,1),
	QQMt=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,tmom,1),
	IQQMa=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQMa);
    final double [] dEmomx=TMatr.getMatr_x_A(neq,TMatr.mulMM(neq,QQMt,IQQMa),java.util.Arrays.copyOf(dEmom,neq)),
	Emomx=getF_from_dF_byParts(dEmomx);
    final EVXData Ex=DeprecatedHelperEVXData.getEVXData((neq+1)/2,Emomx,tmom,paccess,x0);
    EVXData.matchEVXData(basis_Ex,Ex,false,eps);
    System.err.println(name+":BiQVMTest:test_EX(): test OK neq="+neq+" eps="+eps);
}

void test_Monomials()
{
    final BiQVM<DTYPE> q=getBiQVM();

    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] tmom=amoms[0],imom=amoms[1],VolMom=amoms[2],Vmom=amoms[3],Pmom=amoms[4],P2mom=amoms[5],dpmom=amoms[6],dAmom=amoms[7],dRmom=amoms[8];

    UnitTests.checkDoubleArrayRel(getMonomialMomentsConvertedFromBasis(q.sVmom),"conv(q.sVmom)",Vmom,"Vmom",eps);
    UnitTests.checkDoubleArrayRel(getMonomialMomentsConvertedFromBasis(q.sPmom),"conv(q.sPmom)",Pmom,"Pmom",eps);
    UnitTests.checkDoubleArrayRel(getMonomialMomentsConvertedFromBasis(q.sP2mom),"conv(q.sP2mom)",P2mom,"P2mom",eps);

    System.err.println("VolMom="+java.util.Arrays.toString(VolMom));
    System.err.println("Vmom="+java.util.Arrays.toString(Vmom));
    System.err.println("conv(Vmom)="+java.util.Arrays.toString(getMonomialMomentsConvertedFromBasis(q.sVmom)));

    final DeprecatedHelperDataDirectAccessWithBasis<?> paccess=getPolAccess();
    final EVXData Vt=DeprecatedHelperEVXData.getEVXData(neq,Vmom,tmom,paccess,q.getX0()),
	Vol_t=DeprecatedHelperEVXData.getEVXData(neq,VolMom,tmom,paccess,q.getX0());
    EVXData.matchEVXData(q.Vt,Vt,false,eps);

    System.err.println("n="+neq+" MONOM:Vol_t.ev="+java.util.Arrays.toString(Vol_t.getEValues()));
    System.err.println("n="+neq+" MONOM:Vt.ev="+java.util.Arrays.toString(Vt.getEValues()));

    test_EX(tmom,dpmom,imom,paccess,q.getX0(),q.Cx);
    test_EX(tmom,dRmom,imom,paccess,q.getX0(),q.Rx);

    System.err.println(name+":BiQVMTest:test_Monomials(): test OK neq="+neq+" eps="+eps);
}

void test_BQVM()
{
    final BiQVM<DTYPE> q=getBiQVM();
    final BQVM<DTYPE> s=getBQVM();

    EVXData.matchEVXData(q.I,s.IT,eps);
    EVXData.matchEVXData(q.pI,s.PVOL,eps);
    EVXData.matchEVXData(q.dVSt,s.VS,eps);

    UnitTests.checkDoubleRel(q.pt0,"q.pt0",s.ppt0,"s.ppt0",eps);
    UnitTests.checkDoubleRel(q.ptH,"q.ptH",s.pptH,"s.pptH",eps);
    UnitTests.checkDoubleRel(q.pv0,"q.pv0",s.ppv0,"s.ppv0",eps);
    UnitTests.checkDoubleRel(q.pvH,"q.pvH",s.ppvH,"s.ppvH",eps);

    UnitTests.checkDoubleRel(q.c_dp_i_x0,"q.c_dp_i_x0",s.c_dp_i_x0,"s.c_dp_i_x0",eps);

    System.err.println(name+":BiQVMTest:test_BQVM(): test OK neq="+neq+" eps="+eps);
}

void test_LQVM()
{
    final BiQVM<DTYPE> q=getBiQVM();
    final LQVM<DTYPE> s=getLQVM();

    UnitTests.checkDoubleRel(q.J.dp_dp0,"q.J.dp_dp0",s.sigma2_dpt0,"s.sigma2_dpt0",eps);

    System.err.println(name+":BiQVMTest:test_LQVM(): test OK neq="+neq+" eps="+eps);
}

void test_ZVQVM()
{
    final BiQVM<DTYPE> q=getBiQVM();
    final ZVQVM<DTYPE> s=getZVQVM();

    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,q.access.getQQ_Matrs(neq,1,new double [][]{q.stmom})[0]);
    final double dc0=q.access.fx(TMatr.getMatr_x_A(neq,IQQt,java.util.Arrays.copyOf(q.sdCmomx,neq)),q.getX0());
    UnitTests.checkDoubleRel(dc0,"dc0",s.ovdp0*Math.sqrt(s.K00),"s.ovdp0*sqrt(s.K00)",eps);
    //UnitTests.checkDoubleRel(-q.Cx0,"-q.Cx0",s.ovdp0*Math.sqrt(s.K00),"s.ovdp0*sqrt(s.K00)",eps);

    System.err.println(name+":BiQVMTest:test_ZVQVM(): test OK neq="+neq+" eps="+eps);
}


void test_VQVM()
{
    final BiQVM<DTYPE> q=getBiQVM();
    final VQVM<DTYPE> s=getVQVM();

    UnitTests.checkDoubleRel(q.sigma2_i_0,"q.sigma2_i_0",s.sigma2_i_psi,"s.sigma2_i_psi",eps);
    UnitTests.checkDoubleRel(q.sigma2_dp_0,"q.sigma2_dp_0",s.sigma2_dp0_psi,"s.sigma2_dp0_psi",eps);
    UnitTests.checkDoubleRel(q.sigma2_vs_0,"q.sigma2_vs_0",s.sigma2_vs_psi,"s.sigma2_vs_psi",eps);


    System.err.println(name+":BiQVMTest:test_VQVM(): test OK neq="+neq+" eps="+eps);
}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":BiQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Extra();
    test_Monomials();
    test_getDQQ();
    test_QQDX();
    test_BQVM();
    test_LQVM();
    test_ZVQVM();
    test_VQVM();
    test_setFDF();
    print_Diag();

    System.err.println(name+":BiQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":BiQVMTest: starting tests; ver=$Id: BiQVM.java,v 1.160 2022/08/08 20:10:45 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":BiQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: BiQVM.java,v 1.160 2022/08/08 20:10:45 mal Exp $");
}
} // END OF class BiQVMTest

}// END OF class BiQVM
