package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.LogPolynomialsRatio;
import com.polytechnik.outils.*;
import com.polytechnik.realtimebook.*;

public class BiQVMLaguerre extends BiQVM<QVMDataL> {

static final Laguerre B=new Laguerre();

BiQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static BiQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final BiQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new BiQVMLaguerre(neq,o);
}
double getRho(final double x){return Math.exp(-x);}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
int setDF(final int n,final double [] res,final double [] f)
{
    if(f.length!=n) throw new IllegalArgumentException();
    final double [] d=B.getDifferentiated(f);
    for(int k=n-1;--k>=0;) res[k]=-d[k];
    res[n-1]=0;
    return n-1;
}
TwoCloseRootsFinder.RootsData getTR(final double [] kv,final double [] kt)
{
    final double x0=getX0();
    if(kv.length!=kt.length) throw new IllegalArgumentException("kv.length="+kv.length+"!=kt.length="+kt.length);
    return TwoCloseRootsFinder.findRootsData(x0,2.1,3000,new LogPolynomialsRatio(kv,kt,getABasis().B));
}
static class BiQVMLaguerreTest extends BiQVMTest<QVMDataL> {
BiQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
double [] getX0ToTest(){return new double []{0.05,0.1,-0.5,-0.1};}
DeprecatedHelperDataDirectAccessWithBasis<?> getPolAccess()
{
    return new DeprecatedHelperQVMDataDirectAccessWithBasis(){
	public int setDFMoments(final double [] dFmoments,final double [] Fmoments,final int n)
	{
	    for(int k=0;k<n;k++){dFmoments[k]=-((k<=0?0:k*Fmoments[k-1])-Fmoments[k]);}
	    return n;
	}
    };
}
long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
BiQVMLaguerre getBiQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
BQVMLaguerre getBQVM(){return BQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
LQVMLaguerre getLQVM(){return RadauLQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
ZVQVMLaguerre getZVQVM(){return ZVQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
VQVMLaguerre getVQVM(){return VQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData getPData(final int nmoms)
{
    final QVMData qdata=QVMData.calculateQVMData(2,nmoms,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
    return qdata;
}
double [] getF_from_dF_byParts(final double [] dF)
{
    final int nmoms=dF.length;
    final double [] F=new double [nmoms], t=new double [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	// Prudnikov 1.3.2.6, page 113, vol 1, 2002.
	double s=1;
	for(int j=k;j>=0;j--){t[j]=s;s*=j;}
	F[k]=0-TMatr.mulVVn(k+1,t,dF);
    }
    return F;
}
double [][] getMonomialMoments(final QVMDataL odata)
{
    final int nmoms=2*neq-1;
    final QVMData qd=getPData(nmoms);
    final double []
	imoms=MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,odata.p_offset),
	dpmoms=MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_DP,odata.p_offset),
	dp2moms=MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_DP,odata.p_offset);

    return new double [][]{
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomT(nmoms),
	imoms,
	TMatr.getVectorScaled(MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,qd.npm*qd.ntm,QVMData.MOM_dT,odata.p_offset),-qd.v_scale),
	TMatr.getVectorScaled(getF_from_dF_byParts(imoms),-1),
	getF_from_dF_byParts(dpmoms),
	getF_from_dF_byParts(dp2moms),
	dpmoms,
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_A,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_RTS,odata.p_offset)
    };
}


void test_Extra()
{
    final BiQVMLaguerre q=getBiQVM();
    final QVMLL s=QVMLL.calculateQVMLL(neq,currenttime,tau,tb,tb.length,null,null);
    UnitTests.checkDoubleRel(q.J.osp0,"q.J.osp0",s.osp,"s.osp",eps);

    System.err.println(name+":BiQVMTest:test_Extra(): test OK neq="+neq+" eps="+eps);
}

} // END of class BiQVMLaguerreTest

public static void main(String [] args)
{
    new BiQVMLaguerreTest("BiQVMLaguerre").runTests(5);
} // END main



} // END OF class BiQVMLaguerre
