package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.LogPolynomialsRatio;
import com.polytechnik.outils.*;
import com.polytechnik.realtimebook.*;

public class BiQVMLegendreShifted extends BiQVM<QVMDataP> {

static final LegendreShifted B=new LegendreShifted();

BiQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static BiQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final BiQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new BiQVMLegendreShifted(neq,o);
}
double getRho(final double x){return x;}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
double getX0(){return 1;}
double getPOffset(){return odata.p_offset;}
int setDF(final int n,final double [] res,final double [] f)
{
    if(f.length!=n||res.length!=n) throw new IllegalArgumentException();
    System.arraycopy(B.getXDiffF(f),0,res,0,n);
    return n;
}
TwoCloseRootsFinder.RootsData getTR(final double [] kv,final double [] kt)
{
    final double x0=getX0();
    if(kv.length!=kt.length) throw new IllegalArgumentException("kv.length="+kv.length+"!=kt.length="+kt.length);
    return TwoCloseRootsFinder.findRootsData(x0,0.5,3000,new LogPolynomialsRatio(kv,kt,getABasis().B));
}
static class BiQVMLegendreShiftedTest extends BiQVMTest<QVMDataP> {
BiQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
double [] getX0ToTest(){return new double []{0.98,1.1,1.3,0.9};}
DeprecatedHelperDataDirectAccessWithBasis<?> getPolAccess()
{
    return new DeprecatedHelperQVMDataDirectAccessWithBasis(){
	public int setDFMoments(final double [] dFmoments,final double [] Fmoments,final int n)
	{
	    for(int k=0;k<n;k++){dFmoments[k]=(k+1)*Fmoments[k];}
	    return n;
	}
    };
}
long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
BiQVMLegendreShifted getBiQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
BQVMLegendreShifted getBQVM(){return BQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
LQVMLegendreShifted getLQVM(){return RadauLQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
ZVQVMLegendreShifted getZVQVM(){return ZVQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
VQVMLegendreShifted getVQVM(){return VQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData [] getPData(final int nmoms)
{
    final QVMData [] qdata=new QVMData [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(2,1+r.nextInt(3),4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}
double [] getF_from_dF_byParts(final double [] dF)
{
    final int nmoms=dF.length;
    final double [] F=new double [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	F[k]=0-dF[k]/(1+k);
    }
    return F;
}

double [][] getMonomialMoments(final QVMDataP odata)
{
    final int nmoms=2*neq-1;
    final QVMData [] qd=getPData(nmoms);
    final double [] mv=new double [nmoms],
	imoms=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,false,odata.p_offset),
	dpmoms=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_DP,false,odata.p_offset),
	dp2moms=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_DP,false,odata.p_offset);
    for(int k=0;k<nmoms;k++)
    {
	final QVMData qq=qd[k];
	final double [] m=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(1,new QVMData[]{qq},qq.ntm*qq.npm,QVMData.MOM_dT,false,odata.p_offset);
	mv[k]=m[0]*(-qq.v_scale)*(qq.tau/((double) qd[0].tau));
    }
    return new double [][]{
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomT(nmoms),
	imoms,
	mv,
	TMatr.getVectorScaled(getF_from_dF_byParts(imoms),-1),
	getF_from_dF_byParts(dpmoms),
	getF_from_dF_byParts(dp2moms),
	dpmoms,
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_A,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_RTS,true,odata.p_offset)
    };
}


void test_Extra()
{
    final BiQVMLegendreShifted q=getBiQVM();
    final QVMPP s=QVMPP.calculateQVMPP(neq,currenttime,tau,tb,tb.length,null,null);
    UnitTests.checkDoubleRel(q.J.osp0,"q.J.osp0",s.osp,"s.osp",eps);

    System.err.println(name+":BiQVMTest:test_Extra(): test OK neq="+neq+" eps="+eps);
}



} // END of class BiQVMLegendreShiftedTest

public static void main(String [] args)
{
    new BiQVMLegendreShiftedTest("BiQVMLegendreShifted").runTests(5);
} // END main



} // END OF class BiQVMLegendreShifted
