package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Quadrature;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.realtimebook.*;

/** BQVM and Rurik measure (continued). t &amp; v operators.
 *  @version $Id: BjQVM.java,v 1.52 2022/08/08 20:10:45 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class BjQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;

/** Rurik evolution. */
final EVXData I,A,Vt,Ti,Tt,Ta,plusDpt,minusDpt,Isell,Ibuy,P2i,P2t,P2Ri,P2Rt,P2V;
final double v_dp_I,v_dp_dp,v_I_I,cv_dp_I,ckv_dp_I,cv_vs_I,ckv_vs_I,
v_vs_I,v_rvs_I,norm_psii,x_I,x_Tt,dp_T,dp_Tt,norm_psit;
final EVXData RI,RItau,RtI,RTi,RTt,RIsell,RIbuy,RplusDpt,RminusDpt;

final double ca_dp_a,norm_psia,c_a,x_Ta;
final double x_Isell,w_Isell,x_Ibuy,w_Ibuy;

final double Rv_dp_I,Rv_vs_I,Rnorm_psii;
final double RtI_dp_I,RtI_vs_I,RtInorm_psii,RtI_x,P2i_x;

BjQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final int nmoms=2*neq-1+2;
    final double [] imom=new double [nmoms],tmom=new double [nmoms],
	vsmom=new double [nmoms],rvsmom=new double [nmoms],amom=new double [nmoms],
	momdpplus=new double [nmoms],momdpminus=new double [nmoms],
	momsell=new double [nmoms],mombuy=new double [nmoms],
	dpmom=new double [nmoms],dp2mom=new double [nmoms];

    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setM(dp2mom,2,access.M_DP,odata,nmoms);
    access.setM(vsmom,0,access.M_VS,odata,nmoms);
    access.setM(rvsmom,0,access.M_RVS,odata,nmoms);
    access.setM(amom,0,access.M_A,odata,nmoms);
    access.setM(momdpplus,1,access.M_DPPLUS,odata,nmoms);
    access.setM(momdpminus,1,access.M_DPMINUS,odata,nmoms);
    for(int j=nmoms;--j>=0;){momdpminus[j]=-momdpminus[j];}
    access.setM(momsell,0,access.M_ISELL,odata,nmoms);
    access.setM(mombuy,0,access.M_IBUY,odata,nmoms);
    smomsell=momsell;
    smombuy=mombuy;
    final double [] Pmom=new double [nmoms],P2mom=new double [nmoms];
    access.setFMoments(Pmom,dpmom,nmoms);
    access.setFMoments(P2mom,dp2mom,nmoms);

    final double [] Vmom=new double [nmoms];
    access.setFMoments(Vmom,imom,nmoms);
    for(int k=nmoms;--k>=0;) {Vmom[k]=-Vmom[k];}
    final double [] Timom=getXMoments(imom),Ttmom=getXMoments(tmom),Tamom=getXMoments(amom);

    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXData(neq,imom,tmom,access,x0);
    this.A=DeprecatedHelperEVXData.getEVXData(neq,amom,imom,access,x0);
    this.Vt=DeprecatedHelperEVXData.getEVXData(neq,Vmom,tmom,access,x0);
    this.Ti=DeprecatedHelperEVXData.getEVXData(neq,Timom,imom,access,x0);
    this.Tt=DeprecatedHelperEVXData.getEVXData(neq,Ttmom,tmom,access,x0);
    this.Ta=DeprecatedHelperEVXData.getEVXData(neq,Tamom,amom,access,x0);
    this.plusDpt=DeprecatedHelperEVXData.getEVXData(neq,getXMoments(momdpplus),momdpplus,access,x0);
    this.minusDpt=DeprecatedHelperEVXData.getEVXData(neq,getXMoments(momdpminus),momdpminus,access,x0);
    Isell=DeprecatedHelperEVXData.getEVXData(neq,getXMoments(momsell),momsell,access,x0);
    Ibuy=DeprecatedHelperEVXData.getEVXData(neq,getXMoments(mombuy),mombuy,access,x0);


    final double [][] QQ=access.getQQ_Matrs(neq,1,new double [][]{imom,tmom,dpmom,vsmom,Timom,Ttmom,rvsmom,amom,Tamom});
    final double [] QQMi=QQ[0],QQMt=QQ[1],QQMdp=QQ[2],QQMvs=QQ[3],QQTi=QQ[4],QQTt=QQ[5],QQMrvs=QQ[6],QQMa=QQ[7],QQTa=QQ[8];
    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQMt),
	IQQi=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQMi),
	IQQa=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQMa),
	QQMidp=TMatr.mulMMM(neq,QQMdp,IQQt,QQMi);
    final double [] tmp=new double [QQMt.length];

    if(true)
    {
	final double [] psi=isX0Right()?Ti.psiH:Ti.psiL;
	this.v_dp_I=TMatr.getAMatrA(psi,QQMidp,neq);
	this.v_vs_I=TMatr.getAMatrA(psi,QQMvs,neq);
	this.v_rvs_I=TMatr.getAMatrA(psi,QQMrvs,neq);
	this.norm_psii=TMatr.getAMatrA(psi,QQMi,neq);
	this.x_I=TMatr.getAMatrA(psi,QQTi,neq);
	this.v_dp_dp=BiQVM.geAM(neq,QQMdp,IQQt,QQMdp,psi);
	this.v_I_I=BiQVM.geAM(neq,QQMi,IQQt,QQMi,psi);
	this.cv_dp_I=BiQVM.getACORR(neq,QQMdp,QQMi,IQQt,psi);
	this.cv_vs_I=TMatr.getAMatrA(psi,QQMvs,neq)/TMatr.getAMatrA(psi,QQMi,neq);

	final double [] KQQdp=ZQVM.getIQQ_F_IQQ(neq,IQQt,QQMdp,tmp),
	    KQQidp=ZQVM.getIQQ_F_IQQ(neq,IQQi,QQMidp,tmp),
	    KQQii=ZQVM.getIQQ_F_IQQ(neq,IQQi,TMatr.mulMMM(neq,QQMi,IQQt,QQMi),tmp),
	    KQQdpdp=ZQVM.getIQQ_F_IQQ(neq,IQQi,TMatr.mulMMM(neq,QQMdp,IQQt,QQMdp),tmp),
	    KQQvs=ZQVM.getIQQ_F_IQQ(neq,IQQi,QQMvs,tmp);

	final double [][] kkk=access.getKK(neq,new double [][] {IQQt,IQQi,KQQidp,KQQii,KQQdpdp,KQQvs});
	final double [] kki=kkk[1],kidp=kkk[2],kii=kkk[3],kdpdp=kkk[4],kvs=kkk[5];
	this.ckv_dp_I=access.fx(kidp,x_I)/Math.sqrt(access.fx(kii,x_I)*access.fx(kdpdp,x_I));
	this.ckv_vs_I=access.fx(kvs,x_I)/access.fx(kki,x_I);
    }
    if(true)
    {
	final double [] psi=isX0Right()?Tt.psiH:Tt.psiL;
	this.x_Tt=TMatr.getAMatrA(psi,QQTt,neq);
	this.dp_Tt=TMatr.getAMatrA(psi,QQMdp,neq);
    }
    if(true)
    {
	final double [] psi=isX0Right()?Vt.psiH:Vt.psiL;
	this.dp_T=TMatr.getAMatrA(psi,QQMdp,neq);
	this.norm_psit=TMatr.getAMatrA(psi,QQMt,neq);
    }

    if(true)
    {
	final double [] psi=isX0Right()?Ta.psiH:Ta.psiL;
	this.x_Ta=TMatr.getAMatrA(psi,QQTa,neq);
	this.ca_dp_a=BiQVM.getACORR(neq,QQMdp,QQMa,IQQa,psi);
	this.c_a=TMatr.getAMatrA(psi,QQMdp,neq);
	this.norm_psia=TMatr.getAMatrA(psi,QQMa,neq);
    }
    sQQt=QQMt;
    sQQi=QQMi;
    sQQa=QQMa;

    if(true)
    {
	final double [] psi=isX0Right()?Isell.psiH:Isell.psiL;
	this.x_Isell=Isell.ev[isX0Right()?psi.length-1:0];
	this.w_Isell=1/Math.pow(access.fx(psi,x_Isell),2);
    }
    if(true)
    {
	final double [] psi=isX0Right()?Ibuy.psiH:Ibuy.psiL;
	this.x_Ibuy=Ibuy.ev[isX0Right()?psi.length-1:0];
	this.w_Ibuy=1/Math.pow(access.fx(psi,x_Ibuy),2);
    }

    final double [] Rtmom=getRMoments(tmom),Rimom=getRMoments(imom),
	Rdpmom=getRMoments(dpmom),Rvsmom=getRMoments(vsmom);
    this.RI=DeprecatedHelperEVXData.getEVXData(neq,Rimom,Rtmom,access,x0);
    this.RItau=DeprecatedHelperEVXData.getEVXData(neq,imom,Rtmom,access,x0);
    this.RtI=DeprecatedHelperEVXData.getEVXData(neq,Rtmom,imom,access,x0);

    this.P2i=DeprecatedHelperEVXData.getEVXData(neq,P2mom,imom,access,x0);
    this.P2V=DeprecatedHelperEVXData.getEVXData(neq,P2mom,Vmom,access,x0);
    this.P2t=DeprecatedHelperEVXData.getEVXData(neq,P2mom,tmom,access,x0);
    this.P2Ri=DeprecatedHelperEVXData.getEVXData(neq,P2mom,Rimom,access,x0);
    this.P2Rt=DeprecatedHelperEVXData.getEVXData(neq,P2mom,Rtmom,access,x0);


    if(true)
    {
	final double [] psi=this.RtI.psiL,RQQi=access.getQQ_Matrs(neq,1,new double [][]{Rimom})[0];
	this.RtI_dp_I=TMatr.getAMatrA(psi,QQMidp,neq);
	this.RtI_vs_I=TMatr.getAMatrA(psi,QQMvs,neq);
	this.RtI_x=TMatr.getAMatrA(psi,RQQi,neq);
	this.RtInorm_psii=TMatr.getAMatrA(psi,QQMi,neq);
	this.P2i_x=TMatr.getAMatrA(P2i.psiL,RQQi,neq);
    }


    RTi=DeprecatedHelperEVXData.getEVXData(neq-1,getRXMoments(imom),Rimom,access,x0);
    RTt=DeprecatedHelperEVXData.getEVXData(neq-1,getRXMoments(tmom),Rtmom,access,x0);
    RIsell=DeprecatedHelperEVXData.getEVXData(neq-1,getRXMoments(momsell),getRMoments(momsell),access,x0);
    RIbuy=DeprecatedHelperEVXData.getEVXData(neq-1,getRXMoments(mombuy),getRMoments(mombuy),access,x0);
    //System.err.println("n="+(neq-1)+" evS="+java.util.Arrays.toString(RIbuy.getEValues()));
    RplusDpt=DeprecatedHelperEVXData.getEVXData(neq-1,getRXMoments(momdpplus),getRMoments(momdpplus),access,x0);
    RminusDpt=DeprecatedHelperEVXData.getEVXData(neq-1,getRXMoments(momdpminus),getRMoments(momdpminus),access,x0);


    final double [][] RQQ=access.getQQ_Matrs(neq-1,1,new double [][]{Rtmom,Rimom,Rdpmom,Rvsmom});
    final double [] RQQMt=RQQ[0],RQQMi=RQQ[1],RQQMdp=RQQ[2],RQQMvs=RQQ[3];
    final double [] IRQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq-1,neq-1,RQQMt),
	RQQMidp=TMatr.mulMMM(neq-1,RQQMdp,IRQQt,RQQMi);
    if(true)
    {
	final double [] psi=isX0Right()?RTi.psiH:RTi.psiL;
	this.Rv_dp_I=TMatr.getAMatrA(psi,RQQMidp,neq-1);
	this.Rv_vs_I=TMatr.getAMatrA(psi,RQQMvs,neq-1);
	this.Rnorm_psii=TMatr.getAMatrA(psi,RQQMi,neq-1);
    }



}

abstract boolean isX0Right();
abstract double getX0();
abstract double [] getXMoments(final double [] moms);
abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double [] getRXMoments(final double [] moms);
abstract double [] getRMoments(final double [] moms);

/** For unit test. */
final double [] sQQt,sQQi,sQQa,smomsell,smombuy;


static abstract class BjQVMTest<DTYPE> extends QTest {

abstract BjQVM<DTYPE> getBjQVM();
abstract BiQVM<DTYPE> getBiQVM();
abstract DiQVM<DTYPE> getDiQVM();
abstract LQVM<DTYPE> getLQVM();

BjQVMTest(final String name,final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_psi(final EVXData q,final double [] QQ,final DeprecatedHelperDataDirectAccessWithBasis<?> access)
{
    for(int k=0;k<q.neq;k++)
    {
	final double [] psi=access.getOK(q.neq,QQ,q.ev[k]),
	    psie=new double [q.neq];
	System.arraycopy(q.ev,q.neq+k*q.neq,psie,0,q.neq);
	final double f=psie[psie.length-1]/psi[psi.length-1];
	for(int j=0;j<q.neq;j++) {psi[j]*=f;}
	UnitTests.checkDoubleArrayRel1(psi,"psi",psie,"psie",eps);
    }

    System.err.println(name+":BjQVMTest:test_psi(): test OK, neq="+q.neq+" eps="+eps);
}


void test_Common()
{
    final BjQVM<DTYPE> q=getBjQVM();

    // diag
    System.err.println("n="+neq+" q.I.ev="+java.util.Arrays.toString(q.I.getEValues()));
    System.err.println("n="+neq+" q.Vt.ev="+java.util.Arrays.toString(q.Vt.getEValues()));
    System.err.println("n="+neq+" q.Ti.ev="+java.util.Arrays.toString(q.Ti.getEValues()));
    System.err.println("n="+neq+" q.Tt.ev="+java.util.Arrays.toString(q.Tt.getEValues()));


    UnitTests.checkDoubleRel(q.RtInorm_psii,"q.RtInorm_psii",1,"1",eps);
    UnitTests.checkDoubleRel(q.Rnorm_psii,"q.Rnorm_psii",1,"1",eps);
    UnitTests.checkDoubleRel(q.norm_psii,"q.norm_psii",1,"1",eps);
    UnitTests.checkDoubleRel(q.norm_psit,"q.norm_psit",1,"1",eps);
    UnitTests.checkDoubleRel(q.norm_psia,"q.norm_psia",1,"1",eps);
    UnitTests.checkDoubleRel(q.x_I,"q.x_I",q.isX0Right()?q.Ti.sH:q.Ti.sL,"q.Ti.s",eps);

    UnitTests.checkDoubleRel(q.ckv_dp_I,"q.ckv_dp_I",q.cv_dp_I,"q.cv_dp_I",eps);
    UnitTests.checkDoubleRel(q.ckv_vs_I,"q.ckv_vs_I",q.cv_vs_I,"q.cv_vs_I",eps);

    test_psi(q.Ti,q.sQQi,q.access);
    test_psi(q.Tt,q.sQQt,q.access);
    test_psi(q.Ta,q.sQQa,q.access);


    System.err.println(name+":BjQVMTest:test_Common(): test OK, {t,i}-nodes test OK, neq="+neq+" eps="+eps);
}

void test_DiQVM()
{
    final BjQVM<DTYPE> q=getBjQVM();
    final DiQVM<DTYPE> s=getDiQVM();

    EVXData.matchEVXData(q.RI,s.RI,eps);
    EVXData.matchEVXData(q.RItau,s.RItau,eps);
    EVXData.matchEVXData(q.Vt,s.RvT_QQt,eps);
    EVXData.matchEVXData(q.Ti,s.RI_QQv,eps);
    EVXData.matchEVXData(q.Tt,s.RtT_QQt,eps);

    System.err.println(name+":BjQVMTest:test_DiQVM(): test OK neq="+neq+" eps="+eps);
}

void test_BiQVM()
{
    final BjQVM<DTYPE> q=getBjQVM();
    final BiQVM<DTYPE> s=getBiQVM();

    EVXData.matchEVXData(q.P2i,s.P2i,eps);
    EVXData.matchEVXData(q.P2t,s.P2t,eps);
    EVXData.matchEVXData(q.P2V,s.P2V,eps);

    System.err.println(name+":BjQVMTest:test_BiQVM(): test OK neq="+neq+" eps="+eps);
}


void test_LQVM()
{
    final BjQVM<DTYPE> q=getBjQVM();
    final LQVM<DTYPE> s=getLQVM();

    final double [] x_t=new double [neq],x_i=new double [neq];
    for(int k=0;k<neq;k++)
    {
	x_t[k]=q.isX0Right()?q.Tt.ev[neq-1-k]:q.Tt.ev[k];
	x_i[k]=q.isX0Right()?q.Ti.ev[neq-1-k]:q.Ti.ev[k];
    }

    UnitTests.checkDoubleArrayRel(x_t,"x_t",s.xtnodes,"s.xtnodes",eps);
    UnitTests.checkDoubleArrayRel(x_i,"x_i",s.xinodes,"s.xinodes",eps);
    UnitTests.checkDoubleRel(q.x_I,"q.x_I",s.xinodes[0],"s.xinodes[0]",eps);
    UnitTests.checkDoubleRel(q.x_Tt,"q.x_Tt",s.xtnodes[0],"s.xtnodes[0]",eps);
    UnitTests.checkDoubleRel(q.dp_Tt,"q.dp_Tt",s.dptnodes2[0],"s.dptnodes2[0]",eps);
    UnitTests.checkDoubleRel(q.v_vs_I,"q.v_vs_I",s.vsinodes2[0],"s.vsinodes2[0]",eps);
    UnitTests.checkDoubleRel(q.v_dp_I,"q.v_dp_I",s.dpinodes2[0],"s.dpinodes2[0]",eps);


    System.err.println(name+":BjQVMTest:test_LQVM(): test OK, q.x_I="+q.x_I+" q.x_Tt="+q.x_Tt+" neq="+neq+" eps="+eps);
}


abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getPBasis();
void tst_xW(final double [] moms,final double qx,final double qw,final double qrx,final boolean flag_right)
{
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> O=getPBasis();
    final double [][] p=O.getOrthogonalPolynomialsFirstKind(neq+1,moms);
    {
    final Quadrature xw=O.getGaussQuadratureFromPolynomials(neq,p);
    final double [] x=xw.x,w=xw.w;
    final int ind=flag_right?neq-1:0;
    UnitTests.checkDoubleRel(qx,"qx",x[ind],"x[ind]",eps);
    UnitTests.checkDoubleRel(qw,"qw",w[ind],"w[ind]",eps);
    }

    {
    final Quadrature xw=O.getRadauQuadratureFromPolynomials(neq-1,p);
    final double [] x=xw.x,w=xw.w;
    // the last element is Radau node. skip it.
    final int ind=flag_right?neq-2:0;
    System.err.println("qrx="+qrx+"\tradau:"+java.util.Arrays.toString(x));
    UnitTests.checkDoubleRel(qrx,"qrx",x[ind],"x[ind]",eps);
    }


    System.err.println(name+":BjQVMTest:tst_xW(): test OK, neq="+neq+" eps="+eps);
}

void test_Pols()
{
    final BjQVM<DTYPE> q=getBjQVM();
    tst_xW(q.smombuy,q.x_Ibuy,q.w_Ibuy,q.isX0Right()?q.RIbuy.sH:q.RIbuy.sL,q.isX0Right());
    tst_xW(q.smomsell,q.x_Isell,q.w_Isell,q.isX0Right()?q.RIsell.sH:q.RIsell.sL,q.isX0Right());
    System.err.println(name+":BjQVMTest:tst_Pols(): test OK, neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":BjQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_DiQVM();
    test_BiQVM();
    test_LQVM();
    test_Pols();

    System.err.println(name+":BjQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":BjQVMTest: starting tests; ver=$Id: BjQVM.java,v 1.52 2022/08/08 20:10:45 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":BjQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: BjQVM.java,v 1.52 2022/08/08 20:10:45 mal Exp $");
}
} // END OF class BjQVMTest

}// END OF class BjQVM
