package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.realtimebook.*;

public class BjQVMLaguerre extends BjQVM<QVMDataL> {

    static final Laguerre B=new Laguerre();

BjQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static BjQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final BjQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1+1+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new BjQVMLaguerre(neq,o);
}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
double getX0(){return 0;}
boolean isX0Right(){return false;}
double [] getXMoments(final double [] moms)
{
    return B.getXQkMomentsFromQkMoments(moms);
}
double [] getRXMoments(final double [] moms)
{
    final double [] mm=new double [moms.length-2];
    for(int k=moms.length-2;--k>=0;)
    {
	final double [] tmp=new double [k+1];tmp[k]=1;
	final double [] tmp2=B.getxP(tmp);
	final double [] tmp3=B.getxP(tmp2);
	if(tmp3.length!=k+3) throw new RuntimeException();
	mm[k]=TMatr.mulVVn1(tmp3,moms);
    }
    return mm;
}
double [] getRMoments(final double [] moms)
{
    final double [] mm=new double [moms.length-1];
    for(int k=moms.length-1;--k>=0;)
    {
	final double [] tmp=new double [k+1];tmp[k]=1;
	final double [] tmp2=B.getxP(tmp);
	mm[k]=TMatr.mulVVn1(tmp2,moms);
    }
    return mm;
}

static class BjQVMLaguerreTest extends BjQVMTest<QVMDataL> {
BjQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
BjQVMLaguerre getBjQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DiQVMLaguerre getDiQVM(){return DiQVMLaguerre.calculate(neq+1,currenttime,tau,tb,tb.length,null);}
BiQVMLaguerre getBiQVM(){return BiQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
LQVMLaguerre getLQVM(){return LQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getPBasis()
{
    return new com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis();
}

} // END of class BjQVMLaguerreTest

public static void main(String [] args)
{
    new BjQVMLaguerreTest("BjQVMLaguerre").runTests(5);
} // END main



} // END OF class BjQVMLaguerre
