package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.realtimebook.*;

public class BjQVMLegendreShifted extends BjQVM<QVMDataP> {

static final LegendreShifted B=new LegendreShifted();

BjQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static BjQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final BjQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1+1+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new BjQVMLegendreShifted(neq,o);
}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
double getX0(){return 1;}
boolean isX0Right(){return true;}
double [] getXMoments(final double [] moms)
{
    return B.getXQkMomentsFromQkMoments(moms);
}

double [] getRXMoments(final double [] moms)
{
    final double [] mm=new double [moms.length-2];
    for(int k=moms.length-2;--k>=0;)
    {
	final double [] tmp=new double [k+1];tmp[k]=1;
	final double [] tmp2=B.get1mxP(tmp);
	final double [] tmp3=B.getxP(tmp2);
	if(tmp3.length!=k+3) throw new RuntimeException();
	mm[k]=TMatr.mulVVn1(tmp3,moms);
    }
    return mm;
}
double [] getRMoments(final double [] moms)
{
    final double [] mm=new double [moms.length-1];
    for(int k=moms.length-1;--k>=0;)
    {
	final double [] tmp=new double [k+1];tmp[k]=1;
	final double [] tmp2=B.get1mxP(tmp);
	mm[k]=TMatr.mulVVn1(tmp2,moms);
    }
    return mm;
}

static class BjQVMLegendreShiftedTest extends BjQVMTest<QVMDataP> {
BjQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(-687311L),1e-4);
}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
BjQVMLegendreShifted getBjQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DiQVMLegendreShifted getDiQVM(){return DiQVMLegendreShifted.calculate(neq+1,currenttime,tau,tb,tb.length,null);}
BiQVMLegendreShifted getBiQVM(){return BiQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
LQVMLegendreShifted getLQVM(){return LQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getPBasis()
{
    return new com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis();
}

} // END of class BjQVMLegendreShiftedTest

public static void main(String [] args)
{
    new BjQVMLegendreShiftedTest("BjQVMLegendreShifted").runTests(5);
} // END main



} // END OF class BjQVMLegendreShifted
