package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.realtimebook.*;

/** Rurik measure is back. To take over the throne.
 *  The money for development is here.
 *  @version $Id: BkQVM.java,v 1.43 2022/07/15 00:01:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class BkQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;

final EVXData I;
final EVRData it_prestolL,it_prestolH;

BkQVM(final int n,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final double [][] _QQ=getQQMatrices(n);
    final double [] _DQQD=getVolMatrix(_QQ[0],n);
    final double [] basis=getBasisCond(n); // basis: nd x n matrix
    final int nd=basis.length/n;
    final double [] QQ[]=new double [1+_QQ.length][];

    // convert all matrices to the basis with proper boundary conditions
    for(int k=0;k<_QQ.length+1;k++)
    {
	final double [] m=k<_QQ.length?_QQ[k]:_DQQD;
	QQ[k]=EVXData.convertMatrFromQtoB_Basis(n,m,basis,nd,0);
	// The result is saved back to m,
	// which is now nd x nd matrix in basis returned by getBasisCond(n).
    }
    final double [] QQt=QQ[0],QQi=QQ[1],QQf0=QQ[2],QQf1=QQ[3],QQf2=QQ[4],DQQD=QQ[QQ.length-1];

    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(nd,QQi,QQt,access,getX0());

    this.it_prestolL=new EVRData(0,this.I,DQQD,QQt,QQf0,QQf1,QQf2,nd);
    this.it_prestolH=new EVRData(nd-1,this.I,DQQD,QQt,QQf0,QQf1,QQf2,nd);
}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double getX0();
abstract double [] getEDPsi(final double [] psi);

double [][] getQQMatrices(final int n)
{
    final int nmoms=2*n-1;

    final double [] tmom=new double [nmoms],imom=new double [nmoms],
	pimom=new double [nmoms],p2imom=new double [nmoms];
    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    access.setM(p2imom,2,access.M_I,odata,nmoms);

    return access.getQQ_Matrs(n,1,new double [][]{tmom,imom,imom,pimom,p2imom});
}

/** Volatility matrix.
 */
double [] getVolMatrix(final double [] QQt,final int n)
{
    final double [] DQQD=new double [n*n],pp=new double [n];
    final double [][] bb=new double [n][];
    for(int k=0;k<n;k++)
    {
	pp[k]=1;final double [] dpsik=bb[k]=getEDPsi(pp);pp[k]=0;
	for(int l=0;l<=k;l++)
	{
	    final double [] dpsil=bb[l];
	    DQQD[n*k+l]=DQQD[n*l+k]=TMatr.getAMatrB(dpsik.length,dpsik,0,dpsil.length,dpsil,0,QQt,n);
	}
    }
    return DQQD;
}

final VolMatrix VOLATILITY_EDPsi=this::getEDPsi;

/** A simple version of basis: basik_k=Q_k - c Q_0,
 *  where c is such that fx(getEDPsi(basis_k),x0)=0.
 */
double [] getBasisCond(final int n)
{
    final int nd=n-1;
    final double [] basis=new double [n*nd],pp=new double [n];
    final double x0=getX0();
    double dq0=Double.NaN;
    for(int k=0,kk=0;k<n;k++)
    {
	pp[k]=1;final double [] dpsik=getEDPsi(pp);pp[k]=0;
	final double dpp=access.fx(dpsik,x0);
	if(k<=0){dq0=dpp;}
	else
	{
	    basis[n*kk+0]=-dpp/dq0;
	    basis[n*kk+k]=1;
	    kk++;
	}
    }
    return basis;
}


static class EVRData {

final double Pf,sigma2b_Pf,sigma2_g,dPf,rho;

EVRData(final int ind_lev,final EVXData I,final double [] DQQD,final double [] QQt,final double [] QQf0,final double [] QQf1,final double [] QQf2,final int ldQQ)
{
    this.Pf=I.getFPsi(ind_lev,QQf1);
    final double [] psipert=DeprecatedHelperEVXData.getPsiPertr(I,ind_lev,DQQD);
    this.dPf=I.getFPsiPsi(ind_lev,psipert,QQf1);
    this.sigma2_g=TMatr.getAMatrA(psipert.length,psipert,QQt,ldQQ);
    this.sigma2b_Pf=I.getFPsi(ind_lev,QQf0)*I.getFPsi(ind_lev,QQf2)-this.Pf*this.Pf;
    this.rho=this.dPf/Math.sqrt(this.sigma2_g*this.sigma2b_Pf);
}

} // END OF class EVRData



static abstract class BkQVMTest<DTYPE> extends QTest {

abstract BkQVM<DTYPE> getBkQVM();
abstract DkQVM<?> getDkQVM();

BkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_Prestol(final String txt,final EVRData q,final DkQVM.WkkBoundary s)
{
    //UnitTests.checkDoubleRel(q.Pf,"q.Pf",s.Pf+getPShift_to_getDkQVM(),"s.Pf",eps);
    UnitTests.checkDoubleRel(q.dPf,"q.dPf",s.dPf,"s.dPf",eps);
    UnitTests.checkDoubleRel(q.sigma2_g,"q.sigma2_g",s.sigma2_g,"s.sigma2_g",eps);
    UnitTests.checkDoubleRel(q.sigma2b_Pf,"q.sigma2b_Pf",s.sigma2b_Pf,"s.sigma2b_Pf",eps);

    System.err.println(name+":BkQVMTest:test_Prestol():"+txt+": test OK neq="+neq+" eps="+eps);
}

void test_DkQVM()
{
    final BkQVM<DTYPE> q=getBkQVM();
    final DkQVM<?> s=getDkQVM();

    test_Prestol("it:L",q.it_prestolL,s.bit_pi_L);
    test_Prestol("it:H",q.it_prestolH,s.bit_pi_H);

    System.err.println(name+":BkQVMTest:test_DkQVM(): test OK neq="+neq+" eps="+eps);
}

void test_DQQD()
{
    final BkQVM<DTYPE> q=getBkQVM();
    final int n=neq;
    for(double [] QQ:q.getQQMatrices(n))
    {
	final double [] d1=q.getVolMatrix(QQ,n),d2=q.VOLATILITY_EDPsi.getVolMatrix(n,QQ);
	UnitTests.checkDoubleArrayRel(d1,"d1",d2,"d2",eps);
    }
    System.err.println(name+":BkQVMTest:test_DQQD(): test OK n="+n+" eps="+eps);
}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":BkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_DkQVM();
    test_DQQD();

    System.err.println(name+":BkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":BkQVMTest: starting tests; ver=$Id: BkQVM.java,v 1.43 2022/07/15 00:01:20 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":BkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: BkQVM.java,v 1.43 2022/07/15 00:01:20 mal Exp $");
}
} // END OF class BkQVMTest

}// END OF class BkQVM
