package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;

/** Book in time/price space.
 *  @version $Id: BoQVM.java,v 1.7 2019/01/06 23:56:13 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class BoQVM<DTYPE> {


final double p_buy,k0_buy,p_sell,k0_sell;
private final double [] Kbuy,Ksell;

final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final DeprecatedHelperDataDirectAccessWithBasis<?> paccess;
private final DTYPE odata_buy,odata_sell;

BoQVM(final int neq,final int neq_p,final DTYPE odata_buy,final DTYPE odata_sell)
{
    this.odata_buy=odata_buy;
    this.odata_sell=odata_sell;
    this.access=getAccess();
    this.paccess=getPAccess();

    final int nmoms=2*neq-1,nmomsp=2*neq_p-1;
    final double [] pmoms_buy=new double [nmomsp],pmoms_sell=new double [nmomsp];
    final double [] tmom=new double [nmoms];
    access.setMT(tmom,nmoms);
    final double [] QQt=access.getQQ_Matrs(neq,1,new double [][]{tmom})[0],
	IQQ=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQt),
	temp=new double [QQt.length],tempmoms=new double [nmoms],
	kk=access.getKK(neq,new double [][] {IQQ})[0];
    final double x0=getX0(),K00=access.fx(kk,x0);
    for(int kp=0;kp<nmomsp;kp++)
    {
	if(true)
	{
	access.setM(tempmoms,kp,access.M_I,this.odata_buy,nmoms);
	final double [] qq=access.getQQ_Matrs(neq,1,new double [][]{tempmoms})[0];
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,temp,qq,neq,IQQ,neq);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,qq,IQQ,neq,temp,neq);
	final double [] kkp=access.getKK(neq,new double [][] {qq})[0];
	pmoms_buy[kp]=access.fx(kkp,x0)/K00;
	}
	if(true)
	{
	access.setM(tempmoms,kp,access.M_I,this.odata_sell,nmoms);
	final double [] qq=access.getQQ_Matrs(neq,1,new double [][]{tempmoms})[0];
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,temp,qq,neq,IQQ,neq);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,qq,IQQ,neq,temp,neq);
	final double [] kkp=access.getKK(neq,new double [][] {qq})[0];
	pmoms_sell[kp]=access.fx(kkp,x0)/K00;
	}
    }

    final double [][] QQ=paccess.getQQ_Matrs(neq_p,1,new double [][]{pmoms_buy,pmoms_sell});
    final double [] iQQbuy=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq_p,neq_p,QQ[0]),
	iQQsell=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq_p,neq_p,QQ[1]);
    final double [][] kkk=paccess.getKK(neq_p,new double [][] {iQQbuy,iQQsell});
    this.Kbuy=kkk[0];
    this.Ksell=kkk[1];

    this.p_buy=getPEdge(this.odata_buy);
    this.p_sell=getPEdge(this.odata_sell);

    final double poffset_buy=getPOffset(this.odata_buy),poffset_sell=getPOffset(this.odata_sell);
    this.k0_buy=paccess.fx(this.Kbuy,p_buy-poffset_buy);
    this.k0_sell=paccess.fx(this.Ksell,p_sell-poffset_sell);
}

abstract double getPOffset(final DTYPE odata);
abstract double getPEdge(final DTYPE odata);
abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double getX0();
DeprecatedHelperDataDirectAccessWithBasis<?> getPAccess() {return new DeprecatedHelperQVMDataBLpDirectAccessWithBasis();}



} // END of class BoQVM
