package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.realtimebook.*;

public class CQVMLaguerre extends CQVM<QVMDataL> {

CQVMLaguerre(final int neq,final QVMDataL [] odata){super(neq,odata);}

public static CQVMLaguerre calculate(final int neq_t,
					    final int neq_i,
					    final long currenttime,
					    final long tau,
					    final Trade [][] tba,
					    final int [] tbasize,
					    final CQVMLaguerre ref)
{
    if(ref!=null && ref.odata.length!=neq_i)
	throw new RuntimeException("Dimension mismatch: neq_i="+neq_i+" ref.odata.length="+ref.odata.length);
    final int npm=3;
    final QVMDataL [] o=new QVMDataL [neq_i];
    for(int k=0;k<neq_i;k++)
    {
	o[k]=QVMDataL.calculateQVMDataL(2*neq_t-1,
					npm,
					currenttime,
					tau,
					tba[k],
					tbasize[k],
					ref!=null?ref.odata[k]:null);
    }
    return new CQVMLaguerre(neq_t,o);
}

DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
double getX0(){return 0;}

static class CQVMLaguerreTest extends CQVMTest<QVMDataL> {
CQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
long getTau(final Trade [] tb)
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
CQVMLaguerre getCQVM(){return calculate(neq,tba.length,currenttime,tau,tba,tba_size,null);}
CQVMLaguerre getCQVM1(){return calculate(neq,1,currenttime,tau,tba,tba_size,null);}
BQVMLaguerre getBQVM(final int ni){return BQVMLaguerre.calculate(neq,currenttime,tau,tba[ni],tba_size[ni],null);}

} // END of class CQVMLaguerreTest

public static void main(String [] args)
{
    new CQVMLaguerreTest("CQVMLaguerre").runTests(5,5);
} // END main



} // END OF class CQVMLaguerre
