package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.realtimebook.*;

/** Unit test for CQVMLaguerre. Uses non-orthogonal monomial basis.
 */
public class CQVMMonomialsT extends CQVM<QVMData> {

CQVMMonomialsT(final int neq,final QVMData [] odata){super(neq,odata);}

public static CQVMMonomialsT calculate(final int neq_t,
				       final int neq_i,
				       final long currenttime,
				       final long tau,
				       final Trade [][] tba,
				       final int [] tbasize,
				       final CQVMMonomialsT ref)
{
    if(ref!=null && ref.odata.length!=neq_i)
	throw new RuntimeException("Dimension mismatch: neq_i="+neq_i+" ref.odata.length="+ref.odata.length);
    final int nvm=1,npm=3;
    final QVMData [] o=new QVMData [neq_i];
    for(int k=0;k<neq_i;k++)
    {
	o[k]=QVMData.calculateQVMData(nvm,
				      2*neq_t-1,
				      npm,
				      currenttime,
				      tau,
				      tba[k],
				      tbasize[k],
				      ref!=null?ref.odata[k]:null);
    }
    return new CQVMMonomialsT(neq_t,o);
}

double [] getOKDiagT(final int n,final double [] QQt)
{
    // use generic code for non-diagonal basis
    return access.getOK(n,QQt,getX0());
}
double [] getSymEVDiagT(final int n,final double [] QQv,final double [] QQt)
{
    // use generic code for non-diagonal basis
    return solveGeneralizedEV(n,QQv,QQt);
}


DeprecatedHelperQVMDataDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataDirectAccessWithBasis();}
double getX0(){return 0;}



static class CQVMMonomialsTTest extends CQVMTest<QVMData> {
CQVMMonomialsTTest(final String name)
{
    super(name,new java.util.Random(21121L),1e-4);
}
long getTau(final Trade [] tb)
{
    return (tb[tb.length-1].time-tb[0].time)/11;
}
CQVMMonomialsT getCQVM(){return calculate(neq,tba.length,currenttime,tau,tba,tba_size,null);}
CQVMMonomialsT getCQVM1(){return calculate(neq,1,currenttime,tau,tba,tba_size,null);}
BQVMLaguerre getBQVM(final int ni){return BQVMLaguerre.calculate(neq,currenttime,tau,tba[ni],tba_size[ni],null);}
void test_Common()
{
    final CQVMMonomialsT q=getCQVM();
    final CQVMLaguerre s=CQVMLaguerre.calculate(neq,tba.length,currenttime,tau,tba,tba_size,null);
    UnitTests.checkDoubleRel(q.K00t,"q.K00t",s.K00t,"s.K00t",eps);
    UnitTests.checkDoubleRel(q.sa0,"q.sa0",s.sa0,"s.sa0",eps);
    UnitTests.checkDoubleRel(Math.abs(q.waL),"q.waL",Math.abs(s.waL),"s.waL",eps);
    UnitTests.checkDoubleRel(q.saL,"q.saL",s.saL,"s.saL",eps);
    UnitTests.checkDoubleRel(Math.abs(q.waH),"q.waH",Math.abs(s.waH),"s.waH",eps);
    UnitTests.checkDoubleRel(q.saH,"q.saH",s.saH,"s.saH",eps);

    System.err.println(name+":CQVMMonomialsTTest:test_Common(): test OK neq="+neq+" neq_i="+tba.length+" eps="+eps);
}

} // END of class CQVMMonomialsTTest

public static void main(String [] args)
{
    new CQVMMonomialsTTest("CQVMMonomialsT").runTests(4,7);
} // END main



} // END OF class CQVMMonomialsT
