package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Critical exponent specific calculations.
 *  @version $Id: CalculateBookCriticalExponent.java,v 1.45 2012/08/21 23:52:55 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateBookCriticalExponent
{

    private double [] prices_sorted,val_fun,val_dlp;
    private long [] vol_sorted;
    private final double [] regr_res=new double [3];

    double beta_crit=Double.NaN,erbeta_crit=Double.NaN;

    private final boolean flag_lower;
    private final Distribution<BookItem> book;
    CalculateBookCriticalExponent(final boolean flag_lower,final Distribution<BookItem> book)
    {
	this.flag_lower=flag_lower;
	this.book=book;
	clearData();
    }


    void clearData()
    {

	//new RuntimeException().printStackTrace(System.err);
	this.beta_crit=this.erbeta_crit=Double.NaN;
	if(this.prices_sorted==null)
	{
	  this.prices_sorted=new double [10];
	  this.vol_sorted=new long [this.prices_sorted.length];
	  this.val_fun=new double [this.prices_sorted.length];
	  this.val_dlp=new double [this.prices_sorted.length];
	}
	for(int i=this.prices_sorted.length;i-->0;)
	{
	    this.prices_sorted[i]=this.val_fun[i]=this.val_dlp[i]=Double.NaN;
	    this.vol_sorted[i]=0;
	}
    }


    void calculateCrit(final long currenttime,
		       final double pcurrent_buy,
		       final double pcurrent_sell,
		       final int n_points_max,
		       final double price_buy_cutoff,
		       final double price_sell_cutoff)
    {
	final int tsize=this.book.getSize();

	final boolean PRINT_DATA_DEBUG=false;
	    //currenttime>=10*3600*TraderInterface.TIME_SCALE && tsize>0;

	final BookItem [] tb=this.book.getBuffer();

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+"============BOOK CRITICAL t=\t"+currenttime+"\t NORDERS=\t"+tsize+
			       "\tp_buy\t"+pcurrent_buy+"\tp_sell\t"+pcurrent_sell+
			       "\tflower\t"+this.flag_lower+
			       "\torders\t"+tsize);
	}

	if(false && PRINT_DATA_DEBUG)
	{
	    System.err.println("num\t"+
			       "time\t"+
			       "shares\t"+
			       "buy_sell_indicator\t"+
			       "price\t");
	    for(int i=0;i<tsize;i++)
	    {
		final BookItem t=tb[i];
		System.err.println(String.valueOf(i)+"\t"+
				   t.time+"\t"+
				   t.shares+"\t"+
				   t.buy_sell_indicator+"\t"+
				   t.price);
		if(i>0)
		{
		    final double prev_price=tb[i-1].price;
		    if((this.flag_lower?prev_price<t.price:prev_price>t.price)||
		       !(t.buy_sell_indicator==(this.flag_lower?BookItem.BOOK_BUY:BookItem.BOOK_SELL))
		       )
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		}
	    }
	    System.err.println("oend\t");
	}

	final double [] prices_s,val_fun_s,val_dlp_s;
	final long [] vol_s;
	if(this.prices_sorted.length<n_points_max)
	{
	    prices_s=this.prices_sorted=new double [n_points_max];
	    vol_s=this.vol_sorted=new long [n_points_max];
	    val_fun_s=this.val_fun=new double [n_points_max];
	    val_dlp_s=this.val_dlp=new double [n_points_max];
	}
	else
	{
	    prices_s=this.prices_sorted;
	    vol_s=this.vol_sorted;
	    val_fun_s=this.val_fun;
	    val_dlp_s=this.val_dlp;
	}

	double prev_price=Double.NaN;
	int np=0;
	long vol=0,volsamep=0,volrprocessed=0;
	int last_ind=-1,nsamep=0,nprocessed=0;
        gloop:for(int i=0;i<tsize;i++)
	{
	    final BookItem t=tb[i];
	    final double p=t.price;
	    // check for poison
	    if(this.flag_lower?(prev_price<p||t.buy_sell_indicator!=BookItem.BOOK_BUY):
	       (prev_price>p||t.buy_sell_indicator!=BookItem.BOOK_SELL))
	    {
		throw new RuntimeException("Ordering is broken");
	    }

	    if(!(this.flag_lower?t.price>=price_buy_cutoff:t.price<=price_sell_cutoff))
	    {
		break gloop;
	    }
	    if(!(prev_price==p) && volsamep>0)
	    {
		volrprocessed+=volsamep;
		nprocessed+=nsamep;
		if(vol!=volrprocessed||(last_ind+1)!=nprocessed)
		{
		    throw new IllegalStateException();
		}
		val_fun_s[np]=getF(val_dlp_s[np]=(this.flag_lower?Math.log(pcurrent_sell-prev_price):Math.log(prev_price-pcurrent_buy)),vol_s[np]=volrprocessed);
		prices_s[np++]=prev_price;
		if(np>=n_points_max)
		{
		    volsamep=0;
		    nsamep=0;
		    break gloop;
		}
		volsamep=t.shares;
		nsamep=1;
	    }
	    else
	    {
		volsamep+=t.shares;
		nsamep++;
	    }
	    prev_price=p;
	    vol+=t.shares;
	    last_ind=i;
	}
	if(volsamep>0)
	{
	    volrprocessed+=volsamep;
	    nprocessed+=nsamep;
	    val_fun_s[np]=getF(val_dlp_s[np]=(this.flag_lower?Math.log(pcurrent_sell-prev_price):Math.log(prev_price-pcurrent_buy)),vol_s[np]=volrprocessed);
	    prices_s[np++]=prev_price;
	    volsamep=0;
	    nsamep=0;
	}
	if(vol!=volrprocessed||(last_ind+1)!=nprocessed)
	{
	    throw new IllegalStateException();
	}


	if((PRINT_DATA_DEBUG || OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS) && tsize>0)
	{
	    int n=0,lasti=-1;
	    long vs=0,vv=0;
	    double pp=tb[0].price;
	    for(int i=0;i<tsize;i++)
	    {
		final BookItem t=tb[i];
		if(!(this.flag_lower?t.price>=price_buy_cutoff:t.price<=price_sell_cutoff))
		{
		    break;
		}
		if(!(t.price!=pp))
		{
		    vs+=t.shares;
		    if(pp!=prices_s[n])
		    {
			throw new IllegalStateException();
		    }
		}
		else
		{
		    if(vs!=(vol_s[n]-(n>0?vol_s[n-1]:0)))
		    {
			throw new IllegalStateException();
		    }
		    pp=t.price;
		    n++;
		    if(n>=n_points_max)
		    {
			vs=0;
			break;
		    }
		    vs=t.shares;
		}
		vv+=t.shares;
		lasti=i;
	    }
	    if(vs>0)
	    {
		if(vs!=(vol_s[n]-(n>0?vol_s[n-1]:0)))
		{
		    throw new IllegalStateException();
		}
		n++;
		vs=0;
	    }
	    if(n!=np||vv!=volrprocessed||vv!=vol||lasti!=last_ind)
	    {
		throw new IllegalStateException();
	    }
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println(" Expensive check OK n="+n+" np="+np+
				   " vv="+vv+" volrprocessed="+volrprocessed+" vol="+vol+
				   " lasti="+lasti+" last_ind="+last_ind+
				   " tsize="+tsize);
	    }
	}


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("pnum\t"+
			       "vol shares\t"+
			       "price\t"+
			       "log dp\t"+
			       "log v\t"+
			       "dv\t");
	    for(int i=0;i<np;i++)
	    {
		System.err.println(String.valueOf(i)+"\t"+
				   vol_s[i]+"\t"+
				   prices_s[i]+"\t"+
				   (this.flag_lower?Math.log(pcurrent_sell-prices_s[i]):Math.log(prices_s[i]-pcurrent_buy))+"\t"+
				   Math.log(vol_s[i])+"\t"+
				   (vol_s[i]-(i>0?vol_s[i-1]:0))+"\t");
	    }
	    System.err.println("osend\tnp\t"+np+
			       "\tvolrprocessed\t"+volrprocessed+
			       "\tn_orders="+nprocessed);
	}


	int ptr_end=np-1;

	if(ptr_end>=0 && np>n_points_max/2)
	{
	    regr2(regr_res,ptr_end+1,val_fun_s,val_dlp_s,vol_s);
	    this.beta_crit=regr_res[0];
	    this.erbeta_crit=regr_res[2];
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("ptr_\t"+ptr_end);
	    System.err.println("f_\t"+(ptr_end<0?Double.NaN:val_fun_s[ptr_end]));
	    System.err.println("dlp_\t"+(ptr_end<0?Double.NaN:val_dlp_s[ptr_end]));
	    System.err.println();
	    System.err.println("vol_\t"+(ptr_end<0?-1:vol_s[ptr_end]));
	    System.err.println("P_\t"+(ptr_end<0?Double.NaN:prices_s[ptr_end]));

	    System.err.println("beta_crit\t"+this.beta_crit);
	    System.err.println("lc\t"+(ptr_end<0?Double.NaN:regr_res[1]));
	    System.err.println("erbeta_crit\t"+this.erbeta_crit);
	}
    }


    private static void regr2(final double [] res,
			      final int n,
			      final double [] y,
			      final double [] x,
			      final long [] vol)
    {
	double xx=0,xy=0,yy=0,ax=0,ay=0,w=0;
	for(int i=0;i<n;i++)
	{
	    final double wi=1.0; // weight
	    final double xi=x[i],yi=y[i];
	    xx+=wi*xi*xi;
	    xy+=wi*xi*yi;
	    yy+=wi*yi*yi;
	    ax+=wi*xi;
	    ay+=wi*yi;
	    w+=wi;
	}



	final double det_r=xx*w-ax*ax;
	final double g=(xy*w-ay*ax)/det_r;
	final double lc=(xx*ay-ax*xy)/det_r;
	final double err_r=(g*g*xx+lc*lc*w+yy+2.0*g*lc*ax-2.0*g*xy-2.0*lc*ay)/((n-1)*w);
	res[0]=g;
	res[1]=lc;
	res[2]=err_r;

    }

    private static final double getF(final double dlp,final long v)
    {
	return Math.log((double)v)-CalculateBookVolumePriceThreshold.BETA*dlp;
    }


    /** Unit test.
     */
    public static void main(String [] args)
    {

	ModelParameters.ModelParametersData mparametersdata=null;
	try {
	    mparametersdata=
		new ModelParameters("/","/",new String [] {"TEST"}).getParameters("TEST");
	} catch(final java.io.IOException e){
	    throw new RuntimeException(e);
	}

	final int T_SAMEP=1900;

	final java.util.Random r=new java.util.Random(-991L);
	final Distribution<BookItem> book=new Distribution<BookItem>();
	final int tsize=mparametersdata.good_price_n_points_max;
	book.replaceBuffer(new BookItem[tsize*T_SAMEP]);
	final CalculateBookCriticalExponent test_buy=
	    new CalculateBookCriticalExponent(true,book);
	final CalculateBookCriticalExponent test_sell=
	    new CalculateBookCriticalExponent(false,book);


	// test fixed gamma
	final double G=1.1,C=Math.exp(22.2);


	// BUY TEST
	test_buy.clearData();
	final double p0=100.0;
	double p=p0,p1=-1;
	double vprev=0;
	for(int i=0;i<tsize;i++)
	{
	    p-=0.05*(1.0+(((double)i)/tsize));
	    if(i<=0)
	    {
		p1=p;
	    }
	    int n=0;
	    for(int j=0;j<T_SAMEP;j++)
	    {
		if(r.nextDouble()<0.5)
		{
		    n++;
		}
	    }
	    final double vsum=(C*Math.pow(p0-p,G));
	    final double dv=vsum-vprev;
	    final double norm=((n+1)*n)/2;
	    for(int j=0;j<n;j++)
	    {
		book.add(new BookItem(p,
				      (int)((j+1)*dv/norm),
				      null,
				      BookItem.BOOK_BUY,
				      i,
				      -1,null));
	    }
	    vprev=vsum;
	}
	test_buy.calculateCrit(
	    10000000L,
	    p1,
	    p0,
	    mparametersdata.good_price_n_points_max,
	    p1/mparametersdata.good_price_book_range_filter_antecedent,
	    p0*mparametersdata.good_price_book_range_filter_antecedent);


	// SELL
	test_sell.clearData();
	book.clearAll();
	p=p0;
	p1=-1;
	vprev=0;
	for(int i=0;i<tsize;i++)
	{
	    p+=0.05*(1.0+(((double)i)/tsize));
	    if(i<=0)
	    {
		p1=p;
	    }
	    int n=0;
	    for(int j=0;j<T_SAMEP;j++)
	    {
		if(r.nextDouble()<0.5)
		{
		    n++;
		}
	    }
	    final double vsum=(C*Math.pow(p-p0,G));
	    final double dv=vsum-vprev;
	    final double norm=((n+1)*n)/2;
	    for(int j=0;j<n;j++)
	    {
		book.add(new BookItem(p,
				      (int)((j+1)*dv/norm),
				      null,
				      BookItem.BOOK_SELL,
				      i,
				      -1,null));
	    }
	    vprev=vsum;
	}
	test_sell.calculateCrit(
	    100000L,
	    p0,
	    p1,
	    mparametersdata.good_price_n_points_max,
	    p0/mparametersdata.good_price_book_range_filter_antecedent,
	    p1*mparametersdata.good_price_book_range_filter_antecedent);
    }
}
