package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit.
 *  @version $Id: CalculateBookLiquidityDeficit.java,v 1.60 2017/09/18 12:35:55 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateBookLiquidityDeficit {

    private final Distribution<BookItem> buy,sell;
    private final Distribution<Trade> trades;

    CalculateBookLiquidityDeficit(final Distribution<BookItem> buy,
				  final Distribution<BookItem> sell,
				  final Distribution<Trade> trades)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
    }

    long deficit=0;
    double expected_price_change=Double.NaN,expected_buy=Double.NaN,expected_sell=Double.NaN;
    int ind_lev_tmin_book_buy=-1,ind_lev_tmin_book_sell=-1;
    long deficit_buy=-1,deficit_sell=-1;
    long t_crit_buy=-1,t_crit_sell=-1,t_tmin_book_buy=-1,t_tmin_book_sell=-1;
    long t_tmin_btrades_buy=-1,t_tmin_btrades_sell=-1;

    void clear()
    {
	this.deficit=0;
	this.expected_price_change=Double.NaN;

	this.expected_buy=this.expected_sell=Double.NaN;
	this.ind_lev_tmin_book_buy=this.ind_lev_tmin_book_sell=-1;
	this.deficit_buy=this.deficit_sell=-1;
	this.t_crit_buy=this.t_crit_sell=this.t_tmin_book_buy=this.t_tmin_book_sell=-1;
	this.t_tmin_btrades_buy=this.t_tmin_btrades_sell=-1;
	// FIXME
    }


    void calculateLiquidityDeficit(final long currenttime,
				   final int n_levels_book,
				   final int n_levels_trades,
				   final double pcurrent_buy,
				   final double pcurrent_sell)
    {
	// init arrays
	final long []
	    vb_buy=new long [n_levels_book],
	    vb_sell=new long [n_levels_book],
	    svb_buy=new long [n_levels_book],
	    svb_sell=new long [n_levels_book],
	    vt_buy=new long [n_levels_trades],
	    vt_sell=new long [n_levels_trades],
	    svt_buy=new long [n_levels_trades],
	    svt_sell=new long [n_levels_trades],
	    timet_levels_buy=new long [n_levels_trades],
	    timet_levels_sell=new long [n_levels_trades];
	final double []
	    book_prices_buy=new double [n_levels_book],
	    book_prices_sell=new double [n_levels_book],
	    trades_prices_buy=new double [n_levels_trades],
	    trades_prices_sell=new double [n_levels_trades];


	final boolean PRINT_DATA_DEBUG=false;//true;
	//currenttime==35308657000000L;
	//currenttime==32894696000000L;
	//currenttime==34580422000000L;
	//currenttime>=9*3600*TraderInterface.TIME_SCALE && bsize>0;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+"============Liquidity deficit t=\t"+currenttime+
			       "\tn_buy\t"+this.buy.getSize()+"\tn_sell\t"+this.sell.getSize()+"\ttsize\t"+this.trades.getSize());
	}

	// fill buy orders
	int n_levels_book_buy=-1;
	if(true)
	{
	    final BookItem [] buf_buy=this.buy.getBuffer();
	    final int n_buy=this.buy.getSize();
	    double prev_price=Double.POSITIVE_INFINITY;
	    for(int i=0;i<n_buy;i++)
	    {
		final BookItem t=buf_buy[i];
		if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
		{
		    if(prev_price<t.price||t.buy_sell_indicator!=BookItem.BOOK_BUY)
		    {
			throw new RuntimeException("Buy prices are not not sorted:"+
						   " prev_price="+prev_price+
						   " t.price="+t.price+
						   " or t.buy_sell_indicator="+t.buy_sell_indicator+
						   "!="+BookItem.BOOK_BUY+
						   " currenttime="+currenttime);
		    }
		}
		final long tau=(currenttime-t.time)/ModelParameters.DISTANCE_TIME_SCALE;
		if(t.price-prev_price<-OrdersFlow.EPS)
		{ // next level
		    if(++n_levels_book_buy>=n_levels_book)
		    {
			break;
		    }
		    prev_price=book_prices_buy[n_levels_book_buy]=t.price;
		    vb_buy[n_levels_book_buy]=t.shares;
		    svb_buy[n_levels_book_buy]=t.shares*tau;
		}
		else
		{
		    vb_buy[n_levels_book_buy]+=t.shares;
		    svb_buy[n_levels_book_buy]+=t.shares*tau;
		}
	    }
	    if(n_levels_book_buy<n_levels_book)
	    {
		n_levels_book_buy++;
	    }

	    if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    {
		checkBookDataSet(book_prices_buy,
				 vb_buy,
				 svb_buy,
				 this.buy,
				 true,
				 n_levels_book_buy,
				 n_levels_book_buy,
				 currenttime);
	    }
	}


	// fill sell orders
	int n_levels_book_sell=-1;
	if(true)
	{
	    final BookItem [] buf_sell=this.sell.getBuffer();
	    final int n_sell=this.sell.getSize();
	    double prev_price=Double.NEGATIVE_INFINITY;
	    for(int i=0;i<n_sell;i++)
	    {
		final BookItem t=buf_sell[i];
		if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
		{
		    if(prev_price>t.price||t.buy_sell_indicator!=BookItem.BOOK_SELL)
		    {
			throw new RuntimeException("Sell prices are not not sorted:"+
						   " prev_price="+prev_price+
						   " t.price="+t.price+
						   " or t.buy_sell_indicator="+t.buy_sell_indicator+
						   "!="+BookItem.BOOK_SELL+
						   " currenttime="+currenttime);
		    }
		}
		final long tau=(currenttime-t.time)/ModelParameters.DISTANCE_TIME_SCALE;
		if(t.price-prev_price>OrdersFlow.EPS)
		{ // next level
		    if(++n_levels_book_sell>=n_levels_book)
		    {
			break;
		    }
		    prev_price=book_prices_sell[n_levels_book_sell]=t.price;
		    vb_sell[n_levels_book_sell]=t.shares;
		    svb_sell[n_levels_book_sell]=t.shares*tau;

		}
		else
		{
		    vb_sell[n_levels_book_sell]+=t.shares;
		    svb_sell[n_levels_book_sell]+=t.shares*tau;
		}
	    }
	    if(n_levels_book_sell<n_levels_book)
	    {
		n_levels_book_sell++;
	    }
	    if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    {
		checkBookDataSet(book_prices_sell,
				 vb_sell,
				 svb_sell,
				 this.sell,
				 false,
				 n_levels_book_sell,
				 n_levels_book_sell,
				 currenttime);
	    }
	}


	// fill trades
	int n_levels_trades_buy=-1,n_levels_trades_sell=-1,
	    n_levels_direction_trades_buy=-1,n_levels_direction_trades_sell=-1;
	if(true)
	{
	    final int tsize=this.trades.getSize();
	    final Trade [] tb=this.trades.getBuffer();
	    double prev_buy_price=Double.NEGATIVE_INFINITY,prev_sell_price=Double.POSITIVE_INFINITY;
	    boolean sum_buy=true,sum_sell=true,fdir_buy=true,fdir_sell=true;
	    long prevt=Long.MAX_VALUE;
	    for(int i=tsize,fexit=2;i-->0 && fexit>0;)
	    {
		final Trade t=tb[i];
		if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
		{
		    if(t.time>prevt)
		    {
			throw new RuntimeException("The data is not sorted:"+
						   prevt+" and "+t.time+
						   " currenttime="+currenttime);
		    }
		    prevt=t.time;
		}
		if(t.origtime<=0)
		{
		    // FIXME, take care of hidden trades later.
		    continue;
		}
		final long tau=(currenttime-t.origtime)/ModelParameters.DISTANCE_TIME_SCALE;
		if(t.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    if(sum_buy)
		    {
			final double diff=t.price-prev_buy_price;
			if(Math.abs(diff)>OrdersFlow.EPS)
			{ // next level
			    if(++n_levels_trades_buy>=n_levels_trades)
			    {
				fexit--;
				sum_buy=false;
			    }
			    else
			    {
				prev_buy_price=trades_prices_buy[n_levels_trades_buy]=t.price;
				vt_buy[n_levels_trades_buy]=t.shares;
				svt_buy[n_levels_trades_buy]=t.shares*tau;
				timet_levels_buy[n_levels_trades_buy]=t.time;
				if(fdir_buy)
				{
				    if(diff>OrdersFlow.EPS)
				    {
					n_levels_direction_trades_buy=n_levels_trades_buy;
				    }
				    else
				    {
					fdir_buy=false;
				    }
				}
			    }
			}
			else
			{
			    vt_buy[n_levels_trades_buy]+=t.shares;
			    svt_buy[n_levels_trades_buy]+=t.shares*tau;
			    timet_levels_buy[n_levels_trades_buy]=t.time;
			}
		    }
		}
		else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    if(sum_sell)
		    {
			final double diff=t.price-prev_sell_price;
			if(Math.abs(diff)>OrdersFlow.EPS)
			{ // next level
			    if(++n_levels_trades_sell>=n_levels_trades)
			    {
				fexit--;
				sum_sell=false;
			    }
			    else
			    {
				prev_sell_price=trades_prices_sell[n_levels_trades_sell]=t.price;
				vt_sell[n_levels_trades_sell]=t.shares;
				svt_sell[n_levels_trades_sell]=t.shares*tau;
				timet_levels_sell[n_levels_trades_sell]=t.time;
				if(fdir_sell)
				{
				    if(diff<-OrdersFlow.EPS)
				    {
					n_levels_direction_trades_sell=n_levels_trades_sell;
				    }
				    else
				    {
					fdir_sell=false;
				    }
				}
			    }
			}
			else
			{
			    vt_sell[n_levels_trades_sell]+=t.shares;
			    svt_sell[n_levels_trades_sell]+=t.shares*tau;
			    timet_levels_sell[n_levels_trades_sell]=t.time;
			}
		    }
		}
	    }
	    if(sum_buy)
	    {
		n_levels_trades_buy++;
	    }
	    n_levels_direction_trades_buy++;
	    if(sum_sell)
	    {
		n_levels_trades_sell++;
	    }
	    n_levels_direction_trades_sell++;
	    if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    {
		checkTradesDataSet(trades_prices_buy,
				   vt_buy,
				   svt_buy,
				   this.trades,
				   true,
				   n_levels_trades_buy,
				   n_levels_direction_trades_buy,
				   timet_levels_buy,
				   currenttime);
		checkTradesDataSet(trades_prices_sell,
				   vt_sell,
				   svt_sell,
				   this.trades,
				   false,
				   n_levels_trades_sell,
				   n_levels_direction_trades_sell,
				   timet_levels_sell,
				   currenttime);
	    }
	}

	// now analyze data


	// buy side
	final long v_crit_buy,tau_crit_buy,v_book_buy,tau_book_buy,tau_tmin_book_buy,tau_tmin_btrades_buy;
	final double p_crit_buy,p_tmin_book_buy,p_tmin_btrades_buy;
	final int levnum_tmin_book_buy,levnum_tmin_btrades_buy,ptr_book_start_buy;
	if(n_levels_book_buy<=1||n_levels_trades_buy<=1)
	{
	    v_crit_buy=tau_crit_buy=v_book_buy=tau_book_buy=tau_tmin_book_buy=tau_tmin_btrades_buy=-1;
	    p_crit_buy=p_tmin_book_buy=p_tmin_btrades_buy=Double.NaN;
	    levnum_tmin_book_buy=levnum_tmin_btrades_buy=ptr_book_start_buy=-1;
	}
	else // at least two orders & trade levels required
	{
	    p_crit_buy=trades_prices_buy[0];
	    if(Math.abs(p_crit_buy-book_prices_buy[0])>OrdersFlow.EPS)
	    {
		ptr_book_start_buy=0;
		v_crit_buy=vt_buy[0];
		tau_crit_buy=svt_buy[0]/v_crit_buy;
	    }
	    else
	    {
		ptr_book_start_buy=1;
		v_crit_buy=vb_buy[0]+vt_buy[0];
		tau_crit_buy=(svb_buy[0]+svt_buy[0])/v_crit_buy;
	    }
	    v_book_buy=vb_buy[ptr_book_start_buy];
	    tau_book_buy=svb_buy[ptr_book_start_buy]/v_book_buy;
	    long tmin=Long.MAX_VALUE,s=0,v=0;
	    int ptr_min=-1;
	    for(int i=ptr_book_start_buy;i<n_levels_book_buy;i++)
	    {
		s+=svb_buy[i];
		v+=vb_buy[i];
		final long t=s/v;
		if(t<tmin)
		{
		    tmin=t;
		    ptr_min=i;
		}
	    }
	    levnum_tmin_book_buy=ptr_min-ptr_book_start_buy;
	    p_tmin_book_buy=book_prices_buy[ptr_min];
	    tau_tmin_book_buy=tmin;
	    // trades
	    if(ptr_book_start_buy>0)
	    {
		s=svb_buy[0];
		v=vb_buy[0];
	    }
	    else
	    {
		s=v=0;
	    }
	    tmin=Long.MAX_VALUE;
	    ptr_min=-1;
	    for(int i=0;i<n_levels_trades_buy;i++)
	    {
		s+=svt_buy[i];
		v+=vt_buy[i];
		final long t=s/v;
		if(t<tmin)
		{
		    tmin=t;
		    ptr_min=i;
		}
	    }
	    levnum_tmin_btrades_buy=ptr_min;
	    p_tmin_btrades_buy=trades_prices_buy[levnum_tmin_btrades_buy];
	    tau_tmin_btrades_buy=tmin;
	}

	// sell side
	final long v_crit_sell,tau_crit_sell,v_book_sell,tau_book_sell,tau_tmin_book_sell,tau_tmin_btrades_sell;
	final double p_crit_sell,p_tmin_book_sell,p_tmin_btrades_sell;
	final int levnum_tmin_book_sell,levnum_tmin_btrades_sell,ptr_book_start_sell;
	if(n_levels_book_sell<=1||n_levels_trades_sell<=1)
	{
	    v_crit_sell=tau_crit_sell=v_book_sell=tau_book_sell=tau_tmin_book_sell=tau_tmin_btrades_sell=-1;
	    p_crit_sell=p_tmin_book_sell=p_tmin_btrades_sell=Double.NaN;
	    levnum_tmin_book_sell=levnum_tmin_btrades_sell=ptr_book_start_sell=-1;
	}
	else // at least two orders & trade levels required
	{
	    p_crit_sell=trades_prices_sell[0];
	    if(Math.abs(p_crit_sell-book_prices_sell[0])>OrdersFlow.EPS)
	    {
		ptr_book_start_sell=0;
		v_crit_sell=vt_sell[0];
		tau_crit_sell=svt_sell[0]/v_crit_sell;
	    }
	    else
	    {
		ptr_book_start_sell=1;
		v_crit_sell=vb_sell[0]+vt_sell[0];
		tau_crit_sell=(svb_sell[0]+svt_sell[0])/v_crit_sell;
	    }
	    v_book_sell=vb_sell[ptr_book_start_sell];
	    tau_book_sell=svb_sell[ptr_book_start_sell]/v_book_sell;
	    long tmin=Long.MAX_VALUE,s=0,v=0;
	    int ptr_min=-1;
	    for(int i=ptr_book_start_sell;i<n_levels_book_sell;i++)
	    {
		s+=svb_sell[i];
		v+=vb_sell[i];
		final long t=s/v;
		if(t<tmin)
		{
		    tmin=t;
		    ptr_min=i;
		}
	    }
	    levnum_tmin_book_sell=ptr_min-ptr_book_start_sell;
	    p_tmin_book_sell=book_prices_sell[ptr_min];
	    tau_tmin_book_sell=tmin;
	    // trades
	    if(ptr_book_start_sell>0)
	    {
		s=svb_sell[0];
		v=vb_sell[0];
	    }
	    else
	    {
		s=v=0;
	    }
	    tmin=Long.MAX_VALUE;
	    ptr_min=-1;
	    for(int i=0;i<n_levels_trades_sell;i++)
	    {
		s+=svt_sell[i];
		v+=vt_sell[i];
		final long t=s/v;
		if(t<tmin)
		{
		    tmin=t;
		    ptr_min=i;
		}
	    }
	    levnum_tmin_btrades_sell=ptr_min;
	    p_tmin_btrades_sell=trades_prices_sell[levnum_tmin_btrades_sell];
	    tau_tmin_btrades_sell=tmin;

	    if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    {
		checkCritValues(
		    v_crit_buy,
		    tau_crit_buy,
		    v_book_buy,
		    tau_book_buy,
		    tau_tmin_book_buy,
		    tau_tmin_btrades_buy,
		    p_crit_buy,
		    p_tmin_book_buy,
		    p_tmin_btrades_buy,
		    levnum_tmin_book_buy,
		    levnum_tmin_btrades_buy,
		    ptr_book_start_buy,

		    n_levels_book_buy,
		    book_prices_buy,
		    vb_buy,
		    svb_buy,

		    n_levels_trades_buy,
		    trades_prices_buy,
		    vt_buy,
		    svt_buy);

		checkCritValues(
		    v_crit_sell,
		    tau_crit_sell,
		    v_book_sell,
		    tau_book_sell,
		    tau_tmin_book_sell,
		    tau_tmin_btrades_sell,
		    p_crit_sell,
		    p_tmin_book_sell,
		    p_tmin_btrades_sell,
		    levnum_tmin_book_sell,
		    levnum_tmin_btrades_sell,
		    ptr_book_start_sell,

		    n_levels_book_sell,
		    book_prices_sell,
		    vb_sell,
		    svb_sell,

		    n_levels_trades_sell,
		    trades_prices_sell,
		    vt_sell,
		    svt_sell);
	    }
	}


	// results

	this.deficit=(tau_crit_buy-tau_book_buy)-(tau_crit_sell-tau_book_sell);
	this.expected_price_change=(p_tmin_book_buy-p_crit_buy)+(p_tmin_book_sell-p_crit_sell);

	this.expected_buy=p_tmin_book_buy-p_crit_buy;
	this.expected_sell=p_tmin_book_sell-p_crit_sell;
	this.deficit_buy=tau_crit_buy-tau_book_buy;
	this.deficit_sell=tau_crit_sell-tau_book_sell;

	this.ind_lev_tmin_book_buy=levnum_tmin_book_buy;
	this.ind_lev_tmin_book_sell=levnum_tmin_book_sell;
	this.t_crit_buy=tau_crit_buy;
	this.t_crit_sell=tau_crit_sell;
	this.t_tmin_book_buy=tau_tmin_book_buy;
	this.t_tmin_book_sell=tau_tmin_book_sell;
	this.t_tmin_btrades_buy=tau_tmin_btrades_buy;
	this.t_tmin_btrades_sell=tau_tmin_btrades_sell;
	// end of calculations
    }



    /** Constrains check.
     */
    private static void checkBookDataSet(final double [] prices,
					 final long [] sizes,
					 final long [] check2,
					 final Distribution<BookItem> items,
					 final boolean otype,
					 final int n_max,
					 final int n_monotonic,
					 final long currenttime)
    {
	final OrdersFlow o=new OrdersFlow(otype),o2=new OrdersFlow(otype);
	final int tsize=items.getSize();
	final BookItem [] tb=items.getBuffer();
	for(int i=0;i<tsize;i++)
	{
	    final BookItem t=tb[i];
	    o.addOrderCheckPrev(t.price,t.shares);
	    final long tau=t.shares*((currenttime-t.time)/ModelParameters.DISTANCE_TIME_SCALE);
	    // +t.shares to avoid 0
	    o2.addOrderCheckPrev(t.price,tau+t.shares);
	}
	o.getTotalVolume();
	o2.getTotalVolume();
	checkPreparedData(prices,sizes,check2,o,o2,otype,n_max,n_monotonic,currenttime);
    }
    private static void checkTradesDataSet(final double [] prices,
					   final long [] sizes,
					   final long [] check2,
					   final Distribution<Trade> items,
					   final boolean otype,
					   final int n_max,
					   final int n_monotonic,
					   final long [] t_levels,
					   final long currenttime)
    {
	final OrdersFlow o=new OrdersFlow(otype),o2=new OrdersFlow(otype);
	final int tsize=items.getSize();
	final Trade [] tb=items.getBuffer();
	long prevt=Long.MAX_VALUE;
	double prev_buy=Double.NaN,prev_sell=Double.NaN;
	long t_prev_buy=-1,t_prev_sell=-1;
	int npbuy=0,npsell=0;
	for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException(
			    "The data is not time sorted:"+
			    " t.time="+t.time+
			    " prevt="+prevt+
			    " otype="+otype+
			    " currenttime="+currenttime);
	    }
	    prevt=t.time;
	    if(t.origtime<=0)
	    {
		continue;
	    }
	    final long tau=t.shares*((currenttime-t.origtime)/ModelParameters.DISTANCE_TIME_SCALE);
	    if(otype)
	    {
		if(t.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    o.addOrderCheckPrev(t.price,t.shares);
		    // +t.shares to avoid 0
		    o2.addOrderCheckPrev(t.price,tau+t.shares);
		    if(!(Math.abs(prev_buy-t.price)<=OrdersFlow.EPS))
		    {
			prev_buy=t.price;
			if(npbuy>0 && npbuy<=n_max &&
			   t_levels[npbuy-1]!=t_prev_buy)
			{
			   throw new RuntimeException(
			       "The time interval mismatch:"+
			       " t_levels["+(npbuy-1)+"]="+t_levels[npbuy-1]+
			       " t_prev_buy="+t_prev_buy+
			       " otype="+otype+
			       " currenttime="+currenttime);
			}
			npbuy++;
			t_prev_buy=t.time;
		    }
		    else
		    {
			t_prev_buy=t.time;
		    }
		}
	    }
	    else
	    {
		if(t.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    o.addOrderCheckPrev(t.price,t.shares);
		    // +t.shares to avoid 0
		    o2.addOrderCheckPrev(t.price,tau+t.shares);
		    if(!(Math.abs(prev_sell-t.price)<=OrdersFlow.EPS))
		    {
			prev_sell=t.price;
			if(npsell>0 && npsell<=n_max &&
			   t_levels[npsell-1]!=t_prev_sell)
			{
			   throw new RuntimeException(
			       "The time interval mismatch:"+
			       " t_levels["+(npsell-1)+"]="+t_levels[npsell-1]+
			       " t_prev_sell="+t_prev_sell+
			       " otype="+otype+
			       " currenttime="+currenttime);
			}
			npsell++;
			t_prev_sell=t.time;
		    }
		    else
		    {
			t_prev_sell=t.time;
		    }
		}
	    }
	}
	//o.dump("o:");
	//o2.dump("o2:");
	checkPreparedData(prices,sizes,check2,o,o2,!otype,n_max,n_monotonic,currenttime);
    }


    private static void checkPreparedData(final double [] prices,
					  final long [] sizes,
					  final long [] check2,
					  final OrdersFlow o,
					  final OrdersFlow o2,
					  final boolean direction_count_n_ticks,
					  final int n_max,
					  final int n_monotonic,
					  final long currenttime)
    {
	final int [] count=new int []{0,0,0};
	final double [] prevp=new double []{direction_count_n_ticks?Double.POSITIVE_INFINITY:Double.NEGATIVE_INFINITY};
	o.forEachEntry(new OrdersFlow.IterateThrough(){
		public boolean execute(final double price,final long shares)
		{
		    if(count[2]==0)
		    {
			if(direction_count_n_ticks?(price<=prevp[0]):(price>=prevp[0]))
			{
			    prevp[0]=price;
			    count[1]++;
			}
			else
			{
			    count[2]=1;
			}
		    }
		    final int i=count[0]++;
		    if(prices[i]!=price||sizes[i]!=shares)
		    {
			throw new RuntimeException(
			    "Price or size mismatchVolume Mismatch:"+
			    " prices["+i+"]="+prices[i]+
			    " price="+price+
			    " sizes["+i+"]="+sizes[i]+
			    " shares="+shares+
			    " direction_count_n_ticks="+direction_count_n_ticks+
			    " currenttime="+currenttime);
		    }
		    if(count[0]>=n_max)
		    {
			return false;
		    }
		    return true;
		}
	    });
	if(count[0]!=n_max||count[1]!=n_monotonic)
	{
	    throw new RuntimeException("n_max mismatch:"+
				       " count[0]="+count[0]+
				       " n_max="+n_max+
				       " count[1]="+count[1]+
				       " n_monotonic="+n_monotonic+
				       " direction_count_n_ticks="+direction_count_n_ticks+
				       " currenttime="+currenttime);
	}
	prevp[0]=direction_count_n_ticks?Double.POSITIVE_INFINITY:Double.NEGATIVE_INFINITY;
	count[0]=count[1]=count[2]=0;
	o2.forEachEntry(new OrdersFlow.IterateThrough(){
		public boolean execute(final double price,final long shares)
		{
		    if(count[2]==0)
		    {
			if(direction_count_n_ticks?(price<=prevp[0]):(price>=prevp[0]))
			{
			    prevp[0]=price;
			    count[1]++;
			}
			else
			{
			    count[2]=1;
			}
		    }
		    final int i=count[0]++;

		    final long shares_adj=shares-sizes[i]; // adjustment to +t.shares we used in tau to avoid 0

		    if(prices[i]!=price||check2[i]!=shares_adj)
		    {
			throw new RuntimeException(
			    "Price or size mismatchVolume Mismatch:"+
			    " prices["+i+"]="+prices[i]+
			    " price="+price+
			    " check2["+i+"]="+check2[i]+
			    " shares_adj="+shares_adj+
			    " direction_count_n_ticks="+direction_count_n_ticks+
			    " currenttime="+currenttime);
		    }
		    if(count[0]>=n_max)
		    {
			return false;
		    }
		    return true;
		}
	    });
	if(count[0]!=n_max||count[1]!=n_monotonic)
	{
	    throw new RuntimeException("n_max mismatch:"+
				       " count[0]="+count[0]+
				       " n_max="+n_max+
				       " count[1]="+count[1]+
				       " n_monotonic="+n_monotonic+
				       " direction_count_n_ticks="+direction_count_n_ticks+
				       " currenttime="+currenttime);
	}
    }



    private static void checkCritValues(
	final long v_crit,
	final long tau_crit,
	final long v_book,
	final long tau_book,
	final long tau_tmin_book,
	final long tau_tmin_btrades,
	final double p_crit,
	final double p_tmin_book,
	final double p_tmin_btrades,
	final int levnum_tmin_book,
	final int levnum_tmin_btrades,
	final int ptr_book_start,

	final int n_levels_book,
	final double [] book_prices,
	final long [] vb,
	final long [] svb,

	final int n_levels_trades,
	final double [] trades_prices,
	final long [] vt,
	final long [] svt)
    {
	final long v_crit_a,
	    tau_crit_a,
	    v_book_a,
	    tau_book_a,
	    tau_tmin_book_a,
	    tau_tmin_btrades_a;
	final double p_crit_a,p_tmin_book_a,p_tmin_btrades_a;
	final int levnum_tmin_book_a,
	    levnum_tmin_btrades_a,
	    ptr_book_start_a;

	if(n_levels_book<=1||n_levels_trades<=1)
	{
	   v_crit_a=
	       tau_crit_a=
	       v_book_a=
	       tau_book_a=
	       tau_tmin_book_a=
	       tau_tmin_btrades_a=-1;
	   p_crit_a=p_tmin_book_a=p_tmin_btrades_a=Double.NaN;
	   levnum_tmin_book_a=
	       levnum_tmin_btrades_a=
	       ptr_book_start_a=-1;
	}
	else
	{
	    long v,s;
	    p_crit_a=trades_prices[0];
	    if(Math.abs(p_crit_a-book_prices[0])>OrdersFlow.EPS)
	    {
		v_crit_a=vt[0];
		tau_crit_a=svt[0]/v_crit_a;
		v_book_a=vb[0];
		tau_book_a=svb[0]/v_book_a;
		ptr_book_start_a=0;
		v=s=0;
	    }
	    else
	    {
		v_crit_a=vt[0]+vb[0];
		tau_crit_a=(svt[0]+svb[0])/v_crit_a;
		v_book_a=vb[1];
		tau_book_a=svb[1]/v_book_a;
		ptr_book_start_a=1;
		v=vb[0];
		s=svb[0];
	    }
	    long tmin=Long.MAX_VALUE;
	    int ptr_min=-1;
	    for(int i=0;i<n_levels_trades;i++)
	    {
		v+=vt[i];
		s+=svt[i];
		final long t=s/v;
		if(t<tmin)
		{
		    ptr_min=i;
		    tmin=t;
		}
	    }
	    tau_tmin_btrades_a=tmin;
	    levnum_tmin_btrades_a=ptr_min;
	    p_tmin_btrades_a=trades_prices[ptr_min];

	    v=s=0;
	    tmin=Long.MAX_VALUE;
	    ptr_min=-1;
	    for(int i=ptr_book_start_a;i<n_levels_book;i++)
	    {
		v+=vb[i];
		s+=svb[i];
		final long t=s/v;
		if(t<tmin)
		{
		    ptr_min=i;
		    tmin=t;
		}
	    }
	    tau_tmin_book_a=tmin;
	    p_tmin_book_a=book_prices[ptr_min];
	    levnum_tmin_book_a=ptr_min-ptr_book_start_a;
	}

	if(v_crit_a!=v_crit||
	   tau_crit_a!=tau_crit||
	   v_book_a!=v_book||
	   tau_book_a!=tau_book||
	   tau_tmin_book_a!=tau_tmin_book||
	   tau_tmin_btrades_a!=tau_tmin_btrades||

	   Double.isNaN(p_crit_a)!=Double.isNaN(p_crit)||
	   (!Double.isNaN(p_crit_a) && p_crit_a!=p_crit)||

	   Double.isNaN(p_tmin_book_a)!=Double.isNaN(p_tmin_book)||
	   (!Double.isNaN(p_tmin_book_a) && p_tmin_book_a!=p_tmin_book)||

	   Double.isNaN(p_tmin_btrades_a)!=Double.isNaN(p_tmin_btrades)||
	   (!Double.isNaN(p_tmin_btrades_a) && p_tmin_btrades_a!=p_tmin_btrades)||

	   levnum_tmin_book_a!=levnum_tmin_book||
	   levnum_tmin_btrades_a!=levnum_tmin_btrades||
	   ptr_book_start_a!=ptr_book_start)
	{
	    throw new RuntimeException(
		"Mismatched results:"+
		" v_crit_a="+v_crit_a+" v_crit="+v_crit+
		" tau_crit_a="+tau_crit_a+" tau_crit="+tau_crit+
		" v_book_a="+v_book_a+ " v_book="+v_book+
		" tau_book_a="+tau_book_a+" tau_book="+tau_book+
		" tau_tmin_book_a="+tau_tmin_book_a+" tau_tmin_book="+tau_tmin_book+
		" tau_tmin_btrades_a="+tau_tmin_btrades_a+" tau_tmin_btrades="+tau_tmin_btrades+
		" p_crit_a="+p_crit_a+" p_crit="+p_crit+
		" p_tmin_book_a="+p_tmin_book_a+" p_tmin_book="+p_tmin_book+
		" p_tmin_btrades_a="+p_tmin_btrades_a+" p_tmin_btrades="+p_tmin_btrades+
		" levnum_tmin_book_a="+levnum_tmin_book_a+" levnum_tmin_book="+levnum_tmin_book+
		" levnum_tmin_btrades_a="+levnum_tmin_btrades_a+" levnum_tmin_btrades="+levnum_tmin_btrades+
		" ptr_book_start_a="+ptr_book_start_a+" ptr_book_start="+ptr_book_start);
	}
    }

}
