package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Modelling of book virtual shifts. Return of the Jedi.
 *  @version $Id: CalculateBookVirtualShifts.java,v 1.55 2009/06/20 22:17:52 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateBookVirtualShifts {

    /** Simplistic model(for now).
     */
    private static final long FACTOR_CONVERT_ALL_TO_EQUILIBRIUM=2;


    private final OrdersFlow obuy=new OrdersFlow(true),osell=new OrdersFlow(false);
    private final MatchOrdersVariousShifts virtualshifts=new MatchOrdersVariousShifts();
    private final Distribution<MatchOrdersVariousShifts.MatchWithShift>
	allshifts=new Distribution<MatchOrdersVariousShifts.MatchWithShift>();


    private final Distribution<BookItem> buy,sell,tmpordersbuf;
    private final Distribution<Trade> trades,tmptradesbuf;

    CalculateBookVirtualShifts(final Distribution<BookItem> buy,
			       final Distribution<BookItem> sell,
			       final Distribution<Trade> trades,
			       final Distribution<BookItem> tmpordersbuf,
			       final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
	this.tmpordersbuf=tmpordersbuf;
	this.tmptradesbuf=tmptradesbuf;
    }

    MatchOrdersVariousShifts.MatchWithShift
	end_of_active=null,
	end_of_active2=null,
	equilibrium_with_trades=null,
	end_of_interval=null;
    long time_equilibrium=-1;
    double max_convexity=Double.NaN,
	   dspread=Double.NaN,
	   equilibrium_trades_max_min=Double.NaN,
	   equilibrium_convexity=Double.NaN;



    void clear()
    {
	this.end_of_active=
	this.end_of_active2=
	this.equilibrium_with_trades=
	this.end_of_interval=null;
	this.max_convexity=this.dspread=this.equilibrium_trades_max_min=this.equilibrium_convexity=Double.NaN;
	this.time_equilibrium=-1;

	// clear internal buffers
	this.allshifts.clearAll();
	this.obuy.clear();
	this.osell.clear();
    }


    void calculateVirtualShifts(final long currenttime,
				final double pcurrent_buy,
				final double pcurrent_sell,
				final double max_shift,
				final long trades_cleared_before)
    {
	final int I_START_TO_LOOK=1;


	this.virtualshifts.copyOrderedOrdersToFlow(this.buy,this.obuy);
	this.virtualshifts.copyOrderedOrdersToFlow(this.sell,this.osell);
	this.virtualshifts.matchWithVariousShifts(this.obuy,this.osell,this.allshifts);
	final int bsize=this.allshifts.getSize(),tsize=this.trades.getSize();
	final MatchOrdersVariousShifts.MatchWithShift [] ob=this.allshifts.getBuffer();
	final Trade [] tb=this.trades.getBuffer();


	final boolean PRINT_DATA_DEBUG=false;//true;
	//currenttime==35308657000000L;
	//currenttime==32894696000000L;
	//currenttime==34580422000000L;
	//currenttime>=9*3600*TraderInterface.TIME_SCALE && bsize>0;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+"============Vritual Shifts t=\t"+currenttime+
			       "\tN_SHIFTS FOUND\t"+bsize+"\tTRADES\t"+tsize);
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("num\t"+
			       "shift0\t"+
			       "delta0\t"+
			       "shift\t"+
			       "eff_shift\t"+
			       "delta\t"+

			       // median info
			       "ptr_to_median\t"+
			       "eff_med_shift\t"+
			       "delta at med\t"+

			       // extra debug info
			       "type\t"+
			       "pmatch nodp\t"+
			       "dp\t"+

			       // convexity
		               "convexity_ratio\t"+
			       "max_convexity_ratio\t"+
			       "vmatch\t"+
			       "\t"+
			       "TRADE IND FROM LAST\t"+
			       "TRADE SHIFT\t"+
			       "TRADE VOL\t"+
			       "TRADE TAU\t"+
			       "\t"+
			       "EQ Volume\t"+
			       "EQ delta\t"+
			       "EQ shift\t"+
			       "EQ min/max\t"+
			       "EQ time\t");
	}



	long vtrades=0,t_prev_vmatch=-1,trade_prev_t=Long.MAX_VALUE;
	double min_trade_price=Double.POSITIVE_INFINITY,max_trade_price=Double.NEGATIVE_INFINITY;
	boolean flag_no_equilibrium=true;

	final double ds=this.dspread=pcurrent_sell-pcurrent_buy;
	double t_prev_shift=-ds,maxconv=0;
        for(int i=0,ptr_to_median=0,ptr_to_trades=tsize-1;i<bsize;i++)
	{
	    final MatchOrdersVariousShifts.MatchWithShift t=ob[i];
	    if(t_prev_shift>t.shift)
	    {
		throw new RuntimeException("Ordering is broken i="+i+
					   " t_prev_shift="+t_prev_shift+
					   " t.shift="+t.shift+
					   " ds="+ds+
					   " currenttime="+currenttime);
	    }
	    t_prev_shift=t.shift;
	    if(t_prev_vmatch>t.vmatch)
	    {
		throw new RuntimeException("Incorrect volume ordering i="+i+
					   " t_prev_vmatch="+t_prev_vmatch+
					   " t.vmatch="+t.vmatch+
					   " ds="+ds+
					   " currenttime="+currenttime);
	    }
	    t_prev_vmatch=t.vmatch;
	    final double current_shift=t.shift+ds;
	    if(!(current_shift<=max_shift))
	    {
		// no more accepted shifts or NaN
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       " current_shift="+current_shift+"\t"+
				       " max_shift="+max_shift);
		}
		break;
	    }

	    // current match. use to save as the last match
	    this.end_of_interval=t;

	    // ----------------------------------------------------------
	    // calculate median for this shift.
	    final long vm=t.vmatch/2;
	    MatchOrdersVariousShifts.MatchWithShift tmed=null;
	    double median=Double.NaN,median_underestimated=Double.NaN;
	    for(int m=ptr_to_median;m<=i;m++)
	    {
		final MatchOrdersVariousShifts.MatchWithShift tm=ob[m];
		if(false && PRINT_DATA_DEBUG)
		{
		    System.err.println(":"+String.valueOf(i)+"\t"+String.valueOf(m)+"\t"+
				       tm.vmatch+"\t"+
				       vm+"\t");
		}
		// overestimated median
		if(tm.vmatch>=vm)
		{
		    ptr_to_median=m;
		    tmed=tm;
		    final double dpm;
		    if(m>0)
		    {
			median_underestimated=ob[m-1].shift+ds;
			dpm=tm.shift-ob[m-1].shift;

		    }
		    else
		    {
			median_underestimated=0;
			dpm=ds;
		    }
		    median=tm.shift+ds+
			// adjustment to overestimation
			((vm-tm.vmatch)*dpm)/(tm.vmatch-(m>0?ob[m-1].vmatch:0));
		    break;
		}
	    }
	    if(tmed==null)
	    {
		throw new RuntimeException("This should never happen");
	    }

	    final double r=median_underestimated/current_shift;
	    if(r>=maxconv && i>=I_START_TO_LOOK)
	    {
		this.max_convexity=maxconv=r;
		this.end_of_active=tmed;
		this.end_of_active2=t;
	    }
	    // ----------------------------------------------------------


	    // ----------------------------------------------------------
	    // scan trades until we reach max-min equals to current shift
	    for(;ptr_to_trades>=0 && (max_trade_price-min_trade_price)<=current_shift;
		ptr_to_trades--)
	    {
		final Trade tr=tb[ptr_to_trades];
		if(tr.time>trade_prev_t)
		{
		    throw new RuntimeException("Trades are not ordered tr.time="+tr.time+
					       " trade_prev_t="+trade_prev_t+
					       " currenttime="+currenttime);
		}
		trade_prev_t=tr.time;
		if(tr.price<=min_trade_price)
		{
		    min_trade_price=tr.price;
		}
		if(tr.price>=max_trade_price)
		{
		    max_trade_price=tr.price;
		}
		vtrades+=tr.shares;
	    }
            // ----------------------------------------------------------
	    // now we have calculated the values of trades, corresponding to given shift:
	    // min_trade_price
	    // max_trade_price
	    // ptr_to_trades - pointer to next trade at exit.
	    // vtrades - volume of all trades before the last accepted
	    if((vtrades/FACTOR_CONVERT_ALL_TO_EQUILIBRIUM)>=t.vmatch &&
	       i>=I_START_TO_LOOK)
	    {
		if(flag_no_equilibrium)
		{
		    this.equilibrium_with_trades=t;
		    this.equilibrium_trades_max_min=max_trade_price-min_trade_price;
		    this.time_equilibrium=(ptr_to_trades+1<tsize?(currenttime-tb[ptr_to_trades+1].time):-1);
		    this.equilibrium_convexity=median/current_shift;
		    flag_no_equilibrium=false;
		}
	    }
	    else
	    {
		flag_no_equilibrium=true;
	    }

            // ----------------------------------------------------------
	    // constrains check
	    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    {
		final Trade [] tmpb=this.tmptradesbuf.getBuffer();
		// cheat buffer
		final int ptr_buf=ptr_to_trades+1;
		final int n=tsize-ptr_buf;
		System.arraycopy(tb,ptr_buf,tmpb,0,n);
		if(n<tmpb.length)
		{
		    tmpb[n]=null;
		}
		this.tmptradesbuf.setSize(n);

		// very expensive constrains.
		final CalculateTradeHContrib hc=
		    new CalculateTradeHContrib(this.tmptradesbuf,this.tmptradesbuf);
		hc.calculateContribs(new long [] {currenttime-(ptr_buf<tsize?tb[ptr_buf].time:0)}, // take anything
				     currenttime,
				     pcurrent_buy,
				     pcurrent_sell,
				     trades_cleared_before);
		if(hc.vol_all[0]!=vtrades||
		   Double.isNaN(hc.minprice[0])!=Double.isNaN(min_trade_price)||
		   (!Double.isNaN(min_trade_price) && hc.minprice[0]!=min_trade_price)||
		   Double.isNaN(hc.maxprice[0])!=Double.isNaN(max_trade_price)||
		   (!Double.isNaN(max_trade_price) && hc.maxprice[0]!=max_trade_price))
		{
		    throw new RuntimeException("Data mismatch: hc.vol_all[0]="+hc.vol_all[0]+
					       " vtrades="+vtrades+
					       " hc.minprice[0]="+hc.minprice[0]+
					       " min_trade_price="+min_trade_price+
					       " hc.maxprice[0]="+hc.maxprice[0]+
					       " max_trade_price="+max_trade_price+
					       " ptr_to_trades="+ptr_to_trades+
					       " currenttime="+currenttime);

		}

	    }



	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println(
		    String.valueOf(i)+"\t"+
		    t.shift0+"\t"+
		    t.delta0+"\t"+
		    t.shift+"\t"+
		    current_shift+"\t"+
		    t.delta+"\t"+
		    // median info
		    ptr_to_median+"\t"+
		    median_underestimated+"\t"+
		    tmed.delta+"\t"+
		    // extra debug info
		    t.type+"\t"+
		    t.test_pmatch+"\t"+
		    t.test_dp+"\t"+
		    // convexity
		    r+"\t"+
		    this.max_convexity+"\t"+
		    t.vmatch+"\t"+
		    "\t"+
		    ((tsize-1)-(ptr_to_trades+1))+"\t"+
		    (max_trade_price-min_trade_price)+"\t"+
		    vtrades+"\t"+
		    (ptr_to_trades+1<tsize?(currenttime-tb[ptr_to_trades+1].time):-1)+"\t\t"+
		    (this.equilibrium_with_trades!=null?this.equilibrium_with_trades.vmatch:-1)+"\t"+
		    (this.equilibrium_with_trades!=null?this.equilibrium_with_trades.delta:Double.NaN)+"\t"+
		    (this.equilibrium_with_trades!=null?this.equilibrium_with_trades.shift:Double.NaN)+"\t"+
		    this.equilibrium_trades_max_min+"\t"+
		    this.time_equilibrium+"\t");
	    }
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("END");
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println(
		"max_convexity\t"+this.max_convexity+"\n"+
		"dspread\t"+this.dspread+"\n");
		if(this.end_of_active!=null)
		{
		    System.err.println("END OF ACTIVE\n"+
				       "shift\t"+this.end_of_active.shift+"\n"+
				       "delta\t"+this.end_of_active.delta+"\n"+
				       "shift0\t"+this.end_of_active.shift0+"\n"+
				       "delta0\t"+this.end_of_active.delta0+"\n"+
				       "vol_shift\t"+this.end_of_active.vmatch+"\n");
		}
		if(this.end_of_active2!=null)
		{
		    System.err.println("END OF ACTIVE2\n"+
				       "2shift\t"+this.end_of_active2.shift+"\n"+
				       "2delta\t"+this.end_of_active2.delta+"\n"+
				       "2shift0\t"+this.end_of_active2.shift0+"\n"+
				       "2delta0\t"+this.end_of_active2.delta0+"\n"+
				       "2vol_shift\t"+this.end_of_active2.vmatch+"\n");
		}
		if(this.equilibrium_with_trades!=null)
		{
		    System.err.println("EQUILIBRIUM\n"+
				       "time_equilibrium\t"+this.time_equilibrium+"\n"+
				       "equilibrium_trades_max_min\t"+this.equilibrium_trades_max_min+"\n"+
				       "eq_shift\t"+this.equilibrium_with_trades.shift+"\n"+
				       "eq_delta\t"+this.equilibrium_with_trades.delta+"\n"+
				       "eq_shift0\t"+this.equilibrium_with_trades.shift0+"\n"+
				       "eq_delta0\t"+this.equilibrium_with_trades.delta0+"\n"+
				       "eq_vol_shift\t"+this.equilibrium_with_trades.vmatch+"\n");
		}
		if(this.end_of_interval!=null)
		{
		    System.err.println("END OF INTERVAL\n"+
				       "interval_end_shift\t"+this.end_of_interval.shift+"\n"+
				       "interval_end_delta\t"+this.end_of_interval.delta+"\n"+
				       "interval_end_shift0\t"+this.end_of_interval.shift0+"\n"+
				       "interval_end_delta0\t"+this.end_of_interval.delta0+"\n"+
				       "interval_end_vol_shift\t"+this.end_of_interval.vmatch+"\n");
		}
	}

    }
}
