package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Modelling of book & trades(as presudo-orders) virtual shifts. Return of the Jedi ver 2.
 *  @version $Id: CalculateBookVirtualShifts2.java,v 1.27 2020/02/06 00:09:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateBookVirtualShifts2 {


    private final OrdersFlow obuy=new OrdersFlow(true),osell=new OrdersFlow(false);
    private final MatchOrdersVariousShifts virtualshifts=new MatchOrdersVariousShifts();
    private final Distribution<MatchOrdersVariousShifts.MatchWithShift>
	allshifts=new Distribution<MatchOrdersVariousShifts.MatchWithShift>();

    private final Distribution<BookItem> buy,sell,tmpordersbuf;
    private final Distribution<Trade> trades,tmptradesbuf;

    CalculateBookVirtualShifts2(final Distribution<BookItem> buy,
				final Distribution<BookItem> sell,
				final Distribution<Trade> trades,
				final Distribution<BookItem> tmpordersbuf,
				final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
	this.tmpordersbuf=tmpordersbuf;
	this.tmptradesbuf=tmptradesbuf;
    }

    double [] tot_minprice=null,tot_maxprice=null,trade_tot_median_low_tau=null,trade_tot_median_high_tau=null;
    long [] v_tot_alltrades=null,v_tot_hiddentrades=null;
    double [] max_convexity=null;
    MatchOrdersVariousShifts.MatchWithShift []
	convexity_interv=null,
	convexity_interv2=null,
	end_of_interval=null;

    double dspread=Double.NaN;

    void clear()
    {
	this.tot_minprice=
	this.tot_maxprice=
	this.trade_tot_median_low_tau=
	this.trade_tot_median_high_tau=null;
	this.v_tot_alltrades=
	this.v_tot_hiddentrades=null;
	this.max_convexity=null;

	this.convexity_interv=
	this.convexity_interv2=
	this.end_of_interval=null;

	this.dspread=Double.NaN;

	// clear internal buffers
	this.allshifts.clearAll();
	this.obuy.clear();
	this.osell.clear();
    }


    void calculateVirtualShifts(final long [] delays,
				final long currenttime,
				final double pcurrent_buy,
				final double pcurrent_sell,
				final double max_shift,
				final long trades_cleared_before)
    {

	final int tsize=this.trades.getSize();
	final Trade [] tb=this.tmptradesbuf.getBuffer();
	System.arraycopy(this.trades.getBuffer(),0,tb,0,tsize);
	this.dspread=pcurrent_sell-pcurrent_buy;


	final boolean PRINT_DATA_DEBUG=false;//true;
	//currenttime==35308657000000L;
	//currenttime==32894696000000L;
	//currenttime==34580422000000L;
	//currenttime>=9*3600*TraderInterface.TIME_SCALE && tsize>0;


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+"============Vritual Shifts t=\t"+currenttime+
			       "\tTRADES\t"+tsize+
			       "\tn_buy\t"+this.buy.getSize()+
			       "\tn_sell\t"+this.sell.getSize()+
			       "\tpcurrent_buy\t"+pcurrent_buy+
			       "\tpcurrent_sell\t"+pcurrent_sell);


	    System.err.println("num\t"+
			       "time\t"+
			       "shares\t"+
			       "tradetype\t"+
			       "buy_sell_indicator\t"+
			       "price\t"+
			       "matchnumber\t"+
			       "orderreferencenumber"+
			       "\t\t"+
			       "odiff\t"+
			       "otime\t"+
			       "oshares\t"+
			       "oprice");
	}


	// init arrays
	final int nt=delays.length;
	this.tot_minprice=new double [nt];
	this.tot_maxprice=new double [nt];
	this.trade_tot_median_low_tau=new double [nt];
	this.trade_tot_median_high_tau=new double [nt];
	this.max_convexity=new double [nt];
	this.v_tot_alltrades=new long [nt];
	this.v_tot_hiddentrades=new long [nt];
	this.convexity_interv=new MatchOrdersVariousShifts.MatchWithShift [nt];
	this.convexity_interv2=new MatchOrdersVariousShifts.MatchWithShift [nt];
	this.end_of_interval=new MatchOrdersVariousShifts.MatchWithShift [nt];

	final int [] tau_interval_range_ptr=new int [nt];
	for(int i=0;i<nt;i++)
	{
	    // flag not set
	    tau_interval_range_ptr[i]=-1;
	    this.tot_minprice[i]=
	    this.tot_maxprice[i]=
	    this.trade_tot_median_low_tau[i]=
	    this.trade_tot_median_high_tau[i]=
	    this.max_convexity[i]=Double.NaN;
	}


	if(true)
	{ // calculate volume
	    long tbefore=currenttime-delays[0];
	    long prevt=Long.MAX_VALUE;
	    gloop:for(int i=tsize,cur_pos=0;i-- >0;)
	    {
		final Trade t=tb[i];
		if(t.time>prevt)
		{
		    throw new RuntimeException("The data is not sorted:"+
					       prevt+" and "+t.time+
					       " currenttime="+currenttime);
		}
		prevt=t.time;
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.shares+"\t"+
				       t.tradetype+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.price+"\t"+
				       t.matchnumber+"\t"+
				       t.orderreferencenumber+
				       (t.order!=null?
					"\t\t"+(t.time-t.order.time)+"\t"+
					t.order.time+"\t"+
					t.order.shares+"\t"+
					t.order.price:
					"\t\t-10\t\t\t"));
		}
		for(;!(t.time>=tbefore);)
		{
		    if(++cur_pos>=nt)
		    {
			break gloop;
		    }
		    tbefore=currenttime-delays[cur_pos];
		    final int i1=cur_pos-1;
		    this.v_tot_alltrades[cur_pos]=this.v_tot_alltrades[i1];
		    this.v_tot_hiddentrades[cur_pos]=this.v_tot_hiddentrades[i1];
		}
		if(/* tbefore>0L && */tbefore<trades_cleared_before)
		{
		    throw new RuntimeException("Something is terrible wrong with old data removal"+
					       " currenttime="+currenttime);
		}

		// volume
		this.v_tot_alltrades[cur_pos]+=t.shares;
		if(t.order==null)
		{
		    this.v_tot_hiddentrades[cur_pos]+=t.shares;
		}
		tau_interval_range_ptr[cur_pos]=i;
	    }
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("tau\t"+
				   "vol_all\t"+
				   "vol_hidden\t");
		for(int i=0;i<nt;i++)
		{
		  System.err.println(String.valueOf(delays[i])+"\t"+
				     this.v_tot_alltrades[i]+"\t"+
				     this.v_tot_hiddentrades[i]+"\t");
		}
	    }
	    // set to previuos, if current not available
	    for(int i=1;i<nt;i++)
	    {
		if(tau_interval_range_ptr[i]<0)
		{
		    final int i1=i-1;
		    this.v_tot_alltrades[i]=this.v_tot_alltrades[i1];
		    this.v_tot_hiddentrades[i]=this.v_tot_hiddentrades[i1];
		}
	    }
	} // END calculate volume


	// set max time before we sort buffer
	final long tnotafter=tsize>0?tb[tsize-1].time:-1L;
	// calculate median in price space for slices in time space
	for(int cur_pos=0;cur_pos<nt;cur_pos++)
	{
	    final int ptr_start=tau_interval_range_ptr[cur_pos];
	    if(ptr_start<0)
	    {
		// no points at all for this interval
		continue;
	    }
	    // clear buffers
	    this.obuy.clear();
	    this.osell.clear();
	    this.allshifts.clearAll();
	    final long tnotbefore=currenttime-delays[cur_pos];
	    if(true)
	    {
		// order array by price.
		// the buffer is considered poisoned after this.
		com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(tb,ptr_start,tsize-1,CompareItems.cmp_by_price_only);
		final long vtotal=this.v_tot_alltrades[cur_pos];
		this.tot_minprice[cur_pos]=tb[tsize-1].price;
		this.tot_maxprice[cur_pos]=tb[ptr_start].price;

		final long lev=vtotal/2;//(long)(0.5*vtotal);

		long vol=0,prevvol=0;
		double prev_price=Double.NaN;
		for(int i=ptr_start;i<tsize;i++)
		{
		    final Trade t=tb[i];
		    final double p=t.price;
		    if(prev_price<p || !(tnotbefore<=t.time && t.time<=tnotafter)) // will be false for NaN
		    {
			throw new RuntimeException("Ordering is broken"+
						   " currenttime="+currenttime);
		    }
		    prev_price=p;
		    if(0>(vol+=t.shares))
		    {
			throw new RuntimeException("Integer overflow currenttime="+currenttime);
		    }
		    if(vol>lev && lev>=prevvol)
		    {
			final int ji=i;
			int je=-1;
			long vx=vol;
			for(int jo=ji;jo>=ptr_start;jo--)
			{
			    vx-=tb[jo].shares;
			    if((vtotal-vx)>lev)
			    {
				je=jo;
				break;
			    }
			}
			if(je<0)
			{
			    throw new IllegalStateException("jinit="+ji+
							    " jend="+je+
							    " vol="+vol+
							    " lev="+lev+
							    " vtotal="+vtotal+
							    " currenttime="+currenttime);
			}

			// ============= VIRTUAL SHIFT MATCHES ====================
			if(pcurrent_sell>=pcurrent_buy) // NaN check
			{
			    prepareVirtualShifts(currenttime,
						 cur_pos,
						 pcurrent_buy,
						 pcurrent_sell,
						 ptr_start,
						 je,
						 tsize,
						 ji,
						 tb,
						 PRINT_DATA_DEBUG,
						 vtotal-vol+tb[ji].shares,
						 vx+tb[je].shares);

			    // perform all possible virtual shifts
			    // and save the results in this.allshifts
			    this.virtualshifts.matchWithVariousShifts(this.obuy,this.osell,this.allshifts);

			    // calculate assymetry
			    processVirtualMatches(currenttime,
						  cur_pos,
						  pcurrent_buy,
						  pcurrent_sell,
						  max_shift,
						  /*this.dspread+(this.tot_maxprice[cur_pos]-this.tot_minprice[cur_pos]),*/
						  PRINT_DATA_DEBUG);
			}
			// ============= END VIRTUAL SHIFT MATCHES ====================

		    }
		    prevvol=vol;
		}
		if(vol!=vtotal)
		{
		    throw new RuntimeException("Broken invariant. This should never happen."+
					       " currenttime="+currenttime);
		}
	    }
	}


	// set to previuos, if current not available
	for(int i=1;i<nt;i++)
	{
	    if(tau_interval_range_ptr[i]<0)
	    {
		final int i1=i-1;
		this.tot_minprice[i]=this.tot_minprice[i1];
		this.tot_maxprice[i]=this.tot_maxprice[i1];
		this.trade_tot_median_low_tau[i]=this.trade_tot_median_low_tau[i1];
		this.trade_tot_median_high_tau[i]=this.trade_tot_median_high_tau[i1];

		this.max_convexity[i]=this.max_convexity[i1];
		this.convexity_interv[i]=this.convexity_interv[i1];
		this.convexity_interv2[i]=this.convexity_interv2[i1];
		this.end_of_interval[i]=this.end_of_interval[i1];
	    }
	}


	// EXPENSIVE CONSTRAINS CHECK
	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    // check volume & medians
	    final CalculateTradeVolume tv=
		new CalculateTradeVolume(this.trades,this.tmptradesbuf);
	    tv.calculateTradeVolume(
		delays,
		currenttime,
		0.5,
		0.92,
		0.5,
		pcurrent_buy,
		pcurrent_sell,
		trades_cleared_before);
	    for(int i=0;i<nt;i++)
	    {
		if(this.v_tot_alltrades[i]!=tv.v_tot_alltrades[i] ||
		   this.v_tot_hiddentrades[i]!=tv.v_tot_hiddentrades[i] ||
		   Double.isNaN(this.tot_minprice[i])!=Double.isNaN(tv.tot_minprice[i]) ||
		   (!Double.isNaN(this.tot_minprice[i]) && this.tot_minprice[i]!=tv.tot_minprice[i]) ||
		   Double.isNaN(this.tot_maxprice[i])!=Double.isNaN(tv.tot_maxprice[i]) ||
		   (!Double.isNaN(this.tot_maxprice[i]) && this.tot_maxprice[i]!=tv.tot_maxprice[i]) ||
		   Double.isNaN(this.trade_tot_median_low_tau[i])!=Double.isNaN(tv.trade_tot_median_low_tau[i])||
		   (!Double.isNaN(this.trade_tot_median_low_tau[i]) && this.trade_tot_median_low_tau[i]!=tv.trade_tot_median_low_tau[i]) ||
		   Double.isNaN(this.trade_tot_median_high_tau[i])!=Double.isNaN(tv.trade_tot_median_high_tau[i])||
		   (!Double.isNaN(this.trade_tot_median_high_tau[i]) && this.trade_tot_median_high_tau[i]!=tv.trade_tot_median_high_tau[i]))
		{
		    throw new RuntimeException(
			"Data mismatch:"+
			" v_tot_alltrades["+i+"]="+this.v_tot_alltrades[i]+
			" tv.v_tot_alltrades["+i+"]="+tv.v_tot_alltrades[i]+
			" v_tot_hiddentrades["+i+"]="+this.v_tot_hiddentrades[i]+
			" tv.v_tot_hiddentrades["+i+"]="+tv.v_tot_hiddentrades[i]+
			" tot_minprice["+i+"]="+this.tot_minprice[i]+
			" tv.tot_minprice["+i+"]="+tv.tot_minprice[i]+
			" tot_maxprice["+i+"]="+this.tot_maxprice[i]+
			" tv.tot_maxprice["+i+"]="+tv.tot_maxprice[i]+
			" trade_tot_median_low_tau["+i+"]="+this.trade_tot_median_low_tau[i]+
			" tv.trade_tot_median_low_tau["+i+"]="+tv.trade_tot_median_low_tau[i]+
			" trade_tot_median_high_tau["+i+"]="+this.trade_tot_median_high_tau[i]+
			" tv.trade_tot_median_high_tau["+i+"]="+tv.trade_tot_median_high_tau[i]+
			" delay="+delays[i]+
			" currenttime="+currenttime);
		}
	    }
	}
    }



    private void prepareVirtualShifts(final long currenttime,
				      final int cur_pos,
				      final double pcurrent_buy,
				      final double pcurrent_sell,
				      final int ptr_start,
				      final int ptr_jsell,
				      final int tsize,
				      final int ptr_jbuy,
				      final Trade [] tb,
				      final boolean PRINT_DATA_DEBUG,
				      final long vmb,
				      final long vms)
    {
	this.trade_tot_median_low_tau[cur_pos]=tb[ptr_jbuy].price;
	this.trade_tot_median_high_tau[cur_pos]=tb[ptr_jsell].price;

	// save trades as virtual orders to price matcher.
	for(int jo=ptr_jbuy;jo<tsize;jo++)
	{ // decreasing orders, put them to buy
	    final Trade t=tb[jo];
	    this.obuy.addOrderCheckPrev(t.price,t.shares);
	}
	for(int jo=ptr_jsell;jo>=ptr_start;jo--)
	{ // increasing orders, put them to sell
	    final Trade t=tb[jo];
	    this.osell.addOrderCheckPrev(t.price,t.shares);
	}

	this.obuy.shiftPrices(pcurrent_buy-this.trade_tot_median_low_tau[cur_pos]);
	this.osell.shiftPrices(pcurrent_sell-this.trade_tot_median_high_tau[cur_pos]);
	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    // check ordering
	    this.obuy.getTotalVolume();
	    this.osell.getTotalVolume();
	    // check matching
	    if(pcurrent_sell>pcurrent_buy)
	    {
		final MatchOrdersNoShift.MatchResults mtrades=
		    new MatchOrdersNoShift().match(this.obuy,this.osell);
		if(mtrades.v_executed!=0||
		   mtrades.v_buy!=vmb||
		   mtrades.v_sell!=vms)
		{
		    throw new RuntimeException("Mismatching volume="+mtrades.v_executed+
					       " should be 0"+
					       " mtrades.v_buy="+mtrades.v_buy+
					       " vmb="+vmb+
					       " mtrades.v_sell="+mtrades.v_sell+
					       " vms="+vms+
					       " currenttime="+currenttime);
		}
	    }
	}

	// now add buy & sell orders from the book
	final BookItem [] buf_buy=this.buy.getBuffer(),buf_sell=this.sell.getBuffer();
	for(int i=0,n=this.buy.getSize();i<n;i++)
	{
	    final BookItem b=buf_buy[i];
	    this.obuy.addOrderCheckPrev(b.price,b.shares);
	}
	this.obuy.mergeIdenticalPrices();
	for(int i=0,n=this.sell.getSize();i<n;i++)
	{
	    final BookItem b=buf_sell[i];
	    this.osell.addOrderCheckPrev(b.price,b.shares);
	}
	this.osell.mergeIdenticalPrices();

	// all virtual orders are now saved in this.obuy & this.osell
	// we are ready for matching
    }


    private void processVirtualMatches(final long currenttime,
				       final int cur_pos,
				       final double pcurrent_buy,
				       final double pcurrent_sell,
				       final double max_shift,
				       final boolean PRINT_DATA_DEBUG)
    {
	final double ds=this.dspread;
	final int bsize=this.allshifts.getSize();
	final MatchOrdersVariousShifts.MatchWithShift [] ob=this.allshifts.getBuffer();


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("--------------- cur_pos\t"+cur_pos);
	    System.err.println("num\t"+
			       "shift0\t"+
			       "delta0\t"+
			       "shift\t"+
			       "eff_shift\t"+
			       "delta\t"+

			       // median info
			       "ptr_to_median\t"+
			       "eff_med_shift_underest\t"+
			       "delta at med\t"+

			       // extra debug info
			       "type\t"+
			       "pmatch nodp\t"+
			       "dp\t"+

			       // convexity
		               "convexity_ratio\t"+
			       "max_convexity_ratio\t"+
			       "vmatch\t"+
			       "\t");
	}


	long prev_vmatch=-1;
	double prev_shift=-ds,maxconv=0;
        for(int i=0,ptr_to_median=0;i<bsize;i++)
	{
	    final MatchOrdersVariousShifts.MatchWithShift t=ob[i];
	    if(prev_shift>t.shift)
	    {
		throw new RuntimeException("Ordering is broken i="+i+
					   " prev_shift="+prev_shift+
					   " t.shift="+t.shift+
					   " ds="+ds+
					   " currenttime="+currenttime);
	    }
	    prev_shift=t.shift;
	    if(prev_vmatch>t.vmatch)
	    {
		throw new RuntimeException("Incorrect volume ordering i="+i+
					   " prev_vmatch="+prev_vmatch+
					   " t.vmatch="+t.vmatch+
					   " ds="+ds+
					   " currenttime="+currenttime);
	    }
	    prev_vmatch=t.vmatch;
	    final double current_shift=t.shift+ds;
	    if(!(current_shift<=max_shift))
	    {
		// no more accepted shifts or NaN
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println("END\t"+
				       " current_shift="+current_shift+"\t"+
				       " max_shift="+max_shift);
		}
		break;
	    }

	    // current match. use to save as the last match
	    this.end_of_interval[cur_pos]=t;

	    // ----------------------------------------------------------
	    // calculate median for this shift.
	    final long vm=t.vmatch/2;
	    MatchOrdersVariousShifts.MatchWithShift tmed=null;
	    double median=Double.NaN,median_underestimated=Double.NaN;
	    for(int m=ptr_to_median;m<=i;m++)
	    {
		final MatchOrdersVariousShifts.MatchWithShift tm=ob[m];
		if(false && PRINT_DATA_DEBUG)
		{
		    System.err.println(":"+String.valueOf(i)+"\t"+String.valueOf(m)+"\t"+
				       tm.vmatch+"\t"+
				       vm+"\t");
		}
		// overestimated median
		if(tm.vmatch>=vm)
		{
		    ptr_to_median=m;
		    tmed=tm;
		    final double dpm;
		    if(m>0)
		    {
			median_underestimated=ob[m-1].shift+ds;
			dpm=tm.shift-ob[m-1].shift;

		    }
		    else
		    {
			median_underestimated=0;
			dpm=ds;
		    }
		    median=tm.shift+ds+
			// adjustment to overestimation
			((vm-tm.vmatch)*dpm)/(tm.vmatch-(m>0?ob[m-1].vmatch:0));
		    break;
		}
	    }
	    if(tmed==null)
	    {
		throw new RuntimeException("This should never happen");
	    }

	    // maximal convexity
	    final double r=median_underestimated/current_shift;
	    if(r>=maxconv)
	    {
		this.max_convexity[cur_pos]=maxconv=r;
		this.convexity_interv[cur_pos]=tmed;
		this.convexity_interv2[cur_pos]=t;
	    }
	    // ----------------------------------------------------------


	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println(
		    String.valueOf(i)+"\t"+
		    t.shift0+"\t"+
		    t.delta0+"\t"+
		    t.shift+"\t"+
		    current_shift+"\t"+
		    t.delta+"\t"+
		    // median info
		    ptr_to_median+"\t"+
		    median_underestimated+"\t"+
		    tmed.delta+"\t"+
		    // extra debug info
		    t.type+"\t"+
		    t.test_pmatch+"\t"+
		    t.test_dp+"\t"+
		    // convexity
		    r+"\t"+
		    this.max_convexity[cur_pos]+"\t"+
		    t.vmatch+
		    "\t");
	    }

	} // END loop over all shifts
    }
}
