package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Modelling of book virtual shifts. Return of the Jedi ver 3.
 *  @version $Id: CalculateBookVirtualShiftsTau.java,v 1.15 2009/06/22 23:38:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateBookVirtualShiftsTau {


    private final OrdersFlow obuy=new OrdersFlow(true),osell=new OrdersFlow(false);
    private final MatchOrdersVariousShifts virtualshifts=new MatchOrdersVariousShifts();
    private final Distribution<MatchOrdersVariousShifts.MatchWithShift>
	allshifts=new Distribution<MatchOrdersVariousShifts.MatchWithShift>();


    private final Distribution<BookItem> buy,sell;

    CalculateBookVirtualShiftsTau(final Distribution<BookItem> buy,
				  final Distribution<BookItem> sell)
    {
	this.buy=buy;
	this.sell=sell;
    }

    MatchOrdersVariousShifts.MatchWithShift [] match_results_tau=null;
    long [] a_tau_buy=null,a_tau_sell=null,a_tau=null,a_tau_diff=null;
    double dspread=Double.NaN;


    void clear()
    {
	this.match_results_tau=null;
	this.a_tau_buy=this.a_tau_sell=this.a_tau=this.a_tau_diff=null;
	this.dspread=Double.NaN;

	// clear internal buffers
	this.allshifts.clearAll();
	this.obuy.clear();
	this.osell.clear();
    }


    void calculateVirtualShifts(final long currenttime,
				final long [] delays,
				final double pcurrent_buy,
				final double pcurrent_sell,
				final double max_shift)
    {
	final int nt=delays.length;
	// init arrays
	this.match_results_tau=new MatchOrdersVariousShifts.MatchWithShift[nt];
	this.a_tau_buy=new long [nt];
	this.a_tau_sell=new long [nt];
	this.a_tau=new long [nt];
	this.a_tau_diff=new long [nt];
	for(int i=0;i<nt;i++)
	{
	    this.a_tau_buy[i]=
		this.a_tau_sell[i]=
		this.a_tau[i]=-1;
	    this.a_tau_diff[i]=0;
	}
	// copy data
	this.virtualshifts.copyOrderedOrdersToFlow(this.buy,this.obuy);
	this.virtualshifts.copyOrderedOrdersToFlow(this.sell,this.osell);
	this.virtualshifts.matchWithVariousShifts(this.obuy,this.osell,this.allshifts);
	final int bsize=this.allshifts.getSize();
	final MatchOrdersVariousShifts.MatchWithShift [] ob=this.allshifts.getBuffer();
	final BookItem [] buf_buy=this.buy.getBuffer(),buf_sell=this.sell.getBuffer();
	final int n_buy=this.buy.getSize(),n_sell=this.sell.getSize();


	final boolean PRINT_DATA_DEBUG=false;//true;
	//currenttime==35308657000000L;
	//currenttime==32894696000000L;
	//currenttime==34580422000000L;
	//currenttime>=9*3600*TraderInterface.TIME_SCALE && bsize>0;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+"============Vritual Shifts Tau t=\t"+currenttime+
			       "\tN_SHIFTS FOUND\t"+bsize+"\tTRADES\t"+"N/A");
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("num\t"+
			       "shift0\t"+
			       "delta0\t"+
			       "shift\t"+
			       "eff_shift\t"+
			       "delta\t"+

			       // extra debug info
			       "type\t"+
			       "pmatch nodp\t"+
			       "dp\t"+
			       "tau_aver\t"+
			       "taubefore\t"+

			       // volume & tau
			       "vmatch\t"+
			       "a_tau_buy\t"+
			       "a_tau_sell\t"+
			       "a_tau\t"+
			       "a_tau_diff\t"+
			       "cur_pos\t");
	}

	final double ds=this.dspread=pcurrent_sell-pcurrent_buy;
	double mt_prev_shift=-ds;

	long mt_prev_vmatch=-1,vbuy=0,vsell=0,sumbuy=0,sumsell=0;
	long taubefore=delays[0]/ModelParameters.DISTANCE_TIME_SCALE;
        gloop:for(int i=0,cur_pos=0,ptr_buy=0,ptr_sell=0;i<bsize;i++)
	{
	    final MatchOrdersVariousShifts.MatchWithShift mt=ob[i];
	    if(mt_prev_shift>mt.shift)
	    {
		throw new RuntimeException("Ordering is broken i="+i+
					   " mt_prev_shift="+mt_prev_shift+
					   " mt.shift="+mt.shift+
					   " ds="+ds+
					   " currenttime="+currenttime);
	    }
	    mt_prev_shift=mt.shift;
	    if(mt_prev_vmatch>mt.vmatch)
	    {
		throw new RuntimeException("Incorrect volume ordering i="+i+
					   " mt_prev_vmatch="+mt_prev_vmatch+
					   " mt.vmatch="+mt.vmatch+
					   " ds="+ds+
					   " currenttime="+currenttime);
	    }
	    mt_prev_vmatch=mt.vmatch;
	    final double current_shift=mt.shift+ds;
	    if(!(current_shift<=max_shift))
	    {
		// no more accepted shifts or NaN
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       " current_shift="+current_shift+"\t"+
				       " max_shift="+max_shift);
		}
		break gloop;
	    }


	    // calculate weighted time of order placement
	    // median is better, but require sorting, which would slow down here.
	    for(final double pbuy_lev=pcurrent_buy-mt.pobuy-OrdersFlow.EPS;
		ptr_buy<n_buy && buf_buy[ptr_buy].price>=pbuy_lev;ptr_buy++)
	    {
		final BookItem t=buf_buy[ptr_buy];
		vbuy+=t.shares;
		if(0>(sumbuy+=t.shares*((currenttime-t.time)/ModelParameters.DISTANCE_TIME_SCALE)))
		{
		    throw new RuntimeException("Integer overflow currenttime="+currenttime);
		}
	    }
	    for(final double psell_lev=pcurrent_sell+mt.posell+OrdersFlow.EPS;
		ptr_sell<n_sell && buf_sell[ptr_sell].price<=psell_lev;ptr_sell++)
	    {
		final BookItem t=buf_sell[ptr_sell];
		vsell+=t.shares;
		if(0>(sumsell+=t.shares*((currenttime-t.time)/ModelParameters.DISTANCE_TIME_SCALE)))
		{
		    throw new RuntimeException("Integer overflow currenttime="+currenttime);
		}
	    }
	    if(Math.min(vbuy,vsell)!=mt.vmatch ||
	       (ptr_buy>0 && Math.abs(pcurrent_buy-mt.pobuy-buf_buy[ptr_buy-1].price)>OrdersFlow.EPS)||
	       (ptr_sell>0 && Math.abs(pcurrent_sell+mt.posell-buf_sell[ptr_sell-1].price)>OrdersFlow.EPS))
	    {
		throw new RuntimeException("Incorrect volume matching i="+i+
					   " vbuy="+vbuy+
					   " vsell="+vsell+
					   " mt.vmatch="+mt.vmatch+
					   " mt.test_pmatch="+mt.test_pmatch+
					   " buy_price="+buf_buy[ptr_buy-1].price+
					   " sell_price="+buf_sell[ptr_sell-1].price+
					   " ds="+ds+
					   " currenttime="+currenttime);
	    }
	    final long v=vbuy+vsell;
	    // tau_aver typically increases with i, but may be sometimes non-monotonic
	    final long tau_aver=(sumbuy+sumsell)/v;
	    for(;!(tau_aver<=taubefore);)
	    {
		if(++cur_pos>=nt)
		{
		    break gloop;
		}
		taubefore=delays[cur_pos]/ModelParameters.DISTANCE_TIME_SCALE;
	    }
	    this.match_results_tau[cur_pos]=mt;
	    this.a_tau_buy[cur_pos]=sumbuy/vbuy;
	    this.a_tau_sell[cur_pos]=sumsell/vsell;
	    this.a_tau[cur_pos]=tau_aver;
	    this.a_tau_diff[cur_pos]=(sumsell-sumbuy)/v;

	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println(
		    String.valueOf(i)+"\t"+
		    mt.shift0+"\t"+
		    mt.delta0+"\t"+
		    mt.shift+"\t"+
		    current_shift+"\t"+
		    mt.delta+"\t"+

		    // extra debug info
		    mt.type+"\t"+
		    mt.test_pmatch+"\t"+
		    mt.test_dp+"\t"+
		    tau_aver+"\t"+
		    taubefore+"\t"+

		    // volume & tau
		    mt.vmatch+"\t"+
		    this.a_tau_buy[cur_pos]+"\t"+
		    this.a_tau_sell[cur_pos]+"\t"+
		    this.a_tau[cur_pos]+"\t"+
		    this.a_tau_diff[cur_pos]+"\t"+
		    cur_pos+"\t");
	    }
	}

	for(int i=0;i<nt;i++)
	{
	    if(i>0 && this.match_results_tau[i]==null)
	    {
		final int i1=i-1;
		if(this.match_results_tau[i1]!=null)
		{
		    this.match_results_tau[i]=this.match_results_tau[i1];
		    this.a_tau_buy[i]=this.a_tau_buy[i1];
		    this.a_tau_sell[i]=this.a_tau_sell[i1];
		    this.a_tau[i]=this.a_tau[i1];
		    this.a_tau_diff[i]=this.a_tau_diff[i1];
		}
	    }
	    final long tend=delays[i]/ModelParameters.DISTANCE_TIME_SCALE;
	    if(this.a_tau_buy[i]<0)
	    {
		this.a_tau_buy[i]=tend;
	    }
	    if(this.a_tau_sell[i]<0)
	    {
		this.a_tau_sell[i]=tend;
	    }
	    if(this.a_tau[i]<0)
	    {
		this.a_tau[i]=tend;
	    }
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("END");
	}

	if(PRINT_DATA_DEBUG)
	{

	    System.err.println("delay\tshift\tdelta\tshift0\tdelta0\tvol_shift\ta_tau_buy\ta_tau_sell\ta_tau\ta_tau_diff");
	    for(int i=0;i<nt;i++)
	    {
		System.err.print(delays[i]+"\t");
		final MatchOrdersVariousShifts.MatchWithShift mt=this.match_results_tau[i];
		if(mt!=null)
		{
		    System.err.print(mt.shift+"\t"+
				     mt.delta+"\t"+
				     mt.shift0+"\t"+
				     mt.delta0+"\t"+
				     mt.vmatch+"\t");
		}
		else
		{
		    System.err.print("?\t"+
				     "?\t"+
				     "?\t"+
				     "?\t"+
				     "?\t");
		}
		System.err.println(this.a_tau_buy[i]+"\t"+
				   this.a_tau_sell[i]+"\t"+
				   this.a_tau[i]+"\t"+
				   this.a_tau_diff[i]);
	    }
	}
    }
}
