package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;



/** Calculate book volume and median distributions.
 *  Similar to CalculateTradeVolume, but we do not merge them yet,
 *  both classes are still evolving.
 *  @version $Id: CalculateBookVolume.java,v 1.64 2020/02/06 00:09:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateBookVolume {


 private static final boolean
	DEBUG_TEST_PRINTVOLUME=false,
	DEBUG_TEST_PRINTVOLUME_ALLTRADES=false;


    private final Distribution<BookItem> buy,sell,tmpordersbuf;
    CalculateBookVolume(final Distribution<BookItem> buy,
			final Distribution<BookItem> sell,
			final Distribution<BookItem> tmpordersbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.tmpordersbuf=tmpordersbuf;
    }


	long []
	// volumes
	    v_buy_tau=null,
	    v_sell_tau=null;

	int []
	// medians
	    disttofrac_all_buy=null,
	    disttofrac_all_sell=null,
	    disttofrac_all=null;


    double [] book_median_buy_tau=null,book_median_sell_tau=null;

    long vtotal_buy,vtotal_sell;

    void setData(final int nt)
    {
	if(nt<0)
	{
	    this.v_buy_tau=
	    this.v_sell_tau=null;
	    this.disttofrac_all_buy=
	    this.disttofrac_all_sell=
	    this.disttofrac_all=null;
	    this.book_median_buy_tau=this.book_median_sell_tau=null;
	}
	else
	{
	    this.v_buy_tau=new long [nt];
	    this.v_sell_tau=new long [nt];
	    this.disttofrac_all_buy=new int [nt];
	    this.disttofrac_all_sell=new int [nt];
	    this.disttofrac_all=new int [nt];
	    this.book_median_buy_tau=new double [nt];
	    this.book_median_sell_tau=new double [nt];
	}
	this.vtotal_buy=this.vtotal_sell=-1;
    }


    void calculateBookVolume(final long [] delays,
			     final long currenttime,
			     final double frac,
			     final double pcurrent_buy,
			     final double pcurrent_sell,
			     final double price_buy_cutoff,
			     final double price_sell_cutoff)
    {
	if(frac <0.0 || frac>1.0)
	{
	    throw new IllegalArgumentException("frac="+frac);
	}

	final int nt=delays.length;
	setData(nt);

	final int nbuy=this.buy.getSize(),nsell=this.sell.getSize();
	final int tsize=nbuy+nsell;
	final BookItem [] tb=this.tmpordersbuf.getBuffer();
	if(tsize>tb.length)
	{
	    throw new RuntimeException("This should never happen");
	}
	System.arraycopy(this.buy.getBuffer(),0,tb,0,nbuy);
	System.arraycopy(this.sell.getBuffer(),0,tb,nbuy,nsell);



	/* COMMAND:
	  java -server -Xmx6500000K com/polytechnik/trading/GenerateTrainingData /pdata/prices/out_ml/ com/polytechnik/trading/test_intrument_list.txt /rdata/itch/itch3/S072308-v3.zip >/tmp/diag_testCalculateBookVolume
	*/
	final boolean PRINT_DATA_DEBUG=false;
	//currenttime==34344233000000L;
	//currenttime==32894696000000L;
	//currenttime==34580422000000L;
	//currenttime>=9*3600*TraderInterface.TIME_SCALE && tsize>0;


	// order array by time.
	// the buffer is considered poisoned after this.
	com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(tb,0,tsize-1,CompareItems.cmp_by_time);


	if(DEBUG_TEST_PRINTVOLUME||PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+"============BOOK VOLUME t=\t"+currenttime+"\tNPRDERS=\t"+tsize+
			       "\tNBUY=\t"+nbuy+"\tNSELL=\t"+nsell+"\tCUT:\t"+
			       price_buy_cutoff+"\t"+price_sell_cutoff+"\t"+
			       pcurrent_buy+"\t"+pcurrent_sell);
	}
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("num\t"+
			       "time\t"+
			       "shares\t"+
			       "buy_sell_indicator\t"+
			       "price\t");
	}

	final int [] tau_interval_range_ptr=new int [nt];
	for(int i=nt;i-- >0;)
	{
	    // flag not set
	    tau_interval_range_ptr[i]=-1;
	    // -1 as not set
	    this.disttofrac_all_buy[i]=
		this.disttofrac_all_sell[i]=
		this.disttofrac_all[i]=-1;
	    this.book_median_buy_tau[i]=this.book_median_sell_tau[i]=Double.NaN;
	}
	long vc_total_buy=0,vc_total_sell=0;
	int n_out_of_range=0;
	long tbefore=currenttime-delays[0];
	long prevt=Long.MAX_VALUE;
        gloop:for(int i=tsize,cur_pos=0;i-- >0;)
	{
	    final BookItem t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time);
	    }
	    else if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    {
		if(i>0)
		{
		    final int i1=i,i2=i-1;
		    if(tb[i2].time>tb[i1].time)
		    {
			throw new RuntimeException(
			    "Ordering is broken"+
			    " i1="+i1+" t1="+tb[i1].time+
			    " i2="+i2+" t2="+tb[i2].time);
		    }
		    else if(tb[i2].time==tb[i1].time && (tb[i2].orderreferencenumber>tb[i1].orderreferencenumber))
		    {
			throw new RuntimeException(
			    "Ordering is broken"+
			    " i1="+i1+" orderreferencenumber1="+tb[i1].orderreferencenumber+
			    " i2="+i2+" orderreferencenumber2="+tb[i2].orderreferencenumber);
		    }
		}
	    }
	    prevt=t.time;
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println(String.valueOf(i)+"\t"+
				   t.time+"\t"+
				   t.shares+"\t"+
				   t.buy_sell_indicator+"\t"+
				   t.price);
	    }
	    for(;!(t.time>=tbefore);)
	    {
		if(++cur_pos>=nt)
		{
		    for(int j=i;j>=0;j--)
		    {
			final BookItem tj=tb[j];
			if(tj.buy_sell_indicator==BookItem.BOOK_BUY)
			{
			    vc_total_buy+=tj.shares;
			}
			else if(tj.buy_sell_indicator==BookItem.BOOK_SELL)
			{
			    vc_total_sell+=tj.shares;
			}
			else
			{
			    throw new IllegalStateException();
			}
		    }
		    break gloop;
		}
		tbefore=currenttime-delays[cur_pos];
	    }
	    if(DEBUG_TEST_PRINTVOLUME_ALLTRADES)
	    {
		System.err.println("t_time="+t.time+" tbefore="+tbefore);
	    }
	    final int s=t.shares;
	    tau_interval_range_ptr[cur_pos]=i;
	    if(t.buy_sell_indicator==BookItem.BOOK_BUY)
	    {
		vc_total_buy+=s;
		if(!(t.price>=price_buy_cutoff))
		{
		    n_out_of_range++;
		    continue;
		}
		this.v_buy_tau[cur_pos]+=s;
	    }
	    else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
	    {
		vc_total_sell+=s;
		if(!(t.price<=price_sell_cutoff))
		{
		    n_out_of_range++;
		    continue;
		}
		this.v_sell_tau[cur_pos]+=s;
	    }
	}
	for(int i=1;i<nt;i++)
	{
	    final int i1=i-1;
	    this.v_buy_tau[i]+=this.v_buy_tau[i1];
	    this.v_sell_tau[i]+=this.v_sell_tau[i1];
	}
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("tau\t"+
			       "v_buy_tau\t"+
			       "v_sell_tau\t\tn_out_of_range\t"+n_out_of_range);
	    for(int i=0;i<nt;i++)
	    {
		System.err.println(String.valueOf(delays[i])+"\t"+
				   this.v_buy_tau[i]+"\t"+
				   this.v_sell_tau[i]+"\t\t");
	    }
	}

	this.vtotal_buy=vc_total_buy;
	this.vtotal_sell=vc_total_sell;

	long vdisttofrac_all_buy=0,
	    vdisttofrac_all_sell=0,

	    prev_vdisttofrac_all_buy=0,
	    prev_vdisttofrac_all_sell=0,
	    prev_vdisttofrac_all=0,

	    lev_disttofrac_all_buy=-1,
	    lev_disttofrac_all_sell=-1,
	    lev_disttofrac_all=-1;

	int
	    ptr_disttofrac_all_buy=-1,
	    ptr_disttofrac_all_sell=-1,
	    ptr_disttofrac_all=-1;

	tbefore=0L;
        gloop2:for(int i=tsize,cur_pos=-1;i-- >0;)
	{
	    final BookItem t=tb[i];
	    for(;cur_pos<0||!(t.time>=tbefore);)
	    {
		if(cur_pos>=0 &&(
		       vdisttofrac_all_buy!=this.v_buy_tau[cur_pos]||
		       vdisttofrac_all_sell!=this.v_sell_tau[cur_pos])
		   )
		{
		    throw new RuntimeException("Broken invariant. This should never happen. currenttime="+currenttime);
		}
		if(++cur_pos>=nt)
		{
		    break gloop2;
		}
		tbefore=currenttime-delays[cur_pos];
	    }
	    if(t.buy_sell_indicator==BookItem.BOOK_BUY)
	    {
		if(!(t.price>=price_buy_cutoff))
		{
		    continue;
		}
		vdisttofrac_all_buy+=t.shares;
	    }
	    else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
	    {
		if(!(t.price<=price_sell_cutoff))
		{
		    continue;
		}
		vdisttofrac_all_sell+=t.shares;
	    }
	    final long vallsum=vdisttofrac_all_buy+vdisttofrac_all_sell;
	    if(vallsum<0||
	       vdisttofrac_all_buy<0||
	       vdisttofrac_all_sell<0)
	    {
		throw new RuntimeException("Integer overflow currenttime="+currenttime);
	    }



	    for(;ptr_disttofrac_all_buy<nt && vdisttofrac_all_buy>lev_disttofrac_all_buy;)
	    {
		if(vdisttofrac_all_buy>lev_disttofrac_all_buy &&
		   prev_vdisttofrac_all_buy<=lev_disttofrac_all_buy)
		{
		    if(ptr_disttofrac_all_buy>=cur_pos)
		    {
			this.disttofrac_all_buy[ptr_disttofrac_all_buy]=i;
		    }
		}
		if(++ptr_disttofrac_all_buy<nt)
		{
		    lev_disttofrac_all_buy=(long)(frac*this.v_buy_tau[ptr_disttofrac_all_buy]);
		}
		else
		{
		    break;
		}
	    }


	    for(;ptr_disttofrac_all_sell<nt && vdisttofrac_all_sell>lev_disttofrac_all_sell;)
	    {
		if(vdisttofrac_all_sell>lev_disttofrac_all_sell &&
		   prev_vdisttofrac_all_sell<=lev_disttofrac_all_sell)
		{
		    if(ptr_disttofrac_all_sell>=cur_pos)
		    {
			this.disttofrac_all_sell[ptr_disttofrac_all_sell]=i;
		    }
		}
		if(++ptr_disttofrac_all_sell<nt)
		{
		    lev_disttofrac_all_sell=(long)(frac*this.v_sell_tau[ptr_disttofrac_all_sell]);
		}
		else
		{
		    break;
		}
	    }


	    for(;ptr_disttofrac_all<nt && vallsum>lev_disttofrac_all;)
	    {
		if(vallsum>lev_disttofrac_all &&
		   prev_vdisttofrac_all<=lev_disttofrac_all)
		{
		    if(ptr_disttofrac_all>=cur_pos)
		    {
			this.disttofrac_all[ptr_disttofrac_all]=i;
		    }
		}
		if(++ptr_disttofrac_all<nt)
		{
		    lev_disttofrac_all=(long)(frac*
					     (this.v_buy_tau[ptr_disttofrac_all]+
					      this.v_sell_tau[ptr_disttofrac_all])
					     );
		}
		else
		{
		    break;
		}
	    }

	    prev_vdisttofrac_all_buy=vdisttofrac_all_buy;
	    prev_vdisttofrac_all_sell=vdisttofrac_all_sell;
	    prev_vdisttofrac_all=vallsum;
	}

	    // constrains (incomplete). check possible misses.
	    for(int i=0;i<nt;i++)
	    {
		/*
		  System.err.println("ind_buy="+this.disttofrac_all_buy[i]+
				   " ind_sell="+this.disttofrac_all_sell[i]+
				   " v_buy="+this.v_buy_tau[i]+
				   " v_sell="+this.v_sell_tau[i]);
		*/
		final int i1=i-1;
		if(
		   (this.disttofrac_all_buy[i]<0 &&
		       0<this.v_buy_tau[i]-(i1>=0?this.v_buy_tau[i1]:0))||
		   (this.disttofrac_all_sell[i]<0 &&
		       0<this.v_sell_tau[i]-(i1>=0?this.v_sell_tau[i1]:0))||
		   (this.disttofrac_all[i]<0 &&
		       (this.disttofrac_all_buy[i]>=0 || this.disttofrac_all_sell[i]>=0))
		   )
		{
		    throw new RuntimeException("Some points may be missed. currenttime="+currenttime);
		}
	    }

	    // try to use previous if current one has no data.
	    for(int i=1;i<nt;i++)
	    {
		final int i1=i-1;
		if(this.disttofrac_all_buy[i]<0)
		{
		    this.disttofrac_all_buy[i]=this.disttofrac_all_buy[i1];
		}
		if(this.disttofrac_all_sell[i]<0)
		{
		    this.disttofrac_all_sell[i]=this.disttofrac_all_sell[i1];
		}
		if(this.disttofrac_all[i]<0)
		{
		    this.disttofrac_all[i]=this.disttofrac_all[i1];
		}
	    }


	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("tau\t"+
				   "ind_disttofrac_all_buy\t"+
				   "ind_disttofrac_all_sell\t"+
				   "ind_disttofrac_all\t"
				   );
		for(int i=0;i<nt;i++)
		{
		  System.err.println(String.valueOf(delays[i])+"\t"+
				     this.disttofrac_all_buy[i]+"\t"+
				     this.disttofrac_all_sell[i]+"\t"+
				     this.disttofrac_all[i]);
		}
	    }

	    // set actual values
	    for(int i=0;i<nt;i++)
	    {
		final int di=(int)(delays[i]/ModelParameters.DISTANCE_TIME_SCALE); // we store delay as int

		if(this.disttofrac_all_buy[i]<0)
		{
		    this.disttofrac_all_buy[i]=di;
		}
		else
		{
		    this.disttofrac_all_buy[i]=(int)((currenttime-tb[this.disttofrac_all_buy[i]].time)/ModelParameters.DISTANCE_TIME_SCALE);
		}
		if(this.disttofrac_all_sell[i]<0)
		{
		    this.disttofrac_all_sell[i]=di;
		}
		else
		{
		    this.disttofrac_all_sell[i]=(int)((currenttime-tb[this.disttofrac_all_sell[i]].time)/ModelParameters.DISTANCE_TIME_SCALE);
		}
		if(this.disttofrac_all[i]<0)
		{
		    this.disttofrac_all[i]=di;
		}
		else
		{
		    this.disttofrac_all[i]=(int)((currenttime-tb[this.disttofrac_all[i]].time)/ModelParameters.DISTANCE_TIME_SCALE);
		}

		// constrains check
		if(this.disttofrac_all_buy[i]>di ||
		   this.disttofrac_all_sell[i]>di ||
		   this.disttofrac_all[i]>di)
		{
		    throw new RuntimeException("Broken limit constraints; currenttime="+currenttime);
		}
		final int i1=i-1;
		if(i1>=0 && (
		       this.disttofrac_all_buy[i]<this.disttofrac_all_buy[i1] ||
		       this.disttofrac_all_sell[i]<this.disttofrac_all_sell[i1] ||
		       this.disttofrac_all[i]<this.disttofrac_all[i1]
			    ))
		{
		    throw new RuntimeException("Broken ordering constraints; currenttime="+currenttime);
		}
	    }


	    // calculate median in price space for slices in time space
	    for(int cur_pos=0;cur_pos<nt;cur_pos++)
	    {
		final int ptr_start=tau_interval_range_ptr[cur_pos];
		if(ptr_start<0)
		{
		    // no points for this interval
		    // previous interval is the same as this one
		    continue;
		}
		final long tnotafter=currenttime-delays[cur_pos];
		// order array by type and price.
		// the buffer is considered poisoned after this.
		com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(tb,ptr_start,tsize-1,Distribution.cmp_orders_by_price_and_type);
		if(false && PRINT_DATA_DEBUG)
		{
		    System.err.println("tau\t"+delays[cur_pos]+"\t"+cur_pos+"\tsorted by price");
		    System.err.println("num\t"+
				       "time\t"+
				       "shares\t"+
				       "buy_sell_indicator\t"+
				       "price\t");
		    for(int i=ptr_start;i<tsize;i++)
		    {
			final BookItem t=tb[i];
			System.err.println(String.valueOf(i)+"\t"+
				   t.time+"\t"+
				   t.shares+"\t"+
				   t.buy_sell_indicator+"\t"+
				   t.price);
		    }
		}

		// median for buy
		long vol=0,prevvol=0,lev=(long)(frac*this.v_buy_tau[cur_pos]);
		double prev_price=Double.NaN;
		for(int i=ptr_start;i<tsize;i++)
		{
		    final BookItem t=tb[i];
		    if(t.buy_sell_indicator!=BookItem.BOOK_BUY)
		    {
			break;
		    }
		    if(prev_price<t.price || t.time<tnotafter) // will be false for NaN
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    else if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
		    {
			if(i>ptr_start)
			{
			    final int i1=i,i2=i-1;
			    if(tb[i2].price<tb[i1].price)
			    {
				throw new RuntimeException(
				    "Ordering is broken"+
				    " i1="+i1+" p1="+tb[i1].price+
				    " i2="+i2+" p2="+tb[i2].price);
			    }
			    else
			    {
				if(tb[i2].price==tb[i1].price)
				{
				    if(tb[i2].time>tb[i1].time)
				    {
					throw new RuntimeException(
					    "Ordering is broken"+
					    " i1="+i1+" t1="+tb[i1].time+
					    " i2="+i2+" t2="+tb[i2].time);
				    }
				    else if(tb[i2].time==tb[i1].time && (tb[i2].orderreferencenumber>tb[i1].orderreferencenumber))
				    {
					throw new RuntimeException(
					    "Ordering is broken"+
					    " i1="+i1+" orderreferencenumber1="+tb[i1].orderreferencenumber+
					    " i2="+i2+" orderreferencenumber2="+tb[i2].orderreferencenumber);
				    }
				}
			    }
			}
		    }
		    prev_price=t.price;
		    if(!(t.price>=price_buy_cutoff))
		    {
			// typically should never happen
			continue;
		    }
		    vol+=t.shares;
		    if(vol<0)
		    {
			throw new RuntimeException("Integer overflow currenttime="+currenttime);
		    }
		    if(vol>lev && lev>=prevvol)
		    {
			this.book_median_buy_tau[cur_pos]=t.price;
			// we can add break here.
			// do not do this to check constrain later
		    }
		    prevvol=vol;
		}
		if(vol!=this.v_buy_tau[cur_pos])
		{
		    throw new RuntimeException("Broken invariant. This should never happen. currenttime="+currenttime);
		}

		// median for sell
		vol=prevvol=0;
		lev=(long)(frac*this.v_sell_tau[cur_pos]);
		prev_price=Double.NaN;
		for(int i=tsize-1;i>=ptr_start;i--)
		{
		    final BookItem t=tb[i];
		    if(t.buy_sell_indicator!=BookItem.BOOK_SELL)
		    {
			break;
		    }
		    if(prev_price>t.price || t.time<tnotafter) // will be false for NaN
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    else if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
		    {
			if(i<tsize-1)
			{
			    final int i1=i,i2=i+1;
			    if(tb[i2].price>tb[i1].price)
			    {
				throw new RuntimeException(
				    "Ordering is broken"+
				    " i1="+i1+" p1="+tb[i1].price+
				    " i2="+i2+" p2="+tb[i2].price);
			    }
			    else
			    {
				if(tb[i2].price==tb[i1].price)
				{
				    if(tb[i2].time>tb[i1].time)
				    {
					throw new RuntimeException(
					    "Ordering is broken"+
					    " i1="+i1+" t1="+tb[i1].time+
					    " i2="+i2+" t2="+tb[i2].time);
				    }
				    else if(tb[i2].time==tb[i1].time && (tb[i2].orderreferencenumber>tb[i1].orderreferencenumber))
				    {
					throw new RuntimeException(
					    "Ordering is broken"+
					    " i1="+i1+" orderreferencenumber1="+tb[i1].orderreferencenumber+
					    " i2="+i2+" orderreferencenumber2="+tb[i2].orderreferencenumber);
				    }
				}
			    }
			}
		    }
		    prev_price=t.price;
		    if(!(t.price<=price_sell_cutoff))
		    {
			// typically should never happen
			continue;
		    }
		    vol+=t.shares;
		    if(vol<0)
		    {
			throw new RuntimeException("Integer overflow currenttime="+currenttime);
		    }
		    if(vol>lev && lev>=prevvol)
		    {
			this.book_median_sell_tau[cur_pos]=t.price;
			// we can add break here.
			// do not do this to check constrain later
		    }
		    prevvol=vol;
		}
		if(vol!=this.v_sell_tau[cur_pos])
		{
		    throw new RuntimeException("Broken invariant. This should never happen. currenttime="+currenttime);
		}
	    }

            // check previous.
	    for(int i=0;i<nt;i++)
	    {
		final int i1=i-1;
		final long vb=this.v_buy_tau[i]-(i1>=0?this.v_buy_tau[i1]:0),
		    vs=this.v_sell_tau[i]-(i1>=0?this.v_sell_tau[i1]:0);
		if(tau_interval_range_ptr[i]<0 && (
		       !Double.isNaN(this.book_median_buy_tau[i])||
		       !Double.isNaN(this.book_median_sell_tau[i])||
		       vb>0||vs>0))
		{
		    throw new RuntimeException("Broken constrains");
		}
		if(i1>=0)
		{
		    // we did not calculate intervals if they have no points.
		    // use previously calculated intervals
		    if(Double.isNaN(this.book_median_buy_tau[i])&&!Double.isNaN(this.book_median_buy_tau[i1]))
		    {
			this.book_median_buy_tau[i]=this.book_median_buy_tau[i1];
		    }
		    if(Double.isNaN(this.book_median_sell_tau[i])&&!Double.isNaN(this.book_median_sell_tau[i1]))
		    {
			this.book_median_sell_tau[i]=this.book_median_sell_tau[i1];
		    }
		}
	    }
	    for(int i=0;i<nt;i++)
	    {
		if(Double.isNaN(this.book_median_buy_tau[i])!=(this.v_buy_tau[i]==0))
		{
		     throw new RuntimeException("buy_vol."+delays[i]+"="+this.v_buy_tau[i]+
						" should be 0; currenttime="+currenttime);
		}
		if(Double.isNaN(this.book_median_sell_tau[i])!=(this.v_sell_tau[i]==0))
		{
		    throw new RuntimeException("sell_vol."+delays[i]+"="+this.v_sell_tau[i]+
					       " should be 0; currenttime="+currenttime);
		}
	    }


	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("tau\t"+
				   "vol_buy\t"+
				   "vol_sell\t"+
				   "vol_total\t"+
				   "disttofrac_all_buy\t"+
				   "disttofrac_all_sell\t"+
				   "disttofrac_all\t"+
				   "\t"+
				   "median_buy\t"+
				   "median_sell\t");
		for(int i=0;i<nt;i++)
		{
		  System.err.println(String.valueOf(delays[i])+"\t"+
				     this.v_buy_tau[i]+"\t"+
				     this.v_sell_tau[i]+"\t"+
				     (this.v_buy_tau[i]+this.v_sell_tau[i])+"\t"+
				     this.disttofrac_all_buy[i]+"\t"+
				     this.disttofrac_all_sell[i]+"\t"+
				     this.disttofrac_all[i]+"\t"+
				     "\t"+
				     this.book_median_buy_tau[i]+"\t"+
				     this.book_median_sell_tau[i]);
		}
	    }
    }

}
