package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** A class to make calculate volume for fixed thresholds.
 *  @version $Id: CalculateBookVolumePriceFixedThreshold.java,v 1.28 2009/09/21 23:52:28 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateBookVolumePriceFixedThreshold
{

    long [] vol_threshold_vcalc=null,tau_threshold_vcalc=null;
    double [] paver=null,plast=null;


    private final boolean flag_lower;
    private final Distribution<BookItem> book;
    CalculateBookVolumePriceFixedThreshold(final boolean flag_lower,
					   final Distribution<BookItem> book)
    {
	this.flag_lower=flag_lower;
	this.book=book;
    }


    void clearData()
    {
	this.vol_threshold_vcalc=this.tau_threshold_vcalc=null;
	this.paver=this.plast=null;
    }


    void calculateThresholds(final long currenttime,
			     final double pcurrent_buy,
			     final double pcurrent_sell,
			     final double [] price_threshold_vcalc)
    {

	final int tsize=this.book.getSize();

	final boolean PRINT_DATA_DEBUG=false;
            //currenttime==33942596000000L;
	    //currenttime==33942191000000L;
	    //currenttime>=10*3600*TraderInterface.TIME_SCALE && tsize>100 && price_threshold_vcalc.length==1;

	final BookItem [] tb=this.book.getBuffer();
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+"============BOOK VOLUME THRESHOLDS t=\t"+currenttime+"\t NORDERS=\t"+tsize+
			       "\tp_buy\t"+pcurrent_buy+"\tp_sell\t"+pcurrent_sell+
			       "\tflower\t"+this.flag_lower);
	}
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("num\t"+
			       "time\t"+
			       "shares\t"+
			       "buy_sell_indicator\t"+
			       "price\t"+
			       "level\t"+
			       "cur_pos");
	}

	if(tsize>0&&tb[0].price!=(this.flag_lower?pcurrent_buy:pcurrent_sell))
	{
	    throw new RuntimeException("Something is wrong with price levels:"+
				       " tb[0].price="+tb[0].price+
				       " pcurrent_buy="+pcurrent_buy+
				       " pcurrent_sell="+pcurrent_sell+
				       " flag_lower="+this.flag_lower);
	}

	double lev=Double.NaN,prev_price=Double.NaN;
	final int nt=price_threshold_vcalc.length;
	this.vol_threshold_vcalc=new long [nt];
	this.tau_threshold_vcalc=new long [nt];
	this.paver=new double [nt];
	this.plast=new double [nt];
	int cur_pos=-1;
        gloop:for(int i=0;i<tsize;i++)
	{
	    final BookItem t=tb[i];
	    final double p=t.price;
	    // check for poison
	    if(this.flag_lower?(prev_price<p||t.buy_sell_indicator!=BookItem.BOOK_BUY):
	       (prev_price>p||t.buy_sell_indicator!=BookItem.BOOK_SELL))
	    {
		throw new RuntimeException("Ordering is broken");
	    }
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.print(String.valueOf(i)+"\t"+
				 t.time+"\t"+
				 t.shares+"\t"+
				 t.buy_sell_indicator+"\t"+
				 p);
	    }
	    for(;!(this.flag_lower?(p>=lev):(p<=lev));)
	    {
		if(cur_pos>=0)
		{
		    this.plast[cur_pos]=prev_price;
		}
		if(++cur_pos>=nt)
		{
		    if(PRINT_DATA_DEBUG)
		    {
			System.err.println("\t\t");
		    }
		    break gloop;
		}
		lev=price_threshold_vcalc[cur_pos];
		if(cur_pos>0)
		{
		    this.paver[cur_pos]=this.paver[cur_pos-1];
		    if(this.flag_lower?
		       (price_threshold_vcalc[cur_pos]>price_threshold_vcalc[cur_pos-1]):
		       (price_threshold_vcalc[cur_pos]<price_threshold_vcalc[cur_pos-1]))
		    {
			throw new RuntimeException("Not sorted price_threshold_vcalc. currenttime="+currenttime);
		    }
		}
	    }
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("\t"+lev+"\t"+cur_pos);
	    }
	    final long tau=(currenttime-t.time)/ModelParameters.T_SUMM_SCALE;
	    this.vol_threshold_vcalc[cur_pos]+=t.shares;
	    this.tau_threshold_vcalc[cur_pos]+=tau*t.shares;
	    this.paver[cur_pos]+=t.price*t.shares;
	    prev_price=p;
	}
	for(int c0=cur_pos;cur_pos<nt;cur_pos++)
	{
	    if(cur_pos>=0)
	    {
		this.paver[cur_pos]=c0>=0?this.paver[c0]:Double.NaN;
		this.plast[cur_pos]=prev_price;
	    }
	}
	for(int i=1;i<nt;i++)
	{
	    final int i1=i-1;
	    this.vol_threshold_vcalc[i]+=this.vol_threshold_vcalc[i1];
	    this.tau_threshold_vcalc[i]+=this.tau_threshold_vcalc[i1];
	}
	for(int i=0;i<nt;i++)
	{
	    if(this.vol_threshold_vcalc[i]>0)
	    {
		this.tau_threshold_vcalc[i]=
		    (this.tau_threshold_vcalc[i]/this.vol_threshold_vcalc[i])
		    *ModelParameters.T_SUMM_SCALE;
		this.paver[i]=this.paver[i]/this.vol_threshold_vcalc[i];
	    }
	    else
	    {
		this.tau_threshold_vcalc[i]=-1;
		this.paver[i]=Double.NaN;
	    }
	}
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("tau");
	    System.err.println();
	    System.err.println("i\tp0\tlev_actual\tvol_shares\ttau\taver\tflower");
	    for(int i=0;i<nt;i++)
	    {
		System.err.println(String.valueOf(i)+"\t"+
				   (tsize>0?tb[0].price:Double.NaN)+"\t"+
				   price_threshold_vcalc[i]+"\t"+
				   this.vol_threshold_vcalc[i]+"\t"+
				   this.tau_threshold_vcalc[i]+"\t"+
				   this.paver[i]+"\t"+
				   this.flag_lower);
	    }
	}
    }
}
