package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** A class to make total distribution of orders.
 *  @version $Id: CalculateBookVolumePriceThreshold.java,v 1.93 2009/07/09 21:39:39 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateBookVolumePriceThreshold
{

    private static final double BETA_FARMER_ZLOVKO=1.5,BETA_STD=1.0;
    static final double BETA=BETA_STD;


    int [] n_orders=null;
    long [] out_shares=null;
    double [] book_median=null,book_d2=null;
    int [] levels_interval_range_ptr=null,ind_to_median_ptr=null;
    long vtotalorders;
    double [] pc_r=null;


    private final boolean flag_lower;
    private final Distribution<BookItem> book;
    CalculateBookVolumePriceThreshold(final boolean flag_lower,
				      final Distribution<BookItem> book)
    {
	this.flag_lower=flag_lower;
	this.book=book;
    }


    void setData(final int nt)
    {
	if(nt<0)
	{
	    this.out_shares=null;
	    this.book_median=this.book_d2=this.pc_r=null;
	    this.n_orders=this.levels_interval_range_ptr=this.ind_to_median_ptr=null;
	}
	else
	{
	    this.n_orders=new int [nt];
	    this.out_shares=new long [nt];
	    this.book_median=new double [nt];
	    this.book_d2=null;
	    this.levels_interval_range_ptr=new int [nt];
	    this.ind_to_median_ptr=new int [nt];
	    this.pc_r=new double[nt];
	}
	this.vtotalorders=-1;
    }


    void calculateThresholds(final double [] levels,
			     final long currenttime,
			     final double frac,
			     final double pcurrent_buy,
			     final double pcurrent_sell)
    {

	if(frac<0.0 || frac>1.0)
	{
	    throw new IllegalArgumentException("frac="+frac);
	}
	final int tsize=this.book.getSize();

	final boolean PRINT_ONLY_SUMMED_BY_PRICE=false;
	final boolean PRINT_DATA_DEBUG=false;
	    //currenttime==33942596000000L;
	    //currenttime==33942191000000L;
	    //currenttime>=10*3600*TraderInterface.TIME_SCALE && tsize>0;

	final int nt=levels.length;
	setData(nt);
	final long [] med_shares=new long [nt];
	for(int i=nt;i-->0;)
	{
	    this.pc_r[i]=Double.NaN;
	    // -1 as not set
	    this.levels_interval_range_ptr[i]=this.ind_to_median_ptr[i]=-1;
	    this.pc_r[i]=this.book_median[i]=Double.NaN;
	    med_shares[i]=-1;
	}
	final BookItem [] tb=this.book.getBuffer();
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+"============BOOK VOLUME t=\t"+currenttime+"\t NORDERS=\t"+tsize+
			       "\tp_buy\t"+pcurrent_buy+"\tp_sell\t"+pcurrent_sell+"\tfrac\t"+frac+
			       "\tflower\t"+this.flag_lower);
	}
	if(PRINT_DATA_DEBUG)
	{
	    if(PRINT_ONLY_SUMMED_BY_PRICE)
	    {
		System.err.println("num\t"+
				   "shares\t"+
				   "shares_sum\t"+
				   "price\t"+
				   "ln price_diff\t"+
				   "f\t"+
				   "cur_pos");
	    }
	    else
	    {
		System.err.println("num\t"+
				   "time\t"+
				   "shares\t"+
				   "buy_sell_indicator\t"+
				   "price\t"+
				   "level\t"+
				   "cur_pos");
	    }
	}

	if(tsize>0&&tb[0].price!=(this.flag_lower?pcurrent_buy:pcurrent_sell))
	{
	    throw new RuntimeException("Something is wrong with price levels:"+
				       " tb[0].price="+tb[0].price+
				       " pcurrent_buy="+pcurrent_buy+
				       " pcurrent_sell="+pcurrent_sell+
				       " flag_lower="+this.flag_lower);
	}


	double lev=Double.NaN,prev_price=Double.NaN,fmax=Double.NEGATIVE_INFINITY;
	long vremaining=0,volsamep=0,volrprocessed=0;
	int last_cur_pos=-1;
        gloop:for(int i=0,cur_pos=-1;i<tsize;i++)
	{
	    final BookItem t=tb[i];
	    final double p=t.price;
	    // check for poison
	    if(this.flag_lower?(prev_price<p||t.buy_sell_indicator!=BookItem.BOOK_BUY):
	       (prev_price>p||t.buy_sell_indicator!=BookItem.BOOK_SELL))
	    {
		throw new RuntimeException("Ordering is broken");
	    }
	    if(PRINT_DATA_DEBUG && !PRINT_ONLY_SUMMED_BY_PRICE)
	    {
		System.err.print(String.valueOf(i)+"\t"+
				 t.time+"\t"+
				 t.shares+"\t"+
				 t.buy_sell_indicator+"\t"+
				 p);
	    }
	    for(;!(this.flag_lower?(p>=lev):(p<=lev));)
	    {
		if(++cur_pos>=nt)
		{
		    if(PRINT_DATA_DEBUG && !PRINT_ONLY_SUMMED_BY_PRICE)
		    {
			System.err.println("\t\t");
		    }
		    // calculate the remaining volume
		    for(int j=i;j<tsize;j++)
		    {
			vremaining+=tb[j].shares;
		    }
		    break gloop;
		}
		lev=levels[cur_pos];
	    }
	    if(PRINT_DATA_DEBUG && !PRINT_ONLY_SUMMED_BY_PRICE)
	    {
		System.err.println("\t"+lev+"\t"+cur_pos);
	    }
	    this.out_shares[cur_pos]+=t.shares;
	    this.n_orders[cur_pos]++;
	    this.levels_interval_range_ptr[cur_pos]=i;
	    if(!(prev_price==p) && volsamep>0)
	    {
		volrprocessed+=volsamep;
		final double f=Math.log(volrprocessed)-BETA*(this.flag_lower?Math.log(pcurrent_sell-prev_price):Math.log(prev_price-pcurrent_buy));
		if(f>fmax)
		{
		    fmax=f;
		    this.pc_r[last_cur_pos]=prev_price;
		}
		if(PRINT_DATA_DEBUG && PRINT_ONLY_SUMMED_BY_PRICE)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       volsamep+"\t"+
				       volrprocessed+"\t"+
				       prev_price+"\t"+
				       (this.flag_lower?Math.log(pcurrent_sell-prev_price):Math.log(prev_price-pcurrent_buy))+"\t"+
				       f+"\t"+
				       last_cur_pos);
		}
		volsamep=t.shares;
	    }
	    else
	    {
		volsamep+=t.shares;
	    }
	    prev_price=p;
	    last_cur_pos=cur_pos;
	} // END gloop
	if(volsamep>0)
	{
	    volrprocessed+=volsamep;
	    final double f=Math.log(volrprocessed)-BETA*(this.flag_lower?Math.log(pcurrent_sell-prev_price):Math.log(prev_price-pcurrent_buy));
	    if(f>fmax)
	    {
		fmax=f;
		this.pc_r[last_cur_pos]=prev_price;
	    }
	    if(PRINT_DATA_DEBUG && PRINT_ONLY_SUMMED_BY_PRICE)
	    {
		System.err.println("N/A\t"+
				   volsamep+"\t"+
				   volrprocessed+"\t"+
				   prev_price+"\t"+
				   (this.flag_lower?Math.log(pcurrent_sell-prev_price):Math.log(prev_price-pcurrent_buy))+"\t"+
				   f+"\t"+
				   last_cur_pos);
	    }
	    volsamep=0;
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("tau");
	}


	for(int i=1;i<nt;i++)
	{
	    final int i1=i-1;
	    this.n_orders[i]+=this.n_orders[i1];
	    this.out_shares[i]+=this.out_shares[i1];
	    if(Double.isNaN(this.pc_r[i]))
	    {
		this.pc_r[i]=this.pc_r[i1];
	    }
	}
	this.vtotalorders=vremaining+this.out_shares[nt-1];
	if(volrprocessed!=this.out_shares[nt-1])
	{
	    throw new IllegalStateException();
	}

	long vol=0,prev_vol=0,lev_vol=-1;
	int ptr_med=-1;
	lev=Double.NaN;
        gloop2:for(int i=0,cur_pos=-1;i<tsize;i++)
	{
	    final BookItem t=tb[i];
	    for(;!(this.flag_lower?(t.price>=lev):(t.price<=lev));)
	    {
		if(cur_pos>=0 &&
		   (vol!=this.out_shares[cur_pos] ||
		    (this.levels_interval_range_ptr[cur_pos]<0? (this.out_shares[cur_pos]-(cur_pos>0?this.out_shares[cur_pos-1]:0))>0 : (i-1)!=this.levels_interval_range_ptr[cur_pos]))
		   )
		{
		    throw new RuntimeException("Broken invariant. This should never happen. currenttime="+currenttime+
					       " vol="+vol+" this.out_shares["+cur_pos+"]="+
					       this.out_shares[cur_pos]+
					       " i-1="+(i-1)+" this.levels_interval_range_ptr["+cur_pos+"]="+this.levels_interval_range_ptr[cur_pos]
					       );
		}
		if(++cur_pos>=nt)
		{
		    break gloop2;
		}
		lev=levels[cur_pos];
	    }
	    vol+=t.shares;
	    if(vol<0)
	    {
		throw new RuntimeException("Integer overflow currenttime="+currenttime);
	    }

	    for(;ptr_med<nt && vol>lev_vol;)
	    {
		if(vol>lev_vol && prev_vol<=lev_vol)
		{
		    if(ptr_med>=cur_pos)
		    {
			this.ind_to_median_ptr[ptr_med]=i;
			med_shares[ptr_med]=vol;
		    }
		}
		if(++ptr_med<nt)
		{
		    lev_vol=(long)(frac*this.out_shares[ptr_med]);
		}
		else
		{
		    break;
		}
	    }
	    prev_vol=vol;
	} // END gloop2


	// constrains (incomplete). check possible misses.
	for(int i=0;i<nt;i++)
	{
	    final int i1=i-1;
	    if((this.ind_to_median_ptr[i]<0 || this.levels_interval_range_ptr[i]<0)&&
	       0<(this.out_shares[i]-(i1>=0?this.out_shares[i1]:0)))
	    {
		throw new RuntimeException("Some points may be missed. currenttime="+currenttime);
	    }
	}
	// try to use previous if current one has no data.
	for(int i=1;i<nt;i++)
	{
	    final int i1=i-1;
	    if(this.ind_to_median_ptr[i]<0)
	    {
		this.ind_to_median_ptr[i]=this.ind_to_median_ptr[i1];
		med_shares[i]=med_shares[i1];
	    }
	    if(this.levels_interval_range_ptr[i]<0)
	    {
		this.levels_interval_range_ptr[i]=this.levels_interval_range_ptr[i1];
	    }
	}
	// set actual values
	for(int i=0;i<nt;i++)
	{
	    if(this.ind_to_median_ptr[i]<0)
	    {
		// we can get here only if no suitable order found (vol=0)
		this.book_median[i]=Double.NaN;
		if(this.out_shares[i]!=0||med_shares[i]>=0||this.levels_interval_range_ptr[i]>=0)
		{
		    throw new RuntimeException(
			"vol="+vol+" should be 0;"+
			" med_shares["+i+"]="+med_shares[i]+
			" levels_interval_range_ptr["+i+"]="+this.levels_interval_range_ptr[i]+
			" out_shares["+i+"]="+this.out_shares[i]+
			" currenttime="+currenttime);
		}
	    }
	    else
	    {
		if(this.ind_to_median_ptr[i]>this.levels_interval_range_ptr[i])
		{
		    throw new RuntimeException(" this.ind_to_median_ptr[i]>this.levels_interval_range_ptr[i] ");
		}
		this.book_median[i]=tb[this.ind_to_median_ptr[i]].price;
	    }
	    final int i1=i-1;
	    if(i1>=0 && ((this.ind_to_median_ptr[i] < this.ind_to_median_ptr[i1]) ||
			(this.flag_lower?this.book_median[i]>this.book_median[i1]:this.book_median[i]<this.book_median[i1]))
	       )
	    {
		throw new RuntimeException("Broken limit constraints;"+
					   " ind_to_median_ptr["+i+"]="+this.ind_to_median_ptr[i]+
					   " ind_to_median_ptr["+i1+"]="+this.ind_to_median_ptr[i1]+
					   " book_median["+i+"]="+this.book_median[i]+
					   " book_median["+i1+"]="+this.book_median[i1]+
					   " currenttime="+currenttime);
	    }
	}

	this.book_d2=calculateD2(tsize,
				 tb,
				 med_shares,
				 this.ind_to_median_ptr,
				 this.out_shares,
				 this.levels_interval_range_ptr);

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("lev\tlev_actual\tind_median\tmedian_price\tD2\tvol_shares\tn_shares\tPc_r\tflower");
	    for(int i=0;i<nt;i++)
	    {
		System.err.println(
		    (this.flag_lower?pcurrent_buy/levels[i]:levels[i]/pcurrent_sell)+"\t"+
		    String.valueOf(levels[i])+"\t"+
		    this.ind_to_median_ptr[i]+"\t"+
		    this.book_median[i]+"\t"+
		    this.book_d2[i]+"\t"+
		    this.out_shares[i]+"\t"+
		    this.n_orders[i]+"\t"+
		    this.pc_r[i]+"\t"+
		    this.flag_lower);
	    }
	}
    }


    private static double [] calculateD2(final int tsize,
					 final BookItem [] tb,
					 final long [] med_shares,
					 final int [] ptr_to_med,
					 final long [] out_shares,
					 final int [] ptr_to_last)
    {

	long m0_shares=0;
	final double p0;
	if(tsize>0)
	{
	    p0=tb[0].price;
	    for(int p=0;p<tsize;p++)
	    {
		if(tb[p].price==p0)
		{
		    m0_shares+=tb[p].shares;
		}
		else
		{
		    break;
		}
	    }
	    if(m0_shares<=0)
	    {
		throw new IllegalArgumentException();
	    }
	}
	else
	{
	    p0=Double.NaN;
	}

	final int nt=med_shares.length;
	final double [] d2=new double [nt];
	for(int i=0;i<nt;i++)
	{
	    if(ptr_to_med[i]<0)
	    {
		d2[i]=Double.NaN;
		continue;
	    }
	    else
	    {
		final double m=tb[ptr_to_med[i]].price;
		if(med_shares[i]<=0||Double.isNaN(m))
		{
		    throw new IllegalStateException();
		}
		long ms=med_shares[i];
		// join orders of the same price for median level
		for(int p=ptr_to_med[i]+1;p<tsize;p++)
		{
		    if(tb[p].price==m)
		    {
			ms+=tb[p].shares;
		    }
		    else
		    {
			break;
		    }
		}
		final long me=out_shares[i];
		final double dme=(double)me;

		final double pe=tb[ptr_to_last[i]].price;
		d2[i]=(((pe-m)/p0)*(dme/((double)(me-ms))))-(((m-p0)/p0)*(dme/((double)(ms-m0_shares))));
		if(Double.isInfinite(d2[i]))
		{
		    d2[i]=Double.NaN;
		}
	    }
	}
	return d2;
    }


}
