package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;



/** Calculate trade volume and median distributions.
 *  @version $Id: CalculateConsequentData.java,v 1.64 2020/02/06 00:09:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateConsequentData {

    // volumes
    long [] v_buy_tau=null,v_sell_tau=null,v_tot_tau=null;
    double [] median_buy_tau=null,
	median_sell_tau=null,
	min_sell=null,
	risk_min_sell=null,
	max_buy=null,
	risk_max_buy=null,
	median2_buy_tau=null,
	median2_sell_tau=null,
	price_buy_tau=null,
	price_sell_tau=null,
	min_tot_tau=null,
	max_tot_tau=null,
	median_low_tot_tau=null,
	median_high_tot_tau=null;
    private int [] tau_interval_range_ptr=null;
    CalculateKendall.DPPCalculateKendall [] dkpp_tau=null;
    CalculateKendall.TradeDStats [] fmf=null;

    private final Distribution<Trade> savedtrades,tmptradesbuf;
    CalculateConsequentData(final Distribution<Trade> savedtrades,
			    final Distribution<Trade> tmptradesbuf)
    {
	this.savedtrades=savedtrades;
	this.tmptradesbuf=tmptradesbuf;
    }


    void clearData()
    {
	this.v_buy_tau=this.v_sell_tau=this.v_tot_tau=null;
	this.median_buy_tau=
	this.median_sell_tau=
        this.median2_buy_tau=
	this.median2_sell_tau=
	this.median_low_tot_tau=
	this.median_high_tot_tau=
	this.min_sell=
	this.risk_min_sell=
	this.max_buy=
	this.risk_max_buy=
	this.price_buy_tau=
	this.price_sell_tau=
	this.min_tot_tau=
	this.max_tot_tau=null;
	this.tau_interval_range_ptr=null;
	this.dkpp_tau=null;
	this.fmf=null;
    }

    private void initData(final int nt,
			  final boolean calculate_medians,
			  final boolean calculate_dp_tau,
			  final boolean calculate_fmf)
    {
	this.v_buy_tau=new long [nt];
	this.v_sell_tau=new long [nt];
	this.v_tot_tau=new long [nt];
	if(calculate_medians)
	{
	    this.median_buy_tau=new double [nt];
	    this.median_sell_tau=new double [nt];
	    this.median2_buy_tau=new double [nt];
	    this.median2_sell_tau=new double [nt];
	    this.median_low_tot_tau=new double [nt];
	    this.median_high_tot_tau=new double [nt];
	}
	else
	{
	    this.median_buy_tau=
	    this.median_sell_tau=
	    this.median2_buy_tau=
	    this.median2_sell_tau=
	    this.median_low_tot_tau=
	    this.median_high_tot_tau=null;
	}
	if(calculate_dp_tau)
	{
	    this.dkpp_tau=new CalculateKendall.DPPCalculateKendall [nt];
	}
	else
	{
	    this.dkpp_tau=null;
	}
	if(calculate_fmf)
	{
	    this.fmf=new CalculateKendall.TradeDStats [nt];
	}
	else
	{
	    this.fmf=null;
	}

	this.min_sell=new double [nt];
	this.risk_min_sell=new double [nt];
	this.max_buy=new double [nt];
	this.risk_max_buy=new double [nt];
	this.price_buy_tau=new double [nt];
	this.price_sell_tau=new double [nt];
	this.min_tot_tau=new double [nt];
	this.max_tot_tau=new double [nt];
	this.tau_interval_range_ptr=new int [nt];

	for(int i=0;i<nt;i++)
	{
	    this.min_sell[i]=
	    this.risk_min_sell[i]=
	    this.max_buy[i]=
	    this.risk_max_buy[i]=
	    this.price_buy_tau[i]=
	    this.price_sell_tau[i]=
	    this.min_tot_tau[i]=
	    this.max_tot_tau[i]=Double.NaN;
	    this.tau_interval_range_ptr[i]=-1;
	    if(calculate_medians)
	    {
		this.median_buy_tau[i]=
		this.median_sell_tau[i]=
		this.median2_buy_tau[i]=
		this.median2_sell_tau[i]=
		this.median_low_tot_tau[i]=
		this.median_high_tot_tau[i]=Double.NaN;
	    }
	}
    }


    private void sumVolume(final int nt,final long currenttime)
    {
	// try to use previous if current one has no data.
	for(int i=0;i<nt;i++)
	{
	    final int i1=i-1;
	    if(i1>=0)
	    {
		if(Double.isNaN(this.min_sell[i]))
		{
		    this.min_sell[i]=this.min_sell[i1];
		}
		if(Double.isNaN(this.risk_min_sell[i]))
		{
		    this.risk_min_sell[i]=this.risk_min_sell[i1];
		}
		if(Double.isNaN(this.max_buy[i]))
		{
		    this.max_buy[i]=this.max_buy[i1];
		}
		if(Double.isNaN(this.risk_max_buy[i]))
		{
		    this.risk_max_buy[i]=this.risk_max_buy[i1];
		}
		if(Double.isNaN(this.price_buy_tau[i]))
		{
		    this.price_buy_tau[i]=this.price_buy_tau[i1];
		}
		if(Double.isNaN(this.price_sell_tau[i]))
		{
		    this.price_sell_tau[i]=this.price_sell_tau[i1];
		}
		if(Double.isNaN(this.min_tot_tau[i]))
		{
		    this.min_tot_tau[i]=this.min_tot_tau[i1];
		}
		if(Double.isNaN(this.max_tot_tau[i]))
		{
		    this.max_tot_tau[i]=this.max_tot_tau[i1];
		}
		this.v_buy_tau[i]+=this.v_buy_tau[i1];
		this.v_sell_tau[i]+=this.v_sell_tau[i1];
		this.v_tot_tau[i]+=this.v_tot_tau[i1];
	    }
	    if( ((this.v_buy_tau[i]>0) &&
		 (Double.isNaN(this.max_buy[i]) || Double.isNaN(this.risk_max_buy[i]) || Double.isNaN(this.price_buy_tau[i]))) ||
		((this.v_sell_tau[i]>0) &&
		 (Double.isNaN(this.min_sell[i]) || Double.isNaN(this.risk_min_sell[i]) || Double.isNaN(this.price_sell_tau[i]))) ||
		((this.v_tot_tau[i]>0) &&
		 (Double.isNaN(this.min_tot_tau[i])||Double.isNaN(this.max_tot_tau[i])))
		)
	    {
		throw new RuntimeException("Broken constrains for i="+i+
					   " v_buy_tau["+i+"]="+this.v_buy_tau[i]+
					   " max_buy["+i+"]="+this.max_buy[i]+
					   " risk_max_buy["+i+"]="+this.risk_max_buy[i]+
					   " price_buy_tau["+i+"]="+this.price_buy_tau[i]+
					   " v_sell_tau["+i+"]="+this.v_sell_tau[i]+
					   " min_sell["+i+"]="+this.min_sell[i]+
					   " risk_min_sell["+i+"]="+this.risk_min_sell[i]+
					   " price_sell_tau["+i+"]="+this.price_sell_tau[i]+
					   " v_tot_tau["+i+"]="+this.v_tot_tau[i]+
					   " min_tot_tau["+i+"]="+this.min_tot_tau[i]+
					   " max_tot_tau["+i+"]="+this.max_tot_tau[i]+
					   " tau_interval_range_ptr["+i+"]="+this.tau_interval_range_ptr[i]+
					   " currenttime="+currenttime);
	    }
	}
    }


    private void setPreviousMedianIfNoCurrentAvaileable(final int nt,final long currenttime)
    {

	    // try to use previous if current one has no data.
	    for(int i=0;i<nt;i++)
	    {
		final int i1=i-1;
		if(i1>=0)
		{
		    if(Double.isNaN(this.median_buy_tau[i]))
		    {
			this.median_buy_tau[i]=this.median_buy_tau[i1];
		    }
		    if(Double.isNaN(this.median2_buy_tau[i]))
		    {
			this.median2_buy_tau[i]=this.median2_buy_tau[i1];
		    }
		    if(Double.isNaN(this.median_sell_tau[i]))
		    {
			this.median_sell_tau[i]=this.median_sell_tau[i1];
		    }
		    if(Double.isNaN(this.median2_sell_tau[i]))
		    {
			this.median2_sell_tau[i]=this.median2_sell_tau[i1];
		    }
		    if(Double.isNaN(this.median_low_tot_tau[i]))
		    {
			this.median_low_tot_tau[i]=this.median_low_tot_tau[i1];
		    }
		    if(Double.isNaN(this.median_high_tot_tau[i]))
		    {
			this.median_high_tot_tau[i]=this.median_high_tot_tau[i1];
		    }
		}
		if( ((this.v_buy_tau[i]>0) && (Double.isNaN(this.median_buy_tau[i]) || Double.isNaN(this.median2_buy_tau[i]))) ||
		    ((this.v_sell_tau[i]>0) && (Double.isNaN(this.median_sell_tau[i]) || Double.isNaN(this.median2_sell_tau[i])))||
		    ((this.v_tot_tau[i]>0) && (Double.isNaN(this.median_low_tot_tau[i]) || Double.isNaN(this.median_high_tot_tau[i])))
		    )
		{
		    throw new RuntimeException("Broken constrains for i="+i+
					       " v_buy_tau["+i+"]="+this.v_buy_tau[i]+
					       " median_buy_tau["+i+"]="+this.median_buy_tau[i]+
					       " median2_buy_tau["+i+"]="+this.median2_buy_tau[i]+
					       " v_sell_tau_tau["+i+"]="+this.v_sell_tau[i]+
					       " median_sell_tau["+i+"]="+this.median_sell_tau[i]+
					       " median2_sell_tau["+i+"]="+this.median2_sell_tau[i]+
					       " v_tot_tau["+i+"]="+this.v_tot_tau[i]+
					       " median_low_tot_tau["+i+"]="+this.median_low_tot_tau[i]+
					       " median_high_tot_tau["+i+"]="+this.median_high_tot_tau[i]+
					       " currenttime="+currenttime
					       );
		}
	    }


    }

    void calculateStats(final long currenttime,
			final long ptime,
			final double frac,
			final double frac2,
			final double pcurrent_buy,
			final double pcurrent_sell,
			final long [] tau_ahead,
			final long trades_cleared_before)
	{

	    if(frac <0.0 || frac>1.0 || frac2<0.0 || frac2>1.0)
	    {
		throw new IllegalArgumentException("frac="+frac+" frac2="+frac2);
	    }

	    final int tsize=this.savedtrades.getSize();
	    final Trade [] tb=this.tmptradesbuf.getBuffer();
	    System.arraycopy(this.savedtrades.getBuffer(),0,tb,0,tsize);

	    final int nt=tau_ahead.length;
	    // init arrays
	    final boolean calculate_medians=CalculateConsequents.CALCULATE_MEDIANS;
	    final boolean calculate_dp_tau=CalculateConsequents.CALCULATE_DP_TAU;
	    final boolean calculate_fmf=CalculateConsequents.CALCULATE_FMF;
	    initData(nt,calculate_medians,calculate_dp_tau,calculate_fmf);
	    for(int i=0;i<nt;i++)
	    {
		if(i>0 && tau_ahead[i]<=tau_ahead[i-1])
		{
		    throw new IllegalStateException("Intervals not sorted");
		}
	    }


	    final boolean PRINT_DATA_DEBUG=false;
		//currenttime==35298269000000L;
		//currenttime==35554819000000L;
		//currenttime>=10*3600*TraderInterface.TIME_SCALE && tsize>0;


	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("=="+getClass().getName()+"============CONSEQUENT FROM TRADES t=\t"+currenttime+"\t NTRADES=\t"+tsize+
				   "\tcur_buy\t"+pcurrent_buy+
				   "\tcur_sell\t"+pcurrent_sell+
				   "\tfrac\t"+frac+
				   "\tfrac2\t"+frac2+
				   "\tptime\t"+ptime);
	    }
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("num\t"+
				   "time\t"+
				   "shares\t"+
				   "tradetype\t"+
				   "buy_sell_indicator\t"+
				   "price\t"+
				   "matchnumber\t"+
				   "orderreferencenumber");
	    }

	    if(ptime<trades_cleared_before)
	    {
		throw new RuntimeException("Something is terrible wrong with old data removal");
	    }

	    // binary search
	    int px=0;
	    boolean prnotfound=false;
	    ps:if(tsize>0)
	    {
		int pl=px,pr=tsize-1;
		if(tb[pl].time>ptime)
		{
		    px=pl;
		    break ps;
		}
		if(tb[pr].time<=ptime)
		{
		    px=pr;
		    prnotfound=true;
		    break ps;
		}
		if(pl>pr)
		{
		    throw new RuntimeException("Something is terrible wrong with ordering");
		}
		for(;pr-pl>1;)
		{
		    final int p=(pl+pr)/2;
		    if(tb[p].time>ptime)
		    {
			pr=p;
		    }
		    else
		    {
			pl=p;
		    }

		}
		px=pr;
	    }
	    if(tsize>0 && (
		   (!prnotfound && px<tsize && !(tb[px].time>ptime)) ||
		   (!prnotfound && px<tsize-1 && !(tb[px+1].time>ptime)) ||
		   (prnotfound && !(tb[px].time<=ptime)) ||
		   (px>0 && !(tb[px-1].time<=ptime)) ||
		   (px>1 && !(tb[px-2].time<=ptime))
			   ))
	    {
		throw new RuntimeException("This should never hapen "+
					   " data not sorted "+
					   " currenttime="+currenttime+
					   " px="+px+
					   " t_data="+tb[px].time);
	    }


	    double lev_min=Double.POSITIVE_INFINITY,lev_max=Double.NEGATIVE_INFINITY;
	    double rlev_min=Double.POSITIVE_INFINITY,rlev_max=Double.NEGATIVE_INFINITY;
	    double tot_lev_min=Double.POSITIVE_INFINITY,tot_lev_max=Double.NEGATIVE_INFINITY;
	    long tbefore=0L,prevt=-1;
	    gloop:for(int i=!prnotfound?px:tsize,cur_pos=-1;i<tsize;i++)
	    {
		final Trade t=tb[i];
		if(t.time<prevt)
		{
		    // a check for poisoned buffer
		    throw new RuntimeException("The data is not sorted:"+
					       prevt+" and "+t.time+
					       " currenttime="+currenttime);
		}
		prevt=t.time;
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.shares+"\t"+
				       t.tradetype+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.price+"\t"+
				       t.matchnumber+"\t"+
				       t.orderreferencenumber+
				       (t.order!=null?
					"\t\t"+(t.time-t.order.time)+"\t"+
					t.order.time+"\t"+
					t.order.shares+"\t"+
					t.order.price:
					"\t\t-10\t\t\t"));
		}
		for(;cur_pos<0||!(t.time<=tbefore);)
		{
		    if(++cur_pos>=nt)
		    {
			break gloop;
		    }
		    tbefore=ptime+tau_ahead[cur_pos];
		}
		this.tau_interval_range_ptr[cur_pos]=i;
		if(t.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    this.price_buy_tau[cur_pos]=t.price;
		    // risk as min
		    if(t.price<=rlev_min)
		    {
			rlev_min=t.price;
		    }
		    // max buy
		    if(t.price>lev_max)
		    {
			this.max_buy[cur_pos]=lev_max=t.price;
			this.risk_max_buy[cur_pos]=rlev_min;
		    }
		    if(0>(this.v_buy_tau[cur_pos]+=t.shares))
		    {
			throw new IllegalStateException("Overflow");
		    }
		}
		else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    this.price_sell_tau[cur_pos]=t.price;
		    // risk as max
		    if(t.price>=rlev_max)
		    {
			rlev_max=t.price;
		    }
		    // min sell
		    if(t.price<lev_min)
		    {
			this.min_sell[cur_pos]=lev_min=t.price;
			this.risk_min_sell[cur_pos]=rlev_max;
		    }
		    if(0>(this.v_sell_tau[cur_pos]+=t.shares))
		    {
			throw new IllegalStateException("Overflow");
		    }
		}
		if(t.price<tot_lev_min)
		{
		    this.min_tot_tau[cur_pos]=tot_lev_min=t.price;
		}
		if(t.price>tot_lev_max)
		{
		    this.max_tot_tau[cur_pos]=tot_lev_max=t.price;
		}
		if(0>(this.v_tot_tau[cur_pos]+=t.shares))
		{
		    throw new IllegalStateException("Overflow");
		}
	    }

	    sumVolume(nt,currenttime);

	    if(calculate_dp_tau)
	    {
		// calculate dkpp before buffer is poisoned by median calculations
		for(int cur_pos=0;cur_pos<nt;cur_pos++)
		{
		    int ptr_end=-1;
		    for(int i=cur_pos;i>=0;i--)
		    {
			if(this.tau_interval_range_ptr[i]>=0)
			{
			    ptr_end=this.tau_interval_range_ptr[i];
			    break;
			}
		    }
		    //System.err.println("px="+px+" ptr_end="+ptr_end);
		    this.dkpp_tau[cur_pos]=calculateDPTau(cur_pos,
							  tb,
							  !prnotfound&&px<=ptr_end?px:-1,
							  ptr_end,
							  currenttime);
		}
	    }

	    if(calculate_fmf)
	    {
		// calculate fmf before buffer is poisoned by median calculations
		for(int cur_pos=0;cur_pos<nt;cur_pos++)
		{
		    int ptr_end=-1;
		    for(int i=cur_pos;i>=0;i--)
		    {
			if(this.tau_interval_range_ptr[i]>=0)
			{
			    ptr_end=this.tau_interval_range_ptr[i];
			    break;
			}
		    }
		    this.fmf[cur_pos]=CalculateKendall.calculateTradesKendallDD(
			currenttime,
			tb,
			!prnotfound&&px<=ptr_end?px:-1,
			ptr_end+1);
		}
	    }

	    if(calculate_medians)
	    {
		// calculate median in price space for slices in time space
		for(int cur_pos=0;cur_pos<nt;cur_pos++)
		{
		    final int ptr_start=this.tau_interval_range_ptr[cur_pos];
		    if(ptr_start<0)
		    {
			// no points for this interval
			// previous interval is the same as this one
			continue;
		    }
		    calculateMedians(cur_pos,
				     tb,
				     px,
				     ptr_start,
				     ptime,
				     ptime+tau_ahead[cur_pos],
				     frac,
				     frac2,
				     currenttime);
		}
		setPreviousMedianIfNoCurrentAvaileable(nt,currenttime);
	    }

	    if(PRINT_DATA_DEBUG)
	    {

		System.err.println("tau\t\t"+
				   "vol_buy\t"+
				   "max_buy\t"+
				   "risk_max_buy\t"+
				   "price_buy_tau\t"+
				   "median_buy_tau\t"+
				   "median2_buy_tau\t"+
				   "\t"+
				   "vol_sell\t"+
				   "min_sell\t"+
				   "risk_min_sell\t"+
				   "price_sell_tau\t"+
				   "median_sell_tau\t"+
				   "median2_sell_tau\t"+
				   "\t"+
				   "vol_tot\t"+
				   "min_tot\t"+
				   "max_tot\t"+
				   "median_low_tot\t"+
				   "median_high_tot\t"
				   );

		for(int i=0;i<nt;i++)
		{
		    System.err.println(String.valueOf(tau_ahead[i])+"\t\t"+
				       this.v_buy_tau[i]+"\t"+
				       this.max_buy[i]+"\t"+
				       this.risk_max_buy[i]+"\t"+
				       this.price_buy_tau[i]+"\t"+
				       (calculate_medians?this.median_buy_tau[i]:Double.NaN)+"\t"+
				       (calculate_medians?this.median2_buy_tau[i]:Double.NaN)+"\t\t"+
				       this.v_sell_tau[i]+"\t"+
				       this.min_sell[i]+"\t"+
				       this.risk_min_sell[i]+"\t"+
				       this.price_sell_tau[i]+"\t"+
				       (calculate_medians?this.median_sell_tau[i]:Double.NaN)+"\t"+
				       (calculate_medians?this.median2_sell_tau[i]:Double.NaN)+"\t\t"+
				       this.v_tot_tau[i]+"\t"+
				       this.min_tot_tau[i]+"\t"+
				       this.max_tot_tau[i]+"\t"+
				       (calculate_medians?this.median_low_tot_tau[i]:Double.NaN)+"\t"+
				       (calculate_medians?this.median_high_tot_tau[i]:Double.NaN)+"\t"
				       );
		}
	    }

	    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    {
		for(int i=0;i<nt;i++)
		{
		    if(true)
		    {
			// check last price
			double p0_buy=Double.NaN,p0_sell=Double.NaN;
			long pt=Long.MAX_VALUE;
			final long tlast=ptime+tau_ahead[i];
			final Trade [] tbx=this.savedtrades.getBuffer();
			for(int j=tsize;j-->0;)
			{
			    final Trade t=tbx[j];
			    if(t.time<=ptime)
			    {
				break;
			    }
			    if(t.time>pt)
			    {
				// a check for poisoned buffer
				throw new RuntimeException();
			    }
			    pt=t.time;
			    if(t.buy_sell_indicator==BookItem.BOOK_BUY)
			    {
				if(Double.isNaN(p0_buy)&&pt<=tlast)
				{
				    p0_buy=t.price;
				}
			    }
			    else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
			    {
				if(Double.isNaN(p0_sell)&&pt<=tlast)
				{
				    p0_sell=t.price;
				}
			    }
			}
			// check p0_buy & p0_sell
			if(Double.isNaN(p0_buy)!=Double.isNaN(this.price_buy_tau[i]) ||
			   (!Double.isNaN(this.price_buy_tau[i]) && p0_buy!=this.price_buy_tau[i]) ||
			   Double.isNaN(p0_sell)!=Double.isNaN(this.price_sell_tau[i]) ||
			   (!Double.isNaN(this.price_sell_tau[i]) && p0_sell!=this.price_sell_tau[i]))
			{
			    throw new RuntimeException("price tau mismatch:"+
						       " p0_buy="+p0_buy+
						       " price_buy_tau="+this.price_buy_tau[i]+
						       " p0_sell="+p0_sell+
						       " price_sell_tau="+this.price_sell_tau[i]);
			}
		    }



		    final int ptr=this.tau_interval_range_ptr[i];
		    if(ptr<0)
		    {
			continue;
		    }
		    // cheat CalculateTradeVolume, create specially crafted data for testing.
		    System.arraycopy(this.savedtrades.getBuffer(),0,tb,0,tb.length);
		    for(int j=tsize-1;j>ptr;j--)
		    {
			tb[j]=null;
		    }
		    this.tmptradesbuf.setSize(ptr+1);

		    // compare with calculation in CalculateTradeVolume
		    final CalculateTradeVolume ctv=new CalculateTradeVolume(this.tmptradesbuf,this.tmptradesbuf);
		    ctv.calculateTradeVolume(new long []{tau_ahead[i]-1},
					     ptime+tau_ahead[i],
					     frac,
					     frac2,
					     0.5,
					     pcurrent_buy,
					     pcurrent_sell,
					     trades_cleared_before);
		    checkConstrains(ctv,0,i,calculate_medians);
		}
	    }
	}


    private void checkConstrains(final CalculateTradeVolume ctv,
				 final int TOCHECK,
				 final int cur_pos,
				 final boolean calculate_medians)
    {
	if(ctv.v_alltrades_buy_tau[TOCHECK]!=this.v_buy_tau[cur_pos]||
	   ctv.v_alltrades_sell_tau[TOCHECK]!=this.v_sell_tau[cur_pos])
	{
	    throw new RuntimeException(
		"Volume mismatch:"+
		" ctv.v_alltrades_buy_tau["+TOCHECK+"]="+ctv.v_alltrades_buy_tau[TOCHECK]+
		" v_buy_tau["+cur_pos+"]="+this.v_buy_tau[cur_pos]+
		" ctv.v_alltrades_sell_tau["+TOCHECK+"]="+ctv.v_alltrades_sell_tau[TOCHECK]+
		" v_sell_tau["+cur_pos+"]="+this.v_sell_tau[cur_pos]);
	}

	if(Double.isNaN(ctv.maxprice[TOCHECK])!=Double.isNaN(this.max_buy[cur_pos]) ||
	   (!Double.isNaN(this.max_buy[cur_pos]) && ctv.maxprice[TOCHECK]!=this.max_buy[cur_pos]) ||
	   Double.isNaN(ctv.minprice[TOCHECK])!=Double.isNaN(this.min_sell[cur_pos]) ||
	   (!Double.isNaN(this.min_sell[cur_pos]) && ctv.minprice[TOCHECK]!=this.min_sell[cur_pos]))
	{
	    throw new RuntimeException(
		"Max buy/Min sell mismatch:"+
		" ctv.maxprice["+TOCHECK+"]="+ctv.maxprice[TOCHECK]+
		" max_buy["+cur_pos+"]="+this.max_buy[cur_pos]+
		" ctv.minprice["+TOCHECK+"]="+ctv.minprice[TOCHECK]+
		" min_sell["+cur_pos+"]="+this.min_sell[cur_pos]);
	}

	if(calculate_medians)
	{
	if(Double.isNaN(ctv.trade_median_buy_tau[TOCHECK])!=Double.isNaN(this.median_buy_tau[cur_pos]) ||
	   (!Double.isNaN(this.median_buy_tau[cur_pos]) && ctv.trade_median_buy_tau[TOCHECK]!=this.median_buy_tau[cur_pos]) ||
	   Double.isNaN(ctv.trade_median_sell_tau[TOCHECK])!=Double.isNaN(this.median_sell_tau[cur_pos]) ||
	   (!Double.isNaN(this.median_sell_tau[cur_pos]) && ctv.trade_median_sell_tau[TOCHECK]!=this.median_sell_tau[cur_pos]))
	{
	    throw new RuntimeException(
		"Median buy/Median sell se mismatch:"+
		" ctv.trade_median_buy_tau["+TOCHECK+"]="+ctv.trade_median_buy_tau[TOCHECK]+
		" median_buy_tau["+cur_pos+"]="+this.median_buy_tau[cur_pos]+
		" ctv.trade_median_sell_tau["+TOCHECK+"]="+ctv.trade_median_sell_tau[TOCHECK]+
		" median_sell_tau["+cur_pos+"]="+this.median_sell_tau[cur_pos]);
	}


	if(Double.isNaN(ctv.trade_median_f2_buy_tau[TOCHECK])!=Double.isNaN(this.median2_buy_tau[cur_pos]) ||
	   (!Double.isNaN(this.median2_buy_tau[cur_pos]) && ctv.trade_median_f2_buy_tau[TOCHECK]!=this.median2_buy_tau[cur_pos]) ||
	   Double.isNaN(ctv.trade_median_f2_sell_tau[TOCHECK])!=Double.isNaN(this.median2_sell_tau[cur_pos]) ||
	   (!Double.isNaN(this.median2_sell_tau[cur_pos]) && ctv.trade_median_f2_sell_tau[TOCHECK]!=this.median2_sell_tau[cur_pos]))
	{
	    throw new RuntimeException(
		"Median2 buy/Median2 sell se mismatch:"+
		" ctv.trade_median_f2_buy_tau["+TOCHECK+"]="+ctv.trade_median_f2_buy_tau[TOCHECK]+
		" median2_buy_tau["+cur_pos+"]="+this.median2_buy_tau[cur_pos]+
		" ctv.trade_median_f2_sell_tau["+TOCHECK+"]="+ctv.trade_median_f2_sell_tau[TOCHECK]+
		" median2_sell_tau["+cur_pos+"]="+this.median2_sell_tau[cur_pos]);
	}
	}

	if(ctv.v_tot_alltrades[TOCHECK]!=this.v_tot_tau[cur_pos])
	{
	    throw new RuntimeException(
		"Volume mismatch:"+
		" ctv.v_tot_alltrades["+TOCHECK+"]="+ctv.v_tot_alltrades[TOCHECK]+
		" this.v_tot_tau["+cur_pos+"]="+this.v_tot_tau[cur_pos]);
	}
	if(Double.isNaN(this.min_tot_tau[cur_pos])!=Double.isNaN(ctv.tot_minprice[TOCHECK])||
	   (!Double.isNaN(this.min_tot_tau[cur_pos]) && this.min_tot_tau[cur_pos]!=ctv.tot_minprice[TOCHECK])||
	   Double.isNaN(this.max_tot_tau[cur_pos])!=Double.isNaN(ctv.tot_maxprice[TOCHECK])||
	   (!Double.isNaN(this.max_tot_tau[cur_pos]) && this.max_tot_tau[cur_pos]!=ctv.tot_maxprice[TOCHECK]))
	{
	    throw new RuntimeException(
		"Min/Max mismatch:"+
		" ctv.tot_minprice["+TOCHECK+"]="+ctv.tot_minprice[TOCHECK]+
		" this.min_tot_tau["+cur_pos+"]="+this.min_tot_tau[cur_pos]+
		" ctv.tot_maxprice["+TOCHECK+"]="+ctv.tot_maxprice[TOCHECK]+
		" this.max_tot_tau["+cur_pos+"]="+this.max_tot_tau[cur_pos]);
	}
	if(calculate_medians)
	{
	if(Double.isNaN(this.median_low_tot_tau[cur_pos])!=Double.isNaN(ctv.trade_tot_median_low_tau[TOCHECK])||
	   (!Double.isNaN(this.median_low_tot_tau[cur_pos]) && this.median_low_tot_tau[cur_pos]!=ctv.trade_tot_median_low_tau[TOCHECK])||
	   Double.isNaN(this.median_high_tot_tau[cur_pos])!=Double.isNaN(ctv.trade_tot_median_high_tau[TOCHECK])||
	   (!Double.isNaN(this.median_high_tot_tau[cur_pos]) && this.median_high_tot_tau[cur_pos]!=ctv.trade_tot_median_high_tau[TOCHECK]))
	{
	    throw new RuntimeException(
		"Medians mismatch:"+
		" ctv.trade_tot_median_low_tau["+TOCHECK+"]="+ctv.trade_tot_median_low_tau[TOCHECK]+
		" this.median_low_tot_tau["+cur_pos+"]="+this.median_low_tot_tau[cur_pos]+
		" ctv.trade_tot_median_high_tau["+TOCHECK+"]="+ctv.trade_tot_median_high_tau[TOCHECK]+
		" this.median_high_tot_tau["+cur_pos+"]="+this.median_high_tot_tau[cur_pos]);
	}
	}
    }


    private CalculateKendall.DPPCalculateKendall calculateDPTau(final int cur_pos,
								final Trade [] tb,
								final int p_start,
								final int p_end,
								final long currenttime)
    {
	final CalculateKendall.DPPCalculateKendall dkpp=
	    CalculateKendall.calculateTradesKendallEndDKPPDiffVScaled(
		currenttime,
		tb,
		p_start,
		p_end+1,
		0);
	return dkpp;
    }


    private void calculateMedians(final int cur_pos,
				  final Trade [] tb,
				  final int p_start,
				  final int p_end,
				  final long tleft,
				  final long tright,
				  final double frac,
				  final double frac2,
				  final long currenttime)
    {
	if(true)
	{
	    // order array by type and price.
	    // the buffer is considered poisoned after this.
	    com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(tb,p_start,p_end,CompareItems.cmp_by_type_and_price);
	    // median for buy
	    long vol=0,prevvol=0,lev=(long)(frac*this.v_buy_tau[cur_pos]),lev2=(long)(frac2*this.v_buy_tau[cur_pos]);
	    double prev_price=Double.NaN;
	    for(int i=p_start;i<=p_end;i++)
	    {
		final Trade t=tb[i];
		if(t.buy_sell_indicator!=BookItem.BOOK_BUY)
		{
		    break;
		}
		final double p=t.price;
		if(prev_price<p || !(tleft<t.time && t.time<=tright)) // will be false for NaN
		{
		    throw new RuntimeException("Ordering is broken");
		}
		prev_price=p;
		vol+=t.shares;
		if(vol<0)
		{
		    throw new RuntimeException("Integer overflow currenttime="+currenttime);
		}
		if(vol>lev && lev>=prevvol)
		{
		    this.median_buy_tau[cur_pos]=p;
		}
		if(vol>lev2 && lev2>=prevvol)
		{
		    this.median2_buy_tau[cur_pos]=p;
		}
		prevvol=vol;
	    }
	    if(vol!=this.v_buy_tau[cur_pos])
	    {
		throw new RuntimeException("Broken invariant. This should never happen."+
					   " vol="+vol+
					   " this.v_buy_tau["+cur_pos+"]="+this.v_buy_tau[cur_pos]+
					   " currenttime="+currenttime);
	    }
	    if(vol>0)
	    {
		if(tb[p_start].price!=this.max_buy[cur_pos])
		{
		    throw new RuntimeException("Max buy price="+tb[p_start].price+
					       " does not match "+this.max_buy[cur_pos]+
					       ". currenttime="+currenttime);
		}
	    }

	    // median for sell
	    vol=prevvol=0;
	    lev=(long)(frac*this.v_sell_tau[cur_pos]);
	    lev2=(long)(frac2*this.v_sell_tau[cur_pos]);
	    prev_price=Double.NaN;
	    for(int i=p_end;i>=p_start;i--)
	    {
		final Trade t=tb[i];
		if(t.buy_sell_indicator!=BookItem.BOOK_SELL)
		{
		    break;
		}
		final double p=t.price;
		if(prev_price>p || !(tleft<t.time && t.time<=tright)) // will be false for NaN
		{
		    throw new RuntimeException("Ordering is broken");
		}
		prev_price=p;
		vol+=t.shares;
		if(vol<0)
		{
		    throw new RuntimeException("Integer overflow currenttime="+currenttime);
		}
		if(vol>lev && lev>=prevvol)
		{
		    this.median_sell_tau[cur_pos]=p;
		}
		if(vol>lev2 && lev2>=prevvol)
		{
		    this.median2_sell_tau[cur_pos]=p;
		}
		prevvol=vol;
	    }
	    if(vol!=this.v_sell_tau[cur_pos])
	    {
		throw new RuntimeException("Broken invariant. This should never happen."+
					   " vol="+vol+
					   " this.v_buy_tau["+cur_pos+"]="+this.v_sell_tau[cur_pos]+
					   " currenttime="+currenttime);
	    }
	    if(vol>0)
	    {
		if(tb[p_end].price!=this.min_sell[cur_pos])
		{
		    throw new RuntimeException("Min sell price="+tb[p_end].price+
					       " does not match "+this.min_sell[cur_pos]+
					       ". currenttime="+currenttime);
		}
	    }
	}

	if(true)
	{
	    // order array by price.
	    // the buffer is considered poisoned after this.
	    com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(tb,p_start,p_end,CompareItems.cmp_by_price_only);
	    final long lev=(long)(frac*this.v_tot_tau[cur_pos]);
	    long vol=0;
	    double prev_price=Double.NaN;
	    for(int i=p_start;i<=p_end;i++)
	    {
		final Trade t=tb[i];
		final double p=t.price;
		if(prev_price<p || !(tleft<t.time && t.time<=tright)) // will be false for NaN
		{
		    throw new RuntimeException("Ordering is broken");
		}
		prev_price=p;
		vol+=t.shares;
		if(vol<0)
		{
		    throw new RuntimeException("Integer overflow currenttime="+currenttime);
		}
		if(vol>lev)
		{
		    this.median_low_tot_tau[cur_pos]=p;
		    break;
		}
	    }

	    vol=0;
	    prev_price=Double.NaN;
	    for(int i=p_end;i>=p_start;i--)
	    {
		final Trade t=tb[i];
		final double p=t.price;
		if(prev_price>p || !(tleft<t.time && t.time<=tright)) // will be false for NaN
		{
		    throw new RuntimeException("Ordering is broken");
		}
		prev_price=p;
		vol+=t.shares;
		if(vol<0)
		{
		    throw new RuntimeException("Integer overflow currenttime="+currenttime);
		}
		if(vol>lev)
		{
		    this.median_high_tot_tau[cur_pos]=p;
		    break;
		}
	    }

	    if(p_start<=p_end)
	    {
		if(tb[p_end].price!=this.min_tot_tau[cur_pos]||
		   tb[p_start].price!=this.max_tot_tau[cur_pos])
		{
		    throw new RuntimeException("Min/Max mismatch:"+
					       " min found="+tb[p_end].price+
					       " min prev found="+this.min_tot_tau[cur_pos]+
					       " max found="+ tb[p_start].price+
					       " max prev found="+this.max_tot_tau[cur_pos]+
					       " currenttime="+currenttime);
		}
	    }
	}
    }


}
