package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */


/** A class to calculate actual attributes values for ML program.
 *  @version $Id: CalculateConsequents.java,v 1.64 2010/04/14 08:42:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateConsequents {

    /** Global flag to calculate consequent by looking forward.
     */
    static final boolean CALCULATE_LOOK_FORWARD=false || CalculateAntecedents.ATTRIB_TEST_ALL;

    static final boolean CALCULATE_MEDIANS=(false || CalculateAntecedents.ATTRIB_TEST_ALL) && CALCULATE_LOOK_FORWARD;
    static final boolean CALCULATE_DP_TAU=(true || CalculateAntecedents.ATTRIB_TEST_ALL) && CALCULATE_LOOK_FORWARD;
    static final boolean CALCULATE_FMF=(false || CalculateAntecedents.ATTRIB_TEST_ALL) && CALCULATE_LOOK_FORWARD;
    static final boolean
	ATTRIB_MINMAX=true && CALCULATE_LOOK_FORWARD,
	ATTRIB_MEDIAN=true && CALCULATE_MEDIANS,
	ATTRIB_EXACTT=true && CALCULATE_LOOK_FORWARD,
	ATTRIB_TOT_MEDIAN=true && CALCULATE_MEDIANS;



    static final String
	PMIN_PREFIX="Pc.price.sell.min.",
	PMAX_PREFIX="Pc.price.buy.max.",
	PMED_BUY_PREFIX="Pm.price.buy.up.",
	PMED_SELL_PREFIX="Pm.price.sell.down.",
	PTAU_BUY_PREFIX="Pt.price.buy.up.",
	PTAU_SELL_PREFIX="Pt.price.sell.down.",
	PTOT_MED_PREFIX="Pmtot.",
	PDKPP_TAU="Pdkpp.price.";



    final StringAttributesWithDescription sattribs_names=new StringAttributesWithDescription("ignore");
    final StringAttributes sattribs=new StringAttributes();
    final StringAttributesWithDescription dattribs_names=new StringAttributesWithDescription("ignore");
    final DoubleAttributes dattribs=new DoubleAttributes();
    final StringAttributesWithDescription nattribs_names=new StringAttributesWithDescription("ignore");
    final NominalAttributes nattribs=new NominalAttributes();

    void clearData()
    {
	this.sattribs_names.clearData();
	this.sattribs.clearData();
	this.dattribs_names.clearData();
	this.dattribs.clearData();
	this.nattribs_names.clearData();
	this.nattribs.clearData();
    }


    private int setAttrInstrInfo(final boolean flag_processheader,
				 final String ticker,
				 final long currenttime)
    {
	if(flag_processheader)
	{
	    this.sattribs_names.addValue("Ticker");
	    this.sattribs_names.addValue("t00.call");
	}
	else
	{
	    this.sattribs.addValue(ticker);
	    this.sattribs.addValue(String.valueOf(currenttime));
	}
	return 2;
    }


    private int setAttrValues(final boolean flag_processheader,
			      final String ticker,
			      final ModelParameters.ModelParametersData mparametersdata,
			      final CalculateConsequentData consequents,
			      final CalculationDataRef c)
    {
	final int nt=mparametersdata.tau_embedding_consequent.length;
	int n=0;
	for(int i=0;i<nt;i++)
	{
	    if(flag_processheader)
	    {
		final String t=String.valueOf(mparametersdata.tau_embedding_consequent[i]/ModelParameters.DISTANCE_TIME_SCALE);
		if(ATTRIB_MINMAX)
		{
		    n+=4;
		    this.dattribs_names.addValue(PMIN_PREFIX,t,null);
		    this.dattribs_names.addValue(PMIN_PREFIX+"R_",t,null);
		    this.dattribs_names.addValue(PMAX_PREFIX,t,null);
		    this.dattribs_names.addValue(PMAX_PREFIX+"R_",t,null);
		}
		if(CALCULATE_DP_TAU)
		{
		    n+=4;
		    this.dattribs_names.addValue(PDKPP_TAU+"end_",t,null);
		    this.dattribs_names.addValue(PDKPP_TAU+"mm_",t,null);
		    this.dattribs_names.addValue(PDKPP_TAU+"k2end_",t,null);
		    this.dattribs_names.addValue(PDKPP_TAU+"k2mm_",t,null);
		}
		if(ATTRIB_MEDIAN)
		{
		    n+=2;
		    this.dattribs_names.addValue(PMED_BUY_PREFIX,t,null);
		    this.dattribs_names.addValue(PMED_SELL_PREFIX,t,null);
		}
		if(ATTRIB_EXACTT)
		{
		    n+=2;
		    this.dattribs_names.addValue(PTAU_BUY_PREFIX,t,null);
		    this.dattribs_names.addValue(PTAU_SELL_PREFIX,t,null);
		}
		if(ATTRIB_TOT_MEDIAN)
		{
		    n++;
		    this.dattribs_names.addValue(PTOT_MED_PREFIX,t,null);
		}
	    }
	    else
	    {
		if(ATTRIB_MINMAX)
		{
		    n+=4;
		    final double rmin=consequents.min_sell[i]/c.p_buy;
		    this.dattribs.addValue(rmin);
		    this.dattribs.addValue((consequents.risk_min_sell[i]<=c.p_buy)?rmin:rmin+(consequents.risk_min_sell[i]/c.p_buy-1));
		    final double rmax=consequents.max_buy[i]/c.p_sell;
		    this.dattribs.addValue(rmax);
		    this.dattribs.addValue((consequents.risk_max_buy[i]>=c.p_sell)?rmax:rmax+(consequents.risk_max_buy[i]/c.p_sell-1));

		}
		if(CALCULATE_DP_TAU)
		{
		    n+=4;
		    final CalculateKendall.DPPCalculateKendall dkpp=consequents.dkpp_tau[i];
		    this.dattribs.addValue(dkpp.DK1dp_end);
		    this.dattribs.addValue(dkpp.DK1dp_mm);
		    this.dattribs.addValue(dkpp.DK2dp_end);
		    this.dattribs.addValue(dkpp.DK2dp_mm);
		}
		if(ATTRIB_MEDIAN)
		{
		    n+=2;
		    this.dattribs.addValue(consequents.median_buy_tau[i]/c.p_sell);
		    this.dattribs.addValue(consequents.median_sell_tau[i]/c.p_buy);
		}
		if(ATTRIB_EXACTT)
		{
		    n+=2;
		    this.dattribs.addValue(consequents.price_buy_tau[i]/c.p_sell);
		    this.dattribs.addValue(consequents.price_sell_tau[i]/c.p_buy);
		}
		if(ATTRIB_TOT_MEDIAN)
		{
		    n++;
		    this.dattribs.addValue((consequents.median_low_tot_tau[i]+consequents.median_high_tot_tau[i])/(c.p_buy+c.p_sell));
		}
	    }
	}
	return n;
    }


    int setConsequentsAttribs(final String ticker,
				 final ModelParameters.ModelParametersData mparametersdata,
				 final long currenttime,
				 final CalculateConsequentData consequents,
				 final CalculationDataRef c,
				 final AttributesCount check_signatures)
    {

	int n=0;
	n+=setAttrInstrInfo(false,ticker,currenttime);
	n+=setAttrValues(false,
			 ticker,
			 mparametersdata,
			 consequents,
			 c);

	final int nd=this.dattribs.getNValues(),
	    nn=this.nattribs.getNValues(),
	    ns=this.sattribs.getNValues();
	if(n!=nd+nn+ns||
	   nd!=check_signatures.n_doubles||
	   nn!=check_signatures.n_nominals||
	   ns!=check_signatures.n_strings)
	{
	    throw new RuntimeException("This should never happen nd="+nd+
				       " check_signatures.n_doubles="+check_signatures.n_doubles+
				       " nn="+nn+" check_signatures.n_nominals="+
				       check_signatures.n_nominals+" ns="+
				       ns+" check_signatures.n_strings="+check_signatures.n_strings);
	}
	return n;
    }



    AttributesCount setAttributesNames(
	final String ticker,
	final ModelParameters.ModelParametersData mparametersdata)
    {
	int n=0;
	n+=setAttrInstrInfo(true,ticker,0L);
	n+=setAttrValues(true,
			 ticker,
			 mparametersdata,
			 null,
			 null);

	// should we add names to AttributesCount ?
	return new AttributesCount(
	    this.dattribs_names.getNValues(),
	    this.sattribs_names.getNValues(),
	    this.nattribs_names.getNValues(),
	    n);
    }

}
