package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;

/** DCoefs in simplistic model.
 *  @version $Id: CalculateDCoefs.java,v 1.28 2020/02/05 08:16:29 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */

class CalculateDCoefs {

    private static final int COEFS_SUM=5,COEFS_D=6,COEFS_AVER=7;


    static double [] getPCoefsP3(final double [] pt,final double [] pv)
    {
	final double m0t=pt[0],m1t=pt[1],m2t=pt[2],m0v=pv[0],m1v=pv[1];
	final double r=-m0v/m0t,Z=m2t*m0t-m1t*m1t,i0=m0v/m0t,dpp=m0t*m0t*(m1v/m0v-m1t/m0t)/Z;
	return new double [] {
	    r/m0t,
	    r*(dpp/m0t),
	    r*(m0t/Z),
	    r*dpp*(m0t/Z)
	};
    }

    static double [] getPCoefs(final double [] pt,final double [] pv,final int flag_sum)
    {

	final double m0t=pt[0],m1t=pt[1],m2t=pt[2],m0v=pv[0],m1v=pv[1],oo=m1t/m0t;
	final double F0_x=1.0/m0t,F0_d=-(m0v/m0t)/m0t;
	final double Z=m2t*m0t-m1t*m1t,
	    Zp2_d=m0t,Zp1_d=2*(m0t*oo-m1t),Zp0_d=oo*(oo*m0t-2*m1t)+(1-2*m0t)*m2t+2*m1t*m1t;
	final double K=m1v*m0t-m1t*m0v,Kp1_d=-m0v,Kp0_d=(1-2*m0t)*m1v+2*m0v*m1t-m0v*oo;
	final double Ip0=m0v/m0t,Ip1=K/Z;
	final double F1p2_x=m0t/Z,
	    F1N_p2=-Zp2_d*K/(Z*Z),F1N_p1=(Z*Kp1_d-Zp1_d*K)/(Z*Z),F1N_p0=(Z*(Kp0_d-K/m0t)-K*Zp0_d)/(Z*Z);

	if(flag_sum==COEFS_SUM)
	{
	    // two terms: first term I F'x, second term F'd
	    return new double []{
		F0_x*Ip0+F0_d, // [0]-th should be =0 for offset we use
		F0_x*Ip1+F1N_p0,
		F1p2_x*Ip0+F1N_p1,
		F1p2_x*Ip1+F1N_p2
	    };
	}
	else if(flag_sum==COEFS_D)
	{
	    // second term only
	    return new double []{
		F0_d,
		F1N_p0,
		F1N_p1,
		F1N_p2
	    };
	}
	else if(flag_sum==COEFS_AVER)
	{
	    // two terms: first term I_average F'x, second term F'd
	    return new double []{
		F0_x*Ip0+F0_d, // [0]-th should be =0 for offset we use
		           F1N_p0,
		F1p2_x*Ip0+F1N_p1,
		           F1N_p2
	    };
	}
	else
	{
	    throw new RuntimeException();
	}
    }


    // for testing
    private static double getF(final double [] pt,final double [] pv,final double o,
			       final double x,final double p, final double d)
    {
	final double e=Math.exp(-d),dp=p-o;
	final double m0t=e*pt[0]+d,m1t=e*pt[1]+d*dp,m2t=e*pt[2]+d*dp*dp,
	    m0v=e*pv[0]+x,m1v=e*pv[1]+x*dp;
	//return (m1v*m0t-m1t*m0v); //check K
	//return (m2t*m0t-m1t*m1t); // check Z
	//return (dp-m1t/m0t); // check R
	return m0v/m0t+(dp-m1t/m0t)*((m1v*m0t-m1t*m0v)/(m2t*m0t-m1t*m1t));
    }




    private static void checkDs(final double [] pt,final double [] pv,
				final double p,final double o,final double ddif,final double eps)
    {
	final double m0t=pt[0],m1t=pt[1],m2t=pt[2],m0v=pv[0],m1v=pv[1],oo=m1t/m0t;
	final double F0_x=1.0/m0t,F0_d=-(m0v/m0t)/m0t;
	final double Z=m2t*m0t-m1t*m1t,
	    Zp2_d=m0t,Zp1_d=2*(m0t*oo-m1t),Zp0_d=oo*(oo*m0t-2*m1t)+(1-2*m0t)*m2t+2*m1t*m1t;
	final double K=m1v*m0t-m1t*m0v,Kp1_d=-m0v,Kp0_d=(1-2*m0t)*m1v+2*m0v*m1t-m0v*oo;
	final double Ip0=m0v/m0t,Ip1=K/Z;
	final double F1p2_x=m0t/Z,
	    F1N_p2=-Zp2_d*K/(Z*Z),F1N_p1=(Z*Kp1_d-Zp1_d*K)/(Z*Z),F1N_p0=(Z*(Kp0_d-K/m0t)-K*Zp0_d)/(Z*Z);

	/*
	return new double []{
	    F0_x*Ip0+F0_d, // [0]-th should be =0
	    F0_x*Ip1+F1N_p0,
	    F1p2_x*Ip0+F1N_p1,
	    F1p2_x*Ip1+F1N_p2
	};
	*/

	//System.err.println("F1N_p2="+F1N_p2+" F1p2_x*Ip1="+(F1p2_x*Ip1));

	final double i0=getF(pt,pv,o,0,p,0);
	final double i_x=getF(pt,pv,o,ddif,p,0);
	final double i_d=getF(pt,pv,o,0,p,ddif);
	final double dpm=p-o-pt[1]/pt[0];

	UnitTests.checkDoubleRel(i0,"i0",Ip0+dpm*Ip1,"Ip0+dpm*Ip1",eps);
	UnitTests.checkDoubleRel((i_x-i0)/ddif,"(i_x-i0)/ddif",F0_x+F1p2_x*dpm*dpm,"F0_x+F1p2_x*dpm*dpm",eps);

	UnitTests.checkDoubleRel((i_d-i0)/ddif,"(i_d-i0)/ddif",F0_d+dpm*(F1N_p0+F1N_p1*dpm+F1N_p2*dpm*dpm),"F0_d+dpm*(F1N_p0+F1N_p1*dpm+F1N_p2*dpm*dpm)",eps);


	//UnitTests.checkDoubleRel((i_d-i0)/ddif,"K:(i_d-i0)/ddif",Kp0_d+Kp1_d*dpm,"Kp0_d+Kp1_d*dpm",eps);
	//UnitTests.checkDoubleRel((i_d-i0)/ddif,"Z:(i_d-i0)/ddif",Zp0_d+(Zp1_d+Zp2_d*dpm)*dpm,"Zp0_d+(Zp1_d+Zp2_d*dpm)*dpm",eps);
	//UnitTests.checkDoubleRel((i_d-i0)/ddif,"R:(i_d-i0)/ddif",-dpm/m0t,"-dpm/m0t",eps);


	final double [] pcoe=getPCoefs(pt,pv,COEFS_SUM);
	//System.err.println("pcoe[0]="+pcoe[0]+" pcoe[1]="+pcoe[1]+" pcoe[2]="+pcoe[2]+" pcoe[3]="+pcoe[3]);
	/*
	final double di_r=pcoe[0]+dpm*(pcoe[1]+dpm*(pcoe[2]+dpm*pcoe[3]));
	final double di_o=i0*(i_x-i0)/ddif+(i_d-i0)/ddif;
	//System.err.println("di_r="+di_r+" di_o="+di_o);
	//UnitTests.checkDoubleRel(di_o,"di_o",di_r,"di_r",eps);
	*/

	final double [] pcoeD=getPCoefs(pt,pv,COEFS_D),pcoeDa=getPCoefsP3(pt,pv);
	UnitTests.checkDoubleArrayRel(pcoeD,"getPCoefs",pcoeDa,"getPCoefsP3",eps);

    }



    private static void checkCalculations(final int nptest,final java.util.Random r,final double eps)
    {
	final double [] pt=new double[]{r.nextDouble(),r.nextDouble(),r.nextDouble()},
	                pv=new double[]{r.nextDouble(),r.nextDouble()};
	final double o=r.nextDouble();
	final double dd=1e-6;
	System.err.println("checkCalculations(): starting D-tests,dd="+dd+" nptest="+nptest);


	for(int i=0;i<nptest;i++)
	{
	    final double p=r.nextDouble();
	    checkDs(pt,pv,p,o,dd,eps);
	}

	System.err.println("checkCalculations(): D-tests OK for all nptest="+nptest+" P's");
    }


    public static void main(String [] args)
    {
	final long t0=System.currentTimeMillis();
	final java.util.Random r=new java.util.Random(-79091);
	final double eps=7e-4;

	int n=0;
	for(int i=0;i<100;i++)
	{
	    checkCalculations(10000,r,eps);
	    n++;
	}

	final long t1=System.currentTimeMillis();
	System.err.println("CalculateDCoefs::main(): "+n+" tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: CalculateDCoefs.java,v 1.28 2020/02/05 08:16:29 mal Exp $");
    }

}
