package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit.
 *  @version $Id: CalculateLiquidityDeficit.java,v 1.48 2010/04/13 13:45:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit {

    private final Distribution<BookItem> orders;
    private final Distribution<Trade> trades,tmptradesbuf;
    private final boolean flag_lower;

    CalculateLiquidityDeficit(final Distribution<BookItem> orders,
			      final Distribution<BookItem> tmpordersbuf,
			      final Distribution<Trade> trades,
			      final boolean flag_lower,
			      final Distribution<Trade> tmptradesbuf)
    {
	this.orders=orders;
	this.trades=trades;
	this.flag_lower=flag_lower;
	this.tmptradesbuf=tmptradesbuf;
    }

    long []
	v_orders=null,
	tau_orders=null,
	taumin_orders=null,
	tau_trades=null,
	v_flow_exe=null,
	v_flow_hidden_exe=null;
    long []
	v_flow_ot_exe=null,
	v_flow_ot_hidden_exe=null,
	v_exe_thistype=null,
	v_exe_ot_thistype=null;
    double [] price_cut=null,price_taumin=null,
	price_tau=null,price_ot_tau=null,
	price_aver_tau=null,price_ot_aver_tau=null;

    void clear()
    {
	this.price_cut=
	this.price_taumin=
	this.price_tau=
	this.price_ot_tau=
	this.price_aver_tau=
	this.price_ot_aver_tau=null;
	this.v_orders=
	this.tau_orders=
	this.tau_trades=
	this.taumin_orders=
	this.v_flow_exe=
	this.v_flow_hidden_exe=
	this.v_flow_ot_exe=
	this.v_flow_ot_hidden_exe=
	this.v_exe_thistype=
	this.v_exe_ot_thistype=null;
    }


    void calculateLiquidityDeficit(final double [] pricelevels,
				   final long currenttime,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final long trades_cleared_before)
    {

	final boolean PRINT_DATA_DEBUG=false;
	    //currenttime==34434604000000L;
	    //currenttime==34354409000000L;
	    //currenttime==34223484000000L;
	    //currenttime>=9*3600*TraderInterface.TIME_SCALE && tsize>0;
	    //currenttime==34204365000000L;


	final int nt=pricelevels.length;


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tFTYPE=\t"+this.flag_lower+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\nNORDERS\t"+this.orders.getSize()+
			       "\tcur_buy\t"+pcurrent_buy+
			       "\tcur_sell\t"+pcurrent_sell);
	}


	if(true)
	{
	    final int [] ptr_to_book=new int [nt];
	    java.util.Arrays.fill(ptr_to_book,-1);
	    this.price_cut=new double [nt];
	    java.util.Arrays.fill(this.price_cut,Double.NaN);
	    this.price_taumin=new double [nt];
	    java.util.Arrays.fill(this.price_taumin,Double.NaN);

	    final long [] v=this.v_orders=new long [nt];
	    final long [] vsums=this.tau_orders=new long [nt];
	    this.taumin_orders=new long [nt];
	    java.util.Arrays.fill(this.taumin_orders,Long.MIN_VALUE);

	    final int tsize=this.orders.getSize();
	    final BookItem [] tb=this.orders.getBuffer();
	    final boolean flglow=this.flag_lower;
	    if(tsize>0&&tb[0].price!=(flglow?pcurrent_buy:pcurrent_sell))
	    {
		throw new RuntimeException("Something is wrong with price levels:"+
					   " tb[0].price="+tb[0].price+
					   " pcurrent_buy="+pcurrent_buy+
					   " pcurrent_sell="+pcurrent_sell+
					   " flag_lower="+this.flag_lower+
					   " currenttime="+currenttime);
	    }
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("num\t"+
				   "time\t"+
				   "shares\t"+
				   "buy_sell_indicator\t"+
				   "orderreferencenumber\t"+
				   "price\t"+
				   "cur_pos");
	    }

	    double lev=Double.NaN,prev_price=Double.NaN;
	    long vcheck=0,vscheck=0,taumin_val=Long.MAX_VALUE;
            gloop:for(int i=0,cur_pos=-1;i<tsize;i++)
	    {
		final BookItem t=tb[i];
		final double p=t.price;
		// check for poison
		if(flglow?(prev_price<p||t.buy_sell_indicator!=BookItem.BOOK_BUY):
		   (prev_price>p||t.buy_sell_indicator!=BookItem.BOOK_SELL))
		{
		    throw new RuntimeException("Ordering is broken currenttime="+currenttime);
		}
		prev_price=p;

		for(;!(flglow?(p>=lev):(p<=lev));)
		{
		    if(++cur_pos>=nt)
		    {
			break gloop;
		    }
		    lev=pricelevels[cur_pos];
		    if(cur_pos>0)
		    {
			if(flglow?
			   (pricelevels[cur_pos]>pricelevels[cur_pos-1]):
			   (pricelevels[cur_pos]<pricelevels[cur_pos-1]))
			{
			    throw new RuntimeException("Not sorted pricelevels."+
						       " currenttime="+currenttime);
			}
		    }
		}
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.shares+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.orderreferencenumber+"\t"+
				       p+"\t"+
				       cur_pos);
		}
		final long wtau=t.shares*((currenttime-t.time)/ModelParameters.T_SUMM_SCALE);
		v[cur_pos]+=t.shares;
		vsums[cur_pos]+=wtau;
	   	if((vcheck+=t.shares)<0)
		{
		    throw new RuntimeException("Overflow. currenttime="+currenttime);
		}
		if((vscheck+=wtau)<0)
		{
		    throw new RuntimeException("Overflow. currenttime="+currenttime);
		}
		final long tm=vscheck/vcheck;
		if(tm<=taumin_val)
		{
		    taumin_val=this.taumin_orders[cur_pos]=tm;
		    this.price_taumin[cur_pos]=t.price;
		}
		ptr_to_book[cur_pos]=i;
		this.price_cut[cur_pos]=t.price;
	    } // END gloop

	    // take previous if current not available
	    for(int i=1;i<nt;i++)
	    {
		final int i1=i-1;
		v[i]+=v[i1];
		vsums[i]+=vsums[i1];
		if(ptr_to_book[i]<0)
		{
		    this.price_cut[i]=this.price_cut[i1];
		    ptr_to_book[i]=ptr_to_book[i1];
		}
		if(this.taumin_orders[i]<0)
		{
		    this.taumin_orders[i]=this.taumin_orders[i1];
		    this.price_taumin[i]=this.price_taumin[i1];
		}
	    }
	    if(vcheck!=v[nt-1]||vscheck!=vsums[nt-1])
	    {
		throw new RuntimeException(
		"Check mismatch:"+
		" vcheck="+vcheck+
		" vcheck_expected["+(nt-1)+"]="+v[nt-1]+
		" vscheck="+vscheck+
		" vscheck_expected="+vsums[nt-1]+
		" currenttime="+currenttime);
	    }
	    // normalization
	    for(int i=0;i<nt;i++)
	    {
		final long vc=v[i];
		if(vc>0)
		{
		    this.tau_orders[i]=ModelParameters.T_SUMM_SCALE*(vsums[i]/vc);
		}
		else
		{
		    this.tau_orders[i]=-1;
		}
		if(this.taumin_orders[i]>=0)
		{
		    this.taumin_orders[i]*=ModelParameters.T_SUMM_SCALE;
		}
		else
		{
		    if(this.taumin_orders[i]!=Long.MIN_VALUE)
		    {
			throw new RuntimeException(
			    "taumin_orders["+i+"]="+this.taumin_orders[i]+
			    " currenttime="+currenttime);
		    }
		    this.taumin_orders[i]=-1;
		}
	    }
	    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    {
		final CalculateBookVolumePriceThreshold cvt=
		    new CalculateBookVolumePriceThreshold(this.flag_lower,this.orders);
		cvt.calculateThresholds(pricelevels,
					currenttime,
					0.5,
					pcurrent_buy,
					pcurrent_sell);
		for(int i=0;i<nt;i++)
		{
		    if(this.v_orders[i]!=cvt.out_shares[i]||
		       ptr_to_book[i]!=cvt.levels_interval_range_ptr[i]||
		       (ptr_to_book[i]+1)!=cvt.n_orders[i])
		    {
			throw new RuntimeException(
			    "Volume mismatch:"+
			    " v_orders["+i+"]="+this.v_orders[i]+
			    " cvt.out_shares["+i+"]="+cvt.out_shares[i]+
			    " ptr_to_book["+i+"]="+ptr_to_book[i]+
			    " cvt.levels_interval_range_ptr["+i+"]="+cvt.levels_interval_range_ptr[i]+
			    " cvt.n_orders["+i+"]="+cvt.n_orders[i]+
			    " currenttime="+currenttime);
		    }
		}
	    }
	}


	this.v_exe_thistype=new long [nt];
        this.v_exe_ot_thistype=new long [nt];
	this.v_flow_exe=new long [nt];
	this.v_flow_hidden_exe=new long [nt];
	this.v_flow_ot_exe=new long [nt];
	this.v_flow_ot_hidden_exe=new long [nt];
	this.tau_trades=new long [nt];
	this.price_tau=new double [nt];
	this.price_ot_tau=new double [nt];
	this.price_aver_tau=new double [nt];
	this.price_ot_aver_tau=new double [nt];
	if(true)
	{
	    final int type_market_exe=this.flag_lower?BookItem.BOOK_SELL:BookItem.BOOK_BUY,
		type_limit_exe=this.flag_lower?BookItem.BOOK_BUY:BookItem.BOOK_SELL;
	    final int tsize=this.trades.getSize();
	    final Trade [] tb=this.trades.getBuffer();

	    for(int cur_pos=0;cur_pos<nt;cur_pos++)
	    { // a lop over all delays
		long tbefore=currenttime-this.tau_orders[cur_pos],prevt=Long.MAX_VALUE;
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println("CUR_POS\t"+cur_pos+"\t"+
				       "tau\t"+this.tau_orders[cur_pos]+
				       "tbefore\t"+tbefore);
		    System.err.println("num\t"+
				       "time\t"+
				       "origtime\t"+
				       "shares\t"+
				       "tradetype\t"+
				       "buy_sell_indicator\t"+
				       "price\t"+
				       "matchnumber\t"+
				       "orderreferencenumber");
		}
	   	if(/* tbefore>0L && */ false && tbefore<trades_cleared_before)
		{
		    throw new RuntimeException("Something is terrible wrong with old data removal");
		}
		final long vlevel=this.v_orders[cur_pos];
		long vcompare=0,vscompare=0;
	        vloop:for(int i=tsize;i-- >0;)
		{
		    final Trade t=tb[i];
		    if(t.origtime>0)
		    {
			if(t.buy_sell_indicator==type_market_exe||
			   (t.buy_sell_indicator==type_limit_exe && t.origtime>=tbefore))
			{
			    vcompare+=t.shares;
			    if((vscompare+=t.shares*((currenttime-(t.buy_sell_indicator==type_market_exe?t.time:CalculateLiquidityDeficit2.getorigtimeadj(t.origtime,t.time)))/ModelParameters.T_SUMM_SCALE))<0)
			    {
				throw new RuntimeException("Overflow. currenttime="+currenttime);
			    }
			    if(vcompare>=vlevel)
			    {
				break vloop;
			    }
			}
		    }
		}
		if(vcompare>0)
		{
		    this.tau_trades[cur_pos]=ModelParameters.T_SUMM_SCALE*(vscompare/vcompare);
		}
		else
		{
		    this.tau_trades[cur_pos]=-1;
		}


		long vol_flow=0,vol_flow_hidden=0,volexe_thistype=0,
		    vol_ot_flow=0,vol_ot_flow_hidden=0,volexe_ot_thistype=0;
		double p_last_tau=Double.NaN,p_ot_last_tau=Double.NaN,paver=0,potaver=0;
	        gloop:for(int i=tsize;i-- >0;)
		{
		    final Trade t=tb[i];
		    if(t.time>prevt)
		    {
			throw new RuntimeException("The data is not sorted:"+
						   prevt+" and "+t.time+
						   " currenttime="+currenttime);
		    }
		    prevt=t.time;
		    if(PRINT_DATA_DEBUG)
		    {
			System.err.println(String.valueOf(i)+"\t"+
					   t.time+"\t"+
					   t.origtime+"\t"+
					   t.shares+"\t"+
					   t.tradetype+"\t"+
					   t.buy_sell_indicator+"\t"+
					   t.price+"\t"+
					   t.matchnumber+"\t"+
					   t.orderreferencenumber+
					   (t.order!=null?
					    "\t\t"+(t.time-t.order.time)+"\t"+
					    t.order.time+"\t"+
					    t.order.shares+"\t"+
					    t.order.price:
					    "\t\t-10\t\t\t"));
		    }
		    if(!(t.time>=tbefore))
		    {
			break gloop;
		    }

		    if(t.origtime>0)
		    {
			if(t.buy_sell_indicator==type_market_exe||
			   (t.buy_sell_indicator==type_limit_exe && t.origtime>=tbefore))
			{
			    vol_flow+=t.shares;
			    if(t.order==null)
			    {
				vol_flow_hidden+=t.shares;
			    }
			}
			if(t.buy_sell_indicator==type_limit_exe||
			   (t.buy_sell_indicator==type_market_exe && t.origtime>=tbefore))
			{
			    vol_ot_flow+=t.shares;
			    if(t.order==null)
			    {
				vol_ot_flow_hidden+=t.shares;
			    }
			}
		    }
		    if(t.buy_sell_indicator==type_limit_exe)
		    {
			volexe_thistype+=t.shares;
			p_last_tau=t.price;
			paver+=t.price*t.shares;
		    }
		    else if(t.buy_sell_indicator==type_market_exe)
		    {
			volexe_ot_thistype+=t.shares;
			p_ot_last_tau=t.price;
			potaver+=t.price*t.shares;
		    }
		}
	    	this.v_exe_thistype[cur_pos]=volexe_thistype;
		this.v_flow_exe[cur_pos]=vol_flow;
		this.v_flow_hidden_exe[cur_pos]=vol_flow_hidden;
	    	this.v_exe_ot_thistype[cur_pos]=volexe_ot_thistype;
		this.v_flow_ot_exe[cur_pos]=vol_ot_flow;
		this.v_flow_ot_hidden_exe[cur_pos]=vol_ot_flow_hidden;
		this.price_tau[cur_pos]=p_last_tau;
		this.price_ot_tau[cur_pos]=p_ot_last_tau;
		this.price_aver_tau[cur_pos]=paver/volexe_thistype;
		this.price_ot_aver_tau[cur_pos]=potaver/volexe_ot_thistype;
		if(this.tau_orders[cur_pos]<0 &&
		   (this.v_exe_thistype[cur_pos]!=0 ||
		    this.v_flow_exe[cur_pos]!=0 ||
		    this.v_flow_hidden_exe[cur_pos]!=0||
		    this.v_exe_ot_thistype[cur_pos]!=0 ||
		    this.v_flow_ot_exe[cur_pos]!=0 ||
		    this.v_flow_ot_hidden_exe[cur_pos]!=0))
		{
		   throw new RuntimeException(
		       "Something is terrible wrong with tau delay."+
		       " tau_orders["+cur_pos+"]="+this.tau_orders[cur_pos]+
		       " v_exe_thistype["+cur_pos+"]="+this.v_exe_thistype[cur_pos]+
		       " v_flow_exe["+cur_pos+"]="+this.v_flow_exe[cur_pos]+
		       " v_flow_hidden_exe["+cur_pos+"]="+this.v_flow_hidden_exe[cur_pos]+
		       " v_exe_ot_thistype["+cur_pos+"]="+this.v_exe_ot_thistype[cur_pos]+
		       " v_flow_ot_exe["+cur_pos+"]="+this.v_flow_ot_exe[cur_pos]+
		       " v_flow_ot_hidden_exe["+cur_pos+"]="+this.v_flow_ot_hidden_exe[cur_pos]+
		       " currenttime="+currenttime);
		}
	    } // END a lop over all delays

	    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    {
		final CalculateTradeVolume tv=new CalculateTradeVolume(this.trades,this.tmptradesbuf);
		for(int i=0;i<nt;i++)
		{
		    tv.calculateTradeVolume(new long []{this.tau_orders[i]},
					    currenttime,
					    0.5,
					    0.92,
					    0.5,
					    pcurrent_buy,
					    pcurrent_sell,
					    Long.MIN_VALUE);
		    final long v_all_buy,v_all_sell,v_flow_allbuy_tausell,v_flow_allsell_taubuy;
		    final double p_t0_buy,p_t0_sell;
		    if(this.flag_lower)
		    {
			v_all_buy=this.v_exe_thistype[i];
			v_all_sell=this.v_exe_ot_thistype[i];
			v_flow_allsell_taubuy=this.v_flow_exe[i];
			v_flow_allbuy_tausell=this.v_flow_ot_exe[i];
			p_t0_buy=this.price_tau[i];
			p_t0_sell=this.price_ot_tau[i];
		    }
		    else
		    {
			v_all_buy=this.v_exe_ot_thistype[i];
			v_all_sell=this.v_exe_thistype[i];
			v_flow_allsell_taubuy=this.v_flow_ot_exe[i];
			v_flow_allbuy_tausell=this.v_flow_exe[i];
			p_t0_buy=this.price_ot_tau[i];
			p_t0_sell=this.price_tau[i];
		    }

		    if(v_all_buy!=tv.v_alltrades_buy_tau[0]||
		       v_all_sell!=tv.v_alltrades_sell_tau[0]||
		       Double.isNaN(p_t0_buy)!=Double.isNaN(tv.trade_t0_buy_tau[0])||
		       (!Double.isNaN(p_t0_buy) && p_t0_buy!=tv.trade_t0_buy_tau[0])||
		       Double.isNaN(p_t0_sell)!=Double.isNaN(tv.trade_t0_sell_tau[0])||
		       (!Double.isNaN(p_t0_sell) && p_t0_sell!=tv.trade_t0_sell_tau[0])||

		       // FIXME, change the conditions below after origination time interpolation is done
		       //this.v_flow_hidden_exe[i]!=0||this.v_flow_ot_hidden_exe[i]!=0||
		       v_flow_allbuy_tausell!=
		       (tv.v_alltrades_buy_tau[0]/* -tv.v_hiddentrades_buy_tau[0] */+tv.v_eo_sell_tau[0])||
		       v_flow_allsell_taubuy!=
		       (tv.v_alltrades_sell_tau[0]/* -tv.v_hiddentrades_sell_tau[0] */+tv.v_eo_buy_tau[0]))
		    {
			throw new RuntimeException(
			    "Volume mismatch:"+
			    " orig tau["+i+"]="+this.tau_orders[i]+
			    " v_all_buy="+v_all_buy+
			    " tv.v_alltrades_buy_tau="+tv.v_alltrades_buy_tau[0]+
			    " v_all_sell="+v_all_sell+
			    " tv.v_alltrades_sell_tau="+tv.v_alltrades_sell_tau[0]+
			    " p_t0_buy="+p_t0_buy+
			    " tv.trade_t0_buy_tau="+tv.trade_t0_buy_tau[0]+
			    " p_t0_sell="+p_t0_sell+
			    " tv.trade_t0_sell_tau="+tv.trade_t0_sell_tau[0]+

			    " v_flow_allbuy_tausell="+v_flow_allbuy_tausell+
			    " tv.v_alltrades_buy_tau="+tv.v_alltrades_buy_tau[0]+
			    " tv.v_hiddentrades_buy_tau="+tv.v_hiddentrades_buy_tau[0]+
			    " tv.v_eo_sell_tau="+tv.v_eo_sell_tau[0]+

			    " v_flow_allsell_taubuy="+v_flow_allsell_taubuy+
			    " tv.v_alltrades_sell_tau="+tv.v_alltrades_sell_tau[0]+
			    " tv.v_hiddentrades_sell_tau="+tv.v_hiddentrades_sell_tau[0]+
			    " tv.v_eo_buy_tau="+tv.v_eo_buy_tau[0]+

			    " v_flow_hidden_exe["+i+"]="+this.v_flow_hidden_exe[i]+
			    " v_flow_ot_hidden_exe["+i+"]="+this.v_flow_ot_hidden_exe[i]+

			    " this.flag_lower="+this.flag_lower+
			    " currenttime="+currenttime);
		    }
		}
	    } // END OF TESTS
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("lev\t"+
			       "price_cut\t"+
			       "v_orders\t"+
			       "tau_orders\t"+
			       "price_taumin\t"+
			       "taumin_orders\t"+
			       "tau_trades\t"+
			       "v_flow_exe\t"+
			       "v_flow_hidden_exe\t");
	    for(int i=0;i<nt;i++)
	    {
		System.err.println(String.valueOf(pricelevels[i])+"\t"+
				   this.price_cut[i]+"\t"+
				   this.v_orders[i]+"\t"+
				   this.tau_orders[i]+"\t"+
				   this.price_taumin[i]+"\t"+
				   this.taumin_orders[i]+"\t"+
				   this.tau_trades[i]+"\t"+
				   this.v_flow_exe[i]+"\t"+
				   this.v_flow_hidden_exe[i]+"\t");
	    }
	}

    }
}
