package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2009
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;

/** Liquidity deficit v10.
 *  @version $Id: CalculateLiquidityDeficit10.java,v 1.64 2011/11/05 20:27:28 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit10 {

    private final Distribution<BookItem> buy,sell;
    private final Distribution<Trade> trades;

    CalculateLiquidityDeficit10(final Distribution<BookItem> buy,
				final Distribution<BookItem> sell,
				final Distribution<Trade> trades,
				final Distribution<BookItem> tmpordersbuf,
				final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
    }

    CalculateKendall.TradeKStats [] tdata;
    CalculateKendall.TradeKT2Stats [] ktdata;
    CalculateKendall.TradeDStats [] tfmdata;
    CalculateKendall.TradeDSStats [] tdsdata;
    long [] t_vol_median;

    void clear()
    {
	this.tdata=null;
	this.ktdata=null;
	this.tfmdata=null;
	this.t_vol_median=null;
    }


    void calculateLiquidityDeficit(final long [] delays,
				   final long currenttime,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final long trades_cleared_before)
    {
	final int nt=delays.length;
	final boolean PRINT_DATA_DEBUG=false;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tN_ORDERS BUY\t"+this.buy.getSize()+
			       "\tN_ORDERS SELL\t"+this.sell.getSize()+
			       "\tnt\t"+nt+
			       "\ncur_buy\t"+pcurrent_buy+
			       "\tcur_sell\t"+pcurrent_sell);
	}

	final Trade [] tb=this.trades.getBuffer();
	final int tsize=trades.getSize();
	this.tdata=new CalculateKendall.TradeKStats [nt];
	this.ktdata=new CalculateKendall.TradeKT2Stats [nt];
	this.tfmdata=new CalculateKendall.TradeDStats [nt];
	this.t_vol_median=new long [nt];
	this.tdsdata=new CalculateKendall.TradeDSStats [nt];

	for(int i=0;i<nt;i++)
	{
	    final int ptr=CalculateLiquidityDeficit9.findPtrToT_Inside(currenttime,currenttime-delays[i],tsize,tb,null);
	    this.tdata[i]=CalculateKendall.calculateTradesKendallTT(
		currenttime,
		tb,
		ptr,
		tsize);
	    this.ktdata[i]=CalculateKendall.calculateTradesKendallTT2(
		currenttime,
		tb,
		ptr,
		tsize);
	    if(true)
	    {
		UnitTests.checkDouble(this.tdata[i].k1v,"tdata["+i+"].k1v",this.ktdata[i].k1v,"ktdata["+i+"].k1v");
		UnitTests.checkDouble(this.tdata[i].k2v,"tdata["+i+"].k2v",this.ktdata[i].k2v,"ktdata["+i+"].k2v");
		UnitTests.checkDouble(this.tdata[i].k3v,"tdata["+i+"].k3v",this.ktdata[i].k3v,"ktdata["+i+"].k3v");
	    }
	    this.tfmdata[i]=CalculateKendall.calculateTradesKendallDD(
		currenttime,
		tb,
		ptr,
		tsize);
	    final int ptrv=CalculateLiquidityDeficit9.findPtrToV_Inside(currenttime,
									1+this.tfmdata[i].vi/2,
									tsize,
									tb);
	    t_vol_median[i]=ptrv>=0?tb[ptrv].time:currenttime;
	    this.tdsdata[i]=CalculateKendall.calculateTradesKendallDS(
		currenttime,
		tb,
		ptr,
		tsize);
	}
    }
}
