package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit v11.
 *  @version $Id: CalculateLiquidityDeficit11.java,v 1.38 2010/04/13 13:45:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit11 {

    private final Distribution<BookItem> buy,sell,tmpordersbuf;
    private final Distribution<Trade> trades;

    CalculateLiquidityDeficit11(final Distribution<BookItem> buy,
				final Distribution<BookItem> sell,
				final Distribution<Trade> trades,
				final Distribution<BookItem> tmpordersbuf,
				final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
	this.tmpordersbuf=tmpordersbuf;
    }


    CalculateKendall.BookTradesBKStats [] book_buy=null,book_sell=null;
    CalculateKendall.BookKStats [] bdata=null;
    CalculateKendall.TradeKStats [] tdata=null;

    void clear()
    {
	this.book_buy=this.book_sell=null;
	this.bdata=null;
	this.tdata=null;
    }


    void calculateLiquidityDeficit(final double [] buy_pricelevels,
				   final double [] sell_pricelevels,
				   final long currenttime,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final long trades_cleared_before,
				   final long vol_lev0)
    {
	final int nt=buy_pricelevels.length;
	if(sell_pricelevels.length!=nt)
	{
	    throw new IllegalArgumentException();
	}

	final boolean PRINT_DATA_DEBUG=false;
	final long vol_fact=1;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tN_ORDERS BUY\t"+this.buy.getSize()+
			       "\tN_ORDERS SELL\t"+this.sell.getSize()+
			       "\tnt\t"+nt+
			       "\ncur_buy\t"+pcurrent_buy+
			       "\tcur_sell\t"+pcurrent_sell);
	}


	final int bsizebuy=this.buy.getSize(),bsizesell=this.sell.getSize(),tsize=this.trades.getSize();
	final BookItem [] bbuy=this.buy.getBuffer(),bsell=this.sell.getBuffer(),tmp=this.tmpordersbuf.getBuffer();
	final Trade [] tb=this.trades.getBuffer();

	final double [] lev=new double [nt];

	final int [] ptr_book_buy=new int [nt],nl_buy=new int [nt];
	final double [] paver_buy=new double [nt];
	final long [] vol_book_buy=new long [nt],tau_book_unnormalized_buy=new long [nt],tau_buy=new long [nt];
	for(int i=0;i<nt;i++)
	{
	    lev[i]=buy_pricelevels[i]-OrdersFlow.EPS;
	}
	CalculateLiquidityDeficit5.calculateBookLevelsStat(
	    lev,
	    currenttime,
	    true,
	    bbuy,
	    bsizebuy,
	    vol_book_buy,
	    tau_book_unnormalized_buy,
	    tau_buy,
	    ptr_book_buy,
	    paver_buy,
	    nl_buy);


	final int [] ptr_book_sell=new int [nt],nl_sell=new int [nt];
	final double [] paver_sell=new double [nt];
	final long [] vol_book_sell=new long [nt],tau_book_unnormalized_sell=new long [nt],tau_sell=new long [nt];
	for(int i=0;i<nt;i++)
	{
	    lev[i]=sell_pricelevels[i]+OrdersFlow.EPS;
	}
	CalculateLiquidityDeficit5.calculateBookLevelsStat(
	    lev,
	    currenttime,
	    false,
	    bsell,
	    bsizesell,
	    vol_book_sell,
	    tau_book_unnormalized_sell,
	    tau_sell,
	    ptr_book_sell,
	    paver_sell,
	    nl_sell);



	this.book_buy=new CalculateKendall.BookTradesBKStats [nt];
	this.book_sell=new CalculateKendall.BookTradesBKStats [nt];
	this.bdata=new CalculateKendall.BookKStats [nt];
	this.tdata=new CalculateKendall.TradeKStats [nt];
	for(int i=0;i<nt;i++)
	{
	    this.book_buy[i]=CalculateKendall.calculateBookTradesBKStats(
		currenttime,
		bbuy,
		0,
		1+ptr_book_buy[i],
		tb,
		tsize);
	    this.book_buy[i].checkTauAndV(vol_book_buy[i],tau_buy[i],nl_buy[i]);

	    this.book_sell[i]=CalculateKendall.calculateBookTradesBKStats(
		currenttime,
		bsell,
		0,
		1+ptr_book_sell[i],
		tb,
		tsize);
	    this.book_sell[i].checkTauAndV(vol_book_sell[i],tau_sell[i],nl_sell[i]);


	    this.bdata[i]=CalculateKendall.calculateBookKStats(currenttime,
							       bbuy,
							       1+ptr_book_buy[i],
							       bsell,
							       1+ptr_book_sell[i],
							       tmp);
	    this.bdata[i].checkTauAndV(vol_book_buy[i],tau_buy[i],nl_buy[i],
				       vol_book_sell[i],tau_sell[i],nl_sell[i]);

	    final long v_cut=vol_fact*(vol_book_buy[i]+vol_book_sell[i]);
	    final int ptr_vol=CalculateLiquidityDeficit9.findPtrToV(currenttime,v_cut,tsize,tb);


	    this.tdata[i]=CalculateKendall.calculateTradesKendallTT(
		currenttime,
		tb,
		ptr_vol,
		tsize);
	}

    }




}

