package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit ver 12.
 *  @version $Id: CalculateLiquidityDeficit12.java,v 1.14 2010/02/11 20:22:48 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit12 {

    private final Distribution<BookItem> buy,sell,tmpordersbuf;
    private final Distribution<Trade> trades;

    CalculateLiquidityDeficit12(final Distribution<BookItem> buy,
				final Distribution<BookItem> sell,
				final Distribution<Trade> trades,
				final Distribution<BookItem> tmpordersbuf,
				final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
	this.tmpordersbuf=tmpordersbuf;
    }


    CalculateKendall.BookTradesBKStats [] book_buy=null,book_sell=null;
    CalculateKendall.BookTradesBKStats [] book_buy_pr=null,book_sell_pr=null;
    CalculateKendall.BookTradesK2Stats [] book_buy_dt=null,book_sell_dt=null;
    CalculateKendall.TradesK1K2K3 [] book_buy_tau=null,book_sell_tau=null;

    void clear()
    {
	this.book_buy=this.book_sell=null;
	this.book_buy_pr=this.book_sell_pr=null;
	this.book_buy_dt=this.book_sell_dt=null;
	this.book_buy_tau=this.book_sell_tau=null;
    }


    void calculateLiquidityDeficit(final long currenttime,
				   final int n_levels_trades,
				   final long trades_cleared_before)
    {
	final boolean PRINT_DATA_DEBUG=false;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tN_ORDERS BUY\t"+this.buy.getSize()+
			       "\tN_ORDERS SELL\t"+this.sell.getSize()+
			       "\tn_levels_trades\t"+n_levels_trades);
	}

	final int nt=n_levels_trades,bsizebuy=this.buy.getSize(),bsizesell=this.sell.getSize(),tsize=this.trades.getSize();
	final BookItem [] bbuy=this.buy.getBuffer(),bsell=this.sell.getBuffer(),tmpb=this.tmpordersbuf.getBuffer();
	final Trade [] tb=this.trades.getBuffer();

	final int [] ptr_book_buy=new int [nt];
	final long [] vb_buy=new long [nt],sv_nn_buy=new long [nt];
	CalculateLiquidityDeficit3.calculateBookLevelsTau(
	    currenttime,
	    nt,
	    PRINT_DATA_DEBUG,
	    0,
	    bbuy,
	    bsizebuy,
	    true,

	    // output:
	    ptr_book_buy,
	    vb_buy, // volume per level (not accumulated)
	    sv_nn_buy // unnormalized tau (not accumulated)
	);


	final int [] ptr_book_sell=new int [nt];
	final long [] vb_sell=new long [nt],sv_nn_sell=new long [nt];
	CalculateLiquidityDeficit3.calculateBookLevelsTau(
	    currenttime,
	    nt,
	    PRINT_DATA_DEBUG,
	    0,
	    bsell,
	    bsizesell,
	    false,

	    // output:
	    ptr_book_sell,
	    vb_sell, // volume per level (not accumulated)
	    sv_nn_sell  // unnormalized tau (not accumulated)
        );

	this.book_buy=new CalculateKendall.BookTradesBKStats [nt];
	this.book_sell=new CalculateKendall.BookTradesBKStats [nt];
	this.book_buy_pr=new CalculateKendall.BookTradesBKStats [nt];
	this.book_sell_pr=new CalculateKendall.BookTradesBKStats [nt];
	this.book_buy_dt=new CalculateKendall.BookTradesK2Stats [nt];
	this.book_sell_dt=new CalculateKendall.BookTradesK2Stats [nt];
	this.book_buy_tau=new CalculateKendall.TradesK1K2K3 [nt];
	this.book_sell_tau=new CalculateKendall.TradesK1K2K3 [nt];

	long vbuy=0,staubuy=0,vsell=0,stausell=0;
	int nl_buy=0,no_buy=0,nl_sell=0,no_sell=0;
	for(int i=0;i<nt;i++)
	{
	    if(ptr_book_buy[i]>=0)
	    {
		no_buy=1+ptr_book_buy[i];
		nl_buy++;
	    }
	    vbuy+=vb_buy[i];
	    staubuy+=sv_nn_buy[i];
	    final long tau_buy=vbuy>0?(staubuy/vbuy)*ModelParameters.T_SUMM_SCALE:-1;
	    this.book_buy[i]=CalculateKendall.calculateBookTradesBKStats(
		currenttime,
		bbuy,
		0,
		no_buy,
		tb,
		tsize);
	    this.book_buy[i].checkTauAndV(vbuy,tau_buy,nl_buy);
	    this.book_buy_pr[i]=CalculateKendall.calculateBookTradesBKStatsIntraLevelRemoved(
		currenttime,
		bbuy,
		ptr_book_buy,
		i+1,
		tb,
		tsize);
	    this.book_buy_pr[i].checkTauAndV(vbuy,tau_buy,nl_buy);
	    this.book_buy_dt[i]=CalculateKendall.calculateBookTradesK2Stats(
		currenttime,
		bbuy,
		no_buy,
		tb,
		tsize,
		tmpb);
	    this.book_buy_dt[i].checkTauAndV(vbuy,tau_buy);
	    this.book_buy_tau[i]=CalculateKendall.calculateTradesK1K2K3(currenttime,
									tb,
									tsize,
									currenttime-tau_buy,
									this.book_buy_dt[i].p0);


	    if(ptr_book_sell[i]>=0)
	    {
		no_sell=1+ptr_book_sell[i];
		nl_sell++;
	    }
	    vsell+=vb_sell[i];
	    stausell+=sv_nn_sell[i];
	    final long tau_sell=vsell>0?(stausell/vsell)*ModelParameters.T_SUMM_SCALE:-1;
	    this.book_sell[i]=CalculateKendall.calculateBookTradesBKStats(
		currenttime,
		bsell,
		0,
		no_sell,
		tb,
		tsize);
	    this.book_sell[i].checkTauAndV(vsell,tau_sell,nl_sell);
	    this.book_sell_pr[i]=CalculateKendall.calculateBookTradesBKStatsIntraLevelRemoved(
		currenttime,
		bsell,
		ptr_book_sell,
		i+1,
		tb,
		tsize);
	    this.book_sell_pr[i].checkTauAndV(vsell,tau_sell,nl_sell);
	    this.book_sell_dt[i]=CalculateKendall.calculateBookTradesK2Stats(
		currenttime,
		bsell,
		no_sell,
		tb,
		tsize,
		tmpb);
	    this.book_sell_dt[i].checkTauAndV(vsell,tau_sell);
	    this.book_sell_tau[i]=CalculateKendall.calculateTradesK1K2K3(currenttime,
									 tb,
									 tsize,
									 currenttime-tau_sell,
									 this.book_sell_dt[i].p0);
	}
    }


    CalculateKendall.BookTradesBKStats getBookBuyLiquidityLast()
    {
	final double p0_sell=this.book_sell[0].p0,
	    s=p0_sell-this.book_buy[0].p0,
	    pcut=p0_sell+s+OrdersFlow.EPS;
	CalculateKendall.BookTradesBKStats pe=null;
	for(CalculateKendall.BookTradesBKStats p:this.book_buy)
	{
	    if(p.k1_ptrades_po>=pcut)
	    {
		pe=p;
	    }
	    else
	    {
		break;
	    }
	}
	return pe;
    }

    CalculateKendall.BookTradesBKStats getBookSellLiquidityLast()
    {
	final double p0_buy=this.book_buy[0].p0,
	    s=this.book_sell[0].p0-p0_buy,
	    pcut=p0_buy-s-OrdersFlow.EPS;
	CalculateKendall.BookTradesBKStats pe=null;
	for(CalculateKendall.BookTradesBKStats p:this.book_sell)
	{
	    if(p.k1_ptrades_po<=pcut)
	    {
		pe=p;
	    }
	    else
	    {
		break;
	    }
	}
	return pe;
    }
}
