package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit v13.
 *  @version $Id: CalculateLiquidityDeficit13.java,v 1.8 2010/04/14 09:37:30 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit13 {

    private final Distribution<BookItem> buy,sell,tmpordersbuf;
    private final Distribution<Trade> trades,tmptradesbuf;

    CalculateLiquidityDeficit13(final Distribution<BookItem> buy,
				final Distribution<BookItem> sell,
				final Distribution<Trade> trades,
				final Distribution<BookItem> tmpordersbuf,
				final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
	this.tmpordersbuf=tmpordersbuf;
	this.tmptradesbuf=tmptradesbuf;
    }


    CalculateKendall.TradeEMoments [] tdata=null;

    void clear()
    {
	this.tdata=null;
    }

    void calculateLiquidityDeficit(final long [] delays,
				   final long currenttime,
				   final long trades_cleared_before)
    {
	calculateLiquidityDeficitRecurrent(delays,currenttime,trades_cleared_before,null);
    }


    void calculateLiquidityDeficitRecurrent(final long [] delays,
					    final long currenttime,
					    final long trades_cleared_before,
					    final CalculateKendall.TradeEMoments [] prev)
    {

	final int nt=delays.length;
	final boolean PRINT_DATA_DEBUG=false;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tN_ORDERS BUY\t"+this.buy.getSize()+
			       "\tN_ORDERS SELL\t"+this.sell.getSize()+
			       "\tnt\t"+nt);
	}

	final Trade [] tb=this.trades.getBuffer();
	final int tsize=trades.getSize();
	this.tdata=new CalculateKendall.TradeEMoments [nt];

	for(int i=0;i<nt;i++)
	{
	    this.tdata[i]=CalculateKendall.calculateEMoments(
		currenttime,
		delays[i],
		tb,
		tsize,
		prev!=null?prev[i]:null);
	}
    }

}

