package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit v15.
 *  @version $Id: CalculateLiquidityDeficit15.java,v 1.40 2010/12/14 23:01:00 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit15 {

    private final Distribution<Trade> trades,tmptradesbuf;

    CalculateLiquidityDeficit15(final Distribution<Trade> trades,
				final Distribution<Trade> tmptradesbuf)
    {
	this.trades=trades;
	this.tmptradesbuf=tmptradesbuf;
    }


    static class VXM {
	final long currenttime,tau;

	final CalculateKendall.TradeIP tip;
	final CalculateKendall.OTradeIP otip;
	final CalculateKendall.DBTradeIP dbtip;
	final CalculateKendall.DBSTradeIP<CalculateKendall.TradeIPBSData> dbstip;
	final CalculateKendall.DTBSTradeIP dtbstip;
	final CalculateKendall.DTradeIPS sdtip;
	final CalculateKendall.DBSTradeIP<CalculateKendall.TradeIPTBSData> dpt;
	final CalculateKendall.DIPTradeIP kdt;

	VXM(final long currenttime,
	    final long tau,
	    final CalculateKendall.TradeIP tip,
	    final CalculateKendall.OTradeIP otip,
	    final CalculateKendall.DBTradeIP dbtip,
	    final CalculateKendall.DBSTradeIP<CalculateKendall.TradeIPBSData> dbstip,
	    final CalculateKendall.DTBSTradeIP dtbstip,
	    final CalculateKendall.DTradeIPS sdtip,
	    final CalculateKendall.DBSTradeIP<CalculateKendall.TradeIPTBSData> dpt,
	    final CalculateKendall.DIPTradeIP kdt)
	{
	    this.currenttime=currenttime;
	    this.tau=tau;
	    this.tip=tip;
	    this.otip=otip;
	    this.dbtip=dbtip;
	    this.dbstip=dbstip;
	    this.dtbstip=dtbstip;
	    this.sdtip=sdtip;
	    this.dpt=dpt;
	    this.kdt=kdt;
	}
    }


    VXM [] tdata=null;

    void clear()
    {
	this.tdata=null;
    }


    void calculateLiquidityDeficitRecurrent(final long [] delays,
					    final long currenttime,
					    final long trades_cleared_before,
					    final VXM [] prev)
    {
	final int nt=delays.length,N_EM=5;
	final boolean PRINT_DATA_DEBUG=false;


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tN TRADES=\t"+this.trades.getSize()+
			       "\tnt\t"+nt);
	}

	final boolean flag_skipnoorigtime=true;
	this.tdata=new VXM [nt];
	for(int i=0;i<nt;i++)
	{
	    final VXM amprev=prev!=null?prev[i]:null;
	    final long tau=delays[i];
	    this.tdata[i]=new VXM(currenttime,
				  tau,
				  CalculateKendall.calculateTradeIP(N_EM,
								    currenttime,
								    tau,
								    this.trades.getBuffer(),
								    this.trades.getSize(),
								    amprev!=null?amprev.tip:null),
				  CalculateKendall.calculateOTradeIP(N_EM,
								     currenttime,
								     tau,
								     this.trades.getBuffer(),
								     this.trades.getSize(),
								     amprev!=null?amprev.otip:null),
				  CalculateKendall.DBTradeIP.calculateDBTradeIP(N_EM,
								      currenttime,
								      tau,
								      this.trades.getBuffer(),
								      this.trades.getSize(),
								      amprev!=null?amprev.dbtip:null),
				  CalculateKendall.TradeIPBSData.calculateDBSTradeIP(N_EM,
								      currenttime,
								      tau,
								      this.trades.getBuffer(),
								      this.trades.getSize(),
								      amprev!=null?amprev.dbstip:null),
				  CalculateKendall.DTBSTradeIP.calculateDTBSTradeIP(N_EM,
								      currenttime,
								      tau,
								      this.trades.getBuffer(),
								      this.trades.getSize(),
								      amprev!=null?amprev.dtbstip:null),
				  CalculateKendall.DTradeIPS.calculateDTradeIPS(N_EM,
								      currenttime,
								      tau,
								      this.trades.getBuffer(),
								      this.trades.getSize(),
								      amprev!=null?amprev.sdtip:null),
				  CalculateKendall.TradeIPTBSData.calculateDBSTradeIP(N_EM,
								      currenttime,
								      tau,
								      this.trades.getBuffer(),
								      this.trades.getSize(),
								      amprev!=null?amprev.dpt:null),
				  CalculateKendall.DIPTradeIP.calculateDIPTradeIP(N_EM,
								      currenttime,
								      tau,
								      this.trades.getBuffer(),
								      this.trades.getSize(),
								      amprev!=null?amprev.kdt:null)
				  );
	}
    }
 }

