package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2010
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit v17.
 *  @version $Id: CalculateLiquidityDeficit17.java,v 1.20 2012/12/24 17:24:18 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit17 {

    private final Distribution<Trade> trades;

    CalculateLiquidityDeficit17(final Distribution<Trade> trades)
    {
	this.trades=trades;
    }


    /** Results container.
     */
    static class VQ {
	final long currenttime,tau;
	final QQVMLaguerre qqvmll;
	final QQVMLegendreShifted qqvmpp;
	final ZQVMLaguerre zqvmll;
	final ZQVMLegendreShifted zqvmpp;
	final ZVQVMLaguerre zvqvmll;
	final ZVQVMLegendreShifted zvqvmpp;
	final BQVMLaguerre bqvmll;
	final BQVMLegendreShifted bqvmpp;
	final BiQVMLaguerre biqvmll;
	final BiQVMLegendreShifted biqvmpp;
	final DiQVMLaguerre diqvmll;
	final DiQVMLegendreShifted diqvmpp;
	final BjQVMLaguerre bjqvmll;
	final BjQVMLegendreShifted bjqvmpp;
	final DjQVMLaguerre djqvmll;
	final DjQVMLegendreShifted djqvmpp;
	VQ(final long currenttime,
	   final long tau,
	   final ZQVMLaguerre zqvmll,
	   final ZQVMLegendreShifted zqvmpp,
	   final ZVQVMLaguerre zvqvmll,
	   final ZVQVMLegendreShifted zvqvmpp,
	   final QQVMLaguerre qqvmll,
	   final QQVMLegendreShifted qqvmpp,
	   final BQVMLaguerre bqvmll,
	   final BQVMLegendreShifted bqvmpp,
	   final BiQVMLaguerre biqvmll,
	   final BiQVMLegendreShifted biqvmpp,
	   final DiQVMLaguerre diqvmll,
	   final DiQVMLegendreShifted diqvmpp,
	   final BjQVMLaguerre bjqvmll,
	   final BjQVMLegendreShifted bjqvmpp,
	   final DjQVMLaguerre djqvmll,
	   final DjQVMLegendreShifted djqvmpp)
	{
	    this.currenttime=currenttime;
	    this.tau=tau;
	    this.zqvmll=zqvmll;
	    this.zqvmpp=zqvmpp;
	    this.zvqvmll=zvqvmll;
	    this.zvqvmpp=zvqvmpp;
	    this.qqvmll=qqvmll;
	    this.qqvmpp=qqvmpp;
	    this.bqvmll=bqvmll;
	    this.bqvmpp=bqvmpp;
	    this.biqvmll=biqvmll;
	    this.biqvmpp=biqvmpp;
	    this.diqvmll=diqvmll;
	    this.diqvmpp=diqvmpp;
	    this.bjqvmll=bjqvmll;
	    this.bjqvmpp=bjqvmpp;
	    this.djqvmll=djqvmll;
	    this.djqvmpp=djqvmpp;
	}
    }
    VQ [] tdata=null;


    void clear()
    {
	this.tdata=null;
    }

    void calculateLiquidityDeficitRecurrent(final int neq_t,
					    final long [] delays,
					    final long currenttime,
					    final long trades_cleared_before,
					    final VQ [] prev)
    {
	final int nt=delays.length;
	final boolean PRINT_DATA_DEBUG=false;


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tN TRADES=\t"+this.trades.getSize()+
			       "\tnt\t"+nt);
	}

	this.tdata=new VQ [nt];
	for(int i=0;i<nt;i++)
	{
	    final VQ amprev=prev!=null?prev[i]:null;
	    final long tau=delays[i];
	    this.tdata[i]=new VQ(currenttime,
				 tau,
				 ZQVMLaguerre.calculate(neq_t,
							currenttime,
							tau,
							this.trades.getBuffer(),
							this.trades.getSize(),
							amprev!=null?amprev.zqvmll:null),
				 ZQVMLegendreShifted.calculate(neq_t,
							       currenttime,
							       tau,
							       this.trades.getBuffer(),
							       this.trades.getSize(),
							       amprev!=null?amprev.zqvmpp:null),
				 ZVQVMLaguerre.calculate(neq_t,
							currenttime,
							tau,
							this.trades.getBuffer(),
							this.trades.getSize(),
							amprev!=null?amprev.zvqvmll:null),
				 ZVQVMLegendreShifted.calculate(neq_t,
							       currenttime,
							       tau,
							       this.trades.getBuffer(),
							       this.trades.getSize(),
							       amprev!=null?amprev.zvqvmpp:null),
				 QQVMLaguerre.calculate(neq_t,
							currenttime,
							tau,
							this.trades.getBuffer(),
							this.trades.getSize(),
							amprev!=null?amprev.qqvmll:null),
				 QQVMLegendreShifted.calculate(neq_t,
							       currenttime,
							       tau,
							       this.trades.getBuffer(),
							       this.trades.getSize(),
							       amprev!=null?amprev.qqvmpp:null),
				 BQVMLaguerre.calculate(neq_t,
							currenttime,
							tau,
							this.trades.getBuffer(),
							this.trades.getSize(),
							amprev!=null?amprev.bqvmll:null),
				 BQVMLegendreShifted.calculate(neq_t,
							       currenttime,
							       tau,
							       this.trades.getBuffer(),
							       this.trades.getSize(),
							       amprev!=null?amprev.bqvmpp:null),
				 BiQVMLaguerre.calculate(neq_t,
							currenttime,
							tau,
							this.trades.getBuffer(),
							this.trades.getSize(),
							amprev!=null?amprev.biqvmll:null),
				 BiQVMLegendreShifted.calculate(neq_t,
							       currenttime,
							       tau,
							       this.trades.getBuffer(),
							       this.trades.getSize(),
							       amprev!=null?amprev.biqvmpp:null),
				 DiQVMLaguerre.calculate(neq_t,
							currenttime,
							tau,
							this.trades.getBuffer(),
							this.trades.getSize(),
							amprev!=null?amprev.diqvmll:null),
				 DiQVMLegendreShifted.calculate(neq_t,
							       currenttime,
							       tau,
							       this.trades.getBuffer(),
							       this.trades.getSize(),
							       amprev!=null?amprev.diqvmpp:null),
				 BjQVMLaguerre.calculate(neq_t,
							currenttime,
							tau,
							this.trades.getBuffer(),
							this.trades.getSize(),
							amprev!=null?amprev.bjqvmll:null),
				 BjQVMLegendreShifted.calculate(neq_t,
							       currenttime,
							       tau,
							       this.trades.getBuffer(),
							       this.trades.getSize(),
							       amprev!=null?amprev.bjqvmpp:null),
				 DjQVMLaguerre.calculate(neq_t,
							currenttime,
							tau,
							this.trades.getBuffer(),
							this.trades.getSize(),
							amprev!=null?amprev.djqvmll:null),
				 DjQVMLegendreShifted.calculate(neq_t,
							       currenttime,
							       tau,
							       this.trades.getBuffer(),
							       this.trades.getSize(),
							       amprev!=null?amprev.djqvmpp:null));
	}
    }
}

