package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2010
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit v18.
 *  @version $Id: CalculateLiquidityDeficit18.java,v 1.7 2013/01/17 17:04:19 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit18 {

private final Distribution<Trade> trades;
private final Distribution<BookItem> buy,sell;

    CalculateLiquidityDeficit18(final Distribution<Trade> trades,
				final Distribution<BookItem> buy,
				final Distribution<BookItem> sell)
    {
	this.trades=trades;
	this.buy=buy;
	this.sell=sell;
    }


    /** Results container.
     */
    static class VQ {
	final long currenttime,tau;
	final DQVMLaguerre dqvmll;
	final DQVMLegendreShifted dqvmpp;
	final QVMFQVMDataBL b;
	final DoQVMLaguerre doqvmll;
	final DoQVMLegendreShifted doqvmpp;
	final HaarQVM haarqvm;
	final DkQVMLaguerre dkqvmll;
	final DkQVMLegendreShifted dkqvmpp;
	VQ(final long currenttime,
	   final long tau,
	   final DQVMLaguerre dqvmll,
	   final DQVMLegendreShifted dqvmpp,
	   final QVMFQVMDataBL b,
	   final DoQVMLaguerre doqvmll,
	   final DoQVMLegendreShifted doqvmpp,
	   final HaarQVM haarqvm,
	   final DkQVMLaguerre dkqvmll,
	   final DkQVMLegendreShifted dkqvmpp)
	{
	    this.currenttime=currenttime;
	    this.tau=tau;
	    this.dqvmll=dqvmll;
	    this.dqvmpp=dqvmpp;
	    this.b=b;
	    this.doqvmll=doqvmll;
	    this.doqvmpp=doqvmpp;
	    this.haarqvm=haarqvm;
	    this.dkqvmll=dkqvmll;
	    this.dkqvmpp=dkqvmpp;
	}
    }
    VQ [] tdata=null;


    void clear()
    {
	this.tdata=null;
    }

    void calculateLiquidityDeficitRecurrent(final int neq_t,
					    final long [] delays,
					    final long currenttime,
					    final long trades_cleared_before,
					    final VQ [] prev)
    {
	final int nt=delays.length;
	final boolean PRINT_DATA_DEBUG=false;


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tN TRADES=\t"+this.trades.getSize()+
			       "\tnt\t"+nt);
	}

	this.tdata=new VQ [nt];
	for(int i=0;i<nt;i++)
	{
	    final VQ amprev=prev!=null?prev[i]:null;
	    final long tau=delays[i];
	    this.tdata[i]=new VQ(currenttime,
				 tau,
				 DQVMLaguerre.calculate(neq_t,
							currenttime,
							tau,
							this.trades.getBuffer(),
							this.trades.getSize(),
							amprev!=null?amprev.dqvmll:null),
				 DQVMLegendreShifted.calculate(neq_t,
							       currenttime,
							       tau,
							       this.trades.getBuffer(),
							       this.trades.getSize(),
							       amprev!=null?amprev.dqvmpp:null),
				 QVMFQVMDataBL.calculate(neq_t,
							 currenttime,
							 tau,
							 this.buy.getBuffer(),
							 this.buy.getSize(),
							 this.sell.getBuffer(),
							 this.sell.getSize()),
				 DoQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.buy.getBuffer(),
							 this.buy.getSize(),
							 this.sell.getBuffer(),
							 this.sell.getSize()),
				 DoQVMLegendreShifted.calculate(neq_t,
							 currenttime,
							 tau,
							 this.buy.getBuffer(),
							 this.buy.getSize(),
							 this.sell.getBuffer(),
								this.sell.getSize()),
				 HaarQVM.calculate(neq_t,
						   currenttime,
						   this.buy.getBuffer(),
						   this.buy.getSize(),
						   this.sell.getBuffer(),
						   this.sell.getSize()),
				 DkQVMLaguerre.calculate(neq_t,
							currenttime,
							tau,
							this.trades.getBuffer(),
							this.trades.getSize(),
							amprev!=null?amprev.dkqvmll:null),
				 DkQVMLegendreShifted.calculate(neq_t,
							       currenttime,
							       tau,
							       this.trades.getBuffer(),
							       this.trades.getSize(),
							       amprev!=null?amprev.dkqvmpp:null));
	}
    }
}

