package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2010
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit v19.
 *  @version $Id: CalculateLiquidityDeficit19.java,v 1.8 2013/11/12 18:46:07 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit19 {

private final Distribution<Trade> trades;
private final Distribution<BookItem> buy,sell;

    CalculateLiquidityDeficit19(final Distribution<Trade> trades,
				final Distribution<BookItem> buy,
				final Distribution<BookItem> sell)
    {
	this.trades=trades;
	this.buy=buy;
	this.sell=sell;
    }


    /** Results container.
     */
    static class VQ {
	final long currenttime,tau;
	final EkQVMLaguerre ekqvmll;
	final EkQVMLegendreShifted ekqvmpp;
	final IkQVMLaguerre ikqvmll;
	final IkQVMLegendreShifted ikqvmpp;
	final JkQVMLaguerre jkqvmll;
	final JkQVMLegendreShifted jkqvmpp;
	final KkQVMLaguerre kkqvmll;
	final KkQVMLegendreShifted kkqvmpp;
	final LkQVMLaguerre lkqvmll;
	final LkQVMLegendreShifted lkqvmpp;
	final MkQVMLaguerre mkqvmll;
	final MkQVMLegendreShifted mkqvmpp;

	VQ(final long currenttime,
	   final long tau,
	   final EkQVMLaguerre ekqvmll,
	   final EkQVMLegendreShifted ekqvmpp,
	   final IkQVMLaguerre ikqvmll,
	   final IkQVMLegendreShifted ikqvmpp,
	   final JkQVMLaguerre jkqvmll,
	   final JkQVMLegendreShifted jkqvmpp,
	   final KkQVMLaguerre kkqvmll,
	   final KkQVMLegendreShifted kkqvmpp,
	   final LkQVMLaguerre lkqvmll,
	   final LkQVMLegendreShifted lkqvmpp,
	   final MkQVMLaguerre mkqvmll,
	   final MkQVMLegendreShifted mkqvmpp
	   )
	{
	    this.currenttime=currenttime;
	    this.tau=tau;
	    this.ekqvmll=ekqvmll;
	    this.ekqvmpp=ekqvmpp;
	    this.ikqvmll=ikqvmll;
	    this.ikqvmpp=ikqvmpp;
	    this.jkqvmll=jkqvmll;
	    this.jkqvmpp=jkqvmpp;
	    this.kkqvmll=kkqvmll;
	    this.kkqvmpp=kkqvmpp;
	    this.lkqvmll=lkqvmll;
	    this.lkqvmpp=lkqvmpp;
	    this.mkqvmll=mkqvmll;
	    this.mkqvmpp=mkqvmpp;
	}
    }
    VQ [] tdata=null;


    void clear()
    {
	this.tdata=null;
    }

    void calculateLiquidityDeficitRecurrent(final int neq_t,
					    final long [] delays,
					    final long currenttime,
					    final long trades_cleared_before,
					    final VQ [] prev)
    {
	final int nt=delays.length;
	final boolean PRINT_DATA_DEBUG=false;


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tN TRADES=\t"+this.trades.getSize()+
			       "\tnt\t"+nt);
	}

	this.tdata=new VQ [nt];
	for(int i=0;i<nt;i++)
	{
	    final VQ amprev=prev!=null?prev[i]:null;
	    final long tau=delays[i];
	    this.tdata[i]=new VQ(currenttime,
				 tau,
				 EkQVMLaguerre.calculate(neq_t,
							currenttime,
							tau,
							this.trades.getBuffer(),
							this.trades.getSize(),
							amprev!=null?amprev.ekqvmll:null),
				 EkQVMLegendreShifted.calculate(neq_t,
							       currenttime,
							       tau,
							       this.trades.getBuffer(),
							       this.trades.getSize(),
								amprev!=null?amprev.ekqvmpp:null),
				 IkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.ikqvmll:null),
				 IkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.ikqvmpp:null),
				 JkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.jkqvmll:null),
				 JkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.jkqvmpp:null),
				 KkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.kkqvmll:null),
				 KkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.kkqvmpp:null),
				 LkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.lkqvmll:null),
				 LkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.lkqvmpp:null),
				 MkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.mkqvmll:null),
				 MkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.mkqvmpp:null)
				);
	}
    }
}

