package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit. Updated version.
 *  @version $Id: CalculateLiquidityDeficit2.java,v 1.84 2009/10/11 09:21:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit2 {

    private static final long FACTOR_LIQUIDITY_AMPLIFICATION=1;// 2?

    private final OrdersFlow obuy=new OrdersFlow(true),osell=new OrdersFlow(false);
    private final MatchOrdersVariousShifts virtualshifts=new MatchOrdersVariousShifts();
    private final Distribution<MatchOrdersVariousShifts.MatchWithShift>
	allshifts=new Distribution<MatchOrdersVariousShifts.MatchWithShift>();

    private final Distribution<BookItem> buy,sell,tmpordersbuf;
    private final Distribution<Trade> trades,tmptradesbuf;

    CalculateLiquidityDeficit2(final Distribution<BookItem> buy,
			       final Distribution<BookItem> sell,
			       final Distribution<Trade> trades,
			       final Distribution<BookItem> tmpordersbuf,
			       final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
	this.tmpordersbuf=tmpordersbuf;
	this.tmptradesbuf=tmptradesbuf;
    }



    long [] vol_orders_buy=null,tau_orders_buy=null,
	vol_orders_sell=null,tau_orders_sell=null;
    double [] price_orders_levels_buy=null,price_orders_levels_sell=null,
	price_orders_aver_buy=null,price_orders_aver_sell=null,
	price_orders_taver_buy=null,price_orders_taver_sell=null,
	price_orders_sqtaver_buy=null,price_orders_sqtaver_sell=null;
    private double []
	w_orders_sqtaver_buy=null,w_orders_sqtaver_sell=null;


    MatchOrdersVariousShifts.MatchWithShift [] orders_mref=null;
    double dspread=Double.NaN;
    long []
	v_trades_flow_exe_buy=null,
	v_trades_flow_exe_sell=null,
	v_trades_exe_buy=null,
	v_trades_exe_sell=null;
    double [] price_trades_tau_buy=null,price_trades_tau_sell=null,
	price_trades_aver_tau_buy=null,price_trades_aver_tau_sell=null;

    long []
	v_cut_trades_flow_exe=null,
	tau_trades_buy=null,
	tau_trades_sell=null;


    long [] v_trades_buy_exe,
	tau_trades_ending_buy=null,
	tau_trades_limit_buy,
	tau_trades_market_sell,
	v_trades_sell_exe,
	tau_trades_ending_sell=null,
	tau_trades_limit_sell=null,
	tau_trades_market_buy=null,
	tau_trades_plev_buy=null,
	tau_trades_plev_sell=null;
    double [] price_trades_aver_tau_trades_buy=null,price_trades_aver_tau_trades_sell=null,
	price_trades_aver_tauF_trades_buy=null,price_trades_aver_tauF_trades_sell=null;
    long [] v_trades_tau_trades_buy=null,v_trades_tau_trades_sell=null;


    int [] ptr_book_buy=null,ptr_book_sell=null;
    double [] kendall_buy=null,kendall_sell=null;

    void clear()
    {

	this.vol_orders_buy=
	this.tau_orders_buy=
	this.vol_orders_sell=
	this.tau_orders_sell=
	this.v_cut_trades_flow_exe=
	this.tau_trades_buy=
	this.tau_trades_sell=null;
	this.price_orders_levels_buy=
	this.price_orders_levels_sell=
	this.price_orders_aver_buy=
        this.price_orders_aver_sell=
	this.price_orders_taver_buy=
        this.price_orders_taver_sell=
	this.price_orders_sqtaver_buy=
	this.price_orders_sqtaver_sell=
	this.w_orders_sqtaver_buy=
	this.w_orders_sqtaver_sell=null;
	this.orders_mref=null;
	this.dspread=Double.NaN;


	this.v_trades_flow_exe_buy=
	this.v_trades_flow_exe_sell=
	this.v_trades_exe_buy=
	this.v_trades_exe_sell=null;

	this.price_trades_tau_buy=
	this.price_trades_tau_sell=
	this.price_trades_aver_tau_buy=
        this.price_trades_aver_tau_sell=null;


	this.v_trades_buy_exe=
	this.tau_trades_ending_buy=
	this.tau_trades_limit_buy=
	this.tau_trades_market_sell=
	this.v_trades_sell_exe=
	this.tau_trades_ending_sell=
	this.tau_trades_limit_sell=
	this.tau_trades_market_buy=
	this.tau_trades_plev_buy=
	this.tau_trades_plev_sell=
	this.v_trades_tau_trades_buy=
        this.v_trades_tau_trades_sell=null;

	this.price_trades_aver_tau_trades_buy=
	this.price_trades_aver_tau_trades_sell=
	this.price_trades_aver_tauF_trades_buy=
	this.price_trades_aver_tauF_trades_sell=null;


	this.ptr_book_buy=this.ptr_book_sell=null;
	this.kendall_buy=this.kendall_sell=null;

	// clear internal buffers
	this.allshifts.clearAll();
	this.obuy.clear();
	this.osell.clear();
    }


    void calculateLiquidityDeficit(final double [] shiftlevels,
				   final long currenttime,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final long trades_cleared_before)
    {

	final boolean PRINT_DATA_DEBUG=false;//true;//false;
	    //currenttime==34434604000000L;
	    //currenttime==34354409000000L;
	    //currenttime==34223484000000L;
	    //currenttime>=9*3600*TraderInterface.TIME_SCALE && tsize>0;
	    //currenttime==34204365000000L;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println();
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tBUY ORDERS\t"+this.buy.getSize()+
			       "\tSELL ORDERS\t"+this.sell.getSize()+
			       "\tcur_buy\t"+pcurrent_buy+
			       "\tcur_sell\t"+pcurrent_sell);
	}



	// book time & volume
	calculateBookVirtualShifts(shiftlevels,
				   currenttime,
				   pcurrent_buy,
				   pcurrent_sell,
				   PRINT_DATA_DEBUG);

	if(true)
	{
	    final int nt=shiftlevels.length;
	    final long [] tmp_o=new long [nt],tmp_uo=new long [nt];
	    // FIXME, use optimized version
	    checkCalculateKendallRanks(this.ptr_book_buy,
				       this.buy.getBuffer(),
				       currenttime,
				       PRINT_DATA_DEBUG,
				       tmp_o,
				       tmp_uo);
	    this.kendall_buy=new double [nt];
	    for(int i=0;i<nt;i++)
	    {
		final long v=tmp_o[i]+tmp_uo[i];
		if(v>0)
		{
		    this.kendall_buy[i]=tmp_o[i]/((double)v);
		}
		else
		{
		    this.kendall_buy[i]=Double.NaN;
		}
	    }
	    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    { // expensive constrains
		final long [] v=new long [nt],
		    pl_tl=new long [nt],
		    pg_tl=new long [nt],
		    pl_tg=new long [nt],
		    pg_tg=new long [nt];
		check2CalculateKendallRanks(this.ptr_book_buy,
					    this.buy.getBuffer(),
					    currenttime,
					    PRINT_DATA_DEBUG,
					    false,
					    v,
					    pl_tl,
					    pg_tl,
					    pl_tg,
					    pg_tg);
		for(int i=0;i<nt;i++)
		{
		    if(v[i]!=this.vol_orders_buy[i]||
		       pl_tg[i]!=tmp_o[i]||
		       pl_tl[i]!=tmp_uo[i])
		    {
			throw new RuntimeException(
			    "Mismatch:"+
			     " v["+i+"]="+v[i]+
			    " vol_orders_buy["+i+"]="+this.vol_orders_buy[i]+
			    " pl_tg["+i+"]="+pl_tg[i]+
			    " tmp_o["+i+"]="+tmp_o[i]+
			    " pl_tl["+i+"]="+pl_tl[i]+
			    " tmp_uo["+i+"]="+tmp_uo[i]+
			    " currenttime="+currenttime);
		    }
		}

	    }
	    checkCalculateKendallRanks(this.ptr_book_sell,
				       this.sell.getBuffer(),
				       currenttime,
				       PRINT_DATA_DEBUG,
				       tmp_o,
				       tmp_uo);
	    this.kendall_sell=new double [nt];
	    for(int i=0;i<nt;i++)
	    {
		final long v=tmp_o[i]+tmp_uo[i];
		if(v>0)
		{
		    this.kendall_sell[i]=tmp_o[i]/((double)v);
		}
		else
		{
		    this.kendall_sell[i]=Double.NaN;
		}
	    }
	    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    { // expensive constrains
		final long [] v=new long [nt],
		    pl_tl=new long [nt],
		    pg_tl=new long [nt],
		    pl_tg=new long [nt],
		    pg_tg=new long [nt];
		check2CalculateKendallRanks(this.ptr_book_sell,
					    this.sell.getBuffer(),
					    currenttime,
					    PRINT_DATA_DEBUG,
					    true,
					    v,
					    pl_tl,
					    pg_tl,
					    pl_tg,
					    pg_tg);
		for(int i=0;i<nt;i++)
		{
		    if(v[i]!=this.vol_orders_sell[i]||
		       pg_tg[i]!=tmp_o[i]||
		       pg_tl[i]!=tmp_uo[i])
		    {
			throw new RuntimeException(
			    "Mismatch:"+
			    " v["+i+"]="+v[i]+
			    " vol_orders_sell["+i+"]="+this.vol_orders_sell[i]+
			    " pg_tg["+i+"]="+pg_tg[i]+
			    " tmp_o["+i+"]="+tmp_o[i]+
			    " pg_tl["+i+"]="+pg_tl[i]+
			    " tmp_uo["+i+"]="+tmp_uo[i]+
			    " currenttime="+currenttime);
		    }
		}
	    }
	}


	// trades disbalance
	calculateFlowVCut(shiftlevels,
			  currenttime,
			  pcurrent_buy,
			  pcurrent_sell,
			  PRINT_DATA_DEBUG);


	calculateFlowBuySellVCut(shiftlevels,
				 currenttime,
				 pcurrent_buy,
				 pcurrent_sell,
				 PRINT_DATA_DEBUG);

	calculateTauBuySellVCut(shiftlevels,
				currenttime,
				pcurrent_buy,
				pcurrent_sell,
				PRINT_DATA_DEBUG);

	// trades
	calculateTradesExeFlow(shiftlevels.length,
			       currenttime,
			       PRINT_DATA_DEBUG);


	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // expensive constrains
	    final int nt=shiftlevels.length;
	    final CalculateLiquidityDeficit
		ld_buy=new CalculateLiquidityDeficit(buy,
						     tmpordersbuf,
						     trades,
						     true,
						     tmptradesbuf),
		ld_sell=new CalculateLiquidityDeficit(sell,
						      tmpordersbuf,
						      trades,
						      false,
						      tmptradesbuf);
	    ld_buy.calculateLiquidityDeficit(this.price_orders_levels_buy,
					     currenttime,
					     pcurrent_buy,
					     pcurrent_sell,
					     trades_cleared_before);
	    ld_sell.calculateLiquidityDeficit(this.price_orders_levels_sell,
					      currenttime,
					      pcurrent_buy,
					      pcurrent_sell,
					      trades_cleared_before);
	    // check the results are matching
	    for(int i=0;i<nt;i++)
	    {
		// check the book
		if(this.vol_orders_buy[i]!=ld_buy.v_orders[i]||
		   this.tau_orders_buy[i]!=ld_buy.tau_orders[i]||
		   Double.isNaN(this.price_orders_levels_buy[i])!=Double.isNaN(ld_buy.price_cut[i])||
		   (!Double.isNaN(this.price_orders_levels_buy[i]) && this.price_orders_levels_buy[i]!=ld_buy.price_cut[i])||
		   this.vol_orders_sell[i]!=ld_sell.v_orders[i]||
		   this.tau_orders_sell[i]!=ld_sell.tau_orders[i]||
		   Double.isNaN(this.price_orders_levels_sell[i])!=Double.isNaN(ld_sell.price_cut[i])||
		   (!Double.isNaN(this.price_orders_levels_sell[i]) && this.price_orders_levels_sell[i]!=ld_sell.price_cut[i]))
		{
		    throw new RuntimeException(
			"Book mismatch:"+
			" vol_orders_buy["+i+"]="+this.vol_orders_buy[i]+
			" ld_buy.v_orders["+i+"]="+ld_buy.v_orders[i]+
			" tau_orders_buy["+i+"]="+this.tau_orders_buy[i]+
			" ld_buy.tau_orders["+i+"]="+ld_buy.tau_orders[i]+
			" price_orders_levels_buy["+i+"]="+this.price_orders_levels_buy[i]+
			" ld_buy.price_cut["+i+"]="+ld_buy.price_cut[i]+
			" vol_orders_sell["+i+"]="+this.vol_orders_sell[i]+
			" ld_sell.v_orders["+i+"]="+ld_sell.v_orders[i]+
			" tau_orders_sell["+i+"]="+this.tau_orders_sell[i]+
			" ld_sell.tau_orders["+i+"]="+ld_sell.tau_orders[i]+
			" price_orders_levels_sell["+i+"]="+this.price_orders_levels_sell[i]+
			" ld_sell.price_cut["+i+"]="+ld_sell.price_cut[i]+
			" currenttime="+currenttime);
		}

		// check the trades
		if(this.v_trades_flow_exe_buy[i]!=ld_buy.v_flow_exe[i]||
		   this.v_trades_exe_buy[i]!=ld_buy.v_exe_thistype[i]||
		   Double.isNaN(this.price_trades_tau_buy[i])!=Double.isNaN(ld_buy.price_tau[i])||
		   (!Double.isNaN(this.price_trades_tau_buy[i]) && this.price_trades_tau_buy[i]!=ld_buy.price_tau[i])||
		   Double.isNaN(this.price_trades_aver_tau_buy[i])!=Double.isNaN(ld_buy.price_aver_tau[i])||
		   (!Double.isNaN(this.price_trades_aver_tau_buy[i]) && this.price_trades_aver_tau_buy[i]!=ld_buy.price_aver_tau[i])||
		   this.v_trades_flow_exe_sell[i]!=ld_sell.v_flow_exe[i]||
		   this.v_trades_exe_sell[i]!=ld_sell.v_exe_thistype[i]||
		   Double.isNaN(this.price_trades_tau_sell[i])!=Double.isNaN(ld_sell.price_tau[i])||
		   (!Double.isNaN(this.price_trades_tau_sell[i]) && this.price_trades_tau_sell[i]!=ld_sell.price_tau[i])||
		   Double.isNaN(this.price_trades_aver_tau_sell[i])!=Double.isNaN(ld_sell.price_aver_tau[i])||
		   (!Double.isNaN(this.price_trades_aver_tau_sell[i]) && this.price_trades_aver_tau_sell[i]!=ld_sell.price_aver_tau[i]))
		{
		    throw new RuntimeException(
			"Trades flow mismatch:"+
			" v_trades_flow_exe_buy["+i+"]="+this.v_trades_flow_exe_buy[i]+
			" ld_buy.v_flow_exe["+i+"]="+ld_buy.v_flow_exe[i]+
			" v_trades_exe_buy["+i+"]="+this.v_trades_exe_buy[i]+
			" ld_buy.v_exe_thistype["+i+"]="+ld_buy.v_exe_thistype[i]+
			" price_trades_tau_buy["+i+"]="+this.price_trades_tau_buy[i]+
			" ld_buy.price_tau["+i+"]="+ld_buy.price_tau[i]+
			" price_trades_aver_tau_buy["+i+"]="+this.price_trades_aver_tau_buy[i]+
			" ld_buy.price_aver_tau["+i+"]="+ld_buy.price_aver_tau[i]+
			" v_trades_flow_exe_sell["+i+"]="+this.v_trades_flow_exe_sell[i]+
			" ld_sell.v_flow_exe["+i+"]="+ld_sell.v_flow_exe[i]+
			" v_trades_exe_sell["+i+"]="+this.v_trades_exe_sell[i]+
			" ld_sell.v_exe_thistype["+i+"]="+ld_sell.v_exe_thistype[i]+
			" price_trades_tau_sell["+i+"]="+this.price_trades_tau_sell[i]+
			" ld_sell.price_tau["+i+"]="+ld_sell.price_tau[i]+
			" price_trades_aver_tau_sell["+i+"]="+this.price_trades_aver_tau_sell[i]+
			" ld_sell.price_aver_tau["+i+"]="+ld_sell.price_aver_tau[i]+
			" currenttime="+currenttime);
		}
	    }
	}// END expensive constrains
    }



private void calculateBookVirtualShifts(final double [] shiftlevels,
					final long currenttime,
					final double pcurrent_buy,
					final double pcurrent_sell,
					final boolean PRINT_DATA_DEBUG)
{
    final int nt=shiftlevels.length;
    this.vol_orders_buy=new long [nt];
    this.tau_orders_buy=new long [nt];
    this.vol_orders_sell=new long [nt];
    this.tau_orders_sell=new long [nt];
    this.price_orders_levels_buy=new double[nt];
    this.price_orders_levels_sell=new double[nt];
    this.price_orders_aver_buy=new double[nt];
    this.price_orders_aver_sell=new double[nt];
    this.price_orders_taver_buy=new double[nt];
    this.price_orders_taver_sell=new double[nt];
    this.price_orders_sqtaver_buy=new double[nt];
    this.price_orders_sqtaver_sell=new double[nt];
    this.w_orders_sqtaver_buy=new double[nt];
    this.w_orders_sqtaver_sell=new double[nt];
    this.ptr_book_buy=new int [nt];
    this.ptr_book_sell=new int [nt];

    final MatchOrdersVariousShifts.MatchWithShift [] lmref=this.orders_mref=
	new MatchOrdersVariousShifts.MatchWithShift [nt];
    final double ds=this.dspread=pcurrent_sell-pcurrent_buy;

    // perform all possible virtual shifts
    // and save the results in this.allshifts
    this.virtualshifts.copyOrderedOrdersToFlow(this.buy,this.obuy);
    this.virtualshifts.copyOrderedOrdersToFlow(this.sell,this.osell);
    this.virtualshifts.matchWithVariousShifts(this.obuy,this.osell,this.allshifts);
    final int msize=this.allshifts.getSize();
    final MatchOrdersVariousShifts.MatchWithShift [] ob=this.allshifts.getBuffer();


    if(PRINT_DATA_DEBUG)
    {
	System.err.println("num\t"+
			   "shift0\t"+
			   "delta0\t"+
			   "shift\t"+
			   "delta\t"+
			   "eff_shift\t"+
			   // extra debug info
			   "type\t"+
			   "pmatch nodp\t"+
			   "dp\t"+
			   "vmatch\t"+
			   "lev\t"+
			   "cur_pos");
    }


    double lev=Double.NaN,prev_shift=-ds;
    long t_prev_vmatch=0;
    gloop:for(int i=0,cur_pos=-1;i<msize;i++)
    {
	final MatchOrdersVariousShifts.MatchWithShift t=ob[i];
	if(t.shift<prev_shift || t.vmatch<t_prev_vmatch)
	{
	    throw new RuntimeException("Ordering is broken"+
				       " t.shift="+t.shift+
				       " prev_shift="+prev_shift+
				       " t.vmatch="+t.vmatch+
				       " t_prev_vmatch="+t_prev_vmatch+
				       " currenttime="+currenttime);
	}
	prev_shift=t.shift;
	t_prev_vmatch=t.vmatch;

	final double current_shift=t.shift+ds;
	for(;!(current_shift<=lev);)
	{
	    if(++cur_pos>=nt)
	    {
		break gloop;
	    }
	    if(lev>shiftlevels[cur_pos])
	    {
		throw new RuntimeException("Not sorted shiftlevels."+
					   " currenttime="+currenttime);
	    }
	    lev=shiftlevels[cur_pos];
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println(
		String.valueOf(i)+"\t"+
		t.shift0+"\t"+
		t.delta0+"\t"+
		t.shift+"\t"+
		t.delta+"\t"+
		current_shift+"\t"+
		// extra debug info
		t.type+"\t"+
		t.test_pmatch+"\t"+
		t.test_dp+"\t"+
		t.vmatch+"\t"+
		lev+"\t"+
		cur_pos
			       );
	}
	lmref[cur_pos]=t;
    }


    final int n_buy=this.buy.getSize(),n_sell=this.sell.getSize();
    final BookItem [] buf_buy=this.buy.getBuffer(),buf_sell=this.sell.getBuffer();
    for(int ptr_book_buy=0,ptr_book_sell=0,cur_pos=0;cur_pos<nt;cur_pos++)
    {
	final MatchOrdersVariousShifts.MatchWithShift t,tcur=lmref[cur_pos];
	if(tcur!=null)
	{
	    t=tcur;
	}
	else if(cur_pos>0)
	{
	    t=lmref[cur_pos]=lmref[cur_pos-1];
	}
	else
	{
	    t=tcur;
	}
	ptr_book_buy=scanBookPortion(cur_pos,
				     this.vol_orders_buy,
				     this.tau_orders_buy,
				     this.price_orders_levels_buy,
				     this.price_orders_aver_buy,
				     this.price_orders_taver_buy,
				     this.price_orders_sqtaver_buy,
				     this.w_orders_sqtaver_buy,
				     buf_buy,
				     n_buy,
				     t!=null?t.vbuy0:0,
				     ptr_book_buy,
				     currenttime);
	ptr_book_sell=scanBookPortion(cur_pos,
				      this.vol_orders_sell,
				      this.tau_orders_sell,
				      this.price_orders_levels_sell,
				      this.price_orders_aver_sell,
				      this.price_orders_taver_sell,
				      this.price_orders_sqtaver_sell,
				      this.w_orders_sqtaver_sell,
				      buf_sell,
				      n_sell,
				      t!=null?t.vsell0:0,
				      ptr_book_sell,
				      currenttime);
	this.ptr_book_buy[cur_pos]=ptr_book_buy-1;
	this.ptr_book_sell[cur_pos]=ptr_book_sell-1;
    }
    for(int i=0;i<nt;i++)
    {
	// normalization
	if(this.vol_orders_buy[i]>0)
	{
	    this.price_orders_aver_buy[i]/=this.vol_orders_buy[i];
	    this.price_orders_taver_buy[i]/=this.tau_orders_buy[i];
	    this.tau_orders_buy[i]=(this.tau_orders_buy[i]/this.vol_orders_buy[i])
		*ModelParameters.T_SUMM_SCALE;
	    this.price_orders_sqtaver_buy[i]/=this.w_orders_sqtaver_buy[i];
	}
	else
	{
	    this.tau_orders_buy[i]=-1;
	    this.price_orders_aver_buy[i]=
	    this.price_orders_taver_buy[i]=
	    this.price_orders_sqtaver_buy[i]=Double.NaN;
	}
	if(this.vol_orders_sell[i]>0)
	{
	    this.price_orders_aver_sell[i]/=this.vol_orders_sell[i];
	    this.price_orders_taver_sell[i]/=this.tau_orders_sell[i];
	    this.tau_orders_sell[i]=(this.tau_orders_sell[i]/this.vol_orders_sell[i])
		*ModelParameters.T_SUMM_SCALE;
	    this.price_orders_sqtaver_sell[i]/=this.w_orders_sqtaver_sell[i];
	}
	else
	{
	    this.tau_orders_sell[i]=-1;
	    this.price_orders_aver_sell[i]=
	    this.price_orders_taver_sell[i]=
	    this.price_orders_sqtaver_sell[i]=Double.NaN;
	}


	// constrains check
	final MatchOrdersVariousShifts.MatchWithShift t=lmref[i];
	if(this.vol_orders_buy[i]!=(t!=null?t.vbuy0:0)||
	   this.vol_orders_sell[i]!=(t!=null?t.vsell0:0)||
	   (i>0 && t==null && lmref[i-1]!=null))
	{
	    throw new RuntimeException("Volume constrains violation:"+
				       " vol_orders_buy["+i+"]="+this.vol_orders_buy[i]+
				       " t.vbuy0="+(t!=null?t.vbuy0:0)+
				       " vol_orders_sell["+i+"]="+this.vol_orders_sell[i]+
				       " t.vsell0="+(t!=null?t.vsell0:0)+
				       " t="+t+
				       " orders_mref[i-1]="+(i>0?lmref[i-1]:null)+
				       " currenttime="+currenttime);
	}
    }

    if(PRINT_DATA_DEBUG)
    {
	System.err.println("rnum\tratio\tshift_lev\teff_shift\tshift\tdelta\tshift0\tdelta0\t\t"+
			   "vol_orders_buy\ttau_orders_buy\tprice_orders_levels_buy\tprice_orders_aver_buy\tprice_orders_taver_buy\tprice_orders_sqtaver_buy\t\t"+
			   "vol_orders_sell\ttau_orders_sell\tprice_orders_levels_sell\tprice_orders_aver_sell\tprice_orders_taver_sell\tprice_orders_sqtaver_sell");
	for(int i=0;i<nt;i++)
	{
	    final MatchOrdersVariousShifts.MatchWithShift t=lmref[i];
	    System.err.println(String.valueOf(i)+"\t"+
			       (shiftlevels[i]/(0.5*(pcurrent_buy+pcurrent_sell)))+"\t"+
			       shiftlevels[i]+"\t"+
			       (t!=null?
				((t.shift+ds)+"\t"+t.shift+"\t"+t.delta+"\t"+t.shift0+"\t"+t.delta0):
				("\t\t\t\t"))+"\t\t"+
			       this.vol_orders_buy[i]+"\t"+
			       this.tau_orders_buy[i]+"\t"+
			       this.price_orders_levels_buy[i]+"\t"+
			       this.price_orders_aver_buy[i]+"\t"+
			       this.price_orders_taver_buy[i]+"\t"+
			       this.price_orders_sqtaver_buy[i]+"\t\t"+
			       this.vol_orders_sell[i]+"\t"+
			       this.tau_orders_sell[i]+"\t"+
			       this.price_orders_levels_sell[i]+"\t"+
			       this.price_orders_aver_sell[i]+"\t"+
			       this.price_orders_taver_sell[i]+"\t"+
			       this.price_orders_sqtaver_sell[i]);
	}
    }


    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
    {
	for(int i=0;i<nt;i++)
	{
	    checkBookAverages(currenttime,
			      this.vol_orders_buy[i],
			      buf_buy,
			      this.price_orders_levels_buy[i],
			      this.price_orders_aver_buy[i],
			      this.price_orders_taver_buy[i],
			      this.price_orders_sqtaver_buy[i],
			      this.w_orders_sqtaver_buy[i],
			      this.tau_orders_buy[i]);

	    checkBookAverages(currenttime,
			      this.vol_orders_sell[i],
			      buf_sell,
			      this.price_orders_levels_sell[i],
			      this.price_orders_aver_sell[i],
			      this.price_orders_taver_sell[i],
			      this.price_orders_sqtaver_sell[i],
			      this.w_orders_sqtaver_sell[i],
			      this.tau_orders_sell[i]);
	}
    }
}



    private int scanBookPortion(final int cur_pos,
				final long [] vol,
				final long [] svol,
				final double [] price_cut,
				final double [] paver_cut,
				final double [] pavert_cut,
				final double [] paversqt_cut,
				final double [] wsqt_cut,
				final BookItem [] buf,
				final int n_buf,
				final long vtarget,
				final int ptr_init,
				final long currenttime)
    {

	long v,sv;
	double p,paver,tpaver,sqtaver,wsqt;
	if(cur_pos>0)
	{
	    final int i1=cur_pos-1;
	    v=vol[i1];
	    sv=svol[i1];
	    p=price_cut[i1];
	    paver=paver_cut[i1];
	    tpaver=pavert_cut[i1];
	    sqtaver=paversqt_cut[i1];
	    wsqt=wsqt_cut[i1];
	}
	else
	{
	    v=sv=0;
	    p=Double.NaN;
	    paver=tpaver=sqtaver=wsqt=0;
	}
	int i=ptr_init;
	for(;i<n_buf && v<vtarget;i++)
	{
	    final BookItem t=buf[i];
	    if((v+=t.shares)<0)
	    {
		throw new RuntimeException("Overflow."+
					   " currenttime="+currenttime);
	    }
	    p=t.price;
	    paver+=p*t.shares;

	    final long tau=(currenttime-t.time)/ModelParameters.T_SUMM_SCALE;
	    final long wtau=t.shares*tau;
	    if((sv+=wtau)<0)
	    {
		throw new RuntimeException("Overflow."+
					   " currenttime="+currenttime);
	    }
	    tpaver+=p*wtau;

	    final double wst=t.shares*Math.sqrt(tau);
	    sqtaver+=p*wst;
	    wsqt+=wst;
	}
	vol[cur_pos]=v;
	svol[cur_pos]=sv;
	price_cut[cur_pos]=p;
	paver_cut[cur_pos]=paver;
	pavert_cut[cur_pos]=tpaver;
	paversqt_cut[cur_pos]=sqtaver;
	wsqt_cut[cur_pos]=wsqt;
	return i;
    }

    private void calculateKendallRanks(final int cur_pos,
				       final int [] ptr_book,
				       final BookItem [] buf,
				       final int n_buf,
				       final long currenttime,
				       final boolean PRINT_DATA_DEBUG,
				       final long [] k_ord,
				       final long [] k_unord,
				       final long [] v_cut)
    {
	// FIXME
    }

    private void checkCalculateKendallRanks(final int [] ptr_book,
					    final BookItem [] buf,
					    final long currenttime,
					    final boolean PRINT_DATA_DEBUG,
					    final long [] k_ord,
					    final long [] k_unord)
    {
	final int nt=ptr_book.length;
	for(int i=0;i<nt;i++)
	{
	    final int pe=ptr_book[i];
	    long vv_ord=0,vv_unord=0;
	    int js=0,ns=0,nsc=0;
	    for(int j=0;j<=pe && js<=pe;j++)
	    {
		final BookItem tj=buf[j];
		final long sharestj=tj.shares,timetj=tj.time;
		for(int pjs=js;js<=pe;)
		{
		    if(tj.price==buf[js].price)
		    {
			js++;
		    }
		    else
		    {
			nsc+=(js-pjs)*(1+pe-js);
			break;
		    }
		}
		for(int k=js;k<=pe;k++)
		{
		    final BookItem tk=buf[k];
		    final long n=tk.shares*sharestj;
		    if(tk.time>timetj)
		    {
			vv_ord+=n;
		    }
		    else
		    {
			vv_unord+=n;
		    }
		    ns++;
		}
	    }
	    if(ns!=nsc)
	    {
		throw new RuntimeException("Terms number mismatch:"+
					   " ns="+ns+
					   " nsc="+nsc+
					   " currenttime="+currenttime);
	    }
	    k_ord[i]=vv_ord;
	    k_unord[i]=vv_unord;
	}
    }


  private void check2CalculateKendallRanks(final int [] ptr_book,
					   final BookItem [] buf,
					   final long currenttime,
					   final boolean PRINT_DATA_DEBUG,
					   final boolean time_eq_ok,
					   final long [] v,
					   final long [] pl_tl,
					   final long [] pg_tl,
					   final long [] pl_tg,
					   final long [] pg_tg)

    {
	final int nt=ptr_book.length;
	for(int i=0;i<nt;i++)
	{
	    final int pe=ptr_book[i];
	    long s_ll=0,s_gl=0,s_lg=0,s_gg=0,vv=0;
	    for(int j=0;j<=pe;j++)
	    {
		final BookItem tj=buf[j];
		final long sharestj=tj.shares,timetj=tj.time;
		final double pricetj=tj.price;
		vv+=sharestj;
		for(int k=0;k<=pe;k++)
		{
		    final BookItem tk=buf[k];
		    final long n=tk.shares*sharestj;
		    if(tk.price<pricetj)
		    {
			if(time_eq_ok?tk.time>=timetj:tk.time>timetj)
			{
			    s_lg+=n;
			}
			else
			{
			    s_ll+=n;
			}
		    }
		    else if(tk.price>pricetj)
		    {
			if(time_eq_ok?tk.time<=timetj:tk.time<timetj)
			{
			    s_gl+=n;
			}
			else
			{
			    s_gg+=n;
			}
		    }
		}
	    }
	    v[i]=vv;
	    pl_tl[i]=s_ll;
	    pg_tl[i]=s_gl;
	    pl_tg[i]=s_lg;
	    pg_tg[i]=s_gg;
	    if(s_ll!=s_gg||s_lg!=s_gl)
	    {
		throw new RuntimeException("Constrains mismatch:"+
					   " cur_pos="+i+
					   " s_ll="+s_ll+
					   " s_gg="+s_gg+
					   " s_lg="+s_lg+
					   " s_gl="+s_gl+
					   " currenttime="+currenttime);
	    }
	}
    }


private void calculateTradesExeFlow(final int nt,
				    final long currenttime,
				    final boolean PRINT_DATA_DEBUG)
{

    // init arrays
    this.v_trades_flow_exe_buy=new long [nt];
    this.v_trades_flow_exe_sell=new long [nt];
    this.v_trades_exe_buy=new long [nt];
    this.v_trades_exe_sell=new long [nt];

    this.price_trades_tau_buy=new double [nt];
    this.price_trades_tau_sell=new double [nt];
    this.price_trades_aver_tau_buy=new double [nt];
    this.price_trades_aver_tau_sell=new double [nt];


    calculateFlow(nt,
		  currenttime,
		  PRINT_DATA_DEBUG,
		  BookItem.BOOK_BUY,
		  BookItem.BOOK_SELL,
		  this.tau_orders_buy,
		  this.v_trades_flow_exe_buy,
		  this.v_trades_exe_buy,
		  this.price_trades_tau_buy,
		  this.price_trades_aver_tau_buy);

    calculateFlow(nt,
		  currenttime,
		  PRINT_DATA_DEBUG,
		  BookItem.BOOK_SELL,
		  BookItem.BOOK_BUY,
		  this.tau_orders_sell,
		  this.v_trades_flow_exe_sell,
		  this.v_trades_exe_sell,
		  this.price_trades_tau_sell,
		  this.price_trades_aver_tau_sell);
}

void calculateFlow(final int nt,
		   final long currenttime,
		   final boolean PRINT_DATA_DEBUG,
		   final int type_limit_exe,
		   final int type_market_exe,
		   final long [] tau_orders,
		   // output parameters
		   final long [] v_flow_exe,
		   final long [] v_exe,
		   final double [] price_tau,
		   final double [] price_aver_tau)
{
    final int tsize=this.trades.getSize();
    final Trade [] tb=this.trades.getBuffer();
    for(int cur_pos=0;cur_pos<nt;cur_pos++)
    { // a lop over all delays
	long tbefore=currenttime-tau_orders[cur_pos],prevt=Long.MAX_VALUE;
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("CUR_POS\t"+cur_pos+"\t"+
			       "tau\t"+tau_orders[cur_pos]+
			       "tbefore\t"+tbefore);
	    System.err.println("num\t"+
			       "time\t"+
			       "origtime\t"+
			       "shares\t"+
			       "tradetype\t"+
			       "buy_sell_indicator\t"+
			       "price\t"+
			       "matchnumber\t"+
			       "orderreferencenumber");
	}
	long vol_flow=0,volexe=0;
	double p_last_tau=Double.NaN,paver_tau=0;
        gloop:for(int i=tsize;i-- >0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println(String.valueOf(i)+"\t"+
				   t.time+"\t"+
				   t.origtime+"\t"+
				   t.shares+"\t"+
				   t.tradetype+"\t"+
				   t.buy_sell_indicator+"\t"+
				   t.price+"\t"+
				   t.matchnumber+"\t"+
				   t.orderreferencenumber+
				   (t.order!=null?
				    "\t\t"+(t.time-t.order.time)+"\t"+
				    t.order.time+"\t"+
				    t.order.shares+"\t"+
				    t.order.price:
				    "\t\t-10\t\t\t"));
	    }
	    if(!(t.time>=tbefore))
	    {
		break gloop;
	    }

	    if(t.origtime>0)
	    {
		if(t.buy_sell_indicator==type_market_exe||
		   (t.buy_sell_indicator==type_limit_exe && t.origtime>=tbefore))
		{
		    vol_flow+=t.shares;
		}
		if(t.buy_sell_indicator==type_limit_exe)
		{
		    volexe+=t.shares;
		    p_last_tau=t.price;
		    paver_tau+=t.price*t.shares;
		}
	    }
	}

	v_flow_exe[cur_pos]=vol_flow;
	v_exe[cur_pos]=volexe;
	price_tau[cur_pos]=p_last_tau;
	price_aver_tau[cur_pos]=volexe>0?paver_tau/volexe:Double.NaN;
    } // END a lop over all delays
}





private void calculateFlowVCut(final double [] shiftlevels,
			       final long currenttime,
			       final double pcurrent_buy,
			       final double pcurrent_sell,
			       final boolean PRINT_DATA_DEBUG)
{

    final MatchOrdersVariousShifts.MatchWithShift [] lmref=this.orders_mref;
    final int nt=lmref.length;
    this.v_cut_trades_flow_exe=new long [nt];
    java.util.Arrays.fill(this.v_cut_trades_flow_exe,-1);
    this.tau_trades_buy=new long [nt];
    this.tau_trades_sell=new long [nt];

    if(PRINT_DATA_DEBUG)
    {
	System.err.println("!calculateFlowVCut\t"+
			   "levels\t"+nt+"\t"+
			   "trades\t"+this.trades.getSize());
	System.err.println("num\t"+
			   "time\t"+
			   "origtime\t"+
			   "shares\t"+
			   "tradetype\t"+
			   "buy_sell_indicator\t"+
			   "price\t"+
			   "matchnumber\t"+
			   "orderreferencenumber\t"+
			   "cur_pos\t"+
			   "v\t"+
			   "vcut");
    }

    final int tsize=this.trades.getSize();
    final Trade [] tb=this.trades.getBuffer();
    long vcut=-1,prevt=Long.MAX_VALUE,svbuy=0,svsell=0,v=0;
    gloop:for(int i=tsize,cur_pos=-1;i-- >0;)
    {
	final Trade t=tb[i];
	if(t.time>prevt)
	{
	    throw new RuntimeException("The data is not sorted:"+
				       prevt+" and "+t.time+
				       " currenttime="+currenttime);
	}
	prevt=t.time;
	if(t.origtime>0)
	{
	    v+=t.shares;
	    final long
		stau_orig=t.shares*((currenttime-getorigtimeadj(t.origtime,t.time))/ModelParameters.T_SUMM_SCALE),
		stau_exe=t.shares*((currenttime-t.time)/ModelParameters.T_SUMM_SCALE);
	    if(t.buy_sell_indicator==BookItem.BOOK_BUY)
	    {
		svsell+=stau_exe; // buy trade i.e. market sell order just came
		svbuy+=stau_orig; // buy trade i.e. limit buy order was posted some time ago
	    }
	    else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
	    {
		svbuy+=stau_exe; // sell trade i.e. market buy order just came
		svsell+=stau_orig; // sell trade i.e. limit sell order was posted some time ago
	    }
	    else
	    {
		throw new IllegalStateException();
	    }
	    if(svbuy<0||svsell<0||v<0)
	    {
		throw new RuntimeException("Overflow");
	    }
	    for(;!(v<=vcut);)
	    {
		if(++cur_pos>=nt)
		{
		    break gloop;
		}
		final long vcut_new=lmref[cur_pos]!=null?(FACTOR_LIQUIDITY_AMPLIFICATION*lmref[cur_pos].vmatch):-1;
		if(vcut>vcut_new)
		{
		    throw new RuntimeException("Not sorted vcut."+
					       " currenttime="+currenttime);
		}
		vcut=vcut_new;
	    }
	    // underestimated
	    this.v_cut_trades_flow_exe[cur_pos]=v;
	    this.tau_trades_buy[cur_pos]=svbuy;
	    this.tau_trades_sell[cur_pos]=svsell;
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println(String.valueOf(i)+"\t"+
			       t.time+"\t"+
			       t.origtime+"\t"+
			       t.shares+"\t"+
			       t.tradetype+"\t"+
			       t.buy_sell_indicator+"\t"+
			       t.price+"\t"+
			       t.matchnumber+"\t"+
			       t.orderreferencenumber+"\t"+
			       cur_pos+"\t"+
			       v+"\t"+
			       vcut+"\t"+
			       (t.order!=null?
				"\t\t"+(t.time-t.order.time)+"\t"+
				t.order.time+"\t"+
				t.order.shares+"\t"+
				t.order.price:
				"\t\t-10\t\t\t"));
	}
    }
    for(int i=0;i<nt;i++)
    { // use previous if current not available
	if(this.v_cut_trades_flow_exe[i]<0)
	{
	    if(i>0)
	    {
		final int i1=i-1;
		this.v_cut_trades_flow_exe[i]=this.v_cut_trades_flow_exe[i1];
		this.tau_trades_buy[i]=this.tau_trades_buy[i1];
		this.tau_trades_sell[i]=this.tau_trades_sell[i1];
	    }
	    else
	    {
		this.v_cut_trades_flow_exe[i]=0;
	    }
	}
	if(i>0)
	{
	    final int i1=i-1;
	    if(this.v_cut_trades_flow_exe[i]<0 || this.v_cut_trades_flow_exe[i1]<0 ||
	       this.v_cut_trades_flow_exe[i]<this.v_cut_trades_flow_exe[i1]||
	       this.tau_trades_buy[i]<this.tau_trades_buy[i1]||
	       this.tau_trades_sell[i]<this.tau_trades_sell[i1])
	    {
		throw new RuntimeException(
		    "Volume size unordered:"+
		    " v_cut_trades_flow_exe["+i+"]="+this.v_cut_trades_flow_exe[i]+
		    " v_cut_trades_flow_exe["+i1+"]="+this.v_cut_trades_flow_exe[i1]+
		    " tau_trades_buy["+i+"]="+this.tau_trades_buy[i1]+
		    " tau_trades_buy["+i1+"]="+this.tau_trades_buy[i1]+
		    " tau_trades_sell["+i+"]="+this.tau_trades_sell[i]+
		    " tau_trades_sell["+i1+"]="+this.tau_trades_sell[i1]);
	    }
	}
    } // END use previous if current not available

    for(int i=0;i<nt;i++)
    { // normalization
	final long n=this.v_cut_trades_flow_exe[i];
	if(n>0)
	{
	    this.tau_trades_buy[i]=(this.tau_trades_buy[i]/n)
		*ModelParameters.T_SUMM_SCALE;
	    this.tau_trades_sell[i]=(this.tau_trades_sell[i]/n)
		*ModelParameters.T_SUMM_SCALE;
	}
	else
	{
	    this.tau_trades_buy[i]=this.tau_trades_sell[i]=-1;
	}
    } // END normalization

    if(PRINT_DATA_DEBUG)
    {
	final double ds=this.dspread;
	System.err.println("rnum\tratio\tshift_lev\teff_shift\tshift\tdelta\tshift0\tdelta0\t\t"+
			   "vmatch\tvexe\ttau_buy\ttau_sell");
	for(int i=0;i<nt;i++)
	{
	    final MatchOrdersVariousShifts.MatchWithShift t=lmref[i];
	    System.err.println(String.valueOf(i)+"\t"+
			       (shiftlevels[i]/(0.5*(pcurrent_buy+pcurrent_sell)))+"\t"+
			       shiftlevels[i]+"\t"+
			       (t!=null?
				((t.shift+ds)+"\t"+t.shift+"\t"+t.delta+"\t"+t.shift0+"\t"+t.delta0):
				("\t\t\t\t"))+"\t\t"+
			       (t!=null?String.valueOf(t.vmatch):"??")+"\t"+
			       this.v_cut_trades_flow_exe[i]+"\t"+
			       this.tau_trades_buy[i]+"\t"+
			       this.tau_trades_sell[i]);
	}
    }
}


private void calculateTauBuySellVCut(final double [] shiftlevels,
				     final long currenttime,
				     final double pcurrent_buy,
				     final double pcurrent_sell,
				     final boolean PRINT_DATA_DEBUG)
{
    final MatchOrdersVariousShifts.MatchWithShift [] lmref=this.orders_mref;
    final int nt=lmref.length;
    this.tau_trades_plev_buy=new long [nt];
    java.util.Arrays.fill(this.tau_trades_plev_buy,Long.MIN_VALUE);
    this.tau_trades_plev_sell=new long [nt];
    java.util.Arrays.fill(this.tau_trades_plev_sell,Long.MIN_VALUE);

    final int tsize=this.trades.getSize();
    final Trade [] tb=this.trades.getBuffer();
    final double plev_buy=pcurrent_buy-OrdersFlow.EPS,plev_sell=pcurrent_sell+OrdersFlow.EPS;
    long vcut_buy=-1,vcut_sell=-1,v_buy=0,v_sell=0;
    boolean scan_buy=true,scan_sell=true;
    gloop:for(int i=tsize,cur_pos_buy=-1,cur_pos_sell=-1;i-- >0;)
    {
	final Trade t=tb[i];
	if(t.origtime>0)
	{
	    if(t.buy_sell_indicator==BookItem.BOOK_BUY)
	    {
	        s:if(scan_buy)
		{
		    if(t.price<plev_buy)
		    {
			v_buy+=t.shares;
			for(;!(v_buy<=vcut_buy);)
			{
			    if(++cur_pos_buy>=nt)
			    {
				scan_buy=false;
				break s;
			    }
			    final long vnew=lmref[cur_pos_buy]!=null?(FACTOR_LIQUIDITY_AMPLIFICATION*lmref[cur_pos_buy].vbuy0):-1;
			    if(vcut_buy>vnew)
			    {
				throw new RuntimeException("Not sorted vcut."+
							   " currenttime="+currenttime);
			    }
			    vcut_buy=vnew;
			}
			// underestimated
			this.tau_trades_plev_buy[cur_pos_buy]=currenttime-t.time;
		    }
		}
	    }
	    else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
	    {
	        s:if(scan_sell)
		{
		    if(t.price>plev_sell)
		    {
			v_sell+=t.shares;
			for(;!(v_sell<=vcut_sell);)
			{
			    if(++cur_pos_sell>=nt)
			    {
				scan_sell=false;
				break s;
			    }
			    final long vnew=lmref[cur_pos_sell]!=null?(FACTOR_LIQUIDITY_AMPLIFICATION*lmref[cur_pos_sell].vsell0):-1;
			    if(vcut_sell>vnew)
			    {
				throw new RuntimeException("Not sorted vcut."+
							   " currenttime="+currenttime);
			    }
			    vcut_sell=vnew;
			}
			// underestimated
			this.tau_trades_plev_sell[cur_pos_sell]=currenttime-t.time;
		    }
		}
	    }
	    else
	    {
		throw new IllegalStateException();
	    }
	    if(!scan_buy && !scan_sell)
	    {
		break gloop;
	    }
	}
    }
    for(int i=0;i<nt;i++)
    { // use previous if current not available
	if(this.tau_trades_plev_buy[i]<0)
	{
	    if(i>0)
	    {
		final int i1=i-1;
		this.tau_trades_plev_buy[i]=this.tau_trades_plev_buy[i1];
	    }
	    else
	    {
		this.tau_trades_plev_buy[i]=-1;
	    }
	}
	if(this.tau_trades_plev_sell[i]<0)
	{
	    if(i>0)
	    {
		final int i1=i-1;
		this.tau_trades_plev_sell[i]=this.tau_trades_plev_sell[i1];
	    }
	    else
	    {
		this.tau_trades_plev_sell[i]=-1;
	    }
	}
    } // END use previous if current not available


    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
    {
	for(int i=0;i<nt;i++)
	{
	    final MatchOrdersVariousShifts.MatchWithShift t=lmref[i];
	    checkTausCut(currenttime,
			 i,
			 BookItem.BOOK_BUY,
			 (t!=null?FACTOR_LIQUIDITY_AMPLIFICATION*t.vbuy0:0),
			 plev_buy,
			 this.tau_trades_plev_buy[i]);
	    checkTausCut(currenttime,
			 i,
			 BookItem.BOOK_SELL,
			 (t!=null?FACTOR_LIQUIDITY_AMPLIFICATION*t.vsell0:0),
			 plev_sell,
			 this.tau_trades_plev_sell[i]);
	}
    }
}

private void checkTausCut(final long currenttime,
			  final int cur_pos,
			  final int type,
			  final long vcut,
			  final double plev,
			  final long tau_cut_check)
{
    final int tsize=this.trades.getSize();
    final Trade [] tb=this.trades.getBuffer();

    //System.err.println("Call, cur_pos="+cur_pos+" type="+type+" plev="+plev+" vcut="+vcut);
    long v=0,prevt=Long.MAX_VALUE,taux=-1;
    gloop:for(int i=tsize;i-- >0;)
    {
	final Trade t=tb[i];
	if(t.time>prevt)
	{
	    throw new RuntimeException("The data is not sorted:"+
				       prevt+" and "+t.time+
				       " currenttime="+currenttime);
	}
	prevt=t.time;
	if(t.origtime>0 && t.buy_sell_indicator==type &&
	   (type==BookItem.BOOK_BUY?(t.price<plev):(t.price>plev)))
	{
	    v+=t.shares;
	    if(v>vcut)
	    {
		v-=t.shares;
		break gloop;
	    }
	    taux=currenttime-t.time;
	    //System.err.println("Xi="+i+" T="+t.shares+" v="+v+" cur_pos="+cur_pos+" tau="+taux);
	}
    }
    if(taux!=tau_cut_check)
    {
	throw new RuntimeException(
	    "Mismatch:"+
	    " tau_cut_check="+tau_cut_check+
	    " taux="+taux+
	    " cur_pos="+cur_pos+
	    " type="+type+
	    " v="+v+
	    " currenttime="+currenttime);
    }
}




private void calculateFlowBuySellVCut(final double [] shiftlevels,
				      final long currenttime,
				      final double pcurrent_buy,
				      final double pcurrent_sell,
				      final boolean PRINT_DATA_DEBUG)
{

    final MatchOrdersVariousShifts.MatchWithShift [] lmref=this.orders_mref;
    final int nt=lmref.length;
    this.v_trades_buy_exe=new long [nt];
    java.util.Arrays.fill(this.v_trades_buy_exe,-1);
    this.tau_trades_limit_buy=new long [nt];
    this.tau_trades_market_sell=new long [nt];
    this.tau_trades_ending_buy=new long [nt];

    this.v_trades_sell_exe=new long [nt];
    java.util.Arrays.fill(this.v_trades_sell_exe,-1);
    this.tau_trades_limit_sell=new long [nt];
    this.tau_trades_market_buy=new long [nt];
    this.tau_trades_ending_sell=new long [nt];

    final int tsize=this.trades.getSize();
    final Trade [] tb=this.trades.getBuffer();
    long vcut_buy=-1,vcut_sell=-1,prevt=Long.MAX_VALUE,
	svbuy_market=0,
	svbuy_limit=0,
	svsell_market=0,
	svsell_limit=0,
	v_buy=0,
	v_sell=0;
    boolean scan_buy=true,scan_sell=true;
    gloop:for(int i=tsize,cur_pos_buy=-1,cur_pos_sell=-1;i-- >0;)
    {
	final Trade t=tb[i];
	if(t.time>prevt)
	{
	    throw new RuntimeException("The data is not sorted:"+
				       prevt+" and "+t.time+
				       " currenttime="+currenttime);
	}
	prevt=t.time;
	if(t.origtime>0)
	{
	    final long
		tau=currenttime-t.time,
		stau_orig=t.shares*((currenttime-getorigtimeadj(t.origtime,t.time))/ModelParameters.T_SUMM_SCALE),
		stau_exe=t.shares*(tau/ModelParameters.T_SUMM_SCALE);
	    if(t.buy_sell_indicator==BookItem.BOOK_BUY)
	    {
	        s:if(scan_buy)
		{
		    v_buy+=t.shares;
		    svsell_market+=stau_exe; // buy trade i.e. market sell order just came
		    svbuy_limit+=stau_orig; // buy trade i.e. limit buy order was posted some time ago
		    if(v_buy<0||svsell_market<0||svbuy_limit<0)
		    {
			throw new RuntimeException("Overflow");
		    }
		    for(;!(v_buy<=vcut_buy);)
		    {
			if(++cur_pos_buy>=nt)
			{
			    scan_buy=false;
			    break s;
			}
			final long vnew=lmref[cur_pos_buy]!=null?(FACTOR_LIQUIDITY_AMPLIFICATION*lmref[cur_pos_buy].vbuy0):-1;
			if(vcut_buy>vnew)
			{
			    throw new RuntimeException("Not sorted vcut."+
						       " currenttime="+currenttime);
			}
			vcut_buy=vnew;
		    }
		    // underestimated
		    this.v_trades_buy_exe[cur_pos_buy]=v_buy;
		    this.tau_trades_limit_buy[cur_pos_buy]=svbuy_limit;
		    this.tau_trades_market_sell[cur_pos_buy]=svsell_market;
		    this.tau_trades_ending_buy[cur_pos_buy]=tau;
		}
	    }
	    else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
	    {
	        s:if(scan_sell)
		{
		    v_sell+=t.shares;
		    svbuy_market+=stau_exe; // sell trade i.e. market buy order just came
		    svsell_limit+=stau_orig; // sell trade i.e. limit sell order was posted some time ago
		    if(v_sell<0||svbuy_market<0||svsell_limit<0)
		    {
			throw new RuntimeException("Overflow");
		    }
		    for(;!(v_sell<=vcut_sell);)
		    {
			if(++cur_pos_sell>=nt)
			{
			    scan_sell=false;
			    break s;
			}
			final long vnew=lmref[cur_pos_sell]!=null?(FACTOR_LIQUIDITY_AMPLIFICATION*lmref[cur_pos_sell].vsell0):-1;
			if(vcut_sell>vnew)
			{
			    throw new RuntimeException("Not sorted vcut."+
						       " currenttime="+currenttime);
			}
			vcut_sell=vnew;
		    }
		    // underestimated
		    this.v_trades_sell_exe[cur_pos_sell]=v_sell;
		    this.tau_trades_limit_sell[cur_pos_sell]=svsell_limit;
		    this.tau_trades_market_buy[cur_pos_sell]=svbuy_market;
		    this.tau_trades_ending_sell[cur_pos_sell]=tau;
		}
	    }
	    else
	    {
		throw new IllegalStateException();
	    }
	    if(!scan_buy && !scan_sell)
	    {
		break gloop;
	    }
	}
    }

    for(int i=0;i<nt;i++)
    { // use previous if current not available
	if(this.v_trades_buy_exe[i]<0)
	{
	    if(i>0)
	    {
		final int i1=i-1;
		this.v_trades_buy_exe[i]=this.v_trades_buy_exe[i1];
		this.tau_trades_limit_buy[i]=this.tau_trades_limit_buy[i1];
		this.tau_trades_market_sell[i]=this.tau_trades_market_sell[i1];
		this.tau_trades_ending_buy[i]=this.tau_trades_ending_buy[i1];
	    }
	    else
	    {
		this.v_trades_buy_exe[i]=0;
	    }
	}

	if(this.v_trades_sell_exe[i]<0)
	{
	    if(i>0)
	    {
		final int i1=i-1;
		this.v_trades_sell_exe[i]=this.v_trades_sell_exe[i1];
		this.tau_trades_limit_sell[i]=this.tau_trades_limit_sell[i1];
		this.tau_trades_market_buy[i]=this.tau_trades_market_buy[i1];
		this.tau_trades_ending_sell[i]=this.tau_trades_ending_sell[i1];
	    }
	    else
	    {
		this.v_trades_sell_exe[i]=0;
	    }
	}
    } // END use previous if current not available

    for(int i=0;i<nt;i++)
    { // normalization
	final long n1=this.v_trades_buy_exe[i];
	if(n1>0)
	{
	    this.tau_trades_limit_buy[i]=(this.tau_trades_limit_buy[i]/n1)
		*ModelParameters.T_SUMM_SCALE;
	    this.tau_trades_market_sell[i]=(this.tau_trades_market_sell[i]/n1)
		*ModelParameters.T_SUMM_SCALE;
	}
	else
	{
	    this.tau_trades_limit_buy[i]=this.tau_trades_market_sell[i]=this.tau_trades_ending_buy[i]=-1;
	}
	final long n2=this.v_trades_sell_exe[i];
	if(n2>0)
	{
	    this.tau_trades_limit_sell[i]=(this.tau_trades_limit_sell[i]/n2)
		*ModelParameters.T_SUMM_SCALE;
	    this.tau_trades_market_buy[i]=(this.tau_trades_market_buy[i]/n2)
		*ModelParameters.T_SUMM_SCALE;
	}
	else
	{
	    this.tau_trades_limit_sell[i]=this.tau_trades_market_buy[i]=this.tau_trades_ending_sell[i]=-1;
	}

    } // END normalization


    if(true)
    {
	this.price_trades_aver_tau_trades_buy=new double [nt];
	this.price_trades_aver_tau_trades_sell=new double [nt];
	this.price_trades_aver_tauF_trades_buy=new double [nt];
	this.price_trades_aver_tauF_trades_sell=new double [nt];
	this.v_trades_tau_trades_buy=new long [nt];
        this.v_trades_tau_trades_sell=new long [nt];
	final double [] tmpd=new double [nt];
	final long [] tmpl=new long [nt],tbuy=new long [nt],tsell=new long [nt],
	    tbuyF=new long [nt],tsellF=new long [nt];
	for(int i=0;i<nt;i++)
	{
	    if(this.v_trades_buy_exe[i]>0)
	    {
		tbuy[i]=this.tau_trades_limit_buy[i]-this.tau_trades_market_sell[i];
		tbuyF[i]=2*tbuy[i];
	    }
	    else
	    {
		tbuy[i]=tbuyF[i]=-1;
	    }
	    if(this.v_trades_sell_exe[i]>0)
	    {
		tsell[i]=this.tau_trades_limit_sell[i]-this.tau_trades_market_buy[i];
		tsellF[i]=2*tsell[i];
	    }
	    else
	    {
		tsell[i]=tsellF[i]=-1;
	    }
	}

	calculateFlow(nt,
		      currenttime,
		      PRINT_DATA_DEBUG,
		      BookItem.BOOK_BUY,
		      BookItem.BOOK_SELL,
		      tbuy,
		      tmpl,
		      this.v_trades_tau_trades_buy,
		      tmpd,
		      this.price_trades_aver_tau_trades_buy);
	calculateFlow(nt,
		      currenttime,
		      PRINT_DATA_DEBUG,
		      BookItem.BOOK_BUY,
		      BookItem.BOOK_SELL,
		      tbuyF,
		      tmpl,
		      tmpl,
		      tmpd,
		      this.price_trades_aver_tauF_trades_buy);
	calculateFlow(nt,
		      currenttime,
		      PRINT_DATA_DEBUG,
		      BookItem.BOOK_SELL,
		      BookItem.BOOK_BUY,
		      tsell,
		      tmpl,
		      this.v_trades_tau_trades_sell,
		      tmpd,
		      this.price_trades_aver_tau_trades_sell);
	calculateFlow(nt,
		      currenttime,
		      PRINT_DATA_DEBUG,
		      BookItem.BOOK_SELL,
		      BookItem.BOOK_BUY,
		      tsellF,
		      tmpl,
		      tmpl,
		      tmpd,
		      this.price_trades_aver_tauF_trades_sell);
    }


    if(PRINT_DATA_DEBUG)
    {
	final double ds=this.dspread;
	System.err.println("rnum\tratio\tshift_lev\teff_shift\tshift\tdelta\tshift0\tdelta0\t"+
			   "vmatch\t\t"+
			   "vbuy0\tvbuy\ttau_limit_buy\ttau_market_sell\ttau_ending_buy\tpa_buy\t\t"+
			   "vsell0\tvsell\ttau_limit_sell\ttau_market_buy\ttau_ending_sell\tpa_sell\t");

	for(int i=0;i<nt;i++)
	{
	    final MatchOrdersVariousShifts.MatchWithShift t=lmref[i];
	    System.err.println(String.valueOf(i)+"\t"+
			       (shiftlevels[i]/(0.5*(pcurrent_buy+pcurrent_sell)))+"\t"+
			       shiftlevels[i]+"\t"+
			       (t!=null?
				((t.shift+ds)+"\t"+t.shift+"\t"+t.delta+"\t"+t.shift0+"\t"+t.delta0+"\t"+t.vmatch):
				("\t\t\t\t\t"))+"\t\t"+
			       (t!=null?t.vbuy0:0)+"\t"+
			       this.v_trades_buy_exe[i]+"\t"+
			       this.tau_trades_limit_buy[i]+"\t"+
			       this.tau_trades_market_sell[i]+"\t"+
			       this.tau_trades_ending_buy[i]+"\t"+
			       this.price_trades_aver_tau_trades_buy[i]+"\t\t"+
			       (t!=null?t.vsell0:0)+"\t"+
			       this.v_trades_sell_exe[i]+"\t"+
			       this.tau_trades_limit_sell[i]+"\t"+
			       this.tau_trades_market_buy[i]+"\t"+
			       this.tau_trades_ending_sell[i]+"\t"+
			       this.price_trades_aver_tau_trades_sell+"\t");
	}
    }
    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
    {
	for(int i=0;i<nt;i++)
	{
	    final MatchOrdersVariousShifts.MatchWithShift t=lmref[i];
	    checkTaus(currenttime,
		      BookItem.BOOK_BUY,
		      (t!=null?FACTOR_LIQUIDITY_AMPLIFICATION*t.vbuy0:0),
		      this.v_trades_buy_exe[i],
		      this.tau_trades_limit_buy[i],
		      this.tau_trades_market_sell[i],
		      this.tau_trades_ending_buy[i]);
	    checkTaus(currenttime,
		      BookItem.BOOK_SELL,
		      (t!=null?FACTOR_LIQUIDITY_AMPLIFICATION*t.vsell0:0),
		      this.v_trades_sell_exe[i],
		      this.tau_trades_limit_sell[i],
		      this.tau_trades_market_buy[i],
		      this.tau_trades_ending_sell[i]);
	}
    }
}


private void checkTaus(final long currenttime,
		       final int type,
		       final long vcut,
		       final long v_check,
		       final long tau_limit_check,
		       final long tau_market_check,
		       final long tau_end_check)
{
    final int tsize=this.trades.getSize();
    final Trade [] tb=this.trades.getBuffer();
    long v=0,smarket=0,slimit=0,prevt=Long.MAX_VALUE,taux=Long.MAX_VALUE;
    gloop:for(int i=tsize;i-- >0;)
    {
	final Trade t=tb[i];
	if(t.time>prevt)
	{
	    throw new RuntimeException("The data is not sorted:"+
				       prevt+" and "+t.time+
				       " currenttime="+currenttime);
	}
	prevt=t.time;
	if(t.origtime>0 && t.buy_sell_indicator==type)
	{
	    final long
		tau=currenttime-t.time,
		stau_orig=t.shares*((currenttime-getorigtimeadj(t.origtime,t.time))/ModelParameters.T_SUMM_SCALE),
		stau_exe=t.shares*(tau/ModelParameters.T_SUMM_SCALE);
	    v+=t.shares;
	    if(v>vcut)
	    {
		v-=t.shares;
		break gloop;
	    }
	    smarket+=stau_exe;
	    slimit+=stau_orig;
	    taux=tau;
	}
    }
    final long tau_limit,tau_market,tau_end;
    if(v>0)
    {
	tau_limit=(slimit/v)*ModelParameters.T_SUMM_SCALE;
	tau_market=(smarket/v)*ModelParameters.T_SUMM_SCALE;
	tau_end=taux;
    }
    else
    {
	tau_limit=tau_market=tau_end=-1;
    }
    if(v_check!=v||
       tau_limit_check!=tau_limit||
       tau_market_check!=tau_market||
       tau_end_check!=tau_end)
    {
	throw new RuntimeException(
	    "Mismatch:"+
	    " v_check="+v_check+
	    " v="+v+
	    " tau_limit_check="+tau_limit_check+
	    " tau_limit="+tau_limit+
	    " tau_market_check="+tau_market_check+
	    " tau_market="+tau_market+
	    " tau_end_check="+tau_end_check+
	    " tau_end="+tau_end+
	    " type="+type+
	    " currenttime="+currenttime);
    }
}





private void checkBookAverages(final long currenttime,
			       final long vcut,
			       final BookItem [] buf,
			       final double price_cut,
			       final double price_aver,
			       final double price_taver,
			       final double price_sqtaver,
			       final double w_sqtaver,
			       final long tau_aver)
{
    long v=0,ta=0;
    double pe=Double.NaN,pa=0,pat=0,psqta=0,w=0;
    for(int i=0;v<vcut;i++)
    {
	final BookItem t=buf[i];
	v+=t.shares;

	pe=t.price;
	pa+=pe*t.shares;

	final long tau=(currenttime-t.time)/ModelParameters.T_SUMM_SCALE;
	final long wtau=t.shares*tau;
	ta+=wtau;
	pat+=pe*wtau;

	final double ws=t.shares*Math.sqrt(tau);
	psqta+=pe*ws;
	w+=ws;
    }
    if(v>0)
    {
	pa/=v;
	pat/=ta;
	ta=(ta/v)*ModelParameters.T_SUMM_SCALE;
	psqta/=w;
    }
    else
    {
	pa=pat=psqta=Double.NaN;
	ta=-1;
    }

    if(v!=vcut||
       ta!=tau_aver||
       Double.isNaN(pe)!=Double.isNaN(price_cut)||
       (!Double.isNaN(pe) && pe!=price_cut)||
       Double.isNaN(pa)!=Double.isNaN(price_aver)||
       (!Double.isNaN(pa) && pa!=price_aver)||
       Double.isNaN(pat)!=Double.isNaN(price_taver)||
       (!Double.isNaN(pat) && pat!=price_taver)||
       Double.isNaN(psqta)!=Double.isNaN(price_sqtaver)||
       (!Double.isNaN(psqta) && psqta!=price_sqtaver)||
       Double.isNaN(w)!=Double.isNaN(w_sqtaver)||
       (!Double.isNaN(w) && w!=w_sqtaver))
    {
	throw new RuntimeException(
	    "Mismatch:"+
	    " v="+v+
	    " vcut="+vcut+
	    " ta="+ta+
	    " tau_aver="+tau_aver+
	    " pe="+pe+
	    " price_cut="+price_cut+
	    " pa="+pa+
	    " price_aver="+price_aver+
	    " pat="+pat+
	    " price_taver="+price_taver+
	    " psqta="+psqta+
	    " price_sqtaver="+price_sqtaver+
	    " w="+w+
	    " w_sqtaver="+w_sqtaver+
	    " currenttime="+currenttime);
    }
}



static long getorigtimeadj(final long origtime,final long exe)
{
    if(true)
    {
	// unadjusted time
	return origtime;
    }
    else
    {
	// adjusted time
	final long t1=exe-BookBuilder.TIME_KEEP_ORDER_INFO;
	return origtime<t1?t1:origtime;
    }
}


}
