package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2010
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit v20.
 *  @version $Id: CalculateLiquidityDeficit20.java,v 1.21 2016/04/25 21:58:37 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit20 {

private final Distribution<Trade> trades;
private final Distribution<BookItem> buy,sell;

    CalculateLiquidityDeficit20(final Distribution<Trade> trades,
				final Distribution<BookItem> buy,
				final Distribution<BookItem> sell)
    {
	this.trades=trades;
	this.buy=buy;
	this.sell=sell;
    }


    /** Results container.
     */
    static class VQ {
	final long currenttime,tau;
	final NkQVMLaguerre nkqvmll;
	final NkQVMLegendreShifted nkqvmpp;
	final OkQVMLaguerre okqvmll;
	final OkQVMLegendreShifted okqvmpp;
	final QkQVMLaguerre qkqvmll;
	final QkQVMLegendreShifted qkqvmpp;
	final PkQVMLaguerre pkqvmll;
	final PkQVMLegendreShifted pkqvmpp;
	final QkQVMpi qkqvmpi;
	final QmQVMpi qmqvmpi;
	final RkQVMLaguerre rkqvmll;
	final RkQVMLegendreShifted rkqvmpp;
	final RkiQVMLaguerre rkiqvmll;
	final RkiQVMLegendreShifted rkiqvmpp;
	final SkQVMLaguerre skqvmll;
	final SkQVMLegendreShifted skqvmpp;
	final TkQVMLaguerre tkqvmll;
	final TkQVMLegendreShifted tkqvmpp;
	final UkQVMLaguerre ukqvmll;
	final UkQVMLegendreShifted ukqvmpp;
	final UkbQVMBMP ukbqvmbmp;
	final UnQVMLaguerre unqvmll;
	final UnQVMLegendreShifted unqvmpp;
	final VkQVMLaguerre vkqvmll;
	final VkQVMLegendreShifted vkqvmpp;
	final YkQVMLaguerre ykqvmll;
	final YkQVMLegendreShifted ykqvmpp;
	final YpkQVMLaguerre ypkqvmll;
	final YpkQVMLegendreShifted ypkqvmpp;
	final YpnlQVMLaguerre ypnlqvmll;
	final YpnlQVMLegendreShifted ypnlqvmpp;
	final YkDpILaguerre ykdpill;
	final YkDpILegendreShifted ykdpipp;

	VQ(final long currenttime,
	   final long tau,
	   final NkQVMLaguerre nkqvmll,
	   final NkQVMLegendreShifted nkqvmpp,
	   final OkQVMLaguerre okqvmll,
	   final OkQVMLegendreShifted okqvmpp,
	   final QkQVMLaguerre qkqvmll,
	   final QkQVMLegendreShifted qkqvmpp,
	   final PkQVMLaguerre pkqvmll,
	   final PkQVMLegendreShifted pkqvmpp,
	   final QkQVMpi qkqvmpi,
	   final QmQVMpi qmqvmpi,
	   final RkQVMLaguerre rkqvmll,
	   final RkQVMLegendreShifted rkqvmpp,
	   final RkiQVMLaguerre rkiqvmll,
	   final RkiQVMLegendreShifted rkiqvmpp,
	   final SkQVMLaguerre skqvmll,
	   final SkQVMLegendreShifted skqvmpp,
	   final TkQVMLaguerre tkqvmll,
	   final TkQVMLegendreShifted tkqvmpp,
	   final UkQVMLaguerre ukqvmll,
	   final UkQVMLegendreShifted ukqvmpp,
	   final UkbQVMBMP ukbqvmbmp,
	   final UnQVMLaguerre unqvmll,
	   final UnQVMLegendreShifted unqvmpp,
	   final VkQVMLaguerre vkqvmll,
	   final VkQVMLegendreShifted vkqvmpp,
	   final YkQVMLaguerre ykqvmll,
	   final YkQVMLegendreShifted ykqvmpp,
	   final YpkQVMLaguerre ypkqvmll,
	   final YpkQVMLegendreShifted ypkqvmpp,
	   final YpnlQVMLaguerre ypnlqvmll,
	   final YpnlQVMLegendreShifted ypnlqvmpp,
	   final YkDpILaguerre ykdpill,
	   final YkDpILegendreShifted ykdpipp
	   )
	{
	    this.currenttime=currenttime;
	    this.tau=tau;
	    this.nkqvmll=nkqvmll;
	    this.nkqvmpp=nkqvmpp;
	    this.okqvmll=okqvmll;
	    this.okqvmpp=okqvmpp;
	    this.qkqvmll=qkqvmll;
	    this.qkqvmpp=qkqvmpp;
	    this.pkqvmll=pkqvmll;
	    this.pkqvmpp=pkqvmpp;
	    this.qkqvmpi=qkqvmpi;
	    this.qmqvmpi=qmqvmpi;
	    this.rkqvmll=rkqvmll;
	    this.rkqvmpp=rkqvmpp;
	    this.rkiqvmll=rkiqvmll;
	    this.rkiqvmpp=rkiqvmpp;
	    this.skqvmll=skqvmll;
	    this.skqvmpp=skqvmpp;
	    this.tkqvmll=tkqvmll;
	    this.tkqvmpp=tkqvmpp;
	    this.ukqvmll=ukqvmll;
	    this.ukqvmpp=ukqvmpp;
	    this.ukbqvmbmp=ukbqvmbmp;
	    this.unqvmll=unqvmll;
	    this.unqvmpp=unqvmpp;
	    this.vkqvmll=vkqvmll;
	    this.vkqvmpp=vkqvmpp;
	    this.ykqvmll=ykqvmll;
	    this.ykqvmpp=ykqvmpp;
	    this.ypkqvmll=ypkqvmll;
	    this.ypkqvmpp=ypkqvmpp;
	    this.ypnlqvmll=ypnlqvmll;
	    this.ypnlqvmpp=ypnlqvmpp;
	    this.ykdpill=ykdpill;
	    this.ykdpipp=ykdpipp;
	}
    }
    VQ [] tdata=null;


    void clear()
    {
	this.tdata=null;
    }

    void calculateLiquidityDeficitRecurrent(final int neq_t,
					    final long [] delays,
					    final long currenttime,
					    final long trades_cleared_before,
					    final VQ [] prev)
    {
	final int nt=delays.length;
	final boolean PRINT_DATA_DEBUG=false;


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tN TRADES=\t"+this.trades.getSize()+
			       "\tnt\t"+nt);
	}

	this.tdata=new VQ [nt];
	for(int i=0;i<nt;i++)
	{
	    final VQ amprev=prev!=null?prev[i]:null;
	    final long tau=delays[i];
	    this.tdata[i]=new VQ(currenttime,
				 tau,
				 NkQVMLaguerre.calculate1(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.nkqvmll:null),
				 NkQVMLegendreShifted.calculate1(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.nkqvmpp:null),
				 OkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.okqvmll:null),
				 OkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.okqvmpp:null),
				 QkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.qkqvmll:null),
				 QkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.qkqvmpp:null),
				 PkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.pkqvmll:null),
				 PkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.pkqvmpp:null),
				 QkQVMpi.calculate(neq_t,
						   currenttime,
						   tau,
						   this.trades.getBuffer(),
						   this.trades.getSize(),
						   amprev!=null?amprev.qkqvmpi:null),
				 QmQVMpi.calculate(neq_t,
						   currenttime,
						   tau,
						   this.trades.getBuffer(),
						   this.trades.getSize(),
						   amprev!=null?amprev.qmqvmpi:null),
				 RkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.rkqvmll:null),
				 RkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.rkqvmpp:null),
				 RkiQVMLaguerre.calculate(neq_t,
							  currenttime,
							  tau,
							  this.trades.getBuffer(),
							  this.trades.getSize(),
							  amprev!=null?amprev.rkiqvmll:null),
				 RkiQVMLegendreShifted.calculate(neq_t,
								 currenttime,
								 tau,
								 this.trades.getBuffer(),
								 this.trades.getSize(),
								 amprev!=null?amprev.rkiqvmpp:null),
				 SkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.skqvmll:null),
				 SkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.skqvmpp:null),
				 TkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.tkqvmll:null),
				 TkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.tkqvmpp:null),
				 UkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.ukqvmll:null),
				 UkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.ukqvmpp:null),
				 UkbQVMBMP.calculate(neq_t,
						     currenttime,
						     tau,
						     this.buy.getBuffer(),
						     this.buy.getSize(),
						     this.sell.getBuffer(),
						     this.sell.getSize(),
						     ModelParameters.TRY_PCUT),
				 UnQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.unqvmll:null),
				 UnQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.unqvmpp:null),
				 VkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.vkqvmll:null),
				 VkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.vkqvmpp:null),
				 YkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.ykqvmll:null),
				 YkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.ykqvmpp:null),
				 YpkQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.ypkqvmll:null),
				 YpkQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.ypkqvmpp:null),
				 YpnlQVMLaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.ypnlqvmll:null),
				 YpnlQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.ypnlqvmpp:null),
				 YkDpILaguerre.calculate(neq_t,
							 currenttime,
							 tau,
							 this.trades.getBuffer(),
							 this.trades.getSize(),
							 amprev!=null?amprev.ykdpill:null),
				 YkDpILegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								amprev!=null?amprev.ykdpipp:null)
				 );
	}
    }
}

