package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit v21.
 *  @version $Id: CalculateLiquidityDeficit21.java,v 1.14 2019/03/13 11:55:51 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit21 {

private final Distribution<Trade> trades;
private final Distribution<BookItem> buy,sell;

    CalculateLiquidityDeficit21(final Distribution<Trade> trades,
				final Distribution<BookItem> buy,
				final Distribution<BookItem> sell)
    {
	this.trades=trades;
	this.buy=buy;
	this.sell=sell;
    }


    /** Results container.
     */
    static class VQ {
	final long currenttime,tau;
	final ImpactQVMLaguerre iqvmll;
	final ImpactQVMLegendreShifted iqvmpp;
	final QkQVMpi qkqvmpi;
	final QkPnLpi_PBasis qkpnlpbasis;
	final ImpactQVMMuseLaguerre iqvmmll;
	final ImpactQVMMuseLegendreShifted iqvmmpp;
	final DynMicroImpactLaguerre dmill;
	final DynMicroImpactLegendreShifted dmipp;
	final DynMicroImpact_pi dmipi;
	final DynImpactAsymmetryLaguerre diall;
	final DynImpactAsymmetryLegendreShifted diapp;
	final DynImpactAsymmetry_pi diapi;
	final IAImpactLaguerre iall;
	final IAImpactLegendreShifted iapp;
	final IAImpact_pi iapi;
	final IStateLaguerre isll;
	final IStateLegendreShifted ispp;
	final IState_pi ispi;
	final IHStateLaguerre ihll;
	final IHStateLegendreShifted ihpp;
	final IHStateDMLaguerre ihdmll;
	final IHStateDMLegendreShifted ihdmpp;
	final ScalpedMaxIProjectionLaguerre cmill;
	final ScalpedMaxIProjectionLegendreShifted cmipp;

	VQ(final long currenttime,
	   final long tau,
	   final ImpactQVMLaguerre iqvmll,
	   final ImpactQVMLegendreShifted iqvmpp,
	   final QkQVMpi qkqvmpi,
	   final QkPnLpi_PBasis qkpnlpbasis,
	   final ImpactQVMMuseLaguerre iqvmmll,
	   final ImpactQVMMuseLegendreShifted iqvmmpp,
	   final DynMicroImpactLaguerre dmill,
	   final DynMicroImpactLegendreShifted dmipp,
	   final DynMicroImpact_pi dmipi,
	   final DynImpactAsymmetryLaguerre diall,
	   final DynImpactAsymmetryLegendreShifted diapp,
	   final DynImpactAsymmetry_pi diapi,
	   final IAImpactLaguerre iall,
	   final IAImpactLegendreShifted iapp,
	   final IAImpact_pi iapi,
	   final IStateLaguerre isll,
	   final IStateLegendreShifted ispp,
	   final IState_pi ispi,
	   final IHStateLaguerre ihll,
	   final IHStateLegendreShifted ihpp,
	   final IHStateDMLaguerre ihdmll,
	   final IHStateDMLegendreShifted ihdmpp,
	   final ScalpedMaxIProjectionLaguerre cmill,
	   final ScalpedMaxIProjectionLegendreShifted cmipp
	   )
	{
	    this.currenttime=currenttime;
	    this.tau=tau;
	    this.iqvmll=iqvmll;
	    this.iqvmpp=iqvmpp;
	    this.qkqvmpi=qkqvmpi;
	    this.qkpnlpbasis=qkpnlpbasis;
	    this.iqvmmll=iqvmmll;
	    this.iqvmmpp=iqvmmpp;
	    this.dmill=dmill;
	    this.dmipp=dmipp;
	    this.dmipi=dmipi;
	    this.diall=diall;
	    this.diapp=diapp;
	    this.diapi=diapi;
	    this.iall=iall;
	    this.iapp=iapp;
	    this.iapi=iapi;
	    this.isll=isll;
	    this.ispp=ispp;
	    this.ispi=ispi;
	    this.ihll=ihll;
	    this.ihpp=ihpp;
	    this.ihdmll=ihdmll;
	    this.ihdmpp=ihdmpp;
	    this.cmill=cmill;
	    this.cmipp=cmipp;
	}
    }
    VQ [] tdata=null;


    void clear()
    {
	this.tdata=null;
    }

    void calculateLiquidityDeficitRecurrent(final int neq_t,
					    final long [] delays,
					    final long currenttime,
					    final long trades_cleared_before,
					    final VQ [] prev)
    {
	final int nt=delays.length;
	final boolean PRINT_DATA_DEBUG=false;


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tN TRADES=\t"+this.trades.getSize()+
			       "\tnt\t"+nt);
	}

	this.tdata=new VQ [nt];
	for(int i=0;i<nt;i++)
	{
	    final VQ amprev=prev!=null?prev[i]:null;
	    final long tau=delays[i];
	    this.tdata[i]=new VQ(currenttime,
				 tau,
				 ImpactQVMLaguerre.calculate(neq_t,
							     currenttime,
							     tau,
							     this.trades.getBuffer(),
							     this.trades.getSize(),
							     amprev!=null?amprev.iqvmll:null),
				 ImpactQVMLegendreShifted.calculate(neq_t,
								currenttime,
								tau,
								this.trades.getBuffer(),
								this.trades.getSize(),
								    amprev!=null?amprev.iqvmpp:null),
				 QkQVMpi.calculate(neq_t,
						   currenttime,
						   tau,
						   this.trades.getBuffer(),
						   this.trades.getSize(),
						   amprev!=null?amprev.qkqvmpi:null),
				 QkPnLpi_PBasis.calculate(neq_t,
						   currenttime,
						   tau,
						   this.trades.getBuffer(),
						   this.trades.getSize(),
							  amprev!=null?amprev.qkpnlpbasis:null),
				 ImpactQVMMuseLaguerre.calculate(neq_t,
								 currenttime,
								 tau,
								 this.trades.getBuffer(),
								 this.trades.getSize(),
								 amprev!=null?amprev.iqvmmll:null),
				 ImpactQVMMuseLegendreShifted.calculate(neq_t,
									currenttime,
									tau,
									this.trades.getBuffer(),
									this.trades.getSize(),
									amprev!=null?amprev.iqvmmpp:null),
				 DynMicroImpactLaguerre.calculate(neq_t,
								  currenttime,
								  tau,
								  this.trades.getBuffer(),
								  this.trades.getSize(),
								  amprev!=null?amprev.dmill:null),
				 DynMicroImpactLegendreShifted.calculate(neq_t,
									 currenttime,
									 tau,
									 this.trades.getBuffer(),
									 this.trades.getSize(),
									 amprev!=null?amprev.dmipp:null),
				 DynMicroImpact_pi.calculate(neq_t,
							     currenttime,
							     tau,
							     this.trades.getBuffer(),
							     this.trades.getSize(),
							     amprev!=null?amprev.dmipi:null),
				 DynImpactAsymmetryLaguerre.calculate(neq_t,
								      currenttime,
								      tau,
								      this.trades.getBuffer(),
								      this.trades.getSize(),
								      amprev!=null?amprev.diall:null),
				 DynImpactAsymmetryLegendreShifted.calculate(neq_t,
									     currenttime,
									     tau,
									     this.trades.getBuffer(),
									     this.trades.getSize(),
									     amprev!=null?amprev.diapp:null),
				 DynImpactAsymmetry_pi.calculate(neq_t,
								 currenttime,
								 tau,
								 this.trades.getBuffer(),
								 this.trades.getSize(),
								 amprev!=null?amprev.diapi:null),
				 IAImpactLaguerre.calculate(neq_t,
							    currenttime,
							    tau,
							    this.trades.getBuffer(),
							    this.trades.getSize(),
							    amprev!=null?amprev.iall:null),
				 IAImpactLegendreShifted.calculate(neq_t,
								   currenttime,
								   tau,
								   this.trades.getBuffer(),
								   this.trades.getSize(),
								   amprev!=null?amprev.iapp:null),
				 IAImpact_pi.calculate(neq_t,
						       currenttime,
						       tau,
						       this.trades.getBuffer(),
						       this.trades.getSize(),
						       amprev!=null?amprev.iapi:null),
				 IStateLaguerre.calculate(neq_t,
							  currenttime,
							  tau,
							  this.trades.getBuffer(),
							  this.trades.getSize(),
							  amprev!=null?amprev.isll:null),
				 IStateLegendreShifted.calculate(neq_t,
								 currenttime,
								 tau,
								 this.trades.getBuffer(),
								 this.trades.getSize(),
								 amprev!=null?amprev.ispp:null),
				 IState_pi.calculate(neq_t,
						     currenttime,
						     tau,
						     this.trades.getBuffer(),
						     this.trades.getSize(),
						     amprev!=null?amprev.ispi:null),
				 IHStateLaguerre.calculate(neq_t,
							   currenttime,
							   tau,
							   this.trades.getBuffer(),
							   this.trades.getSize(),
							   amprev!=null?amprev.ihll:null),
				 IHStateLegendreShifted.calculate(neq_t,
								  currenttime,
								  tau,
								  this.trades.getBuffer(),
								  this.trades.getSize(),
								  amprev!=null?amprev.ihpp:null),
				 IHStateDMLaguerre.calculate(neq_t,
							     currenttime,
							     tau,
							     this.trades.getBuffer(),
							     this.trades.getSize(),
							     amprev!=null?amprev.ihdmll:null),
				 IHStateDMLegendreShifted.calculate(neq_t,
								    currenttime,
								    tau,
								    this.trades.getBuffer(),
								    this.trades.getSize(),
								    amprev!=null?amprev.ihdmpp:null),
				 ScalpedMaxIProjectionLaguerre.calculate(neq_t,
								   currenttime,
								   tau,
								   this.trades.getBuffer(),
								   this.trades.getSize(),
								   amprev!=null?amprev.cmill:null),
				 ScalpedMaxIProjectionLegendreShifted.calculate(neq_t,
									  currenttime,
									  tau,
									  this.trades.getBuffer(),
									  this.trades.getSize(),
									  amprev!=null?amprev.cmipp:null)
				 );
	}
    }
}

