package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit. Updated version.
 *  @version $Id: CalculateLiquidityDeficit3.java,v 1.182 2022/07/18 08:17:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit3 {

    private final Distribution<BookItem> buy,sell,tmpordersbuf;
    private final Distribution<Trade> trades,tmptradesbuf;

    CalculateLiquidityDeficit3(final Distribution<BookItem> buy,
			       final Distribution<BookItem> sell,
			       final Distribution<Trade> trades,
			       final Distribution<BookItem> tmpordersbuf,
			       final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
	this.tmpordersbuf=tmpordersbuf;
	this.tmptradesbuf=tmptradesbuf;
    }

    int []
	ptr_trades_buy=null,
	ptr_trades_sell=null,
	ptr_book_buy=null,
	ptr_book_sell=null;

    double []
	price_book_level_buy=null,
	price_book_split_buy=null,
	price_book_level_sell=null,
	price_book_split_sell=null;


    long []
	v_trades_exe_buy=null,
	v_trades_exe_sell=null,
	tte_trades_buy=null,
	tte_trades_sell=null,
	tte_trades_buy_max=null,
	tte_trades_sell_max=null,
	v_trades_buy_to_tte_max=null,
	v_trades_sell_to_tte_max=null,

	v_book_acc_buy=null,
	v_book_acc_sell=null,
	tte_book_lev_buy=null,
	tte_book_lev_sell=null,
	tte_book_acc_buy=null,
	tte_book_acc_sell=null;

    double []
	price_aver_trades_tte_buy=null,
	price_aver_trades_tte_sell=null,
	price_max_trades_tte_buy=null,
	price_min_trades_tte_sell=null,

	price_aver_book_acc_tte_buy=null,
	price_aver_book_acc_tte_sell=null,
	price_max_book_acc_tte_buy=null,
	price_min_book_acc_tte_sell=null,

	price_inv_ttrades_buy=null,
	price_inv_ttrades_sell=null,
	price_inv_tbook_buy=null,
	price_inv_tbook_sell=null,

	price_min_t_book_acc_tte_buy=null,
	price_max_t_book_acc_tte_buy=null,
	price_min_t_book_acc_tte_sell=null,
	price_max_t_book_acc_tte_sell=null;

    long [] vol_trades_t_book_acc_tte_buy=null,
	vol_trades_t_book_acc_tte_sell=null;

    int []
	n_levs_inv_ttrades_buy=null,
	n_levs_inv_ttrades_sell=null;

    void clear()
    {
	this.ptr_trades_buy=
	this.ptr_trades_sell=
	this.ptr_book_buy=
	this.ptr_book_sell=null;

	this.price_book_level_buy=
	this.price_book_split_buy=
	this.price_book_level_sell=
	this.price_book_split_sell=null;


	this.v_trades_exe_buy=
	this.v_trades_exe_sell=
	this.tte_trades_buy=
	this.tte_trades_sell=
	this.tte_trades_buy_max=
	this.tte_trades_sell_max=
	this.v_trades_buy_to_tte_max=
	this.v_trades_sell_to_tte_max=
	this.v_book_acc_buy=
	this.v_book_acc_sell=
	this.tte_book_lev_buy=
	this.tte_book_lev_sell=
	this.tte_book_acc_buy=
	this.tte_book_acc_sell=null;


	this.price_aver_trades_tte_buy=
	this.price_aver_trades_tte_sell=
	this.price_max_trades_tte_buy=
	this.price_min_trades_tte_sell=
	this.price_aver_book_acc_tte_buy=
	this.price_aver_book_acc_tte_sell=
	this.price_max_book_acc_tte_buy=
	this.price_min_book_acc_tte_sell=
	this.price_inv_ttrades_buy=
	this.price_inv_ttrades_sell=
	this.price_inv_tbook_buy=
	this.price_inv_tbook_sell=null;

	this.price_min_t_book_acc_tte_buy=
	this.price_max_t_book_acc_tte_buy=
	this.price_min_t_book_acc_tte_sell=
	this.price_max_t_book_acc_tte_sell=null;

	this.vol_trades_t_book_acc_tte_buy=
	this.vol_trades_t_book_acc_tte_sell=null;

	this.n_levs_inv_ttrades_buy=
	this.n_levs_inv_ttrades_sell=null;

    }


    void calculateLiquidityDeficit(final long currenttime,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final int n_levels_trades,
				   final long trades_cleared_before)
    {

	final boolean PRINT_DATA_DEBUG=false;
	    //currenttime==34434604000000L;
	    //currenttime==34354409000000L;
	    //currenttime==34223484000000L;
	    //currenttime>=10*3600*TraderInterface.TIME_SCALE;
	    //currenttime==34204365000000L;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println();
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tN_LEVELS=\t"+n_levels_trades+
			       "\tcur_buy\t"+pcurrent_buy+
			       "\tcur_sell\t"+pcurrent_sell);
	}

	final int nt=n_levels_trades;
	final long [] tmp=new long [nt],tmp2=new long [nt];
	final int [] tmpd=new int [nt];


	this.ptr_book_buy=new int [nt];
	this.price_book_level_buy=new double [nt];
	this.price_book_split_buy=new double [nt];
	calculateBookTauBSplit(currenttime,
			       nt,
			       PRINT_DATA_DEBUG,
			       this.buy,
			       true,

			       this.ptr_book_buy,
			       tmp,
			       tmp2,
			       this.price_book_level_buy,
			       this.price_book_split_buy,

			       tmpd);
	this.ptr_book_sell=new int [nt];
	this.price_book_level_sell=new double [nt];
	this.price_book_split_sell=new double [nt];
	calculateBookTauBSplit(currenttime,
			       nt,
			       PRINT_DATA_DEBUG,
			       this.sell,
			       false,

			       this.ptr_book_sell,
			       tmp,
			       tmp2,
			       this.price_book_level_sell,
			       this.price_book_split_sell,

			       tmpd);



	calculateTradeTaus(currenttime,
			   pcurrent_buy,
			   pcurrent_sell,
			   nt,
			   trades_cleared_before,
			   PRINT_DATA_DEBUG,
			   0,
			   0,
			   0,
			   0);

	this.price_aver_trades_tte_buy=new double [nt];
	this.price_aver_trades_tte_sell=new double [nt];
	this.price_max_trades_tte_buy=new double [nt];
	this.price_min_trades_tte_sell=new double [nt];
	calculatePriceD(currenttime,
			pcurrent_buy,
			pcurrent_sell,
			nt,
			trades_cleared_before,
			PRINT_DATA_DEBUG,
			this.trades,
			this.tte_trades_buy,
			this.tte_trades_sell,
			tmp,
			tmp2,
			this.price_aver_trades_tte_buy,
			this.price_aver_trades_tte_sell,
			this.price_max_trades_tte_buy,
			this.price_min_trades_tte_sell,

			this.tmptradesbuf);

	if(true)
	{
	    this.tte_book_lev_buy=new long [nt];
	    this.v_book_acc_buy=new long [nt];
	    this.tte_book_acc_buy=new long [nt];
	    calculateBookTau(currenttime,
			     pcurrent_buy,
			     pcurrent_sell,
			     nt,
			     PRINT_DATA_DEBUG,
			     this.buy.getBuffer(),
			     this.buy.getSize(),
			     true,
			     this.ptr_book_buy,
			     tmp,
			     this.tte_book_lev_buy,
			     this.v_book_acc_buy,
			     this.tte_book_acc_buy);
	    this.price_min_t_book_acc_tte_buy=new double [nt];
	    this.price_max_t_book_acc_tte_buy=new double [nt];
	    this.vol_trades_t_book_acc_tte_buy=new long [nt];
	    CalculateLiquidityDeficit4.calculateTradesMM(
		currenttime,
		this.tte_book_acc_buy,
		nt,
		this.trades,
		PRINT_DATA_DEBUG,
		tmpd,
		// output
		this.price_min_t_book_acc_tte_buy,
		this.price_max_t_book_acc_tte_buy,
		null,
		this.vol_trades_t_book_acc_tte_buy);

	    this.tte_book_lev_sell=new long [nt];
	    this.v_book_acc_sell=new long [nt];
	    this.tte_book_acc_sell=new long [nt];
	    calculateBookTau(currenttime,
			     pcurrent_buy,
			     pcurrent_sell,
			     nt,
			     PRINT_DATA_DEBUG,
			     this.sell.getBuffer(),
			     this.sell.getSize(),
			     false,
			     this.ptr_book_sell,
			     tmp,
			     this.tte_book_lev_sell,
			     this.v_book_acc_sell,
			     this.tte_book_acc_sell);
	    this.price_min_t_book_acc_tte_sell=new double [nt];
	    this.price_max_t_book_acc_tte_sell=new double [nt];
	    this.vol_trades_t_book_acc_tte_sell=new long [nt];
	    CalculateLiquidityDeficit4.calculateTradesMM(
		currenttime,
		this.tte_book_acc_sell,
		nt,
		this.trades,
		PRINT_DATA_DEBUG,
		tmpd,
		// output
		this.price_min_t_book_acc_tte_sell,
		this.price_max_t_book_acc_tte_sell,
		null,
		this.vol_trades_t_book_acc_tte_sell);

	    this.price_inv_ttrades_buy=new double [nt];
	    this.price_inv_tbook_buy=new double [nt];
	    this.n_levs_inv_ttrades_buy=new int [nt];
	    this.price_inv_ttrades_sell=new double [nt];
	    this.price_inv_tbook_sell=new double [nt];
	    this.n_levs_inv_ttrades_sell=new int [nt];
	    this.tte_trades_buy_max=new long [nt];
	    this.tte_trades_sell_max=new long [nt];
	    this.v_trades_buy_to_tte_max=new long [nt];
	    this.v_trades_sell_to_tte_max=new long [nt];

	    for(int i=0;i<nt;i++)
	    {

		calculateBookInvertionPrice(currenttime,
					    0,
					    this.buy,
					    true,
					    this.tte_trades_buy[i],
					    false,
					    PRINT_DATA_DEBUG,
					    i,
					    this.price_inv_ttrades_buy,
					    this.n_levs_inv_ttrades_buy);
		calculateBookInvertionPrice(currenttime,
					    0,
					    this.sell,
					    false,
					    this.tte_trades_sell[i],
					    false,
					    PRINT_DATA_DEBUG,
					    i,
					    this.price_inv_ttrades_sell,
					    this.n_levs_inv_ttrades_sell);


		calculateBookInvertionPrice(currenttime,
					    0,
					    this.buy,
					    true,
					    this.tte_book_acc_buy[i],
					    false,
					    PRINT_DATA_DEBUG,
					    i,
					    this.price_inv_tbook_buy,
					    tmpd);
		calculateBookInvertionPrice(currenttime,
					    0,
					    this.sell,
					    false,
					    this.tte_book_acc_sell[i],
					    false,
					    PRINT_DATA_DEBUG,
					    i,
					    this.price_inv_tbook_sell,
					    tmpd);

		final double pl_buy=this.ptr_book_buy[i]<0?Double.POSITIVE_INFINITY:this.buy.getBuffer()[this.ptr_book_buy[i]].price;
		final double pl_sell=this.ptr_book_sell[i]<0?Double.NEGATIVE_INFINITY:this.sell.getBuffer()[this.ptr_book_sell[i]].price;
		final  CalculateLiquidityDeficit7.PTRToPriceTrange
		    ptr_buy=CalculateLiquidityDeficit7.findPtrToTradesPrice(
			currenttime,
			this.trades,
			pl_buy,
			Double.NaN),
		    ptr_sell=CalculateLiquidityDeficit7.findPtrToTradesPrice(
			currenttime,
			this.trades,
			Double.NaN,
			pl_sell);
		this.tte_trades_buy_max[i]=ptr_buy.t_range_cut>0?currenttime-ptr_buy.t_range_cut:-1;
		this.v_trades_buy_to_tte_max[i]=ptr_buy.vi;
		this.tte_trades_sell_max[i]=ptr_sell.t_range_cut>0?currenttime-ptr_sell.t_range_cut:-1;
		this.v_trades_sell_to_tte_max[i]=ptr_sell.vi;
	    }


	    this.price_aver_book_acc_tte_buy=new double [nt];
	    this.price_aver_book_acc_tte_sell=new double [nt];
	    this.price_max_book_acc_tte_buy=new double [nt];
	    this.price_min_book_acc_tte_sell=new double [nt];
	    calculatePriceD(currenttime,
			    pcurrent_buy,
			    pcurrent_sell,
			    nt,
			    trades_cleared_before,
			    PRINT_DATA_DEBUG,
			    this.trades,
			    this.tte_book_acc_buy,
			    this.tte_book_acc_sell,
			    tmp,
			    tmp2,
			    this.price_aver_book_acc_tte_buy,
			    this.price_aver_book_acc_tte_sell,
			    this.price_max_book_acc_tte_buy,
			    this.price_min_book_acc_tte_sell,

			    this.tmptradesbuf);

	}
    }




    private void calculateTradeTaus(final long currenttime,
				    final double pcurrent_buy,
				    final double pcurrent_sell,
				    final int nt,
				    final long trades_cleared_before,
				    final boolean PRINT_DATA_DEBUG,
				    final long v0_buy,
				    final long sv0_buy,
				    final long v0_sell,
				    final long sv0_sell)
    {
	this.v_trades_exe_buy=new long [nt];
	this.v_trades_exe_sell=new long [nt];
	this.tte_trades_buy=new long [nt];
	this.tte_trades_sell=new long [nt];

	this.ptr_trades_buy=new int [nt];
	this.ptr_trades_sell=new int [nt];
	// fill trades
	final int tsize=this.trades.getSize();
	final Trade [] tb=this.trades.getBuffer();
	double
	    price_buy_min_re=Double.POSITIVE_INFINITY,
	    price_buy_max_re=Double.NEGATIVE_INFINITY,
	    price_sell_min_re=Double.POSITIVE_INFINITY,
	    price_sell_max_re=Double.NEGATIVE_INFINITY;
	boolean sum_buy=true,sum_sell=true;
	long prevt=Long.MAX_VALUE;
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("num\t"+
			       "time\t"+
			       "origtime\t"+
			       "shares\t"+
			       "tradetype\t"+
			       "buy_sell_indicator\t"+
			       "price\t"+
			       "matchnumber\t"+
			       "orderreferencenumber");
	}
	long v_buy=v0_buy,sv_buy=sv0_buy,v_sell=v0_sell,sv_sell=sv0_sell;
	int xp_buy=-1,xp_sell=-1,n_levels_trades_buy=-1,n_levels_trades_sell=-1;
        gloop: for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println(String.valueOf(i)+"\t"+
				   t.time+"\t"+
				   t.origtime+"\t"+
				   t.shares+"\t"+
				   t.tradetype+"\t"+
				   t.buy_sell_indicator+"\t"+
				   t.price+"\t"+
				   t.matchnumber+"\t"+
				   t.orderreferencenumber);
	    }
	    if(t.origtime<=0)
	    {
		continue;
	    }
	    final long tte=(t.time-t.origtime)/ModelParameters.T_SUMM_SCALE;
	    if(t.buy_sell_indicator==BookItem.BOOK_BUY)
	    {
		if(sum_buy)
		{
		    final boolean fmin=t.price<price_buy_min_re,fmax=t.price>price_buy_max_re;
		    if(fmin||fmax)
		    { // next level
			if(n_levels_trades_buy>=0)
			{
			    this.v_trades_exe_buy[n_levels_trades_buy]=v_buy;
			    this.tte_trades_buy[n_levels_trades_buy]=sv_buy;
			    this.ptr_trades_buy[n_levels_trades_buy]=xp_buy;
			}
			if(++n_levels_trades_buy>=nt)
			{
			    sum_buy=false;
			    if(!sum_sell)
			    {
				break gloop;
			    }
			    continue gloop;
			}
			if(fmin)
			{
			    price_buy_min_re=(t.price>pcurrent_buy?pcurrent_buy:t.price)-OrdersFlow.EPS;
			}
			if(fmax)
			{
			    price_buy_max_re=(t.price<pcurrent_buy?pcurrent_buy:t.price)+OrdersFlow.EPS;
			}
		    }
		    v_buy+=t.shares;
		    sv_buy+=t.shares*tte;
		    xp_buy=i;
		}
	    }
	    else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
	    {
		if(sum_sell)
		{
		    final boolean fmin=t.price<price_sell_min_re,fmax=t.price>price_sell_max_re;
		    if(fmin||fmax)
		    { // next level
			if(n_levels_trades_sell>=0)
			{
			    this.v_trades_exe_sell[n_levels_trades_sell]=v_sell;
			    this.tte_trades_sell[n_levels_trades_sell]=sv_sell;
			    this.ptr_trades_sell[n_levels_trades_sell]=xp_sell;
			}
			if(++n_levels_trades_sell>=nt)
			{
			    sum_sell=false;
			    if(!sum_buy)
			    {
				break gloop;
			    }
			    continue gloop;
			}
			if(fmin)
			{
			    price_sell_min_re=(t.price>pcurrent_sell?pcurrent_sell:t.price)-OrdersFlow.EPS;
			}
			if(fmax)
			{
			    price_sell_max_re=(t.price<pcurrent_sell?pcurrent_sell:t.price)+OrdersFlow.EPS;
			}
		    }
		    v_sell+=t.shares;
		    sv_sell+=t.shares*tte;
		    xp_sell=i;
		}
	    }
	}
	for(;n_levels_trades_buy<nt;n_levels_trades_buy++)
	{
	    if(n_levels_trades_buy>=0)
	    {
		this.v_trades_exe_buy[n_levels_trades_buy]=v_buy;
		this.tte_trades_buy[n_levels_trades_buy]=sv_buy;
		this.ptr_trades_buy[n_levels_trades_buy]=xp_buy;
	    }
	}
	for(;n_levels_trades_sell<nt;n_levels_trades_sell++)
	{
	    if(n_levels_trades_sell>=0)
	    {
		this.v_trades_exe_sell[n_levels_trades_sell]=v_sell;
		this.tte_trades_sell[n_levels_trades_sell]=sv_sell;
		this.ptr_trades_sell[n_levels_trades_sell]=xp_sell;
	    }
	}
	for(int i=0;i<nt;i++)
	{
	    if(this.v_trades_exe_buy[i]>0)
	    {
		this.tte_trades_buy[i]=(this.tte_trades_buy[i]/this.v_trades_exe_buy[i])
		    *ModelParameters.T_SUMM_SCALE;
	    }
	    else
	    {
		this.tte_trades_buy[i]=-1;
		if(this.ptr_trades_buy[i]!=-1)
		{
		   throw new RuntimeException(
			"Mismatch: "+
			" ptr_trades_buy["+i+"]="+this.ptr_trades_buy[i]+
			" currenttime="+currenttime);
		}
	    }
	    if(this.v_trades_exe_sell[i]>0)
	    {
		this.tte_trades_sell[i]=(this.tte_trades_sell[i]/this.v_trades_exe_sell[i])
		    *ModelParameters.T_SUMM_SCALE;
	    }
	    else
	    {
		this.tte_trades_sell[i]=-1;
		if(this.ptr_trades_sell[i]!=-1)
		{
		   throw new RuntimeException(
			"Mismatch: "+
			" ptr_trades_sell["+i+"]="+this.ptr_trades_sell[i]+
			" currenttime="+currenttime);
		}
	    }
	}

	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // expensive constrains
	    checkCalculateTaus(currenttime,
			       pcurrent_buy,
			       pcurrent_sell,
			       nt,
			       trades_cleared_before,
			       PRINT_DATA_DEBUG);
	} // END expensive constrains

	if(PRINT_DATA_DEBUG)
	{ // expensive constrains
	    	    System.err.println("nl\t\t"+
			       "tte_buy\t"+
			       "v_buy\t"+
			       "ptr_trades_buy\t\t"+
			       "tte_sell\t"+
			       "v_sell\t"+
			       "ptr_trades_sell");
	    for(int i=0;i<nt;i++)
	    {
		System.err.println(String.valueOf(i+1)+"\t\t"+
				   this.tte_trades_buy[i]+"\t"+
				   this.v_trades_exe_buy[i]+"\t"+
				   this.ptr_trades_buy[i]+"\t\t"+
				   this.tte_trades_sell[i]+"\t"+
				   this.v_trades_exe_sell[i]+"\t"+
				   this.ptr_trades_sell[i]);
	    }
	}
    }

    private void checkCalculateTaus(final long currenttime,
				    final double pcurrent_buy,
				    final double pcurrent_sell,
				    final int nt,
				    final long trades_cleared_before,
				    final boolean PRINT_DATA_DEBUG)
    {
	final int tsize=this.trades.getSize();
	final Trade [] tb=this.trades.getBuffer();
	final CalculateTradeVolume tv=
	    new CalculateTradeVolume(this.tmptradesbuf,this.tmptradesbuf);
	final Trade [] tmpb=this.tmptradesbuf.getBuffer();
	for(int i=0;i<nt;i++)
	{
	    if(true)
	    {
		final int ptr_buf=this.ptr_trades_buy[i]>=0?this.ptr_trades_buy[i]:tsize;
		final int n=tsize-ptr_buf;
		System.arraycopy(tb,ptr_buf,tmpb,0,n);
		if(n<tmpb.length)
		{
		    tmpb[n]=null;
		}
		this.tmptradesbuf.setSize(n);
		int nl=i;
		for(;nl>0;)
		{
		    if(this.v_trades_exe_buy[nl]!=this.v_trades_exe_buy[nl-1])
		    {
			break;
		    }
		    nl--;
		}
		checkLevelsNumberInTrades(currenttime,
					  pcurrent_buy,
					  BookItem.BOOK_BUY,
					  tmpb,
					  n,
					  PRINT_DATA_DEBUG,
					  this.ptr_trades_buy[i]<0?0:nl+1);

		tv.calculateTradeVolume(
		    new long [] {currenttime-(ptr_buf<tsize?tb[ptr_buf].time:0)}, // take anything
		    currenttime,
		    0.5,
		    0.92,
		    0.5,
		    pcurrent_buy,
		    pcurrent_sell,
		    0);
		if(this.tte_trades_buy[i]!=tv.time_to_execute_aver_buy[0]||
		   this.v_trades_exe_buy[i]!=tv.v_tte_known_buy_tau[0])
		{
		    throw new RuntimeException(
			"Mismatch:"+
			" tte_trades_buy["+i+"]="+this.tte_trades_buy[i]+
			" tv.time_to_execute_aver_buy="+tv.time_to_execute_aver_buy[0]+
			" v_trades_exe_buy["+i+"]="+this.v_trades_exe_buy[i]+
			" tv.v_tte_known_buy_tau="+tv.v_tte_known_buy_tau[0]+
			" currenttime="+currenttime);
		}
	    }
	    if(true)
	    {
		final int ptr_buf=this.ptr_trades_sell[i]>=0?this.ptr_trades_sell[i]:tsize;
		final int n=tsize-ptr_buf;
		System.arraycopy(tb,ptr_buf,tmpb,0,n);
		if(n<tmpb.length)
		{
		    tmpb[n]=null;
		}
		this.tmptradesbuf.setSize(n);
		int nl=i;
		for(;nl>0;)
		{
		    if(this.v_trades_exe_sell[nl]!=this.v_trades_exe_sell[nl-1])
		    {
			break;
		    }
		    nl--;
		}
		checkLevelsNumberInTrades(currenttime,
					  pcurrent_sell,
					  BookItem.BOOK_SELL,
					  tmpb,
					  n,
					  PRINT_DATA_DEBUG,
					  this.ptr_trades_sell[i]<0?0:nl+1);

		tv.calculateTradeVolume(
		    new long [] {currenttime-(ptr_buf<tsize?tb[ptr_buf].time:0)}, // take anything
		    currenttime,
		    0.5,
		    0.92,
		    0.5,
		    pcurrent_buy,
		    pcurrent_sell,
		    0);
		if(this.tte_trades_sell[i]!=tv.time_to_execute_aver_sell[0]||
		   this.v_trades_exe_sell[i]!=tv.v_tte_known_sell_tau[0])
		{
		    throw new RuntimeException(
			"Mismatch:"+
			" tte_trades_sell["+i+"]="+this.tte_trades_sell[i]+
			" tv.time_to_execute_aver_sell="+tv.time_to_execute_aver_sell[0]+
			" v_trades_exe_sell["+i+"]="+this.v_trades_exe_sell[i]+
			" tv.v_tte_known_sell_tau="+tv.v_tte_known_sell_tau[0]+
			" currenttime="+currenttime);
		}
	    }
	}
    }


    private void checkLevelsNumberInTrades(final long currenttime,
					   final double pcur,
					   final int ttype,
					   final Trade [] tb,
					   final int tsize,
					   final boolean PRINT_DATA_DEBUG,
					   final int levels_number)
    {
	double l_min=Double.POSITIVE_INFINITY,l_max=Double.NEGATIVE_INFINITY;
	int nlev=0,p=tsize;
	for(;p-->0;)
	{
	    final Trade t=tb[p];
	    if(t.origtime>0 && t.buy_sell_indicator==ttype)
	    {
		int nls=0;
		if(t.price<l_min)
		{
		    l_min=(t.price>pcur?pcur:t.price)-OrdersFlow.EPS;
		    nls=1;
		}
		if(t.price>l_max)
		{
		    l_max=(t.price<pcur?pcur:t.price)+OrdersFlow.EPS;
		    nls=1;
		}
		nlev+=nls;
	    }
	}
	if(p!=-1 || nlev!=levels_number)
	{
	    throw new RuntimeException("Interval range mismatch:"+
				       " p="+p+" (should be -1)"+
				       " nlev="+nlev+" (should be "+levels_number+")"+
				       " type="+ttype+
				       " currenttime="+currenttime);
	}
    }


    private static void calculatePriceD(final long currenttime,
					final double pcurrent_buy,
					final double pcurrent_sell,
					final int nt,
					final long trades_cleared_before,
					final boolean PRINT_DATA_DEBUG,
					final Distribution<Trade> trades,
					final long [] tau_buy,
					final long [] tau_sell,
					// output parameters:
					final long [] v_trades_tau_buy,
					final long [] v_trades_tau_sell,
					final double [] price_aver_tau_buy,
					final double [] price_aver_tau_sell,
					final double [] price_max_tau_buy,
					final double [] price_min_tau_sell,

					// temporary variables
					final Distribution<Trade> tmptradesbuf)
    {
	final int tsize=trades.getSize();
	final Trade [] tb=trades.getBuffer();
	for(int cur_pos=0;cur_pos<nt;cur_pos++)
	{
	    final long t0_buy=currenttime-tau_buy[cur_pos],
		t0_sell=currenttime-tau_sell[cur_pos];
	    double aver_p_buy=0,aver_p_sell=0,max_buy=Double.NEGATIVE_INFINITY,min_sell=Double.POSITIVE_INFINITY;
	    long v_buy=0,v_sell=0;
	    boolean sum_buy=true,sum_sell=true;
	    gloop:for(int i=tsize;i-->0;)
	    {
		final Trade t=tb[i];
		final int s=t.shares;
		final double p=t.price;
		if(t.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    if(sum_buy)
		    {
			if(t.time>=t0_buy)
			{
			    v_buy+=s;
			    aver_p_buy+=p*s;
			    if(p>=max_buy)
			    {
				max_buy=p;
			    }
			}
			else
			{
			    sum_buy=false;
			    if(!sum_sell)
			    {
				break gloop;
			    }
			    continue gloop;
			}
		    }
		}
		else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    if(sum_sell)
		    {
			if(t.time>=t0_sell)
			{
			    v_sell+=s;
			    aver_p_sell+=p*s;
			    if(p<=min_sell)
			    {
				min_sell=p;
			    }
			}
			else
			{
			    sum_sell=false;
			    if(!sum_buy)
			    {
				break gloop;
			    }
			    continue gloop;
			}
		    }
		}
	    }

	    price_aver_tau_buy[cur_pos]=v_buy>0?aver_p_buy/v_buy:Double.NaN;
	    v_trades_tau_buy[cur_pos]=v_buy;
	    price_aver_tau_sell[cur_pos]=v_sell>0?aver_p_sell/v_sell:Double.NaN;
	    v_trades_tau_sell[cur_pos]=v_sell;
	    price_max_tau_buy[cur_pos]=Double.isInfinite(max_buy)?Double.NaN:max_buy;
	    price_min_tau_sell[cur_pos]=Double.isInfinite(min_sell)?Double.NaN:min_sell;
	}


	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // expensive constrains
	    final CalculateLiquidityDeficit2 ld=
		new CalculateLiquidityDeficit2(null,
					       null,
					       trades,
					       null,
					       tmptradesbuf);
	    final long [] tmpl=new long [nt],v=new long [nt];
	    final double [] tmpd=new double[nt],pa=new double [nt];
	    ld.calculateFlow(nt,
			     currenttime,
			     PRINT_DATA_DEBUG,
			     BookItem.BOOK_BUY,
			     BookItem.BOOK_SELL,
			     tau_buy,
			     tmpl,
			     v,
			     tmpd,
			     pa);
	    for(int i=0;i<nt;i++)
	    {
		if(v_trades_tau_buy[i]!=v[i]||
		   Double.isNaN(price_aver_tau_buy[i])!=Double.isNaN(pa[i])||
		   (!Double.isNaN(price_aver_tau_buy[i]) &&
		    price_aver_tau_buy[i]!=pa[i]))
		{
		    throw new RuntimeException(
			"Mismatch: "+
			" v_trades_tau_buy["+i+"]="+v_trades_tau_buy[i]+
			" v["+i+"]="+v[i]+
			" price_aver_tau_buy["+i+"]="+price_aver_tau_buy[i]+
			" pa["+i+"]="+pa[i]);
		}
	    }
	    ld.calculateFlow(nt,
			     currenttime,
			     PRINT_DATA_DEBUG,
			     BookItem.BOOK_SELL,
			     BookItem.BOOK_BUY,
			     tau_sell,
			     tmpl,
			     v,
			     tmpd,
			     pa);
	    for(int i=0;i<nt;i++)
	    {
		if(v_trades_tau_sell[i]!=v[i]||
		   Double.isNaN(price_aver_tau_sell[i])!=Double.isNaN(pa[i])||
		   (!Double.isNaN(price_aver_tau_sell[i]) &&
		    price_aver_tau_sell[i]!=pa[i]))
		{
		    throw new RuntimeException(
			"Mismatch: "+
			" v_trades_tau_sell["+i+"]="+v_trades_tau_sell[i]+
			" v["+i+"]="+v[i]+
			" price_aver_tau_sell["+i+"]="+price_aver_tau_sell[i]+
			" pa["+i+"]="+pa[i]);
		}
	    }
	} // END expensive constrains
    }


    static void calculateBookInvertionPrice(final long currenttime,
					    final int pos_to_start,
					    final Distribution<BookItem> buf,
					    final boolean flag_lower,
					    final long tau_level,
					    final boolean flag_invertion,
					    final boolean PRINT_DATA_DEBUG,
					    final int cur_pos_save,
					    // output to save to
					    final double [] price_inv_book,
					    final int [] n_levs_inv_book)
    {
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println();
	    System.err.println("==calculateBookInvertionPrice"+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNORDERS=\t"+buf.getSize()+
			       "\tTAU LEVEL=\t"+tau_level+
			       "\tflag_invertion=\t"+flag_invertion+
			       "\tpos_to_start=\t"+pos_to_start+
			       "\t flag_lower=\t"+flag_lower);
	}
	final int tsize=buf.getSize();
	final BookItem [] tb=buf.getBuffer();
	double prevp=Double.NaN,price_inv=Double.NaN;
	int ptr_inv_ind=-1,n_levs=0,n_levs_inv=0;
	long v=0,sv=0,v_inv=0,svt_inv=-1;
	final long tau_cut=tau_level<0?(flag_invertion?Long.MAX_VALUE:Long.MIN_VALUE):tau_level;
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("TAU_CUT\t"+tau_cut+"\tTAU_CUT_SECONDS\t"+(tau_cut/TraderInterface.TIME_SCALE));
	    System.err.println("num\t"+
			       "time\t"+
			       "shares\t"+
			       "buy_sell_indicator\t"+
			       "price\t"+
			       "orderreferencenumber\t");
	}
        gloop:if(true)
	{
	    for(int i=pos_to_start;i<tsize;i++)
	    {
		final BookItem t=tb[i];
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.shares+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.price+"\t"+
				       t.orderreferencenumber+"\t");
		}
		if(!(t.price==prevp))
		{
		    if(n_levs>0)
		    {
			// one price level reached, can calculate
			final long svt=v>0?(sv/v)*ModelParameters.T_SUMM_SCALE:-1;
			if(false&&PRINT_DATA_DEBUG)
			{
			    System.err.println("TAU_CUT\t"+tau_cut+"\tsvt\t"+svt+
					       "\tflag_invertion\t"+flag_invertion);
			}
			if(flag_invertion?svt>=tau_cut:svt<=tau_cut)
			{
			    n_levs_inv=n_levs;
			    ptr_inv_ind=i-1;
			    price_inv=prevp;
			    v_inv=v;
			    svt_inv=svt;
			}
			else
			{
			    break gloop;
			}
		    }
		    n_levs++;
		    prevp=t.price;
		}
		v+=t.shares;
		sv+=t.shares*
		    ((currenttime-t.time)/ModelParameters.T_SUMM_SCALE);
	    }
	    final long svt=v>0?(sv/v)*ModelParameters.T_SUMM_SCALE:-1;
	    if(flag_invertion?svt>=tau_cut:svt<=tau_cut)
	    {
		n_levs_inv=n_levs;
		ptr_inv_ind=tsize-1;
		price_inv=prevp;
		v_inv=v;
		svt_inv=svt;
	    }
	}
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("n_levs_inv\t"+n_levs_inv+"\n"+
			       "ptr_inv_ind\t"+ptr_inv_ind+"\n"+
			       "price_inv\t"+price_inv+"\n"+
			       "v_inv\t"+v_inv+"\n"+
			       "svt_inv\t"+svt_inv+"\n"+
			       "\n"+
			       "n_levs\t"+n_levs+"\n"+
			       "price\t"+prevp+"\n"+
			       "v\t"+v);
	}

	if(n_levs_inv>0 && price_inv!=tb[ptr_inv_ind].price)
	{
	    throw new RuntimeException("Inversion price mismatch:"+
				       " n_levs_inv="+n_levs_inv+
				       " price_inv="+price_inv+
				       " tb["+ptr_inv_ind+"].price="+tb[ptr_inv_ind].price+
				       " currenttime="+currenttime);
	}


	// now we have: price_inv -inversion price, n_levs_inv - the number of inverted levels.
	price_inv_book[cur_pos_save]=price_inv;
	n_levs_inv_book[cur_pos_save]=n_levs_inv;

	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // expensive constrains
	    final int nt=price_inv_book.length+n_levs+10;
	    final int [] ptr_book=new int [nt];
	    final long []
		vb_acc=new long [nt],
		svb_acc=new long [nt];
	    calculateBookLevelsTau(currenttime,
				   nt,
				   false,
				   pos_to_start,
				   tb,
				   tsize,
				   flag_lower,
				   ptr_book,
				   vb_acc,
				   svb_acc);
	    // accumulate
	    for(int i=1;i<nt;i++)
	    {
		final int i1=i-1;
		vb_acc[i]+=vb_acc[i1];
		svb_acc[i]+=svb_acc[i1];
	    }

	    int cn_levs_inv=0;
	    for(int i=0;i<n_levs;i++)
	    {
		svb_acc[i]=vb_acc[i]>0?(svb_acc[i]/vb_acc[i])*ModelParameters.T_SUMM_SCALE:-1;
		if(flag_invertion?svb_acc[i]>=tau_cut:svb_acc[i]<=tau_cut)
		{
		    cn_levs_inv=i+1;
		}
		else
		{
		    break;
		}
	    }
	    if(n_levs_inv!=cn_levs_inv)
	    {
		throw new RuntimeException(
		    "Inversion index mismatch:"+
		    " n_levs_inv="+n_levs_inv+
		    " cn_levs_inv="+cn_levs_inv+
		    " pos_to_start="+pos_to_start+
		    " N_ORDERS="+tsize+
		    " flag_lower="+flag_lower+
		    " currenttime="+currenttime);
	    }
	    if(n_levs_inv>0 &&
		(vb_acc[n_levs_inv-1]!=v_inv||
		 svb_acc[n_levs_inv-1]!=svt_inv||
		 Double.isNaN(price_inv)!=Double.isNaN(tb[ptr_book[n_levs_inv-1]].price)||
		 (!Double.isNaN(price_inv) && price_inv!=tb[ptr_book[n_levs_inv-1]].price)))
	    {
		throw new RuntimeException(
		    "End point mismatch:"+
		    " v_inv="+v_inv+
		    " vb_acc["+(n_levs_inv-1)+"]="+vb_acc[n_levs_inv-1]+
		    " svt_inv="+svt_inv+
		    " svb_acc["+(n_levs_inv-1)+"]="+svb_acc[n_levs_inv-1]+
		    " price_inv="+price_inv+
		    " check_price_inv="+tb[ptr_book[n_levs_inv-1]].price+
		    " currenttime="+currenttime);
	    }
	}
    }


    static void calculateBookLevelsTau(final long currenttime,
				       final int nt,
				       final boolean PRINT_DATA_DEBUG,
				       final int ptr_to_start,
				       final BookItem [] tb,
				       final int tsize,
				       final boolean flag_lower,
				       // output parameters:
				       final int [] ptr_book,
				       final long [] vb_levels,
				       final long [] svb_not_normalized_levels)
    {
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("num\t"+
			       "time\t"+
			       "shares\t"+
			       "buy_sell_indicator\t"+
			       "price\t"+
			       "orderreferencenumber\t");
	}

	long v=0,sv=0;
	int pi=-1,n_levels=-1;
	if(flag_lower)
	{
	    double lev=Double.POSITIVE_INFINITY;
	    for(int i=ptr_to_start;i<tsize;i++)
	    {
		final BookItem t=tb[i];
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.shares+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.price+"\t"+
				       t.orderreferencenumber+"\t");
		}
		final long tau=(currenttime-t.time)/ModelParameters.T_SUMM_SCALE;
		if(t.price<lev)
		{ // next level
		    if(n_levels>=0)
		    {
			vb_levels[n_levels]=v;
			svb_not_normalized_levels[n_levels]=sv;
			ptr_book[n_levels]=pi;
			pi=-1;
		    }
		    v=sv=0;
		    if(++n_levels>=nt)
		    {
			break;
		    }
		    lev=t.price-OrdersFlow.EPS;
		}
		v+=t.shares;
		sv+=t.shares*tau;
		pi=i;
	    }
	}
	else
	{
	    double lev=Double.NEGATIVE_INFINITY;
	    for(int i=ptr_to_start;i<tsize;i++)
	    {
		final BookItem t=tb[i];
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.shares+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.price+"\t"+
				       t.orderreferencenumber+"\t");
		}
		final long tau=(currenttime-t.time)/ModelParameters.T_SUMM_SCALE;
		if(t.price>lev)
		{ // next level
		    if(n_levels>=0)
		    {
			vb_levels[n_levels]=v;
			svb_not_normalized_levels[n_levels]=sv;
			ptr_book[n_levels]=pi;
			pi=-1;
		    }
		    v=sv=0;
		    if(++n_levels>=nt)
		    {
			break;
		    }
		    lev=t.price+OrdersFlow.EPS;
		}
		v+=t.shares;
		sv+=t.shares*tau;
		pi=i;
	    }
	}
	if(v>0)
	{
	    vb_levels[n_levels]=v;
	    svb_not_normalized_levels[n_levels]=sv;
	    ptr_book[n_levels]=pi;
	    v=sv=0;
	    pi=-1;
	    n_levels++;
	}
	for(;n_levels<nt;n_levels++)
	{
	    if(n_levels>=0)
	    {
		vb_levels[n_levels]=svb_not_normalized_levels[n_levels]=0;
		ptr_book[n_levels]=-1;
	    }
	}
    }




    private static void calculateBookTau(final long currenttime,
					 final double pcurrent_buy,
					 final double pcurrent_sell,
					 final int nt,
					 final boolean PRINT_DATA_DEBUG,
					 final BookItem [] tb,
					 final int tsize,
					 final boolean flag_lower,
					 // output parameters:
					 final int [] ptr_book,
					 final long [] vb,
					 final long [] tau_lev,
					 final long [] vb_acc,
					 final long [] tau_acc_lev)
    {

	calculateBookLevelsTau(currenttime,
			       nt,
			       PRINT_DATA_DEBUG,
			       0,
			       tb,
			       tsize,
			       flag_lower,
			       ptr_book,
			       vb,
			       tau_lev);
	// make accumulation & normalization
	for(int i=0;i<nt;i++)
	{
	    if(i>0)
	    {
		final int i1=i-1;
		vb_acc[i]=vb_acc[i1]+vb[i];
		tau_acc_lev[i]=tau_acc_lev[i1]+tau_lev[i];
	    }
	    else
	    {
		vb_acc[i]=vb[i];
		tau_acc_lev[i]=tau_lev[i];
	    }
	}
	for(int i=0;i<nt;i++)
	{
	    final long vl=vb[i];
	    if(vl>0)
	    {
		tau_lev[i]=(tau_lev[i]/vl)*ModelParameters.T_SUMM_SCALE;
	    }
	    else
	    {
		tau_lev[i]=-1;
	    }
	    final long vla=vb_acc[i];
	    if(vla>0)
	    {
		tau_acc_lev[i]=(tau_acc_lev[i]/vla)*ModelParameters.T_SUMM_SCALE;
	    }
	    else
	    {
		tau_acc_lev[i]=-1;
	    }
	}
	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // expensive constrains
	    for(int i=0;i<nt;i++)
	    {
		checkBookLevTaus(currenttime,
				 i+1,
				 PRINT_DATA_DEBUG,
				 tb,
				 tsize,
				 ptr_book,
				 vb,
				 tau_lev,
				 vb_acc,
				 tau_acc_lev);
	    }
	}
    }



    static void accumulateBookTauBSplitFromLevels(final long currenttime,
						  final int nt,
						  final boolean PRINT_DATA_DEBUG,
						  final Distribution<BookItem> book,
						  final boolean flag_lower,

						  final int [] ptr_book,

						  // input & output for accumulation
						  final long [] vb_level,
						  final long [] svb_level,

						  // output parameters:
						  final double [] price_level,
						  final double [] price_split,

						  final int [] levels_number)
    {
	final BookItem [] tb=book.getBuffer();
	final int tsize=book.getSize();

	long v=0,sv=0;
	for(int isplit=0;isplit<nt;isplit++)
	{
	    v+=vb_level[isplit];
	    sv+=svb_level[isplit];
	    final long tau=v>0?(sv/v)*ModelParameters.T_SUMM_SCALE:-1;
	    vb_level[isplit]=v;
	    svb_level[isplit]=tau;

	    final int pb=ptr_book[isplit];
	    if(pb<0)
	    {
		price_level[isplit]=Double.NaN;
		price_split[isplit]=Double.NaN;
	    }
	    else
	    {
		price_level[isplit]=pb+1<tsize?tb[pb+1].price:Double.NaN;
		calculateBookInvertionPrice(currenttime,
					    pb+1,
					    book,
					    flag_lower,
					    tau,
					    false,
					    PRINT_DATA_DEBUG,
					    isplit,
					    price_split,
					    levels_number);
	    }
	}
    }




   private static void calculateBookTauBSplit(final long currenttime,
					      final int nt,
					      final boolean PRINT_DATA_DEBUG,
					      final Distribution<BookItem> book,
					      final boolean flag_lower,

					      // output parameters:
					      final int [] ptr_book,
					      final long [] vb_level,
					      final long [] svb_level,
					      final double [] price_level,
					      final double [] price_split,
					      // temporary arrays
					      final int [] tmpi)
    {
	final BookItem [] tb=book.getBuffer();
	final int tsize=book.getSize();
	calculateBookLevelsTau(currenttime,
			       nt,
			       PRINT_DATA_DEBUG,
			       0,
			       tb,
			       tsize,
			       flag_lower,
			       ptr_book,
			       vb_level,
			       svb_level
			       );

	accumulateBookTauBSplitFromLevels(currenttime,
					  nt,
					  PRINT_DATA_DEBUG,
					  book,
					  flag_lower,
					  ptr_book,
					  vb_level,
					  svb_level,

					  price_level,
					  price_split,
					  tmpi);


	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // expensive constrains

	    int n_levs=0;
	    double pp=Double.NaN;
	    for(int i=0;i<tsize;i++)
	    {
		final BookItem t=tb[i];
		if(t.price!=pp)
		{
		    pp=t.price;
		    n_levs++;
		}
	    }
	    final int ntl=nt+n_levs+10;
	    final int [] xptr_book=new int [ntl];
	    final long []
		vb_acc=new long [ntl],
		svb_acc=new long [ntl];
	    calculateBookLevelsTau(currenttime,
				   ntl,
				   false,
				   0,
				   tb,
				   tsize,
				   flag_lower,
				   xptr_book,
				   vb_acc,
				   svb_acc);
	    long vx=0,svx=0;
	    for(int i=0;i<nt;i++)
	    {
		vx+=vb_acc[i];
		svx+=svb_acc[i];
		final long tau=vx>0?(svx/vx)*ModelParameters.T_SUMM_SCALE:-1;
		if(xptr_book[i]!=ptr_book[i]||
		   vx!=vb_level[i]||
		   tau!=svb_level[i])
		{
		    throw new RuntimeException("Pointers mismatch:"+
					       " xptr_book["+i+"]="+xptr_book[i]+
					       " ptr_book["+i+"]="+ptr_book[i]+
					       " vx="+vx+
					       " vb_level["+i+"]="+vb_level[i]+
					       " tau="+tau+
					       " svb_level["+i+"]="+svb_level[i]+
					       " currenttime="+currenttime);
		}
		final double pl;
		if(xptr_book[i]<0)
		{
		    pl=Double.NaN;
		}
		else
		{
		    pl=xptr_book[i]+1<tsize?tb[xptr_book[i]+1].price:Double.NaN;
		}
		if(Double.isNaN(pl)!=Double.isNaN(price_level[i])||
		   (!Double.isNaN(pl) && pl!=price_level[i]))
		{
		    throw new RuntimeException("Price level mismatch:"+
					       " pl="+pl+
					       " price_level["+i+"]="+price_level[i]+
					       " xptr_book["+i+"]+1="+(xptr_book[i]+1)+
					       " tsize="+tsize+
					       " currenttime="+currenttime);
		}
		long va=0,sva=0;
		double psplit=Double.NaN;
		for(int j=i+1;j<ntl;j++)
		{
		  va+=vb_acc[j];
		  sva+=svb_acc[j];
		  final long taux=va>0?(sva/va)*ModelParameters.T_SUMM_SCALE:-1;
		  if(xptr_book[j]>=0 && taux<=tau)
		  {
		      psplit=tb[xptr_book[j]].price;
		  }
		  else
		  {
		      break;
		  }
		}
		if(Double.isNaN(psplit)!=Double.isNaN(price_split[i])||
		   (!Double.isNaN(psplit) && psplit!=price_split[i]))
		{
		    throw new RuntimeException("Price split mismatch:"+
					       " psplit="+psplit+
					       " price_split["+i+"]="+price_split[i]+
					       " currenttime="+currenttime);
		}
	    }
	}
    }


    private static double getSpirmanRankCorr(final long [] taus, final int nt)
    {
	final int [] tmpr=com.polytechnik.outils.TMPQSortAlgorithm.sortArrayIndexes(
	    nt,new com.polytechnik.utils.IntPTRCompare(){
		    public int compare(final int p1,final int p2)
		    {
			return Long.compare(taus[p1],taus[p2]);
		    }
		});

	long ds=0;
	for(int i=0;i<nt;i++)
	{
	    final int d=tmpr[i]-i;
	    ds+=d*d;
	}
	final long lnt=nt;
	return 1.0-((double)(6*ds))/(nt*((double)(lnt*lnt-1)));
    }

    public static void main(String [] args)
    {
     for(int nt=8;nt<17000;nt+=20)
     {
	System.err.println("=============NT="+nt);
	final long [] taus=new long [nt];
	for(int i=0;i<nt;i++)
	{
	    taus[i]=10000+3*i;
	}
	System.err.println("-R="+getSpirmanRankCorr(taus,nt));
	for(int i=0;i<nt;i++)
	{
	    taus[i]=10000-3*i;
	}
	System.err.println("+R="+getSpirmanRankCorr(taus,nt));

	for(int i=0;i<nt;i++)
	{
	    taus[i]=(long)(10000-3*Math.sin(i));
	}
	System.err.println("sR="+getSpirmanRankCorr(taus,nt));
     }
    }



    private static long []  getvsv(final long currenttime,
				   final int nlev,
				   final boolean PRINT_DATA_DEBUG,
				   final BookItem [] tb,
				   final int tsize)
    {
	double prevp=Double.POSITIVE_INFINITY;
	long v=0,sv=0;
	int ie=-1;
	for(int i=0,cur_pos=-1;i<tsize;i++)
	{
	    final BookItem t=tb[i];
	    final long tau=(currenttime-t.time)/ModelParameters.T_SUMM_SCALE;
	    if(t.price!=prevp)
	    {
		if(++cur_pos>=nlev)
		{
		    break;
		}
		prevp=t.price;
	    }
	    v+=t.shares;
	    sv+=t.shares*tau;
	    ie=i;
	}
	return new long []{v,sv,ie};
    }



    private static void checkBookLevTaus(final long currenttime,
					 final int nlev,
					 final boolean PRINT_DATA_DEBUG,
					 final BookItem [] tb,
					 final int tsize,
					 final int [] ptr_book,
					 final long [] vb,
					 final long [] svb,
					 final long [] vb_acc,
					 final long [] svb_acc)
    {
	final long []
	    vsv2=getvsv(currenttime,
			nlev,
			PRINT_DATA_DEBUG,
			tb,
			tsize),
	    vsv1=getvsv(currenttime,
			nlev-1,
			PRINT_DATA_DEBUG,
			tb,
			tsize);
	final long v=vsv2[0]-vsv1[0],svx=vsv2[1]-vsv1[1],v_acc=vsv2[0],svx_acc=vsv2[1],sv,sv_acc;
	final int ie2=v>0?(int)vsv2[2]:-1;
	if(v>0)
	{
	    sv=(svx/v)*ModelParameters.T_SUMM_SCALE;
	}
	else
	{
	    sv=-1;
	}
	if(v_acc>0)
	{
	    sv_acc=(svx_acc/v_acc)*ModelParameters.T_SUMM_SCALE;
	}
	else
	{
	    sv_acc=-1;
	}
	final int cur_pos=nlev-1;
	if(v!=vb[cur_pos]||sv!=svb[cur_pos]||
	   v_acc!=vb_acc[cur_pos]||sv_acc!=svb_acc[cur_pos]||
	   ie2!=ptr_book[cur_pos])
	{
	    throw new RuntimeException(
		" Mismatch:"+
		" v="+v+
		" vb["+cur_pos+"]="+vb[cur_pos]+
		" sv="+sv+
		" svb["+cur_pos+"]="+svb[cur_pos]+
		" v_acc="+v_acc+
		" vb_acc["+cur_pos+"]="+vb_acc[cur_pos]+
		" sv_acc="+sv_acc+
		" svb_acc["+cur_pos+"]="+svb_acc[cur_pos]+
		" ie="+ie2+
		" ptr_book["+cur_pos+"]="+ptr_book[cur_pos]+
		" currenttime="+currenttime);
	}
    }



    static void calculateBookLimits(final long currenttime,
				    final Distribution<BookItem> book,
				    final Distribution<Trade> trades,
				    final boolean flag_lower,
				    final long tau_level,
				    final double price_level,
				    final boolean PRINT_DATA_DEBUG,
				    final int cur_pos_save,
				    // output to save to
				    final long [] tau_cut,
				    final double [] price_cut,
				    final long [] vol_book_cut,
				    final long [] tau_book_cut,
				    final long [] tau_trades_cut,

				    final double [] price_d_book,
				    final double [] price_d_trades,
				    // buffers for constrains
				    final Distribution<Trade> tmptradesbuf)
    {
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println();
	    System.err.println("==calculateBookLimits"+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNORDERS=\t"+book.getSize()+
			       "\tNTRADES=\t"+trades.getSize()+
			       "\tTAU LEVEL=\t"+tau_level+
			       "\t flag_lower=\t"+flag_lower);
	}
	final int tsize=trades.getSize(),bsize=book.getSize();
	final Trade [] tb=trades.getBuffer();
	final BookItem [] bb=book.getBuffer();
	final long tcut=currenttime-tau_level;
	final int ttype=flag_lower?BookItem.BOOK_BUY:BookItem.BOOK_SELL;
	final boolean SKIP_TTYPE=true;


	long v=0,svb=0,svt=0,vt=0;
	int no_book=0,ptr_trades=-1,n_levs_trades=0,ptr_price_level=-1;
	double price_lev=Double.NaN;
	for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time<tcut)
	    {
		break;
	    }
	    else if(SKIP_TTYPE || t.buy_sell_indicator==ttype)
	    {
		final double p=t.price;
		if(flag_lower?!(p>=price_lev):!(p<=price_lev))
		{
		    if(flag_lower?p<price_level:p>price_level)
		    {
			ptr_price_level=i;
			break;
		    }
		    price_lev=p+(flag_lower?-OrdersFlow.EPS:OrdersFlow.EPS);
		    final long treachs=currenttime-t.time,treach=treachs/ModelParameters.T_SUMM_SCALE;
		    n_levs_trades++;
		    ptr_trades=i;
		    final double pbook_lev=p+(flag_lower?OrdersFlow.EPS:-OrdersFlow.EPS);
		    for(;no_book<bsize;)
		    {
			final BookItem b=bb[no_book];
			if(flag_lower?!(b.price>=pbook_lev):!(b.price<=pbook_lev))
			{
			    break;
			}
			final int s=b.shares;
			final long taus=currenttime-b.time,tau=taus/ModelParameters.T_SUMM_SCALE;
			v+=s;
			svb+=s*tau;
			svt+=s*treach;
			if(taus>treachs)
			{
			    System.err.println(
				"CalculateLiquidityDeficit3:calculateBookLimits out of interval range order"+
				" dtau="+(taus-treachs)+
				" time="+BookItem.getTimeDiag(currenttime)+
				" currenttime="+currenttime);
			}
			no_book++;
		    }
		}
		vt+=t.shares;
	    }
	}

	final double price_e;
	if(ptr_trades<0)
	{
	    price_e=price_cut[cur_pos_save]=Double.NaN;
	    tau_cut[cur_pos_save]=-1;
	}
	else
	{
	    price_e=price_cut[cur_pos_save]=tb[ptr_trades].price;
	    tau_cut[cur_pos_save]=currenttime-tb[ptr_trades].time;
	}
	vol_book_cut[cur_pos_save]=v;
	final long t_book,t_trades;
	if(v>0)
	{
	    t_book=tau_book_cut[cur_pos_save]=(svb/v)*ModelParameters.T_SUMM_SCALE;
	    t_trades=tau_trades_cut[cur_pos_save]=(svt/v)*ModelParameters.T_SUMM_SCALE;
	}
	else
	{
	    t_book=tau_book_cut[cur_pos_save]=t_trades=tau_trades_cut[cur_pos_save]=-1;
	}



      final double [] px=new double []{Double.NaN,Double.NaN};
      if(true)
      {
	final long [] dl;
	final int pp_book,pp_trades;
	if(t_book<=t_trades)
	{
	    dl=new long []{t_book,t_trades};
	    pp_book=0;
	    pp_trades=1;
	}
	else
	{
	    System.err.println(
		"CalculateLiquidityDeficit3:calculateBookLimits incorrect delays:"+
		" tau_book="+t_book+
		" tau_trades="+t_trades+
		" time="+BookItem.getTimeDiag(currenttime)+
		" currenttime="+currenttime);
	    dl=new long []{t_trades,t_book};
	    pp_book=1;
	    pp_trades=0;
	}

	final int nn=dl.length;
	long tbefore=Long.MAX_VALUE;
	double m=Double.NaN;
	int cur_pos=-1;
        gloop:for(int i=tsize;i-- >0;)
	{
	    final Trade t=tb[i];
	    if(SKIP_TTYPE || t.buy_sell_indicator==ttype)
	    {
		for(;!(t.time>=tbefore);)
		{
		    if(++cur_pos>=nn)
		    {
			break gloop;
		    }
		    tbefore=currenttime-dl[cur_pos];
		    if(cur_pos>0)
		    {
			px[cur_pos]=px[cur_pos-1];
		    }
		}
		if(flag_lower?!(t.price>=m):!(t.price<=m))
		{
		    m=px[cur_pos]=t.price;
		}
	    }
	}
	for(cur_pos++;cur_pos<nn;cur_pos++)
	{
	    if(cur_pos>0)
	    {
		px[cur_pos]=px[cur_pos-1];
	    }
	}

	price_d_book[cur_pos_save]=px[pp_book];
	price_d_trades[cur_pos_save]=px[pp_trades];
      }


	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // expensive constrains
	    final CalculateTradeVolume tv=
		new CalculateTradeVolume(tmptradesbuf,tmptradesbuf);
	    final Trade [] tmpb=tmptradesbuf.getBuffer();
	    System.arraycopy(tb,0,tmpb,0,tsize);
	    int pt=-1;
	    for(int i=tsize;i-->0;)
	    {
		if(SKIP_TTYPE||tmpb[i].buy_sell_indicator==ttype)
		{
		    if(flag_lower?tmpb[i].price<price_level:tmpb[i].price>price_level)
		    {
			pt=i;
			break;
		    }
		}
	    }
	    if(ptr_price_level>=0)
	    {
		if(pt!=ptr_price_level)
		{
		    throw new RuntimeException("Price cut mismatch:"+
					       " ptr_trades="+ptr_trades+
					       " pt="+pt+
					       " ptr_price_level="+ptr_price_level+
					       " currenttime="+currenttime);
		}
	    }
	    else
	    {
		if(ptr_trades>=0 && ptr_trades<=pt)
		{
		    throw new RuntimeException("Price cut mismatch:"+
					       " ptr_trades="+ptr_trades+
					       " pt="+pt+
					       " ptr_price_level="+ptr_price_level+
					       " currenttime="+currenttime);
		}
	    }
	    final int nnv=tsize-(pt+1);
	    System.arraycopy(tmpb,pt+1,tmpb,0,nnv);
	    if(nnv<tmpb.length)
	    {
		tmpb[nnv]=null;
	    }
	    tmptradesbuf.setSize(nnv);
	    tv.calculateTradeVolume(new long [] {tau_level},
				    currenttime,
				    0.5,
				    0.92,
				    0.5,
				    Double.NaN,
				    Double.NaN,
				    0);
	    final double price_e_check;
	    final long vt_check;
	    if(SKIP_TTYPE)
	    {
		price_e_check=flag_lower?tv.tot_minprice[0]:tv.tot_maxprice[0];
		vt_check=tv.v_tot_alltrades[0];
	    }
	    else
	    {
		price_e_check=flag_lower?tv.cc_minprice_buy[0]:tv.cc_maxprice_sell[0];
		vt_check=flag_lower?tv.v_alltrades_buy_tau[0]:tv.v_alltrades_sell_tau[0];
	    }
	    if(vt!=vt_check||
	       Double.isNaN(price_e)!=Double.isNaN(price_e_check)||
	       (!Double.isNaN(price_e) && price_e!=price_e_check))
	    {
		throw new RuntimeException("Trades price cut mismatch:"+
					   " vt="+vt+
					   " vt_check="+vt_check+
					   " price_e="+price_e+
					   " price_e_check="+price_e_check+
					   " flag_lower="+flag_lower+
					   " currenttime="+currenttime);
	    }

	    final CalculateBookVolumePriceFixedThreshold bv=
		new CalculateBookVolumePriceFixedThreshold(flag_lower,book);
	    bv.calculateThresholds(currenttime,
				   flag_lower && bsize>0?bb[0].price:Double.NaN,
				   !flag_lower && bsize>0?bb[0].price:Double.NaN,
				   new double [] {price_e+(flag_lower?OrdersFlow.EPS:-OrdersFlow.EPS)});
	    if(v!=bv.vol_threshold_vcalc[0]||
	       t_book!=bv.tau_threshold_vcalc[0])
	    {
		throw new RuntimeException("Book volume/tau mismatch:"+
					   " v_book="+v+
					   " bv.vol_threshold_vcalc="+bv.vol_threshold_vcalc[0]+
					   " tau_book="+t_book+
					   " bv.tau_threshold_vcalc="+bv.tau_threshold_vcalc[0]+
					   " flag_lower="+flag_lower+
					   " currenttime="+currenttime);
	    }
	    System.arraycopy(tb,0,tmpb,0,tsize);
	    if(tsize<tmpb.length)
	    {
		tmpb[tsize]=null;
	    }
	    tmptradesbuf.setSize(tsize);

	    final long [] delays;
	    final int pp_book,pp_trades;
	    if(t_book<=t_trades)
	    {
		delays=new long []{t_book,t_trades};
		pp_book=0;
		pp_trades=1;
	    }
	    else
	    {
		delays=new long []{t_trades,t_book};
		pp_book=1;
		pp_trades=0;
	    }
	    tv.calculateTradeVolume(delays,
				    currenttime,
				    0.5,
				    0.92,
				    0.5,
				    Double.NaN,
				    Double.NaN,
				    0);
	    final double px_book,px_trades;
	    if(flag_lower)
	    {
		if(SKIP_TTYPE)
		{
		    px_book=tv.tot_minprice[pp_book];
		    px_trades=tv.tot_minprice[pp_trades];
		}
		else
		{
		    px_book=tv.cc_minprice_buy[pp_book];
		    px_trades=tv.cc_minprice_buy[pp_trades];
		}
	    }
	    else
	    {
		if(SKIP_TTYPE)
		{
		    px_book=tv.tot_maxprice[pp_book];
		    px_trades=tv.tot_maxprice[pp_trades];
		}
		else
		{
		    px_book=tv.cc_maxprice_sell[pp_book];
		    px_trades=tv.cc_maxprice_sell[pp_trades];
		}
	    }
	    final double porig_book=px[pp_book],porig_trades=px[pp_trades];
	    if(Double.isNaN(px_book)!=Double.isNaN(porig_book)||
	       (!Double.isNaN(px_book) && px_book!=porig_book)||
	       Double.isNaN(px_trades)!=Double.isNaN(porig_trades)||
	       (!Double.isNaN(px_trades) && px_trades!=porig_trades))
	    {
		throw new RuntimeException("Price levels mismatch:"+
					   " porig_book="+porig_book+
					   " px_book="+px_book+
					   " t_book="+t_book+
					   " porig_trades="+porig_trades+
					   " px_trades="+px_trades+
					   " t_trades="+t_trades+
					   " flag_lower="+flag_lower+
					   " currenttime="+currenttime);
	    }
	}
    }

}
