package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit ver 4.
 *  @version $Id: CalculateLiquidityDeficit4.java,v 1.118 2022/07/18 08:17:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit4 {

    private final Distribution<BookItem> buy,sell;
    private final Distribution<Trade> trades,tmptradesbuf;

    CalculateLiquidityDeficit4(final Distribution<BookItem> buy,
			       final Distribution<BookItem> sell,
			       final Distribution<Trade> trades,
			       final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
	this.tmptradesbuf=tmptradesbuf;
    }

    double lev_E_buy=Double.NaN,lev_E_sell=Double.NaN,
	lev_iE_buy=Double.NaN,
	lev_iE_sell=Double.NaN,
	lev_iE2_buy=Double.NaN,
	lev_iE2_sell=Double.NaN,
	lev_iE3_buy=Double.NaN,
	lev_iE3_sell=Double.NaN;
    double [] lev_Econv_buy=null,lev_Econv_sell=null;
    long [] v_book_buy=null,tau_book_buy=null,v_book_sell=null,tau_book_sell=null;
    double []
	inversion_length_buy=null,
        inversion_length_sell=null,
	rinversion_length_buy=null,
        rinversion_length_sell=null,
	sinversion_length_buy=null,
        sinversion_length_sell=null,
	price_lev_cut_buy=null,
	price_lev_cut_sell=null,
	price_inv_tau_book_buy=null,
	price_inv_tau_book_sell=null,
	price_dinv_tau_book_buy=null,
	price_dinv_tau_book_sell=null,
	price_trades_max_back_tau_book_buy=null,
	price_trades_max_back_tau_book_sell=null,
	price_trades_aver_back_tau_book_buy=null,
	price_trades_min_back_tau_book_buy=null,
	price_trades_min_back_tau_book_sell=null,
	price_trades_aver_back_tau_book_sell=null,

	pe_lev_buy=null,
	pe_book_buy=null,
	pe_trades_buy=null,
	pe_lev_sell=null,
	pe_book_sell=null,
	pe_trades_sell=null,


	price_trades_min_back_bweighted_buy=null,
	price_trades_max_back_bweighted_sell=null;


    void clear()
    {

	this.v_book_buy=
	this.tau_book_buy=
        this.v_book_sell=
	this.tau_book_sell=null;

	this.lev_E_buy=this.lev_E_sell=
	    this.lev_iE_buy=this.lev_iE_sell=
	    this.lev_iE2_buy=this.lev_iE2_sell=
	    this.lev_iE3_buy=this.lev_iE3_sell=Double.NaN;
	this.inversion_length_buy=
        this.inversion_length_sell=
	this.rinversion_length_buy=
        this.rinversion_length_sell=
	this.sinversion_length_buy=
        this.sinversion_length_sell=
	this.price_lev_cut_buy=
	this.price_lev_cut_sell=
	this.lev_Econv_buy=
	this.lev_Econv_sell=
	this.price_inv_tau_book_buy=
	this.price_inv_tau_book_sell=
	this.price_trades_max_back_tau_book_buy=
	this.price_trades_max_back_tau_book_sell=
	this.price_trades_aver_back_tau_book_buy=
	this.price_trades_min_back_tau_book_buy=
	this.price_trades_min_back_tau_book_sell=
        this.price_trades_aver_back_tau_book_sell=
	this.pe_lev_buy=
	this.pe_book_buy=
	this.pe_trades_buy=
	this.pe_lev_sell=
	this.pe_book_sell=
	this.pe_trades_sell=
        this.price_trades_min_back_bweighted_buy=
	this.price_trades_max_back_bweighted_sell=null;
    }



    private static void calculateTaus(final long currenttime,
				      final int nt,
				      final Distribution<BookItem> buf,
				      final boolean PRINT_DATA_DEBUG,
				      final boolean flag_lower,
				      final double [] price_levels,
				      // output parameters
				      final long [] out_volume,
				      final long [] out_tau,
				      final double [] lev_cut)
    {
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("num\t"+
			       "time\t"+
			       "shares\t"+
			       "buy_sell_indicator\t"+
			       "price\t"+
			       "orderreferencenumber\t");
	}
	final int tsize=buf.getSize();
	final BookItem [] tb=buf.getBuffer();
	double plev=Double.NaN;
	long v=0,sv=0;
	int n_levels=-1;
	if(flag_lower)
	{
	    double lev=Double.POSITIVE_INFINITY;
	    bloop:for(int i=0;i<tsize;i++)
	    {
		final BookItem t=tb[i];
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.shares+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.price+"\t"+
				       t.orderreferencenumber+"\t");
		}
		final long tau=(currenttime-t.time)/ModelParameters.T_SUMM_SCALE;
		for(;!(t.price>=lev);)
		{ // next level
		    if(n_levels>=0)
		    {
			out_volume[n_levels]=v;
			out_tau[n_levels]=sv;
			lev_cut[n_levels]=plev;
		    }
		    if(++n_levels>=nt)
		    {
			break bloop;
		    }
		    lev=price_levels[n_levels];
		}
		v+=t.shares;
		sv+=t.shares*tau;
		plev=t.price;
	    }
	}
	else
	{
	    double lev=Double.NEGATIVE_INFINITY;
	    bloop:for(int i=0;i<tsize;i++)
	    {
		final BookItem t=tb[i];
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.shares+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.price+"\t"+
				       t.orderreferencenumber+"\t");
		}
		final long tau=(currenttime-t.time)/ModelParameters.T_SUMM_SCALE;
		for(;!(t.price<=lev);)
		{ // next level
		    if(n_levels>=0)
		    {
			out_volume[n_levels]=v;
			out_tau[n_levels]=sv;
			lev_cut[n_levels]=plev;
		    }
		    if(++n_levels>=nt)
		    {
			break bloop;
		    }
		    lev=price_levels[n_levels];
		}
		v+=t.shares;
		sv+=t.shares*tau;
		plev=t.price;
	    }
	}
	for(;n_levels<nt;n_levels++)
	{
	    if(n_levels>=0)
	    {
		out_volume[n_levels]=v;
		out_tau[n_levels]=sv;
		lev_cut[n_levels]=plev;
	    }
	}
	for(int i=0;i<nt;i++)
	{
	    final long vv=out_volume[i];
	    if(vv>0)
	    {
		out_tau[i]=(out_tau[i]/vv)*ModelParameters.T_SUMM_SCALE;
	    }
	    else
	    {
		out_tau[i]=-1;
	    }
	}
    }

    static class CMPL implements com.polytechnik.utils.IntPTRCompare{
	private final long [] taus;
	CMPL(final long [] taus)
	{
	    this.taus=taus;
	}
	public int compare(final int p1,final int p2)
	{
	    return Long.compare(taus[p1],taus[p2]);
	}
    }

    static void calculateTradesMM(final long currenttime,
				  final long [] delays,
				  final int nt,
				  final Distribution<Trade> trades,
				  final boolean PRINT_DATA_DEBUG,
				  final int [] px, // temporary buffer
				  // output
				  final double [] min,
				  final double [] max,
				  final double [] aver,
				  final long [] vol)
    {
	final int [] inds=com.polytechnik.outils.TMPQSortAlgorithm.sortArrayIndexes(nt,new CMPL(delays));
	System.arraycopy(inds,0,px,0,inds.length);

	final int tsize=trades.getSize();
	final Trade [] tb=trades.getBuffer();
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("num\t"+
			       "time\t"+
			       "origtime\t"+
			       "shares\t"+
			       "tradetype\t"+
			       "buy_sell_indicator\t"+
			       "price\t"+
			       "matchnumber\t"+
			       "orderreferencenumber");
	}
	long tbefore=Long.MAX_VALUE,v=0;
	double xmin=Double.NaN,xmax=Double.NaN,xaver=0;
	int cur_pos=-1;
        gloop:for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println(String.valueOf(i)+"\t"+
				   t.time+"\t"+
				   t.origtime+"\t"+
				   t.shares+"\t"+
				   t.tradetype+"\t"+
				   t.buy_sell_indicator+"\t"+
				   t.price+"\t"+
				   t.matchnumber+"\t"+
				   t.orderreferencenumber);
	    }
	    for(;!(t.time>=tbefore);)
	    {
		if(cur_pos>=0)
		{
		    final int ptr=px[cur_pos];
		    if(min!=null)
		    {
			min[ptr]=xmin;
		    }
		    if(max!=null)
		    {
			max[ptr]=xmax;
		    }
		    if(aver!=null)
		    {
			aver[ptr]=v>0?xaver/v:Double.NaN;
		    }
		    if(vol!=null)
		    {
			vol[ptr]=v;
		    }
		}
		if(++cur_pos>=nt)
		{
		    break gloop;
		}
		tbefore=currenttime-delays[px[cur_pos]];
	    }
	    if(!(t.price>xmin))
	    {
		xmin=t.price;
	    }
	    if(!(t.price<xmax))
	    {
		xmax=t.price;
	    }
	    v+=t.shares;
	    xaver+=t.price*t.shares;
	}
	for(;cur_pos<nt;cur_pos++)
	{
	    if(cur_pos>=0)
	    {
		final int ptr=px[cur_pos];
		if(min!=null)
		{
		    min[ptr]=xmin;
		}
		if(max!=null)
		{
		    max[ptr]=xmax;
		}
		if(aver!=null)
		{
		    aver[ptr]=v>0?xaver/v:Double.NaN;
		}
		if(vol!=null)
		{
		    vol[ptr]=v;
		}
	    }
	}
    }




    void calculateLiquidityDeficit(final double [] buy_pricelevels,
				   final double [] sell_pricelevels,
				   final long currenttime,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final long trades_cleared_before)
    {

	final boolean PRINT_DATA_DEBUG=false;
	    //currenttime==34434604000000L;
	    //currenttime==34354409000000L;
	    //currenttime==34223484000000L;
	    //currenttime>=9*3600*TraderInterface.TIME_SCALE && tsize>0;
	    //currenttime==34204365000000L;


	final int nt=buy_pricelevels.length;
	if(sell_pricelevels.length!=nt)
	{
	    throw new IllegalArgumentException();
	}


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tN_BUY\t"+this.buy.getSize()+
			       "\tN_SELL\t"+this.sell.getSize()+
			       "\nLEVELS\t"+nt+
			       "\ncur_buy\t"+pcurrent_buy+
			       "\tcur_sell\t"+pcurrent_sell);
	}


	this.v_book_buy=new long [nt];
	this.tau_book_buy=new long [nt];
	this.price_lev_cut_buy=new double [nt];
	calculateTaus(currenttime,
		      nt,
		      this.buy,
		      PRINT_DATA_DEBUG,
		      true,
		      buy_pricelevels,
		      this.v_book_buy,
		      this.tau_book_buy,
		      this.price_lev_cut_buy);

        this.v_book_sell=new long [nt];
	this.tau_book_sell=new long [nt];
	this.price_lev_cut_sell=new double [nt];
	calculateTaus(currenttime,
		      nt,
		      this.sell,
		      PRINT_DATA_DEBUG,
		      false,
		      sell_pricelevels,
		      this.v_book_sell,
		      this.tau_book_sell,
		      this.price_lev_cut_sell);


	this.price_trades_min_back_tau_book_buy=new double [nt];
	this.price_trades_max_back_tau_book_buy=new double [nt];
	this.price_trades_aver_back_tau_book_buy=new double [nt];
	checkCapacity(nt); // increase capacity for temporary array this.ptr_sorttau
	calculateTradesMM(currenttime,
			  this.tau_book_buy,
			  nt,
			  this.trades,
			  PRINT_DATA_DEBUG,
			  this.ptr_sorttau,
			  this.price_trades_min_back_tau_book_buy,
			  this.price_trades_max_back_tau_book_buy,
			  this.price_trades_aver_back_tau_book_buy,
			  null);


	this.price_trades_min_back_tau_book_sell=new double [nt];
	this.price_trades_max_back_tau_book_sell=new double [nt];
	this.price_trades_aver_back_tau_book_sell=new double [nt];
	calculateTradesMM(currenttime,
			  this.tau_book_sell,
			  nt,
			  this.trades,
			  PRINT_DATA_DEBUG,
			  this.ptr_sorttau,
			  this.price_trades_min_back_tau_book_sell,
			  this.price_trades_max_back_tau_book_sell,
			  this.price_trades_aver_back_tau_book_sell,
			  null);


	final double [] tmp=new double [nt];
	calculateBookLimits(currenttime,
			    this.buy,
			    this.trades,
			    PRINT_DATA_DEBUG,
			    true);
	final int n_buy_levels=this.n_levs;
	final int ptrb=getMMEElev(currenttime,
				  pcurrent_buy,
				  PRINT_DATA_DEBUG,
				  true,
				  this.n_levs,
				  this.p_lev,
				  this.p_B_lev,
				  this.p_T_lev);
	this.lev_E_buy=ptrb<0?Double.NaN:this.p_lev[ptrb];

	this.lev_Econv_buy=new double [nt];
	getMMCutEElev2(currenttime,
		       pcurrent_buy,
		       buy_pricelevels,
		       PRINT_DATA_DEBUG,
		       true,
		       this.n_levs,
		       this.p_lev,
		       this.p_B_lev,
		       this.lev_Econv_buy);

	this.price_inv_tau_book_buy=new double [nt];
	this.price_dinv_tau_book_buy=new double [nt];
	for(int i=0;i<nt;i++)
	{
	    if(true)
	    {
		final int ptr=findBookInversion(currenttime,
						PRINT_DATA_DEBUG,
						false,
						this.tau_book_buy[i],
						this.n_levs,
						this.p_lev,
						this.stau_book_levels);
		this.price_inv_tau_book_buy[i]=ptr<0?Double.NaN:this.p_lev[ptr];
	    }
	    if(true)
	    {
		final int ptr=findBookInversion(currenttime,
						PRINT_DATA_DEBUG,
						true,
						this.tau_book_buy[i],
						this.n_levs,
						this.p_lev,
						this.stau_book_levels);
		this.price_dinv_tau_book_buy[i]=ptr<0?Double.NaN:this.p_lev[ptr];
	    }
	}



	this.pe_lev_buy=new double [nt];
	this.pe_book_buy=new double [nt];
	this.pe_trades_buy=new double [nt];
	findInversion(currenttime,
		      PRINT_DATA_DEBUG,
		      true,
		      this.n_levs,
		      this.p_lev,
		      this.p_B_lev,
		      this.p_T_lev,
		      buy_pricelevels,

		      this.pe_lev_buy,
		      this.pe_book_buy,
		      this.pe_trades_buy);
        this.price_trades_min_back_bweighted_buy=new double [nt];
	calculatePresetLevels(currenttime,
			      PRINT_DATA_DEBUG,
			      true,
			      this.n_levs,
			      this.p_lev,
			      this.p_B_lev,
			      this.stau_book_levels,
			      this.p_T_lev,
			      this.stau_trades_levels,
			      buy_pricelevels,

			      tmp,
			      this.price_trades_min_back_bweighted_buy);
	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    for(int i=0;i<nt;i++)
	    {
		if(Double.isNaN(tmp[i])!=Double.isNaN(this.price_trades_min_back_tau_book_buy[i])||
		   (!Double.isNaN(tmp[i]) && tmp[i]!=this.price_trades_min_back_tau_book_buy[i]))
		{
		    throw new RuntimeException(
			"Book tback mismatch:"+
			" tmp["+i+"]="+tmp[i]+
			" price_trades_min_back_tau_book_buy["+i+"]="+this.price_trades_min_back_tau_book_buy[i]+
			" nt="+nt+
			" n_levs="+this.n_levs+
			" N_BUY="+this.buy.getSize()+
			" N_TRADES="+this.trades.getSize()+
			" currenttime="+currenttime);
		}
	    }
	}

	calculateBookLimits(currenttime,
			    this.sell,
			    this.trades,
			    PRINT_DATA_DEBUG,
			    false);
	final int n_sell_levels=this.n_levs;
	final int ptrs=getMMEElev(currenttime,
				  pcurrent_sell,
				  PRINT_DATA_DEBUG,
				  false,
				  this.n_levs,
				  this.p_lev,
				  this.p_B_lev,
				  this.p_T_lev);
	this.lev_E_sell=ptrs<0?Double.NaN:this.p_lev[ptrs];

	this.lev_Econv_sell=new double [nt];
	getMMCutEElev2(currenttime,
		       pcurrent_sell,
		       sell_pricelevels,
		       PRINT_DATA_DEBUG,
		       false,
		       this.n_levs,
		       this.p_lev,
		       this.p_B_lev,
		       this.lev_Econv_sell);

	this.price_inv_tau_book_sell=new double [nt];
	this.price_dinv_tau_book_sell=new double [nt];
	for(int i=0;i<nt;i++)
	{
	    if(true)
	    {
		final int ptr=findBookInversion(currenttime,
						PRINT_DATA_DEBUG,
						false,
						this.tau_book_sell[i],
						this.n_levs,
						this.p_lev,
						this.stau_book_levels);
		this.price_inv_tau_book_sell[i]=ptr<0?Double.NaN:this.p_lev[ptr];
	    }
	    if(true)
	    {
		final int ptr=findBookInversion(currenttime,
						PRINT_DATA_DEBUG,
						true,
						this.tau_book_sell[i],
						this.n_levs,
						this.p_lev,
						this.stau_book_levels);
		this.price_dinv_tau_book_sell[i]=ptr<0?Double.NaN:this.p_lev[ptr];
	    }
	}

	this.pe_lev_sell=new double [nt];
	this.pe_book_sell=new double [nt];
	this.pe_trades_sell=new double [nt];
	findInversion(currenttime,
		      PRINT_DATA_DEBUG,
		      false,
		      this.n_levs,
		      this.p_lev,
		      this.p_B_lev,
		      this.p_T_lev,
		      sell_pricelevels,

		      this.pe_lev_sell,
		      this.pe_book_sell,
		      this.pe_trades_sell);
	this.price_trades_max_back_bweighted_sell=new double [nt];
	calculatePresetLevels(currenttime,
			      PRINT_DATA_DEBUG,
			      false,
			      this.n_levs,
			      this.p_lev,
			      this.p_B_lev,
			      this.stau_book_levels,
			      this.p_T_lev,
			      this.stau_trades_levels,
			      sell_pricelevels,

			      tmp,
			      this.price_trades_max_back_bweighted_sell);
	/*
	final double xx=
	    (this.price_trades_max_back_tau_book_sell[2]-pcurrent_sell)/
	    (this.price_trades_max_back_bweighted_sell[2]-pcurrent_sell);
	if(this.price_trades_max_back_tau_book_sell[2]>this.price_trades_max_back_bweighted_sell[2]||
	   xx>1)
	{
	    System.err.println(
		"OFF:"+
		" XX="+xx+
		" this.price_trades_max_back_tau_book_sell[2]="+
		this.price_trades_max_back_tau_book_sell[2]+
		" this.price_trades_max_back_bweighted_sell[2]="+
		this.price_trades_max_back_bweighted_sell[2]+
		" p_buy="+pcurrent_buy+
		" p_sell="+pcurrent_sell);
	}
	*/




	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    for(int i=0;i<nt;i++)
	    {
		if(Double.isNaN(tmp[i])!=Double.isNaN(this.price_trades_max_back_tau_book_sell[i])||
		   (!Double.isNaN(tmp[i]) && tmp[i]!=this.price_trades_max_back_tau_book_sell[i]))
		{
		    throw new RuntimeException(
			"Book tback mismatch:"+
			" tmp["+i+"]="+tmp[i]+
			" price_trades_max_back_tau_book_sell["+i+"]="+this.price_trades_max_back_tau_book_sell[i]+
			" nt="+nt+
			" n_levs="+this.n_levs+
			" N_SELL="+this.sell.getSize()+
			" N_TRADES="+this.trades.getSize()+
			" currenttime="+currenttime);
		}
	    }
	}

	final double [] pinvs=new double [3];
	final int [] tmp_ptr=new int [nt];
	final long [] tmp_v=new long [nt],tmp_sv=new long [nt];

	this.inversion_length_buy=new double [nt];
	this.rinversion_length_buy=new double [nt];
	this.sinversion_length_buy=new double [nt];
	this.lev_iE_buy=
	calculateInversionPerLevel(buy_pricelevels,
				   nt,
				   currenttime,
				   pcurrent_buy,
				   pcurrent_sell,
				   this.buy,
				   PRINT_DATA_DEBUG,
				   true,
				   n_buy_levels,
					  // output parameters
				   tmp,
				   tmp_v,
				   tmp_sv,

				   this.inversion_length_buy,
				   this.rinversion_length_buy,
				   this.sinversion_length_buy,
				   pinvs,
				   // temporary
				   this.ptr_sorttau,
				   this.v_book_lev,
				   this.stau_book_levels,
				   tmp_ptr);
	this.lev_iE2_buy=pinvs[1];
	this.lev_iE3_buy=pinvs[2];
	for(int i=0;i<nt;i++)
	{
	    if(tmp_v[i]!=this.v_book_buy[i]||
	       tmp_sv[i]!=this.tau_book_buy[i]||
	       Double.isNaN(tmp[i])!=Double.isNaN(this.price_lev_cut_buy[i])||
	       (!Double.isNaN(tmp[i]) && tmp[i]!=this.price_lev_cut_buy[i]))
		throw new RuntimeException(
		    "Buy mismatch:"+
		    " tmp_v["+i+"]="+tmp_v[i]+
		    " v_book_buy["+i+"]="+this.v_book_buy[i]+
		    " tmp_sv["+i+"]="+tmp_sv[i]+
		    " tau_book_buy["+i+"]="+this.tau_book_buy[i]+
		    " price="+tmp[i]+
		    " price_calc="+this.price_lev_cut_buy[i]+
		    " N_BUY="+this.buy.getSize()+
		    " flag_lower="+true+
		    " currenttime="+currenttime);
	}

        this.inversion_length_sell=new double [nt];
        this.rinversion_length_sell=new double [nt];
        this.sinversion_length_sell=new double [nt];
	this.lev_iE_sell=
	calculateInversionPerLevel(sell_pricelevels,
				   nt,
				   currenttime,
				   pcurrent_buy,
				   pcurrent_sell,
				   this.sell,
				   PRINT_DATA_DEBUG,
				   false,
				   n_sell_levels,
				   // output parameters
				   tmp,
				   tmp_v,
				   tmp_sv,

				   this.inversion_length_sell,
				   this.rinversion_length_sell,
				   this.sinversion_length_sell,
				   pinvs,
				   // temporary
				   this.ptr_sorttau,
				   this.v_book_lev,
				   this.stau_book_levels,
				   tmp_ptr);
	this.lev_iE2_sell=pinvs[1];
	this.lev_iE3_sell=pinvs[2];
	for(int i=0;i<nt;i++)
	{
	    if(tmp_v[i]!=this.v_book_sell[i]||
	       tmp_sv[i]!=this.tau_book_sell[i]||
	       Double.isNaN(tmp[i])!=Double.isNaN(this.price_lev_cut_sell[i])||
	       (!Double.isNaN(tmp[i]) && tmp[i]!=this.price_lev_cut_sell[i]))
		throw new RuntimeException(
		    "Sell mismatch:"+
		    " tmp_v["+i+"]="+tmp_v[i]+
		    " v_book_sell["+i+"]="+this.v_book_sell[i]+
		    " tmp_sv["+i+"]="+tmp_sv[i]+
		    " tau_book_sell["+i+"]="+this.tau_book_sell[i]+
		    " price="+tmp[i]+
		    " price_calc="+this.price_lev_cut_sell[i]+
		    " N_SELL="+this.sell.getSize()+
		    " flag_lower="+false+
		    " currenttime="+currenttime);
	}

	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    final CalculateBookVolumePriceFixedThreshold
		tbuy=new CalculateBookVolumePriceFixedThreshold(true,this.buy),
		tsell=new CalculateBookVolumePriceFixedThreshold(false,this.sell);
	    tbuy.calculateThresholds(currenttime,
				     pcurrent_buy,
				     pcurrent_sell,
				     buy_pricelevels);
	    tsell.calculateThresholds(currenttime,
				      pcurrent_buy,
				      pcurrent_sell,
				      sell_pricelevels);
	    for(int i=0;i<nt;i++)
	    {
		if(this.v_book_buy[i]!=tbuy.vol_threshold_vcalc[i]||
		   this.tau_book_buy[i]!=tbuy.tau_threshold_vcalc[i]||
		   this.v_book_sell[i]!=tsell.vol_threshold_vcalc[i]||
		   this.tau_book_sell[i]!=tsell.tau_threshold_vcalc[i]||
		   Double.isNaN(this.price_lev_cut_buy[i])!=Double.isNaN(tbuy.plast[i])||
		   (!Double.isNaN(this.price_lev_cut_buy[i]) &&
		    this.price_lev_cut_buy[i]!=tbuy.plast[i])||
		   Double.isNaN(this.price_lev_cut_sell[i])!=Double.isNaN(tsell.plast[i])||
		   (!Double.isNaN(this.price_lev_cut_sell[i]) &&
		    this.price_lev_cut_sell[i]!=tsell.plast[i])
		   )
		{
		    throw new RuntimeException(
			"Book tau mismatch:"+
			" v_book_buy["+i+"]="+this.v_book_buy[i]+
			" tbuy.vol_threshold_vcalc["+i+"]="+tbuy.vol_threshold_vcalc[i]+
			" tau_book_buy["+i+"]="+this.tau_book_buy[i]+
			" tbuy.tau_threshold_vcalc["+i+"]="+tbuy.tau_threshold_vcalc[i]+
			" v_book_sell["+i+"]="+this.v_book_sell[i]+
			" tsell.vol_threshold_vcalc["+i+"]="+tsell.vol_threshold_vcalc[i]+
			" tau_book_sell["+i+"]="+this.tau_book_sell[i]+
			" tsell.tau_threshold_vcalc["+i+"]="+tsell.tau_threshold_vcalc[i]+
			" price_lev_cut_buy["+i+"]="+this.price_lev_cut_buy[i]+
			" tbuy.plast["+i+"]="+tbuy.plast[i]+
			" price_lev_cut_sell["+i+"]="+this.price_lev_cut_sell[i]+
			" tsell.plast["+i+"]="+tsell.plast[i]+
			" N_BUY="+this.buy.getSize()+
			" N_SELL="+this.sell.getSize()+
			" currenttime="+currenttime);
		}
	    }
	    final CalculateTradeHContrib tv=new CalculateTradeHContrib(this.trades,this.tmptradesbuf);
	    for(int i=0;i<nt;i++)
	    {
		tv.calculateContribs(new long []{this.tau_book_buy[i]},
				     currenttime,
				     pcurrent_buy,
				     pcurrent_sell,
				     0);
		if(Double.isNaN(this.price_trades_min_back_tau_book_buy[i])!=Double.isNaN(tv.minprice[0])||
		   (!Double.isNaN(this.price_trades_min_back_tau_book_buy[i]) &&
		    this.price_trades_min_back_tau_book_buy[i]!=tv.minprice[0])||
		   Double.isNaN(this.price_trades_max_back_tau_book_buy[i])!=Double.isNaN(tv.maxprice[0])||
		   (!Double.isNaN(this.price_trades_max_back_tau_book_buy[i]) &&
		    this.price_trades_max_back_tau_book_buy[i]!=tv.maxprice[0])||
		   Double.isNaN(this.price_trades_aver_back_tau_book_buy[i])!=Double.isNaN(tv.averprice[0])||
		   (!Double.isNaN(this.price_trades_aver_back_tau_book_buy[i]) &&
		    this.price_trades_aver_back_tau_book_buy[i]!=tv.averprice[0])
		   )
		{
		    throw new RuntimeException(
			"min/max/aver on trades mismatch:"+
			" price_trades_min_back_tau_book_buy["+i+"]="+this.price_trades_min_back_tau_book_buy[i]+
			" tv.minprice="+tv.minprice[0]+
			" price_trades_max_back_tau_book_buy["+i+"]="+this.price_trades_max_back_tau_book_buy[i]+
			" tv.maxprice="+tv.maxprice[0]+
			" price_trades_aver_back_tau_book_buy["+i+"]="+this.price_trades_aver_back_tau_book_buy[i]+
			" tv.averprice="+tv.averprice[0]+
			" tau_buy="+this.tau_book_buy[i]+
			" currenttime="+currenttime);
		}
		tv.calculateContribs(new long []{this.tau_book_sell[i]},
				     currenttime,
				     pcurrent_buy,
				     pcurrent_sell,
				     0);

		if(Double.isNaN(this.price_trades_min_back_tau_book_sell[i])!=Double.isNaN(tv.minprice[0])||
		   (!Double.isNaN(this.price_trades_min_back_tau_book_sell[i]) &&
		    this.price_trades_min_back_tau_book_sell[i]!=tv.minprice[0])||
		   Double.isNaN(this.price_trades_max_back_tau_book_sell[i])!=Double.isNaN(tv.maxprice[0])||
		   (!Double.isNaN(this.price_trades_max_back_tau_book_sell[i]) &&
		    this.price_trades_max_back_tau_book_sell[i]!=tv.maxprice[0])||
		   Double.isNaN(this.price_trades_aver_back_tau_book_sell[i])!=Double.isNaN(tv.averprice[0])||
		   (!Double.isNaN(this.price_trades_aver_back_tau_book_sell[i]) &&
		    this.price_trades_aver_back_tau_book_sell[i]!=tv.averprice[0])
		   )
		{
		    throw new RuntimeException(
			"min/max/aver on trades mismatch:"+
			" price_trades_min_back_tau_book_sell["+i+"]="+this.price_trades_min_back_tau_book_sell[i]+
			" tv.minprice="+tv.minprice[0]+
			" price_trades_max_back_tau_book_sell["+i+"]="+this.price_trades_max_back_tau_book_sell[i]+
			" tv.maxprice_sell="+tv.maxprice[0]+
			" price_trades_aver_back_tau_book_sell["+i+"]="+this.price_trades_aver_back_tau_book_sell[i]+
			" tv.averprice_sell="+tv.averprice[0]+
			" tau_sell="+this.tau_book_sell[i]+
			" currenttime="+currenttime);
		}
	    }

	    final double [] out_p_invs=new double[2];
	    final long [] out_t_invs=new long [3];
	    final double iet_buy=
		CalculateLiquidityDeficit6.calculateInversionPerLevel(
		    currenttime,
		    pcurrent_buy,
		    pcurrent_sell,
		    this.buy,
		    PRINT_DATA_DEBUG,
		    true,
		    out_p_invs,
		    out_t_invs);
	    final double iet_buy_cont=out_p_invs[1];

	    final double iet_sell=
		CalculateLiquidityDeficit6.calculateInversionPerLevel(
		    currenttime,
		    pcurrent_buy,
		    pcurrent_sell,
		    this.sell,
		    PRINT_DATA_DEBUG,
		    false,
		    out_p_invs,
		    out_t_invs);
	    final double iet_sell_cont=out_p_invs[1];

	    if(Double.isNaN(iet_buy)!=Double.isNaN(this.lev_iE_buy)||
	       (!Double.isNaN(iet_buy) && iet_buy!=this.lev_iE_buy)||
	       Double.isNaN(iet_sell)!=Double.isNaN(this.lev_iE_sell)||
	       (!Double.isNaN(iet_sell) && iet_sell!=this.lev_iE_sell)||

	       Double.isNaN(iet_buy_cont)!=Double.isNaN(this.lev_iE3_buy)||
	       (!Double.isNaN(iet_buy_cont) && iet_buy_cont!=this.lev_iE3_buy)||
	       Double.isNaN(iet_sell_cont)!=Double.isNaN(this.lev_iE3_sell)||
	       (!Double.isNaN(iet_sell_cont) && iet_sell_cont!=this.lev_iE3_sell))
	    {
		throw new RuntimeException(
		    "Book inv mismatch:"+
		    " iet_buy="+iet_buy+
		    " lev_iE_buy="+this.lev_iE_buy+
		    " iet_sell="+iet_sell+
		    " lev_iE_sell="+this.lev_iE_sell+
		    " iet_buy_cont="+iet_buy_cont+
		    " lev_iE3_buy="+this.lev_iE3_buy+
		    " iet_sell_cont="+iet_sell_cont+
		    " lev_iE3_sell="+this.lev_iE3_sell+
		    " N_BUY="+this.buy.getSize()+
		    " N_SELL="+this.sell.getSize()+
		    " currenttime="+currenttime);
	    }
	}
    }



    private long []
	v_book_lev=new long [20],
	stau_book_levels=new long [v_book_lev.length],
	stau_trades_levels=new long [v_book_lev.length];
    private double []
	p_lev=new double [v_book_lev.length],
	p_B_lev=new double [v_book_lev.length],
	p_T_lev=new double [v_book_lev.length],
	p_AB_lev=new double [v_book_lev.length],
	p_AT_lev=new double [v_book_lev.length];
    private int []
	ptr_sorttau=new int [v_book_lev.length];
    int n_levs=0;
    private void checkCapacity(final int ptr_save)
    {
	if(ptr_save<this.v_book_lev.length)
	{
	    return;
	}
	final int ns=1+2*this.v_book_lev.length,n_tocopy=ptr_save;
	final long [] v1=new long [ns],sb1=new long [ns],st1=new long [ns];
	final double [] pl=new double [ns],pb=new double [ns],pt=new double [ns],pab=new double [ns],pat=new double [ns];
	final int [] ptr=new int [ns];
	System.arraycopy(this.v_book_lev,0,v1,0,n_tocopy);
	this.v_book_lev=v1;
	System.arraycopy(this.stau_book_levels,0,sb1,0,n_tocopy);
	this.stau_book_levels=sb1;
	System.arraycopy(this.stau_trades_levels,0,st1,0,n_tocopy);
	this.stau_trades_levels=st1;
	System.arraycopy(this.p_lev,0,pl,0,n_tocopy);
	this.p_lev=pl;
	System.arraycopy(this.p_B_lev,0,pb,0,n_tocopy);
	this.p_B_lev=pb;
	System.arraycopy(this.p_T_lev,0,pt,0,n_tocopy);
	this.p_T_lev=pt;
	System.arraycopy(this.p_AB_lev,0,pab,0,n_tocopy);
	this.p_AB_lev=pab;
	System.arraycopy(this.p_AT_lev,0,pat,0,n_tocopy);
	this.p_AT_lev=pat;
	System.arraycopy(this.ptr_sorttau,0,ptr,0,n_tocopy);
	this.ptr_sorttau=ptr;
    }



    private void calculateBookLimits(final long currenttime,
				     final Distribution<BookItem> book,
				     final Distribution<Trade> trades,
				     final boolean PRINT_DATA_DEBUG,
				     final boolean flag_lower)
    {
	this.n_levs=0;
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println();
	    System.err.println("=="+getClass().getName()+".calculateBookLimits"+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNORDERS=\t"+book.getSize()+
			       "\tNTRADES=\t"+trades.getSize()+
			       "\t flag_lower=\t"+flag_lower);

	    System.err.println("num\t"+
			       "time\t"+
			       "shares\t"+
			       "buy_sell_indicator\t"+
			       "price\t"+
			       "orderreferencenumber\t");
	}

	final int tsize=trades.getSize(),bsize=book.getSize();
	final Trade [] tb=trades.getBuffer();
	final BookItem [] bb=book.getBuffer();

	double pbook=Double.NaN,p_lev_cut=Double.NaN;
	long v=0,svb=0,svt=0;
	long treachs=0,treach=0;
	int ptr_levs=-1,ptr_trades=tsize-1;
	for(int i=0;i<bsize;i++)
	{
	    final BookItem b=bb[i];
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println(String.valueOf(i)+"\t"+
				   b.time+"\t"+
				   b.shares+"\t"+
				   b.buy_sell_indicator+"\t"+
				   b.price+"\t"+
				   b.orderreferencenumber+"\t");
	    }
	    if(flag_lower?!(b.price>=p_lev_cut):!(b.price<=p_lev_cut))
	    {
		//System.err.println("INIT ptr_levs="+ptr_levs+" ptr_trades="+ptr_trades);
		if(ptr_levs>=0)
		{
		    checkCapacity(ptr_levs);
		    this.v_book_lev[ptr_levs]=v;
		    this.stau_book_levels[ptr_levs]=svb;
		    this.stau_trades_levels[ptr_levs]=svt;
		    this.p_lev[ptr_levs]=pbook;
		    this.ptr_sorttau[ptr_levs]=ptr_trades; // save temporarily
		}
		ptr_levs++;
		pbook=b.price;
		p_lev_cut=pbook+(flag_lower?-OrdersFlow.EPS:OrdersFlow.EPS);
		// scan trades
		if(ptr_trades>=0)
		{
		    for(;;)
		    {
			final Trade t=tb[ptr_trades];
			if(flag_lower?!(t.price>=p_lev_cut):!(t.price<=p_lev_cut))
			{
			    treachs=currenttime-t.time;
			    treach=treachs/ModelParameters.T_SUMM_SCALE;
			    break;
			}
			if(--ptr_trades<0)
			{
			    treachs=currenttime-(tsize>0?tb[0].time:0);
			    treach=treachs/ModelParameters.T_SUMM_SCALE;
			    break;
			}
		    }
		}
		//System.err.println("END ptr_levs="+ptr_levs+" ptr_trades="+ptr_trades);
	    }
	    final long taus=currenttime-b.time,tau=taus/ModelParameters.T_SUMM_SCALE;
	    if(taus>treachs && ptr_trades>=0)
	    {
		System.err.println(
		    "CalculateLiquidityDeficit4:calculateBookLimits out of interval range order"+
		    " dtau="+(taus-treachs)+
		    " taus="+taus+
		    " treachs="+treachs+
		    " time="+BookItem.getTimeDiag(currenttime)+
		    " currenttime="+currenttime);
	    }
	    final long s=b.shares;
	    v+=s;
	    svb+=s*tau;
	    svt+=s*treach;
	}
	if(ptr_levs>=0)
	{
	    checkCapacity(ptr_levs);
	    this.v_book_lev[ptr_levs]=v;
	    this.stau_book_levels[ptr_levs]=svb;
	    this.stau_trades_levels[ptr_levs]=svt;
	    this.p_lev[ptr_levs]=pbook;
	    this.ptr_sorttau[ptr_levs]=ptr_trades; // save temporarily
	}
	final int nt=this.n_levs=1+ptr_levs;
	for(int i=0;i<nt;i++)
	{
	    final long vv=this.v_book_lev[i];
	    if(vv>0)
	    {
		this.stau_book_levels[i]=(this.stau_book_levels[i]/vv)*ModelParameters.T_SUMM_SCALE;
		this.stau_trades_levels[i]=(this.stau_trades_levels[i]/vv)*ModelParameters.T_SUMM_SCALE;
	    }
	    else
	    {
		this.stau_book_levels[i]=this.stau_trades_levels[i]=-1;
	    }
	}
	final int [] tmp_pointers_to_trades;
	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    tmp_pointers_to_trades=this.ptr_sorttau.clone(); // read from temporary array
	}
	else
	{
	    tmp_pointers_to_trades=null;
	}

	// now we have the number of levels known
	// scan trades to determine critical point
	calculateTradesMM(currenttime,
			  this.stau_book_levels,
			  nt,
			  this.trades,
			  PRINT_DATA_DEBUG,
			  this.ptr_sorttau,
			  flag_lower?this.p_B_lev:null,
			  flag_lower?null:this.p_B_lev,
			  this.p_AB_lev,
			  null);

	calculateTradesMM(currenttime,
			  this.stau_trades_levels,
			  nt,
			  this.trades,
			  PRINT_DATA_DEBUG,
			  this.ptr_sorttau,
			  flag_lower?this.p_T_lev:null,
			  flag_lower?null:this.p_T_lev,
			  this.p_AT_lev,
			  null);
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("N_LEVELS\t"+nt);
	    System.err.println("n\tp_lev\tp_book\tp_trades\tpa_book\tpa_trades\ttau_book\ttau_trades\tv_book");
	    for(int i=0;i<nt;i++)
	    {
		System.err.println(String.valueOf(i)+"\t"+
				   this.p_lev[i]+"\t"+
				   this.p_B_lev[i]+"\t"+
				   this.p_T_lev[i]+"\t"+
				   this.p_AB_lev[i]+"\t"+
				   this.p_AT_lev[i]+"\t"+
				   this.stau_book_levels[i]+"\t"+
				   this.stau_trades_levels[i]+"\t"+
				   this.v_book_lev[i]);

	    }
	}
	// OK, now we have price levels calculated, scan them and compare.



	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    // expensive constrains
	    int pp_ptr_levs=-1;
	    double pp=Double.NaN;
	    long vv=0,svv=0;
	    for(int i=0;i<bsize;i++)
	    {
		final BookItem b=bb[i];
		if(b.price!=pp)
		{
		    if(pp_ptr_levs>=0)
		    {
			final long st;
			if(vv>0)
			{
			    st=(svv/vv)*ModelParameters.T_SUMM_SCALE;
			}
			else
			{
			    st=-1;
			}
			if(vv!=this.v_book_lev[pp_ptr_levs]||
			   pp!=this.p_lev[pp_ptr_levs]||
			   st!=this.stau_book_levels[pp_ptr_levs])
			{
			    throw new RuntimeException(
				"Volume or level mismatch:"+
				" pp="+pp+
				" p_lev["+pp_ptr_levs+"]="+this.p_lev[pp_ptr_levs]+
				" vv="+vv+
				" v_book_lev["+pp_ptr_levs+"]="+this.v_book_lev[pp_ptr_levs]+
				" st="+st+
				" stau_book_levels["+pp_ptr_levs+"]="+this.stau_book_levels[pp_ptr_levs]+
				" pp_ptr_levs="+pp_ptr_levs+
				" nt="+nt+
				" bsize="+bsize+
				" tsize="+tsize+
				" flag_lower="+flag_lower+
				" currenttime="+currenttime);
			}
		    }
		    pp_ptr_levs++;
		    pp=b.price;
		}
		vv+=b.shares;
		svv+=b.shares*((currenttime-b.time)/ModelParameters.T_SUMM_SCALE);
	    }
	    final int nx_levels=1+pp_ptr_levs;
	    if(nx_levels!=nt ||
	       Double.isNaN(pp)!=Double.isNaN(pbook)||
	       (!Double.isNaN(pp) && pp!=pbook)||
	       v!=vv||
	       svv!=svb)
	    {
		throw new RuntimeException(
		    "Number of levels or volume mismatch:"+
		    " pp="+pp+
		    " pbook="+pbook+
		    " nx_levels="+nx_levels+
		    " nt="+nt+
		    " v="+v+
		    " vv="+vv+
		    " svv="+svv+
		    " svb="+svb+
		    " bsize="+bsize+
		    " tsize="+tsize+
		    " flag_lower="+flag_lower+
		    " currenttime="+currenttime);
	    }
	    final double []
		price_cut=new double [1],
		price_d_book=new double [1],
		price_d_trades=new double [1];
	    final long []
		tau_cut=new long [1],
		vol_book_cut=new long [1],
		tau_book_cut=new long [1],
		tau_trades_cut=new long [1];
	    for(int i=0;i<nt;i++)
	    {
		if(tmp_pointers_to_trades[i]<0)
		{
		    // will not match to old version if no trades avail, skip constrain
		    continue;
		}
		final double t_price=tb[tmp_pointers_to_trades[i]].price;
		// skip constrain when book cut price is calculated differently
		for(int j=i+1,i0=i;j<nt && flag_lower?!(this.p_lev[j]<t_price+OrdersFlow.EPS):
			!(this.p_lev[j]>t_price-OrdersFlow.EPS);)
		{
		    if(tmp_pointers_to_trades[i0]!=tmp_pointers_to_trades[j])
		    {
			throw new RuntimeException(
			    "Pointers mismatch:"+
			    " t_price="+t_price+
			    " p_lev["+i0+"]="+this.p_lev[i0]+
			    " p_lev["+j+"]="+this.p_lev[j]+
			    " tmp_pointers_to_trades["+i0+"]="+tmp_pointers_to_trades[i0]+
			    " tmp_pointers_to_trades["+j+"]="+tmp_pointers_to_trades[j]+
			    " bsize="+bsize+
			    " tsize="+tsize+
			    " flag_lower="+flag_lower+
			    " currenttime="+currenttime);
		    }
		    i++;
		    j++;
		}

		final double p_cut=t_price+(flag_lower?-OrdersFlow.EPS:OrdersFlow.EPS);
		CalculateLiquidityDeficit3.calculateBookLimits(currenttime,
							       book,
							       trades,
							       flag_lower,
							       currenttime,
							       p_cut,
							       false,
							       0,
							       tau_cut,
							       price_cut,
							       vol_book_cut,
							       tau_book_cut,
							       tau_trades_cut,

							       price_d_book,
							       price_d_trades,
							       tmptradesbuf);
		if(Double.isNaN(t_price)!=Double.isNaN(price_cut[0])||
		   (!Double.isNaN(t_price) && t_price!=price_cut[0])||
		   Double.isNaN(this.p_B_lev[i])!=Double.isNaN(price_d_book[0])||
		   (!Double.isNaN(this.p_B_lev[i])&& this.p_B_lev[i]!=price_d_book[0])||
		   Double.isNaN(this.p_T_lev[i])!=Double.isNaN(price_d_trades[0])||
		   (!Double.isNaN(this.p_T_lev[i])&& this.p_T_lev[i]!=price_d_trades[0])||
		   this.v_book_lev[i]!=vol_book_cut[0]||
		   this.stau_book_levels[i]!=tau_book_cut[0]||
		   this.stau_trades_levels[i]!=tau_trades_cut[0])
		{
		    throw new RuntimeException(
			"Mismatch:"+
			" nt="+nt+
			" p_lev["+i+"]="+this.p_lev[i]+
			" t_price="+t_price+
			" price_cut="+price_cut[0]+
			" p_B_lev["+i+"]="+this.p_B_lev[i]+
			" price_d_book="+price_d_book[0]+
			" p_T_lev["+i+"]="+this.p_T_lev[i]+
			" price_d_trades="+price_d_trades[0]+
			" v_book_lev["+i+"]="+this.v_book_lev[i]+
			" vol_book_cut="+vol_book_cut[0]+
			" stau_book_levels["+i+"]="+this.stau_book_levels[i]+
			" tau_book_cut="+tau_book_cut[0]+
			" stau_trades_levels["+i+"]="+this.stau_trades_levels[i]+
			" tau_trades_cut="+tau_trades_cut[0]+
			" bsize="+bsize+
			" tsize="+tsize+
			" flag_lower="+flag_lower+
			" currenttime="+currenttime);
		}

	    }
	}

    }



    private static void findInversion(final long currenttime,
				      final boolean PRINT_DATA_DEBUG,
				      final boolean flag_lower,
				      final int n_data_points,
				      final double [] p_levs,
				      final double [] p_book,
				      final double [] p_trades,
				      final double [] price_thresholds,

				      // output parameters
				      final double [] pext_levs,
				      final double [] pext_book,
				      final double [] pext_trades)
    {
	final int nt=price_thresholds.length;
	double
	    x_atmin=Double.NaN,x_atmax=Double.NaN,
	    xbmin=Double.POSITIVE_INFINITY,xbmax=Double.NEGATIVE_INFINITY,
	    xt_atmin=Double.NaN,xt_atmax=Double.NaN,
	    lev=Double.NaN;
	int cur_pos=-1;
        gloop:for(int i=0;i<n_data_points;i++)
	{
	    final double p=p_levs[i],pb=p_book[i];
	    for(;flag_lower?!(p>=lev):!(p<=lev);)
	    {
		if(cur_pos>=0)
		{
		    if(flag_lower)
		    {
			pext_levs[cur_pos]=x_atmin;
			pext_book[cur_pos]=Double.isInfinite(xbmin)?Double.NaN:xbmin;
			pext_trades[cur_pos]=xt_atmin;
		    }
		    else
		    {
			pext_levs[cur_pos]=x_atmax;
			pext_book[cur_pos]=Double.isInfinite(xbmax)?Double.NaN:xbmax;
			pext_trades[cur_pos]=xt_atmax;
		    }
		}
		if(++cur_pos>=nt)
		{
		    break gloop;
		}
		lev=price_thresholds[cur_pos];
	    }
	    if(pb<=xbmin)
	    {
		x_atmin=p;
		xbmin=pb;
		xt_atmin=p_trades[i];
	    }
	    if(pb>=xbmax)
	    {
		x_atmax=p;
		xbmax=pb;
		xt_atmax=p_trades[i];
	    }
	}
	for(;cur_pos<nt;cur_pos++)
	{
	    if(cur_pos>=0)
	    {
		if(flag_lower)
		{
		    pext_levs[cur_pos]=x_atmin;
		    pext_book[cur_pos]=Double.isInfinite(xbmin)?Double.NaN:xbmin;
		    pext_trades[cur_pos]=xt_atmin;
		}
		else
		{
		    pext_levs[cur_pos]=x_atmax;
		    pext_book[cur_pos]=Double.isInfinite(xbmax)?Double.NaN:xbmax;
		    pext_trades[cur_pos]=xt_atmax;
		}
	    }
	}

	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    // expensive constrains
	    for(int c=0;c<nt;c++)
	    {
		final double cut=price_thresholds[c];
		double xlev=Double.NaN,xb=flag_lower?Double.POSITIVE_INFINITY:Double.NEGATIVE_INFINITY,xt=Double.NaN;
		double prev_price=flag_lower?Double.POSITIVE_INFINITY:Double.NEGATIVE_INFINITY;
		for(int i=0;i<n_data_points;i++)
		{
		    final double x=p_levs[i];
		    if(flag_lower?prev_price<=x:prev_price>=x)
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    prev_price=x;
		    if(flag_lower?!(x>=cut):!(x<=cut))
		    {
			break;
		    }
		    final double v=p_book[i];
		    if(flag_lower)
		    {
			if(v<=xb)
			{
			    xlev=x;
			    xb=v;
			    xt=p_trades[i];
			}
		    }
		    else
		    {
			if(v>=xb)
			{
			    xlev=x;
			    xb=v;
			    xt=p_trades[i];
			}
		    }
		}
		if(Double.isInfinite(xb))
		{
		    xb=Double.NaN;
		}
		if(Double.isNaN(xlev)!=Double.isNaN(pext_levs[c])||
		   (!Double.isNaN(xlev) && xlev!=pext_levs[c])||
		   Double.isNaN(xb)!=Double.isNaN(pext_book[c])||
		   (!Double.isNaN(xb) && xb!=pext_book[c])||
		   Double.isNaN(xt)!=Double.isNaN(pext_trades[c])||
		   (!Double.isNaN(xt) && xt!=pext_trades[c]))
		{
		     throw new RuntimeException(
			"Mismatch:"+
			" nt="+nt+
			" n_data_points="+n_data_points+
			" xlev="+xlev+
			" pext_levs["+c+"]="+pext_levs[c]+
			" xb="+xb+
			" pext_book["+c+"]="+pext_book[c]+
			" xt="+xt+
			" pext_trades["+c+"]="+pext_trades[c]+
			" cut="+cut+
			" flag_lower="+flag_lower+
			" currenttime="+currenttime);
		}
	    }
	}
    }


    private static void calculatePresetLevels(final long currenttime,
					      final boolean PRINT_DATA_DEBUG,
					      final boolean flag_lower,
					      final int n_data_points,
					      final double [] p_levs,
					      final double [] p_book,
					      final long [] tau_book,
					      final double [] p_trades,
					      final long [] tau_trades,
					      final double [] price_thresholds,

					      // output parameters
					      final double [] price_book_bweighted,
					      final double [] price_trades_bweighted)
    {
	final int nt=price_thresholds.length;
	double xb=Double.NaN,xt=Double.NaN,lev=Double.NaN;
	int cur_pos=-1;
        gloop:for(int i=0;i<n_data_points;i++)
	{
	    final double p=p_levs[i];
	    for(;flag_lower?!(p>=lev):!(p<=lev);)
	    {
		if(cur_pos>=0)
		{
		    if(false || PRINT_DATA_DEBUG)
		    {
			final int il=i-1;
			if(tau_book[il]>tau_trades[il])
			{
			    System.err.println(
				"calculatePresetLevels: time inversion"+
				" N="+il+
				" N_total="+n_data_points+
				" p_lev="+p_levs[il]+
				" p_book="+p_book[il]+
				" xb="+xb+
				" t_book="+tau_book[il]+
				" p_trades="+p_trades[il]+
				" xt="+xt+
				" t_trades="+tau_trades[il]+
				" flag_lower="+flag_lower+
				" currenttime="+currenttime);
			}
		    }
		    price_book_bweighted[cur_pos]=xb;
		    price_trades_bweighted[cur_pos]=xt;
		}
		if(++cur_pos>=nt)
		{
		    break gloop;
		}
		lev=price_thresholds[cur_pos];
	    }
	    xb=p_book[i];
	    xt=p_trades[i];
	}
	for(;cur_pos<nt;cur_pos++)
	{
	    if(cur_pos>=0)
	    {
		price_book_bweighted[cur_pos]=xb;
		price_trades_bweighted[cur_pos]=xt;
	    }
	}
    }


    private static int getMMEElev(final long currenttime,
				  final double pcurrent,
				  final boolean PRINT_DATA_DEBUG,
				  final boolean flag_lower,
				  final int n_data_points,
				  final double [] p_levs,
				  final double [] p_book,
				  final double [] p_trades)
    {
	final double ll=pcurrent+(flag_lower?-OrdersFlow.EPS:OrdersFlow.EPS);
	int ptr=-1;
	for(int i=0;i<n_data_points;i++)
	{
	    if(flag_lower?!(p_book[i]<ll):!(p_book[i]>ll))
	    {
		ptr=i;
	    }
	    else
	    {
		break;
	    }
	}
	return ptr;
    }


    private static int getMMCutEElev(final long currenttime,
				     final double pcurrent,
				     final boolean PRINT_DATA_DEBUG,
				     final boolean flag_lower,
				     final double FRACTOMAX_ACCEPT,
				     final double FRAC_FORCEACCEPT,
				     final int n_data_points,
				     final double [] p_levs,
				     final double [] p_book,
				     final double [] p_trades)
    {
	final double lev_forceaccept=(flag_lower?1:-1)*
	    (pcurrent*FRAC_FORCEACCEPT+OrdersFlow.EPS);
	int ptr=-1;
	for(int i=0;i<n_data_points;i++)
	{
	    final double pd=pcurrent-p_levs[i];
	    final double d=p_book[i]-p_trades[i];
	    //final double d=p_book[i]-p_levs[i];
	    if(Double.isNaN(d)&&(flag_lower?pd<=lev_forceaccept:pd>=lev_forceaccept))
	    {
		ptr=i;
		continue;
	    }
	    if(flag_lower?
	       !(d<=Math.max(pd*FRACTOMAX_ACCEPT,lev_forceaccept)):
	       !(d>=Math.min(pd*FRACTOMAX_ACCEPT,lev_forceaccept)))
	    {
		break;
	    }
	    ptr=i;
	}
	return ptr;
    }





    private static int getMMCutEElev2(final long currenttime,
				      final double pcurrent,
				      final double [] price_thresholds,
				      final boolean PRINT_DATA_DEBUG,
				      final boolean flag_lower,
				      final int n_data_points,
				      final double [] p_levs,
				      final double [] p_book,

				       // output parameters
				      final double [] price_conv)
    {
	final int nt=price_thresholds.length;
	int cur_pos=-1,ptr=-1;
	double lev=Double.NaN,dlev=0;
        gloop:for(int i=0;i<n_data_points;i++)
	{
	    final double p=p_levs[i];
	    for(;flag_lower?!(p>=lev):!(p<=lev);)
	    {
		if(cur_pos>=0)
		{
		    price_conv[cur_pos]=ptr<0?Double.NaN:p_levs[ptr];
		}
		if(++cur_pos>=nt)
		{
		    break gloop;
		}
		lev=price_thresholds[cur_pos];
	    }
            // d2 convexity test P_cut+P_0 -2P_book <>0
	    final double d2=p+pcurrent-2.0*p_book[i];
	    if(flag_lower?d2>=dlev:d2<=dlev)
	    {
		dlev=d2;
		ptr=i;
	    }
	}
	for(;cur_pos<nt;cur_pos++)
	{
	    if(cur_pos>=0)
	    {
		price_conv[cur_pos]=ptr<0?Double.NaN:p_levs[ptr];
	    }
	}
	return ptr;
    }


    private static int findBookInversion(final long currenttime,
					 final boolean PRINT_DATA_DEBUG,
					 final boolean flag_not_inv,
					 final long tau,
					 final int n_data_points,
					 final double [] p_levs,
					 final long [] book_tau)
    {
	int ptr=-1;
	if(!flag_not_inv)
	{
	    // inversion
	    for(int i=0;i<n_data_points;i++)
	    {
		if(book_tau[i]>=tau)
		{
		    ptr=i;
		}
		else
		{
		    break;
		}
	    }
	}
	else
	{
	    // anti-inversion
	    for(int i=0;i<n_data_points;i++)
	    {
		if(book_tau[i]<=tau)
		{
		    ptr=i;
		}
		else
		{
		    break;
		}
	    }
	}
	return ptr;
    }



   static double calculateInversionPerLevel(final double [] price_levels,
					    final int nt,
					    final long currenttime,
					    final double pcurrent_buy,
					    final double pcurrent_sell,
					    final Distribution<BookItem> book,
					    final boolean PRINT_DATA_DEBUG,
					    final boolean flag_lower,
					    final int n_total_levels,
					    // output parameters
					    final double [] book_levels,
					    final long [] vb_levels,
					    final long [] tau_levels,
					    final double [] inversion_length,
					    final double [] rinversion_length,
					    final double [] sinversion_length,

					    // output array of size 3
					    final double [] pinnvs,

					    // temporary arrays
					    final int [] ptr_book,
					    final long [] buf_v,
					    final long [] buf_sv,
					    final int [] ptr_levels)
{
    final int bsize=book.getSize();
    final BookItem [] bb=book.getBuffer();
       CalculateLiquidityDeficit3.calculateBookLevelsTau(
	   currenttime,
	   n_total_levels,
	   PRINT_DATA_DEBUG,
	   0,
	   bb,
	   bsize,
	   flag_lower,
	   ptr_book,
	   buf_v,
	   buf_sv);

    // pinv- inversion level.
    double pinv=flag_lower?pcurrent_buy:pcurrent_sell,pinv2=pinv,pinv3=pinv;
    long v=0,sv=0,tau_pp=-1,tau_lpp=-1;
    boolean fpinv=true,fpinv2=true,fpinv3=true;
    for(int i=0;i<n_total_levels && (fpinv || fpinv2 || fpinv3) ;i++)
    {
	final long vx=buf_v[i],svx=buf_sv[i];
	final long tau_l=vx>0?((svx/vx)*ModelParameters.T_SUMM_SCALE):-1;
	if(fpinv3)
	{
	    if(tau_l>=tau_lpp)
	    {
		pinv3=bb[ptr_book[i]].price;
	    }
	    else
	    {
		fpinv3=false;
	    }
	}
	tau_lpp=tau_l;
	if(fpinv2)
	{
	    if(tau_l>=tau_pp)
	    {
		pinv2=bb[ptr_book[i]].price;
	    }
	    else
	    {
		fpinv2=false;
	    }
	}
	v+=vx;
	sv+=svx;
	//System.err.println("I4="+i+" vx="+vx+" svx="+svx+" v="+v+" sv="+sv+" p="+bb[ptr_book[i]].price+" flag_lower="+flag_lower);
	final long tau_p=v>0?((sv/v)*ModelParameters.T_SUMM_SCALE):-1;
	if(fpinv)
	{
	    if(tau_l>=tau_p)
	    {
		pinv=bb[ptr_book[i]].price;
	    }
	    else
	    {
		fpinv=false;
	    }
	}
	tau_pp=tau_p;
    }
    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
    {
	// it should be pinv=pinv2 for exact arithmetics,
	// but because of integer modulo divison
	// we have the relations:
	// pinv<=pinv2 for flag_lower=true
	// and pinv>=pinv2 for flag_lower=false
	if(Double.isNaN(pinv)!=Double.isNaN(pinv2)||
	   (!Double.isNaN(pinv) && !(flag_lower?pinv<=pinv2:pinv>=pinv2)))
	{
	     throw new RuntimeException(
		"Book inversion mismatch:"+
		" pinv="+pinv+
		" pinv2="+pinv2+
		" pinv3="+pinv3+
		" N_ORDERS="+book.getSize()+
		" flag_lower="+flag_lower+
		" currenttime="+currenttime);
	}
    }

    pinnvs[0]=pinv;
    pinnvs[1]=pinv2;
    pinnvs[2]=pinv3;

    double lev=Double.NaN;
    int cur_pos=-1,pb=-1;
    v=sv=0;
    gloop:for(int i=0;i<n_total_levels;i++)
    {
	final double p=bb[ptr_book[i]].price;
	for(;flag_lower?!(p>=lev):!(p<=lev);)
	{
	    if(cur_pos>=0)
	    {
		vb_levels[cur_pos]=v;
		tau_levels[cur_pos]=sv;
		ptr_levels[cur_pos]=pb;
	    }
	    if(++cur_pos>=nt)
	    {
		break gloop;
	    }
	    lev=price_levels[cur_pos];
	}
	v+=buf_v[i];
	sv+=buf_sv[i];
	pb=i;
    }
    for(;cur_pos<nt;cur_pos++)
    {
	if(cur_pos>=0)
	{
	    vb_levels[cur_pos]=v;
	    tau_levels[cur_pos]=sv;
	    ptr_levels[cur_pos]=pb;
	}
    }
    // normalization
    for(int i=0;i<nt;i++)
    {
	final long tau;
	if(vb_levels[i]>0)
	{
	    tau=tau_levels[i]=(tau_levels[i]/vb_levels[i])*ModelParameters.T_SUMM_SCALE;
	    book_levels[i]=bb[ptr_book[ptr_levels[i]]].price;

	}
	else
	{
	    tau=tau_levels[i]=-1;
	    book_levels[i]=Double.NaN;
	}

	// scan book portion
	double dp=0,rdp=0,rldp=0,pp=flag_lower?pcurrent_sell:pcurrent_buy;
	long vv=0,svv=0,tprev=-1,tlprev=-1;
	for(int j=0,ptrl=ptr_levels[i];j<=ptrl;j++)
	{
	    final double pl=bb[ptr_book[j]].price;
	    final long vx=buf_v[j],svx=buf_sv[j];
	    if(vx>0)
	    {
		final double pd=pl-pp;
		final long t=(svx/vx)*ModelParameters.T_SUMM_SCALE;
		if(t>tau)
		{
		    dp+=pd;
		}
		if(t<tprev)
		{
		    rdp+=pd;
		}
		if(t<tlprev)
		{
		    rldp+=pd;
		}
		tlprev=t;
	    }
	    else
	    {
		tlprev=-1;
	    }
	    pp=pl;
	    vv+=vx;
	    svv+=svx;
	    if(vv>0)
	    {
		tprev=(svv/vv)*ModelParameters.T_SUMM_SCALE;
	    }
	    else
	    {
		tprev=-1;
	    }
	}
	if(vv!=vb_levels[i]||tprev!=tau_levels[i])
	{
	    throw new RuntimeException(
		"Book mismatch:"+
		" vv="+vv+
		" vb_levels["+i+"]="+vb_levels[i]+
		" tprev="+tprev+
		" tau_levels["+i+"]="+tau_levels[i]+
		" N_ORDERS="+book.getSize()+
		" flag_lower="+flag_lower+
		" currenttime="+currenttime);
	}
	inversion_length[i]=dp;
	rinversion_length[i]=rdp;
	sinversion_length[i]=rldp;
    }
    return pinv;
}




}
