package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit ver 5.
 *  @version $Id: CalculateLiquidityDeficit5.java,v 1.35 2020/02/06 00:09:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit5 {

    private final Distribution<BookItem> orders,tmpordersbuf;
    private final Distribution<Trade> trades;
    private final boolean flag_lower;

    CalculateLiquidityDeficit5(final Distribution<BookItem> orders,
			       final Distribution<Trade> trades,
			       final boolean flag_lower,
			       final Distribution<BookItem> tmpordersbuf,
			       final Distribution<Trade> tmptradesbuf)
    {
	this.orders=orders;
	this.trades=trades;
	this.flag_lower=flag_lower;
	this.tmpordersbuf=tmpordersbuf;
    }

    long [] vol_orders=null,tau_orders=null;
    double []
	price_orders_levels=null,
	price_aver_t_orders=null,
	price_aver_t_trades=null;

    void clear()
    {

	this.vol_orders=this.tau_orders=null;
	this.price_orders_levels=
	this.price_aver_t_orders=
	this.price_aver_t_trades=null;
    }


    void calculateLiquidityDeficit(final double [] price_levels,
				   final long currenttime,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final long trades_cleared_before)
    {
	final boolean PRINT_DATA_DEBUG=false;
	final int nt=price_levels.length;
	final int bsize=this.orders.getSize(),tsize=this.trades.getSize();

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tN_ORDERS\t"+this.orders.getSize()+
			       "\tflag_lower\t"+this.flag_lower+
			       "\nLEVELS\t"+nt+
			       "\ncur_buy\t"+pcurrent_buy+
			       "\tcur_sell\t"+pcurrent_sell);
	}

	this.vol_orders=new long [nt];
	this.tau_orders=new long [nt];
	this.price_orders_levels=new double [nt];
	this.price_aver_t_orders=new double [nt];
	this.price_aver_t_trades=new double [nt];
	final int [] ptr_book=new int [nt],tmp=new int [nt];

	final BookItem [] bb=this.tmpordersbuf.getBuffer();
	System.arraycopy(this.orders.getBuffer(),0,bb,0,bsize);
	final Trade [] tb=this.trades.getBuffer();
	final boolean flag_lower=this.flag_lower;


	calculateBookLevelsStat(price_levels,
				currenttime,
				flag_lower,
				bb,
				bsize,
				this.vol_orders,
				new long [nt], // temporary array
				this.tau_orders,
				ptr_book,
				this.price_aver_t_orders,
				tmp);


	// scan book portions
	for(int p=0;p<nt;p++)
	{
	    int ptrt=tsize-1;
	    double price_t=Double.NaN,pta=0;
	    // order array by time.
	    // the buffer is considered poisoned after this.
	    final int bptr=ptr_book[p];
	    // sort book orders in "latest order is in the beginning" direction,
	    // which is opposite to how we typically sort orders & trades
	    // (CompareItems.cmp_by_time comparator).
	    // this choice is to avoid quicksort "worst case" input data
	    com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(bb,0,bptr,CompareItems.cmp_by_time_latest_at_0);
	    long vb=0,svb=0,prevtb=Long.MAX_VALUE,prevtt=Long.MAX_VALUE;
	    for(int i=0;i<=bptr;i++)
	    {
		final BookItem b=bb[i];
		if(b.time>prevtb)
		{
		    throw new IllegalArgumentException("Book sorting error");
		}
		prevtb=b.time;
	        if(ptrt>=0)
		{
		    // find the closest trade
		    for(;;)
		    {
			final Trade t=tb[ptrt];
			if(t.time>prevtt)
			{
			    throw new IllegalArgumentException("Trades sorting error");
			}
			prevtt=t.time;
			if(prevtt<prevtb)
			{
			    price_t=t.price;
			    break;
			}
			if(--ptrt<0)
			{
			    price_t=tsize>0?tb[0].price:Double.NaN;
			    break;
			}
		    }
		}
		final long s=b.shares;
		vb+=s;
		svb+=s*((currenttime-prevtb)/ModelParameters.T_SUMM_SCALE);
		final double pxx=((flag_lower?price_t<=b.price:price_t>=b.price)?b.price:price_t);
		pta+=s*pxx /* price_t */;
	    }
	    if(vb>0)
	    {
		this.price_aver_t_trades[p]=pta/vb;
		svb=(svb/vb)*ModelParameters.T_SUMM_SCALE;
	    }
	    else
	    {
		this.price_aver_t_trades[p]=Double.NaN;
		svb=-1;
	    }
	    if(this.vol_orders[p]!=vb||
	       this.tau_orders[p]!=svb)
	    {
		throw new RuntimeException(
		    "Book data mismatch:"+
		    " vol_orders["+p+"]="+this.vol_orders[p]+
		    " vb="+vb+
		    " tau_orders["+p+"]="+this.tau_orders[p]+
		    " svb="+svb+
		    " ptr_book["+p+"]="+ptr_book[p]+
		    " price_aver_t_orders["+p+"]="+this.price_aver_t_orders[p]+
		    " price_aver_t_trades["+p+"]="+this.price_aver_t_trades[p]+
		    " bsize="+bsize+
		    " tsize="+tsize+
		    " flag_lower="+flag_lower+
		    " currenttime="+currenttime);
	    }
	}


	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    // expensive constrains
	     final CalculateBookVolumePriceFixedThreshold
		 ft=new CalculateBookVolumePriceFixedThreshold(flag_lower,this.orders);
	     ft.calculateThresholds(currenttime,
				    pcurrent_buy,
				    pcurrent_sell,
				    price_levels);
	     for(int i=0;i<nt;i++)
	     {
		 if(this.vol_orders[i]!=ft.vol_threshold_vcalc[i]||
		    this.tau_orders[i]!=ft.tau_threshold_vcalc[i]||
		    Double.isNaN(this.price_aver_t_orders[i])!=Double.isNaN(ft.paver[i])||
		    (!Double.isNaN(this.price_aver_t_orders[i]) &&
		     this.price_aver_t_orders[i]!=ft.paver[i])
		    )
		 {
		     throw new RuntimeException(
			 "Book tau mismatch:"+
			 " vol_orders["+i+"]="+this.vol_orders[i]+
			 " ft.vol_threshold_vcalc["+i+"]="+ft.vol_threshold_vcalc[i]+
			 " tau_orders["+i+"]="+this.tau_orders[i]+
			 " ft.tau_threshold_vcalc["+i+"]="+ft.tau_threshold_vcalc[i]+
			 " price_aver_t_orders["+i+"]="+this.price_aver_t_orders[i]+
			 " ft.paver["+i+"]="+ft.paver[i]+
			 " bsize="+bsize+
			 " tsize="+tsize+
			 " flag_lower="+flag_lower+
			 " currenttime="+currenttime);
		 }
	     }
	}
    }




    static void calculateBookLevelsStat(final double [] price_levels,
					final long currenttime,
					final boolean flag_lower,
					final BookItem [] bb,
					final int bsize,

					// output parameters
					final long [] vol_orders,
					final long [] tau_orders_unnormalized,
					final long [] tau_orders,
					final int [] ptr_book,
					final double [] price_aver_orders,
					final int [] n_pricelevels)
    {
	final int nt=price_levels.length;

	double lev=Double.NaN,pba=0,prevp=flag_lower?Double.POSITIVE_INFINITY:Double.NEGATIVE_INFINITY,pl=Double.NaN;
	int cur_pos=-1,ptrb=-1,n_plevels=0;
	long v=0,sv=0,prevt=-1;
        bloop:for(int i=0;i<bsize;i++)
	{
	    final BookItem b=bb[i];
	    if(flag_lower?b.price>prevp:b.price<prevp)
	    {
		throw new IllegalArgumentException("Book price sorting error.");
	    }
	    /* if(b.price==prevp && b.time<prevt)
	    {
		throw new IllegalArgumentException("Book time ordering is broken.");
	    }
	    */
	    prevt=b.time;
	    prevp=b.price;
	    for(;flag_lower?!(prevp>=lev):!(prevp<=lev);)
	    { // next level
		if(cur_pos>=0)
		{
		    vol_orders[cur_pos]=v;
		    tau_orders_unnormalized[cur_pos]=sv;
		    price_aver_orders[cur_pos]=pba;
		    ptr_book[cur_pos]=ptrb;
		    n_pricelevels[cur_pos]=n_plevels;
		}
		if(++cur_pos>=nt)
		{
		    break bloop;
		}
		lev=price_levels[cur_pos];
	    }
	    final long s=b.shares;
	    v+=s;
	    sv+=s*((currenttime-b.time)/ModelParameters.T_SUMM_SCALE);
	    pba+=s*prevp;
	    ptrb=i;
	    if(pl!=prevp)
	    {
		n_plevels++;
		pl=prevp;
	    }
	}
	for(;cur_pos<nt;cur_pos++)
	{
	    if(cur_pos>=0)
	    {
		vol_orders[cur_pos]=v;
		tau_orders_unnormalized[cur_pos]=sv;
		price_aver_orders[cur_pos]=pba;
		ptr_book[cur_pos]=ptrb;
		n_pricelevels[cur_pos]=n_plevels;
	    }
	}

	// normalize book data
	for(int i=0;i<nt;i++)
	{
	    final long vv=vol_orders[i];
	    if(vv>0)
	    {
		tau_orders[i]=(tau_orders_unnormalized[i]/vv)*ModelParameters.T_SUMM_SCALE;
		price_aver_orders[i]=price_aver_orders[i]/vv;
	    }
	    else
	    {
		tau_orders[i]=-1;
		price_aver_orders[i]=Double.NaN;
	    }
	}

    }
}
