package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit ver 6.
 *  @version $Id: CalculateLiquidityDeficit6.java,v 1.127 2010/01/24 22:13:14 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit6 {

    private static final boolean SUMM_WITH_BOOK_LEVEL0=true;

    private final Distribution<BookItem> buy,sell,tmpordersbuf;
    private final Distribution<Trade> trades,tmptradesbuf;

    CalculateLiquidityDeficit6(final Distribution<BookItem> buy,
			       final Distribution<BookItem> sell,
			       final Distribution<Trade> trades,
			       final Distribution<BookItem> tmpordersbuf,
			       final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
	this.tmpordersbuf=tmpordersbuf;
	this.tmptradesbuf=tmptradesbuf;
    }

    long t_cont_iE_buy=-1,t_cont_iE_sell=-1,t_cont_iE3_buy=-1,t_cont_iE3_sell=-1,t_max_cont_iE3_buy=-1,t_max_cont_iE3_sell=-1;

    double lev_iE_buy=Double.NaN,
	lev_iE_sell=Double.NaN;
    double
	lev_iE3_buy=Double.NaN,
	lev_iE3_sell=Double.NaN;
    double []
	price_book_buy=null,
	price_book_sell=null;

    double []
	price_book_level_buy=null,
	price_book_split_buy=null,
	price_book_level_sell=null,
	price_book_split_sell=null;


    long []
	tau_trades_buy=null,
	tau_trades_sell=null,
	v_trades_buy=null,
	v_trades_sell=null,
	tau_book_acc_buy=null,
	tau_book_acc_sell=null;
    double [] pinv_ttrades_buy=null,pinv_ttrades_sell=null;

    long tbuy0_min=-1,tbuy0_max=-1,tsell0_min=-1,tsell0_max=-1;
    double pi0_tmin_buy=Double.NaN,pi0_tmax_buy=Double.NaN,pi0_tmin_sell=Double.NaN,pi0_tmax_sell=Double.NaN;

    void clear()
    {
	this.lev_iE_buy=this.lev_iE_sell=this.lev_iE3_buy=this.lev_iE3_sell=Double.NaN;
	this.t_cont_iE_buy=
	this.t_cont_iE_sell=
	this.t_cont_iE3_buy=
	this.t_cont_iE3_sell=
        this.t_max_cont_iE3_buy=
        this.t_max_cont_iE3_sell=-1;

	this.tau_trades_buy=
	this.tau_trades_sell=
	this.v_trades_buy=
	this.v_trades_sell=
	this.tau_book_acc_buy=
	this.tau_book_acc_sell=null;
	this.price_book_buy=
	this.price_book_sell=
	this.pinv_ttrades_buy=
	this.pinv_ttrades_sell=
	this.price_book_level_buy=
	this.price_book_split_buy=
	this.price_book_level_sell=
	this.price_book_split_sell=null;


	this.tbuy0_min=this.tbuy0_max=this.tsell0_min=this.tsell0_max=-1;
	this.pi0_tmin_buy=this.pi0_tmax_buy=this.pi0_tmin_sell=this.pi0_tmax_sell=Double.NaN;

    }


    private static void calculateTradesTau(final long currenttime,
					   final double pcurrent_buy,
					   final double pcurrent_sell,
					   final int nt,
					   final long trades_cleared_before,
					   final boolean PRINT_DATA_DEBUG,

					   final boolean flag_lower,
					   final Distribution<Trade> trades,
					   final double [] p0,
					   final double pexit,

					   // output parameters
					   final long [] vb,
					   final long [] svb_not_normalized_levels)
    {
	final double pexitlev=flag_lower?pexit-OrdersFlow.EPS:pexit+OrdersFlow.EPS;
	final int tsize=trades.getSize();
	final Trade [] tb=trades.getBuffer();

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println(" pexitlev="+pexitlev+
			       " tsize="+tsize+
			       " flag_lower="+flag_lower+
			       " nt="+nt);
	    for(int i=0;i<nt;i++)
	    {
		System.err.println("lev\t"+i+"\t=\t"+p0[i]);
	    }
	    System.err.println("num\t"+
			       "time\t"+
			       "origtime\t"+
			       "shares\t"+
			       "tradetype\t"+
			       "buy_sell_indicator\t"+
			       "price\t"+
			       "matchnumber\t"+
			       "orderreferencenumber");
	}

	long v=0,sv=0,prevt=Long.MAX_VALUE;
	int n_levels=-1;
	double p0lev=Double.NaN;
        gloop:for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;

	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println(String.valueOf(i)+"\t"+
				   t.time+"\t"+
				   t.origtime+"\t"+
				   t.shares+"\t"+
				   t.tradetype+"\t"+
				   t.buy_sell_indicator+"\t"+
				   t.price+"\t"+
				   t.matchnumber+"\t"+
				   t.orderreferencenumber);
	    }

	    if(t.origtime<=0)
	    {
		continue gloop;
	    }
	    if(flag_lower)
	    {
		if(t.buy_sell_indicator!=BookItem.BOOK_BUY)
		{
		    continue gloop;
		}
		final double p=t.price;
		if(!(p>=pexitlev))
		{
		    // way off, got poison
		    v=sv=0;
		    break gloop;
		}
		for(;!(p<=p0lev);)
		{
		    if(n_levels>=0)
		    {
			vb[n_levels]=v;
			svb_not_normalized_levels[n_levels]=sv;
		    }
		    if(++n_levels>=nt)
		    {
			break gloop;
		    }
		    if((n_levels>0 && p0[n_levels]<=p0[n_levels-1])||
		       p0[n_levels]<pexit)
		    {
			throw new IllegalArgumentException(
			    "Unordered levels:"+
			    " n_levels="+n_levels+
			    " p0["+n_levels+"]="+p0[n_levels]+
			    " p0["+(n_levels-1)+"]="+
			    (n_levels>0?String.valueOf(p0[n_levels-1]):"out or range")+
			    " pexit="+pexit+
			    " N_TRADES="+trades.getSize()+
			    " flag_lower="+flag_lower+
			    " currenttime="+currenttime);
		    }
		    p0lev=p0[n_levels]+OrdersFlow.EPS;
		    if(Double.isNaN(p0lev))
		    {
			// nothing more to watch
			v=sv=0;
			break gloop;
		    }
		}
		v+=t.shares;
		final long tte=(t.time-t.origtime)/ModelParameters.T_SUMM_SCALE;
		sv+=t.shares*tte;
	    }
	    else
	    {
		if(t.buy_sell_indicator!=BookItem.BOOK_SELL)
		{
		    continue gloop;
		}
		final double p=t.price;
		if(!(p<=pexitlev))
		{
		    // way off, got poison
		    v=sv=0;
		    break gloop;
		}
		for(;!(p>=p0lev);)
		{
		    if(n_levels>=0)
		    {
			vb[n_levels]=v;
			svb_not_normalized_levels[n_levels]=sv;
		    }
		    if(++n_levels>=nt)
		    {
			break gloop;
		    }
		    if((n_levels>0 && p0[n_levels]>=p0[n_levels-1])||
		       p0[n_levels]>pexit)
		    {
			throw new IllegalArgumentException(
			    "Unordered levels:"+
			    " n_levels="+n_levels+
			    " p0["+n_levels+"]="+p0[n_levels]+
			    " p0["+(n_levels-1)+"]="+
			    (n_levels>0?String.valueOf(p0[n_levels-1]):"out or range")+
			    " pexit="+pexit+
			    " N_TRADES="+trades.getSize()+
			    " flag_lower="+flag_lower+
			    " currenttime="+currenttime);
		    }
		    p0lev=p0[n_levels]-OrdersFlow.EPS;
		    if(Double.isNaN(p0lev))
		    {
			// nothing more to watch
			v=sv=0;
			break gloop;
		    }
		}
		v+=t.shares;
		final long tte=(t.time-t.origtime)/ModelParameters.T_SUMM_SCALE;
		sv+=t.shares*tte;
	    }
	}

	// save remaining results
	for(;n_levels<nt;n_levels++)
	{
	    if(n_levels>=0)
	    {
		if(Double.isNaN(p0[n_levels])||Double.isNaN(pexitlev))
		{
		    v=sv=0;
		}
		vb[n_levels]=v;
		svb_not_normalized_levels[n_levels]=sv;
	    }
	}
    }


    void calculateLiquidityDeficit(final long currenttime,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final int n_levels_trades,
				   final long trades_cleared_before)
    {
	final boolean PRINT_DATA_DEBUG=false;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tN_ORDERS BUY\t"+this.buy.getSize()+
			       "\tN_ORDERS SELL\t"+this.sell.getSize()+
			       "\tn_levels_trades\t"+n_levels_trades+
			       "\ncur_buy\t"+pcurrent_buy+
			       "\tcur_sell\t"+pcurrent_sell);
	}

	final int nt=n_levels_trades,bsizebuy=this.buy.getSize(),bsizesell=this.sell.getSize();
	final BookItem [] bbuy=this.buy.getBuffer(),bsell=this.sell.getBuffer();
	final int [] tmpd=new int [nt];


	final int [] ptr_book_buy=new int [nt];
	final long [] vb_buy=new long [nt],sv_nn_buy=new long [nt];
	CalculateLiquidityDeficit3.calculateBookLevelsTau(
	    currenttime,
	    nt,
	    PRINT_DATA_DEBUG,
	    0,
	    bbuy,
	    bsizebuy,
	    true,

	    // output:
	    ptr_book_buy,
	    vb_buy, // volume per level (not accumulated)
	    sv_nn_buy // unnormalized tau (not accumulated)
	);


	final int [] ptr_book_sell=new int [nt];
	final long [] vb_sell=new long [nt],sv_nn_sell=new long [nt];
	CalculateLiquidityDeficit3.calculateBookLevelsTau(
	    currenttime,
	    nt,
	    PRINT_DATA_DEBUG,
	    0,
	    bsell,
	    bsizesell,
	    false,

	    // output:
	    ptr_book_sell,
	    vb_sell, // volume per level (not accumulated)
	    sv_nn_sell  // unnormalized tau (not accumulated)
        );

	// zero level adjustment, if enabled
	final long v0_toadd_buy,sv0_toadd_buy,v0_toadd_sell,sv0_toadd_sell;
	final int pb0_buy,pb0_sell;
	if(SUMM_WITH_BOOK_LEVEL0)
	{
	    v0_toadd_buy=vb_buy[0];
	    sv0_toadd_buy=sv_nn_buy[0];
	    pb0_buy=ptr_book_buy[0]+1;

	    v0_toadd_sell=vb_sell[0];
	    sv0_toadd_sell=sv_nn_sell[0];
	    pb0_sell=ptr_book_sell[0]+1;
	}
	else
	{
	    v0_toadd_buy=sv0_toadd_buy=v0_toadd_sell=sv0_toadd_sell=0;
	    pb0_buy=pb0_sell=0;
	}

	// exit level spread ajustment, currently not used
	final double d,s=pcurrent_sell-pcurrent_buy;
	if(s<0)
	{
	    d=0;
	    System.err.println(this.getClass().getName()+
			       ":Warning: negative spread:"+
			       " pcurrent_sell="+pcurrent_sell+
			       " pcurrent_buy="+pcurrent_buy+
			       " nt="+nt+
			       " N_SELL="+this.sell.getSize()+
			       " N_TRADES="+this.trades.getSize()+
			       " currenttime="+currenttime);
	}
	else
	{
	    d=SPREAD_FACTOR*s;
	}

	// determine price levels from pointer to a book
	this.price_book_buy=new double [nt];
	this.price_book_sell=new double [nt];
	for(int i=0;i<nt;i++)
	{
	    this.price_book_buy[i]=ptr_book_buy[i]>=0?bbuy[ptr_book_buy[i]].price:Double.NaN;
	    this.price_book_sell[i]=ptr_book_sell[i]>=0?bsell[ptr_book_sell[i]].price:Double.NaN;
	}

	// buy trades tau
	this.v_trades_buy=new long [nt];
	this.tau_trades_buy=new long [nt];
	calculateTradesTau(currenttime,
			   pcurrent_buy,
			   pcurrent_sell,
			   nt,
			   trades_cleared_before,
			   PRINT_DATA_DEBUG,
			   true,
			   this.trades,
			   this.price_book_sell, // sell book levels for buy trades
			   pcurrent_buy-d, // trades scanning exit level (min constrain)
			   this.v_trades_buy,
			   this.tau_trades_buy);

	// sell trades tau
	this.v_trades_sell=new long [nt];
	this.tau_trades_sell=new long [nt];
	calculateTradesTau(currenttime,
			   pcurrent_buy,
			   pcurrent_sell,
			   nt,
			   trades_cleared_before,
			   PRINT_DATA_DEBUG,
			   false,
			   this.trades,
			   this.price_book_buy, // buy book levels for sell trades
			   pcurrent_sell+d, // trades scanning exit level (max constrain)

			   this.v_trades_sell,
			   this.tau_trades_sell);


	final long [] save_unnorm_v_trades_buy,save_unnorm_sv_trades_buy,
	    save_unnorm_v_trades_sell,save_unnorm_sv_trades_sell;
	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // for expensive constrains
	    save_unnorm_v_trades_buy=this.v_trades_buy.clone();
	    save_unnorm_sv_trades_buy=this.tau_trades_buy.clone();
	    save_unnorm_v_trades_sell=this.v_trades_sell.clone();
	    save_unnorm_sv_trades_sell=this.tau_trades_sell.clone();
	}
	else
	{
	    save_unnorm_v_trades_buy=
	    save_unnorm_sv_trades_buy=
	    save_unnorm_v_trades_sell=
	    save_unnorm_sv_trades_sell=null;
	}

	// normalize data and calculate inversion level
	this.pinv_ttrades_buy=new double [nt];
	this.pinv_ttrades_sell=new double [nt];
	for(int i=0;i<nt;i++)
	{
	    // normalize buy
	    final int pb_buy;
	    if(this.v_trades_buy[i]>0)
	    {
		// add 0-th level from book
		this.v_trades_buy[i]+=v0_toadd_buy;
		this.tau_trades_buy[i]+=sv0_toadd_buy;
		this.tau_trades_buy[i]=
		    (this.tau_trades_buy[i]/this.v_trades_buy[i])*ModelParameters.T_SUMM_SCALE;
		pb_buy=pb0_buy;
	    }
	    else
	    {
		this.tau_trades_buy[i]=-1;
		pb_buy=0;
	    }
	    // buy inversion for t=this.tau_trades_buy[i]
	    CalculateLiquidityDeficit3.calculateBookInvertionPrice(
		currenttime,
		pb_buy,
		this.buy,
		true,
		this.tau_trades_buy[i],
		false,
		PRINT_DATA_DEBUG,
		i,

		this.pinv_ttrades_buy,
		tmpd);

	    // normalize sell
	    final int pb_sell;
	    if(this.v_trades_sell[i]>0)
	    {
		// add 0-th level from book
		this.v_trades_sell[i]+=v0_toadd_sell;
		this.tau_trades_sell[i]+=sv0_toadd_sell;
		this.tau_trades_sell[i]=
		    (this.tau_trades_sell[i]/this.v_trades_sell[i])*ModelParameters.T_SUMM_SCALE;
		pb_sell=pb0_sell;
	    }
	    else
	    {
		this.tau_trades_sell[i]=-1;
		pb_sell=0;
	    }
            // sell inversion for t=this.tau_trades_sell[i]
	    CalculateLiquidityDeficit3.calculateBookInvertionPrice(
		currenttime,
		pb_sell,
		this.sell,
		false,
		this.tau_trades_sell[i],
		false,
		PRINT_DATA_DEBUG,
		i,

		this.pinv_ttrades_sell,
		tmpd);

	}


	// buy: split price levels
	this.price_book_level_buy=new double [nt];
	this.price_book_split_buy=new double [nt];
	CalculateLiquidityDeficit3.accumulateBookTauBSplitFromLevels(
	    currenttime,
	    nt,
	    PRINT_DATA_DEBUG,
	    this.buy,
	    true,
	    ptr_book_buy,

	    // for accumulation, data read and saved in place
	    vb_buy, // accumulated volume on output
	    sv_nn_buy, // tau of accumulated book on output
	    // for output:
	    this.price_book_level_buy,
	    this.price_book_split_buy,
	    tmpd);
	// accumulated tte data is in sv_nn_buy
	this.tau_book_acc_buy=sv_nn_buy;

        // sell: split price levels
	this.price_book_level_sell=new double [nt];
	this.price_book_split_sell=new double [nt];
	CalculateLiquidityDeficit3.accumulateBookTauBSplitFromLevels(
	    currenttime,
	    nt,
	    PRINT_DATA_DEBUG,
	    this.sell,
	    false,
	    ptr_book_sell,

	    // for accumulation, data read and saved in place
	    vb_sell, // accumulated volume on output
	    sv_nn_sell, // tau of accumulated book on output
	    // for output:
	    this.price_book_level_sell,
	    this.price_book_split_sell,
	    tmpd);
	// accumulated tte data is in sv_nn_sell
	this.tau_book_acc_sell=sv_nn_sell;


	final double [] out_save_p=new double [2];
	final long [] out_save_taus=new long [3];
	// different inversion
	this.lev_iE_buy=calculateInversionPerLevel(currenttime,
						   pcurrent_buy,
						   pcurrent_sell,
						   this.buy,
						   PRINT_DATA_DEBUG,
						   true,
						   out_save_p,
						   out_save_taus);
	this.lev_iE3_buy=out_save_p[1];
	this.t_cont_iE_buy=out_save_taus[0];
	this.t_cont_iE3_buy=out_save_taus[1];
	this.t_max_cont_iE3_buy=out_save_taus[2];

	this.lev_iE_sell=calculateInversionPerLevel(currenttime,
						    pcurrent_buy,
						    pcurrent_sell,
						    this.sell,
						    PRINT_DATA_DEBUG,
						    false,
						    out_save_p,
						    out_save_taus);
	this.lev_iE3_sell=out_save_p[1];
	this.t_cont_iE_sell=out_save_taus[0];
	this.t_cont_iE3_sell=out_save_taus[1];
	this.t_max_cont_iE3_sell=out_save_taus[2];




	if(true)
	{

	    // buy
	    if(ptr_book_buy[0]>=0)
	    {
		this.tbuy0_min=currenttime-bbuy[ptr_book_buy[0]].time;
		this.tbuy0_max=currenttime-bbuy[0].time; // oldest is in the beginning
	    }
	    else
	    {
		this.tbuy0_min=this.tbuy0_max=-1;
	    }
	    if(this.tbuy0_max<this.tbuy0_min)
	    {
		throw new RuntimeException("Sorting problem");
	    }
	    CalculateLiquidityDeficit3.calculateBookInvertionPrice(
		currenttime,
		1+ptr_book_buy[0],
		this.buy,
		true,
		this.tbuy0_max,
		false,
		PRINT_DATA_DEBUG,
		0,
		out_save_p,
		tmpd);
	    this.pi0_tmax_buy=out_save_p[0];
	    CalculateLiquidityDeficit3.calculateBookInvertionPrice(
		currenttime,
		1+ptr_book_buy[0],
		this.buy,
		true,
		this.tbuy0_min,
		false,
		PRINT_DATA_DEBUG,
		0,
		out_save_p,
		tmpd);
	    this.pi0_tmin_buy=out_save_p[0];

	    // sell
	    if(ptr_book_sell[0]>=0)
	    {
		this.tsell0_min=currenttime-bsell[ptr_book_sell[0]].time;
		this.tsell0_max=currenttime-bsell[0].time; // oldest is in the beginning
	    }
	    else
	    {
		this.tsell0_min=this.tsell0_max=-1;
	    }
	    if(this.tsell0_max<this.tsell0_min)
	    {
		throw new RuntimeException("Sorting problem");
	    }
	    CalculateLiquidityDeficit3.calculateBookInvertionPrice(
		currenttime,
		1+ptr_book_sell[0],
		this.sell,
		false,
		this.tsell0_max,
		false,
		PRINT_DATA_DEBUG,
		0,
		out_save_p,
		tmpd);
	    this.pi0_tmax_sell=out_save_p[0];
	    CalculateLiquidityDeficit3.calculateBookInvertionPrice(
		currenttime,
		1+ptr_book_sell[0],
		this.sell,
		false,
		this.tsell0_min,
		false,
		PRINT_DATA_DEBUG,
		0,
		out_save_p,
		tmpd);
	    this.pi0_tmin_sell=out_save_p[0];
	}






	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // expensive constrains
	    if(ptr_book_buy[0]==0)
	    {
		final long t0=this.tau_book_acc_buy[0];
		final double p0=this.price_book_split_buy[0];
		if((this.tbuy0_max/ModelParameters.T_SUMM_SCALE)*ModelParameters.T_SUMM_SCALE!=t0||
		   (this.tbuy0_min/ModelParameters.T_SUMM_SCALE)*ModelParameters.T_SUMM_SCALE!=t0||
		   Double.isNaN(this.pi0_tmax_buy)!=Double.isNaN(p0)||
		   (!Double.isNaN(this.pi0_tmax_buy) && this.pi0_tmax_buy!=p0)||
		   Double.isNaN(this.pi0_tmin_buy)!=Double.isNaN(p0)||
		   (!Double.isNaN(this.pi0_tmin_buy) && this.pi0_tmin_buy!=p0))
		{
		    throw new RuntimeException("Buy pointer mismatch:"+
					       " t0="+t0+
					       " tbuy0_max="+this.tbuy0_max+
					       " tbuy0_min="+this.tbuy0_min+
					       " p0="+p0+
					       " pi0_tmax_buy="+this.pi0_tmax_buy+
					       " pi0_tmin_buy="+this.pi0_tmin_buy+
					       " N_BUY="+this.buy.getSize()+
					       " N_TRADES="+this.trades.getSize()+
					       " currenttime="+currenttime);
		}

	    }
	    if(ptr_book_sell[0]==0)
	    {
		final long t0=this.tau_book_acc_sell[0];
		final double p0=this.price_book_split_sell[0];
		if((this.tsell0_max/ModelParameters.T_SUMM_SCALE)*ModelParameters.T_SUMM_SCALE!=t0||
		   (this.tsell0_min/ModelParameters.T_SUMM_SCALE)*ModelParameters.T_SUMM_SCALE!=t0||
		   Double.isNaN(this.pi0_tmax_sell)!=Double.isNaN(p0)||
		   (!Double.isNaN(this.pi0_tmax_sell) && this.pi0_tmax_sell!=p0)||
		   Double.isNaN(this.pi0_tmin_sell)!=Double.isNaN(p0)||
		   (!Double.isNaN(this.pi0_tmin_sell) && this.pi0_tmin_sell!=p0))
		{
		    throw new RuntimeException("Sell pointer mismatch:"+
					       " t0="+t0+
					       " tsell0_max="+this.tsell0_max+
					       " tsell0_min="+this.tsell0_min+
					       " p0="+p0+
					       " pi0_tmax_sell="+this.pi0_tmax_sell+
					       " pi0_tmin_sell="+this.pi0_tmin_sell+
					       " N_SELL="+this.sell.getSize()+
					       " N_TRADES="+this.trades.getSize()+
					       " currenttime="+currenttime);
		}

	    }



	    if(SUMM_WITH_BOOK_LEVEL0)
	    { // alternative view check
		final long [] vtb_buy=new long [1],svtb_buy=new long [1];
		final int [] ptrt_buy=new int [1];
		CalculateLiquidityDeficit3.calculateBookLevelsTau(
		    currenttime,
		    1,
		    PRINT_DATA_DEBUG,
		    0,
		    bbuy,
		    bsizebuy,
		    true,
		    ptrt_buy,
		    vtb_buy,
		    svtb_buy);
		if(ptrt_buy[0]!=(pb0_buy-1)||
		   vtb_buy[0]!=v0_toadd_buy||
		   svtb_buy[0]!=sv0_toadd_buy)
		{
		    throw new RuntimeException("Buy pointer mismatch:"+
					       " ptrt_buy[0]="+ptrt_buy[0]+
					       " pb0_buy="+pb0_buy+
					       " vtb_buy[0]="+vtb_buy[0]+
					       " v0_toadd_buy="+v0_toadd_buy+
					       " svtb_buy[0]="+svtb_buy[0]+
					       " sv0_toadd_buy="+sv0_toadd_buy+
					       " nt="+nt+
					       " N_BUY="+this.buy.getSize()+
					       " N_TRADES="+this.trades.getSize()+
					       " currenttime="+currenttime);
		}
		final long [] vtb_sell=new long [1],svtb_sell=new long [1];
		final int [] ptrt_sell=new int [1];
		CalculateLiquidityDeficit3.calculateBookLevelsTau(
		    currenttime,
		    1,
		    PRINT_DATA_DEBUG,
		    0,
		    bsell,
		    bsizesell,
		    false,
		    ptrt_sell,
		    vtb_sell,
		    svtb_sell);
		if(ptrt_sell[0]!=(pb0_sell-1)||
		   vtb_sell[0]!=v0_toadd_sell||
		   svtb_sell[0]!=sv0_toadd_sell)
		{
		    throw new RuntimeException("Sell pointer mismatch:"+
					       " ptrt_sell[0]="+ptrt_sell[0]+
					       " pb0_sell="+pb0_sell+
					       " vtb_sell[0]="+vtb_sell[0]+
					       " v0_toadd_sell="+v0_toadd_sell+
					       " svtb_sell[0]="+svtb_sell[0]+
					       " sv0_toadd_sell="+sv0_toadd_sell+
					       " nt="+nt+
					       " N_SELL="+this.sell.getSize()+
					       " N_TRADES="+this.trades.getSize()+
					       " currenttime="+currenttime);
		}

		final double [] pl=new double [1],pls=new double [1];
		final long [] vv=new long [1],svv=new long [1];
		for(int i=0;i<nt;i++)
		{
		    if(save_unnorm_v_trades_buy[i]>0)
		    {
			vv[0]=vtb_buy[0]+save_unnorm_v_trades_buy[i];
			svv[0]=svtb_buy[0]+save_unnorm_sv_trades_buy[i];
			CalculateLiquidityDeficit3.accumulateBookTauBSplitFromLevels(
			    currenttime,
			    1,
			    PRINT_DATA_DEBUG,
			    this.buy,
			    true,
			    ptrt_buy,
			    vv, // changed on output
			    svv, // changed on output
			    pl,
			    pls,
			    tmpd);
			if(vv[0]!=this.v_trades_buy[i] ||
			   svv[0]!=this.tau_trades_buy[i] ||
			   Double.isNaN(pl[0])!=Double.isNaN(this.price_book_level_buy[0]) ||
			   (!Double.isNaN(pl[0]) && pl[0]!=this.price_book_level_buy[0]) ||
			   Double.isNaN(pls[0])!=Double.isNaN(this.pinv_ttrades_buy[i]) ||
			   (!Double.isNaN(pls[0]) && pls[0]!=this.pinv_ttrades_buy[i]))
			{
			    throw new RuntimeException(
				"Buy side levels mismatch:"+
				" vv[0]="+vv[0]+
				" v_trades_buy["+i+"]="+this.v_trades_buy[i]+
				" svv[0]="+svv[0]+
				" tau_trades_buy["+i+"]="+this.tau_trades_buy[i]+
				" pl[0]="+pl[0]+
				" price_book_level_buy[0]="+this.price_book_level_buy[0]+
				" pls[0]="+pls[0]+
				" pinv_ttrades_buy["+i+"]="+this.pinv_ttrades_buy[i]+
				" nt="+nt+
				" N_BUY="+this.buy.getSize()+
				" N_TRADES="+this.trades.getSize()+
				" currenttime="+currenttime);
			}
		    }

		    if(save_unnorm_v_trades_sell[i]>0)
		    {
			vv[0]=vtb_sell[0]+save_unnorm_v_trades_sell[i];
			svv[0]=svtb_sell[0]+save_unnorm_sv_trades_sell[i];
			CalculateLiquidityDeficit3.accumulateBookTauBSplitFromLevels(
			    currenttime,
			    1,
			    PRINT_DATA_DEBUG,
			    this.sell,
			    false,
			    ptrt_sell,
			    vv, // changed on output
			    svv, // changed on output
			    pl,
			    pls,
			    tmpd);
			if(vv[0]!=this.v_trades_sell[i] ||
			   svv[0]!=this.tau_trades_sell[i] ||
			   Double.isNaN(pl[0])!=Double.isNaN(this.price_book_level_sell[0]) ||
			   (!Double.isNaN(pl[0]) && pl[0]!=this.price_book_level_sell[0]) ||
			   Double.isNaN(pls[0])!=Double.isNaN(this.pinv_ttrades_sell[i]) ||
			   (!Double.isNaN(pls[0]) && pls[0]!=this.pinv_ttrades_sell[i]))
			{
			    throw new RuntimeException(
				"Sell side levels mismatch:"+
				" vv[0]="+vv[0]+
				" v_trades_sell["+i+"]="+this.v_trades_sell[i]+
				" svv[0]="+svv[0]+
				" tau_trades_sell["+i+"]="+this.tau_trades_sell[i]+
				" pl[0]="+pl[0]+
				" price_book_level_sell[0]="+this.price_book_level_sell[0]+
				" pls[0]="+pls[0]+
				" pinv_ttrades_sell["+i+"]="+this.pinv_ttrades_sell[i]+
				" nt="+nt+
				" N_SELL="+this.sell.getSize()+
				" N_TRADES="+this.trades.getSize()+
				" currenttime="+currenttime);
			}
		    }

		}
	    } // END alternative view check


	    final CalculateLiquidityDeficit3 l3=
		new CalculateLiquidityDeficit3(this.buy,
					       this.sell,
					       this.trades,
					       this.tmpordersbuf,
					       this.tmptradesbuf);
	    l3.calculateLiquidityDeficit(currenttime,
					 pcurrent_buy,
					 pcurrent_sell,
					 n_levels_trades,
					 trades_cleared_before);
	    for(int i=0;i<nt;i++)
	    {

		// check buy
		if(l3.v_book_acc_buy[i]!=vb_buy[i]|| // book volume accumulated
		   l3.tte_book_acc_buy[i]!=this.tau_book_acc_buy[i]|| // book tau on level
		   l3.ptr_book_buy[i]!=ptr_book_buy[i]|| // pointer to last order in book at given level
		   Double.isNaN(l3.price_book_level_buy[i])!=Double.isNaN(this.price_book_level_buy[i])||
		   (!Double.isNaN(this.price_book_level_buy[i]) && // split base level
		    l3.price_book_level_buy[i]!=this.price_book_level_buy[i])||
		   Double.isNaN(l3.price_book_split_buy[i])!=Double.isNaN(this.price_book_split_buy[i])||
		   (!Double.isNaN(this.price_book_split_buy[i]) && // split value
		    l3.price_book_split_buy[i]!=this.price_book_split_buy[i]))
		{
		    throw new RuntimeException(
			"Buy side mismatch:"+
			" vb_buy["+i+"]="+vb_buy[i]+
			" l3.v_book_acc_buy["+i+"]="+l3.v_book_acc_buy[i]+
			" ptr_book_buy["+i+"]="+ptr_book_buy[i]+
			" l3.ptr_book_buy["+i+"]="+l3.ptr_book_buy[i]+
			" price_book_level_buy["+i+"]="+this.price_book_level_buy[i]+
			" l3.price_book_level_buy["+i+"]="+l3.price_book_level_buy[i]+
			" price_book_split_buy["+i+"]="+this.price_book_split_buy[i]+
			" l3.price_book_split_buy["+i+"]="+l3.price_book_split_buy[i]+
			" nt="+nt+
			" N_BUY="+this.buy.getSize()+
			" N_TRADES="+this.trades.getSize()+
			" currenttime="+currenttime);
		}

		// check sell
		if(l3.v_book_acc_sell[i]!=vb_sell[i]|| // book volume accumulated
		   l3.tte_book_acc_sell[i]!=this.tau_book_acc_sell[i]|| // book tau on level
		   l3.ptr_book_sell[i]!=ptr_book_sell[i]||
		   Double.isNaN(l3.price_book_level_sell[i])!=Double.isNaN(this.price_book_level_sell[i])||
		   (!Double.isNaN(this.price_book_level_sell[i]) && // split base level
		    l3.price_book_level_sell[i]!=this.price_book_level_sell[i])||
		   Double.isNaN(l3.price_book_split_sell[i])!=Double.isNaN(this.price_book_split_sell[i])||
		   (!Double.isNaN(this.price_book_split_sell[i]) && // split value
		    l3.price_book_split_sell[i]!=this.price_book_split_sell[i]))
		{
		    throw new RuntimeException(
			"Sell side mismatch:"+
			" vb_sell["+i+"]="+vb_sell[i]+
			" l3.v_book_acc_sell["+i+"]="+l3.v_book_acc_sell[i]+
			" ptr_book_sell["+i+"]="+ptr_book_sell[i]+
			" l3.ptr_book_sell["+i+"]="+l3.ptr_book_sell[i]+
			" price_book_level_sell["+i+"]="+this.price_book_level_sell[i]+
			" l3.price_book_level_sell["+i+"]="+l3.price_book_level_sell[i]+
			" price_book_split_sell["+i+"]="+this.price_book_split_sell[i]+
			" l3.price_book_split_sell["+i+"]="+l3.price_book_split_sell[i]+
			" nt="+nt+
			" N_SELL="+this.sell.getSize()+
			" N_TRADES="+this.trades.getSize()+
			" currenttime="+currenttime);
		}

		checkTradesVolumeAndTaus(currenttime,
					 true,
					 this.trades,
					 i,
					 v0_toadd_buy,
					 sv0_toadd_buy,
					 this.price_book_sell[i],
					 pcurrent_buy-d,
					 this.v_trades_buy[i],
					 this.tau_trades_buy[i]);

		checkTradesVolumeAndTaus(currenttime,
					 false,
					 this.trades,
					 i,
					 v0_toadd_sell,
					 sv0_toadd_sell,
					 this.price_book_buy[i],
					 pcurrent_sell+d,
					 this.v_trades_sell[i],
					 this.tau_trades_sell[i]);
	    }
	}
    }


    private static final double SPREAD_FACTOR=0.0;

    private static void checkTradesVolumeAndTaus(final long currenttime,
						 final boolean flag_lower,
						 final Distribution<Trade> trades,
						 final int lev_num,
						 final long v_toadd,
						 final long sv_toadd,
						 final double p0,
						 final double pexit,
						 final long v_check,
						 final long tau_check)
    {
	final boolean PRINT_DEBUG=false;
	if(PRINT_DEBUG)
	{
	    System.err.println("=============checkTradesVolumeAndTaus:"+
			       " trades="+trades.getSize()+
			       " lev_num="+lev_num+
			       " v_toadd="+v_toadd+
			       " sv_toadd="+sv_toadd+
			       " p0="+p0+
			       " pexit="+pexit+
			       " v_check="+v_check+
			       " tau_check="+tau_check);
	}
	final int tsize=trades.getSize();
	final Trade [] tb=trades.getBuffer();
	long v=0,sv=0;
	if(flag_lower)
	{
	    final double pexitok=p0+OrdersFlow.EPS,pexitfailed=pexit-OrdersFlow.EPS;
	    for(int i=tsize;i-->0;)
	    {
		final Trade t=tb[i];
		if(PRINT_DEBUG)
		{
		    System.err.println(String.valueOf(i)+":\tt.price="+t.price+
				       " pexitok="+pexitok+
				       " pexitfailed="+pexitfailed+
				       " t.buy_sell_indicator="+t.buy_sell_indicator+
				       " t.origtime="+t.origtime);
		}
		if(t.origtime<=0 || t.buy_sell_indicator!=BookItem.BOOK_BUY)
		{
		    continue;
		}
		if(!(t.price>=pexitfailed))
		{
		    v=sv=0;
		    break;
		}
		if(!(t.price<=pexitok))
		{
		    break;
		}
		v+=t.shares;
		sv+=t.shares*((t.time-t.origtime)/ModelParameters.T_SUMM_SCALE);
	    }
	}
	else
	{
	    final double pexitok=p0-OrdersFlow.EPS,pexitfailed=pexit+OrdersFlow.EPS;
	    for(int i=tsize;i-->0;)
	    {
		final Trade t=tb[i];
		if(PRINT_DEBUG)
		{
		    System.err.println(String.valueOf(i)+":\tt.price="+t.price+
				       " pexitok="+pexitok+
				       " pexitfailed="+pexitfailed+
				       " t.buy_sell_indicator="+t.buy_sell_indicator+
				       " t.origtime="+t.origtime);
		}
		if(t.origtime<=0 || t.buy_sell_indicator!=BookItem.BOOK_SELL)
		{
		    continue;
		}
		if(!(t.price<=pexitfailed))
		{
		    v=sv=0;
		    break;
		}
		if(!(t.price>=pexitok))
		{
		    break;
		}
		v+=t.shares;
		sv+=t.shares*((t.time-t.origtime)/ModelParameters.T_SUMM_SCALE);
	    }
	}
	final long tau;
	if(v>0)
	{
	    v+=v_toadd;
	    sv+=sv_toadd;
	    tau=(sv/v)*ModelParameters.T_SUMM_SCALE;
	}
	else
	{
	    tau=-1;
	}

	if(v!=v_check||tau!=tau_check)
	{
	    throw new RuntimeException(
			"Trades tau/volume mismatch:"+
			" v="+v+
			" v_check="+v_check+
			" tau="+tau+
			" tau_check="+tau_check+
			" v_toadd="+v_toadd+
			" sv_toadd="+sv_toadd+
			" N_TRADES="+trades.getSize()+
			" flag_lower="+flag_lower+
			" currenttime="+currenttime);
	}
    }


static double calculateInversionPerLevel(final long currenttime,
					 final double pcurrent_buy,
					 final double pcurrent_sell,
					 final Distribution<BookItem> book,
					 final boolean PRINT_DATA_DEBUG,
					 final boolean flag_lower,
					 // output array of size 2
					 final double [] out_invs,
					 // output array of size 3
					 final long [] out_taus
					 )
{
    final long vcut=Long.MAX_VALUE;
    final int ptr_to_start=0,tsize=book.getSize();
    final BookItem [] tb=book.getBuffer();

    if(PRINT_DATA_DEBUG)
    {
	System.err.println("calculateInversionPerLevel flag_lower="+flag_lower+
			   " N_ORDERS="+tsize);
    }

    long v=0,sv=0,vx=0,svx=0,tau_lpp=-1,t_p=-1,t_p_cont=-1,t_p_max_cont=-1,tlprev=Long.MAX_VALUE;
    int pi=-1,ptr_inv=-1,ptr_inv_cont=-1,n_levels=-1;
    boolean fcont=true;
    if(flag_lower)
    {
	double lev=Double.POSITIVE_INFINITY;
	for(int i=ptr_to_start;i<tsize;i++)
	{
	    final BookItem t=tb[i];
	    if(t.price<lev)
	    { // next level
		if(n_levels>=0)
		{
		    v+=vx;
		    sv+=svx;
		    if(PRINT_DATA_DEBUG)
		    {
			System.err.println("NL="+n_levels+" vx="+vx+" svx="+svx+" v="+v+" sv="+sv+" p="+tb[pi].price+" flag_lower="+flag_lower);
		    }
		    final long tau_l=vx>0?((svx/vx)*ModelParameters.T_SUMM_SCALE):-1;
		    final long tau_p=v>0?((sv/v)*ModelParameters.T_SUMM_SCALE):-1;
		    if(fcont)
		    {
			if(tau_l>=tau_lpp)
			{
			    ptr_inv_cont=pi;
			    t_p_cont=tau_p;
			    t_p_max_cont=tau_l;
			}
			else
			{
			    fcont=false;
			}
		    }
		    tau_lpp=tau_l;
		    if(tau_l>=tau_p)
		    {
			ptr_inv=pi;
			t_p=tau_p;
		    }
		    else
		    {
			pi=-1;
			vx=svx=0;
			break;
		    }
		}
		pi=-1;
		vx=svx=0;
		++n_levels;
		lev=t.price-OrdersFlow.EPS;
		tlprev=-1;
	    }
	    if(vx<=vcut)
	    { // if we have volume cut - skip
		final long tau=(currenttime-t.time)/ModelParameters.T_SUMM_SCALE;
		vx+=t.shares;
		svx+=t.shares*tau;
	    }
	    pi=i;
	    if(t.time<tlprev)
	    {
		throw new RuntimeException("Book orders are not sorted");
	    }
	    tlprev=t.time;
	}
    }
    else
    {
	double lev=Double.NEGATIVE_INFINITY;
	for(int i=ptr_to_start;i<tsize;i++)
	{
	    final BookItem t=tb[i];
	    if(t.price>lev)
	    { // next level
		if(n_levels>=0)
		{
		    v+=vx;
		    sv+=svx;
		    if(PRINT_DATA_DEBUG)
		    {
			System.err.println("NL="+n_levels+" vx="+vx+" svx="+svx+" v="+v+" sv="+sv+" p="+tb[pi].price+" flag_lower="+flag_lower);
		    }
		    final long tau_l=vx>0?((svx/vx)*ModelParameters.T_SUMM_SCALE):-1;
		    final long tau_p=v>0?((sv/v)*ModelParameters.T_SUMM_SCALE):-1;
		    if(fcont)
		    {
			if(tau_l>=tau_lpp)
			{
			    ptr_inv_cont=pi;
			    t_p_cont=tau_p;
			    t_p_max_cont=tau_l;
			}
			else
			{
			    fcont=false;
			}
		    }
		    tau_lpp=tau_l;
		    if(tau_l>=tau_p)
		    {
			ptr_inv=pi;
			t_p=tau_p;
		    }
		    else
		    {
			pi=-1;
			vx=svx=0;
			break;
		    }
		}
		pi=-1;
		vx=svx=0;
		++n_levels;
		lev=t.price+OrdersFlow.EPS;
		tlprev=-1;
	    }
	    if(vx<=vcut)
	    { // if we have volume cut - skip
		final long tau=(currenttime-t.time)/ModelParameters.T_SUMM_SCALE;
		vx+=t.shares;
		svx+=t.shares*tau;
	    }
	    pi=i;
	    if(t.time<tlprev)
	    {
		throw new RuntimeException("Book orders are not sorted");
	    }
	    tlprev=t.time;
	}
    }
    if(pi>=0)
    {
	v+=vx;
	sv+=svx;
	final long tau_l=vx>0?((svx/vx)*ModelParameters.T_SUMM_SCALE):-1;
	final long tau_p=v>0?((sv/v)*ModelParameters.T_SUMM_SCALE):-1;
	if(fcont)
	{
	    if(tau_l>=tau_lpp)
	    {
		ptr_inv_cont=pi;
		t_p_cont=tau_p;
		t_p_max_cont=tau_l;
	    }
	    else
	    {
		fcont=false;
	    }
	}
	tau_lpp=tau_l;
	if(tau_l>=tau_p)
	{
	    ptr_inv=pi;
	    t_p=tau_p;
	}
	pi=-1;
	vx=svx=0;
	++n_levels;
    }

    final double pinv=ptr_inv<0?Double.NaN:tb[ptr_inv].price,
	pinv_cont=ptr_inv_cont<0?Double.NaN:tb[ptr_inv_cont].price;
    out_invs[0]=pinv;
    out_invs[1]=pinv_cont;
    out_taus[0]=t_p;
    out_taus[1]=t_p_cont;
    out_taus[2]=t_p_max_cont;

    return pinv;

}


}
