package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit ver 7.
 *  @version $Id: CalculateLiquidityDeficit7.java,v 1.243 2020/02/06 00:09:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit7 {

    private static final long TSCALE=ModelParameters.T_SUMM_SCALE/100;

    /** Spread based adjustment.
     * Set to Double.NaN to disable.
     */
    static final int PTR_FAILURE_INDEX=3;
    private static final boolean OKEXIT_LEVEL_FROM_OPPOSITE_TYPE=false;


    private final Distribution<BookItem> buy,sell;
    private final Distribution<Trade> trades;

    CalculateLiquidityDeficit7(final Distribution<BookItem> buy,
			       final Distribution<BookItem> sell,
			       final Distribution<Trade> trades,
			       final Distribution<BookItem> tmpordersbuf,
			       final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
    }


    double []
	price_book_buy=null,
	price_book_sell=null;


    TradesVolumeAndTaus2 [] trades_buy=null,trades_sell=null;
    BookVolumeAndTaus2 [] book_buy=null,book_sell=null;

    void clear()
    {

	this.price_book_buy=
	this.price_book_sell=null;
	this.trades_buy=this.trades_sell=null;
	this.book_buy=
	this.book_sell=null;
    }


    void calculateLiquidityDeficit(final long currenttime,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final int n_levels_trades,
				   final long trades_cleared_before)
    {
	final boolean PRINT_DATA_DEBUG=false;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tN_ORDERS BUY\t"+this.buy.getSize()+
			       "\tN_ORDERS SELL\t"+this.sell.getSize()+
			       "\tn_levels_trades\t"+n_levels_trades+
			       "\ncur_buy\t"+pcurrent_buy+
			       "\tcur_sell\t"+pcurrent_sell);
	}

	final int nt=n_levels_trades,bsizebuy=this.buy.getSize(),bsizesell=this.sell.getSize();
	final BookItem [] bbuy=this.buy.getBuffer(),bsell=this.sell.getBuffer();

	final int [] ptr_book_buy=new int [nt];
	final long [] vb_buy=new long [nt],sv_nn_buy=new long [nt];
	CalculateLiquidityDeficit3.calculateBookLevelsTau(
	    currenttime,
	    nt,
	    PRINT_DATA_DEBUG,
	    0,
	    bbuy,
	    bsizebuy,
	    true,

	    // output:
	    ptr_book_buy,
	    vb_buy, // volume per level (not accumulated)
	    sv_nn_buy // unnormalized tau (not accumulated)
	);


	final int [] ptr_book_sell=new int [nt];
	final long [] vb_sell=new long [nt],sv_nn_sell=new long [nt];
	CalculateLiquidityDeficit3.calculateBookLevelsTau(
	    currenttime,
	    nt,
	    PRINT_DATA_DEBUG,
	    0,
	    bsell,
	    bsizesell,
	    false,

	    // output:
	    ptr_book_sell,
	    vb_sell, // volume per level (not accumulated)
	    sv_nn_sell  // unnormalized tau (not accumulated)
        );


	this.price_book_buy=new double [nt];
	this.price_book_sell=new double [nt];
	for(int i=0;i<nt;i++)
	{
	    this.price_book_buy[i]=ptr_book_buy[i]<0?Double.NaN:bbuy[ptr_book_buy[i]].price;
	    this.price_book_sell[i]=ptr_book_sell[i]<0?Double.NaN:bsell[ptr_book_sell[i]].price;
	}

	// failure exit levels
	final double
	    p_failure_buy=this.price_book_buy[PTR_FAILURE_INDEX],
	    p_failure_sell=this.price_book_sell[PTR_FAILURE_INDEX];

	long vvb_buy=0,vvb_sell=0,svvb_buy=0,svvb_sell=0;
	this.trades_buy=new TradesVolumeAndTaus2 [nt];
	this.trades_sell=new TradesVolumeAndTaus2 [nt];
	this.book_buy=new BookVolumeAndTaus2 [nt];
	this.book_sell=new BookVolumeAndTaus2 [nt];

	for(int i=0;i<nt;i++)
	{

	    final PTRToRangeAndPE ptr_buy=findPtrToTrades(
		currenttime,
		this.trades,
		true,
		OKEXIT_LEVEL_FROM_OPPOSITE_TYPE?
		// sell book levels for buy trades or from the same
		this.price_book_sell[i]:(pcurrent_buy+(pcurrent_buy-this.price_book_buy[i])),
		Double.NaN,
		p_failure_buy // trades scanning exit level (min constrain)
		);

	    this.trades_buy[i]=
		calculateTradesVolumeAndTaus2(
		    currenttime,
		    this.trades,
		    true,
		    ptr_buy.ptr_t_range_cut);
	    if(ptr_buy.vi!=this.trades_buy[i].vi)
	    {
		throw new IllegalArgumentException("Volume Mismatch:"+
						   " ptr_buy.vi="+ptr_buy.vi+
						   " trades_buy["+i+"].vi="+this.trades_buy[i].vi+
						   " currenttime="+currenttime);

	    }

	    this.book_buy[i]=
		calculateBookVolumeAndTauSplitted(
		    currenttime,
		    this.buy,
		    this.trades,
		    true,
		    this.trades_buy[i].t_range_cut,
		    Double.POSITIVE_INFINITY,
		    this.price_book_buy[i]);
	    if(Double.isNaN(this.price_book_buy[i]))
	    {
		vvb_buy=svvb_buy=0;
	    }
	    else
	    {
		vvb_buy+=vb_buy[i];
		svvb_buy+=sv_nn_buy[i];
	    }
	    if((this.book_buy[i].v_out+this.book_buy[i].v_in)!=vvb_buy||
	        (TSCALE==ModelParameters.T_SUMM_SCALE &&
		 (this.book_buy[i].sv_out+this.book_buy[i].sv_in)!=svvb_buy)
	       )
	    {
		throw new RuntimeException(
		    "Buy volume mismatch:"+
		    " level num="+i+
		    " v_out+v_in="+
		    (this.book_buy[i].v_out+this.book_buy[i].v_in)+
		    " vvb_buy="+vvb_buy+
		    " sv_out+sv_in="+
		    (this.book_buy[i].sv_out+this.book_buy[i].sv_in)+
		    " svvb_buy="+svvb_buy+
		    " price_level="+this.price_book_buy[i]+
		    " t_range_cut_buy="+this.trades_buy[i].t_range_cut+
		    " N_BUY="+this.buy.getSize()+
		    " N_SELL="+this.sell.getSize()+
		    " currenttime="+currenttime);
	    }

	    final PTRToRangeAndPE ptr_sell=findPtrToTrades(
		currenttime,
		this.trades,
		false,
		OKEXIT_LEVEL_FROM_OPPOSITE_TYPE?
		// buy book levels for sell trades or from the same
		this.price_book_buy[i]:(pcurrent_sell+(pcurrent_sell-this.price_book_sell[i])),
		Double.NaN,
		p_failure_sell // trades scanning exit level (max constrain)
		);

	    this.trades_sell[i]=
		calculateTradesVolumeAndTaus2(
		    currenttime,
		    this.trades,
		    false,
		    ptr_sell.ptr_t_range_cut);
	    if(ptr_sell.vi!=this.trades_sell[i].vi)
	    {
		throw new IllegalArgumentException("Volume Mismatch:"+
						   " ptr_sell.vi="+ptr_sell.vi+
						   " trades_sell["+i+"].vi="+this.trades_sell[i].vi+
						   " currenttime="+currenttime);

	    }

	    this.book_sell[i]=
		calculateBookVolumeAndTauSplitted(
		    currenttime,
		    this.sell,
		    this.trades,
		    false,
		    this.trades_sell[i].t_range_cut,
		    Double.NEGATIVE_INFINITY,
		    this.price_book_sell[i]);

	    if(Double.isNaN(this.price_book_sell[i]))
	    {
		vvb_sell=svvb_sell=0;
	    }
	    else
	    {
		vvb_sell+=vb_sell[i];
		svvb_sell+=sv_nn_sell[i];
	    }
	    if((this.book_sell[i].v_out+this.book_sell[i].v_in)!=vvb_sell||
	        (TSCALE==ModelParameters.T_SUMM_SCALE &&
		 (this.book_sell[i].sv_out+this.book_sell[i].sv_in)!=svvb_sell)
	       )
	    {
		throw new RuntimeException(
		    "Sell volume mismatch:"+
		    " level num="+i+
		    " v_out+v_in="+
		    (this.book_sell[i].v_out+this.book_sell[i].v_in)+
		    " vvb_sell="+vvb_sell+
		    " sv_out+sv_in="+
		    (this.book_sell[i].sv_out+this.book_sell[i].sv_in)+
		    " svvb_sell="+svvb_sell+
		    " price_level="+this.price_book_sell[i]+
		    " t_range_cut_sell="+this.trades_sell[i].t_range_cut+
		    " N_BUY="+this.buy.getSize()+
		    " N_SELL="+this.sell.getSize()+
		    " currenttime="+currenttime);
	    }
	}

    }

    static class TradesVolumeAndTaus2 {
	/** Result. */
	final long t_range_cut,vi;

	final double paver;

	/** Unnormalized tau. */
	private final long sv_buy_out,sv_sell_out,sv_buy_out_as_in,sv_sell_out_as_in,sv_buy_in,sv_sell_in,svdi_buy_out_as_in,svdi_sell_out_as_in;

	/** Additional values we may use for testing & analyzis. */
	private final long v_buy_out,v_buy_in,v_sell_out,v_sell_in,sv_buy_out_2,sv_sell_out_2,se_buy,se_sell;
	private final long svr_buy,svr_sell,currenttime;

	final double svsq_buy,svsq_sell;

	final boolean flag_lower;

	TradesVolumeAndTaus2(final long t_range_cut,
			     final long sv_buy_out,
			     final long sv_buy_in,
			     final long sv_buy_out_as_in,
			     final long sv_sell_out,
			     final long sv_sell_in,
			     final long sv_sell_out_as_in,
			     final boolean flag_lower,
			     final long v_buy_out,
			     final long v_buy_in,
			     final long v_sell_out,
			     final long v_sell_in,
			     final long svr_buy,
			     final long svr_sell,
			     final long currenttime,
			     final long vi,
			     final double svsq_buy,
			     final double svsq_sell,
			     final long svdi_buy_out_as_in,
			     final long svdi_sell_out_as_in,
			     final long sv_buy_out_2,
			     final long sv_sell_out_2,
			     final long se_buy,
			     final long se_sell,
			     final double paver)
	{
	    this.flag_lower=flag_lower;
	    this.t_range_cut=t_range_cut;
	    this.sv_buy_out=sv_buy_out;
	    this.sv_buy_in=sv_buy_in;
	    this.sv_buy_out_as_in=sv_buy_out_as_in;
	    this.sv_sell_out=sv_sell_out;
	    this.sv_sell_in=sv_sell_in;
	    this.sv_sell_out_as_in=sv_sell_out_as_in;
	    this.v_buy_out=v_buy_out;
	    this.v_buy_in=v_buy_in;
	    this.v_sell_out=v_sell_out;
	    this.v_sell_in=v_sell_in;
	    this.svr_buy=svr_buy;
	    this.svr_sell=svr_sell;
	    this.currenttime=currenttime;
	    this.vi=vi;
	    this.svsq_buy=svsq_buy;
	    this.svsq_sell=svsq_sell;
	    this.svdi_buy_out_as_in=svdi_buy_out_as_in;
	    this.svdi_sell_out_as_in=svdi_sell_out_as_in;
	    this.sv_buy_out_2=sv_buy_out_2;
	    this.sv_sell_out_2=sv_sell_out_2;
	    this.se_buy=se_buy;
	    this.se_sell=se_sell;
	    this.paver=paver;
	}

	double getOldFraction()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    return ((double)getVOld())/v;
	}
	double getiOldFraction()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    return ((double)getiVOld())/v;
	}

	long getV()
	{
	    return this.v_buy_out+this.v_buy_in+this.v_sell_out+this.v_sell_in;
	}


	long getTauR_Buy()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return -1;
	    }
	    final long sv_cut=this.v_buy_out*((this.currenttime-this.t_range_cut)/TSCALE);
	    return ((this.svr_buy+sv_cut)/v)*TSCALE;
	}

	long getTauRTotal_Buy()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return -1;
	    }
	    return ((this.svr_buy+this.sv_buy_out_2)/v)*TSCALE;
	}

        long getTauR_Sell()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return -1;
	    }
	    final long sv_cut=this.v_sell_out*((this.currenttime-this.t_range_cut)/TSCALE);
	    return ((this.svr_sell+sv_cut)/v)*TSCALE;
	}

        long getTauRTotal_Sell()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return -1;
	    }
	    return ((this.svr_sell+this.sv_sell_out_2)/v)*TSCALE;
	}

	double getTauRel()
	{
	    final long t;
	    if(this.flag_lower)
	    {
		t=getTauR_Buy();
	    }
	    else
	    {
		t=getTauR_Sell();
	    }
	    if(t<0)
	    {
		return Double.NaN;
	    }
	    return ((double)t)/((double)(this.currenttime-this.t_range_cut));
	}

	double getiTauRel()
	{
	    final long t;
	    if(this.flag_lower)
	    {
		t=getTauR_Sell();
	    }
	    else
	    {
		t=getTauR_Buy();
	    }
	    if(t<0)
	    {
		return Double.NaN;
	    }
	    return ((double)t)/((double)(this.currenttime-this.t_range_cut));
	}


	private double getTauDiff(final long t_buy,final long t_sell)
	{
	    if(t_buy<0||t_sell<0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long t1,t2;
	    if(this.flag_lower)
	    {
		t1=t_sell;
		t2=t_buy;
	    }
	    else
	    {
		t1=t_buy;
		t2=t_sell;
	    }
	    final double td=this.currenttime-this.t_range_cut;
	    return (t1-t2)/td;
	}

	double getTauRDiff()
	{
	    return getTauDiff(getTauR_Buy(),getTauR_Sell());
	}

	long getTauRE_Buy()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return -1;
	    }
	    return ((this.sv_buy_in+this.sv_buy_out_as_in)/v)*TSCALE;
	}

        long getTauRE_Sell()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return -1;
	    }
	    return ((this.sv_sell_in+this.sv_sell_out_as_in)/v)*TSCALE;
	}

	double getTauREDiff()
	{
	    return getTauDiff(getTauRE_Buy(),getTauRE_Sell());
	}


	long getVOld()
	{
	    return (this.flag_lower?this.v_buy_out:this.v_sell_out);
	}
	long getiVOld()
	{
	    return (this.flag_lower?this.v_sell_out:this.v_buy_out);
	}

	long getTauOld()
	{
	    if(this.t_range_cut<0)
	    {
		return -1;
	    }
	    if(this.flag_lower)
	    {
		if(this.v_buy_out>0)
		{
		    return (this.sv_buy_out/this.v_buy_out)*TSCALE;
		}
	    }
	    else
	    {
		if(this.v_sell_out>0)
		{
		    return (this.sv_sell_out/this.v_sell_out)*TSCALE;
		}
	    }
	    return 0;
	}

	double getVUnsatisfiedRel(final BookVolumeAndTaus2 unsatisfied)
	{
	    if(this.t_range_cut!=unsatisfied.t_range_cut ||
	       this.flag_lower==unsatisfied.flag_lower)
	    {
		throw new IllegalArgumentException(
		    "t_range_cut mismatch:"+
		    " t_range_cut="+this.t_range_cut+
		    " unsatisfied.t_range_cut="+unsatisfied.t_range_cut+
		    " flag_lower="+this.flag_lower+
		    " unsatisfied.flag_lower="+unsatisfied.flag_lower);
	    }
	    final long v=getV(),vu=unsatisfied.getV();
	    if((v<=0&&vu<=0)||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    return vu/((double)(v+vu));
	}

	double getVUnsatisfiedInRel(final BookVolumeAndTaus2 unsatisfied)
	{
	    if(this.t_range_cut!=unsatisfied.t_range_cut ||
	       this.flag_lower==unsatisfied.flag_lower)
	    {
		throw new IllegalArgumentException(
		    "t_range_cut mismatch:"+
		    " t_range_cut="+this.t_range_cut+
		    " unsatisfied.t_range_cut="+unsatisfied.t_range_cut+
		    " flag_lower="+this.flag_lower+
		    " unsatisfied.flag_lower="+unsatisfied.flag_lower);
	    }
	    final long v=getV(),vu=unsatisfied.getV();
	    if((v<=0&&vu<=0)||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    return vu/((double)(v-getVOld()+vu));
	}


	double getTauInDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long wbuy=this.v_buy_in+this.v_sell_out+this.v_sell_in,
		wsell=this.v_buy_out+this.v_buy_in+this.v_sell_in;
	    final long td=this.currenttime-this.t_range_cut,ts=td/TSCALE;
	    final long tau=(((wbuy*ts-this.svr_buy)-(wsell*ts-this.svr_sell))/v)*TSCALE;
	    return (tau)/((double)td);
	}
	double getTauInLDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long td=this.currenttime-this.t_range_cut,ts=td/TSCALE;
	    final long tau=(((this.v_buy_out*ts+this.svr_buy)-(this.v_sell_out*ts+this.svr_sell))/v)*TSCALE;
	    return -(tau)/((double)td);
	}
	double getTauTDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long td=this.currenttime-this.t_range_cut;
	    final long tau=((this.se_buy-this.se_sell)/v)*TSCALE;
	    return -(tau)/((double)td);
	}

	double getTauInTESDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long s=this.se_buy+this.se_sell;
	    if(s<=0)
	    {
		return Double.NaN;
	    }
	    return -(this.se_buy-this.se_sell)/((double)s);
	}
	double getTauInESLDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long td=this.currenttime-this.t_range_cut,ts=td/TSCALE;
	    final long sbuy=this.v_buy_out*ts+this.svr_buy,ssell=this.v_sell_out*ts+this.svr_sell;
	    final long s=sbuy+ssell;
	    if(s<=0)
	    {
		return Double.NaN;
	    }
	    return -(sbuy-ssell)/((double)s);
	}

	double getTauInDiff(final BookVolumeAndTaus2 bbuy,final BookVolumeAndTaus2 bsell)
	{
	    if(bbuy.t_range_cut!=this.t_range_cut||
	       bsell.t_range_cut!=this.t_range_cut||
	       !bbuy.flag_lower||bsell.flag_lower)
	    {
		throw new IllegalArgumentException();
	    }

	    final long wbuy=this.v_buy_in+this.v_sell_out+this.v_sell_in+bbuy.v_in+bbuy.v_start,
		svbuy=this.svr_buy+bbuy.sv_in+bbuy.sv_start,
		wsell=this.v_buy_out+this.v_buy_in+this.v_sell_in+bsell.v_in+bsell.v_start,
		svsell=this.svr_sell+bsell.sv_in+bsell.sv_start,
		v=getV()+(bsell.getV()+bsell.v_start+bbuy.getV()+bbuy.v_start)/2;
	    if(v<=0 ||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long td=this.currenttime-this.t_range_cut,ts=td/TSCALE;
	    final long tau=(((wbuy*ts-svbuy)-(wsell*ts-svsell))/v)*TSCALE;
	    return (tau)/((double)td);
	}

	double getETauInDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long td=this.currenttime-this.t_range_cut;
	    final long tau=(((this.sv_buy_in+this.sv_buy_out_as_in)-(this.sv_sell_in+this.sv_sell_out_as_in))/v)*TSCALE;
	    return -(tau)/((double)td);
	}

	double getESTauInDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long sbuy=this.sv_buy_in+this.sv_buy_out_as_in,
		ssell=this.sv_sell_in+this.sv_sell_out_as_in;
	    final long s=sbuy+ssell;
	    if(s<=0)
	    {
		return Double.NaN;
	    }
	    return -(sbuy-ssell)/((double)s);
	}

	double getESDITauInDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long sbuy=this.sv_buy_in+this.svdi_buy_out_as_in,
		ssell=this.sv_sell_in+this.svdi_sell_out_as_in;
	    final long s=sbuy+ssell;
	    if(s<=0)
	    {
		return Double.NaN;
	    }
	    return -(sbuy-ssell)/((double)s);
	}

	double getAESDITauInDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long sbuy=this.sv_buy_in+this.svdi_buy_out_as_in,
		ssell=this.sv_sell_in+this.svdi_sell_out_as_in;
	    final long s=sbuy+ssell;
	    final long td=this.currenttime-this.t_range_cut;
	    return ((s/v)*TSCALE)/((double)td);
	}

	double getEDITauInDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long td=this.currenttime-this.t_range_cut;
	    final long tau=(((this.sv_buy_in+this.svdi_buy_out_as_in)-(this.sv_sell_in+this.svdi_sell_out_as_in))/v)*TSCALE;
	    return -(tau)/((double)td);
	}

	double getEVInDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long dv=(this.v_buy_in+this.v_buy_out)-(this.v_sell_in+this.v_sell_out);
	    return -(dv)/((double)v);
	}
	double getSQInDiff()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final double dsq=this.svsq_buy-this.svsq_sell,sqtd=getweight(this.currenttime-this.t_range_cut);
	    return -(dsq)/(v*sqtd);
	}
    }



    static class PTRToRangeAndPE extends CalculateLiquidityDeficit9.PTRToRange {

	final long t_p_opposite,vi;
	final double p_t_range_cut,p_opposite;
	final int ptr_p_opposite;

	PTRToRangeAndPE(final int ptr_t_range_cut,
			final long t_range_cut,
			final boolean flag_lower,
			final double p_t_range_cut,
			final double p_opposite,
			final int ptr_p_opposite,
			final long t_p_opposite,
			final long vi)
	{
	    super(ptr_t_range_cut,t_range_cut,flag_lower);
	    this.p_t_range_cut=p_t_range_cut;
	    this.p_opposite=p_opposite;
	    this.ptr_p_opposite=ptr_p_opposite;
	    this.t_p_opposite=t_p_opposite;
	    this.vi=vi;
	}
    }


    private static final boolean FLAG_TAU_AUTOADJUSTMENT=true;


    static PTRToRangeAndPE findPtrToTrades(
	final long currenttime,
	final Distribution<Trade> trades,
	final boolean flag_lower,
	final double p0,
	final double p0_s,
	final double pexit_failure)
    {

	final boolean PRINT_DATA_DEBUG=false;
	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=============findPtrToTrades:"+
			       " trades="+trades.getSize()+
			       " p0="+p0+
			       " p0_s="+p0_s+
			       " pexit_failure="+pexit_failure+
			       " flag_lower="+flag_lower+
			       " currenttime="+BookItem.getTimeDiag(currenttime));
	}


	final int tsize=trades.getSize();
	final Trade [] tb=trades.getBuffer();
	long vi=0,prevt=Long.MAX_VALUE;
	int ptr_t_range_cut=-1,ptr_p_opposite=-1;
	if(flag_lower)
	{
	    final double
		pexitfailed=pexit_failure-OrdersFlow.EPS,
		pexitok=p0+OrdersFlow.EPS,
		pexitok_s=p0_s+OrdersFlow.EPS;
	    double pl_opposite=Double.NaN;
	    for(int i=tsize;i-->0;)
	    {
		final Trade t=tb[i];
		if(t.time>prevt)
		{
		    throw new RuntimeException("The data is not sorted:"+
					       prevt+" and "+t.time+
					       " currenttime="+currenttime);
		}
		prevt=t.time;

		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.origtime+"\t"+
				       t.shares+"\t"+
				       t.tradetype+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.price+"\t"+
				       t.matchnumber+"\t"+
				       t.orderreferencenumber);
		}
		if(t.origtime<=0)
		{
		    continue;
		}
		if(!(t.price>=pexitfailed))
		{
		    break;
		}
		if(!(t.price>pl_opposite))
		{
		    pl_opposite=t.price+OrdersFlow.EPS;
		    ptr_p_opposite=i;
		}
		if(t.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    if(t.price>pexitok)
		    {
			ptr_t_range_cut=i;
			break;
		    }
		}
		else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    if(t.price>pexitok_s)
		    {
			ptr_t_range_cut=i;
			break;
		    }
		}
		vi+=t.shares;
	    }
	}
	else
	{
	    final double
		pexitfailed=pexit_failure+OrdersFlow.EPS,
		pexitok=p0-OrdersFlow.EPS,
		pexitok_s=p0_s-OrdersFlow.EPS;
	    double pl_opposite=Double.NaN;
	    for(int i=tsize;i-->0;)
	    {
		final Trade t=tb[i];
		if(t.time>prevt)
		{
		    throw new RuntimeException("The data is not sorted:"+
					       prevt+" and "+t.time+
					       " currenttime="+currenttime);
		}
		prevt=t.time;

		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.origtime+"\t"+
				       t.shares+"\t"+
				       t.tradetype+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.price+"\t"+
				       t.matchnumber+"\t"+
				       t.orderreferencenumber);
		}
		if(t.origtime<=0)
		{
		    continue;
		}
		if(!(t.price<=pexitfailed))
		{
		    break;
		}
		if(!(t.price<pl_opposite))
		{
		    pl_opposite=t.price-OrdersFlow.EPS;
		    ptr_p_opposite=i;
		}
		if(t.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    if(t.price<pexitok)
		    {
			ptr_t_range_cut=i;
			break;
		    }
		}
		else if(t.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    if(t.price<pexitok_s)
		    {
			ptr_t_range_cut=i;
			break;
		    }
		}
		vi+=t.shares;
	    }
	}
	if(ptr_t_range_cut<0)
	{
	    vi=0;
	    ptr_p_opposite=-1;
	}

	return new PTRToRangeAndPE(ptr_t_range_cut,
				   ptr_t_range_cut<0?-1:tb[ptr_t_range_cut].time,
				   flag_lower,
				   ptr_t_range_cut<0?Double.NaN:tb[ptr_t_range_cut].price,
				   ptr_p_opposite<0?Double.NaN:tb[ptr_p_opposite].price,
				   ptr_p_opposite,
				   ptr_p_opposite<0?-1:tb[ptr_p_opposite].time,
				   vi);
    }



    static class PTRToPriceTrange {
	final long t_range_cut,vi,currenttime;
	final int ptr_t_range_cut;
	PTRToPriceTrange(final long t_range_cut,
			 final long vi,
			 final long currenttime,
			 final int ptr_t_range_cut)
	{
	    this.t_range_cut=t_range_cut;
	    this.vi=vi;
	    this.currenttime=currenttime;
	    this.ptr_t_range_cut=ptr_t_range_cut;
	}
    }


    static PTRToPriceTrange findPtrToTradesPrice(
	final long currenttime,
	final Distribution<Trade> trades,
	final double p_min,
	final double p_max)
    {


	final int tsize=trades.getSize();
	final Trade [] tb=trades.getBuffer();
	long vi=0,prevt=Long.MAX_VALUE;
	int ptr_t_range_cut=-1;

	final double l_min=p_min-OrdersFlow.EPS,l_max=p_max+OrdersFlow.EPS;
	for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;
	    ptr_t_range_cut=i;

	    if(t.price<l_min)
	    {
		break;
	    }
	    if(t.price>l_max)
	    {
		break;
	    }
	    vi+=t.shares;
	}

	return new PTRToPriceTrange(ptr_t_range_cut<0?-1:tb[ptr_t_range_cut].time,
				    vi,
				    currenttime,
				    ptr_t_range_cut);
    }

    static TradesVolumeAndTaus2 calculateTradesVolumeAndTaus2(
	final long currenttime,
	final Distribution<Trade> trades,
	final boolean flag_lower,
	final int ptr_t_range_cut)
    {
	final boolean PRINT_DATA_DEBUG=false,PRINT_DATA_WARNINGS=false;
	final int tsize=trades.getSize();
	final Trade [] tb=trades.getBuffer();
	final long t_range_cut=ptr_t_range_cut<0?-1:tb[ptr_t_range_cut].time;

	// now scan again and count the liquidity
	long v_buy_in=0,v_buy_out=0,
	    v_sell_in=0,v_sell_out=0,
	    sv_buy_out=0,sv_sell_out=0,
	    sv_buy_in=0,sv_sell_in=0,
	    sv_buy_out_as_in=0,sv_sell_out_as_in=0,
	    sv_buy_out_2=0,sv_sell_out_2=0,
	    se_buy=0,se_sell=0,
	    vc=0,svdi_buy_out_as_in=0,svdi_sell_out_as_in=0;
	double svsq_buy=0,svsq_sell=0;
	long svr_buy=0,svr_sell=0;
	final int ptr_interval=ptr_t_range_cut<0?tsize:ptr_t_range_cut+1;
	final long tau_int=currenttime-t_range_cut;
	double spaver=0;
	for(int i=tsize;i-->ptr_interval;)
	{
	    final Trade t=tb[i];
	    final long torg=t.origtime;
	    if(torg<=0)
	    {
		continue;
	    }
	    final int s=t.shares;
	    final long te;
	    if(FLAG_TAU_AUTOADJUSTMENT)
	    {
		te=t_range_cut;
	    }
	    else
	    {
		te=t.time;
	    }

	    final long dt=t.time-torg;
	    if(t.buy_sell_indicator==BookItem.BOOK_BUY)
	    {
		svr_sell+=s*((currenttime-t.time)/TSCALE); // limit buy means immediate sell just came
		se_buy+=s*(dt/TSCALE);
		if(torg<=t_range_cut)
		{
		    v_buy_out+=s;
		    sv_buy_out+=s*((te-torg)/TSCALE);
		    sv_buy_out_as_in+=s*((t.time-t_range_cut)/TSCALE);
		    sv_buy_out_2+=s*((currenttime-torg)/TSCALE);
		    if(dt<=tau_int)
		    {
			svsq_buy+=s*getweight(dt);
			svdi_buy_out_as_in+=s*(dt/TSCALE);
		    }
		    else
		    {
			svsq_buy+=s*getweight(tau_int);
			svdi_buy_out_as_in+=s*(tau_int/TSCALE);
		    }
		}
		else
		{
		    v_buy_in+=s;
		    svr_buy+=s*((currenttime-torg)/TSCALE); // recent buy
		    sv_buy_in+=s*(dt/TSCALE);
		    svsq_buy+=s*getweight(dt);
		}
	    }
	    else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
	    {
		svr_buy+=s*((currenttime-t.time)/TSCALE); // limit sell means immediate buy just came
		se_sell+=s*(dt/TSCALE);
		if(torg<=t_range_cut)
		{
		    v_sell_out+=s;
		    sv_sell_out+=s*((te-torg)/TSCALE);
		    sv_sell_out_as_in+=s*((t.time-t_range_cut)/TSCALE);
		    sv_sell_out_2+=s*((currenttime-torg)/TSCALE);
		    if(dt<=tau_int)
		    {
			svsq_sell+=s*getweight(dt);
			svdi_sell_out_as_in+=s*(dt/TSCALE);
		    }
		    else
		    {
			svsq_sell+=s*getweight(tau_int);
			svdi_sell_out_as_in+=s*(tau_int/TSCALE);
		    }
		}
		else
		{
		    v_sell_in+=s;
		    svr_sell+=s*((currenttime-torg)/TSCALE); // recent sell
		    sv_sell_in+=s*(dt/TSCALE);
		    svsq_sell+=s*getweight(dt);
		}
	    }
	    else
	    {
		vc+=s;
	    }
	    spaver+=t.price*s;
	}
	if(flag_lower)
	{
	    if(v_sell_out!=0)
	    {
		if(PRINT_DATA_DEBUG||PRINT_DATA_WARNINGS)
		{
		    System.err.println(
		    "CalculateLiquidityDeficit7:calculateTradesVolumeAndTaus2"+
		    " Trades v_sell_out should be 0"+
		    " v_buy_out="+v_buy_out+
		    " v_sell_out="+v_sell_out+
		    " t_range_cut="+t_range_cut+
		    " tsize="+tsize+
		    " t_range_cut="+t_range_cut+
		    " ptr_interval="+ptr_interval+
		    " np="+(tsize-ptr_interval)+
		    " N_TRADES="+trades.getSize()+
		    " flag_lower="+flag_lower+
		    " currenttime="+BookItem.getTimeDiag(currenttime));
		}
	    }
	}
	else
	{
	    if(v_buy_out!=0)
	    {
		if(PRINT_DATA_DEBUG||PRINT_DATA_WARNINGS)
		{
		    System.err.println(
		    "CalculateLiquidityDeficit7:calculateTradesVolumeAndTaus2"+
		    " Trades v_buy_out should be 0"+
		    " v_buy_out="+v_buy_out+
		    " v_sell_out="+v_sell_out+
		    " t_range_cut="+t_range_cut+
		    " tsize="+tsize+
		    " ptr_interval="+ptr_interval+
		    " N_TRADES="+trades.getSize()+
		    " flag_lower="+flag_lower+
		    " currenttime="+BookItem.getTimeDiag(currenttime));
		}
	    }
	}


	final long v0=v_buy_in+v_buy_out+v_sell_in+v_sell_out+vc;
	final TradesVolumeAndTaus2 tr=
	    new TradesVolumeAndTaus2(t_range_cut,
				     sv_buy_out,
				     sv_buy_in,
				     sv_buy_out_as_in,
				     sv_sell_out,
				     sv_sell_in,
				     sv_sell_out_as_in,
				     flag_lower,
				     v_buy_out,
				     v_buy_in,
				     v_sell_out,
				     v_sell_in,
				     svr_buy,
				     svr_sell,
				     currenttime,
				     v0,
				     svsq_buy,
				     svsq_sell,
				     svdi_buy_out_as_in,
				     svdi_sell_out_as_in,
				     sv_buy_out_2,
				     sv_sell_out_2,
				     se_buy,
				     se_sell,
				     v0>0?spaver/v0:Double.NaN);
	if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // for expensive constrains

	    checkTradeVols(currenttime,t_range_cut,trades,flag_lower,tr,ptr_t_range_cut);
	}
	return tr;
    }


    private static void checkTradeVols(final long currenttime,
				       final long t_range_cut,
				       final Distribution<Trade> trades,
				       final boolean flag_lower,
				       final TradesVolumeAndTaus2 tr,
				       final int ptr_t_range_cut)
    {
	final int tsize=trades.getSize();
	final Trade [] tb=trades.getBuffer();
	long vr_buy=0,vr_sell=0,svr_buy=0,svr_sell=0,rsvr_buy=0,rsvr_sell=0;
	long v_buy_in=0,v_buy_out=0,
	    v_sell_in=0,v_sell_out=0,
	    sv_buy_out=0,sv_sell_out=0,
	    sv_buy_in=0,sv_sell_in=0,
	    sv_buy_out_as_in=0,sv_sell_out_as_in=0,
	    svdi_buy=0,svdi_sell=0,se_buy=0,se_sell=0;
	double svsq_buy=0,svsq_sell=0;
	final long tau_int=currenttime-t_range_cut;
	final int pe=ptr_t_range_cut<0?tsize:ptr_t_range_cut+1;
	for(int i=tsize;i-->pe;)
	{
	    final Trade t=tb[i];
	    final long torg=t.origtime;
	    if(torg<0)
	    {
		continue;
	    }
	    final int s=t.shares;
	    final long te;
	    if(FLAG_TAU_AUTOADJUSTMENT)
	    {
		te=t_range_cut;
	    }
	    else
	    {
		te=t.time;
	    }
	    final long dt=t.time-torg;
	    if(t.buy_sell_indicator==BookItem.BOOK_BUY)
	    {
		vr_sell+=s;
		svr_sell+=s*((currenttime-t.time)/TSCALE);
		rsvr_sell+=s*((t.time-t_range_cut)/TSCALE);
		se_buy+=s*(dt/TSCALE);
		if(torg<=t_range_cut)
		{
		    v_buy_out+=s;
		    sv_buy_out+=s*((te-torg)/TSCALE);
		    sv_buy_out_as_in+=s*((t.time-t_range_cut)/TSCALE);
		    if(dt<=tau_int)
		    {
			svsq_buy+=s*getweight(dt);
			svdi_buy+=s*(dt/TSCALE);
		    }
		    else
		    {
			svsq_buy+=s*getweight(tau_int);
			svdi_buy+=s*(tau_int/TSCALE);
		    }
		}
		else
		{
		    v_buy_in+=s;
		    vr_buy+=s;
		    svr_buy+=s*((currenttime-torg)/TSCALE);
		    sv_buy_in+=s*(dt/TSCALE);
		    svdi_buy+=s*(dt/TSCALE);
		    rsvr_buy+=s*((torg-t_range_cut)/TSCALE);
		    svsq_buy+=s*getweight(dt);
		}
	    }
	    else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
	    {
		vr_buy+=s;
		svr_buy+=s*((currenttime-t.time)/TSCALE);
		rsvr_buy+=s*((t.time-t_range_cut)/TSCALE);
		se_sell+=s*(dt/TSCALE);
		if(torg<=t_range_cut)
		{
		    v_sell_out+=s;
		    sv_sell_out+=s*((te-torg)/TSCALE);
		    sv_sell_out_as_in+=s*((t.time-t_range_cut)/TSCALE);
		    if(dt<=tau_int)
		    {
			svsq_sell+=s*getweight(dt);
			svdi_sell+=s*(dt/TSCALE);
		    }
		    else
		    {
			svsq_sell+=s*getweight(tau_int);
			svdi_sell+=s*(tau_int/TSCALE);
		    }
		}
		else
		{
		    v_sell_in+=s;
		    vr_sell+=s;
		    svr_sell+=s*((currenttime-torg)/TSCALE);
		    sv_sell_in+=s*(dt/TSCALE);
		    svdi_sell+=s*(dt/TSCALE);
		    rsvr_sell+=s*((torg-t_range_cut)/TSCALE);
		    svsq_sell+=s*getweight(dt);
		}
	    }
	}

	if(v_buy_out!=tr.v_buy_out||
	   v_buy_in!=tr.v_buy_in||
	   vr_buy!=(tr.v_buy_in+tr.v_sell_out+tr.v_sell_in)||
	   sv_buy_out!=tr.sv_buy_out||
	   sv_buy_in!=tr.sv_buy_in||
	   sv_buy_out_as_in!=tr.sv_buy_out_as_in||
	   svr_buy!=tr.svr_buy||
	   v_sell_out!=tr.v_sell_out||
	   v_sell_in!=tr.v_sell_in||
	   vr_sell!=(tr.v_sell_in+tr.v_buy_out+tr.v_buy_in)||
	   sv_sell_out!=tr.sv_sell_out||
	   sv_sell_in!=tr.sv_sell_in||
	   sv_sell_out_as_in!=tr.sv_sell_out_as_in||
	   svr_sell!=tr.svr_sell||
	   svdi_buy!=(tr.sv_buy_in+tr.svdi_buy_out_as_in)||
	   svdi_sell!=(tr.sv_sell_in+tr.svdi_sell_out_as_in)||
	   se_buy!=tr.se_buy||
	   se_sell!=tr.se_sell)
	{
	    throw new RuntimeException(
		"Volume mismatch:"+
		" v_buy_out="+v_buy_out+
		" tr.v_buy_out="+tr.v_buy_out+
		" v_buy_in="+v_buy_in+
		" vr_buy="+vr_buy+
		" vr_buy(check)="+(tr.v_buy_in+tr.v_sell_out+tr.v_sell_in)+
		" tr.v_buy_in="+tr.v_buy_in+
		" sv_buy_out="+sv_buy_out+
		" tr.sv_buy_out="+tr.sv_buy_out+
		" sv_buy_in="+sv_buy_in+
		" tr.sv_buy_in="+tr.sv_buy_in+
		" sv_buy_out_as_in="+sv_buy_out_as_in+
		" tr.sv_buy_out_as_in="+tr.sv_buy_out_as_in+
		" svr_buy="+svr_buy+
		" tr.svr_buy="+tr.svr_buy+
		" v_sell_out="+v_sell_out+
		" tr.v_sell_out="+tr.v_sell_out+
		" v_sell_in="+v_sell_in+
		" vr_sell="+vr_sell+
		" vr_sell(check)="+(tr.v_sell_in+tr.v_buy_out+tr.v_buy_in)+
		" tr.v_sell_in="+tr.v_sell_in+
		" sv_sell_out="+sv_sell_out+
		" tr.sv_sell_out="+tr.sv_sell_out+
		" sv_sell_in="+sv_sell_in+
		" tr.sv_sell_in="+tr.sv_sell_in+
		" sv_sell_out_as_in="+sv_sell_out_as_in+
		" tr.sv_sell_out_as_in="+tr.sv_sell_out_as_in+
		" svr_sell="+svr_sell+
		" tr.svr_sell="+tr.svr_sell+
		" svdi_buy="+svdi_buy+
		" (tr.sv_buy_in+tr.svdi_buy_out_as_in)="+(tr.sv_buy_in+tr.svdi_buy_out_as_in)+
		" svdi_sell="+svdi_sell+
		" (tr.sv_sell_in+tr.svdi_sell_out_as_in)="+(tr.sv_sell_in+tr.svdi_sell_out_as_in)+
		" se_buy="+se_buy+
		" tr.se_buy="+tr.se_buy+
		" se_sell="+se_sell+
		" tr.se_sell="+tr.se_sell+
		" t_range_cut="+t_range_cut+
		" TRADES="+trades.getSize()+
		" flag_lower="+flag_lower+
		" currenttime="+currenttime);
	}

	final long v=v_buy_in+v_buy_out+v_sell_in+v_sell_out,
	           se=tau_int/TSCALE;
	final long tau_r_buy=v>0?((svr_buy+se*v_buy_out)/v)*TSCALE:-1,
	    tau_r_sell=v>0?((svr_sell+se*v_sell_out)/v)*TSCALE:-1,
	    tau_re_buy=v>0?((sv_buy_in+sv_buy_out_as_in)/v)*TSCALE:-1,
	    tau_re_sell=v>0?((sv_sell_in+sv_sell_out_as_in)/v)*TSCALE:-1;
	final double eps=TSCALE==1?0:9.0e-5;
	double rel,relL,relE,vE,sq,relsvdi;
	if(v>0&&t_range_cut>=0)
	{
	    final long td=currenttime-t_range_cut;
	    final long dr=((rsvr_buy-rsvr_sell)/v)*TSCALE;
	    rel=dr/((double)td);
	    final long drL=(((svr_buy+se*v_buy_out)-(svr_sell+se*v_sell_out))/v)*TSCALE;
	    relL=-drL/((double)td);
	    relE=-((((sv_buy_in+sv_buy_out_as_in)-(sv_sell_in+sv_sell_out_as_in))/v)*TSCALE)/((double)td);
	    vE=-((v_buy_in+v_buy_out)-(v_sell_in+v_sell_out))/((double)v);
	    sq=-(svsq_buy-svsq_sell)/(v*getweight(currenttime-t_range_cut));
	    relsvdi=-(((svdi_buy-svdi_sell)/v)*TSCALE)/((double)td);
	}
	else
	{
	    rel=relL=relE=vE=sq=relsvdi=Double.NaN;
	}
	if(Double.isNaN(rel)!=Double.isNaN(tr.getTauInDiff())||
	   (!Double.isNaN(rel) && (TSCALE==1?rel!=tr.getTauInDiff():!(Math.abs(rel-tr.getTauInDiff())<=eps)))||
	   Double.isNaN(relL)!=Double.isNaN(tr.getTauInLDiff())||
	   (!Double.isNaN(relL) && relL!=tr.getTauInLDiff())||
	   Double.isNaN(relE)!=Double.isNaN(tr.getETauInDiff())||
	   (!Double.isNaN(relE) && relE!=tr.getETauInDiff())||
	   Double.isNaN(vE)!=Double.isNaN(tr.getEVInDiff())||
	   (!Double.isNaN(vE) && vE!=tr.getEVInDiff())||
	   Double.isNaN(sq)!=Double.isNaN(tr.getSQInDiff())||
	   (!Double.isNaN(sq) && sq!=tr.getSQInDiff())||
	   Double.isNaN(relsvdi)!=Double.isNaN(tr.getEDITauInDiff())||
	   (!Double.isNaN(relsvdi) && relsvdi!=tr.getEDITauInDiff())
	   )
	{
	    throw new RuntimeException(
		"Tau rel mismatch:"+
		" rel="+rel+
		" tr.getTauInDiff()="+tr.getTauInDiff()+
		" tr.getV()="+tr.getV()+
		" eps="+eps+
		" relL="+relL+
		" tr.getTauInLDiff()="+tr.getTauInLDiff()+
		" relE="+relE+
		" tr.getETauInDiff()="+tr.getETauInDiff()+
		" vE="+vE+
		" tr.getEVInDiff()="+tr.getEVInDiff()+
		" sq="+sq+
		" tr.getSQInDiff()="+tr.getSQInDiff()+
		" relsvdi="+relsvdi+
		" tr.getEDITauInDiff()="+tr.getEDITauInDiff()+
		" TRADES="+trades.getSize()+
		" flag_lower="+flag_lower+
		" currenttime="+currenttime);
	}

	final long tau_out,v_out;
	if(flag_lower)
	{
	    v_out=v_buy_out;
	    tau_out=t_range_cut<0?-1:(v_buy_out>0?(sv_buy_out/v_buy_out)*TSCALE:0);
	}
	else
	{
	    v_out=v_sell_out;
	    tau_out=t_range_cut<0?-1:(v_sell_out>0?(sv_sell_out/v_sell_out)*TSCALE:0);
	}
	if(tau_r_buy!=tr.getTauR_Buy()||
	   tau_r_sell!=tr.getTauR_Sell()||
	   tau_re_buy!=tr.getTauRE_Buy()||
	   tau_re_sell!=tr.getTauRE_Sell()||
	   tau_out!=tr.getTauOld()||
	   v_out!=tr.getVOld()||
	   v!=tr.getV()||
	   (vr_buy+v_buy_out)!=v||
	   (vr_sell+v_sell_out)!=v)
	{
	   throw new RuntimeException(
	       "Tau R mismatch:"+
	       " tau_r_buy="+tau_r_buy+
	       " tr.getTauR_Buy()="+tr.getTauR_Buy()+
	       " tau_r_sell="+tau_r_sell+
	       " tr.getTauR_Sell()="+tr.getTauR_Sell()+
	       " tau_out="+tau_out+
	       " tr.getTauOld()="+tr.getTauOld()+
	       " v_out="+v_out+
	       " tr.getVOld()="+tr.getVOld()+
	       " v="+v+
	       " tr.getV()="+tr.getV()+
	       " TRADES="+trades.getSize()+
	       " flag_lower="+flag_lower+
	       " currenttime="+currenttime);
	}
	if(t_range_cut>=0 &&
	   (tau_re_buy>t_range_cut||
	    tau_re_sell>t_range_cut||
	    tau_r_buy>t_range_cut||
	    tau_r_sell>t_range_cut))
	{
	  throw new RuntimeException(
	      "Tau incorrect scale:"+
	      " t_range_cut="+t_range_cut+
	      " tau_re_buy="+tau_re_buy+
	      " tau_re_sell="+tau_re_sell+
	      " tau_r_buy="+tau_r_buy+
	      " tau_r_sell="+tau_r_sell+
	      " TRADES="+trades.getSize()+
	       " flag_lower="+flag_lower+
	      " currenttime="+currenttime);
	}

    }



    static class BookVolumeAndTaus2 {
	final long t_range_cut,v_out,v_in,sv_out,sv_in,currenttime,tau_a_trades_back,v_start,sv_start,ttrade,sv_dist_to_trades_cut,sv_dist_to_trades_total,v_trades,sv_trades_buy,sv_trades_sell;
	final boolean flag_lower;
	final int n_levels_before_start,n_levels;
	BookVolumeAndTaus2(final long v_start,
			   final long sv_start,
			   final boolean flag_lower,
			   final long t_range_cut,
			   final long v_out,
			   final long v_in,
			   final long sv_out,
			   final long sv_in,
			   final long currenttime,
			   final int n_levels_before_start,
			   final int n_levels,
			   final long ttrade,
			   final long sv_dist_to_trades_cut,
			   final long sv_dist_to_trades_total,
			   final long tau_a_trades_back,
			   final long v_trades,
			   final long sv_trades_buy,
			   final long sv_trades_sell)
	{
	    this.v_start=v_start;
	    this.sv_start=sv_start;
	    this.t_range_cut=t_range_cut;
	    this.v_out=v_out;
	    this.v_in=v_in;
	    this.sv_out=sv_out;
	    this.sv_in=sv_in;
	    this.currenttime=currenttime;
	    this.flag_lower=flag_lower;
	    this.n_levels_before_start=n_levels_before_start;
	    this.n_levels=n_levels;
	    this.ttrade=ttrade;
	    this.sv_dist_to_trades_cut=sv_dist_to_trades_cut;
	    this.sv_dist_to_trades_total=sv_dist_to_trades_total;
	    this.tau_a_trades_back=tau_a_trades_back;
	    this.v_trades=v_trades;
	    this.sv_trades_buy=sv_trades_buy;
	    this.sv_trades_sell=sv_trades_sell;
	}
	double getOldFraction()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    return ((double)this.v_out)/v;
	}
	long getV()
	{
	    return this.v_in+this.v_out;
	}
	long getTauOut()
	{
	    return this.v_out>0?(this.sv_out/this.v_out)*TSCALE:-1;

	}
	long getTauIn()
	{
	    return this.v_in>0?(this.sv_in/this.v_in)*TSCALE:-1;
	}
	long getTauTotal()
	{
	    final long v=getV();
	    return v>0?((this.sv_in+this.sv_out)/v)*TSCALE:-1;
	}
	long getTauDistToTrades()
	{
	    final long v=getV();
	    return v>0?(this.sv_dist_to_trades_total/v)*TSCALE:-1;
	}
	long getTauDistToTradesMAX()
	{
	    return this.ttrade<0?-1:this.currenttime-this.ttrade;
	}

	long getTauR()
	{
	    final long v=getV();
	    if(v<=0||this.t_range_cut<0)
	    {
		return -1;
	    }
	    final long sv_cut=this.v_out*((this.currenttime-this.t_range_cut)/TSCALE);
	    return ((sv_cut+this.sv_in)/v)*TSCALE;
	}

	private double getTauDiff(final long t_this,final long t_op,final BookVolumeAndTaus2 opp)
	{
	    if(this.t_range_cut!=opp.t_range_cut ||
	       this.flag_lower==opp.flag_lower)
	    {
		throw new IllegalArgumentException(
		    "t_range_cut mismatch:"+
		    " t_range_cut="+this.t_range_cut+
		    " opp.t_range_cut="+opp.t_range_cut+
		    " flag_lower="+this.flag_lower+
		    " opp.flag_lower="+opp.flag_lower);
	    }
	    if(this.t_range_cut<0||t_this<0||t_op<0)
	    {
		return Double.NaN;
	    }
	    final double td=this.currenttime-this.t_range_cut;
	    return (t_op-t_this)/td;
	}

	double getTauRDiff(final BookVolumeAndTaus2 opp)
	{
	    return getTauDiff(getTauIn(),opp.getTauIn(),opp);
	}
	double getTauREDiff(final BookVolumeAndTaus2 opp)
	{
	    return getTauDiff(getTauR(),opp.getTauR(),opp);
	}
	double getTauTotalDiff(final BookVolumeAndTaus2 opp)
	{
	    return getTauDiff(getTauTotal(),opp.getTauTotal(),opp);
	}

	double getTauRel()
	{
	    final long t=getTauR();
	    if(t<0)
	    {
		return Double.NaN;
	    }
	    return ((double)t)/((double)(this.currenttime-this.t_range_cut));
	}
	double getTauRelA()
	{
	    final long t=getTauR();
	    if(t<0)
	    {
		return Double.NaN;
	    }
	    return ((double)t)/((double)this.tau_a_trades_back);
	}

	double getTauTradesRel()
	{
	    if(this.tau_a_trades_back<0)
	    {
		return Double.NaN;
	    }
	    final double td=this.currenttime-this.t_range_cut;
	    return ((double)this.tau_a_trades_back)/td;
	}

	double getTauInDiff(final BookVolumeAndTaus2 opp)
	{
	    if(this.t_range_cut!=opp.t_range_cut ||
	       this.flag_lower==opp.flag_lower)
	    {
		throw new IllegalArgumentException();
	    }
	    final long w1=this.v_in+this.v_start,
		sv1=this.sv_in+this.sv_start,
		w2=opp.v_in+opp.v_start,
		sv2=opp.sv_in+opp.sv_start,
		v=(this.getV()+this.v_start+opp.getV()+opp.v_start)/2;
	    if(v<=0 ||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    final long td=this.currenttime-this.t_range_cut,ts=td/TSCALE;
	    final long tau=(((w1*ts-sv1)-(w2*ts-sv2))/v)*TSCALE;
	    return (tau)/((double)td);
	}

	double getTauR_Diff(final TradesVolumeAndTaus2 tr)
	{
	    if(this.t_range_cut!=tr.t_range_cut)
	    {
		throw new IllegalArgumentException();
	    }
	    final long t_b=getTauR(),t_tr;
	    if(this.flag_lower)
	    {
		t_tr=tr.getTauR_Buy();
	    }
	    else
	    {
		t_tr=tr.getTauR_Sell();
	    }
	    final long td=this.currenttime-this.t_range_cut;
	    if(t_b<0||t_tr<0||td<=0)
	    {
	        return Double.NaN;
	    }
	    return (t_b-t_tr)/((double)td);
	}

	long getTauTrades_Buy()
	{
	    if(this.ttrade<0||this.v_trades<=0)
	    {
		return -1;
	    }
	    return (this.sv_trades_buy/this.v_trades)*TSCALE;
	}

	long getTauTrades_Sell()
	{
	    if(this.ttrade<0||this.v_trades<=0)
	    {
		return -1;
	    }
	    return (this.sv_trades_sell/this.v_trades)*TSCALE;
	}
    }



    static BookVolumeAndTaus2 calculateBookVolumeAndTauSplitted(
	final long currenttime,
	final Distribution<BookItem> book,
	final Distribution<Trade> trades,
	final boolean flag_lower,
	final long t_range_cut,
	final double p_start,
	final double p_end)
    {
	final int bsize=book.getSize(),tsize=trades.getSize();
	final BookItem [] bb=book.getBuffer();
	final Trade [] tb=trades.getBuffer();
	long ttrades_s=-1,ttrades_s_total=-1,ttrade=-1;
	long v_trades=0,sv_trades_buy=0,sv_trades_sell=0;
	long v_out=0,v_in=0,sv_out=0,sv_in=0,sv_a=0,sv_at=0,v_start=0,sv_start=0;
	int n_levels_before_start=0,n_levels=0,ptr_trades=tsize-1;
	if(flag_lower)
	{
	    final double pexitlev=p_end-OrdersFlow.EPS,
		pstartlev=p_start-OrdersFlow.EPS;
	    double pc=Double.NaN;
	    for(int i=0;i<bsize;i++)
	    {
		final BookItem b=bb[i];
		if(!(b.price<=pstartlev))
		{
		    if(!(b.price>=pc))
		    {
			pc=b.price-OrdersFlow.EPS;
			n_levels_before_start++;
		    }
		    v_start+=b.shares;
		    if(b.time<=t_range_cut)
		    { // old order
			sv_start+=b.shares*((currenttime-t_range_cut)/TSCALE);
		    }
		    else
		    { // new order
			sv_start+=b.shares*((currenttime-b.time)/TSCALE);
		    }
		    continue;
		}
		if(!(b.price>=pexitlev))
		{
		    break;
		}
		if(!(b.price>=pc))
		{
		    pc=b.price-OrdersFlow.EPS;
		    n_levels++;
		    // scan trades
		    if(ptr_trades>=0)
		    {
			for(;;)
			{
			    final Trade t=tb[ptr_trades];
			    if(!(t.price>=pc))
			    {
				ttrade=t.time;
				break;
			    }
			    if(t.origtime>0)
			    {
				v_trades+=t.shares;
				if(t.buy_sell_indicator==BookItem.BOOK_BUY)
				{
				    sv_trades_buy+=t.shares*((currenttime-t.origtime)/TSCALE);
				    sv_trades_sell+=t.shares*((currenttime-t.time)/TSCALE);
				}
				else
				{
				    sv_trades_buy+=t.shares*((currenttime-t.time)/TSCALE);
				    sv_trades_sell+=t.shares*((currenttime-t.origtime)/TSCALE);
				}
			    }
			    ttrade=t.time;
			    if(--ptr_trades<0)
			    {
				//ttrade=-1;
				//v_trades=sv_trades_buy=sv_trades_sell=0;
				break;
			    }
			}
			ttrades_s_total=(currenttime-ttrade)/TSCALE;
			if(ttrade<=t_range_cut)
			{
			    ttrades_s=(currenttime-t_range_cut)/TSCALE;
			}
			else
			{
			    ttrades_s=ttrades_s_total;
			}
		    }
		}
		if(b.time<=t_range_cut)
		{ // old order
		    v_out+=b.shares;
		    sv_out+=b.shares*((currenttime-b.time)/TSCALE);
		}
		else
		{ // new order
		    v_in+=b.shares;
		    sv_in+=b.shares*((currenttime-b.time)/TSCALE);
		}
		sv_a+=b.shares*ttrades_s;
		sv_at+=b.shares*ttrades_s_total;
	    }
	}
	else
	{
	    final double pexitlev=p_end+OrdersFlow.EPS,
		pstartlev=p_start+OrdersFlow.EPS;
	    double pc=Double.NaN;
	    for(int i=0;i<bsize;i++)
	    {
		final BookItem b=bb[i];
		if(!(b.price>=pstartlev))
		{
		    if(!(b.price<=pc))
		    {
			pc=b.price+OrdersFlow.EPS;
			n_levels_before_start++;
		    }
		    v_start+=b.shares;
		    if(b.time<=t_range_cut)
		    { // old order
			sv_start+=b.shares*((currenttime-t_range_cut)/TSCALE);
		    }
		    else
		    { // new order
			sv_start+=b.shares*((currenttime-b.time)/TSCALE);
		    }
		    continue;
		}
		if(!(b.price<=pexitlev))
		{
		    break;
		}
		if(!(b.price<=pc))
		{
		    pc=b.price+OrdersFlow.EPS;
		    n_levels++;
		    // scan trades
		    if(ptr_trades>=0)
		    {
			for(;;)
			{
			    final Trade t=tb[ptr_trades];
			    if(!(t.price<=pc))
			    {
				ttrade=t.time;
				break;
			    }
			    if(t.origtime>0)
			    {
				v_trades+=t.shares;
				if(t.buy_sell_indicator==BookItem.BOOK_BUY)
				{
				    sv_trades_buy+=t.shares*((currenttime-t.origtime)/TSCALE);
				    sv_trades_sell+=t.shares*((currenttime-t.time)/TSCALE);
				}
				else
				{
				    sv_trades_buy+=t.shares*((currenttime-t.time)/TSCALE);
				    sv_trades_sell+=t.shares*((currenttime-t.origtime)/TSCALE);
				}
			    }
			    ttrade=t.time;
			    if(--ptr_trades<0)
			    {
				//ttrade=-1;
				//v_trades=sv_trades_buy=sv_trades_sell=0;
				break;
			    }
			}
			ttrades_s_total=(currenttime-ttrade)/TSCALE;
			if(ttrade<=t_range_cut)
			{
			    ttrades_s=(currenttime-t_range_cut)/TSCALE;
			}
			else
			{
			    ttrades_s=ttrades_s_total;
			}
		    }
		}
		if(b.time<=t_range_cut)
		{ // old order
		    v_out+=b.shares;
		    sv_out+=b.shares*((currenttime-b.time)/TSCALE);
		}
		else
		{ // new order
		    v_in+=b.shares;
		    sv_in+=b.shares*((currenttime-b.time)/TSCALE);
		}
		sv_a+=b.shares*ttrades_s;
		sv_at+=b.shares*ttrades_s_total;
	    }
	}

	final long v=v_in+v_out;
	final BookVolumeAndTaus2 r=new BookVolumeAndTaus2(
	    v_start,
	    sv_start,
	    flag_lower,
	    t_range_cut,
	    v_out,
	    v_in,
	    sv_out,
	    sv_in,
	    currenttime,
	    n_levels_before_start,
	    n_levels,
	    ttrade,
	    sv_a,
	    sv_at,
	    t_range_cut<0 || v<=0?-1:(sv_a/v)*TSCALE,
	    v_trades,
	    sv_trades_buy,
	    sv_trades_sell);

	if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // for expensive constrains
	    checkBootTradeTau(currenttime,
			      book,
			      trades,
			      flag_lower,
			      p_start,
			      p_end,
			      r);
	}
	return r;
    }



    private static void checkBootTradeTau(final long currenttime,
					  final Distribution<BookItem> book,
					  final Distribution<Trade> trades,
					  final boolean flag_lower,
					  final double p_start,
					  final double p_end,
					  final BookVolumeAndTaus2 r)
	{
	    final double pexitlev=flag_lower?p_end-OrdersFlow.EPS:p_end+OrdersFlow.EPS,
		pstartlev=flag_lower?p_start-OrdersFlow.EPS:p_start+OrdersFlow.EPS;
	    final int bsize=book.getSize(),tsize=trades.getSize();
	    final BookItem [] bb=book.getBuffer();
	    final Trade [] tb=trades.getBuffer();

	    long vb=0;
	    double pb=Double.NaN;
	    for(int i=0;i<bsize;i++)
	    {
		final BookItem b=bb[i];
		if(flag_lower?!(b.price<=pstartlev):!(b.price>=pstartlev))
		{
		    vb+=b.shares;
		    continue;
		}
		if(flag_lower?!(b.price>=pexitlev):!(b.price<=pexitlev))
		{
		    break;
		}
		vb+=b.shares;
		pb=b.price;
	    }
	    if(vb!=r.v_in+r.v_out+r.v_start)
	    {
		throw new RuntimeException("Volume mismatch: vb="+vb+
					   " r(v)="+(r.v_in+r.v_out+r.v_start)+
					   " flag_lower="+flag_lower+
					   " N_ORDERS="+book.getSize()+
					   " N_TRADES="+trades.getSize()+
					   " currenttime="+currenttime);
	    }
	    final double p_start_cut=flag_lower?p_start-OrdersFlow.EPS:p_start+OrdersFlow.EPS;
	    final double pcut=flag_lower?pb-OrdersFlow.EPS:pb+OrdersFlow.EPS;
	    long v_trades=0,sv_trades_buy=0,sv_trades_sell=0,ttrade=-1;
	    if(!Double.isNaN(pcut)&&(flag_lower?pb<p_start_cut:pb>p_start_cut))
	    {
		for(int i=tsize;i-->0;)
		{
		    final Trade t=tb[i];
		    if(flag_lower?!(t.price>=pcut):!(t.price<=pcut))
		    {
			ttrade=t.time;
			break;
		    }
		    if(t.origtime>0)
		    {
			v_trades+=t.shares;
			if(t.buy_sell_indicator==BookItem.BOOK_BUY)
			{
			    sv_trades_buy+=t.shares*((currenttime-t.origtime)/TSCALE);
			    sv_trades_sell+=t.shares*((currenttime-t.time)/TSCALE);
			}
			else
			{
			    sv_trades_buy+=t.shares*((currenttime-t.time)/TSCALE);
			    sv_trades_sell+=t.shares*((currenttime-t.origtime)/TSCALE);
			}
		    }
		    ttrade=t.time;
		}
		if(ttrade<0)
		{
		    v_trades=sv_trades_buy=sv_trades_sell=0;
		}
	    }

	    if(v_trades!=r.v_trades||
	       sv_trades_buy!=r.sv_trades_buy||
	       sv_trades_sell!=r.sv_trades_sell||
	       ttrade!=r.ttrade)
	    {
		throw new RuntimeException("Time mismatch: "+
					   " v_trades="+v_trades+
					   " r.v_trades="+r.v_trades+
					   " sv_trades_buy="+sv_trades_buy+
					   " r.sv_trades_buy="+r.sv_trades_buy+
					   " sv_trades_sell="+sv_trades_sell+
					   " r.sv_trades_sell="+r.sv_trades_sell+
					   " ttrade="+ttrade+
					   " r.ttrade="+r.ttrade+
					   " flag_lower="+flag_lower+
					   " N_ORDERS="+book.getSize()+
					   " N_TRADES="+trades.getSize()+
					   " currenttime="+currenttime);
	    }
	}


    private static double getweight(final long tau)
    {
	//return Math.sqrt(tau);
	final double tr=tau;
	return tr*tr;
    }

    static class TradesVolumeAndTausMedian {
	/** Result. */
	final long t_range_cut,currenttime,vi,tau_med_diff,tau_med_sum,v_buy,v_sell;
	TradesVolumeAndTausMedian(final long t_range_cut,
				  final long currenttime,
				  final long vi,
				  final long tau_med_diff,
				  final long tau_med_sum,
				  final long v_buy,
				  final long v_sell)
	{
	    this.t_range_cut=t_range_cut;
	    this.currenttime=currenttime;
	    this.vi=vi;
	    this.tau_med_diff=tau_med_diff;
	    this.tau_med_sum=tau_med_sum;
	    this.v_buy=v_buy;
	    this.v_sell=v_sell;
	}

	double getTauMedRel()
	{
	    if(this.vi<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    return -this.tau_med_diff/((double)(this.currenttime-this.t_range_cut));
	}

	double getTauMedRelS()
	{
	    if(this.vi<=0||this.t_range_cut<0||this.tau_med_sum<=0)
	    {
		return Double.NaN;
	    }
	    return -this.tau_med_diff/((double)this.tau_med_sum);
	}

	double getVRel()
	{
	    if(this.vi<=0||this.t_range_cut<0)
	    {
		return Double.NaN;
	    }
	    return -(this.v_buy-this.v_sell)/((double)this.vi);
	}
    }

    static TradesVolumeAndTausMedian calculateTradesVolumeAndTausMedian(
	final long currenttime,
	final Distribution<Trade> trades,
	final Trade [] tmpbuf,
	final int ptr_t_range_cut)
    {

	final boolean PRINT_DATA_DEBUG=false;

	final int tsize=trades.getSize();
	final Trade [] origtb=trades.getBuffer();
	final long t_range_cut=ptr_t_range_cut<0?-1:origtb[ptr_t_range_cut].time;
	final int ptr_interval=ptr_t_range_cut<0?tsize:ptr_t_range_cut+1;
	long vi=0,v_buy=0,v_sell=0;
	for(int i=tsize;i-->ptr_interval;)
	{
	    final Trade t=origtb[i];
	    if(t.origtime<0)
	    {
		continue;
	    }
	    if(t.buy_sell_indicator==BookItem.BOOK_BUY)
	    {
		v_buy+=t.shares;
	    }
	    else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
	    {
		v_sell+=t.shares;
	    }
	    vi+=t.shares;
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println();
	    System.err.println(
		"=="+
		"============ CalculateLiquidityDeficit7.calculateTradesVolumeAndTausMedian t=\t"+currenttime+
		"\tNTRADES=\t"+trades.getSize()+
		"\tptr_interval=\t"+ptr_interval+
		" vi="+vi);
	}

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("num\t"+
			       "time\t"+
			       "origtime\t"+
			       "shares\t"+
			       "tradetype\t"+
			       "buy_sell_indicator\t"+
			       "price\t"+
			       "matchnumber\t"+
			       "orderreferencenumber");
	}

	final Trade [] tb=tmpbuf;
	final int nelements=tsize-ptr_interval;
	System.arraycopy(origtb,ptr_interval,tb,0,nelements);
	com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(tb,0,nelements-1,CompareItems.cmp_by_type_and_execution_D);
	final long tau_med_diff=calculateMedianTau(tb,currenttime,nelements,vi,FLAG_DIFF);

	if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // ================== for expensive constrains
	    int n_buy=0,n_buy0=0,n_sell=0;
	    long prev_tau=Long.MAX_VALUE,vm=0;
	    for(int i=0;i<nelements;i++)
	    {
		final Trade t=tb[i];
		if(t.origtime<0)
		{
		    break;
		}
		vm+=t.shares;
		final long tau=getTau(t,FLAG_DIFF);
		if(t.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    if(n_sell>0)
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    n_buy++;
		}
		else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    if(n_sell<=0)
		    {
			// first sell
			n_buy0=n_buy;
		    }
		    if(n_buy>n_buy0)
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    n_sell++;
		}
		else
		{
		    throw new RuntimeException("Ordering is broken");
		}
		if(tau>prev_tau)
		{
		    throw new RuntimeException("Ordering is broken");
		}
		prev_tau=tau;
	    }
	    if(vm!=vi)
	    {
		throw new RuntimeException("Volume mismatch: "+
					   " vm="+vm+
					   " vi="+vi+
					   " currenttime="+currenttime);
	    }
	} // ================== end constrains


	System.arraycopy(origtb,ptr_interval,tb,0,nelements);
	com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(tb,0,nelements-1,CompareItems.cmp_by_execution_S);
	final long tau_med_sum=calculateMedianTau(tb,currenttime,nelements,vi,FLAG_SUM);

	if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{ // ================== for expensive constrains
	    long prev_tau=Long.MAX_VALUE,vm=0;
	    for(int i=0;i<nelements;i++)
	    {
		final Trade t=tb[i];
		if(t.origtime<0)
		{
		    break;
		}
		vm+=t.shares;
		final long tau=getTau(t,FLAG_SUM);
		if(tau>prev_tau)
		{
		    throw new RuntimeException("Ordering is broken");
		}
		prev_tau=tau;
	    }
	    if(vm!=vi)
	    {
		throw new RuntimeException("Volume mismatch: "+
					   " vm="+vm+
					   " vi="+vi+
					   " currenttime="+currenttime);
	    }
	} // ================== end constrains

	return new TradesVolumeAndTausMedian(t_range_cut,
					     currenttime,
					     vi,
					     tau_med_diff,
					     tau_med_sum,
					     v_buy,
					     v_sell);
    }


    static long calculateMedianTau(final Trade [] tb,
				   final long currenttime,
				   final int nelements,
				   final long vi,
				   final int tau_flags)
    {
	final boolean PRINT_DATA_DEBUG=false;

	long vm=0;
	boolean med_not_set=true;
	long tau_med=0;
	final long vi2=vi/2;
	for(int i=0;i<nelements;i++)
	{
	    final Trade t=tb[i];
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println(String.valueOf(i)+"\t"+
				   t.time+"\t"+
				   t.origtime+"\t"+
				   t.shares+"\t"+
				   t.tradetype+"\t"+
				   t.buy_sell_indicator+"\t"+
				   t.price+"\t"+
				   t.matchnumber+"\t"+
				   t.orderreferencenumber);
	    }
	    if(t.origtime<0)
	    {
		break;
	    }
	    vm+=t.shares;
	    if(vm>vi2)
	    {
		if(med_not_set)
		{
		    med_not_set=false;
		    if(vi-vm+t.shares>vi2)
		    {
			tau_med=getTau(t,tau_flags);
		    }
		    else
		    {
			if(i<=0)
			{
			    throw new RuntimeException("This should never happen");
			}
			long th=getTau(t,tau_flags),tl=getTau(tb[i-1],tau_flags),vh2=2*vm,vl2=2*(vm-t.shares);
			tau_med=(th*(vi-vl2)+tl*(vh2-vi))/(vh2-vl2);
		    }
		}
	    }
	}
	if(vm!=vi)
	{
	    throw new RuntimeException("Volume mismatch: "+
				       " vm="+vm+
				       " vi="+vi+
				       " currenttime="+currenttime);
	}
	return tau_med;
    }

    static final int FLAG_DIFF=89,FLAG_SUM=90;
    static long getTau(final Trade t,final int tau_flags)
    {
	if(tau_flags==FLAG_SUM)
	{
	    return t.time-t.origtime;
	}

	if(t.buy_sell_indicator==BookItem.BOOK_BUY)
	{
	    return t.time-t.origtime;
	}
	else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
	{
	    return t.origtime-t.time;
	}
	throw new RuntimeException("Ordering is broken");
    }


}
