package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit.
 *  Den' os'moi. D.I.Donskoi i "zasadnyi polk".
 *  @version $Id: CalculateLiquidityDeficit8.java,v 1.54 2010/04/13 13:45:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit8 {


    private final Distribution<BookItem> buy,sell;
    final Distribution<Trade> trades;

    CalculateLiquidityDeficit8(final Distribution<BookItem> buy,
			       final Distribution<BookItem> sell,
			       final Distribution<Trade> trades,
			       final Distribution<BookItem> tmpordersbuf,
			       final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
    }


    LDData [] vlev=null;

    void clear()
    {
	this.vlev=null;
    }


    void calculateLiquidityDeficit(final long currenttime,
				   final double [] delta_ok,
				   final double [] delta_faiure,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final long trades_cleared_before)
    {
	final int nt=delta_ok.length;
	if(delta_faiure.length!=nt)
	{
	    throw new IllegalArgumentException();
	}

	final boolean PRINT_DATA_DEBUG=false;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tN_ORDERS BUY\t"+this.buy.getSize()+
			       "\tN_ORDERS SELL\t"+this.sell.getSize()+
			       "\tnt\t"+nt+
			       "\ncur_buy\t"+pcurrent_buy+
			       "\tcur_sell\t"+pcurrent_sell);
	}

	this.vlev=new LDData [nt];

	for(int i=0;i<nt;i++)
	{
	    final double d_ok=delta_ok[i],d_fail=delta_faiure[i];

	    // BUY side

	    final CalculateLiquidityDeficit7.PTRToRangeAndPE ptr_buy=
		CalculateLiquidityDeficit7.findPtrToTrades(
		    currenttime,
		    this.trades,
		    true,
		    pcurrent_buy+d_ok,
		    pcurrent_sell+d_ok,
		    pcurrent_buy-d_fail);

	    final CalculateLiquidityDeficit7.BookVolumeAndTaus2 book_buy_st=
		CalculateLiquidityDeficit7.calculateBookVolumeAndTauSplitted(
		    currenttime,
		    this.buy,
		    this.trades,
		    true,
		    ptr_buy.t_range_cut,
		    ptr_buy.p_opposite,
		    ptr_buy.p_opposite-d_ok);

	    final CalculateLiquidityDeficit7.TradesVolumeAndTaus2 tr_buy=
		CalculateLiquidityDeficit7.calculateTradesVolumeAndTaus2(
		    currenttime,
		    this.trades,
		    true,
		    ptr_buy.ptr_t_range_cut);
	    final CalculateKendall [] r_buy=
		CalculateKendall.calculateTradesKendallRankTOrigDiff(
		    currenttime,
		    this.trades,
		    ptr_buy.ptr_t_range_cut,
		    this.trades.getSize());
	    final  CalculateKendall [] rr3_buy=CalculateKendall.calculateTradesKendallRankTP3OrigDiff(
		currenttime,
		this.trades,
		ptr_buy.ptr_t_range_cut,
		this.trades.getSize());

	    final CalculateLiquidityDeficit7.BookVolumeAndTaus2 book_no_sell=
		CalculateLiquidityDeficit7.calculateBookVolumeAndTauSplitted(
		    currenttime,
		    this.sell,
		    this.trades,
		    false,
		    ptr_buy.t_range_cut,
		    Double.NEGATIVE_INFINITY,
		    pcurrent_sell+d_ok);

	    // SELL side

	    final CalculateLiquidityDeficit7.PTRToRangeAndPE ptr_sell=
		CalculateLiquidityDeficit7.findPtrToTrades(
		    currenttime,
		    this.trades,
		    false,
		    pcurrent_sell-d_ok,
		    pcurrent_buy-d_ok,
		    pcurrent_sell+d_fail);

	    final CalculateLiquidityDeficit7.BookVolumeAndTaus2 book_sell_st=
		CalculateLiquidityDeficit7.calculateBookVolumeAndTauSplitted(
		    currenttime,
		    this.sell,
		    this.trades,
		    false,
		    ptr_sell.t_range_cut,
		    ptr_sell.p_opposite,
		    ptr_sell.p_opposite+d_ok);

	    final CalculateLiquidityDeficit7.TradesVolumeAndTaus2 tr_sell=
		CalculateLiquidityDeficit7.calculateTradesVolumeAndTaus2(
		    currenttime,
		    this.trades,
		    false,
		    ptr_sell.ptr_t_range_cut);
	    final CalculateKendall [] r_sell=
		CalculateKendall.calculateTradesKendallRankTOrigDiff(
		    currenttime,
		    this.trades,
		    ptr_sell.ptr_t_range_cut,
		    this.trades.getSize());
	    final  CalculateKendall [] rr3_sell=CalculateKendall.calculateTradesKendallRankTP3OrigDiff(
		currenttime,
		this.trades,
		ptr_sell.ptr_t_range_cut,
		this.trades.getSize());

	    final CalculateLiquidityDeficit7.BookVolumeAndTaus2 book_no_buy=
		CalculateLiquidityDeficit7.calculateBookVolumeAndTauSplitted(
		    currenttime,
		    this.buy,
		    this.trades,
		    true,
		    ptr_sell.t_range_cut,
		    Double.POSITIVE_INFINITY,
		    pcurrent_buy-d_ok);

	    this.vlev[i]=new LDData(d_ok,
				    d_fail,
				    ptr_buy,
				    book_buy_st,
				    book_no_sell,
				    ptr_buy.p_opposite,
				    tr_buy,
				    pcurrent_sell+d_ok,
				    ptr_sell,
				    book_sell_st,
				    book_no_buy,
				    ptr_sell.p_opposite,
				    tr_sell,
				    pcurrent_buy-d_ok,

				    r_buy[0],
				    r_buy[1],
				    r_buy[2],
				    r_buy[6],
				    r_sell[0],
				    r_sell[1],
				    r_sell[2],
				    r_sell[6],


				    rr3_buy[0],
				    rr3_buy[1],
				    rr3_buy[3],
				    rr3_buy[4],
				    rr3_sell[0],
				    rr3_sell[1],
				    rr3_sell[3],
				    rr3_sell[4]);
	}
    }


    static class LDData {
	final double p_buy_l0,p_sell_l0,pinc_buy,pinc_sell,d_ok,d_fail;
	private final CalculateLiquidityDeficit7.PTRToRangeAndPE ptr_buy,ptr_sell;
	final CalculateLiquidityDeficit7.BookVolumeAndTaus2
	    book_buy_st,book_sell_st,book_no_buy,book_no_sell;
	final CalculateLiquidityDeficit7.TradesVolumeAndTaus2 tr_buy,tr_sell;
	final CalculateKendall buy_trades_krank,buy_trades_krank_buy,buy_trades_krank_sell,buy_trades_krank_tau,
	    sell_trades_krank,sell_trades_krank_buy,sell_trades_krank_sell,sell_trades_krank_tau;
	CalculateKendall trades0_dir_buy,trades2_dir_buy,trades3_dir_buy,trades3_ddir_buy,
	    trades0_dir_sell,trades2_dir_sell,trades3_dir_sell,trades3_ddir_sell;
	LDData(final double d_ok,
	       final double d_fail,
	       final CalculateLiquidityDeficit7.PTRToRangeAndPE ptr_buy,
	       final CalculateLiquidityDeficit7.BookVolumeAndTaus2 book_buy_st,
	       final CalculateLiquidityDeficit7.BookVolumeAndTaus2 book_no_sell,
	       final double p_buy_l0,
	       final CalculateLiquidityDeficit7.TradesVolumeAndTaus2 tr_buy,
	       final double pinc_buy,
	       final CalculateLiquidityDeficit7.PTRToRangeAndPE ptr_sell,
	       final CalculateLiquidityDeficit7.BookVolumeAndTaus2 book_sell_st,
	       final CalculateLiquidityDeficit7.BookVolumeAndTaus2 book_no_buy,
	       final double p_sell_l0,
	       final CalculateLiquidityDeficit7.TradesVolumeAndTaus2 tr_sell,
	       final double pinc_sell,
	       final CalculateKendall buy_trades_krank,
	       final CalculateKendall buy_trades_krank_buy,
	       final CalculateKendall buy_trades_krank_sell,
	       final CalculateKendall buy_trades_krank_tau,
	       final CalculateKendall sell_trades_krank,
	       final CalculateKendall sell_trades_krank_buy,
	       final CalculateKendall sell_trades_krank_sell,
	       final CalculateKendall sell_trades_krank_tau,
	       final CalculateKendall trades0_dir_buy,
	       final CalculateKendall trades2_dir_buy,
	       final CalculateKendall trades3_dir_buy,
	       final CalculateKendall trades3_ddir_buy,
	       final CalculateKendall trades0_dir_sell,
	       final CalculateKendall trades2_dir_sell,
	       final CalculateKendall trades3_dir_sell,
	       final CalculateKendall trades3_ddir_sell)
	{
	    this.ptr_buy=ptr_buy;
	    this.book_buy_st=book_buy_st;
	    this.book_no_sell=book_no_sell;
	    this.p_buy_l0=p_buy_l0;
	    this.tr_buy=tr_buy;
	    this.pinc_buy=pinc_buy;
	    this.ptr_sell=ptr_sell;
	    this.book_sell_st=book_sell_st;
	    this.book_no_buy=book_no_buy;
	    this.p_sell_l0=p_sell_l0;
	    this.tr_sell=tr_sell;
	    this.pinc_sell=pinc_sell;
	    this.d_ok=d_ok;
	    this.d_fail=d_fail;
	    this.buy_trades_krank=buy_trades_krank;
	    this.buy_trades_krank_buy=buy_trades_krank_buy;
	    this.buy_trades_krank_sell=buy_trades_krank_sell;
	    this.buy_trades_krank_tau=buy_trades_krank_tau;
	    this.sell_trades_krank=sell_trades_krank;
	    this.sell_trades_krank_buy=sell_trades_krank_buy;
	    this.sell_trades_krank_sell=sell_trades_krank_sell;
	    this.sell_trades_krank_tau=sell_trades_krank_tau;


	    this.trades0_dir_buy=trades0_dir_buy;
	    this.trades2_dir_buy=trades2_dir_buy;
	    this.trades3_dir_buy=trades3_dir_buy;
	    this.trades3_ddir_buy=trades3_ddir_buy;
	    this.trades0_dir_sell=trades0_dir_sell;
	    this.trades2_dir_sell=trades2_dir_sell;
	    this.trades3_dir_sell=trades3_dir_sell;
	    this.trades3_ddir_sell=trades3_ddir_sell;
	}
    }

}
