package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit v9.
 *  @version $Id: CalculateLiquidityDeficit9.java,v 1.118 2010/07/08 16:49:58 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit9 {

    private final Distribution<BookItem> buy,sell,tmpordersbuf;
    private final Distribution<Trade> trades,tmptradesbuf;

    CalculateLiquidityDeficit9(final Distribution<BookItem> buy,
			       final Distribution<BookItem> sell,
			       final Distribution<Trade> trades,
			       final Distribution<BookItem> tmpordersbuf,
			       final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
	this.tmpordersbuf=tmpordersbuf;
	this.tmptradesbuf=tmptradesbuf;
    }


    double []
	price_book_buy=null,
	price_book_sell=null;
    int [] n_levels_buy=null,n_levels_sell=null;

    CalculateLiquidityDeficit7.TradesVolumeAndTaus2 [] tdiff;
    CalculateLiquidityDeficit7.BookVolumeAndTaus2 [] book_buy,book_sell;
    CalculateLiquidityDeficit7.TradesVolumeAndTausMedian [] tmed;
    CalculateKendall [] trades_krank,trades_krank_buy,trades_krank_sell,trades_krank_bs,trades_krank_tau,trades_krank_cd,trades_krank_XY,trades_krank_self,book_krank_XY,book_krank_cumm;
    CalculateKendall [] trades_0krank,trades_0krank_buy,trades_0krank_sell,trades_0krank_dir,trades_dkrank,trades_dkrank_buy,trades_dkrank_sell,trades_book_pkrank_buy,trades_book_pkrank_sell;
    CalculateKendall [] book_side_buy,book_side_sell;
    CalculateKendall.KendallBookTradesCoupling [] trades_book_buy_dir,trades_book_sell_dir,trades_trades_Dcoupled;

    CalculateKendall [] trades0_dir,trades3_dir,trades3_ddir,ntrades0_dir,ntrades2_dir,ntrades3_dir,trades_dbs,trades_D3,trades_D2,trades_D2_buy,trades_D2_sell,trades_Dcoupled_buy,trades_Dcoupled_sell,trades_D2sum,book_trades_Dcoupled,trades_ko,trades_ko_orig;

    CalculateKendall.KendallMStats [] trades_p;
    CalculateKendall.MSAverageKendall [] trades_krank_forward;
    CalculateKendall.DAvergeCalculateKendall [] trades_krank_VS;

    void clear()
    {

	this.price_book_buy=this.price_book_sell=null;
	this.n_levels_buy=this.n_levels_sell=null;
	this.tdiff=null;
	this.tmed=null;
	this.trades_krank=this.trades_krank_buy=this.trades_krank_sell=this.trades_krank_bs=this.trades_krank_tau=this.trades_krank_cd=this.trades_krank_XY=this.trades_krank_self=this.book_krank_XY=this.book_krank_cumm=null;
	this.book_buy=this.book_sell=null;
	this.trades_0krank=this.trades_0krank_buy=this.trades_0krank_sell=this.trades_0krank_dir=this.trades_dkrank=this.trades_dkrank_buy=this.trades_dkrank_sell=this.trades_book_pkrank_buy=this.trades_book_pkrank_sell=null;
	this.book_side_buy=this.book_side_sell=null;
	this.trades_book_buy_dir=this.trades_book_sell_dir=this.trades_trades_Dcoupled=null;
	this.trades0_dir=this.trades3_dir=this.trades3_ddir=null;
	this.ntrades0_dir=this.ntrades2_dir=this.ntrades3_dir=this.trades_dbs=this.trades_D3=this.trades_D2=this.trades_D2_buy=this.trades_D2_sell=this.trades_Dcoupled_buy=this.trades_Dcoupled_sell=this.trades_D2sum=null;
	this.book_trades_Dcoupled=null;
	this.trades_ko=this.trades_ko_orig=null;

	this.trades_p=null;
	this.trades_krank_forward=null;
	this.trades_krank_VS=null;
    }


    void calculateLiquidityDeficit(final double [] buy_pricelevels,
				   final double [] sell_pricelevels,
				   final long currenttime,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final long trades_cleared_before,
				   final int vol_fact,
				   final long vol_lev0)
    {
	final int nt=buy_pricelevels.length;
	if(sell_pricelevels.length!=nt)
	{
	    throw new IllegalArgumentException();
	}

	final boolean PRINT_DATA_DEBUG=false;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tN_ORDERS BUY\t"+this.buy.getSize()+
			       "\tN_ORDERS SELL\t"+this.sell.getSize()+
			       "\tnt\t"+nt+
			       "\ncur_buy\t"+pcurrent_buy+
			       "\tcur_sell\t"+pcurrent_sell);
	}


	final int bsizebuy=this.buy.getSize(),bsizesell=this.sell.getSize();
	final BookItem [] bbuy=this.buy.getBuffer(),bsell=this.sell.getBuffer();
	final double [] tmp=new double [nt],lev=new double [nt];
	final long [] tmpl=new long [nt];

	final int [] ptr_book_buy=new int [nt];
	final long [] vol_book_buy=new long [nt],tau_book_unnormalized_buy=new long [nt];
	this.n_levels_buy=new int [nt];
	for(int i=0;i<nt;i++)
	{
	    lev[i]=buy_pricelevels[i]-OrdersFlow.EPS;
	}
	CalculateLiquidityDeficit5.calculateBookLevelsStat(
	    lev,
	    currenttime,
	    true,
	    bbuy,
	    bsizebuy,
	    vol_book_buy,
	    tau_book_unnormalized_buy,
	    tmpl,
	    ptr_book_buy,
	    tmp,
	    this.n_levels_buy);


	final int [] ptr_book_sell=new int [nt];
	final long [] vol_book_sell=new long [nt],tau_book_unnormalized_sell=new long [nt];
	this.n_levels_sell=new int [nt];
	for(int i=0;i<nt;i++)
	{
	    lev[i]=sell_pricelevels[i]+OrdersFlow.EPS;
	}
	CalculateLiquidityDeficit5.calculateBookLevelsStat(
	    lev,
	    currenttime,
	    false,
	    bsell,
	    bsizesell,
	    vol_book_sell,
	    tau_book_unnormalized_sell,
	    tmpl,
	    ptr_book_sell,
	    tmp,
	    this.n_levels_sell);


	this.price_book_buy=new double [nt];
	this.price_book_sell=new double [nt];
	for(int i=0;i<nt;i++)
	{
	    this.price_book_buy[i]=ptr_book_buy[i]<0?Double.NaN:bbuy[ptr_book_buy[i]].price;
	    this.price_book_sell[i]=ptr_book_sell[i]<0?Double.NaN:bsell[ptr_book_sell[i]].price;
	}

	this.tdiff=new CalculateLiquidityDeficit7.TradesVolumeAndTaus2 [nt];
	this.tmed=new CalculateLiquidityDeficit7.TradesVolumeAndTausMedian [nt];
	this.trades_krank=new CalculateKendall [nt];
	this.trades_krank_buy=new CalculateKendall [nt];
	this.trades_krank_sell=new CalculateKendall [nt];
	this.trades_krank_bs=new CalculateKendall [nt];
	this.trades_krank_tau=new CalculateKendall [nt];
	this.trades_krank_cd=new CalculateKendall [nt];
	this.trades_krank_XY=new CalculateKendall [nt];
	this.trades_krank_self=new CalculateKendall [nt];
	this.book_krank_XY=new CalculateKendall [nt];
	this.book_krank_cumm=new CalculateKendall [nt];

	this.trades_0krank=new CalculateKendall [nt];
	this.trades_0krank_buy=new CalculateKendall [nt];
	this.trades_0krank_sell=new CalculateKendall [nt];
	this.trades_0krank_dir=new CalculateKendall [nt];
	this.trades_dkrank=new CalculateKendall [nt];
	this.trades_dkrank_buy=new CalculateKendall [nt];
	this.trades_dkrank_sell=new CalculateKendall [nt];
	this.trades_book_pkrank_buy=new CalculateKendall [nt];
	this.trades_book_pkrank_sell=new CalculateKendall [nt];


	this.trades_book_buy_dir=new CalculateKendall.KendallBookTradesCoupling [nt];
	this.trades_book_sell_dir=new CalculateKendall.KendallBookTradesCoupling [nt];
	this.trades_trades_Dcoupled=new CalculateKendall.KendallBookTradesCoupling [nt];
	this.book_side_buy=new CalculateKendall [nt];
	this.book_side_sell=new CalculateKendall [nt];

	this.trades0_dir=new CalculateKendall [nt];
	this.trades3_dir=new CalculateKendall [nt];
	this.trades3_ddir=new CalculateKendall [nt];

	this.ntrades0_dir=new CalculateKendall [nt];
	this.ntrades2_dir=new CalculateKendall [nt];
	this.ntrades3_dir=new CalculateKendall [nt];
	this.trades_dbs=new CalculateKendall [nt];
	this.trades_D3=new CalculateKendall [nt];
	this.trades_D2=new CalculateKendall [nt];
	this.trades_D2_buy=new CalculateKendall [nt];
	this.trades_D2_sell=new CalculateKendall [nt];
	this.trades_Dcoupled_buy=new CalculateKendall [nt];
	this.trades_Dcoupled_sell=new CalculateKendall [nt];
	this.trades_D2sum=new CalculateKendall [nt];
	this.book_trades_Dcoupled=new CalculateKendall [nt];
	this.trades_p=new CalculateKendall.KendallMStats [nt];

	this.book_buy=new CalculateLiquidityDeficit7.BookVolumeAndTaus2 [nt];
	this.book_sell=new CalculateLiquidityDeficit7.BookVolumeAndTaus2 [nt];

	this.trades_ko=new CalculateKendall [nt];
	this.trades_ko_orig=new CalculateKendall [nt];
	this.trades_krank_forward=new CalculateKendall.MSAverageKendall [nt];
	this.trades_krank_VS=new CalculateKendall.DAvergeCalculateKendall [nt];

	for(int i=0;i<nt;i++)
	{

	    final long v_cut=vol_fact*(vol_book_buy[i]+vol_book_sell[i]);
	    final int ptr=findPtrToV(currenttime,v_cut,trades.getSize(),trades.getBuffer());
	    this.tdiff[i]=CalculateLiquidityDeficit7.calculateTradesVolumeAndTaus2(
		currenttime,
		this.trades,
		true, // unused
		ptr);
	    this.tmed[i]=CalculateLiquidityDeficit7.calculateTradesVolumeAndTausMedian(
		currenttime,
		this.trades,
		this.tmptradesbuf.getBuffer(),
		ptr);
	    if(this.tdiff[i].vi!=this.tmed[i].vi)
	    {
		throw new RuntimeException(
		    "Volume mismatch:"+
		    " tdiff["+i+"]="+this.tdiff[i].vi+
		    " tmed["+i+"].vi="+this.tmed[i].vi+
		    " N_TRADES="+this.trades.getSize()+
		    " currenttime="+currenttime);
	    }
	    final CalculateKendall [] r=
		CalculateKendall.calculateTradesKendallRankTOrigDiff(
		    currenttime,
		    this.trades,
		    ptr,
		    this.trades.getSize());
	    this.trades_krank[i]=r[0];
	    this.trades_krank_buy[i]=r[1];
	    this.trades_krank_sell[i]=r[2];
	    this.trades_krank_bs[i]=r[3];
	    this.trades_krank_self[i]=r[4];
	    this.trades_krank_XY[i]=r[5];
	    this.trades_krank_tau[i]=r[6];
	    this.trades_krank_cd[i]=r[7];

	    final CalculateKendall [] rr=
		CalculateKendall.calculateTradesKendallRankTPOrigDiff(
		    currenttime,
		    this.trades,
		    ptr,
		    this.trades.getSize());
	    this.trades_0krank[i]=rr[0];
	    this.trades_0krank_buy[i]=rr[1];
	    this.trades_0krank_sell[i]=rr[2];
	    this.trades_0krank_dir[i]=rr[3];
	    this.trades_dkrank[i]=rr[4];

	    this.trades_dkrank_buy[i]= // FIXME, for now
		this.trades_dkrank_sell[i]=this.trades_0krank_dir[i];

	    final int ptr_delayed=findPtrToV(currenttime,v_cut,ptr,trades.getBuffer());
	    final CalculateKendall k1=
		CalculateKendall.calculateTradesKendallRankTP2OrigDiff(
		    currenttime,
		    this.trades,
		    ptr_delayed,
		    ptr).krank2,
		k2=CalculateKendall.calculateTradesKendallRankTP2OrigDiff(
		    currenttime,
		    this.trades,
		    ptr,
		    this.trades.getSize()).krank2,
		ktot=CalculateKendall.calculateTradesKendallRankTP2OrigDiff(
		    currenttime,
		    this.trades,
		    ptr_delayed,
		    this.trades.getSize()).krank2;
	    this.trades_ko[i]=new CalculateKendall(currenttime,
						   k1.vi+k2.vi,
						   k1.v_cons+k2.v_cons,
						   k1.v_discons+k2.v_discons,
						   k1.v_zero+k2.v_zero+k1.vi*k2.vi);

	    this.trades_ko_orig[i]=ktot;


	    this.trades_book_buy_dir[i]=CalculateKendall.calculateBookSideTradesKendall(
		currenttime,
		1+ptr_book_buy[i],
		this.buy.getBuffer(),
		this.trades,
		ptr);
	    final double bnbuy=this.trades_book_buy_dir[i].krank_forward_cut.vi>0?this.trades_book_buy_dir[i].krank_forward_cut.vi:Double.NaN;
	    final CalculateKendall bt_coupling_buy=this.trades_book_buy_dir[i].krank_coupling;


	    this.trades_book_sell_dir[i]=CalculateKendall.calculateBookSideTradesKendall(
		currenttime,
		1+ptr_book_sell[i],
		this.sell.getBuffer(),
		this.trades,
		ptr);
	    final double bnsell=this.trades_book_sell_dir[i].krank_forward_cut.vi>0?this.trades_book_sell_dir[i].krank_forward_cut.vi:Double.NaN;
	    final CalculateKendall bt_coupling_sell=this.trades_book_sell_dir[i].krank_coupling;

	   this.book_trades_Dcoupled[i]=new CalculateKendall.DCalculateKendall(
	       currenttime,
	       0,
	       bt_coupling_buy.v_cons/bnbuy+bt_coupling_sell.v_cons/bnsell,
	       bt_coupling_buy.v_discons/bnbuy+bt_coupling_sell.v_discons/bnsell,
	       bt_coupling_buy.v_zero/bnbuy+bt_coupling_sell.v_zero/bnsell,
	       (bt_coupling_buy.v_cons-bt_coupling_buy.v_discons)/bnbuy+
	       (bt_coupling_sell.v_cons-bt_coupling_sell.v_discons)/bnsell);

	   final int ptr2=findPtrToV(currenttime,2*v_cut,trades.getSize(),trades.getBuffer());
	   this.trades_trades_Dcoupled[i]=CalculateKendall.calculateTradesTradesCouplingKendall(
	       currenttime,
	       this.trades.getBuffer(),
	       ptr2,
	       ptr,
	       this.trades.getSize());


	    this.book_side_buy[i]=CalculateKendall.calculateBookKendallRankOneSide(
		currenttime,
		1+ptr_book_buy[i],
		this.buy);
	    this.book_side_sell[i]=CalculateKendall.calculateBookKendallRankOneSide(
		currenttime,
		1+ptr_book_sell[i],
		this.sell);

	    final  CalculateKendall [] rr3=CalculateKendall.calculateTradesKendallRankTP3OrigDiff(
		currenttime,
		this.trades,
		ptr,
		this.trades.getSize());
	    this.trades0_dir[i]=rr3[0];
	    this.trades3_dir[i]=rr3[3];
	    this.trades3_ddir[i]=rr3[4];
	    this.ntrades0_dir[i]=rr3[5];
	    this.ntrades2_dir[i]=rr3[6];
	    this.ntrades3_dir[i]=rr3[7];
	    this.trades_dbs[i]=rr3[8];

	    this.trades_p[i]=CalculateKendall.calculateTradesKendallRankTP2OrigPriceDiff(
		currenttime,
		this.trades,
		ptr,
		this.trades.getSize(),
		pcurrent_buy,
		pcurrent_sell);


	    this.trades_D3[i]=CalculateKendall.calculateBookTradesKendallD3(
		currenttime,
		1+ptr_book_buy[i],
		this.buy,
		1+ptr_book_sell[i],
		this.sell,
		this.trades,
		ptr,
		this.tmpordersbuf.getBuffer());

	    this.trades_D2_buy[i]=CalculateKendall.calculateBookTradesKD2Diff(
		currenttime,
		1+ptr_book_buy[i],
		this.buy,
		this.trades,
		ptr);
	    this.trades_D2_sell[i]=CalculateKendall.calculateBookTradesKD2Diff(
		currenttime,
		1+ptr_book_sell[i],
		this.sell,
		this.trades,
		ptr);

	    final CalculateKendall kbuy=this.trades_D2_buy[i],ksell=this.trades_D2_sell[i];
	    final double nvbuy=kbuy.v_cons+kbuy.v_discons+kbuy.v_zero,
		nvsell=ksell.v_cons+ksell.v_discons+ksell.v_zero,
		vcons=kbuy.v_cons/nvbuy+ksell.v_discons/nvsell,
		vdiscons=kbuy.v_discons/nvbuy+ksell.v_cons/nvsell,
		vzero=kbuy.v_zero/nvbuy+ksell.v_zero/nvsell;

	    this.trades_D2[i]=new CalculateKendall.DCalculateKendall(
		currenttime,
		0,
		vcons,
		vdiscons,
		vzero,
		vcons-vdiscons);


	    this.trades_Dcoupled_buy[i]=CalculateKendall.calculateBookTradesCoupledDiff(
		currenttime,
		1+ptr_book_buy[i],
		this.buy,
		this.trades,
		ptr);

	    this.trades_Dcoupled_sell[i]=CalculateKendall.calculateBookTradesCoupledDiff(
		currenttime,
		1+ptr_book_sell[i],
		this.sell,
		this.trades,
		ptr);

	    this.trades_D2sum[i]=CalculateKendall.calculateBookTradesKendallD2Sum(
		currenttime,
		1+ptr_book_buy[i],
		this.buy,
		1+ptr_book_sell[i],
		this.sell,
		this.trades,
		ptr,
		this.tmpordersbuf.getBuffer());

	    // we have no +1 in kranks, so volume would differ
	    if(false && this.tdiff[i].vi!=this.trades_krank[i].vi)
	    {
		throw new RuntimeException(
		    "Volume mismatch:"+
		    " tdiff["+i+"]="+this.tdiff[i].vi+
		    " trades_krank["+i+"].vi="+this.trades_krank[i].vi+
		    " N_TRADES="+this.trades.getSize()+
		    " currenttime="+currenttime);
	    }


	    this.trades_krank_forward[i]=CalculateKendall.calculateTradesKendallEndPriceDiff(
		currenttime,
		this.trades.getBuffer(),
		ptr,
		trades.getSize(),
		pcurrent_buy,
		pcurrent_sell);
	    this.trades_krank_VS[i]=CalculateKendall.calculateTradesKendallEndPriceDiffVScaled(
		currenttime,
		this.trades.getBuffer(),
		ptr,
		trades.getSize(),
		pcurrent_buy,
		pcurrent_sell,
		vol_lev0);

	    this.book_krank_XY[i]=CalculateKendall.calculateBookKendallRankXY(
		currenttime,
		1+ptr_book_buy[i],
		this.buy,
		1+ptr_book_sell[i],
		this.sell);
	    this.book_krank_cumm[i]=CalculateKendall.calculateBookKendallCumm(
		currenttime,
		1+ptr_book_buy[i],
		this.buy,
		1+ptr_book_sell[i],
		this.sell,
		this.tmpordersbuf.getBuffer());


	    final CalculateKendall [] rr2=CalculateKendall.calculateBookPTradesKendall(
		currenttime,
		1+ptr_book_buy[i],
		this.buy,
		1+ptr_book_sell[i],
		this.sell,
		this.trades,
		ptr);
	    this.trades_book_pkrank_buy[i]=rr2[0];
	    this.trades_book_pkrank_sell[i]=rr2[1];

	    this.book_buy[i]=CalculateLiquidityDeficit7.calculateBookVolumeAndTauSplitted(
		currenttime,
		this.buy,
		this.trades,
		true,
		this.tdiff[i].t_range_cut,
		Double.POSITIVE_INFINITY,
		buy_pricelevels[i]);
	    if(this.book_buy[i].getV()!=vol_book_buy[i]
	       // || (this.book_buy[i].sv_in+this.book_buy[i].sv_out)!=tau_book_unnormalized_buy[i]
	       )
	    {
		throw new RuntimeException(
		    "Constrains mismatch:"+
		    " book_buy["+i+"].getV()="+this.book_buy[i].getV()+
		    " vol_book_buy["+i+"]="+vol_book_buy[i]+
		    " (book_buy["+i+"].sv_in+book_buy["+i+"].sv_out)="+
		    (this.book_buy[i].sv_in+this.book_buy[i].sv_out)+
		    " tau_book_unnormalized_buy["+i+"]="+tau_book_unnormalized_buy[i]+
		    " N_BUY="+this.buy.getSize()+
		    " N_SELL="+this.sell.getSize()+
		    " currenttime="+currenttime);
	    }
	    this.book_sell[i]=CalculateLiquidityDeficit7.calculateBookVolumeAndTauSplitted(
		currenttime,
		this.sell,
		this.trades,
		false,
		this.tdiff[i].t_range_cut,
		Double.NEGATIVE_INFINITY,
		sell_pricelevels[i]);
	    if(this.book_sell[i].getV()!=vol_book_sell[i]
	       // || (this.book_sell[i].sv_in+this.book_sell[i].sv_out)!=tau_book_unnormalized_sell[i]
	       )
	    {
		throw new RuntimeException(
		    "Constrains mismatch:"+
		    " book_sell["+i+"].getV()="+this.book_sell[i].getV()+
		    " vol_book_sell["+i+"]="+vol_book_sell[i]+
		    " (book_sell["+i+"].sv_in+book_sell["+i+"].sv_out)="+
		    (this.book_sell[i].sv_in+this.book_sell[i].sv_out)+
		    " tau_book_unnormalized_sell["+i+"]="+tau_book_unnormalized_sell[i]+
		    " N_BUY="+this.buy.getSize()+
		    " N_SELL="+this.sell.getSize()+
		    " currenttime="+currenttime);
	    }
	}
    }


    static class PTRToRange {
	final long t_range_cut;
	final int ptr_t_range_cut;
	final boolean flag_lower;
	PTRToRange(final int ptr_t_range_cut,
		   final long t_range_cut,
		   final boolean flag_lower)
	{
	    this.ptr_t_range_cut=ptr_t_range_cut;
	    this.t_range_cut=t_range_cut;
	    this.flag_lower=flag_lower;
	}
    }


    static PTRToRange findPtrTrades(final long currenttime,
				    final Distribution<Trade> trades,
				    final double p_buy,
				    final double p_sell)
    {
	final int tsize=trades.getSize();
	final Trade [] tb=trades.getBuffer();
	long vi=0,prevt=Long.MAX_VALUE;
	int ptr_t_range_cut=-1;
	final double pe_up=p_sell+OrdersFlow.EPS,pe_down=p_buy-OrdersFlow.EPS;
	boolean flag_lower=true;
	for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;

	    if(t.origtime<=0)
	    {
		continue;
	    }
	    if(t.price<pe_down)
	    {
		flag_lower=false;
		ptr_t_range_cut=i;
		break;
	    }
	    if(t.price>pe_up)
	    {
		flag_lower=true;
		ptr_t_range_cut=i;
		break;
	    }
	    vi+=t.shares;
	}
	return new PTRToRange(ptr_t_range_cut,
			      ptr_t_range_cut<0?-1:tb[ptr_t_range_cut].time,
			      flag_lower);
    }



    static int findPtrToV(final long currenttime,
			  final long v_cut,
			  final int tsize,
			  final Trade [] tb)
    {
	long vi=0,prevt=Long.MAX_VALUE;
	int ptr_t_range_cut=-1;
	for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;

	    if(t.origtime<=0)
	    {
		continue;
	    }
	    vi+=t.shares;
	    if(vi>v_cut)
	    {
		ptr_t_range_cut=i;
		break;
	    }
	}
	return ptr_t_range_cut;
    }


    static int findPtrToP(final long currenttime,
			  final Distribution<Trade> trades,
			  final double p_min,
			  final double p_max)
    {
	final int tsize=trades.getSize();
	final Trade [] tb=trades.getBuffer();
	long prevt=Long.MAX_VALUE;
	int ptr_t_range_cut=-1;
	for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;

	    if(t.origtime<=0)
	    {
		continue;
	    }
	    if(!(t.price>=p_min) || !(t.price<=p_max))
	    {
		ptr_t_range_cut=i;
		break;
	    }
	}
	return ptr_t_range_cut;
    }



    static class PtrToMed {
	final long vi,v_cut,v_median,currenttime,t_cut,t_median;
	final int ptr_cut,ptr_median;
	PtrToMed(final long vi,
		 final long v_cut,
		 final long v_median,
		 final long t_cut,
		 final long t_median,
		 final long currenttime,
		 final int ptr_cut,
		 final int ptr_median)
	{
	    this.vi=vi;
	    this.v_cut=v_cut;
	    this.v_median=v_median;
	    this.t_cut=t_cut;
	    this.t_median=t_median;
	    this.currenttime=currenttime;
	    this.ptr_cut=ptr_cut;
	    this.ptr_median=ptr_median;
	}
    }
    static PtrToMed findPtrToVAndMedian(final long currenttime,
					final long v_cut,
					final Distribution<Trade> trades)
    {
	final int tsize=trades.getSize();
	final Trade [] tb=trades.getBuffer();
	long vi=0,prevt=Long.MAX_VALUE;
	int ptr_t_range_cut=-1;
	for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;

	    if(t.origtime<=0)
	    {
		continue;
	    }
	    vi+=t.shares;
	    if(vi>v_cut)
	    {
		ptr_t_range_cut=i;
		break;
	    }
	}
	final long v_med_cut=vi/2;
	final int pend=ptr_t_range_cut<0?tsize:ptr_t_range_cut;
	long v=0;
	int ptr_t_median=-1;
	for(int i=tsize;i-->pend;)
	{
	    final Trade t=tb[i];
	    if(t.origtime<=0)
	    {
		continue;
	    }
	    v+=t.shares;
	    if(v>=v_med_cut)
	    {
		ptr_t_median=i;
		break;
	    }
	}
	return new PtrToMed(vi,
			    v_cut,
			    v,
			    ptr_t_range_cut>=0?tb[ptr_t_range_cut].time:-1,
			    ptr_t_median>=0?tb[ptr_t_median].time:-1,
			    currenttime,
			    ptr_t_range_cut,
			    ptr_t_median);
    }




    static int findPtrToT(final long currenttime,
			  final long t_cut,
			  final int tsize,
			  final Distribution<Trade> trades)
    {
	final Trade [] tb=trades.getBuffer();
	long vi=0,prevt=Long.MAX_VALUE;
	int ptr_t_range_cut=-1;
	for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;

	    if(t.origtime<=0)
	    {
		continue;
	    }
	    vi+=t.shares;
	    if(prevt<t_cut)
	    {
		ptr_t_range_cut=i;
		break;
	    }
	}
	return ptr_t_range_cut;
    }


    static int findPtrToT_Inside(final long currenttime,
				 final long t_cut,
				 final int tsize,
				 final BookItem [] tb,
				 final long [] out_vol)
    {
	long prevt=Long.MAX_VALUE,vol=0,tsum=0,idsum=0;
	int ptr_t_range_cut=-1;
	for(int i=tsize;i-->0;)
	{
	    final BookItem t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;
	    if(prevt<t_cut)
	    {
		break;
	    }
	    ptr_t_range_cut=i;
	    vol+=t.shares;
	    tsum+=prevt;
	    idsum+=t.orderreferencenumber;
	}
	if(out_vol!=null)
	{
	    out_vol[0]=vol;
	    out_vol[1]=tsum;
	    out_vol[2]=idsum;
	}
	return ptr_t_range_cut;
    }


    static int findPtrToV_Inside(final long currenttime,
				 final long v_cut,
				 final int tsize,
				 final Trade [] tb)
    {
	long vi=0,prevt=Long.MAX_VALUE;
	int ptr_t_range_cut=-1;
	for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;

	    vi+=t.shares;
	    if(vi>v_cut)
	    {
		break;
	    }
	    ptr_t_range_cut=i;
	}
	return ptr_t_range_cut;
    }


   static int findPtrToVolatility(final long currenttime,
				  final int n_min,
				  final double volatility,
				  final int tsize,
				  final Trade [] tb)
    {
	final double volatility2=volatility*volatility;
	final int is=tsize-n_min;
	int ptr_t_range_cut=-1;
	long prevt=Long.MAX_VALUE;
	double sv=0,sv2=0,v=0;
	for(int i=tsize;i-->0;)
	{
	    final Trade t=tb[i];
	    if(t.time>prevt)
	    {
		throw new RuntimeException("The data is not sorted:"+
					   prevt+" and "+t.time+
					   " currenttime="+currenttime);
	    }
	    prevt=t.time;
	    ptr_t_range_cut=i;

	    final double s=t.shares;
	    v+=s;
	    sv+=t.price*s;
	    sv2+=t.price*t.price*s;
	    if(i>=is)
	    {
		continue;
	    }
	    final double pa=sv/v,vol2=sv2/v-pa*pa;
	    if(!(vol2<volatility2))
	    {
		break;
	    }
	}
	return ptr_t_range_cut;
    }

}
