package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Liquidity deficit v9.
 *  @version $Id: CalculateLiquidityDeficit9T.java,v 1.31 2009/12/16 13:37:08 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateLiquidityDeficit9T {

    private final Distribution<Trade> trades,tmptradesbuf;

    CalculateLiquidityDeficit9T(final Distribution<Trade> trades,
				final Distribution<Trade> tmptradesbuf)
    {
	this.trades=trades;
	this.tmptradesbuf=tmptradesbuf;
    }



    CalculateLiquidityDeficit7.TradesVolumeAndTaus2 [] tdiff;
    CalculateLiquidityDeficit7.TradesVolumeAndTausMedian [] tmed;
    CalculateKendall [] trades_krank,trades_krank_buy,trades_krank_sell,trades_krank_bs,trades_krank_tau,trades_krank_cd,trades_krank_cd_delayed,trades_krank_XY,trades_krank_self,trades_krank_o_doublecouple,trades_krank_dinterval,trades_krank_V;

    CalculateKendall.KendallMultipleStats [] ms_trades_krank,ms_trades_krank_tau,trades_krank_o,trades_krank_o_delayed,trades_krank_o_delayed_halfT;

    CalculateKendall.KendallMStats [] trades_p;

    CalculateKendall.MSAverageKendall [] trades_krank_forward;
    CalculateKendall.DAvergeCalculateKendall [] trades_krank_VS;
    CalculateKendall.DDiffCalculateKendall [] trades_krank_dd;
    CalculateKendall.DPTCalculateKendall [] trades_pt;

    void clear()
    {
	this.tdiff=null;
	this.tmed=null;
	this.trades_krank=this.trades_krank_buy=this.trades_krank_sell=this.trades_krank_bs=this.trades_krank_tau=this.trades_krank_cd=this.trades_krank_cd_delayed=this.trades_krank_XY=this.trades_krank_self=this.trades_krank_o_doublecouple=null;
	this.ms_trades_krank=this.ms_trades_krank_tau=this.trades_krank_o=this.trades_krank_o_delayed=this.trades_krank_o_delayed_halfT=null;
	this.trades_krank_dinterval=null;

	this.trades_p=null;
	this.trades_krank_V=null;
	this.trades_krank_forward=null;
	this.trades_krank_VS=null;
	this.trades_krank_dd=null;
	this.trades_pt=null;
    }


    void calculateLiquidityDeficit(final long [] delays,
				   final long currenttime,
				   final long trades_cleared_before,
				   final double pcurrent_buy,
				   final double pcurrent_sell,
				   final long vol_lev0)
    {
	final int nt=delays.length;
	final boolean PRINT_DATA_DEBUG=false;

	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+
			       "============Liquidity Deficit t=\t"+currenttime+
			       "\tNTRADES=\t"+this.trades.getSize()+
			       "\tnt\t"+nt);
	}


	this.tdiff=new CalculateLiquidityDeficit7.TradesVolumeAndTaus2 [nt];
	this.tmed=new CalculateLiquidityDeficit7.TradesVolumeAndTausMedian [nt];
	this.trades_krank=new CalculateKendall [nt];
	this.trades_krank_buy=new CalculateKendall [nt];
	this.trades_krank_sell=new CalculateKendall [nt];
	this.trades_krank_bs=new CalculateKendall [nt];
	this.trades_krank_tau=new CalculateKendall [nt];
	this.trades_krank_cd=new CalculateKendall [nt];
	this.trades_krank_cd_delayed=new CalculateKendall [nt];
	this.trades_krank_XY=new CalculateKendall [nt];
	this.trades_krank_self=new CalculateKendall [nt];
	this.trades_krank_o=new CalculateKendall.KendallMultipleStats [nt];
	this.trades_krank_o_delayed=new CalculateKendall.KendallMultipleStats [nt];
	this.trades_krank_o_delayed_halfT=new CalculateKendall.KendallMultipleStats [nt];
	this.trades_krank_dinterval=new CalculateKendall [nt];
	this.ms_trades_krank_tau=new CalculateKendall.KendallMultipleStats [nt];
	this.trades_krank_o_doublecouple=new CalculateKendall [nt];
	this.trades_krank_V=new CalculateKendall [nt];
	this.trades_krank_forward=new CalculateKendall.MSAverageKendall [nt];
	this.trades_krank_VS=new CalculateKendall.DAvergeCalculateKendall [nt];
	this.trades_krank_dd=new CalculateKendall.DDiffCalculateKendall [nt];
	this.trades_pt=new CalculateKendall.DPTCalculateKendall [nt];

	this.trades_p=new CalculateKendall.KendallMStats [nt];

	final int [] ptr_to_cut=new int [nt];
	for(int i=0;i<nt;i++)
	{
	    final int ptr=CalculateLiquidityDeficit9.findPtrToT(currenttime,currenttime-delays[i],trades.getSize(),trades);
	    ptr_to_cut[i]=ptr;
	    this.tdiff[i]=CalculateLiquidityDeficit7.calculateTradesVolumeAndTaus2(
		currenttime,
		this.trades,
		true, // unused
		ptr);
	    this.tmed[i]=CalculateLiquidityDeficit7.calculateTradesVolumeAndTausMedian(
		currenttime,
		this.trades,
		this.tmptradesbuf.getBuffer(),
		ptr);
	    if(this.tdiff[i].vi!=this.tmed[i].vi)
	    {
		throw new RuntimeException(
		    "Volume mismatch:"+
		    " tdiff["+i+"]="+this.tdiff[i].vi+
		    " tmed["+i+"].vi="+this.tmed[i].vi+
		    " N_TRADES="+this.trades.getSize()+
		    " currenttime="+currenttime);
	    }
	    final CalculateKendall [] r=
		CalculateKendall.calculateTradesKendallRankTOrigDiff(
		    currenttime,
		    this.trades,
		    ptr,
		    this.trades.getSize());
	    this.trades_krank[i]=r[0];
	    this.trades_krank_buy[i]=r[1];
	    this.trades_krank_sell[i]=r[2];
	    this.trades_krank_bs[i]=r[3];
	    this.trades_krank_self[i]=r[4];
	    this.trades_krank_XY[i]=r[5];
	    this.trades_krank_tau[i]=r[6];
	    this.trades_krank_cd[i]=r[7];

	    this.trades_p[i]=CalculateKendall.calculateTradesKendallRankTP2OrigPriceDiff(
		currenttime,
		this.trades,
		ptr,
		this.trades.getSize(),
		pcurrent_buy,
		pcurrent_sell);

	    if(false && this.tdiff[i].vi!=this.trades_krank[i].vi)
	    {
		throw new RuntimeException(
		    "Volume mismatch:"+
		    " tdiff["+i+"]="+this.tdiff[i].vi+
		    " trades_krank["+i+"].vi="+this.trades_krank[i].vi+
		    " N_TRADES="+this.trades.getSize()+
		    " currenttime="+currenttime);
	    }

	    final int ptr_delayed=CalculateLiquidityDeficit9.findPtrToT(currenttime,currenttime-2*delays[i],trades.getSize(),trades);
	    final CalculateKendall [] rd=
		CalculateKendall.calculateTradesKendallRankTOrigDiff(
		    currenttime,
		    this.trades,
		    ptr_delayed,
		    ptr);
	    this.trades_krank_cd_delayed[i]=rd[7];

	    this.trades_krank_o[i]=CalculateKendall.calculateTradesKendallRankTP2OrigDiff(
		currenttime,
		this.trades,
		ptr,
		this.trades.getSize());
	    this.trades_krank_o_delayed[i]=CalculateKendall.calculateTradesKendallRankTP2OrigDiff(
		currenttime,
		this.trades,
		ptr_delayed,
		ptr);
	    final CalculateKendall kc=this.trades_krank_o[i].krank2,kd=this.trades_krank_o_delayed[i].krank2;
	    this.trades_krank_o_doublecouple[i]=
		new CalculateKendall(kc.currenttime,
				     kc.vi+kd.vi,
				     kc.v_cons+kd.v_cons,
				     kc.v_discons+kd.v_discons,
				     kc.v_zero+kd.v_zero+kd.vi*kc.vi);

	    final int ptr_ht=CalculateLiquidityDeficit9.findPtrToT(currenttime,currenttime-delays[i]/2,trades.getSize(),trades);
	    final int ptr_delayed_ht=CalculateLiquidityDeficit9.findPtrToT(currenttime,currenttime-(delays[i]+delays[i]/2),trades.getSize(),trades);
	    this.trades_krank_o_delayed_halfT[i]=CalculateKendall.calculateTradesKendallRankTP2OrigDiff(
		currenttime,
		this.trades,
		ptr_delayed_ht,
		ptr_ht);

	    this.ms_trades_krank_tau[i]=CalculateKendall.calculateTradesKendallRankMultipleTP3OrigTau(
		currenttime,
		this.trades,
		delays[i]);


	    final int ptr_div2=CalculateLiquidityDeficit9.findPtrToT(
		currenttime,currenttime-delays[i]/4,trades.getSize(),trades);
	    final int ptr_divu=CalculateLiquidityDeficit9.findPtrToT(
		currenttime,currenttime-3*(delays[i]/4),ptr_div2,trades);
	    final CalculateKendall ks=CalculateKendall.calculateTradesKendallRankTP2OrigDiff(
		currenttime,
		this.trades,
		ptr_divu,
		ptr_div2).krank2;
	    this.trades_krank_dinterval[i]=new CalculateKendall(ks.currenttime,
								2*ks.vi,
								2*ks.v_cons,
								2*ks.v_discons,
								2*ks.v_zero+ks.vi*ks.vi);


	    this.trades_krank_V[i]=CalculateKendall.calculateBookKendallRankV(
		currenttime,
		this.trades,
		ptr,
		trades.getSize());
	    this.trades_krank_forward[i]=CalculateKendall.calculateTradesKendallEndPriceDiff(
		currenttime,
		this.trades.getBuffer(),
		ptr,
		trades.getSize(),
		pcurrent_buy,
		pcurrent_sell);
	    this.trades_krank_VS[i]=CalculateKendall.calculateTradesKendallEndPriceDiffVScaled(
		currenttime,
		this.trades.getBuffer(),
		ptr,
		trades.getSize(),
		pcurrent_buy,
		pcurrent_sell,
		vol_lev0);
	    this.trades_krank_dd[i]=CalculateKendall.calculateTradesKendallDDiffVScaled(
		currenttime,
		this.trades.getBuffer(),
		ptr,
		trades.getSize(),
		pcurrent_buy,
		pcurrent_sell);
	    this.trades_pt[i]=CalculateKendall.calculateTradesKendallPT(
		currenttime,
		this.trades.getBuffer(),
		ptr,
		trades.getSize());
	}
	this.ms_trades_krank=CalculateKendall.calculateTradesKendallRankMultipleTP3OrigDiff(
	    currenttime,
	    this.trades,
	    ptr_to_cut,
	    true);
    }

}
