package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Modelling of order cancel/add events.
 *  An ideas behind looking at cancel orders is:
 *  1. cancelled order is some kind of "hidden wishes".
 *  2. order cancel rate is expected to be high in a moment of price search.
 *  @version $Id: CalculatePseudotradesFlow.java,v 1.5 2009/10/11 09:21:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculatePseudotradesFlow {

    private final MatchOrdersNoShift matcher=new MatchOrdersNoShift();
    private final OrdersFlow
	porder_obuy=new OrdersFlow(true),
	porder_osell=new OrdersFlow(false),
	obuy=new OrdersFlow(true),
	osell=new OrdersFlow(false),
	orders_buy=new OrdersFlow(true),
	orders_sell=new OrdersFlow(false);

    private final Distribution<BookItem> buy,sell;
    private final Distribution<?extends BookItem> porders;

    CalculatePseudotradesFlow(final Distribution<BookItem> buy,
			      final Distribution<BookItem> sell,
			      final Distribution<?extends BookItem> porders)
    {
	this.buy=buy;
	this.sell=sell;
	this.porders=porders;
    }


    MatchOrdersNoShift.MatchResults []
	match_porders_only=null,
	match_porders_and_book=null;
    double dspread=Double.NaN;

    void clear()
    {
	this.match_porders_only=null;
	this.match_porders_and_book=null;
	this.dspread=Double.NaN;

	// clear internal buffers
	this.porder_obuy.clear();
	this.porder_osell.clear();
	this.obuy.clear();
	this.osell.clear();
	this.orders_buy.clear();
	this.orders_sell.clear();
    }


    void calculatePseudoFlow(final long [] delays,
			     final long currenttime,
			     final double pcurrent_buy,
			     final double pcurrent_sell,
			     final long trades_cleared_before)
    {

	final int tsize=this.porders.getSize();
	final BookItem [] tb=this.porders.getBuffer();
	this.dspread=pcurrent_sell-pcurrent_buy;


	final boolean PRINT_DATA_DEBUG=false;//true;
	//currenttime==35308657000000L;
	//currenttime==32894696000000L;
	//currenttime==34580422000000L;
	//currenttime>=9*3600*TraderInterface.TIME_SCALE && tsize>0;


	if(PRINT_DATA_DEBUG)
	{
	    System.err.println("=="+getClass().getName()+"============Vritual Shifts t=\t"+currenttime+
			       "\tPSEUDOTRADES\t"+tsize+
			       "\tn_buy\t"+this.buy.getSize()+
			       "\tn_sell\t"+this.sell.getSize()+
			       "\tpcurrent_buy\t"+pcurrent_buy+
			       "\tpcurrent_sell\t"+pcurrent_sell);


	    System.err.println("num\t"+
			       "time\t"+
			       "shares\t"+
			       "buy_sell_indicator\t"+
			       "attribution\t"+
			       "price\t");
	}

	// init arrays
	final int nt=delays.length;
	this.match_porders_only=new MatchOrdersNoShift.MatchResults [nt];
	this.match_porders_and_book=new MatchOrdersNoShift.MatchResults [nt];

	final int [] tau_interval_range_ptr=new int [nt];
	for(int i=0;i<nt;i++)
	{
	    tau_interval_range_ptr[i]=-1;
	}

	//  buy & sell orders from the book
	final long vordersbuy,vorderssell;
	if(true)
	{
	    final BookItem [] buf_buy=this.buy.getBuffer(),buf_sell=this.sell.getBuffer();
	    for(int i=0,n=this.buy.getSize();i<n;i++)
	    {
		final BookItem t=buf_buy[i];
		this.orders_buy.addOrderCheckPrev(t.price,t.shares);
	    }
	    vordersbuy=this.orders_buy.getTotalVolume();
	    for(int i=0,n=this.sell.getSize();i<n;i++)
	    {
		final BookItem t=buf_sell[i];
		this.orders_sell.addOrderCheckPrev(t.price,t.shares);
	    }
	    vorderssell=this.orders_sell.getTotalVolume();
	}

	if(true)
	{ // calculate volume
	    long tbefore=currenttime-delays[0];
	    long prevt=Long.MAX_VALUE;
	    gloop:for(int i=tsize,cur_pos=0;i-- >0;)
	    {
		final BookItem t=tb[i];
		if(t.time>prevt)
		{
		    throw new RuntimeException("The data is not sorted:"+
					       prevt+" and "+t.time+
					       " currenttime="+currenttime);
		}
		prevt=t.time;
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.shares+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.attribution+"\t"+
				       t.price+"\t");
		}
		for(;!(t.time>=tbefore);)
		{
		    processMatches(currenttime,
				   cur_pos,
				   pcurrent_buy,
				   pcurrent_sell,
				   PRINT_DATA_DEBUG);
		    if(++cur_pos>=nt)
		    {
			break gloop;
		    }
		    tbefore=currenttime-delays[cur_pos];
		}
		if(/* tbefore>0L && */tbefore<trades_cleared_before)
		{
		    throw new RuntimeException("Something is terrible wrong with old data removal"+
					       " currenttime="+currenttime);
		}

		if(t.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    this.porder_obuy.addOrderCheckPrev(t.price,t.shares);
		}
		else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    this.porder_osell.addOrderCheckPrev(t.price,t.shares);
		}
		tau_interval_range_ptr[cur_pos]=i;
	    }
	} // END calculate volume


	// set to previuos, if current not available
	for(int i=1;i<nt;i++)
	{
	    if(tau_interval_range_ptr[i]<0)
	    {
		final int i1=i-1;
		this.match_porders_only[i]=this.match_porders_only[i1];
		this.match_porders_and_book[i]=this.match_porders_and_book[i1];
	    }
	}


	for(int i=0;i<nt;i++)
	{
	    if((vordersbuy+this.match_porders_only[i].v_buy)!=this.match_porders_and_book[i].v_buy||
	       (vorderssell+this.match_porders_only[i].v_sell)!=this.match_porders_and_book[i].v_sell)
	    {
		throw new RuntimeException(
		    "Volume mismatch:"+
		    " v_buy="+(vordersbuy+this.match_porders_only[i].v_buy)+
		    " v_buy_porders_and_book="+this.match_porders_and_book[i].v_buy+
		    " v_sell="+(vorderssell+this.match_porders_only[i].v_sell)+
		    " v_sell_porders_and_book="+this.match_porders_and_book[i].v_sell);
	    }
	}

    }


    private void processMatches(final long currenttime,
				final int cur_pos,
				final double pcurrent_buy,
				final double pcurrent_sell,
				final boolean PRINT_DATA_DEBUG)
    {
	this.obuy.clear();
	this.obuy.addOrdersFrom(this.porder_obuy);
	this.obuy.mergeIdenticalPrices();
	this.osell.clear();
	this.osell.addOrdersFrom(this.porder_osell);
	this.osell.mergeIdenticalPrices();
	this.match_porders_only[cur_pos]=this.matcher.match(this.obuy,this.osell);

	this.obuy.addOrdersFrom(this.orders_buy);
	this.obuy.mergeIdenticalPrices();
	this.osell.addOrdersFrom(this.orders_sell);
	this.osell.mergeIdenticalPrices();
	this.match_porders_and_book[cur_pos]=this.matcher.match(this.obuy,this.osell);
    }
}
