package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.*;

/** A helper class to calculate Skewness and ProbabilityCorrelation in a given |psi> state.
 * The skewness is calculated out of for  {@literal <psi|p^sI|psi>} s=0,1,2,3 moments.
 * The probability correlation is calculated in the d=2 basis
 * psi and Dpsi, for the three 2x2 matrices
 * {@literal <psi|g|psi>} {@literal <psi|g|Dpsi>}
 * {@literal <Dpsi|g|psi>} {@literal <Dpsi|g|Dpsi>}
 * where g={1,I,(p-Pa)I}, and Pa={@literal <psi|pI|psi>/<psi|I|psi>}.
 * @version $Id: CalculateSkewnessAndProbabilityCorrelationInPsiState.java,v 1.9 2022/06/17 20:37:50 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class CalculateSkewnessAndProbabilityCorrelationInPsiState {
    public final Skewness sk;
    public final ProbabilityCorrelation pc;
    public final double p_m,var_ppi;
    public CalculateSkewnessAndProbabilityCorrelationInPsiState(
	final double [] psi,final double [] _Dpsi,
	final double [] QQt,final double [] QQi,final double [] QQpi,final double [] QQp2i,final double [] QQp3i,
	final double [] QQpm,final double [] QQm)
    {
	final int n=psi.length;
	this.p_m=TMatr.getAMatrA(psi,QQpm,n)/TMatr.getAMatrA(psi,QQm,n);
	final double i=TMatr.getAMatrA(psi,QQi,n),pi=TMatr.getAMatrA(psi,QQpi,n);
	// skewness
	this.sk=Skewness.getSkewnessFromMonomialMoments(i,pi,TMatr.getAMatrA(psi,QQp2i,n),TMatr.getAMatrA(psi,QQp3i,n));

	// probability correlation
	if(_Dpsi.length>psi.length) throw new IllegalArgumentException();
	final double [] QQppi_M=TMatr.sumAcB(QQpi,-(pi/i),QQi),Dpsi=java.util.Arrays.copyOf(_Dpsi,n);
	// for numerical stability only remove the projection
	TMatr.sumP1(Dpsi,psi,-TMatr.getAMatrB(psi,Dpsi,QQt,n));
	this.var_ppi=2*TMatr.getAMatrB(psi,Dpsi,QQppi_M,n);
	this.pc=new ProbabilityCorrelation(
	    new double []{
		TMatr.getAMatrB(psi,psi,QQt,n),
		TMatr.getAMatrB(psi,Dpsi,QQt,n), // =0
		TMatr.getAMatrB(Dpsi,psi,QQt,n), // =0
		TMatr.getAMatrB(Dpsi,Dpsi,QQt,n)},
	    new double []{
		TMatr.getAMatrB(psi,psi,QQppi_M,n), // =0
		TMatr.getAMatrB(psi,Dpsi,QQppi_M,n),
		TMatr.getAMatrB(Dpsi,psi,QQppi_M,n),
		TMatr.getAMatrB(Dpsi,Dpsi,QQppi_M,n)},
	    new double []{
		TMatr.getAMatrB(psi,psi,QQi,n),
		TMatr.getAMatrB(psi,Dpsi,QQi,n),
		TMatr.getAMatrB(Dpsi,psi,QQi,n),
		TMatr.getAMatrB(Dpsi,Dpsi,QQi,n)});
    }
}
