package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;


/** Dynamics modelling with new flow transformation. The Empire Strikes Back.
 *  Similar to CalculateDynamics, but use new approach to flow.
 *  @version $Id: CalculateTRDynamics.java,v 1.43 2010/04/13 13:45:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateTRDynamics {

    private static final double MED_LEVEL=0.5;
    private static final boolean MM_SHIFTS=true;

    private final MatchOrdersNoShift matcher=new MatchOrdersNoShift();

    private final OrdersFlow
	buy_flow_tau=new OrdersFlow(true),
	sell_flow_tau=new OrdersFlow(false),
	buy_flow2_tau=new OrdersFlow(true),
	sell_flow2_tau=new OrdersFlow(false),

	mmbuy_flows1_tau=MM_SHIFTS?new OrdersFlow(true):null,
	mmsell_flows1_tau=MM_SHIFTS?new OrdersFlow(false):null,
	mmbuy_flows2_tau=MM_SHIFTS?new OrdersFlow(true):null,
	mmsell_flows2_tau=MM_SHIFTS?new OrdersFlow(false):null;


    private final Distribution<BookItem> buy,sell;
    private final Distribution<Trade> trades,tmptradesbuf;

    CalculateTRDynamics(final Distribution<BookItem> buy,
			final Distribution<BookItem> sell,
			final Distribution<Trade> trades,
			final Distribution<BookItem> tmpordersbuf,
			final Distribution<Trade> tmptradesbuf)
    {
	this.buy=buy;
	this.sell=sell;
	this.trades=trades;
	this.tmptradesbuf=tmptradesbuf;
    }


    long []
	v_trades_tau=null,
	v_hidden_trades_tau=null,
	v_effD1_matched_tau=null,
	v_effD2_matched_tau=null;
    int [] minlevelsD1=null,minlevelsD2=null;
    double []
	price_bookD1_modelled=null,
	price_bookD2_modelled=null,
	price_book_flow_tau=null,
	price_mmshift_tau=null,
	price_mmshift_w_tau=null,
	tm_price_min=null,
	tm_price_max=null,
	tm_price_median=null;



    void clear()
    {
	this.v_trades_tau=
	this.v_hidden_trades_tau=
	this.v_effD1_matched_tau=
	this.v_effD2_matched_tau=null;
	this.price_bookD1_modelled=
	this.price_bookD2_modelled=
	this.price_book_flow_tau=
	this.price_mmshift_tau=
	this.price_mmshift_w_tau=
	this.tm_price_min=
	this.tm_price_max=
	this.tm_price_median=null;
	this.minlevelsD1=
	this.minlevelsD2=null;
    }


    void calculateProjectedFlow(final long [] delays,
				final long currenttime,
				final double pcurrent_buy,
				final double pcurrent_sell,
				final long trades_cleared_before)
    {

	final int nt=delays.length;
	this.v_trades_tau=new long [nt];
	this.v_hidden_trades_tau=new long [nt];
	this.v_effD1_matched_tau=new long [nt];
	this.v_effD2_matched_tau=new long [nt];
	this.price_bookD1_modelled=new double [nt];
	this.price_bookD2_modelled=new double [nt];
	this.price_book_flow_tau=new double [nt];
	this.price_mmshift_tau=MM_SHIFTS?new double [nt]:null;
	this.price_mmshift_w_tau=MM_SHIFTS?new double [nt]:null;
	this.tm_price_min=new double [nt];
	this.tm_price_max=new double [nt];
	this.tm_price_median=new double [nt];
	this.minlevelsD1=new int [nt];
	this.minlevelsD2=new int [nt];

	final int tsize=this.trades.getSize();

	final boolean PRINT_DATA_DEBUG=false;
	//currenttime==34344233000000L;
	//currenttime==32894696000000L;
	//currenttime==34580422000000L;
	//currenttime>=9*3600*TraderInterface.TIME_SCALE && tsize>0;

	final Trade [] tb=this.trades.getBuffer();
	final BookItem [] buf_buy=this.buy.getBuffer(),buf_sell=this.sell.getBuffer();

	// read trades,populate list of orders
	// A LOOP OVER ALL TIME INTERVALS
	for(int cur_pos=0;cur_pos<nt;cur_pos++)
	{
	    final long tafter=currenttime-delays[cur_pos];
	    if(tafter<trades_cleared_before)
	    {
		throw new RuntimeException("Something is terrible wrong with old data removal"+
					   " currenttime="+currenttime);
	    }

	    // clear buffers to calculate trade distribution
	    this.buy_flow_tau.clear();
	    this.sell_flow_tau.clear();

	    // ALL TRADES in given interval
	    // A trade generate two orders in order flow between tafter and now.
	    // First - maket order which is just executed.
	    // Second - limit order, which corresponded to one matching the First execution.
	    long  prevt=Long.MAX_VALUE,vol_exe=0,vh_exe=0,vcross_exe=0;
	    for(int i=tsize;i-->0;)
	    {
		final Trade t=tb[i];
		if(t.time>prevt)
		{
		    throw new RuntimeException("The data is not sorted:"+
					       prevt+" and "+t.time+
					       " currenttime="+currenttime);
		}
		prevt=t.time;
		if(!(t.time>=tafter))
		{
		    break;
		}
		final int s=t.shares;
		this.buy_flow_tau.addOrderCheckPrev(t.price,s);
		this.sell_flow_tau.addOrderCheckPrev(t.price,s);

		final boolean hidden=t.order==null;
		vol_exe+=s;
		if(hidden)
		{
		    vh_exe+=s;
		}

		if(t.buy_sell_indicator!=BookItem.BOOK_BUY && t.buy_sell_indicator!=BookItem.BOOK_SELL)
		{
		    vcross_exe+=s;
		}
	    }
	    this.v_trades_tau[cur_pos]=vol_exe;
	    this.v_hidden_trades_tau[cur_pos]=vh_exe;

	    // now calculate median, low & high
	    final OrdersFlow.DistStat
		dstotal_buy=this.buy_flow_tau.getStat(MED_LEVEL),
		dstotal_sell=this.sell_flow_tau.getStat(MED_LEVEL);
	    this.tm_price_min[cur_pos]=dstotal_buy.min;
	    this.tm_price_max[cur_pos]=dstotal_buy.max;
	    this.tm_price_median[cur_pos]=0.5*(dstotal_buy.median+dstotal_sell.median);
	    if(Double.isNaN(dstotal_buy.min)!=Double.isNaN(dstotal_sell.min) ||
	       (!Double.isNaN(dstotal_buy.min) && dstotal_buy.min!=dstotal_sell.min)||
	       Double.isNaN(dstotal_buy.max)!=Double.isNaN(dstotal_sell.max) ||
	       (!Double.isNaN(dstotal_buy.max) && dstotal_buy.max!=dstotal_sell.max))
	    {
		throw new RuntimeException("min/max mismatch: "+
					   " dstotal_buy.min="+dstotal_buy.min+
					   " dstotal_sell.min="+dstotal_sell.min+
					   " dstotal_buy.max="+dstotal_buy.max+
					   " dstotal_sell.max="+dstotal_sell.max+
					   " delay="+delays[cur_pos]+
					   " currenttime="+currenttime);
	    }

	    // Note that mergeIdenticalPrices() is already called inside getStat() above.
	    // No need to call it here second time here, before doing match().
	    final MatchOrdersNoShift.MatchResults mtrades=this.matcher.match(this.buy_flow_tau,this.sell_flow_tau);
	    if(vol_exe!=mtrades.v_buy ||
	       vol_exe!=mtrades.v_sell ||
	       vol_exe!=dstotal_buy.vtotal||
	       vol_exe!=dstotal_sell.vtotal)
	    {
		throw new RuntimeException("volume mismatch: "+
					   " vol_exe="+vol_exe+
					   " mtrades.v_buy="+mtrades.v_buy+
					   " mtrades.v_sell="+mtrades.v_sell+
					   " dstotal_buy.vtotal="+dstotal_buy.vtotal+
					   " dstotal_sell.vtotal="+dstotal_sell.vtotal+
					   " delay="+delays[cur_pos]+
					   " currenttime="+currenttime);
	    }

	    // make sure the results of match correspond to calculated medians
	    // there may be a degeneracy, so consider the test to be OK
	    // if price matches to any two of the medians.
	    // two medians may be different only because of discrete volume & prices
	    if(Double.isNaN(mtrades.p_match_sell)!=Double.isNaN(dstotal_buy.median) ||
	       (!Double.isNaN(mtrades.p_match_sell) && (
		   mtrades.p_match_sell!=dstotal_buy.median  &&
		   mtrades.p_match_sell!=dstotal_sell.median))||
	       Double.isNaN(mtrades.p_match_buy)!=Double.isNaN(dstotal_sell.median) ||
		(!Double.isNaN(mtrades.p_match_buy) && (
		    mtrades.p_match_buy!=dstotal_buy.median &&
		    mtrades.p_match_buy!=dstotal_sell.median)))
	    {
		throw new RuntimeException("medians mismatch: "+
					   " mtrades.p_match_sell="+mtrades.p_match_sell+
					   " dstotal_buy.median="+dstotal_buy.median+
					   " mtrades.p_match_buy="+mtrades.p_match_buy+
					   " dstotal_sell.median="+dstotal_sell.median+
					   " vol_exe="+vol_exe+
					   " matched="+mtrades.v_executed+
					   " delay="+delays[cur_pos]+
					   " currenttime="+currenttime);
	    }
	    final double pf_eq=0.5*(mtrades.p_buy+mtrades.p_sell);
	    this.price_book_flow_tau[cur_pos]=pf_eq;


	    // Make shifted flow
	    this.buy_flow2_tau.clear();
	    this.sell_flow2_tau.clear();

	    this.buy_flow2_tau.addOrdersFrom(this.buy_flow_tau);
	    //this.buy_flow2_tau.dump("INIT");
	    this.sell_flow2_tau.addOrdersFrom(this.sell_flow_tau);


	    final double shift1=pcurrent_buy-dstotal_buy.max,shift2=pcurrent_sell-dstotal_buy.min;
	    if(MM_SHIFTS)
	    {
		// also try different kind of shifts

		if(!Double.isNaN(shift1))
		{
		    this.mmbuy_flows1_tau.clear();
		    this.mmsell_flows1_tau.clear();
		    this.mmbuy_flows1_tau.addOrdersFrom(this.buy_flow_tau);
		    this.mmsell_flows1_tau.addOrdersFrom(this.sell_flow_tau);
		    convertOrdersFlow(this.mmbuy_flows1_tau,
				      this.mmsell_flows1_tau,
				      shift1);
		}
		if(!Double.isNaN(shift2))
		{
		    this.mmbuy_flows2_tau.clear();
		    this.mmsell_flows2_tau.clear();
		    this.mmbuy_flows2_tau.addOrdersFrom(this.buy_flow_tau);
		    this.mmsell_flows2_tau.addOrdersFrom(this.sell_flow_tau);
		    convertOrdersFlow(this.mmbuy_flows2_tau,
				      this.mmsell_flows2_tau,
				      shift2);
		}
	    }

	    // shift to current
	    //final double tm_shift=0.5*(pcurrent_buy+pcurrent_sell)-pf_eq;

	    // shift to opposite side (from past median to current)
	    // here tm_shift=F2-F1

	    // corespond to model c-F1=F2-c
	    // equal probabily change up/down
	    final double tm_shift=pcurrent_buy+pcurrent_sell-(mtrades.p_buy+mtrades.p_sell);
	    /*
	    // there is a similar alternative
	    // corresponding to condition: c/F1=F2/c
	    // equal probability % change up/down
	    final double cur=0.5*(pcurrent_buy+pcurrent_sell)
	    final double tm_shift=(cur*cur/pf_eq)-pf_eq;
	    */
	    if(!Double.isNaN(tm_shift))
	    {
		// convert flow according to some model.
		convertOrdersFlow(this.buy_flow2_tau,
				  this.sell_flow2_tau,
				  tm_shift);
	    }
	    //this.buy_flow2_tau.dump("after convertOrdersFlow");






	    // Add initial distribution of orders.
	    // Here we add all orders in the book to flow, which we calculated earlier.
	    // The recent orders may get added twice:
	    // first time as flow (possibly converted) and second time as initial state.
	    for(int i=this.buy.getSize();i-- >0;)
	    {
		final BookItem t=buf_buy[i];
		this.buy_flow_tau.addOrderCheckPrev(t.price,t.shares);
		this.buy_flow2_tau.addOrderCheckPrev(t.price,t.shares);
		if(MM_SHIFTS)
		{
		    this.mmbuy_flows1_tau.addOrderCheckPrev(t.price,t.shares);
		    this.mmbuy_flows2_tau.addOrderCheckPrev(t.price,t.shares);
		}
	    }
	    //this.buy_flow2_tau.dump("initial distribution added");
	    for(int i=this.sell.getSize();i-- >0;)
	    {
		final BookItem t=buf_sell[i];
		this.sell_flow_tau.addOrderCheckPrev(t.price,t.shares);
		this.sell_flow2_tau.addOrderCheckPrev(t.price,t.shares);
		if(MM_SHIFTS)
		{
		    this.mmsell_flows1_tau.addOrderCheckPrev(t.price,t.shares);
		    this.mmsell_flows2_tau.addOrderCheckPrev(t.price,t.shares);
		}
	    }


	    this.buy_flow_tau.mergeIdenticalPrices();
	    this.sell_flow_tau.mergeIdenticalPrices();
	    // now we have expected buy & sell orders distribution.
	    // match them and try to extract characteristics.
	    final MatchOrdersNoShift.MatchResults m1=this.matcher.match(this.buy_flow_tau,this.sell_flow_tau);
	    this.price_bookD1_modelled[cur_pos]=0.5*(m1.p_buy+m1.p_sell);
	    this.v_effD1_matched_tau[cur_pos]=m1.v_executed;
	    this.minlevelsD1[cur_pos]=Math.min(m1.nmatched_pricelevels_buy,m1.nmatched_pricelevels_sell);

	    //this.buy_flow2_tau.dump("before merge");
	    final MatchOrdersNoShift.MatchResults m2;
	    if(!Double.isNaN(tm_shift))
	    {
		this.buy_flow2_tau.mergeIdenticalPrices();
		this.sell_flow2_tau.mergeIdenticalPrices();
		// now we have expected buy & sell orders distribution.
		// match them and try to extract characteristics.
		m2=this.matcher.match(this.buy_flow2_tau,this.sell_flow2_tau);
		this.price_bookD2_modelled[cur_pos]=0.5*(m2.p_buy+m2.p_sell);
		this.v_effD2_matched_tau[cur_pos]=m2.v_executed;
		this.minlevelsD2[cur_pos]=Math.min(m2.nmatched_pricelevels_buy,m2.nmatched_pricelevels_sell);

		if(m1.v_buy!=m2.v_buy||m1.v_sell!=m2.v_sell)
		{
		    throw new RuntimeException("Volume Mismatch:"+
					       " m1.v_buy="+m1.v_buy+
					       " m2.v_buy="+m2.v_buy+
					       " m1.v_sell="+m1.v_sell+
					       " m2.v_sell="+m2.v_sell+
					       " delay="+delays[cur_pos]+
					       " currenttime="+currenttime);
		}
	    }
	    else
	    {
		m2=null;
		this.price_bookD2_modelled[cur_pos]=Double.NaN;
	    }


	    if(MM_SHIFTS)
	    {
		final double eqs1;
		if(!Double.isNaN(shift1))
		{
		    this.mmbuy_flows1_tau.mergeIdenticalPrices();
		    this.mmsell_flows1_tau.mergeIdenticalPrices();
		    final MatchOrdersNoShift.MatchResults ms1=this.matcher.match(this.mmbuy_flows1_tau,this.mmsell_flows1_tau);
		    eqs1=0.5*(ms1.p_buy+ms1.p_sell);
		}
		else
		{
		    eqs1=Double.NaN;
		}

		final double eqs2;
		if(!Double.isNaN(shift2))
		{
		    this.mmbuy_flows2_tau.mergeIdenticalPrices();
		    this.mmsell_flows2_tau.mergeIdenticalPrices();
		    final MatchOrdersNoShift.MatchResults ms2=this.matcher.match(this.mmbuy_flows2_tau,this.mmsell_flows2_tau);
		    eqs2=0.5*(ms2.p_buy+ms2.p_sell);
		}
		else
		{
		    eqs2=Double.NaN;
		}

		if(!Double.isNaN(tm_shift))
		{
		    // note weights are permutated
		    final long w1=m2.v_executed-mtrades.v_executed,w2=m1.v_executed-mtrades.v_executed;
		    this.price_mmshift_tau[cur_pos]=(eqs1-shift1-pf_eq)+(eqs2-shift2-pf_eq);
		    this.price_mmshift_w_tau[cur_pos]=((eqs1-shift1-pf_eq)*w1+(eqs2-shift2-pf_eq)*w2)/(w1+w2);
		}
		else
		{
		    this.price_mmshift_tau[cur_pos]=this.price_mmshift_w_tau[cur_pos]=Double.NaN;
		}
	    }

	    // CALCULATIONS END




	    // EXPENSIVE CONSTRAINS CHECK
	    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    {
		// buffers no longer used, we may use them as temporary buffers
		final OrdersFlow tmp_buy=this.buy_flow_tau;
		final OrdersFlow tmp_sell=this.sell_flow_tau;
		tmp_sell.clear();
		tmp_buy.clear();

		final double frac=MED_LEVEL;
		final int TOCHECK=cur_pos;
		final long [] delays1=delays;

		final CalculateBookVolumePriceThreshold
		    cv_buy=new CalculateBookVolumePriceThreshold(true,this.buy),
		    cv_sell=new CalculateBookVolumePriceThreshold(false,this.sell);
		cv_buy.calculateThresholds(new double [] {Double.NEGATIVE_INFINITY},
					   currenttime,
					   frac,
					   pcurrent_buy,
					   pcurrent_sell);
		cv_sell.calculateThresholds(new double [] {Double.POSITIVE_INFINITY},
					    currenttime,
					    frac,
					    pcurrent_buy,
					    pcurrent_sell);

		final CalculateTradeVolume tv=
		    new CalculateTradeVolume(this.trades,this.tmptradesbuf);
		tv.calculateTradeVolume(
		    delays1,
		    currenttime,
		    frac,
		    0.92,
		    0.5,
		    pcurrent_buy,
		    pcurrent_sell,
		    trades_cleared_before);
		if(true)
		{
		    if(vol_exe!=tv.v_tot_alltrades[TOCHECK]||vh_exe!=tv.v_tot_hiddentrades[TOCHECK])
		    {
			throw new RuntimeException("Volume mistmatch:"+
						   " vol_exe="+vol_exe+
						   " tv.v_tot_alltrades["+TOCHECK+"]="+tv.v_tot_alltrades[TOCHECK]+
						   " vh_exe="+vh_exe+
						   " tv.v_tot_hiddentrades["+TOCHECK+"]="+tv.v_tot_hiddentrades[TOCHECK]+
						   " delay="+delays1[TOCHECK]+
						   " currenttime="+currenttime);
		    }
		}
		if(true)
		{
		    if(Double.isNaN(this.tm_price_min[cur_pos])!=Double.isNaN(tv.tot_minprice[TOCHECK])||
		       (!Double.isNaN(this.tm_price_min[cur_pos]) && this.tm_price_min[cur_pos]!=tv.tot_minprice[TOCHECK])||
		       Double.isNaN(this.tm_price_max[cur_pos])!=Double.isNaN(tv.tot_maxprice[TOCHECK])||
		       (!Double.isNaN(this.tm_price_max[cur_pos]) && this.tm_price_max[cur_pos]!=tv.tot_maxprice[TOCHECK])||
		       Double.isNaN(dstotal_buy.median)!=Double.isNaN(tv.trade_tot_median_low_tau[TOCHECK])||
		       (!Double.isNaN(dstotal_buy.median) && dstotal_buy.median!=tv.trade_tot_median_low_tau[TOCHECK])||
		       Double.isNaN(dstotal_sell.median)!=Double.isNaN(tv.trade_tot_median_high_tau[TOCHECK])||
		       (!Double.isNaN(dstotal_sell.median) && dstotal_sell.median!=tv.trade_tot_median_high_tau[TOCHECK])
		       )
		    {
			throw new RuntimeException(
			    "Prices mistmatch:"+
			    " this.tm_price_min["+cur_pos+"]="+this.tm_price_min[cur_pos]+
			    " tv.tot_minprice["+TOCHECK+"]="+tv.tot_minprice[TOCHECK]+
			    " this.tm_price_max["+cur_pos+"]="+this.tm_price_max[cur_pos]+
			    " tv.tot_maxprice["+TOCHECK+"]="+tv.tot_maxprice[TOCHECK]+
			    " dstotal_buy.median="+dstotal_buy.median+
			    " tv.trade_tot_median_low_tau["+TOCHECK+"]="+tv.trade_tot_median_low_tau[TOCHECK]+
			    " dstotal_sell.median="+dstotal_sell.median+
			    " tv.trade_tot_median_high_tau["+TOCHECK+"]="+tv.trade_tot_median_high_tau[TOCHECK]+
			    " delay="+delays1[TOCHECK]+
			    " currenttime="+currenttime);
		    }
		}


		if(true)
		{
		    final long
			vexe=tv.v_alltrades_buy_tau[TOCHECK]+tv.v_alltrades_sell_tau[TOCHECK]+vcross_exe,
			vhexe=tv.v_hiddentrades_buy_tau[TOCHECK]+tv.v_hiddentrades_sell_tau[TOCHECK]+vcross_exe;
		    if(vol_exe!=vexe||vhexe!=vh_exe)
		    {
			throw new RuntimeException("Volume mistmatch:"+
						   " vexe="+vexe+
						   " vol_exe="+vol_exe+
						   " vhexe="+vhexe+
						   " vh_exe="+vh_exe+
						   " delay="+delays1[TOCHECK]+
						   " currenttime="+currenttime);
		    }
		}


		if(true)
		{
		    // check buy volume matches
		    final long v=
			// V0
			cv_buy.vtotalorders+
			tv.v_alltrades_buy_tau[TOCHECK]+tv.v_alltrades_sell_tau[TOCHECK]+
			// cross are not counted in CalculateTradeVolume
			vcross_exe;

			if(v!=m1.v_buy)
			{
			    throw new RuntimeException("Effective buy volume mismatch:"+
						       " v="+v+
						       " m1.v_buy="+m1.v_buy+
						       " delay="+delays1[TOCHECK]+
						       " currenttime="+currenttime);
			}
		}
		if(true)
		{
		    // check sell volume matches
		    final long v=
			// V0
			cv_sell.vtotalorders+
			tv.v_alltrades_buy_tau[TOCHECK]+tv.v_alltrades_sell_tau[TOCHECK]+
			// cross are not counted in CalculateTradeVolume
			vcross_exe;

			if(v!=m1.v_sell)
			{
			    throw new RuntimeException("Effective buy volume mismatch:"+
						       " v="+v+
						       " m1.v_sell="+m1.v_sell+
						       " delay="+delays1[TOCHECK]+
						       " currenttime="+currenttime);
			}
		}
		if(true)
		{
		    // check buy medians
		    for(int i=this.buy.getSize();i-- >0;)
		    {
			final BookItem t=buf_buy[i];
			tmp_buy.addOrderCheckPrev(t.price,t.shares);
		    }
		    final OrdersFlow.DistStat ds=tmp_buy.getStat(frac);
		    if(Double.isNaN(ds.median)!=Double.isNaN(cv_buy.book_median[0]) ||
		       (!Double.isNaN(ds.median) && ds.median!=cv_buy.book_median[0])||
		       ds.vtotal!=cv_buy.out_shares[0])
		    {
			throw new RuntimeException("Tau buy median or volume mismatch:"+
						   " med="+ds.median+
						   " med_c="+cv_buy.book_median[0]+
						   " vol="+ds.vtotal+
						   " vol_c="+cv_buy.out_shares[0]+
						   " delay="+delays1[TOCHECK]+
						   " currenttime="+currenttime);
		    }
		}
		if(true)
		{
		    // check sell medians
		    for(int i=this.sell.getSize();i-- >0;)
		    {
			final BookItem t=buf_sell[i];
			tmp_sell.addOrderCheckPrev(t.price,t.shares);
		    }
		    final OrdersFlow.DistStat ds=tmp_sell.getStat(frac);
		    if(Double.isNaN(ds.median)!=Double.isNaN(cv_sell.book_median[0])||
		       (!Double.isNaN(ds.median) && ds.median!=cv_sell.book_median[0])||
		       ds.vtotal!=cv_sell.out_shares[0])
		    {
			throw new RuntimeException("Tau sell median or volume mismatch:"+
						   " med="+ds.median+
						   " med_c="+cv_sell.book_median[0]+
						   " vol="+ds.vtotal+
						   " vol_c="+cv_sell.out_shares[0]+
						   " delay="+delays1[TOCHECK]+
						   " currenttime="+currenttime);
		    }
		}

	    } // END EXPENSIVE CONSTRAINS CHECK


	} // END LOOP OVER ALL TIME INTERVALS

    }


	private void convertOrdersFlow(final OrdersFlow obuy,
				       final OrdersFlow osell,
				       final double shift)
	{
	    // simplistic model for now.
	    obuy.shiftPrices(shift);
	    osell.shiftPrices(shift);
	}
}
