package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;



/** Simplified version of CalculateTradeVolume to calculate
 *  hidden trades contribution only.
 *  @version $Id: CalculateTradeHContrib.java,v 1.6 2009/12/21 17:17:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateTradeHContrib {

    double []
    // min & max prices.
	minprice=null,
	maxprice=null,
	averprice=null;
    long [] vol_all=null,vol_hidden=null;

    private final Distribution<Trade> trades,tmptradesbuf;
    CalculateTradeHContrib(final Distribution<Trade> trades,
			   final Distribution<Trade> tmptradesbuf)
    {
	this.trades=trades;
	this.tmptradesbuf=tmptradesbuf;
    }
    void clear()
    {
	this.minprice=this.maxprice=this.averprice=null;
	this.vol_all=this.vol_hidden=null;
    }


    void calculateContribs(final long [] delays,
			   final long currenttime,
			   final double pcurrent_buy,
			   final double pcurrent_sell,
			   final long trades_cleared_before)
	{

	    final int tsize=this.trades.getSize();


	    final boolean PRINT_DATA_DEBUG=false;
	    //currenttime==34434604000000L;
	    //currenttime==34354409000000L;
	    //currenttime==34223484000000L;
	    //currenttime>=9*3600*TraderInterface.TIME_SCALE && tsize>0;
	    //currenttime==34204365000000L;


	    // init arrays
	    final int nt=delays.length;
	    this.minprice=new double [nt];
	    this.maxprice=new double [nt];
	    this.averprice=new double [nt];
	    this.vol_all=new long [nt];
	    this.vol_hidden=new long [nt];


	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("=="+getClass().getName()+"============TRADE TIME VOLUME t=\t"+currenttime+"\t NTRADES=\t"+tsize+
				   "\tcur_buy\t"+pcurrent_buy+
				   "\tcur_sell\t"+pcurrent_sell);
	    }
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("num\t"+
				   "time\t"+
				   "shares\t"+
				   "tradetype\t"+
				   "buy_sell_indicator\t"+
				   "price\t"+
				   "matchnumber\t"+
				   "orderreferencenumber");
	    }

	    for(int i=0;i<nt;i++)
	    {
		this.minprice[i]=this.maxprice[i]=Double.NaN;
		this.averprice[i]=0;
	    }


	    final Trade [] tb=this.trades.getBuffer();
	    long tbefore=currenttime-delays[0];
	    long prevt=Long.MAX_VALUE;
	    double min=Double.POSITIVE_INFINITY,max=Double.NEGATIVE_INFINITY;
	    gloop:for(int i=tsize,cur_pos=0;i-- >0;)
	    {
		final Trade t=tb[i];
		if(t.time>prevt)
		{
		    throw new RuntimeException("The data is not sorted:"+
					       prevt+" and "+t.time);
		}
		prevt=t.time;
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.shares+"\t"+
				       t.tradetype+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.price+"\t"+
				       t.matchnumber+"\t"+
				       t.orderreferencenumber+
				       (t.order!=null?
					"\t\t"+(t.time-t.order.time)+"\t"+
					t.order.time+"\t"+
					t.order.shares+"\t"+
					t.order.price:
					"\t\t-10\t\t\t"));
		}
		for(;!(t.time>=tbefore);)
		{
		    if(++cur_pos>=nt)
		    {
			break gloop;
		    }
		    tbefore=currenttime-delays[cur_pos];
		}
		if(/* tbefore>0L && */tbefore<trades_cleared_before)
		{
		    throw new RuntimeException("Something is terrible wrong with old data removal");
		}
		if(t.price<=min)
		{
		    min=this.minprice[cur_pos]=t.price;
		}
		if(t.price>=max)
		{
		    max=this.maxprice[cur_pos]=t.price;
		}
		if(0>(this.vol_all[cur_pos]+=t.shares))
		{
		    throw new RuntimeException("Overflow");
		}
		this.averprice[cur_pos]+=t.price*t.shares;
		if(t.order==null)
		{
		    this.vol_hidden[cur_pos]+=t.shares;
		}
	    }


	    for(int i=1;i<nt;i++)
	    {
		final int i1=i-1;
		this.vol_all[i]+=this.vol_all[i1];
		this.averprice[i]+=this.averprice[i1];
		this.vol_hidden[i]+=this.vol_hidden[i1];
		if(Double.isNaN(this.minprice[i]))
		{
		    this.minprice[i]=this.minprice[i1];
		}
		if(Double.isNaN(this.maxprice[i]))
		{
		    this.maxprice[i]=this.maxprice[i1];
		}
	    }
	    for(int i=0;i<nt;i++)
	    {
		final long v=this.vol_all[i];
		this.averprice[i]=v>0?this.averprice[i]/v:Double.NaN;
	    }

	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("tau\t"+
				   "vol_all\t"+
				   "vol_hidden\t"+
				   "minprice\t"+
				   "maxprice\t"+
				   "averprice\t");
		for(int i=0;i<nt;i++)
		{
		  System.err.println(String.valueOf(delays[i])+"\t"+
				     this.vol_all[i]+"\t"+
				     this.vol_hidden[i]+"\t"+
				     this.minprice[i]+"\t"+
				     this.maxprice[i]+"\t"+
				     this.averprice[i]+"\t");
		}
	    }




            // EXPENSIVE CONSTRAINS CHECK
	    if(PRINT_DATA_DEBUG||OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	    {
		final CalculateTradeVolume tv=
		    new CalculateTradeVolume(this.trades,this.tmptradesbuf);
		tv.calculateTradeVolume(
		    delays,
		    currenttime,
		    0.5,
		    0.92,
		    0.5,
		    pcurrent_buy,
		    pcurrent_sell,
		    trades_cleared_before);


		for(int i=0;i<nt;i++)
		{
		    if(this.vol_all[i]!=tv.v_tot_alltrades[i] ||
		       this.vol_hidden[i]!=tv.v_tot_hiddentrades[i] ||
		       Double.isNaN(this.minprice[i])!=Double.isNaN(tv.tot_minprice[i]) ||
		       (!Double.isNaN(this.minprice[i]) && this.minprice[i]!=tv.tot_minprice[i]) ||
		       Double.isNaN(this.maxprice[i])!=Double.isNaN(tv.tot_maxprice[i]) ||
		       (!Double.isNaN(this.maxprice[i]) && this.maxprice[i]!=tv.tot_maxprice[i]))
		    {
			throw new RuntimeException(
			    "Data mismatch:"+
			    " vol_all["+i+"]="+this.vol_all[i]+
			    " tv.v_tot_alltrades["+i+"]="+tv.v_tot_alltrades[i]+
			    " vol_hidden["+i+"]="+this.vol_hidden[i]+
			    " tv.v_tot_hiddentrades["+i+"]="+tv.v_tot_hiddentrades[i]+
			    " minprice["+i+"]="+this.minprice[i]+
			    " tv.tot_minprice["+i+"]="+tv.tot_minprice[i]+
			    " maxprice["+i+"]="+this.maxprice[i]+
			    " tv.tot_maxprice["+i+"]="+tv.tot_maxprice[i]+
			    " delay="+delays[i]+
			    " currenttime="+currenttime);
		    }
		}
	    }


	}
}
