package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;



/** Calculate trade volume and median distributions.
 *  @version $Id: CalculateTradeVolume.java,v 1.142 2020/02/06 00:09:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateTradeVolume {

    private static final boolean
	DEBUG_TEST_PRINTVOLUME=false,
	DEBUG_TEST_PRINTVOLUME_ALLTRADES=false;

        double []
	// min sell & max buy
	    minprice=null,
	    maxprice=null,
	// for internal use
	    cc_minprice_buy=null,
	    cc_maxprice_sell=null,
	    tot_minprice=null,
	    tot_maxprice=null;

	long []
	// volumes
	    v_alltrades_buy_tau=null,
	    v_alltrades_sell_tau=null,
	    v_hiddentrades_buy_tau=null,
	    v_hiddentrades_sell_tau=null,
	    v_tot_alltrades=null,
	    v_tot_hiddentrades=null,
	    v_tte_known_buy_tau=null,
	    v_tte_known_sell_tau=null,
	// aver tte
	    time_to_execute_aver_buy=null,
	    time_to_execute_aver_sell=null,

	// effective orders
	    v_eo_buy_tau=null,
	    v_eo_sell_tau=null,

	// volumes of trades with price above current
	    v_abovecurrent_buy_tau=null,
	    v_belowcurrent_sell_tau=null;

        int []
	// the number of trades & orders
	    n_trades_buy_tau=null,
	    n_trades_sell_tau=null,
	    n_orders_buy_tau=null,
	    n_orders_sell_tau=null;
        double []
	    opt_buy_aver=null,
	    opt_sell_aver=null,
	    opt_aver=null;

        int []
	    time_to_execute_buy=null,time_to_execute_sell=null;

	int []
            // medians
	    disttofrac_all_buy=null,
	    disttofrac_all_sell=null,
	    disttofrac_all=null,
	    disttofrac_hidden_buy=null,
	    disttofrac_hidden_sell=null,
	    disttofrac_hidden=null;

    int [] disttomin=null,disttomax=null;

    double [] trade_median_buy_tau=null,trade_median_sell_tau=null,
	trade_median_f2_buy_tau=null,trade_median_f2_sell_tau=null,
	trade_t0_buy_tau=null,trade_t0_sell_tau=null,
	trade_tm_buy_tau=null,trade_tm_sell_tau=null,
	trade_tot_median_low_tau=null,trade_tot_median_high_tau=null;

    private final Distribution<Trade> savedtrades,tmptradesbuf;
    CalculateTradeVolume(final Distribution<Trade> savedtrades,
			 final Distribution<Trade> tmptradesbuf)
	{
	    this.savedtrades=savedtrades;
	    this.tmptradesbuf=tmptradesbuf;
	}
	void setData(final int nt)
	{

	    if(nt<0)
	    {
		this.v_alltrades_buy_tau=
		this.v_alltrades_sell_tau=
		this.v_hiddentrades_buy_tau=
		this.v_hiddentrades_sell_tau=
		this.v_eo_buy_tau=
		this.v_eo_sell_tau=
		this.v_abovecurrent_buy_tau=
		this.v_belowcurrent_sell_tau=
		this.v_tot_alltrades=
		this.v_tot_hiddentrades=
		this.v_tte_known_buy_tau=
		this.v_tte_known_sell_tau=
		this.time_to_execute_aver_buy=
		this.time_to_execute_aver_sell=null;

		this.disttofrac_all_buy=
	        this.disttofrac_all_sell=
	        this.disttofrac_all=
	        this.disttofrac_hidden_buy=
	        this.disttofrac_hidden_sell=
		this.disttofrac_hidden=
		this.disttomin=
		this.disttomax=
		this.n_trades_buy_tau=
		this.n_trades_sell_tau=
		this.n_orders_buy_tau=
		this.n_orders_sell_tau=
		this.time_to_execute_buy=
		this.time_to_execute_sell=null;
		this.opt_buy_aver=
		this.opt_sell_aver=
		this.opt_aver=
		this.minprice=
		this.maxprice=
		this.cc_minprice_buy=
		this.cc_maxprice_sell=
		this.tot_minprice=
		this.tot_maxprice=null;
		this.trade_median_buy_tau=
		this.trade_median_sell_tau=
		this.trade_median_f2_buy_tau=
		this.trade_median_f2_sell_tau=
		this.trade_t0_buy_tau=
		this.trade_t0_sell_tau=
		this.trade_tm_buy_tau=
		this.trade_tot_median_low_tau=
		this.trade_tot_median_high_tau=null;
	    }
	    else
	    {
		this.v_alltrades_buy_tau=new long [nt];
		this.v_alltrades_sell_tau=new long [nt];
		this.v_hiddentrades_buy_tau=new long [nt];
		this.v_hiddentrades_sell_tau=new long [nt];
		this.v_eo_buy_tau=new long [nt];
		this.v_eo_sell_tau=new long [nt];
		this.v_abovecurrent_buy_tau=new long [nt];
		this.v_belowcurrent_sell_tau=new long [nt];
		this.v_tot_alltrades=new long [nt];
		this.v_tot_hiddentrades=new long [nt];
		this.v_tte_known_buy_tau=new long [nt];
		this.v_tte_known_sell_tau=new long [nt];
		this.time_to_execute_aver_buy=new long [nt];
		this.time_to_execute_aver_sell=new long [nt];
		this.disttofrac_all_buy=new int [nt];
		this.disttofrac_all_sell=new int [nt];
		this.disttofrac_all=new int [nt];
		this.disttofrac_hidden_buy=new int [nt];
		this.disttofrac_hidden_sell=new int [nt];
		this.disttofrac_hidden=new int [nt];
		this.disttomin=new int [nt];
		this.disttomax=new int [nt];
		this.n_trades_buy_tau=new int [nt];
		this.n_trades_sell_tau=new int [nt];
		this.n_orders_buy_tau=new int [nt];
		this.n_orders_sell_tau=new int [nt];
		this.time_to_execute_buy=new int [nt];
		this.time_to_execute_sell=new int [nt];
		this.opt_buy_aver=new double [nt];
		this.opt_sell_aver=new double [nt];
		this.opt_aver=new double [nt];
		this.minprice=new double [nt];
		this.maxprice=new double [nt];
		this.cc_minprice_buy=new double [nt];
		this.cc_maxprice_sell=new double [nt];
		this.tot_minprice=new double [nt];
		this.tot_maxprice=new double [nt];
		this.trade_median_buy_tau=new double [nt];
		this.trade_median_sell_tau=new double [nt];
		this.trade_median_f2_buy_tau=new double [nt];
		this.trade_median_f2_sell_tau=new double [nt];
		this.trade_t0_buy_tau=new double [nt];
		this.trade_t0_sell_tau=new double [nt];
		this.trade_tm_buy_tau=new double [nt];
		this.trade_tm_sell_tau=new double [nt];
		this.trade_tot_median_low_tau=new double [nt];
		this.trade_tot_median_high_tau=new double [nt];
	    }
	}


    void calculateTradeVolume(final long [] delays,
			      final long currenttime,
			      final double frac,
			      final double frac2,
			      final double frac_dist,
			      final double pcurrent_buy,
			      final double pcurrent_sell,
			      final long trades_cleared_before)
	{
	    if(frac <0.0 || frac>1.0 || frac2<0.0 || frac2>1.0 || frac>=frac2)
	    {
		throw new IllegalArgumentException("frac="+frac+" frac2="+frac2);
	    }

	    final int tsize=this.savedtrades.getSize();

	    /* use command
	       java -server -Xmx6500000K com/polytechnik/trading/GenerateTrainingData /pdata/prices/out_ml/ /rdata/itch/itch3/S072308-v3.zip
	       t=34223484000000
	    */
	    final boolean PRINT_DATA_DEBUG=false;
	    //currenttime==34434604000000L;
	    //currenttime==34354409000000L;
	    //currenttime==34223484000000L;
	    //currenttime>=9*3600*TraderInterface.TIME_SCALE && tsize>0;
	    //currenttime==34204365000000L;

	    final Trade [] tb=this.tmptradesbuf.getBuffer();
	    System.arraycopy(this.savedtrades.getBuffer(),0,tb,0,tsize);

	    // init arrays
	    final int nt=delays.length;
	    setData(nt);

	    final int [] tau_interval_range_ptr=new int [nt];
	    final int [] ptrlast_buy=new int [nt],ptrlast_sell=new int [nt];
	    final long [] v_cross=new long [nt];



	    if(DEBUG_TEST_PRINTVOLUME||PRINT_DATA_DEBUG)
	    {
		System.err.println("=="+getClass().getName()+"============TRADE TIME VOLUME t=\t"+currenttime+"\t NTRADES=\t"+tsize+
				   "\tcur_buy\t"+pcurrent_buy+
				   "\tcur_sell\t"+pcurrent_sell+
				   "\tfrac\t"+frac+
				   "\tfrac2\t"+frac2+
				   "\tfrac_dist\t"+frac_dist);
	    }
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("num\t"+
				   "time\t"+
				   "shares\t"+
				   "tradetype\t"+
				   "buy_sell_indicator\t"+
				   "price\t"+
				   "matchnumber\t"+
				   "orderreferencenumber");
	    }

	    for(int i=0;i<nt;i++)
	    {
                // flag not set
		tau_interval_range_ptr[i]=-1;

		// -1 as not set
		this.disttofrac_all_buy[i]=
		    this.disttofrac_all_sell[i]=
		    this.disttofrac_all[i]=
		    this.disttofrac_hidden_buy[i]=
		    this.disttofrac_hidden_sell[i]=
		    this.disttofrac_hidden[i]=
		    this.disttomin[i]=
		    this.disttomax[i]=
		    ptrlast_buy[i]=
		    ptrlast_sell[i]=
		    this.n_orders_buy_tau[i]=
		    this.n_orders_sell_tau[i]=
		    this.time_to_execute_buy[i]=
		    this.time_to_execute_sell[i]=-1;
		this.opt_buy_aver[i]=
		    this.opt_sell_aver[i]=
		    this.opt_aver[i]=
		    this.minprice[i]=
		    this.maxprice[i]=
		    this.cc_minprice_buy[i]=
		    this.cc_maxprice_sell[i]=
		    this.tot_minprice[i]=
		    this.tot_maxprice[i]=
		    this.trade_median_buy_tau[i]=
		    this.trade_median_sell_tau[i]=
		    this.trade_median_f2_buy_tau[i]=
		    this.trade_median_f2_sell_tau[i]=
		    this.trade_t0_buy_tau[i]=
		    this.trade_t0_sell_tau[i]=
		    this.trade_tm_buy_tau[i]=
		    this.trade_tm_sell_tau[i]=
		    this.trade_tot_median_low_tau[i]=
		    this.trade_tot_median_high_tau[i]=Double.NaN;
	    }


	    long tbefore=currenttime-delays[0];
	    long prevt=Long.MAX_VALUE;
	    gloop:for(int i=tsize,cur_pos=0;i-- >0;)
	    {
		final Trade t=tb[i];
		if(t.time>prevt)
		{
		    throw new RuntimeException("The data is not sorted:"+
					       prevt+" and "+t.time);
		}
		prevt=t.time;
		if(PRINT_DATA_DEBUG)
		{
		    System.err.println(String.valueOf(i)+"\t"+
				       t.time+"\t"+
				       t.shares+"\t"+
				       t.tradetype+"\t"+
				       t.buy_sell_indicator+"\t"+
				       t.price+"\t"+
				       t.matchnumber+"\t"+
				       t.orderreferencenumber+
				       (t.order!=null?
					"\t\t"+(t.time-t.order.time)+"\t"+
					t.order.time+"\t"+
					t.order.shares+"\t"+
					t.order.price:
					"\t\t-10\t\t\t"));
		}
		for(;!(t.time>=tbefore);)
		{
		    if(++cur_pos>=nt)
		    {
			break gloop;
		    }
		    final long tb1=tbefore;
		    tbefore=currenttime-delays[cur_pos];
		    if(tb1<tbefore)
		    {
			throw new RuntimeException("delays are not sorted"+
						   " tb1="+tb1+
						   " tbefore="+tbefore+
						   " currenttime="+currenttime);
		    }
		}
		if(/* tbefore>0L && */tbefore<trades_cleared_before)
		{
		    throw new RuntimeException("Something is terrible wrong with old data removal");
		}
		if(DEBUG_TEST_PRINTVOLUME_ALLTRADES)
		{
		    System.err.println("t_time="+t.time+" tbefore="+tbefore);
		}
		tau_interval_range_ptr[cur_pos]=i;
		final int s=t.shares;
		if(t.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    this.v_alltrades_buy_tau[cur_pos]+=s;
		    this.n_trades_buy_tau[cur_pos]++;
		    if(t.price>pcurrent_buy)
		    {
			this.v_abovecurrent_buy_tau[cur_pos]+=s;
		    }
		    if(t.order==null)
		    {
			this.v_hiddentrades_buy_tau[cur_pos]+=s;
		    }
		    if(t.origtime>0)
		    {
			this.v_tte_known_buy_tau[cur_pos]+=s;
			this.time_to_execute_aver_buy[cur_pos]+=s*((t.time-t.origtime)/ModelParameters.T_SUMM_SCALE);
		    }
		}
		else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    this.v_alltrades_sell_tau[cur_pos]+=s;
		    this.n_trades_sell_tau[cur_pos]++;
		    if(t.price<pcurrent_sell)
		    {
			this.v_belowcurrent_sell_tau[cur_pos]+=s;
		    }
		    if(t.order==null)
		    {
			this.v_hiddentrades_sell_tau[cur_pos]+=s;
		    }
		    if(t.origtime>0)
		    {
			this.v_tte_known_sell_tau[cur_pos]+=s;
			this.time_to_execute_aver_sell[cur_pos]+=s*((t.time-t.origtime)/ModelParameters.T_SUMM_SCALE);
		    }
		}
		else
		{
		    v_cross[cur_pos]+=s;
		}
	    }
	    for(int i=0;i<nt;i++)
	    {
		if(i>0)
		{
		    final int i1=i-1;
		    this.v_alltrades_buy_tau[i]+=this.v_alltrades_buy_tau[i1];
		    this.n_trades_buy_tau[i]+=this.n_trades_buy_tau[i1];
		    this.v_alltrades_sell_tau[i]+=this.v_alltrades_sell_tau[i1];
		    this.n_trades_sell_tau[i]+=this.n_trades_sell_tau[i1];
		    this.v_hiddentrades_buy_tau[i]+=this.v_hiddentrades_buy_tau[i1];
		    this.v_hiddentrades_sell_tau[i]+=this.v_hiddentrades_sell_tau[i1];
		    this.v_abovecurrent_buy_tau[i]+=this.v_abovecurrent_buy_tau[i1];
		    this.v_belowcurrent_sell_tau[i]+=this.v_belowcurrent_sell_tau[i1];
		    this.v_tte_known_buy_tau[i]+=this.v_tte_known_buy_tau[i1];
		    this.v_tte_known_sell_tau[i]+=this.v_tte_known_sell_tau[i1];
		    this.time_to_execute_aver_buy[i]+=this.time_to_execute_aver_buy[i1];
		    this.time_to_execute_aver_sell[i]+=this.time_to_execute_aver_sell[i1];
		    v_cross[i]+=v_cross[i1];
		}
		this.v_tot_alltrades[i]=this.v_alltrades_buy_tau[i]+
		    this.v_alltrades_sell_tau[i]+v_cross[i];
		this.v_tot_hiddentrades[i]=this.v_hiddentrades_buy_tau[i]+
		    this.v_hiddentrades_sell_tau[i]+v_cross[i];
	    }
	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("tau\t"+
				   "v_alltrades_buy_tau\t"+
				   "n_trades_buy_tau\t"+
				   "v_alltrades_sell_tau\t"+
				   "n_trades_sell_tau\t"+
				   "v_all\t"+
				   "v_hiddentrades_buy_tau\t"+
				   "v_hiddentrades_sell_tau\t"+
				   "v_hidden\t"+
				   "v_abovecurrent_buy_tau\t"+
				   "v_belowcurrent_sell_tau");
		for(int i=0;i<nt;i++)
		{
		  System.err.println(String.valueOf(delays[i])+"\t"+
				     this.v_alltrades_buy_tau[i]+"\t"+
				     this.n_trades_buy_tau[i]+"\t"+
				     this.v_alltrades_sell_tau[i]+"\t"+
				     this.n_trades_sell_tau[i]+"\t"+
				     (this.v_alltrades_buy_tau[i]+this.v_alltrades_sell_tau[i])+"\t"+
				     this.v_hiddentrades_buy_tau[i]+"\t"+
				     this.v_hiddentrades_sell_tau[i]+"\t"+
				     (this.v_hiddentrades_buy_tau[i]+this.v_hiddentrades_sell_tau[i])+"\t"+
				     this.v_abovecurrent_buy_tau[i]+"\t"+
				     this.v_belowcurrent_sell_tau[i]);
		}
	    }


	    long vdisttofrac_all_buy=0,
	         vdisttofrac_all_sell=0,
	         vdisttofrac_hidden_buy=0,
	         vdisttofrac_hidden_sell=0;

	    long prev_vdisttofrac_all_buy=0,
	         prev_vdisttofrac_all_sell=0,
		 prev_vdisttofrac_all=0,
	         prev_vdisttofrac_hidden_buy=0,
	         prev_vdisttofrac_hidden_sell=0,
		 prev_vdisttofrac_hidden=0;

	    long lev_disttofrac_all_buy=-1,
		 lev_disttofrac_all_sell=-1,
		 lev_disttofrac_all=-1,
		 lev_disttofrac_hidden_buy=-1,
		 lev_disttofrac_hidden_sell=-1,
		 lev_disttofrac_hidden=-1;

	    int ptr_disttofrac_all_buy=-1,
		ptr_disttofrac_all_sell=-1,
		ptr_disttofrac_all=-1,
		ptr_disttofrac_hidden_buy=-1,
		ptr_disttofrac_hidden_sell=-1,
		ptr_disttofrac_hidden=-1;

	    double lev_min=Double.POSITIVE_INFINITY,lev_max=Double.NEGATIVE_INFINITY;
	    tbefore=prevt=Long.MAX_VALUE;
	    gloop2:for(int i=tsize,cur_pos=-1;i-- >0;)
	    {
		final Trade t=tb[i];
		if(t.time>prevt)
		{
		    throw new RuntimeException("The data is not sorted:"+
					       prevt+" and "+t.time);
		}
		prevt=t.time;
		for(;cur_pos<0||!(t.time>=tbefore);)
		{
		    if(cur_pos>=0 &&(
			   vdisttofrac_all_buy!=this.v_alltrades_buy_tau[cur_pos]||
			   vdisttofrac_all_sell!=this.v_alltrades_sell_tau[cur_pos]||
			   vdisttofrac_hidden_buy!=this.v_hiddentrades_buy_tau[cur_pos]||
			   vdisttofrac_hidden_sell!=this.v_hiddentrades_sell_tau[cur_pos]) )
		    {
			throw new RuntimeException("Broken invariant. This should never happen. currenttime="+currenttime);
		    }
		    if(++cur_pos>=nt)
		    {
			break gloop2;
		    }
		    final long tb1=tbefore;
		    tbefore=currenttime-delays[cur_pos];
		    if(tb1<tbefore)
		    {
			throw new RuntimeException("delays are not sorted"+
						   " tb1="+tb1+
						   " tbefore="+tbefore+
						   " currenttime="+currenttime);
		    }
		}


		if(t.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    if(t.price>lev_max)
		    {
			lev_max=t.price;
			this.disttomax[cur_pos]=i;
		    }
		    vdisttofrac_all_buy+=t.shares;
		    if(t.order==null)
		    {
			vdisttofrac_hidden_buy+=t.shares;
		    }
		    ptrlast_buy[cur_pos]=i;
		}
		else if(t.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    if(t.price<lev_min)
		    {
			lev_min=t.price;
			this.disttomin[cur_pos]=i;
		    }
		    vdisttofrac_all_sell+=t.shares;
		    if(t.order==null)
		    {
			vdisttofrac_hidden_sell+=t.shares;
		    }
		    ptrlast_sell[cur_pos]=i;
		}
		/*
		else {
		    throw new RuntimeException(String.valueOf(currenttime));
		}
		*/
		final long vallsum=vdisttofrac_all_buy+vdisttofrac_all_sell,
		    vhiddensum=vdisttofrac_hidden_buy+vdisttofrac_hidden_sell;
		if(vallsum<0||
		   vdisttofrac_all_buy<0||
		   vdisttofrac_all_sell<0||
		   vhiddensum<0||
		   vdisttofrac_hidden_buy<0||
		   vdisttofrac_hidden_sell<0)
		{
		    throw new RuntimeException("Integer overflow currenttime="+currenttime);
		}


		if(false && PRINT_DATA_DEBUG)
		{
		    System.err.println("LOOP ptr_disttofrac_all_buy="+ptr_disttofrac_all_buy+
				       " prev_vdisttofrac_all_buy="+
				       prev_vdisttofrac_all_buy+
				       " vdisttofrac_all_buy="+
				       vdisttofrac_all_buy+
				       " lev_disttofrac_all_buy="+
				       lev_disttofrac_all_buy+
				       " i="+i+
				       " time="+t.time);
		}


		for(;ptr_disttofrac_all_buy<nt && vdisttofrac_all_buy>lev_disttofrac_all_buy;)
		{
		    if(vdisttofrac_all_buy>lev_disttofrac_all_buy &&
		       prev_vdisttofrac_all_buy<=lev_disttofrac_all_buy)
		    {
			if(ptr_disttofrac_all_buy>=cur_pos)
			{
			    this.disttofrac_all_buy[ptr_disttofrac_all_buy]=i;
			}
		    }
		    if(++ptr_disttofrac_all_buy<nt)
		    {
			lev_disttofrac_all_buy=(long)(frac_dist*this.v_alltrades_buy_tau[ptr_disttofrac_all_buy]);
		    }
		    else
		    {
			break;
		    }
		}

		if(false && PRINT_DATA_DEBUG)
		{
		    System.err.println("LOOP AFTRER ptr_disttofrac_all_buy="+ptr_disttofrac_all_buy+
				       " prev_vdisttofrac_all_buy="+
				       prev_vdisttofrac_all_buy+
				       " vdisttofrac_all_buy="+
				       vdisttofrac_all_buy+
				       " lev_disttofrac_all_buy="+
				       lev_disttofrac_all_buy+
				       " i="+i+
				       " time="+t.time);
		}

		for(;ptr_disttofrac_all_sell<nt && vdisttofrac_all_sell>lev_disttofrac_all_sell;)
		{
		    if(vdisttofrac_all_sell>lev_disttofrac_all_sell &&
		       prev_vdisttofrac_all_sell<=lev_disttofrac_all_sell)
		    {
			if(ptr_disttofrac_all_sell>=cur_pos)
			{
			    this.disttofrac_all_sell[ptr_disttofrac_all_sell]=i;
			}
		    }
		    if(++ptr_disttofrac_all_sell<nt)
		    {
			lev_disttofrac_all_sell=(long)(frac_dist*this.v_alltrades_sell_tau[ptr_disttofrac_all_sell]);
		    }
		    else
		    {
			break;
		    }
		}


		for(;ptr_disttofrac_all<nt && vallsum>lev_disttofrac_all;)
		{
		    if(vallsum>lev_disttofrac_all &&
		       prev_vdisttofrac_all<=lev_disttofrac_all)
		    {
			if(ptr_disttofrac_all>=cur_pos)
			{
			    this.disttofrac_all[ptr_disttofrac_all]=i;
			}
		    }
		    if(++ptr_disttofrac_all<nt)
		    {
			lev_disttofrac_all=(long)(frac_dist*
						 (this.v_alltrades_buy_tau[ptr_disttofrac_all]+
						  this.v_alltrades_sell_tau[ptr_disttofrac_all])
						 );
		    }
		    else
		    {
			break;
		    }
		}

		for(;ptr_disttofrac_hidden_buy<nt && vdisttofrac_hidden_buy>lev_disttofrac_hidden_buy;)
		{
		    if(vdisttofrac_hidden_buy>lev_disttofrac_hidden_buy &&
		       prev_vdisttofrac_hidden_buy<=lev_disttofrac_hidden_buy)
		    {
			if(ptr_disttofrac_hidden_buy>=cur_pos)
			{
			    this.disttofrac_hidden_buy[ptr_disttofrac_hidden_buy]=i;
			}
		    }
		    if(++ptr_disttofrac_hidden_buy<nt)
		    {
			lev_disttofrac_hidden_buy=(long)(frac_dist*this.v_hiddentrades_buy_tau[ptr_disttofrac_hidden_buy]);
		    }
		    else
		    {
			break;
		    }
		}

		for(;ptr_disttofrac_hidden_sell<nt && vdisttofrac_hidden_sell>lev_disttofrac_hidden_sell;)
		{
		    if(vdisttofrac_hidden_sell>lev_disttofrac_hidden_sell &&
		       prev_vdisttofrac_hidden_sell<=lev_disttofrac_hidden_sell)
		    {
			if(ptr_disttofrac_hidden_sell>=cur_pos)
			{
			    this.disttofrac_hidden_sell[ptr_disttofrac_hidden_sell]=i;
			}
		    }
		    if(++ptr_disttofrac_hidden_sell<nt)
		    {
			lev_disttofrac_hidden_sell=(long)(frac_dist*this.v_hiddentrades_sell_tau[ptr_disttofrac_hidden_sell]);
		    }
		    else
		    {
			break;
		    }
		}


		for(;ptr_disttofrac_hidden<nt && vhiddensum>lev_disttofrac_hidden;)
		{
		    if(vhiddensum>lev_disttofrac_hidden &&
		       prev_vdisttofrac_hidden<=lev_disttofrac_hidden)
		    {
			if(ptr_disttofrac_hidden>=cur_pos)
			{
			    this.disttofrac_hidden[ptr_disttofrac_hidden]=i;
			}
		    }
		    if(++ptr_disttofrac_hidden<nt)
		    {
			lev_disttofrac_hidden=(long)(frac_dist*
						    (this.v_hiddentrades_buy_tau[ptr_disttofrac_hidden]+
						     this.v_hiddentrades_sell_tau[ptr_disttofrac_hidden])
						    );
		    }
		    else
		    {
			break;
		    }
		}


		prev_vdisttofrac_all_buy=vdisttofrac_all_buy;
		prev_vdisttofrac_all_sell=vdisttofrac_all_sell;
		prev_vdisttofrac_all=vallsum;
		prev_vdisttofrac_hidden_buy=vdisttofrac_hidden_buy;
		prev_vdisttofrac_hidden_sell=vdisttofrac_hidden_sell;
		prev_vdisttofrac_hidden=vhiddensum;

	    }


	    // constrains (incomplete). check possible misses.
	    for(int i=0;i<nt;i++)
	    {
		final int i1=i-1;
		if(
		   (this.disttofrac_all_buy[i]<0 &&
		      0<this.v_alltrades_buy_tau[i]-(i1>=0?this.v_alltrades_buy_tau[i1]:0))||
		   (this.disttofrac_all_sell[i]<0 &&
		      0<this.v_alltrades_sell_tau[i]-(i1>=0?this.v_alltrades_sell_tau[i1]:0))||
		   (this.disttofrac_hidden_buy[i]<0 &&
		      0<this.v_hiddentrades_buy_tau[i]-(i1>=0?this.v_hiddentrades_buy_tau[i1]:0))||
		   (this.disttofrac_hidden_sell[i]<0 &&
		      0<this.v_hiddentrades_sell_tau[i]-(i1>=0?this.v_hiddentrades_sell_tau[i1]:0))||
		   (this.disttofrac_all[i]<0 &&
		      (this.disttofrac_all_buy[i]>=0 || this.disttofrac_all_sell[i]>=0))||
		   (this.disttofrac_hidden[i]<0 &&
		      (this.disttofrac_hidden_buy[i]>=0 || this.disttofrac_hidden_sell[i]>=0))
		   )
		{
		    throw new RuntimeException("Some points may be missed. currenttime="+currenttime);
		}
	    }



	    // try to use previous if current one has no data.
	    for(int i=1;i<nt;i++)
	    {
		final int i1=i-1;
		if(this.disttofrac_all_buy[i]<0)
		{
		    this.disttofrac_all_buy[i]=this.disttofrac_all_buy[i1];
		}
		if(this.disttofrac_all_sell[i]<0)
		{
		    this.disttofrac_all_sell[i]=this.disttofrac_all_sell[i1];
		}
		if(this.disttofrac_all[i]<0)
		{
		    this.disttofrac_all[i]=this.disttofrac_all[i1];
		}
		if(this.disttofrac_hidden_buy[i]<0)
		{
		    this.disttofrac_hidden_buy[i]=this.disttofrac_hidden_buy[i1];
		}
		if(this.disttofrac_hidden_sell[i]<0)
		{
		    this.disttofrac_hidden_sell[i]=this.disttofrac_hidden_sell[i1];
		}
		if(this.disttofrac_hidden[i]<0)
		{
		    this.disttofrac_hidden[i]=this.disttofrac_hidden[i1];
		}
		if(this.disttomin[i]<0)
		{
		    this.disttomin[i]=this.disttomin[i1];
		}
		if(this.disttomax[i]<0)
		{
		    this.disttomax[i]=this.disttomax[i1];
		}
		if(ptrlast_buy[i]<0)
		{
		    ptrlast_buy[i]=ptrlast_buy[i1];
		}
		if(ptrlast_sell[i]<0)
		{
		    ptrlast_sell[i]=ptrlast_sell[i1];
		}
	    }

	    if(PRINT_DATA_DEBUG)
	    {
		System.err.println("lev_min\t"+lev_min+"\tlev_max\t"+lev_max);
		System.err.println("pcurrent_buy\t"+pcurrent_buy+"\tpcurrent_sell\t"+pcurrent_sell);
		System.err.println("tau\t"+
				   "ind_disttofrac_all_buy\t"+
				   "ind_disttofrac_all_sell\t"+
				   "ind_disttofrac_all\t"+
				   "ind_disttofrac_hidden_buy\t"+
				   "ind_disttofrac_hidden_sell\t"+
				   "ind_disttofrac_hidden\t"+
				   "ind_disttomin\t"+
				   "ind_disttomax\t"+
				   "ind_ptrlast_buy\t"+
				   "ind_ptrlast_sell\t");
		for(int i=0;i<nt;i++)
		{
		  System.err.println(String.valueOf(delays[i])+"\t"+
				     this.disttofrac_all_buy[i]+"\t"+
				     this.disttofrac_all_sell[i]+"\t"+
				     this.disttofrac_all[i]+"\t"+
				     this.disttofrac_hidden_buy[i]+"\t"+
				     this.disttofrac_hidden_sell[i]+"\t"+
				     this.disttofrac_hidden[i]+"\t"+
				     this.disttomin[i]+"\t"+
				     this.disttomax[i]+"\t"+
				     ptrlast_buy[i]+"\t"+
				     ptrlast_sell[i]+"\t");
		}
	    }


	    // set actual values
	    for(int i=0;i<nt;i++)
	    {
		if(this.v_tte_known_buy_tau[i]>0)
		{
		    this.time_to_execute_aver_buy[i]=
			(this.time_to_execute_aver_buy[i]/this.v_tte_known_buy_tau[i])
			*ModelParameters.T_SUMM_SCALE;
		}
		else
		{
		    this.time_to_execute_aver_buy[i]=-1;
		}
		if(this.v_tte_known_sell_tau[i]>0)
		{
		    this.time_to_execute_aver_sell[i]=
			(this.time_to_execute_aver_sell[i]/this.v_tte_known_sell_tau[i])
			*ModelParameters.T_SUMM_SCALE;
		}
		else
		{
		    this.time_to_execute_aver_sell[i]=-1;
		}

		final int di=(int)(delays[i]/ModelParameters.DISTANCE_TIME_SCALE); // we store delay as int

		if(this.disttofrac_all_buy[i]<0)
		{
		    this.disttofrac_all_buy[i]=di;
		    //this.trade_tm_buy_tau[i]=pcurrent_buy;
		}
		else
		{
		    final Trade t=tb[this.disttofrac_all_buy[i]];
		    if(t.buy_sell_indicator!=BookItem.BOOK_BUY||this.disttofrac_all_buy[i]<ptrlast_buy[i])
		    {
			throw new RuntimeException();
		    }
		    this.disttofrac_all_buy[i]=(int)((currenttime-t.time)/ModelParameters.DISTANCE_TIME_SCALE);
		    this.trade_tm_buy_tau[i]=t.price;
		}
		if(this.disttofrac_all_sell[i]<0)
		{
		    this.disttofrac_all_sell[i]=di;
		    //this.trade_tm_buy_tau[i]=pcurrent_sell;
		}
		else
		{
		    final Trade t=tb[this.disttofrac_all_sell[i]];
		    if(t.buy_sell_indicator!=BookItem.BOOK_SELL||this.disttofrac_all_sell[i]<ptrlast_sell[i])
		    {
			throw new RuntimeException();
		    }
		    this.disttofrac_all_sell[i]=(int)((currenttime-t.time)/ModelParameters.DISTANCE_TIME_SCALE);
		    this.trade_tm_sell_tau[i]=t.price;
		}
		if(this.disttofrac_all[i]<0)
		{
		    this.disttofrac_all[i]=di;
		}
		else
		{
		    this.disttofrac_all[i]=(int)((currenttime-tb[this.disttofrac_all[i]].time)/ModelParameters.DISTANCE_TIME_SCALE);
		}
		if(this.disttofrac_hidden_buy[i]<0)
		{
		    this.disttofrac_hidden_buy[i]=di;
		}
		else
		{
		    this.disttofrac_hidden_buy[i]=(int)((currenttime-tb[this.disttofrac_hidden_buy[i]].time)/ModelParameters.DISTANCE_TIME_SCALE);
		}
		if(this.disttofrac_hidden_sell[i]<0)
		{
		    this.disttofrac_hidden_sell[i]=di;
		}
		else
		{
		    this.disttofrac_hidden_sell[i]=(int)((currenttime-tb[this.disttofrac_hidden_sell[i]].time)/ModelParameters.DISTANCE_TIME_SCALE);
		}
		if(this.disttofrac_hidden[i]<0)
		{
		    this.disttofrac_hidden[i]=di;
		}
		else
		{
		    this.disttofrac_hidden[i]=(int)((currenttime-tb[this.disttofrac_hidden[i]].time)/ModelParameters.DISTANCE_TIME_SCALE);
		}

		if(this.disttomin[i]>=0)
		{
		    final Trade t=tb[this.disttomin[i]];
		    this.minprice[i]=t.price;
		    this.disttomin[i]=(int)((currenttime-t.time)/ModelParameters.DISTANCE_TIME_SCALE);
		    if(this.disttomin[i]>di||this.disttomin[i]<0||t.buy_sell_indicator!=BookItem.BOOK_SELL)
		    {
			throw new RuntimeException("Broken ordering constraints. disttomin["+i+"]="+
						   this.disttomin[i]+" di="+di);
		    }
		}

		if(this.disttomax[i]>=0)
		{
		    final Trade t=tb[this.disttomax[i]];
		    this.maxprice[i]=t.price;
		    this.disttomax[i]=(int)((currenttime-t.time)/ModelParameters.DISTANCE_TIME_SCALE);
		    if(this.disttomax[i]>di||this.disttomax[i]<0||t.buy_sell_indicator!=BookItem.BOOK_BUY)
		    {
			throw new RuntimeException("Broken ordering constraints. disttomax["+i+"]="+
						   this.disttomax[i]+" di="+di);
		    }
		}

		if(ptrlast_buy[i]>=0)
		{
		    this.trade_t0_buy_tau[i]=tb[ptrlast_buy[i]].price;
		}
		else
		{
		    //this.trade_t0_buy_tau[i]=pcurrent_buy;
		}

		if(ptrlast_sell[i]>=0)
		{
		    this.trade_t0_sell_tau[i]=tb[ptrlast_sell[i]].price;
		}
		else
		{
		    //this.trade_t0_sell_tau[i]=pcurrent_sell;
		}

		// constrains check
		if(this.disttofrac_all_buy[i]>di ||
		   this.disttofrac_all_sell[i]>di ||
		   this.disttofrac_all[i]>di ||
		   this.disttofrac_hidden_buy[i]>di ||
		   this.disttofrac_hidden_sell[i]>di ||
		   this.disttofrac_hidden[i]>di)
		{
		    throw new RuntimeException("Broken limit constraints; currenttime="+currenttime);
		}
		final int i1=i-1;
		if(i1>=0 && (
		       this.disttofrac_all_buy[i]<this.disttofrac_all_buy[i1] ||
		       this.disttofrac_all_sell[i]<this.disttofrac_all_sell[i1] ||
		       this.disttofrac_all[i]<this.disttofrac_all[i1] ||
		       this.disttofrac_hidden_buy[i]<this.disttofrac_hidden_buy[i1] ||
		       this.disttofrac_hidden_sell[i]<this.disttofrac_hidden_sell[i1] ||
		       this.disttofrac_hidden[i]<this.disttofrac_hidden[i1]
			    ))
		{
		    throw new RuntimeException("Broken ordering constraints; currenttime="+currenttime);
		}
	    }

	    // set max time before we sort buffer
	    final long tnotbefore=tsize>0?tb[tsize-1].time:-1L;
	    // calculate median in price space for slices in time space
	    for(int cur_pos=0,prev_ptr_start=-1;cur_pos<nt;cur_pos++)
	    {
		final int ptr_start=tau_interval_range_ptr[cur_pos]<0?prev_ptr_start:tau_interval_range_ptr[cur_pos];
		prev_ptr_start=ptr_start;
		if(ptr_start<0)
		{
		    // no points at all for this interval
		    continue;
		}
		final long tnotafter=currenttime-delays[cur_pos];
	      if(true)
	      {
		// order array by type and price.
		// the buffer is considered poisoned after this.
		com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(tb,ptr_start,tsize-1,CompareItems.cmp_by_type_and_price);
		if(false && PRINT_DATA_DEBUG)
		{
		    System.err.println("tau\t"+delays[cur_pos]+"\t"+cur_pos+"\tsorted by price");
		    System.err.println("num\t"+
				       "time\t"+
				       "shares\t"+
				       "buy_sell_indicator\t"+
				       "price\t"+
				       "tradetype\t"+
				       "matchid\t"+
				       "orderid\t");
		    for(int i=ptr_start;i<tsize;i++)
		    {
			final Trade t=tb[i];
			System.err.println(String.valueOf(i)+"\t"+
					   t.time+"\t"+
					   t.shares+"\t"+
					   t.buy_sell_indicator+"\t"+
					   t.price+"\t"+
					   t.tradetype+"\t"+
					   t.matchnumber+"\t"+
					   t.orderreferencenumber+"\t");
		    }
		}

		// median for buy
		long vol=0,prevvol=0,lev=(long)(frac*this.v_alltrades_buy_tau[cur_pos]),lev2=(long)(frac2*this.v_alltrades_buy_tau[cur_pos]);
		double prev_price=Double.NaN;
		int n_trades=0;
		for(int i=ptr_start;i<tsize;i++)
		{
		    final Trade t=tb[i];
		    if(t.buy_sell_indicator!=BookItem.BOOK_BUY)
		    {
			break;
		    }
		    final double p=t.price;
		    if(prev_price<p || !(tnotafter<=t.time && t.time<=tnotbefore)) // will be false for NaN
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    prev_price=p;
		    vol+=t.shares;
		    n_trades++;
		    if(vol<0)
		    {
			throw new RuntimeException("Integer overflow currenttime="+currenttime);
		    }
		    if(vol>lev && lev>=prevvol)
		    {
			if(PRINT_DATA_DEBUG)
			{
			    System.err.println("BUY_LEV_t_"+delays[cur_pos]+"\t"+
					       "vol\t"+vol+"\tP\t"+p+"\t"+lev+"\tPvol\t"+prevvol);
			}
			this.trade_median_buy_tau[cur_pos]=p;
		    }
		    if(vol>lev2 && lev2>=prevvol)
		    {
			if(PRINT_DATA_DEBUG)
			{
			    System.err.println("BUY_LEV2_t_"+delays[cur_pos]+"\t"+
					       "vol\t"+vol+"\tP\t"+p+"\t"+lev2+"\tPvol\t"+prevvol);
			}
			this.trade_median_f2_buy_tau[cur_pos]=p;
		    }
		    prevvol=vol;
		    if(t.origtime>0 && t.origtime>=tnotafter)
		    {
			// only active orders for now
			this.v_eo_buy_tau[cur_pos]+=t.shares;
		    }

		}
		if(vol!=this.v_alltrades_buy_tau[cur_pos]||
		   n_trades!=this.n_trades_buy_tau[cur_pos])
		{
		    throw new RuntimeException("Broken invariant. This should never happen. currenttime="+currenttime);
		}
		if(n_trades>0)
		{
		    this.cc_minprice_buy[cur_pos]=tb[ptr_start+n_trades-1].price;
		    if(tb[ptr_start].price!=this.maxprice[cur_pos])
		    {
			throw new RuntimeException("Max buy price="+tb[ptr_start].price+
						   " does not match "+this.maxprice[cur_pos]+
						   ". currenttime="+currenttime);
		    }
		}

		// median for sell
		vol=prevvol=0;
		lev=(long)(frac*this.v_alltrades_sell_tau[cur_pos]);
		lev2=(long)(frac2*this.v_alltrades_sell_tau[cur_pos]);
		prev_price=Double.NaN;
		n_trades=0;
		for(int i=tsize-1;i>=ptr_start;i--)
		{
		    final Trade t=tb[i];
		    if(t.buy_sell_indicator!=BookItem.BOOK_SELL)
		    {
			break;
		    }
		    final double p=t.price;
		    if(prev_price>p || !(tnotafter<=t.time && t.time<=tnotbefore)) // will be false for NaN
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    prev_price=p;
		    vol+=t.shares;
		    n_trades++;
		    if(vol<0)
		    {
			throw new RuntimeException("Integer overflow currenttime="+currenttime);
		    }
		    if(vol>lev && lev>=prevvol)
		    {
			if(PRINT_DATA_DEBUG)
			{
			    System.err.println("SELL_LEV_t_"+delays[cur_pos]+"\t"+
					       "vol\t"+vol+"\tP\t"+p+"\t"+lev+"\tPvol\t"+prevvol);
			}
			this.trade_median_sell_tau[cur_pos]=p;
		    }
		    if(vol>lev2 && lev2>=prevvol)
		    {
			if(PRINT_DATA_DEBUG)
			{
			    System.err.println("SELL_LEV2_t_"+delays[cur_pos]+"\t"+
					       "vol\t"+vol+"\tP\t"+p+"\t"+lev2+"\tPvol\t"+prevvol);
			}
			this.trade_median_f2_sell_tau[cur_pos]=p;
		    }
		    prevvol=vol;
		    if(t.origtime>0 && t.origtime>=tnotafter)
		    {
			// only active orders for now
			/*
			System.err.println("\tind="+i+"\t"+"t.origtime\t"+t.origtime+
					   "\t"+tnotafter+"\tvol\t"+vol+
					   " diff\t"+(t.origtime-tnotafter));
			System.err.println("Adding "+
					   String.valueOf(i)+"\t"+
					   t.time+"\t"+
					   t.shares+"\t"+
					   t.buy_sell_indicator+"\t"+
					   t.price+"\t"+
					   t.tradetype+"\t"+
					   t.matchnumber+"\t"+
					   t.orderreferencenumber+"\t");
			*/
			this.v_eo_sell_tau[cur_pos]+=t.shares;
		    }
		}
		if(vol!=this.v_alltrades_sell_tau[cur_pos]||
		   n_trades!=this.n_trades_sell_tau[cur_pos])
		{
		    throw new RuntimeException("Broken invariant. This should never happen. currenttime="+currenttime);
		}
		if(n_trades>0)
		{
		    this.cc_maxprice_sell[cur_pos]=tb[tsize-n_trades].price;
		    if(tb[tsize-1].price!=this.minprice[cur_pos])
		    {
			throw new RuntimeException("Min sell price="+tb[tsize-1].price+
						   " does not match "+this.minprice[cur_pos]+
						   ". currenttime="+currenttime);
		    }
		}
	      }


	      if(true)
	      {
		// order array by price.
		// the buffer is considered poisoned after this.
		com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(tb,ptr_start,tsize-1,CompareItems.cmp_by_price_only);
		final long vtotal=this.v_tot_alltrades[cur_pos];
		if(ptr_start<tsize)
		{
		     this.tot_minprice[cur_pos]=tb[tsize-1].price;
		     this.tot_maxprice[cur_pos]=tb[ptr_start].price;
		}

		final long lev=(long)(frac*vtotal);

		// median from low end
		long vol=0,prevvol=0;
		double prev_price=Double.NaN;
		for(int i=ptr_start;i<tsize;i++)
		{
		    final Trade t=tb[i];
		    final double p=t.price;
		    if(prev_price<p || !(tnotafter<=t.time && t.time<=tnotbefore)) // will be false for NaN
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    prev_price=p;
		    vol+=t.shares;
		    if(vol<0)
		    {
			throw new RuntimeException("Integer overflow currenttime="+currenttime);
		    }
		    if(vol>lev && lev>=prevvol)
		    {
			this.trade_tot_median_low_tau[cur_pos]=p;
		    }
		    prevvol=vol;
		}
		if(vol!=vtotal)
		{
		    throw new RuntimeException("Broken invariant. This should never happen. currenttime="+currenttime);
		}

		// median from high end
		vol=prevvol=0;
		prev_price=Double.NaN;
		for(int i=tsize-1;i>=ptr_start;i--)
		{
		    final Trade t=tb[i];
		    final double p=t.price;
		    if(prev_price>p || !(tnotafter<=t.time && t.time<=tnotbefore)) // will be false for NaN
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    prev_price=p;
		    vol+=t.shares;
		    if(vol<0)
		    {
			throw new RuntimeException("Integer overflow currenttime="+currenttime);
		    }
		    if(vol>lev && lev>=prevvol)
		    {
			this.trade_tot_median_high_tau[cur_pos]=p;
		    }
		    prevvol=vol;
		}
		if(vol!=vtotal)
		{
		    throw new RuntimeException("Broken invariant. This should never happen. currenttime="+currenttime);
		}
	      }

	      if(ATTRIB_ORDERS_PER_TRADE)
	      {
		// order traders by order id.
		// we work with poisoned buffer, but because
		// elements are the same the result should be correct.
		com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(tb,ptr_start,tsize-1,CompareItems.cmp_by_type_and_orderid);
		double opt_buy=0;
		long prev_orderid=Long.MAX_VALUE;
		int n_ord=0,n_trades=0,nto=0,n_trd_check=0;
		long vol=0;
		for(int i=ptr_start;i<tsize;i++)
		{
		    final Trade t=tb[i];
		    if(t.buy_sell_indicator!=BookItem.BOOK_BUY)
		    {
			break;
		    }
		    vol+=t.shares;
		    n_trades++;
		    final long to=t.orderreferencenumber;
		    if(prev_orderid<to || t.time<tnotafter || t.time>tnotbefore)
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    if(to<=0)
		    {
			if(nto>0)
			{
			    opt_buy+=1.0/nto;
			    n_trd_check+=nto;
			}
			nto=1;
			n_ord++;
		    }
		    else if(to!=prev_orderid)
		    {
			if(nto>0)
			{
			    opt_buy+=1.0/nto;
			    n_trd_check+=nto;
			}
			nto=1;
			n_ord++;
		    }
		    else
		    {
			nto++;
		    }
		    prev_orderid=to;
		}
		if(nto>0)
		{
		    opt_buy+=1.0/nto;
		    n_trd_check+=nto;
		    nto=0;
		}
		if(vol!=this.v_alltrades_buy_tau[cur_pos]||
		   n_trades!=this.n_trades_buy_tau[cur_pos]||
		   n_trd_check!=n_trades)
		{
		    throw new RuntimeException(
			"Broken invariant. This should never happen."+
			" vol="+vol+
			" v_alltrades_buy_tau["+cur_pos+"]="+this.v_alltrades_buy_tau[cur_pos]+
			" n_trades="+n_trades+
			" n_trades_buy_tau["+cur_pos+"]="+this.n_trades_buy_tau[cur_pos]+
			" n_trd_check="+n_trd_check+
			" currenttime="+currenttime);
		}
		this.n_orders_buy_tau[cur_pos]=n_ord;
		this.opt_buy_aver[cur_pos]=opt_buy/n_ord;

		double opt_sell=0;
		prev_orderid=-1;
		n_ord=n_trades=nto=n_trd_check=0;
		vol=0;
		for(int i=tsize-1;i>=ptr_start;i--)
		{
		    final Trade t=tb[i];
		    if(t.buy_sell_indicator!=BookItem.BOOK_SELL)
		    {
			break;
		    }
		    vol+=t.shares;
		    n_trades++;
		    final long to=t.orderreferencenumber;
		    if(prev_orderid>to || t.time<tnotafter || t.time>tnotbefore)
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    if(to<=0)
		    {
			if(nto>0)
			{
			    opt_sell+=1.0/nto;
			    n_trd_check+=nto;
			}
			nto=1;
			n_ord++;
		    }
		    else if(to!=prev_orderid)
		    {
			if(nto>0)
			{
			    opt_sell+=1.0/nto;
			    n_trd_check+=nto;
			}
			nto=1;
			n_ord++;
		    }
		    else
		    {
			nto++;
		    }
		    prev_orderid=to;
		}
		if(nto>0)
		{
		    opt_sell+=1.0/nto;
		    n_trd_check+=nto;
		    nto=0;
		}
		if(vol!=this.v_alltrades_sell_tau[cur_pos]||
		   n_trades!=this.n_trades_sell_tau[cur_pos]||
		   n_trd_check!=n_trades)
		{
		    throw new RuntimeException(
			"Broken invariant. This should never happen."+
			" vol="+vol+
			" v_alltrades_sell_tau["+cur_pos+"]="+this.v_alltrades_sell_tau[cur_pos]+
			" n_trades="+n_trades+
			" n_trades_sell_tau["+cur_pos+"]="+this.n_trades_sell_tau[cur_pos]+
			" n_trd_check="+n_trd_check+
			" currenttime="+currenttime);
		}
		this.n_orders_sell_tau[cur_pos]=n_ord;
		this.opt_sell_aver[cur_pos]=opt_sell/n_ord;
		this.opt_aver[cur_pos]=(opt_buy+opt_sell)/(this.n_orders_buy_tau[cur_pos]+this.n_orders_sell_tau[cur_pos]);

	      } // END order traders by order id.

	      if(ATTRIB_TRADES_TIME_TO_EXECUTION)
	      {
		  // order traders by time to execute
		  // we work with poisoned buffer, but because
		  // elements are the same the result should be correct.
		com.polytechnik.outils.TMPQSortAlgorithm.objectQuickSortPointers(tb,ptr_start,tsize-1,CompareItems.cmp_by_type_and_execution);
		// median tte for buy
		long st=0,vol=0,prevvol=0,lev=(long)(frac*this.v_tte_known_buy_tau[cur_pos]);
		long prev_tte=-1;
		for(int i=ptr_start;i<tsize;i++)
		{
		    final Trade t=tb[i];
		    if(t.buy_sell_indicator!=BookItem.BOOK_BUY || t.origtime<=0)
		    {
			break;
		    }
		    final long tte=t.time-t.origtime;
		    if(prev_tte>tte || tte<0 || t.time<tnotafter || t.time>tnotbefore)
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    prev_tte=tte;
		    vol+=t.shares;
		    st+=t.shares*(tte/ModelParameters.T_SUMM_SCALE);
		    if(vol>lev && lev>=prevvol)
		    {
			if(PRINT_DATA_DEBUG)
			{
			    System.err.println("BUY_LEV_t_"+delays[cur_pos]+"\t"+
					       "vol\t"+vol+"\ttte\t"+tte+"\t"+lev+"\tPvol\t"+prevvol);
			}
			this.time_to_execute_buy[cur_pos]=(int)(tte/ModelParameters.DISTANCE_TIME_SCALE);
		    }
		    prevvol=vol;
		}
		if(vol>0)
		{
		    st=(st/vol)*ModelParameters.T_SUMM_SCALE;
		}
		else
		{
		    st=-1;
		}
		if(vol!=this.v_tte_known_buy_tau[cur_pos]||st!=this.time_to_execute_aver_buy[cur_pos])
		{
		    throw new RuntimeException(
			"Broken invariant."+
			" vol="+vol+
			" this.v_tte_known_buy_tau["+cur_pos+"]="+this.v_tte_known_buy_tau[cur_pos]+
			" st="+st+
			" this.time_to_execute_aver_buy["+cur_pos+"]="+this.time_to_execute_aver_buy[cur_pos]+
			" This should never happen. currenttime="+currenttime);
		}

		// median tte for sell
		st=vol=prevvol=0;
		lev=(long)(frac*this.v_tte_known_sell_tau[cur_pos]);
		prev_tte=-1;
		for(int i=tsize-1;i>=ptr_start;i--)
		{
		    final Trade t=tb[i];
		    if(t.buy_sell_indicator!=BookItem.BOOK_SELL || t.origtime<=0)
		    {
			break;
		    }
		    final long tte=t.time-t.origtime;
		    if(prev_tte>tte || tte<0 || t.time<tnotafter || t.time>tnotbefore)
		    {
			throw new RuntimeException("Ordering is broken");
		    }
		    prev_tte=tte;
		    vol+=t.shares;
		    st+=t.shares*(tte/ModelParameters.T_SUMM_SCALE);
		    if(vol>lev && lev>=prevvol)
		    {
			if(PRINT_DATA_DEBUG)
			{
			    System.err.println("SELL_LEV_t_"+delays[cur_pos]+"\t"+
					       "vol\t"+vol+"\ttte\t"+tte+"\t"+lev+"\tPvol\t"+prevvol);
			}
			this.time_to_execute_sell[cur_pos]=(int)(tte/ModelParameters.DISTANCE_TIME_SCALE);
		    }
		    prevvol=vol;
		}
		if(vol>0)
		{
		    st=(st/vol)*ModelParameters.T_SUMM_SCALE;
		}
		else
		{
		    st=-1;
		}
		if(vol!=this.v_tte_known_sell_tau[cur_pos]||st!=this.time_to_execute_aver_sell[cur_pos])
		{
		    throw new RuntimeException(
			"Broken invariant."+
			" vol="+vol+
			" this.v_tte_known_sell_tau["+cur_pos+"]="+this.v_tte_known_sell_tau[cur_pos]+
			" st="+st+
			" this.time_to_execute_aver_sell["+cur_pos+"]="+this.time_to_execute_aver_sell[cur_pos]+
			" This should never happen. currenttime="+currenttime);
		}
	      } // END order traders by time to execution

	    }


	    // check previous.
	    for(int i=0;i<nt;i++)
	    {
		final int i1=i-1;
		final long vb=this.v_alltrades_buy_tau[i]-(i1>=0?this.v_alltrades_buy_tau[i1]:0),
		    vs=this.v_alltrades_sell_tau[i]-(i1>=0?this.v_alltrades_sell_tau[i1]:0),
		    vb_e=this.v_alltrades_buy_tau[i]-this.v_hiddentrades_buy_tau[i]-
		    (i1>=0?this.v_alltrades_buy_tau[i1]-this.v_hiddentrades_buy_tau[i1]:0),
		    vs_e=this.v_alltrades_sell_tau[i]-this.v_hiddentrades_sell_tau[i]-
		    (i1>=0?this.v_alltrades_sell_tau[i1]-this.v_hiddentrades_sell_tau[i1]:0);

		final boolean nan_mbuy=Double.isNaN(this.trade_median_buy_tau[i]),
		    nan_msell=Double.isNaN(this.trade_median_sell_tau[i]);
		if((nan_mbuy!=Double.isNaN(this.trade_median_f2_buy_tau[i]))||
		   (nan_msell!=Double.isNaN(this.trade_median_f2_sell_tau[i]))||
		   (ATTRIB_ORDERS_PER_TRADE &&
		    ((vb>0 && this.n_orders_buy_tau[i]<=0)||
		     (vs>0 && this.n_orders_sell_tau[i]<=0))) ||
		   (ATTRIB_TRADES_TIME_TO_EXECUTION &&
		    ((vb_e>0 && this.time_to_execute_buy[i]<0)||
		     (vs_e>0 && this.time_to_execute_sell[i]<0)))
		   )
		{
		    throw new RuntimeException("Broken constrains for i="+i+
					       " tau_interval_range_ptr="+tau_interval_range_ptr[i]+
					       " n_orders_buy_tau="+this.n_orders_buy_tau[i]+
					       " vb="+vb+
					       " n_orders_sell_tau="+this.n_orders_sell_tau[i]+
					       " vs="+vs+
					       " time_to_execute_buy="+this.time_to_execute_buy[i]+
					       " time_to_execute_sell="+this.time_to_execute_sell[i]+
					       " trade_median_buy_tau="+this.trade_median_buy_tau[i]+
					       " trade_median_sell_tau="+this.trade_median_sell_tau[i]+
					       " trade_median_f2_buy_tau="+this.trade_median_f2_buy_tau[i]+
					       " trade_median_f2_sell_tau="+this.trade_median_f2_sell_tau[i]);
		}
	    }
	    for(int i=0;i<nt;i++)
	    {
		if(Double.isNaN(this.trade_median_buy_tau[i])!=(this.v_alltrades_buy_tau[i]==0)||
		   Double.isNaN(this.trade_median_f2_buy_tau[i])!=(this.v_alltrades_buy_tau[i]==0))
		{
		     throw new RuntimeException(
			 "buy_vol."+delays[i]+"="+this.v_alltrades_buy_tau[i]+
			 " should be 0;"+
			 " this.trade_median_buy_tau["+i+"]="+this.trade_median_buy_tau[i]+
			 " this.trade_median_f2_buy_tau["+i+"]="+this.trade_median_f2_buy_tau[i]+
			 " currenttime="+currenttime);
		}

		if(Double.isNaN(this.trade_median_sell_tau[i])!=(this.v_alltrades_sell_tau[i]==0)||
		   Double.isNaN(this.trade_median_f2_sell_tau[i])!=(this.v_alltrades_sell_tau[i]==0))
		{
		    throw new RuntimeException(
			"sell_vol."+delays[i]+"="+this.v_alltrades_sell_tau[i]+
			" should be 0;"+
			" this.trade_median_sell_tau["+i+"]="+this.trade_median_sell_tau[i]+
			" this.trade_median_f2_sell_tau["+i+"]="+this.trade_median_f2_sell_tau[i]+
			" currenttime="+currenttime);
		}
		if(this.trade_median_buy_tau[i]>this.maxprice[i]||
		   this.trade_median_f2_buy_tau[i]>this.trade_median_buy_tau[i]||
		   this.trade_median_sell_tau[i]<this.minprice[i]||
		   this.trade_median_f2_sell_tau[i]<this.trade_median_sell_tau[i])
		{
		    throw new RuntimeException(
			"Range mismatch:"+
			" this.trade_median_buy_tau["+i+"]="+this.trade_median_buy_tau[i]+
			" this.trade_median_sell_tau["+i+"]="+this.trade_median_sell_tau[i]+
			" this.trade_median_f2_buy_tau["+i+"]="+this.trade_median_f2_buy_tau[i]+
			" this.trade_median_f2_sell_tau["+i+"]="+this.trade_median_f2_sell_tau[i]);
		}
	    }



	    for(int i=0;i<nt;i++)
	    {
		if(this.disttomin[i]<0)
		{
		    this.disttomin[i]=0;
		}
		if(this.disttomax[i]<0)
		{
		    this.disttomax[i]=0;
		}
		if(this.n_orders_buy_tau[i]<0)
		{
		    this.n_orders_buy_tau[i]=0;
		}
		if(this.n_orders_sell_tau[i]<0)
		{
		    this.n_orders_sell_tau[i]=0;
		}

		if(i>0)
		{
		    final int i1=i-1;
		    if(this.n_orders_buy_tau[i]<this.n_orders_buy_tau[i1]||
		       this.n_orders_sell_tau[i]<this.n_orders_sell_tau[i1])
		    {
			throw new RuntimeException(
			"Ordering is broken:"+
			" this.n_orders_buy_tau["+i+"]="+this.n_orders_buy_tau[i]+
			" this.n_orders_buy_tau["+i1+"]="+this.n_orders_buy_tau[i1]+
			" this.n_orders_sell_tau["+i+"]="+this.n_orders_sell_tau[i]+
			" this.n_orders_sell_tau["+i1+"]="+this.n_orders_sell_tau[i1]);
		    }
		}
	    }

	    if(PRINT_DATA_DEBUG)
	    {

		System.err.println("tau\t"+
				   "vol_buy\t"+
				   "disttofrac_all_buy\t"+
				   "vol_sell\t"+
				   "disttofrac_all_sell\t"+
				   "vol\t"+
				   "disttofrac_all\t"+
				   "vol_oe_buy\t"+
				   "vol_oe_sell\t"+

				   "\t"+

				   "vol_hidden_buy\t"+
				   "disttofrac_hidden_buy\t"+
				   "vol_hidden_sell\t"+
				   "disttofrac_hidden_sell\t"+
				   "vol_hidden\t"+
				   "disttofrac_hidden\t"+

				   "\t"+

				   "v_buy_abovecurrent\t"+
				   "v_sell_belowcurrent\t"+

				   "\t"+

				   "minprice_sell\t"+
				   "disttomin\t"+
				   "max sell\t"+
				   "maxprice_buy\t"+
				   "disttomax\t"+
				   "min buy\t"+

				   "\t"+

				   "n_orders_buy\t"+
				   "n_trades_buy\t"+
				   "time_to_execute_buy\t"+

				   "n_orders_sell\t"+
				   "n_trades_sell\t"+
				   "time_to_execute_sell\t"+

				   "\t"+

				   "median_buy_"+frac+"\t"+
				   "median_sell_"+frac+"\t"+

				   "median_f2_buy_"+frac2+"\t"+
				   "median_f2_sell_"+frac2);

		for(int i=0;i<nt;i++)
		{
		    System.err.println(String.valueOf(delays[i])+"\t"+
				       this.v_alltrades_buy_tau[i]+"\t"+
				       this.disttofrac_all_buy[i]+"\t"+
				       this.v_alltrades_sell_tau[i]+"\t"+
				       this.disttofrac_all_sell[i]+"\t"+
				       (this.v_alltrades_buy_tau[i]+this.v_alltrades_sell_tau[i])+"\t"+
				       this.disttofrac_all[i]+"\t"+
				       this.v_eo_buy_tau[i]+"\t"+
				       this.v_eo_sell_tau[i]+"\t"+

				       "\t"+

				       this.v_hiddentrades_buy_tau[i]+"\t"+
				       this.disttofrac_hidden_buy[i]+"\t"+
				       this.v_hiddentrades_sell_tau[i]+"\t"+
				       this.disttofrac_hidden_sell[i]+"\t"+
				       (this.v_hiddentrades_buy_tau[i]+this.v_hiddentrades_sell_tau[i])+"\t"+
				       this.disttofrac_hidden[i]+"\t"+

				       "\t"+

				       this.v_abovecurrent_buy_tau[i]+"\t"+
				       this.v_belowcurrent_sell_tau[i]+"\t"+

				       "\t"+

				       this.minprice[i]+"\t"+
				       this.disttomin[i]+"\t"+
				       this.cc_maxprice_sell[i]+"\t"+
				       this.maxprice[i]+"\t"+
				       this.disttomax[i]+"\t"+
				       this.cc_minprice_buy[i]+"\t"+

				       "\t"+

				       this.n_orders_buy_tau[i]+"\t"+
				       this.n_trades_buy_tau[i]+"\t"+
				       this.time_to_execute_buy[i]+"\t"+

				       this.n_orders_sell_tau[i]+"\t"+
				       this.n_trades_sell_tau[i]+"\t"+
				       this.time_to_execute_sell[i]+"\t"+

				       "\t"+

				       this.trade_median_buy_tau[i]+"\t"+
				       this.trade_median_sell_tau[i]+"\t"+

				       this.trade_median_f2_buy_tau[i]+"\t"+
				       this.trade_median_f2_sell_tau[i]



				       );
		}
	    }
	}

    private static final boolean
	ATTRIB_ORDERS_PER_TRADE=true,
	ATTRIB_TRADES_TIME_TO_EXECUTION=true;
}
