package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.realtimebook.*;

/** Extension of Rurik: Raiser &amp; Vidka.
 *  @version $Id: CiQVM.java,v 1.33 2022/07/28 12:27:15 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class CiQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final EVXData I,I0,DP0,VS0;

CiQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();
    final int nmoms=2*neq-1;
    final double [] imom=new double [nmoms],tmom=new double [nmoms],
	dpmom=new double [nmoms],
	vsmom=new double [nmoms];
    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setM(vsmom,0,access.M_VS,odata,nmoms);
    final double [] Ddpmom=new double [nmoms],Dvsmom=new double [nmoms];
    access.setDFMoments(Ddpmom,dpmom,nmoms);
    access.setDFMoments(Dvsmom,vsmom,nmoms);

    final double x0=getX0();
    final double [][] QQ=access.getQQ_Matrs(neq,1,new double [][]{tmom,imom});
    final double [] QQt=QQ[0],QQv=QQ[1];
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQv,QQt,access,x0);

    final int n0=neq-1;
    final double [][] basis=getBasisFX0(n0,access,x0);

    final double [][] QQ0=getQQInBasisFromQQorig(n0,basis,QQ,neq),
	DQQ0=getQQInBasisFromMomentsorig(n0,basis,new double [][]{Ddpmom,Dvsmom},access);
    final double [] QQ0t=QQ0[0],QQ0v=QQ0[1],DQQ0dp=DQQ0[0],DQQ0vs=DQQ0[1];
    final DeprecatedHelperBWrapperDataDirectAccessWithBasis<DTYPE> access0=new DeprecatedHelperBWrapperDataDirectAccessWithBasis<DTYPE>(access,basis);
    this.I0=DeprecatedHelperEVXData.getEVXDataFromQQ(n0,QQ0v,QQ0t,access0,x0);
    this.DP0=DeprecatedHelperEVXData.getEVXDataFromQQ(n0,DQQ0dp,QQ0t,access0,x0);
    this.VS0=DeprecatedHelperEVXData.getEVXDataFromQQ(n0,DQQ0vs,QQ0t,access0,x0);

    if(true)
    {
	stmom=tmom;
	simom=imom;
	sdpmom=dpmom;
	svsmom=vsmom;
	sbasis=basis;
    }

}
/** For unit tests. */
final double [] stmom,simom,sdpmom,svsmom;
final double [][] sbasis;
abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double getX0();

/** Get basis f_k(x)=Q_{k+1}(x)-Q_{k+1}(x0)
 */
static double [][] getBasisFX0(final int n,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
{
    final double [] Q0=new double []{1};
    final double f0=access.fx(Q0,x0);
    final double [][] basis=new double [n][];
    for(int k=basis.length;--k>=0;)
    {
	final int d=k+1;
	final double [] b=basis[k]=new double [d+1];
	b[d]=1;
	b[0]=-access.fx(b,x0)/f0;
    }
    return basis;
}
/** Calculate QQ matrix in basis, using QQ matrix in original basis.
 */
static double [][] getQQInBasisFromQQorig(final int n,final double [][] basis,final double [][] QQorig,final int ld_QQorig)
{
    final double [][] QQ=new double [QQorig.length][n*n];
    for(int k=0;k<n;k++)
    {
	final double [] basisk=basis[k];
	for(int l=0;l<=k;l++)
	{
	    final double [] basisl=basis[l];
	    for(int j=QQ.length;--j>=0;)
	    {
		QQ[j][l+k*n]=QQ[j][k+l*n]=TMatr.getAMatrB(basisk.length,basisk,0,basisl.length,basisl,0,QQorig[j],ld_QQorig);
	    }
	}
    }
    return QQ;
}


/** Calculate QQ/DQQ matrix in basis, using moments in original basis.
 */
private static double [][] getQQInBasisFromMomentsorig(final int n,final double [][] basis,final double [][] moms,final DeprecatedHelperDataDirectAccessWithBasis<?> access)
{
    // max dimension, assuming original basis is polynomials.
    final int nm=2*getMaxD(n,basis)-1;
    final double [] tmp=new double [nm],summul=new double [nm];
    final double [][] r=new double [moms.length][n*n];
    for(int k=0;k<n;k++)
    {
	final double [] basisk=basis[k];
	for(int l=0;l<=k;l++)
	{
	    final double [] basisl=basis[l];
	    setMult2Pol(summul,basisk,basisk.length,basisl,basisl.length,tmp,access);
	    final int ne=basisk.length+basisl.length-1;
	    for(int j=r.length,kl=n*k+l,lk=n*l+k;--j>=0;)
	    {
		r[j][kl]=r[j][lk]=TMatr.mulVVn(ne,summul,moms[j]);
	    }
	}
    }
    return r;
}


/** Calculate basis(k)QQ_{kl}basis(l).
 *  The result is returned in original basis, not in basis.
 */
static double [][] getKK(final int n,final double [][] basis,final double [][] QQ,final DeprecatedHelperDataDirectAccessWithBasis<?> access)
{
    // max dimension, assuming original basis is polynomials.
    final int nm=2*getMaxD(n,basis)-1;
    final double [][] kk=new double [QQ.length][nm];
    final double [] tmp=new double [nm],summul=new double [nm];
    for(int k=0;k<n;k++)
    {
	final double [] basisk=basis[k];
	for(int l=0;l<n;l++)
	{
	    final double [] basisl=basis[l];
	    setMult2Pol(summul,basisk,basisk.length,basisl,basisl.length,tmp,access);
	    for(int j=kk.length,kl=k*n+l,ne=basisk.length+basisl.length-1;--j>=0;)
	    {
		DeprecatedHelperTMatr.sumP1(kk[j],summul,ne,QQ[j][kl]);
	    }
	}
    }
    return kk;
}


private static void setMult2Pol(final double [] res,final double [] p1,final int n1,final double [] p2,final int n2,final double [] tmp,final DeprecatedHelperDataDirectAccessWithBasis<?> access)
{
    for(int k=n1+n2-1;--k>=0;) res[k]=0;
    for(int k=n1;--k>=0;)
    {
	final double c1=p1[k];
	for(int l=n2;--l>=0;)
	{
	    final double b=c1*p2[l];
	    if(b==0) continue; // optimization
	    final int na=access.setQQCoefs(k,l,tmp);
	    DeprecatedHelperTMatr.sumP1(res,tmp,na,b);
	}
    }
}

private static int getMaxD(final int n,final double [][] basis)
{
    int nm=0;
    for(int k=0;k<n;k++)
    {
	if(nm<=basis[k].length) nm=basis[k].length;
    }
    return nm;
}


static abstract class CiQVMTest<DTYPE> extends QTest {

abstract CiQVM<DTYPE> getCiQVM();
abstract BQVM<DTYPE> getBQVM();
abstract double [][] getMonomialMoments(final DTYPE o);
CiQVMTest(final String name,final java.util.Random r,final double eps)
{
    super(name,r,eps);
}

void test_Common()
{
    final CiQVM<DTYPE> q=getCiQVM();
    System.err.println(name+":CiQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}

double [][] getRandomBasis(final int n)
{
    final double [][] basis=new double [n][];
    for(int k=0;k<n;k++)
    {
	final double [] b=basis[k]=new double [k+1];
	b[k]=4+r.nextDouble();
	for(int l=0;l<k;l++) b[l]=r.nextDouble();
    }
    return basis;
}

void test_Basis()
{
    final CiQVM<DTYPE> q=getCiQVM();
    for(int t=0;t<2;t++)
    {
	final double [][] basis;
	if(t<=0){basis=getRandomBasis(neq);}
	else {
	    if(neq<=1) continue;
	    basis=getBasisFX0(neq-1,q.access,q.getX0());
	    for(int k=0;k<basis.length;k++)
	    {
		UnitTests.checkDoubleRel(q.access.fx(basis[k],q.getX0()),"basis[k](x0)",0,"0",eps);
	    }
	}
	for(double [] v:new double [][]{q.stmom,q.simom,q.sdpmom,q.svsmom})
	{
	    final int n=basis.length;
	    final double [] QQ=q.access.getQQ_Matrs(neq,1,new double [][]{v})[0];
	    final double [] QQb=getQQInBasisFromQQorig(n,basis,new double [][]{QQ},neq)[0];
	    final double [] QQa=getQQInBasisFromMomentsorig(n,basis,new double [][]{v},q.access)[0];
	    UnitTests.checkDoubleArrayRel(QQb,"QQb",QQa,"QQa",eps);
	    if(t<=0)
	    {
		final double [] IQQ=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQ);
		final double [] kk=q.access.getKK(n,new double [][] {IQQ})[0];
		final double [] IQQb=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQb);
		final double [] kkb=getKK(n,basis,new double [][]{IQQb},q.access)[0];
		UnitTests.checkDoubleArrayRel(kk,"kk",kkb,"kkb",eps);
	    }
	}
    }
    System.err.println(name+":CiQVMTest:test_Basis(): test OK neq="+neq+" eps="+eps);
}

abstract double [] getDFCoefsPolynomials(final double [] coefs);
abstract int setDFMomentsPolynomails(final double [] dFmoments,final double [] Fmoments,final int n);
void test_EVXData_RadauPolynomials(final int neq,final boolean flag_cut,final EVXData d,final double x0,final double [] mom_v,final boolean flag_D,final double [] mom_t,final double eps)
{
    final double [] momx;
    if(!flag_D) momx=mom_v;
    else
    {
	momx=new double [mom_v.length];
	setDFMomentsPolynomails(momx,mom_v,mom_v.length);
    }
    final double [] QQMt=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_t,1);
    final double [] o=NQVM.NQVMTest.getOK(neq,QQMt,x0);
    final double [][] rbasis=XQVM.XQVMTest.getRadauBasis(neq,o,x0,QQMt)[0];
    final int n;
    if(flag_cut) n=neq-1;
    else n=neq;
    final DeprecatedHelperDataDirectAccessWithBasis<QVMData> A0access=new DeprecatedHelperQVMDataDirectAccessWithBasis();
    final DeprecatedHelperBWrapperDataDirectAccessWithBasis<QVMData> AAaccess=new DeprecatedHelperBWrapperDataDirectAccessWithBasis<QVMData>(A0access,rbasis);

    final double [] qqt=CiQVM.getQQInBasisFromQQorig(n,rbasis,new double [][]{QQMt},neq)[0],
	qqv=CiQVM.getQQInBasisFromMomentsorig(n,rbasis,new double [][]{momx},A0access)[0];
    if(flag_D)
    {
	final double [] QQd=new double [n*n];
	for(int k=0;k<n;k++)
	{
	    UnitTests.checkDoubleRel(Polynomials.getD0(rbasis[k],x0),"basis[k](x0)",0,"0",eps);
	    for(int l=0;l<n;l++)
	    {
		final double [] dm=getDFCoefsPolynomials(Polynomials.mult2Pol(rbasis[k],rbasis[l]));
		QQd[k*n+l]=TMatr.mulVVn1(dm,mom_v);
	    }
	}
	UnitTests.checkDoubleArrayRel(QQd,"QQd",qqv,"qqv",eps);
    }
    final EVXData ev=DeprecatedHelperEVXData.getEVXDataFromQQ(n,qqv,qqt,AAaccess,x0);
    EVXData.matchEVXData(d,ev,false,!flag_cut,eps);
    System.err.println("CiQVM::CiQVMTest:test_EVX_RadauPolynomials(): test OK neq="+neq+" eps="+eps);
}

void test_Monomials_BasisExpansion()
{
    final CiQVM<DTYPE> q=getCiQVM();
    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] mom_t=amoms[0],mom_v=amoms[1],mom_dp=amoms[2],mom_vs=amoms[3];

    test_EVXData_RadauPolynomials(neq,false,q.I,q.getX0(),mom_v,false,mom_t,eps);
    test_EVXData_RadauPolynomials(neq,true,q.I0,q.getX0(),mom_v,false,mom_t,eps);
    test_EVXData_RadauPolynomials(neq,true,q.DP0,q.getX0(),mom_dp,true,mom_t,eps);
    test_EVXData_RadauPolynomials(neq,true,q.VS0,q.getX0(),mom_vs,true,mom_t,eps);

    System.err.println("CiQVM::CiQVMTest:test_Monomials_BasisExpansion(): test OK neq="+neq+" eps="+eps);
}

void test_BQVM()
{
    final CiQVM<DTYPE> q=getCiQVM();
    final BQVM<DTYPE> s=getBQVM();
    EVXData.matchEVXData(q.I,s.IT,false,eps);
    UnitTests.checkDoubleArrayRel(q.I0.getEValues(),"q.I0.getEValues()",s.IT0R.getEValues(),"s.IT0R.getEValues()",eps);
    System.err.println("q.I0.o_o="+q.I0.o_o+" q.I0.s0="+q.I0.s0+" q.I0.wL="+q.I0.wL+" q.I0.wH="+q.I0.wH);
    System.err.println(name+":CiQVMTest:test_BQVM(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":CiQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Basis();
    test_BQVM();
    test_Monomials_BasisExpansion();

    System.err.println(name+":CiQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":CiQVMTest: starting tests; ver=$Id: CiQVM.java,v 1.33 2022/07/28 12:27:15 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":CiQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: CiQVM.java,v 1.33 2022/07/28 12:27:15 mal Exp $");
}

} // END OF class CiQVMTest
}
