package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** An interface to classification predictor.
 *  @version $Id: ClassificationPredictorC5.java,v 1.85 2022/07/28 21:03:02 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ClassificationPredictorC5 extends ClassificationPredictor{

    private static final int C5_MINPOINTS=5;
    private static final String C5_PATH="/home/polytechnik/rulequest/c5.0";
    private static final int C5_NFRAC=40;

    private static class Condition {
	static final int CNTYPE_NOMINAL=976,CNTYPE_DOUBLE=977;
	final int condition_type;
	private final String name;
	final int ptr_value;
	Condition(final String name,final int condition_type,final int ptr_value)
	{
	    this.name=eliminateMultipleInstances(name);
	    this.condition_type=condition_type;
	    this.ptr_value=ptr_value;
	}
	boolean isMatchedDouble(final double [] val)
	{
	    throw new RuntimeException("Should never be called");
	}

	boolean isMatchedNominal(final int [] val)
	{
	    throw new RuntimeException("Should never be called");
	}


	private static final java.util.HashMap<String,String>
	    stringeliminator=new java.util.HashMap<String,String>(507,0.27f);
	/** similar to one in ITCHParser.
	 */
	private static String eliminateMultipleInstances(final String s)
	{
	    if(s==null)
	    {
		return null;
	    }
	    synchronized(stringeliminator)
	    {
		final String s2=stringeliminator.get(s);
		if(s2!=null)
		{
		    return s2;
		}
		stringeliminator.put(s,s);
	    }
	    return s;
	}


	static Condition readCondition(final String s,
				       final StatusData sd,
				       final String [] string_attribs,
				       final String [] double_attribs,
				       final String [] nominal_attribs)
	{
	    final java.util.Map<String,String> m=
		com.polytechnik.outils.EncodeDecode.parseAttributes(s);
	    final String t=m.get("type");
	    if(t.equals("1"))
	    {
		return NomCondition.readNomCondition(m,
						     s,
						     nominal_attribs,
						     sd.antecedents.n_nominals);
	    }
	    else if(t.equals("2"))
	    {
		return DoubleCondition.readDoubleCondition(m,
							   s,
							   double_attribs,
							   sd.antecedents.n_doubles);
	    }
	    throw new RuntimeException("Invalid condition="+s+" read.");
	}
    }


    static int getAttribsPtr(final String name,final String [] attribs_names)
    {

	final StringBuilder buf=new StringBuilder();
	int jf=-1;
	for(int j=0;j<attribs_names.length;j++)
	{
	    if(FilestemData.escape_fs(attribs_names[j],buf).equals(name))
	    {
		if(jf<0)
		{
		    jf=j;
		}
		else
		{
		    throw new RuntimeException(
			"Multiple attribute "+name+" found in "+
			java.util.Arrays.toString(attribs_names));
		}
	    }
	}
	if(jf<0)
	{
	    throw new RuntimeException("No attribute "+name+" found in "+
				       java.util.Arrays.toString(attribs_names));
	}
	return jf;
    }


    private static class NomCondition extends Condition {
	private final int val;
	private NomCondition(final String name,
			     final int ptr_value,
			     final int val)
	{
	    super(name,CNTYPE_NOMINAL,ptr_value);
	    this.val=val;
	}
	boolean isMatchedNominal(final int [] vals)
	{
	    final int val=vals[this.ptr_value];
	    return (val >=0) && (val==this.val);
	}
	static NomCondition readNomCondition(final java.util.Map<String,String> m,
					     final String s,
					     final String [] attribs_names,
					     final int n_attribs)
	{
	    if(n_attribs!=attribs_names.length)
	    {
		throw new RuntimeException("Invalid number of attributes for condition="+s+
					   " "+n_attribs+" and "+attribs_names.length);
	    }
	    final String t=m.get("type"),att=m.get("att"),val=m.get("val");
	    if((t==null||!t.equals("1"))||att==null||val==null)
	    {
		throw new RuntimeException("Invalid condition="+s+" read.");
	    }
	    final int vval=Integer.parseInt(val);
	    if(vval<0)
	    {
		throw new RuntimeException("Invalid condition="+s+" read; val="+vval+"<0.");
	    }
	    final int ptr=getAttribsPtr(att,attribs_names);
	    return new NomCondition(attribs_names[ptr],ptr,vval);
	  }
    }


      private static class DoubleCondition extends Condition {
	  private static final int COND_LT=101,COND_GT=102;
	  private final float cut;
	  private final int cond;
	  private DoubleCondition(final String name,
				  final int ptr_value,
				  final float cut,
				  final char result)
	  {
	      super(name,CNTYPE_DOUBLE,ptr_value);
	      this.cut=cut;
	      if(result=='<')
	      {
		  this.cond=COND_LT;
	      }
	      else if(result=='>')
	      {
		  this.cond=COND_GT;
	      }
	      else
	      {
		  throw new RuntimeException("Unknown result="+result);
	      }

	  }
	  boolean isMatchedDouble(final double [] vals)
	  {
	      // convert to float to match precisely with C5 code
	      final float val=(float)vals[this.ptr_value];
	      if(Float.isNaN(val)||Float.isInfinite(val))
	      {
		  return false;
	      }
	      if(this.cond==COND_LT)
	      {
		  return val<=this.cut;
	      }
	      else if(this.cond==COND_GT)
	      {
		  return val>this.cut;
	      }
	      return false;
	  }
	  static DoubleCondition readDoubleCondition(final java.util.Map<String,String> m,
						     final String s,
						     final String [] attribs_names,
						     final int n_attribs)
	  {
	      if(n_attribs!=attribs_names.length)
	      {
		  throw new RuntimeException("Invalid number of attributes for condition="+s+
					     " "+n_attribs+" and "+attribs_names.length);
	      }
	      final String t=m.get("type"),att=m.get("att"),cut=m.get("cut"),result=m.get("result");
	      if((t==null||!t.equals("2"))||
		 att==null||
		 cut==null||
		 (result==null||result.length()!=1))
	      {
		  throw new RuntimeException("Invalid condition="+s+" read.");
	      }
	      final int ptr=getAttribsPtr(att,attribs_names);
	      return new DoubleCondition(attribs_names[ptr],
					 ptr,
					 Float.parseFloat(cut),
					 result.charAt(0));
	  }
      }


    private static class Rule {
	final float cover;
	final float ok;
	final float lift;
	final int consequent_value;

	final Condition [] conditions;

	Rule(final float cover,
	     final float ok,
	     final float lift,
	     final int consequent_value,
	     final Condition [] conditions)
	{
	    this.cover=cover;
	    this.ok=ok;
	    this.lift=lift;
	    this.consequent_value=consequent_value;
	    this.conditions=conditions;
	}

	boolean isMatched(final String [] sattribs,
			  final double [] datribs,
			  final int [] nattribs)
	{
	    final Condition [] cx=this.conditions;
	    for(int i=cx.length;i-->0;)
	    {
		final Condition c=cx[i];
		if(c.condition_type==Condition.CNTYPE_DOUBLE)
		{
		    if(!c.isMatchedDouble(datribs))
		    {
			return false;
		    }
		}
		else if(c.condition_type==Condition.CNTYPE_NOMINAL)
		{
		    if(!c.isMatchedNominal(nattribs))
		    {
			return false;
		    }
		}
		else
		{
		    throw new RuntimeException("Unknown condition type="+c.condition_type);
		}
	    }
	    return true;
	}
    }


    private static int getConsequentValue(final String s)
    {
	final int c=Integer.parseInt(s);
	if(c!=0 && c!=1)
	{
	    throw new RuntimeException(
		"Invalid string="+s+
		" currently only 0 and 1 are supported as class variables.");
	}
	return c;
    }



    private static class ReadRules {
	final Rule [][] rules;
	final String id;
	final int [] default_values;
	ReadRules(final Rule [][] rules,
		  final int [] default_values,
		  final String id)
	{
	    this.rules=rules;
	    this.id=id;
	    this.default_values=default_values;
	}
    }


    private static ReadRules readRules(final String fname,
				       final StatusData sd,
				       final String [] string_attribs,
				       final String [] double_attribs,
				       final String [] nominal_attribs) throws java.io.IOException {
	if(PRINT_DEBUG)
	{
	    System.err.print("ClassificationPredictorC5:: reading rules from "+fname);
	}
	final java.io.BufferedReader r=
	    new java.io.BufferedReader(new java.io.FileReader(fname));
	final String id=r.readLine();
	if(!id.startsWith("id=\"See5/C5.0"))
	{
	    throw new RuntimeException("Invalid Data in "+fname+" id="+id);
	}
	final String costs=r.readLine();
	final String entries;
	if(!costs.equals("costs=\"1\""))
	{
	    //throw new RuntimeException("Invalid Data in "+fname+" costs="+costs);
	    // cost is missed
	    entries=costs;
	}
	else
	{
	    entries=r.readLine();
	}
	if(!entries.startsWith("entries=\""))
	{
	    throw new RuntimeException("Invalid Data in "+fname+" entries="+entries);
	}
	final java.util.Map<String,String> me=
	    com.polytechnik.outils.EncodeDecode.parseAttributes(entries);
	final String s_n_entries=me.get("entries");
	if(s_n_entries==null)
	{
	    throw new RuntimeException("Invalid entries number="+entries+" read.");
	}
	final int n_entries=Integer.parseInt(s_n_entries);

	if(n_entries<=0)
	{
	    throw new RuntimeException("Invalid entries="+entries);
	}

	int n_rules_total=0;
	final Rule [][] rentries=new Rule [n_entries][];
	final int [] defaults=new int [n_entries];
	for(int ne=0;ne<n_entries;ne++)
	{
	    final String sconds=r.readLine();
	    final java.util.Map<String,String> mg=
		com.polytechnik.outils.EncodeDecode.parseAttributes(sconds);
	    final String s_n_rules=mg.get("rules"),s_c_default=mg.get("default");
	    if(s_n_rules==null||s_c_default==null)
	    {
		throw new RuntimeException("Invalid condition="+sconds+" read.");
	    }
	    defaults[ne]=getConsequentValue(s_c_default);
	    final int n_rules=Integer.parseInt(s_n_rules);
	    final Rule [] rules=rentries[ne]=new Rule [n_rules];
	    for(int i=0;i<n_rules;i++)
	    {
		final String s=r.readLine();
		final java.util.Map<String,String> m=
		    com.polytechnik.outils.EncodeDecode.parseAttributes(s);
		final String scn=m.get("conds"),
		    scv=m.get("cover"),
		    sok=m.get("ok"),
		    scl=m.get("class"),
		    slf=m.get("lift");
		if(scn==null || scv==null || sok==null || scl==null || slf==null)
		{
		    throw new RuntimeException("Invalid condition="+s+" read.");
		}
		final int n_conds=Integer.parseInt(scn),c=getConsequentValue(scl);
		final float lift=Float.parseFloat(slf),cover=Float.parseFloat(scv),ok=Float.parseFloat(sok);
		final Condition [] conditions=new Condition [n_conds];
		for(int j=0;j<n_conds;j++)
		{
		    conditions[j]=Condition.readCondition(r.readLine(),
							  sd,
							  string_attribs,
							  double_attribs,
							  nominal_attribs);
		}
		rules[i]=new Rule(cover,ok,lift,c,conditions);
	    }
	    if(PRINT_DEBUG)
	    {
		System.err.print("; read "+n_rules+" rules");
	    }
	    n_rules_total+=n_rules;
	}
	r.close();
	if(PRINT_DEBUG)
	{
	    System.err.println("; read "+n_rules_total+" rules total, "+n_entries+" entries OK.");
	}
	return new ReadRules(rentries,defaults,id);
    }


    private static final int N_TRIALS=10;

    static String getCMD(final PrepareFilestems.ConsequentStat stat)
    {
	final int redundancy_level=(int)(stat.tau_look_ahead/stat.tau_discretization);
	final int minpoints=redundancy_level>0?redundancy_level*C5_MINPOINTS:C5_MINPOINTS;
	final int n=stat.n_points_out_of_level/C5_NFRAC;
	return C5_PATH+" -w -t "+N_TRIALS+" -r -m "+(n>minpoints?n:minpoints)+" -f "+stat.filestem;
    }



    private final String filestem;
    private final ReadRules rrules;
    final PrepareFilestems.ConsequentStat stat;
    private final GetRefPrice getrefprice;

    ClassificationPredictorC5(final StatusData sd,
			      final long tau,
			      final String pname,
			      final String [] string_attribs,
			      final String [] double_attribs,
			      final String [] nominal_attribs,
			      final GetRefPrice getrefprice)
    {
	super(tau);
	this.getrefprice=getrefprice;
	// read rules generated by C5
	String fname=null;
	try {
	    this.filestem=sd.mparametersdata.read_knowledgedir+
		com.polytechnik.outils.EncodeDecode.urlencode(sd.ticker)+
		pname+String.valueOf(tau/ModelParameters.DISTANCE_TIME_SCALE);
	    fname=this.filestem+".rules";
	    this.rrules=readRules(fname,sd,string_attribs,double_attribs,nominal_attribs);
	    this.stat=PrepareFilestems.ConsequentStat.readConsequentStat(this.filestem+".consequent_dicr");
	    if(!this.filestem.equals(this.stat.filestem)||
	       this.stat.tau_look_ahead!=tau)
	    {
		throw new RuntimeException("Failestem mismatch: "+this.filestem+
					   " and "+this.stat.filestem);
	    }
	} catch(java.io.IOException e) {
	    e.printStackTrace(System.err);
	    throw new RuntimeException("Failed to read data from "+fname);
	}
    }

    static class PredictionResultsC5 extends PredictionResults {
	final int [] predictions;
	final double [] prediction_confidence;
	PredictionResultsC5(final int result,
			    final double confidence,
			    final String extra_description,
			    final PrepareFilestems.ConsequentStat consequentstat,
			    final long tau,
			    final double pricelevel,
			    final int [] predictions,
			    final double [] prediction_confidence)
	{
	    super(result,confidence,extra_description,consequentstat,tau,pricelevel);
	    this.predictions=predictions;
	    this.prediction_confidence=prediction_confidence;
	}
    }



    public PredictionResultsC5 getPrediction(final String [] sattribs,
					     final double [] datribs,
					     final int [] nattribs,
					     final CalculationData cd)
    {
	float tvotes=0;
	final float [] votes_classsum=new float []{0.0f,0.0f};
	final StringBuilder buf=ClassificationPredictorNewTrade.DEBUGGING_OUT_RULESCALCULATION?new StringBuilder():null;
	final int nrs=this.rrules.rules.length;
	int n_rules_matched_total=0;
	final int [] predicted=new int [nrs];
	final double [] predicted_confidence=new double [nrs];
	for(int en=0;en<nrs;en++)
	{
	    final float [] classsum=new float []{0.0f,0.0f};
	    final Rule [] rules=this.rrules.rules[en];
	    final int default_value=this.rrules.default_values[en];

	    int totalweight=0;
	    int n_rules_matched=0;

	    for(int ir=0;ir<rules.length;ir++)
	    {
		final Rule r=rules[ir];
		if(r.consequent_value<0||r.consequent_value>=classsum.length)
		{
		    throw new RuntimeException();
		}
		if(r.isMatched(sattribs,datribs,nattribs))
		{
		    final int vote=(int)((1000 * (r.ok + 1.0) / (r.cover + 2.0)) + 0.5);
		    classsum[r.consequent_value]+=vote;
		    totalweight+=1000;

		    if(ClassificationPredictorNewTrade.DEBUGGING_OUT_RULESCALCULATION)
		    {
			if(n_rules_matched_total>0)
			{
			    buf.append(" ");
			}
			if(nrs>1)
			{
			    buf.append(en);
			    buf.append("/");
			}
			// C5 rule numbering starts with 1
			buf.append(1+ir);
		    }
		    n_rules_matched++;
		    n_rules_matched_total++;
		}
	    }

	    // normalize weight
	    if (totalweight<=0)
	    {
		classsum[default_value] = 0.5f;
	    }
	    else
	    {
		for(int c=classsum.length; c-->0;)
		{
		    classsum[c] /= totalweight;
		}
	    }


	    // C5 logic
	    int bestclass=default_value;
	    for(int i=0;i<classsum.length;i++)
	    {
		if(classsum[i]>classsum[bestclass])
		{
		    bestclass=i;
		}
	    }
	    final float confidence=classsum[bestclass];
	    predicted[en]=bestclass;
	    predicted_confidence[en]=confidence;

	    if(nrs==1)
	    {
		// single classifier
		return new PredictionResultsC5(bestclass==1?P_YES:P_NO,
					       confidence,
					       ClassificationPredictorNewTrade.DEBUGGING_OUT_RULESCALCULATION?buf.toString():null,
					       this.stat,
					       this.tau,
					       this.getrefprice.getPriceLevel(cd,this.stat.level),
					       predicted,
					       predicted_confidence);
	    }

	    // sum by votes of different classifiers
	    votes_classsum[bestclass]+=confidence;
	    tvotes+=confidence;
	}
	if(nrs<3)
	{
	    throw new RuntimeException("Ross Quinlan said there are should be at least 3 classifiers");
	}
	// normalize weights
	for(int i=0;i<votes_classsum.length;i++)
	{
	    // Probably incorrect in C5 hooks.c
	    // but Ross Quinlan <quinlan@rulequest.com>
	    // claim it is correct for >=3 classifiers
	    // anyway - the worst this can be is incorrect "correctness".
	    votes_classsum[i]/=tvotes;
	}
	// take from the 0-th classifier,
	// Ross Quinlan <quinlan@rulequest.com> wrote in an e-mail on Mar 19, 2009
	int bestclass=this.rrules.default_values[0]; // 0 as the default
	for(int i=0;i<votes_classsum.length;i++)
	{
	    if(votes_classsum[i]>votes_classsum[bestclass])
	    {
		bestclass=i;
	    }
	}
	final float confidence=votes_classsum[bestclass];
	return new PredictionResultsC5(bestclass==1?P_YES:P_NO,
				       confidence,
				       ClassificationPredictorNewTrade.DEBUGGING_OUT_RULESCALCULATION?buf.toString():null,
				       this.stat,
				       this.tau,
				       this.getrefprice.getPriceLevel(cd,this.stat.level),
				       predicted,
				       predicted_confidence);
    }
}
