package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

/** A class to actual position generation.
 *  @version $Id: ClassificationPredictorNewTrade.java,v 1.63 2010/04/14 08:42:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class ClassificationPredictorNewTrade {


    private static final ClassificationPredictor.GetRefPrice
	MINLEVELCALC=new ClassificationPredictor.GetRefPrice()
	    {
		public double getPriceLevel(final CalculationData cd,final double level)
		{
		    return cd.p_buy*level;
		}
	    },
	MAXLEVELCALC=new ClassificationPredictor.GetRefPrice()
	    {
		public double getPriceLevel(final CalculationData cd,final double level)
		{
		    return cd.p_sell*level;
		}
	    };

    private final ClassificationPredictorMultiScale
	MIN_SELL_DOWN,
	MEDIAN_SELL_DOWN,
	MAX_BUY_UP,
	MEDIAN_BUY_UP,
	DTAU_BUY_UP,
	DTAU_SELL_DOWN;


    final StatusData sd;
    ClassificationPredictorNewTrade(final StatusData sd,
				    final String [] string_attribs,
				    final String [] double_attribs,
				    final String [] nominal_attribs)
    {
	this.sd=sd;
	final StringBuilder buf=new StringBuilder(20);
	this.MIN_SELL_DOWN=new ClassificationPredictorMultiScale(
	    "_F_"+FilestemData.escape_fs(CalculateConsequents.PMIN_PREFIX,buf),sd,string_attribs,double_attribs,nominal_attribs,MINLEVELCALC);
	this.MEDIAN_SELL_DOWN=new ClassificationPredictorMultiScale(
	    "_F_"+FilestemData.escape_fs(CalculateConsequents.PMED_SELL_PREFIX,buf),sd,string_attribs,double_attribs,nominal_attribs,MINLEVELCALC);
	this.MAX_BUY_UP=new ClassificationPredictorMultiScale(
	    "_F_"+FilestemData.escape_fs(CalculateConsequents.PMAX_PREFIX,buf),sd,string_attribs,double_attribs,nominal_attribs,MAXLEVELCALC);
	this.MEDIAN_BUY_UP=new ClassificationPredictorMultiScale(
	    "_F_"+FilestemData.escape_fs(CalculateConsequents.PMED_BUY_PREFIX,buf),sd,string_attribs,double_attribs,nominal_attribs,MAXLEVELCALC);
	this.DTAU_BUY_UP=new ClassificationPredictorMultiScale(
	    "_F_"+FilestemData.escape_fs(CalculateConsequents.PTAU_BUY_PREFIX,buf),sd,string_attribs,double_attribs,nominal_attribs,MAXLEVELCALC);
	this.DTAU_SELL_DOWN=new ClassificationPredictorMultiScale(
	    "_F_"+FilestemData.escape_fs(CalculateConsequents.PTAU_SELL_PREFIX,buf),sd,string_attribs,double_attribs,nominal_attribs,MINLEVELCALC);
    }


/** Perform predicton.
 */
    MultiScalePredictionResults makePredictions(final CalculationData cd,
						final CalculateAntecedents attribscalculator)
    {
	attribscalculator.setAntecedentAttribs(
	    this.sd.ticker,
	    this.sd.mparametersdata,
	    cd,
	    this.sd.antecedents);

	final String [] sattribs=attribscalculator.sattribs.getInternalBuffer();
	final double [] datribs=attribscalculator.dattribs.getInternalBuffer();
	final int [] nattribs=attribscalculator.nattribs.getInternalBuffer();

	return new MultiScalePredictionResults(this.MIN_SELL_DOWN.getMultiPredictions(sattribs,datribs,nattribs,cd),
					       this.MEDIAN_SELL_DOWN.getMultiPredictions(sattribs,datribs,nattribs,cd),
					       this.MAX_BUY_UP.getMultiPredictions(sattribs,datribs,nattribs,cd),
					       this.MEDIAN_BUY_UP.getMultiPredictions(sattribs,datribs,nattribs,cd),
					       this.sd,
					       cd);
    }


    static final boolean DEBUGGING_OUT_RULESCALCULATION=false;


    // DEBUGGING GET ALL PREDICTIONS FOR TESTING
    ClassificationPredictor.PredictionResults [][] getPredictionsForTesting(
	final CalculateAntecedents attribscalculator,
	final CalculationData cd)
    {
	if(!DEBUGGING_OUT_RULESCALCULATION)
	{
	    return null;
	}

	final String [] sattribs=attribscalculator.sattribs.getInternalBuffer();
	final double [] datribs=attribscalculator.dattribs.getInternalBuffer();
	final int [] nattribs=attribscalculator.nattribs.getInternalBuffer();

	final ClassificationPredictor.PredictionResults [] op_min=this.MIN_SELL_DOWN.getMultiPredictions(sattribs,datribs,nattribs,cd);
	final ClassificationPredictor.PredictionResults [] med_sell=this.MEDIAN_SELL_DOWN.getMultiPredictions(sattribs,datribs,nattribs,cd);
	final ClassificationPredictor.PredictionResults [] op_max=this.MAX_BUY_UP.getMultiPredictions(sattribs,datribs,nattribs,cd);
	final ClassificationPredictor.PredictionResults [] med_buy=this.MEDIAN_BUY_UP.getMultiPredictions(sattribs,datribs,nattribs,cd);
	final ClassificationPredictor.PredictionResults [] dtau_buy=this.DTAU_BUY_UP.getMultiPredictions(sattribs,datribs,nattribs,cd);
	final ClassificationPredictor.PredictionResults [] dtau_sell=this.DTAU_SELL_DOWN.getMultiPredictions(sattribs,datribs,nattribs,cd);
	final ClassificationPredictor.PredictionResults [][] r=
	    new ClassificationPredictor.PredictionResults [op_min.length][];
	for(int i=0;i<r.length;i++)
	{
	    final ClassificationPredictor.PredictionResults [] p=r[i]=
		new ClassificationPredictor.PredictionResults [PrepareFilestems.N_PREEDICTORS];
	    p[0]=op_min[i];
	    p[1]=med_sell[i];
	    p[2]=op_max[i];
	    p[3]=med_buy[i];
	    p[4]=dtau_buy[i];
	    p[5]=dtau_sell[i];
	}
	return r;
    }

}
