package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;

/** Comparators to compare orders & trades.
 *  @version $Id: CompareItems.java,v 1.8 2009/10/28 14:34:34 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CompareItems {


    static final java.util.Comparator<BookItem>
	cmp_by_time=new java.util.Comparator<BookItem>()
	{
	    public int compare(final BookItem o1,final BookItem o2)
	    {
		return o1.time<o2.time?-1:(o1.time>o2.time?1:(o1.orderreferencenumber<o2.orderreferencenumber?-1:(o1.orderreferencenumber>o2.orderreferencenumber?1:0)));
	    }
	},
	cmp_by_time_latest_at_0=new java.util.Comparator<BookItem>()
	{
	    public int compare(final BookItem o1,final BookItem o2)
	    {
		return o1.time<o2.time?1:(o1.time>o2.time?-1:(o1.orderreferencenumber<o2.orderreferencenumber?1:(o1.orderreferencenumber>o2.orderreferencenumber?-1:0)));
	    }
	};



    static final java.util.Comparator<Trade>
	cmp_by_price_only=new java.util.Comparator<Trade>()
	{
	    public int compare(final Trade o1,final Trade o2)
	    {
		// lowest down
		// use matchnumber to avoid possible instability in ordering
		return o1.price<o2.price?1:(o1.price>o2.price?-1:(o1.matchnumber<o2.matchnumber?-1:o1.matchnumber>o2.matchnumber?1:0));
	    }
	},
	cmp_by_type_and_price=new java.util.Comparator<Trade>()
	{
	    public int compare(final Trade o1,final Trade o2)
	    {
		if(o1.buy_sell_indicator!=o2.buy_sell_indicator)
		{
		    // buy on top, sell on bottom.
		    if(o1.buy_sell_indicator==BookItem.BOOK_BUY)
		    {
			return -1;
		    }
		    if(o2.buy_sell_indicator==BookItem.BOOK_BUY)
		    {
			return 1;
		    }
		    if(o1.buy_sell_indicator==BookItem.BOOK_SELL)
		    {
			return 1;
		    }
		    if(o2.buy_sell_indicator==BookItem.BOOK_SELL)
		    {
			return -1;
		    }
		}
		// lowest down
		return o1.price<o2.price?1:(o1.price>o2.price?-1:0);
	    }
	},
	cmp_by_type_and_orderid=new java.util.Comparator<Trade>()
	{
	    public int compare(final Trade o1,final Trade o2)
	    {
		if(o1.buy_sell_indicator!=o2.buy_sell_indicator)
		{
		    // buy on top, sell on bottom.
		    if(o1.buy_sell_indicator==BookItem.BOOK_BUY)
		    {
			return -1;
		    }
		    if(o2.buy_sell_indicator==BookItem.BOOK_BUY)
		    {
			return 1;
		    }
		    if(o1.buy_sell_indicator==BookItem.BOOK_SELL)
		    {
			return 1;
		    }
		    if(o2.buy_sell_indicator==BookItem.BOOK_SELL)
		    {
			return -1;
		    }
		}
		// sort trades by orderreferencenumber, lowest down
		return o1.orderreferencenumber<o2.orderreferencenumber?1:(o1.orderreferencenumber>o2.orderreferencenumber?-1:0);
	    }
	},
	cmp_by_type_and_execution=new java.util.Comparator<Trade>()
	{
	    public int compare(final Trade o1,final Trade o2)
	    {
		if(o1.buy_sell_indicator!=o2.buy_sell_indicator)
		{
		    // buy on top, sell on bottom.
		    if(o1.buy_sell_indicator==BookItem.BOOK_BUY)
		    {
			return -1;
		    }
		    if(o2.buy_sell_indicator==BookItem.BOOK_BUY)
		    {
			return 1;
		    }
		    if(o1.buy_sell_indicator==BookItem.BOOK_SELL)
		    {
			return 1;
		    }
		    if(o2.buy_sell_indicator==BookItem.BOOK_SELL)
		    {
			return -1;
		    }
		}
		if(o1.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    if(o1.origtime<=0)
		    {
			// null on bottom
			return o2.origtime<=0?0:1;
		    }
		    else if(o2.origtime<=0)
		    {
			// null on bottom
			return o1.origtime<=0?0:-1;
		    }
		    final long tte1=o1.time-o1.origtime,tte2=o2.time-o2.origtime;
		    // lowest up
		    return tte1<tte2?-1:(tte1>tte2?1:0);
		}
		else if(o1.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    if(o1.origtime<=0)
		    {
			// null on top
			return o2.origtime<=0?0:-1;
		    }
		    else if(o2.origtime<=0)
		    {
			// null on bottom
			return o1.origtime<=0?0:1;
		    }
		    final long tte1=o1.time-o1.origtime,tte2=o2.time-o2.origtime;
		    // lowest down
		    return tte1<tte2?1:(tte1>tte2?-1:0);
		}

		// not particulary important
		return 0;
	    }
	},
	cmp_by_type_and_execution_D=new java.util.Comparator<Trade>()
	{
	    public int compare(final Trade o1,final Trade o2)
	    {
		if(o1.origtime<=0)
		{
		    // null on bottom
		    return o2.origtime<=0?BookItem.cmpByTime(o1,o2):1;
		}
		else if(o2.origtime<=0)
		{
		    // null on bottom
		    return o1.origtime<=0?BookItem.cmpByTime(o1,o2):-1;
		}

		if(o1.buy_sell_indicator!=o2.buy_sell_indicator)
		{
		    // buy on top, sell on bottom.
		    if(o1.buy_sell_indicator==BookItem.BOOK_BUY)
		    {
			return -1;
		    }
		    if(o2.buy_sell_indicator==BookItem.BOOK_BUY)
		    {
			return 1;
		    }
		    if(o1.buy_sell_indicator==BookItem.BOOK_SELL)
		    {
			return 1;
		    }
		    if(o2.buy_sell_indicator==BookItem.BOOK_SELL)
		    {
			return -1;
		    }
		}

		final long tte1=o1.time-o1.origtime,tte2=o2.time-o2.origtime;
		if(o1.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    // high tte up
		    return tte1>tte2?-1:(tte1<tte2?1:BookItem.cmpByTime(o1,o2));
		}
		else if(o1.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    // high tte down
		    return tte1>tte2?1:(tte1<tte2?-1:-BookItem.cmpByTime(o1,o2));
		}

		// not particulary important
		return 0;
	    }
	},
	cmp_by_execution_S=new java.util.Comparator<Trade>()
	{
	    public int compare(final Trade o1,final Trade o2)
	    {
		if(o1.origtime<=0)
		{
		    // null on bottom
		    return o2.origtime<=0?BookItem.cmpByTime(o1,o2):1;
		}
		else if(o2.origtime<=0)
		{
		    // null on bottom
		    return o1.origtime<=0?BookItem.cmpByTime(o1,o2):-1;
		}

		final long tte1=o1.time-o1.origtime,tte2=o2.time-o2.origtime;
		// high tte up
		return tte1>tte2?-1:(tte1<tte2?1:BookItem.cmpByTime(o1,o2));
	    }
	};

}
