package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.PolynomialsRatio;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: D2Ratio.java,v 1.5 2020/09/27 20:33:08 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class D2Ratio {
    final double F0,F2,F4;
    D2Ratio(final double [] psi,
	  final double [] QQi,final double [] QQt,
	  final VolMatrix VOLATILITY_EDPsi)
    {
	final int n=psi.length;
	final double [] Dpsi=VOLATILITY_EDPsi.getDPsi(psi),D2psi=VOLATILITY_EDPsi.getDPsi(Dpsi);
	// as a function of squared time shift.
	final double [] i=new double []{
	    TMatr.getAMatrA(psi,QQi,n),
	    -TMatr.getAMatrA(Dpsi,QQi,n)+TMatr.getAMatrB(psi,D2psi,QQi,n),
	    0.25*TMatr.getAMatrA(D2psi,QQi,n)
	},t=new double []{
	    TMatr.getAMatrA(psi,QQt,n),
	    -TMatr.getAMatrA(Dpsi,QQt,n)+TMatr.getAMatrB(psi,D2psi,QQt,n),
	    0.25*TMatr.getAMatrA(D2psi,QQt,n)
	    };
	final PolynomialsRatio R=new PolynomialsRatio(t,i,POLB.B);
	F0=R.getF(0);
	F2=R.getDF(0);
	F4=R.getD2F(0);
    }
    private static final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> POLB=
	new com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis();

}
