package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Skewness;

/**
 *  Find max_psi min_p <(p-p_aver_V)^2 I psi^2>/<psi^2>
 *  @version $Id: DIminP2VmaxI.java,v 1.3 2020/09/27 20:33:08 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DIminP2VmaxI {
    private static final int N_iterations=10;

    public final double p_offset,
	p_init_i,p_init_V,
	p_iter_i,p_iter_V
	;
    public final EVXData I;
public DIminP2VmaxI(final int n,
		    final double p_offset,
		    final double [] QQt,final double [] QQi,final double [] QQpi,final double [] QQp2i,final double [] QQp3i,
		    final double [] QQV,final double [] QQpV,
		    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		    final double eps)
{
    this.p_offset=p_offset;
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int mind=n-1; // maximal eigenvalue index
    this.p_init_i=I.getFPsi(mind,QQpi)/I.getFPsi(mind,QQi);
    this.p_init_V=I.getFPsi(mind,QQpV)/I.getFPsi(mind,QQV);

    double [] psi=I.getPsi(mind);
    double pi=p_init_i,pV=p_init_V;
    final double [] QQpx2I=new double [n*n];
    for(int it=0;it<N_iterations;it++)
    {
	final double px=pV;
	for(int k=0;k<QQpx2I.length;k++) QQpx2I[k]=QQp2i[k]+px*(-2*QQpi[k]+px*QQi[k]);
	EVXData e=EVXData.getEVXDataFromQQ(n,QQpx2I,QQt,M);
	pi=e.getFPsi(mind,QQpi)/e.getFPsi(mind,QQi);
	pV=e.getFPsi(mind,QQpV)/e.getFPsi(mind,QQV);
	psi=e.getPsi(mind);
	if(eps>0)
	{
	    // convergence diag
	    System.err.println("DIminP2VmaxI: it="+it+" px="+px+" e="+e.ev[mind]+" I(e)="+e.getFPsi(mind,QQi));
	}
    }
    this.p_iter_i=pi;
    this.p_iter_V=pV;

    if(eps>0){
	System.err.println("DIminP2VmaxI: test OK n="+n+" eps="+eps);
    }
}
}
