package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Skewness;

/**
 *  Find {@literal max_psi min_p <(p-p_aver)^2 I psi^2>/<psi^2> }.
 *  @version $Id: DIminP2maxI.java,v 1.7 2022/05/31 19:53:07 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DIminP2maxI {
    private static final int N_iterations=10;

    public final double p_offset,
	p_IH,
	p_iter,
	varpvart;
    public final EVXData I;
    public final Skewness sk_IH,sk_iter;
public DIminP2maxI(final int n,
		   final double p_offset,
		   final double [] QQt,final double [] QQi,final double [] QQpi,final double [] QQp2i,final double [] QQp3i,
		   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		   final VolMatrix VOLATILITY_EDPsi,
		   final double eps)
{
    this.p_offset=p_offset;
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int ixH=n-1; // maximal eigenvalue index
    this.p_IH=I.getFPsi(ixH,QQpi)/I.getFPsi(ixH,QQi);

    double p=p_IH;
    double [] psi=I.getPsi(ixH);
    this.sk_IH=Skewness.getSkewnessFromMonomialMoments(
	    TMatr.getAMatrA(psi,QQi,n),
	    TMatr.getAMatrA(psi,QQpi,n),
	    TMatr.getAMatrA(psi,QQp2i,n),
	    TMatr.getAMatrA(psi,QQp3i,n));
    final double [] QQpx2I=new double [n*n];
    for(int it=0;it<N_iterations;it++)
    {
	for(int k=0;k<QQpx2I.length;k++) QQpx2I[k]=QQp2i[k]+p*(-2*QQpi[k]+p*QQi[k]);
	EVXData e=EVXData.getEVXDataFromQQ(n,QQpx2I,QQt,M);
	p=e.getFPsi(ixH,QQpi)/e.getFPsi(ixH,QQi);
	psi=e.getPsi(ixH);
	if(eps>0)
	{
	    // convergence diag
	    System.err.println("DIminP2maxI: it="+it+" p_IH="+p_IH+" p="+p+" e="+e.ev[ixH]+" I="+I.ev[ixH]+" I(e)="+e.getFPsi(ixH,QQi));
	}
    }
    this.p_iter=p;
    final double [] QQpxI=TMatr.sumAcB(QQpi,-p,QQi),Dpsi=VOLATILITY_EDPsi.getDPsi(psi),
	rq=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQpxI,QQt,n);
    varpvart=rq[1];
    if(eps>0){
	 UnitTests.checkDoubleRel(rq[0],"rq[0]",0,"0",eps);
    }
    this.sk_iter=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(psi,QQi,n),
	TMatr.getAMatrA(psi,QQpi,n),
	TMatr.getAMatrA(psi,QQp2i,n),
	TMatr.getAMatrA(psi,QQp3i,n));
    if(eps>0){
	// match the results from DynYp.java: revision 1.10 date: 2016/03/07 13:57:05
	final DynYp d=DynYp.get_DynYp(n,
				      QQi,QQt,QQpi,
				      QQi,QQpi,QQp2i,QQt,
				      true,VOLATILITY_EDPsi,M);
	UnitTests.checkDoubleRel(p_iter,"p_iter",d.p_IJp,"d.p_IJp",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,QQi,n)*varpvart,"I*varpvart",-d.var_pt_IJp,"-d.var_pt_IJp",eps);
	// not an eigenvector of QQpxI
	//UnitTests.checkDoubleRel(rq[2],"rq[2]",-d.var_pt2_IJp,"-d.var_pt2_IJp",eps);
    }
    if(eps>0){
	System.err.println("DIminP2maxI: test OK n="+n+" eps="+eps);
    }
}
}
