package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Skewness;

/**
 *  Find min_psi min_p <(p-p_aver)^2 psi^2>/<Ipsi^2>
 *  @version $Id: DIminP2tmaxI.java,v 1.6 2022/08/08 20:10:45 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DIminP2tmaxI {
    private static final int N_iterations=10;

    public final double p_offset,
	pt_IH,
	p_iter,
	varpvart;
    public final EVXData I;
public final Skewness sk_iter;
public DIminP2tmaxI(final int n,
		    final double p_offset,
		    final double [] QQt,final double [] QQi,
		    final double [] QQp0t,final double [] QQp1t,final double [] QQp2t,final double [] QQp3t,
		    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		    final VolMatrix VOLATILITY_EDPsi,
		    final double eps)
{
    this.p_offset=p_offset;
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int ixH=n-1; // maximal eigenvalue index
    this.pt_IH=I.getFPsi(ixH,QQp1t)/I.getFPsi(ixH,QQp0t);

    final int mind=0;
    double p=pt_IH;
    double [] psi=I.getPsi(ixH);
    final double [] QQpx2t=new double [n*n];
    for(int it=0;it<N_iterations;it++)
    {
	for(int k=0;k<QQpx2t.length;k++) QQpx2t[k]=QQp2t[k]+p*(-2*QQp1t[k]+p*QQp0t[k]);
	EVXData e=EVXData.getEVXDataFromQQ(n,QQpx2t,QQi,M);
	p=e.getFPsi(mind,QQp1t)/e.getFPsi(mind,QQp0t);
	psi=e.getPsi(mind);
	if(eps>0)
	{
	    // convergence diag
	    System.err.println("DIminP2tmaxI: it="+it+" pt_IH="+pt_IH+" p="+p+" e="+e.ev[mind]);
	}
    }
    this.p_iter=p;
    final double [] QQpxt=TMatr.sumAcB(QQp1t,-p,QQp0t),Dpsi=VOLATILITY_EDPsi.getDPsi(psi),
	rq=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQpxt,QQi,n);
    varpvart=rq[1];
    this.sk_iter=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(psi,QQp0t,n),
	TMatr.getAMatrA(psi,QQp1t,n),
	TMatr.getAMatrA(psi,QQp2t,n),
	TMatr.getAMatrA(psi,QQp3t,n));
    if(eps>0){
	final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
	// match the YpkQVM.java.~1.15~ 2016/03/19 10:26:34
	final DynYp d=DynYp.get_DynYp(n,
				      QQi,QQt,TMatr.mulMMM(n,QQp1t,IQQt,QQi),
				      QQp0t,QQp1t,QQp2t,QQi,
				      false,VOLATILITY_EDPsi,M);
	final double Ipsi=TMatr.getAMatrA(psi,QQi,n)/TMatr.getAMatrA(psi,QQt,n);
	UnitTests.checkDoubleRel(p_iter,"p_iter",d.p_IJp,"d.p_IJp",eps);
	UnitTests.checkDoubleRel(Ipsi*varpvart,"Ipsi*varpvart",-d.var_pt_IJp,"-d.var_pt_IJp",eps);
    }
    if(eps>0){
	 UnitTests.checkDoubleRel(rq[0],"rq[0]",0,"0",eps);
    }

    if(eps>0){
	System.err.println("DIminP2tmaxI: test OK n="+n+" eps="+eps);
    }
}
}
