package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** A simplistic +- of dS/dt correlate to the I.
 *  @version $Id: DIsel.java,v 1.34 2022/08/08 20:10:45 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DIsel {
    public final EVXData I;
    public final double p_offset,
    // The https://arxiv.org/abs/1510.05510 answers.
	p_IH,dp_IH,dp_ort_IH,
    // other answers
	reg_corr_Position_I,reg_corr_dp_dI,reg_corr_d2p_I
	;

final SepSkewness pt_ss,dpdt_ss;
/** Try "in psiH state".
 */
public DIsel(final int n,
	     final double p_offset,
	     final double [] QQt,final double [] QQi,final double [] QQpi,final double [] QQpt,final double [] QQdpdt,
	     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	     final VolMatrix VOLATILITY_EDPsi,
	     final double eps)
{
    this.p_offset=p_offset;
    final int ixH=n-1; // maximal eigenvalue index
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    this.p_IH=I.getFPsi(ixH,QQpi)/I.ev[ixH];
    final double [] rq=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(I.psiH,VOLATILITY_EDPsi.getDPsi(I.psiH),null,QQdpdt,QQt,n);dp_IH=rq[0];dp_ort_IH=rq[1];
    final double []
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQdidt=EVXData.convertMatrFromQtoB_Basis(n,new QQdidtMatrix(QQdidtMatrix.BOUNDARY_IeqIH,I,QQi,M,VOLATILITY_EDPsi).QQdidt,I.ev),
	bQQdpdt=EVXData.convertMatrFromQtoB_Basis(n,QQdpdt,I.ev),
	bQQpt=EVXData.convertMatrFromQtoB_Basis(n,QQpt,I.ev),
	bQQposition=TMatr.sumaAbB(p_IH,bQQi,-1,bQQpi);
    final double [] bpsiH=new double [n];bpsiH[ixH]=1;
    if(eps>0)
    {
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsiH,bQQi,n),"<psiH|I|psiH>",I.sH,"I.sH",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsiH,bQQpi,n),"<psiH|pI|psiH>",p_IH*I.sH,"p_IH*I.sH",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsiH,bQQposition,n),"<psiH|bQQposition|psiH>",0,"0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsiH,bQQdidt,n),"<psiH|bQQdidt|psiH>",0,"0",eps);
    }
    final double [] ev_bQQposition=M.EV.getEV(n,bQQposition,true);
    double slam_i=0,slam=0,sw=0;
    for(int j=0;j<n;j++)
    for(int k=0;k<n;k++)
    {
	final double psiI_j_psidS_k=ev_bQQposition[n+n*k+j],psiH_psidS_k=DeprecatedHelperTMatr.mulVVo(n,bpsiH,0,ev_bQQposition,n+n*k),
	    w=Math.pow(psiH_psidS_k,2)*Math.pow(psiI_j_psidS_k,2);
	sw+=w;
	slam+=w*ev_bQQposition[k];
	slam_i+=w*ev_bQQposition[k]*I.ev[j];
    }

    if(eps>0){
	UnitTests.checkDoubleRel(sw,"sw",1,"1",eps);
	UnitTests.checkDoubleRel(slam,"slam",0,"0",eps);
    }
    this.reg_corr_Position_I=slam_i;
    this.reg_corr_dp_dI=TMatr.getAMatrA(bpsiH,TMatr.mulMatrices(n,new double [][]{bQQdpdt,bQQdidt}),n);

    final double [] bQQd2pdt2_dp0=EVXData.convertMatrFromQtoB_Basis(n,TMatr.scaleVectorInPlace(VOLATILITY_EDPsi.getDiffMatrix(n,QQdpdt),-1),I.ev),
	bDpsiH=EVXData.convertPsiFromQtoB_Basis(n,VOLATILITY_EDPsi.getDPsi(I.psiH),I.ev,QQt);
    final double []
	bQQi_bQQd2pdt2_dp0=TMatr.mulMM(n,bQQi,bQQd2pdt2_dp0),
	bQQi_bQQdpdt=TMatr.mulMM(n,bQQi,bQQdpdt);
    final double dSaver_I=-(
	TMatr.getAMatrA(bpsiH,bQQi_bQQd2pdt2_dp0,n)+
	TMatr.getAMatrB(bDpsiH,bpsiH,bQQi_bQQdpdt,n)+
	TMatr.getAMatrB(bpsiH,bDpsiH,bQQi_bQQdpdt,n));
    reg_corr_d2p_I=dSaver_I;
    if(eps>0){
	final double dSaver_Iconst=-(
	    TMatr.getAMatrA(bpsiH,bQQd2pdt2_dp0,n)+
	    TMatr.getAMatrB(bpsiH,bDpsiH,bQQdpdt,n)+
	    TMatr.getAMatrB(bDpsiH,bpsiH,bQQdpdt,n));
	UnitTests.checkDoubleRel(dSaver_Iconst,"dSaver_Iconst",0,"0",eps);
    }

    pt_ss=new SepSkewness(bpsiH,bQQpt,null,M.EV,eps);
    dpdt_ss=new SepSkewness(bpsiH,bQQdpdt,null,M.EV,eps);
    if(eps>0){
	SepSkewness.matchSepSkewness(pt_ss,new SepSkewness(I.psiH,QQpt,QQt,M.EV,eps),eps);
	SepSkewness.matchSepSkewness(dpdt_ss,new SepSkewness(I.psiH,QQdpdt,QQt,M.EV,eps),eps);
    }


    if(eps>0){
	System.err.println("DIsel: test OK reg_corr_Position_I="+reg_corr_Position_I+" n="+n+" eps="+eps);
    }
}


/** Try "after psiH".
 */
public DIsel(
    final int n,
    final double p_offset,
    final double [] QQt,final double [] QQi,final double [] QQpi,final double [] QQpt,final double [] QQdpdt,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
    final VolMatrix VOLATILITY_EDPsi,
    final java.util.function.Function<double [],double []> getPsi2WIntegratedDt,
    final double eps)
{
    this.p_offset=p_offset;
    final int ixH=n-1; // maximal eigenvalue index
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    this.p_IH=I.getFPsi(ixH,QQpi)/I.ev[ixH];
    final double [] psiWeight=getPsi2WIntegratedDt.apply(I.psiH);
    if(psiWeight.length!=2*n-1) throw new IllegalArgumentException();
    final double [] UNIT=TMatr.getUnitMatrix(n),
	// Gram matrix inverse can be obtained without solving a linear system.
	IQQt=EVXData.convertMatrFromBtoQ_Basis(n,UNIT,I.ev,UNIT),
	//IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	bDensityMatrix=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,M.B.getMomentsOfMeasureProducingPolynomialInKK_MQQM(n,psiWeight,IQQt)),I.ev),
	bQQdidt=EVXData.convertMatrFromQtoB_Basis(n,new QQdidtMatrix(QQdidtMatrix.BOUNDARY_IeqIH,I,QQi,M,VOLATILITY_EDPsi).QQdidt,I.ev),
	bQQpt=EVXData.convertMatrFromQtoB_Basis(n,QQpt,I.ev),
	bQQdpdt=EVXData.convertMatrFromQtoB_Basis(n,QQdpdt,I.ev),bQQdpdt_ort=bQQdpdt.clone();
    for(int k=0;k<n;k++) bQQdpdt_ort[k+n*k]=0;
    dp_IH=TMatr.getSpurMM(n,bQQdpdt,bDensityMatrix);
    dp_ort_IH=TMatr.getSpurMM(n,bQQdpdt_ort,bDensityMatrix);

    if(eps>0)
    {
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,bQQdidt,bDensityMatrix),"Spur |bQQdidt|DM|",0,"0",eps);
    }
    reg_corr_Position_I=Double.NaN;
    this.reg_corr_dp_dI=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{bQQdpdt,bQQdidt,bDensityMatrix}));

    final double [] bQQd2pdt2_dp0=EVXData.convertMatrFromQtoB_Basis(n,TMatr.scaleVectorInPlace(VOLATILITY_EDPsi.getDiffMatrix(n,QQdpdt),-1),I.ev),bQQi=TMatr.getDiagMatrix(I.getEValues()),bpsiH=new double [n];
    bpsiH[ixH]=1;
    final double []
	bQQi_bQQd2pdt2_dp0=TMatr.mulMM(n,bQQi,bQQd2pdt2_dp0),
	bQQi_bQQdpdt=TMatr.mulMM(n,bQQi,bQQdpdt);
    final double dSaver_I=-(
	TMatr.getSpurMM(n,bQQi_bQQd2pdt2_dp0,bDensityMatrix)+
	TMatr.getAMatrA(bpsiH,bQQi_bQQdpdt,n));
    reg_corr_d2p_I=dSaver_I;
    if(eps>0){
	final double dSaver_Iconst=-(
	    TMatr.getSpurMM(n,bQQd2pdt2_dp0,bDensityMatrix)+
	    TMatr.getAMatrA(bpsiH,bQQdpdt,n));
	UnitTests.checkDoubleRel(dSaver_Iconst,"dSaver_Iconst",0,"0",eps);
	// does not match, different boundary condition
	//UnitTests.checkDoubleRel(dSaver_I,"dSaver_I",reg_corr_dp_dI,"reg_corr_dp_dI",eps);
    }
    pt_ss=new SepSkewness(I.psiH,QQpt,QQt,M.EV,eps);
    dpdt_ss=new SepSkewness(I.psiH,QQdpdt,QQt,M.EV,eps);
    if(eps>0){
	System.err.println("DIsel: test OK reg_corr_Position_I="+reg_corr_Position_I+" n="+n+" eps="+eps);
    }
}

}
