package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** A simplistic +- of p-<p> and I-<I> correlation.
 *  @version $Id: DIselCorrPI.java,v 1.10 2022/08/07 15:44:01 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DIselCorrPI {
    public final EVXData I;
    public final double p_offset,
    // The https://arxiv.org/abs/1510.05510 answers.
	p_IH,
    // other answers
	w_pp,w_pm,w_mp,w_mm,corr
	;

public DIselCorrPI(final int n,
		   final double p_offset,
		   final double [] QQt,final double [] QQi,final double [] QQpt,
		   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		   final double eps)
{
    this.p_offset=p_offset;
    final int ixH=n-1; // maximal eigenvalue index
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double []
	bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpt=EVXData.convertMatrFromQtoB_Basis(n,QQpt,I.ev);
    final double [] bpsiH=new double [n];bpsiH[ixH]=1;
    this.p_IH=TMatr.getAMatrA(bpsiH,bQQpt,n);
    final double [] ev_bQQpt=M.EV.getEV(n,bQQpt,true),Ip=new double [n],wp=new double [n];
    for(int k=0;k<n;k++){
	wp[k]=Math.pow(DeprecatedHelperTMatr.mulVVo(n,bpsiH,0,ev_bQQpt,n+k*n),2);
	Ip[k]=TMatr.getAMatrA(n,ev_bQQpt,n+k*n,bQQi,n);
    }
    final double Ia=TMatr.mulVVn1(wp,Ip);
    if(eps>0)
    {
	UnitTests.checkDoubleRel(TMatr.sumElements(wp),"sum(wp)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(Ip),"sum(Ip)",TMatr.getSpurM(n,bQQi),"spur(I)",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(java.util.Arrays.copyOf(ev_bQQpt,n)),"sum(ev_bQQpt)",TMatr.getSpurM(n,bQQpt),"spur(p)",eps);
	UnitTests.checkDoubleRel(TMatr.mulVVn1(wp,ev_bQQpt),"sum(wp*lambda_p)",p_IH,"p_IH",eps);
    }
    double corr=0,w_pp=0,w_pm=0,w_mp=0,w_mm=0;
    for(int k=0;k<n;k++){
	final double w=wp[k];
	corr+=w*(Ip[k]-Ia)*(ev_bQQpt[k]-p_IH);
	if(Ip[k]>Ia){
	    if(ev_bQQpt[k]>p_IH) w_pp+=w;
	    else w_pm+=w;
	} else {
	    if(ev_bQQpt[k]>p_IH) w_mp+=w;
	    else w_mm+=w;
	}
    }
    this.corr=corr;
    this.w_pp=w_pp;
    this.w_pm=w_pm;
    this.w_mp=w_mp;
    this.w_mm=w_mm;

    if(eps>0){
	System.err.println("DIselCorrPI: test OK n="+n+" eps="+eps);
    }
}


}
