package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.ProbabilityCorrelation;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** d^2/dt^2 P is back. Consider the |P| operator eigenvalues.
 * The |I| is modified in a way it has the same eigenvalues as the |p| operator.
 * @version $Id: DIselEVp.java,v 1.13 2022/08/05 11:25:31 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class DIselEVp {
    public final EVXData P;
    public final double p_offset,
    // The https://arxiv.org/abs/1510.05510 answers.
	p_IH,var2p_IH,
	p_dIH,var2p_dIH,
	p_dIHn,var2p_dIHn,
	p_dIHV,var2p_dIHV,
	dirpI_plus,dirpV_plus,dirpI_minus,dirpV_minus,
	p_IH_above,p_IH_below,I_IH_above,I_IH_below
	;
    public final ProbabilityCorrelation pc_dIHn;

    /** For debugging and unit tests.
     */
    public final double [] QQi_diagP;

public DIselEVp(final int n,
		final double p_offset,
		final double p_last,
		final double [] QQt,final double [] QQi,
		final double [] QQpt,
		final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		final VolMatrix VOLATILITY_EDPsi,
		final double eps)
{
    this.p_offset=p_offset;
    this.P=EVXData.getEVXDataFromQQ(n,QQpt,QQt,M);
    final double []
	bQQi=EVXData.convertMatrFromQtoB_Basis(n,QQi,P.ev),
	bQQp=TMatr.getDiagMatrix(P.getEValues()),UNIT=TMatr.getUnitMatrix(n),
	bQQi_diagP=new double [n*n];
    for(int k=0;k<n;k++) bQQi_diagP[k+n*k]=bQQi[k+n*k];
    QQi_diagP=EVXData.convertMatrFromBtoQ_Basis(n,bQQi_diagP,P.ev,QQt);
    if(eps>0){
    }

    final ProbabilityCorrelation [] pc=new ProbabilityCorrelation [n];
    final double [] d2I=new double [n],d2Inondiag=new double [n],d2p=new double [n],te=new double [n],ONE=M.B.getONE();
    for(int k=0;k<n;k++){
	final double [] psik=P.getPsi(k),bDpsik=EVXData.convertPsiFromQtoB_Basis(n,VOLATILITY_EDPsi.getDPsi(psik),P.ev,QQt),bpsik=new double [n];
	bDpsik[k]=0; // remove psik projection, now bpsik and bDpsik are orthogonal 2D eigenvalues solution
	bpsik[k]=1;
	final double norm2bDpsik=TMatr.mulVV(bDpsik,bDpsik);
	// changes in p and I, the scale is norm2bDpsik.
	d2I[k]=TMatr.getAMatrA(bDpsik,bQQi_diagP,n)/norm2bDpsik-bQQi_diagP[k+n*k]; // diagonal
	d2p[k]=TMatr.getAMatrA(bDpsik,bQQp,n)/norm2bDpsik-bQQp[k+n*k]; // diagonal
	final double [] QQi2=new double []{
		TMatr.getAMatrB(bpsik,bpsik,bQQi,n),
		TMatr.getAMatrB(bpsik,bDpsik,bQQi,n),
		TMatr.getAMatrB(bDpsik,bpsik,bQQi,n),
		TMatr.getAMatrB(bDpsik,bDpsik,bQQi,n)
	};
	pc[k]=new ProbabilityCorrelation(
	    new double [] {
		TMatr.getAMatrB(bpsik,bpsik,UNIT,n),
		TMatr.getAMatrB(bpsik,bDpsik,UNIT,n),
		TMatr.getAMatrB(bDpsik,bpsik,UNIT,n),
		TMatr.getAMatrB(bDpsik,bDpsik,UNIT,n)
	    },new double []{
		TMatr.getAMatrB(bpsik,bpsik,bQQp,n),
		TMatr.getAMatrB(bpsik,bDpsik,bQQp,n),
		TMatr.getAMatrB(bDpsik,bpsik,bQQp,n),
		TMatr.getAMatrB(bDpsik,bDpsik,bQQp,n)
	    },
	    QQi2);
	d2Inondiag[k]=QQi2[1+2*1]/norm2bDpsik-QQi2[0+2*0];
	// Lebesgue quadrature weight
	te[k]=Math.pow(TMatr.getAMatrB(psik,ONE,QQt,n),2);
	if(eps>0){
	    final double [] Dpsik=VOLATILITY_EDPsi.getDPsi(psik);
	    final double [] rqIk=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psik,Dpsik,null,QQi_diagP,QQt,n),
		rqPk=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psik,Dpsik,null,QQpt,QQt,n);
	    UnitTests.checkDoubleRel(rqPk[0],"rqPk[0]",P.ev[k],"P.ev[k]",eps);
	    UnitTests.checkDoubleRel(rqPk[1],"rqPk[1]",0,"0",eps);
	    UnitTests.checkDoubleRel(rqPk[2],"rqPk[2]",2*d2p[k]*norm2bDpsik,"2*d2p[k]*norm2bDpsik",eps);
	    UnitTests.checkDoubleRel(rqIk[0],"rqIk[0]",bQQi_diagP[k+n*k],"bQQi_diagP[k+n*k]",eps);
	    UnitTests.checkDoubleRel(rqIk[1],"rqIk[1]",0,"0",eps);
	    UnitTests.checkDoubleRel(rqIk[2],"rqIk[2]",2*d2I[k]*norm2bDpsik,"2*d2I[k]*norm2bDpsik",eps);
	    UnitTests.checkDoubleRel(d2p[k],"d2p[k]",Math.signum(d2p[k])*(pc[k].pH-pc[k].pL),"sign*(pc[k].pH-pc[k].pL)",eps);
	    UnitTests.checkDoubleRel(Math.abs(pc[k].corr),"|pc[k].corr|",Math.abs(d2Inondiag[k]/(pc[k].iL-pc[k].iH)),"|d2Inondiag[k]/(pc[k].iL-pc[k].iH|)",eps);
	}
    }
    int ind_maxI=0,ind_maxdIdiag=0,ind_maxdInondiag=0,ind_maxdIV=0;
    for(int k=0;k<n;k++){
	if(bQQi_diagP[ind_maxI+n*ind_maxI]<=bQQi_diagP[k+n*k]) ind_maxI=k;
	if(d2I[ind_maxdIdiag]>=d2I[k]) ind_maxdIdiag=k;
	if(d2Inondiag[ind_maxdInondiag]>=d2Inondiag[k]) ind_maxdInondiag=k;
	if(d2I[ind_maxdIV]*te[ind_maxdIV]>=d2I[k]*te[k]) ind_maxdIV=k;
    }
    this.p_IH=P.ev[ind_maxI];
    this.var2p_IH=d2p[ind_maxI];
    this.p_dIH=P.ev[ind_maxdIdiag];
    this.var2p_dIH=d2p[ind_maxdIdiag];
    this.p_dIHn=P.ev[ind_maxdInondiag];
    this.var2p_dIHn=d2p[ind_maxdInondiag];
    this.pc_dIHn=pc[ind_maxdInondiag];
    this.p_dIHV=P.ev[ind_maxdIV];
    this.var2p_dIHV=d2p[ind_maxdIV];

    double dirpI_plus=0,dirpI_minus=0,dirpV_plus=0,dirpV_minus=0;
    for(int k=0;k<n;k++){
	if(d2I[k]<0){
	    // negative second derivative is a max of I
	    dirpV_plus+=d2I[k]*d2p[k]*te[k];
	    dirpI_plus+=d2I[k]*d2p[k];
	} else {
	    // positive second derivative is a min of I
	    dirpV_minus+=d2I[k]*d2p[k]*te[k];
	    dirpI_minus+=d2I[k]*d2p[k];
	}
    }
    this.dirpI_plus=dirpI_plus;
    this.dirpV_plus=dirpV_plus;
    this.dirpI_minus=dirpI_minus;
    this.dirpV_minus=dirpV_minus;

    // conditional min/max
    final BasisSplit ipm=new BasisSplit(n,P.ev,(pev) -> pev>p_last,QQi,M.EV,eps);
    this.p_IH_above=TMatr.getAMatrA(ipm.sel_plus.psiH,QQpt,n);
    I_IH_above=ipm.sel_plus.I_H;
    this.p_IH_below=TMatr.getAMatrA(ipm.sel_minus.psiH,QQpt,n);
    I_IH_below=ipm.sel_minus.I_H;
    if(eps>0){
	if(ipm.sel_plus.n_elements_in_basis>0) UnitTests.checkDoubleRel(TMatr.getAMatrA(ipm.sel_plus.psiH,QQt,n),"1",1,"1",eps);
	if(ipm.sel_minus.n_elements_in_basis>0) UnitTests.checkDoubleRel(TMatr.getAMatrA(ipm.sel_minus.psiH,QQt,n),"1",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(te),"Sum(te)",QQt[0],"QQt[0]",eps);
    }

    if(eps>0){
	System.err.println("DIselEVp: test OK n="+n+" eps="+eps);
    }
}


}
