package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.ProbabilityCorrelation;
import com.polytechnik.utils.Skewness;

/** Try (p-p_last)I/V -> min/max
 * @version $Id: DIselPIV.java,v 1.4 2020/09/27 20:33:08 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class DIselPIV {
    public final EVXData PIV,IV,IT;
    final double I_L,p_L,I_H,p_H,p_HIV,p_HIT
	;

public DIselPIV(final int n,final double p_last,
	       // The input V & T to be defined as positive (V_{last}-V), change the sign if required
	       final double [] QQT,final double [] QQV,final double [] QQpV,
	       final double [] QQt,final double [] QQi,final double [] QQpi,
	       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	       final VolMatrix VOLATILITY_EDPsi,
	       final double eps)
{
    final double [] QQpnl=TMatr.sumAcB(QQpi,-p_last,QQi);
    this.PIV=EVXData.getEVXDataFromQQ(n,QQpnl,QQV,M);
    I_L=PIV.getFPsi(0,QQi)/PIV.getFPsi(0,QQt);
    p_L=PIV.getFPsi(0,QQpi)/PIV.getFPsi(0,QQi);
    I_H=PIV.getFPsi(n-1,QQi)/PIV.getFPsi(n-1,QQt);
    p_H=PIV.getFPsi(n-1,QQpi)/PIV.getFPsi(n-1,QQi);
    IV=EVXData.getEVXDataFromQQ(n,QQi,QQV,M);
    p_HIV=IV.getFPsi(n-1,QQpi)/IV.getFPsi(n-1,QQi);
    IT=EVXData.getEVXDataFromQQ(n,QQi,QQT,M);
    p_HIT=IT.getFPsi(n-1,QQpi)/IT.getFPsi(n-1,QQi);
}
}
