package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.EVSolver;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Try P*I , P , and P/I skewness in max I state.
 * @version $Id: DIselPdivI.java,v 1.11 2022/08/08 20:10:46 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class DIselPdivI {
    public final EVXData I;
    public final double p_offset,
    // The https://arxiv.org/abs/1510.05510 answers.
	p_IH
	;
public final PlusMinusProjections sk_PmulI,sk_P,sk_PdivI;

public DIselPdivI(final int n,
		       final double p_offset,
		       final double [] QQt,final double [] QQi,
		       final double [] QQpi,
		       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		       final double eps)
{
    this.p_offset=p_offset;
    final int ixH=n-1; // maximal eigenvalue index
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);

    final double [] bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev);
    this.p_IH=bQQpi[ixH+n*ixH]/I.ev[ixH];
    final double [] bpsi_IH=new double [n],bQQpnl=TMatr.sumAcB(bQQpi,-p_IH,bQQi);
    bpsi_IH[ixH]=1;
    this.sk_PmulI=new PlusMinusProjections(n,bQQpnl,bpsi_IH,M.EV,eps);
    final double [] invbQQi=new double [n*n],invSqrtbQQi=new double [n*n];
    for(int k=0;k<n;k++){
	invbQQi[k+n*k]=1/I.ev[k];
	invSqrtbQQi[k+n*k]=1/Math.sqrt(I.ev[k]);
    }
    this.sk_P=new PlusMinusProjections(n,TMatr.mulMMM(n,invSqrtbQQi,bQQpnl,invSqrtbQQi),bpsi_IH,M.EV,eps);
    this.sk_PdivI=new PlusMinusProjections(n,TMatr.mulMMM(n,invbQQi,bQQpnl,invbQQi),bpsi_IH,M.EV,eps);
    if(eps>0){
	UnitTests.checkDoubleRel(sk_PmulI.ev_threshold,"sk_PmulI.ev_threshold",0,"0",eps);
	UnitTests.checkDoubleRel(sk_P.ev_threshold,"sk_P.ev_threshold",0,"0",eps);
	UnitTests.checkDoubleRel(sk_PdivI.ev_threshold,"sk_PdivI.ev_threshold",0,"0",eps);
    }

    if(eps>0){
	System.err.println("DIselPdivI: test OK n="+n+" eps="+eps);
    }
}

    public static class PlusMinusProjections {
	public final double [] ev,w;
	public final double ev_threshold,w_plus,w_minus;
	public PlusMinusProjections(final int n,final double [] bQQf,final double [] psiToProject,final EVSolver EV,final double eps)
	{
	    this.ev=EV.getEV(n,bQQf,true);
	    this.ev_threshold=TMatr.getAMatrA(psiToProject,bQQf,n);
	    this.w=new double [n];
	    double w_plus=0,w_minus=0;
	    for(int k=0;k<n;k++){
		final double psik_psiToProject=DeprecatedHelperTMatr.mulVVo(n,ev,n+k*n,psiToProject,0),
		    wk=w[k]=Math.pow(psik_psiToProject,2),evk=ev[k];
		if(evk>=ev_threshold){
		    w_plus+=wk;
		}
		if(evk<ev_threshold){
		    w_minus+=wk;
		}
	    }
	    this.w_plus=w_plus;
	    this.w_minus=w_minus;
	    if(eps>0){
		UnitTests.checkDoubleRel(w_plus+w_minus,"w_plus+w_minus",1,"1",eps);
		UnitTests.checkDouble(TMatr.mulVVn1(w,ev),"w*ev",ev_threshold,"ev_threshold",eps);
		UnitTests.checkDoubleRel(TMatr.sumElements(w),"sum w",1,"1",eps);
		System.err.println("DIselPdivI.PlusMinusProjections: test OK n="+n+"ev_threshold="+ev_threshold+" eps="+eps);
	    }
	}
    } // END OF class PlusMinusProjections

}
