package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;

/**
 *  @version $Id: DIselPnLSensitivity.java,v 1.14 2022/08/05 11:25:32 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DIselPnLSensitivity {

public final EVXData I;
    public final double p_offset,p_IH,pprev_IH,pprevimin_IH,pimax_IH0,pimin_IH0,pimaxa_IH0,pimina_IH0,
	distance_IH,
    p_IH_dist_plus,p_IL_dist_plus,I_IH_dist_plus,dist_IH_dist_plus,
    p_IH_dist_minus,p_IL_dist_minus,I_IH_dist_minus,dist_IH_dist_minus
    ;

public DIselPnLSensitivity(final int n,
			   final double p_offset,
			   final double [] QQt,final double [] QQi,final double [] QQpi,
			   final double [] QQdistance,
			   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			   final com.polytechnik.utils.VolMatrix VOLATILITY_EDPsi,
			   final double eps)
{
    this.p_offset=p_offset;
    // The https://arxiv.org/abs/1510.05510 answers:
    final int ixH=n-1; // maximal eigenvalue index
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double []
	bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	bQQdistance=EVXData.convertMatrFromQtoB_Basis(n,QQdistance,I.ev),
	ev_bQQdistance=M.EV.getEV(n,bQQdistance,true);
    this.p_IH=bQQpi[ixH+n*ixH]/I.ev[ixH];
    this.distance_IH=bQQdistance[ixH+n*ixH];
    final double [] bIdist=new double [n]; for(int k=0;k<n;k++) bIdist[k]=bQQdistance[k+n*k];
    final int [] ind=com.polytechnik.utils.SortArrays.sortArrayIndexes(bIdist);
    int iprev=-1,iprevimin=-1;
    for(int k=n;--k>=0;){
	if(ind[k]==ixH) {
	    if(k>0) iprev=ind[k-1];
	    for(int j=k;--j>=0;){
		final double [] psi=I.getPsi(ind[j]),Dpsi=VOLATILITY_EDPsi.getDPsi(psi),
		    rq=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQi,QQt,n);
		if(eps>0){
		  UnitTests.checkDoubleRel(rq[0],"rq[0]",I.ev[ind[j]],"I.ev[ind[j]]",eps);
		  UnitTests.checkDoubleRel(rq[1],"rq[1]",0,"0",eps);
		}
		// min
		if(rq[2]>0) {iprevimin=ind[j];break;}
	    }
	    break;
	}
    }
    pprev_IH=iprev>=0?(bQQpi[iprev+n*iprev]/I.ev[iprev]):Double.NaN;
    pprevimin_IH=iprevimin>=0?(bQQpi[iprevimin+n*iprevimin]/I.ev[iprevimin]):Double.NaN;
    int indimax=-1,indimin=-1;
    for(int k=n;--k>=0;){
	final double [] psik=I.getPsi(ind[k]),Dpsik=VOLATILITY_EDPsi.getDPsi(psik),
	    rqk=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psik,Dpsik,null,QQi,QQt,n);
	if(rqk[2]<0){ // max
	    indimax=ind[k];
	    for(int j=k;--j>=0;){
		final double [] psij=I.getPsi(ind[j]),Dpsij=VOLATILITY_EDPsi.getDPsi(psij),
		    rqj=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psij,Dpsij,null,QQi,QQt,n);
		if(rqj[2]>0){ // min
		    indimin=ind[j];break;
		}
	    }
	    break;
	}
    }
    pimax_IH0=indimax>=0?(bQQpi[indimax+n*indimax]/I.ev[indimax]):Double.NaN;
    pimin_IH0=indimin>=0?(bQQpi[indimin+n*indimin]/I.ev[indimin]):Double.NaN;
    int indimaxa=-1,indimina=-1;
    for(int k=n;--k>=0;){
	final int indk=ind[k];
	final double [] psik=I.getPsi(indk),Dpsik=VOLATILITY_EDPsi.getDPsi(psik),
	    rqk=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psik,Dpsik,null,QQi,QQt,n);
	if(rqk[2]<0) { // max
	    indimaxa=indk;
	} else if(indimaxa>=0 && rqk[2]>0){ // min after max
	    indimina=indk;
	    break;
	}
    }
    pimaxa_IH0=indimaxa>=0?(bQQpi[indimaxa+n*indimaxa]/I.ev[indimaxa]):Double.NaN;
    pimina_IH0=indimina>=0?(bQQpi[indimina+n*indimina]/I.ev[indimina]):Double.NaN;
    if(eps>0){
	System.err.println("iprev="+iprev+" iprevimin="+iprevimin+" indimax="+indimax+" indimin="+indimin+" indimaxa="+indimaxa+" indimina="+indimina+" bIdist="+java.util.Arrays.toString(bIdist)+" ind="+java.util.Arrays.toString(ind));
    }

    // conditional min/max
    final BasisSplit ipm=new BasisSplit(n,ev_bQQdistance,(dist) -> dist>distance_IH,bQQi,M.EV,eps);
    if(eps>0){
	if(ipm.sel_plus.n_elements_in_basis>0) {
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(EVXData.convertPsiFromBtoQ_Basis(n,ipm.sel_plus.psiH,I.ev),QQi,n),"ipm.sel_plus.I_H",ipm.sel_plus.I_H,"ipm.sel_plus.I_H",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(EVXData.convertPsiFromBtoQ_Basis(n,ipm.sel_plus.psiL,I.ev),QQi,n),"ipm.sel_plus.I_L",ipm.sel_plus.I_L,"ipm.sel_plus.I_L",eps);
	}
	if(ipm.sel_minus.n_elements_in_basis>0) {
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(EVXData.convertPsiFromBtoQ_Basis(n,ipm.sel_minus.psiH,I.ev),QQi,n),"ipm.sel_minus.I_H",ipm.sel_minus.I_H,"ipm.sel_minus.I_H",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(EVXData.convertPsiFromBtoQ_Basis(n,ipm.sel_minus.psiL,I.ev),QQi,n),"ipm.sel_minus.I_L",ipm.sel_minus.I_L,"ipm.sel_minus.I_L",eps);
	}
    }
    I_IH_dist_plus=ipm.sel_plus.I_H;
    p_IH_dist_plus=TMatr.getAMatrA(ipm.sel_plus.psiH,bQQpi,n)/I_IH_dist_plus;
    p_IL_dist_plus=TMatr.getAMatrA(ipm.sel_plus.psiL,bQQpi,n)/ipm.sel_plus.I_L;
    dist_IH_dist_plus=TMatr.getAMatrA(ipm.sel_plus.psiH,bQQdistance,n);
    I_IH_dist_minus=ipm.sel_minus.I_H;
    p_IH_dist_minus=TMatr.getAMatrA(ipm.sel_minus.psiH,bQQpi,n)/I_IH_dist_minus;
    p_IL_dist_minus=TMatr.getAMatrA(ipm.sel_minus.psiL,bQQpi,n)/ipm.sel_minus.I_L;
    dist_IH_dist_minus=TMatr.getAMatrA(ipm.sel_minus.psiH,bQQdistance,n);

    if(eps>0){
	System.err.println("DIselPnLSensitivity: test OK n="+n+" eps="+eps);
    }

}


}
