package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.ProbabilityCorrelation;
import com.polytechnik.utils.Skewness;

/** Try (again) V/T -> max instead of I=dV/dt-> max.
 * @version $Id: DIselVT.java,v 1.33 2022/08/08 20:10:46 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class DIselVT {
public final double
    p_IH,pi_IH,pt_IH,pV_IH,dpdt_IH,dpdt_IH_orig,T_IH,VT_IH,dPdT_IH_orig,dPdTw_IH_orig,dPdTwI_IH_orig,
    p_IHv,pi_IHv,pt_IHv,pV_IHv,dpdt_IHv,r_IHv,dp_IHv,dp_IHv_orig
	;
public final EVXData I,VT;
public final double [] psi_M;
public final ProbabilityCorrelation pc_IHv,pct_IHv;
public final Skewness sk_IHv;

public DIselVT(final int n,final double p_last,
	       // The input V & T to be defined as positive (V_{last}-V), change the sign if required
	       final double [] QQT,final double [] QQV,final double [] QQpV,final double [] QQdpdt_orig,
	       final double [] QQt,final double [] QQpt,
	       final double [] QQi,final double [] QQpi,final double [] QQp2i,final double [] QQp3i,
	       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	       final VolMatrix VOLATILITY_EDPsi,
	       final java.util.function.Function<double [],double []> getPsi2WIntegratedDt,
	       final double eps)
{
    // calculate QQp from QQpi:
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] invSqrtbQQi=new double [n*n],QQ0=TMatr.getQQaa(M.B.getX0Moments(n,I.x0));
    for(int k=0;k<n;k++){invSqrtbQQi[k+n*k]=1/Math.sqrt(I.ev[k]);}
    this.QQp=EVXData.convertMatrFromBtoQ_Basis(n,TMatr.mulMMM(n,invSqrtbQQi,EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),invSqrtbQQi),I.ev,QQt);
    this.QQdpdt=TMatr.sumaAbB(p_last,QQ0,-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQp));
    final int ixH=n-1; // maximal eigenvalue index
    if(eps>0){
	// match QQdpdt
	final double [] QQdpdt_fromQQpt=TMatr.sumaAbB(p_last,QQ0,-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQpt));
	UnitTests.checkDoubleArrayRel(QQdpdt_fromQQpt,"QQdpdt_fromQQpt",QQdpdt_orig,"QQdpdt_orig",eps);
    }

    pi_IH=I.getFPsi(ixH,QQpi)/I.getFPsi(ixH,QQi);
    pt_IH=I.getFPsi(ixH,QQpt)/I.getFPsi(ixH,QQt);
    p_IH=I.getFPsi(ixH,QQp)/I.getFPsi(ixH,QQt);
    pV_IH=I.getFPsi(ixH,QQpV)/I.getFPsi(ixH,QQV);
    dpdt_IH=I.getFPsi(ixH,this.QQdpdt);
    dpdt_IH_orig=I.getFPsi(ixH,QQdpdt_orig);
    T_IH=I.getFPsi(ixH,QQT);
    VT_IH=I.getFPsi(ixH,QQV)/T_IH;
    dPdT_IH_orig=(p_last-pt_IH)/T_IH;

    // V/T eigenvalues solution
    this.VT=EVXData.getEVXDataFromQQ(n,QQV,QQT,M);
    pi_IHv=VT.getFPsi(ixH,QQpi)/VT.getFPsi(ixH,QQi);
    pt_IHv=VT.getFPsi(ixH,QQpt)/VT.getFPsi(ixH,QQt);
    r_IHv=VT.getFPsi(ixH,QQt);
    p_IHv=VT.getFPsi(ixH,QQp)/r_IHv;
    pV_IHv=VT.getFPsi(ixH,QQpV)/VT.getFPsi(ixH,QQV);
    dp_IHv=VT.getFPsi(ixH,this.QQdpdt);
    dpdt_IHv=dp_IHv/r_IHv;
    dp_IHv_orig=VT.getFPsi(ixH,QQdpdt_orig);

    this.psi_M=VT.getPsi(ixH);

    // some tries. not much relevant code.
    final ProbabilityCorrelation [] pc=new ProbabilityCorrelation[n],pct=new ProbabilityCorrelation[n];
    final Skewness [] sk=new Skewness [n];
    for(int k=0;k<n;k++){
	final double [] psi=VT.getPsi(k),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	final double var2VT=2*(TMatr.getAMatrA(Dpsi,QQV,n)-VT.ev[k]*TMatr.getAMatrA(Dpsi,QQT,n)),
	    norm2psi=TMatr.getAMatrA(psi,QQt,n),Tk=TMatr.getAMatrA(psi,QQT,n)/norm2psi;

	// (p,I) probability correlation, use QQp matrix for price.
	pc[k]=new ProbabilityCorrelation(
	    new double [] {
		TMatr.getAMatrB(psi,psi,QQt,n),
		TMatr.getAMatrB(psi,Dpsi,QQt,n),
		TMatr.getAMatrB(Dpsi,psi,QQt,n),
		TMatr.getAMatrB(Dpsi,Dpsi,QQt,n)
	    },new double []{
		TMatr.getAMatrB(psi,psi,QQp,n),
		TMatr.getAMatrB(psi,Dpsi,QQp,n),
		TMatr.getAMatrB(Dpsi,psi,QQp,n),
		TMatr.getAMatrB(Dpsi,Dpsi,QQp,n)
	    },
	    new double []{
		TMatr.getAMatrB(psi,psi,QQi,n),
		TMatr.getAMatrB(psi,Dpsi,QQi,n),
		TMatr.getAMatrB(Dpsi,psi,QQi,n),
		TMatr.getAMatrB(Dpsi,Dpsi,QQi,n)
	    });
	// (p,I) probability correlation, use QQpt matrix for price.
	pct[k]=new ProbabilityCorrelation(
	    new double [] {
		TMatr.getAMatrB(psi,psi,QQt,n),
		TMatr.getAMatrB(psi,Dpsi,QQt,n),
		TMatr.getAMatrB(Dpsi,psi,QQt,n),
		TMatr.getAMatrB(Dpsi,Dpsi,QQt,n)
	    },new double []{
		TMatr.getAMatrB(psi,psi,QQpt,n),
		TMatr.getAMatrB(psi,Dpsi,QQpt,n),
		TMatr.getAMatrB(Dpsi,psi,QQpt,n),
		TMatr.getAMatrB(Dpsi,Dpsi,QQpt,n)
	    },
	    new double []{
		TMatr.getAMatrB(psi,psi,QQi,n),
		TMatr.getAMatrB(psi,Dpsi,QQi,n),
		TMatr.getAMatrB(Dpsi,psi,QQi,n),
		TMatr.getAMatrB(Dpsi,Dpsi,QQi,n)
	    });
	// price skewness
	sk[k]= Skewness.getSkewnessFromMonomialMoments(
	    TMatr.getAMatrA(psi,QQi,n),
	    TMatr.getAMatrA(psi,QQpi,n),
	    TMatr.getAMatrA(psi,QQp2i,n),
	    TMatr.getAMatrA(psi,QQp3i,n));
	// Unit Test
	if(eps>0){
	    // Check the dV/dt and V/T are the same in the states of V/T operator:
	    UnitTests.checkDoubleRel(VT.ev[k],"VT.ev[k]",TMatr.getAMatrA(psi,QQV,n)/TMatr.getAMatrA(psi,QQT,n),"V/T",eps);
	    UnitTests.checkDoubleRel(VT.ev[k],"VT.ev[k]",TMatr.getAMatrA(psi,QQi,n)/TMatr.getAMatrA(psi,QQt,n),"dV/dt",eps);
	    final double []
		rqV=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQV,QQT,n),
		rqI=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQi,QQt,n);
	    // dV/dt and V/T are the same
	    UnitTests.checkDoubleRel(rqV[0],"rqV[0]",rqI[0],"rqI[0]",eps);
	    // first variation is zero in an eigenvalue state
	    UnitTests.checkDoubleRel(rqV[1],"rqV[1]",0,"0",eps);
	    // match second variation to simple expression
	    UnitTests.checkDoubleRel(rqV[2],"rqV[2]",var2VT,"var2VT",eps);
	    System.err.println("k="+k+" V/T=dV/dt="+rqI[0]+" Tk="+Tk+" rqV[2]="+rqV[2]+" rqI[1]="+rqI[1]+" norm2psi="+norm2psi);
	    UnitTests.checkDoubleRel(2*TMatr.getAMatrB(psi,Dpsi,QQT,n),"2<psi|T|Dpsi>",norm2psi,"norm2psi",eps);
	    // !!! Second variation of V/T is equal to first variation of dV/dt multiplied by norm2psi !!!
	    UnitTests.checkDoubleRel(rqV[2],"rqV[2]",rqI[1]*norm2psi,"rqI[1]*norm2psi",eps);
	    final double var2vt=TMatr.getAMatrA(Dpsi,QQV,n)-VT.ev[k]*TMatr.getAMatrA(Dpsi,QQT,n),
		var1I=TMatr.getAMatrB(psi,Dpsi,QQi,n)-VT.ev[k]*TMatr.getAMatrB(psi,Dpsi,QQt,n);
	    UnitTests.checkDoubleRel(var2vt,"var2vt",var1I,"var1I",eps);
	    // We have QQi & QQt in arguments solely for numerical stability,
	    // we can calculate them from QQV & QQT by differentiation.
	    // Check the relation as unit test:
	    UnitTests.checkDoubleArrayRel1(QQt,"QQt",VOLATILITY_EDPsi.getDiffMatrix(n,QQT),"D(QQT)",eps);
	    UnitTests.checkDoubleArrayRel1(QQi,"QQi",VOLATILITY_EDPsi.getDiffMatrix(n,QQV),"D(QQV)",eps);
	}
    }
    // Save probability correlation and skewness in the state of maximal V/T
    pc_IHv=pc[ixH];
    pct_IHv=pct[ixH];
    sk_IHv=sk[ixH];

    final double [] psiH=VT.psiH,psiWeight=getPsi2WIntegratedDt.apply(psiH);
    if(psiWeight.length!=2*n-1) throw new IllegalArgumentException();
    final double [] UNIT=TMatr.getUnitMatrix(n),
	// Gram matrix inverse can be obtained without solving a linear system.
	IQQt=EVXData.convertMatrFromBtoQ_Basis(n,UNIT,I.ev,UNIT),
	//IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	DensityMatrixVTH=M.B.getQQMatr(n,M.B.getMomentsOfMeasureProducingPolynomialInKK_MQQM(n,psiWeight,IQQt)),
	DensityMatrixIH=M.B.getQQMatr(n,M.B.getMomentsOfMeasureProducingPolynomialInKK_MQQM(n,getPsi2WIntegratedDt.apply(I.psiH),IQQt));
    if(eps>0){
	final double th1=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,DensityMatrixVTH,IQQt,QQt})),
	    vh=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,DensityMatrixVTH,IQQt,QQi})),
	    dph_orig=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,DensityMatrixVTH,IQQt,QQdpdt_orig}))/TMatr.getAMatrA(VT.psiH,QQt,n),
	    thI1=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,DensityMatrixIH,IQQt,QQt})),
	    dphI_orig=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,DensityMatrixIH,IQQt,QQdpdt_orig})),
	    dph=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,DensityMatrixVTH,IQQt,this.QQdpdt}))/TMatr.getAMatrA(VT.psiH,QQt,n);
	UnitTests.checkDoubleRel(th1,"th1",1,"1",eps);
	UnitTests.checkDoubleRel(vh,"vh",VT.sH,"VT.sH",eps);
	UnitTests.checkDoubleRel(dph_orig,"dph_orig",p_last-pt_IHv,"p_last-pt_IHv",eps);
	UnitTests.checkDoubleRel(dphI_orig,"dphI_orig",p_last-pt_IH,"p_last-pt_IH",eps);
	UnitTests.checkDoubleRel(thI1,"thI1",T_IH,"T_IH",eps);
	// possible boundary condition issues
	if(false) UnitTests.checkDoubleRel(dph,"dph",p_last-p_IHv,"p_last-p_IHv",eps);
    }
    dPdTw_IH_orig=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,DensityMatrixIH,IQQt,QQdpdt_orig,IQQt,QQt}))/TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,DensityMatrixIH,IQQt,QQt}));
    dPdTwI_IH_orig=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,DensityMatrixIH,IQQt,QQdpdt_orig,IQQt,QQi}))/TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,DensityMatrixIH,IQQt,QQi}));
    if(eps>0){
	UnitTests.checkDoubleRel(dPdTw_IH_orig,"dPdTw_IH_orig",dPdT_IH_orig,"dPdT_IH_orig",eps);
	System.err.println("DIselVT: test OK n="+n+" eps="+eps);
    }
}
    /** For unit tests.
     */
    final double [] QQp,QQdpdt;

public static void matchDIselVTsimple(final DIselVT a,final DIselVTsimple b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    UnitTests.checkDoubleRel(a.dpdt_IH,"a.dpdt_IH",b.dpdt_IH,"b.dpdt_IH",eps);
    UnitTests.checkDoubleRel(a.dpdt_IHv,"a.dpdt_IHv",b.dpdt_IHv,"b.dpdt_IHv",eps);
    UnitTests.checkDoubleRel(a.dp_IHv,"a.dp_IHv",b.dp_IHv,"b.dp_IHv",eps);
    UnitTests.checkDoubleRel(a.r_IHv,"a.r_IHv",b.r_IHv,"b.r_IHv",eps);
    UnitTests.checkDoubleRel(a.p_IH-a.pV_IH,"a.p_IH-a.pV_IH",a.p_IH-a.pV_IH,"a.p_IH-a.pV_IH",eps);
    UnitTests.checkDoubleRel(a.p_IHv-a.pV_IHv,"a.p_IHv-a.pV_IHv",a.p_IHv-a.pV_IHv,"a.p_IHv-a.pV_IHv",eps);

    System.err.println("DIstatesVT.matchDIselVTsimple test OK n="+a.VT.neq+" eps="+eps);
}

}
