package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Try (again) V/T -> max instead of previusly studied I=dV/dt-> max.
 * @version $Id: DIselVTsimple.java,v 1.7 2022/08/08 20:10:46 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class DIselVTsimple {
public final double
    p_IH,pV_IH,dpdt_IH,
    p_IHv,pV_IHv,dpdt_IHv,r_IHv,dp_IHv
	;
public final EVXData I;
public final double [] QQp,QQdpdt,psi_M;

public DIselVTsimple(final int n,final double p_last,
		     // The input V & T to be defined as positive (V_{last}-V), change the sign if required
		     final double [] QQT,final double [] QQV,final double [] QQpV,
		     final double [] QQt,final double [] QQi,final double [] QQpi,
		     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		     final VolMatrix VOLATILITY_EDPsi,
		     final double eps)
{
    // calculate QQp from QQpi:
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] invSqrtbQQi=new double [n*n];
    for(int k=0;k<n;k++){invSqrtbQQi[k+n*k]=1/Math.sqrt(I.ev[k]);}
    this.QQp=EVXData.convertMatrFromBtoQ_Basis(n,TMatr.mulMMM(n,invSqrtbQQi,EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),invSqrtbQQi),I.ev,QQt);
    this.QQdpdt=TMatr.sumaAbB(p_last,TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQp));
    final int ixH=n-1; // maximal eigenvalue index
    p_IH=I.getFPsi(ixH,QQp);
    pV_IH=I.getFPsi(ixH,QQpV)/I.getFPsi(ixH,QQV);
    dpdt_IH=I.getFPsi(ixH,QQdpdt);
    if(eps>0){
	// We have QQt, QQi & QQpi in arguments solely for numerical stability,
	// we can calculate them from QQV & QQT by differentiation.
	// Check the relation as a unit test:
	for(double [][] m:new double [][][]{
		{QQt,QQT},
		{QQi,QQV},
		{QQpi,QQpV}
	    }){
	    UnitTests.checkDoubleArrayRel1(m[0],"QQF",VOLATILITY_EDPsi.getDiffMatrix(n,m[1]),"D(QQF)",eps);
	}
    }

    final double [] bQQV=EVXData.convertMatrFromQtoB_Basis(n,QQV,I.ev),
	bQQT=EVXData.convertMatrFromQtoB_Basis(n,QQT,I.ev),
	bbasis=M.EV.getGEV_Generalized(n,bQQV,bQQT,true),
	bpsi=EVXData.getPsiFromBasis(ixH,bbasis,n);
    psi_M=EVXData.convertPsiFromBtoQ_Basis(n,bpsi,I.ev);
    r_IHv=TMatr.getAMatrA(psi_M,QQt,n);
    p_IHv=TMatr.getAMatrA(psi_M,QQp,n)/r_IHv;
    dp_IHv=TMatr.getAMatrA(psi_M,QQdpdt,n);
    dpdt_IHv=dp_IHv/r_IHv;
    pV_IHv=TMatr.getAMatrA(psi_M,QQpV,n)/TMatr.getAMatrA(psi_M,QQV,n);

    if(eps>0){
	System.err.println("DIselVTsimple: test OK n="+n+" eps="+eps);
    }
}

}
