package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.ProbabilityCorrelation;
import com.polytechnik.utils.Skewness;

/** Try (again) V/T -> max instead of I=dV/dt-> max.
 * @version $Id: DIstatesVT.java,v 1.17 2020/09/27 20:33:09 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class DIstatesVT {
public final double
    pV_IHv,pi_IHv,pT_IHv,pt_IHv,dp_IHv,r_IHv,pvar_IHv,var1pi,var1pv
	;
public final EVXData VT;
public final double [] psi_M;

public DIstatesVT(final int n,final double p_last,final boolean flag_max,
		  final double [] QQt,final double [] QQT,final double [] QQdpdt,
		  final double [] QQV,final double [] QQpV,final double [] QQp2V,final double [] QQp3V,final double [] QQp0T,final double [] QQp1T,
		  final double [] QQi,final double [] QQpi,final double [] QQp2i,final double [] QQp3i,final double [] QQp0t,final double [] QQp1t,
		  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		  final VolMatrix VOLATILITY_EDPsi,
		  final double eps)
{

    this.VT=EVXData.getEVXDataFromQQ(n,QQV,QQT,M);
    final int ixH=flag_max?n-1:0;
    pi_IHv=VT.getFPsi(ixH,QQpi)/VT.getFPsi(ixH,QQi);
    pV_IHv=VT.getFPsi(ixH,QQpV)/VT.getFPsi(ixH,QQV);
    r_IHv=VT.getFPsi(ixH,QQt);
    dp_IHv=VT.getFPsi(ixH,QQdpdt);
    pt_IHv=VT.getFPsi(ixH,QQp1t)/VT.getFPsi(ixH,QQp0t);
    pT_IHv=VT.getFPsi(ixH,QQp1T)/VT.getFPsi(ixH,QQp0T);
    final double [] psi=VT.getPsi(ixH),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
    final double var2VT=2*(TMatr.getAMatrA(Dpsi,QQV,n)-VT.ev[ixH]*TMatr.getAMatrA(Dpsi,QQT,n)),
	norm2psi=TMatr.getAMatrA(psi,QQt,n),Tk=TMatr.getAMatrA(psi,QQT,n)/norm2psi;
    this.psi_M=TMatr.getVectorScaled(psi,1/Math.sqrt(norm2psi));
    final double []
	    rqpi=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQpi,QQi,n),
	rqpv=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQpV,QQV,n);
    var1pi=rqpi[1];
    var1pv=rqpv[1];
    final double []
	rqpit=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQpi,QQt,n),
	rqit=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQi,QQt,n);
    pvar_IHv=rqpit[1]/rqit[1];

    if(eps>0){
	final double fact=VT.getFPsi(ixH,QQi)/VT.getFPsi(ixH,QQV);
	UnitTests.checkDoubleRel(var1pv,"var1pv",fact*(pi_IHv-pV_IHv),"fact*(pi_IHv-pV_IHv)",eps);
	final double [] QQDi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi),QQDpi=VOLATILITY_EDPsi.getDiffMatrix(n,QQpi);
	final double factdi=VT.getFPsi(ixH,QQDi)/VT.getFPsi(ixH,QQi),pdi=VT.getFPsi(ixH,QQDpi)/VT.getFPsi(ixH,QQDi);
	UnitTests.checkDoubleRel(var1pi,"var1pi",factdi*(pdi-pi_IHv),"factdi*(pdi-pi_IHv)",eps);

	final double []
	    rqV=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQV,QQT,n),
	    rqI=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQi,QQt,n);
	// dV/dt and V/T are the same
	UnitTests.checkDoubleRel(rqV[0],"rqV[0]",rqI[0],"rqI[0]",eps);
	// first variation is zero in an eigenvalue state
	UnitTests.checkDoubleRel(rqV[1],"rqV[1]",0,"0",eps);
	// match second variation to simple expression
	UnitTests.checkDoubleRel(rqV[2],"rqV[2]",var2VT,"var2VT",eps);
	System.err.println("V/T=dV/dt="+rqI[0]+" Tk="+Tk+" rqV[2]="+rqV[2]+" rqI[1]="+rqI[1]+" norm2psi="+norm2psi);
	UnitTests.checkDoubleRel(2*TMatr.getAMatrB(psi,Dpsi,QQT,n),"2<psi|T|Dpsi>",norm2psi,"norm2psi",eps);
	// !!! Second variation of V/T is equal to first variation of dV/dt multiplied by norm2psi !!!
	UnitTests.checkDoubleRel(rqV[2],"rqV[2]",rqI[1]*norm2psi,"rqI[1]*norm2psi",eps);
	final double var2vt=TMatr.getAMatrA(Dpsi,QQV,n)-VT.ev[ixH]*TMatr.getAMatrA(Dpsi,QQT,n),
	    var1I=TMatr.getAMatrB(psi,Dpsi,QQi,n)-VT.ev[ixH]*TMatr.getAMatrB(psi,Dpsi,QQt,n);
	UnitTests.checkDoubleRel(var2vt,"var2vt",var1I,"var1I",eps);
	UnitTests.checkDoubleRel(rqit[1]*0.5,"rqit[1]*0.5",var1I/norm2psi,"var1I/norm2psi",eps);
	// We have QQi & QQt in arguments solely for numerical stability,
	// we can calculate them from QQV & QQT by differentiation.
	// Check the relation as a unit test:
	for(double [][] m:new double [][][]{
		{QQt,QQT},
		{QQi,QQV},
		{QQpi,QQpV},
		{QQp2i,QQp2V},
		{QQp3i,QQp3V}
	    }){
	    UnitTests.checkDoubleArrayRel1(m[0],"QQF",VOLATILITY_EDPsi.getDiffMatrix(n,m[1]),"D(QQF)",eps);
	}
    }

    if(eps>0){
	System.err.println("DIstatesVT: test OK n="+n+" eps="+eps);
    }
}

public static void matchDIselVT(final DIstatesVT a,final DIselVT b,final double eps)
{
    EVXData.matchEVXData(a.VT,b.VT,false,eps);
    UnitTests.checkDoubleRel(a.dp_IHv,"a.dp_IHv",b.dp_IHv_orig,"b.dp_IHv_orig",eps);
    UnitTests.checkDoubleRel(a.r_IHv,"a.r_IHv",b.r_IHv,"b.r_IHv",eps);

    System.err.println("DIstatesVT.mmatchDIselVT test OK n="+a.VT.neq+" eps="+eps);
}
}
