package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;
import com.polytechnik.realtimebook.*;

/** Old results. Th calss is typically used for unit test.
 *  @version $Id: DMI.java,v 1.12 2022/07/15 19:57:09 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DMI {
    public final Skewness SK_P_movaver,SK_P_IH;
    public final EVXData I;
    public final double p_offset,p_IH,pt_IH,dpdt_orig_IH,dp_wv_IH,dp_wt_IH,sk_dp_wv_IH,DpV_IH,DpT_IH,
	I_dPIL,p_dPIL,dPI_dPIL,I_dPIH,p_dPIH,dPI_dPIH;
    DMI(final IMomentsData mm,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	this.p_offset=mm.p_offset;
	// Exponential moving average.
	this.SK_P_movaver=Skewness.getSkewnessFromMonomialMoments(mm.QQi[0]/mm.QQt[0],mm.QQpi[0]/mm.QQt[0],mm.QQp2i[0]/mm.QQt[0],mm.QQp3i[0]/mm.QQt[0]);

	final int n=mm.n,ixL=0,ixH=n-1; // The index of max I, the eigenvalues are sorted.
	// Our "old" solution p_IH: price corresponding the state of max I of past sample.
	this.I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
	// The SK_P_IH.xa is equal to p_IH=<psi|pI|psi>/<psi|I|psi>.
	this.SK_P_IH=Skewness.getSkewnessFromMonomialMoments(
	    I.ev[ixH],
	    I.getFPsi(ixH,mm.QQpi),
	    I.getFPsi(ixH,mm.QQp2i),
	    I.getFPsi(ixH,mm.QQp3i));
	this.p_IH=I.getFPsi(ixH,mm.QQpi)/I.ev[ixH]; // same as SK_P_IH.xa
	// Same as p_IH, but averaged with dt, not dv: <psi|p|psi>/<psi|psi>.
	this.pt_IH=I.getFPsi(ixH,mm.QQpt_sample)/I.getFPsi(ixH,mm.QQt_sample);
	// <psi|dp/dt|psi>
	this.dpdt_orig_IH=I.getFPsi(ixH,mm.QQdpdt);
	this.DpV_IH=mm.QQDpV==null?Double.NaN:I.getFPsi(ixH,mm.QQDpV)/I.getFPsi(ixH,mm.QQV);
	this.DpT_IH=mm.QQDpT==null?Double.NaN:I.getFPsi(ixH,mm.QQDpT)/I.getFPsi(ixH,mm.QQT_sample);

	// New type of answer:
	this.dp_wt_IH=I.getFPsi(ixH,mm.QQDpwt)/I.getFPsi(ixH,mm.QQt_sample);
	this.dp_wv_IH=I.getFPsi(ixH,mm.QQDpwv)/I.ev[ixH];
	final EVXData dPI=EVXData.getEVXDataFromQQ(n,mm.QQDpwv,mm.QQt,M);
	final double dpI_IH=I.getFPsi(ixH,mm.QQDpwv);
	this.sk_dp_wv_IH=(2*dpI_IH-dPI.sL-dPI.sH)/(dPI.sL-dPI.sH);

	I_dPIL=dPI.getFPsi(ixL,mm.QQi);
	p_dPIL=dPI.getFPsi(ixL,mm.QQpi)/I_dPIL;
	dPI_dPIL=dPI.ev[ixL];
	I_dPIH=dPI.getFPsi(ixH,mm.QQi);
	p_dPIH=dPI.getFPsi(ixH,mm.QQpi)/I_dPIH;
	dPI_dPIH=dPI.ev[ixH];
    }

static void matchMuDynMicroImpact(final DMI q,final MuDynMicroImpact s,final double eps)
{
    EVXData.matchEVXData(q.I,s.I,false,eps);
    UnitTests.checkDoubleRel(q.p_IH,"q.p_IH",s.p_IH,"s.p_IH",eps);
    UnitTests.checkDoubleRel(q.pt_IH,"q.pt_IH",s.pt_IH,"s.pt_IH",eps);
    UnitTests.checkDoubleRel(q.SK_P_IH.xa,"q.SK_P_IH.xa",s.p_IH,"s.p_IH",eps);
    UnitTests.checkDoubleRel(q.dpdt_orig_IH,"q.dpdt_orig_IH",s.dpdt_orig_IH,"s.dpdt_orig_IH",eps);
    UnitTests.checkDoubleRel(q.dp_wv_IH,"q.dp_wv_IH",s.dp_wv_IH,"s.dp_wv_IH",eps);
    UnitTests.checkDoubleRel(q.dp_wt_IH,"q.dp_wt_IH",s.dp_wt_IH,"s.dp_wt_IH",eps);
    //UnitTests.checkDoubleRel(q.sk_dp_wv_IH,"q.sk_dp_wv_IH",s.sk_dp_wv_IH,"s.sk_dp_wv_IH",eps);

    System.err.println("DMI.matchMuDynMicroImpact test OK n="+q.I.neq+" eps="+eps);
}

static void matchDMI(final DMI a,final DMI b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    Skewness.matchSkewness(a.SK_P_movaver.getSameSkewnessXShifted(a.p_offset),b.SK_P_movaver.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SK_P_IH.getSameSkewnessXShifted(a.p_offset),b.SK_P_IH.getSameSkewnessXShifted(b.p_offset),eps);
    final double offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+offset_shift,"b.p_IH+offset_shift",eps);
    UnitTests.checkDoubleRel(a.pt_IH,"a.pt_IH",b.pt_IH+offset_shift,"b.pt_IH+offset_shift",eps);
    UnitTests.checkDoubleRel(a.dpdt_orig_IH,"a.dpdt_orig_IH",b.dpdt_orig_IH,"b.dpdt_orig_IH",eps);
    UnitTests.checkDoubleRel(a.dp_wv_IH,"a.dp_wv_IH",b.dp_wv_IH,"b.dp_wv_IH",eps);
    UnitTests.checkDoubleRel(a.dp_wt_IH,"a.dp_wt_IH",b.dp_wt_IH,"b.dp_wt_IH",eps);
    UnitTests.checkDoubleRel(a.sk_dp_wv_IH,"a.sk_dp_wv_IH",b.sk_dp_wv_IH,"b.sk_dp_wv_IH",eps);
    UnitTests.checkDoubleRel(a.DpV_IH,"a.DpV_IH",b.DpV_IH,"b.DpV_IH",eps);
    UnitTests.checkDoubleRel(a.DpT_IH,"a.DpT_IH",b.DpT_IH,"b.DpT_IH",eps);

    UnitTests.checkDoubleRel(a.I_dPIH,"a.I_dPIH",b.I_dPIH,"b.I_dPIH",eps);
    UnitTests.checkDoubleRel(a.p_dPIH,"a.p_dPIH",b.p_dPIH+offset_shift,"b.p_dPIH+offset_shift",eps);
    UnitTests.checkDoubleRel(a.dPI_dPIH,"a.dPI_dPIH",b.dPI_dPIH,"b.dPI_dPIH",eps);
    UnitTests.checkDoubleRel(a.I_dPIL,"a.I_dPIL",b.I_dPIL,"b.I_dPIL",eps);
    UnitTests.checkDoubleRel(a.p_dPIL,"a.p_dPIL",b.p_dPIL+offset_shift,"b.p_dPIL+offset_shift",eps);
    UnitTests.checkDoubleRel(a.dPI_dPIL,"a.dPI_dPIL",b.dPI_dPIL,"b.dPI_dPIL",eps);

    System.err.println("DMI.matchDMI test OK n="+a.I.neq+" eps="+eps);
}

static void matchQkPnLpi_PBasis(final DMI q,final QkPnLpi_PBasis s,final double eps)
{
    EVXData.matchEVXData(q.I,s.pnldiskewness_nodI.I,false,eps);
    Skewness.matchSkewness(q.SK_P_movaver.getSameSkewnessXShifted(q.p_offset),s.pnldiskewness_nodI.movaver_S.getSameSkewnessXShifted(s.pnldiskewness_nodI.p_offset),eps);
    Skewness.matchSkewness(q.SK_P_IH.getSameSkewnessXShifted(q.p_offset),s.pnldiskewness_nodI.IHf_S.getSameSkewnessXShifted(s.pnldiskewness_nodI.p_offset),eps);
    UnitTests.checkDoubleRel(q.dpdt_orig_IH,"q.dpdt_orig_IH",s.ist.dp_orig_IH,"s.ist.dp_orig_IH",eps);
    final double p_offset_shift=s.ist.p_offset-q.p_offset;
    UnitTests.checkDoubleRel(q.p_IH,"q.p_IH",s.ist.p_IH+p_offset_shift,"s.ist.p_IH+p_offset_shift",eps);

    UnitTests.checkDoubleRel(q.pt_IH,"q.pt_IH",s.pnlvarmupi.pt_IH+p_offset_shift,"s.pnlvarmupi.pt_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(q.p_IH,"q.p_IH",s.pnlvarmupi.p_IH+p_offset_shift,"s.pnlvarmupi.p_IH+p_offset_shift",eps);

    System.err.println("DMI.matchQkPnLpi_PBasis test OK n="+q.I.neq+" eps="+eps);
}

static void matchIAImpact(final DMI a,final IAImpact b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double offset_shift=b.mu2ia.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+offset_shift,"b.p_IH+offset_shift",eps);

    UnitTests.checkDoubleRel(a.DpV_IH,"a.DpV_IH",b.FDpwv_IH/b.Fv_IH,"b.FDpwv_IH/b.Fv_IH",eps);
    UnitTests.checkDoubleRel(a.DpT_IH,"a.DpT_IH",b.FDpwt_IH/b.Ft_IH,"b.FDpwt_IH/b.Ft_IH",eps);
    System.err.println("DMI.matchIAImpact test OK n="+a.I.neq+" eps="+eps);
}
} // END OF public class DMI
