package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.LebesgueQuadratureWithEVData;

/** Try dI/I.
 *  @version $Id: DM_DIRel.java,v 1.10 2022/08/07 15:44:02 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DM_DIRel {
public final EVXData I,DI;
    public final double p_offset,
    // The https://arxiv.org/abs/1510.05510 answers.
    p_IH
    ;
DM_DIRel(final int n,final double p_offset,final double p_last,
      final double [] QQt,
      final double [] QQi,final double [] QQpi,
      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
      final VolMatrix VOLATILITY_EDPsi,
      final java.util.function.Function<double [],double []> getPsi2WIntegratedDt,
      final double eps)
{
    this.p_offset=p_offset;
    // The https://arxiv.org/abs/1510.05510 answers:
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    this.p_IH=TMatr.getAMatrA(I.psiH,QQpi,n)/I.sH;

    // New answers
    final double I_0=I.s0;
    final double [] QQ0=TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),
	QQdidt0=TMatr.sumaAbB(I_0,QQ0,-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQi));
    // the eigenvalues are the dI/dt/I rates
    this.DI=EVXData.getEVXDataFromQQ(n,QQdidt0,QQi,M);

    if(eps>0)
    {
	final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,QQdidt0,IQQt),"Spur(dI/dt)",0,"0",eps);
	System.err.println("DM_DIRel.matchDM_DIRel test OK n="+n+" 1/tau rates="+java.util.Arrays.toString(DI.getEValues())+" sum(rates)="+TMatr.sumElements(DI.getEValues())+" eps="+eps);
    }
}
}
