package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Density matrix for the state: "after max spike in I",
 *  where the I has an "impact from the future" contribution.
 *  @see DM_DI
 *  @see BasisTransformPiTiPrediction
 *  @version $Id: DM_DIf.java,v 1.28 2022/08/08 20:10:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DM_DIf {
public final EVXData I;
    public final double p_offset,
    // The https://arxiv.org/abs/1510.05510 answers.
    p_IH,pt_IH,dpdt_IH,T_IH,
    // new answers
	dp_spurDMIH, // =p_last-pt_IH
	dp_wIf,dp_wI
    ;
public DM_DIf(final int n,final double p_offset,final double p_last,
	      final double [] QQt,final double [] QQpt,
	      final double [] QQi,final double [] QQpi,
	      final double [] QQdpdt,
	      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	      final java.util.function.Function<double [],double []> getPsi2WIntegratedDt,
	      final double eps)
{
    this.p_offset=p_offset;
    // The https://arxiv.org/abs/1510.05510 answers:
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    this.p_IH=TMatr.getAMatrA(I.psiH,QQpi,n)/I.sH;
    this.pt_IH=TMatr.getAMatrA(I.psiH,QQpt,n);
    this.dpdt_IH=TMatr.getAMatrA(I.psiH,QQdpdt,n);

    final double I_0=I.s0,I_0_feq=I.sH,dI=I_0_feq-I_0,dV=dI/I.o_o;
    final double [] psiWeight=getPsi2WIntegratedDt.apply(I.psiH);
    if(psiWeight.length!=2*n-1) throw new IllegalArgumentException();
    final double [] UNIT=TMatr.getUnitMatrix(n),
	// Gram matrix inverse can be obtained without solving a linear system.
	IQQt=EVXData.convertMatrFromBtoQ_Basis(n,UNIT,I.ev,UNIT),
	//IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	bDensityMatrix=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,M.B.getMomentsOfMeasureProducingPolynomialInKK_MQQM(n,psiWeight,IQQt)),I.ev),
	bQQ0=TMatr.getQQaa(I.psik_at_x0),
	bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQif=TMatr.sumaAbB(dV,bQQ0,1,bQQi),
	bQQdpdt=EVXData.convertMatrFromQtoB_Basis(n,QQdpdt,I.ev);
    // Spur calculation.
    // To match old calculations (time and volume of psiH state).
    this.T_IH=TMatr.getSpurM(n,bDensityMatrix);
    // new calculations
    this.dp_spurDMIH=TMatr.getSpurMM(n,bQQdpdt,bDensityMatrix);
    this.dp_wIf=TMatr.getSpurM(n,TMatr.mulMMM(n,bQQdpdt,bQQif,bDensityMatrix))/
	TMatr.getSpurM(n,TMatr.mulMMM(n,bQQ0,bQQif,bDensityMatrix));
    this.dp_wI=TMatr.getSpurM(n,TMatr.mulMMM(n,bQQdpdt,bQQi,bDensityMatrix))/
	TMatr.getSpurM(n,TMatr.mulMMM(n,bQQ0,bQQi,bDensityMatrix));
    if(eps>0)
    {
    	UnitTests.checkDoubleRel(dp_spurDMIH,"dp_spurDMIH",p_last-pt_IH,"p_last-pt_IH",eps);
	// normalizing test
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,bQQ0,bDensityMatrix),"1",1,"1",eps);
	// impact from the future test
	final double [] bpsi0=EVXData.convertPsiFromQtoB_Basis(n,I.psi0,I.ev,QQt),
	    bpsiH=EVXData.convertPsiFromQtoB_Basis(n,I.psiH,I.ev,QQt);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQif,n),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);
	final double IH0f=I.sH+dI*Math.pow(I.wH,2);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsiH,bQQif,n),"<psiH|If|psiH>",IH0f,"IH0f",eps);

	System.err.println("DM_DIf: test OK n="+n+" eps="+eps);
    }
}


}
