package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 *  @version $Id: DM_HT.java,v 1.33 2022/08/07 15:44:02 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DM_HT {
    public final EVXData I,V;
    public final double p_offset,p_IH,pt_IH,dpdt_IH,T_IH,V_IH,p_VH;
    final double I_IL_BeforeIH,p_IL_BeforeIH,T_IL_BeforeIH,V_IL_BeforeIH;
    DM_HT(final int n,final double p_offset,
	  final double [] QQt,final double [] QQpt,
	  final double [] QQi,final double [] QQpi,
	  final double [] QQdpdt,final double [] QQT,final double [] QQV,
	  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	  final java.util.function.Function<double [],double []> getPsi2WIntegratedDt,
	  final double eps)
    {
	this.p_offset=p_offset;
	this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	this.p_IH=TMatr.getAMatrA(I.psiH,QQpi,n)/I.sH;
	this.pt_IH=TMatr.getAMatrA(I.psiH,QQpt,n);
	this.dpdt_IH=TMatr.getAMatrA(I.psiH,QQdpdt,n);
	this.T_IH=TMatr.getAMatrA(I.psiH,QQT,n);
	this.V_IH=TMatr.getAMatrA(I.psiH,QQV,n);
	if(eps>0)
	{
	    final double [] IpsiH=I.psiH,IpsiWeight=getPsi2WIntegratedDt.apply(IpsiH);
	    if(IpsiWeight.length!=2*n-1) throw new IllegalArgumentException();
	    final double [] UNIT=TMatr.getUnitMatrix(n),
		// Gram matrix inverse can be obtained without solving a linear system.
		IQQt=EVXData.convertMatrFromBtoQ_Basis(n,UNIT,I.ev,UNIT),
		//IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
		DensityMatrix=M.B.getQQMatr(n,M.B.getMomentsOfMeasureProducingPolynomialInKK_MQQM(n,IpsiWeight,IQQt));
	    UnitTests.checkDoubleRel(T_IH,"<I.psiH|QQT|I.psiH>",TMatr.getSpurMM(n,IQQt,DensityMatrix),"spur(rho)",eps);
	    UnitTests.checkDoubleRel(V_IH,"<I.psiH|QQV|I.psiH>",TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{QQi,IQQt,DensityMatrix,IQQt})),"spur(I*rho)",eps);
	}
	this.V=EVXData.getEVXDataFromQQ(n,QQV,QQt,M);
	if(eps>0)
	{
	    // check the I=dV/dt <-> V relation
	    for(int k=0;k<n;k++){
		final double [] psi=V.getPsi(k);
		UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,QQi,n),"<psi|I|psi>",V.ev[k]*Math.pow(M.B.getD0(psi,I.x0),2),"V.ev[k]*psik^2(x0)",eps);
	    }
	}
	this.p_VH=TMatr.getAMatrA(V.psiH,QQpi,n)/TMatr.getAMatrA(V.psiH,QQi,n);
	if(eps>0)
	{
	    final double [] psiV_average=V.getPsiAveraged(QQt,M);
	    final double [] ONE=M.B.getONE();
	    UnitTests.checkDoubleRel(TMatr.mulVV(psiV_average,psiV_average),"sum(psiV_average*psiV_average)",TMatr.getAMatrA(ONE,QQt,n),"QQt[0]",eps);
	    System.err.println("DM_HT: selftest OK T_0="+TMatr.getAMatrA(I.psi0,QQT,n)+" V_IH="+V_IH+" Vspectrum="+java.util.Arrays.toString(V.getEValues())+" n="+n+" eps="+eps);
	}
	// select the eigenvectors "before I.psiH"
	int ind_start=0;
	for(int k=n;--k>=0;){
	    if(V.ev[k]<=V_IH) break;
	    ind_start=k;
	}
	final int nd=n-ind_start;
	//System.err.println("nd="+nd);

	final boolean flag_remove_psiH_projection=true;
	// Create restricted basis: 0..nd-1 is V.ev[ind_start:n-1].
	// Also we may remove I.psiH projection to obtain orthogonal to I.psiH answer.
	// copy selected V.ev basis functions
	final double [] basis_selected=java.util.Arrays.copyOfRange(V.ev,n+(n-nd)*n,n+n*n);
	for(int ok=0;flag_remove_psiH_projection && ok<nd;ok++)
	{
	    // make with I.psiH projection removed.
	    final double psikpsiIH=V.getFPsiPsi(ok+(n-nd),I.psiH,QQt);
	    for(int l=0;l<n;l++) basis_selected[ok*n+l]-=psikpsiIH*I.psiH[l];
	}
	// Convert I to the basis_selected
	final double [] bQQiBeforeIH=EVXData.convertMatrFromQtoB_Basis(n,QQi,basis_selected,nd,0),
	    bQQtBeforeIH=EVXData.convertMatrFromQtoB_Basis(n,QQt,basis_selected,nd,0);
	final double [] ev_bQQiBeforeIH=M.EV.getGEV_Generalized(nd,bQQiBeforeIH,bQQtBeforeIH,true),
	    bpsiVIL=EVXData.getPsiFromBasis(0 /* 0-th ev is the min I in restricted basis. */,ev_bQQiBeforeIH,nd),
	    // convert back to regular basis Qk(x)
	    psiIL_BeforeIH=EVXData.convertPsiFromBtoQ_Basis(nd,bpsiVIL,n,basis_selected,0);
	// calculate matrix elements of interest
	I_IL_BeforeIH=TMatr.getAMatrA(psiIL_BeforeIH,QQi,n);
	p_IL_BeforeIH=TMatr.getAMatrA(psiIL_BeforeIH,QQpi,n)/I_IL_BeforeIH;
	T_IL_BeforeIH=TMatr.getAMatrA(psiIL_BeforeIH,QQT,n);
	V_IL_BeforeIH=TMatr.getAMatrA(psiIL_BeforeIH,QQV,n);
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(I_IL_BeforeIH,"I_IL_BeforeIH",ev_bQQiBeforeIH[0],"ev_bQQiBeforeIH[min ev]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiIL_BeforeIH,QQt,n),"<psi|psi>",1,"1",eps);
	    if(flag_remove_psiH_projection) // to be orthogonal to max I
		UnitTests.checkDoubleRel(TMatr.getAMatrB(psiIL_BeforeIH,I.psiH,QQt,n),"<psi|psiH>",0,"0",eps);
	    // check the matrices
	    for(int ok=0;ok<nd;ok++) for(int oj=0;oj<nd;oj++)
	    {
		final double [] psik=TMatr.sumAcB(V.getPsi((n-nd)+ok),flag_remove_psiH_projection?-V.getFPsiPsi((n-nd)+ok,I.psiH,QQt):0,I.psiH),
		    psij=TMatr.sumAcB(V.getPsi((n-nd)+oj),flag_remove_psiH_projection?-V.getFPsiPsi((n-nd)+oj,I.psiH,QQt):0,I.psiH);
		final double Ikj=TMatr.getAMatrB(psij,psik,QQi,n),tjk=TMatr.getAMatrB(psij,psik,QQt,n);
		UnitTests.checkDoubleRel(Ikj,"Ikj",bQQiBeforeIH[ok+nd*oj],"bQQiBeforeIH[ok+nd*oj]",eps);
		UnitTests.checkDouble(bQQtBeforeIH[ok+nd*oj],"bQQtBeforeIH[ok+nd*oj]",tjk,"tjk",eps);
		if(flag_remove_psiH_projection)
		    UnitTests.checkDoubleRel(TMatr.getAMatrB(psik,I.psiH,QQt,n),"<psik|I.psiH>",0,"0",eps);
	    }
	    System.err.println("DM_HT: selftest OK n="+n+" nd="+nd+" I_IL_BeforeIH="+I_IL_BeforeIH+" I.sH="+I.sH+" V_IL_BeforeIH="+V_IL_BeforeIH+" V_IH="+V_IH+" eps="+eps);
	}
    }
} // END OF public class DM_HT
