package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.UnitTests;
/**
 * The answer in a state of a mixed state with a density matrix or a pure state of a wavefunction.
 * For the density matrix the basis is assumend to be orthogonal, i.e. spur(A) is sum A_{k,k}, not spur (A*G^{-1}) for a non--orthogonal basis.
 * @version $Id: DMstateAnswer.java,v 1.15 2022/08/07 15:44:02 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class DMstateAnswer {
    public final double wt,wi,pi,dpdt,wpsiH_projection;
    /** Pure/Mixed state answer.
     */
    private DMstateAnswer(final double wt,final double wi,final double pi,final double dpdt,final double wpsiH_projection)
    {
	this.wt=wt;this.wi=wi;this.pi=pi;this.dpdt=dpdt;this.wpsiH_projection=wpsiH_projection;;
    }
    public static DMstateAnswer mixedState(final int n,final double [] bDM,
					   final int dummy,// bQQt to be equal to the unit matrix, non-orthogonal basis is not implemented yet.
					   final double [] bQQi,
					   final double [] bQQpi,
					   final double [] bQQdpdt,
					   final double [] bpsiH)
    {
	if(bDM.length!=n*n||bpsiH.length!=n) throw new IllegalArgumentException();
	return new DMstateAnswer(TMatr.getSpurM(n,bDM),
				 TMatr.getSpurMM(n,bDM,bQQi),
				 TMatr.getSpurMM(n,bDM,bQQpi)/TMatr.getSpurMM(n,bDM,bQQi),
				 TMatr.getSpurMM(n,bDM,bQQdpdt),
				 TMatr.getAMatrA(n,bpsiH,bDM,n));
    }
    /** Pure state answer. The basis may or may not be an orthogonal one.
     */
    public static DMstateAnswer pureState(final int n,final double [] psi,
					  final double [] QQt,final double [] QQi,
					  final double [] QQpi,
					  final double [] QQdpdt,
					  final double [] psiH)
    {
	if(psi.length!=n||psiH.length!=n) throw new IllegalArgumentException();
	return new DMstateAnswer(TMatr.getAMatrA(psi,QQt,n),
				 TMatr.getAMatrA(psi,QQi,n),
				 TMatr.getAMatrA(psi,QQpi,n)/TMatr.getAMatrA(psi,QQi,n),
				 TMatr.getAMatrA(psi,QQdpdt,n),
				 Math.pow(TMatr.getAMatrB(psiH,psi,QQt,n),2));
    }


public static void matchDMstateAnswer(final DMstateAnswer a,final double a_p_offset,final DMstateAnswer b,final double b_p_offset,final double eps)
{
    final double p_offset_shift=b_p_offset-a_p_offset;
    UnitTests.checkDoubleRel(a.wt,"a.wt",b.wt,"b.wt",eps);
    UnitTests.checkDoubleRel(a.wi,"a.wi",b.wi,"b.wi",eps);
    UnitTests.checkDoubleRel(a.pi,"a.pi",b.pi+p_offset_shift,"b.pi+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dpdt,"a.dpdt",b.dpdt,"b.dpdt",eps);
    UnitTests.checkDoubleRel(a.wpsiH_projection,"a.wpsiH_projection",b.wpsiH_projection,"b.wpsiH_projection",eps);

    System.err.println("DMstateAnswer.matchDMstateAnswer test OK p_offset_shift="+p_offset_shift+" eps="+eps);
}

}
