package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.SortArrays;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.realtimebook.*;

/** Rurik measure, spectral property and pseudotrades.
 *  @version $Id: DQVM.java,v 1.135 2022/07/28 12:27:15 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class DQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final EVXData IT;

final double K00,odp0,sigma2_odp0,odp0a,ndp0,neqe;
final double [] Xx,R;
final double fXx,ndp_x0,K_x0,K_fXx,ndp_fXx,odp_x0,odp_fXx;
final double jXx,jWa0;

DQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final int nmoms=2*neq-1;

    final double [] tmom=new double [nmoms],imom=new double [nmoms],dpmom=new double [nmoms];
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    final double x0=getX0();

    final double [][] QQ=access.getQQ_Matrs(neq,1,new double [][]{tmom,imom,dpmom});
    final double [] QQdp=QQ[2];
    this.IT=isDiagQQt()?DeprecatedHelperEVXData.getEVXDataFromQQfDiagQQa(neq,QQ[1],QQ[0],access,x0):
	DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQ[1],QQ[0],access,x0);

    final double [][] r=getRoots(IT.psiL);
    this.Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0(Xx,x0,isEdgeAtRight());
    this.fXx=indX>=0?Xx[indX]:Double.NaN;

    final double [] tmp=QQ[0];// re-use as temporary array //new double [QQt.length];
    /* // QQt=QQ[0] & QQi=QQ[1] are no longer needed
    // Convert all matrices to the basis of eigenvectors IT.ev
    // QQt & QQi are diagonal in new basis, set directly to avoid rounding errors.
    java.util.Arrays.fill(QQi,0);java.util.Arrays.fill(QQt,0);
    for(int k=0,j=0;k<neq;k++,j+=(neq+1)) {QQt[j]=1;QQi[j]=IT.ev[k];}
    */
    // Convert the remaining matrices.
    for(int k=2;k<QQ.length;k++)
    {
	DeprecatedHelperTMatr.multiply_Matr_x_MatrTrans(neq,tmp,QQ[k],neq,0,IT.ev,neq,neq+0*neq);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(neq,QQ[k],IT.ev,neq,neq+0*neq,tmp,neq,0);
    }

    final int ind_start=1,n=neq-ind_start,oqq=ind_start+ind_start*neq;
    final double Em=IT.ev[ind_start-1];
    final double [] psi_x0=new double [n],Gm1_psi_x0=new double [n],
	psi_dp_aver=new double [n],
	psitmp=new double [neq];
    this.R=new double [neq];
    final double [] ISQQx=new double [n*n];
    for(int k=ind_start,ks=0;k<neq;k++,ks++)
    {
	System.arraycopy(IT.ev,neq+k*neq,psitmp,0,neq);
	// the QQi & QQt matrices are diagonal in new basis
	Gm1_psi_x0[ks]=(ISQQx[ks+n*ks]=1/(IT.ev[k]-Em))*(psi_x0[ks]=access.fx(psitmp,x0));
	DeprecatedHelperTMatr.sumP1(this.R,psitmp,neq,Gm1_psi_x0[ks]);
	psi_dp_aver[ks]=TMatr.mulVVn1(psitmp,dpmom);
    }

    this.K00=TMatr.mulVVn1(psi_x0,Gm1_psi_x0);
    this.odp0=TMatr.mulVVn1(psi_dp_aver,Gm1_psi_x0)/Math.sqrt(K00);
    this.sigma2_odp0=TMatr.getAMatrB(n,psi_dp_aver,psi_dp_aver,ISQQx,n)-odp0*odp0;

    //System.err.println("n="+n+" QQdp.length="+QQdp.length+" neq="+neq+" oqq="+oqq);
    // Radon-Nikodym derivative dp/dmu at x=x0.
    this.ndp0=TMatr.getAMatrB(n,Gm1_psi_x0,0,n,Gm1_psi_x0,0,
			      DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(neq,QQdp,ind_start-1),n
			      //QQdp,neq,oqq
			      )/K00;


    final double [] KSQQdp=new double [n*n];
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,QQdp,neq,oqq,ISQQx,n,0);
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,KSQQdp,ISQQx,n,0,tmp,n,0);
    // convert back to polynomial basis

    final double [][] KQQ=convertToQQ(neq,n,new double [][] {ISQQx,KSQQdp},IT.ev,neq+ind_start*neq);
    final double [][] kkk=access.getKK(neq,KQQ);
    final double [] kkt=kkk[0],kkdp=kkk[1];
    this.ndp_x0=access.fx(kkdp,x0)/access.fx(kkt,x0);
    this.K_x0=access.fx(kkt,x0);
    this.ndp_fXx=access.fx(kkdp,fXx)/access.fx(kkt,fXx);
    this.K_fXx=access.fx(kkt,fXx);

    final double [] px=TMatr.getMatr_x_A(neq,KQQ[0],java.util.Arrays.copyOf(dpmom,neq));
    odp_x0=access.fx(px,x0)/Math.sqrt(access.fx(kkt,x0));
    odp_fXx=access.fx(px,fXx)/Math.sqrt(access.fx(kkt,fXx));


    int indJ=-1;
    double wa0=Double.NEGATIVE_INFINITY;
    for(int j=0;j<(Xx!=null?Xx.length:0);j++)
    {
	final double xj=Xx[j];
	double s_00=0,s_a0=0,s_aa=0;
	for(int k=ind_start,ks=0;k<neq;k++,ks++)
	{
	    System.arraycopy(IT.ev,neq+k*neq,psitmp,0,neq);
	    final double f0=access.fx(psitmp,x0),fa=access.fx(psitmp,xj),iqq=ISQQx[ks+n*ks];
	    s_00+=f0*f0*iqq;
	    s_aa+=fa*fa*iqq;
	    s_a0+=f0*fa*iqq;
	}
	final double ww=s_a0/Math.sqrt(s_aa*s_00);
	if(Math.abs(ww)>=wa0)
	{
	    wa0=Math.abs(ww);
	    indJ=j;
	}
    }
    if(indJ>=0)
    {
	jXx=Xx[indJ];
	jWa0=wa0;
    }
    else
    {
	jXx=jWa0=Double.NaN;
    }



    // for unit test
    final double [] QQx=new double [neq*neq];for(int k=0;k<neq;k++){QQx[k+neq*k]=IT.ev[k]-Em;}
    this.neqe=TMatr.getAMatrB(n,Gm1_psi_x0,0,n,Gm1_psi_x0,0,
			      DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(neq,QQx,ind_start-1),n
	                      //QQx,neq,oqq
	)/K00;
    this.odp0a=TMatr.getAMatrB(n,psi_x0,psi_dp_aver,ISQQx,n)/Math.sqrt(K00);
    this.simom=imom;
    this.stmom=tmom;
    this.sdpmom=dpmom;

}

final double [] simom,stmom,sdpmom;

static double [][] convertToQQ(final int nr,final int ns,final double [][] SK,final double [] ev,final int off_ev)
{
    final double [] tmp=new double [nr*ns];
    final double [][] QK=new double [SK.length][];
    for(int j=0;j<QK.length;j++)
    {
	final double [] Q=QK[j]=new double [nr*nr],S=SK[j];
	for(int k=0;k<ns;k++)
	{
	    for(int l=0;l<nr;l++)
	    {
		double s=0;
		for(int m=0;m<ns;m++)
		{
		    //System.err.println("ind="+(off_ev+m*nr+l)+" off_ev="+off_ev+"ev.length="+ev.length);
		    s+=S[k*ns+m]*ev[off_ev+m*nr+l];
		}
		tmp[k+l*ns]=s;
	    }
	}
	for(int k=0;k<nr;k++)
	{
	    for(int l=0;l<nr;l++)
	    {
		double s=0;
		for(int m=0;m<ns;m++)
		{
		    s+=ev[off_ev+m*nr+k]*tmp[m+l*ns];
		}
		Q[k*nr+l]=s;
	    }
	}
    }
    return QK;
}

boolean isDiagQQt(){return true;}
double getDS()
{
    return Double.NaN; // FIXME
}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double getX0();

abstract PolynomialRootsFinder getPolynomialRootsFinderInBasis();
abstract boolean isEdgeAtRight();
void sortArray(double [] re,double [] im)
{
    if(isEdgeAtRight()) {SortArrays.sortArrayWithLoadDesc(re.length,re,im);}
    else {SortArrays.sortArrayWithLoad(re.length,re,im);}
}

double [][] getRoots(final double [] psi)
{
    final int n=psi.length-1;
    if(n<=0) return null;
    final PolynomialRootsFinder.PRoots u=getPolynomialRootsFinderInBasis().findRoots(psi);
    final double [] re_root=u.re_root,im_root=u.im_root;
    if(u.nroots!=n) return null;
    sortArray(re_root,im_root);
    return new double [][]{re_root,im_root};
}


static abstract class DQVMTest<DTYPE> {

int neq;
long currenttime,tau;
Trade [] tb;

abstract DQVM<DTYPE> getDQVM();
abstract DQVM<?> getDQVMbasis();
abstract DjQVM<?> getDjQVM();

abstract long getTau();

final String name;
final java.util.Random r;
final double eps;
DQVMTest(final String name,
	 final java.util.Random r,final double eps)
{
    this.name=name;
    this.r=r;
    this.eps=eps;
}

void test_BVectorRemoved()
{
    final DQVM<DTYPE> q=getDQVM();

    final double [][] QQ=q.access.getQQ_Matrs(neq,1,new double [][]{q.stmom,q.simom,q.sdpmom});
    final double [] QQt=QQ[0],QQi=QQ[1],QQdp=QQ[2];

    int neq=this.neq;
    final double x0=q.getX0(),fXx=q.fXx;
    final int n=neq-1,ind_skip=0;
    final double [] psiskip_x0=new double [n],psiskip_Xx=new double [n],psiskip_dp_aver=new double [n],psitmp=new double [neq];
    final double [] SQQt=new double [n*n],SQQi=new double [n*n],SQQdp=new double [n*n];

    for(int k=0,ks=-1;k<neq;k++)
    {
	//System.err.println("k="+k+" diag="+(QQi[k+neq*k]-IT.sL*QQt[k+neq*k]));
	if(k==ind_skip) continue;
	++ks;
	System.arraycopy(q.IT.ev,neq+k*neq,psitmp,0,neq);
	psiskip_x0[ks]=q.access.fx(psitmp,x0);
	psiskip_Xx[ks]=q.access.fx(psitmp,fXx);
	psiskip_dp_aver[ks]=TMatr.mulVVn1(psitmp,q.sdpmom);
	for(int l=0,ls=-1;l<neq;l++)
	{
	    if(l==ind_skip) continue;
	    ++ls;
	    final double [] psitmpl=new double [neq];
	    System.arraycopy(q.IT.ev,neq+l*neq,psitmpl,0,neq);
	    SQQt[ks+n*ls]=TMatr.getAMatrB(neq,psitmp,psitmpl,QQt,neq);
	    SQQi[ks+n*ls]=TMatr.getAMatrB(neq,psitmp,psitmpl,QQi,neq);
	    SQQdp[ks+n*ls]=TMatr.getAMatrB(neq,psitmp,psitmpl,QQdp,neq);
	}
    }
    final double [] SQQx=SQQi.clone();TMatr.sumP1(SQQx,SQQt,-q.IT.sL);
    final double [] ISQQx=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,SQQx);

    neq=-1; // to catch mis-usage

    final double K00=TMatr.getAMatrB(n,psiskip_x0,psiskip_x0,ISQQx,n),
	odp0=TMatr.getAMatrB(n,psiskip_x0,psiskip_dp_aver,ISQQx,n)/Math.sqrt(K00),
	sigma2_odp0=TMatr.getAMatrB(n,psiskip_dp_aver,psiskip_dp_aver,ISQQx,n)-odp0*odp0;
    double [] tmp=TMatr.getMatr_x_A(n,ISQQx,psiskip_x0);
    final double odp0a=TMatr.mulVVn1(psiskip_dp_aver,tmp)/Math.sqrt(K00),
    // Radon-Nikodym derivative.
	ndp0=TMatr.getAMatrB(n,tmp,tmp,SQQdp,n)/K00,
	neqe=TMatr.getAMatrB(n,tmp,tmp,SQQx,n)/K00;

    UnitTests.checkDoubleRel(q.K00,"q.K00",K00,"K00",eps);
    UnitTests.checkDoubleRel(q.odp0,"q.odp0",odp0,"odp0",eps);
    UnitTests.checkDoubleRel(q.odp_x0,"q.odp_x0",odp0,"odp0",eps);
    if(neq>2) UnitTests.checkDoubleRel(q.sigma2_odp0,"q.sigma2_odp0",sigma2_odp0,"sigma2_odp0",eps);
    UnitTests.checkDoubleRel(q.odp0a,"q.odp0a",odp0a,"odp0a",eps);
    UnitTests.checkDoubleRel(q.ndp0,"q.ndp0",ndp0,"ndp0",eps);
    UnitTests.checkDoubleRel(q.neqe,"q.neqe",1,"1",eps);
    UnitTests.checkDoubleRel(q.K_x0,"q.K_x0",K00,"K00",eps);
    UnitTests.checkDoubleRel(q.ndp_x0,"q.ndp_x0",ndp0,"ndp0",eps);

    tmp=TMatr.getMatr_x_A(n,ISQQx,psiskip_Xx);
    final double K_fXx=TMatr.getAMatrB(n,tmp,tmp,SQQx,n),ndp_fXx=TMatr.getAMatrB(n,tmp,tmp,SQQdp,n)/K_fXx;
    UnitTests.checkDoubleRel(q.ndp_fXx,"q.ndp_fXx",ndp_fXx,"ndp_fXx",eps);
    UnitTests.checkDoubleRel(q.K_fXx,"q.K_fXx",K_fXx,"K_fXx",eps);


    System.err.println(name+":DQVMTest:test_BVectorRemoved(): test OK neq="+neq+" eps="+eps);
}

void printRoots(final String txt,final double [] pol,final boolean flag_test_im0,final DQVM<DTYPE> q)
{
    final int n=pol.length-1;
    final PolynomialRootsFinder.PRoots u=q.getPolynomialRootsFinderInBasis().findRoots(pol);
    final double [] re=u.re_root,im=u.im_root;
    if(u.nroots!=n) throw new RuntimeException("u.nroots="+u.nroots+" n="+n);
    SortArrays.sortArrayWithLoad(n,re,im);
    System.err.println(txt+".re="+java.util.Arrays.toString(re));
    System.err.println(txt+".im="+java.util.Arrays.toString(im));

    if(flag_test_im0){UnitTests.checkDoubleArrayRel(im,"im",new double [n],"0",eps);}
}

void test_Common()
{
    final DQVM<DTYPE> q=getDQVM();

    UnitTests.checkDoubleRel(q.odp0,"q.odp0",q.odp0a,"q.odp0a",eps);
    UnitTests.checkDoubleRel(q.neqe,"q.neqe",1,"1",eps);
    if(neq<=2) UnitTests.checkDoubleRel(q.sigma2_odp0,"q.sigma2_odp0",0,"0",eps);
    UnitTests.checkDoubleRel(q.K00,"q.K00",q.access.fx(q.R,q.getX0()),"R(x0)",eps);

    printRoots("R",q.R,false,q);
    printRoots("psiL",q.IT.psiL,true,q);
    printRoots("psiH",q.IT.psiH,true,q);

    UnitTests.checkDoubleRel(q.K00,"q.K00",q.K_x0,"q.K_x0",eps);
    UnitTests.checkDoubleRel(q.ndp0,"q.ndp0",q.ndp_x0,"q.ndp_x0",eps);

    System.err.println(name+":DQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}
void setData(final int tsize,final int neq)
{
    this.neq=neq;
    this.tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
    this.currenttime=tb[tb.length-1].time;
    this.tau=getTau();
    //System.err.println("tau="+tau);
}
boolean shouldTestPsi(){return true;}
double [] getRootsToMatch(final double [] r){return r.clone();}
void test_DjQVM()
{
    final DjQVM<?> s=getDjQVM();
    if(s==null) return;
    final DQVM<DTYPE> q=getDQVM();

    EVXData.matchEVXData(q.IT,s.I,shouldTestPsi(),true,eps);
    final double [] sx=getRootsToMatch(s.Xx);
    q.sortArray(sx,null);
    UnitTests.checkDoubleArrayRel(q.Xx,"q.Xx",sx,"sx",eps);
    final double fXx=getRootsToMatch(new double []{s.fXx})[0];
    UnitTests.checkDoubleRel(q.fXx,"q.fXx",fXx,"fXx",eps);

    System.err.println(name+":DQVMTest:test_DjQVM(): test OK neq="+neq+" eps="+eps);
}
void test_DQVMbasis()
{
    final DQVM<?> s=getDQVMbasis();
    if(s==null) return; // skip test
    final DQVM<DTYPE> q=getDQVM();

    EVXData.matchEVXData(q.IT,s.IT,false,true,eps);
    UnitTests.checkDoubleRel(q.K00,"q.K00",s.K00,"s.K00",eps);
    UnitTests.checkDoubleRel(q.odp0,"q.odp0",s.odp0,"s.odp0",eps);
    if(neq>2) UnitTests.checkDoubleRel(q.sigma2_odp0,"q.sigma2_odp0",s.sigma2_odp0,"s.sigma2_odp0",eps);
    UnitTests.checkDoubleRel(q.odp0a,"q.odp0a",s.odp0a,"s.odp0a",eps);
    UnitTests.checkDoubleRel(q.ndp0,"q.ndp0",s.ndp0,"s.ndp0",eps);
    //UnitTests.checkDoubleRel(q.fXx,"q.fXx",s.fXx,"s.fXx",eps);
    UnitTests.checkDoubleRel(q.K_x0,"q.K_x0",s.K_x0,"s.K_x0",eps);
    UnitTests.checkDoubleRel(q.K_fXx,"q.K_fXx",s.K_fXx,"s.K_fXx",eps);
    UnitTests.checkDoubleRel(q.ndp_x0,"q.ndp_x0",s.ndp_x0,"s.ndp_x0",eps);
    UnitTests.checkDoubleRel(q.ndp_fXx,"q.ndp_fXx",s.ndp_fXx,"s.ndp_fXx",eps);

    System.err.println(name+":DQVMTest:test_DQVMbasis(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":DQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_DQVMbasis();
    test_DjQVM();
    test_BVectorRemoved();

    System.err.println(name+":DQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":DQVMTest: starting tests; ver=$Id: DQVM.java,v 1.135 2022/07/28 12:27:15 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":DQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: DQVM.java,v 1.135 2022/07/28 12:27:15 mal Exp $");
}
} // END OF class DQVMTest

}// END OF class DQVM
