package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.utils.PolynomialRootsConfederateMatrix;
import com.polytechnik.realtimebook.*;

/** Unit test for DQVMLaguerre.
 */
public class DQVMMonomials extends DQVM<QVMData> {

DQVMMonomials(final int neq,final QVMData odata){super(neq,odata);}

public static DQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final DQVMMonomials ref)
{
    final int npm=3;
    final QVMData o=QVMData.calculateQVMData(1,2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new DQVMMonomials(neq,o);
}
boolean isDiagQQt(){return false;}
DeprecatedHelperQVMDataDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataDirectAccessWithBasis();}
double getX0(){return 0;}
boolean isEdgeAtRight(){return true;}
public PolynomialRootsFinder getPolynomialRootsFinderInBasis()
{
    return PolynomialRootsConfederateMatrix.ABASISMONOMIALS;
}

static class DQVMMonomialsTest extends DQVMTest<QVMData> {
DQVMMonomialsTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
DQVM<?> getDQVMbasis(){return DQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
DQVMMonomials getDQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DjQVMLaguerre getDjQVM(){return DjQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
boolean shouldTestPsi(){return false;}
double [] getRootsToMatch(final double [] r){final double [] x=r.clone();for(int k=x.length;--k>=0;) {x[k]=-x[k];} return x;}
} // END of class DQVMMonomialsTest

public static void main(String [] args)
{
    new DQVMMonomialsTest("DQVMMonomials").runTests(5);
} // END main



} // END OF class DQVMMonomials
