package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/** Radau measure and infinitesimal shift.
 *  @version $Id: DREVXData.java,v 1.26 2022/11/20 12:41:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DREVXData {
    final EVXData ev;
    final double p_L,i_L,var_L,f_L,p_H,i_H,var_H,f_H,p_0,i_0,var_0,p_m;
    DREVXData(final EVXData ev,final double p_L,final double i_L,final double var_L,final double f_L,
	      final double p_H,final double i_H,final double var_H,final double f_H,
	      final double p_0,final double i_0,final double var_0,final double p_m)
    {
	this.ev=ev;
	this.p_L=p_L;
	this.i_L=i_L;
	this.var_L=var_L;
	this.f_L=f_L;
	this.p_H=p_H;
	this.i_H=i_H;
	this.var_H=var_H;
	this.f_H=f_H;
	this.p_0=p_0;
	this.i_0=i_0;
	this.var_0=var_0;
	this.p_m=p_m;
    }


static DREVXData get_DREVXData(final int n0,final double [] QQi,final double [] QQt,final double [] QQpi,final double [] QQf,final double [] QQfn,
			       final VolMatrix VOLATILITY_EDPsi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] RQQt=VolMatrix.getRadauMatrix(n0,QQt,M),
	RQQDi=VOLATILITY_EDPsi.getDRadauMatrix(n0,QQi,M),
	RQQDpi=VOLATILITY_EDPsi.getDRadauMatrix(n0,QQpi,M),
	RQQi=VolMatrix.getRadauMatrix(n0,QQi,M),
	RQQpi=VolMatrix.getRadauMatrix(n0,QQpi,M),
	RQQf=VolMatrix.getRadauMatrix(n0,QQf,M),
	RQQfn=VolMatrix.getRadauMatrix(n0,QQfn,M);

    final int n=n0-1;
    final EVXData e=EVXData.getEVXDataFromQQ(n,RQQDi,RQQt,M);
    final double i_L=TMatr.getAMatrA(e.psiL,RQQi,n),
	p_L=TMatr.getAMatrA(e.psiL,RQQpi,n)/i_L,
	var_L=TMatr.getAMatrA(e.psiL,RQQDpi,n)-e.sL*p_L,
	f_L=TMatr.getAMatrA(e.psiL,RQQf,n)/TMatr.getAMatrA(e.psiL,RQQfn,n),
	i_H=TMatr.getAMatrA(e.psiH,RQQi,n),
	p_H=TMatr.getAMatrA(e.psiH,RQQpi,n)/i_H,
	var_H=TMatr.getAMatrA(e.psiH,RQQDpi,n)-e.sH*p_H,
	f_H=TMatr.getAMatrA(e.psiH,RQQf,n)/TMatr.getAMatrA(e.psiH,RQQfn,n),
	i_0=TMatr.getAMatrA(e.psi0,RQQi,n),
	p_0=TMatr.getAMatrA(e.psi0,RQQpi,n)/i_0,
	var_0=TMatr.getAMatrA(e.psi0,RQQDpi,n)-e.s0*p_0,
	p_m=(TMatr.getAMatrA(e.psiH,RQQDpi,n)-TMatr.getAMatrA(e.psiL,RQQDpi,n))/(e.sH-e.sL);
    //System.err.println("DREVXData: e.sL="+e.sL+" e.sH="+e.sH);
    //if(e.sH<0) throw new RuntimeException();

    return new DREVXData(e,p_L,i_L,var_L,f_L,p_H,i_H,var_H,f_H,p_0,i_0,var_0,p_m);
}
static DREVXData get0_DREVXData(final int n0,final double [] QQi,final double [] QQt,final double [] QQpi,final double [] QQf,final double [] QQfn,
			       final VolMatrix VOLATILITY_EDPsi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] RQQt=VolMatrix.getRadau2Matrix(n0,QQt,M),
	RQQDi=VOLATILITY_EDPsi.getDRadau2Matrix(n0,QQi,M),
	RQQDpi=VOLATILITY_EDPsi.getDRadau2Matrix(n0,QQpi,M),
	RQQi=VolMatrix.getRadau2Matrix(n0,QQi,M),
	RQQpi=VolMatrix.getRadau2Matrix(n0,QQpi,M),
	RQQf=VolMatrix.getRadau2Matrix(n0,QQf,M),
	RQQfn=VolMatrix.getRadau2Matrix(n0,QQfn,M);

    final int n=n0-1;
    final EVXData e=EVXData.getEVXDataFromQQ(n,RQQDi,RQQt,M);
    final double i_L=TMatr.getAMatrA(e.psiL,RQQi,n),
	p_L=TMatr.getAMatrA(e.psiL,RQQpi,n)/i_L,
	var_L=TMatr.getAMatrA(e.psiL,RQQDpi,n)-e.sL*p_L,
	f_L=TMatr.getAMatrA(e.psiL,RQQf,n)/TMatr.getAMatrA(e.psiL,RQQfn,n),
	i_H=TMatr.getAMatrA(e.psiH,RQQi,n),
	p_H=TMatr.getAMatrA(e.psiH,RQQpi,n)/i_H,
	var_H=TMatr.getAMatrA(e.psiH,RQQDpi,n)-e.sH*p_H,
	f_H=TMatr.getAMatrA(e.psiH,RQQf,n)/TMatr.getAMatrA(e.psiH,RQQfn,n),
	i_0=TMatr.getAMatrA(e.psi0,RQQi,n),
	p_0=TMatr.getAMatrA(e.psi0,RQQpi,n)/i_0,
	var_0=TMatr.getAMatrA(e.psi0,RQQDpi,n)-e.s0*p_0,
	p_m=(TMatr.getAMatrA(e.psiH,RQQDpi,n)-TMatr.getAMatrA(e.psiL,RQQDpi,n))/(e.sH-e.sL);

    return new DREVXData(e,p_L,i_L,var_L,f_L,p_H,i_H,var_H,f_H,p_0,i_0,var_0,p_m);
}


    static double [] test_getDRadauMatrix(final int n0,final double [] QQ,final VolMatrix VOLATILITY_EDPsi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	final int n=n0-1;
	final double [] RQQD=new double [n*n];
	for(int k=0;k<n;k++)
	{
	    final double [] p=new double [k+1];p[k]=1;
	    final double [] rp=M.getRadauMultiplied(p),Drp=VOLATILITY_EDPsi.getDPsi(rp);
	    for(int j=0;j<n;j++)
	    {
		final double [] pp=new double [j+1];pp[j]=1;
		RQQD[k*n+j]=TMatr.getAMatrB(Drp,pp,QQ,n0)+TMatr.getAMatrB(rp,VOLATILITY_EDPsi.getDPsi(pp),QQ,n0);
	    }
	}
	return RQQD;
    }
    static double [] test_getRadauMatrix(final int n0,final double [] QQ,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	final int n=n0-1;
	final double [] RQQ=new double [n*n];
	for(int k=0;k<n;k++)
	{
	    final double [] p=new double [k+1];p[k]=1;
	    final double [] rp=M.getRadauMultiplied(p);
	    for(int j=0;j<n;j++)
	    {
		final double [] pp=new double [j+1];pp[j]=1;
		RQQ[k*n+j]=TMatr.getAMatrB(rp,pp,QQ,n0);
	    }
	}
	return RQQ;
    }

    static void test_DREVXData(final DREVXData q,
			       final int n0,final double [] QQi,final double [] QQt,final double [] QQpi,
			       final VolMatrix VOLATILITY_EDPsi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final double eps)
    {
	final double [] RQQt=VolMatrix.getRadauMatrix(n0,QQt,M),
	    RQQi=VolMatrix.getRadauMatrix(n0,QQi,M),
	    sRQQi=test_getRadauMatrix(n0,QQi,M),
	    RQQDi=VOLATILITY_EDPsi.getDRadauMatrix(n0,QQi,M),
	    sRQQDi=test_getDRadauMatrix(n0,QQi,VOLATILITY_EDPsi,M);
	UnitTests.checkDoubleArrayRel(RQQDi,"RQQDi",sRQQDi,"sRQQDi",eps);
	UnitTests.checkDoubleArrayRel(RQQi,"RQQi",sRQQi,"sRQQi",eps);
	final int n=n0-1;
	UnitTests.checkDoubleRel(Math.abs(q.ev.wL),"q.ev.wL",Math.abs(TMatr.getAMatrB(q.ev.psi0,q.ev.psiL,RQQt,n)),"<psi0|psiL>",eps);
	UnitTests.checkDoubleRel(Math.abs(q.ev.wH),"q.ev.wH",Math.abs(TMatr.getAMatrB(q.ev.psi0,q.ev.psiH,RQQt,n)),"<psi0|psiH>",eps);

	if(true)
	{
	    final double [] QQ0=M.B.getQQMatr(n,M.B.getX0Moments(2*n-1,q.ev.x0));
	    /*
	    System.err.println("RQQt="+java.util.Arrays.toString(RQQt));
	    System.err.println("RQQi="+java.util.Arrays.toString(RQQi));
	    System.err.println("QQ0="+java.util.Arrays.toString(QQ0));
	    //System.err.println("ev0b(orig)="+java.util.Arrays.toString(com.polytechnik.algorithms.EigenvaluesSymmetric.getGEV(n,QQ0,n,RQQt,n,true)));
	    System.err.println("ev0b.dsygvx_dsyevr="+java.util.Arrays.toString(com.polytechnik.lapack.Eigenvalues.getGEV_dsygvx_dsyevr(n,QQ0,n,RQQt,n,true)));
	    System.err.println("ev0b.dsygvx="+java.util.Arrays.toString(com.polytechnik.lapack.Eigenvalues.getGEV_dsygvx(n,QQ0,n,RQQt,n,true)));
	    */
	    final EVXData e0=EVXData.getEVXDataFromQQ(n,QQ0,RQQt,M);
	    System.err.println("e0.ev="+java.util.Arrays.toString(e0.getEValues()));
	    if(!Double.isNaN(e0.ev[0])) // sometimes fail
	    UnitTests.checkDoubleArrayRel(EVXData.getMSign(e0.psiH),"e0.psiH",EVXData.getMSign(q.ev.psi0),"q.ev.psi0",eps);
	}
	final DREVXData q0=get_DREVXData(n0,M.B.getQQMatr(n0,M.B.getX0Moments(2*n0-1,q.ev.x0)),QQt,QQpi,QQpi,QQi,VOLATILITY_EDPsi,M);
	if(!Double.isNaN(q0.ev.ev[0])) // sometimes fail
	UnitTests.checkDoubleArrayRel(EVXData.getMSign(q0.ev.psiL),"q0.ev.psiL",EVXData.getMSign(q.ev.psi0),"q.ev.psi0",eps);
	final DREVXData qc=get_DREVXData(n0,QQi,QQt,QQi,QQpi,QQi,VOLATILITY_EDPsi,M);
	UnitTests.checkDoubleRel(qc.var_L,"qc.var_L",0,"0",eps);
	UnitTests.checkDoubleRel(qc.var_H,"qc.var_H",0,"0",eps);
	final DREVXData qC=get_DREVXData(n0,QQt,QQt,QQpi,QQpi,QQi,VOLATILITY_EDPsi,M);
	System.err.println("qC.ev="+java.util.Arrays.toString(qC.ev.getEValues())+" QQt="+java.util.Arrays.toString(QQt)); // all ev must be =0

	System.err.println("DREVXData:test_DREVXData: test OK n0="+n0+" eps="+eps);
    }


}

