package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */

/** One more interface to a basis.
 *  @see com.polytechnik.freemoney.CommonlyUsedMoments
 *  @version $Id: DataInterfaceDirect.java,v 1.44 2022/07/29 20:31:08 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public interface DataInterfaceDirect<DTYPE> {

/** Set the value of timeserie moments.
 * @param m A buffer to save the moments of at least {@code nmoms} size.
 * @param n Price power before the moment.
 * @param mom Time seqience ID from {@link DataInterfaceDirectTradesHistorical}.
 * @param odata The object that stores the data.
 * @param nmoms The number of moments to save.
 */
void setM(final double [] m,final int n,final int mom,final DTYPE odata,final int nmoms);

/** Sets exact values of \int Q(x) dmu.
 *  The setM(final double [] m,0,M_T,odata,nmoms) set sample value of \int Q(x) dmu,
 *  and this method sets exact value. The results should be very close for a reasonable grid.
 * @see com.polytechnik.freemoney.CommonlyUsedMoments#getdtMoments
 * @param m A buffer to save the moments of at least {@code nmoms} size.
 * @param nmoms The number of moments to save.
 */
void setMT(final double [] m,final int nmoms);

/** Price offset. All price moments are calculated relatively this offset,
 * this greatly increases numerical stability.
 * To obtain "actual" price from the values saved in odata --
 * add the value returned by this method.
 * @see com.polytechnik.freemoney.CommonlyUsedMoments#getPOffset
 */
double getPOffset(final DTYPE odata);

/** Last price (with offset), actual price is {@link #getPLast}+{@link #getPOffset}. */
double getPLast(final DTYPE odata);

/** Last time. */
long getTLast(final DTYPE odata);

/** Exponent time scale. */
long getTau(final DTYPE odata);

/** Total Volume. */
long getTotalVolume(final DTYPE odata);

}
