package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.EVSolver;

/** A helper to access EVXData from deprecated code.
 *  @version $Id: DeprecatedHelperEVXData.java,v 1.22 2022/11/09 16:03:54 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
@Deprecated
class DeprecatedHelperEVXData {


static EVXData getEVXData(final int n,final double [] momf,final double [] mom,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{momf,mom});
    return getEVXDataFromQQ(n,QQ[0],QQ[1],access,x0);
}

static EVXData getEVXDataFromQQ(final int n,final double [] QQf,final double [] QQa,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
{
    return _getEVXDataFromQQ(n,solveGeneralizedEV(n,QQf,QQa),access,x0);
}

static EVXData getEVXDataFromQQfDiagQQa(final int n,final double [] QQf,final double [] QQa,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
{
    return _getEVXDataFromQQ(n,EVXData.getSymEVDiagT(n,QQf,QQa,EV),access,x0);
}

private static EVXData _getEVXDataFromQQ(final int n,final double [] evS,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
{
    if(evS.length!=n+n*n) throw new IllegalArgumentException();
    final double [] psiL=new double [n],psiH=new double [n],psi0=new double [n],psik_at_x0=new double [n];
    final double [] r=psiL; // use as a temporary array
    double s_s_o_o=0,ss0=0,s_wL=0,s_wH=0;
    for(int k=0;k<n;k++)
    {
	System.arraycopy(evS,n+(k)*n,r,0,n);
	final double fkx0=access.fx(r,x0),f2kx0=fkx0*fkx0;
	psik_at_x0[k]=fkx0;
	TMatr.sumP1(psi0,r,fkx0);
	if(k<=0) s_wL=fkx0;
	if(k>=n-1) s_wH=fkx0;
	s_s_o_o+=f2kx0;
	ss0+=f2kx0*evS[k];
    }
    final double sL=evS[0],sH=evS[n-1];
    System.arraycopy(evS,n+(0)*n,psiL,0,n);
    System.arraycopy(evS,n+(n-1)*n,psiH,0,n);
    final double sqrt_s_s_o_o=Math.sqrt(s_s_o_o);
    final double wL=s_wL/sqrt_s_s_o_o,wH=s_wH/sqrt_s_s_o_o,s0=ss0/s_s_o_o;
    for(int k=0;k<n;k++) psi0[k]/=sqrt_s_s_o_o;

    return new EVXData(n,wL,sL,psiL,wH,sH,psiH,psi0,evS,psik_at_x0,s0,s_s_o_o,x0);
}

static double [] getX0Moms(final int nmoms,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
{
    final double [] m=new double [nmoms],r=new double [nmoms];
    for(int k=nmoms;--k>=0;)
    {
	r[k]=1;
	m[k]=access.fx(r,x0);
	r[k]=0;
    }
    return m;
}

static final EVSolver EV=EVSolver.getDefaultEVSolver();
static double [] getSymEV(final int n,final double [] QQ)
{
    //return com.polytechnik.algorithms.EigenvaluesSymmetric.getEV(n,QQ,n,true);
    //return com.polytechnik.lapack.Eigenvalues.getEV_dsyevx(n,QQ,n,true);
    return EV.getEV(n,QQ,true);
}
static double [] solveGeneralizedEV(final int n,final double [] A,final double [] B)
{
    //return com.polytechnik.algorithms.EigenvaluesSymmetric.getGEV(n,A,n,B,n,true);
    //return com.polytechnik.lapack.Eigenvalues.getGEV_dsygvx(n,A,n,B,n,true);
    return EV.getGEV_Generalized(n,A,B,true);
}

/** Remove one vector from matrix.
 * return n-1 x n-1 matrix with one vector removed from original n x n matrix.
 */
static double [] getOneVectorRemovedFromMatrix(final int n,final double [] QQ,final int ind_toremove)
{
    if(ind_toremove<0||ind_toremove>=n||QQ.length!=n*n) throw new IllegalArgumentException();
    final int nd=n-1;
    final double [] bb=new double [nd*nd];
    for(int k=0,kj=0;k<n;k++)
    {
	if(k==ind_toremove) continue;
	for(int l=0,lj=0;l<n;l++)
	{
	    if(l==ind_toremove) continue;
	    bb[kj*nd+lj]=QQ[k*n+l];
	    lj++;
	}
	kj++;
    }
    return bb;
}

/** Remove one vector from basis.
 * return nb-1 x n matrix with one vector removed.
 */
static double [] getBasisOneVectorRemoved(final int nb,final double [] basis,final int n,final int basis_offset,final double [] psi_toremove,final double [] QQt)
{
    final int nd=nb-1;
    final double [] bb=new double [n*nd],psi=new double [n];
    final double psi_toremove2norm=TMatr.getAMatrA(psi_toremove,QQt,n);
    int ind_lownorm=-1;double lownorm2=Double.NaN;
    for(int k=0;k<nb;k++)
    {
	System.arraycopy(basis,basis_offset+k*n,psi,0,n);
	TMatr.sumP1(psi,psi_toremove,-TMatr.getAMatrB(psi_toremove,psi,QQt,n)/psi_toremove2norm);
	final double psi2norm=TMatr.getAMatrA(psi,QQt,n);
	if(ind_lownorm<0 || !(psi2norm>=lownorm2)){lownorm2=psi2norm;ind_lownorm=k;}
	TMatr.scaleVectorInPlace(psi,1/Math.sqrt(psi2norm));
	if(k<nd)
	{
	    System.arraycopy(psi,0,bb,k*n,n);
	}
	else if(!(ind_lownorm<0) && ind_lownorm<nd)
	{
	    System.arraycopy(psi,0,bb,ind_lownorm*n,n);
	}
    }
    return bb;
}

/** Remove one vector from basis.
 * return nd x n matrix with one vector removed.
 */
static double [] getBasisOneVectorRemoved(final int n,final double [] basis,final int basis_offset,final int ind_toremove)
{
    if(false)
    {
	if(ind_toremove<0||ind_toremove>=n) throw new IllegalArgumentException();
	final int nd=n-1;
	final double [] bb=new double [n*nd];
	for(int k=0,j=0;k<n;k++)
	{
	    if(k==ind_toremove) continue;
	    System.arraycopy(basis,basis_offset+k*n,bb,j*n,n);
	    j++;
	}
	return bb;
    }
    return getBasisWithVectorsRemoved(n,basis,n,basis_offset,new int []{ind_toremove});
}
/** Remove several vectors from basis.
 * return nd x n matrix with ind_toremove.length vectors removed.
 */
static double [] getBasisWithVectorsRemoved(final int nb,final double [] basis,final int n,final int basis_offset,final int [] ind_toremove)
{
    	final int nd=nb-ind_toremove.length;
	final double [] bb=new double [nd*n];
	int nx=0;
        e:for(int k=0;k<nb;k++)
	{
	    for(int j=ind_toremove.length;--j>=0;) if(k==ind_toremove[j]) continue e;
	    if(nx>=nd) throw new IllegalArgumentException("n="+n+" ind_toremove="+java.util.Arrays.toString(ind_toremove));
	    System.arraycopy(basis,k*n+basis_offset,bb,nx*n,n);
	    nx++;
	    if(nx>nd) throw new RuntimeException("getBasisWithVectorsRemoved:n="+n+" k="+k+" nx="+nx+" nd="+nd+" ind_toremove="+java.util.Arrays.toString(ind_toremove));
	}
	if(nx!=nd) throw new RuntimeException("getBasisWithVectorsRemoved:n="+n+" nx="+nx+" nd="+nd+" ind_toremove="+java.util.Arrays.toString(ind_toremove));
	return bb;
}



/** Least squares approximation. Uses polynomial basis.
 */
static double [] getLinsysSol(final int neq,
				     final double [] mom,final int mom_offset,
				     final double [] rpart,final int rpart_offset)
{
    final double [] r=new double [neq],matr=new double [neq*neq],norma=new double [neq];
    for(int k=0;k<neq;k++)
    {
	r[k]=rpart[k+rpart_offset]/(norma[k]=Math.sqrt(Math.abs(mom[mom_offset+2*k]))); // abs for negatively defined matrix
	for(int l=0;l<=k;l++)
	{
	    matr[k+neq*l]=matr[l+neq*k]=mom[mom_offset+k+l]/norma[l]/norma[k];
	}
    }
    final int [] ipvt=new int [neq];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,neq,neq,ipvt);
    if(pinfo!=0)
    {
	java.util.Arrays.fill(r,Double.NaN);
    }
    else
    {
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq,neq,ipvt,r,1);
	for(int k=0;k<neq;k++){r[k]/=norma[k];}
    }
    return r;
}

/** Least squares approximation. Uses polynomial basis. Several right parts.
 */
static double [][] getLinsysSolEqs(final int neq,
					  final double [] mom,final int mom_offset,
					  final double [][] rparts)
{
    final double [] matr=new double [neq*neq],norma=new double [neq];
    final double [][] rr=new double [rparts.length][neq];
    for(int k=0;k<neq;k++)
    {
	norma[k]=Math.sqrt(mom[mom_offset+2*k]);
	for(int j=0;j<rr.length;j++){rr[j][k]=rparts[j][k]/norma[k];}
	matr[k+neq*k]=1;
	for(int l=0;l<k;l++)
	{
	    matr[k+neq*l]=matr[l+neq*k]=mom[mom_offset+k+l]/norma[l]/norma[k];
	}
    }
    final int [] ipvt=new int [neq];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,neq,neq,ipvt);
    if(pinfo!=0)
    {
	for(int j=0;j<rr.length;j++){java.util.Arrays.fill(rr[j],Double.NaN);}
	return rr;
    }
    else
    {
	for(int j=0;j<rr.length;j++)
	{
	    final double [] r=rr[j];
	    com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq,neq,ipvt,r,1);
	    for(int k=0;k<neq;k++){r[k]/=norma[k];}
	}
    }
    return rr;
}


/** Like getLinsysSol but uses orthogonal polynomials basis.
 *  The basis is presented as an array of regular polynomilas (monomial basis) in BASIS.
 */
static double [] getLinsysSolPBasis(final int neq,
					    final double [] mom,final int mom_offset,final int mom_increment,
					    final double [] rpart,final int rpart_offset,final int rpart_increment,
					    final double [][] BASIS)
{
    final double [] r=new double [neq],matr=new double [neq*neq];
    for(int k=0;k<neq;k++)
    {
	r[k]=rpart[rpart_offset+k*rpart_increment];
	for(int l=0;l<=k;l++)
	{
	    // When we cannot calculate conversion coefficients exactly (I just have no time for this),
	    // use synthetic division. May be numerically unstable.
	    final double [] mulexpansion=TMatr.expandOverLowerDiagBasis(com.polytechnik.utils.Polynomials.mult2Pol(BASIS[k],BASIS[l]),BASIS);
	    double s=0;
	    for(int m=k+l;m>=0;m--)
	    {
		s+=mulexpansion[m]*mom[mom_offset+m*mom_increment];
	    }
	    matr[k+neq*l]=matr[l+neq*k]=s;
	    //System.err.println("matr["+k+"]["+l+"]="+s+" mom_offset="+mom_offset+" mom_increment="+mom_increment+" rpart_offset="+rpart_offset+" rpart_increment="+rpart_increment);
	}
    }
    final int [] ipvt=new int [neq];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,neq,neq,ipvt);
    if(pinfo!=0)
    {
	java.util.Arrays.fill(r,Double.NaN);
    }
    else
    {
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq,neq,ipvt,r,1);
    }
    return r;
}

/** Deprecated code moved out of LegendreShifted.
 */
static double [] getXPnprimeplusPnMoments(final double [] Pcoefs)
{
    final int n=Pcoefs.length-1;
    final double [] d=new double [n+1];
    double s=0;
    for(int k=0;k<=n;++k)
    {
	final double x;
	d[k]=s*(2*k+1)+(k+1)*(x=Pcoefs[k]);
	s+=x;
    }
    return d;
}

/** Calculate basis QQt matrQ basis^T
 */
static double [] convertOperatorFromQtoB_Basis(final int n,final double [] matrQ,final double [] QQt,final double [] basis,final int basis_offset)
{
    if(n*n!=matrQ.length||matrQ.length!=QQt.length||basis_offset+n*n!=basis.length) throw new IllegalArgumentException();
    final double [] tmp=new double [n*n],res=new double [n*n];
    // matrQ: n x n matrix in original Q basis.
    DeprecatedHelperTMatr.multiply_MA_x_MBTrans(tmp,n,matrQ,n,0,n,basis,n,n,basis_offset+0*n);
    DeprecatedHelperTMatr.multiply_MA_x_MB(res,n,QQt,n,0,n,tmp,n,n,0);
    DeprecatedHelperTMatr.multiply_MA_x_MB(tmp,n,basis,n,basis_offset+0*n,n,res,n,n,0);
    System.arraycopy(tmp,0,res,0,n*n);
    return res;
}

static double [] convertQX0FromQtoB_Basis(final int n,final double [] QX0,final double [] eVB)
{
    if(QX0.length!=n || n+n*n!=eVB.length) throw new IllegalArgumentException();
    final double [] BX0=new double [n];
    DeprecatedHelperTMatr.multiply_MA_x_MB(BX0,n,eVB,n,n+0*n,n,QX0,1,1,0);
    return BX0;
}

/** First order perturbation theory to psi_{ind_lev}.
 *  @return sum_{k=0;k!=ind_lev}^{k=neq-1} {@literal |psi_k><psi_k|QQ|psi_{ind_lev}>}/(ev[ind_lev]-ev[k])
 */
static double [] getPsiPertr(final EVXData e,final int ind_lev,final double [] QQ)
{
    if(ind_lev<0||ind_lev>=e.neq||QQ.length!=e.neq*e.neq)
	throw new RuntimeException("ind_lev="+ind_lev+" neq="+e.neq+" QQ.length="+QQ.length);
    final double [] dpsi=new double [e.neq];
    for(int k=0;k<e.neq;k++)
    {
	if(k!=ind_lev)
	{
	    final double s=e.getFPsiPsi(ind_lev,k,QQ)/(e.ev[ind_lev]-e.ev[k]);
	    DeprecatedHelperTMatr.sumP1(dpsi,e.ev,e.neq+k*e.neq,e.neq,s);
	}
    }
    return dpsi;
}


/** Calculates Perturbation term
sum_k {@literal <psi_{ind_lev}|VQQ|psi_k><psi_{ind_lev}|FQQ|psi_k>}/(ev[ind_lev]-ev[k])
corresponding to {@literal <psi_{ind_lev}|FQQ|var psi_{ind_lev}>} term when the hamiltonian
perturbation is V.
 */
static double getPertr1Term(final EVXData e,final int ind_lev,final double [] VQQ,final double [] FQQ)
{
    if(FQQ.length!=e.neq*e.neq) throw new IllegalArgumentException();
    double s=0;
    for(int k=0;k<e.neq;k++)
    {
	if(k!=ind_lev)
	{
	    s+=e.getFPsiPsi(ind_lev,k,VQQ)*e.getFPsiPsi(k,ind_lev,FQQ)/(e.ev[ind_lev]-e.ev[k]);
	}
    }
    return s;
}


/** Test orthogonality unit test, skipping i=0 eigenvector.
 */
static void test_orthogonalitySKIP0I(
    final String txt,
    final int n,final double [] basis,final double [] QQi,final double [] QQt,
    final double eps)
{
    final boolean skip_I0_orthogonality_test=true;
    for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,basis,n);
	for(int l=0;l<n;l++)
	{
	    final double [] psil=EVXData.getPsiFromBasis(l,basis,n);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psik,psil,QQt,n),"<psik|psik>",l==k?1:0,l==k?"1":"0",eps);
	    if(k==l)
		UnitTests.checkDoubleRel(TMatr.getAMatrB(psik,psil,QQi,n),"<psik|I|psik>",basis[k],"basis[k]",eps);
	    else if(!skip_I0_orthogonality_test || (k!=0 && l!=0))
		UnitTests.checkDoubleRel(TMatr.getAMatrB(psik,psil,QQi,n),"<psik|I|psil>",0,"0",eps);
	}
    }
    System.err.println(txt+":test_orthogonality"+(skip_I0_orthogonality_test?" With 0-th element skipped":"")+" test OK n="+n+" eps="+eps);
}



} // END OF class DeprecatedHelperEVXData
