package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.Laguerre;
import com.polytechnik.freemoney.*;

/** Laguerre basis.
 *  @version $Id: DeprecatedHelperLaguerreBasis.java,v 1.21 2022/07/10 10:59:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DeprecatedHelperLaguerreBasis implements DeprecatedHelperBasisInterface {

final QVMDataL odata;
DeprecatedHelperLaguerreBasis(final QVMDataL odata)
{
    this.odata=odata;
}

public int get_ntm(){return odata.ntm;}
public int get_npm(){return odata.npm;}
public double getPOffset(){return odata.p_offset;}
public double getPLast(){return odata.trade_last==null?Double.NaN:odata.trade_last.price;}
public double get_o_o(){return get_ntm();}
public double get_f_o(final double [] f)
{
    if(f.length!=get_ntm()) throw new RuntimeException();
    return WIntegratorLaguerre.B.getD0(f,0);
}
public double get_f_g(final double [] f,final double [] g)
{
    if(f.length!=get_ntm()) throw new RuntimeException();
    return WIntegratorLaguerre.B.get2LaguerreAveraged(f,g);
}
public void setIPn(final double [] i,final int n)
{
    if(n>=odata.npm) throw new IllegalArgumentException("n="+n+" odata.npm="+odata.npm);
    final int neq_t=get_ntm();
    for(int k=0,o=n;k<neq_t;k++,o+=odata.npm)
    {
	i[k]=odata.vmoments_Ltau_x_p[o]/odata.vmoments_Ltau_x_p[0];
    }
}
public void setDPn(final double [] pt,final int n)
{
    if(n>=odata.npm) throw new IllegalArgumentException("n="+n+" odata.npm="+odata.npm);
    final int neq_t=get_ntm();
    if(n<=0)
    { // delta function
	for(int k=0;k<neq_t;k++){pt[k]=1;}
    }
    else
    {
	final double pl=odata.trade_last!=null?odata.trade_last.price:Double.NaN,poffs=odata.p_offset;
	if(false && pl!=poffs) throw new RuntimeException("I have no time to convert basis to non-default offset; pl="+pl+" poffs="+poffs);
	final double dpln=Math.pow(pl-poffs,n);
	for(int k=0,o=n;k<neq_t;k++,o+=odata.npm){pt[k]=dpln-odata.pmoments_Ltau_x_p[o];}
    }
}
static void semDPnInternal(final double [] pt,final int n,final int neq_t,final QVMDataL odata,final double [] abc)
{
    if(n<=0){for(int k=0;k<neq_t;k++){pt[k]=1;}return;}
    final double pl=odata.trade_last!=null?odata.trade_last.price:Double.NaN,poffs=odata.p_offset;
    final double dpln=Math.pow(pl-poffs,n);
    for(int k=0,o=n;k<neq_t;k++,o+=odata.npm){pt[k]=dpln-abc[o];}
}
public void setIBS(final double [] i)
{
    if(get_npm()<2) throw new RuntimeException("npm="+get_npm());
    final int neq_t=get_ntm();
    double siplus=0,siminus=0,splusplus=0,sminusminus=0;
    for(int k=0,o=0;k<neq_t;k++,o+=odata.npm)
    {
	final double ik=odata.vmoments_Ltau_x_p[o]/odata.vmoments_Ltau_x_p[0],
	    plus=odata.pluspmoments_Ltau_x_p[1+o],
	    minus=odata.minuspmoments_Ltau_x_p[1+o];
	siplus+=ik*plus;
	siminus+=ik*minus;
	splusplus+=plus*plus;
	sminusminus+=minus*minus;
    }
    siplus/=splusplus;
    siminus/=sminusminus;
    for(int k=0,o=0;k<neq_t;k++,o+=odata.npm)
    {
	i[k]=siplus*odata.pluspmoments_Ltau_x_p[1+o]-siminus*odata.minuspmoments_Ltau_x_p[1+o];
    }
}
public void setI(final double [] i)
{
    final int neq_t=get_ntm();
    for(int k=0,o=0;k<neq_t;k++,o+=odata.npm)
    {
	i[k]=odata.vmoments_Ltau_x_p[o]/odata.vmoments_Ltau_x_p[0];
    }
}

public static DeprecatedHelperLaguerreBasis calculateBasis(final int ntm,
					   final int npm,
					   final long currenttime,
					   final long tau,
					   final com.polytechnik.realtimebook.Trade [] tb,
					   final int tsize,
					   final DeprecatedHelperLaguerreBasis ref)
{
    final QVMDataL o=QVMDataL.calculateQVMDataL(ntm,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new DeprecatedHelperLaguerreBasis(o);
}

} // END OF class DeprecatedHelperLaguerreBasis
