package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.freemoney.*;

/** LegendreShifted basis.
 *  @version $Id: DeprecatedHelperLegendreShiftedBasis.java,v 1.22 2022/07/10 10:59:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DeprecatedHelperLegendreShiftedBasis implements DeprecatedHelperBasisInterface {

final QVMDataP odata;
DeprecatedHelperLegendreShiftedBasis(final QVMDataP odata)
{
    this.odata=odata;
}

public int get_ntm(){return odata.ntm;}
public int get_npm(){return odata.npm;}
public double getPOffset(){return odata.p_offset;}
public double getPLast(){return odata.trade_last==null?Double.NaN:odata.trade_last.price;}
public double get_o_o(){return Math.pow(get_ntm(),2);}
public double get_f_o(final double [] f)
{
    if(f.length!=get_ntm()) throw new RuntimeException();
    return WIntegratorLegendreShifted.B.getD0(f,1);
}
public double get_f_g(final double [] f,final double [] g)
{
    if(get_ntm()!=g.length) throw new RuntimeException();
    return WIntegratorLegendreShifted.B.get2LegendreAveraged(f,g);
}
public void setIPn(final double [] i,final int n)
{
    if(n>=odata.npm) throw new IllegalArgumentException("n="+n+" odata.npm="+odata.npm);
    final int neq_t=get_ntm();
    for(int k=0,o=n;k<neq_t;k++,o+=odata.npm)
    {
	i[k]=(2*k+1)*(odata.vmoments_Ptau_x_p[o]/odata.vmoments_Ptau_x_p[0]);
    }
}
public void setDPn(final double [] pt,final int n)
{
    if(n>=odata.npm) throw new IllegalArgumentException("n="+n+" odata.npm="+odata.npm);
    final int neq_t=get_ntm();
    if(n<=0)
    { // delta function
	for(int k=0;k<neq_t;k++){pt[k]=2*k+1;}
    }
    else
    {
	final double pl=odata.trade_last!=null?odata.trade_last.price:Double.NaN,poffs=odata.p_offset;
	if(false && pl!=poffs) throw new RuntimeException("I have no time to convert basis to non-default offset; pl="+pl+" poffs="+poffs);
	final double dpln=Math.pow(pl-poffs,n);
	for(int k=0,o=n;k<neq_t;k++,o+=odata.npm){pt[k]=(2*k+1)*(dpln-odata.pmoments_Ptau_x_p[o]);}
    }
}
static void semDPnInternal(final double [] pt,final int n,final int neq_t,final QVMDataP odata,final double [] abc)
{
    if(n<=0){for(int k=0;k<neq_t;k++){pt[k]=2*k+1;}return;}
    final double pl=odata.trade_last!=null?odata.trade_last.price:Double.NaN,poffs=odata.p_offset;
    final double dpln=Math.pow(pl-poffs,n);
    for(int k=0,o=n;k<neq_t;k++,o+=odata.npm){pt[k]=(2*k+1)*(dpln-abc[o]);}
}
public void setIBS(final double [] i)
{
    if(get_npm()<2) throw new RuntimeException("npm="+get_npm());
    final int neq_t=get_ntm();
    double siplus=0,siminus=0,splusplus=0,sminusminus=0;
    for(int k=0,o=0;k<neq_t;k++,o+=odata.npm)
    {
	final double ik=(2*k+1)*odata.vmoments_Ptau_x_p[o]/odata.vmoments_Ptau_x_p[0],
	    plus=odata.pluspmoments_Ptau_x_p[1+o],
	    minus=odata.minuspmoments_Ptau_x_p[1+o];
	siplus+=ik*plus;
	siminus+=ik*minus;
	splusplus+=(2*k+1)*(plus*plus);
	sminusminus+=(2*k+1)*(minus*minus);
    }
    siplus/=splusplus;
    siminus/=sminusminus;
    for(int k=0,o=0;k<neq_t;k++,o+=odata.npm)
    {
	i[k]=(2*k+1)*(siplus*odata.pluspmoments_Ptau_x_p[1+o]-siminus*odata.minuspmoments_Ptau_x_p[1+o]);
    }
}
public void setI(final double [] i)
{
    final int neq_t=get_ntm();
    for(int k=0,o=0;k<neq_t;k++,o+=odata.npm)
    {
	i[k]=(2*k+1)*(odata.vmoments_Ptau_x_p[o]/odata.vmoments_Ptau_x_p[0]);
    }
}

public static DeprecatedHelperLegendreShiftedBasis calculateBasis(final int ntm,
						  final int npm,
						  final long currenttime,
						  final long tau,
						  final com.polytechnik.realtimebook.Trade [] tb,
						  final int tsize,
						  final DeprecatedHelperLegendreShiftedBasis ref)
{
    final QVMDataP o=QVMDataP.calculateQVMDataP(ntm,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new DeprecatedHelperLegendreShiftedBasis(o);
}
} // END OF class DeprecatedHelperLegendreShiftedBasis
