package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.outils.*;
import com.polytechnik.realtimebook.*;

/** Extension of Rurik: Raiser &amp; Vidka.
 *  @version $Id: DiQVM.java,v 1.42 2022/08/08 20:10:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class DiQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
final EVXData I,RI,RItau,dRI,Rdp,dRdp,Rvs,dRvs,dRdpv,dRvsv,RI_QQv,RvT_QQt,RtT_QQt;
final double c_RI_Rdp_L,c_RI_Rdp_H,c_RI_Rdp_0,
    c_RI_Rvs_L,c_RI_Rvs_H,c_RI_Rvs_0,
    c_RI_dRdp_L,c_RI_dRdp_H,c_RI_dRdp_0,
    c_RI_dRvs_L,c_RI_dRvs_H,c_RI_dRvs_0;
final double RI_psiH,dRI_psiH,Rdp_psiH,dRdp_psiH,Rvs_psiH,dRvs_psiH,
    cx_RI_Rdp_H,cx_RI_dRdp_H,cx_RI_Rvs_H,cx_RI_dRvs_H;
final double Rdx_H,dRdx_H,cX_RI_dRdp_H;
final double pt0,ptH,ptL,pv0,pvH,pvL;
final TwoCloseRootsFinder.RootsData r_mdp0,r_mvdp0;

DiQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();
    this.M=getABasis();
    final int nmoms=2*neq-1;
    final double [] imom=new double [nmoms],tmom=new double [nmoms],
	dpmom=new double [nmoms],
	vsmom=new double [nmoms],
	ptmom=new double [nmoms],pvmom=new double [nmoms];
    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setM(vsmom,0,access.M_VS,odata,nmoms);
    access.setM(ptmom,1,access.M_T,odata,nmoms);
    access.setM(pvmom,1,access.M_I,odata,nmoms);

    final double x0=getX0();
    final double [] QQi=M.B.getQQMatr(neq,imom),QQt=M.B.getQQMatr(neq,tmom);
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQi,QQt,access,x0);

    final double [] Rtmom=M.getRadauMomentsFromMoments(tmom),
	Rimom=M.getRadauMomentsFromMoments(imom),
	dRimom=getDRadauMomentsFromMoments(imom),
	Rdpmom=M.getRadauMomentsFromMoments(dpmom),
	dRdpmom=getDRadauMomentsFromMoments(dpmom),
	Rvsmom=M.getRadauMomentsFromMoments(vsmom),
	dRvsmom=getDRadauMomentsFromMoments(vsmom),
	Rptmom=M.getRadauMomentsFromMoments(ptmom),
	Rpvmom=M.getRadauMomentsFromMoments(pvmom);

    final int nr=neq-1;
    final double [] RQQt=M.B.getQQMatr(nr,Rtmom),
	nrQQi=M.B.getQQMatr(nr,imom),
	RQQi=M.B.getQQMatr(nr,Rimom),dRQQi=M.B.getQQMatr(nr,dRimom),
	RQQdp=M.B.getQQMatr(nr,Rdpmom),RQQvs=M.B.getQQMatr(nr,Rvsmom),
	dRQQdp=M.B.getQQMatr(nr,dRdpmom),dRQQvs=M.B.getQQMatr(nr,dRvsmom),
	RQQpt=M.B.getQQMatr(nr,Rptmom),RQQpv=M.B.getQQMatr(nr,Rpvmom);

    final double [] Vmom=new double [nmoms];
    access.setFMoments(Vmom,imom,nmoms); for(int k=0;k<nmoms;k++) Vmom[k]=-Vmom[k];


    this.RvT_QQt=DeprecatedHelperEVXData.getEVXDataFromQQ(nr,M.B.getQQMatr(nr,Vmom),M.B.getQQMatr(nr,tmom),access,x0);
    this.RI_QQv=DeprecatedHelperEVXData.getEVXData(nr,M.B.getXQkMomentsFromQkMoments(imom),imom,access,x0);
    this.RtT_QQt=DeprecatedHelperEVXData.getEVXData(nr,M.B.getXQkMomentsFromQkMoments(tmom),tmom,access,x0);
    this.RI=DeprecatedHelperEVXData.getEVXDataFromQQ(nr,RQQi,RQQt,access,x0);
    this.RItau=DeprecatedHelperEVXData.getEVXDataFromQQ(nr,nrQQi,RQQt,access,x0);
    this.dRI=DeprecatedHelperEVXData.getEVXDataFromQQ(nr,dRQQi,RQQt,access,x0);
    this.Rdp=DeprecatedHelperEVXData.getEVXDataFromQQ(nr,RQQdp,RQQt,access,x0);
    this.dRdp=DeprecatedHelperEVXData.getEVXDataFromQQ(nr,dRQQdp,RQQt,access,x0);
    this.Rvs=DeprecatedHelperEVXData.getEVXDataFromQQ(nr,RQQvs,RQQt,access,x0);
    this.dRvs=DeprecatedHelperEVXData.getEVXDataFromQQ(nr,dRQQvs,RQQt,access,x0);
    this.dRdpv=DeprecatedHelperEVXData.getEVXDataFromQQ(nr,dRQQdp,RQQi,access,x0);
    this.dRvsv=DeprecatedHelperEVXData.getEVXDataFromQQ(nr,dRQQvs,RQQi,access,x0);

    final double [] IRQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(nr,nr,RQQt);

    c_RI_Rdp_L=BiQVM.getACORR(nr,RQQdp,RQQi,IRQQt,RI.psiL);
    c_RI_Rdp_H=BiQVM.getACORR(nr,RQQdp,RQQi,IRQQt,RI.psiH);
    c_RI_Rdp_0=BiQVM.getACORR(nr,RQQdp,RQQi,IRQQt,RI.psi0);
    c_RI_Rvs_L=BiQVM.getACORR(nr,RQQvs,RQQi,IRQQt,RI.psiL);
    c_RI_Rvs_H=BiQVM.getACORR(nr,RQQvs,RQQi,IRQQt,RI.psiH);
    c_RI_Rvs_0=BiQVM.getACORR(nr,RQQvs,RQQi,IRQQt,RI.psi0);
    c_RI_dRdp_L=BiQVM.getACORR(nr,dRQQdp,RQQi,IRQQt,RI.psiL);
    c_RI_dRdp_H=BiQVM.getACORR(nr,dRQQdp,RQQi,IRQQt,RI.psiH);
    c_RI_dRdp_0=BiQVM.getACORR(nr,dRQQdp,RQQi,IRQQt,RI.psi0);
    c_RI_dRvs_L=BiQVM.getACORR(nr,dRQQvs,RQQi,IRQQt,RI.psiL);
    c_RI_dRvs_H=BiQVM.getACORR(nr,dRQQvs,RQQi,IRQQt,RI.psiH);
    c_RI_dRvs_0=BiQVM.getACORR(nr,dRQQvs,RQQi,IRQQt,RI.psi0);

    final double p_offset=getPOffset();

    ptH=p_offset+TMatr.getAMatrB(RI.psiH,RI.psiH,RQQpt,nr);
    ptL=p_offset+TMatr.getAMatrB(RI.psiL,RI.psiL,RQQpt,nr);
    pt0=p_offset+TMatr.getAMatrB(RI.psi0,RI.psi0,RQQpt,nr);
    pvH=p_offset+TMatr.getAMatrB(RI.psiH,RI.psiH,RQQpv,nr)/RI.sH;
    pvL=p_offset+TMatr.getAMatrB(RI.psiL,RI.psiL,RQQpv,nr)/RI.sL;
    pv0=p_offset+TMatr.getAMatrB(RI.psi0,RI.psi0,RQQpv,nr)/RI.s0;

    RI_psiH=TMatr.getAMatrA(RI.psiH,RQQi,nr);
    dRI_psiH=TMatr.getAMatrA(RI.psiH,dRQQi,nr);
    Rdp_psiH=TMatr.getAMatrA(RI.psiH,RQQdp,nr);
    dRdp_psiH=TMatr.getAMatrA(RI.psiH,dRQQdp,nr);
    Rvs_psiH=TMatr.getAMatrA(RI.psiH,RQQvs,nr);
    dRvs_psiH=TMatr.getAMatrA(RI.psiH,dRQQvs,nr);

    final double [] fQQx=new double [nr*nr];
    for(int k=fQQx.length;--k>=0;){fQQx[k]=RQQdp[k]-RQQi[k]*(Rdp_psiH/RI_psiH);}
    cx_RI_Rdp_H=BiQVM.getACORR(nr,fQQx,RQQi,IRQQt,RI.psiH);
    for(int k=fQQx.length;--k>=0;){fQQx[k]=dRQQdp[k]-dRQQi[k]*(Rdp_psiH/RI_psiH);}
    cx_RI_dRdp_H=BiQVM.getACORR(nr,fQQx,RQQi,IRQQt,RI.psiH);
    for(int k=fQQx.length;--k>=0;){fQQx[k]=RQQvs[k]-RQQi[k]*(Rvs_psiH/RI_psiH);}
    cx_RI_Rvs_H=BiQVM.getACORR(nr,fQQx,RQQi,IRQQt,RI.psiH);
    for(int k=fQQx.length;--k>=0;){fQQx[k]=dRQQvs[k]-dRQQi[k]*(Rvs_psiH/RI_psiH);}
    cx_RI_dRvs_H=BiQVM.getACORR(nr,fQQx,RQQi,IRQQt,RI.psiH);

    final double [] dxmom=new double [nmoms];
    for(int k=dxmom.length;--k>=0;) {dxmom[k]=dpmom[k]-(Rdp_psiH/RI_psiH)*imom[k];}
    final double [] Rdxmom=M.getRadauMomentsFromMoments(dxmom),
	dRdxmom=getDRadauMomentsFromMoments(dxmom);
    final double [] RQQdx=M.B.getQQMatr(nr,Rdxmom),dRQQdx=M.B.getQQMatr(nr,dRdxmom);
    Rdx_H=TMatr.getAMatrA(RI.psiH,RQQdx,nr);
    dRdx_H=TMatr.getAMatrA(RI.psiH,dRQQdx,nr);
    cX_RI_dRdp_H=BiQVM.getACORR(nr,dRQQdx,RQQi,IRQQt,RI.psiH);


    if(true)
    {
	final double [] tmp=new double [nr*nr];
	final double [] KRQQdp=ZQVM.getIQQ_F_IQQ(nr,IRQQt,RQQdp,tmp),
	    KRQQi=ZQVM.getIQQ_F_IQQ(nr,IRQQt,RQQi,tmp),
	    KdRQQdp=ZQVM.getIQQ_F_IQQ(nr,IRQQt,dRQQdp,tmp),
	    KRQQiGm1QQdp=ZQVM.getIQQ_F_IQQ(nr,IRQQt,TMatr.mulMMM(nr,RQQi,IRQQt,RQQdp),tmp);
	final double [][] kkk=access.getKK(nr,new double [][] {IRQQt,KRQQdp,KdRQQdp,KRQQiGm1QQdp,KRQQi});
	final double [] rkkt=kkk[0],rkkdp=kkk[1],rdkkdp=kkk[2],rkkvdp=kkk[3],rkki=kkk[4];
	r_mdp0=getTRD0(rkkt,rkkdp);
	r_mvdp0=getTRD0(rkkt,rkkvdp);

    }


    if(true)
    {
	simom=imom;
	stmom=tmom;
	sdpmom=dpmom;
	svsmom=vsmom;
	sdxmom=dxmom;
    }
}

/** For unit tests. */
final double [] simom,stmom,sdpmom,svsmom,sdxmom;

abstract TwoCloseRootsFinder.RootsData getTRD0(final double [] kkt,final double [] kkf);
abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getX0();
abstract double getPOffset();
abstract double [] getDRadauMomentsFromMoments(final double [] mom);


static abstract class DiQVMTest<DTYPE> extends QTest {

abstract DiQVM<DTYPE> getDiQVM();
abstract double [][] getMonomialMoments(final DTYPE o);
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getPolynomialBasis();
DiQVMTest(final String name,final java.util.Random r,final double eps)
{
    super(name,r,eps);
}

void test_Common()
{
    final DiQVM<DTYPE> q=getDiQVM();
    UnitTests.checkDoubleRel(q.Rdx_H,"q.Rdx_H",0,"0",eps);
    UnitTests.checkDoubleRel(q.dRdx_H,"q.dRdx_H",q.dRdp_psiH-q.dRI_psiH*(q.Rdp_psiH/q.RI_psiH),"dRdp-a*dRI",eps);
    UnitTests.checkDoubleRel(q.RI_psiH,"q.RI_psiH",q.RI.sH,"q.RI.sH",eps);
    UnitTests.checkDoubleRel(q.cX_RI_dRdp_H,"q.cX_RI_dRdp_H",q.cx_RI_dRdp_H,"q.cx_RI_dRdp_H",eps);

    System.err.println(name+":DiQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}

final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=getPolynomialBasis();


abstract double [] getDFCoefsPolynomials(final double [] coefs);
double [] getDRadauMomentsFromMoments(final double [] mom)
{
    final int n=mom.length-1;
    //System.err.println("n="+n+" mom="+java.util.Arrays.toString(mom));
    final double [] res=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] tmp=new double [k+1];
	tmp[k]=1;
	final double [] dm=getDFCoefsPolynomials(tmp);
	res[k]=TMatr.mulVVn1(dm,mom);
	//System.err.println("k="+k+" resk="+res[k]+" dm="+java.util.Arrays.toString(dm));

    }
    return res;
}

void test_RadauBasis(final DiQVM<DTYPE> q,final double [] mom_basis,final double [] mom_polynomial)
{
    UnitTests.checkDoubleArrayRel(getMonomialMomentsConvertedFromBasis(mom_basis),"conv(mom_basis)",mom_polynomial,"mom_polynomial",eps);
    UnitTests.checkDoubleArrayRel(getMonomialMomentsConvertedFromBasis(q.M.getRadauMomentsFromMoments(mom_basis)),"conv(Radau mom_basis)",M.getRadauMomentsFromMoments(mom_polynomial),"Radau mom_polynomial",eps);
    UnitTests.checkDoubleArrayRel1(getMonomialMomentsConvertedFromBasis(q.getDRadauMomentsFromMoments(mom_basis)),"conv(dRadau mom_basis)",getDRadauMomentsFromMoments(mom_polynomial),"dRadau mom_polynomial",eps);

    System.err.println(name+":DiQVMTest:test_RadauBasis(): test OK neq="+neq+" eps="+eps);
}


void test_Basis()
{
    final DiQVM<DTYPE> q=getDiQVM();
    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] mom_t=amoms[0],mom_v=amoms[1],mom_dp=amoms[2],mom_vs=amoms[3];
    test_RadauBasis(q,q.stmom,mom_t);
    final double [] m0=new double [q.stmom.length-1];
    UnitTests.checkDoubleArrayRel(q.getDRadauMomentsFromMoments(q.stmom),"dRadau q.stmom",m0,"m0",eps);
    UnitTests.checkDoubleArrayRel(getDRadauMomentsFromMoments(mom_t),"dRadau mom_t",m0,"m0",eps);

    test_RadauBasis(q,q.simom,mom_v);
    test_RadauBasis(q,q.sdpmom,mom_dp);
    test_RadauBasis(q,q.svsmom,mom_vs);

    System.err.println(name+":DiQVMTest:test_Basis(): test OK neq="+neq+" eps="+eps);
}



void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":DiQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Basis();

    System.err.println(name+":DiQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":DiQVMTest: starting tests; ver=$Id: DiQVM.java,v 1.42 2022/08/08 20:10:46 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":DiQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: DiQVM.java,v 1.42 2022/08/08 20:10:46 mal Exp $");
}

} // END OF class DiQVMTest


}
