package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.outils.*;
import com.polytechnik.realtimebook.*;

public class DiQVMLaguerre extends DiQVM<QVMDataL> {
static final Laguerre B=new Laguerre();

DiQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static DiQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final DiQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new DiQVMLaguerre(neq,o);
}
TwoCloseRootsFinder.RootsData getTRD0(final double [] kkt,final double [] kkf)
    {return TwoCloseRootsFinder.findRootsData(getX0(),2.1,3000,new TwoRootsNoDiv0(kkt,kkf,M));}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
double [] getDRadauMomentsFromMoments(final double [] mom)
{
    final int n=mom.length-1;
    final double [] res=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] tmp0=new double [k+1];tmp0[k]=1;
	final double [] tmp=B.getxP(tmp0);
	final double [] tmp2=B.getDifferentiated(tmp);
	TMatr.sumP1(tmp,tmp2,-1);
	res[k]=TMatr.mulVVn1(tmp,mom);
    }
    return res;
}

static class DiQVMLaguerreTest extends DiQVMTest<QVMDataL> {
DiQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
DiQVMLaguerre getDiQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
double [] getDFCoefsPolynomials(final double [] coefs)
{
    final double [] tq=Polynomials.mult2Pol(coefs,new double [] {0,1});
    TMatr.sumP1(tq,Polynomials.getPolynomialDifferentiated(tq),-1);
    return tq;
}
OrthogonalPolynomialsMonomialsBasis getPolynomialBasis()
{
    return new OrthogonalPolynomialsMonomialsBasis(){
	@Override
	public double [] getRadauMultiplied(final double [] nodes0)
	{
	    return Polynomials.getxP(nodes0);
	}
    };
}

QVMData getPData(final int nmoms)
{
    final QVMData qdata=QVMData.calculateQVMData(2,nmoms,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
    return qdata;
}

double [][] getMonomialMoments(final QVMDataL odata)
{
    final int nmoms=2*neq-1;
    final QVMData qd=getPData(nmoms);
    final double []
	imoms=MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,odata.p_offset),
	dpmoms=MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_DP,odata.p_offset);

    return new double [][]{
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomT(nmoms),
	imoms,
	dpmoms,
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_VS,odata.p_offset)
    };
}

} // END of class DiQVMLaguerreTest

public static void main(String [] args)
{
    new DiQVMLaguerreTest("DiQVMLaguerre").runTests(5);
} // END main



} // END OF class DiQVMLaguerre
