package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.outils.*;
import com.polytechnik.realtimebook.*;

public class DiQVMLegendreShifted extends DiQVM<QVMDataP> {
static final LegendreShifted B=new LegendreShifted();
DiQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static DiQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final DiQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new DiQVMLegendreShifted(neq,o);
}
TwoCloseRootsFinder.RootsData getTRD0(final double [] kkt,final double [] kkf)
    {return TwoCloseRootsFinder.findRootsData(getX0(),0.5,3000,new TwoRootsNoDiv0(kkt,kkf,M));}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
double getX0(){return 1;}
double getPOffset(){return odata.p_offset;}
double [] getDRadauMomentsFromMoments(final double [] mom)
{
    final int n=mom.length-1;
    final double [] res=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] tmp=new double [k+1];tmp[k]=1;
	final double [] tmp2=B.getxP(tmp);
	final double [] tmp3=B.get1mxP(tmp2);
	final double [] dm=B.getDifferentiated(tmp3);
	res[k]=TMatr.mulVVn1(dm,mom);
    }
    return res;
}

static class DiQVMLegendreShiftedTest extends DiQVMTest<QVMDataP> {
DiQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
DiQVMLegendreShifted getDiQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
double [] getDFCoefsPolynomials(final double [] coefs)
{
    return Polynomials.getPolynomialDifferentiated(Polynomials.mult2Pol(coefs,new double [] {0,1,-1}));
}
OrthogonalPolynomialsMonomialsBasis getPolynomialBasis()
{
    return new OrthogonalPolynomialsMonomialsBasis();
}
QVMData [] getPData(final int nmoms)
{
    final QVMData [] qdata=new QVMData [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(2,1+r.nextInt(3),4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}

double [][] getMonomialMoments(final QVMDataP odata)
{
    final int nmoms=2*neq-1;
    final QVMData [] qd=getPData(nmoms);
    final double [] mv=new double [nmoms],
	imoms=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,false,odata.p_offset),
	dpmoms=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_DP,false,odata.p_offset);
    return new double [][]{
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomT(nmoms),
	imoms,
	dpmoms,
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_VS,false,odata.p_offset)
    };
}

} // END of class DiQVMLegendreShiftedTest

public static void main(String [] args)
{
    new DiQVMLegendreShiftedTest("DiQVMLegendreShifted").runTests(5);
} // END main



} // END OF class DiQVMLegendreShifted
