package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.realtimebook.*;

public class DjQVMLaguerre extends DjQVM<QVMDataL> {
static final Laguerre B=new Laguerre();

DjQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static DjQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final DjQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1+1+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new DjQVMLaguerre(neq,o);
}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
boolean isEdgeAtRight(){return false;}

static class DjQVMLaguerreTest extends DjQVMTest<QVMDataL> {
DjQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
DjQVMLaguerre getDjQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
BjQVMLaguerre getBjQVM(){return BjQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
BiQVMLaguerre getBiQVM(){return BiQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
ZQVMLaguerre getZQVM(){return ZQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}

double [][] getMonomialMoments(final QVMDataL odata){return null;}

double [] getDpol(final double [] kkt,final double [] kki)
{
    final double [] Dkkt=B.getDifferentiated(kkt),
	Dkki=B.getDifferentiated(kki);
    final double [] d2=B.mult2Pol(Dkkt,kki),d1=B.mult2Pol(Dkki,kkt);
    TMatr.sumP1(d2,d1,-1);
    // the last coef is 0
    final double [] d=new double [d2.length-1];
    System.arraycopy(d2,0,d,0,d2.length-1);
    return d;
}

void test_R2R(final double [] ev)
{
    final double [] x=B.getRadauRoots(neq+1);
    for(int k=0;k<neq;k++)
    {
	UnitTests.checkDoubleRel(ev[k],"ev[k]",x[k+1],"x[k+1]",eps);
    }
    System.err.println(name+":DjQVMTest:test_R2R: test OK neq="+neq+" eps="+eps);
}

} // END of class DjQVMLaguerreTest

public static void main(String [] args)
{
    new DjQVMLaguerreTest("DjQVMLaguerre").runTests(5);
} // END main



} // END OF class DjQVMLaguerre
