package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.realtimebook.*;

public class DjQVMLegendreShifted extends DjQVM<QVMDataP> {
static final LegendreShifted B=new LegendreShifted();
DjQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static DjQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final DjQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1+1+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new DjQVMLegendreShifted(neq,o);
}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
double getX0(){return 1;}
double getPOffset(){return odata.p_offset;}
boolean isEdgeAtRight(){return true;}

static class DjQVMLegendreShiftedTest extends DjQVMTest<QVMDataP> {
DjQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
DjQVMLegendreShifted getDjQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
BjQVMLegendreShifted getBjQVM(){return BjQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
BiQVMLegendreShifted getBiQVM(){return BiQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
ZQVMLegendreShifted getZQVM(){return ZQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}

double [][] getMonomialMoments(final QVMDataP odata){return null;}

double [] getDpol(final double [] kkt,final double [] kki)
{
    final double [] Dkkt=B.getDifferentiated(kkt),Dkki=B.getDifferentiated(kki);
    final double [] d1=B.mult2Pol(Dkki,kkt),d2=B.mult2Pol(Dkkt,kki);
    TMatr.sumP1(d2,d1,-1);
    // the last coef is 0
    final double [] d=new double [d2.length-1];
    System.arraycopy(d2,0,d,0,d2.length-1);
    return d;
}

void test_R2R(final double [] ev)
{
    final double [] x=B.getRadauRoots(neq+1);
    for(int k=0;k<neq;k++)
    {
	UnitTests.checkDoubleRel(1-ev[neq-1-k],"1-ev[neq-1-k]",x[k],"x[k]",eps);
    }
    System.err.println(name+":DjQVMTest:test_R2R: test OK neq="+neq+" eps="+eps);
}

} // END of class DjQVMLegendreShiftedTest

public static void main(String [] args)
{
    new DjQVMLegendreShiftedTest("DjQVMLegendreShifted").runTests(5);
} // END main



} // END OF class DjQVMLegendreShifted
