package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class DkQVMLaguerre extends DkQVM<QVMDataL> {
static final Laguerre B=new Laguerre();

DkQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static DkQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final DkQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1+1+1+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new DkQVMLaguerre(neq,o);
}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
boolean isEdgeAtRight(){return false;}
double [] getDMatr(final int n)
{
    final double [] m=getDop(n,getBasisPolynomials());
    for(int k=m.length;--k>=0;) m[k]=-m[k];
    return m;
}
double [] getDPsi(final double [] psi)
{
    final double [] m=dx_getDop(psi,getBasisPolynomials());
    for(int k=m.length;--k>=0;) m[k]=-m[k];
    return m;
}
/** Calcucales psi/2-psi'
 */
double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}

/** Calculates psi sdiv (x-x0)=(x-x0)*q(x)+r.
 *  Then return r/2-q(x).
 */
double [] getEDPsiX(final double [] psi,final double x0)
{
    final com.polytechnik.utils.BasisFunctionsCalculatable B=getBasisPolynomials();
    double [] q=new double [Math.max(psi.length-1,1)];
    q[0]=1;
    final double b0=B.getD0(new double []{1},x0);
    final com.polytechnik.utils.SDivRes sdr=B.sdiv1(psi,x0);
    final double r=sdr.r[0];
    if(psi.length<=1) java.util.Arrays.fill(q,0);
    else q=sdr.q;
    q[0]-=(r/2)/b0;
    for(int k=q.length;--k>=0;){q[k]=-q[k];}
    return q;
}
double getIntegratedToEnd(final double x,final double [] psi)
{
    final double [] ipsi=WIntegratorLaguerre.B.getLaguerreExpFIntegrated(psi);
    return M.B.getD0(ipsi,getX0())-M.B.getD0(ipsi,x)*Math.exp(-x);
}
double getXTestEnd(final int n){return 2*n;}
double getXTestEndUnitTest(){return 130;}
Laguerre getBasisPolynomials()
{
    return new Laguerre();
}
/** Basis: (x-x0)Q_k(x)+2*Q_k(x0)
 */
double [] getBasisFuncsBoundaryCond(final int n)
{
    final double x0=getX0();
    final int nd=n-1;
    final double [] basis=new double [n*nd];
    final com.polytechnik.utils.BasisFunctionsCalculatable B=getBasisPolynomials();
    double q0=Double.NaN;
    for(int k=0;k<nd;k++)
    {
	final double [] p=new double [k+1];
	p[k]=1;
	final double qk=B.getD0(p,x0);
	if(k<=0){q0=qk;}
	final double [] pp=B.getaxbP(p,1,-x0);
	pp[0]+=2*(qk/q0);
	p[k]=0;
	System.arraycopy(pp,0,basis,k*n,k+2);
    }
    return basis;
}

static class DkQVMLaguerreTest extends DkQVMTest<QVMDataL> {
DkQVMLaguerreTest(final long initrandom)
{
    super("DkQVMLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
DkQVMLaguerre getDkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DjQVMLaguerre getDjQVM(){return DjQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
BiQVMLaguerre getBiQVM(){return BiQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
EQVMLaguerre getEQVM(){return EQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

void test_FMoments(final int nmoms)
{
    final DkQVMLaguerre q=getDkQVM();
    final double [] Tmom;
    {
    final double [] tmom=new double [nmoms];
    q.access.setMT(tmom,nmoms);
    Tmom=new double [nmoms];
    q.access.setFMoments(Tmom,tmom,nmoms);
    for(int k=Tmom.length;--k>=0;){Tmom[k]=-Tmom[k];}
    }
    final double [] YTmom;
    {
    final double [] tmom=new double [nmoms+1];
    q.access.setMT(tmom,nmoms+1);
    YTmom=q.M.getRadauMomentsFromMoments(tmom);
    }
    UnitTests.checkDoubleArrayRel(Tmom,"Tmom",YTmom,"YTmom",eps);
    System.err.println("DkQVM:WpPsi:test_FMoments: test OK nmoms="+nmoms+" eps="+eps);
}

} // END of class DkQVMLaguerreTest

public static void main(String [] args)
{
if(false)
{
    for(long rr=1;rr<10000;rr++)
    {
	try{
	new DkQVMLaguerreTest(rr).runTests(5);
	} catch(RuntimeException e) {
	    e.printStackTrace(System.err);
	    System.err.println("------------random="+rr+" failed");
	    continue;
	}
	System.err.println("rr="+rr+" OK");
	break;
    }
}
else
{
    new DkQVMLaguerreTest(1185).runTests(5);
}
} // END main



} // END OF class DkQVMLaguerre
