package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class DkQVMLegendreShifted extends DkQVM<QVMDataP> {
static final LegendreShifted B=new LegendreShifted();
DkQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static DkQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final DkQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1+1+1+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new DkQVMLegendreShifted(neq,o);
}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
double getX0(){return 1;}
double getPOffset(){return odata.p_offset;}
boolean isEdgeAtRight(){return true;}
double [] getDMatr(final int n)
{
    return getXDop(n,getBasisPolynomials());
}
double [] getDPsi(final double [] psi)
{
    return dx_getXDop(psi,getBasisPolynomials());
}
/** Calculates psi/2+x*psi'
 */
double [] getEDPsi(final double [] psi){return WIntegratorLegendreShifted.getEDPsi(psi);}

/** Calculates psi sdiv (x-x0)=(x-x0)*q(x)+r.
 *  Then return r/2+x*q(x).
 */
double [] getEDPsiX(final double [] psi,final double x0)
{
    final com.polytechnik.utils.BasisFunctionsCalculatable B=getBasisPolynomials();
    double [] xq=new double [psi.length];
    final double b0=B.getD0(new double []{1},x0);
    final com.polytechnik.utils.SDivRes sdr=B.sdiv1(psi,x0);
    final double r=sdr.r[0],q[]=sdr.q;
    if(q.length>0)xq=B.getaxbP(q,1,0);
    else java.util.Arrays.fill(xq,0);
    xq[0]+=(r/2)/b0;
    return xq;
}
double getIntegratedToEnd(final double x,final double [] psi)
{
    final double [] ipsi=WIntegratorLegendreShifted.B.getIntegrated(psi);
    return M.B.getD0(ipsi,getX0())-M.B.getD0(ipsi,x);
}
double getXTestEnd(final int n){return 0;}
double getXTestEndUnitTest(){return 0;}
LegendreShifted getBasisPolynomials()
{
    return new LegendreShifted();
}
/** Basis: (x-x0)Q_k(x)-2x0*Q_k(x0)
 */
double [] getBasisFuncsBoundaryCond(final int n)
{
    final double x0=getX0();
    final int nd=n-1;
    final double [] basis=new double [n*nd];
    final com.polytechnik.utils.BasisFunctionsCalculatable B=getBasisPolynomials();
    double q0=Double.NaN;
    for(int k=0;k<nd;k++)
    {
	final double [] p=new double [k+1];
	p[k]=1;
	final double qk=B.getD0(p,x0);
	if(k<=0){q0=qk;}
	final double [] pp=B.getaxbP(p,1,-x0);
	pp[0]-=2*x0*(qk/q0);
	p[k]=0;
	System.arraycopy(pp,0,basis,k*n,k+2);
    }
    return basis;
}
static class DkQVMLegendreShiftedTest extends DkQVMTest<QVMDataP> {
DkQVMLegendreShiftedTest(final long initrandom)
{
    super("DkQVMLegendreShifted",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
DkQVMLegendreShifted getDkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DjQVMLegendreShifted getDjQVM(){return DjQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
BiQVMLegendreShifted getBiQVM(){return BiQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
EQVMLegendreShifted getEQVM(){return EQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}

void test_FMoments(final int nmoms)
{
    final DkQVMLegendreShifted q=getDkQVM();
    final double [] Tmom;
    {
    final double [] tmom=new double [nmoms];
    q.access.setMT(tmom,nmoms);
    Tmom=new double [nmoms];
    q.access.setFMoments(Tmom,tmom,nmoms);
    for(int k=Tmom.length;--k>=0;){Tmom[k]=-Tmom[k];}
    }
    final double [] YTmom=new double [nmoms];
    {
	class Intgr implements com.polytechnik.utils.QUANC8.DoubleFunction {
	    private final double [] P;
	    Intgr(final int k)
	    {
		this.P=new double [k+1];
		this.P[k]=1;
	    }
	    public double f(final double x)
	    {
		return q.M.B.getD0(P,x)*Math.log(q.getX0()/x);
	    }
	}
	final double abserr=0,relerr=1e-8;
	final int nomax=5000000;
	for(int k=0;k<nmoms;k++)
	{
	    final com.polytechnik.utils.QUANC8 res=
		    com.polytechnik.utils.QUANC8.integrate(
			new Intgr(k),
			1e-17,
			q.getX0(),
			abserr,
			relerr,
			nomax);
	    if(res.flag!=0)
	    {
		throw new RuntimeException("flag="+res.flag+" k="+k);
	    }
	    YTmom[k]=res.integral;
	    // exact value
	    // Prudnikov, Brychkov, Marichev, volume 2, 2003,
	    // formulae 2.17.8.2 , page 389.
	    // !!! Note: looks there is an error in sign in 2.17.8.2.
	    final double resexact;
	    if(k>0) resexact=0.5*((k%2==0?1:-1)*2.0/(k*(k+1)));
	    else resexact=1;
	    UnitTests.checkDoubleRel(res.integral,"res.integral",resexact,"resexact",eps);
	    //YTmom[k]=resexact;
	}

    }
    UnitTests.checkDoubleArrayRel(Tmom,"Tmom",YTmom,"YTmom",eps);
    System.err.println("Tmom="+java.util.Arrays.toString(Tmom)+"\n"+
		       "YTmom="+java.util.Arrays.toString(YTmom));
    System.err.println("DkQVM:WpPsi:test_FMoments: test OK nmoms="+nmoms+" eps="+eps);
}

} // END of class DkQVMLegendreShiftedTest

public static void main(String [] args)
{
if(false)
{
    for(long rr=1;rr<10000;rr++)
    {
	try{
	new DkQVMLegendreShiftedTest(rr).runTests(5);
	} catch(RuntimeException e) {
	    e.printStackTrace(System.err);
	    System.err.println("------------random="+rr+" failed");
	    continue;
	}
	System.err.println("rr="+rr+" OK");
	break;
    }
}
else
{
    new DkQVMLegendreShiftedTest(16L).runTests(5);
}
} // END main



} // END OF class DkQVMLegendreShifted
