package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;

/** Book in price space.
 *  @version $Id: DoQVMLaguerre.java,v 1.2 2019/01/06 23:56:13 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DoQVMLaguerre extends DoQVM<QVMDataBL> {

DoQVMLaguerre(final int neq,final QVMDataBL odata_buy,final QVMDataBL odata_sell){super(neq,odata_buy,odata_sell);}

double getX0(){return 0;}
double getPOffset(final QVMDataBL odata){return odata.p_offset;}
double getPEdge(final QVMDataBL odata){return odata.edge!=null?odata.edge.price:Double.NaN;}
DeprecatedHelperQVMDataBLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataBLDirectAccessWithBasis();}

public static DoQVMLaguerre calculate(final int neq,
				      final long currenttime,
				      final long tau,
				      final BookItem [] buy,
				      final int buysize,
				      final BookItem [] sell,
				      final int sellsize)
{
    final int ntm=2*neq-1,npm=3;
    final QVMDataBL obuy=QVMDataBL.calculateQVMDataBL(ntm,npm,
						      currenttime,
						      tau,
						      buy,
						      buysize),
	osell=QVMDataBL.calculateQVMDataBL(ntm,npm,
					   currenttime,
					   tau,
					   sell,
					   sellsize);
    return new DoQVMLaguerre(neq,obuy,osell);
}


static class DoQVMLaguerreTest extends DoQVMTest<QVMDataBL> {
DoQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
long getTau()
{
    final long a=(tbuy[tbuy.length-1].time-tbuy[0].time),
	b=(tsell[tsell.length-1].time-tsell[0].time);
    return Math.max(a,b)/24;
}
BQVMLaguerre getBQVM()
{
    final Trade [] tb=tbuy.length>tsell.length?tbuy:tsell;
    return BQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);
}
DoQVMLaguerre getDoQVM(){return calculate(neq,currenttime,tau,tbuy_byp,tbuy_byp.length,tsell_byp,tsell_byp.length);}

} // END of class DoQVMLaguerreTest

public static void main(String [] args)
{
    new DoQVMLaguerreTest("DoQVMLaguerre").runTests(5);
} // END main



} // END of class DoQVMLaguerre
