package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;

/** Book in price space.
 *  @version $Id: DoQVMLegendreShifted.java,v 1.2 2019/01/06 23:56:13 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DoQVMLegendreShifted extends DoQVM<QVMDataBP> {

DoQVMLegendreShifted(final int neq,final QVMDataBP odata_buy,final QVMDataBP odata_sell){super(neq,odata_buy,odata_sell);}

double getX0(){return 1;}
double getPOffset(final QVMDataBP odata){return odata.p_offset;}
double getPEdge(final QVMDataBP odata){return odata.edge!=null?odata.edge.price:Double.NaN;}
DeprecatedHelperQVMDataBPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataBPDirectAccessWithBasis();}

public static DoQVMLegendreShifted calculate(final int neq,
				      final long currenttime,
				      final long tau,
				      final BookItem [] buy,
				      final int buysize,
				      final BookItem [] sell,
				      final int sellsize)
{
    final int ntm=2*neq-1,npm=3;
    final QVMDataBP obuy=QVMDataBP.calculateQVMDataBP(ntm,npm,
						      currenttime,
						      tau,
						      buy,
						      buysize),
	osell=QVMDataBP.calculateQVMDataBP(ntm,npm,
					   currenttime,
					   tau,
					   sell,
					   sellsize);
    return new DoQVMLegendreShifted(neq,obuy,osell);
}


static class DoQVMLegendreShiftedTest extends DoQVMTest<QVMDataBP> {
DoQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
long getTau()
{
    final long a=(tbuy[tbuy.length-1].time-tbuy[0].time),
	b=(tsell[tsell.length-1].time-tsell[0].time);
    return (Math.max(a,b)/3)/(9*5*7*8)*(9*5*7*8);
}
BQVMLegendreShifted getBQVM()
{
    final Trade [] tb=tbuy.length>tsell.length?tbuy:tsell;
    return BQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);
}
DoQVMLegendreShifted getDoQVM(){return calculate(neq,currenttime,tau,tbuy_byp,tbuy_byp.length,tsell_byp,tsell_byp.length);}

} // END of class DoQVMLegendreShiftedTest

public static void main(String [] args)
{
    new DoQVMLegendreShiftedTest("DoQVMLegendreShifted").runTests(5);
} // END main



} // END of class DoQVMLegendreShifted
