package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

class DynDVHPnL {

    final double p_ID,I_ID,p_IH,var_p_IH,dp_IH,I_IH;
    DynDVHPnL(final int n,
	      final double [] QQi,final double [] QQt, final double [] QQpi,
	      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
    {
	final double [] DQQi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi),
	    DQQt=VOLATILITY_EDPsi.getDiffMatrix(n,QQt),
	    DQQpi=VOLATILITY_EDPsi.getDiffMatrix(n,QQpi);
	final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M),
	    Id=EVXData.getEVXDataFromQQ(n,DQQi,QQt,M),
	    Td=EVXData.getEVXDataFromQQ(n,DQQt,QQt,M);
	final double [] IDQQi=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,DQQi);
	final double [] Q0=M.B.getX0Moments(n,M.getRadauRoot()),
	    psiID=TMatr.getMatr_x_A(n,IDQQi,Q0);
	final double lamID=1/TMatr.mulVV(psiID,Q0);
	final double nr=TMatr.getAMatrA(psiID,DQQt,n);
	I_ID=lamID;
	p_ID=TMatr.getAMatrA(psiID,DQQpi,n)/TMatr.getAMatrA(psiID,DQQi,n);

	final int iX=n-1;
	I_IH=I.ev[iX];
	p_IH=I.getFPsi(iX,QQpi)/I_ID;
	final double [] DpsiH=VOLATILITY_EDPsi.getDPsi(I.psiH);
	TMatr.sumP1(DpsiH,I.psiH,-TMatr.getAMatrB(DpsiH,I.psiH,QQt,n));
	var_p_IH=TMatr.getAMatrB(DpsiH,I.psiH,QQpi,n);
	dp_IH=var_p_IH/I.sH;
	if(false)
	{
	    System.err.println("lamID="+(lamID)+"=<psiID|DQQi|psiID>/<psiID|DQQt|psiID>="+(TMatr.getAMatrA(psiID,DQQi,n)/TMatr.getAMatrA(psiID,DQQt,n)));
	    System.err.println("DQQi |psiID>="+java.util.Arrays.toString(TMatr.getMatr_x_A(n,DQQi,psiID)));
	    System.err.println("lamD * DQQt | psiID>="+java.util.Arrays.toString(TMatr.getVectorScaled(TMatr.getMatr_x_A(n,DQQt,psiID),lamID)));
	    System.err.println("DynDVHPnKL I.ev="+java.util.Arrays.toString(I.getEValues()));
	    System.err.println("DynDVHPnKL Id.ev="+java.util.Arrays.toString(Id.getEValues()));
	    System.err.println("DynDVHPnKL Td.ev="+java.util.Arrays.toString(Td.getEValues()));
	    for(int k=0;k<n;k++)
	    {
		System.err.print("k="+k+":");
		final double [] psik=I.getPsi(k);
		for(int l=0;l<n;l++)
		{
		    final double [] psil=I.getPsi(l);
		    double f=TMatr.getAMatrB(psik,psil,DQQi,n);
		    if(Math.abs(f)<1e-7) f=0;
		    System.err.print("\t"+f);
		}
		System.err.println();
	    }
	}
    }
}
