package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

class DynDVarPnL {
    final double i_I0,i_IL,i_IH,p_IL,p_IH;
    final DD dII_L,dII_H;

    DynDVarPnL(final int n,final double [] QQt,
	       final double [] QQi,final double [] QQpi,
	       final double [] QQdi,final double [] QQdpi,
	       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final int i0=0,iL=1,iH=n-1;
	final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
	this.i_I0=basis[i0];
	this.i_IL=basis[iL];
	this.p_IL=TMatr.getAMatrA(n,basis,n+iL*n,QQpi,n)/i_IL;
	this.i_IH=basis[iH];
	this.p_IH=TMatr.getAMatrA(n,basis,n+iH*n,QQpi,n)/i_IH;

	final int nd=n-1;
	final double [] RQQi;
	if(!(i0==0)) throw new RuntimeException("Not supported i0="+i0);
	if(true) {RQQi=new double [nd*nd];for(int k=0;k<nd;k++) RQQi[k+nd*k]=basis[k+1];}
	else RQQi=EVXData.convertMatrFromQtoB_Basis(n,QQi,basis,nd,n+n+0);
	final double [] RQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis,nd,n+n+0),
	    RQQdi=EVXData.convertMatrFromQtoB_Basis(n,QQdi,basis,nd,n+n+0),
	    RQQdpi=EVXData.convertMatrFromQtoB_Basis(n,QQdpi,basis,nd,n+n+0);
	final double [] basisDII=EVXData.getSymEVDiagT(nd,RQQdi,RQQi,M.EV);
	this.dII_L=new DD(0,basisDII,nd,RQQpi,RQQdpi);
	this.dII_H=new DD(nd-1,basisDII,nd,RQQpi,RQQdpi);
    }

static class DD {
    final int ind;
    final double rate,p,pd,dif;
    DD(final int i,final double [] basis,final int n,
       final double [] QQpi,final double [] QQdpi)
    {
	this.ind=i;
	this.rate=basis[ind];
	this.p=TMatr.getAMatrA(n,basis,n+ind*n,QQpi,n);
	final double d=TMatr.getAMatrA(n,basis,n+ind*n,QQdpi,n);
	this.dif=d-p*rate;
	this.pd=d/rate;
    }
}

static void matchDynDVarPnL(final DynDVarPnL a,final DynDVarPnL b,final double eps)
{
    UnitTests.checkDoubleRel(a.i_I0,"a.i_I0",b.i_I0,"b.i_I0",eps);
    UnitTests.checkDoubleRel(a.i_IL,"a.i_IL",b.i_IL,"b.i_IL",eps);
    UnitTests.checkDoubleRel(a.p_IL,"a.p_IL",b.p_IL,"b.p_IL",eps);
    UnitTests.checkDoubleRel(a.i_IH,"a.i_IH",b.i_IH,"b.i_IH",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    for(DD [] x:new DD[][]{{a.dII_L,b.dII_L},{a.dII_H,b.dII_H}})
    {
	final DD aa=x[0],bb=x[1];
	UnitTests.checkDoubleRel(aa.rate,"aa.rate",bb.rate,"bb.rate",eps);
	UnitTests.checkDoubleRel(aa.p,"aa.p",bb.p,"bb.p",eps);
	UnitTests.checkDoubleRel(aa.pd,"aa.pd",bb.pd,"bb.pd",eps);
	UnitTests.checkDoubleRel(aa.dif,"aa.dif",bb.dif,"bb.dif",eps);
    }

    System.err.println("DynDVarPnL.matchDynDVarPnL(): test OK eps="+eps);
}

}

