package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Use <Q_k dp/dt dv/dt> moments
 *  @version $Id: DynDpDi.java,v 1.16 2020/09/27 20:33:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynDpDi {
    final double [] ev;
    final double I_0,I_L,I_H,p_IH,Dp;
    DynDpDi(final double [] ev,
	    final double I_0,final double I_L,final double I_H,final double p_IH,final double Dp)
    {
	this.ev=ev;
	this.I_0=I_0;
	this.I_L=I_L;
	this.I_H=I_H;
	this.p_IH=p_IH;
	this.Dp=Dp;
    }

static DynDpDi get0_DynDpDi(final int n,final double [] imoms,final double [] tmoms,final double [] dpmoms,final double [] ndpmoms,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final DataInterfaceFDFMoments access)
{
    final double [] QQt=M.B.getQQMatr(n,tmoms),QQi=M.B.getQQMatr(n,imoms);
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final int iL=1,iH=n-1,i0=0,nmoms=2*n-1;
    final double I_0=basis[i0],I_L=basis[iL],I_H=basis[iH];
    final double [] dpImoms=getdpImoms(nmoms,ndpmoms,tmoms,dpmoms,M,false),
	PImoms=new double [nmoms],Pmoms=new double [nmoms];
    access.setFMoments(PImoms,dpImoms,nmoms);
    access.setFMoments(Pmoms,dpmoms,nmoms);
    final double [] QQP=M.B.getQQMatr(n,Pmoms),QQPI=M.B.getQQMatr(n,PImoms);
    final double p_IH=TMatr.getAMatrA(n,basis,n+iH*n,QQP,n),Dp=TMatr.getAMatrA(n,basis,n+iH*n,QQPI,n);
    return new DynDpDi(basis,I_0,I_L,I_H,p_IH,Dp);
}

static DynDpDi getR_DynDpDi(final int nn,final double [] imoms,final double [] tmoms,final double [] dpmoms,final double [] ndpmoms,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final DataInterfaceFDFMoments access)
{
    final int n=nn-1;
    final double [] RQQt=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(tmoms)),
	RQQi=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(imoms));
    final double [] basis=M.EV.getGEV_Generalized(n,RQQi,RQQt,true);
    final int iL=0,iH=n-1,nmoms=2*n-1+1;
    final double I_0=Double.NaN,I_L=basis[iL],I_H=basis[iH];
    final double [] dpImoms=getdpImoms(nmoms,ndpmoms,tmoms,dpmoms,M,false),
	PImoms=new double [nmoms],Pmoms=new double [nmoms];
    access.setFMoments(PImoms,dpImoms,nmoms);
    access.setFMoments(Pmoms,dpmoms,nmoms);
    final double [] RQQP=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(Pmoms)),
	RQQPI=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(PImoms));
    final double p_IH=TMatr.getAMatrA(n,basis,n+iH*n,RQQP,n),Dp=TMatr.getAMatrA(n,basis,n+iH*n,RQQPI,n);
    if(false && true)
    {
	System.err.println("<psiH|RQQt|psiH>="+TMatr.getAMatrA(n,basis,n+iH*n,RQQt,n)+" must be =1");
	System.err.println("<psiH|RQQi|psiH>="+TMatr.getAMatrA(n,basis,n+iH*n,RQQi,n)+" must be ="+I_H);
    }
    return new DynDpDi(basis,I_0,I_L,I_H,p_IH,Dp);
}


    /** Calculates QQi (QQt)^{-1} <dpQ>
     */
    static double [] getdpImoms(final int n,final double [] imoms,final double [] tmoms,final double [] dpmoms,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final boolean flag_diag_QQt)
    {
	final double [] QQt=M.B.getQQMatr(n,tmoms),QQi=M.B.getQQMatr(n,imoms);
	final double [] r=new double [n];
	if(flag_diag_QQt)
	{
	    for(int k=0;k<n;k++) r[k]=dpmoms[k]/QQt[k*n+k];
	}
	else
	{
	    System.arraycopy(dpmoms,0,r,0,n);
	    final int [] ipvt=new int [n];
	    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(QQt,n,n,ipvt);
	    if(pinfo!=0) {
		java.util.Arrays.fill(r,Double.NaN);
		return r;
	    }
	    com.polytechnik.utils.Linsystems.dgesl_m1(QQt,n,n,ipvt,r,1);
	}
	return TMatr.getMatr_x_A(n,QQi,r);
    }

static void matchDynDpDi(final DynDpDi a,final DynDpDi b,final double eps)
{
    UnitTests.checkDoubleRel(a.I_L,"a.I_L",b.I_L,"b.I_L",eps);
    UnitTests.checkDoubleRel(a.I_H,"a.I_H",b.I_H,"b.I_H",eps);
    UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I_0,"b.I_0",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.Dp,"a.Dp",b.Dp,"b.Dp",eps);
    System.err.println("DynDpDi:matchDynDpDi test OK eps="+eps);
}

}
