package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

class DynEHPnL {

    final double p_IH,pd_IH,var_p_IH;
    final EVXData I;
    final double dirL,dirH,dirH0,dir0,dirS;
    final double p_IH2,i_IH2,dirH2;
    final double pi_pL,pnl_pL,pi_pH,pnl_pH,i_pL,i_pH;

    final double [] psiPnlL,psiPnlH,psiE;

    static final boolean FLAG_PRINT_DIAG=false;

    DynEHPnL(final int n,
	     final double [] QQi,final double [] QQt, final double [] QQpi,
	     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	final int ind_toremove=n-1,nd=n-1;
	this.p_IH=TMatr.getAMatrA(n,I.ev,n+ind_toremove*n,QQpi,n)/I.ev[ind_toremove];
	final double [] QQpnl=TMatr.sumAcB(QQpi,-p_IH,QQi);

	if(FLAG_PRINT_DIAG)
	{
	    System.err.println("<psiH|QQpnl|psiH>="+TMatr.getAMatrA(I.psiH,QQpnl,n)+" must be =0");
	}

	final double [] evE,psipnlL,psipnlH;
	if(false)
	{
	    final double [] bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpnl,I.ev);
	    evE=M.EV.getEV(nd,DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(n,bQQpnl,ind_toremove),true);
	    psipnlL=getPsiInOrigBasis(0,evE,n,I.ev,ind_toremove);
	    psipnlH=getPsiInOrigBasis(nd-1,evE,n,I.ev,ind_toremove);
	}
	else
	{
	    final double [] basisNoH=DeprecatedHelperEVXData.getBasisOneVectorRemoved(n,I.ev,n,ind_toremove);
	    final double [] bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpnl,basisNoH,nd,0);
	    evE=M.EV.getEV(nd,java.util.Arrays.copyOf(bQQpnl,nd*nd),true);
	    final double [] res=new double [nd*n];
	    DeprecatedHelperTMatr.multiply_MA_x_MB(res,nd,evE,nd,nd,nd,basisNoH,n,n,0); // res: nd x n
	    psipnlL=EVXData.getPsiFromBasis(0,res,n,0);
	    psipnlH=EVXData.getPsiFromBasis(nd-1,res,n,0);
	}
	this.psiPnlL=psipnlL;
	this.psiPnlH=psipnlH;
	this.psiE=I.getPsi(ind_toremove);

	pnl_pL=evE[0];
	pnl_pH=evE[nd-1];
	i_pL=TMatr.getAMatrA(psipnlL,QQi,n);
	pi_pL=TMatr.getAMatrA(psipnlL,QQpi,n)/i_pL;
	i_pH=TMatr.getAMatrA(psipnlH,QQi,n);
	pi_pH=TMatr.getAMatrA(psipnlH,QQpi,n)/i_pH;

	if(FLAG_PRINT_DIAG)
	{
	    final double [] psi=I.psiH;
	    System.err.println("evE[0]="+evE[0]+" <psipnlL|PnL|psipnlL>="+(TMatr.getAMatrA(psipnlL,QQpi,n)-p_IH*TMatr.getAMatrA(psipnlL,QQi,n)));
	    System.err.println("evE[nd-1]="+evE[nd-1]+" <psipnlH|PnL|psipnlH>="+(TMatr.getAMatrA(psipnlH,QQpi,n)-p_IH*TMatr.getAMatrA(psipnlH,QQi,n)));
	    System.err.println("<psipnlL|psipnlL>="+TMatr.getAMatrA(psipnlL,QQt,n));
	    System.err.println("<psipnlH|psipnlH>="+TMatr.getAMatrA(psipnlH,QQt,n));
	    System.err.println("<psipnlL|psipnlH>="+TMatr.getAMatrB(psipnlL,psipnlH,QQt,n));
	    System.err.println("<psipnlL|psi>="+TMatr.getAMatrB(psipnlL,psi,QQt,n));
	    System.err.println("<psipnlH|psi>="+TMatr.getAMatrB(psipnlH,psi,QQt,n));
	    System.err.println("<psipnlL|I|psi>="+TMatr.getAMatrB(psipnlL,psi,QQi,n));
	    System.err.println("<psipnlH|I|psi>="+TMatr.getAMatrB(psipnlH,psi,QQi,n));
	}

	final double [] basis0=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M),
	    psiH0=EVXData.getPsiFromBasis(n-1,basis0,n);

	dirL=Math.pow(TMatr.getAMatrB(I.psiL,psipnlH,QQt,n),2)-Math.pow(TMatr.getAMatrB(I.psiL,psipnlL,QQt,n),2);
	dirH=Math.pow(TMatr.getAMatrB(I.psiH,psipnlH,QQt,n),2)-Math.pow(TMatr.getAMatrB(I.psiH,psipnlL,QQt,n),2);
	dirH0=Math.pow(TMatr.getAMatrB(psiH0,psipnlH,QQt,n),2)-Math.pow(TMatr.getAMatrB(psiH0,psipnlL,QQt,n),2);
	dir0=Math.pow(TMatr.getAMatrB(I.psi0,psipnlH,QQt,n),2)-Math.pow(TMatr.getAMatrB(I.psi0,psipnlL,QQt,n),2);
	final double [] psi=I.psiH,Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	TMatr.sumP1(Dpsi,psi,-TMatr.getAMatrB(Dpsi,psi,QQt,n));
	var_p_IH=TMatr.getAMatrB(Dpsi,psi,QQpi,n);
	pd_IH=TMatr.getAMatrA(Dpsi,QQpi,n)/TMatr.getAMatrA(Dpsi,QQi,n);
	TMatr.scaleVectorInPlace(Dpsi,1/Math.sqrt(TMatr.getAMatrA(Dpsi,QQt,n)));
	dirS=Math.pow(TMatr.getAMatrB(Dpsi,psipnlH,QQt,n),2)-Math.pow(TMatr.getAMatrB(Dpsi,psipnlL,QQt,n),2);

	final double [] psiH2=I.getPsi(n-2);
	i_IH2=I.ev[n-2];
	p_IH2=TMatr.getAMatrA(psiH2,QQpi,n)/i_IH2;
	dirH2=Math.pow(TMatr.getAMatrB(psiH2,psipnlH,QQt,n),2)-Math.pow(TMatr.getAMatrB(psiH2,psipnlL,QQt,n),2);
    }

    /** Sum directly instead of matrix multiplication like in EVXData.convertMatrFromQtoB_Basis.
     */
    static double [] getPsiInOrigBasis(final int ind,final double [] evE,final int n,final double [] basis,final int ind_toremove)
    {
	final int nd=n-1;
	final double [] psi=new double [n];
	for(int k=0,o=nd+ind*nd;k<n;k++)
	{
	    if(k==ind_toremove) continue;
	    DeprecatedHelperTMatr.sumP1(psi,basis,n+k*n,n,evE[o++]);
	}
	return psi;
    }

    static void match_DynEHbPnL(final DynEHPnL q,final DynEHbPnL s,final double eps)
    {
	EVXData.matchEVXData(q.I,s.I,eps);
	UnitTests.checkDoubleRel(q.p_IH,"q.p_IH",s.p_IH,"s.p_IH",eps);
	UnitTests.checkDoubleRel(q.var_p_IH,"q.var_p_IH",s.var_p_IH,"s.var_p_IH",eps);
	UnitTests.checkDoubleRel(q.dirL,"q.dirL",s.dirL,"s.dirL",eps);
	UnitTests.checkDoubleRel(q.dirS,"q.dirS",s.dirS,"s.dirS",eps);
	System.err.println("DynEHPnL:match_DynEHbPnL(): test OK n="+q.I.neq+" eps="+eps);
    }
}
