package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

class DynEHbPnL {

    final double p_IH,var_p_IH;
    final EVXData I;
    final double dirL,dirS;

    DynEHbPnL(final int n,
	      final double [] QQi,final double [] QQt, final double [] QQpi,
	      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	final int ind_toremove=n-1,nd=n-1;
	final double [] psi=I.getPsi(ind_toremove),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	this.p_IH=TMatr.getAMatrA(psi,QQpi,n)/I.ev[ind_toremove];
	TMatr.sumP1(Dpsi,psi,-TMatr.getAMatrB(Dpsi,psi,QQt,n));
	this.var_p_IH=TMatr.getAMatrB(Dpsi,psi,QQpi,n);
	final double [] QQpnl=TMatr.sumAcB(QQpi,-p_IH,QQi),
	    basisNoH=DeprecatedHelperEVXData.getBasisOneVectorRemoved(n,I.ev,n,ind_toremove),
	    bDpsi=EVXData.convertPsiFromQtoB_Basis(Dpsi,basisNoH,nd,0,QQt,n);
	TMatr.scaleVectorInPlace(bDpsi,1/Math.sqrt(TMatr.mulVV(bDpsi,bDpsi)));
	final double [] bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpnl,basisNoH,nd,0);
	final double [] evE=M.EV.getEV(nd,java.util.Arrays.copyOf(bQQpnl,nd*nd),true);
	this.dirS=Math.pow(DeprecatedHelperTMatr.mulVVo(nd,bDpsi,0,evE,nd+(nd-1)*nd),2)-Math.pow(DeprecatedHelperTMatr.mulVVo(nd,bDpsi,0,evE,nd+0*nd),2);
	this.dirL=Math.pow(evE[nd+(nd-1)*nd],2)-Math.pow(evE[nd+0*nd],2);

    }
}
