package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;

/**
 *
 *  @version $Id: DynEVdI.java,v 1.30 2022/08/07 15:44:02 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DynEVdI {



    public final EVXData I;
    public final double p_offset,p_last__with_p_offset_included,p_IH,I_0_feq,dI,dW,Pf,sigma2_Pf,PfH,PfH0;
    public final Skewness SK,SKf,SK0;
    DynEVdI(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,
	    final double p_IH,final double I_0_feq,
	    final double [] psik_psi0,
	    final double [] psik_I_psik,final double [] psik_pI_psik,
	    final double [] psik_p2I_psik,final double [] psik_p3I_psik,
	    final double [] bbasis,
	    final double [] psi0_pmI_psi0,final double [] psiH_pmI_psiH,
	    final double eps)
    {
	this.I=I;
	this.p_offset=p_offset;
	this.p_last__with_p_offset_included=p_last__with_p_offset_included;
	this.p_IH=p_IH;
	this.I_0_feq=I_0_feq;
	final int n=I.neq;
	this.dI=I_0_feq-I.s0;
	final double [] w0k=TMatr.squareArrayElements(psik_psi0);
	this.dW=TMatr.mulVV(w0k,w0k);
	final double mf0=TMatr.mulVV(w0k,psik_I_psik),
	    mf1=TMatr.mulVV(w0k,psik_pI_psik),
	    mf2=TMatr.mulVV(w0k,psik_p2I_psik),
	    mf3=TMatr.mulVV(w0k,psik_p3I_psik);
	this.PfH=(psiH_pmI_psiH[1]-mf1)/(dI*dW);
	this.PfH0=(psiH_pmI_psiH[1]-psi0_pmI_psi0[1])/dI;
	final double
	    mx0=dI*(1-dW),
	    mx1=mf1-psi0_pmI_psi0[1],
	    mx2=mf2-psi0_pmI_psi0[2],
	    mx3=mf3-psi0_pmI_psi0[3];
	this.Pf=mx1/mx0;
	this.sigma2_Pf=mx2/mx0-Math.pow(Pf,2);
	SK=Skewness.getSkewnessFromMonomialMoments(mx0,mx1,mx2,mx3);
	this.SK0=Skewness.getSkewnessFromMonomialMoments(psi0_pmI_psi0[0],psi0_pmI_psi0[1],psi0_pmI_psi0[2],psi0_pmI_psi0[3]);
	this.SKf=Skewness.getSkewnessFromMonomialMoments(mf0,mf1,mf2,mf3);
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(mf0+dW*dI,"mf0+dW*dI",I_0_feq,"I_0_feq",eps);
	    UnitTests.checkDoubleRel(psi0_pmI_psi0[0]+dI,"psi0_pmI_psi0[0]+dI",I_0_feq,"I_0_feq",eps);
	    UnitTests.checkDoubleRel(TMatr.mulVV(w0k,java.util.Arrays.copyOf(bbasis,n)),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);
	    for(int k=0;k<n;k++) UnitTests.checkDoubleRel(psik_I_psik[k]+dI*w0k[k],"psik_I_psik[k]+dI*w0k[k]",bbasis[k],"bbasis[k]",eps);
	    UnitTests.checkDoubleRel(mf0-psi0_pmI_psi0[0],"mx0",mx0,"mx0",eps);
	    UnitTests.checkDoubleRel((SKf.xa*(SKf.w1+SKf.w2)-SK0.xa*(SK0.w1+SK0.w2))/(SKf.w1+SKf.w2-(SK0.w1+SK0.w2)),"Pf",Pf,"Pf",eps);
	     UnitTests.checkDoubleRel((psiH_pmI_psiH[0]-mf0)/(dI*dW),"1",1,"1",eps);
	     UnitTests.checkDoubleRel((psiH_pmI_psiH[0]-psi0_pmI_psi0[0])/dI,"1",1,"1",eps);
	    System.err.println("DynEVdI.DynEVdI selftest2 OK n="+n+" eps="+eps);
	}

    }

/**
 * Solve {@literal <psi|I|psi>/<psi|psi>}=lambda extremum problem, find eigenstates &amp; eigenvalues.
 * find psi^{IH} and lambda^{IH}=max lambda, used as an estimator of
 * the level of "future" execution.
 * Then dI=lambda^{IH}-{@literal <psi0|I|psi0>} (contribution from the future, of not yet executed, trades).
 * Then operator I^{f} (future I) is: I^{f}= I + {@literal |psi0>dI<psi0|}
 * Find the eigenstates of this new I^{f} matrix,
 * in dynamic impact approximation its eigenstates should be also the P^{f} operator eigenstates.
 *
 * For the moments of {@literal |psik>} state then:
 * {@literal <psi0|pI|psi0>}+P^{f}*dI=P^{*}*lambda^{IH}
 * sum({@literal <psik|pI^{f}|psik>}{@literal <psik|psi0>}^2)=P^{*}*lambda^{IH}
 * what gives
 * P^{f}*dI*sum({@literal <psik|psi0>}^4) + sum({@literal <psik|pI|psik>}{@literal <psik|psi0>}^2) = {@literal <psi0|pI|psi0>}+P^{f}*dI
 * and P^{f}*dI=(sum({@literal <psik|pI|psik>}{@literal <psik|psi0>}^2-{@literal <psi0|pI|psi0>})/(1-sum({@literal <psik|psi0>}^4)).
 */
public static DynEVdI get_DynEVdI(final MomentsData mm,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				  final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final int ixH=n-1; // the state with maximal I
    final double I_0=I.s0,I_0_feq=I.ev[ixH],dI=I_0_feq-I_0;

    // Create bQQi_feq (operator of "future" I), with the |bpsi0>dI<bpsi0| contribution from the future:
    // bQQi_feq = bQQi + |bpsi0>dI<bpsi0|
    final double [] bQQi_feq=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o)),bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));
    for(int k=0;k<n;k++) bQQi_feq[k+n*k]+=I.ev[k];

    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp2i,I.ev),
	bQQp3i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp3i,I.ev);

    // Diagonalize "future" I:
    final double [] bbasis=M.EV.getEV(n,bQQi_feq,true),
	psik_psi0=new double [n],psik_I_psik=new double[n],psik_pI_psik=new double[n],
	psik_p2I_psik=new double[n],psik_p3I_psik=new double[n],
	psi0_pmI_psi0=new double []{
	    TMatr.mulV2V(bpsi0,I.getEValues()),
	    TMatr.getAMatrA(bpsi0,bQQpi,n),
	    TMatr.getAMatrA(bpsi0,bQQp2i,n),
	    TMatr.getAMatrA(bpsi0,bQQp3i,n)
	},
	psiH_pmI_psiH=new double []{
	    I.ev[ixH],
	    bQQpi[ixH+n*ixH],
	    bQQp2i[ixH+n*ixH],
	    bQQp3i[ixH+n*ixH]
	};
    for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n);

	psik_psi0[k]=TMatr.mulVV(psik,bpsi0);
	psik_I_psik[k]=TMatr.mulV2V(psik,I.getEValues());
	psik_pI_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n);
	psik_p2I_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQp2i,n);
	psik_p3I_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQp3i,n);
    }

    if(eps>0)
    {
	EVXData.test_orthogonality("DynEVdI.get_DynEVdI:I.ev",n,I.ev,mm.QQi,mm.QQt,eps); // old basis
	EVXData.test_orthogonality("DynEVdI.get_DynEVdI:bbasis",n,bbasis,bQQi_feq,TMatr.getUnitMatrix(n),eps); // new basis
	// check some matrix elements
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(psik_psi0,java.util.Arrays.copyOf(bbasis,n)),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi0,bpsi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(bpsi0,I.getEValues()),"<psi0|I|psi0>",I.s0,"I.s0",eps);
	UnitTests.checkDoubleRel(psi0_pmI_psi0[0],"psi0_pmI_psi0[0]",I.s0,"I.s0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_feq,n),"<psi0|I^f|psi0>",I_0_feq,"I_0_feq",eps);
	final double [] w0k=TMatr.squareArrayElements(psik_psi0);
	UnitTests.checkDoubleRel(TMatr.sumElements(w0k),"sum(w0k)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(w0k,psik_I_psik)+dI*TMatr.mulVV(w0k,w0k),"sum(<psik|I|psik>*<psik|psi0>^2)+dI*sum(<psik|psi0>^4)",I_0_feq,"I_0_feq",eps);

	// check spur invariance
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues()),"sum(I.ev)",TMatr.sumElements(psik_I_psik),"sum(psik_I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues())+dI,"sum(I.ev)+dI",TMatr.sumElements(java.util.Arrays.copyOf(bbasis,n)),"sum(I^{f})",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQpi),"Spur(bQQpi)",TMatr.sumElements(psik_pI_psik),"sum(psik_pI_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQp2i),"Spur(bQQp2i)",TMatr.sumElements(psik_p2I_psik),"sum(psik_p2I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQp3i),"Spur(bQQp3i)",TMatr.sumElements(psik_p3I_psik),"sum(psik_p3I_psik)",eps);

	final double [][] mpI=new double [][]{mm.QQi,mm.QQpi,mm.QQp2i,mm.QQp3i};
	for(int k=0;k<mpI.length;k++)
	{
	    UnitTests.checkDoubleRel(psi0_pmI_psi0[k],"psi0_pmI_psi0[k]",TMatr.getAMatrA(I.psi0,mpI[k],n),"<psi0|QQpmI|psi0>",eps);
	}
	System.err.println("DynEVdI.get_DynEVdI selftest OK n="+n+" eps="+eps);
    }

    return new DynEVdI(I,mm.p_offset,mm.p_last,
		       bQQpi[ixH+n*ixH]/I.ev[ixH],I_0_feq,
		       psik_psi0,psik_I_psik,psik_pI_psik,psik_p2I_psik,psik_p3I_psik,bbasis,
		       psi0_pmI_psi0,psiH_pmI_psiH,
		       eps);
}


}
