package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

class DynHPnL {
    final double I_0,I_L,I_H,p_IH,I_H1,p_IH1,dir,PfI_0,PfI_L,PfI_H,
	distance2_IH_cL,distance2_IH_cH;
    final EVXData Ipnl;
    final double [] basis,fbasis;

    DynHPnL(final int n,
	    final double [] QQi,final double [] QQt, final double [] QQf, final double [] QQfn,final double [] QQV,final double [] QQpV,
	    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	    final K2Distance distance2)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final int i0=0,iL=1,iH=n-1;
	this.basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
	this.I_0=basis[i0];
	this.I_L=basis[iL];
	this.I_H=basis[iH];

	final double fnorm=(QQfn==QQi?basis[iH]:(QQfn==QQt?1:TMatr.getAMatrA(n,basis,n+iH*n,QQfn,n)));
	this.p_IH=TMatr.getAMatrA(n,basis,n+iH*n,QQf,n)/fnorm;
	final double [] psiH=EVXData.getPsiFromBasis(iH,basis,n),one=M.getRadauMultiplied(new double []{1});
	final double norm_H1=TMatr.getAMatrB(psiH,one,QQt,n);
	this.I_H1=TMatr.getAMatrB(psiH,one,QQi,n)/norm_H1;
	this.p_IH1=TMatr.getAMatrB(psiH,one,QQf,n)/TMatr.getAMatrB(psiH,one,QQfn,n);

	final double [] QQpnl=TMatr.sumAcB(QQf,-p_IH,QQfn);

	// version 1
	this.fbasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQpnl,QQt,M);
	this.PfI_0=fbasis[i0];
	this.PfI_L=fbasis[iL];
	this.PfI_H=fbasis[iH];

	// version 2
	this.Ipnl=EVXData.getEVXDataFromQQ(n,QQpnl,QQt,M);
	this.dir=Math.pow(Ipnl.wH,2)-Math.pow(Ipnl.wL,2);
	this.distance2_IH_cL=distance2.getDistance2(psiH,Ipnl.psiL);
	this.distance2_IH_cH=distance2.getDistance2(psiH,Ipnl.psiH);
	if(false)
	{
	    System.err.println("Ipnl.ev="+java.util.Arrays.toString(Ipnl.getEValues()));
	}

	// version 3
	int ind_pnl=-1;double swpp=0;
	for(int k=0;k<n;k++)
	{
	    final double wpp=Math.abs(Ipnl.getFPsiPsi(k,psiH,QQt));
	    if(ind_pnl<0||wpp>=swpp){ind_pnl=k;swpp=wpp;}
	}
	final double [] QQPNL=TMatr.sumAcB(QQpV,-p_IH,QQV);
	TMatr.sumP1(QQPNL,QQt,-TMatr.getAMatrA(psiH,QQPNL,n));
	this.pix_IH_IE=Ipnl.getFPsi(ind_pnl,QQpnl);
	this.pnl_IH_IE=Ipnl.getFPsi(ind_pnl,QQPNL);
	this.w_IH_IE=swpp;
	this.distance2_IH_IE=distance2.getDistance2(psiH,Ipnl.getPsi(ind_pnl));
    }
    final double distance2_IH_IE,pix_IH_IE,pnl_IH_IE,w_IH_IE;

    static void test_DynHPnL(
	final int n,
	final double [] QQi,final double [] QQt, final double [] QQf, final double [] QQfn,
	final double [] QQV,final double [] QQpV,
	final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	final K2Distance distance2,
	final double eps)
    {
	final DynHPnL q=new DynHPnL(n,QQi,QQt,QQf,QQfn,QQV,QQpV,M,distance2);
	final double [] QQpnl=TMatr.sumAcB(QQf,-q.p_IH,QQfn);

	final int i0=0,iL=1,iH=n-1;
	UnitTests.checkDoubleRel(TMatr.getAMatrA(n,q.basis,n+iH*n,QQpnl,n),"<pisIH|QQpnl|psiH>",0,"0",eps);

	final int nd=n-1;
	final double [] bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpnl,q.basis,nd,n+n);
	final double [] epnl=M.EV.getEV(nd,bQQpnl,false);

	final double PfI_0=TMatr.getAMatrA(n,q.basis,n+i0*n,QQpnl,n),
	    PfI_L=epnl[0],PfI_H=epnl[nd-1];
	System.err.println("q.PfI_0="+q.PfI_0+"\t PfI_0="+PfI_0);
	System.err.println("q.PfI_L="+q.PfI_L+"\t PfI_L="+PfI_L);
	System.err.println("q.PfI_H="+q.PfI_H+"\t PfI_H="+PfI_H);

	UnitTests.checkDoubleRel(q.PfI_0,"q.PfI_0",PfI_0,"PfI_0",eps);
	if(n>2)
	{
	UnitTests.checkDoubleRel(q.PfI_L,"q.PfI_L",PfI_L,"PfI_0",eps);
	UnitTests.checkDoubleRel(q.PfI_H,"q.PfI_H",PfI_H,"PfI_H",eps);
	}

	UnitTests.checkDoubleRel(q.PfI_0,"q.PfI_0",q.Ipnl.s0,"q.Ipnl.s0",eps);

	final double [] psiH=EVXData.getPsiFromBasis(iH,q.basis,n),one=M.getRadauMultiplied(new double []{1});
	final double p_IH1=TMatr.getAMatrB(psiH,one,QQf,n)/TMatr.getAMatrB(psiH,one,QQfn,n);
	System.err.println("q.I_H="+q.I_H+"\tq.I_H1="+q.I_H1);
	System.err.println("q.p_IH="+q.p_IH+"\tq.p_IH1="+q.p_IH1);
	UnitTests.checkDoubleRel(q.I_H,"q.I_H",q.I_H1,"q.I_H1",eps);
	UnitTests.checkDoubleRel(q.p_IH1,"q.p_IH1",p_IH1,"p_IH1",eps);

	System.err.println("DynHPnL:test_DynHPnL(): test OK n="+n+" eps="+eps);

    }
}

