package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: DynImpactAsymmetry.java,v 1.39 2022/07/27 16:14:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class DynImpactAsymmetry<DTYPE> {

public static class State {
    public final long seq;
    State(final State prev)
    {
	this.seq=(prev==null?0:1+prev.seq);
    }
}
public final State sst;

public final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
final VolMatrix VOLATILITY_EDPsi=this::getEDPsi;
public final EVXData I;
public final double p_offset;
public final Mu2P2Asymmetry dia_average,dia_spur,dia_IH;
public final Mu2P2Set dia_mu;
public final ImpactFromTheFuture2 ifa;
final ImpactFromTheFuture_Pi ifa_Pi;
final ImpactFromTheFuture_P0i ifa_P0i;
final ImpactFromTheFuture_Pfi ifa_Pfi;
final ImpactFromTheFuture_Ci ifa_Ci;
final InDIState Rindistate,RindistateF,Rindistate2;
final InDIStateI0 indistateI0f,indistateI0;
final InIStateVT inistateVT;
final InDIStateMean indistatemean;

final DTYPE odata;

DynImpactAsymmetry(final int n,final DTYPE odata,final State rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");
    final double eps=0; // disable selftests

    this.odata=odata;
    this.M=getABasis();
    final IMomentsData mm=getIMomentsData(n);
    this.p_offset=mm.p_offset;
    // moving average --like
    dia_average=new Mu2P2Asymmetry(mm.QQt_sample[0],mm.QQpt_sample[0],mm.QQp2t_sample[0],
				   mm.QQi[0],mm.QQpi[0],mm.QQp2i[0],eps);
    this.I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double [][] pmm=getPriceMoments_Iev(I,mm);
    final int ixH=n-1;
    // the state of max I
    dia_IH=new Mu2P2Asymmetry(pmm[0][ixH],pmm[1][ixH],pmm[2][ixH],pmm[3][ixH],pmm[4][ixH],pmm[5][ixH],eps);
    // spur
    dia_spur=new Mu2P2Asymmetry(TMatr.sumElements(pmm[0]),TMatr.sumElements(pmm[1]),TMatr.sumElements(pmm[2]),
				TMatr.sumElements(pmm[3]),TMatr.sumElements(pmm[4]),TMatr.sumElements(pmm[5]),eps);
    // weighted beta, not weighted moments
    dia_mu=Mu2P2Set.getMu2P2Set_MU(mm,M,eps);
    // impact from the future
    ifa=new ImpactFromTheFuture2(mm,M,eps);
    ifa_Pi=new ImpactFromTheFuture_Pi(mm,M,VOLATILITY_EDPsi,eps);
    ifa_P0i=new ImpactFromTheFuture_P0i(mm,M,eps);
    ifa_Pfi=new ImpactFromTheFuture_Pfi(mm,M,eps);
    ifa_Ci=new ImpactFromTheFuture_Ci(mm,M,eps);
    Rindistate=new InDIState(mm,M,VOLATILITY_EDPsi,true,false,eps);
    RindistateF=new InDIState(mm,M,VOLATILITY_EDPsi,true,true,eps);
    Rindistate2=new InDIState(mm,M,VOLATILITY_EDPsi,false,false,eps);
    indistateI0f=new InDIStateI0(mm,M,VOLATILITY_EDPsi,true,eps);
    indistateI0=new InDIStateI0(mm,M,VOLATILITY_EDPsi,false,eps);
    inistateVT=new InIStateVT(n,mm.p_offset,mm.QQt,mm.QQi,mm.QQpi,VOLATILITY_EDPsi,M,0.0);
    indistatemean=new InDIStateMean(mm,M,VOLATILITY_EDPsi,0);

    this.sst=new State(rst);
}
protected IMomentsData getIMomentsData(final int n){return IMomentsData.get_IMomentsData(n,M,odata,getAccess());}
abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double [] getEDPsi(final double [] psi);

static double [][] getPriceMoments_Iev(final EVXData I,final IMomentsData mm)
{
    final int n=I.neq;
    final double [] p0t=new double [n],p1t=new double [n],p2t=new double [n],
	p0i=new double [n],p1i=new double [n],p2i=new double [n];
    for(int k=0;k<n;k++) {
	p0t[k]=I.getFPsi(k,mm.QQt_sample);
	p1t[k]=I.getFPsi(k,mm.QQpt_sample);
	p2t[k]=I.getFPsi(k,mm.QQp2t_sample);
	p0i[k]=I.getFPsi(k,mm.QQi);
	p1i[k]=I.getFPsi(k,mm.QQpi);
	p2i[k]=I.getFPsi(k,mm.QQp2i);
    }
    return new double [][]{p0t,p1t,p2t,p0i,p1i,p2i};
}
/** A class for unit test.
*/
static abstract class DynImpactAsymmetryTest<DTYPE> extends QTest {

abstract DynImpactAsymmetry<DTYPE> getDynImpactAsymmetry();
abstract ImpactQVM<DTYPE> getImpactQVM();

DynImpactAsymmetryTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}

boolean testRadau(){return true;}

void test_Extra(){}

void test_Self()
{
    final DynImpactAsymmetry<DTYPE> q=getDynImpactAsymmetry();
    final int n=neq,ixH=n-1;
    final IMomentsData mm=q.getIMomentsData(n);
    final double [][] pmm=getPriceMoments_Iev(q.I,mm);

    Mu2P2Asymmetry.matchMu2P2Asymmetry(new Mu2P2Asymmetry(pmm[0][ixH],pmm[1][ixH],pmm[2][ixH],pmm[3][ixH],pmm[4][ixH],pmm[5][ixH],eps),mm.p_offset,q.dia_IH,q.p_offset,eps);
    Mu2P2Asymmetry.matchMu2P2Asymmetry(new Mu2P2Asymmetry(TMatr.sumElements(pmm[0]),TMatr.sumElements(pmm[1]),TMatr.sumElements(pmm[2]),TMatr.sumElements(pmm[3]),TMatr.sumElements(pmm[4]),TMatr.sumElements(pmm[5]),eps),mm.p_offset,q.dia_spur,q.p_offset,eps);
    Mu2P2Set.getMu2P2Set_MU(mm,q.M,eps);
    new ImpactFromTheFuture2(mm,q.M,eps);
    new ImpactFromTheFuture_Pi(mm,q.M,q.VOLATILITY_EDPsi,eps);
    new ImpactFromTheFuture_P0i(mm,q.M,eps);
    ImpactFromTheFuture_Ci.matchImpactFromTheFuture_Pfi(q.ifa_Ci,q.ifa_Pfi,eps);
    ImpactFromTheFuture_Ci.matchImpactFromTheFuture_Pfi(new ImpactFromTheFuture_Ci(mm,q.M,eps),new ImpactFromTheFuture_Pfi(mm,q.M,eps),eps);
    if(testRadau()) {
	new InDIState(mm,q.M,q.VOLATILITY_EDPsi,true,false,eps);
	new InDIState(mm,q.M,q.VOLATILITY_EDPsi,true,true,eps);
    }
    else System.err.println(name+":DynImpactAsymmetryTest.test_Self(): Radau Test skiped");
    new InDIState(mm,q.M,q.VOLATILITY_EDPsi,false,false,eps);
    if(testRadau()) {
	new InDIStateI0(mm,q.M,q.VOLATILITY_EDPsi,true,eps);
	new InDIStateI0(mm,q.M,q.VOLATILITY_EDPsi,false,eps);
	new InIStateV(mm,q.M,eps);
	new InDIStateMean(mm,q.M,q.VOLATILITY_EDPsi,eps);
    }
    new InIStateVT(n,mm.p_offset,mm.QQt,mm.QQi,mm.QQpi,q.VOLATILITY_EDPsi,q.M,eps);

    // to fail
    //new InDIState(mm,q.M,q.VOLATILITY_EDPsi,false,true,eps);

    System.err.println(name+":DynImpactAsymmetryTest.test_Self(): test OK n="+n+" eps="+eps);
}

void test_ImpactQVM()
{
    final int n=neq;
    final DynImpactAsymmetry<DTYPE> q=getDynImpactAsymmetry();
    final ImpactQVM<DTYPE> s=getImpactQVM();
    if(s==null)
    {
	System.err.println(name+":DynImpactAsymmetryTest.test_ImpactQVM(): skipping test OK n="+n+" as getImpactQVM()=null eps="+eps);
	return;
    }
    matchImpactQVM(q,s,eps);

    System.err.println(name+":DynImpactAsymmetryTest.test_ImpactQVM(): test OK n="+n+" eps="+eps);
}


static void matchImpactQVM(final DynImpactAsymmetry<?> q,final ImpactQVM<?> s,final double eps)
{

    EVXData.matchEVXData(q.I,s.pnldiskewness_nodI.I,false,eps);
    final double p_offset_shift=s.iex.p_offset-q.p_offset;
    UnitTests.checkDoubleRel(q.dia_IH.mH.pi,"q.dia_IH.mH.pi",s.iex.p_IH+p_offset_shift,"s.iex.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(q.dia_average.mH.pi,"q.dia_average.mH.pi",s.pnldidsk.SK_movaver__nodI.xa+p_offset_shift,"s.pnldidsk.SK_movaver__nodI.xa+p_offset_shift",eps);
    UnitTests.checkDoubleRel(q.dia_spur.mH.pi,"q.dia_spur.mH.pi",s.pnldidsk.SK_spur__nodI.xa+p_offset_shift,"s.pnldidsk.SK_spur__nodI.xa+p_offset_shift",eps);
    UnitTests.checkDoubleRel(q.ifa.deg_from_dyn,"q.ifa.deg_from_dyn",s.pnldidsk.deg_from_dyn,"s.pnldidsk.deg_from_dyn",eps);
    UnitTests.checkDoubleRel(q.ifa.Pf_from_dyn,"q.ifa.Pf_from_dyn",s.pnldidsk.Pf_from_f0+p_offset_shift,"s.pnldidsk.Pf_from_f0+p_offset_shift",eps);

    final double p_offset_shift_a=s.pnldicorr_nodI_t.p_offset-q.p_offset;
    UnitTests.checkDoubleRel(q.dia_average.mH.pt+1,"q.dia_average.mH.pt",s.pnldicorr_nodI_t.movaver_cc.SK_pt.xa+1+p_offset_shift_a,"s.pnldicorr_nodI_t.movaver_cc.SK_pt.xa+p_offset_shift_a",eps);

    System.err.println("DynImpactAsymmetry.matchImpactQVM test OK n="+q.I.neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":DynImpactAsymmetryTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Extra();
    test_ImpactQVM();
    test_Self();

    System.err.println(name+":DynImpactAsymmetryTest.test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_min,final int neq_max)
{
    System.err.println(name+":DynImpactAsymmetryTest: starting tests; ver=$Id: DynImpactAsymmetry.java,v 1.39 2022/07/27 16:14:10 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=neq_min;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":DynImpactAsymmetryTest.main: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: DynImpactAsymmetry.java,v 1.39 2022/07/27 16:14:10 mal Exp $");
}
} // END OF class DynImpactAsymmetryTest




}// END OF class DynImpactAsymmetry
