package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: DynImpactAsymmetryMonomials.java,v 1.10 2022/07/10 10:41:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DynImpactAsymmetryMonomials extends DynImpactAsymmetry<QVMData> {
DynImpactAsymmetryMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static DynImpactAsymmetryMonomials calculate(final int neq,
					   final long currenttime,
					   final long tau,
					   final Trade [] tb,
					   final int tsize,
					   final DynImpactAsymmetryMonomials ref)
{
    final int nvm=1,npm=4,nr=0;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1+nr,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new DynImpactAsymmetryMonomials(neq,o,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}

static class DynImpactAsymmetryMonomialsTest extends DynImpactAsymmetryTest<QVMData> {
DynImpactAsymmetryMonomialsTest(final long initrandom)
{
    super("DynImpactAsymmetryMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}



@Override
void test_Extra(){
    // These to be identical
    final DynImpactAsymmetryMonomials q=getDynImpactAsymmetry();
    final DynImpactAsymmetryLaguerre s=getDynImpactAsymmetryLaguerre();

    matchImpactQVM(q,getImpactQVMLaguerre(),eps);
    matchImpactQVM(s,getImpactQVM(),eps);
    Mu2P2Asymmetry.matchMu2P2Asymmetry(q.dia_average,q.p_offset,s.dia_average,s.p_offset,eps);
    Mu2P2Asymmetry.matchMu2P2Asymmetry(q.dia_spur,q.p_offset,s.dia_spur,s.p_offset,eps);
    Mu2P2Asymmetry.matchMu2P2Asymmetry(q.dia_IH,q.p_offset,s.dia_IH,s.p_offset,eps);
    ImpactFromTheFuture_Ci.matchImpactFromTheFuture_Pfi(q.ifa_Ci,s.ifa_Pfi,eps);
    ImpactFromTheFuture_Ci.matchImpactFromTheFuture_Pfi(s.ifa_Ci,q.ifa_Pfi,eps);

    System.err.println(name+":DynImpactAsymmetryMonomialsTest:test_Extra(): test DynImpactAsymmetryLaguerre OK neq="+neq+" eps="+eps);
}


DynImpactAsymmetryMonomials getDynImpactAsymmetry(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DynImpactAsymmetryLaguerre getDynImpactAsymmetryLaguerre(){return DynImpactAsymmetryLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
ImpactQVMMonomials getImpactQVM(){return ImpactQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}
ImpactQVMLaguerre getImpactQVMLaguerre(){return ImpactQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}


} // END of class DynImpactAsymmetryMonomialsTest

public static void main(String [] args)
{
    new DynImpactAsymmetryMonomialsTest(11127L).runTests(2,5);
} // END main



} // END OF class DynImpactAsymmetryMonomials
