package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: DynImpactAsymmetry_pi.java,v 1.8 2018/07/03 13:06:39 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DynImpactAsymmetry_pi extends DynImpactAsymmetry<QVMDataL> {
DynImpactAsymmetry_pi(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}


public static DynImpactAsymmetry_pi calculate(final int neq,
					   final long currenttime,
					   final long tau,
					   final Trade [] tb,
					   final int tsize,
					   final DynImpactAsymmetry_pi ref)
{
    final int ntm=1,npm=2*neq+2;
    final QVMDataL o=QVMDataL.calculateQVMDataL(ntm,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new DynImpactAsymmetry_pi(neq,o,ref!=null?ref.sst:null);
}
@Override
protected IMomentsData getIMomentsData(final int n){return IMomentsData.get_PriceMonomials_IMomentsData(n,odata,getAccess(),getABasis());}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){
    return new OrthogonalPolynomialsMonomialsBasisX(1,-getAccess().getPLast(odata)); // getPLast does have p_offset included
}
/** Different meaning, but use anyway.
 */
double [] getEDPsi(final double [] psi){return M.B.getDifferentiated(psi);}

static class DynImpactAsymmetry_piTest extends DynImpactAsymmetryTest<QVMDataL> {
DynImpactAsymmetry_piTest(final long initrandom)
{
    super("DynImpactAsymmetry_pi",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
@Override
boolean testRadau(){return false;}
@Override
void test_Extra(){
    final DynImpactAsymmetry_pi q=getDynImpactAsymmetry();
    final QkPnLpi_PBasis s=getQkPnLpi_PBasis();
    EVXData.matchEVXData(q.I,s.pnldiskewness_nodI.I,false,eps);
    final double p_offset_shift=s.pnldiskewness_nodI.p_offset-q.p_offset;

    UnitTests.checkDoubleRel(q.dia_average.mL.pi,"q.dia_average.mL.pi",s.pnldiskewness_nodI.movaver_S.xa+p_offset_shift,"s.s.pnldiskewness_nodI.movaver_S.xa",eps);
    UnitTests.checkDoubleRel(q.dia_IH.mL.pi,"q.dia_IH.mL.pi",s.pnldiskewness_nodI.IHf_S.xa+p_offset_shift,"s.pnldiskewness_nodI.IHf_S.xa",eps);
    UnitTests.checkDoubleRel(q.dia_IH.mL.pt,"q.dia_IH.mL.pt",s.pnlvarmupi.pt_IH+p_offset_shift,"s.pnlvarmupi.pt_IH",eps);
    UnitTests.checkDoubleRel(q.dia_IH.mL.pi,"q.dia_IH.mL.pi",s.pnlvarmupi.p_IH+p_offset_shift,"s.pnlvarmupi.p_IH",eps);

    System.err.println(name+":DynImpactAsymmetry_piTest:test_Extra(): test DynImpactAsymmetry_pi OK neq="+neq+" eps="+eps);
}

ImpactQVM<QVMDataL> getImpactQVM(){return null; /* Skip this test. */}
QkPnLpi_PBasis getQkPnLpi_PBasis(){return QkPnLpi_PBasis.calculate(neq,currenttime,tau,tb,tb.length,null);}
DynImpactAsymmetry_pi getDynImpactAsymmetry(){return calculate(neq,currenttime,tau,tb,tb.length,null);}


} // END of class DynImpactAsymmetry_piTest

public static void main(String [] args)
{
    new DynImpactAsymmetry_piTest(1274L).runTests(2,5);
} // END main



} // END OF class DynImpactAsymmetry_pi
