package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynIndPnL.java,v 1.7 2020/09/27 20:33:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynIndPnL {
    final EVXData I;
    final int ind_t,ind_i;
    final double P_IH,P_ind_t,Pd_ind_t,P_ind_i,Pd_ind_i;
    DynIndPnL(final EVXData I,
	      final int ind_t,final int ind_i,
	      final double P_IH,final double P_ind_t,final double Pd_ind_t,final double P_ind_i,final double Pd_ind_i)
    {
	this.I=I;
	this.ind_t=ind_t;
	this.ind_i=ind_i;
	this.P_IH=P_IH;
	this.P_ind_t=P_ind_t;
	this.Pd_ind_t=Pd_ind_t;
	this.P_ind_i=P_ind_i;
	this.Pd_ind_i=Pd_ind_i;
    }
/**
 *  Variate <psi|pI|psi>/<psi|psi>.
 */
static DynIndPnL get_DynIndPnL(final int n,
			       final double [] QQi,final double [] QQt,final double [] QQpi,
			       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    int ind_t=0,ind_i=0,ind_IH=n-1;
    double v_t=0,v_i=0;
    for(int k=0;k<n;k++)
    {
	final double p2x0=Math.pow(M.B.getD0(I.getPsi(k),I.x0),2),Ip2x0=I.ev[k]*p2x0;
	if(p2x0>=v_t){v_t=p2x0;ind_t=k;}
	if(Ip2x0>=v_i){v_i=Ip2x0;ind_i=k;}
    }
    final double P_ind_t=I.getFPsi(ind_t,QQpi)/I.ev[ind_t],
	P_ind_i=I.getFPsi(ind_i,QQpi)/I.ev[ind_i],
	P_IH=I.getFPsi(ind_IH,QQpi)/I.ev[ind_IH];
    final double [] Dpsi_t=VOLATILITY_EDPsi.getDPsi(I.getPsi(ind_t));
    final double Pd_ind_t=I.getFPsiPsi(ind_t,Dpsi_t,QQpi)/I.getFPsiPsi(ind_t,Dpsi_t,QQi);
    final double [] Dpsi_i=VOLATILITY_EDPsi.getDPsi(I.getPsi(ind_i));
    final double Pd_ind_i=I.getFPsiPsi(ind_i,Dpsi_i,QQpi)/I.getFPsiPsi(ind_i,Dpsi_i,QQi);
    return new DynIndPnL(I,ind_t,ind_i,P_IH,P_ind_t,Pd_ind_t,P_ind_i,Pd_ind_i);
}
}
