package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

class DynM0PnL {

    final double p_IH,avar_p_IH,pb_IH,ib_IH;
    DynM0PnL(final double p_IH,final double avar_p_IH,final double pb_IH,final double ib_IH)
    {
	this.p_IH=p_IH;
	this.avar_p_IH=avar_p_IH;
	this.pb_IH=pb_IH;
	this.ib_IH=ib_IH;
    }


static DynM0PnL get_DynM0PnL(
    final int n,
    final double [] QQi,final double [] QQt, final double [] QQpi,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");
    final double [] basis=EdComm.getIRadauBasisPsi0IOrthogonalFromQQ(n,QQi,QQt,M);
    final int ind_toremove=0;
    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis);
    final double [] bpsi=new double [n];
    System.arraycopy(bQQpi,n*ind_toremove,bpsi,0,n);
    bpsi[ind_toremove]=0;
    TMatr.scaleVectorInPlace(bpsi,1/Math.sqrt(TMatr.mulVV(bpsi,bpsi)));
    double ib_IH=0;for(int k=0;k<n;k++) ib_IH+=basis[k]*Math.pow(bpsi[k],2);
    final double p_IH=bQQpi[ind_toremove*n+ind_toremove]/basis[ind_toremove],
	avar_p_IH=DeprecatedHelperTMatr.mulVVo(n,bpsi,0,bQQpi,n*ind_toremove),
	pb_IH=TMatr.getAMatrA(bpsi,bQQpi,n)/ib_IH;
    return new DynM0PnL(p_IH,avar_p_IH,pb_IH,ib_IH);
}


static DynM0PnL get_v2_DynM0PnL(
    final int n,
    final double [] QQi,final double [] QQt, final double [] QQpi,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");

    final double [] basis=EdComm.getIRadauBasisPsi0IOrthogonalFromQQ(n,QQi,QQt,M);
    final int ind_toremove=0;

    double pb_IH=0,ib_IH=0,s_norm=0;
    for(int k=0;k<n;k++)
    {
	if(k==ind_toremove) continue;
	final double mk=TMatr.getAMatrB(n,basis,n+ind_toremove*n,basis,n+k*n,QQpi,n);
	for(int l=0;l<n;l++)
	{
	    if(l==ind_toremove) continue;
	    final double a=mk*TMatr.getAMatrB(n,basis,n+l*n,basis,n+ind_toremove*n,QQpi,n);
	    s_norm+=a*TMatr.getAMatrB(n,basis,n+k*n,basis,n+l*n,QQt,n); // only k==l are non-zero.
	    ib_IH+=a*TMatr.getAMatrB(n,basis,n+k*n,basis,n+l*n,QQi,n);  // only k==l are non-zero.
	    pb_IH+=a*TMatr.getAMatrB(n,basis,n+k*n,basis,n+l*n,QQpi,n); // all (k,l) may be non-zero.
	}
	if(true)
	{
	    System.err.println("<psi"+ind_toremove+"|I|psi"+k+">="+TMatr.getAMatrB(n,basis,n+ind_toremove*n,basis,n+k*n,QQi,n));
	}
    }
    pb_IH/=ib_IH;
    ib_IH/=s_norm;
    final double p_IH=TMatr.getAMatrA(n,basis,n+ind_toremove*n,QQpi,n)/basis[ind_toremove],
	avar_p_IH=Math.sqrt(s_norm);

    return new DynM0PnL(p_IH,avar_p_IH,pb_IH,ib_IH);
}


public static void matchDynM0PnL(final DynM0PnL a,final DynM0PnL b,final double eps)
{
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(Math.abs(a.avar_p_IH),"a.avar_p_IH",Math.abs(b.avar_p_IH),"b.avar_p_IH",eps);
    UnitTests.checkDoubleRel(a.ib_IH,"a.ib_IH",b.ib_IH,"b.ib_IH",eps);
    UnitTests.checkDoubleRel(a.pb_IH,"a.pb_IH",b.pb_IH,"b.pb_IH",eps);

    System.err.println("DynM0PnL:matchDynM0PnL test OK  eps="+eps);
}


}
