package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

class DynMPnL {

    final double p_IH,avar_p_IH,pb_IH,ib_IH;
    final EVXData I;
    DynMPnL(final EVXData I,final double p_IH,final double avar_p_IH,final double pb_IH,final double ib_IH)
    {
	this.I=I;
	this.p_IH=p_IH;
	this.avar_p_IH=avar_p_IH;
	this.pb_IH=pb_IH;
	this.ib_IH=ib_IH;
    }

private static final boolean PRINT_DIAG=true;

static DynMPnL get_DynMPnL(
    final int n,
    final double [] QQi,final double [] QQt, final double [] QQpi,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int ind_toremove=n-1;
    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev);
    final double [] bpsi=new double [n];
    System.arraycopy(bQQpi,n*ind_toremove,bpsi,0,n);
    bpsi[ind_toremove]=0;
    TMatr.scaleVectorInPlace(bpsi,1/Math.sqrt(TMatr.mulVV(bpsi,bpsi)));
    double ib_IH=0;for(int k=0;k<n;k++) ib_IH+=I.ev[k]*Math.pow(bpsi[k],2);
    final double p_IH=bQQpi[ind_toremove*n+ind_toremove]/I.ev[ind_toremove],
	avar_p_IH=DeprecatedHelperTMatr.mulVVo(n,bpsi,0,bQQpi,n*ind_toremove),
	pb_IH=TMatr.getAMatrA(bpsi,bQQpi,n)/ib_IH;
    return new DynMPnL(I,p_IH,avar_p_IH,pb_IH,ib_IH);
}

static DynMPnL get_v2_DynMPnL(
    final int n,
    final double [] QQi,final double [] QQt, final double [] QQpi,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");

    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int ind_toremove=n-1;
    final double p_IH=I.getFPsiPsi(ind_toremove,ind_toremove,QQpi)/I.ev[ind_toremove];
    final double [] beta=new double [n];
    for(int k=0;k<n;k++) beta[k]=I.getFPsiPsi(ind_toremove,k,QQpi);
    beta[ind_toremove]=0;
    TMatr.scaleVectorInPlace(beta,1/Math.sqrt(TMatr.mulVV(beta,beta)));
    final double [] psibeta=new double [n];
    for(int k=0;k<n;k++)
    {
	DeprecatedHelperTMatr.sumP1(psibeta,I.ev,n+k*n,n,beta[k]);
    }
    final double avar_p_IH=I.getFPsiPsi(ind_toremove,psibeta,QQpi),
	ib_IH=TMatr.getAMatrA(psibeta,QQi,n),
	pb_IH=TMatr.getAMatrA(psibeta,QQpi,n)/ib_IH;

    if(PRINT_DIAG)
    {
	final double [] psi=I.getPsi(ind_toremove);
	System.err.println(
	    "DynMPnL:get_v2_DynMPnL:avar_p_IH^2="+Math.pow(avar_p_IH,2)+"\n"+
	    "<psi|psibeta>="+  TMatr.getAMatrB(psi,psibeta,QQt,n)+" must be =0\n"+
	    "<psi|I|psibeta>="+TMatr.getAMatrB(psi,psibeta,QQi,n)+" must be =0\n"+
	    "<psibeta|I|psibeta>="+TMatr.getAMatrA(psibeta,QQi,n)+" must be < "+I.ev[ind_toremove]+"\n"+
	    "<psi|I|psi>="+TMatr.getAMatrA(psi,QQi,n)+" must be = "+I.ev[ind_toremove]
			   );
    }

    return new DynMPnL(I,p_IH,avar_p_IH,pb_IH,ib_IH);
}


static DynMPnL get_v3_DynMPnL(
    final int n,
    final double [] QQi,final double [] QQt, final double [] QQpi,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");

    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int ind_toremove=n-1,nd=n-1;
    final double p_IH=I.getFPsiPsi(ind_toremove,ind_toremove,QQpi)/I.ev[ind_toremove];
    final double [] bQQpi=new double [nd*nd];
    for(int k=0,o=0;k<n;k++)
    {
	if(k==ind_toremove) continue;
	final double fk=I.getFPsiPsi(ind_toremove,k,QQpi);
	for(int l=0;l<n;l++)
	{
	    if(l==ind_toremove) continue;
	    final double fl=I.getFPsiPsi(ind_toremove,l,QQpi);
	    bQQpi[o++]=fk*fl;
	}
    }

    final int e=nd-1;
    final double [] evb=M.EV.getEV(nd,bQQpi,true),psie=EVXData.getPsiFromBasis(e,evb,nd);
    final double avar_p_IH=Math.sqrt(evb[e]);
    double ib_IH=0,pb_IH=0;
    for(int k=0,kj=0;k<n;k++) {
	if(k==ind_toremove) continue;
	final double pk=psie[kj++];
	for(int l=0,lj=0;l<n;l++)
	{
	    if(l==ind_toremove) continue;
	    final double pl=psie[lj++];
	    pb_IH+=pk*pl*I.getFPsiPsi(l,k,QQpi);
	    ib_IH+=pk*pl*I.getFPsiPsi(l,k,QQi);
	}
    }
    pb_IH/=ib_IH;


    if(PRINT_DIAG)
    {
	System.err.println("DynMPnL:get_v3_DynMPnL:evb:="+java.util.Arrays.toString(java.util.Arrays.copyOf(evb,nd))+" there should be a single (=max) non-zero eignevalue");
    }

    return new DynMPnL(I,p_IH,avar_p_IH,pb_IH,ib_IH);
}



static DynMPnL get_v4_DynMPnL(
    final int n,
    final double [] QQi,final double [] QQt, final double [] QQpi,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");

    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int ind_toremove=n-1;
    final double [] psi=I.getPsi(ind_toremove),psibeta=TMatr.getMatr_x_A(n,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),TMatr.getMatr_x_A(n,QQpi,psi));
    TMatr.sumP1(psibeta,psi,-TMatr.getAMatrB(psibeta,psi,QQt,n));
    TMatr.scaleVectorInPlace(psibeta,1/Math.sqrt(TMatr.getAMatrA(psibeta,QQt,n)));

    final double p_IH=TMatr.getAMatrA(psi,QQpi,n)/I.ev[ind_toremove],
	avar_p_IH=TMatr.getAMatrB(psi,psibeta,QQpi,n),
	ib_IH=TMatr.getAMatrA(psibeta,QQi,n),
	pb_IH=TMatr.getAMatrA(psibeta,QQpi,n)/ib_IH;

    if(PRINT_DIAG)
    {
	System.err.println(
	    "DynMPnL:get_v4_DynMPnL:avar_p_IH^2="+Math.pow(avar_p_IH,2)+"\n"+
	    "<psi|psibeta>="+  TMatr.getAMatrB(psi,psibeta,QQt,n)+" must be =0\n"+
	    "<psi|I|psibeta>="+TMatr.getAMatrB(psi,psibeta,QQi,n)+" must be =0\n"+
	    "<psibeta|I|psibeta>="+TMatr.getAMatrA(psibeta,QQi,n)+" must be < "+I.ev[ind_toremove]+"\n"+
	    "<psi|I|psi>="+TMatr.getAMatrA(psi,QQi,n)+" must be = "+I.ev[ind_toremove]
			   );
    }

    return new DynMPnL(I,p_IH,avar_p_IH,pb_IH,ib_IH);
}


static DynMPnL get_v5_DynMPnL(
    final int n,
    final double [] QQi,final double [] QQt, final double [] QQpi,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");

    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int ind_toremove=n-1;

    double pb_IH=0,ib_IH=0,s_norm=0;
    for(int k=0;k<n;k++)
    {
	if(k==ind_toremove) continue;
	final double mk=I.getFPsiPsi(ind_toremove,k,QQpi);
	for(int l=0;l<n;l++)
	{
	    if(l==ind_toremove) continue;
	    final double a=mk*I.getFPsiPsi(l,ind_toremove,QQpi);
	    s_norm+=a*I.getFPsiPsi(k,l,QQt); // only k==l are non-zero.
	    ib_IH+=a*I.getFPsiPsi(k,l,QQi);  // only k==l are non-zero.
	    pb_IH+=a*I.getFPsiPsi(k,l,QQpi); // all (k,l) may be non-zero.
	}
    }
    pb_IH/=ib_IH;
    ib_IH/=s_norm;
    final double p_IH=I.getFPsiPsi(ind_toremove,ind_toremove,QQpi)/I.ev[ind_toremove],
	avar_p_IH=Math.sqrt(s_norm);

    return new DynMPnL(I,p_IH,avar_p_IH,pb_IH,ib_IH);
}


public static void matchDynMPnL(final DynMPnL a,final DynMPnL b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,false,eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(Math.abs(a.avar_p_IH),"a.avar_p_IH",Math.abs(b.avar_p_IH),"b.avar_p_IH",eps);
    UnitTests.checkDoubleRel(a.ib_IH,"a.ib_IH",b.ib_IH,"b.ib_IH",eps);
    UnitTests.checkDoubleRel(a.pb_IH,"a.pb_IH",b.pb_IH,"b.pb_IH",eps);

    System.err.println("DynMPnL:matchDynMPnL test OK neq="+a.I.neq+" eps="+eps);
}


}
