package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

class DynMUPnL {
    final double I_0,I_L,I_H,pi_IH,pt_IH,pmu_IH,pas_IH;

    DynMUPnL(final int n,
	     final double [] QQi,final double [] QQt, final double [] QQpi, final double [] QQpt,
	     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final boolean flag_0)
    {

	final double [] basis;
	final int iL,iH;
	if(flag_0)
	{
	    if(n<2) throw new IllegalArgumentException("n="+n+"<2");
	    final int i0=0;iL=1;iH=n-1;
	    basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
	    I_0=basis[i0];
	}
	else
	{
	    iL=0;iH=n-1;
	    basis=M.EV.getGEV_Generalized(n,QQi,QQt,true);
	    I_0=Double.NaN;
	}
	this.I_L=basis[iL];
	this.I_H=basis[iH];
	final double [] QQpmu=TMatr.sumAcB(QQpi,-I_L,QQpt),
	    psi_IL=EVXData.getPsiFromBasis(iL,basis,n),psi_IH=EVXData.getPsiFromBasis(iH,basis,n);
	this.pi_IH=TMatr.getAMatrA(psi_IH,QQpi,n)/I_H;
	this.pt_IH=TMatr.getAMatrA(psi_IH,QQpt,n);
	this.pmu_IH=TMatr.getAMatrA(psi_IH,QQpmu,n)/(I_H-I_L);
	this.pas_IH=(TMatr.getAMatrA(psi_IH,QQpmu,n)-TMatr.getAMatrA(psi_IL,QQpmu,n))/(I_H-I_L);
	if(false)
	{
	    final double [] QQmu=TMatr.sumAcB(QQi,-I_L,QQt);
	    System.err.println("<psi_IL|QQmu|psi_IL>="+TMatr.getAMatrA(psi_IL,QQmu,n)+" must be =0");
	    System.err.println("<psi_IH|QQmu|psi_IH>="+TMatr.getAMatrA(psi_IH,QQmu,n)+" must be ="+(I_H-I_L));
	    System.err.println("<psi_IH|QQpmu|psi_IH>="+TMatr.getAMatrA(psi_IH,QQpmu,n));
	}
    }
}
