package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: DynMicroImpact.java,v 1.55 2022/07/15 19:57:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class DynMicroImpact<DTYPE> {

public static class State {
    public final long seq;
    State(final State prev)
    {
	this.seq=(prev==null?0:1+prev.seq);
    }
}
public final State sst;

public final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;

public final DMI dmiIH;
public final MuDynMicroImpact mudmi;
public final Mu2DynMicroImpact mu2dmi;
public final PDynMicroImpact pdmi;
public final DynMicroVT dmvt;
public final Mu2PXImpact mu2px;
public final Mu2PinIL mu2pinil;
final InIStateV inistateV;
final InIStateVo inistateVo;
final InIStatePI inistatePI;
final InIStatePIo inistatePIo;

final InIStatePIf inistatePIfF,inistatePIfB;
final InIStatePIf2 inistatePIf2;
final DTYPE odata;

DynMicroImpact(final int n,final DTYPE odata,final State rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.odata=odata;
    this.M=getABasis();
    final IMomentsData mm=getIMomentsData(n);
    // Calculations in IH state (old result).
    this.dmiIH=new DMI(mm,M);
    // The measure dmu=dv-lam*dt and a repeat of some calculations above.
    this.mudmi=MuDynMicroImpact.get_MuDynMicroImpact(mm,M,0);
    // Try "symmetric" answer.
    this.mu2dmi=new Mu2DynMicroImpact(mm,M,0);
    // Try dp/dt=lambda dV/dt, find min/max lambda and the price p, corresponding to it.
    this.pdmi=PDynMicroImpact.get_PDynMicroImpact(mm,M,0);
    // Try V/T-> max.
    this.dmvt=mm.QQV==null?null:new DynMicroVT(mm,M,0);
    // Dry different kind of impact
    this.mu2px=new Mu2PXImpact(mm,M,0);
    this.mu2pinil=new Mu2PinIL(mm.p_last,dmiIH.I.psiL,dmiIH.I.sL,
			       mm.QQi,mm.QQt,
			       mm.QQi,mm.QQpi,mm.QQp2i,mm.QQt_sample,mm.QQpt_sample,mm.QQp2t_sample,M,0);
    inistateV=mm.QQV==null?null:new InIStateV(mm,M,0);
    inistateVo=mm.QQV==null?null:new InIStateVo(n,mm.p_offset,mm.QQt,mm.QQi,mm.QQpi,mm.QQV,mm.QQpV,mm.QQp2V,mm.QQp3V,M);
    inistatePI=new InIStatePI(mm,M,0);
    inistatePIo=new InIStatePIo(n,mm.p_offset,mm.p_last,mm.QQt,mm.QQi,mm.QQpi,M,0);

    inistatePIfF=new InIStatePIf(mm,M,true,0);
    inistatePIfB=new InIStatePIf(mm,M,false,0);
    inistatePIf2=new InIStatePIf2(mm,M,0);

    this.sst=new State(rst);
}
protected IMomentsData getIMomentsData(final int n){return IMomentsData.get_IMomentsData(n,M,odata,getAccess());}
abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();

/** A class for unit test.
*/
static abstract class DynMicroImpactTest<DTYPE> extends QTest {

abstract DynMicroImpact<DTYPE> getDynMicroImpact();
abstract ImpactQVM<DTYPE> getImpactQVM();
abstract IAImpact<DTYPE> getIAImpact();

DynMicroImpactTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_Extra(){}

void printSomeData()
{
    final int n=neq;
    final DynMicroImpact<DTYPE> q=getDynMicroImpact();
    final IMomentsData mm=q.getIMomentsData(n);
    System.err.println("QQt       ="+java.util.Arrays.toString(mm.QQt));
    System.err.println("QQt_sample="+java.util.Arrays.toString(mm.QQt_sample));
    System.err.println("QQT       ="+java.util.Arrays.toString(mm.QQT));
    System.err.println("QQT_sample="+java.util.Arrays.toString(mm.QQT_sample));
}

void test_ImpactQVM()
{
    final int n=neq;
    final DynMicroImpact<DTYPE> q=getDynMicroImpact();
    final ImpactQVM<DTYPE> s=getImpactQVM();
    if(true&&s!=null)
    { // a selftest
	final IMomentsData mm=q.getIMomentsData(n);
	DMI.matchMuDynMicroImpact(q.dmiIH,q.mudmi,eps);
	MuDynMicroImpact.matchMuDynMicroImpact(q.mudmi,MuDynMicroImpact.get_MuDynMicroImpact(mm,q.M,eps),eps);
	PDynMicroImpact.get_PDynMicroImpact(mm,q.M,eps);
	if(q.dmvt!=null) DynMicroVT.matchDynMicroVT(q.dmvt,new DynMicroVT(mm,q.M,eps),eps);
	MuDynMicroImpact.matchMu2DynMicroImpact(q.mudmi,new Mu2DynMicroImpact(mm,q.M,eps),eps);
	Mu2PXImpact.matchMu2PXImpact(new Mu2PXImpact(mm,q.M,eps),q.mu2px,eps);
	DynMicroVT.matchInIStateV(q.dmvt,new InIStateV(mm,q.M,eps),eps);
	InIStateV.matchInIStateVo(q.inistateV,q.inistateVo,eps);
	InIStatePI.matchInIStatePIo(q.inistatePI,new InIStatePIo(n,mm.p_offset,mm.p_last,mm.QQt,mm.QQi,mm.QQpi,q.M,eps),eps);
	new InIStatePI(mm,q.M,eps);
	new InIStatePIf(mm,q.M,true,eps);
	new InIStatePIf(mm,q.M,false,eps);

	new InIStatePIf2(mm,q.M,eps);
    }
    DMI.matchIAImpact(q.dmiIH,getIAImpact(),eps);

    if(s==null)
    {
	System.err.println(name+":DynMicroImpactTest.test_ImpactQVM(): skipping test OK n="+n+" as getImpactQVM()=null eps="+eps);
	return;
    }
    matchImpactQVM(q,s,eps);

    System.err.println(name+":DynMicroImpactTest.test_ImpactQVM(): test OK n="+n+" eps="+eps);
}


static void matchImpactQVM(final DynMicroImpact<?> q,final ImpactQVM<?> s,final double eps)
{

    EVXData.matchEVXData(q.dmiIH.I,s.pnldiskewness_nodI.I,false,eps);
    Skewness.matchSkewness(q.dmiIH.SK_P_IH.getSameSkewnessXShifted(q.dmiIH.p_offset),s.pnldiskewness_nodI.IHf_S.getSameSkewnessXShifted(s.pnldiskewness_nodI.p_offset),eps);
    Skewness.matchSkewness(q.dmiIH.SK_P_movaver.getSameSkewnessXShifted(q.dmiIH.p_offset),s.pnldiskewness_nodI.movaver_S.getSameSkewnessXShifted(s.pnldiskewness_nodI.p_offset),eps);
    UnitTests.checkDoubleRel(q.dmiIH.dpdt_orig_IH,"q.dmiIH.dpdt_orig_IH",s.ist.dp_orig_IH,"s.ist.dp_orig_IH",eps);
    UnitTests.checkDoubleRel(q.pdmi.dpdt_orig_IH,"q.pdmi.dpdt_orig_IH",s.ist.dp_orig_IH,"s.ist.dp_orig_IH",eps);

    final double p_offset_shift=s.iex.p_offset-q.mudmi.p_offset;
    UnitTests.checkDoubleRel(q.mudmi._pt_IH,"q.mudmi._pt_IH",s.iex.pt_IH+p_offset_shift,"s.iex.pt_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(q.mudmi.p_IH,"q.mudmi.p_IH",s.iex.p_IH+p_offset_shift,"s.iex.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(q.pdmi.p_IH,"q.pdmi.p_IH",s.iex.p_IH+p_offset_shift,"s.iex.p_IH+p_offset_shift",eps);
    PnLVar3Mu.matchMuDynMicroImpact(s.pnlvar3mu,q.mudmi,eps);

    System.err.println("DynMicroImpact.matchImpactQVM test OK n="+q.dmiIH.I.neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":DynMicroImpactTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Extra();
    test_ImpactQVM();
    printSomeData();

    System.err.println(name+":DynMicroImpactTest.test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_min,final int neq_max)
{
    System.err.println(name+":DynMicroImpactTest: starting tests; ver=$Id: DynMicroImpact.java,v 1.55 2022/07/15 19:57:10 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=neq_min;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":DynMicroImpactTest.main: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: DynMicroImpact.java,v 1.55 2022/07/15 19:57:10 mal Exp $");
}
} // END OF class DynMicroImpactTest




}// END OF class DynMicroImpact
