package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.Skewness;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: DynMicroImpactMonomials.java,v 1.14 2018/06/01 07:02:05 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class DynMicroImpactMonomials extends DynMicroImpact<QVMData> {
DynMicroImpactMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static DynMicroImpactMonomials calculate(final int neq,
					   final long currenttime,
					   final long tau,
					   final Trade [] tb,
					   final int tsize,
					   final DynMicroImpactMonomials ref)
{
    final int nvm=1,npm=4,nr=0;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1+nr,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new DynMicroImpactMonomials(neq,o,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}

static class DynMicroImpactMonomialsTest extends DynMicroImpactTest<QVMData> {
DynMicroImpactMonomialsTest(final long initrandom)
{
    super("DynMicroImpactMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}



@Override
void test_Extra(){
    // These to be identical
    final DynMicroImpactMonomials q=getDynMicroImpact();
    final DynMicroImpactLaguerre s=getDynMicroImpactLaguerre();

    matchImpactQVM(q,getImpactQVMLaguerre(),eps);
    DMI.matchDMI(q.dmiIH,s.dmiIH,eps);
    MuDynMicroImpact.matchMuDynMicroImpact(q.mudmi,s.mudmi,eps);
    DynMicroVT.matchDynMicroVT(q.dmvt,s.dmvt,eps);
    MuDynMicroImpact.matchMu2DynMicroImpact(q.mudmi,s.mu2dmi,eps);
    Mu2PXImpact.matchMu2PXImpact(q.mu2px,s.mu2px,eps);
    DynMicroVT.matchInIStateV(q.dmvt,s.inistateV,eps);
    DynMicroVT.matchInIStateV(s.dmvt,q.inistateV,eps);
    InIStateV.matchInIStateVo(q.inistateV,s.inistateVo,eps);
    InIStateV.matchInIStateVo(s.inistateV,q.inistateVo,eps);
    InIStatePI.matchInIStatePIo(q.inistatePI,s.inistatePIo,eps);
    InIStatePI.matchInIStatePIo(s.inistatePI,q.inistatePIo,eps);

    System.err.println(name+":DynMicroImpactMonomialsTest:test_Extra(): test DynMicroImpactLaguerre OK neq="+neq+" eps="+eps);
}


DynMicroImpactMonomials getDynMicroImpact(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DynMicroImpactLaguerre getDynMicroImpactLaguerre(){return DynMicroImpactLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
ImpactQVMMonomials getImpactQVM(){return ImpactQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}
IAImpactMonomials getIAImpact(){return IAImpactMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}
ImpactQVMLaguerre getImpactQVMLaguerre(){return ImpactQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}


} // END of class DynMicroImpactMonomialsTest

public static void main(String [] args)
{
    new DynMicroImpactMonomialsTest(11127L).runTests(2,5);
} // END main



} // END OF class DynMicroImpactMonomials
