package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;


/** <V\psi^2>=\lambda<T\psi^2> is back. Similar to DynVTPnL
 * but now with the micro impact.
 *  @version $Id: DynMicroVT.java,v 1.14 2022/07/28 22:30:48 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynMicroVT {
    final EVXData VT,V;
    final double p_offset,pi_VTH,nDpV_VTH,nDpT_VTH,DpV_VTH,DpV_L,DpV_H;
    DynMicroVT(final IMomentsData mm,
	       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	       final double eps)
    {
	final int n=mm.n,ixH=n-1;
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");
	p_offset=mm.p_offset;
	this.V=EVXData.getEVXDataFromQQ(n,mm.QQV,mm.QQt,M);
	// Inverse sing to have _QQT positively defined, QQT is the (T-T0) operator.
	final double [] _QQV=TMatr.getVectorScaled(mm.QQV,-1),
	    _QQT=TMatr.getVectorScaled(/*true?mm.QQT_sample:*/mm.QQT,-1),
	    _QQDpV=TMatr.getVectorScaled(mm.QQDpV,-1);
	this.VT=EVXData.getEVXDataFromQQ(n,_QQV,_QQT,M);
	pi_VTH=VT.getFPsi(ixH,mm.QQpi)/VT.getFPsi(ixH,mm.QQi);
	DpV_VTH=VT.getFPsi(ixH,_QQDpV);
	nDpV_VTH=VT.getFPsi(ixH,mm.QQDpV)/VT.getFPsi(ixH,mm.QQV);
	nDpT_VTH=VT.getFPsi(ixH,mm.QQDpT)/VT.getFPsi(ixH,mm.QQT_sample);
	final double [] _bQQDpV=EVXData.convertMatrFromQtoB_Basis(n,_QQDpV,VT.ev);
	final double [] bbasis=M.EV.getEV(n,_bQQDpV,true);
	DpV_L=bbasis[0];
	DpV_H=bbasis[n-1];
	if(eps>0)
	{
	    // Check eigenvalues problem is solved OK.
	    EVXData.test_orthogonality("DynMicroVT.VT.ev",n,VT.ev,_QQV,_QQT,eps);
	    final EVXData DP=EVXData.getEVXDataFromQQ(n,_QQDpV,_QQT,M);
	    UnitTests.checkDoubleArrayRel(DP.getEValues(),"DP.ev",java.util.Arrays.copyOf(bbasis,n),"bbasis",eps);
	}
    }

static void matchDynMicroVT(final DynMicroVT a,final DynMicroVT b,final double eps)
{
    EVXData.matchEVXData(a.V,b.V,false,false,eps);
    EVXData.matchEVXData(a.VT,b.VT,false,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.pi_VTH,"a.pi_VTH",b.pi_VTH+p_offset_shift,"b.pi_VTH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.DpV_VTH,"a.DpV_VTH",b.DpV_VTH,"b.DpV_VTH",eps);
    UnitTests.checkDoubleRel(a.nDpV_VTH,"a.nDpV_VTH",b.nDpV_VTH,"b.nDpV_VTH",eps);
    UnitTests.checkDoubleRel(a.nDpT_VTH,"a.nDpT_VTH",b.nDpT_VTH,"b.nDpT_VTH",eps);
    UnitTests.checkDoubleRel(a.DpV_L,"a.DpV_L",b.DpV_L,"b.DpV_L",eps);
    UnitTests.checkDoubleRel(a.DpV_H,"a.DpV_H",b.DpV_H,"b.DpV_H",eps);

    System.err.println("DynMicroVT:matchDynMicroVT test OK n="+a.VT.neq+" eps="+eps);
}
static void matchInIStateV(final DynMicroVT a,final InIStateV b,final double eps)
{
    EVXData.matchEVXData(a.V,b.V,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    System.err.println("DynMicroVT:matchInIStateV test OK n="+a.V.neq+" eps="+eps);
}
}

