package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

class DynPnL {
    final int n;
    final double pEQ,pEB;
    final double pnlL,pnlH,I_pnlL,I_pnlH;
    DynPnL(final int n,
	   final double [] basis,final int basis_offset,
	   final int [] ind_evtoskip,
	   final double [] QQpi,final double [] QQi,final double [] QQt,
	   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	this(n,basis,basis_offset,ind_evtoskip,QQpi,QQi,QQt,TMatr.getAMatrA(n,basis,basis_offset+n*ind_evtoskip[0],QQpi,n)/TMatr.getAMatrA(n,basis,basis_offset+n*ind_evtoskip[0],QQi,n),M);
	if(ind_evtoskip.length!=1) throw new RuntimeException("ind_evtoskip="+java.util.Arrays.toString(ind_evtoskip));
    }

    DynPnL(final int n,
	   final double [] basis,final int basis_offset,
	   final int [] ind_evtoskip,
	   final double [] QQpi,final double [] QQi,final double [] QQt,
	   final double pB,
	   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	this.n=n;
	final int nd=n-ind_evtoskip.length;
	final double [] bbasis=DeprecatedHelperEVXData.getBasisWithVectorsRemoved(n,basis,n,basis_offset,ind_evtoskip);

	final double [] BQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,bbasis,nd,0),
	    BQQi=EVXData.convertMatrFromQtoB_Basis(n,QQi,bbasis,nd,0),
	    BQQt=EVXData.convertMatrFromQtoB_Basis(n,QQt,bbasis,nd,0);

	final double [] BQQpnl=TMatr.sumAcB(BQQpi,-pB,BQQi);

	if(nd<=0){
	    this.pnlL=this.pnlH=this.I_pnlL=this.I_pnlH=Double.NaN;
	} else {
	    final double [] ev=M.EV.getGEV_Generalized(nd,BQQpnl,BQQt,true);
	    final double [] psi_pnlL=EVXData.getPsiFromBasis(0,ev,nd),
		psi_pnlH=EVXData.getPsiFromBasis(nd-1,ev,nd);
	    this.I_pnlL=TMatr.getAMatrA(psi_pnlL,BQQi,nd);
	    this.I_pnlH=TMatr.getAMatrA(psi_pnlH,BQQi,nd);
	    this.pnlL=ev[0];this.pnlH=ev[nd-1];
	}
	this.pEQ=(this.pnlL+this.pnlH)/(I_pnlL+I_pnlH)+pB;
	this.pEB=pB;
    }
}

