package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/** Radau measure is back.
 *  @version $Id: DynR.java,v 1.16 2022/07/14 23:48:04 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynR {
    final double I_0,I_L,I_H,p_IH,vp_IH;
    DynR(final double I_0,final double I_L,final double I_H,final double p_IH,final double vp_IH)
    {
	this.I_0=I_0;
	this.I_L=I_L;
	this.I_H=I_H;
	this.p_IH=p_IH;
	this.vp_IH=vp_IH;
    }

static DynR get_DynR(final int n,
		     final double [] QQi,final double [] QQt,final double [] QQpi,
			   final VolMatrix VOLATILITY_EDPsi,
			   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final int nd=n-1;
	final double [] RQQt=VolMatrix.getRadauMatrix(n,QQt,M),
	    RQQi=VolMatrix.getRadauMatrix(n,QQi,M);
	final double [] evR=M.EV.getGEV_Generalized(nd,RQQi,RQQt,true);
	final int ie=nd-1;
	final double I_L=evR[0],I_H=evR[ie],x0=M.getRadauRoot();

	double p_IH=Double.NaN;
	final double [] psi0=M.B.getX0Moments(n,x0);
	for(int k=0;k<n;k++){psi0[k]/=Math.sqrt(QQt[k+n*k]);}
	for(int k=0;k<nd;k++){
	    final double [] psik=EVXData.getPsiFromBasis(k,evR,nd),Rpsik=M.getRadauMultiplied(psik);
	    if(k==ie) p_IH=TMatr.getAMatrB(psik,Rpsik,QQpi,n)/evR[k];
	    TMatr.sumP1(psi0,Rpsik,-TMatr.getAMatrB(psi0,psik,QQt,n));
	}
	TMatr.scaleVectorInPlace(psi0,1/Math.sqrt(TMatr.getAMatrA(psi0,QQt,n)));
	final double I_0=TMatr.getAMatrA(psi0,QQi,n);
	if(false)
	{
	    for(int k=0;k<nd;k++)
	    {
		final double [] psik=EVXData.getPsiFromBasis(k,evR,nd);
		System.err.println("DynR:<psi0|psik>="+TMatr.getAMatrB(psi0,psik,QQt,n));
		//System.err.println("DynR:<psik|Rpsik>="+TMatr.getAMatrA(psik,RQQt,nd));
	    }
	}
	final double vp_IH=Double.NaN;

	return new DynR(I_0,I_L,I_H,p_IH,vp_IH);
    }
}

