package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/** Radau measure is back.
 *  @version $Id: DynRPnL.java,v 1.31 2022/11/15 10:40:09 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynRPnL {
    final double I_L,I_H,p_IH,var_p_IH,var_p0_IH,var_I_IH,Im_IH,pm_IH,rm_IH,Dpsi2x0,rateT_IH,rateI_IH,f_IH,Df_IH;
    DynRPnL(final double I_L,final double I_H,final double p_IH,final double var_p_IH,final double var_p0_IH,final double var_I_IH,
	    final double Im_IH,final double pm_IH,final double rm_IH,
	    final double Dpsi2x0,final double rateT_IH,final double rateI_IH,
	    final double f_IH,final double Df_IH,final double [] evDIRadau)
    {
	this.I_L=I_L;
	this.I_H=I_H;
	this.p_IH=p_IH;
	this.var_p_IH=var_p_IH;
	this.var_p0_IH=var_p0_IH;
	this.var_I_IH=var_I_IH;
	this.Im_IH=Im_IH;
	this.pm_IH=pm_IH;
	this.rm_IH=rm_IH;
	this.Dpsi2x0=Dpsi2x0;
	this.rateT_IH=rateT_IH;
	this.rateI_IH=rateI_IH;
	this.f_IH=f_IH;
	this.Df_IH=Df_IH;
	this.evDIRadau=evDIRadau;
    }
final double [] evDIRadau;

static DynRPnL get_DynRPnL(final int n,
			   final double [] QQi,final double [] QQt,final double [] QQpi,final double [] QQf,
			   final VolMatrix VOLATILITY_EDPsi,
			   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final int nd=n-1;
	final double [] psiR=new double[nd],trans=new double [nd*n];
	for(int k=0;k<nd;k++)
	{
	    psiR[k]=1;
	    System.arraycopy(M.getRadauMultiplied(psiR),0,trans,k*n,n);
	    psiR[k]=0;
	}

	final double [] RQQi=new double [nd*nd],RQQt=new double [nd*nd];
	DeprecatedHelperTMatr.multiply_MA_x_MB(RQQi,nd,trans,n,0,n,QQi,nd,n,0);
	DeprecatedHelperTMatr.multiply_MA_x_MB(RQQt,nd,trans,n,0,n,QQt,nd,n,0);

	//UnitTests.checkDoubleArrayRel(RQQi,"RQQi",VolMatrix.getRadauMatrix(n,QQi,M),"getRadauMatrix",1e-5);
	//System.err.println("DRadau Matrix="+java.util.Arrays.toString(VOLATILITY_EDPsi.getDRadauMatrix(n,QQi,M)));

	final double [] evR=M.EV.getGEV_Generalized(nd,RQQi,RQQt,true);
	final int ie=nd-1;
	final double I_L=evR[0],I_H=evR[ie];
	final double [] psi=EVXData.getPsiFromBasis(ie,evR,nd),Dpsi=VOLATILITY_EDPsi.getDPsi(psi),
	    Rpsi=M.getRadauMultiplied(psi),DRpsi=VOLATILITY_EDPsi.getDPsi(Rpsi);
	final double p_IH=TMatr.getAMatrB(psi,Rpsi,QQpi,n)/I_H;
	final double [] QQpnl=TMatr.sumAcB(QQpi,-p_IH,QQi);
	final double var_p_IH=TMatr.getAMatrB(Dpsi,Rpsi,QQpnl,n)+TMatr.getAMatrB(psi,DRpsi,QQpnl,n),
	    var_p0_IH=TMatr.getAMatrB(Dpsi,Rpsi,QQpi,n)+TMatr.getAMatrB(psi,DRpsi,QQpi,n),
	    var_I_IH=TMatr.getAMatrB(Dpsi,Rpsi,QQi,n)+TMatr.getAMatrB(psi,DRpsi,QQi,n),
	    Dpsi2x0=Math.pow(M.B.getD0(psi,M.getRadauRoot()),2),
	    rtm_IH=TMatr.getAMatrA(psi,QQt,n),rim_IH=TMatr.getAMatrA(psi,QQi,n),
	    Im_IH=rim_IH/rtm_IH,pm_IH=TMatr.getAMatrA(psi,QQpi,n)/rim_IH,rm_IH=rim_IH/I_H,
	    f_IH=TMatr.getAMatrB(psi,Rpsi,QQf,n),
	    Df_IH=TMatr.getAMatrB(Dpsi,Rpsi,QQf,n)+TMatr.getAMatrB(psi,DRpsi,QQf,n);

	final double [] evDIRadau;
	if(false||true)
	{
	    System.err.println("DynRPnL:I_H="+I_H+" <psi|QQi|Rpsi>="+TMatr.getAMatrB(psi,Rpsi,QQi,n)+" <psi|QQt|Rpsi>="+TMatr.getAMatrB(psi,Rpsi,QQt,n));
	    System.err.println("DynRPnL: <Dpsi|QQt|Rpsi>+<psi|QQt|DRpsi>="+(TMatr.getAMatrB(Dpsi,Rpsi,QQt,n)+TMatr.getAMatrB(psi,DRpsi,QQt,n))+" must be =0");
	    final double [] DRQQi=VOLATILITY_EDPsi.getDRadauMatrix(n,QQi,M),
		evDRi=evDIRadau=M.EV.getGEV_Generalized(nd,DRQQi,RQQt,true),
		psiDIH=EVXData.getPsiFromBasis(nd-1,evDRi,nd);
	    final DynVar1 d=new DynVar1(psi,QQi,QQt,QQpi,M,VOLATILITY_EDPsi);
	    System.err.println("DynRPnL:d.avar_p="+d.avar_p+" var_p_IH="+var_p_IH+
			       " d.DI="+d.DI+" var_I_IH="+var_I_IH+
			       " still incorrect:d.a.x_a_b="+d.a.x_a_b);
	    System.err.println("evDRi[nd-1]="+evDRi[nd-1]);
	    //if(evDRi[nd-1]<0) throw new RuntimeException("evDRi[nd-1]="+evDRi[nd-1]);
	}

	return new DynRPnL(I_L,I_H,p_IH,var_p_IH,var_p0_IH,var_I_IH,Im_IH,pm_IH,rm_IH,Dpsi2x0,rtm_IH,rim_IH,f_IH,Df_IH,evDIRadau);
    }


static DynRPnL get_frommoments_DynRPnL(final int n,
	    final double [] imom,final double [] tmom,final double [] pimom,final double [] fmom,
	    final VolMatrix VOLATILITY_EDPsi,
	    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");
	final int nd=n-1;
	final double [] RQQi=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(imom)),RQQt=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(tmom)),RQQpi=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(pimom));
	final double [] evR=M.EV.getGEV_Generalized(nd,RQQi,RQQt,true);
	final int ie=nd-1;
	final double [] psi=EVXData.getPsiFromBasis(ie,evR,nd);
	final double I_L=evR[0],I_H=evR[ie],p_IH=TMatr.getAMatrA(psi,RQQpi,nd)/I_H;
	final double [] pnlmom=TMatr.sumAcB(pimom,-p_IH,imom);
	final double [] pp=M.B.mult2Pol(psi,psi),r=M.getRadauMultiplied(new double []{1}),
	    Dpp=VOLATILITY_EDPsi.getDPsi(pp),Dr=VOLATILITY_EDPsi.getDPsi(r);
	final double var_p_IH=TMatr.mulVVn1(M.B.mult2Pol(pp,Dr),pnlmom)+TMatr.mulVVn1(M.B.mult2Pol(Dpp,r),pnlmom),
	    var_p0_IH=TMatr.mulVVn1(M.B.mult2Pol(pp,Dr),pimom)+TMatr.mulVVn1(M.B.mult2Pol(Dpp,r),pimom),
	    var_I_IH=TMatr.mulVVn1(M.B.mult2Pol(pp,Dr),imom)+TMatr.mulVVn1(M.B.mult2Pol(Dpp,r),imom),
	    Dpsi2x0=Math.pow(M.B.getD0(psi,M.getRadauRoot()),2),
	    rtm_IH=TMatr.mulVVn1(pp,tmom),rim_IH=TMatr.mulVVn1(pp,imom),
	    Im_IH=rim_IH/rtm_IH,pm_IH=TMatr.mulVVn1(pp,pimom)/rim_IH,rm_IH=rim_IH/I_H,
	    f_IH=TMatr.mulVVn1(M.B.mult2Pol(pp,r),fmom),
	    Df_IH=TMatr.mulVVn1(M.B.mult2Pol(pp,Dr),fmom)+TMatr.mulVVn1(M.B.mult2Pol(Dpp,r),fmom);
	final double [] evDIRadau;
	if(false||true)
	{
	    final double var_t_IH=TMatr.mulVVn1(M.B.mult2Pol(pp,Dr),tmom)+TMatr.mulVVn1(M.B.mult2Pol(Dpp,r),tmom);
	    System.err.println("DynRPnL: <pp|QQt|Dr>+<Dpp|QQt|r>="+var_t_IH+" must be =0");
	    final double [] QQi=M.B.getQQMatr(n,imom),QQt=M.B.getQQMatr(n,tmom),
		QQpi=M.B.getQQMatr(n,pimom),
		DRQQi=VOLATILITY_EDPsi.getDRadauMatrix(n,QQi,M),
		DRQQt=VOLATILITY_EDPsi.getDRadauMatrix(n,QQt,M),
		evDRi=evDIRadau=M.EV.getGEV_Generalized(nd,DRQQi,RQQt,true),
		psiDIH=EVXData.getPsiFromBasis(nd-1,evDRi,nd);
	    final DynVar1 d=new DynVar1(psi,QQi,QQt,QQpi,M,VOLATILITY_EDPsi);
	    //System.err.println("DynRPnL:DRQQt="+java.util.Arrays.toString(DRQQt)+" RQQt="+java.util.Arrays.toString(RQQt));
	    System.err.println("DynRPnL:d.avar_p="+d.avar_p+" var_p_IH="+var_p_IH+
			       " d.DI="+d.DI+" var_I_IH="+var_I_IH+
			       " still incorrect:d.a.x_a_b="+d.a.x_a_b+
			       " evDRi[nd-1]="+evDRi[nd-1]);
	    //if(evDRi[nd-1]<0) throw new RuntimeException("evDRi[nd-1]="+evDRi[nd-1]);
	}
	return new DynRPnL(I_L,I_H,p_IH,var_p_IH,var_p0_IH,var_I_IH,Im_IH,pm_IH,rm_IH,Dpsi2x0,rtm_IH,rim_IH,f_IH,Df_IH,evDIRadau);
    }


static void matchDynRPnL(final DynRPnL a,final DynRPnL b,final double eps)
{
    UnitTests.checkDoubleRel(a.I_L,"a.I_L",b.I_L,"b.I_L",eps);
    UnitTests.checkDoubleRel(a.I_H,"a.I_H",b.I_H,"b.I_H",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.var_p_IH,"a.var_p_IH",b.var_p_IH,"b.var_p_IH",eps);
    UnitTests.checkDoubleRel(a.var_p0_IH,"a.var_p0_IH",b.var_p0_IH,"b.var_p0_IH",eps);
    UnitTests.checkDoubleRel(a.var_I_IH,"a.var_I_IH",b.var_I_IH,"b.var_I_IH",eps);
    UnitTests.checkDoubleRel(a.Im_IH,"a.Im_IH",b.Im_IH,"b.Im_IH",eps);
    UnitTests.checkDoubleRel(a.pm_IH,"a.pm_IH",b.pm_IH,"b.pm_IH",eps);
    UnitTests.checkDoubleRel(a.rm_IH,"a.rm_IH",b.rm_IH,"b.rm_IH",eps);
    UnitTests.checkDoubleRel(a.Dpsi2x0,"a.Dpsi2x0",b.Dpsi2x0,"b.Dpsi2x0",eps);
    UnitTests.checkDoubleRel(a.rateT_IH,"a.rateT_IH",b.rateT_IH,"b.rateT_IH",eps);
    UnitTests.checkDoubleRel(a.rateI_IH,"a.rateI_IH",b.rateI_IH,"b.rateI_IH",eps);
    UnitTests.checkDoubleRel(a.f_IH,"a.f_IH",b.f_IH,"b.f_IH",eps);
    UnitTests.checkDoubleRel(a.Df_IH,"a.Df_IH",b.Df_IH,"b.Df_IH",eps);

    System.err.println("DynRPnL:matchDynRPnL test OK  eps="+eps);
}


}

