package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** P&L via I(p).
 *  @version $Id: DynRPnLI.java,v 1.32 2022/08/05 11:25:32 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynRPnLI {
    final double I_0,I_L,I_H,p_IL,p_IH,px,pas,Ix;
    DynRPnLI(final double I_0,final double I_L,final double I_H,final double p_IL,final double p_IH,final double px,final double pas,final double Ix)
    {
	this.I_0=I_0;this.I_L=I_L;this.I_H=I_H;this.p_IL=p_IL;this.p_IH=p_IH;this.px=px;this.pas=pas;this.Ix=Ix;
    }
    static final boolean PRINT_DIAG=false;

    static DynRPnLI get_DynRPnLI(final int n,
				 final double [] QQi,final double [] QQt, final double [] QQpi,
				 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	final double [] basis=M.EV.getGEV_Generalized(n,QQi,QQt,true),bQQi,
	    bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis);
	if(false) bQQi=EVXData.convertMatrFromQtoB_Basis(n,QQi,basis);
	else bQQi=TMatr.getDiagMatrix(java.util.Arrays.copyOf(basis,n));
	// OK, bQQi & bQQpi are now in proper basis
	final PnLICalc J=new PnLICalc(n,bQQi,bQQpi,M.EV);
	if(false)
	{
	    final double e=1e-4,eps=-1;//1e-3; // negative eps:disable check
	    for(double p=-1;p<1;p+=0.31)
	    {
		J.testDPrint(p,e,eps); // !!! This test clober the state of J
	    }
	}
	J.iterateNewton(0.0);
	final int iL=0,iH=n-1;
	final double px=J.p,Ix=J.I_L,
	    I_0=Double.NaN,
	    I_L=basis[iL],p_IL=TMatr.getAMatrA(n,basis,n+iL*n,QQpi,n)/I_L,
	    I_H=basis[iH],p_IH=TMatr.getAMatrA(n,basis,n+iH*n,QQpi,n)/I_H,
	    pas=(p_IH*I_H-p_IL*I_L)/(I_H-I_L);
	// Note: the pas is a very good approximation of px
	// because it is actually first prder perturbation to I_L=I_H
	// equation determining px.
	if(PRINT_DIAG)
	{
	    System.err.println("get_DynRPnLI:J="+J.getAsText());
	}
	return new DynRPnLI(I_0,I_L,I_H,p_IL,p_IH,px,pas,Ix);
    }


    static DynRPnLI get0_DynRPnLI(final int n,
				  final double [] QQi,final double [] QQt, final double [] QQpi,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	final int i0=0,iL=1,iH=n-1,nd=n-1;
	final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M),bQQi;
	if(!(i0==0)) throw new RuntimeException("Not supported i0="+i0);
	final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis,nd,n+n+0);
	if(false) bQQi=EVXData.convertMatrFromQtoB_Basis(n,QQi,basis,nd,n+n+0);
	else {bQQi=new double [nd*nd];for(int k=0;k<nd;k++) bQQi[k+nd*k]=basis[k+1];}

	// OK, bQQi & bQQpi are now in proper basis
	final PnLICalc J=new PnLICalc(nd,bQQi,bQQpi,M.EV);
	J.iterateNewton(0.0);
	final double px=J.p,Ix=J.I_L,
	    I_0=basis[i0],
	    I_L=basis[iL],p_IL=TMatr.getAMatrA(n,basis,n+iL*n,QQpi,n)/I_L,
	    I_H=basis[iH],p_IH=TMatr.getAMatrA(n,basis,n+iH*n,QQpi,n)/I_H,
	    pas=(p_IH*I_H-p_IL*I_L)/(I_H-I_L);
	// Note: the pas is a very good approximation of px
	// because it is actually first prder perturbation to I_L=I_H
	// equation determining px.
	if(PRINT_DIAG)
	{
	    System.err.println("get0_DynRPnLI:J="+J.getAsText());
	}
	return new DynRPnLI(I_0,I_L,I_H,p_IL,p_IH,px,pas,Ix);
    }


    static DynRPnLI get_DynPnL(final int n,
			       final double [] QQi,final double [] QQt, final double [] QQpi,
			       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	final double [] basis=M.EV.getGEV_Generalized(n,QQi,QQt,true),
	    bQQi,bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis);
	if(false) bQQi=EVXData.convertMatrFromQtoB_Basis(n,QQi,basis);
	else bQQi=TMatr.getDiagMatrix(java.util.Arrays.copyOf(basis,n));

	// OK, bQQi & bQQpi are now in proper basis
	final PnLCalc J=new PnLCalc(n,bQQi,bQQpi,M.EV);
	if(false)
	{
	    final double e=1e-4,eps=1e-3;//1e-3; // negative eps:disable check
	    for(double p=-1;p<1;p+=0.31)
	    {
		J.testDPrint(p,e,eps); // !!! This test clober the state of J
	    }
	}
	J.iterateNewton(0.0);
	final int iL=0,iH=n-1;
	final double px=J.p,Ix=J.getDF(),
	    I_0=Double.NaN,
	    I_L=basis[iL],p_IL=TMatr.getAMatrA(n,basis,n+iL*n,QQpi,n)/I_L,
	    I_H=basis[iH],p_IH=TMatr.getAMatrA(n,basis,n+iH*n,QQpi,n)/I_H,
	    pas=(p_IH*I_H-p_IL*I_L)/(I_H-I_L);
	if(PRINT_DIAG)
	{
	    System.err.println("get_DynPnL:J="+J.getAsText());
	}
	return new DynRPnLI(I_0,I_L,I_H,p_IL,p_IH,px,pas,Ix);
    }

     static DynRPnLI get0_DynPnL(final int n,
				 final double [] QQi,final double [] QQt, final double [] QQpi,
				 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	final int i0=0,iL=1,iH=n-1,nd=n-1;
	final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M),
	    bQQi,bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis,nd,n+n+0);
	if(!(i0==0)) throw new RuntimeException("Not supported i0="+i0);
	if(false) bQQi=EVXData.convertMatrFromQtoB_Basis(n,QQi,basis,nd,n+n+0);
	else {bQQi=new double [nd*nd];for(int k=0;k<nd;k++) bQQi[k+nd*k]=basis[k+1];}

	// OK, bQQi & bQQpi are now in proper basis
	final PnLCalc J=new PnLCalc(nd,bQQi,bQQpi,M.EV);
	J.iterateNewton(0.0);
	final double px=J.p,Ix=J.getDF(),
	    I_0=basis[i0],
	    I_L=basis[iL],p_IL=TMatr.getAMatrA(n,basis,n+iL*n,QQpi,n)/I_L,
	    I_H=basis[iH],p_IH=TMatr.getAMatrA(n,basis,n+iH*n,QQpi,n)/I_H,
	    pas=(p_IH*I_H-p_IL*I_L)/(I_H-I_L);
	if(PRINT_DIAG)
	{
	    System.err.println("get0_DynPnL:J="+J.getAsText());
	}
	return new DynRPnLI(I_0,I_L,I_H,p_IL,p_IH,px,pas,Ix);
    }


}
