package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Radau measure is back.
 *  @version $Id: DynRPnLlast.java,v 1.6 2020/09/27 20:33:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynRPnLlast {
    final double I_pnlL,pnl_pnlL,p_pnlL,I_pnlH,pnl_pnlH,p_pnlH;
    DynRPnLlast(final double I_pnlL,final double pnl_pnlL,
		final double I_pnlH,final double pnl_pnlH)
    {
	this.I_pnlL=I_pnlL;this.pnl_pnlL=pnl_pnlL;this.p_pnlL=pnl_pnlL/I_pnlL;
	this.I_pnlH=I_pnlH;this.pnl_pnlH=pnl_pnlH;this.p_pnlH=pnl_pnlH/I_pnlH;
    }

    static DynRPnLlast get_frommoments_DynRPnLlast(final int n,
						   final double [] momi,final double [] momt, final double [] mompi,
						   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final int nd=n-1;
	final double [] QQi=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(momi)),
	    QQt=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(momt)),
	    QQpi=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(mompi));
	final double [] evE=M.EV.getGEV_Generalized(nd,QQpi,QQt,true);

	final int iL=0,iH=nd-1;
	final double I_pnlL=TMatr.getAMatrA(nd,evE,nd+iL*nd,QQi,nd),
	    pnl_pnlL=evE[iL],
	    I_pnlH=TMatr.getAMatrA(nd,evE,nd+iH*nd,QQi,nd),
	    pnl_pnlH=evE[iH];
	if(false)
	{
	    System.err.println("DynRPnLlast:evE[iL]="+evE[iL]+" test="+TMatr.getAMatrA(nd,evE,nd+iL*nd,QQpi,nd)+" 1="+TMatr.getAMatrA(nd,evE,nd+iL*nd,QQt,nd));
	    System.err.println("DynRPnLlast:evE[iH]="+evE[iH]+" test="+TMatr.getAMatrA(nd,evE,nd+iH*nd,QQpi,nd)+" 1="+TMatr.getAMatrA(nd,evE,nd+iH*nd,QQt,nd));
	}
	return new DynRPnLlast(I_pnlL,pnl_pnlL,I_pnlH,pnl_pnlH);
    }
}
