package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/** Radau measure and infinitesimal shift.
 *  @version $Id: DynRXPnL.java,v 1.12 2022/06/27 07:19:11 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynRXPnL {
    final EVXData ev;
    final double p_IH,pr_IH,ir_IH,tr_IH;
    DynRXPnL(final EVXData ev,final double p_IH,final double pr_IH,final double ir_IH,final double tr_IH)
    {
	this.ev=ev;
	this.p_IH=p_IH;
	this.pr_IH=pr_IH;
	this.ir_IH=ir_IH;
	this.tr_IH=tr_IH;
    }


static double [] getVarRPsi(final double [] psi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    return M.getRadauMultiplied(psi);
}
static double [] getVarRDPsi(final double [] psi,final VolMatrix VOLATILITY_EDPsi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    return TMatr.scaleVectorInPlace(M.getRadauMultiplied(VOLATILITY_EDPsi.getDPsi(psi)),2);
}

static DynRXPnL get_DynRXPnL(final int n,final double [] QQi,final double [] QQt,final double [] QQpi,
			     final boolean flag_DR,
			     final VolMatrix VOLATILITY_EDPsi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final int nd=n-1;
    final EVXData e=EVXData.getEVXDataFromQQ(nd,DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(n,QQi,nd),DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(n,QQt,nd),M);
    final double [] psi=e.psiH,varpsi=flag_DR?getVarRDPsi(psi,VOLATILITY_EDPsi,M):getVarRPsi(psi,M);
    final double p_IH=TMatr.getAMatrA(psi,QQpi,n)/e.sH,
	tr_IH=TMatr.getAMatrB(psi,varpsi,QQt,n),
	jr=TMatr.getAMatrB(psi,varpsi,QQi,n),
	ir_IH=jr/tr_IH,
	pr_IH=TMatr.getAMatrB(psi,varpsi,QQpi,n)/jr;
    if(false && flag_DR)
    {
	final double [] xpsi=M.getxP(psi);
	System.err.println("get_DynRXPnL flag_DR="+flag_DR+" <psi|QQt|varpsi>="+tr_IH+" <psi|x QQt|psi>="+TMatr.getAMatrB(psi,xpsi,QQt,n)+" jr-I_IH*tr_IH="+(jr-e.sH*tr_IH));
    }
    return new DynRXPnL(e,p_IH,pr_IH,ir_IH,tr_IH);
}

    static void matchDynRXPnL(final DynRXPnL a,final DynRXPnL b,final double eps)
    {
	EVXData.matchEVXData(a.ev,b.ev,false,false,eps);
	UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
	UnitTests.checkDoubleRel(a.pr_IH,"a.pr_IH",b.pr_IH,"b.pr_IH",eps);
	UnitTests.checkDoubleRel(a.ir_IH,"a.ir_IH",b.ir_IH,"b.ir_IH",eps);
	UnitTests.checkDoubleRel(a.tr_IH,"a.tr_IH",b.tr_IH,"b.tr_IH",eps);

	System.err.println("DynRXPnL:matchDynRXPnL test OK n="+a.ev.neq+"  eps="+eps);
    }

}
