package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 *  Try <psi|I|Dpsi> eigenfunctions (Aristotle vs Galilei).
 *  @version $Id: DynSD2PnL.java,v 1.7 2020/09/27 20:33:11 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynSD2PnL {

    final EVXData ID;
    final double
	p_DIL,pd_DIL,I_DIL,DI_DIL,avar_p_DIL,
	p_DIH,pd_DIH,I_DIH,DI_DIH,avar_p_DIH,
	dist2_DIL_DIH;

    DynSD2PnL(final EVXData ID,
	     final double p_DIL,final double pd_DIL,final double I_DIL,final double DI_DIL,final double avar_p_DIL,
	     final double p_DIH,final double pd_DIH,final double I_DIH,final double DI_DIH,final double avar_p_DIH,
	      final double dist2_DIL_DIH)
    {
	this.ID=ID;
	this.p_DIL=p_DIL;
	this.pd_DIL=pd_DIL;
	this.I_DIL=I_DIL;
	this.DI_DIL=DI_DIL;
	this.avar_p_DIL=avar_p_DIL;
	this.p_DIH=p_DIH;
	this.pd_DIH=pd_DIH;
	this.I_DIH=I_DIH;
	this.DI_DIH=DI_DIH;
	this.avar_p_DIH=avar_p_DIH;
	this.dist2_DIL_DIH=dist2_DIL_DIH;
    }

/** Try DI.
 */
static DynSD2PnL get_DynSD2PnL(
    final int n,
    final double [] _QQi,final double [] QQt, final double [] _QQpi,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,final K2Distance distance2)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");
    final int iX=n-1;
    final double [] QQDi=VOLATILITY_EDPsi.getDiffMatrix(n,_QQi),QQDpi=VOLATILITY_EDPsi.getDiffMatrix(n,_QQpi);
    final EVXData ID=EVXData.getEVXDataFromQQ(n,QQDi,QQt,M);
    final int iDIL=0,iDIH=n-1;
    final double [] DpsiL=VOLATILITY_EDPsi.getDPsi(ID.psiL),DpsiH=VOLATILITY_EDPsi.getDPsi(ID.psiH);

    final double pDpL=2*ID.getFPsiPsi(iDIL,DpsiL,QQt),
	ID_DIL=ID.sL,pID_DIL=ID.getFPsi(iDIL,QQDpi),p_DIL=pID_DIL/ID_DIL,
	pID2_DIL=2*ID.getFPsiPsi(iDIL,DpsiL,QQDpi),
	pd_DIL=pID2_DIL/(ID.sL*pDpL),
	avar_p_DIL=pID2_DIL-pID_DIL*pDpL,
	pDpH=2*ID.getFPsiPsi(iDIH,DpsiH,QQt),
	ID_DIH=ID.sH,pID_DIH=ID.getFPsi(iDIH,QQDpi),p_DIH=pID_DIH/ID_DIH,
	pID2_DIH=2*ID.getFPsiPsi(iDIH,DpsiH,QQDpi),
	pd_DIH=pID2_DIH/(ID.sH*pDpH),
	avar_p_DIH=pID2_DIH-pID_DIH*pDpH;

    final double dist2_DIL_DIH=distance2.getDistance2(ID.psiL,ID.psiH);

    return new DynSD2PnL(ID,
			 p_DIL,pd_DIL,ID_DIL,ID.sL*pDpL,avar_p_DIL,
			 p_DIH,pd_DIH,ID_DIH,ID.sH*pDpH,avar_p_DIH,
			 dist2_DIL_DIH);
}


public static void testVar1(final DynSD2PnL a,
    final int n,final double [] QQi,final double [] QQt,
    final double [] QQpi,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,final double eps)
{
    final double [] QQDi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi),QQDpi=VOLATILITY_EDPsi.getDiffMatrix(n,QQpi);
    final EVXData ID=EVXData.getEVXDataFromQQ(n,QQDi,QQt,M),I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double ID_sH_bound=I.sH*Math.pow(M.B.getD0(I.psiH,I.x0),2);
    System.err.println("DynSD2PnL:testVar1: ID.sL="+ID.sL+" ID.sH="+ID.sH+" must be >0; low bound="+ID_sH_bound);

    {
    final DynVar1 v=new DynVar1(ID.psiL,QQDi,QQt,QQDpi,VOLATILITY_EDPsi);
    UnitTests.checkDoubleRel(a.p_DIL,"a.p_DIL",v.p,"v.p",eps);
    //UnitTests.checkDoubleRel(a.pd_DIL,"a.pd_DIL",v.pd,"v.pd",eps);
    UnitTests.checkDoubleRel(a.I_DIL,"a.I_DIL",v.I,"v.I",eps);
    //UnitTests.checkDoubleRel(a.DI_DIL,"a.DI_DIL",v.DI,"v.DI",eps);
    UnitTests.checkDoubleRel(a.avar_p_DIL,"a.avar_p_DIL",v.avar_p,"v.avar_p",eps);
    UnitTests.checkDoubleRel(a.avar_p_DIL,"a.avar_p_DIL",2*v.a.x_a_b,"2*v.a.x_a_b",eps);
    }
    {
    final DynVar1 v=new DynVar1(ID.psiH,QQDi,QQt,QQDpi,VOLATILITY_EDPsi);
    UnitTests.checkDoubleRel(a.p_DIH,"a.p_DIH",v.p,"v.p",eps);
    UnitTests.checkDoubleRel(a.pd_DIH,"a.pd_DIH",v.pd,"v.pd",eps);
    UnitTests.checkDoubleRel(a.I_DIH,"a.I_DIH",v.I,"v.I",eps);
    UnitTests.checkDoubleRel(a.DI_DIH,"a.DI_DIH",v.DI,"v.DI",eps);
    UnitTests.checkDoubleRel(a.avar_p_DIH,"a.avar_p_DIH",v.avar_p,"v.avar_p",eps);
    UnitTests.checkDoubleRel(a.avar_p_DIH,"a.avar_p_DIH",2*v.a.x_a_b,"2*v.a.x_a_b",eps);
    if(ID.sH < ID_sH_bound) throw new RuntimeException("ID.sH="+ID.sH+" ID_sH_bound="+ID_sH_bound+" (low bound)");
    }
    System.err.println("DynSD2PnL:testVar1 test OK neq="+a.ID.neq+" eps="+eps);
}

}
