package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Use <psi|V|psi> along with <psi|I|psi>.
 *  @version $Id: DynSIV.java,v 1.7 2020/09/27 20:33:11 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynSIV {
    final EVXData I;
    final double I1,p1,I2,p2,tau;
    DynSIV(final EVXData I,
	   final double I1,final double p1,
	   final double I2,final double p2,
	   final double tau)
    {
	this.I=I;
	this.I1=I1;
	this.p1=p1;
	this.I2=I2;
	this.p2=p2;
	this.tau=tau;
    }

static DynSIV get_DynSIV(final int n,final double [] imoms,final double [] tmoms,final double [] pimoms,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");
    final double [] QQt=M.B.getQQMatr(n,tmoms),QQi=M.B.getQQMatr(n,imoms);
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] QQpi=M.B.getQQMatr(n,pimoms),RQQi=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(imoms));
    final int i1=n-1,i2=i1-1;
    final double I1=I.ev[i1],I2=I.ev[i2];
    final double p1=I.getFPsi(i1,QQpi)/I1,p2=I.getFPsi(i2,QQpi)/I2,
	tau=I.getFPsi(i1,RQQi)/I1-I.getFPsi(i2,RQQi)/I2;
    return new DynSIV(I,I1,p1,I2,p2,tau);
}


static void matchDynSIV(final DynSIV a,final DynSIV b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,false,eps);
    UnitTests.checkDoubleRel(a.I1,"a.I1",b.I1,"b.I1",eps);
    UnitTests.checkDoubleRel(a.p1,"a.p1",b.p1,"b.p1",eps);
    UnitTests.checkDoubleRel(a.I2,"a.I2",b.I2,"b.I2",eps);
    UnitTests.checkDoubleRel(a.p2,"a.p2",b.p2,"b.p2",eps);
    UnitTests.checkDoubleRel(a.tau,"a.tau",b.tau,"b.tau",eps);

    System.err.println("DynSIV:matchDynSIV test OK eps="+eps);
}

}
