package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

class DynSPnL {
    final double I_0,I_L,I_H,p_IH,pnl_IH,I_pnlL,p_pnlL,w_IH_pnlL,I_pnlH,p_pnlH,w_IH_pnlH;
    DynSPnL(final double I_0,final double I_L,final double I_H,final double p_IH,final double pnl_IH,final double I_pnlL,final double p_pnlL,final double w_IH_pnlL,final double I_pnlH,final double p_pnlH,final double w_IH_pnlH)
    {
	this.I_0=I_0;this.I_L=I_L;this.I_H=I_H;this.p_IH=p_IH;this.pnl_IH=pnl_IH;
	this.I_pnlL=I_pnlL;this.p_pnlL=p_pnlL;this.w_IH_pnlL=w_IH_pnlL;
	this.I_pnlH=I_pnlH;this.p_pnlH=p_pnlH;this.w_IH_pnlH=w_IH_pnlH;
    }

    static DynSPnL get_DynSPnL(final int n,
			       final double [] QQi,final double [] QQt, final double [] QQpi,
			       final boolean flag_p_IH_as_offset,
			       final VolMatrix VOLATILITY_EDPsi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final int i0=0,iL=1,iH=n-1;
	final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
	final double I_0=basis[i0],I_L=basis[iL],I_H=basis[iH],
	    p_IH=TMatr.getAMatrA(n,basis,n+iH*n,QQpi,n)/basis[iH];

	final int nd=n-1;
	final double [] sQQpnl=TMatr.sumAcB(QQpi,-(flag_p_IH_as_offset?p_IH:0),QQi),
	    QQpnl=VOLATILITY_EDPsi.getDiffMatrix(n,sQQpnl);

	if(!(i0==0)) throw new RuntimeException("Not supported i0="+i0);
	final double [] RQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpnl,basis,nd,n+n+0);
	final double [] evE=M.EV.getEV(nd,RQQpnl,true),tmp=new double [nd*n];
	DeprecatedHelperTMatr.multiply_MA_x_MB(tmp,nd,evE,nd,nd,nd,basis,n,n,n+n+0); // res: nd x n
	final double [] psipnlL=EVXData.getPsiFromBasis(0,tmp,n,0),
	    psipnlH=EVXData.getPsiFromBasis(nd-1,tmp,n,0),
	    psiH=EVXData.getPsiFromBasis(iH,basis,n);
	final double
	    I_pnlL=TMatr.getAMatrA(psipnlL,QQi,n),
	    p_pnlL=TMatr.getAMatrA(psipnlL,QQpi,n)/I_pnlL,
	    w_IH_pnlL=TMatr.getAMatrB(psipnlL,psiH,QQt,n),
	    I_pnlH=TMatr.getAMatrA(psipnlH,QQi,n),
	    p_pnlH=TMatr.getAMatrA(psipnlH,QQpi,n)/I_pnlH,
	    w_IH_pnlH=TMatr.getAMatrB(psipnlH,psiH,QQt,n),
	    pnl_IH=TMatr.getAMatrA(psiH,QQpnl,n);
	if(false)
	{
	    System.err.println("DynSPnL:get_DynSPnL:evE[0]="+evE[0]+" 2<psipnlL|sQQpnl|DpsipnlL>="+2*TMatr.getAMatrB(psipnlL,VOLATILITY_EDPsi.getDPsi(psipnlL),sQQpnl,n)+" <psipnlL|QQpnl|psipnlL>="+TMatr.getAMatrA(psipnlL,QQpnl,n)+" psipnlL(x0)="+M.B.getD0(psipnlL,M.getRadauRoot()));
	    System.err.println("DynSPnL:get_DynSPnL:evE[nd-1]="+evE[nd-1]+" 2<psipnlH|sQQpnl|DpsipnlH>="+2*TMatr.getAMatrB(psipnlH,VOLATILITY_EDPsi.getDPsi(psipnlH),sQQpnl,n)+" <psipnlH|QQpnl|psipnlH>="+TMatr.getAMatrA(psipnlH,QQpnl,n)+" psipnlH(x0)="+M.B.getD0(psipnlH,M.getRadauRoot()));
	}
	return new DynSPnL(I_0,I_L,I_H,p_IH,pnl_IH,I_pnlL,p_pnlL,w_IH_pnlL,I_pnlH,p_pnlH,w_IH_pnlH);
    }

    static DynSPnL get_fromDP_DynSPnL(final int n,
				      final double [] QQi,final double [] QQt,final double [] QQpi, final double [] QQdp,
				      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final int i0=0,iL=1,iH=n-1;
	final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
	final double I_0=basis[i0],I_L=basis[iL],I_H=basis[iH],
	    p_IH=TMatr.getAMatrA(n,basis,n+iH*n,QQpi,n)/basis[iH];


	final double [] QQpnl=TMatr.mulMatrices(n,new double [][] {QQdp,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),QQi});
	TMatr.scaleVectorInPlace(TMatr.selectHermitianPartInPlace(n,QQpnl),2);
	final double [] evE=EdComm.getIRadauBasisPsi0FromQQ(n,QQpnl,QQt,M);
	final double [] psipnlL=EVXData.getPsiFromBasis(iL,evE,n),
	    psipnlH=EVXData.getPsiFromBasis(iH,evE,n),
	    psiH=EVXData.getPsiFromBasis(iH,basis,n);

	final double
	    I_pnlL=TMatr.getAMatrA(psipnlL,QQi,n),
	    p_pnlL=TMatr.getAMatrA(psipnlL,QQpi,n)/I_pnlL,
	    w_IH_pnlL=TMatr.getAMatrB(psipnlL,psiH,QQt,n),
	    I_pnlH=TMatr.getAMatrA(psipnlH,QQi,n),
	    p_pnlH=TMatr.getAMatrA(psipnlH,QQpi,n)/I_pnlH,
	    w_IH_pnlH=TMatr.getAMatrB(psipnlH,psiH,QQt,n),
	    pnl_IH=TMatr.getAMatrA(psiH,QQpnl,n);

	return new DynSPnL(I_0,I_L,I_H,p_IH,pnl_IH,I_pnlL,p_pnlL,w_IH_pnlL,I_pnlH,p_pnlH,w_IH_pnlH);
    }

}

