package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.VolMatrix;

/** Skewness in spectrum space. See also our early approach with QkQVM and EAssym.
 *  @version $Id: DynSki.java,v 1.14 2022/08/07 15:44:02 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynSki {
    final double [] m0,m1,m2,m3;
    final double p_IH,I_IH;
    final Skewness sk,sk_IH;
    DynSki(final double [] m0,final double [] m1,final double [] m2,final double [] m3)
    {
	this.m0=m0;
	this.m1=m1;
	this.m2=m2;
	this.m3=m3;
	final int n=m0.length,ixH=n-1;
	I_IH=m0[ixH];
	p_IH=m1[ixH]/I_IH;
	this.sk=Skewness.getSkewnessFromMonomialMoments(TMatr.sumElements(m0),TMatr.sumElements(m1),TMatr.sumElements(m2),TMatr.sumElements(m3));
	this.sk_IH=Skewness.getSkewnessFromMonomialMoments(m0[ixH],m1[ixH],m2[ixH],m3[ixH]);
    }

static DynSki getFromBasis_DynSki(final int nb,final double [] basis,final int n,final int basis_offset,final int [] ind_toremove,
				  final double [] QQi,final double [] QQpi,final double [] QQp2i,final double [] QQp3i,final double [] alt_m0)
{
    final int nd=nb-ind_toremove.length;int nx=0;
    final double [] m0=new double [nd],m1=new double [nd],m2=new double [nd],m3=new double [nd];
    for(int k=0;k<nb;k++)
    {
	if(com.polytechnik.utils.ReadWriteUtils.hasElement(ind_toremove,k)) continue;
	if(nx>=nd) throw new IllegalArgumentException("n="+n+" ind_toremove="+java.util.Arrays.toString(ind_toremove));
	m0[nx]=alt_m0!=null?alt_m0[k]:TMatr.getAMatrA(n,basis,basis_offset+n*k,QQi,n);
	m1[nx]=TMatr.getAMatrA(n,basis,basis_offset+n*k,QQpi,n);
	m2[nx]=TMatr.getAMatrA(n,basis,basis_offset+n*k,QQp2i,n);
	m3[nx]=TMatr.getAMatrA(n,basis,basis_offset+n*k,QQp3i,n);
	if(++nx>nd) throw new IllegalArgumentException("n="+n+" ind_toremove="+java.util.Arrays.toString(ind_toremove));
    }
    if(nx!=nd) throw new RuntimeException("getBasisWithVectorsRemoved:n="+n+" nx="+nx+" nd="+nd+" ind_toremove="+java.util.Arrays.toString(ind_toremove));
    return new DynSki(m0,m1,m2,m3);
}

static DynSki get0_DynSki(final int n,final double [] QQi,final double [] QQt,
			  final double [] QQpi,final double [] QQp2i,final double [] QQp3i,
			  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final int i0=0,nd=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    return getFromBasis_DynSki(n,basis,n,n,new int []{i0},QQi,QQpi,QQp2i,QQp3i,basis);
    //return getFromBasis_DynSki(nd,basis,n,n+n,new int []{},QQi,QQpi,QQp2i,QQp3i,DeprecatedHelperTMatr.remove0Element(basis));
}

static DynSki getR_DynSki(final int n,final double [] QQi,final double [] QQt,
			  final double [] QQpi,final double [] QQp2i,final double [] QQp3i,
			  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final int nd=n-1;
    final double [] RQQt=VolMatrix.getRadauMatrix(n,QQt,M),
	RQQi=VolMatrix.getRadauMatrix(n,QQi,M),
	RQQpi=VolMatrix.getRadauMatrix(n,QQpi,M),
	RQQp2i=VolMatrix.getRadauMatrix(n,QQp2i,M),
	RQQp3i=VolMatrix.getRadauMatrix(n,QQp3i,M),
	basis=M.EV.getGEV_Generalized(nd,RQQi,RQQt,true);
    return getFromBasis_DynSki(nd,basis,nd,nd,new int []{},RQQi,RQQpi,RQQp2i,RQQp3i,basis);
}


static DynSki get_DynSki(final int n,final double [] QQi,final double [] QQt,
			 final double [] QQpi,final double [] QQp2i,final double [] QQp3i,
			 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] basis=M.EV.getGEV_Generalized(n,QQi,QQt,true);
    return getFromBasis_DynSki(n,basis,n,n,new int []{},QQi,QQpi,QQp2i,QQp3i,basis);
}

static Skewness get_fromTR_DynSki(final int n,final double [] QQi,final double [] QQt,
			 final double [] QQpi,final double [] QQp2i,final double [] QQp3i)
{
    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
    final double m0=TMatr.getSpurMM(n,QQi,IQQt),
	m1=TMatr.getSpurMM(n,QQpi,IQQt),
	m2=TMatr.getSpurMM(n,QQp2i,IQQt),
	m3=TMatr.getSpurMM(n,QQp3i,IQQt);
    return Skewness.getSkewnessFromMonomialMoments(m0,m1,m2,m3);
}

static void matchDynSki(final DynSki a,final DynSki b,final double eps)
{
    UnitTests.checkDoubleArrayRel(a.m0,"a.m0",b.m0,"b.m0",eps);
    UnitTests.checkDoubleArrayRel(a.m1,"a.m1",b.m1,"b.m1",eps);
    UnitTests.checkDoubleArrayRel(a.m2,"a.m2",b.m2,"b.m2",eps);
    UnitTests.checkDoubleArrayRel(a.m3,"a.m3",b.m3,"b.m3",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.I_IH,"a.I_IH",b.I_IH,"b.I_IH",eps);
    Skewness.matchSkewness(a.sk,b.sk,eps);

    System.err.println("DynSki:matchDynSki test OK eps="+eps);
}

}
