package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

class DynTIPnL {
    final double p_IH,p_I0,var_p_IHI0;

    DynTIPnL(final double p_IH,final double p_I0, final double var_p_IHI0)
    {
	this.p_IH=p_IH;
	this.p_I0=p_I0;
	this.var_p_IHI0=var_p_IHI0;
    }


    static DynTIPnL get_DynTIPnL(final int n,
				 final double [] QQi,final double [] QQt,
				 final double [] QQpi,
				 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
	final int ix=n-1,i0=0;
	final double [] psi=EVXData.getPsiFromBasis(ix,basis,n),psi0=EdComm.getPsi0QQtOrthogonalBasisAtX0(n,QQi,n-1,basis,n+n,true,M);
	TMatr.scaleVectorInPlace(psi,1/Math.sqrt(basis[ix]));

	final double p_IH=TMatr.getAMatrA(psi,QQpi,n),p_I0=TMatr.getAMatrA(psi0,QQpi,n);
	final double [] one=M.getRadauMultiplied(new double []{1});
	final double sign=TMatr.getAMatrB(psi,one,QQi,n),
	    var_p_IHI0=Math.signum(sign)*TMatr.getAMatrB(psi,psi0,QQpi,n);
	return new DynTIPnL(p_IH,p_I0,var_p_IHI0);
    }

    static DynTIPnL get_v2_DynTIPnL(final int n,
				    final double [] QQi,final double [] QQt,
				    final double [] QQpi,
				    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQt,QQi,M);
	final int ix=1,i0=0;
	final double [] psi=EVXData.getPsiFromBasis(ix,basis,n),psi0=EVXData.getPsiFromBasis(i0,basis,n);
	final double p_IH=TMatr.getAMatrA(psi,QQpi,n),p_I0=TMatr.getAMatrA(psi0,QQpi,n);
	final double [] one=M.getRadauMultiplied(new double []{1});
	if(true)
	{
	    System.err.println("<psi|I|psi0>="+TMatr.getAMatrB(psi,psi0,QQi,n)+" =0");
	    final double apsi=TMatr.getAMatrB(psi,one,QQt,n)/TMatr.getAMatrB(psi,one,QQi,n);
	    System.err.println("ev[x]="+basis[ix]+" should be equal to apsi="+apsi);
	}
	final double sign=TMatr.getAMatrB(psi,one,QQi,n),
	    var_p_IHI0=Math.signum(sign)*TMatr.getAMatrB(psi,psi0,QQpi,n);
	return new DynTIPnL(p_IH,p_I0,var_p_IHI0);
    }


    static DynTIPnL get0_DynTIPnL(final int n,
				  final int ind_toremove,
				  final double [] QQi,final double [] QQt, final double [] QQpi,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
    {
	final boolean print_diag=false;
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");
	final double [] basis=EdComm.getIRadauBasisPsi0IOrthogonalFromQQ(n,QQi,QQt,M);

	final double [] psi=EVXData.getPsiFromBasis(ind_toremove,basis,n),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	double [] bDpsi=new double [n],bPpsi=new double [n];
	for(int k=0;k<n;k++)
	{
	    if(k==ind_toremove) continue;
	    bDpsi[k]=TMatr.getAMatrB(n,basis,n+k*n,Dpsi,0,QQt,n);
	    bPpsi[k]=TMatr.getAMatrB(n,basis,n+k*n,psi,0,QQpi,n);
	    if(print_diag)
	    {
		System.err.println("<psi"+ind_toremove+"|QQi|psi"+k+">="+TMatr.getAMatrB(n,basis,n+k*n,psi,0,QQi,n)+" must be =0");
		System.err.println("bDpsi[k]="+bDpsi[k]+"\t<->="+TMatr.getAMatrB(VOLATILITY_EDPsi.getDPsi(EVXData.getPsiFromBasis(k,basis,n)),psi,QQt,n)+" must change sign");
	    }
	}
	final int i0=0;
	final double p_IH=TMatr.getAMatrA(n,basis,n+ind_toremove*n,QQpi,n)/basis[ind_toremove],
	    p_I0=TMatr.getAMatrA(n,basis,n+i0*n,QQpi,n)/basis[i0],
	    var_p_IH=TMatr.mulVV(bPpsi,bDpsi);
	if(print_diag)
	{
	    System.err.println("<psi0|QQt|psi0>="+TMatr.getAMatrA(n,basis,n+i0*n,QQt,n));
	}
	return new DynTIPnL(p_IH,p_I0,var_p_IH);
    }

    static DynTIPnL getH0_DynTIPnL(final int n,
				   final double [] QQi,final double [] QQt, final double [] QQpi,
				   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
    {
	final boolean print_diag=false;
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");
	final double [] basis=EdComm.getIRadauBasisPsi0IOrthogonalFromQQ(n,QQi,QQt,M);
	final int i0=0,iH=n-1;

	final double [] psi0=EVXData.getPsiFromBasis(i0,basis,n),Dpsi0=VOLATILITY_EDPsi.getDPsi(psi0),
	    psiH=EVXData.getPsiFromBasis(iH,basis,n);
	final double p_IH=TMatr.getAMatrA(psiH,QQpi,n)/basis[iH],
	    p_I0=TMatr.getAMatrA(psi0,QQpi,n)/basis[i0],
	    var_p_IH=TMatr.getAMatrB(psi0,psiH,QQpi,n)*TMatr.getAMatrB(psiH,Dpsi0,QQt,n);
	return new DynTIPnL(p_IH,p_I0,var_p_IH);
    }

    static DynTIPnL getD_DynTIPnL(final int n,
				  final int ind_toremove,
				  final double [] QQi,final double [] QQt, final double [] QQpi,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
    {
	final boolean print_diag=false;
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");
	final double [] basis=EdComm.getIRadauBasisPsi0IOrthogonalFromQQ(n,QQi,QQt,M);

	final double [] psi=EVXData.getPsiFromBasis(ind_toremove,basis,n),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	TMatr.sumP1(Dpsi,psi,-TMatr.getAMatrB(Dpsi,psi,QQi,n)/basis[ind_toremove]);

	final int i0=0;
	final double p_IH=TMatr.getAMatrA(n,basis,n+ind_toremove*n,QQpi,n)/basis[ind_toremove],
	    p_I0=TMatr.getAMatrA(n,basis,n+i0*n,QQpi,n)/basis[i0],
	    var_p_IH=TMatr.getAMatrB(psi,Dpsi,QQpi,n);
	if(print_diag)
	{
	    System.err.println("<Dpsi|QQi|psi>="+TMatr.getAMatrB(Dpsi,psi,QQi,n));
	    System.err.println("<psi0|QQt|psi0>="+TMatr.getAMatrA(n,basis,n+i0*n,QQt,n));
	}
	return new DynTIPnL(p_IH,p_I0,var_p_IH);
    }


    public static void matchDynTIPnL(final DynTIPnL a,final DynTIPnL b,final double eps)
{
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.p_I0,"a.p_I0",b.p_I0,"b.p_I0",eps);
    UnitTests.checkDoubleRel(a.var_p_IHI0,"a.var_p_IHI0",b.var_p_IHI0,"b.var_p_IHI0",eps);

    System.err.println("DynTIPnL:matchDynTIPnL test OK  eps="+eps);
}

}
