package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/** Consider V eigenfuncions.
 * After Dirk Laurie <dirk@aims.ac.za> notes about
 * differential equation on w(x) on Nov, 14, 2015.
 *  @version $Id: DynV.java,v 1.9 2020/09/27 20:33:11 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynV {
    final double diraver,pa_I,pa_V,p_I,p_V,I,V,p2x0;
    final double [] evV;
    final int ind;
DynV(final double diraver,
     final double pa_I,
     final double pa_V,
     final double p_I,
     final double p_V,
     final double I,
     final double V,
     final double p2x0,
     final double [] evV,final int ind)
{
    this.diraver=diraver;
    this.pa_I=pa_I;
    this.pa_V=pa_V;
    this.p_I=p_I;
    this.p_V=p_V;
    this.I=I;
    this.V=V;
    this.p2x0=p2x0;
    this.evV=evV;
    this.ind=ind;
}


static DynV get_DynV(final int n,
		     final double [] QQV,final double [] QQt,
		     final double [] QQpV,
		     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		     final VolMatrix VOLATILITY_EDPsi,
		     final double eps)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final double [] evV=M.EV.getGEV_Generalized(n,QQV,QQt,true);

    int ind=-1;
    double I=-1,pI=Double.NaN,pV=Double.NaN,p2x0=Double.NaN,diraver=0,si=0,spi=0,spv=0;
    for(int k=0;k<n;k++)
    {
	final double [] psi=EVXData.getPsiFromBasis(k,evV,n),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	final double i=2*TMatr.getAMatrB(psi,Dpsi,QQV,n),psi2x0=2*TMatr.getAMatrB(psi,Dpsi,QQt,n),
	    pi=2*TMatr.getAMatrB(psi,Dpsi,QQpV,n),pv=TMatr.getAMatrA(psi,QQpV,n);
	diraver+=pi-pv*psi2x0;
	si+=i;
	spi+=pi;
	spv+=pv*psi2x0;
	if(i>=I){
	    ind=k;
	    I=i;
	    p2x0=psi2x0;
	    pI=pi;
	    pV=pv;
	}
	if(eps>0)
	{
	    final double [] QQi=VOLATILITY_EDPsi.getDiffMatrix(n,QQV);
	    final double x0=M.getRadauRoot();
	    System.err.println("DynV:k="+k+" lam="+evV[k]+" i="+i+" psi2x0="+psi2x0);
	    UnitTests.checkDoubleRel(i,"i",psi2x0*evV[k],"psi2x0*evV[k]",eps);
	    UnitTests.checkDoubleRel(i,"i",TMatr.getAMatrA(psi,QQi,n),"<psi|QQi|psi>",eps);
	    UnitTests.checkDoubleRel(Math.pow(M.B.getD0(psi,x0),2),"psi(x0)^2",psi2x0,"psi2x0",eps);
	}
    }
    final double p_I=pI/I,p_V=pV*p2x0/I;
    return new DynV(diraver,spi/si,spv/si,p_I,p_V,I,ind>=0?evV[ind]:Double.NaN,p2x0,evV,ind);
}

static void matchV(final int n,
		   final double [] QQV,final double [] QQt,
		   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		   final VolMatrix VOLATILITY_EDPsi,
		   final double eps)
{
    final DynV q=get_DynV(n,QQV,QQt,QQV,M,VOLATILITY_EDPsi,eps);
    UnitTests.checkDoubleRel(q.diraver,"q.diraver",0,"0",eps);
    UnitTests.checkDoubleRel(q.p_I,"q.p_I",q.p_V,"q.p_V",eps);
    UnitTests.checkDoubleRel(q.pa_I,"q.pa_I",q.pa_V,"q.pa_V",eps);
    System.err.println("DynV:matchV test OK  eps="+eps);
}

}
