package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

class DynVHPnL {
    /** See UkQVM.var_p_IH and unit test there UkQVM.test_getvPsifPsi()
     * where different formulas give exacty the same answer, like dI perturbation with I_0 assumed to be equal I_H,
     * which cancels the denominator.
     */
    final double p_IH,var_p_IH,sw_p_IH,pb_IH,ib_IH,pd_IH,id_IH;
    final double [] wk;
    final double dir_a,dp_a,dp_IH;
    final EVXData I;

    DynVHPnL(final EVXData I,
	     final double [] wk,
	     final double p_IH,final double var_p_IH,final double sw_p_IH,
	     final double pb_IH,final double ib_IH,final double pd_IH,final double id_IH)
    {
	this.I=I;
	this.wk=wk;
	this.p_IH=p_IH;
	this.var_p_IH=var_p_IH;
	this.sw_p_IH=sw_p_IH;
	this.pb_IH=pb_IH;
	this.ib_IH=ib_IH;
	this.pd_IH=pd_IH;
	this.id_IH=id_IH;
	double sdp_a=0,s_ia=0,sadp_a=0;
	for(int k=0;k<wk.length;k++)
	{
	    sdp_a+=wk[k];
	    sadp_a+=Math.abs(wk[k]);
	    s_ia+=I.ev[k];
	}
	dir_a=sdp_a/sadp_a;
	dp_a=sdp_a/s_ia;
	dp_IH=wk[I.neq-1]/I.ev[I.neq-1];
    }

/** Calculate <psiH|QQpi|psik><psik|QQmd|psiH>.
 */
static DynVHPnL getme_DynVHPnL(final int n,
			       final double [] QQi,final double [] QQt,
			       final double [] QQpi,final double [] QQmd,
			       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");

    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	bQQmd=EVXData.convertMatrFromQtoB_Basis(n,QQmd,I.ev);

    final int ind_toremove=n-1;
    final double [] bPpsi=new double [n],bDpsi=new double [n];
    for(int k=0;k<n;k++) {
	bPpsi[k]=bQQpi[n*ind_toremove+k];
	bDpsi[k]=bQQmd[n*k+ind_toremove];
    }
    bDpsi[ind_toremove]=bPpsi[ind_toremove]=0;

    final double p_IH=bQQpi[ind_toremove*n+ind_toremove]/I.ev[ind_toremove],
	var_p_IH=TMatr.mulVVn1(bPpsi,bDpsi),
	n2bPpsi=TMatr.mulVV(bPpsi,bPpsi),n2bDpsi=TMatr.mulVV(bDpsi,bDpsi),
	sw_p_IH=var_p_IH/Math.sqrt(n2bPpsi*n2bDpsi),
	pb=TMatr.getAMatrA(bPpsi,bQQpi,n),pd=TMatr.getAMatrA(bDpsi,bQQpi,n);
    double ib=0,id=0;
    for(int k=0;k<n;k++){ib+=I.ev[k]*Math.pow(bPpsi[k],2);id+=I.ev[k]*Math.pow(bDpsi[k],2);}
    final double [] wk=new double [n];
    for(int k=0;k<n;k++)
    {
	double s=0;
	for(int l=0;l<n;l++)
	{
	    if(l==k) continue;
	    s+=bQQpi[n*k+l]*bQQmd[n*l+k];
	}
	wk[k]=s;
    }

    return new DynVHPnL(I,wk,p_IH,var_p_IH,sw_p_IH,pb/ib,ib/n2bPpsi,pd/id,id/n2bDpsi);
}

/* Same as getme_DynVHPnL. */
static DynVHPnL getme_v2_DynVHPnL(final int n,
				  final double [] QQi,final double [] QQt,
				  final double [] QQpi,final double [] QQmd,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int ind_toremove=n-1;
    final double p_IH=I.getFPsiPsi(ind_toremove,ind_toremove,QQpi)/I.ev[ind_toremove];
    double s_var_p_IH=0,s_np=0,s_ndt=0,s_pb=0,s_ib=0,s_pd=0,s_id=0;
    for(int k=0;k<n;k++)
    {
	if(k==ind_toremove) continue;
	final double kep=I.getFPsiPsi(k,ind_toremove,QQpi),kedt=I.getFPsiPsi(k,ind_toremove,QQmd);
	s_np+=kep*kep;
	s_ndt+=kedt*kedt;
	s_var_p_IH+=kep*kedt;
	for(int l=0;l<n;l++)
	{
	    if(l==ind_toremove) continue;
	    final double pp=I.getFPsiPsi(k,l,QQpi),lep=I.getFPsiPsi(l,ind_toremove,QQpi),ledt=I.getFPsiPsi(l,ind_toremove,QQmd);
	    s_pb+=kep*lep*pp;
	    s_pd+=kedt*ledt*pp;
	}
	s_ib+=Math.pow(kep,2)*I.ev[k];
	s_id+=Math.pow(kedt,2)*I.ev[k];
    }
    final double [] wk=new double [n];
    for(int k=0;k<n;k++)
    {
	double s=0;
	for(int l=0;l<n;l++)
	{
	    if(l==k) continue;
	    s+=I.getFPsiPsi(l,k,QQpi)*I.getFPsiPsi(l,k,QQmd);
	}
	wk[k]=s;
    }

    return new DynVHPnL(I,wk,p_IH,s_var_p_IH,s_var_p_IH/Math.sqrt(s_np*s_ndt),s_pb/s_ib,s_ib/s_np,s_pd/s_id,s_id/s_ndt);
}


static DynVHPnL get_DynVHPnL(final int n,
			     final double [] QQi,final double [] QQt, final double [] QQpi,
			     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev);

	final int ind_toremove=n-1;
	final double [] _psi=I.getPsi(ind_toremove),bDpsi=EVXData.convertPsiFromQtoB_Basis(n,VOLATILITY_EDPsi.getDPsi(_psi),I.ev,QQt);
	bDpsi[ind_toremove]=0;
	final double [] bPpsi=_psi;
	System.arraycopy(bQQpi,n*ind_toremove,bPpsi,0,n);
	bPpsi[ind_toremove]=0;
	final double p_IH=bQQpi[ind_toremove*n+ind_toremove]/I.ev[ind_toremove],
	    var_p_IH=TMatr.mulVVn1(bPpsi,bDpsi),
	    n2bPpsi=TMatr.mulVV(bPpsi,bPpsi),n2bDpsi=TMatr.mulVV(bDpsi,bDpsi),
	    sw_p_IH=var_p_IH/Math.sqrt(n2bPpsi*n2bDpsi),
	    pb=TMatr.getAMatrA(bPpsi,bQQpi,n),pd=TMatr.getAMatrA(bDpsi,bQQpi,n);
	double ib=0,id=0;
	for(int k=0;k<n;k++){ib+=I.ev[k]*Math.pow(bPpsi[k],2);id+=I.ev[k]*Math.pow(bDpsi[k],2);}
	if(false)
	{
	    final double [] evqmlnr=M.EV.getGEV_Generalized(2,
		new double []{2*n2bPpsi*var_p_IH,n2bPpsi*n2bDpsi+var_p_IH*var_p_IH,
			      n2bPpsi*n2bDpsi+var_p_IH*var_p_IH,2*var_p_IH*n2bDpsi},
		new double []{n2bPpsi,var_p_IH,var_p_IH,n2bDpsi},true);
	    //TMatr.scaleVectorInPlace(bPpsi,1/Math.sqrt(n2bPpsi));
	    //TMatr.scaleVectorInPlace(bDpsi,1/Math.sqrt(n2bDpsi));
	    final int nd=n-1;
	    final double [] m=new double [nd*nd];
	    for(int k=0,o=0;k<n;k++)
	    {
		if(k==ind_toremove) continue;
		for(int l=0;l<n;l++)
		{
		    if(l==ind_toremove) continue;
		    m[o++]=bDpsi[k]*bPpsi[l]+bDpsi[l]*bPpsi[k];
		}
	    }
	    final double [] evM=M.EV.getEV(nd,m,true);
	    // there should be just two non-zero eigenvalues
	    System.err.println("evM="+java.util.Arrays.toString(java.util.Arrays.copyOf(evM,nd)));
	    System.err.println("evqmlnr="+java.util.Arrays.toString(java.util.Arrays.copyOf(evqmlnr,2)));
	    final double [] evmlnr=M.EV.getGEV_Generalized(2,new double []{2*sw_p_IH,1+sw_p_IH*sw_p_IH,1+sw_p_IH*sw_p_IH,2*sw_p_IH},new double []{1,sw_p_IH,sw_p_IH,1},true);

	    System.err.println("evmlnr="+java.util.Arrays.toString(java.util.Arrays.copyOf(evmlnr,2)));
	}
	final double [] wk=new double [n];
	if(false)
	{
	    for(int k=0;k<n;k++)
	    {
		final double [] bD=EVXData.convertPsiFromQtoB_Basis(n,VOLATILITY_EDPsi.getDPsi(I.getPsi(k)),I.ev,QQt);
		double s=0;
		for(int l=0;l<n;l++)
		{
		    if(l==k) continue;
		    s+=bQQpi[k*n+l]*bD[l];
		}
		wk[k]=s;
	    }
	}
	else if(false)
	{

	    final double [] bDQQpi=EVXData.convertMatrFromQtoB_Basis(n,VOLATILITY_EDPsi.getDiffMatrix(n,QQpi),I.ev),
		bDQQt=EVXData.convertMatrFromQtoB_Basis(n,VOLATILITY_EDPsi.getDiffMatrix(n,QQt),I.ev);
	    for(int k=0;k<n;k++)
	    {
		wk[k]=0.5*(bDQQpi[k*n+k]-bQQpi[k*n+k]*bDQQt[k*n+k]);
	    }
	}
	else
	{
	    final double [] bD=DeprecatedHelperEVXData.convertOperatorFromQtoB_Basis(n,VOLATILITY_EDPsi.getDMatrix(n),QQt,I.ev,n+0);
	    for(int k=0;k<n;k++)
	    {
		double s=0;
		for(int l=0;l<n;l++)
		{
		    if(l==k) continue;
		    s+=bQQpi[k*n+l]*bD[l*n+k];
		}
		wk[k]=s;
	    }
	}

	return new DynVHPnL(I,wk,p_IH,var_p_IH,sw_p_IH,pb/ib,ib/n2bPpsi,pd/id,id/n2bDpsi);
    }

/* Same as get_DynVHPnL. */
static DynVHPnL get_v2_DynVHPnL(final int n,
				final double [] QQi,final double [] QQt, final double [] QQpi,
				final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");
	final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	final int ind_toremove=n-1;
	final double [] Dpsi=VOLATILITY_EDPsi.getDPsi(I.getPsi(ind_toremove));
	final double p_IH=I.getFPsiPsi(ind_toremove,ind_toremove,QQpi)/I.ev[ind_toremove];
	double s_var_p_IH=0,s_np=0,s_ndt=0,s_pb=0,s_ib=0,s_pd=0,s_id=0;
	final int nd=n-1;
	final double [] m=new double [nd*nd];
	for(int k=0,o=0;k<n;k++)
	{
	    if(k==ind_toremove) continue;
	    final double kep=I.getFPsiPsi(k,ind_toremove,QQpi),kedt=I.getFPsiPsi(k,Dpsi,QQt);
	    s_np+=kep*kep;
	    s_ndt+=kedt*kedt;
	    s_var_p_IH+=kep*kedt;
	    for(int l=0;l<n;l++)
	    {
		if(l==ind_toremove) continue;
		final double pp=I.getFPsiPsi(k,l,QQpi),lep=I.getFPsiPsi(l,ind_toremove,QQpi),ledt=I.getFPsiPsi(l,Dpsi,QQt);
		s_pb+=kep*lep*pp;
		s_pd+=kedt*ledt*pp;
		m[o++]=kep*ledt+lep*kedt;
	    }
	    s_ib+=Math.pow(kep,2)*I.ev[k];
	    s_id+=Math.pow(kedt,2)*I.ev[k];
	}

	if(false)
	{
	final double [] evM=M.EV.getEV(nd,m,true);
	// there should be just two non-zero eigenvalues
	System.err.println("get_v2_DynVHPnL:evM="+java.util.Arrays.toString(java.util.Arrays.copyOf(evM,nd)));
	}
	final double [] wk=new double [n];
	for(int k=0;k<n;k++)
	{
	    final double [] dpsi=VOLATILITY_EDPsi.getDPsi(I.getPsi(k));
	    wk[k]=I.getFPsiPsi(k,dpsi,QQpi)-I.getFPsi(k,QQpi)*I.getFPsiPsi(k,dpsi,QQt);
	}

	return new DynVHPnL(I,wk,p_IH,s_var_p_IH,s_var_p_IH/Math.sqrt(s_np*s_ndt),s_pb/s_ib,s_ib/s_np,s_pd/s_id,s_id/s_ndt);
    }

/* Same as get_DynVHPnL. */
static DynVHPnL get_v3_DynVHPnL(final int n,
				final double [] QQi,final double [] QQt, final double [] QQpi,
				final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");
	final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	final int ind_toremove=n-1;
	final double [] psi=I.getPsi(ind_toremove),psibeta=TMatr.getMatr_x_A(n,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),TMatr.getMatr_x_A(n,QQpi,psi));
	TMatr.sumP1(psibeta,psi,-TMatr.getAMatrB(psibeta,psi,QQt,n));
	final double [] Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	TMatr.sumP1(Dpsi,psi,-TMatr.getAMatrB(Dpsi,psi,QQt,n));

	final double p_IH=TMatr.getAMatrA(psi,QQpi,n)/I.ev[ind_toremove],
	    var_p_IH=TMatr.getAMatrB(psibeta,Dpsi,QQt,n),
	    s_np=TMatr.getAMatrA(psibeta,QQt,n),
	    s_ndt=TMatr.getAMatrA(Dpsi,QQt,n),
	    s_pb=TMatr.getAMatrA(psibeta,QQpi,n),
	    s_ib=TMatr.getAMatrA(psibeta,QQi,n),
	    s_pd=TMatr.getAMatrA(Dpsi,QQpi,n),
	    s_id=TMatr.getAMatrA(Dpsi,QQi,n);
	final double [] wk=new double [n];
	for(int k=0;k<n;k++)
	{
	    final double [] _psi=I.getPsi(k),_Dpsi=VOLATILITY_EDPsi.getDPsi(_psi);
	    TMatr.sumP1(_Dpsi,_psi,-TMatr.getAMatrB(_Dpsi,_psi,QQt,n));
	    wk[k]=TMatr.getAMatrB(_psi,_Dpsi,QQpi,n);
	}

	return new DynVHPnL(I,wk,p_IH,var_p_IH,var_p_IH/Math.sqrt(s_np*s_ndt),s_pb/s_ib,s_ib/s_np,s_pd/s_id,s_id/s_ndt);
    }


public static void matchDynVHPnL(final DynVHPnL a,final DynVHPnL b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,false,eps);
    UnitTests.checkDoubleArrayRel(a.wk,"a.wk",b.wk,"b.wk",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.var_p_IH,"a.var_p_IH",b.var_p_IH,"b.var_p_IH",eps);
    UnitTests.checkDoubleRel(a.var_p_IH,"a.var_p_IH",a.wk[a.I.neq-1],"a.wk[a.I.neq-1]",eps);
    UnitTests.checkDoubleRel(b.var_p_IH,"b.var_p_IH",b.wk[b.I.neq-1],"b.wk[b.I.neq-1]",eps);
    UnitTests.checkDoubleRel(a.sw_p_IH,"a.sw_p_IH",b.sw_p_IH,"b.sw_p_IH",eps);
    UnitTests.checkDoubleRel(a.pb_IH,"a.pb_IH",b.pb_IH,"b.pb_IH",eps);
    UnitTests.checkDoubleRel(a.ib_IH,"a.ib_IH",b.ib_IH,"b.ib_IH",eps);
    UnitTests.checkDoubleRel(a.pd_IH,"a.pd_IH",b.pd_IH,"b.pd_IH",eps);
    UnitTests.checkDoubleRel(a.id_IH,"a.id_IH",b.id_IH,"b.id_IH",eps);

    System.err.println("DynVHPnL:matchDynVHPnL test OK neq="+a.I.neq+" eps="+eps);
}
}
