package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;


/** Like DynVTPnL but with projection to |o>.
 *  @version $Id: DynVT0PnL.java,v 1.7 2020/09/27 20:33:11 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVT0PnL {
    final double [] basis;
    final double pnl_I,p_I,I_I,pnl_V,p_V,I_V,V,varPnL,varT;



     DynVT0PnL(final int n,
	     final double [] QQV,final double [] QQT,final double [] QQpV,
	     final VolMatrix VOLATILITY_EDPsi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	//this.basis=EdComm.getIRadauBasisPsi0IOrthogonalFromQQ(n,QQV,QQT,M);
	this.basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQV,QQT,M);
	final int ie=n-1;
	final double [] psi=EVXData.getPsiFromBasis(ie,basis,n),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);

	this.pnl_V=TMatr.getAMatrA(psi,QQpV,n);
	this.I_V=basis[ie];
	this.p_V=pnl_V/I_V;
	final double pix=2*TMatr.getAMatrB(psi,Dpsi,QQpV,n),ix=2*TMatr.getAMatrB(psi,Dpsi,QQV,n),tx=2*TMatr.getAMatrB(psi,Dpsi,QQT,n);
	this.V=I_V/tx;
	this.pnl_I=pix/tx;
	this.I_I=ix/tx;
	this.p_I=pix/ix;
	this.varPnL=pnl_I-pnl_V;
	this.varT=0;
	if(false)
	{
	    // does not match because D operator transofms psi to out of original basis psi(x0)=0
	    System.err.println("DynVT0PnL:I_I="+I_I+" I_V="+I_V);
	}
    }



}

