package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;


/** <V\psi^2>=\lambda<T\psi^2> is back.
 *  The idea is back from May 2013, (while talking to M. Kotnitsky),
 *  that at eigenvector the I is equal to V/T.
 *  @version $Id: DynVTPnL.java,v 1.15 2020/09/27 20:33:11 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVTPnL {
    final EVXData VT;
    final double pnl_I,p_I,I_I,pnl_V,p_V,I_V,V,varPnL,varT;

    DynVTPnL(final int n,
	     final double [] QQi,final double [] QQt,final double [] QQpi,
	     final double [] QQV,final double [] QQT,final double [] QQpV,
	     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final double eps_selftest)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	this.VT=EVXData.getEVXDataFromQQ(n,QQV,QQT,M);
	final int ie=n-1;
	this.pnl_V=VT.getFPsi(ie,QQpV);
	this.I_V=VT.ev[ie];
	this.p_V=pnl_V/I_V;
	final double pix=VT.getFPsi(ie,QQpi),ix=VT.getFPsi(ie,QQi),tx=VT.getFPsi(ie,QQt);
	this.V=I_V/tx;
	this.pnl_I=pix/tx;
	this.I_I=ix/tx;
	this.p_I=pix/ix;
	this.varPnL=pnl_I-pnl_V;
	this.varT=Math.pow(M.B.getD0(VT.getPsi(ie),VT.x0)/tx,2)-1;

	test_Self(QQi,QQt,QQpi,QQV,QQT,QQpV,M,ie,eps_selftest);
    }


    void test_Self(final double [] QQi,final double [] QQt,final double [] QQpi,
		   final double [] QQV,final double [] QQT,final double [] QQpV,
		   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final int ie,final double eps)
    {
	if(!(eps>=0)) return;
	final int n=VT.neq;
	for(int k=0;k<n;k++)
	 {
	     final double i=VT.getFPsi(k,QQi)/VT.getFPsi(k,QQt),
		 vt=VT.getFPsi(k,QQV)/VT.getFPsi(k,QQT);
	     System.err.println("DynVTPnL:<psi|I|psi>/<psi|psi>="+i+"\t<psi|V|psi>/<psi|T|psi>="+vt);
	     UnitTests.checkDoubleRel(i,"i",vt,"vt",eps);
	     UnitTests.checkDoubleRel(i,"i",VT.ev[k],"VT.ev[k]",eps);
	     UnitTests.checkDoubleRel(VT.getFPsi(k,QQT),"<psi|T|psi>",1,"1",eps);
	     UnitTests.checkDoubleRel(VT.getFPsi(k,QQV),"<psi|V|psi>",VT.ev[k],"VT.ev[k]",eps);
	     if(k==ie)
	     {
		 final double pnli=VT.getFPsi(k,QQpi)/VT.getFPsi(k,QQt),pnlvt=VT.getFPsi(k,QQpV);
		 UnitTests.checkDoubleRel(pnli,"pnli",pnl_I,"pnl_I",eps);
		 UnitTests.checkDoubleRel(pnlvt,"pnlvt",pnl_V,"pnl_V",eps);
		 final double p_i=VT.getFPsi(k,QQpi)/VT.getFPsi(k,QQi),p_vt=VT.getFPsi(k,QQpV)/VT.getFPsi(k,QQV),v=VT.getFPsi(k,QQV)/VT.getFPsi(k,QQt),
		     vart=Math.pow(M.B.getD0(VT.getPsi(k),VT.x0),2)*VT.getFPsi(k,QQT)/Math.pow(VT.getFPsi(k,QQt),2)-1;
		 UnitTests.checkDoubleRel(p_i,"p_i",p_I,"p_I",eps);
		 UnitTests.checkDoubleRel(p_vt,"p_vt",p_V,"p_V",eps);
		 UnitTests.checkDoubleRel(v,"v",V,"V",eps);
		 UnitTests.checkDoubleRel(vart,"vart",varT,"varT",eps);
	     }
	 }
	UnitTests.checkDoubleRel(I_I,"I_I",I_V,"I_V",eps);
	UnitTests.checkDoubleRel(varPnL,"varPnL",pnl_I-pnl_V,"pnl_I-pnl_V",eps);
	System.err.println("DynVTPnL:test_Self: test OK n="+n+" eps="+eps);
    }

     DynVTPnL(final int n,
	     final double [] QQV,final double [] QQT,final double [] QQpV,
	     final VolMatrix VOLATILITY_EDPsi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	this.VT=EVXData.getEVXDataFromQQ(n,QQV,QQT,M);
	final int ie=n-1;
	final double [] psi=VT.getPsi(ie),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);

	this.pnl_V=TMatr.getAMatrA(psi,QQpV,n);
	this.I_V=VT.ev[ie];
	this.p_V=pnl_V/I_V;
	final double pix=2*TMatr.getAMatrB(psi,Dpsi,QQpV,n),ix=2*TMatr.getAMatrB(psi,Dpsi,QQV,n),tx=2*TMatr.getAMatrB(psi,Dpsi,QQT,n);
	this.V=I_V/tx;
	this.pnl_I=pix/tx;
	this.I_I=ix/tx;
	this.p_I=pix/ix;
	this.varPnL=pnl_I-pnl_V;
	this.varT=Math.pow(M.B.getD0(psi,VT.x0)/tx,2)-1;
    }

static void matchDynVTPnL(final DynVTPnL a,final DynVTPnL b,final double eps)
{
    EVXData.matchEVXData(a.VT,b.VT,false,false,eps);
    UnitTests.checkDoubleRel(a.pnl_V,"a.pnl_V",b.pnl_V,"b.pnl_V",eps);
    UnitTests.checkDoubleRel(a.I_V,"a.I_V",b.I_V,"b.I_V",eps);
    UnitTests.checkDoubleRel(a.p_V,"a.p_V",b.p_V,"b.p_V",eps);
    UnitTests.checkDoubleRel(a.pnl_I,"a.pnl_I",b.pnl_I,"b.pnl_I",eps);
    UnitTests.checkDoubleRel(a.I_I,"a.I_I",b.I_I,"b.I_I",eps);
    UnitTests.checkDoubleRel(a.p_I,"a.p_I",b.p_I,"b.p_I",eps);
    UnitTests.checkDoubleRel(a.varPnL,"a.varPnL",b.varPnL,"b.varPnL",eps);
    UnitTests.checkDoubleRel(a.varT,"a.varT",b.varT,"b.varT",eps);
    UnitTests.checkDoubleRel(a.I_V,"a.I_V",a.I_I,"a.I_I",eps);
    UnitTests.checkDoubleRel(b.I_V,"b.I_V",b.I_I,"b.I_I",eps);

    System.err.println("DynVTPnL:matchDynVTPnL test OK n="+a.VT.neq+" eps="+eps);
}

}

