package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

class DynVar1PnL {
    final double I_0,I_L,I_H,p_IL,dif_IL,var1_pI_L,var2_pI_L,p_IH,dif_IH,var1_pI_H,var2_pI_H,pDiff;
    final double [] basis;

    DynVar1PnL(final int n,
	       final double [] QQi,final double [] QQr,final double [] QQpi,final double [] QQt,
	       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,final boolean flag_div_EE)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final int i0=0,iL=1,iH=n-1;
	this.basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQr,M);
	this.I_0=basis[i0];
	this.I_L=basis[iL];
	this.I_H=basis[iH];

	final double piH;
	{
	    final double [] psi=EVXData.getPsiFromBasis(iH,basis,n),Dpsi=VOLATILITY_EDPsi.getDPsi(psi),
		varpsi=UkQVM.getPsiPert0Skipped(n,basis,iH,QQpi,flag_div_EE);
	    this.p_IH=(piH=TMatr.getAMatrA(psi,QQpi,n))/basis[iH];
	    this.dif_IH=TMatr.getAMatrB(psi,Dpsi,QQpi,n)-p_IH*TMatr.getAMatrB(psi,Dpsi,QQi,n);
	    this.var1_pI_H=TMatr.getAMatrB(varpsi,Dpsi,QQt,n);
	    this.var2_pI_H=TMatr.getAMatrB(VOLATILITY_EDPsi.getDPsi(varpsi),Dpsi,QQt,n);
	}

	final double piL;
	{
	    final double [] psi=EVXData.getPsiFromBasis(iL,basis,n),Dpsi=VOLATILITY_EDPsi.getDPsi(psi),
		varpsi=UkQVM.getPsiPert0Skipped(n,basis,iL,QQpi,flag_div_EE);
	    this.p_IL=(piL=TMatr.getAMatrA(psi,QQpi,n))/basis[iL];
	    this.dif_IL=TMatr.getAMatrB(psi,Dpsi,QQpi,n)-p_IL*TMatr.getAMatrB(psi,Dpsi,QQi,n);
	    this.var1_pI_L=TMatr.getAMatrB(varpsi,Dpsi,QQt,n);
	    this.var2_pI_L=TMatr.getAMatrB(VOLATILITY_EDPsi.getDPsi(varpsi),Dpsi,QQt,n);
	}
	this.pDiff=(piH-piL)/(basis[iH]-basis[iL]);
    }

    static double [] getVars(final int n,final int ind_pert,
	       final double [] QQi,final double [] QQr,final double [] QQpi,final double [] QQt,
	       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,final boolean flag_div_EE)
    {
	final int i0=0,iL=1,iH=n-1;
	final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQr,M);
	final double [] psi=EVXData.getPsiFromBasis(ind_pert,basis,n),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	double s1=0,s2=0;
	for(int k=0+1;k<n;k++)
	{
	    if(k==ind_pert) continue;
	    final double [] psik=EVXData.getPsiFromBasis(k,basis,n);
	    final double f=TMatr.getAMatrB(psi,psik,QQpi,n)/(flag_div_EE?(basis[ind_pert]-basis[k]):1);
	    s1+=f*TMatr.getAMatrB(Dpsi,psik,QQt,n);
	    s2+=f*TMatr.getAMatrB(Dpsi,VOLATILITY_EDPsi.getDPsi(psik),QQt,n);
	}
	return new double []{TMatr.getAMatrA(psi,QQpi,n)/TMatr.getAMatrA(psi,QQi,n),s1,s2};
    }


static void test_DynVar1PnL(
    final int n,
    final double [] QQi,final double [] QQr,final double [] QQpi,final double [] QQt,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,final double eps)
{
    for(boolean flag_div_EE:new boolean []{false,true})
    {
	final DynVar1PnL q=new DynVar1PnL(n,QQi,QQr,QQpi,QQt,M,VOLATILITY_EDPsi,flag_div_EE);
	final int i0=0,iL=1,iH=n-1;
	final double [] L=getVars(n,iL,QQi,QQr,QQpi,QQt,M,VOLATILITY_EDPsi,flag_div_EE),
	    H=getVars(n,iH,QQi,QQr,QQpi,QQt,M,VOLATILITY_EDPsi,flag_div_EE);
	UnitTests.checkDoubleRel(q.p_IL,"q.p_IL",L[0],"L[0]",eps);
	UnitTests.checkDoubleRel(q.var1_pI_L,"q.var1_pI_L",L[1],"L[1]",eps);
	UnitTests.checkDouble(q.var2_pI_L,"q.var2_pI_L",L[2],"L[2]",eps);
	UnitTests.checkDoubleRel(q.p_IH,"q.p_IH",H[0],"H[0]",eps);
	UnitTests.checkDoubleRel(q.var1_pI_H,"q.var1_pI_H",H[1],"H[1]",eps);
	UnitTests.checkDouble(q.var2_pI_H,"q.var2_pI_H",H[2],"H[2]",eps);
    }

    System.err.println("DynVar1PnL:test_DynVar1PnL(): test OK n="+n+" eps="+eps);
}
}

