package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynVar2C.java,v 1.32 2022/07/28 12:27:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVar2C {
    final EVXData I;
    final double I_0,I_IL,I_IH,p_IX,I_IX,pi_sum,pid_sum,pd_IX,pdd_IX,var_pI_IX,var_pI_spur,var2I_IX,p_var2I_IX,var_pI_sum,var_pI_sumA,p2_sum,pp_sum,pa_sum,pe_sum,pae_sum,dp_var_pI_sumA,impD,impU;
    final ACorr a;
    DynVar2C(final EVXData I,final double I_0,final double I_IL,final double I_IH,final double p_IX,final double I_IX,final double pi_sum,final double pid_sum,final double pd_IX,final double pdd_IX,
	     final double var_pI_IX,final double var_pI_spur,final double var2I_IX,final double p_var2I_IX,final double var_pI_sum,final double var_pI_sumA,final double dp_var_pI_sumA,
	     final double p2_sum,final double pp_sum,final double pa_sum,
	     final double pe_sum,final double pae_sum,
	     final ACorr a,
	     final double impD,final double impU)
{
    this.I=I;
    this.I_0=I_0;
    this.I_IL=I_IL;
    this.I_IH=I_IH;
    this.p_IX=p_IX;
    this.I_IX=I_IX;
    this.pi_sum=pi_sum;
    this.pid_sum=pid_sum;
    this.pd_IX=pd_IX;
    this.pdd_IX=pdd_IX;
    this.var_pI_IX=var_pI_IX;
    this.var_pI_spur=var_pI_spur;
    this.var2I_IX=var2I_IX;
    this.p_var2I_IX=p_var2I_IX;
    this.var_pI_sum=var_pI_sum;
    this.var_pI_sumA=var_pI_sumA;
    this.dp_var_pI_sumA=dp_var_pI_sumA;
    this.p2_sum=p2_sum;
    this.pp_sum=pp_sum;
    this.pa_sum=pa_sum;
    this.pe_sum=pe_sum;
    this.pae_sum=pae_sum;
    this.a=a;
    this.impD=impD;
    this.impU=impU;
}
    static boolean PRINT_DIAG=false;

static final int SELECTION_IH=101,SELECTION_IPSIX02=102;
/**
 *  For psiH: I|psiH>=lambda_max |psiH>
 *  variate <psi|pI|psi>/<psi|psi>.
 */
static DynVar2C get_DynVar2C(final int n,
			     final double [] QQi,final double [] QQt,final double [] QQpi,
			     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,
			     final int selection_IX)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int iL=0,iX;
    if(selection_IX==SELECTION_IH) iX=n-1;
    else if(selection_IX==SELECTION_IPSIX02)
    {
	int i=n-1;double imm=-1;
	for(int k=0;k<n;k++)
	{
	    final double ik=I.ev[k]*Math.pow(M.B.getD0(I.getPsi(k),I.x0),2);
	    if(ik>=imm){i=k;imm=ik;}
	}
	iX=i;
    }
    else throw new IllegalArgumentException("selection_IX="+selection_IX+" is not supported");
    final double [] psi=I.getPsi(iX),varpsi=VOLATILITY_EDPsi.getDPsi(psi),var2psi=VOLATILITY_EDPsi.getDPsi(varpsi),
	dvarpsi=TMatr.sumAcB(varpsi,-TMatr.getAMatrB(varpsi,psi,QQt,n),psi);

    final double [] rqI=RayleighQuotient.getRQVariationValues(psi,varpsi,var2psi,QQi,QQt,n),
	rqpI=RayleighQuotient.getRQVariationValues(psi,varpsi,var2psi,QQpi,QQt,n);
    final double I_IX=I.ev[iX],pI_IX=I.getFPsi(iX,QQpi),p_IX=pI_IX/I_IX,
	pd_IX=I.getFPsiPsi(iX,varpsi,QQpi)/I.getFPsiPsi(iX,varpsi,QQi),
	pdd_IX=TMatr.getAMatrA(dvarpsi,QQpi,n)/TMatr.getAMatrA(dvarpsi,QQi,n),
	var_pI_IX=rqpI[1],
	var2I_IX=rqI[2],p_var2I_IX=rqpI[2]/var2I_IX,
	pI_IL=I.getFPsi(iL,QQpi);
    double asi=0,aspi=0,si=0,spi=0,sid=0,spid=0,svarpi=0,sia=0,spvi=0,svi=0,svarpiA=0,
	aesi=0,aespi=0,esi=0,espi=0,sv2i=0,
	s_dp_dI=0,s_dp_dp=0,s_dI_dI=0,var_pI_spur=0;
    for(int k=0;k<I.neq;k++)
    {
	final double [] psik=I.getPsi(k),Dpsik=VOLATILITY_EDPsi.getDPsi(psik);
	final double vpp=2*TMatr.getAMatrB(psik,Dpsik,QQpi,n),
	    varpik=2*(0.5*vpp-TMatr.getAMatrA(psik,QQpi,n)*TMatr.getAMatrB(psik,Dpsik,QQt,n)),
	    pik=TMatr.getAMatrA(psik,QQpi,n),pk=pik/I.ev[k],
	    v2I=2*(TMatr.getAMatrA(Dpsik,QQi,n)-I.ev[k]*TMatr.getAMatrA(Dpsik,QQt,n)),
	    psi2x0=2*TMatr.getAMatrB(psik,Dpsik,QQt,n),
	    ii0=I.ev[k]*psi2x0;
	var_pI_spur+=varpik;
	asi+=I.ev[k];
	aspi+=pik;
	aesi+=ii0;
	aespi+=vpp;
	if(v2I<0)
	{
	    esi+=ii0;
	    espi+=vpp;
	    svarpi+=varpik;
	    svarpiA+=varpik;
	    sia+=I.ev[k];
	    svi+=v2I;
	    spvi+=pk*v2I;
	    sv2i+=v2I;

	    si+=ii0;
	    spi+=pik*psi2x0;
	    sid+=2*TMatr.getAMatrB(psik,Dpsik,QQi,n);
	    spid+=2*TMatr.getAMatrB(psik,Dpsik,QQpi,n);
	}
	else
	{
	    svarpiA-=varpik;
	    sv2i-=v2I;
	}
	s_dp_dI+=(varpik/I.ev[k])*v2I;
	s_dp_dp+=Math.pow(varpik/I.ev[k],2);
	s_dI_dI+=v2I*v2I;

	if(PRINT_DIAG)
	{
	    System.err.println("DynVar2C:k="+k+" v2I="+v2I+" varpik="+varpik);
	    if(k==iX){
		System.err.println("DynVar2C:varpik="+varpik+" var_pI_IX="+var_pI_IX);
		System.err.println("DynVar2C:v2I="+v2I+" var2I_IX="+var2I_IX);
	    }
	}
    }
    final double pi_sum=spi/si,pid_sum=spid/sid,var_pI_sum=svarpi,var_pI_sumA=svarpiA,pp_sum=spvi/svi,pa_sum=aspi/asi,
	pe_sum=espi/esi,pae_sum=aespi/aesi,dp_var_pI_sumA=svarpiA/sv2i;

    if(PRINT_DIAG)
    {
	System.err.println("DynVar2C:var_pI_sum="+var_pI_sum+" (pid_sum-pi_sum)*sid="+((pid_sum-pi_sum)*sid)+" si="+si+" sid="+sid+" pid_sum="+pid_sum+" pe_sum="+pe_sum);
    }

    //final double [] QQDip=getDQQfiMatrOrthogonal(I,QQpi,QQt,VOLATILITY_EDPsi);
    //final double [] evDip=M.EV.getEV(n,QQDip,true);
    //final double impD=evDip[0],impU=evDip[n-1];
    final double impD=Double.NaN,impU=Double.NaN;
    if(false && PRINT_DIAG)
    {
	System.err.println("DynVar2C: QQDt="+java.util.Arrays.toString(getDQQfiMatrOrthogonal(I,QQt,QQt,VOLATILITY_EDPsi)));
	final double [] QQDi=getDQQfiMatrOrthogonal(I,QQi,QQt,VOLATILITY_EDPsi);
	final double [] evDi=M.EV.getEV(n,QQDi,false);
	System.err.println("DynVar2C: QQDi="+java.util.Arrays.toString(QQDi));
	System.err.println("DynVar2C: evDi="+java.util.Arrays.toString(evDi));
    }

    double s2pi=0,s2i=0;
    for(int k=0;k<I.neq;k++)
    {
	final double [] psik=I.getPsi(k),Dpsik=VOLATILITY_EDPsi.getDPsi(psik),
	    D2psik=VOLATILITY_EDPsi.getDPsi(Dpsik);
	 final double [] rqIk=RayleighQuotient.getRQVariationValues(psik,Dpsik,D2psik,QQi,QQt,n),
	     rqpIk=RayleighQuotient.getRQVariationValues(psik,Dpsik,D2psik,QQpi,QQt,n);
	 final double v2I=rqIk[2],v2pI=rqpIk[2],sgn;
	 if(v2I<0)
	 { // max I
	     sgn=1;
	 }
	 else
	 { // min I
	     sgn=-1; // may be to use 0 ??
	 }
	 s2i+=sgn*v2I;
	 s2pi+=sgn*v2pI;
	 if(PRINT_DIAG)
	 {
	     System.err.println("DynVar2C:k="+k+" v2I="+v2I+" rqpIk[1]="+rqpIk[1]+" rqpIk[2]="+rqpIk[2]+" rqpIk[3]="+rqpIk[3]+" rqIk[2]="+rqIk[2]+" rqIk[3]="+rqIk[3]);
	 }
    }
    final double p2_sum=s2pi/s2i;

    return new DynVar2C(I,I.s0,I.sL,I.sH,p_IX,I_IX,pi_sum,pid_sum,pd_IX,pdd_IX,var_pI_IX,var_pI_spur,var2I_IX,p_var2I_IX,var_pI_sum,var_pI_sumA,dp_var_pI_sumA,p2_sum,pp_sum,pa_sum,pe_sum,pae_sum,new ACorr(s_dp_dI,s_dp_dp,s_dI_dI),impD,impU);
}

static double [] getDpsiOrthoginal(final double [] psi,final double [] QQt,final int n,final VolMatrix VOLATILITY_EDPsi)
{
    final double [] Dpsi=VOLATILITY_EDPsi.getDPsi(psi),dpsi=new double [psi.length];
    System.arraycopy(Dpsi,0,dpsi,0,Dpsi.length);
    TMatr.sumP1(dpsi,psi,-TMatr.getAMatrB(psi,Dpsi,QQt,n));
    return dpsi;
}
static double [] getDQQfiMatrOrthogonal(final EVXData I,final double [] QQfi,final double [] QQt,final VolMatrix VOLATILITY_EDPsi)
{
    final int n=I.neq;
    final double [] QQDfi=new double [n*n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=I.getPsi(k),dpsik=getDpsiOrthoginal(psik,QQt,n,VOLATILITY_EDPsi);
	for(int l=0;l<=k;l++)
	{
	    final double [] psil=I.getPsi(l),dpsil=getDpsiOrthoginal(psil,QQt,n,VOLATILITY_EDPsi);
	    QQDfi[k*n+l]=QQDfi[l*n+k]=TMatr.getAMatrB(psil,dpsik,QQfi,n)+TMatr.getAMatrB(dpsil,psik,QQfi,n);
	}
    }
    return QQDfi;
}

}

