package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Linsystems;

/**
 *  @version $Id: DynVar2Ca.java,v 1.14 2020/09/27 20:33:11 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVar2Ca {
    final EVXData I;
    final double p_IH,var1pI,dp2It,dp1P;
	DynVar2Ca(final EVXData I,final double p_IH,final double var1pI,final double dp2It,final double dp1P)
{
    this.I=I;
    this.p_IH=p_IH;
    this.var1pI=var1pI;
    this.dp2It=dp2It;
    this.dp1P=dp1P;
}


/**
 *  For psiH: I|psiH>=lambda_max |psiH>
 *  variate <psi|pI|psi>/<psi|psi> and similar.
 */
static DynVar2Ca get_DynVar2Ca(final int n,
			       final double [] QQi,final double [] QQt,final double [] QQpi,
			       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
{
    final int iX=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] psi=I.psiH,Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
    TMatr.sumP1(Dpsi,psi,-TMatr.getAMatrB(Dpsi,psi,QQt,n));
    final double IH=I.sH,p_IH=TMatr.getAMatrA(psi,QQpi,n)/IH,
	var1pI=2*TMatr.getAMatrB(psi,Dpsi,QQpi,n);
    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	bQQi=new double [n*n],bQQt=new double [n*n];
    for(int k=0;k<n;k++) {bQQi[k+k*n]=I.ev[k];bQQt[k+k*n]=1;}
    final double [] bDpsi=EVXData.convertPsiFromQtoB_Basis(n,Dpsi,I.ev,QQt),bpsi=new double [n];
    bpsi[iX]=1;
    final double [] bQQpipH=new double [n*n],bQQiiH=new double [n*n];
    for(int o=0;o<n*n;o++) {
	bQQpipH[o]=bQQpi[o]-p_IH*bQQi[o];
	bQQiiH[o]=bQQi[o]-IH*bQQt[o];
    }
    final double dp2It=TMatr.getAMatrA(bDpsi,bQQpipH,n)/TMatr.getAMatrA(bDpsi,bQQiiH,n),
	bvar1pI=2*TMatr.getAMatrB(bpsi,bDpsi,bQQpi,n);

    final int nd=n-1;
    final double [] pih=new double [nd];
    for(int k=0,o=0;k<n;k++)
    {
	if(k==iX) continue;
	pih[o++]=bQQpi[iX+k*n];
    }
    final double [] spih=pih.clone(),MpipHi=DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(n,bQQpipH,iX);
    final int [] ipvt=new int [nd];
    final int pinfo=Linsystems.dgefa_m1(MpipHi,nd,nd,ipvt);
    if(pinfo!=0)
    {
	java.util.Arrays.fill(pih,Double.NaN);
    }
    else
    {
	Linsystems.dgesl_m1(MpipHi,nd,nd,ipvt,pih,1);
    }
    final double dp1P=-TMatr.mulVV(pih,spih)/IH;

    if(false)
    {
	System.err.println("get_DynVar2Ca:pih="+java.util.Arrays.toString(spih)+" MpipHi="+java.util.Arrays.toString(MpipHi)+" IH="+IH);
	System.err.println("get_DynVar2Ca:dp1P="+dp1P);
	System.err.println("get_DynVar2Ca:Invpih="+java.util.Arrays.toString(pih));
	System.err.println("get_DynVar2Ca:bDpsi="+java.util.Arrays.toString(bDpsi));
	System.err.println("var1pI="+var1pI+" bvar1pI="+bvar1pI+" have to match");
    }
    return new DynVar2Ca(I,p_IH,var1pI,dp2It,dp1P);
}


static DynVar2Ca test_get_DynVar2Ca(final int n,
				    final double [] QQi,final double [] QQt,final double [] QQpi,
				    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
{
    final int iX=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] psi=I.psiH,Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
    TMatr.sumP1(Dpsi,psi,-TMatr.getAMatrB(Dpsi,psi,QQt,n));
    final double IH=I.sH,p_IH=TMatr.getAMatrA(psi,QQpi,n)/IH,
	var1pI=2*TMatr.getAMatrB(psi,Dpsi,QQpi,n),
	id2=TMatr.getAMatrA(Dpsi,QQi,n),
	dp2It=(TMatr.getAMatrA(Dpsi,QQpi,n)-p_IH*id2)/(id2-IH*TMatr.getAMatrA(Dpsi,QQt,n));
    final int nd=n-1;
    final double [] pih=new double [nd],MpipHi=new double [nd*nd];
    for(int k=0,ok=0;k<n;k++)
    {
	if(k==iX) continue;
	pih[ok]=I.getFPsiPsi(iX,k,QQpi);
	for(int l=0,ol=0;l<n;l++)
	{
	    if(l==iX) continue;
	    MpipHi[ok+ol*nd]=I.getFPsiPsi(k,l,QQpi)-p_IH*I.getFPsiPsi(k,l,QQi);
	    ol++;
	}
	ok++;
    }
    final double dp1P=-TMatr.getAMatrA(pih,Linsystems.getInvertedMatrix(nd,nd,MpipHi),nd)/IH;
    if(false)
    {
	System.err.println("test_get_DynVar2Ca:pih="+java.util.Arrays.toString(pih)+" MpipHi="+java.util.Arrays.toString(MpipHi)+" IH="+IH);
	System.err.println("test_get_DynVar2Ca:dp1P="+dp1P);
    }
    return new DynVar2Ca(I,p_IH,var1pI,dp2It,dp1P);
}

static void matchDynVar2Ca(final DynVar2Ca a,final DynVar2Ca b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.var1pI,"a.var1pI",b.var1pI,"b.var1pI",eps);
    UnitTests.checkDoubleRel(a.dp2It,"a.dp2It",b.dp2It,"b.dp2It",eps);
    UnitTests.checkDoubleRel(a.dp1P,"a.dp1P",b.dp1P,"b.dp1P",eps);
    System.err.println("DynVarC:matchDynVar2Ca test OK n="+a.I.neq+" eps="+eps);
}


}

