package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynVar2DIDP.java,v 1.42 2022/08/07 15:44:02 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVar2DIDP {
    final EVXData I;
    final double p_offset,p_last__with_p_offset_included,p_IH,I_0_feq,I_0_dIeq0,sumdp,sumdi,sumdpdi,
	delta_P,delta_Pdyn,Pf_from_dI,Pf_from_ptpi,Pf_from_pt_true_pi,Pf_from_dyn,Pf_from_f0,sumdpifmik,sumdpifmik_ee,
	Pf_SPUR;

    DynVar2DIDP(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,
		final double p_IH,final double I_0_feq,final double I_0_dIeq0,
		final double sumdp,final double sumdi,final double sumdpdi,
		final double delta_P,final double delta_Pdyn,final double Pf_from_dI,final double Pf_from_ptpi,final double Pf_from_pt_true_pi,final double Pf_from_dyn,final double Pf_from_f0,
		final double sumdpifmik,final double sumdpifmik_ee,
		final double Pf_SPUR,
		final double eps)
    {
	this.I=I;
	this.p_offset=p_offset;
	this.p_last__with_p_offset_included=p_last__with_p_offset_included;
	this.p_IH=p_IH;
	this.I_0_feq=I_0_feq;
	this.I_0_dIeq0=I_0_dIeq0;
	this.sumdp=sumdp;
	this.sumdi=sumdi;
	this.sumdpdi=sumdpdi;
	this.delta_P=delta_P;
	this.delta_Pdyn=delta_Pdyn;
	this.Pf_from_dI=Pf_from_dI;
	this.Pf_from_ptpi=Pf_from_ptpi;
	this.Pf_from_pt_true_pi=Pf_from_pt_true_pi;
	this.Pf_from_dyn=Pf_from_dyn;
	this.Pf_from_f0=Pf_from_f0;
	this.sumdpifmik=sumdpifmik;
	this.sumdpifmik_ee=sumdpifmik_ee;
	this.Pf_SPUR=Pf_SPUR;
    }


/**
 * Find psi state, dI/dt eigenvector: dI/dt|psi>=lambda |psi>,
 * Calculation of <Q_i|dI/dt|Q_j> matrix can be done by parts
 * <Q_i|dI/dt|Q_j> = I^{f} Q_i(x0) Q_j(x0) - (<Q_i|I|D(Q_j)> + <D(Q_i)|I|Q_j>)
 * assume I^{f}=IH.
 * Also manipulate with spur.
 * No optimization in this version. Just a try.
 */
static DynVar2DIDP get_DynVar2DIDP(final MomentsData mm,
				   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				   final VolMatrix VOLATILITY_EDPsi,
				   final boolean add_dI,
				   final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double I_0=I.s0,I_0_feq=add_dI?I.sH:I.s0,dI=I_0_feq-I_0;
    final double [] QQ0=TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),
	QQi_feq=TMatr.sumAcB(mm.QQi,dI/I.o_o,QQ0),
	QQpi_feq=TMatr.sumAcB(mm.QQpi,mm.p_last*dI/I.o_o,QQ0),
	QQDi_feq=TMatr.scaleVectorInPlace(VOLATILITY_EDPsi.getDiffMatrix(n,QQi_feq),-1),
	QQDpi_rfeq=VOLATILITY_EDPsi.getDiffMatrix(n,QQpi_feq);
    final double I_0_dIeq0=-TMatr.getAMatrA(I.psi0,QQDi_feq,n)/I.o_o;
    // For |dI/dt| operator assume future I is such that <psi0|dI/dt|psi0>=0.
    TMatr.sumP1(QQDi_feq,QQ0,I_0_dIeq0);
    final double [] bQQdp=EVXData.convertMatrFromQtoB_Basis(n,mm.QQdp,I.ev),
	bQQDi_feq=EVXData.convertMatrFromQtoB_Basis(n,QQDi_feq,I.ev),
	bQQi_feq=EVXData.convertMatrFromQtoB_Basis(n,QQi_feq,I.ev),
	bQQDpi_rfeq=EVXData.convertMatrFromQtoB_Basis(n,QQDpi_rfeq,I.ev),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQpi_feq=EVXData.convertMatrFromQtoB_Basis(n,QQpi_feq,I.ev);

    final double sumdp=TMatr.getSpurM(n,bQQdp),
	sumdi=TMatr.getSpurM(n,bQQDi_feq),
	sumdpdi=TMatr.getSpurMM(n,bQQdp,bQQDi_feq);

    if(eps>0)
    {
	final EVXData DI=EVXData.getEVXDataFromQQ(n,QQDi_feq,mm.QQt,M);
	double Sdpdi=0,Sdi=0;
	for(int k=0;k<n;k++)
	{
	    Sdi+=Math.pow(DI.psik_at_x0[k],2)*DI.ev[k];
	    Sdpdi+=DI.getFPsi(k,mm.QQdp)*DI.ev[k];
	}
	UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psi0,QQi_feq,n),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psi0,QQDi_feq,n),"<psi0|DIf|psi0>",0,"0",eps);
	UnitTests.checkDoubleRel(DI.s0,"DI.s0",0,"0",eps);
	UnitTests.checkDoubleRel(Sdi,"Sdi",0,"0",eps);
	UnitTests.checkDoubleRel(Sdpdi,"Sdpdi",sumdpdi,"sumdpdi",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(DI.getEValues()),"spur(DI)",(I_0_dIeq0-I_0_feq)*I.o_o,"(I_0_dIeq0-I_0_feq)*o_o",eps);
	UnitTests.checkDoubleRel(I_0_dIeq0,"I_0_dIeq0",dI+TMatr.getAMatrA(I.psi0,VOLATILITY_EDPsi.getDiffMatrix(n,mm.QQi),n)/I.o_o,"dI+I_0_dIeq0(nodi)",eps);
	// Check spur invariance
	final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,mm.QQt);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,mm.QQdp,IQQt,QQDi_feq})),"spur(G^{-1} | QQdp | G^{-1} |QQDi_feq)",sumdpdi,"sumdpdi",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,QQDi_feq})),"spur(G^{-1} | QQDi_feq)",TMatr.sumElements(DI.getEValues()),"spur(DI)",eps);
	UnitTests.checkDoubleRel(sumdi,"sumdi",TMatr.sumElements(DI.getEValues()),"spur(DI)",eps);

	System.err.println("DynVar2DIDP: test OK n="+n+" dI="+dI+" I_0_feq="+I_0_feq+" I_0_dIeq0="+I_0_dIeq0+" DI.s0="+DI.s0+" spur(DI)="+TMatr.sumElements(DI.getEValues())+" add_dI="+add_dI+" eps="+eps);
    }

    // Try |I^f| basis
    final double [] bbasis=M.EV.getEV(n,bQQi_feq,true),bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o)),
	psik_psi0=new double [n],psik_dp_psik=new double [n],If_m_Ik=new double [n],
	psik_pIf_psik=new double [n],
	psik_pI_psik=new double [n],
	psik_pdi_psik=new double [n],psik_pj_psik=new double [n],psik_wdi_psik=new double [n],
	psikB_pI_psikB=new double [n],psikB_psi0=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n);

	psik_dp_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQdp,n);
	psik_psi0[k]=TMatr.mulVV(psik,bpsi0);
	If_m_Ik[k]=I_0_feq-bbasis[k];
	psik_pI_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n);
	psik_pIf_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi_feq,n);
	psik_pdi_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQDpi_rfeq,n)/bbasis[k];
	psik_pj_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n)/bbasis[k];
	psik_wdi_psik[k]=Math.pow(psik_psi0[k],2)*dI/bbasis[k];
	psikB_psi0[k]=I.getFPsiPsi(k,I.psi0,mm.QQt);
	psikB_pI_psikB[k]=I.getFPsi(k,mm.QQpi);
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(TMatr.mulV2V(psik,I.getEValues())/bbasis[k]+psik_wdi_psik[k],"wpf+wpo",1,"1",eps);
	}
    }
    final double [] w0=TMatr.squareArrayElements(psik_psi0);
    final double delta_P=-TMatr.mulVVV(w0,If_m_Ik,psik_dp_psik)/(TMatr.mulV2V(w0,If_m_Ik)*I.o_o),
	delta_Pdyn=(TMatr.mulVV(w0,psik_dp_psik)-TMatr.getAMatrA(I.psi0,mm.QQdp,n))/(I.o_o*(1-TMatr.mulVV(w0,w0))),
	Pf_from_dI=TMatr.mulVVV(w0,If_m_Ik,psik_pdi_psik)/(TMatr.mulV2V(w0,If_m_Ik)*I.o_o),
	Pf_from_ptpi=-TMatr.mulVVV(w0,If_m_Ik,psik_pj_psik)/TMatr.mulVVV(w0,If_m_Ik,psik_wdi_psik),
	Pf_from_pt_true_pi=(I_0_feq*TMatr.mulVV(w0,psik_pj_psik)-TMatr.getAMatrA(I.psi0,mm.QQpi,n))/(dI-I_0_feq*TMatr.mulVV(w0,psik_wdi_psik)),
	Pf_from_dyn=(TMatr.mulV2V(psik_psi0,psik_pI_psik)-TMatr.mulV2V(psikB_psi0,psikB_pI_psikB))/dI/(1-TMatr.mulVV(w0,w0)),
	Pf_from_f0=(TMatr.mulV2V(psik_psi0,psik_pI_psik)-TMatr.getAMatrA(I.psi0,mm.QQpi,n))/dI/(1-TMatr.mulVV(w0,w0));

    if(eps>0)
    {
	UnitTests.checkDoubleRel(TMatr.sumElements(w0),"sum(w0)",1,"1",eps);
	UnitTests.checkDoubleRel(I_0_feq,"I_0_feq",TMatr.mulVV(w0,java.util.Arrays.copyOf(bbasis,n)),"<psi0|If|psi0>",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(If_m_Ik,w0),"sum(If_m_Ik*w0)",0,"0",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(psik_dp_psik),"sum(psik_dp_psik)",sumdp,"sumdp",eps);
    }
    if(eps>0)
    {
	for(int k=0;k<n;k++)
	{
	    final double [] psi=EVXData.convertPsiFromBtoQ_Basis(n,EVXData.getPsiFromBasis(k,bbasis,n),I.ev),
		Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	    final double pdi=2*TMatr.getAMatrB(psi,Dpsi,QQpi_feq,n)/bbasis[k],
		psik2x0=2*TMatr.getAMatrB(psi,Dpsi,QQi_feq,n)/bbasis[k];
	    UnitTests.checkDoubleRel(psik_pdi_psik[k],"psik_pdi_psik[k]",pdi,"pdi",eps);
	    UnitTests.checkDoubleRel(Math.pow(M.B.getD0(psi,I.x0),2),"psik^2(x0)",psik2x0,"psik2x0",eps);
	}
    }

    final double [] weightk_ee=new double [n];
    final double I_0_dIeq0_ee=TMatr.mulV2V(w0,java.util.Arrays.copyOf(bbasis,n))/TMatr.mulVV(w0,w0);
    for(int k=0;k<n;k++)
    {
	weightk_ee[k]=(I_0_dIeq0_ee-bbasis[k])*w0[k];
    }
    if(eps>0)
    {
	UnitTests.checkDoubleRel(TMatr.mulV2V(psik_psi0,If_m_Ik),"sum((I_0_feq-Ik)*psik^2(x0)",0,"0",eps);
	//UnitTests.checkDoubleRel(I_0_dIeq0_ee,"I_0_dIeq0_ee",I_0_dIeq0,"I_0_dIeq0",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(weightk_ee),"sum(weightk_ee)",I_0_dIeq0_ee-I_0_feq,"I_0_dIeq0_ee-I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(weightk_ee,w0),"sum(weightk_ee*w0)",0,"0",eps);
    }


    return new DynVar2DIDP(I,mm.p_offset,mm.p_last,
			   TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH,
			   I_0_feq,I_0_dIeq0,
			   sumdp,sumdi,sumdpdi,
			   delta_P,delta_Pdyn,Pf_from_dI,Pf_from_ptpi,Pf_from_pt_true_pi,Pf_from_dyn,Pf_from_f0,
			   TMatr.mulVV(psik_dp_psik,If_m_Ik),
			   TMatr.mulVV(psik_dp_psik,weightk_ee),
			   TMatr.sumElements(psik_pIf_psik)/TMatr.sumElements(java.util.Arrays.copyOf(bbasis,n)),
			   eps);

}


}

