package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.DiffSkewness;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynVar2DISkewness.java,v 1.30 2022/07/28 12:27:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVar2DISkewness {
    final EVXData I;
    final double p_offset,p_last__with_p_offset_included,p_IH,I_0_feq,IHf_w,ILf_w,DIHf_w,DIhaf_w,DILf_w;
    final Skewness IHf_S,IHf_S__nodI,ILf_S,ILf_S__nodI,DIHf_S,DIHf_S__nodI,DIhaf_S,DIhaf_S__nodI,DILf_S,DILf_S__nodI,DIIH_S;
    final double p_DI,sk_DI,pd_DI,pd_DII,pd_DI0,pd_DIDp,dpd_DI;

    DynVar2DISkewness(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,
		      final double p_IH,final double I_0_feq,
		      final Skewness IHf_S,final Skewness IHf_S__nodI,final double IHf_w,
		      final Skewness ILf_S,final Skewness ILf_S__nodI,final double ILf_w,
		      final Skewness DIHf_S,final Skewness DIHf_S__nodI,final double DIHf_w,
		      final Skewness DIhaf_S,final Skewness DIhaf_S__nodI,final double DIhaf_w,
		      final Skewness DILf_S,final Skewness DILf_S__nodI,final double DILf_w,
		      final Skewness DIIH_S,
		      final double p_DI,final double sk_DI,final double pd_DI,final double pd_DII,final double pd_DI0,final double pd_DIDp,final double dpd_DI,
		      final double eps)
    {
	this.I=I;
	this.p_offset=p_offset;
	this.p_last__with_p_offset_included=p_last__with_p_offset_included;
	this.p_IH=p_IH;
	this.I_0_feq=I_0_feq;
	this.IHf_S=IHf_S;
	this.IHf_S__nodI=IHf_S__nodI;
	this.IHf_w=IHf_w;
	this.ILf_S=ILf_S;
	this.ILf_S__nodI=ILf_S__nodI;
	this.ILf_w=ILf_w;
	this.DIHf_S=DIHf_S;
	this.DIHf_w=DIHf_w;
	this.DIHf_S__nodI=DIHf_S__nodI;
	this.DIhaf_S=DIhaf_S;
	this.DIhaf_w=DIhaf_w;
	this.DIhaf_S__nodI=DIhaf_S__nodI;
	this.DILf_S=DILf_S;
	this.DILf_S__nodI=DILf_S__nodI;
	this.DILf_w=DILf_w;
	this.DIIH_S=DIIH_S;
	this.p_DI=p_DI;
	this.sk_DI=sk_DI;
	this.pd_DI=pd_DI;
	this.pd_DII=pd_DII;
	this.pd_DI0=pd_DI0;
	this.pd_DIDp=pd_DIDp;
	this.dpd_DI=dpd_DI;
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(DIIH_S.w1+DIIH_S.w2,"DIIH_S: w1+w2",1,"1",eps);
	}
    }


/**
 * Find psi state, dI/dt eigenvector: dI/dt|psi>=lambda |psi>,
 * Calculation of <Q_i|dI/dt|Q_j> matrix can be done by parts
 * <Q_i|dI/dt|Q_j> = I^{f} Q_i(x0) Q_j(x0) - (<Q_i|I|D(Q_j)> + <D(Q_i)|I|Q_j>)
 * assume I^{f}=IH.
 * calculate p skewness.
 * Compare with DynVar2DynImpactDI.java where dp/dt is calculated.
 * No optimization in this version. Just a try.
 */
static DynVar2DISkewness get_DynVar2DISkewness(final MomentsData mm,
					       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
					       final VolMatrix VOLATILITY_EDPsi,
					       final double eps)
{
    final int n=mm.n,ixH=n-1; // the state with maximal I
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double I_0=I.s0,I_0_feq=I.sH,dI=I_0_feq-I_0,
	p_last__with_p_offset_included=mm.p_last;
    final double [] QX0=M.B.getX0Moments(n,I.x0),QQ0=TMatr.getQQaa(QX0),
	QQDi=TMatr.scaleVectorInPlace(VOLATILITY_EDPsi.getDiffMatrix(n,mm.QQi),-1),
	QQDpi_part=TMatr.scaleVectorInPlace(VOLATILITY_EDPsi.getDiffMatrix(n,mm.QQpi),-1);
    TMatr.sumP1(QQDi,QQ0,I_0_feq);
    final EVXData DI=EVXData.getEVXDataFromQQ(n,QQDi,mm.QQt,M),
	DII=EVXData.getEVXDataFromQQ(n,QQDi,mm.QQi,M);
    final double [] QQi_feq=TMatr.sumAcB(mm.QQi,dI/I.o_o,QQ0);
    final EVXData I_feq=EVXData.getEVXDataFromQQ(n,QQi_feq,mm.QQt,M);
    if(eps>0)
    {
	// Match QQDi matrix on |psi> states:
	for(int k=0;k<n;k++)
	{
	    UnitTests.checkDoubleRel(I.getFPsi(k,QQDi),"<psik|QQDi|psik>",(I.sH-I.ev[k])*Math.pow(M.B.getD0(I.getPsi(k),I.x0),2),"(I.sH-I.ev[k])*psik^2(x0)",eps);
	}
	// Match spur.
	UnitTests.checkDoubleRel(TMatr.sumElements(DI.getEValues()),"sum(DI.ev)",dI*DI.o_o,"dI*DI.o_o",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psi0,QQDi,n),"<psi0|QQDi|psi0>",DI.s0,"DI.s0",eps);
	System.err.println("DynVar2DISkewness: DI.wH^2="+Math.pow(DI.wH,2)+" dI="+dI+" dI/(IH-IL)="+(dI/(I.sH-I.sL)));
	// Match future I.
	UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psi0,QQi_feq,n),"<psi0|QQi_feq|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(I_feq.s0,"I_feq.s0",I_0_feq,"I_0_feq",eps);

	System.err.println("DynVar2DISkewness: test OK n="+n+" dI="+(I_0_feq-I.s0)+" DI.s0="+DI.s0+" eps="+eps);
    }

    final DiffSkewness IHf_S=DiffSkewness.getDiffSkewness_FromMonomials(
	    TMatr.getAMatrA(I_feq.psiH,mm.QQi,n),
	    TMatr.getAMatrA(I_feq.psiH,mm.QQpi,n),
	    TMatr.getAMatrA(I_feq.psiH,mm.QQp2i,n),
	    TMatr.getAMatrA(I_feq.psiH,mm.QQp3i,n),
	    dI*Math.pow(I_feq.wH,2)),
	ILf_S=DiffSkewness.getDiffSkewness_FromMonomials(
	    TMatr.getAMatrA(I_feq.psiL,mm.QQi,n),
	    TMatr.getAMatrA(I_feq.psiL,mm.QQpi,n),
	    TMatr.getAMatrA(I_feq.psiL,mm.QQp2i,n),
	    TMatr.getAMatrA(I_feq.psiL,mm.QQp3i,n),
	    dI*Math.pow(I_feq.wL,2)),
	DIHf_S=DiffSkewness.getDiffSkewness_FromMonomials(
	    TMatr.getAMatrA(DI.psiH,mm.QQi,n),
	    TMatr.getAMatrA(DI.psiH,mm.QQpi,n),
	    TMatr.getAMatrA(DI.psiH,mm.QQp2i,n),
	    TMatr.getAMatrA(DI.psiH,mm.QQp3i,n),
	    dI*Math.pow(DI.wH,2)),
	DIhaf_S=DiffSkewness.getDiffSkewness_FromMonomials(
	    DI.getFPsi(ixH-1,mm.QQi),
	    DI.getFPsi(ixH-1,mm.QQpi),
	    DI.getFPsi(ixH-1,mm.QQp2i),
	    DI.getFPsi(ixH-1,mm.QQp3i),
	    dI*Math.pow(DI.psik_at_x0[ixH-1],2)/DI.o_o),
	DILf_S=DiffSkewness.getDiffSkewness_FromMonomials(
	    TMatr.getAMatrA(DI.psiL,mm.QQi,n),
	    TMatr.getAMatrA(DI.psiL,mm.QQpi,n),
	    TMatr.getAMatrA(DI.psiL,mm.QQp2i,n),
	    TMatr.getAMatrA(DI.psiL,mm.QQp3i,n),
	    dI*Math.pow(DI.wL,2));

    double spif=0,sif=0,spifi=0,sifi=0,spifil=0,sifil=0,sdpi=0,sdi=0;
    for(int k=0;k<n;k++)
    {
	sifi+=DII.getFPsi(k,QQ0)*I_0_feq;
	spifi+=DII.ev[k]*DII.getFPsi(k,mm.QQpi)-DII.getFPsi(k,QQDpi_part);
	final double dif0=DI.getFPsi(k,QQ0)*I_0_feq,pdif=DI.getFPsi(k,QQDpi_part);
	sif+=dif0;
	spif+=DI.ev[k]*DI.getFPsi(k,mm.QQpi)/DI.getFPsi(k,mm.QQi)-pdif;
	sifil+=dif0-DI.ev[k];
	spifil+= -pdif;
	sdi+=Math.pow(DI.psik_at_x0[k],2)*DI.ev[k];
	sdpi+=DI.getFPsi(k,mm.QQdp)*DI.ev[k];
	if(eps>0)
	{
	    final double [] QQDi_part=TMatr.scaleVectorInPlace(VOLATILITY_EDPsi.getDiffMatrix(n,mm.QQi),-1);
	    final double iifi=DII.getFPsi(k,QQ0)*I_0_feq,iirfi=DII.ev[k]*DII.getFPsi(k,mm.QQi)-DII.getFPsi(k,QQDi_part);
	    UnitTests.checkDoubleRel(iifi,"iifi",iirfi,"iirfi",eps);
	    UnitTests.checkDoubleRel(DII.getFPsi(k,mm.QQi),"DII.getFPsi(k,mm.QQi)",1,"1",eps);
	    final double iif=DI.getFPsi(k,QQ0)*I_0_feq,iirf=DI.ev[k]*DI.getFPsi(k,mm.QQi)/DI.getFPsi(k,mm.QQi)-DI.getFPsi(k,QQDi_part);
	    UnitTests.checkDoubleRel(iif,"iif",iirf,"iirf",eps);
	    UnitTests.checkDoubleRel(DI.getFPsi(k,mm.QQt),"DI.getFPsi(k,mm.QQt)",1,"1",eps);
	}
    }
    final Skewness [] DIsk=new Skewness [n];
    double sp=0,ssk=0,sw=0;
    for(int k=0;k<n;k++)
    {
	DIsk[k]=Skewness.getSkewnessFromMonomialMoments(DI.getFPsi(k,mm.QQi),
							DI.getFPsi(k,mm.QQpi),
							DI.getFPsi(k,mm.QQp2i),
							DI.getFPsi(k,mm.QQp3i));
	final double w=DI.ev[k];
	sp+=DIsk[k].xa*w;
	ssk+=DIsk[k].sk*w;
	sw+=w;
    }
    if(eps>0)
    {
	double si=0;
	for(Skewness s: DIsk){si+=s.w1+s.w2;}
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues()),"sum(I.ev)",si,"si",eps);
	UnitTests.checkDoubleRel(sw,"sw",dI*I.o_o,"dI*I.o_o",eps);
	Skewness.matchSkewness(DIHf_S.sknodi,DIsk[ixH],eps);
	Skewness.matchSkewness(DIhaf_S.sknodi,DIsk[ixH-1],eps);
	Skewness.matchSkewness(DILf_S.sknodi,DIsk[0],eps);
	UnitTests.checkDoubleRel(sifil,"sifil",I.s0*I.o_o,"I.s0*I.o_o",eps);
	UnitTests.checkDoubleRel(sif,"sif",I.sH*I.o_o,"I.sH*I.o_o",eps);
	UnitTests.checkDoubleRel(sdi,"sdi",DI.s0*I.o_o,"DI.s0*I.o_o",eps);
    }

    return new DynVar2DISkewness(
	I,mm.p_offset,p_last__with_p_offset_included,
	TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH,I_0_feq,
	IHf_S.getSkewnessAt(p_last__with_p_offset_included),IHf_S.sknodi,Math.pow(I_feq.wH,2),
	ILf_S.getSkewnessAt(p_last__with_p_offset_included),ILf_S.sknodi,Math.pow(I_feq.wL,2),
	DIHf_S.getSkewnessAt(p_last__with_p_offset_included),DIHf_S.sknodi,Math.pow(DI.wH,2),
	DIhaf_S.getSkewnessAt(p_last__with_p_offset_included),DIhaf_S.sknodi,Math.pow(DI.psik_at_x0[ixH-1],2)/DI.o_o,
	DILf_S.getSkewnessAt(p_last__with_p_offset_included),DILf_S.sknodi,Math.pow(DI.wL,2),
	Skewness.getSkewnessFromMonomialMoments(DII.getFPsi(ixH-1,mm.QQi),
						DII.getFPsi(ixH-1,mm.QQpi),
						DII.getFPsi(ixH-1,mm.QQp2i),
						DII.getFPsi(ixH-1,mm.QQp3i)),
	sp/sw,ssk/sw,spif/sif,spifi/sifi,spifil/sifil,-sdpi/sdi,sdpi/TMatr.sumElements(DI.getEValues()),
	eps);
}


}

