package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynVar2DiagProj.java,v 1.36 2022/08/07 15:44:02 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVar2DiagProj {
    final EVXData I;
    final double p_IH,var_pI_IH,pI_dI_IH,p_dI_IH,var2_pI_IH,p_var2_IH,p_var2_A,p_A_I0,var_pI_A;
    final double [] dp_I,p_I,var2_I,psi2k_x0;
    DynVar2DiagProj(final EVXData I,final double p_IH,final double var_pI_IH,
		    final double pI_dI_IH,final double p_dI_IH,
		    final double [] var1_pI,final double [] var_pI_dI,
		    final double [] var2_I,final double [] var2_pI,
		    final double [] pIk,final double [] psi2k_x0)
{
    this.I=I;
    this.p_IH=p_IH;
    this.var_pI_IH=var_pI_IH;
    this.pI_dI_IH=pI_dI_IH;
    this.p_dI_IH=p_dI_IH;
    this.var2_pI_IH=var2_pI[I.neq-1];
    this.p_var2_IH=var2_pI_IH/var2_I[I.neq-1];
    this.dp_I=new double [I.neq];
    this.p_I=new double [I.neq];
    this.var2_I=var2_I;
    this.psi2k_x0=psi2k_x0;
    double si=0,spi=0;
    for(int k=0;k<I.neq;k++)
    {
	dp_I[k]=-var1_pI[k]/I.ev[k];
	p_I[k]=pIk[k]/I.ev[k];
	if(var2_I[k]<0)
	{
	    si+=var2_I[k];
	    spi+=var2_pI[k];
	}
	else
	{
	    si-=var2_I[k];
	    spi-=var2_pI[k];
	}
    }
    this.p_var2_A=spi/si;
    this.p_A_I0=TMatr.mulVVn1(psi2k_x0,pIk)/TMatr.mulVVn1(psi2k_x0,I.ev);
    this.var_pI_A=TMatr.sumElements(var1_pI);
}


/**
 * For psi, I eigenvector: I|psi>=lambda |psi>
 * variate <psi|I|psi>/<psi|psi>
 * then assume the price p is diagonal in the same basis,
 * i.e. pI is diagonal in the basis I is diagonal.
 * Then try second variation.
 */
static DynVar2DiagProj get_DynVar2DiagProj(final int n,
				   final double [] QQi,final double [] QQt,final double [] QQpi,
				   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,
				   final double eps)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] psi_IH=I.psiH,Dpsi_IH=VOLATILITY_EDPsi.getDPsi(psi_IH);
    final double pI_IH=TMatr.getAMatrA(psi_IH,QQpi,n),p_IH=pI_IH/I.sH,
	var_pI_IH=2*(TMatr.getAMatrB(psi_IH,Dpsi_IH,QQpi,n)-pI_IH*TMatr.getAMatrB(psi_IH,Dpsi_IH,QQt,n));
    double spI_dI_IH=0,sp_dI_IH=0;
    for(int k=0;k<n;k++)
    {
	final double a=I.getFPsiPsi(k,psi_IH,QQpi)*(I.sH-I.ev[k])*I.getFPsiPsi(k,Dpsi_IH,QQt);
	spI_dI_IH+=a;
	sp_dI_IH+=a/Math.sqrt(I.sH*I.ev[k]);
    }

    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	bpi_diag=new double [n],
	bQQdI=EVXData.convertMatrFromQtoB_Basis(n,TMatr.sumAcB(TMatr.scaleVectorInPlace(TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),I.sH),-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQi)),I.ev);
    final double [] var1_pI=new double [n],var1_pI0=new double [n],var2_I=new double [n],var2_pI=new double [n],pIk=new double[n],psi2k_x0=new double [n],var_pI_dI=new double [n],bQQpi_nodiag=bQQpi.clone();
    for(int k=0;k<n;k++){
	bpi_diag[k]=bQQpi[k+n*k];
	bQQpi_nodiag[k+n*k]=0;
    }
    for(int k=0;k<n;k++)
    {
	final double [] psik=I.getPsi(k),Dpsik=VOLATILITY_EDPsi.getDPsi(psik),
	    bDpsik=EVXData.convertPsiFromQtoB_Basis(n,Dpsik,I.ev,QQt);
	final double pI=bQQpi[k+n*k],bdd=TMatr.mulVV(bDpsik,bDpsik);
	var2_I[k]=2*(TMatr.mulV2V(bDpsik,I.getEValues())-I.ev[k]*bdd);
	var2_pI[k]=2*(TMatr.mulV2V(bDpsik,bpi_diag)-pI*bdd);
	pIk[k]=pI;
	psi2k_x0[k]=2*bDpsik[k];
	var1_pI[k]=2*(DeprecatedHelperTMatr.mulVVo(n,bDpsik,0,bQQpi,k*n)-pI*bDpsik[k]);
	var_pI_dI[k]=DeprecatedHelperTMatr.mulVVo(n,bQQpi_nodiag,k*n,bQQdI,k*n);
	if(eps>0)
	{
	    final double psi2x0=Math.pow(M.B.getD0(psik,I.x0),2);
	    UnitTests.checkDoubleRel(psi2k_x0[k],"psi2k_x0[k]",psi2x0,"psi2x0",eps);
	}
    }
    if(eps>0)
    { // run unit test
	EVXData.test_orthogonality("DynVar2DiagProj.I.ev",n,I.ev,QQi,QQt,eps);
	UnitTests.checkDoubleArrayRel(psi2k_x0,"psi2k_x0",TMatr.squareArrayElements(I.psik_at_x0),"(I.psik_at_x0)^2",eps);
	final double [] bQQpi_diag=new double [n*n],QQpi_diag=new double [n*n];
	for(int k=0;k<n;k++) bQQpi_diag[k+n*k]=bQQpi[k+n*k];
	for(int k=0;k<n;k++)
	{
	    final double [] psiQk=new double [n];psiQk[k]=1;
	    final double [] bpsik=EVXData.convertPsiFromQtoB_Basis(n,psiQk,I.ev,QQt);
	    for(int l=0;l<n;l++)
	    {
		final double [] psiQl=new double [n];psiQl[l]=1;
		final double [] bpsil=EVXData.convertPsiFromQtoB_Basis(n,psiQl,I.ev,QQt);
		QQpi_diag[k*n+l]=TMatr.getAMatrB(bpsik,bpsil,bQQpi_diag,n);
	    }
	}
	final double [] _QQpi_diag=EVXData.convertMatrFromBtoQ_Basis(n,bQQpi_diag,I.ev,QQt);
	UnitTests.checkDoubleArrayRel(QQpi_diag,"QQpi_diag",_QQpi_diag,"_QQpi_diag",eps);
	for(int k=0;k<n;k++)
	{
	    final double [] psi=I.getPsi(k),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	    final double [] rqI=RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQi,QQt,n);
	    UnitTests.checkDoubleRel(rqI[0],"rqI[0]",I.ev[k],"I.ev[k]",eps);
	    UnitTests.checkDoubleRel(rqI[1],"rqI[1]",0,"0",eps);
	    UnitTests.checkDoubleRel(rqI[2],"rqI[2]",var2_I[k],"var2_I[k]",eps);
	    final double [] rqpIdia=RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQpi_diag,QQt,n);
	    UnitTests.checkDoubleRel(rqpIdia[0],"rqpIdia[0]",bpi_diag[k],"bpi_diag[k]",eps);
	    UnitTests.checkDoubleRel(rqpIdia[1],"rqpIdia[1]",0,"0",eps);
	    UnitTests.checkDoubleRel(rqpIdia[2],"rqpIdia[2]",var2_pI[k],"var2_pI[k]",eps);
	    final double [] rqpI=RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQpi,QQt,n);
	    UnitTests.checkDoubleRel(rqpI[0],"rqpI[0]",bpi_diag[k],"bpi_diag[k]",eps);
	    UnitTests.checkDoubleRel(rqpI[1],"rqpI[1]",var1_pI[k],"var1_pI[k]",eps);
	}
	UnitTests.checkDoubleRel(pI_IH,"pI_IH",pIk[n-1],"pIk[n-1]",eps);
	UnitTests.checkDoubleRel(var_pI_IH,"var_pI_IH",var1_pI[n-1],"var1_pI[n-1]",eps);
	UnitTests.checkDoubleRel(spI_dI_IH,"pI_dI_IH",var_pI_dI[n-1],"var_pI_dI[n-1]",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(var_pI_dI),"sum(var_pI_dI)",TMatr.getSpurMM(n,bQQpi_nodiag,bQQdI),"spur(bQQdI*bQQpi_nodiag)",eps);
	System.err.println("DynVar2DiagProj.get_DynVar2DiagProj: all internal tests OK n="+n+" eps="+eps);
    }

    return new DynVar2DiagProj(I,p_IH,var_pI_IH,spI_dI_IH,sp_dI_IH,var1_pI,var_pI_dI,var2_I,var2_pI,pIk,psi2k_x0);
}


/*
static void matchDynVar2C(final DynVar2DiagProj a,final DynVar2C b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IX,"b.p_IX",eps);
    UnitTests.checkDoubleRel(a.varPi,"a.varPi",b.var_pI_sum,"b.var_pI_sum",eps);
    UnitTests.checkDoubleRel(a.varAPi,"a.varAPi",b.var_pI_sumA,"b.var_pI_sumA",eps);
    UnitTests.checkDoubleRel(a.varHPi,"a.varHPi",b.var_pI_IX,"b.var_pI_IX",eps);
    System.err.println("DynVarPiPt:matchDynVar2C test OK n="+a.I.neq+" eps="+eps);
}
*/


}

