package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynVar2DynImpact.java,v 1.19 2022/07/28 11:15:39 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVar2DynImpact {
    final EVXData I;
    final double p_IL,p_IH,pa_sum,dp_EQ;
    final ACorr a,aI0,ap0,am,am0;
    DynVar2DynImpact(final EVXData I,
		     final double [] dp,final double [] dI,final double [] p,final double [] psi2x0)
{
    this.I=I;
    this.p_IL=p[0];
    this.p_IH=p[p.length-1];
    this.pa_sum=TMatr.mulVVn1(p,I.ev)/TMatr.sumElements(I.getEValues());
    this.a=new ACorr(TMatr.mulVV(dp,dI),TMatr.mulVV(dp,dp),TMatr.mulVV(dI,dI));
    final double [] dI0=new double [dI.length],dp0=new double [dI.length];
    for(int k=0;k<dI0.length;k++){
	dI0[k]=dI[k]*psi2x0[k];
	dp0[k]=dp[k]*psi2x0[k];
    }
    this.aI0=new ACorr(TMatr.mulVV(dp,dI0),TMatr.mulVV(dp,dp),TMatr.mulVV(dI0,dI0));
    this.ap0=new ACorr(TMatr.mulVV(dp0,dI),TMatr.mulVV(dp0,dp0),TMatr.mulVV(dI,dI));
    double sab=0,saa=0,sbb=0,sdi=0,sab0=0,saa0=0,sbb0=0;
    for(int k=0;k<dI.length;k++)
    {
	if(dI[k]<0)
	{
	    sab+=dp[k]*dI[k];
	    saa+=dp[k]*dp[k];
	    sbb+=dI[k]*dI[k];
	    sdi+=dI[k];
	    sab0+=dp[k]*dI0[k];
	    saa0+=dp[k]*dp[k];
	    sbb0+=dI0[k]*dI0[k];
	}
    }
    this.am=new ACorr(sab,saa,sbb);
    this.dp_EQ=sab/sdi;
    this.am0=new ACorr(sab0,saa0,sbb0);

    if(PRINT_DIAG)
    {
	System.err.println("DynVar2DynImpact: I0="+I.s0+" = spur(I,psi^2(x0))/spur(psi^2(x0))="+(TMatr.mulVVn1(psi2x0,I.ev)/TMatr.sumElements(psi2x0)));
    }
}

    static final boolean PRINT_DIAG=false;

/**
 * For psi, I eigenvector: I|psi>=lambda |psi>
 * variate <psi|I|psi>/<psi|psi> and <psi|pI|psi>/<psi|psi>
 * on infinitesimal psi time-shift variation |Dpsi>=|D|psi>,
 * then consider dynamic impact through density matrix state.
 */
static DynVar2DynImpact get_DynVar2DynImpact(final int n,
					     final double [] QQi,final double [] QQt,final double [] QQpi,
					     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] dp=new double [n],dI=new double [n],p=new double [n],psi2x0=new double [n];
    for(int k=0;k<I.neq;k++)
    {
	final double [] psik=I.getPsi(k),Dpsik=VOLATILITY_EDPsi.getDPsi(psik);
	final double pIk=TMatr.getAMatrA(psik,QQpi,n),psik2x0=2*TMatr.getAMatrB(psik,Dpsik,QQt,n),
	    varpik=2*TMatr.getAMatrB(psik,Dpsik,QQpi,n)-pIk*psik2x0;
	dp[k]=varpik/I.ev[k];
	dI[k]=2*(TMatr.getAMatrA(Dpsik,QQi,n)-I.ev[k]*TMatr.getAMatrA(Dpsik,QQt,n));
	p[k]=pIk/I.ev[k];
	psi2x0[k]=psik2x0;
    }
    if(PRINT_DIAG)
    {
	System.err.println("dI="+java.util.Arrays.toString(dI));
    }
    return new DynVar2DynImpact(I,dp,dI,p,psi2x0);
}


static void matchDynVar2C(final DynVar2DynImpact a,final DynVar2C b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IX,"b.p_IX",eps);
    UnitTests.checkDoubleRel(a.pa_sum,"a.pa_sum",b.pa_sum,"b.pa_sum",eps);
    ACorr.matchACorr(a.a,b.a,eps);
    System.err.println("DynVar2DynImpact:matchDynVar2C test OK n="+a.I.neq+" eps="+eps);
}


}

