package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynVar2I.java,v 1.39 2022/07/28 22:30:49 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVar2I {
final double I_0,I_IL,I_IH,p_IX,P_DIL,p_DIL,pe_DIL,Di_DIL,P_DIH,p_DIH,pe_DIH,Di_DIH;
DynVar2I(final double I_0,final double I_IL,final double I_IH,final double p_IX,
	 final double P_DIL,final double p_DIL,final double pe_DIL,final double Di_DIL,
	 final double P_DIH,final double p_DIH,final double pe_DIH,final double Di_DIH)
{
    this.I_0=I_0;
    this.I_IL=I_IL;
    this.I_IH=I_IH;
    this.p_IX=p_IX;
    this.P_DIL=P_DIL;
    this.p_DIL=p_DIL;
    this.pe_DIL=pe_DIL;
    this.Di_DIL=Di_DIL;
    this.P_DIH=P_DIH;
    this.p_DIH=p_DIH;
    this.pe_DIH=pe_DIH;
    this.Di_DIH=Di_DIH;
}


static DynVar2I get_DynVar2I(final int n,
			     final double [] QQi,final double [] QQt,final double [] QQpi,
			     final double pl_m_offset,final boolean use_pl_pm_offset,
			     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double I_IX=I.ev[iX],pI_IX=I.getFPsi(iX,QQpi),p_IX=pI_IX/I_IX,If=I_IX;
    final double [] QX0=M.B.getX0Moments(n,I.x0),QQ0=TMatr.getQQaa(QX0),
	QQDi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi),
	BX0=DeprecatedHelperEVXData.convertQX0FromQtoB_Basis(n,QX0,I.ev);
    TMatr.sumP1(QQDi,QQ0,-If);
    final double [] bQQDi=EVXData.convertMatrFromQtoB_Basis(n,QQDi,I.ev), /*bQQDi[iX+iX*n]=0;*/ /* reduce rounding errors */
	bQQDpi=EVXData.convertMatrFromQtoB_Basis(n,VOLATILITY_EDPsi.getDiffMatrix(n,QQpi),I.ev),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev);
    final double [] evDi=M.EV.getEV(n,bQQDi,true);
    final int iDIL=0,iDIH=n-1;
    final double pf=(use_pl_pm_offset?pl_m_offset:p_IX),
	Di_DIL=evDi[iDIL],j_DIL=If*Math.pow(DeprecatedHelperTMatr.mulVVo(n,BX0,0,evDi,n+(iDIL)*n),2),
	jp_DIL=TMatr.getAMatrA(n,evDi,n+(iDIL)*n,bQQDpi,n),
	I_DIL=TMatr.mulV2V(EVXData.getPsiFromBasis(iDIL,evDi,n),I.getEValues()),
	P_DIL=TMatr.getAMatrA(n,evDi,n+(iDIL)*n,bQQpi,n)/I_DIL,
	p_DIL=(jp_DIL-pf*j_DIL)/Di_DIL,pe_DIL=jp_DIL/(Di_DIL+j_DIL),
	Di_DIH=evDi[iDIH],j_DIH=If*Math.pow(DeprecatedHelperTMatr.mulVVo(n,BX0,0,evDi,n+(iDIH)*n),2),
	jp_DIH=TMatr.getAMatrA(n,evDi,n+(iDIH)*n,bQQDpi,n),
	I_DIH=TMatr.mulV2V(EVXData.getPsiFromBasis(iDIH,evDi,n),I.getEValues()),
	P_DIH=TMatr.getAMatrA(n,evDi,n+(iDIH)*n,bQQpi,n)/I_DIH,
	p_DIH=(jp_DIH-pf*j_DIH)/Di_DIH,pe_DIH=jp_DIH/(Di_DIH+j_DIH);
    //System.err.println("DynVar2I: Di_DIL="+Di_DIL+" Di_DIH="+Di_DIH+" bQQDi="+java.util.Arrays.toString(bQQDi));

    return new DynVar2I(I.s0,I.sL,I.sH,p_IX,P_DIL,p_DIL,pe_DIL,Di_DIL,P_DIH,p_DIH,pe_DIH,Di_DIH);
}
/**
 */
static DynVar2I test_get_DynVar2I(final int n,
				  final double [] QQi,final double [] QQt,final double [] QQpi,
				  final double pl_m_offset,final boolean use_pl_pm_offset,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,final double eps)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] QQ0=VOLATILITY_EDPsi.getDiffMatrix(n,QQt),
	QQDi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi),QQDpi=VOLATILITY_EDPsi.getDiffMatrix(n,QQpi),_QQDpi=QQDpi.clone();
    final double I_IX=I.ev[iX],pI_IX=I.getFPsi(iX,QQpi),p_IX=pI_IX/I_IX;
    for(int k=0;k<QQDi.length;k++){
	QQDi[k]=QQDi[k]-I_IX*QQ0[k];
	QQDpi[k]=QQDpi[k]-(use_pl_pm_offset?I_IX*pl_m_offset:pI_IX)*QQ0[k];
    }
    final EVXData ID=EVXData.getEVXDataFromQQ(n,QQDi,QQt,M);
    final double p_DIL=ID.getFPsi(0,QQDpi)/ID.sL,
	I_DIL=ID.getFPsi(0,QQi),P_DIL=ID.getFPsi(0,QQpi)/I_DIL,
	pe_DIL=ID.getFPsi(0,_QQDpi)/(ID.sL+I_IX*ID.getFPsi(0,QQ0)),
	p_DIH=ID.getFPsi(n-1,QQDpi)/ID.sH,
	I_DIH=ID.getFPsi(n-1,QQi),P_DIH=ID.getFPsi(n-1,QQpi)/I_DIH,
	pe_DIH=ID.getFPsi(n-1,_QQDpi)/(ID.sH+I_IX*ID.getFPsi(n-1,QQ0));

    if(eps>=0)
    {
	final double [] psi=I.getPsi(iX);
	final DynVar1 d=new DynVar1(I.psiH,QQi,QQt,QQpi,VOLATILITY_EDPsi);
	final double di=TMatr.getAMatrA(psi,QQDi,n),dpi=TMatr.getAMatrA(psi,QQDpi,n);
	System.err.println("DynVar2I: use_pl_pm_offset="+use_pl_pm_offset+" <psi|QQDi|psi>="+di+"=0; <psi|QQDpi|psi>="+dpi+"=d.avar_p="+d.avar_p+" ID.ev="+java.util.Arrays.toString(ID.getEValues()));
	System.err.println("DynVar2I: ID.psiL(x0)="+M.B.getD0(ID.psiL,ID.x0)+"ID.psiH(x0)="+M.B.getD0(ID.psiH,ID.x0));
	UnitTests.checkDoubleRel(di,"di",0,"0",eps);
	if(!use_pl_pm_offset) UnitTests.checkDoubleRel(dpi,"dpi",d.avar_p,"d.avar_p",eps);
	UnitTests.checkDoubleRel(ID.sL,"ID.sL",ID.getFPsi(0,QQDi),"<psi_DIL|Di|psi_DIL>",eps);
	UnitTests.checkDoubleRel(ID.sH,"ID.sH",ID.getFPsi(n-1,QQDi),"<psi_DIH|Di|psi_DIH>",eps);
	for(int k=0;k<n;k++)
	{
	    UnitTests.checkDoubleRel(I.getFPsi(k,QQDi),"<psi_k|QQDi|psi_k>",k==iX?0:((I.ev[k]-I_IX)*Math.pow(M.B.getD0(I.getPsi(k),I.x0),2)),"(I.ev[k]-I_IX)*psik(x0)^2",eps);
	}
	final DynVar2I qL=get_DynVar2I(n,QQi,QQt,QQpi,pe_DIL,true,M,VOLATILITY_EDPsi);
	UnitTests.checkDoubleRel(qL.p_DIL,"qL.p_DIL",pe_DIL,"pe_DIL",eps);
	final DynVar2I qH=get_DynVar2I(n,QQi,QQt,QQpi,pe_DIH,true,M,VOLATILITY_EDPsi);
	UnitTests.checkDoubleRel(qH.p_DIH,"qH.p_DIH",pe_DIH,"pe_DIH",eps);
    }
    return new DynVar2I(I.s0,I.sL,I.sH,p_IX,P_DIL,p_DIL,pe_DIL,ID.sL,P_DIH,p_DIH,pe_DIH,ID.sH);
}

static DynVar2I getD_DynVar2I(final int n,
			     final double [] QQi,final double [] QQt,final double [] QQpi,
			     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double I_IX=I.ev[iX],pI_IX=I.getFPsi(iX,QQpi),p_IX=pI_IX/I_IX;

    final double [] QQDpi=VOLATILITY_EDPsi.getDiffMatrix(n,QQpi),
	QQDi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi);
    final EVXData ID=EVXData.getEVXDataFromQQ(n,QQDi,QQt,M); // ID.sH>0 is bounded by I_k*psi_k(x0)^2
    final int iDIL=0,iDIH=n-1;
    final double
	Di_DIL=ID.ev[iDIL],
	p_DIL=ID.getFPsi(iDIL,QQpi)/ID.getFPsi(iDIL,QQi),
	pe_DIL=ID.getFPsi(iDIL,QQDpi)/Di_DIL,
	Di_DIH=ID.ev[iDIH],
	p_DIH=ID.getFPsi(iDIH,QQpi)/ID.getFPsi(iDIH,QQi),
	pe_DIH=ID.getFPsi(iDIH,QQDpi)/Di_DIH;
    //System.err.println("DynVar2I.getD_DynVar2I:Di_DIL="+Di_DIL+" Di_DIH="+Di_DIH);
    //if(ID.sH<0) throw new RuntimeException();

    return new DynVar2I(I.s0,I.sL,I.sH,p_IX,p_DIL,p_DIL,pe_DIL,Di_DIL,p_DIL,p_DIH,pe_DIH,Di_DIH);
}


static DynVar2I get0_DynVar2I(final int n,
			      final double [] QQi,final double [] QQt,final double [] QQpi,
			      final double pl_m_offset,final boolean use_pl_pm_offset,
			      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double I_IX=basis[iX],pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),p_IX=pI_IX/I_IX,If=I_IX;
    final double [] QX0=M.B.getX0Moments(n,M.getRadauRoot()),QQ0=TMatr.getQQaa(QX0),
	QQDi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi),
	BX0=DeprecatedHelperEVXData.convertQX0FromQtoB_Basis(n,QX0,basis);
    TMatr.sumP1(QQDi,QQ0,-If);
    final double [] bQQDi=EVXData.convertMatrFromQtoB_Basis(n,QQDi,basis),
	bQQDpi=EVXData.convertMatrFromQtoB_Basis(n,VOLATILITY_EDPsi.getDiffMatrix(n,QQpi),basis),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis),
	bQQi=EVXData.convertMatrFromQtoB_Basis(n,QQi,basis);
    final double [] evDi=M.EV.getEV(n,bQQDi,true);
    final int iDIL=0,iDIH=n-1;
    final double pf=(use_pl_pm_offset?pl_m_offset:p_IX),
	Di_DIL=evDi[iDIL],j_DIL=If*Math.pow(DeprecatedHelperTMatr.mulVVo(n,BX0,0,evDi,n+n*iDIL),2),
	jp_DIL=TMatr.getAMatrA(n,evDi,n+(iDIL)*n,bQQDpi,n),
	P_DIL=TMatr.getAMatrA(n,evDi,n+(iDIL)*n,bQQpi,n)/TMatr.getAMatrA(n,evDi,n+(iDIL)*n,bQQi,n),
	p_DIL=(jp_DIL-pf*j_DIL)/Di_DIL,pe_DIL=jp_DIL/(Di_DIL+j_DIL),
	Di_DIH=evDi[iDIH],j_DIH=If*Math.pow(DeprecatedHelperTMatr.mulVVo(n,BX0,0,evDi,n+n*iDIH),2),
	jp_DIH=TMatr.getAMatrA(n,evDi,n+(iDIH)*n,bQQDpi,n),
	P_DIH=TMatr.getAMatrA(n,evDi,n+(iDIH)*n,bQQpi,n)/TMatr.getAMatrA(n,evDi,n+(iDIH)*n,bQQi,n),
	p_DIH=(jp_DIH-pf*j_DIH)/Di_DIH,pe_DIH=jp_DIH/(Di_DIH+j_DIH);
    //System.err.println("DynVar2I: Di_DIL="+Di_DIL+" Di_DIH="+Di_DIH+" bQQDi="+java.util.Arrays.toString(bQQDi));

    return new DynVar2I(basis[0],basis[1],basis[n-1],p_IX,P_DIL,p_DIL,pe_DIL,Di_DIL,P_DIH,p_DIH,pe_DIH,Di_DIH);
}

static DynVar2I test_get0_DynVar2I(final int n,
				   final double [] QQi,final double [] QQt,final double [] QQpi,
				   final double pl_m_offset,final boolean use_pl_pm_offset,
				   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,final double eps)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double I_IX=basis[iX],pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),p_IX=pI_IX/I_IX;
    final double [] QQ0=VOLATILITY_EDPsi.getDiffMatrix(n,QQt),
	QQDi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi),QQDpi=VOLATILITY_EDPsi.getDiffMatrix(n,QQpi),_QQDpi=QQDpi.clone();
    for(int k=0;k<QQDi.length;k++){
	QQDi[k]=QQDi[k]-I_IX*QQ0[k];
	QQDpi[k]=QQDpi[k]-(use_pl_pm_offset?pl_m_offset:pI_IX)*QQ0[k];
    }
    final EVXData ID=EVXData.getEVXDataFromQQ(n,QQDi,QQt,M);
    final double p_DIL=ID.getFPsi(0,QQDpi)/ID.sL,
	P_DIL=ID.getFPsi(0,QQpi)/ID.getFPsi(0,QQi),
	pe_DIL=ID.getFPsi(0,_QQDpi)/(ID.sL+I_IX*ID.getFPsi(0,QQ0)),
	p_DIH=ID.getFPsi(n-1,QQDpi)/ID.sH,
	P_DIH=ID.getFPsi(n-1,QQpi)/ID.getFPsi(n-1,QQi),
	pe_DIH=ID.getFPsi(n-1,_QQDpi)/(ID.sH+I_IX*ID.getFPsi(n-1,QQ0));
    if(eps>=0)
    {
	final double [] psi=EVXData.getPsiFromBasis(iX,basis,n);
	final DynVar1 d=new DynVar1(psi,QQi,QQt,QQpi,VOLATILITY_EDPsi);
	final double di=TMatr.getAMatrA(psi,QQDi,n),dpi=TMatr.getAMatrA(psi,QQDpi,n);
	System.err.println("DynVar2I.test_get0_DynVar2I: Di_DIL="+ID.sL+" Di_DIH="+ID.sH+" ID.ev="+java.util.Arrays.toString(ID.ev));
	System.err.println("DynVar2I.test_get0_DynVar2I: use_pl_pm_offset="+use_pl_pm_offset+" <psi|QQDi|psi>="+di+"; <psi|QQDpi|psi>="+dpi+"; d.avar_p="+d.avar_p);
	final DynVar2I qL=get0_DynVar2I(n,QQi,QQt,QQpi,pe_DIL,true,M,VOLATILITY_EDPsi);
	UnitTests.checkDoubleRel(qL.p_DIL,"qL.p_DIL",pe_DIL,"pe_DIL",eps);
	final DynVar2I qH=get0_DynVar2I(n,QQi,QQt,QQpi,pe_DIH,true,M,VOLATILITY_EDPsi);
	UnitTests.checkDoubleRel(qH.p_DIH,"qH.p_DIH",pe_DIH,"pe_DIH",eps);
    }
    return new DynVar2I(basis[0],basis[1],basis[n-1],p_IX,P_DIL,p_DIL,pe_DIL,ID.sL,P_DIH,p_DIH,pe_DIH,ID.sH);
}

public static void matchDynVar2I(final DynVar2I a,final DynVar2I b,final double eps)
{
    UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I_0,"b.I_0",eps);
    UnitTests.checkDoubleRel(a.I_IL,"a.I_IL",b.I_IL,"b.I_IL",eps);
    UnitTests.checkDoubleRel(a.I_IH,"a.I_IH",b.I_IH,"b.I_IH",eps);
    UnitTests.checkDoubleRel(a.p_IX,"a.p_IX",b.p_IX,"b.p_IX",eps);
    UnitTests.checkDoubleRel(a.P_DIL,"a.P_DIL",b.P_DIL,"b.P_DIL",eps);
    UnitTests.checkDoubleRel(a.p_DIL,"a.p_DIL",b.p_DIL,"b.p_DIL",eps);
    UnitTests.checkDoubleRel(a.pe_DIL,"a.pe_DIL",b.pe_DIL,"b.pe_DIL",eps);
    UnitTests.checkDoubleRel(a.Di_DIL,"a.Di_DIL",b.Di_DIL,"b.Di_DIL",eps);
    UnitTests.checkDoubleRel(a.P_DIH,"a.P_DIH",b.P_DIH,"b.P_DIH",eps);
    UnitTests.checkDoubleRel(a.p_DIH,"a.p_DIH",b.p_DIH,"b.p_DIH",eps);
    UnitTests.checkDoubleRel(a.pe_DIH,"a.pe_DIH",b.pe_DIH,"b.pe_DIH",eps);
    UnitTests.checkDoubleRel(a.Di_DIH,"a.Di_DIH",b.Di_DIH,"b.Di_DIH",eps);

    System.err.println("DynVar2I:matchDynVar2I test OK  eps="+eps);
}
}

