package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 *  Consider dI/dt. Some kind similar to DynVar2I.java and DynDI.java (Dirk Laurie <dirk@aims.ac.za> notes).
 *  @version $Id: DynVar2Ia.java,v 1.16 2020/09/27 20:33:12 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVar2Ia {
final EVXData I;
final double p_IX,I_IX;
DynVar2Ia(final EVXData I,
	  final double p_IX,final double I_IX)
{
    this.I=I;
    this.p_IX=p_IX;
    this.I_IX=I_IX;
}


static DynVar2Ia get_DynVar2Ia(final int n,
			     final double [] QQi,final double [] QQt,final double [] QQpi,
			     final double pl_m_offset,
			     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double I_IX=I.ev[iX],pI_IX=I.getFPsi(iX,QQpi),p_IX=pI_IX/I_IX,If=I_IX;
    final double [] QQ0=TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),
	QQDi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi),QQDpi=VOLATILITY_EDPsi.getDiffMatrix(n,QQpi);
    TMatr.sumP1(QQDi,QQ0,-If);
    TMatr.sumP1(QQDpi,QQ0,-If*pl_m_offset);
    final double [] bQQDi=EVXData.convertMatrFromQtoB_Basis(n,QQDi,I.ev),/*bQQDi[iX+iX*n]=0;*/ /* reduce rounding errors */
	bQQDpi=EVXData.convertMatrFromQtoB_Basis(n,QQDpi,I.ev),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev);

    if(false)
    {
	final double [] QQe=DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(n,bQQDi,iX);
	final double [] ev=M.EV.getEV(n-1,QQe,false);
	System.err.println("get_DynVar2Ia: ev="+java.util.Arrays.toString(ev));
	for(int k=0;k<n;k++) System.err.println("bQQDi["+k+","+k+"]="+bQQDi[k+n*k]);
	for(int k=0,o=0;k<n;k++)
	for(int l=0;l<n;l++)
	{
	    //if(k==iX||l==iX) continue;
	    final double [] psik=I.getPsi(k),psil=I.getPsi(l);
	    System.err.println("QQ["+k+","+l+"]="+bQQDi[o]+" test="+(
			       TMatr.getAMatrB(psik,VOLATILITY_EDPsi.getDPsi(psil),QQi,n)
			       +TMatr.getAMatrB(psil,VOLATILITY_EDPsi.getDPsi(psik),QQi,n)
			       -I_IX*M.B.getD0(psik,I.x0)*M.B.getD0(psil,I.x0))
			       );
	    o++;
	}
    }


    return new DynVar2Ia(I,p_IX,I_IX);
}

}

