package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynVar2PiPt.java,v 1.7 2020/09/27 20:33:12 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVar2PiPt {
    final EVXData I;
    final double p_IL,p_IH,varPit,varAPit,varHPit,varPi,varAPi,varHPi;
    DynVar2PiPt(final EVXData I,final double p_IL,final double p_IH,
		final double varPit,final double varAPit,final double varHPit,
		final double varPi,final double varAPi,final double varHPi)
{
    this.I=I;
    this.p_IL=p_IL;
    this.p_IH=p_IH;
    this.varPit=varPit;
    this.varAPit=varAPit;
    this.varHPit=varHPit;
    this.varPi=varPi;
    this.varAPi=varAPi;
    this.varHPi=varHPi;
}

    static boolean PRINT_DIAG=false;

/**
 * For I eigenvector psi: I|psi>=lambda |psi>
 * variate <psi|I|psi>/<psi|psi>
 * then try
 * 2(<psi|pI|Dpsi> - <psi|I|psi><psi|p|Dpsi>)
 * as direction indicator, what is different from the approach we previously tried:
 * 2(<psi|pI|Dpsi> - <psi|pI|psi><psi|Dpsi>).
 */
static DynVar2PiPt get_DynVar2PiPt(final int n,
				   final double [] QQi,final double [] QQt,final double [] QQpi,final double [] QQpt,
				   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iH=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double p_IL=TMatr.getAMatrA(I.psiL,QQpi,n)/I.sL,
	p_IH=TMatr.getAMatrA(I.psiH,QQpi,n)/I.sH;

    double svarPit=0,svarAPit=0,svarPi=0,svarAPi=0,svarHPit=0,svarHPi=0;
    for(int k=0;k<I.neq;k++)
    {
	final double [] psik=I.getPsi(k),Dpsik=VOLATILITY_EDPsi.getDPsi(psik);
	final double v2I=2*(TMatr.getAMatrA(Dpsik,QQi,n)-I.ev[k]*TMatr.getAMatrA(Dpsik,QQt,n)),
	    Dpi=TMatr.getAMatrB(psik,Dpsik,QQpi,n),
	    Dpt=TMatr.getAMatrB(psik,Dpsik,QQpt,n),
	    pi=TMatr.getAMatrA(psik,QQpi,n),Dt=TMatr.getAMatrB(psik,Dpsik,QQt,n),
	    vpit=2*(Dpi-I.ev[k]*Dpt),vpi=2*(Dpi-pi*Dt);
	if(k==iH)
	{
	    svarHPit=vpit;
	    svarHPi=vpi;
	}
	if(v2I<0)
	{
	    svarPit+=vpit;
	    svarPi+=vpi;
	    svarAPit+=vpit;
	    svarAPi+=vpi;
	}
	else
	{
	    svarAPit-=vpit;
	    svarAPi-=vpi;
	}
    }

    return new DynVar2PiPt(I,p_IL,p_IH,svarPit,svarAPit,svarHPit,svarPi,svarAPi,svarHPi);
}

static void matchDynVar2C(final DynVar2PiPt a,final DynVar2C b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IX,"b.p_IX",eps);
    UnitTests.checkDoubleRel(a.varPi,"a.varPi",b.var_pI_sum,"b.var_pI_sum",eps);
    UnitTests.checkDoubleRel(a.varAPi,"a.varAPi",b.var_pI_sumA,"b.var_pI_sumA",eps);
    UnitTests.checkDoubleRel(a.varHPi,"a.varHPi",b.var_pI_IX,"b.var_pI_IX",eps);
    System.err.println("DynVarPiPt:matchDynVar2C test OK n="+a.I.neq+" eps="+eps);
}


}

