package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 *  @version $Id: DynVarC.java,v 1.41 2022/08/05 11:25:32 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarC {
    final double I_0,I_IL,I_IH,p_IX,I_IX,p_cL,Id_cL,p_cH,Id_cH;
    DynVarC(final double I_0,final double I_IL,final double I_IH,final double p_IX,final double I_IX,final double p_cL,final double Id_cL,final double p_cH,final double Id_cH)
{
    this.I_0=I_0;
    this.I_IL=I_IL;
    this.I_IH=I_IH;
    this.p_IX=p_IX;
    this.I_IX=I_IX;
    this.p_cL=p_cL;
    this.Id_cL=Id_cL;
    this.p_cH=p_cH;
    this.Id_cH=Id_cH;
}

/**
 *  For psiH: I|psiH>=lambda_max |psiH>
 *  use (<psi|pI|psi>-<psiH|pI|psiH>)/(<psi|I|psi>-<psiH|I|psiH>) as estimation of p,
 *  what lead to eigenvalues equation
 *  pI|psi>-<psiH|pI|psiH>|psi> = lam * ( I|psi> - <psiH|I|psiH>|psi>)
 *  This version considers only psi orthogonal to psiH (probably wrong approach),
 *  but only this alows to have positive right part.
 */
static DynVarC get_DynVarC(final int n,
			   final double [] QQi,final double [] QQt,final double [] QQpi,
			   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			   final boolean flag_L)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=flag_L?0:n-1,nd=n-1;
    final double [] basis=M.EV.getGEV_Generalized(n,QQi,QQt,true),
	bQQi=TMatr.getDiagMatrix(java.util.Arrays.copyOf(basis,n)),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis);
    final double I_IX=bQQi[iX+n*iX],pI_IX=bQQpi[iX+n*iX],p_IX=pI_IX/I_IX;
    final double [] bbQQpj=DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(n,bQQpi,iX),
	bbQQj=DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(n,bQQi,iX);
    for(int k=0;k<nd;k++){bbQQpj[k+nd*k]-=pI_IX;bbQQj[k+nd*k]-=I_IX;}
    if(!flag_L)
    {
	TMatr.scaleVectorInPlace(bbQQpj,-1);
	TMatr.scaleVectorInPlace(bbQQj,-1);
    }
    final double [] pbasis=EVXData.getSymEVDiagT(nd,bbQQpj,bbQQj,M.EV);
    final double p_cL=pbasis[0],Id_cL=1/DeprecatedHelperTMatr.mulVVo(nd,pbasis,nd+nd*0,pbasis,nd+nd*0),
	p_cH=pbasis[nd-1],Id_cH=1/DeprecatedHelperTMatr.mulVVo(nd,pbasis,nd+nd*(nd-1),pbasis,nd+nd*(nd-1));

    return new DynVarC(Double.NaN,basis[0],basis[n-1],p_IX,I_IX,p_cL,Id_cL,p_cH,Id_cH);
}

/** Unit test for test_get_DynVarC.
 */
static DynVarC test_get_DynVarC(final int n,
				final double [] QQi,final double [] QQt,final double [] QQpi,
				final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				final boolean flag_L)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=flag_L?0:n-1,nd=n-1,sg=flag_L?1:-1;
    final double [] basis=M.EV.getGEV_Generalized(n,QQi,QQt,true),
	psi=EVXData.getPsiFromBasis(iX,basis,n);
    final double pI_IX=TMatr.getAMatrA(psi,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;
    final double [] bQQpj=new double [nd*nd],bQQj=new double [nd*nd],bQQt=new double [nd*nd];
    for(int k=0,jk=0;k<n;k++)
    {
	if(k==iX) continue;
	for(int l=0,jl=0;l<n;l++)
	{
	    if(l==iX) continue;
	    bQQpj[jk*nd+jl]=sg*(TMatr.getAMatrB(n,basis,n+k*n,basis,n+l*n,QQpi,n)-pI_IX*TMatr.getAMatrB(n,basis,n+k*n,basis,n+l*n,QQt,n));
	    bQQj[jk*nd+jl]=sg*(TMatr.getAMatrB(n,basis,n+k*n,basis,n+l*n,QQi,n)-I_IX*TMatr.getAMatrB(n,basis,n+k*n,basis,n+l*n,QQt,n));
	    bQQt[jk*nd+jl]=TMatr.getAMatrB(n,basis,n+k*n,basis,n+l*n,QQt,n);
	    jl++;
	}
	jk++;
    }
    final double [] pbasis=M.EV.getGEV_Generalized(nd,bQQpj,bQQj,true);
    final double p_cL=pbasis[0],Id_cL=1/TMatr.getAMatrA(nd,pbasis,nd+0*nd,bQQt,nd),
	p_cH=pbasis[nd-1],Id_cH=1/TMatr.getAMatrA(nd,pbasis,nd+(nd-1)*nd,bQQt,nd);
    return new DynVarC(Double.NaN,basis[0],basis[n-1],p_IX,I_IX,p_cL,Id_cL,p_cH,Id_cH);
}

/**
 * For psiH: I|psiH>=lambda_max |psiH>
 * Use P&L=<psi|pI|psi>-<psiH|pI|psiH>- p_last*(<psi|I|psi>-<psiH|I|psiH>).
 * then variation over p_last is=<psiH|I|psiH> - <psi|I|psi>
 * for psi: P&L|psi>=lam |psi>.
 * This version considers only psi orthogonal to psiH (probably wrong approach).
 */
static DynVarC get_PnL_DynVarC(final int n,
			       final double [] QQi,final double [] QQt,final double [] QQpi,
			       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			       final double op_exit,
			       final boolean flag_L)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=flag_L?0:n-1,nd=n-1;
    final double [] basis=M.EV.getGEV_Generalized(n,QQi,QQt,true),
	bQQi=TMatr.getDiagMatrix(java.util.Arrays.copyOf(basis,n)),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis);
    final double I_IX=bQQi[iX+n*iX],pI_IX=bQQpi[iX+n*iX],p_IX=pI_IX/I_IX;
    final double [] bbQQpi=DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(n,bQQpi,iX),
	bbQQi=DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(n,bQQi,iX);
    final double [] bbQQpnl=new double [nd*nd];
    for(int k=0;k<nd;k++){bbQQpnl[k+nd*k]=-(pI_IX-op_exit*I_IX);}
    for(int k=0;k<bbQQpnl.length;k++){bbQQpnl[k]+=bbQQpi[k]-op_exit*bbQQi[k];}

    final double [] pbasis=M.EV.getEV(nd,bbQQpnl,true);
    final double p_cL=pbasis[0],Id_cL=I_IX-TMatr.getAMatrA(nd,pbasis,nd+0*nd,bbQQi,nd),
	p_cH=pbasis[nd-1],Id_cH=I_IX-TMatr.getAMatrA(nd,pbasis,nd+(nd-1)*nd,bbQQi,nd);

    return new DynVarC(Double.NaN,basis[0],basis[n-1],p_IX,I_IX,p_cL,Id_cL,p_cH,Id_cH);
}
/** Same as get_PnL_DynVarC. Used for unit test.
 */
static DynVarC test_get_PnL_DynVarC(final int n,
				    final double [] QQi,final double [] QQt,final double [] QQpi,
				    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				    final double op_exit,
				    final boolean flag_L)
{
    final int iX=flag_L?0:n-1;
    final double [] basis=M.EV.getGEV_Generalized(n,QQi,QQt,true);
    final DynPnL d=new DynPnL(n,basis,n,new int []{iX},QQpi,QQi,QQt,op_exit,M);
    final double I_IX=basis[iX],pI_IX=TMatr.getAMatrA(n,basis,n+n*iX,QQpi,n),p_IX=pI_IX/I_IX,
	dpnl=pI_IX-op_exit*I_IX;
    return new DynVarC(Double.NaN,basis[0],basis[n-1],p_IX,I_IX,d.pnlL-dpnl,I_IX-d.I_pnlL,d.pnlH-dpnl,I_IX-d.I_pnlH);
}

/**
 * For psiH: I|psiH>=lambda_max |psiH>
 * Use P&L=<psi|pI|psi>-<psiH|pI|psiH>- p_last*(<psi|I|psi>-<psiH|I|psiH>).
 * then variation over p_last is=<psiH|I|psiH> - <psi|I|psi>
 * for psi as P&L eigenfunction: P&L|psi>=lam |psi>.
 * This version assumes boundary conditions psi(x0)=psiH(x0)=0.
 */
static DynVarC get0_PnL_DynVarC(final int n,
			       final double [] QQi,final double [] QQt,final double [] QQpi,
			       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			       final double op_exit)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1,nd=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;
    final double [] bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis,nd,n+n+0);
    final double dpnl=pI_IX-op_exit*I_IX;
    for(int k=0;k<nd;k++) {bQQpnl[k+k*nd]-=op_exit*basis[k+1]+dpnl;}
    final double [] pbasis=M.EV.getEV(nd,bQQpnl,true);
    final double p_cL=pbasis[0],Id_cL=I_IX-TMatr.mulV2V(EVXData.getPsiFromBasis(0,pbasis,nd),java.util.Arrays.copyOfRange(basis,1,n)),
	p_cH=pbasis[nd-1],Id_cH=I_IX-TMatr.mulV2V(EVXData.getPsiFromBasis(nd-1,pbasis,nd),java.util.Arrays.copyOfRange(basis,1,n));

    return new DynVarC(basis[0],basis[1],basis[n-1],p_IX,I_IX,p_cL,Id_cL,p_cH,Id_cH);
}

/** Unit test for get0_PnL_DynVarC.
*/
static DynVarC test_get0_PnL_DynVarC(final int n,
				     final double [] QQi,final double [] QQt,final double [] QQpi,
				     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				     final double op_exit)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1,nd=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;
    final double [] QQpnl=new double [n*n];
    for(int k=0;k<QQpnl.length;k++) QQpnl[k]=QQpi[k]-pI_IX*QQt[k]-op_exit*(QQi[k]-I_IX*QQt[k]);
    final double [] pbasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQpnl,QQt,M);
    return new DynVarC(basis[0],basis[1],basis[n-1],p_IX,I_IX,
		       pbasis[1],I_IX-TMatr.getAMatrA(n,pbasis,n+1*n,QQi,n),
		       pbasis[n-1],I_IX-TMatr.getAMatrA(n,pbasis,n+(n-1)*n,QQi,n));
}

/** Unit test for get0_PnL_DynVarC.
*/
static DynVarC test2_get0_PnL_DynVarC(final int n,
				     final double [] QQi,final double [] QQt,final double [] QQpi,
				     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				     final double op_exit)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1,nd=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX,
	dpnl=pI_IX-op_exit*I_IX;
    final DynPnL d=new DynPnL(n,basis,n,new int []{0},QQpi,QQi,QQt,op_exit,M);
    return new DynVarC(basis[0],basis[1],basis[n-1],p_IX,I_IX,
		       d.pnlL-dpnl,I_IX-d.I_pnlL,d.pnlH-dpnl,I_IX-d.I_pnlH);
}

/**
 * Similar to get0_PnL_DynVarC, but uses p_IH for op_exit.
 */
static DynVarC get0_PnLX_DynVarC(final int n,
				 final double [] QQi,final double [] QQt,final double [] QQpi,
				 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1,nd=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;
    final double [] bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis,nd,n+n+0);
    for(int k=0;k<nd;k++) {bQQpnl[k+k*nd]-=p_IX*basis[k+1];}
    final double [] pbasis=M.EV.getEV(nd,bQQpnl,true);
    final double p_cL=pbasis[0],Id_cL=I_IX-TMatr.mulV2V(EVXData.getPsiFromBasis(0,pbasis,nd),java.util.Arrays.copyOfRange(basis,1,n)),
	p_cH=pbasis[nd-1],Id_cH=I_IX-TMatr.mulV2V(EVXData.getPsiFromBasis(nd-1,pbasis,nd),java.util.Arrays.copyOfRange(basis,1,n));

    return new DynVarC(basis[0],basis[1],basis[n-1],p_IX,I_IX,p_cL,Id_cL,p_cH,Id_cH);
}
static DynVarC test_get0_PnLX_DynVarC(final int n,
				      final double [] QQi,final double [] QQt,final double [] QQpi,
				      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1,nd=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;
    final DynPnL d=new DynPnL(n,basis,n,new int []{0},QQpi,QQi,QQt,p_IX,M);
    return new DynVarC(basis[0],basis[1],basis[n-1],p_IX,I_IX,
		       d.pnlL,I_IX-d.I_pnlL,d.pnlH,I_IX-d.I_pnlH);
}
/** Same as get0_PnLX_DynVarC,
 * but no psi(x0)=psiH(x0)=0 constrain.
 */
static DynVarC get_PnLX_DynVarC(final int n,
				final double [] QQi,final double [] QQt,final double [] QQpi,
				final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final EVXData ev=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double pI_IX=ev.getFPsi(iX,QQpi),I_IX=ev.ev[iX],p_IX=pI_IX/I_IX;
    final double [] bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpi,ev.ev,n,n+0);
    for(int k=0;k<n;k++) {bQQpnl[k+k*n]-=p_IX*ev.ev[k];}
    final double [] pbasis=M.EV.getEV(n,bQQpnl,true);
    final double p_cL=pbasis[0],Id_cL=I_IX-TMatr.mulV2V(EVXData.getPsiFromBasis(0,pbasis,n),ev.getEValues()),
	p_cH=pbasis[n-1],Id_cH=I_IX-TMatr.mulV2V(EVXData.getPsiFromBasis(n-1,pbasis,n),ev.getEValues());

    return new DynVarC(ev.s0,ev.sL,ev.sH,p_IX,I_IX,p_cL,Id_cL,p_cH,Id_cH);
}
static DynVarC test_get_PnLX_DynVarC(final int n,
				     final double [] QQi,final double [] QQt,final double [] QQpi,
				     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final EVXData ev=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double pI_IX=ev.getFPsi(iX,QQpi),I_IX=ev.ev[iX],p_IX=pI_IX/I_IX;
    final DynPnL d=new DynPnL(n,ev.ev,n,new int []{},QQpi,QQi,QQt,p_IX,M);
    return new DynVarC(ev.s0,ev.sL,ev.sH,p_IX,I_IX,
		       d.pnlL,I_IX-d.I_pnlL,d.pnlH,I_IX-d.I_pnlH);
}

/** For |I|psiH>=I_IH |psiH> with a constrain psiH(x0)=0,
 * find p_IX=<psiH|pI|psiH>/I_IH and  variate P&L=\int (p-p_IX)dV, what lead to
 * <psi|C|psi>=lambda <psi|psi> (psi here has no constrain),
 * and find psi_cL & psi_cH corresponding to min/max P&L.
 * Note: a) psi is invarint to C->C+o shift (and lambda is not),
 * and b) if instead of C we consider C/V then this would give us p_{average}-p_IH.
 * (But consideration of price was much worse than P&L).
 *
 * This method is almost identical to DynVarFiniteC.get_PnLCV_DynVarFiniteC
 * but few differences:
 * 1. No distance_* variables, use only V - difference as distance.
 * 2. No infinitesimal variations, as they are probably useless for variation of P&L,
 * but are good for variation of Ip at eigenfunction of I.
 */
static DynVarC get_PnLCV_DynVarC(final int n,
				 final double [] QQi,final double [] QQt,final double [] QQpi,
				 final double [] QQV,final double [] QQpV,
				 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int i0=0,iX=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;
    final double [] QQpnl=TMatr.sumAcB(QQpV,-p_IX,QQV);
    TMatr.sumP1(QQpnl,QQt,-TMatr.getAMatrA(n,basis,n+iX*n,QQpnl,n));
    final double [] bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpnl,basis),
	bQQV=EVXData.convertMatrFromQtoB_Basis(n,QQV,basis);
    final double [] pbasis=M.EV.getEV(n,bQQpnl,true);
    final double V_IX=bQQV[n*iX+iX],p_cL=pbasis[0],Id_cL=V_IX-TMatr.getAMatrA(n,pbasis,n+0*n,bQQV,n),
	p_cH=pbasis[n-1],Id_cH=V_IX-TMatr.getAMatrA(n,pbasis,n+(n-1)*n,bQQV,n);
    return new DynVarC(basis[i0],basis[1],basis[n-1],p_IX,I_IX,p_cL,Id_cL,p_cH,Id_cH);
}
static DynVarC test_get_PnLCV_DynVarC(final int n,
				      final double [] QQi,final double [] QQt,final double [] QQpi,
				      final double [] QQV,final double [] QQpV,
				      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int i0=0,iX=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX,V_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQV,n);
    final double [] QQpnl=TMatr.sumAcB(QQpV,-p_IX,QQV);
    TMatr.sumP1(QQpnl,QQt,-TMatr.getAMatrA(n,basis,n+iX*n,QQpnl,n));
    final EVXData ev=EVXData.getEVXDataFromQQ(n,QQpnl,QQt,M);
    return new DynVarC(basis[i0],basis[1],basis[n-1],p_IX,I_IX,
		       ev.sL,V_IX-ev.getFPsi(0,QQV),ev.sH,V_IX-ev.getFPsi(n-1,QQV));
}

/** For |I|psiH>=I_IH |psiH> with a constrain psiH(x0)=0,
 * find p_IX=<psiH|pI|psiH>/I_IH and then check
 * (<psiH|C|psik>,<psik|V|psiH>) pseudocorrelation.
 */
static DynVarC get_PnLCV0_DynVarC(final int n,
				 final double [] QQi,final double [] QQt,final double [] QQpi,
				 final double [] QQV,final double [] QQpV,
				 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int i0=0,iX=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;
    final double [] bQQC=EVXData.convertMatrFromQtoB_Basis(n,TMatr.sumAcB(QQpV,-p_IX,QQV),basis),
	bQQV=EVXData.convertMatrFromQtoB_Basis(n,QQV,basis);
    final double [] bVpsi=new double [n],bCpsi=new double [n];
    for(int k=0;k<n;k++)
    {
	bVpsi[k]=bQQV[iX*n+k];
	bCpsi[k]=bQQC[iX*n+k];
    }
    final double X_IX=bCpsi[iX];
    bVpsi[iX]=bCpsi[iX]=0;
    final double x_a_b=TMatr.mulVV(bVpsi,bCpsi),x_a_a=TMatr.mulVV(bVpsi,bVpsi),x_b_b=TMatr.mulVV(bCpsi,bCpsi),
	rho=x_a_b/Math.sqrt(x_a_a*x_b_b);
    return new DynVarC(basis[i0],basis[1],basis[n-1],p_IX,I_IX,x_a_a,x_a_b,x_b_b,rho);
}

static void matchDynVarC(final DynVarC a,final DynVarC b,final double eps)
{
    UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I_0,"b.I_0",eps);
    UnitTests.checkDoubleRel(a.I_IL,"a.I_IL",b.I_IL,"b.I_IL",eps);
    UnitTests.checkDoubleRel(a.I_IH,"a.I_IH",b.I_IH,"b.I_IH",eps);
    UnitTests.checkDoubleRel(a.I_IX,"a.I_IX",b.I_IX,"b.I_IX",eps);
    UnitTests.checkDoubleRel(a.p_IX,"a.p_IX",b.p_IX,"b.p_IX",eps);
    UnitTests.checkDoubleRel(a.Id_cL,"a.Id_cL",b.Id_cL,"b.Id_cL",eps);
    UnitTests.checkDoubleRel(a.p_cL,"a.p_cL",b.p_cL,"b.p_cL",eps);
    UnitTests.checkDoubleRel(a.Id_cH,"a.Id_cH",b.Id_cH,"b.Id_cH",eps);
    UnitTests.checkDoubleRel(a.p_cH,"a.p_cH",b.p_cH,"b.p_cH",eps);
    System.err.println("DynVarC:matchDynVarC test OK eps="+eps);
}

}

