package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 *  Like DynVarCD, but no I.
 *  @version $Id: DynVarCDV.java,v 1.16 2022/07/28 12:27:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarCDV {
    final double I_0,I_IL,I_IH,p_IX,I_IX,pm_IX;
    final ACorr a;
DynVarCDV(final double I_0,final double I_IL,final double I_IH,
	  final double p_IX,final double I_IX,final double pm_IX,
	  final ACorr a)
{
    this.I_0=I_0;
    this.I_IL=I_IL;
    this.I_IH=I_IH;
    this.p_IX=p_IX;
    this.I_IX=I_IX;
    this.pm_IX=pm_IX;
    this.a=a;
}




/** Try (<psiH|QQC|psik>,<psik|QQD|psiH>) pseudocorrelation,
 *  here QQC=QQpV-p_IX*QQV with psiH(x0)=0 boundary condition.
 */
static DynVarCDV get0_DynVarCDVCV(final int n,
				  final double [] QQi,final double [] QQt,final double [] QQpi,
				  final double [] QQD,final double [] QQV,final double [] QQpV,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");
    final int i0=0,iX=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;
    final double [] bQQC=EVXData.convertMatrFromQtoB_Basis(n,TMatr.sumAcB(QQpV,-p_IX,QQV),basis),
	bQQD=EVXData.convertMatrFromQtoB_Basis(n,QQD,basis);
    final double [] bDpsi=new double [n],bCpsi=new double [n];
    for(int k=0;k<n;k++)
    {
	bDpsi[k]=bQQD[k*n+iX];
	bCpsi[k]=bQQC[iX*n+k];
    }
    //System.err.println("0:bDpsi="+java.util.Arrays.toString(bDpsi));
    bDpsi[iX]=bCpsi[iX]=0;
    final double x_a_b=TMatr.mulVV(bDpsi,bCpsi),x_a_a=TMatr.mulVV(bDpsi,bDpsi),x_b_b=TMatr.mulVV(bCpsi,bCpsi);
    return new DynVarCDV(basis[i0],basis[1],basis[n-1],p_IX,I_IX,x_a_b/x_a_a+p_IX,new ACorr(x_a_b,x_a_a,x_b_b));
}
static DynVarCDV test_get0_DynVarCDVCV(final int n,
				       final double [] QQi,final double [] QQt,final double [] QQpi,
				       final double [] QQD,final double [] QQV,final double [] QQpV,
				       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");
    final int i0=0,iX=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;
    final double [] bDpsi=new double [n],bCpsi=new double [n];
    for(int k=0;k<n;k++)
    {
	if(k==iX) continue;
	bCpsi[k]=TMatr.getAMatrB(n,basis,n+iX*n,basis,n+k*n,QQpV,n)-p_IX*TMatr.getAMatrB(n,basis,n+iX*n,basis,n+k*n,QQV,n);
	bDpsi[k]=TMatr.getAMatrB(n,basis,n+k*n,basis,n+iX*n,QQD,n);
    }
    final double x_a_b=TMatr.mulVV(bDpsi,bCpsi),x_a_a=TMatr.mulVV(bDpsi,bDpsi),x_b_b=TMatr.mulVV(bCpsi,bCpsi);
    return new DynVarCDV(basis[i0],basis[1],basis[n-1],p_IX,I_IX,x_a_b/x_a_a+p_IX,new ACorr(x_a_b,x_a_a,x_b_b));
}
static DynVarCDV test_get0D_DynVarCDVCV(final int n,
					final double [] QQi,final double [] QQt,final double [] QQpi,
					final boolean flag_ID,
					final VolMatrix VOLATILITY_EDPsi,final double [] QQV,final double [] QQpV,
					final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new IllegalArgumentException("n="+n+"<2");
    final int i0=0,iX=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M),
	Dpsi=VOLATILITY_EDPsi.getDPsi(EVXData.getPsiFromBasis(iX,basis,n));
    final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;

    final double [] bDpsi=new double [n],bCpsi=new double [n];
    for(int k=0;k<n;k++)
    {
	if(k==iX) continue;
	bCpsi[k]=TMatr.getAMatrB(n,basis,n+iX*n,basis,n+k*n,QQpV,n)-p_IX*TMatr.getAMatrB(n,basis,n+iX*n,basis,n+k*n,QQV,n);
	bDpsi[k]=TMatr.getAMatrB(n,Dpsi,0,basis,n+k*n,flag_ID?QQi:QQt,n);
    }
    //System.err.println("tD0:bDpsi="+java.util.Arrays.toString(bDpsi)+" flag_ID="+flag_ID+" psi="+java.util.Arrays.toString(psi)+" Dpsi="+java.util.Arrays.toString(Dpsi));
    final double x_a_b=TMatr.mulVV(bDpsi,bCpsi),x_a_a=TMatr.mulVV(bDpsi,bDpsi),x_b_b=TMatr.mulVV(bCpsi,bCpsi);
    return new DynVarCDV(basis[i0],basis[1],basis[n-1],p_IX,I_IX,x_a_b/x_a_a+p_IX,new ACorr(x_a_b,x_a_a,x_b_b));
}
static DynVarCDV get_DynVarCDVCV(final int n,
				 final double [] QQi,final double [] QQt,final double [] QQpi,
				 final double [] QQD,final double [] QQV,final double [] QQpV,
				 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double pI_IX=I.getFPsi(iX,QQpi),I_IX=I.ev[iX],p_IX=pI_IX/I_IX;
    final double [] bDpsi=new double [n],bCpsi=new double [n];
    for(int k=0;k<n;k++)
    {
	bDpsi[k]=I.getFPsiPsi(k,iX,QQD);
	bCpsi[k]=I.getFPsiPsi(iX,k,QQpV)-p_IX*I.getFPsiPsi(iX,k,QQV);
    }
    bDpsi[iX]=bCpsi[iX]=0;
    final double x_a_b=TMatr.mulVV(bDpsi,bCpsi),x_a_a=TMatr.mulVV(bDpsi,bDpsi),x_b_b=TMatr.mulVV(bCpsi,bCpsi);
    return new DynVarCDV(I.s0,I.sL,I.sH,p_IX,I_IX,x_a_b/x_a_a+p_IX,new ACorr(x_a_b,x_a_a,x_b_b));
}


public static void matchDynVarCD(final DynVarCDV a,final DynVarCD b,final double eps)
{
    UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I.s0,"b.I.s0",eps);
    UnitTests.checkDoubleRel(a.I_IL,"a.I_IL",b.I.sL,"b.I.sL",eps);
    UnitTests.checkDoubleRel(a.I_IH,"a.I_IH",b.I.sH,"b.I.sHH",eps);
    UnitTests.checkDoubleRel(a.p_IX,"a.p_IX",b.p_IX,"b.I_IX",eps);
    UnitTests.checkDoubleRel(a.I_IX,"a.I_IX",b.I_IX,"b.I_IX",eps);
    ACorr.matchACorr(a.a,b.a,eps);
    System.err.println("DynVarCDV:matchDynVarC test OK eps="+eps);
}
public static void matchDynVarCDV(final DynVarCDV a,final DynVarCDV b,final double eps)
{
    UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I_0,"b.I_0",eps);
    UnitTests.checkDoubleRel(a.I_IL,"a.I_IL",b.I_IL,"b.I_IL",eps);
    UnitTests.checkDoubleRel(a.I_IH,"a.I_IH",b.I_IH,"b.I_IH",eps);
    UnitTests.checkDoubleRel(a.p_IX,"a.p_IX",b.p_IX,"b.I_IX",eps);
    UnitTests.checkDoubleRel(a.I_IX,"a.I_IX",b.I_IX,"b.I_IX",eps);
    UnitTests.checkDoubleRel(a.pm_IX,"a.pm_IX",b.pm_IX,"b.pm_IX",eps);
    ACorr.matchACorr(a.a,b.a,eps);
    System.err.println("DynVarCDV:matchDynVarCDV test OK eps="+eps);
}

}

