package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.RayleighQuotient;

/**
 *  @version $Id: DynVarFiniteD.java,v 1.17 2022/07/28 12:27:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarFiniteD {
    final double I_0,I_IL,I_IH,p_IX,I_IX,pd_cL,Id_cL,pd_cH,Id_cH,distance2_cL_cH,distance2_IH_cH,distance2_IH_cL,pix_cL,pix_cH,pnl_cL,pnl_cH;
    DynVarFiniteD(final double I_0,final double I_IL,final double I_IH,final double p_IX,final double I_IX,
		  final double pd_cL,final double Id_cL,final double pd_cH,final double Id_cH,
		  final double pix_cL,final double pix_cH,
		  final double pnl_cL,final double pnl_cH,
		  final double distance2_cL_cH,final double distance2_IH_cH,final double distance2_IH_cL)
{
    this.I_0=I_0;
    this.I_IL=I_IL;
    this.I_IH=I_IH;
    this.p_IX=p_IX;
    this.I_IX=I_IX;
    this.pd_cL=pd_cL;
    this.Id_cL=Id_cL;
    this.pd_cH=pd_cH;
    this.Id_cH=Id_cH;
    this.pix_cL=pix_cL;
    this.pix_cH=pix_cH;
    this.pnl_cL=pnl_cL;
    this.pnl_cH=pnl_cH;
    this.distance2_cL_cH=distance2_cL_cH;
    this.distance2_IH_cH=distance2_IH_cH;
    this.distance2_IH_cL=distance2_IH_cL;
}

static final int FLAG_SEL_MINMAX=201,
	FLAG_SEL_CLOSE0=202,
	FLAG_SEL_CLOSEPSIH=203,
	FLAG_SEL_CLOSE1PSIH=204,
	FLAG_SEL_CLOSEWPSIH=205;


/**
 *
 */
static DynVarFiniteD get_DynVarFiniteD(final int n,
				       final double [] QQi,final double [] QQt,final double [] QQpi,
				       final double [] QQV,final double [] QQpV,
				       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				       final K2Distance distance2,
				       final boolean flag_0,
				       final boolean flag_V,
				       final int flag_Sel)

{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final double I_0,I_IL,I_IH,I_IX;final double [] basis,psi_IX;
    final int iX;
    if(flag_0)
    {
	iX=n-1;
	basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
	psi_IX=EVXData.getPsiFromBasis(iX,basis,n);
	I_0=basis[0];I_IL=basis[1];I_IH=basis[n-1];I_IX=basis[iX];
    }
    else
    {
	iX=n-1;
	final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	basis=I.ev;psi_IX=I.psiH;
	I_0=I.s0;I_IL=I.sL;I_IX=I_IH=I.sH;
    }

    final double pI_IX=TMatr.getAMatrA(psi_IX,QQpi,n),p_IX=pI_IX/I_IX,V_IX=TMatr.getAMatrA(psi_IX,QQV,n);
    final double [] QQpnl=TMatr.sumAcB(QQpV,-p_IX,QQV),bQQev;
    TMatr.sumP1(QQpnl,QQt,-TMatr.getAMatrA(psi_IX,QQpnl,n));
    final double [] bQQpix=EVXData.convertMatrFromQtoB_Basis(n,TMatr.sumAcB(QQpi,-p_IX,QQi),basis,n,n+0),
	bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpnl,basis,n,n+0);

    // P&L min/max
    final double [] pbasis=M.EV.getEV(n,flag_V?bQQpnl:bQQpix,true);
    final int icL,icH;
    if(flag_Sel==FLAG_SEL_CLOSE0){
	int ind=-1;
	for(int k=1;k<n;k++) if(pbasis[k-1]<=0 && pbasis[k]>=0) {ind=k;break;}
	if(ind>0) {icL=ind-1;icH=ind;}else{icL=0;icH=n-1;}
    }else if(flag_Sel==FLAG_SEL_MINMAX){
	icL=0;icH=n-1;
    }else if(flag_Sel==FLAG_SEL_CLOSEPSIH) {
	int indcL=-1,indcH=-1;
	double wcL=0,wcH=0;
	for(int k=0;k<n;k++){
	    final double w=Math.abs(pbasis[n+k*n+iX]);
	    if(pbasis[k]<=0) {if(w>=wcL){wcL=w;indcL=k;}}
	    else {if(w>=wcH){wcH=w;indcH=k;}}
	}
	if(indcL<0||indcH<0){icL=0;icH=n-1;}
	else{icL=indcL;icH=indcH;}
    } else if(flag_Sel==FLAG_SEL_CLOSE1PSIH){
	int ind=-1;
	double wc=0;
	for(int k=0;k<n;k++){
	    final double w=Math.abs(pbasis[n+k*n+iX]);
	    if(w>=wc){wc=w;ind=k;}
	}
	if(ind<0){icL=0;icH=n-1;}
	else{
	    if(pbasis[ind]<=0) {icL=ind;icH=-1;}
	    else {icL=-1;icH=ind;}
	}
    } else if(flag_Sel==FLAG_SEL_CLOSEWPSIH) {
	int ind=-1;double wc=0;
	for(int k=0;k<n;k++){
	    final double w=Math.abs(pbasis[n+k*n+iX]);
	    if(ind<0||w>=wc){wc=w;ind=k;}
	}
	icL=icH=ind;
    } else throw new RuntimeException("flag_Sel="+flag_Sel);


    final double [] psi_cL=icL<0?psi_IX:EVXData.convertPsiFromBtoQ_Basis(n,EVXData.getPsiFromBasis(icL,pbasis,n),basis),
	psi_cH=icH<0?psi_IX:EVXData.convertPsiFromBtoQ_Basis(n,EVXData.getPsiFromBasis(icH,pbasis,n),basis);
    final double pd_cL=icL<0?0:pbasis[icL],Id_cL=V_IX-TMatr.getAMatrA(psi_cL,QQV,n),
	pd_cH=icH<0?0:pbasis[icH],Id_cH=V_IX-TMatr.getAMatrA(psi_cH,QQV,n);

    if(false)
    {
	System.err.println("P&L[n-1,n-1]="+bQQpnl[(n-1)*n+(n-1)]+"=0");
    }
    return new DynVarFiniteD(I_0,I_IL,I_IH,p_IX,I_IX,pd_cL,Id_cL,pd_cH,Id_cH,
			     icL<0?Double.NaN:(flag_V?TMatr.getAMatrA(n,pbasis,n+icL*n,bQQpix,n):pbasis[icL]),
			     icH<0?Double.NaN:(flag_V?TMatr.getAMatrA(n,pbasis,n+icH*n,bQQpix,n):pbasis[icH]),
			     icL<0?Double.NaN:(flag_V?pbasis[icL]:TMatr.getAMatrA(n,pbasis,n+icL*n,bQQpnl,n)),
			     icH<0?Double.NaN:(flag_V?pbasis[icH]:TMatr.getAMatrA(n,pbasis,n+icH*n,bQQpnl,n)),
			     distance2.getDistance2(psi_cL,psi_cH),
			     distance2.getDistance2(psi_IX,psi_cH),
			     distance2.getDistance2(psi_IX,psi_cL));
}

static DynVarFiniteD test_get_DynVarFiniteD(final int n,
					    final double [] QQi,final double [] QQt,final double [] QQpi,
					    final double [] QQV,final double [] QQpV,
					    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
					    final K2Distance distance2,
					    final boolean flag_0,
					    final boolean flag_V,
					    final int flag_Sel)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final double I_0,I_IL,I_IH,I_IX;final double [] basis,psi_IX;
    if(flag_0)
    {
	final int iX=n-1;
	basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
	psi_IX=EVXData.getPsiFromBasis(iX,basis,n);
	I_0=basis[0];I_IL=basis[1];I_IH=basis[n-1];I_IX=basis[iX];
    }
    else
    {
	final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	basis=I.ev;psi_IX=I.psiH;
	I_0=I.s0;I_IL=I.sL;I_IX=I_IH=I.sH;
    }

    final double pI_IX=TMatr.getAMatrA(psi_IX,QQpi,n),p_IX=pI_IX/I_IX,V_IX=TMatr.getAMatrA(psi_IX,QQV,n);
    final double [] QQpix=TMatr.sumAcB(QQpi,-p_IX,QQi),
	QQpnl=TMatr.sumAcB(QQpV,-p_IX,QQV);
    TMatr.sumP1(QQpnl,QQt,-TMatr.getAMatrA(psi_IX,QQpnl,n));

    final EVXData PnL=EVXData.getEVXDataFromQQ(n,flag_V?QQpnl:QQpix,QQt,M);
    // P&L min/max
    final int icL,icH;
    if(flag_Sel==FLAG_SEL_CLOSE0){
	int ind=-1;
	for(int k=1;k<n;k++) if(PnL.ev[k-1]<=0 && PnL.ev[k]>=0) {ind=k;break;}
	if(ind>0) {icL=ind-1;icH=ind;}else{icL=0;icH=n-1;}
    }else if(flag_Sel==FLAG_SEL_MINMAX){
	icL=0;icH=n-1;
    } else if(flag_Sel==FLAG_SEL_CLOSEPSIH) {
	int indcL=-1,indcH=-1;
	double wcL=0,wcH=0;
	for(int k=0;k<n;k++){
	    final double w=Math.abs(PnL.getFPsiPsi(k,psi_IX,QQt));
	    if(PnL.ev[k]<=0) {if(w>=wcL){wcL=w;indcL=k;}}
	    else {if(w>=wcH){wcH=w;indcH=k;}}
	}
	if(indcL<0||indcH<0){icL=0;icH=n-1;}
	else{icL=indcL;icH=indcH;}
    } else if(flag_Sel==FLAG_SEL_CLOSE1PSIH){
	int ind=-1;
	double wc=0;
	for(int k=0;k<n;k++){
	    final double w=Math.abs(PnL.getFPsiPsi(k,psi_IX,QQt));
	    if(w>=wc){wc=w;ind=k;}
	}
	if(ind<0){icL=0;icH=n-1;}
	else{
	    if(PnL.ev[ind]<=0) {icL=ind;icH=-1;}
	    else {icL=-1;icH=ind;}
	}
    } else if(flag_Sel==FLAG_SEL_CLOSEWPSIH) {
	int ind=-1;double wc=0;
	for(int k=0;k<n;k++){
	    final double w=Math.abs(PnL.getFPsiPsi(k,psi_IX,QQt));
	    if(ind<0||w>=wc){wc=w;ind=k;}
	}
	icL=icH=ind;
    } else throw new RuntimeException("flag_Sel="+flag_Sel);

	final double [] psi_cL=icL<0?psi_IX:PnL.getPsi(icL),psi_cH=icH<0?psi_IX:PnL.getPsi(icH);
    final double pd_cL=icL<0?0:PnL.ev[icL],Id_cL=V_IX-TMatr.getAMatrA(psi_cL,QQV,n),
	pd_cH=icH<0?0:PnL.ev[icH],Id_cH=V_IX-TMatr.getAMatrA(psi_cH,QQV,n);
    if(false)
    {
	System.err.println("test:P&L[n-1,n-1]="+TMatr.getAMatrA(psi_IX,QQpnl,n)+"=0");
    }
    return new DynVarFiniteD(I_0,I_IL,I_IH,p_IX,I_IX,pd_cL,Id_cL,pd_cH,Id_cH,
			     icL<0?Double.NaN:PnL.getFPsi(icL,QQpix),
			     icH<0?Double.NaN:PnL.getFPsi(icH,QQpix),
			     icL<0?Double.NaN:PnL.getFPsi(icL,QQpnl),
			     icH<0?Double.NaN:PnL.getFPsi(icH,QQpnl),
			     distance2.getDistance2(psi_cL,psi_cH),
			     distance2.getDistance2(psi_IX,psi_cH),
			     distance2.getDistance2(psi_IX,psi_cL));
}

static void matchDynVarFiniteD(final DynVarFiniteD a,final DynVarFiniteD b,final double eps)
{
    UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I_0,"b.I_0",eps);
    UnitTests.checkDoubleRel(a.I_IL,"a.I_IL",b.I_IL,"b.I_IL",eps);
    UnitTests.checkDoubleRel(a.I_IH,"a.I_IH",b.I_IH,"b.I_IH",eps);
    UnitTests.checkDoubleRel(a.I_IX,"a.I_IX",b.I_IX,"b.I_IX",eps);
    UnitTests.checkDoubleRel(a.p_IX,"a.p_IX",b.p_IX,"b.p_IX",eps);
    UnitTests.checkDoubleRel(a.Id_cL,"a.Id_cL",b.Id_cL,"b.Id_cL",eps);
    UnitTests.checkDoubleRel(a.pd_cL,"a.pd_cL",b.pd_cL,"b.pd_cL",eps);
    UnitTests.checkDoubleRel(a.Id_cH,"a.Id_cH",b.Id_cH,"b.Id_cH",eps);
    UnitTests.checkDoubleRel(a.pd_cH,"a.pd_cH",b.pd_cH,"b.pd_cH",eps);
    UnitTests.checkDoubleRel(a.pix_cL,"a.pix_cL",b.pix_cL,"b.pix_cL",eps);
    UnitTests.checkDoubleRel(a.pix_cH,"a.pix_cH",b.pix_cH,"b.pix_cH",eps);
    UnitTests.checkDoubleRel(a.pnl_cL,"a.pnl_cL",b.pnl_cL,"b.pnl_cL",eps);
    UnitTests.checkDoubleRel(a.pnl_cH,"a.pnl_cH",b.pnl_cH,"b.pnl_cH",eps);
    UnitTests.checkDouble(a.distance2_cL_cH,"a.distance2_cL_cH",b.distance2_cL_cH,"b.distance2_cL_cH",eps);
    UnitTests.checkDouble(a.distance2_IH_cH,"a.distance2_IH_cH",b.distance2_IH_cH,"b.distance2_IH_cH",eps);
    UnitTests.checkDouble(a.distance2_IH_cL,"a.distance2_IH_cL",b.distance2_IH_cL,"b.distance2_IH_cL",eps);
    System.err.println("DynVarFiniteD:matchDynVarFiniteD test OK eps="+eps);
}
static void matchDynVarFiniteC(final DynVarFiniteD a,final DynVarFiniteC b,final boolean flag_match_Id,final double eps)
{
    UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I_0,"b.I_0",eps);
    UnitTests.checkDoubleRel(a.I_IL,"a.I_IL",b.I_IL,"b.I_IL",eps);
    UnitTests.checkDoubleRel(a.I_IH,"a.I_IH",b.I_IH,"b.I_IH",eps);
    UnitTests.checkDoubleRel(a.I_IX,"a.I_IX",b.I_IX,"b.I_IX",eps);
    UnitTests.checkDoubleRel(a.p_IX,"a.p_IX",b.p_IX,"b.p_IX",eps);
    if(flag_match_Id)
    {
	UnitTests.checkDoubleRel(a.Id_cL,"a.Id_cL",b.Id_cL,"b.Id_cL",eps);
	UnitTests.checkDoubleRel(a.Id_cH,"a.Id_cH",b.Id_cH,"b.Id_cH",eps);
    }
    UnitTests.checkDoubleRel(a.pd_cL,"a.pd_cL",b.pd_cL,"b.pd_cL",eps);
    UnitTests.checkDoubleRel(a.pd_cH,"a.pd_cH",b.pd_cH,"b.pd_cH",eps);
    UnitTests.checkDouble(a.distance2_cL_cH,"a.distance2_cL_cH",b.distance2_cL_cH,"b.distance2_cL_cH",eps);
    UnitTests.checkDouble(a.distance2_IH_cH,"a.distance2_IH_cH",b.distance2_IH_cH,"b.distance2_IH_cH",eps);
    UnitTests.checkDouble(a.distance2_IH_cL,"a.distance2_IH_cL",b.distance2_IH_cL,"b.distance2_IH_cL",eps);
    System.err.println("DynVarFiniteD:matchDynVarFiniteC test OK eps="+eps);
}
}

