package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 *  Try finite variation again. Use two levels with max I.
 *  @version $Id: DynVarFiniteD1.java,v 1.8 2020/09/27 20:33:12 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarFiniteD1 {
    final EVXData I;
    final double p_IX,I_IX,p_IX2,I_IX2,dist2,pB,pF,dist2m,pBm,pFm,p1m,p2m;
    DynVarFiniteD1(final EVXData I,final double p_IX,final double I_IX,final double p_IX2,final double I_IX2,final double dist2,final double pB,final double pF,final double dist2m,final double pBm,final double pFm,final double p1m,final double p2m)
{
    this.I=I;
    this.p_IX=p_IX;
    this.I_IX=I_IX;
    this.p_IX2=p_IX2;
    this.I_IX2=I_IX2;
    this.dist2=dist2;
    this.pB=pB;
    this.pF=pF;
    this.dist2m=dist2m;
    this.pBm=pBm;
    this.pFm=pFm;
    this.p1m=p1m;
    this.p2m=p2m;
}


/**
 *
 */
static DynVarFiniteD1 get_DynVarFiniteD1(
    final int n,
    final double [] QQi,final double [] QQt,final double [] QQpi,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final K2Distance distance2)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int iX=n-1,iX2=iX-1;
    final double I_IX=I.ev[iX],pI_IX=I.getFPsi(iX,QQpi),p_IX=pI_IX/I_IX,
	I_IX2=I.ev[iX2],pI_IX2=I.getFPsi(iX2,QQpi),p_IX2=pI_IX2/I_IX2;
    final double [] psi_IX=I.getPsi(iX),psi_IX2=I.getPsi(iX2);
    final double dist2=distance2.getDistance2(psi_IX,psi_IX2),pB,pF;
    if(dist2<0){pB=p_IX;pF=p_IX2;}
    else{pB=p_IX2;pF=p_IX;}
    int i1=-1,i2=-1;double mpI=Double.NaN;
    for(int k=0;k<I.neq;k++)for(int l=0;l<k;l++)
    {
	final double m=Math.abs(I.getFPsiPsi(k,l,QQpi));
	if(!(m<mpI)){mpI=m;i1=k;i2=l;}
    }
    final double pBm,pFm,dist2m,p1m,p2m;
    if(i1>=0)
    {
	dist2m=distance2.getDistance2(I.getPsi(i1),I.getPsi(i2));
	p1m=I.getFPsi(i1,QQpi)/I.ev[i1];
	p2m=I.getFPsi(i2,QQpi)/I.ev[i2];
	if(dist2m<0) {pBm=p1m;pFm=p2m;}
	else {pBm=p2m;pFm=p1m;}
    }
    else pBm=pFm=dist2m=p1m=p2m=Double.NaN;
    return new DynVarFiniteD1(I,p_IX,I_IX,p_IX2,I_IX2,dist2,pB,pF,dist2m,pBm,pFm,p1m,p2m);
}

}

