package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynVarFiniteE.java,v 1.38 2022/07/28 12:27:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarFiniteE {
final double I_0,I_IL,I_IH,p_IX,I_IX,p_IE,I_IE,w_IX_IE,pix_IX_IE,pnl_IX_IE,pix_0,pnl_0,distance2_IX_IE;
DynVarFiniteE(final double I_0,final double I_IL,final double I_IH,
	      final double p_IX,final double I_IX,
	      final double p_IE,final double I_IE,
	      final double w_IX_IE,final double pix_IX_IE,final double pnl_IX_IE,
	      final double pix_0,final double pnl_0,
	      final double distance2_IX_IE)
{
    this.I_0=I_0;
    this.I_IL=I_IL;
    this.I_IH=I_IH;
    this.p_IX=p_IX;
    this.I_IX=I_IX;
    this.p_IE=p_IE;
    this.I_IE=I_IE;
    this.w_IX_IE=w_IX_IE;
    this.pix_IX_IE=pix_IX_IE;
    this.pnl_IX_IE=pnl_IX_IE;
    this.pix_0=pix_0;
    this.pnl_0=pnl_0;
    this.distance2_IX_IE=distance2_IX_IE;
}


/**
 * For psiH: I|psiH>=lambda_max |psiH>, I_IH=lambda_max,
 * then calculate p_IH=<psiH|pI|psiH>/I_IH.
 * Introduce P&L=<psi|(C-p_IH*V - P&L_0 )|psi>/<psi|psi> operator.
 * Depending on flagEV_pI find eigenvector of either |pI-p_IH*I|psi>=lam |psi>
 * P&L|psi>=lam |psi>, providing maximal value of |<psi|psiH>|.
 * The pix_IX_IE =<psi|pI - p_IH*I|psi> and pnl_IX_IE=<psi|P&L|psi>.
 * This approach assumes (for flagEV_pI=true) that if I and pI operators are diagonal
 * in the same basis (an example: discrete measure with the number of support
 * points equal to n-1) then the answer will be 0, (bacause psi=psiH).
 * The selection criteria also assumes that only points in the vicinity od |psiH> are important.
 */
static DynVarFiniteE get_PnLX_DynVarFiniteE(final int n,
					    final double [] QQi,final double [] QQt,final double [] QQpi,
					    final double [] QQV,final double [] QQpV,
					    final boolean flagEV_pI,
					    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
					    final K2Distance distance2)

{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double pI_IX=I.getFPsi(iX,QQpi),I_IX=I.ev[iX],p_IX=pI_IX/I_IX;
    final double [] QQpnl=TMatr.sumAcB(QQpV,-p_IX,QQV),
	bpsi0=EVXData.convertPsiFromQtoB_Basis(n,I.psi0,I.ev,QQt);
    TMatr.sumP1(QQpnl,QQt,-I.getFPsi(iX,QQpnl));
    final double [] bQQpix=EVXData.convertMatrFromQtoB_Basis(n,TMatr.sumAcB(QQpi,-p_IX,QQi),I.ev,n,n+0),
	bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpnl,I.ev,n,n+0);

    // P&L operator diagonalization.
    final double [] pbasis=M.EV.getEV(n,flagEV_pI?bQQpix:bQQpnl,true);
    int ind_pnl=-1;
    for(int k=0;k<n;k++)
    {
	// Find max |<psi|psi_IX>|.
	if(ind_pnl<0||Math.abs(pbasis[n+k*n+iX])>=Math.abs(pbasis[n+ind_pnl*n+iX])) ind_pnl=k;
    }
    final double pix_IX_IE=flagEV_pI?pbasis[ind_pnl]:TMatr.getAMatrA(n,pbasis,n+ind_pnl*n,bQQpix,n),
	pnl_IX_IE=flagEV_pI?TMatr.getAMatrA(n,pbasis,n+ind_pnl*n,bQQpnl,n):pbasis[ind_pnl],
	w_IX_IE=pbasis[n+ind_pnl*n+iX],
	I_IE=TMatr.mulV2V(EVXData.getPsiFromBasis(ind_pnl,pbasis,n),I.getEValues()),p_IE=pix_IX_IE/I_IE+p_IX;
    return new DynVarFiniteE(I.s0,I.sL,I.sH,p_IX,I_IX,p_IE,I_IE,w_IX_IE,pix_IX_IE,pnl_IX_IE,
			     TMatr.getAMatrA(bpsi0,bQQpix,n),TMatr.getAMatrA(bpsi0,bQQpnl,n),
			     distance2.getDistance2(I.getPsi(iX),EVXData.convertPsiFromBtoQ_Basis(n,EVXData.getPsiFromBasis(ind_pnl,pbasis,n),I.ev)));
}


static DynVarFiniteE test_get_PnLX_DynVarFiniteE(final int n,
						 final double [] QQi,final double [] QQt,final double [] QQpi,
						 final double [] QQV,final double [] QQpV,
						 final boolean flagEV_pI,
						 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
						 final K2Distance distance2)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] psi_IX=I.getPsi(iX);
    final double pI_IX=I.getFPsi(iX,QQpi),I_IX=I.ev[iX],p_IX=pI_IX/I_IX;
    final double [] QQpix=TMatr.sumAcB(QQpi,-p_IX,QQi),QQpnl=TMatr.sumAcB(QQpV,-p_IX,QQV);
    TMatr.sumP1(QQpnl,QQt,-TMatr.getAMatrA(psi_IX,QQpnl,n));
    final EVXData ev=EVXData.getEVXDataFromQQ(n,flagEV_pI?QQpix:QQpnl,QQt,M);
    int ind_pnl=-1;double swpp=0;
    for(int k=0;k<n;k++)
    {
	final double wpp=Math.abs(ev.getFPsiPsi(k,psi_IX,QQt));
	if(ind_pnl<0||wpp>=swpp) {ind_pnl=k;swpp=wpp;}
    }
    final double pix_IX_IE=ev.getFPsi(ind_pnl,QQpix),pnl_IX_IE=ev.getFPsi(ind_pnl,QQpnl),
	w_IX_IE=swpp,I_IE=ev.getFPsi(ind_pnl,QQi),p_IE=ev.getFPsi(ind_pnl,QQpi)/I_IE;
    return new DynVarFiniteE(I.s0,I.sL,I.sH,p_IX,I_IX,p_IE,I_IE,w_IX_IE,pix_IX_IE,pnl_IX_IE,
			     TMatr.getAMatrA(I.psi0,QQpix,n),TMatr.getAMatrA(I.psi0,QQpnl,n),
			     distance2.getDistance2(psi_IX,ev.getPsi(ind_pnl)));
}

static DynVarFiniteE getR_PnLX_DynVarFiniteE(final int n,
					     final double [] QQi,final double [] QQt,final double [] QQpi,
					     final double [] QQV,final double [] QQpV,
					     final boolean flagEV_pI,
					     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
					     final K2Distance distance2)

{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int nd=n-1,iX=nd-1;
    final double [] basis=M.EV.getGEV_Generalized(nd,VolMatrix.getRadauMatrix(n,QQi,M),VolMatrix.getRadauMatrix(n,QQt,M),true),
	psi_IX=EVXData.getPsiFromBasis(iX,basis,nd),Rpsi_IX=M.getRadauMultiplied(psi_IX);
    final double pI_IX=TMatr.getAMatrB(psi_IX,Rpsi_IX,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;
    final double [] QQpix=TMatr.sumAcB(QQpi,-p_IX,QQi),QQpnl=TMatr.sumAcB(QQpV,-p_IX,QQV);
    TMatr.sumP1(QQpnl,QQt,-TMatr.getAMatrB(psi_IX,Rpsi_IX,QQpnl,n));
    final double [] bRQQpix=EVXData.convertMatrFromQtoB_Basis(nd,VolMatrix.getRadauMatrix(n,QQpix,M),basis,nd,nd+0),
	bRQQpnl=EVXData.convertMatrFromQtoB_Basis(nd,VolMatrix.getRadauMatrix(n,QQpnl,M),basis,nd,nd+0);
    final double [] pbasis=M.EV.getEV(nd,flagEV_pI?bRQQpix:bRQQpnl,true);
    int ind_pnl=-1;
    for(int k=0;k<nd;k++)
    {
	// Find max |<psi|psi_IX>|.
	if(ind_pnl<0||Math.abs(pbasis[nd+k*nd+iX])>=Math.abs(pbasis[nd+ind_pnl*nd+iX])) ind_pnl=k;
    }
    final double [] psi_Shf=EVXData.convertPsiFromBtoQ_Basis(nd,EVXData.getPsiFromBasis(ind_pnl,pbasis,nd),basis),
	Rpsi_Shf=M.getRadauMultiplied(psi_Shf);
    final double pix_IX_IE=flagEV_pI?pbasis[ind_pnl]:TMatr.getAMatrA(nd,pbasis,nd+ind_pnl*nd,bRQQpix,nd),
	pnl_IX_IE=flagEV_pI?TMatr.getAMatrA(nd,pbasis,nd+ind_pnl*nd,bRQQpnl,nd):pbasis[ind_pnl],
	w_IX_IE=pbasis[nd+ind_pnl*nd+iX],
	I_IE=TMatr.mulV2V(EVXData.getPsiFromBasis(ind_pnl,pbasis,nd),java.util.Arrays.copyOf(basis,nd)),p_IE=TMatr.getAMatrB(psi_Shf,Rpsi_Shf,QQpi,n)/I_IE;
    return new DynVarFiniteE(Double.NaN,basis[0],basis[nd-1],p_IX,I_IX,p_IE,I_IE,w_IX_IE,pix_IX_IE,pnl_IX_IE,
			     Double.NaN,Double.NaN,
			     distance2.getK2(psi_IX,Rpsi_IX,psi_Shf,Rpsi_Shf)-distance2.getK2(psi_Shf,Rpsi_Shf,psi_IX,Rpsi_IX));
}

static DynVarFiniteE test_getR_PnLX_DynVarFiniteE(final int n,
						  final double [] QQi,final double [] QQt,final double [] QQpi,
						  final double [] QQV,final double [] QQpV,
						  final boolean flagEV_pI,
						  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
						  final K2Distance distance2)

{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int nd=n-1,iX=nd-1;
    final double [] RQQt=VolMatrix.getRadauMatrix(n,QQt,M),RQQi=VolMatrix.getRadauMatrix(n,QQi,M),
	RQQpi=VolMatrix.getRadauMatrix(n,QQpi,M);
    final EVXData I=EVXData.getEVXDataFromQQ(nd,RQQi,RQQt,M);
    final double pI_IX=I.getFPsi(iX,RQQpi),I_IX=I.ev[iX],p_IX=pI_IX/I_IX;
    final double [] psi_IX=I.getPsi(iX),Rpsi_IX=M.getRadauMultiplied(psi_IX),
	QQpnl=TMatr.sumAcB(QQpV,-p_IX,QQV);
    TMatr.sumP1(QQpnl,QQt,-TMatr.getAMatrB(psi_IX,Rpsi_IX,QQpnl,n));
    final double [] RQQpix=TMatr.sumAcB(RQQpi,-p_IX,RQQi),
	RQQpnl=VolMatrix.getRadauMatrix(n,QQpnl,M);
    final EVXData ev=EVXData.getEVXDataFromQQ(nd,flagEV_pI?RQQpix:RQQpnl,RQQt,M);
    int ind_pnl=-1;double swpp=0;
    for(int k=0;k<nd;k++)
    {
	final double wpp=Math.abs(ev.getFPsiPsi(k,psi_IX,RQQt));
	if(ind_pnl<0||wpp>=swpp) {ind_pnl=k;swpp=wpp;}
    }
    final double pix_IX_IE=ev.getFPsi(ind_pnl,RQQpix),pnl_IX_IE=ev.getFPsi(ind_pnl,RQQpnl),
	w_IX_IE=swpp,I_IE=ev.getFPsi(ind_pnl,RQQi),p_IE=ev.getFPsi(ind_pnl,RQQpi)/I_IE;
    final double [] psi_Shf=ev.getPsi(ind_pnl),Rpsi_Shf=M.getRadauMultiplied(psi_Shf);
    return new DynVarFiniteE(Double.NaN,I.sL,I.sH,p_IX,I_IX,p_IE,I_IE,w_IX_IE,pix_IX_IE,pnl_IX_IE,
			     Double.NaN,Double.NaN,
			     distance2.getK2(psi_IX,Rpsi_IX,psi_Shf,Rpsi_Shf)-distance2.getK2(psi_Shf,Rpsi_Shf,psi_IX,Rpsi_IX));
}

static DynVarFiniteE get0_PnLX_DynVarFiniteE(final int n,
					     final double [] QQi,final double [] QQt,final double [] QQpi,
					     final double [] QQV,final double [] QQpV,
					     final boolean flagEV_pI,
					     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
					     final VolMatrix psi_Selector_VOLATILITY_EDPsi,
					     final K2Distance distance2)

{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=n-1;
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M),psi_IX=EVXData.getPsiFromBasis(iX,basis,n),
	bpsi_Selectror=psi_Selector_VOLATILITY_EDPsi==null?null:
	EVXData.convertPsiFromQtoB_Basis(n,psi_Selector_VOLATILITY_EDPsi.getDPsi(psi_IX),basis,QQt);
    final double pI_IX=TMatr.getAMatrA(psi_IX,QQpi,n),I_IX=basis[iX],p_IX=pI_IX/I_IX;
    final double [] QQpnl=TMatr.sumAcB(QQpV,-p_IX,QQV),
	bpsi0=EVXData.convertPsiFromQtoB_Basis(n,EVXData.getPsiFromBasis(0,basis,n),basis,QQt);
    TMatr.sumP1(QQpnl,QQt,-TMatr.getAMatrA(psi_IX,QQpnl,n));
    final double [] bQQpix=EVXData.convertMatrFromQtoB_Basis(n,TMatr.sumAcB(QQpi,-p_IX,QQi),basis,n,n+0),
	bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,QQpnl,basis,n,n+0);
    final double [] pbasis=M.EV.getEV(n,flagEV_pI?bQQpix:bQQpnl,true);
    int ind_pnl=-1;double swpp=0;
    for(int k=0;k<n;k++)
    {
	final double wpp=bpsi_Selectror==null?
	    Math.abs(pbasis[n+k*n+iX]):
	    Math.abs( DeprecatedHelperTMatr.mulVVo(n,bpsi_Selectror,0,pbasis,n+k*n));
	// Find max |<psi|psi_Selector>|.
	if(ind_pnl<0||wpp>=swpp) {ind_pnl=k;swpp=wpp;}
    }
    final double [] psi_Shf=EVXData.convertPsiFromBtoQ_Basis(n,EVXData.getPsiFromBasis(ind_pnl,pbasis,n),basis);
    final double pix_IX_IE=flagEV_pI?pbasis[ind_pnl]:TMatr.getAMatrA(n,pbasis,n+ind_pnl*n,bQQpix,n),
	pnl_IX_IE=flagEV_pI?TMatr.getAMatrA(n,pbasis,n+ind_pnl*n,bQQpnl,n):pbasis[ind_pnl],w_IX_IE=swpp,
	I_IE=TMatr.getAMatrA(psi_Shf,QQi,n),p_IE=TMatr.getAMatrA(psi_Shf,QQpi,n)/I_IE;
    return new DynVarFiniteE(basis[0],basis[1],basis[n-1],p_IX,I_IX,p_IE,I_IE,w_IX_IE,pix_IX_IE,pnl_IX_IE,
			     TMatr.getAMatrA(bpsi0,bQQpix,n),TMatr.getAMatrA(bpsi0,bQQpnl,n),
			     distance2.getDistance2(psi_IX,psi_Shf));
}

static DynVarFiniteE test_get0_PnLX_DynVarFiniteE(final int n,
						  final double [] QQi,final double [] QQt,final double [] QQpi,
						  final double [] QQV,final double [] QQpV,
						  final boolean flagEV_pI,
						  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
						  final VolMatrix psi_Selector_VOLATILITY_EDPsi,
						  final K2Distance distance2)

{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int nd=n-1,iX=nd-1;
    final EVXData I=EVXData.getEVXDataFromQQ(nd,VolMatrix.getRadau2Matrix(n,QQi,M),VolMatrix.getRadau2Matrix(n,QQt,M),M);
    final double pI_IX=I.getFPsi(iX,VolMatrix.getRadau2Matrix(n,QQpi,M)),I_IX=I.ev[iX],p_IX=pI_IX/I_IX;
    final double [] psi_IX=M.getRadauMultiplied(I.getPsi(iX)),
	psi_Selectror=psi_Selector_VOLATILITY_EDPsi==null?psi_IX:psi_Selector_VOLATILITY_EDPsi.getDPsi(psi_IX),
	QQpix=TMatr.sumAcB(QQpi,-p_IX,QQi),QQpnl=TMatr.sumAcB(QQpV,-p_IX,QQV);
    TMatr.sumP1(QQpnl,QQt,-TMatr.getAMatrA(psi_IX,QQpnl,n));
    final EVXData ev=EVXData.getEVXDataFromQQ(n,flagEV_pI?QQpix:QQpnl,QQt,M);
    int ind_pnl=-1;double swpp=0;
    for(int k=0;k<n;k++)
    {
	final double wpp=Math.abs(ev.getFPsiPsi(k,psi_Selectror,QQt));
	if(ind_pnl<0||wpp>=swpp) {ind_pnl=k;swpp=wpp;}
    }
    final double pix_IX_IE=ev.getFPsi(ind_pnl,QQpix),pnl_IX_IE=ev.getFPsi(ind_pnl,QQpnl),
	w_IX_IE=swpp,I_IE=ev.getFPsi(ind_pnl,QQi),p_IE=ev.getFPsi(ind_pnl,QQpi)/I_IE;
    return new DynVarFiniteE(TMatr.getAMatrA(ev.psi0,QQi,n),I.sL,I.sH,
			     p_IX,I_IX,p_IE,I_IE,w_IX_IE,pix_IX_IE,pnl_IX_IE,
			     TMatr.getAMatrA(ev.psi0,QQpix,n),TMatr.getAMatrA(ev.psi0,QQpnl,n),
			     distance2.getDistance2(psi_IX,ev.getPsi(ind_pnl)));
}

static void matchDynVarFiniteE(final DynVarFiniteE a,final DynVarFiniteE b,final double eps)
{
    UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I_0,"b.I_0",eps);
    UnitTests.checkDoubleRel(a.I_IL,"a.I_IL",b.I_IL,"b.I_IL",eps);
    UnitTests.checkDoubleRel(a.I_IH,"a.I_IH",b.I_IH,"b.I_IH",eps);
    UnitTests.checkDoubleRel(a.I_IX,"a.I_IX",b.I_IX,"b.I_IX",eps);
    UnitTests.checkDoubleRel(a.p_IX,"a.p_IX",b.p_IX,"b.p_IX",eps);
    UnitTests.checkDoubleRel(a.I_IE,"a.I_IE",b.I_IE,"b.I_IE",eps);
    UnitTests.checkDoubleRel(a.p_IE,"a.p_IE",b.p_IE,"b.p_IE",eps);
    UnitTests.checkDoubleRel(a.pix_IX_IE,"a.pix_IX_IE",b.pix_IX_IE,"b.pix_IX_IE",eps);
    UnitTests.checkDoubleRel(a.pnl_IX_IE,"a.pnl_IX_IE",b.pnl_IX_IE,"b.pnl_IX_IE",eps);
    UnitTests.checkDoubleRel(a.pix_0,"a.pix_0",b.pix_0,"b.pix_0",eps);
    UnitTests.checkDoubleRel(a.pnl_0,"a.pnl_0",b.pnl_0,"b.pnl_0",eps);
    UnitTests.checkDoubleRel(Math.abs(a.w_IX_IE),"a.w_IX_IE",Math.abs(b.w_IX_IE),"b.w_IX_IE",eps);
    UnitTests.checkDouble(a.distance2_IX_IE,"a.distance2_IX_IE",b.distance2_IX_IE,"b.distance2_IX_IE",eps);

    System.err.println("DynVarFiniteE:matchDynVarFiniteE test OK eps="+eps);
}

static void matchDynVarFiniteD(final DynVarFiniteE a,final DynVarFiniteD b,final double eps)
{
    UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I_0,"b.I_0",eps);
    UnitTests.checkDoubleRel(a.I_IL,"a.I_IL",b.I_IL,"b.I_IL",eps);
    UnitTests.checkDoubleRel(a.I_IH,"a.I_IH",b.I_IH,"b.I_IH",eps);
    UnitTests.checkDoubleRel(a.I_IX,"a.I_IX",b.I_IX,"b.I_IX",eps);
    UnitTests.checkDoubleRel(a.p_IX,"a.p_IX",b.p_IX,"b.p_IX",eps);
    UnitTests.checkDoubleRel(a.pix_IX_IE,"a.pix_IX_IE",b.pix_cL,"b.pix_cL",eps);
    UnitTests.checkDoubleRel(a.pix_IX_IE,"a.pix_IX_IE",b.pix_cH,"b.pix_cH",eps);
    UnitTests.checkDoubleRel(a.pnl_IX_IE,"a.pnl_IX_IE",b.pnl_cL,"b.pnl_cL",eps);
    UnitTests.checkDoubleRel(a.pnl_IX_IE,"a.pnl_IX_IE",b.pnl_cH,"b.pnl_cH",eps);
    UnitTests.checkDouble(a.distance2_IX_IE,"a.distance2_IX_IE",b.distance2_IH_cL,"b.distance2_IH_cL",eps);
    UnitTests.checkDouble(a.distance2_IX_IE,"a.distance2_IX_IE",b.distance2_IH_cH,"b.distance2_IH_cH",eps);

    System.err.println("DynVarFiniteE:matchDynVarFiniteD test OK eps="+eps);
}

static void matchDynHPnL(final DynVarFiniteE a,final DynHPnL b,final double eps)
{
    UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I_0,"b.I_0",eps);
    UnitTests.checkDoubleRel(a.I_IL,"a.I_IL",b.I_L,"b.I_L",eps);
    UnitTests.checkDoubleRel(a.I_IH,"a.I_IH",b.I_H,"b.I_H",eps);
    UnitTests.checkDoubleRel(a.I_IX,"a.I_IX",b.I_H,"b.I_H",eps);
    UnitTests.checkDoubleRel(a.p_IX,"a.p_IX",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.pix_IX_IE,"a.pix_IX_IE",b.pix_IH_IE,"b.pix_IH_IE",eps);
    UnitTests.checkDoubleRel(a.pnl_IX_IE,"a.pnl_IX_IE",b.pnl_IH_IE,"b.pnl_IH_IE",eps);
    UnitTests.checkDoubleRel(Math.abs(a.w_IX_IE),"a.w_IX_IE",Math.abs(b.w_IH_IE),"b.w_IH_IE",eps);
    UnitTests.checkDouble(a.distance2_IX_IE,"a.distance2_IX_IE",b.distance2_IH_IE,"b.distance2_IH_IE",eps);

    System.err.println("DynVarFiniteE:matchDynHPnL test OK eps="+eps);
}


}

