package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynVarFiniteF.java,v 1.22 2022/07/29 19:35:39 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarFiniteF {
final EVXData J;
    final double pm_jL,pm_jH,pi_jL,pi_jH,pv_jL,pv_jH,var_pCm_jL,var_pCm_jH;
    DynVarFiniteF(final EVXData J,final double pm_jL,final double pm_jH,final double pi_jL,final double pi_jH,final double pv_jL,final double pv_jH,final double var_pCm_jL,final double var_pCm_jH)
{
    this.J=J;
    this.pm_jL=pm_jL;
    this.pm_jH=pm_jH;
    this.pi_jL=pi_jL;
    this.pi_jH=pi_jH;
    this.pv_jL=pv_jL;
    this.pv_jH=pv_jH;
    this.var_pCm_jL=var_pCm_jL;
    this.var_pCm_jH=var_pCm_jH;
}


/**
 * Try P&L matrix I*R-V
 */
static DynVarFiniteF get_PnLX_DynVarFiniteF(final int n,
					    final double [] QQi,final double [] QQt,final double [] QQpi,
					    final double [] QQRi,final double [] QQRpi,
					    final double [] QQV,final double [] QQpV,
					    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
					    final VolMatrix VOLATILITY_EDPsi)

{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final double [] QQj=TMatr.sumAcB(QQRi,-1,QQV),QQpj=TMatr.sumAcB(QQRpi,-1,QQpV);
    final EVXData J=EVXData.getEVXDataFromQQ(n,QQj,QQt,M);
    final int iL=0,iH=n-1;
    final double pm_jL=J.getFPsi(iL,QQpj)/J.sL,pm_jH=J.getFPsi(iH,QQpj)/J.sH,
	pi_jL=J.getFPsi(iL,QQpi)/J.getFPsi(iL,QQi),pi_jH=J.getFPsi(iH,QQpi)/J.getFPsi(iH,QQi),
	pv_jL=J.getFPsi(iL,QQpV)/J.getFPsi(iL,QQV),pv_jH=J.getFPsi(iH,QQpV)/J.getFPsi(iH,QQV);

    final double [] Dpsi_jL=VOLATILITY_EDPsi.getDPsi(J.psiL);
    final double var_pCm_jL=2*(J.getFPsiPsi(iL,Dpsi_jL,QQpj)-J.getFPsi(iL,QQpj)*J.getFPsiPsi(iL,Dpsi_jL,QQt));
    final double [] Dpsi_jH=VOLATILITY_EDPsi.getDPsi(J.psiH);
    final double var_pCm_jH=2*(J.getFPsiPsi(iH,Dpsi_jH,QQpj)-J.getFPsi(iH,QQpj)*J.getFPsiPsi(iH,Dpsi_jH,QQt));


    if(false)
    {
	final double eps=1e-5;
	System.err.println("J.sl="+J.sL+" J.sH="+J.sH);
	final double v0=QQi[0],Rv0=QQRi[0];
	final double [] ONE=M.B.getONE();
	final double [] R=M.getRadauMultiplied(ONE);
	final double vv0=TMatr.getAMatrB(R,ONE,QQV,n);
	System.err.println("v0="+v0+" Rv0="+Rv0+" vv0="+vv0);
	if(v0<0) throw new RuntimeException("v0="+v0);
	if(Rv0<0) throw new RuntimeException("Rv0="+Rv0);
	if(vv0<0) throw new RuntimeException("vv0="+vv0);
	//UnitTests.checkDoubleRel(v0,"v0",vv0,"vv0",eps);
    }
    return new DynVarFiniteF(J,pm_jL,pm_jH,pi_jL,pi_jH,pv_jL,pv_jH,var_pCm_jL,var_pCm_jH);
}

static DynVarFiniteF test_get_PnLX_DynVarFiniteF_frommoments(
    final int n,final double [] imom,final double [] tmom,final double [] pimom,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,
    final java.util.function.Function<double [],double []> convert_dFmomentsToFmomentsMatchRadau,
    final double eps)
{
    final double [] Rimom=M.getRadauMomentsFromMoments(imom),Rpimom=M.getRadauMomentsFromMoments(pimom),
	Vmom=convert_dFmomentsToFmomentsMatchRadau.apply(imom),
	pVmom=convert_dFmomentsToFmomentsMatchRadau.apply(pimom),
	QQt=M.B.getQQMatr(n,tmom),QQRi=M.B.getQQMatr(n,Rimom),QQV=M.B.getQQMatr(n,Vmom),
	QQi=M.B.getQQMatr(n,imom),QQj=TMatr.sumAcB(QQRi,-1,QQV),
	QQRpi=M.B.getQQMatr(n,Rpimom),QQpV=M.B.getQQMatr(n,pVmom),
	QQpi=M.B.getQQMatr(n,pimom),QQpj=TMatr.sumAcB(QQRpi,-1,QQpV);
    final EVXData J=EVXData.getEVXDataFromQQ(n,QQj,QQt,M);
    final int iL=0,iH=n-1;
    if(eps>0)
    {
	final double [] Trmom=convert_dFmomentsToFmomentsMatchRadau.apply(tmom),
	    Rtmom=M.getRadauMomentsFromMoments(tmom);
	System.err.println("Trmom="+java.util.Arrays.toString(Trmom));
	System.err.println("Rtmom="+java.util.Arrays.toString(Rtmom));
	UnitTests.checkDoubleArrayRel(Trmom,"Trmom",Rtmom,"Rtmom",eps);
	System.err.println("DynVarFiniteF:Trmom&Rtmom match");
	EVXData.test_orthogonality("DynVarFiniteF:J",J.neq,J.ev,QQj,QQt,eps);
    }
    return new DynVarFiniteF(J,J.getFPsi(iL,QQpj)/J.sL,J.getFPsi(iH,QQpj)/J.sH,
			     J.getFPsi(iL,QQpi)/J.getFPsi(iL,QQi),J.getFPsi(iH,QQpi)/J.getFPsi(iH,QQi),
			     J.getFPsi(iL,QQpV)/J.getFPsi(iL,QQV),J.getFPsi(iH,QQpV)/J.getFPsi(iH,QQV),
			     new DynVar1(J.psiL,QQpj,QQt,QQpj,VOLATILITY_EDPsi).var_I,
			     new DynVar1(J.psiH,QQpj,QQt,QQpj,VOLATILITY_EDPsi).var_I);
}


static void matchDynVarFiniteF(final DynVarFiniteF a,final DynVarFiniteF b,final double eps)
{
    EVXData.matchEVXData(a.J,b.J,false,false,eps);
    UnitTests.checkDoubleRel(a.pm_jL,"a.pm_jL",b.pm_jL,"b.pm_jL",eps);
    UnitTests.checkDoubleRel(a.pm_jH,"a.pm_jH",b.pm_jH,"b.pm_jH",eps);
    UnitTests.checkDoubleRel(a.pi_jL,"a.pi_jL",b.pi_jL,"b.pi_jL",eps);
    UnitTests.checkDoubleRel(a.pi_jH,"a.pi_jH",b.pi_jH,"b.pi_jH",eps);
    UnitTests.checkDoubleRel(a.pv_jL,"a.pv_jL",b.pv_jL,"b.pv_jL",eps);
    UnitTests.checkDoubleRel(a.pv_jH,"a.pv_jH",b.pv_jH,"b.pv_jH",eps);
    UnitTests.checkDoubleRel(a.var_pCm_jL,"a.var_pCm_jL",b.var_pCm_jL,"b.var_pCm_jL",eps);
    UnitTests.checkDoubleRel(a.var_pCm_jH,"a.var_pCm_jH",b.var_pCm_jH,"b.var_pCm_jH",eps);
    System.err.println("DynVarFiniteF:matchDynVarF test OK eps="+eps);
}
}

