package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynVarFiniteFV.java,v 1.19 2022/07/29 19:35:39 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarFiniteFV {
final double I_0,I_IL,I_IH,p_IH,V_IH;
final double J_IX,I_IX,p_IX,V_IX,tau_IX,pj_IX,Dpj_IX;
DynVarFiniteFV(final double I_0,final double I_IL,final double I_IH,final double p_IH,final double V_IH,
	       final double J_IX,final double I_IX,final double p_IX,final double V_IX,final double tau_IX,final double pj_IX,final double Dpj_IX)
{
    this.I_0=I_0;
    this.I_IL=I_IL;
    this.I_IH=I_IH;
    this.p_IH=p_IH;
    this.V_IH=V_IH;
    this.J_IX=J_IX;
    this.I_IX=I_IX;
    this.p_IX=p_IX;
    this.V_IX=V_IX;
    this.tau_IX=tau_IX;
    this.pj_IX=pj_IX;
    this.Dpj_IX=Dpj_IX;
}


static final int M_VTOTAL=781,M_MINUS_V_VIH=782,M_PLUS_V_VIH=783,M_REL_V_I=784,M_rI=785;
static DynVarFiniteFV test_get_PnLX_DynVarFiniteFV_frommoments(
    final int n,final double [] imom,final double [] tmom,final double [] pimom,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
    final java.util.function.Function<double [],double []> convert_dFmomentsToFmomentsMatchRadau,
    final int flag_j,
    final boolean flag_HIGH_I,
    final double eps,
    final VolMatrix VOLATILITY_EDPsi)
{
    final double [] Rtmom=M.getRadauMomentsFromMoments(tmom),
	QQt=M.B.getQQMatr(n,tmom),RQQt=M.B.getQQMatr(n,Rtmom),
	RQQi=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(imom)),
	QQV=M.B.getQQMatr(n,convert_dFmomentsToFmomentsMatchRadau.apply(imom)),
	RQQpi=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(pimom));

    final double [] ibasis=M.EV.getGEV_Generalized(n,RQQi,RQQt,true);
    final int iH=n-1;
    final double p_IH=TMatr.getAMatrA(n,ibasis,n+iH*n,RQQpi,n)/ibasis[iH],
	V_IH=ibasis[iH]/TMatr.getAMatrA(n,ibasis,n+iH*n,QQt,n);

    final double [] RQQj=new double [n*n];
    final double J_IX,I_IX,p_IX,ot,V_IX,pj_IX,Dpj_IX;
    if(flag_j==M_rI)
    {
	final double [] basis=M.EV.getGEV_Generalized(n,RQQi,QQt,true);
	final int iX=flag_HIGH_I?n-1:0;
	final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,RQQpi,n);
	J_IX=basis[iX];
	final double oot=TMatr.getAMatrA(n,basis,n+iX*n,RQQt,n);
	I_IX=TMatr.getAMatrA(n,basis,n+iX*n,RQQi,n)/oot;
	p_IX=pI_IX/basis[iX];
	ot=1/oot;
	V_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQV,n);
	pj_IX=Dpj_IX=Double.NaN;
    }
    else if(flag_j==M_REL_V_I)
    {
	final int iX=flag_HIGH_I?0:n-1;
	final double [] basis=M.EV.getGEV_Generalized(n,QQV,RQQi,true),
	    QQpV=M.B.getQQMatr(n,convert_dFmomentsToFmomentsMatchRadau.apply(pimom)),
	    psi=EVXData.getPsiFromBasis(iX,basis,n),
	    Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	final double Rpqt=TMatr.getAMatrA(psi,RQQpi,n),
	    Rqt=TMatr.getAMatrA(psi,RQQt,n),
	    qt=TMatr.getAMatrA(psi,QQt,n);
	J_IX=basis[iX];
	I_IX=1/Rqt;
	p_IX=Rpqt;
	ot=qt/Rqt;
	V_IX=TMatr.getAMatrA(psi,QQV,n)/qt;
	pj_IX=TMatr.getAMatrA(psi,QQpV,n)-basis[iX]*Rpqt;
	Dpj_IX=TMatr.getAMatrB(psi,Dpsi,QQpV,n)-basis[iX]*TMatr.getAMatrB(psi,Dpsi,RQQpi,n);
	for(int k=0;k<RQQi.length;k++) RQQj[k]=QQV[k]-basis[iX]*RQQi[k];
    }
    else
    {
	if(flag_j==M_VTOTAL)
	{
	    for(int k=0;k<RQQi.length;k++) RQQj[k]=RQQi[k]-QQV[k];
	}
	else if(flag_j==M_MINUS_V_VIH)
	{
	    for(int k=0;k<RQQi.length;k++) RQQj[k]=RQQi[k]-(QQV[k]-V_IH*QQt[k]);
	}
	else if(flag_j==M_PLUS_V_VIH)
	{
	    for(int k=0;k<RQQi.length;k++) RQQj[k]=RQQi[k]+(QQV[k]-V_IH*QQt[k]);
	}
	else throw new RuntimeException("flag_j="+flag_j);
	final double [] basis=M.EV.getGEV_Generalized(n,RQQj,RQQt,true);
	final int iX=flag_HIGH_I?n-1:0;
	final double pI_IX=TMatr.getAMatrA(n,basis,n+iX*n,RQQpi,n);
	J_IX=basis[iX];
	I_IX=TMatr.getAMatrA(n,basis,n+iX*n,RQQi,n);
	p_IX=pI_IX/I_IX;
	ot=TMatr.getAMatrA(n,basis,n+iX*n,QQt,n);
	V_IX=TMatr.getAMatrA(n,basis,n+iX*n,QQV,n)/ot;
	pj_IX=Dpj_IX=Double.NaN;
    }

    if(eps>0)
    {
	final double [] Trmom=convert_dFmomentsToFmomentsMatchRadau.apply(tmom);
	System.err.println("Trmom="+java.util.Arrays.toString(Trmom));
	System.err.println("Rtmom="+java.util.Arrays.toString(Rtmom));
	UnitTests.checkDoubleArrayRel(Trmom,"Trmom",Rtmom,"Rtmom",eps);

	final double oh=TMatr.getAMatrA(n,ibasis,n+iH*n,QQt,n),
	    j=TMatr.getAMatrA(n,ibasis,n+iH*n,RQQj,n)/oh,
	    vv=TMatr.getAMatrA(n,ibasis,n+iH*n,QQV,n)/oh;
	if(flag_j==M_VTOTAL)
	    UnitTests.checkDoubleRel(j,"j",V_IH-vv,"V_IH-vv",eps);
	else if(flag_j==M_PLUS_V_VIH)
	    UnitTests.checkDoubleRel(j,"j",vv,"vv",eps);
	else if(flag_j==M_MINUS_V_VIH)
	    UnitTests.checkDoubleRel(j,"j",2*V_IH-vv,"2*V_IH-vv",eps);
	else if(flag_j==M_REL_V_I)
	    UnitTests.checkDoubleRel(j,"j",vv-J_IX*V_IH,"vv-J_IX*V_IH",eps);

	System.err.println("DynVarFiniteFV:Trmom&Rtmom match");
    }

    return new DynVarFiniteFV(Double.NaN,ibasis[0],ibasis[iH],p_IH,V_IH,
			      J_IX,I_IX,p_IX,V_IX,1/ot,pj_IX,Dpj_IX);
}

}

