package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Linsystems;

/**
 *  @version $Id: DynVarIC.java,v 1.9 2020/09/27 20:33:12 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarIC {
final EVXData I,C;
final double [] I_nom,I_denom,C_nom,C_denom;
final double [] var_I,var_C;
final double p_IH,vp_IH,rvp_IH;
DynVarIC(final EVXData I,final double [] I_nom,final double [] I_denom,
	 final EVXData C,final double [] C_nom,final double [] C_denom,
	 final double [] var_I,final double [] var_C)
{
    this.I=I;
    this.I_nom=I_nom;
    this.I_denom=I_denom;
    this.C=C;
    this.C_nom=C_nom;
    this.C_denom=C_denom;
    this.p_IH=(C_nom[0]/C_denom[0])/(I_nom[0]/I_denom[0]);
    this.vp_IH=(C_nom[1]/C_denom[1])/(I_nom[1]/I_denom[1]);
    this.var_I=var_I;
    this.var_C=var_C;
    this.rvp_IH=var_C[2]/var_I[2];
}


/**
 */
static DynVarIC get_DynVarIC(final int n,
			     final double [] QQi,final double [] QQt,final double [] QQc,
			     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,
			     final double eps)
{
    final int iX=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M),C=EVXData.getEVXDataFromQQ(n,QQc,QQt,M);
    final double [] I_Dpsi=VOLATILITY_EDPsi.getDPsi(I.psiH),
	C_Dpsi=VOLATILITY_EDPsi.getDPsi(C.psiH);
    final double [] varI=RayleighQuotient.getRQVariationValues(I.psiH,I_Dpsi,VOLATILITY_EDPsi.getDPsi(I_Dpsi),QQi,QQt,n),
	varC=RayleighQuotient.getRQVariationValues(C.psiH,C_Dpsi,VOLATILITY_EDPsi.getDPsi(C_Dpsi),QQc,QQt,n);
    TMatr.sumP1(I_Dpsi,I.psiH,-TMatr.getAMatrB(I_Dpsi,I.psiH,QQt,n));
    TMatr.sumP1(C_Dpsi,C.psiH,-TMatr.getAMatrB(C_Dpsi,C.psiH,QQt,n));
    final DynVarIC r=
	new DynVarIC(I,new double []{I.sH,TMatr.getAMatrA(I_Dpsi,QQi,n)},new double []{1,TMatr.getAMatrA(I_Dpsi,QQt,n)},
		     C,new double []{C.sH,TMatr.getAMatrA(C_Dpsi,QQc,n)},new double []{1,TMatr.getAMatrA(C_Dpsi,QQt,n)},
		     varI,varC);
    if(eps>0)
    {
	UnitTests.checkDoubleRel(varI[0],"varI[0]",I.sH,"I.sH",eps);
	UnitTests.checkDoubleRel(varI[1],"varI[1]",0,"0",eps);
	UnitTests.checkDoubleRel(varC[0],"varC[0]",C.sH,"C.sH",eps);
	UnitTests.checkDoubleRel(varC[1],"varC[1]",0,"0",eps);
	final double I_pp=TMatr.getAMatrB(I_Dpsi,I.psiH,QQi,n),C_pp=TMatr.getAMatrB(C_Dpsi,C.psiH,QQc,n);
	final double [] rqI=RayleighQuotient.getRQVariationValues(I.psiH,VOLATILITY_EDPsi.getDPsi(I.psiH),null,QQi,QQt,n),
	    rqC=RayleighQuotient.getRQVariationValues(C.psiH,VOLATILITY_EDPsi.getDPsi(C.psiH),null,QQc,QQt,n);
	UnitTests.checkDoubleRel(I_pp,"I_pp",0,"0",eps);
	UnitTests.checkDoubleRel(rqI[0],"rqI[0]",r.I_nom[0],"r.I_nom[0]",eps);
	UnitTests.checkDoubleRel(rqI[1],"rqI[1]",0,"0",eps);
	UnitTests.checkDoubleRel(rqI[2]/2,"rqI[2]/2",r.I_nom[1]-r.I_nom[0]*r.I_denom[1],"r.I_nom[1]-r.I_nom[0]*r.I_denom[1]",eps);
	UnitTests.checkDoubleRel(C_pp,"C_pp",0,"0",eps);
	UnitTests.checkDoubleRel(rqC[0],"rqC[0]",r.C_nom[0],"r.C_nom[0]",eps);
	UnitTests.checkDoubleRel(rqC[1],"rqC[1]",0,"0",eps);
	UnitTests.checkDoubleRel(rqC[2]/2,"rqC[2]/2",r.C_nom[1]-r.C_nom[0]*r.C_denom[1],"r.C_nom[1]-r.C_nom[0]*r.C_denom[1]",eps);
	System.err.println("get_DynVarIC:<C_psi|c|C_Dpsi>="+C_pp+" <I_psi|I|I_Dpsi>="+I_pp);
	System.err.println("get_DynVarIC:varI[2]="+varI[2]+" rqI[2]="+rqI[2]+" varC[2]="+varC[2]+" rqC[2]="+rqC[2]);
    }
    return r;
}


static void matchDynVarIC(final DynVarIC a,final DynVarIC b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,eps);
    EVXData.matchEVXData(a.C,b.C,eps);
/*    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.var1pI,"a.var1pI",b.var1pI,"b.var1pI",eps);
    UnitTests.checkDoubleRel(a.dp2It,"a.dp2It",b.dp2It,"b.dp2It",eps);
    UnitTests.checkDoubleRel(a.dp1P,"a.dp1P",b.dp1P,"b.dp1P",eps);
*/
    System.err.println("DynVarC:matchDynVarIC test OK n="+a.I.neq+" eps="+eps);
}


}

