package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Linsystems;

/**
 *  @version $Id: DynVarIC2.java,v 1.6 2020/09/27 20:33:12 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarIC2 {
final EVXData I;
final double [] var_I,var_C;
final double p_IH,vp_IH;
DynVarIC2(final EVXData I,final double [] var_I,final double [] var_C)
{
    this.I=I;
    this.var_I=var_I;
    this.var_C=var_C;
    this.p_IH=var_C[0]/var_I[0];
    this.vp_IH=var_C[2]/var_I[2];
}


/**
 */
static DynVarIC2 get_DynVarIC2(final int n,
			       final double [] QQi,final double [] QQt,final double [] QQc,
			       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,
			       final boolean flag_0,
			       final double eps)
{
    final int iX=n-1;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] psi,Dpsi,D2psi;
    if(flag_0)
    {
	psi=I.psiH;
	Dpsi=I.psi0;
	D2psi=null;
    }
    else
    {
	psi=I.psiH;
	Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	D2psi=VOLATILITY_EDPsi.getDPsi(Dpsi);
    }
    final double [] varI=RayleighQuotient.getRQVariationValues(psi,Dpsi,D2psi,QQi,QQt,n),
	varC=RayleighQuotient.getRQVariationValues(psi,Dpsi,D2psi,QQc,QQt,n);
    if(eps>0)
    {
	UnitTests.checkDoubleRel(varI[0],"varI[0]",I.sH,"I.sH",eps);
	UnitTests.checkDoubleRel(varI[1],"varI[1]",0,"0",eps);

	System.err.println("get_DynVarIC2:flag_0="+flag_0+" varI="+java.util.Arrays.toString(varI)+" varC="+java.util.Arrays.toString(varC));
    }

    return new DynVarIC2(I,varI,varC);
}



static void matchDynVarIC2(final DynVarIC2 a,final DynVarIC2 b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,eps);
/*    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.var1pI,"a.var1pI",b.var1pI,"b.var1pI",eps);
    UnitTests.checkDoubleRel(a.dp2It,"a.dp2It",b.dp2It,"b.dp2It",eps);
    UnitTests.checkDoubleRel(a.dp1P,"a.dp1P",b.dp1P,"b.dp1P",eps);
*/
    System.err.println("DynVarC:matchDynVarIC2 test OK n="+a.I.neq+" eps="+eps);
}


}

