package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Variation of <psi|pI|psi>/<psi|psi> at eigenvectors
 *  of I|psi>=lambda |psi>
 */
class DynVarIPnL {
    final double p_A,p_IH,pd_IH,var_p_IH,t_IH,p_IL,pd_IL,var_p_IL,t_IL;
    final EVXData I;

    DynVarIPnL(final int n,
	       final double [] QQi,final double [] QQt,
	       final double [] QQpi, final double [] D,
	       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	this.p_A=QQpi[0]/QQi[0];
	this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);

	{
	final double [] Psi=I.getPsi(n-1),varPsi=TMatr.getMatr_x_A(n,D,Psi);
	final double pi=TMatr.getAMatrA(Psi,QQpi,n),mpi=TMatr.getAMatrB(Psi,varPsi,QQpi,n),mt=TMatr.getAMatrB(Psi,varPsi,QQt,n);
	this.p_IH=pi/I.sH;
	this.pd_IH=mpi/(I.sH*mt);
	this.var_p_IH=mpi-pi*mt;
	this.t_IH=mt;
	}

	{
	final double [] Psi=I.getPsi(0),varPsi=TMatr.getMatr_x_A(n,D,Psi);
	final double pi=TMatr.getAMatrA(Psi,QQpi,n),mpi=TMatr.getAMatrB(Psi,varPsi,QQpi,n),mt=TMatr.getAMatrB(Psi,varPsi,QQt,n);
	this.p_IL=pi/I.sL;
	this.pd_IL=mpi/(I.sL*mt);
	this.var_p_IL=mpi-pi*mt;
	this.t_IL=mt;
	}
    }

}

