package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.SortArrays;

/**
 * July, 26, 2016 gift I just received directly from John Pierpont Morgan I,
 * see KkQVM.java as an early (failed) version of a gift from the same person.
 * Let us check whether this one would be any better.
 * @version $Id: DynVarJPMorgan.java,v 1.13 2020/09/27 20:33:12 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarJPMorgan {
    final EVXData I,P;
    final double p_IH,var_p_IH,
	I_MP,P_MP,v2_P_MP,var2_p_IH_PROJ;
    DynVarJPMorgan(final EVXData I,final EVXData P,
		   final double p_IH,final double var_p_IH,
		   final double I_MP,final double P_MP,final double v2_P_MP,
		   final double var2_p_IH_PROJ
		   )
    {
	this.I=I;
	this.P=P;
	this.p_IH=p_IH;
	this.var_p_IH=var_p_IH;
	this.I_MP=I_MP;
	this.P_MP=P_MP;
	this.v2_P_MP=v2_P_MP;
	this.var2_p_IH_PROJ=var2_p_IH_PROJ;
    }

/** Consider |pI|psi>=\lapbda_P I|psi> states,
 * find the one giving max <psi|I|psi>/<psi|psi>
 * and consider second variation of \lapbda_P
 */
static DynVarJPMorgan get_DynVarJPMorgan(final int n,
					 final double [] QQi,final double [] QQt,final double [] QQpi,
					 final VolMatrix VOLATILITY_EDPsi,
					 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
					 final double eps)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");
	final int i0=0,iL=1,iH=n-1;
	final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M),
	    P=EVXData.getEVXDataFromQQ(n,QQpi,QQi,M);
	final int iX=n-1;
	final double [] psi_IH=I.getPsi(iX),Dpsi_IH=VOLATILITY_EDPsi.getDPsi(psi_IH);
	final double pI_IH=I.getFPsi(iX,QQpi),p_IH=pI_IH/I.ev[iX],
	    var_p_IH=2*(I.getFPsiPsi(iX,Dpsi_IH,QQpi)-pI_IH*I.getFPsiPsi(iX,Dpsi_IH,QQt));
	final double [] var2P=new double [n],var2I=new double [n],I_P=new double [n],prob_IH_Pk=new double [n];
	for(int k=0;k<n;k++)
	{
	    final double [] psiI=I.getPsi(k),psiP=P.getPsi(k);
	    final double [] rqI=RayleighQuotient.getRQVariationValues(psiI,VOLATILITY_EDPsi.getDPsi(psiI),null,QQi,QQt,n),
		rqP=RayleighQuotient.getRQVariationValues(psiP,VOLATILITY_EDPsi.getDPsi(psiP),null,QQpi,QQi,n);
	    var2I[k]=rqI[2];
	    var2P[k]=rqP[2];
	    I_P[k]=1/TMatr.getAMatrA(psiP,QQt,n);
	    prob_IH_Pk[k]=Math.pow(TMatr.getAMatrB(psiP,psi_IH,QQi,n),2)/I.sH;
	    if(eps>0)
	    {
		UnitTests.checkDoubleRel(rqI[0],"rqI[0]",I.ev[k],"I.ev[k]",eps);
		UnitTests.checkDoubleRel(rqI[1],"rqI[1]",0,"0",eps);
		UnitTests.checkDoubleRel(rqP[0],"rqP[0]",P.ev[k],"P.ev[k]",eps);
		UnitTests.checkDoubleRel(rqP[1],"rqP[1]",0,"0",eps);
		UnitTests.checkDoubleRel(TMatr.getAMatrA(psiP,QQi,n),"<psiP|I|psiP>",1,"1",eps);
		UnitTests.checkDoubleRel(TMatr.getAMatrA(psiI,QQt,n),"<psiI|psiI>",1,"1",eps);
	    }
	}

	final int [] ip=SortArrays.sortArrayIndexes(I_P);
	final int intP_maxI=ip[ip.length-1];

	if(eps>0)
	{
	    System.err.println("DynVarJPMorgan: I_P="+java.util.Arrays.toString(I_P)+" ind="+java.util.Arrays.toString(ip)+" prob_IH="+java.util.Arrays.toString(prob_IH_Pk)+" sum(prob_IH)="+TMatr.sumElements(prob_IH_Pk));
	    UnitTests.checkDoubleRel(TMatr.sumElements(prob_IH_Pk),"sum(prob_IH_Pk)",1,"1",eps);
	}


	return new DynVarJPMorgan(I,P,p_IH,var_p_IH,
				  I_P[intP_maxI],P.ev[intP_maxI],var2P[intP_maxI],
				  TMatr.mulVV(prob_IH_Pk,var2P));
    }
}
