package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.SortArrays;

/** The measure dmu=dv-I_{low}*dt is back.
 *  @version $Id: DynVarMu.java,v 1.23 2020/09/27 20:33:12 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarMu {
    private static final boolean PRINT_DEBUG=false;
    final EVXData I;
    final double p_IH,p_IL,p_ia,p_va,i_max,p_i_max,x_i_max,i_c,p_i_c,x_i_c;
    final double [] x,i,ci,p,tau;
    DynVarMu(final EVXData I,final double p_IH,final double p_IL,
	     final double [] x,final double [] i,final double [] ci,final double [] p,final double [] pnli,final double [] tau,final boolean isEdgeAtRight)
    {
	this.I=I;
	this.p_IH=p_IH;
	this.p_IL=p_IL;
	this.x=x;
	this.i=i;
	this.ci=ci;
	this.p=p;
	this.tau=tau;

	this.p_ia=TMatr.sumElements(ci)/TMatr.sumElements(i);
	this.p_va=TMatr.mulVV(ci,tau)/TMatr.mulVV(i,tau);

	final int ind_imax=SortArrays.sortArrayIndexes(i)[i.length-1];
	this.i_max=i[ind_imax];
	this.p_i_max=p[ind_imax];
	this.x_i_max=x[ind_imax];
	final int ind_c=DjQVM.getIndX0(x,I.x0,isEdgeAtRight);
	if(ind_c>=0)
	{
	    this.i_c=i[ind_c];
	    this.p_i_c=p[ind_c];
	    this.x_i_c=x[ind_c];
	}
	else
	{
	    this.i_c=this.p_i_c=this.x_i_c=Double.NaN;
	}
    }

static DynVarMu get_DynVarMu(final int n,
			     final double [] QQi,final double [] QQt,
			     final double [] QQpi,
			     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			     final boolean flag_L)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final double [] ONE=M.B.getONE();
    final boolean isEdgeAtRight=M.B.getD1(M.getRadauMultiplied(ONE),0)<0;
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int iL=0,iH=n-1;
    final double lam,clam,plam,pI_IH=I.getFPsi(iH,QQpi),p_IH=pI_IH/I.sH,
	pI_IL=I.getFPsi(iL,QQpi),p_IL=pI_IL/I.sL;
    final double [] psi;
    if(flag_L)
    {
	psi=I.psiL;
	lam=I.sL;
	clam=pI_IL;
	plam=p_IL;
    }
    else
    {
	psi=I.psiH;
	lam=I.sH;
	clam=pI_IH;
	plam=p_IH;
    }
    final int nd=I.neq-1;
    final double [] psiroots=M.getXNodes(psi).re_root,i=new double [nd],ci=new double [nd],p=new double [nd],pnli=new double [nd],tau=new double [nd];
    if(isEdgeAtRight)
    { // swap psiroots elements
	for(int k=psiroots.length/2;--k>=0;) {
	    final double v=psiroots[k];
	    psiroots[k]=psiroots[nd-1-k];
	    psiroots[nd-1-k]=v;
	}
    }
    for(int k=0;k<psiroots.length;k++)
    {
	final double xk=psiroots[k];
	final double [] l=M.B.sdiv1(psi,xk).q;
	final double lt=TMatr.getAMatrA(l,QQt,n),li=TMatr.getAMatrA(l,QQi,n),lpi=TMatr.getAMatrA(l,QQpi,n);
	tau[k]=lt/Math.pow(M.B.getD0(l,xk),2);
	final double lilt=li/lt,plilt=lpi/lt;
	i[k]=lilt-lam;
	ci[k]=plilt-clam;
	p[k]=ci[k]/i[k];
	//pnli[k]=plilt-plam*lilt;
    }
    if(PRINT_DEBUG)
    {
	System.err.println("DynVarMu: n="+n+" flag_L="+flag_L+" QQi[0]-lam*QQt[0]="+(QQi[0]-lam*QQt[0])+" quadr="+TMatr.mulVV(tau,i));
	System.err.println("DynVarMu: I.psiL roots="+java.util.Arrays.toString(M.getXNodes(I.psiL).re_root));
	System.err.println("DynVarMu: I.psiH roots="+java.util.Arrays.toString(M.getXNodes(I.psiH).re_root));
    }

    return new DynVarMu(I,p_IH,p_IL,psiroots,i,ci,p,pnli,tau,isEdgeAtRight);
}

}
