package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.PolynomialsRatio;

/** The measure dmu=dv-I_{low}*dt is back.
 *  @version $Id: DynVarMuX.java,v 1.46 2022/11/20 12:41:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarMuX {
    private static final boolean PRINT_DEBUG=false;
    final EVXData I;
    final double p_IH,p_IL,xenter,penter,ienter,xexit,pexit,iexit;
    final double [] x;
    DynVarMuX(final EVXData I,final double p_IH,final double p_IL,
	      final double xenter,final double penter,final double ienter,
	      final double xexit,final double pexit,final double iexit,
	      final double [] x)
    {
	this.I=I;
	this.p_IH=p_IH;
	this.p_IL=p_IL;
	this.xenter=xenter;
	this.penter=penter;
	this.ienter=ienter;
	this.xexit=xexit;
	this.pexit=pexit;
	this.iexit=iexit;
	this.x=x;
    }

static DynVarMuX get_DynVarMuX(final int n,
			       final double [] QQi,final double [] QQt,
			       final double [] QQpi,
			       final double [] QQXi,
			       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			       final boolean flag_L,
			       final double eps_debug)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int iL=0,iH=n-1,iX;
    final double lam,plam,p_IH=I.getFPsi(iH,QQpi)/I.sH,p_IL=I.getFPsi(iL,QQpi)/I.sL;
    final double [] psi;
    if(flag_L)
    {
	psi=I.psiL;
	lam=I.sL;
	plam=p_IL;
	iX=iL;
    }
    else
    {
	psi=I.psiH;
	lam=I.sH;
	plam=p_IH;
	iX=iH;
    }
    final double xexit=TMatr.getAMatrA(psi,QQXi,n)/lam,pexit=plam,iexit=lam;
    final double [] psiroots=M.getXNodes(psi).re_root;

    final double [] ONE=M.B.getONE();
    final boolean isEdgeAtRight=M.B.getD1(M.getRadauMultiplied(ONE),0)<0;
    int ind_xenter=-1;
    if(isEdgeAtRight)
    {
	for(int k=psiroots.length;--k>=0;) if(psiroots[k]<=xexit) {ind_xenter=k;break;}
    }
    else
    {
	for(int k=0;k<psiroots.length;k++) if(psiroots[k]>=xexit) {ind_xenter=k;break;}
    }
    final double xenter=ind_xenter<0?Double.NaN:psiroots[ind_xenter];
    final double [] psienter=getPsiX(I,iX,xenter,M);
    final double ienter=TMatr.getAMatrA(psienter,QQi,n),penter=TMatr.getAMatrA(psienter,QQpi,n)/ienter;

    if(eps_debug>0||PRINT_DEBUG)
    {
	final double eps=eps_debug>0?eps_debug:1e-5;
	System.err.println("DynVarMuX: I.psiL roots="+java.util.Arrays.toString(M.getXNodes(I.psiL).re_root));
	System.err.println("DynVarMuX: I.psiH roots="+java.util.Arrays.toString(M.getXNodes(I.psiH).re_root));
	System.err.println("DynVarMuX: psienter roots="+java.util.Arrays.toString(M.getXNodes(psienter).re_root));
	System.err.println("DynVarMuX:  xenter="+xenter+" xexit="+xexit+
			   " xpsienter="+(TMatr.getAMatrA(psienter,QQXi,n)/TMatr.getAMatrA(psienter,QQi,n))+
			   " psi(xenter)="+M.B.getD0(psi,xenter)+" <psienter|psi>="+TMatr.getAMatrB(psi,psienter,QQt,n)+" <psiener|I|psi>="+TMatr.getAMatrB(psi,psienter,QQi,n)+" <psienter|psienter>="+TMatr.getAMatrA(psienter,QQt,n)+" isEdgeAtRight="+isEdgeAtRight);

	final double [] X=M.getxP(ONE),X2=M.getxP(X);
	// verify some QQXi elements.
	UnitTests.checkDoubleRel(TMatr.getAMatrB(ONE,ONE,QQXi,n),"<ONE|QQXi|ONE>",TMatr.getAMatrB(ONE,X,QQi,n),"<ONE|QQi|X>",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(ONE,X,QQXi,n),"<ONE|QQXi|X>",TMatr.getAMatrB(X,X,QQi,n),"<X|QQi|X>",eps);
	if(n>2)UnitTests.checkDoubleRel(TMatr.getAMatrB(X,X,QQXi,n),"<X|QQXi|X>",TMatr.getAMatrB(X2,X,QQi,n),"<X2|QQi|X>",eps);

	if(!Double.isNaN(xenter))
	{
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,psienter,QQt,n),"<psi|psienter>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,psienter,QQi,n),"<psi|I|psienter>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psienter,QQt,n),"<psienter|psienter>",1,"1",eps);
	    UnitTests.checkDoubleRel(M.B.getD0(psi,xenter),"psi(xenter)",0,"0",eps);
	    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
		psixenter=TMatr.getMatr_x_A(n,IQQt,M.B.getX0Moments(n,xenter));
	    TMatr.scaleVectorInPlace(psixenter,1/Math.sqrt(TMatr.getAMatrA(psixenter,QQt,n)));
	    UnitTests.checkDoubleArrayRel1(EVXData.getMSign(psienter),"psienter",EVXData.getMSign(psixenter),"psixenter",eps);
	}
	System.err.println("DynVarMuX: test OK n="+n+" eps="+eps);
    }

    return new DynVarMuX(I,p_IH,p_IL,xenter,penter,ienter,xexit,pexit,iexit,psiroots);
}

static double [] getPsiX(final EVXData I,final int ind_skip,final double x,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final int n=I.neq;
    final double [] psi=new double [n];
    double s=0;
    for(int k=0;k<n;k++)
    {
	if(k==ind_skip) continue;
	final double [] psik=I.getPsi(k);
	final double pk=M.B.getD0(psik,x);
	TMatr.sumP1(psi,psik,pk);
	s+=pk*pk;
    }
    TMatr.scaleVectorInPlace(psi,1/Math.sqrt(s));
    return psi;
}

static DynVarMuX get_DynVarMuXX(final int n,
				final double [] QQi,final double [] QQt,final double [] QQpi,
				final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				final double eps_debug)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int iL=0,iH=n-1;
    final double p_IH=I.getFPsi(iH,QQpi)/I.sH,p_IL=I.getFPsi(iL,QQpi)/I.sL;
    final double [] xpsiL=M.getXNodes(I.psiL).re_root;
    // Find xexit: x corresponding to max I
    int ind_xexit=-1;double iexit=Double.NEGATIVE_INFINITY;double [] psiexit=null;
    for(int k=0;k<xpsiL.length;k++)
    {
	final double [] psik=getPsiX(I,iL,xpsiL[k],M);
	final double i=TMatr.getAMatrA(psik,QQi,n);
	if(i>=iexit){ind_xexit=k;iexit=i;psiexit=psik;}
	if(false && PRINT_DEBUG)
	{
	    System.err.println("DynVarMuX: xpsiL="+xpsiL[k]+" i="+i+" I.sH="+I.sH);
	}
    }
    final double xexit,pexit;
    if(psiexit!=null)
    {
	xexit=xpsiL[ind_xexit];
	pexit=TMatr.getAMatrA(psiexit,QQpi,n)/iexit;
    }
    else
    {
	xexit=pexit=Double.NaN;
    }
    int ind_xenter=-1;
    final double [] xpsiH=M.getXNodes(I.psiH).re_root;
    final double [] ONE=M.B.getONE();
    final boolean isEdgeAtRight=M.B.getD1(M.getRadauMultiplied(ONE),0)<0;
    // Find xenter: x most closed to xexit, but before it.
    if(isEdgeAtRight)
    {
	for(int k=xpsiH.length;--k>=0;) if(xpsiH[k]<=xexit) {ind_xenter=k;break;}
    }
    else
    {
	for(int k=0;k<xpsiH.length;k++) if(xpsiH[k]>=xexit) {ind_xenter=k;break;}
    }
    final double xenter,ienter,penter;double [] psienter=null;
    if(ind_xenter>=0)
    {
	xenter=xpsiH[ind_xenter];
	psienter=getPsiX(I,iH,xenter,M);
	ienter=TMatr.getAMatrA(psienter,QQi,n);
	penter=TMatr.getAMatrA(psienter,QQpi,n)/ienter;
    }
    else
    {
	xenter=ienter=penter=Double.NaN;
    }
    if(eps_debug>0||PRINT_DEBUG)
    {
	final double eps=eps_debug>0?eps_debug:1e-5;
	if(isEdgeAtRight?xenter>xexit:xenter<xexit) throw new RuntimeException("isEdgeAtRight="+isEdgeAtRight+" xenter="+xenter+" xexit="+xexit);
	System.err.println("DynVarMuX.get_DynVarMuXX:xpsiL="+java.util.Arrays.toString(xpsiL));
	System.err.println("DynVarMuX.get_DynVarMuXX:xpsiH="+java.util.Arrays.toString(xpsiH));
	System.err.println("DynVarMuX.get_DynVarMuXX: xenter="+xenter+" xexit="+xexit+" iexit="+iexit+" I.sH="+I.sH);
	final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	    Knom=M.B.getKK(n,TMatr.mulMatrices(n,new double [][]{IQQt,QQi,IQQt})),Kdenom=M.B.getKK(n,IQQt);
	final PolynomialsRatio pr=new PolynomialsRatio(Kdenom,Knom,M.B);
	for(int k=0;k<xpsiL.length;k++)
	{
	    final double x=xpsiL[k];
	    UnitTests.checkDoubleRel(M.B.getD0(I.psiL,x),"I.psiL(xpsiL)",0,"0",eps);
	    final double [] psik=getPsiX(I,iL,x,M);
	    UnitTests.checkDoubleRel(pr.getF(x),"I(xpsiL)",TMatr.getAMatrA(psik,QQi,n),"<psik|I|psik>",eps);
	    // Not a "true" maximum.
	    //UnitTests.checkDoubleRel(pr.getDF(x),"dI/dx(xpsiL)",0,"0",eps);
	    //if(pr.getD2F(x)>0) throw new RuntimeException("xpsiL="+x+" pr.getD2F(xpsiL)="+pr.getD2F(x));
	}
	for(int k=0;k<xpsiH.length;k++)
	{
	    final double x=xpsiH[k];
	    UnitTests.checkDoubleRel(M.B.getD0(I.psiH,x),"I.psiH(xpsiH)",0,"0",eps);
	    final double [] psik=getPsiX(I,iH,x,M);
	    UnitTests.checkDoubleRel(pr.getF(x),"I(xpsiH)",TMatr.getAMatrA(psik,QQi,n),"<psik|I|psik>",eps);
	    // Not a "true" minimum.
	    //UnitTests.checkDoubleRel(pr.getDF(x),"dI/dx(xpsiH)",0,"0",eps);
	    //if(pr.getD2F(x)<0) throw new RuntimeException("xpsiH="+x+" pr.getD2F(xpsiH)="+pr.getD2F(x));
	}
	if(psienter!=null)
	{
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(I.psiH,psienter,QQt,n),"<I.psiH|psienter>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(I.psiH,psienter,QQi,n),"<I.psiH|I|psienter>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psienter,QQt,n),"<psienter|psienter>",1,"1",eps);
	}
	if(psiexit!=null)
	{
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(I.psiL,psiexit,QQt,n),"<I.psiL|psiexit>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(I.psiL,psiexit,QQi,n),"<I.psiL|I|psiexit>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiexit,QQt,n),"<psiexit|psiexit>",1,"1",eps);
	}
	if(psienter!=null && psiexit!=null)
	{
	    System.err.println("DynVarMuX.get_DynVarMuXX:<psienter|psiexit>="+TMatr.getAMatrB(psienter,psiexit,QQt,n));
	}

	System.err.println("DynVarMuX.get_DynVarMuXX: test OK n="+n+" eps="+eps);
    }

    return new DynVarMuX(I,p_IH,p_IL,xenter,penter,ienter,xexit,pexit,iexit,xpsiH);
}


static DynVarMuX get_DynVarMuX(final int n,
			       final double [] QQi,final double [] QQt,final double [] QQpi,
			       final PsiIntegratedByInterval INTRV,
			       final double eps_debug)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=INTRV.M;

    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int iL=0,iH=n-1,iX;
    final double lam,plam,p_IH=I.getFPsi(iH,QQpi)/I.sH,p_IL=I.getFPsi(iL,QQpi)/I.sL;
    final double [] psi;
    if(false)
    {
	psi=I.psiL;
	lam=I.sL;
	plam=p_IL;
	iX=iL;
    }
    else
    {
	psi=I.psiH;
	lam=I.sH;
	plam=p_IH;
	iX=iH;
    }
    final double pexit=plam,iexit=lam;
    final double [] psiroots=M.getXNodes(psi).re_root;

    final PsiIntegratedByInterval.Intervals in=INTRV.getIntervals(psiroots,M.B.mult2Pol(psi,psi));
    final double xexit=in.index_w_max_interval>0?in.x[in.index_w_max_interval-1]:Double.NaN,xenter=in.x_w_max_interval;
    final double [] psienter=getPsiX(I,iX,xenter,M);
    final double ienter=TMatr.getAMatrA(psienter,QQi,n),penter=TMatr.getAMatrA(psienter,QQpi,n)/ienter;

    if(eps_debug>0||PRINT_DEBUG)
    {
	final double eps=eps_debug>0?eps_debug:1e-5;
	System.err.println("DynVarMuX: psi roots="+java.util.Arrays.toString(M.getXNodes(psi).re_root));
	System.err.println("DynVarMuX: in.x="+java.util.Arrays.toString(in.x));
	System.err.println("DynVarMuX: in.w="+java.util.Arrays.toString(in.w));
	System.err.println("DynVarMuX: in.index_w_max_interval="+in.index_w_max_interval+" xenter="+xenter+" xexit="+xexit+" P(enter,exit)="+in.w_max_interval);
	for(int k=0;k<in.x.length;k++)
	{
	    System.err.print("DynVarMuX: x="+in.x[k]+" w="+in.w[k]);
	    if(k>0)
	    {
		final double analytical_w_kk=in.w[k-1]-in.w[k],
		    numerical_w_kk=INTRV.test_getWeight_Numerically(in.x[INTRV.isEdgeAtRight?k:k-1],in.x[INTRV.isEdgeAtRight?k-1:k],M.B.mult2Pol(psi,psi));
		System.err.print("; ["+in.x[k-1]+" to "+in.x[k]+" analytical_w_kk="+analytical_w_kk+" numerical_w_kk="+numerical_w_kk);
		UnitTests.checkDoubleRel(analytical_w_kk,"analytical_w_kk",numerical_w_kk,"numerical_w_kk",eps);
		// the 0-th element is edge, then roots for k>0
		UnitTests.checkDoubleRel(M.B.getD0(psi,in.x[k]),"psi(x)",0,"0",eps);
	    }
	    else
	    {
		// check normalizing
		UnitTests.checkDoubleRel(in.w[k],"in.w[0]",1,"1",eps);
	    }
	    System.err.println();
	}

	System.err.println("DynVarMuX: test OK n="+n+" eps="+eps);
    }

    return new DynVarMuX(I,p_IH,p_IL,xenter,penter,ienter,xexit,pexit,iexit,psiroots);
}

static DynVarMuX get_DynVarMuX_SUM(final int n,
				   final double [] QQi,final double [] QQt,final double [] QQpi,
				   final PsiIntegratedByInterval INTRV,
				   final double eps_debug)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=INTRV.M;

    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int iL=0,iH=n-1,iX;
    final double lam,plam,p_IH=I.getFPsi(iH,QQpi)/I.sH,p_IL=I.getFPsi(iL,QQpi)/I.sL;
    final double [] psi;
    if(false)
    {
	psi=I.psiL;
	lam=I.sL;
	plam=p_IL;
	iX=iL;
    }
    else
    {
	psi=I.psiH;
	lam=I.sH;
	plam=p_IH;
	iX=iH;
    }
    final double pexit=plam;
    final double [] psiroots=M.getXNodes(psi).re_root;

    final PsiIntegratedByInterval.Intervals in=INTRV.getIntervals(psiroots,M.B.mult2Pol(psi,psi));
    double si=0,spi=0,sw=0;
    for(int k=1;k<in.w.length;k++)
    {
	final double xk=in.x[k],wkk=in.w[k-1]-in.w[k],w_iexit=lam*wkk;
	sw+=wkk;
	final double [] psienter=getPsiX(I,iX,xk,M);
	final double ient=TMatr.getAMatrA(psienter,QQi,n),pent=TMatr.getAMatrA(psienter,QQpi,n)/ient;
	// On what I to calculate P&L: i_enter, i_exit or (i_exit-i_enter)?
	if(false)
	{
	    si+=w_iexit;
	    spi+=pent*w_iexit; // Try to calculate on weighted i_exit.
	}
	else
	{
	    si+=wkk*(lam-ient);
	    spi+=wkk*(lam*pexit-ient*pent);
	}
	// Note: for this k:  weight=wkk ; i_enter=ient, p_enter=pent, i_exit=lam, p_exit=plam.
    }
    final double iexit=lam,ienter=si,penter=spi/si;
    // Kludge. Save weight to x
    final double xexit=in.w[0],xenter=sw;

    if(eps_debug>0||PRINT_DEBUG)
    {
	final double eps=eps_debug>0?eps_debug:1e-5;
	System.err.println("DynVarMuX: psi roots="+java.util.Arrays.toString(M.getXNodes(psi).re_root));
	System.err.println("DynVarMuX: in.x="+java.util.Arrays.toString(in.x));
	System.err.println("DynVarMuX: in.w="+java.util.Arrays.toString(in.w));
	System.err.println("DynVarMuX: sw="+sw+" in.w[0]="+in.w[0]);
	double analytical_w_kk=in.w[0]-in.w[in.w.length-1],
	    numerical_w_kk=INTRV.test_getWeight_Numerically(in.x[in.x.length-1],in.x[0],M.B.mult2Pol(psi,psi));
	if(!INTRV.isEdgeAtRight) numerical_w_kk=-numerical_w_kk;
	UnitTests.checkDoubleRel(analytical_w_kk,"analytical_w_kk",sw,"sw",eps);
	UnitTests.checkDoubleRel(analytical_w_kk,"analytical_w_kk",numerical_w_kk,"numerical_w_kk",eps);
	// check normalizing
	UnitTests.checkDoubleRel(in.w[0],"in.w[0]",1,"1",eps);
	System.err.println("DynVarMuX.get_DynVarMuX_SUM: test OK n="+n+" eps="+eps);
    }

    return new DynVarMuX(I,p_IH,p_IL,xenter,penter,ienter,xexit,pexit,iexit,psiroots);
}


}
