package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

class DynVarPnL {
    final double I_0,I_L,I_H,p_IH,pd_IH,pdo_IH,dif_IH,p_IL,pd_IL,pdo_IL,dif_IL,pDiff;
    final double [] basis;

    DynVarPnL(final int n,
	    final double [] QQi,final double [] QQt, final double [] QQf, final double [] QQfn,
	    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	final int i0=0,iL=1,iH=n-1;
	this.basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
	this.I_0=basis[i0];
	this.I_L=basis[iL];
	this.I_H=basis[iH];

	final double fH,fnH;
	{
	final double [] psi=EVXData.getPsiFromBasis(iH,basis,n),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	final double fnorm=fnH=(QQfn==QQi?basis[iH]:(QQfn==QQt?1:TMatr.getAMatrA(psi,QQfn,n)));
	this.p_IH=(fH=TMatr.getAMatrA(psi,QQf,n))/fnorm;
	this.pdo_IH=TMatr.getAMatrA(Dpsi,QQf,n)/TMatr.getAMatrA(Dpsi,QQfn,n);
	TMatr.sumP1(Dpsi,psi,-TMatr.getAMatrB(Dpsi,psi,QQfn,n)/fnorm);
	this.pd_IH=TMatr.getAMatrA(Dpsi,QQf,n)/TMatr.getAMatrA(Dpsi,QQfn,n);
	this.dif_IH=TMatr.getAMatrB(psi,Dpsi,QQf,n);
	}

	final double fL,fnL;
	{
	final double [] psi=EVXData.getPsiFromBasis(iL,basis,n),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	final double fnorm=fnL=(QQfn==QQi?basis[iL]:(QQfn==QQt?1:TMatr.getAMatrA(psi,QQfn,n)));
	this.p_IL=(fL=TMatr.getAMatrA(psi,QQf,n))/fnorm;
	this.pdo_IL=TMatr.getAMatrA(Dpsi,QQf,n)/TMatr.getAMatrA(Dpsi,QQfn,n);
	TMatr.sumP1(Dpsi,psi,-TMatr.getAMatrB(Dpsi,psi,QQfn,n)/fnorm);
	this.pd_IL=TMatr.getAMatrA(Dpsi,QQf,n)/TMatr.getAMatrA(Dpsi,QQfn,n);
	this.dif_IL=TMatr.getAMatrB(psi,Dpsi,QQf,n);
	}
	this.pDiff=(fH-fL)/(fnH-fnL);

    }

}

