package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.SortArrays;
import com.polytechnik.utils.IntPTRCompare;

/** The measure dmu=dv-I_{low}*dt is back.
 *  @version $Id: DynVarX.java,v 1.17 2022/07/17 21:23:52 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarX {
    final double p_IX,I_IX,x_IX,px_IX,pv_IX,p_IX2,I_IX2,x_IX2,pv_IX2,pax_MU,pB,pF,Pjtau;
    final ACorr px_mu,px_i;
    DynVarX(final double p_IX,final double I_IX,final double x_IX,final double px_IX,final double pv_IX,
	    final double p_IX2,final double I_IX2,final double x_IX2,final double pv_IX2,final double pax_MU,final double pB,final double pF,final double Pjtau,final ACorr px_mu,final ACorr px_i)
    {
	this.p_IX=p_IX;
	this.I_IX=I_IX;
	this.x_IX=x_IX;
	this.px_IX=px_IX;
	this.pv_IX=pv_IX;
	this.p_IX2=p_IX2;
	this.I_IX2=I_IX2;
	this.x_IX2=x_IX2;
	this.pv_IX2=pv_IX2;
	this.pax_MU=pax_MU;
	this.pB=pB;
	this.pF=pF;
	this.Pjtau=Pjtau;
	this.px_mu=px_mu;
	this.px_i=px_i;
    }

static DynVarX get_DynVarX(final int n,
			   final double [] QQi,final double [] QQt,
			   final double [] QQpi,final double [] QQpt,
			   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			   final boolean flag_MU)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iX=0;
    final double [] basis=M.EV.getGEV_Generalized(n,QQi,QQt,true),
	psi=EVXData.getPsiFromBasis(iX,basis,n),psiroots=M.getXNodes(psi).re_root;
    final double lam=basis[iX];
    final double [] wmu=new double [psiroots.length],imu=new double [psiroots.length],
	pmu=new double [psiroots.length],pv=new double [psiroots.length],
	xpmu=new double [psiroots.length],tau=new double [psiroots.length];
    double smu=0,spaxmu=0,spjtau=0,sjtau=0;
    for(int k=0;k<psiroots.length;k++)
    {
	final double x=psiroots[k];
	final double [] Rpsi=M.B.sdiv1(psi,x).q;
	final double r2i=TMatr.getAMatrA(Rpsi,QQi,n),r2t=TMatr.getAMatrA(Rpsi,QQt,n),
	    R2psiX=Math.pow(M.B.getD0(Rpsi,x),2),r2pi=TMatr.getAMatrA(Rpsi,QQpi,n);
	smu+=(wmu[k]=(r2i-lam*r2t)/Math.pow(M.B.getD0(Rpsi,x),2));
	imu[k]=r2i/r2t-lam;
	pmu[k]=(r2pi-lam*(QQpt==null?0:TMatr.getAMatrA(Rpsi,QQpt,n)))/(r2i-lam*(QQpt==null?0:r2t));
	pv[k]=r2pi/r2i;
	tau[k]=r2t/R2psiX;
	spjtau+=pv[k]/tau[k];
	sjtau+=1/tau[k];
	final double xp=TMatr.getAMatrB(Rpsi,psi,QQpi,n)-lam*(QQpt==null?0:TMatr.getAMatrB(Rpsi,psi,QQpt,n));
	xpmu[k]=xp/r2t;
	spaxmu+=xp/R2psiX;
    }
    final int [] indw=SortArrays.sortArrayIndexes(psiroots.length,IntPTRCompare.sortDescending(flag_MU?wmu:imu));
    if(false)
    {
	final int [] indwj=SortArrays.sortArrayIndexes(flag_MU?wmu:imu);
	System.err.println("smu="+smu+" v[0]-lam*t[0]="+(QQi[0]-lam*QQt[0])+" flag_MU="+flag_MU);
	for(int k=0;k<psiroots.length;k++)
	{
	    if(indw[k]!=indwj[psiroots.length-1-k]) throw new RuntimeException("indw["+k+"]="+indw[k]+" indwj["+(psiroots.length-1-k)+"]="+indwj[psiroots.length-1-k]);
	    System.err.println(""+k+"\tind="+indw[k]+" w="+((flag_MU?wmu:imu)[indw[k]])+" wj="+((flag_MU?wmu:imu)[indwj[k]]));
	}
    }
    final double pB,pF;
    final int i0=indw[0],i1=indw.length>1?indw[1]:i0;
    if(psiroots[i0]<psiroots[i1]){pB=pmu[i0];pF=pmu[i1];}else{pB=pmu[i1];pF=pmu[i0];}
    return new DynVarX(pmu[i0],imu[i0],psiroots[i0],xpmu[i0],pv[i0],pmu[i1],imu[i1],psiroots[i1],pv[i1],spaxmu/smu,pB,pF,
		       spjtau/sjtau,
		       ACorr.getACorr_ABW(psiroots.length,pv,psiroots,wmu),
		       ACorr.getACorr_ABW(psiroots.length,pv,psiroots,imu));
}

}
