package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/** The measure dmu=dv-I_{low}*dt is back.
 *  @version $Id: DynVarXi.java,v 1.15 2020/09/27 20:33:13 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarXi {
    final double p_IX,I_IX,x_IX,sPnlMuB,sPnlMuA,sPnlIB,sPnlIA,sMuB,sMuA;
    DynVarXi(final double p_IX,final double I_IX,final double x_IX,
	     final double sPnlMuB,final double sPnlMuA,
	     final double sPnlIB,final double sPnlIA,
	     final double sMuB,final double sMuA)
    {
	this.p_IX=p_IX;
	this.I_IX=I_IX;
	this.x_IX=x_IX;
	this.sPnlMuB=sPnlMuB;
	this.sPnlMuA=sPnlMuA;
	this.sPnlIB=sPnlIB;
	this.sPnlIA=sPnlIA;
	this.sMuB=sMuB;
	this.sMuA=sMuA;
    }


static DynVarXi get_DynVarXi(final int n,
			     final double [] QQi,final double [] QQt,
			     final double [] QQpi,
			     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			     final boolean flag_select_MU_max)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iSel=0;
    final double [] basis=M.EV.getGEV_Generalized(n,QQi,QQt,true),
	psi=EVXData.getPsiFromBasis(iSel,basis,n),psiroots=M.getXNodes(psi).re_root; /* psiroots are sorted. */
    final double lam=basis[iSel];
    final double [] wmu=new double [psiroots.length],imu=new double [psiroots.length],
	pv=new double [psiroots.length];
    int ind_max_Mu=-1,ind_max_I=-1;
    for(int k=0;k<psiroots.length;k++)
    {
	final double x=psiroots[k];
	final double [] Rpsi=M.B.sdiv1(psi,x).q;
	final double r2i=TMatr.getAMatrA(Rpsi,QQi,n),r2t=TMatr.getAMatrA(Rpsi,QQt,n),
	    R2psiX=Math.pow(M.B.getD0(Rpsi,x),2),r2pi=TMatr.getAMatrA(Rpsi,QQpi,n);
	wmu[k]=(r2i-lam*r2t)/R2psiX;
	imu[k]=r2i/r2t-lam;
	pv[k]=r2pi/r2i;
	if(ind_max_Mu<0||wmu[k]>=wmu[ind_max_Mu]) ind_max_Mu=k;
	if(ind_max_I<0||imu[k]>=imu[ind_max_I]) ind_max_I=k;
    }
    final double [] ONE=M.B.getONE();
    final double [] RONE=M.getRadauMultiplied(ONE);
    final boolean flag_dir_right=M.B.getD1(RONE,0)<0;

    final int ind=flag_select_MU_max?ind_max_Mu:ind_max_I;
    double p_IX=Double.NaN,I_IX=Double.NaN,x_IX=Double.NaN,
	sPnlMuB=0,sPnlMuA=0,sPnlIB=0,sPnlIA=0,sMuB=0,sMuA=0;
    if(ind>=0)
    {
	p_IX=pv[ind];
	I_IX=imu[ind];
	x_IX=psiroots[ind];
	for(int k=0;k<psiroots.length;k++)
	{
	    final double dp=pv[k]-pv[ind];
	    if(k<ind){
		sPnlMuB+=dp*wmu[k];
		sPnlIB+=dp*imu[k];
		sMuB+=wmu[k];
	    } else if(k>ind) {
		sPnlMuA+=dp*wmu[k];
		sPnlIA+=dp*imu[k];
		sMuA+=wmu[k];
	    }
	}
    }
    if(false)
    {
	System.err.println("DynVarXi:sMuB+sMuA+wmu[ind]="+(sMuB+sMuA+wmu[ind])+"=QQi[0,0]-lam*QQt[0,0]="+(QQi[0]-lam*QQt[0])+" wmu="+java.util.Arrays.toString(wmu));
    }
    return new DynVarXi(p_IX,I_IX,x_IX,sPnlMuB,sPnlMuA,sPnlIB,sPnlIA,sMuB,sMuA);
}

static DynVarXi getR_DynVarXi(final int n,
			      final double [] QQi,final double [] QQt,
			      final double [] QQpi,
			      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			      final double p_last,
			      final boolean flag_select_MU_max)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final int iSel=0,nd=n-1;
    final double [] RQQi=VolMatrix.getRadauMatrix(n,QQi,M),
	RQQt=VolMatrix.getRadauMatrix(n,QQt,M),
	RQQpi=VolMatrix.getRadauMatrix(n,QQpi,M),
	basis=M.EV.getGEV_Generalized(nd,RQQi,RQQt,true),
	psi=EVXData.getPsiFromBasis(iSel,basis,nd),psiroots=M.getXNodes(psi).re_root;
    final double lam=basis[iSel];
    final double [] rt=new  double [psiroots.length+1],
	wmu=new double [psiroots.length+1],imu=new double [psiroots.length+1],
	pv=new double [psiroots.length+1],ONE=M.B.getONE();
    final double [] RONE=M.getRadauMultiplied(ONE);
    final boolean flag_dir_right=M.B.getD1(RONE,0)<0;
    final double x0=M.getRadauRoot();
    System.arraycopy(psiroots,0,rt,0,psiroots.length);
    rt[psiroots.length]=x0;
    com.polytechnik.utils.SortArrays.sortArrayWithLoad(rt.length,rt,null);

    int ind_max_Mu=-1,ind_max_I=-1;
    for(int k=0;k<rt.length;k++)
    {
	final double x=rt[k],r2i,r2t,r2pi,R2psiX;
	if(x==x0)
	{
	    r2i=TMatr.getAMatrA(psi,QQi,n);
	    r2t=TMatr.getAMatrA(psi,QQt,n);
	    r2pi=TMatr.getAMatrA(psi,QQpi,n);
	    R2psiX=Math.pow(M.B.getD0(psi,x),2);
	    pv[k]=p_last; /* r2pi/r2i; */
	}
	else
	{
	    final double [] Rpsi=M.B.sdiv1(psi,x).q;
	    r2i=TMatr.getAMatrA(Rpsi,RQQi,nd);
	    r2t=TMatr.getAMatrA(Rpsi,RQQt,nd);
	    r2pi=TMatr.getAMatrA(Rpsi,RQQpi,nd);
	    R2psiX=Math.pow(M.B.getD0(Rpsi,x),2)*M.B.getD0(RONE,x);
	    pv[k]=r2pi/r2i;
	}
	wmu[k]=(r2i-lam*r2t)/R2psiX;
	imu[k]=r2i/r2t-lam;
	if(ind_max_Mu<0||wmu[k]>=wmu[ind_max_Mu]) ind_max_Mu=k;
	if(ind_max_I<0||imu[k]>=imu[ind_max_I]) ind_max_I=k;
    }

    final int ind=flag_select_MU_max?ind_max_Mu:ind_max_I;
    double p_IX=Double.NaN,I_IX=Double.NaN,x_IX=Double.NaN,
	sPnlMuB=0,sPnlMuA=0,sPnlIB=0,sPnlIA=0,sMuB=0,sMuA=0;
    if(ind>=0)
    {
	p_IX=pv[ind];
	I_IX=imu[ind];
	x_IX=rt[ind];
	for(int k=0;k<rt.length;k++)
	{
	    final double dp=pv[k]-pv[ind];
	    if(k<ind){
		sPnlMuB+=dp*wmu[k];
		sPnlIB+=dp*imu[k];
		sMuB+=wmu[k];
	    } else if(k>ind) {
		sPnlMuA+=dp*wmu[k];
		sPnlIA+=dp*imu[k];
		sMuA+=wmu[k];
	    }
	}
    }
    if(false)
    {
	System.err.println("DynVarXi:R:sMuB+sMuA+wmu[ind]="+(sMuB+sMuA+wmu[ind])+"=QQi[0,0]-lam*QQt[0,0]="+(QQi[0]-lam*QQt[0])+" wmu="+java.util.Arrays.toString(wmu));
    }
    return new DynVarXi(p_IX,I_IX,x_IX,sPnlMuB,sPnlMuA,sPnlIB,sPnlIA,sMuB,sMuA);
}


}
