package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** The measure dmu=dv-I_{low}*dt is back.
 *  @version $Id: DynVarXk.java,v 1.43 2022/07/28 12:27:17 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynVarXk {
    private static final boolean FLAG_DEBUG_AND_PRINT=false;
    final double varpIk,p_IX,I_IX,p_EQ,pa_EQ,pat_EQ,pm_EQ,pmv_EQ,xpm_EQ,pd_EQ,pad_EQ,pdm_EQ,x_EQ,xa_EQ,xm_EQ,pax_MU,
	op_EQ,oI_EQ;
    DynVarXk(final int n,
	     final double [] QQi,final double [] QQt,final double [] QQpi,final double [] QQpt,
	     final boolean flag_IL,
	     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	final int iX=flag_IL?0:n-1;
	final double [] basis=M.EV.getGEV_Generalized(n,QQi,QQt,true),
	    psi=EVXData.getPsiFromBasis(iX,basis,n),psiroots=M.getXNodes(psi).re_root;
	I_IX=basis[iX];
	final double pI_IX=TMatr.getAMatrA(psi,QQpi,n),t0=QQt[0]; // assume Q0 is 1
	p_IX=pI_IX/I_IX;
	double s1C=0,s2pI=0,s2I=0,sa2pI=0,sa2I=0,rI_max=0,pmm_EQ=Double.NaN,pmmv_EQ=Double.NaN,
	    xp_Imax=Double.NaN,
	    sd2pI=0,sad2pI=0,pdmm_EQ=Double.NaN,
	    s2xI=0,sa2xI=0,xmm_EQ=Double.NaN,cl=0,vl=0,
	    s2pt=0,s2t=0,spaxmu=0;
	for(int k=0;k<psiroots.length;k++)
	{
	    final double x=psiroots[k];
	    final double [] Rpsi=M.B.sdiv1(psi,x).q;
	    final double varpIXk=TMatr.getAMatrB(psi,Rpsi,QQpi,n)-pI_IX*TMatr.getAMatrB(psi,Rpsi,QQt,n);
	    s1C+=varpIXk;
	    final double r2=TMatr.getAMatrA(Rpsi,QQt,n),R2psiX=Math.pow(M.B.getD0(Rpsi,x),2),
		r2pi=TMatr.getAMatrA(Rpsi,QQpi,n),
		pImunn=r2pi-I_IX*TMatr.getAMatrA(Rpsi,QQpt,n),
		pdIRmm=r2pi/r2-pI_IX,
		padIRmm=(r2pi-pI_IX*r2)/R2psiX,
		rpI=pImunn/r2,rapI=pImunn/R2psiX,r2i=TMatr.getAMatrA(Rpsi,QQi,n),
		rI=r2i/r2-I_IX,wk=(r2i-I_IX*r2)/R2psiX,pk=rpI/rI,tauk=r2/R2psiX;
	    s2pI+=rpI;
	    s2xI+=x*rI;
	    s2I+=rI;
	    sa2pI+=rapI;
	    sa2xI+=x*wk;
	    sa2I+=wk;
	    sd2pI+=pdIRmm;
	    sad2pI+=padIRmm;
	    s2pt+=tauk*pk;
	    s2t+=tauk;
	    cl+=r2pi/R2psiX/t0;
	    vl+=r2i/R2psiX/t0;
	    spaxmu+=(TMatr.getAMatrB(psi,Rpsi,QQpi,n)-I_IX*TMatr.getAMatrB(psi,Rpsi,QQpt,n))/R2psiX;
	    //System.err.println("pk*wk="+(pk*wk)+" rapI="+rapI);

	    if(k==0||(flag_IL?rI>=rI_max:rI<=rI_max)){
		rI_max=rI;pmm_EQ=pk;pdmm_EQ=pdIRmm/rI;xmm_EQ=x;
		xp_Imax=(TMatr.getAMatrB(psi,Rpsi,QQpi,n)-I_IX*TMatr.getAMatrB(psi,Rpsi,QQpt,n))/r2;
		pmmv_EQ=r2pi/r2i;
	    }

	    if(FLAG_DEBUG_AND_PRINT)
	    {
		final double eps=1e-5,
		    _IX=TMatr.getAMatrA(psi,QQi,n),
		    prp=TMatr.getAMatrB(psi,Rpsi,QQt,n),
		    varIX=TMatr.getAMatrB(psi,Rpsi,QQi,n)-_IX*prp,
		    varpIX=TMatr.getAMatrB(psi,Rpsi,QQpi,n)-TMatr.getAMatrA(psi,QQpi,n)*prp,
		    _rI=TMatr.getAMatrA(Rpsi,QQi,n)/TMatr.getAMatrA(Rpsi,QQt,n)-_IX;
		System.err.println("k="+k+" xk="+x+" <psi|Rpsi>="+prp+" varpIXk="+varpIXk+
				   "\tvarIX="+varIX+"=0"+
				   "\t<psi|I|psi>="+TMatr.getAMatrA(psi,QQi,n)+"="+_IX+"="+I_IX+
				   "\t<psi|psi>="+TMatr.getAMatrB(psi,psi,QQt,n)+"=1"+
				   "\t<Rpsi|Rpsi>="+r2+
				   "\t IL="+basis[0]+" IH="+basis[n-1]+" psi(xk)="+M.B.getD0(psi,x)+"=0");
		UnitTests.checkDoubleRel(varIX,"varIX_"+k,0,"0",eps);
		UnitTests.checkDoubleRel(varpIX,"varpIX_"+k,varpIXk,"varpIXk",eps);
		UnitTests.checkDoubleRel(_IX,"<psi|I|psi>",I_IX,"I_IX",eps);
		UnitTests.checkDoubleRel(rI,"rI",_rI,"_rI",eps);
		UnitTests.checkDoubleRel(M.B.getD0(psi,x),"psi(xk)",0,"0",eps);
		if(flag_IL?rI<0:rI>0) throw new RuntimeException("rI="+rI);

		if(k>=psiroots.length-1) // last point
		{
		    // assume Q0 in basis is 1.
		    System.err.println("mu0 match sa2I="+sa2I+" s2I="+s2I+" I_IX="+I_IX);
		    UnitTests.checkDoubleRel(sa2I,"sa2I",QQi[0]-I_IX*QQt[0],"mu[0]",eps);
		}
		// sing not always preserved
		//if(prp>0) throw new RuntimeException("prp="+prp);
	    }
	}
	varpIk=s1C;
	p_EQ=s2pI/s2I;
	pa_EQ=sa2pI/sa2I;
	pm_EQ=pmm_EQ;
	pmv_EQ=pmmv_EQ;
	xpm_EQ=xp_Imax;
	pd_EQ=sd2pI/s2I;
	pad_EQ=sad2pI/sa2I;
	pdm_EQ=pdmm_EQ;
	x_EQ=s2xI/s2I;
	xa_EQ=sa2xI/sa2I;
	xm_EQ=xmm_EQ;
	pat_EQ=s2pt/s2t;
	oI_EQ=vl;
	op_EQ=cl/vl;
	pax_MU=spaxmu/sa2I;
    }

static void test_DynVarXk(final int n,final double [] imom,final double [] tmom,final double [] pimom,final double [] ptmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final double eps)
{
    final int nd=n-1;
    final double ra_shift=58.5221;

    final double [] QQt=M.B.getQQMatr(n,tmom),QQi=M.B.getQQMatr(n,imom),
	QQpi=M.B.getQQMatr(n,pimom),QQpt=M.B.getQQMatr(n,ptmom),
	RQQt=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(tmom)),
	RQQi=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(imom)),
	RQQpi=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(pimom)),
	RQQpt=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(ptmom)),
	RQQXt=M.B.getQQMatr(nd,M.B.getXQkMomentsFromQkMoments(M.getRadauMomentsFromMoments(tmom))),
	RQQXi=M.B.getQQMatr(nd,M.B.getXQkMomentsFromQkMoments(M.getRadauMomentsFromMoments(imom)));

    for(boolean flag_L:new boolean []{false,true})
    {
	for(int j=0;j<2;j++)
	{
	    final int nx;
	    final double [] qqi,qqt,qqpi,qqpt;
	    if(j<=0) {nx=n;qqi=QQi;qqt=QQt;qqpi=QQpi;qqpt=QQpt;}
	    else {nx=nd;qqi=RQQi;qqt=RQQt;qqpi=RQQpi;qqpt=RQQpt;}
	    final DynVarXk q=new  DynVarXk(nx,qqi,qqt,qqpi,qqpt,flag_L,M);
	    //UnitTests.checkDoubleRel(q.I_IX,"q.I_IX",q.oI_EQ,"q.oI_EQ",eps);

	    {
		final DynVarXk s=new DynVarXk(nx,TMatr.sumAcB(qqi,ra_shift*q.I_IX,qqt),qqt,
					      TMatr.sumAcB(qqpi,ra_shift*q.I_IX,qqpt),qqpt,flag_L,M);
		UnitTests.checkDoubleRel(q.p_EQ,"q.p_EQ",s.p_EQ,"s.p_EQ",eps);
		UnitTests.checkDoubleRel(q.pa_EQ,"q.pa_EQ",s.pa_EQ,"s.pa_EQ",eps);
		UnitTests.checkDoubleRel(q.pm_EQ,"q.pm_EQ",s.pm_EQ,"s.pm_EQ",eps);
		UnitTests.checkDoubleRel(q.x_EQ,"q.x_EQ",s.x_EQ,"s.x_EQ",eps);
		UnitTests.checkDoubleRel(q.xa_EQ,"q.xa_EQ",s.xa_EQ,"s.xa_EQ",eps);
		UnitTests.checkDoubleRel(q.xm_EQ,"q.xm_EQ",s.xm_EQ,"s.xm_EQ",eps);
	    }
	    if(nx==nd)
	    { // make sure l_k is the eigenfunction of x with measure dmu.
		final DynVarXk s=new DynVarXk(nx,qqi,qqt,RQQXi,RQQXt,flag_L,M);
		UnitTests.checkDoubleRel(q.x_EQ,"q.x_EQ",s.p_EQ,"s.p_EQ",eps);
		UnitTests.checkDoubleRel(q.xa_EQ,"q.xa_EQ",s.pa_EQ,"s.pa_EQ",eps);
		UnitTests.checkDoubleRel(q.xm_EQ,"q.xm_EQ",s.pm_EQ,"s.pm_EQ",eps);
	    }
	    if(nx>1)
	    {
		final DynVarXk s=new  DynVarXk(nx,qqi,qqt,qqi,qqt,flag_L,M);
		UnitTests.checkDoubleRel(s.varpIk,"s.varpIk",0,"0",eps);
		UnitTests.checkDoubleRel(s.p_EQ,"s.p_EQ",1,"1",eps);
		UnitTests.checkDoubleRel(s.pa_EQ,"s.pa_EQ",1,"1",eps);
		UnitTests.checkDoubleRel(s.pm_EQ,"s.pm_EQ",1,"1",eps);
	    }
	}
    }
    System.err.println("DynVarXk:test_DynVarXk_shift(): test OK n="+n+" eps="+eps);
}

}
