package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: DynY.java,v 1.13 2022/07/28 12:27:17 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynY {
    final EVXData I;
    final double p_IH,pd_IH,Id_IH,p_JL,r_JL,p_JH,r_JH,p_DIL,p_DIH;
    DynY(final EVXData I,
	 final double p_IH,final double pd_IH,final double Id_IH,final double p_JL,final double r_JL,final double p_JH,final double r_JH,final double p_DIL,final double p_DIH)
    {
	this.I=I;
	this.p_IH=p_IH;
	this.pd_IH=pd_IH;
	this.Id_IH=Id_IH;
	this.p_JL=p_JL;
	this.r_JL=r_JL;
	this.p_JH=p_JH;
	this.r_JH=r_JH;
	this.p_DIL=p_DIL;
	this.p_DIH=p_DIH;
    }


static DynY get_DynY(final int n,
		     final double [] QQi,final double [] QQt,final double [] QQpi,final double [] QQp2i,
		     final VolMatrix VOLATILITY_EDPsi,
		     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	// First stage: in time space.
	final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	final int iX=n-1; // maximal I
	final double [] psi=I.getPsi(iX),Dpsi=VOLATILITY_EDPsi.getDPsi(psi),
	    dpsi=TMatr.sumAcB(Dpsi,-TMatr.getAMatrB(Dpsi,psi,QQt,n),psi);

	// Second stage: in price space.
	final double dd=TMatr.getAMatrA(dpsi,QQt,n),
	    m0=I.ev[iX],m1=I.getFPsi(iX,QQpi),m2=I.getFPsi(iX,QQp2i),
	    d0=TMatr.getAMatrA(dpsi,QQi,n)/dd,d1=TMatr.getAMatrA(dpsi,QQpi,n)/dd,d2=TMatr.getAMatrA(dpsi,QQp2i,n)/dd;
	final double [] evP=M.EV.getGEV_Generalized(2,new double []{d0,d1,d1,d2},new double []{m0,m1,m1,m2},true);

	final double D0=0,D1=TMatr.getAMatrB(psi,dpsi,QQpi,n),D2=TMatr.getAMatrB(psi,dpsi,QQp2i,n);
	final double [] eDI=M.EV.getGEV_Generalized(2,new double []{D0,D1,D1,D2},new double []{m0,m1,m1,m2},true);
	if(true)
	{
	    System.err.println("DynY:<psi|dpsi>=0="+TMatr.getAMatrB(psi,dpsi,QQt,n)+" <psi|I|dpsi>=0="+TMatr.getAMatrB(psi,dpsi,QQi,n));
	    System.err.println("DynY:<jL|I|jH>=0="+(evP[2]*evP[4]*m0+m1*(evP[3]*evP[4]+evP[2]*evP[5])+m2*(evP[3]*evP[5])));
	    System.err.println("DynY:<jL|Id|jH>=0="+(evP[2]*evP[4]*d0+d1*(evP[3]*evP[4]+evP[2]*evP[5])+d2*(evP[3]*evP[5])));
	    System.err.println("DynY:<DIL|Id|DIH>=0="+(eDI[2]*eDI[4]*D0+D1*(eDI[3]*eDI[4]+eDI[2]*eDI[5])+D2*(eDI[3]*eDI[5])));
	}
	final double p_IH=m1/m0,pd_IH=d1/d0,Id_IH=d0/dd,
	    // eigenfunctions (phi_JL=evP[2]+p*evP[3] ; phi_JH=evP[4]+p*evP[5]) roots:
	    p_JL=-evP[2]/evP[3],p_JH=-evP[4]/evP[5],
	    p_DIL=-eDI[2]/eDI[3],p_DIH=-eDI[4]/eDI[5];
	return new DynY(I,p_IH,pd_IH,Id_IH,p_JL,evP[0],p_JH,evP[1],p_DIL,p_DIH);
    }

}

