package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.RayleighQuotient;

/**
 *  @version $Id: DynYp.java,v 1.22 2022/07/28 12:27:17 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynYp {
    private static final boolean FLAG_PRINT_DEBUG=true;
    final EVXData I,Ip;
    final double p_IH,pd_IH,p_IJp,pd_IJp,r_IJp,I_IJp,var_p_IH,var_pt_IJp,var_pt2_IJp,var_pp_IpH,varp_pp_IpH;
    DynYp(final EVXData I,final EVXData Ip,
	  final double p_IH,final double pd_IH,final double p_IJp,final double pd_IJp,
	  final double r_IJp,final double I_IJp,final double var_p_IH,final double var_pt_IJp,final double var_pt2_IJp,
	  final double var_pp_IpH,final double varp_pp_IpH)
    {
	this.I=I;
	this.Ip=Ip;
	this.p_IH=p_IH;
	this.pd_IH=pd_IH;
	this.p_IJp=p_IJp;
	this.pd_IJp=pd_IJp;
	this.r_IJp=r_IJp;
	this.I_IJp=I_IJp;
	this.var_p_IH=var_p_IH;
	this.var_pt_IJp=var_pt_IJp;
	this.var_pt2_IJp=var_pt2_IJp;
	this.var_pp_IpH=var_pp_IpH;
	this.varp_pp_IpH=varp_pp_IpH;
    }

private static final int N_iterations=10;

static DynYp get_DynYp(final int n,
		       final double [] QQi,final double [] QQt,final double [] QQpi,
		       final double [] QQr,final double [] QQpr,final double [] QQp2r,final double [] QQs,
		       final boolean flag_H,
		       final VolMatrix VOLATILITY_EDPsi,
		       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	if(n<2) throw new IllegalArgumentException("n="+n+"<2");

	// First stage: in time space.
	final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	final int iX=n-1; // maximal I
	final double [] psi_IH=I.getPsi(iX),Dpsi_IH=VOLATILITY_EDPsi.getDPsi(psi_IH),
	    dpsi_IH=TMatr.sumAcB(Dpsi_IH,-TMatr.getAMatrB(Dpsi_IH,psi_IH,QQt,n),psi_IH);
	final double pI_IH=I.getFPsi(iX,QQpi),p_IH=pI_IH/I.ev[iX],
	    var_p_IH=2*(I.getFPsiPsi(iX,Dpsi_IH,QQpi)-pI_IH*I.getFPsiPsi(iX,Dpsi_IH,QQt)),
	    pd_IH=TMatr.getAMatrA(dpsi_IH,QQpi,n)/TMatr.getAMatrA(dpsi_IH,QQi,n);

	// Second stage: iterate in price space.
	final int iJ=flag_H?n-1:0;
	double p_IJp=p_IH,var_pp_IpH=Double.NaN,varp_pp_IpH=Double.NaN;
	EVXData Ip=null;
	final double [] QQpx2I=new double [n*n];
	for(int it=0;it<N_iterations;it++)
	{
	    for(int k=0;k<QQpx2I.length;k++) QQpx2I[k]=p_IJp*(p_IJp*QQr[k]-2*QQpr[k])+QQp2r[k];
	    Ip=EVXData.getEVXDataFromQQ(n,QQpx2I,QQs,M);
	    if(it<=0)
	    {
		final double [] psi_IJp=Ip.getPsi(iJ),Dpsi_IJp=VOLATILITY_EDPsi.getDPsi(psi_IJp);
		final double dd=Ip.getFPsiPsi(iJ,Dpsi_IJp,QQs);
		for(int k=0;k<QQpx2I.length;k++) QQpx2I[k]=p_IJp*QQr[k]-QQpr[k];
		varp_pp_IpH=-Ip.getFPsi(iJ,QQpx2I);
		var_pp_IpH=-2*(Ip.getFPsiPsi(iJ,Dpsi_IJp,QQpx2I)-(-varp_pp_IpH)*dd);
	    }
	    p_IJp=Ip.getFPsi(iJ,QQpr)/Ip.getFPsi(iJ,QQr);
	    if(FLAG_PRINT_DEBUG)
	    {
		System.err.println("DynYp: it="+it+" p_IJp="+p_IJp+" p_IH="+p_IH+" I.ev[iX]="+I.ev[iX]+
				   " Ip(iJ)="+(Ip.getFPsi(iJ,QQi)/Ip.getFPsi(iJ,QQt))+" Ip.ev[iJ]="+Ip.ev[iJ]);
	    }
	}
	final double [] psi_IJp=Ip.getPsi(iJ),Dpsi_IJp=VOLATILITY_EDPsi.getDPsi(psi_IJp),
	    dpsi_IJp=TMatr.sumAcB(Dpsi_IJp,-TMatr.getAMatrB(Dpsi_IJp,psi_IJp,QQs,n),psi_IJp);
	final double pd_IJp=TMatr.getAMatrA(dpsi_IJp,QQpr,n)/TMatr.getAMatrA(dpsi_IJp,QQr,n);

	for(int k=0;k<QQpx2I.length;k++) QQpx2I[k]=p_IJp*QQr[k]-QQpr[k];
	final double dd=Ip.getFPsiPsi(iJ,Dpsi_IJp,QQs),
	    I_IJp=Ip.getFPsi(iJ,QQi)/Ip.getFPsi(iJ,QQt),
	    var_pt_IJp=I_IJp*(2*(Ip.getFPsiPsi(iJ,Dpsi_IJp,QQpx2I)-Ip.getFPsi(iJ,QQpx2I)*dd)),
	    var_pt2_IJp=2*TMatr.getAMatrA(dpsi_IJp,QQpx2I,n);
	if(FLAG_PRINT_DEBUG)
	{
	    System.err.println("DynYp: n="+n+" <(p-px)psi^2I>="+(p_IJp*Ip.getFPsi(iJ,QQi)-Ip.getFPsi(iJ,QQpi)));
	    System.err.println("DynYp:var_p_IH="+var_p_IH+" var_pt_IJp="+var_pt_IJp);
	}
	return new DynYp(I,Ip,p_IH,pd_IH,p_IJp,pd_IJp,Ip.ev[iJ],I_IJp,var_p_IH,var_pt_IJp,var_pt2_IJp,var_pp_IpH,varp_pp_IpH);
    }

static DynYp test_get_DynYp(
    final int n,
    final double [] QQi,final double [] QQt,final double [] QQpi,
    final double [] QQr,final double [] QQpr,final double [] QQp2r,final double [] QQs,
    final boolean flag_H,
    final VolMatrix VOLATILITY_EDPsi,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final DynVar2C d2c=DynVar2C.get_DynVar2C(n,QQi,QQt,QQpi,M,VOLATILITY_EDPsi,DynVar2C.SELECTION_IH);
    final double p_IH=d2c.p_IX,var_p_IH=d2c.var_pI_IX;
    double p_IJp=p_IH,var_pp_IpH=Double.NaN,varp_pp_IpH=Double.NaN;
    final int iJ=flag_H?n-1:0;
    final double [] QQvv=new double [n*n];
    EVXData Ip=null;
    for(int it=0;it<N_iterations;it++)
    {
	for(int k=0;k<n*n;k++) QQvv[k]=p_IJp*(p_IJp*QQr[k]-2*QQpr[k])+QQp2r[k];
	Ip=EVXData.getEVXDataFromQQ(n,QQvv,QQs,M);
	if(it<=0)
	{
	    final double [] psi=Ip.getPsi(iJ),Dpsi=VOLATILITY_EDPsi.getDPsi(psi),
		dpsi=TMatr.sumAcB(Dpsi,-TMatr.getAMatrB(Dpsi,psi,QQs,n),psi);
	    for(int k=0;k<n*n;k++) QQvv[k]=p_IJp*QQr[k]-QQpr[k];
	    final double [] rq=RayleighQuotient.getRQVariationValues(psi,dpsi,null,QQvv,QQs,n);
	    var_pp_IpH=-rq[1];
	    varp_pp_IpH=-rq[0];
	}
	p_IJp=Ip.getFPsi(iJ,QQpr)/Ip.getFPsi(iJ,QQr);
    }
    final double [] psi=Ip.getPsi(iJ),Dpsi=VOLATILITY_EDPsi.getDPsi(psi),
	dpsi=TMatr.sumAcB(Dpsi,-TMatr.getAMatrB(Dpsi,psi,QQs,n),psi);
    final double pd_IJp=TMatr.getAMatrA(dpsi,QQpr,n)/TMatr.getAMatrA(dpsi,QQr,n);
    for(int k=0;k<n*n;k++) QQvv[k]=p_IJp*QQr[k]-QQpr[k];
    final double [] rq=RayleighQuotient.getRQVariationValues(psi,dpsi,null,QQvv,QQs,n);
    final double I_IJp=Ip.getFPsi(iJ,QQi)/Ip.getFPsi(iJ,QQt),var_pt_IJp=I_IJp*rq[1],var_pt2_IJp=rq[2];
    if(FLAG_PRINT_DEBUG)
    {
	System.err.println("DynYp:Dpsi="+java.util.Arrays.toString(Dpsi));
	System.err.println("DynYp: <psi|QQvv|Dpsi>="+TMatr.getAMatrB(psi,Dpsi,QQvv,n));
	System.err.println("DynYp: <psi|S|Dpsi>="+TMatr.getAMatrB(psi,Dpsi,QQs,n));
	System.err.println("DynYp: <psi|s|dpsi>="+TMatr.getAMatrB(psi,dpsi,QQs,n));
	System.err.println("DynYp: <psi|var|Dpsi>="+TMatr.getAMatrB(psi,Dpsi,QQvv,n));
    }
    return new DynYp(d2c.I,Ip,p_IH,d2c.pdd_IX,p_IJp,pd_IJp,Ip.ev[iJ],I_IJp,var_p_IH,var_pt_IJp,var_pt2_IJp,var_pp_IpH,varp_pp_IpH);
}

static void match_DynYp(final DynYp a,final DynYp b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,eps);
    EVXData.matchEVXData(a.Ip,b.Ip,eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.pd_IH,"a.pd_IH",b.pd_IH,"b.pd_IH",eps);
    UnitTests.checkDoubleRel(a.var_p_IH,"a.var_p_IH",b.var_p_IH,"b.var_p_IH",eps);
    UnitTests.checkDoubleRel(a.p_IJp,"a.p_IJp",b.p_IJp,"b.p_IJp",eps);
    UnitTests.checkDoubleRel(a.pd_IJp,"a.pd_IJp",b.pd_IJp,"b.pd_IJp",eps);
    UnitTests.checkDoubleRel(a.r_IJp,"a.r_IJp",b.r_IJp,"b.r_IJp",eps);
    UnitTests.checkDoubleRel(a.I_IJp,"a.I_IJp",b.I_IJp,"b.I_IJp",eps);
    UnitTests.checkDoubleRel(a.var_pt_IJp,"a.var_pt_IJp",b.var_pt_IJp,"b.var_pt_IJp",eps);
    UnitTests.checkDoubleRel(a.var_pt2_IJp,"a.var_pt2_IJp",b.var_pt2_IJp,"b.var_pt2_IJp",eps);
    UnitTests.checkDoubleRel(a.var_pp_IpH,"a.var_pp_IpH",b.var_pp_IpH,"b.var_pp_IpH",eps);
    UnitTests.checkDoubleRel(a.varp_pp_IpH,"a.varp_pp_IpH",b.varp_pp_IpH,"b.varp_pp_IpH",eps);
    System.err.println("DynYp:match_DynYp test OK n="+a.I.neq+" eps="+eps);
}

}

