package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Use <psi|V|psi> along with <psi|I|psi>.
 *  @version $Id: DynpIV.java,v 1.8 2022/07/28 22:30:49 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DynpIV {
    final double [] eI,eV;
    final double I_0,I_L,I_H,p_IH,V_IH,I_V,p_V,V_V,I_IH2,p_IH2;
    DynpIV(final double [] eI,final double [] eV,
	   final double I_0,final double I_L,final double I_H,
	   final double p_IH,final double V_IH,
	   final double I_V,final double p_V,final double V_V,
	   final double I_IH2,final double p_IH2)
    {
	this.eI=eI;
	this.eV=eV;
	this.I_0=I_0;
	this.I_L=I_L;
	this.I_H=I_H;
	this.p_IH=p_IH;
	this.V_IH=V_IH;
	this.I_V=I_V;
	this.p_V=p_V;
	this.V_V=V_V;
	this.I_IH2=I_IH2;
	this.p_IH2=p_IH2;
    }

static DynpIV get0_DynpIV(final int n,final double [] imoms,final double [] tmoms,final double [] pimoms,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final DataInterfaceFDFMoments access)
{
    final int iL=1,iH=n-1,iH2=iH>iL?iH-1:iH,i0=0,nmoms=2*n-1;
    final double [] QQt=M.B.getQQMatr(n,tmoms),QQi=M.B.getQQMatr(n,imoms),Vmoms=new double [nmoms];
    access.setFMoments(Vmoms,imoms,nmoms);
    final double [] QQV=M.B.getQQMatr(n,Vmoms),QQpi=M.B.getQQMatr(n,pimoms);
    final double [] eI=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M),
	eV=EdComm.getIRadauBasisPsi0FromQQ(n,QQV,QQt,M);
    final double I_0=eI[i0],I_L=eI[iL],I_H=eI[iH],
	p_IH=TMatr.getAMatrA(n,eI,n+iH*n,QQpi,n)/eI[iH],
	V_IH=TMatr.getAMatrA(n,eI,n+iH*n,QQV,n),
	I_V=TMatr.getAMatrA(n,eV,n+iL*n,QQi,n),
	p_V=TMatr.getAMatrA(n,eV,n+iL*n,QQpi,n)/I_V,
	V_V=eV[iL],
	I_IH2=eI[iH2],
	p_IH2=TMatr.getAMatrA(n,eI,n+iH2*n,QQpi,n)/eI[iH2];
    return new DynpIV(eI,eV,I_0,I_L,I_H,p_IH,V_IH,I_V,p_V,V_V,I_IH2,p_IH2);
}

static DynpIV getR_DynpIV(final int nn,final double [] imoms,final double [] tmoms,final double [] pimoms,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final DataInterfaceFDFMoments access)
{
    final int n=nn-1,iL=0,iH=n-1,iH2=iH>iL?iH-1:iH,nmoms=2*n-1+1;
    final double [] RQQt=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(tmoms)),
	RQQi=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(imoms)),Vmoms=new double [nmoms];
    access.setFMoments(Vmoms,imoms,nmoms);
    final double [] RQQV=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(Vmoms)),
	RQQpi=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(pimoms));
    final double [] eI=M.EV.getGEV_Generalized(n,RQQi,RQQt,true),
	eV=M.EV.getGEV_Generalized(n,RQQV,RQQt,true);
    final double I_0=Double.NaN,I_L=eI[iL],I_H=eI[iH],
	p_IH=TMatr.getAMatrA(n,eI,n+iH*n,RQQpi,n)/eI[iH],
	V_IH=TMatr.getAMatrA(n,eI,n+iH*n,RQQV,n),
	I_V=TMatr.getAMatrA(n,eV,n+iL*n,RQQi,n),
	p_V=TMatr.getAMatrA(n,eV,n+iL*n,RQQpi,n)/I_V,
	V_V=eV[iL],
	I_IH2=eI[iH2],
	p_IH2=TMatr.getAMatrA(n,eI,n+iH2*n,RQQpi,n)/eI[iH2];
    if(false && true)
    {
	System.err.println("V_IH="+V_IH+" I_V="+I_V);
	System.err.println("eI="+java.util.Arrays.toString(java.util.Arrays.copyOf(eI,n)));
	System.err.println("eV="+java.util.Arrays.toString(java.util.Arrays.copyOf(eV,n)));
	System.err.println("<psiH|RQQt|psiH>="+TMatr.getAMatrA(n,eI,n+iH*n,RQQt,n)+" must be =1");
	System.err.println("<psiH|RQQi|psiH>="+TMatr.getAMatrA(n,eI,n+iH*n,RQQi,n)+" must be ="+I_H);
	System.err.println("<psiV|RQQt|psiV>="+TMatr.getAMatrA(n,eV,n+iL*n,RQQt,n)+" must be =1");
	System.err.println("<psiV|RQQV|psiV>="+TMatr.getAMatrA(n,eV,n+iL*n,RQQV,n)+" must be ="+V_V);
	System.err.println("<psiH2|RQQi|psiH2>="+TMatr.getAMatrA(n,eI,n+iH2*n,RQQi,n)+" must be ="+I_IH2);
    }
    return new DynpIV(eI,eV,I_0,I_L,I_H,p_IH,V_IH,I_V,p_V,V_V,I_IH2,p_IH2);
}

static void matchDynpIV(final DynpIV a,final DynpIV b,final double eps)
{
    UnitTests.checkDoubleRel(a.I_L,"a.I_L",b.I_L,"b.I_L",eps);
    UnitTests.checkDoubleRel(a.I_H,"a.I_H",b.I_H,"b.I_H",eps);
    UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I_0,"b.I_0",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.V_IH,"a.V_IH",b.V_IH,"b.V_IH",eps);
    UnitTests.checkDoubleRel(a.p_V,"a.p_V",b.p_V,"b.p_V",eps);
    UnitTests.checkDoubleRel(a.I_V,"a.I_V",b.I_V,"b.I_V",eps);
    UnitTests.checkDoubleRel(a.V_V,"a.V_V",b.V_V,"b.V_V",eps);
    System.err.println("DynpIV:matchDynpIV test OK eps="+eps);
}

}
