package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.Quadrature;

/** Assymetry.
 *  @version $Id: EAssym.java,v 1.30 2022/06/27 07:19:12 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class EAssym {

final double I,pe,sigma2_pe,xi3_pe,xi3,pm,sigma4_pm,sk,plow,phigh;
final Skewness SK;

EAssym(final double [] psi,final double [] p0mom,final double [] p1mom,final double [] p2mom,final double [] p3mom,final double [] p4mom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] psi2=M.B.mult2Pol(psi,psi);
    final double m0=M.B.getPQkMomentsFromQkMoments(psi2,p0mom)[0],
	m1=M.B.getPQkMomentsFromQkMoments(psi2,p1mom)[0],
	m2=M.B.getPQkMomentsFromQkMoments(psi2,p2mom)[0],
	m3=M.B.getPQkMomentsFromQkMoments(psi2,p3mom)[0],
	m4=M.B.getPQkMomentsFromQkMoments(psi2,p4mom)[0];
    this.SK=Skewness.getSkewnessFromMonomialMoments(m0,m1,m2,m3);
    this.I=m0;
    this.pe=m1/m0;
    final double pe2=Math.pow(this.pe,2);
    this.sigma2_pe=m2/m0-pe2;
    this.xi3_pe=m3/m0-this.pe*(3*this.sigma2_pe+pe2);
    this.xi3=this.xi3_pe/Math.pow(this.sigma2_pe,3.0/2.0);

    final double [] F=new double []{m4/m0,-4*m3/m0,6*m2/m0,-4*m1/m0,1},DF=Polynomials.getPolynomialDifferentiated(F);
    final com.polytechnik.utils.PolynomialRootsFinder.PRoots u=com.polytechnik.utils.PolynomialRootsElementary.ROOTSFINDER.findRoots(DF);
    final double [] xre=u.re_root,xim=u.im_root;
    int ind=-1;double vmin=Double.POSITIVE_INFINITY;
    final double saveF0=F[0];F[0]=0; // reduce rounding errors
    for(int k=0;k<u.nroots;k++)
    {
	if(false && (Double.compare(xim[k],0)!=0 || Polynomials.getD1(DF,xre[k])<0) ) continue;
	final double v=Polynomials.getD0(F,xre[k]);
	//System.err.println("DP("+xre[k]+")="+Polynomials.getD0(DF,xre[k])+" im="+xim[k]+" F(xre)="+v+" D2F="+Polynomials.getD1(DF,xre[k]));
	if(ind<0||v<vmin){vmin=v;ind=k;}
    }
    F[0]=saveF0;
    if(ind<0){this.pm=this.sigma4_pm=Double.NaN;}
    else{this.pm=xre[ind];this.sigma4_pm=Polynomials.getD0(F,this.pm);}
    //System.err.println("F="+java.util.Arrays.toString(F));

    final double [] FM=new double []{m0,m1,m2,m3,m4};
    final Quadrature QUAD=MB.getGaussQuadratureFromMoments(2,FM);
    final double [] xq=QUAD.x,wq=QUAD.w;
    this.sk=(wq[0]-wq[1])/(wq[0]+wq[1]);
    this.plow=xq[0];
    this.phigh=xq[1];
    //System.err.println("xq="+java.util.Arrays.toString(xq));
    //System.err.println("wq="+java.util.Arrays.toString(wq));

    this.spsi=psi;
    this.sp0mom=p0mom;
    this.sp1mom=p1mom;
    this.sp2mom=p2mom;
    this.sp3mom=p3mom;
    this.sp4mom=p4mom;
    this.M=M;
}
final double [] spsi,sp0mom,sp1mom,sp2mom,sp3mom,sp4mom;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
static final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> MB=com.polytechnik.utils.DiffSkewness.MB;

void match_EAssym(final EAssym q,final double eps,final boolean test_pe)
{
    UnitTests.checkDoubleRel(I,"I",q.I,"q.I",eps);
    if(test_pe){
	UnitTests.checkDoubleRel(pe,"pe",q.pe,"q.pe",eps);
	UnitTests.checkDoubleRel(pm,"pm",q.pm,"q.pm",eps);
    }
    UnitTests.checkDoubleRel(pe-pm,"pe-pm",q.pe-q.pm,"q.pe-q.pm",eps);
    UnitTests.checkDoubleRel(sigma2_pe,"sigma2_pe",q.sigma2_pe,"q.sigma2_pe",eps);
    UnitTests.checkDoubleRel(sigma4_pm,"sigma4_pm",q.sigma4_pm,"q.sigma4_pm",eps);
    UnitTests.checkDoubleRel(xi3_pe,"xi3_pe",q.xi3_pe,"q.xi3_pe",eps);
    UnitTests.checkDoubleRel(xi3,"xi3",q.xi3,"q.xi3",eps);
    UnitTests.checkDoubleRel(sk,"sk",q.sk,"q.sk",eps);
    //Skewness.matchSkewness(this.SK,q.SK,eps); offset is different, no match of average/median
    UnitTests.checkDoubleRel(SK.sk,"SK.sk",q.SK.sk,"q.SK.sk",eps);
    System.err.println("EAssym:match_EAssym: test OK n="+spsi.length+" eps="+eps);
}

void test_Self(final double eps)
{
    final int n=spsi.length;
    final double m0=TMatr.getAMatrA(spsi,M.B.getQQMatr(n,sp0mom),n),m1=TMatr.getAMatrA(spsi,M.B.getQQMatr(n,sp1mom),n),m2=TMatr.getAMatrA(spsi,M.B.getQQMatr(n,sp2mom),n),m3=TMatr.getAMatrA(spsi,M.B.getQQMatr(n,sp3mom),n),m4=TMatr.getAMatrA(spsi,M.B.getQQMatr(n,sp4mom),n);
    final double pe=m1/m0,sigma2=m2/m0-pe*pe,
	xi=(m3/m0-3*pe*(m2/m0)+2*pe*pe*pe)/Math.pow(sigma2,3.0/2.0);
    UnitTests.checkDoubleRel(xi,"xi",this.xi3,"this.xi3",eps);

    final double [] F=TMatr.scaleVectorInPlace(new double []{m4,-4*m3,6*m2,-4*m1,m0},1/m0);
    //System.err.println("tF="+java.util.Arrays.toString(F));
    UnitTests.checkDoubleRel(Polynomials.getD1(F,pm),"dF/dp|pm="+pm,0,"0",eps);
    UnitTests.checkDoubleRel(Math.signum(Polynomials.getD2(F,pm)),"sign(d^2F/dp^2|pm+"+pm+")",1,"1",eps);
    UnitTests.checkDoubleRel(Polynomials.getD0(F,pm),"F(pm="+pm+")",sigma4_pm,"sigma4_pm",eps);
    UnitTests.checkDoubleRel(Polynomials.getD1(F,pe)/(-4),"(1/-4)dF/dp|pe",xi3_pe,"xi3_pe",eps);
    UnitTests.checkDoubleRel(Polynomials.getD2(F,pe)/(4*3),"(1/12)d^2F/dp^2|pe",sigma2_pe,"sigma2_pe",eps);
    UnitTests.checkDoubleRel(Polynomials.getD2(Polynomials.getPolynomialDifferentiated(F),pe)/(-2*4*3),"(-1/24)d^3F/dp^3|pe",0,"0",eps);

    final double [] P=MB.getSingleOrthogonalPolynomial(2,new double [] {m0,m1,m2,m3});
    UnitTests.checkDoubleRel(Polynomials.getD0(P,plow),"P(plow)",0,"0",eps);
    UnitTests.checkDoubleRel(Polynomials.getD0(P,phigh),"P(phigh)",0,"0",eps);
    UnitTests.checkDoubleRel(sk,"sk",SK.sk,"SK.sk",eps);
    UnitTests.checkDoubleRel(pe,"pe",SK.xa,"SK.xa",eps);
    UnitTests.checkDoubleRel(plow,"plow",SK.x1,"SK.x1",eps);
    UnitTests.checkDoubleRel(phigh,"phigh",SK.x2,"SK.x2",eps);

    System.err.println("EAssym:test_Self: test OK n="+n+" eps="+eps);
}




} // END OF class EAssym
