package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;

/** QVM calculations.
 *  @version $Id: EQVM.java,v 1.86 2022/06/27 13:32:45 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class EQVM<DTYPE> {

public final double io,di,dip,scale_io,scale_io0,scale_ix,sigma2_eq,dp_eq,osp,tosp,scale_dip0,Si,ii,scale_dip0a,dpi_eq_scaled,scaleo_dp_eq,scaleo_io,dip_o,sigma2_dip,sigma2_dip_o,
    iplus,iminus,dix,dixp,dixpm,scale_dix,di_plus,di_minus,dip_plus,dip_minus,dpplus_o,dpminus_o,jo,scale_j0,jop,djimp,djdmdp,
    dj,djp,dpj,
    djp_sd0,dpj_sd0,dpo_sd0,io_sd0,si_sd0,ss_sd0,ii_sd0,
    scale_jo,scale_di_plus,scale_di_minus,mosp_plus,mosp_minus,tmosp_plus,tmosp_minus,dsi_eq,dsj,pm_osp,js,jdjdp,jdp_eq,j_f,j_pmosp,jbs_o,jbs_plus_o,jbs_minus_o,jbs_d2,jdjp_plus,jdjp_minus,jdjp,jdjp_d,jdjp_dd0,scale_jbs,scale_jbs0,djsdjd,sigma2_djsdjd,djsdjd_o,sigma2_djsdjd_o,scale_jsd,jsd,jsosp,djsdjd_x,jsosp_x,dpii,dpii_o,sigma2_dpii,sigma2_dpii_o,
    ca_re_pi_o,ca_im_pi_o,ca_imo_pi_o,ca_sigma2_pi_o,ca_re_pi,ca_im_pi,ca_sigma2_pi,
    cb_re_pi_o,cb_im_pi_o,cb_imo_pi_o,cb_sigma2_pi_o,cb_re_pi,cb_im_pi,cb_sigma2_pi,
    j2_bs,j2_pm,j2x_bs,j2x_pm,j2_bs_o,j2_pm_o;

final DTYPE odata;
final DeprecatedHelperDataInterface<DTYPE> access;
EQVM(final int neq_t,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess(neq_t);

    final double [] i=new double [neq_t],dpplus=new double [neq_t],dpminus=new double [neq_t],o=new double [neq_t];
    access.setM(i,0,access.M_I,odata);
    access.setM(dpplus,1,access.M_DPPLUS,odata);
    access.setM(dpminus,1,access.M_DPMINUS,odata);
    access.setM(o,0,access.M_O,odata);

    final int npd=3;
    final double [] matr=new double [npd*npd],rp=new double [npd],rps=new double [npd];
    for(int k=0;k<npd;k++)
    {
	final double [] dpk=k<=0?o:(k<=1?dpplus:dpminus);
	for(int l=0;l<=k;l++)
	{
	    final double [] dpl=l<=0?o:(l<=1?dpplus:dpminus);
	    matr[k+l*npd]=matr[l+k*npd]=access.get_f_g(dpk,dpl);
	}
	rp[k]=rps[k]=access.get_f_g(dpk,i);
    }
    final int [] ipvt=new int [npd];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,npd,npd,ipvt);
    if(pinfo!=0){java.util.Arrays.fill(rp,Double.NaN);}
    else{com.polytechnik.utils.Linsystems.dgesl_m1(matr,npd,npd,ipvt,rp,1);}

    final double o_o=access.get_o_o(),i_o=rps[0],fi_o=i_o/o_o,
	i_i=access.get_f_g(i,i),x_i_i=i_i-i_o*fi_o,
	sigma_i_i=i_i-TMatr.mulVVn(npd,rps,rp);
    this.scale_dix=Math.sqrt(o_o*sigma_i_i);

    final double dpplus_o=access.get_f_o(dpplus),dpminus_o=access.get_f_o(dpminus);
    this.dix=-rp[0]*o_o;
    this.dixp=(rp[1]+rp[2]);
    this.dixpm=(rp[1]-rp[2]);
    this.iplus=rp[1]*dpplus_o;
    this.iminus=rp[2]*dpminus_o;

    final double dpplus_dpplus=access.get_f_g(dpplus,dpplus),x_dpplus_dpplus=dpplus_dpplus-dpplus_o*(dpplus_o/o_o),
	dpminus_dpminus=access.get_f_g(dpminus,dpminus),x_dpminus_dpminus=dpminus_dpminus-dpminus_o*(dpminus_o/o_o),
	dpplus_dpminus=access.get_f_g(dpplus,dpminus),
	i_dpplus=rps[1],i_dpminus=rps[2],
	x_i_dpplus=i_dpplus-dpplus_o*fi_o,
	x_i_dpminus=i_dpminus-dpminus_o*fi_o;
    this.dip_plus=x_i_dpplus/x_dpplus_dpplus;
    this.di_plus=this.dip_plus*dpplus_o-i_o;
    this.dpplus_o=dpplus_o;
    this.scale_di_plus=Math.sqrt(o_o*(x_i_i-x_i_dpplus*(x_i_dpplus/x_dpplus_dpplus)));
    this.mosp_plus=x_i_dpplus/Math.sqrt(x_i_i*x_dpplus_dpplus);
    this.tmosp_plus=i_dpplus/Math.sqrt(i_i*dpplus_dpplus);

    this.dip_minus=x_i_dpminus/x_dpminus_dpminus;
    this.di_minus=this.dip_minus*dpminus_o-i_o;
    this.dpminus_o=dpminus_o;
    this.scale_di_minus=Math.sqrt(o_o*(x_i_i-x_i_dpminus*(x_i_dpminus/x_dpminus_dpminus)));
    this.mosp_minus=x_i_dpminus/Math.sqrt(x_i_i*x_dpminus_dpminus);
    this.tmosp_minus=i_dpminus/Math.sqrt(i_i*dpminus_dpminus);

    final double fplus=i_dpplus/dpplus_dpplus,fminus=i_dpminus/dpminus_dpminus;
    final double [] j=new double [neq_t],dp=new double [neq_t],mdp=new double [neq_t];
    for(int k=0;k<neq_t;k++){
	j[k]=fplus*dpplus[k]-fminus*dpminus[k];
	mdp[k]=dpplus[k]-dpminus[k];
    }
    access.setM(dp,1,access.M_DP,odata);
    final double dp_o=access.get_f_o(dp),j_o=access.get_f_o(j),mdp_o=access.get_f_o(mdp),
	dp_dp=access.get_f_g(dp,dp),dp_i=access.get_f_g(dp,i);
    TMatr.sumP1(dp,o,-dp_o/o_o);
    TMatr.sumP1(j,o,-j_o/o_o);
    TMatr.sumP1(i,o,-fi_o);
    TMatr.sumP1(mdp,o,-mdp_o/o_o);
    final double x_i_dp=access.get_f_g(i,dp),x_dp_dp=access.get_f_g(dp,dp);
    this.jo=j_o;
    this.scale_j0=Math.sqrt(o_o*access.get_f_g(j,j));
    this.jop=(access.get_f_g(j,dp)/x_dp_dp)*dp_o;
    this.djdmdp=(access.get_f_g(i,mdp)/access.get_f_g(mdp,mdp));
    this.djimp=this.djdmdp*mdp_o-i_o;

    this.io=i_o;
    this.scale_io0=Math.sqrt(o_o*x_i_i);
    final double sigma_ii=x_i_i-x_i_dp*(x_i_dp/x_dp_dp);
    this.scale_io=Math.sqrt(o_o*sigma_ii);
    this.scale_ix=Math.sqrt(i_o*i_o+o_o*sigma_ii);
    this.scaleo_io=Math.sqrt(o_o*i_i);
    this.scaleo_dp_eq=Math.sqrt(o_o*dp_dp);
    this.dip=(x_i_dp/x_dp_dp);
    sigma2_dip=x_i_i/x_dp_dp-dip*dip;
    this.dip_o=dp_i/dp_dp;
    sigma2_dip_o=i_i/dp_dp-dip_o*dip_o;
    dpii=x_i_dp/x_i_i;
    sigma2_dpii=x_dp_dp/x_i_i-dpii*dpii;
    dpii_o=dp_i/i_i;
    sigma2_dpii_o=dp_dp/i_i-dpii_o*dpii_o;

    this.di=this.dip*dp_o-i_o;
    this.sigma2_eq=x_dp_dp/o_o;
    this.dp_eq=dp_o;
    this.osp=-x_i_dp/Math.sqrt(x_i_i*x_dp_dp);
    this.tosp=-dp_i/Math.sqrt(i_i*dp_dp);
    this.scale_dip0=Math.sqrt(x_i_i/x_dp_dp);
    this.scale_dip0a=Math.sqrt(x_i_i/dp_dp);
    this.Si=this.dip*(this.dip*dp_dp-2*dp_i)+i_i;
    this.ii=i_i;
    this.dpi_eq_scaled=(dp_o-(x_i_dp/x_i_i)*i_o)/Math.sqrt(o_o*x_dp_dp);

    final double [] dsp=new double [neq_t];
    for(int k=0;k<neq_t;k++){dsp[k]=dpplus[k]-dpminus[k];}
    final double dsp_o=access.get_f_o(dsp);
    TMatr.sumP1(dsp,o,-dsp_o/o_o);
    final double x_i_dsp=access.get_f_g(i,dsp),x_dsp_dsp=access.get_f_g(dsp,dsp);
    this.djp=(x_i_dsp/x_dsp_dsp);
    this.dpj=(x_i_dsp/x_i_i);
    this.dj=this.djp*dsp_o-i_o;
    this.scale_jo=Math.sqrt(o_o*(x_i_i-x_i_dsp*(x_i_dsp/x_dsp_dsp)));
    this.dsi_eq=(i_dpplus/dpplus_dpplus)*dpplus_o-(i_dpminus/dpminus_dpminus)*dpminus_o;
    this.dsj=(x_i_dpplus/x_dpplus_dpplus-i_dpplus/dpplus_dpplus)*dpplus_o-
	(x_i_dpminus/x_dpminus_dpminus-i_dpminus/dpminus_dpminus)*dpminus_o;
    this.pm_osp=dpplus_dpminus/Math.sqrt(dpplus_dpplus*dpminus_dpminus);

    final double [] jplus=new double [neq_t],jminus=new double [neq_t];
    access.setM(jplus,0,access.M_ISELL/*M_IPLUS*/,odata);
    access.setM(jminus,0,access.M_IBUY/*M_IMINUS*/,odata);
    this.jbs_o=access.get_f_o(jplus)-access.get_f_o(jminus);
    this.jbs_plus_o=access.get_f_o(jplus);
    this.jbs_minus_o=access.get_f_o(jminus);
    this.jbs_d2=(jbs_plus_o*jbs_plus_o)/access.get_f_g(jplus,jplus)-
	(jbs_minus_o*jbs_minus_o)/access.get_f_g(jminus,jminus);
    if(true)
    {
	final double [] s=new double [neq_t],d=new double [neq_t];
	TMatr.sumP1(s,jplus,1);
	TMatr.sumP1(s,jminus,1);
	TMatr.sumP1(d,jplus,1);
	TMatr.sumP1(d,jminus,-1);
	this.djsdjd_x=access.get_f_g(d,s)/access.get_f_g(s,s);
	this.jsosp_x=access.get_f_g(d,s)/Math.sqrt(access.get_f_g(s,s)*access.get_f_g(d,d));
    }

    TMatr.sumP1(jplus,o,-jbs_plus_o/o_o);
    TMatr.sumP1(jminus,o,-jbs_minus_o/o_o);
    final double x_jplus_dpplus=access.get_f_g(jplus,dpplus),
	x_jminus_dpminus=access.get_f_g(jminus,dpminus);
    this.jdjp_plus=x_jplus_dpplus/x_dpplus_dpplus;
    this.jdjp_minus=x_jminus_dpminus/x_dpminus_dpminus;
    final double [] xd=new double [neq_t];
    TMatr.sumP1(xd,jplus,1);
    TMatr.sumP1(xd,jminus,-1);
    if(true)
    {
	final double [] xdp=new double [neq_t];
	access.setM(xdp,1,access.M_DP,odata);
	this.jdjp_dd0=access.get_f_g(xdp,xd)/access.get_f_g(xdp,xdp);
    }
    this.jdjp_d=access.get_f_g(dp,xd)/x_dp_dp;
    this.scale_jbs=Math.sqrt(o_o*(access.get_f_g(xd,xd)-access.get_f_g(dp,xd)*this.jdjp_d));
    this.scale_jbs0=Math.sqrt(o_o*access.get_f_g(xd,xd));
    final double [] xs=new double [neq_t];
    TMatr.sumP1(xs,jplus,1);
    TMatr.sumP1(xs,jminus,1);
    this.jdjp=access.get_f_g(dsp,xs)/x_dsp_dsp;

    final double x_xd_xs=access.get_f_g(xd,xs),
	x_xs_xs=access.get_f_g(xs,xs),x_xd_xd=access.get_f_g(xd,xd);
    this.djsdjd=x_xd_xs/x_xs_xs;
    this.sigma2_djsdjd=x_xd_xd/x_xs_xs-djsdjd*djsdjd;
    final double xs_xs=(x_xs_xs+i_o*(i_o/o_o));
    this.djsdjd_o=(x_xd_xs+i_o*(jbs_o/o_o))/xs_xs;
    this.sigma2_djsdjd_o=(x_xd_xd+jbs_o*(jbs_o/o_o))/xs_xs-djsdjd_o*djsdjd_o;

    this.jsosp=x_xd_xs/Math.sqrt(x_xs_xs*x_xd_xd);
    this.scale_jsd=Math.sqrt(o_o*(x_xd_xd-x_xd_xs*djsdjd));
    this.jsd=(jbs_plus_o-jbs_minus_o)-djsdjd*(jbs_plus_o+jbs_minus_o);


    if(true)
    {
	access.setM(i,0,access.M_I,odata);
	access.setM(dpplus,1,access.M_DPPLUS,odata);
	access.setM(dpminus,1,access.M_DPMINUS,odata);
	final double [] s=new double [neq_t],d=new double [neq_t];
	access.setM(d,1,access.M_DP,odata);
	for(int k=0;k<neq_t;k++){s[k]=dpplus[k]-dpminus[k];}
	this.djp_sd0=access.get_f_g(i,s)/access.get_f_g(s,s);
	this.dpj_sd0=access.get_f_g(i,s)/access.get_f_g(i,i);
	this.dpo_sd0=access.get_f_o(s);
	this.io_sd0=access.get_f_o(i);
	this.si_sd0=access.get_f_g(i,s);
	this.ss_sd0=access.get_f_g(s,s);
	this.ii_sd0=access.get_f_g(i,i);

	final double f=Math.sqrt(access.get_f_g(s,s)/access.get_f_g(d,d)),fp1=(f+1)/2,fm1=(f-1)/2;
	for(int k=0;k<neq_t;k++)
	{
	    final double p=dpplus[k],m=dpminus[k];
	    dpplus[k]=fp1*p+fm1*m;
	    dpminus[k]=fm1*p+fp1*m;
	    s[k]=dpplus[k]-dpminus[k];
	    d[k]=dpplus[k]+dpminus[k];
	}
	this.js=(access.get_f_g(i,dpplus)/access.get_f_g(dpplus,dpplus))*access.get_f_o(dpplus)-
	    (access.get_f_g(i,dpminus)/access.get_f_g(dpminus,dpminus))*access.get_f_o(dpminus);
	TMatr.sumP1(s,o,-access.get_f_o(s)/o_o);
	TMatr.sumP1(i,o,-access.get_f_o(i)/o_o);
	this.jdjdp=access.get_f_g(i,s)/access.get_f_g(s,s);
	this.jdp_eq=access.get_f_o(d);
	// for unit tests
	this.j_f=f;
	this.j_pmosp=access.get_f_g(dpplus,dpminus)/Math.sqrt(access.get_f_g(dpplus,dpplus)*access.get_f_g(dpminus,dpminus));
    }

    if(true)
    {
	access.setM(dpplus,1,access.M_DPPLUS,odata);
	access.setM(dpminus,1,access.M_DPMINUS,odata);
	for(int k=0;k<neq_t;k++) dpminus[k]=-dpminus[k];
	access.setM(jplus,0,access.M_ISELL/*M_IPLUS*/,odata);
	access.setM(jminus,0,access.M_IBUY/*M_IMINUS*/,odata);
	final double [] bplus=new double [neq_t],bminus=new double [neq_t];
	access.setM(bplus,0,access.M_ISELL,odata);
	access.setM(bminus,0,access.M_IBUY,odata);

	j2_bs=access.get_f_g(bplus,bplus)-access.get_f_g(bminus,bminus);
	j2_pm=access.get_f_g(jplus,jplus)-access.get_f_g(jminus,jminus);
	j2_bs_o=access.get_f_o(bplus)-access.get_f_o(bminus);
	j2_pm_o=access.get_f_o(jplus)-access.get_f_o(jminus);

	if(true)
	{
	    final double pp_pi=access.get_f_g(dpplus,jplus),
		pp_mi=access.get_f_g(dpplus,jminus),
		mp_pi=access.get_f_g(dpminus,jplus),
		mp_mi=access.get_f_g(dpminus,jminus);
	    final double np=access.get_f_g(dpplus,dpplus)+access.get_f_g(dpminus,dpminus),
		ni=access.get_f_g(jplus,jplus)+access.get_f_g(jminus,jminus);
	    ca_re_pi_o=(pp_pi+mp_mi)/np;
	    ca_im_pi_o=(pp_mi-mp_pi)/np;
	    ca_imo_pi_o=(access.get_f_o(dpplus)*access.get_f_o(jminus)-access.get_f_o(dpminus)*access.get_f_o(jplus))/o_o/np;
	    ca_sigma2_pi_o=ni/np-(ca_re_pi_o*ca_re_pi_o+ca_im_pi_o*ca_im_pi_o);
	}
	if(true)
	{
	    final double pp_pi=access.get_f_g(dpplus,bplus),
		pp_mi=access.get_f_g(dpplus,bminus),
		mp_pi=access.get_f_g(dpminus,bplus),
		mp_mi=access.get_f_g(dpminus,bminus);
	    final double np=access.get_f_g(dpplus,dpplus)+access.get_f_g(dpminus,dpminus),
		ni=access.get_f_g(bplus,bplus)+access.get_f_g(bminus,bminus);
	    cb_re_pi_o=(pp_pi+mp_mi)/np;
	    cb_im_pi_o=(pp_mi-mp_pi)/np;
	    cb_imo_pi_o=(access.get_f_o(dpplus)*access.get_f_o(bminus)-access.get_f_o(dpminus)*access.get_f_o(bplus))/o_o/np;
	    cb_sigma2_pi_o=ni/np-(cb_re_pi_o*cb_re_pi_o+cb_im_pi_o*cb_im_pi_o);
	}
	if(true)
	{
	    TMatr.sumP1(dpplus,o,-access.get_f_o(dpplus)/o_o);
	    TMatr.sumP1(dpminus,o,-access.get_f_o(dpminus)/o_o);
	    TMatr.sumP1(jplus,o,-access.get_f_o(jplus)/o_o);
	    TMatr.sumP1(jminus,o,-access.get_f_o(jminus)/o_o);
	    TMatr.sumP1(bplus,o,-access.get_f_o(bplus)/o_o);
	    TMatr.sumP1(bminus,o,-access.get_f_o(bminus)/o_o);
	    j2x_bs=access.get_f_g(bplus,bplus)-access.get_f_g(bminus,bminus);
	    j2x_pm=access.get_f_g(jplus,jplus)-access.get_f_g(jminus,jminus);
	}
	if(true)
	{
	    final double pp_pi=access.get_f_g(dpplus,jplus),
		pp_mi=access.get_f_g(dpplus,jminus),
		mp_pi=access.get_f_g(dpminus,jplus),
		mp_mi=access.get_f_g(dpminus,jminus);
	    final double np=access.get_f_g(dpplus,dpplus)+access.get_f_g(dpminus,dpminus),
		ni=access.get_f_g(jplus,jplus)+access.get_f_g(jminus,jminus);
	    ca_re_pi=(pp_pi+mp_mi)/np;
	    ca_im_pi=(pp_mi-mp_pi)/np;
	    ca_sigma2_pi=ni/np-(ca_re_pi*ca_re_pi+ca_im_pi*ca_im_pi);
	}
	if(true)
	{
	    final double pp_pi=access.get_f_g(dpplus,bplus),
		pp_mi=access.get_f_g(dpplus,bminus),
		mp_pi=access.get_f_g(dpminus,bplus),
		mp_mi=access.get_f_g(dpminus,bminus);
	    final double np=access.get_f_g(dpplus,dpplus)+access.get_f_g(dpminus,dpminus),
		ni=access.get_f_g(bplus,bplus)+access.get_f_g(bminus,bminus);
	    cb_re_pi=(pp_pi+mp_mi)/np;
	    cb_im_pi=(pp_mi-mp_pi)/np;
	    cb_sigma2_pi=ni/np-(cb_re_pi*cb_re_pi+cb_im_pi*cb_im_pi);
	}
    }
}

abstract DeprecatedHelperDataInterface<DTYPE> getAccess(final int neq_t);

void test_Consistency(final double eps,final double epssum)
{
    final int neq_t=access.get_NT();
    System.err.println("EQVM: testing consistency neq_t="+neq_t+" eps="+eps+" epssum="+epssum);
    UnitTests.checkDoubleRel(this.iplus+this.iminus-this.io,"iplus+iminus-io",this.dix,"dix",eps);
    System.err.println("0-th coef OK, neq_t="+neq_t);

    final double [] i=new double [neq_t],o=new double [neq_t],dp=new double [neq_t];
    access.setM(i,0,access.M_I,odata);
    access.setM(dp,1,access.M_DP,odata);
    access.setM(o,0,access.M_O,odata);

    final double i_i=access.get_f_g(i,i),dp_dp=access.get_f_g(dp,dp);
    final double io=access.get_f_g(i,o),dp_o=access.get_f_o(dp),o_o=access.get_f_g(o,o);
    UnitTests.checkDoubleRel(io,"io",this.io,"this.io",eps);
    UnitTests.checkDoubleRel(access.get_f_g(o,o),"access.get_f_g(o,o)",access.get_f_o(o),"access.get_f_o(o)",eps);
    UnitTests.checkDoubleRel(access.get_o_o(),"access.get_o_o()",access.get_f_o(o),"access.get_f_o(o)",eps);
    UnitTests.checkDoubleRel(access.get_f_g(i,o),"access.get_f_g(i,o)",access.get_f_o(i),"access.get_f_o(i)",eps);

    TMatr.sumP1(i,o,-io/o_o);
    final double x_i_i=access.get_f_g(i,i),scale_io0=Math.sqrt(o_o*x_i_i);
    UnitTests.checkDoubleRel(scale_io0,"scale_io0",this.scale_io0,"this.scale_io0",eps);
    System.err.println("<i|o> OK, neq_t="+neq_t);
    UnitTests.checkDoubleRel(djp,"djp",djdmdp,"djdmdp",eps);
    UnitTests.checkDoubleRel(dj,"dj",djimp,"djimp",eps);

    UnitTests.checkDoubleRel(-this.osp,"-this.osp",this.dip*(o_o*Math.sqrt(this.sigma2_eq)/this.scale_io0),"dip*scale",eps);


    UnitTests.checkDoubleRel(djp,"djp",jdjdp,"jdjdp",eps);
    UnitTests.checkDoubleRel(j_f*dp_eq,"j_f*dp_eq",jdp_eq,"jdp_eq",eps);
    UnitTests.checkDoubleRel(j_pmosp,"j_pmosp",0,"0",eps);

    UnitTests.checkDoubleRel(io,"io",jbs_plus_o+jbs_minus_o,"jbs_plus_o+jbs_minus_o",eps);
    UnitTests.checkDoubleRel(djp,"djp",jdjp,"jdjp",eps);

    TMatr.sumP1(dp,o,-dp_o/o_o);
    final double x_dp_dp=access.get_f_g(dp,dp);
    final double f=x_dp_dp/x_i_i,fo=dp_dp/i_i;
    UnitTests.checkDoubleRel(dip*f,"dip*f",dpii,"dpii",eps);
    UnitTests.checkDoubleRel(dip_o*fo,"dip_o*f_o",dpii_o,"dpii_o",eps);
    UnitTests.checkDoubleRel(sigma2_dip*f*f,"sigma2_dip*f*f",sigma2_dpii,"sigma2_dpii",eps);
    UnitTests.checkDoubleRel(sigma2_dip_o*fo*fo,"sigma2_dip_o*fo*fo",sigma2_dpii_o,"sigma2_dpii_o",eps);

    System.err.println("EQVM: testing consistency OK, neq_t="+neq_t+" eps="+eps+" epssum="+epssum);
}


} // END OF class EQVM
