package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.realtimebook.*;

/** QVM calculations.
 *  @version $Id: EQVMLegendreShifted.java,v 1.51 2019/01/06 23:50:30 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class EQVMLegendreShifted extends EQVM<QVMDataP> {

EQVMLegendreShifted(final int neq_t,final QVMDataP odata)
{
    super(neq_t,odata);
}
DeprecatedHelperQVMDataPAccess getAccess(final int neq_t){return new DeprecatedHelperQVMDataPAccess(neq_t);}

public static EQVMLegendreShifted calculate(final int neq_t,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final EQVMLegendreShifted ref)
{
    final QVMDataP o=QVMDataP.calculateQVMDataP(neq_t,
						3,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new EQVMLegendreShifted(neq_t,o);
}

private static void checkMatchingAQVM(final int neq_t,final java.util.Random r,final long currenttime,final long tau,final Trade [] tb,final int tsize,final double eps,final double epssum)
{
  if(neq_t<4)
  {
      System.err.println("EQVMLegendreShifted:checkMatchingAQVM: neq_t="+neq_t+" is too low, skipping test");
      return;
  }
  System.err.println("EQVMLegendreShifted:checkMatchingAQLM: neq_t="+neq_t+" eps="+eps+" epssum="+epssum);
  final EQVMLegendreShifted q=EQVMLegendreShifted.calculate(neq_t,currenttime,tau,tb,tsize,null);
  if(true)
  {
      final DeprecatedHelperLegendreShiftedBasis plus=new DeprecatedHelperLegendreShiftedBasis(q.odata){
	      public void setDPn(final double [] pt,final int n){
		  if(n>1) throw new RuntimeException("n="+n);
		  DeprecatedHelperLegendreShiftedBasis.semDPnInternal(pt,n,neq_t,q.odata,q.odata.pluspmoments_Ptau_x_p);
	      }};
      AQVM s=new AQVMLegendreShifted(new DeprecatedHelperLegendreShiftedBasis(q.odata),new DeprecatedHelperLegendreShiftedBasis []{plus});
      UnitTests.checkDoubleRel(q.dix,"q.dix",s.di,"s.di",epssum);
      UnitTests.checkDoubleRel(q.scale_dix,"q.scale_dix",s.scale_io,"s.scale_io",epssum);
      UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",s.scale_io0,"s.scale_io0",eps);
      s=new AQVMLegendreShifted(plus,new DeprecatedHelperLegendreShiftedBasis []{});
      UnitTests.checkDoubleRel(q.di_plus,"q.di_plus",s.di,"s.di",epssum);
      UnitTests.checkDoubleRel(q.dip_plus,"q.dip_plus",s.dip,"s.dip",epssum);
      UnitTests.checkDoubleRel(q.dpplus_o,"q.dpplus_o",s.dp_eq,"s.dp_eq",epssum);
      final DeprecatedHelperLegendreShiftedBasis minus=new DeprecatedHelperLegendreShiftedBasis(q.odata){
	      public void setDPn(final double [] pt,final int n){
		  if(n>1) throw new RuntimeException("n="+n);
		  DeprecatedHelperLegendreShiftedBasis.semDPnInternal(pt,n,neq_t,q.odata,q.odata.minuspmoments_Ptau_x_p);
	      }};
      s=new AQVMLegendreShifted(minus,new DeprecatedHelperLegendreShiftedBasis []{});
      UnitTests.checkDoubleRel(q.di_minus,"q.di_minus",s.di,"s.di",epssum);
      UnitTests.checkDoubleRel(q.dip_minus,"q.dip_minus",s.dip,"s.dip",epssum);
      UnitTests.checkDoubleRel(q.dpminus_o,"q.dpminus_o",s.dp_eq,"s.dp_eq",epssum);
  }

  System.err.println("EQVMLegendreShifted:checkMatchingAQLM: test OK, neq_t="+neq_t+" eps="+eps+" epssum="+epssum);
}
private static void checkMatchingValues(final int neq_t,final java.util.Random r,final long currenttime,final long tau,final Trade [] tb,final int tsize,final double eps,final double epssum)
{
    if(neq_t<4)
    {
	System.err.println("EQVMLegendreShifted:checkMatchingValues: neq_t="+neq_t+" is too low, skipping test");
	return;
    }
    System.err.println("EQVMLegendreShifted:checkMatchingValues: neq_t="+neq_t+" eps="+eps+" epssum="+epssum);

    final EQVMLegendreShifted q=EQVMLegendreShifted.calculate(neq_t,currenttime,tau,tb,tsize,null);
    q.test_Consistency(eps,epssum);

    if(true)
    {
	final EQVMLegendreShifted s=new EQVMLegendreShifted(neq_t,q.odata){
	   DeprecatedHelperQVMDataPAccess getAccess(final int neq_t){
	      return new DeprecatedHelperQVMDataPAccess(neq_t){
		 public void setM(final double [] m,final int n,final int mom,final QVMDataP odata)
		  {
		      if(mom==M_DPMINUS) super.setM(m,n,M_DP,odata);
		      else super.setM(m,n,mom,odata);
		  }};}};
	UnitTests.checkDoubleRel(q.dix,"q.dix",s.dix,"s.dix",epssum);
	UnitTests.checkDoubleRel(q.iplus+q.iminus,"q.iplus+q.iminus",s.iplus+s.iminus,"s.iplus+s.iminus",epssum);
	UnitTests.checkDoubleRel(q.scale_dix,"q.scale_dix",s.scale_dix,"s.scale_dix",epssum);
	UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",s.scale_io0,"s.scale_io0",eps);
    }
   if(true)
    {
	final QVMDLegendreShifted s=new QVMDLegendreShifted(new DeprecatedHelperLegendreShiftedBasis(q.odata));
	UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",s.scale_io0,"s.scale_io0",eps);
	UnitTests.checkDoubleRel(q.scale_io,"q.scale_io",s.scale_io,"s.scale_io",eps);
	UnitTests.checkDoubleRel(q.di,"q.di",s.di,"s.di",eps);
	UnitTests.checkDoubleRel(q.dip,"q.dip",s.dip,"s.dip",eps);
	UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",s.sigma2_eq,"s.sigma2_eq",eps);
	UnitTests.checkDoubleRel(q.osp,"q.osp",s.osp,"s.osp",eps);
	UnitTests.checkDoubleRel(q.scale_dip0,"q.scale_dip0",s.scale_dip0,"s.scale_dip0",eps);
	System.err.println("QVMD OK");
    }
    if(true)
    {
	final QVMPP s=new QVMPP(neq_t,q.odata,null);
	UnitTests.checkDoubleRel(q.io,"q.io",s.io,"s.io",eps);
	UnitTests.checkDoubleRel(q.scale_ix,"q.scale_ix",s.scale_ix,"s.scale_ix",eps);
	UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",s.scale_io0,"s.scale_io0",eps);
	UnitTests.checkDoubleRel(q.scaleo_io,"q.scaleo_io",s.scaleo_io,"s.scaleo_io",eps);
	UnitTests.checkDoubleRel(q.scale_io,"q.scale_io",s.scale_io1,"s.scale_io1",eps);
	UnitTests.checkDoubleRel(q.di,"q.di",s.di_1,"s.di_1",eps);
	UnitTests.checkDoubleRel(q.dip,"q.dip",s.dip_1,"s.dip_1",eps);
	UnitTests.checkDoubleRel(q.dip_o,"q.dip_o",s.dip_1o,"s.dip_1o",eps);
	UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",s.sigma2_eq,"s.sigma2_eq",eps);
	UnitTests.checkDoubleRel(q.osp,"q.osp",s.osp,"s.osp",eps);
	UnitTests.checkDoubleRel(q.scale_dip0,"q.scale_dip0",s.scale_dip0,"s.scale_dip0",eps);
	UnitTests.checkDoubleRel(q.scale_dip0a,"q.scale_dip0a",s.scale_dip0a,"s.scale_dip0a",eps);
	UnitTests.checkDoubleRel(q.dp_eq,"q.dp_eq",(s.odata.trade_last.price-s.p_eq)*q.access.get_o_o(),"(p_last-p_eq)*o_o",eps);
	UnitTests.checkDoubleRel(q.dp_eq,"q.dp_eq",s.dp_eq,"s.dp_eq",eps);
	UnitTests.checkDoubleRel(q.scaleo_dp_eq,"q.scaleo_dp_eq",s.scaleo_dp_eq,"s.scaleo_dp_eq",eps);
	UnitTests.checkDoubleRel(q.dpi_eq_scaled,"q.dpi_eq_scale",(s.odata.trade_last.price-s.pi_eq)/Math.sqrt(s.sigma2_eq),"(plast-pi_eq)/sqrt(sigma2_eq)",eps);
	//UnitTests.checkDoubleRel(q.jbs_o,"q.jbs_o",q.access.get_f_o(s.isellcoefs)-q.access.get_f_o(s.ibuycoefs),"s-b",eps);
    }
    if(true)
    {
	final EQVMLegendreShifted s=new EQVMLegendreShifted(neq_t,q.odata){
	   DeprecatedHelperQVMDataPAccess getAccess(final int neq_t){
	      return new DeprecatedHelperQVMDataPAccess(neq_t){
		 public void setM(final double [] m,final int n,final int mom,final QVMDataP odata)
		  {
		      if(mom==M_DP) {
			  final double [] tmp=new double [get_NT()];
			  super.setM(tmp,n,M_DPMINUS,odata);
			  super.setM(m,n,M_DPPLUS,odata);
			  TMatr.sumP1(m,tmp,-1);
		      }
		      else super.setM(m,n,mom,odata);
		  }};}};
	UnitTests.checkDoubleRel(q.dj,"q.dj",s.di,"s.di",epssum);
	UnitTests.checkDoubleRel(q.djp,"q.djp",s.dip,"s.dip",eps);
	UnitTests.checkDoubleRel(q.scale_jo,"q.scale_jo",s.scale_io,"s.scale_io",eps);
	UnitTests.checkDoubleRel(s.dj,"s.dj",s.di,"s.di",epssum);
	UnitTests.checkDoubleRel(s.scale_jo,"s.scale_jo",s.scale_io,"s.scale_io",eps);
    }

    System.err.println("EQVMLegendreShifted:checkMatchingValues: tests OK  neq_t="+neq_t+" eps="+eps+" epssum="+epssum);
}

private static void checkCalculations(final int tsize,final java.util.Random r,final int neq_t,final double eps,final double epssum)
{
    final Trade [] tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
    final long currenttime=tb[tb.length-1].time+1000;
    final long tau=(tb[tb.length-1].time-tb[0].time)/3;

    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println("EQVMLegendreShifted:checkCalculations(): test with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+
		       " neq_t="+neq_t);

    checkMatchingValues(neq_t,r,currenttime,tau,tb,tsize,eps,epssum);
    checkMatchingAQVM(neq_t,r,currenttime,tau,tb,tsize,eps,epssum);

    System.err.println("EQVMLegendreShifted:checkCalculations(): OK with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+
		       " neq_t="+neq_t+" eps="+eps+" epssum="+epssum);
}


public static void main(String [] args)
{
    System.err.println("EQVMLegendreShifted::main(): starting tests; ver=$Id: EQVMLegendreShifted.java,v 1.51 2019/01/06 23:50:30 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79091);
    final double eps=1e-6,epssum=1e-4;
    for(int neq_t=1;neq_t<=11;neq_t++)
    {
	for(int i=0;i<118;i++)
	{
	    checkCalculations(1010+r.nextInt(1010),r,neq_t,eps,epssum);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println("EQVMLegendreShifted::main(): tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: EQVMLegendreShifted.java,v 1.51 2019/01/06 23:50:30 mal Exp $");}

} // END OF class EQVMLegendreShifted
