package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Polynomials;

/** Assymetry.
 *  @version $Id: ETssym.java,v 1.11 2022/06/27 13:32:45 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ETssym {

final double I,pm,var,pi,pe;


ETssym(final double [] psi,final double [] imom,final double [] pimom,final double [] p0mom,final double [] p1mom,final double [] p2mom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] psi2=M.B.mult2Pol(psi,psi);
    final double m0=M.B.getPQkMomentsFromQkMoments(psi2,p0mom)[0],
	m1=M.B.getPQkMomentsFromQkMoments(psi2,p1mom)[0],
	m2=M.B.getPQkMomentsFromQkMoments(psi2,p2mom)[0],
	mi=M.B.getPQkMomentsFromQkMoments(psi2,imom)[0],
	mpi=M.B.getPQkMomentsFromQkMoments(psi2,pimom)[0];
    final double [] QQ=new double []{m0,m1,m1,m2},r=new double []{mi,mpi};
    final int np=2;
    final int [] ipvt=new int [np];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(QQ,np,np,ipvt);
    if(pinfo!=0){java.util.Arrays.fill(r,Double.NaN);}
    else {com.polytechnik.utils.Linsystems.dgesl_m1(QQ,np,np,ipvt,r,1);}
    this.I=mi/m0;
    this.var=r[1];
    this.pm=-r[1]/r[0];
    this.pi=mpi/mi;
    this.pe=m1/m0;

    this.spsi=psi;
    this.simom=imom;
    this.spimom=pimom;
    this.sp0mom=p0mom;
    this.sp1mom=p1mom;
    this.sp2mom=p2mom;
    this.M=M;
}
final double [] spsi,simom,spimom,sp0mom,sp1mom,sp2mom;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;


void match_ETssym(final ETssym q,final double eps,final boolean test_pe)
{
    UnitTests.checkDoubleRel(I,"I",q.I,"q.I",eps);
    UnitTests.checkDoubleRel(pm,"pm",q.pm,"q.pm",eps);
    UnitTests.checkDoubleRel(var,"var",q.var,"q.var",eps);
    UnitTests.checkDoubleRel(pe,"pe",q.pe,"q.pe",eps);
    UnitTests.checkDoubleRel(pi,"pi",q.pi,"q.pi",eps);

    System.err.println("ETssym:match_ETssym: test OK n="+spsi.length+" eps="+eps);
}

void test_Self(final double eps)
{
    final int n=spsi.length;
    final double m0=TMatr.getAMatrA(spsi,M.B.getQQMatr(n,sp0mom),n),m1=TMatr.getAMatrA(spsi,M.B.getQQMatr(n,sp1mom),n),m2=TMatr.getAMatrA(spsi,M.B.getQQMatr(n,sp2mom),n),mi=TMatr.getAMatrA(spsi,M.B.getQQMatr(n,simom),n),mpi=TMatr.getAMatrA(spsi,M.B.getQQMatr(n,spimom),n);

    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> b=new com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis();
    final double [][] P=b.getOrthogonalPolynomialsFirstKind(2,new double [] {m0,m1,m2});
    final double [] mm=new double []{mi,mpi};
    final double [] p0=P[0],p1=P[1];

    UnitTests.checkDoubleRel(pi,"pi",-p1[0]/p1[1],"-p1[0]/p1[1]",eps);
    UnitTests.checkDoubleRel(I,"I",TMatr.mulVVn1(p0,mm),"<p0i>",eps);
    UnitTests.checkDoubleRel(var,"var",TMatr.mulVVn1(p1,mm)/p1[1],"<p1i>",eps);

    System.err.println("ETssym:test_Self: test OK n="+n+" eps="+eps);
}




} // END OF class ETssym
