package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Time evolution as commutator acting on |psi0>.
 *  Uses finite variation and I-basis.
 *  @version $Id: EbComm.java,v 1.19 2022/11/19 11:08:57 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class EbComm {

public final double Pdir_L0,Pdir_H0,Pdir_LH,I_0,I_L,I_H,f_0,f_L,f_H,f_L0,f_H0,f_LH;

EbComm(final int n,final double [] imom,final double [] tmom,final double [] fmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");
    final double [] basis=getBasis(n,imom,tmom,M);
    // QQt is diagonal in new basis.
    final double [] QQi=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,imom),basis),
	QQf=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,fmom),basis);

    final double [] QQc=TMatr.mulMM(n,QQi,QQf);
    // Anti-Hermitian part (QQi QQf - QQf QQi) is responsible for time evolution.
    TMatr.scaleVectorInPlace(TMatr.selectAntiHermitianPartInPlace(n,QQc),2);

    final int i0=0,iL=1,iH=n-1;
    this.Pdir_L0=QQc[iL*n+i0];
    this.Pdir_H0=QQc[iH*n+i0];
    this.Pdir_LH=QQc[iL*n+iH];
    this.I_0=basis[i0];
    this.I_L=basis[iL];
    this.I_H=basis[iH];

    this.f_L0=QQf[iL*n+i0];
    this.f_H0=QQf[iH*n+i0];
    this.f_LH=QQf[iL*n+iH];
    this.f_0=QQf[i0*n+i0];
    this.f_L=QQf[iL*n+iL];
    this.f_H=QQf[iH*n+iH];

    this.simom=imom;
    this.stmom=tmom;
    this.sfmom=fmom;
    this.M=M;
}
final double [] simom,stmom,sfmom;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;

double [] getBasis(final int n,final double [] imom,final double [] tmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    return EdComm.getIRadauBasisPsi0(n,imom,tmom,M);
}


} // END OF class EbComm
