package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Time evolution as matrix element.
 *  Uses finite variation and I-basis.
 *  @version $Id: EbiComm.java,v 1.7 2022/06/27 10:50:02 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class EbiComm {

final EVXData I;
public final double ft_L0,ft_H0,ft_LH,fi_L0,fi_H0,fi_LH;

EbiComm(final int n,final double [] imom,final double [] tmom,final double [] ftmom,final double [] fimom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");
    final double [] QQt=M.B.getQQMatr(n,tmom),QQi=M.B.getQQMatr(n,imom),QQft=M.B.getQQMatr(n,ftmom),QQfi=M.B.getQQMatr(n,fimom);
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);

    final double [] basis=getBasis(I,tmom),psi0=getPsi0(I,tmom);
    final int iL=0,iH=n-1;
    this.ft_LH=TMatr.getAMatrB(n,basis,n+iL*n,basis,n+iH*n,QQft,n);
    this.fi_LH=TMatr.getAMatrB(n,basis,n+iL*n,basis,n+iH*n,QQfi,n);

    this.ft_L0=TMatr.getAMatrB(n,basis,n+iL*n,psi0,0,QQft,n);
    this.fi_L0=TMatr.getAMatrB(n,basis,n+iL*n,psi0,0,QQfi,n);

    this.ft_H0=TMatr.getAMatrB(n,basis,n+iH*n,psi0,0,QQft,n);
    this.fi_H0=TMatr.getAMatrB(n,basis,n+iH*n,psi0,0,QQfi,n);

}


static double [] getBasis(final EVXData I,final double [] tmom)
{
    final int n=I.neq;
    final double [] basis=new double [n+n*n],psi=new double [n];
    for(int k=0;k<n;k++)
    {
	basis[k]=k;
	System.arraycopy(I.ev,n+k*n,psi,0,n);
	final double w=TMatr.mulVVn1(psi,tmom);
	if(w<0) TMatr.scaleVectorInPlace(psi,-1);
	System.arraycopy(psi,0,basis,n+k*n,n);
    }
    return basis;
}
static double [] getPsi0(final EVXData I,final double [] tmom)
{
    final int n=I.neq;
    final double [] psi=new double [n];
    System.arraycopy(I.psi0,0,psi,0,n);
    final double w=TMatr.mulVVn1(psi,tmom);
    if(w<0) TMatr.scaleVectorInPlace(psi,-1);
    return psi;
}

} // END OF class EbiComm
