package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Time evolution as commutator acting on |psi0>.
 *  Users infinitesimal shits. For finite shifts see class EdComm.
 *  @version $Id: EiComm.java,v 1.26 2022/11/19 11:08:57 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class EiComm {
final EVXData I;
final double Dsens,Dsens_Rrho,Dsens_rho;

EiComm(final int n,final double [] imom,final double [] tmom,final double [] fmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] QQt=M.B.getQQMatr(n,tmom),QQi=M.B.getQQMatr(n,imom);
    this.I=isDiagQQt()?EVXData.getEVXDataFromQQfDiagQQa(n,QQi,QQt,M):
	EVXData.getEVXDataFromQQ(n,QQi,QQt,M);

    final double [] tmppsi=new double [n],
	Dpsi0B=new double [n],psi0B=new double [n];
    // QQi & QQt are diagonal in new basis.
    java.util.Arrays.fill(QQi,0);
    for(int k=0;k<n;k++){
	QQi[k*n+k]=this.I.ev[k]; // QQi in new basis is diagonal.
	System.arraycopy(this.I.ev,n+k*n,tmppsi,0,n);
	Dpsi0B[k]=M.B.getD1(tmppsi,this.I.x0);
	psi0B[k]=M.B.getD0(tmppsi,this.I.x0);
    }
    TMatr.scaleVectorInPlace(psi0B,1/Math.sqrt(TMatr.mulVV(psi0B,psi0B)));
    // Remove |psi0B> from |Dpsi0B> for consistency of normalizing.
    TMatr.sumP1(Dpsi0B,psi0B,-TMatr.mulVV(Dpsi0B,psi0B));
    // Normalize Dpsi0B.
    TMatr.scaleVectorInPlace(Dpsi0B,1/Math.sqrt(TMatr.mulVV(Dpsi0B,Dpsi0B)));

    final double [] QQf=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,fmom),this.I.ev);
    final double [] QQc=TMatr.mulMM(n,QQi,QQf);
    // Anti-Hermitian part (QQi QQf - QQf QQi) is responsible for time evolution.
    TMatr.scaleVectorInPlace(TMatr.selectAntiHermitianPartInPlace(n,QQc),2);
    // Time evolution of psi0: |varpsi0B> = QQc|psi0B>.
    final double [] varpsi0B=TMatr.getMatr_x_A(n,QQc,psi0B),varDpsi0B=TMatr.getMatr_x_A(n,QQc,Dpsi0B);
    this.Dsens=-TMatr.mulVV(varDpsi0B,psi0B); // "-" sign for transponse of QQc
    this.Dsens_Rrho=this.Dsens/Math.sqrt(TMatr.mulVV(varDpsi0B,varDpsi0B));
    this.Dsens_rho=this.Dsens/Math.sqrt(TMatr.mulVV(varpsi0B,varpsi0B));

    this.stmom=tmom;
    this.simom=imom;
    this.sfmom=fmom;
    this.M=M;
}

final double [] stmom,simom,sfmom;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;

boolean isDiagQQt(){return false;}


void test_ACorr(final double eps)
{
    final int n=this.I.neq;
    final ACorr a=ACorr.getCommutatorVarDPsiX0(n,simom,sfmom,stmom,M,this.I.x0,true);
    UnitTests.checkDoubleRel(a.x_a_b,"a.x_a_b",this.Dsens,"Dsens",eps);
    UnitTests.checkDoubleRel(a.osp,"a.osp",this.Dsens_Rrho,"Dsens_Rrho",eps);
    final ACorr b=ACorr.getCommutatorVarDPsiX0(n,simom,sfmom,stmom,M,this.I.x0,false);
    UnitTests.checkDoubleRel(b.x_a_b,"b.x_a_b",this.Dsens,"Dsens",eps);
    UnitTests.checkDoubleRel(b.osp,"b.osp",this.Dsens_rho,"Dsens_rho",eps);

    System.err.println("EiComm:test_ACorr: test OK n="+n+" eps="+eps);
}

void test_Self(final double eps)
{
    final int n=this.I.neq;
    test_ACorr(eps);
    System.err.println("EiComm:test_Common: test OK n="+n+" eps="+eps);
}

} // END OF class EiComm
