package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Time evolution as perturbation to two level model.
 *  John Pierpont Morgan is back.
 *  @version $Id: EjComm.java,v 1.16 2020/09/27 20:33:14 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class EjComm {

final double I_0,I_L,I_H;
final TSTwoLevels tL0,tH0;

EjComm(final int n,final double [] imom,final double [] tmom,final double [] fmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");
    final double [] basis=getBasis(n,imom,tmom,M);

    final int i0=0,iL=1,iH=n-1;
    this.I_0=basis[i0];
    this.I_L=basis[iL];
    this.I_H=basis[iH];

    final double [] QQi=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,imom),basis),
	QQf=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,fmom),basis);

    this.tL0=TSTwoLevels.getTSTwoLevelsDiagInd(n,QQi,QQf,M,iL,i0);
    this.tH0=TSTwoLevels.getTSTwoLevelsDiagInd(n,QQi,QQf,M,iH,i0);

    this.simom=imom;
    this.stmom=tmom;
    this.sfmom=fmom;
    this.M=M;
    this.sn=n;
}
final double [] simom,stmom,sfmom;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
final int sn;


double [] getBasis(final int n,final double [] imom,final double [] tmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    return EdComm.getIRadauBasisPsi0(n,imom,tmom,M);
}



void test_Self(final double eps)
{
    final int n=sn;
    final double [] basis=getBasis(n,simom,stmom,M);
    final int i0=0,iL=1,iH=n-1;
    this.tL0.match_Values(TSTwoLevels.test_getTSTwoLevels(simom,stmom,sfmom,M,EVXData.getPsiFromBasis(iL,basis,n),EVXData.getPsiFromBasis(i0,basis,n)),eps);
    this.tH0.match_Values(TSTwoLevels.test_getTSTwoLevels(simom,stmom,sfmom,M,EVXData.getPsiFromBasis(iH,basis,n),EVXData.getPsiFromBasis(i0,basis,n)),eps);

    System.err.println("EjComm:test_Self: test OK n="+n+" eps="+eps);
}




} // END OF class EjComm
