package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Dead John Pierpont Morgan is back.
 *  @version $Id: EjaComm.java,v 1.10 2020/09/27 20:33:14 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class EjaComm {

final EVXData I;
final double p_t,p_i,p_nt,p_ni,spuri;

EjaComm(final int n,final double [] imom,final double [] tmom,final double [] fimom,final double [] ftmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] QQi=M.B.getQQMatr(n,imom),QQt=M.B.getQQMatr(n,tmom);
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);

    // convert QQfi & QQft to new basis
    final double [] QQfi=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,fimom),this.I.ev),
	QQft=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,ftmom),this.I.ev);

    double snt=0,sni=0,d=0,sik=0,spi=0,spt=0;
    for(int k=0;k<n;k++)
    {
	final double lam=this.I.ev[k],pi=QQfi[k+n*k],pt=QQft[k+n*k];
	sni+=pi/lam;
	snt+=pt;
	d+=(pi-pt*lam);
	sik+=lam;
	spi+=pi;
	spt+=pt*lam;
    }
    this.p_nt=snt/n;
    this.p_ni=sni/n;
    this.p_t=spt/sik;
    this.p_i=spi/sik;
    this.spuri=sik;
    //System.err.println("n="+n+" spt="+spt+" spi="+spi+" sik="+sik);

    this.simom=imom;
    this.stmom=tmom;
    this.sfimom=fimom;
    this.sftmom=ftmom;
    this.M=M;
}
final double [] simom,stmom,sfimom,sftmom;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;


void test_Self(final double eps)
{
    final int n=I.neq;
    final ACorr i=ACorr.getACorr_SPUR(n,simom,stmom,stmom,M);
    UnitTests.checkDoubleRel(i.x_a_b,"i.x_a_b",spuri,"spuri",eps);

    final ACorr fi=ACorr.getACorr_SPUR(n,sfimom,stmom,stmom,M);
    UnitTests.checkDoubleRel(fi.x_a_b,"fi.x_a_b",p_i*spuri,"p_i*spuri",eps);

    final ACorr ft=ACorr.getACorr_SPUR(n,sftmom,simom,stmom,M);
    UnitTests.checkDoubleRel(ft.x_a_b,"ft.x_a_b",p_t*spuri,"p_t*spuri",eps);

    System.err.println("EjaComm:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class EjaComm
