package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.realtimebook.*;
import com.polytechnik.outils.*;

/** P&amp;L version.
 *  @version $Id: EkQVM.java,v 1.185 2022/10/30 21:29:54 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class EkQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
final EVXData IT;
final Wpvt pVT,pvt;
final WEpvt epvt,epvt_sam,epvtH_sam,epvta,epvtA,epvtaD,epvtD_sam,eRVSvt_sam,eVSvt_sam,evsvt_sam,epvtaDH,epvtDH_sam,eapvt,eapvtH_sam,eapvt_sam,eapvaL,eapvaH;
final WvarPsi wv;
final WProj WPpvt;
final W2Proj wpnlL_sam,wpnlH_sam,wpnlL,wpnlH,
    wpnlVSL_sam, wpnlVSH_sam,wpnlvsL_sam,wpnlvsH_sam;
final PNLvar pvar_L,pvar_H,svar_L,svar_H,Svar_L,Svar_H,pivar_L,pivar_H;
final PNLGvar gvar,gvar_sam;
final PNLIvar vpivar_L,vpivar_H;
final PNLMvar it_p,bs_p;
final UnrelPNLvar up,upH,upP;
final PsiAverPertr pertIT;
final PNLVolatility pnlvol,pnlvolT;
final VarItP vitp;
final RVarIpsi rpL,rpH;
final PNLIvar rp_L,rp_H;
final EjpComm jp_dptI;

EkQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();
    this.M=getABasis();

    final int nmoms=2*neq-1+1;

    final double [] tmom=new double [nmoms],imom=new double [nmoms],
	dpmom=new double [nmoms],Ptmom=new double [nmoms],
	dp2mom=new double [nmoms],P2tmom=new double [nmoms],
	tmom_sam=new double [nmoms],ptmom_sam=new double [nmoms],pimom=new double [nmoms],p2imom=new double [nmoms],
	amom=new double [nmoms],pamom=new double [nmoms];

    access.setMT(tmom,nmoms);
    access.setM(tmom_sam,0,access.M_T,odata,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(ptmom_sam,1,access.M_T,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    access.setM(p2imom,2,access.M_I,odata,nmoms);
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setM(dp2mom,2,access.M_DP,odata,nmoms);
    access.setM(amom,0,access.M_A,odata,nmoms);
    access.setM(pamom,1,access.M_A,odata,nmoms);
    access.setFMoments(Ptmom,dpmom,nmoms);
    access.setFMoments(P2tmom,dp2mom,nmoms);
    final double [] pimomo=new double [nmoms];
    final double pl_m_poffset=getDPL();
    for(int k=0;k<nmoms;k++){pimomo[k]=pimom[k]-(pl_m_poffset)*imom[k];} // adjust to p_offset=p_last

    this.vitp=new VarItP(neq,imom,tmom,pimomo,Ptmom);
    {
    final double [][] QQ=access.getQQ_Matrs(neq,1,new double [][]{tmom,imom,Ptmom});
    final EVXData j=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQ[2],QQ[0],access,getX0());
    this.rpL=new RVarIpsi(0,j,QQ[1],QQ[0],neq);
    this.rpH=new RVarIpsi(neq-1,j,QQ[1],QQ[0],neq);
    final double [] o=new double [nmoms];
    rp_L=new PNLIvar(neq,Ptmom,tmom,o,imom,EkQVM.FLAG_I_L);
    rp_H=new PNLIvar(neq,Ptmom,tmom,o,imom,EkQVM.FLAG_I_H);
    }

    final double [] vsmom=new double [nmoms],tsmom=new double [nmoms],
	VSmom=new double [nmoms],TSmom=new double [nmoms],
	rvsmom=new double [nmoms],rtsmom=new double [nmoms],
	RVSmom=new double [nmoms],RTSmom=new double [nmoms];
    access.setM(vsmom,0,access.M_VS,odata,nmoms);
    access.setM(tsmom,0,access.M_TS,odata,nmoms);
    access.setFMoments(VSmom,vsmom,nmoms);
    access.setFMoments(TSmom,tsmom,nmoms);
    access.setM(rvsmom,0,access.M_RVS,odata,nmoms);
    access.setM(rtsmom,0,access.M_RTS,odata,nmoms);
    access.setFMoments(RVSmom,rvsmom,nmoms);
    access.setFMoments(RTSmom,rtsmom,nmoms);

    final double [] Imom=getFmom(imom),Tmom=getFmom(tmom),Tmom_sam=getFmom(tmom_sam),
	PTmom_sam=getFmom(ptmom_sam),PImom=getFmom(pimom);

    final double x0=getX0();
    this.IT=DeprecatedHelperEVXData.getEVXData(neq,imom,tmom,access,x0);
    this.pertIT=new PsiAverPertr(IT.neq-1,IT.sL,1,imom,tmom,pimom,Ptmom,p2imom,P2tmom,M,isEdgeAtRight(),x0);
    this.pVT=new Wpvt(neq,tmom,Tmom_sam,Imom,PTmom_sam,PImom);
    this.pvt=new Wpvt(neq,tmom,tmom_sam,imom,ptmom_sam,pimom);
    this.epvt=new WEpvt(neq,imom,tmom,pimom,imom,ptmom_sam,tmom_sam,dpmom,tmom);
    this.epvt_sam=new WEpvt(neq,imom,tmom_sam,pimom,imom,ptmom_sam,tmom_sam,dpmom,tmom);
    this.epvtH_sam=new WEpvtH(neq,imom,tmom_sam,pimom,imom,ptmom_sam,tmom_sam,dpmom,tmom);
    this.epvta=new WEpvt(neq,imom,tmom,pimom,imom,Ptmom,tmom,dpmom,tmom);
    this.epvtA=new WEpvt(neq,imom,tmom,PImom,Imom,PTmom_sam,Tmom_sam,dpmom,tmom);
    this.epvtaD=new EDWEpvt(neq,imom,tmom,pimom,imom,Ptmom,tmom,dpmom,tmom);
    this.epvtD_sam=new EDWEpvt(neq,imom,tmom_sam,pimom,imom,ptmom_sam,tmom_sam,dpmom,tmom);
    this.epvtaDH=new EDWEpvtH(neq,imom,tmom,pimom,imom,Ptmom,tmom,dpmom,tmom);
    this.epvtDH_sam=new EDWEpvtH(neq,imom,tmom_sam,pimom,imom,ptmom_sam,tmom_sam,dpmom,tmom);
    this.eapvt=new WEpvt(neq,amom,tmom,pamom,amom,Ptmom,tmom,dpmom,tmom);
    this.eapvt_sam=new WEpvt(neq,amom,tmom_sam,pamom,amom,ptmom_sam,tmom_sam,dpmom,tmom);
    this.eapvtH_sam=new WEpvtH(neq,amom,tmom_sam,pamom,amom,ptmom_sam,tmom_sam,dpmom,tmom);
    this.eapvaL=new WEpvt(neq,imom,amom,pimom,imom,pamom,amom,dpmom,tmom);
    this.eapvaH=new WEpvtH(neq,imom,amom,pimom,imom,pamom,amom,dpmom,tmom);


    this.eRVSvt_sam=new WEpvt(neq,imom,tmom_sam,RVSmom,imom,RTSmom,tmom_sam,dpmom,tmom);
    this.eVSvt_sam=new WEpvt(neq,imom,tmom_sam,VSmom,imom,TSmom,tmom_sam,dpmom,tmom);
    this.evsvt_sam=new WEpvt(neq,imom,tmom_sam,vsmom,imom,tsmom,tmom_sam,dpmom,tmom);

    this.WPpvt=new WProj(neq,imom,tmom_sam,pimom,ptmom_sam);
    this.wpnlL_sam=new W2Proj(neq,imom,tmom_sam,pimom,ptmom_sam,EkQVM.FLAG_I_L);
    this.wpnlH_sam=new W2Proj(neq,imom,tmom_sam,pimom,ptmom_sam,EkQVM.FLAG_I_H);
    this.wpnlL=new W2Proj(neq,imom,tmom,pimom,Ptmom,EkQVM.FLAG_I_L);
    this.wpnlH=new W2Proj(neq,imom,tmom,pimom,Ptmom,EkQVM.FLAG_I_H);
    this.wpnlVSL_sam=new W2Proj(neq,imom,tmom_sam,VSmom,TSmom,EkQVM.FLAG_I_L);
    this.wpnlVSH_sam=new W2Proj(neq,imom,tmom_sam,VSmom,TSmom,EkQVM.FLAG_I_H);
    this.wpnlvsL_sam=new W2Proj(neq,imom,tmom_sam,vsmom,tsmom,EkQVM.FLAG_I_L);
    this.wpnlvsH_sam=new W2Proj(neq,imom,tmom_sam,vsmom,tsmom,EkQVM.FLAG_I_H);

    this.wv=new WvarPsi(neq,imom,tmom,pimom,imom,ptmom_sam,tmom_sam);

    this.pvar_L=new PNLvar(neq,imom,tmom_sam,pimom,ptmom_sam,EkQVM.FLAG_I_L);
    this.pvar_H=new PNLvar(neq,imom,tmom_sam,pimom,ptmom_sam,EkQVM.FLAG_I_H);
    this.pivar_L=new PNLvar(neq,imom,tmom,null,pimom,EkQVM.FLAG_I_L);
    this.pivar_H=new PNLvar(neq,imom,tmom,null,pimom,EkQVM.FLAG_I_H);
    this.svar_L=new PNLvar(neq,imom,tmom_sam,vsmom,tsmom,EkQVM.FLAG_I_L);
    this.svar_H=new PNLvar(neq,imom,tmom_sam,vsmom,tsmom,EkQVM.FLAG_I_H);
    this.Svar_L=new PNLvar(neq,imom,tmom_sam,VSmom,TSmom,EkQVM.FLAG_I_L);
    this.Svar_H=new PNLvar(neq,imom,tmom_sam,VSmom,TSmom,EkQVM.FLAG_I_H);
    this.gvar_sam=new PNLGvar(neq,imom,tmom_sam,pimom,ptmom_sam,dpmom);
    this.gvar=new PNLGvar(neq,imom,tmom,pimom,Ptmom,dpmom);
    this.vpivar_L=new PNLIvar(neq,imom,tmom,pimom,p2imom,EkQVM.FLAG_I_L);
    this.vpivar_H=new PNLIvar(neq,imom,tmom,pimom,p2imom,EkQVM.FLAG_I_H);

    final double [] vbuymom=new double [nmoms],vsellmom=new double [nmoms],
	pvbuymom=new double [nmoms],pvsellmom=new double [nmoms];

    access.setM(vbuymom,0,access.M_IBUY,odata,nmoms);
    access.setM(vsellmom,0,access.M_ISELL,odata,nmoms);
    access.setM(pvbuymom,1,access.M_IBUY,odata,nmoms);
    access.setM(pvsellmom,1,access.M_ISELL,odata,nmoms);

    this.it_p=new PNLMvar(neq,imom,tmom,pvbuymom,vbuymom,pvsellmom,vsellmom);
    this.bs_p=new PNLMvar(neq,vbuymom,vsellmom,pvbuymom,vbuymom,pvsellmom,vsellmom);
    this.up=new UnrelPNLvar(neq,imom,pimom,tmom,pvbuymom,vbuymom,pvsellmom,vsellmom);
    this.upH=new UnrelPNLvar(neq,imom,pimom,tmom,pvbuymom,vbuymom,pvsellmom,vsellmom){
	    @Override
	    double [] getDpsi(final double x){return this.I.psiH;}
	};
    final double []
	aplus=new double [nmoms],aminus=new double [nmoms],
	paplus=new double [nmoms],paminus=new double [nmoms];
    access.setM(aplus,1,access.M_DPPLUS,odata,nmoms);
    access.setM(aminus,1,access.M_DPMINUS,odata,nmoms);
    access.setM(paplus,2,access.M_DPPLUS,odata,nmoms);
    access.setM(paminus,2,access.M_DPMINUS,odata,nmoms);
    for(int k=nmoms;--k>=0;)
    {
	aminus[k]=-aminus[k];
	paplus[k]/=2;
	paminus[k]/=-2;
    }
    this.upP=new UnrelPNLvar(neq,amom,pamom,tmom,paminus,aminus,paplus,aplus);

    this.pnlvol=new PNLVolatility(neq,imom,tmom,pimom,Ptmom,VOLATILITY_EDPsi);
    this.pnlvolT=new PNLVolatility(neq,imom,tmom,pimom,Ptmom,VOLATILITY_DiffT);
    this.jp_dptI=new EjpComm(neq,imom,tmom,dpmom,M);
}


double [] getFmom(final double [] moms)
{
    final double [] fmoms=new double [moms.length];
    access.setFMoments(fmoms,moms,fmoms.length);
    for(int k=fmoms.length;--k>=0;){fmoms[k]=-fmoms[k];}
    return fmoms;
}


final GetVolMatrix VOLATILITY_EDPsi=new GetVolMatrix(){
	private final VolMatrix y=new VolMatrix(){
		@Override
		public double [] getDPsi(final double [] psi){return getEDPsi(psi);}
	    };
	public VolMatrix getVolatility(final double x){return y;}
    },
    VOLATILITY_DiffT=new GetVolMatrix(){
public VolMatrix getVolatility(final double x){
    return new VolMatrix(){
	@Override
	public double [] getDPsi(final double [] psi){
	    final double [] pp=getEDPsi(psi);
	    TMatr.sumP1(pp,psi,-0.5);
	    return pp;
	}
    };
}
};

abstract double [] getEDPsi(final double [] psi);
abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getX0();
abstract double getPOffset();
abstract double getPLast();
abstract boolean isEdgeAtRight();
abstract TwoCloseRootsFinder.RootsData execTR(final PTwoRoots tr);
abstract double [] getCoefsForPNLInDPspace(final double [] psi);
abstract long getVolume();


/** Unrealized P&L version.
 */
class UnrelPNLvar {
final EVXData I;
    final double fXx,Il,Isell,Ibuy,pIsell,pIbuy,PNL,dI,PNLl,pIl;
UnrelPNLvar(final int n,final double [] imom,final double [] pimom,final double [] tmom,
	    final double [] pbuymoma,final double [] buymoma,
	    final double [] psellmoma,final double [] sellmoma)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,pimom,tmom,pbuymoma,buymoma,psellmoma,sellmoma});
    final double [] QQi=QQ[0],QQpi=QQ[1],QQt=QQ[2],QQpbuy=QQ[3],QQbuy=QQ[4],QQpsell=QQ[5],QQsell=QQ[6];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);

    final double [][] r=DkQVM.getRoots(this.I.psiL,M,isEdgeAtRight());
    final double [] Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0_NonSortedX(Xx,x0,isEdgeAtRight());
    this.fXx=indX>=0?Xx[indX]:Double.NaN;

    final double [] psi=getDpsi(this.fXx);
    final double nt=TMatr.getAMatrA(psi.length,psi,QQt,n);
    this.Il=TMatr.getAMatrA(psi.length,psi,QQi,n)/nt;
    this.Isell=TMatr.getAMatrA(psi.length,psi,QQsell,n)/nt;
    this.Ibuy=TMatr.getAMatrA(psi.length,psi,QQbuy,n)/nt;
    this.pIsell=TMatr.getAMatrA(psi.length,psi,QQpsell,n)/nt;
    this.pIbuy=TMatr.getAMatrA(psi.length,psi,QQpbuy,n)/nt;

    this.pIl=TMatr.getAMatrA(psi.length,psi,QQpi,n)/nt;

    final double dpl=getDPL();
    this.dI=this.Ibuy-this.Isell;
    this.PNL=-((this.pIbuy-this.pIsell)-dpl*this.dI);
    this.PNLl=-(this.pIl-dpl*this.Il);
}


double [] getDpsi(final double x)
{
    final double [] psi=this.I.psiL;
    return M.B.sdiv1(psi,x).q;
}

void test_Self(final double eps)
{
    final int n=this.I.neq;

    UnitTests.checkDoubleRel(Il,"Il",Isell+Ibuy,"Isell+Ibuy",eps);

    System.err.println("EkQVM:UnrelPNLvar:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class UnrelPNLvar

double getDPL(){return getPLast()-getPOffset();}


class PNLVolatility {
final EVXData D;
final double Pt_L,I_L,Pi_L,dP_L,Pt_H,I_H,Pi_H,dP_H;
PNLVolatility(final int n,final double [] imom,final double [] tmom,
	      final double [] pimom,final double [] ptmom,final GetVolMatrix getvolatility)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,pimom,ptmom});
    final double x0=getX0();
    final double [] QQi=QQ[0],QQt=QQ[1],QQpi=QQ[2],QQpt=QQ[3],DQQD=getvolatility.getVolatility(x0).getVolMatrix(n,QQt);
    this.D=DeprecatedHelperEVXData.getEVXDataFromQQ(n,DQQD,QQt,access,x0);

    this.Pt_L=TMatr.getAMatrA(D.psiL.length,D.psiL,QQpt,n);
    this.I_L=TMatr.getAMatrA(D.psiL.length,D.psiL,QQi,n);
    this.Pi_L=TMatr.getAMatrA(D.psiL.length,D.psiL,QQpi,n)/I_L;
    {final int ind_lev=0;
    final double [] psipert=DeprecatedHelperEVXData.getPsiPertr(D,ind_lev,QQi);
    this.dP_L=D.getFPsiPsi(ind_lev,psipert,QQpt);
    }

    this.Pt_H=TMatr.getAMatrA(D.psiH.length,D.psiH,QQpt,n);
    this.I_H=TMatr.getAMatrA(D.psiH.length,D.psiH,QQi,n);
    this.Pi_H=TMatr.getAMatrA(D.psiH.length,D.psiH,QQpi,n)/I_H;
    {final int ind_lev=n-1;
    final double [] psipert=DeprecatedHelperEVXData.getPsiPertr(D,ind_lev,QQi);
    this.dP_H=D.getFPsiPsi(ind_lev,psipert,QQpt);
    }
    stmom=tmom;
    simom=imom;
    sptmom=ptmom;
    spimom=pimom;
}
final double [] stmom,simom,sptmom,spimom;

void test_Self(final double eps)
{
    final int n=this.D.neq;
    System.err.println("PNLVolatility:DQQD=EV*QQt:EV="+java.util.Arrays.toString(D.getEValues()));

    System.err.println("PNLVolatility:psiL="+java.util.Arrays.toString(D.psiL));
    double [][] r=DkQVM.getRoots(D.psiL,M,isEdgeAtRight());
    if(r==null)  System.err.println("psiL:No Roots");
    else System.err.println("Re(psiL)="+java.util.Arrays.toString(r[0])+"\tIm(psiL)="+java.util.Arrays.toString(r[1]));

    System.err.println("PNLVolatility:psiH="+java.util.Arrays.toString(D.psiH));
    r=DkQVM.getRoots(D.psiH,M,isEdgeAtRight());
    if(r==null)  System.err.println("psiH:No Roots");
    else System.err.println("Re(psiH)="+java.util.Arrays.toString(r[0])+"\tIm(psiH)="+java.util.Arrays.toString(r[1]));

    final double [] QQi=M.B.getQQMatr(n,simom),QQpt=M.B.getQQMatr(n,sptmom);
    UnitTests.checkDoubleRel(dP_L,"dP_L",DeprecatedHelperEVXData.getPertr1Term(D,0,QQi,QQpt),"D.getPertr1Term",eps);
    UnitTests.checkDoubleRel(dP_H,"dP_H",DeprecatedHelperEVXData.getPertr1Term(D,n-1,QQi,QQpt),"D.getPertr1Term",eps);

    System.err.println("EkQVM:PNLVolatility:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class PNLVolatility



class PNLMvar {
final EVXData I;
final double pa_buy_L,pa_buy_H,pa_sell_L,pa_sell_H;
PNLMvar(final int n,final double [] imom,final double [] tmom,
	final double [] pbuymoma,final double [] buymoma,
	final double [] psellmoma,final double [] sellmoma)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,pbuymoma,buymoma,psellmoma,sellmoma});
    final double [] QQi=QQ[0],QQt=QQ[1],QQpbuy=QQ[2],QQbuy=QQ[3],QQpsell=QQ[4],QQsell=QQ[5];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);

    this.pa_buy_L=TMatr.getAMatrA(I.psiL.length,I.psiL,QQpbuy,n)/TMatr.getAMatrA(I.psiL.length,I.psiL,QQbuy,n);
    this.pa_buy_H=TMatr.getAMatrA(I.psiH.length,I.psiH,QQpbuy,n)/TMatr.getAMatrA(I.psiH.length,I.psiH,QQbuy,n);
    this.pa_sell_L=TMatr.getAMatrA(I.psiL.length,I.psiL,QQpsell,n)/TMatr.getAMatrA(I.psiL.length,I.psiL,QQsell,n);
    this.pa_sell_H=TMatr.getAMatrA(I.psiH.length,I.psiH,QQpsell,n)/TMatr.getAMatrA(I.psiH.length,I.psiH,QQsell,n);

}

} // END OF class PNLMvar

class PNLGvar {
final EVXData I;
final double pt,pto,pnlt,pi,pio,pnli,ptd,ptdo;
PNLGvar(final int n,final double [] imom,final double [] tmom,
	final double [] pimom,final double [] ptmom,final double [] dpmom)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,pimom,ptmom});
    final double [] QQi=QQ[0],QQt=QQ[1],QQpi=QQ[2],QQpt=QQ[3];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);

    final double [] psi=this.I.psiL,psio=this.I.psiH;

    final double [][] QQr=access.getQQ_Matrs(n,1,new double [][]{
	    M.getRadauMomentsFromMoments(imom),
	    M.getRadauMomentsFromMoments(tmom),
	    M.getRadauMomentsFromMoments(pimom),
	    M.getRadauMomentsFromMoments(ptmom)});
    final double [] QQri=QQr[0],QQrt=QQr[1],QQrpi=QQr[2],QQrpt=QQr[3];
    {
    final double [] m=getPP(psi,psio,QQt,QQrt,QQpt,QQrpt,n);
    pt=m[0];
    pto=m[1];
    pnlt=m[2];
    }
    if(dpmom!=null)
    {
	final double [] st=getCoefsForPNLInDPspace(psi),sto=getCoefsForPNLInDPspace(psio);
	ptd=TMatr.mulVVn1(st,dpmom);
	ptdo=TMatr.mulVVn1(sto,dpmom);
    }
    else
    {
	ptd=ptdo=Double.NaN;
    }
    {
    final double [] m=getPP(psi,psio,QQi,QQri,QQpi,QQrpi,n);
    pi=m[0];
    pio=m[1];
    pnli=m[2];
    }
    this.simom=imom;
    this.stmom=tmom;
    this.spimom=pimom;
    this.sptmom=ptmom;
    this.sdpmom=dpmom;
}
final double [] simom,stmom,spimom,sptmom,sdpmom;


void test_Self(final double eps)
{
    final int n=this.I.neq;
    {
    final WEpvt q=new WEpvt(n,simom,stmom,spimom,simom,sptmom,stmom,sdpmom,stmom);
    UnitTests.checkDoubleRel(pt,"pt",q.Ptb,"q.Ptb",eps);
    UnitTests.checkDoubleRel(pto,"pto",q.Pto,"q.Pto",eps);
    UnitTests.checkDoubleRel(pi,"pi",q.Pib,"q.Pib",eps);
    UnitTests.checkDoubleRel(pio,"pio",q.Pio,"q.Pio",eps);
    }
    {
    final PNLMvar q=new PNLMvar(n,simom,stmom,spimom,simom,sptmom,stmom);
    UnitTests.checkDoubleRel(q.pa_buy_L,"q.pa_buy_L",pi,"pi",eps);
    UnitTests.checkDoubleRel(q.pa_buy_H,"q.pa_buy_H",pio,"pio",eps);
    UnitTests.checkDoubleRel(q.pa_sell_L,"q.pa_sell_L",pt,"pt",eps);
    UnitTests.checkDoubleRel(q.pa_sell_H,"q.pa_sell_H",pto,"pto",eps);
    }
    if(false){
	final PNLGvar q=new PNLGvar(n,simom,stmom,spimom,sptmom,null);
    UnitTests.checkDoubleRel(q.pnlt,"q.pnlt",0,"0",eps);
    UnitTests.checkDoubleRel(q.pnli,"q.pnli",0,"0",eps);
    }
    System.err.println("EkQVM:PNLGvar:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class PNLGvar
static double [] getPP(final double [] psi,final double [] psio,final double [] QQt,final double [] QQrt,final double [] QQpt,final double [] QQrpt,final int ldQQ)
{
    final double nt=TMatr.getAMatrA(psi.length,psi,QQt,ldQQ),nto=TMatr.getAMatrA(psio.length,psio,QQt,ldQQ),
	pt=TMatr.getAMatrA(psi.length,psi,QQpt,ldQQ)/nt,pto=TMatr.getAMatrA(psio.length,psio,QQpt,ldQQ)/nto,
	rt=TMatr.getAMatrA(psi.length,psi,QQrt,ldQQ)/nt,rto=TMatr.getAMatrA(psio.length,psio,QQrt,ldQQ)/nto,
	rpt=TMatr.getAMatrA(psi.length,psi,QQrpt,ldQQ)/nt,rpto=TMatr.getAMatrA(psio.length,psio,QQrpt,ldQQ)/nto;
    final double pnl=rpt-pto*rt-pt*rto+rpto;
    return new double []{pt,pto,pnl};
}



class PNLvar {
final EVXData I;
final int ind_lev;
final double [] g;
final double dpnl;

PNLvar(final int n,final double [] imom,final double [] tmom,
       final double [] pimom,final double [] ptmom,
       final int flag_lh)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom});
    final double [] QQi=QQ[0],QQt=QQ[1];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);

    final double [] psi;
    if(flag_lh==FLAG_I_L){psi=this.I.psiL;this.ind_lev=0;}
    else if(flag_lh==FLAG_I_H){psi=this.I.psiH;this.ind_lev=n-1;}
    else throw new RuntimeException("flag_lh="+flag_lh);

    final double d0, d1[]=new double [n], d2[]=new double [n*n];
    if(true)
    {
	final double [] QQpi=pimom==null?null:access.getQQ_Matrs(n,1,new double [][]{pimom})[0],
	    QQpt=ptmom==null?null:access.getQQ_Matrs(n,1,new double [][]{ptmom})[0];

	final RayleighQuotient.RQVariationInMatrixForm
	    pnli=QQpi==null?null:new RayleighQuotient.RQVariationInMatrixForm(psi,n,QQpi,QQi,n),
	    pnlt=QQpt==null?null:new RayleighQuotient.RQVariationInMatrixForm(psi,n,QQpt,QQt,n);
	d0=(pnli==null?0:pnli.d0)-(pnlt==null?0:pnlt.d0);
	for(int k=d1.length;--k>=0;){d1[k]=(pnli==null?0:pnli.d1[k])-(pnlt==null?0:pnlt.d1[k]);}
	for(int k=d2.length;--k>=0;){d2[k]=(pnli==null?0:pnli.d2[k])-(pnlt==null?0:pnlt.d2[k]);}
    }
    final double [] basis=RVarIpsi.getBasisCond(this.ind_lev,this.I);
    final int nd=basis.length/n;
    final double [] D1=new double [nd],D2=new double [nd*nd],tmp=new double [n*nd];

    // Convert d1 & d2 to new basis.
    // D2=basis d2 basis^T
    TMatr.multiply_MA_x_MBTrans(tmp,n,d2,n,basis,nd); // tmp: n x nd
    TMatr.multiply_MA_x_MB(D2,nd,basis,n,tmp,nd); // D2: nd x nd
    // D1=d1 basis^T
    TMatr.multiply_MA_x_MBTrans(D1,1,d1,n,basis,nd); // D1: 1 x nd

    // find G=-D2^{-1} D1 and dpnl=-0.5<G|D2|G>=0.5*<G|D1>
    {
	final double [] DD2=D2.clone(),G=new double [nd];
	final int [] ipvt=new int [nd];
	final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(DD2,nd,nd,ipvt);
	if(pinfo!=0) {
	    java.util.Arrays.fill(G,Double.NaN);
	} else {
	    for(int k=D1.length;--k>=0;){G[k]=-D1[k];}
	    com.polytechnik.utils.Linsystems.dgesl_m1(DD2,nd,nd,ipvt,G,1);
	}
	this.dpnl=-0.5*TMatr.getAMatrA(nd,G,D2,nd);
	this.g=new double [n];
	// g=G basis
	TMatr.multiply_MA_x_MB(g,1,G,nd,basis,n);
    }
    this.sd0=d0;
    this.sd1=d1;
    this.sd2=d2;
    this.simom=imom;
    this.stmom=tmom;
    this.spimom=pimom;
    this.sptmom=ptmom;
}
final double sd0, sd1[],sd2[];
final double [] simom,stmom,spimom,sptmom;

void test_Self(final double eps)
{
    final int n=this.I.neq;
    final double bdpnl=-0.5*TMatr.getAMatrA(g.length,g,sd2,n);
    UnitTests.checkDoubleRel(dpnl,"dpnl",bdpnl,"bdpnl",eps);
    final double [] psi=new double [n];
    System.arraycopy(this.I.ev,n+ind_lev*n,psi,0,n);
    final double [] QQi=M.B.getQQMatr(n,simom),QQt=M.B.getQQMatr(n,stmom);
    final double [] mi=spimom==null?new double [3]:RayleighQuotient.getRQVariationValues(psi,g,null,M.B.getQQMatr(n,spimom),QQi,n),
	mt=sptmom==null?new double [3]:RayleighQuotient.getRQVariationValues(psi,g,null,M.B.getQQMatr(n,sptmom),QQt,n);
    final double f0=mi[0]-mt[0],f1=mi[1]-mt[1],f2=mi[2]-mt[2];
    UnitTests.checkDoubleRel(f0,"f0",sd0,"sd0",eps);
    UnitTests.checkDoubleRel(f1,"f1",TMatr.mulVVn1(sd1,g),"<d1|g>",eps);
    UnitTests.checkDoubleRel(f2,"f2",TMatr.getAMatrA(g.length,g,sd2,n),"<g|d2|g>",eps);
    // extremum condition
    UnitTests.checkDoubleRel(-2*0.5*f2,"-2*0.5*f2",f1,"f1",eps);
    // orthogonality condition
    UnitTests.checkDoubleRel(TMatr.getAMatrB(g.length,g,0,psi.length,psi,0,QQt,n),"<g|QQt|psi>",0,"0",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(g.length,g,0,psi.length,psi,0,QQi,n),"<g|QQi|psi>",0,"0",eps);
    // diag
    System.err.println("psi="+java.util.Arrays.toString(psi));
    System.err.println("g="+java.util.Arrays.toString(g));

    System.err.println("EkQVM:PNLvar:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class PNLvar



class PNLIvar {
final EVXData I;
final int ind_lev;
final double [] g;
final double dvol,varpdvol;

PNLIvar(final int n,final double [] imom,final double [] tmom,
       final double [] pimom,final double [] p2imom,
       final int flag_lh)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,pimom});
    final double [] QQi=QQ[0],QQt=QQ[1],QQpi=QQ[2];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);

    final double [] psi;
    if(flag_lh==FLAG_I_L){psi=this.I.psiL;this.ind_lev=0;}
    else if(flag_lh==FLAG_I_H){psi=this.I.psiH;this.ind_lev=n-1;}
    else throw new RuntimeException("flag_lh="+flag_lh);

    final double pm=TMatr.getAMatrA(psi.length,psi,QQpi,n)/TMatr.getAMatrA(psi.length,psi,QQi,n);

    final int nmoms=2*n-1;
    final double [] f0mom=new double [nmoms],f1mom=new double [nmoms],f2mom=new double [nmoms];
    for(int k=nmoms;--k>=0;)
    {
	f2mom[k]=p2imom[k]+pm*(-2*pimom[k]+pm*imom[k]);
	f1mom[k]=pimom[k]-pm*imom[k];
	f0mom[k]=imom[k];
    }
    final double [][] QQf=access.getQQ_Matrs(n,1,new double [][]{f0mom,f1mom,f2mom});
    final double [] QQf0=QQf[0],QQf1=QQf[1],QQf2=QQf[2];
    final RayleighQuotient.RQVariationInMatrixForm
	r=new RayleighQuotient.RQVariationInMatrixForm(psi,n,QQf2,QQt,n),
	r1=new RayleighQuotient.RQVariationInMatrixForm(psi,n,QQf1,QQt,n);
    final double d0=r.d0,d1[]=r.d1,d2[]=r.d2;

    final double [] basis=RVarIpsi.getBasisCond(this.ind_lev,this.I);
    final int nd=basis.length/n;
    final double [] D1=new double [nd],D2=new double [nd*nd],tmp=new double [n*nd];

    // Convert d1 & d2 to new basis.
    // D2=basis d2 basis^T
    TMatr.multiply_MA_x_MBTrans(tmp,n,d2,n,basis,nd); // tmp: n x nd
    TMatr.multiply_MA_x_MB(D2,nd,basis,n,tmp,nd); // D2: nd x nd
    // D1=d1 basis^T
    TMatr.multiply_MA_x_MBTrans(D1,1,d1,n,basis,nd); // D1: 1 x nd

    // find G=-D2^{-1} D1 and dvol=-0.5<G|D2|G>=0.5*<G|D1>
    {
	final double [] DD2=D2.clone(),G=new double [nd];
	final int [] ipvt=new int [nd];
	final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(DD2,nd,nd,ipvt);
	if(pinfo!=0) {
	    java.util.Arrays.fill(G,Double.NaN);
	} else {
	    for(int k=D1.length;--k>=0;){G[k]=-D1[k];}
	    com.polytechnik.utils.Linsystems.dgesl_m1(DD2,nd,nd,ipvt,G,1);
	}
	this.dvol=-0.5*TMatr.getAMatrA(nd,G,D2,nd);
	this.g=new double [n];
	// g=G basis
	TMatr.multiply_MA_x_MB(g,1,G,nd,basis,n);
	this.varpdvol=r1.d0+TMatr.mulVVn1(g,r1.d1)+0.5*TMatr.getAMatrA(n,g,r1.d2,n);
    }
    this.sd0=d0;
    this.sd1=d1;
    this.sd2=d2;
    this.simom=imom;
    this.stmom=tmom;
    this.spimom=pimom;
    this.sp2imom=p2imom;
    sr1=r1;
}
final double sd0, sd1[],sd2[];
final double [] simom,stmom,spimom,sp2imom;
final RayleighQuotient.RQVariationInMatrixForm sr1;


void test_Self(final double eps)
{
    final int n=this.I.neq;
    final double bdvol=-0.5*TMatr.getAMatrA(g.length,g,sd2,n);
    UnitTests.checkDoubleRel(dvol,"dvol",bdvol,"bdvol",eps);
    UnitTests.checkDoubleRel(sr1.d0,"sr1.d0",0,"0",eps);
    //UnitTests.checkDoubleRel(-2*0.5*TMatr.getAMatrA(n,g,sr1.d2,n),"-<g|r1.d2|g>",TMatr.mulVVn1(g,sr1.d1),"<g|r1.d1>",eps);

    final double [] psi=new double [n];
    System.arraycopy(this.I.ev,n+ind_lev*n,psi,0,n);
    final double [] QQpi=M.B.getQQMatr(n,spimom),QQt=M.B.getQQMatr(n,stmom),QQi=M.B.getQQMatr(n,simom);
    final double pm=TMatr.getAMatrA(psi.length,psi,QQpi,n)/TMatr.getAMatrA(psi.length,psi,QQi,n);
    final double [] f2mom=new double [sp2imom.length],f1mom=new double [spimom.length];
    for(int k=f2mom.length;--k>=0;) {
	f2mom[k]=sp2imom[k]+pm*(-2*spimom[k]+pm*simom[k]);
	f1mom[k]=spimom[k]-pm*simom[k];
    }
    final double [] QQf2=M.B.getQQMatr(n,f2mom),QQf1=M.B.getQQMatr(n,f1mom);

    final double [] m=RayleighQuotient.getRQVariationValues(psi,g,null,QQf2,QQt,n);
    final double f0=m[0],f1=m[1],f2=m[2];
    UnitTests.checkDoubleRel(f0,"f0",sd0,"sd0",eps);
    UnitTests.checkDoubleRel(f1,"f1",TMatr.mulVVn1(g,sd1),"<d1|g>",eps);
    UnitTests.checkDoubleRel(f2,"f2",TMatr.getAMatrA(g.length,g,sd2,n),"<g|d2|g>",eps);
    final double [] m1=RayleighQuotient.getRQVariationValues(psi,g,null,QQf1,QQt,n);
    UnitTests.checkDoubleRel(m1[0]+m1[1]+m1[2]/2,"var",varpdvol,"varpdvol",eps);
    // extremum condition
    UnitTests.checkDoubleRel(-2*0.5*f2,"-2*0.5*f2",f1,"f1",eps);
    // orthogonality condition
    UnitTests.checkDoubleRel(TMatr.getAMatrB(g.length,g,0,psi.length,psi,0,QQt,n),"<g|QQt|psi>",0,"0",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(g.length,g,0,psi.length,psi,0,QQi,n),"<g|QQi|psi>",0,"0",eps);
    // diag
    System.err.println("psi="+java.util.Arrays.toString(psi));
    System.err.println("g="+java.util.Arrays.toString(g));

    System.err.println("EkQVM:PNLIvar:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class PNLIvar



class Wpvt {
final int n;
final TwoCloseRootsFinder.RootsData trd;
Wpvt(final int n,final double [] bmom,
     final double [] tmom,final double [] imom,
     final double [] ptmom,final double [] pimom)
{
    this.n=n;
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{bmom,tmom,imom,ptmom,pimom});
    final double [] QQb=QQ[0],QQt=QQ[1],QQi=QQ[2],QQpt=QQ[3],QQpi=QQ[4],
	IQQ=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQb);
    final double [] tmp=new double [n*n];
    final double [] KQQt=DkQVM.getDTxAxD(n,IQQ,QQt,tmp),KQQpt=DkQVM.getDTxAxD(n,IQQ,QQpt,tmp),
	KQQi=DkQVM.getDTxAxD(n,IQQ,QQi,tmp),KQQpi=DkQVM.getDTxAxD(n,IQQ,QQpi,tmp);
    final double [][] kkk=access.getKK(n,new double [][] {KQQt,KQQpt,KQQi,KQQpi});

    final PTwoRoots tr=new PTwoRoots(kkk[0],kkk[2],kkk[1],kkk[3],access);
    this.trd=execTR(tr);
}

void test_Self(final double eps)
{
    System.err.println("EkQVM:Wpvt:test_Self: test OK n="+n+" eps="+eps);
}
} // END OF class Wpvt


class WEpvt {
final EVXData I,PmuT;
final double fXx,dgi,dgt,g,Fg,sigma2_g,pmu,sigma2_pmu;
final double d0f,d1f,d2f,d2fc;
final double Ptb,Pto,Ptd,Pib,Pio,Pid;
final double d0i,d1i,d2i,d2ic;
final double gd,ipnl,g_PL,ipnl_PL,dg_PL,dgo_PL,dP_PL,g_PH,ipnl_PH,dg_PH,dgo_PH,dP_PH;
final double Mu,pMu,xpMu,l0;
final PsiAverPertr pert;
final double dPto,dPtb,dPtd;

WEpvt(final int n,final double [] imom,final double [] tmom,
      final double [] pimoma,final double [] imoma,
      final double [] ptmoma,final double [] tmoma,final double [] dpdtmom,final double [] tmomd)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,pimoma,imoma,ptmoma,tmoma,dpdtmom,tmomd});
    final double [] QQi=QQ[0],QQt=QQ[1],QQpia=QQ[2],QQia=QQ[3],QQpta=QQ[4],QQta=QQ[5],QQdpt=QQ[6],QQtd=QQ[7];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);

    final double [] psi,psio;
    final double lam,sign;
    if(flag_psiH())
    {
	psi=this.I.psiH;
	psio=this.I.psiL;
	lam=this.I.sH;
	sign=-1;
    }
    else
    {
	psi=this.I.psiL;
	psio=this.I.psiH;
	lam=this.I.sL;
	sign=1;
    }
    final double [][] r=DkQVM.getRoots(psi,M,isEdgeAtRight());
    final double [] Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0_NonSortedX(Xx,x0,isEdgeAtRight());
    this.fXx=indX>=0?Xx[indX]:Double.NaN;
    final double [] Dpsi=getDpsi(psi,this.fXx),D2psi=getD2psi(psi,Dpsi,this.fXx);

    this.l0=access.fx(Dpsi,this.fXx);
    this.Mu=(TMatr.getAMatrA(Dpsi.length,Dpsi,QQia,n)-lam*TMatr.getAMatrA(Dpsi.length,Dpsi,QQta,n))/(l0*l0);
    this.pMu=(TMatr.getAMatrA(Dpsi.length,Dpsi,QQpia,n)-lam*TMatr.getAMatrA(Dpsi.length,Dpsi,QQpta,n))/(l0*l0);
    this.xpMu=(TMatr.getAMatrB(psi.length,psi,0,Dpsi.length,Dpsi,0,QQpia,n)-lam*TMatr.getAMatrB(psi.length,psi,0,Dpsi.length,Dpsi,0,QQpta,n))/(l0*l0);


    final double [] mpi=DkQVM.getfDDx(psi,Dpsi,D2psi,QQpia,QQia,n),
	mpt=DkQVM.getfDDx(psi,Dpsi,D2psi,QQpta,QQta,n),
	mit=DkQVM.getfDDx(psi,Dpsi,D2psi,QQi,QQt,n);
    final double nt=TMatr.getAMatrA(psi.length,psi,QQta,n),
	ni=TMatr.getAMatrA(psi.length,psi,QQia,n);
    this.Ptb=TMatr.getAMatrA(psi.length,psi,QQpta,n)/nt;
    this.Pto=TMatr.getAMatrA(psio.length,psio,QQpta,n)/TMatr.getAMatrA(psio.length,psio,QQta,n);
    this.Ptd=TMatr.getAMatrA(Dpsi.length,Dpsi,QQpta,n)/TMatr.getAMatrA(Dpsi.length,Dpsi,QQta,n);

    this.dPtb=TMatr.getAMatrA(psi.length,psi,QQdpt,n)/TMatr.getAMatrA(psio.length,psio,QQtd,n);
    this.dPto=TMatr.getAMatrA(psio.length,psio,QQdpt,n)/TMatr.getAMatrA(psio.length,psio,QQtd,n);
    this.dPtd=TMatr.getAMatrA(Dpsi.length,Dpsi,QQdpt,n)/TMatr.getAMatrA(Dpsi.length,Dpsi,QQtd,n);

    this.Pib=TMatr.getAMatrA(psi.length,psi,QQpia,n)/ni;
    this.Pio=TMatr.getAMatrA(psio.length,psio,QQpia,n)/TMatr.getAMatrA(psio.length,psio,QQia,n);
    this.Pid=TMatr.getAMatrA(Dpsi.length,Dpsi,QQpia,n)/TMatr.getAMatrA(Dpsi.length,Dpsi,QQia,n);
    this.dgi=-2*TMatr.getAMatrB(psi.length,psi,0,Dpsi.length,Dpsi,0,QQia,n)/ni;
    this.dgt=-2*TMatr.getAMatrB(psi.length,psi,0,Dpsi.length,Dpsi,0,QQta,n)/nt;

    this.d0f=mpi[0]-mpt[0];
    this.d1f=mpi[1]-mpt[1];
    this.d2f=mpi[2]-mpt[2];
    this.d2fc=mpi[3]-mpt[3];

    final double nnt=TMatr.getAMatrA(psi.length,psi,QQt,n);
    final double [] QQpmu=new double [n*n];
    for(int k=QQpmu.length;--k>=0;) {QQpmu[k]=QQpia[k]-lam*QQpta[k];}
    this.g=TMatr.getAMatrB(psi.length,psi,0,Dpsi.length,Dpsi,0,QQt,n)/nnt;
    this.pmu=TMatr.getAMatrA(psi.length,psi,QQpmu,n)/nnt;
    this.Fg=TMatr.getAMatrB(psi.length,psi,0,Dpsi.length,Dpsi,0,QQpmu,n)/nnt-this.g*pmu;
    this.sigma2_g=TMatr.getAMatrA(Dpsi.length,Dpsi,QQt,n)/nnt-this.g*this.g;
    final double [] QQpmu2=DkQVM.getDTxAxD(n,QQpmu,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),new double [n*n]);
    this.sigma2_pmu=TMatr.getAMatrA(psi.length,psi,QQpmu2,n)/nnt-pmu*pmu;
    {
    final double [] rr=getGG(psi,Dpsi,QQpmu,QQt,n);
    this.gd=rr[0];
    this.ipnl=rr[1];
    }

    this.PmuT=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpmu,QQt,access,x0);
    {
    final double [] rr=getGG(psi,PmuT.psiL,QQpmu,QQt,n);
    g_PL=rr[0];
    ipnl_PL=rr[1]; //Math.pow(g_PL,2)*(PmuT.sL-pmu);
    dg_PL=TMatr.getAMatrB(psi.length,psi,0,PmuT.psiL.length,PmuT.psiL,0,QQt,n)/nnt;
    dgo_PL=TMatr.getAMatrB(psio.length,psio,0,PmuT.psiL.length,PmuT.psiL,0,QQt,n)/nnt;
    dP_PL=PmuT.sL-pmu;
    }
    {
    final double [] rr=getGG(psi,PmuT.psiH,QQpmu,QQt,n);
    g_PH=rr[0];
    ipnl_PH=rr[1];
    dg_PH=TMatr.getAMatrB(psi.length,psi,0,PmuT.psiH.length,PmuT.psiH,0,QQt,n)/nnt;
    dgo_PH=TMatr.getAMatrB(psio.length,psio,0,PmuT.psiH.length,PmuT.psiH,0,QQt,n)/nnt;
    dP_PH=PmuT.sH-pmu;
    }

    final double [] mpit=DkQVM.getfDDx(psi,Dpsi,D2psi,QQpia,QQt,n);
    this.d0i=mpit[0];
    this.d1i=mpit[1];
    this.d2i=mpit[2];
    this.d2ic=mpit[3];

    this.pert=new PsiAverPertr(n-1,lam,sign,imoma,tmoma,pimoma,ptmoma,null,null,M,isEdgeAtRight(),x0);

    spi=mpi;
    spt=mpt;
    sit=mit;
}
boolean flag_psiH()
{
    return false;
}
/** Scale g roughly to [-1:1] interval.
 */
double getGScaled()
{
    return this.g*(2/this.I.o_o)*2;
}
final double [] spi,spt,sit;
double [] getDpsi(final double [] psi,final double x)
{
    return M.B.sdiv1(psi,x).q;
}
double [] getD2psi(final double [] psi,final double [] Dpsi,final double x)
{
    return null;
}

void test_DkQVM_WkkPsi(final double [] ds,final DkQVM.WkkPsi s,final double eps)
{
    UnitTests.checkDoubleRel(fXx,"fXx",s.fXx,"s.fXx",eps);
    UnitTests.checkDoubleRel(g,"g",s.g,"s.g",eps);
    //UnitTests.checkDoubleRel(sigma2_g,"sigma2_g",s.sigma2_g,"s.sigma2_g",eps);
    UnitTests.checkDoubleRel(ds[0],"ds[0]",s.Pf/s.Ipsi,"s.Pf/s.Ipsi",eps);
    UnitTests.checkDoubleRel(ds[1],"ds[1]",-2*s.dPf/s.Ipsi,"-2*s.dPf/s.Ipsi",eps);
    if(s.I2!=0 && !Double.isNaN(s.I2))UnitTests.checkDoubleRel(ds[2],"ds[2]",(s.I2/s.Ipsi)*(s.PI2-s.Pf/s.Ipsi),"(s.I2/s.Ipsi)*(s.PI2-s.Pf/s.Ipsi)",eps);
    if(s.I2c!=0 && !Double.isNaN(s.I2c))UnitTests.checkDoubleRel(ds[3],"ds[3]",(s.I2c/s.Ipsi)*(s.PI2c-s.Pf/s.Ipsi),"(s.I2c/s.Ipsi)*(s.PI2c-s.Pf/s.Ipsi)",eps);

    System.err.println("EkQVM:WEpvt:test_DkQVM_WkkPsi: test OK n="+this.I.neq+" eps="+eps);
}
void test_DkQVM_WkePsi(final double [] ds,final DkQVM.WkePsi s,final double eps)
{
    UnitTests.checkDoubleRel(g,"g",s.g,"s.g",eps);
    //UnitTests.checkDoubleRel(sigma2_g,"sigma2_g",s.sigma2_g,"s.sigma2_g",eps);
    UnitTests.checkDoubleRel(ds[0],"ds[0]",s.Pf/s.Ipsi,"s.Pf/s.Ipsi",eps);
    UnitTests.checkDoubleRel(ds[1],"ds[1]",-2*s.dPf/s.Ipsi,"-2*s.dPf/s.Ipsi",eps);

    System.err.println("EkQVM:WEpvt:test_DkQVM_WkePsi: test OK n="+this.I.neq+" eps="+eps);
}
void test_DkQVM_WmuPsi(final DkQVM.WmuPsi s,final double eps)
{
    UnitTests.checkDoubleRel(fXx,"fXx",s.jL.fXx,"s.jL.fXx",eps);
    UnitTests.checkDoubleRel(g,"g",s.g_t,"s.g_t",eps);
    UnitTests.checkDoubleRel(sigma2_g,"sigma2_g",s.sigma2_g_t,"s.sigma2_g_t",eps);
    UnitTests.checkDoubleRel(sigma2_pmu,"sigma2_pmu",s.sigma2_Pf,"s.sigma2_Pf",eps);
    UnitTests.checkDoubleRel(g,"g",s.g_i,"s.g_i",eps);
    UnitTests.checkDoubleRel(Fg,"Fg",s.Fg,"s.Fg",eps);
    UnitTests.checkDoubleRel(d1f*I.sL/-2,"d1f*I.sL/-2",s.Fg,"s.Fg",eps);
    // When built on an eigenfunction these must match exactly.
    UnitTests.checkDoubleRel(d2f-d2fc,"d2f-d2fc",4*d1f*g,"4*d1f*g",eps);
    UnitTests.checkDoubleRel(d0f,"d0f",Pib-Ptb,"Pib-Ptb",eps);

    System.err.println("EkQVM:WEpvt:test_DkQVM_WmuPsi: test OK n="+this.I.neq+" eps="+eps);
}

void test_Self(final double eps)
{
    UnitTests.checkDoubleRel(d0f,"d0f",Pib-Ptb,"Pib-Ptb",eps);
    UnitTests.checkDoubleRel(g,"g",gd,"gd",eps);
    UnitTests.checkDoubleRel(Fg*g,"Fg*g",ipnl,"ipnl",eps);
    UnitTests.checkDoubleRel(ipnl_PL,"ipnl_PL",Math.pow(g_PL,2)*dP_PL,"g^2(PL-pmu)",eps);
    UnitTests.checkDoubleRel(ipnl_PH,"ipnl_PH",Math.pow(g_PH,2)*dP_PH,"g^2(PH-pmu)",eps);
    UnitTests.checkDoubleRel(Math.abs(dg_PL),"|dg_PL|",Math.abs(g_PL),"|g_PL|",eps);
    UnitTests.checkDoubleRel(Math.abs(dg_PH),"|dg_PH|",Math.abs(g_PH),"|g_PH|",eps);

    System.err.println("EkQVM:WEpvt:Pid="+Pid+" pert.Pi="+pert.Pi);
    //UnitTests.checkDoubleRel(pert.pertp/pert.fact,"pert.pertp/pert.fact",Pid-Ptd,"Pid-Ptd",eps);

    System.err.println("EkQVM:WEpvt:test_Self: test OK n="+this.I.neq+" eps="+eps);
}
} // END OF class WEpvt

class WEpvtH extends WEpvt {
WEpvtH(final int n,final double [] imom,final double [] tmom,
	final double [] pimoma,final double [] imoma,
       final double [] ptmoma,final double [] tmoma,final double [] dpmom,final double [] tmomd)
{
    super(n,imom,tmom,pimoma,imoma,ptmoma,tmoma,dpmom,tmomd);
}
@Override
boolean flag_psiH(){return true;}
} // END OF class WEpvtH



static double [] getGG(final double [] psi,final double [] Dpsi,final double [] QQpmu,final double [] QQt,final int ldQQ)
{
    final double nt=TMatr.getAMatrA(psi.length,psi,QQt,ldQQ),
	g=TMatr.getAMatrB(psi.length,psi,0,Dpsi.length,Dpsi,0,QQt,ldQQ)/nt,
	pmu=TMatr.getAMatrA(psi.length,psi,QQpmu,ldQQ),
	p_n_l_mul_I=g*(TMatr.getAMatrB(psi.length,psi,0,Dpsi.length,Dpsi,0,QQpmu,ldQQ)-pmu*g);
    return new double []{g,p_n_l_mul_I};
}


class EDWEpvt extends WEpvt {
EDWEpvt(final int n,final double [] imom,final double [] tmom,
	final double [] pimoma,final double [] imoma,
	final double [] ptmoma,final double [] tmoma,final double [] dpmom,final double [] tmomd)
{
    super(n,imom,tmom,pimoma,imoma,ptmoma,tmoma,dpmom,tmomd);
}
@Override
double getGScaled(){
	//(this.g*(2/this.I.o_o))*2-1;
    final double fn=access.fx(getDpsi(this.I.psi0,this.fXx),getX0())/access.fx(this.I.psi0,getX0());
    return (this.g*fn)*2-1;
}
@Override
double [] getDpsi(final double [] psi,final double x){return getEDPsi(psi);}
@Override
double [] getD2psi(final double [] psi,final double [] Dpsi,final double x){return getEDPsi(Dpsi);}
} // END OF class EDWEpvt

class EDWEpvtH extends EDWEpvt {
EDWEpvtH(final int n,final double [] imom,final double [] tmom,
	final double [] pimoma,final double [] imoma,
	 final double [] ptmoma,final double [] tmoma,final double [] dpmom,final double [] tmomd)
{
    super(n,imom,tmom,pimoma,imoma,ptmoma,tmoma,dpmom,tmomd);
}
@Override
boolean flag_psiH(){return true;}
} // END OF class EDWEpvtH





class WProj {
final EVXData I;
final double pmu_iL,pmu_iH,
    g_iL_PL,P_iL_PL,g_iL_PH,P_iL_PH,
    g_iL_PLprev,g_iL_PHprev,
    I_iL_PL,I_iL_PH,w_iL_PL,w_iL_PH,
    g0_iL_PL,g0_iL_PH,
    go_iL_PL,go_iL_PH,
    g_iH_PL,g_iH_PH,P_iH_PL,P_iH_PH,
    g_iH_PLprev,g_iH_PHprev,
    I_iH_PL,I_iH_PH,w_iH_PL,w_iH_PH,
    g0_iH_PL,g0_iH_PH,
    go_iH_PL,go_iH_PH;
final int in_iL_PL,in_iL_PH,in_iH_PL,in_iH_PH;

WProj(final int n,final double [] imom,final double [] tmom,
      final double [] pimoma,final double [] ptmoma)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,pimoma,ptmoma});
    final double [] QQi=QQ[0],QQt=QQ[1],QQpia=QQ[2],QQpta=QQ[3];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);
    final double [] QQpmu=new double [QQpia.length];
    if(true)
    {
    final double [] psi=this.I.psiL,psio=this.I.psiH,psi0=this.I.psi0;
    for(int k=QQpmu.length;--k>=0;) {QQpmu[k]=QQpia[k]-this.I.sL*QQpta[k];}
    final EVXData Pmu=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpmu,QQt,access,x0);
    final double pmu=this.pmu_iL=TMatr.getAMatrA(psi.length,psi,QQpmu,n);
    this.g_iL_PL=TMatr.getAMatrB(psi.length,psi,0,Pmu.psiL.length,Pmu.psiL,0,QQt,n);
    this.g_iL_PLprev=TMatr.getAMatrB(psi.length,psi,0,n,Pmu.ev,n+n*(n>1?1:0),QQt,n);
    this.P_iL_PL=Pmu.sL-pmu;
    this.g_iL_PH=TMatr.getAMatrB(psi.length,psi,0,Pmu.psiH.length,Pmu.psiH,0,QQt,n);
    this.g_iL_PHprev=TMatr.getAMatrB(psi.length,psi,0,n,Pmu.ev,n+n*(n>1?n-2:n-1),QQt,n);
    this.P_iL_PH=Pmu.sH-pmu;
    this.I_iL_PL=TMatr.getAMatrA(Pmu.psiL.length,Pmu.psiL,QQi,n);
    this.I_iL_PH=TMatr.getAMatrA(Pmu.psiH.length,Pmu.psiH,QQi,n);

    this.go_iL_PL=TMatr.getAMatrB(psio.length,psio,0,Pmu.psiL.length,Pmu.psiL,0,QQt,n);
    this.go_iL_PH=TMatr.getAMatrB(psio.length,psio,0,Pmu.psiL.length,Pmu.psiH,0,QQt,n);
    this.g0_iL_PL=TMatr.getAMatrB(psi0.length,psi0,0,Pmu.psiL.length,Pmu.psiL,0,QQt,n);
    this.g0_iL_PH=TMatr.getAMatrB(psi0.length,psi0,0,Pmu.psiL.length,Pmu.psiH,0,QQt,n);
    double sl=0,sh=0;
    int off_l=-1,off_h=-1;
    for(int k=0;k<n;k++)
    {
	final double wk=TMatr.getAMatrB(psi.length,psi,0,n,Pmu.ev,n+n*k,QQt,n),ek=Pmu.ev[k];
	if(ek<pmu) {sl=Math.abs(wk);off_l=k;}
	if(ek>pmu && sh<=0) {sh=Math.abs(wk);off_h=k;}
    }
    this.w_iL_PL=sl;
    this.in_iL_PL=off_l;
    this.w_iL_PH=sh;
    this.in_iL_PH=off_h;
    }
    if(true)
    {
    final double [] psi=this.I.psiH,psio=this.I.psiL,psi0=this.I.psi0;
    for(int k=QQpmu.length;--k>=0;) {QQpmu[k]=QQpia[k]-this.I.sH*QQpta[k];}
    final EVXData Pmu=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpmu,QQt,access,x0);
    final double pmu=this.pmu_iH=TMatr.getAMatrA(psi.length,psi,QQpmu,n);
    this.g_iH_PL=TMatr.getAMatrB(psi.length,psi,0,Pmu.psiL.length,Pmu.psiL,0,QQt,n);
    this.g_iH_PLprev=TMatr.getAMatrB(psi.length,psi,0,n,Pmu.ev,n+n*(n>1?1:0),QQt,n);
    this.P_iH_PL=Pmu.sL-pmu;
    this.g_iH_PH=TMatr.getAMatrB(psi.length,psi,0,Pmu.psiH.length,Pmu.psiH,0,QQt,n);
    this.g_iH_PHprev=TMatr.getAMatrB(psi.length,psi,0,n,Pmu.ev,n+n*(n>1?n-2:n-1),QQt,n);
    this.P_iH_PH=Pmu.sH-pmu;
    this.I_iH_PL=TMatr.getAMatrA(Pmu.psiL.length,Pmu.psiL,QQi,n);
    this.I_iH_PH=TMatr.getAMatrA(Pmu.psiH.length,Pmu.psiH,QQi,n);

    this.go_iH_PL=TMatr.getAMatrB(psio.length,psio,0,Pmu.psiL.length,Pmu.psiL,0,QQt,n);
    this.go_iH_PH=TMatr.getAMatrB(psio.length,psio,0,Pmu.psiH.length,Pmu.psiH,0,QQt,n);
    this.g0_iH_PL=TMatr.getAMatrB(psi0.length,psi0,0,Pmu.psiL.length,Pmu.psiL,0,QQt,n);
    this.g0_iH_PH=TMatr.getAMatrB(psi0.length,psi0,0,Pmu.psiH.length,Pmu.psiH,0,QQt,n);
    double sl=0,sh=0;
    int off_l=-1,off_h=-1;
    for(int k=0;k<n;k++)
    {
	final double wk=TMatr.getAMatrB(psi.length,psi,0,n,Pmu.ev,n+n*k,QQt,n),ek=Pmu.ev[k];
	if(ek<pmu) {sl=Math.abs(wk);off_l=k;}
	if(ek>pmu && sh<=0) {sh=Math.abs(wk);off_h=k;}
    }
    this.w_iH_PL=sl;
    this.in_iH_PL=off_l;
    this.w_iH_PH=sh;
    this.in_iH_PH=off_h;
    }
    simom=imom;
    stmom=tmom;
    spimom=pimoma;
    sptmom=ptmoma;
}
final double [] simom,stmom,spimom,sptmom;

void test_WEpvt(final WEpvt s,final boolean test_iL,final double eps)
{
    if(test_iL)
    {
	UnitTests.checkDoubleRel(pmu_iL,"pmu_iL",s.pmu,"s.pmu",eps);
	UnitTests.checkDoubleRel(Math.abs(g_iL_PL),"|g_iL_PL|",Math.abs(s.g_PL),"|s.g_PL|",eps);
	UnitTests.checkDoubleRel(Math.abs(g_iL_PH),"|g_iL_PH|",Math.abs(s.g_PH),"|s.g_PH|",eps);
	UnitTests.checkDoubleRel(Math.abs(go_iL_PL),"|go_iL_PL|",Math.abs(s.dgo_PL),"|s.dgo_PL|",eps);
	UnitTests.checkDoubleRel(Math.abs(go_iL_PH),"|go_iL_PH|",Math.abs(s.dgo_PH),"|s.dgo_PH|",eps);
	UnitTests.checkDoubleRel(P_iL_PL,"P_iL_PL",s.dP_PL,"s.dP_PL",eps);
	UnitTests.checkDoubleRel(P_iL_PH,"P_iL_PH",s.dP_PH,"s.dP_PH",eps);
    }
    else
    {
	UnitTests.checkDoubleRel(pmu_iH,"pmu_iH",s.pmu,"s.pmu",eps);
	UnitTests.checkDoubleRel(Math.abs(g_iH_PL),"|g_iH_PL|",Math.abs(s.g_PL),"|s.g_PL|",eps);
	UnitTests.checkDoubleRel(Math.abs(g_iH_PH),"|g_iH_PH|",Math.abs(s.g_PH),"|s.g_PH|",eps);
	UnitTests.checkDoubleRel(Math.abs(go_iH_PL),"|go_iH_PL|",Math.abs(s.dgo_PL),"|s.dgo_PL|",eps);
	UnitTests.checkDoubleRel(Math.abs(go_iH_PH),"|go_iH_PH|",Math.abs(s.dgo_PH),"|s.dgo_PH|",eps);
	UnitTests.checkDoubleRel(P_iH_PL,"P_iH_PL",s.dP_PL,"s.dP_PL",eps);
	UnitTests.checkDoubleRel(P_iH_PH,"P_iH_PH",s.dP_PH,"s.dP_PH",eps);
    }
    System.err.println("EkQVM:WProj:test_WEpvt: test OK n="+this.I.neq+" test_iL="+test_iL+" eps="+eps);
}

void test_Self(final double eps)
{
    WEpvt s=new WEpvt(I.neq,simom,stmom,spimom,simom,sptmom,stmom,sptmom,stmom);
    test_WEpvt(s,true,eps);
    s=new WEpvtH(I.neq,simom,stmom,spimom,simom,sptmom,stmom,sptmom,stmom);
    test_WEpvt(s,false,eps);
    //UnitTests.checkDoubleRel(w_iL_PL+w_iL_PH,"w_iL_PL+w_iL_PH",1,"1",eps);
    //UnitTests.checkDoubleRel(w_iH_PL+w_iH_PH,"w_iH_PL+w_iH_PH",1,"1",eps);

    System.err.println("EkQVM:WProj:test_Self: test OK n="+this.I.neq+" eps="+eps);
}
} // END OF class WProj





class VarItP {
final EVXData P1t;
final double I_PL,I_PH,varI_PL,varI_PH;
final double VT_PL,VT_PH,varVT_PL,varVT_PH;
final double varVP,avarVP,Pt,Pv;
final double PT0_L,PV0_L,PT0_H,PV0_H;

VarItP(final int n,final double [] imom,final double [] tmom,final double [] pimom,final double [] ptmom)
{
    final double [] Vmom=new double [imom.length],Tmom=new double [tmom.length],
	pVmom=new double [imom.length],pTmom=new double [ptmom.length];
    access.setFMoments(Vmom,imom,Vmom.length);
    access.setFMoments(Tmom,tmom,Tmom.length);
    access.setFMoments(pVmom,pimom,pVmom.length);
    access.setFMoments(pTmom,ptmom,pTmom.length);
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,ptmom,Vmom,Tmom,pVmom,pTmom});
    final double [] QQi=QQ[0],QQt=QQ[1],QQp=QQ[2],QQV=QQ[3],QQT=QQ[4],QQpV=QQ[5],QQpT=QQ[6];
    final double x0=getX0();
    this.P1t=DeprecatedHelperEVXData.getEVXData(n,ptmom,tmom,access,x0);
    final double nTL,nVL;
    {
    final double [] psi=P1t.psiL,dpsi=getEDPsi(psi);
    this.I_PL=TMatr.getAMatrA(psi,QQi,n);
    this.varI_PL=TMatr.getAMatrB(psi,dpsi,QQi,n)-this.I_PL*TMatr.getAMatrB(psi,dpsi,QQt,n);
    nTL=TMatr.getAMatrA(psi,QQT,n);
    nVL=TMatr.getAMatrA(psi,QQV,n);
    this.VT_PL=nVL/nTL;
    this.varVT_PL=TMatr.getAMatrB(psi,dpsi,QQV,n)/nTL-this.VT_PL*TMatr.getAMatrB(psi,dpsi,QQT,n)/nTL;
    }
    final double nTH,nVH;
    {
    final double [] psi=P1t.psiH,dpsi=getEDPsi(psi);
    this.I_PH=TMatr.getAMatrA(psi,QQi,n);
    this.varI_PH=TMatr.getAMatrB(psi,dpsi,QQi,n)-this.I_PH*TMatr.getAMatrB(psi,dpsi,QQt,n);
    nTH=TMatr.getAMatrA(psi,QQT,n);
    nVH=TMatr.getAMatrA(psi,QQV,n);
    this.VT_PH=nVH/nTH;
    this.varVT_PH=TMatr.getAMatrB(psi,dpsi,QQV,n)/nTH-this.VT_PH*TMatr.getAMatrB(psi,dpsi,QQT,n)/nTH;
    }
    varVP=(nVH-nVL)/(nTH-nTL);
    avarVP=(nVH-nVL)/Math.abs(nTH-nTL);

    Pt=(TMatr.getAMatrA(P1t.psiH,QQpT,n)-TMatr.getAMatrA(P1t.psiL,QQpT,n))/
	(TMatr.getAMatrA(P1t.psiH,QQT,n)-TMatr.getAMatrA(P1t.psiL,QQT,n));
    Pv=(TMatr.getAMatrA(P1t.psiH,QQpV,n)-TMatr.getAMatrA(P1t.psiL,QQpV,n))/
	(TMatr.getAMatrA(P1t.psiH,QQV,n)-TMatr.getAMatrA(P1t.psiL,QQV,n));

    PT0_L=TMatr.getAMatrA(P1t.psiL,QQpT,n)/TMatr.getAMatrA(P1t.psiL,QQT,n);
    PV0_L=TMatr.getAMatrA(P1t.psiL,QQpV,n)/TMatr.getAMatrA(P1t.psiL,QQV,n);
    PT0_H=TMatr.getAMatrA(P1t.psiH,QQpT,n)/TMatr.getAMatrA(P1t.psiH,QQT,n);
    PV0_H=TMatr.getAMatrA(P1t.psiH,QQpV,n)/TMatr.getAMatrA(P1t.psiH,QQV,n);

    simom=imom;
    stmom=tmom;
    sptmom=ptmom;
    spimom=pimom;
}
    final double [] simom,stmom,sptmom,spimom;


void test_Self(final double eps)
{
    final double [] o=new double [stmom.length];
    WEpvt s=new EDWEpvt(P1t.neq,sptmom,stmom,o,o,simom,stmom,o,o);
    UnitTests.checkDoubleRel(s.Ptb,"s.Ptb",I_PL,"I_PL",eps);
    UnitTests.checkDoubleRel(s.spt[0],"s.spt[0]",I_PL,"I_PL",eps);
    UnitTests.checkDoubleRel(-0.5*s.spt[1],"-0.5*s.spt[1]",varI_PL,"varI_PL",eps);
    UnitTests.checkDoubleRel(s.Pto,"s.Pto",I_PH,"I_PH",eps);
    s=new EDWEpvtH(P1t.neq,sptmom,stmom,o,o,simom,stmom,o,o);
    UnitTests.checkDoubleRel(s.spt[0],"s.spt[0]",I_PH,"I_PH",eps);
    UnitTests.checkDoubleRel(-0.5*s.spt[1],"-0.5*s.spt[1]",varI_PH,"varI_PH",eps);

    s=new EDWEpvt(P1t.neq,sptmom,stmom,o,o,getFmom(simom),getFmom(stmom),o,o);
    UnitTests.checkDoubleRel(s.spt[0],"s.spt[0]",VT_PL,"VT_PL",eps);
    s=new EDWEpvtH(P1t.neq,sptmom,stmom,o,o,getFmom(simom),getFmom(stmom),o,o);
    UnitTests.checkDoubleRel(s.spt[0],"s.spt[0]",VT_PH,"VT_PH",eps);
    UnitTests.checkDoubleRel(-0.5*s.spt[1],"-0.5*s.spt[1]",varVT_PH,"varVT_PH",eps);


    s=new EDWEpvt(P1t.neq,sptmom,stmom,o,o,getFmom(spimom),getFmom(simom),o,o);
    UnitTests.checkDoubleRel(s.spt[0],"s.spt[0]",PV0_L,"PV0_L",eps);
    s=new EDWEpvtH(P1t.neq,sptmom,stmom,o,o,getFmom(spimom),getFmom(simom),o,o);
    UnitTests.checkDoubleRel(s.spt[0],"s.spt[0]",PV0_H,"PV0_H",eps);
    s=new EDWEpvt(P1t.neq,sptmom,stmom,o,o,getFmom(sptmom),getFmom(stmom),o,o);
    UnitTests.checkDoubleRel(s.spt[0],"s.spt[0]",PT0_L,"PT0_L",eps);
    s=new EDWEpvtH(P1t.neq,sptmom,stmom,o,o,getFmom(sptmom),getFmom(stmom),o,o);
    UnitTests.checkDoubleRel(s.spt[0],"s.spt[0]",PT0_H,"PT0_H",eps);

    System.err.println("EkQVM:VarItP:test_Self: test OK n="+this.P1t.neq+" eps="+eps);
}
} // END OF class VarItP





/** Calculate P&L matrix, which is defined as:
 *  P&L=<Dpsi|QQt|psi>(<Dpsi|QQf|psi>-<psi|QQf|psi><Dpsi|QQt|psi>)=<Dpsi|MP&L|Dpsi>.
 *  This method calculates MP&L.
 *  Note that
 *  + Only Hermitian part of MP&L contribute to P&L, so under MP&L
 *    we will understand the Hermitian part of it.
 *  + The MP&L has exactly two non-zero eigenvalues MP&L|q>=lambda QQt|q>
 *    which are equal in modules and have opposite sign.
 *  + The projection weights of these two eigenvectors to original |psi> equals 1/2, <q|QQt|psi>=sqrt(1/2).
 *  + The projection weights of these two eigenvectors to any eigenvector of QQi|g>=l QQt|g>
 *    are equal, |<q_low|QQt|g>|=|<q_high|QQt|g>|.
 */
static double [] getQQPNL(final double [] psi,final double [] QQf,final double [] QQt,final int ldQQ)
{
    final int n=psi.length;
    final double nt=TMatr.getAMatrA(psi.length,psi,QQt,ldQQ),
	pmu=TMatr.getAMatrA(psi.length,psi,QQf,ldQQ)/nt;

    final double [] r=new double [n],s=new double [n],QQpnl=new double [n*n];
    for(int k=0;k<n;k++)
    {
	r[k]=DeprecatedHelperTMatr.mulVVo(n,QQt,k*ldQQ,psi,0)/nt;
	s[k]=DeprecatedHelperTMatr.mulVVo(n,QQf,k*ldQQ,psi,0)/nt-pmu*r[k];
	for(int l=0;l<=k;l++)
	{
	    // Take Hermitian part of the matrix,
	    // only hermitian part contribute to P&L.
	    QQpnl[k*n+l]=QQpnl[l*n+k]=0.5*(r[k]*s[l]+r[l]*s[k]);
	}
    }
    return QQpnl;
}


static final int FLAG_I_L=893,FLAG_I_H=894;
class W2Proj {
final EVXData I,PNL;
final double w_psi_PL,w_psi_PH,w_psi0_PL,w_psi0_PH,w_sum,w_diff2;
final int flag_lh;

W2Proj(final int n,final double [] imom,final double [] tmom,
       final double [] pimoma,final double [] ptmoma,final int flag_lh)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,pimoma,ptmoma});
    final double [] QQi=QQ[0],QQt=QQ[1],QQpia=QQ[2],QQpta=QQ[3];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);
    this.flag_lh=flag_lh;

    final double [] psi,psio;final double lam;
    if(flag_lh==FLAG_I_L){psi=this.I.psiL;psio=this.I.psiH;lam=this.I.sL;}
    else if(flag_lh==FLAG_I_H){psi=this.I.psiH;psio=this.I.psiL;lam=this.I.sH;}
    else throw new RuntimeException("flag_lh="+flag_lh);

    final double [] QQpmu=new double [QQpia.length];
    for(int k=QQpmu.length;--k>=0;) {QQpmu[k]=QQpia[k]-lam*QQpta[k];}

    // P&L matrix has exactly two non-zero eigenvalues (positive and negative)
    // with the same module. All the remaining eigenvalues are 0.
    final double [] QQpnl=getQQPNL(psi,QQpmu,QQt,n);
    this.PNL=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpnl,QQt,access,x0);

    // The |<psi|PNL.psiL>|=|<psi|PNL.psiH>|=1/sqrt(2).
    // The |<psio|PNL.psiL>|=|<psio|PNL.psiH>|
    final double [] psi0=I.psi0;
    this.w_psi0_PL=TMatr.getAMatrB(psi0.length,psi0,0,PNL.psiL.length,PNL.psiL,0,QQt,n);
    this.w_psi0_PH=TMatr.getAMatrB(psi0.length,psi0,0,PNL.psiH.length,PNL.psiH,0,QQt,n);
    this.w_psi_PL=TMatr.getAMatrB(psi.length,psi,0,PNL.psiL.length,PNL.psiL,0,QQt,n);
    this.w_psi_PH=TMatr.getAMatrB(psi.length,psi,0,PNL.psiH.length,PNL.psiH,0,QQt,n);

    this.w_sum=w_psi_PH*w_psi0_PH+w_psi_PL*w_psi0_PL;
    this.w_diff2=Math.pow(w_psi_PH*w_psi0_PH,2)-Math.pow(w_psi_PL*w_psi0_PL,2);

    simom=imom;
    stmom=tmom;
    spimom=pimoma;
    sptmom=ptmoma;
}
final double [] simom,stmom,spimom,sptmom;



void test_Self(final double eps)
{
    final int n=this.I.neq;
    System.err.println("w_psi_PL="+w_psi_PL+"\tw_psi_PH="+w_psi_PH);
    System.err.println("w_psi0_PL="+w_psi0_PL+"\tw_psi0_PH="+w_psi0_PH);
    System.err.println("PNL.ev="+java.util.Arrays.toString(PNL.getEValues()));
    //System.err.println("PNL.psiL="+java.util.Arrays.toString(PNL.psiL));
    //System.err.println("PNL.psiH="+java.util.Arrays.toString(PNL.psiH));

    // Test that n-2 eigenvalues are 0.
    for(int k=1;k<n-1;k++)
    {
	UnitTests.checkDoubleRel(PNL.ev[k],"PNL.ev[k]",0,"0",eps);
    }
    // Check first and last eigenvalues differ only in sign.
    if(n>=2)
    {
	UnitTests.checkDoubleRel(PNL.ev[0],"PNL.ev[0]",-PNL.ev[n-1],"-PNL.ev[n-1]",eps);
    }

    if(true)
    { // Check an expansion of psi on PNL.psiL & PNL.psiH give exactly psi,
      // regardless of the basis dimension.
	final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{stmom});
	final double [] QQt=QQ[0];
	final double [] psi;
	if(flag_lh==FLAG_I_L){psi=this.I.psiL;}
	else if(flag_lh==FLAG_I_H){psi=this.I.psiH;}
	else throw new RuntimeException("flag_lh="+flag_lh);
	final double [] pp=new double [n];
	TMatr.sumP1(pp,PNL.psiL,w_psi_PL);
	TMatr.sumP1(pp,PNL.psiH,w_psi_PH);
	UnitTests.checkDoubleArrayRel(pp,"pp",psi,"psi",eps);

	UnitTests.checkDoubleRel(Math.abs(w_psi0_PL),"w_psi0_PL",Math.abs(PNL.wL),"PNL.wL",eps);
	UnitTests.checkDoubleRel(Math.abs(w_psi0_PH),"w_psi0_PH",Math.abs(PNL.wH),"PNL.wL",eps);

	if(flag_lh==FLAG_I_L)
	    UnitTests.checkDoubleRel(Math.abs(w_sum),"w_sum",Math.abs(I.wL),"I.wL",eps);
	else if(flag_lh==FLAG_I_H)
	    UnitTests.checkDoubleRel(Math.abs(w_sum),"w_sum",Math.abs(I.wH),"I.wL",eps);
    }


    System.err.println("EkQVM:W2Proj:test_Self: test OK n="+this.I.neq+" eps="+eps);
}
} // END OF class W2Proj





static class VarPsi {
    final double P_psi,P_vpsi,PA_pp,dP_psi,dP_vpsi;
    VarPsi(final double [] psi,final double [] vpsi,
	   final double [] QQpia,final double [] QQia,
	   final double [] QQpta,final double [] QQta,
	   final int ldQQ)
    {
	this.P_psi=getP(psi,QQpia,QQia,ldQQ)-getP(psi,QQpta,QQta,ldQQ);
	this.P_vpsi=getP(vpsi,QQpia,QQia,ldQQ)-getP(vpsi,QQpta,QQta,ldQQ);
	this.dP_psi=getVP(psi,vpsi,QQpia,QQia,ldQQ)-getVP(psi,vpsi,QQpta,QQta,ldQQ);
	this.dP_vpsi=getVP(vpsi,psi,QQpia,QQia,ldQQ)-getVP(vpsi,psi,QQpta,QQta,ldQQ);
	this.PA_pp=getPA(psi,1,vpsi,1,QQpia,QQia,ldQQ)-getPA(psi,1,vpsi,1,QQpta,QQta,ldQQ);
    }
    static double getP(final double [] psi,final double [] QQfa,final double [] QQa,final int ldQQ)
    {
	return TMatr.getAMatrA(psi.length,psi,QQfa,ldQQ)/TMatr.getAMatrA(psi.length,psi,QQa,ldQQ);
    }
    static double getVP(final double [] psi,final double [] vpsi,final double [] QQfa,final double [] QQa,final int ldQQ)
    {
	final double nr=TMatr.getAMatrA(psi.length,psi,QQa,ldQQ);
	return TMatr.getAMatrA(vpsi.length,vpsi,QQfa,ldQQ)/nr-
	    (TMatr.getAMatrA(vpsi.length,vpsi,QQa,ldQQ)/nr)*
	    (TMatr.getAMatrA(psi.length,psi,QQfa,ldQQ)/nr);
    }
    static double getPA(final double [] psi,double ppsi,final double [] vpsi,final double pvpsi,final double [] QQfa,final double [] QQa,final int ldQQ)
    {
	return
	(ppsi*TMatr.getAMatrA(psi.length,psi,QQfa,ldQQ)+
	 pvpsi*TMatr.getAMatrA(vpsi.length,vpsi,QQfa,ldQQ))/
	(ppsi*TMatr.getAMatrA(psi.length,psi,QQa,ldQQ)+
	 pvpsi*TMatr.getAMatrA(vpsi.length,vpsi,QQa,ldQQ));
    }

}



class WvarPsi {
final EVXData I;
final double fXx;
final VarPsi PL_H,PH_L,PL_dL0,PL_dL;

WvarPsi(final int n,final double [] imom,final double [] tmom,
      final double [] pimoma,final double [] imoma,
      final double [] ptmoma,final double [] tmoma)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,pimoma,imoma,ptmoma,tmoma});
    final double [] QQi=QQ[0],QQt=QQ[1],QQpia=QQ[2],QQia=QQ[3],QQpta=QQ[4],QQta=QQ[5];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);

    final double [] psi=this.I.psiL;
    final double [][] r=DkQVM.getRoots(psi,M,isEdgeAtRight());
    final double [] Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0_NonSortedX(Xx,x0,isEdgeAtRight());
    this.fXx=indX>=0?Xx[indX]:Double.NaN;

    this.PL_H=new VarPsi(this.I.psiL,this.I.psiH,QQpia,QQia,QQpta,QQta,n);
    this.PH_L=new VarPsi(this.I.psiH,this.I.psiL,QQpia,QQia,QQpta,QQta,n);
    this.PL_dL=new VarPsi(this.I.psiL,getDpsi(this.I.psiL,this.fXx),QQpia,QQia,QQpta,QQta,n);
    this.PL_dL0=new VarPsi(this.I.psiL,getDpsi0(this.I.psiL,this.fXx),QQpia,QQia,QQpta,QQta,n);


    simom=imom;
    stmom=tmom;
    spimoma=pimoma;
    simoma=imoma;
    sptmoma=ptmoma;
    stmoma=tmoma;
}
final double [] simom,stmom,spimoma,simoma,sptmoma,stmoma;

double [] getDpsi(final double [] psi,final double x)
{
    return M.B.sdiv1(psi,x).q;
}
double [] getDpsi0(final double [] psi,final double x)
{
    final double [] Dpsi=M.B.sdiv1(psi,x).q;
    return M.getRadauMultiplied(Dpsi);
}


void test_Self(final double eps)
{
    final WEpvt q=new WEpvt(this.I.neq,simom,stmom,spimoma,simoma,sptmoma,stmoma,sptmoma,stmoma);
    UnitTests.checkDoubleRel(fXx,"fXx",q.fXx,"q.fXx",eps);
    UnitTests.checkDoubleRel(PL_dL.P_psi,"PL_dL.P_psi",q.d0f,"q.d0f",eps);
    UnitTests.checkDoubleRel(PL_dL.dP_psi,"PL_dL.dP_psi",q.d2fc/2,"q.d2fc/2",eps);
    EVXData.matchEVXData(this.I,q.I,true,true,eps);

    System.err.println("EkQVM:WvarPsi:test_Self: test OK n="+this.I.neq+" eps="+eps);
}
} // END OF class WvarPsi




static abstract class EkQVMTest<DTYPE> extends QTest {

abstract EkQVM<DTYPE> getEkQVM();
abstract DkQVM<?> getDkQVM();


EkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}




void test_Common()
{
    final EkQVM<DTYPE> q=getEkQVM();
    test_getQQPNL(q.access);

    q.eapvaL.test_Self(eps);
    q.eapvaH.test_Self(eps);
    q.eapvt.test_Self(eps);
    q.eapvt_sam.test_Self(eps);
    q.pVT.test_Self(eps);
    q.pvt.test_Self(eps);
    q.epvt.test_Self(eps);
    //UnitTests.checkDoubleRel(q.epvt.dgi,"q.epvt.dgi",q.epvt.dgt,"q.epvt.dgt",eps); // match approximately
    q.epvt_sam.test_Self(eps);
    UnitTests.checkDoubleRel(q.epvt_sam.dgi,"q.epvt_sam.dgi",q.epvt_sam.dgt,"q.epvt_sam.dgt",eps);
    q.epvta.test_Self(eps);
    UnitTests.checkDoubleRel(q.epvta.dgi,"q.epvta.dgi",q.epvta.dgt,"q.epvta.dgt",eps);

    q.wv.test_Self(eps);
    q.WPpvt.test_Self(eps);

    q.wpnlL_sam.test_Self(eps);
    q.wpnlH_sam.test_Self(eps);
    q.wpnlL.test_Self(eps);
    q.wpnlH.test_Self(eps);
    q.wpnlVSL_sam.test_Self(eps);
    q.wpnlVSH_sam.test_Self(eps);
    q.wpnlvsL_sam.test_Self(eps);
    q.wpnlvsH_sam.test_Self(eps);

    q.pvar_L.test_Self(eps);
    q.pvar_H.test_Self(eps);
    q.pivar_L.test_Self(eps);
    UnitTests.checkDoubleRel(-q.pivar_L.sd0,"-q.pivar_L.sd0",q.epvt.d0i,"q.epvt.d0i",eps);
    q.pivar_H.test_Self(eps);
    UnitTests.checkDoubleRel(-q.pivar_H.sd0,"-q.pivar_H.sd0",q.epvtaDH.d0i,"q.epvtaDH.d0i",eps);
    q.svar_L.test_Self(eps);
    q.svar_H.test_Self(eps);
    q.Svar_L.test_Self(eps);
    q.Svar_H.test_Self(eps);


    q.gvar.test_Self(eps);
    q.vpivar_L.test_Self(eps);
    q.vpivar_H.test_Self(eps);
    q.up.test_Self(eps);
    UnitTests.checkDoubleRel(-q.up.PNLl/q.up.Il,"-q.up.PNLl/q.up.Il",q.epvt.Pid,"q.epvt.Pid",eps);
    q.upH.test_Self(eps);
    q.upP.test_Self(eps);
    q.pnlvol.test_Self(eps);
    q.pnlvolT.test_Self(eps);

    q.vitp.test_Self(eps);

    q.rpL.test_Self(eps);
    UnitTests.checkDoubleRel(q.rpL.dI_lev0,"q.rpL.dI_lev0",q.rp_L.dvol,"q.rp_L.dvol",eps);
    q.rpH.test_Self(eps);
    UnitTests.checkDoubleRel(q.rpH.dI_lev0,"q.rpH.dI_lev0",q.rp_H.dvol,"q.rp_H.dvol",eps);

    q.jp_dptI.test_Self(eps,q.access);


    System.err.println(name+":EkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}

void test_getQQPNL(final DeprecatedHelperDataDirectAccessWithBasis<?> access)
{
    final int n=neq;
    final int ntests=4,nsum=1000,nsumpsi=2*n+5,ldQQ=n+4;
    final double xa=0,xb=1.0;
    for(int t=0;t<ntests;t++)
    {
	final double [] tmom=DkQVM.getRandomPositivelyDefined(2*ldQQ-1,nsum,r,access,xa,xb),
	    pmumom=DkQVM.getRandomPositivelyDefined(2*ldQQ-1,nsum,r,access,xa,xb);
	final double [][] QQ=access.getQQ_Matrs(ldQQ,1,new double [][]{tmom,pmumom});
	final double [] QQt=QQ[0],QQpmu=QQ[1];
	final double [] psi=DkQVM.getRandomPositivelyDefined(n,nsumpsi,r,access,xa,xb),
	    Dpsi=DkQVM.getRandomPositivelyDefined(n,nsumpsi,r,access,xa,xb);
	final double [] m=DkQVM.getfDsx(psi,Dpsi,QQpmu,QQt,ldQQ);
	final double pnldd=((m[2]-m[3])/((-2)*(-2)*(-2)));
	final double [] pnlmatr=getQQPNL(psi,QQpmu,QQt,ldQQ);
	UnitTests.checkDoubleRel(pnldd,"pnldd",TMatr.getAMatrA(Dpsi.length,Dpsi,pnlmatr,n),"<Dpsi|pnlmatr|Dpsi>",eps);
    }

    System.err.println(name+":EkQVMTest:test_getQQPNL: test OK n="+n+" eps="+eps);
}


void test_DkQVM()
{
    final DkQVM<?> s=getDkQVM();
    final EkQVM<DTYPE> q=getEkQVM();

    EVXData.matchEVXData(q.epvt.I,s.IT,true,true,eps);
    EVXData.matchEVXData(q.epvt_sam.I,s.IT_sam,true,true,eps);
    q.epvt.test_DkQVM_WkkPsi(q.epvt.spi,s.ekpit_psiIL,eps);
    if(true)
    {
	UnitTests.checkDoubleRel(q.epvt.d0i,"q.epvt.d0i",s.ekpit_psiIL.Pf,"s.ekpit_psiIL.Pf",eps);
	UnitTests.checkDoubleRel(q.epvt.d1i,"q.epvt.d1i",-2*s.ekpit_psiIL.dPf,"-2*s.ekpit_psiIL.dPf",eps);
	if(s.ekpit_psiIL.I2!=0 && !Double.isNaN(s.ekpit_psiIL.I2)) UnitTests.checkDoubleRel(q.epvt.d2i,"q.epvt.d2i",s.ekpit_psiIL.PI2*s.ekpit_psiIL.I2,"s.ekpit_psiIL.PI2*s.ekpit_psiIL.I2",eps);
	if(s.ekpit_psiIL.I2c!=0 && !Double.isNaN(s.ekpit_psiIL.I2c)) UnitTests.checkDoubleRel(q.epvt.d2ic,"q.epvt.d2ic",s.ekpit_psiIL.PI2c*s.ekpit_psiIL.I2c,"s.ekpit_psiIL.PI2c*s.ekpit_psiIL.I2c",eps);
    }
    if(true)
    {
	UnitTests.checkDoubleRel(q.epvtD_sam.g,"q.epvtD_sam.g",s.wqmuPp.g_psiL,"s.wqmuPp.g_psiL",eps);
	UnitTests.checkDoubleRel(q.epvtD_sam.sigma2_g,"q.epvtD_sam.sigma2_g",s.wqmuPp.sigma2_g_psiL,"s.wqmuPp.sigma2_g_psiL",eps);
    }

    q.epvt_sam.test_DkQVM_WkkPsi(q.epvt_sam.spt,s.ekpti_psiIL,eps);
    q.epvt_sam.test_DkQVM_WmuPsi(s.wmup_sam,eps);
    q.eVSvt_sam.test_DkQVM_WmuPsi(s.wmuv_sam,eps);
    q.evsvt_sam.test_DkQVM_WmuPsi(s.wmus_sam,eps);

    q.epvtaD.test_DkQVM_WkePsi(q.epvtaD.spi,s.epit_psiIL,eps);
    q.epvtD_sam.test_DkQVM_WkePsi(q.epvtD_sam.spt,s.epti_psiIL,eps);
    //q.epvtD_sam.test_DkQVM_WkePsi(q.epvtDH_sam.spt,s.epti_psiIH,eps);
    if(!Double.isNaN(s.wmup_sam.p_x))
    {
    UnitTests.checkDoubleRel(q.epvt_sam.Mu,"q.epvt_sam.Mu",s.wmup_sam.mu,"s.wmup_sam.mu",eps);
    UnitTests.checkDoubleRel(q.epvt_sam.xpMu/q.epvt_sam.Mu,"q.epvt_sam.xpMu/q.epvt_sam.Mu",s.wmup_sam.xXD0,"s.wmup_sam.xXD0",eps);
    UnitTests.checkDoubleRel(q.epvt_sam.pMu/q.epvt_sam.Mu,"q.epvt_sam.pMu/q.epvt_sam.Mu",s.wmup_sam.p_x,"s.wmup_sam.p_x",eps);
    }

    EVXData.matchEVXData(q.pnlvol.D,s.pik.DT,true,true,eps);

    UnitTests.checkDoubleRel(q.epvta.dPtb,"q.epvta.dPtb",s.jdpt.dp_IL,"s.jdpt.dp_IL",eps);
    UnitTests.checkDoubleRel(q.epvta.dPto,"q.epvta.dPto",s.jdpt.dp_IH,"s.jdpt.dp_IH",eps);


    EVXData.matchEVXData(q.vitp.P1t,s.P1t_pV.Px,true,true,eps);
    EVXData.matchEVXData(q.vitp.P1t,s.P1t_pT.Px,true,true,eps);
    UnitTests.checkDoubleRel(q.vitp.PT0_L,"q.vitp.PT0_L",s.P1t_pT.PpsiL,"s.P1t_pT.PpsiL",eps);
    UnitTests.checkDoubleRel(q.vitp.PV0_L,"q.vitp.PV0_L",s.P1t_pV.PpsiL,"s.P1t_pV.PpsiL",eps);
    UnitTests.checkDoubleRel(q.vitp.PT0_H,"q.vitp.PT0_H",s.P1t_pT.PpsiH,"s.P1t_pT.PpsiH",eps);
    UnitTests.checkDoubleRel(q.vitp.PV0_H,"q.vitp.PV0_H",s.P1t_pV.PpsiH,"s.P1t_pV.PpsiH",eps);




    System.err.println(name+":DkQVMTest:test_DkQVM(): test OK neq="+neq+" eps="+eps);
}

void test_diff()
{
    final EkQVM<DTYPE> q=getEkQVM();
    final double [] psi=q.gvar.I.psiL,mom=q.gvar.stmom;
    final double [] pnlcoefs=q.getCoefsForPNLInDPspace(psi);
    final double x0=q.getX0(),wx0=1; // w(x0)=1 for the measures we use
    final double f=q.access.fx(pnlcoefs,x0)*wx0;
    UnitTests.checkDoubleRel(f,"f",1,"1",eps);
    UnitTests.checkDoubleRel(q.gvar.pt-q.gvar.pto,"q.gvar.pt-q.gvar.pto",-(q.gvar.ptd-q.gvar.ptdo),"-(q.gvar.pdt-q.gvar.ptdo)",eps);

    System.err.println(name+":DkQVMTest:test_diff(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":EkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_DkQVM();
    test_diff();

    System.err.println(name+":EkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":EkQVMTest: starting tests; ver=$Id: EkQVM.java,v 1.185 2022/10/30 21:29:54 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":EkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: EkQVM.java,v 1.185 2022/10/30 21:29:54 mal Exp $");
}
} // END OF class EkQVMTest

}// END OF class EkQVM
