package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2020
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.EVXData;

/** Save the state.
 * @version $Id: EnterILowExitIHighStateSaver.java,v 1.28 2020/12/30 09:20:21 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class EnterILowExitIHighStateSaver extends SimpleStateSaver {

    public final boolean flag_exit;

    final EnterPoint enterPoint;
    final java.util.ArrayList<ExitPoint> exitPoints;


    /** Initial state.
     */
    public EnterILowExitIHighStateSaver(){
	// first call, no previous state available
	super();
	this.flag_exit=false;
	this.enterPoint=null;
	this.exitPoints=new java.util.ArrayList<ExitPoint>();
    }

    /** Recurrently updated state.
     */
    public EnterILowExitIHighStateSaver(final EnterILowExitIHighStateSaver prev,
					final long time,final double P,final long dV,
					final EVXData I)
    {
	super(prev,time,P,dV,I);
	this.exitPoints=prev.exitPoints;
	final ExitPoint prev_exitPoint=exitPoints.size()<=0?null:exitPoints.get(0);
	// A spike in I=dV/dt and the price is valid and executed volime dV>0
	if(index_max_psi_projection==I.neq-1){
	    // a high I
	    if(P>0 && dV>0
	       && (prev.enterPoint!=null && I.sH>=prev.enterPoint.IH)
	       && (prev_exitPoint==null || I.sH>=prev_exitPoint.IH)
	       ) {
		// EXIT a postion condition OK
		flag_exit=true;
		enterPoint=prev.enterPoint; // same enter, only for possible exit adjustment
		final ExitPoint exitPoint=new ExitPoint(time,P,I.sH,enterPoint);
		exitPoints.clear();
		exitPoints.add(exitPoint);
		if(prev_exitPoint!=null){
		    exitPoints.add(prev_exitPoint);
		}
		// We can exit at this spike with some P&L (we already have a position available)
		// old P&L adjustment for "previous exit" should be subtracted to avoid duplicates
	    } else {
		flag_exit=false;
		enterPoint=prev.enterPoint;
	    }
	} else {
	    flag_exit=false;
	    // Check for a low I=dV/dt event and the price is valid.
	    if(P>0 &&
	       (index_max_psi_projection>=0 && index_max_psi_projection<(I.neq-1)) // I is low enough
	       &&
	       (prev.enterPoint==null||(I.sH<=prev.enterPoint.IH||exitPoints.size()>0)) // either better enter or already had an exit
	       ){
		// ENTER a postion condition OK
	        enterPoint=new EnterPoint(time,P,I.sH);
	        exitPoints.clear(); // disable exit adjustment
	    } else {
		enterPoint=prev.enterPoint;
	    }
	}

    }

    public ExitPoint getPnL(){
	if(!flag_exit) return null;
	return exitPoints.size()<=0?null:exitPoints.get(0);
    }
    public ExitPoint getPnLAdjustment(){
	if(!flag_exit) return null;
	return exitPoints.size()<=1?null:exitPoints.get(1);
    }
    public ExitPoint getPnLEstimateIfExitAtLastPrice(){
	if(P>0 && enterPoint!=null && exitPoints.size()<=0)
	    return new ExitPoint(time,P,I.sH,enterPoint);
	return null;
    }

} // END OF class EnterILowExitIHighStateSaver
