package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Time evolution as perturbation to |psi0>.
 *  @version $Id: EpComm.java,v 1.10 2020/09/27 20:33:14 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class EpComm {

public final double I_0,I_L,I_H,pE1,pE0,pEm1,pEm2,pH0,pL0;
public final double pLH,fLH,ILH;

EpComm(final int n,final double [] imom,final double [] tmom,final double [] fmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");
    final double [] basis=getBasis(n,imom,tmom,M);

    final int i0=0,iL=1,iH=n-1;
    this.I_0=basis[i0];
    this.I_L=basis[iL];
    this.I_H=basis[iH];

    // QQt is diagonal in new basis.
    final double [] QQt=M.B.getQQMatr(n,tmom),
	QQi=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,imom),basis),
	QQf=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,fmom),basis);

    double s1=0,s0=0,sm1=0,sm2=0,sH0=0,sL0=0;
    for(int k=0;k<n;k++)
    {
	if(k==i0) continue;
	final double ek=basis[k],i=QQi[i0*n+k],f=QQf[i0*n+k],fi=i*f;
	s0+=fi;
	s1+=fi*ek;
	sm1+=fi/ek;
	sm2=fi/(ek*ek);
	if(k==iL) sL0=fi;
	if(k==iH) sH0=fi;
    }
    pE1=s1;
    pE0=s0;
    pEm1=sm1;
    pEm2=sm2;
    pH0=sH0;
    pL0=sL0;
    fLH=QQf[iL*n+iH];
    ILH=QQi[iL*n+iH];
    pLH=fLH*ILH;


    this.simom=imom;
    this.stmom=tmom;
    this.sfmom=fmom;
    this.M=M;
    this.sn=n;
}
final double [] simom,stmom,sfmom;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
final int sn;

double [] getBasis(final int n,final double [] imom,final double [] tmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    return EdComm.getIRadauBasisPsi0(n,imom,tmom,M);
}


void match_X0Psi(final MkQVM<?>.X0Psi s,final double eps)
{
    UnitTests.checkDoubleRel(pEm1,"pEm1",s.mAI,"s.mAI",eps);
    UnitTests.checkDoubleRel(pE0,"pE0",s.mA,"s.mA",eps);
    System.err.println("ILH="+ILH);
    System.err.println("EpComm:match_X0Psi: test OK n="+sn+" eps="+eps);
}

} // END OF class EpComm
