package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Time evolution as commutator acting on perturbation at x0.
 *  @version $Id: EpiComm.java,v 1.24 2022/11/19 11:08:57 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class EpiComm {
final EVXData I;
final double sensL,sensH;

EpiComm(final int n,final double [] imom,final double [] tmom,final double [] fmom,final double [] pertmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] QQt=M.B.getQQMatr(n,tmom),QQi=M.B.getQQMatr(n,imom);
    this.I=isDiagQQt()?EVXData.getEVXDataFromQQfDiagQQa(n,QQi,QQt,M):
	EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] QQf=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,fmom),this.I.ev),
	QQpert=EVXData.convertMatrFromQtoB_Basis(n,M.B.getQQMatr(n,pertmom),this.I.ev);
    // QQi & QQt are diagonal in new basis.
    java.util.Arrays.fill(QQi,0);
    for(int k=0;k<n;k++){
	QQi[k*n+k]=this.I.ev[k]; // QQi in new basis is diagonal.
    }
    final double [] QQc=TMatr.mulMM(n,QQi,QQf);
    // Anti-Hermitian part (QQi QQf - QQf QQi) is responsible for time evolution.
    TMatr.scaleVectorInPlace(TMatr.selectAntiHermitianPartInPlace(n,QQc),2);

    final int iL=0,iH=n-1;
    double sL=0,sH=0;
    for(int k=0;k<n;k++)
    {
	if(k!=iL){sL+=QQpert[iL*n+k]*QQc[k*n+iL]/(this.I.ev[iL]-this.I.ev[k]);}
	if(k!=iH){sH+=QQpert[iH*n+k]*QQc[k*n+iH]/(this.I.ev[iH]-this.I.ev[k]);}
    }
    this.sensL=sL;
    this.sensH=sH;

    this.stmom=tmom;
    this.simom=imom;
    this.sfmom=fmom;
    this.spertmom=pertmom;
    this.M=M;
}
boolean isDiagQQt(){return false;}
final double [] stmom,simom,sfmom,spertmom;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;

void test_Self(final double eps)
{
    final int n=this.I.neq;
    final double [] QQt=M.B.getQQMatr(n,stmom),QQi=M.B.getQQMatr(n,simom),QQf=M.B.getQQMatr(n,sfmom),QQpert=M.B.getQQMatr(n,spertmom),
	IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
    final double [] QQc=TMatr.sumAcB(
	TMatr.mulMatrices(n,new double [][] {QQi,IQQt,QQf}),
	-1,
	TMatr.mulMatrices(n,new double [][] {QQf,IQQt,QQi}));

    UnitTests.checkDoubleRel(TMatr.getAMatrB(DeprecatedHelperEVXData.getPsiPertr(I,0,QQpert),I.psiL,QQc,n),"<ppsiL|QQc|psiL>",sensL,"sensL",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(DeprecatedHelperEVXData.getPsiPertr(I,n-1,QQpert),I.psiH,QQc,n),"<ppsiH|QQc|psiH>",sensH,"sensH",eps);

    System.err.println("EpiComm:test_Common: test OK n="+n+" eps="+eps);
}

} // END OF class EpiComm
