package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.EVXData;

/** An experimental attempt to split contributions to P&amp;L
 * as dI/dt eigenstates components.
 * @see com.polytechnik.freemoney.SplitdIHdt
 * @version $Id: ExperimentaldIHdt.java,v 1.11 2022/11/19 11:08:57 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class ExperimentaldIHdt {


    public final double dPIHdt_spurM,dpdt_spurM,dIHdt_spurM;
    public final double pv_dI,pv_maxdIcontrib,pv_mindIcontrib,Tv_maxdIcontrib,Tv_mindIcontrib,pv_posdI,pv_negdI;
    public final double dpdt_posdI_spurM,dpdt_negdI_spurM;
    public final EVXData IH;



    ExperimentaldIHdt(final EVXData IH,
		      final double p_offset,
		      final double p_last,
		      final double [] bQQdIHdt,
		      final double [] bQQdPIHdt,
		      final double [] bQQDensityMatrix,
		      final double [] bQQdp,
		      final double [] bQQTi,
		      final double [] bQQi,
		      // basis operations
		      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		      // for unit tests
		      final double eps){
	this.IH=IH;
	final int n=IH.neq,indMax=n-1;
	dIHdt_spurM=TMatr.getSpurMM(n,bQQdIHdt,bQQDensityMatrix);
	dPIHdt_spurM=TMatr.getSpurMM(n,bQQdPIHdt,bQQDensityMatrix);
	dpdt_spurM=TMatr.getSpurMM(n,bQQdp,bQQDensityMatrix);
	pv_dI=p_offset+dPIHdt_spurM/dIHdt_spurM;

	final double [] bQQdIRho=TMatr.selectHermitianPartInPlace(n,TMatr.mulMM(n,bQQdIHdt,bQQDensityMatrix)),
	    bQQdPIRho=TMatr.selectHermitianPartInPlace(n,TMatr.mulMM(n,bQQdPIHdt,bQQDensityMatrix));

	final double [] ev_bQQdIRho=M.EV.getEV(n,bQQdIRho,true),
	    bbQQdIRho=TMatr.getDiagMatrix(java.util.Arrays.copyOf(ev_bQQdIRho,n)),
	    bbQQdPIRho=EVXData.convertMatrFromQtoB_Basis(n,bQQdPIRho,ev_bQQdIRho),
	    bbQQ_selPosdIdt=new double [n*n],
	    bbQQ_selNegdIdt=new double [n*n];
	pv_maxdIcontrib=p_offset+bbQQdPIRho[indMax*n+indMax]/bbQQdIRho[indMax*n+indMax];
	pv_mindIcontrib=p_offset+bbQQdPIRho[0*n+0]/bbQQdIRho[0*n+0];
	final double [] psi_maxdIcontrib=EVXData.getPsiFromBasis(indMax,ev_bQQdIRho,n),
	    psi_mindIcontrib=EVXData.getPsiFromBasis(0,ev_bQQdIRho,n);
	Tv_maxdIcontrib=TMatr.getAMatrA(psi_maxdIcontrib,bQQTi,n)/TMatr.getAMatrA(psi_maxdIcontrib,bQQi,n);
	Tv_mindIcontrib=TMatr.getAMatrA(psi_mindIcontrib,bQQTi,n)/TMatr.getAMatrA(psi_mindIcontrib,bQQi,n);

	double sumdPI_posdI=0,sumdI_posdI=0,sumdPI_negdI=0,sumdI_negdI=0;
	for(int k=0;k<n;k++){
	    final int diagk=k*n+k;
	    if(ev_bQQdIRho[k]>0){
		bbQQ_selPosdIdt[diagk]=1;
		sumdI_posdI+=bbQQdIRho[diagk];
		sumdPI_posdI+=bbQQdPIRho[diagk];
	    } else if(ev_bQQdIRho[k]<=0){
		bbQQ_selNegdIdt[diagk]=1;
		sumdI_negdI+=bbQQdIRho[diagk];
		sumdPI_negdI+=bbQQdPIRho[diagk];
	    }
	}
	pv_posdI=p_offset+sumdPI_posdI/sumdI_posdI;
	pv_negdI=p_offset+sumdPI_negdI/sumdI_negdI;
	final double [] UNIT=TMatr.getUnitMatrix(n),
	    // convert back to b-basis
	    bQQ_selPosdIdt=EVXData.convertMatrFromBtoQ_Basis(n,bbQQ_selPosdIdt,ev_bQQdIRho,UNIT),
	    bQQ_selNegdIdt=EVXData.convertMatrFromBtoQ_Basis(n,bbQQ_selNegdIdt,ev_bQQdIRho,UNIT);
	dpdt_posdI_spurM=TMatr.getSpurMMM(n,bQQdp,bQQDensityMatrix,bQQ_selPosdIdt);
	dpdt_negdI_spurM=TMatr.getSpurMMM(n,bQQdp,bQQDensityMatrix,bQQ_selNegdIdt);

	if(eps>0){
	    UnitTests.checkDoubleRel1(sumdI_posdI,"sumdI_posdI",TMatr.getSpurMMM(n,bQQdIHdt,bQQDensityMatrix,bQQ_selPosdIdt),"spur(bQQdIHdt,bQQDensityMatrix,bQQ_selPosdIdt)",eps);
	    UnitTests.checkDoubleRel1(sumdI_negdI,"sumdI_negdI",TMatr.getSpurMMM(n,bQQdIHdt,bQQDensityMatrix,bQQ_selNegdIdt),"spur(bQQdIHdt,bQQDensityMatrix,bQQ_selNegdIdt)",eps);
	    UnitTests.checkDoubleRel1(sumdPI_posdI,"sumdI_posdI",TMatr.getSpurMMM(n,bQQdPIHdt,bQQDensityMatrix,bQQ_selPosdIdt),"spur(bQQdPIHdt,bQQDensityMatrix,bQQ_selPosdIdt)",eps);
	    UnitTests.checkDoubleRel1(sumdPI_negdI,"sumdI_negdPI",TMatr.getSpurMMM(n,bQQdPIHdt,bQQDensityMatrix,bQQ_selNegdIdt),"spur(bQQdPIHdt,bQQDensityMatrix,bQQ_selNegdIdt)",eps);

	    UnitTests.checkDoubleRel1(dpdt_posdI_spurM+dpdt_negdI_spurM,"dpdt_posdI_spurM+dpdt_negdI_spurM",dpdt_spurM,"dpdt_spurM",eps);
	    UnitTests.checkDoubleRel1(sumdI_posdI+sumdI_negdI,"sumdI_posdI+sumdI_negdI",dIHdt_spurM,"dIHdt_spurM",eps);
	    UnitTests.checkDoubleRel1(sumdPI_posdI+sumdPI_negdI,"sumdPI_posdI+sumdPI_negdI",dPIHdt_spurM,"dIHdt_spurM",eps);
	    UnitTests.checkDoubleRel1(TMatr.getSpurM(n,bbQQdIRho),"spur(bbQQdIRho)",dIHdt_spurM,"dIHdt_spurM",eps);
	    UnitTests.checkDoubleRel1(TMatr.getSpurMM(n,bQQdPIHdt,bQQDensityMatrix),"spur(bQQdPIHdt*bQQDensityMatrix)",dPIHdt_spurM,"dPIHdt_spurM",eps);
	    UnitTests.checkDoubleRel1(TMatr.getSpurM(n,bQQdPIRho),"spur(bQQdPIRho)",dPIHdt_spurM,"dPIHdt_spurM",eps);
	    UnitTests.checkDoubleRel1(TMatr.getSpurM(n,bbQQdPIRho),"spur(bbQQdPIRho)",dPIHdt_spurM,"dPIHdt_spurM",eps);
	    System.err.println("ExperimentaldIHdt: EV contribs  dI/dt ev_bQQdIRho="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_bQQdIRho,n))+" n="+n+" eps="+eps);
	}
    }

     /** For unit test.
     */
    public static void match(final ExperimentaldIHdt a,final ExperimentaldIHdt b,final double eps){
	EVXData.matchEVXData(a.IH,b.IH,false,eps);

	UnitTests.checkDoubleRel(a.dpdt_spurM,"a.dpdt_spurM",b.dpdt_spurM,"b.dpdt_spurM",eps);
	UnitTests.checkDoubleRel1(a.dIHdt_spurM,"a.dIHdt_spurM",b.dIHdt_spurM,"b.dIHdt_spurM",eps);
	if(Math.abs(a.dIHdt_spurM)>eps){
	    UnitTests.checkDoubleRel1(a.pv_dI,"a.pv_dI",b.pv_dI,"b.pv_dI",eps);
	}
	// depend on p_offset
	//UnitTests.checkDoubleRel(a.dPIHdt_spurM,"a.dPIHdt_spurM",b.dPIHdt_spurM,"b.dPIHdt_spurM",eps);

	UnitTests.checkDoubleRel(a.pv_maxdIcontrib,"a.pv_maxdIcontrib",b.pv_maxdIcontrib,"b.pv_maxdIcontrib",eps);
	UnitTests.checkDoubleRel(a.pv_mindIcontrib,"a.pv_mindIcontrib",b.pv_mindIcontrib,"b.pv_mindIcontrib",eps);
	UnitTests.checkDoubleRel(a.pv_posdI,"a.pv_posdI",b.pv_posdI,"b.pv_posdI",eps);
	UnitTests.checkDoubleRel(a.pv_negdI,"a.pv_negdI",b.pv_negdI,"b.pv_negdI",eps);

	UnitTests.checkDoubleRel(a.dpdt_posdI_spurM,"a.dpdt_posdI_spurM",b.dpdt_posdI_spurM,"b.dpdt_posdI_spurM",eps);
	UnitTests.checkDoubleRel(a.dpdt_negdI_spurM,"a.dpdt_negdI_spurM",b.dpdt_negdI_spurM,"b.dpdt_negdI_spurM",eps);

	System.err.println("ExperimentaldIHdt:match test OK n="+a.IH.neq+" eps="+eps);
    }
}
