package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;

/** QVM calculations.
 *  @version $Id: FQVM.java,v 1.27 2022/06/27 13:32:45 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class FQVM<DTYPE> {

public final double io,scaleo_io,di,dip,scale_io,scale_io0,sigma2_eq,dp_eq,scaleo_dp_eq,scale_dip0;

final DTYPE odata;
final DTYPE [] idata;
final DeprecatedHelperDataInterface<DTYPE> access;
FQVM(final int neq_t,final DTYPE odata,final DTYPE [] idata)
{
    this.odata=odata;
    this.idata=idata;
    this.access=getAccess(neq_t);
    final int neq_p=2+idata.length;
    if(neq_p>neq_t) throw new RuntimeException("neq_t="+neq_t+" < neq_p="+neq_p+"=("+idata.length+"+2)");

    final double [] i=new double [neq_t];
    access.setM(i,0,access.M_I,odata);

    final double [] dpk=new double [neq_t],dpl=new double [neq_t];
    final double [] matr=new double [neq_p*neq_p],
	rp_im=new double [neq_p],rps_im=new double [neq_p];
    double dp_dp=Double.NaN,dp_o=Double.NaN;
    for(int k=0;k<neq_p;k++)
    {
	if(k<=0) access.setM(dpk,0,access.M_O,odata);
	else if(k<=1)
	{
	    access.setM(dpk,1,access.M_DP,odata);
	    dp_o=access.get_f_o(dpk);
	    dp_dp=access.get_f_g(dpk,dpk);
	}
	else access.setM(dpk,1,access.M_DP,idata[k-2]);
	for(int l=0;l<=k;l++)
	{
	    if(l<=0) access.setM(dpl,0,access.M_O,odata);
	    else if(l<=1) access.setM(dpl,1,access.M_DP,odata);
	    else access.setM(dpl,1,access.M_DP,idata[l-2]);
	    matr[k+l*neq_p]=matr[l+k*neq_p]=access.get_f_g(dpk,dpl);
	}
	rp_im[k]=rps_im[k]=access.get_f_g(i,dpk);
    }
    final double o_o=access.get_o_o(),
	i_o=this.io=rps_im[0],i_i=access.get_f_g(i,i),
	x_i_i=i_i-i_o*(i_o/o_o),x_dp_dp=dp_dp-dp_o*(dp_o/o_o);
    final int [] ipvt=new int [neq_p];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,neq_p,neq_p,ipvt);
    if(pinfo!=0){java.util.Arrays.fill(rp_im,Double.NaN);}
    else{com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq_p,neq_p,ipvt,rp_im,1);}

    final double sigma_i_i=i_i-TMatr.mulVV(rps_im,rp_im);
    this.scale_io=Math.sqrt(o_o*sigma_i_i);
    this.scale_io0=Math.sqrt(o_o*x_i_i);
    this.scaleo_io=Math.sqrt(o_o*i_i);
    this.scaleo_dp_eq=Math.sqrt(o_o*dp_dp);

    this.sigma2_eq=x_dp_dp/o_o;
    this.dp_eq=dp_o;
    this.scale_dip0=Math.sqrt(x_i_i/x_dp_dp);

    this.di=-rp_im[0]*o_o;
    this.dip=rp_im[1];
}
abstract DeprecatedHelperDataInterface<DTYPE> getAccess(final int neq_t);



static abstract class FQVMTest<DTYPE> {

abstract FQVM<DTYPE> getFQVMWithRandomData(final int neq_t,final int neq_i,final java.util.Random r,final int tsize);
abstract EQVM<DTYPE> getEQVM(final int neq_t,final DTYPE odata);
abstract AQVM getAQVM(final int neq_t,final DTYPE odata,final DTYPE [] idata);

final String name;
final double eps,epssum;
FQVMTest(final String name,final double eps,final double epssum)
{
    this.name=name;
    this.eps=eps;
    this.epssum=epssum;
}

void matchEQVM(final int tsize,final java.util.Random r,final int neq_t)
{
    System.err.println(name+":FQVMTest:matchEQVM(): neq_t="+neq_t);
    final FQVM<DTYPE> q=getFQVMWithRandomData(neq_t,0,r,tsize);
    final EQVM<DTYPE> s=getEQVM(neq_t,q.odata);
    UnitTests.checkDoubleRel(q.io,"q.io",s.io,"s.io",eps);
    UnitTests.checkDoubleRel(q.scaleo_io,"q.scaleo_io",s.scaleo_io,"s.scaleo_io",eps);
    UnitTests.checkDoubleRel(q.scaleo_dp_eq,"q.scaleo_dp_eq",s.scaleo_dp_eq,"s.scaleo_dp_eq",eps);
    UnitTests.checkDoubleRel(q.di,"q.di",s.di,"s.di",eps);
    UnitTests.checkDoubleRel(q.dip,"q.dip",s.dip,"s.dip",eps);
    UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",s.scale_io0,"s.scale_io0",eps);
    UnitTests.checkDoubleRel(q.scale_io,"q.scale_io",s.scale_io,"s.scale_io",eps);
    UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",s.sigma2_eq,"s.sigma2_eq",eps);
    UnitTests.checkDoubleRel(q.dip/q.scale_dip0,"q.dip_1/scale",-s.osp,"-s.osp",eps);
    UnitTests.checkDoubleRel(q.scale_dip0,"q.scale_dip0",s.scale_dip0,"s.scale_dip0",eps);
    UnitTests.checkDoubleRel(q.dp_eq,"q.dp_eq",s.dp_eq,"s.dip",eps);
    System.err.println(name+":FQVMTest:matchEQVM(): TEST OK, neq_t="+neq_t);
}

void matchAQVM(final int tsize,final java.util.Random r,final int neq_t,final int neq_i)
{
    System.err.println(name+":FQVMTest:matchAQVM(): neq_t="+neq_t+" neq_i="+neq_i);
    final FQVM<DTYPE> q=getFQVMWithRandomData(neq_t,neq_i,r,tsize);
    final AQVM s=getAQVM(neq_t,q.odata,q.idata);

    UnitTests.checkDoubleRel(q.io,"q.io",s.io,"s.io",eps);
    UnitTests.checkDoubleRel(q.scaleo_io,"q.scaleo_io",s.scaleo_io,"s.scaleo_io",eps);
    UnitTests.checkDoubleRel(q.scaleo_dp_eq,"q.scaleo_dp_eq",s.scaleo_dp_eq,"s.scaleo_dp_eq",eps);
    UnitTests.checkDoubleRel(q.dp_eq,"q.dp_eq",s.dp_eq,"s.dp_eq",eps);
    UnitTests.checkDoubleRel(q.di,"q.di",s.di,"s.di",eps);
    UnitTests.checkDoubleRel(q.dip,"q.dip",s.dip,"s.dip",eps);
    UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",s.scale_io0,"s.scale_io0",eps);
    UnitTests.checkDoubleRel(q.scale_io,"q.scale_io",s.scale_io,"s.scale_io",eps);
    UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",s.sigma2_eq,"s.sigma2_eq",eps);
    UnitTests.checkDoubleRel(q.scale_dip0,"q.scale_dip0",s.scale_dip0,"s.scale_dip0",eps);

    System.err.println(name+":FQVMTest:matchAQVM(): TEST OK, neq_t="+neq_t+" neq_i="+neq_i);
}

void checkExternal(final int tsize,final java.util.Random r,final int neq_t,final int neq_i){}
void checkCalculations(final int tsize,final java.util.Random r,final int neq_t,final int neq_i)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":FQVMTest:checkCalculations(): test with N points="+tsize+
		       " neq_t="+neq_t+" neq_i="+neq_i+" eps="+eps+" epssum="+epssum);
    matchEQVM(tsize,r,neq_t);
    matchAQVM(tsize,r,neq_t,neq_i);
    checkExternal(tsize,r,neq_t,neq_i);
    System.err.println(name+":FQVMTest:checkCalculations(): test OK with N points="+tsize+
		       " neq_t="+neq_t+" neq_i="+neq_i+" eps="+eps+" epssum="+epssum);
}
void runTests()
{
    System.err.println(name+":FQVMTest: starting tests; ver=$Id: FQVM.java,v 1.27 2022/06/27 13:32:45 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79091);
    for(int neq_t=1;neq_t<=12;neq_t++)
    {
	for(int neq_i=0;neq_i<=Math.min(neq_t-4,5);neq_i++)
	{
	    for(int i=0;i<118;i++)
	    {
		checkCalculations(2210+r.nextInt(2210),r,neq_t,neq_i);
	    }
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":FQVMTest: tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: FQVM.java,v 1.27 2022/06/27 13:32:45 mal Exp $");
}

} // END OF class FQVMTest

}// END OF class FQVM
