package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.realtimebook.*;

/** QVM calculations.
 *  @version $Id: FQVMLegendreShifted.java,v 1.12 2019/01/06 23:50:30 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class FQVMLegendreShifted extends FQVM<QVMDataP> {

FQVMLegendreShifted(final int neq_t,final QVMDataP odata,final QVMDataP [] idata)
{
    super(neq_t,odata,idata);
}
DeprecatedHelperQVMDataPAccess getAccess(final int neq_t){return new DeprecatedHelperQVMDataPAccess(neq_t);}

public static FQVMLegendreShifted calculate(final int neq_t,
				     final int neq_i,
				     final long currenttime,
				     final long tau,
				     final Trade [] tb,
				     final int tsize,
				     final Trade [][] itb,
				     final int [] itbsize,
				     final FQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(neq_t,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    final QVMDataP [] i=new QVMDataP [neq_i];
    for(int k=0;k<neq_i;k++)
    {
	i[k]=QVMDataP.calculateQVMDataP(neq_t,
					npm,
					currenttime,
					tau,
					itb[k],
					itbsize[k],
					ref!=null?ref.idata[k]:null);
    }
    return new FQVMLegendreShifted(neq_t,o,i);
}

public static void main(String [] args)
{
    new FQVMTest<QVMDataP>("FQVMLegendreShifted",1e-6,1e-3){
	FQVMLegendreShifted getFQVMWithRandomData(final int neq_t,final int neq_i,final java.util.Random r,final int tsize)
	{
	    final Trade [] o_tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
	    long currenttime=o_tb[o_tb.length-1].time+1000;
	    final long tau=(o_tb[o_tb.length-1].time-o_tb[0].time)/3;
	    final Trade [][] i_tb=new Trade [neq_i][];
	    final int [] i_tb_size=new int [neq_i];
	    for(int i=0;i<neq_i;i++)
	    {
		final Trade [] tb=i_tb[i]=CalculateKendall.TradeEMoments.getRandomTrades(tsize+r.nextInt(tsize+1),r);
		i_tb_size[i]=tb.length;
		if(currenttime<tb[tb.length-1].time)
		{
		    currenttime=tb[tb.length-1].time;
		}
	    }
	    return calculate(neq_t,neq_i,currenttime,tau,o_tb,tsize,i_tb,i_tb_size,null);
	}
	EQVMLegendreShifted getEQVM(final int neq_t,final QVMDataP odata)
	{
	    return new EQVMLegendreShifted(neq_t,odata);
	}
	AQVMLegendreShifted getAQVM(final int neq_t,final QVMDataP odata,final QVMDataP [] idata)
	{
	    final DeprecatedHelperLegendreShiftedBasis [] ibasis=new DeprecatedHelperLegendreShiftedBasis [idata.length];
	    for(int k=0;k<idata.length;k++)
	    {
		ibasis[k]=new DeprecatedHelperLegendreShiftedBasis(idata[k]);
	    }
	    return new AQVMLegendreShifted(new DeprecatedHelperLegendreShiftedBasis(odata),ibasis);
	}
	void checkExternal(final int tsize,final java.util.Random r,final int neq_t,final int neq_i)
	{
	    System.err.println(name+":FQVMTest:QVMPP(): neq_t="+neq_t);
	    final FQVMLegendreShifted q=getFQVMWithRandomData(neq_t,0,r,tsize);
	    final QVMPP s=new QVMPP(neq_t,q.odata,null);
	    UnitTests.checkDoubleRel(q.io,"q.io",s.io,"s.io",eps);
	    UnitTests.checkDoubleRel(q.di,"q.di",s.di_1,"s.di_1",eps);
	    UnitTests.checkDoubleRel(q.dip,"q.dip",s.dip_1,"s.dip_1",eps);
	    UnitTests.checkDoubleRel(q.scaleo_dp_eq,"q.scaleo_dp_eq",s.scaleo_dp_eq,"s.scaleo_dp_eq",eps);
	    UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",s.scale_io0,"s.scale_io0",eps);
	    UnitTests.checkDoubleRel(q.scale_io,"q.scale_io",s.scale_io1,"s.scale_io1",eps);
	    UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",s.sigma2_eq,"s.sigma2_eq",eps);
	    UnitTests.checkDoubleRel(q.dip/q.scale_dip0,"q.dip/scale",-s.osp,"-s.osp",eps);
	    UnitTests.checkDoubleRel(q.scale_dip0,"q.scale_dip0",s.scale_dip0,"s.scale_dip0",eps);
	    System.err.println(name+":FQVMTest:QVMPP(): TEST OK neq_t="+neq_t);
	}
     }.runTests();
}

} // END OF class FQVMLegendreShifted
