package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import java.io.StreamTokenizer;
import com.polytechnik.utils.ReadWriteUtils;

/** A set of utilities to read/write filestem data format.
 *  @version $Id: FilestemData.java,v 1.137 2022/07/28 21:03:02 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class FilestemData implements DataAnalyzable
{

    private static final boolean DO_NOT_SKIP_UNKNOWN_ANTECEDENT_LINES=true;
    private static final boolean PRINT_PRG_DEBUG=true;

    static final int P_DATA=0,P_TEST=1;
    private static final String [] EXTS=new String []{".data",".test"};

    private final String fprefix;
    private final String source,source_id;
    final int data_or_test_type;
    FilestemData(final String fprefix,
		 final String source,
		 final int data_or_test_type,
		 final String source_id)
    {
	this.fprefix=fprefix;
	this.source=source;
	this.data_or_test_type=data_or_test_type;
	this.source_id=source_id;
    }


    private static class FDataStored {
	final java.io.PrintWriter antecedent;
	final java.io.PrintWriter consequent;
	final String fprefix;
	FDataStored(final java.io.PrintWriter antecedent,
		    final java.io.PrintWriter consequent,
		    final String fprefix)
	{
	    this.antecedent=antecedent;
	    this.consequent=consequent;
	    this.fprefix=fprefix;
	}

    }

    private final java.util.Map<String,FDataStored> fd=
	java.util.Collections.synchronizedMap(new java.util.HashMap<String,FDataStored>(4001,0.3f));

    public void setTicker(final String ticker,
			  final CalculateAntecedents calc_antecedents,
			  final CalculateConsequents calc_consequents,
			  final StatusData sd)
    {

	final String tickerescaped;
	final String fprefixt;
	synchronized(this.fd)
	{
	    try {
		tickerescaped=com.polytechnik.outils.EncodeDecode.urlencode(ticker);
		fprefixt=this.fprefix+tickerescaped;
		final FDataStored outx=this.fd.get(ticker);
		if(outx!=null)
		{
		    throw new RuntimeException("The ticker="+ticker+" is already set");
		}
		final java.io.PrintWriter outd=
		    new java.io.PrintWriter(new java.io.FileWriter(fprefixt+".predheader"));
		printAttribsDescription(sd,outd,true,calc_antecedents,calc_consequents);
		outd.close();
		final String fpr=fprefixt+EXTS[this.data_or_test_type];
		this.fd.put(ticker,
			    new FDataStored(new java.io.PrintWriter(new java.io.FileWriter(fpr+"pre.antecedent")),
					    new java.io.PrintWriter(new java.io.FileWriter(fpr+"pre.consequent")),
					    fprefixt));
	    } catch(final java.io.IOException e) {
		throw new RuntimeException(e);
	    }
	}

	try {
	    final java.io.PrintWriter out=new java.io.PrintWriter(new java.io.FileWriter(fprefixt+".prenames"));
	    out.println("|_ticker="+tickerescaped);
	    out.println("|_source="+source);
	    printConsequentDataForExport(out,sd.mparametersdata);
	    printAttribsDescription(sd,out,false,calc_antecedents,calc_consequents);
	    out.close();
	} catch(final java.io.IOException e) {
	    throw new RuntimeException(e);
	}
    }


    private void printConsequentDataForExport(final java.io.PrintWriter out,
					      final ModelParameters.ModelParametersData mparametersdata)
	throws java.io.IOException
    {
	final int nt=mparametersdata.tau_embedding_consequent.length;
	out.print("|_tau=");
	for(int i=0;i<nt;i++)
	{
	    if(i>0)
	    {
		out.print(",");
	    }
	    out.print(String.valueOf(mparametersdata.tau_embedding_consequent[i]/ModelParameters.DISTANCE_TIME_SCALE));
	}
	out.println();
    }


    /** Make a filestem name file.
     */
    private void printAttribsDescription(final StatusData sd,
					 final java.io.PrintWriter out,
					 final boolean flag_header,
					 final CalculateAntecedents calc_antecedents,
					 final CalculateConsequents calc_consequents)
    {
	final StringBuilder buf=new StringBuilder(20);
	final String [] string_consq=calc_consequents.sattribs_names.getInternalBuffer(),
	    string_consq_descr=calc_consequents.sattribs_names.getDescriptionBuffer();
	final int n_string_consq=calc_consequents.sattribs_names.getNValues();
	final String [] double_consequent=calc_consequents.dattribs_names.getInternalBuffer(),
	    double_consequent_descr=calc_consequents.dattribs_names.getDescriptionBuffer();
	final int n_double_consequent=calc_consequents.dattribs_names.getNValues();
	final String [] nominal_consq=calc_consequents.nattribs_names.getInternalBuffer(),
	    nominal_consq_descr=calc_consequents.nattribs_names.getDescriptionBuffer();
	final int n_nominal_consq=calc_consequents.nattribs_names.getNValues();
	if(flag_header)
	{
	    out.print("|");
	}
	else
	{
	    out.print("|_consequent_string_all=");
	    for(int i=0;i<n_string_consq;i++)
	    {
		if(i>0)
		{
		    out.print(",");
		}
		out.print(escape_fs(string_consq[i],buf));
	    }
	    out.println();

	    out.print("|_consequent_double_all=");
	    for(int i=0;i<n_double_consequent;i++)
	    {
		if(i>0)
		{
		    out.print(",");
		}
		out.print(escape_fs(double_consequent[i],buf));
	    }
	    out.println();

	    out.print("|_consequent_nominal_all=");
	    for(int i=0;i<n_nominal_consq;i++)
	    {
		if(i>0)
		{
		    out.print(",");
		}
		out.print(escape_fs(nominal_consq[i],buf));
	    }
	    out.println();

	    out.println("|");
	    out.println("|");
	}
	final String [] string_antcd=calc_antecedents.sattribs_names.getInternalBuffer(),
	    string_antcd_descr=calc_antecedents.sattribs_names.getDescriptionBuffer();
	final int n_string_antcd=calc_antecedents.sattribs_names.getNValues();
	final String [] double_antcd=calc_antecedents.dattribs_names.getInternalBuffer(),
	    double_antcd_descr=calc_antecedents.dattribs_names.getDescriptionBuffer();
	final int n_double_antcd=calc_antecedents.dattribs_names.getNValues();
	final String [] nominal_antcd=calc_antecedents.nattribs_names.getInternalBuffer(),
	    nominal_antcd_descr=calc_antecedents.nattribs_names.getDescriptionBuffer();
	final int n_nominal_antcd=calc_antecedents.nattribs_names.getNValues();
	for(int i=0;i<n_string_antcd;i++)
	{
	    out.print(escape_fs(string_antcd[i],buf));
	    if(flag_header)
	    {
		out.print(",");
	    }
	    else
	    {
		out.print(":");
		out.println(string_antcd_descr[i]);
	    }
	}
	for(int i=0;i<n_double_antcd;i++)
	{
	    out.print(escape_fs(double_antcd[i],buf));
	    if(flag_header)
	    {
		out.print(",");
	    }
	    else
	    {
		out.print(":");
		out.println(double_antcd_descr[i]);
	    }
	}
	for(int i=0;i<n_nominal_antcd;i++)
	{
	    out.print(escape_fs(nominal_antcd[i],buf));
	    if(flag_header)
	    {
		out.print(",");
	    }
	    else
	    {
		out.print(":");
		out.println(nominal_antcd_descr[i]);
	    }
	}



	if(ClassificationPredictorNewTrade.DEBUGGING_OUT_RULESCALCULATION)
	{
	    // DEBUGGING
	    out.print("__xxattrib__");
	    if(flag_header)
	    {
		out.print(",");
	    }
	    else
	    {
		out.println(":xEOL,EOL");
	    }
	    final String [] p1=new String []{
		"R_"+FilestemData.escape_fs(CalculateConsequents.PMIN_PREFIX,buf)+"_discr_",
		"R_"+FilestemData.escape_fs(CalculateConsequents.PMED_SELL_PREFIX,buf)+"_discr_",
		"R_"+FilestemData.escape_fs(CalculateConsequents.PMAX_PREFIX,buf)+"_discr_",
		"R_"+FilestemData.escape_fs(CalculateConsequents.PMED_BUY_PREFIX,buf)+"_discr_",
		"R_"+FilestemData.escape_fs(CalculateConsequents.PTAU_BUY_PREFIX,buf)+"_discr_",
		"R_"+FilestemData.escape_fs(CalculateConsequents.PTAU_SELL_PREFIX,buf)+"_discr_"
	    };
	    final String [] p3=new String []{"_pred","_confidence","_rules_matched"};
	    if(p1.length!=PrepareFilestems.N_PREEDICTORS)
	    {
		throw new IllegalStateException("array size mismatch");
	    }
	    for(int i=0;i<sd.mparametersdata.tau_embedding_consequent.length;i++)
	    {
		final String t=String.valueOf(sd.mparametersdata.tau_embedding_consequent[i]/ModelParameters.DISTANCE_TIME_SCALE);
		for(int j=0;j<PrepareFilestems.N_PREEDICTORS;j++)
		{
		    final String nmp=p1[j]+t;
		    for(int k=0;k<p3.length;k++)
		    {
			final String nm=nmp+p3[k];
			out.print(nm);
			if(flag_header)
			{
			    out.print(",");
			}
			else
			{
			    out.println(":ignore");
			}
		    }
		}
	    }
	    // END OF DEBUGGING
	}



	for(int i=0;i<n_string_consq;i++)
	{
	    out.print(escape_fs(string_consq[i],buf));
	    if(flag_header)
	    {
		out.print(",");
	    }
	    else
	    {
		out.print(":");
		out.println(string_consq_descr[i]);
	    }
	}

	for(int i=0;i<n_double_consequent;i++)
	{
	    out.print(escape_fs(double_consequent[i],buf));
	    if(flag_header)
	    {
		out.print(",");
	    }
	    else
	    {
		out.print(":");
		out.print(double_consequent_descr[i]);
		out.println(" |:continuous |consequent_class");
	    }
	}
	for(int i=0;i<n_nominal_consq;i++)
	{
	    out.print(escape_fs(nominal_consq[i],buf));
	    if(flag_header)
	    {
		out.print(",");
	    }
	    else
	    {
		out.print(":");
		out.print(nominal_consq_descr[i]);
		out.println(" |:0,1 |consequent_class");
	    }
	}
	out.print("__source_id__");
	if(flag_header)
	{
	    out.print(",");
	}
	else
	{
	    out.println(":ignore");
	}
	out.print("__Lattrib__");
	if(flag_header)
	{
	    out.println();
	}
	else
	{
	    out.println(":EOL1,EOL");
	}
    }


    private int printDataConsequent(final java.io.PrintWriter out,
				    final CalculateConsequents calc_consequents)
    {
	int n=0;
	final String [] string_consq=calc_consequents.sattribs.getInternalBuffer();
	final int n_string_consq=calc_consequents.sattribs.getNValues();
	final double [] double_consequent=calc_consequents.dattribs.getInternalBuffer();
	final int n_double_consequent=calc_consequents.dattribs.getNValues();
	final int [] nominal_consq=calc_consequents.nattribs.getInternalBuffer();
	final int n_nominal_consq=calc_consequents.nattribs.getNValues();

	final StringBuilder buf=new StringBuilder(20);
	// strings
	for(int i=0;i<n_string_consq;i++,n++)
	{
	    out.print(escape_fs(string_consq[i],buf));
	    out.print(",");
	}
	// double
	for(int i=0;i<n_double_consequent;i++,n++)
	{
	    if(Double.isNaN(double_consequent[i])||Double.isInfinite(double_consequent[i]))
	    {
		out.print("?,");
	    }
	    else
	    {
		out.print(double_consequent[i]);
		out.print(",");
	    }
	}
	// nominal
	for(int i=0;i<n_nominal_consq;i++,n++)
	{
	    if(nominal_consq[i]<0)
	    {
		out.print("?,");
	    }
	    else
	    {
		out.print(nominal_consq[i]);
		out.print(",");
	    }
	}
	return n;
    }


    private int printDataAntecedent(final java.io.PrintWriter out,
				     final CalculateAntecedents calc_antecedents)
    {
	int n=0;
	final String [] string_antcd=calc_antecedents.sattribs.getInternalBuffer();
	final int n_string_antcd=calc_antecedents.sattribs.getNValues();
	final double [] double_antcd=calc_antecedents.dattribs.getInternalBuffer();
	final int n_double_antcd=calc_antecedents.dattribs.getNValues();
	final int [] nominal_antcd=calc_antecedents.nattribs.getInternalBuffer();
	final int n_nominal_antcd=calc_antecedents.nattribs.getNValues();

	final StringBuilder buf=new StringBuilder(20);
	// string
	for(int i=0;i<n_string_antcd;i++,n++)
	{
	    out.print(escape_fs(string_antcd[i],buf));
	    out.print(",");
	}
	// double
	for(int i=0;i<n_double_antcd;i++,n++)
	{
	    if(Double.isNaN(double_antcd[i])||Double.isInfinite(double_antcd[i]))
	    {
		out.print("?,");
	    }
	    else
	    {
		out.print(double_antcd[i]);
		out.print(",");
	    }
	}
	// nominal
	for(int i=0;i<n_nominal_antcd;i++,n++)
	{
	    out.print(nominal_antcd[i]);
	    out.print(",");
	}
	return n;
    }


    int __DUMPprintAntecedents(final long record_id,
			       final String ticker,
			       final CalculateAntecedents calc_antecedents,
			       final ClassificationPredictor.PredictionResults [][] predictors)
    {
	if(!ClassificationPredictorNewTrade.DEBUGGING_OUT_RULESCALCULATION)
	{
	    return -1;
	}
	int n=0;
	final java.io.PrintWriter out=this.fd.get(ticker).antecedent;
	out.print(record_id);
	out.print(':');
	n+=printDataAntecedent(out,calc_antecedents);
	out.print("xEOL");
	// insert rule matching results for later comparison with the original classifier.
	for(int i=0;i<predictors.length;i++)
	{
	    for(int it=0;it<predictors[i].length;it++)
	    {
		final ClassificationPredictor.PredictionResults rr=predictors[i][it];
		out.print(",");
		out.print(rr.result);
		out.print(",");
		out.print(rr.confidence);
		out.print(",");
		out.print(rr.extra_description.length()>0?rr.extra_description:"N/A");
	    }
	}

	out.print(",");
	out.println("vEOL");
	return n;
    }


    public int printAntecedents(final long record_id,
				final String ticker,
				final CalculateAntecedents calc_antecedents)
    {
	int n=0;
	final java.io.PrintWriter out=this.fd.get(ticker).antecedent;
	out.print(record_id);
	out.print(':');
	n+=printDataAntecedent(out,calc_antecedents);
	out.println("vEOL");
	return n;
    }


    public int printConsequents(final long record_id,
				final String ticker,
				final CalculateConsequents calc_consequents)
    {
	int n=0;
	final java.io.PrintWriter out=this.fd.get(ticker).consequent;
	out.print(record_id);
	out.print(':');
	n+=printDataConsequent(out,calc_consequents);

	final double [] double_consequent=calc_consequents.dattribs.getInternalBuffer();
	int n_non_nan=calc_consequents.dattribs.getNValues();
	for(int i=n_non_nan;i-- >0;)
	{
	  if(Double.isNaN(double_consequent[i]))
	  {
	      n_non_nan--;
	  }
	}
	out.println(!DO_NOT_SKIP_UNKNOWN_ANTECEDENT_LINES && n_non_nan<=0?"SKIP":"vEOL");
	return n;
    }



    private static long getID(String s,String [] set)
    {
	final int l=s.indexOf(':');
	if(l<0)
	{
	    throw new RuntimeException("Invalid string "+s);
	}
	if(s.endsWith(",vEOL"))
	{
	    set[0]=s.substring(l+1,s.length()-4 /* vEOL length */);
	}
	else if(s.endsWith(",SKIP"))
	{
	    set[0]=null;
	}
	else
	{
	    throw new RuntimeException("Invalid string "+s);
	}
	return Long.parseLong(s.substring(0,l));
    }


    int makedatafile(final String fstem,
		     final int n_data_already_saved,
		     final int n_test_already_saved)
	throws java.io.IOException
    {
	final String f1=fstem+".predheader";
	final String hdrline=ReadWriteUtils.getFileContent(f1).trim();

	for(int d : new int[]{P_DATA,P_TEST})
	{
	    if((n_data_already_saved>0 && d==P_DATA) ||
	       (n_test_already_saved>0 && d==P_TEST))
	    {
		final String f=fstem+EXTS[d];
		final java.io.BufferedReader rh=
		    new java.io.BufferedReader(new java.io.FileReader(f));
		final String hdrline2=rh.readLine().trim();
		rh.close();
		if(!hdrline.equals(hdrline2)||hdrline2.indexOf(',')<0)
		{
		    throw new RuntimeException("Header mismatch in "+f+" and "+f1);
		}
	    }
	}
	final int n_already_saved=this.data_or_test_type==P_DATA?
	    n_data_already_saved:n_test_already_saved;

	final String fname=fstem+EXTS[this.data_or_test_type];
	System.err.println("Saving data to "+fname);
	final java.io.PrintWriter out;
	if(n_already_saved<=0)
	{
	    out=new java.io.PrintWriter(new java.io.FileWriter(fname));
	    out.println(hdrline);
	}
	else
	{
	    out=new java.io.PrintWriter(new java.io.FileWriter(fname,true));
	}

	final String fpr=fstem+EXTS[this.data_or_test_type];
	final java.io.BufferedReader r2=
	    new java.io.BufferedReader(new java.io.FileReader(fpr+"pre.antecedent"));
	r2.readLine(); // skip first line
	final java.io.BufferedReader r3=
	    new java.io.BufferedReader(new java.io.FileReader(fpr+"pre.consequent"));
	String s3;
	final String [] set=new String [1];
	int n=0,nrec=n_already_saved;
	while(null!=(s3=r3.readLine()))
	{
	    n++;
	    final long id3=getID(s3,set);
	    final String ctxt=set[0];
	    final String s2=r2.readLine();
	    final long id2=getID(s2,set);
	    if(id2!=id3)
	    {
		throw new RuntimeException("ID mismatch "+id2+" and "+id3+" line="+n);
	    }
	    final String atxt=set[0];
	    if(ctxt==null)
	    {
		continue;
	    }
	    nrec++;
	    out.print(atxt);
	    out.print(ctxt);
	    out.print(this.source_id);
	    out.print(",");
	    out.println("EOL");
	}
	out.close();
	r2.close();
	r3.close();

	// append the number or lines read to .names file
	final java.io.PrintWriter outn=new java.io.PrintWriter(new java.io.FileWriter(fstem+".names"));
	if(this.data_or_test_type==P_DATA)
	{
	    outn.println("|_n_data_lines="+nrec);
	    outn.println("|_n_test_lines="+n_test_already_saved);
	}
	else if(this.data_or_test_type==P_TEST)
	{
	    outn.println("|_n_data_lines="+n_data_already_saved);
	    outn.println("|_n_test_lines="+nrec);
	}
	else
	{
	    throw new IllegalArgumentException();
	}
	outn.print(ReadWriteUtils.getFileContent(fstem+".prenames"));
	outn.close();

	System.err.println("Converted "+nrec+" lines to "+fname);

	return nrec;
    }



    /** A very specific espace for filestem format.
     */
    static String escape_fs(final String val,final StringBuilder buf)
    {
	final int val_len=val.length();
	buf.setLength(0);
	buf.ensureCapacity(2*val_len+1);
	for(int i = 0; i<val_len; i++)
	{
	    final char c = val.charAt(i);
	    switch(c) {
	    case '.':
		buf.append("_");
		break;
	    case ',':
		buf.append("_");
		break;
	    case '|':
	    case ':':
		buf.append("_");
		break;
	    case ' ':
	    case '\t':
		buf.append("_");
		break;
	    default:
		if(c<' ')
		{
		    buf.append("_");
		}
		else
		{
		    buf.append(c);
		}
	    }
	}
	return buf.toString();
    }



    void closeAll(final PrepareFilestemsPostConversion processor)
	throws java.io.IOException
    {
	for(FDataStored out : this.fd.values())
	{
	    out.antecedent.close();
	    out.consequent.close();
	    if(processor!=null)
	    {
		processor.accept(out.fprefix,this);
	    }
	}
	this.fd.clear();
    }




    private static int checkTimeserieHeader(final StreamTokenizer tk,
					    final NamesFileData nd,
					    final String filename)
	throws java.io.IOException
    {
	final String [] rdata=new String [nd.all_attribs.length];
	final int n_columns=ReadWriteUtils.readOneLine(tk,rdata);
	if(n_columns!=nd.all_attribs.length)
	{
	    throw new RuntimeException("Invalid data:Columns number mismatch: "+
				       n_columns+" and "+nd.all_attribs.length+
				       " in files "+filename+" and "+nd.filename);
	}
	for(int i=0;i<rdata.length;i++)
	{
	    if(i<=0)
	    {
		for(;rdata[0].startsWith("|");)
		{
		    rdata[0]=rdata[0].substring(1);
		}
	    }
	    if(!rdata[i].trim().equals(nd.all_attribs[i]))
	    {
		throw new RuntimeException("Attribute name mismatch: "+rdata[i]+
					   " and "+nd.all_attribs[i]+
					   " in files "+filename+" and "+nd.filename);
	    }
	}
	return 1; // 1 line read
    }


    /** Tokenises the file content and calls
     *  processor.test(String []) for each line.
     */
    public static void processDataFile(final NamesFileData nd,
				       final java.util.function.Predicate<String []> processor,
				       final int data_or_test_type)
	throws java.io.IOException
    {

	final String filename=nd.getFData(data_or_test_type);

	if(PRINT_PRG_DEBUG)
	{
	    System.err.print("processDataFile:: file="+filename+" TYPE="+EXTS[data_or_test_type]);
	}
	final java.io.BufferedReader br=
	    new java.io.BufferedReader(new java.io.FileReader(filename));
	final StreamTokenizer tk=ReadWriteUtils.getTokenizer(",",br);
	tk.quoteChar('\"');
	final int nd_lines=nd.getFLines(data_or_test_type);

	int nlinesread=checkTimeserieHeader(tk,nd,filename),npointsread=0;
        final String [] sread=new String [nd.all_attribs.length];
	for(;tk.ttype!=StreamTokenizer.TT_EOF;)
	{
	    final int nrecordsinline=ReadWriteUtils.readOneLine(tk,sread);
	    nlinesread++;
            // System.err.println(" LINE READ, "+nrecordsinline+" records.");
	    if(tk.ttype==StreamTokenizer.TT_EOF && nrecordsinline<=0)
	    {
		// EOF after LF
		break;
	    }
            if(nrecordsinline!=sread.length)
	    {
                if(nrecordsinline>0) // non-empty line
		{
		    System.err.println(
			"Invalid line #"+nlinesread+" STR="+
			java.util.Arrays.toString(java.util.Arrays.copyOf(sread,nrecordsinline))+
			" read from data file. namsfile="+filename+
			" with nrecordsinline="+nrecordsinline+
			" should be="+sread.length);
		}
		else
		{
		    System.err.println("Empty line #"+nlinesread+
				       " read from data file. namsfile="+filename);
		}
		throw new RuntimeException(
			"Invalid line #"+nlinesread+" STR="+
			java.util.Arrays.toString(java.util.Arrays.copyOf(sread,nrecordsinline))+
			" read from data file. namsfile="+filename);
	    }
            else
	    { // line OK, process it
		if(++npointsread>nd_lines)
		{
		    throw new RuntimeException("The number of points mismatch in "+
					       filename+"(>"+npointsread+" points) and "+
					       nd.filename+"("+nd_lines+" points for "+
					       EXTS[data_or_test_type]+").");
		}
		processor.test(sread);
	    }
	}
	br.close();
	if(PRINT_PRG_DEBUG)
	{
	    System.err.println(" read "+nlinesread+" lines; npointsread="+
			       npointsread+".");
	}
	if(npointsread!=nd_lines)
	{
	    throw new RuntimeException("The number of points mismatch in "+
				       filename+"("+npointsread+" points) and "+
				       nd.filename+"("+nd_lines+" points for "+
				       EXTS[data_or_test_type]+").");
	}
    }



    static class NamesFileData {
	final String filename;
	final String source;
	final String [] tau_consequent;
	final String ticker;
	final String [] all_attribs;
	private final int n_data_lines,n_test_lines;

	private NamesFileData(final String filename,
			      final String source,
			      final String [] tau_consequent,
			      final String ticker,
			      final String [] all_attribs,
			      final int n_data_lines,
			      final int n_test_lines)
	{
	    this.filename=filename;
	    this.source=source;
	    this.tau_consequent=tau_consequent;
	    this.ticker=ticker;
	    this.all_attribs=all_attribs;
	    this.n_data_lines=n_data_lines;
	    this.n_test_lines=n_test_lines;
	}

	String getPrefix()
	{
	    if(!this.filename.endsWith(".names"))
	    {
		throw new RuntimeException("Invalid fname="+this.filename);
	    }
	    return this.filename.substring(0,this.filename.length()-".names".length());
	}


	String getFData(final int data_or_test_type)
	{
	    return getPrefix()+EXTS[data_or_test_type];
	}

	int getFLines(final int data_or_test_type)
	{
	    return data_or_test_type==P_DATA?this.n_data_lines:this.n_test_lines;
	}


	static NamesFileData readNamesFileData(final String filename)
	    throws java.io.IOException
	{
	    final java.io.BufferedReader r=
		new java.io.BufferedReader(new java.io.FileReader(filename));
	    String [] tau_consequent=null;
	    String source=null,ticker=null;
	    int n_data_lines=-1,n_test_lines=-1;
	    final java.util.ArrayList<String> all_attribs=new java.util.ArrayList<String>();
	    for(String s;null!=(s=r.readLine());)
	    {
		if(s.startsWith("|_"))
		{ // pseudocomment
		    final int p=s.indexOf("=");
		    if(p<0)
		    {
			throw new RuntimeException();
		    }
		    final String n=s.substring(2,p);
		    final String v=s.substring(p+1).trim();
		    if(n.equals("source"))
		    {
			source=v;
		    }
		    else if(n.equals("ticker"))
		    {
			ticker=com.polytechnik.outils.EncodeDecode.urldecode(v);
		    }
		    else if(n.equals("tau"))
		    {
			final String [] vl=ReadWriteUtils.extractnames(v,", \t\r\n");
			tau_consequent=new String [vl.length];
			for(int i=tau_consequent.length; i-- >0;)
			{
			    tau_consequent[i]=String.valueOf(Long.valueOf(vl[i]));
			}
		    }
		    else if(n.equals("n_data_lines"))
		    {
			n_data_lines=Integer.parseInt(v);
		    }
		    else if(n.equals("n_test_lines"))
		    {
			n_test_lines=Integer.parseInt(v);
		    }
		}
		else if(!s.startsWith("|"))
		{
		    final int p=s.indexOf(":");
		    if(p>=0)
		    {
			all_attribs.add(s.substring(0,p).trim());
		    }
		}
	    }
	    r.close();

	    return new NamesFileData(filename,
				     source,
				     tau_consequent,
				     ticker,
				     all_attribs.toArray(String[]::new),
				     n_data_lines,
				     n_test_lines);
	}
    }

}
