package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;

/** A class to perform simple parser testing.
 *  The data is read from a file.
 *  @version $Id: GenerateStats.java,v 1.10 2022/07/28 21:03:02 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class GenerateStats extends com.polytechnik.itch.DummyTrader {

    static final boolean PRINT_DEBUG=true;

    private final String out_dir,source;
    private final java.util.HashMap<String,CalculateStats> map=
	new java.util.HashMap<String,CalculateStats>(6001,0.3f);
    GenerateStats(final String [] instruments,
		  final String out_dir,
		  final String source)
    {
	for(String s:instruments)
	{
	    map.put(s,new CalculateStats());
	}
	this.out_dir=out_dir;
	this.source=source;
	if(PRINT_DEBUG)
	{
	    System.err.println("======================================================");
	    System.err.println("Generating stats for "+
			       java.util.Arrays.toString(instruments));
	    System.err.println("By $Id: GenerateStats.java,v 1.10 2022/07/28 21:03:02 mal Exp $");
	    System.err.println("At "+new java.util.Date());
	    System.err.println("Source "+this.source);
	    System.err.println("Output directory "+this.out_dir);
	    System.err.println("======================================================");
	}
    }

    static final long T_START=TraderInterface.TIME_SCALE*(10*3600+0*60); // 10:00
    static final long T_END=TraderInterface.TIME_SCALE*(15*3600+30*60); // 15:30
    static final long T_QUANT=TraderInterface.TIME_SCALE*1; // 1 second

    @Override
    public void update(final OrdersAndTrades ot,
		       final long currenttime,
		       final int eventid){
	ot.clearBefore(currenttime); // clear trades
	if(!(T_START<=currenttime && T_START<=T_END))
	{
	    return;
	}
	final CalculateStats s=map.get(ot.ticker);
	if(s==null)
	{
	    return;
	}
	if(!s.shouldCheck(currenttime))
	{
	    return;
	}
	s.setNextCheck(currenttime+T_QUANT);
	s.makeContrib(ot.orderbook,currenttime);
    }


    void printStats() throws java.io.IOException
    {
	for(String t:map.keySet())
	{
	    final CalculateStats c=map.get(t);
	    c.printStats(out_dir,source,t);
	}
    }

    void prepareForNextSourceToRun() throws java.io.IOException
    {
	for(CalculateStats c:map.values())
	{
	    c.prepareForSecondSourceToRun();
	}
    }


    private static class CalculateStats implements
	    com.polytechnik.utils.LongHashMap.TLongObjectProcedure<BookItem>
    {
	long nexttime_check=0,prev_t=-1;
	void setNextCheck(final long nextcheck)
	{
	    this.nexttime_check=nextcheck;
	}
	boolean shouldCheck(final long currenttime)
	{
	    return currenttime>=this.nexttime_check;
	}


	double sum_ap=0,sum_s=0,sum_ds=0,sum_t=0;
	long n_sum=0,n_calls=0;


	double min_sell=Double.POSITIVE_INFINITY,max_buy=Double.NEGATIVE_INFINITY;
	void makeContrib(final RealtimeBook book,final long currenttime)
	{
	    final long t=prev_t;
	    prev_t=currenttime;
	    if(t<=0)
	    {
		return;
	    }
	    final long d=currenttime-t;
	    if(d<=0)
	    {
		return;
	    }
	    min_sell=Double.POSITIVE_INFINITY;
	    max_buy=Double.NEGATIVE_INFINITY;
	    book.forEachEntry(this);

	    if(!(max_buy<min_sell))
	    {
		return;
	    }
	    n_sum++;
	    final double s=min_sell-max_buy,dd=d;
	    sum_s+=s*dd;
	    sum_ds+=s*s*dd;
	    sum_t+=dd;
	    sum_ap+=0.5*(min_sell+max_buy)*dd;
	}


	public boolean execute(final long id,final BookItem b)
	{
	    n_calls++;
	    if(b.buy_sell_indicator==BookItem.BOOK_BUY)
	    {
		if(b.price>max_buy)
		{
		    max_buy=b.price;
		}
	    }
	    else if(b.buy_sell_indicator==BookItem.BOOK_SELL)
	    {
		if(b.price<min_sell)
		{
		    min_sell=b.price;
		}
	    }
	    return true;
	}


	double getAverSpread()
	{
	    return sum_s/sum_t;
	}
	double getStdDevSpread()
	{
	    final double s=getAverSpread();
	    return Math.sqrt(sum_ds/sum_t-s*s);
	}

	double getPAver()
	{
	    return sum_ap/sum_t;
	}


	void prepareForSecondSourceToRun()
	{
	    nexttime_check=0;
	    prev_t=-1;
	}



	void printStats(final String out_dir,
			final String source,
			final String ticker) throws java.io.IOException
	{

	    final String fname=out_dir+
		com.polytechnik.outils.EncodeDecode.urlencode(ticker)+
		".stat";
	    final java.io.PrintWriter out=new java.io.PrintWriter(fname);
	    out.println("GENERATED=\""+new java.util.Date()+"\"");
	    out.println("GENERATOR=\"$Id: GenerateStats.java,v 1.10 2022/07/28 21:03:02 mal Exp $\"");
	    out.println("DATASOURCE=\""+source+"\"");
	    out.println("TICKER=\""+ticker+"\"");
	    out.println("SPREAD=\""+getAverSpread()+"\"");
	    out.println("SPREAD_STDDEV=\""+getStdDevSpread()+"\"");
	    out.println("PRICE_AVER=\""+getPAver()+"\"");
	    out.println("TERMS ADDED=\""+n_sum+"\"");
	    out.close();
	}
    }


    public static void main(String [] args)
	throws Exception
    {

	if(args.length<3)
	{
	    System.err.println("GenerateStats dir_out securities_file itch_file_1 itch_file_2 ... ");
	    System.exit(1);
	}
	final long t0=System.currentTimeMillis();
	System.err.println("Version");
	System.err.println("Started="+new java.util.Date(t0));
	System.getProperties().list(System.err);
	System.err.println("END Version");
	final StringBuilder buf=new StringBuilder();
	for(int i=2;i<args.length;i++)
	{
	    buf.append(args[i]);
	    buf.append("; ");
	}
	final GenerateStats gs=new GenerateStats(GenerateTrainingData.getInstrimentsListFromFile(args[1]),
						 args[0],
						 buf.toString());
	for(int i=2;i<args.length;i++)
	{
	    gs.prepareForNextSourceToRun();
	    gs.runParser(args[i]);
	}
	gs.printStats();

	final long t1=System.currentTimeMillis();
	System.err.println("Finished="+new java.util.Date(t1)+" Started="+new java.util.Date(t0));
	System.err.println("Processed "+gs.source+" in "+
			   ((t1-t0)/1000L)+
			   " seconds. by $Id: GenerateStats.java,v 1.10 2022/07/28 21:03:02 mal Exp $"+
			   "\nnow="+new java.util.Date(t1));
    }

}
