package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */


/** A class to perform simple parser testing.
 *  The data is read from a file.
 *  @version $Id: GenerateTrainingData.java,v 1.70 2022/06/07 10:52:26 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class GenerateTrainingData {

    private static final int N_PROCESSES_TO_START=1;

    static class ParserData {
	private final com.polytechnik.itch.ITCHParser parser;
	ParserData(final DataAnalyzable anl,
		   final ModelParameters modelparameters)
	{
	    this.parser=new  com.polytechnik.itch.ITCHParser(
		new Trader(anl,0,modelparameters,null,true,false,"??")
							     );
	}
	void runParser(final String file) throws java.io.IOException
	{
	    new com.polytechnik.itch.FileFeeder(parser,file).processStream();
	}
    } // END OF class ParserData

    /** Read tickers. One per line.
     */
    public static String [] getInstrimentsListFromFile(final String fname)
	throws java.io.IOException
    {
	final java.io.BufferedReader rh=new java.io.BufferedReader(new java.io.FileReader(fname));
	final java.util.ArrayList<String> a=new java.util.ArrayList<String>(50);
	for(String l;(l=rh.readLine())!=null;)
	{
	    l=l.trim();
	    if(l.startsWith("#")||l.length()<=0)
	    {
		continue;
	    }
	    a.add(l);
	}
	return a.toArray(String[]::new);
    }

    static class AParams {
	final String fprefix,inprefix;
	final String [] instruments_list;
	final String [] data;
	final String [] test;

	AParams(final String fprefix,
		final String inprefix,
		final String [] instruments_list,
		final String [] data,
		final String [] test)
	{
	    this.fprefix=fprefix;
	    this.inprefix=inprefix;
	    this.instruments_list=instruments_list;
	    this.data=data;
	    this.test=test;
	}
	String getDescription()
	{
	    final StringBuilder buf=new StringBuilder();
	    buf.append("IN_DIR="+inprefix+"\n");
	    buf.append("OUT_DIR="+fprefix+"\n");
	    buf.append("INSTRUMENT LIST:\n");
	    for(int i=0;i<instruments_list.length;i++)
	    {
		buf.append("\t"+instruments_list[i]+"\n");
	    }
	    buf.append("INPUT DATA FILES:\n");
	    for(int i=0;i<data.length;i++)
	    {
		buf.append("\t"+data[i]+"\n");
	    }
	    buf.append("INPUT TEST FILES:\n");
	    for(int i=0;i<test.length;i++)
	    {
		buf.append("\t"+test[i]+"\n");
	    }
	    return buf.toString();
	}
	static AParams getArgsProcessed(String [] args)
	    throws java.io.IOException
	{
	    if(args.length<4)
	    {
		System.err.println("GenerateTrainingData dir_out dir_in securities_file itch_file_1 itch_file_2 .... [-t itch_test_file_1 itch_test_file_2 ...] ");
		return null;
	    }
	    final String fprefix=args[0];
	    if(!fprefix.endsWith("/"))
	    {
		throw new RuntimeException("fprefix="+fprefix+" does not end with a /");
	    }
	    final String inprefix=args[1];
	    if(!inprefix.endsWith("/"))
	    {
		throw new RuntimeException("inprefix="+inprefix+" does not end with a /");
	    }
	    final String [] instruments_list=getInstrimentsListFromFile(args[2]);

	    final java.util.ArrayList<String> data=new java.util.ArrayList<String>();
	    final java.util.ArrayList<String> test=new java.util.ArrayList<String>();
	    boolean flag_data=true;
	    for(int i=3;i<args.length;i++)
	    {
		final String s=args[i];
		if(s.equals("-t"))
		{
		    flag_data=false;
		    continue;
		}
		if(s.equals("-d"))
		{
		    flag_data=true;
		    continue;
		}

		if(flag_data)
		{
		    data.add(s);
		}
		else
		{
		    test.add(s);
		}
	    }
	    if(data.size()<=0)
	    {
		System.err.println("GenerateTrainingData dir_out securities_file itch_file_1 itch_file_2 .... [-t itch_test_file_1 itch_test_file_2 ...] ");
		return null;
	    }
	    return new AParams(fprefix,
			       inprefix,
			       instruments_list,data.
			       toArray(String[]::new),
			       test.toArray(String[]::new));
	}
    }

    public static void main(String [] args)
	throws Exception
    {
	final AParams p=AParams.getArgsProcessed(args);
	if(p==null)
	{
	    System.exit(1);
	}

	final long t0=System.currentTimeMillis();
	System.err.println("$Id: GenerateTrainingData.java,v 1.70 2022/06/07 10:52:26 mal Exp $"+
			   "\nnow="+new java.util.Date(t0)+
			   "\n"+p.getDescription());
	System.err.println("Version");
	System.err.println("Started="+new java.util.Date(t0));
	System.getProperties().list(System.err);
	System.err.println("END Version");

	final ModelParameters m=new ModelParameters(p.inprefix,
						    p.fprefix,
						    p.instruments_list);
	final PrepareFilestemsPostConversion postconversion=
	    new PrepareFilestemsPostConversion(N_PROCESSES_TO_START,m);
	postconversion.startPostProcess();

	final StringBuilder buf=new StringBuilder();
	final int nd=p.data.length,nt=p.test.length,nf=nd+nt;
	int n=0;
	for(int i=0;i<nf;i++)
	{
	    final int ftype;
	    final String txt,f;
	    if(i<nd)
	    { // data
		f=p.data[i];
		ftype=FilestemData.P_DATA;
		txt="DATA";
	    }
	    else
	    { // test
		f=p.test[i-nd];
		ftype=FilestemData.P_TEST;
		txt="TEST";
	    }
	    System.err.println("Processing "+f+" ("+txt+") by $Id: GenerateTrainingData.java,v 1.70 2022/06/07 10:52:26 mal Exp $");
	    buf.append(f);
	    buf.append("(");
	    buf.append(txt);
	    buf.append(")");
	    buf.append("; ");
	    final FilestemData fd=new FilestemData(p.fprefix,buf.toString(),ftype,String.valueOf(i));
	    System.err.println("GenerateTrainingData processing "+f+" as "+txt+" file.");
	    new ParserData(fd,m).runParser(f);
	    if(i+1==nf)
	    {
		// last file
		postconversion.setToDoDescritization();
	    }
	    fd.closeAll(postconversion);
	    System.err.println("GenerateTrainingData DONE processing "+f);
	    System.err.flush();
	    n++;
	}
	postconversion.finishPostProcess();

	final long t1=System.currentTimeMillis();
	System.err.println("Finished="+new java.util.Date(t1)+" Started="+new java.util.Date(t0));
	System.err.println("Processed "+n+" files: "+nd+" DATA "+nt+" TEST "+" in "+
			   ((t1-t0)/1000L)+
			   " seconds. by $Id: GenerateTrainingData.java,v 1.70 2022/06/07 10:52:26 mal Exp $"+
			   "\nnow="+new java.util.Date(t1));
    }
}
