package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: IAImpact.java,v 1.103 2022/11/19 11:08:57 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class IAImpact<DTYPE> {

public static class State {
    public final long seq;
    State(final State prev)
    {
	this.seq=(prev==null?0:1+prev.seq);
    }
}
final  com.polytechnik.utils.VolMatrix VOLATILITY_EDPsi=this::getEDPsi;

public final State sst;

public final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
public final EVXData I,A,IA,IdpAA,Ia,PnL,PnLa,PnLad,Vmax;
public final double varA1,p_IH,pt_IH,pa_IH,dpn_IH,dpi_IH,varp1_IAH,pi_IAH,pa_IAH,dp_IAH,wv_IAH,wt_IAH,dp_IAL,PnL_H_dp,PnL_H_dir,PnLa_H_dir,PnLad_H_dir,dm_IAH;
public final Skewness sk_IAH,sk_PnL,sk_PnLa,sk_Vmax,sk_PnLad,sk_I,sk_Ia;
public final Mu2IAroots iad;
public final Mu2IArootsA iar;
public final Mu2F pIH,pAH,pIAH;
public final Mu2IA mu2ia;
public final double Fdp_IH,Fad_IH,FDpwv_IH,Fv_IH,FDpwt_IH,Ft_IH,dpw_IH;
final DTYPE odata;
/** Not worth a function or costructor argument.
 */
private static double EPS_KLUDGY_WAY_TO_TURN_ON_THE_DEBUGGING=0;
IAImpact(final int n,final DTYPE odata,final State rst)
{
    //if(n<=1) throw new RuntimeException("n="+n+" must be >1");
    final double eps=EPS_KLUDGY_WAY_TO_TURN_ON_THE_DEBUGGING;
    this.odata=odata;
    this.M=getABasis();
    final JMomentsData mm=getJMomentsData(n);
    // Our "old" solution p_IH: price corresponding the state of max I of past sample.
    this.I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH;
    pt_IH=TMatr.getAMatrA(I.psiH,mm.QQpt_sample,n)/TMatr.getAMatrA(I.psiH,mm.QQt_sample,n);
    pa_IH=TMatr.getAMatrA(I.psiH,mm.QQpa,n)/TMatr.getAMatrA(I.psiH,mm.QQa,n);
    dpn_IH=TMatr.getAMatrA(I.psiH,mm.QQdpdt,n)/TMatr.getAMatrA(I.psiH,mm.QQad,n);
    dpi_IH=TMatr.getAMatrA(I.psiH,mm.QQDpwv,n)/TMatr.getAMatrA(I.psiH,mm.QQi,n);
    if(mm.QQFdp!=null){
	Fdp_IH=TMatr.getAMatrA(I.psiH,mm.QQFdp,n);
	Fad_IH=TMatr.getAMatrA(I.psiH,mm.QQFad,n);
	FDpwv_IH=TMatr.getAMatrA(I.psiH,mm.QQFDpwv,n);
	Fv_IH=TMatr.getAMatrA(I.psiH,mm.QQFi,n);
	FDpwt_IH=TMatr.getAMatrA(I.psiH,mm.QQFDpwt,n);
	Ft_IH=TMatr.getAMatrA(I.psiH,mm.QQFt_sample,n);
	dpw_IH=Fdp_IH/Fad_IH-FDpwv_IH/Fv_IH;
    } else Fdp_IH=Fad_IH=FDpwv_IH=Fv_IH=FDpwt_IH=Ft_IH=dpw_IH=Double.NaN;

    // variate da/dt in the state of dV/dt
    final double [] rqH=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(
	I.psiH,VOLATILITY_EDPsi.getDPsi(I.psiH),null,mm.QQa,mm.QQt,n);
    varA1=rqH[1];
    sk_I=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(I.psiH,mm.QQi,n),
	TMatr.getAMatrA(I.psiH,mm.QQpi,n),
	TMatr.getAMatrA(I.psiH,mm.QQp2i,n),
	TMatr.getAMatrA(I.psiH,mm.QQp3i,n));
    Ia=EVXData.getEVXDataFromQQ(n,mm.QQDawv,mm.QQDawt,M);
    sk_Ia=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(Ia.psiH,mm.QQDawv,n),
	TMatr.getAMatrA(Ia.psiH,mm.QQpDawv,n),
	TMatr.getAMatrA(Ia.psiH,mm.QQp2Dawv,n),
	TMatr.getAMatrA(Ia.psiH,mm.QQp3Dawv,n));

    // Try P&L extremal states
    PnL=EVXData.getEVXDataFromQQ(n,mm.QQDawv,mm.QQt,M);
    PnL_H_dir=TMatr.getAMatrA(PnL.psiH,mm.QQDpwv,n)/PnL.sH;
    PnL_H_dp=TMatr.getAMatrA(PnL.psiH,mm.QQdpdt,n);
    sk_PnL=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(PnL.psiH,mm.QQDawv,n),
	TMatr.getAMatrA(PnL.psiH,mm.QQpDawv,n),
	TMatr.getAMatrA(PnL.psiH,mm.QQp2Dawv,n),
	TMatr.getAMatrA(PnL.psiH,mm.QQp3Dawv,n));

    PnLa=EVXData.getEVXDataFromQQ(n,mm.QQDawv,mm.QQa,M);
    PnLa_H_dir=TMatr.getAMatrA(PnLa.psiH,mm.QQDpwv,n)/PnLa.sH;
    sk_PnLa=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(PnLa.psiH,mm.QQDawv,n),
	TMatr.getAMatrA(PnLa.psiH,mm.QQpDawv,n),
	TMatr.getAMatrA(PnLa.psiH,mm.QQp2Dawv,n),
	TMatr.getAMatrA(PnLa.psiH,mm.QQp3Dawv,n));
    Vmax=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQad,M);
    sk_Vmax=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(Vmax.psiH,mm.QQi,n),
	TMatr.getAMatrA(Vmax.psiH,mm.QQpi,n),
	TMatr.getAMatrA(Vmax.psiH,mm.QQp2i,n),
	TMatr.getAMatrA(Vmax.psiH,mm.QQp3i,n));
    PnLad=EVXData.getEVXDataFromQQ(n,mm.QQDawv,mm.QQad,M);
    PnLad_H_dir=TMatr.getAMatrA(PnLad.psiH,mm.QQDpwv,n)/PnLad.sH;
    sk_PnLad=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(PnLad.psiH,mm.QQDawv,n),
	TMatr.getAMatrA(PnLad.psiH,mm.QQpDawv,n),
	TMatr.getAMatrA(PnLad.psiH,mm.QQp2Dawv,n),
	TMatr.getAMatrA(PnLad.psiH,mm.QQp3Dawv,n));

    iar=new Mu2IArootsA(I.psiL,I.sL,mm.QQi,mm.QQt,
			mm.QQt_sample,mm.QQpt_sample,mm.QQp2t_sample,
			mm.QQi,mm.QQpi,mm.QQp2i,
			mm.QQa,mm.QQpa,mm.QQp2a,M,eps);

    // try dV/da
    this.IA=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQa,M);
    dm_IAH=TMatr.getAMatrA(IA.psiH,mm.QQpi,n)-IA.sH*TMatr.getAMatrA(IA.psiH,mm.QQpa,n);
    iad=new Mu2IAroots(IA.psiL,IA.sL,mm.QQi,mm.QQa,
		       mm.QQi,mm.QQpi,mm.QQp2i,mm.QQa,mm.QQpa,mm.QQp2a,M,eps);
    final double [] rqpIAH=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(
	IA.psiH,VOLATILITY_EDPsi.getDPsi(IA.psiH),null,mm.QQpi,mm.QQa,n);
    varp1_IAH=rqpIAH[1];
    pi_IAH=rqpIAH[0]/IA.sH;
    pa_IAH=TMatr.getAMatrA(IA.psiH,mm.QQpa,n);
    dp_IAH=TMatr.getAMatrA(IA.psiH,mm.QQdpdt,n);
    wv_IAH=TMatr.getAMatrA(IA.psiH,mm.QQDpwv,n);
    wt_IAH=TMatr.getAMatrA(IA.psiH,mm.QQDpwt,n);
    dp_IAL=TMatr.getAMatrA(IA.psiL,mm.QQdpdt,n);
    sk_IAH=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(IA.psiH,mm.QQi,n),
	TMatr.getAMatrA(IA.psiH,mm.QQpi,n),
	TMatr.getAMatrA(IA.psiH,mm.QQp2i,n),
	TMatr.getAMatrA(IA.psiH,mm.QQp3i,n));
    final double [] IQQ=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,mm.QQt),
	QQaa=TMatr.mulMatrices(n,new double [][]{mm.QQa,IQQ,mm.QQa}),
	QQdpI=TMatr.scaleVectorInPlace(TMatr.mulMatrices(n,new double [][]{mm.QQdpdt,IQQ,mm.QQi}),0.5);
    TMatr.scaleVectorInPlace(TMatr.selectHermitianPartInPlace(n,QQdpI),2); // factor 2 is here
    this.IdpAA=EVXData.getEVXDataFromQQ(n,QQdpI,QQaa,M);

    // try old versions
    this.A=EVXData.getEVXDataFromQQ(n,mm.QQa,mm.QQt,M);
    final int ixH=n-1;
    pIH=new Mu2F(I.getFPsi(ixH,mm.QQt_sample),I.getFPsi(ixH,mm.QQpt_sample),I.getFPsi(ixH,mm.QQp2t_sample),
		 I.getFPsi(ixH,mm.QQi),I.getFPsi(ixH,mm.QQpi),I.getFPsi(ixH,mm.QQp2i),eps);
    pAH=new Mu2F(A.getFPsi(ixH,mm.QQt_sample),A.getFPsi(ixH,mm.QQpt_sample),A.getFPsi(ixH,mm.QQp2t_sample),
		 A.getFPsi(ixH,mm.QQa),A.getFPsi(ixH,mm.QQpa),A.getFPsi(ixH,mm.QQp2a),eps);
    pIAH=new Mu2F(IA.getFPsi(ixH,mm.QQa),IA.getFPsi(ixH,mm.QQpa),IA.getFPsi(ixH,mm.QQp2a),
		  IA.getFPsi(ixH,mm.QQi),IA.getFPsi(ixH,mm.QQpi),IA.getFPsi(ixH,mm.QQp2i),eps);

    this.mu2ia=new Mu2IA(mm.p_offset,I.psiH,I.sH,
			 mm.QQi,mm.QQt,
			 mm.QQt_sample,mm.QQpt_sample,mm.QQp2t_sample,
			 mm.QQi,mm.QQpi,mm.QQp2i,
			 mm.QQa,mm.QQpa,mm.QQp2a,eps);


    this.sst=new State(rst);
}
protected JMomentsData getJMomentsData(final int n){return JMomentsData.get_JMomentsData(n,M,odata,getAccess());}
abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double [] getEDPsi(final double [] psi);


/** A class for unit test.
*/
static abstract class IAImpactTest<DTYPE> extends QTest {

abstract IAImpact<DTYPE> getIAImpact();
abstract PnLImpactA<DTYPE> getPnLImpactA();

IAImpactTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_Extra(){}

void test_PnLImpactA()
{
    final int n=neq;
    final IAImpact<DTYPE> q=getIAImpact();
    final PnLImpactA<DTYPE> s=getPnLImpactA();
    matchPnLImpactA(q,s,eps);
}

void test_IAImpact()
{
    final int n=neq;
    final IAImpact<DTYPE> q=getIAImpact();
    final JMomentsData mm=q.getJMomentsData(n);

    new Mu2IA(mm.p_offset,q.I.psiH,q.I.sH,
	      mm.QQi,mm.QQt,
	      mm.QQt_sample,mm.QQpt_sample,mm.QQp2t_sample,
	      mm.QQi,mm.QQpi,mm.QQp2i,
	      mm.QQa,mm.QQpa,mm.QQp2a,eps);
    new Mu2IAroots(q.IA.psiL,q.IA.sL,mm.QQi,mm.QQa,
		   mm.QQi,mm.QQpi,mm.QQp2i,mm.QQa,mm.QQpa,mm.QQp2a,q.M,eps);
    new Mu2IArootsA(q.I.psiL,q.I.sL,mm.QQi,mm.QQt,
		    mm.QQt_sample,mm.QQpt_sample,mm.QQp2t_sample,
		    mm.QQi,mm.QQpi,mm.QQp2i,
		    mm.QQa,mm.QQpa,mm.QQp2a,q.M,eps);
    System.err.println(name+":IAImpactTest.test_IAmpact(): test OK n="+n+" eps="+eps);
}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":IAImpactTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Extra();
    test_IAImpact();
    test_PnLImpactA();

    System.err.println(name+":IAImpactTest.test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_min,final int neq_max)
{
    System.err.println(name+":IAImpactTest: starting tests; ver=$Id: IAImpact.java,v 1.103 2022/11/19 11:08:57 mal Exp $");
    EPS_KLUDGY_WAY_TO_TURN_ON_THE_DEBUGGING=eps;
    final long t0=System.currentTimeMillis();
    for(int neq=neq_min;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
     EPS_KLUDGY_WAY_TO_TURN_ON_THE_DEBUGGING=0;
    final long t1=System.currentTimeMillis();
    System.err.println(name+":IAImpactTest.main: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: IAImpact.java,v 1.103 2022/11/19 11:08:57 mal Exp $");
}
} // END OF class IAImpactTest
static void matchPnLImpactA(final IAImpact<?> a,final PnLImpactA<?> b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    EVXData.matchEVXData(a.PnL,b.PnL,false,eps);
    EVXData.matchEVXData(a.PnLa,b.PnLa,false,eps);
    EVXData.matchEVXData(a.IA,b.IA,false,eps);
    EVXData.matchEVXData(a.Vmax,b.Vmax,false,eps);
    final double p_offset_shift=b.p_offset-a.mu2ia.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.PnL_H_dp,"a.PnL_H_dp",b.PnL_H_dp,"b.PnL_H_dp",eps);
    UnitTests.checkDoubleRel(a.PnL_H_dir,"a.PnL_H_dir",b.PnL_H_dir,"b.PnL_H_dir",eps);
    UnitTests.checkDoubleRel(a.PnLa_H_dir,"a.PnLa_H_dir",b.PnLa_H_dir,"b.PnLa_H_dir",eps);
    Skewness.matchSkewness(a.sk_PnLa.getSameSkewnessXShifted(a.mu2ia.p_offset),b.sk_PnLa.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.sk_IAH.getSameSkewnessXShifted(a.mu2ia.p_offset),b.sk_IAH.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.sk_Vmax.getSameSkewnessXShifted(a.mu2ia.p_offset),b.sk_Vmax.getSameSkewnessXShifted(b.p_offset),eps);

    System.err.println("IAImpact.matchPnLImpactA test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}

}// END OF class IAImpact
