package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: IAImpactLaguerre.java,v 1.9 2022/07/10 10:41:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IAImpactLaguerre extends IAImpact<QVMDataL> {
IAImpactLaguerre(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}


public static IAImpactLaguerre calculate(final int neq,
					  final long currenttime,
					  final long tau,
					  final Trade [] tb,
					  final int tsize,
					  final IAImpactLaguerre ref)
{
    final int npm=4,nr=0;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*(neq-1)+1+nr,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new IAImpactLaguerre(neq,o,ref!=null?ref.sst:null);
}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}

static class IAImpactLaguerreTest extends IAImpactTest<QVMDataL> {
IAImpactLaguerreTest(final long initrandom)
{
    super("IAImpactLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
IAImpactLaguerre getIAImpact(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
PnLImpactALaguerre getPnLImpactA(){return PnLImpactALaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}


} // END of class IAImpactLaguerreTest

public static void main(String [] args)
{
    new IAImpactLaguerreTest(11127L).runTests(2,5);
} // END main



} // END OF class IAImpactLaguerre
