package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: IAImpactMonomials.java,v 1.14 2022/07/15 19:57:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IAImpactMonomials extends IAImpact<QVMData> {
IAImpactMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static IAImpactMonomials calculate(final int neq,
					   final long currenttime,
					   final long tau,
					   final Trade [] tb,
					   final int tsize,
					   final IAImpactMonomials ref)
{
    final int nvm=1,npm=4,nr=0;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1+nr,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new IAImpactMonomials(neq,o,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}

static class IAImpactMonomialsTest extends IAImpactTest<QVMData> {
IAImpactMonomialsTest(final long initrandom)
{
    super("IAImpactMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}



@Override
void test_Extra(){
    // These to be identical
    final IAImpactMonomials q=getIAImpact();
    final IAImpactLaguerre s=getIAImpactLaguerre();
    matchPnLImpactA(q,getPnLImpactA(),eps);
    matchPnLImpactA(s,getPnLImpactA(),eps);

    EVXData.matchEVXData(q.I,s.I,false,eps);
    UnitTests.checkDoubleRel(q.varA1,"q.varA1",s.varA1,"s.varA1",eps);
    EVXData.matchEVXData(q.IdpAA,s.IdpAA,false,eps);

    Mu2IA.matchMu2IA(q.mu2ia,s.mu2ia,eps);
    com.polytechnik.utils.Skewness.matchSkewness(q.sk_IAH.getSameSkewnessXShifted(q.mu2ia.p_offset),s.sk_IAH.getSameSkewnessXShifted(s.mu2ia.p_offset),eps);

    System.err.println(name+":IAImpactMonomialsTest:test_Extra(): test IAImpactLaguerre OK neq="+neq+" eps="+eps);
}


IAImpactMonomials getIAImpact(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
PnLImpactAMonomials getPnLImpactA(){return PnLImpactAMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}
IAImpactLaguerre getIAImpactLaguerre(){return IAImpactLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}


} // END of class IAImpactMonomialsTest

public static void main(String [] args)
{
    new IAImpactMonomialsTest(11127L).runTests(2,5);
} // END main



} // END OF class IAImpactMonomials
