package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.Skewness;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: IAImpact_pi.java,v 1.7 2018/07/03 13:06:40 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IAImpact_pi extends IAImpact<QVMDataL> {
IAImpact_pi(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}

public static IAImpact_pi calculate(final int neq,
					   final long currenttime,
					   final long tau,
					   final Trade [] tb,
					   final int tsize,
					   final IAImpact_pi ref)
{
    final int ntm=1,npm=2*neq+2;
    final QVMDataL o=QVMDataL.calculateQVMDataL(ntm,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new IAImpact_pi(neq,o,ref!=null?ref.sst:null);
}
@Override
protected JMomentsData getJMomentsData(final int n){return JMomentsData.get_PriceMonomials_JMomentsData(n,odata,getAccess());}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){
    return new OrthogonalPolynomialsMonomialsBasisX(1,-getAccess().getPLast(odata)); // getPLast does have p_offset included
}
/** Different meaning, but use anyway.
 */
double [] getEDPsi(final double [] psi){return M.B.getDifferentiated(psi);}

static class IAImpact_piTest extends IAImpactTest<QVMDataL> {
IAImpact_piTest(final long initrandom)
{
    super("IAImpact_pi",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

@Override
void test_Extra(){
    final IAImpact_pi q=getIAImpact();
    final QkPnLpi_PBasis s=getQkPnLpi_PBasis();
    final double p_offset_shift=s.ist.p_offset-q.mu2ia.p_offset;
    EVXData.matchEVXData(q.I,s.ist.eI,false,true,eps);
    UnitTests.checkDoubleRel(q.p_IH,"a.p_IH",s.ist.p_IH+p_offset_shift,"s.ist.p_IH+p_offset_shift",eps);
    System.err.println(name+":IAImpact_piTest:test_Extra(): test IAImpact_pi OK neq="+neq+" eps="+eps);
}

QkPnLpi_PBasis getQkPnLpi_PBasis(){return QkPnLpi_PBasis.calculate(neq,currenttime,tau,tb,tb.length,null);}
IAImpact_pi getIAImpact(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
PnLImpactA_pi getPnLImpactA(){return PnLImpactA_pi.calculate(neq,currenttime,tau,tb,tb.length,null);}


} // END of class IAImpact_piTest

public static void main(String [] args)
{
    new IAImpact_piTest(1274L).runTests(2,5);
} // END main



} // END OF class IAImpact_pi
