package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.IstatesConditional;
import com.polytechnik.utils.IstatesConditionalV2;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.ProbabilityCorrelation;

/**
 * @version $Id: IHStateDM.java,v 1.237 2022/08/08 20:10:46 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
*/
public abstract class IHStateDM<DTYPE> {

public static class State {
    public final long seq;
    State(final State prev)
    {
	this.seq=(prev==null?0:1+prev.seq);
    }
}
public final State sst;
public final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=getABasis();
public final double p_offset,p_last,p_average,pt_average;

public final DM_DI dmdi;
public final DM_HT dmht;
public final DM_DIRel dmdirel;
public final DM_DIf dmdif;

public final DIselDM diselDM;

public final DIselIH
    diselIH2sepdpdt,diselIH2sepd2pdt2,diselIH2sepdDpwvdt,diselIH2sepdIdt,diselIH2sepPIH,
    diselIHsepdpdt,diselIHsepd2pdt2,diselIHsepdDpwvdt,diselIHsepdIdt,diselIHsepPIH,diselIHsepDMpsiH,
    diselIHplast,diselIH2plast,diselIHdpI;
public final DIselIH2pi diselPSkewness,diselPSkewnessF,diselPSkewness0,diselPSkewness0F;
public final DIselPsi0 diselPsi0,diselPsi0F;
public final DIselIH2PositiondI disel2PositiondI;
public final DIselIHPositiondI diselPositiondI_inPsiH,diselPositionPdI_inPsiH,diselPositiondI_inPsiH_dynamicimpact,
    diselPositiondI_Dp_afterPsiH,diselPositiondI_Dp_afterPsiH_p_last,
    diselPositiondI_DpWW_afterPsiH,diselPositiondI_DpWW_afterPsiH_p_last,
    diselPositiondI_Dpv_afterPsiH,
    diselPositiondI_dyn_afterPsiH,diselPositiondI_dyn_afterPsiH_p_last;
public final DIsel disel_corr_dS_I_inPsiH,disel_corr_dS_I_afterPsiH;
public final DIselIHskewness diselsk;
public final DIselCorrPI diselcorrpi;
public final DIselD2P diseld2pt,diseld2pi;
public final DIselD2PstateIH diseld2ptIH,diseld2piIH,diseld2ptIHpure,diseld2piIHpure;
public final DIselEVp diselevp;
public final DIselPnLSensitivity diselpnlsens;
public final IstatesSelection istatesselection;
public final DIselPIProbabilityCorrelation diselpiprobabilitycorrPi,diselpiprobabilitycorrPt;
public final DIselPdivI diselpdivI;
public final DIselP2divI diselp2divI;
public final DIselVT diselVT;
public final DIselVTsimple diselVTsimple;
public final DIstatesVT distatesVT,distatesVTmin;
public final DIminP2maxI diminP2maxI,diminP2maxVT,diminP2maxIT,diminP2maxIV;
public final DIminP2VmaxI diminp2vmaxI,diminp2vmaxVT;
public final MaxPnLratio maxpnlratioI,maxpnlratioV;

public final MinMaxPnLratioNorm minpnlratioNI,maxpnlratioNI,minpnlratioNV,maxpnlratioNV;
public final DIminP2tmaxI minp2tmaxI;
public final MaxPtPv2I maxptpv2i;
public final MaxPPl2I maxppl2,maxppl2div,maxppl2I,maxppl2V;
public final MaxPPl2Iinbasis maxppl2iinbasis;
final MaxIP2ratio maxip2I,maxip2V;
final Mu2PXinIL mu2pxinil;
final MaxPP2VI maxpp2vi;
final MinIPlPV minIPlPV,minVPlPI;
final IstatesLocalized istateslocalized;
final IstatesConditional istatesconditionalIcI,istatesconditionalIcV,istatesconditionalIcdI,istatesconditionalIcdP,istatesconditionalIcd2P,istatesconditionalV,istatesconditionalVcI;
final IstatesConditionalV2 istatesconditionalIcIV2;

final double istatesconditionalIcV_var1,istatesconditionalIcdI_var1,istatesconditionalIcdP_var1,istatesconditionalIcd2P_var1,istatesconditionalIcI_var1,
	istatesconditionalIcV_px,istatesconditionalIcdI_px,istatesconditionalIcdP_px,istatesconditionalIcd2P_px,istatesconditionalIcI_px;
final CalculateSkewnessAndProbabilityCorrelationInPsiState istatesconditional_skpc,istatesconditionalV_skpc;
final IstatesPPl istatesppl;
final InIstatesPV inistatesPV;
final IstatesPselection ipselV,ipselI;
final DIselPIV diselPIV;

final DTYPE odata;
final VolMatrix VOLATILITY_EDPsi=this::getEDPsi;

/** Not worth a function or costructor argument.
 */
private static double EPS_KLUDGY_WAY_TO_TURN_ON_THE_DEBUGGING=0;

public IHStateDM(final int n,final DTYPE odata,final State rst)
{

    final double eps=EPS_KLUDGY_WAY_TO_TURN_ON_THE_DEBUGGING; // turn on selftests if required
    this.odata=odata;
    final RMomentsData mmmoms=getRMoments(2*n-1,odata),mm=mmmoms.asQQMatrices(n,M);
    final double [] QQt=mm.tmom,QQi=mm.imom,QQpi=mm.pimom,QQp2i=mm.p2imom,QQpt=mm.ptmom,QQp2t=mm.p2tmom,QQp3t=mm.p3tmom,QQdpdt=mm.dpmom,
	QQT=TMatr.getVectorScaled(mm.Tmom,-1)/* the RMomentData has negative sign convention. */,
	QQV=TMatr.getVectorScaled(mm.Vmom,-1)/* the RMomentData has negative sign convention. */;
    p_offset=mm.p_offset;
    p_last=mm.p_last;
    p_average=QQpi[0]/QQi[0];
    pt_average=QQpt[0]/QQt[0];


    this.dmdi=new DM_DI(n,mm.p_offset,mm.p_last,QQt,QQpt,QQi,QQpi,QQdpdt,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,eps>0?getRMoments(2*n-1,odata):null,eps);
    this.dmdirel=new DM_DIRel(n,mm.p_offset,mm.p_last,QQt,QQi,QQpi,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,eps);
    this.dmht=new DM_HT(n,mm.p_offset,QQt,QQpt,QQi,QQpi,QQdpdt,QQT,QQV,M,this::getPsi2WIntegratedDt,eps);
    this.dmdif=new DM_DIf(n,mm.p_offset,mm.p_last,QQt,QQpt,QQi,QQpi,QQdpdt,M,this::getPsi2WIntegratedDt,eps);

    diselDM=new DIselDM(n,mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,VOLATILITY_EDPsi,eps);

    diselIHsepdpdt=new DIselIH(n,DIselIH.METHOD_FULL_EV_QQDEV,QQdpdt,mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
    diselIHsepd2pdt2=new DIselIH(n,DIselIH.METHOD_FULL_EV_QQDEV,getQQdSmatr(n,QQdpdt),mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
    diselIHsepdDpwvdt=new DIselIH(n,DIselIH.METHOD_FULL_EV_QQDEV,getQQdSmatr(n,mm.Dpwv),mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
    diselIHsepdIdt=new DIselIH(n,DIselIH.METHOD_FULL_EV_QQDEV,getQQdSmatr(n,QQi),mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
    diselIHsepPIH=new DIselIH(n,DIselIH.METHOD_FULL_EV_QQDEV,null /*TMatr.sumAcB(QQpi,-p_IH,QQi)*/,mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
    diselIHsepDMpsiH=new DIselIH(n,DIselIH.METHOD_FULL_EV_QQDEV,dmdi.DensityMatrix_psiHintegrated,mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
    diselIHplast=new DIselIH(n,DIselIH.METHOD_FULL_EV_QQDEV,TMatr.sumAcB(QQpi,-mm.p_last,QQi),mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
    if(true)
    {
	final EVXData I=diselDM.I;final double p_IH=diselDM.p_IH;
	final double []
	    QQ0=TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),
	    QQDEVseparation=TMatr.sumaAbB(mm.p_last*I.sH,QQ0,-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQpi));
	diselIHdpI=new DIselIH(n,DIselIH.METHOD_FULL_EV_QQDEV,QQDEVseparation,mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
	if(eps>0)
	{
	    //UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psiH,QQDEVseparation,n),"<psiH|QQDEVseparation|psiH>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psiH,VOLATILITY_EDPsi.getDiffMatrix(n,QQi),n),"<psiH|I|psiH>",I.sH*TMatr.getAMatrA(I.psiH,QQ0,n),"IH*<psiH|QQ0|psiH>",eps);
	    UnitTests.checkDoubleRel(diselIHplast.dEVseparation_IH,"diselIHplast.dEVseparation_IH",(p_IH-p_last)*I.sH,"(p_IH-p_last)*I.sH",eps);
	}
    }

    diselIH2sepdpdt=new DIselIH(n,DIselIH.METHOD_2EV_QQDEV,QQdpdt,mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
    diselIH2sepd2pdt2=new DIselIH(n,DIselIH.METHOD_2EV_QQDEV,getQQdSmatr(n,QQdpdt),mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
    diselIH2sepdDpwvdt=new DIselIH(n,DIselIH.METHOD_2EV_QQDEV,getQQdSmatr(n,mm.Dpwv),mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
    diselIH2sepdIdt=new DIselIH(n,DIselIH.METHOD_2EV_QQDEV,getQQdSmatr(n,QQi),mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);
    diselIH2sepPIH=new DIselIH(n,DIselIH.METHOD_2EV_QQDEV,null /*TMatr.sumAcB(QQpi,-p_IH,QQi)*/,mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps*0); // skip the unit test
    diselIH2plast=new DIselIH(n,DIselIH.METHOD_2EV_QQDEV,TMatr.sumAcB(QQpi,-mm.p_last,QQi),mm.p_offset,mm.p_last,QQt,QQi,QQpi,QQdpdt,M,eps);

    diselPSkewness=new DIselIH2pi(n,false,false,mm.p_offset,mm.p_last,QQt,QQi,QQpi,M,eps);
    diselPSkewnessF=new DIselIH2pi(n,true,false,mm.p_offset,mm.p_last,QQt,QQi,QQpi,M,eps);
    diselPSkewness0=new DIselIH2pi(n,false,true,mm.p_offset,mm.p_last,QQt,QQi,QQpi,M,eps);
    diselPSkewness0F=new DIselIH2pi(n,true,true,mm.p_offset,mm.p_last,QQt,QQi,QQpi,M,eps);
    if(eps>0){
	UnitTests.checkDoubleRel(diselIHsepd2pdt2.QQDEVseparation[0],"diselIHsepd2pdt2.QQDEVseparation[0]",0,"0",eps);
	UnitTests.checkDoubleRel(diselIHsepdDpwvdt.QQDEVseparation[0],"diselIHsepdDpwvdt.QQDEVseparation[0]",0,"0",eps);
	UnitTests.checkDoubleRel(diselIHsepdIdt.QQDEVseparation[0],"diselIHsepdIdt.QQDEVseparation[0]",0,"0",eps);
	UnitTests.checkDoubleRel(diselIHsepPIH.dEVseparation_IH,"<psiH|sep|psiH>",0,"0",eps);
	UnitTests.checkDoubleRel(diselIH2sepPIH.dEVseparation_IH,"<psiH|sep|psiH>",0,"0",eps);
    }
    diselPsi0=new DIselPsi0(n,false,p_offset,p_last,QQt,QQi,QQpi,M,eps);
    diselPsi0F=new DIselPsi0(n,true,p_offset,p_last,QQt,QQi,QQpi,M,eps);

    disel2PositiondI=new DIselIH2PositiondI(n,p_offset,QQt,QQi,QQpi,M,VOLATILITY_EDPsi,eps);
    diselPositiondI_inPsiH=DIselIHPositiondI.getLocalInPsiH(n,p_offset,p_last,QQt,QQi,QQpi,QQi,M,VOLATILITY_EDPsi,false,eps);
    diselPositionPdI_inPsiH=DIselIHPositiondI.getLocalInPsiH(n,p_offset,p_last,QQt,QQi,QQpt,QQt,M,VOLATILITY_EDPsi,false,eps);
    diselPositiondI_inPsiH_dynamicimpact=DIselIHPositiondI.getLocalInPsiH(n,p_offset,p_last,QQt,QQi,QQpi,QQi,M,VOLATILITY_EDPsi,true,eps);

    diselPositiondI_Dp_afterPsiH=DIselIHPositiondI.getAfterPsiH(n,p_offset,QQt,QQi,QQpi,QQdpdt,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,false,false,eps);
    diselPositiondI_DpWW_afterPsiH=DIselIHPositiondI.getAfterPsiH(n,p_offset,QQt,QQi,QQpi,QQdpdt,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,false,true,eps);
    diselPositiondI_Dp_afterPsiH_p_last=DIselIHPositiondI.getAfterPsiH(n,p_offset,QQt,QQi,QQpi,QQdpdt,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,true,false,eps);
    diselPositiondI_DpWW_afterPsiH_p_last=DIselIHPositiondI.getAfterPsiH(n,p_offset,QQt,QQi,QQpi,QQdpdt,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,true,true,eps);
    diselPositiondI_Dpv_afterPsiH=DIselIHPositiondI.getAfterPsiH(n,p_offset,QQt,QQi,QQpi,mm.Dpwv,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,false,false,eps);

    diselPositiondI_dyn_afterPsiH=DIselIHPositiondI.getAfterPsiH(n,p_offset,p_last,QQt,QQi,QQpi,QQi,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,false,eps);
    diselPositiondI_dyn_afterPsiH_p_last=DIselIHPositiondI.getAfterPsiH(n,p_offset,p_last,QQt,QQi,QQpi,QQi,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,true,eps);


    if(eps>0){
	UnitTests.checkDoubleRel(diselPositiondI_inPsiH.dm_spur,"diselPositiondI_inPsiH.dm_spur",1,"1",eps);
	UnitTests.checkDoubleRel(diselPositiondI_Dp_afterPsiH_p_last.dm_spur,"diselPositiondI_Dp_afterPsiH.dm_spur",dmdi.T_IH,"dmdi.T_IH",eps);
	UnitTests.checkDoubleRel(diselPositiondI_Dp_afterPsiH_p_last.position_eq,"diselPositiondI_Dp_afterPsiH_p_last.position_eq",mm.p_last-dmdi.pt_IH,"mm.p_last-dmdi.pt_IH",eps);
    }

    this.disel_corr_dS_I_inPsiH=new DIsel(n,p_offset,QQt,QQi,QQpi,QQpt,QQdpdt,M,VOLATILITY_EDPsi,eps);
    this.disel_corr_dS_I_afterPsiH=new DIsel(n,p_offset,QQt,QQi,QQpi,QQpt,mm.Dpwv,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,eps);

    diselsk=new DIselIHskewness(n,p_offset,QQt,QQi,QQpi,mm.p2imom,mm.p3imom,QQpt,M,VOLATILITY_EDPsi,eps);
    diselcorrpi=new DIselCorrPI(n,p_offset,QQt,QQi,QQpt,M,eps);
    diseld2pt=new DIselD2P(n,p_offset,QQt,QQi,QQpt,QQt,M,VOLATILITY_EDPsi,eps);
    diseld2pi=new DIselD2P(n,p_offset,QQt,QQi,QQpi,QQi,M,VOLATILITY_EDPsi,eps);

    diseld2ptIH=new DIselD2PstateIH(n,p_offset,QQt,QQi,QQpt,QQt,false,M,VOLATILITY_EDPsi,eps);
    diseld2piIH=new DIselD2PstateIH(n,p_offset,QQt,QQi,QQpi,QQi,false,M,VOLATILITY_EDPsi,eps);
    diseld2ptIHpure=new DIselD2PstateIH(n,p_offset,QQt,QQi,QQpt,QQt,true,M,VOLATILITY_EDPsi,eps);
    diseld2piIHpure=new DIselD2PstateIH(n,p_offset,QQt,QQi,QQpi,QQi,true,M,VOLATILITY_EDPsi,eps);
    diselevp=new DIselEVp(n,p_offset,p_last,QQt,QQi,QQpt,M,VOLATILITY_EDPsi,eps);
    diselpnlsens=new DIselPnLSensitivity(n,p_offset,QQt,QQi,QQpi,mm.Vmom,M,VOLATILITY_EDPsi,eps);
    istatesselection=new IstatesSelection(n,p_offset,p_last,QQt,QQi,QQpi,mm.Vmom,M,VOLATILITY_EDPsi,eps);
    diselpiprobabilitycorrPi=new DIselPIProbabilityCorrelation(n,p_offset,QQt,QQi,QQpi,QQi,M,VOLATILITY_EDPsi,eps);
    diselpiprobabilitycorrPt=new DIselPIProbabilityCorrelation(n,p_offset,QQt,QQi,QQpt,QQt,M,VOLATILITY_EDPsi,eps);
    diselpdivI=new DIselPdivI(n,p_offset,QQt,QQi,QQpi,M,eps);
    /* the RMomentData has negative sign convention. */
    final double [] QQpV=TMatr.getVectorScaled(mm.pVmom,-1),
	QQp2V=TMatr.getVectorScaled(mm.p2Vmom,-1),
	QQp3V=TMatr.getVectorScaled(mm.p3Vmom,-1);
    minIPlPV=new MinIPlPV(n,p_offset,p_last,QQt,QQi,QQpi,QQp2i,QQV,QQpV,M,eps);
    minVPlPI=new MinIPlPV(n,p_offset,p_last,QQT,QQV,QQpV,QQp2V,QQi,QQpi,M,eps);
    istateslocalized=new IstatesLocalized(n,p_offset,p_last,QQt,QQi,QQpi,QQp2i,mm.p3imom,M,eps);
    diselVT=new DIselVT(n,p_last,QQT,QQV,QQpV,QQdpdt,QQt,QQpt,QQi,QQpi,mm.p2imom,mm.p3imom,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,eps);
    diselVTsimple=new DIselVTsimple(n,p_last,QQT,QQV,QQpV,QQt,QQi,QQpi,M,VOLATILITY_EDPsi,eps);

    istatesconditionalIcV=new IstatesConditional(n,QQt,QQi,TMatr.sumAcB(QQpV,-p_last,QQV),M,eps);
    istatesconditionalIcV_var1=getVar1(istatesconditionalIcV);
    istatesconditionalIcV_px=getPX(istatesconditionalIcV,QQpi,QQi);

    istatesconditionalIcdI=new IstatesConditional(n,QQt,QQi,TMatr.scaleVectorInPlace(VOLATILITY_EDPsi.getDiffMatrix(n,TMatr.sumAcB(QQpi,-p_last,QQi)),-1),M,eps);
    istatesconditionalIcdI_var1=getVar1(istatesconditionalIcdI);
    istatesconditionalIcdI_px=getPX(istatesconditionalIcdI,QQpi,QQi);

    istatesconditionalIcdP=new IstatesConditional(n,QQt,QQi,QQdpdt,M,eps);
    istatesconditionalIcdP_var1=getVar1(istatesconditionalIcdP);
    istatesconditionalIcdP_px=getPX(istatesconditionalIcdP,QQpi,QQi);

    istatesconditionalIcd2P=new IstatesConditional(n,QQt,QQi,TMatr.scaleVectorInPlace(VOLATILITY_EDPsi.getDiffMatrix(n,QQdpdt),-1),M,eps);
    istatesconditionalIcd2P_var1=getVar1(istatesconditionalIcd2P);
    istatesconditionalIcd2P_px=getPX(istatesconditionalIcd2P,QQpi,QQi);

    istatesconditionalIcI=new IstatesConditional(n,QQt,QQi,TMatr.sumAcB(QQpi,-p_last,QQi),M,eps);
    istatesconditionalIcI_var1=getVar1(istatesconditionalIcI);
    istatesconditionalIcI_px=getPX(istatesconditionalIcI,QQpV,QQV);
    istatesconditionalIcIV2=new IstatesConditionalV2(n,QQt,QQi,TMatr.sumAcB(QQpi,-p_last,QQi),M,eps);

    istatesconditionalVcI=new IstatesConditional(n,QQT,QQV,TMatr.sumAcB(QQpi,-p_last,QQi),M,eps);
    istatesconditional_skpc=
	new CalculateSkewnessAndProbabilityCorrelationInPsiState(
	    istatesconditionalIcI.psi_M,VOLATILITY_EDPsi.getDPsi(istatesconditionalIcI.psi_M),QQt,QQi,QQpi,QQp2i,mm.p3imom,QQpV,QQV);
    istatesconditionalV=new IstatesConditional(n,QQT,QQV,TMatr.sumAcB(QQpV,-p_last,QQV),M,eps);
    istatesconditionalV_skpc=
	new CalculateSkewnessAndProbabilityCorrelationInPsiState(
	    istatesconditionalV.psi_M,VOLATILITY_EDPsi.getDPsi(istatesconditionalV.psi_M),QQT,QQV,QQpV,QQp2V,QQp3V,QQpi,QQi);
    if(eps>0){
	if(Math.abs(istatesconditionalV_skpc.var_ppi)>0){
	    final double im=TMatr.getAMatrA(istatesconditionalV.psi_M,QQi,n),pim=TMatr.getAMatrA(istatesconditionalV.psi_M,QQpi,n)/im;
	    UnitTests.checkDoubleRel(istatesconditionalV_skpc.var_ppi,"istatesconditionalV_skpc.var_ppi",im*(pim-p_last),"im*(pim-p_last)",eps);
	}
    }

    diselp2divI=new DIselP2divI(n,p_offset,mm.p_last,QQt,QQi,QQpi,mm.p2imom,QQV,QQpV,M,VOLATILITY_EDPsi,eps);
    maxpp2vi=new MaxPP2VI(n,p_offset,QQt,QQi,
			  QQt,QQpt,QQp2t,QQp3t,
			  QQV,QQpV,M,eps);
    diminp2vmaxI=new DIminP2VmaxI(n,p_offset,
				  QQt,QQi,QQpi,QQp2i,mm.p3imom,
				  QQV,QQpV,M,eps);
    diminp2vmaxVT=new DIminP2VmaxI(n,p_offset,
				   QQT,QQV,QQpV,QQp2V,QQp3V,
				   QQi,QQpi,M,eps);
    diminP2maxI=new DIminP2maxI(n,p_offset,QQt,QQi,QQpi,mm.p2imom,mm.p3imom,M,VOLATILITY_EDPsi,eps);
    diminP2maxVT=new DIminP2maxI(n,p_offset,QQT,QQV,QQpV,QQp2V,QQp3V,M,VOLATILITY_EDPsi,eps);
    diminP2maxIT=new DIminP2maxI(n,p_offset,QQT,QQi,QQpi,mm.p2imom,mm.p3imom,M,VOLATILITY_EDPsi,eps);
    diminP2maxIV=new DIminP2maxI(n,p_offset,QQV,QQi,QQpi,mm.p2imom,mm.p3imom,M,VOLATILITY_EDPsi,eps);

    final DataInterfaceToMoments<DTYPE> access=getAccess();
    final double [] pTmom=new double [mmmoms.n],p2Tmom=new double [mmmoms.n],p3Tmom=new double [mmmoms.n];
    access.setFMoments(pTmom,mmmoms.ptmom,mmmoms.n);
    access.setFMoments(p2Tmom,mmmoms.p2tmom,mmmoms.n);
    access.setFMoments(p3Tmom,mmmoms.p3tmom,mmmoms.n);
    final double [] QQpT=TMatr.getVectorScaled(M.B.getQQMatr(n,pTmom),-1),
	QQp2T=TMatr.getVectorScaled(M.B.getQQMatr(n,p2Tmom),-1),
	QQp3T=TMatr.getVectorScaled(M.B.getQQMatr(n,p3Tmom),-1);

    distatesVT=new DIstatesVT(n,p_last,true,QQt,QQT,QQdpdt,
			      QQV,QQpV,QQp2V,QQp3V,QQT,QQpT,
			      QQi,QQpi,QQp2i,mm.p3imom,QQt,QQpt,
			      M,VOLATILITY_EDPsi,eps);
    distatesVTmin=new DIstatesVT(n,p_last,false,QQt,QQT,QQdpdt,
			      QQV,QQpV,QQp2V,QQp3V,QQT,QQpT,
			      QQi,QQpi,QQp2i,mm.p3imom,QQt,QQpt,
			      M,VOLATILITY_EDPsi,eps);


    minp2tmaxI=new DIminP2tmaxI(n,p_offset,QQt,QQi,
				QQt,QQpt,QQp2t,QQp3t,M,VOLATILITY_EDPsi,eps);
    maxpnlratioI=new MaxPnLratio(n,p_offset,QQt,QQi,
				QQt,QQpt,QQp2t,
				QQi,QQpi,QQp2i,M,eps);
    maxpnlratioV=new MaxPnLratio(n,p_offset,QQT,QQV,
				 QQT,QQpT,QQp2T,
				 QQV,QQpV,QQp2V,M,eps);

    minpnlratioNI=new MinMaxPnLratioNorm(n,MinMaxPnLratioNorm.MIN_ratio,p_offset,QQt,QQi,
					 QQt,QQpt,QQp2t,
					 QQi,QQpi,QQp2i,M,VOLATILITY_EDPsi,eps);
    maxpnlratioNI=new MinMaxPnLratioNorm(n,MinMaxPnLratioNorm.MAX_ratio,p_offset,QQt,QQi,
					 QQt,QQpt,QQp2t,
					 QQi,QQpi,QQp2i,M,VOLATILITY_EDPsi,eps);
    minpnlratioNV=new MinMaxPnLratioNorm(n,MinMaxPnLratioNorm.MIN_ratio,p_offset,QQT,QQV,
					 QQT,QQpT,QQp2T,
					 QQV,QQpV,QQp2V,M,VOLATILITY_EDPsi,eps);
    maxpnlratioNV=new MinMaxPnLratioNorm(n,MinMaxPnLratioNorm.MAX_ratio,p_offset,QQT,QQV,
					 QQT,QQpT,QQp2T,
					 QQV,QQpV,QQp2V,M,VOLATILITY_EDPsi,eps);

    maxptpv2i=new MaxPtPv2I(n,MinMaxPnLratioNorm.MAX_ratio,p_offset,QQt,QQi,
			    QQt,QQpt,QQp2t,
			    QQi,QQpi,QQp2i,M,VOLATILITY_EDPsi,eps);

    maxppl2=new MaxPPl2I(n,p_offset,mm.p_last,
			  QQt,QQi,
			  QQi,QQpi,QQp2i,
			  QQi,QQpi,QQp2i,QQV,QQpV,M,eps);
    maxppl2div=new MaxPPl2I(n,p_offset,mm.p_last,
			    QQt,TMatr.mulMMM(n,QQi,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),QQi),
			    QQi,QQpi,QQp2i,
			    QQi,QQpi,QQp2i,QQV,QQpV,M,eps);
    maxppl2I=new MaxPPl2I(n,p_offset,mm.p_last,
			  QQt,QQi,
			  QQt,QQpt,QQp2t,
			  QQi,QQpi,QQp2i,QQV,QQpV,M,eps);
    maxppl2V=new MaxPPl2I(n,p_offset,mm.p_last,
			  QQT,QQV,
			  QQT,QQpT,QQp2T,
			  QQV,QQpV,QQp2V,QQi,QQpi,M,eps);
    maxppl2iinbasis=new MaxPPl2Iinbasis(n,p_offset,mm.p_last,
					QQt,QQi,QQpi,QQp2i,
					QQV,QQpV,QQT,QQpT,M,eps);
    if(eps>0){
	UnitTests.checkDoubleRel(maxppl2div.MI,"maxppl2div.MI",maxppl2iinbasis.MI,"maxppl2iinbasis.MI",eps);
	UnitTests.checkDoubleRel(maxppl2div.pi,"maxppl2div.pi",maxppl2iinbasis.pt,"maxppl2iinbasis.pt",eps);
	UnitTests.checkDoubleRel(maxppl2div.pV,"maxppl2div.pV",maxppl2iinbasis.pVt,"maxppl2iinbasis.pVt",eps);
    }
     maxip2I=new MaxIP2ratio(n,p_offset,mm.p_last,
			     QQt,QQi,
			     QQt,QQpt,QQp2t,
			     QQi,QQpi,QQp2i,M,eps);
     maxip2V=new MaxIP2ratio(n,p_offset,mm.p_last,
			     QQT,QQV,
			     QQT,QQpT,QQp2T,
			     QQV,QQpV,QQp2V,M,eps);
     mu2pxinil=new Mu2PXinIL(p_offset,diselDM.I.psiL,diselDM.I.sL,
			     QQi,QQt,
			     QQi,QQpi,QQp2i,
			     QQt,QQpt,QQp2t,M,eps);
     istatesppl=new IstatesPPl(n,p_last,QQt,QQi,QQpi,M,eps);
     inistatesPV=new InIstatesPV(n,QQt,QQT,QQV,QQpV,QQi,QQpi,M,VOLATILITY_EDPsi,eps);
     ipselV=new IstatesPselection(n,QQt,QQV,QQpV,QQi,QQpi,M,VOLATILITY_EDPsi,eps);
     ipselI=new IstatesPselection(n,QQt,QQi,QQpi,QQi,QQpi,M,VOLATILITY_EDPsi,eps);
     diselPIV=new DIselPIV(n,p_last,
			   QQT,QQV,QQpV,
			   QQt,QQi,QQpi,
			   M,VOLATILITY_EDPsi,eps);

    final double [] TTmom=new double [mmmoms.n],TVmom=new double [mmmoms.n],TpVmom=new double [mmmoms.n],Tp2Vmom=new double [mmmoms.n],Tp3Vmom=new double [mmmoms.n];
    access.setFMoments(TTmom,mmmoms.Tmom,mmmoms.n);
    access.setFMoments(TVmom,mmmoms.Vmom,mmmoms.n);
    access.setFMoments(TpVmom,mmmoms.pVmom,mmmoms.n);
    access.setFMoments(Tp2Vmom,mmmoms.p2Vmom,mmmoms.n);
    access.setFMoments(Tp3Vmom,mmmoms.p3Vmom,mmmoms.n);

    if(true){
	dumpTempDataAtSomePoints(getClass().getName(),mm,access.getTau(odata),access.getTLast(odata),eps);
    }

    // for future extension
    this.sst=new State(rst);
}
/** Calculate p''*w+p'*w', seems to be the same in two bases.
 */
double [] getQQdSmatr(final int n,final double [] QQPi)
{
    return _getQQdSmatr(n,QQPi,VOLATILITY_EDPsi);
}
/** Seems to be the same in two bases.
 */
static double [] _getQQdSmatr(final int n,final double [] QQPi,VolMatrix VOLATILITY_EDPsi)
{
    return TMatr.sumaAbB(-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQPi),1,QQPi);
}
RMomentsData getRMoments(final int nmoms,final DTYPE odata)
{
    return RMomentsData.get_MomentsData(nmoms,odata,getAccess());
}
// Basis-dependence hooks.
abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double [] getEDPsi(final double [] psi);
abstract double [] getPsi2WIntegratedDt(final double [] psi);




void dumpTempDataAtSomePoints(final String fprefix,final RMomentsData mm,long tau,long tlast,final double eps)
{
    if(tau!=128000000000L) return;

    final int n=mm.n;
    for(double tdump:new double []{9.94373,9.96124,9.97245,9.98045,9.99917,10.0082,10.0482,10.1427}){

	if(false){
	dumpDIselIH("DIselIHsepdpdt",fprefix,mm,tdump,tau,tlast,M,diselIHsepdpdt);
	dumpDIselIH("DIselIHsepd2pdt2",fprefix,mm,tdump,tau,tlast,M,diselIHsepd2pdt2);
	dumpDIselIH("DIselIHsepdDpwvdt",fprefix,mm,tdump,tau,tlast,M,diselIHsepdDpwvdt);
	dumpDIselIH("DIselIHsepdIdt",fprefix,mm,tdump,tau,tlast,M,diselIHsepdIdt);
	dumpDIselIH("DIselIHsepPIH",fprefix,mm,tdump,tau,tlast,M,diselIHsepPIH);
	dumpDIselIH("DIselIHsepDMpsiH",fprefix,mm,tdump,tau,tlast,M,diselIHsepDMpsiH);

	dumpDIselIH("DIselIH2sepdpdt",fprefix,mm,tdump,tau,tlast,M,diselIH2sepdpdt);
	dumpDIselIH("DIselIH2sepd2pdt2",fprefix,mm,tdump,tau,tlast,M,diselIH2sepd2pdt2);
	dumpDIselIH("DIselIH2sepdDpwvdt",fprefix,mm,tdump,tau,tlast,M,diselIH2sepdDpwvdt);
	dumpDIselIH("DIselIH2sepdIdt",fprefix,mm,tdump,tau,tlast,M,diselIH2sepdIdt);
	dumpDIselIH("DIselIH2sepPIH",fprefix,mm,tdump,tau,tlast,M,diselIH2sepPIH);
	}

	if(false){
	dumpDIselD2P("DIselD2P_pt",fprefix,mm,tdump,tau,tlast,M,diseld2pt);
	dumpDIselD2P("DIselD2P_pi",fprefix,mm,tdump,tau,tlast,M,diseld2pi);
	dumpDIselD2PstateIH("DIselD2PstateIH_diseld2ptIH",fprefix,mm,tdump,tau,tlast,M,diseld2ptIH);
	dumpDIselD2PstateIH("DIselD2PstateIH_diseld2piIH",fprefix,mm,tdump,tau,tlast,M,diseld2piIH);
	dumpDIselD2PstateIH("DIselD2PstateIH_diseld2ptIHpure",fprefix,mm,tdump,tau,tlast,M,diseld2ptIHpure);
	dumpDIselD2PstateIH("DIselD2PstateIH_diseld2piIHpure",fprefix,mm,tdump,tau,tlast,M,diseld2piIHpure);
	}

	dumpIstatesSelection("IHStateDM",fprefix,mm,tdump,tau,tlast,M,istatesselection);
    }
}

static void dumpIstatesSelection(final String fprefixClass,final String fprefix,final RMomentsData mm,double tdump,long tau,long tlast,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final IstatesSelection istatesSelection)
{
    dumpFback2(fprefixClass+"_IstatesSelection_"+fprefix,tdump,tau,tlast,
	       istatesSelection.I.x0,0,10000,
	       istatesSelection.I,
	       mm.tmom,mm.imom,
	       TMatr.sumAcB(mm.ptmom,mm.p_offset,mm.tmom),mm.tmom,
	       TMatr.sumAcB(mm.pimom,mm.p_offset,mm.imom),mm.imom,
	       TMatr.sumAcB(istatesSelection.QQp,mm.p_offset,mm.tmom),
	       TMatr.sumAcB(istatesSelection.QQp_diagI,mm.p_offset,mm.tmom),
	       TMatr.sumAcB(istatesSelection.QQp_offdiagI,mm.p_offset,mm.tmom),
	       TMatr.sumAcB(istatesSelection.QQp_offdiagInoIH,mm.p_offset,mm.tmom),
	       M);

}

static void dumpDIselIH(final String fprefixClass,final String fprefix,final RMomentsData mm,double tdump,long tau,long tlast,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final DIselIH diselIH)
{
    final int n=diselIH.I.neq;
    dumpFback(fprefixClass+"_DMIH_"+fprefix,tdump,tau,tlast,
	      diselIH.I.x0,0,10000,diselIH.I.psiH,diselIH.I.psiL,
	      mm.tmom,mm.imom,
	      TMatr.sumAcB(mm.ptmom,mm.p_offset,mm.tmom),mm.tmom,
	      TMatr.sumAcB(mm.pimom,mm.p_offset,mm.imom),mm.imom,
	      mm.dpmom,mm.Dpwt,mm.Dpwv,diselIH.QQDEVseparation,n,diselIH.DMIH_plus,diselIH.DMIH_minus,M);

}
static void dumpDIselD2P(final String fprefixClass,final String fprefix,final RMomentsData mm,double tdump,long tau,long tlast,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final DIselD2P diseld2p)
{
    final int n=diseld2p.I.neq;
    dumpFback(fprefixClass+"_DMIH_"+fprefix,tdump,tau,tlast,
	      diseld2p.I.x0,0,10000,diseld2p.I.psiH,diseld2p.I.psiL,
	      mm.tmom,mm.imom,
	      TMatr.sumAcB(mm.ptmom,mm.p_offset,mm.tmom),mm.tmom,
	      TMatr.sumAcB(mm.pimom,mm.p_offset,mm.imom),mm.imom,
	      mm.dpmom,
	      mm.Dpwt,mm.Dpwv,
	      TMatr.sumAcB(diseld2p.QQp_diagI,mm.p_offset,mm.tmom),
	      n,/*diseld2p.QQd_plus*/mm.tmom,/*diseld2p.QQd_minus*/mm.tmom,M);

}
static void dumpDIselD2PstateIH(final String fprefixClass,final String fprefix,final RMomentsData mm,double tdump,long tau,long tlast,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final DIselD2PstateIH diseld2pstateih)
{
    final int n=diseld2pstateih.I.neq;
    dumpFback(fprefixClass+"_DMIH_"+fprefix,tdump,tau,tlast,
	      diseld2pstateih.I.x0,0,10000,diseld2pstateih.I.psiH,diseld2pstateih.I.psiL,
	      mm.tmom,mm.imom,
	      TMatr.sumAcB(mm.ptmom,mm.p_offset,mm.tmom),mm.tmom,
	      TMatr.sumAcB(mm.pimom,mm.p_offset,mm.imom),mm.imom,
	      mm.dpmom,
	      mm.Dpwt,mm.Dpwv,
	      TMatr.sumAcB(diseld2pstateih.QQp_diagI,mm.p_offset,mm.tmom),
	      n,mm.tmom,mm.tmom,M);

}


static final java.util.HashMap<String,String> h=new java.util.HashMap<String,String>();
public static void dumpFback(
    final String out_fname_prefix,final double t0dumpinhours,long tau,final long t,
    final double x0,final double xend,final int nsteps,
    final double [] psiH,final double [] psiL,
    final double [] QQt,final double [] QQi,
    final double [] QQpt,final double [] QQt_norm,
    final double [] QQpi,final double [] QQi_norm,
    final double [] QQdpdt,
    final double [] QQft,final double [] QQfi,
    final double [] QQsepar,final int n,
    final double [] DMplus,final double [] DMminus,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    java.io.PrintWriter out=getFnameToDump(out_fname_prefix,n,t0dumpinhours,tau,t);
    if(out==null) return;

    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
    out.println("|1:x\t2:I\t3:pt\t4:pi\t5:dp/dt\t6:ft\t7:fi\t8:separ\t9:w_psiH2\t10:w_psiL2\t11:DMplus\t12:DMminus");
    out.println("|t="+t);
    for(int nt=0;nt<=nsteps;nt++)
    {
	final double x=x0+((xend-x0)*nt)/nsteps;
	final double [] psixm=TMatr.getMatr_x_A(n,IQQt,M.B.getX0Moments(n,x));
	final double normpsi=TMatr.getAMatrA(psixm,QQt,n),
	    w_psiH=Math.pow(TMatr.getAMatrB(psiH,psixm,QQt,n),2)/normpsi,
	    w_psiL=Math.pow(TMatr.getAMatrB(psiL,psixm,QQt,n),2)/normpsi,
	    I=TMatr.getAMatrA(psixm,QQi,n)/normpsi,
	    separ=TMatr.getAMatrA(psixm,QQsepar,n)/normpsi,
	    dpdt=TMatr.getAMatrA(psixm,QQdpdt,n)/normpsi,
	    ft=TMatr.getAMatrA(psixm,QQft,n)/normpsi,
	    fi=TMatr.getAMatrA(psixm,QQfi,n)/normpsi,
	    pt=TMatr.getAMatrA(psixm,QQpt,n)/TMatr.getAMatrA(psixm,QQt_norm,n),
	    pi=TMatr.getAMatrA(psixm,QQpi,n)/TMatr.getAMatrA(psixm,QQi_norm,n),
	    dmplus=TMatr.getAMatrA(psixm,DMplus,n)/normpsi,
	    dmminus=TMatr.getAMatrA(psixm,DMminus,n)/normpsi;

	out.println(String.valueOf(x)+"\t"+I+"\t"+pt+"\t"+pi+"\t"+dpdt+"\t"+ft+"\t"+fi+"\t"+separ+"\t"+w_psiH+"\t"+w_psiL+"\t"+dmplus+"\t"+dmminus);
    }
    out.close();
}

static java.io.PrintWriter getFnameToDump(final String out_fname_prefix,final int n,final double t0dumpinhours,long tau,final long t)
{
    final long t0=(long)(t0dumpinhours*3600e9);
    if(t<t0) return null;
    final String out_fname_key=out_fname_prefix+"_n_"+n+"_tdump_"+t0dumpinhours+"_tau_"+tau;
    if(h.get(out_fname_key)!=null) return null;
    h.put(out_fname_key,out_fname_key);
    final String out_fname=out_fname_key+"."+t+".dump.txt";
    java.io.PrintWriter out=null;
    try {
	out=new java.io.PrintWriter(new java.io.FileWriter(out_fname));
    } catch (Throwable e) {throw new RuntimeException(e);}
    return out;
}

public static void dumpFback2(
    final String out_fname_prefix,final double t0dumpinhours,long tau,final long t,
    final double x0,final double xend,final int nsteps,
    final EVXData I,
    final double [] QQt,final double [] QQi,
    final double [] QQpt,final double [] QQt_norm,
    final double [] QQpi,final double [] QQi_norm,
    final double [] QQp,final double [] QQp_diagI,final double [] QQp_offdiagI,final double [] QQp_offdiagInoIH,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final int n=I.neq;
    java.io.PrintWriter out=getFnameToDump(out_fname_prefix,n,t0dumpinhours,tau,t);
    if(out==null) return;

    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
    out.print("|1:x\t2:I\t3:pt\t4:pi\t5:p\t6:p_diagI\t7:p_offdiagI\t8:p_offdiagInoIH");
    for(int k=0;k<n;k++) out.print("\t"+(k+9)+":wpsi"+k);
    out.println();
    out.println("|t="+t);
    for(int nt=0;nt<=nsteps;nt++)
    {
	final double x=x0+((xend-x0)*nt)/nsteps;
	final double [] psixm=TMatr.getMatr_x_A(n,IQQt,M.B.getX0Moments(n,x));
	final double normpsi=TMatr.getAMatrA(psixm,QQt,n),
	    i=TMatr.getAMatrA(psixm,QQi,n)/normpsi,
	    pt=TMatr.getAMatrA(psixm,QQpt,n)/TMatr.getAMatrA(psixm,QQt_norm,n),
	    pi=TMatr.getAMatrA(psixm,QQpi,n)/TMatr.getAMatrA(psixm,QQi_norm,n),
	    p=TMatr.getAMatrA(psixm,QQp,n)/normpsi,
	    p_diagI=TMatr.getAMatrA(psixm,QQp_diagI,n)/normpsi,
	    p_offdiagI=TMatr.getAMatrA(psixm,QQp_offdiagI,n)/normpsi,
	    p_offdiagInoIH=TMatr.getAMatrA(psixm,QQp_offdiagInoIH,n)/normpsi;
	out.print(String.valueOf(x)+"\t"+i+"\t"+pt+"\t"+pi+"\t"+p+"\t"+p_diagI+"\t"+p_offdiagI+"\t"+p_offdiagInoIH);
	for(int k=0;k<n;k++){
	    final double wk=Math.pow(I.getFPsiPsi(k,psixm,QQt),2)/normpsi;
	    out.print("\t"+wk);
	}
	out.println();
    }
    out.close();
}

/** A class for unit test.
*/
static abstract class IHStateDMTest<DTYPE> extends QTest {

abstract IHStateDM<DTYPE> getIHStateDM();
abstract ImpactQVM<DTYPE> getImpactQVM();

IHStateDMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_Extra(){}

void test_ImpactQVM()
{
    final int n=neq;
    final IHStateDM<DTYPE> q=getIHStateDM();
    InIStateIHDensity.matchDM_DI(getImpactQVM().inistateihdensity,q.dmdi,eps);
    DIstatesVT.matchDIselVT(q.distatesVT,q.diselVT,eps);
    DIselVT.matchDIselVTsimple(q.diselVT,q.diselVTsimple,eps);
    System.err.println(name+":IHStateDMTest.test_ImpactQVM(): test OK n="+n+" eps="+eps);
}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":IHStateDMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Extra();
    test_ImpactQVM();

    System.err.println(name+":IHStateDMTest.test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_min,final int neq_max)
{
    System.err.println(name+":IHStateDMTest: starting tests; ver=$Id: IHStateDM.java,v 1.237 2022/08/08 20:10:46 mal Exp $");
    EPS_KLUDGY_WAY_TO_TURN_ON_THE_DEBUGGING=eps;
    final long t0=System.currentTimeMillis();
    for(int neq=neq_min;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    EPS_KLUDGY_WAY_TO_TURN_ON_THE_DEBUGGING=0;
    final long t1=System.currentTimeMillis();
    System.err.println(name+":IHStateDMTest.main: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: IHStateDM.java,v 1.237 2022/08/08 20:10:46 mal Exp $");
}
} // END OF class IHStateDMTest


double getVar1(final IstatesConditional is){
    return Double.NaN;
    //return com.polytechnik.utils.RayleighQuotient.getRQVariationValues(is.psi_M,VOLATILITY_EDPsi.getDPsi(is.psi_M),null,is.QQc,is.QQt,is.psi_M.length)[1];
}
double getPX(final IstatesConditional is,final double [] QQnom,final double [] QQdenom){
    final int n=is.psi_M.length;
    return TMatr.getAMatrA(is.psi_M,QQnom,n)/TMatr.getAMatrA(is.psi_M,QQdenom,n);
}

static void matchScalpedMaxIProjection(final IHStateDM<?> a,final ScalpedMaxIProjection<?> b,final double eps){


    System.err.println("IHStateDM:matchScalpedMaxIProjection test OK eps="+eps);
}

}
