package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: IHStateDMLaguerre.java,v 1.5 2022/07/10 10:41:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IHStateDMLaguerre extends IHStateDM<QVMDataL> {
IHStateDMLaguerre(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}


public static IHStateDMLaguerre calculate(final int neq,
					  final long currenttime,
					  final long tau,
					  final Trade [] tb,
					  final int tsize,
					  final IHStateDMLaguerre ref)
{

    final int npm=4,nmoms=2*neq-1;
    final QVMDataL o=QVMDataL.calculateQVMDataL(nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new IHStateDMLaguerre(neq,o,ref!=null?ref.sst:null);
}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double [] getPsi2WIntegratedDt(final double [] psi){return WIntegratorLaguerre.getPsi2WIntegratedDt(psi);}
double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}


static class IHStateDMLaguerreTest extends IHStateDMTest<QVMDataL> {
IHStateDMLaguerreTest(final long initrandom)
{
    super("IHStateDMLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
ImpactQVMLaguerre getImpactQVM(){return ImpactQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
IHStateDMLaguerre getIHStateDM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class IHStateDMLaguerreTest

public static void main(String [] args)
{
    new IHStateDMLaguerreTest(11127L).runTests(2,5);
} // END main


}
