package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: IHStateDMMonomials.java,v 1.31 2022/07/10 10:41:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IHStateDMMonomials extends IHStateDM<QVMData> {
    IHStateDMMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static IHStateDMMonomials calculate(final int neq,
					  final long currenttime,
					  final long tau,
					  final Trade [] tb,
					  final int tsize,
					  final IHStateDMMonomials ref)
{

    final int nvm=1,npm=4,nmoms=2*neq-1;
    final QVMData o=QVMData.calculateQVMData(nvm,nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
					     ref!=null?ref.odata:null);

    return new IHStateDMMonomials(neq,o,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double [] getPsi2WIntegratedDt(final double [] psi){return WIntegratorMonomials.getPsi2WIntegratedDt(psi);}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}


static class IHStateDMMonomialsTest extends IHStateDMTest<QVMData> {
IHStateDMMonomialsTest(final long initrandom)
{
    super("IHStateDMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}



@Override
void test_Extra(){
    // These to be identical
    final IHStateDMMonomials qm=getIHStateDM();
    final IHStateDMLaguerre ql=getIHStateDMLaguerre();
    final ImpactQVMMonomials sm=getImpactQVM();
    final ImpactQVMLaguerre sl=getImpactQVMLaguerre();
    DM_DI.matchDM_DI(qm.dmdi,ql.dmdi,eps);
    DM_DI.matchDM_DI(ql.dmdi,qm.dmdi,eps);
    InIStateIHDensity.matchDM_DI(sl.inistateihdensity,qm.dmdi,eps);
    InIStateIHDensity.matchDM_DI(sm.inistateihdensity,ql.dmdi,eps);

    DIselIH.matchDIselIH(qm.diselIHsepdpdt,ql.diselIHsepdpdt,eps);
    DIselIH.matchDIselIH(qm.diselIHsepd2pdt2,ql.diselIHsepd2pdt2,eps);
    DIselIH.matchDIselIH(qm.diselIHsepdDpwvdt,ql.diselIHsepdDpwvdt,eps);
    DIselIH.matchDIselIH(qm.diselIHsepdIdt,ql.diselIHsepdIdt,eps);
    DIselIH.matchDIselIH(qm.diselIHsepPIH,ql.diselIHsepPIH,eps);
    DIselIH.matchDIselIH(qm.diselIHsepDMpsiH,ql.diselIHsepDMpsiH,eps);

    DIselIH.matchDIselIH(qm.diselIH2sepdpdt,ql.diselIH2sepdpdt,eps);
    DIselIH.matchDIselIH(qm.diselIH2sepd2pdt2,ql.diselIH2sepd2pdt2,eps);
    DIselIH.matchDIselIH(qm.diselIH2sepdDpwvdt,ql.diselIH2sepdDpwvdt,eps);
    DIselIH.matchDIselIH(qm.diselIH2sepdIdt,ql.diselIH2sepdIdt,eps);
    DIselIH.matchDIselIH(qm.diselIH2sepPIH,ql.diselIH2sepPIH,eps);

    DIselIH2pi.matchDIselIH2pi(qm.diselPSkewness,ql.diselPSkewness,eps);
    DIselIH2pi.matchDIselIH2pi(qm.diselPSkewnessF,ql.diselPSkewnessF,eps);
    DIselIH2pi.matchDIselIH2pi(qm.diselPSkewness0,ql.diselPSkewness0,eps);
    DIselIH2pi.matchDIselIH2pi(qm.diselPSkewness0F,ql.diselPSkewness0F,eps);

    DIselPsi0.matchDIselPsi0(qm.diselPsi0,ql.diselPsi0,eps);
    DIselPsi0.matchDIselPsi0(qm.diselPsi0F,ql.diselPsi0F,eps);

    DIselIH2PositiondI.matchDIselIH2PositiondI(qm.disel2PositiondI,ql.disel2PositiondI,eps);
    DIselIHPositiondI.matchDIselIHPositiondI(qm.diselPositiondI_inPsiH,ql.diselPositiondI_inPsiH,eps);
    DIselIHPositiondI.matchDIselIHPositiondI(qm.diselPositionPdI_inPsiH,ql.diselPositionPdI_inPsiH,eps);
    DIselIHPositiondI.matchDIselIHPositiondI(qm.diselPositiondI_Dp_afterPsiH,ql.diselPositiondI_Dp_afterPsiH,eps);
    DIselIHPositiondI.matchDIselIHPositiondI(qm.diselPositiondI_Dpv_afterPsiH,ql.diselPositiondI_Dpv_afterPsiH,eps);
    DIselIHskewness.matchDIselIHskewness(qm.diselsk,ql.diselsk,eps);

    MaxPPl2I.matchMaxPPl2I(qm.maxppl2V,ql.maxppl2V,eps);
    MaxPPl2I.matchMaxPPl2I(qm.maxppl2I,ql.maxppl2I,eps);

    MaxPPl2Iinbasis.matchMaxPPl2Iinbasis(qm.maxppl2iinbasis,ql.maxppl2iinbasis,eps);

    DIstatesVT.matchDIselVT(qm.distatesVT,ql.diselVT,eps);
    DIstatesVT.matchDIselVT(ql.distatesVT,qm.diselVT,eps);
    DIselVT.matchDIselVTsimple(ql.diselVT,qm.diselVTsimple,eps);
    DIselVT.matchDIselVTsimple(qm.diselVT,ql.diselVTsimple,eps);

    System.err.println(name+":DM_DIMonomialsTest:test_Extra(): test DM_DILaguerre OK neq="+neq+" eps="+eps);
}


IHStateDMMonomials getIHStateDM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
IHStateDMLaguerre getIHStateDMLaguerre(){return IHStateDMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
ImpactQVMMonomials getImpactQVM(){return ImpactQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}
ImpactQVMLaguerre getImpactQVMLaguerre(){return ImpactQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class IHStateDMMonomialsTest

public static void main(String [] args)
{
    new IHStateDMMonomialsTest(434L).runTests(2,4);
} // END main

}
