package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: IHStateLaguerre.java,v 1.23 2022/07/10 10:41:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IHStateLaguerre extends IHState<QVMDataL> {
IHStateLaguerre(final int neq,final QVMDataL odata,final QVMDataL odata2,final State rst){super(neq,odata,odata2,rst);}


public static IHStateLaguerre calculate(final int neq,
					  final long currenttime,
					  final long tau,
					  final Trade [] tb,
					  final int tsize,
					  final IHStateLaguerre ref)
{

    final int npm=4,nmoms=2*(neq-1)+1;
    final QVMDataL o=QVMDataL.calculateQVMDataL(nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null),
	o2=QVMDataL.calculateQVMDataL(4*(neq-1)+1,
				      npm,
				      currenttime,
				      tau/2,
				      tb,
				      tsize,
				      ref!=null?ref.odata2:null);

    return new IHStateLaguerre(neq,o,o2,ref!=null?ref.sst:null);
}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
RMomentsData getPsiMoments(final int nmoms2,final RMomentsData mom,final double [] psi,final boolean flag_gt)
{
    return getPsiMomentsTwoExponents(nmoms2,mom,odata,odata2,getAccess(),
				     (final double [] m,final double [] m2)->KkQVMLaguerre._getSMoments(nmoms2,m,m2,flag_gt,psi,psi));
}



static <T> RMomentsData getPsiMomentsTwoExponents(final int nmoms2,final RMomentsData mom,final T odata,final T odata2,final DataInterfaceToMoments<T> a,final java.util.function.BiFunction<double [],double [],double []>getM2Moments)
{
    if(mom.flagQQ) throw new IllegalArgumentException();
    final double dp_offset=a.getPOffset(odata2)-a.getPOffset(odata);
    final RMomentsData mom2=RMomentsData.get_MomentsData(mom.n+nmoms2-1,odata2,a);
    // shift mom2 price offset to the same one as mom
    Polynomials.shiftPOffsetInPlace(mom2.n,mom2.tmom,mom2.ptmom,mom2.p2tmom,mom2.p3tmom,-dp_offset);
    Polynomials.shiftPOffsetInPlace(mom2.n,mom2.imom,mom2.pimom,mom2.p2imom,mom2.p3imom,-dp_offset);
    Polynomials.shiftPOffsetInPlace(mom2.n,mom2.amom,mom2.pamom,mom2.p2amom,mom2.p3amom,-dp_offset);
    // to have proper dt/tau  (tau is twice lower for mom2),
    // the getM2Moments assumes dt/tau integration and scale it, double the result to compensate
    TMatr.scaleVectorInPlace(mom2.imom,2);
    TMatr.scaleVectorInPlace(mom2.pimom,2);
    TMatr.scaleVectorInPlace(mom2.p2imom,2);
    TMatr.scaleVectorInPlace(mom2.p3imom,2);
    TMatr.scaleVectorInPlace(mom2.amom,2);
    TMatr.scaleVectorInPlace(mom2.pamom,2);
    TMatr.scaleVectorInPlace(mom2.p2amom,2);
    TMatr.scaleVectorInPlace(mom2.p3amom,2);
    TMatr.scaleVectorInPlace(mom2.dpmom,2);
    return new RMomentsData(nmoms2,mom.p_offset,mom.p_last,
			    getM2Moments.apply(mom.tmom,mom2.tmom),
			    getM2Moments.apply(mom.ptmom,mom2.ptmom),
			    getM2Moments.apply(mom.p2tmom,mom2.p2tmom),
			    getM2Moments.apply(mom.p3tmom,mom2.p3tmom),
			    getM2Moments.apply(mom.imom,mom2.imom),
			    getM2Moments.apply(mom.pimom,mom2.pimom),
			    getM2Moments.apply(mom.p2imom,mom2.p2imom),
			    getM2Moments.apply(mom.p3imom,mom2.p3imom),
			    getM2Moments.apply(mom.amom,mom2.amom),
			    getM2Moments.apply(mom.pamom,mom2.pamom),
			    getM2Moments.apply(mom.p2amom,mom2.p2amom),
			    getM2Moments.apply(mom.p3amom,mom2.p3amom),
			    getM2Moments.apply(mom.dpmom,mom2.dpmom),
			    getM2Moments.apply(mom.Tmom,mom2.Tmom),
			    getM2Moments.apply(mom.Vmom,mom2.Vmom),
			    getM2Moments.apply(mom.pVmom,mom2.pVmom),
			    getM2Moments.apply(mom.p2Vmom,mom2.p2Vmom),
			    getM2Moments.apply(mom.p3Vmom,mom2.p3Vmom),
			    getM2Moments.apply(mom.Dpwv,mom2.Dpwv),
			    getM2Moments.apply(mom.Dpwt,mom2.Dpwt),
			    false);
}

static class IHStateLaguerreTest extends IHStateTest<QVMDataL> {
IHStateLaguerreTest(final long initrandom)
{
    super("IHStateLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
IkQVMLaguerre getIkQVM(){return IkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
IAImpactLaguerre getIAImpact(){return IAImpactLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
IHStateLaguerre getIHState(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

public static void main(String [] args)
{
    new IHStateLaguerreTest(11127L).runTests(2,5);
} // END main
} // END of class IHStateLaguerreTest


}
