package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: IHStateLegendreShifted.java,v 1.14 2022/07/10 10:41:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IHStateLegendreShifted extends IHState<QVMDataP> {
IHStateLegendreShifted(final int neq,final QVMDataP odata,final State rst){super(neq,odata,null,rst);}


public static IHStateLegendreShifted calculate(final int neq,
					  final long currenttime,
					  final long tau,
					  final Trade [] tb,
					  final int tsize,
					  final IHStateLegendreShifted ref)
{

    final int npm=4,nmoms=2*(neq-1)+1+ 2*(neq-1)+1;
    final QVMDataP o=QVMDataP.calculateQVMDataP(nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new IHStateLegendreShifted(neq,o,ref!=null?ref.sst:null);
}
RMomentsData getRMomentsInitial(final int nmoms,final QVMDataP odata)
{
    return RMomentsData.get_MomentsData(2*nmoms,odata,getAccess());
}
QVMDataPDirectAccess getAccess(){return new QVMDataPDirectAccess();}
OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
RMomentsData getPsiMoments(final int nmoms2,final RMomentsData mom,final double [] psi,final boolean flag_gt)
{
    if(mom.flagQQ) throw new IllegalArgumentException();
    return new RMomentsData(nmoms2,mom.p_offset,mom.p_last,
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.tmom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.ptmom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.p2tmom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.p3tmom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.imom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.pimom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.p2imom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.p3imom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.amom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.pamom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.p2amom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.p3amom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.dpmom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.Tmom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.Vmom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.pVmom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.p2Vmom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.p3Vmom,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.Dpwv,flag_gt,psi,psi),
			    KkQVMLegendreShifted._getSMoments(nmoms2,mom.Dpwt,flag_gt,psi,psi),
			    false);
}



static class IHStateLegendreShiftedTest extends IHStateTest<QVMDataP> {
IHStateLegendreShiftedTest(final long initrandom)
{
    super("IHStateLegendreShifted",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
IkQVMLegendreShifted getIkQVM(){return IkQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
IAImpactLegendreShifted getIAImpact(){return IAImpactLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
IHStateLegendreShifted getIHState(){return calculate(neq,currenttime,tau,tb,tb.length,null);}


public static void main(String [] args)
{
    new IHStateLegendreShiftedTest(7163L).runTests(2,7);
} // END main
} // END of class IHStateLegendreShiftedTest


}


