package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: IHStateMonomials.java,v 1.14 2018/08/12 21:42:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IHStateMonomials extends IHState<QVMData> {
IHStateMonomials(final int neq,final QVMData odata,final QVMData odata2,final State rst){super(neq,odata,odata2,rst);}


public static IHStateMonomials calculate(final int neq,
					  final long currenttime,
					  final long tau,
					  final Trade [] tb,
					  final int tsize,
					  final IHStateMonomials ref)
{

    final int nvm=1,npm=4,nmoms=2*(neq-1)+1;
    final QVMData o=QVMData.calculateQVMData(nvm,nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null),
	o2=QVMData.calculateQVMData(nvm,4*(neq-1)+1,
				      npm,
				      currenttime,
				      tau/2,
				      tb,
				      tsize,
				      ref!=null?ref.odata2:null);

    return new IHStateMonomials(neq,o,o2,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
RMomentsData getPsiMoments(final int nmoms2,final RMomentsData mom,final double [] psi,final boolean flag_gt)
{
    return IHStateLaguerre.getPsiMomentsTwoExponents(nmoms2,mom,odata,odata2,getAccess(),
				     (final double [] m,final double [] m2)->KkQVMMonomials._getSMoments(nmoms2,m,m2,flag_gt,psi,psi));
}


static class IHStateMonomialsTest extends IHStateTest<QVMData> {
IHStateMonomialsTest(final long initrandom)
{
    super("IHStateMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}



@Override
void test_Extra(){
    // These to be identical
    final IHStateMonomials q=getIHState();
    final IHStateLaguerre s=getIHStateLaguerre();
    matchIHState(q,s,eps);
    System.err.println(name+":IHStateMonomialsTest:test_Extra(): test IHStateLaguerre OK neq="+neq+" eps="+eps);
}


IHStateMonomials getIHState(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
IHStateLaguerre getIHStateLaguerre(){return IHStateLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
IkQVMLaguerre getIkQVMLaguerre(){return IkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
IkQVMMonomials getIkQVM(){return IkQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}
IAImpactMonomials getIAImpact(){return IAImpactMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}


public static void main(String [] args)
{
    new IHStateMonomialsTest(434L).runTests(2,4);
} // END main
} // END of class IHStateMonomialsTest


}
