package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
/**
 * Helper class, like MomentsData, but different set of moments to calculate.
 *  @version $Id: IMomentsData.java,v 1.25 2022/06/30 08:18:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IMomentsData {
    public final int n;
    public final double p_offset,p_last;
    public final double [] QQi,QQt,QQpi,QQp2i,QQp3i,
	QQt_sample,QQpt_sample,QQDp,QQpt_parts,QQp2t_sample,QQp3t_sample,QQdpdt,
	QQDpwv,QQDpwt,QQDawv,QQDawt,QQa,QQT,QQV,QQpV,QQp2V,QQp3V,QQDpV,QQT_sample,QQDpT;
    public IMomentsData(final int n,final double p_offset,final double p_last,
			final double [] QQi,final double [] QQt,final double [] QQpi,final double [] QQp2i,final double [] QQp3i,
			final double [] QQt_sample,final double [] QQpt_sample,final double [] QQDp,final double [] QQpt_parts,final double [] QQp2t_sample,final double [] QQp3t_sample,
			final double [] QQdpdt,
			final double [] QQDpwv,final double [] QQDpwt,final double [] QQDawv,final double [] QQDawt,
			final double [] QQa,
			final double [] QQT,final double [] QQV,final double [] QQpV,final double [] QQp2V,final double [] QQp3V,
			final double [] QQDpV,
			final double [] QQT_sample,final double [] QQDpT)
    {
	this.n=n;
	this.p_offset=p_offset;
	this.p_last=p_last;
	this.QQi=QQi;
	this.QQt=QQt;
	this.QQpi=QQpi;
	this.QQp2i=QQp2i;
	this.QQp3i=QQp3i;
	this.QQt_sample=QQt_sample;
	this.QQpt_sample=QQpt_sample;
	this.QQDp=QQDp;
	this.QQpt_parts=QQpt_parts;
	this.QQDpwv=QQDpwv;
	this.QQDpwt=QQDpwt;
	this.QQDawv=QQDawv;
	this.QQDawt=QQDawt;
	this.QQa=QQa;
	this.QQp2t_sample=QQp2t_sample;
	this.QQp3t_sample=QQp3t_sample;
	this.QQdpdt=QQdpdt;
	this.QQT=QQT;
	this.QQV=QQV;
	this.QQpV=QQpV;
	this.QQp2V=QQp2V;
	this.QQp3V=QQp3V;
	this.QQDpV=QQDpV;
	this.QQT_sample=QQT_sample;
	this.QQDpT=QQDpT;
    }


/** Calculate the moments in time basis.
 * QQi_jk={@literal <Q_j I Q_k>} , QQt_jk={@literal <Q_j Q_k>}, QQpi_jk={@literal <Q_j pI Q^k>},
 */
public static <DTYPE> IMomentsData get_IMomentsData(final int n,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final DTYPE odata,final DataInterfaceToMoments<DTYPE> access)
{
    final int nmoms=2*n-1;
    final double [] mom=new double [nmoms],mmom=new double [nmoms],tmom=new double [nmoms],dpmom=new double [nmoms];
    access.setMT(tmom,nmoms);final double [] QQt=M.B.getQQMatr(n,tmom);
    access.setFMoments(mmom,tmom,nmoms);final double [] QQT=M.B.getQQMatr(n,mmom);
    access.setM(mom,0,access.M_I,odata,nmoms);final double [] QQi=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQV=M.B.getQQMatr(n,mmom);
    access.setM(mom,1,access.M_I,odata,nmoms);final double [] QQpi=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQpV=M.B.getQQMatr(n,mmom);
    access.setM(mom,2,access.M_I,odata,nmoms);final double [] QQp2i=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQp2V=M.B.getQQMatr(n,mmom);
    access.setM(mom,3,access.M_I,odata,nmoms);final double [] QQp3i=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQp3V=M.B.getQQMatr(n,mmom);
    access.setM(mom,0,access.M_T,odata,nmoms);final double [] QQt_sample=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQT_sample=M.B.getQQMatr(n,mmom);
    access.setM(mom,1,access.M_T,odata,nmoms);final double [] QQpt_sample=M.B.getQQMatr(n,mom);
    access.setM(mom,2,access.M_T,odata,nmoms);final double [] QQp2t_sample=M.B.getQQMatr(n,mom);
    access.setM(mom,3,access.M_T,odata,nmoms);final double [] QQp3t_sample=M.B.getQQMatr(n,mom);
    access.setM(mom,0,access.M_DPwV,odata,nmoms);final double [] QQDpwv=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQDpV=M.B.getQQMatr(n,mmom);
    access.setM(mom,0,access.M_DPwT,odata,nmoms);final double [] QQDpwt=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQDpT=M.B.getQQMatr(n,mmom);
    access.setM(mom,0,access.M_DAwV,odata,nmoms);final double [] QQDawv=M.B.getQQMatr(n,mom);
    access.setM(mom,0,access.M_DAwT,odata,nmoms);final double [] QQDawt=M.B.getQQMatr(n,mom);
    access.setM(mom,0,access.M_A,odata,nmoms);final double [] QQa=M.B.getQQMatr(n,mom);

    final double pl_m_poffset=access.getPLast(odata);// p_offset is already included in access.getPLast(DTYPE odata).
    access.setM(dpmom,1,access.M_DP,odata,nmoms); /* Polynomials.shiftPOffsetInPlace(nmoms,dp0mom,dp1mom,dp2mom,dp3mom,pl_m_poffset); dp0mom=0, no need */final double [] QQdpdt=M.B.getQQMatr(n,dpmom);

    access.setFMoments(mom,dpmom,nmoms);final double [] QQDp=M.B.getQQMatr(n,mom);
    Polynomials.shiftPOffsetInPlace(nmoms,tmom,mom,null,null,-pl_m_poffset);final double [] QQpt_parts=M.B.getQQMatr(n,mom);

    return new IMomentsData(n,access.getPOffset(odata),access.getPLast(odata),QQi,QQt,QQpi,QQp2i,QQp3i,
			    QQt_sample,QQpt_sample,QQDp,QQpt_parts,QQp2t_sample,QQp3t_sample,QQdpdt,
			    QQDpwv,QQDpwt,QQDawv,QQDawt,QQa,QQT,QQV,QQpV,QQp2V,QQp3V,QQDpV,QQT_sample,QQDpT);
}

/** Calculate moments in p^k basis. QQi_jk={@literal <p^j I p^k>},
 * QQt_jk={@literal <p^j p^k>}, QQpi_jk={@literal <p^j pI p^k>}.
 * Limited set of moments, because in p^k basis, some V- and T- moments
 * cannot be calculated using integration by parts,
 * and direct calculation is not implemented for most DTYPEs used.
 */
public static <DTYPE> IMomentsData get_PriceMonomials_IMomentsData(final int n,final DTYPE odata,final DataInterfaceToMoments<DTYPE> access,final OrthogonalPolynomialsMonomialsBasis M)
{
    final int nmoms=2*n+2;
    final double [] r=new double [1],imom=new double [nmoms],tmom=new double [nmoms],
	Dpwv_mom=new double [nmoms],Dpwt_mom=new double [nmoms],Dawv_mom=new double [nmoms],Dawt_mom=new double [nmoms],amom=new double [nmoms],dpdtmom=new double [nmoms-1];
    for(int k=0;k<nmoms;k++)
    {
	access.setM(r,k,access.M_I,odata,1);imom[k]=r[0];
	access.setM(r,k,access.M_T,odata,1);tmom[k]=r[0];
	access.setM(r,k,access.M_DPwV,odata,1);Dpwv_mom[k]=r[0];
	access.setM(r,k,access.M_DPwT,odata,1);Dpwt_mom[k]=r[0];
	access.setM(r,k,access.M_DAwV,odata,1);Dawv_mom[k]=r[0];
	access.setM(r,k,access.M_DAwT,odata,1);Dawt_mom[k]=r[0];
	access.setM(r,k,access.M_A,odata,1);amom[k]=r[0];
	if(k>0)
	{
	    // In monomials basis: d(p^k)=kp^{k-1}dp.
	    access.setM(r,k,access.M_DP,odata,1);dpdtmom[k-1]=r[0]/k;
	    if(false)
	    {
		final double [] R=new double [1];
		access.setFMoments(R,r,1);
		System.err.println("k="+k+" tmom[k]="+tmom[k]+" R[0]="+R[0]+" should match approximately if p_offset=p_last");
	    }
	}
    }
    final double [] QQt=M.B.getQQMatr(n,tmom),
	pimom=M.B.getXQkMomentsFromQkMoments(imom),p2imom=M.B.getXQkMomentsFromQkMoments(pimom),p3imom=M.B.getXQkMomentsFromQkMoments(p2imom),
	ptmom=M.B.getXQkMomentsFromQkMoments(tmom),p2tmom=M.B.getXQkMomentsFromQkMoments(ptmom),p3tmom=M.B.getXQkMomentsFromQkMoments(p2tmom),
	QQpt=M.B.getQQMatr(n,ptmom),QQDp=TMatr.sumAcB(QQpt,-access.getPLast(odata),QQt);
    return new IMomentsData(n,access.getPOffset(odata),access.getPLast(odata),
			    M.B.getQQMatr(n,imom),QQt,
			    M.B.getQQMatr(n,pimom),M.B.getQQMatr(n,p2imom),M.B.getQQMatr(n,p3imom),
			    QQt,QQpt,QQDp,QQpt,M.B.getQQMatr(n,p2tmom),M.B.getQQMatr(n,p3tmom),M.B.getQQMatr(n,dpdtmom),
			    M.B.getQQMatr(n,Dpwv_mom),M.B.getQQMatr(n,Dpwt_mom),M.B.getQQMatr(n,Dawv_mom),M.B.getQQMatr(n,Dawt_mom),M.B.getQQMatr(n,amom),null,null,null,null,null,null,null,null);
}

}
