package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: IState.java,v 1.7 2020/09/27 20:33:14 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class IState<DTYPE> {

static class PS extends State2 {
    final double p;
    PS(final long t,final double I,final double p){super(t,I);this.p=p;}
}
public final StateSaver2<PS> sst;

final DataInterfaceToMoments<DTYPE> access=getAccess();
public final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
public final EVXData I;
final DTYPE odata;

IState(final int n,final DTYPE odata,final StateSaver2<PS> rst)
{
    //if(n<=1) throw new RuntimeException("n="+n+" must be >1");
    final double eps=0;
    this.odata=odata;
    this.M=getABasis();
    final JMomentsData mm=getJMomentsData(n);
    this.I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);

    final StateSaver2<PS> rs=rst==null?StateSaver2.<PS>getDefault():rst;
    if(true)
    {
	final double THR_IL=0.6,THR_IH=0.6;
	final double pl=access.getPOffset(odata)+access.getPLast(odata);
	if(Math.pow(I.wL,2)>=THR_IL) this.sst=rs.getUpdatedIL(new PS(access.getTLast(odata),I.s0,pl));
	else if(Math.pow(I.wH,2)>=THR_IH) this.sst=rs.getUpdatedIH(new PS(access.getTLast(odata),I.s0,pl));
	else this.sst=rs.getUpdatedNothing();
    }
}
protected JMomentsData getJMomentsData(final int n){return JMomentsData.get_JMomentsData(n,M,odata,access);}
abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();

/** A class for unit test.
*/
static abstract class IStateTest<DTYPE> extends QTest {

abstract IState<DTYPE> getIState();

IStateTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_Extra(){}



void test_IState()
{
    final int n=neq;
    final IState<DTYPE> q=getIState();
    final JMomentsData mm=q.getJMomentsData(n);
    EVXData.test_orthogonality("IState:I.ev",n,q.I.ev,mm.QQi,mm.QQt,eps);

    System.err.println(name+":IStateTest.test_IState(): test OK n="+n+" eps="+eps);
}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":IStateTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Extra();
    test_IState();

    System.err.println(name+":IStateTest.test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_min,final int neq_max)
{
    System.err.println(name+":IStateTest: starting tests; ver=$Id: IState.java,v 1.7 2020/09/27 20:33:14 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=neq_min;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":IStateTest.main: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: IState.java,v 1.7 2020/09/27 20:33:14 mal Exp $");
}
} // END OF class IStateTest




}// END OF class IState
