package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 * The |If|=|I| + {@literal |psi0>dI<psi0|}  states is back.
 *  @version $Id: IfStates.java,v 1.6 2022/08/05 11:25:32 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IfStates {


public final EVXData I;
    public final double p_IH,PfmPl,I_0_feq;

IfStates(final int n,final double p_last,
	 final double [] QQt,final double [] QQi,final double [] QQpi,
	 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	 final double eps)
{
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    I_0_feq=I.sH;
    final double I_0=I.s0,dI=I_0_feq-I_0,dV=dI/I.o_o;

    final double []
	bQQ0=TMatr.getQQaa(I.psik_at_x0),
	bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQi_feq=TMatr.sumAcB(bQQi,dV,bQQ0),
	bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o)),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	bbasis=M.EV.getEV(n,bQQi_feq,true);
    if(eps>0){
	// Check the values in |psi0> state:
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_feq,n),"<pis0|QQI_feq|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psi0,QQi,n),"<pis0|QQi|psi0>",I.s0,"I.s0",eps);
    }
    double spw=0,sw=0;
    for(int k=0;k<n;k++){
	final double [] bpsi=EVXData.getPsiFromBasis(k,bbasis,n);
	final double w=Math.pow(TMatr.mulVV(bpsi0,bpsi),2),i=TMatr.getAMatrA(bpsi,bQQi,n);
	sw+=w*dI;
	spw-=TMatr.getAMatrA(bpsi,bQQpi,n)-p_last*i;
    }
    if(eps>0){
	// Check the values in |psi0> state:
	UnitTests.checkDoubleRel(sw,"sw",dI,"dI",eps);
    }
    PfmPl=spw/sw;
    p_IH=bQQpi[(n-1)+n*(n-1)]/bQQi[(n-1)+n*(n-1)];
    if(eps>0){
	System.err.println("IfStates: test OK n="+n+" eps="+eps);
    }
}

}
