package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.realtimebook.*;

/** Double integral with Kendall-like pair estimator.
 *  @version $Id: IkQVM.java,v 1.77 2022/07/28 22:30:49 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class IkQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;

final EVXData I,P1t;
/*final*/ GGPData g,gP,giPL,giPH;
final double [] psiX,psiY;
final double I_P1t_L,I_P1t_H;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;

IkQVM(final int n,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();
    this.M=getABasis();

    final double [][] MM=getPMoments(n);
    final double [] tmom=MM[0],imom=MM[1],Ptmom=MM[2],P2tmom=MM[3],pimom=MM[4];
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{tmom,imom,Ptmom});
    final double [] QQt=QQ[0],QQi=QQ[1],QQpt=QQ[2];

    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,getX0());
    this.P1t=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpt,QQt,access,getX0());

    this.stmom=tmom;
    this.simom=imom;
    this.spimom=pimom;
    this.sPtmom=Ptmom;
    this.sP2tmom=P2tmom;

    final double [][] ppp=getPsiXY();
    this.psiX=ppp[0];
    this.psiY=ppp[1];


    this.I_P1t_L=TMatr.getAMatrA(P1t.psiL.length,P1t.psiL,QQi,n);
    this.I_P1t_H=TMatr.getAMatrA(P1t.psiH.length,P1t.psiH,QQi,n);
    this.g=getGGP(this.psiX,this.psiY);
    this.gP=getGGP(P1t.psiL,P1t.psiH);
    this.giPL=getGGI(P1t.psiL,P1t.psiL);
    this.giPH=getGGI(P1t.psiH,P1t.psiH);

}
final double [] stmom,sPtmom,sP2tmom,simom,spimom;

double [][] getPsiXY()
{
    if(false) return new double [][]{I.psiH,I.psiH};
    if(false) return new double [][]{I.psiL,I.psiL};
    if(false) return getPsiXY_SD();
    if(false) return getPsiXY_SD1();
    return new double [][]{I.psiL,I.psiH};
}

double [][] getPsiXY_SD1()
{
    final double [][] ppp=getPsiXY_SD();
    ppp[1]=ppp[0]; // use the same psi
    return ppp;
}
double [][] getPsiXY_SD()
{
    final int n=I.neq;
    final double [] psi=I.psiL;

    double [] DpsiX=new double [psi.length-1];
    final double x0=getX0();
    if(psi.length==2)
    {
	DpsiX[0]=1;
    }
    else
    {
	final double [][] r=DkQVM.getRoots(psi,M,isEdgeAtRight());
	final double [] Xx=r!=null?r[0]:null;
	//final int indX=DjQVM.getIndX0_NonSortedX(Xx,getX0(),isEdgeAtRight());
	//final double fXx=indX>=0?Xx[indX]:Double.NaN,fXy=(indX>=0&&indX<(Xx.length-1))?Xx[indX+1]:Double.NaN;
	final int indX=(Xx!=null && Xx.length>0)?(Xx.length-1):-1;
	final double fXx=indX>=0?Xx[indX]:Double.NaN;
	DpsiX=M.B.sdiv1(psi,fXx).q;
    }
    final double [] rDpsiX=M.getxP(DpsiX);
    for(int k=0;k<DpsiX.length;k++)
    {
	rDpsiX[k]-=x0*DpsiX[k];
    }
    final double [] QQt=M.B.getQQMatr(n,stmom);
    final double nx=Math.sqrt(TMatr.getAMatrA(rDpsiX.length,rDpsiX,QQt,n));
    for(int k=0;k<rDpsiX.length;k++){rDpsiX[k]/=nx;}
    //System.err.println("rDpsiX(x0)="+M.B.getD0(rDpsiX,getX0()));
    return new double [][]{rDpsiX,I.psiH};
}

double [][] getPMoments(final int n)
{
    final int nmoms=2*n-1;
    final double [] tmom=new double [nmoms],imom=new double [nmoms],
	pimom=new double [nmoms],
	Ptmom=new double [nmoms],P2tmom=new double [nmoms];
    final double [] dp0mom=Ptmom,dp1mom=tmom,dp2mom=imom;
    access.setM(dp0mom,0,access.M_DP,odata,nmoms);
    access.setM(dp1mom,1,access.M_DP,odata,nmoms);
    access.setM(dp2mom,2,access.M_DP,odata,nmoms);
    final double pl_m_poffset=getPLast()-getPOffset();
    Polynomials.shiftPOffsetInPlace(nmoms,dp0mom,dp1mom,dp2mom,null,pl_m_poffset);
    access.setFMoments(Ptmom,dp1mom,nmoms);
    access.setFMoments(P2tmom,dp2mom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    access.setMT(tmom,nmoms);
    return new double [][]{tmom,imom,Ptmom,P2tmom,pimom};
    //return new double [][]{tmom,Ptmom,Ptmom,P2tmom};
}

GGPData getGGP(final double [] psiX,final double [] psiY)
{
    return getGGP(true,psiX,psiY);
}
GGPData getGGPI(final boolean flag2,final double [] psiX,final double [] psiY)
{
   final double [][] mm=getPGMoments();
   if(mm==null) return null;
   return getGGP(flag2,psiX,psiY,stmom,spimom,stmom,mm[0],mm[3]);
}
GGPData getGGP(final boolean flag2,final double [] psiX,final double [] psiY)
{
   final double [][] mm=getPGMoments();
   if(mm==null) return null;
   return getGGP(flag2,psiX,psiY,stmom,sPtmom,sP2tmom,mm[0],mm[2]);
}
GGPData getGGI(final double [] psiX,final double [] psiY)
{
    return getGGI(true,psiX,psiY);
}
GGPData getGGI(final boolean flag2,final double [] psiX,final double [] psiY)
{
   final double [][] mm=getPGMoments();
   if(mm==null) return null;
   return getGGP(flag2,psiX,psiY,stmom,simom,stmom,mm[0],mm[1]);
}

/** A class to store the data components of
\int\psi(x) omega(x) dx \int phi(y) omega(y) dy (p(x)-p(y))
*/
static class GGPData {
final double Pt_x,Pt_y,C_y_gt_x,wC_y_gt_x,C_y_lt_x,wC_y_lt_x,a2_Pt_x_Pt_y,Cdiff,sigma2_Cdiff;
final double Px_y_gt_x,Px_y_lt_x;
    GGPData(final double Pt_x,
	    final double Pt_y,
	    final double C_y_gt_x,
	    final double wC_y_gt_x,
	    final double C_y_lt_x,
	    final double wC_y_lt_x,
	    final double a2_Pt_x_Pt_y,
	    final double Px_y_gt_x,
	    final double Px_y_lt_x)
    {
	this.Pt_x=Pt_x;
	this.Pt_y=Pt_y;
	this.C_y_gt_x=C_y_gt_x;
	this.wC_y_gt_x=wC_y_gt_x;
	this.C_y_lt_x=C_y_lt_x;
	this.wC_y_lt_x=wC_y_lt_x;
	this.a2_Pt_x_Pt_y=a2_Pt_x_Pt_y;
	this.Cdiff=this.C_y_gt_x-this.C_y_lt_x;
	this.sigma2_Cdiff=this.a2_Pt_x_Pt_y-Math.pow(this.Cdiff,2);
	this.Px_y_gt_x=Px_y_gt_x;
	this.Px_y_lt_x=Px_y_lt_x;
    }
} // END OF class GGPData

GGPData getGGP(final boolean flag2,final double [] psiX,final double [] psiY,final double [] tmom,final double [] pmom,final double [] p2mom,final double [] tGmom,final double [] pGmom)
{
    final double [] psiX2=flag2?M.B.mult2Pol(psiX,psiX):psiX,psiY2=flag2?M.B.mult2Pol(psiY,psiY):psiY,
	ggXY=getGGCoefs(psiX2,psiY2),ggYX=getGGCoefs(psiY2,psiX2);
    final double nx=TMatr.mulVVn1(psiX2,tmom),ny=TMatr.mulVVn1(psiY2,tmom);
    final double px=TMatr.mulVVn1(psiX2,pmom),py=TMatr.mulVVn1(psiY2,pmom);
    final double pxiny=TMatr.mulVVn1(ggXY,pGmom),pyinx=TMatr.mulVVn1(ggYX,pGmom),
	wygtx=TMatr.mulVVn1(ggYX,tGmom),wyltx=TMatr.mulVVn1(ggXY,tGmom),
	a2_Pt_x_Pt_y=TMatr.mulVVn1(psiX2,p2mom)+TMatr.mulVVn1(psiY2,p2mom)-2*px*py;
    return new GGPData(px/nx,py/ny,
		       px*ny-(pxiny+pyinx),
		       wygtx,
		       (pxiny+pyinx)-py*nx,
		       wyltx,
		       a2_Pt_x_Pt_y,
		       px*ny-pxiny,
		       pxiny);
}

double [] getGGCoefs(final double [] psi2,final double [] phi2)
{
    return M.B.mult2Pol(psi2,getPsiWIntegrated(phi2));
}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double getX0();
/** Calculates t- and p- moments for second measure.
 */
abstract double [][] getPGMoments();
abstract double [] getPsiWIntegrated(final double [] psi);
abstract double getPLast();
abstract double getPOffset();
abstract boolean isEdgeAtRight();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();


static abstract class IkQVMTest<DTYPE> extends QTest {

abstract IkQVM<DTYPE> getIkQVM();
abstract DkQVM<?> getDkQVM();

IkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}

abstract double getXR(final IkQVM<?> q);
abstract double getXL(final IkQVM<?> q);
abstract double omega(final double x);
abstract double omegaForMoments(final double x);
double getSignCompensationForMoments(){return 1;}


class TestIntegral {
    static final int INTR_YGTX=401,INTR_YLTX=402,INTR_SIGNYX=403,INTR_ALL=404;

    final double [] p,psi,phi;
    final DeprecatedHelperDataDirectAccessWithBasis<?> access;
    final double xL,xR;
    final double abserr=1e-10,relerr=1e-9;
    final int nomax=50000;

    TestIntegral(final double [] p,final IkQVM<DTYPE> q)
    {
	this.access=q.access;
	this.xL=getXL(q);
	this.xR=getXR(q);
	this.p=p;
	this.psi=q.psiX;
	this.phi=q.psiY;
    }
    double x,y;
    int intr_type;
    boolean flag_squarePdiff;
    final com.polytechnik.utils.QUANC8.DoubleFunction
	integrateX=new com.polytechnik.utils.QUANC8.DoubleFunction(){
		public double f(final double ax)
		{
		    x=ax;
		    final com.polytechnik.utils.QUANC8 i=
			com.polytechnik.utils.QUANC8.integrate(integrateY,
							       (intr_type==INTR_YGTX?x:xL),
							       (intr_type==INTR_YLTX?x:xR),
							       abserr,
							       relerr,
							       nomax);
		    if(!(i.flag==0)) throw new RuntimeException("i.flag="+i.flag);
		    return i.integral;
		}
	    },
	integrateY=new com.polytechnik.utils.QUANC8.DoubleFunction(){
		public double f(final double ay)
		{
		    y=ay;
		    final double vpsi=access.fx(psi,x),vphi=access.fx(phi,y);
		    double f=Math.pow(vpsi,2)*omega(x)*Math.pow(vphi,2)*omega(y);
		    if(p!=null)
		    {
			final double px=access.fx(p,x),py=access.fx(p,y);
			double d=px-py;
			if(flag_squarePdiff){d=d*d;}
			f*=d;
		    }
		    if(intr_type==INTR_SIGNYX){return (y>x?f:-f)*getSignCompensationForMoments();}
		    return f;
		}
	    };
    double [] getPMoments(final int nmoms,final boolean flag_omega2)
    {
	final double [] pmoments=new double [nmoms];
	for(int k=0;k<pmoments.length;k++)
	{
	    final double [] Q=new double [k+1];
	    Q[k]=1;
	    final com.polytechnik.utils.QUANC8.DoubleFunction
		integrate=new com.polytechnik.utils.QUANC8.DoubleFunction(){
			public double f(final double x){return
				getSignCompensationForMoments()*
				(p!=null?access.fx(p,x):1)*
				access.fx(Q,x)*
				(flag_omega2?omegaForMoments(x):omega(x));}
		    };
	    final com.polytechnik.utils.QUANC8 i=
		com.polytechnik.utils.QUANC8.integrate(integrate,
						       xL,xR,
						       abserr,
						       relerr,
						       nomax);
	    //System.err.println("i.integral*(2*k+1)="+(i.integral*(2*k+1))+"\tp[k]="+(k<p.length?p[k]:0));
	    if(!(i.flag==0)) throw new RuntimeException("i.flag="+i.flag+" k="+k);
	    pmoments[k]=i.integral;
	}
	return pmoments;
    }
    double getIntegral(final int intr_type){return getIntegral(intr_type,false);}
    double getIntegralP2(){return getIntegral(INTR_ALL,true);}
    double getIntegral(final int intr_type,final boolean flag_squarePdiff)
    {
	this.intr_type=intr_type;
	this.flag_squarePdiff=flag_squarePdiff;
	final com.polytechnik.utils.QUANC8 i=
	    com.polytechnik.utils.QUANC8.integrate(integrateX,
						   xL,xR,
						   abserr,
						   relerr,
						   nomax);
	if(!(i.flag==0)) throw new RuntimeException("i.flag="+i.flag);
	return i.integral;
    }
} // END OF class TestIntegral

double [] getRandomP(final int maxd)
{
    final double [] p=new double [maxd];
    for(int k=p.length;--k>=0;) p[k]=1-2*r.nextDouble();
    return p;
}

class IntegrateF implements com.polytechnik.utils.QUANC8.DoubleFunction {
final double [] f;final DeprecatedHelperDataDirectAccessWithBasis<?> access;
    IntegrateF(final double [] f,final DeprecatedHelperDataDirectAccessWithBasis<?> access){this.f=f;this.access=access;}
    public double f(final double x){return access.fx(f,x)*omega(x);}
} // END OF class IntegrateF

void test_Integral()
{
    final IkQVM<DTYPE> q=getIkQVM();
    final int maxd=4*(neq-1)+1;
    final double [] p=getRandomP(maxd);
    final TestIntegral ti=new TestIntegral(p,q),tiw=new TestIntegral(null,q),ti2=new TestIntegral(q.M.B.mult2Pol(p,p),q);
    UnitTests.checkDoubleRel(q.access.fx(q.getPsiWIntegrated(q.M.B.mult2Pol(q.psiX,q.psiX)),q.getX0()),"int(psiX)",1,"1",eps);
    UnitTests.checkDoubleRel(q.access.fx(q.getPsiWIntegrated(q.M.B.mult2Pol(q.psiY,q.psiY)),q.getX0()),"int(psiY)",1,"1",eps);
    for(int tint=0;tint<10;tint++)
    {
	final double xx=ti.xR+0.11*(r.nextDouble())*(ti.xL-ti.xR);
	for(double [] f:new double [][]{q.psiX,q.psiY,q.I.psiL,q.I.psiH})
	{
	    final double [] f2=q.M.B.mult2Pol(f,f);
	    final com.polytechnik.utils.QUANC8 i=
		com.polytechnik.utils.QUANC8.integrate(new IntegrateF(f2,q.access),ti.xL,xx,ti.abserr,ti.relerr,ti.nomax);
	    if(!(i.flag==0)) throw new RuntimeException("i.flag="+i.flag);
	    final double ii=q.access.fx(q.getPsiWIntegrated(f2),xx);
	    UnitTests.checkDoubleRel(ii*omega(xx),"ii*omega(xx="+xx+")",i.integral*getSignCompensationForMoments(),"i.integral",eps);
	}
    }

    final double [] pmoments=ti.getPMoments(2*neq-1,false),
	p2moments=ti2.getPMoments(2*neq-1,false),
	pmomentsg=ti.getPMoments(4*(neq-1)+1+1,true),
	tmoments=tiw.getPMoments(2*neq-1,false),
	tmomentsg=tiw.getPMoments(4*(neq-1)+1+1,true);
    final double [] qtmoms=q.getPGMoments()[0];
    UnitTests.checkDoubleArrayRel(qtmoms,"qtmoms",java.util.Arrays.copyOf(tmomentsg,qtmoms.length),"tmomentsg",eps);
    UnitTests.checkDoubleArrayRel(q.stmom,"q.stmom",tmoments,"tmoments",eps);

    final GGPData g=q.getGGP(true,ti.psi,ti.phi,tmoments,pmoments,p2moments,tmomentsg,pmomentsg);

    UnitTests.checkDoubleRel(g.C_y_gt_x,"g.C_y_gt_x",ti.getIntegral(ti.INTR_YGTX),"ti.getIntegral(INTR_YGTX)",eps);
    UnitTests.checkDoubleRel(q.g.wC_y_gt_x,"q.g.wC_y_gt_x",tiw.getIntegral(tiw.INTR_YGTX),"tiw.getIntegral(INTR_YGTX)",eps);
    UnitTests.checkDoubleRel(g.C_y_lt_x,"g.C_y_lt_x",ti.getIntegral(ti.INTR_YLTX),"ti.getIntegral(INTR_YLTX)",eps);
    UnitTests.checkDoubleRel(q.g.wC_y_lt_x,"q.g.wC_y_lt_x",tiw.getIntegral(tiw.INTR_YLTX),"tiw.getIntegral(INTR_YLTX)",eps);

    UnitTests.checkDoubleRel(q.g.wC_y_lt_x+q.g.wC_y_gt_x,"q.g.wC_y_lt_x+q.g.wC_y_gt_x",1,"1",eps);
    UnitTests.checkDoubleRel(g.Pt_x-g.Pt_y,"g.Pt_x-g.Pt_y",ti.getIntegral(ti.INTR_ALL),"ti.getIntegral(INTR_ALL)",eps);
    UnitTests.checkDoubleRel(1,"1",tiw.getIntegral(tiw.INTR_ALL),"tiw.getIntegral(INTR_ALL)",eps);
    UnitTests.checkDoubleRel(g.Cdiff,"g.Cdiff",ti.getIntegral(ti.INTR_SIGNYX),"ti.getIntegral(INTR_SIGNYX)",eps);
    UnitTests.checkDoubleRel(g.a2_Pt_x_Pt_y,"g.a2_Pt_x_Pt_y",ti.getIntegralP2(),"ti.getIntegralP2()",eps);

    UnitTests.checkDoubleRel(q.giPL.wC_y_lt_x+q.giPL.wC_y_gt_x,"q.giPL.wC_y_lt_x+q.giPL.wC_y_gt_x",1,"1",eps);
    UnitTests.checkDoubleRel(q.giPL.wC_y_lt_x,"q.giPL.wC_y_lt_x",0.5,"0.5",eps);
    UnitTests.checkDoubleRel(q.giPH.wC_y_lt_x,"q.giPH.wC_y_lt_x",0.5,"0.5",eps);
    System.err.println("q.giPH.Cdiff="+q.giPH.Cdiff);

    System.err.println(name+":IkQVMTest:test_Integral(): test OK neq="+neq+" p.length="+p.length+" eps="+eps);
}
void test_Extra(){}


void test_DkQVM()
{
    final DkQVM<?> s=getDkQVM();
    final IkQVM<DTYPE> q=getIkQVM();

    EVXData.matchEVXData(q.P1t,s.P1t.Px,false,false,eps);

    System.err.println(name+":KkQVMTest:test_DkQVM(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":IkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Integral();
    test_DkQVM();
    test_Extra();

    System.err.println(name+":IkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":IkQVMTest: starting tests; ver=$Id: IkQVM.java,v 1.77 2022/07/28 22:30:49 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":IkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: IkQVM.java,v 1.77 2022/07/28 22:30:49 mal Exp $");
}
} // END OF class IkQVMTest

}// END OF class IkQVM
