package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class IkQVMLaguerre extends IkQVM<QVMDataL> {
final QVMDataL odata2;
IkQVMLaguerre(final int neq,final QVMDataL odata,final QVMDataL odata2){
    super(neq,odata);
    this.odata2=odata2;
    this.g=getGGP(psiX,psiY);
    this.gP=getGGP(P1t.psiL,P1t.psiH);
    this.giPL=getGGI(P1t.psiL,P1t.psiL);
    this.giPH=getGGI(P1t.psiH,P1t.psiH);
}

public static IkQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final IkQVMLaguerre ref)
{
    final int npm=3,nmoms=4*(neq-1)+1;
    final QVMDataL o=QVMDataL.calculateQVMDataL(nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null),
	o2=QVMDataL.calculateQVMDataL(nmoms,
				      npm,
				      currenttime,
				      tau/2,
				      tb,
				      tsize,
				      ref!=null?ref.odata2:null);
    return new IkQVMLaguerre(neq,o,o2);
}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
double getX0(){return 0;}
double [][] getPGMoments(){
    if(odata2==null) return null;
    // the moments have different weight for Laguerre basis.
    final int nmoms=4*(I.neq-1)+1;
    final double [] pmom=new double [nmoms],tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms];
    access.setM(tmom,1,access.M_DP,odata2,nmoms);
    access.setFMoments(pmom,tmom,nmoms);
    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata2,nmoms);
    access.setM(pimom,1,access.M_I,odata2,nmoms);
    final double [] conv=new double [nmoms],prevconv=new double [nmoms-1],
	tmomb=new double [nmoms],pmomb=new double [nmoms],imomb=new double [nmoms],pimomb=new double [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	WIntegratorLaguerre.B.setNewtonBinomialLikeCoefs(k,conv,prevconv,0.5,0);
	imomb[k]=0.5*TMatr.mulVVn(k+1,conv,imom);
	pimomb[k]=0.5*TMatr.mulVVn(k+1,conv,pimom);
	tmomb[k]=0.5*TMatr.mulVVn(k+1,conv,tmom);
	pmomb[k]=0.5*TMatr.mulVVn(k+1,conv,pmom);
    }
    return new double [][]{tmomb,imomb,pmomb,pimomb};
}
double [] getPsiWIntegrated(final double [] psi)
{
    return KkQVMLaguerre.getPsiWIntegrated(psi);
}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getPOffset(){return odata.p_offset;}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
boolean isEdgeAtRight(){return false;}

static class IkQVMLaguerreTest extends IkQVMTest<QVMDataL> {
IkQVMLaguerreTest(final long initrandom)
{
    super("IkQVMLaguerre",new java.util.Random(initrandom),1e-2);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

IkQVMLaguerre getIkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DkQVMLaguerre getDkQVM(){return DkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

double getXL(final IkQVM<?> q){return 16+2*neq;}
double getXR(final IkQVM<?> q){return q.getX0();}
double omega(final double x){return Math.exp(-x);}
double omegaForMoments(final double x){return Math.exp(-2*x);}
double getSignCompensationForMoments(){return -1;}

} // END of class IkQVMLaguerreTest

public static void main(String [] args)
{
    new IkQVMLaguerreTest(16117L).runTests(3);
} // END main



} // END OF class IkQVMLaguerre
