package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class IkQVMLegendreShifted extends IkQVM<QVMDataP> {
IkQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static IkQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final IkQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(4*(neq-1)+2,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new IkQVMLegendreShifted(neq,o);
}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
double getX0(){return 1;}
double [][] getPGMoments(){
    // the moments with the same weight for Legendre basis.
    final int nmoms=4*(I.neq-1)+2;
    final double [] pmom=new double [nmoms],tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms];
    access.setM(tmom,1,access.M_DP,odata,nmoms);
    access.setFMoments(pmom,tmom,nmoms);
    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    return new double [][]{tmom,imom,pmom,pimom};
}
double [] getPsiWIntegrated(final double [] psi)
{
    return KkQVMLegendreShifted.getPsiWIntegrated(psi);
}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getPOffset(){return odata.p_offset;}
OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
boolean isEdgeAtRight(){return true;}

static class IkQVMLegendreShiftedTest extends IkQVMTest<QVMDataP> {
IkQVMLegendreShiftedTest(final long initrandom)
{
    super("IkQVMLegendreShifted",new java.util.Random(initrandom),1e-3);
}
double [] convertToMonomials(final double [] p){return WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
IkQVMLegendreShifted getIkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DkQVMLegendreShifted getDkQVM(){return DkQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}

double getXL(final IkQVM<?> q){return 0;}
double getXR(final IkQVM<?> q){return q.getX0();}
double omega(final double x){return 1;}
double omegaForMoments(final double x){return 1;}


} // END of class IkQVMLegendreShiftedTest

public static void main(String [] args)
{
    new IkQVMLegendreShiftedTest(162147L).runTests(4);
} // END main



} // END OF class IkQVMLegendreShifted
