package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.realtimebook.*;

public class IkQVMMonomials extends IkQVM<QVMData> {
final QVMData odata2;
IkQVMMonomials(final int neq,final QVMData odata,final QVMData odata2){
    super(neq,odata);
    this.odata2=odata2;
    this.g=getGGP(psiX,psiY);
    this.gP=getGGP(P1t.psiL,P1t.psiH);
    this.giPL=getGGI(P1t.psiL,P1t.psiL);
    this.giPH=getGGI(P1t.psiH,P1t.psiH);
}

public static IkQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final IkQVMMonomials ref)
{
    final int nvm=1,npm=3,nmoms=4*(neq-1)+1;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     nmoms,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null),
	o2=QVMData.calculateQVMData(nvm,
				    nmoms,
				    npm,
				    currenttime,
				    tau/2,
				    tb,
				    tsize,
				    ref!=null?ref.odata2:null);
    return new IkQVMMonomials(neq,o,o2);
}
DeprecatedHelperQVMDataDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataDirectAccessWithBasis();}
double getX0(){return 0;}
double [][] getPGMoments(){
    if(odata2==null) return null;
    // the moments have different weight for Monomials basis.
    final int nmoms=4*(I.neq-1)+1;
    final double [] pmom=new double [nmoms],tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms];
    access.setM(tmom,1,access.M_DP,odata2,nmoms);
    access.setFMoments(pmom,tmom,nmoms);
    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata2,nmoms);
    access.setM(pimom,1,access.M_I,odata2,nmoms);
    double s=1;
    for(int k=0;k<nmoms;k++,s/=2)
    {
	imom[k]=0.5*(imom[k]*s);
	pimom[k]=0.5*(pimom[k]*s);
	tmom[k]=0.5*(tmom[k]*s);
	pmom[k]=0.5*(pmom[k]*s);
    }
    return new double [][]{tmom,imom,pmom,pimom};
}
double [] getPsiWIntegrated(final double [] psi)
{
    return KkQVMMonomials.getPsiWIntegrated(psi);
}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getPOffset(){return odata.p_offset;}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis();}
boolean isEdgeAtRight(){return true;}

static class IkQVMMonomialsTest extends IkQVMTest<QVMData> {
IkQVMMonomialsTest(final long initrandom)
{
    super("IkQVMMonomials",new java.util.Random(initrandom),1e-3);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

IkQVMMonomials getIkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
IkQVMLaguerre getIkQVMLaguerre(){return IkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
DkQVMLaguerre getDkQVM(){return DkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

void test_Extra(){
    final IkQVMMonomials q=getIkQVM();
    final IkQVMLaguerre s=getIkQVMLaguerre();
    UnitTests.checkDoubleRel(q.g.Pt_x,"q.g.Pt_x",s.g.Pt_x,"s.g.Pt_x",eps);
    UnitTests.checkDoubleRel(q.g.Pt_y,"q.g.Pt_y",s.g.Pt_y,"s.g.Pt_y",eps);
    UnitTests.checkDoubleRel(q.g.C_y_gt_x,"q.g.C_y_gt_x",s.g.C_y_gt_x,"s.g.C_y_gt_x",eps);
    UnitTests.checkDoubleRel(q.g.wC_y_gt_x,"q.g.wC_y_gt_x",s.g.wC_y_gt_x,"s.g.wC_y_gt_x",eps);
    UnitTests.checkDoubleRel(q.g.C_y_lt_x,"q.g.C_y_lt_x",s.g.C_y_lt_x,"s.g.C_y_lt_x",eps);
    UnitTests.checkDoubleRel(q.g.wC_y_lt_x,"q.g.wC_y_lt_x",s.g.wC_y_lt_x,"s.g.wC_y_lt_x",eps);
    UnitTests.checkDoubleRel(q.g.a2_Pt_x_Pt_y,"q.g.a2_Pt_x_Pt_y",s.g.a2_Pt_x_Pt_y,"s.g.a2_Pt_x_Pt_y",eps);

    UnitTests.checkDoubleRel(q.I_P1t_L,"q.I_P1t_L",s.I_P1t_L,"s.I_P1t_L",eps);
    UnitTests.checkDoubleRel(q.I_P1t_H,"q.I_P1t_H",s.I_P1t_H,"s.I_P1t_H",eps);

    UnitTests.checkDoubleRel(q.gP.wC_y_gt_x,"q.gP.wC_y_gt_x",s.gP.wC_y_gt_x,"s.gP.wC_y_gt_x",eps);
    UnitTests.checkDoubleRel(q.gP.wC_y_lt_x,"q.gP.wC_y_lt_x",s.gP.wC_y_lt_x,"s.gP.wC_y_lt_x",eps);
    UnitTests.checkDoubleRel(q.giPL.wC_y_gt_x,"q.giPL.wC_y_gt_x",s.giPL.wC_y_gt_x,"s.giPL.wC_y_gt_x",eps);
    UnitTests.checkDoubleRel(q.giPL.wC_y_lt_x,"q.giPL.wC_y_lt_x",s.giPL.wC_y_lt_x,"s.giPL.wC_y_lt_x",eps);
    UnitTests.checkDoubleRel(q.giPH.wC_y_gt_x,"q.giPH.wC_y_gt_x",s.giPH.wC_y_gt_x,"s.giPH.wC_y_gt_x",eps);
    UnitTests.checkDoubleRel(q.giPH.wC_y_lt_x,"q.giPH.wC_y_lt_x",s.giPH.wC_y_lt_x,"s.giPH.wC_y_lt_x",eps);



    System.err.println(name+":IkQVMTest:test_Extra(): test OK (Laguerre match) neq="+neq+" eps="+eps);
}

double getXL(final IkQVM<?> q){return -(16+2*neq);}
double getXR(final IkQVM<?> q){return q.getX0();}
double omega(final double x){return Math.exp(x);}
double omegaForMoments(final double x){return Math.exp(2*x);}

} // END of class IkQVMMonomialsTest

public static void main(String [] args)
{
    new IkQVMMonomialsTest(16117L).runTests(3);
} // END main



} // END OF class IkQVMMonomials
