package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: ImpactExample.java,v 1.51 2022/11/19 11:08:57 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactExample {
    public final double I_bc_0,I_bc_IL,I_bc_IH,p_bc_IH,dp_orig_bc_IH;
    public final EVXData edI,eI,eIdp;
    public final double p_dIH,dp_dIH,dp_orig_dIH,p_IH,pt_IH,var_pI_IH,dp_IH,dpt_IH,dp_orig_IH,dp_0,p_0,dp_orig_0,p_offset;
    final double p_vIeq,I_eIdp_L,I_eIdp_H,w_IH_eIdp_L,w_IH_eIdp_H,w_IL_eIdp_L,w_IL_eIdp_H;

    /** For unit test. */
    private final double [] dp_dI,p_dI,dp_orig_dI,var2_dI,dp_I,vpI_I,p_I,pt_I,dpt_I,dp_orig_I,psik_Dpsik,var2_I;
    ImpactExample(final double I_bc_0,final double I_bc_IL,final double I_bc_IH,final double p_bc_IH,final double dp_orig_bc_IH,
		  final EVXData edI,final double [] dp_dI,final double [] p_dI,final double [] dp_orig_dI,final double [] var2_dI,
		  final EVXData eI,final double [] dp_I,final double [] vpI_I,final double [] p_I,
		  final double [] pt_I,final double [] dpt_I,
		  final double [] dp_orig_I,final double [] psik_Dpsik,final double [] var2_I,
		  final double dp_0,final double p_0,final double dp_orig_0,
		  final EVXData eIdp,final double I_eIdp_L,final double I_eIdp_H,final double w_IH_eIdp_L,final double w_IH_eIdp_H,final double w_IL_eIdp_L,final double w_IL_eIdp_H,
		  final double p_offset)
{
    this.p_offset=p_offset;
    this.I_bc_0=I_bc_0;
    this.I_bc_IL=I_bc_IL;
    this.I_bc_IH=I_bc_IH;
    this.p_bc_IH=p_bc_IH;
    this.dp_orig_bc_IH=dp_orig_bc_IH;
    this.edI=edI;
    this.dp_dI=dp_dI;
    this.p_dI=p_dI;
    this.dp_orig_dI=dp_orig_dI;
    this.var2_dI=var2_dI;
    this.eI=eI;
    this.dp_I=dp_I;
    this.vpI_I=vpI_I;
    this.p_I=p_I;
    this.pt_I=pt_I;
    this.dpt_I=dpt_I;
    this.dp_orig_I=dp_orig_I;
    this.psik_Dpsik=psik_Dpsik;
    this.var2_I=var2_I;
    final int n=eI.neq;
    this.eIdp=eIdp;
    this.p_vIeq=TMatr.sumElements(vpI_I)/TMatr.mulV2V(eI.psik_at_x0,eI.getEValues());

    this.I_eIdp_L=I_eIdp_L;
    this.I_eIdp_H=I_eIdp_H;
    this.w_IH_eIdp_L=w_IH_eIdp_L;
    this.w_IH_eIdp_H=w_IH_eIdp_H;
    this.w_IL_eIdp_L=w_IL_eIdp_L;
    this.w_IL_eIdp_H=w_IL_eIdp_H;

    this.p_dIH=p_dI[n-1];
    this.dp_dIH=dp_dI[n-1];
    this.dp_orig_dIH=dp_orig_dI[n-1];
    this.p_IH=p_I[n-1];
    this.pt_IH=pt_I[n-1];
    this.dp_IH=dp_I[n-1];
    this.dpt_IH=dpt_I[n-1];
    this.dp_orig_IH=dp_orig_I[n-1];
    this.var_pI_IH=-dp_IH*eI.sH;
    this.p_0=p_0;
    this.dp_0=dp_0;
    this.dp_orig_0=dp_orig_0;
    double dp_orig_eIplus=0,dp_orig_eIminus=0,dp_orig_eI_v2I=0,dp_orig_eI_A=0,dp_orig_eI_Aa=0;
    for(int k=0;k<n;k++)
    {
	if(var2_I[k]>0){dp_orig_eIplus+=dp_orig_I[k]*eI.ev[k];}
	else{dp_orig_eIminus+=dp_orig_I[k]*eI.ev[k];}
	dp_orig_eI_v2I+=dp_orig_I[k]*var2_I[k];
	dp_orig_eI_A+=dp_orig_I[k]*(eI.ev[k]-eI.sL);
	dp_orig_eI_Aa+=dp_orig_I[k]*eI.ev[k];
    }
    this.dp_orig_eIplus=dp_orig_eIplus;
    this.dp_orig_eIminus=dp_orig_eIminus;
    this.dp_orig_eI_v2I=dp_orig_eI_v2I;
    this.dp_orig_eI_A=dp_orig_eI_A;
    this.dp_orig_eI_Aa=dp_orig_eI_Aa;
}
    final double dp_orig_eIplus,dp_orig_eIminus,dp_orig_eI_v2I,dp_orig_eI_A,dp_orig_eI_Aa;


/**
 * Calculate dp/dt on various states.
 */
public static ImpactExample get_ImpactExample(final int n,
					      final double [] QQi,final double [] QQt,final double [] QQpi,final double [] QQdp,
					      final double [] QQpt,
					      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,
					      final double p_offset,
					      final double eps)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    // answer with psi(x0)=0 boundary condition
    final double [] bc_basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
    final double p_bc_IH=TMatr.getAMatrA(n,bc_basis,n+(n-1)*n,QQpi,n)/bc_basis[n-1],
	dp_orig_bc_IH=TMatr.getAMatrA(n,bc_basis,n+(n-1)*n,QQdp,n);

    final EVXData eI=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] dp_I=new double [n],p_I=new double [n],vpI_I=new double [n],pt_I=new double [n],dpt_I=new double [n],dp_orig_I=new double [n],psik_Dpsik=new double [n],var2_I=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=eI.getPsi(k),Dpsik=VOLATILITY_EDPsi.getDPsi(psik);
	final double pIk=TMatr.getAMatrA(psik,QQpi,n),Ik=eI.ev[k],psik2x0=2*TMatr.getAMatrB(psik,Dpsik,QQt,n),vpIk=2*TMatr.getAMatrB(psik,Dpsik,QQpi,n),pk=TMatr.getAMatrA(psik,QQpt,n),vpk=2*TMatr.getAMatrB(psik,Dpsik,QQpt,n);
	p_I[k]=pIk/Ik;
	pt_I[k]=pk/1;
	dp_I[k]=-(vpIk/Ik-p_I[k]*psik2x0);
	dpt_I[k]=-(vpk-pt_I[k]*psik2x0);
	vpI_I[k]=vpIk;
	dp_orig_I[k]=TMatr.getAMatrA(psik,QQdp,n);
	psik_Dpsik[k]=2*TMatr.getAMatrB(psik,Dpsik,QQt,n);
	var2_I[k]=2*(TMatr.getAMatrA(Dpsik,QQi,n)-eI.ev[k]*TMatr.getAMatrA(Dpsik,QQt,n));
	if(eps>0)
	{
	    final double [] rq=RayleighQuotient.getRQVariationValues(psik,Dpsik,null,QQpi,QQi,n);
	    UnitTests.checkDoubleRel(p_I[k],"p_I[k]",rq[0],"rq[0]",eps);
	    UnitTests.checkDoubleRel(dp_I[k],"dp_I[k]",-rq[1],"-rq[1]",eps);
	    final double [] rqt=RayleighQuotient.getRQVariationValues(psik,Dpsik,null,QQpt,QQt,n);
	    UnitTests.checkDoubleRel(pt_I[k],"pt_I[k]",rqt[0],"rqt[0]",eps);
	    UnitTests.checkDoubleRel(dpt_I[k],"dpt_I[k]",-rqt[1],"-rqt[1]",eps);
	    UnitTests.checkDoubleRel(psik2x0,"psik2x0",Math.pow(eI.psik_at_x0[k],2),"(eI.psik_at_x0[k])^2",eps);
	    UnitTests.checkDoubleRel(psik2x0,"psik2x0",Math.pow(M.B.getD0(psik,eI.x0),2),"psik(x0)^2",eps);
	    UnitTests.checkDoubleRel(psik2x0*eI.ev[k],"psik2x0*eI.ev[k]",2*TMatr.getAMatrB(psik,Dpsik,QQi,n),"2<psik|I|Dpsik>",eps);
	    final double [] rqe=RayleighQuotient.getRQVariationValues(psik,Dpsik,null,QQi,QQt,n);
	    UnitTests.checkDoubleRel(eI.ev[k],"eI.ev[k]",rqe[0],"rqe[0]",eps);
	    UnitTests.checkDoubleRel(0,"0",rqe[1],"rqe[1]",eps);
	    UnitTests.checkDoubleRel(var2_I[k],"var2_I[k]",rqe[2],"rqe[2]",eps);
	    System.err.println("ImpactExample.get_ImpactExample: test eI OK n="+n+" k="+k+" eps="+eps);
	}
    }

    // Assume I^{*}=0 boundary condition.
    final double [] QQDi=TMatr.scaleVectorInPlace(VOLATILITY_EDPsi.getDiffMatrix(n,QQi),-1);
    final EVXData edI=EVXData.getEVXDataFromQQ(n,QQDi,QQt,M);
    final double [] dp_dI=new double [n],p_dI=new double [n],dp_orig_dI=new double [n],var2_dI=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=edI.getPsi(k),Dpsik=VOLATILITY_EDPsi.getDPsi(psik);
	final double pIk=TMatr.getAMatrA(psik,QQpi,n),Ik=TMatr.getAMatrA(psik,QQi,n);
	p_dI[k]=pIk/Ik;
	dp_dI[k]=(-2*TMatr.getAMatrB(psik,Dpsik,QQpi,n)-p_dI[k]*edI.ev[k])/Ik;
	dp_orig_dI[k]=TMatr.getAMatrA(psik,QQdp,n);
	var2_dI[k]=2*(TMatr.getAMatrA(Dpsik,QQDi,n)-edI.ev[k]*TMatr.getAMatrA(Dpsik,QQt,n));
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(eI.getPsi(k),QQDi,n),"<psiIk|dI|psiIk>",-eI.ev[k]*Math.pow(M.B.getD0(eI.getPsi(k),eI.x0),2),"-eI.ev[k]*psiIk(x0)^2",eps);
	    final double psik2x0=2*TMatr.getAMatrB(psik,Dpsik,QQt,n);
	    final double [] rq=RayleighQuotient.getRQVariationValues(psik,Dpsik,null,QQpi,QQi,n);
	    UnitTests.checkDoubleRel(p_dI[k],"p_dI[k]",rq[0],"rq[0]",eps);
	    UnitTests.checkDoubleRel(dp_dI[k],"dp_dI[k]",-rq[1],"-rq[1]",eps);
	    UnitTests.checkDoubleRel(psik2x0,"psik2x0",Math.pow(M.B.getD0(psik,edI.x0),2),"psik(x0)^2",eps);
	    UnitTests.checkDoubleRel(edI.ev[k],"edI.ev[k]",-2*TMatr.getAMatrB(psik,Dpsik,QQi,n),"-2<psik|I|Dpsik>",eps);
	    final double [] rqe=RayleighQuotient.getRQVariationValues(psik,Dpsik,null,QQDi,QQt,n);
	    UnitTests.checkDoubleRel(edI.ev[k],"edI.ev[k]",rqe[0],"rqe[0]",eps);
	    UnitTests.checkDoubleRel(0,"0",rqe[1],"rqe[1]",eps);
	    UnitTests.checkDoubleRel(var2_dI[k],"var2_dI[k]",rqe[2],"rqe[2]",eps);
	    System.err.println("ImpactExample.get_ImpactExample: test edI OK n="+n+" k="+k+" eps="+eps);
	}
    }

    final double [] psi0=EVXData.getPsiFromBasis(0,bc_basis,n),Dpsi0=VOLATILITY_EDPsi.getDPsi(psi0);
    final double p_0=TMatr.getAMatrA(psi0,QQpi,n)/bc_basis[0],
	dp_0=-2*(TMatr.getAMatrB(psi0,Dpsi0,QQpi,n)-p_0*TMatr.getAMatrB(psi0,Dpsi0,QQi,n))/bc_basis[0],
	dp_orig_0=TMatr.getAMatrA(psi0,QQdp,n);

    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	QQidp=TMatr.mulMatrices(n,new double [][]{QQdp,IQQt,QQi});
    TMatr.selectHermitianPartInPlace(n,QQidp);
    final EVXData eIdp=EVXData.getEVXDataFromQQ(n,QQidp,QQt,M);

    if(eps>0)
    {
	final double [] rq=RayleighQuotient.getRQVariationValues(psi0,Dpsi0,null,QQpi,QQi,n);
	UnitTests.checkDoubleRel(p_0,"p_0",rq[0],"rq[0]",eps);
	UnitTests.checkDoubleRel(dp_0,"dp_0",-rq[1],"-rq[1]",eps);
	UnitTests.checkDoubleRel(eI.o_o,"eI.o_o",Math.pow(M.B.getD0(psi0,eI.x0),2),"psi0(x0)^2",eps);
	UnitTests.checkDoubleRel(bc_basis[0],"bc_basis[0]",eI.s0,"eI.s0",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(eI.psik_at_x0,eI.getEValues())/TMatr.mulVV(eI.psik_at_x0,eI.psik_at_x0),"sum(I*psi(x0)^2)/sum(si(x0)^2)",eI.s0,"eI.s0",eps);

	// The spur must be invariant.
	final double [] b=TMatr.mulMatrices(n,new double [][]{IQQt,QQdp,IQQt,QQi});
	final double sp=TMatr.getSpurM(n,b);
	UnitTests.checkDoubleRel(sp,"sp",TMatr.mulVVn1(dp_orig_I,eI.ev),"sp(dp_orig_I,eI.ev)",eps);

	UnitTests.checkDoubleArrayRel(TMatr.scaleVectorInPlace(DeprecatedHelperTMatr.sumAB(QQidp.length,TMatr.mulMatrices(n,new double [][]{QQdp,IQQt,QQi}),TMatr.mulMatrices(n,new double [][]{QQi,IQQt,QQdp})),0.5),"(QQi IQQt QQdp + QQdp IQQt QQi)/2",QQidp,"QQidp",eps);
	// check spur invariance
	UnitTests.checkDoubleRel(TMatr.mulVVn1(dp_orig_I,eI.ev),"sp dp_orig_I,eI.ev",TMatr.sumElements(eIdp.getEValues()),"sum(eIdp.ev)",eps);

	System.err.println("ImpactExample.get_ImpactExample: all test OK n="+n+" eps="+eps);
    }

    return new ImpactExample(bc_basis[0],bc_basis[1],bc_basis[n-1],p_bc_IH,dp_orig_bc_IH,
			     edI,dp_dI,p_dI,dp_orig_dI,var2_dI,
			     eI,dp_I,vpI_I,p_I,pt_I,dpt_I,dp_orig_I,psik_Dpsik,var2_I,
			     dp_0,p_0,dp_orig_0,
			     eIdp,eIdp.getFPsi(0,QQi),eIdp.getFPsi(n-1,QQi),TMatr.getAMatrB(eIdp.psiL,eI.psiH,QQt,n),TMatr.getAMatrB(eIdp.psiH,eI.psiH,QQt,n),TMatr.getAMatrB(eIdp.psiL,eI.psiL,QQt,n),TMatr.getAMatrB(eIdp.psiH,eI.psiL,QQt,n),
			     p_offset);
}

static void matchDynVar2DynImpactDI(final ImpactExample a,final DynVar2DynImpactDI b, final double eps)
{
    final int n=a.eI.neq;
    UnitTests.checkDoubleRel(a.I_bc_0,"a.I_bc_0",b.I_0,"b.I_0",eps);
    UnitTests.checkDoubleRel(a.I_bc_IL,"a.I_bc_IL",b.I_IL,"b.I_IL",eps);
    UnitTests.checkDoubleRel(a.I_bc_IH,"a.I_bc_IH",b.I_IH,"b.I_IH",eps);
    UnitTests.checkDoubleRel(a.p_bc_IH,"a.p_bc_IH",b.p_IH,"b.p_IH",eps);

    UnitTests.checkDoubleArrayRel(a.edI.getEValues(),"a.edI",b.dI,"b.dI",eps);
    UnitTests.checkDoubleArrayRel(a.dp_dI,"a.dp_dI",b.dp,"b.dp",eps);
    UnitTests.checkDoubleArrayRel(a.p_dI,"a.p_dI",b.p,"b.p",eps);
    UnitTests.checkDoubleRel(Math.pow(a.edI.wL,2)*a.edI.o_o,"psiL(x0)^2",b.psi2x0[0],"b.psi2x0[0]",eps);
    UnitTests.checkDoubleRel(Math.pow(a.edI.wH,2)*a.edI.o_o,"psiH(x0)^2",b.psi2x0[n-1],"b.psi2x0[n-1]",eps);

    System.err.println("ImpactExample.matchDynVar2DynImpactDI: test OK n="+n+" eps="+eps);
}

static void matchDynVar2DiagProj(final ImpactExample a,final DynVar2DiagProj b, final double eps)
{
    final int n=a.eI.neq;
    EVXData.matchEVXData(a.eI,b.I,eps);
    UnitTests.checkDoubleArrayRel(a.dp_I,"a.dp_I",b.dp_I,"b.dp_I",eps);
    UnitTests.checkDoubleArrayRel(a.p_I,"a.p_I",b.p_I,"b.p_I",eps);
    UnitTests.checkDoubleArrayRel(a.var2_I,"a.var2_I",b.var2_I,"b.var2_I",eps);
    UnitTests.checkDoubleRel(a.dp_IH,"a.dp_IH",-b.var_pI_IH/b.I.sH,"-b.var_pI_IH/b.I.sH",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.var_pI_IH,"a.var_pI_IH",b.var_pI_IH,"b.var_pI_IH",eps);
    UnitTests.checkDoubleRel(Math.pow(a.eI.wL,2)*a.eI.o_o,"psiL(x0)^2",b.psi2k_x0[0],"b.psi2k_x0[0]",eps);
    UnitTests.checkDoubleRel(Math.pow(a.eI.wH,2)*a.eI.o_o,"psiH(x0)^2",b.psi2k_x0[n-1],"b.psi2k_x0[n-1]",eps);
    System.err.println("ImpactExample.matchDynVar2DiagProj: test OK n="+n+" eps="+eps);
}

static void matchImpactExample(final ImpactExample a,final ImpactExample b, final double eps)
{
    final int n=a.eI.neq;
    final double p_offset_shift=b.p_offset-a.p_offset;

    UnitTests.checkDoubleRel(a.I_bc_0,"a.I_bc_0",b.I_bc_0,"b.I_bc_0",eps);
    UnitTests.checkDoubleRel(a.I_bc_IL,"a.I_bc_IL",b.I_bc_IL,"b.I_bc_IL",eps);
    UnitTests.checkDoubleRel(a.I_bc_IH,"a.I_bc_IH",b.I_bc_IH,"b.I_bc_IH",eps);
    UnitTests.checkDoubleRel(a.dp_orig_bc_IH,"a.dp_orig_bc_IH",b.dp_orig_bc_IH,"b.dp_orig_bc_IH",eps);
    UnitTests.checkDoubleRel(a.p_bc_IH,"a.p_bc_IH",b.p_bc_IH+p_offset_shift,"b.p_bc_IH+p_offset_shift",eps);

    UnitTests.checkDoubleRel(a.dp_0,"a.dp_0",b.dp_0,"b.dp_0",eps);
    UnitTests.checkDoubleRel(a.dp_orig_0,"a.dp_orig_0",b.dp_orig_0,"b.dp_orig_0",eps);
    UnitTests.checkDoubleRel(a.p_0,"a.p_0",b.p_0+p_offset_shift,"b.p_0+p_offset_shift",eps);

    final double [] o_shift=new double [n];
    java.util.Arrays.fill(o_shift,p_offset_shift);

    EVXData.matchEVXData(a.edI,b.edI,false,eps);
    UnitTests.checkDoubleArrayRel(a.var2_dI,"a.var2_dI",b.var2_dI,"b.var2_dI",eps);
    UnitTests.checkDoubleArrayRel(a.dp_dI,"a.dp_dI",b.dp_dI,"b.dp_dI",eps);
    UnitTests.checkDoubleArrayRel(a.dp_orig_dI,"a.dp_orig_dI",b.dp_orig_dI,"b.dp_orig_dI",eps);
    UnitTests.checkDoubleArrayRel(a.p_dI,"a.p_dI",DeprecatedHelperTMatr.sumAB(n,b.p_dI,o_shift),"b.p_dI+p_offset_shift",eps);

    EVXData.matchEVXData(a.eI,b.eI,false,eps);
    UnitTests.checkDoubleArrayRel(a.psik_Dpsik,"a.psik_Dpsik",b.psik_Dpsik,"b.psik_Dpsik",eps);
    UnitTests.checkDoubleArrayRel(a.var2_I,"a.var2_I",b.var2_I,"b.var2_I",eps);
    UnitTests.checkDoubleArrayRel(a.dp_I,"a.dp_I",b.dp_I,"b.dp_I",eps);
    UnitTests.checkDoubleArrayRel(a.dp_orig_I,"a.dp_orig_I",b.dp_orig_I,"b.dp_orig_I",eps);
    UnitTests.checkDoubleArrayRel(a.p_I,"a.p_I",DeprecatedHelperTMatr.sumAB(n,b.p_I,o_shift),"b.p_I+p_offset_shift",eps);
    UnitTests.checkDoubleArrayRel(a.pt_I,"a.pt_I",DeprecatedHelperTMatr.sumAB(n,b.pt_I,o_shift),"b.pt_I+p_offset_shift",eps);
    UnitTests.checkDoubleArrayRel(a.dpt_I,"a.dpt_I",b.dpt_I,"b.dpt_I",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dp_IH,"a.dp_IH",b.dp_IH,"b.dp_IH",eps);
    UnitTests.checkDoubleRel(a.pt_IH,"a.pt_IH",b.pt_IH+p_offset_shift,"b.pt_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dpt_IH,"a.dpt_IH",b.dpt_IH,"b.dpt_IH",eps);
    UnitTests.checkDoubleRel(a.var_pI_IH,"a.var_pI_IH",b.var_pI_IH,"b.var_pI_IH",eps);


    EVXData.matchEVXData(a.eIdp,b.eIdp,false,eps);

    System.err.println("ImpactExample.matchImpactExample: test OK n="+n+" eps="+eps);
}

static void matchImpactIStatesExample(final ImpactExample a,final ImpactIStatesExample b, final double eps)
{
    final int n=a.eI.neq;

    final double p_offset_shift=b.p_offset-a.p_offset;
    final double [] o_shift=new double [n];
    java.util.Arrays.fill(o_shift,p_offset_shift);

    EVXData.matchEVXData(a.eI,b.eI,false,eps);
    UnitTests.checkDoubleArrayRel(a.eIdp.getEValues(),"a.eIdp",java.util.Arrays.copyOf(b.eIdp,n),"b.b.eIdp",eps);
    UnitTests.checkDoubleRel(a.I_eIdp_L,"a.I_eIdp_L",b.I_eIdp_L,"b.I_eIdp_L",eps);
    UnitTests.checkDoubleRel(a.I_eIdp_H,"a.I_eIdp_H",b.I_eIdp_H,"b.I_eIdp_H",eps);
    UnitTests.checkDoubleRel(Math.abs(a.w_IH_eIdp_L),"a.w_IH_eIdp_L",Math.abs(b.w_IH_eIdp_L),"b.w_IH_eIdp_L",eps);
    UnitTests.checkDoubleRel(Math.abs(a.w_IH_eIdp_H),"a.w_IH_eIdp_H",Math.abs(b.w_IH_eIdp_H),"b.w_IH_eIdp_H",eps);
    UnitTests.checkDoubleRel(Math.abs(a.w_IL_eIdp_L),"a.w_IL_eIdp_L",Math.abs(b.w_IL_eIdp_L),"b.w_IL_eIdp_L",eps);
    UnitTests.checkDoubleRel(Math.abs(a.w_IL_eIdp_H),"a.w_IL_eIdp_H",Math.abs(b.w_IL_eIdp_H),"b.w_IL_eIdp_H",eps);

    UnitTests.checkDoubleArrayRel(a.psik_Dpsik,"a.psik_Dpsik",b.psik_Dpsik,"b.psik_Dpsik",eps);
    UnitTests.checkDoubleArrayRel(a.var2_I,"a.var2_I",b.var2_I,"b.var2_I",eps);
    UnitTests.checkDoubleArrayRel(a.dp_orig_I,"a.dp_orig_I",b.dp_orig_I,"b.dp_orig_I",eps);
    UnitTests.checkDoubleArrayRel(a.p_I,"a.p_I",DeprecatedHelperTMatr.sumAB(n,b.p_I,o_shift),"b.p_I+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dp_orig_eI_A,"a.dp_orig_eI_A",b.dp_orig_eI_A,"b.dp_orig_eI_A",eps);
    UnitTests.checkDoubleRel(a.dp_orig_eI_Aa,"a.dp_orig_eI_Aa",b.dp_orig_eI_Aa,"b.dp_orig_eI_Aa",eps);
    UnitTests.checkDoubleRel(a.dp_orig_eI_v2I,"a.dp_orig_eI_v2I",b.dp_orig_eI_v2I,"b.dp_orig_eI_v2I",eps);

    System.err.println("ImpactExample.matchImpactIStatesExample: test OK n="+n+" eps="+eps);
}

static void matchPnLdIdP(final ImpactExample a,final PnLdIdP b, final double eps)
{
    final int n=a.eI.neq;
    final double p_offset_shift=b.p_offset-a.p_offset;
    EVXData.matchEVXData(a.eI,b.I,false,eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dp_orig_IH,"a.dp_orig_IH",b.dp_IH,"b.dp_IH",eps);
    UnitTests.checkDoubleRel(a.dp_orig_0,"a.dp_orig_0",b.dp_0,"b.dp_0",eps);

    System.err.println("ImpactExample.matchPnLdIdP: test OK n="+n+" eps="+eps);
}

static void matchPnLdID(final ImpactExample a,final PnLdID b, final double eps)
{
    final int n=a.eI.neq;
    final double p_offset_shift=b.p_offset-a.p_offset;
    EVXData.matchEVXData(a.eI,b.I,false,eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.var_pI_IH,"a.var_pI_IH",b.var_pI_IH,"b.var_pI_IH",eps);
    UnitTests.checkDoubleRel(a.dp_IH,"a.dp_IH",b.dp_IH,"b.dp_IH",eps);
    UnitTests.checkDoubleRel(a.dp_orig_IH,"a.dp_orig_IH",b.dp_orig_IH,"b.dp_orig_IH",eps);
    UnitTests.checkDoubleRel(a.dp_orig_0,"a.dp_orig_0",b.dp_orig_0,"b.dp_orig_0",eps);

    System.err.println("ImpactExample.matchPnLdID: test OK n="+n+" eps="+eps);
}

}
