package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 * Impact from the future asymmetry base class.
 *  @version $Id: ImpactFromTheFuture2.java,v 1.35 2022/08/07 15:44:02 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactFromTheFuture2 {
/** I=dV/dt eigenfunctions and eigenvectors.
 */
public final EVXData I;
public final double p_offset,p_last__with_p_offset_included,p_IH,pt_IH,I_0_feq,dI;
/** Past moments on future wavefunctions.
 */
public final double [] psik_psi0,psik_I_psi0,psik_pI_psi0,psik_I_psik,psik_pI_psik,psik_p2I_psik,psik_t_psik,psik_pt_psik,psik_p2t_psik;
/** For unit tests and possible future use.
 */
public final double [] bbasis,bpsi0;
/** For unit tests. To match PnLdIDSk.{deg_from_dyn,Pf_from_f0}.
 */
public final double deg_from_dyn,Pf_from_dyn;
public ImpactFromTheFuture2(final IMomentsData mm,
			    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			    final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    this.I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final int ixH=n-1; // the state with maximal I
    final double I_0=I.s0;
    I_0_feq=I.ev[ixH];
    dI=I_0_feq-I_0;
    p_last__with_p_offset_included=mm.p_last;
    p_offset=mm.p_offset;
    p_IH=I.getFPsi(ixH,mm.QQpi)/I.ev[ixH];
    pt_IH=I.getFPsi(ixH,mm.QQpt_sample)/I.getFPsi(ixH,mm.QQt_sample);

    bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));
    // Create bQQi_feq (operator of "future" I), with the |bpsi0>dI<bpsi0| contribution from the future:
    // bQQi_feq = bQQi + |bpsi0>dI<bpsi0|
    final double [] bQQi_feq=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o));
    for(int k=0;k<n;k++) bQQi_feq[k+n*k]+=I.ev[k];

    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp2i,I.ev),
	_bQQt=EVXData.convertMatrFromQtoB_Basis(n,mm.QQt_sample,I.ev),
	_bQQpt=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpt_sample,I.ev),
	_bQQp2t=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp2t_sample,I.ev);

    psik_psi0=new double [n];
    psik_I_psi0=new double [n];
    psik_pI_psi0=new double [n];
    psik_I_psik=new double [n];
    psik_pI_psik=new double [n];
    psik_p2I_psik=new double [n];
    psik_t_psik=new double [n];
    psik_pt_psik=new double [n];
    psik_p2t_psik=new double [n];
    // Diagonalize "future" I:
    bbasis=M.EV.getEV(n,bQQi_feq,true);
    for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n);

	psik_psi0[k]=TMatr.mulVV(psik,bpsi0);
	psik_I_psi0[k]=TMatr.mulVVV(psik,bpsi0,I.getEValues());
	psik_pI_psi0[k]=TMatr.getAMatrB(n,bpsi0,0,bbasis,n+k*n,bQQpi,n);
	psik_I_psik[k]=TMatr.mulV2V(psik,I.getEValues());
	psik_pI_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n);
	psik_p2I_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQp2i,n);
	psik_t_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,_bQQt,n);
	psik_pt_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,_bQQpt,n);
	psik_p2t_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,_bQQp2t,n);
	if(eps>0)
	{
	    final double di=dI*Math.pow(psik_psi0[k],2);
	    UnitTests.checkDoubleRel(psik_I_psik[k]+di,"psik_I_psik[k]+di",bbasis[k],"bbasis[k]",eps);
	}
    }
    deg_from_dyn=1-TMatr.sumElements(TMatr.squareArrayElements(TMatr.squareArrayElements(psik_psi0)));
    Pf_from_dyn=-(TMatr.getAMatrA(bpsi0,bQQpi,n)-TMatr.mulV2V(psik_psi0,psik_pI_psik))/(dI*deg_from_dyn);
    if(eps>0)
    {
	EVXData.test_orthogonality("ImpactFromTheFuture2:I.ev",n,I.ev,mm.QQi,mm.QQt,eps); // old basis
	EVXData.test_orthogonality("ImpactFromTheFuture2:bbasis",n,bbasis,bQQi_feq,TMatr.getUnitMatrix(n),eps); // new basis
	// check some matrix elements
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(psik_psi0,java.util.Arrays.copyOf(bbasis,n)),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi0,bpsi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(bpsi0,I.getEValues()),"<psi0|I|psi0>",I.s0,"I.s0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_feq,n),"<psi0|I^f|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_pI_psi0),"<psi0|pI|psi0>",TMatr.getAMatrA(I.psi0,mm.QQpi,n),"<psi0|pI|psi0>",eps);

	// check spur invariance
	final double [] bQQi=TMatr.getDiagMatrix(I.getEValues()),IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,mm.QQt);
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues()),"sum(I.ev)",TMatr.sumElements(psik_I_psik),"sum(psik_I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues())+dI,"sum(I.ev)+dI",TMatr.sumElements(java.util.Arrays.copyOf(bbasis,n)),"sum(I^{f})",eps);
	UnitTests.checkDoubleRel(-(TMatr.mulVV(psik_psi0,psik_I_psi0)-TMatr.mulV2V(psik_psi0,psik_I_psik)),"dI*deg",dI*deg_from_dyn,"dI*deg_from_dyn",eps);
	for(double [][] x: new double [][][]{ // triples to test
		{bQQi,psik_I_psik,mm.QQi},
		{bQQpi,psik_pI_psik,mm.QQpi},
		{bQQp2i,psik_p2I_psik,mm.QQp2i},
		{_bQQt,psik_t_psik,mm.QQt_sample},
		{_bQQpt,psik_pt_psik,mm.QQpt_sample},
		{_bQQp2t,psik_p2t_psik,mm.QQp2t_sample}
	    })
	{
	    final double [] bQQf=x[0],psik_f_psik=x[1],QQf=x[2];
	    UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQf),"Spur(bQQf)",TMatr.sumElements(psik_f_psik),"sum(psik_f_psik)",eps);
	    UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,QQf),"Spur(QQf)",TMatr.sumElements(psik_f_psik),"sum(psik_f_psik)",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psi0,QQf,n),"<psi0|f|psi0>",TMatr.getAMatrA(bpsi0,bQQf,n),"<psi0|f|psi0>",eps);
	    final double [] bbQQf=EVXData.convertMatrFromQtoB_Basis(n,bQQf,bbasis);
	    UnitTests.checkDoubleRel(TMatr.getSpurM(n,bbQQf),"Spur(bbQQf)",TMatr.sumElements(psik_f_psik),"sum(psik_f_psik)",eps);
	    for(int j=0;j<n;j++) UnitTests.checkDoubleRel(bbQQf[j+n*j],"bbQQf[j,j]",psik_f_psik[j],"psik_f_psik[j]",eps);
	}
	System.err.println("ImpactFromTheFuture2 selftest OK n="+n+" eps="+eps);
    }
}

}
