package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 * Impact from the future asymmetry in price basis
 *  @version $Id: ImpactFromTheFuture_Ci.java,v 1.19 2020/09/27 20:33:15 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactFromTheFuture_Ci extends ImpactFromTheFuture2 {
public final double dir_scaled,var_f_IH,var_pmu_IH,var_pmu_IHa,var_pmu_IHm,var_pmu_IHf,var_pmu_IHfa,var_pmu_IHfm;
    final double var_pmu_IH_qsk,var_pmu_IHf_qsk;
final ImpactFuture2Asymmetry [] iaf=new ImpactFuture2Asymmetry [I.neq];
final Mu2 [] mu2=new Mu2 [I.neq];
ImpactFromTheFuture_Ci(final IMomentsData mm,
		       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		       final double eps)
{
    super(mm,M,eps);
    final int n=I.neq,ixH=n-1;
    double bdiff=0,bdiff_norm=0;
    for(int k=0;k<n;k++)
    {
	final double nf=1/psik_t_psik[k]; // sample and exact give slightly different normalizing
	iaf[k]=new ImpactFuture2Asymmetry(
	    psik_t_psik[k]*nf,psik_pt_psik[k]*nf,psik_p2t_psik[k]*nf,
	    psik_I_psik[k],psik_pI_psik[k],psik_p2I_psik[k],
	    dI*Math.pow(psik_psi0[k],2),p_last__with_p_offset_included,eps);
	mu2[k]=new Mu2(psik_t_psik[k]*nf,psik_pt_psik[k]*nf,psik_p2t_psik[k]*nf,
		       psik_I_psik[k],psik_pI_psik[k],psik_p2I_psik[k],eps);
	final double w=iaf[k].di*(iaf[k].pc.pL-iaf[k].pc.pH);
	bdiff+=w*iaf[k].pc.corr;
	bdiff_norm+=w;
	if(eps>0)
	{
	    final ImpactFuture2Asymmetry ia=iaf[k];
	    UnitTests.checkDoubleRel(ia.lamconst,"ia.lamconst",bbasis[k],"bbasis[k]",eps);
	    System.err.println("ImpactFromTheFuture_Ci: k="+k+" ia.mL.beta="+ia.mL.beta+" ia.mH.beta="+ia.mH.beta);
	}
    }
    dir_scaled=bdiff/bdiff_norm;
    var_f_IH=iaf[ixH].var_f;
    var_pmu_IHf=mu2[ixH].Ip.getDF(p_last__with_p_offset_included);
    var_pmu_IHfa=mu2[ixH].Ip.getDF(mu2[ixH].pa);
    var_pmu_IHfm=mu2[ixH].DIpDpm;
    var_pmu_IHf_qsk=mu2[ixH].qsk;
    final Mu2 muIH=new Mu2(I.getFPsi(ixH,mm.QQt_sample),I.getFPsi(ixH,mm.QQpt_sample),I.getFPsi(ixH,mm.QQp2t_sample),
			   I.getFPsi(ixH,mm.QQi),I.getFPsi(ixH,mm.QQpi),I.getFPsi(ixH,mm.QQp2i),eps);

    var_pmu_IH=muIH.Ip.getDF(p_last__with_p_offset_included);
    var_pmu_IHa=muIH.Ip.getDF(muIH.pa);
    var_pmu_IHm=muIH.DIpDpm;
    var_pmu_IH_qsk=muIH.qsk;
}

static void matchImpactFromTheFuture_Pfi(final ImpactFromTheFuture_Ci a,final ImpactFromTheFuture_Pfi b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    UnitTests.checkDoubleRel(a.I_0_feq,"a.I_0_feq",b.I_0_feq,"b.I_0_feq",eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dir_scaled,"a.dir_scaled",b.dir_scaled,"b.dir_scaled",eps);
    for(int k=0;k<a.I.neq;k++)
    {
	final ImpactFuture2Asymmetry af=a.iaf[k];
	final double nrm=af.di/af.normbeta;
        UnitTests.checkDoubleRel(af.mL.beta*nrm,"af.mL.beta",b.betas[k][0],"b.betas[k][0]",eps);
        UnitTests.checkDoubleRel(af.mH.beta*nrm,"af.mH.beta",b.betas[k][1],"b.betas[k][1]",eps);
	com.polytechnik.utils.ProbabilityCorrelation.matchProbabilityCorrelation(af.pc,b.pcs[k],eps);
    }
    System.err.println("ImpactFromTheFuture_Ci.matchImpactFromTheFuture_Pfi test OK n="+a.I.neq+" eps="+eps);
}
}
