package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;

/**
 * Impact from the future asymmetry in price basis
 *  @version $Id: ImpactFromTheFuture_Dpi.java,v 1.18 2022/07/01 17:28:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactFromTheFuture_Dpi extends ImpactFromTheFuture2 {

final double [] impact_from_the_future_asymmetry=new double [I.neq],
    var2_from_the_future_asymmetry=new double [I.neq];
final double sum_impact;
ImpactFromTheFuture_Dpi(final IMomentsData mm,
			final OrthogonalPolynomialsMonomialsBasis M,
			final double eps)
{
    super(mm,M,eps);
    final int n=I.neq;
    for(int k=0;k<n;k++)
    {
	final double [] bpsi=EVXData.getPsiFromBasis(k,bbasis,n),
	    psi=EVXData.convertPsiFromBtoQ_Basis(n,bpsi,I.ev),
	    Dpsi=M.B.getDifferentiated(psi),
	    bDpsi=EVXData.convertPsiFromQtoB_Basis(n,java.util.Arrays.copyOf(Dpsi,n),I.ev,mm.QQt);
	final double Dpsi_psi0=TMatr.mulVV(bpsi0,bDpsi),Dpsi_psi=TMatr.mulVV(bpsi,bDpsi),
	    psi_psi0=psik_psi0[k],wk=Math.pow(psi_psi0,2);
	impact_from_the_future_asymmetry[k]=2*psi_psi0*(Dpsi_psi0-psi_psi0*Dpsi_psi);
	var2_from_the_future_asymmetry[k]=
	    Math.pow(Dpsi_psi0,2)-wk*TMatr.mulVV(bDpsi,bDpsi)
	    -2*impact_from_the_future_asymmetry[k]*Dpsi_psi;
	final double [] nomc=new double []{wk,2*psi_psi0*Dpsi_psi0,Math.pow(Dpsi_psi0,2)},
	    dnomc=new double []{1,2*Dpsi_psi,TMatr.mulVV(bDpsi,bDpsi)};
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,Dpsi,mm.QQt,n),"<psi|Dpsi>",TMatr.mulVV(bpsi,bDpsi),"<bpsi|bDpsi>",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,Dpsi,mm.QQi,n),"<psi|I|Dpsi>",TMatr.mulVVV(bpsi,bDpsi,I.getEValues()),"<bpsi|I|bDpsi>",eps);
	    UnitTests.checkDoubleArrayRel(Dpsi,"Dpsi",com.polytechnik.utils.Polynomials.getPolynomialDifferentiated(psi),"D(psi)",eps);
	    final double itest=-2*(TMatr.getAMatrB(psi,Dpsi,mm.QQi,n)-
				   TMatr.getAMatrA(psi,mm.QQi,n)*TMatr.getAMatrB(psi,Dpsi,mm.QQt,n));
	    UnitTests.checkDoubleRel(dI*impact_from_the_future_asymmetry[k],"dI*impact_from_the_future_asymmetry[k]",itest,"itest",eps);
	    final com.polytechnik.utils.PolynomialsRatio pr=
		new com.polytechnik.utils.PolynomialsRatio(dnomc,nomc,M.B);
	    UnitTests.checkDoubleRel(pr.getF(0),"pr.getF(0)",wk,"wk",eps);
	    UnitTests.checkDoubleRel(pr.getDF(0),"pr.getDF(0)",impact_from_the_future_asymmetry[k],"impact_from_the_future_asymmetry[k]",eps);
	    UnitTests.checkDoubleRel(0.5*pr.getD2F(0),"0.5*pr.getD2F(0)",var2_from_the_future_asymmetry[k],"var2_from_the_future_asymmetry[k]",eps);
	    final double [] rq=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(
		psi,Dpsi,null,TMatr.scaleVectorInPlace(TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),1/I.o_o),mm.QQt,n);
	    UnitTests.checkDoubleRel(pr.getF(0),"pr.getF(0)",rq[0],"r1[0]",eps);
	    UnitTests.checkDoubleRel(pr.getDF(0),"pr.getDF(0)",rq[1],"rq[1]",eps);
	    UnitTests.checkDoubleRel(pr.getD2F(0),"pr.getD2F(0)",rq[2],"rq[2]",eps);
	}
    }
    sum_impact=TMatr.sumElements(impact_from_the_future_asymmetry);
    if(eps>0)
    {
	if(n==2)
	{
	    final ImpactFuture2Asymmetry i=
		new ImpactFuture2Asymmetry(mm.QQt[0],mm.QQt[1],mm.QQt[3],
					   mm.QQi[0],mm.QQi[1],mm.QQi[3],
					   dI/I.o_o,p_last__with_p_offset_included,eps);
	    //UnitTests.checkDoubleRel(sum_impact,"sum_impact",i.mL.beta+i.mH.beta,"i.mL.beta+i.mH.beta",eps);
	   UnitTests.checkDoubleRel(bbasis[0],"bbasis[0]",i.mL.lam,"i.mL.lam",eps);
	   UnitTests.checkDoubleRel(bbasis[1],"bbasis[1]",i.mH.lam,"i.mH.lam",eps);
	}
	System.err.println("ImpactFromTheFuture_Dpi: test OK, n="+n+" eps="+eps);
    }
}
}
