package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.ProbabilityCorrelation;
import com.polytechnik.utils.Skewness;

/**
 * Impact from the future asymmetry, hybrid approach in d=2 price basis.
 *  @version $Id: ImpactFromTheFuture_P0i.java,v 1.30 2020/09/27 20:33:15 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactFromTheFuture_P0i extends ImpactFromTheFuture2 {
/** Market direction. */
    public final double dir_Dpf_I,dir_Dp_pf,dir_Dp_I,dir_iDp_pf;
    public final double [] dPnL_Dpf_I=new double [I.neq],dPnL_Dp_pf=new double [I.neq],
	dPnL_Dp_I=new double [I.neq],dPnL_iDp_pf=new double [I.neq];
public ImpactFromTheFuture_P0i(final IMomentsData mm,
			       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			       final double eps)
{
    super(mm,M,eps);
    final int n=I.neq,d=2;
    final double Pf=p_last__with_p_offset_included;
    final double [] // monomial basis in price space
	Mpf=new double [] {1,Pf,Pf,Pf*Pf},MDpf=new double [] {0,1,1,2*Pf};
    for(int k=0;k<n;k++)
    {
	final double [] Mi=new double []{psik_I_psik[k],psik_pI_psik[k],psik_pI_psik[k],psik_p2I_psik[k]},
	    Mt=new double []{psik_t_psik[k],psik_pt_psik[k],psik_pt_psik[k],psik_p2t_psik[k]},
	    MDp=new double []{0,psik_t_psik[k],psik_t_psik[k],2*psik_pt_psik[k]},
	    MiDp=new double []{0,psik_I_psik[k],psik_I_psik[k],2*psik_pI_psik[k]};
	final double ib=psik_I_psik[k],di=dI*Math.pow(psik_psi0[k],2),w=Math.min(ib,di),
		pa=psik_pt_psik[k]/psik_t_psik[k],sigma2=psik_p2t_psik[k]/psik_t_psik[k]-pa*pa,
		o_o=(1+Math.pow(Pf-pa,2)/sigma2)/psik_t_psik[k];
	final double [] MDb=TMatr.sumaAbB(1/o_o,MDpf,-0.5,MDp);
	dPnL_Dpf_I[k]=w*(new ProbabilityCorrelation(Mt,MDpf,Mi,com.polytechnik.utils.DiffSkewness.MB,true).corr);
	dPnL_Dp_pf[k]=w*(new ProbabilityCorrelation(Mt,MDp,Mpf,com.polytechnik.utils.DiffSkewness.MB,true).corr);
	dPnL_Dp_I[k]=w*(new ProbabilityCorrelation(Mt,MDp,Mi,com.polytechnik.utils.DiffSkewness.MB,true).corr);
	dPnL_iDp_pf[k]=w*(new ProbabilityCorrelation(Mt,MiDp,Mpf,com.polytechnik.utils.DiffSkewness.MB,true).corr);
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(ib+di,"ib+di",bbasis[k],"bbasis[k]",eps);
	    final double [] ev=M.EV.getGEV_Generalized(d,Mi,Mt,true); // eigenvalues problem
	    final double psi2L_Pf=TMatr.getAMatrA(d,ev,d+0*d,Mpf,d),psi2H_Pf=TMatr.getAMatrA(d,ev,d+(d-1)*d,Mpf,d),
		skewness0=(psi2L_Pf-psi2H_Pf)/(psi2L_Pf+psi2H_Pf);
	    final ProbabilityCorrelation pc_IQQl=
		new ProbabilityCorrelation(Mt,Mpf,Mi,com.polytechnik.utils.DiffSkewness.MB,true);
	    UnitTests.checkDoubleRel(skewness0,"skewness0",-pc_IQQl.corr,"-pc_IQQl.corr",eps);
	    final Skewness sk0=Skewness.getG0SkewnessFromMonomialMoments(
		psik_t_psik[k],psik_pt_psik[k],psik_p2t_psik[k],
		psik_I_psik[k],psik_pI_psik[k],psik_p2I_psik[k],Pf /* define |psi0> at Pf. */);
	    UnitTests.checkDoubleRel(skewness0,"skewness0",sk0.sk,"sk0.sk",eps);
	    final ImpactFuture2Asymmetry i=new ImpactFuture2Asymmetry(
		psik_t_psik[k],psik_pt_psik[k],psik_p2t_psik[k],
		psik_I_psik[k],psik_pI_psik[k],psik_p2I_psik[k],
		0,Pf,eps);
	    //UnitTests.checkDoubleRel(skewness0,"skewness0",i.gamma0.sk,"i.gamma0.sk",eps);
	    UnitTests.checkDoubleRel(psi2L_Pf,"psi2L_Pf",Math.pow(i.mL.psi_at_Pf,2),"i.mL.psi_at_Pf^2)",eps);
	    UnitTests.checkDoubleRel(psi2H_Pf,"psi2H_Pf",Math.pow(i.mH.psi_at_Pf,2),"i.mH.psi_at_Pf^2)",eps);
	    //final ProbabilityCorrelation pcf=new ProbabilityCorrelation(Mt,MDpf,Mpf,com.polytechnik.utils.DiffSkewness.MB,true);
	    //UnitTests.checkDoubleRel(pcf.corr,"pcf.corr",0,"0",eps);
	    System.err.println("ImpactFromTheFuture_P0i: test OK, k="+k+" eps="+eps);
	}
    }
    dir_Dpf_I=TMatr.sumElements(dPnL_Dpf_I)/dI;
    dir_Dp_pf=TMatr.sumElements(dPnL_Dp_pf)/dI;
    dir_Dp_I=TMatr.sumElements(dPnL_Dp_I)/dI;
    dir_iDp_pf=TMatr.sumElements(dPnL_iDp_pf)/dI;
}

}
