package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.ProbabilityCorrelation;

/**
 * Impact from the future asymmetry, hybrid approach in d=2 price basis.
 *  @version $Id: ImpactFromTheFuture_Pfi.java,v 1.23 2022/07/27 16:14:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactFromTheFuture_Pfi extends ImpactFromTheFuture2 {
public final double dir_scaled;
public final ProbabilityCorrelation [] pcs=new ProbabilityCorrelation [I.neq];
public final double [] wcorr=new double [I.neq];
/** For unit test. */
final double dir_L,dir_H;
final double [][] betas=new double [I.neq][];
public ImpactFromTheFuture_Pfi(final IMomentsData mm,
			       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> _M,
			       final double eps)
{
    super(mm,_M,eps);
    final int n=I.neq,d=2;
    final double Pf=p_last__with_p_offset_included;

    // monomial basis in price space
    final com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis MB=
	com.polytechnik.utils.DiffSkewness.MB;
    // In price space. Regular differentiation.
    final VolMatrix varpsi=MB.B::getDifferentiated;

    final double [] Mpf=new double [] {1,Pf,Pf,Pf*Pf};
    for(int k=0;k<n;k++)
    {
	final double ib=psik_I_psik[k],di=dI*Math.pow(psik_psi0[k],2),nf=psik_t_psik[k]/1;
	final double [] Mi=new double []{psik_I_psik[k],psik_pI_psik[k],psik_pI_psik[k],psik_p2I_psik[k]},
	    Mt=TMatr.scaleVectorInPlace(new double []{psik_t_psik[k],psik_pt_psik[k],psik_pt_psik[k],psik_p2t_psik[k]},1/nf),
	    Mif=TMatr.sumAcB(Mi,di,Mpf);

        //===============  Calculate using probability correlation ======================
	//  o_o=(psi0(Pf))^2 in a kludgy way, see below o_o_nr same thing from an eigenproblem.
	final double pa=Mt[1]/Mt[0],sigma2=Mt[3]/Mt[0]-pa*pa,o_o=(1+Math.pow(Pf-pa,2)/sigma2)/Mt[0];
	// The Matrix.
	final double [] MDb=TMatr.sumaAbB(1/o_o,varpsi.getDiffMatrix(d,Mpf),-0.5,varpsi.getDiffMatrix(d,Mt));
	pcs[k]=new ProbabilityCorrelation(Mt,MDb,Mif,MB,true);
	// This weight is always negative.
	wcorr[k]=di*(pcs[k].pL-pcs[k].pH);
        //===============  END Calculate using probability correlation ======================
	if(eps>0)
	{
	    final double [] _MDb=TMatr.sumaAbB(1/o_o,new double []{0,1,1,2*Pf},-0.5,new double []{0,Mt[0],Mt[0],2*Mt[1]});
	    UnitTests.checkDoubleArrayRel(MDb,"MDb",_MDb,"_MDb",eps);
	}

	// same calculation, without probability correlation concept, direct eigenvalues problem
	betas[k]=new double [d];
	final double [] ev=MB.EV.getGEV_Generalized(d,Mif,Mt,true); // eigenvalues problem
	double o_o_nr=0;
	for(int j=0;j<d;j++)
	{
	    final double [] phi=EVXData.getPsiFromBasis(j,ev,d),Dphi=varpsi.getDPsi(phi),
		rqf=TMatr.scaleVectorInPlace(com.polytechnik.utils.RayleighQuotient.getRQVariationValues(phi,Dphi,null,Mpf,Mt,d),di);
	    betas[k][j]=rqf[1];
	    o_o_nr+=TMatr.getAMatrA(phi,Mpf,d);
	    if(eps>0)
	    {
		final double [] rq=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(phi,Dphi,null,Mi,Mt,d);
		UnitTests.checkDoubleRel(TMatr.getAMatrA(phi,Mt,d),"<phi|M|phi>",1,"1",eps);
		UnitTests.checkDoubleRel(rq[0]+rqf[0],"rq[0]+rqf[0]",ev[j],"ev[j]",eps);
		UnitTests.checkDoubleRel(rq[1]+rqf[1],"rq[1]+rqf[1]",0,"0",eps);
		final double vv=2*(TMatr.getAMatrB(phi,Dphi,Mpf,d)-TMatr.getAMatrA(phi,Mpf,d)*TMatr.getAMatrB(phi,Dphi,Mt,d));
		System.err.println("k="+k+" j="+j+" vv="+vv);
		UnitTests.checkDoubleRel(rqf[1],"rqf[1]",di*vv,"di*vv",eps);
	    }
	}
	if(eps>0)
	{ // o_o_nr to match o_o
	    UnitTests.checkDoubleRel(o_o_nr,"o_o_nr",o_o,"o_o",eps);
	}
	// proper normalization: <phi|phi_0>=phi(Pf)/sqrt(phi1^2(Pf)+phi2^2(Pf)).
	TMatr.scaleVectorInPlace(betas[k],1/o_o_nr);
	if(eps>0)
	{
	    final ProbabilityCorrelation pc=pcs[k];
	    UnitTests.checkDoubleRel(Mif[0]/Mt[0],"Mif[0]/Mt[0]",bbasis[k],"bbasis[k]",eps);
	    UnitTests.checkDoubleRel(pc.corr*(pc.pL-pc.pH),"pc.corr*(pc.pL-pc.pH)",(betas[k][0]-betas[k][1])/di,"(betas[k][0]-betas[k][1])/di",eps);
	    System.err.println("ImpactFromTheFuture_Pfi: test OK, k="+k+" eps="+eps);
	}
    }

    double dir_L=0,dir_H=0,bdiff=0,bdiff_norm=0;
    for(int k=0;k<n;k++)
    {
	dir_L+=betas[k][0];
	dir_H+=betas[k][1];
	bdiff+=wcorr[k]*pcs[k].corr;
	bdiff_norm+=wcorr[k];
    }
    this.dir_L=dir_L/dI;
    this.dir_H=dir_H/dI;
    this.dir_scaled=bdiff/bdiff_norm;
    if(eps>0)
    {
	UnitTests.checkDoubleRel(dir_L-dir_H,"dir_L-dir_H",bdiff,"bdiff",eps);
    }
}
}
