package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 * Impact from the future asymmetry in price basis
 *  @version $Id: ImpactFromTheFuture_Pi.java,v 1.18 2022/07/01 17:28:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactFromTheFuture_Pi extends ImpactFromTheFuture2 {

final double [] beta_past=new double [I.neq],beta_future=new double [I.neq],
    beta_dP=new double [I.neq],beta_dPdt=new double [I.neq],
    beta_Pi=new double [I.neq];
final double sum_impact_dP,sum_impact_dPdt,dP_eq,Pi_eq;
ImpactFromTheFuture_Pi(final IMomentsData mm,
		       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		       final VolMatrix VOLATILITY_EDPsi,
		       final double eps)
{
    super(mm,M,eps);
    final int n=I.neq;
    for(int k=0;k<n;k++)
    {
	final double [] bpsi=EVXData.getPsiFromBasis(k,bbasis,n),
	    psi=EVXData.convertPsiFromBtoQ_Basis(n,bpsi,I.ev),
	    Dpsi=VOLATILITY_EDPsi.getDPsi(psi),
	    psi0=EVXData.convertPsiFromBtoQ_Basis(n,bpsi0,I.ev),
	    bDpsi=EVXData.convertPsiFromQtoB_Basis(n,java.util.Arrays.copyOf(Dpsi,n),I.ev,mm.QQt);
	final double Dpsi_psi0=TMatr.mulVV(bpsi0,bDpsi),Dpsi_psi=TMatr.mulVV(bpsi,bDpsi),
	    psi_psi0=psik_psi0[k],wk=Math.pow(psi_psi0,2);
	beta_future[k]=dI*2*psi_psi0*(Dpsi_psi0-psi_psi0*Dpsi_psi);
	beta_past[k]=2*(TMatr.getAMatrB(psi,Dpsi,mm.QQi,n)-
			TMatr.getAMatrA(psi,mm.QQi,n)*TMatr.getAMatrB(psi,Dpsi,mm.QQt,n));
	beta_dP[k]=2*psi_psi0*(TMatr.getAMatrB(Dpsi,psi0,mm.QQdpdt,n)-psi_psi0*TMatr.getAMatrB(Dpsi,psi,mm.QQdpdt,n));
	beta_dPdt[k]=2*psi_psi0*(TMatr.getAMatrB(psi,psi0,mm.QQdpdt,n)-psi_psi0*TMatr.getAMatrA(psi,mm.QQdpdt,n));
	beta_Pi[k]=2*psi_psi0*(TMatr.getAMatrB(psi,psi0,mm.QQpi,n)-psi_psi0*TMatr.getAMatrA(psi,mm.QQpi,n));
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(2*psi_psi0*(TMatr.getAMatrB(psi,psi0,mm.QQi,n)-psi_psi0*TMatr.getAMatrA(psi,mm.QQi,n)),"beta Pi",-2*dI*(wk-wk*wk),"-2*dI*(wk-wk*wk)",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,Dpsi,mm.QQt,n),"<psi|Dpsi>",TMatr.mulVV(bpsi,bDpsi),"<bpsi|bDpsi>",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,Dpsi,mm.QQi,n),"<psi|I|Dpsi>",TMatr.mulVVV(bpsi,bDpsi,I.getEValues()),"<bpsi|I|bDpsi>",eps);
	    UnitTests.checkDoubleArrayRel1(psi0,"psi0",I.psi0,"I.psi0",eps);
	    UnitTests.checkDoubleRel(beta_past[k],"beta_past[k]",-beta_future[k],"-beta_future[k]",eps);
	    final double [] nomc=new double []{wk,2*psi_psi0*Dpsi_psi0,Math.pow(Dpsi_psi0,2)},
		dnomc=new double []{1,2*Dpsi_psi,TMatr.mulVV(bDpsi,bDpsi)};
	    final com.polytechnik.utils.PolynomialsRatio pr=
		new com.polytechnik.utils.PolynomialsRatio(dnomc,TMatr.getVectorScaled(nomc,dI),com.polytechnik.utils.DiffSkewness.MB.B);
	    UnitTests.checkDoubleRel(pr.getF(0),"pr.getF(0)",wk*dI,"wk*dI",eps);
	    UnitTests.checkDoubleRel(pr.getDF(0),"pr.getDF(0)",beta_future[k],"beta_future[k]",eps);
	    final double [] rq=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(
		psi,Dpsi,null,TMatr.scaleVectorInPlace(TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),dI/I.o_o),mm.QQt,n);
	    UnitTests.checkDoubleRel(pr.getF(0),"pr.getF(0)",rq[0],"r1[0]",eps);
	    UnitTests.checkDoubleRel(pr.getDF(0),"pr.getDF(0)",rq[1],"rq[1]",eps);
	    UnitTests.checkDoubleRel(pr.getD2F(0),"pr.getD2F(0)",rq[2],"rq[2]",eps);
	}
    }
    sum_impact_dP=TMatr.sumElements(beta_dP);
    sum_impact_dPdt=TMatr.sumElements(beta_dPdt);
    final double [] w=TMatr.squareArrayElements(psik_psi0);
    dP_eq=-TMatr.sumElements(beta_dPdt)/(2*(1-TMatr.mulVV(w,w)));
    Pi_eq=-TMatr.sumElements(beta_Pi)/(dI*2*(1-TMatr.mulVV(w,w)));
    if(eps>0)
    {

	UnitTests.checkDoubleRel(Pf_from_dyn,"Pf_from_dyn",Pi_eq,"Pi_eq",eps);
	System.err.println("ImpactFromTheFuture_Pi: test OK, n="+n+" eps="+eps);
    }
}
}
