package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: ImpactIStatesExample.java,v 1.34 2022/07/28 22:30:49 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactIStatesExample {
    public final EVXData eI;
    public final double p_IH,dp_orig_IH,vp_IH,vp_spur,vpI_spur,v2p_IH,dpI_orig_IH,dp_orig_eI_A,dp_orig_eI_Aa,dp_orig_eI_v2I,p_offset;
    final double [] eIdp;
    final double I_eIdp_L,I_eIdp_H,w_IH_eIdp_L,w_IH_eIdp_H,w_IL_eIdp_L,w_IL_eIdp_H;

/** For unit test. */
    final double [] p_I,dp_orig_I,psik_Dpsik,var2_I,p2volI_I,var_p2volI_I;
    ImpactIStatesExample(final EVXData eI,final double [] eIdp,final double [] p_I,final double [] dp_orig_I,final double [] psik_Dpsik,final double [] var2_I,final double p_offset,final double [] p2volI_I,final double [] var_p2volI_I)
{
    this.p_offset=p_offset;
    this.eI=eI;
    this.eIdp=eIdp;
    this.p_I=p_I;
    this.dp_orig_I=dp_orig_I;
    this.psik_Dpsik=psik_Dpsik;
    this.var2_I=var2_I;
    this.p2volI_I=p2volI_I;
    this.var_p2volI_I=var_p2volI_I;
    final int n=eI.neq;

    this.p_IH=p_I[n-1];
    this.vp_IH=psik_Dpsik[n-1];
    this.v2p_IH=var2_I[n-1];
    this.vp_spur=TMatr.sumElements(psik_Dpsik);
    this.vpI_spur=TMatr.mulVVn1(psik_Dpsik,eI.ev);
    this.dp_orig_IH=dp_orig_I[n-1];
    this.dpI_orig_IH=dp_orig_IH*eI.sH;
    this.dp_orig_eI_Aa=TMatr.mulVVn1(dp_orig_I,eI.ev);
    this.dp_orig_eI_A=this.dp_orig_eI_Aa-eI.sL*TMatr.sumElements(dp_orig_I);
    this.dp_orig_eI_v2I=TMatr.mulVV(dp_orig_I,var2_I);
    I_eIdp_L=TMatr.mulV2V(EVXData.getPsiFromBasis(0,eIdp,n),eI.getEValues());
    I_eIdp_H=TMatr.mulV2V(EVXData.getPsiFromBasis(n-1,eIdp,n),eI.getEValues());
    w_IH_eIdp_L=eIdp[n+0*n+n-1];
    w_IH_eIdp_H=eIdp[n+(n-1)*n+n-1];
    w_IL_eIdp_L=eIdp[n+0*n+0];
    w_IL_eIdp_H=eIdp[n+(n-1)*n+0];
}

/**
 * Calculate dp/dt on dV/dt states.
 */
public static ImpactIStatesExample get_ImpactIStatesExample(final MomentsData mm,
					      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,
					      final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");

    final EVXData eI=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double [] p_I=new double [n],dp_orig_I=new double [n],var2_I=new double [n],
	p2volI_I=new double [n],var_p2volI_I=new double [n],psik_Dpsik=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=eI.getPsi(k),Dpsik=VOLATILITY_EDPsi.getDPsi(psik);
	final double pIk=TMatr.getAMatrA(psik,mm.QQpi,n),Ik=eI.ev[k],psik_I_Dpsik=2*TMatr.getAMatrB(psik,Dpsik,mm.QQi,n);
	p_I[k]=pIk/Ik;
	dp_orig_I[k]=TMatr.getAMatrA(psik,mm.QQdp,n);
	var2_I[k]=2*(TMatr.getAMatrA(Dpsik,mm.QQi,n)-eI.ev[k]*TMatr.getAMatrA(Dpsik,mm.QQt,n));
	p2volI_I[k]=TMatr.getAMatrA(psik,mm.QQp2i,n)+p_I[k]*(-2*pIk+p_I[k]*eI.ev[k]);
	psik_Dpsik[k]=2*TMatr.getAMatrB(psik,Dpsik,mm.QQt,n);
	var_p2volI_I[k]=2*(TMatr.getAMatrB(psik,Dpsik,mm.QQp2i,n)+p_I[k]*(-2*TMatr.getAMatrB(psik,Dpsik,mm.QQpi,n)+p_I[k]*0.5*psik_I_Dpsik)-p2volI_I[k]*0.5*psik_Dpsik[k]);
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(Ik*psik_Dpsik[k],"Ik*psik_Dpsik[k]",psik_I_Dpsik,"psik_I_Dpsik",eps);
	    final double [] rqe=RayleighQuotient.getRQVariationValues(psik,Dpsik,null,mm.QQi,mm.QQt,n);
	    UnitTests.checkDoubleRel(eI.ev[k],"eI.ev[k]",rqe[0],"rqe[0]",eps);
	    UnitTests.checkDoubleRel(0,"0",rqe[1],"rqe[1]",eps);
	    UnitTests.checkDoubleRel(var2_I[k],"var2_I[k]",rqe[2],"rqe[2]",eps);
	    final double [] QQa=new double [n*n];
	    for(int l=QQa.length;--l>=0;) QQa[l]=mm.QQp2i[l]+p_I[k]*(-2*mm.QQpi[l]+p_I[k]*mm.QQi[l]);
	    final double [] rqa=RayleighQuotient.getRQVariationValues(psik,Dpsik,null,QQa,mm.QQt,n);
	    UnitTests.checkDoubleRel(p2volI_I[k],"p2volI_I[k]",rqa[0],"rqa[0]",eps);
	    UnitTests.checkDoubleRel(var_p2volI_I[k],"var_p2volI_I[k]",rqa[1],"rqa[1]",eps);
	    System.err.println("ImpactIStatesExample.get_ImpactIStatesExample: test eI OK n="+n+" k="+k+" eps="+eps);
	}
    }
    final double [] bQQdp=EVXData.convertMatrFromQtoB_Basis(n,mm.QQdp,eI.ev),
	bQQIdp=new double [n*n];
    for(int k=0;k<n;k++)
	for(int l=0;l<n;l++)
	    bQQIdp[k*n+l]=bQQdp[k*n+l]*(eI.ev[l]+eI.ev[k])/2;
    final double [] eIdp=M.EV.getEV(n,bQQIdp,true);
    if(eps>0)
    {
	// check spur basis invariance
	UnitTests.checkDoubleRel(TMatr.mulVVn1(dp_orig_I,eI.ev),"sp dp_orig_I,eI.ev",TMatr.sumElements(java.util.Arrays.copyOf(eIdp,n)),"sum(ev eIdp)",eps);
	System.err.println("ImpactIStatesExample.get_ImpactIStatesExample: invariance test eI OK n="+n+" eps="+eps);
    }

    return new ImpactIStatesExample(eI,eIdp,p_I,dp_orig_I,psik_Dpsik,var2_I,mm.p_offset,p2volI_I,var_p2volI_I);
}


static void matchImpactIStatesExample(final ImpactIStatesExample a,final ImpactIStatesExample b, final double eps)
{
    final int n=a.eI.neq;

    final double p_offset_shift=b.p_offset-a.p_offset;
    final double [] o_shift=new double [n];
    java.util.Arrays.fill(o_shift,p_offset_shift);

    EVXData.matchEVXData(a.eI,b.eI,false,eps);
    UnitTests.checkDoubleArrayRel(java.util.Arrays.copyOf(a.eIdp,n),"a.eIdp",java.util.Arrays.copyOf(b.eIdp,n),"b.b.eIdp",eps);
    UnitTests.checkDoubleArrayRel(a.dp_orig_I,"a.dp_orig_I",b.dp_orig_I,"b.dp_orig_I",eps);
    UnitTests.checkDoubleArrayRel(a.var2_I,"a.var2_I",b.var2_I,"b.var2_I",eps);
    UnitTests.checkDoubleArrayRel(a.p_I,"a.p_I",DeprecatedHelperTMatr.sumAB(n,b.p_I,o_shift),"b.p_I+p_offset_shift",eps);

    System.err.println("ImpactIStatesExample.matchImpactIStatesExample: test OK n="+n+" eps="+eps);
}

static void matchPnLDiagProj(final ImpactIStatesExample a,final PnLDiagProj b, final double eps)
{
    final int n=a.eI.neq;

    final double p_offset_shift=b.p_offset-a.p_offset;
    final double [] o_shift=new double [n];
    java.util.Arrays.fill(o_shift,p_offset_shift);

    EVXData.matchEVXData(a.eI,b.I,false,eps);
    UnitTests.checkDoubleArrayRel(a.dp_orig_I,"a.dp_orig_I",b.dp,"b.dp",eps);
    UnitTests.checkDoubleArrayRel(a.p_I,"a.p_I",DeprecatedHelperTMatr.sumAB(n,b.p,o_shift),"b.p+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);

    System.err.println("ImpactIStatesExample.matchPnLDiagProj: test OK p_offset_shift="+p_offset_shift+" n="+n+" eps="+eps);
}

static void matchPnLVarVolatilitypi(final ImpactIStatesExample a,final PnLVarVolatilitypi b, final double eps)
{
    final int n=a.eI.neq;
    final double p_offset_shift=b.p_offset-a.p_offset;
    final double [] o_shift=new double [n];
    java.util.Arrays.fill(o_shift,p_offset_shift);

    EVXData.matchEVXData(a.eI,b.I,false,eps);
    UnitTests.checkDoubleArrayRel(a.p_I,"a.p_I",DeprecatedHelperTMatr.sumAB(n,b.p,o_shift),"b.p+p_offset_shift",eps);
    UnitTests.checkDoubleArrayRel(a.p2volI_I,"a.p2volI_I",b.Volatility,"b.Volatility",eps);
    UnitTests.checkDoubleArrayRel(a.var_p2volI_I,"a.var_p2volI_I",b.varVolatility,"b.varVolatility",eps);

    System.err.println("ImpactIStatesExample.matchPnLVarVolatilitypi: test OK p_offset_shift="+p_offset_shift+" n="+n+" eps="+eps);

}

static void matchPnLVar3Mupi(final ImpactIStatesExample a,final PnLVar3Mupi b, final double eps)
{
    final int n=a.eI.neq;

    final double p_offset_shift=b.p_offset-a.p_offset;
    EVXData.matchEVXData(a.eI,b.I,false,eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.vp_IH,"a.vp_IH",b.vp_IH,"b.vp_IH",eps);
    UnitTests.checkDoubleRel(a.v2p_IH,"a.v2p_IH",b.v2p_IH,"b.v2p_IH",eps);
    UnitTests.checkDoubleRel(a.vpI_spur,"a.vpI_spur",b.vpI_spur,"b.vpI_spur",eps);
    UnitTests.checkDoubleArrayRel(a.psik_Dpsik,"a.psik_Dpsik",b.psik_Dpsik,"b.psik_Dpsik",eps);
    UnitTests.checkDoubleArrayRel(a.var2_I,"a.var2_I",b.var2_I,"b.var2_I",eps);

    System.err.println("ImpactIStatesExample.matchPnLVar3Mupi: test OK p_offset_shift="+p_offset_shift+" n="+n+" eps="+eps);
}

}
