package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.ProbabilityCorrelation;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: ImpactQVM.java,v 1.137 2022/07/15 19:57:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class ImpactQVM<DTYPE> {

public static class State {
    public final long seq;
    State(final State prev)
    {
	this.seq=(prev==null?0:1+prev.seq);
    }
}
public final State sst;

public final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=getABasis();
public final ImpactExample iex;
public final ImpactIStatesExample ist;
public final PnLDiagProj pnldiag;
final PnLMu pnlmuIL,pnlmuIH;
final PnLMuH pnlmuh;
final PnLVH pnlVH;
final DynVar2DiagProj dvardiagproj;
final PnLI0 pnlI0,bpnlI0;
final PnL2I0 pnl2bI0,pnl2iterI0;
final PnLbI0 pnlbI0;
final PnLfIHI0 pnlfihi0;
public final PnLdIf0 pnldif0,pnldif0_dyn_IH,pnldif0_dyn_PURE;
final PnLdIfdyn pnldifdyn;
final PnLI0e pnli0e,pnli0e_lI;
public final PnLI0IH pnli0ih;
public final PnLdIDyn pnldidyn_NONE,pnldidyn_IH,pnldidyn_PURE;
final PnLI0B pnli0b;
public final PnLdIdP pnldidp;
public final PnLdIdPRQ pnldidprq;
public final PnLdID pnldid;
public final PnLdIS pnldis;
public final PnLdISk pnldisk;
public final PnLdIVSk pnldivsk;
public final PnLdIV4 pnldiv4_nodI,pnldiv4_nodI_t,pnldiv4,pnldiv4_t;
public final PnLdICorr pnldicorr_nodI,pnldicorr_nodI_t,pnldicorr,pnldicorr_t;
public final PnLdISkewness pnldiskewness_nodI,pnldiskewness;
public final PnLdIDSk pnldidsk;
public final PnLdIdI pnldidi;
final PnLdIP pnldip,pnldip_nodI;
final DynVar2DISkewness dv2diskewness;
final DynVar2RDISkewness dv2rdiskewness,dv2rdiiskewness;
final DynVar2DIDP dv2didp_nodI,dv2didp;
final DynEVdI dynevdi;
final DynEVdI0 dynevdi0;
final PnLdIpx pnldipx;
final PnLdISpur pnldispur;
final PnLdIE pnldie,pnldie_dyn,pnldie_dyn__nodI;
final PnLdIE2 pnldie2,pnldie2__nodI;
final PnLVar3Mu pnlvar3mu;
final PnLSimpleSkewness pnlss;
final PnLFutureSk pnlfutureSk;
final PnLSensitivity pnlsens_pl,pnlsens_p0,pnlsens_pih;
final MicroPnLdI mpnlP,mpnlDP,mpnlDPDT;
final InIStateIHDensity inistateihdensity;
final DTYPE odata;
final VolMatrix VOLATILITY_EDPsi=this::getEDPsi;

ImpactQVM(final int n,final DTYPE odata,final State rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.odata=odata;
    final MomentsData mm=getMomentsData(n);
    this.ist=ImpactIStatesExample.get_ImpactIStatesExample(mm,M,VOLATILITY_EDPsi,0);
    this.pnldiag=PnLDiagProj.get_PnLDiagProj(mm,M,0);
    this.pnlmuIL=PnLMu.get_PnLMu(mm,M,true,0);
    this.pnlmuIH=PnLMu.get_PnLMu(mm,M,false,0);
    this.pnlmuh=PnLMuH.get_PnLMuH(mm,M,0);
    this.pnlVH=PnLVH.get_PnLVH(mm,M,0);
    this.dvardiagproj=DynVar2DiagProj.get_DynVar2DiagProj(n,mm.QQi,mm.QQt,mm.QQpi,M,VOLATILITY_EDPsi,0);
    this.pnlI0=PnLI0.get_PnLI0(mm,M,0);
    this.bpnlI0=PnLI0.get_PnLI0_NO_0_atBoundary(mm,M,0);
    this.pnl2bI0=PnL2I0.test_get_PnL2I0(mm,PnL2I0.STDFACTIH_NUMSTABILITY,M,0);
    this.pnl2iterI0=PnL2I0.test_get_iter_PnL2I0(mm,PnL2I0.STDFACTIH_NUMSTABILITY,PnL2I0.STDITERATIONS,M,0);
    this.pnlbI0=PnLbI0.get_PnLbI0(mm,M,0);
    this.pnlfihi0=PnLfIHI0.get_PnLfIHI0(mm,M,0);
    this.pnldif0=PnLdIf0.get_PnLdIf0(mm,M,0);
    this.pnldif0_dyn_IH=PnLdIf0.get_PnLdIf0(mm,M,0,PnLdIDyn.IMPACTAPPROXIMATION_DYNAMIC_IH);
    this.pnldif0_dyn_PURE=PnLdIf0.get_PnLdIf0(mm,M,0,PnLdIDyn.IMPACTAPPROXIMATION_DYNAMIC_PURE);
    this.pnldifdyn=PnLdIfdyn.get_PnLdIfdyn(mm,M,0);
    this.pnli0e=PnLI0e.get_PnLI0e(mm,M,false,0);
    this.pnli0e_lI=PnLI0e.get_PnLI0e(mm,M,true,0);
    this.pnldidyn_NONE=PnLdIDyn.get_PnLdIDyn(mm,M,0,PnLdIDyn.IMPACTAPPROXIMATION_NONE);
    this.pnldidyn_IH=PnLdIDyn.get_PnLdIDyn(mm,M,0,PnLdIDyn.IMPACTAPPROXIMATION_DYNAMIC_IH);
    this.pnldidyn_PURE=PnLdIDyn.get_PnLdIDyn(mm,M,0,PnLdIDyn.IMPACTAPPROXIMATION_DYNAMIC_PURE);
    this.pnli0ih=PnLI0IH.get_PnLI0IH(mm,M,0);
    this.pnli0b=PnLI0B.get_PnLI0B(mm,M,0);
    this.pnldidp=PnLdIdP.get_PnLdIdP(mm,M,0);
    this.pnldidprq=PnLdIdPRQ.get_PnLdIdPRQ(mm,M,0);
    this.pnldid=PnLdID.get_PnLdID(mm,M,VOLATILITY_EDPsi,0);
    this.pnldis=PnLdIS.get_PnLdIS(mm,M,0);
    this.pnldisk=PnLdISk.get_PnLdISk(mm,M,0);
    this.pnldiskewness_nodI=PnLdISkewness.get_PnLdISkewness(mm,M,false,0);
    this.pnldiskewness=PnLdISkewness.get_PnLdISkewness(mm,M,true,0);
    this.pnldidsk=PnLdIDSk.get_PnLdIDSk(mm,M,0);
    this.pnldidi=PnLdIdI.get_PnLdIdI(mm,M,0);
    this.dv2diskewness=DynVar2DISkewness.get_DynVar2DISkewness(mm,M,VOLATILITY_EDPsi,0);
    this.dv2rdiskewness=DynVar2RDISkewness.get_DynVar2RDISkewness(mm,M,VOLATILITY_EDPsi,false,0);
    this.dv2rdiiskewness=DynVar2RDISkewness.get_DynVar2RDISkewness(mm,M,VOLATILITY_EDPsi,true,0);
    this.dv2didp_nodI=DynVar2DIDP.get_DynVar2DIDP(mm,M,VOLATILITY_EDPsi,false,0);
    this.dv2didp=DynVar2DIDP.get_DynVar2DIDP(mm,M,VOLATILITY_EDPsi,true,0);
    this.dynevdi=DynEVdI.get_DynEVdI(mm,M,0);
    this.dynevdi0=DynEVdI0.get_DynEVdI0(mm,M,VOLATILITY_EDPsi,0);
    this.pnldipx=PnLdIpx.get_PnLdIpx(mm,M,0);
    this.pnldispur=PnLdISpur.get_PnLdISpur(mm,M,0);
    pnlsens_pl=PnLSensitivity.get_PnLSensitivity(mm,M,PnLSensitivity.PE_PLAST,0);
    pnlsens_p0=PnLSensitivity.get_PnLSensitivity(mm,M,PnLSensitivity.PE_P0,0);
    pnlsens_pih=PnLSensitivity.get_PnLSensitivity(mm,M,PnLSensitivity.PE_PIH,0);
    if(true) { // trivial probability correlation (with fixed tau).
     final MomentsData mm2=getMomentsData(2);
     this.pnlss=new PnLSimpleSkewness(mm2,M);
    }
    this.pnlvar3mu=PnLVar3Mu.get_PnLVar3Mu(mm,M,0);
    this.pnlfutureSk=PnLFutureSk.get_PnLFutureSk(mm,M,0);
    mpnlP=MicroPnLdI.get_MicroPnLdI(mm,MicroPnLdI.G_PRICE,M,0);
    mpnlDP=MicroPnLdI.get_MicroPnLdI(mm,MicroPnLdI.G_DPRICE,M,0);
    mpnlDPDT=MicroPnLdI.get_MicroPnLdI(mm,MicroPnLdI.G_DPRICEDT,M,0);
    inistateihdensity=new InIStateIHDensity(n,mm.p_offset,mm.QQt,mm.QQi,mm.QQpi,mm.QQdp,M,VOLATILITY_EDPsi,this::getPsi2WIntegratedDt,0);

    if(true)
    {
	final DataInterfaceToMoments<DTYPE> access=getAccess();
	final int nmoms=2*n-1+3;

	final double [][] mimt=YpkQVM.get_PV_PT_Moments(nmoms,odata,access);
	final double [] tmom=mimt[0],imom=mimt[4],p1imom=mimt[5],
	    p2imom=mimt[6],p3imom=mimt[7],_tmom=mimt[8],p4imom=new double [nmoms];
	access.setM(p4imom,4,access.M_I,odata,nmoms);
	/*
	final double [] tmom=new double [nmoms],imom=new double [nmoms],p1imom=new double [nmoms],
	    p2imom=new double [nmoms],p3imom=new double [nmoms],p4imom=new double [nmoms];
	access.setM(imom,0,access.M_I,odata,nmoms);
	access.setM(p1imom,1,access.M_I,odata,nmoms);
	access.setM(p2imom,2,access.M_I,odata,nmoms);
	access.setM(p3imom,3,access.M_I,odata,nmoms);
	access.setM(p4imom,4,access.M_I,odata,nmoms);
	access.setMT(tmom,nmoms);
	*/
	this.pnldivsk=PnLdIVSk.get_PnLdIVSk(n,tmom,imom,p1imom,p2imom,p3imom,access.getPOffset(odata),access.getPLast(odata),
					    this::getPsi2WIntegratedDt,M,0);
	final double [] QQp4i=M.B.getQQMatr(n,p4imom),_QQt=M.B.getQQMatr(n,_tmom);
	this.pnldiv4_nodI=PnLdIV4.get_PnLdIV4_QQ(n,mm.p_offset,
						 mm.QQt,
						 mm.QQi,
						 mm.QQpi,
						 mm.QQp2i,
						 mm.QQp3i,
						 QQp4i,
						 false,
						 mm.p_last,M,0);
	this.pnldiv4_nodI_t=PnLdIV4.get_PnLdIV4_QQ(n,mm.p_offset,
						   _QQt,
						   mm.QQi,
						   mm.QQpi,
						   mm.QQp2i,
						   mm.QQp3i,
						   QQp4i,
						   false,
						   mm.p_last,M,0);
	this.pnldiv4=PnLdIV4.get_PnLdIV4_QQ(n,mm.p_offset,
					    mm.QQt,
					    mm.QQi,
					    mm.QQpi,
					    mm.QQp2i,
					    mm.QQp3i,
					    QQp4i,
					    true,
					    mm.p_last,M,0);
	this.pnldiv4_t=PnLdIV4.get_PnLdIV4_QQ(n,mm.p_offset,
					      _QQt,
					      mm.QQi,
					      mm.QQpi,
					      mm.QQp2i,
					      mm.QQp3i,
					      QQp4i,
					    true,
					    mm.p_last,M,0);
	final double [] p1tmom=mimt[1],p2tmom=mimt[2],p3tmom=mimt[3],
	    QQp1t=M.B.getQQMatr(n,p1tmom),QQp2t=M.B.getQQMatr(n,p2tmom),QQp3t=M.B.getQQMatr(n,p3tmom);
	this.iex=ImpactExample.get_ImpactExample(n,mm.QQi,mm.QQt,mm.QQpi,mm.QQdp,QQp1t,M,VOLATILITY_EDPsi,mm.p_offset,0);
	this.pnldicorr_nodI=PnLdICorr.get_PnLdICorr(n,
						    access.getPOffset(odata),
						    tmom,p1tmom,p2tmom,p3tmom,
						    imom,p1imom,p2imom,p3imom,
						    false,
						    access.getPLast(odata),
						    M,0);
	this.pnldicorr_nodI_t=PnLdICorr.get_PnLdICorr(n,
						      access.getPOffset(odata),
						      mimt[8],mimt[9],mimt[10],mimt[11],
						      imom,p1imom,p2imom,p3imom,
						      false,
						      access.getPLast(odata),
						      M,0);
	this.pnldicorr=PnLdICorr.get_PnLdICorr(n,
					       access.getPOffset(odata),
					       tmom,p1tmom,p2tmom,p3tmom,
					       imom,p1imom,p2imom,p3imom,
					       true,
					       access.getPLast(odata),
					       M,0);
	this.pnldicorr_t=PnLdICorr.get_PnLdICorr(n,
						 access.getPOffset(odata),
						 mimt[8],mimt[9],mimt[10],mimt[11],
						 imom,p1imom,p2imom,p3imom,
						 true,
						 access.getPLast(odata),
						 M,0);
	pnldip=PnLdIP.get_PnLdIP(mm,M,QQp1t,QQp2t,QQp3t,true,0);
	pnldip_nodI=PnLdIP.get_PnLdIP(mm,M,QQp1t,QQp2t,QQp3t,false,0);
	pnldie=PnLdIE.get_PnLdIE(n,M,mm.p_offset,mm.QQt,mm.QQi,mm.QQpi,mm.QQp2i,mm.QQp3i,QQp1t,QQp2t,QQp3t,0);
	pnldie_dyn=PnLdIE.get_PnLdIE_DYN(mm,M,true,0);
	pnldie_dyn__nodI=PnLdIE.get_PnLdIE_DYN(mm,M,false,0);
	pnldie2=PnLdIE2.get_PnLdIE2(n,M,mm.p_offset,mm.p_last,true,mm.QQt,mm.QQi,mm.QQpi,mm.QQp2i,mm.QQp3i,QQp1t,QQp2t,QQp3t,0);
	pnldie2__nodI=PnLdIE2.get_PnLdIE2(n,M,mm.p_offset,mm.p_last,false,mm.QQt,mm.QQi,mm.QQpi,mm.QQp2i,mm.QQp3i,QQp1t,QQp2t,QQp3t,0);
    }

    this.sst=new State(rst);


}
MomentsData getMomentsData(final int n){
    return MomentsData.get_MomentsData(n,M,odata,getAccess());
}

abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double [] getEDPsi(final double [] psi);
abstract double [] getPsi2WIntegratedDt(final double [] psi);


static abstract class ImpactQVMTest<DTYPE> extends QTest {

abstract ImpactQVM<DTYPE> getImpactQVM();
abstract YpnlQVM<DTYPE> getYpnlQVM();
abstract QkQVM<DTYPE> getQkQVM();

ImpactQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_Extra(){}

void test_YpnlQVM()
{
    final int n=neq;
    final ImpactQVM<DTYPE> q=getImpactQVM();
    final YpnlQVM<DTYPE> s=getYpnlQVM();
    DynIVT.matchPnLDiagProj(q.pnldiag,s.ivt,eps);
    if(true)
    { // Test the QQV, QQT & QQpV matrices are the same.
	final MomentsData mm=q.getMomentsData(n);
	final EVXData VT=EVXData.getEVXDataFromQQ(n,TMatr.getVectorScaled(mm.QQV,-1),TMatr.getVectorScaled(mm.QQT,-1),q.M);
	EVXData.matchEVXData(VT,s._vt.VT,false,eps);
	final double p_V=VT.getFPsi(n-1,mm.QQpV)/VT.getFPsi(n-1,mm.QQV);
	// In monomials basis offset if different for YpnlQVM, skip the test
	// The YpnlQVM always have p_last as p_offset.
	final double p_offset_shift=mm.p_last;
	UnitTests.checkDoubleRel(p_V-p_offset_shift,"p_V",s._vt.p_V,"s._vt.p_V",eps);
    }

    System.err.println(name+":ImpactQVMTest.test_YpnlQVM(): test OK n="+n+" eps="+eps);
}
void test_QkQVM()
{
    final int n=neq;
    final ImpactQVM<DTYPE> q=getImpactQVM();
    final QkQVM<DTYPE> s=getQkQVM();
    Skewness.matchSkewness(q.pnldisk.SK_IH,s.aIb.SK,eps);
    Skewness.matchSkewness(q.pnldisk.SK_IHf__nodI,s.aFutureI.SK,eps);
    Skewness.matchSkewness(q.pnldisk.SK_IHf,s.aFutureIpl.SK,eps);

    System.err.println(name+":ImpactQVMTest.test_QkQVM(): test OK n="+n+" eps="+eps);
}
void test_ImpactQVM()
{
    final int n=neq;
    final ImpactQVM<DTYPE> q=getImpactQVM();
    final MomentsData mm=q.getMomentsData(n);
    final double [][] mimt=YpkQVM.get_PV_PT_Moments(2*n-1,q.odata,q.getAccess());
    final double [] p1tmom=mimt[1],QQp1t=q.M.B.getQQMatr(n,p1tmom);

    final ImpactExample tiex=ImpactExample.get_ImpactExample(n,mm.QQi,mm.QQt,mm.QQpi,mm.QQdp,QQp1t,q.M,q.VOLATILITY_EDPsi,mm.p_offset,eps);
    ImpactExample.matchImpactExample(tiex,q.iex,eps);
    ImpactExample.matchImpactIStatesExample(tiex,q.ist,eps);
    ImpactExample.matchImpactIStatesExample(tiex,ImpactIStatesExample.get_ImpactIStatesExample(mm,q.M,q.VOLATILITY_EDPsi,eps),eps);
    ImpactIStatesExample.matchPnLDiagProj(q.ist,PnLDiagProj.get_PnLDiagProj(mm,q.M,eps),eps);
    ImpactIStatesExample.matchPnLDiagProj(q.ist,q.pnldiag,eps);
    UnitTests.checkDoubleRel(q.pnldicorr_nodI.IHf_cc.SK_pt.xa,"q.pnldicorr_nodI.IHf_cc.SK_pt.xa,",q.iex.pt_IH,"q.iex.pt_IH",eps);
    UnitTests.checkDoubleRel(q.pnldicorr_nodI.IHf_cc.SK_pv.xa,"q.pnldicorr_nodI.IHf_cc.SK_pv.xa",q.iex.p_IH,"q.iex.p_IH",eps);

    PnLMu.matchPnLMu(q.pnlmuIL,PnLMu.get_PnLMu(mm,q.M,true,eps),eps);
    PnLMu.matchPnLMu(q.pnlmuIH,PnLMu.get_PnLMu(mm,q.M,false,eps),eps);
    PnLMu.matchPnLMuH(q.pnlmuIH,PnLMuH.get_PnLMuH(mm,q.M,eps),eps);
    PnLMu.matchPnLMuH(q.pnlmuIH,q.pnlmuh,eps);

    PnLVH.matchPnLVH(q.pnlVH,PnLVH.test_get_PnLVH(mm,q.M,eps),eps);
    PnLVH.matchDynVar2DiagProj(q.pnlVH,q.dvardiagproj,eps);
    PnLVH.matchDynVar2DiagProj(q.pnlVH,DynVar2DiagProj.get_DynVar2DiagProj(n,mm.QQi,mm.QQt,mm.QQpi,q.M,q.VOLATILITY_EDPsi,eps),eps);


    if(n>2) PnLI0.matchPnLI0(q.pnlI0,PnLI0.get_PnLI0(mm,q.M,eps),eps);
    PnLI0.matchPnLI0(q.bpnlI0,PnLI0.get_PnLI0_NO_0_atBoundary(mm,q.M,eps),eps);
    PnL2I0.matchPnL2I0(q.pnl2bI0,PnL2I0.test_get_PnL2I0(mm,PnL2I0.STDFACTIH_NUMSTABILITY,q.M,eps),eps);
    PnL2I0.matchPnL2I0(q.pnl2bI0,PnL2I0.test2_get_PnL2I0(mm,PnL2I0.STDFACTIH_NUMSTABILITY,q.M,eps),eps);
    PnL2I0.matchPnL2I0(q.pnl2iterI0,PnL2I0.test_get_iter_PnL2I0(mm,PnL2I0.STDFACTIH_NUMSTABILITY,PnL2I0.STDITERATIONS,q.M,eps),eps);
    PnL2I0.matchPnL2I0(q.pnl2iterI0,PnL2I0.test2_get_iter_PnL2I0(mm,PnL2I0.STDFACTIH_NUMSTABILITY,PnL2I0.STDITERATIONS,q.M,eps),eps);

    PnLbI0.matchPnLbI0(q.pnlbI0,PnLbI0.test_get_PnLbI0(mm,q.M,eps),eps);
    PnLbI0.matchPnLbI0(q.pnlbI0,PnLbI0.get_PnLbI0(mm,q.M,eps),eps);
    PnLbI0.matchPnLfIHI0(q.pnlbI0,q.pnlfihi0,eps);
    PnLbI0.matchPnLfIHI0(q.pnlbI0,PnLfIHI0.get_PnLfIHI0(mm,q.M,eps),eps);

    PnLfIHI0.matchPnLdIf0(q.pnlfihi0,q.pnldif0,eps);
    PnLfIHI0.matchPnLdIf0(q.pnlfihi0,PnLdIf0.get_PnLdIf0(mm,q.M,eps),eps);

    PnLdIf0.matchPnLdIfdyn(PnLdIf0.get_PnLdIf0(mm,q.M,eps,PnLdIDyn.IMPACTAPPROXIMATION_DYNAMIC_PURE),q.pnldifdyn,eps);
    PnLdIf0.matchPnLdIfdyn(PnLdIf0.get_PnLdIf0(mm,q.M,eps,PnLdIDyn.IMPACTAPPROXIMATION_DYNAMIC_PURE),PnLdIfdyn.get_PnLdIfdyn(mm,q.M,eps),eps);

    PnLI0.matchPnLI0e(q.bpnlI0,PnLI0e.get_PnLI0e(mm,q.M,false,eps),eps);
    PnLI0.matchPnLI0e(q.bpnlI0,PnLI0e.get_PnLI0e(mm,q.M,true,eps),eps);
    PnLI0e.matchPnLI0e(q.pnli0e,PnLI0e.test_get_PnLI0e(mm,q.M,false,eps),eps);
    PnLI0e.matchPnLI0e(q.pnli0e_lI,PnLI0e.test_get_PnLI0e(mm,q.M,true,eps),eps);

    PnLdIf0.matchPnLdIDyn(q.pnldif0,PnLdIDyn.get_PnLdIDyn(mm,q.M,eps,PnLdIDyn.IMPACTAPPROXIMATION_NONE),eps);
    PnLdIf0.matchPnLdIDyn(q.pnldif0_dyn_IH,PnLdIDyn.get_PnLdIDyn(mm,q.M,eps,PnLdIDyn.IMPACTAPPROXIMATION_DYNAMIC_IH),eps);
    PnLdIf0.matchPnLdIDyn(q.pnldif0_dyn_PURE,PnLdIDyn.get_PnLdIDyn(mm,q.M,eps,PnLdIDyn.IMPACTAPPROXIMATION_DYNAMIC_PURE),eps);

    PnLI0IH.matchPnLI0IH(q.pnli0ih,PnLI0IH.test_get_PnLI0IH(mm,q.M,eps),eps);
    PnLI0IH.matchPnLI0IH(PnLI0IH.get_PnLI0IH(mm,q.M,eps),PnLI0IH.test_get_PnLI0IH(mm,q.M,eps),eps);
    PnLI0B.get_PnLI0B(mm,q.M,eps);

    PnLdIf0.matchPnLdIdP(q.pnldif0,PnLdIdP.get_PnLdIdP(mm,q.M,eps),eps);
    ImpactExample.matchPnLdIdP(q.iex,PnLdIdP.get_PnLdIdP(mm,q.M,eps),eps);

    PnLdIdP.matchPnLdIdPRQ(q.pnldidp,PnLdIdPRQ.get_PnLdIdPRQ(mm,q.M,eps),eps);

    ImpactExample.matchPnLdID(q.iex,PnLdID.get_PnLdID(mm,q.M,q.VOLATILITY_EDPsi,eps),eps);
    PnLdIf0.matchPnLdIS(q.pnldif0,PnLdIS.get_PnLdIS(mm,q.M,eps),eps);
    PnLdIdP.matchPnLdISk(q.pnldidp,PnLdISk.get_PnLdISk(mm,q.M,eps),eps);

    PnLdICorr.matchPnLdISkewness(q.pnldicorr_nodI,q.pnldiskewness_nodI,eps);
    PnLdICorr.matchPnLdISkewness(q.pnldicorr,q.pnldiskewness,eps);

    PnLdICorr.matchPnLdIP(q.pnldicorr_nodI,q.pnldip_nodI,eps);
    PnLdICorr.matchPnLdIP(q.pnldicorr,q.pnldip,eps);

    PnLdISk.matchPnLdISkewness(q.pnldisk,q.pnldiskewness,eps);

    PnLdISkewness.matchPnLdISkewness(q.pnldiskewness_nodI,PnLdISkewness.get_PnLdISkewness(mm,q.M,false,eps),eps);
    PnLdISkewness.matchPnLdISkewness(q.pnldiskewness,PnLdISkewness.get_PnLdISkewness(mm,q.M,true,eps),eps);

    PnLdISkewness.matchPnLdIDSk(q.pnldiskewness,PnLdIDSk.get_PnLdIDSk(mm,q.M,eps),eps);
    PnLdISkewness.matchPnLdIDSk__nodI(q.pnldiskewness_nodI,q.pnldidsk,eps);
    PnLdISkewness.matchPnLdIdI(q.pnldiskewness,PnLdIdI.get_PnLdIdI(mm,q.M,eps),eps);

    PnLdISkewness.matchDynVar2DISkewness(q.pnldiskewness,DynVar2DISkewness.get_DynVar2DISkewness(mm,q.M,q.VOLATILITY_EDPsi,eps),eps);
    // unit test
    DynVar2RDISkewness.get_DynVar2RDISkewness(mm,q.M,q.VOLATILITY_EDPsi,false,eps);
    DynVar2RDISkewness.get_DynVar2RDISkewness(mm,q.M,q.VOLATILITY_EDPsi,true,eps);
    PnLdI2Skewness.get_PnLdI2Skewness(mm,q.M,q.VOLATILITY_EDPsi,eps);

    DynVar2DIDP.get_DynVar2DIDP(mm,q.M,q.VOLATILITY_EDPsi,false,eps);
    PnLdISkewness.matchDynVar2DIDP(q.pnldiskewness,DynVar2DIDP.get_DynVar2DIDP(mm,q.M,q.VOLATILITY_EDPsi,true,eps),eps);
    PnLdISkewness.matchDynVar2DIDP(q.pnldiskewness_nodI,DynVar2DIDP.get_DynVar2DIDP(mm,q.M,q.VOLATILITY_EDPsi,false,eps),eps);
    PnLdIDSk.matchDynVar2DIDP(q.pnldidsk,q.dv2didp,eps);
    PnLdIDSk.matchPnLdIpx(q.pnldidsk,q.pnldipx,eps);

    DynEVdI0.matchDynEVdI(q.dynevdi0,DynEVdI.get_DynEVdI(mm,q.M,eps),eps);
    DynEVdI0.get_DynEVdI0(mm,q.M,q.VOLATILITY_EDPsi,eps);
    PnLdIpx.get_PnLdIpx(mm,q.M,eps);
    PnLdIDSk.matchPnLdISpur(q.pnldidsk,PnLdISpur.get_PnLdISpur(mm,q.M,eps),eps);
    PnLVar3Mu.matchPnLVar3Mu(PnLVar3Mu.get_PnLVar3Mu(mm,q.M,eps),q.pnlvar3mu,eps);
    PnLFutureSk.get_PnLFutureSk(mm,q.M,eps);
    PnLSensitivity.get_PnLSensitivity(mm,q.M,PnLSensitivity.PE_PLAST,eps);
    PnLSensitivity.get_PnLSensitivity(mm,q.M,PnLSensitivity.PE_P0,eps);
    PnLSensitivity.get_PnLSensitivity(mm,q.M,PnLSensitivity.PE_PIH,eps);

    PnLdIDSk.matchMicroPnLdI(q.pnldidsk,MicroPnLdI.get_MicroPnLdI(mm,MicroPnLdI.G_PRICE,q.M,eps),eps);
    MicroPnLdI.get_MicroPnLdI(mm,MicroPnLdI.G_DPRICE,q.M,eps);
    MicroPnLdI.get_MicroPnLdI(mm,MicroPnLdI.G_DPRICEDT,q.M,eps);
    PnLVH.matchInIStateIHDensity(q.pnlVH,new InIStateIHDensity(n,mm.p_offset,mm.QQt,mm.QQi,mm.QQpi,mm.QQdp,q.M,q.VOLATILITY_EDPsi,q::getPsi2WIntegratedDt,eps),eps);

    System.err.println(name+":ImpactQVMTest.test_ImpactQVM(): test OK n="+n+" eps="+eps);
}


void test_VSk()
{
    final int n=neq;
    final ImpactQVM<DTYPE> q=getImpactQVM();
    final DataInterfaceToMoments<DTYPE> access=q.getAccess();
    final int nmoms=2*n-1+3;
    final double [][] mimt=YpkQVM.get_PV_PT_Moments(nmoms,q.odata,access);
    final double [] tmom=mimt[0],imom=mimt[4],p1imom=mimt[5],
	p2imom=mimt[6],p3imom=mimt[7],p4imom=new double [nmoms];
    access.setM(p4imom,4,access.M_I,q.odata,nmoms);
    /*
    final double [] tmom=new double [nmoms],imom=new double [nmoms],p1imom=new double [nmoms],
	p2imom=new double [nmoms],p3imom=new double [nmoms],p4imom=new double [nmoms];
    access.setM(imom,0,access.M_I,q.odata,nmoms);
    access.setM(p1imom,1,access.M_I,q.odata,nmoms);
    access.setM(p2imom,2,access.M_I,q.odata,nmoms);
    access.setM(p3imom,3,access.M_I,q.odata,nmoms);
    access.setM(p4imom,4,access.M_I,q.odata,nmoms);
    access.setMT(tmom,nmoms);
    */
    PnLdISk.matchPnLdIVSk(
	q.pnldisk,
	PnLdIVSk.get_PnLdIVSk(n,tmom,imom,p1imom,p2imom,p3imom,
			      access.getPOffset(q.odata),access.getPLast(q.odata),
			      (double [] pp) ->q.getPsi2WIntegratedDt(pp),q.M,eps),
	eps);
    final double [] QQt=q.M.B.getQQMatr(n,tmom),
	QQi=q.M.B.getQQMatr(n,imom),
	QQpi=q.M.B.getQQMatr(n,p1imom),
	QQp2i=q.M.B.getQQMatr(n,p2imom),
	QQp3i=q.M.B.getQQMatr(n,p3imom),
	QQp4i=q.M.B.getQQMatr(n,p4imom);
    PnLdISk.matchPnLdIV4(q.pnldisk,
			 PnLdIV4.get_PnLdIV4_QQ(n,access.getPOffset(q.odata),
						QQt,
						QQi,
						QQpi,
						QQp2i,
						QQp3i,
						QQp4i,
						true,
						access.getPLast(q.odata),q.M,eps),
			 PnLdIV4.get_PnLdIV4_QQ(n,access.getPOffset(q.odata),
						QQt,
						QQi,
						QQpi,
						QQp2i,
						QQp3i,
						QQp4i,
						false,
						access.getPLast(q.odata),q.M,eps),
			 eps);
    final double [] p1tmom=mimt[1],p2tmom=mimt[2],p3tmom=mimt[3],
	QQp1t=q.M.B.getQQMatr(n,p1tmom),QQp2t=q.M.B.getQQMatr(n,p2tmom),QQp3t=q.M.B.getQQMatr(n,p3tmom);
    if(true)
    {
	final double [] _tmom=mimt[8],_p1tmom=mimt[9],_p2tmom=mimt[10],_p3tmom=mimt[11];
	System.err.println(" p1tmom="+java.util.Arrays.toString(p1tmom));
	System.err.println("_p1tmom="+java.util.Arrays.toString(_p1tmom));
	System.err.println(" p2tmom="+java.util.Arrays.toString(p2tmom));
	System.err.println("_p2tmom="+java.util.Arrays.toString(_p2tmom));
	System.err.println(" p3tmom="+java.util.Arrays.toString(p3tmom));
	System.err.println("_p3tmom="+java.util.Arrays.toString(_p3tmom));
    }

    PnLdIV4.matchPnLdICorr(q.pnldiv4_nodI,
			   PnLdICorr.get_PnLdICorr(n,
						   access.getPOffset(q.odata),
						   tmom,p1tmom,p2tmom,p3tmom,
						   imom,p1imom,p2imom,p3imom,
						   false,
						   access.getPLast(q.odata),
						   q.M,eps),eps);
     PnLdIV4.matchPnLdICorr(q.pnldiv4_nodI_t,
			   PnLdICorr.get_PnLdICorr(n,
						   access.getPOffset(q.odata),
						   mimt[8],mimt[9],mimt[10],mimt[11],
						   imom,p1imom,p2imom,p3imom,
						   false,
						   access.getPLast(q.odata),
						   q.M,eps),eps);
    PnLdIV4.matchPnLdICorr(q.pnldiv4,
			   PnLdICorr.get_PnLdICorr(n,
						   access.getPOffset(q.odata),
						   tmom,p1tmom,p2tmom,p3tmom,
						   imom,p1imom,p2imom,p3imom,
						   true,
						   access.getPLast(q.odata),
						   q.M,eps),eps);
    PnLdIV4.matchPnLdICorr(q.pnldiv4_t,
			   PnLdICorr.get_PnLdICorr(n,
						   access.getPOffset(q.odata),
						   mimt[8],mimt[9],mimt[10],mimt[11],
						   imom,p1imom,p2imom,p3imom,
						   true,
						   access.getPLast(q.odata),
						   q.M,eps),eps);
    Skewness.matchSkewness(q.pnldisk.SK_movaver.getSameSkewnessXShifted(q.pnldisk.p_offset),q.pnldicorr_nodI.movaver_cc.SK_pv.getSameSkewnessXShifted(q.pnldicorr_nodI.p_offset),eps);

    final MomentsData mm=q.getMomentsData(n);
    PnLdICorr.matchPnLdIP(q.pnldicorr_nodI,PnLdIP.get_PnLdIP(mm,q.M,QQp1t,QQp2t,QQp3t,false,eps),eps);
    PnLdICorr.matchPnLdIP(q.pnldicorr,PnLdIP.get_PnLdIP(mm,q.M,QQp1t,QQp2t,QQp3t,true,eps),eps);
    PnLdIE.matchPnLdIP(PnLdIE.get_PnLdIE(n,q.M,mm.p_offset,mm.QQt,mm.QQi,mm.QQpi,mm.QQp2i,mm.QQp3i,QQp1t,QQp2t,QQp3t,eps),q.pnldip_nodI,eps);
    PnLdIE.matchPnLdISkewness(PnLdIE.get_PnLdIE_DYN(mm,q.M,true,eps),q.pnldiskewness,eps);
    PnLdIE.matchPnLdISkewness(PnLdIE.get_PnLdIE_DYN(mm,q.M,false,eps),q.pnldiskewness_nodI,eps);

    PnLdIE2.matchPnLdIDSk(PnLdIE2.get_PnLdIE2(n,q.M,mm.p_offset,mm.p_last,true,mm.QQt,mm.QQi,mm.QQpi,mm.QQp2i,mm.QQp3i,QQp1t,QQp2t,QQp3t,eps),q.pnldidsk,eps);
    PnLdIE2.matchPnLdIE(PnLdIE2.get_PnLdIE2(n,q.M,mm.p_offset,mm.p_last,false,mm.QQt,mm.QQi,mm.QQpi,mm.QQp2i,mm.QQp3i,QQp1t,QQp2t,QQp3t,eps),q.pnldie,eps);

    System.err.println(name+":ImpactQVMTest.test_VSk(): test OK n="+n+" eps="+eps);
}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":ImpactQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Extra();
    test_ImpactQVM();
    test_VSk();
    test_YpnlQVM();
    test_QkQVM();

    System.err.println(name+":ImpactQVMTest.test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_min,final int neq_max)
{
    System.err.println(name+":ImpactQVMTest: starting tests; ver=$Id: ImpactQVM.java,v 1.137 2022/07/15 19:57:10 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=neq_min;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":ImpactQVMTest.main: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: ImpactQVM.java,v 1.137 2022/07/15 19:57:10 mal Exp $");
}
} // END OF class ImpactQVMTest




}// END OF class ImpactQVM
