package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: ImpactQVMLaguerre.java,v 1.19 2022/07/10 10:41:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactQVMLaguerre extends ImpactQVM<QVMDataL> {
ImpactQVMLaguerre(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}


public static ImpactQVMLaguerre calculate(final int neq,
					  final long currenttime,
					  final long tau,
					  final Trade [] tb,
					  final int tsize,
					  final ImpactQVMLaguerre ref)
{
    final int npm=5,nr=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*(neq-1)+1+nr,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new ImpactQVMLaguerre(neq,o,ref!=null?ref.sst:null);
}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double [] getPsi2WIntegratedDt(final double [] psi){return WIntegratorLaguerre.getPsi2WIntegratedDt(psi);}
double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}

static class ImpactQVMLaguerreTest extends ImpactQVMTest<QVMDataL> {
ImpactQVMLaguerreTest(final long initrandom)
{
    super("ImpactQVMLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
ImpactQVMLaguerre getImpactQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
YpnlQVMLaguerre getYpnlQVM(){return YpnlQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
QkQVMLaguerre getQkQVM(){return QkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class ImpactQVMLaguerreTest

public static void main(String [] args)
{
    new ImpactQVMLaguerreTest(11127L).runTests(2,5);
} // END main



} // END OF class ImpactQVMLaguerre
