package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.Skewness;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: ImpactQVMMonomials.java,v 1.66 2022/11/16 14:34:11 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactQVMMonomials extends ImpactQVM<QVMData> {
ImpactQVMMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static ImpactQVMMonomials calculate(final int neq,
					   final long currenttime,
					   final long tau,
					   final Trade [] tb,
					   final int tsize,
					   final ImpactQVMMonomials ref)
{
    final int nvm=1,npm=5,nr=3;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1+nr,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new ImpactQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double [] getPsi2WIntegratedDt(final double [] psi){return WIntegratorMonomials.getPsi2WIntegratedDt(psi);}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}

static class ImpactQVMMonomialsTest extends ImpactQVMTest<QVMData> {
ImpactQVMMonomialsTest(final long initrandom)
{
    super("ImpactQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}



@Override
void test_Extra(){
    final ImpactQVMMonomials q=getImpactQVM();
    final ImpactQVMLaguerre s=getImpactQVMLaguerre();

    ImpactExample.matchImpactExample(s.iex,q.iex,eps);
    ImpactExample.matchImpactIStatesExample(q.iex,s.ist,eps);
    ImpactExample.matchImpactIStatesExample(s.iex,q.ist,eps);
    ImpactIStatesExample.matchImpactIStatesExample(q.ist,s.ist,eps);
    ImpactIStatesExample.matchPnLDiagProj(q.ist,q.pnldiag,eps);
    ImpactIStatesExample.matchPnLDiagProj(s.ist,q.pnldiag,eps);
    ImpactIStatesExample.matchPnLDiagProj(q.ist,s.pnldiag,eps);
    PnLMu.matchPnLMu(q.pnlmuIL,s.pnlmuIL,eps);
    PnLMu.matchPnLMu(q.pnlmuIH,s.pnlmuIH,eps);
    PnLMu.matchPnLMuH(q.pnlmuIH,s.pnlmuh,eps);
    PnLMu.matchPnLMuH(s.pnlmuIH,q.pnlmuh,eps);
    PnLVH.matchPnLVH(q.pnlVH,s.pnlVH,eps);
    if(neq>2) PnLI0.matchPnLI0(q.pnlI0,s.pnlI0,eps);
    PnLI0.matchPnLI0(q.bpnlI0,s.bpnlI0,eps);
    PnL2I0.matchPnL2I0(q.pnl2bI0,s.pnl2bI0,eps);
    PnL2I0.matchPnL2I0(q.pnl2iterI0,s.pnl2iterI0,eps);
    PnLbI0.matchPnLbI0(q.pnlbI0,s.pnlbI0,eps);
    PnLbI0.matchPnLfIHI0(q.pnlbI0,s.pnlfihi0,eps);
    PnLbI0.matchPnLfIHI0(s.pnlbI0,q.pnlfihi0,eps);
    PnLfIHI0.matchPnLdIf0(q.pnlfihi0,s.pnldif0,eps);
    PnLfIHI0.matchPnLdIf0(s.pnlfihi0,q.pnldif0,eps);
    PnLI0.matchPnLI0e(q.bpnlI0,s.pnli0e,eps);
    PnLI0.matchPnLI0e(s.bpnlI0,q.pnli0e,eps);

    PnLdIf0.matchPnLdIdP(q.pnldif0,s.pnldidp,eps);
    PnLdIf0.matchPnLdIdP(s.pnldif0,q.pnldidp,eps);
    ImpactExample.matchPnLdIdP(q.iex,s.pnldidp,eps);
    ImpactExample.matchPnLdIdP(s.iex,q.pnldidp,eps);
    PnLdIdP.matchPnLdIdPRQ(q.pnldidp,s.pnldidprq,eps);
    PnLdIdP.matchPnLdIdPRQ(s.pnldidp,q.pnldidprq,eps);

    ImpactExample.matchPnLdID(q.iex,s.pnldid,eps);
    ImpactExample.matchPnLdID(s.iex,q.pnldid,eps);

    PnLdIf0.matchPnLdIS(q.pnldif0,s.pnldis,eps);
    PnLdIf0.matchPnLdIS(s.pnldif0,q.pnldis,eps);
    PnLdIdP.matchPnLdISk(q.pnldidp,s.pnldisk,eps);
    PnLdIdP.matchPnLdISk(s.pnldidp,q.pnldisk,eps);

    PnLdISk.matchPnLdIVSk(s.pnldisk,q.pnldivsk,eps);
    PnLdISk.matchPnLdIVSk(q.pnldisk,s.pnldivsk,eps);
    PnLdISk.matchPnLdIV4(q.pnldisk,s.pnldiv4,s.pnldiv4_nodI,eps);
    PnLdISk.matchPnLdIV4(s.pnldisk,q.pnldiv4,q.pnldiv4_nodI,eps);
    PnLdISk.matchPnLdISkewness(q.pnldisk,s.pnldiskewness,eps);
    PnLdISk.matchPnLdISkewness(s.pnldisk,q.pnldiskewness,eps);

    PnLdIV4.matchPnLdICorr(q.pnldiv4_nodI,s.pnldicorr_nodI,eps);
    PnLdIV4.matchPnLdICorr(s.pnldiv4_nodI,q.pnldicorr_nodI,eps);
    PnLdIV4.matchPnLdICorr(q.pnldiv4,s.pnldicorr,eps);
    PnLdIV4.matchPnLdICorr(s.pnldiv4,q.pnldicorr,eps);

    PnLdICorr.matchPnLdICorr(q.pnldicorr_nodI,s.pnldicorr_nodI,eps);
    PnLdICorr.matchPnLdICorr(q.pnldicorr,s.pnldicorr,eps);
    PnLdICorr.matchPnLdISkewness(q.pnldicorr_nodI,s.pnldiskewness_nodI,eps);
    PnLdICorr.matchPnLdISkewness(q.pnldicorr,s.pnldiskewness,eps);

    PnLdISkewness.matchPnLdISkewness(q.pnldiskewness_nodI,s.pnldiskewness_nodI,eps);
    PnLdISkewness.matchPnLdISkewness(q.pnldiskewness,s.pnldiskewness,eps);
    PnLdISkewness.matchPnLdIDSk(q.pnldiskewness,s.pnldidsk,eps);
    PnLdISkewness.matchPnLdIDSk(s.pnldiskewness,q.pnldidsk,eps);
    PnLdISkewness.matchPnLdIDSk__nodI(q.pnldiskewness_nodI,s.pnldidsk,eps);
    PnLdISkewness.matchPnLdIDSk__nodI(s.pnldiskewness_nodI,q.pnldidsk,eps);
    PnLdISkewness.matchPnLdIdI(q.pnldiskewness,s.pnldidi,eps);
    PnLdISkewness.matchPnLdIdI(s.pnldiskewness,q.pnldidi,eps);
    PnLdISkewness.matchDynVar2DISkewness(q.pnldiskewness,s.dv2diskewness,eps);
    PnLdISkewness.matchDynVar2DISkewness(s.pnldiskewness,q.dv2diskewness,eps);
    PnLdISkewness.matchDynVar2DIDP(q.pnldiskewness,s.dv2didp,eps);
    PnLdISkewness.matchDynVar2DIDP(s.pnldiskewness,q.dv2didp,eps);
    PnLdISkewness.matchDynVar2DIDP(q.pnldiskewness_nodI,s.dv2didp_nodI,eps);
    PnLdISkewness.matchDynVar2DIDP(s.pnldiskewness_nodI,q.dv2didp_nodI,eps);

    PnLdIDSk.matchDynVar2DIDP(q.pnldidsk,s.dv2didp,eps);
    PnLdIDSk.matchDynVar2DIDP(s.pnldidsk,q.dv2didp,eps);
    PnLdIDSk.matchPnLdIpx(q.pnldidsk,s.pnldipx,eps);
    PnLdIDSk.matchPnLdIpx(s.pnldidsk,q.pnldipx,eps);
    DynEVdI0.matchDynEVdI(q.dynevdi0,s.dynevdi,eps);
    DynEVdI0.matchDynEVdI(s.dynevdi0,q.dynevdi,eps);
    PnLdIDSk.matchPnLdISpur(q.pnldidsk,s.pnldispur,eps);
    PnLdIDSk.matchPnLdISpur(s.pnldidsk,q.pnldispur,eps);

    PnLdIE.matchPnLdIP(q.pnldie,s.pnldip_nodI,eps);
    PnLdIE.matchPnLdIP(s.pnldie,q.pnldip_nodI,eps);
    PnLdIE.matchPnLdISkewness(q.pnldie_dyn,s.pnldiskewness,eps);
    PnLdIE.matchPnLdISkewness(q.pnldie_dyn__nodI,q.pnldiskewness_nodI,eps);

    PnLdIE2.matchPnLdIDSk(q.pnldie2,s.pnldidsk,eps);
    PnLdIE2.matchPnLdIDSk(s.pnldie2,q.pnldidsk,eps);
    PnLdIE2.matchPnLdIE(q.pnldie2__nodI,s.pnldie,eps);
    PnLdIE2.matchPnLdIE(s.pnldie2__nodI,q.pnldie,eps);

    PnLSimpleSkewness.matchPnLSimpleSkewness(q.pnlss,s.pnlss,eps);
    PnLVar3Mu.matchPnLVar3Mu(q.pnlvar3mu,s.pnlvar3mu,eps);
    PnLdIDSk.matchMicroPnLdI(q.pnldidsk,s.mpnlP,eps);
    InIStateIHDensity.matchInIStateIHDensity(q.inistateihdensity,s.inistateihdensity,eps);
    InIStateIHDensity.matchInIStateIHDensity(s.inistateihdensity,q.inistateihdensity,eps);

    System.err.println(name+":ImpactQVMMonomialsTest:test_Extra(): test ImpactQVMLaguerre OK neq="+neq+" eps="+eps);
}


ImpactQVMLaguerre getImpactQVMLaguerre(){return ImpactQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
ImpactQVMMonomials getImpactQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
YpnlQVMMonomials getYpnlQVM(){return YpnlQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}
QkQVMMonomials getQkQVM(){return QkQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}


} // END of class ImpactQVMMonomialsTest

public static void main(String [] args)
{
    new ImpactQVMMonomialsTest(1127L).runTests(2,5);
} // END main



} // END OF class ImpactQVMMonomials
