package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.ProbabilityCorrelation;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: ImpactQVMMuseMonomials.java,v 1.9 2022/07/15 19:57:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactQVMMuseMonomials extends ImpactQVMMuse<QVMData> {
ImpactQVMMuseMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static ImpactQVMMuseMonomials calculate(final int neq,
					   final long currenttime,
					   final long tau,
					   final Trade [] tb,
					   final int tsize,
					   final ImpactQVMMuseMonomials ref)
{
    final int nvm=1,npm=4,nr=0;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1+nr,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new ImpactQVMMuseMonomials(neq,o,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}

static class ImpactQVMMuseMonomialsTest extends ImpactQVMMuseTest<QVMData> {
ImpactQVMMuseMonomialsTest(final long initrandom)
{
    super("ImpactQVMMuseMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}



@Override
void test_Extra(){
    // These to be identical
    final ImpactQVMMuseMonomials q=getImpactQVMMuse();
    final ImpactQVMMuseLaguerre s=getImpactQVMMuseLaguerre();

    Skewness.matchSkewness(q.SK_P_IH.getSameSkewnessXShifted(q.p_offset),s.SK_P_IH.getSameSkewnessXShifted(s.p_offset),eps);
    PnLSimpleSkewness.matchPnLSimpleSkewness(q.pnlss,s.pnlss,eps);

    System.err.println(name+":ImpactQVMMuseMonomialsTest:test_Extra(): test ImpactQVMMuseLaguerre OK neq="+neq+" eps="+eps);
}


ImpactQVMMuseLaguerre getImpactQVMMuseLaguerre(){return ImpactQVMMuseLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
ImpactQVMMuseMonomials getImpactQVMMuse(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
ImpactQVMMonomials getImpactQVM(){return ImpactQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}


} // END of class ImpactQVMMuseMonomialsTest

public static void main(String [] args)
{
    new ImpactQVMMuseMonomialsTest(11127L).runTests(2,5);
} // END main



} // END OF class ImpactQVMMuseMonomials
