package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.ProbabilityCorrelation;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: ImpactQVMMuse_pi.java,v 1.13 2022/07/15 19:57:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ImpactQVMMuse_pi extends ImpactQVMMuse<QVMDataL> {
ImpactQVMMuse_pi(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}


public static ImpactQVMMuse_pi calculate(final int neq,
					   final long currenttime,
					   final long tau,
					   final Trade [] tb,
					   final int tsize,
					   final ImpactQVMMuse_pi ref)
{
    final int ntm=1,npm=2*neq+2;
    final QVMDataL o=QVMDataL.calculateQVMDataL(ntm,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new ImpactQVMMuse_pi(neq,o,ref!=null?ref.sst:null);
}
@Override
protected MomentsData getMomentsData(final int n){return MomentsData.get_PriceMonomials_MomentsData(n,odata,getAccess(),getABasis());}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){
    final QVMDataLDirectAccess access=getAccess();
    return new OrthogonalPolynomialsMonomialsBasisX(1,-access.getPLast(odata)); // getPLast does have p_offset included
}

static class ImpactQVMMuse_piTest extends ImpactQVMMuseTest<QVMDataL> {
ImpactQVMMuse_piTest(final long initrandom)
{
    super("ImpactQVMMuse_pi",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

QkPnLpi_PBasis getQkPnLpi_PBasis(){return QkPnLpi_PBasis.calculate(neq,currenttime,tau,tb,tb.length,null);}

@Override
void test_Extra(){
    final ImpactQVMMuse_pi q=getImpactQVMMuse();
    final QkPnLpi_PBasis s=getQkPnLpi_PBasis();

    EVXData.matchEVXData(q.I,s.pnldiskewness_nodI.I,false,eps);
    Skewness.matchSkewness(q.SK_P_IH.getSameSkewnessXShifted(q.p_offset),s.pnldiskewness_nodI.IHf_S.getSameSkewnessXShifted(s.pnldiskewness_nodI.p_offset),eps);
    Skewness.matchSkewness(q.pnlss.SK_P_average.getSameSkewnessXShifted(q.p_offset),s.pnldiskewness_nodI.movaver_S.getSameSkewnessXShifted(s.pnldiskewness_nodI.p_offset),eps);
    ProbabilityCorrelation.matchProbabilityCorrelationWithOffset(q.pnlss.PCORR_average,q.pnlss.p_offset,s.PCORR_average,s.pnldiskewness_nodI.p_offset,eps);
//    PnLdISkewness.matchPnLdIDSk(s.pnldiskewness,PnLdIDSk.get_PnLdIDSk(q.getMomentsData(neq),q.M,eps),eps);
    PnLdISkewness.matchPnLdIDSk(s.pnldiskewness,q.pnldidsk,eps);
    System.err.println(name+":ImpactQVMMuse_piTest:test_Extra(): test ImpactQVMMuse_pi OK neq="+neq+" eps="+eps);
}


ImpactQVMMuse_pi getImpactQVMMuse(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
ImpactQVM<QVMDataL> getImpactQVM(){return null; /* Skip this test. */}


} // END of class ImpactQVMMuse_piTest

public static void main(String [] args)
{
    new ImpactQVMMuse_piTest(11127L).runTests(2,5);
} // END main



} // END OF class ImpactQVMMuse_pi
