package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/**
 * dI/dt=d^2V/dt^2 operator eigenstates:
 * |dI/dt|psi>=lambda|psi>.
 *  @version $Id: InDIStateMean.java,v 1.9 2022/07/01 17:28:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class InDIStateMean {
public final EVXData I,DI;
final double p_average,dp_withDIaverage,dpdt_withDIaverage,dpdta_withDIaverage;
/** Consider the I(p) in the state of dI/dt.
 * @param mm Input matrices.
 * @param M Basis operator.
 * @param VOLATILITY_EDPsi Volatility operator.
 * @param eps Run selftest with this eps, if positive.
 */
public InDIStateMean(final IMomentsData mm,
		     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		     final VolMatrix VOLATILITY_EDPsi,
		     final double eps)
{
    final int n=mm.n;
    this.I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final boolean flag_mean=true;
    final double I_0=I.s0,I_0_feq=flag_mean?mm.QQi[0]/mm.QQt[0]:I_0;
    final double [] QQ0=TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),
	QQdi=TMatr.sumaAbB(I_0_feq,QQ0,-1,VOLATILITY_EDPsi.getDiffMatrix(n,mm.QQi));
    DI=EVXData.getEVXDataFromQQ(n,QQdi,mm.QQt,M);
    final double [] _psiaverage=DI.getPsiAveraged(mm.QQt,M),taudis=TMatr.squareArrayElements(_psiaverage),p=new double [n],dpdt=new double [n];
    for(int k=0;k<n;k++)
    {
	p[k]=DI.getFPsi(k,mm.QQpi)/DI.getFPsi(k,mm.QQi);
	dpdt[k]=DI.getFPsi(k,mm.QQdpdt);
    }
    if(eps>0)
    {
	UnitTests.checkDoubleRel(QQdi[0],"QQdi[0]",0,"0",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(taudis),"sum taudis",mm.QQt[0],"mm.QQt[0]",eps);
	UnitTests.checkDoubleRel(TMatr.mulVVn1(taudis,DI.ev),"sum DI.ev taudis",0 /*QQdi[0]*/,"QQdi[0]",eps);
    }
    p_average=TMatr.mulVV(p,taudis)/TMatr.sumElements(taudis);
    dp_withDIaverage=TMatr.mulVVV(taudis,DI.getEValues(),p);
    dpdt_withDIaverage=TMatr.mulVVV(taudis,DI.getEValues(),dpdt);
    dpdta_withDIaverage=TMatr.mulVVV(TMatr.squareArrayElements(taudis),DI.getEValues(),dpdt);

    if(eps>0)
    { // test a model matrix with |psi0> being an eigenvector

	System.err.println("InDIStateMean: dI spectrum="+java.util.Arrays.toString(DI.getEValues())+" taudis="+java.util.Arrays.toString(taudis)+" sum(DI.ev taudis)="+TMatr.mulVVn1(taudis,DI.ev));
	System.err.println("InDIStateMean: test OK n="+n+" eps="+eps);
    }
}

}
