package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;

/**
 * Consider I=dV/dt and (p-p_{IH})I operators eigenstates.
 *  @version $Id: InIStatePIo.java,v 1.16 2022/08/05 11:25:32 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class InIStatePIo {
public final EVXData I;
public final Skewness PnLskL;
public final double p_offset,p_last__with_p_offset_included,p_IH;

public InIStatePIo(final int n,final double p_offset,final double p_last__with_p_offset_included,
		   final double [] QQt,final double [] QQi,final double [] QQpi,
		   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		   final double eps)
{
    if(n<=1) throw new IllegalArgumentException();
    this.p_offset=p_offset;
    this.p_last__with_p_offset_included=p_last__with_p_offset_included;
    final double Pf=p_last__with_p_offset_included;
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int iH=n-1; // the state of max I
    // Convert to the basis of I=dV/dt operator (just for convenience).
    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev);
    p_IH=bQQpi[iH+iH*n]/I.ev[iH];
    // not very successful approach
    final double []  bQQi=TMatr.getDiagMatrix(I.getEValues()),bQQPnL=TMatr.sumAcB(bQQpi,-Pf,bQQi),
	evPnLbasis=M.EV.getEV(n,bQQPnL,true),wk=new double [n],pnlk=new double [n];
    for(int k=0;k<n;k++)
    {
	wk[k]=Math.pow(evPnLbasis[n+k*n+iH],2); // <pisH|psik>^2
	pnlk[k]=evPnLbasis[k]; // <psik|PnL|psik>
    }
    PnLskL=Skewness.getSkewnessFromMonomialMoments(
	TMatr.sumElements(wk),
	TMatr.mulVV(pnlk,wk),
	TMatr.mulV2V(pnlk,wk),
	TMatr.mulVVV(pnlk,TMatr.squareArrayElements(pnlk),wk));
    if(eps>0)
    {
	UnitTests.checkDoubleRel(TMatr.mulVV(wk,pnlk),"sum(wk,pnlk)",(p_IH-Pf)*I.sH,"(p_IH-Pf)*I.sH",eps);
	UnitTests.checkDoubleRel(PnLskL.xa,"PnskL.xa",(p_IH-Pf)*I.sH,"(p_IH-Pf)*I.sH",eps);
	UnitTests.checkDoubleRel(bQQPnL[iH+n*iH],"bQQPnL[iH+n*iH]",(p_IH-Pf)*I.sH,"(p_IH-Pf)*I.sH",eps);
	System.err.println("InIStatePIo: I.s0="+I.s0+" PnLskL.xa="+PnLskL.xa);
    }
}

}
