package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.ProbabilityCorrelation;

/**
 * Consider I=dV/dt and V operator eigenstates.
 *  @version $Id: InIStateV.java,v 1.45 2022/07/28 22:30:49 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class InIStateV {
public final EVXData I,V;
public final Skewness sk_V,sk_VV,sk_VIH;
public final double p_offset,dV_shares,p_IH,Dpw_dir,Dp_dir,dpw_dir,dp_dir,pt_IX,p_IX,I_IX,Dpw_VV,Dp_VV,
    dpw_V1,dp_V1,pi_V1,V_V1,dpw_V2,dp_V2,pi_V2,V_V2,Vs,dVs,
    V_IH,V_VIH,p_VIH,Dp_VIH,Dpw_VIH,picorr_VIH,pVcorr_VIH,
    degeneracyVdiag,p_IHVdiag,pp_IHVdiag,dirpicorr,cvLcorr,cvIcorr,pi_Imaxv,pV_Imaxv;
public final int ind_ev;
public final Skewness [] skV;

public InIStateV(final IMomentsData mm,
		 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		 final double eps)
{
    final int n=mm.n;final boolean flag_Add_Impact_From_The_future=true;
    if(n<=1) throw new IllegalArgumentException();
    this.p_offset=mm.p_offset;
    this.I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH;
    final double I_0=I.s0,I_0_feq=flag_Add_Impact_From_The_future?I.sH:I_0,dI=I_0_feq-I_0,dV=dI/I.o_o;
    this.dV_shares=dV;
    this.V=EVXData.getEVXDataFromQQ(n,mm.QQV,mm.QQt,M);
    // scan back to find first eigenstate lower than -dV
    int ix=n-1;
    for(int k=n;--k>=0;)
    {
	ix=k;
	// V is defined to have negative sign and V=0 at x=x0.
	if(V.ev[k] <= -dV) break;
    }
    this.ind_ev=ix;
    final double [] psi=V.getPsi(this.ind_ev);
    Dpw_dir=TMatr.getAMatrA(psi,mm.QQDpV,n);
    Dp_dir=TMatr.getAMatrA(psi,mm.QQDp,n);
    dpw_dir=TMatr.getAMatrA(psi,mm.QQDpwv,n);
    dp_dir=TMatr.getAMatrA(psi,mm.QQdpdt,n);
    I_IX=TMatr.getAMatrA(psi,mm.QQi,n);
    p_IX=TMatr.getAMatrA(psi,mm.QQpi,n)/I_IX;
    pt_IX=TMatr.getAMatrA(psi,mm.QQpt_sample,n)/TMatr.getAMatrA(psi,mm.QQt_sample,n);
    sk_V=Skewness.getSkewnessFromMonomialMoments(
	-TMatr.getAMatrA(psi,mm.QQV,n),
	-TMatr.getAMatrA(psi,mm.QQpV,n),
	-TMatr.getAMatrA(psi,mm.QQp2V,n),
	-TMatr.getAMatrA(psi,mm.QQp3V,n));
    final int ie=n-1,iee=ie-1;
    sk_VV=Skewness.getSkewnessFromMonomialMoments(
	V.getFPsi(ie,mm.QQV)-V.getFPsi(iee,mm.QQV),
	V.getFPsi(ie,mm.QQpV)-V.getFPsi(iee,mm.QQpV),
	V.getFPsi(ie,mm.QQp2V)-V.getFPsi(iee,mm.QQp2V),
	V.getFPsi(ie,mm.QQp3V)-V.getFPsi(iee,mm.QQp3V));
    Dpw_VV=V.getFPsi(ie,mm.QQDpV)-V.getFPsi(iee,mm.QQDpV);
    Dp_VV=V.getFPsi(ie,mm.QQDp)-V.getFPsi(iee,mm.QQDp);
    dpw_V1=V.getFPsi(ie,mm.QQDpwv);
    dp_V1=V.getFPsi(ie,mm.QQdpdt);
    pi_V1=V.getFPsi(ie,mm.QQpi)/V.getFPsi(ie,mm.QQi);
    V_V1=V.ev[ie];
    dpw_V2=V.getFPsi(iee,mm.QQDpwv);
    dp_V2=V.getFPsi(iee,mm.QQdpdt);
    pi_V2=V.getFPsi(iee,mm.QQpi)/V.getFPsi(iee,mm.QQi);
    V_V2=V.ev[iee];
    skV=new Skewness [n];
    for(int k=0;k<n;k++)
    {
	skV[k]=Skewness.getSkewnessFromMonomialMoments(
	    -V.getFPsi(k,mm.QQV),
	    -V.getFPsi(k,mm.QQpV),
	    -V.getFPsi(k,mm.QQp2V),
	    -V.getFPsi(k,mm.QQp3V));
    }
    double sV=0,sVd=0;
    for(int k=0;k<n;k++)
    {
	final Skewness sk=skV[k];
	sV+=(sk.w1+sk.w2)*Math.pow(V.psik_at_x0[k],2);
	sVd+=(sk.w1-sk.w2)*Math.pow(V.psik_at_x0[k],2);
    }
    this.Vs=sV;
    this.dVs=sVd;

    V_IH=TMatr.getAMatrA(I.psiH,mm.QQV,n);
    ix=n-1;
    for(int k=n;--k>=0;)
    {
	ix=k;
	// V is defined to have negative sign and V=0 at x=x0.
	if(V.ev[k] <= V_IH) break;
    }
    final double [] psiI=I.psiH,psiV=V.getPsi(ix);
    V_VIH=V.ev[ix];
    p_VIH=V.getFPsi(ix,mm.QQpi)/V.getFPsi(ix,mm.QQi);
    Dp_VIH=V.getFPsi(ix,mm.QQDp)-TMatr.getAMatrA(I.psiH,mm.QQDp,n);
    Dpw_VIH=V.getFPsi(ix,mm.QQDpV)-TMatr.getAMatrA(I.psiH,mm.QQDpV,n);
    sk_VIH=Skewness.getSkewnessFromMonomialMoments(
	-(V.getFPsi(ix,mm.QQV)-TMatr.getAMatrA(I.psiH,mm.QQV,n)),
	-(V.getFPsi(ix,mm.QQpV)-TMatr.getAMatrA(I.psiH,mm.QQpV,n)),
	-(V.getFPsi(ix,mm.QQp2V)-TMatr.getAMatrA(I.psiH,mm.QQp2V,n)),
	-(V.getFPsi(ix,mm.QQp3V)-TMatr.getAMatrA(I.psiH,mm.QQp3V,n)));
    // try probability correlation
    final double [] Mt=new double []{TMatr.getAMatrB(psiI,psiI,mm.QQt,n),
				     TMatr.getAMatrB(psiI,psiV,mm.QQt,n),
				     TMatr.getAMatrB(psiV,psiI,mm.QQt,n),
				     TMatr.getAMatrB(psiV,psiV,mm.QQt,n)},
	Mp=new double []{TMatr.getAMatrB(psiI,psiI,mm.QQDp,n),
			 TMatr.getAMatrB(psiI,psiV,mm.QQDp,n),
			 TMatr.getAMatrB(psiV,psiI,mm.QQDp,n),
			 TMatr.getAMatrB(psiV,psiV,mm.QQDp,n)},
	Mi=new double []{TMatr.getAMatrB(psiI,psiI,mm.QQi,n),
			 TMatr.getAMatrB(psiI,psiV,mm.QQi,n),
			 TMatr.getAMatrB(psiV,psiI,mm.QQi,n),
			 TMatr.getAMatrB(psiV,psiV,mm.QQi,n)},
	MV=new double []{TMatr.getAMatrB(psiI,psiI,mm.QQV,n),
			 TMatr.getAMatrB(psiI,psiV,mm.QQV,n),
			 TMatr.getAMatrB(psiV,psiI,mm.QQV,n),
			 TMatr.getAMatrB(psiV,psiV,mm.QQV,n)};
    // for such a basis skewness-like is incorrect (Q[0] is not a constant), put probability correlation is correct.
	picorr_VIH=new ProbabilityCorrelation(Mt,Mp,Mi,M,true).corr;
	pVcorr_VIH=new ProbabilityCorrelation(Mt,Mp,MV,M,true).corr;
    int indkmaxI=0;
    double sidiag=0,spidiag=0,swdiag=0,si0diag=0,Imaxv=0;
    for(int k=0;k<n;k++)
    {
	final double wk=Math.pow(V.getFPsiPsi(k,I.psiH,mm.QQt),2),
	    ik=V.getFPsi(k,mm.QQi),pik=V.getFPsi(k,mm.QQpi);
	swdiag+=wk;
	sidiag+=wk*ik;
	spidiag+=wk*pik;
	si0diag+=Math.pow(V.getFPsiPsi(k,I.psi0,mm.QQt),2)*(V.getFPsi(k,mm.QQi));
	if(ik>=Imaxv){
	    Imaxv=ik;indkmaxI=k;
	}
    }
    pi_Imaxv=V.getFPsi(indkmaxI,mm.QQpi)/V.getFPsi(indkmaxI,mm.QQi);
    pV_Imaxv=V.getFPsi(indkmaxI,mm.QQpV)/V.getFPsi(indkmaxI,mm.QQV);
    if(eps>0){
	System.err.println("InIStateV: I.s0="+I.s0+" si0diag="+si0diag);
    }
    final double piH=TMatr.getAMatrA(I.psiH,mm.QQpi,n);
    degeneracyVdiag=1-sidiag/I.sH;
    p_IHVdiag=spidiag/sidiag;
    pp_IHVdiag=(piH-spidiag)/(I.sH-sidiag);
    double spcorr=0,sw=0,cvL=0,cvI=0;
    for(int k=0;k<n-2;k++)
    {
	final double [] mi=new double []{V.getFPsiPsi(k,k,mm.QQi),
					 V.getFPsiPsi(k,k+1,mm.QQi),
					 V.getFPsiPsi(k+1,k,mm.QQi),
					 V.getFPsiPsi(k+1,k+1,mm.QQi)},
	    mV=new double []{V.getFPsiPsi(k,k,mm.QQV),
			     V.getFPsiPsi(k,k+1,mm.QQV),
			     V.getFPsiPsi(k+1,k,mm.QQV),
			     V.getFPsiPsi(k+1,k+1,mm.QQV)},
	    mt=new double []{V.getFPsiPsi(k,k,mm.QQt),
			     V.getFPsiPsi(k,k+1,mm.QQt),
			     V.getFPsiPsi(k+1,k,mm.QQt),
			     V.getFPsiPsi(k+1,k+1,mm.QQt)},
	    mp=new double []{V.getFPsiPsi(k,k,mm.QQDp),
			     V.getFPsiPsi(k,k+1,mm.QQDp),
			     V.getFPsiPsi(k+1,k,mm.QQDp),
			     V.getFPsiPsi(k+1,k+1,mm.QQDp)};
	    final ProbabilityCorrelation c=new ProbabilityCorrelation(mt,mp,mi,M,true),cV=new ProbabilityCorrelation(mt,mp,mV,M,true);
	    cvL=cV.corr;
	    sw+=(c.pH-c.pL)*(c.iH-c.iL);
	    spcorr+=c.corr*(c.pH-c.pL)*(c.iH-c.iL);
	    if(k==ix)
	    {
		cvI=cV.corr;
	    }
	    if(eps>0){
		System.err.println("InIStateV: mt="+java.util.Arrays.toString(mt)+" mV="+java.util.Arrays.toString(mV));
	    }
    }
    dirpicorr=spcorr/sw;
    cvLcorr=cvL;
    cvIcorr=cvI;
    if(eps>0)
    {
	UnitTests.checkDoubleRel(swdiag,"swdiag",1,"1",eps);
	final int d=2;
	final double [] evp=M.EV.getGEV_Generalized(d,Mp,Mt,true),
	    evi=M.EV.getGEV_Generalized(d,Mi,Mt,true);
	final double q00=TMatr.getAMatrB(d,evi,d+0*d,evp,d+0*d,Mt,d),
	    q01=TMatr.getAMatrB(d,evi,d+0*d,evp,d+1*d,Mt,d),
	    q10=TMatr.getAMatrB(d,evi,d+1*d,evp,d+0*d,Mt,d),
	    q11=TMatr.getAMatrB(d,evi,d+1*d,evp,d+1*d,Mt,d);
	final double pcorr_VIH=((Math.pow(q00,2)+Math.pow(q11,2))-(Math.pow(q01,2)+Math.pow(q10,2)))/d;
	UnitTests.checkDoubleRel(pcorr_VIH,"pcorr_VIH",picorr_VIH,"picorr_VIH",eps);
    }

    if(eps>0)
    {
	UnitTests.checkDoubleRel(sV,"sV",TMatr.sumElements(I.getEValues()),"spur(I.ev)",eps);
	final double v_ix_psi02=(-V.ev[ind_ev])*Math.pow(M.B.getD0(psi,I.x0),2);
	UnitTests.checkDoubleRel(I_IX,"I_IX",v_ix_psi02,"v_ix_psi02",eps);
	System.err.println("InIStateV: I-spectrum="+java.util.Arrays.toString(I.getEValues())+" V-spectrum="+java.util.Arrays.toString(V.getEValues())+" I at V="+java.util.Arrays.toString(TMatr.scaleVectorInPlace(TMatr.getVAB(TMatr.squareArrayElements(V.psik_at_x0),V.getEValues()),-1))+" I_IX="+I_IX+" dV_shares="+dV_shares);
	matchInIStateVo(this,new InIStateVo(n,mm.p_offset,mm.QQt,mm.QQi,mm.QQpi,mm.QQV,mm.QQpV,mm.QQp2V,mm.QQp3V,M),eps);
	System.err.println("InIStateV: test OK n="+n+" dI="+(I_0_feq-I.s0)+" dV_shares="+dV_shares+" eps="+eps);
    }
}

static void matchInIStateVo(final InIStateV a,final InIStateVo b,final double eps)
{

    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.V_IH,"a.V_IH",b.V_IH,"b.V_IH",eps);
    UnitTests.checkDoubleRel(a.V_VIH,"a.V_VIH",b.V_VIH,"b.V_VIH",eps);
    Skewness.matchSkewness(a.sk_VIH.getSameSkewnessXShifted(a.p_offset),b.sk_VIH.getSameSkewnessXShifted(b.p_offset),eps);

    System.err.println("InIStateV.matchInIStateVo test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}

}
