package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;

/**
 * Consider I=dV/dt and V operator eigenstates. Optimized version.
 *  @version $Id: InIStateVo.java,v 1.12 2020/09/27 20:33:15 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class InIStateVo {
public final EVXData I;
public final Skewness sk_VIH;
public final double p_offset,p_IH,V_IH,V_VIH;

public InIStateVo(final int n,final double p_offset,
		  final double [] QQt,final double [] QQi,final double [] QQpi,
		  final double [] QQV,final double [] QQpV,final double [] QQp2V,final double [] QQp3V,
		  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    //if(n<=1) throw new IllegalArgumentException();
    this.p_offset=p_offset;
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    // Convert to the basis of I=dV/dt operator (just for convenience).
    final double [] bQQV=EVXData.convertMatrFromQtoB_Basis(n,QQV,I.ev),
	bQQpV=EVXData.convertMatrFromQtoB_Basis(n,QQpV,I.ev),
	bQQp2V=EVXData.convertMatrFromQtoB_Basis(n,QQp2V,I.ev),
	bQQp3V=EVXData.convertMatrFromQtoB_Basis(n,QQp3V,I.ev);
    final int iI=n-1; // the state of max I
    this.p_IH=I.getFPsi(iI,QQpi)/I.ev[iI];

    this.V_IH=bQQV[iI+n*iI];
    final double [] bVbasis=M.EV.getEV(n,bQQV,true);
    int iV=n-1;
    for(int k=n;--k>=0;)
    {
	iV=k;
	// V is defined to have negative sign and V=0 at x=x0.
	if(bVbasis[k] <= V_IH) break;
    }
    this.V_VIH=bVbasis[iV];
    // Now the iV is the first V state before the max I state.
    this.sk_VIH=Skewness.getSkewnessFromMonomialMoments(
	-(TMatr.getAMatrA(n,bVbasis,n+iV*n,bQQV,n)-bQQV[iI+n*iI]),
	-(TMatr.getAMatrA(n,bVbasis,n+iV*n,bQQpV,n)-bQQpV[iI+n*iI]),
	-(TMatr.getAMatrA(n,bVbasis,n+iV*n,bQQp2V,n)-bQQp2V[iI+n*iI]),
	-(TMatr.getAMatrA(n,bVbasis,n+iV*n,bQQp3V,n)-bQQp3V[iI+n*iI]));
}
}
