package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Try  pV/V states. These are the states, where the price is equal to moving average.
 * @version $Id: InIstatesPV.java,v 1.6 2020/09/27 20:33:15 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class InIstatesPV {
public final double
    Imax,pv_Imax,pi_Imax,pv_distmin,var2pV
	;
public final EVXData PV;

public InIstatesPV(final int n,
		   final double [] QQt,final double [] QQT,
		   final double [] QQV,final double [] QQpV,
		   final double [] QQi,final double [] QQpi,
		   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		   final VolMatrix VOLATILITY_EDPsi,
		   final double eps)
{
    this.PV=EVXData.getEVXDataFromQQ(n,QQpV,QQV,M);
    if(eps>0){
	// in this state average and local prices are equal
	for(int k=0;k<n;k++){
	    final double [] psi=PV.getPsi(k);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,QQpi,n)/TMatr.getAMatrA(psi,QQi,n),"pi",TMatr.getAMatrA(psi,QQpV,n)/TMatr.getAMatrA(psi,QQV,n),"pV",eps);
	}
    }
    final double [] nrm=PV.getFPsi(QQt),
	i=DeprecatedHelperTMatr.getRatio(n,PV.getFPsi(QQi),nrm),
	dist=DeprecatedHelperTMatr.getRatio(n,PV.getFPsi(QQT),nrm);
    int indimax=-1,inddistmin=-1;
    for(int k=0;k<n;k++){
	if(indimax<0 || i[k]>=i[indimax]) indimax=k;
	if(inddistmin<0 || dist[k]<=dist[inddistmin]) inddistmin=k;
    }
    Imax=i[indimax];
    pv_Imax=PV.getFPsi(indimax,QQpV)/PV.getFPsi(indimax,QQV);
    pi_Imax=PV.getFPsi(indimax,QQpi)/PV.getFPsi(indimax,QQi);
    pv_distmin=PV.ev[inddistmin];

    final double [] spsi=PV.getPsi(indimax),Dspsi=VOLATILITY_EDPsi.getDPsi(spsi),
	rq=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(spsi,Dspsi,null,QQpV,QQV,n);
    var2pV=rq[2];

    if(eps>0){
	UnitTests.checkDoubleRel(rq[0],"rq[0]",pv_Imax,"pv_Imax",eps);
	UnitTests.checkDoubleRel(rq[1],"rq[1]",0,"1",eps);
	System.err.println("InIstatesPV: test OK n="+n+" eps="+eps);
    }
}
    }
