package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.freemoney.FreeMoneyForAll;
import com.polytechnik.freemoney.MatricesFromPI;
import com.polytechnik.freemoney.CommonBasisOperations;
/**
 * Since spike approach is back.
 * @see com.polytechnik.freemoney.FreeMoneyForAll
 * @version $Id: IntervalSincePsiM.java,v 1.239 2022/09/29 21:00:05 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
*/
public class IntervalSincePsiM {
    public final EVXData I;

    public final double pv_average,Tv_average;

    public final double I_M,Tv_M,T_M,V_M,pv_M,pt_M,p_M,VdivT_M,covPTi_M,
	Tv_spurM,pv_spurM,pt_spurM,dpdt_M,dpdtPI_M;
    public final double
	dpdt_spurM,Idpdt_spurM,covPTi_spurM,
	dpdtDivI_spurM,
	dpdtPI_spurM,IdpdtPI_spurM,I2dpdtPI_spurM,dpdtDivIPI_spurM
	;
    public final double pFPI0_spurM,pF0_spurM,pFm_spurM;
    public final double pdI_spurM,pdIPI_spurM,dpdI_spurM;


    public final double dPIdtF_spurM,pFPI_posdI_spurM,pFPI_negdI_spurM;
    public final ExperimentaldIHdt dIHdtSplitF,dIHdtSplitI0,dIHdtSplitIeq0;

    public final double
	dIdt_posdI_spurM,dIdt_negdI_spurM,
	proj0_posdI_spurM,proj0_negdI_spurM,
	dpdt_posdI_spurM,dpdt_negdI_spurM,
	dpdtPI_posdI_spurM,dpdtPI_negdI_spurM
	;


    IntervalSincePsiM(final int n,
		      final double p_offset,
		      final double p_last,
		      final double [] psiM,

		      // input moment matrices
		      final double [] QQt,
		      final double [] QQi,
		      final double [] QQpi,
		      final double [] QQTi,
		      final double [] QQTpi,

		      final double [] QQdp,
		      final double [] QQpt,

		      final double [] QQT,
		      final double [] QQV,
		      final double [] QQTdp,
		      // basis operations
		      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		      final java.util.function.Function<double [],double []> getPolynomialWIntegratedDt,
		      final java.util.function.Function<double [],double []> getEDPsi,

		      // for unit test
		      final double [] tmom,
		      final double [] imom,
		      final double eps){

	this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	final double [] UNIT=TMatr.getUnitMatrix(n),
	    // Gram matrix inverse can be obtained without solving a linear system.
	    IQQt=EVXData.convertMatrFromBtoQ_Basis(n,UNIT,I.ev,UNIT);
	final double []
	    bQQi=TMatr.getDiagMatrix(I.getEValues()),
	    invIk=new double [n];
	for(int k=0;k<n;k++) invIk[k]=1.0/I.ev[k];
	final double []
	    bQQdpPI=MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(0,QQpi,I,getEDPsi,QQi,p_last),
	    bQQIdpPI=MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(1,QQpi,I,getEDPsi,QQi,p_last),
	    bQQI2dpPI=MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(2,QQpi,I,getEDPsi,QQi,p_last),
	    bQQdpDivIPI=MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(-1,QQpi,I,getEDPsi,QQi,p_last),
	    bQQdpDivI2PI=MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(-2,QQpi,I,getEDPsi,QQi,p_last),
	    bQQI2=TMatr.getDiagMatrix(TMatr.squareArrayElements(I.getEValues())),
	    bQQDivI=TMatr.getDiagMatrix(invIk),
	    bQQDivI2=TMatr.getDiagMatrix(TMatr.squareArrayElements(invIk));


	final double []
	    bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	    bQQTi=EVXData.convertMatrFromQtoB_Basis(n,QQTi,I.ev),
	    bQQTpi=EVXData.convertMatrFromQtoB_Basis(n,QQTpi,I.ev),
	    bQQV=EVXData.convertMatrFromQtoB_Basis(n,QQV,I.ev),
	    bQQT=EVXData.convertMatrFromQtoB_Basis(n,QQT,I.ev),
	    // p from original sample
	    bQQpt=EVXData.convertMatrFromQtoB_Basis(n,QQpt,I.ev),
	    // dp/dt from original sample
	    bQQdp=EVXData.convertMatrFromQtoB_Basis(n,QQdp,I.ev),
	    // T*dp/dt from original sample
	    bQQTdp=EVXData.convertMatrFromQtoB_Basis(n,QQTdp,I.ev),

	    mom_dp_asItWereFromMoments=M.B.getMomentsOfMeasureProducingPolynomialInKK_MQQM(n,M.B.getKK(n,TMatr.mulMMM(n,IQQt,EVXData.convertMatrFromBtoQ_Basis(n,bQQdpPI,I.ev,QQt),IQQt)),IQQt),
	    QQp_asItWereFromMoments=TMatr.sumaAbB(p_last,QQt,1,M.B.getQQMatr(n,CommonBasisOperations.getTransformFMomentsFromDF(getPolynomialWIntegratedDt,mom_dp_asItWereFromMoments))),
	    bQQp_asItWereFromMoments=EVXData.convertMatrFromQtoB_Basis(n,QQp_asItWereFromMoments,I.ev);

	if(eps>0){

	    // Check that pure state DM= |psiM><psiM| matches the polynomial psiM^2(x)
	    final double [] QQDensityMatrixPurePsiM=M.B.getQQMatr(n,M.B.getMomentsOfMeasureProducingPolynomialInKK_MQQM(n,M.B.mult2Pol(psiM,psiM),IQQt)),
		pol_QQDensityMatrixPurePsiM=M.B.getKK(n,TMatr.mulMMM(n,IQQt,QQDensityMatrixPurePsiM,IQQt));
	    UnitTests.checkDoubleArrayRel1(M.B.mult2Pol(psiM,psiM),"psiM^2(x)",pol_QQDensityMatrixPurePsiM,"pol_QQDensityMatrixPurePsiM",eps);
	    // The eigenvalues lambda of eigenproblem
	    // |QQDensityMatrixPurePsiM|psi>=lambda|QQt|psi>
	    // are not all 0 except maximal equals to 1, some are even negative!
	    // This is related to negative eigenvalues of ev_bQQDensityMatrix below.
	    final double [] ev_QQDensityMatrixPurePsiM=M.EV.getGEV_Generalized(n,QQDensityMatrixPurePsiM,QQt,false);
	    System.err.println("ev_QQDensityMatrixPurePsiM="+java.util.Arrays.toString(ev_QQDensityMatrixPurePsiM)+", some can be negative!!! 1=sum ev="+TMatr.sumElements(ev_QQDensityMatrixPurePsiM)+" sum ev^2="+TMatr.mulVV(ev_QQDensityMatrixPurePsiM,ev_QQDensityMatrixPurePsiM)+", not necessary 1!!!");
	    // The sum of eigenvalues is 1 (normalizing condition on psiM^2(x))
	    UnitTests.checkDoubleRel(TMatr.sumElements(ev_QQDensityMatrixPurePsiM),"sum(ev)",1,"1",eps);
	    // The sum of eigenvalues squared is not necessary 1.
	    //UnitTests.checkDoubleRel(TMatr.mulVV(ev_QQDensityMatrixPurePsiM,ev_QQDensityMatrixPurePsiM),"sum(ev^2)",1,"1",eps);
	}

	final double []
	    DpsiM=getEDPsi.apply(psiM),
	    bpsiM=EVXData.convertPsiFromQtoB_Basis(n,psiM,I.ev,QQt),
	    bDpsiM=EVXData.convertPsiFromQtoB_Basis(n,DpsiM,I.ev,QQt)
	    ;

	// Selected pure state |psiM>.
	I_M=TMatr.getAMatrA(bpsiM,bQQi,n);
	Tv_M=TMatr.getAMatrA(bpsiM,bQQTi,n)/I_M;
	T_M=TMatr.getAMatrA(bpsiM,bQQT,n);
	V_M=TMatr.getAMatrA(bpsiM,bQQV,n);
	pt_M=p_offset+TMatr.getAMatrA(bpsiM,bQQpt,n);
	pv_M=p_offset+TMatr.getAMatrA(bpsiM,bQQpi,n)/I_M;
	dpdt_M=TMatr.getAMatrA(bpsiM,bQQdp,n);
	dpdtPI_M=TMatr.getAMatrA(bpsiM,bQQdpPI,n);
	p_M=p_offset+TMatr.getAMatrA(bpsiM,bQQp_asItWereFromMoments,n);
	VdivT_M=V_M/T_M;
	pv_average=p_offset+QQpi[0]/QQi[0];
	Tv_average=QQTi[0]/QQi[0];
	covPTi_M=TMatr.getAMatrA(bpsiM,bQQTpi,n)/I_M-(TMatr.getAMatrA(bpsiM,bQQpi,n)/I_M)*Tv_M;


	// A mixed state with |QQDensityMatrix| density matrix obtained as "an interval since |psiM> till now".
	final double []
	    // As the calculations require multiplication operator -- perfrom them in Q-basis,
	    // where multiplication is readily available.
	    //
	    // Integrate psiM^2 omega(x) dt: time since psiM spike
	    // dmu=pol_QQDensityMatrix(x(t))*omega(t)dt
	    pol_QQDensityMatrix=getPolynomialWIntegratedDt.apply(M.B.mult2Pol(psiM,psiM)),
	    // Construct the measure corresponding to pol_QQDensityMatrix polynomial
	    // using Theorem 3 of https://arxiv.org/abs/1807.06007 : find the measure generating given polynomial,
	    // Find measure moments, then create density matrix corresponding to this measure.
	    QQDensityMatrix=M.B.getQQMatr(n,M.B.getMomentsOfMeasureProducingPolynomialInKK_MQQM(n,pol_QQDensityMatrix,IQQt)),
	    // Convert it to b-basis to avoid extra IQQt in spur to simplify calculations.
	    bQQDensityMatrix=EVXData.convertMatrFromQtoB_Basis(n,QQDensityMatrix,I.ev)
	    ;

	Tv_spurM=TMatr.getSpurMM(n,bQQTi,bQQDensityMatrix)/V_M;
	pt_spurM=p_offset+TMatr.getSpurMM(n,bQQpt,bQQDensityMatrix)/T_M;
	pv_spurM=p_offset+TMatr.getSpurMM(n,bQQpi,bQQDensityMatrix)/V_M;
	dpdt_spurM=TMatr.getSpurMM(n,bQQdp,bQQDensityMatrix);
	Idpdt_spurM=TMatr.getSpurMMM(n,bQQdp,bQQi,bQQDensityMatrix);
	covPTi_spurM=TMatr.getSpurMM(n,bQQTpi,bQQDensityMatrix)/V_M-(TMatr.getSpurMM(n,bQQpi,bQQDensityMatrix)/V_M)*Tv_spurM;

	dpdtDivI_spurM=TMatr.getSpurMMM(n,bQQdp,bQQDivI,bQQDensityMatrix);

	dpdtPI_spurM=TMatr.getSpurMM(n,bQQdpPI,bQQDensityMatrix);
	IdpdtPI_spurM=TMatr.getSpurMM(n,bQQIdpPI,bQQDensityMatrix);
	I2dpdtPI_spurM=TMatr.getSpurMM(n,bQQI2dpPI,bQQDensityMatrix);
	dpdtDivIPI_spurM=TMatr.getSpurMM(n,bQQdpDivIPI,bQQDensityMatrix);


	// Future value of execution flow. The value is equal to the maximal eigenvalue I.sH
	// The Impact from the future is then: I.sH-I.s0
	// However, as we sometimes use as |psiM> not the |psiH>,
	// the eigenvector of maximal eigenvalue of |I|psi>=lambda|psi>,
	// here we set Ifuture=<psiM|I|psiM>. Again, proper |psiM> is |psiH> and hence Ifuture=I.sH
	final double Ifuture=I_M,I0=I.s0,dIF=I_M-I0;

	final double []
	    bQQdIdt=MatricesFromPI.getbQQpdIdtFromQQp(QQt,I,getEDPsi,I0),
	    QQ0=TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),
	    // we assume omega(x0)=1
	    bQQ0=EVXData.convertMatrFromQtoB_Basis(n,QQ0,I.ev),
	    bQQdIdtF=MatricesFromPI.getbQQpdIdtFromQQp(QQt,I,getEDPsi,Ifuture),
	    // The d(pI)/dt moments. They are easy to calculate:
	    bQQdPIdtF=MatricesFromPI.getbQQdpdtFromQQp_withLastP(QQpi,I,getEDPsi,QQt,p_last*Ifuture)
	    ;


	if(eps>0){
	    UnitTests.checkDoubleRel1(TMatr.getSpurMM(n,bQQdIdtF,bQQDensityMatrix),"(Spur bQQdIdtF*bQQDensityMatrix)",0,"0)",eps);
	    System.err.println("Spur (bQQdIdtF*bQQi*bQQDensityMatrix)="+TMatr.getSpurMMM(n,bQQdIdtF,bQQi,bQQDensityMatrix)+" I.sH="+I.sH+" I_M="+I_M+" n="+n);
	    // does not hold
	    // UnitTests.checkDoubleRel1(TMatr.getSpurMMM(n,bQQdIdtF,bQQi,bQQDensityMatrix),"Spur (bQQdIdtF*bQQi*bQQDensityMatrix)",0,"0)",eps);
	    UnitTests.checkDoubleRel1(TMatr.getSpurMM(n,bQQdIdt,bQQDensityMatrix),"Spur (bQQdIdt*bQQDensityMatrix)",-dIF,"-dIF)",eps);

	}

	dPIdtF_spurM=TMatr.getSpurMM(n,bQQdPIdtF,bQQDensityMatrix);
	dIHdtSplitF=new ExperimentaldIHdt(
	    I,
	    p_offset,
	    p_last,
	    bQQdIdtF,
	    bQQdPIdtF,
	    bQQDensityMatrix,
	    bQQdp,
	    bQQTi,bQQi,
	    M,eps);
	dIHdtSplitI0=new ExperimentaldIHdt(
	    I,
	    p_offset,
	    p_last,
	    MatricesFromPI.getbQQpdIdtFromQQp(QQt,I,getEDPsi,I.s0),
	    MatricesFromPI.getbQQdpdtFromQQp_withLastP(QQpi,I,getEDPsi,QQt,p_last*I.s0),
	    bQQDensityMatrix,
	    bQQdp,
	    bQQTi,bQQi,
	    M,eps);
	dIHdtSplitIeq0=new ExperimentaldIHdt(
	    I,
	    p_offset,
	    p_last,
	    MatricesFromPI.getbQQpdIdtFromQQp(QQt,I,getEDPsi,0),
	    MatricesFromPI.getbQQdpdtFromQQp_withLastP(QQpi,I,getEDPsi,QQt,p_last*0),
	    bQQDensityMatrix,
	    bQQdp,
	    bQQTi,bQQi,
	    M,eps);
	if(eps>0){
	    UnitTests.checkDoubleRel1(dIHdtSplitF.dIHdt_spurM,"dIHdtSplitF.dIHdt_spurM",0,"0",eps);
	    UnitTests.checkDoubleRel1(dIHdtSplitI0.dIHdt_spurM,"dIHdtSplitI0.dIHdt_spurM",I.s0-I_M,"I.s0-I_M",eps);
	    UnitTests.checkDoubleRel1(dIHdtSplitIeq0.dIHdt_spurM,"dIHdtSplitIeq0.dIHdt_spurM",-I_M,"-I_M",eps);
	    UnitTests.checkDoubleRel1(dIHdtSplitIeq0.pv_dI,"dIHdtSplitIeq0.pv_dI",pv_M,"pv_M",eps);
	}


	// try dI/dt split once again (last tried in DM_DI.java in September 2018 during my last visit to Michigan).
	final double [] ev_bQQdIdtF=M.EV.getEV(n,bQQdIdtF,true),
	    bbQQdIdtF=TMatr.getDiagMatrix(java.util.Arrays.copyOf(ev_bQQdIdtF,n)),
	    bbQQDensityMatrix=EVXData.convertMatrFromQtoB_Basis(n,bQQDensityMatrix,ev_bQQdIdtF),
	    bbQQdp=EVXData.convertMatrFromQtoB_Basis(n,bQQdp,ev_bQQdIdtF),
	    bbQQdpPI=EVXData.convertMatrFromQtoB_Basis(n,bQQdpPI,ev_bQQdIdtF),
	    bbQQ0=EVXData.convertMatrFromQtoB_Basis(n,bQQ0,ev_bQQdIdtF),
	    bbQQdPIdtF=EVXData.convertMatrFromQtoB_Basis(n,bQQdPIdtF,ev_bQQdIdtF),
	    bbQQ_selPosdIdtF=new double [n*n],
	    bbQQ_selNegdIdtF=new double [n*n];
	for(int k=0;k<n;k++){
	    final int diagk=k*n+k;
	    if(ev_bQQdIdtF[k]>0){
		bbQQ_selPosdIdtF[diagk]=1;
	    } else if(ev_bQQdIdtF[k]<=0){
		bbQQ_selNegdIdtF[diagk]=1;
	    }
	}

	dIdt_posdI_spurM=TMatr.getSpurMMM(n,bbQQdIdtF,bbQQ_selPosdIdtF,bbQQDensityMatrix);
	dIdt_negdI_spurM=TMatr.getSpurMMM(n,bbQQdIdtF,bbQQ_selNegdIdtF,bbQQDensityMatrix);
	final double
	    dPIdt_posdI_spurM=TMatr.getSpurMMM(n,bbQQdPIdtF,bbQQ_selPosdIdtF,bbQQDensityMatrix),
	    dPIdt_negdI_spurM=TMatr.getSpurMMM(n,bbQQdPIdtF,bbQQ_selNegdIdtF,bbQQDensityMatrix);
	pFPI_posdI_spurM=p_offset+dPIdt_posdI_spurM/dIdt_posdI_spurM;
	pFPI_negdI_spurM=p_offset+dPIdt_negdI_spurM/dIdt_negdI_spurM;


	proj0_posdI_spurM=TMatr.getSpurMMM(n,bbQQ0,bbQQ_selPosdIdtF,bbQQDensityMatrix);
	proj0_negdI_spurM=TMatr.getSpurMMM(n,bbQQ0,bbQQ_selNegdIdtF,bbQQDensityMatrix);
	dpdt_posdI_spurM=TMatr.getSpurMMM(n,bbQQdp,bbQQ_selPosdIdtF,bbQQDensityMatrix);
	dpdt_negdI_spurM=TMatr.getSpurMMM(n,bbQQdp,bbQQ_selNegdIdtF,bbQQDensityMatrix);
	dpdtPI_posdI_spurM=TMatr.getSpurMMM(n,bbQQdpPI,bbQQ_selPosdIdtF,bbQQDensityMatrix);
	dpdtPI_negdI_spurM=TMatr.getSpurMMM(n,bbQQdpPI,bbQQ_selNegdIdtF,bbQQDensityMatrix);


	if(eps>0){
	    // check d(pI)/dt moments, they is easy to calculate
	    final double
		bb_dPIdtF_spurM=TMatr.getSpurMM(n,bbQQdPIdtF,bbQQDensityMatrix),
		v0eq_spurM=TMatr.getSpurMM(n,bQQ0,bQQDensityMatrix),
		Plast_m_pv_M=(p_last+p_offset)-pv_M,
		PposdI_m_PnegdI=pFPI_posdI_spurM-pFPI_negdI_spurM;
	    UnitTests.checkDoubleRel1(v0eq_spurM,"v0eq_spurM",1,"1",eps);
	    UnitTests.checkDoubleRel1(TMatr.getSpurMM(n,bbQQ0,bbQQDensityMatrix),"spurv(bbQQ0*bbQQDensityMatrix)",1,"1",eps);
	    UnitTests.checkDoubleRel1(Plast_m_pv_M*Ifuture,"Plast_m_pv_M*Ifuture",dPIdtF_spurM,"dPIdtF_spurM",eps);
	    UnitTests.checkDoubleRel1(Plast_m_pv_M*Ifuture,"Plast_m_pv_M*Ifuture",bb_dPIdtF_spurM,"bb_dPIdtF_spurM",eps);
	    UnitTests.checkDoubleRel1(TMatr.getSpurMM(n,bbQQdIdtF,bbQQDensityMatrix),"spur(bbQQdIdtF*bbQQDensityMatrix)",0,"0",eps);

	    // check split
	    UnitTests.checkDoubleRel1(dIdt_posdI_spurM+dIdt_negdI_spurM,"dIdt_posdI_spurM+dIdt_negdI_spurM",0,"0",eps);
	    UnitTests.checkDoubleRel1(dPIdt_posdI_spurM+dPIdt_negdI_spurM,"dPIdt_posdI_spurM+dPIdt_negdI_spurM",dPIdtF_spurM,"dPIdtF_spurM",eps);
	    UnitTests.checkDoubleRel1(proj0_posdI_spurM+proj0_negdI_spurM,"proj0_posdI_spurM+proj0_negdI_spurM",1,"1",eps);
	    UnitTests.checkDoubleRel1(dpdt_posdI_spurM+dpdt_negdI_spurM,"dpdt_posdI_spurM+dpdt_negdI_spurM",dpdt_spurM,"dpdt_spurM",eps);
	    UnitTests.checkDoubleRel1(dpdtPI_posdI_spurM+dpdtPI_negdI_spurM,"dpdtPI_posdI_spurM+dpdtPI_negdI_spurM",dpdtPI_spurM,"dpdtPI_spurM",eps);

	    // check PposdI-PnegdI relation
	    UnitTests.checkDoubleRel1(PposdI_m_PnegdI*(dIdt_posdI_spurM/Ifuture),"PposdI_m_PnegdI*(dIdt_posdI_spurM/Ifuture)",Plast_m_pv_M,"Plast_m_pv_M",eps);

	    System.err.println(" I.sH="+I.sH+" I_M="+I_M+" dIdt_posdI_spurM="+dIdt_posdI_spurM+" dIdt_negdI_spurM="+dIdt_negdI_spurM+" proj0_posdI_spurM="+proj0_posdI_spurM+" proj0_negdI_spurM="+proj0_negdI_spurM+" dpdtPI_spurM="+dpdtPI_spurM+" Plast_m_pv_M="+Plast_m_pv_M+" n="+n);

	}


	final double []
	    bQQpdIdtPI0=TMatr.sumaAbB(p_last*I0,bQQ0,1,MatricesFromPI.getbQQpdIdtFromQQpi(QQpi,I,getEDPsi,0)),
	    bQQpdIdt0=TMatr.sumaAbB(p_last*I0,bQQ0,1,MatricesFromPI.getbQQpdIdtFromQQp(QQpt,I,getEDPsi,0));
	pFm_spurM=p_offset-TMatr.getSpurMMM(n,bQQdIdt,bQQpt,bQQDensityMatrix)/dIF;
	pFPI0_spurM=p_offset-TMatr.getSpurMM(n,bQQpdIdtPI0,bQQDensityMatrix)/dIF;
	pF0_spurM=p_offset-TMatr.getSpurMM(n,bQQpdIdt0,bQQDensityMatrix)/dIF;
	pdI_spurM=TMatr.getSpurMMM(n,bQQdIdtF,bQQpt,bQQDensityMatrix);
	pdIPI_spurM=TMatr.getSpurMMM(n,bQQdIdtF,MatricesFromPI.getbQQpFrombQQpi(bQQpi,I),bQQDensityMatrix);
	dpdI_spurM=TMatr.getSpurMMM(n,bQQdIdtF,bQQdp,bQQDensityMatrix);


	if(eps>0){


	    // In the approximation we use bQQIdpPI = sqrt(I)*bQQdpPI*sqrt(I)
	    UnitTests.checkDoubleArrayRel1(bQQIdpPI,"bQQIdpPI",MatricesFromPI.getbQQpiFrombQQp(bQQdpPI,I),"sqrt(I)*bQQdpPI*sqrt(I)",eps);
	    UnitTests.checkDoubleArrayRel1(MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(3,QQpi,I,getEDPsi,QQi,p_last),"bQQI3dpPI",MatricesFromPI.getbQQpiFrombQQp(bQQI2dpPI,I),"sqrt(I)*bQQI2dpPI*sqrt(I)",eps);

	    // Test invariance of integration by parts
	    for(var bQQdF:java.util.List.of(bQQdpPI,bQQIdpPI,bQQI2dpPI,bQQdpDivIPI,bQQdpDivI2PI)){
		// QQdF was adjusted to p-p_last to have no out of integral term
		final double [] QQdF=EVXData.convertMatrFromBtoQ_Basis(n,bQQdF,I.ev,QQt),
		    // we have muliplication available as M.B only in Q-basis
		    polynomial=M.B.getKK(n,TMatr.mulMMM(n,IQQt,QQdF,IQQt)),
		    momentsdF=M.B.getMomentsOfMeasureProducingPolynomialInKK_MQQM(n,polynomial,IQQt),
		    momentsF=com.polytechnik.freemoney.CommonBasisOperations.getTransformFMomentsFromDF(getPolynomialWIntegratedDt,momentsdF),
		    QQF=M.B.getQQMatr(n,momentsF);
		// QQF assumes last value is 0
		UnitTests.checkDoubleRel(TMatr.getSpurMM(n,bQQdF,bQQDensityMatrix),"spur(bQQdF*bQQDensityMatrix)",0-TMatr.getAMatrA(psiM,QQF,n),"0 - <psiM|QQF|psiM>",eps);
	    }
	    // Test I-powers
	    final java.util.function.DoubleFunction<double []> getX0Matr=(powI)->
		TMatr.scaleVectorInPlace(
		    TMatr.getQQaa(java.util.stream.IntStream.range(0,n).mapToDouble(k->Math.pow(I.ev[k],powI)*I.psik_at_x0[k]).toArray()),
		    -1);
	    UnitTests.checkDoubleArrayRel1(getX0Matr.apply(0),"-psij*psik",MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(0,QQi,I,getEDPsi,QQi,0),"dpdt from QQi",eps);
	    UnitTests.checkDoubleArrayRel1(getX0Matr.apply(0.5),"-sqrt(Ij)*psij*psik*sqrt(Ik)",MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(1,QQi,I,getEDPsi,QQi,0),"I dpdt from QQi",eps);
	    UnitTests.checkDoubleArrayRel1(getX0Matr.apply(1),"-Ij*psij*psik*Ik",MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(2,QQi,I,getEDPsi,QQi,0),"I^2 dpdt from QQi",eps);
	    UnitTests.checkDoubleArrayRel1(getX0Matr.apply(-0.5),"-1/sqrt(Ij)*psij*psik*1/sqrt(Ik)",MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(-1,QQi,I,getEDPsi,QQi,0),"1/I dpdt from QQi",eps);
	    UnitTests.checkDoubleArrayRel1(getX0Matr.apply(-1),"-1/Ij*psij*psik*1/Ik",MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(-2,QQi,I,getEDPsi,QQi,0),"1/I^2 dpdt from QQi",eps);
	}


	final VolMatrix VOLATILITY_EDPsi=getEDPsi::apply;
	if(eps>0){
	    final double [] _IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
	    // Inverse Gram matrix check
	    UnitTests.checkDoubleArrayRel1(IQQt,"IQQt",_IQQt,"_IQQt",eps);
	    UnitTests.checkDoubleArrayRel1(TMatr.mulMM(n,QQt,IQQt),"QQt * IQQt",UNIT,"UNIT",eps);

	    final double V=TMatr.mulVVn1(pol_QQDensityMatrix,imom),T=TMatr.mulVVn1(pol_QQDensityMatrix,tmom);
	    UnitTests.checkDoubleRel(T,"T",T_M,"T_M",eps);
	    UnitTests.checkDoubleRel(T,"T",TMatr.getSpurM(n,bQQDensityMatrix),"spur(bQQDensityMatrix)",eps);
	    UnitTests.checkDoubleRel(T,"T",TMatr.getSpurMM(n,IQQt,QQDensityMatrix),"spur(IQQt*QQDensityMatrix)",eps); // in original basis
	    UnitTests.checkDoubleRel(V,"V",V_M,"V_M",eps);
	    UnitTests.checkDoubleRel(V,"V",TMatr.getSpurMM(n,bQQi,bQQDensityMatrix),"spur(bQQi*bQQDensityMatrix)",eps);
	    UnitTests.checkDoubleRel(V,"V",TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,QQi,IQQt,QQDensityMatrix})),"spur(IQQt*QQi*IQQt*QQDensityMatrix)",eps); // in original basis

	    // assume omega(x0)=1
	    UnitTests.checkDoubleRel(M.B.getD0(pol_QQDensityMatrix,I.x0),"pol_QQDensityMatrix(x0)",1,"1",eps);
	    final double [] bQQDDensityMatrix=EVXData.convertMatrFromQtoB_Basis(n,VOLATILITY_EDPsi.getDiffMatrix(n,QQDensityMatrix),I.ev);
	    UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQDDensityMatrix),"spur(bQQDDensityMatrix)",1,"1",eps);
	}


	if(eps>0){
	    final double
		pt=TMatr.getAMatrA(bpsiM,bQQpt,n),
		p_asItWereFromMoments=TMatr.getAMatrA(bpsiM,bQQp_asItWereFromMoments,n);

	    // exact because bQQpt is obtained from moments
	    UnitTests.checkDoubleRel(p_last-pt,"p_last-pt",dpdt_spurM,"dpdt_spurM",eps);
	    // exact because bQQp_asItWereFromMoments corresponds to a measure
	    UnitTests.checkDoubleRel(p_last-p_asItWereFromMoments,"p_last-p_asItWereFromMoments",dpdtPI_spurM,"dpdtPI_spurM",eps);

	    final double [] bQQDTpi=
		EVXData.convertMatrFromQtoB_Basis(n,TMatr.sumAcB(new double [n*n],-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQTpi)),I.ev);
	    final double Tpi=TMatr.getAMatrA(psiM,QQTpi,n);
	    UnitTests.checkDoubleRel(0-Tpi,"0-Tpi",TMatr.getSpurMM(n,bQQDTpi,bQQDensityMatrix),"spur(bQQDTpi*bQQDensityMatrix)",eps);

	}


	if(eps>0){
	    // in DM proper basis
	    final double [] ev_bQQDensityMatrix=M.EV.getEV(n,bQQDensityMatrix,true),
		cbQQDensityMatrix=TMatr.getDiagMatrix(java.util.Arrays.copyOf(ev_bQQDensityMatrix,n)),
		cbpsiM=EVXData.convertPsiFromQtoB_Basis(n,bpsiM,ev_bQQDensityMatrix,UNIT),
		cbDpsiM=EVXData.convertPsiFromQtoB_Basis(n,bDpsiM,ev_bQQDensityMatrix,UNIT),
		cbQQi=EVXData.convertMatrFromQtoB_Basis(n,bQQi,ev_bQQDensityMatrix),
		cbQQpi=EVXData.convertMatrFromQtoB_Basis(n,bQQpi,ev_bQQDensityMatrix),
		cbQQTi=EVXData.convertMatrFromQtoB_Basis(n,bQQTi,ev_bQQDensityMatrix),
		cbQQdp=EVXData.convertMatrFromQtoB_Basis(n,bQQdp,ev_bQQDensityMatrix),
		cbQQpt=EVXData.convertMatrFromQtoB_Basis(n,bQQpt,ev_bQQDensityMatrix),
		cbQQT=EVXData.convertMatrFromQtoB_Basis(n,bQQT,ev_bQQDensityMatrix)
		;

	    UnitTests.checkDoubleRel(V_M,"V_M",TMatr.getSpurMM(n,cbQQi,cbQQDensityMatrix),"spur(cbQQi*cbQQDensityMatrix)",eps);
	    UnitTests.checkDoubleRel(T_M,"T_M",TMatr.getSpurM(n,cbQQDensityMatrix),"spur(cbQQDensityMatrix)",eps);
	    UnitTests.checkDoubleRel(T_M,"T_M",TMatr.getAMatrA(cbpsiM,cbQQT,n),"<cbpsiM|cbQQT|cbpsiM>",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(bpsiM,bDpsiM,bQQpi,n),"<bpsiM|bQQpi|bDpsiM>",TMatr.getAMatrB(cbpsiM,cbDpsiM,cbQQpi,n),"<cbpsiM|cbQQpi|cbDpsiM>",eps);
	    UnitTests.checkDoubleRel(dpdt_spurM,"dpdt_spurM",TMatr.getSpurMM(n,cbQQdp,cbQQDensityMatrix),"spur(cbQQdp*cbQQDensityMatrix)",eps);
	    UnitTests.checkDoubleRel(pt_spurM-p_offset,"pt_spurM-p_offset",TMatr.getSpurMM(n,cbQQpt,cbQQDensityMatrix)/T_M,"spur(cbQQpt*cbQQDensityMatrix)/T_M",eps);
	    UnitTests.checkDoubleRel(Tv_M,"Tv_M",TMatr.getAMatrA(cbpsiM,cbQQTi,n)/TMatr.getAMatrA(cbpsiM,cbQQi,n),"<cbpsiM|cbQQTi|cbpsiM>/<cbpsiM|cbQQi|cbpsiM>",eps);
	    // assume omega(x0)=1
	    UnitTests.checkDoubleRel1(2*TMatr.getAMatrB(cbpsiM,cbDpsiM,UNIT,n),"2*<cbpsiM|cbDpsiM>",Math.pow(M.B.getD0(psiM,I.x0),2),"psiM(x0)^2",eps);

	    System.err.println("IntervalSincePsiM: test OK DM eigenvalues="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_bQQDensityMatrix,n))+", some can be negative!!! n="+n+" eps="+eps);
	}



	if(eps>0){
	    final double [] QQSDensityMatrix=M.B.getQQMatr(n,M.B.getMomentsOfMeasureProducingPolynomialInKK(n,pol_QQDensityMatrix)),
		invQQSDensityMatrix=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQSDensityMatrix),
		bQQSDensityMatrix=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,EVXData.convertMatrFromQtoB_Basis(n,invQQSDensityMatrix,I.ev))
		;

	    UnitTests.checkDoubleArrayRel1(pol_QQDensityMatrix,"pol_QQDensityMatrix",M.B.getKK(n,QQSDensityMatrix),"getKK from QQSDensityMatrix",eps);
	    final double T=TMatr.mulVVn1(pol_QQDensityMatrix,tmom),
		piC=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][]{IQQt,QQpi,IQQt,QQDensityMatrix}));


	    System.err.println("T="+T+" T_M="+T_M+" SP (QQt*QQS)="+TMatr.getSpurMM(n,QQt,QQSDensityMatrix)+
			       " SP QQS="+TMatr.getSpurM(n,QQSDensityMatrix));
	    System.err.println("bQQSDensityMatrix="+java.util.Arrays.toString(bQQSDensityMatrix));
	    System.err.println("bQQDensityMatrix="+java.util.Arrays.toString(bQQDensityMatrix));


	    UnitTests.checkDoubleRel(T_M,"T_M",TMatr.getSpurMM(n,QQt,QQSDensityMatrix),"spur(QQt*QQSDensityMatrix)",eps); // in original basis
	    UnitTests.checkDoubleRel(V_M,"V_M",TMatr.getSpurMM(n,QQi,QQSDensityMatrix),"spur(QQi*QQSDensityMatrix)",eps); // in original basis
	    UnitTests.checkDoubleRel(T_M,"T_M",TMatr.getSpurM(n,bQQSDensityMatrix),"spur(bQQSDensityMatrix)",eps);
	    UnitTests.checkDoubleRel(V_M,"V_M",TMatr.getSpurMM(n,bQQi,bQQSDensityMatrix),"spur(bQQi*bQQSDensityMatrix)",eps);
	    UnitTests.checkDoubleRel(piC,"piC",TMatr.getSpurMM(n,bQQpi,bQQSDensityMatrix),"spur(bQQpi*bQQSDensityMatrix)",eps);
	    final double [] ev_bQQSDensityMatrix=M.EV.getEV(n,bQQSDensityMatrix,false);
	    UnitTests.checkDoubleRel(T_M,"T_M",TMatr.sumElements(ev_bQQSDensityMatrix),"sum(ev_bQQSDensityMatrix)",eps);
	    final var DM=EVXData.getEVXDataFromQQ(n,invQQSDensityMatrix,QQt,M);
	    if(!java.util.Arrays.stream(DM.getEValues()).allMatch(t-> t>0)){
		System.err.println("IntervalSincePsiM: Sometimes we have some DM.getEValues())="+java.util.Arrays.toString(DM.getEValues())+" negative! What is this?");
	    }
	    final var DMj=EVXData.getEVXDataFromQQ(n,QQSDensityMatrix,IQQt,M);
	    UnitTests.checkDoubleArrayRel1(DMj.getEValues(),"DMj.getEValues()",java.util.Arrays.copyOf(ev_bQQSDensityMatrix,n),"ev ev_bQQSDensityMatrix",eps);

	    if(!java.util.Arrays.stream(ev_bQQSDensityMatrix).allMatch(t-> t>0)){
		System.err.println("IntervalSincePsiM: Sometimes we have some ev_bQQSDensityMatrix="+java.util.Arrays.toString(ev_bQQSDensityMatrix)+" negative! What is this?");
	    }

	}



	if(eps>0){
	    // match calculations
            final double []
		_bQQdIdtF=EVXData.convertMatrFromQtoB_Basis(
		    n,
		    TMatr.sumaAbB(Ifuture,QQ0,-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQi)),
		    I.ev),
		__bQQdIdtF=MatricesFromPI.getbQQpdIdtFromQQpi(QQi,I,getEDPsi,Ifuture),
		__bQQdIdt=MatricesFromPI.getbQQpdIdtFromQQpi(QQi,I,getEDPsi,I0),
		_bQQdPIdtF=EVXData.convertMatrFromQtoB_Basis(
		    n,
		    TMatr.sumaAbB(p_last*Ifuture,QQ0,-1,VOLATILITY_EDPsi.getDiffMatrix(n,QQpi)),
		    I.ev);
	    UnitTests.checkDoubleArrayRel1(bQQdIdtF,"bQQdIdtF",_bQQdIdtF,"_bQQdIdtF",eps);
	    UnitTests.checkDoubleArrayRel1(bQQdIdtF,"bQQdIdtF",__bQQdIdtF,"__bQQdIdtF",eps);
	    UnitTests.checkDoubleArrayRel1(bQQdIdt,"bQQdIdt",__bQQdIdt,"__bQQdIdt",eps);
	    UnitTests.checkDoubleArrayRel1(bQQdPIdtF,"bQQdPIdtF",_bQQdPIdtF,"_bQQdPIdtF",eps);

	    // should be zero because Ifuture is equal to I_M
	    final double dIdtF_spurM=TMatr.getSpurMM(n,bQQdIdtF,bQQDensityMatrix),
		Plast_m_pv_M=(p_last+p_offset)-pv_M;
	    UnitTests.checkDoubleRel(dIdtF_spurM,"dIdtF_spurM",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getSpurMM(n,_bQQdPIdtF,bQQDensityMatrix),"spur(_bQQdIPdt*bQQDensityMatrix)",Ifuture*Plast_m_pv_M,"Ifuture*Plast_m_pv_M",eps);

	    System.err.println("IntervalSincePsiM: V_M="+V_M+" I_M="+I_M);
	}

    }


static void match(final IntervalSincePsiM a,final IntervalSincePsiM b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    // in this version output prices (public fields) are without an offset

    UnitTests.checkDoubleRel(a.I_M,"a.I_M",b.I_M,"b.I_M",eps);
    UnitTests.checkDoubleRel(a.pv_M,"a.pv_M",b.pv_M,"b.pv_M",eps);
    UnitTests.checkDoubleRel(a.pt_M,"a.pt_M",b.pt_M,"b.pt_M",eps);
    UnitTests.checkDoubleRel(a.p_M,"a.p_M",b.p_M,"b.p_M",eps);
    UnitTests.checkDoubleRel(a.Tv_M,"a.Tv_M",b.Tv_M,"b.Tv_M",eps);
    UnitTests.checkDoubleRel(a.T_M,"a.T_M",b.T_M,"b.T_M",eps);
    UnitTests.checkDoubleRel(a.V_M,"a.V_M",b.V_M,"b.V_M",eps);
    UnitTests.checkDoubleRel(a.VdivT_M,"a.VdivT_M",b.VdivT_M,"b.VdivT_M",eps);


    UnitTests.checkDoubleRel(a.Tv_spurM,"a.Tv_spurM",b.Tv_spurM,"b.Tv_spurM",eps);
    UnitTests.checkDoubleRel(a.pv_spurM,"a.pv_spurM",b.pv_spurM,"b.pv_spurM",eps);
    UnitTests.checkDoubleRel(a.pt_spurM,"a.pt_spurM",b.pt_spurM,"b.pt_spurM",eps);
    UnitTests.checkDoubleRel(a.dpdt_spurM,"a.dpdt_spurM",b.dpdt_spurM,"b.dpdt_spurM",eps);
    UnitTests.checkDoubleRel(a.Idpdt_spurM,"a.Idpdt_spurM",b.Idpdt_spurM,"b.Idpdt_spurM",eps);
    UnitTests.checkDoubleRel(a.dpdtDivI_spurM,"a.dpdtDivI_spurM",b.dpdtDivI_spurM,"b.dpdtDivI_spurM",eps);

    UnitTests.checkDoubleRel(a.dpdtPI_spurM,"a.dpdtPI_spurM",b.dpdtPI_spurM,"b.dpdtPI_spurM",eps);
    UnitTests.checkDoubleRel(a.IdpdtPI_spurM,"a.IdpdtPI_spurM",b.IdpdtPI_spurM,"b.IdpdtPI_spurM",eps);
    UnitTests.checkDoubleRel(a.I2dpdtPI_spurM,"a.I2dpdtPI_spurM",b.I2dpdtPI_spurM,"b.I2dpdtPI_spurM",eps);
    UnitTests.checkDoubleRel(a.dpdtDivIPI_spurM,"a.dpdtDivIPI_spurM",b.dpdtDivIPI_spurM,"b.dpdtDivIPI_spurM",eps);

    UnitTests.checkDoubleRel(a.pFPI0_spurM,"a.pFPI0_spurM",b.pFPI0_spurM,"b.pFPI0_spurM",eps);
    UnitTests.checkDoubleRel(a.pF0_spurM,"a.pF0_spurM",b.pF0_spurM,"b.pF0_spurM",eps);
    UnitTests.checkDoubleRel(a.pFm_spurM,"a.pFm_spurM",b.pFm_spurM,"b.pFm_spurM",eps);


    ExperimentaldIHdt.match(a.dIHdtSplitF,b.dIHdtSplitF,eps);
    ExperimentaldIHdt.match(a.dIHdtSplitI0,b.dIHdtSplitI0,eps);
    ExperimentaldIHdt.match(a.dIHdtSplitIeq0,b.dIHdtSplitIeq0,eps);


    UnitTests.checkDoubleRel(a.dIdt_posdI_spurM,"a.dIdt_posdI_spurM",b.dIdt_posdI_spurM,"b.dIdt_posdI_spurM",eps);
    UnitTests.checkDoubleRel(a.dIdt_negdI_spurM,"a.dIdt_negdI_spurM",b.dIdt_negdI_spurM,"b.dIdt_negdI_spurM",eps);
    UnitTests.checkDoubleRel(a.pFPI_posdI_spurM,"a.pFPI_posdI_spurM",b.pFPI_posdI_spurM,"b.pFPI_posdI_spurM",eps);
    UnitTests.checkDoubleRel(a.pFPI_negdI_spurM,"a.pFPI_negdI_spurM",b.pFPI_negdI_spurM,"b.pFPI_negdI_spurM",eps);

    // their sum matck OK
    UnitTests.checkDoubleRel(a.dPIdtF_spurM,"a.dPIdtF_spurM",b.dPIdtF_spurM,"b.dPIdtF_spurM",eps);


    System.err.println("IntervalSincePsiM test OK n="+a.I.neq+" eps="+eps);
}


static void matchFreeMoneyForAll(final IntervalSincePsiM a,final FreeMoneyForAll b,final double eps){


    System.err.println("IntervalSincePsiM.matchFreeMoneyForAll test OK n="+a.I.neq+" eps="+eps);
}

}
