package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.LebesgueQuadratureWithEVData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 *  @version $Id: IstatesLebesgue.java,v 1.4 2020/09/27 20:33:15 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IstatesLebesgue {
public final LebesgueQuadratureWithEVData L;
public final EVXData I;
    public final double p_offset,p_last,p_IH;
    final double [] Pk;

public IstatesLebesgue(final int n,
		       final double p_offset,final double p_last,
		       final double [] QQt,final double [] QQi,final double [] QQpi,
		       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		       final double eps)
{
    this.p_offset=p_offset;
    this.p_last=p_last;
    this.L=new LebesgueQuadratureWithEVData(n,QQi,QQt,M);
    // The https://arxiv.org/abs/1510.05510 answers:
    final int ixH=n-1; // maximal eigenvalue index
    this.I=EVXData.getEVXDataFromLebesgueQuadrature(L);
    final double [] pI=I.getFPsi(QQpi);
    this.Pk=new double [n];
    for(int k=0;k<n;k++){Pk[k]=pI[k]/I.ev[k];}
    this.p_IH=Pk[ixH];

}
static double getWLagrange2(final int s,final double p,final double [] pk){
    final int n=pk.length;
    double s2=1;
    for(int k=0;k<n;k++){
	if(k!=s){
	    s2*=p-pk[k];
	}
    }
    return Math.pow(s2,2);
}
}
