package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.LebesgueQuadratureWithEVData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 * Compare |(p-p_last)I| and |I| operator eigenvalues.
 *  @version $Id: IstatesPPl.java,v 1.6 2022/06/27 23:07:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IstatesPPl {
public final EVXData I;
public final double p_last,p_IH,PPl_IH,PPl_diag;

public IstatesPPl(final int n,final double p_last,
		  final double [] QQt,final double [] QQi,final double [] QQpi,
		  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		  final double eps)
{
    this.p_last=p_last;
    final int ixH=n-1; // maximal eigenvalue index
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    this.p_IH=I.getFPsi(ixH,QQpi)/I.ev[ixH];
    final double [] bpsiH=EVXData.convertPsiFromQtoB_Basis(n,I.getPsi(ixH),I.ev,QQt),
	bQQpnl=EVXData.convertMatrFromQtoB_Basis(n,TMatr.sumAcB(QQpi,-p_last,QQi),I.ev),
	// to check whether a solution exist
	ev_bQQpnl=M.EV.getEV(n,bQQpnl,true);
    PPl_IH=TMatr.getAMatrA(bpsiH,bQQpnl,n);
    PPl_diag=TMatr.mulV2V(EVXData.convertPsiFromQtoB_Basis(n,bpsiH,ev_bQQpnl,TMatr.getUnitMatrix(n)),java.util.Arrays.copyOf(ev_bQQpnl,n));
    if(eps>0){
	UnitTests.checkDoubleRel(PPl_IH,"PPl_IH",(p_IH-p_last)*I.sH,"(p_IH-p_last)*I.sH",eps);
	System.err.println("IstatesPPl: test OK n="+n+" eps="+eps);
    }
}
}
