package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Try select p-states first, then find max I among them. Similar to InIstatesPV.
 * @version $Id: IstatesPselection.java,v 1.5 2020/09/27 20:33:16 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class IstatesPselection {
public final double
    Imax,pm_Imax,pi_Imax,var2pm
	;
public final EVXData P;

public IstatesPselection(final int n,
		   final double [] QQt,
		   final double [] QQm,final double [] QQpm,
		   final double [] QQi,final double [] QQpi,
		   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		   final VolMatrix VOLATILITY_EDPsi,
		   final double eps)
{
    this.P=EVXData.getEVXDataFromQQ(n,QQpm,QQm,M);

    final double [] nrm=P.getFPsi(QQt),i=DeprecatedHelperTMatr.getRatio(n,P.getFPsi(QQi),nrm);
    int indimax=-1;
    for(int k=0;k<n;k++){
	if(indimax<0 || i[k]>=i[indimax]) indimax=k;
    }
    Imax=i[indimax];
    pm_Imax=P.ev[indimax];
    pi_Imax=P.getFPsi(indimax,QQpi)/P.getFPsi(indimax,QQi);

    final double [] spsi=P.getPsi(indimax),Dspsi=VOLATILITY_EDPsi.getDPsi(spsi),
	rq=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(spsi,Dspsi,null,QQpm,QQm,n);
    var2pm=rq[2];

    if(eps>0){
	UnitTests.checkDoubleRel(rq[0],"rq[0]",pm_Imax,"pm_Imax",eps);
	UnitTests.checkDoubleRel(rq[1],"rq[1]",0,"0",eps);
	System.err.println("IstatesPselection: test OK n="+n+" eps="+eps);
    }
}
    }
