package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.ProbabilityCorrelation;

/**
 *  @version $Id: IstatesSelection.java,v 1.36 2022/08/08 20:10:47 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class IstatesSelection {

public final EVXData I;
    public final double p_offset,p_IH,var2p_IH,distance_IH,
	pimax_IH,var2_pimax_IH,pimin_IH,
	var2_pdimax_0,var2_pdimax,
	p_diVmax,var2p_diVmax,
	p_iVmax,var2p_iVmax,
	p_ditmax,var2p_ditmax,
	p_IHwi,p_IHws,dpH
	;

public IstatesSelection(final int n,
			   final double p_offset,final double p_last,
			   final double [] QQt,final double [] QQi,final double [] QQpi,
			   final double [] QQdistance,
			   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			   final VolMatrix VOLATILITY_EDPsi,
			   final double eps)
{
    this.p_offset=p_offset;
    // The https://arxiv.org/abs/1510.05510 answers:
    final int ixH=n-1; // maximal eigenvalue index
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] w0ik=TMatr.squareArrayElements(I.psik_at_x0),
	Ikpsi02=TMatr.scaleVectorInPlace(TMatr.getVAB(w0ik,I.getEValues()),1/TMatr.sumElements(w0ik));
    final int indw0imax=com.polytechnik.utils.SortArrays.sortArrayIndexes(Ikpsi02)[n-1];
    p_IHwi=I.getFPsi(indw0imax,QQpi)/I.ev[indw0imax];
    double spw=0,sw=0;
    for(int k=0;k<n;k++){
	if(I.ev[k]>I.s0){
	    spw+=I.getFPsi(k,QQpi)*w0ik[k];
	    sw+=I.ev[k]*w0ik[k];
	}
    }
    p_IHws=spw/sw;
    final int [] indw0ik=com.polytechnik.utils.SortArrays.sortArrayIndexes(w0ik);
    double sdp=0;
    for(int kind=n;--kind>=1;){// loop:n-1..1
	final int kcur=indw0ik[kind],kprev=indw0ik[kind-1];
	if(I.ev[kcur]>=I.ev[kprev]){
	    sdp+=(I.getFPsi(kcur,QQpi)/I.ev[kcur]-I.getFPsi(kprev,QQpi)/I.ev[kprev])*w0ik[kcur];
	}
    }
    this.dpH=sdp;
    if(eps>0){
	UnitTests.checkDoubleRel(TMatr.sumElements(Ikpsi02),"I0",I.s0,"I.s0",eps);
    }


    final double []
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	bQQdistance=EVXData.convertMatrFromQtoB_Basis(n,QQdistance,I.ev),
	bIdist=TMatr.getMatrixDiagElements(n,bQQdistance),
	bQQp_diagI=new double [n*n],var2I=new double [n],var2p=new double [n],normdd=new double [n],te=new double [n],ONE=M.B.getONE();
    for(int k=0;k<n;k++) bQQp_diagI[k+n*k]=bQQpi[k+n*k]/I.ev[k];
    for(int k=0;k<n;k++){
	final double [] psik=I.getPsi(k),bDpsi=EVXData.convertPsiFromQtoB_Basis(n,VOLATILITY_EDPsi.getDPsi(psik),I.ev,QQt);
	bDpsi[k]=0; // remove psik projection, now bpsi and bDpsi are orthogonal 2D eigenvalues solution
	final double dd=normdd[k]=TMatr.mulVV(bDpsi,bDpsi);
	var2I[k]=TMatr.mulV2V(bDpsi,I.getEValues())/dd-I.ev[k];
	var2p[k]=TMatr.getAMatrA(bDpsi,bQQp_diagI,n)/dd-bQQp_diagI[k+n*k];
	te[k]=Math.pow(TMatr.getAMatrB(psik,ONE,QQt,n),2);
	if(eps>0){
	    final double [] psi=psik,Dpsi=VOLATILITY_EDPsi.getDPsi(psi),
		QQp=EVXData.convertMatrFromBtoQ_Basis(n,bQQp_diagI,I.ev,QQt),
		rqI=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQi,QQt,n),
		rqp=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQp,QQt,n);
	    UnitTests.checkDoubleRel(rqI[0],"rqI[0]",I.ev[k],"I.ev[k]",eps);
	    UnitTests.checkDoubleRel(rqI[1],"rqI[1]",0,"0",eps);
	    UnitTests.checkDoubleRel(rqI[2],"rqI[2]",var2I[k]*2*normdd[k],"var2I[k]*2*normdd[k]",eps);
	    UnitTests.checkDoubleRel(rqp[0],"rqp[0]",bQQp_diagI[k+n*k],"bQQp_diagI[k+n*k]",eps);
	    UnitTests.checkDoubleRel(rqp[1],"rqp[1]",0,"0",eps);
	    UnitTests.checkDoubleRel(rqp[2],"rqp[2]",var2p[k]*2*normdd[k],"var2p[k]*2*normdd[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,QQdistance,n),"<psi|QQdistance|psi>",bQQdistance[k+n*k],"bQQdistance[k+n*k]",eps);
	    UnitTests.checkDoubleRel(2*TMatr.getAMatrB(psi,Dpsi,QQt,n),"2*<psi|QQt|Dpsi>",Math.pow(I.psik_at_x0[k],2),"psik(x0)^2",eps);
	}
    }
    this.p_IH=bQQp_diagI[ixH+n*ixH];
    this.distance_IH=bQQdistance[ixH+n*ixH];
    this.var2p_IH=var2p[ixH];

    final int [] ind=com.polytechnik.utils.SortArrays.sortArrayIndexes(n,com.polytechnik.utils.IntPTRCompare.sortDescending(bIdist));

    int indimax=-1,indimin=-1;
    for(int indk:ind){
	final double v2Ik=var2I[indk];
	if(indk==ixH){indimax=indk;}
	else if(v2Ik<0) { // max I
	    // select the greatest max
	    if(indimax<0)indimax=indk;
	    else if(I.ev[indk]>=I.ev[indimax]) indimax=indk;
	} else if(indimax>=0 && v2Ik>0){ // min after max
	    indimin=indk;
	    break;
	}
    }
    pimax_IH=indimax>=0?bQQp_diagI[indimax+n*indimax]:Double.NaN;
    var2_pimax_IH=indimax>=0?var2p[indimax]:Double.NaN;
    pimin_IH=indimin>=0?bQQp_diagI[indimin+n*indimin]:Double.NaN;
    if(eps>0){
	System.err.println("IstatesSelection: indimax="+indimax+" indimin="+indimin+" bIdist="+java.util.Arrays.toString(bIdist)+" ind="+java.util.Arrays.toString(ind));
    }
    int inddimax=-1;
    for(int indk:ind){
	final double v2Ik=var2I[indk];
	if(v2Ik<0) { // max I
	    // select the greatest dimax
	    if(inddimax<0)inddimax=indk;
	    else if(var2I[indk]<=var2I[inddimax]) inddimax=indk;
	} else if(inddimax>=0 && v2Ik>0){ // min after max
	    break;
	}
    }
    var2_pdimax_0=inddimax>=0?var2p[inddimax]:Double.NaN;
    var2_pdimax=var2p[com.polytechnik.utils.SortArrays.sortArrayIndexes(var2I)[0]];
    final int diVmax=com.polytechnik.utils.SortArrays.sortArrayIndexes(TMatr.getVAB(te,var2I))[0];
    p_diVmax=bQQp_diagI[diVmax+n*diVmax];
    var2p_diVmax=var2p[diVmax];
    final int iVmax=com.polytechnik.utils.SortArrays.sortArrayIndexes(TMatr.getVAB(te,I.getEValues()))[0];
    p_iVmax=bQQp_diagI[iVmax+n*iVmax];
    var2p_iVmax=var2p[iVmax];
    final int ditmax=com.polytechnik.utils.SortArrays.sortArrayIndexes(TMatr.getVAB(te,TMatr.getVAB(te,TMatr.getVAB(normdd,var2I))))[0];
    p_ditmax=bQQp_diagI[ditmax+n*ditmax];
    var2p_ditmax=var2p[ditmax];
    final double [] w0=TMatr.squareArrayElements(I.psik_at_x0);
    double sp=0,swp=0,sv2p=0,swv2p=0;
    for(int k=0;k<n;k++){
	final double v2Ik=var2I[k],v2pk=var2p[k],w0k=w0[k];
	if(v2Ik<0){
	    sp+=w0k*bQQp_diagI[k+n*k];
	    swp+=w0k;
	    sv2p+=w0k*v2pk;
	    swv2p+=w0k;
	}
    }
    p_d2Im=sp/swp;
    v2p_d2Im=sv2p/swv2p;

    // to dump prices
    QQp_diagI=EVXData.convertMatrFromBtoQ_Basis(n,bQQp_diagI,I.ev,QQt);
    final double [] invSqrtbQQi=new double [n*n];
    for(int k=0;k<n;k++){
	invSqrtbQQi[k+n*k]=1/Math.sqrt(I.ev[k]);
    }
    final double [] bQQp=TMatr.mulMMM(n,invSqrtbQQi,bQQpi,invSqrtbQQi);
    QQp=EVXData.convertMatrFromBtoQ_Basis(n,bQQp,I.ev,QQt);
    // replace all diag elements by last price
    final double pl_m_poffset=p_last;// p_offset is already included in p_last;
    final double [] bQQpo=bQQp.clone();
    for(int k=0;k<n;k++) bQQpo[k+n*k]=pl_m_poffset;
    QQp_offdiagI=EVXData.convertMatrFromBtoQ_Basis(n,bQQpo,I.ev,QQt);
    dp0_offdiagI=TMatr.getAMatrA(I.psi0,QQp_offdiagI,n)-pl_m_poffset;

    final double [] psiIH=I.psiH,DpsiIH=VOLATILITY_EDPsi.getDPsi(psiIH);
    // try _offdiagI
    var1p_offdiagI=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psiIH,DpsiIH,null,QQp_offdiagI,QQt,n)[1];
    final ProbabilityCorrelation [] pc=new ProbabilityCorrelation[n];
    double spc=0,swpc=0;boolean flag_sum=true;
    for(int k=n;--k>=0;){
	final double [] psi=I.getPsi(k),Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	pc[k]=new ProbabilityCorrelation(
	    new double [] {
		TMatr.getAMatrB(psi,psi,QQt,n),
		TMatr.getAMatrB(psi,Dpsi,QQt,n),
		TMatr.getAMatrB(Dpsi,psi,QQt,n),
		TMatr.getAMatrB(Dpsi,Dpsi,QQt,n)
	    },new double []{
		TMatr.getAMatrB(psi,psi,QQp_offdiagI,n),
		TMatr.getAMatrB(psi,Dpsi,QQp_offdiagI,n),
		TMatr.getAMatrB(Dpsi,psi,QQp_offdiagI,n),
		TMatr.getAMatrB(Dpsi,Dpsi,QQp_offdiagI,n)
	    },
	    new double []{
		TMatr.getAMatrB(psi,psi,QQi,n),
		TMatr.getAMatrB(psi,Dpsi,QQi,n),
		TMatr.getAMatrB(Dpsi,psi,QQi,n),
		TMatr.getAMatrB(Dpsi,Dpsi,QQi,n)
	    });
	if(!(var2I[k]<0)){flag_sum=false;}
	else if(flag_sum) {// take maximums
	    //System.err.println("k="+k+" n="+n);
	    final double wk=(pc[k].iL-pc[k].iH)*(pc[k].pL-pc[k].pH);
	    spc+=wk*pc[k].corr;
	    swpc+=wk;
	}
    }
    pc_offdiagI=pc[ixH];
    pc_wh=spc/swpc;

    // try _offdiagInoIH
    final double [] bQQpo_offdiagInoIH=bQQp.clone();
    for(int k=0;k<n;k++) if(k!=ixH) bQQpo_offdiagInoIH[k+n*k]=pl_m_poffset;
    QQp_offdiagInoIH=EVXData.convertMatrFromBtoQ_Basis(n,bQQpo_offdiagInoIH,I.ev,QQt);
    final double [] rqpodIH=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psiIH,DpsiIH,null,QQp_offdiagInoIH,QQt,n);
    var1p_offdiagInoIH=rqpodIH[1];
    pc_offdiagInoIH=new ProbabilityCorrelation(
	    new double [] {
		TMatr.getAMatrB(psiIH,psiIH,QQt,n),
		TMatr.getAMatrB(psiIH,DpsiIH,QQt,n),
		TMatr.getAMatrB(DpsiIH,psiIH,QQt,n),
		TMatr.getAMatrB(DpsiIH,DpsiIH,QQt,n)
	    },new double []{
		TMatr.getAMatrB(psiIH,psiIH,QQp_offdiagInoIH,n),
		TMatr.getAMatrB(psiIH,DpsiIH,QQp_offdiagInoIH,n),
		TMatr.getAMatrB(DpsiIH,psiIH,QQp_offdiagInoIH,n),
		TMatr.getAMatrB(DpsiIH,DpsiIH,QQp_offdiagInoIH,n)
	    },
	    new double []{
		TMatr.getAMatrB(psiIH,psiIH,QQi,n),
		TMatr.getAMatrB(psiIH,DpsiIH,QQi,n),
		TMatr.getAMatrB(DpsiIH,psiIH,QQi,n),
		TMatr.getAMatrB(DpsiIH,DpsiIH,QQi,n)
	    });
    if(eps>0){
	UnitTests.checkDoubleRel(rqpodIH[0],"rqpodIH[0]",p_IH,"p_IH",eps);
	for(int k=0;k<n;k++){
	    UnitTests.checkDoubleRel(I.getFPsi(k,QQp_offdiagI),"<psik|QQp_offdiagI|psik>",pl_m_poffset,"p_last",eps);
	    if(k==ixH) UnitTests.checkDoubleRel(I.getFPsi(k,QQp_offdiagInoIH),"<psik|QQp_offdiagInoIH|psik>",p_IH,"p_IH",eps);
	    else UnitTests.checkDoubleRel(I.getFPsi(k,QQp_offdiagInoIH),"<psik|QQp_offdiagInoIH|psik>",pl_m_poffset,"p_last",eps);
	    for(int j=k+1;j<n;j++){
		UnitTests.checkDoubleRel(I.getFPsiPsi(k,j,QQp_diagI),"<psik|QQp_diagI|psij>",0,"0",eps);
	    }
	}
    }
    if(eps>0){
	System.err.println("IstatesSelection: test OK n="+n+" eps="+eps);
    }

}
final double [] QQp,QQp_diagI,QQp_offdiagI,QQp_offdiagInoIH;
final double dp0_offdiagI,p_d2Im,v2p_d2Im,var1p_offdiagI,var1p_offdiagInoIH,pc_wh;
final ProbabilityCorrelation pc_offdiagI,pc_offdiagInoIH;
}
