package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.freemoney.*;

/** In addition has an integrator with atau/2 time scale.
 * @version $Id: IvsVT_CalculateMoments_Laguerre.java,v 1.19 2022/07/28 08:04:53 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class IvsVT_CalculateMoments_Laguerre extends CommonlyUsedMomentsLaguerre {

    /** Integrator with tau/2.
     */
    private final WIntegrator wi2;

    IvsVT_CalculateMoments_Laguerre(final int n,final double atau){
	super(1+2*(n-1),atau);
	this.wi2=getEmptyWIntegrator(1+4*(n-1),atau/2,2);
    }

    @Override public void updateBasisToNewTimeReference(final long currenttime,final double new_p_offset){
	final double old_p_offset=getPOffset();
	super.updateBasisToNewTimeReference(currenttime,new_p_offset);
	final double dpoffset=new_p_offset-old_p_offset;
	TMatr.sumP1(wi2.integration_moments[1/* (P-p_offset)*dv */],wi2.integration_moments[0/* dv */],-dpoffset);
	// basis functions shift argument offset
	wi2.updateSavedMomentsToNewTimeReference(currenttime);
    }
    @Override public void addObservationNoBasisShift(final long currenttime,final long t,final double P,final double dv){
	// an intergrator with atau
	super.addObservationNoBasisShift(currenttime,t,P,dv);
	// an intergrator with atau/2
	wi2.addSingleObservation(currenttime,t,new double []{dv/*[0]*/,(P-getPOffset())*dv/*[1]*/});
    }
    double [] getPPHOrig2Moments(double p_eq){
	final double [] pimom2=TMatr.sumAcB(wi2.integration_moments[1],-p_eq,wi2.integration_moments[0]);
	// to have proper dt/tau  (tau is twice lower for odata.d2),
	// the getLaguerreExpFIntegrated assumes dt/tau integration and scale it, double the result to compensate
	TMatr.scaleVectorInPlace(pimom2,2);

	//System.err.println("wi2:"+java.util.Arrays.toString(wi2.integration_moments[0])+" sum[0]="+wi2.sum[0]+" wi2.currenntime="+wi2.currenttime_previous+" tau="+wi2.atau);
	// different basis: tau twice lower
	return KkQVMLaguerre.getMomentsConvertedArg(pimom2,2);
    }

}
