package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
/**
 * Helper class, like MomentsData, but different set of moments to calculate.
 *  @version $Id: JMomentsData.java,v 1.21 2022/06/17 20:37:51 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class JMomentsData {
    public final int n;
    public final double p_offset,p_last;
    public final double [] QQt,
	QQi,QQpi,QQp2i,QQp3i,
	QQt_sample,QQpt_sample,QQp2t_sample,QQp3t_sample,
	QQDpwv,QQDpwt,QQDawv,QQpDawv,QQp2Dawv,QQp3Dawv,QQDawt,
	QQad,QQa,QQpa,QQp2a,QQp3a,QQdpdt,
	QQFdp,QQFad,QQFDpwv,QQFi,QQFDpwt,QQFt_sample;
    JMomentsData(final int n,final double p_offset,final double p_last,final double [] QQt,
		 final double [] QQi,final double [] QQpi,final double [] QQp2i,final double [] QQp3i,
		 final double [] QQt_sample,final double [] QQpt_sample,final double [] QQp2t_sample,final double [] QQp3t_sample,
		 final double [] QQDpwv,final double [] QQDpwt,final double [] QQDawv,final double [] QQpDawv,final double [] QQp2Dawv,final double [] QQp3Dawv,final double [] QQDawt,
		 final double [] QQad,final double [] QQa,final double [] QQpa,final double [] QQp2a,final double [] QQp3a,
		 final double [] QQdpdt,
		 final double [] QQFdp,final double [] QQFad,final double [] QQFDpwv,final double [] QQFi,final double [] QQFDpwt,final double [] QQFt_sample)
    {
	this.n=n;
	this.p_offset=p_offset;
	this.p_last=p_last;
	this.QQt=QQt;
	this.QQi=QQi;
	this.QQpi=QQpi;
	this.QQp2i=QQp2i;
	this.QQp3i=QQp3i;
	this.QQt_sample=QQt_sample;
	this.QQpt_sample=QQpt_sample;
	this.QQp2t_sample=QQp2t_sample;
	this.QQp3t_sample=QQp3t_sample;
	this.QQDpwv=QQDpwv;
	this.QQDpwt=QQDpwt;
	this.QQDawv=QQDawv;
	this.QQpDawv=QQpDawv;
	this.QQp2Dawv=QQp2Dawv;
	this.QQp3Dawv=QQp3Dawv;
	this.QQDawt=QQDawt;
	this.QQad=QQad;
	this.QQa=QQa;
	this.QQpa=QQpa;
	this.QQp2a=QQp2a;
	this.QQp3a=QQp3a;
	this.QQdpdt=QQdpdt;
	this.QQFdp=QQFdp;
	this.QQFad=QQFad;
	this.QQFDpwv=QQFDpwv;
	this.QQFi=QQFi;
	this.QQFDpwt=QQFDpwt;
	this.QQFt_sample=QQFt_sample;
    }

/** Calculate the moments in time basis.
 * QQi_jk={@literal <Q_j I Q_k>} , QQt_jk={@literal <Q_j Q_k>}, QQpi_jk={@literal <Q_j pI Q^k>},
 */
public static <DTYPE> JMomentsData get_JMomentsData(final int n,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final DTYPE odata,final DataInterfaceToMoments<DTYPE> access)
{
    final int nmoms=2*n-1;
    final double [] mom=new double [nmoms],mmom=new double [nmoms];
    access.setMT(mom,nmoms);final double [] QQt=M.B.getQQMatr(n,mom);
    access.setM(mom,0,access.M_T,odata,nmoms);
    access.setFMoments(mmom,mom,nmoms);final double [] QQFt_sample=M.B.getQQMatr(n,mmom);
    access.setM(mom,0,access.M_I,odata,nmoms);final double [] QQi=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQFi=M.B.getQQMatr(n,mmom);
    access.setM(mom,1,access.M_I,odata,nmoms);final double [] QQpi=M.B.getQQMatr(n,mom);
    access.setM(mom,2,access.M_I,odata,nmoms);final double [] QQp2i=M.B.getQQMatr(n,mom);
    access.setM(mom,3,access.M_I,odata,nmoms);final double [] QQp3i=M.B.getQQMatr(n,mom);
    access.setM(mom,0,access.M_T,odata,nmoms);final double [] QQt_sample=M.B.getQQMatr(n,mom);
    access.setM(mom,1,access.M_T,odata,nmoms);final double [] QQpt_sample=M.B.getQQMatr(n,mom);
    access.setM(mom,2,access.M_T,odata,nmoms);final double [] QQp2t_sample=M.B.getQQMatr(n,mom);
    access.setM(mom,3,access.M_T,odata,nmoms);final double [] QQp3t_sample=M.B.getQQMatr(n,mom);
    access.setM(mom,0,access.M_DPwV,odata,nmoms);final double [] QQDpwv=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQFDpwv=M.B.getQQMatr(n,mmom);
    access.setM(mom,0,access.M_DPwT,odata,nmoms);final double [] QQDpwt=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQFDpwt=M.B.getQQMatr(n,mmom);
    access.setM(mom,0,access.M_DAwV,odata,nmoms);final double [] QQDawv=M.B.getQQMatr(n,mom);
    access.setM(mom,1,access.M_DAwV,odata,nmoms);final double [] QQpDawv=M.B.getQQMatr(n,mom);
    access.setM(mom,2,access.M_DAwV,odata,nmoms);final double [] QQp2Dawv=M.B.getQQMatr(n,mom);
    access.setM(mom,3,access.M_DAwV,odata,nmoms);final double [] QQp3Dawv=M.B.getQQMatr(n,mom);
    access.setM(mom,0,access.M_DAwT,odata,nmoms);final double [] QQDawt=M.B.getQQMatr(n,mom);
    access.setM(mom,0,access.M_AD,odata,nmoms);final double [] QQad=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQFad=M.B.getQQMatr(n,mmom);
    access.setM(mom,0,access.M_A,odata,nmoms);final double [] QQa=M.B.getQQMatr(n,mom);
    access.setM(mom,1,access.M_A,odata,nmoms);final double [] QQpa=M.B.getQQMatr(n,mom);
    access.setM(mom,2,access.M_A,odata,nmoms);final double [] QQp2a=M.B.getQQMatr(n,mom);
    access.setM(mom,3,access.M_A,odata,nmoms);final double [] QQp3a=M.B.getQQMatr(n,mom);
    access.setM(mom,1,access.M_DP,odata,nmoms);final double [] QQdpdt=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQFdp=M.B.getQQMatr(n,mmom);

    return new JMomentsData(n,access.getPOffset(odata),access.getPLast(odata),QQt,
			    QQi,QQpi,QQp2i,QQp3i,
			    QQt_sample,QQpt_sample,QQp2t_sample,QQp3t_sample,
			    QQDpwv,QQDpwt,QQDawv,QQpDawv,QQp2Dawv,QQp3Dawv,QQDawt,
			    QQad,QQa,QQpa,QQp2a,QQp3a,QQdpdt,
			    QQFdp,QQFad,QQFDpwv,QQFi,QQFDpwt,QQFt_sample);
}

/** Calculate moments in p^k basis. QQi_jk={@literal <p^j I p^k>},
 * QQt_jk={@literal <p^j p^k>}, QQpi_jk={@literal <p^j pI p^k>}.
 */
public static <DTYPE> JMomentsData get_PriceMonomials_JMomentsData(final int n,final DTYPE odata,final DataInterfaceToMoments<DTYPE> access)
{
    final OrthogonalPolynomialsMonomialsBasis M=com.polytechnik.utils.DiffSkewness.MB;
    final int nmoms=2*n+2;
    final double [] r=new double [1],imom=new double [nmoms],tmom=new double [nmoms],Dpwv_mom=new double [nmoms],Dpwt_mom=new double [nmoms],Dawv_mom=new double [nmoms],Dawt_mom=new double [nmoms],admom=new double [nmoms],amom=new double [nmoms],dpdtmom=new double [nmoms-1];
    for(int k=0;k<nmoms;k++)
    {
	access.setM(r,k,access.M_I,odata,1);imom[k]=r[0];
	access.setM(r,k,access.M_T,odata,1);tmom[k]=r[0];
	access.setM(r,k,access.M_DPwV,odata,1);Dpwv_mom[k]=r[0];
	access.setM(r,k,access.M_DPwT,odata,1);Dpwt_mom[k]=r[0];
	access.setM(r,k,access.M_DAwV,odata,1);Dawv_mom[k]=r[0];
	access.setM(r,k,access.M_DAwT,odata,1);Dawt_mom[k]=r[0];
	access.setM(r,k,access.M_AD,odata,1);admom[k]=r[0];
	access.setM(r,k,access.M_A,odata,1);amom[k]=r[0];
	if(k>0)
	{
	    // In monomials basis: d(p^k)=kp^{k-1}dp.
	    access.setM(r,k,access.M_DP,odata,1);dpdtmom[k-1]=r[0]/k;
	}
    }
    final double [] QQt=M.B.getQQMatr(n,tmom),
	pimom=M.B.getXQkMomentsFromQkMoments(imom),p2imom=M.B.getXQkMomentsFromQkMoments(pimom),p3imom=M.B.getXQkMomentsFromQkMoments(p2imom),
	ptmom=M.B.getXQkMomentsFromQkMoments(tmom),p2tmom=M.B.getXQkMomentsFromQkMoments(ptmom),p3tmom=M.B.getXQkMomentsFromQkMoments(p2tmom),
	pamom=M.B.getXQkMomentsFromQkMoments(amom),p2amom=M.B.getXQkMomentsFromQkMoments(pamom),p3amom=M.B.getXQkMomentsFromQkMoments(p2amom),
	pDawv_mom=M.B.getXQkMomentsFromQkMoments(Dawv_mom),p2Dawv_mom=M.B.getXQkMomentsFromQkMoments(pDawv_mom),p3Dawv_mom=M.B.getXQkMomentsFromQkMoments(p2Dawv_mom);
    return new JMomentsData(n,access.getPOffset(odata),access.getPLast(odata),QQt,
			    M.B.getQQMatr(n,imom),M.B.getQQMatr(n,pimom),M.B.getQQMatr(n,p2imom),M.B.getQQMatr(n,p3imom),
			    QQt,M.B.getQQMatr(n,ptmom),M.B.getQQMatr(n,p2tmom),M.B.getQQMatr(n,p3tmom),
			    M.B.getQQMatr(n,Dpwv_mom),M.B.getQQMatr(n,Dpwt_mom),
			    M.B.getQQMatr(n,Dawv_mom),M.B.getQQMatr(n,pDawv_mom),M.B.getQQMatr(n,p2Dawv_mom),M.B.getQQMatr(n,p3Dawv_mom),
			    M.B.getQQMatr(n,Dawt_mom),
			    M.B.getQQMatr(n,admom),M.B.getQQMatr(n,amom),M.B.getQQMatr(n,pamom),M.B.getQQMatr(n,p2amom),M.B.getQQMatr(n,p3amom),
			    M.B.getQQMatr(n,dpdtmom),
			    null,null,null,null,null,null);
}

}
