package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.realtimebook.*;

/** First and Second derivate estimation using orhogonal polynomials.
 *  @version $Id: JkQVM.java,v 1.40 2022/06/30 08:18:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class JkQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;

final EVXData I,P1t,P2t;
final double P1t_Il,P1t_Ih,Im[],Imax,Imax1,P1t_PImax,P1t_PImax1;

final double psi_dp,psi_X,fP0,sigma2_fP0,fP1,sigma2_fP1,fP2,sigma2_fP2;
final double [] psi;

JkQVM(final int n,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();
    this.M=getABasis();


    final int nmoms=2*n-1;
    final double [] tmom=new double [nmoms+1],imom=new double [nmoms];
    access.setMT(tmom,nmoms+1);
    access.setM(imom,0,access.M_I,odata,nmoms);
    final double [] QQt=M.B.getQQMatr(n,tmom),QQi=M.B.getQQMatr(n,imom);

    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,getX0());

    final double [][] MM=getFMoments();
    final double [] dpmom=new double [nmoms];
    access.setDFMoments(dpmom,MM[1],nmoms); for(int k=nmoms;--k>=0;){dpmom[k]=-dpmom[k];}
    final double [] QQp0=M.B.getQQMatr(n,MM[0]),QQp1=M.B.getQQMatr(n,MM[1]),QQp2=M.B.getQQMatr(n,MM[2]);
    this.P2t=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQp2,QQt,access,getX0());
    this.P1t=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQp1,QQp0,access,getX0());
    this.P1t_Il=TMatr.getAMatrA(P1t.psiL.length,P1t.psiL,QQi,n);
    this.P1t_Ih=TMatr.getAMatrA(P1t.psiH.length,P1t.psiH,QQi,n);
    this.Im=new double [n];
    for(int k=0;k<P1t.neq;k++)
    {
	Im[k]=TMatr.getAMatrB(n,P1t.ev,n+k*n,n,P1t.ev,n+k*n,QQi,n);
    }
    final int [] indIm=com.polytechnik.utils.SortArrays.sortArrayIndexes(Im);
    if(n>=1)
    {
	Imax=Im[indIm[indIm.length-1]];
	P1t_PImax=P1t.ev[indIm[indIm.length-1]];
    }
    else{Imax=P1t_PImax=Double.NaN;}
    if(n>=2)
    {
	Imax1=Im[indIm[indIm.length-2]];
	P1t_PImax1=P1t.ev[indIm[indIm.length-2]];
    }
    else{Imax1=P1t_PImax1=Double.NaN;}

    this.psi=getPsi(n,QQi,QQt,QQp2);

    final double [] psi2=M.B.mult2Pol(psi,psi);
    this.psi_dp=TMatr.mulVVn1(psi2,dpmom);
    this.psi_X=TMatr.mulVVn1(psi2,M.B.getXQkMomentsFromQkMoments(tmom));

    final double [] f0mom=M.B.getPQkMomentsFromQkMoments(psi2,MM[0]),
	f1mom=M.B.getPQkMomentsFromQkMoments(psi2,MM[1]),
	f2mom=M.B.getPQkMomentsFromQkMoments(psi2,MM[2]);
    final double [][] QP=M.getOrthogonalPolynomialsFirstKind(3,f0mom);
    final double nm=1/Math.pow(M.B.getD0(QP[0],0),2);
    this.fP0=TMatr.mulVVn1(QP[0],f1mom)/nm;
    this.sigma2_fP0=TMatr.mulVVn1(M.B.mult2Pol(QP[0],QP[0]),f2mom)/nm-this.fP0*this.fP0;
    this.fP1=TMatr.mulVVn1(QP[1],f1mom)/nm;
    this.sigma2_fP1=TMatr.mulVVn1(M.B.mult2Pol(QP[1],QP[1]),f2mom)/nm-this.fP1*this.fP1;
    this.fP2=TMatr.mulVVn1(QP[2],f1mom)/nm;
    this.sigma2_fP2=TMatr.mulVVn1(M.B.mult2Pol(QP[2],QP[2]),f2mom)/nm-this.fP2*this.fP2;

    sQP=QP;

}
final double [][] sQP;

double [] getPsi(final int n,final double [] QQi,final double [] QQt,final double [] QQp2){
    if(false) return new double []{1};
    if(false) return I.psiH;
    return P2t.psiL;
}
double [][] getFMoments()
{
    final int nmoms=(2*I.neq-1)+4;
    if(true)
    {
    final double [] Ptmom=new double [nmoms],P2tmom=new double [nmoms],
	dp0mom=new double [nmoms],dp1mom=new double [nmoms],dp2mom=new double [nmoms];
    access.setM(dp0mom,0,access.M_DP,odata,nmoms);
    access.setM(dp1mom,1,access.M_DP,odata,nmoms);
    access.setM(dp2mom,2,access.M_DP,odata,nmoms);
    final double pl_m_poffset=getPLast()-getPOffset();
    Polynomials.shiftPOffsetInPlace(nmoms,dp0mom,dp1mom,dp2mom,null,pl_m_poffset);
    access.setFMoments(Ptmom,dp1mom,nmoms);
    access.setFMoments(P2tmom,dp2mom,nmoms);
    access.setMT(dp0mom,nmoms);
    return new double [][]{dp0mom,Ptmom,P2tmom};
    }

    final double [] p0imom=new double [nmoms],p1imom=new double [nmoms],p2imom=new double [nmoms];
    access.setM(p0imom,0,access.M_I,odata,nmoms);
    access.setM(p1imom,1,access.M_I,odata,nmoms);
    access.setM(p2imom,2,access.M_I,odata,nmoms);
    return new double [][]{p0imom,p1imom,p2imom};
}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();

abstract double getX0();
abstract double getPLast();
abstract double getPOffset();

void test_DkQVMWpPsi(final DkQVM.WpPsi q,final double eps)
{
    EVXData.matchEVXData(P1t,q.Px,false,false,eps);
    UnitTests.checkDoubleRel(P1t_Il,"P1t_Il",q.Im[0],"q.Im[0]",eps);
    UnitTests.checkDoubleRel(P1t_Ih,"P1t_Ih",q.Im[q.Im.length-1],"q.Im[q.Im.length-1]",eps);
    UnitTests.checkDoubleArrayRel(Im,"Im",q.Im,"q.Im",eps);
    if(Imax1>Imax) throw new RuntimeException("Imax1="+Imax1+" Imax="+Imax);

    System.err.println("JkQVML:test_DkQVMWpPsi: test OK neq="+P1t.neq+" eps="+eps);

}



/** For unit test.
 */
static abstract class JkQVMTest<DTYPE> extends QTest {

abstract JkQVM<DTYPE> getJkQVM();
abstract DkQVM<?> getDkQVM();


JkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}

void test_Polynomials()
{
    final JkQVM<DTYPE> q=getJkQVM();

    final double [][] MM=q.getFMoments();
    final double [] psi=q.psi,psi2=q.M.B.mult2Pol(psi,psi);
    for(int k=0;k<q.sQP.length;k++)
    {
	for(int l=0;l<=k;l++)
	{
	    final double [] pp=q.M.B.mult2Pol(q.M.B.mult2Pol(q.sQP[l],q.sQP[k]),psi2);
	    UnitTests.checkDoubleRel(TMatr.mulVVn1(pp,MM[0]),"<Ql|Qk>",l==k?1:0,l==k?"1":"0",eps);
	}
    }

    final double P0=q.M.B.getD0(q.sQP[0],0),n0=TMatr.mulVVn1(psi2,MM[0]),
	fP0=P0*TMatr.mulVVn1(psi2,MM[1])/n0,
	sigma2_fP0=(TMatr.mulVVn1(psi2,MM[2])/n0)*P0*P0-Math.pow(fP0,2);
    UnitTests.checkDoubleRel(fP0,"fP0",q.fP0,"q.fP0",eps);
    UnitTests.checkDoubleRel(sigma2_fP0,"sigma2_fP0",q.sigma2_fP0,"q.sigma2_fP0",eps);

    final double [] xf0=q.M.B.getXQkMomentsFromQkMoments(MM[0]);
    final double xa=TMatr.mulVVn1(psi2,xf0)/n0;
    final double [] xpsi2=q.M.getxP(psi2),x2psi2=q.M.getxP(xpsi2);
    for(int k=xpsi2.length;--k>=0;){
	x2psi2[k]+=xa*(-2*xpsi2[k]+(k<psi2.length?xa*psi2[k]:0));
	xpsi2[k]+=-(k<psi2.length?xa*psi2[k]:0);
    }
    final double n1=TMatr.mulVVn1(x2psi2,MM[0]),a1=1/Math.sqrt(n1),
	fP1=a1*(TMatr.mulVVn1(xpsi2,MM[1])/n0),
	sigma2_fP1=a1*a1*(TMatr.mulVVn1(x2psi2,MM[2])/n0)-fP1*fP1;
    UnitTests.checkDoubleRel(fP1,"fP1",q.fP1,"q.fP1",eps);
    UnitTests.checkDoubleRel(sigma2_fP1,"sigma2_fP1",q.sigma2_fP1,"q.sigma2_fP1",eps);

    System.err.println(name+":JkQVMTest:test_Polynomials(): test OK neq="+neq+" eps="+eps);
}


void test_DkQVM()
{
    final DkQVM<?> s=getDkQVM();
    final JkQVM<DTYPE> q=getJkQVM();

    q.test_DkQVMWpPsi(s.P1t,eps);

    System.err.println(name+":JkQVMTest:test_DkQVM(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":JkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Polynomials();
    test_DkQVM();

    System.err.println(name+":JkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":JkQVMTest: starting tests; ver=$Id: JkQVM.java,v 1.40 2022/06/30 08:18:24 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":JkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: JkQVM.java,v 1.40 2022/06/30 08:18:24 mal Exp $");
}
} // END OF class JkQVMTest


}// END OF class JkQVM
