package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;

abstract class K3Distance {

final double [] psiPnlL,psiPnlH,psiE;

final double w_LEH,w_HEL,w_LHE,w_HLE,w_ELH,w_EHL;

K3Distance(double [] psiPnlL,double [] psiPnlH,double [] psiE)
{
    this.psiPnlL=psiPnlL;
    this.psiPnlH=psiPnlH;
    this.psiE=psiE;

    w_LEH=getK3(psiPnlH,psiE,psiPnlL);
    w_HEL=getK3(psiPnlL,psiE,psiPnlH);
    w_LHE=getK3(psiE,psiPnlH,psiPnlL);
    w_HLE=getK3(psiE,psiPnlL,psiPnlH);
    w_ELH=getK3(psiPnlH,psiPnlL,psiE);
    w_EHL=getK3(psiPnlL,psiPnlH,psiE);

}

/** Returns norm at arg(a)>arg(b)>arg(c).
 */
abstract double getK3(final double [] a,final double [] b,final double [] c);


void test_Self(final double [] QQt,final double eps)
{
    UnitTests.checkDoubleRel(w_LEH+w_HEL+w_LHE+w_HLE+w_ELH+w_EHL,"sum(w)",1,"1",eps);
    final int n=this.psiE.length;
    UnitTests.checkDoubleRel(TMatr.getAMatrB(psiE,psiPnlH,QQt,n),"<psiE|psiPnlH>",0,"0",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(psiE,psiPnlL,QQt,n),"<psiE|psiPnlL>",0,"0",eps);
    if(n>2) UnitTests.checkDoubleRel(TMatr.getAMatrB(psiPnlL,psiPnlH,QQt,n),"<psiPnlL|psiPnlH>",0,"0",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(psiE,psiE,QQt,n),"<psiE|psiE>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(psiPnlL,psiPnlL,QQt,n),"<psiPnlL|psiPnlL>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(psiPnlH,psiPnlH,QQt,n),"<psiPnlH|psiPnlH>",1,"1",eps);

    System.err.println("K3Distance:test_Self(): test OK n="+n+" eps="+eps);
}

}
