package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Quadrature;
import com.polytechnik.utils.RecurrenceAB;
import com.polytechnik.realtimebook.*;

/** Impact QVM calculations with the basis of second kind.
 *  @version $Id: KQVM.java,v 1.121 2022/07/28 10:18:50 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class KQVM<DTYPE> {

final double [] x_lexcess,w_lexcess,x_ldeficit,w_ldeficit,dp_lexcess,dp_ldeficit,xr_ldeficit,wr_ldeficit;
final double dp_ld,dp_nld,dp_le,dp_nle,dp_ll;
final double [] apnodes;


final double df;
final double [] x_excess,x_deficit;
final double [] xnodes,wnodes,dpnodes,dipnodes,sxnodes,swnodes,sdpnodes,sdipnodes;
final double [] axnodes,awnodes,adpnodes,adipnodes,saxnodes,sawnodes,sadpnodes,sadipnodes;

final DTYPE odata;
final DataInterfaceToMoments<DTYPE> access;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> basis;
KQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();
    this.basis=getBasis();

    final double x0;
    if(true)
    {
	final double [] tmp=basis.getRadauMultiplied(new double []{1});
	x0=-basis.B.getD0(tmp,0)/basis.B.getD1(tmp,0);
    }

    if(true)
    {
	final double [] mom=new double [2*neq+5];
	access.setM(mom,0,access.M_I,odata,mom.length);
	final double [][] p=basis.getOrthogonalPolynomialsFirstKind(neq+3,mom);
	final RecurrenceAB abp=basis.getAB(p);
	final RecurrenceAB ab=basis.getABRadauModified(p.length-1,new RecurrenceAB(DeprecatedHelperTMatr.remove0Element(abp.getAllAk()),DeprecatedHelperTMatr.remove0Element(abp.getAllBk())));
	final double [][] q=basis.getOrthogonalPolynomialsFromAB(0,ab);

	final double [] l_deficit=q[q.length-1];
	xr_ldeficit=basis.getXNodesFromAB(l_deficit.length-1,ab);
	sortNodes(xr_ldeficit,null);
	wr_ldeficit=basis.getWeightsChristoffel(xr_ldeficit,q);
	final double fd=Math.pow(basis.B.getD0(q[0],0),2);for(int k=wr_ldeficit.length;--k>=0;) wr_ldeficit[k]*=fd;

    }
    if(true)
    {
	final double [] mom=new double [2*neq+3];

	access.setM(mom,0,access.M_I,odata,mom.length);
	final double [][] p=basis.getOrthogonalPolynomialsFirstKind(neq+2,mom);

	// set price moments
	access.setM(mom,1,access.M_DPPLUS,odata,1);
	final double dpp0=mom[0];
	access.setM(mom,1,access.M_DPMINUS,odata,1);
	final double d0=dpp0-mom[0];
	access.setM(mom,1,access.M_DP,odata,mom.length);
	for(int k=mom.length;--k>=0;) mom[k]/=d0;

	// simplistic extrapolation
	final double [] nlef=p[neq+1];
	dp_nle=TMatr.mulVVn1(nlef,mom)/basis.B.getD0(nlef,x0);
	final double [] ndef=basis.getSecondKindOrthogonalPolynomialsFromFirstKind(p)[neq];
	dp_nld=TMatr.mulVVn1(ndef,mom)/basis.B.getD0(ndef,x0);

	// more complex Radau approach
	if(true)
	{
	    final RecurrenceAB p_ab=basis.getAB(p);
	    final RecurrenceAB ab=basis.getABRadauModified(p.length,p_ab);
	    final double [][] r=basis.getOrthogonalPolynomialsFromAB(0,ab),
		q=basis.getOrthogonalPolynomialsFromAB(1,ab);

	    final double [] l_excess=r[r.length-1],l_deficit=q[q.length-1];
	    dp_ld=TMatr.mulVVn1(l_deficit,mom)/basis.B.getD0(l_deficit,x0);
	    final double [] letmp=basis.B.sdiv1(l_excess,x0).q;
	    dp_le=TMatr.mulVVn1(letmp,mom)/basis.B.getD0(letmp,x0);

	    x_lexcess=basis.getXNodesFromAB(l_excess.length-1,ab);
	    x_ldeficit=basis.getXNodesFromAB(l_deficit.length-1,new RecurrenceAB(DeprecatedHelperTMatr.remove0Element(ab.getAllAk()),DeprecatedHelperTMatr.remove0Element(ab.getAllBk())));
	    sortNodes(x_lexcess,null);
	    sortNodes(x_ldeficit,null);
	    w_lexcess=basis.getWeightsChristoffel(x_lexcess,r);
	    final double fe=Math.pow(basis.B.getD0(p[0],0),2);for(int k=w_lexcess.length;--k>=0;) w_lexcess[k]*=fe;
	    w_ldeficit=basis.getWeightsChristoffel(x_ldeficit,q);
	    final double fd=Math.pow(basis.B.getD0(q[0],0),2);for(int k=w_ldeficit.length;--k>=0;) w_ldeficit[k]*=fd;

	    dp_lexcess=basis.getWeightsByIntegration(l_excess,x_lexcess,mom);
	    dp_ldeficit=basis.getWeightsByIntegration(l_deficit,x_ldeficit,mom);

	    final double [] nodes=getMult2Pol(l_excess,l_deficit);
	    final double [] xn=new double [x_lexcess.length+x_ldeficit.length];
	    System.arraycopy(x_lexcess,0,xn,0,x_lexcess.length);
	    System.arraycopy(x_ldeficit,0,xn,x_lexcess.length,x_ldeficit.length);
	    sortNodes(xn,null);
	    final double [] dp_int=basis.getWeightsByIntegration(nodes,xn,mom);
	    dp_ll=dp_int[0];
	}
    }
    if(true)
    {
	final double [] mom=new double [2*neq+3];
	access.setM(mom,0,access.M_I,odata,mom.length);
	final double [][] p=basis.getRadauPolynomials(basis.getOrthogonalPolynomialsFirstKind(neq+2,mom));
	final RecurrenceAB ab=basis.getAB(p);
	final double [] l_excess=p[neq];
	final double [][] q=basis.getOrthogonalPolynomialsFromAB(1,ab);
	final double [] l_deficit=q[neq-1];

	final double [] nodes=getMult2Pol(l_excess,l_deficit);

	access.setM(mom,1,access.M_DPPLUS,odata,1);
	final double dpp0=mom[0];
	access.setM(mom,1,access.M_DPMINUS,odata,1);
	final double d0=dpp0-mom[0];
	access.setM(mom,1,access.M_DP,odata,mom.length);
	for(int k=mom.length;--k>=0;) mom[k]/=d0;
	this.df=TMatr.mulVVn1(nodes,mom)/basis.B.getD0(nodes,x0);

	if(true)
	{
	x_excess=basis.getXNodesFromAB(l_excess.length-1,ab);
	x_deficit=basis.getXNodesFromAB(l_deficit.length-1,new RecurrenceAB(DeprecatedHelperTMatr.remove0Element(ab.getAllAk()),DeprecatedHelperTMatr.remove0Element(ab.getAllBk())));
	sortNodes(x_excess,null);
	sortNodes(x_deficit,null);
	}
    }

    final double [] mom=new double [2*neq+5];
    access.setM(mom,0,access.M_I,odata,mom.length);
    final double [][] p=basis.getOrthogonalPolynomialsFirstKind(neq+3,mom);
    final Quadrature.GaussQuadratureWithNodesPolynomialData nxwq=basis.getRadauQuadratureFromPolynomials(neq,p);
    final double [][] nxw=LQVM.scaleWInd1(new double [][]{nxwq.x,nxwq.w,nxwq.nodespolynomial});
    sortNodes(xnodes=nxw[0],wnodes=nxw[1]);
    final Quadrature.GaussQuadratureWithNodesPolynomialData sxwq=basis.getRadauQuadratureFromPolynomials(neq,basis.getSecondKindOrthogonalPolynomialsFromFirstKind(p));
    final double [][] sxw=LQVM.scaleWInd1(new double [][]{sxwq.x,sxwq.w,sxwq.nodespolynomial});
    sortNodes(sxnodes=sxw[0],swnodes=sxw[1]);

    final Quadrature.GaussQuadratureWithNodesPolynomialData naxwq=basis.getGaussQuadratureFromPolynomials(neq,p);
    final double [][] naxw=LQVM.scaleWInd1(new double [][]{naxwq.x,naxwq.w,naxwq.nodespolynomial});
    sortNodes(axnodes=naxw[0],awnodes=naxw[1]);
    final Quadrature.GaussQuadratureWithNodesPolynomialData snaxwq=basis.getGaussQuadratureFromPolynomials(neq,basis.getSecondKindOrthogonalPolynomialsFromFirstKind(p));
    final double [][] snaxw=LQVM.scaleWInd1(new double [][]{snaxwq.x,snaxwq.w,snaxwq.nodespolynomial});
    sortNodes(saxnodes=snaxw[0],sawnodes=snaxw[1]);
    if(true)
    {
	final double p_offset=getPOffset(),v0=mom[0];
	access.setM(mom,1,access.M_I,odata,mom.length);
	for(int k=mom.length;--k>=0;) {mom[k]/=v0;}
	apnodes=basis.getWeightsByIntegration(naxw[2],axnodes,mom);
	for(int k=apnodes.length;--k>=0;) apnodes[k]=p_offset+apnodes[k]/awnodes[k];
    }


    if(true){
	access.setM(mom,1,access.M_DPPLUS,odata,1);
	final double dpp0=mom[0];
	access.setM(mom,1,access.M_DPMINUS,odata,1);
	final double d0=dpp0-mom[0];
	access.setM(mom,1,access.M_DP,odata,mom.length);
	for(int k=mom.length;--k>=0;) mom[k]/=d0;
	dpnodes=basis.getWeightsByIntegration(nxw[2],xnodes,mom);
	sdpnodes=basis.getWeightsByIntegration(sxw[2],sxnodes,mom);
	adpnodes=basis.getWeightsByIntegration(naxw[2],axnodes,mom);
	sadpnodes=basis.getWeightsByIntegration(snaxw[2],saxnodes,mom);

	final double [] e=new double [neq+1];
	access.setM(mom,0,access.M_ISELL,odata,neq+1);
	access.setM(e,0,access.M_IBUY,odata,neq+1);
	final double vv0=mom[0]+e[0];
	for(int k=neq+1;--k>=0;) {e[k]=(mom[k]-e[k])/vv0;}
	dipnodes=basis.getWeightsByIntegration(nxw[2],xnodes,e);
	sdipnodes=basis.getWeightsByIntegration(sxw[2],sxnodes,e);
	adipnodes=basis.getWeightsByIntegration(naxw[2],axnodes,e);
	sadipnodes=basis.getWeightsByIntegration(snaxw[2],saxnodes,e);
    }
}

abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getBasis();
abstract void sortNodes(final double [] x,final double [] w);
abstract double [] getMult2Pol(final double [] x,final double [] y);
abstract double getPOffset();

static abstract class KQVMTest<DTYPE> {

final com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis M;
int neq;
long currenttime,tau;
Trade [] tb;
abstract void setData(final int tsize,final int neq);

abstract KQVM<DTYPE> getKQVM();
abstract void test_Moments();
abstract void test_RQVM();
abstract void test_SQVM();

final String name;
final java.util.Random r;
final double eps;
KQVMTest(final String name,
	 final com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis M,
	 final java.util.Random r,final double eps)
{
    this.M=M;
    this.name=name;
    this.r=r;
    this.eps=eps;
}
abstract double [] convertMonomialsToBasis(final double [] m);
double [] convertBasisMomentsToMonomialsMoments(final double [] b)
{
    final double [] m=new double [b.length];
    for(int k=0;k<b.length;k++)
    {
	final double [] t=new double [k+1];
	t[k]=1;
	m[k]=TMatr.mulVVn1(convertMonomialsToBasis(t),b);
    }
    return m;
}

void test_DFMod(final KQVM<DTYPE> q,final double [] mom_p,final double [] mom_dp,final double x0)
{
    UnitTests.checkDoubleArrayRel(q.x_lexcess,"q.x_lexcess",q.xnodes,"q.xnodes",eps);
    UnitTests.checkDoubleArrayRel(q.w_lexcess,"q.w_lexcess",q.wnodes,"q.wnodes",eps);
    UnitTests.checkDoubleArrayRel(q.dp_lexcess,"q.dp_lexcess",q.dpnodes,"q.dpnodes",eps);
    UnitTests.checkDoubleRel(TMatr.sumElements(q.w_lexcess),"sum(q.w_lexcess)",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.sumElements(q.w_ldeficit),"sum(q.w_ldeficit)",1,"1",eps);
    for(int k=0;k<q.x_ldeficit.length;k++)
    { // there should be exactly one root of type 2 polynomial between two roots of type 1 polynomial
	double xd=q.x_ldeficit[k],xL=q.x_lexcess[k],xR=q.x_lexcess[k+1];
	if(xL>xR){final double tmp=xL;xL=xR;xR=tmp;}
	if(!(xL<xd && xd<xR)) throw new RuntimeException("xL="+xL+" xR="+xR+" xd="+xd);
    }
    final double [] def;
    {
     final RecurrenceAB mp_ab=M.getAB(M.getOrthogonalPolynomialsFirstKind(neq+2,mom_p));
     final RecurrenceAB ab=M.getABRadauModified(neq+2,mp_ab);
     def=M.getOrthogonalPolynomialsFromAB(1,ab)[neq];
    }
    UnitTests.checkDoubleRel(q.dp_le,"q.dp_le",q.dpnodes[0],"q.dpnodes[0]",eps);
    UnitTests.checkDoubleRel(q.dp_ld,"q.dp_ld",TMatr.mulVVn1(def,mom_dp)/Polynomials.getD0(def,x0),"dp_ld",eps);
    final double [] lef=M.getOrthogonalPolynomialsFirstKind(neq+2,mom_p)[neq+1];
    UnitTests.checkDoubleRel(q.dp_nle,"q.dp_nle",TMatr.mulVVn1(lef,mom_dp)/Polynomials.getD0(lef,x0),"dp_nle",eps);
    final double [] ldf=M.getSecondKindOrthogonalPolynomialsFromFirstKind(M.getOrthogonalPolynomialsFirstKind(neq+2,mom_p))[neq];
    UnitTests.checkDoubleRel(q.dp_nld,"q.dp_nld",TMatr.mulVVn1(ldf,mom_dp)/Polynomials.getD0(ldf,x0),"dp_nld",eps);

    // check xr_ldeficit & wr_ldeficit
    UnitTests.checkDoubleRel(q.xr_ldeficit[0],"q.xr_ldeficit[0]",x0,"x0",eps);
    UnitTests.checkDoubleRel(TMatr.sumElements(q.wr_ldeficit),"sum(q.wr_ldeficit)",1,"1",eps);
    final Quadrature.GaussQuadratureWithNodesPolynomialData nqq=M.getRadauQuadratureFromPolynomials(neq,M.getSecondKindOrthogonalPolynomialsFromFirstKind(M.getOrthogonalPolynomialsFirstKind(neq+3,mom_p)));
    final double [][] nq=LQVM.scaleWInd1(new double [][]{nqq.x,nqq.w,nqq.nodespolynomial});
    q.sortNodes(nq[0],nq[1]);
    final double [] xt=nq[0],wt=nq[1];
    UnitTests.checkDoubleArrayRel(q.xr_ldeficit,"q.xr_ldeficit",xt,"xt",eps);
    UnitTests.checkDoubleArrayRel(q.wr_ldeficit,"q.wr_ldeficit",wt,"wt",eps);
    UnitTests.checkDoubleArrayRel(q.xr_ldeficit,"q.xr_ldeficit",q.sxnodes,"q.sxnodes",eps);
    UnitTests.checkDoubleArrayRel(q.wr_ldeficit,"q.wr_ldeficit",q.swnodes,"q.swnodes",eps);

    System.err.println(name+":KQVMTest:test_DFMod(): test OK neq="+neq+" eps="+eps);
}

void test_DF(final KQVM<DTYPE> q,final double [] mom_p,final double [] mom_dp,final double x0)
{
    test_DFMod(q,mom_p,mom_dp,x0);

    final double [][] rr=M.getRadauPolynomials(M.getOrthogonalPolynomialsFirstKind(neq+2,mom_p));
    final double [][] qr=M.getSecondKindOrthogonalPolynomialsFromFirstKind(rr);
    final double [] nodes=Polynomials.mult2Pol(rr[neq],qr[neq-1]);
    final double df=TMatr.mulVVn1(nodes,mom_dp)/Polynomials.getD0(nodes,x0);
    UnitTests.checkDoubleRel(df,"df",q.df,"q.df",eps);

    // average of nodes polynomials should be =0
    final double [] n_q1=M.getRadauMultiplied(rr[neq]),n_q1q2=M.getRadauMultiplied(nodes);
    UnitTests.checkDoubleRel(TMatr.mulVVn1(n_q1,mom_p)/mom_p[0],"<radau q1>",0,"0",eps);
    UnitTests.checkDoubleRel(TMatr.mulVVn1(n_q1q2,mom_p)/mom_p[0],"<radau q1 q2>",0,"0",eps);

    // match polynomial roots
    final RecurrenceAB ab_r=M.getAB(rr),ab_q=M.getAB(qr);
    final double [] xr=M.getXNodesFromAB(rr[neq].length-1,ab_r),xq=M.getXNodesFromAB(qr[neq-1].length-1,ab_q);
    q.sortNodes(xr,null);
    q.sortNodes(xq,null);
    UnitTests.checkDoubleArrayRel(xr,"xr",q.x_excess,"q.x_excess",eps);
    UnitTests.checkDoubleArrayRel(xq,"xq",q.x_deficit,"q.x_deficit",eps);
    for(int k=0;k<q.x_deficit.length;k++)
    { // there should be exactly one root of type 2 polynomial between two roots of type 1 polynomial
	double xd=q.x_deficit[k],xL=q.x_excess[k],xR=q.x_excess[k+1];
	if(xL>xR){final double tmp=xL;xL=xR;xR=tmp;}
	if(!(xL<xd && xd<xR)) throw new RuntimeException("xL="+xL+" xR="+xR+" xd="+xd);
    }
    System.err.println(name+":KQVMTest:test_DF(): test OK neq="+neq+" eps="+eps);
}


void test_MonomialsRadau(final double [] mom_p)
{
    final KQVM<DTYPE> q=getKQVM();
    final Quadrature.GaussQuadratureWithNodesPolynomialData npq=M.getRadauQuadratureFromPolynomials(neq,M.getOrthogonalPolynomialsFirstKind(neq+2,mom_p));
    final double [][] np=LQVM.scaleWInd1(new double [][]{npq.x,npq.w,npq.nodespolynomial});
    q.sortNodes(np[0],np[1]);
    final double [] xt=np[0],wt=np[1];
    UnitTests.checkDoubleArrayRel1(q.xnodes,"q.xnodes",xt,"xt",eps);
    UnitTests.checkDoubleArrayRel(q.wnodes,"q.wnodes",wt,"wt",eps);
    UnitTests.checkDoubleArrayRel1(q.wnodes,"q.wnodes",TMatr.getVectorScaled(M.getWeightsByIntegration(np[2],xt,mom_p),1/mom_p[0]),"wnodes",eps);

    final Quadrature.GaussQuadratureWithNodesPolynomialData nsq=M.getRadauQuadratureFromPolynomials(neq,M.getSecondKindOrthogonalPolynomialsFromFirstKind(M.getOrthogonalPolynomialsFirstKind(neq+3,mom_p)));
    final double [][] ns=LQVM.scaleWInd1(new double [][]{nsq.x,nsq.w,nsq.nodespolynomial});
    q.sortNodes(ns[0],ns[1]);
    UnitTests.checkDoubleArrayRel(q.sxnodes,"q.sxnodes",ns[0],"ns[0]",eps);
    UnitTests.checkDoubleArrayRel(q.swnodes,"q.swnodes",ns[1],"ns[1]",eps);

    System.err.println(name+":KQVMTest:test_MonomialsRadau(): test OK neq="+neq+" eps="+eps);
}
void test_Monomials(final double [] mom_p)
{
    final KQVM<DTYPE> q=getKQVM();
    final Quadrature.GaussQuadratureWithNodesPolynomialData npq=M.getGaussQuadratureFromPolynomials(neq,M.getOrthogonalPolynomialsFirstKind(neq+1,mom_p));
    final double [][] np=LQVM.scaleWInd1(new double [][]{npq.x,npq.w,npq.nodespolynomial});
    q.sortNodes(np[0],np[1]);
    final double [] xt=np[0],wt=np[1];
    UnitTests.checkDoubleArrayRel(q.axnodes,"q.axnodes",xt,"xt",eps);
    UnitTests.checkDoubleArrayRel(q.awnodes,"q.awnodes",wt,"wt",eps);
    UnitTests.checkDoubleArrayRel1(q.awnodes,"q.awnodes",TMatr.getVectorScaled(M.getWeightsByIntegration(np[2],xt,mom_p),1/mom_p[0]),"awnodes",eps);

    final Quadrature.GaussQuadratureWithNodesPolynomialData nsq=M.getGaussQuadratureFromPolynomials(neq,M.getSecondKindOrthogonalPolynomialsFromFirstKind(M.getOrthogonalPolynomialsFirstKind(neq+2,mom_p)));
    final double [][] ns=LQVM.scaleWInd1(new double [][]{nsq.x,nsq.w,nsq.nodespolynomial});
    q.sortNodes(ns[0],ns[1]);
    UnitTests.checkDoubleArrayRel(q.saxnodes,"q.saxnodes",ns[0],"ns[0]",eps);
    UnitTests.checkDoubleArrayRel(q.sawnodes,"q.sawnodes",ns[1],"ns[1]",eps);

    System.err.println(name+":KQVMTest:test_Monomials(): test OK neq="+neq+" eps="+eps);
}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":KQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Moments();
    test_RQVM();
    test_SQVM();

    System.err.println(name+":KQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":KQVMTest: starting tests; ver=$Id: KQVM.java,v 1.121 2022/07/28 10:18:50 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<118;i++)
	{
	    test_Calculations(7310+r.nextInt(7310),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":KQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: KQVM.java,v 1.121 2022/07/28 10:18:50 mal Exp $");
}
} // END OF class KQVMTest



}// END OF class KQVM
