package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.Quadrature;
import com.polytechnik.realtimebook.*;

/** QVM calculations.
 *  @version $Id: KQVMLaguerre.java,v 1.45 2022/06/26 17:54:35 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class KQVMLaguerre extends KQVM<QVMDataL> {
    static final Laguerre B=new Laguerre();
KQVMLaguerre(final int neq,final QVMDataL odata)
{
    super(neq,odata);
}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
double getPOffset(){return odata.p_offset;}
OrthogonalPolynomialsLaguerreBasis getBasis(){return new OrthogonalPolynomialsLaguerreBasis();}
public static KQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final KQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq+5,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new KQVMLaguerre(neq,o);
}

void sortNodes(final double [] x,final double [] w)
{
    com.polytechnik.utils.SortArrays.sortArrayWithLoad(x.length,x,w);
}
double [] getMult2Pol(final double [] x,final double [] y)
{
    return B.mult2Pol(x,y);
}


public static void main(String [] args)
{
    new KQVMTest<QVMDataL>("KQVMLaguerre",
     new com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis()
     {
	 @Override
	 public double [] getRadauMultiplied(final double [] nodes0)
	 {
	     return Polynomials.getxP(nodes0);
	 }
     },
     new java.util.Random(68720L),1e-5){
	void setData(final int tsize,final int neq)
	{
	    this.neq=neq;
	    this.tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
	    this.currenttime=tb[tb.length-1].time+1000;
	    this.tau=(tb[tb.length-1].time-tb[0].time)/24;
	}
	KQVMLaguerre getKQVM()
	{
	    return calculate(neq,currenttime,tau,tb,tb.length,null);
	}

	QVMData getPData(final KQVMLaguerre q)
	{
	    //System.err.println("neq="+neq+" q.odata.ntm="+q.odata.ntm);
	    final QVMData qdata=QVMData.calculateQVMData(1,q.odata.ntm,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
	    return qdata;
	}


void test_RQVM()
{
    final KQVMLaguerre q=getKQVM();
    final RQVMLaguerre s=RQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);
    UnitTests.checkDoubleArrayRel(q.xnodes,"q.xnodes",s.xnodes,"s.xnodes",eps);
    UnitTests.checkDoubleArrayRel(q.wnodes,"q.wnodes",s.wnodes,"s.wnodes",eps);
    UnitTests.checkDoubleArrayRel(q.dpnodes,"q.dpnodes",s.dpnodes,"s.dpnodes",eps);
    UnitTests.checkDoubleArrayRel(q.dipnodes,"q.dipnodes",s.dipnodes,"s.dipnodes",eps);

    if(true)
    {
	final double [] mom_i=MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_VSELL,q.odata.p_offset),
	    mom_d=MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_VBUY,q.odata.p_offset);
	final double v0=mom_i[0]+mom_d[0];
	for(int k=mom_i.length;--k>=0;) {final double i=mom_i[k],d=mom_d[k];mom_i[k]=(i+d);mom_d[k]=(i-d)/v0;}
	final Quadrature.GaussQuadratureWithNodesPolynomialData qq=M.getRadauQuadratureFromPolynomials(neq,M.getSecondKindOrthogonalPolynomialsFromFirstKind(M.getOrthogonalPolynomialsFirstKind(neq+3,mom_i)));
	final double [][] sxw=LQVM.scaleWInd1(new double [][]{qq.x,qq.w,qq.nodespolynomial});
	q.sortNodes(sxw[0],sxw[1]);
	UnitTests.checkDoubleArrayRel1(q.sxnodes,"q.sxnodes",sxw[0],"sxw[0]",eps);
	UnitTests.checkDoubleArrayRel1(q.swnodes,"q.swnodes",sxw[1],"sxw[1]",eps);
	UnitTests.checkDoubleArrayRel1(q.sdipnodes,"q.sdipnodes",M.getWeightsByIntegration(sxw[2],sxw[0],mom_d),"sdip",eps);
    }

    System.err.println(name+":KQVMLaguerreTest:test_RQVM(): Test OK, neq="+neq+" eps="+eps);
}
void test_SQVM()
{
    final KQVMLaguerre q=getKQVM();
    final SQVMLaguerre s=SQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);
    UnitTests.checkDoubleArrayRel(q.axnodes,"q.axnodes",s.xnodes,"s.xnodes",eps);
    UnitTests.checkDoubleArrayRel(q.awnodes,"q.awnodes",s.wnodes,"s.wnodes",eps);
    UnitTests.checkDoubleArrayRel(q.adpnodes,"q.adpnodes",s.dpnodes,"s.dpnodes",eps);
    UnitTests.checkDoubleArrayRel(q.adipnodes,"q.adipnodes",s.dipnodes,"s.dipnodes",eps);
    UnitTests.checkDoubleArrayRel(q.apnodes,"q.apnodes",s.pnodes,"s.pnodes",eps);

    if(true)
    {
	final double [] mom_i=MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_VSELL,q.odata.p_offset),
	    mom_d=MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_VBUY,q.odata.p_offset);
	final double v0=mom_i[0]+mom_d[0];
	for(int k=mom_i.length;--k>=0;) {final double i=mom_i[k],d=mom_d[k];mom_i[k]=(i+d);mom_d[k]=(i-d)/v0;}
	final Quadrature.GaussQuadratureWithNodesPolynomialData qq=M.getGaussQuadratureFromPolynomials(neq,M.getSecondKindOrthogonalPolynomialsFromFirstKind(M.getOrthogonalPolynomialsFirstKind(neq+2,mom_i)));
	final double [][] sxw=LQVM.scaleWInd1(new double [][]{qq.x,qq.w,qq.nodespolynomial});
	q.sortNodes(sxw[0],sxw[1]);
	UnitTests.checkDoubleArrayRel(q.saxnodes,"q.saxnodes",sxw[0],"sxw[0]",eps);
	UnitTests.checkDoubleArrayRel(q.sawnodes,"q.sawnodes",sxw[1],"sxw[1]",eps);
	UnitTests.checkDoubleArrayRel1(q.sadipnodes,"q.sadipnodes",M.getWeightsByIntegration(sxw[2],sxw[0],mom_d),"sdip",eps);
    }

    System.err.println(name+":KQVMLaguerreTest:test_SQVM(): Test OK, neq="+neq+" eps="+eps);
}

double [] convertMonomialsToBasis(final double [] m)
{
    return B.convertFunctionTo_Basis_from_Monomials(m);
}
void test_Moments()
{
    final  KQVMLaguerre q=getKQVM();
    test_MonomialsRadau(MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_dV,q.odata.p_offset));
    test_Monomials(MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_dV,q.odata.p_offset));
    final double [] mom_db=new double [2*neq+2];
    if(true)
    {
	q.access.setM(mom_db,1,q.access.M_DPPLUS,q.odata,1);
	final double dpp0=mom_db[0];
	q.access.setM(mom_db,1,q.access.M_DPMINUS,q.odata,1);
	final double d0=dpp0-mom_db[0];
	q.access.setM(mom_db,1,q.access.M_DP,q.odata,mom_db.length);
	for(int k=mom_db.length;--k>=0;) mom_db[k]/=d0;
    }
    test_DF(q,MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_dV,q.odata.p_offset),convertBasisMomentsToMonomialsMoments(mom_db),0);
}
}.runTests(7);
} // END main


} // END OF class KQVMLaguerre
