package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.Quadrature;
import com.polytechnik.realtimebook.*;

/** QVM calculations.
 *  @version $Id: KQVMLegendreShifted.java,v 1.68 2022/06/26 17:54:35 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class KQVMLegendreShifted extends KQVM<QVMDataP> {
    static final LegendreShifted B=new LegendreShifted();
KQVMLegendreShifted(final int neq,final QVMDataP odata)
{
    super(neq,odata);
}
QVMDataPDirectAccess getAccess(){return new QVMDataPDirectAccess();}
double getPOffset(){return odata.p_offset;}
OrthogonalPolynomialsLegendreShiftedBasis getBasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
public static KQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final KQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq+5,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new KQVMLegendreShifted(neq,o);
}

void sortNodes(final double [] x,final double [] w)
{
    com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(x.length,x,w);
}
double [] getMult2Pol(final double [] x,final double [] y)
{
    return B.mult2Pol(x,y);
}

public static void main(String [] args)
{
    new KQVMTest<QVMDataP>("KQVMLegendreShifted",
      new com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis()
      {
	  @Override
	  public double [] getRadauMultiplied(final double [] nodes0)
	  {
	      return Polynomials.get1mxP(nodes0);
	  }
      },
      new java.util.Random(1863L),1e-5){
	void setData(final int tsize,final int neq)
	{
	    this.neq=neq;
	    this.tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
	    this.currenttime=tb[tb.length-1].time+1000;
	    this.tau=((tb[tb.length-1].time-tb[0].time)/2)/(9*5*7*8*11*13)*(9*5*7*8*11*13);
	    //System.err.println("tau="+tau);
	}
	KQVMLegendreShifted getKQVM()
	{
	    return calculate(neq,currenttime,tau,tb,tb.length,null);
	}

	QVMData [] getPData(final KQVMLegendreShifted q)
	{
	    final QVMData [] qdata=new QVMData [q.odata.ntm];
	    for(int k=0;k<q.odata.ntm;k++)
	    {
		final long t=tau/(1+k);
		if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
		qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
	    }
	    return qdata;
	}


void test_RQVM()
{
    final KQVMLegendreShifted q=getKQVM();
    final RQVMLegendreShifted s=RQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);
    UnitTests.checkDoubleArrayRel(q.xnodes,"q.xnodes",s.xnodes,"s.xnodes",eps);
    UnitTests.checkDoubleArrayRel(q.wnodes,"q.wnodes",s.wnodes,"s.wnodes",eps);
    UnitTests.checkDoubleArrayRel1(q.dpnodes,"q.dpnodes",s.dpnodes,"s.dpnodes",eps);
    UnitTests.checkDoubleArrayRel(q.dipnodes,"q.dipnodes",s.dipnodes,"s.dipnodes",eps);

    if(true)
    {
	final double [] mom_i=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_VSELL,false,q.odata.p_offset),
	    mom_d=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_VBUY,false,q.odata.p_offset);
	final double v0=mom_i[0]+mom_d[0];
	for(int k=mom_i.length;--k>=0;) {final double i=mom_i[k],d=mom_d[k];mom_i[k]=(i+d);mom_d[k]=(i-d)/v0;}
	final Quadrature.GaussQuadratureWithNodesPolynomialData qq=M.getRadauQuadratureFromPolynomials(neq,M.getSecondKindOrthogonalPolynomialsFromFirstKind(M.getOrthogonalPolynomialsFirstKind(neq+3,mom_i)));
	final double [][] sxw=LQVM.scaleWInd1(new double [][]{qq.x,qq.w,qq.nodespolynomial});
	q.sortNodes(sxw[0],sxw[1]);
	UnitTests.checkDoubleArrayRel(q.sxnodes,"q.sxnodes",sxw[0],"sxw[0]",eps);
	UnitTests.checkDoubleArrayRel(q.swnodes,"q.swnodes",sxw[1],"sxw[1]",eps);
	UnitTests.checkDoubleArrayRel1(q.sdipnodes,"q.sdipnodes",M.getWeightsByIntegration(sxw[2],sxw[0],mom_d),"sdip",eps);
    }

    System.err.println(name+":KQVMLegendreShiftedTest:test_RQVM(): Test OK, neq="+neq+" eps="+eps);
}
void test_SQVM()
{
    final KQVMLegendreShifted q=getKQVM();
    final SQVMLegendreShifted s=SQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);
    UnitTests.checkDoubleArrayRel(q.axnodes,"q.axnodes",s.xnodes,"s.xnodes",eps);
    UnitTests.checkDoubleArrayRel(q.awnodes,"q.awnodes",s.wnodes,"s.wnodes",eps);
    UnitTests.checkDoubleArrayRel(q.adpnodes,"q.adpnodes",s.dpnodes,"s.dpnodes",eps);
    UnitTests.checkDoubleArrayRel(q.adipnodes,"q.adipnodes",s.dipnodes,"s.dipnodes",eps);
    UnitTests.checkDoubleArrayRel(q.apnodes,"q.apnodes",s.pnodes,"s.pnodes",eps);

    if(true)
    {
	final double [] mom_i=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_VSELL,false,q.odata.p_offset),
	    mom_d=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_VBUY,false,q.odata.p_offset);
	final double v0=mom_i[0]+mom_d[0];
	for(int k=mom_i.length;--k>=0;) {final double i=mom_i[k],d=mom_d[k];mom_i[k]=(i+d);mom_d[k]=(i-d)/v0;}
	final Quadrature.GaussQuadratureWithNodesPolynomialData qq=M.getGaussQuadratureFromPolynomials(neq,M.getSecondKindOrthogonalPolynomialsFromFirstKind(M.getOrthogonalPolynomialsFirstKind(neq+2,mom_i)));
	final double [][] sxw=LQVM.scaleWInd1(new double [][]{qq.x,qq.w,qq.nodespolynomial});
	q.sortNodes(sxw[0],sxw[1]);
	UnitTests.checkDoubleArrayRel(q.saxnodes,"q.saxnodes",sxw[0],"sxw[0]",eps);
	UnitTests.checkDoubleArrayRel(q.sawnodes,"q.sawnodes",sxw[1],"sxw[1]",eps);
	UnitTests.checkDoubleArrayRel1(q.sadipnodes,"q.sadipnodes",M.getWeightsByIntegration(sxw[2],sxw[0],mom_d),"sadip",eps);
    }

    System.err.println(name+":KQVMLegendreShiftedTest:test_SQVM(): Test OK, neq="+neq+" eps="+eps);
}

double [] convertMonomialsToBasis(final double [] m)
{
    return B.convertFunctionTo_Basis_from_Monomials(m);
}
void test_Moments()
{
    final  KQVMLegendreShifted q=getKQVM();
    test_MonomialsRadau(MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_dV,false,q.odata.p_offset));
    test_Monomials(MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_dV,false,q.odata.p_offset));
    final double [] mom_db=new double [2*neq+2];
    System.err.println("neq="+neq);
    if(true)
    {
	q.access.setM(mom_db,1,q.access.M_DPPLUS,q.odata,1);
	final double dpp0=mom_db[0];
	q.access.setM(mom_db,1,q.access.M_DPMINUS,q.odata,1);
	final double d0=dpp0-mom_db[0];
	q.access.setM(mom_db,1,q.access.M_DP,q.odata,mom_db.length);
	for(int k=mom_db.length;--k>=0;) mom_db[k]/=d0;
    }
    test_DF(q,MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(q.odata.ntm,getPData(q),0,QVMData.MOM_dV,false,q.odata.p_offset),convertBasisMomentsToMonomialsMoments(mom_db),1);
}
}.runTests(5);
} // END main


} // END OF class KQVMLegendreShifted
