package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.realtimebook.*;

/** John Pierpont Morgan and finite variations.
 *  @version $Id: KkQVM.java,v 1.96 2022/06/30 08:18:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class KkQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;

final EVXData I,P1t,P1i,pVt,pVV;
final double Imax,P_Imax,Imax1,P_Imax1,Imin,P_Imin,Imin1,P_Imin1;
final double I_P1t_L,I_P1t_H,w_PH_gt_PL,w_PH_lt_PL;

final double [] Im;
final double w_PL_IL,w_PH_IL,w_PL_IH,w_PH_IH,Paver_I;

final double w_PH_gt_IL,w_PH_lt_IL,w_PL_gt_IL,w_PL_lt_IL,
    w_PH_gt_IH,w_PH_lt_IH,w_PL_gt_IH,w_PL_lt_IH;
final double w_IH_gt_IL,w_IH_lt_IL;
final double w_PH_IH_PL,w_PL_IH_PH;
final double w_PH_PL_IL,w_PL_PH_IL,w_IL_PH_PL,w_IL_PL_PH;

final double w_IHPL_ILPH,w_IHPH_ILPL;
final double w_PL_PH_PM,w_PH_PL_PM;

final double w_IHPL_IL,w_IL_IHPL,w_IHPH_IL,w_IL_IHPH;
final double w_PL_I_PH;
final double pi_PL,pi_PH;

final double w_pVtH_gt_pVtL,w_pVtH_lt_pVtL;
final double pVt_PL,pVt_PH;
final double sens_PL_pertI,sens_PH_pertI,sens_IL_pertPt,sens_IH_pertPt,sens_IL_pertPnL,sens_IH_pertPnL;
final RVarIpsi rpL,rpH;
final double wrpL_J0_gt_J1,wrpL_J0_lt_J1,wrpH_J0_gt_J1,wrpH_J0_lt_J1;
final PnLe pnl;
final WDistance distanceP1t,distanceI;

KkQVM(final int n,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final double [][] QQmoms=getMoments(n);
    {
	final double [] tmom=QQmoms[0],imom=QQmoms[1],pimom=QQmoms[5];
	pnl=new PnLe(n,imom,tmom,pimom);
    }
    final double [][] QQ=access.getQQ_Matrs(n,1,QQmoms);
    final double [] QQt=QQ[0],QQi=QQ[1],QQj=QQ[2],QQpt=QQ[3],QQp2t=QQ[4],QQpi=QQ[5],QQpV=QQ[6],QQV=QQ[7],QQT=QQ[8],QQpT=QQ[9];

    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,getX0());
    this.P1t=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpt,QQj,access,getX0());
    this.P1i=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpi,QQi,access,getX0());
    this.w_PL_I_PH=TMatr.getAMatrB(P1t.psiL,P1t.psiH,QQi,n);
    this.pVt=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpV,QQt,access,getX0());
    pVt_PL=pVt.sL/TMatr.getAMatrA(pVt.psiL,QQV,n);
    pVt_PH=pVt.sH/TMatr.getAMatrA(pVt.psiH,QQV,n);
    this.pVV=DeprecatedHelperEVXData.getEVXDataFromQQ(n,TMatr.getVectorScaled(QQpV,-1),TMatr.getVectorScaled(QQV,-1),access,getX0());

    this.Im=new double [n];
    double  pai=0,ai=0;
    for(int k=0;k<n;k++)
    {
	final double tk=TMatr.getAMatrB(n,P1t.ev,n+k*n,n,P1t.ev,n+k*n,QQt,n);
	this.Im[k]=TMatr.getAMatrB(n,P1t.ev,n+k*n,n,P1t.ev,n+k*n,QQi,n)/tk;
	pai+=this.Im[k]*P1t.ev[k];
	ai+=this.Im[k];
    }
    this.Paver_I=pai/ai;
    final int [] indIm=com.polytechnik.utils.SortArrays.sortArrayIndexes(Im);
    if(n>=1){final int ind=indIm[indIm.length-1];Imax=Im[ind];P_Imax=P1t.ev[ind];}
    else {Imax=P_Imax=Double.NaN;}
    if(n>=2){final int ind=indIm[indIm.length-2];Imax1=Im[ind];P_Imax1=P1t.ev[ind];}
    else{Imax1=P_Imax1=Double.NaN;}
    if(n>=1){final int ind=indIm[0];Imin=Im[ind];P_Imin=P1t.ev[ind];}
    else {Imin=P_Imin=Double.NaN;}
    if(n>=2){final int ind=indIm[1];Imin1=Im[ind];P_Imin1=P1t.ev[ind];}
    else{Imin1=P_Imin1=Double.NaN;}
    this.I_P1t_L=Im[0];
    this.I_P1t_H=Im[Im.length-1];
    pi_PL=TMatr.getAMatrA(P1t.psiL,QQpi,n)/TMatr.getAMatrA(P1t.psiL,QQi,n);
    pi_PH=TMatr.getAMatrA(P1t.psiH,QQpi,n)/TMatr.getAMatrA(P1t.psiH,QQi,n);

    this.w_PL_IL=TMatr.getAMatrB(P1t.psiL,I.psiL,QQt,n);
    this.w_PH_IL=TMatr.getAMatrB(P1t.psiH,I.psiL,QQt,n);
    this.w_PL_IH=TMatr.getAMatrB(P1t.psiL,I.psiH,QQt,n);
    this.w_PH_IH=TMatr.getAMatrB(P1t.psiH,I.psiH,QQt,n);

    double [] rr=getW_GT_LT(P1t.psiH,P1t.psiL);
    this.w_PH_gt_PL=rr[0];
    this.w_PH_lt_PL=rr[1];

    rr=getW_GT_LT(pVt.psiH,pVt.psiL);
    w_pVtH_gt_pVtL=rr[0];
    w_pVtH_lt_pVtL=rr[1];

    rr=getW_GT_LT(I.psiH,I.psiL);
    w_IH_gt_IL=rr[0];
    w_IH_lt_IL=rr[1];


    rr=getW_GT_LT(P1t.psiH,I.psiL);
    w_PH_gt_IL=rr[0];
    w_PH_lt_IL=rr[1];

    rr=getW_GT_LT(P1t.psiL,I.psiL);
    w_PL_gt_IL=rr[0];
    w_PL_lt_IL=rr[1];

    rr=getW_GT_LT(P1t.psiH,I.psiH);
    w_PH_gt_IH=rr[0];
    w_PH_lt_IH=rr[1];

    rr=getW_GT_LT(P1t.psiL,I.psiH);
    w_PL_gt_IH=rr[0];
    w_PL_lt_IH=rr[1];

    this.w_PH_IH_PL=getK3(P1t.psiH,I.psiH,P1t.psiL);
    this.w_PL_IH_PH=getK3(P1t.psiL,I.psiH,P1t.psiH);

    w_PH_PL_IL=getK3(P1t.psiH,P1t.psiL,I.psiL);
    w_PL_PH_IL=getK3(P1t.psiL,P1t.psiH,I.psiL);
    w_IL_PH_PL=getK3(I.psiL,P1t.psiH,P1t.psiL);
    w_IL_PL_PH=getK3(I.psiL,P1t.psiL,P1t.psiH);


    w_IHPL_ILPH=getK2(I.psiL,P1t.psiH,I.psiH,P1t.psiL);
    w_IHPH_ILPL=getK2(I.psiL,P1t.psiL,I.psiH,P1t.psiH);


    w_IHPL_IL=getK2(I.psiL,I.psiL,I.psiH,P1t.psiL);
    w_IL_IHPL=getK2(I.psiH,P1t.psiL,I.psiL,I.psiL);
    w_IHPH_IL=getK2(I.psiL,I.psiL,I.psiH,P1t.psiH);
    w_IL_IHPH=getK2(I.psiH,P1t.psiH,I.psiL,I.psiL);

    final int indm=(n-1)/2;
    final double [] psiM=new double [n];
    System.arraycopy(P1t.ev,n+indm*n,psiM,0,n);
    w_PL_PH_PM=getK3(psiM,P1t.psiH,P1t.psiL);
    w_PH_PL_PM=getK3(psiM,P1t.psiL,P1t.psiH);


    sens_PL_pertI=getWPerturbatuion(0,P1t,QQi,n);

    sens_PH_pertI=getWPerturbatuion(n-1,P1t,QQi,n);

    sens_IL_pertPt=getWPerturbatuion(0,I,QQpt,n);

    sens_IH_pertPt=getWPerturbatuion(n-1,I,QQpt,n);

    sens_IL_pertPnL=getWPerturbatuion(0,I,sumV(QQpi,-I.sL,QQpt),n);

    sens_IH_pertPnL=getWPerturbatuion(n-1,I,sumV(QQpi,-I.sH,QQpt),n);

    this.rpL=new RVarIpsi(0,P1t,QQi,QQt,n);
    rr=getW_GT_LT(getPsiNormalized(rpL.psi0,QQt,n),getPsiNormalized(rpL.psi1,QQt,n));
    wrpL_J0_gt_J1=rr[0];
    wrpL_J0_lt_J1=rr[1];

    this.rpH=new RVarIpsi(n-1,P1t,QQi,QQt,n);
    rr=getW_GT_LT(getPsiNormalized(rpH.psi0,QQt,n),getPsiNormalized(rpH.psi1,QQt,n));
    wrpH_J0_gt_J1=rr[0];
    wrpH_J0_lt_J1=rr[1];

    distanceP1t=new WDistance(P1t,QQi,QQi,QQpi,n);
    distanceI=new WDistance(I,QQpt,QQi,QQpi,n);

}

static double [] getEVBasisNormalize(final EVXData J,final double [] QQt)
{
    final int n=J.neq;
    final double [] ev=new double [n*n];
    for(int k=0;k<n;k++)
    {
	final double nr=Math.sqrt(TMatr.getAMatrB(n,J.ev,n+k*n,J.ev,n+k*n,QQt,n));
	for(int l=0;l<n;l++)
	{
	    ev[k*n+l]=J.ev[n+k*n+l]/nr;
	}
    }
    return ev;
}

class WDistance extends Distance {
final double [] I_distancesorted,_I_distancesorted;
final double [] e_distancesorted,pi_distancesorted,sens_distancesorted;
WDistance(final EVXData J,final double [] QQpertr,final double [] QQi,final double [] QQpi,final int ldQQ)
{
    super(J.neq,J.ev,J.neq,J.neq);
    final int n=J.neq;
    this.I_distancesorted=getOrderedEMfromQQ(QQi,ldQQ);
    this.pi_distancesorted=getOrderedEMfromQQ(QQpi,ldQQ);
    this.e_distancesorted=new double [n];
    this.sens_distancesorted=new double [n];
    this._I_distancesorted=new double [n];
    final double [] psi=new double [n];
    for(int k=0;k<n;k++)
    {
	final int l=t_order[k];
	this.e_distancesorted[k]=J.ev[l];
	this.pi_distancesorted[l]/=this.I_distancesorted[l];
	this._I_distancesorted[k]=J.getFPsi(l,QQi);
	final double [] psipert=DeprecatedHelperEVXData.getPsiPertr(J,l,QQpertr);
	System.arraycopy(J.ev,n+n*l,psi,0,n);
	this.sens_distancesorted[k]=getK2(psipert,psi,psi,psi)-getK2(psi,psi,psipert,psi);
    }
}
double getK2Distance(final double [] a,final double [] b){return getK2(a,a,b,b);}
void test_Self(final double eps)
{
    UnitTests.checkDoubleArrayRel(I_distancesorted,"I_distancesorted",_I_distancesorted,"_I_distancesorted",eps);
    super.test_Self(eps);
}
}


double getWPerturbatuion(final int ind_lev,final EVXData j,final double [] QQpert,final int ldQQ)
{
    final double [] psi=new double [j.neq],psipert=DeprecatedHelperEVXData.getPsiPertr(j,ind_lev,QQpert);
    System.arraycopy(j.ev,j.neq+(ind_lev)*j.neq,psi,0,j.neq);
    return getK2(psipert,psi,psi,psi)-getK2(psi,psi,psipert,psi);
}




boolean isTMom(){return true;}

double [][] getMoments(final int n)
{
    final int nmoms=2*n-1;
    final double [] Pmom=new double [nmoms],P2mom=new double [nmoms],jmom=new double [nmoms],
	tmom=new double [nmoms],imom=new double [nmoms];

    final double pl_m_poffset=getPLast()-getPOffset();
    if(isTMom())
    {
	final double [] dp0mom=jmom,dp1mom=tmom,dp2mom=imom;
	access.setM(dp0mom,0,access.M_DP,odata,nmoms);
	access.setM(dp1mom,1,access.M_DP,odata,nmoms);
	access.setM(dp2mom,2,access.M_DP,odata,nmoms);
	Polynomials.shiftPOffsetInPlace(nmoms,dp0mom,dp1mom,dp2mom,null,pl_m_poffset);
	access.setFMoments(Pmom,dp1mom,nmoms);
	access.setFMoments(P2mom,dp2mom,nmoms);
	access.setMT(jmom,nmoms);
    }
    else
    {
	access.setM(jmom,0,access.M_I,odata,nmoms);
	access.setM(Pmom,1,access.M_I,odata,nmoms);
	access.setM(P2mom,2,access.M_I,odata,nmoms);
	Polynomials.shiftPOffsetInPlace(nmoms,jmom,Pmom,P2mom,null,pl_m_poffset);
    }

    final double [] pimom=new double [nmoms],pVmom=new double [nmoms],Vmom=new double [nmoms],Tmom=new double [nmoms],pTmom=new double [nmoms];
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    for(int k=0;k<nmoms;k++){pimom[k]+=(-pl_m_poffset)*imom[k];} // adjust to p_offset=p_last
    access.setFMoments(pVmom,pimom,nmoms);
    access.setFMoments(Vmom,imom,nmoms);
    access.setFMoments(Tmom,tmom,nmoms);
    access.setFMoments(pTmom,Pmom,nmoms);
    access.setMT(tmom,nmoms);
    return new double [][]{tmom,imom,jmom,Pmom,P2mom,pimom,pVmom,Vmom,Tmom,pTmom};
}

double getDS()
{
    if(this.Im.length<=0) return Double.NaN;
    return (this.Im[this.Im.length-1]-this.Im[0])/I.s0;
}

static double [] sumV(final double [] QQfg,final double f,final double [] QQg)
{
    final double [] QQ=new double [QQfg.length];
    for(int k=QQ.length;--k>=0;)
    {
	QQ[k]=QQfg[k]+f*QQg[k];
    }
    return QQ;
}
static double [] getPsiNormalized(final double [] psi,final double [] QQt,final int ldQQ)
{
    final double nrm=Math.sqrt(TMatr.getAMatrA(psi,QQt,ldQQ));
    final double [] npsi=new double [psi.length];
    for(int k=psi.length;--k>=0;){npsi[k]=psi[k]/nrm;}
    return npsi;
}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double [] getW_GT_LT(final double [] psiX,final double [] psiY);

/** Caclucates <psia^2(x) psib^2(y) psic^2(z)>_{x>y>z}
 */
abstract double getK3(final double [] psia,final double [] psib,final double [] psic);

/** Caclucates <psia(x) psib(x) psic(y) psid(y)>_{x>y}
 */
abstract double getK2(final double [] psia,final double [] psib,final double [] psic,final double [] psid);

abstract double [] getSTMoments(final int nmomsr,final double [] psiS1,final double [] psiS2);

double [] getSTMoments_FromgetK2(final int nmomsr,final double [] psiS1,final double [] psiS2)
{
    final double [] one=new double []{1};
    one[0]/=access.fx(one,0);
    final double [] moms=new double [nmomsr];
    for(int k=0;k<nmomsr;k++)
    {
	final double [] r=new double [k+1];r[k]=1;
	moms[k]=getK2(one,r,psiS1,psiS2);
    }
    return moms;
}

abstract double getX0();
abstract double getPLast();
abstract double getPOffset();


void test_DkQVMWpPsi(final DkQVM.WpPsi q,final double eps)
{
    EVXData.matchEVXData(P1t,q.Px,false,false,eps);
    final double [] tIm;
    if(isTMom()){
	tIm=q.Im;
	UnitTests.checkDoubleRel(pi_PL,"pi_PL",q.Pm[0],"q.Px.Pm[0]",eps);
	UnitTests.checkDoubleRel(pi_PH,"pi_PH",q.Pm[q.Px.neq-1],"q.Px.Pm[s.Px.neq-1]",eps);
    }
    else{tIm=new double [q.Im.length];for(int k=0;k<tIm.length;k++){tIm[k]=1/q.Im[k];}}
    UnitTests.checkDoubleArrayRel(Im,"Im",tIm,"tIm",eps);

    if(Imax1>Imax) throw new RuntimeException("Imax1="+Imax1+" Imax="+Imax);
    if(Imin1<Imin) throw new RuntimeException("Imin1="+Imin1+" Imin="+Imin);

    final int ind=DjQVM.getIndMax(Im);
    UnitTests.checkDoubleRel(Im[ind],"Im[ind]",Imax,"Imax",eps);
    UnitTests.checkDoubleRel(P1t.ev[ind],"P1t.ev[ind]",P_Imax,"P_Imax",eps);
    System.err.println("KkQVML:isTMom()="+isTMom()+" Imax1="+Imax1+" Imax="+Imax);

    System.err.println("KkQVML:test_DkQVMWpPsi: isTMom()="+isTMom()+" test OK neq="+P1t.neq+" eps="+eps);

}


class PnLe {
    final EVXData I,PnL;
    final double PpsiH,pL,pH,iL,iH;
    final double w_pnlL_gt_pnlH,w_pnlL_lt_pnlH,
	w_pnlL_gt_I,w_pnlL_lt_I,w_pnlH_gt_I,w_pnlH_lt_I;
    final double pnlIH,pnlIL;
    PnLe(final int n,final double [] imom,final double [] tmom,final double [] pimom)
    {
	final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{tmom,imom,pimom});
	final double [] QQt=QQ[0],QQi=QQ[1],QQpi=QQ[2];
	this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,getX0());
	final double [] psiExit=this.I.psiH;
	this.PpsiH=TMatr.getAMatrA(psiExit,QQpi,n)/this.I.sH;

	final double [] QQpnl=new double [QQi.length];
	for(int k=QQpnl.length;--k>=0;){QQpnl[k]=QQpi[k]-this.PpsiH*QQi[k];}
	this.PnL=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpnl,QQt,access,getX0());
	this.iL=TMatr.getAMatrA(this.PnL.psiL,QQi,n);
	this.iH=TMatr.getAMatrA(this.PnL.psiH,QQi,n);
	this.pL=this.PnL.sL/iL;
	this.pH=this.PnL.sH/iH;
	this.pnlIH=TMatr.getAMatrA(this.I.psiH,QQpnl,n);
	this.pnlIL=TMatr.getAMatrA(this.I.psiL,QQpnl,n);

	double [] rr=getW_GT_LT(PnL.psiH,PnL.psiL);
	this.w_pnlL_gt_pnlH=rr[0];
	this.w_pnlL_lt_pnlH=rr[1];

	rr=getW_GT_LT(PnL.psiL,psiExit);
	w_pnlL_gt_I=rr[0];
	w_pnlL_lt_I=rr[1];

	rr=getW_GT_LT(PnL.psiH,psiExit);
	w_pnlH_gt_I=rr[0];
	w_pnlH_lt_I=rr[1];
    }

    void test_DkQVM(final DkQVM<?> s,final double eps)
    {
	final int n=this.I.neq;
	EVXData.matchEVXData(this.I,s.IT,false,false,eps);
	UnitTests.checkDoubleRel(pnlIH,"pnlIH",0,"0",eps);
	UnitTests.checkDoubleRel(pnlIL/this.I.sL,"pnlIL/I.sL",s.epit_psiIL.Pf/s.epit_psiIL.Ipsi-s.epit_psiIH.Pf/s.epit_psiIH.Ipsi,"s.epit_psiIL.Pf/s.epit_psiIL.Ipsi-s.epit_psiIH.Pf/s.epit_psiIH.Ipsi",eps);

	System.err.println("PnLe:test_KkQVM: test OK n="+n+" eps="+eps);
    }
} // END OF class PnLe



/** For unit test.
 */
static abstract class KkQVMTest<DTYPE> extends QTest {

abstract KkQVM<DTYPE> getKkQVM();
abstract DkQVM<?> getDkQVM();
abstract IkQVM<?> getIkQVM();
abstract MkQVM<?> getMkQVM();

final int nmaxintnumerical;final double epsintnumerical;
KkQVMTest(final String name,
	  final java.util.Random r,final double eps,
	  final int nmaxintnumerical,final double epsintnumerical)
{
    super(name,r,eps);
    this.nmaxintnumerical=nmaxintnumerical;
    this.epsintnumerical=epsintnumerical;
}


void test_Common()
{
    final KkQVM<DTYPE> q=getKkQVM();
    UnitTests.checkDoubleRel(q.w_IHPL_IL+q.w_IL_IHPL,"q.w_IHPL_IL+q.w_IL_IHPL",q.w_PL_IH,"q.w_PL_IH",eps);
    UnitTests.checkDoubleRel(q.w_IHPH_IL+q.w_IL_IHPH,"q.w_IHPH_IL+q.w_IL_IHPH",q.w_PH_IH,"q.w_PH_IH",eps);
    q.distanceP1t.test_Self(eps);

    System.err.println(name+":KkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}

void test_DkQVM()
{
    final DkQVM<?> s=getDkQVM();
    final KkQVM<DTYPE> q=getKkQVM();

    q.test_DkQVMWpPsi(q.isTMom()?s.P1t:s.P1i,eps);
    if(!q.isTMom())
    {
	final double [] Imt=new double [s.P1i.Im.length];
	for(int k=0;k<Imt.length;k++){Imt[k]=1/s.P1i.Im[k];}
	UnitTests.checkDoubleArrayRel(q.Im,"q.Im",Imt,"Imt",eps);
    }
    else
    {
	EVXData.matchEVXData(q.pVt,s.pVt.Px,false,false,eps);
	EVXData.matchEVXData(q.pVV,s.pVV.Px,false,false,eps);

	/*
	double [] rr=q.getW_GT_LT(s.mL_PL.m.I.psiL,s.mL_PL.dpsi);
	UnitTests.checkDoubleRel(rr[0],"gt",q.w_PL_gt_vI,"q.w_PL_gt_vI",eps);
	UnitTests.checkDoubleRel(rr[1],"lt",q.w_PL_lt_vI,"q.w_PL_lt_vI",eps);
	rr=q.getW_GT_LT(s.mL_PL.m.I.psiH,s.mL_PH.dpsi);
	UnitTests.checkDoubleRel(rr[0],"gt",q.w_PH_gt_vI,"q.w_PH_gt_vI",eps);
	UnitTests.checkDoubleRel(rr[1],"lt",q.w_PH_lt_vI,"q.w_PH_lt_vI",eps);
	*/
    }

    q.pnl.test_DkQVM(s,eps);

    System.err.println(name+":KkQVMTest:test_DkQVM(): test OK neq="+neq+" eps="+eps);
}

void test_IkQVM()
{
    final KkQVM<DTYPE> q=getKkQVM();
    final IkQVM<?> s=getIkQVM();

    if(q.isTMom())
    {
	EVXData.matchEVXData(q.P1t,s.P1t,false,false,eps);
	UnitTests.checkDoubleRel(q.I_P1t_L,"q.I_P1t_L",s.I_P1t_L,"s.I_P1t_L",eps);
	UnitTests.checkDoubleRel(q.I_P1t_H,"q.I_P1t_H",s.I_P1t_H,"s.I_P1t_H",eps);
	UnitTests.checkDoubleRel(q.w_PH_gt_PL,"q.w_PH_gt_PL",s.gP.wC_y_gt_x,"s.gP.wC_y_gt_x",eps);
	UnitTests.checkDoubleRel(q.w_PH_lt_PL,"q.w_PH_lt_PL",s.gP.wC_y_lt_x,"s.gP.wC_y_lt_x",eps);
    }

    System.err.println(name+":KkQVMTest:test_IkQVM(): test OK neq="+neq+" eps="+eps);
}


void test_MkQVM()
{
    final KkQVM<DTYPE> q=getKkQVM();
    final MkQVM<?> s=getMkQVM();

    if(q.isTMom())
    {
	EVXData.matchEVXData(q.P1t,s.P1t,false,false,eps);
	UnitTests.checkDoubleArrayRel(q.distanceP1t.sens_distancesorted,"q.distanceP1t.sens_distancesorted",s.sens_P1t_distancesorted,"s.sens_P1t_distancesorted",eps);
	UnitTests.checkDoubleArrayRel(q.distanceP1t.I_distancesorted,"q.distanceP1t.I_distancesorted",s.distanceP1t.I_distancesorted,"s.distanceP1t.I_distancesorted",eps);
	UnitTests.checkDoubleArrayRel(q.distanceP1t.e_distancesorted,"q.distanceP1t.e_distancesorted",s.distanceP1t.p_distancesorted,"s.distanceP1t.p_distancesorted",eps);

    }

    System.err.println(name+":KkQVMTest:test_MkQVM(): test OK neq="+neq+" eps="+eps);
}

void test_STMoments()
{
    final int n=neq,nmoms=2*n-1,ntests=1000;
    final KkQVM<DTYPE> q=getKkQVM();
    for(int t=0;t<ntests;t++)
    {
	final double [] psik=new double [n-r.nextInt(n)],
	    psil=new double [n-r.nextInt(n)];
	for(int k=0;k<psik.length;k++) psik[k]=1-2*r.nextDouble();
	for(int k=0;k<psil.length;k++) psil[k]=1-2*r.nextDouble();
	UnitTests.checkDoubleArrayRel(q.getSTMoments(nmoms,psik,psil),"q.getSTMoments(nmoms,psik,psil)",q.getSTMoments_FromgetK2(nmoms,psik,psil),"q.getSTMoments_FromgetK2(nmoms,psik,psil)",eps);
    }

    System.err.println(name+":KkQVMTest:test_STMoments(): test OK neq="+neq+" ntests="+ntests+" eps="+eps);
}




abstract double omega(final double x);
abstract double getXR(final KkQVM<?> q);
abstract double getXL(final KkQVM<?> q);
double getSignCompensationForMoments(){return 1;}

class TestIntegral3 {
    static final int INTR_XGTYGTZ=501,INTR_ALL=502;

    final double [] psix,psiy,psiz;
    final DeprecatedHelperDataDirectAccessWithBasis<?> access;
    final double xL,xR;
    double abserr=1e-9,relerr=1e-9;
    final int nomax=50000;
    com.polytechnik.utils.QUANC8.DoubleFunction toIntegrate;

    TestIntegral3(final double [] psix,final double [] psiy,final double [] psiz,final KkQVM<?> q)
    {
	this.access=q.access;
	this.xL=getXL(q);
	this.xR=getXR(q);
	this.psix=psix;
	this.psiy=psiy;
	this.psiz=psiz;
	toIntegrate=integrateX;
    }
    TestIntegral3(final double [] psiy,final double [] psiz,final KkQVM<?> q)
    {
	this(new double []{1},psiy,psiz,q);
	psix[0]/=access.fx(psix,0);
	this.x=xR;
	toIntegrate=integrateY;
    }

    double x,y,z;
    int intr_type;
    final com.polytechnik.utils.QUANC8.DoubleFunction
	integrateX=new com.polytechnik.utils.QUANC8.DoubleFunction(){
		public double f(final double ax)
		{
		    x=ax;
		    final com.polytechnik.utils.QUANC8 i=
			com.polytechnik.utils.QUANC8.integrate(integrateY,
							       (xL),
							       (intr_type==INTR_XGTYGTZ?x:xR),
							       abserr,
							       relerr,
							       nomax);
		    if(!(i.flag==0)) throw new RuntimeException("i.flag="+i.flag);
		    return i.integral;
		}
	    },
	integrateY=new com.polytechnik.utils.QUANC8.DoubleFunction(){
		public double f(final double ay)
		{
		    y=ay;
		    final com.polytechnik.utils.QUANC8 i=
			com.polytechnik.utils.QUANC8.integrate(integrateZ,
							       (xL),
							       (intr_type==INTR_XGTYGTZ?y:xR),
							       abserr,
							       relerr,
							       nomax);
		    if(!(i.flag==0)) throw new RuntimeException("i.flag="+i.flag);
		    return i.integral;
		}
	    },
	integrateZ=new com.polytechnik.utils.QUANC8.DoubleFunction(){
		public double f(final double az)
		{
		    z=az;
		    final double vx=access.fx(psix,x),vy=access.fx(psiy,y),vz=access.fx(psiz,z);
		    double f=(vx*vx)*omega(x)*(vy*vy)*omega(y)*(vz*vz)*omega(z);
		    return f*getSignCompensationForMoments();
		    //return f;
		}
	    };
    double getIntegral(final int intr_type)
    {
	this.intr_type=intr_type;
	final com.polytechnik.utils.QUANC8 i=
	    com.polytechnik.utils.QUANC8.integrate(toIntegrate,
						   xL,xR,
						   abserr,
						   relerr,
						   nomax);
	if(!(i.flag==0)) throw new RuntimeException("i.flag="+i.flag);
	return i.integral;
    }
} // END OF class TestIntegral3


void test_IntegralNumerical(final double eps,final boolean test_k3)
{
    final KkQVM<DTYPE> q=getKkQVM();


    for(double [][] x:new double [][][]{
	    {q.P1t.psiL,q.I.psiH,q.P1t.psiH},
	    {q.P1t.psiH,q.I.psiH,q.P1t.psiL}}
	) {
	final double [] psix=x[0],psiy=x[1],psiz=x[2];
	if(test_k3)
	{
	    final TestIntegral3 ti=new TestIntegral3(psix,psiy,psiz,q);
	    UnitTests.checkDoubleRel(ti.getIntegral(TestIntegral3.INTR_ALL),"<ALL>",1,"1",eps);
	    UnitTests.checkDoubleRel(ti.getIntegral(TestIntegral3.INTR_XGTYGTZ),"<GT>",q.getK3(psix,psiy,psiz),"K3(psix,psiy,psiz)",eps);
	}
	UnitTests.checkDoubleRel(q.getK2(psix,psix,psiy,psiy),"k2",q.getW_GT_LT(psix,psiy)[0],"k2(gt)",eps);

	UnitTests.checkDoubleRel(q.getK2(psiy,psiy,psiz,psiz),"k2",new TestIntegral3(psiy,psiz,q).getIntegral(TestIntegral3.INTR_XGTYGTZ)*getSignCompensationForMoments(),"ti2.getIntegral_XGTYGTZ",eps);
	UnitTests.checkDoubleRel(1,"1",new TestIntegral3(psiy,psiz,q).getIntegral(TestIntegral3.INTR_ALL)*getSignCompensationForMoments(),"ti2.getIntegral_ALL",eps);
    }

    System.err.println(name+":KkQVMTest:test_IntegralNumerical(): test OK neq="+neq+" eps="+eps+(!test_k3?" K3 skipped":" K3 OK"));
}




void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":KkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_DkQVM();
    test_IkQVM();
    test_MkQVM();
    test_IntegralNumerical(epsintnumerical,neq<=nmaxintnumerical);
    test_STMoments();

    System.err.println(name+":KkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":KkQVMTest: starting tests; ver=$Id: KkQVM.java,v 1.96 2022/06/30 08:18:24 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":KkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: KkQVM.java,v 1.96 2022/06/30 08:18:24 mal Exp $");
}
} // END OF class KkQVMTest


}// END OF class KkQVM
