package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/*
 * @version $Id: KkQVMLaguerre.java,v 1.53 2022/07/28 12:27:17 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class KkQVMLaguerre extends KkQVM<QVMDataL> {

KkQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static KkQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final KkQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new KkQVMLaguerre(neq,o);
}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}

static double [] getPsiWIntegrated(final double [] polynomial)
{
    return WIntegratorLaguerre.B.getLaguerreExpFIntegrated(polynomial);
}

/** Calculates 1/exp(-t/2)\int_x^{inf} psi(x)exp(t/2) dt.
 * This function is an inverse to WIntegratorLaguerre.getEDPsi.
 * Possible unstable calculations used. Use _getEIntPsi_test for unit tests only.
 * For production - inverse matrix VolMatrix.getDMatrix.
 */
static double [] _getEIntPsi_test(final double [] psi)
{
    return TMatr.scaleVectorInPlace(getPxArgumentScaled(getPsiWIntegrated(getPxArgumentScaled(psi,2)),0.5),2);
}

static double [] getAB(final double [] a,final double [] b)
{
    return WIntegratorLaguerre.B.mult2Pol(a,b);
}
double [] getW_GT_LT(final double [] psiX,final double [] psiY)
{
    final double []
	psiX2=getAB(psiX,psiX),psiY2=getAB(psiY,psiY);
    final double [] p_x_gt_y=getPxArgumentScaled(getAB(psiX2,getPsiWIntegrated(psiY2)),1.0/2),p_x_lt_y=getPxArgumentScaled(getAB(psiY2,getPsiWIntegrated(psiX2)),1.0/2);

    final int nm=Math.max(p_x_gt_y.length,p_x_lt_y.length);
    final double [] tmp=new double [nm];
    access.setMT(tmp,nm);
    return new double []{(1.0/2)*TMatr.mulVVn1(p_x_gt_y,tmp),(1.0/2)*TMatr.mulVVn1(p_x_lt_y,tmp)};
}


static double [] getMomentsConvertedArg(final double [] factmoms,final double expfact)
{
    final double [] moms=new double [factmoms.length],
	tmp=new double [factmoms.length],tmp2=new double [factmoms.length];
    for(int k=0;k<moms.length;k++)
    {
	WIntegratorLaguerre.B.setNewtonBinomialLikeCoefs(k,tmp,tmp2,1/expfact,0);
	moms[k]=TMatr.mulVVn(k+1,tmp,factmoms)/expfact;
    }
    return moms;
}


static double [] getPxArgumentScaled(final double [] p,final double expfact)
{
    final double [] r=new double [p.length],
	tmp=new double [p.length],tmp2=new double [p.length];
    for(int k=0;k<p.length;k++)
    {
	WIntegratorLaguerre.B.setNewtonBinomialLikeCoefs(k,tmp,tmp2,expfact,0);
	DeprecatedHelperTMatr.sumP1(r,tmp,k+1,p[k]);
    }
    return r;
}


double getK3(final double [] psiX,final double [] psiY,final double [] psiZ)
{
    return _getK3(psiX,psiY,psiZ);
}

/** Calculates
 *  \int_{0}^{inf} dx psiX^2(x) exp(-x) \int_{x}^{inf} dy psiY^2(y) exp(-y) \int_{y}^{inf} dz psiZ^2(z) exp(-z)
 */
public static double _getK3(final double [] psiX,final double [] psiY,final double [] psiZ)
{
 final double []
     psiX2=getAB(psiX,psiX),psiY2=getAB(psiY,psiY),psiZ2=getAB(psiZ,psiZ);
 final double [] pp=getPxArgumentScaled(getAB(psiX2,getPxArgumentScaled(getPsiWIntegrated(getPxArgumentScaled(getAB(psiY2,getPsiWIntegrated(psiZ2)),1.0/2)),2.0)),1.0/3);
 /*access.setMT(tmp,pp.length);
 return (1.0/(2*3))*TMatr.mulVVn1(pp,tmp);
 */
 return pp[0]/6;// analytical answer is known for this basis
}
double getK2(final double [] psia,final double [] psib,final double [] psic,final double [] psid)
{
    return _getK2(psia,psib,psic,psid);
}
/** Calculates
 *  \int_{0}^{inf} dx psia(x)*psib(x) exp(-x) \int_{x}^{inf} dy psic(y) psid(y) exp(-y)
 */
public static double _getK2(final double [] psia,final double [] psib,final double [] psic,final double [] psid)
{
    final double []
	psiX2=getAB(psia,psib),psiY2=getAB(psic,psid);
    final double [] pp=getPxArgumentScaled(getAB(psiX2,getPsiWIntegrated(psiY2)),1.0/2);
    /*
    access.setMT(tmp,pp.length);
    return (1.0/2)*TMatr.mulVVn1(pp,tmp);
    */
    return pp[0]/2;// analytical answer is known for this basis
}
public static final K2Distance K2DISTANCE=new K2Distance(){public double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return _getK2(a,aa,b,bb);}};

double [] getSTMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{
    return _getSTMoments(nmomsr,psiS1,psiS2);
}

static double [] _getSTMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{

    final int nmoms=nmomsr+(psiS1.length-1+psiS2.length-1);
    final double [] intG=getPsiWIntegrated(getAB(psiS1,psiS2));
    final double [] tmom=new double [nmoms];
    tmom[0]=1; // analytical answer is known for this basis
    final double [] r=WIntegratorLaguerre.B.getPQkMomentsFromQkMoments(intG,getMomentsConvertedArg(tmom,2));
    if(r.length!=nmomsr) throw new RuntimeException();
    return r;
}
static double [] _getSMoments(final int nmomsr,final double [] momx,final double [] mom2x,final boolean flag_gt,final double [] psiS1,final double [] psiS2)
{
    final int nmoms=nmomsr+(psiS1.length-1+psiS2.length-1);
     final double [] intG=getPsiWIntegrated(getAB(psiS1,psiS2));

    final double [] xmom2=WIntegratorLaguerre.B.getPQkMomentsFromQkMoments(intG,getMomentsConvertedArg(mom2x,2));
    if(xmom2.length!=nmomsr) throw new RuntimeException("xmom2.length="+xmom2.length+" nmomsr="+nmomsr);
    if(flag_gt)
    {
	if(momx.length<xmom2.length) throw new RuntimeException();
	final double x0=0;
	final double intGx0=WIntegratorLaguerre.B.getD0(intG,x0); // must be equal to 1
	for(int k=nmomsr;--k>=0;)
	{
	    xmom2[k]=intGx0*momx[k]-xmom2[k];
	}
    }
    return xmom2;
}

double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}

static class KkQVMLaguerreTest extends KkQVMTest<QVMDataL> {
KkQVMLaguerreTest(final long initrandom)
{
    super("KkQVMLaguerre",new java.util.Random(initrandom),1e-5,
	  3,1e-2);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
KkQVMLaguerre getKkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DkQVMLaguerre getDkQVM(){return DkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
IkQVMLaguerre getIkQVM(){return IkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
MkQVMLaguerre getMkQVM(){return MkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
double getXL(final KkQVM<?> q){return 15+2*neq;}
double getXR(final KkQVM<?> q){return q.getX0();}
double omega(final double x){return Math.exp(-x);}
double getSignCompensationForMoments(){return -1;}

} // END of class KkQVMLaguerreTest

public static void main(String [] args)
{
if(false)
{
    for(long rr=1;rr<10000;rr++)
    {
	try{
	new  KkQVMLaguerreTest(rr).runTests(4);
	} catch(RuntimeException e) {
	    e.printStackTrace(System.err);
	    System.err.println("------------random="+rr+" failed");
	    continue;
	}
	System.err.println("rr="+rr+" OK");
	break;
    }
}
else
{
    new  KkQVMLaguerreTest(5L).runTests(4);
}
} // END main



} // END OF class KkQVMLaguerre
