package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/*
 * @version $Id: KkQVMLegendreShifted.java,v 1.51 2022/07/10 10:41:21 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class KkQVMLegendreShifted extends KkQVM<QVMDataP> {
KkQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static KkQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final KkQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new KkQVMLegendreShifted(neq,o);
}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}

/** Integration.
 */
static double [] getPsiWIntegrated(final double [] psi)
{
    return WIntegratorLegendreShifted.B.getIntegrated(psi);
}


/** Calculates 1/sqrt(x)\int_0^x psi(x)\sqrt(x)/x dx.
 * This function is an inverse to WIntegratorLegendreShifted.getEDPsi.
 * Possible unstable calculations used. Use _getEIntPsi_test for unit tests only.
 * For production - inverse matrix VolMatrix.getDMatrix.
 */
static double [] _getEIntPsi_test(final double [] psi)
{
    final double [] monomials=WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(psi);
    for(int k=0;k<monomials.length;k++) monomials[k]/=(k+0.5);
    return WIntegratorLegendreShifted.B.convertFunctionTo_Basis_from_Monomials(monomials);
}

static double [] getAB(final double [] a,final double [] b)
{
    return WIntegratorLegendreShifted.B.mult2Pol(a,b);
}

double [] getW_GT_LT(final double [] psiX,final double [] psiY)
{
    final double [] tmp=new double [Math.max(psiX.length,psiY.length)*4-4+1+1],
	psiX2=getAB(psiX,psiX),psiY2=getAB(psiY,psiY);
    final double [] IpsiX2=getPsiWIntegrated(psiX2),IpsiY2=getPsiWIntegrated(psiY2);
    final double [] p_x_gt_y=getAB(psiX2,IpsiY2),p_x_lt_y=getAB(psiY2,IpsiX2);
    if(true)
    {
	final int nm=Math.max(p_x_gt_y.length,p_x_lt_y.length);
	access.setMT(tmp,nm);
	return new double []{TMatr.mulVVn1(p_x_gt_y,tmp),TMatr.mulVVn1(p_x_lt_y,tmp)};
    }
    else
    {
	final double x0=getX0();
	return new double []{access.fx(WIntegratorLegendreShifted.B.getIntegrated(p_x_gt_y),x0),
			     access.fx(WIntegratorLegendreShifted.B.getIntegrated(p_x_lt_y),x0)};
    }
}
double getK3(final double [] psiX,final double [] psiY,final double [] psiZ)
{
    return _getK3(psiX,psiY,psiZ);
}

/** Calculates
 *  \int_{0}^{1} dx psiX^2(x) \int_{0}^x dy psiY^2(y) \int_{0}^y dz psiZ^2(z)
 */
public static double _getK3(final double [] psiX,final double [] psiY,final double [] psiZ)
{
    final double [] psiX2=getAB(psiX,psiX),psiY2=getAB(psiY,psiY),psiZ2=getAB(psiZ,psiZ);
    final double [] pp=getAB(psiX2,getPsiWIntegrated(getAB(psiY2,getPsiWIntegrated(psiZ2))));
    /*access.setMT(tmp,pp.length);
      return TMatr.mulVVn1(pp,tmp);
    */
    return pp[0];// analytical answer is known for this basis
}
double getK2(final double [] psia,final double [] psib,final double [] psic,final double [] psid)
{
    return _getK2(psia,psib,psic,psid);
}
/** Calculates
 *  \int_{0}^{1} dx psia(x)*psib(x) \int_{0}^x dy psic(y) psid(y)
 */
public static double _getK2(final double [] psia,final double [] psib,final double [] psic,final double [] psid)
{
    final double []
	psiX2=getAB(psia,psib),psiY2=getAB(psic,psid);
    final double [] pp=getAB(psiX2,getPsiWIntegrated(psiY2));
    /*access.setMT(tmp,pp.length);
    return TMatr.mulVVn1(pp,tmp);
    */
    return pp[0];// analytical answer is known for this basis
}
public static final K2Distance K2DISTANCE=new K2Distance(){public double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return _getK2(a,aa,b,bb);}};

double [] getSTMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{
    return _getSTMoments(nmomsr,psiS1,psiS2);
}

static double [] _getSTMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{
    final int nmoms=nmomsr+(psiS1.length-1+psiS2.length-1)+1;
    final double [] tmom=new double [nmoms];
    final double [] intG=getPsiWIntegrated(getAB(psiS1,psiS2));

    java.util.Arrays.fill(tmom,0);
    tmom[0]=1; // analytical answer is known for this basis
    final double [] r=WIntegratorLegendreShifted.B.getPQkMomentsFromQkMoments(intG,tmom);
    if(r.length!=nmomsr) throw new RuntimeException();
    return r;
}
static double [] _getSMoments(final int nmomsr,final double [] momx,final boolean flag_gt,final double [] psiS1,final double [] psiS2)
{
    final int nmoms=nmomsr+(psiS1.length-1+psiS2.length-1)+1;
    final double [] intG=getPsiWIntegrated(getAB(psiS1,psiS2));
    if(flag_gt)
    {
	final double x0=1;
	intG[0]-=WIntegratorLegendreShifted.B.getD0(intG,x0);
	TMatr.scaleVectorInPlace(intG,-1);
    }

    final double [] r=WIntegratorLegendreShifted.B.getPQkMomentsFromQkMoments(intG,momx);
    if(r.length!=nmomsr) throw new RuntimeException("r.length="+r.length+" nmomsr="+nmomsr);
    return r;
}


double getX0(){return 1;}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}

static class KkQVMLegendreShiftedTest extends KkQVMTest<QVMDataP> {
KkQVMLegendreShiftedTest(final long initrandom)
{
    super("KkQVMLegendreShifted",new java.util.Random(initrandom),1e-5,
	  4,1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
KkQVMLegendreShifted getKkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DkQVMLegendreShifted getDkQVM(){return DkQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
IkQVMLegendreShifted getIkQVM(){return IkQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
MkQVMLegendreShifted getMkQVM(){return MkQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}

double getXL(final KkQVM<?> q){return 0;}
double getXR(final KkQVM<?> q){return q.getX0();}
double omega(final double x){return 1;}

} // END of class KkQVMLegendreShiftedTest

public static void main(String [] args)
{
    new KkQVMLegendreShiftedTest(3L).runTests(6);
} // END main



} // END OF class KkQVMLegendreShifted
