package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/*
 * @version $Id: KkQVMMonomials.java,v 1.53 2022/07/10 10:41:21 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class KkQVMMonomials extends KkQVM<QVMData> {
KkQVMMonomials(final int neq,final QVMData odata){super(neq,odata);}

public static KkQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final KkQVMMonomials ref)
{
    final int nvm=1,npm=3,nmoms=2*neq-1;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     nmoms,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new KkQVMMonomials(neq,o);
}
DeprecatedHelperQVMDataDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataDirectAccessWithBasis();}

static double [] getPsiWIntegrated(final double [] polynomial)
{
    return WIntegratorMonomials.getPolynomialWIntegratedDt(polynomial);
}


/** Calculates 1/exp(t/2)\int_{-inf}^x psi(x)exp(t/2) dt.
 * This function is an inverse to _getEDPsi.
 * Possible unstable calculations used. Use WIntegratorMonomials.getEIntPsi_test for unit tests only.
 * For production - inverse matrix VolMatrix.getDMatrix.
 */
static double [] _getEIntPsi_test(final double [] psi)
{
    return TMatr.scaleVectorInPlace(getPxscaled(getPsiWIntegrated(getPxscaled(psi,2)),0.5),2);
}

static double [] getAB(final double [] a,final double [] b)
{
    return WIntegratorMonomials.B.mult2Pol(a,b);
}
double [] getW_GT_LT(final double [] psiX,final double [] psiY)
{
    final double [] psiX2=getAB(psiX,psiX),psiY2=getAB(psiY,psiY);
    final double [] p_x_gt_y=getPxscaled(getAB(psiX2,getPsiWIntegrated(psiY2)),1.0/2),
	p_x_lt_y=getPxscaled(getAB(psiY2,getPsiWIntegrated(psiX2)),1.0/2);
    final double [] tmom=new double [Math.max(p_x_gt_y.length,p_x_lt_y.length)];
    access.setMT(tmom,tmom.length);
    return new double []{0.5*TMatr.mulVVn1(p_x_gt_y,tmom),0.5*TMatr.mulVVn1(p_x_lt_y,tmom)};
}

static double [] getPxscaled(final double [] p,final double a)
{
    return Polynomials.getPolynomialArgumentScaled(p,a);
}

double getK3(final double [] psiX,final double [] psiY,final double [] psiZ)
{
    return _getK3(psiX,psiY,psiZ);
}

/** Calculates
 *  \int_{-inf}^{0} dx psiX^2(x) exp(x) \int_{-inf}^x dy psiY^2(y) exp(y) \int_{-inf}^y dz psiZ^2(z) exp(z)
 */
public static double _getK3(final double [] psiX,final double [] psiY,final double [] psiZ)
{
 final double [] psiX2=getAB(psiX,psiX),psiY2=getAB(psiY,psiY),psiZ2=getAB(psiZ,psiZ);
 final double [] pp=getPxscaled(getAB(psiX2,getPxscaled(getPsiWIntegrated(getPxscaled(getAB(psiY2,getPsiWIntegrated(psiZ2)),0.5)),2)),1.0/3);
 /*final double [] tmp=new double [pp.length];
 access.setMT(tmp,pp.length);
 return (1.0/(2*3))*TMatr.mulVVn1(pp,tmp);
 */
 // analytical answer is known for this basis
 long f=1;double s=0;
 for(int k=0;k<pp.length;f*=-(++k)){s+=f*pp[k];}
 return s/6;
}

double getK2(final double [] psia,final double [] psib,final double [] psic,final double [] psid)
{
    return _getK2(psia,psib,psic,psid);
}
/** Calculates
 *  \int_{-inf}^{0} dx psia(x)*psib(x) exp(x) \int_{-inf}^x dy psic(y) psid(y) exp(y)
 */
public static double _getK2(final double [] psia,final double [] psib,final double [] psic,final double [] psid)
{
    final double []  psiX2=getAB(psia,psib),psiY2=getAB(psic,psid);
    final double [] pp=getPxscaled(getAB(psiX2,getPsiWIntegrated(psiY2)),1.0/2);
    /*access.setMT(tmp,pp.length);
    return (1.0/2)*TMatr.mulVVn1(pp,tmp);
    */
    // analytical answer is known for this basis
    long f=1;double s=0;
    for(int k=0;k<pp.length;f*=-(++k)){s+=f*pp[k];}
    return s/2;
}
public static final K2Distance K2DISTANCE=new K2Distance(){public double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return _getK2(a,aa,b,bb);}};

double [] getSTMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{
    return _getSTMoments(nmomsr,psiS1,psiS2);
}

static double [] _getSTMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{
    final int nmoms=nmomsr+(psiS1.length-1+psiS2.length-1);
    final double [] intG=getPsiWIntegrated(getAB(psiS1,psiS2));
    final double [] tmom=new double [nmoms];
    // analytical answer is known for this basis
    double f=1;for(int k=0;k<nmoms;f*=-(++k)) tmom[k]=f;
    final double a=1.0/2;
    final double [] r=TMatr.scaleVectorInPlace(WIntegratorMonomials.B.getPQkMomentsFromQkMoments(intG,getPxscaled(tmom,a)),a);
    if(r.length!=nmomsr) throw new RuntimeException();
    return r;
}

static double [] _getSMoments(final int nmomsr,final double [] momx,final double [] mom2x,final boolean flag_gt,final double [] psiS1,final double [] psiS2)
{
    final int nmoms=nmomsr+(psiS1.length-1+psiS2.length-1);
    final double [] intG=getPsiWIntegrated(getAB(psiS1,psiS2));

    final double a=1.0/2;
    final double [] xmom2=TMatr.scaleVectorInPlace(WIntegratorMonomials.B.getPQkMomentsFromQkMoments(intG,getPxscaled(mom2x,a)),a);
    if(xmom2.length!=nmomsr) throw new RuntimeException("xmom2.length="+xmom2.length+" nmomsr="+nmomsr);
    if(flag_gt)
    {
	if(momx.length<xmom2.length) throw new RuntimeException();
	final double x0=0;
	final double intGx0=WIntegratorMonomials.B.getD0(intG,x0); // must be equal to 1
	for(int k=nmomsr;--k>=0;)
	{
	    xmom2[k]=intGx0*momx[k]-xmom2[k];
	}
    }
    return xmom2;
}

double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}

static class KkQVMMonomialsTest extends KkQVMTest<QVMData> {
KkQVMMonomialsTest(final long initrandom)
{
    super("KkQVMMonomials",new java.util.Random(initrandom),1e-5,
	  3,1e-2);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

KkQVMMonomials getKkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DkQVMLaguerre getDkQVM(){return DkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
IkQVMLaguerre getIkQVM(){return IkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
MkQVMLaguerre getMkQVM(){return MkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

double getXL(final KkQVM<?> q){return -(15+2*neq);}
double getXR(final KkQVM<?> q){return q.getX0();}
double omega(final double x){return Math.exp(x);}

} // END of class KkQVMMonomialsTest

public static void main(String [] args)
{
if(false)
{
    for(long rr=1;rr<10000;rr++)
    {
	try{
	new KkQVMMonomialsTest(rr).runTests(4);
	} catch(RuntimeException e) {
	    e.printStackTrace(System.err);
	    System.err.println("------------random="+rr+" failed");
	    continue;
	}
	System.err.println("rr="+rr+" OK");
	break;
    }
}
else
{
    new KkQVMMonomialsTest(5L).runTests(4);
}

} // END main



} // END OF class KkQVMMonomials
