package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.Quadrature;
import com.polytechnik.realtimebook.*;

/** QVM calculations.
 *  @version $Id: LQVMLaguerre.java,v 1.49 2022/06/26 17:54:35 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class LQVMLaguerre extends LQVM<QVMDataL> {
static final Laguerre B=new Laguerre();
LQVMLaguerre(final int neq,final QVMDataL odata)
{
    super(neq,odata);
}
double getPOffset(){return odata.p_offset;}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getBasis(){return new OrthogonalPolynomialsLaguerreBasis();}
void sortNodes(final double [] x,final double [] w){com.polytechnik.utils.SortArrays.sortArrayWithLoad(x.length,x,w);}
double [][] getTNodes(final int n)
{
    final double [] x=B.getRoots(n);
    return new double [][]{x,B.getWeights(x),B.getNodesPolynomial(n)};
}
double [][] getINodes(final double [][] p)
{
    final Quadrature.GaussQuadratureWithNodesPolynomialData q=basis.getGaussQuadratureFromPolynomials(p.length-1,p);
    return scaleWInd1(new double [][]{q.x,q.w,q.nodespolynomial});
}
boolean isAfter(final double x1,final double x2){return x1<x2;}

public static LQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final LQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new LQVMLaguerre(neq,o);
}


static class LQVMLaguerreTest extends LQVMTest<QVMDataL> {
LQVMLaguerreTest(final String name)
{
    super(name,
	  new com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis()
	  {
	      @Override
	      public double [] getRadauMultiplied(final double [] nodes0)
	      {
		  return Polynomials.getxP(nodes0);
	      }
	  },
	  new java.util.Random(6873L),1e-6);
}
void setData(final int tsize,final int neq)
{
    this.neq=neq;
    this.tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
    this.currenttime=tb[tb.length-1].time+1000;
    this.tau=(tb[tb.length-1].time-tb[0].time)/24;
}
LQVMLaguerre getLQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

QVMData getPData(final LQVMLaguerre q)
{
    //System.err.println("neq="+neq+" q.odata.ntm="+q.odata.ntm);
    final QVMData qdata=QVMData.calculateQVMData(1,q.odata.ntm,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
    return qdata;
}

void test_OLD_QVM()
{
    final LQVMLaguerre q=getLQVM();
    final QVMLL s=QVMLL.calculateQVMLL(q.xtnodes.length,currenttime,tau,tb,tb.length,null,null);
    System.err.println("q.xinodes[0]="+q.xinodes[0]+" q.wttnodes[0]="+q.wttnodes[0]+" s.neq_t="+s.neq_t+" neq="+neq+" q.xtnodes.length="+q.xtnodes.length);

    final double [] mul=new double [q.xtnodes.length+1];
    mul[mul.length-1]=1;
    final double [] dp=s.get_dmoments0(q.xtnodes.length);
    for(int k=0;k<dp.length;k++) dp[k]=((s.odata.trade_last.price-s.odata.p_offset)*dp[k]-s.dPcoefs[k]);
    final double [] dptnodes=new double [q.xtnodes.length];
    for(int k=0;k<q.xtnodes.length;k++)
    {
	UnitTests.checkDoubleRel(q.basis.B.getD0(mul,q.xtnodes[k]),"mul(x)",0,"0",eps);
	final double [] b=q.basis.B.sdiv1(mul,q.xtnodes[k]).q;
	final double f0=q.basis.B.getD0(b,q.xtnodes[k]);
	for(int j=b.length;--j>=0;) b[j]/=f0;
	dptnodes[k]=s.get_f_g(s.neq_t,b,dp);
	UnitTests.checkDoubleRel(q.wttnodes[k],"q.wttnodes[k]",b[0],"b[0]",eps);
    }
    UnitTests.checkDoubleArrayRel(q.dptnodes,"q.dptnodes",dptnodes,"dptnodes",eps);
    UnitTests.checkDoubleRel(q.dp2t,"q.dp2t",s.get_f_g(s.neq_t,dp,dp),"<dp|dp>",eps);

    System.err.println(name+":test_OLD_QVM(): Test OK, neq="+neq+" eps="+eps+" class="+q.getClass().getName());
    test_dp2i(false);
}
void test_SQVM()
{
    final LQVMLaguerre q=getLQVM();
    final SQVMLaguerre s=SQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);
    UnitTests.checkDoubleArrayRel(q.xinodes,"q.xinodes",s.xnodes,"s.xnodes",eps);
    UnitTests.checkDoubleArrayRel(q.wiinodes,"q.wiinodes",s.wnodes,"s.wnodes",eps);
    UnitTests.checkDoubleArrayRel(q.dpinodes,"q.dpinodes",TMatr.getVectorScaled(s.dpnodes,s.dp00),"s.dpnodes*s.dp00",eps);
    UnitTests.checkDoubleArrayRel(q.dsinodes,"q.dsinodes",s.dipnodes,"s.dipnodes",eps);
    //UnitTests.checkDoubleArrayRel(q.pinodes,"q.pinodes",s.pnodes,"s.pnodes",eps);

    System.err.println(name+":test_SQVM(): Test OK, neq="+neq+" eps="+eps+" class="+q.getClass().getName());
}
double [] getTMoments(final int n)
{
    final double [] mom=new double [n];
    for(int k=n;--k>=0;)
    {
	final double [] a=new double [k+1];
	a[k]=1;
	mom[k]=B.convertFunctionTo_Basis_from_Monomials(a)[0];
    }
    return mom;
}
void test_Moments()
{
    final  LQVMLaguerre q=getLQVM();
    final QVMData qvm=getPData(q);
    final double [] mom_i=MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(qvm.ntm,qvm,0,QVMData.MOM_dV,q.odata.p_offset);
    test_Monomials(mom_i,getTMoments(mom_i.length),MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(mom_i.length,qvm,1,QVMData.MOM_DP,q.odata.p_offset),TMatr.getVectorScaled(MQVMLaguerre.MQVMLaguerreTest.getMonomialMomDS(qvm.ntm,qvm),1/mom_i[0]));
}
} // END of class LQVMLaguerreTest

public static void main(String [] args)
{
    new LQVMLaguerreTest("LQVMLaguerre").runTests(6);
} // END main


} // END OF class LQVMLaguerre
