package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.Quadrature;

/** QVM calculations.
 *  @version $Id: LQVMLegendreShifted.java,v 1.62 2022/06/26 17:54:35 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class LQVMLegendreShifted extends LQVM<QVMDataP> {
static final LegendreShifted B=new LegendreShifted();
LQVMLegendreShifted(final int neq,final QVMDataP odata)
{
    super(neq,odata);
}
double getPOffset(){return odata.p_offset;}
QVMDataPDirectAccess getAccess(){return new QVMDataPDirectAccess();}
OrthogonalPolynomialsLegendreShiftedBasis getBasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
void sortNodes(final double [] x,final double [] w){com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(x.length,x,w);}
double [][] getTNodes(final int n)
{
    final double [] x=B.getRoots(n);
    return new double [][]{x,B.getWeights(x),B.getNodesPolynomial(n)};
}
double [][] getINodes(final double [][] p)
{
    final Quadrature.GaussQuadratureWithNodesPolynomialData q=basis.getGaussQuadratureFromPolynomials(p.length-1,p);
    return scaleWInd1(new double [][]{q.x,q.w,q.nodespolynomial});
}
boolean isAfter(final double x1,final double x2){return x1>x2;}

public static LQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final LQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new LQVMLegendreShifted(neq,o);
}


static class LQVMLegendreShiftedTest extends LQVMTest<QVMDataP> {
LQVMLegendreShiftedTest(final String name)
{
    super(name,
	  new com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis()
	  {
	      @Override
	      public double [] getRadauMultiplied(final double [] nodes0)
	      {
		  return Polynomials.get1mxP(nodes0);
	      }
	  },
	  new java.util.Random(6871L),1e-5);
}
void setData(final int tsize,final int neq)
{
    this.neq=neq;
    this.tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
    this.currenttime=tb[tb.length-1].time+1000;
    this.tau=((tb[tb.length-1].time-tb[0].time)/2)/(9*5*7*8*11*13)*(9*5*7*8*11*13);
    //System.err.println("tau="+tau);
}
LQVMLegendreShifted getLQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

QVMData [] getPData(final LQVMLegendreShifted q)
{
    final QVMData [] qdata=new QVMData [q.odata.ntm];
    for(int k=0;k<q.odata.ntm;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}

void test_OLD_QVM()
{
    final LQVMLegendreShifted q=getLQVM();
    final QVMPP s=QVMPP.calculateQVMPP(q.xtnodes.length,currenttime,tau,tb,tb.length,null,null);
    System.err.println("q.xinodes[0]="+q.xinodes[0]+" q.wttnodes[0]="+q.wttnodes[0]+" s.neq_t="+s.neq_t+" neq="+neq+" q.xtnodes.length="+q.xtnodes.length);

    final double [] mul=new double [q.xtnodes.length+1];
    mul[mul.length-1]=1;
    final double [] dp=new double [q.xtnodes.length];dp[0]=1;
    s.set_dmoments(dp.length,dp,dp);
    for(int k=0;k<dp.length;k++) dp[k]=((s.odata.trade_last.price-s.odata.p_offset)*dp[k]-s.dPcoefs[k]);
    final double [] dptnodes=new double [q.xtnodes.length];
    for(int k=0;k<q.xtnodes.length;k++)
    {
	UnitTests.checkDoubleRel(q.basis.B.getD0(mul,q.xtnodes[k]),"mul(x)",0,"0",eps);
	final double [] b=q.basis.B.sdiv1(mul,q.xtnodes[k]).q;
	final double f0=q.basis.B.getD0(b,q.xtnodes[k]);
	for(int j=b.length;--j>=0;) b[j]/=f0;
	dptnodes[k]=s.get_f_g(s.neq_t,b,dp);
	UnitTests.checkDoubleRel(q.wttnodes[k],"q.wttnodes[k]",b[0],"b[0]",eps);
    }
    UnitTests.checkDoubleArrayRel(q.dptnodes,"q.dptnodes",dptnodes,"dptnodes",eps);
    UnitTests.checkDoubleRel(q.dp2t,"q.dp2t",s.get_f_g(s.neq_t,dp,dp),"<dp|dp>",eps);

    System.err.println(name+":test_OLD_QVM(): Test OK, neq="+neq+" eps="+eps+" class="+q.getClass().getName());
    test_dp2i(false);
}
void test_SQVM()
{
    final LQVMLegendreShifted q=getLQVM();
    final SQVMLegendreShifted s=SQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);
    UnitTests.checkDoubleArrayRel(q.xinodes,"q.xinodes",s.xnodes,"s.xnodes",eps);
    UnitTests.checkDoubleArrayRel(q.wiinodes,"q.wiinodes",s.wnodes,"s.wnodes",eps);
    UnitTests.checkDoubleArrayRel(q.dpinodes,"q.dpinodes",TMatr.getVectorScaled(s.dpnodes,s.dp00),"s.dpnodes*s.dp00",eps);
    UnitTests.checkDoubleArrayRel(q.dsinodes,"q.dsinodes",s.dipnodes,"s.dipnodes",eps);
    //UnitTests.checkDoubleArrayRel(q.pinodes,"q.pinodes",s.pnodes,"s.pnodes",eps);

    System.err.println(name+":test_SQVM(): Test OK, neq="+neq+" eps="+eps+" class="+q.getClass().getName());
}
double [] getTMoments(final int n)
{
    final double [] mom=new double [n];
    for(int k=n;--k>=0;)
    {
	final double [] a=new double [k+1];
	a[k]=1;
	mom[k]=B.convertFunctionTo_Basis_from_Monomials(a)[0];
    }
    return mom;
}
void test_Moments()
{
    final LQVMLegendreShifted q=getLQVM();
    final QVMData [] qvm=getPData(q);
    final double [] mom_i=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(qvm.length,qvm,0,QVMData.MOM_dV,false,q.odata.p_offset);
    test_Monomials(mom_i,getTMoments(mom_i.length),MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(qvm.length,qvm,1,QVMData.MOM_DP,false,q.odata.p_offset),TMatr.getVectorScaled(MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomDS(mom_i.length,qvm),1/mom_i[0]));
}
} // END of class LQVMLegendreShiftedTest

public static void main(String [] args)
{
    new LQVMLegendreShiftedTest("LQVMLegendreShifted").runTests(5);
} // END main


} // END OF class LQVMLegendreShifted
