package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.realtimebook.*;

/** Finite variations in progress.
 *  @version $Id: LkQVM.java,v 1.112 2022/07/13 09:29:50 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class LkQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;

final EVXData I,P1t;
PData bpiL,fpiL,bpiH,fpiH,bipL,fipL,bipH,fipH;
PnLData pnlIL,pnlIH;
XRData xIL,xIH,xNIL,xNIH;
PEData pedt_IL,pedt_IH,pedi_IL,pedi_IH;
ProbVarData itpi_IL,itpi_IH,itpt_IL,itpt_IH;
ProbIVarData itpnl_IL,itpnl_IH;
ProbPVarData itvpt_IL,itvpt_IH;
AveragePEData dpt_IL,dpt_IH,vs_IL,vs_IH,Svs_IL,Svs_IH;
LkQVM(final int n,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();
    this.M=getABasis();

    final double [][] moms=getMoments(2*n-1);
    stmom=moms[0];simom=moms[1];sPtmom=moms[2];spimom=moms[3];sdpmom=moms[4];sx0moms=moms[5];svsmom=moms[6];sSvsmom=moms[7];
    final double [][] QQ=access.getQQ_Matrs(n,1,moms);
    final double [] QQt=QQ[0],QQi=QQ[1],QQpt=QQ[2];
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,getX0());
    this.P1t=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpt,QQt,access,getX0());
    lateInit();
}
final double [] stmom,simom,spimom,sPtmom,sdpmom,sx0moms,svsmom,sSvsmom;
void lateInit()
{
    this.bpiL=new PData(I.psiL,true);
    this.fpiL=new PData(I.psiL,false);
    this.bpiH=new PData(I.psiH,true);
    this.fpiH=new PData(I.psiH,false);

    this.bipL=new PData(P1t.neq,false,P1t.psiL,true);
    this.fipL=new PData(P1t.neq,false,P1t.psiL,false);
    this.bipH=new PData(P1t.neq,false,P1t.psiH,true);
    this.fipH=new PData(P1t.neq,false,P1t.psiH,false);

    this.pnlIL=new PnLData(I.psiL);
    this.pnlIH=new PnLData(I.psiH);
    this.xIL=new XRData(I.psiL,false);
    this.xIH=new XRData(I.psiH,false);
    this.xNIL=new XRData(I.psiL,true);
    this.xNIH=new XRData(I.psiH,true);

    this.pedt_IL=new PEData(I.psiL);
    this.pedt_IH=new PEData(I.psiH);
    this.pedi_IL=new PEData(I.psiL,false,1,3,0);
    this.pedi_IH=new PEData(I.psiH,false,1,3,0);

    this.itpi_IL=new ProbVarData(I.neq,stmom,simom,spimom,0,false);
    this.itpi_IH=new ProbVarData(I.neq,stmom,simom,spimom,I.neq-1,false);
    this.itpt_IL=new ProbVarData(I.neq,stmom,simom,sPtmom,0,false);
    this.itpt_IH=new ProbVarData(I.neq,stmom,simom,sPtmom,I.neq-1,false);

    this.itvpt_IL=new ProbPVarData(I.psiL,stmom,sPtmom,false);
    this.itvpt_IH=new ProbPVarData(I.psiH,stmom,sPtmom,false);

    this.itpnl_IL=new ProbIVarData(I.neq,stmom,sPtmom,simom,spimom,0,false);
    this.itpnl_IH=new ProbIVarData(I.neq,stmom,sPtmom,simom,spimom,I.neq-1,false);

    this.dpt_IL=new AveragePEData(I.psiL,4);
    this.dpt_IH=new AveragePEData(I.psiH,4);
    vs_IL=new AveragePEData(I.psiL,6);
    vs_IH=new AveragePEData(I.psiH,6);
    Svs_IL=new AveragePEData(I.psiL,7);
    Svs_IH=new AveragePEData(I.psiH,7);
}

class PData {
final EVXData J;
    final double w_PL_GT_PH,w_PH_GT_PL;
    PData(final double [] psiS,final boolean flag_gt){this(psiS.length,true,psiS,flag_gt);}
    PData(final int n,final boolean flag_P,final double [] psiS,final boolean flag_gt)
    {
	final double [][] moms=getSMoments(2*n-1,flag_gt,psiS,psiS);
	final double [][] QQS=access.getQQ_Matrs(n,1,moms);
	final double [] QQSt=QQS[0],QQSi=QQS[1],QQSpt=QQS[2];
	this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(n,flag_P?QQSpt:QQSi,QQSt,access,getX0());
	w_PL_GT_PH=getW_PsiA_GT_PsiB(J.psiL,J.psiH,flag_gt,psiS);
	w_PH_GT_PL=getW_PsiA_GT_PsiB(J.psiH,J.psiL,flag_gt,psiS);
    }
}

double getD1G(final int ind_lev,final EVXData J,final double [] QQ)
{
    final double [] psi=J.getPsi(ind_lev),pert=DeprecatedHelperEVXData.getPsiPertr(J,ind_lev,QQ);
    return getK2(pert,psi,psi,psi)-getK2(psi,psi,pert,psi);
}

double [] getX0moms(final int nmoms)
{
    return M.B.getX0Moments(nmoms,getX0());
}



class PEData {
final EVXData J;
final double [] prob,f,fprob,i,fi,iprob,varf,senspsi,sensf,sensft;
final int indt,indft,indpert,indi=1,indfi=3;final double [] psi;final boolean flag_gt;
PEData(final double [] psi)
{
    this(psi,false,0,2,1);
}
PEData(final double [] psi,final boolean flag_gt,final int indt,final int indft,final int indpert)
{
    this.indt=indt;this.indft=indft;this.indpert=indpert;this.psi=psi;this.flag_gt=flag_gt;
    final int n=psi.length;
    final double [][] QQ=access.getQQ_Matrs(n,1,getMSmom(2*n-1,psi,flag_gt,indt,indft,indpert));
    final double [] QQprob=QQ[0],QQfprob=QQ[1],QQiprob=QQ[2],QQi=QQ[3],QQfi=QQ[4],QQj=QQ[5],QQjprob=QQ[6],QQt=QQ[7],QQf=QQ[8];
    this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQprob,QQt,access,getX0());
    this.prob=new double [n];
    this.f=new double [n];
    this.sensft=new double [n];
    this.fprob=new double [n];
    this.fi=new double [n];
    this.i=new double [n];
    this.iprob=new double [n];
    this.varf=new double [n];
    this.senspsi=new double [n];
    this.sensf=new double [n];
    final double [] psik=new double [n],QQpert=new double [n*n];
    for(int k=0;k<n;k++)
    {
	prob[k]=J.ev[k];
	f[k]=TMatr.getAMatrB(n,J.ev,n+k*n,J.ev,n+k*n,QQf,n);
	fprob[k]=TMatr.getAMatrB(n,J.ev,n+k*n,J.ev,n+k*n,QQfprob,n)-prob[k]*f[k];
	i[k]=TMatr.getAMatrB(n,J.ev,n+k*n,J.ev,n+k*n,QQi,n);
	fi[k]=TMatr.getAMatrB(n,J.ev,n+k*n,J.ev,n+k*n,QQfi,n)/i[k];
	iprob[k]=TMatr.getAMatrB(n,J.ev,n+k*n,J.ev,n+k*n,QQiprob,n)-prob[k]*i[k];
	final double e=J.ev[k];
	System.arraycopy(J.ev,n+k*n,psik,0,n);
	if(true)
	{
	final double [] pert=DeprecatedHelperEVXData.getPsiPertr(J,k,QQf);
	this.sensft[k]=getK2(pert,psik,psik,psik)-getK2(psik,psik,pert,psik);
	}
	if(true)
	{
	for(int l=0;l<QQpert.length;l++){QQpert[l]=QQjprob[l]-e*QQj[l];}
	final double [] pert=DeprecatedHelperEVXData.getPsiPertr(J,k,QQpert);
	final double [] var=RayleighQuotient.getRQVariationValues(psik,pert,null,QQf,QQt,n);
	this.f[k]=var[0];
	this.varf[k]=var[1];
	this.senspsi[k]=getK2(pert,psik,psik,psik)-getK2(psik,psik,pert,psik);
	}
	if(true)
	{
	for(int l=0;l<QQpert.length;l++){QQpert[l]=QQfprob[l]-e*QQf[l];}
	final double [] pert=DeprecatedHelperEVXData.getPsiPertr(J,k,QQpert);
	this.sensf[k]=getK2(pert,psik,psik,psik)-getK2(psik,psik,pert,psik);
	}
    }
}
    double [][] getMSmom(final int nmoms,final double [] psiS,final boolean flag_gt,final int indt,final int indft,final int indpert)
{
    final double [][] moms=getMoments(nmoms),Smoms=getSMoments(nmoms,flag_gt,psiS,psiS);
    return new double [][]{Smoms[indt],Smoms[indft],Smoms[indi],moms[indi],moms[indfi],moms[indpert],Smoms[indpert],moms[indt],moms[indft]};
}
void test_Self(final double eps)
{
    final int n=J.neq;
    final PEData g=new PEData(this.psi,!flag_gt,indt,indft,indpert),h=new PEData(this.psi,flag_gt,indt,indt,indpert),v=new PEData(this.psi,flag_gt,indt,indft,indt);
    for(int k=0;k<n;k++)
    {
	UnitTests.checkDoubleRel(this.prob[k],"this.prob[k]",1-g.prob[n-1-k],"1-g.prob[n-1-k]",eps);
	UnitTests.checkDoubleRel(this.fprob[k],"this.fprob[k]",-g.fprob[n-1-k],"-g.fprob[n-1-k]",eps);
	if(indt!=indi) UnitTests.checkDoubleRel(this.iprob[k],"this.iprob[k]",-g.iprob[n-1-k],"-g.iprob[n-1-k]",eps);
	else UnitTests.checkDoubleRel(this.iprob[k],"this.iprob[k]",0,"0",eps);

	UnitTests.checkDoubleRel(0,"0",h.fprob[k],"h.fprob[k]",eps);
	UnitTests.checkDoubleRel(1,"1",h.f[k],"h.f[k]",eps);
	UnitTests.checkDoubleRel(0,"0",v.varf[k],"h.v.varf[k]",eps);
    }
    System.err.println("prob="+java.util.Arrays.toString(this.prob));
    System.err.println("i="+java.util.Arrays.toString(this.i));
    for(int k=0;k<n;k++)
    {
	//System.err.println("Psiprob("+k+")="+java.util.Arrays.toString(this.J.getPsi(k)));
    }

    System.err.println("LkQVM:PEData:test_Self: test OK n="+n+" eps="+eps);

    test_Pert(eps);
}

void test_Pert(final double eps)
{
    final int n=J.neq;
    final double [] psik=new double [n],psil=new double [n];
    final double [] varp=new double [n];
    final double [] probjmoms,jmoms,fmoms;
    if(true)
    {
	final int nmoms=2*n-1;
	double [][] mm=getMoments(nmoms);
	jmoms=mm[indpert];fmoms=mm[indft];
	mm=getSMoments(nmoms,this.flag_gt,this.psi,this.psi);
	probjmoms=mm[indpert];
    }
    for(int k=0;k<n;k++)
    {
	System.arraycopy(J.ev,n+k*n,psik,0,n);
	final double ek=J.ev[k];
	final double []
	    probj_k=M.B.getPQkMomentsFromQkMoments(psik,probjmoms),
	    j_k=M.B.getPQkMomentsFromQkMoments(psik,jmoms),
	    f_k=M.B.getPQkMomentsFromQkMoments(psik,fmoms);
	double s=0;
	for(int l=0;l<n;l++)
	{
	    System.arraycopy(J.ev,n+l*n,psil,0,n);
	    final double el=J.ev[l];
	    if(k!=l)
	    {
		s+=((TMatr.mulVVn1(psil,probj_k)-ek*TMatr.mulVVn1(psil,j_k))/(ek-el))*
		    TMatr.mulVVn1(psil,f_k);
	    }
	}
	varp[k]=2*s;
    }
    UnitTests.checkDoubleArrayRel(this.varf,"this.varf",varp,"varp",eps);

    System.err.println("LkQVM:PEData:test_Pert: test OK n="+n+" eps="+eps);
}


} // END OF class PEData


class AveragePEData {
final int indt=0,indft;
final double [] psi;
    final double wA_LT,dpA_LT,wA_GT,dpA_GT,dpPsi_LT,dpPsi_GT,wPsi_LT,wPsi_GT;

AveragePEData(final double [] psi,final int indft)
{
    this.indft=indft;this.psi=psi;
    final int n=psi.length,nmoms=2*n-1;
    {
    final double [][] Smoms=getSMoments(nmoms,false,psi,psi);
    this.dpA_LT=Smoms[indft][0];
    this.wA_LT=Smoms[indt][0];
    final double [][] SQQ=access.getQQ_Matrs(n,1,Smoms);
    final double [] SQQt=SQQ[indt],SQQf=SQQ[indft];
    this.dpPsi_LT=TMatr.getAMatrA(psi,SQQf,n);
    this.wPsi_LT=TMatr.getAMatrA(psi,SQQt,n);
    }
    {
    final double [][] Smoms=getSMoments(nmoms,true,psi,psi);
    this.dpA_GT=Smoms[indft][0];
    this.wA_GT=Smoms[indt][0];
    final double [][] SQQ=access.getQQ_Matrs(n,1,Smoms);
    final double [] SQQt=SQQ[indt],SQQf=SQQ[indft];
    this.dpPsi_GT=TMatr.getAMatrA(psi,SQQf,n);
    this.wPsi_GT=TMatr.getAMatrA(psi,SQQt,n);
    }

}
void test_Self(final double eps)
{
    final int n=this.psi.length,nmoms=2*n-1;
    UnitTests.checkDoubleRel(wA_LT+wA_GT,"wA_LT+wA_GT",1,"1",eps);
    final double [][] moms=getMoments(nmoms);
    UnitTests.checkDoubleRel(dpA_LT+dpA_GT,"dpA_LT+dpA_GT",moms[indft][0],"<f>",eps);
    UnitTests.checkDoubleRel(wPsi_LT,"wPsi_LT",0.5,"0.5",eps);
    UnitTests.checkDoubleRel(wPsi_GT,"wPsi_GT",0.5,"0.5",eps);
    final double dpPsi=TMatr.getAMatrA(this.psi,access.getQQ_Matrs(n,1,moms)[indft],n);
    UnitTests.checkDoubleRel(dpPsi_LT+dpPsi_GT,"dpPsi_LT+dpPsi_GT",dpPsi,"dpPsi",eps);

    for(boolean flag_gt: new boolean []{false,true})
    {
    final double [][] Smoms=getSMoments(nmoms,flag_gt,psi,psi);
    final double [][] SQQ=access.getQQ_Matrs(n,1,Smoms);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,SQQ[indt],n),"<psi|SQQt|psi>",0.5,"0.5",eps);
    if(flag_gt)
    {
	UnitTests.checkDoubleArrayRel(Smoms[5],"Sx0moms",new double [nmoms],"0",eps);
    }
    else
    {
	UnitTests.checkDoubleArrayRel(Smoms[5],"Sx0moms",getX0moms(nmoms),"getX0moms(nmoms)",eps);
    }
    }
    System.err.println("wA_LT="+wA_LT+" dpA_LT="+dpA_LT);
    System.err.println("wA_GT="+wA_GT+" dpA_GT="+dpA_GT);

    System.err.println("LkQVM:AveragePEData:test_Self: test OK n="+n+" eps="+eps);
}
} // END OF class AveragePEData



class ProbVarData {
final EVXData I,J;
final double [] probpert,i,fi;
final double d1G;
final int ind_lev;

ProbVarData(final int n,final double [] tmom,final double [] imom,final double [] fimom,final int ind_lev,final boolean flag_gt)
{
    this.ind_lev=ind_lev;
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{tmom,imom,fimom});
    final double [] QQt=QQ[0],QQi=QQ[1],QQfi=QQ[2];
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,getX0());
    final double [] psi=this.I.getPsi(ind_lev),psipert=DeprecatedHelperEVXData.getPsiPertr(this.I,ind_lev,QQfi);
    this.d1G=getK2(psipert,psi,psi,psi)-getK2(psi,psi,psipert,psi);
    // We need t-moments only. They can be calculated exactly.
    final double [][] SQQ=access.getQQ_Matrs(n,1,new double [][]{getSMoments(2*n-1,flag_gt,psi,psi)[0],getSMoments(2*n-1,flag_gt,psi,psipert)[0]});
    final double [] QQj=SQQ[0],QQjpert=TMatr.scaleVectorInPlace(SQQ[1],2);
    this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQj,QQt,access,getX0());
    //System.err.println("psi="+java.util.Arrays.toString(psi)+"\tflag_gt="+flag_gt);
    //System.err.println("ev="+java.util.Arrays.toString(this.J.ev));
    this.probpert=new double [n];
    this.i=new double [n];
    this.fi=new double [n];
    for(int k=0;k<n;k++)
    {
	this.i[k]=this.J.getFPsi(k,QQi);
	this.fi[k]=this.J.getFPsi(k,QQfi)/this.i[k];
	this.probpert[k]=this.J.getFPsi(k,QQjpert);
    }
    usflag_gt=flag_gt;
    ustmom=tmom;
    usimom=imom;
    usfimom=fimom;
}
final boolean usflag_gt;
final double [] ustmom,usimom,usfimom;

void test_Self(final double eps)
{
    final int n=this.J.neq;
    final ProbVarData g=new ProbVarData(n,ustmom,usimom,usfimom,ind_lev,!usflag_gt);

    for(int k=0;k<n;k++)
    {
	UnitTests.checkDoubleRel(this.J.ev[k],"this.J.ev[k]",1-g.J.ev[n-1-k],"1-g.J.ev[n-1-k]",eps);
	UnitTests.checkDoubleRel(this.probpert[k],"this.probpert[k]",-g.probpert[n-1-k],"-g.probpert[n-1-k]",eps);
    }

    System.err.println("LkQVM:ProbVarData:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class ProbVarData



class ProbIVarData {
final EVXData I,J;
final double [] probpert,i,mu,fmu;
final double d1G;
final int ind_lev;

ProbIVarData(final int n,final double [] tmom,final double [] ftmom,final double [] imom,final double [] fimom,final int ind_lev,final boolean flag_gt)
{
    this.ind_lev=ind_lev;
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{tmom,imom,ftmom,fimom});
    final double [] QQt=QQ[0],QQi=QQ[1],QQft=QQ[2],QQfi=QQ[3];
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,getX0());
    final double I_ind_lev=this.I.ev[ind_lev];
    final double [] QQpert=new double [n*n];
    for(int k=QQpert.length;--k>=0;){QQpert[k]=QQfi[k]-I_ind_lev*QQft[k];}
    final double [] psi=this.I.getPsi(ind_lev),psipert=DeprecatedHelperEVXData.getPsiPertr(this.I,ind_lev,QQpert);
    this.d1G=getK2(psipert,psi,psi,psi)-getK2(psi,psi,psipert,psi);
    // We need t-moments only. They can be calculated exactly.
    final double [][] SQQ=access.getQQ_Matrs(n,1,new double [][]{getSMoments(2*n-1,flag_gt,psi,psi)[0],getSMoments(2*n-1,flag_gt,psi,psipert)[0]});
    final double [] QQj=SQQ[0],QQjpert=TMatr.scaleVectorInPlace(SQQ[1],2);
    this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQj,QQt,access,getX0());

    this.probpert=new double [n];
    this.i=new double [n];
    this.mu=new double [n];
    this.fmu=new double [n];
    for(int k=0;k<n;k++)
    {
	this.i[k]=this.J.getFPsi(k,QQi);
	this.mu[k]=this.i[k]-I_ind_lev;
	this.fmu[k]=this.J.getFPsi(k,QQpert)/this.mu[k];
	this.probpert[k]=this.J.getFPsi(k,QQjpert);
    }
    usflag_gt=flag_gt;
    ustmom=tmom;
    usftmom=ftmom;
    usimom=imom;
    usfimom=fimom;
}
final boolean usflag_gt;
final double [] ustmom,usimom,usftmom,usfimom;

void test_Self(final double eps)
{
    final int n=this.J.neq;
    {
    final ProbIVarData s=new ProbIVarData(n,ustmom,usftmom,usimom,usfimom,ind_lev,!usflag_gt);
    for(int k=0;k<n;k++)
    {
	UnitTests.checkDoubleRel(this.J.ev[k],"this.J.ev[k]",1-s.J.ev[n-1-k],"1-s.J.ev[n-1-k]",eps);
	UnitTests.checkDoubleRel(this.probpert[k],"this.probpert[k]",-s.probpert[n-1-k],"-s.probpert[n-1-k]",eps);
    }
    }

    {
    final double [] pertmom=new double [2*n-1];
    for(int k=0;k<pertmom.length;k++){pertmom[k]=usfimom[k]-this.I.ev[this.ind_lev]*usftmom[k];}
    final ProbVarData s=new ProbVarData(n,ustmom,usimom,pertmom,this.ind_lev,this.usflag_gt);
    EVXData.matchEVXData(this.I,s.I,false,false,eps);
    EVXData.matchEVXData(this.J,s.J,false,false,eps);
    UnitTests.checkDoubleRel(this.d1G,"this.d1G",s.d1G,"s.d1G",eps);
    UnitTests.checkDoubleArrayRel(this.probpert,"this.probpert",s.probpert,"s.probpert",eps);
    UnitTests.checkDoubleArrayRel(this.i,"this.i",s.i,"s.i",eps);
    for(int k=0;k<n;k++)
    {
	UnitTests.checkDoubleRel(this.mu[k],"this.mu[k]",s.i[k]-s.I.ev[s.ind_lev],"s.i[k]-s.I.ev[s.ind_lev]",eps);
	UnitTests.checkDoubleRel(this.fmu[k]*(this.mu[k]/this.i[k]),"this.fmu[k]*(mu/I)",s.fi[k],"s.fi[k]",eps);
    }
    }

    System.err.println("LkQVM:ProbIVarData:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class ProbIVarData




class ProbPVarData {
final EVXData J;
final double [] prob,f,sens;

ProbPVarData(final double [] psi,final double [] tmom,final double [] pertmom,final boolean flag_gt)
{
    final int n=psi.length;
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{tmom,getSMoments(2*n-1,flag_gt,psi,psi)[0],pertmom});
    final double [] QQt=QQ[0],QQS=QQ[1],QQpert=QQ[2];
    this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQS,QQt,access,getX0());

    this.prob=new double [n];
    this.f=new double [n];
    this.sens=new double [n];
    for(int k=0;k<n;k++)
    {
	this.prob[k]=this.J.getFPsi(k,QQS);
	this.f[k]=this.J.getFPsi(k,QQpert);
	this.sens[k]=getD1G(k,this.J,QQpert);
    }
}
void match_PEData(final PEData s,final double eps)
{
    final int n=this.J.neq;
    EVXData.matchEVXData(this.J,s.J,false,false,eps);

    UnitTests.checkDoubleArrayRel(this.f,"this.f",s.f,"s.f",eps);
    UnitTests.checkDoubleArrayRel(this.sens,"this.sens",s.sensft,"s.sensft",eps);

    UnitTests.checkDoubleArrayRel(this.J.getEValues(),"this.J.getEValues()",this.prob,"this.prob",eps);

    System.err.println("LkQVM:ProbPVarData:match_PEData: test OK n="+n+" eps="+eps);
}

} // END OF class ProbPVarData






class XRData {
final double fXx;
final PnLData pnl;
final PData bp,fp;

XRData(final double [] psi,final boolean flag_Nevai)
{
    final int n=psi.length;
    final double [][] r=DkQVM.getRoots(psi,M,isEdgeAtRight());
    final double [] Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0_NonSortedX(Xx,getX0(),isEdgeAtRight());
    this.fXx=indX>=0?Xx[indX]:Double.NaN;
    final double [] Dpsi=flag_Nevai?getPsiNevaiX(n,this.fXx):getDpsiNormalized(psi,this.fXx,false);
    this.pnl=new PnLData(Dpsi);
    this.bp=new PData(n,true,Dpsi,true);
    this.fp=new PData(n,true,Dpsi,false);
}
void test_Self(final double eps)
{
    System.err.println("fXx="+fXx);
    pnl.test_Self(eps);
    //bp.test_Self(eps);
    //fp.test_Self(eps);
    System.err.println("LkQVM:XRData:test_Self: test OK n="+bp.J.neq+" eps="+eps);
}


} // END OF class XRData


double [] getDpsiNormalized(final double [] psi,final double x,final boolean flag_x0)
{
    final int n=psi.length;
    final double [] pp;
    {
	final double [] Dpsi=M.B.sdiv1(psi,x).q;
	if(flag_x0)
	{
	    pp=M.getRadauMultiplied(Dpsi);
	}
	else
	{
	    pp=Dpsi;
	}
    }

    // normalize
    return TMatr.scaleVectorInPlace(pp,1/Math.sqrt(TMatr.getAMatrA(pp,M.B.getQQMatr(pp.length,stmom),pp.length)));
}

/** Nevai function.
 */
double [] getPsiNevaiX(final int n,final double x)
{
    final double [] pp=new double [n],rr=new double [n];
    for(int k=0;k<n;k++)
    {
	rr[k]=1;
	pp[k]=M.B.getD0(rr,x);
	rr[k]=0;
    }
    System.arraycopy(pp,0,rr,0,n);
    final double [] QQ=M.B.getQQMatr(n,stmom);
    final int [] ipvt=new int [n];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(QQ,n,n,ipvt);
    if(pinfo!=0) {java.util.Arrays.fill(pp,Double.NaN);}
    else{com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,n,ipvt,pp,1);}
    return TMatr.scaleVectorInPlace(pp,1/Math.sqrt(TMatr.mulVVn1(pp,rr)));
}


class PnLData {
final double pPsiS,gtPnL,ltPnL;
final double gt,gtI,gtpI,gtPi,gtPt,gtDp,lt,ltI,ltpI,ltPi,ltPt,ltDp;
PnLData(final double [] psiS){this(psiS,psiS);}

PnLData(final double [] psiS,final double [] psiW)
{
	final int n=psiS.length;
	{
	    final double [][] QQ=access.getQQ_Matrs(n,1,getSMoments(2*n-1,true,psiW,psiW));
	final double [] QQSt=QQ[0],QQSi=QQ[1],QQSpt=QQ[2],QQSpi=QQ[3],QQSdp=QQ[4];
	gtI=TMatr.getAMatrA(psiS,QQSi,n);
	gtpI=TMatr.getAMatrA(psiS,QQSpi,n);
	gtPi=gtpI/gtI;
	final double nt=this.gt=TMatr.getAMatrA(psiS,QQSt,n);
	gtPt=TMatr.getAMatrA(psiS,QQSpt,n)/nt;
	gtDp=TMatr.getAMatrA(psiS,QQSdp,n)/nt;
	//System.err.println("<psiS|QQSt|psiS>="+nt+" (must be 1/2)");
	}

	{
	    final double [][] QQ=access.getQQ_Matrs(n,1,getSMoments(2*n-1,false,psiW,psiW));
	final double [] QQSt=QQ[0],QQSi=QQ[1],QQSpt=QQ[2],QQSpi=QQ[3],QQSdp=QQ[4];
	ltI=TMatr.getAMatrA(psiS,QQSi,n);
	ltpI=TMatr.getAMatrA(psiS,QQSpi,n);
	ltPi=ltpI/ltI;
	final double nt=this.lt=TMatr.getAMatrA(psiS,QQSt,n);
	ltPt=TMatr.getAMatrA(psiS,QQSpt,n)/nt;
	ltDp=TMatr.getAMatrA(psiS,QQSdp,n)/nt;
	//System.err.println("<psiS|QQSt|psiS>="+nt+" (must be 1/2)");
	}
	this.pPsiS=(gtpI+ltpI)/(gtI+ltI);
	gtPnL=gtpI-this.pPsiS*gtI;
	ltPnL=ltpI-this.pPsiS*ltI;
	//System.err.println("gtPnL="+gtPnL+" ltPnL="+ltPnL+" sum="+(gtPnL+ltPnL)+" (must be 0)\ngtI="+gtI+" ltI="+ltI);
	this.sPsiS=psiS;
}
private final double [] sPsiS;

void test_Self(final double eps)
{
    final double [] psiS=sPsiS;
    final int n=psiS.length;
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{stmom,simom,sPtmom,spimom,sdpmom});
    final double [] QQt=QQ[0],QQi=QQ[1],QQpt=QQ[2],QQpi=QQ[3],QQdp=QQ[4];
    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiS,QQt,n),"<psiS|QQt|psiS>",lt+gt,"lt+gt",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiS,QQi,n),"<psiS|QQi|psiS>",ltI+gtI,"ltI+gtI",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiS,QQpi,n),"<psiS|QQpi|psiS>",ltpI+gtpI,"ltpI+gtpI",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiS,QQpi,n)/TMatr.getAMatrA(psiS,QQi,n),"<P>_{i:psiS}",(ltPi*ltI+gtPi*gtI)/(ltI+gtI),"(ltPi*ltI+gtPi*gtI)/(ltI+gtI)",eps);
    final double nt=TMatr.getAMatrA(psiS,QQt,n);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiS,QQpt,n)/nt,"<P>_{t:psiS}",(ltPt+gtPt)/2,"(ltPt+gtPt)/2",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiS,QQdp,n)/nt,"<Dp>_{t:psiS}",(ltDp+gtDp)/2,"(ltDp+gtDp)/2",eps);

    System.err.println("gt="+gt+" lt="+lt+" nt="+nt);
    System.err.println("gtI="+gtI+" ltI="+ltI);
    System.err.println("gtpI="+gtpI+" ltpI="+ltpI);
    System.err.println("gtPt="+gtPt+" ltPt="+ltPt);
    System.err.println("gtDp="+gtDp+" ltDp="+ltDp);

    System.err.println("LkQVM:PnLData:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class PnLData



abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb);
abstract double getX0();
abstract double getPOffset();
abstract boolean isEdgeAtRight();

double [][] getMoments(final int nmoms)
{

    final double [] Ptmom=new double [nmoms],dpmom=new double [nmoms],tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms],
	vsmom=new double [nmoms],Svsmom=new double [nmoms];

    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setFMoments(Ptmom,dpmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    access.setMT(tmom,nmoms);
    access.setM(vsmom,0,access.M_RVS,odata,nmoms);
    access.setM(Svsmom,0,access.M_VS,odata,nmoms);

    return new double [][]{tmom,imom,Ptmom,pimom,dpmom,getX0moms(nmoms),vsmom,Svsmom};
}

abstract double [][] getSMoments(final int nmoms,final boolean flag_gt,final double [] psiS1,final double [] psiS2);
abstract double getW_PsiA_GT_PsiB(final double [] psiA,final double [] psiB,final boolean flag_gt,final double [] psiS);



static abstract class LkQVMTest<DTYPE> extends QTest {

abstract LkQVM<DTYPE> getLkQVM();
abstract IkQVM<DTYPE> getIkQVM();

LkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}

void test_Common()
{
    final LkQVM<DTYPE> q=getLkQVM();
    q.pnlIL.test_Self(eps);
    q.pnlIH.test_Self(eps);
    q.xIL.test_Self(eps);
    q.xIH.test_Self(eps);
    q.xNIL.test_Self(eps);
    q.xNIH.test_Self(eps);
    q.pedt_IL.test_Self(eps);
    q.pedt_IH.test_Self(eps);
    q.pedi_IL.test_Self(eps);UnitTests.checkDoubleArrayRel(q.pedi_IL.f,"q.pedi_IL.f",q.pedi_IL.fi,"q.pedi_IL.fi",eps);
    q.pedi_IH.test_Self(eps);
    q.pedi_IH.test_Self(eps);UnitTests.checkDoubleArrayRel(q.pedi_IH.f,"q.pedi_IH.f",q.pedi_IH.fi,"q.pedi_IH.fi",eps);
    q.itpi_IL.test_Self(eps);
    q.itpi_IH.test_Self(eps);
    q.itpt_IL.test_Self(eps);
    q.itpt_IH.test_Self(eps);

    q.itvpt_IL.match_PEData(q.pedt_IL,eps);
    q.itvpt_IH.match_PEData(q.pedt_IH,eps);

    q.itpnl_IL.test_Self(eps);
    q.itpnl_IH.test_Self(eps);

    q.dpt_IL.test_Self(eps);
    q.dpt_IH.test_Self(eps);
    q.vs_IL.test_Self(eps);
    q.vs_IH.test_Self(eps);
    q.Svs_IL.test_Self(eps);
    q.Svs_IH.test_Self(eps);

    System.err.println(name+":LkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}

void test_IkQVM()
{
  final LkQVM<DTYPE> q=getLkQVM();
  final IkQVM<DTYPE> s=getIkQVM();
  final int n=neq;

  for(int ll=0;ll<q.I.neq;ll++)
  {
    final double [] psiW=q.I.getPsi(ll);
    final double [][] momsGT=q.getSMoments(2*n-1,true,psiW,psiW),momsLT=q.getSMoments(2*n-1,false,psiW,psiW);
    final double [] momtGT=momsGT[0],momiGT=momsGT[1],momptGT=momsGT[2],mompiGT=momsGT[3],
	momtLT=momsLT[0],momiLT=momsLT[1],momptLT=momsLT[2],mompiLT=momsLT[3];

    final double [] psiW2=q.M.B.mult2Pol(psiW,psiW);
    final double [] moms=new double [2*n-1];
    for(int k=0;k<moms.length;k++)
    {
	moms[k]=1;
	final IkQVM.GGPData ggp=s.getGGP(false,moms,psiW2),ggi=s.getGGI(false,moms,psiW2),ggpi=s.getGGPI(false,moms,psiW2);
	UnitTests.checkDoubleRel(momtGT[k],"momtGT[k]",ggp.wC_y_gt_x,"ggp.wC_y_gt_x",eps);
	UnitTests.checkDoubleRel(momtLT[k],"momtLT[k]",ggp.wC_y_lt_x,"ggp.wC_y_lt_x",eps);

	UnitTests.checkDoubleRel(momptGT[k],"momptGT[k]",ggp.Px_y_gt_x,"ggp.Px_y_gt_x",eps);
	UnitTests.checkDoubleRel(momptLT[k],"momptLT[k]",ggp.Px_y_lt_x,"ggp.Px_y_lt_x",eps);

	UnitTests.checkDoubleRel(momiGT[k],"momiGT[k]",ggi.Px_y_gt_x,"ggi.Px_y_gt_x",eps);
	UnitTests.checkDoubleRel(momiLT[k],"momiLT[k]",ggi.Px_y_lt_x,"ggi.Px_y_lt_x",eps);

	UnitTests.checkDoubleRel(mompiGT[k],"mompiGT[k]",ggpi.Px_y_gt_x,"ggpi.Px_y_gt_x",eps);
	UnitTests.checkDoubleRel(mompiLT[k],"mompiLT[k]",ggpi.Px_y_lt_x,"ggpi.Px_y_lt_x",eps);
	moms[k]=0;
    }
  }

    System.err.println(name+":LkQVMTest:test_IkQVM(): test OK neq="+neq+" eps="+eps);
}

void test_getW_PsiA_GT_PsiB()
{
    final LkQVM<DTYPE> q=getLkQVM();

    final double [] psiS=q.I.psiH;
    for(boolean flag_P: new boolean []{false,true})
    for(boolean flag_gt: new boolean []{false,true})
    {
	final LkQVM<DTYPE>.PData p=q.new PData(psiS.length,flag_P,psiS,flag_gt);
	final double w_psiA_GT_psiB=q.getW_PsiA_GT_PsiB(p.J.psiL,p.J.psiH,flag_gt,psiS);
	final double w_psiB_GT_psiA=q.getW_PsiA_GT_PsiB(p.J.psiH,p.J.psiL,flag_gt,psiS);
	System.err.println("test_getW_PsiA_GT_PsiB():flag_gt="+flag_gt+" w_psiA_GT_psiB="+w_psiA_GT_psiB+" w_psiB_GT_psiA="+w_psiB_GT_psiA+" sum="+(w_psiA_GT_psiB+w_psiB_GT_psiA));
	UnitTests.checkDoubleRel(w_psiA_GT_psiB+w_psiB_GT_psiA,"w_psiA_GT_psiB+w_psiB_GT_psiA",1,"1",eps);

	final double f=2/*0.98137344*/,f1=1/*.0975237344*/,f2=/*0.9804237344*/1,ff=Math.pow(f1*f2*f*f,2);
	final double fw_psiA_GT_psiB=q.getW_PsiA_GT_PsiB(TMatr.getVectorScaled(p.J.psiL,f1),TMatr.getVectorScaled(p.J.psiH,f2),flag_gt,TMatr.getVectorScaled(psiS,f));
	final double fw_psiB_GT_psiA=q.getW_PsiA_GT_PsiB(TMatr.getVectorScaled(p.J.psiH,f1),TMatr.getVectorScaled(p.J.psiL,f2),flag_gt,TMatr.getVectorScaled(psiS,f));
	UnitTests.checkDoubleRel(w_psiA_GT_psiB,"w_psiA_GT_psiB",fw_psiA_GT_psiB/ff,"fw_psiA_GT_psiB/ff",eps);
	UnitTests.checkDoubleRel(w_psiB_GT_psiA,"w_psiB_GT_psiA",fw_psiB_GT_psiA/ff,"fw_psiB_GT_psiA/ff",eps);



    }
    System.err.println(name+":LkQVMTest:test_getW_PsiA_GT_PsiB(): test OK neq="+neq+" eps="+eps);
}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":LkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_IkQVM();
    test_getW_PsiA_GT_PsiB();

    System.err.println(name+":LkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":LkQVMTest: starting tests; ver=$Id: LkQVM.java,v 1.112 2022/07/13 09:29:50 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":LkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: LkQVM.java,v 1.112 2022/07/13 09:29:50 mal Exp $");
}
} // END OF class LkQVMTest




}// END OF class LkQVM
