package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: LkQVMLaguerre.java,v 1.54 2022/11/20 18:08:54 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class LkQVMLaguerre extends LkQVM<QVMDataL> {
final QVMDataL odata2;
LkQVMLaguerre(final int neq,final QVMDataL odata,final QVMDataL odata2){
    super(neq,odata);
    this.odata2=odata2;
    lateInit();
}
void lateInit()
{
    if(odata2==null) return;
    super.lateInit();
}

public static LkQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final LkQVMLaguerre ref)
{
    final int npm=3,nmoms=2*(neq-1)+1,nmoms2=4*(neq-1)+1;
    final QVMDataL o=QVMDataL.calculateQVMDataL(nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null),
	o2=QVMDataL.calculateQVMDataL(nmoms2,
				      npm,
				      currenttime,
				      tau/2,
				      tb,
				      tsize,
				      ref!=null?ref.odata2:null);
    return new LkQVMLaguerre(neq,o,o2);
}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return KkQVMLaguerre._getK2(a,aa,b,bb);}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
boolean isEdgeAtRight(){return false;}


double [][] getSMoments(final int nmomsr,final boolean flag_gt,final double [] psiS1,final double [] psiS2)
{
    final int nmoms=nmomsr+(psiS1.length-1+psiS2.length-1);
    final double [] intG=KkQVMLaguerre.getPsiWIntegrated(KkQVMLaguerre.getAB(psiS1,psiS2));

    final double [] Ptmom=new double [nmoms],dpmom=new double [nmoms],tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms],x0moms=TMatr.scaleVectorInPlace(getX0moms(nmoms),2),vsmom=new double [nmoms],Svsmom=new double [nmoms];
    access.setM(dpmom,1,access.M_DP,odata2,nmoms);
    access.setFMoments(Ptmom,dpmom,nmoms);
    access.setM(imom,0,access.M_I,odata2,nmoms);
    access.setM(pimom,1,access.M_I,odata2,nmoms);
    access.setMT(tmom,nmoms);
    access.setM(vsmom,0,access.M_RVS,odata2,nmoms);
    access.setM(Svsmom,0,access.M_VS,odata2,nmoms);

    final double [] tmom2=M.B.getPQkMomentsFromQkMoments(intG,KkQVMLaguerre.getMomentsConvertedArg(tmom,2)),
	imom2=M.B.getPQkMomentsFromQkMoments(intG,KkQVMLaguerre.getMomentsConvertedArg(imom,2)),
	pimom2=M.B.getPQkMomentsFromQkMoments(intG,KkQVMLaguerre.getMomentsConvertedArg(pimom,2)),
	Ptmom2=M.B.getPQkMomentsFromQkMoments(intG,KkQVMLaguerre.getMomentsConvertedArg(Ptmom,2)),
	dpmom2=M.B.getPQkMomentsFromQkMoments(intG,KkQVMLaguerre.getMomentsConvertedArg(dpmom,2)),
	x0moms2=M.B.getPQkMomentsFromQkMoments(intG,KkQVMLaguerre.getMomentsConvertedArg(x0moms,2)),
	vsmom2=M.B.getPQkMomentsFromQkMoments(intG,KkQVMLaguerre.getMomentsConvertedArg(vsmom,2)),
	Svsmom2=M.B.getPQkMomentsFromQkMoments(intG,KkQVMLaguerre.getMomentsConvertedArg(Svsmom,2));
    if(tmom2.length!=nmomsr||stmom.length<tmom2.length) throw new RuntimeException();

    if(flag_gt)
    {
	final double intGx0=M.B.getD0(intG,getX0())*Math.exp(0);
	//System.err.println("intGx0="+intGx0+" (must be equal to 1 because of normalization)");
	//System.err.println("x0moms2="+java.util.Arrays.toString(x0moms2)+" x0moms="+java.util.Arrays.toString(x0moms));
	for(int k=nmomsr;--k>=0;)
	{
	    tmom2[k]=intGx0*stmom[k]-tmom2[k];
	    imom2[k]=intGx0*simom[k]-imom2[k];
	    pimom2[k]=intGx0*spimom[k]-pimom2[k];
	    Ptmom2[k]=intGx0*sPtmom[k]-Ptmom2[k];
	    dpmom2[k]=intGx0*sdpmom[k]-dpmom2[k];
	    x0moms2[k]=intGx0*sx0moms[k]-x0moms2[k];
	    vsmom2[k]=intGx0*svsmom[k]-vsmom2[k];
	    Svsmom2[k]=intGx0*sSvsmom[k]-Svsmom2[k];
	}
    }
    return new double [][]{tmom2,imom2,Ptmom2,pimom2,dpmom2,x0moms2,vsmom2,Svsmom2};
}
double getW_PsiA_GT_PsiB(final double [] psiA,final double [] psiB,final boolean flag_gt,final double [] psiS)
{
    final boolean FLAG_OPTIMIZE_NORMALBASIS=true;
    final int nmoms=2*(psiA.length-1)+2*(2*(psiS.length-1))+2*(psiB.length-1)+1;
    final double [] intG=KkQVMLaguerre.getPsiWIntegrated(KkQVMLaguerre.getAB(psiS,psiS));
    final double x0=getX0(),intGx0=M.B.getD0(intG,x0)*Math.exp(0);
    //System.err.println("intG("+x0+")="+intGx0+" (must be 1 because of normalization psiS)");
    final double [] psiA2=KkQVMLaguerre.getAB(psiA,psiA),psiB2=KkQVMLaguerre.getAB(psiB,psiB);
    final double [] BintG=KkQVMLaguerre.getAB(psiB2,intG),BintG_arg2x=KkQVMLaguerre.getPxArgumentScaled(BintG,1.0/2),
	intBintG_arg2x=TMatr.scaleVectorInPlace(KkQVMLaguerre.getPsiWIntegrated(BintG_arg2x),0.5),
	AintG=KkQVMLaguerre.getAB(psiA2,intG),AintG_arg2x=KkQVMLaguerre.getPxArgumentScaled(AintG,1.0/2),
	f4_arg4x=TMatr.scaleVectorInPlace(KkQVMLaguerre.getPxArgumentScaled(KkQVMLaguerre.getAB(AintG_arg2x,intBintG_arg2x),1.0/2),1.0/4);

    final double f;
    if(flag_gt)
    {
	final double [] intB=KkQVMLaguerre.getPsiWIntegrated(psiB2),intA=KkQVMLaguerre.getPsiWIntegrated(psiA2),
	    f3a_arg3x=TMatr.scaleVectorInPlace(KkQVMLaguerre.getPxArgumentScaled(KkQVMLaguerre.getAB(AintG,intB),1.0/3),intGx0/3),
//	    f3b_arg3x=TMatr.scaleVectorInPlace(KkQVMLaguerre.getAB(KkQVMLaguerre.getPxArgumentScaled(psiA2,1.0/3),KkQVMLaguerre.getPxArgumentScaled(intBintG_arg2x,2.0/3)),intGx0/3),
	    f3bi_arg3x=TMatr.scaleVectorInPlace(KkQVMLaguerre.getPxArgumentScaled(KkQVMLaguerre.getAB(BintG,intA),1.0/3),intGx0/3),
	    f3bia_arg2x=TMatr.getVectorScaled(BintG_arg2x,M.B.getD0(intA,x0)*intGx0/2),
	    f2_arg2x=TMatr.scaleVectorInPlace(KkQVMLaguerre.getPxArgumentScaled(KkQVMLaguerre.getAB(psiA2,intB),1.0/2),intGx0*intGx0/2);
//	System.err.println("flag_gt="+flag_gt+"\t(M.B.getD0(intB,0)*intGx0-M.B.getD0(intBintG_arg2x,0))="+(M.B.getD0(intB,0)*intGx0-M.B.getD0(intBintG_arg2x,0))+" (must be 1 because of normalization psiB)");

//	System.err.println("f2="+M.B.getD0(KkQVMLaguerre.getPsiWIntegrated(f2_arg2x),2*x0)+"\tf2._getK2="+(intGx0*intGx0*KkQVMLaguerre._getK2(psiA,psiA,psiB,psiB))+"\tf2_arg2x[0]="+f2_arg2x[0]+"\tintGx0="+intGx0);
//	System.err.println("f3bia_arg2x[0]-f3bi_arg3x[0]="+(f3bia_arg2x[0]-f3bi_arg3x[0]));
//	System.err.println("f3b_arg3x[0]="+f3b_arg3x[0]);

	// optimization
	f=FLAG_OPTIMIZE_NORMALBASIS?
	    (f4_arg4x[0]-f3a_arg3x[0]-(f3bia_arg2x[0]-f3bi_arg3x[0])+f2_arg2x[0])
	    :
	    (M.B.getD0(KkQVMLaguerre.getPsiWIntegrated(f4_arg4x),4*x0)-M.B.getD0(KkQVMLaguerre.getPsiWIntegrated(f3a_arg3x),3*x0)-
	     (M.B.getD0(KkQVMLaguerre.getPsiWIntegrated(f3bia_arg2x),2*x0)-M.B.getD0(KkQVMLaguerre.getPsiWIntegrated(f3bi_arg3x),3*x0))
	     +M.B.getD0(KkQVMLaguerre.getPsiWIntegrated(f2_arg2x),2*x0))
	    ;
    }
    else
    {
//	System.err.println("flag_gt="+flag_gt+"\tM.fx(intBintG_arg2x,0)="+M.B.getD0(intBintG_arg2x,0)+" (must be 1 because of normalization psiB)");
	// optimization
	f=FLAG_OPTIMIZE_NORMALBASIS?(f4_arg4x[0]):(M.B.getD0(KkQVMLaguerre.getPsiWIntegrated(f4_arg4x),4*x0));
    }
    return f;
}



static class LkQVMLaguerreTest extends LkQVMTest<QVMDataL> {
LkQVMLaguerreTest(final long initrandom)
{
    super("LkQVMLaguerre",new java.util.Random(initrandom),1e-3);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

LkQVMLaguerre getLkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
IkQVMLaguerre getIkQVM(){return IkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class LkQVMLaguerreTest

public static void main(String [] args)
{
    new LkQVMLaguerreTest(111117L).runTests(4);
} // END main



} // END OF class LkQVMLaguerre
