package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: LkQVMLegendreShifted.java,v 1.30 2022/07/10 10:41:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class LkQVMLegendreShifted extends LkQVM<QVMDataP> {
LkQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static LkQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final LkQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(4*(neq-1)+2,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new LkQVMLegendreShifted(neq,o);
}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return KkQVMLegendreShifted._getK2(a,aa,b,bb);}
double getX0(){return 1;}
double getPOffset(){return odata.p_offset;}
boolean isEdgeAtRight(){return true;}


double [][] getSMoments(final int nmomsr,final boolean flag_gt,final double [] psiS1,final double [] psiS2)
{
    final int nmoms=nmomsr+(psiS1.length-1+psiS2.length-1)+1;
    final double [] tmp=new double [nmoms];
    final double [] intG=KkQVMLegendreShifted.getPsiWIntegrated(KkQVMLegendreShifted.getAB(psiS1,psiS2));
    if(flag_gt)
    {
	intG[0]-=M.B.getD0(intG,getX0());
	TMatr.scaleVectorInPlace(intG,-1);
    }
    final double [] Ptmom=new double [nmoms],dpmom=new double [nmoms],tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms],vsmom=new double [nmoms],Svsmom=new double [nmoms];
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setFMoments(Ptmom,dpmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    access.setMT(tmom,nmoms);
    access.setM(vsmom,0,access.M_RVS,odata,nmoms);
    access.setM(Svsmom,0,access.M_VS,odata,nmoms);

    final double [][] rr=new double [][]{
	M.B.getPQkMomentsFromQkMoments(intG,tmom),
	M.B.getPQkMomentsFromQkMoments(intG,imom),
	M.B.getPQkMomentsFromQkMoments(intG,Ptmom),
	M.B.getPQkMomentsFromQkMoments(intG,pimom),
	M.B.getPQkMomentsFromQkMoments(intG,dpmom),
	M.B.getPQkMomentsFromQkMoments(intG,getX0moms(nmoms)),
	M.B.getPQkMomentsFromQkMoments(intG,vsmom),
	M.B.getPQkMomentsFromQkMoments(intG,Svsmom)
    };
    if(rr[0].length!=nmomsr) throw new RuntimeException();
    return rr;
}
double getW_PsiA_GT_PsiB(final double [] psiA,final double [] psiB,final boolean flag_gt,final double [] psiS)
{
    final boolean FLAG_OPTIMIZE_NORMALBASIS=true;
    final int nmoms=2*(psiA.length-1)+2*(2*(psiS.length-1)+1)+2*(psiB.length-1)+1+1;
    final double [] tmp=new double [nmoms];
    final double [] intG=KkQVMLegendreShifted.getPsiWIntegrated(KkQVMLegendreShifted.getAB(psiS,psiS));
    if(flag_gt)
    {
	intG[0]-=M.B.getD0(intG,getX0());
	TMatr.scaleVectorInPlace(intG,-1);
	//System.err.println("flag_gt="+flag_gt+":intG("+getX0()+")="+M.B.getD0(intG,getX0())+" (must be 0 because of normalization psiS)");
    }
    else
    {
	//System.err.println("flag_gt="+flag_gt+":intG("+getX0()+")="+M.B.getD0(intG,getX0())+" (must be 1 because of normalization psiS)");
    }
    final double [] s=KkQVMLegendreShifted.getAB(KkQVMLegendreShifted.getAB(psiA,psiA),KkQVMLegendreShifted.getAB(intG,KkQVMLegendreShifted.getPsiWIntegrated(KkQVMLegendreShifted.getAB(intG,KkQVMLegendreShifted.getAB(psiB,psiB)))));

    // Optimization, taking into account normality of the basis.
    return FLAG_OPTIMIZE_NORMALBASIS?(s[0]):(M.B.getD0(KkQVMLegendreShifted.getPsiWIntegrated(s),getX0()));
}


static class LkQVMLegendreShiftedTest extends LkQVMTest<QVMDataP> {
LkQVMLegendreShiftedTest(final long initrandom)
{
    super("LkQVMLegendreShifted",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
LkQVMLegendreShifted getLkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
IkQVMLegendreShifted getIkQVM(){return IkQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}


} // END of class LkQVMLegendreShiftedTest

public static void main(String [] args)
{
    new LkQVMLegendreShiftedTest(162147L).runTests(5);
} // END main



} // END OF class LkQVMLegendreShifted
