package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: LwQVMLaguerre.java,v 1.19 2022/07/10 10:41:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class LwQVMLaguerre extends LwQVM<QVMDataL> {
final QVMDataL odata2;
LwQVMLaguerre(final int neq,final QVMDataL odata,final QVMDataL odata2){
    super(neq,odata);
    this.odata2=odata2;
    lateInit(neq);
}
void lateInit(final int n)
{
    if(odata2==null) return;
    super.lateInit(n);
}

public static LwQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final LwQVMLaguerre ref)
{
    final int npm=3,nmoms=2*(neq-1)+1,nmoms2=4*(neq-1)+1;
    final QVMDataL o=QVMDataL.calculateQVMDataL(nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null),
	o2=QVMDataL.calculateQVMDataL(nmoms2,
				      npm,
				      currenttime,
				      tau/2,
				      tb,
				      tsize,
				      ref!=null?ref.odata2:null);
    return new LwQVMLaguerre(neq,o,o2);
}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return KkQVMLaguerre._getK2(a,aa,b,bb);}

double [][] getSMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{
    final int nmoms=nmomsr+(psiS1.length-1+psiS2.length-1);
    final double [] intG=KkQVMLaguerre.getPsiWIntegrated(KkQVMLaguerre.getAB(psiS1,psiS2));
    final double [] tmom=new double [nmoms],imom=new double [nmoms];
    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata2,nmoms);
    final double []
	tmom2=WIntegratorLaguerre.B.getPQkMomentsFromQkMoments(intG,KkQVMLaguerre.getMomentsConvertedArg(tmom,2)),
	imom2=WIntegratorLaguerre.B.getPQkMomentsFromQkMoments(intG,KkQVMLaguerre.getMomentsConvertedArg(imom,2));
    if(tmom2.length!=nmomsr) throw new RuntimeException();

    return new double [][]{tmom2,imom2};
}

double [] getSTMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{
    return KkQVMLaguerre._getSTMoments(nmomsr,psiS1,psiS2);
}


static class LwQVMLaguerreTest extends LwQVMTest<QVMDataL> {
LwQVMLaguerreTest(final long initrandom)
{
    super("LwQVMLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

LwQVMLaguerre getLwQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
LkQVMLaguerre getLkQVM(){return LkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
MkQVMLaguerre getMkQVM(){return MkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class LwQVMLaguerreTest

public static void main(String [] args)
{
    new LwQVMLaguerreTest(1117L).runTests(4);
} // END main



} // END OF class LwQVMLaguerre
