package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: LwQVMLegendreShifted.java,v 1.18 2022/07/10 10:41:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class LwQVMLegendreShifted extends LwQVM<QVMDataP> {

LwQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static LwQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final LwQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(4*(neq-1)+2,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new LwQVMLegendreShifted(neq,o);
}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
double getX0(){return 1;}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return KkQVMLegendreShifted._getK2(a,aa,b,bb);}

double [][] getSMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{

    final int nmoms=nmomsr+(psiS1.length-1+psiS2.length-1)+1;
    final double [] tmom=new double [nmoms],imom=new double [nmoms];
    final double [] intG=KkQVMLegendreShifted.getPsiWIntegrated(KkQVMLegendreShifted.getAB(psiS1,psiS2));

    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);

    final double [][] rr=new double [][]{
	WIntegratorLegendreShifted.B.getPQkMomentsFromQkMoments(intG,tmom),
	WIntegratorLegendreShifted.B.getPQkMomentsFromQkMoments(intG,imom)
    };
    if(rr[0].length!=nmomsr) throw new RuntimeException();
    return rr;
}

double [] getSTMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{
    return KkQVMLegendreShifted._getSTMoments(nmomsr,psiS1,psiS2);
}


static class LwQVMLegendreShiftedTest extends LwQVMTest<QVMDataP> {
LwQVMLegendreShiftedTest(final long initrandom)
{
    super("LwQVMLegendreShifted",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
LwQVMLegendreShifted getLwQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
LkQVMLegendreShifted getLkQVM(){return LkQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
MkQVMLegendreShifted getMkQVM(){return MkQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}


} // END of class LwQVMLegendreShiftedTest

public static void main(String [] args)
{
    new LwQVMLegendreShiftedTest(162147L).runTests(5);
} // END main



} // END OF class LwQVMLegendreShifted
