package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: LwQVMMonomials.java,v 1.18 2022/07/10 10:41:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class LwQVMMonomials extends LwQVM<QVMData> {
final QVMData odata2;
LwQVMMonomials(final int neq,final QVMData odata,final QVMData odata2){
    super(neq,odata);
    this.odata2=odata2;
    lateInit(neq);
}
void lateInit(final int n)
{
    if(odata2==null) return;
    super.lateInit(n);
}

public static LwQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final LwQVMMonomials ref)
{
    final int nvm=1,npm=3,nmoms=2*(neq-1)+1,nmoms2=4*(neq-1)+1;
    final QVMData o=QVMData.calculateQVMData(nvm,nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null),
	o2=QVMData.calculateQVMData(nvm,nmoms2,
				      npm,
				      currenttime,
				      tau/2,
				      tb,
				      tsize,
				      ref!=null?ref.odata2:null);
    return new LwQVMMonomials(neq,o,o2);
}
DeprecatedHelperQVMDataDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataDirectAccessWithBasis();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return KkQVMMonomials._getK2(a,aa,b,bb);}


double [][] getSMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{

    final int nmoms=nmomsr+(psiS1.length-1+psiS2.length-1);
    final double [] intG=KkQVMMonomials.getPsiWIntegrated(KkQVMMonomials.getAB(psiS1,psiS2));
    final double [] tmom=new double [nmoms],imom=new double [nmoms];
    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata2,nmoms);
    final double a=1.0/2;
    final double []
	tmom2=WIntegratorMonomials.B.getPQkMomentsFromQkMoments(intG,KkQVMMonomials.getPxscaled(tmom,a)),
	imom2=WIntegratorMonomials.B.getPQkMomentsFromQkMoments(intG,KkQVMMonomials.getPxscaled(imom,a));
    if(tmom2.length!=nmomsr) throw new RuntimeException();
    TMatr.scaleVectorInPlace(tmom2,a);
    TMatr.scaleVectorInPlace(imom2,a);
    return new double [][]{tmom2,imom2};
}

double [] getSTMoments(final int nmomsr,final double [] psiS1,final double [] psiS2)
{
    return KkQVMMonomials._getSTMoments(nmomsr,psiS1,psiS2);
}


static class LwQVMMonomialsTest extends LwQVMTest<QVMData> {
LwQVMMonomialsTest(final long initrandom)
{
    super("LwQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

LwQVMMonomials getLwQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
LkQVMLaguerre getLkQVM(){return LkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
MkQVMLaguerre getMkQVM(){return MkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class LwQVMMonomialsTest

public static void main(String [] args)
{
    new LwQVMMonomialsTest(1117L).runTests(4);
} // END main



} // END OF class LwQVMMonomials
