package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.realtimebook.*;

/** Impact QVM calculations with the basis Q_s dv/dt.
 *  @version $Id: MQVM.java,v 1.36 2022/06/27 13:32:45 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class MQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;

/**
 * dp0  = dp/dv * dt/dv
 * dp0v = dp/dt * dv/dt
 * dp0i = dp/dv
 */
final double dp0,ds0,sigma2_dp0,sigma2_ds0,dp0v,ds0v,dp0i,ds0i,i0,dp00,v00;
final double [] dp_dFdVdTdV,ds_dFdVdTdV,dp_dV,ds_dV,dp_dI,ds_dI;

MQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final double [] mom=new double [2*neq-1];
    access.setM(mom,0,access.M_I,odata,mom.length);

    final double [] mom1=new double [mom.length],moms=new double [mom.length];
    access.setM(moms,0,access.M_ISELL,odata,moms.length);
    access.setM(mom1,0,access.M_IBUY,odata,moms.length);
    v00=moms[0]+mom1[0];for(int k=moms.length;--k>=0;) moms[k]=(moms[k]-mom1[k])/v00;

    access.setM(mom1,1,access.M_DPPLUS,odata,1);
    final double _pp0=mom1[0];
    access.setM(mom1,1,access.M_DPMINUS,odata,1);
    dp00=_pp0-mom1[0];
    access.setM(mom1,1,access.M_DP,odata,mom1.length);

    final double [][] rx=getdFdVdTdVexpansion(neq,mom,new double [][]{mom1,moms});
    dp_dFdVdTdV=rx[0];
    dp0=get_f_o(neq,dp_dFdVdTdV);
    sigma2_dp0=get_f_g(neq,dp_dFdVdTdV,dp_dFdVdTdV)*get_o_o(neq)-dp0*dp0;
    dp_dI=rx[1];
    i0=TMatr.mulVVn1(rx[2],mom)/mom[0];
    dp0i=get_f_o(neq,dp_dI);
    dp_dV=rx[3];
    dp0v=get_f_o(neq,dp_dV);

    ds_dFdVdTdV=rx[4];
    ds0=get_f_o(neq,ds_dFdVdTdV);
    sigma2_ds0=get_f_g(neq,ds_dFdVdTdV,ds_dFdVdTdV)*get_o_o(neq)-ds0*ds0;
    ds_dI=rx[5];
    ds0i=get_f_o(neq,ds_dI);
    ds_dV=rx[7];
    ds0v=get_f_o(neq,ds_dV);
}
abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE>  getAccess();
abstract void setQNorms(final int n,final double [] norm);
abstract double get_f_g(final int n,final double [] f,final double [] g);
abstract double get_f_o(final int n,final double [] f);
abstract double get_o_o(final int n);

double [][] getdFdVdTdVexpansion(final int neq,final double [] mom_v,final double [][] mom_dp)
{
    final int n=neq;
    final double v00=mom_v[0];
    final double [] matrvv=new double [n*n],tmp=new double [2*n-1],
	Qnorm=new double [n],matr=new double [n*n];
    setQNorms(n,Qnorm);
    for(int k=0;k<n;k++)
    {
	final int ok=n*k;
	for(int l=0;l<n;l++)
	{
	    final int na=access.setQQCoefs(l,k,tmp);
	    matrvv[ok+l]=TMatr.mulVVn(na,tmp,mom_v)/v00;
	}

	for(int l=0;l<=k;l++)
	{
	    final int ol=n*l;
	    double s=0;
	    for(int j=n;--j>=0;) s+=Qnorm[j]*matrvv[ok+j]*matrvv[ol+j];
	    matr[l+ok]=matr[k+ol]=s;
	}
    }
    //  matr   is now equal to <Q Q>_v <Q Q>^{-1}_t <Q Q>_v
    //  matrvv is now equal to <Q Q>_v

    // obtain (<Q Q>_t <Q Q>^{-1}_v <Q Q>_t)^{-1}=<Q Q>^{-1}_t <Q Q>_v <Q Q>^{-1}_t
    // and save it to matro.
    final double [] matro=new double [n*n];
    for(int k=0;k<n;k++)
    {
	double qk=Qnorm[k];for(int l=0;l<n;l++) matro[k*n+l]=qk*Qnorm[l]*matrvv[k*n+l];
    }

    final double [][] rpart=new double [mom_dp.length*4][n];
    final int [] ipvt=new int [n],ipvt2=new int [n];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,n,n,ipvt);
    final int pinfo2=com.polytechnik.utils.Linsystems.dgefa_m1(matrvv,n,n,ipvt2);
    final double [] gg=new double [n];
    if(pinfo!=0||pinfo2!=0) for(int k=0;k<rpart.length;k++) java.util.Arrays.fill(rpart[k],Double.NaN);
    else for(int i=0;i<rpart.length;i+=4)
	 {
	     final int k=i/4;

	     // dp/dv * dt/dv
	     System.arraycopy(mom_dp[k],0,rpart[i],0,n);
	     // rpart is now equal to <dp/dt Q>_t, solve linsystem, result is saved to rpart.
	     com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,rpart[i],1);

	     // dp/dv
	     System.arraycopy(mom_dp[k],0,rpart[i+1],0,n);
	     com.polytechnik.utils.Linsystems.dgesl_m1(matrvv,n,n,ipvt2,rpart[i+1],1);

	     // l at radau end point
	     final double [] ff=rpart[i+2];
	     for(int j=0;j<n;j++){gg[j]=1;ff[j]=get_f_o(n,gg);gg[j]=0;}
	     com.polytechnik.utils.Linsystems.dgesl_m1(matrvv,n,n,ipvt2,ff,1);
	     final double ffo=get_f_o(n,ff);
	     for(int j=0;j<n;j++) ff[j]/=ffo;

	     // dp/dt * dv/dt
	     final double [] mm=mom_dp[k];
	     for(int l=0;l<n;l++)
	     {
		 double s=0;for(int j=0;j<n;j++) s+=mm[j]*matro[l*n+j];
		 rpart[i+3][l]=s;
	     }
	 }
    return rpart;
}

static abstract class MQVMTest<DTYPE> extends QTest {

abstract MQVM<DTYPE> getMQVM();
abstract double [][] getMonomialMoments();
abstract void test_Moments();
abstract double getX0();

MQVMTest(final String name,
	 final java.util.Random r,final double eps)
{
    super(name,r,eps);
}
void test_Common()
{
    System.err.println(name+":MQVMTest:test_Commons(): test OK neq="+neq+" eps="+eps);
}


double [] monomials_getdFdVdTdVexpansion(final int n,final double [] mom_v,final double [] mom_t,final double [] mom_dp)
{
    final double [][] basis=new double [n][];
    final double [] matr=new double [n*n];
    final int [] ipvt=new int [n];
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<=k;l++)
	{
	    matr[l+n*k]=matr[k+l*n]=mom_t[l+k]/mom_t[0];
	}
    }
    final int tinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,n,n,ipvt);
    if(tinfo!=0) throw new RuntimeException("tinfo="+tinfo);

    for(int k=0;k<n;k++)
    {
	final double [] b=basis[k]=new double [n];
	for(int l=0;l<n;l++) b[l]=mom_v[l+k]/mom_v[0];
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,b,1);
    }

    // set matr for alfas
    final double [] rp=new double [n];
    for(int k=0;k<n;k++)
    {
	rp[k]=mom_dp[k];
	for(int l=0;l<=k;l++)
	{
	    double s=0;for(int j=0;j<n;j++) s+=(mom_v[j+k]/mom_v[0])*basis[l][j];
	    matr[l+k*n]=matr[k+l*n]=s;
	}
    }

    final double [][] matr_t=new double [n][n],matr_v=new double [n][n];
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<n;l++)
	{
	    matr_t[k][l]=mom_t[k+l]/mom_t[0];
	    matr_v[k][l]=mom_v[k+l]/mom_v[0];
	}
    }
    // matr=G_v G^{-1}_t G_v
    final double [][] matr_test=DeprecatedHelperTMatr.mulM_x_N(matr_v,DeprecatedHelperTMatr.mulM_x_N(com.polytechnik.utils.Linsystems.getInvertedMatrix(matr_t),matr_v));
    for(int k=0;k<n;k++)
    {
	final double [] m=new double [n];
	System.arraycopy(matr,k*n,m,0,n);
	UnitTests.checkDoubleArrayRel(m,"m",matr_test[k],"matr_test[k]",eps);
	//System.err.println("k="+k+"\tm="+java.util.Arrays.toString(m));
	//System.err.println("\tm="+java.util.Arrays.toString(matr_test[k]));
    }

    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,n,n,ipvt);
    if(pinfo!=0) throw new RuntimeException("pinfo="+pinfo);
    com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,rp,1);

    return rp;
}


void test_Monomials_getdFdVdTdVexpansion()
{
    final double [][] amoms=getMonomialMoments();
    final double [] mom_v=amoms[0],mom_t=amoms[1],mom_dp=amoms[2],mom_is=amoms[3];

    final MQVM<DTYPE> q=getMQVM();
    final double [] dp_exp=monomials_getdFdVdTdVexpansion(neq,mom_v,mom_t,mom_dp);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.dp_dFdVdTdV),"conv(q.dp_dFdVdTdV)",dp_exp,"dp_exp",eps);
    UnitTests.checkDoubleRel(q.dp0,"q.dp0",Polynomials.getD0(dp_exp,getX0()),"dp_exp(x0)",eps);
    UnitTests.checkDoubleRel(q.sigma2_dp0/q.get_o_o(neq)+q.dp0*q.dp0/q.get_o_o(neq),"<fdp|fdp>",TMatr.mulVVn1(Polynomials.mult2Pol(dp_exp,dp_exp),mom_t),"<fdp|fdp> monomials",eps);

    final double [] ds_exp=monomials_getdFdVdTdVexpansion(neq,mom_v,mom_t,mom_is);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.ds_dFdVdTdV),"conv(q.ds_dFdVdTdV)",ds_exp,"ds_exp",eps);
    UnitTests.checkDoubleRel(q.ds0,"q.ds0",Polynomials.getD0(ds_exp,getX0()),"ds_exp(x0)",eps);
    UnitTests.checkDoubleRel(q.sigma2_ds0/q.get_o_o(neq)+q.ds0*q.ds0/q.get_o_o(neq),"<fds|fds>",TMatr.mulVVn1(Polynomials.mult2Pol(ds_exp,ds_exp),mom_t),"<fds|fds> monomials",eps);


    // swap mom_t & mom_v
    final double [] dp_dV=monomials_getdFdVdTdVexpansion(neq,mom_t,mom_v,mom_dp);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.dp_dV),"conv(q.dp_dV)",dp_dV,"dp_dV",eps);
    UnitTests.checkDoubleRel(q.dp0v,"q.dp0v",Polynomials.getD0(dp_dV,getX0()),"dp_dV(x0)",eps);

    final double [] ds_dV=monomials_getdFdVdTdVexpansion(neq,mom_t,mom_v,mom_is);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.ds_dV),"conv(q.ds_dV)",ds_dV,"ds_dV",eps);
    UnitTests.checkDoubleRel(q.ds0v,"q.ds0v",Polynomials.getD0(ds_dV,getX0()),"ds_dV(x0)",eps);

    System.err.println(name+":MQVMTest:test_Monomials(): test OK neq="+neq+" eps="+eps+" class="+q.getClass().getName());
}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":MQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Moments();
    test_Monomials_getdFdVdTdVexpansion();

    System.err.println(name+":MQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":MQVMTest: starting tests; ver=$Id: MQVM.java,v 1.36 2022/06/27 13:32:45 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<118;i++)
	{
	    test_Calculations(7250+r.nextInt(7550),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":MQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: MQVM.java,v 1.36 2022/06/27 13:32:45 mal Exp $");
}
} // END OF class MQVMTest



}// END OF class MQVM
