package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class MQVMLaguerre extends MQVM<QVMDataL> {

    static final Laguerre B=new Laguerre();

MQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static MQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final MQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new MQVMLaguerre(neq,o);
}


DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
void setQNorms(final int n,final double [] norm)
{
    final double [] tmp=new double [n];tmp[0]=1;
    System.arraycopy(B.getL1FromL0(tmp),0,norm,0,n);
}
double get_f_g(final int n,final double [] f,final double [] g){
    if(n!=f.length) throw new RuntimeException();
    return B.get2LaguerreAveraged(f,g);
}
double get_f_o(final int n,final double [] f){
    if(f.length!=n) throw new RuntimeException();
    return B.getD0(f,0);
}
double get_o_o(final int n){return n;}

static class MQVMLaguerreTest extends MQVMTest<QVMDataL> {
MQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-5);
}
double getX0(){return 0;}
long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
MQVMLaguerre getMQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData getPData(final int nmoms)
{
    final QVMData qdata=QVMData.calculateQVMData(1,nmoms,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
    return qdata;
}
static double [] getMonomialMomT(final int nmoms)
{
    final double [] mom_t=new double [nmoms];
    for(int k=nmoms;--k>=0;)
    {
	final double [] a=new double [k+1];a[k]=1;
	mom_t[k]=B.convertFunctionTo_Basis_from_Monomials(a)[0];
    }
    return mom_t;
}
static double [] getMonomialMoments(final int nmoms,final QVMData d,final int mom_num,final int QVMData_whatmom,final double p_offset)
{
    final double [] mom=new double [nmoms];
    for(int k=nmoms;--k>=0;)
    {
	final int o=k*d.npm;
	final double dof=-(p_offset-d.p_offset);
	final double [] mconv=d.shiftOffset(0,0,dof,QVMData_whatmom);
	mom[k]=mconv[o+mom_num];
    }
    WIntegratorMonomials.inverseOddCoefs(nmoms,mom,mom);
    return mom;
}
static double [] getMonomialMomDP(final int nmoms,final QVMData d)
{
    return getMonomialMoments(nmoms,d,1,QVMData.MOM_DP,0);
}
static double [] getMonomialMomTS(final int nmoms,final QVMData d)
{
    return getMonomialMoments(nmoms,d,0,QVMData.MOM_TS,0);
}
static double [] getMonomialMomVS(final int nmoms,final QVMData d)
{
    return getMonomialMoments(nmoms,d,0,QVMData.MOM_VS,0);
}
static double [] getMonomialMomRVS(final int nmoms,final QVMData d)
{
    return getMonomialMoments(nmoms,d,0,QVMData.MOM_RVS,0);
}
static double [] getMonomialMomRTS(final int nmoms,final QVMData d)
{
    return getMonomialMoments(nmoms,d,0,QVMData.MOM_RTS,0);
}
static double [] getMonomialMomDS(final int nmoms,final QVMData d)
{
    final double [] mom_sell=getMonomialMoments(nmoms,d,0,QVMData.MOM_VSELL,0),
	mom_buy=getMonomialMoments(nmoms,d,0,QVMData.MOM_VBUY,0);
    TMatr.sumP1(mom_sell,mom_buy,-1);
    return mom_sell;
}
double [][] getMonomialMoments()
{
    final int nmoms=2*neq-1;
    return getMonomialMoments(nmoms,getPData(nmoms));
}
static double [][] getMonomialMoments(final int nmoms,final QVMData d)
{
    final double [] mom_v=getMonomialMoments(nmoms,d,0,QVMData.MOM_dV,0);
    return new double [][]{mom_v,getMonomialMomT(nmoms),getMonomialMomDP(nmoms,d),TMatr.getVectorScaled(getMonomialMomDS(nmoms,d),1/mom_v[0]),getMonomialMoments(nmoms,d,0,QVMData.MOM_VBUY,0),getMonomialMoments(nmoms,d,0,QVMData.MOM_VSELL,0)};
}

void test_Moments(){
    final MQVMLaguerre q=getMQVM();
    final RadauLQVMLaguerre s=RadauLQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);
    UnitTests.checkDoubleRel(q.dp0i,"q.dp0i",s.dpinodes[0]/s.wiinodes[0],"s.dpinodes[0]/s.wiinodes[0]",eps);
    UnitTests.checkDoubleRel(q.i0,"q.i0",s.wiinodes[0],"s.wiinodes[0]",eps);
    UnitTests.checkDoubleRel(q.ds0i,"q.ds0i",s.dsinodes[0]/s.wiinodes[0],"s.dsinodes[0]/s.wiinodes[0]",eps);
    System.err.println(name+":MQVMLaguerreTest:test_Moments(): test OK neq="+neq+" eps="+eps+" class="+q.getClass().getName());
}

} // END of class MQVMLaguerreTest

public static void main(String [] args)
{
    new MQVMLaguerreTest("MQVMLaguerre").runTests(5);
} // END main


} // END OF class MQVMLaguerre
