package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.realtimebook.*;

public class MQVMLegendreShifted extends MQVM<QVMDataP> {
    static final LegendreShifted B=new LegendreShifted();
MQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static MQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final MQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new MQVMLegendreShifted(neq,o);
}

DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
void setQNorms(final int n,final double [] norm)
{
    final double [] tmp=new double [n];tmp[0]=1;
    System.arraycopy(DeprecatedHelperEVXData.getXPnprimeplusPnMoments(tmp),0,norm,0,n);
}
double get_f_g(final int n,final double [] f,final double [] g){
    if(n!=f.length) throw new RuntimeException();
    return B.get2LegendreAveraged(f,g);
}
double get_f_o(final int n,final double [] f){
    if(f.length!=n) throw new RuntimeException();
    return B.getD0(f,1);
}
double get_o_o(final int n){return n*n;}

static class MQVMLegendreShiftedTest extends MQVMTest<QVMDataP> {
MQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-5);
}
double getX0(){return 1;}
long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/2)/(9*5*7*8*11*13)*(9*5*7*8*11*13);
}
MQVMLegendreShifted getMQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData [] getPData(final int nmoms)
{
    final QVMData [] qdata=new QVMData [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}
double [][] getMonomialMoments()
{
    final int nmoms=2*neq-1;
    return getMonomialMoments(nmoms,getPData(nmoms));
}
static double [] getMonomialMomT(final int nmoms)
{
    final double [] mom_t=new double [nmoms];
    for(int k=nmoms;--k>=0;)
    {
	final double [] a=new double [k+1];a[k]=1;
	mom_t[k]=B.convertFunctionTo_Basis_from_Monomials(a)[0];
    }
    return mom_t;
}
static double [] getMonomialMoments(final int nmoms,final QVMData [] mm,final int mom_num,final int QVMData_whatmom,final boolean flag_scaletau,final double p_offset)
{
    final double [] mom=new double [nmoms];
    for(int k=nmoms;--k>=0;)
    {
	final QVMData d=mm[k];
	final int o=0*d.npm;
	final double dof=-(p_offset-d.p_offset);
	final double [] mconv=d.shiftOffset(0,0,dof,QVMData_whatmom);
	final double f;
	if(flag_scaletau){f=d.tau/(double)mm[0].tau;}
	else {f=1;}
	mom[k]=f*mconv[o+mom_num];
    }
    return mom;
}

static double [] getMonomialMomDP(final int nmoms,final QVMData [] mm)
{
    return getMonomialMoments(nmoms,mm,1,QVMData.MOM_DP,false,0);
}
static double [] getMonomialMomTS(final int nmoms,final QVMData [] mm)
{
    return getMonomialMoments(nmoms,mm,0,QVMData.MOM_TS,true,0);
}
static double [] getMonomialMomVS(final int nmoms,final QVMData [] mm)
{
    return getMonomialMoments(nmoms,mm,0,QVMData.MOM_VS,false,0);
}
static double [] getMonomialMomRVS(final int nmoms,final QVMData [] mm)
{
    return getMonomialMoments(nmoms,mm,0,QVMData.MOM_RVS,false,0);
}
static double [] getMonomialMomRTS(final int nmoms,final QVMData [] mm)
{
    return getMonomialMoments(nmoms,mm,0,QVMData.MOM_RTS,true,0);
}
static double [] getMonomialMomDS(final int nmoms,final QVMData [] mm)
{
    final double [] mom_sell=getMonomialMoments(nmoms,mm,0,QVMData.MOM_VSELL,false,0),
	mom_buy=getMonomialMoments(nmoms,mm,0,QVMData.MOM_VBUY,false,0);
    TMatr.sumP1(mom_sell,mom_buy,-1);
    return mom_sell;
}

static double [][] getMonomialMoments(final int nmoms,final QVMData [] mm)
{
    final double [] mom_v=getMonomialMoments(nmoms,mm,0,QVMData.MOM_dV,false,0);
    return new double [][]{mom_v,getMonomialMomT(nmoms),getMonomialMomDP(nmoms,mm),TMatr.getVectorScaled(getMonomialMomDS(nmoms,mm),1/mom_v[0]),getMonomialMoments(nmoms,mm,0,QVMData.MOM_VBUY,false,0),getMonomialMoments(nmoms,mm,0,QVMData.MOM_VSELL,false,0)};
}

void test_Moments(){
    final MQVMLegendreShifted q=getMQVM();
    final RadauLQVMLegendreShifted s=RadauLQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);
    UnitTests.checkDoubleRel(q.dp0i,"q.dp0i",s.dpinodes[0]/s.wiinodes[0],"s.dpinodes[0]/s.wiinodes[0]",eps);
    UnitTests.checkDoubleRel(q.i0,"q.i0",s.wiinodes[0],"s.wiinodes[0]",eps);
    UnitTests.checkDoubleRel(q.ds0i,"q.ds0i",s.dsinodes[0]/s.wiinodes[0],"s.dsinodes[0]/s.wiinodes[0]",eps);
    System.err.println(name+":MQVMLegendreShiftedTest:test_Moments(): test OK neq="+neq+" eps="+eps+" class="+q.getClass().getName());
}

} // END of class MQVMLegendreShiftedTest

public static void main(String [] args)
{
    new MQVMLegendreShiftedTest("MQVMLegendreShifted").runTests(5);
} // END main



} // END OF class MQVMLegendreShifted
