package com.polytechnik.trading;
import com.polytechnik.utils.*;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/**
 * The state of max_{over all psi} {@literal <(p-p1)^2 I psi^2>/<(p-p2)^2 psi^2>},
 * with p1 and p2 calculated accorfing some algorithm.
 *
 *  @version $Id: MaxIP2ratio.java,v 1.4 2022/06/17 20:37:51 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class MaxIP2ratio {
    private static final int N_iterations=10;


    public final double p_offset,
	p_init_V,
	p_init_t,
	p_iter_V,
	p_iter_t,
	M_iter,
	MI_iter;
    public final EVXData I;
    public final double [] psi_M;
    public MaxIP2ratio(final int n,final double p_offset,final double p_last,
		       final double [] QQt,final double [] QQi,
		       final double [] QQp0t,final double [] QQp1t,final double [] QQp2t,
		       final double [] QQp0i,final double [] QQp1i,final double [] QQp2i,
		       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		       final double eps)
{
    this.p_offset=p_offset;
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);

    final int mind=n-1;
    // QQp0t & QQt may be slightly different because of sampling.
    this.p_init_V=I.getFPsi(mind,QQp1i)/I.getFPsi(mind,QQp0i);
    this.p_init_t=I.getFPsi(mind,QQp1t)/I.getFPsi(mind,QQp0t);

    double pt=p_init_t,pi=p_init_V,m=I.ev[mind],
	pl_m_poffset=p_last,// p_offset is already included in p_last
	psi[]=I.getPsi(mind);

    final double [] QQpx2I=new double [n*n],QQpx2t=new double [n*n];
    for(int it=0;it<N_iterations;it++)
    {
	final double pnom=pi,pdenom=pl_m_poffset;
	for(int k=QQpx2I.length;--k>=0;) {
	    QQpx2I[k]=QQp2i[k]+pnom*(-2*QQp1i[k]+pnom*QQp0i[k]);
	    QQpx2t[k]=QQp2t[k]+pdenom*(-2*QQp1t[k]+pdenom*QQp0t[k]);
	}
	final EVXData e=EVXData.getEVXDataFromQQ(n,QQpx2I,QQpx2t,M);
	pi=e.getFPsi(mind,QQp1i)/e.getFPsi(mind,QQp0i);
	pt=e.getFPsi(mind,QQp1t)/e.getFPsi(mind,QQp0t);
	m=e.ev[mind];
	psi=e.getPsi(mind);
	if(eps>0)
	{
	    // convergence diag
	    System.err.println("MaxIP2ratio: it="+it+" pi="+pi+" pt="+pt+" e="+e.ev[mind]+" p_init_V="+p_init_V+" p_init_t="+p_init_t+" IH="+I.ev[mind]);
	}
    }
    this.p_iter_V=pi;
    this.p_iter_t=pt;
    this.M_iter=m;
    this.MI_iter=TMatr.getAMatrA(psi,QQp0i,n)/TMatr.getAMatrA(psi,QQp0t,n);
    // save the wavefunction (just in case we need to calculate other moments)
    this.psi_M=psi;

    if(eps>0){
	System.err.println("MaxIP2ratio: test OK n="+n+" eps="+eps);
    }
}
public static void matchMaxIP2ratio(final MaxIP2ratio a,final MaxIP2ratio b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_init_V,"a.p_init_V",b.p_init_V+offset_shift,"b.p_init_V+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_init_t,"a.p_init_t",b.p_init_t+offset_shift,"b.p_init_t+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_iter_V,"a.p_iter_V",b.p_iter_V+offset_shift,"b.p_iter_V+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_iter_t,"a.p_iter_t",b.p_iter_t+offset_shift,"b.p_iter_t+offset_shift",eps);

    UnitTests.checkDoubleRel(a.M_iter,"a.M_iter",b.M_iter,"b.M_iter",eps);
    UnitTests.checkDoubleRel(a.MI_iter,"a.MI_iter",b.MI_iter,"b.MI_iter",eps);

    System.err.println("MaxIP2ratio:matchMaxIP2ratio test OK eps="+eps);
}

}
