package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Skewness;

/**
 *  Find max < I psi^2>/<(p-p_aver_V)^2 psi^2>
 *  @version $Id: MaxPP2VI.java,v 1.3 2020/09/27 20:33:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class MaxPP2VI {
    private static final int N_iterations=10;

    public final double p_offset,
	p_init_i,p_init_V,
	p_iter_i,p_iter_V
	;
    public final EVXData I;
    public final Skewness sk;
public MaxPP2VI(final int n,
		final double p_offset,
		final double [] QQt,final double [] QQi,
		final double [] QQp0t,final double [] QQp1t,final double [] QQp2t,final double [] QQp3t,
		final double [] QQV,final double [] QQpV,
		final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		final double eps)
{
    this.p_offset=p_offset;
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int mind=n-1; // maximal eigenvalue index
    this.p_init_i=I.getFPsi(mind,QQp1t)/I.getFPsi(mind,QQp0t);
    this.p_init_V=I.getFPsi(mind,QQpV)/I.getFPsi(mind,QQV);

    double [] psi=I.getPsi(mind);
    double pi=p_init_i,pV=p_init_V;
    final double [] QQpx2I=new double [n*n],QQpx2t=new double [n*n];
    for(int it=0;it<N_iterations;it++)
    {
	final double px=pV;
	for(int k=QQpx2I.length;--k>=0;) {
	    QQpx2I[k]=(QQi[k]);
	    QQpx2t[k]=QQp2t[k]+px*(-2*QQp1t[k]+px*QQp0t[k]);
	}
	final EVXData e=EVXData.getEVXDataFromQQ(n,QQpx2I,QQpx2t,M);
	pi=e.getFPsi(mind,QQp1t)/e.getFPsi(mind,QQp0t);
	pV=e.getFPsi(mind,QQpV)/e.getFPsi(mind,QQV);
	psi=e.getPsi(mind);
	if(eps>0)
	{
	    // convergence diag
	    System.err.println("MaxPP2VI: it="+it+" px="+px+" e="+e.ev[mind]+" I(e)="+e.getFPsi(mind,QQi));
	}
    }
    this.p_iter_i=pi;
    this.p_iter_V=pV;
    sk=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(psi,QQp0t,n),
	TMatr.getAMatrA(psi,QQp1t,n),
	TMatr.getAMatrA(psi,QQp2t,n),
	TMatr.getAMatrA(psi,QQp3t,n));


    if(eps>0){
	System.err.println("MaxPP2VI: test OK n="+n+" eps="+eps);
    }
}
}
