package com.polytechnik.trading;
import com.polytechnik.utils.*;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/**
 * The state of max_{over all psi} {@literal <I psi^2>/<(p-p_{last})^2  psi^2>}.
 *
 *  @version $Id: MaxPPl2I.java,v 1.21 2022/06/17 20:37:51 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class MaxPPl2I {

    public final double p_offset,p_last,
	pt,pi,pV,V,
	p_root_Mmin,p_root_Mmax,
	MI,
	mp0V,mp1V,mp2V,mp0t,mp1t,mp2t,varp,DIR;
    public final double [] psi_M;
    public MaxPPl2I(final int n,final double p_offset,final double p_last,
		    final double [] QQt,final double [] QQi,
		    final double [] QQp0t,final double [] QQp1t,final double [] QQp2t,
		    final double [] QQp0i,final double [] QQp1i,final double [] QQp2i,
		    final double [] QQV,final double [] QQpV,
		    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		    final double eps)
{
    this.p_offset=p_offset;
    this.p_last=p_last;

    final double pl_m_poffset=p_last;// p_offset is already included in p_last;
    final double [] QQpx2I=new double [n*n],QQpx2t=new double [n*n];
    for(int k=QQpx2I.length;--k>=0;) {
	QQpx2I[k]=(QQi[k]);
	QQpx2t[k]=QQp2t[k]+pl_m_poffset*(-2*QQp1t[k]+pl_m_poffset*QQp0t[k]);
    }
    final int mind=0;
    final EVXData e=EVXData.getEVXDataFromQQ(n,QQpx2t,QQpx2I,M);
    this.psi_M=e.getPsi(mind);
    pi=e.getFPsi(mind,QQp1i)/e.getFPsi(mind,QQp0i);
    pt=e.getFPsi(mind,QQp1t)/e.getFPsi(mind,QQp0t);
    pV=e.getFPsi(mind,QQpV)/e.getFPsi(mind,QQV);
    V=e.getFPsi(mind,QQV)/e.getFPsi(mind,QQt);
    MI=e.ev[mind];

    // Price moments of the measure psi^2 dV
    mp0V=TMatr.getAMatrA(psi_M,QQp0i,n);
    mp1V=TMatr.getAMatrA(psi_M,QQp1i,n); // =pi*mp0V
    mp2V=TMatr.getAMatrA(psi_M,QQp2i,n);
    // Price moments of the measure psi^2 dt
    mp0t=TMatr.getAMatrA(psi_M,QQp0t,n);
    mp1t=TMatr.getAMatrA(psi_M,QQp1t,n); // =pt*mp0t
    mp2t=TMatr.getAMatrA(psi_M,QQp2t,n);

    varp=(mp1V-pl_m_poffset*mp0V)-MI*(mp1t-pl_m_poffset*mp0t);

    // p_iter_V-p_iter_t scale determination, d=2, find min/max of <(p-px)^2>_V/<(p-px)^2>_t
    final double [] evS=M.EV.getGEV_Generalized(
	2,new double []{mp0V,mp1V,mp1V,mp2V},
	  new double []{mp0t,mp1t,mp1t,mp2t},true),
	phiL=EVXData.getPsiFromBasis(0,evS,2),
	phiH=EVXData.getPsiFromBasis(1,evS,2);
    p_root_Mmin=-phiL[0]/phiL[1];
    p_root_Mmax=-phiH[0]/phiH[1];
    // the abs(p_root_min-p_root_max) is kind of a scale for pv-pt
    DIR=(pi-pt)/Math.abs(p_root_Mmin-p_root_Mmax);

    if(eps>0){
	System.err.println("MaxPnLratio: test OK n="+n+" eps="+eps);
    }


}
public static void matchMaxPPl2I(final MaxPPl2I a,final MaxPPl2I b,final double eps)
{
    //EVXData.matchEVXData(a.I,b.I,false,eps);

    final double offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.pi,"a.pi",b.pi+offset_shift,"b.pi+offset_shift",eps);
    UnitTests.checkDoubleRel(a.pV,"a.pV",b.pV+offset_shift,"b.pV+offset_shift",eps);
    UnitTests.checkDoubleRel(a.pt,"a.pt",b.pt+offset_shift,"b.pt+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_root_Mmin,"a.p_root_Mmin",b.p_root_Mmin+offset_shift,"b.p_root_Mmin+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_root_Mmax,"a.p_root_Mmax",b.p_root_Mmax+offset_shift,"b.p_root_Mmax+offset_shift",eps);
    UnitTests.checkDoubleRel(a.MI,"a.MI",b.MI,"b.MI",eps);
    UnitTests.checkDoubleRel(a.V,"a.V",b.V,"b.V",eps);
    UnitTests.checkDoubleRel(a.varp,"a.varp",b.varp,"b.varp",eps);
    UnitTests.checkDoubleRel(a.DIR,"a.DIR",b.DIR,"b.DIR",eps);

    System.err.println("MaxPPl2I:matchMaxPPl2I test OK eps="+eps);
}

}
