package com.polytechnik.trading;
import com.polytechnik.utils.*;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/**
 * The state of max_{over all psi} {@literal <I psi^2>/<(p-p_{last})^2  psi^2>}.
 * Almost the same as the MaxPPl2I but the calculations in the basis of |I| operator to increase numerical stability.
 * Also see DIselP2divI.java for similar calculations.
 *
 *  @version $Id: MaxPPl2Iinbasis.java,v 1.21 2022/08/08 20:10:47 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class MaxPPl2Iinbasis {

    public final double p_offset,p_last,
	p_IH,pV_IH,MI,pi,pt,pV,pT,pj,pVj,pVt,pTj,V,Vj,T,Tj
	;
    public final double [] psi_M;
    public final EVXData I;
    public MaxPPl2Iinbasis(final int n,final double p_offset,final double p_last,
			   final double [] QQt,final double [] QQi,
			   final double [] QQpi,final double [] QQp2i,
			   final double [] QQV,final double [] QQpV,
			   final double [] QQT,final double [] QQpT,
			   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			   final double eps)
{
    this.p_offset=p_offset;
    this.p_last=p_last;
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int ixH=n-1; // maximal eigenvalue index
    this.p_IH=I.getFPsi(ixH,QQpi)/I.getFPsi(ixH,QQi);
    this.pV_IH=I.getFPsi(ixH,QQpV)/I.getFPsi(ixH,QQV);

    final int mind=0; // solve (p-pl)^2|psi>=lambda |I|psi>
    final double [] bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),bQQppli=bQQpi.clone(),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,QQp2i,I.ev),bQQppl2i=bQQp2i.clone(),
	bQQV=EVXData.convertMatrFromQtoB_Basis(n,QQV,I.ev),
	bQQpV=EVXData.convertMatrFromQtoB_Basis(n,QQpV,I.ev),
	bQQT=EVXData.convertMatrFromQtoB_Basis(n,QQT,I.ev),
	bQQpT=EVXData.convertMatrFromQtoB_Basis(n,QQpT,I.ev);
    final double pl_m_poffset=p_last;// p_offset is already included in p_last;
    com.polytechnik.utils.Polynomials.shiftPOffsetInPlace(n*n,bQQi,bQQppli,bQQppl2i,null,pl_m_poffset);
    final double [] invbQQi=new double [n*n],invSqrtbQQi=new double [n*n];
    for(int k=0;k<n;k++){
	invbQQi[k+n*k]=1/I.ev[k];
	invSqrtbQQi[k+n*k]=1/Math.sqrt(I.ev[k]);
    }
    final double [] ev_bQQppl2divI=M.EV.getEV(n,TMatr.mulMMM(n,invbQQi,bQQppl2i,invbQQi),true),
	bpsi_M=EVXData.getPsiFromBasis(mind,ev_bQQppl2divI,n);
    if(eps>0){
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi_M,bpsi_M),"<bpsi_M|bpsi_M>",1,"1",eps);
    }
    this.pi=TMatr.getAMatrA(bpsi_M,bQQpi,n)/TMatr.getAMatrA(bpsi_M,bQQi,n);
    this.pt=TMatr.getAMatrA(bpsi_M,TMatr.mulMMM(n,invbQQi,bQQpi,invbQQi),n)/TMatr.getAMatrA(bpsi_M,invbQQi,n);
    this.pj=TMatr.getAMatrA(bpsi_M,TMatr.mulMMM(n,invSqrtbQQi,bQQpi,invSqrtbQQi),n);

    this.pV=TMatr.getAMatrA(bpsi_M,bQQpV,n)/TMatr.getAMatrA(bpsi_M,bQQV,n);
    this.pVj=TMatr.getAMatrA(bpsi_M,TMatr.mulMMM(n,invSqrtbQQi,bQQpV,invSqrtbQQi),n)/TMatr.getAMatrA(bpsi_M,TMatr.mulMMM(n,invSqrtbQQi,bQQV,invSqrtbQQi),n);
    this.pVt=TMatr.getAMatrA(bpsi_M,TMatr.mulMMM(n,invbQQi,bQQpV,invbQQi),n)/TMatr.getAMatrA(bpsi_M,TMatr.mulMMM(n,invbQQi,bQQV,invbQQi),n);

    this.pT=TMatr.getAMatrA(bpsi_M,bQQpT,n)/TMatr.getAMatrA(bpsi_M,bQQT,n);
    this.pTj=TMatr.getAMatrA(bpsi_M,TMatr.mulMMM(n,invSqrtbQQi,bQQpT,invSqrtbQQi),n)/TMatr.getAMatrA(bpsi_M,TMatr.mulMMM(n,invSqrtbQQi,bQQT,invSqrtbQQi),n);


    this.V=TMatr.getAMatrA(bpsi_M,bQQV,n);
    this.Vj=TMatr.getAMatrA(bpsi_M,TMatr.mulMMM(n,invSqrtbQQi,bQQV,invSqrtbQQi),n)/TMatr.getAMatrA(bpsi_M,invbQQi,n);
    this.T=TMatr.getAMatrA(bpsi_M,bQQT,n);
    this.Tj=TMatr.getAMatrA(bpsi_M,TMatr.mulMMM(n,invSqrtbQQi,bQQT,invSqrtbQQi),n)/TMatr.getAMatrA(bpsi_M,invbQQi,n);

    this.MI=ev_bQQppl2divI[mind];
    this.psi_M=EVXData.convertPsiFromBtoQ_Basis(n,bpsi_M,I.ev);

    if(eps>0){
	final MaxPPl2I d=new MaxPPl2I(
	    n,p_offset,p_last,
	    QQt,QQi,
	    EVXData.convertMatrFromBtoQ_Basis(n,TMatr.mulMMM(n,invSqrtbQQi,bQQi,invSqrtbQQi),I.ev,QQt),
	    EVXData.convertMatrFromBtoQ_Basis(n,TMatr.mulMMM(n,invSqrtbQQi,bQQpi,invSqrtbQQi),I.ev,QQt),
	    EVXData.convertMatrFromBtoQ_Basis(n,TMatr.mulMMM(n,invSqrtbQQi,bQQp2i,invSqrtbQQi),I.ev,QQt),
	    QQi,QQpi,QQp2i,
	    QQV,QQpV,M,eps);
	UnitTests.checkDoubleRel(MI,"MI",d.MI,"d.MI",eps);
	UnitTests.checkDoubleRel(pt,"pt",d.pt,"d.pt",eps);
	UnitTests.checkDoubleRel(pj,"pj",d.pi,"d.pi",eps);
	UnitTests.checkDoubleRel(pVj,"pVj",d.pV,"d.pV",eps);
	UnitTests.checkDoubleRel(Vj,"Vj",d.V,"d.p",eps);
    }
    if(eps>0){
	final DIselP2divI d=new DIselP2divI(
	    n,p_offset,p_last,
	    QQt,QQi,QQpi,QQp2i,
	    QQV,QQpV,M,null,eps);
	UnitTests.checkDoubleRel(pi,"pi",d.pi,"d.pi",eps);
	UnitTests.checkDoubleRel(pV,"pV",d.pV,"d.pV",eps);
	UnitTests.checkDoubleRel(V,"V",d.V,"d.V",eps);
    }


    if(eps>0){
	System.err.println("MaxPPl2Iinbasis: test OK n="+n+" eps="+eps);
    }


}
public static void matchMaxPPl2Iinbasis(final MaxPPl2Iinbasis a,final MaxPPl2Iinbasis b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+offset_shift,"b.p_IH+offset_shift",eps);
    UnitTests.checkDoubleRel(a.pV_IH,"a.pV_IH",b.pV_IH+offset_shift,"b.pV_IH+offset_shift",eps);
    UnitTests.checkDoubleRel(a.pi,"a.pi",b.pi+offset_shift,"b.pi+offset_shift",eps);
    UnitTests.checkDoubleRel(a.pt,"a.pt",b.pt+offset_shift,"b.pt+offset_shift",eps);
    UnitTests.checkDoubleRel(a.pV,"a.pV",b.pV+offset_shift,"b.pV+offset_shift",eps);
    UnitTests.checkDoubleRel(a.pT,"a.pT",b.pT+offset_shift,"b.pT+offset_shift",eps);
    UnitTests.checkDoubleRel(a.V,"a.V",b.V,"b.V",eps);
    UnitTests.checkDoubleRel(a.T,"a.T",b.T,"b.T",eps);
    UnitTests.checkDoubleRel(a.pj,"a.pj",b.pj+offset_shift,"b.pj+offset_shift",eps);
    UnitTests.checkDoubleRel(a.pVj,"a.pVj",b.pVj+offset_shift,"b.pVj+offset_shift",eps);
    UnitTests.checkDoubleRel(a.pTj,"a.pTj",b.pTj+offset_shift,"b.pTj+offset_shift",eps);
    UnitTests.checkDoubleRel(a.Vj,"a.Vj",b.Vj,"b.Vj",eps);
    UnitTests.checkDoubleRel(a.Tj,"a.Tj",b.Tj,"b.Tj",eps);

    System.err.println("MaxPPl2Iinbasis:matchMaxPPl2Iinbasis test OK eps="+eps);
}

}
