package com.polytechnik.trading;
import com.polytechnik.utils.*;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/**
 * The state of max_{over all psi} {@literal <(p-p_aver_t)^2 I psi^2>/<(p-p_aver_i)^2 psi^2>},
 * with p_aver_i={@literal <p I psi^2>/<I psi^2>}, and p_aver_t={@literal <p psi^2>/<psi^2>},
 * (note: the p_aver_i and p_aver_t are swapped in nominator/denominator).
 *
 *  @version $Id: MaxPnLratio.java,v 1.54 2022/06/17 20:37:51 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class MaxPnLratio {
    private static final int N_iterations=10,MIN_ratio=160,MAX_ratio=161;
    // The parameters for experiments
    private final int flag_relation=MAX_ratio;
    private final boolean flag_swap_PtPv=true;


    public final double p_offset,
	p_init_V,
	p_init_t,
	p_iter_V,
	p_iter_t,
	p_root_Mmin,p_root_Mmax,
	M_iter,
	MI_iter,
	mp0V,mp1V,mp2V,mp0t,mp1t,mp2t,varp,DIR;
    public final EVXData I;
    public final double [] psi_M;
    public MaxPnLratio(final int n,final double p_offset,
		       final double [] QQt,final double [] QQi,
		       final double [] QQp0t,final double [] QQp1t,final double [] QQp2t,
		       final double [] QQp0i,final double [] QQp1i,final double [] QQp2i,
		       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		       final double eps)
{
    this.p_offset=p_offset;
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);

    final int mind;
    if(flag_relation==MIN_ratio) mind=0;
    else if(flag_relation==MAX_ratio) mind=n-1;
    else throw new IllegalArgumentException("flag_relation="+flag_relation);
    // QQp0t & QQt may be slightly different because of sampling.
    this.p_init_V=I.getFPsi(mind,QQp1i)/I.getFPsi(mind,QQp0i);
    this.p_init_t=I.getFPsi(mind,QQp1t)/I.getFPsi(mind,QQp0t);

    double pt=p_init_t,pi=p_init_V,m=I.ev[mind],psi[]=I.getPsi(mind);

    final double [] QQpx2I=new double [n*n],QQpx2t=new double [n*n];
    for(int it=0;it<N_iterations;it++)
    {
	if(flag_swap_PtPv){ // swap pt <-> pi in matrix
	    for(int k=QQpx2I.length;--k>=0;) {
		QQpx2I[k]=QQp2i[k]+pt*(-2*QQp1i[k]+pt*QQp0i[k]);
		QQpx2t[k]=QQp2t[k]+pi*(-2*QQp1t[k]+pi*QQp0t[k]);
	    }
	} else { // do not swap pt <-> pi in matrix
	    for(int k=QQpx2I.length;--k>=0;) {
		QQpx2I[k]=QQp2i[k]+pi*(-2*QQp1i[k]+pi*QQp0i[k]);
		QQpx2t[k]=QQp2t[k]+pt*(-2*QQp1t[k]+pt*QQp0t[k]);
	    }
	}
	if(eps>0){
	    final double dpa2=Math.pow(flag_swap_PtPv?(pt-pi):0,2),
		stdev2i=TMatr.getAMatrA(psi,QQp2i,n)/TMatr.getAMatrA(psi,QQp0i,n)-pi*pi,
		stdev2t=TMatr.getAMatrA(psi,QQp2t,n)/TMatr.getAMatrA(psi,QQp0t,n)-pt*pt;
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,QQpx2I,n)/TMatr.getAMatrA(psi,QQp0i,n),"QQpx2I",stdev2i+dpa2,"stdev2i+dpa2",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,QQpx2t,n)/TMatr.getAMatrA(psi,QQp0t,n),"QQpx2t",stdev2t+dpa2,"stdev2t+dpa2",eps);
	}
	final EVXData e=EVXData.getEVXDataFromQQ(n,QQpx2I,QQpx2t,M);
	pi=e.getFPsi(mind,QQp1i)/e.getFPsi(mind,QQp0i);
	pt=e.getFPsi(mind,QQp1t)/e.getFPsi(mind,QQp0t);
	m=e.ev[mind];
	psi=e.getPsi(mind);
	if(eps>0)
	{
	    // convergence diag
	    System.err.println("MaxPnLratio: it="+it+" pi="+pi+" pt="+pt+" e="+e.ev[mind]+" p_init_V="+p_init_V+" p_init_t="+p_init_t+" IH="+I.ev[mind]+" flag_swap_PtPv="+flag_swap_PtPv);
	}
    }
    if(false) if(Double.isNaN(pt)) throw new RuntimeException();
    this.p_iter_V=pi;
    this.p_iter_t=pt;
    this.M_iter=m;
    this.MI_iter=TMatr.getAMatrA(psi,QQp0i,n)/TMatr.getAMatrA(psi,QQp0t,n);
    // save the wavefunction (just in case we need to calculate other moments)
    this.psi_M=psi;
    // Price moments of the measure psi^2 dV
    mp0V=TMatr.getAMatrA(psi,QQp0i,n);
    mp1V=TMatr.getAMatrA(psi,QQp1i,n); // =p_iter_i*mp0V
    mp2V=TMatr.getAMatrA(psi,QQp2i,n);
    // Price moments of the measure psi^2 dt
    mp0t=TMatr.getAMatrA(psi,QQp0t,n);
    mp1t=TMatr.getAMatrA(psi,QQp1t,n); // =p_iter_t*mp0t
    mp2t=TMatr.getAMatrA(psi,QQp2t,n);

    varp=flag_swap_PtPv?(mp1V-pt*mp0V)-m*(mp1t-pi*mp0t):(mp1V-pi*mp0V)-m*(mp1t-pt*mp0t);

    // p_iter_V-p_iter_t scale determination, d=2, find min/max of <(p-px)^2>_V/<(p-px)^2>_t
    final double [] evS=M.EV.getGEV_Generalized(
	2,new double []{mp0V,mp1V,mp1V,mp2V},
	  new double []{mp0t,mp1t,mp1t,mp2t},true),
	phiL=EVXData.getPsiFromBasis(0,evS,2),
	phiH=EVXData.getPsiFromBasis(1,evS,2);
    p_root_Mmin=-phiL[0]/phiL[1];
    p_root_Mmax=-phiH[0]/phiH[1];
    // the abs(p_root_min-p_root_max) is kind of a scale for p_iter_V-p_iter_t
    DIR=(p_iter_V-p_iter_t)/Math.abs(p_root_Mmin-p_root_Mmax);

    if(eps>0){
	System.err.println("MaxPnLratio: test OK n="+n+" eps="+eps);
    }
}
public static void matchMaxPnLratio(final MaxPnLratio a,final MaxPnLratio b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_init_V,"a.p_init_V",b.p_init_V+offset_shift,"b.p_init_V+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_init_t,"a.p_init_t",b.p_init_t+offset_shift,"b.p_init_t+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_iter_V,"a.p_iter_V",b.p_iter_V+offset_shift,"b.p_iter_V+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_iter_t,"a.p_iter_t",b.p_iter_t+offset_shift,"b.p_iter_t+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_root_Mmin,"a.p_root_Mmin",b.p_root_Mmin+offset_shift,"b.p_root_Mmin+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_root_Mmax,"a.p_root_Mmax",b.p_root_Mmax+offset_shift,"b.p_root_Mmax+offset_shift",eps);
    UnitTests.checkDoubleRel(a.M_iter,"a.M_iter",b.M_iter,"b.M_iter",eps);
    UnitTests.checkDoubleRel(a.MI_iter,"a.MI_iter",b.MI_iter,"b.MI_iter",eps);
    UnitTests.checkDoubleRel(a.varp,"a.varp",b.varp,"b.varp",eps);
    UnitTests.checkDoubleRel(a.DIR,"a.DIR",b.DIR,"b.DIR",eps);

    System.err.println("MaxPnLratio:matchMaxPnLratio test OK eps="+eps);
}

}
