package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** Try equilibrium price but for micro-impact.
 * The MicroPnLdI.Pf_from_pt_true_pi and MicroPnLdI.Pf_from_f0
 * are for price calculation, here for micro-impact.
 */
public class MicroPnLdI {

public final EVXData I;
public final double p_offset,I_0_feq,dI,
    g_0,g_IH,deg_from_pt_true_pi,gf_from_pt_true_pi,deg_from_dyn,gf_from_f0;
MicroPnLdI(final EVXData I,final double p_offset,final double I_0_feq,
	   final double gI_IH,final double gI_0,
	   final double [] psik_psi0,final double [] psik_I_psik,
	   final double [] psik_gI_psik,final double [] bbasis,
	   final double eps)
{
    this.I=I;
    this.p_offset=p_offset;
    this.I_0_feq=I_0_feq;
    this.dI=I_0_feq-I.s0;
    this.g_IH=gI_IH/I.sH;
    this.g_0=gI_0/I.s0;
    final int n=I.neq;
    final double [] w0k=TMatr.squareArrayElements(psik_psi0),wIHIk=new double [n];
    for(int k=0;k<n;k++){wIHIk[k]=I_0_feq/bbasis[k];}
    this.deg_from_pt_true_pi=1-TMatr.mulV2V(w0k,wIHIk);
    this.gf_from_pt_true_pi=(TMatr.mulVVV(w0k,wIHIk,psik_gI_psik)-gI_0)/(dI*deg_from_pt_true_pi);
    this.deg_from_dyn=1-TMatr.mulVV(w0k,w0k);
    this.gf_from_f0=(TMatr.mulVV(w0k,psik_gI_psik)-gI_0)/(dI*deg_from_dyn);
}

public static final int G_PRICE=413,G_DPRICE=414,G_DPRICEDT=415;
public static MicroPnLdI get_MicroPnLdI(final MomentsData mm,
					final int whatG,
					final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
					final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final int ixH=n-1; // the state with maximal I
    final double I_0=I.s0,I_0_feq=I.ev[ixH],dI=I_0_feq-I_0;

    // Create bQQi_feq (operator of "future" I), with the |bpsi0>dI<bpsi0| contribution from the future:
    // bQQi_feq = bQQi + |bpsi0>dI<bpsi0|
    final double [] bQQi_feq=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o)),bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));
    for(int k=0;k<n;k++) bQQi_feq[k+n*k]+=I.ev[k];
    final double [] QQgi;
    if(whatG==G_PRICE) QQgi=mm.QQpi;
    else if(whatG==G_DPRICE) QQgi=mm.QQDpwv;
    else if(whatG==G_DPRICEDT) QQgi=mm.QQdp;
    else throw new IllegalArgumentException("Unknown whatG="+whatG);

    final double [] bQQgi=EVXData.convertMatrFromQtoB_Basis(n,QQgi,I.ev);

    // Diagonalize "future" I:
    final double [] bbasis=M.EV.getEV(n,bQQi_feq,true),
	psik_psi0=new double [n],psik_I_psik=new double[n],psik_gI_psik=new double[n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n);

	psik_psi0[k]=TMatr.mulVV(psik,bpsi0);
	psik_I_psik[k]=TMatr.mulV2V(psik,I.getEValues());
	psik_gI_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQgi,n);
    }
    if(eps>0)
    {
	EVXData.test_orthogonality("MicroPnLdI.get_MicroPnLdI:I.ev",n,I.ev,mm.QQi,mm.QQt,eps); // old basis
	EVXData.test_orthogonality("MicroPnLdI.get_MicroPnLdI:bbasis",n,bbasis,bQQi_feq,TMatr.getUnitMatrix(n),eps); // new basis
	// check some matrix elements
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(psik_psi0,java.util.Arrays.copyOf(bbasis,n)),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi0,bpsi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(bpsi0,I.getEValues()),"<psi0|I|psi0>",I.s0,"I.s0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_feq,n),"<psi0|I^f|psi0>",I_0_feq,"I_0_feq",eps);
	// check spur invariance
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues()),"sum(I.ev)",TMatr.sumElements(psik_I_psik),"sum(psik_I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues())+dI,"sum(I.ev)+dI",TMatr.sumElements(java.util.Arrays.copyOf(bbasis,n)),"sum(I^{f})",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQgi),"Spur(bQQgi)",TMatr.sumElements(psik_gI_psik),"sum(psik_gI_psik)",eps);
	System.err.println("MicroPnLdI.get_MicroPnLdI selftest OK n="+n+" eps="+eps);
    }


    return new MicroPnLdI(
	I,mm.p_offset,I_0_feq,
	bQQgi[ixH+n*ixH],TMatr.getAMatrA(bpsi0,bQQgi,n),
	psik_psi0,psik_I_psik,psik_gI_psik,bbasis,eps);
}

}
