package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;

/**
 * Find min < (p-p_{last})*(p-p_{aver_V}) *I*psi^2>/<psi^2>,
 * the min is negative on success.
 *  @version $Id: MinIPlPV.java,v 1.12 2022/11/20 12:41:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class MinIPlPV {
    private static final int N_iterations=10;

    public final double p_offset,p_last,
	p_init_i,p_init_V,
	p_iter_i,p_iter_V,MI,IM,p2neg
	;
    public final double [] psi_M;
    public final EVXData I;
public MinIPlPV(final int n,
		final double p_offset,final double p_last,
		final double [] QQt,final double [] QQi,final double [] QQpi,final double [] QQp2i,
		final double [] QQV,final double [] QQpV,
		final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		final double eps)
{
    this.p_offset=p_offset;
    this.p_last=p_last;
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final int ixH=n-1; // maximal eigenvalue index
    this.p_init_i=I.getFPsi(ixH,QQpi)/I.getFPsi(ixH,QQi);
    this.p_init_V=I.getFPsi(ixH,QQpV)/I.getFPsi(ixH,QQV);

    double [] bpsi_M=new double [n];bpsi_M[ixH]=1;
    double pi=p_init_i,pV=p_init_V,m=I.sH;
    final double [] bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,QQp2i,I.ev),
	bQQV=EVXData.convertMatrFromQtoB_Basis(n,QQV,I.ev),
	bQQpV=EVXData.convertMatrFromQtoB_Basis(n,QQpV,I.ev),
	bQQpx2I=new double [n*n];
    for(int it=0,mind=0/* min is negative */;it<N_iterations;it++)
    {
	final double pl=p_last,px=pV;
	for(int k=bQQpx2I.length;--k>=0;) {
	    bQQpx2I[k]=bQQp2i[k]-(pl+px)*bQQpi[k]+(pl*px)*bQQi[k];
	}
	final double [] ev_bQQpx2I=M.EV.getEV(n,bQQpx2I,true);
	bpsi_M=EVXData.getPsiFromBasis(mind,ev_bQQpx2I,n);
	pi=TMatr.getAMatrA(bpsi_M,bQQpi,n)/TMatr.getAMatrA(bpsi_M,bQQi,n);
	pV=TMatr.getAMatrA(bpsi_M,bQQpV,n)/TMatr.getAMatrA(bpsi_M,bQQV,n);
	m=ev_bQQpx2I[mind]; // must be negative on success
	if(eps>0){
	    final double [] QQpx2I=new double [n*n];
	    for(int k=QQpx2I.length;--k>=0;) {QQpx2I[k]=QQp2i[k]-(pl+px)*QQpi[k]+(pl*px)*QQi[k];}
	    final EVXData e=EVXData.getEVXDataFromQQ(n,QQpx2I,QQt,M);
	    final double _pi=e.getFPsi(mind,QQpi)/e.getFPsi(mind,QQi),
		_pV=e.getFPsi(mind,QQpV)/e.getFPsi(mind,QQV);
	    UnitTests.checkDoubleRel(pi,"pi",_pi,"_pi",eps);
	    UnitTests.checkDoubleRel(pV,"pV",_pV,"_pV",eps);
	    UnitTests.checkDoubleArrayRel(EVXData.getMSign(EVXData.convertPsiFromBtoQ_Basis(n,bpsi_M,I.ev)),"bpsi_M",EVXData.getMSign(e.getPsi(mind)),"e.psi",eps);
	}
	if(eps>0)
	{
	    // convergence diag
	    System.err.println("MinIPlPV: it="+it+" p_init_i="+p_init_i+" pi="+pi+" pV="+pV+" m="+m);
	}
    }
    this.p_iter_i=pi;
    this.p_iter_V=pV;
    this.MI=m;
    this.IM=TMatr.getAMatrA(bpsi_M,bQQi,n);
    // must be negative on success
    this.p2neg=MI/IM;
    this.psi_M=EVXData.convertPsiFromBtoQ_Basis(n,bpsi_M,I.ev);

    if(eps>0){
	System.err.println("MinIPlPV: test OK n="+n+" eps="+eps);
    }
}
}
