package com.polytechnik.trading;
import com.polytechnik.utils.*;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/**
 * The state of min/max {over all psi} {@literal <(p-p_aver_t)^2 I psi^2>/<psi^2>},
 * with p_aver_i={@literal <p I psi^2>/<I psi^2>}, and p_aver_t={@literal <p psi^2>/<psi^2>}.
 *
 *  @version $Id: MinMaxPnLratioNorm.java,v 1.11 2022/11/20 12:41:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class MinMaxPnLratioNorm {
    private static final int N_iterations=10;

    public final double p_offset,
	p_init_V,
	p_init_t,
	p_iter_V,
	p_iter_t,
	p_root_Mmin,p_root_Mmax,
	M_iter,
	varpvart,
	mp0V,mp1V,mp2V,mp0t,mp1t,mp2t,DIR;
    public final EVXData I;
    public final double [] psi_M;
    public static final int MIN_ratio=60,MAX_ratio=61;
    public MinMaxPnLratioNorm(
	final int n,final int flag_relation,
	final double p_offset,
	final double [] QQt,final double [] QQi,
	final double [] QQp0t,final double [] QQp1t,final double [] QQp2t,
	final double [] QQp0i,final double [] QQp1i,final double [] QQp2i,
	final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	final VolMatrix VOLATILITY_EDPsi, // if null, varpvart will not be calculated
	final double eps)
{
    this.p_offset=p_offset;
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);

    final double [] bQQp0t=EVXData.convertMatrFromQtoB_Basis(n,QQp0t,I.ev),
	bQQp1t=EVXData.convertMatrFromQtoB_Basis(n,QQp1t,I.ev),
	bQQp2t=EVXData.convertMatrFromQtoB_Basis(n,QQp2t,I.ev),
	bQQp0i=EVXData.convertMatrFromQtoB_Basis(n,QQp0i,I.ev),
	bQQp1i=EVXData.convertMatrFromQtoB_Basis(n,QQp1i,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,QQp2i,I.ev),
	bQQpx2I=new double [n*n];
    final int mind;
    if(flag_relation==MIN_ratio) mind=0;
    else if(flag_relation==MAX_ratio) mind=n-1;
    else throw new IllegalArgumentException("flag_relation="+flag_relation);
    double [] bpsi=new double [n];bpsi[mind]=1;
    // QQp0t & QQt may be slightly different because of sampling.
    this.p_init_V=TMatr.getAMatrA(bpsi,bQQp1i,n)/TMatr.getAMatrA(bpsi,bQQp0i,n);
    this.p_init_t=TMatr.getAMatrA(bpsi,bQQp1t,n)/TMatr.getAMatrA(bpsi,bQQp0t,n);
    double pt=p_init_t,pi=p_init_V,m=I.ev[mind];
    for(int it=0;it<N_iterations;it++)
    {
	if(false && eps>0)
	{
	    System.err.println("init MinMaxPnLratioNorm: it="+it+" pi="+pi+" pt="+pt+" p_init_V="+p_init_V+" p_init_t="+p_init_t);
	}
	for(int k=bQQpx2I.length;--k>=0;) {
	    bQQpx2I[k]=bQQp2i[k]+pt*(-2*bQQp1i[k]+pt*bQQp0i[k]);
	}
	final double [] ev=M.EV.getEV(n,bQQpx2I,true);
	bpsi=EVXData.getPsiFromBasis(mind,ev,n);
	pt=TMatr.getAMatrA(bpsi,bQQp1t,n)/TMatr.getAMatrA(bpsi,bQQp0t,n);
	pi=TMatr.getAMatrA(bpsi,bQQp1i,n)/TMatr.getAMatrA(bpsi,bQQp0i,n);
	m=ev[mind];
	if(eps>0)
	{
	    // convergence diag
	    System.err.println("MinMaxPnLratioNorm: it="+it+" pi="+pi+" pt="+pt+" e="+ev[mind]+" p_init_V="+p_init_V+" p_init_t="+p_init_t);
	}
    }
    this.p_iter_V=pi;
    this.p_iter_t=pt;
    this.M_iter=m;
    // save the wavefunction (just in case we need to calculate other moments)
    this.psi_M=EVXData.convertPsiFromBtoQ_Basis(n,bpsi,I.ev);
    // Price moments of the measure psi^2 dV
    mp0V=TMatr.getAMatrA(bpsi,bQQp0i,n);
    mp1V=TMatr.getAMatrA(bpsi,bQQp1i,n); // =p_iter_i*mp0V
    mp2V=TMatr.getAMatrA(bpsi,bQQp2i,n);
    // Price moments of the measure psi^2 dt
    mp0t=TMatr.getAMatrA(bpsi,bQQp0t,n);
    mp1t=TMatr.getAMatrA(bpsi,bQQp1t,n); // =p_iter_t*mp0t
    mp2t=TMatr.getAMatrA(bpsi,bQQp2t,n);

    // p_iter_V-p_iter_t scale determination, d=2, find min/max of <(p-px)^2>_V/<(p-px)^2>_t
    final double [] evS=M.EV.getGEV_Generalized(
	2,new double []{mp0V,mp1V,mp1V,mp2V},
	  new double []{mp0t,mp1t,mp1t,mp2t},true),
	phiL=EVXData.getPsiFromBasis(0,evS,2),
	phiH=EVXData.getPsiFromBasis(1,evS,2);
    p_root_Mmin=-phiL[0]/phiL[1];
    p_root_Mmax=-phiH[0]/phiH[1];
    // the abs(p_root_min-p_root_max) is kind of a scale for p_iter_V-p_iter_t
    DIR=(p_iter_V-p_iter_t)/Math.abs(p_root_Mmin-p_root_Mmax);
    if(VOLATILITY_EDPsi==null){
	// Infinitesimal time--shift is not available
	this.varpvart=Double.NaN;
    } else {
	final double [] bQQpxI=TMatr.sumAcB(bQQp1i,-pt,bQQp0i),
	    bDpsi=EVXData.convertPsiFromQtoB_Basis(n,VOLATILITY_EDPsi.getDPsi(EVXData.convertPsiFromBtoQ_Basis(n,bpsi,I.ev)),I.ev,QQt);
	this.varpvart=2*(TMatr.getAMatrB(bpsi,bDpsi,bQQpxI,n)-TMatr.getAMatrA(bpsi,bQQpxI,n)*TMatr.mulVV(bpsi,bDpsi));
    }
    if(eps>0)
    {
	final MaxPtPv2I d=new MaxPtPv2I(n,flag_relation,p_offset,
					QQt,QQi,
					QQp0t,QQp1t,QQp2t,
					QQp0i,QQp1i,QQp2i,
					M,VOLATILITY_EDPsi,eps);
	UnitTests.checkDoubleRel(p_iter_V,"p_iter_V",d.p_iter_V,"d.p_iter_V",eps);
	UnitTests.checkDoubleRel(p_iter_t,"p_iter_t",d.p_iter_t,"d.p_iter_t",eps);
	UnitTests.checkDoubleRel(M_iter,"M_iter",d.M_iter,"d.M_iter",eps);
	UnitTests.checkDoubleRel(DIR,"DIR",d.DIR,"d.DIR",eps);
	UnitTests.checkDoubleRel(varpvart,"varpvart",d.varpvart,"d.varpvart",eps);
	UnitTests.checkDoubleArrayRel1(EVXData.getMSign(psi_M),"psi_M",EVXData.getMSign(d.psi_M),"d.psi_M",eps);
	System.err.println("MinMaxPnLratioNorm: test OK n="+n+" eps="+eps);
    }
}

public static void matchMinMaxPnLratioNorm(final MinMaxPnLratioNorm a,final MinMaxPnLratioNorm b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_init_V,"a.p_init_V",b.p_init_V+offset_shift,"b.p_init_V+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_init_t,"a.p_init_t",b.p_init_t+offset_shift,"b.p_init_t+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_iter_V,"a.p_iter_V",b.p_iter_V+offset_shift,"b.p_iter_V+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_iter_t,"a.p_iter_t",b.p_iter_t+offset_shift,"b.p_iter_t+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_root_Mmin,"a.p_root_Mmin",b.p_root_Mmin+offset_shift,"b.p_root_Mmin+offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_root_Mmax,"a.p_root_Mmax",b.p_root_Mmax+offset_shift,"b.p_root_Mmax+offset_shift",eps);
    UnitTests.checkDoubleRel(a.M_iter,"a.M_iter",b.M_iter,"b.M_iter",eps);
    UnitTests.checkDoubleRel(a.varpvart,"a.varpvart",b.varpvart,"b.varpvart",eps);
    UnitTests.checkDoubleRel(a.DIR,"a.DIR",b.DIR,"b.DIR",eps);

    System.err.println("MinMaxPnLratioNorm:matchMinMaxPnLratioNorm test OK eps="+eps);
}

}
