package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.ReadWriteUtils;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.realtimebook.*;

/** Test whether the John Pierpont Morgan was right.
 *  @version $Id: MkQVM.java,v 1.249 2022/11/20 12:41:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class MkQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;

final EVXData I,P1t,Ip;
final ACorr ccorr_pvs_dI,xxC,xxCp,xxCd,xxCpd;
final MDistance distance,distanceP1t;
final double d1G,d2G,varPi,d1G_IL,d1G_IH;
final double [] sens_P1t_distancesorted;
final PertPsi peVS,peVB,pePLUS,peMINUS,PePLUS,PeMINUS,Pe,PeVS,PeVB,PVeVS,PVeVB;
final FPertPsi Ai,ai,AV,aV,Itip,itPi,itPt,itPm,itPm_sam,itPvF;
final LPertPsi tip,itp,aip,itPvL;
final RadauPert Rip;
final X0Psi itdp,itpi,itpt,itVs,itSVs,PtI;
final RX0Psi Ritpi,Ritpt;
final PIPsi pipsi;
final DPIPsi dpipsi,vsipsi;
final EEComm eePit,eePiIt,eeVsit,eepVspit,eeVsitL,eeVsitH;
final EdComm edpVsit,edVsit,edpit,edpiit,eddpit;
final EiComm eipVsit,eiVsit;
final double DpI_psiH,varpI_psiH;

MkQVM(final int n,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();
    this.M=getABasis();

    final double [][] QQ=access.getQQ_Matrs(n,1,getMoments(2*n-1));
    final double [] QQt=QQ[0],QQi=QQ[1],QQpt=QQ[2],QQpi=QQ[3],QQdp=QQ[4],QQpio=QQ[5];
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,getX0());
    {
    this.DpI_psiH=TMatr.getAMatrA(I.psiH,VOLATILITY_EDPsi.getDiffMatrix(n,QQpio),n);
    final double [] DpsiH=getEDPsi(I.psiH);
    this.varpI_psiH=TMatr.getAMatrB(I.psiH,DpsiH,QQpio,n)-
	TMatr.getAMatrB(I.psiH,I.psiH,QQpio,n)*TMatr.getAMatrB(I.psiH,DpsiH,QQt,n);
    }

    this.P1t=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpt,QQt,access,getX0());
    this.selection=selectBasisByD2(n,I.ev,n,n,QQi,QQt,n,-1);
    final int nd=this.selection.length;
    final double [] basis=getSelectedEV(n,I.ev,this.selection);
    this.distance=new MDistance(nd,basis,n,nd,QQt,QQi,QQpt,QQpi,QQdp,n);
    ind_lev=this.selection.length<=0?-1:this.selection[this.distance.t_order[0]];
    if(ind_lev<0)
    {
	pertpsiI=psiI=null;
	d1G=d2G=varPi=d1G_IL=d1G_IH=Double.NaN;
    }
    else
    {
	psiI=I.getPsi(ind_lev);
	pertpsiI=DeprecatedHelperEVXData.getPsiPertr(I,ind_lev,QQpi);
	// skip normalizing
	//TMatr.scaleVectorInPlace(pertpsiI,1/Math.sqrt(TMatr.getAMatrA(pertpsiI,QQt,n)));
	d1G=getK2(pertpsiI,psiI,psiI,psiI)-getK2(psiI,psiI,pertpsiI,psiI);
	d2G=getK2(pertpsiI,pertpsiI,psiI,psiI)-getK2(psiI,psiI,pertpsiI,pertpsiI);
	varPi=distance.varPi_distancesorted[0];

	d1G_IL=getD1G(0,I,QQpi);
	d1G_IH=getD1G(n-1,I,QQpi);
    }

    this.distanceP1t=new MDistance(n,P1t.ev,n,n,QQt,QQi,QQpt,QQpi,QQdp,n);
    this.sens_P1t_distancesorted=new double [n];
    final double [] psi=new double [n];
    for(int l=0;l<n;l++)
    {
	final double [] psipert=DeprecatedHelperEVXData.getPsiPertr(P1t,this.distanceP1t.t_order[l],QQi);
	System.arraycopy(this.distanceP1t.evv_distancesorted,l*n,psi,0,n);
	this.sens_P1t_distancesorted[l]=getK2(psipert,psi,psi,psi)-getK2(psi,psi,psipert,psi);
    }

    if(true)
    {
	final int nmoms=2*n-1;
	final double [] imom=new double [nmoms],tmom=new double [nmoms],isellmom=new double [nmoms],ibuymom=new double [nmoms];
	access.setM(imom,0,access.M_I,odata,nmoms);
	access.setMT(tmom,nmoms);
	access.setM(isellmom,0,access.M_ISELL,odata,nmoms);
	access.setM(ibuymom,0,access.M_IBUY,odata,nmoms);
	this.peVS=new PertPsi(n,imom,tmom,isellmom);
	this.peVB=new PertPsi(n,imom,tmom,ibuymom);
	final double [] amom=new double [nmoms],plusmom=new double [nmoms],minusmom=new double [nmoms];
	access.setM(amom,0,access.M_A,odata,nmoms);
	access.setM(plusmom,1,access.M_DPPLUS,odata,nmoms);
	access.setM(minusmom,1,access.M_DPMINUS,odata,nmoms);
	TMatr.scaleVectorInPlace(minusmom,-1);
	this.pePLUS=new PertPsi(n,amom,tmom,plusmom);
	this.peMINUS=new PertPsi(n,amom,tmom,minusmom);

	final double [] dpmom=new double [nmoms];
	access.setM(dpmom,1,access.M_DP,odata,nmoms);
	final double [] Amom=getFmom(amom),Ptmom=getFmom(dpmom);
	this.PePLUS=new PertPsi(n,Amom,tmom,getFmom(plusmom));
	this.PeMINUS=new PertPsi(n,Amom,tmom,getFmom(minusmom));
	this.Pe=new PertPsi(n,Amom,tmom,Ptmom);

	final double [] Vmom=getFmom(imom);
	this.PeVS=new PertPsi(n,Vmom,tmom,getFmom(isellmom));
	this.PeVB=new PertPsi(n,Vmom,tmom,getFmom(ibuymom));

	final double po=getPOffset();
	final double [] pimom=new double [nmoms],pisellmom=new double [nmoms],pibuymom=new double [nmoms];
	access.setM(pimom,1,access.M_I,odata,nmoms);
	TMatr.sumP1(pimom,imom,po);
	access.setM(pisellmom,1,access.M_ISELL,odata,nmoms);
	TMatr.sumP1(pisellmom,isellmom,po);
	access.setM(pibuymom,1,access.M_IBUY,odata,nmoms);
	TMatr.sumP1(pibuymom,ibuymom,po);
	final double [] PVmom=getFmom(pimom);
	this.PVeVS=new PertPsi(n,PVmom,tmom,getFmom(pisellmom));
	this.PVeVB=new PertPsi(n,PVmom,tmom,getFmom(pibuymom));
	this.Ip=DeprecatedHelperEVXData.getEVXData(n,pimom,tmom,access,getX0());
	final double [] vsmom=new double [nmoms],pvsmom=new double [nmoms],dpimom=new double [nmoms];
	access.setM(vsmom,0,access.M_RVS,odata,nmoms);
	access.setM(pvsmom,1,access.M_RVS,odata,nmoms);
	TMatr.sumP1(pvsmom,vsmom,po);
	access.setDFMoments(dpimom,pimom,nmoms);
	TMatr.scaleVectorInPlace(dpimom,-1);
	this.ccorr_pvs_dI=ACorr.getACorrAverage_Psi_and_Basis(n,pvsmom,dpimom,tmom,M,getX0());
	this.xxC=ACorr.getCommutatorVarDPsiX0(n,imom,getFmom(vsmom),tmom,M,getX0());
	this.xxCp=ACorr.getCommutatorVarDPsiX0(n,pimom,getFmom(pvsmom),tmom,M,getX0());
	this.xxCd=ACorr.getCommutatorVarDPsiX0(n,imom,getFmom(vsmom),tmom,M,getX0(),true);
	this.xxCpd=ACorr.getCommutatorVarDPsiX0(n,pimom,getFmom(pvsmom),tmom,M,getX0(),true);
	this.eepVspit=new EEComm(n,pimom,tmom,getFmom(pvsmom));
	this.edpVsit=new EdComm(n,pimom,tmom,getFmom(pvsmom),M,isEdgeAtRight());
	this.eipVsit=new EiComm(n,pimom,tmom,getFmom(pvsmom),M);


	final double [] x0mom=getX0moms(nmoms);

	Ai=new FPertPsi(n,Amom,tmom,imom,Ptmom);
	ai=new FPertPsi(n,amom,tmom,imom,Ptmom);
	AV=new FPertPsi(n,Amom,tmom,Vmom,Ptmom);
	aV=new FPertPsi(n,amom,tmom,Vmom,Ptmom);

	itPvF=new FPertPsi(n,imom,tmom,pimom,pimom);

	final double [] tmom_sam=new double [nmoms],ptmom_sam=new double [nmoms],
	    imom_sam=new double [nmoms],pimom_sam=new double [nmoms];
	access.setM(tmom_sam,0,access.M_T,odata,nmoms);
	access.setM(ptmom_sam,1,access.M_T,odata,nmoms);
	access.setM(imom_sam,0,access.M_I,odata,nmoms);
	access.setM(pimom_sam,1,access.M_I,odata,nmoms);
	itPm_sam=new FPertPsi(n,imom_sam,tmom_sam,x0mom,pimom_sam,ptmom_sam);
	itPi=new FPertPsi(n,imom_sam,tmom,x0mom,pimom_sam);
	itPm=new FPertPsi(n,imom_sam,tmom,x0mom,pimom_sam,Ptmom);
	itPt=new FPertPsi(n,imom,tmom,x0mom,Ptmom);

	this.itdp=new X0Psi(n,imom,tmom,dpmom);
	this.itpi=new X0Psi(n,imom,tmom,pimom_sam);
	this.itpt=new X0Psi(n,imom,tmom,Ptmom);
	this.PtI=new X0Psi(n,Ptmom,tmom,imom);
	final double [] Svsmom=new double [nmoms];
	access.setM(Svsmom,0,access.M_VS,odata,nmoms);
	this.itVs=new X0Psi(n,imom,tmom,getFmom(vsmom));
	this.itSVs=new X0Psi(n,imom,tmom,getFmom(Svsmom));

	final double pol=getPOffset()-getPLast();

	this.pipsi=new PIPsi(n,imom,tmom,TMatr.sumAcB(pimom_sam,pol,imom),Ptmom);
	this.dpipsi=new DPIPsi(n,imom,tmom,dpmom);
	this.vsipsi=new DPIPsi(n,imom,tmom,vsmom);
	this.eePit=new EEComm(n,imom,tmom,getFmom(dpmom));
	this.eePiIt=new EEComm(n,imom,tmom,pimom_sam);
	this.eeVsit=new EEComm(n,imom,tmom,getFmom(vsmom));
	this.eeVsitL=new EEComm(n,imom,tmom,getFmom(vsmom)){
		@Override
		double [] getPsi0(final EVXData j){return j.psiL;}
	    };
	this.eeVsitH=new EEComm(n,imom,tmom,getFmom(vsmom)){
		@Override
		double [] getPsi0(final EVXData j){return j.psiH;}
	    };
	this.edVsit=new EdComm(n,imom,tmom,getFmom(vsmom),M,isEdgeAtRight());
	this.eiVsit=new EiComm(n,imom,tmom,getFmom(vsmom),M);
	this.edpit=new EdComm(n,imom,tmom,Ptmom,M,isEdgeAtRight());
	this.edpiit=new EdComm(n,imom,tmom,pimom_sam,M,isEdgeAtRight());
	this.eddpit=new EdComm(n,imom,tmom,dpmom,M,isEdgeAtRight());
    }
    if(true)
    {
	final int nmoms=2*n-1+1;
	final double [] imom=new double [nmoms],tmom=new double [nmoms],
	    dpmom=new double [nmoms],Ptmom=new double [nmoms],pimom=new double [nmoms],
	    amom=new double [nmoms],Pamom=new double [nmoms];
	access.setM(imom,0,access.M_I,odata,nmoms);
	access.setMT(tmom,nmoms);
	access.setM(dpmom,1,access.M_DP,odata,nmoms);
	access.setFMoments(Ptmom,dpmom,dpmom.length);
	access.setM(pimom,1,access.M_I,odata,nmoms);
	access.setM(amom,0,access.M_A,odata,nmoms);
	access.setM(Pamom,1,access.M_A,odata,nmoms);

	Ritpi=new RX0Psi(n,imom,tmom,pimom);
	Ritpt=new RX0Psi(n,imom,tmom,Ptmom);

	aip=new LPertPsi(n,amom,imom,Pamom);
	tip=new LPertPsi(n,tmom,imom,Ptmom);
	itp=new LPertPsi(n,imom,tmom,pimom);
	itPvL=new LPertPsi(n,imom,pimom,pimom);
	Itip=new FPertPsi(n,M.B.getXQkMomentsFromQkMoments(tmom),tmom,imom,Ptmom);
    }
    if(true)
    {
	final int nmoms=2*n-1+3;
	final double [] imom=new double [nmoms],pimom=new double [nmoms],p2imom=new double [nmoms];
	access.setM(imom,0,access.M_I,odata,nmoms);
	access.setM(pimom,1,access.M_I,odata,nmoms);
	access.setM(p2imom,2,access.M_I,odata,nmoms);
	this.Rip=new RadauPert(n,imom,pimom,p2imom);
    }


    sQQt=QQt;sQQi=QQi;sQQpt=QQpt;sQQpi=QQpi;sQQdp=QQdp;
}
private final double [] psiI,pertpsiI;
private final int [] selection;
private final int ind_lev;
private final double [] sQQt,sQQi,sQQpt,sQQpi,sQQdp;

double [] getFmom(final double [] moms)
{
    final double [] fmoms=new double [moms.length];
    access.setFMoments(fmoms,moms,fmoms.length);
    //TMatr.scaleVectorInPlace(fmoms,-1);
    return fmoms;
}

double getD1G(final int ind_lev,final EVXData J,final double [] QQ)
{
    final double [] psi=J.getPsi(ind_lev),pert=DeprecatedHelperEVXData.getPsiPertr(J,ind_lev,QQ);
    return getK2(pert,psi,psi,psi)-getK2(psi,psi,pert,psi);
}


int [] selectBasisByD2(final int n,final double [] basis,final int basisLD,final int basisOff,final double [] QQf,final double [] QQt,final int ldQQ,final int sign_D2)
{
    final int [] selected=new int [n];
    final double [] psi=new double [basisLD];
    int nb=0;
    for(int k=0;k<n;k++)
    {
	System.arraycopy(basis,basisOff+k*basisLD,psi,0,basisLD);
	final double [] rq=RayleighQuotient.getRQVariationValues(psi,getEDPsi(psi),null,QQf,QQt,ldQQ);
	if(Math.signum(rq[2])==Math.signum(sign_D2))
	{
	    selected[nb++]=k;
	}
    }
    return java.util.Arrays.copyOf(selected,nb);
}

static double [] getSelectedEV(final int n,final double [] ev,final int [] selected)
{
    final int nb=selected.length;
    final double [] sel=new double [nb+n*nb];
    for(int k=0;k<nb;k++)
    {
	final int l=selected[k];
	sel[k]=ev[l];
	System.arraycopy(ev,n+l*n,sel,nb+k*n,n);
    }
    return sel;
}

class MDistance extends Distance {
final double [] I_distancesorted,p_distancesorted,varPt_distancesorted,varPi_distancesorted,dp_distancesorted;

MDistance(final int n,final double [] basis,final int basisLD,final int basisOff,
	  final double [] QQt,final double [] QQi,final double [] QQpt,final double [] QQpi,final double [] QQdp,final int ldQQ)
{
    super(n,basis,basisLD,basisOff);
    this.p_distancesorted=getOrderedEMfromQQ(QQpt,ldQQ);
    this.dp_distancesorted=getOrderedEMfromQQ(QQdp,ldQQ);
    this.I_distancesorted=getOrderedEMfromQQ(QQi,ldQQ);
    this.varPt_distancesorted=new double [n];
    this.varPi_distancesorted=new double [n];
    final double [] psi=new double [basisLD];
    for(int k=0;k<n;k++)
    {
	System.arraycopy(this.evv_distancesorted,k*basisLD,psi,0,basisLD);
	final double [] dpsi=getEDPsi(psi);
	this.varPt_distancesorted[k]=RayleighQuotient.getRQVariationValues(psi,dpsi,null,QQpt,QQt,ldQQ)[1];
	this.varPi_distancesorted[k]=RayleighQuotient.getRQVariationValues(psi,dpsi,null,QQpi,QQt,ldQQ)[1];
    }
}
void test_Self(final double eps)
{
    System.err.println("nselected="+t_order.length);
    super.test_Self(eps);
}

double getK2Distance(final double [] a,final double [] b){return getK2(a,a,b,b);}
} // END OF class MDistance


class PertPsi {
final EVXData J;
final double [] fpert,senspert;
PertPsi(final int n,final double [] jmom,final double [] tmom,final double [] pertmom)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{jmom,tmom,pertmom});
    final double [] QQj=QQ[0],QQt=QQ[1],QQpert=QQ[2];
    this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQj,QQt,access,getX0());
    this.fpert=new double [n];
    this.senspert=new double [n];
    for(int k=0;k<n;k++)
    {
	this.fpert[k]=J.getFPsi(k,QQpert);
	this.senspert[k]=getD1G(k,J,QQpert);
    }
}
} // END OF class PertPsi

class FPertPsi {
final EVXData J;
final double [] senspsi,var2j,f,varf,varFn;
FPertPsi(final int n,final double [] jmom,final double [] tmom,final double [] pertmom,final double [] fmom)
{
    this(n,jmom,tmom,pertmom,fmom,new double [2*n-1]);
}
FPertPsi(final int n,final double [] jmom,final double [] tmom,final double [] pertmom,final double [] fmom,final double [] ftmom)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{jmom,tmom,pertmom,fmom,ftmom});
    final double [] QQj=QQ[0],QQt=QQ[1],QQpert=QQ[2],QQf=QQ[3],QQft=QQ[4];
    this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQj,QQt,access,getX0());
    this.senspsi=new double [n];
    this.f=new double [n];
    this.varf=new double [n];
    this.varFn=new double [n];
    this.var2j=new double [n];
    final double [] QQfsum=new double [n*n];
    for(int k=0;k<n;k++)
    {
	final double ek=this.J.ev[k];
	final double [] psi=this.J.getPsi(k),pert=DeprecatedHelperEVXData.getPsiPertr(this.J,k,QQpert);
	this.var2j[k]=RayleighQuotient.getRQVariationValues(psi,pert,null,QQj,QQt,n)[2];
	this.senspsi[k]=getK2(pert,psi,psi,psi)-getK2(psi,psi,pert,psi);
	for(int l=QQfsum.length;--l>=0;){QQfsum[l]=QQf[l]-ek*QQft[l];}
	final double [] var=RayleighQuotient.getRQVariationValues(psi,pert,null,QQfsum,QQt,n);
	this.f[k]=var[0];
	this.varf[k]=var[1];
	this.varFn[k]=this.varf[k]/TMatr.getAMatrA(psi,QQpert,n);
    }
    sjmom=jmom;
    stmom=tmom;
    spertmom=pertmom;
    sfmom=fmom;
    sftmom=ftmom;
}
final double [] sjmom,stmom,spertmom,sfmom,sftmom;
void test_Self(final double eps)
{
    final int n=J.neq;
    final double [] o=new double [n];
    final FPertPsi fj=new FPertPsi(n,sjmom,stmom,spertmom,sjmom);
    UnitTests.checkDoubleArrayRel(fj.varf,"fj.varf",o,"0",eps);
    final FPertPsi ft=new FPertPsi(n,sjmom,stmom,spertmom,stmom);
    UnitTests.checkDoubleArrayRel(ft.varf,"ft.varf",o,"0",eps);

    final double [] psik=new double [n],psil=new double [n];
    final double [] varp=new double [n];
    for(int k=0;k<n;k++)
    {
	System.arraycopy(J.ev,n+k*n,psik,0,n);
	final double ek=J.ev[k];
	final double [] pertpsikmom=M.B.getPQkMomentsFromQkMoments(psik,spertmom),
	    fpsikmom=M.B.getPQkMomentsFromQkMoments(psik,sfmom),
	    ftpsikmom=M.B.getPQkMomentsFromQkMoments(psik,sftmom);
	double s=0;
	for(int l=0;l<n;l++)
	{
	    System.arraycopy(J.ev,n+l*n,psil,0,n);
	    final double el=J.ev[l];
	    if(k!=l)
	    {
		s+=(TMatr.mulVVn1(psil,pertpsikmom)/(ek-el))*
		    (TMatr.mulVVn1(psil,fpsikmom)-ek*TMatr.mulVVn1(psil,ftpsikmom));
	    }
	}
	varp[k]=2*s;
    }
    UnitTests.checkDoubleArrayRel(this.varf,"this.varf",varp,"varp",eps);

    print_PertM(eps);
    System.err.println("MkQVM:FPertPsi:test_Self: test OK n="+n+" eps="+eps);
}

void print_PertM(final double eps)
{
    final int n=J.neq;
    final double [] QQt=M.B.getQQMatr(n,stmom),QQpert=M.B.getQQMatr(n,spertmom);
    for(int k=0;k<n;k++)
    {
	final double [] psi=J.getPsi(k),pertPsi=DeprecatedHelperEVXData.getPsiPertr(J,k,QQpert),
	    pertPsiB=EVXData.convertPsiFromQtoB_Basis(n,pertPsi,J.ev,QQt);

	System.err.println("Bpert"+k+"="+java.util.Arrays.toString(pertPsiB));
    }
}


double [] getD0Pert()
{
    final int n=this.J.neq;
    final double [] Fn=new double [n],m0=getX0moms(n);
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{sfmom,sftmom});
    final double [] QQfi=QQ[0],QQft=QQ[1];
    final double x0=getX0();
    for(int k=0;k<n;k++)
    {
	final double [] psik=this.J.getPsi(k);
	final double ek=this.J.ev[k],psikx0=TMatr.mulVVn1(psik,m0);
	double s=0;
	for(int l=0;l<n;l++)
	{
	    if(k!=l)
	    {
		final double [] psil=this.J.getPsi(l);
		final double psilx0=TMatr.mulVVn1(psil,m0);
		s+=((TMatr.getAMatrB(psik,psil,QQfi,n)-ek*TMatr.getAMatrB(psik,psil,QQft,n))
		    /(ek-this.J.ev[l]))*(psilx0/psikx0);
	    }
	}
	Fn[k]=2*s;
    }
    return Fn;
}
} // END OF class FPertPsi


double [] getX0moms(final int nmoms)
{
    return M.B.getX0Moments(nmoms,getX0());
}

class LPertPsi {
final EVXData J;
final double [] senspsi,f,X,Xf,varf;
LPertPsi(final int n,final double [] tmom,final double [] pertmom,final double [] fmom)
{
    final double [] Xtmom=M.B.getXQkMomentsFromQkMoments(tmom),Xpertmom=M.B.getXQkMomentsFromQkMoments(pertmom),Xfmom=M.B.getXQkMomentsFromQkMoments(fmom);
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{Xtmom,tmom,Xpertmom,pertmom,fmom,Xfmom});
    final double [] XQQt=QQ[0],QQt=QQ[1],XQQpert=QQ[2],QQpert=QQ[3],QQf=QQ[4],XQQf=QQ[5];
    this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(n,XQQt,QQt,access,getX0());
    this.senspsi=new double [n];
    this.f=new double [n];
    this.Xf=new double [n];
    this.X=new double [n];
    this.varf=new double [n];
    final double [] QQj=new double [n*n];
    for(int k=0;k<n;k++)
    {
	final double e=this.X[k]=J.ev[k];
	final double [] psi=J.getPsi(k);
	this.Xf[k]=TMatr.getAMatrA(psi,XQQf,n)-e*TMatr.getAMatrA(psi,QQf,n);
	for(int l=0;l<XQQpert.length;l++){QQj[l]=XQQpert[l]-e*QQpert[l];}
	final double [] pert=DeprecatedHelperEVXData.getPsiPertr(J,k,QQj);
	this.senspsi[k]=getK2(pert,psi,psi,psi)-getK2(psi,psi,pert,psi);
	final double [] var=RayleighQuotient.getRQVariationValues(psi,pert,null,QQf,QQt,n);
	this.f[k]=var[0];
	this.varf[k]=var[1];
    }
    stmom=tmom;
    spertmom=pertmom;
    sfmom=fmom;
}
final double [] stmom,spertmom,sfmom;
void test_Self(final double eps)
{
    final int n=J.neq;
    final double [] psik=new double [n],psil=new double [n];
    final double [] varp=new double [n];
    final double [] xpertmom=M.B.getXQkMomentsFromQkMoments(spertmom);
    for(int k=0;k<n;k++)
    {
	System.arraycopy(J.ev,n+k*n,psik,0,n);
	final double ek=J.ev[k];
	final double [] pertpsikmom=M.B.getPQkMomentsFromQkMoments(psik,spertmom),
	    xpertpsikmom=M.B.getPQkMomentsFromQkMoments(psik,xpertmom),
	    fpsikmom=M.B.getPQkMomentsFromQkMoments(psik,sfmom);
	double s=0;
	for(int l=0;l<n;l++)
	{
	    System.arraycopy(J.ev,n+l*n,psil,0,n);
	    final double el=J.ev[l];
	    if(k!=l)
	    {
		s+=((TMatr.mulVVn1(psil,xpertpsikmom)-ek*TMatr.mulVVn1(psil,pertpsikmom))/(ek-el))*
		    TMatr.mulVVn1(psil,fpsikmom);
	    }
	}
	varp[k]=2*s;
    }

    final LPertPsi l=new LPertPsi(J.neq,stmom,spertmom,stmom);

    for(int k=0;k<n;k++)
    {
	UnitTests.checkDoubleRel(varf[k],"varf[k]",varp[k],"varp[k]",eps);
	UnitTests.checkDoubleRel(l.Xf[k],"l.Xf[k]",0,"0",eps);
    }
    System.err.println("MkQVM:LPertPsi:test_Self: test OK n="+n+" eps="+eps);
}
} // END OF class LPertPsi




class RadauPert {
final EVXData J;
final double [] X,sigma2_X,f,sigma2_f,Xf,corr;
final double w0;
RadauPert(final int n,final double [] moms,final double [] fmoms,final double [] f2moms)
{
    final double [] Rmoms=M.getRadauMomentsFromMoments(moms),
	XRmoms=M.B.getXQkMomentsFromQkMoments(Rmoms),
	XXRmoms=M.B.getXQkMomentsFromQkMoments(XRmoms),
	Rfmoms=M.getRadauMomentsFromMoments(fmoms),
	Rf2moms=M.getRadauMomentsFromMoments(f2moms),
	XRfmoms=M.B.getXQkMomentsFromQkMoments(Rfmoms);
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{Rmoms,XRmoms,XXRmoms,Rfmoms,XRfmoms,Rf2moms});
    final double [] QQRm=QQ[0],XQQRm=QQ[1],XXQQRm=QQ[2],QQRf=QQ[3],XQQRf=QQ[4],QQRf2=QQ[5];
    final double x0=getX0();
    this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(n,XQQRm,QQRm,access,x0);
    if(true)
    {
	final double [] pp=new double [n];
	System.arraycopy(J.ev,n+0*n,pp,0,n);
	final double [] ppx=M.getxP(pp);
	TMatr.sumP1(ppx,pp,-J.ev[0]);
	this.w0=TMatr.mulVVn1(ppx,moms)/M.B.getD0(ppx,x0);
    }

    this.X=new double [n];
    this.sigma2_X=new double [n];
    this.f=new double [n];
    this.sigma2_f=new double [n];
    this.Xf=new double [n];
    this.corr=new double [n];
    for(int k=0;k<n;k++)
    {
	this.X[k]=J.ev[k];
	this.sigma2_X[k]=TMatr.getAMatrB(n,J.ev,n+k*n,J.ev,n+k*n,XXQQRm,n)-Math.pow(this.X[k],2);
	this.f[k]=TMatr.getAMatrB(n,J.ev,n+k*n,J.ev,n+k*n,QQRf,n);
	this.sigma2_f[k]=TMatr.getAMatrB(n,J.ev,n+k*n,J.ev,n+k*n,QQRf2,n)-Math.pow(this.f[k],2);
	this.Xf[k]=TMatr.getAMatrB(n,J.ev,n+k*n,J.ev,n+k*n,XQQRf,n)-this.f[k]*this.X[k];
	this.corr[k]=this.Xf[k]/Math.sqrt(this.sigma2_X[k]*this.sigma2_f[k]);
    }

    smoms=moms;
    sfmoms=fmoms;
    sf2moms=f2moms;
}
final double [] smoms,sfmoms,sf2moms;
void test_Self(final double eps)
{
    final int n1=J.neq+1;
    final double [] Q0=new double [n1];
    final double x0=getX0();
    for(int k=0;k<Q0.length;k++)
    {
	final double [] r=new double [k+1];r[k]=1;
	Q0[k]=M.B.getD0(r,x0);
    }
    final double [] P=access.getLSQ(n1,M.B.getQQMatr(n1,smoms),new double [][]{Q0})[0];
    final double _w0=1/M.B.getD0(P,x0);
    UnitTests.checkDoubleRel(this.w0,"this.w0",_w0,"_w0",eps);
    final double [] _X=M.getXNodes(P).re_root;
    UnitTests.checkDoubleArrayRel(this.X,"this.X",_X,"_X",eps);
    final double [] _sigma2_X=new double [_X.length],
	_f=new double [_X.length],_Xf=new double [_X.length],_sigma2_f=new double [_X.length];
    final double [] ppXX=M.getRadauMultiplied(M.B.mult2Pol(P,P));

    for(int k=0;k<_X.length;k++)
    {
	final double [] pp=M.B.sdiv1(P,_X[k]).q;
	final double [] ppr2=M.getRadauMultiplied(M.B.mult2Pol(pp,pp)),Xppr2=M.getxP(ppr2);
	final double nr=TMatr.mulVVn1(ppr2,smoms),xk=TMatr.mulVVn1(Xppr2,smoms)/nr;
	UnitTests.checkDoubleRel(_X[k],"_X[k]",xk,"xk",eps);
	_sigma2_X[k]=TMatr.mulVVn1(ppXX,smoms)/nr;
	_f[k]=TMatr.mulVVn1(ppr2,sfmoms)/nr;
	_sigma2_f[k]=TMatr.mulVVn1(ppr2,sf2moms)/nr-Math.pow(_f[k],2);
	_Xf[k]=TMatr.mulVVn1(Xppr2,sfmoms)/nr-xk*_f[k];
	final double corr=_Xf[k]/Math.sqrt(_sigma2_X[k]*_sigma2_f[k]);
	System.err.println("corr("+k+")="+corr);
    }
    UnitTests.checkDoubleArrayRel(this.sigma2_X,"this.sigma2_X",_sigma2_X,"_sigma2_X",eps);
    UnitTests.checkDoubleArrayRel(this.f,"this.f",_f,"_f",eps);
    UnitTests.checkDoubleArrayRel(this.sigma2_f,"this.sigma2_f",_sigma2_f,"_sigma2_f",eps);
    UnitTests.checkDoubleArrayRel(this.Xf,"this.Xf",_Xf,"_Xf",eps);


    final double [] xs=M.B.getXQkMomentsFromQkMoments(smoms);
    final RadauPert r=new RadauPert(J.neq,smoms,xs,M.B.getXQkMomentsFromQkMoments(xs));
    for(int k=0;k<r.corr.length;k++)
    {
    	UnitTests.checkDoubleRel(r.corr[k],"r.corr[k]",1,"1",eps);
	//System.err.println("Xcorr("+k+")="+r.corr[k]);
    }

    System.err.println("MkQVM:RadauPert:test_Self: test OK n="+J.neq+" eps="+eps);
}
} // END OF class RadauPert





class X0Psi {
final EVXData I,J;
final double w_L,w_H,fI_L,fI_H,fJ_L,fJ_H,sens_fJ_L,sens_fJ_H,sens_f,sensI_f,sensL_f,sensH_f;
final double [][] PsiX0;
final double [] varpsi0,varIpsi0,varLpsi0,varHpsi0;
final double mL0_f,mL0_I,mH0_f,mH0_I,mA,mA_rho,sigma2_f,sigma2_I,mAL,mAH,mA0,mAI,mAI_rho,mAId_rho;
X0Psi(final int n,final double [] imom,final double [] tmom,final double [] fmom)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,fmom});
    final double [] QQi=QQ[0],QQt=QQ[1],QQf=QQ[2];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);
    this.fI_L=this.I.getFPsi(0,QQf);
    this.fI_H=this.I.getFPsi(n-1,QQf);

    final double [] one=new double []{1};
    one[0]/=access.fx(one,0);
    final double [] mul=M.getRadauMultiplied(M.getRadauMultiplied(one));

    final double [] Mimom=M.B.getPQkMomentsFromQkMoments(mul,imom),
	Mtmom=M.B.getPQkMomentsFromQkMoments(mul,tmom),Mfmom=M.B.getPQkMomentsFromQkMoments(mul,fmom);
    final int nd=n-1;
    final double [][] MQQ=access.getQQ_Matrs(nd,1,new double [][]{Mimom,Mtmom,Mfmom});
    final double [] MQQi=MQQ[0],MQQt=MQQ[1],MQQf=MQQ[2];
    this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(nd,MQQi,MQQt,access,x0);
    this.fJ_L=this.J.getFPsi(0,MQQf);
    this.fJ_H=this.J.getFPsi(nd-1,MQQf);
    this.sens_fJ_L=getD1GRM(0,this.J,MQQf);
    this.sens_fJ_H=getD1GRM(nd-1,this.J,MQQf);

    this.PsiX0=new double [nd][];
    for(int k=0;k<this.PsiX0.length;k++)
    {
	this.PsiX0[k]=M.getRadauMultiplied(this.J.getPsi(k));
	// Already normalized, no need to normalize here. If broken - will be caught in test_Self.
	//TMatr.scaleVectorInPlace(this.PsiX0[k],1/Math.sqrt(TMatr.getAMatrA(this.PsiX0[k],QQt,n)));
    }
    this.w_L=TMatr.getAMatrB(this.I.psiL,this.PsiX0[0],QQt,n);
    this.w_H=TMatr.getAMatrB(this.I.psiH,this.PsiX0[nd-1],QQt,n);

    this.varpsi0=new double [n];
    this.varIpsi0=new double [n];
    this.varLpsi0=new double [n];
    this.varHpsi0=new double [n];
    final double iL=this.I.sL,iH=this.I.sH,i0=this.I.s0;
    final double [] psi0=this.I.psi0;
    double mA=0,sigma2_f=0,sigma2_I=0,mAL=0,mAH=0,mA0=0;
    double mAI=0,sigma2mAI_f=0,sigma2mAI_I=0,sigma2mAId_I=0;
    for(int k=0;k<this.PsiX0.length;k++)
    {
	final double mf=TMatr.getAMatrB(psi0,this.PsiX0[k],QQf,n),
	    mI=TMatr.getAMatrB(psi0,this.PsiX0[k],QQi,n);
	TMatr.sumP1(varpsi0,this.PsiX0[k],mf);
	TMatr.sumP1(varIpsi0,this.PsiX0[k],mf/this.J.ev[k]);
	if(k==0) TMatr.sumP1(varLpsi0,this.PsiX0[k],mf);
	if(k==(this.PsiX0.length-1)) TMatr.sumP1(varHpsi0,this.PsiX0[k],mf);
	final double ss=mf*mI;
	mA+=ss;
	sigma2_f+=mf*mf;
	sigma2_I+=mI*mI;
	mAL+=ss/(iL-this.J.ev[k]);
	mAH+=ss/(iH-this.J.ev[k]);
	mA0+=ss/(i0-this.J.ev[k]);

	mAI+=ss/this.J.ev[k];
	sigma2mAI_f+=(mf*mf)/this.J.ev[k];
	sigma2mAI_I+=(mI*mI)/this.J.ev[k];
	sigma2mAId_I+=Math.pow(mI/this.J.ev[k],2);
    }
    this.mA=mA;
    this.sigma2_f=sigma2_f;
    this.sigma2_I=sigma2_I;
    this.mA_rho=mA/Math.sqrt(sigma2_f*sigma2_I);
    this.mAL=mAL;
    this.mAH=mAH;
    this.mA0=mA0;

    this.mAI=mAI;
    this.mAI_rho=mAI/Math.sqrt(sigma2mAI_f*sigma2mAI_I);
    this.mAId_rho=mAI/Math.sqrt(sigma2_f*sigma2mAId_I);


    this.sens_f=getK2(varpsi0,psi0,psi0,psi0)-getK2(psi0,psi0,varpsi0,psi0);
    this.sensI_f=getK2(varIpsi0,psi0,psi0,psi0)-getK2(psi0,psi0,varIpsi0,psi0);
    this.sensL_f=getK2(varLpsi0,psi0,psi0,psi0)-getK2(psi0,psi0,varLpsi0,psi0);
    this.sensH_f=getK2(varHpsi0,psi0,psi0,psi0)-getK2(psi0,psi0,varHpsi0,psi0);

    mL0_f=TMatr.getAMatrB(psi0,this.PsiX0[0],QQf,n);
    mL0_I=TMatr.getAMatrB(psi0,this.PsiX0[0],QQi,n);
    mH0_f=TMatr.getAMatrB(psi0,this.PsiX0[this.PsiX0.length-1],QQf,n);
    mH0_I=TMatr.getAMatrB(psi0,this.PsiX0[this.PsiX0.length-1],QQi,n);

    this.simom=imom;
    this.stmom=tmom;
    this.sfmom=fmom;
}
final double [] simom,stmom,sfmom;

double getD1GRM(final int ind_lev,final EVXData j,final double [] QQ)
{
    final double [] psi=j.getPsi(ind_lev),pert=DeprecatedHelperEVXData.getPsiPertr(j,ind_lev,QQ);
    final double [] psim=M.getRadauMultiplied(psi),pertm=M.getRadauMultiplied(pert);
    return getK2(pertm,psim,psim,psim)-getK2(psim,psim,pertm,psim);
}

void test_Self(final double eps)
{
    test_EpPsi(eps);

    final int n=this.I.neq,nd=n-1;
    final double x0=getX0();
    final double [] QQt=M.B.getQQMatr(n,stmom),QQi=M.B.getQQMatr(n,simom),QQf=M.B.getQQMatr(n,sfmom),QQx0=M.B.getQQMatr(n,getX0moms(2*n-1));

    final EVXData j=DeprecatedHelperEVXData.getEVXDataFromQQ(n,TMatr.sumAcB(QQi,this.I.sH*1e6,QQx0),QQt,access,x0);

    UnitTests.checkDoubleArrayRel(EVXData.getMSign(this.I.psi0),"I.psi0",EVXData.getMSign(j.getPsi(n-1)),"j.psi[n-1]",eps);
    UnitTests.checkDoubleArrayRel1(EVXData.getMSign(this.varpsi0),"varpsi0",EVXData.getMSign(TMatr.scaleVectorInPlace(DeprecatedHelperEVXData.getPsiPertr(j,n-1,QQf),j.sH)),"getPsiPertr(k)",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(this.varpsi0,this.I.psi0,QQt,n),"<varpsi0|psi0>",0,"0",eps);

    for(int k=0;k<this.PsiX0.length;k++)
    {
	UnitTests.checkDoubleRel(TMatr.getAMatrA(this.PsiX0[k],QQt,n),"<psi|psi>",1,"1",eps);
	UnitTests.checkDoubleRel(M.B.getD0(this.PsiX0[k],x0),"fx(PsiX0[k],x0)",0,"0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(this.PsiX0[k],this.I.psi0,QQt,n),"<psi|psi0>",0,"0",eps);
	UnitTests.checkDoubleRel(this.J.ev[k],"J.ev",j.ev[k],"j.ev",eps);

	UnitTests.checkDoubleArrayRel1(EVXData.getMSign(this.PsiX0[k]),"PsiX0[k]",EVXData.getMSign(j.getPsi(k)),"j.psi[k]",eps);

    }
    UnitTests.checkDoubleRel(this.fJ_L,"this.fJ_L",TMatr.getAMatrA(this.PsiX0[0],QQf,n),"<psiL|QQf|psiL>",eps);
    UnitTests.checkDoubleRel(this.fJ_H,"this.fJ_H",TMatr.getAMatrA(this.PsiX0[nd-1],QQf,n),"<psiH|QQf|psiH>",eps);

    final X0Psi y=new X0Psi(n,sfmom,stmom,simom); // swap i<->f
    System.err.println("mA="+this.mA+" y.mA="+y.mA);
    // have to match because orthogonal transform between two basises do exist.
    UnitTests.checkDoubleRel(this.mA,"mA",y.mA,"y.mA",eps);
    UnitTests.checkDoubleRel(this.mA_rho,"mA_rho",y.mA_rho,"y.mA_rho",eps);

    System.err.println("MkQVM:X0Psi:test_Self: test OK n="+n+" eps="+eps);
}

void test_EpPsi(final double eps)
{
    final EpComm q=new EpComm(this.I.neq,simom,stmom,sfmom,M);
    q.match_X0Psi(this,eps);
}

} // END OF class X0Psi



class PIPsi {
final EVXData I,P;
final double mA,mA_rho,mL,mH,mA0,mAE,mAd0;
PIPsi(final int n,final double [] imom,final double [] tmom,final double [] fimom,final double [] ftmom)
{
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,fimom,ftmom});
    final double [] QQi=QQ[0],QQt=QQ[1],QQfi=QQ[2],QQft=QQ[3];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);

    final double [] QQM=TMatr.mulMMM(n,QQft,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),QQi);
    TMatr.selectHermitianPartInPlace(n,QQM);
    final double [] QQMP=TMatr.sumAcB(QQfi,-1,QQM);

    this.P=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQMP,QQt,access,x0);

    final double [] psi0=this.I.psi0;
    double mA=0,mL=0,mH=0;
    for(int k=0;k<n;k++)
    {
	final double m=this.I.getFPsiPsi(k,psi0,QQt)*this.I.getFPsiPsi(k,psi0,QQfi)-this.I.getFPsiPsi(k,psi0,QQft)*this.I.getFPsiPsi(k,psi0,QQi);
	mA+=m;
	if(k==0) mL=m;
	if(k==(n-1)) mH=m;
    }
    this.mA=mA;
    this.mA_rho=Double.NaN;
    this.mL=mL;
    this.mH=mH;
    final double m00=TMatr.getAMatrA(psi0,QQft,n)*TMatr.getAMatrA(psi0,QQi,n);
    this.mA0=TMatr.getAMatrA(psi0,QQM,n)-m00;
    this.mAE=TMatr.getAMatrA(psi0,QQfi,n);
    this.mAd0=this.mAE-m00;

    this.simom=imom;
    this.stmom=tmom;
    this.sfimom=fimom;
    this.sftmom=ftmom;
}
final double [] simom,stmom,sfimom,sftmom;


void test_Self(final double eps)
{
    final int n=this.I.neq;

    final PIPsi y=new PIPsi(n,sftmom,stmom,sfimom,simom); // swap i<->f
    System.err.println("PIPsi: mA="+this.mA+" y.mA="+y.mA);
    // have to match because orthogonal transform between two basises do exist.
    UnitTests.checkDoubleRel(this.mA,"mA",y.mA,"y.mA",eps);
    UnitTests.checkDoubleRel(this.mA_rho,"mA_rho",y.mA_rho,"y.mA_rho",eps);

    final double [] QQt=M.B.getQQMatr(n,stmom),QQi=M.B.getQQMatr(n,simom),QQfi=M.B.getQQMatr(n,sfimom),QQft=M.B.getQQMatr(n,sftmom);
    final double [] psi0=this.I.psi0;
    final double _mA=TMatr.getAMatrA(psi0,QQfi,n)-TMatr.getAMatrA(psi0,TMatr.mulMMM(n,QQft,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),QQi),n);
    UnitTests.checkDoubleRel(this.mA,"mA",_mA,"_mA",eps);
    UnitTests.checkDoubleRel(this.mA,"mA",this.P.s0,"this.P.s0",eps);

    System.err.println("MkQVM:PIPsi:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class PIPsi


static double [] getRCorr(final double [] psi,final double [] QQa,final double [] QQb,final double [] QQt,final int ldQQ)
{
    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(ldQQ,ldQQ,QQt);
    final double [] AB=TMatr.mulMMM(ldQQ,QQa,IQQt,QQb),
	AA=TMatr.mulMMM(ldQQ,QQa,IQQt,QQa),
	BB=TMatr.mulMMM(ldQQ,QQb,IQQt,QQb);
    final double ab=TMatr.getAMatrA(psi,AB,ldQQ),aa=TMatr.getAMatrA(psi,AA,ldQQ),bb=TMatr.getAMatrA(psi,BB,ldQQ),
	rho=ab/Math.sqrt(aa*bb);
    return new double []{ab,aa,bb,rho};
}

class EEComm extends EComm {
    final double sens,diff,diffa,Idiff,xdiff;
    EEComm(final int n,final double [] imom,final double [] tmom,final double [] fmom)
    {
	super(n,imom,tmom,fmom,M);
	// Sensitivity give time direction of perturbation.
	this.sens=getK2(varpsi0,psi0,psi0,psi0)-getK2(psi0,psi0,varpsi0,psi0);
	final double [] dpsi0=getEDPsi(psi0),QQt=M.B.getQQMatr(n,tmom),QQi=M.B.getQQMatr(n,imom),IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
	this.diff=TMatr.getAMatrB(this.varpsi0,dpsi0,QQt,n);
	this.diffa=-TMatr.getAMatrB(getEDPsi(this.varpsi0),psi0,QQt,n);

	final double [] Ipsi0=TMatr.getMatr_x_A(n,IQQt,TMatr.getMatr_x_A(n,QQi,psi0)),
	    xdpsi0=TMatr.getMatr_x_A(n,IQQt,getD0(n,getX0()));
	//System.err.println("MkQVM:Ipsi0="+java.util.Arrays.toString(Ipsi0));
	TMatr.sumP1(Ipsi0,psi0,-TMatr.getAMatrB(Ipsi0,psi0,QQt,n));
	this.Idiff=TMatr.getAMatrB(this.varpsi0,Ipsi0,QQt,n);
	this.xdiff=TMatr.getAMatrB(this.varpsi0,xdpsi0,QQt,n);
	//System.err.println("Idiff="+this.Idiff+" ab="+TMatr.getAMatrB(this.varpsi0,psi0,QQi,n));
	//this.Idiff_rho=this.Idiff/Math.sqrt(TMatr.getAMatrA(this.varpsi0,QQt,n)*TMatr.getAMatrA(Ipsi0,QQt,n));
    }
    double [] getDpsi(final double [] psi){return getEDPsi(psi);}
    double [] getD0(final int n,final double x0)
    {
	final double [] d=new double [n],tmp=new double [n];
	for(int k=0;k<n;k++)
	{
	    tmp[k]=1;
	    d[k]=M.B.getD0(getDpsi(tmp),x0);
	    tmp[k]=0;
	}
	return d;
    }
    void test_ACorr(final double eps)
    {
	final int n=this.I.neq;
	final double x0=this.I.x0;
	final double [][] pp=ACorr.getCommutatorVarPsiX0(n,simom,sfmom,stmom,M,x0);
	UnitTests.checkDoubleArrayRel1(EVXData.getMSign(this.psi0),"psi0",EVXData.getMSign(pp[0]),"pp[0]",eps);
	UnitTests.checkDoubleArrayRel1(EVXData.getMSign(this.varpsi0),"varpsi0",EVXData.getMSign(pp[1]),"pp[1]",eps);

	final ACorr a=ACorr.getCommutatorVarDPsiX0(n,simom,sfmom,stmom,M,x0);
	// UnitTests.checkDoubleRel(a.x_a_b*getSignDir(),"a.x_a_b*getSignDir()",this.xDsens/Math.sqrt(I.o_o),"xDsens",eps); // changes normalizong
	UnitTests.checkDoubleRel(a.osp*getSignDir(),"a.osp*getSignDir()",this.xDsens_rho,"xDsens_rho",eps);
	UnitTests.checkDoubleRel(-a.osp*getSignDir(),"-a.osp*getSignDir()",this.Dsens_rho,"Dsens_rho",eps);
	UnitTests.checkDoubleRel(a.osp,"a.osp",this.xxDsens_rho,"xxDsens_rho",eps);
	final ACorr b=ACorr.getCommutatorVarDPsiX0(n,simom,sfmom,stmom,M,x0,true);
	UnitTests.checkDoubleRel(b.osp,"b.osp",this.xxDsens_Rrho,"xxDsens_Rrho",eps);
	// UnitTests.checkDoubleRel(a.x_a_b,"a.x_a_b",this.xxDsens,"xxDsens",eps); // changes normalizong

	System.err.println("MkQVM:EEComm:test_ACorr: test OK n="+n+" eps="+eps);
    }
    void test_Self(final double eps)
    {
	if(this.psi0==this.I.psi0) {
	    test_ACorr(eps);
	    UnitTests.checkDoubleRel(diffa,"diffa",diff,"diff",eps);
	}
	UnitTests.checkDoubleRel(this.Dsens,"this.Dsens",diff,"diff",eps);
	UnitTests.checkDoubleRel(this.xDsens,"this.xDsens",xdiff,"xdiff",eps);
	if(this.psi0==this.I.psi0) UnitTests.checkDoubleRel(this.IDsens,"this.IDsens",Idiff,"Idiff",eps);
	System.err.println("EEComm: diff="+diff+" diffa="+diffa);
	System.err.println("diff="+this.diff+" xdiff="+this.xdiff+" flag0="+(psi0==this.I.psi0)+" Idiff="+Idiff);
	super.test_Self(eps);
    }
} // END OF class EEComm

double getSignDir(){return 1;}


class DPIPsi {
final EVXData I;
final double I0d,mAverd,mAverd_rho,mAver,mAI0,mAc,mAver_rho,mApsi0,mAver_a,mAver_aa,mAver_oDi,mAver_oDid,mAver_oDi0,mAver_oDiof,mA_ospd,mA_ospd_rho;
final ACorr Acorr_f_i,Acorr_f_di,Acorr_t_di;
final ACorr acorr_f_i,acorr_f_di;
final ACorr bcorr_f_i,bcorr_f_di;
final ACorr ccorr_f_i,ccorr_f_di,ccorr_i_di,ccorr_t_i,ccorr_t_di,ccorr_f_t;
final ACorr SPcorr_f_i,SPcorr_f_di,SPcorr_f_odi,SPrcorr_f_i,SPrcorr_f_di,SPrrcorr_f_i,SPrrcorr_f_di;
DPIPsi(final int n,final double [] imom,final double [] tmom,final double [] fmom)
{
    final int nmoms=2*n-1;
    final double [] o=getX0moms(nmoms);
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{imom,tmom,fmom,o});
    final double [] QQi=QQ[0],QQt=QQ[1],QQf=QQ[2],QQo=QQ[3];
    final double x0=getX0();
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);

    final double [] Dimom=new double [nmoms],ImDimom=new double [nmoms];
    access.setDFMoments(Dimom,imom,nmoms);
    // here we assume that omega(x0)=1
    for(int k=nmoms;--k>=0;){ImDimom[k]=this.I.s0*o[k] /* *omega(x0) */ -Dimom[k];}
    final double [][] QQM=access.getQQ_Matrs(n,1,new double [][]{Dimom,ImDimom});
    final double [] QQDi=QQM[0],QQImDi=QQM[1],IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),psia=getPsiA(n,QQt);
    final double p0op0=TMatr.getAMatrA(this.I.psi0,QQo,n),
	i0d=TMatr.getAMatrA(this.I.psi0,QQDi,n)/p0op0,
	f0=TMatr.getAMatrA(this.I.psi0,QQf,n)/p0op0;
    this.I0d=i0d;
    final double [] oDimom=new double [nmoms];
    for(int k=nmoms;--k>=0;){oDimom[k]=i0d*o[k] /* *omega(x0) */ -Dimom[k];}
    final double [] QQoDi=access.getQQ_Matrs(n,1,new double [][]{oDimom})[0],
	QQof=TMatr.sumAcB(QQf,-f0,QQo);
    //System.err.println("<psi0|QQoDi|psi0>="+TMatr.getAMatrA(this.I.psi0,QQoDi,n));
    //System.err.println("<psi0|QQf-f0*QQo|psi0>="+TMatr.getAMatrA(this.I.psi0,QQof,n));


    final double [] QQfti=TMatr.mulMMM(n,QQf,IQQt,QQi);
    this.mAver=TMatr.getAMatrA(psia,TMatr.mulMMM(n,QQf,IQQt,QQImDi),n);
    this.mAverd=this.mAver-TMatr.getAMatrB(psia,this.I.psi0,QQf,n)*TMatr.getAMatrB(psia,this.I.psi0,QQImDi,n);
    this.mAverd_rho=this.mAverd/Math.sqrt(
	(TMatr.getAMatrA(psia,TMatr.mulMMM(n,QQf,IQQt,QQf),n)
	 -Math.pow(TMatr.getAMatrB(psia,this.I.psi0,QQf,n),2))
	*
	(TMatr.getAMatrA(psia,TMatr.mulMMM(n,QQImDi,IQQt,QQImDi),n)
	 -Math.pow(TMatr.getAMatrB(psia,this.I.psi0,QQImDi,n),2))
    );
    this.mAver_a=TMatr.getAMatrA(I.psi0,QQfti,n)-
	this.I.s0*TMatr.getAMatrA(this.I.psi0,QQf,n);
    this.mAver_aa=TMatr.getAMatrA(psia,TMatr.mulMMM(n,QQf,IQQt,QQi),n)-
	this.I.s0*TMatr.getAMatrA(psia,QQf,n);
    this.mA_ospd=TMatr.getAMatrA(psia,QQfti,n)-TMatr.getAMatrB(psia,this.I.psi0,QQf,n)*TMatr.getAMatrB(psia,this.I.psi0,QQi,n);
    this.mA_ospd_rho=this.mA_ospd/Math.sqrt(
	(TMatr.getAMatrA(psia,TMatr.mulMMM(n,QQf,IQQt,QQf),n)
	 -Math.pow(TMatr.getAMatrB(psia,this.I.psi0,QQf,n),2))
	*
	(TMatr.getAMatrA(psia,TMatr.mulMMM(n,QQi,IQQt,QQi),n)
	 -Math.pow(TMatr.getAMatrB(psia,this.I.psi0,QQi,n),2))
    );
    final double [] MQQoDi=TMatr.mulMMM(n,QQf,IQQt,QQoDi);
    this.mAver_oDi=TMatr.getAMatrA(psia,MQQoDi,n);
    this.mAver_oDid=this.mAver_oDi-TMatr.getAMatrB(psia,this.I.psi0,QQf,n)*TMatr.getAMatrB(psia,this.I.psi0,QQoDi,n);
    this.mAver_oDi0=TMatr.getAMatrA(this.I.psi0,MQQoDi,n);
    this.mAver_oDiof=TMatr.getAMatrA(psia,TMatr.mulMMM(n,QQof,IQQt,QQoDi),n);

    double smAI0=0,smAc=0,sigma2_f=0,sigma2_ImDi=0;
    for(int k=0;k<n;k++)
    {
	final double fm=this.I.getFPsiPsi(k,psia,QQf),om=this.I.getFPsiPsi(k,psia,QQo),Dim=this.I.getFPsiPsi(k,psia,QQDi);
	smAI0+=fm*om;
	smAc-=fm*Dim;
	sigma2_f+=fm*fm;
	sigma2_ImDi+=Math.pow(om*this.I.s0-Dim,2);
	//System.err.println("this.I.getFPsiPsi(k,psia,QQDi)="+this.I.getFPsiPsi(k,psia,QQDi));
	//System.err.println("this.I.getFPsiPsi(k,psia,QQo)="+this.I.getFPsiPsi(k,psia,QQo));
    }
    this.mAI0=smAI0;
    this.mAc=smAc;
    this.mAver_rho=(this.mAI0*this.I.s0+this.mAc)/Math.sqrt(sigma2_f*sigma2_ImDi);

    final double [] corr0=getRCorr(this.I.psi0,QQf,QQImDi,QQt,n);
    this.mApsi0=corr0[0];

    Acorr_f_i=ACorr.getACorr_STD(n,fmom,imom,tmom,M);
    Acorr_f_di=ACorr.getACorr_STD(n,fmom,ImDimom,tmom,M);
    Acorr_t_di=ACorr.getACorr_STD(n,tmom,ImDimom,tmom,M);

    acorr_f_i=ACorr.getACorr_F(n,fmom,imom,tmom,M,x0);
    acorr_f_di=ACorr.getACorr_F(n,fmom,ImDimom,tmom,M,x0);

    bcorr_f_i=ACorr.getACorrAverage_Psi(n,fmom,imom,tmom,M,x0);
    bcorr_f_di=ACorr.getACorrAverage_Psi(n,fmom,ImDimom,tmom,M,x0);

    ccorr_f_i=ACorr.getACorrAverage_Psi_and_Basis(n,fmom,imom,tmom,M,x0);
    ccorr_f_di=ACorr.getACorrAverage_Psi_and_Basis(n,fmom,ImDimom,tmom,M,x0);
    ccorr_i_di=ACorr.getACorrAverage_Psi_and_Basis(n,imom,ImDimom,tmom,M,x0);
    ccorr_t_i=ACorr.getACorrAverage_Psi_and_Basis(n,tmom,imom,tmom,M,x0);
    ccorr_t_di=ACorr.getACorrAverage_Psi_and_Basis(n,tmom,ImDimom,tmom,M,x0);
    ccorr_f_t=ACorr.getACorrAverage_Psi_and_Basis(n,fmom,tmom,tmom,M,x0);

    SPcorr_f_i=ACorr.getACorr_SPUR(n,fmom,imom,tmom,M);
    SPcorr_f_di=ACorr.getACorr_SPUR(n,fmom,ImDimom,tmom,M);
    SPcorr_f_odi=ACorr.getACorr_SPUR(n,fmom,oDimom,tmom,M);
    SPrcorr_f_i=ACorr.getACorr_SPUR(n-1,M.getRadauMomentsFromMoments(fmom),M.getRadauMomentsFromMoments(imom),M.getRadauMomentsFromMoments(tmom),M);
    SPrcorr_f_di=ACorr.getACorr_SPUR(n-1,M.getRadauMomentsFromMoments(fmom),M.getRadauMomentsFromMoments(ImDimom),M.getRadauMomentsFromMoments(tmom),M);
    SPrrcorr_f_i=ACorr.getACorr_SPUR(n-1,M.getRadauMomentsFromMoments(M.getRadauMomentsFromMoments(fmom)),M.getRadauMomentsFromMoments(M.getRadauMomentsFromMoments(imom)),M.getRadauMomentsFromMoments(M.getRadauMomentsFromMoments(tmom)),M);
    SPrrcorr_f_di=ACorr.getACorr_SPUR(n-1,M.getRadauMomentsFromMoments(M.getRadauMomentsFromMoments(fmom)),M.getRadauMomentsFromMoments(M.getRadauMomentsFromMoments(ImDimom)),M.getRadauMomentsFromMoments(M.getRadauMomentsFromMoments(tmom)),M);

    this.simom=imom;
    this.stmom=tmom;
    this.sfmom=fmom;
    this.sDimom=Dimom;
    this.sImDimom=ImDimom;
    this.soDimom=oDimom;
    this.spsia=psia;
}
final double [] simom,stmom,sfmom,sDimom,sImDimom,soDimom,spsia;

double [] getPsiA(final int n,final double [] QQt)
{
    final double [] psia=new double [n];
    psia[0]=1;
    psia[0]/=Math.sqrt(TMatr.getAMatrA(psia,QQt,n));
    return psia;
}


void test_Self(final double eps)
{
    test_X0Psi(eps);
    test_ACorr(eps);
    test_bcorr_X0Psi(eps);
    test_ccorr_X0Psi(eps);
    test_SPURcorr(eps);

    final int n=this.I.neq;

    // match moments to BiQVM.getDIMom calculations
    UnitTests.checkDoubleArrayRel(sImDimom,"sImDimom",BiQVM.getDIMoms(n,simom,stmom,access,getX0(),1,false),"BiQVM.getDIMoms",eps);
    UnitTests.checkDoubleArrayRel(soDimom,"soDimom",BiQVM.getDIMoms(n,simom,stmom,access,getX0(),1,true),"BiQVM.getDIMoms",eps);

    final double [] QQt=M.B.getQQMatr(n,stmom),QQi=M.B.getQQMatr(n,simom),QQf=M.B.getQQMatr(n,sfmom),QQImDi=M.B.getQQMatr(n,sImDimom),QQoDi=M.B.getQQMatr(n,soDimom);
    UnitTests.checkDoubleRel(this.I.s0-TMatr.getAMatrA(spsia,QQi,n),"I0-<I>",TMatr.getAMatrA(spsia,QQImDi,n),"<Di>",eps);
    UnitTests.checkDoubleRel(this.I.s0-simom[0],"I0-<I>",sImDimom[0],"sImDimom[0]",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(this.I.psi0,QQoDi,n),"<psi0|QQoDi|psi0>",0,"0",eps);

    double sA=0,sigma2_f=0,sigma2_ImDi=0;
    for(int k=0;k<n;k++)
    {
	final double fm=this.I.getFPsiPsi(k,spsia,QQf),im=this.I.getFPsiPsi(k,spsia,QQImDi);
	sA+=fm*im;
	sigma2_f+=fm*fm;
	sigma2_ImDi+=im*im;
    }
    UnitTests.checkDoubleRel(this.mAver,"mAver",sA,"sA",eps);
    UnitTests.checkDoubleRel(this.mAver,"mAver",this.mAI0*this.I.s0+this.mAc,"mAI0*I0+mAc",eps);
    UnitTests.checkDoubleRel(this.mAver_rho,"mAver_rho",sA/Math.sqrt(sigma2_f*sigma2_ImDi),"sA/Math.sqrt(sigma2_f*sigma2_ImDi)",eps);

    final double [] corr=getRCorr(spsia,QQf,QQImDi,QQt,n);
    UnitTests.checkDoubleRel(this.mAver,"mAver",corr[0],"corr[0]",eps);
    UnitTests.checkDoubleRel(sigma2_f,"sigma2_f",corr[1],"corr[1]",eps);
    UnitTests.checkDoubleRel(sigma2_ImDi,"sigma2_ImDi",corr[2],"corr[2]",eps);


    final int nmoms=2*n-1;
    {
    final double [] omom=stmom;
    final DPIPsi y=new DPIPsi(n,TMatr.sumAcB(simom,this.I.sH,omom),stmom,TMatr.sumAcB(sfmom,1.3*0,omom));
    UnitTests.checkDoubleRel(this.mApsi0,"mApsi0",y.mApsi0,"y.mApsi0",eps);
    UnitTests.checkDoubleRel(this.mAver_oDi,"mAver_oDi",y.mAver_oDi,"y.mAver_oDi",eps);
    // all moments match
    UnitTests.checkDoubleArrayRel(this.soDimom,"this.soDimom",y.soDimom,"y.soDimom",eps);
    }
    {
    final double [] omom=getX0moms(nmoms);
    DPIPsi y=new DPIPsi(n,TMatr.sumAcB(simom,this.I.sH,omom),stmom,TMatr.sumAcB(sfmom,1.3*Math.max(1,Math.abs(sfmom[0])),omom));
    UnitTests.checkDoubleRel(this.mAver_a,"mAver_a",y.mAver_a,"y.mAver_a",eps);
    UnitTests.checkDoubleRel(this.mA_ospd,"mA_ospd",y.mA_ospd,"y.mA_ospd",eps);
    UnitTests.checkDoubleRel(this.mA_ospd_rho,"mA_ospd_rho",y.mA_ospd_rho,"y.mA_ospd_rho",eps);

    ACorr.matchACorr(this.acorr_f_i,y.acorr_f_i,eps);
    ACorr.matchACorr(this.bcorr_f_i,y.bcorr_f_i,eps);
    ACorr.matchACorr(this.ccorr_f_i,y.ccorr_f_i,eps);
    ACorr.matchACorr(this.ccorr_f_di,y.ccorr_f_di,eps);
    ACorr.matchACorr(this.ccorr_i_di,y.ccorr_i_di,eps);
    ACorr.matchACorr(this.SPrcorr_f_i,y.SPrcorr_f_i,eps);
    ACorr.matchACorr(this.SPrrcorr_f_i,y.SPrrcorr_f_i,eps);
    ACorr.matchACorr(this.SPrrcorr_f_di,y.SPrrcorr_f_di,eps);

    // do not match because of delta function derivatives
    System.err.println("o+I0-Dimom="+java.util.Arrays.toString(BiQVM.getDIMoms(n,simom,stmom,access,getX0(),1,true)));
    System.err.println("oo+I0-Dimom="+java.util.Arrays.toString(BiQVM.getDIMoms(n,y.simom,y.stmom,access,getX0(),1,true)));
    //UnitTests.checkDoubleArrayRel(this.soDimom,"this.soDimom",y.soDimom,"y.soDimom",eps);
    //UnitTests.checkDoubleRel(this.mAverd,"mAverd",y.mAverd,"y.mAverd",eps);
    //UnitTests.checkDoubleRel(this.mAver_oDid,"mAver_oDid",y.mAver_oDid,"y.mAver_oDid",eps);

    // Check delta on dp has no effect.
    y=new DPIPsi(n,simom,stmom,TMatr.sumAcB(sfmom,1.3*Math.max(1,Math.abs(sfmom[0])),omom));
    UnitTests.checkDoubleRel(this.mAverd,"mAverd",y.mAverd,"y.mAverd",eps);
    UnitTests.checkDoubleRel(this.mAverd_rho,"mAverd_rho",y.mAverd_rho,"y.mAverd_rho",eps);
    UnitTests.checkDoubleRel(this.mAver_oDid,"mAver_oDid",y.mAver_oDid,"y.mAver_oDid",eps);
    ACorr.matchACorr(this.bcorr_f_di,y.bcorr_f_di,eps);
    ACorr.matchACorr(this.SPrcorr_f_di,y.SPrcorr_f_di,eps);
    UnitTests.checkDoubleRel(this.SPcorr_f_odi.x_a_b,"this.SPcorr_f_odi.x_a_b",y.SPcorr_f_odi.x_a_b,"y.SPcorr_f_odi.x_a_b",eps);


    final X0Psi z=new X0Psi(n,simom,stmom,sfmom);
    UnitTests.checkDoubleRel(this.mAver_a,"mAver_a",z.mA,"y.mA",eps);
    }

    System.err.println("MkQVM:DPIPsi:test_Self: test OK n="+n+" eps="+eps);
}

void test_X0Psi(final double eps)
{
    final int n=this.I.neq;
    final X0Psi x=new X0Psi(n,sImDimom,stmom,sfmom);
    final double [] QQf=M.B.getQQMatr(n,sfmom),QQImDi=M.B.getQQMatr(n,sImDimom),QQt=M.B.getQQMatr(n,stmom),QQi=M.B.getQQMatr(n,simom);
    double sA=0,sAd=0,smA_osp=0,smA_ospd=0,sigma2Id=0,sigma2IId=0,sigma2fd=0;
    final double [] psia=spsia;
    for(int k=0;k<n;k++)
    {
	final double [] psi;
	if(k<(n-1)) psi=x.PsiX0[k];
	else psi=this.I.psi0;
	final double fm=TMatr.getAMatrB(psia,psi,QQf,n),im=TMatr.getAMatrB(psi,psia,QQImDi,n),Im=TMatr.getAMatrB(psi,psia,QQi,n);
	sA+=fm*im;
	smA_osp+=fm*Im;
	if(k<(n-1)){
	    sAd=sA;
	    smA_ospd=smA_osp;
	    UnitTests.checkDoubleRel(M.B.getD0(psi,getX0()),"psi(x0)",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(this.I.psi0,psi,QQt,n),"<psi0|psi>",0,"0",eps);
	    sigma2Id+=Im*Im;
	    sigma2IId+=im*im;
	    sigma2fd+=fm*fm;
	}
	UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,psi,QQt,n),"<psi|psi>",1,"1",eps);
    }
    System.err.println("sA="+sA+" mAver="+this.mAver);
    System.err.println("sAd="+sAd+" mAverd="+this.mAverd+" mAver_oDid="+mAver_oDid);
    UnitTests.checkDoubleRel(sA,"sA",this.mAver,"this.mAver",eps);
    UnitTests.checkDoubleRel(sAd,"sAd",this.mAverd,"this.mAverd",eps);
    UnitTests.checkDoubleRel(sAd,"sAd",this.mAver_oDid,"this.mAver_oDid",eps);
    UnitTests.checkDoubleRel(smA_ospd,"smA_ospd",this.mA_ospd,"this.mA_ospd",eps);

    final double smAverd_rho=sAd/Math.sqrt(sigma2fd*sigma2IId),
	smA_ospd_rho=smA_ospd/Math.sqrt(sigma2fd*sigma2Id);
    UnitTests.checkDoubleRel(smAverd_rho,"smAverd_rho",this.mAverd_rho,"this.mAverd_rho",eps);
    UnitTests.checkDoubleRel(smA_ospd_rho,"smA_ospd_rho",this.mA_ospd_rho,"this.mA_ospd_rho",eps);

    System.err.println("MkQVM:DPIPsi:test_X0Psi: test OK n="+n+" eps="+eps);
}

void test_ACorr(final double eps)
{
    final int n=this.I.neq;
    final ACorr i=ACorr.getACorr_F(n,sImDimom,sfmom,stmom,M,getX0());
    UnitTests.checkDoubleRel(i.x_a_b,"i.x_a_b",this.mAverd,"this.mAverd",eps);
    UnitTests.checkDoubleRel(i.osp,"i.osp",this.mAverd_rho,"this.mAverd_rho",eps);

    final ACorr j=ACorr.getACorr_F(n,simom,sfmom,stmom,M,getX0());
    UnitTests.checkDoubleRel(j.x_a_b,"j.x_a_b",this.mA_ospd,"this.mA_ospd",eps);
    UnitTests.checkDoubleRel(j.osp,"j.osp",this.mA_ospd_rho,"this.mA_ospd_rho",eps);

    System.err.println("MkQVM:DPIPsi:test_ACorr: test OK n="+n+" eps="+eps);
}


void test_bcorr_X0Psi(final double eps)
{
    final int n=this.I.neq;
    final double [] QQf=M.B.getQQMatr(n,sfmom),QQImDi=M.B.getQQMatr(n,sImDimom),QQt=M.B.getQQMatr(n,stmom),QQi=M.B.getQQMatr(n,simom);
    final double [] psi0=this.I.psi0,psiB=new double [n];psiB[0]=1;
    TMatr.sumP1(psiB,psi0,-TMatr.getAMatrB(psiB,psi0,QQt,n));
    TMatr.scaleVectorInPlace(psiB,1/Math.sqrt(TMatr.getAMatrA(psiB,QQt,n)));
    UnitTests.checkDoubleRel(M.B.getD0(psiB,getX0()),"psiB(x0)",0,"0",eps);

    double x_f_f=0,x_i_i=0,x_di_di=0,x_f_i=0,x_f_di=0;
    for(int k=0;k<n;k++)
    {
	final double [] psik=this.I.getPsi(k);
	final double fm=TMatr.getAMatrB(psiB,psik,QQf,n),
	    dim=TMatr.getAMatrB(psiB,psik,QQImDi,n),
	    im=TMatr.getAMatrB(psiB,psik,QQi,n);
	x_f_f+=fm*fm;
	x_i_i+=im*im;
	x_di_di+=dim*dim;
	x_f_i+=fm*im;
	x_f_di+=fm*dim;
    }

    UnitTests.checkDoubleRel(x_f_f,"x_f_f",this.bcorr_f_i.x_a_a,"this.bcorr_f_i.x_a_a",eps);
    UnitTests.checkDoubleRel(x_i_i,"x_i_i",this.bcorr_f_i.x_b_b,"this.bcorr_f_i.x_b_b",eps);
    UnitTests.checkDoubleRel(x_f_i,"x_f_i",this.bcorr_f_i.x_a_b,"this.bcorr_f_i.x_a_b",eps);

    UnitTests.checkDoubleRel(x_f_f,"x_f_f",this.bcorr_f_di.x_a_a,"this.bcorr_f_di.x_a_a",eps);
    UnitTests.checkDoubleRel(x_di_di,"x_di_di",this.bcorr_f_di.x_b_b,"this.bcorr_f_di.x_b_b",eps);
    UnitTests.checkDoubleRel(x_f_di,"x_f_di",this.bcorr_f_di.x_a_b,"this.bcorr_f_di.x_a_b",eps);

    System.err.println("MkQVM:DPIPsi:test_bcorr_X0Psi: test OK n="+n+" eps="+eps);
}


void test_SPURcorr(final double eps)
{
    final int n=this.I.neq;
    final double [] QQf=M.B.getQQMatr(n,sfmom),QQImDi=M.B.getQQMatr(n,sImDimom),QQt=M.B.getQQMatr(n,stmom),QQi=M.B.getQQMatr(n,simom);

    final double [] BQQf=new double [n*n],BQQi=new double [n*n],BQQdi=new double [n*n];

    for(int k=0;k<n;k++)
    {
	final double [] psik=this.I.getPsi(k);
	for(int l=0;l<n;l++)
	{
	    final double [] psil=this.I.getPsi(l);
	    BQQf[k*n+l]=TMatr.getAMatrB(psik,psil,QQf,n);
	    BQQi[k*n+l]=TMatr.getAMatrB(psik,psil,QQi,n);
	    BQQdi[k*n+l]=TMatr.getAMatrB(psik,psil,QQImDi,n);
	}
    }
    final double x_f_f=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][] {BQQf,BQQf})),
	x_f_i=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][] {BQQf,BQQi})),
	x_i_i=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][] {BQQi,BQQi})),
	x_di_di=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][] {BQQdi,BQQdi})),
	x_f_di=TMatr.getSpurM(n,TMatr.mulMatrices(n,new double [][] {BQQf,BQQdi}));

    UnitTests.checkDoubleRel(x_f_f,"x_f_f",this.SPcorr_f_i.x_a_a,"this.SPcorr_f_i.x_a_a",eps);
    UnitTests.checkDoubleRel(x_i_i,"x_i_i",this.SPcorr_f_i.x_b_b,"this.SPcorr_f_i.x_b_b",eps);
    UnitTests.checkDoubleRel(x_f_i,"x_f_i",this.SPcorr_f_i.x_a_b,"this.SPcorr_f_i.x_a_b",eps);

    UnitTests.checkDoubleRel(x_f_f,"x_f_f",this.SPcorr_f_di.x_a_a,"this.SPcorr_f_di.x_a_a",eps);
    UnitTests.checkDoubleRel(x_di_di,"x_di_di",this.SPcorr_f_di.x_b_b,"this.SPcorr_f_di.x_b_b",eps);
    UnitTests.checkDoubleRel(x_f_di,"x_f_di",this.SPcorr_f_di.x_a_b,"this.SPcorr_f_di.x_a_b",eps);

    System.err.println("MkQVM:DPIPsi:test_SPURcorr: test OK n="+n+" eps="+eps);
}


void test_ccorr_X0Psi(final double eps)
{
    final int n=this.I.neq;
    final X0Psi x=new X0Psi(n,sImDimom,stmom,sfmom);
    final double [] QQf=M.B.getQQMatr(n,sfmom),QQImDi=M.B.getQQMatr(n,sImDimom),QQt=M.B.getQQMatr(n,stmom),QQi=M.B.getQQMatr(n,simom);
    final double [] psi0=this.I.psi0,psiB=new double [n];psiB[0]=1;
    TMatr.sumP1(psiB,psi0,-TMatr.getAMatrB(psiB,psi0,QQt,n));
    TMatr.scaleVectorInPlace(psiB,1/Math.sqrt(TMatr.getAMatrA(psiB,QQt,n)));
    UnitTests.checkDoubleRel(M.B.getD0(psiB,getX0()),"psiB(x0)",0,"0",eps);

    double x_f_f=0,x_i_i=0,x_di_di=0,x_f_i=0,x_f_di=0,x_i_di=0;;
    for(int k=0;k<x.PsiX0.length;k++)
    {
	final double [] psik=x.PsiX0[k];
	final double fm=TMatr.getAMatrB(psiB,psik,QQf,n),
	    dim=TMatr.getAMatrB(psiB,psik,QQImDi,n),
	    im=TMatr.getAMatrB(psiB,psik,QQi,n);
	x_f_f+=fm*fm;
	x_i_i+=im*im;
	x_di_di+=dim*dim;
	x_f_i+=fm*im;
	x_f_di+=fm*dim;
	x_i_di+=im*dim;
	UnitTests.checkDoubleRel(M.B.getD0(psik,getX0()),"psik(x0)",0,"0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(psi0,psik,QQt,n),"<psi0|psik>",0,"0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(psik,psik,QQt,n),"<psik|psik>",1,"1",eps);
    }

    UnitTests.checkDoubleRel(x_f_f,"x_f_f",this.ccorr_f_i.x_a_a,"this.ccorr_f_i.x_a_a",eps);
    UnitTests.checkDoubleRel(x_i_i,"x_i_i",this.ccorr_f_i.x_b_b,"this.ccorr_f_i.x_b_b",eps);
    UnitTests.checkDoubleRel(x_f_i,"x_f_i",this.ccorr_f_i.x_a_b,"this.ccorr_f_i.x_a_b",eps);

    UnitTests.checkDoubleRel(x_f_f,"x_f_f",this.ccorr_f_di.x_a_a,"this.ccorr_f_di.x_a_a",eps);
    UnitTests.checkDoubleRel(x_di_di,"x_di_di",this.ccorr_f_di.x_b_b,"this.ccorr_f_di.x_b_b",eps);
    UnitTests.checkDoubleRel(x_f_di,"x_f_di",this.ccorr_f_di.x_a_b,"this.ccorr_f_di.x_a_b",eps);

    UnitTests.checkDoubleRel(x_i_i,"x_i_i",this.ccorr_i_di.x_a_a,"this.ccorr_i_di.x_a_a",eps);
    UnitTests.checkDoubleRel(x_di_di,"x_di_di",this.ccorr_i_di.x_b_b,"this.ccorr_i_di.x_b_b",eps);
    UnitTests.checkDoubleRel(x_i_di,"x_i_di",this.ccorr_i_di.x_a_b,"this.ccorr_i_di.x_a_b",eps);

    System.err.println("MkQVM:DPIPsi:test_ccorr_X0Psi: test OK n="+n+" eps="+eps);
}


} // END OF class DPIPsi




class RX0Psi {
final EVXData J;
final double sens_fJ_L,sens_fJ_H;
RX0Psi(final int n,final double [] imom,final double [] tmom,final double [] fmom)
{
    final double [] Rimom=M.getRadauMomentsFromMoments(imom),
	Rtmom=M.getRadauMomentsFromMoments(tmom),
	Rfmom=M.getRadauMomentsFromMoments(fmom);
    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][]{Rimom,Rtmom,Rfmom});
    final double [] QQi=QQ[0],QQt=QQ[1],QQf=QQ[2];
    final double x0=getX0();
    this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);
    this.sens_fJ_L=getD1GRM(0,this.J,QQf);
    this.sens_fJ_H=getD1GRM(n-1,this.J,QQf);

    this.simom=imom;
    this.stmom=tmom;
    this.sfmom=fmom;
}
final double [] simom,stmom,sfmom;

double getD1GRM(final int ind_lev,final EVXData j,final double [] QQ)
{
    final double [] psi=j.getPsi(ind_lev),pert=DeprecatedHelperEVXData.getPsiPertr(j,ind_lev,QQ);
    final double [] psim=M.getRadauMultiplied(psi);
    return getK2(pert,psim,psi,psim)-getK2(psi,psim,pert,psim);
}

void test_Self(final double eps)
{
    final int n=this.J.neq;
    final double x0=getX0();

    final double [] psi=this.J.getPsi(ind_lev),psim=M.getRadauMultiplied(psi);
    UnitTests.checkDoubleRel(getK2(psi,psim,psi,psim),"<psi2|psi2>",0.5,"0.5",eps);

    System.err.println("MkQVM:RX0Psi:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class RX0Psi





boolean isEdgeAtRight(){return true;}
abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getX0();
abstract double getPOffset();
abstract double getPLast();
abstract double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb);
abstract double [] getEDPsi(final double [] psi);
final VolMatrix VOLATILITY_EDPsi=this::getEDPsi;

double [][] getMoments(final int nmoms)
{
    final double [] Ptmom=new double [nmoms],dpmom=new double [nmoms],tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms];

    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setFMoments(Ptmom,dpmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    access.setMT(tmom,nmoms);
    final double [] pimomo=TMatr.sumAcB(pimom,getPOffset()-getPLast(),imom);

    return new double [][]{tmom,imom,Ptmom,pimom,dpmom,pimomo};
}


static abstract class MkQVMTest<DTYPE> extends QTest {

abstract MkQVM<DTYPE> getMkQVM();

abstract DkQVM<?> getDkQVM();

MkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}

static void testComplimentatyPertPsi(final MkQVM<?>.PertPsi a,final MkQVM<?>.PertPsi b,final double eps)
{
    EVXData.matchEVXData(a.J,b.J,true,true,eps);
    final int n=a.J.neq;
    final double [] sum=DeprecatedHelperTMatr.sumAB(n,a.senspert,b.senspert),o=new double [n];
    System.err.println("a.senspert="+java.util.Arrays.toString(a.senspert));
    System.err.println("sum="+java.util.Arrays.toString(sum));
    UnitTests.checkDoubleArrayRel(sum,"sum",o,"0",eps);
    final double [] sumf=DeprecatedHelperTMatr.sumAB(n,a.fpert,b.fpert);
    UnitTests.checkDoubleArrayRel(a.J.getEValues(),"J.getEValues()",sumf,"sumf",eps);
    for(int k=0;k<n;k++)
    {
	UnitTests.checkDoubleRel(sum[k],"sum["+k+"]",0,"0",eps);
	UnitTests.checkDoubleRel(a.J.ev[k],"a.J.ev["+k+"]",sumf[k],"sumf[k]",eps);
	UnitTests.checkDoubleRel(b.J.ev[k],"b.J.ev["+k+"]",sumf[k],"sumf[k]",eps);
    }
    System.err.println("MkQVMTest:testComplimentatyPertPsi(): test OK n="+n+" eps="+eps);
}

void test_Common()
{
    final MkQVM<DTYPE> q=getMkQVM();
    q.distance.test_Self(eps);

    final int n=q.I.neq;
    UnitTests.checkDoubleRel(TMatr.getAMatrB(q.pertpsiI,q.psiI,q.sQQt,n),"<pertpsiI|QQt|psiI>",0,"0",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(q.pertpsiI,q.psiI,q.sQQi,n),"<pertpsiI|QQi|psiI>",0,"0",eps);
    //UnitTests.checkDoubleRel(TMatr.getAMatrA(q.pertpsiI,q.sQQt,n),"<pertpsiI|QQt|pertpsiI>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(q.psiI,q.sQQi,n),"<psiI|QQi|psiI>",q.I.ev[q.ind_lev],"q.I.ev[q.ind_lev]",eps);
    UnitTests.checkDoubleRel(2*(TMatr.getAMatrB(q.psiI,q.getEDPsi(q.psiI),q.sQQpi,n)-
				TMatr.getAMatrB(q.psiI,q.psiI,q.sQQpi,n)*TMatr.getAMatrB(q.psiI,q.getEDPsi(q.psiI),q.sQQt,n)),"var(<psiI|QQpi|psiI>)",q.varPi,"q.varPi",eps);


    System.err.println("q.selection="+java.util.Arrays.toString(q.selection));
    final double [] psi=new double [n];
    final double [][] rqis=new double [n][],rqpts=new double [n][],rqpis=new double [n][],rqdps=new double [n][];
    for(int k=0;k<n;k++)
    {
	System.arraycopy(q.I.ev,n+k*n,psi,0,n);
	final double [] dpsi=q.getEDPsi(psi);
	final double [] rq=rqis[k]=RayleighQuotient.getRQVariationValues(psi,dpsi,null,q.sQQi,q.sQQt,n);
	rqpts[k]=RayleighQuotient.getRQVariationValues(psi,dpsi,null,q.sQQpt,q.sQQt,n);
	rqpis[k]=RayleighQuotient.getRQVariationValues(psi,dpsi,null,q.sQQpi,q.sQQt,n);
	rqdps[k]=RayleighQuotient.getRQVariationValues(psi,dpsi,null,q.sQQdp,q.sQQt,n);
	UnitTests.checkDoubleRel(rq[0],"rq[0]",q.I.ev[k],"q.I.ev[k]",eps);
	UnitTests.checkDoubleRel(rq[1],"rq[1]",0,"0",eps);
	final double var2=2*(TMatr.getAMatrA(dpsi,q.sQQi,n)-q.I.ev[k]*TMatr.getAMatrA(dpsi,q.sQQt,n));
	UnitTests.checkDoubleRel(var2,"var2",rq[2],"rq[2]",eps);
	System.err.println("k="+k+"\tI="+q.I.ev[k]+"\trq="+java.util.Arrays.toString(rq));
	if(ReadWriteUtils.hasElement(q.selection,k)?(!(rq[2]<0)):(!(rq[2]>0)))
	{
	    throw new RuntimeException("k="+k+" rq[2]="+rq[2]+" incorrect sign for this selection");
	}
    }

    UnitTests.checkDoubleRel(
	q.getD1G(q.selection[q.distance.t_order[0]],q.I,q.sQQpi),"d1G from QQpi",
	q.d1G,"d1G",eps);

    final int [] sortedorignums=new int [q.distance.t_order.length];
    for(int k=0;k<q.distance.t_order.length;k++)
    {
	final int l=sortedorignums[k]=q.selection[q.distance.t_order[k]];
	UnitTests.checkDoubleRel(q.I.ev[l],"q.I.ev[l]",q.distance.I_distancesorted[k],"q.distance.I_distancesorted[k]",eps);
	UnitTests.checkDoubleRel(rqpts[l][0],"rqpts[l][0]",q.distance.p_distancesorted[k],"q.distance.p_distancesorted[k]",eps);
	UnitTests.checkDoubleRel(rqdps[l][0],"rqdps[l][0]",q.distance.dp_distancesorted[k],"q.distance.dp_distancesorted[k]",eps);
	UnitTests.checkDoubleRel(rqpts[l][1],"rqpts[l][1]",q.distance.varPt_distancesorted[k],"q.distance.varPt_distancesorted[k]",eps);
	UnitTests.checkDoubleRel(rqpis[l][1],"rqpis[l][1]",q.distance.varPi_distancesorted[k],"q.distance.varPi_distancesorted[k]",eps);
    }
    System.err.println("sortedorignums="+java.util.Arrays.toString(sortedorignums));

    testComplimentatyPertPsi(q.peVS,q.peVB,eps);
    testComplimentatyPertPsi(q.PeVS,q.PeVB,eps);
    testComplimentatyPertPsi(q.PVeVS,q.PVeVB,eps);
    testComplimentatyPertPsi(q.pePLUS,q.peMINUS,eps);
    if(true)
    {
	testComplimentatyPertPsi(q.PePLUS,q.PeMINUS,eps);
	final double [] sens05=TMatr.getVectorScaled(q.Pe.senspert,0.5);
	for(int k=0;k<n;k++)
	{
	    UnitTests.checkDoubleRel(q.PePLUS.senspert[k],"q.PePLUS.senspert["+k+"]",sens05[k],"sens05[k]",eps);
	    UnitTests.checkDoubleRel(q.PeMINUS.senspert[k],"q.PeMINUS.senspert["+k+"]",-sens05[k],"-sens05[k]",eps);
	}
    }

    q.itPvL.test_Self(eps);
    q.aip.test_Self(eps);
    q.tip.test_Self(eps);
    q.itp.test_Self(eps);
    EVXData.matchEVXData(q.tip.J,q.Itip.J,true,true,eps);
    UnitTests.checkDoubleArrayRel(q.tip.f,"q.tip.f",q.Itip.f,"q.Itip.f",eps);

    q.Ai.test_Self(eps);
    q.ai.test_Self(eps);
    q.AV.test_Self(eps);
    q.aV.test_Self(eps);
    q.itPi.test_Self(eps);
    UnitTests.checkDoubleArrayRel(q.itPi.varFn,"q.itPi.varFn",q.itPi.getD0Pert(),"q.itPi.getD0Pert()",eps);
    q.itPt.test_Self(eps);
    UnitTests.checkDoubleArrayRel(q.itPi.senspsi,"q.itPi.senspsi",q.itPt.senspsi,"q.itPt.senspsi",eps);
    UnitTests.checkDoubleArrayRel(q.itPi.var2j,"q.itPi.var2j",q.itPt.var2j,"q.itPt.var2j",eps);
    q.itPm.test_Self(eps);
    q.itPm_sam.test_Self(eps);
    UnitTests.checkDoubleArrayRel(q.itPm_sam.varFn,"q.itPm_sam.varFn",q.itPm_sam.getD0Pert(),"q.itPm_sam.getD0Pert()",eps);
    // q.PtI.test_Self(eps); disable, may fail because of degeneracy

    q.itPvF.test_Self(eps);
    UnitTests.checkDoubleRel(q.d1G_IL,"q.d1G_IL",q.itPvF.senspsi[0],"q.itPvF.senspsi[0]",eps);
    UnitTests.checkDoubleRel(q.d1G_IH,"q.d1G_IH",q.itPvF.senspsi[n-1],"q.itPvF.senspsi[n-1]",eps);

    q.Rip.test_Self(eps);
    q.itdp.test_Self(eps);
    q.itpi.test_Self(eps);
    q.itpt.test_Self(eps);
    q.itVs.test_Self(eps);
    q.itSVs.test_Self(eps);

    q.Ritpi.test_Self(eps);
    q.Ritpt.test_Self(eps);

    q.pipsi.test_Self(eps);
    UnitTests.checkDoubleRel(q.itpt.mA,"q.itpt.mA",q.pipsi.mA0,"q.pipsi.mA0",eps);

    q.dpipsi.test_Self(eps);
    q.vsipsi.test_Self(eps);

    q.eePiIt.test_Self(eps);
    q.eePit.test_Self(eps);
    q.eeVsit.test_Self(eps);
    q.eeVsit.match_EiComm(q.eiVsit,eps);
    q.eeVsitL.test_Self(eps);
    q.eeVsitH.test_Self(eps);
    //q.eepVspit.test_Self(eps);
    q.eepVspit.match_EiComm(q.eipVsit,eps);

    q.edVsit.test_Self(eps);
    q.edVsit.test_IRadau_MkQVM_X0Psi_Psix0(q.itpt.PsiX0,q.itpt.I.psi0,eps);
    q.edpVsit.test_Self(eps);
    q.eiVsit.test_Self(eps);
    q.eipVsit.test_Self(eps);

    System.err.println(name+":MkQVMTest:test_Common(): test OK neq="+n+" eps="+eps);
}

void test_DkQVM()
{
  final MkQVM<DTYPE> q=getMkQVM();
  final DkQVM<?> s=getDkQVM();

  UnitTests.checkDoubleRel(q.dpipsi.mA_ospd,"q.dpipsi.mA_ospd",s.acorr_dp_i.x_a_b,"s.acorr_dp_i.x_a_b",eps);
  UnitTests.checkDoubleRel(q.dpipsi.mA_ospd_rho,"q.dpipsi.mA_ospd_rho",s.acorr_dp_i.osp,"s.acorr_dp_i.osp",eps);

  UnitTests.checkDoubleRel(q.dpipsi.mAverd,"q.dpipsi.mAverd",s.acorr_dp_di.x_a_b,"s.acorr_dp_di.x_a_b",eps);
  UnitTests.checkDoubleRel(q.dpipsi.mAverd_rho,"q.dpipsi.mAverd_rho",s.acorr_dp_di.osp,"s.acorr_dp_di.osp",eps);

  System.err.println(name+":MkQVMTest:test_DkQVM(): test OK neq="+neq+" eps="+eps);
}




void test_Extra(){}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":MkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_DkQVM();
    test_Extra();

    System.err.println(name+":MkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":MkQVMTest: starting tests; ver=$Id: MkQVM.java,v 1.249 2022/11/20 12:41:33 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":MkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: MkQVM.java,v 1.249 2022/11/20 12:41:33 mal Exp $");
}
} // END OF class MkQVMTest




}// END OF class MkQVM
