package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: MkQVMLaguerre.java,v 1.21 2022/07/10 10:41:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class MkQVMLaguerre extends MkQVM<QVMDataL> {
MkQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}


public static MkQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final MkQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*(neq-1)+1+3,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new MkQVMLaguerre(neq,o);
}
boolean isEdgeAtRight(){return false;}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return KkQVMLaguerre._getK2(a,aa,b,bb);}
double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}
double getSignDir(){return -1;}

static class MkQVMLaguerreTest extends MkQVMTest<QVMDataL> {
MkQVMLaguerreTest(final long initrandom)
{
    super("MkQVMLaguerre",new java.util.Random(initrandom),1e-2);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
MkQVMLaguerre getMkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
DkQVMLaguerre getDkQVM(){return DkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class MkQVMLaguerreTest

public static void main(String [] args)
{
    new MkQVMLaguerreTest(1857L).runTests(6);
} // END main



} // END OF class MkQVMLaguerre
