package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: MkQVMMonomials.java,v 1.24 2022/07/10 10:41:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class MkQVMMonomials extends MkQVM<QVMData> {
MkQVMMonomials(final int neq,final QVMData odata){super(neq,odata);}


public static MkQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final MkQVMMonomials ref)
{
    final int nvm=1,npm=3;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1+3,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new MkQVMMonomials(neq,o);
}
DeprecatedHelperQVMDataDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataDirectAccessWithBasis();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return KkQVMMonomials._getK2(a,aa,b,bb);}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}


static class MkQVMMonomialsTest extends MkQVMTest<QVMData> {
MkQVMMonomialsTest(final long initrandom)
{
    super("MkQVMMonomials",new java.util.Random(initrandom),1e-2);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
MkQVMMonomials getMkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
MkQVMLaguerre getMkQVMTest(){return MkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
DkQVMLaguerre getDkQVM(){return DkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

void test_Extra()
{
    final MkQVMMonomials q=getMkQVM();
    final MkQVMLaguerre s=getMkQVMTest();
    EVXData.matchEVXData(q.I,s.I,false,true,eps);

    UnitTests.checkDoubleArrayRel(q.distance.I_distancesorted,"q.distance.I_distancesorted",s.distance.I_distancesorted,"s.distance.I_distancesorted",eps);
    UnitTests.checkDoubleArrayRel(q.distance.dp_distancesorted,"q.distance.dp_distancesorted",s.distance.dp_distancesorted,"s.distance.sp_distancesorted",eps);
    UnitTests.checkDoubleArrayRel(q.distance.varPt_distancesorted,"q.distance.varPt_distancesorted",s.distance.varPt_distancesorted,"s.distance.varPt_distancesorted",eps);
    UnitTests.checkDoubleArrayRel(q.distance.varPi_distancesorted,"q.distance.varPi_distancesorted",s.distance.varPi_distancesorted,"s.distance.varPi_distancesorted",eps);

    UnitTests.checkDoubleRel(q.d1G,"q.d1G",s.d1G,"s.d1G",eps);
    UnitTests.checkDoubleRel(q.d2G,"q.d2G",s.d2G,"s.d2G",eps);
    UnitTests.checkDoubleRel(q.varPi,"q.varPi",s.varPi,"s.varPi",eps);

    EVXData.matchEVXData(q.pipsi.I,s.pipsi.I,false,true,eps);
    EVXData.matchEVXData(q.pipsi.P,s.pipsi.P,false,true,eps);

    UnitTests.checkDoubleRel(q.eePit.Dsens,"q.eePit.Dsens",s.eePit.Dsens,"q.eePit.Dsens",eps);
    UnitTests.checkDoubleRel(q.eePit.Dsens_rho,"q.eePit.Dsens_rho",s.eePit.Dsens_rho,"q.eePit.Dsens_rho",eps);
    UnitTests.checkDoubleRel(q.eeVsit.Dsens,"q.eeVsit.Dsens",s.eeVsit.Dsens,"q.eeVsit.Dsens",eps);
    UnitTests.checkDoubleRel(q.eeVsit.Dsens_rho,"q.eeVsit.Dsens_rho",s.eeVsit.Dsens_rho,"q.eeVsit.Dsens_rho",eps);

    System.err.println(name+":MkQVMTest:test_Extra(): test OK neq="+neq+" eps="+eps);
}

} // END of class MkQVMMonomialsTest

public static void main(String [] args)
{
    new MkQVMMonomialsTest(1117L).runTests(5);
} // END main



} // END OF class MkQVMMonomials
