package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
/**
 *  @version $Id: MomentsData.java,v 1.24 2022/06/17 20:37:51 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */

public class MomentsData {
    public final int n;
    public final double p_offset,p_last;
    public final double [] QQi,QQt,QQpi,QQp2i,QQp3i,QQDpwv,QQT,QQV,QQpV,QQp2V,QQp3V,QQdp;
    public MomentsData(final int n,final double p_offset,final double p_last,
		final double [] QQi,final double [] QQt,final double [] QQpi,final double [] QQp2i,final double [] QQp3i,final double [] QQDpwv,
		final double [] QQT,final double [] QQV,final double [] QQpV,final double [] QQp2V,final double [] QQp3V,
		final double [] QQdp)
    {
	this.n=n;
	this.p_offset=p_offset;
	this.p_last=p_last;
	this.QQi=QQi;
	this.QQt=QQt;
	this.QQpi=QQpi;
	this.QQp2i=QQp2i;
	this.QQp3i=QQp3i;
	this.QQDpwv=QQDpwv;
	this.QQT=QQT;
	this.QQV=QQV;
	this.QQpV=QQpV;
	this.QQp2V=QQp2V;
	this.QQp3V=QQp3V;
	this.QQdp=QQdp;
    }

/** Calculate the moments in time basis.
 * QQi_jk={@literal <Q_j I Q_k>}, QQt_jk={@literal <Q_j Q_k>}, QQpi_jk={@literal <Q_j pI Q^k>},
 * QQV_jk={@literal <Q_j V Q_k>}, QQpV_jk={@literal <Q_j C Q_k>} (capital C=\int pdV)
 */
public static <DTYPE> MomentsData get_MomentsData(final int n,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final DTYPE odata,final DataInterfaceToMoments<DTYPE> access)
{
    final int nmoms=2*n-1;
    final double [] mom=new double [nmoms],mmom=new double [nmoms];
    access.setMT(mom,nmoms);final double [] QQt=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQT=M.B.getQQMatr(n,mmom);
    access.setM(mom,0,access.M_I,odata,nmoms);final double [] QQi=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQV=M.B.getQQMatr(n,mmom);
    access.setM(mom,1,access.M_I,odata,nmoms);final double [] QQpi=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQpV=M.B.getQQMatr(n,mmom);
    access.setM(mom,2,access.M_I,odata,nmoms);final double [] QQp2i=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQp2V=M.B.getQQMatr(n,mmom);
    access.setM(mom,3,access.M_I,odata,nmoms);final double [] QQp3i=M.B.getQQMatr(n,mom);
    access.setFMoments(mmom,mom,nmoms);final double [] QQp3V=M.B.getQQMatr(n,mmom);
    access.setM(mom,1,access.M_DP,odata,nmoms);final double [] QQdp=M.B.getQQMatr(n,mom);
    access.setM(mom,0,access.M_DPwV,odata,nmoms);final double [] QQDpwv=M.B.getQQMatr(n,mom);
    return new MomentsData(n,access.getPOffset(odata),access.getPLast(odata),QQi,QQt,QQpi,QQp2i,QQp3i,QQDpwv,QQT,QQV,QQpV,QQp2V,QQp3V,QQdp);
}

/** Calculate moments in p^k basis. QQi_jk={@literal <p^j I p^k>},
 * QQt_jk={@literal <p^j p^k>}, QQpi_jk={@literal <p^j pI p^k>}.
 * Does not calculate QQV and QQpV. See the other get_PriceMonomials_MomentsData
 * method for the calculation of QQV_jk={@literal <p^j V p^k>}.
 */
public static <DTYPE> MomentsData get_PriceMonomials_MomentsData(final int n,final DTYPE odata,final DataInterfaceToMoments<DTYPE> access,final OrthogonalPolynomialsMonomialsBasis M)
{
    final int nmoms=2*n+2;
    final double [] r=new double [1],imom=new double [nmoms],tmom=new double [nmoms],
	Dpwv_mom=new double [nmoms],dpmom=new double [nmoms-1];
    for(int k=0;k<nmoms;k++)
    {
	access.setM(r,k,access.M_T,odata,1);tmom[k]=r[0];
	access.setM(r,k,access.M_I,odata,1);imom[k]=r[0];
	access.setM(r,k,access.M_DPwV,odata,1);Dpwv_mom[k]=r[0];
	if(k>0)
	{
	    access.setM(r,k,access.M_DP,odata,1);
	    // In monomials basis: d(p^k)=kp^{k-1}dp.
	    dpmom[k-1]=r[0]/k;
	    if(false)
	    {
		final double [] R=new double [1];
		access.setFMoments(R,r,1);
		System.err.println("k="+k+" tmom[k]="+tmom[k]+" R[0]="+R[0]+" should match approximately if p_offset=p_last");
	    }
	}
    }
    final double [] pi=M.B.getXQkMomentsFromQkMoments(imom),p2i=M.B.getXQkMomentsFromQkMoments(pi);
    return new MomentsData(n,access.getPOffset(odata),access.getPLast(odata),
			   M.B.getQQMatr(n,imom),
			   M.B.getQQMatr(n,tmom),
			   M.B.getQQMatr(n,pi),
			   M.B.getQQMatr(n,p2i),
			   M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(p2i)),
			   M.B.getQQMatr(n,Dpwv_mom),
			   null, // for now not available
			   null,
			   null,
			   null,
			   null,
			   M.B.getQQMatr(n,dpmom));
}

/** Calculate moments in p^k basis.  The QQV_jk={@literal <p^j V p^k>}  is correct,
 * but QQpV is incorrect, calculated QQpV_jk={@literal <p^j pV p^k>},
 * not QQpV_jk={@literal <p^j C p^k>} (where the capital C=\int pdV) as it supposed to be.
 */
public static MomentsData get_PriceMonomials_MomentsData(final int n,final QVMData odata,final OrthogonalPolynomialsMonomialsBasis M)
{
    final QVMDataDirectAccess access=ACCESS_QVMData;
    final int nmoms=2*n+2;
    final double [] r=new double [1],imom=new double [nmoms],tmom=new double [nmoms],
	Dpwv_mom=new double [nmoms],Ttmom=new double [nmoms],Vtmom=new double [nmoms],dpmom=new double [nmoms-1];
    if(odata.nvm<2 || odata.ntm<1 || odata.npm<nmoms) throw new IllegalArgumentException("odata.nvm="+odata.nvm+" odata.ntm="+odata.ntm);
    for(int k=0;k<nmoms;k++)
    {
	access.setM(r,k,access.M_T,odata,1);tmom[k]=r[0];
	access.setM(r,k,access.M_I,odata,1);imom[k]=r[0];
	access.setM(r,k,access.M_DPwV,odata,1);Dpwv_mom[k]=r[0];
	// \int p^k V omega(t)dt, element [np=k][nt=0][nv=1]
	Vtmom[k]=odata.v_scale*odata.tmoments_vol_x_tau_x_p[k+odata.npm*0+odata.npm*odata.ntm*1];
	// \int p^k T omega(t)dt, element [np=k][nt=1][nv=0]
	Ttmom[k]=odata.tmoments_vol_x_tau_x_p[k+odata.npm*1+odata.npm*odata.ntm*0];
	if(k>0)
	{
	    access.setM(r,k,access.M_DP,odata,1);
	    // In monomials basis: d(p^k)=kp^{k-1}dp.
	    dpmom[k-1]=r[0]/k;
	    if(false)
	    {
		final double [] R=new double [1];
		access.setFMoments(R,r,1);
		System.err.println("k="+k+" tmom[k]="+tmom[k]+" R[0]="+R[0]+" should match approximately if p_offset=p_last");
	    }
	}
    }
    final double [] pi=M.B.getXQkMomentsFromQkMoments(imom),p2i=M.B.getXQkMomentsFromQkMoments(pi);
    return new MomentsData(n,access.getPOffset(odata),access.getPLast(odata),
			   M.B.getQQMatr(n,imom),
			   M.B.getQQMatr(n,tmom),
			   M.B.getQQMatr(n,pi),
			   M.B.getQQMatr(n,p2i),
			   M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(p2i)),
			   M.B.getQQMatr(n,Dpwv_mom),
			   M.B.getQQMatr(n,Ttmom),
			   M.B.getQQMatr(n,Vtmom),
			   // This is incorrect, this NOT the \int p dv, but p \int dv.
			   M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(Vtmom)),
			   null,
			   null,
			   M.B.getQQMatr(n,dpmom));
}
private static final QVMDataDirectAccess ACCESS_QVMData=new QVMDataDirectAccess();

}
