package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.ProbabilityCorrelation;
/**
 * I(p) n=2 interpolation version.
 * @version $Id: Mu2.java,v 1.28 2022/07/28 12:38:06 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class Mu2 {
/** To calculate I(p), I'p, I''p.
 */
public final com.polytechnik.utils.DerivativesCalculateable Ip;
public final double IL,IH,root_psiL,root_psiH,pa,pm,DIpDpm,pc_weight,qsk;
public final ProbabilityCorrelation pc;
final double [] psiL,psiH;
Mu2(final double p0t,final double p1t,final double p2t,
    final double p0i,final double p1i,final double p2i,
    final double eps)
{
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=
	com.polytechnik.utils.DiffSkewness.MB;
    final int d=2;
    final double [] Mi=M.B.getQQMatr(d,new double []{p0i,p1i,p2i}),
	Mt=M.B.getQQMatr(d,new double []{p0t,p1t,p2t});
    final double [] ev=M.EV.getGEV_Generalized(d,Mi,Mt,true); // eigenvalues problem
    psiL=EVXData.getPsiFromBasis(0,ev,d);psiH=EVXData.getPsiFromBasis(d-1,ev,d);
    final double [] psiL2=M.B.mult2Pol(psiL,psiL),psiH2=M.B.mult2Pol(psiH,psiH);
    this.Ip=new com.polytechnik.utils.PolynomialsRatio(
	DeprecatedHelperTMatr.sumAB(psiL2.length,psiL2,psiH2),
	TMatr.sumaAbB(ev[0],psiL2,ev[d-1],psiH2),M.B);
    IL=ev[0];IH=ev[d-1];root_psiL=-M.B.getD0(psiL,0)/M.B.getD1(psiL,0);root_psiH=-M.B.getD0(psiH,0)/M.B.getD1(psiH,0);pa=Mt[1]/Mt[0];
    pm=(Math.abs(psiH[1])*root_psiH+Math.abs(psiL[1])*root_psiL)/(Math.abs(psiL[1])+Math.abs(psiH[1]));
    this.qsk=(2*pm-root_psiL-root_psiH)/Math.abs(root_psiL-root_psiH);
    DIpDpm=Ip.getDF(pm); // DIpDpm=pc.corr*pc_weight
    final double [] DPpm=new double []{0,1,1,2*pm};
    this.pc=new ProbabilityCorrelation(Mt,DPpm,Mi,M,true);
    this.pc_weight=0.5*(pc.iL-pc.iH)*(pc.pL-pc.pH)/(Math.pow(M.B.getD0(psiL,pm),2)+Math.pow(M.B.getD0(psiH,pm),2));
    if(eps>0)
    {
	UnitTests.checkDoubleRel(Ip.getF(pa),"Ip(pa)",Mi[0]/Mt[0],"Mi[0]/Mt[0]",eps);
	UnitTests.checkDoubleRel(Ip.getF(pm),"Ip(pm)",(ev[0]+ev[1])/2,"(ev[0]+ev[1])/2",eps);
	final double Iinf=(Mi[3]+pa*(-2*Mi[1]+pa*Mi[0]))/(Mt[3]-pa*pa*Mt[0]);
	UnitTests.checkDoubleRel(Ip.getF(1/(eps*eps)),"I(inf)",Iinf,"Iinf",eps);
	UnitTests.checkDoubleRel(Ip.getF(root_psiH),"I(root_psiH)",IL,"IL",eps);
	UnitTests.checkDoubleRel(Ip.getF(root_psiL),"I(root_psiL)",IH,"IH",eps);
	UnitTests.checkDoubleRel(Ip.getDF(root_psiL),"I'(root_psiL)",0,"0",eps);
	UnitTests.checkDoubleRel(Ip.getDF(root_psiH),"I'(root_psiH)",0,"0",eps);
	UnitTests.checkDoubleRel(DIpDpm,"I'(pm)",pc.corr*pc_weight,"pc.corr*pc_weight",eps);
	//UnitTests.checkDoubleRel(qsk,"qsk",pc.corr,"pc.corr",eps); // not correct!
	final com.polytechnik.utils.Skewness sk0pm=
	    com.polytechnik.utils.Skewness.getG0Skewness(Mt,Mi,M,pm);
	UnitTests.checkDoubleRel(sk0pm.sk,"sk0pm.sk",0,"0",eps);
	System.err.println("Mu2: test OK, Ip.getF(pa)="+Ip.getF(pa)+" Ip.getDF(pa)="+Ip.getDF(pa)+
			   " Ip.getDF(pm)="+Ip.getDF(pm)+" (ev[0]-ev[1])/(r0-r1)="+((ev[0]-ev[1])/(root_psiL-root_psiH))+
			   " Ip.getD2F(pa)="+Ip.getD2F(pa)+" Ip.getD2F(pm)="+Ip.getD2F(pm)+" eps="+eps);
    }
}
}
