package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.SortArrays;

/**
 * Try dmu measure symmetrically (on psiL &amp; psiH).
 * @version $Id: Mu2DynMicroImpact.java,v 1.37 2022/11/16 19:32:42 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class Mu2DynMicroImpact {
    public final EVXData I;
    public final double p_offset,p_IH,pt_IH,pnl_ILtoIH,vol_pnl;
    public final Mu muL,muH;

    Mu2DynMicroImpact(final IMomentsData mm,
		      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		      final double eps)
{
    final int n=mm.n,ixH=n-1;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    this.I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    this.p_offset=mm.p_offset;
    p_IH=I.getFPsi(ixH,mm.QQpi)/I.ev[ixH];
    muL=new Mu(I.psiL,I.sL,mm.QQi,mm.QQt,mm.QQpi,mm.QQi,M,eps);
    muH=new Mu(I.psiH,1/I.sH,mm.QQt,mm.QQi,mm.QQpt_sample,mm.QQt_sample,M,eps);
    pt_IH=muH.p_lam;
    final PnLMuPoints p=new PnLMuPoints(new PnLMuPoints.CMP(muL,muH,M));
    this.pnl_ILtoIH=p.Cend;
    this.vol_pnl=p.Voltr;

    if(eps>0)
    {
	UnitTests.checkDoubleRel(p.Vend,"p.Vend",0,"0",eps);
	UnitTests.checkDoubleRel1(p.Cend,"p.Cend",TMatr.sumElements(p.pVol),"sum(p.pVol)",eps);
	UnitTests.checkDoubleRel(0,"0",TMatr.sumElements(p.Vol),"sum(p.Vol)",eps);
	UnitTests.checkDoubleRel(1,"1",TMatr.sumElements(p.wL),"sum(p.wL)",eps);
	UnitTests.checkDoubleRel(1,"1",TMatr.sumElements(p.wH),"sum(p.wH)",eps);
	System.err.println("p.Vol="+java.util.Arrays.toString(p.Vol)+" p.pVol="+java.util.Arrays.toString(p.pVol)+" p.X="+java.util.Arrays.toString(p.X));
	System.err.print("Mu2DynMicroImpact.ORDER:");
	for(int ptrk:p.ptr) System.err.print(p.c.isL(ptrk)?"L":"H");
	System.err.println(": roots muL.x="+java.util.Arrays.toString(muL.x)+" muH.x="+java.util.Arrays.toString(muL.x));
	final java.util.List<Double> l=java.util.stream.IntStream.of(p.ptr).mapToDouble(j -> p.c.getX(j)).boxed().toList();
	System.err.println("sorted roots="+l);
	for(int j=0;j<p.X.length;j++)
	{
	    UnitTests.checkDoubleRel(p.X[j],"p.X[j]",l.get(j).doubleValue(),"l(j)",eps*0);
	}
    }
}



static class Mu extends PnLMuPoints.RMu {
    final double lam,p_lam,p_Mu;

    Mu(final double [] psi,final double lam,final double [] QQa,final double [] QQb,
       final double [] QQpa,final double [] QQpa_scale,
       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final double eps)
    {
	super(M.getXNodes(psi).re_root);
	final int n=psi.length;
	this.lam=lam;
	// Find the roots of |psi>.
	// For the |psi>, corresponding to min/max all the roots are simple, real, distinct.
	this.p_lam=TMatr.getAMatrA(psi,QQpa,n)/TMatr.getAMatrA(psi,QQpa_scale,n);
	final double plam=lam*p_lam;
	// Build Gauss quadrature for the measure dmu=da-lam*db.
	for(int k=0;k<x.length;k++)
	{
	    final double xk=x[k];
	    final double [] l=M.B.sdiv1(psi,xk).q;
	    final double nrmlk=M.B.getD0(l,xk);
	    TMatr.scaleVectorInPlace(l,1/nrmlk);
	    final double m0a=TMatr.getAMatrA(l,QQa,n),m0b=TMatr.getAMatrA(l,QQb,n),
		pm0a=m0a*(TMatr.getAMatrA(l,QQpa,n)/TMatr.getAMatrA(l,QQpa_scale,n));
	    wmu[k]=m0a-lam*m0b;
	    p_wmu[k]=pm0a-plam*m0b;
	}
	this.p_Mu=TMatr.sumElements(p_wmu)/TMatr.sumElements(wmu);
	// Check the quadrature is built OK.
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(lam,"lam",TMatr.getAMatrA(psi,QQa,n)/TMatr.getAMatrA(psi,QQb,n),"<psi|QQa|psi>/<psi|QQb|psi>",eps);
	    final double [] QQmu=TMatr.sumAcB(QQa,-lam,QQb),o=new double []{1},ox=M.B.getaxbP(o,1,0);
	    UnitTests.checkDoubleRel(TMatr.sumElements(wmu),"sum(wmu)",TMatr.getAMatrA(o,QQmu,n),"QQmu[0]",eps);
	    UnitTests.checkDoubleRel(TMatr.mulVV(x,wmu),"sum(x*wmu)",TMatr.getAMatrB(o,ox,QQmu,n),"XQQmu[0]",eps);
	    UnitTests.checkDoubleRel(TMatr.mulV2V(x,wmu),"sum(x^2*wmu)",TMatr.getAMatrA(ox,QQmu,n),"X^2QQmu[0]",eps);
	    System.err.println("Mu2DynMicroImpact.Mu: invariant internal test OK n="+n+" lam="+lam+" eps="+eps);
	}
    }
}

}

