package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.ProbabilityCorrelation;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 * Calculate avegrages of different kind.
 * @version $Id: Mu2F.java,v 1.10 2020/09/27 20:33:17 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class Mu2F {
public final double pt,pi,root_psiL,root_psiH;
public final ProbabilityCorrelation pc;
public Mu2F(final double p0t,final double p1t,final double p2t,
	    final double p0i,final double p1i,final double p2i,
	    final double eps)
{
    pt=p1t/p0t;
    pi=p1i/p0i;
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=com.polytechnik.utils.DiffSkewness.MB;
    final VolMatrix varpsi=new VolMatrix(){
	    public double [] getDPsi(final double [] phi){
		// In price space. Regular differentiation.
		return M.B.getDifferentiated(phi);
	    }
    };
    final int d=2;
    final double [] Mi=M.B.getQQMatr(d,new double []{p0i,p1i,p2i}),
	Mt=M.B.getQQMatr(d,new double []{p0t,p1t,p2t}),
	iev=M.EV.getGEV_Generalized(d,Mi,Mt,true), // eigenvalues problem
	psiL=EVXData.getPsiFromBasis(0,iev,d),psiH=EVXData.getPsiFromBasis(d-1,iev,d);
    this.pc=new ProbabilityCorrelation(Mt,varpsi.getDiffMatrix(d,Mt),Mi,M,true);
    root_psiL=-M.B.getD0(psiL,0)/M.B.getD1(psiL,0);
    root_psiH=-M.B.getD0(psiH,0)/M.B.getD1(psiH,0);
    if(eps>0)
    {
	UnitTests.checkDoubleRel(M.B.getD0(psiL,root_psiL),"psiL(root)",0,"0",eps);
	UnitTests.checkDoubleRel(M.B.getD0(psiH,root_psiH),"psiH(root)",0,"0",eps);
	ProbabilityCorrelation.matchProbabilityCorrelation(new Mu2P2Asymmetry(p0t,p1t,p2t,p0i,p1i,p2i,eps).pc,
							   this.pc,eps);
	System.err.println("Mu2F:test OK root_psiL="+root_psiL+" root_psiH="+root_psiH+" eps="+eps);
    }
}
}
