package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 * Try I=dv/dt and ds/dt are back (da=|dp|).
 * @version $Id: Mu2IA.java,v 1.16 2022/11/20 12:41:33 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class Mu2IA {
public final Mu2IAdir dir;
public final double p_offset;
public Mu2IA(final double p_offset,final double [] psi,final double lam,
	 final double [] QQi,final double [] QQt,
	 final double [] QQp0t_sample,final double [] QQp1t_sample,final double [] QQp2t_sample,
	 final double [] QQp0i,final double [] QQp1i,final double [] QQp2i,
	 final double [] QQp0a,final double [] QQp1a,final double [] QQp2a,
	 final double eps)
{
    this.p_offset=p_offset;
    final int n=psi.length;
    dir=new Mu2IAdir(TMatr.getAMatrA(psi,QQp0t_sample,n),TMatr.getAMatrA(psi,QQp1t_sample,n),TMatr.getAMatrA(psi,QQp2t_sample,n),
		     TMatr.getAMatrA(psi,QQp0i,n),TMatr.getAMatrA(psi,QQp1i,n),TMatr.getAMatrA(psi,QQp2i,n),
		     TMatr.getAMatrA(psi,QQp0a,n),TMatr.getAMatrA(psi,QQp1a,n),TMatr.getAMatrA(psi,QQp2a,n),eps);
}

/** Similar to com.polytechnik.utils.ProbabilityCorrelation,
 * but here we directly manipulate with |psi>
 * so it is better to do it separately.
 */
public static class Mu2IAdir {
public final double root_ipsiL,root_apsiL,dir;
public Mu2IAdir(final double p0t,final double p1t,final double p2t,
	 final double p0i,final double p1i,final double p2i,
	 final double p0a,final double p1a,final double p2a,
	 final double eps)
{
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=
	com.polytechnik.utils.DiffSkewness.MB;
    final int d=2;
    final double [] Mi=M.B.getQQMatr(d,new double []{p0i,p1i,p2i}),
	Mt=M.B.getQQMatr(d,new double []{p0t,p1t,p2t}),
	Ma=M.B.getQQMatr(d,new double []{p0a,p1a,p2a}),
	iev=M.EV.getGEV_Generalized(d,Mi,Mt,true), // eigenvalues problem
	ipsiL=EVXData.getPsiFromBasis(0,iev,d),ipsiH=EVXData.getPsiFromBasis(d-1,iev,d),
	bMa=EVXData.convertMatrFromQtoB_Basis(d,Ma,iev);
    final double [] baev=M.EV.getEV(d,bMa,true),
	apsiL=EVXData.convertPsiFromBtoQ_Basis(d,EVXData.getPsiFromBasis(0,baev,d),iev),
	apsiH=EVXData.convertPsiFromBtoQ_Basis(d,EVXData.getPsiFromBasis(d-1,baev,d),iev);
    root_ipsiL=-M.B.getD0(ipsiL,0)/M.B.getD1(ipsiL,0);
    root_apsiL=-M.B.getD0(apsiL,0)/M.B.getD1(apsiL,0);
    // The baev basis: {lambda_aL,lambda_aH, // eigenvalues
    //                  <apsiL|ipsiL>,<apsiL|ipsiH>,
    //                  <apsiH|ipsiL>,<apsiH|ipsiH>}.
    // The matrix is unitary:
    //  <apsiL|ipsiL>^2+<apsiL|ipsiH>^2=1
    //  <apsiH|ipsiL>^2+<apsiH|ipsiH>^2=1
    //  <apsiL|ipsiL>*<apsiH|ipsiH>-<apsiL|ipsiH>*<apsiH|ipsiL>=+-1
    // i.e.:
    // <apsiL|ipsiL>= +- <apsiH|ipsiH>
    // <apsiL|ipsiH>= -+ <apsiH|ipsiL>
    //
    // The direction is:
    // <apsiH|ipsiL>*signum
    dir=baev[d+(d-1)*d+0]*Math.signum(M.B.getD0(apsiH,root_apsiL))*Math.signum(M.B.getD1(ipsiL,root_ipsiL));
    if(eps>0)
    {
	final Mu2F pIH=new Mu2F(p0t,p1t,p2t,p0i,p1i,p2i,eps),
	    pAH=new Mu2F(p0t,p1t,p2t,p0a,p1a,p2a,eps);
	UnitTests.checkDoubleRel(root_apsiL,"root_apsiL",pAH.root_psiL,"pAH.root_psiL",eps);
	UnitTests.checkDoubleRel(root_ipsiL,"root_ipsiL",pIH.root_psiL,"pIH.root_psiL",eps);
    }
    if(eps>0)
    {
	final double [] aev=M.EV.getGEV_Generalized(d,Ma,Mt,true);
	for(int j=0;j<d;j++) {
	    final double [] apsi=EVXData.getPsiFromBasis(j,aev,d);
	    UnitTests.checkDoubleArrayRel1(EVXData.getMSign(j<=0?apsiL:apsiH),"apsi",EVXData.getMSign(apsi),"apsi",eps);
	}
	UnitTests.checkDoubleRel(M.B.getD0(apsiL,root_apsiL),"apsiL(root)",0,"0",eps);
	UnitTests.checkDoubleRel(M.B.getD0(ipsiL,root_ipsiL),"ipsiL(root)",0,"0",eps);
	System.err.println("Mu2IA.Mu2IAdir: baev="+java.util.Arrays.toString(baev));
	System.err.println("Mu2IA.Mu2IAdir:test OK root_ipsiL="+root_ipsiL+" root_apsiL="+root_apsiL+" eps="+eps);
    }
}
} // END OF class Mu2IAdir

static void matchMu2IA(final Mu2IA a,final Mu2IA b,final double eps)
{
    //EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;

    UnitTests.checkDoubleRel(a.dir.root_ipsiL,"a.dir.root_ipsiL",b.dir.root_ipsiL+p_offset_shift,"b.dir.root_ipsiL+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dir.root_apsiL,"a.dir.root_apsiL",b.dir.root_apsiL+p_offset_shift,"b.dir.root_apsiL+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dir.dir,"a.dir.dir",b.dir.dir,"b.dir.dir",eps);

    System.err.println("Mu2IA.matchMu2IA test OK p_offset_shift="+p_offset_shift+" eps="+eps);
}

} // END OF class Mu2IA
