package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
/**
 * P-X n=2 psi_{I low} asymmetry version.
 * @version $Id: Mu2PX.java,v 1.33 2020/09/27 20:33:17 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class Mu2PX {
public final double root_xpsiL,root_ppsiL,ppsiL_I_ppsiL,xpsiL_I_xpsiL,dirLLt,dirLLi,dirLHt,dirLHi;
Mu2PX(final double _0t,final double p1t,final double p2t,
      final double _0i,final double p1i,final double p2i,
      final double x1t,final double x2t,
      final double x1i,final double x2i,
      final double pxt,final double pxi,
      final double eps)
{
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=
	com.polytechnik.utils.DiffSkewness.MB;
    final int d=2;
    final double [] pMi=M.B.getQQMatr(d,new double []{_0i,p1i,p2i}),
	pMt=M.B.getQQMatr(d,new double []{_0t,p1t,p2t}),
	pev=M.EV.getGEV_Generalized(d,pMi,pMt,true), // eigenvalues problem
	xMi=M.B.getQQMatr(d,new double []{_0i,x1i,x2i}),
	xMt=M.B.getQQMatr(d,new double []{_0t,x1t,x2t}),
	xev=M.EV.getGEV_Generalized(d,xMi,xMt,true),
	pxMt=new double [] {_0t,x1t,p1t,pxt},
	pxMi=new double [] {_0i,x1i,p1i,pxi},
	ppsiL=EVXData.getPsiFromBasis(0,pev,d),ppsiH=EVXData.getPsiFromBasis(d-1,pev,d),
	xpsiL=EVXData.getPsiFromBasis(0,xev,d),xpsiH=EVXData.getPsiFromBasis(d-1,xev,d);
    final double ppsiL_xpsiL=TMatr.getAMatrB(ppsiL,xpsiL,pxMt,d),ppsiL_I_xpsiL=TMatr.getAMatrB(ppsiL,xpsiL,pxMi,d),
	sgnLL=Math.signum(M.B.getD1(ppsiL,0))*Math.signum(M.B.getD1(xpsiL,0));
    // the root of |psi> of low I is the position of high I.
    root_xpsiL=-M.B.getD0(xpsiL,0)/M.B.getD1(xpsiL,0);
    root_ppsiL=-M.B.getD0(ppsiL,0)/M.B.getD1(ppsiL,0);
    xpsiL_I_xpsiL=xev[0];
    ppsiL_I_ppsiL=pev[0];
    dirLLt=ppsiL_xpsiL*sgnLL;
    dirLLi=(ppsiL_I_xpsiL/Math.sqrt(ppsiL_I_ppsiL*xpsiL_I_xpsiL))*sgnLL;
    final double ppsiL_xpsiH=TMatr.getAMatrB(ppsiL,xpsiH,pxMt,d),ppsiL_I_xpsiH=TMatr.getAMatrB(ppsiL,xpsiH,pxMi,d),
	sgnLH=Math.signum(M.B.getD1(ppsiL,0))*Math.signum(M.B.getD0(xpsiH,root_xpsiL));
    dirLHt=ppsiL_xpsiH*sgnLH;
    dirLHi=(ppsiL_I_xpsiH/Math.sqrt(ppsiL_I_ppsiL*xev[d-1]))*sgnLH;

    if(eps>0)
    {
	// typical x offset is x[k], the root is 0.
	//UnitTests.checkDoubleRel(root_xpsiL,"root_xpsiL",0,"0",eps);

	System.err.println("Mu2PX: ppsiL_xpsiL="+ppsiL_xpsiL+" ppsiL_xpsiH="+ppsiL_xpsiH+" ppsiH_xpsiL="+TMatr.getAMatrB(ppsiH,xpsiL,pxMt,d)+" ppsiH_xpsiH="+TMatr.getAMatrB(ppsiH,xpsiH,pxMt,d));
	for(double [] p:new double [][]{ppsiL,ppsiH})for(double [] x:new double [][]{xpsiL,xpsiH})
	{
	    System.err.println("Mu2PX: <ppsi|xpsi>="+TMatr.getAMatrB(p,x,pxMt,d)+" <ppsi|I|xpsi>="+TMatr.getAMatrB(p,x,pxMi,d));
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(p,pMt,d),"<ppsi|ppsi>",1,"1",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(x,xMt,d),"<xpsi|xpsi>",1,"1",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(p,pMi,d),"<ppsi|I|ppsi>",pev[p==ppsiL?0:1],"pev",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(x,xMi,d),"<xpsi|I|xpsi>",xev[x==xpsiL?0:1],"xev",eps);
	    final double mpxt=p[0]*x[0]*_0t+p[1]*x[0]*p1t+p[0]*x[1]*x1t+p[1]*x[1]*pxt,
		mpxi=p[0]*x[0]*_0i+p[1]*x[0]*p1i+p[0]*x[1]*x1i+p[1]*x[1]*pxi;
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(p,x,pxMt,d),"<psip|psix>",mpxt,"mpxt",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(p,x,pxMi,d),"<psip|I|psix>",mpxi,"mpxi",eps);
	    if(p==ppsiL){
		if(x==xpsiL){
		    UnitTests.checkDoubleRel(ppsiL_xpsiL,"ppsiL_xpsiL",mpxt,"mpxt",eps);
		    UnitTests.checkDoubleRel(ppsiL_I_xpsiL,"ppsiL_I_xpsiL",mpxi,"mpxi",eps);
		}
		if(x==xpsiH){
		    UnitTests.checkDoubleRel(ppsiL_xpsiH,"ppsiL_xpsiH",mpxt,"mpxt",eps);
		    UnitTests.checkDoubleRel(ppsiL_I_xpsiH,"ppsiL_I_xpsiH",mpxi,"mpxi",eps);
		}
	    }
	}
	EVXData.test_orthogonality("Mu2PX.pev",d,pev,pMi,pMt,eps);
	EVXData.test_orthogonality("Mu2PX.xev",d,xev,xMi,xMt,eps);
	System.err.println("Mu2PX: test OK eps="+eps);
    }

    // ==========================================================================
    // Some d=3 experiments (not very successful).
    // basis order is: ppsiL, xpsiL
    final double [] LLMi=new double []{ppsiL_I_ppsiL,ppsiL_I_xpsiL,ppsiL_I_xpsiL,xpsiL_I_xpsiL},
	LLMt=new double []{1,ppsiL_xpsiL,ppsiL_xpsiL,1},
	LLev=M.EV.getGEV_Generalized(d,LLMi,LLMt,true),
	LLpsi=EVXData.getPsiFromBasis(0,LLev,d);
    bdir=(LLpsi[1]*M.B.getD1(xpsiL,0))/(LLpsi[0]*M.B.getD1(ppsiL,0));

    // Try d=3 basis (const,price,time--like).
    final int dd=3;
    // 3 functions in basis, basis order is: const,p,x
    final double [] Mi=new double []{
	_0i,p1i,x1i,
	p1i,p2i,pxi,
	x1i,pxi,x2i
    }, Mt=new double []{
	_0t,p1t,x1t,
	p1t,p2t,pxt,
	x1t,pxt,x2t
    },evdd=M.EV.getGEV_Generalized(dd,Mi,Mt,true),
    cpx_psiL=EVXData.getPsiFromBasis(0,evdd,dd),
    cpx_psiH=EVXData.getPsiFromBasis(dd-1,evdd,dd);
    IL_cpx=evdd[0];
    Ia_cpx=Mi[0]/Mt[0];
    bdir_cpx=cpx_psiL[2]/cpx_psiL[1];
    mdir_cpx=Mt[0]*cpx_psiL[2]*cpx_psiL[1];
    mdirH_cpx=Mt[0]*cpx_psiH[2]*cpx_psiH[1];

    if(eps>0)
    {
	if(!Double.isNaN(cpx_psiL[0])) // a kludge for degenerated basis
	{
	    EVXData.test_orthogonality("Mu2PX",dd,evdd,Mi,Mt,eps); // basis orthogonality test
	    final double [] rq=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(
		cpx_psiL,new double []{cpx_psiL[1]},new double []{},
		cpx_psiL,new double []{cpx_psiL[2]},new double []{},
		Mi,Mt,dd);
	    UnitTests.checkDoubleRel(rq[0],"rq[0]",IL_cpx,"IL_cpx",eps);
	    UnitTests.checkDoubleRel(rq[1],"rq[1]",0,"0",eps);
	    System.err.println("(Ia_cpx-IL_cpx)="+(Ia_cpx-IL_cpx)+" mdir_cpx="+mdir_cpx+" rq="+java.util.Arrays.toString(rq)+" cpx_psiL="+java.util.Arrays.toString(cpx_psiL)+" Mt="+java.util.Arrays.toString(Mt));
	    if(Ia_cpx>1) UnitTests.checkDoubleRel(rq[2],"rq[2]",2*(Ia_cpx-IL_cpx)*mdir_cpx,"2*(Ia_cpx-IL_cpx)*mdir_cpx",eps);
	}
	System.err.println("Mu2PX: test OK LLev[0]="+LLev[0]+" LLev[1]="+LLev[1]+" ppsiL_I_ppsiL="+ppsiL_I_ppsiL+" xpsiL_I_xpsiL="+xpsiL_I_xpsiL+" mdir_cpx="+mdir_cpx+" mdirH_cpx="+mdirH_cpx+" eps="+eps);
    }
}
/** d=3 Experiments. */
final double bdir,bdir_cpx,IL_cpx,Ia_cpx,mdir_cpx,mdirH_cpx;

}
