package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.EVSolver;

/**
 * Try dmu measure p- and x- basis hybrid, in price basis it gives regular dmu- measure: dmu=dv-lam^[IL]dt.
 * @version $Id: Mu2PXImpact.java,v 1.44 2022/07/28 22:30:50 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class Mu2PXImpact {
    public final EVXData I;
    public final double p_offset,p_IH,pt_IH;
    final Mu muL,muH;
    final Skewness skb;
    public final Mu2P2Asymmetry muasym_average;
    final PnLMuPoints p;

Mu2PXImpact(final IMomentsData mm,
	    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	    final double eps)
{
    final int n=mm.n,ixH=n-1;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    this.I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    this.p_offset=mm.p_offset;
    p_IH=I.getFPsi(ixH,mm.QQpi)/I.ev[ixH];
    pt_IH=I.getFPsi(ixH,mm.QQpt_sample)/I.getFPsi(ixH,mm.QQt_sample);
    muasym_average=new Mu2P2Asymmetry(mm.QQt_sample[0],mm.QQpt_sample[0],mm.QQp2t_sample[0],
				      mm.QQi[0],mm.QQpi[0],mm.QQp2i[0],eps);

    muL=new Mu(mm.p_last,I.psiL,I.sL,mm.QQi,mm.QQt,
	       mm.QQpi,mm.QQp2i,mm.QQi,
	       mm.QQpt_sample,mm.QQp2t_sample,mm.QQt_sample,
	       M,eps);
    muH=new Mu(mm.p_last,I.psiH,1/I.sH,mm.QQt,mm.QQi,
	       mm.QQpt_sample,mm.QQp2t_sample,mm.QQt_sample,
	       mm.QQpi,mm.QQp2i,mm.QQi,
	       M,eps);
    skb=Skewness.getGSkewnessFromMonomialMoments(muH.p0mu,muH.p1mu,muH.p2mu,
						 muL.p0mu,muL.p1mu,muL.p2mu);

    p=new PnLMuPoints(new PnLMuPoints.CMP(muL,muH,M));
}


public static class Mu extends PnLMuPoints.RMu {
    public final double lam,p0mu,p1mu,p2mu,wasymm;
    public final double [] P,wmu_o;
    public final Mu2P2Asymmetry [] muasym;
Mu(final double Pf,final double [] psi,final double lam,final double [] QQa,final double [] QQb,
   final double [] QQpa,final double [] QQp2a,final double [] QQa_scale,
   final double [] QQpb,final double [] QQp2b,final double [] QQb_scale,
   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final double eps)
{
    super(M.getXNodes(psi).re_root);
    if(true){
	final double [] ONE=M.B.getONE(),RONE=M.getRadauMultiplied(ONE);
	final boolean flag_dir_right=M.B.getD1(RONE,0)<0;
	if(flag_dir_right) com.polytechnik.utils.SortArrays.sortArrayWithLoad(x.length,x,null);
	else com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(x.length,x,null);
    }
    final int n=psi.length,d=2;
    this.lam=lam;
    this.wmu_o=new double [x.length];
    this.P=new double [x.length];
    this.muasym=new Mu2P2Asymmetry [x.length];
    // The measure dmu=da-lam*db.
    for(int k=0;k<x.length;k++)
    {
	final double xk=x[k];
	final double [] l=M.B.sdiv1(psi,xk).q;
	final double nrmlk=M.B.getD0(l,xk);
	TMatr.scaleVectorInPlace(l,1/nrmlk);
	final double m0a=TMatr.getAMatrA(l,QQa,n),aadjscale_sample=m0a/TMatr.getAMatrA(l,QQa_scale,n),
	    m0ap1=TMatr.getAMatrA(l,QQpa,n)*aadjscale_sample,
	    m0ap2=TMatr.getAMatrA(l,QQp2a,n)*aadjscale_sample,
	    m0b=TMatr.getAMatrA(l,QQb,n),badjscale_sample=m0b/TMatr.getAMatrA(l,QQb_scale,n),
	    m0bp1=TMatr.getAMatrA(l,QQpb,n)*badjscale_sample,
	    m0bp2=TMatr.getAMatrA(l,QQp2b,n)*badjscale_sample;
	this.muasym[k]=new Mu2P2Asymmetry(m0b,m0bp1,m0bp2,m0a,m0ap1,m0ap2,eps);
	//System.err.println("Mu2PXImpact: k="+k+" "+m0a+" "+m0ap1+" "+m0ap2+" "+m0b+" "+m0bp1+" "+m0bp2);
	this.wmu_o[k]=m0a-lam*m0b;
	final double [] Map=new double []{m0a,m0ap1,m0ap1,m0ap2},Mbp=new double []{m0b,m0bp1,m0bp1,m0bp2},
		ev_P=M.EV.getGEV_Generalized(d,Map,Mbp,true),
		phiP_IL=EVXData.getPsiFromBasis(0,ev_P,d);
	final double lams=ev_P[0];
	this.P[k]=-phiP_IL[0]/phiP_IL[1];
	this.wmu[k]=m0a-lams*m0b;
	this.p_wmu[k]=P[k]*wmu[k];
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(M.B.getD0(l,xk),"l(xk)",1,"1",eps);
	    UnitTests.checkDoubleRel(M.B.getD0(psi,xk),"psi(xk)",0,"0",eps);
	    final double m1b=TMatr.getAMatrB(l,psi,QQb,n)/nrmlk,m2b=TMatr.getAMatrA(psi,QQb,n)/Math.pow(nrmlk,2),m1a=lam*m1b,m2a=lam*m2b;
	    final double [] Ma=new double []{m0a,m1a,m1a,m2a},Mb=new double []{m0b,m1b,m1b,m2b};
	    final double [] ev_X=M.EV.getGEV_Generalized(d,Ma,Mb,true);
	    final Skewness sk=Skewness.getGSkewnessFromMonomialMoments(m0b,m0bp1,m0bp2,m0a,m0ap1,m0ap2);
	    UnitTests.checkDoubleRel(sk.sk,"sk.sk",(2*m0a/m0b-ev_P[0]-ev_P[1])/(ev_P[0]-ev_P[1]),"SK",eps);
	    final double [] phiX_IL=EVXData.getPsiFromBasis(0,ev_X,d),
		phiP_IH=EVXData.getPsiFromBasis(d-1,ev_P,d);
	    final double xeq=-phiX_IL[0]/phiX_IL[1];
	    UnitTests.checkDoubleRel(xk-xk,"xk",xeq,"xeq",eps); // offset is xk

	    final double [] pQQmus=TMatr.sumaAbB(aadjscale_sample,QQpa,-lams*badjscale_sample,QQpb),
		p2QQmus=TMatr.sumaAbB(aadjscale_sample,QQp2a,-lams*badjscale_sample,QQp2b);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(l,pQQmus,n)/wmu[k],"Pk",P[k],"P[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(l,p2QQmus,n)/wmu[k],"Pk^2",Math.pow(P[k],2),"P[k]^2",eps);
	    //System.err.println("Map="+java.util.Arrays.toString(Map));
	    //System.err.println("Mbp="+java.util.Arrays.toString(Mbp));
	    final Mu2P2Asymmetry.MuRQ m=muasym[k].mL;
	    UnitTests.checkDoubleRel(lams,"lams",m.lam,"m.lam",eps);
	    UnitTests.checkDoubleRel(P[k],"P[k]",m.p_psiroot,"m.p_psiroot",eps);
	    UnitTests.checkDoubleRel(wmu[k],"wmu[k]",m.wmu,"m.wmu",eps);
	    UnitTests.checkDoubleRel(m0a/m0b,"m0a/m0b",m.lamconst,"m.lamconst",eps);

	    System.err.println(
		"Mu2PXImpact: test OK, x["+k+"]="+xk+" wmu_o[k]="+wmu_o[k]+" lam="+lam+
		" ev_X="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_X,d))+
		" ev_P="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_P,d))+
		" aadjscale_sample="+aadjscale_sample+" badjscale_sample="+badjscale_sample+" n="+n);
	}
    }
    p0mu=TMatr.sumElements(wmu);
    p1mu=TMatr.mulVV(P,wmu);
    p2mu=TMatr.mulV2V(P,wmu);
    double s=0; for(Mu2P2Asymmetry m:muasym) s+=m.mL.beta*m.mL.wmu;
    wasymm=s;
    if(eps>0)
    {
	 // Test quadratures for dmu=dv-lam*dt
	final double [] QQmu=TMatr.sumAcB(QQa,-lam,QQb),o=new double []{1},ox=M.B.getaxbP(o,1,0);
	    UnitTests.checkDoubleRel(TMatr.sumElements(wmu_o),"sum(wmu_o)",TMatr.getAMatrA(o,QQmu,n),"QQmu[0]",eps);
	    UnitTests.checkDoubleRel(TMatr.mulVV(x,wmu_o),"sum(x*wmu_o)",TMatr.getAMatrB(o,ox,QQmu,n),"XQQmu[0]",eps);
	    UnitTests.checkDoubleRel(TMatr.mulV2V(x,wmu_o),"sum(x^2*wmu_o)",TMatr.getAMatrA(ox,QQmu,n),"X^2QQmu[0]",eps);
	    final Mu2PinIL m=new Mu2PinIL(Pf,psi,lam,QQa,QQb,
					  QQa_scale,QQpa,QQp2a,
					  QQb_scale,QQpb,QQp2b,M,eps);
	    UnitTests.checkDoubleArrayRel1(m.x,"m.x",x,"x",eps);
	    UnitTests.checkDoubleArrayRel1(m.wmu,"m.wmu",wmu_o,"wmu_o",eps);
	    for(int k=0;k<x.length;k++)
	    {
		Mu2P2Asymmetry.matchMu2P2Asymmetry(muasym[k],0,new Mu2P2Asymmetry(m.p0b[k],m.p1b[k],m.p2b[k],m.p0a[k],m.p1a[k],m.p2a[k],eps),0,eps);
	    }
	    System.err.println("Mu2PXImpact: test OK, n="+n+" eps="+eps);
    }
}
} // END OF public static class Mu

public static void matchMu2PXImpact(final Mu2PXImpact a,final Mu2PXImpact b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.pt_IH,"a.pt_IH",b.pt_IH+p_offset_shift,"b.pt_IH+p_offset_shift",eps);

    /* // FIXME
    if(true)
    { // they may be sorted differently
	final int [] ind_a=com.polytechnik.utils.SortArrays.sortArrayIndexes(a.wmu),
	    ind_b=com.polytechnik.utils.SortArrays.sortArrayIndexes(b.wmu);
	for(int k=0;k<ind_a.length;k++)
	{
	    final int ia=ind_a[k],ib=ind_b[k];
	    UnitTests.checkDoubleRel(a.wmu[ia],"a.wmu[ia]",b.wmu[ib],"b.wmu[ib]",eps);
	    UnitTests.checkDoubleRel(a.wmu_o[ia],"a.wmu_o[ia]",b.wmu_o[ib],"b.wmu_o[ib]",eps);
	    UnitTests.checkDoubleRel(a.P[ia],"a.P[ia]",b.P[ib]+p_offset_shift,"b.P[ib]+p_offset_shift",eps);
	}
    }
    if(a.I.neq>2) Skewness.matchSkewness(a.skmus.getSameSkewnessXShifted(a.p_offset),b.skmus.getSameSkewnessXShifted(b.p_offset),eps);
    */

    System.err.println("Mu2PXImpact.matchMu2PXImpact test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}
}
