package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.ProbabilityCorrelation;

/**
 * Try min_px {@literal <(p-px)^2l^2 I>}/(p-px)^2l^2).
 * @version $Id: Mu2PXinIL.java,v 1.7 2022/06/17 20:37:51 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class Mu2PXinIL extends Mu2PsiRoots {
final double [] px,ix;
final double p_offset,pcmax,imax,ximax,pc1,pc2,x1,x2,i1,i2;
Mu2PXinIL(final double p_offset,
	  final double [] psi,final double lam,
	  final double [] QQi,final double [] QQt,
	  final double [] QQp0i_sample,final double [] QQp1i_sample,final double [] QQp2i_sample,
	  final double [] QQp0t_sample,final double [] QQp1t_sample,final double [] QQp2t_sample,
	  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final double eps)
{
    super(psi,lam,QQi,QQt,QQp0i_sample,QQp1i_sample,QQp2i_sample,
	  QQp0t_sample,QQp1t_sample,QQp2t_sample,M,eps);
    this.p_offset=p_offset;
    px=new double [x.length];
    ix=new double [x.length];
    final int d=2;
    for(int k=0;k<x.length;k++){
	// instead of min_xx <(x-xx)^2 l^2 I>/<(x-xx)^2 l^2>, that give the xx=xk solution, try with the price.
	final double [] Mi=new double []{p0a[k],p1a[k],p1a[k],p2a[k]},
	    Mt=new double []{p0b[k],p1b[k],p1b[k],p2b[k]},
	    evS=M.EV.getGEV_Generalized(d,Mi,Mt,true),
	    phiL=EVXData.getPsiFromBasis(0,evS,d);
	ix[k]=evS[0];
	// the root of phiL is the price of max I
	px[k]=-phiL[0]/phiL[1];
    }

    final double x0=M.getRadauRoot();
    int indpc=-1;
    for(int k=x.length;--k>=0;){
	if(flag_dir_right && x[k]<=x0){indpc=k;break;}
	if(!flag_dir_right && x[k]>=x0){indpc=k;break;}
    }
    pc1=indpc>=0?px[indpc]:Double.NaN;
    pc2=indpc>=1?px[indpc-1]:Double.NaN;
    x1=indpc>=0?x[indpc]:Double.NaN;
    x2=indpc>=1?x[indpc-1]:Double.NaN;
    i1=indpc>=0?ix[indpc]:Double.NaN;
    i2=indpc>=1?ix[indpc-1]:Double.NaN;

    int indmax=0;
    for(int k=x.length;--k>=0;){
	if(indmax<0 || p0a[k]/p0b[k]>=p0a[indmax]/p0b[indmax]){
	    indmax=k;
	}
	//System.err.println("k="+k+" ik="+(p0a[k]/p0b[k])+" indmax="+indmax);
    }
    pcmax=px[indmax];
    imax=ix[indmax];
    ximax=x[indmax];

    if(eps>0){
	System.err.println("Mu2PXinIL: x roots="+java.util.Arrays.toString(x)+" test OK eps="+eps);
    }
}
}
