package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.ProbabilityCorrelation;

/**
 * Try dmu measure p- and x- basis hybrid, in price basis it gives regular dmu- measure: dmu=dv-lam^[IL]dt.
 * @version $Id: Mu2PinIL.java,v 1.37 2020/09/27 20:33:17 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class Mu2PinIL extends Mu2PsiRoots {
final double betaW,betaA,betaM,qsk;
Mu2PinIL(final double Pf,
	 final double [] psi,final double lam,
	 final double [] QQi,final double [] QQt,
	 final double [] QQp0i_sample,final double [] QQp1i_sample,final double [] QQp2i_sample,
	 final double [] QQp0t_sample,final double [] QQp1t_sample,final double [] QQp2t_sample,
	 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> _M,final double eps)
{
    super(psi,lam,QQi,QQt,QQp0i_sample,QQp1i_sample,QQp2i_sample,
	  QQp0t_sample,QQp1t_sample,QQp2t_sample,_M,eps);
    final Mu2PX [] mpx=new Mu2PX [x.length];
    double betaw=0,betaa=0,betam=0,sqsk=0;
    for(int k=0;k<x.length;k++)
    {
	// Try I(P) direct differentiation: variate Pf. Not a correct approach, but try it anyway.
	final Mu2 m2=new Mu2(p0b[k],p1b[k],p2b[k],p0a[k],p1a[k],p2a[k],eps);
	betaw+=m2.Ip.getDF(Pf)*tau[k];
	betaa+=m2.Ip.getDF(m2.pa)*tau[k];
	betam+=m2.DIpDpm*tau[k];
	sqsk+=m2.qsk*wmu[k];

	if(eps>0)
	{
	    final int d=2;
	    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> MB=
		new OrthogonalPolynomialsMonomialsBasisX(1,-Pf);
	    final VolMatrix varpsi=new VolMatrix(){
		    public double [] getDPsi(final double [] phi){
			// In price space. Regular differentiation.
			return MB.B.getDifferentiated(phi);
		    }
		};
	    // Old I(p) approach: variate psi(p).
	    final double [] Mi=MB.B.getQQMatr(d,new double []{p0a[k],p1a[k],p2a[k]}),
		Mt=MB.B.getQQMatr(d,new double []{p0b[k],p1b[k],p2b[k]});
	    final EVXData Ik=EVXData.getEVXDataFromQQ(d,Mi,Mt,MB);
	    final double [] phi=Ik.psi0,Dphi=varpsi.getDPsi(phi),
		rq=com.polytechnik.utils.RayleighQuotient.getRQVariationValues(phi,Dphi,null,Mi,Mt,d);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(Ik.psiL,phi,Mt,d),"<psiL|phi>",MB.B.getD0(Ik.psiL,Pf)/Math.sqrt(Ik.o_o),"<psiL|phi>",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(Ik.psiH,phi,Mt,d),"<psiH|phi>",MB.B.getD0(Ik.psiH,Pf)/Math.sqrt(Ik.o_o),"<psiH|phi>",eps);
	    final double [] QQl={1,Pf,Pf,Pf*Pf};
	    final Skewness sk=Skewness.getG0Skewness(Mt,Mi,MB,Pf);
	    UnitTests.checkDoubleRel((2*Ik.s0-Ik.sL-Ik.sH)/(Ik.sL-Ik.sH),"skewness0",sk.sk,"sk.sk",eps);
	    final ProbabilityCorrelation pc=new ProbabilityCorrelation(Mt,QQl,Mi,MB,true);
	    UnitTests.checkDoubleRel(-pc.corr,"-pc.corr",sk.sk,"sk.sk",eps);
	    UnitTests.checkDoubleRel(rq[0],"rq[0]",m2.Ip.getF(Pf),"pr.getF(Pf)",eps);
	    // Do not match!!!
	    //UnitTests.checkDoubleRel(rq[1],"rq[1]",-0.5*m2.Ip.getDF(Pf),"-0.5*m2.Ip.getDF(Pf)",eps);
	    //UnitTests.checkDoubleRel(rq[2],"rq[2]",0.25*m2.Ip.getD2F(Pf),"0.25*m2.Ip.getD2F(Pf)",eps);

	    final double root_psiL=-Ik.psiL[0]/Ik.psiL[1],root_psiH=-Ik.psiH[0]/Ik.psiH[1];
	    UnitTests.checkDoubleRel(m2.Ip.getF(root_psiL),"I(proot)",Ik.sH,"Ik.sH",eps);
	    UnitTests.checkDoubleRel(m2.Ip.getDF(root_psiL),"I'(proot)",0,"0",eps);
	    UnitTests.checkDoubleRel(m2.Ip.getF(root_psiH),"I(proot)",Ik.sL,"Ik.sL",eps);
	    UnitTests.checkDoubleRel(m2.Ip.getDF(root_psiH),"I'(proot)",0,"0",eps);
	    // Test I(p) interpolation, to match the I(p) in the basis of eigenfunctions.
	    final double [] ibMt=com.polytechnik.utils.Linsystems.getInvertedMatrix(d,d,Mt),
		bMi=TMatr.mulMatrices(d,new double [][] {ibMt,Mi,ibMt});
	    //bMi=EVXData.convertMatrFromQtoB_Basis(d,Mi,ibMt,d,0);
	    final double [] nom=MB.B.getKK(d,bMi),denom=MB.B.getKK(d,ibMt);
	    final com.polytechnik.utils.PolynomialsRatio _pr=
		new com.polytechnik.utils.PolynomialsRatio(denom,nom,MB.B);
	    UnitTests.checkDoubleRel(m2.Ip.getF(Pf),"I(Pf)",_pr.getF(Pf),"I(Pf)",eps);
	    UnitTests.checkDoubleRel(m2.Ip.getDF(Pf),"DI(Pf)",_pr.getDF(Pf),"DI(Pf)",eps);
	    UnitTests.checkDoubleRel(m2.Ip.getD2F(Pf),"D2I(Pf)",_pr.getD2F(Pf),"D2I(Pf)",eps);
	    UnitTests.checkDoubleRel((m2.Ip.getF(m2.pa)-lam)*tau[k],"wmu",wmu[k],"wmu",eps);
	}
	// just to test
	mpx[k]=new Mu2PX(p0b[k],p1b[k],p2b[k],p0a[k],p1a[k],p2a[k],
			 x1b[k],x2b[k],x1a[k],x2a[k],
			 pxb[k],pxa[k],eps);
	if(eps>0)
	{
	    final Mu2PX m=mpx[k];
	    // match I
	    UnitTests.checkDoubleRel(m.xpsiL_I_xpsiL,"m.xpsiL_I_xpsiL",lam,"lam",eps);
	    UnitTests.checkDoubleRel(m.ppsiL_I_ppsiL,"m.ppsiL_I_ppsiL",m2.IL,"m2.IL",eps);
	    // match the roots
	    UnitTests.checkDoubleRel(m.root_ppsiL,"m.root_ppsiL",m2.root_psiL,"m2.root_psiL",eps);
	    // the offset is x[k], the x-root is 0:
	    UnitTests.checkDoubleRel(m.root_xpsiL,"m.root_xpsiL",0,"0",eps);
	}
    }
    betaW=betaw;
    betaA=betaa;
    betaM=betam;
    qsk=sqsk/TMatr.sumElements(wmu);
    double swL=0,swH=0,sdLLt=0,sdLLi=0,sdLHt=0,sdLHi=0;
    for(int k=0;k<mpx.length;k++)
    {
	final Mu2PX m=mpx[k];
	swL+=m.mdir_cpx*wmu[k];
	swH+=m.mdirH_cpx*wmu[k];
	sdLLt+=m.dirLLt*wmu[k];
	sdLLi+=m.dirLLi*wmu[k];
	sdLHt+=m.dirLHt*wmu[k];
	sdLHi+=m.dirLHi*wmu[k];
    }
    sL_cpx=swL/TMatr.sumElements(wmu);
    sH_cpx=swH/TMatr.sumElements(wmu);
    dirLLt=sdLLt/TMatr.sumElements(wmu);
    dirLLi=sdLLi/TMatr.sumElements(wmu);
    dirLHt=sdLHt/TMatr.sumElements(wmu);
    dirLHi=sdLHi/TMatr.sumElements(wmu);
}
    final double dirLLt,dirLLi,dirLHt,dirLHi,sL_cpx,sH_cpx;
}
