package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 * Try dmu measure p- and x- basis hybrid, in price basis it gives regular dmu- measure: dmu=dv-lam^[IL]dt.
 * @version $Id: Mu2PsiRoots.java,v 1.18 2022/06/27 07:19:13 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class Mu2PsiRoots {
public final double lam;
public final double [] p0a,p1a,p2a,p0b,p1b,p2b,x0a,x1a,x2a,x0b,x1b,x2b,pxa,pxb,wmu,tau,x,psi,psisdxk_atxk;
public final boolean flag_dir_right;
Mu2PsiRoots(final double [] psi,final double lam,final double [] QQa,final double [] QQb,
	    final double [] QQp0a_sample,final double [] QQp1a_sample,final double [] QQp2a_sample,
	    final double [] QQp0b_sample,final double [] QQp1b_sample,final double [] QQp2b_sample,
	    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final double eps)
{
    final int n=psi.length;
    this.lam=lam;
    this.psi=psi;
    final double [] ONE=M.B.getONE(),RONE=M.getRadauMultiplied(ONE);
    this.flag_dir_right=M.B.getD1(RONE,0)<0;
    this.x=M.getXNodes(psi).re_root;
    if(flag_dir_right) com.polytechnik.utils.SortArrays.sortArrayWithLoad(x.length,x,null);
    else com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(x.length,x,null);
    wmu=new double [x.length];
    tau=new double [x.length];
    p0a=new double [x.length];
    p1a=new double [x.length];
    p2a=new double [x.length];
    p0b=new double [x.length];
    p1b=new double [x.length];
    p2b=new double [x.length];
    x0a=new double [x.length];
    x1a=new double [x.length];
    x2a=new double [x.length];
    x0b=new double [x.length];
    x1b=new double [x.length];
    x2b=new double [x.length];
    pxa=new double [x.length];
    pxb=new double [x.length];
    psisdxk_atxk=new double [x.length];
    // The measure dmu=da-lam*db.
    for(int k=0;k<x.length;k++)
    {
	final double xk=x[k];
	final double [] l=M.B.sdiv1(psi,xk).q;
	final double nrmlk=M.B.getD0(l,xk);
	psisdxk_atxk[k]=nrmlk;
	TMatr.scaleVectorInPlace(l,1/nrmlk);
	// psi=(x-xk)*nrmlk*l.
	final double m0a=TMatr.getAMatrA(l,QQa,n),aadjscale_sample=m0a/TMatr.getAMatrA(l,QQp0a_sample,n),
	    m0b=TMatr.getAMatrA(l,QQb,n),badjscale_sample=m0b/TMatr.getAMatrA(l,QQp0b_sample,n);
	wmu[k]=m0a-lam*m0b;
	tau[k]=m0b;
	p0a[k]=m0a;
	p1a[k]=TMatr.getAMatrA(l,QQp1a_sample,n)*aadjscale_sample;
	p2a[k]=TMatr.getAMatrA(l,QQp2a_sample,n)*aadjscale_sample;
	p0b[k]=m0b;
	p1b[k]=TMatr.getAMatrA(l,QQp1b_sample,n)*badjscale_sample;
	p2b[k]=TMatr.getAMatrA(l,QQp2b_sample,n)*badjscale_sample;
	x0a[k]=m0a;
	x1a[k]=TMatr.getAMatrB(psi,l,QQa,n)/nrmlk;
	pxa[k]=TMatr.getAMatrB(psi,l,QQp1a_sample,n)*aadjscale_sample/nrmlk;
	x2a[k]=TMatr.getAMatrA(psi,QQa,n)/Math.pow(nrmlk,2);
	x0b[k]=m0b;
	x1b[k]=TMatr.getAMatrB(psi,l,QQb,n)/nrmlk;
	pxb[k]=TMatr.getAMatrB(psi,l,QQp1b_sample,n)*badjscale_sample/nrmlk;
	x2b[k]=TMatr.getAMatrA(psi,QQb,n)/Math.pow(nrmlk,2);
	if(eps>0)
	{
	    //System.err.println("Mu2PsiRoots: k="+k+" "+p0a[k]+" "+p1a[k]+" "+p2a[k]+" "+p0b[k]+" "+p1b[k]+" "+p2b[k]);
	    UnitTests.checkDoubleRel(M.B.getD0(l,xk),"l(xk)",1,"1",eps);
	    UnitTests.checkDoubleRel(M.B.getD0(psi,xk),"psi(xk)",0,"0",eps);
	    UnitTests.checkDoubleRel(x1a[k],"x1a[k]",x1b[k]*lam,"x1b[k]*lam",eps);
	    UnitTests.checkDoubleRel(x2a[k],"x2a[k]",x2b[k]*lam,"x2b[k]*lam",eps);
	}
    }

    if(eps>0)
    {
	 // Test quadratures for dmu=dv-lam*dt
	    final double [] QQmu=TMatr.sumAcB(QQa,-lam,QQb),o=new double []{1},ox=M.B.getaxbP(o,1,0);
	    UnitTests.checkDoubleRel(TMatr.sumElements(wmu),"sum(wmu)",TMatr.getAMatrA(o,QQmu,n),"QQmu[0]",eps);
	    UnitTests.checkDoubleRel(TMatr.mulVV(x,wmu),"sum(x*wmu)",TMatr.getAMatrB(o,ox,QQmu,n),"XQQmu[0]",eps);
	    UnitTests.checkDoubleRel(TMatr.mulV2V(x,wmu),"sum(x^2*wmu)",TMatr.getAMatrA(ox,QQmu,n),"X^2QQmu[0]",eps);
	    System.err.println("Mu2PsiRoots: test OK, n="+n+" eps="+eps);
    }
}
} // END OF class Mu2PsiRoots
