package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 * Try dmu measure dmu=dv-lam^[IL]dt, various moments.
 * @version $Id: Mu2PsiRootsA.java,v 1.8 2022/06/27 07:19:13 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class Mu2PsiRootsA {
public final double lam,psi[],x[],wmu[],tau[];
public final double [][] moms;
Mu2PsiRootsA(final double [] psi,final double lam,final double [] QQi,final double [] QQt,
	     final double [][] inQQ,
	     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final double eps)
{
    final int n=psi.length;
    this.lam=lam;
    this.psi=psi;
    this.x=M.getXNodes(psi).re_root;
    wmu=new double [x.length];
    tau=new double [x.length];
    moms=new double [inQQ.length][x.length];
    // The measure dmu=da-lam*db.
    for(int k=0;k<x.length;k++)
    {
	final double xk=x[k];
	final double [] l=M.B.sdiv1(psi,xk).q;
	final double nrmlk=M.B.getD0(l,xk);
	TMatr.scaleVectorInPlace(l,1/nrmlk);
	// psi=(x-xk)*nrmlk*l.
	final double m0i=TMatr.getAMatrA(l,QQi,n),m0t=TMatr.getAMatrA(l,QQt,n);
	wmu[k]=m0i-lam*m0t;
	tau[k]=m0t;
	for(int j=0;j<moms.length;j++) moms[j][k]=TMatr.getAMatrA(l,inQQ[j],n);
	if(eps>0)
	{
	    //System.err.println("Mu2PsiRoots: k="+k+" "+p0a[k]+" "+p1a[k]+" "+p2a[k]+" "+p0b[k]+" "+p1b[k]+" "+p2b[k]);
	    UnitTests.checkDoubleRel(M.B.getD0(l,xk),"l(xk)",1,"1",eps);
	    UnitTests.checkDoubleRel(M.B.getD0(psi,xk),"psi(xk)",0,"0",eps);
	}
    }

    if(eps>0)
    {
	 // Test quadratures for dmu=dv-lam*dt
	final double [] QQmu=TMatr.sumAcB(QQi,-lam,QQt),o=new double []{1};
	UnitTests.checkDoubleRel(TMatr.sumElements(wmu),"sum(wmu)",TMatr.getAMatrA(o,QQmu,n),"QQmu[0]",eps);
	System.err.println("Mu2PsiRootsA: test OK, n="+n+" eps="+eps);
    }
}
} // END OF class Mu2PsiRootsA
