package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.ProbabilityCorrelation;

/**
 * @version $Id: MuDynMicroImpact.java,v 1.48 2022/07/28 12:27:17 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class MuDynMicroImpact {
    public final EVXData I;
    public final double p_offset,p_IH,pt_IH,_pt_IH,dpdt_orig_IH,dp_wv_IH,dp_wt_IH,/*sk_dp_wv_IH,*/
	dp_it_mu,ddp_it_mu,dp_mu,dpaver_mu,p_mu,paver_mu,mdp_mu,mdpaver_mu,mp_mu,mpaver_mu,pcorr_mu;
    public final double [] wmu,dp_wmu,p_wmu,pt,pi,dpt,dpi;
    public final ProbabilityCorrelation [] pc;

    MuDynMicroImpact(final EVXData I,
		     final double [] x,final double [] wmu,final ProbabilityCorrelation [] pc,
		     final double [] dp_wmu,final double [] p_wmu,final double dpmuaw,final double pmuaw,final double muaw,
		     final double [] mdp_wmu,final double [] mp_wmu,final double mdpmuaw,final double mpmuaw,
		     final double [] pi,final double [] pt,final double [] dpi,final double [] dpt,
		     final IMomentsData mm,
		     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		     final double eps)
{
    this.I=I;
    this.p_offset=mm.p_offset;
    if(true)
    { // Some old calculations (used for unit test).
	final int n=I.neq,ixH=n-1;
	// different from 1 as exact QQt different from QQt_sample
	final double ns_IH=I.getFPsi(ixH,mm.QQt_sample),dpI_wv_IH=I.getFPsi(ixH,mm.QQDpwv);
	p_IH=I.getFPsi(ixH,mm.QQpi)/I.ev[ixH];
	pt_IH=I.getFPsi(ixH,mm.QQpt_sample)/ns_IH;
	_pt_IH=I.getFPsi(ixH,mm.QQpt_parts);
	dpdt_orig_IH=I.getFPsi(ixH,mm.QQdpdt);
	dp_wv_IH=dpI_wv_IH/I.ev[ixH];
	dp_wt_IH=I.getFPsi(ixH,mm.QQDpwt)/ns_IH;
	/*
	final double [] bQQDpwv=EVXData.convertMatrFromQtoB_Basis(n,mm.QQDpwv,I.ev);
	final double [] ev=M.EV.getEV(n,bQQDpwv,n,true);
	sk_dp_wv_IH=(2*dpI_wv_IH-ev[0]-ev[ixH])/(ev[0]-ev[ixH]);
	*/
    }
    this.wmu=wmu;
    this.p_wmu=p_wmu;
    this.dp_wmu=dp_wmu;
    this.dp_mu=TMatr.sumElements(dp_wmu)/TMatr.sumElements(wmu);
    this.dpaver_mu=dpmuaw/muaw;
    this.p_mu=TMatr.sumElements(p_wmu)/TMatr.sumElements(wmu);
    this.paver_mu=pmuaw/muaw;
    mdp_mu=TMatr.sumElements(mdp_wmu)/TMatr.sumElements(wmu);
    mdpaver_mu=mdpmuaw/muaw;
    mp_mu=TMatr.sumElements(mp_wmu)/TMatr.sumElements(wmu);
    mpaver_mu=mpmuaw/muaw;

    this.pc=pc;
    this.pt=pt;
    this.pi=pi;
    this.dpt=dpt;
    this.dpi=dpi;
    this.dp_it_mu=TMatr.mulVV(wmu,TMatr.sumAcB(pi,-1,pt))/TMatr.sumElements(wmu);
    this.ddp_it_mu=TMatr.mulVV(wmu,TMatr.sumAcB(dpi,-1,dpt))/TMatr.sumElements(wmu);
    double ssk=0;for(int k=0;k<pc.length;k++) ssk+=wmu[k]*pc[k].corr;
    this.pcorr_mu=ssk/TMatr.sumElements(wmu);

}

/**
 * The Mu-measure: dmu=dv-I_{low}*dt is back.
 */
public static MuDynMicroImpact get_MuDynMicroImpact(final IMomentsData mm,
						    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
						    final double eps)
{
    final int n=mm.n,ixH=n-1;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);

    // Try Mu-measure, (diskrete measure, built on I.psiL roots).
    final double [] psi=I.psiL,x=M.getXNodes(psi).re_root,
	wmu=new double[x.length],p_wmu=new double[x.length],dp_wmu=new double[x.length],
	mp_wmu=new double[x.length],mdp_wmu=new double[x.length],
	pt=new double[x.length],pi=new double[x.length],
	dpt=new double[x.length],dpi=new double[x.length];
    final ProbabilityCorrelation [] pc=new ProbabilityCorrelation[x.length];
    final double lam=I.sL,p_lam=TMatr.getAMatrA(psi,mm.QQpi,n),dp_lam=TMatr.getAMatrA(psi,mm.QQDpwv,n),
	dpmuaw=mm.QQDpwv[0]-dp_lam*mm.QQt[0],pmuaw=mm.QQpi[0]-p_lam*mm.QQt[0],muaw=mm.QQi[0]-lam*mm.QQt[0],
	mdpmuaw=mm.QQDpwv[0]-lam*mm.QQDpwt[0]*(mm.QQt[0]/mm.QQt_sample[0]),
	mpmuaw=mm.QQpi[0]-lam*mm.QQpt_sample[0]*(mm.QQt[0]/mm.QQt_sample[0]);
    for(int k=0;k<x.length;k++)
    {
	final double xk=x[k];
	final double [] l=M.B.sdiv1(psi,xk).q;
	final double nrmlk=M.B.getD0(l,xk);
	TMatr.scaleVectorInPlace(l,1/nrmlk);
	final double m0t=TMatr.getAMatrA(l,mm.QQt,n),m0I=TMatr.getAMatrA(l,mm.QQi,n),
	    p_m0I=TMatr.getAMatrA(l,mm.QQpi,n),dp_m0I=TMatr.getAMatrA(l,mm.QQDpwv,n);
	wmu[k]=m0I-lam*m0t;
	p_wmu[k]=p_m0I-p_lam*m0t;
	dp_wmu[k]=dp_m0I-dp_lam*m0t;

	final double m1t=TMatr.getAMatrB(l,psi,mm.QQt,n)/nrmlk,m2t=1/Math.pow(nrmlk,2),
	    m1I=lam*m1t,m2I=lam*m2t,dp_m1I=TMatr.getAMatrB(l,psi,mm.QQDpwv,n)/nrmlk,dp_m2I=dp_lam/Math.pow(nrmlk,2);
	pc[k]=new ProbabilityCorrelation(m0t,m1t,m2t,dp_m0I,dp_m1I,dp_m2I,m0I,m1I,m2I,true);

	final double norm_adj=m0t/TMatr.getAMatrA(l,mm.QQt_sample,n), // almost 1
	    dp_m0t=TMatr.getAMatrA(l,mm.QQDpwt,n)*norm_adj,
	    p_m0t=TMatr.getAMatrA(l,mm.QQpt_sample,n)*norm_adj;
	pi[k]=p_m0I/m0I;
	pt[k]=p_m0t/m0t;
	dpi[k]=dp_m0I/m0I;
	dpt[k]=dp_m0t/m0t;
	mp_wmu[k]=p_m0I-lam*p_m0t;
	mdp_wmu[k]=dp_m0I-lam*dp_m0t;
	if(eps>0)
	{
	    final double [] lxx=M.B.getaxbP(l,1,-xk);
	    UnitTests.checkDoubleRel(m0t,"m0t",TMatr.getAMatrA(l,mm.QQt,n),"<l|l>",eps);
	    UnitTests.checkDoubleRel(m1t,"m1t",TMatr.getAMatrB(l,lxx,mm.QQt,n),"<l|x-xk|l>",eps);
	    UnitTests.checkDoubleRel(m2t,"m2t",TMatr.getAMatrA(lxx,mm.QQt,n),"<l|(x-xk)^2|l>",eps);
	    UnitTests.checkDoubleRel(m0I,"m0I",TMatr.getAMatrA(l,mm.QQi,n),"<l|I|l>",eps);
	    UnitTests.checkDoubleRel(m1I,"m1I",TMatr.getAMatrB(l,lxx,mm.QQi,n),"<l|(x-xk)I|l>",eps);
	    UnitTests.checkDoubleRel(m2I,"m2I",TMatr.getAMatrA(lxx,mm.QQi,n),"<l|(x-xk)^2I|l>",eps);
	    UnitTests.checkDoubleRel(dp_m0I,"dp_m0I",TMatr.getAMatrA(l,mm.QQDpwv,n),"<l|dpI|l>",eps);
	    UnitTests.checkDoubleRel(dp_m1I,"dp_m1I",TMatr.getAMatrB(l,lxx,mm.QQDpwv,n),"<l|(x-xk)dpI|l>",eps);
	    UnitTests.checkDoubleRel(dp_m2I,"dp_m2I",TMatr.getAMatrA(lxx,mm.QQDpwv,n),"<l|(x-xk)^2dpI|l>",eps);
	    UnitTests.checkDoubleRel(pc[k].iL,"pc[k].iL",lam,"lam",eps);
	}
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(M.B.getD0(l,xk),"l(xk)",1,"1",eps);
	    UnitTests.checkDoubleRel(M.B.getD0(psi,xk),"psi(xk)",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(l,mm.QQt,n),"<l|l>",m0t,"m0t",eps);
	    if(true)
	    {
		final double [] xIH=M.getXNodes(I.psiH).re_root,a=new double [x.length+xIH.length];
		System.arraycopy(x,0,a,0,x.length);System.arraycopy(xIH,0,a,x.length,xIH.length);
		final double [] ONE=M.B.getONE(),RONE=M.getRadauMultiplied(ONE);
		final boolean flag_dir_right=M.B.getD1(RONE,0)<0;
		if(flag_dir_right) com.polytechnik.utils.SortArrays.sortArrayWithLoad(a.length,a,null);
		else com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(a.length,a,null);
		System.err.print("MuDynMicroImpact:psiL&psiH ROOTS ORDER:");
		for(double r:a)
		{
		b:{for(double y:x){if(y==r){System.err.print("L");break b;}}
		   for(double y:xIH){if(y==r){System.err.print("H");break b;}}
		  }
		}
		System.err.println(": roots xIL="+java.util.Arrays.toString(x)+" xIH="+java.util.Arrays.toString(xIH));
	    }
	    System.err.println("MuDynMicroImpact: test OK, x["+k+"]="+xk+" m0t="+m0t+" m0t_sample="+TMatr.getAMatrA(l,mm.QQt_sample,n));
	}
    }

    if(eps>0)
    {
	// Check eigenvalues problem is solved OK.
	EVXData.test_orthogonality("MuDynMicroImpact.I.ev",n,I.ev,mm.QQi,mm.QQt,eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,TMatr.sumAcB(mm.QQi,-lam,mm.QQt),n),"<psi|QQmu|psi>",0,"0",eps);
	System.err.println("MuDynMicroImpact:x="+java.util.Arrays.toString(x)+
			   " wmu="+java.util.Arrays.toString(wmu)+
			   " dp_wmu="+java.util.Arrays.toString(dp_wmu)+
			   " sum(wmu)="+TMatr.sumElements(wmu)+
			   " sum(dp_wmu)="+TMatr.sumElements(dp_wmu)+" n="+n);
	UnitTests.checkDoubleRel(TMatr.sumElements(wmu),"sum(wmu)",muaw,"muaw",eps);
	System.err.println("MuDynMicroImpact.get_MuDynMicroImpact: test2 internal tests OK, all invariance tests OK n="+n+" eps="+eps);
    }

    return new MuDynMicroImpact(I,x,wmu,pc,dp_wmu,p_wmu,dpmuaw,pmuaw,muaw,
				mdp_wmu,mp_wmu,mdpmuaw,mpmuaw,
				pi,pt,dpi,dpt,mm,M,eps);
}

public static void matchMuDynMicroImpact(final MuDynMicroImpact a,final MuDynMicroImpact b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    if(true)
    { // they may be sorted differently
	final int [] ind_a=com.polytechnik.utils.SortArrays.sortArrayIndexes(a.wmu),
	    ind_b=com.polytechnik.utils.SortArrays.sortArrayIndexes(b.wmu);
	for(int k=0;k<ind_a.length;k++)
	{
	    final int ia=ind_a[k],ib=ind_b[k];
	    UnitTests.checkDoubleRel(a.wmu[ia],"a.wmu[ia]",b.wmu[ib],"b.wmu[ib]",eps);
	    UnitTests.checkDoubleRel(a.dp_wmu[ia],"a.dp_wmu[ia]",b.dp_wmu[ib],"b.dp_wmu[ib]",eps);
	    UnitTests.checkDoubleRel(a.p_wmu[ia],"a.p_wmu[ia]",b.p_wmu[ib]+p_offset_shift*b.wmu[ib],"b.p_wmu[ib]+p_offset_shift*b.wmu[ib]",eps);
	    UnitTests.checkDoubleRel(a.pi[ia],"a.pi[ia]",b.pi[ib]+p_offset_shift,"b.pi[ib]+p_offset_shift",eps);
	    UnitTests.checkDoubleRel(a.pt[ia]-p_offset_shift,"a.pt[ia]-p_offset_shift",b.pt[ib],"b.pt[ib]",eps);
	    UnitTests.checkDoubleRel(a.dpi[ia],"a.dpi[ia]",b.dpi[ib],"b.dpi[ib]",eps);
	    UnitTests.checkDoubleRel(a.dpt[ia],"a.dpt[ia]",b.dpt[ib],"b.dpt[ib]",eps);
	    ProbabilityCorrelation.matchProbabilityCorrelation(a.pc[ia],b.pc[ib],eps);
	}
    }
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.pt_IH,"a.pt_IH",b.pt_IH+p_offset_shift,"b.pt_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a._pt_IH,"a._pt_IH",b._pt_IH+p_offset_shift,"b._pt_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dpdt_orig_IH,"a.dpdt_orig_IH",b.dpdt_orig_IH,"b.dpdt_orig_IH",eps);
    UnitTests.checkDoubleRel(a.dp_wv_IH,"a.dp_wv_IH",b.dp_wv_IH,"b.dp_wv_IH",eps);
    UnitTests.checkDoubleRel(a.dp_wt_IH,"a.dp_wt_IH",b.dp_wt_IH,"b.dp_wt_IH",eps);
    //UnitTests.checkDoubleRel(a.sk_dp_wv_IH,"a.sk_dp_wv_IH",b.sk_dp_wv_IH,"b.sk_dp_wv_IH",eps);


    UnitTests.checkDoubleRel(a.p_mu,"a.p_mu",b.p_mu+p_offset_shift,"b.p_mu+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dp_mu,"a.dp_mu",b.dp_mu,"b.dp_mu",eps);
    UnitTests.checkDoubleRel(a.dp_it_mu,"a.dp_it_mu",b.dp_it_mu,"b.dp_it_mu",eps);
    UnitTests.checkDoubleRel(a.ddp_it_mu,"a.ddp_it_mu",b.ddp_it_mu,"b.ddp_it_mu",eps);
    UnitTests.checkDoubleRel(a.pcorr_mu,"a.pcorr_mu",b.pcorr_mu,"b.pcorr_mu",eps);

    System.err.println("MuDynMicroImpact.matchMuDynMicroImpact test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}

public static void matchMu2DynMicroImpact(final MuDynMicroImpact a,final Mu2DynMicroImpact b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    if(true)
    { // they may be sorted differently
	final int [] ind_a=com.polytechnik.utils.SortArrays.sortArrayIndexes(a.wmu),
	    ind_b=com.polytechnik.utils.SortArrays.sortArrayIndexes(b.muL.wmu);
	for(int k=0;k<ind_a.length;k++)
	{
	    final int ia=ind_a[k],ib=ind_b[k];
	    UnitTests.checkDoubleRel(a.wmu[ia],"a.wmu[ia]",b.muL.wmu[ib],"b.muL.wmu[ib]",eps);
	    UnitTests.checkDoubleRel(a.p_wmu[ia],"a.p_wmu[ia]",b.muL.p_wmu[ib]+p_offset_shift*b.muL.wmu[ib],"b.p_wmu[ib]+p_offset_shift*b.wmu[ib]",eps);
	}
    }
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.pt_IH,"a.pt_IH",b.pt_IH+p_offset_shift,"b.pt_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_mu,"a.p_mu",b.muL.p_Mu+p_offset_shift,"b.muL.p_Mu+p_offset_shift",eps);

    System.err.println("MuDynMicroImpact.matchMu2DynMicroImpact test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}
}

