package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import java.util.*;
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.ReadWriteUtils;

/** A class for backtesting- trade all stocks.
 *  @version $Id: MultiALLStockModelParameters.java,v 1.9 2012/01/13 13:08:57 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class MultiALLStockModelParameters extends  MultiStockModelParameters {

    private final String [] ind;
    MultiALLStockModelParameters(final int DEFAULT_ND,
				 final double DEFAULT_THR_ENTER,
				 final double DEFAULT_THR_EXIT,
				 final long DEFAULT_TAU,
				 final long T_QUANT,
				 final String OUTPUT_DIR,
				 final java.util.Map<String,MultiStockModelParametersData> active,
				 final String [] ind)
    {
	super(DEFAULT_ND,DEFAULT_THR_ENTER,DEFAULT_THR_EXIT,DEFAULT_TAU,T_QUANT,OUTPUT_DIR,active);
	this.ind=ind;
    }


    /** Consider all stocks as active. Used for backtesting. */
    MultiStockModelParametersData getParameters(final String ticker)
    {

	MultiStockModelParametersData m=super.getParameters(ticker);
	if(m!=null)
	{
	    return m;
	}
	return new MultiStockModelParametersData(ticker,
						 this.DEFAULT_ND,
						 this.DEFAULT_TAU,
						 this.DEFAULT_THR_ENTER,
						 this.DEFAULT_THR_EXIT,
						 removeSelfMatches(this.ind,ticker),
						 this.T_QUANT,
						 // all active, all traded
						 true,true,
						 this.OUTPUT_DIR);
    }


    static MultiALLStockModelParameters read(final String fname) throws java.io.IOException
    {
	final ResourceBundle p=ReadWriteUtils.getProperties(fname);
	//System.err.println("keys"+p.keySet());
	final String alt_out=System.getProperty("tr.OUTPUT_DIR");
	final String OUTPUT_DIR=alt_out!=null?alt_out:ReadWriteUtils.getparameter("_OUTPUT_DIR",p,true);
	final int DEFAULT_ND=Integer.valueOf(ReadWriteUtils.getparameter("_DEFAULT_ND",p,true));
	final double DEFAULT_THR_ENTER=Double.valueOf(ReadWriteUtils.getparameter("_DEFAULT_THR_ENTER",p,true));
	final double DEFAULT_THR_EXIT=Double.valueOf(ReadWriteUtils.getparameter("_DEFAULT_THR_EXIT",p,true));
	final long T_QUANT=Long.valueOf(ReadWriteUtils.getparameter("_T_QUANT",p,true))*MILLISECOND_SCALE;
	final long DEFAULT_TAU=Long.valueOf(ReadWriteUtils.getparameter("_DEFAULT_TAU",p,true))*MILLISECOND_SCALE;

	final String globalindstr=ReadWriteUtils.getparameter("_ALLINDS",p,true);
	final String [] ind=ReadWriteUtils.extractnames(globalindstr,",\t\r\n ");
	java.util.Map<String,MultiStockModelParametersData> a=
	    new java.util.HashMap<String,MultiStockModelParametersData>();
	for(String ticker:ind)
	{
	    if(a.get(ticker)!=null)
	    {
		throw new RuntimeException("For ticker="+ticker+" duplicated ticker="+ticker+" found, check _ALLINDS");
	    }
	    a.put(ticker,new MultiStockModelParametersData(ticker,
							   DEFAULT_ND,
							   DEFAULT_TAU,
							   DEFAULT_THR_ENTER,
							   DEFAULT_THR_EXIT,
							   removeSelfMatches(ind,ticker),
							   T_QUANT,
							   true,
							   true,
							   OUTPUT_DIR));
	}

	return new MultiALLStockModelParameters(DEFAULT_ND,
						DEFAULT_THR_ENTER,
						DEFAULT_THR_EXIT,
						DEFAULT_TAU,
						T_QUANT,
						OUTPUT_DIR,
						a,
						ind);
    }


    public static void main(final String [] args) throws java.io.IOException
    {
	if(args.length!=1)
	{
	    System.err.println("MultiALLStockModelParameters.main config_file");
	    System.exit(1);
	}
	final String config_file=args[0];
	System.err.println("Reading config_file="+config_file);
	final MultiALLStockModelParameters ma=read(config_file);

	// dump diag to stderr
	System.err.println(ma.dumpToSting());
    }

}
