package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.broker.*;

/** A class to store multi-scale prediction results.
 *  @version $Id: MultiScalePredictionResults.java,v 1.137 2020/02/08 10:42:27 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */

class MultiScalePredictionResults implements com.polytechnik.utils.CircularList.TimeAccess {


    private static final boolean PRINT_DEBUG=true;


    private final long time;
    private final long vol_p0_buy,vol_p0_sell;
    private final double p0_buy,p0_sell;
    private final ClassificationPredictor.PredictionResults [] OP_DOWN,MED_DOWN,OP_UP,MED_UP;
    private final StatusData sd;
    private double
	t_future_min_p_sell=Double.NaN,
	t_future_max_p_buy=Double.NaN,
	t_speedupexit_min_p_sell=Double.NaN,
	t_speedupexit_max_p_buy=Double.NaN,
	intrn_t_speedupexit_min_p_sell_t_t_speedupexit=Double.NaN,
	intrn_t_speedupexit_max_p_buy_t_t_speedupexit=Double.NaN
	;

    private boolean flag_new=true;
    private long t_future=-1,t_speedupexit=-1;


    private int n_liquidate_attempts=0;
    private long t_start_liquidating=-1,t_lastattempt_liquidating=-1;

    MultiScalePredictionResults(final ClassificationPredictor.PredictionResults [] OP_DOWN,
				final ClassificationPredictor.PredictionResults [] MED_DOWN,
				final ClassificationPredictor.PredictionResults [] OP_UP,
				final ClassificationPredictor.PredictionResults [] MED_UP,
				final StatusData sd,
				final CalculationData cd)
	{
	    this.OP_DOWN=OP_DOWN;
	    this.MED_DOWN=MED_DOWN;
	    this.OP_UP=OP_UP;
	    this.MED_UP=MED_UP;

	    this.sd=sd;
	    this.time=cd.getTime();
	    this.p0_buy=cd.p_buy;
	    this.p0_sell=cd.p_sell;
	    this.vol_p0_buy=cd.vol_p_buy;
	    this.vol_p0_sell=cd.vol_p_sell;
	    if(cd.getTaus()!=this.sd.mparametersdata.tau_embedding)
	    {
		throw new IllegalArgumentException();
	    }
	}

    public long getTime()
    {
	return this.time;
    }

    long [] getTimeReferences()
    {
	/* FIXME
	if(this.t_speedupexit>0)
	{
	    return this.t_speedupexit;
	}
	*/
	return new long []{getTime()};
    }

    private void setSpeedupExitStrategy(final Positions.PositionsInfo pinfo)
    {
	if(!amIPredictorOnTheRun() || this.p0ptr<0)
	{
	    throw new IllegalStateException(" amIPredictorOnTheRun="+amIPredictorOnTheRun()+
					    " p0ptr="+this.p0ptr+
					    " pinfp="+pinfo+
					    " t_predictor="+getTime());
	}
	if(PRINT_DEBUG)
	{
	    System.err.println("setSpeedupExitStrategy: ticker="+this.sd.ticker+
			       " pinfo="+pinfo+
			       " t_speedupexit="+this.t_speedupexit+
			       " time="+(pinfo!=null?pinfo.time:-1L)+
			       " opportunity_direction="+this.opportunity_direction);
	}
	if(pinfo==null)
	{
	    this.t_speedupexit=-1;
	    this.intrn_t_speedupexit_min_p_sell_t_t_speedupexit=
		this.intrn_t_speedupexit_max_p_buy_t_t_speedupexit=Double.NaN;
	}
	else if(!(this.t_speedupexit>0))
	{
	    // speedexit not set
	    this.t_speedupexit=pinfo.time;
	    this.intrn_t_speedupexit_min_p_sell_t_t_speedupexit=this.t_future_min_p_sell;
	    this.intrn_t_speedupexit_max_p_buy_t_t_speedupexit=this.t_future_max_p_buy;
	    if(this.t_speedupexit!=this.t_future)
	    {
		throw new IllegalStateException();
	    }
	}
    }
    void setFutureInfo(final CalculationData cd)
    {
	if(!amIPredictorOnTheRun() || this.p0ptr<0)
	{
	    throw new IllegalStateException();
	}
	this.t_future=cd.getTime();

	if(this.t_speedupexit>0)
	{
	    if(PRINT_DEBUG)
	    {
		// get last element only for debugging.
		// the result is correct only when running in single thread mode.
		final com.polytechnik.realtimebook.Trade lastt=
		    this.sd.ot.tradebook.getLastAddedElement();
		System.err.println("setFutureInfo:speedupexit IS SET: ticker="+this.sd.ticker+
				   " time="+this.t_future+
				   " t_speedupexit="+this.t_speedupexit+
				   //" cd.t_watch="+cd.t_watch+ // FIXME
				   " t_start="+getTime()+
				   " opportunity_direction="+this.opportunity_direction+
				   " cd.p_sell="+cd.p_sell+
				   //" cd.min_sell_t_watch="+cd.min_sell_t_watch+ // FIXME
				   " cd.p_buy="+cd.p_buy+
				   //" cd.max_buy_t_watch="+cd.max_buy_t_watch+ // FIXME
				   (lastt!=null?
				    " lastt.time="+lastt.time+
				    " lastt.price="+lastt.price+
				    " lastt.type="+lastt.tradetype+
				    " lastt.time_diff=" //+(lastt.time-cd.t_watch) FIXME
				    :
				    " lastt=null")+
				   " trades last creared="+this.sd.ot.tradebook.getLastClearedBefore()+
				   " intrn_t_speedupexit_min_p_sell_t_t_speedupexit="+this.intrn_t_speedupexit_min_p_sell_t_t_speedupexit+
				   " intrn_t_speedupexit_max_p_buy_t_t_speedupexit="+this.intrn_t_speedupexit_max_p_buy_t_t_speedupexit+
				   " t_future_min_p_sell="+this.t_future_min_p_sell+
				   " t_future_max_p_buy="+this.t_future_max_p_buy
				   );
	    }
	    // calculate min sell & max buy ny joining the intervals
	    /* FIXME
	    this.t_speedupexit_min_p_sell=cd.min_sell_t_watch;
	    this.t_speedupexit_max_p_buy=cd.max_buy_t_watch;

	    if(Double.isNaN(this.intrn_t_speedupexit_min_p_sell_t_t_speedupexit))
	    {
		this.t_future_min_p_sell=cd.min_sell_t_watch;
	    }
	    else if(Double.isNaN(cd.min_sell_t_watch))
	    {
		this.t_future_min_p_sell=this.intrn_t_speedupexit_min_p_sell_t_t_speedupexit;
	    }
	    else
	    {
		this.t_future_min_p_sell=
		    cd.min_sell_t_watch<=this.intrn_t_speedupexit_min_p_sell_t_t_speedupexit?
		    cd.min_sell_t_watch:this.intrn_t_speedupexit_min_p_sell_t_t_speedupexit;
	    }
	    if(Double.isNaN(this.intrn_t_speedupexit_max_p_buy_t_t_speedupexit))
	    {
		this.t_future_max_p_buy=cd.max_buy_t_watch;
	    }
	    else if(Double.isNaN(cd.max_buy_t_watch))
	    {
		this.t_future_max_p_buy=this.intrn_t_speedupexit_max_p_buy_t_t_speedupexit;
	    }
	    else
	    {
		this.t_future_max_p_buy=
		    cd.max_buy_t_watch>=this.intrn_t_speedupexit_max_p_buy_t_t_speedupexit?
		    cd.max_buy_t_watch:this.intrn_t_speedupexit_max_p_buy_t_t_speedupexit;
	    }
	    if(cd.t_watch!=this.t_speedupexit)
	    {
		throw new IllegalStateException();
	    }
	    */
	}
	else
	{
	    /* FIXME
	    // no intervals to join
	    this.t_speedupexit_min_p_sell=
		this.t_speedupexit_max_p_buy=Double.NaN;
	    this.t_future_min_p_sell=cd.min_sell_t_watch;
	    this.t_future_max_p_buy=cd.max_buy_t_watch;
	    if(cd.t_watch!=getTime())
	    {
		throw new IllegalStateException();
	    }
	    */
	}

	if(PRINT_DEBUG)
	{
	    // CHECK EXTRA CONSTRAINS
	    // turn off later on
	    final long tw=getTime();
	    final int [] nc=new int [4];
	    // try to recalculate min buy & max sell
	    // the result is correct only when running in single thread mode.
	    final double [] mm=new double []{Double.NEGATIVE_INFINITY,Double.POSITIVE_INFINITY,Double.NEGATIVE_INFINITY,Double.POSITIVE_INFINITY};
	    this.sd.ot.tradebook.backwardForEachEntry(new java.util.function.Predicate<com.polytechnik.realtimebook.Trade>(){
		    public boolean test(final com.polytechnik.realtimebook.Trade t)
		    {
			if(!(t.time>=tw))
			{
			    return false;
			}
			if(t.buy_sell_indicator==com.polytechnik.realtimebook.BookItem.BOOK_BUY)
			{
			    /* FIXME
			    if(t.time>=cd.t_watch)
			    {
				nc[0]++;
				if(t.price>mm[0])
				{
				    mm[0]=t.price;
				}
			    }
			    */
			    nc[2]++;
			    if(t.price>mm[2])
			    {
				mm[2]=t.price;
			    }
			}
			else if(t.buy_sell_indicator==com.polytechnik.realtimebook.BookItem.BOOK_SELL)
			{
			    /* FIXME
			    if(t.time>=cd.t_watch)
			    {
				nc[1]++;
				if(t.price<mm[1])
				{
				    mm[1]=t.price;
				}
			    }
			    */
			    nc[3]++;
			    if(t.price<mm[3])
			    {
				mm[3]=t.price;
			    }
			}
			return true;
		    }
		},0);
	    /* FIXME
	    if(Double.isInfinite(mm[0])!=Double.isNaN(cd.max_buy_t_watch)||
	       Double.isInfinite(mm[1])!=Double.isNaN(cd.min_sell_t_watch)||
	       Double.isInfinite(mm[2])!=Double.isNaN(this.t_future_max_p_buy)||
	       Double.isInfinite(mm[3])!=Double.isNaN(this.t_future_min_p_sell))
	    {
		throw new IllegalStateException(" cd.max_buy_t_watch="+cd.max_buy_t_watch+
						" recalculated="+mm[0]+
						" cd.min_sell_t_watch="+cd.min_sell_t_watch+
						" recalculated="+mm[1]+
						" t_future_max_p_buy="+this.t_future_max_p_buy+
						" recalculated="+mm[2]+
						" t_future_min_p_sell="+this.t_future_min_p_sell+
						" recalculated="+mm[3]);
	    }
	    if(!Double.isNaN(cd.max_buy_t_watch) && mm[0]!=cd.max_buy_t_watch)
	    {
		throw new IllegalStateException(" cd.max_buy_t_watch="+cd.max_buy_t_watch+
						" recalculated="+mm[0]+
						" t_future="+ this.t_future+
						" t_speedupexit="+this.t_speedupexit+
						" t_watch="+cd.t_watch+
						" nc="+nc[0]);
	    }
	    if(!Double.isNaN(cd.min_sell_t_watch) && mm[1]!=cd.min_sell_t_watch)
	    {
		throw new IllegalStateException(" cd.min_sell_t_watch="+cd.min_sell_t_watch+
						" recalculated="+mm[1]+
						" t_future="+ this.t_future+
						" t_speedupexit="+this.t_speedupexit+
						" t_watch="+cd.t_watch+
						" nc="+nc[1]);
	    }
	    if(!Double.isNaN(this.t_future_max_p_buy) && mm[2]!=this.t_future_max_p_buy)
	    {
		throw new IllegalStateException(" t_future_max_p_buy="+this.t_future_max_p_buy+
						" recalculated="+mm[2]+
						" nc="+nc[2]);
	    }
	    if(!Double.isNaN(this.t_future_min_p_sell) && mm[3]!=this.t_future_min_p_sell)
	    {
		throw new IllegalStateException(" t_future_min_p_sell="+this.t_future_min_p_sell+
						" recalculated="+mm[3]+
						" nc="+nc[3]);
	    }
	    */
	    //System.err.println("setFutureInfo:speedupexit constrains OK");
	}

    }


    private static final int D_UP=1,D_DOWN=-1;

    // to be set by checkOpportunity() if one is found.
    int p0ptr=-1;
    int opportunity_direction=0;
    int [] ptr_to_selected=null;


    private AOrder checkOpportunity(final Positions.PositionsInfo pinfo)
    {
	if(amIPredictorOnTheRun() || !this.flag_new)
	{
	    throw new IllegalStateException();
	}
	final int MIN_NUMBER_CONSEQUITIVE_PREDICTORS=1;

	// check whether a predictor
	// provide predictions with consequtive time,
	// i.e. the market go up for 64,128,256 seconds.
	// return order, if opportunity exists.

	int n_total_down=0,n_cur_down=0,ptr_max_down=-1,n_max_down=0;
	for(int p=0,n=OP_DOWN.length;p<n;p++)
	{
	    if(OP_DOWN[p].result==ClassificationPredictor.P_YES)
	    {
		if(p-1<0||OP_DOWN[p-1].result!=ClassificationPredictor.P_YES)
		{
		    n_cur_down=0;
		}
		n_cur_down++;
		n_total_down++;
		if(n_cur_down>n_max_down)
		{
		    n_max_down=n_cur_down;
		    ptr_max_down=p;
		}
	    }
	}


	int n_total_up=0,n_cur_up=0,ptr_max_up=-1,n_max_up=0;
	for(int p=0,n=OP_UP.length;p<n;p++)
	{
	    if(OP_UP[p].result==ClassificationPredictor.P_YES)
	    {
		if(p-1<0||OP_UP[p-1].result!=ClassificationPredictor.P_YES)
		{
		    n_cur_up=0;
		}
		n_cur_up++;
		n_total_up++;
		if(n_cur_up>n_max_up)
		{
		    n_max_up=n_cur_up;
		    ptr_max_up=p;
		}
	    }
	}

	if(false && PRINT_DEBUG)
	{
	    System.err.println("OPPORTUNITY "+
			       " ticker="+this.sd.ticker+
			       " time="+pinfo.time+
			       " n_max_down="+n_max_down+
			       " n_total_down="+n_total_down+
			       " n_max_up="+n_max_up+
			       " n_total_up="+n_total_up);
	}

	final int shares,side;
	final double price;
	if(n_max_down>=MIN_NUMBER_CONSEQUITIVE_PREDICTORS && n_total_up<=0)
	{
	    // down
	    this.p0ptr=0;
	    this.opportunity_direction=D_DOWN;
	    this.ptr_to_selected=new int [n_max_down];
	    for(int i=0;i<n_max_down;i++)
	    {
		this.ptr_to_selected[i]=(ptr_max_down-n_max_down+1)+i;
		if(OP_DOWN[this.ptr_to_selected[i]].result!=ClassificationPredictor.P_YES)
		{
		    throw new IllegalStateException();
		}
	    }
	    shares=this.sd.moneymanager.getAvailableShort(pinfo.time,pinfo.p_buy,pinfo.p_sell,0,0);
	    side=BOrder.SIDE_SELL_SHORT;
	    price=pinfo.p_buy;
	}
	else if(n_max_up>=MIN_NUMBER_CONSEQUITIVE_PREDICTORS && n_total_down<=0)
	{
	    // up
	    this.p0ptr=0;
	    this.opportunity_direction=D_UP;
	    this.ptr_to_selected=new int [n_max_up];
	    for(int i=0;i<n_max_up;i++)
	    {
		this.ptr_to_selected[i]=(ptr_max_up-n_max_up+1)+i;
		if(OP_UP[this.ptr_to_selected[i]].result!=ClassificationPredictor.P_YES)
		{
		    throw new IllegalStateException();
		}
	    }
	    shares=this.sd.moneymanager.getAvailableLong(pinfo.time,pinfo.p_buy,pinfo.p_sell,0,0);
	    side=BOrder.SIDE_BUY;
	    price=pinfo.p_sell;
	}
	else
	{
	    // nothing found
	    this.p0ptr=-1;
	    this.opportunity_direction=0;
	    this.ptr_to_selected=null;
	    return null;
	}

	/* FIXME
	return new AOrder(pinfo.time,
			  this.sd.moneymanager.getTimeToDie_Opportunity(pinfo.time),
			  price,
			  shares,
			  side,
			  BOrder.OTYPE_LIMIT,
			  new OrderAttributes("Opportunity",
					      sd.getStrategyID(),
					      OrderAttributes.ORDERATTRIB_ADDNEW));
	*/
	return null;
    }

    private AOrder checkHoldedPositions(final Positions.PositionsInfo pinfo)
    {
	if(!amIPredictorOnTheRun() ||
	   this.t_future<=0 ||
	   this.t_future!=pinfo.time ||
	   this.p0ptr<0)
	{
	    throw new IllegalStateException();
	}

	final AOrder sl=checkStopLoss(pinfo);
	if(sl!=null)
	{
	    return sl;
	}
	// for closing we treat orders same way as position.
	final int shares_eff=pinfo.getEffShares();
	if(shares_eff==0)
	{
	    return null;
	}

	// exit strategy.
	// for moves up
	// think about cur/max and cur/initial ratios
	// if cur/initial large enough and cur/max is say few percent below 1
	// then think about an exit.
	// use similar behaviour for down move and min
	// also take into account when last time there was a signal up/down

	final int p=this.ptr_to_selected[this.p0ptr];
	final long tau=this.sd.mparametersdata.tau_embedding_consequent[p];
	final boolean flag_predictor_expired=!(pinfo.time<=(getTime()+tau));
	final ClassificationPredictor.PredictionResults pr;
	final boolean
	    opportunity_now_crossed,
	    opportunity_mm_crossed,
	    alt_behaviour_is_good,
	    was_positive_move;
	final double suboptimal_fraction,suboptimal_fraction_speedexit;
	if(this.opportunity_direction==D_UP)
	{
	    if(shares_eff<0)
	    {
		// for some reason wrong direction
		return getOrderToClose(-1 /*pinfo.trade_info.current_shares*/,pinfo,"Wrong position direction: sell instead of buy");

	    }
	    pr=OP_UP[p];
	    // move up, can we sell existing position (exchange buy) at profit
	    opportunity_now_crossed=pinfo.p_buy>pr.pricelevel;
	    alt_behaviour_is_good=pinfo.p_sell<OP_DOWN[p].pricelevel;
	    opportunity_mm_crossed=this.t_future_max_p_buy>pr.pricelevel;
	    was_positive_move=this.t_future_max_p_buy>this.p0_sell;
	    final double d=(this.t_future_max_p_buy-this.p0_sell);
	    suboptimal_fraction=(this.t_future_max_p_buy-pinfo.p_buy)/d;
	    suboptimal_fraction_speedexit=(this.t_speedupexit_max_p_buy-pinfo.p_buy)/d;
	}
	else if(this.opportunity_direction==D_DOWN)
	{
	    if(shares_eff>0)
	    {
		// for some reason wrong direction
		return getOrderToClose(-1 /*pinfo.trade_info.current_shares*/,pinfo,"Wrong position direction: buy instead of sell");

	    }
	    pr=OP_DOWN[p];
	    // move down, can we buy to cover (exchange sell) at profit
	    opportunity_now_crossed=pinfo.p_sell<pr.pricelevel;
	    alt_behaviour_is_good=pinfo.p_buy>OP_UP[p].pricelevel;
	    opportunity_mm_crossed=this.t_future_min_p_sell<pr.pricelevel;
	    was_positive_move=this.t_future_min_p_sell<this.p0_buy;
	    final double d=(this.t_future_min_p_sell-this.p0_buy);
	    suboptimal_fraction=(this.t_future_min_p_sell-pinfo.p_sell)/d;
	    suboptimal_fraction_speedexit=(this.t_speedupexit_min_p_sell-pinfo.p_sell)/d;
	}
	else
	{
	    throw new IllegalStateException();
	}
	if(tau!=pr.tau)
	{
	    throw new IllegalArgumentException();
	}

	final boolean flag_lastpredictor=(this.p0ptr+1)>=this.ptr_to_selected.length;
	boolean time_extension_available=was_positive_move && !(suboptimal_fraction>=this.sd.moneymanager.fraction_signal_normalexit);
	final double frac_tau_passed=((double)(pinfo.time-getTime()))/tau;


	// a check for speedup exit
	final boolean speedup_exit;
	if(this.t_speedupexit>0)
	{
	    if(was_positive_move &&
	       !(suboptimal_fraction_speedexit>=this.sd.moneymanager.fraction_signal_speedexit))
	    {
		speedup_exit=false;
		time_extension_available=true;
	    }
	    else
	    {
		speedup_exit=true;
	    }
	}
	else
	{
	    speedup_exit=false;
	}


	final String msg;
	// Check for closing in case of success or failure.
	if(alt_behaviour_is_good)
	{
	    msg="Predictor failed, opposite direction move";
	}
	else if(speedup_exit)
	{
	    msg="Speedup exit suboptimal_fraction="+suboptimal_fraction+
		" suboptimal_fraction_speedexit="+suboptimal_fraction_speedexit;
	}
	else if(time_extension_available)
	{
	    // got an extension
	    msg=null;
	}
	else if(flag_lastpredictor && opportunity_mm_crossed)
	{
	    msg="Predictor OK, goal achieved";
	}
	else if(flag_predictor_expired)
	{
	    if(!opportunity_mm_crossed)
	    {
		msg="Predictor did not reach the target";
	    }
	    else if(!flag_lastpredictor)
	    {
		// we have next scale prediction available.
		msg=null;
	    }
	    else
	    {
		msg="Predictor expired, goal achieved";
	    }
	}
	else
	{
	    // no exit yet
	    msg=null;
	}

	if(PRINT_DEBUG)
	{
	    System.err.println("checkHoldedPositions: ticker="+this.sd.ticker+
			       " time="+pinfo.time+
			       " opportunity_direction="+this.opportunity_direction+
			       " shares_eff="+shares_eff+
			       " P_RET>1="+(this.opportunity_direction==D_UP?
					    pinfo.p_buy/this.p0_sell:1.0/(pinfo.p_sell/this.p0_buy))+
			       " LEVEL="+pr.pricelevel+
			       " msg="+msg+
			       " t0="+getTime()+
			       " p0_buy="+this.p0_buy+
			       " p_buy current="+pinfo.p_buy+
			       " p0_sell="+this.p0_sell+
			       " p_sell current="+pinfo.p_sell+
			       " t_future="+this.t_future+
			       " t_future_min_p_sell="+this.t_future_min_p_sell+
			       " t_future_max_p_buy="+this.t_future_max_p_buy+
			       " was_positive_move="+was_positive_move+
			       " t_speedupexit="+this.t_speedupexit+
			       " t_speedupexit_min_p_sell="+this.t_speedupexit_min_p_sell+
			       " t_speedupexit_max_p_buy="+this.t_speedupexit_max_p_buy+
			       " intrn_t_speedupexit_min_p_sell_t_t_speedupexit="+this.intrn_t_speedupexit_min_p_sell_t_t_speedupexit+
			       " intrn_t_speedupexit_max_p_buy_t_t_speedupexit="+this.intrn_t_speedupexit_max_p_buy_t_t_speedupexit+
			       " time_extension_available="+time_extension_available+
			       " flag_lastpredictor="+flag_lastpredictor+
			       " suboptimal_fraction="+suboptimal_fraction+
			       " LEV_so="+this.sd.moneymanager.fraction_signal_normalexit+
			       " suboptimal_fraction_speedexit="+suboptimal_fraction_speedexit+
			       " LEV_sose="+(suboptimal_fraction_speedexit+this.sd.moneymanager.fraction_signal_speedexit)+
			       " LEV_sose0="+suboptimal_fraction_speedexit+
			       " opportunity_mm_crossed="+opportunity_mm_crossed+
			       " opportunity_now_crossed="+opportunity_now_crossed+
			       " Version: $Id: MultiScalePredictionResults.java,v 1.137 2020/02/08 10:42:27 mal Exp $\n"
			       );
	}


	if(msg!=null)
	{
            // close the positions
	    return getOrderToClose(-1/*pinfo.trade_info.current_shares*/,pinfo,msg+" IND="+this.p0ptr+" PREDICTOR="+pr.getDesctiption());
	}


	// now check wheter to switch to the next predictor
	if((flag_predictor_expired||opportunity_mm_crossed) && !flag_lastpredictor)
	{
	    // switch to next predictor by timeout or crossing event occured
	    this.p0ptr++;
	}


	return null; // continue to hold
    }


    private final int N_ATTEMPTS_WHEN_FORCE_MARKETORDERS=3;


    private AOrder getOrderToClose(final int position_shares_signed,
				   final Positions.PositionsInfo pinfo,
				   final String description)
    {
	if(!amIPredictorOnTheRun())
	{
	    throw new IllegalStateException();
	}

	if(position_shares_signed==0)
	{
	    // dummy order, 0 size
	    /* FIXME
	    return new AOrder(pinfo.time,
			      this.sd.moneymanager.getTimeToDie_ClosePostion(pinfo.time),
			      Double.NaN,
			      0,
			      BOrder.SIDE_BUY_COVER, // does not matter
			      BOrder.OTYPE_MARKET,
			      new OrderAttributes(description,
						  this.sd.getStrategyID(),
						  OrderAttributes.ORDERATTRIB_REPLACEALL));
	    */
	    return null;
	}

	this.n_liquidate_attempts++;
	if(this.t_start_liquidating<0)
	{
	    this.t_start_liquidating=pinfo.time;
	}
	this.t_lastattempt_liquidating=pinfo.time;


	final int shares,side;
	final double price;
	if(position_shares_signed>0)
	{ // long position, sell
	    shares=position_shares_signed;
	    side=BOrder.SIDE_SELL;
	    price=pinfo.p_buy;
	}
	else
	{ // short position, buy to cover.
	    shares=-position_shares_signed;
	    side=BOrder.SIDE_BUY_COVER;
	    price=pinfo.p_sell;
	}
	// close position
	/* FIXME
	return new AOrder(pinfo.time,
			  this.sd.moneymanager.getTimeToDie_ClosePostion(pinfo.time),
			  price,
			  shares,
			  side,
			  getOrderToCloseMarketType(),
			  new OrderAttributes(description,
					      this.sd.getStrategyID(),
					      OrderAttributes.ORDERATTRIB_REPLACEALL));
	*/
	return null;
    }




    private AOrder checkStopLoss(final Positions.PositionsInfo pinfo)
    {

	/*
	if(pinfo.trade_info.current_shares==0 ||
	   ( pinfo.trade_info.current_unrealized_p_and_l
	     >
	     (this.sd.moneymanager.getMaxLoss()-1.0)*Math.abs(pinfo.trade_info.current_money_p_and_l)
	     )
	   )
	{
	    return null;
	}

	// stoploss triggered, close position.
	return getOrderToClose(pinfo.trade_info.current_shares,
			       pinfo,
			       "Stoploss close position money="+
			       pinfo.trade_info.current_money_p_and_l+" P&L="+
			       pinfo.trade_info.current_unrealized_p_and_l+
			       " shares="+pinfo.trade_info.current_shares);
	*/
	return null; // FIXME

    }

    private long getTimeNextPredictorAllowed()
    {
	final long mintau=OP_UP[this.ptr_to_selected[this.p0ptr]].tau;
	return  getTime()+mintau/this.sd.moneymanager.frac_nextpredictor_allowed;
    }


    private boolean amIPredictorOnTheRun()
    {
	/* FIXME
	if(this.sd.getPredictorOnTheRun()==this)
	{
	    return true;
	}
	*/
	return false;
    }


    /** Post a trade if preduction is OK.
     *	@return next time to check opportunity.
     */
    long tradeIfOpportunityIsGood(final BrokerInterface broker)
    {
	/* FIXME
	if(amIPredictorOnTheRun() || !this.flag_new)
	{
	    throw new IllegalStateException();
	}

	final Positions.PositionsInfo pinfo;
	final boolean empty_book;
	synchronized(this.sd.positions)
	{
	    pinfo=this.sd.positions.getInfo(getTime(),this.p0_buy,this.p0_sell);
	    empty_book=(pinfo.order_info.orders_not_cancelled==null && pinfo.trade_info.current_shares==0);
	    if(empty_book &&
	       (this.sd.getGoal()==StatusData.SGOAL_LIQUIDATE||
		this.sd.getGoal()==StatusData.SGOAL_HOLD) )
	    {
		this.sd.setGoal(StatusData.SGOAL_WATCHOPPORTUNITY);
		// FIXME this.sd.setPredictorOnTheRun(null);
		this.sd.nextStrategy();
		this.sd.positions.initiateNewStrategy(pinfo.time);
	    }
	}

	if(false && PRINT_DEBUG)
	{
	    System.err.println("OPPORTUNITY status="+this.sd.getGoal()+
			       " ticker="+this.sd.ticker+
			       " time="+pinfo.time+
			       //" POR="+this.sd.getPredictorOnTheRun()+
			       " ttostart="+this.sd.moneymanager.getTimeToStart()+
			       " ttoend="+this.sd.moneymanager.getTimeToEnd());
	}

	// report current orders & trades info to MoneyManager
	this.sd.moneymanager.notifyPositionsInfo(pinfo);

	AOrder order=null;
	long tau=this.sd.moneymanager.getTimeNextCheck_OpenPostition(pinfo.time);

	if(this.sd.getGoal()==StatusData.SGOAL_WATCHOPPORTUNITY)
	{
	    if(!empty_book)
	    {
		// watch opportunity got some "come late" positions, close immediately
		this.sd.setGoal(StatusData.SGOAL_LIQUIDATE);
		// FIXME this.sd.setPredictorOnTheRun(this);
		order=this.getOrderToClose(pinfo.trade_info.current_shares,
					   pinfo,
					   "Come late position close immediately");
	    }
	    else
	    {
		// empty book
		order=checkOpportunity(pinfo);
		if(order!=null &&
		   order.getShares()>0 &&
		   pinfo.time>this.sd.moneymanager.getTimeToStart() &&
		   pinfo.time<this.sd.moneymanager.getTimeToEnd())
		{
		    // if we found an opportunity - switch state to hold a position
		    this.sd.setGoal(StatusData.SGOAL_HOLD);
		    // FIXME this.sd.setPredictorOnTheRun(this);
		    if(PRINT_DEBUG)
		    {
			System.err.println("OPPORTUNITY Watchopportunity found new order="+order+
					   " ticker="+this.sd.ticker+
					   " time="+pinfo.time+
					   " shares with sign="+order.getSharesSign()*order.getShares()+
					   " effshares_existing="+pinfo.getEffShares());
		    }
		}
		else
		{
		    if(PRINT_DEBUG && order!=null)
		    {
			System.err.println("OPPORTUNITY Watchopportunity SKIPPED="+order+
					   " ticker="+this.sd.ticker+
					   " time="+pinfo.time+
					   " shares with sign="+order.getSharesSign()*order.getShares()+
					   " effshares_existing="+pinfo.getEffShares());
		    }
		    order=null; // no more equity for this order if shares=0
		    tau=pinfo.time+this.sd.mparametersdata.tau_watch_opportunities;
		}
	    }
	}
	else if(this.sd.getGoal()==StatusData.SGOAL_HOLD)
	{
            // we are holding positions
	    // check whether to continue to hold.
	    final MultiScalePredictionResults por=null;// FIXME this.sd.getPredictorOnTheRun();
	    if(pinfo.time<this.sd.moneymanager.getTimeToEnd())
	    {
		order=por.checkHoldedPositions(pinfo);
	    }
	    else
	    {
		// close everything at the end of day
		order=por.getOrderToClose(pinfo.trade_info.current_shares,
					  pinfo,
					  "End of day close time="+pinfo.time+
					  " t_start="+this.sd.moneymanager.getTimeToStart()+
					  " t_end="+this.sd.moneymanager.getTimeToEnd());
	    }
	    if(order==null)
	    {
		// continue to hold
		final AOrder op_order=checkOpportunity(pinfo);
		// check opportunity just in case
		if(op_order!=null)
		{
		    final int shares_eff=pinfo.getEffShares();
		    if(PRINT_DEBUG)
		    {
			System.err.println("OPPORTUNITY Hold position found new opportunity order="+op_order+
					   " ticker="+this.sd.ticker+
					   " time="+pinfo.time+
					   " shares with sign="+op_order.getSharesSign()*op_order.getShares()+
					   " effshares_existing="+shares_eff);
		    }
		    // a new opportunity found
		    if(shares_eff==0 || (shares_eff>0?1:-1)==op_order.getSharesSign())
		    {
			// Same direction of found opportunity - increase position if allowed.
			// If direction is opposite - let checkHoldedPositions
			// close the position later on.
			if(shares_eff==0 || por.getTimeNextPredictorAllowed()<=pinfo.time)
			{
			    // FIXME this.sd.setPredictorOnTheRun(this);
			    if(op_order.getShares()>0)
			    {
				if(PRINT_DEBUG)
				{
				    System.err.println("OPPORTUNITY Hold position found new opportunity INCREASE POSITION order="+op_order+
						       " ticker="+this.sd.ticker+
						       " time="+pinfo.time+
						       " shares with sign="+op_order.getSharesSign()*op_order.getShares()+
						       " effshares_existing="+shares_eff+
						       " was speedupexit="+por.t_speedupexit);
				}
				order=op_order;
			    }
			}
			else
			{
			    if(PRINT_DEBUG)
			    {
				System.err.println("OPPORTUNITY Hold position ignore new opportunity IGNORE POSITION order="+op_order+
						   " ticker="+this.sd.ticker+
						   " time="+pinfo.time+
						   " t_allowed="+por.getTimeNextPredictorAllowed()+
						   " shares with sign="+op_order.getSharesSign()*op_order.getShares()+
						   " effshares_existing="+shares_eff+
						   " current t_speedupexit="+por.t_speedupexit);
			    }
			    // turn off speedup exit if any exist.
			    por.setSpeedupExitStrategy(null);
			}
		    }
		    else
		    { // a trend about to turn. speed up exit strategy
			por.setSpeedupExitStrategy(pinfo);
		    }
		}
	    }
	    else
	    {
		// clear speedup exit if exist
		por.setSpeedupExitStrategy(null);
		// order to liquidate received.
		this.sd.setGoal(StatusData.SGOAL_LIQUIDATE);
	    }
	}
	else if(this.sd.getGoal()==StatusData.SGOAL_LIQUIDATE)
	{
	    // when liquidate
	    // if valid market order present - do nothing
	    // if no valid order exist - and new.
	    // take care of partial execution &
	    // asynchroneous order/trade posting
	    final int shares_totalorders,norders,nopposite;
	    if(pinfo.trade_info.current_shares>0)
	    {
		shares_totalorders=pinfo.order_info.shares_sells;
		norders=pinfo.order_info.n_sells;
		nopposite=pinfo.order_info.n_buys;
	    }
	    else if(pinfo.trade_info.current_shares<0)
	    {
		shares_totalorders=pinfo.order_info.shares_buys;
		norders=pinfo.order_info.n_buys;
		nopposite=pinfo.order_info.n_sells;
	    }
	    else
	    {
		throw new IllegalStateException();
	    }

	    if(!(
		   pinfo.order_info.orders_to_die==null &&
		   norders>0 &&
		   nopposite==0)
	       )
	    {
		// liquidate resubmit
		order=null;
                // FIXME this.sd.getPredictorOnTheRun().getOrderToClose(pinfo.trade_info.current_shares,
		// pinfo,
		// "Liquidation resubmit");
	    }
	}
	else
	{
	    throw new IllegalStateException();
	}


	// cancel all or some of orders
	if(pinfo.order_info.orders_not_cancelled!=null &&
	   (order!=null && order.attributes.sideeffect==OrderAttributes.ORDERATTRIB_REPLACEALL)
	   )
	{
	    broker.cancelPerTickerOrders(pinfo.order_info.orders_not_cancelled,sd);
	}
	else if(pinfo.order_info.orders_to_die!=null)
	{
	    broker.cancelPerTickerOrders(pinfo.order_info.orders_to_die,sd);
	}

	// add new order if any
	if(order!=null && order.getShares()>0)
	{
	    // when closing position some 0 size order may indicate
	    // to cancel all previous order. skip such order.
	    broker.addNewOrder(order,sd);
	    if(PRINT_DEBUG)
	    {
		System.err.println("OPPORTUNITY ADD ORDER order="+order+
				   " ticker="+this.sd.ticker+
				   " time="+pinfo.time+
				   " shares with sign="+order.getSharesSign()*order.getShares()+
				   " sign="+order.getSharesSign()+
				   " effshares_existing="+pinfo.getEffShares());
	    }
	}

	this.flag_new=false;
	// let money manager know we are done
	this.sd.moneymanager.notifyPositionsInfo(null);
	return tau;
	*/
	return -1;
    }

}
