package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.realtimebook.OrdersAndTrades;
import com.polytechnik.realtimebook.DumpTradingDataAnalyzable;
import com.polytechnik.broker.TraderContainer;
import com.polytechnik.broker.MoneyManager;

/** A class to store per stock calculations.
 *  @version $Id: MultiStockStatusData.java,v 1.30 2022/06/07 10:52:26 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class MultiStockStatusData extends TraderContainer
{

    final MultiStockModelParameters.MultiStockModelParametersData mparametersdata;

    final MultiStockTradingAlgorithm PREDICTOR;
    final DumpTradingDataAnalyzable<Opportunity.CalculationResults> DIAG;

    private final MultiStockTrader trader;

    /** Reference to ITCH data. */
    final OrdersAndTrades ot;
    final OrdersAndTrades [] iot;

    MultiStockStatusData(final String ticker,
			 final boolean flag_realtime,
			 final MoneyManager moneymanager,
			 final MultiStockTrader trader,
			 final MultiStockModelParameters.MultiStockModelParametersData mparametersdata,
			 final OrdersAndTrades ot)
    {
	super(ticker,flag_realtime,moneymanager);
	this.mparametersdata=mparametersdata;
	this.trader=trader;
	this.ot=ot;
	this.flag_statusOK=trader.DEFAULT_ALLOW_TO_TRADE;
	this.allow_trading=trader.DEFAULT_ALLOW_TO_TRADE;
	if(mparametersdata.flag_active)
	{
	    this.iot=new OrdersAndTrades [mparametersdata.indexstocks.length];
	    final MultiStockTradingAlgorithm te=
		new MultiStockTradingAlgorithm(this,trader);
	    this.PREDICTOR=te;
	    this.DIAG=te;
	}
	else
	{
	    this.iot=null;
	    this.PREDICTOR=null;
	    this.DIAG=null;
	}
    }

    MultiStockStatusData [] references=null;

    /** Sets iot & references for all stocks.
     */
    static void setAllStockReferences(java.util.Map<String,MultiStockStatusData> statusbyticker)
    {
	final java.util.HashMap<String,java.util.LinkedHashSet<MultiStockStatusData>> sx=
	    new java.util.HashMap<String,java.util.LinkedHashSet<MultiStockStatusData>>();
	for(String s:statusbyticker.keySet())
	{
	    final MultiStockStatusData m=statusbyticker.get(s);
	    if(m.mparametersdata.flag_active)
	    {
		java.util.LinkedHashSet<MultiStockStatusData> sa=sx.get(m.ticker);
		if(sa==null)
		{
		    sa=new java.util.LinkedHashSet<MultiStockStatusData>();
		    sx.put(m.ticker,sa);
		}
		sa.add(m);

		for(int i=0;i<m.iot.length;i++)
		{
		    final MultiStockStatusData r=statusbyticker.get(m.mparametersdata.indexstocks[i]);
		    if(r==null || !r.mparametersdata.flag_towatch)
		    {
			throw new RuntimeException("For ticker="+m.ticker+
						   " found non-existent/inactive dependent ticker="+
						   m.mparametersdata.indexstocks[i]);
		    }
		    m.iot[i]=r.ot;
		    sa=sx.get(r.ticker);
		    if(sa==null)
		    {
			sa=new java.util.LinkedHashSet<MultiStockStatusData>();
			sx.put(r.ticker,sa);
		    }
		    sa.add(m);
		}
	    }
	}
	for(String s:sx.keySet())
	{
	    final MultiStockStatusData m=statusbyticker.get(s);
	    final java.util.LinkedHashSet<MultiStockStatusData> sa=sx.get(s);
	    m.references=sa.toArray(MultiStockStatusData[]::new);
	}
    }


    // ============ Check/Set next processing time.
    private long nexttime_check=0L;
    private boolean force_check=false;
    boolean shouldCheck(final long currenttime)
    {
	if(this.force_check)
	{
	    this.force_check=false;
	    return true;
	}
	if(this.mparametersdata.flag_active)
	{
	    return currenttime>=this.nexttime_check;
	}
	return false;
    }
    public void notifyTrader()
    {
	this.force_check=true;
    }
    void setNextCheck(final long nextcheck)
    {
	/* // remove constrains, when taking orders time may decrease
	if(nextcheck<this.nexttime_check)
	{
	    throw new IllegalArgumentException(" nextcheck="+nextcheck+"<"+
					       " prev nexttime_check="+this.nexttime_check);
	}
	*/
	this.nexttime_check=nextcheck;
    }
    // ============ END Check/Set next processing time.



    // ============ Check whether it is allowed to trade.
    //              A combination of global & per stock flags.
    private boolean flag_statusOK;
    void setStatus(final boolean flag_statusOK,
		   final int round_lot_shares,
		   final boolean round_lot_only,
		   final String market)
    {
	this.flag_statusOK=flag_statusOK;
	this.moneymanager.setRountLotSizes(round_lot_shares,
					   round_lot_only,
					   market);
    }
    private boolean allow_trading;
    void setAllowTrading(final boolean on_off)
    {
	this.allow_trading=on_off;
    }
    boolean isAllowToTrade()
    {
	return
	    this.allow_trading && // Per stock flag, may change during trading session.
	    this.flag_statusOK && // Stock status, set in the beginning of trading session.
	    this.trader.allow_trading; // Global (exchange related) flag, may change during trading session.
    }
    // ============ END Check whether it is allowed to trade.


    // ============ To propagate local time from data source.
    private long tradertime=-1L;
    void setTraderCurrentTime(final long tradertime)
    {
	this.tradertime=tradertime;
    }
    public long getTraderCurrentTime()
    {
	return this.tradertime;
    }
    // ============ END To propagate local time from data source.


    // ============ To keep track of currectly ruuning stock processors.
    /** Handled directly by MultiStockTrader. */
    volatile boolean processstarted=false;
    /** Handled directly by MultiStockTrader. */
    long time_processstarted=0;
    // ============ END To keep track of currectly ruuning stock processors.



    UMlQVMLegendreShifted umlqvmp=null;
    UMlQVMLaguerre umlqvml=null;
}
