package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.broker.*;
import com.polytechnik.realtimebook.*;

/** A first version of trading algorith,
 *  @version $Id: MultiStockTradingAlgorithm.java,v 1.29 2020/02/06 01:28:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class MultiStockTradingAlgorithm extends TradingAlgorithm<MultiStockTradingAlgorithm.MultiStockTradingEvent>  implements DumpTradingDataAnalyzable<Opportunity.CalculationResults> {

/** An event we use. */
class MultiStockTradingEvent extends TradingAlgorithm.TradingEvent {
    final boolean flag_up;
    private final long time_to_die;
    final String comments;

    MultiStockTradingEvent(final boolean flag_up,
			   final long time,
			   final double price,
			   final long time_to_die,
			   final String comments,
			   final boolean flag_close_postion)
    {
	super(flag_up?BOrder.SIDE_BUY:BOrder.SIDE_SELL_SHORT,time,price,flag_close_postion);
	this.time_to_die=time_to_die;
	this.comments=comments;

	this.flag_up=flag_up;
    }

    private boolean cancelled=false;
    void setOrdersCancelled()
    {
	this.cancelled=true;
    }

    public boolean shouldOrderBeAlive(final AOrder o,final long t,final double p_buy,final double p_sell)
    {
	if(this.time_to_die<0)
	{
	    throw new RuntimeException("default time_to_die is disabled");
	}
	if(this.cancelled)
	{
	    return false; // manually cancelled
	}
	if(this.time_to_die<t)
	{
	    return false; // timeout
	}
	return true; // order OK
    }

    public String getDescription()
    {
	return
	    "E:"+sdv.ticker+
	    " price="+this.price+
	    " side="+this.flag_up+
	    " t="+BookItem.getTimeDiag(this.time)+
	    " flag_close_postion="+flag_close_postion+
	    " time_to_die="+BookItem.getTimeDiag(this.time_to_die)+
	    " cancelled="+this.cancelled+
	    " "+this.comments;
    }
}
// END OF MultiStockTradingEvent



    private final MultiStockStatusData sdv;
    private final MultiStockTrader trader;

    MultiStockTradingAlgorithm(final MultiStockStatusData sdv,final MultiStockTrader trader)
    {
	super(sdv);
	this.sdv=sdv;
	this.trader=trader;
    }

    private Opportunity.CalculationResults cdr=null;

    void updateInternalState(final Opportunity.CalculationResults cd)
    {
	if( this.cdr!=null)
	{
	    throw new IllegalStateException("Race is somewhere");
	}
	this.cdr=cd;
	// FIXME
	// do fine grained calculations
    }
    void clear()
    {
	this.cdr=null;
    }


    public MultiStockTradingEvent getEmergencyClose(final MultiStockTradingEvent e,final Positions.PositionsInfo pinfo)
    {
	final int shares_signed=pinfo.trade_info.current_shares;
	final boolean flag_dir=!(shares_signed>0);
	//throw new RuntimeException("FIXME");
	return null; // FIXME
    }

    public MultiStockTradingEvent getExitEvents(final MultiStockTradingEvent e,
						final Positions.PositionsInfo pinfo)
    {
	if(e.flag_close_postion)
	{
	    throw new IllegalArgumentException();
	}
	//throw new RuntimeException("FIXME");
	return null; // FIXME
    }


    public MultiStockTradingEvent getOpportunityEvents(final MultiStockTradingEvent event_ontherun,
						       final Positions.PositionsInfo pinfo)
    {
	//throw new RuntimeException("FIXME");
	return null; // FIXME
    }





// =========================== DUMP REPORTS & DIAGNOSTICS =========================

    private int dir_opportunity=0,dir_exit=0;
    /** A hook to dump order posting events to diagnostics file. */
    @Override
    protected void testing_reportPostedOrders(final AOrder o)
    {
	dir_opportunity=dir_exit=0;
	if(o==null) return;
	if(o.side==AOrder.SIDE_BUY||o.side==AOrder.SIDE_SELL_SHORT)
	{
	    dir_opportunity=-o.getSharesSign();
	}
	else if(o.side==AOrder.SIDE_BUY_COVER||o.side==AOrder.SIDE_SELL)
	{
	    dir_exit=-o.getSharesSign();
	}
    }


    private java.io.PrintWriter ticks_dump_trading_diag_data=null;
    private String ticks_dump_trading_diag_data_fname=null;
    public void setTicker()
    {
	if(this.ticks_dump_trading_diag_data_fname!=null) return;
	try{
	    this.ticks_dump_trading_diag_data_fname=
		this.sdv.mparametersdata.OUTPUT_DIR+
		com.polytechnik.outils.EncodeDecode.urlencode(sdv.ticker)+
		".diag.trading";
	    this.ticks_dump_trading_diag_data=new java.io.PrintWriter(this.ticks_dump_trading_diag_data_fname);
	} catch(final java.io.IOException e) {
	    throw new RuntimeException(e);
	}
	System.err.println("Opened data dump diagnostics for ticker="+this.sdv.ticker+" in diagfile="+this.ticks_dump_trading_diag_data_fname);
	final String h=
	    // 5 general
	    "#ticker\tt00_data\tp_buy\tp_sell\tp_last\t"+
	    // 6 from UMlQVM
	    "jcap_0\tjcap_H\tjvarcap_H\tcap_IH\tpnl_IH\tr\t";

	// header
	final String [] hs=com.polytechnik.utils.ReadWriteUtils.extractnames(h,"\t");
	System.err.println("column\tfield for "+this.sdv.ticker+
			   " in diagfile="+this.ticks_dump_trading_diag_data_fname);
	int n=1;
	for(String s:hs)
	{
	    System.err.print(n);
	    System.err.print("\t");
	    System.err.println(s);
	    this.ticks_dump_trading_diag_data.print(s);
	    this.ticks_dump_trading_diag_data.print("("+n+")\t");
	    n++;
	}
	// indexes
	for(OrdersAndTrades iot : sdv.iot)
	{
	    System.err.print(n);
	    System.err.print("\t");
	    final String nm=iot.ticker;
	    System.err.println(nm+"_p_last");
	    System.err.println(nm+"_r_IH");
	    System.err.println(nm+"_cap_IH");
	    this.ticks_dump_trading_diag_data.print(nm+"_p_last");
	    this.ticks_dump_trading_diag_data.print("("+n+")\t");
	    n++;
	    this.ticks_dump_trading_diag_data.print(nm+"_r_IH");
	    this.ticks_dump_trading_diag_data.print("("+n+")\t");
	    n++;
	    this.ticks_dump_trading_diag_data.print(nm+"_cap_IH");
	    this.ticks_dump_trading_diag_data.print("("+n+")\t");
	    n++;
	}
	this.ticks_dump_trading_diag_data.println();
    }

    public void dumpTickData(Opportunity.CalculationResults c)
    {
      if(this.cdr!=c)
      {
	  throw new IllegalStateException("Race is somewhere");
      }

      if(this.ticks_dump_trading_diag_data_fname==null) return;
      //System.err.println(" ticks_dump_trading_diag_data="+ticks_dump_trading_diag_data);
      if(0!=(this.trader.dump_trading_diagnostics & DumpTradingDataAnalyzable.DUMP_TICKS))
      {
	  final UMlQVM<?> q=sdv.umlqvmp;
	  //System.err.println("q="+q+" c="+c+" ticks_dump_trading_diag_data="+ticks_dump_trading_diag_data);
	  this.ticks_dump_trading_diag_data.print(
	      // 5 general parameters
	      sdv.ticker+"\t"+
	      c.time+"\t"+
	      c.p_buy+"\t"+
	      c.p_sell+"\t"+
	      c.p_last+"\t"+
	      //  6 from UMlQVM
/*	      q.jcap_0+"\t"+
	      q.jcap_H+"\t"+
	      q.jvarcap_H+"\t"+
	      q.cap_IH+"\t"+
	      q.pnl_IH+"\t"+
	      q.r+"\t"
*/""
						  );
//	  for(int k=0;k<q.sst.p_last.length;k++)
	  {
/*	      final double p_l=q.sst.p_last[k],p_le=q.sst.rimF[k].Max.Pc;
	      this.ticks_dump_trading_diag_data.print(p_l+"\t"+(q.sst.rimF[k].Max.b/q.sst.rimF[k].Max.a)+"\t"+q.sst.rimF[k].Max.a+"\t");
*/
	  }
	  this.ticks_dump_trading_diag_data.println();
      }
    }


    private void saveOrdersAndTrades(final String txt,final long t,final double p_buy,final double p_sell) throws java.io.IOException
    {
	synchronized(this.sdv.positions)
        {
	    final String fprefix=this.sdv.mparametersdata.OUTPUT_DIR+
		com.polytechnik.outils.EncodeDecode.urlencode(sdv.ticker);
	    final String fname_trades=fprefix+".trades.csv",
		fname_orders=fprefix+".orders.csv";
	    final java.io.PrintWriter otrades=new java.io.PrintWriter(fname_trades);
	    otrades.println("DESCRIPTION\t"+txt);
	    otrades.println("VERSION\t"+"$Id: MultiStockTradingAlgorithm.java,v 1.29 2020/02/06 01:28:24 mal Exp $");
	    otrades.println("SOURCE\t"+this.trader.description);
	    this.sdv.positions.dumpTradesCSV(otrades,t,p_buy,p_sell);
	    otrades.println("VERSION\t"+"$Id: MultiStockTradingAlgorithm.java,v 1.29 2020/02/06 01:28:24 mal Exp $");
	    otrades.close();
	    final java.io.PrintWriter oorders=new java.io.PrintWriter(fname_orders);
	    oorders.println("DESCRIPTION\t"+txt);
	    oorders.println("VERSION\t"+"$Id: MultiStockTradingAlgorithm.java,v 1.29 2020/02/06 01:28:24 mal Exp $");
	    oorders.println("SOURCE\t"+this.trader.description);
	    this.sdv.positions.dumpOrdersCSV(oorders,t,p_buy,p_sell);
	    oorders.println("VERSION\t"+"$Id: MultiStockTradingAlgorithm.java,v 1.29 2020/02/06 01:28:24 mal Exp $");
	    oorders.close();
	}
    }


    @Override
    protected void testing_saveOrdersAndTrades(final String txt,final Positions.PositionsInfo pinfo)
    {
	try {
	    if(0!=(this.trader.dump_trading_diagnostics & DumpTradingDataAnalyzable.DUMP_POSTIONS_TIME_END))
	    {
		saveOrdersAndTrades(txt,pinfo.time,pinfo.p_buy,pinfo.p_sell);
	    }
	} catch(final java.io.IOException e) {
	    throw new RuntimeException(e);
	}
    }


    public void closeTicker()
    {
      try {
        if(0!=(this.trader.dump_trading_diagnostics & DumpTradingDataAnalyzable.DUMP_TICKS))
	{
	    System.err.println("Closing data dump diagnostics for ticker="+this.sdv.ticker+
			       " diagfile="+this.ticks_dump_trading_diag_data_fname);
	    this.ticks_dump_trading_diag_data.close();
	    this.ticks_dump_trading_diag_data=null;
	    this.ticks_dump_trading_diag_data_fname=null;
	}
	if(0!=(this.trader.dump_trading_diagnostics & DumpTradingDataAnalyzable.DUMP_END_POSTIONS))
	{
	    saveOrdersAndTrades("CLOSE FINAL DUMP",this.sdv.getTraderCurrentTime(),Double.NaN,Double.NaN);
	}
      } catch(final java.io.IOException e) {
	  throw new RuntimeException(e);
      }
    }
// =========================== END DUMP REPORTS & DIAGNOSTICS =========================



}


