package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.realtimebook.*;

/** Non-linear Impact QVM calculations.
 *  @version $Id: NQVM.java,v 1.89 2022/07/28 22:30:50 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class NQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;

final double dpexcess0r,sigma2_dpexcess0r,dsexcess0r,sigma2_dsexcess0r,wexcess0r,
             dpdeficit0r,sigma2_dpdeficit0r,dsdeficit0r,sigma2_dsdeficit0r,wdeficit0r,
    dpi0r,sigma2_dpi0r,dsi0r,sigma2_dsi0r,wii0r,dtdv0r,sigma2_dtdv0r,dp00
    ;

final double wii0S,dpi0S,sigma2_dpi0S,dapi0S,sigma2_dapi0S,dki0S,sigma2_dki0S,dS0S,sigma2_dS0S;

final double vplus00,vminus00,wiplus0r,wiminus0r;
final double dpplus0,dpminus0,dvplus0,dvminus0,w0,dpplus00,dpminus00,dvplus00,dvminus00,w00;
final double dpdvdiff_sigma2,dSa,dSw,dSd;

NQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final double [] mom=new double [2*neq-1];
    access.setM(mom,0,access.M_I,odata,mom.length);

    final double [] mom1=new double [mom.length],moms=new double [mom.length];
    access.setM(moms,0,access.M_ISELL,odata,moms.length);
    access.setM(mom1,0,access.M_IBUY,odata,moms.length);
    final double v00=moms[0]+mom1[0];for(int k=moms.length;--k>=0;) moms[k]=(moms[k]-mom1[k])/v00;

    final double [] moma=new double [mom.length];
    access.setM(moma,1,access.M_DPPLUS,odata,mom.length);
    access.setM(mom1,1,access.M_DPMINUS,odata,mom.length);
    for(int k=moma.length;--k>=0;) moma[k]=moma[k]-mom1[k];
    dp00=moma[0];
    access.setM(mom1,1,access.M_DP,odata,mom1.length);

    final double [] QQv=getQQv_Matr(neq,mom),diag_QQtm1=getQQtm1_DiagMatr(neq);
    final double [] momt=new double [2*neq-1];
    momt[0]=fx(1,new double []{1},0);
    final double [][] me=new double [][]{mom1,moms,momt,moma};
    {
    final double [] matr_excess=getQQv_QQtm1_QQv(neq,QQv,diag_QQtm1);
    final double [][] rr=getRadauLikeBasisVector(neq,matr_excess,neq,me);
    final double [] ri=rr[0],rdp=rr[1],rds=rr[2];
    wexcess0r=TMatr.getAMatrA(neq,ri,matr_excess,neq);
    final double sqrt_wexcess0r=Math.sqrt(wexcess0r);
    //dpexcess0r=TMatr.mulVVn1(neq,ri,mom1)/sqrt_wexcess0r;
    dpexcess0r=get_f_o(neq,rdp)*sqrt_wexcess0r;
    sigma2_dpexcess0r=TMatr.getAMatrA(neq,rdp,matr_excess,neq)-dpexcess0r*dpexcess0r;
    //dsexcess0r=TMatr.mulVVn1(ri,moms)/sqrt_wexcess0r;
    dsexcess0r=get_f_o(neq,rds)*sqrt_wexcess0r;
    sigma2_dsexcess0r=TMatr.getAMatrA(neq,rds,matr_excess,neq)-dsexcess0r*dsexcess0r;
    }
    {
    final double [] matr_deficit=getQQt_QQvm1_QQt(neq,QQv,diag_QQtm1);
    final double [][] rr=getRadauLikeBasisVector(neq,matr_deficit,neq,me);
    final double [] ri=rr[0],rdp=rr[1],rds=rr[2];
    wdeficit0r=TMatr.getAMatrA(neq,ri,matr_deficit,neq);
    final double sqrt_wdeficit0r=Math.sqrt(wdeficit0r);
    //dpdeficit0r=TMatr.mulVVn1(ri,mom1)/sqrt_wdeficit0r;
    dpdeficit0r=get_f_o(neq,rdp)*sqrt_wdeficit0r;
    sigma2_dpdeficit0r=TMatr.getAMatrA(neq,rdp,matr_deficit,neq)-dpdeficit0r*dpdeficit0r;
    //dsdeficit0r=TMatr.mulVVn1(ri,moms)/sqrt_wdeficit0r;
    dsdeficit0r=get_f_o(neq,rds)*sqrt_wdeficit0r;
    sigma2_dsdeficit0r=TMatr.getAMatrA(neq,rds,matr_deficit,neq)-dsdeficit0r*dsdeficit0r;
    }

    {
    final double [][] rr=getRadauLikeBasisVector(neq,QQv,neq,me);
    // rr[0] elements are |o>/<o|o>
    final double [] ri=rr[0],rdp=rr[1],rds=rr[2],rdt=rr[3],rdap=rr[4];
    // wii0r=1/<o|o>
    wii0r=TMatr.mulVVn1(ri,mom)/mom[0]; /* TMatr.getAMatrA(neq,ri,QQv,neq); */
    final double sqrt_wii0r=Math.sqrt(wii0r);
    //dpi0r=TMatr.mulVVn1(ri,mom1)/sqrt_wii0r;
    dpi0r=get_f_o(neq,rdp)*sqrt_wii0r;
    sigma2_dpi0r=TMatr.getAMatrA(neq,rdp,QQv,neq)-dpi0r*dpi0r;
    //dsi0r=TMatr.mulVVn1(ri,moms)/sqrt_wii0r;
    dsi0r=get_f_o(neq,rds)*sqrt_wii0r;
    sigma2_dsi0r=TMatr.getAMatrA(neq,rds,QQv,neq)-dsi0r*dsi0r;
    dtdv0r=get_f_o(neq,rdt)*sqrt_wii0r;
    sigma2_dtdv0r=TMatr.getAMatrA(neq,rdt,QQv,neq)-dtdv0r*dtdv0r;
    }

    {
    final double [] r=getImpactLikeDiff(neq,mom,moms,moma,mom1);
    wii0S=r[0];
    dpi0S=r[1];
    sigma2_dpi0S=r[2];
    dapi0S=r[3];
    sigma2_dapi0S=r[4];
    dki0S=r[5];
    sigma2_dki0S=r[6];
    dS0S=r[7];
    sigma2_dS0S=r[8];
    }

    if(true)
    {
	final int nmoms=2*neq-1;
	final double [] mom_plus=new double [nmoms],mom_minus=new double [nmoms];
	access.setM(mom_plus,0,access.M_ISELL,odata,nmoms);
	access.setM(mom_minus,0,access.M_IBUY,odata,nmoms);
	vplus00=mom_plus[0];
	vminus00=mom_minus[0];
	final double [] o_plus=getRadauLikeBasisVector(neq,getQQv_Matr(neq,mom_plus),neq,null)[0],
	    o_minus=getRadauLikeBasisVector(neq,getQQv_Matr(neq,mom_minus),neq,null)[0];
	wiplus0r=TMatr.mulVVn1(o_plus,mom_plus)/vplus00;
	wiminus0r=TMatr.mulVVn1(o_minus,mom_minus)/vminus00;
    }

    if(true)
    {
	final int nmoms=2*neq-1;
	final double [] imom=new double [nmoms],dpplusmom=new double [nmoms],dpminusmom=new double [nmoms],
	    dvplusmom=new double [nmoms],dvminusmom=new double [nmoms];
	access.setM(imom,0,access.M_I,odata,nmoms);
	access.setM(dpplusmom,1,access.M_DPPLUS,odata,nmoms);
	access.setM(dpminusmom,1,access.M_DPMINUS,odata,nmoms);
	for(int k=nmoms;--k>=0;) dpminusmom[k]=-dpminusmom[k];
	access.setM(dvplusmom,0,access.M_ISELL/*M_IPLUS*/,odata,nmoms);
	access.setM(dvminusmom,0,access.M_IBUY/*M_IMINUS*/,odata,nmoms);
	dpplus00=dpplusmom[0];
	dpminus00=dpminusmom[0];
	dvplus00=dvplusmom[0];
	dvminus00=dvminusmom[0];
	w00=imom[0];
	final double [] o_i=getO(neq,imom,null)[0],
	    o_dpplus=getO(neq,dpplusmom,null)[0],o_dpminus=getO(neq,dpminusmom,null)[0],
	    rvplus[]=getO(neq,dvplusmom,new double [][]{dpplusmom}),o_dvplus=rvplus[0],dpvplus=rvplus[1],
	    rvminus[]=getO(neq,dvminusmom,new double [][]{dpminusmom}),o_dvminus=rvminus[0],dpvminus=rvminus[1];
	dpplus0=1/get_f_o(neq,o_dpplus);
	dpminus0=1/get_f_o(neq,o_dpminus);
	dvplus0=1/get_f_o(neq,o_dvplus);
	dvminus0=1/get_f_o(neq,o_dvminus);
	w0=1/get_f_o(neq,o_i);
	final double [] dpdvdiff=new double [neq],QQa=getQQ_Matr(neq,imom,1);
	for(int k=0;k<neq;k++) dpdvdiff[k]=dpvplus[k]-dpvminus[k];
	dpdvdiff_sigma2=TMatr.getAMatrA(neq,dpdvdiff,QQa,neq)-Math.pow(get_f_o(neq,dpdvdiff),2)*w0;
	dSw=(dpplus0/dvplus0-dpminus0/dvminus0)*Math.sqrt(w0);
	dSd=(dpplus0/Math.sqrt(dvplus0)-dpminus0/Math.sqrt(dvminus0))*Math.sqrt(2);
	dSa=get_f_o(neq,dpdvdiff)*Math.sqrt(w0);
    }
}
abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double [] getQQtm1_DiagMatr(final int n);
/** Finds the roots of nodes polynomial. */
abstract double [] getXNodes(final double [] nodespolynomial);
abstract double get_f_o(final int n,final double [] f);
abstract double fx(final int n,final double [] m,final double x);
abstract double [] do_sdiv1(final double [] n0,final double x);

double [] getWeights(final double [] x,final double [] nodespolynomial,final double [] bmoms)
{
    final double [] w=new double [x.length];
    final int n=nodespolynomial.length-1;
    for(int k=x.length;--k>=0;)
    {
	final double xk=x[k];
	final double [] tmp=do_sdiv1(nodespolynomial,xk);
	w[k]=TMatr.mulVVn1(tmp,bmoms)/fx(n,tmp,xk);
    }
    return w;
}
double [] getWeights2Matr(final double [] x,final double [] nodes,final double [] matrQQ,final int nm)
{
    final double [] w=new double [x.length];
    final int n=nodes.length-1;
    for(int k=x.length;--k>=0;)
    {
	final double xk=x[k];
	final double [] tmp=do_sdiv1(nodes,xk);
	w[k]=TMatr.getAMatrA(n,tmp,matrQQ,nm)/Math.pow(fx(n,tmp,xk),2);
    }
    return w;
}

/** Calculates <Q|Q>_v.
 */
double [] getQQv_Matr(final int n,final double [] mom_v)
{
    return getQQ_Matr(n,mom_v,mom_v[0]);
}
/** Calculates <Q|Q>.
 */
double [] getQQ_Matr(final int n,final double [] mom_v,final double v00)
{
    return access.getQQ_Matrs(n,v00,new double [][]{mom_v})[0];
}


/** Calculates  <Q|Q>_v (<Q|Q>_t)^{-1}  <Q|Q>_v
 */
double [] getQQv_QQtm1_QQv(final int n,final double [] QQv,final double [] diag_QQtm1)
{
    final double [] matr=new double [n*n];
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<n;l++)
	{
	    double s=0;
	    for(int m=0;m<n;m++) s+=QQv[k*n+m]*diag_QQtm1[m]*QQv[m*n+l];
	    matr[k*n+l]=s;
	}
    }
    return matr;
}
/** Calculates  <Q|Q>_t (<Q|Q>_v)^{-1}  <Q|Q>_t
 */
double [] getQQt_QQvm1_QQt(final int n,final double [] QQv,final double [] diag_QQtm1)
{
    final double [] QQvm1=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQv);
    for(int k=0;k<n;k++)
    {
	double qk=diag_QQtm1[k];
	for(int l=0;l<n;l++) QQvm1[k*n+l]/=(qk*diag_QQtm1[l]);
    }
    return QQvm1;
}
/** Calculates n-th basis vector using measure minimiation.
 */
double [] getBasisVector(final int nn,final double [] mom_matr,final int nm)
{
    if(nn>nm) throw new IllegalArgumentException("nn="+nn+" nm="+nm);
    final int n=nn-1;
    final double [] matr=new double [nm*n],pk=new double [n+1];
    System.arraycopy(mom_matr,0,matr,0,nm*n);
    final double sg=pk[n]=getSignToMatch(n);
    for(int l=0;l<n;l++) pk[l]=-sg*mom_matr[nm*n+l];
    final int [] ipvt=new int [n];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,n,nm,ipvt);
    if(pinfo!=0) java.util.Arrays.fill(pk,Double.NaN);
    else com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,nm,ipvt,pk,1);
    // normalize
    final double s=Math.sqrt(TMatr.getAMatrA(n+1,pk,mom_matr,nm));
    for(int l=0;l<=n;l++) pk[l]/=s;

    return pk;
}
double getSignToMatch(final int k){return 1;}

/** Calculates n-th basis vector using measure minimiation.
 */
double [][] getRadauLikeBasisVector(final int n,final double [] mom_matr,final int nm,final double [][] extra_moms)
{
    if(n>nm) throw new IllegalArgumentException("n="+n+" nm="+nm);
    final double [][] res=new double [1+(extra_moms!=null?extra_moms.length:0)][n];
    final double [] matr=new double [nm*n],pk=res[0];
    for(int l=0;l<n;l++)
    {
	final double [] tmp=new double [l+1];tmp[l]=1;
	pk[l]=get_f_o(l+1,tmp);
    }
    System.arraycopy(mom_matr,0,matr,0,nm*n);
    final int [] ipvt=new int [n];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,n,nm,ipvt);
    if(pinfo!=0) for(int l=0;l<res.length;l++) java.util.Arrays.fill(res[l],Double.NaN);
    else for(int l=0;l<res.length;l++){
	    if(l>0) System.arraycopy(extra_moms[l-1],0,res[l],0,n);
	    com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,nm,ipvt,res[l],1);
	}
    // normalize to pk(x0)=1;
    final double f0=get_f_o(pk.length,pk);
    for(int l=0;l<n;l++) pk[l]/=f0;

    return res;
}

/** Calculates |o> and possible extra moments G^{-1} |extra>.
 */
double [][] getO(final int n,final double [] mom,final double [][] extra_moms)
{
    final double [] matr=getQQ_Matr(n,mom,1);
    final double [][] res=new double [1+(extra_moms!=null?extra_moms.length:0)][n];
    final double [] pk=res[0];
    for(int l=n;--l>=0;)
    {
	final double [] tmp=new double [l+1];tmp[l]=1;
	pk[l]=get_f_o(l+1,tmp);
    }
    final int [] ipvt=new int [n];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,n,n,ipvt);
    if(pinfo!=0) for(int l=0;l<res.length;l++) java.util.Arrays.fill(res[l],Double.NaN);
    else for(int l=0;l<res.length;l++){
	    if(l>0) System.arraycopy(extra_moms[l-1],0,res[l],0,n);
	    com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,res[l],1);
	}
    return res;
}

double [] getImpactLikeDiff(final int n,final double [] mom_v,final double [] mom_vs,final double [] mom_dap,final double [] mom_dp)
{
    final double [] QQv=getQQv_Matr(n,mom_v);
    final double [] matr=new double [n*n],o=new double [n],dp=new double [n],dap=new double [n];
    for(int l=0;l<n;l++)
    {
	final double [] tmp=new double [l+1];tmp[l]=1;
	o[l]=get_f_o(l+1,tmp); // delta function
    }
    System.arraycopy(QQv,0,matr,0,n*n);
    System.arraycopy(mom_dp,0,dp,0,n);
    System.arraycopy(mom_dap,0,dap,0,n);

    final int [] ipvt=new int [n];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,n,n,ipvt);
    if(pinfo!=0){
	final double [] r=new double [9];java.util.Arrays.fill(r,Double.NaN);
	return r;
    }
    com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,o,1);
    com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,dp,1);

    com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,dap,1);
    final double [] kvs=TMatr.getMatr_x_A(n,getQQ_Matr(n,mom_vs,1.0),dap); // assume the mom_vs is properly normalized
    com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,kvs,1);

    final double o_o=get_f_o(n,o),sqrt_o_o=Math.sqrt(o_o);
    final double dp_o=get_f_o(n,dp),dp_dp=TMatr.getAMatrA(n,dp,QQv,n);
    final double dap_o=get_f_o(n,dap),dap_dap=TMatr.getAMatrA(n,dap,QQv,n);
    final double kvs_o=get_f_o(n,kvs),kvs_kvs=TMatr.getAMatrA(n,kvs,QQv,n);
    final double [] dS=new double [n];
    for(int k=n;--k>=0;) dS[k]=dp[k]-kvs[k];
    final double dS_o=get_f_o(n,dS),dS_dS=TMatr.getAMatrA(n,dS,QQv,n);

    return new double []{
	1/o_o, //w0
	dp_o/sqrt_o_o,
	dp_dp-dp_o*dp_o/o_o,
	dap_o/sqrt_o_o,
	dap_dap-dap_o*dap_o/o_o,
	kvs_o/sqrt_o_o,
	kvs_kvs-kvs_o*kvs_o/o_o,
	dS_o/sqrt_o_o,
	dS_dS-dS_o*dS_o/o_o
    };
}

static abstract class NQVMTest<DTYPE> extends QTest {

abstract NQVM<DTYPE> getNQVM();
abstract LQVM<DTYPE> getLQVM();
abstract double [][] getMonomialMoments();
abstract double getX0();


NQVMTest(final String name,
	 final java.util.Random r,final double eps)
{
    super(name,r,eps);
}
void test_Common()
{
    final NQVM<DTYPE> q=getNQVM();

    UnitTests.checkDoubleRel(q.wii0S,"q.wii0S",q.wii0r,"q.wii0r",eps);
    UnitTests.checkDoubleRel(q.dpi0S,"q.dpi0S",q.dpi0r,"q.dpi0r",eps);
    UnitTests.checkDoubleRel(q.sigma2_dpi0S,"q.sigma2_dpi0S",q.sigma2_dpi0r,"q.sigma2_dpi0r",eps);
    UnitTests.checkDoubleRel(q.dpi0S-q.dki0S,"q.dpi0S-q.dki0S",q.dS0S,"q.dS0S",eps);


    final double [][] bmoms=getMomentsForTesting(q);
    final double scale=r.nextDouble()*1000;
    final double [] bmom_v=bmoms[0],bmom_vs=bmoms[2],
	bmom_dp=TMatr.getVectorScaled(bmom_vs,scale*bmom_v[0]),bmom_dap=TMatr.getVectorScaled(bmom_v,scale);
    final double [] rb=q.getImpactLikeDiff(neq,bmom_v,bmom_vs,bmom_dap,bmom_dp);
    final double dS0=rb[7],sigma2_dS0=rb[8];
    UnitTests.checkDoubleRel(dS0,"dS0",0,"0",eps);
    UnitTests.checkDoubleRel(sigma2_dS0,"sigma2_dS0",0,"0",eps);


    UnitTests.checkDoubleRel(q.w0/q.w00,"q.w0/q.w00",q.wii0r,"q.wii0r",eps);

    System.err.println(name+":NQVMTest:test_Commons(): test OK neq="+neq+" eps="+eps);
}

static double [][] getMM_Matr(final int n,final double [] mom)
{
    return getMM_Matr(n,mom,mom[0]);
}
static double [][] getMM_Matr(final int n,final double [] mom,final double v0)
{
    final double [][] matr=new double [n][n];
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<n;l++)
	{
	    matr[k][l]=mom[k+l]/v0;
	}
    }
    return matr;
}
static double [] pol_getMM_Matr_scal(final int n,final double [] mom,final double v0)
{
    final double [] matr=new double [n*n];
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<n;l++)
	{
	    matr[k*n+l]=mom[k+l]/v0;
	}
    }
    return matr;
}
static double [][] getSMart(final int n,final double [] mom_v,final double [] mom_t)
{
    final double [][] matr_t=getMM_Matr(n,mom_t),matr_v=getMM_Matr(n,mom_v);

    // matr=G_v G^{-1}_t G_v
    final double [][] matr_s=DeprecatedHelperTMatr.mulM_x_N(matr_v,DeprecatedHelperTMatr.mulM_x_N(com.polytechnik.utils.Linsystems.getInvertedMatrix(matr_t),matr_v));

    // print diag in unit test
    for(int p=0;p<2*n-1;p++)
    {
	System.err.println("----p="+p);
	for(int k=0;k<n;k++)
	{
	    for(int l=0;l<n;l++)
	    {
		if(l+k==p) System.err.println(" matr_v["+k+","+l+"]="+matr_v[k][l]);
	    }
	}
	System.err.println("----p="+p);
	for(int k=0;k<n;k++)
	{
	    for(int l=0;l<n;l++)
	    {
		if(l+k==p) System.err.println(" matr_s["+k+","+l+"]="+matr_s[k][l]);
	    }
	}
    }
    return matr_s;
}
double [] monomials_getBasisexpansion(final int n,final double [][] matr_s)
{
    final int [] ipvt=new int [n-1];
    final double [] pk=new double [n];
    pk[n-1]=1;
    for(int l=0;l<(n-1);l++) pk[l]=-matr_s[n-1][l];
    final double [][] matr=new double [n-1][n-1];
    for(int l=0;l<(n-1);l++) for(int m=0;m<(n-1);m++) matr[l][m]=matr_s[l][m];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa(matr,n-1,ipvt);
    if(pinfo!=0) throw new RuntimeException();
    else com.polytechnik.utils.Linsystems.dgesl(matr,n-1,ipvt,pk,1);
    // normalize
    final double s=Math.sqrt(DeprecatedHelperTMatr.getA_M_A(n,pk,matr_s));
    for(int l=0;l<n;l++) pk[l]/=s;

    return pk;
}
double [] monomials_getRadauLikeBasisexpansion(final int n,final double [][] matr_s)
{
    final int [] ipvt=new int [n];
    final double [] pk=new double [n];
    final double [] matr=new double [n*n];
    for(int l=0;l<n;l++)
    {
	final double [] tmp=new double [l+1];tmp[l]=1;
	pk[l]=Polynomials.getD0(tmp,getX0());
    }
    for(int l=0;l<n;l++) for(int m=0;m<n;m++) matr[l*n+m]=matr_s[l][m];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,n,n,ipvt);
    if(pinfo!=0) throw new RuntimeException();
    else com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,pk,1);

    // normalize
    final double f0=Polynomials.getD0(pk,getX0());
    for(int l=0;l<n;l++) pk[l]/=f0;
    return pk;
}

static double [] getOK(final int n,final double [] matr_s,final double x0)
{
    final double [] pk=new double [n];
    final double [] matr=new double [n*n];
    for(int l=0;l<n;l++)
    {
	final double [] tmp=new double [l+1];tmp[l]=1;
	pk[l]=Polynomials.getD0(tmp,x0);
    }
    System.arraycopy(matr_s,0,matr,0,n*n);
    final int [] ipvt=new int [n];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,n,n,ipvt);
    if(pinfo!=0) throw new RuntimeException();
    else com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,pk,1);

    return pk;
}
static double [] getOK_frommoments(final int n,final double [] moms,final double x0)
{
    return getOK(n,pol_getMM_Matr_scal(n,moms,1),x0);
}



double [][] getMomentsForTesting(final NQVM<DTYPE> q)
{
    final double [] mom=new double [2*neq-1];
    q.access.setM(mom,0,q.access.M_I,q.odata,mom.length);

    final double [] mom1=new double [mom.length],moms=new double [mom.length];
    q.access.setM(moms,0,q.access.M_ISELL,q.odata,moms.length);
    q.access.setM(mom1,0,q.access.M_IBUY,q.odata,moms.length);
    final double v00=moms[0]+mom1[0];for(int k=moms.length;--k>=0;) moms[k]=(moms[k]-mom1[k])/v00;

    final double [] moma=new double [mom.length];
    q.access.setM(moma,1,q.access.M_DPPLUS,q.odata,mom.length);
    q.access.setM(mom1,1,q.access.M_DPMINUS,q.odata,mom.length);
    for(int k=moma.length;--k>=0;) moma[k]=moma[k]-mom1[k];

    q.access.setM(mom1,1,q.access.M_DP,q.odata,mom1.length);
    return new double [][]{mom,mom1,moms,moma};
}

double [][] getMonomialMomentsConvertedFromBasis_dv_dvP_dvM_dpP_dpM(final int n,final NQVM<DTYPE> q)
{
    final double [] mom_v=new double [n],mom_vp=new double [n],mom_vm=new double [n],mom_pp=new double [n],mom_pm=new double [n];
    q.access.setM(mom_v,0,q.access.M_I,q.odata,n);
    q.access.setM(mom_vp,0,q.access.M_ISELL/*M_IPLUS*/,q.odata,n);
    q.access.setM(mom_vm,0,q.access.M_IBUY/*M_IMINUS*/,q.odata,n);
    q.access.setM(mom_pp,1,q.access.M_DPPLUS,q.odata,n);
    q.access.setM(mom_pm,1,q.access.M_DPMINUS,q.odata,n);
    for(int k=0;k<mom_pm.length;k++) mom_pm[k]=-mom_pm[k];
    final double [][] res=new double [][]{mom_v,mom_vp,mom_vm,mom_pp,mom_pm};
    for(int k=0;k<res.length;k++)
    {
	res[k]=getMonomialMomentsConvertedFromBasis(res[k]);
    }
    // result is M^{-1}{v,vp,vm,pp,pm}
    return res;
}

void test2_Monomials_BasisExpansion()
{
    final NQVM<DTYPE> q=getNQVM();
    final double [][] bmoms=getMomentsForTesting(q);
    final double [] bmom_v=bmoms[0],bmom_dp=bmoms[1],bmom_ds=bmoms[2];
    final double [] QQv=q.getQQv_Matr(neq,bmom_v),diag_QQtm1=q.getQQtm1_DiagMatr(neq);
    final double [] bmatr_excess=q.getQQv_QQtm1_QQv(neq,QQv,diag_QQtm1),bmatr_deficit=q.getQQt_QQvm1_QQt(neq,QQv,diag_QQtm1);

    final double []
	bexcess=q.getBasisVector(neq,bmatr_excess,neq),
	bdeficit=q.getBasisVector(neq,bmatr_deficit,neq),
	xexcess=q.getXNodes(bexcess),
	xdeficit=q.getXNodes(bdeficit),
	wexcess=q.getWeights2Matr(xexcess,bexcess,bmatr_excess,neq),
	wdeficit=q.getWeights2Matr(xdeficit,bdeficit,bmatr_deficit,neq),
	dpexcess=q.getWeights(xexcess,bexcess,bmom_dp),
	dpdeficit=q.getWeights(xdeficit,bdeficit,bmom_dp),
	dsexcess=q.getWeights(xexcess,bexcess,bmom_ds),
	dsdeficit=q.getWeights(xdeficit,bdeficit,bmom_ds);

    final double [][] amoms=getMonomialMoments();
    final double [] amom_v=amoms[0],amom_t=amoms[1],amom_dp=amoms[2],amom_ds=amoms[3];

    final double [][] sv=getSMart(neq,amom_v,amom_t),st=getSMart(neq,amom_t,amom_v);
    final double [] basis_v=monomials_getBasisexpansion(neq,sv),
	basis_t=monomials_getBasisexpansion(neq,st);
    UnitTests.checkDoubleArrayRel(convertToMonomials(bexcess),"conv(bexcess)",basis_v,"basis_v",eps);
    UnitTests.checkDoubleArrayRel(convertToMonomials(bdeficit),"conv(bdeficit)",basis_t,"basis_t",eps);

    if(neq>1)
    {
	double [][] nd=SQVM.SQVMTest.checkNodesPolynomialBasis(basis_v,amom_dp,neq-2,getOrderDirection(),eps,r);
	UnitTests.checkDoubleArrayRel(nd[0],"nd[0]",xexcess,"xexcess",eps);
	UnitTests.checkDoubleArrayRel1(nd[1],"nd[1]",dpexcess,"dpexcess",eps);
	nd=SQVM.SQVMTest.checkNodesPolynomialBasis(basis_v,amom_ds,neq-2,getOrderDirection(),eps,r);
	UnitTests.checkDoubleArrayRel(nd[0],"nd[0]",xexcess,"xexcess",eps);
	UnitTests.checkDoubleArrayRel1(nd[1],"nd[1]",dsexcess,"dsexcess",eps);
	nd=SQVM.SQVMTest.checkNodesPolynomialBasis(basis_t,amom_dp,neq-2,getOrderDirection(),eps,r);
	UnitTests.checkDoubleArrayRel(nd[0],"nd[0]",xdeficit,"xdeficit",eps);
	UnitTests.checkDoubleArrayRel1(nd[1],"nd[1]",dpdeficit,"dpdeficit",eps);
	nd=SQVM.SQVMTest.checkNodesPolynomialBasis(basis_t,amom_ds,neq-2,getOrderDirection(),eps,r);
	UnitTests.checkDoubleArrayRel(nd[0],"nd[0]",xdeficit,"xdeficit",eps);
	UnitTests.checkDoubleArrayRel1(nd[1],"nd[1]",dsdeficit,"dsdeficit",eps);
    }

    //System.err.println("xexcess="+java.util.Arrays.toString(xexcess));
    for(int k=0;k<xexcess.length;k++)
    {
	System.err.println("basis_v("+xexcess[k]+")="+Polynomials.getD0(basis_v,xexcess[k]));
	System.err.println("basis_t("+xdeficit[k]+")="+Polynomials.getD0(basis_t,xdeficit[k]));
    }

    System.err.println(name+":NQVMTest:test2_Monomials_BasisExpansion(): test OK neq="+neq+" eps="+eps+" class="+q.getClass().getName());

}
abstract boolean getOrderDirection();
void test_Monomials_BasisExpansion()
{
    final double [][] amoms=getMonomialMoments();
    final double [] mom_v=amoms[0],mom_t=amoms[1],mom_dp=amoms[2],mom_is=amoms[3];

    final NQVM<DTYPE> q=getNQVM();
    final double [][] sv=getSMart(neq,mom_v,mom_t),st=getSMart(neq,mom_t,mom_v),vv=getMM_Matr(neq,mom_v);

    test_sigma2(sv,mom_dp,q.wexcess0r,q.dpexcess0r,q.sigma2_dpexcess0r);
    test_sigma2(sv,mom_is,q.wexcess0r,q.dsexcess0r,q.sigma2_dsexcess0r);
    test_sigma2(st,mom_dp,q.wdeficit0r,q.dpdeficit0r,q.sigma2_dpdeficit0r);
    test_sigma2(st,mom_is,q.wdeficit0r,q.dsdeficit0r,q.sigma2_dsdeficit0r);
    test_sigma2(vv,mom_dp,q.wii0r,q.dpi0r,q.sigma2_dpi0r);
    test_sigma2(vv,mom_is,q.wii0r,q.dsi0r,q.sigma2_dsi0r);
    test_sigma2(vv,mom_t,q.wii0r,q.dtdv0r,q.sigma2_dtdv0r);

    System.err.println(name+":NQVMTest:test_Monomials_BasisExpansion(): test OK neq="+neq+" eps="+eps+" class="+q.getClass().getName());
}
void test_Monomials_dpdv()
{
    final NQVM<DTYPE> q=getNQVM();
    final double [][] amoms=getMonomialMomentsConvertedFromBasis_dv_dvP_dvM_dpP_dpM(2*neq-1,q);
    final double [] mom_v=amoms[0],mom_vp=amoms[1],mom_vm=amoms[2],mom_pp=amoms[3],mom_pm=amoms[4];
    final double [][] QQv=getMM_Matr(neq,mom_v,1),QQvp=getMM_Matr(neq,mom_vp,1),QQvm=getMM_Matr(neq,mom_vm,1),
	QQpp=getMM_Matr(neq,mom_pp,1),QQpm=getMM_Matr(neq,mom_pm,1);
    final double [][] sQQv=com.polytechnik.utils.Linsystems.getInvertedMatrix(QQv),
	sQQvp=com.polytechnik.utils.Linsystems.getInvertedMatrix(QQvp),
	sQQvm=com.polytechnik.utils.Linsystems.getInvertedMatrix(QQvm),
	sQQpp=com.polytechnik.utils.Linsystems.getInvertedMatrix(QQpp),
	sQQpm=com.polytechnik.utils.Linsystems.getInvertedMatrix(QQpm);
    final double [] o0=new double [neq];
    for(int k=o0.length;--k>=0;){
	final double [] tmp=new double [k+1];tmp[k]=1;
	o0[k]=Polynomials.getD0(tmp,getX0());
    }
    final double [] o_v=DeprecatedHelperTMatr.mulMV(sQQv,o0),o_vp=DeprecatedHelperTMatr.mulMV(sQQvp,o0),o_vm=DeprecatedHelperTMatr.mulMV(sQQvm,o0),
	o_pp=DeprecatedHelperTMatr.mulMV(sQQpp,o0),o_pm=DeprecatedHelperTMatr.mulMV(sQQpm,o0);
    final double pp=TMatr.mulVV(o_pp,DeprecatedHelperTMatr.mulMV(QQpp,o_pp)),
	pm=TMatr.mulVV(o_pm,DeprecatedHelperTMatr.mulMV(QQpm,o_pm)),
	vp=TMatr.mulVV(o_vp,DeprecatedHelperTMatr.mulMV(QQvp,o_vp)),
	vm=TMatr.mulVV(o_vm,DeprecatedHelperTMatr.mulMV(QQvm,o_vm)),
	v0=TMatr.mulVV(o_v,DeprecatedHelperTMatr.mulMV(QQv,o_v));
    UnitTests.checkDoubleRel(1/pp,"1/pp",q.dpplus0,"q.dpplus0",eps);
    UnitTests.checkDoubleRel(1/pm,"1/pm",q.dpminus0,"q.dpminus0",eps);
    UnitTests.checkDoubleRel(1/vp,"1/vp",q.dvplus0,"q.dvplus0",eps);
    UnitTests.checkDoubleRel(1/vm,"1/vm",q.dvminus0,"q.dvminus0",eps);
    UnitTests.checkDoubleRel(1/v0,"1/v0",q.w0,"q.w0",eps);
    final double [] dpp=DeprecatedHelperTMatr.mulMV(sQQvp,java.util.Arrays.copyOf(mom_pp,neq)),
	dpm=DeprecatedHelperTMatr.mulMV(sQQvm,java.util.Arrays.copyOf(mom_pm,neq)),
	dd=new double [neq];
    for(int k=0;k<neq;k++) dd[k]=dpp[k]-dpm[k];
    if(neq>1)
    {
	test_sigma2(QQv,DeprecatedHelperTMatr.mulMV(QQv,dd),q.w0,q.dSa,q.dpdvdiff_sigma2);
	test_sigma2(QQv,mom_v,q.w0,Math.sqrt(q.w0),q.w00-q.w0);
	test_sigma2(QQvp,mom_vp,q.dvplus0,Math.sqrt(q.dvplus0),q.dvplus00-q.dvplus0);
	test_sigma2(QQvm,mom_vm,q.dvminus0,Math.sqrt(q.dvminus0),q.dvminus00-q.dvminus0);
	test_sigma2(QQpp,mom_pp,q.dpplus0,Math.sqrt(q.dpplus0),q.dpplus00-q.dpplus0);
	test_sigma2(QQpm,mom_pm,q.dpminus0,Math.sqrt(q.dpminus0),q.dpminus00-q.dpminus0);
    }

    System.err.println(name+":NQVMTest:test_Monomials_dpdv(): test OK neq="+neq+" eps="+eps+" class="+q.getClass().getName());
}

void test_sigma2(final double [][] s,final double [] mom_dp,
		 final double test_w0,final double test_dp0r,final double test_sigma2)
{
    final double [] q0=new double [neq],dp=new double [neq];
    for(int k=q0.length;--k>=0;)
    {
	final double [] tmp=new double [k+1];tmp[k]=1;
	q0[k]=Polynomials.getD0(tmp,getX0());
	dp[k]=mom_dp[k];
    }
    final double [][] Sm=com.polytechnik.utils.Linsystems.getInvertedMatrix(s);
    final double [] adp=DeprecatedHelperTMatr.mulMV(Sm,dp),o=DeprecatedHelperTMatr.mulMV(Sm,q0);
    if(neq!=adp.length) throw new RuntimeException();
    final double o_o=DeprecatedHelperTMatr.getA_M_A(neq,q0,Sm),to_o=DeprecatedHelperTMatr.getA_M_A(neq,o,s),
	dp_o=Polynomials.getD0(adp,getX0()),tdp_o=(TMatr.mulVVn1(o,mom_dp)/o_o)*o_o;
    UnitTests.checkDoubleRel(test_w0,"test_w0",1/o_o,"1/o_o",eps);
    UnitTests.checkDoubleRel(test_dp0r,"test_dp0r",dp_o/Math.sqrt(o_o),"dp_o/Math.sqrt(o_o)",eps);
    UnitTests.checkDoubleRel(dp_o,"dp_o",tdp_o,"tdp_o",eps);
    if(neq!=o.length) throw new RuntimeException();
    UnitTests.checkDoubleRel(Polynomials.getD0(o,getX0()),"o(x0)",o_o,"<o|o>",eps);
    UnitTests.checkDoubleRel(to_o,"to_o",o_o,"<o|o>",eps);

    final double [] tadp=new double [neq];
    for(int k=tadp.length;--k>=0;)
    {
	tadp[k]=adp[k]-(dp_o/o_o)*o[k];
    }
    final double sigma2=DeprecatedHelperTMatr.getA_M_A(neq,tadp,s);
    UnitTests.checkDoubleRel(test_sigma2,"test_sigma2",sigma2,"sigma2",eps);
    UnitTests.checkDoubleRel(DeprecatedHelperTMatr.getA_M_A(neq,dp,Sm),"<dp|dp>",sigma2+dp_o*dp_o/o_o,"sigma2+dp_o*dp_o/o_o",eps);

    final double [] rad=monomials_getRadauLikeBasisexpansion(neq,s);
    UnitTests.checkDoubleArrayRel(rad,"rad",TMatr.getVectorScaled(o,1.0/o_o),"|o>/<o|o>",eps);
    final double w0r=DeprecatedHelperTMatr.getA_M_A(neq,rad,s);
    UnitTests.checkDoubleRel(w0r,"w0r",test_w0,"test_w0",eps);
    final double dp0r=TMatr.mulVVn1(rad,mom_dp)/Math.sqrt(w0r);
    UnitTests.checkDoubleRel(dp0r,"dp0r",test_dp0r,"test_dp0r",eps);
    final double sigma2_dp0r=DeprecatedHelperTMatr.getA_M_A(neq,DeprecatedHelperTMatr.mulMV(Sm,dp),s)-dp0r*dp0r;
    UnitTests.checkDoubleRel(sigma2_dp0r,"sigma2_dp0r",test_sigma2,"test_sigma2",eps);

    System.err.println(name+":NQVMTest:test_sigma2: test OK neq="+neq+" eps="+eps);
}

void test_Monomials_PM()
{
    final double [][] amoms=getMonomialMoments();
    final double [] mom_vbuy=amoms[4],mom_vsell=amoms[5];
    final NQVM<DTYPE> q=getNQVM();
    final double [][] vvsell=getMM_Matr(neq,mom_vsell),vvbuy=getMM_Matr(neq,mom_vbuy);
    test_sigma2(vvsell,TMatr.getVectorScaled(mom_vsell,1/mom_vsell[0]),
		q.wiplus0r,Math.sqrt(q.wiplus0r),1-q.wiplus0r);
    test_sigma2(vvbuy,TMatr.getVectorScaled(mom_vbuy,1/mom_vbuy[0]),
		q.wiminus0r,Math.sqrt(q.wiminus0r),1-q.wiminus0r);
    System.err.println(name+":NQVMTest:test_Monomials_PM: test OK neq="+neq+" eps="+eps);
}

void test_LQVM(){
    final NQVM<DTYPE> q=getNQVM();
    final LQVM<DTYPE> s=getLQVM();
    UnitTests.checkDoubleRel(q.dp00,"q.dp00",s.dp00,"s.dp00",eps);
    final double [][] amoms=getMonomialMoments();
    final double [] mom_v=amoms[0],mom_t=amoms[1],mom_dp=amoms[2],mom_is=amoms[3];

    final double [][] sv=getMM_Matr(neq,mom_v);
    final double [] rd=monomials_getRadauLikeBasisexpansion(neq,sv);
    UnitTests.checkDoubleRel(TMatr.mulVVn1(rd,mom_v)/mom_v[0],"radau(0)",s.wiinodes[0],"s.wiinodes[0]",eps);
    UnitTests.checkDoubleRel(DeprecatedHelperTMatr.getA_M_A(neq,rd,sv),"radau(0)",s.wiinodes[0],"s.wiinodes[0]",eps);
    UnitTests.checkDoubleRel(TMatr.mulVVn1(rd,mom_dp),"radau(0)",s.dpinodes[0],"s.dpinodes[0]",eps);
    UnitTests.checkDoubleRel(TMatr.mulVVn1(rd,mom_t)/mom_t[0],"radau(0)",s.wtinodes[0],"s.wtinodes[0]",eps);
    UnitTests.checkDoubleRel(DeprecatedHelperTMatr.getA_M_A(neq,rd,sv),"radau(0)",s.wiinodes[0],"s.wiinodes[0]",eps);

    UnitTests.checkDoubleRel(q.dpi0r,"q.dpi0r",s.dpinodes[0]/Math.sqrt(s.wiinodes[0]),"s.dpinodes[0]/sqrt(s.wiinodes[0])",eps);
    UnitTests.checkDoubleRel(q.dsi0r,"q.dsi0r",s.dsinodes[0]/Math.sqrt(s.wiinodes[0]),"s.dsinodes[0]/sqrt(s.wiinodes[0])",eps);
    UnitTests.checkDoubleRel(q.wii0r,"q.wii0r",s.wiinodes[0],"s.wiinodes[0]",eps);
    UnitTests.checkDoubleRel(q.sigma2_dpi0r,"q.sigma2_dpi0r",s.dp2i-s.dpinodes[0]*s.dpinodes[0]/s.wiinodes[0],"<dp|dp>-<dp|o><dp|o>/<o|o>",eps);
    UnitTests.checkDoubleRel(q.sigma2_dsi0r,"q.sigma2_dsi0r",s.ds2i-s.dsinodes[0]*s.dsinodes[0]/s.wiinodes[0],"<ds|ds>-<ds|o><ds|o>/<o|o>",eps);
    UnitTests.checkDoubleRel(q.sigma2_dpi0r,"q.sigma2_dpi0r",s.sigma2_dpi0,"s.sigma2_dpi0",eps);
    UnitTests.checkDoubleRel(q.sigma2_dsi0r,"q.sigma2_dsi0r",s.sigma2_dsi0,"s.sigma2_dsi0",eps);


    UnitTests.checkDoubleRel(q.dtdv0r,"q.dtdv0r",s.wtinodes[0]/Math.sqrt(s.wiinodes[0]),"s.wtinodes[0]/Math.sqrt(s.wiinodes[0])",eps);
    double s_sigma2_dtdv0r=0,s_sigma2_dapi0S=0;
    for(int k=1;k<s.wiinodes.length;k++)
    {
	s_sigma2_dtdv0r+=Math.pow(s.wtinodes[k],2)/s.wiinodes[k];
	s_sigma2_dapi0S+=Math.pow(s.dapinodes[k],2)/s.wiinodes[k];
    }
    UnitTests.checkDoubleRel(q.sigma2_dtdv0r,"q.sigma2_dtdv0r",s_sigma2_dtdv0r,"s_sigma2_dtdv0r",eps);


    UnitTests.checkDoubleRel(q.dapi0S,"q.dapi0S",s.dapinodes[0]/Math.sqrt(s.wiinodes[0]),"s.dapinodes[0]/sqrt(s.wiinodes[0])",eps);
    UnitTests.checkDoubleRel(q.sigma2_dapi0S,"q.sigma2_dapi0S",s_sigma2_dapi0S,"s_sigma2_dapi0S",eps);


    System.err.println(name+":NQVMTest:test_LQVM(): test OK neq="+neq+" eps="+eps+" class="+q.getClass().getName());
}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":NQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_LQVM();
    test2_Monomials_BasisExpansion();
    test_Monomials_BasisExpansion();
    test_Monomials_PM();
    test_Monomials_dpdv();

    System.err.println(name+":NQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":NQVMTest: starting tests; ver=$Id: NQVM.java,v 1.89 2022/07/28 22:30:50 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<118;i++)
	{
	    test_Calculations(1250+r.nextInt(7550),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":NQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: NQVM.java,v 1.89 2022/07/28 22:30:50 mal Exp $");
}
} // END OF class NQVMTest

}// END OF class NQVM
