package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.realtimebook.*;

public class NQVMLaguerre extends NQVM<QVMDataL> {
static final Laguerre B=new Laguerre();

NQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static NQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final NQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new NQVMLaguerre(neq,o);
}

DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
/** To match sign to monomials basis. */
double getSignToMatch(final int k){return (k%2==0)?1:-1;}
double [] getQQtm1_DiagMatr(final int n)
{
    final double [] qqm1=new double [n];qqm1[0]=1;
    return B.getL1FromL0(qqm1);
}
double fx(final int n,final double [] m,final double x)
{
    if(m.length!=n) throw new RuntimeException();
    return B.getD0(m,x);
}
double get_f_o(final int n,final double [] f){return fx(n,f,0);}
double [] do_sdiv1(final double [] n0,final double x)
{
    return B.sdiv1(n0,x).q;
}
double [] getXNodes(final double [] nodespolynomial)
{
    final int n=nodespolynomial.length-1;
    //final double [] npol=B.convertFunctionTo_Monomials_from_Basis(n,nodespolynomial);
    //final int nroots=new com.polytechnik.utils.PolynomialRootJenkinsTraubORIG(n).findRoots(n,npol,x,npol);
    final PolynomialRootsFinder.PRoots u=
	new com.polytechnik.outils.LaguerrePolynomialRootsFinder().findRoots(nodespolynomial);
    final double [] x=u.re_root;
    if(u.nroots!=n) java.util.Arrays.fill(x,Double.NaN);
    com.polytechnik.utils.SortArrays.sortArrayWithLoad(x.length,x,null);
    return x;
}


static class NQVMLaguerreTest extends NQVMTest<QVMDataL> {
NQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-5);
}
double getX0(){return 0;}
long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
NQVMLaguerre getNQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
boolean getOrderDirection(){return false;}
QVMData getPData(final int nmoms)
{
    final QVMData qdata=QVMData.calculateQVMData(1,nmoms,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
    return qdata;
}
double [][] getMonomialMoments()
{
    final int nmoms=2*neq-1;
    return MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,getPData(nmoms));
}

RadauLQVMLaguerre getLQVM()
{
    return RadauLQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);
}

} // END of class NQVMLaguerreTest

public static void main(String [] args)
{
    new NQVMLaguerreTest("NQVMLaguerre").runTests(5);
} // END main



} // END OF class NQVMLaguerre
