package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.realtimebook.*;

public class NQVMLegendreShifted extends NQVM<QVMDataP> {
static final LegendreShifted B=new LegendreShifted();

NQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static NQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final NQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new NQVMLegendreShifted(neq,o);
}

DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
double [] getQQtm1_DiagMatr(final int n)
{
    final double [] qqm1=new double [n];qqm1[0]=1;
    return DeprecatedHelperEVXData.getXPnprimeplusPnMoments(qqm1);
}
double fx(final int n,final double [] m,final double x)
{
    if(m.length!=n) throw new RuntimeException();
    return B.getD0(m,x);
}
double get_f_o(final int n,final double [] f){return fx(n,f,1);}
double [] do_sdiv1(final double [] n0,final double x){return B.sdiv1(n0,x).q;}

double [] getXNodes(final double [] nodespolynomial)
{
    final int n=nodespolynomial.length-1;
    //final double [] npol=B.convertFunctionTo_Monomials_from_Basis(n,nodespolynomial);
    //final int nroots=new com.polytechnik.utils.PolynomialRootJenkinsTraubORIG(n).findRoots(n,npol,x,npol);
    final PolynomialRootsFinder.PRoots u=
	new com.polytechnik.outils.LegendreShiftedPolynomialRootsFinder().findRoots(nodespolynomial);
    final double [] x=u.re_root;
    if(u.nroots!=n) java.util.Arrays.fill(x,Double.NaN);
    com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(x.length,x,null);
    return x;
}


static class NQVMLegendreShiftedTest extends NQVMTest<QVMDataP> {
NQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(68731L),1e-5);
}
double getX0(){return 1;}
long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/2)/(9*5*7*8*11*13)*(9*5*7*8*11*13);
}
NQVMLegendreShifted getNQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
boolean getOrderDirection(){return true;}
QVMData [] getPData(final int nmoms)
{
    final QVMData [] qdata=new QVMData [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}
double [][] getMonomialMoments()
{
    final int nmoms=2*neq-1;
    return MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,getPData(nmoms));
}

RadauLQVMLegendreShifted getLQVM()
{
    return RadauLQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);
}

} // END of class NQVMLegendreShiftedTest

public static void main(String [] args)
{
    new NQVMLegendreShiftedTest("NQVMLegendreShifted").runTests(5);
} // END main



} // END OF class NQVMLegendreShifted
