package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.realtimebook.*;

/** Multi-Stock version of MkQVM.
 *  @version $Id: NkQVM.java,v 1.23 2020/09/27 20:33:17 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class NkQVM<DTYPE> {

final EdComm E;
final EVXData I;

final DTYPE [] idata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
NkQVM(final int n,final DTYPE [] idata)
{
    this.idata=idata;
    this.access=getAccess();
    this.M=getABasis();

    final int nmoms=2*n-1;
    final double [] Imom=new double [nmoms],fmom=new double [nmoms],Itmp=new double [nmoms],Iptmp=new double [nmoms],ftmp=new double [nmoms],fptmp=new double [nmoms];
    for(DTYPE odata:idata)
    {
	final double po=getPOffset(odata);
	access.setM(Itmp,0,access.M_I,odata,nmoms);
	access.setM(Iptmp,1,access.M_I,odata,nmoms);
	access.setM(ftmp,0,access.M_RVS,odata,nmoms);
	access.setM(fptmp,1,access.M_RVS,odata,nmoms);
	for(int k=0;k<nmoms;k++)
	{
	    Imom[k]+=Iptmp[k]+po*Itmp[k];
	    fmom[k]+=fptmp[k]+po*ftmp[k];
	}
    }
    final double [] tmom=Iptmp,Fmom=Itmp; // re-use some arrays
    access.setMT(tmom,nmoms);
    access.setFMoments(Fmom,fmom,nmoms);
    this.E=new EdComm(n,Imom,tmom,Fmom,M,isEdgeAtRight()){
	    @Override
	    double [] getBasis(final int n_,final double [] tmom_,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M_,final boolean isEdgeAtRight_)
	    {
		final double [] b=try_getBasis_FromCache(n_,tmom_,M_,isEdgeAtRight_);
		if(b!=null) return b;
		return super.getBasis(n_,tmom_,M_,isEdgeAtRight_);
	    }
	};

    final double [][] QQ=access.getQQ_Matrs(n,1,new double [][] {Imom,tmom,Fmom});
    final double [] QQi=QQ[0],QQt=QQ[1],QQF=QQ[2];
    this.I=isDiagQQt()?DeprecatedHelperEVXData.getEVXDataFromQQfDiagQQa(n,QQi,QQt,access,this.E.x0):
	DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,this.E.x0);
}


boolean isDiagQQt(){return true;}
boolean isEdgeAtRight(){return true;}
double [] try_getBasis_FromCache(final int n,final double [] tmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final boolean isEdgeAtRight){return null;}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getPOffset(final DTYPE odata);


static abstract class NkQVMTest<DTYPE> extends QTest {

abstract NkQVM<DTYPE> getNkQVM1();
abstract MkQVM<DTYPE> getMkQVM();

NkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}

void test_MkQVM()
{
    final NkQVM<DTYPE> q=getNkQVM1();
    final MkQVM<DTYPE> s=getMkQVM();
    EVXData.matchEVXData(q.I,s.Ip,false,true,eps);
    UnitTests.checkDoubleRel(q.E.dir,"q.E.dir",s.edpVsit.dir,"s.edpVsit.dir",eps);

    System.err.println("NkQVMTest:test_MkQVM(): test OK n="+neq+" nstocks forced to 1, eps="+eps);
}
void test_Extra(){}

void test_Common()
{
    final NkQVM<DTYPE> q=getNkQVM1();
    q.E.test_Self(eps);

    System.err.println(name+":NkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}



Trade [][] tdata;

void setDataS(final int nstocks,final int tsize,final int neq)
{
    this.tdata=new Trade [nstocks][];
    long ct=Long.MIN_VALUE;
    for(int i=0;i<nstocks;i++)
    {
	setData(tsize+r.nextInt(tsize),neq); // sets tau as a sideeffect
	this.tdata[i]=tb;
	final long t=tb[tb.length-1].time;
	if(t>=ct){ct=t;}
	tb=null;
    }
    currenttime=ct+330; // maximal last time plus some small delay
    // tau has the value of latest generated tb
}
void test_Calculations(final int nstocks,final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":NkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setDataS(nstocks,tsize,neq);

    test_Common();
    test_MkQVM();
    test_Extra();

    System.err.println(name+":NkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" nstocks="+nstocks+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":NkQVMTest: starting tests; ver=$Id: NkQVM.java,v 1.23 2020/09/27 20:33:17 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int nstocks=1;nstocks<=3;nstocks++)
	{
	    for(int i=0;i<119;i++)
	    {
		test_Calculations(nstocks,1950+r.nextInt(1850),neq);
	    }
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":NkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: NkQVM.java,v 1.23 2020/09/27 20:33:17 mal Exp $");
}

} // END OF class NkQVMTest

} //END OF class NkQVM
