package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: NkQVMLaguerre.java,v 1.13 2022/07/10 10:41:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class NkQVMLaguerre extends NkQVM<QVMDataL> {
NkQVMLaguerre(final int neq,final QVMDataL [] idata){super(neq,idata);}


public static NkQVMLaguerre calculate(final int neq,
				      final long currenttime,
				      final long tau,
				      final Trade [][] itb,
				      final int [] itbsize,
				      final NkQVMLaguerre ref)
{
    final int npm=3,nmoms=2*neq-1;
    final QVMDataL [] o=new QVMDataL [itb.length];
    for(int k=0;k<itb.length;k++)
    {
	o[k]=QVMDataL.calculateQVMDataL(nmoms,
					npm,
					currenttime,
					tau,
					itb[k],
					itbsize[k],
					ref!=null?ref.idata[k]:null);
    }
    return new NkQVMLaguerre(neq,o);
}

public static NkQVMLaguerre calculate1(final int neq,
				       final long currenttime,
				       final long tau,
				       final Trade [] tb,
				       final int tsize,
				       final NkQVMLaguerre ref)
{
    return calculate(neq,currenttime,tau,new Trade [][]{tb},new int []{tsize},ref);
}

@Override
boolean isEdgeAtRight(){return false;}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getPOffset(final QVMDataL odata){return odata.p_offset;}


static class NkQVMLaguerreTest extends NkQVMTest<QVMDataL> {
NkQVMLaguerreTest(final long initrandom)
{
    super("NkQVMLaguerre",new java.util.Random(initrandom),1e-3);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
NkQVMLaguerre getNkQVM1(){return calculate(neq,currenttime,tau,new Trade [][]{tdata[0]},new int []{tdata[0].length},null);}
MkQVMLaguerre getMkQVM(){return MkQVMLaguerre.calculate(neq,currenttime,tau,tdata[0],tdata[0].length,null);}

} // END of class NkQVMLaguerreTest

public static void main(String [] args)
{
    new NkQVMLaguerreTest(817790L).runTests(4);
} // END main



} // END OF class NkQVMLaguerre
