package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class OQVMLaguerre extends OQVM<QVMDataL> {

static final Laguerre B=new Laguerre();

OQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static OQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final OQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new OQVMLaguerre(neq,o);
}

DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
double getXEq(final double [] kk)
{
    if(kk.length<3) return Double.NaN;
    final double [] dk=B.getDifferentiated(kk);
    final PolynomialRootsFinder.PRoots u=com.polytechnik.utils.PolynomialRootsConfederateMatrix.ABASISLAGUERRE.findRoots(dk);
    final double [] x=u.re_root,y=u.im_root;
    // find min x
    double x0=Double.POSITIVE_INFINITY;
    for(int k=0;k<u.nroots;k++)
    {
	if(x[k]<=x0) x0=x[k];
    }
    //System.err.println("getXEq: nroots="+nroots+" Re x="+java.util.Arrays.toString(x)+" Im x="+java.util.Arrays.toString(y));
    return x0;
}
double [] getDPX(final double xeq,final double [] dpv,final double [] ok)
{
    final double exp=Math.exp(-xeq),x_o=getX0(),s=1-exp;
    final int neq=dpv.length;
    final double [] intdp=B.getLaguerreExpFIntegrated(dpv);
    final double dp_eq=access.fx(intdp,xeq)*exp,dp_o=access.fx(intdp,x_o);
    final double dp=dp_o-dp_eq,sigma2=B.get2LaguerreAveraged(dpv,dpv);
    final double sigma2_0=sigma2-Math.pow(B.get2LaguerreAveraged(ok,dpv),2)/B.get2LaguerreAveraged(ok,ok);
    final double cxeq=access.fx(dpv,xeq);
    //System.err.println("getDPX: dp_eq="+dp_eq+" dp_o="+dp_o);
    return new double []{s,dp,sigma2,sigma2_0,cxeq};
}
double [] getKKEQ(final int n)
{
    return B.getChristoffelKernel(n-1);
}
double [] getXdVdT(final double [] kv,final double [] kt)
{
    if(kv.length!=kt.length) throw new RuntimeException("kv.length="+kv.length+" kt.length="+kt.length);
    if(kv.length<3) return new double []{Double.NaN,Double.NaN};
    final double [] tmp=new double [kv.length+kt.length-2];
    final double [] Dkv=B.getDifferentiated(kv),
	Dkt=B.getDifferentiated(kt);
    final double [] P=B.mult2Pol(Dkv,kt),t=B.mult2Pol(Dkt,kv);
    for(int k=P.length;--k>=0;) P[k]=t[k]-P[k];

    // note: the highest power of P=kt'*kv-kv'*kt product is equal exactly 0,
    // beacuse kt & kv are of equal power, so the actual degree of P is 1 less.
    //final int deg=P.length-2;
    final double [] P1less=java.util.Arrays.copyOf(P,P.length-1);
    final PolynomialRootsFinder.PRoots u=com.polytechnik.utils.PolynomialRootsConfederateMatrix.ABASISLAGUERRE.findRoots(P1less);
    com.polytechnik.utils.SortArrays.sortArrayWithLoad(u.nroots,u.re_root,u.im_root);
    //System.err.println("getXdVdT: nroots="+nroots+" rex="+java.util.Arrays.toString(t)+" imx="+java.util.Arrays.toString(tmp));
    //final double x0=getX0();
    double xeq=Double.NaN,feq=Double.NEGATIVE_INFINITY,xout=Double.NaN;;
    for(int k=0;k<u.nroots;k++)
    {
	final double x=u.re_root[k],f=access.fx(kt,x)/access.fx(kv,x);
	if(f>=feq){feq=f;xeq=x;}
	if(x<=getX0()) xout=x;
    }
    return new double []{xeq,xout};
}


static class OQVMLaguerreTest extends OQVMTest<QVMDataL> {
OQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6873L),1e-5);
}
long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
OQVMLaguerre getOQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
NQVMLaguerre getNQVM()
{
    return NQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);
}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
boolean isDescending(){return false;}
QVMData getPData(final int nmoms)
{
    final QVMData qdata=QVMData.calculateQVMData(1,nmoms,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
    return qdata;
}
double [][] getMonomialMoments(final QVMDataL odata)
{
    final int nmoms=2*neq-1;
    final QVMData qd=getPData(nmoms);
    return new double [][]{
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_dV,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_dV,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomT(nmoms),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomDP(nmoms,qd)
    };
}

double [] getdLdpLsigma2(final double [] dp,final double [] ok,final double [] mom_t,final double xeq)
{
    final double [] cc=new double [neq];
    WIntegratorMonomials.inverseOddCoefs(neq,cc,dp);
    final double [] s=new double [neq];
    double [] d=cc;
    for(int k=0,sign=1;k<cc.length;k++,sign=-sign)
    {
	TMatr.sumP1(s,d,sign);
	d=Polynomials.getPolynomialDifferentiated(d);
    }
    final double dp_eq=Polynomials.getD0(s,-xeq)*Math.exp(-xeq),dp_o=Polynomials.getD0(s,0);
    final double [][] mt=NQVM.NQVMTest.getMM_Matr(neq,mom_t,1);
    final double dpL=dp_o-dp_eq,dL=1-Math.exp(-xeq),sigma2=DeprecatedHelperTMatr.getA_M_A(neq,dp,mt);
    final double sigma2_0=sigma2-Math.pow(DeprecatedHelperTMatr.getA_M_B(neq,dp,mt,ok),2)/DeprecatedHelperTMatr.getA_M_A(neq,ok,mt);
    return new double []{dL,dpL,sigma2,sigma2_0};
}

} // END of class OQVMLaguerreTest

public static void main(String [] args)
{
    new OQVMLaguerreTest("OQVMLaguerre").runTests(5);
} // END main



} // END OF class OQVMLaguerre
