package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.realtimebook.*;

public class OQVMLegendreShifted extends OQVM<QVMDataP> {

private static final LegendreShifted B=new LegendreShifted();

OQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static OQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final OQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new OQVMLegendreShifted(neq,o);
}

DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
double getX0(){return 1;}
double getPOffset(){return odata.p_offset;}
double getXEq(final double [] kk)
{
    if(kk.length<3) return Double.NaN;
    final double [] dk=B.getDifferentiated(kk);
    final PolynomialRootsFinder.PRoots u=com.polytechnik.utils.PolynomialRootsConfederateMatrix.ABASISLEGENDRESHIFTED.findRoots(dk);
    final double [] x=u.re_root,y=u.re_root;
    // find max x
    double x0=Double.NEGATIVE_INFINITY;
    for(int k=0;k<u.nroots;k++)
    {
	if(x[k]>=x0) x0=x[k];
    }
    //System.err.println("getXEq: nroots="+nroots+" Re x="+java.util.Arrays.toString(x)+" Im x="+java.util.Arrays.toString(y));
    return x0;
}
double [] getDPX(final double xeq,final double [] dpv,final double [] ok)
{
    final double x_o=getX0(),s=x_o-xeq;
    final int neq=dpv.length;
    final double [] intdp=B.getIntegrated(dpv);
    final double dp_eq=access.fx(intdp,xeq),dp_o=access.fx(intdp,x_o);
    //System.err.println(" dp_eq="+dp_eq+" dp_o="+dp_o);
    final double dp=dp_o-dp_eq,sigma2=B.get2LegendreAveraged(dpv,dpv);
    final double sigma2_0=sigma2-Math.pow(B.get2LegendreAveraged(ok,dpv),2)/B.get2LegendreAveraged(ok,ok);
    final double cxeq=access.fx(dpv,xeq);
    return new double []{s,dp,sigma2,sigma2_0,cxeq};

}
double [] getKKEQ(final int n)
{
    return B.getChristoffelKernel(n-1);
}
double [] getXdVdT(final double [] kv,final double [] kt)
{
    if(kv.length!=kt.length) throw new RuntimeException("kv.length="+kv.length+" kt.length="+kt.length);
    if(kv.length<3) return new double []{Double.NaN,Double.NaN};
    final double [] tmp=new double [kv.length+kt.length-2];
    final double [] Dkv=B.getDifferentiated(kv),
	Dkt=B.getDifferentiated(kt);
    final double [] P=B.mult2Pol(Dkv,kt),t=B.mult2Pol(Dkt,kv);
    for(int k=P.length;--k>=0;) P[k]=t[k]-P[k];

    // note: the highest power of P=kt'*kv-kv'*kt product is equal exactly 0,
    // beacuse kt & kv are of equal power, so the actual degree of P is 1 less.
    //final int deg=P.length-2;
    final double [] P1less=java.util.Arrays.copyOf(P,P.length-1);
    final PolynomialRootsFinder.PRoots u=com.polytechnik.utils.PolynomialRootsConfederateMatrix.ABASISLEGENDRESHIFTED.findRoots(P1less);
    com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(u.nroots,u.re_root,u.im_root);
    //final double x0=getX0();
    double xeq=Double.NaN,feq=Double.NEGATIVE_INFINITY,xout=Double.NaN;
    for(int k=0;k<u.nroots;k++)
    {
	final double x=u.re_root[k],f=access.fx(kt,x)/access.fx(kv,x);
	if(f>=feq){feq=f;xeq=x;}
	if(x>=getX0()) xout=x;
    }
    return new double []{xeq,xout};
}


static class OQVMLegendreShiftedTest extends OQVMTest<QVMDataP> {
OQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-5);
}
long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/2)/(9*5*7*8*11*13)*(9*5*7*8*11*13);
}
OQVMLegendreShifted getOQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
NQVMLegendreShifted getNQVM()
{
    return NQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);
}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
boolean isDescending(){return true;}
QVMData [] getPData(final int nmoms)
{
    final QVMData [] qdata=new QVMData [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}
double [][] getMonomialMoments(final QVMDataP odata)
{
    final int nmoms=2*neq-1;
    final QVMData [] qd=getPData(nmoms);
    return new double [][]{
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomT(nmoms),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomDP(nmoms,qd)
    };
}

double [] getdLdpLsigma2(final double [] dp,final double [] ok,final double [] mom_t,final double xeq)
{
    final double [] int_cdp=Polynomials.getPolynomialIntegrated(dp);
    final double dp_eq=Polynomials.getD0(int_cdp,xeq),dp_o=Polynomials.getD0(int_cdp,1);
    final double [][] mt=NQVM.NQVMTest.getMM_Matr(neq,mom_t,1);
    final double dpL=dp_o-dp_eq,dL=1-xeq,sigma2=DeprecatedHelperTMatr.getA_M_A(neq,dp,mt);
    final double sigma2_0=sigma2-Math.pow(DeprecatedHelperTMatr.getA_M_B(neq,dp,mt,ok),2)/DeprecatedHelperTMatr.getA_M_A(neq,ok,mt);
    return new double []{dL,dpL,sigma2,sigma2_0};
}


} // END of class OQVMLegendreShiftedTest

public static void main(String [] args)
{
    new OQVMLegendreShiftedTest("OQVMLegendreShifted").runTests(5);
} // END main



} // END OF class OQVMLegendreShifted
