package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.realtimebook.*;

/** Test whether the John Pierpont Morgan was right.
 *  @version $Id: OkQVM.java,v 1.29 2022/07/27 16:14:10 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class OkQVM<DTYPE> {

final DTYPE odata;
final DataInterfaceToMoments<DTYPE> access;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;

final EdComm Ed_Pt_I,Ed_Pt_x0,Ed_Vs_I,Ed_Vs_x0,Ed_SVs_I,Ed_Pi_I,Ed_Pi_x0,Ed_I_x0,Ed_Pt_dpI;
final EiComm Ei_Pt_I,Ei_Pt_x0,Ei_Vs_I,Ei_Vs_x0,Ei_SVs_I,Ei_Pi_I,Ei_Pi_x0,Ei_I_x0;
final EpiComm E_cd_Pt,E_cd_Pi,E_cd_Vs;
final EbComm Eb_Pt,Eb_Pi,Eb_Vs;
final EbiComm Ebi_P;
final EcComm Ec_Pt_I,Ec_Pt_V,Ec_dp_I,Ec_Pi_I,Ec_Vs_I,Ec_Vs_V,Ec_vs_I;
final EpComm Ep_itdp,Ep_itpt,Ep_itVs;
final EfiComm Efi_itpt,Efi_itpi,Efi_itVs,Efi_itvs,Efi_itdp,Efi_itddp;
final EjComm Ej_itpt,Ej_itpi,Ej_dptPt,Ej_dpIPt;
final EaComm Ea_itpi,Ea_itdp,Ea_itvs,Ea_dptPt,Ea_dpiPt;

OkQVM(final int n,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();
    this.M=getABasis();

    final int nmoms=2*n-1;
    final double [] imom=new double [nmoms],tmom=new double [nmoms],vsmom=new double [nmoms],Svsmom=new double [nmoms],dpmom=new double [nmoms],pimom=new double [nmoms],x0mom=M.B.getX0Moments(nmoms,getX0());
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setMT(tmom,nmoms);
    access.setM(vsmom,0,access.M_RVS,odata,nmoms);
    access.setM(Svsmom,0,access.M_VS,odata,nmoms);
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);

    Ed_Pt_I=new EdComm(n,imom,tmom,getFmom(dpmom),M,isEdgeAtRight());
    Ed_Pt_dpI=new EdComm(n,dpmom,imom,getFmom(dpmom),M,isEdgeAtRight());
    Ed_Pt_x0=new EdComm(n,x0mom,tmom,getFmom(dpmom),M,isEdgeAtRight());
    Ed_Vs_I=new EdComm(n,imom,tmom,getFmom(vsmom),M,isEdgeAtRight());
    Ed_Vs_x0=new EdComm(n,x0mom,tmom,getFmom(vsmom),M,isEdgeAtRight());
    Ed_SVs_I=new EdComm(n,imom,tmom,getFmom(Svsmom),M,isEdgeAtRight());
    Ed_Pi_I=new EdComm(n,imom,tmom,pimom,M,isEdgeAtRight());
    Ed_Pi_x0=new EdComm(n,x0mom,tmom,pimom,M,isEdgeAtRight());
    Ed_I_x0=new EdComm(n,x0mom,tmom,imom,M,isEdgeAtRight());

    Ei_Pt_I=new EiComm(n,imom,tmom,getFmom(dpmom),M);
    Ei_Pt_x0=new EiComm(n,x0mom,tmom,getFmom(dpmom),M);
    Ei_Vs_I=new EiComm(n,imom,tmom,getFmom(vsmom),M);
    Ei_Vs_x0=new EiComm(n,x0mom,tmom,getFmom(vsmom),M);
    Ei_SVs_I=new EiComm(n,imom,tmom,getFmom(Svsmom),M);
    Ei_Pi_I=new EiComm(n,imom,tmom,pimom,M);
    Ei_Pi_x0=new EiComm(n,x0mom,tmom,pimom,M);
    Ei_I_x0=new EiComm(n,x0mom,tmom,imom,M);

    E_cd_Pt=new EpiComm(n,imom,tmom,getFmom(dpmom),x0mom,M);
    E_cd_Pi=new EpiComm(n,imom,tmom,pimom,x0mom,M);
    E_cd_Vs=new EpiComm(n,imom,tmom,getFmom(vsmom),x0mom,M);

    Eb_Pt=new EbComm(n,imom,tmom,getFmom(dpmom),M);
    Eb_Pi=new EbComm(n,imom,tmom,pimom,M);
    Eb_Vs=new EbComm(n,imom,tmom,getFmom(vsmom),M);

    Ebi_P=new EbiComm(n,imom,tmom,getFmom(dpmom),pimom,M);

    Ec_Pt_I=new EEcComm(n,imom,tmom,getFmom(dpmom),imom,M);
    Ec_Pt_V=new EEcComm(n,imom,tmom,getFmom(dpmom),getFmom(imom),M);
    Ec_dp_I=new EEcComm(n,imom,tmom,dpmom,imom,M);
    Ec_Pi_I=new EEcComm(n,imom,tmom,pimom,imom,M);
    Ec_Vs_I=new EEcComm(n,imom,tmom,getFmom(vsmom),imom,M);
    Ec_Vs_V=new EEcComm(n,imom,tmom,getFmom(vsmom),getFmom(imom),M);
    Ec_vs_I=new EEcComm(n,imom,tmom,vsmom,imom,M);

    Ep_itdp=new EpComm(n,imom,tmom,dpmom,M);
    Ep_itpt=new EpComm(n,imom,tmom,getFmom(dpmom),M);
    Ep_itVs=new EpComm(n,imom,tmom,getFmom(vsmom),M);

    Efi_itpt=new EEfiComm(n,imom,tmom,getFmom(dpmom),M);
    Efi_itpi=new EEfiComm(n,imom,tmom,pimom,M);
    Efi_itVs=new EEfiComm(n,imom,tmom,getFmom(vsmom),M);
    Efi_itdp=new EEfiComm(n,imom,tmom,dpmom,M);
    Efi_itddp=new EEfiComm(n,imom,tmom,getDFmom(dpmom),M);
    Efi_itvs=new EEfiComm(n,imom,tmom,vsmom,M);

    Ej_itpt=new EjComm(n,imom,tmom,getFmom(dpmom),M);
    Ej_itpi=new EjComm(n,imom,tmom,pimom,M);
    Ej_dptPt=new EjComm(n,dpmom,tmom,getFmom(dpmom),M);
    Ej_dpIPt=new EjComm(n,dpmom,imom,getFmom(dpmom),M);

    Ea_itpi=new EaComm(n,imom,tmom,pimom,M);
    Ea_itdp=new EaComm(n,imom,tmom,dpmom,M);
    Ea_itvs=new EaComm(n,imom,tmom,vsmom,M);
    Ea_dptPt=new EaComm(n,dpmom,tmom,getFmom(dpmom),M);
    Ea_dpiPt=new EaComm(n,dpmom,imom,getFmom(dpmom),M);

}

double [] getFmom(final double [] moms)
{
    final double [] fmoms=new double [moms.length];
    access.setFMoments(fmoms,moms,fmoms.length);
    //TMatr.scaleVectorInPlace(fmoms,-1);
    return fmoms;
}
double [] getDFmom(final double [] moms)
{
    final double [] fmoms=new double [moms.length];
    access.setDFMoments(fmoms,moms,fmoms.length);
    TMatr.scaleVectorInPlace(fmoms,-1);
    return fmoms;
}

class EEcComm extends EcComm {
EEcComm(final int n,final double [] imom,final double [] tmom,final double [] fmom,final double [] gmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M){
    super(n,imom,tmom,fmom,gmom,M);
}
@Override double [] getDPsi(final double [] psi){return getEDPsi(psi);}
}// END OF class EEcComm
class EEfiComm extends EfiComm {
EEfiComm(final int n,final double [] imom,final double [] tmom,final double [] fmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M){
    super(n,imom,tmom,fmom,M);
}
@Override double [] getDPsi(final double [] psi){return getEDPsi(psi);}
}// END OF class EEfiComm



boolean isEdgeAtRight(){return true;}
abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getX0();
abstract double [] getEDPsi(final double [] psi);

static abstract class OkQVMTest<DTYPE> extends QTest {

abstract OkQVM<DTYPE> getOkQVM();


OkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_Common()
{
    final OkQVM<DTYPE> q=getOkQVM();
    q.Ed_Pt_I.test_Self(eps);
    //q.Ed_Pt_dpI.test_Self(eps);
    //q.Ed_Pt_x0.test_Self(eps);
    q.Ed_Vs_I.test_Self(eps);
    q.Ed_SVs_I.test_Self(eps);
    q.Ed_Pi_I.test_Self(eps);
    q.Ei_Pt_I.test_Self(eps);
    q.Ei_Pt_x0.test_Self(eps);
    q.Ei_Vs_I.test_Self(eps);
    q.Ei_SVs_I.test_Self(eps);
    q.Ei_Pi_I.test_Self(eps);

    q.E_cd_Pt.test_Self(eps);
    q.E_cd_Pi.test_Self(eps);
    q.E_cd_Vs.test_Self(eps);

    q.Ec_Pt_I.test_Self(eps);
    q.Ec_Pt_V.test_Self(eps);
    q.Ec_dp_I.test_Self(eps);
    q.Ec_Pi_I.test_Self(eps);
    q.Ec_Vs_I.test_Self(eps);
    q.Ec_Vs_V.test_Self(eps);
    q.Ec_vs_I.test_Self(eps);

    q.Efi_itpt.test_Self(eps);
    q.Efi_itpi.test_Self(eps);
    q.Efi_itVs.test_Self(eps);
    q.Efi_itvs.test_Self(eps);
    q.Efi_itdp.test_Self(eps);
    q.Efi_itddp.test_Self(eps);
    UnitTests.checkDoubleRel(q.Efi_itddp.fL,"q.Efi_itddp.fL",q.Efi_itdp.dfL,"q.Efi_itdp.dfL",eps);
    UnitTests.checkDoubleRel(q.Efi_itddp.fH,"q.Efi_itddp.fH",q.Efi_itdp.dfH,"q.Efi_itdp.dfH",eps);

    q.Ej_itpt.test_Self(eps);
    q.Ej_itpi.test_Self(eps);
    q.Ej_dptPt.test_Self(eps);
    q.Ej_dpIPt.test_Self(eps);

    q.Ea_itpi.test_Self(eps);
    UnitTests.checkDoubleRel(q.Efi_itpi.f0/q.Efi_itpi.I_0,"q.Efi_itpi.f0/q.Efi_itpi.I_0",q.Ea_itpi.f0,"q.Ea_itpi.f0",eps);
    q.Ea_itdp.test_Self(eps);
    q.Ea_itvs.test_Self(eps);
    q.Ea_dptPt.test_Self(eps);
    q.Ea_dpiPt.test_Self(eps);

    System.err.println(name+":OkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}


void test_Extra(){}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":OkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Extra();

    System.err.println(name+":OkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":OkQVMTest: starting tests; ver=$Id: OkQVM.java,v 1.29 2022/07/27 16:14:10 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":OkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: OkQVM.java,v 1.29 2022/07/27 16:14:10 mal Exp $");
}
} // END OF class OkQVMTest




}// END OF class OkQVM
