package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;

/** A program to find a trade opportunity.
 *  @version $Id: Opportunity.java,v 1.30 2022/07/28 21:03:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class Opportunity  {

    final Distribution<Trade> savedtrades=new Distribution<Trade>();
    Distribution<Trade> [] istockstrades=Distribution.getDistributionArray(0);
    Trade [][] arrT=new Trade [0][];
    int [] dim_arrT=new int [0];


    static Trade [][] shrinkATradeArray(final int nelements,final Trade [][] array)
    {
	if(nelements>=array.length) return array;
	final Trade [][] ar=new Trade [nelements][];
	System.arraycopy(array,0,ar,0,nelements);
	return ar;
    }

    double p_buy=0.0,p_sell=0.0,p_last=0.0;
    long vol_p_buy=0,vol_p_sell=0;
    private long id=-1;
    MultiStockStatusData sd=null;
    public void setData(final MultiStockStatusData sd,final long id)
    {
	if(this.sd!=null || !(this.id<0))
	{
	    throw new IllegalStateException("An attempt to copy to a non-clean object.");
	}
	this.sd=sd;
	this.id=id;

	// copy trades
	sd.ot.tradebook.copyTrades(this.savedtrades);
	if(this.istockstrades.length<sd.iot.length)
	{
	    final Distribution<Trade> [] ii=Distribution.getDistributionArray(2*sd.iot.length+3);
	    System.arraycopy(this.istockstrades,0,ii,0,this.istockstrades.length);
	    for(int i=this.istockstrades.length;i<ii.length;i++)
	    {
		ii[i]=new Distribution<Trade>();
	    }
	    this.istockstrades=ii;
	    this.arrT=new Trade [ii.length][];
	    this.dim_arrT=new int [ii.length];
	}
	for(int i=0;i<sd.iot.length;i++)
	{
	    final Distribution<Trade> d=this.istockstrades[i];
	    sd.iot[i].tradebook.copyTrades(d);
	    this.arrT[i]=d.getBuffer();
	    this.dim_arrT[i]=d.getSize();
	    //System.err.println("For ticker="+sd.ot.ticker+" copied data for ticker="+sd.iot[i].ticker);
	}

	// read last price
	final int ntr=this.savedtrades.getSize();
	this.p_last=ntr<=0?Double.NaN:this.savedtrades.getBuffer()[ntr-1].price;

	final long currenttime=sd.getTraderCurrentTime();
	// Set current prices, read from book edges.
	final BookItem ib=sd.ot.orderbook.getBestLevel(currenttime,BookItem.BOOK_BUY),
	    is=sd.ot.orderbook.getBestLevel(currenttime,BookItem.BOOK_SELL);
	if(ib!=null){this.p_buy=ib.price;this.vol_p_buy=ib.shares;}
	else {this.p_buy=Double.NaN;this.vol_p_buy=0;}
	if(is!=null){this.p_sell=is.price;this.vol_p_sell=is.shares;}
	else {this.p_sell=Double.NaN;this.vol_p_sell=0;}
    }



    /** @returns: original magic id. */
    public long clear()
    {
	final long old_id=this.id;

	this.sd=null;
	this.id=-1;
	this.p_buy=this.p_sell=this.p_last=0.0;
	this.vol_p_buy=this.vol_p_sell=0;

	// clear trades
	java.util.Arrays.fill(this.arrT,null);
	java.util.Arrays.fill(this.dim_arrT,0);
	this.savedtrades.clearAll();
	for(int i=0;i<this.istockstrades.length;i++)
	{
	    this.istockstrades[i].clearAll();
	}

	return old_id;
    }


    long getMagicID()
    {
	return this.id;
    }


    /** Prepeartion of internal data structures. */
    void prepareData()
    {
    }


    CalculationResults calculateEverything()
    {
	final long currenttime=sd.getTraderCurrentTime();
	final MultiStockStatusData sd=this.sd;


	sd.umlqvml=UMlQVMLaguerre.calculate(sd.mparametersdata.nd,
					    currenttime,
					    sd.mparametersdata.tau,
					    savedtrades.getBuffer(),
					    savedtrades.getSize(),
					    shrinkATradeArray(sd.iot.length,this.arrT),
					    java.util.Arrays.copyOf(this.dim_arrT,sd.iot.length),
					    sd.umlqvml);
	sd.umlqvmp=UMlQVMLegendreShifted.calculate(sd.mparametersdata.nd,
					    currenttime,
					    sd.mparametersdata.tau,
					    savedtrades.getBuffer(),
					    savedtrades.getSize(),
						   java.util.Arrays.copyOf(this.arrT,sd.iot.length),
						   java.util.Arrays.copyOf(this.dim_arrT,sd.iot.length),
					    sd.umlqvmp);
	return new CalculationResults(getMagicID(),
				      currenttime,
				      this.p_buy,
				      this.p_sell,
				      this.p_last,
				      this.vol_p_buy,
				      this.vol_p_sell,
				      this);
    }

    static class CalculationResults {
	final long id,time,vol_p_buy,vol_p_sell;
	final double p_buy,p_sell,p_last;
	final Opportunity topp;
	CalculationResults(final long id,
			   final long time,
			   final double p_buy,
			   final double p_sell,
			   final double p_last,
			   final long vol_p_buy,
			   final long vol_p_sell,
			   final Opportunity topp)
	{
	    this.id=id;
	    this.time=time;
	    this.p_buy=p_buy;
	    this.p_sell=p_sell;
	    this.p_last=p_last;
	    this.vol_p_buy=vol_p_buy;
	    this.vol_p_sell=vol_p_sell;
	    this.topp=topp;
	}
    }


}
