package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2017
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 * @version $Id: PDynMicroImpact.java,v 1.15 2022/08/05 11:25:32 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class PDynMicroImpact {
    public final EVXData I;
    public final double p_offset,p_IH,dpdt_orig_IH,
	imL,p_imL,imH,p_imH;

    PDynMicroImpact(final EVXData I,
		    final double p_offset,final double p_IH,final double dpdt_orig_IH,final double imL,final double p_imL,final double imH,final double p_imH,
		     final double eps)
{
    this.I=I;
    this.p_offset=p_offset;
    this.p_IH=p_IH;
    this.dpdt_orig_IH=dpdt_orig_IH;
    this.imL=imL;this.p_imL=p_imL;this.imH=imH;this.p_imH=p_imH;
}


/**
 * Check the state of min/max dP/dV impact:  |dP/dt|psi>=lambda |dV/dt|psi>.
 */
public static PDynMicroImpact get_PDynMicroImpact(final IMomentsData mm,
						  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
						  final double eps)
{
    final int n=mm.n,ixL=0,ixH=n-1;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQdpdt=EVXData.convertMatrFromQtoB_Basis(n,mm.QQdpdt,I.ev);
    final double [] bQQi=TMatr.getDiagMatrix(I.getEValues()),bimpact=EVXData.getSymEVDiagT(n,bQQdpdt,bQQi,M.EV);
    final double imL=bimpact[ixL],p_imL=TMatr.getAMatrA(n,bimpact,n+ixL*n,bQQpi,n),
	imH=bimpact[ixH],p_imH=TMatr.getAMatrA(n,bimpact,n+ixH*n,bQQpi,n);
    if(eps>0)
    {
	// Check eigenvalues problem is solved OK.
	EVXData.test_orthogonality("PDynMicroImpact.I.ev",n,I.ev,mm.QQi,mm.QQt,eps);
	EVXData.test_orthogonality("PDynMicroImpact.bimpact",n,bimpact,bQQdpdt,bQQi,eps);
	final EVXData DpI=EVXData.getEVXDataFromQQ(n,mm.QQdpdt,mm.QQi,M);
	UnitTests.checkDoubleArrayRel(java.util.Arrays.copyOf(bimpact,n),"bimpact",DpI.getEValues(),"DpI.ev",eps);
	UnitTests.checkDoubleRel(p_imL,"p_imL",DpI.getFPsi(ixL,mm.QQpi),"<psiL|pI|psiL>",eps);
	UnitTests.checkDoubleRel(p_imH,"p_imH",DpI.getFPsi(ixH,mm.QQpi),"<psiH|pI|psiH>",eps);
	System.err.println("PDynMicroImpact.get_PDynMicroImpact: internal test OK, n="+n+" eps="+eps);
    }
    return new PDynMicroImpact(I,mm.p_offset,bQQpi[ixH+n*ixH]/I.ev[ixH],bQQdpdt[ixH+n*ixH],
			       imL,p_imL,imH,p_imH,eps);
}

}

