package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.utils.DerivativesCalculateable;

/**
 *  @version $Id: PTwoRoots.java,v 1.3 2019/01/06 23:18:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class PTwoRoots implements DerivativesCalculateable {
    final double [] kt,kv,pkt,pkv;
    final DeprecatedHelperDataDirectAccessWithBasis<?> access;
    PTwoRoots(final double [] kt,final double [] kv,final double [] pkt,final double [] pkv,final DeprecatedHelperDataDirectAccessWithBasis<?> access)
    {
	this.access=access;
	this.kv=kv;
	this.pkv=pkv;
	this.kt=kt;
	this.pkt=pkt;
    }
    public double getF(final double x)
    {
	return access.fx(pkv,x)/access.fx(kv,x)-access.fx(pkt,x)/access.fx(kt,x);
    }
    public double getDF(final double x)
    {
	final double Kvx=access.fx(kv,x),Ktx=access.fx(kt,x);
	return (access.Dfx(pkv,x)/Kvx-(access.fx(pkv,x)/Kvx)*(access.Dfx(kv,x)/Kvx))
	    -(access.Dfx(pkt,x)/Ktx-(access.fx(pkt,x)/Ktx)*(access.Dfx(kt,x)/Ktx));
    }
    public double getD2F(final double x)
    {
	final double v,t;
	{
	final double Kvx=access.fx(kv,x),pKvx=access.fx(pkv,x),
	    DKvx=access.Dfx(kv,x),DpKvx=access.Dfx(pkv,x),
	    D2Kvx=access.D2fx(kv,x),D2pKvx=access.D2fx(pkv,x),rv=DKvx/Kvx;
	v=(D2pKvx/Kvx)-2*(DpKvx/Kvx)*rv-(pKvx/Kvx)*(D2Kvx/Kvx-2*rv*rv);
	}
	{
	final double Ktx=access.fx(kt,x),pKtx=access.fx(pkt,x),
	    DKtx=access.Dfx(kt,x),DpKtx=access.Dfx(pkt,x),
	    D2Ktx=access.D2fx(kt,x),D2pKtx=access.D2fx(pkt,x),rt=DKtx/Ktx;
	t=(D2pKtx/Ktx)-2*(DpKtx/Ktx)*rt-(pKtx/Ktx)*(D2Ktx/Ktx-2*rt*rt);
	}
	return v-t;
    }
}
