package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: PkQVMMonomials.java,v 1.6 2019/01/06 23:39:12 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PkQVMMonomials extends PkQVM<QVMData> {
PkQVMMonomials(final int neq,final QVMData odata){super(neq,odata);}


public static PkQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final PkQVMMonomials ref)
{
    final int nvm=1,npm=3;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1+1,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new PkQVMMonomials(neq,o);
}
DeprecatedHelperQVMDataDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataDirectAccessWithBasis();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return KkQVMMonomials._getK2(a,aa,b,bb);}

static class PkQVMMonomialsTest extends PkQVMTest<QVMData> {
PkQVMMonomialsTest(final long initrandom)
{
    super("PkQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
PkQVMMonomials getPkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
double getSignToMatchDkQVMBAsis(){return -1;}
DkQVMLaguerre getDkQVM(){return DkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
@Override
void test_Extra(){
    final PkQVMMonomials q=getPkQVM();
    final PkQVMLaguerre s=PkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);
    PkxVol.match2(q.PVt,s.PVt,getSignToMatchDkQVMBAsis(),eps);
    PkxVol.match2(q.PVt_sam,s.PVt_sam,getSignToMatchDkQVMBAsis(),eps);
    PkxVol.match2(q.PVi,s.PVi,getSignToMatchDkQVMBAsis(),eps);
    System.err.println(name+":PkQVMTest:test_Extra(): test OK n="+neq+" eps="+eps);
}

} // END of class PkQVMMonomialsTest

public static void main(String [] args)
{
    new PkQVMMonomialsTest(1117L).runTests(5);
} // END main



} // END OF class PkQVMMonomials
