package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: PnL2I0.java,v 1.30 2022/07/28 12:27:17 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnL2I0 {
    public static final int STDITERATIONS=7;
    public static final double STDFACTIH_NUMSTABILITY=1+1e-5;

    public final EVXData I;
    public final double p_offset,p_IH,I_0_eq,dI_0_eq,P_eq,P_phi0,Pf_eq,P1_eq,phi_fact;
    PnL2I0(final EVXData I,final double p_offset,
	   final double p_IH,
	   final double I_0_eq,final double dI_0_eq,final double phi_fact,
	   final double P_eq,final double P_phi0,final double Pf_eq,final double P1_eq)
    {
	this.I=I;
	this.p_offset=p_offset;
	this.p_IH=p_IH;

	this.I_0_eq=I_0_eq;
	this.dI_0_eq=dI_0_eq;
	this.phi_fact=phi_fact;
	this.P_eq=P_eq;
	this.P_phi0=P_phi0;
	this.Pf_eq=Pf_eq;
	this.P1_eq=P1_eq;
    }


public static PnL2I0 test_get_PnL2I0(final MomentsData mm,
			      final double fact_numstability,
			      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			      final double eps)
{
    final int n=mm.n,iH=n-1,i0=0;
    if(n<2||i0!=0) throw new RuntimeException("n="+n+"<2");
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,mm.QQi,mm.QQt,M),
	bQQi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQi,basis),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,basis);
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double I_0_eq=I.sH*fact_numstability;
    final double [] bphi=new double [n],bpsi0=new double [n];
    bpsi0[i0]=1;
    for(int k=i0+1;k<n;k++){
	bphi[k]=-bQQi[i0+k*n]/(basis[k]-I_0_eq);
    }
    //System.err.println("bphi="+java.util.Arrays.toString(bphi));
    final double
	p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH,
	phi_pI_phi=TMatr.getAMatrA(bphi,bQQpi,n),
	phi_I_phi=TMatr.mulV2V(bphi,java.util.Arrays.copyOf(basis,n)),
	phi_phi=TMatr.mulVV(bphi,bphi),
	psi0_pI_phi=TMatr.getAMatrB(bpsi0,bphi,bQQpi,n),
	psi0_I_phi=TMatr.getAMatrB(bpsi0,bphi,bQQi,n),
	dI_eq=psi0_I_phi,
	P_phi0=phi_pI_phi/phi_I_phi,
	P_eq=(2*psi0_pI_phi+phi_pI_phi)/(2*psi0_I_phi+phi_I_phi),
	Pf_eq=(phi_pI_phi+psi0_pI_phi)/(I_0_eq*phi_phi),
	P1_eq=psi0_pI_phi/psi0_I_phi;
    if(eps>0)
    {
	DeprecatedHelperEVXData.test_orthogonalitySKIP0I("test_get_PnL2I0",n,basis,mm.QQi,mm.QQt,eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi0,bphi),"<bpsi0|bphi>",0,"0",eps);
	UnitTests.checkDoubleRel(phi_I_phi-I_0_eq*phi_phi,"phi_I_phi-I_0_eq*phi_phi",-psi0_I_phi,"-psi0_I_phi",eps);
	System.err.println("PnL2I0.test_get_PnL2I0 psi0_I_phi="+psi0_I_phi+" must be >0 psi0_I_phi/I_0_eq="+(psi0_I_phi/I_0_eq));
	System.err.println("PnL2I0.test_get_PnL2I0 selftest OK n="+n+" eps="+eps);
    }

    final PnL2I0 rr=new PnL2I0(I,mm.p_offset,p_IH,I_0_eq,dI_eq,phi_phi/(1+phi_phi),P_eq,P_phi0,Pf_eq,P1_eq);
    if(eps>0)
    {
	matchPnL2I0(rr,test_get_iter_PnL2I0(mm,fact_numstability,1,M,0),eps);
    }
    return rr;
}


public static PnL2I0 test2_get_PnL2I0(final MomentsData mm,
				      final double fact_numstability,
				      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				      final double eps)
{
    final int n=mm.n,nd=n-1;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH,I_0_eq=I.sH*fact_numstability;
    final double [] r=new double [nd],
	MX=TMatr.sumAcB(VolMatrix.getRadau2Matrix(n,mm.QQi,M),-I_0_eq,VolMatrix.getRadau2Matrix(n,mm.QQt,M));
    for(int k=0;k<nd;k++)
    {
	final double [] pp=new double [k+1];pp[k]=1;
	r[k]=-TMatr.getAMatrB(I.psi0,M.getRadauMultiplied(pp),mm.QQi,n);
    }
    final int [] ipvt=new int [nd];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(MX,nd,nd,ipvt);
    if(pinfo!=0) java.util.Arrays.fill(r,Double.NaN);
    else com.polytechnik.utils.Linsystems.dgesl_m1(MX,nd,nd,ipvt,r,1);
    final double [] phi=M.getRadauMultiplied(r);

    final double
	phi_pI_phi=TMatr.getAMatrA(phi,mm.QQpi,n),
	phi_I_phi=TMatr.getAMatrA(phi,mm.QQi,n),
	phi_phi=TMatr.getAMatrA(phi,mm.QQt,n),
	psi0_I_phi=TMatr.getAMatrB(I.psi0,phi,mm.QQi,n),
	psi0_pI_phi=TMatr.getAMatrB(I.psi0,phi,mm.QQpi,n),
	dI_eq=psi0_I_phi,
	P_phi0=phi_pI_phi/phi_I_phi,
	P_eq=(2*psi0_pI_phi+phi_pI_phi)/(2*psi0_I_phi+phi_I_phi),
	Pf_eq=(phi_pI_phi+psi0_pI_phi)/(I_0_eq*phi_phi),
	P1_eq=psi0_pI_phi/psi0_I_phi;
    if(eps>0)
    {
	EVXData.test_orthogonality("PnL2I0.test2_get_PnL2I0",n,I.ev,mm.QQi,mm.QQt,eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(phi,I.psi0,mm.QQt,n),"<psi0|phi>",0,"0",eps);
	UnitTests.checkDoubleRel(phi_I_phi-I_0_eq*phi_phi,"phi_I_phi-I_0_eq*phi_phi",-psi0_I_phi,"-psi0_I_phi",eps);
	System.err.println("PnL2I0.test2_get_PnL2I0 psi0_I_phi="+psi0_I_phi+" must be >0 psi0_I_phi/I_0_eq="+(psi0_I_phi/I_0_eq));
	System.err.println("PnL2I0.test2_get_PnL2I0 selftest OK n="+n+" eps="+eps);
    }

    return new PnL2I0(I,mm.p_offset,p_IH,I_0_eq,dI_eq,phi_phi/(1+phi_phi),P_eq,P_phi0,Pf_eq,P1_eq);
}

/**
 * An extension of PnLI0 to second order perturbation.
 * Consider variations of {@literal <psi0+phi|I|phi+psi0>/<psi0+phi|I|phi+psi0>}
 * for {@literal <phi|psi0>}=0 (i.e. phi(x0)=0).
 * Expanding numerator &amp; denominator obtain phi functional:
 * (I^future +2{@literal <phi|I|psi0>} + {@literal <phi|I|phi>})/(1 +2{@literal <phi|psi0>} + {@literal <phi|phi>})
 * that now can be variated over the phi.
 * With I^future=lambda_IH +eps the functional is a function with maximux.
 * Then, similar to PnLI0 evaluate price as
 * 1: {@literal <phi|pI|phi>/<phi|I|phi>}
 * 2: (2{@literal <phi|pI|psi0>} + {@literal <phi|pI|phi>})/(2{@literal <phi|I|psi0>} + {@literal <phi|I|phi>})
 * The second one was obtained as pI(phi)=P_eqI(phi)
 */
public static PnL2I0 test_get_iter_PnL2I0(final MomentsData mm,
				   final double fact_numstability,
				   final int n_iterations,
				   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				   final double eps)
{
    final int n=mm.n,iH=n-1,i0=0,nd=n-1;
    if(n<2||i0!=0) throw new RuntimeException("n="+n+"<2");
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,mm.QQi,mm.QQt,M),
	bQQi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQi,basis),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,basis);
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double I_0_eq=I.sH*fact_numstability;
    final double [] bphi=new double [n],bpsi0=new double [n],bQQt=new double [n*n],r=new double [nd];
    bphi[i0]=bpsi0[i0]=1; // initial value: |psi0>
    for(int k=0;k<n;k++) bQQt[k+n*k]=1;
    bQQi[i0+n*i0]=I_0_eq; // future <psi0|I|psi0>
    final int [] ipvt=new int [nd];
    for(int k=0;k<n_iterations;k++)
    {
	final RayleighQuotient.RQVariationInMatrixForm rq=
	    new RayleighQuotient.RQVariationInMatrixForm(bphi,n,bQQi,bQQt,n);
	final double [] MX=DeprecatedHelperEVXData.getOneVectorRemovedFromMatrix(n,rq.d2,i0);
	for(int o=0,l=0;l<n;l++) if(l!=i0) r[o++]=-rq.d1[l];
	final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(MX,nd,nd,ipvt);
	if(pinfo!=0) java.util.Arrays.fill(r,Double.NaN);
	else com.polytechnik.utils.Linsystems.dgesl_m1(MX,nd,nd,ipvt,r,1);
	for(int o=0,l=0;l<n;l++) if(l!=i0) bphi[l]+=r[o++];
	//System.err.println(" iter="+k+" bphi="+java.util.Arrays.toString(bphi));
	if(eps>0 && k==(n_iterations-1))
	{
	    System.err.println("PnL2I0.test_get_iter_PnL2I0: d1="+java.util.Arrays.toString(rq.d1)+" psi="+java.util.Arrays.toString(bphi));
	}
    }
    bphi[i0]=0; // remove inital value

    final double
	p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH,
	phi_pI_phi=TMatr.getAMatrA(bphi,bQQpi,n),
	phi_I_phi=TMatr.mulV2V(bphi,java.util.Arrays.copyOf(basis,n)),
	phi_phi=TMatr.mulVV(bphi,bphi),
	psi0_pI_phi=TMatr.getAMatrB(bpsi0,bphi,bQQpi,n),
	psi0_I_phi=TMatr.getAMatrB(bpsi0,bphi,bQQi,n),
	P_phi0=phi_pI_phi/phi_I_phi,
	P_eq=(2*psi0_pI_phi+phi_pI_phi)/(2*psi0_I_phi+phi_I_phi),
	Pf_eq=(phi_pI_phi+psi0_pI_phi)/(I_0_eq*phi_phi),
	P1_eq=psi0_pI_phi/psi0_I_phi;
    if(eps>0)
    {
	DeprecatedHelperEVXData.test_orthogonalitySKIP0I("PnL2I0.test_get_iter_PnL2I0",n,basis,mm.QQi,mm.QQt,eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi0,bphi),"<bpsi0|bphi>",0,"0",eps);
	System.err.println("PnL2I0.test_get_iter_PnL2I0 selftest OK n="+n+" eps="+eps);
    }

    return new PnL2I0(I,mm.p_offset,p_IH,I_0_eq,psi0_I_phi,phi_phi/(1+phi_phi),P_eq,P_phi0,n_iterations==1?Pf_eq:Double.NaN,P1_eq);
}


public static PnL2I0 test2_get_iter_PnL2I0(final MomentsData mm,
					   final double fact_numstability,
					   final int n_iterations,
					   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
					   final double eps)
{
    final int n=mm.n,nd=n-1;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH,I_0_eq=I.sH*fact_numstability;
    final double [] r2QQi=VolMatrix.getRadau2Matrix(n,mm.QQi,M),r2QQt=VolMatrix.getRadau2Matrix(n,mm.QQt,M),
	r1i=new double [nd],r1t=new double [nd],rphi=new double [nd];
    for(int k=0;k<nd;k++)
    {
	final double [] pp=new double [k+1];pp[k]=1;
	final double [] rpp=M.getRadauMultiplied(pp);
	r1i[k]=2*TMatr.getAMatrB(I.psi0,rpp,mm.QQi,n);
	r1t[k]=2*TMatr.getAMatrB(I.psi0,rpp,mm.QQt,n); // always=0
    }
    final int [] ipvt=new int [nd];
    for(int k=0;k<n_iterations;k++)
    {
	final RayleighQuotient.RQVariationInMatrixForm rq=
	    new RayleighQuotient.RQVariationInMatrixForm(
		new RayleighQuotient.RQFact(rphi,I_0_eq,r1i,r2QQi),
		new RayleighQuotient.RQFact(rphi,1,r1t,r2QQt));
	final double [] MX=rq.d2;
	final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(MX,nd,nd,ipvt);
	if(pinfo!=0) java.util.Arrays.fill(rq.d1,Double.NaN);
	else com.polytechnik.utils.Linsystems.dgesl_m1(MX,nd,nd,ipvt,rq.d1,1);
	TMatr.sumP1(rphi,rq.d1,-1);
    }
    final double [] phi=M.getRadauMultiplied(rphi);

    final double
	phi_pI_phi=TMatr.getAMatrA(phi,mm.QQpi,n),
	phi_I_phi=TMatr.getAMatrA(phi,mm.QQi,n),
	phi_phi=TMatr.getAMatrA(phi,mm.QQt,n),
	psi0_pI_phi=TMatr.getAMatrB(I.psi0,phi,mm.QQpi,n),
	psi0_I_phi=TMatr.getAMatrB(I.psi0,phi,mm.QQi,n),
	P_phi0=phi_pI_phi/phi_I_phi,
	P_eq=(2*psi0_pI_phi+phi_pI_phi)/(2*psi0_I_phi+phi_I_phi),
	Pf_eq=(phi_pI_phi+psi0_pI_phi)/(I_0_eq*phi_phi),
	P1_eq=psi0_pI_phi/psi0_I_phi;
    if(eps>0)
    {
	EVXData.test_orthogonality("PnL2I0.test2_get_iter_PnL2I0",n,I.ev,mm.QQi,mm.QQt,eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(phi,I.psi0,mm.QQt,n),"<psi0|phi>",0,"0",eps);
	// <psi0|(x-x0)Q(x)>=0
	UnitTests.checkDoubleArrayRel(r1t,"r1t",new double [nd],"0",eps);
	System.err.println("PnL2I0.test2_get_iter_PnL2I0 selftest OK n="+n+" eps="+eps);
    }

    return new PnL2I0(I,mm.p_offset,p_IH,I_0_eq,psi0_I_phi,phi_phi/(1+phi_phi),P_eq,P_phi0,n_iterations==1?Pf_eq:Double.NaN,P1_eq);
}


public static void matchPnL2I0(final PnL2I0 a,final PnL2I0 b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);

    UnitTests.checkDoubleRel(a.I_0_eq,"a.I_0_eq",b.I_0_eq,"b.I_0_eq",eps);
    UnitTests.checkDoubleRel(a.dI_0_eq,"a.dI_0_eq",b.dI_0_eq,"b.dI_0_eq",eps);
    UnitTests.checkDoubleRel(a.phi_fact,"a.phi_fact",b.phi_fact,"b.phi_fact",eps);
    UnitTests.checkDoubleRel(a.P_eq,"a.P_eq",b.P_eq+p_offset_shift,"b.P_eq+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.Pf_eq,"a.Pf_eq",b.Pf_eq+p_offset_shift,"b.Pf_eq+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P1_eq,"a.P1_eq",b.P1_eq+p_offset_shift,"b.P1_eq+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P_phi0,"a.P_phi0",b.P_phi0+p_offset_shift,"b.P_phi0+p_offset_shift",eps);
    System.err.println("PnL2I0.matchPnL2I0 test OK n="+a.I.neq+" eps="+eps);
}
}
