package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.SortArrays;

/**
 * Aug, 23, 2016 "Stamford Train" version.
 * @version $Id: PnLDiagProj.java,v 1.60 2022/08/07 15:44:03 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLDiagProj {
    public final EVXData I;
    public final double p_offset,p_last,p_IH,p1,p2,maxPnL,I1,I2,V1,V2,d1,d2,psix0_1,psix0_2;
    public final int e1,e2;
    final double [] pI,p,V,pV,dp;
    final double signedIa0,Ia,signedI_p_last,signedI0_p_last,PnL_L,PnL_H,PnL_pl_L,PnL_pl_H,PnL_IH_L,PnL_IH_H;
    final double i0_I1,i0_I2,i0_p1,i0_p2,i0_psi1,i0_psi2,
	ii0_I1,ii0_I2,ii0_p1,ii0_p2,ii0_psi1,ii0_psi2;
    PnLDiagProj(final EVXData I,final double p_offset,final double p_last,
		final double [] pI,final double [] V,final double [] pV,final double [] dp,
		final double eps)
{
    this.I=I;
    final int iX=I.neq-1;
    this.p_offset=p_offset;
    this.p_last=p_last;
    final int n=I.neq;
    this.pI=pI;
    this.V=V;
    this.pV=pV;
    this.dp=dp;
    this.p=new double [n];
    final double [] psik2_at_x0=TMatr.squareArrayElements(I.psik_at_x0);
    // find enter/exit states, providing max P&L
    double absmaxpnl=-1;int e1=0,e2=0;
    for(int k=0;k<n;k++)
    {
	p[k]=pI[k]/I.ev[k];
    	for(int l=0;l<k;l++)
	{
	    final double abspnl=Math.min(I.ev[k],I.ev[l])*Math.abs(p[k]-p[l]);
	    if(abspnl>=absmaxpnl){absmaxpnl=abspnl;e1=l;e2=k;}
	}
    }
    this.e1=e1;
    this.e2=e2;
    this.p_IH=p[iX];
    final double I_IX=I.ev[iX];
    this.p1=p[e1];this.V1=V[e1];this.I1=I.ev[e1];this.d1=-V1*psik2_at_x0[e1]/I.ev[e1]-1;this.psix0_1=psik2_at_x0[e1];
    this.p2=p[e2];this.V2=V[e2];this.I2=I.ev[e2];this.d2=-V2*psik2_at_x0[e2]/I.ev[e2]-1;this.psix0_2=psik2_at_x0[e2];
    this.maxPnL=absmaxpnl;

    // try the two states close to psi0
    if(true)
    {
	final int [] ind=SortArrays.sortArrayIndexes(psik2_at_x0);
	final int i1=ind[n-2],i2=ind[n-1]; // top 2 max psi0
	this.i0_I1=I.ev[i1];
	this.i0_I2=I.ev[i2];
	this.i0_p1=p[i1];
	this.i0_p2=p[i2];
	this.i0_psi1=psik2_at_x0[i1];
	this.i0_psi2=psik2_at_x0[i2];
    }
    // try the two states close to I*psi0
    if(true)
    {
	final int [] ind=SortArrays.sortArrayIndexes(TMatr.getVAB(psik2_at_x0,I.getEValues()));
	final int i1=ind[n-2],i2=ind[n-1]; // top 2 max I*psi0
	this.ii0_I1=I.ev[i1];
	this.ii0_I2=I.ev[i2];
	this.ii0_p1=p[i1];
	this.ii0_p2=p[i2];
	this.ii0_psi1=psik2_at_x0[i1];
	this.ii0_psi2=psik2_at_x0[i2];
    }

    // Sort the psi states on the value of price, corresponding to psi state.
    final int [] ind_p=SortArrays.sortArrayIndexes(p);
    this.Ia=TMatr.sumElements(I.getEValues());
    // MAX P&L (MEDIAN). Trading psi stated with "buy below median, sell above median" strategy.
    if(true)
    {
	// Position -1<=pos<=1. What fraction of I=dV/dt to take to receive max P&L.
	this.pos=new double [n];
	double skprev=0;
	// All, corresponding to max P&L poitions are +-1,
	// except a single one, close to median, with a lower than 1 value,
	// adjusted to have 0=\sum pos_k I_k.
	// This actually is a trivial "buy below median, sell above median"
	// strategy, one have to buy/sell pos_k*I_k on k--th state depending on pos_k sign.
	for(int k=0;k<n;k++)
	{
	    final int ind=ind_p[k];
	    final double ik=I.ev[ind],diff=skprev-(Ia-skprev-ik),adiff=Math.abs(diff);
	    if(diff<=0) pos[ind]=1;
	    else if(diff>=0) pos[ind]=-1;
	    if(adiff<=ik)
	    { // should be executed only for a single k
		pos[ind]*=adiff/ik;
	    }
	    skprev+=ik;
	}
	this.signedIa0=TMatr.mulVVV(pos,I.getEValues(),psik2_at_x0);
    } // END MAX P&L (MEDIAN).

    // MAX P&L (p_last). Trading psi stated with "buy below p_last, sell above p_last" strategy.
    if(true)
    {
	// Lebesgue Measure: buy below p_last, sell above p_last.
	this.pos_p_last=new double [n];
	for(int k=0;k<n;k++)
	{
	    final double ik=I.ev[k];
	    // p_last does have p_offset included. can be directly compared to p[k].
	    if(p[k]<=p_last) pos_p_last[k]=1;
	    else if(p[k]>=p_last) pos_p_last[k]=-1;
	}
	this.signedI_p_last=TMatr.mulVVn1(pos_p_last,I.ev);
	this.signedI0_p_last=TMatr.mulVVV(pos_p_last,I.getEValues(),psik2_at_x0);
    } // END MAX P&L (p_last).
    // MAX P&L (p_last), single state.
    if(true)
    {
	double PnL_pl_L=0,PnL_pl_H=0;
	for(int k=0;k<n;k++)
	{
	    final double ik=I.ev[k],pik=pI[k],pnl=pik-p_last*ik;
	    if(pnl>=PnL_pl_H) {PnL_pl_H=pnl;}
	    if(pnl<=PnL_pl_L) {PnL_pl_L=pnl;}
	}
	this.PnL_pl_L=PnL_pl_L;
	this.PnL_pl_H=PnL_pl_H;
    }// END MAX P&L (p_last), single state.
     // MAX P&L with an exit at iX, single state.
    if(true)
    {
	double PnL_IH_L=0,PnL_IH_H=0;
	for(int k=0;k<n;k++)
	{
	    final double ik=I.ev[k],pik=pI[k],pnl=pik-p_IH*ik;
	    if(k!=iX && pnl>=PnL_IH_H) {PnL_IH_H=pnl;}
	    if(k!=iX && pnl<=PnL_IH_L) {PnL_IH_L=pnl;}
	}
	this.PnL_IH_L=PnL_IH_L;
	this.PnL_IH_H=PnL_IH_H;
    }// END MAX P&L with an exit at iX, single state.

    // MAX P&L with an exit at iX
    if(true)
    {
	double s=0,ps=0;
	for(int k=0;k<n;k++)
	{
	    final int ind=ind_p[k];
	    if(ind==iX) break;
	    if(eps>0 && p[ind]>=p_IH) throw new RuntimeException();
	    final double ik=I.ev[ind],pik=pI[ind];
	    if(s+ik<=I_IX) {s+=ik;ps+=pik;}
	    else {final double posfact=(I_IX-s)/ik; s+=posfact*ik;ps+=posfact*pik;break;}
	}
	this.PnL_L=p[iX]*s-ps;
	if(eps>0)
	{
	    System.err.println("PnLDiagProj:PnL_L="+PnL_L+" s="+s+" I_IX="+I_IX+" index(iX,ind_p[*])="+com.polytechnik.utils.ReadWriteUtils.elementIndex(ind_p,iX));
	}
	s=ps=0;
	for(int k=n;--k>=0;)
	{
	    final int ind=ind_p[k];
	    if(ind==iX) break;
	    if(eps>0 && p[ind]<=p_IH) throw new RuntimeException();
	    final double ik=I.ev[ind],pik=pI[ind];
	    if(s+ik<=I_IX) {s+=ik;ps+=pik;}
	    else {final double posfact=(I_IX-s)/ik; s+=posfact*ik;ps+=posfact*pik;break;}
	}
	this.PnL_H=p[iX]*s-ps;
	if(eps>0)
	{
	    System.err.println("PnLDiagProj:PnL_H="+PnL_H+" s="+s+" I_IX="+I_IX+" index(iX,ind_p[*])="+com.polytechnik.utils.ReadWriteUtils.elementIndex(ind_p,iX));
	}
    } // END MAX P&L with an exit at iX
    if(eps>0)
    {
	// dS=I*pos ; P&L= - sum p dS ; 0 = sum dS
	final double sumpos=TMatr.mulVVn1(pos,I.ev),sumPpos=-TMatr.mulVV(pos,pI);
	UnitTests.checkDoubleRel(sumpos,"sumpos",0,"0",eps);
	if(sumPpos<0) throw new RuntimeException("sumPpos="+sumPpos+" <0");
	System.err.println("PnLDiagProj: sum(pos*I)="+sumpos+" must be =0; Ia="+Ia+" sumPpos="+sumPpos+" must be >0 pos="+java.util.Arrays.toString(pos));
	final double [] t_i=new double[n],t_p=new double[n],t_pos=new double[n];
	int posne1=0;
	for(int k=0;k<n;k++)
	{
	    final int ind=ind_p[k];
	    t_p[k]=p[ind];
	    if(k>0 && t_p[k]<t_p[k-1]) throw new RuntimeException("data is not sorted");
	    t_i[k]=I.ev[ind];
	    t_pos[k]=pos[ind];
	    if(Math.abs(t_pos[k])!=1)
	    {
		posne1++;
	    }
	}
	if(posne1>1) throw new RuntimeException("posne1="+posne1);
	System.err.println("PnLDiagProj:t_i="+java.util.Arrays.toString(t_i));
	System.err.println("PnLDiagProj:t_p="+java.util.Arrays.toString(t_p));
	System.err.println("PnLDiagProj:t_pos="+java.util.Arrays.toString(t_pos));
	System.err.println("PnLDiagProj: tests OK n="+n+" p_last="+p_last+" p_offset="+p_offset+" eps="+eps);
    }
}
    final double [] pos,pos_p_last;

/**
 * Find (eigenvector,eigenvalue) pairs (lambda,{@literal |psi>})
 * of generalized eigenvalue problem {@literal |I|psi>}=lambda {@literal |psi>}.
 * According to my paper
 * "Market Dynamics. On Supply and Demand Concepts, Feb 14, 2016, arXiv:1602.04423"
 * all infinitesimal {@literal |psi>} variations are useless,
 * eigenvectors of |p| and |I| matrices
 * (here |x| is a matrix in bra-ket notation not abs(x))
 * are approximately the same (in dynamic impact approximation).
 * For the states {@literal |I|psi_k>}=lambda_k {@literal |psi_k>}
 * consider {@literal <psi_j|pI|psi_k>} matrix and assume
 * only diagonal elements of it are non--zero (dynamic impact approximation).
 * Next step is to calculate the state of max P&amp;L, find e1,e2 pair, such that:
 * P&amp;L=max_{e1,e2} min(I_e1,I_e2)*abs(p_e1-p_e2),
 * (in case I=const this is a regular "buy lowest p, sell highest p" strategy,
 * for a non--constant I the value of I also enter into the P&amp;L).
 * These two (e1,e2) states determine "enter" and "exit" conditions of P&amp;L trading strategy.
 * Then determine the direction
 * either from the sign of {@literal <psi_e2|V|psi_e2>}-{@literal <psi_e1|V|psi_e1>}, or,
 * another option, consider \int dx \int dy psi^2_e2(x)psi^2_e1(y)sign(x-y)
 * see KkQVM{LegendreShifted,Laguerre,Monomials}_getK2 as implementation
 * for various bases, where analytical solution exists.
 */
public static PnLDiagProj get_PnLDiagProj(final MomentsData mm,
				   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				   final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);

    final double [] pI=new double[n],V=new double[n],pV=new double[n],dp=new double[n];
    for(int k=0;k<n;k++)
    {
	pI[k]=I.getFPsi(k,mm.QQpi);
	V[k]=I.getFPsi(k,mm.QQV);
	pV[k]=I.getFPsi(k,mm.QQpV);
	dp[k]=I.getFPsi(k,mm.QQdp);
    }
    if(eps>0)
    {
	// Check eigenvalues problem is solved OK.
	EVXData.test_orthogonality("PnLDiagProj.I.ev",n,I.ev,mm.QQi,mm.QQt,eps);
	// Check spur invariance. (Density matrix state invariance.)
	final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,mm.QQt);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQV),"Spur(IQQt,QQV)",TMatr.sumElements(V),"sum(V)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQpV),"Spur(IQQt,QQpV)",TMatr.sumElements(pV),"sum(pV)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQpi),"Spur(IQQt,QQpi)",TMatr.sumElements(pI),"sum(pI)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQdp),"Spur(IQQt,QQdp)",TMatr.sumElements(dp),"sum(dp)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQi),"Spur(IQQt,QQi)",TMatr.sumElements(I.getEValues()),"sum(I)",eps);
	System.err.println("PnLDiagProj.get_PnLDiagProj:all<0 V="+java.util.Arrays.toString(V));
	System.err.println("PnLDiagProj.get_PnLDiagProj:all>0 I="+java.util.Arrays.toString(I.getEValues()));
	System.err.println("PnLDiagProj.get_PnLDiagProj: all internal tests OK, all invariance tests OK n="+n+" eps="+eps);
    }

    return new PnLDiagProj(I,mm.p_offset,mm.p_last,pI,V,pV,dp,eps);
}


}

