package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2020
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.EVXData;
import com.polytechnik.freemoney.WIntegrator;

/** Save the state, WIntegrator is added for convenience of having the state of
 * {@link PnLILowIHighTPV PnLILowIHighTPV} class stored in a single class,
 * @version $Id: PnLILowIHighStateSaverWithWI.java,v 1.30 2022/07/10 10:59:03 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLILowIHighStateSaverWithWI extends  SimpleStateSaver {


    /** Recurrent integrator.
     */
    final WIntegrator wi;

    /** Initial state.
     */
    public PnLILowIHighStateSaverWithWI(final WIntegrator wi){
	super();
	this.wi=wi;if(wi.nobservables!=5) throw new IllegalArgumentException();
    }

    /** Recurrently updated state.
     */
    public PnLILowIHighStateSaverWithWI(final PnLILowIHighStateSaverWithWI prev,
					final long time,final double P,final long dV,
					final EVXData I)
    {
	super(prev,time,P,dV,I);
	this.wi=prev.wi;
    }

/** Update integrator.
 */
void addSingleObservation_PdV(final long currenttime,final double P,final double dV){
    // three arguments, the order: dv:[0],P*dV:[1]
    final long t=currenttime;
    if(dV>0 && Double.isFinite(P)){
	wi.addSingleObservation(currenttime,t,new double []{dV/*[0]*/,P*dV/*[1]*/,0,0,0});
    }
}
void addSingleObservation_PnLs(final long currenttime,/*final long t,*/final double PriceChangePnL,final double AbsPriceChangePnL,final double dIHpnl){
    final long t=currenttime;
    wi.addSingleObservation(currenttime,t,new double []{0,0,dIHpnl/*[2]*/,PriceChangePnL/*[3]*/,AbsPriceChangePnL/*[4]*/});
}

// Convenience methods to access integrator fields by name, not by index.
double [] getDvMoments(){return wi.integration_moments[0/* dV */];}
double [] getPDvMoments(){return wi.integration_moments[1/* P*dV */];}

double [] getDIHpnlMoments(){return wi.integration_moments[2/* dIHpnl */];}
double [] getPcDIHpnlMoments(){return wi.integration_moments[3/* PriceChange*dIHpnl */];}
double [] getAbsPcDIHpnlMoments(){return wi.integration_moments[4/* AbsPriceChange*dIHpnl */];}
/** Plain sum of the directional indicator. */
public double getSumPcDIH(){return wi.sum[3/* PriceChange*dIHpnl */];}
/** For unit test. */
public double getSumDv(){return wi.sum[0/* dV */];}

} // END OF class PnLILowIHighStateSaverWithWI
