package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: PnLImpactA.java,v 1.19 2020/09/27 20:33:18 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 * Condider "P&L impact states", the eigenfunctions of
 * dPnL/dt , providing the extremum of PnL.
 * The state of maximal PnL is of most interest.
 */
public abstract class PnLImpactA<DTYPE> {

public static class State {
    public final long seq;
    State(final State prev)
    {
	this.seq=(prev==null?0:1+prev.seq);
    }
}
public final State sst;

public final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
public final EVXData I,PnL,PnLa,IA,Vmax;
public final double p_offset,p_IH,dp_IH,PnL_H_dp,PnL_H_dir,PnLa_H_dir;
public final Skewness sk_PnLa,sk_IAH,sk_Vmax;
final DTYPE odata;
PnLImpactA(final int n,final DTYPE odata,final State rst)
{
    //if(n<=1) throw new RuntimeException("n="+n+" must be >1");
    final double eps=0;
    this.odata=odata;
    this.M=getABasis();
    final JMomentsData mm=getJMomentsData(n);
    p_offset=mm.p_offset;
    // Our "old" solution p_IH: price corresponding the state of max I of past sample.
    this.I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH;
    dp_IH=TMatr.getAMatrA(I.psiH,mm.QQdpdt,n);

    // Now consider P&L impact extremal states: dPnL/dt extremums.
    PnL=EVXData.getEVXDataFromQQ(n,mm.QQDawv,mm.QQt,M);
    PnL_H_dp=TMatr.getAMatrA(PnL.psiH,mm.QQdpdt,n);
    PnL_H_dir=TMatr.getAMatrA(PnL.psiH,mm.QQDpwv,n)/PnL.sH;
    // Try dPnL/da (equivalent to Vmax)
    PnLa=EVXData.getEVXDataFromQQ(n,mm.QQDawv,mm.QQa,M);
    PnLa_H_dir=TMatr.getAMatrA(PnLa.psiH,mm.QQDpwv,n)/PnLa.sH;
    sk_PnLa=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(PnLa.psiH,mm.QQDawv,n),
	TMatr.getAMatrA(PnLa.psiH,mm.QQpDawv,n),
	TMatr.getAMatrA(PnLa.psiH,mm.QQp2Dawv,n),
	TMatr.getAMatrA(PnLa.psiH,mm.QQp3Dawv,n));
    // Try price impact
    IA=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQa,M);
    sk_IAH=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(IA.psiH,mm.QQi,n),
	TMatr.getAMatrA(IA.psiH,mm.QQpi,n),
	TMatr.getAMatrA(IA.psiH,mm.QQp2i,n),
	TMatr.getAMatrA(IA.psiH,mm.QQp3i,n));
    // Try Vmax
    Vmax=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQad,M);
    sk_Vmax=Skewness.getSkewnessFromMonomialMoments(
	TMatr.getAMatrA(Vmax.psiH,mm.QQi,n),
	TMatr.getAMatrA(Vmax.psiH,mm.QQpi,n),
	TMatr.getAMatrA(Vmax.psiH,mm.QQp2i,n),
	TMatr.getAMatrA(Vmax.psiH,mm.QQp3i,n));

    this.sst=new State(rst);
}
protected JMomentsData getJMomentsData(final int n){return JMomentsData.get_JMomentsData(n,M,odata,getAccess());}
abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();


}// END OF class PnLImpactA
