package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: PnLInPsiHstateLaguerre.java,v 1.48 2022/09/14 21:18:41 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLInPsiHstateLaguerre extends PnLInPsiHstate<SaveTwoData<QVMDataL>> {
PnLInPsiHstateLaguerre(final int neq,final SaveTwoData<QVMDataL> odata){super(neq,odata);}


public static PnLInPsiHstateLaguerre calculate(
    final int neq,
    final long currenttime,
    final long tau,
    final Trade [] tb,
    final int tsize,
    final PnLInPsiHstateLaguerre ref)
{

    final int npm=4;
    final QVMDataL o=QVMDataL.calculateQVMDataL(1+2*(neq-1),
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata.d:null),
    	o2=QVMDataL.calculateQVMDataL(1+4*(neq-1),
				      npm,
				      currenttime,
				      tau/2,
				      tb,
				      tsize,
				      ref!=null?ref.odata.d2:null);
    return new PnLInPsiHstateLaguerre(neq,new SaveTwoData<QVMDataL>(o,o2));
}
@Override OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
@Override double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}
@Override double [] getPolynomialWIntegratedDt(final double [] polynomial){return WIntegratorLaguerre.getPolynomialWIntegratedDt(polynomial);}

@Override SMomentsData getSMoments(final int nmoms)
{
    // Whether to use sampled or integrated by parts t-moments.
    final boolean flag_use_samped_t_moments=false;
    return SMomentsData.get_MomentsData(nmoms,odata.d,getAccess(),flag_use_samped_t_moments);
}

@Override K2Distance getK2Distance(){return KkQVMLaguerre.K2DISTANCE;}

private QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}

double [] getPPHOrig2Moments(int nmoms,double p_eq){
    final var access=getAccess();
    final double [] imom2=new double [nmoms];access.setM(imom2,0,access.M_I,odata.d2,nmoms);
    final double [] pimom2=new double [nmoms];access.setM(pimom2,1,access.M_I,odata.d2,nmoms);
    // to have proper dt/tau  (tau is twice lower for odata.d2),
    // the getLaguerreExpFIntegrated assumes dt/tau integration and scale it, double the result to compensate
    //System.err.println("access2:"+java.util.Arrays.toString(imom2)+" Vol="+odata.d2.vol+" t="+odata.d2.currenttime+" tau="+odata.d2.tau);
    TMatr.scaleVectorInPlace(imom2,2);
    TMatr.scaleVectorInPlace(pimom2,2);
    // shift odata.d2 price offset to be the same one as in odata.d
    final double dp_offset=access.getPOffset(odata.d2)-access.getPOffset(odata.d); // different p_offset
    Polynomials.shiftPOffsetInPlace(nmoms,imom2,pimom2,null,null,-dp_offset);


    // (p-p_eq)*I
    TMatr.sumP1(pimom2,imom2,-p_eq);
    // different basis: tau twice lower
    return KkQVMLaguerre.getMomentsConvertedArg(pimom2,2);
}

private double [] getPPHOrig1Moments(int nmoms,double p_eq){
    final var access=getAccess();
    final double [] imom=new double [nmoms];access.setM(imom,0,access.M_I,odata.d,nmoms);
    final double [] pimom=new double [nmoms];access.setM(pimom,1,access.M_I,odata.d,nmoms);
    // (p-p_eq)*I
    TMatr.sumP1(pimom,imom,-p_eq);
    return pimom;
}

@Override double [] getPositionMoments(final int nmoms,final double p_eq,final double [] psiM){
    final int nmomsOrig=nmoms+2*(psiM.length-1);

    return getPositionMomentsFromOrigMoments(nmoms,psiM,getPPHOrig2Moments(nmomsOrig,p_eq));
}

static double [] getPositionMomentsFromOrigMoments(final int nmoms,
						   final double [] psi,
						   final double [] origMoments){

    final double [] pnlmoms=WIntegratorLaguerre.B.getPQkMomentsFromQkMoments(
	WIntegratorLaguerre.B.mult2Pol(psi,psi),origMoments);

    if(pnlmoms.length!=origMoments.length-2*(psi.length-1)) throw new IllegalArgumentException();
    if(pnlmoms.length!=nmoms) throw new IllegalArgumentException();

    final double [] Smoms=new double [nmoms];
    for(int k=0;k<nmoms;k++){
	final double [] qk=new double [k+1];
	qk[k]=1;
	final double [] intQk=WIntegratorLaguerre.B.getLaguerreExpFIntegrated(qk);
	Smoms[k]=TMatr.mulVVn1(intQk,pnlmoms);
    }
    return Smoms;
}


static class PnLInPsiHstateLaguerreTest extends PnLInPsiHstateTest<SaveTwoData<QVMDataL>> {
PnLInPsiHstateLaguerreTest(final long initrandom)
{
    super("PnLInPsiHstateLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/24)/2*2;
}
PnLInPsiHstateLaguerre getPnLInPsiHstate(){return calculate(neq,currenttime,tau,tb,tb.length,null);}


@Override void test_Extra(){
    final int n=neq;
    final var B=WIntegratorLaguerre.B;
    final PnLInPsiHstateLaguerre q=getPnLInPsiHstate();

    for(var s: q.MT.statesM){
	final double [] psiM2=B.mult2Pol(s.psiM,s.psiM),
	    origP=q.getPPHOrig1Moments(q.odata.d.ntm,s.p_M),
	    origP2=q.getPPHOrig2Moments(q.odata.d2.ntm,s.p_M),
	    Smom=q.getPositionMoments(2*n-1,s.p_M,s.psiM);

	// average check
	UnitTests.checkDoubleRel(TMatr.mulVVn1(psiM2,origP),"<psiM|(p-p_M)*I|psiM>",0,"0",eps);

	// match average position and \integral dt (p-p_eq)*I*psi^2(x)*omega(x)*omega(x)
	UnitTests.checkDoubleRel(Smom[0],"Smoms[0]",TMatr.mulVVn1(psiM2,origP2),"integral (p-p_eq)*I*psiM^2(x)omega^2(x) dt=<(p-p_eq)*I*psiM^2(x)*omega(x)>",eps);

	// test common relation
	for(int k=0;k<n;k++){
	    final double [] psi=q.I.getPsi(k),
		psiDPsi=B.mult2Pol(psi,q.getEDPsi(psi)),
		psi2=B.mult2Pol(psi,psi),
		psiM2psi2=B.mult2Pol(psiM2,psi2);
	    UnitTests.checkDoubleRel(2*TMatr.mulVVn1(psiDPsi,Smom),"<S*D(psi2)>",TMatr.mulVVn1(psiM2psi2,origP2),"int by parts",eps);
	}
    }

    System.err.println(name+":test_Extra(): test OK n="+n+" eps="+eps);
}

@Override void test_IvsVT_CalculateMoments_Extra(){
    final int n=neq;
    final PnLInPsiHstateLaguerre q=getPnLInPsiHstate();
    final int nmoms=q.odata.d.ntm;
    final SMomentsData mm=q.getSMoments(nmoms);

    for(var mD:getMomentsToMatch()){
	UnitTests.checkDoubleArrayRel(mm.imom,"mm.imom",mD.getdVMoments(),"mD.getdVMoments()",eps);
	UnitTests.checkDoubleArrayRel(mm.tmom,"mm.tmom",mD.getdtMoments(),"mD.getdtMoments()",eps);
	// for t^k*exp(-t/tau) bases this is just a multiplication by t
	final double [] Timom=TMatr.scaleVectorInPlace(q.M.B.getXQkMomentsFromQkMoments(mm.imom),-1), // different time direction for Laguerre
	    mD_Timom=java.util.Arrays.copyOf(mD.getTdVMoments(),Timom.length);
	UnitTests.checkDoubleArrayRel(Timom,"Timom",mD_Timom,"mD_Timom",eps);
	final double [] Tamom=TMatr.scaleVectorInPlace(q.M.B.getXQkMomentsFromQkMoments(mm.amom),-1), // different time direction for Laguerre
	    mD_Tamom=java.util.Arrays.copyOf(mD.getTdAMoments(),Tamom.length);
	UnitTests.checkDoubleArrayRel(Tamom,"Tamom",mD_Tamom,"mD_Tamom",eps);

	// check the moments with half tau:
	final double [] mpnl2=q.getPPHOrig2Moments(q.odata.d2.ntm,q.pi_IH-mm.p_offset),
	    mD_mpnl2=mD.getPPHOrig2Moments(q.pi_IH-mD.getPOffset());
	UnitTests.checkDoubleArrayRel(mpnl2,"mpnl2",mD_mpnl2,"mD_mpnl2",eps);
    }

    System.err.println(name+":test_IvsVT_CalculateMoments_Extra(): test OK with N points="+tb.length+" neq="+neq+" eps="+eps);
}


@Override java.util.List<IvsVT_CalculateMoments_Laguerre> getMomentsToMatch(){
    return java.util.List.of(
	CommonlyUsedMoments_TestFromSample.fromSeqDirectly(IvsVT_CalculateMoments_Laguerre::new,neq,currenttime,tau,tb),
	CommonlyUsedMoments_TestFromSample.fromSeqRecurrently(IvsVT_CalculateMoments_Laguerre::new,neq,currenttime,tau,tb),
	CommonlyUsedMoments_TestFromSample.fromSeqRecurrentlyTestSecondarySampling(IvsVT_CalculateMoments_Laguerre::new,neq,currenttime,tau,tb,eps)
			     );
}


public static void main(String [] args)
{
    new PnLInPsiHstateLaguerreTest(1127L).runTests(2,4);
} // END main
} // END of class PnLInPsiHstateLaguerreTest



}
