package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: PnLInPsiHstateLegendreShifted.java,v 1.44 2022/09/14 21:18:41 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLInPsiHstateLegendreShifted extends PnLInPsiHstate<QVMDataP> {
PnLInPsiHstateLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}


public static PnLInPsiHstateLegendreShifted calculate(
    final int neq,
    final long currenttime,
    final long tau,
    final Trade [] tb,
    final int tsize,
    final PnLInPsiHstateLegendreShifted ref)
{

    final int npm=4,nmoms=1+4*(neq-1)+1;
    final QVMDataP o=QVMDataP.calculateQVMDataP(nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new PnLInPsiHstateLegendreShifted(neq,o);
}

@Override OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
@Override double [] getEDPsi(final double [] psi){return WIntegratorLegendreShifted.getEDPsi(psi);}
@Override double [] getPolynomialWIntegratedDt(final double [] polynomial){return WIntegratorLegendreShifted.getPolynomialWIntegratedDt(polynomial);}
@Override SMomentsData getSMoments(final int nmoms)
{
    // Whether to use sampled or integrated by parts t-moments.
    final boolean flag_use_samped_t_moments=false;
    return SMomentsData.get_MomentsData(nmoms,odata,getAccess(),flag_use_samped_t_moments);
}

@Override K2Distance getK2Distance(){return KkQVMLegendreShifted.K2DISTANCE;}

private QVMDataPDirectAccess getAccess(){return new QVMDataPDirectAccess();}

private double [] getPPHOrigMoments(int nmoms,double p_eq){
    final var access=getAccess();
    final double [] imom=new double [nmoms];access.setM(imom,0,access.M_I,odata,nmoms);
    final double [] pimom=new double [nmoms];access.setM(pimom,1,access.M_I,odata,nmoms);
    TMatr.sumP1(pimom,imom,-p_eq); // same basis, same p_offset
    return pimom;
}

@Override double [] getPositionMoments(final int nmoms,final double p_eq,final double [] psiM){
    final int nmomsOrig=nmoms+2*(psiM.length-1)+1;

    return getPositionMomentsFromOrigMoments(nmoms,psiM,getPPHOrigMoments(nmomsOrig,p_eq));
}

static double [] getPositionMomentsFromOrigMoments(final int nmoms,
						   final double [] psi,
						   final double [] origMoments){

    final double [] pnlmoms=WIntegratorLegendreShifted.B.getPQkMomentsFromQkMoments(
	WIntegratorLegendreShifted.B.mult2Pol(psi,psi),origMoments);

    if(pnlmoms.length!=origMoments.length-2*(psi.length-1)) throw new IllegalArgumentException();
    if(pnlmoms.length!=nmoms+1) throw new IllegalArgumentException();

    final double [] Smoms=new double [nmoms];
    for(int k=0;k<nmoms;k++){
	final double [] qk=new double [k+1];
	qk[k]=1;
	final double [] intQk=WIntegratorLegendreShifted.B.getIntegrated(qk);
	Smoms[k]=TMatr.mulVVn1(intQk,pnlmoms);
    }
    return Smoms;
}




static class PnLInPsiHstateLegendreShiftedTest extends PnLInPsiHstateTest<QVMDataP> {
PnLInPsiHstateLegendreShiftedTest(final long initrandom)
{
    super("PnLInPsiHstateLegendreShifted",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
PnLInPsiHstateLegendreShifted getPnLInPsiHstate(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

@Override void test_Extra(){
    final int n=neq;
    final var B=WIntegratorLegendreShifted.B;
    final PnLInPsiHstateLegendreShifted q=getPnLInPsiHstate();

    for(var s: q.MT.statesM){
	final double [] psiM2=B.mult2Pol(s.psiM,s.psiM),
	    origP=q.getPPHOrigMoments(q.odata.ntm,s.p_M),
	    XorigP=B.getXQkMomentsFromQkMoments(origP),
	    Smom=q.getPositionMoments(2*n-1,s.p_M,s.psiM);

	UnitTests.checkDoubleRel(TMatr.mulVVn1(psiM2,origP),"<psiM|(p-p_M)*I|psiM>",0,"0",eps);
	// match average position and <x*(p-p_M)*I*psi^2(x)omega(x)>
	UnitTests.checkDoubleRel(Smom[0],"Smoms[0]",TMatr.mulVVn1(psiM2,XorigP),"<x*(p-p_M)*I*psi^2(x)omega(x)>",eps);

	// test common relation
	for(int k=0;k<n;k++){
	    final double [] psi=q.I.getPsi(k),
		psiDPsi=B.mult2Pol(psi,q.getEDPsi(psi)),
		XpsiPsi=B.getaxbP(B.mult2Pol(psi,psi),1,0),
		psiM2XpsiPsi=B.mult2Pol(psiM2,XpsiPsi);
	    UnitTests.checkDoubleRel(2*TMatr.mulVVn1(psiDPsi,Smom),"<S*D(psi2)>",TMatr.mulVVn1(psiM2XpsiPsi,origP),"int by parts",eps);
	}
    }

    System.err.println(name+":test_Extra(): test OK n="+n+" eps="+eps);
}

@Override void test_IvsVT_CalculateMoments_Extra(){
    final int n=neq;
    final PnLInPsiHstateLegendreShifted q=getPnLInPsiHstate();
    for(var mD: getMomentsToMatch()){
	final double [] mD_Timom=mD.getTdVMoments();
	final int nmoms=mD_Timom.length;
	// formally replace p-> (t.time-currenttime)/(double)s.odata.tau to obtain getTDvMoments() match.
	final QVMDataP o=QVMDataP.calculateQVMDataP(
	    nmoms,
	    3,
	    currenttime,
	    q.odata.tau,
	    java.util.Arrays.stream(tb).map(t->Trade.getDummyTrade((t.time-currenttime)/(double)q.odata.tau,t.shares,Trade.BOOK_UNKNOWN,t.time)).toArray(Trade[]::new),
	    tb.length,
	    null);
	final var access=new QVMDataPDirectAccess();
	final double [] imom=new double [nmoms];access.setM(imom,0,access.M_I,o,nmoms);
	final double [] rTimom=new double [nmoms];access.setM(rTimom,1,access.M_I,o,nmoms);
	TMatr.sumP1(rTimom,imom,o.p_offset);
	UnitTests.checkDoubleArrayRel(rTimom,"rTimom",mD_Timom,"mD_Timom",eps);
    }
    System.err.println(name+":test_IvsVT_CalculateMoments_Extra(): test OK with N points="+tb.length+" neq="+neq+" eps="+eps);
}

@Override java.util.List<IvsVT_CalculateMoments_LegendreShifted> getMomentsToMatch(){
    return java.util.List.of(
	CommonlyUsedMoments_TestFromSample.fromSeqDirectly(IvsVT_CalculateMoments_LegendreShifted::new,neq,currenttime,tau,tb),
	CommonlyUsedMoments_TestFromSample.fromSeqRecurrently(IvsVT_CalculateMoments_LegendreShifted::new,neq,currenttime,tau,tb),
	CommonlyUsedMoments_TestFromSample.fromSeqRecurrentlyTestSecondarySampling(IvsVT_CalculateMoments_LegendreShifted::new,neq,currenttime,tau,tb,eps)
		   );
}


public static void main(String [] args)
{
    new PnLInPsiHstateLegendreShiftedTest(17163L).runTests(2,7);
} // END main
} // END of class PnLInPsiHstateLegendreShiftedTest

}


