package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: PnLInPsiHstateMonomials.java,v 1.56 2022/09/16 21:49:18 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLInPsiHstateMonomials extends PnLInPsiHstate<SaveTwoData<QVMData>> {
    PnLInPsiHstateMonomials(final int neq,final SaveTwoData<QVMData> odata){super(neq,odata);}


public static PnLInPsiHstateMonomials calculate(
    final int neq,
    final long currenttime,
    final long tau,
    final Trade [] tb,
    final int tsize,
    final PnLInPsiHstateMonomials ref)
{

    final int nvm=1,npm=4;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     1+2*(neq-1),
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata.d:null),
	o2=QVMData.calculateQVMData(nvm,
				    1+4*(neq-1),
				    npm,
				    currenttime,
				    tau/2,
				    tb,
				    tsize,
				    ref!=null?ref.odata.d2:null);

    return new PnLInPsiHstateMonomials(neq,new SaveTwoData<QVMData>(o,o2));
}
@Override OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
@Override double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}
@Override double [] getPolynomialWIntegratedDt(final double [] polynomial){return WIntegratorMonomials.getPolynomialWIntegratedDt(polynomial);}

@Override SMomentsData getSMoments(final int nmoms)
{
    // Whether to use sampled or integrated by parts t-moments.
    final boolean flag_use_samped_t_moments=false;
    return SMomentsData.get_MomentsData(nmoms,odata.d,getAccess(),flag_use_samped_t_moments);
}

@Override K2Distance getK2Distance(){return KkQVMMonomials.K2DISTANCE;}

private QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}

double [] getPPHOrig2Moments(int nmoms,double p_eq){
    final var access=getAccess();
    final double [] imom2=new double [nmoms];access.setM(imom2,0,access.M_I,odata.d2,nmoms);
    final double [] pimom2=new double [nmoms];access.setM(pimom2,1,access.M_I,odata.d2,nmoms);
    // to have proper dt/tau  (tau is twice lower for odata.d2),
    // the getPolynomialWIntegratedDt assumes dt/tau integration and scale it, double the result to compensate
    TMatr.scaleVectorInPlace(imom2,2);
    TMatr.scaleVectorInPlace(pimom2,2);
    // shift odata.d2 price offset to be the same one as in odata.d
    final double dp_offset=access.getPOffset(odata.d2)-access.getPOffset(odata.d); // different p_offset
    Polynomials.shiftPOffsetInPlace(nmoms,imom2,pimom2,null,null,-dp_offset);

    // (p-p_eq)*I
    TMatr.sumP1(pimom2,imom2,-p_eq);
    // different basis: tau twice lower
    final double a=1.0/2;
    return TMatr.scaleVectorInPlace(Polynomials.getPolynomialArgumentScaled(pimom2,a),a);
}

@Override double [] getPositionMoments(final int nmoms,final double p_eq,final double [] psiM){
    final int nmomsOrig=nmoms+2*(psiM.length-1);

    return getPositionMomentsFromOrigMoments(nmoms,psiM,getPPHOrig2Moments(nmomsOrig,p_eq));
}

static double [] getPositionMomentsFromOrigMoments(final int nmoms,
						   final double [] psi,
						   final double [] origMoments){

    final double [] pnlmoms=WIntegratorMonomials.B.getPQkMomentsFromQkMoments(
	WIntegratorMonomials.B.mult2Pol(psi,psi),origMoments);

    if(pnlmoms.length!=origMoments.length-2*(psi.length-1)) throw new IllegalArgumentException();
    if(pnlmoms.length!=nmoms) throw new IllegalArgumentException();

    final double [] Smoms=new double [nmoms];
    for(int k=0;k<nmoms;k++){
	final double [] qk=new double [k+1];
	qk[k]=1;
	final double [] intQk=WIntegratorMonomials.getPolynomialWIntegratedDt(qk);
	Smoms[k]=TMatr.mulVVn1(intQk,pnlmoms);
    }
    return Smoms;
}



static class PnLInPsiHstateMonomialsTest extends PnLInPsiHstateTest<SaveTwoData<QVMData>> {
PnLInPsiHstateMonomialsTest(final long initrandom)
{
    super("PnLInPsiHstateMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/24)/2*2;
}



@Override void test_Extra(){
    // These to be identical
    final PnLInPsiHstateMonomials q=getPnLInPsiHstate();
    final PnLInPsiHstateLaguerre s=getPnLInPsiHstateLaguerre();
    matchPnLInPsiHstate(q,s,eps);
    final boolean flagUseSecondarySamplingFordIHdt=false;
    // cross-basis check
    for(var mD:getMomentsToMatch()){
	IntervalSincePsiM.matchFreeMoneyForAll(s.SM,new FreeMoneyForAll(neq,mD,flagUseSecondarySamplingFordIHdt,eps),eps);
	PFuture.match(s.pFV,new FreeMoneyForAll(neq,mD,flagUseSecondarySamplingFordIHdt,eps).pFV,eps);
    }
    System.err.println(name+":PnLInPsiHstateMonomialsTest:test_Extra(): test PnLInPsiHstateLaguerre OK neq="+neq+" eps="+eps);
}


PnLInPsiHstateMonomials getPnLInPsiHstate(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
PnLInPsiHstateLaguerre getPnLInPsiHstateLaguerre(){return PnLInPsiHstateLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}



@Override void test_IvsVT_CalculateMoments_Extra(){
    final int n=neq;
    final PnLInPsiHstateMonomials q=getPnLInPsiHstate();
    final int nmoms=q.odata.d.ntm;
    final SMomentsData mm=q.getSMoments(nmoms);

    for(var mD:getMomentsToMatch()){
	UnitTests.checkDoubleArrayRel(mm.imom,"mm.imom",mD.getdVMoments(),"mD.getdVMoments()",eps);
	UnitTests.checkDoubleArrayRel(mm.tmom,"mm.tmom",mD.getdtMoments(),"mD.getdtMoments()",eps);
	// for t^k*exp(-t/tau) bases this is just a multiplication by t
	final double [] Timom=q.M.B.getXQkMomentsFromQkMoments(mm.imom),
	    mD_Timom=java.util.Arrays.copyOf(mD.getTdVMoments(),Timom.length);
	UnitTests.checkDoubleArrayRel(Timom,"Timom",mD_Timom,"mD_Timom",eps);
	final double [] Tamom=q.M.B.getXQkMomentsFromQkMoments(mm.amom),
	    mD_Tamom=java.util.Arrays.copyOf(mD.getTdAMoments(),Tamom.length);
	UnitTests.checkDoubleArrayRel(Tamom,"Tamom",mD_Tamom,"mD_Tamom",eps);

	// check the moments with half tau:
	final double [] mpnl2=q.getPPHOrig2Moments(q.odata.d2.ntm,q.pi_IH-mm.p_offset),
	    mD_mpnl2=mD.getPPHOrig2Moments(q.pi_IH-mD.getPOffset());
	UnitTests.checkDoubleArrayRel(mpnl2,"mpnl2",mD_mpnl2,"mD_mpnl2",eps);

    }

    System.err.println(name+":test_IvsVT_CalculateMoments_Extra(): test OK with N points="+tb.length+" neq="+neq+" eps="+eps);
}

@Override java.util.List<IvsVT_CalculateMoments_Monomials> getMomentsToMatch(){
    return java.util.List.of(
	CommonlyUsedMoments_TestFromSample.fromSeqDirectly(IvsVT_CalculateMoments_Monomials::new,neq,currenttime,tau,tb),
	CommonlyUsedMoments_TestFromSample.fromSeqRecurrently(IvsVT_CalculateMoments_Monomials::new,neq,currenttime,tau,tb),
	CommonlyUsedMoments_TestFromSample.fromSeqRecurrentlyTestSecondarySampling(IvsVT_CalculateMoments_Monomials::new,neq,currenttime,tau,tb,eps)
			     );
}


public static void main(String [] args)
{
    new PnLInPsiHstateMonomialsTest(143L).runTests(2,4);
} // END main
} // END of class PnLInPsiHstateMonomialsTest

}
