package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** !!! NOT WORKING !!! DO NOT USE !!!
  * Find {@literal \( \left|\psi\right> \) } state such that
  * {@literal \[
  \frac{\left<\psi|(p-\mathtt{pa})^2 I| \psi\right>}{\left<\psi|\psi\right> }
  \xrightarrow[\psi]{\quad }\max
  \] }
  * subject to:
{@literal \[
\left<\psi|(p-\mathtt{pa}) I|\psi\right>=0
\] }
  * The constant
{@literal \( \mathtt{pa}=\left<\psi|p I|\psi\right>/\left<\psi|I|\psi\right> \)}
  * is average price in {@literal \( \left|\psi\right> \)} state.
  * Contrary to pure iteration type approach of {@link DIminP2maxI}
  and Lagrange multipliers technique of {@link PnLMaxState} here we use direct variation
  * as a better converging approach.
  For a given {@literal \( \left|\psi\right> \) }
  calculate {@literal
\begin{align}
\mathtt{I}&=\frac{\left<\psi|I|\psi\right>}{\left<\psi|\psi\right>} \\
\mathtt{pI}&=\frac{\left<\psi|pI|\psi\right>}{\left<\psi|\psi\right>} \\
\mathtt{p^2I}&=\frac{\left<\psi|p^2I|\psi\right>}{\left<\psi|\psi\right>} \\
\mathtt{pa}&=\frac{\left<\psi|pI|\psi\right>}{\left<\psi|I|\psi\right>}
\end{align}
}
Then the {@literal \( P\&L \) }:
{@literal \[
P\&L= \mathtt{p^2I}+\mathtt{pa}*(-2*\mathtt{pI}+\mathtt{pa}*\mathtt{I})
\]
}
Taking into account that Rayleigh quotient variation is {@literal \[
\delta \frac{\left<\psi|A|\psi\right>}{\left<\psi|B|\psi\right>}= 2\left[
\frac{\left<\psi|A|\delta\psi\right>}{\left<\psi|B|\psi\right>}-
\frac{\left<\psi|A|\psi\right>}{\left<\psi|B|\psi\right>}
\frac{\left<\psi|B|\delta\psi\right>}{\left<\psi|B|\psi\right>}\right]
\]} introduce \( var(\cdot) \) operator transforming the constant above to \(n\times n\) matrices: {@literal
\begin{align}
var(\mathtt{I})&=2|I|-2\mathtt{I}|1| \\
var(\mathtt{pI})&=2|pI|-2\mathtt{pI}|1| \\
var(\mathtt{p^2I})&=2|p^2I|-2\mathtt{p^2I}|1| \\
var(\mathtt{pa})&=\frac{2}{\mathtt{I}}|pI|-\frac{2}{\mathtt{I}}\mathtt{pa}|I|
\end{align}
}
Then obtain {@literal \( P\&L \) } operator variation as: {@literal
\begin{align}
\left|{\cal V}\right|&=
var(\mathtt{p^2I})+var(\mathtt{pa})*(-2*\mathtt{pI}+\mathtt{pa}*\mathtt{I})+\mathtt{pa}*(-2*var(\mathtt{pI})+var(\mathtt{pa})*\mathtt{I}+\mathtt{pa}*var(\mathtt{I})) \\
\delta P\&L&= \left<\psi|{\cal V}|\delta\psi\right> =
\end{align}
}
As
The iteration process is then
to solve eigenvalue problem {@literal \[
\left|{\cal PL}|\psi^{[i]}\right>=
\lambda^{[i]} \left|\psi^{[i]}\right>
\] }
and take the state {@literal \( \left|\psi^{[i]}\right> \)}
corresponding to maximal eigenvalue \( \lambda^{[i]} \)
as next iteration {@literal \( \left|\psi\right> \) }.
In case of convergence the maximal eigenvalue \( \lambda^{[i]} \)
is equalt to maximal {@literal \( P\&L \) }.
This algorithm is the most stable
among other tried, such as
{@link DIminP2maxI} and {@link PnLMaxState}.
  * @version $Id: PnLMaxStateDirectVariation.java,v 1.8 2022/08/05 11:25:32 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  */
class PnLMaxStateDirectVariation {
    private static final int N_iterations=10;

    public final double
	p_IH,
	pA_PnLMax,
	PnLMax,I_PnLMax;
    /** In case of good convergence matches {@link #PnLMax} exactly.
     */
    public final double lambda_PnLMax;

    /** The state maximaizing P&amp;L functional.
     */
    public final double [] psi_PnLMax;
    /** The {@literal \( \left|I|\psi\right>=\lambda \left|\psi\right> \)} solution.
     */
    public final EVXData I;

    public PnLMaxStateDirectVariation(
	final int n,
	final double [] QQt,
	final double [] QQi,
	final double [] QQpi,
	final double [] QQp2i,
	final double [] QQp3i,
	final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	final double eps)
{
    // solve generalized eignenvalue problem, find I->max state
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    p_IH=TMatr.getAMatrA(I.psiH,QQpi,n)/I.sH;

    // convert the matrices to |psik> basis to simplify algebra
    final double []
	// bQQt=TMatr.getUnitMatrix(n), no need for right part
	bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(I.neq,QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(I.neq,QQp2i,I.ev)/*,
        bQQp3i=EVXData.convertMatrFromQtoB_Basis(I.neq,QQp3i,I.ev)*/;

    double pa_saved=0,Icur_saved=0,PnLcur_saved=0,lambda_PnLcur_saved=0,psi_saved[]=null;int indPsiInit_saved=-1;
    // temporary arrays
    final double [] mbQQ=new double [n*n];
    for(int indPsiInit=I.neq-1,iTry=0;iTry<Math.min(5/* try this number of eigenvectors */,I.neq);indPsiInit--,iTry++){
	// try several initial values, we use a few |I|psi>=lambda|psi> eigenvectors.
	double [] psi=new double [n];
	psi[indPsiInit]=1; // in |psik> basis
	double pa=Double.NaN,Icur=0,PnLcur=0,lambda_PnLcur=0;
	for(int it=0;it<N_iterations;it++){
	    Icur=TMatr.getAMatrA(psi,bQQi,n);
	    final double pI=TMatr.getAMatrA(psi,bQQpi,n),p2I=TMatr.getAMatrA(psi,bQQp2i,n);
	    // average price
	    pa=pI/Icur;
	    PnLcur=p2I+pa*(-2*pI+pa*Icur);
	    for(int k=0;k<mbQQ.length;k++) {
		final double
		    var_p2I=2*(bQQp2i[k] /* -p2I*bQQt[k]*/), // to right part
		    var_pI=2*(bQQpi[k]/* -pI*bQQt[k]*/), // to right part
		    var_I=2*(bQQi[k]/* -Icur*bQQt[k]*/), // to right part
		    var_pa=2*(bQQpi[k]-pa*bQQi[k])/Icur;
		mbQQ[k]=
		    0.5*(
		    var_p2I+var_pa*(-2*pI+pa*Icur)+pa*(-2*var_pI+var_pa*Icur+pa*var_I)
			 ) ;
	    }

	    if(eps>0){
		UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,TMatr.sumAcBdE(bQQp2i,-2*pa,bQQpi,pa*pa,bQQi),n),"<psi|(p-pa)^2I|psi>",PnLcur,"PnLcur",eps);
		// possible non-monotonic behavior
		// convergence diag
		System.err.println("PnLMaxStateDirectVariation convergence: it="+it+" p_IH="+p_IH+" IH="+I.sH+" pa="+pa+" Icur="+Icur+" PnLcur="+PnLcur+" PnLmaxLambda="+lambda_PnLcur+" indPsiInit="+indPsiInit);
	    }

	    if(it>=N_iterations-1) break;

	    // solve eigenvalue problem
	    final double [] ev_mbQQ=M.EV.getEV(n,mbQQ,true);

	    // iteration: update the |psi> state
	    // select the eigenvector to continue with
	    // continue with selected psi
	    psi=EVXData.getPsiFromBasis(I.neq-1,ev_mbQQ,n);
	    lambda_PnLcur=ev_mbQQ[I.neq-1];
	}

	if(eps>0){
	    // convergence diag
	    System.err.println("PnLMaxStateDirectVariation try with:indPsiInit="+indPsiInit+" p_IH="+p_IH+" IH="+I.sH+" pa="+pa+" Icur="+Icur+" PnLcur="+PnLcur+" PnLmaxLambda="+lambda_PnLcur);
	}
	if(iTry<=0 || (Double.isFinite(PnLcur) && !(PnLcur<=PnLcur_saved))){
	    // save this try
	    pa_saved=pa;
	    Icur_saved=Icur;
	    PnLcur_saved=PnLcur;
	    lambda_PnLcur_saved=lambda_PnLcur;
	    psi_saved=psi;
	    indPsiInit_saved=indPsiInit;
	}
    }
    pA_PnLMax=pa_saved;
    PnLMax=PnLcur_saved;
    lambda_PnLMax=lambda_PnLcur_saved;
    I_PnLMax=Icur_saved;
    // convert found |psi> back to regular basis.
    psi_PnLMax=EVXData.convertPsiFromBtoQ_Basis(n,psi_saved,I.ev);
    // test in original basis
    if(eps>0){
	System.err.println("PnLMaxStateDirectVariation result: with:indPsiInit_saved="+indPsiInit_saved+" PnLMax="+PnLMax+" lambda_PnLMax="+lambda_PnLMax+" pA_PnLMax="+pA_PnLMax+" I_PnLMax="+I_PnLMax);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi_PnLMax,QQt,n),"<psi_PnLMax|psi_PnLMax>",1,"1",eps);
	UnitTests.checkDouble(TMatr.getAMatrA(psi_PnLMax,QQi,n),"<psi_PnLMax|I|psi_PnLMax>",I_PnLMax,"I_PnLMax",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi_PnLMax,QQpi,n)/I_PnLMax,"pI/I",pA_PnLMax,"pA_PnLMax",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi_PnLMax,TMatr.sumAcBdE(QQp2i,-2*pA_PnLMax,QQpi,pA_PnLMax*pA_PnLMax,QQi),n),"PnL",PnLMax,"PnLMax",eps);
    }

}

}
