package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/** !!! NOT WORKING !!! DO NOT USE !!!
 * Try \( mu , lambda \) variation.
 * @version $Id: PnLMaxStateMLVariation.java,v 1.5 2022/08/05 11:25:32 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
class PnLMaxStateMLVariation {
    private static final int N_iterations=10;

    public final double
	p_IH,
	pA_PnLMax,
	PnLMax,I_PnLMax;
    /** In case of good convergence matches {@link #PnLMax} exactly.
     */
    public final double lambda_PnLMax;

    /** The state maximaizing P&amp;L functional.
     */
    public final double [] psi_PnLMax;
    /** The {@literal \( \left|I|\psi\right>=\lambda \left|\psi\right> \)} solution.
     */
    public final EVXData I;

    public PnLMaxStateMLVariation(
	final int n,
	final double [] QQt,
	final double [] QQi,
	final double [] QQpi,
	final double [] QQp2i,
	final double [] QQp3i,
	final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	final double eps)
{
    // solve generalized eignenvalue problem, find I->max state
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    p_IH=TMatr.getAMatrA(I.psiH,QQpi,n)/I.sH;

    // convert the matrices to |psik> basis to simplify algebra
    final double []
	// bQQt=TMatr.getUnitMatrix(n), no need for right part
	bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(I.neq,QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(I.neq,QQp2i,I.ev)/*,
        bQQp3i=EVXData.convertMatrFromQtoB_Basis(I.neq,QQp3i,I.ev)*/;

    double pa_saved=0,Icur_saved=0,PnLcur_saved=0,lambda_PnLcur_saved=0,psi_saved[]=null;int indPsiInit_saved=-1;
    // temporary arrays
    final double [] mbQQ=new double [n*n];
    for(int indPsiInit=I.neq-1,iTry=0;iTry<Math.min(1/* try this number of eigenvectors */,I.neq);indPsiInit--,iTry++){
	// try several initial values, we use a few |I|psi>=lambda|psi> eigenvectors.
	double [] psi=new double [n];
	psi[indPsiInit]=1; // in |psik> basis
	double Icur=TMatr.getAMatrA(psi,bQQi,n),
	    pa=TMatr.getAMatrA(psi,bQQpi,n)/Icur,PnLcur=0,lambda_PnLcur=0;
	for(int it=0;it<N_iterations;it++){
	    // the matrix |(p-pa)^2I|
	    for(int k=0;k<mbQQ.length;k++) mbQQ[k]=(bQQp2i[k]-pa*bQQpi[k])-pa*(bQQpi[k]-pa*bQQi[k]);

	    // solve eigenvalue problem
	    final double [] ev_mbQQ=M.EV.getEV(n,mbQQ,true);

	    // find the eigenvalue providing maximal P&L. If constraint is satisfied -- the max eigenvalue.
	    PnLcur=lambda_PnLcur=Double.NaN;int indLam=-1;
	    for(int s=0;s<n;s++){
		final double [] psiS=EVXData.getPsiFromBasis(s,ev_mbQQ,n);
		final double i=TMatr.getAMatrA(psiS,bQQi,n),
		    pi=TMatr.getAMatrA(psiS,bQQpi,n),
		    p2i=TMatr.getAMatrA(psiS,bQQp2i,n),
		    paS=pi/i,pnlS=(p2i-paS*pi)-paS*(pi-paS*i);
		if(indLam<0 || pnlS>=PnLcur){
		    indLam=s;
		    Icur=i;
		    pa=paS;
		    PnLcur=pnlS;
		    lambda_PnLcur=ev_mbQQ[s];
		    psi=psiS;
		}
	    }
	    if(eps>0){
		UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,TMatr.sumAcBdE(bQQp2i,-2*pa,bQQpi,pa*pa,bQQi),n),"<psi|(p-pa)^2I|psi>",PnLcur,"PnLcur",eps);
		// possible non-monotonic behavior
		// convergence diag
		System.err.println("PnLMaxStateMLVariation convergence: it="+it+" p_IH="+p_IH+" IH="+I.sH+" pa="+pa+" Icur="+Icur+" PnLcur="+PnLcur+" lambda_PnLcur="+lambda_PnLcur+" indPsiInit="+indPsiInit+" indLam="+indLam);
	    }
	}

	if(eps>0){
	    // convergence diag
	    System.err.println("PnLMaxStateMLVariation try with:indPsiInit="+indPsiInit+" p_IH="+p_IH+" IH="+I.sH+" pa="+pa+" Icur="+Icur+" PnLcur="+PnLcur+" lambda_PnLcur="+lambda_PnLcur);
	}
	if(iTry<=0 || (Double.isFinite(PnLcur) && !(PnLcur<=PnLcur_saved))){
	    // save this try
	    pa_saved=pa;
	    Icur_saved=Icur;
	    PnLcur_saved=PnLcur;
	    lambda_PnLcur_saved=lambda_PnLcur;
	    psi_saved=psi;
	    indPsiInit_saved=indPsiInit;
	}
    }
    pA_PnLMax=pa_saved;
    PnLMax=PnLcur_saved;
    lambda_PnLMax=lambda_PnLcur_saved;
    I_PnLMax=Icur_saved;
    // convert found |psi> back to regular basis.
    psi_PnLMax=EVXData.convertPsiFromBtoQ_Basis(n,psi_saved,I.ev);
    // test in original basis
    if(eps>0){
	System.err.println("PnLMaxStateMLVariation result: with:indPsiInit_saved="+indPsiInit_saved+" PnLMax="+PnLMax+" lambda_PnLMax="+lambda_PnLMax+" pA_PnLMax="+pA_PnLMax+" I_PnLMax="+I_PnLMax);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi_PnLMax,QQt,n),"<psi_PnLMax|psi_PnLMax>",1,"1",eps);
	UnitTests.checkDouble(TMatr.getAMatrA(psi_PnLMax,QQi,n),"<psi_PnLMax|I|psi_PnLMax>",I_PnLMax,"I_PnLMax",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi_PnLMax,QQpi,n)/I_PnLMax,"pI/I",pA_PnLMax,"pA_PnLMax",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi_PnLMax,TMatr.sumAcBdE(QQp2i,-2*pA_PnLMax,QQpi,pA_PnLMax*pA_PnLMax,QQi),n),"PnL",PnLMax,"PnLMax",eps);
    }

}

}
