package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.SortArrays;

/** The measure dmu=dv-I_{low}*dt is back.
 *  @version $Id: PnLMu.java,v 1.65 2022/07/28 12:27:18 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class PnLMu {
    final EVXData I;
    final double p_offset,p_IH,p_IL;
    final double [] psiroots,wmu,imu,pwmu,i,pi,tau,d;
    PnLMu(final double p_offset,final EVXData I,final double p_IH,final double p_IL,final double [] psiroots,
	  final double [] wmu,final double [] imu,final double [] pwmu,
	  final double [] i,final double [] pi,final double [] tau,final double [] d,
	  final int iSel, final double p_iSel,final boolean flag_IL)
    {
	this.p_offset=p_offset;
	this.p_IH=p_IH;
	this.p_IL=p_IL;
	this.I=I;
	this.psiroots=psiroots;
	this.wmu=wmu;
	this.imu=imu;
	this.pwmu=pwmu;
	this.i=i;
	this.pi=pi;
	this.tau=tau;
	this.d=d;
	final int nd=psiroots.length;
	if(I.neq-1!=nd||nd!=wmu.length||nd!=imu.length||nd!=pwmu.length||
	   nd!=i.length||nd!=pi.length||nd!=tau.length) throw new IllegalArgumentException();

	int indi=0,indw=0,indv=0;
	for(int k=0;k<nd;k++)
	{
	    if(Math.abs(imu[k])>=Math.abs(imu[indi])) indi=k;
	    if(Math.abs(wmu[k])>=Math.abs(wmu[indw])) indw=k;
	    if(tau[k]*i[k]>=tau[indv]*i[indv]) indv=k; // max volume
	}
	p_wmu=pwmu[indw]/wmu[indw];
	p_imu=pwmu[indi]/wmu[indi];
	p_vm=pi[indv]/i[indv];

	int ind_pnlmin=-1,ind_pnlmax=-1;
	double pnlmin=0,pnlmax=0;
	for(int k=0;k<nd;k++)
	{
	    final double pnl=(pi[k]-p_iSel*i[k])*tau[k];
	    if(k<=0||pnl<=pnlmin){pnlmin=pnl;ind_pnlmin=k;}
	    if(k<=0||pnl>=pnlmax){pnlmax=pnl;ind_pnlmax=k;}
	}
	this.pnlmin=pnlmin;
	this.pnlmax=pnlmax;
	this.x_pnlmin=psiroots[ind_pnlmin];
	this.x_pnlmax=psiroots[ind_pnlmax];

	double mpnlmin=0,mpnlmax=0,mpnlsum=0;
	for(int k=0;k<nd;k++)
	{
	    final double pnl=pwmu[k]-p_iSel*wmu[k];
	    if(k<=0 || pnl<=mpnlmin) mpnlmin=pnl;
	    if(k<=0 || pnl>=mpnlmax) mpnlmax=pnl;
	    mpnlsum+=pnl;
	}
	this.mpnlmin=mpnlmin;
	this.mpnlmax=mpnlmax;
	this.mpnlsum=mpnlsum;
    }
    final double p_wmu,p_imu,p_vm,mpnlmin,mpnlmax,mpnlsum;
    final double pnlmin,pnlmax,x_pnlmin,x_pnlmax;

static final boolean FLAG_PURE_DYNAMIC_IMPACT=false;

/** Pure dynamic impact.
 */
static double [] getMUPI_PURE_DynImpactApproximation(final EVXData I,final double [] QQpi,final int iSel,final double [] QQt)
{
    final int n=I.neq;
    final double [] bmuQQpi_di=new double [n*n];
    // Diagonal matrix in psi_I basis (pure dynamic impact approxiamtion).
    for(int k=0;k<n;k++) bmuQQpi_di[k+n*k]=I.getFPsi(k,QQpi);
    final double plam=bmuQQpi_di[iSel+n*iSel];
    for(int k=0;k<n;k++) bmuQQpi_di[k+n*k]-=plam;

    // Convert to regular basis
    return EVXData.convertMatrFromBtoQ_Basis(n,bmuQQpi_di,I.ev,QQt);
}

/** Non--pure dynamic impact.
 */
static double [] getMUPI_SOME_DynImpactApproximation(final EVXData I,final double [] QQpi,final int iSel,final double [] QQt)
{
    final int n=I.neq;
    final double [] bmuQQpi_di=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev);
    final double plam=bmuQQpi_di[iSel+n*iSel];

    // Set to 0 iSel row/column. This approach may be better than pure dynamic impact.
    // At least one diag element is exactly 0 because of plam substraction.
    for(int k=0;k<n;k++)
    {
	bmuQQpi_di[k+n*k]-=plam;
	bmuQQpi_di[iSel+n*k]=bmuQQpi_di[k+n*iSel]=0;
    }
    // Convert back to regular basis
    return EVXData.convertMatrFromBtoQ_Basis(n,bmuQQpi_di,I.ev,QQt);
}


private static double [][] get_test_getBQQ(final EVXData I,final int iSel,final double [] QQpi)
{
    final int n=I.neq;
    final double lam=I.ev[iSel],plam=I.getFPsi(iSel,QQpi);
    final double [] bmuQQ=new double [n*n],bmuQQp=new double [n*n];
    for(int k=0;k<n;k++)
    {
	bmuQQ[k+n*k]=I.ev[k]-lam;
	if(FLAG_PURE_DYNAMIC_IMPACT)
	{
	    bmuQQp[k+n*k]=I.getFPsi(k,QQpi)-plam;
	}
	else
	{
	    for(int l=0;l<n;l++)
	    {
		if(k!=iSel && l!=iSel)
		{
		    bmuQQp[l+n*k]=I.getFPsiPsi(k,l,QQpi)-(k==l?plam:0);
		}
	    }
	}
    }
    return new double [][]{bmuQQ,bmuQQp};
}

static PnLMu get_PnLMu(final MomentsData mm,
		       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		       final boolean flag_IL,
		       final double eps)
{
    final int n=mm.n,nd=n-1,iSel=flag_IL?0:n-1;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double [] psi=I.getPsi(iSel),psiroots=M.getXNodes(psi).re_root; /* psiroots are sorted. */
    if(psiroots.length!=nd) throw new IllegalArgumentException();
    final double dir;
    if(true)
    { // Order psiroots, time increases from 0 to array last element.
	final double [] ONE=M.B.getONE(),RONE=M.getRadauMultiplied(ONE);
	final boolean flag_dir_right=M.B.getD1(RONE,0)<0;
	if(flag_dir_right)
	{
	    dir=1;
	    SortArrays.sortArrayWithLoad(nd,psiroots,null);
	}
	else
	{
	    dir=-1;
	    SortArrays.sortArrayWithLoadDesc(nd,psiroots,null);
	}
    }

    final double p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH,p_IL=TMatr.getAMatrA(I.psiL,mm.QQpi,n)/I.sL,
	lam=I.ev[iSel],plam=I.getFPsi(iSel,mm.QQpi);
    if(eps>0)
    {
	// check eigenvalue
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,mm.QQi,n),"<psi|I|psi>",lam,"lam_iSel",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,mm.QQpi,n),"<psi|pI|psi>",plam,"plam_iSel",eps);
	// check roots
	for(int k=0;k<nd;k++)
	{
	    UnitTests.checkDoubleRel(M.B.getD0(psi,psiroots[k]),"psi(xk)",0,"0",eps);
	}
	System.err.println("PnLMu.get_PnLMu:roots="+java.util.Arrays.toString(psiroots));
    }

    final double [] muQQp=FLAG_PURE_DYNAMIC_IMPACT?
	getMUPI_PURE_DynImpactApproximation(I,mm.QQpi,iSel,mm.QQt)
	:
	getMUPI_SOME_DynImpactApproximation(I,mm.QQpi,iSel,mm.QQt);


    /* According to my theory arXiv:1510.05510
     * psi(x), corresponding to min or max lambda,
     * have exactly n-1 simple real distinct roots
     * (in polynomial basis psi is a n-1 order polynomial),
     * but not necessary on measure support.
     * The roots of psi_{IL}, corresponding to lowest lambda_{IL},
     * are at the points or large I.
     * There is an alternative theory:
     * consider the measure dmu=dV-lambda_{IL}dt,
     * then consider orthogonal polynomials, generated by dmu measure (only n polynomials exist),
     * and Gauss quadratures built on this measure.
     * Quadrature nodes match exactly to psi_{IL}(x) roots.
     */
    final double [] wmu=new double [nd],imu=new double [nd],pwmu=new double [nd],
	i=new double [nd],pi=new double [nd],tau=new double [nd],d=new double [nd];
    for(int k=0;k<nd;k++)
    {
	final double x=psiroots[k];
	final double [] Rpsi=M.B.sdiv1(psi,x).q;
	final double r2i=TMatr.getAMatrA(Rpsi,mm.QQi,n),r2t=TMatr.getAMatrA(Rpsi,mm.QQt,n),
	    RpsiX=M.B.getD0(Rpsi,x),R2psiX=Math.pow(RpsiX,2);
	i[k]=r2i/r2t;
	pi[k]=TMatr.getAMatrA(Rpsi,mm.QQpi,n)/r2t;
	tau[k]=r2t/R2psiX;
	wmu[k]=(r2i-lam*r2t)/R2psiX;
	imu[k]=i[k]-lam;
	pwmu[k]=TMatr.getAMatrA(Rpsi,muQQp,n)/R2psiX;
	d[k]=dir*TMatr.getAMatrB(Rpsi,psi,mm.QQt,n);

	if(eps>0)
	{
	    final double [] muQQ=TMatr.sumAcB(mm.QQi,-lam,mm.QQt);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,Rpsi,muQQ,n),"<psi|muQQ|Rpsi>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,psi,muQQ,n),"<psi|muQQ|psi>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,Rpsi,muQQp,n),"<psi|muQQp|Rpsi>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,psi,muQQp,n),"<psi|muQQp|psi>",0,"0",eps);
	    UnitTests.checkDoubleRel(dir*TMatr.getAMatrB(psi,Rpsi,mm.QQi,n),"<psi|QQi|lk>",d[k]*lam,"d[k]*lam",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(Rpsi,Rpsi,muQQ,n)/R2psiX,"<Rpsi|muQQ|Rpsi>",wmu[k],"wmu[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(Rpsi,Rpsi,muQQp,n)/R2psiX,"<Rpsi|muQQp|Rpsi>",pwmu[k],"pwmu[k]",eps);
	    UnitTests.checkDoubleRel(wmu[k]/imu[k],"wmu[k]/imu[k]",tau[k],"tau[k]",eps);
	}

	if(eps>0)
	{ // B-basis match.
	    final double [][] rr=get_test_getBQQ(I,iSel,mm.QQpi);
	    final double [] bRpsi=EVXData.convertPsiFromQtoB_Basis(n,java.util.Arrays.copyOf(Rpsi,n),I.ev,mm.QQt),
		bpsi=EVXData.convertPsiFromQtoB_Basis(n,psi,I.ev,mm.QQt),
		bmuQQ=rr[0],bmuQQp=rr[1];

	    UnitTests.checkDoubleRel(TMatr.getAMatrB(bpsi,bRpsi,bmuQQ,n),"<bpsi|bmuQQ|bRpsi>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(bpsi,bpsi,bmuQQ,n),"<bpsi|bmuQQ|bpsi>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(bpsi,bRpsi,bmuQQp,n),"<bpsi|bmuQQp|bRpsi>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(bpsi,bpsi,bmuQQp,n),"<bpsi|bmuQQp|bpsi>",0,"0",eps);
	    UnitTests.checkDoubleRel(dir*TMatr.mulVVV(bpsi,bRpsi,I.getEValues()),"<bpsi|QQi|bRpsi>",d[k]*lam,"d[k]*lam",eps);
	    UnitTests.checkDoubleRel(wmu[k],"wmu[k]",TMatr.getAMatrB(bRpsi,bRpsi,bmuQQ,n)/R2psiX,"<bRpsi|bmuQQ|bRpsi>/R2psiX",eps);
	    UnitTests.checkDoubleRel(pwmu[k],"pwmu[k]",TMatr.getAMatrB(bRpsi,bRpsi,bmuQQp,n)/R2psiX,"<bRpsi|bmuQQp|bRpsi>/R2psiX",eps);
	}
    }

    if(eps>0)
    {
	// Check eigenvalues problem is solved OK.
	EVXData.test_orthogonality("PnLMu.get_PnLMu:I.ev",n,I.ev,mm.QQi,mm.QQt,eps);
	// check averages
	UnitTests.checkDoubleRel(TMatr.sumElements(wmu),"sum(wmu)",mm.QQi[0]-lam*mm.QQt[0],"QQi[0]-lam*QQt[0]",eps);
	//UnitTests.checkDoubleRel(TMatr.sumElements(wmu),"sum(wmu)",muQQ[0],"muQQ[0]",eps);
	// Not holds, quadrature is build on different measure.
	//UnitTests.checkDoubleRel(TMatr.sumElements(pwmu),"sum(pwmu)",muQQp[0],"muQQp[0]",eps);

	System.err.println("PnLMu.get_PnLMu: all internal tests OK, all invariance tests OK n="+n+" FLAG_PURE_DYNAMIC_IMPACT="+FLAG_PURE_DYNAMIC_IMPACT+" flag_IL="+flag_IL+" eps="+eps);
    }

    return new PnLMu(mm.p_offset,I,p_IH,p_IL,psiroots,wmu,imu,pwmu,i,pi,tau,d,iSel,plam/lam,flag_IL);
}

static void matchPnLMu(final PnLMu a,final PnLMu b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_IL,"a.p_IL",b.p_IL+p_offset_shift,"b.p_IL+p_offset_shift",eps);

    UnitTests.checkDoubleArrayRel(a.wmu,"a.wmu",b.wmu,"b.wmu",eps);
    UnitTests.checkDoubleArrayRel(a.imu,"a.imu",b.imu,"b.imu",eps);
    UnitTests.checkDoubleArrayRel(a.pwmu,"a.pwmu",TMatr.sumAcB(b.pwmu,p_offset_shift,b.wmu),"b.pwmu",eps);

    UnitTests.checkDoubleArrayRel(a.i,"a.i",b.i,"b.i",eps);
    UnitTests.checkDoubleArrayRel(a.pi,"a.pi",TMatr.sumAcB(b.pi,p_offset_shift,b.i),"b.pi",eps);
    UnitTests.checkDoubleArrayRel(a.tau,"a.tau",b.tau,"b.tau",eps);
    UnitTests.checkDoubleArrayRel(a.d,"a.d",b.d,"b.d",eps);

    System.err.println("PnLMu.matchPnLMu test OK n="+a.I.neq+" eps="+eps);
}

static void matchPnLMuH(final PnLMu a,final PnLMuH b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_IL,"a.p_IL",b.p_IL+p_offset_shift,"b.p_IL+p_offset_shift",eps);

    UnitTests.checkDoubleArrayRel(a.i,"a.i",b.i,"b.i",eps);
    UnitTests.checkDoubleArrayRel(a.pi,"a.pi",TMatr.sumAcB(b.pi,p_offset_shift,b.i),"b.pi",eps);
    UnitTests.checkDoubleArrayRel(a.tau,"a.tau",b.tau,"b.tau",eps);
    UnitTests.checkDoubleArrayRel(a.d,"a.d",b.d,"b.d",eps);

    System.err.println("PnLMu.matchPnLMu test OK n="+a.I.neq+" eps="+eps);
}

}
