package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.SortArrays;

/** The measure dmu=dv-I_{high}*dt is back.
 *  @version $Id: PnLMuH.java,v 1.15 2020/09/27 20:33:18 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class PnLMuH {
    final EVXData I;
    final double p_offset,p_IH,p_IL;
    final double [] psiroots,i,pi,tau,d,pid;
    PnLMuH(final double p_offset,final EVXData I,final double p_IH,final double p_IL,final double [] psiroots,
	   final double [] i,final double [] pi,final double [] tau,final double [] d,final double [] pid,
	   final int iSel, final double p_iSel)
    {
	this.p_offset=p_offset;
	this.p_IH=p_IH;
	this.p_IL=p_IL;
	this.I=I;
	this.psiroots=psiroots;
	this.i=i;
	this.pi=pi;
	this.tau=tau;
	this.d=d;
	this.pid=pid;
	final int nd=psiroots.length;
	if(I.neq-1!=nd||nd!=i.length||nd!=pi.length||nd!=tau.length||nd!=d.length||nd!=pid.length) throw new IllegalArgumentException();
	for(int k=0;k<nd;k++)
	{
	    final double pnl=(pi[k]-p_iSel*i[k])*d[k]*tau[k],
		pnld=(pid[k]-p_iSel*I.ev[iSel]*d[k])*tau[k];
	}

    }

static PnLMuH get_PnLMuH(final MomentsData mm,
			 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			 final double eps)
{
    final int n=mm.n,nd=n-1,iSel=n-1;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double [] psi=I.getPsi(iSel),psiroots=M.getXNodes(psi).re_root; /* psiroots are sorted. */
    if(psiroots.length!=nd) throw new IllegalArgumentException();
    final double dir;
    if(true)
    { // Order psiroots, time increases from 0 to array last element.
	final double [] ONE=M.B.getONE(),RONE=M.getRadauMultiplied(ONE);
	final boolean flag_dir_right=M.B.getD1(RONE,0)<0;
	if(flag_dir_right)
	{
	    dir=1;
	    SortArrays.sortArrayWithLoad(nd,psiroots,null);
	}
	else
	{
	    dir=-1;
	    SortArrays.sortArrayWithLoadDesc(nd,psiroots,null);
	}
    }

    final double p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH,p_IL=TMatr.getAMatrA(I.psiL,mm.QQpi,n)/I.sL,
	lam=I.ev[iSel],plam=I.getFPsi(iSel,mm.QQpi);
    if(eps>0)
    {
	// check eigenvalue
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,mm.QQi,n),"<psi|I|psi>",lam,"lam_iSel",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,mm.QQpi,n),"<psi|pI|psi>",plam,"plam_iSel",eps);
	// check roots
	for(int k=0;k<nd;k++)
	{
	    UnitTests.checkDoubleRel(M.B.getD0(psi,psiroots[k]),"psi(xk)",0,"0",eps);
	}
	System.err.println("PnLMuH.get_PnLMuH:roots="+java.util.Arrays.toString(psiroots));
    }

    /* According to my theory arXiv:1510.05510
     * psi(x), corresponding to min or max lambda,
     * have exactly n-1 simple real distinct roots
     * (in polynomial basis psi is a n-1 order polynomial),
     * but not necessary on measure support.
     * The roots of psi_{IL}, corresponding to lowest lambda_{IL},
     * are at the points or large I.
     * There is an alternative theory:
     * consider the measure dmu=dV-lambda_{IL}dt,
     * then consider orthogonal polynomials, generated by dmu measure (only n polynomials exist),
     * and Gauss quadratures built on this measure.
     * Quadrature nodes match exactly to psi_{IL}(x) roots.
     */
    final double [] i=new double [nd],pi=new double [nd],tau=new double [nd],d=new double [nd],pid=new double [nd];
    for(int k=0;k<nd;k++)
    {
	final double x=psiroots[k];
	final double [] Rpsi=M.B.sdiv1(psi,x).q;
	final double r2i=TMatr.getAMatrA(Rpsi,mm.QQi,n),r2t=TMatr.getAMatrA(Rpsi,mm.QQt,n),
	    r2pi=TMatr.getAMatrA(Rpsi,mm.QQpi,n),
	    RpsiX=M.B.getD0(Rpsi,x),R2psiX=Math.pow(RpsiX,2);
	i[k]=r2i/r2t;
	pi[k]=TMatr.getAMatrA(Rpsi,mm.QQpi,n)/r2t;
	tau[k]=r2t/R2psiX;
	d[k]=dir*TMatr.getAMatrB(Rpsi,psi,mm.QQt,n);
	pid[k]=dir*TMatr.getAMatrB(Rpsi,psi,mm.QQpi,n);
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(dir*TMatr.getAMatrB(psi,Rpsi,mm.QQi,n)/lam,"<psi|QQi|Rpsi>/lam",d[k],"d[k]",eps);
	}
    }

    if(eps>0)
    {
	// Check eigenvalues problem is solved OK.
	EVXData.test_orthogonality("PnLMuH.get_PnLMuH:I.ev",n,I.ev,mm.QQi,mm.QQt,eps);
	// check averages
	UnitTests.checkDoubleRel(TMatr.mulVV(i,tau)-lam*TMatr.sumElements(tau),"sum(wmu)",mm.QQi[0]-lam*mm.QQt[0],"QQi[0]-lam*QQt[0]",eps);
    }

    return new PnLMuH(mm.p_offset,I,p_IH,p_IL,psiroots,i,pi,tau,d,pid,iSel,plam/lam);
}


}
