package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.SortArrays;

/** A helper class fpr P&L.
 *  @version $Id: PnLMuPoints.java,v 1.11 2020/09/27 20:33:18 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLMuPoints {
    final CMP c;
    final int [] ptr;
    final double [] wL,c_wL,wH,c_wH;

    final double [] Vol,pVol,X;
    final double Vend,Cend,Voltr;
    PnLMuPoints(final CMP c){
	this.c=c;
	this.ptr=SortArrays.sortArrayIndexes(c.d,c);
	final double sumMuwL=TMatr.sumElements(c.ILowMu.wmu),sumMuwH=TMatr.sumElements(c.IHighMu.wmu);
	wL=TMatr.getVectorScaled(c.ILowMu.wmu,1/sumMuwL);
	c_wL=TMatr.getVectorScaled(c.ILowMu.p_wmu,1/sumMuwL);
	wH=TMatr.getVectorScaled(c.IHighMu.wmu,1/sumMuwH);
	c_wH=TMatr.getVectorScaled(c.IHighMu.p_wmu,1/sumMuwH);

	Vol=new double [c.d];pVol=new double [c.d];X=new double [c.d];
	double V=0,C=0,Vtr=0;
	for(int j=0;j<ptr.length;++j)
	{
	    final int ind=ptr[j];
	    double vc=c.getF(ind,wL,wH),cc=c.getF(ind,c_wL,c_wH);
	    if(c.isL(ind))
	    { // Exit point: root of psiL(x). High I, are the "exit points".
		if(vc>V){cc=cc*(V/vc);vc=V;} // not enough liquidity, take what we have to exit.
		vc=-vc;cc=-cc;
	    }
	    else
	    { // Enter point: root of psiH(x). Low I, are the "enter points", take full deficit available.
	    }
	    X[j]=c.getX(ind);
	    V+=vc;C+=cc;
	    Vol[j]+=vc;pVol[j]+=cc;
	}
	// If some position (due to liquidity deficit excess) is left, scan back to remove the remains.
	for(int j=ptr.length;--j>=0 && V>0;)
	{
	    final int ind=ptr[j];
	    double vc=c.getF(ind,wL,wH),cc=c.getF(ind,c_wL,c_wH);
	    if(c.isL(ind))
	    { // Exit point, do nothing
	    }
	    else
	    { // enter point, remove extra deficit
		if(vc>V){cc=cc*(V/vc);vc=V;} // not enough liquidity, take what we have to enter.
		V-=vc;C-=cc;
		Vol[j]-=vc;pVol[j]-=cc;
	    }
	}
	this.Vend=V;
	this.Cend=C;
	double s=0;for(double v:Vol){if(v>0) s+=v;}this.Voltr=s;
	//this.Voltr=java.util.stream.DoubleStream.of(Vol).reduce(0,(sum,v)->sum + (v>0?v:0));

    }


public static class RMu {
    public final double [] x,wmu,p_wmu;
    RMu(final double [] x)
    {
	this.x=x;
	this.wmu=new double[x.length];
	this.p_wmu=new double[x.length];
    }
} // END OF public static class RMu

static class CMP implements com.polytechnik.utils.IntPTRCompare{
    final RMu ILowMu,IHighMu;
    final int o,d;final boolean flag_dir_right;
    CMP(final RMu ILowMu,final RMu IHighMu,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
    {
	this.ILowMu=ILowMu;
	this.IHighMu=IHighMu;
	o=ILowMu.x.length;
	d=ILowMu.x.length+IHighMu.x.length;
	final double [] ONE=M.B.getONE(),RONE=M.getRadauMultiplied(ONE);
	flag_dir_right=M.B.getD1(RONE,0)<0;
    }
    boolean isL(final int k){
	if(k<0||k>=d) throw new IllegalArgumentException("k="+k);
	return k<o?true:false;
    }
    double getF(final int k,final double [] fL,final double [] fH){return isL(k)?fL[k]:fH[k-o];}
    double getX(final int k){return getF(k,ILowMu.x,IHighMu.x);}
    public int compare(int p1,int p2) {
	return Double.compare(getX(p1),getX(p2))*(flag_dir_right?1:-1);
    }
} // END OF static class CMP

} // END OF class PnLMuPoints


