package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;

/**
 *  @version $Id: PnLSensitivity.java,v 1.20 2022/07/13 09:29:51 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLSensitivity {

public final EVXData I;
public final double p_offset,p_IH,p_IL,P0,I_0_feq,
    p_exit,
    pnl_L,I_pnl_L,P_pnl_L,T_pnl_L,V_pnl_L,
    pnl_H,I_pnl_H,P_pnl_H,T_pnl_H,V_pnl_H;
PnLSensitivity(final EVXData I,final double p_offset,
	       final double p_IH,final double p_IL,final double pI_0,final double I_0_feq,
	       final double p_exit,
	       final double pnl_L,final double I_pnl_L,final double P_pnl_L,final double T_pnl_L,final double V_pnl_L,
	       final double pnl_H,final double I_pnl_H,final double P_pnl_H,final double T_pnl_H,final double V_pnl_H,
	       final double eps)
{
    this.I=I;
    this.p_offset=p_offset;
    this.p_IH=p_IH;
    this.p_IL=p_IL;
    this.P0=pI_0/I.s0;
    this.I_0_feq=I_0_feq;

    this.p_exit=p_exit;
    this.pnl_L=pnl_L;
    this.I_pnl_L=I_pnl_L;
    this.P_pnl_L=P_pnl_L;
    this.T_pnl_L=T_pnl_L;
    this.V_pnl_L=V_pnl_L;
    this.pnl_H=pnl_H;
    this.I_pnl_H=I_pnl_H;
    this.P_pnl_H=P_pnl_H;
    this.T_pnl_H=T_pnl_H;
    this.V_pnl_H=V_pnl_H;
}


public static final int PE_PLAST=871,PE_P0=872,PE_PIH=873;
/**
 * Solve <psi|(p-p_last)I|psi>/<psi|psi>=lambda extremum problem, find eigenstates & eigenvalues.
 * to use them as an estimator of P&L min/max.
 */
public static PnLSensitivity get_PnLSensitivity(final MomentsData mm,
						final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
						final int type_of_p_exit,
						final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final int ixH=n-1; // the state with maximal I
    final double I_0=I.s0,I_0_feq=I.ev[ixH];
    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),bQQpnl=bQQpi.clone(),
	bQQT=EVXData.convertMatrFromQtoB_Basis(n,mm.QQT,I.ev),
	bQQV=EVXData.convertMatrFromQtoB_Basis(n,mm.QQV,I.ev),
	bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));
    final double pI0=TMatr.getAMatrA(bpsi0,bQQpi,n),
	p_IH=bQQpi[ixH+n*ixH]/I.ev[ixH],p_IL=bQQpi[0+n*0]/I.ev[0],
	P_last_for_PnL;
    if(type_of_p_exit==PE_PLAST) P_last_for_PnL=mm.p_last;
    else if(type_of_p_exit==PE_P0) P_last_for_PnL=pI0/I.s0;
    else if(type_of_p_exit==PE_PIH) P_last_for_PnL=p_IH;
    else throw new IllegalArgumentException("type_of_p_exit="+type_of_p_exit);
    for(int k=0;k<n;k++) bQQpnl[k+n*k]-=P_last_for_PnL*I.ev[k];

    final double [] ev_bbPnL=M.EV.getEV(n,bQQpnl,true);
    final double pnl_L=ev_bbPnL[0],
	I_pnl_L=TMatr.mulV2V(EVXData.getPsiFromBasis(0,ev_bbPnL,n),I.getEValues()),
	P_pnl_L=TMatr.getAMatrA(n,ev_bbPnL,n+0*n,bQQpi,n)/I_pnl_L,
	T_pnl_L=TMatr.getAMatrA(n,ev_bbPnL,n+0*n,bQQT,n),
	V_pnl_L=TMatr.getAMatrA(n,ev_bbPnL,n+0*n,bQQV,n),
	pnl_H=ev_bbPnL[ixH],
	I_pnl_H=TMatr.mulV2V(EVXData.getPsiFromBasis(ixH,ev_bbPnL,n),I.getEValues()),
	P_pnl_H=TMatr.getAMatrA(n,ev_bbPnL,n+ixH*n,bQQpi,n)/I_pnl_H,
	T_pnl_H=TMatr.getAMatrA(n,ev_bbPnL,n+ixH*n,bQQT,n),
	V_pnl_H=TMatr.getAMatrA(n,ev_bbPnL,n+ixH*n,bQQV,n);
    if(eps>0)
    {
	final double [] QQpnl=TMatr.sumAcB(mm.QQpi,-P_last_for_PnL,mm.QQi);
	final EVXData PnL=EVXData.getEVXDataFromQQ(n,QQpnl,mm.QQt,M);
	UnitTests.checkDoubleArrayRel(java.util.Arrays.copyOf(ev_bbPnL,n),"ev PnL",PnL.getEValues(),"ev PnL",eps);
	if(type_of_p_exit==PE_P0)
	{
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psi0,QQpnl,n),"<psi0|PnL|psi0>",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQpnl,n),"<psi0|PnL|psi0>",0,"0",eps);
	}
	else if(type_of_p_exit==PE_PIH)
	{
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psiH,QQpnl,n),"<psiH|PnL|psiH>",0,"0",eps);
	    UnitTests.checkDoubleRel(bQQpnl[ixH+n*ixH],"<psiH|PnL|psiH>",0,"0",eps);
	}
	UnitTests.checkDoubleRel(pnl_L,"pnl_L",(P_pnl_L-P_last_for_PnL)*I_pnl_L,"(P_pnl_L-P_last_for_PnL)*I_pnl_L",eps);
	UnitTests.checkDoubleRel(pnl_H,"pnl_H",(P_pnl_H-P_last_for_PnL)*I_pnl_H,"(P_pnl_H-P_last_for_PnL)*I_pnl_H",eps);
	System.err.println("PnLSensitivity.get_PnLSensitivity selftest OK type_of_p_exit="+type_of_p_exit+" n="+n+" eps="+eps);
    }

    return new PnLSensitivity(I,mm.p_offset,
			      p_IH,p_IL,pI0,I_0_feq,
			      P_last_for_PnL,
			      pnl_L,I_pnl_L,P_pnl_L,T_pnl_L,V_pnl_L,
			      pnl_H,I_pnl_H,P_pnl_H,T_pnl_H,V_pnl_H,
			      eps);
}


}
