package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
 *  @version $Id: PnLVH.java,v 1.38 2022/07/28 22:30:50 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class PnLVH {
    final EVXData I;
    final double p_offset,p_IH,dpdt_IH,T_IH,V_IH,p_iSel,pV_iSel,V_iSel,
	p_left,pV_left,V_left,p_right,pV_right,V_right,
	p_LR_aver,varpI0,PnL3,pP,PnLv;
    PnLVH(final double p_offset,final int iSel,final EVXData I,
	  final double p_IH,final double dpdt_IH,final double T_IH,final double V_IH,final double p_iSel,final double pV_iSel,final double V_iSel,
	  final double p_left,final double pV_left,final double V_left,final double p_right,final double pV_right,final double V_right,
	  final double varpI0,
	  final double PnL3,final double pP,final double PnLv,
	  final double [] ev_PnL,final double [] I_ev_PnL)
    {
	this.p_offset=p_offset;
	this.p_IH=p_IH;
	this.dpdt_IH=dpdt_IH;
	this.T_IH=T_IH;
	this.V_IH=V_IH;
	this.I=I;
	this.p_iSel=p_iSel;
	this.pV_iSel=pV_iSel;
	this.V_iSel=V_iSel;
	this.p_left=p_left;
	this.pV_left=pV_left;
	this.V_left=V_left;
	this.p_right=p_right;
	this.pV_right=pV_right;
	this.V_right=V_right;
	this.p_LR_aver=(pV_right-pV_left)/(V_right-V_left);
	this.varpI0=varpI0;
	this.PnL3=PnL3;
	this.pP=pP;
	this.PnLv=PnLv;
	this.ev_PnL=ev_PnL;
	this.I_ev_PnL=I_ev_PnL;
	this.PnL_I=ACorr.getACorr_ABW(I.neq,ev_PnL,I_ev_PnL,null);
    }
    final double [] ev_PnL,I_ev_PnL;
    final ACorr PnL_I;

/** Find psi_{IH}, maximizing the <psi_{IH}|I|psi_{IH}>/<psi_{IH}|psi_{IH}>
 * and calculate V_{IH}=<psi_{IH}|V|psi_{IH}>.
 * Then among eigenstates of |V|psi^{[i]}>=lambda^{[i]} |psi^{[i]}>
 * find psi^{left} and psi^{right}, such that
 * lambda^{left} < V_{IH} < lambda^{right}.
 * Then consider price changes (and P&L) for left & right states.
 */
static PnLVH get_PnLVH(final MomentsData mm,
			 final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			 final double eps)
{
    final int n=mm.n,iSel=n-1;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH,
	dpdt_IH=TMatr.getAMatrA(I.psiH,mm.QQdp,n),
	pI_iSel=I.getFPsi(iSel,mm.QQpi),p_iSel=pI_iSel/I.ev[iSel],
	V_iSel=I.getFPsi(iSel,mm.QQV),pV_iSel=I.getFPsi(iSel,mm.QQpV);
    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQT=EVXData.convertMatrFromQtoB_Basis(n,mm.QQT,I.ev),
	bQQV=EVXData.convertMatrFromQtoB_Basis(n,mm.QQV,I.ev),
	bQQpV=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpV,I.ev);
    final double V_IH=bQQV[n-1+n*(n-1)],T_IH=bQQT[n-1+n*(n-1)];
    final double [] Vev=M.EV.getEV(n,bQQV,true);
    int iInd_left=0,iInd_right=1;
    for(int k=1;k<n;k++)
    {
	if(Vev[k-1]<=V_iSel && V_iSel<=Vev[k]) {iInd_left=k-1;iInd_right=k;break;}
    }
    final double p_left=TMatr.getAMatrA(n,Vev,n+iInd_left*n,bQQpi,n)/TMatr.mulV2V(EVXData.getPsiFromBasis(iInd_left,Vev,n),I.getEValues()),
	pV_left=TMatr.getAMatrA(n,Vev,n+iInd_left*n,bQQpV,n),
	V_left=Vev[iInd_left],
	p_right=TMatr.getAMatrA(n,Vev,n+iInd_right*n,bQQpi,n)/TMatr.mulV2V(EVXData.getPsiFromBasis(iInd_right,Vev,n),I.getEValues()),
        pV_right=TMatr.getAMatrA(n,Vev,n+iInd_right*n,bQQpV,n),
	V_right=Vev[iInd_right];
    final double [] bQQpi_PnL=bQQpi.clone();for(int k=0;k<n;k++) bQQpi_PnL[k+n*k]=0;
    final double [] ev_bQQpi_PnL=M.EV.getEV(n,bQQpi_PnL,true),I_ev_bQQpi_PnL=new double [n],
	bpsi0=EVXData.convertPsiFromQtoB_Basis(n,I.psi0,I.ev,mm.QQt);
    for(int k=0;k<n;k++) I_ev_bQQpi_PnL[k]=TMatr.mulV2V(EVXData.getPsiFromBasis(k,ev_bQQpi_PnL,n),I.getEValues());
    final double varpI0=TMatr.getAMatrA(bpsi0,bQQpi_PnL,n),PnL3,pP,PnLv;
    if(true)
    {
	final double [] pp=new double [n];
	for(int k=0;k<n;k++)
	{
	    if(k==iSel) continue;
	    pp[k]=bQQpi[iSel+n*k]/I.ev[iSel];
	}
	final double pPpi=TMatr.getAMatrA(pp,bQQpi,n),pPi=TMatr.mulV2V(pp,I.getEValues());
	PnLv=DeprecatedHelperTMatr.mulVVo(n,pp,0,bQQV,iSel*n);
	PnL3=pPpi-p_iSel*pPi;
	pP=pPpi/pPi;
    }
    if(eps>0)
    {
	System.err.println("PnLVH.ev_bQQpi_PnL="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_bQQpi_PnL,n))+" sum="+TMatr.sumElements(java.util.Arrays.copyOf(ev_bQQpi_PnL,n))+" must be =0 sumI="+TMatr.mulVVn1(I_ev_bQQpi_PnL,ev_bQQpi_PnL));
    }
    return new PnLVH(mm.p_offset,iSel,I,p_IH,dpdt_IH,T_IH,V_IH,p_iSel,pV_iSel,V_iSel,
		     p_left,pV_left,V_left,p_right,pV_right,V_right,
		     varpI0,PnL3,pP,PnLv,
		     java.util.Arrays.copyOf(ev_bQQpi_PnL,n),I_ev_bQQpi_PnL);
}


static PnLVH test_get_PnLVH(final MomentsData mm,
		       final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		       final double eps)
{
    final int n=mm.n,iSel=n-1;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M),
	V=EVXData.getEVXDataFromQQ(n,mm.QQV,mm.QQt,M);
    final double p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH,
	dpdt_IH=TMatr.getAMatrA(I.psiH,mm.QQdp,n),
	T_IH=TMatr.getAMatrA(I.psiH,mm.QQT,n),
	V_IH=TMatr.getAMatrA(I.psiH,mm.QQV,n),
	pI_iSel=I.getFPsi(iSel,mm.QQpi),
	p_iSel=pI_iSel/I.ev[iSel],
	V_iSel=I.getFPsi(iSel,mm.QQV),pV_iSel=I.getFPsi(iSel,mm.QQpV);
    int iInd_left=0,iInd_right=1;
    for(int k=1;k<n;k++)
    {
	if(V.ev[k-1]<=V_iSel && V_iSel<=V.ev[k]) {iInd_left=k-1;iInd_right=k;break;}
    }
    final double p_left=V.getFPsi(iInd_left,mm.QQpi)/V.getFPsi(iInd_left,mm.QQi),
	pV_left=V.getFPsi(iInd_left,mm.QQpV),
	V_left=V.ev[iInd_left],
	p_right=V.getFPsi(iInd_right,mm.QQpi)/V.getFPsi(iInd_right,mm.QQi),
        pV_right=V.getFPsi(iInd_right,mm.QQpV),
	V_right=V.ev[iInd_right];
    final double [] bQQpi_PnL=new double [n*n];
    for(int k=0;k<n;k++)for(int l=0;l<n;l++) if(k!=l) bQQpi_PnL[k+l*n]=I.getFPsiPsi(k,l,mm.QQpi);
    final double [] ev_bQQpi_PnL=M.EV.getEV(n,bQQpi_PnL,true),I_ev_bQQpi_PnL=new double [n];
    double varpI0=0;
    for(int k=0;k<n;k++)
    {
	I_ev_bQQpi_PnL[k]=TMatr.mulV2V(EVXData.getPsiFromBasis(k,ev_bQQpi_PnL,n),I.getEValues());
	final double p0k=I.getFPsiPsi(k,I.psi0,mm.QQt);
	for(int l=0;l<n;l++)
	{
	    if(k!=l) varpI0+=p0k*I.getFPsiPsi(l,I.psi0,mm.QQt)*I.getFPsiPsi(k,l,mm.QQpi);
	}
    }
    final double PnL3,pP,PnLv;
    if(true)
    {
	final double [] QQpiIs=TMatr.sumAcB(mm.QQpi,-p_iSel,mm.QQi);
	final double [] psip=new double [n];
	double sPnLv=0;
	for(int k=0;k<n;k++)
	{
	    psip[k]=I.getFPsiPsi(iSel,k,QQpiIs)/I.ev[iSel];
	    if(k!=iSel) sPnLv+=I.getFPsiPsi(iSel,k,mm.QQpi)*I.getFPsiPsi(k,iSel,mm.QQV);
	}
	PnLv=sPnLv/I.ev[iSel];
	double s=0,sp=0;
	for(int k=0;k<n;k++)
	{
	    for(int l=0;l<n;l++)
	    {
		s+=psip[k]*psip[l]*I.getFPsiPsi(k,l,QQpiIs);
		sp+=psip[k]*psip[l]*I.getFPsiPsi(k,l,mm.QQpi);
	    }
	}
	PnL3=s;
	pP=sp/TMatr.mulV2V(psip,I.getEValues());
	if(eps>0)
	{
	    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,mm.QQt),
		m3=TMatr.mulMatrices(n,new double [][]{QQpiIs,IQQt,QQpiIs,IQQt,QQpiIs});
	    UnitTests.checkDoubleRel(PnL3,"PnL3",I.getFPsi(iSel,m3)/Math.pow(I.ev[iSel],2),"<psiH|m3|psiH>",eps);

	    final double sum=TMatr.sumElements(java.util.Arrays.copyOf(ev_bQQpi_PnL,n)),sumI=TMatr.mulVVn1(I_ev_bQQpi_PnL,ev_bQQpi_PnL);
	    System.err.println("PnLVH.ev_bQQpi_PnL="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_bQQpi_PnL,n))+" sum="+sum+" must be =0 sumI="+sumI);
	    UnitTests.checkDoubleRel(sum,"sum",0,"0",eps);
	    UnitTests.checkDoubleRel(sumI/I.ev[0],"sumI",0,"0",eps);
	    UnitTests.checkDoubleRel(TMatr.sumElements(I_ev_bQQpi_PnL),"sum(I_ev_bQQpi_PnL)",TMatr.sumElements(I.getEValues()),"sum(I.ev)",eps);

	    System.err.println("test_get_PnLVH: S3 test OK for n="+n+" eps="+eps);
	}
    }
    return new PnLVH(mm.p_offset,iSel,I,p_IH,dpdt_IH,T_IH,V_IH,p_iSel,pV_iSel,V_iSel,
		     p_left,pV_left,V_left,p_right,pV_right,V_right,
		     varpI0,PnL3,pP,PnLv,
		     java.util.Arrays.copyOf(ev_bQQpi_PnL,n),I_ev_bQQpi_PnL);
}


static void matchPnLVH(final PnLVH a,final PnLVH b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_iSel,"a.p_iSel",b.p_iSel+p_offset_shift,"b.p_iSel+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_left,"a.p_left",b.p_left+p_offset_shift,"b.p_left+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.V_left,"a.V_left",b.V_left,"b.V_left",eps);
    UnitTests.checkDoubleRel(a.pV_left,"a.pV_left",b.pV_left+p_offset_shift*b.V_left,"b.pV_left+p_offset_shift*b.V_left",eps);
    UnitTests.checkDoubleRel(a.p_right,"a.p_right",b.p_right+p_offset_shift,"b.p_right+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.V_right,"a.V_right",b.V_right,"b.V_right",eps);
    UnitTests.checkDoubleRel(a.pV_right,"a.pV_right",b.pV_right+p_offset_shift*b.V_right,"b.pV_right+p_offset_shift*b.V_right",eps);
    UnitTests.checkDoubleRel(a.PnLv,"a.PnLv",b.PnLv,"b.PnLv",eps);
    UnitTests.checkDoubleRel(a.varpI0,"a.varpI0",b.varpI0,"b.varpI0",eps);
    UnitTests.checkDoubleRel(a.PnL3,"a.PnL3",b.PnL3,"b.PnL3",eps);
    UnitTests.checkDoubleRel(a.pP,"a.pP",b.pP+p_offset_shift,"b.pP+p_offset_shift",eps);
    UnitTests.checkDoubleArrayRel(a.ev_PnL,"a.ev_PnL",b.ev_PnL,"b.ev_PnL",eps);
    UnitTests.checkDoubleArrayRel(a.I_ev_PnL,"a.I_ev_PnL",b.I_ev_PnL,"b.I_ev_PnL",eps);
    UnitTests.checkDoubleRel(a.dpdt_IH,"a.dpdt_IH",b.dpdt_IH,"b.dpdt_IH",eps);

    System.err.println("PnLVH.matchPnLVH test OK n="+a.I.neq+" eps="+eps);
}

static void matchDynVar2DiagProj(final PnLVH a,final DynVar2DiagProj b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH,"b.p_IH",eps);
    UnitTests.checkDoubleRel(a.varpI0,"a.varpI0",b.var_pI_A/b.I.o_o,"b.var_pI_A/b.I.o_o",eps);
    System.err.println("PnLVH.matchDynVar2DiagProj test OK n="+a.I.neq+" eps="+eps);
}
static void matchInIStateIHDensity(final PnLVH a,final InIStateIHDensity b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    final int n=a.I.neq;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.T_IH,"a.T_IH",-b.T_IH,"T_IH",eps);
    UnitTests.checkDoubleRel(a.V_IH,"a.V_IH",-b.V_IH,"V_IH",eps);
    UnitTests.checkDoubleRel(a.dpdt_IH,"a.dpdt_IH",b.dpdt_IH,"dpdt_IH",eps);
    System.err.println("PnLVH.matchInIStateIHDensity test OK n="+a.I.neq+" eps="+eps);
}
}
