package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.ProbabilityCorrelation;
import com.polytechnik.utils.SortArrays;

/**
 * @version $Id: PnLVar3Mu.java,v 1.72 2022/07/28 12:27:18 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLVar3Mu {
    public final EVXData I;
    public final double p_offset,p_IH;
    final double [] wmu,p_wmu,dp_wmu;
    final Skewness [] gSK_P,gSK_I;
    final ProbabilityCorrelation [] pc;

    public final double dir_PCORR,dir_PCORR_PnL,dP_PCORR,dir_gSK_I,dir_gSK_P,dir_gSK_P_PnL,dP_gSK_P;
    PnLVar3Mu(final EVXData I,final double p_offset,final double p_IH,final double p_lam,final Skewness [] gSK_P,final Skewness [] gSK_I,final ProbabilityCorrelation [] pc,final double [] x,final double [] wmu,final double [] p_wmu,final double [] dp_wmu,final double eps)
{
    this.I=I;
    this.p_offset=p_offset;
    this.p_IH=p_IH;
    this.wmu=wmu;
    this.p_wmu=p_wmu;
    this.dp_wmu=dp_wmu;
    this.gSK_P=gSK_P;
    this.gSK_I=gSK_I;
    this.pc=pc;
    double sw=0,sw_pnl=0,swpc=0,swpc_pnl=0,swskP=0,swskP_pnl=0,swskI=0;
    for(int k=0;k<pc.length;k++)
    { // A loop over all roots of psi_{min I}(x), the points of active trading (high I).
	final ProbabilityCorrelation c=pc[k];
	final double w=wmu[k],wpnl=w*(c.pH-c.pL), // weights
	    // Directional information is calculated as dir=<phi|psi_pL>^2-<phi|psi_pH>^2.
	    // We have two choices for phi:
	    // dir_skewness_P corresponds to phi=const
	    dir_skewness_P=gSK_P[k].sk,
	    dir_skewness_I=gSK_I[k].sk,
	    /* "-" because of swapped t and I moments */
	    // dir_probabilitycorr corresponds to phi=(x-xk) (min I),
	    dir_probabilitycorr=-c.corr
	    // Also may try skewness product, (1/i to take care of swapped t and I),
	    // this is probably similar to dir_skewness as I typically has the skewness of the same sign.
	    // ,dir_skewnessP_skewnessI=c.skewnessLike_p*(2*1/c.i_aver-1/c.iH-1/c.iL)/(1/c.iH-1/c.iL)
	    // This try:
	    // ,dir_skewness_probabilitycorr=dir_skewness*dir_probabilitycorr
	    // is no good as p -> -p does not change the sign.
	    ;
	sw+=w;
	sw_pnl+=wpnl;
	swskP+=w*dir_skewness_P;
	swskP_pnl+=wpnl*dir_skewness_P;
	swskI+=w*dir_skewness_I;
	swpc+=w*dir_probabilitycorr;
	swpc_pnl+=wpnl*dir_probabilitycorr;
    }
    dir_PCORR=swpc/sw;
    dir_PCORR_PnL=swpc_pnl/sw_pnl;
    dP_PCORR=swpc_pnl/sw;
    dir_gSK_I=swskI/sw;
    dir_gSK_P=swskP/sw;
    dir_gSK_P_PnL=swskP_pnl/sw_pnl;
    dP_gSK_P=swskP_pnl/sw;
    if(eps>0)
    {
	double swpcpnl=0,swskpnl=0;
	for(int k=0;k<pc.length;k++)
	{
	    final ProbabilityCorrelation c=pc[k];
	    //swskpnl+=wmu[k]*(0.5*(c.pL+c.pH)-c.p_aver);
	    swpcpnl+=wmu[k]*(0.5*(c.pL+c.pH)-p_lam);
	    UnitTests.checkDoubleRel(gSK_I[k].x1,"gSK_I[k].x1",I.sL,"I.sL",eps);
	    // I and t are swapped, iH here correspond to I->min.
	    UnitTests.checkDoubleRel(c.w_pL_iH+c.w_pH_iH,"c.w_pL_iH+c.w_pH_iH",1,"1",eps);
	    UnitTests.checkDoubleRel(1/c.iH,"1/c.iH",I.sL,"I.sL",eps);
	}
	//UnitTests.checkDoubleRel(swskP_pnl,"swskP_pnl",2*swskpnl,"2*swskpnl",eps);
	UnitTests.checkDoubleRel(swpc_pnl,"swpc_pnl",2*swpcpnl,"2*swpcpnl",eps);
    }
}

/**
 * The Mu-measure: dmu=dv-I_{low}*dt is back.
 */
public static PnLVar3Mu get_PnLVar3Mu(final MomentsData mm,
				      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				      final double eps)
{
    final int n=mm.n,d=2;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH;

    // Try Mu-measure
    final double [] psi=I.psiL,x=M.getXNodes(psi).re_root,wmu=new double[x.length],p_wmu=new double[x.length],dp_wmu=new double[x.length];
    final double lam=I.sL,plam=TMatr.getAMatrA(psi,mm.QQpi,n),dplam=TMatr.getAMatrA(psi,mm.QQDpwv,n);
    final ProbabilityCorrelation [] pc=new ProbabilityCorrelation[x.length];
    final Skewness [] gSK_P=new Skewness [x.length], gSK_I=new Skewness [x.length];
    for(int k=0;k<x.length;k++)
    {
	final double xk=x[k];
	final double [] l=M.B.sdiv1(psi,xk).q;
	final double nrmlk=M.B.getD0(l,xk);
	TMatr.scaleVectorInPlace(l,1/nrmlk);
	// The moments in shifted monomials basis: 1, (x-xk), (x-xk)^2.
	final double m0=TMatr.getAMatrA(l,mm.QQt,n),m1=TMatr.getAMatrB(l,psi,mm.QQt,n)/nrmlk,m2=1/Math.pow(nrmlk,2),
	    m0I=TMatr.getAMatrA(l,mm.QQi,n),m1I=lam*m1,m2I=lam*m2,
	    m0pI=TMatr.getAMatrA(l,mm.QQpi,n),m1pI=TMatr.getAMatrB(l,psi,mm.QQpi,n)/nrmlk,m2pI=plam/Math.pow(nrmlk,2),
	    muk=m0I-lam*m0;
	gSK_P[k]=Skewness.getGSkewnessFromMonomialMoments(m0I,m1I,m2I,m0pI,m1pI,m2pI);
	gSK_I[k]=Skewness.getGSkewnessFromMonomialMoments(m0,m1,m2,m0I,m1I,m2I);
	wmu[k]=muk;
	p_wmu[k]=m0pI-plam*m0;
	dp_wmu[k]=TMatr.getAMatrA(l,mm.QQDpwv,n)-dplam*m0;
	final double [] phiB=new double []{xk-I.x0,1},phiC=new double []{1,0},
	  Mt=new double []{m0,m1,m1,m2},Mi=new double []{m0I,m1I,m1I,m2I},Mpi=new double []{m0pI,m1pI,m1pI,m2pI};
	TMatr.scaleVectorInPlace(phiB,1/Math.sqrt(TMatr.getAMatrA(phiB,Mt,d)/m0));
	if(eps>0)
	{
	    if(n==2)
	    {
		Skewness.matchSkewness(Skewness.getGSkewness(mm.QQt,mm.QQi,M),gSK_I[k],eps);
		Skewness.matchSkewness(Skewness.getGSkewness(mm.QQi,mm.QQpi,M),gSK_P[k],eps);
		final PnLSimpleSkewness ss=new PnLSimpleSkewness(mm,M);
		UnitTests.checkDoubleRel(TMatr.getAMatrA(phiB,Mpi,d)/TMatr.getAMatrA(phiB,Mi,d),"p_B",ss.p_B,"ss.p_B",eps);
		Skewness.matchSkewness(ss.gSK_I_average,gSK_I[k],eps);
		Skewness.matchSkewness(ss.gSK_P_average,gSK_P[k],eps);
		System.err.println("PnLVar3Mu: n=2 GSkewness test OK, eps="+eps);
	    }
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(phiB,Mt,d),"<phiB|phiB>",m0,"m0",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(phiC,Mt,d),"<phiB|phiB>",m0,"m0",eps);
	    UnitTests.checkDoubleRel(M.B.getD0(l,xk),"l(xk)",1,"1",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(l,mm.QQt,n),"<l|l>",m0,"m0",eps);
	}
	// Swap i- and t- moments, because we have pI moments handy, the |pI|phi>=lambda_{P}|I|phi>
	// is a way to calculate p-states. For this reason we use 1/I instead of I
	// for generalized eigenproblem |phi>=lambda_{1/I} |I|phi>,
	// so both problems have the same right part: |I|phi>.
	// If one have pt moments, use as arguments these: m0,m1,m2,m0p,m1p,m2p,m0I,m1I,m2I.
	pc[k]=new ProbabilityCorrelation(m0I,m1I,m2I,m0pI,m1pI,m2pI,m0,m1,m2,true);
	if(eps>0)
	{
	    if(n==2)
	    {
		ProbabilityCorrelation.matchProbabilityCorrelation(new ProbabilityCorrelation(mm.QQi,mm.QQpi,mm.QQt,M,true),pc[k],eps);
		System.err.println("PnLVar3Mu: n=2 ProbabilityCorrelation test OK, eps="+eps);
	    }
	    UnitTests.checkDoubleRel(M.B.getD0(psi,xk),"psi(xk)",0,"0",eps);
	    UnitTests.checkDoubleRel(M.B.getD0(l,xk),"l(xk)",1,"1",eps);
	    UnitTests.checkDoubleRel(nrmlk,"nrmlk",M.B.getD1(psi,xk),"M.B.getD1(psi,xk)",eps);
	    UnitTests.checkDoubleRel(gSK_I[k].x1,"gSK_I[k].x1",lam,"lam",eps);
	    UnitTests.checkDoubleRel(gSK_I[k].xa,"gSK_I[k].xa",m0I/m0,"m0I/m0",eps);
	    final double [] lxx=M.B.getaxbP(l,1,-xk);
	    UnitTests.checkDoubleRel(m0,"m0",TMatr.getAMatrA(l,mm.QQt,n),"<l|l>",eps);
	    UnitTests.checkDoubleRel(m1,"m1",TMatr.getAMatrB(l,lxx,mm.QQt,n),"<l|x-xk|l>",eps);
	    UnitTests.checkDoubleRel(m2,"m2",TMatr.getAMatrA(lxx,mm.QQt,n),"<l|(x-xk)^2|l>",eps);
	    UnitTests.checkDoubleRel(m0I,"m0I",TMatr.getAMatrA(l,mm.QQi,n),"<l|I|l>",eps);
	    UnitTests.checkDoubleRel(m1I,"m1I",TMatr.getAMatrB(l,lxx,mm.QQi,n),"<l|(x-xk)I|l>",eps);
	    UnitTests.checkDoubleRel(m2I,"m2I",TMatr.getAMatrA(lxx,mm.QQi,n),"<l|(x-xk)^2I|l>",eps);
	    UnitTests.checkDoubleRel(m0pI,"m0pI",TMatr.getAMatrA(l,mm.QQpi,n),"<l|pI|l>",eps);
	    UnitTests.checkDoubleRel(m1pI,"m1pI",TMatr.getAMatrB(l,lxx,mm.QQpi,n),"<l|(x-xk)pI|l>",eps);
	    UnitTests.checkDoubleRel(m2pI,"m2pI",TMatr.getAMatrA(lxx,mm.QQpi,n),"<l|(x-xk)^2pI|l>",eps);
	    final double []
		ev_MiM=M.EV.getGEV_Generalized(d,Mi,Mt,true),
		phi_IL=EVXData.getPsiFromBasis(0,ev_MiM,d),phi_IH=EVXData.getPsiFromBasis(d-1,ev_MiM,d),
		ev_MpM=M.EV.getGEV_Generalized(d,Mpi,Mi,true),
		phi_pL=EVXData.getPsiFromBasis(0,ev_MpM,d),phi_pH=EVXData.getPsiFromBasis(d-1,ev_MpM,d),
		phi_IC=new double []{1/Math.sqrt(Mi[0])},
	        phi_Il=new double []{0,1/Math.sqrt(Mi[1+d*1])};
	    // Test skewnessLike_p
	    final double skewnessLike_p=Math.pow(TMatr.getAMatrB(phi_pL,phi_IC,Mi,d),2)-Math.pow(TMatr.getAMatrB(phi_pH,phi_IC,Mi,d),2),
		skewnessLike_p_=(2*m0pI/m0I-ev_MpM[0]-ev_MpM[1])/(ev_MpM[0]-ev_MpM[1]),
		pcorr=Math.pow(TMatr.getAMatrB(phi_pL,phi_Il,Mi,d),2)-Math.pow(TMatr.getAMatrB(phi_pH,phi_Il,Mi,d),2),
		pcorr_=(2*plam/lam-ev_MpM[0]-ev_MpM[1])/(ev_MpM[0]-ev_MpM[1]);
	    //UnitTests.checkDoubleRel(skewnessLike_p,"skewnessLike_p",pc[k].skewnessLike_p,"pc[k].skewnessLike_p",eps);
	    //UnitTests.checkDoubleRel(skewnessLike_p_,"skewnessLike_p_",pc[k].skewnessLike_p,"pc[k].skewnessLike_p",eps);
	    // "-" because we swapped "m*I" and "m*" moments.
	    UnitTests.checkDoubleRel(pcorr,"pcorr",-pc[k].corr,"-pc[k].corr",eps);
	    UnitTests.checkDoubleRel(pcorr_,"pcorr_",-pc[k].corr,"-pc[k].corr",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(phi_Il,Mt,d),"<pihi_Il|phi_Il>",1/lam,"1/lam",eps);
	    // Test phi_IL = x-xk (sometimes fails because of numerical instability).
	    UnitTests.checkDoubleRel(phi_IL[0],"phi_IL[0]",0,"0",eps);
	    // Test average & min/max eigenvalues.
	    UnitTests.checkDoubleRel(ev_MiM[0],"ev_MiM[0]",1/pc[k].iH,"1/pc[k].iH",eps);
	    UnitTests.checkDoubleRel(ev_MiM[d-1],"ev_MiM[d-1]",1/pc[k].iL,"1/pc[k].iL",eps);
	    //UnitTests.checkDoubleRel(m0I/m0,"m0I/m0",1/pc[k].i_aver,"1/pc[k].i_aver",eps);
	    UnitTests.checkDoubleRel(ev_MpM[0],"ev_MpM[0]",pc[k].pL,"pc[k].pL",eps);
	    UnitTests.checkDoubleRel(ev_MpM[d-1],"ev_MpM[d-1]",pc[k].pH,"pc[k].pH",eps);

	    System.err.println("PnLVar3Mu: xk="+xk+" IL="+ev_MiM[0]+" IH="+ev_MiM[d-1]+" m0I/m0="+(m0I/m0)+" phi_IL="+java.util.Arrays.toString(phi_IL)+" phi_IH="+java.util.Arrays.toString(phi_IH));
	}
    }

    if(eps>0)
    {
	// Check eigenvalues problem is solved OK.
	EVXData.test_orthogonality("PnLVar3Mu.I.ev",n,I.ev,mm.QQi,mm.QQt,eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,TMatr.sumAcB(mm.QQi,-lam,mm.QQt),n),"<psi|QQmu|psi>",0,"0",eps);
	final double [] sk=new double [gSK_I.length];for(int k=0;k<gSK_I.length;k++){sk[k]=gSK_I[k].sk;}
	System.err.println("PnLVar3Mu:x="+java.util.Arrays.toString(x)+" wmu="+java.util.Arrays.toString(wmu)+" sk="+java.util.Arrays.toString(sk)+" sum(wmu)="+TMatr.sumElements(wmu)+" sum(wmu*sk)="+TMatr.mulVV(wmu,sk)+" n="+n);
	UnitTests.checkDoubleRel(TMatr.sumElements(wmu),"sum(wmu)",mm.QQi[0]-lam*mm.QQt[0],"QQmu[0]",eps);
	// not satisfies
	//UnitTests.checkDoubleRel(TMatr.sumElements(p_wmu),"sum(p_wmu)",mm.QQpi[0]-plam*mm.QQt[0],"QQpmu[0]",eps);


	System.err.println("PnLVar3Mu.get_PnLVar3Mu: test2 internal tests OK, all invariance tests OK n="+n+" eps="+eps);
    }

    return new PnLVar3Mu(I,mm.p_offset,p_IH,plam/lam,gSK_P,gSK_I,pc,x,wmu,p_wmu,dp_wmu,eps);
}

public static void matchPnLVar3Mu(final PnLVar3Mu a,final PnLVar3Mu b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    if(true)
    { // they may be sorted differently
	final int [] ind_a=SortArrays.sortArrayIndexes(a.wmu),
	    ind_b=SortArrays.sortArrayIndexes(b.wmu);
	for(int k=0;k<ind_a.length;k++)
	{
	    UnitTests.checkDoubleRel(a.wmu[ind_a[k]],"a.wmu[ind_a[k]]",b.wmu[ind_b[k]],"b.wmu[ind_b[k]]",eps);
	    UnitTests.checkDoubleRel(a.p_wmu[ind_a[k]],"a.p_wmu[ind_a[k]]",b.p_wmu[ind_b[k]]+p_offset_shift*b.wmu[ind_b[k]],"b.p_wmu[ind_b[k]]+p_offset_shift*wmu",eps);
 	     UnitTests.checkDoubleRel(a.dp_wmu[ind_a[k]],"a.dp_wmu[ind_a[k]]",b.dp_wmu[ind_b[k]],"b.dp_wmu[ind_b[k]]",eps);
	    Skewness.matchSkewness(a.gSK_I[ind_a[k]],b.gSK_I[ind_b[k]],eps);
	    Skewness.matchSkewness(a.gSK_P[ind_a[k]].getSameSkewnessXShifted(a.p_offset),b.gSK_P[ind_b[k]].getSameSkewnessXShifted(b.p_offset),eps);
	    ProbabilityCorrelation.matchProbabilityCorrelationWithOffset(a.pc[ind_a[k]],a.p_offset,b.pc[ind_b[k]],b.p_offset,eps);
	}
    }
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);

    UnitTests.checkDoubleRel(a.dir_PCORR,"a.dir_PCORR",b.dir_PCORR,"b.dir_PCORR",eps);
    UnitTests.checkDoubleRel(a.dir_PCORR_PnL,"a.dir_PCORR_PnL",b.dir_PCORR_PnL,"b.dir_PCORR_PnL",eps);
    UnitTests.checkDoubleRel(a.dP_PCORR,"a.dP_PCORR",b.dP_PCORR,"b.dP_PCORR",eps);
    UnitTests.checkDoubleRel(a.dir_gSK_P,"a.dir_gSK_P",b.dir_gSK_P,"b.dir_gSK_P",eps);
    UnitTests.checkDoubleRel(a.dir_gSK_P_PnL,"a.dir_gSK_P_PnL",b.dir_gSK_P_PnL,"b.dir_gSK_P_PnL",eps);
    UnitTests.checkDoubleRel(a.dP_gSK_P,"a.dP_gSK_P",b.dP_gSK_P,"b.dP_gSK_P",eps);

    System.err.println("PnLVar3Mu.matchPnLVar3Mu test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}


public static void matchMuDynMicroImpact(final PnLVar3Mu a,final MuDynMicroImpact b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    if(true)
    { // they may be sorted differently
	final int [] ind_a=SortArrays.sortArrayIndexes(a.wmu),
	    ind_b=SortArrays.sortArrayIndexes(b.wmu);
	for(int k=0;k<ind_a.length;k++)
	{
	    UnitTests.checkDoubleRel(a.wmu[ind_a[k]],"a.wmu[ind_a[k]]",b.wmu[ind_b[k]],"b.wmu[ind_b[k]]",eps);
	    UnitTests.checkDoubleRel(a.p_wmu[ind_a[k]],"a.p_wmu[ind_a[k]]",b.p_wmu[ind_b[k]]+p_offset_shift*b.wmu[ind_b[k]],"b.p_wmu[ind_b[k]]+p_offset_shift*wmu",eps);
	    UnitTests.checkDoubleRel(a.dp_wmu[ind_a[k]],"a.dp_wmu[ind_a[k]]",b.dp_wmu[ind_b[k]],"b.dp_wmu[ind_b[k]]",eps);
	}
    }
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);

    System.err.println("PnLVar3Mu.MuDynMicroImpact test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}

}

