package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.ProbabilityCorrelation;

/**
 * @version $Id: PnLVar3Mupi.java,v 1.38 2022/08/07 15:44:03 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLVar3Mupi {
    public final EVXData I;
    final double p_offset;
    final double p_IH,vp_IH,v2p_IH,vp_spur,vpI_spur;
    final double [] psik_Dpsik,var2_I,x,w,i,p;
    final Skewness skmu,iskmu;
    //final ProbabilityCorrelation pcorr;
    PnLVar3Mupi(final EVXData I,final double p_offset,final double p_IH,final double [] psik_Dpsik,final double [] var2_I,final Skewness skmu,final Skewness iskmu,final double [] x,final double [] w,final double [] i,final double [] p)
{
    this.I=I;
    this.psik_Dpsik=psik_Dpsik;
    this.var2_I=var2_I;
    this.x=x;
    this.w=w;
    this.i=i;
    this.p=p;
    final int n=I.neq,ixH=n-1;
    this.p_offset=p_offset;
    this.p_IH=p_IH;
    this.vp_IH=psik_Dpsik[ixH];
    this.v2p_IH=var2_I[ixH];
    this.vp_spur=TMatr.sumElements(psik_Dpsik);
    this.vpI_spur=TMatr.mulVVn1(psik_Dpsik,I.ev);
    this.skmu=skmu;
    this.iskmu=iskmu;
    //this.pcorr=new ProbabilityCorrelation(x,w,p,i);
}

/**
 * Find (eigenvector,eigenvalue) pairs (lambda,|psi>)
 * of generalized eigenvalue problem I|psi>=lambda |psi>.
 * (Typically the calculations are performed in price basis)
 * Find the state psi_IH, corresponding to max lambda
 * Then consider variation of various kinds.
 */
public static PnLVar3Mupi get_PnLVar3Mupi(final MomentsData mm,
				      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				      final VolMatrix VOLATILITY_EDPsi,
				      final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double p_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n)/I.sH;

    final double [] var2_I=new double [n],psik_Dpsik=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=I.getPsi(k),Dpsik=VOLATILITY_EDPsi.getDPsi(psik);
	psik_Dpsik[k]=2*TMatr.getAMatrB(psik,Dpsik,mm.QQt,n);
	var2_I[k]=2*(TMatr.getAMatrA(Dpsik,mm.QQi,n)-I.ev[k]*TMatr.getAMatrA(Dpsik,mm.QQt,n));
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(psik_Dpsik[k]*I.ev[k],"psik_Dpsik[k]*I.ev[k]",2*TMatr.getAMatrB(psik,Dpsik,mm.QQi,n),"2*<psik|QQi|Dpsik>",eps);
	}
    }

    if(eps>0)
    {
	// Check eigenvalues problem is solved OK.
	EVXData.test_orthogonality("PnLVar3Mupi.I.ev",n,I.ev,mm.QQi,mm.QQt,eps);
	final double [] DQQi=VOLATILITY_EDPsi.getDiffMatrix(n,mm.QQi),
	    DQQt=VOLATILITY_EDPsi.getDiffMatrix(n,mm.QQt),
	    D2QQi=VOLATILITY_EDPsi.getVolMatrix(n,mm.QQi),
	    D2QQt=VOLATILITY_EDPsi.getVolMatrix(n,mm.QQt),
	    IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,mm.QQt);
	for(int k=0;k<n;k++)
	{
	    UnitTests.checkDoubleRel(psik_Dpsik[k],"psik_Dpsik[k]",I.getFPsi(k,DQQt),"2*<psik|Dpsik>",eps);
	    UnitTests.checkDoubleRel(I.ev[k]*psik_Dpsik[k],"I.ev[k]*psik_Dpsik[k]",I.getFPsi(k,DQQi),"2*<psik|I|Dpsik>",eps);
	    UnitTests.checkDoubleRel(var2_I[k],"var2_I[k]",2*(I.getFPsi(k,D2QQi)-I.ev[k]*I.getFPsi(k,D2QQt)),"var2",eps);
	}
	// Check basis invariance of matrix spur.
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,DQQi),"Spur(IQQt,DQQi)",TMatr.mulVVn1(psik_Dpsik,I.ev),"Spur(psik_Dpsik,I.ev)",eps);
	System.err.println("Spur(IQQt,DQQt)="+TMatr.getSpurMM(n,IQQt,DQQt)+" Spur(psik_Dpsik)="+TMatr.sumElements(psik_Dpsik)+" Spur(psik_Dpsik,I.ev)="+TMatr.mulVVn1(psik_Dpsik,I.ev));
	// The sum(psik_Dpsik) can be considered as constant variation, that is =0?
	// THIS CONDITION HOLDS ONLY FOR SOME (e.g. p) BASES!
	UnitTests.checkDoubleRel(0,"0",TMatr.sumElements(psik_Dpsik),"Spur(psik_Dpsik)",eps);
	System.err.println("PnLVar3Mupi.get_PnLVar3Mupi: all internal tests OK, all invariance tests OK n="+n+" eps="+eps);
    }

    // Try Mu-measure
    final double [] psi=I.psiL,x=M.getXNodes(psi).re_root,w=new double[x.length],i=new double[x.length],p3mu=new double[x.length];
    final double lam=I.sL;
    final double [] QQmu=TMatr.sumAcB(mm.QQi,-lam,mm.QQt);
    for(int k=0;k<x.length;k++)
    {
	final double xk=x[k];
	final double [] l=M.B.sdiv1(psi,xk).q;
	final double lsd1=M.B.getD0(l,xk);
	TMatr.scaleVectorInPlace(l,1/lsd1);
	final double mk=TMatr.getAMatrA(l,QQmu,n),vk=TMatr.getAMatrA(l,mm.QQi,n),tk=TMatr.getAMatrA(l,mm.QQt,n);
	w[k]=mk;
	i[k]=vk/tk-lam;
    }
    final Skewness skmu=Skewness.getSkewnessFromMonomialMoments(
	TMatr.sumElements(w),
	TMatr.mulVV(w,x),
	TMatr.mulVVV(w,x,x),
	TMatr.mulVVV(w,x,TMatr.squareArrayElements(x))),
	iskmu=Skewness.getSkewnessFromMonomialMoments(
	TMatr.sumElements(i),
	TMatr.mulVV(i,x),
	TMatr.mulVVV(i,x,x),
	TMatr.mulVVV(i,x,TMatr.squareArrayElements(x)));
    if(eps>0)
    {
	// Check eigenvalues problem is solved OK.
	EVXData.test_orthogonality("PnLVar3Mupi.I.ev",n,I.ev,mm.QQi,mm.QQt,eps);
	System.err.println("PnLVar3Mupi:x="+java.util.Arrays.toString(x)+" w="+java.util.Arrays.toString(w));
	if(x.length>=2){
	    UnitTests.checkDoubleRel(skmu.w1+skmu.w2,"sum(w)=skmu.w1+skmu.w2",QQmu[0],"QQmu[0]",eps);
	    UnitTests.checkDoubleRel(skmu.xa,"skmu.xa",TMatr.mulVV(w,x)/TMatr.sumElements(w),"aver(x)",eps);
	    UnitTests.checkDoubleRel(skmu.w1*Math.pow(skmu.x1,2)+skmu.w2*Math.pow(skmu.x2,2),"<x2>",TMatr.mulV2V(x,w),"aver(x^2)",eps);
	}
	System.err.println("PnLVar3Mupi.get_PnLVar3Mupi: test2 internal tests OK, all invariance tests OK n="+n+" eps="+eps);
    }


    return new PnLVar3Mupi(I,mm.p_offset,p_IH,psik_Dpsik,var2_I,skmu,iskmu,x,w,i,x);
}


}

