package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;

/**
 * @version $Id: PnLVarMupi.java,v 1.18 2022/06/27 07:19:14 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLVarMupi {
    public final EVXData I;
    final double p_offset;
    final double pt_IH,p_IH,v0,dir0;
    final double [] mu0,mu3;
    PnLVarMupi(final EVXData I,final double p_offset,final double pt_IH,final double p_IH,
	       final double [] x,
	       final double [] mu0,final double [] mu1,final double [] mu2,final double [] mu3,
	       final double eps)
{
    this.I=I;
    this.p_offset=p_offset;
    this.pt_IH=pt_IH;
    this.p_IH=p_IH;
    this.mu0=mu0;
    this.mu3=mu3;
    this.v0=TMatr.sumElements(mu0);
    this.dir0=TMatr.sumElements(mu3);
}

/**
 * One more try for Mu-measure in price basis.
 */
public static PnLVarMupi get_PnLVarMupi(
    final int n,final double p_offset,
    final double [] mom_t,final double [] mom_i,
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
    final double eps)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final double [] QQi=M.B.getQQMatr(n,mom_i),QQt=M.B.getQQMatr(n,mom_t);
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] mom_pi=M.B.getXQkMomentsFromQkMoments(mom_i),
	QQpi=M.B.getQQMatr(n,mom_pi),
	mom_p2i=M.B.getXQkMomentsFromQkMoments(mom_pi),
	mom_p3i=M.B.getXQkMomentsFromQkMoments(mom_p2i),
	mom_pt=M.B.getXQkMomentsFromQkMoments(mom_t),
	QQpt=M.B.getQQMatr(n,mom_pt),
	mom_p2t=M.B.getXQkMomentsFromQkMoments(mom_pt),
	mom_p3t=M.B.getXQkMomentsFromQkMoments(mom_p2t);
    final int nd=n-1;
    final double pt_IH=TMatr.getAMatrA(I.psiH,QQpt,n),p_IH=TMatr.getAMatrA(I.psiH,QQpi,n)/I.sH,lam=I.sL;
    // Try Mu-measure
    final double [] psi=I.psiL,x=M.getXNodes(psi).re_root,
	mu0=new double[x.length],mu1=new double[x.length],mu2=new double[x.length],mu3=new double[x.length],
	QQmu=TMatr.sumAcB(QQi,-lam,QQt),
	QQpmu=M.B.getQQMatr(n,TMatr.sumAcB(mom_pi,-lam,mom_pt)),
	QQp2mu=M.B.getQQMatr(nd,TMatr.sumAcB(mom_p2i,-lam,mom_p2t)),
	QQp3mu=M.B.getQQMatr(nd,TMatr.sumAcB(mom_p3i,-lam,mom_p3t));
    for(int k=0;k<x.length;k++)
    {
	final double xk=x[k];
	final double [] l=M.B.sdiv1(psi,xk).q;
	TMatr.scaleVectorInPlace(l,1/M.B.getD0(l,xk));
	final double m0=TMatr.getAMatrA(l,QQmu,n),
	    m1=TMatr.getAMatrA(l,QQpmu,n),
	    m2=TMatr.getAMatrA(l,QQp2mu,nd),
	    m3=TMatr.getAMatrA(l,QQp3mu,nd);
	mu0[k]=m0;
	mu1[k]=m1-xk*m0;
	mu2[k]=m2+xk*(-2*m1+xk*m0);
	mu3[k]=m3+xk*(-3*m2+xk*(3*m1-xk*m0));
	if(eps>0)
	{
	    // Test the root is calculated OK.
	    UnitTests.checkDoubleRel(M.B.getD0(psi,xk),"psi(xk)",0,"0",eps);
	    UnitTests.checkDoubleRel(M.B.getD0(l,xk),"l(xk)",1,"1",eps);
	    // check mk
	    UnitTests.checkDoubleRel(mu1[k],"mu1[k]",0,"0",eps);
	    UnitTests.checkDoubleRel(mu2[k],"mu2[k]",0,"0",eps);
	    // invariant form for mu3[k]
	    final double dpsixk=M.B.getD1(psi,xk);
	    UnitTests.checkDoubleRel(mu3[k],"mu3[k]",(TMatr.getAMatrA(psi,QQpmu,n)-xk*TMatr.getAMatrA(psi,QQmu,n))/Math.pow(dpsixk,2),"mu3",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,QQmu,n),"<psi|QQmu|psi>",0,"0",eps);
	}
    }

    if(eps>0)
    {
	// Check eigenvalues problem is solved OK.
	EVXData.test_orthogonality("PnLVar3Mu.I.ev",n,I.ev,QQi,QQt,eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,QQmu,n),"<psi|QQmu|psi>",0,"0",eps);
	System.err.println("PnLVar3Mu:x="+java.util.Arrays.toString(x)+" mu0="+java.util.Arrays.toString(mu0));
	UnitTests.checkDoubleRel(TMatr.sumElements(mu0),"sum(mu0)",QQmu[0],"QQmu[0]",eps);
	System.err.println("PnLVar3Mu.get_PnLVar3Mu: test2 internal tests OK, all invariance tests OK n="+n+" eps="+eps);
    }


    return new PnLVarMupi(I,p_offset,pt_IH,p_IH,x,mu0,mu1,mu2,mu3,eps);
}

public static void matchPnLVarMupi(final PnLVarMupi a,final PnLVarMupi b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.v0,"a.v0",b.v0,"b.v0",eps);
    UnitTests.checkDoubleRel(a.dir0,"a.dir0",b.dir0,"b.dir0",eps);
    UnitTests.checkDoubleArrayRel1(a.mu0,"a.mu0",b.mu0,"b.mu0",eps);
    UnitTests.checkDoubleArrayRel1(a.mu3,"a.mu3",b.mu3,"b.mu3",eps);
    System.err.println("PnLVarMupi.matchPnLVarMupi test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}


}
