package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;

/**
  * Last price constraint version.
  * @version $Id: PnLWithConstraintOnPlast.java,v 1.5 2022/08/05 11:25:32 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  */
public class PnLWithConstraintOnPlast {
    private static final int N_iterations=10;

    public final double
	p_IH,
	p_last,
	PnLMax,I_PnLMax;
    /** In case of good convergence matches {@link #PnLMax} exactly.
     */
//    public final double lambda_PnLMax;

    /** Lagrange multiplier (has a dimension of price change).
     */
//    public final double mu_PnLMax;

    /** The state maximaizing P&amp;L functional.
     */
//    public final double [] psi_PnLMax;
    /** The {@literal \( \left|I|\psi\right>=\lambda \left|\psi\right> \)} solution.
     */
    public final EVXData I;

    public PnLWithConstraintOnPlast(
	final int n,
	final double p_last,
	final double [] QQt,
	final double [] QQi,
	final double [] QQpi,
	final double [] QQp2i,
	final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	final double eps)
{
    // solve generalized eignenvalue problem, find I->max state
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    this.p_IH=TMatr.getAMatrA(I.psiH,QQpi,n)/I.sH;
    this.p_last=p_last;
    PnLMax=I_PnLMax=Double.NaN; // FIXME

    // convert the matrices to |psik> basis to simplify algebra
    final double []
	bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(I.neq,QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(I.neq,QQp2i,I.ev),
	bQQc=TMatr.sumAcB(bQQpi,-p_last,bQQi),
	ev_bQQc=M.EV.getEV(n,bQQc,true),
	bQQpnl=TMatr.sumAcB(bQQp2i,-p_last,bQQpi);
    // bQQpnl is now |(p-p_last)^2I| matrix elements:
    TMatr.sumP1(bQQpnl,bQQc,-p_last);

    // Try several initial values. As we have all |I|psi>=lambda|psi> eigenvectors, try them.
    // As we use the |I|psi>=lambda|psi> basis, the eigenvectors are:


}

}
