package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: PnLbI0.java,v 1.57 2022/07/28 12:27:18 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLbI0 {

    public final EVXData I;
    public final double p_offset,p_IH,I_0_eq,I_H_eq,P_eq,Pw_eq,Pe_eq,P_phi0,P1_eq,fact_dI,C_enter,C_exit,P_eq_IHI0,Pf_psi0_dI,Pf_psi0,Pf_psiIH_dI,Pf_psiIH,Pf,Pfdynimp;
    PnLbI0(final EVXData I,
	   final double p_offset,
	   final double p_IH,
	   final double I_0_eq,final double I_H_eq,
	   final double P_eq,final double Pw_eq,final double Pe_eq,final double P_phi0,final double P1_eq,final double fact_dI,
	   final double C_enter,final double C_exit,final double P_eq_IHI0,final double Pf_psi0_dI,final double Pf_psiIH_dI,
	   final double Pf,final double Pfdynimp)
    {
	this.I=I;
	this.p_offset=p_offset;
	this.p_IH=p_IH;
	this.I_0_eq=I_0_eq;
	this.I_H_eq=I_H_eq;
	this.P_eq=P_eq;
	this.Pw_eq=Pw_eq;
	this.Pe_eq=Pe_eq;
	this.P_phi0=P_phi0;
	this.P1_eq=P1_eq;
	this.fact_dI=fact_dI;
	this.C_enter=C_enter;
	this.C_exit=C_exit;
	this.P_eq_IHI0=P_eq_IHI0;
	this.Pf_psi0_dI=Pf_psi0_dI;
	this.Pf_psiIH_dI=Pf_psiIH_dI;
	final double dI=I_0_eq-I.s0;
	this.Pf_psi0=Pf_psi0_dI/dI;
	this.Pf_psiIH=Pf_psiIH_dI/dI;
	this.Pf=Pf;
	this.Pfdynimp=Pfdynimp;
    }


static double geteq_PfdI(final int n,final double [] psiH,final double [] psi0,final double [] varpsi,final double [] bQQpi)
{
    final double [] vpsi=TMatr.sumAcB(varpsi,-TMatr.mulVV(varpsi,psiH),psiH);
    return -TMatr.getAMatrB(psiH,vpsi,bQQpi,n)/(
	TMatr.mulVV(vpsi,psi0)*TMatr.mulVV(psiH,psi0));
}

public static PnLbI0 test_get_PnLbI0(final MomentsData mm,
			      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			      final double eps)
{
    final int n=mm.n,i0=0;
    if(n<2||i0!=0) throw new RuntimeException("n="+n+"<2");
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,mm.QQi,mm.QQt,M),
	bQQi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQi,basis),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,basis);
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double I_0=I.s0,I_0_eq=I.sH,dI=I_0_eq-I_0;
    // Add dI to <bpsi0|bQQi|bpsi0>. A contribution of future, not yet executed, trades.
    bQQi[i0+n*i0]=I_0_eq; // !!! clobbered bQQi. <bpsi0|bQQi|bpsi0> now is incorrect!!!
    final double [] bbasis=M.EV.getEV(n,bQQi,true),bpsiH=EVXData.convertPsiFromQtoB_Basis(n,I.psiH,basis,mm.QQt),bpsi0=new double [n];
    bpsi0[i0]=1;
    final int ixH=n-1; // the state with maximal I
    final double [] bpsifH=EVXData.getPsiFromBasis(ixH,bbasis,n),bphi0=bpsifH.clone();
    bphi0[i0]=0;
    TMatr.scaleVectorInPlace(bphi0,1/Math.sqrt(TMatr.mulVV(bphi0,bphi0)));
    final double C_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n),I_H_eq=bbasis[ixH],
	a_bpsi0=TMatr.mulVV(bpsifH,bpsi0),a_bphi0=TMatr.mulVV(bpsifH,bphi0),
	bpsi0_pI_bphi0=TMatr.getAMatrB(bpsi0,bphi0,bQQpi,n),
	bphi0_pI_bphi0=TMatr.getAMatrA(bphi0,bQQpi,n),
	bpsi0_I_bphi0=TMatr.getAMatrB(bpsi0,bphi0,bQQi,n),
	bphi0_I_bphi0=TMatr.getAMatrA(bphi0,bQQi,n),
	init_bpsi0_I_bpsi0=I_0,
	init_bpsi0_pI_bpsi0=TMatr.getAMatrA(bpsi0,bQQpi,n),
	P_eq_IHI0=(C_IH-init_bpsi0_pI_bpsi0)/dI,
	C_0_eq=bphi0_pI_bphi0+bpsi0_pI_bphi0*(a_bpsi0/a_bphi0-a_bphi0/a_bpsi0),
	dC=C_0_eq-init_bpsi0_pI_bpsi0,
	P_eq=dC/dI,
	denom_Pw_eq=(a_bpsi0*a_bpsi0*init_bpsi0_I_bpsi0+a_bphi0*(2*a_bpsi0*bpsi0_I_bphi0+a_bphi0*bphi0_I_bphi0)),
	nom_Pw_eq=(a_bpsi0*a_bpsi0*init_bpsi0_pI_bpsi0+a_bphi0*(2*a_bpsi0*bpsi0_pI_bphi0+a_bphi0*bphi0_pI_bphi0)),
	Pw_eq=nom_Pw_eq/denom_Pw_eq,
	Pe_eq=(a_bpsi0*a_bpsi0*dI*P_eq+nom_Pw_eq)/bbasis[ixH],
	P1_eq=bpsi0_pI_bphi0/bpsi0_I_bphi0,
	P_phi0=bphi0_pI_bphi0/bphi0_I_bphi0,
	fact_dI=2*a_bpsi0*a_bphi0*bpsi0_I_bphi0/bbasis[ixH],
	bpsifH_pI_bpsifH=TMatr.getAMatrA(bpsifH,bQQpi,n),
	bpsifH_pI_bpsi0=TMatr.getAMatrB(bpsifH,bpsi0,bQQpi,n),
	bpsifH_bpsi0=TMatr.mulVV(bpsifH,bpsi0),
	Pf_psi0_dI=(bpsifH_pI_bpsifH-(bpsifH_pI_bpsi0/bpsifH_bpsi0))/(1-Math.pow(bpsifH_bpsi0,2)),
	bpsifH_pI_bpsiH=TMatr.getAMatrB(bpsifH,bpsiH,bQQpi,n),
	bpsifH_bpsiH=TMatr.mulVV(bpsifH,bpsiH),
	bpsiH_bpsi0=TMatr.mulVV(bpsiH,bpsi0),
	Pf_psiIH_dI=(bpsifH_pI_bpsifH-(bpsifH_pI_bpsiH/bpsifH_bpsiH))/(bpsifH_bpsi0*bpsiH_bpsi0/bpsifH_bpsiH-Math.pow(bpsifH_bpsi0,2)),
	Pf=(C_IH-bpsifH_pI_bpsifH)/(I.sH-(I_H_eq-dI*Math.pow(bpsifH_bpsi0,2)));

        double [] _bQQpi_diag=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev);
	for(int k=0;k<n;k++) for(int l=0;l<n;l++) if(k!=l) _bQQpi_diag[k+n*l]=0;
	_bQQpi_diag=EVXData.convertMatrFromBtoQ_Basis(n,_bQQpi_diag,I.ev,mm.QQt);
	_bQQpi_diag=EVXData.convertMatrFromQtoB_Basis(n,_bQQpi_diag,basis);
	final double Pfdynimp=(C_IH-TMatr.getAMatrA(bpsifH,_bQQpi_diag,n))/(I.sH-(I_H_eq-dI*Math.pow(bpsifH_bpsi0,2)));
    if(eps>0)
    {
	final double [] bQQi_orig=bQQi.clone();
	bQQi_orig[i0+n*i0]=I_0;
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_orig,n),"<bpsi0|I|bpsi0>",I_0,"I_0",eps);
	UnitTests.checkDoubleRel(dI+I_0,"dI+I_0",I_0_eq,"I_0_eq",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psi0,mm.QQpi,n),"<psi0|pI|psi0>",init_bpsi0_pI_bpsi0,"init_bpsi0_pI_bpsi0",eps);// check basis invariance
	UnitTests.checkDoubleRel(Pf_psi0_dI,"Pf_psi0_dI",geteq_PfdI(n,bpsifH,bpsi0,bpsi0,bQQpi),"geteq_PfdI(n,bpsifH,bpsi0,bpsi0,bQQpi)",eps);
	UnitTests.checkDoubleRel(Pf_psiIH_dI,"Pf_psiIH_dI",geteq_PfdI(n,bpsifH,bpsi0,bpsiH,bQQpi),"geteq_PfdI(n,bpsifH,bpsi0,bpsiH,bQQpi)",eps);
	UnitTests.checkDoubleRel(dI,"dI",geteq_PfdI(n,bpsifH,bpsi0,bpsi0,bQQi_orig),"geteq_PfdI(n,bpsifH,bpsi0,bpsi0,bQQi_orig)",eps);

	UnitTests.checkDoubleRel(a_bpsi0*a_bpsi0*dI+denom_Pw_eq,"aver EV[ixH]",bbasis[ixH],"bbasis[ixH]",eps);
	UnitTests.checkDoubleRel(a_bpsi0*a_bpsi0+a_bphi0*a_bphi0,"a_bpsi0*a_bpsi0+a_bphi0*a_bphi0",1,"1",eps);
	UnitTests.checkDoubleRel(Pw_eq,"Pw_eq",TMatr.getAMatrA(bpsifH,bQQpi,n)/(bbasis[ixH]-a_bpsi0*a_bpsi0*dI),"Pw_eq from bpsifH",eps);
	UnitTests.checkDoubleRel(P_eq_IHI0,"P_eq_IHI0",(C_IH-TMatr.getAMatrA(I.psi0,mm.QQpi,n))/(I.sH-I.s0),"P_eq test",eps);
	UnitTests.checkDoubleRel(bbasis[ixH]-a_bpsi0*a_bpsi0*dI,"bbasis[ixH]-a_bpsi0*a_bpsi0*dI",TMatr.getAMatrA(bpsifH,bQQi_orig,n),"<bpsifH|I|bpsifH>",eps);
	System.err.println("PnLbI0: I_0_eq="+I_0_eq+" I_H_eq="+I_H_eq+" a_bpsi0="+a_bpsi0+" a_bphi0="+a_bphi0);
	// Test the (a_bpsi0,a_bphi0) eigenstate of Ix is also
	// an eigenstate if pIx when last price is equal to P_eq.
	final double [] M_pI=new double []{init_bpsi0_pI_bpsi0+dC,bpsi0_pI_bphi0,bpsi0_pI_bphi0,bphi0_pI_bphi0},
	    M_I=new double []{init_bpsi0_I_bpsi0+dI,bpsi0_I_bphi0,bpsi0_I_bphi0,bphi0_I_bphi0},
	    ev=new double []{a_bpsi0,a_bphi0},
	    M_pI_x_ev=TMatr.getMatr_x_A(2,M_pI,ev),M_I_x_ev=TMatr.getMatr_x_A(2,M_I,ev);
	System.err.println("PnLbI0: l0="+(M_pI_x_ev[0]/ev[0])+" l1="+(M_pI_x_ev[1]/ev[1]));
	UnitTests.checkDoubleRel((M_pI_x_ev[0]/ev[0]),"(M_pI_x_ev[0]/ev[0])",Pe_eq*bbasis[ixH],"Pe_eq*bbasis[ixH]",eps);
	UnitTests.checkDoubleRel((M_pI_x_ev[1]/ev[1]),"(M_pI_x_ev[1]/ev[1])",Pe_eq*bbasis[ixH],"Pe_eq*bbasis[ixH]",eps);
	UnitTests.checkDoubleRel((M_I_x_ev[0]/ev[0]),"(M_I_x_ev[0]/ev[0])",bbasis[ixH],"bbasis[ixH]",eps);
	UnitTests.checkDoubleRel((M_I_x_ev[1]/ev[1]),"(M_I_x_ev[1]/ev[1])",bbasis[ixH],"bbasis[ixH]",eps);
	//if(ixH!=(n-1)) throw new RuntimeException();
	System.err.println("PnLbI0.test_get_PnLbI0 selftest OK n="+n+" eps="+eps);
    }

    return new PnLbI0(I,mm.p_offset,C_IH/I.sH,
		      I_0_eq,I_H_eq,P_eq,Pw_eq,Pe_eq,P_phi0,P1_eq,fact_dI,
		      C_IH,C_0_eq,P_eq_IHI0,Pf_psi0_dI,Pf_psiIH_dI,Pf,Pfdynimp);

}


/**
 * Solve {@literal <psi|I|psi>/<psi|psi>}=lambda extremum problem, find eigenstates &amp; eigenvalues.
 * find psi^{IH} and lambda^{IH}=max lambda, used as an estimator of
 * the level of "future" execution.
 * Then dI=lambda^{IH}-{@literal <psi0|I|psi0>} (contribution of future, not yet executed, trades).
 * Add to I the {@literal |psi0><psi0|} operator in a way {@literal <psi0|Ix|psi0>} to be equal lambda^{IH}.
 * Then Ix (future I) is: Ix= I + {@literal |psi0>dI<psi0|}
 * Find the eigenstates of this new Ix matrix, the {@literal |psifH>} corresponding to I_H_eq=max Ix.
 * Expand  {@literal |psifH>}=a_psi0 {@literal |psi0>} + b_phi0 {@literal |phi0>}, where {@literal |phi0>}=({@literal |psifH>}-{@literal <psi0|psifH>|psi0>})/norm,
 * note that {@literal |phi0>} is "the past" (because phi0(x0)=0) and {@literal |psi0>} is "now and in the future".
 * We now have 2x2 already solved eigenproblem for the matrix
 * [{@literal <psi0|Ix|psi0>} , {@literal <psi0|Ix|phi0>}]
 * [{@literal <phi0|Ix|psi0>} , {@literal <phi0|Ix|phi0>}]
 * The eigenvalue of interest is I_H_eq &gt; lambda^{IH}, the eigenvector is (a_psi0,b_phi0).
 * Find the P_eq to require the pIx matrix
 * To have exactly the same eigenvector (a_psi0,b_phi0).
 * Because phi0(x0)=0, future P_eq enter only to {@literal <psi0|pIx|psi0>}=P_eq*dI+{@literal <psi0|pI|psi0>},
 * and matrix elements with {@literal |phi0>} are identical for pIx and pI.
 * [P_eq*dI+{@literal <psi0|pI|psi0>} , {@literal <psi0|pI|phi0>}]
 * [{@literal <phi0|pI|psi0>}         , {@literal <phi0|pI|phi0>}]
 * This gives:
 * P_eq=({@literal <phi0|pI|phi0>}+{@literal <psi0|pI|phi0>}*(a_psi0/a_phi0-a_phi0/a_psi0)-{@literal <psi0|pI|psi0>})/dI
 * This P_eq to be considered as the equilibrium future price.
 * See "Market Dynamics. On Supply and Demand Concepts, Feb 14, 2016, arXiv:1602.04423",
 * where I guess (based on experimental observations) that "pI" and "I"
 * have an extremum on the same |psi> state (in dynamic impact approximation).
 *
 * There is an alternative approach
 * to consider {@literal <psifH|pIx|psifH>}=Pw_eq{@literal <psifH|Ix|psifH>},
 * but this alternative is very much a "glorified average",
 * similar to p_IH of "Mathematical Foundations of Realtime Equity Trading.
 * Liquidity Deficit and Market Dynamics. Automated Trading Machines,
 * Oct 19, 2015, arXiv:1510.05510", where a price, corresponding to
 * the psi^{IH} (calculated on "past" trades) was considered.
 * One can also try the {@literal <psifH|pIx|psifH>}=Pe_eq{@literal <psifH|Ix|psifH>},
 * calculating matrix element {@literal <psi0|pIx|psi0>} with P_eq of above.
 */
public static PnLbI0 get_PnLbI0(final MomentsData mm,
				final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double I_0=I.s0,I_0_eq=I.sH,dI=I_0_eq-I_0;
    final double [] bQQi_eq=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o));
    // Add dI to <bpsi0|bQQi|bpsi0>. A contribution of future, not yet executed, trades.
    for(int k=0;k<n;k++) bQQi_eq[k+n*k]+=I.ev[k];
    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev);
    final double [] bbasis=M.EV.getEV(n,bQQi_eq,true),bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o)),bpsiH=new double [n];
    final int ixH=n-1; // the state with maximal I
    bpsiH[ixH]=1;
    final double [] bpsifH=EVXData.getPsiFromBasis(ixH,bbasis,n),bphi0=bpsifH.clone(),C=new double [n];
    TMatr.sumP1(bphi0,bpsi0,-TMatr.mulVV(bphi0,bpsi0));
    TMatr.scaleVectorInPlace(bphi0,1/Math.sqrt(TMatr.mulVV(bphi0,bphi0)));
    for(int k=0;k<n;k++){C[k]=bQQpi[k+n*k];}
    final double C_IH=TMatr.getAMatrA(I.psiH,mm.QQpi,n),I_H_eq=bbasis[ixH],
	a_bpsi0=TMatr.mulVV(bpsifH,bpsi0),a_bphi0=TMatr.mulVV(bpsifH,bphi0),
	bpsi0_pI_bphi0=TMatr.getAMatrB(bpsi0,bphi0,bQQpi,n),
	bphi0_pI_bphi0=TMatr.getAMatrA(bphi0,bQQpi,n),
	bpsi0_I_bphi0=TMatr.mulVVV(bpsi0,bphi0,I.getEValues()),
	bphi0_I_bphi0=TMatr.mulV2V(bphi0,I.getEValues()),
	init_bpsi0_I_bpsi0=I_0,
	init_bpsi0_pI_bpsi0=TMatr.getAMatrA(bpsi0,bQQpi,n),
	P_eq_IHI0=(C_IH-init_bpsi0_pI_bpsi0)/dI,
	C_0_eq=bphi0_pI_bphi0+bpsi0_pI_bphi0*(a_bpsi0/a_bphi0-a_bphi0/a_bpsi0),
	dC=C_0_eq-init_bpsi0_pI_bpsi0,
	P_eq=dC/dI,
	denom_Pw_eq=(a_bpsi0*a_bpsi0*init_bpsi0_I_bpsi0+a_bphi0*(2*a_bpsi0*bpsi0_I_bphi0+a_bphi0*bphi0_I_bphi0)),
	nom_Pw_eq=(a_bpsi0*a_bpsi0*init_bpsi0_pI_bpsi0+a_bphi0*(2*a_bpsi0*bpsi0_pI_bphi0+a_bphi0*bphi0_pI_bphi0)),
	Pw_eq=nom_Pw_eq/denom_Pw_eq,
	Pe_eq=(a_bpsi0*a_bpsi0*dI*P_eq+nom_Pw_eq)/bbasis[ixH],
	P1_eq=bpsi0_pI_bphi0/bpsi0_I_bphi0,
	P_phi0=bphi0_pI_bphi0/bphi0_I_bphi0,
	fact_dI=2*a_bpsi0*a_bphi0*bpsi0_I_bphi0/bbasis[ixH],
	bpsifH_pI_bpsifH=TMatr.getAMatrA(bpsifH,bQQpi,n),
	bpsifH_pI_bpsi0=TMatr.getAMatrB(bpsifH,bpsi0,bQQpi,n),
	bpsifH_bpsi0=TMatr.mulVV(bpsifH,bpsi0),
	Pf_psi0_dI=(bpsifH_pI_bpsifH-(bpsifH_pI_bpsi0/bpsifH_bpsi0))/(1-Math.pow(bpsifH_bpsi0,2)),
	bpsifH_pI_bpsiH=TMatr.getAMatrB(bpsifH,bpsiH,bQQpi,n),
	bpsifH_bpsiH=TMatr.mulVV(bpsifH,bpsiH),
	bpsiH_bpsi0=TMatr.mulVV(bpsiH,bpsi0),
	Pf_psiIH_dI=(bpsifH_pI_bpsifH-(bpsifH_pI_bpsiH/bpsifH_bpsiH))/(bpsifH_bpsi0*bpsiH_bpsi0/bpsifH_bpsiH-Math.pow(bpsifH_bpsi0,2)),
	Pf=(C_IH-bpsifH_pI_bpsifH)/(I.sH-TMatr.mulV2V(bpsifH,I.getEValues())),
	Pfdynimp=(C_IH-TMatr.mulV2V(bpsifH,java.util.Arrays.copyOf(C,n)))/(I.sH-TMatr.mulV2V(bpsifH,I.getEValues()));
    if(eps>0)
    {
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_eq,n),"<bpsi0|I_eq|bpsi0>",I_0_eq,"I_0_eq",eps);
	UnitTests.checkDoubleRel(dI+I_0,"dI+I_0",I_0_eq,"I_0_eq",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(I.psi0,mm.QQpi,n),"<psi0|pI|psi0>",init_bpsi0_pI_bpsi0,"init_bpsi0_pI_bpsi0",eps);// check basis invariance
	UnitTests.checkDoubleRel(Pf_psi0_dI,"Pf_psi0_dI",geteq_PfdI(n,bpsifH,bpsi0,bpsi0,bQQpi),"geteq_PfdI(n,bpsifH,bpsi0,bpsi0,bQQpi)",eps);
	UnitTests.checkDoubleRel(Pf_psiIH_dI,"Pf_psiIH_dI",geteq_PfdI(n,bpsifH,bpsi0,bpsiH,bQQpi),"geteq_PfdI(n,bpsifH,bpsi0,bpsiH,bQQpi)",eps);

	UnitTests.checkDoubleRel(a_bpsi0*a_bpsi0*dI+denom_Pw_eq,"aver EV[ixH]",bbasis[ixH],"bbasis[ixH]",eps);
	UnitTests.checkDoubleRel(a_bpsi0*a_bpsi0+a_bphi0*a_bphi0,"a_bpsi0*a_bpsi0+a_bphi0*a_bphi0",1,"1",eps);
	UnitTests.checkDoubleRel(Pw_eq,"Pw_eq",TMatr.getAMatrA(bpsifH,bQQpi,n)/(bbasis[ixH]-a_bpsi0*a_bpsi0*dI),"Pw_eq from bpsifH",eps);
	UnitTests.checkDoubleRel(bbasis[ixH]-a_bpsi0*a_bpsi0*dI,"bbasis[ixH]-a_bpsi0*a_bpsi0*dI",TMatr.mulV2V(bpsifH,I.getEValues()),"<bpsifH|I|bpsifH>",eps);
	System.err.println("PnLbI0: I_0_eq="+I_0_eq+" I_H_eq="+I_H_eq+" a_bpsi0="+a_bpsi0+" a_bphi0="+a_bphi0);
	// Test the (a_bpsi0,a_bphi0) eigenstate of Ix is also
	// an eigenstate if pIx when last price is equal to P_eq.
	final double [] M_pI=new double []{init_bpsi0_pI_bpsi0+dC,bpsi0_pI_bphi0,bpsi0_pI_bphi0,bphi0_pI_bphi0},
	    M_I=new double []{init_bpsi0_I_bpsi0+dI,bpsi0_I_bphi0,bpsi0_I_bphi0,bphi0_I_bphi0},
	    ev=new double []{a_bpsi0,a_bphi0},
	    M_pI_x_ev=TMatr.getMatr_x_A(2,M_pI,ev),M_I_x_ev=TMatr.getMatr_x_A(2,M_I,ev);
	System.err.println("PnLbI0: l0="+(M_pI_x_ev[0]/ev[0])+" l1="+(M_pI_x_ev[1]/ev[1]));
	UnitTests.checkDoubleRel((M_pI_x_ev[0]/ev[0]),"(M_pI_x_ev[0]/ev[0])",Pe_eq*bbasis[ixH],"Pe_eq*bbasis[ixH]",eps);
	UnitTests.checkDoubleRel((M_pI_x_ev[1]/ev[1]),"(M_pI_x_ev[1]/ev[1])",Pe_eq*bbasis[ixH],"Pe_eq*bbasis[ixH]",eps);
	UnitTests.checkDoubleRel((M_I_x_ev[0]/ev[0]),"(M_I_x_ev[0]/ev[0])",bbasis[ixH],"bbasis[ixH]",eps);
	UnitTests.checkDoubleRel((M_I_x_ev[1]/ev[1]),"(M_I_x_ev[1]/ev[1])",bbasis[ixH],"bbasis[ixH]",eps);
	//if(ixH!=(n-1)) throw new RuntimeException();
	System.err.println("PnLbI0.get_PnLbI0 selftest OK n="+n+" eps="+eps);
    }

    return new PnLbI0(I,mm.p_offset,C_IH/I.sH,
		      I_0_eq,I_H_eq,P_eq,Pw_eq,Pe_eq,P_phi0,P1_eq,fact_dI,
		      C_IH,C_0_eq,P_eq_IHI0,Pf_psi0_dI,Pf_psiIH_dI,Pf,Pfdynimp);

}

public static void matchPnLbI0(final PnLbI0 a,final PnLbI0 b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P_eq_IHI0,"a.P_eq_IHI0",b.P_eq_IHI0+p_offset_shift,"b.P_eq_IHI0+p_offset_shift",eps);
    if(a.I.neq>2) UnitTests.checkDoubleRel(a.Pf,"a.Pf",b.Pf+p_offset_shift,"b.Pf+p_offset_shift",eps);
    if(a.I.neq>2) UnitTests.checkDoubleRel(a.Pfdynimp,"a.Pfdynimp",b.Pfdynimp+p_offset_shift,"b.Pfdynimp+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.Pf_psi0,"a.Pf_psi0",b.Pf_psi0+p_offset_shift,"b.Pf_psi0+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.Pf_psiIH,"a.Pf_psiIH",b.Pf_psiIH+p_offset_shift,"b.Pf_psiIH+p_offset_shift",eps);

    UnitTests.checkDoubleRel(a.I_0_eq,"a.I_0_eq",b.I_0_eq,"b.I_0_eq",eps);
    UnitTests.checkDoubleRel(a.I_H_eq,"a.I_H_eq",b.I_H_eq,"b.I_H_eq",eps);
    UnitTests.checkDoubleRel(a.fact_dI,"a.fact_dI",b.fact_dI,"b.fact_dI",eps);

    UnitTests.checkDoubleRel(a.P_eq,"a.P_eq",b.P_eq+p_offset_shift,"b.P_eq+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.Pw_eq,"a.Pw_eq",b.Pw_eq+p_offset_shift,"b.Pw_eq+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.Pe_eq,"a.Pe_eq",b.Pe_eq+p_offset_shift,"b.Pe_eq+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P_phi0,"a.P_phi0",b.P_phi0+p_offset_shift,"b.P_phi0+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P1_eq,"a.P1_eq",b.P1_eq+p_offset_shift,"b.P1_eq+p_offset_shift",eps);

    UnitTests.checkDoubleRel(a.C_enter,"a.C_enter",b.C_enter+p_offset_shift*b.I_0_eq,"b.C_enter+p_offset_shift*b.I_0_eq",eps);
    UnitTests.checkDoubleRel(a.C_exit,"a.C_exit",b.C_exit+p_offset_shift*b.I_0_eq,"b.C_exit+p_offset_shift*b.I_0_eq",eps);
    UnitTests.checkDoubleRel(a.Pf_psi0_dI,"a.Pf_psi0_dI",b.Pf_psi0_dI+p_offset_shift*(b.I_0_eq-b.I.s0),"b.Pf_psi0_dI+p_offset_shift*(b.I_0_eq-b.I.s0)",eps);
    UnitTests.checkDoubleRel(a.Pf_psiIH_dI,"a.Pf_psiIH_dI",b.Pf_psiIH_dI+p_offset_shift*(b.I_0_eq-b.I.s0),"b.Pf_psiIH_dI+p_offset_shift*(b.I_0_eq-b.I.s0)",eps);
    System.err.println("PnLbI0.matchPnLbI0 test OK n="+a.I.neq+" eps="+eps);
}

public static void matchPnLfIHI0(final PnLbI0 a,final PnLfIHI0 b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    UnitTests.checkDoubleRel(a.I_0_eq,"a.I_0_eq",b.I_0_feq,"b.I_0_feq",eps);
    UnitTests.checkDoubleRel(a.I_H_eq,"a.I_H_eq",b.I_H_feq,"b.I_H_feq",eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    if(a.I.neq>2) UnitTests.checkDoubleRel(a.Pfdynimp,"a.Pfdynimp",b.Pfdynimp+p_offset_shift,"b.Pfdynimp+p_offset_shift",eps);
    if(a.I.neq>2) UnitTests.checkDoubleRel(a.Pf,"a.Pf",b.Pf+p_offset_shift,"b.Pf+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.Pf_psi0,"a.Pf_psi0",b.Pf_psi0+p_offset_shift,"b.Pf_psi0+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.Pf_psiIH,"a.Pf_psiIH",b.Pf_psiIH+p_offset_shift,"b.Pf_psiIH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.Pf_psi0_dI,"a.Pf_psi0_dI",b.Pf_psi0_dI+p_offset_shift*(b.I_0_feq-b.I.s0),"b.Pf_psi0_dI+p_offset_shift*(b.I_0_feq-b.I.s0)",eps);
    UnitTests.checkDoubleRel(a.Pf_psiIH_dI,"a.Pf_psiIH_dI",b.Pf_psiIH_dI+p_offset_shift*(b.I_0_feq-b.I.s0),"b.Pf_psiIH_dI+p_offset_shift*(b.I_0_feq-b.I.s0)",eps);
    System.err.println("PnLbI0.matchPnLfIHI0 test OK n="+a.I.neq+" eps="+eps);
}


}
