package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.EVSolver;
import com.polytechnik.utils.ProbabilityCorrelation;

/**
 *  @version $Id: PnLdIE.java,v 1.31 2022/08/05 11:25:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdIE {

public static class CorrPI extends ProbabilityCorrelation {
    public final double Pt,Pi;
    CorrPI(final double [] mi,final double [] mt,final EVSolver EV,final double eps)
    {
	super(mt[0],mt[1],mt[2],
	      mt[1],mt[2],mt[3],
	      mi[0],mi[1],mi[2],true);
	this.Pi=mi[1]/mi[0];
	this.Pt=mt[1]/mt[0];
	if(eps>0) test_Self(mi,mt,EV,eps);
    }

    private void test_Self(final double [] mi,final double [] mt,final EVSolver EV,final double eps)
    {
	final int d=2;
	final double [] MI=new double []{mi[0],mi[1],mi[1],mi[2]},
	    MT=new double []{mt[0],mt[1],mt[1],mt[2]},
	    pMT=new double []{mt[1],mt[2],mt[2],mt[3]},
	    ev_MIMT=EV.getGEV_Generalized(d,MI,MT,true),
	    ev_pMTMT=EV.getGEV_Generalized(d,pMT,MT,true);
        UnitTests.checkDoubleRel(pL,"pL",ev_pMTMT[0],"ev_pMTMT[0]",eps);
        UnitTests.checkDoubleRel(pH,"pH",ev_pMTMT[1],"ev_pMTMT[1]",eps);
	UnitTests.checkDoubleRel(iL,"iL",ev_MIMT[0],"ev_MIMT[0]",eps);
	UnitTests.checkDoubleRel(iH,"iH",ev_MIMT[1],"ev_MIMT[1]",eps);
	EVXData.test_orthogonality("CorrPI:ev_MIMT",d,ev_MIMT,MI,MT,eps);
	EVXData.test_orthogonality("CorrPI:ev_pMTMT",d,ev_pMTMT,pMT,MT,eps);

	final double I_0_feq=mi[0]/mt[0],wil=(I_0_feq-iH)/(iL-iH),wih=1-wil;
	final double [] mpsil=EVXData.getPsiFromBasis(0,ev_MIMT,d),mpsih=EVXData.getPsiFromBasis(1,ev_MIMT,d);
	final double [] psie=TMatr.sumaAbB(Math.sqrt(wil),mpsil,Math.sqrt(wih),mpsih);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psie,MI,2),"I_0_feq",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psie,MT,2),"1",1,"1",eps);
	final double pil=-mpsil[0]/mpsil[1],pih=-mpsih[0]/mpsih[1],Pe=wih*pil+pih*wil;
	UnitTests.checkDoubleRel(wil*iL+wih*iH,"I_0_feq",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(Pt,"Pt",Pe,"Pe",eps);
	final double [] mpsipl=EVXData.getPsiFromBasis(0,ev_pMTMT,d),mpsiph=EVXData.getPsiFromBasis(1,ev_pMTMT,d);
	final double cll=Math.pow(TMatr.getAMatrB(mpsipl,mpsil,MT,d),2),
	    clh=Math.pow(TMatr.getAMatrB(mpsipl,mpsih,MT,d),2),
	    chl=Math.pow(TMatr.getAMatrB(mpsiph,mpsil,MT,d),2),
	    chh=Math.pow(TMatr.getAMatrB(mpsiph,mpsih,MT,d),2),
	    _corr=(cll+chh-(clh+chl))/d;
	UnitTests.checkDoubleRel(cll,"cll",w_pL_iL,"w_pL_iL",eps);
	UnitTests.checkDoubleRel(clh,"clh",w_pL_iH,"w_pL_iH",eps);
	UnitTests.checkDoubleRel(chl,"chl",w_pH_iL,"w_pH_iL",eps);
	UnitTests.checkDoubleRel(chh,"chh",w_pH_iH,"w_pH_iH",eps);
	//UnitTests.checkDoubleRel(p_aver,"p_aver",Pe,"Pe",eps);
	//UnitTests.checkDoubleRel(i_aver,"i_aver",I_0_feq,"I_0_feq",eps);

	UnitTests.checkDoubleRel(corr,"corr",_corr,"_corr",eps);
	UnitTests.checkDoubleRel(clh+cll,"clh+cll",1,"1",eps);
	UnitTests.checkDoubleRel(chl+cll,"chl+cll",1,"1",eps);
	UnitTests.checkDoubleRel(chl+chh,"chl+chh",1,"1",eps);
	UnitTests.checkDoubleRel(clh+chh,"clh+chh",1,"1",eps);
	//System.err.println("clh="+clh+" chl="+chl);
	final Skewness sk=Skewness.getSkewnessFromMonomialMoments(mt[0],mt[1],mt[2],mt[3]);
	UnitTests.checkDoubleRel(pL,"pL",sk.x1,"sk.x1",eps);
	UnitTests.checkDoubleRel(pH,"pH",sk.x2,"sk.x2",eps);
	System.err.println("CorrPI: selftest OK: I_0_feq="+I_0_feq+" corr="+corr+" eps="+eps);
    }
}


public final EVXData I;
public final double p_offset,I_0_feq;
public final CorrPI ti,it;
public final Skewness SKi,SKt;
PnLdIE(final EVXData I,final double p_offset,final double I_0_feq,
       final double [] mt,final double [] mi,
       final EVSolver EV,
       final double eps)
{
    this.I=I;
    this.p_offset=p_offset;
    this.I_0_feq=I_0_feq;
    this.it=new CorrPI(mi,mt,EV,eps);
    this.ti=new CorrPI(mt,mi,EV,eps);
    this.SKi=Skewness.getSkewnessFromMonomialMoments(mi[0],mi[1],mi[2],mi[3]);
    this.SKt=Skewness.getSkewnessFromMonomialMoments(mt[0],mt[1],mt[2],mt[3]);

    if(eps>0)
    {
	UnitTests.checkDoubleRel(I_0_feq,"I_0_feq",mi[0]/mt[0],"mi[0]/mt[0]",eps);
	System.err.println("PnLdIE.get_PnLdIE: selftest OK: I_0_feq="+I_0_feq+" n="+I.neq+" eps="+eps);
    }
}



/**
 * Solve {@literal <psi|I|psi>/<psi|psi>  -> max}
 * extremum problem, where I if execution flow
 * operator (with or without impact from the future).
 * Then consider {@literal <psi|I phi(P)^2|psi>/<psi|phi(P)^2|psi> -> min/max}.
 */
public static PnLdIE get_PnLdIE(final int n,
				final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				final double p_offset,
				final double [] QQt,final double [] QQi,
				final double [] QQpi,final double [] QQp2i,final double [] QQp3i,
				final double [] QQpt,final double [] QQp2t,final double [] QQp3t,
				final double eps)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double I_0=I.s0,I_0_feq=I.sH,dI=I_0_feq-I_0;
    final double [] psi=I.psiH;

    final double mi0=I.sH,mi1=TMatr.getAMatrA(psi,QQpi,n),mi2=TMatr.getAMatrA(psi,QQp2i,n),mi3=TMatr.getAMatrA(psi,QQp3i,n),
	mt0=1,mt1=TMatr.getAMatrA(psi,QQpt,n),mt2=TMatr.getAMatrA(psi,QQp2t,n),mt3=TMatr.getAMatrA(psi,QQp3t,n);

    return new PnLdIE(I,p_offset,I_0_feq,
		      new double []{mt0,mt1,mt2,mt3},new double []{mi0,mi1,mi2,mi3},
		      M.EV,eps);
}

public static PnLdIE get_PnLdIE_DYN(final MomentsData mm,
				    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				    final boolean add_dI,
				    final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
        final double I_0=I.s0,I_0_feq=add_dI?I.sH:I.s0,dI=I_0_feq-I_0,
	p_last__with_p_offset_included=mm.p_last;

    final double [] bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp2i,I.ev),
	bQQp3i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp3i,I.ev),
	bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));

    // Create operators with an impact from the future.
    final double pl=p_last__with_p_offset_included;
    final double [] bQQ0=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),1/(I.o_o));
    TMatr.sumP1(bQQi,bQQ0,dI);
    TMatr.sumP1(bQQpi,bQQ0,dI*pl);
    TMatr.sumP1(bQQp2i,bQQ0,dI*Math.pow(pl,2));
    TMatr.sumP1(bQQp3i,bQQ0,dI*Math.pow(pl,3));

    final double [] bbasis=M.EV.getEV(n,bQQi,true),
	psik_I_psik=new double [n],psik_pI_psik=new double [n],psik_p2I_psik=new double [n],psik_p3I_psik=new double [n],w0=new double [n],w0t=new double [n];
for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n);

	final double psik_psi0=TMatr.mulVV(psik,bpsi0);
	w0[k]=Math.pow(psik_psi0,2);
	w0t[k]=w0[k]/bbasis[k];
	psik_I_psik[k]=bbasis[k];
	psik_pI_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n);
	psik_p2I_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQp2i,n);
	psik_p3I_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQp3i,n);
    }

    return new PnLdIE(I,mm.p_offset,I_0_feq,
		      new double []{TMatr.mulVV(w0t,psik_I_psik),TMatr.mulVV(w0t,psik_pI_psik),TMatr.mulVV(w0t,psik_p2I_psik),TMatr.mulVV(w0t,psik_p3I_psik)},
		      new double []{TMatr.mulVV(w0,psik_I_psik),TMatr.mulVV(w0,psik_pI_psik),TMatr.mulVV(w0,psik_p2I_psik),TMatr.mulVV(w0,psik_p3I_psik)},
		      M.EV,eps);
}

static void matchPnLdIP(final PnLdIE a,final PnLdIP b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.it.Pi,"a.it.Pi",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.it.Pt,"a.it.Pt",b.pt_IH+p_offset_shift,"b.pt_IH+p_offset_shift",eps);

    System.err.println("PnLdIE.PnLdIP test OK n="+a.I.neq+" eps="+eps);
}
static void matchPnLdISkewness(final PnLdIE a,final PnLdISkewness b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.it.Pi,"a.it.Pi",b.W0pi_S.xa+p_offset_shift,"b.W0pi_S.xa+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.it.pL,"a.it.pL",b.W0pt_S.x1+p_offset_shift,"b.W0pt_S.x1+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.it.pH,"a.it.pH",b.W0pt_S.x2+p_offset_shift,"b.W0pt_S.x2+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.it.Pt,"a.it.Pt",b.W0pt_S.xa+p_offset_shift,"b.W0pt_S.xa+p_offset_shift",eps);

    UnitTests.checkDoubleRel(a.ti.Pi,"a.ti.Pi",b.W0pt_S.xa+p_offset_shift,"b.W0pt_S.xa+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.ti.pL,"a.ti.pL",b.W0pi_S.x1+p_offset_shift,"b.W0pi_S.x1+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.ti.pH,"a.ti.pH",b.W0pi_S.x2+p_offset_shift,"b.W0pi_S.x2+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.ti.Pt,"a.ti.Pt",b.W0pi_S.xa+p_offset_shift,"b.W0pi_S.xa+p_offset_shift",eps);

    System.err.println("PnLdIE.PnLdISkewness test OK n="+a.I.neq+" eps="+eps);
}
}
