package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;

/**
 *  @version $Id: PnLdIE2.java,v 1.29 2022/08/05 11:25:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdIE2 {

public final EVXData I;
    public final double p_offset,I_0_feq,p_IHf,pt_IHf,p_imin,w_imin,p_imax,w_imax,p_IH;
public final Skewness SKi,SKt;
final boolean add_dI;
PnLdIE2(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,final double I_0_feq,final double p_IHf,final double pt_IHf,final double p_imin,final double w_imin,final double p_imax,final double w_imax,
	final boolean add_dI,
	final double p_IH,final Skewness SKi,final Skewness SKt,
	final double eps)
{
    this.I=I;
    this.p_offset=p_offset;
    this.I_0_feq=I_0_feq;
    this.add_dI=add_dI;
    this.p_IHf=p_IHf;
    this.pt_IHf=pt_IHf;
    this.p_imin=p_imin;
    this.w_imin=w_imin;
    this.p_imax=p_imax;
    this.w_imax=w_imax;
    // for unit tests
    this.p_IH=p_IH;
    this.SKi=SKi;
    this.SKt=SKt;
}



/**
 * Try to solve {min,max}_{phi} max_{psi} {@literal <psi(t)*phi(p)|I|psi(t)*phi(p)>/<psi(t)*phi(p)|psi(t)*phi(p)>}
 * extremum problem, not sure of result.
 */
public static PnLdIE2 get_PnLdIE2(final int n,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				  final double p_offset,
				  final double p_last__with_p_offset_included,
				  final boolean add_dI,
				  final double [] QQt,final double [] QQi,
				  final double [] QQpi,final double [] QQp2i,final double [] QQp3i,
				  final double [] QQpt,final double [] QQp2t,final double [] QQp3t,
				  final double eps)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double I_0=I.s0,I_0_feq=add_dI?I.sH:I.s0,dI=I_0_feq-I_0;

    final double [] bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,QQp2i,I.ev),
	bQQp3i=EVXData.convertMatrFromQtoB_Basis(n,QQp3i,I.ev),
	bQQpt=EVXData.convertMatrFromQtoB_Basis(n,QQpt,I.ev),
	bQQp2t=EVXData.convertMatrFromQtoB_Basis(n,QQp2t,I.ev),
	bQQp3t=EVXData.convertMatrFromQtoB_Basis(n,QQp3t,I.ev);


    // Create operators with an impact from the future.
    final double pl=p_last__with_p_offset_included;
    final double [] bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o)),
	bQQ0=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),1/(I.o_o));
    TMatr.sumP1(bQQi,bQQ0,dI);
    TMatr.sumP1(bQQpi,bQQ0,dI*pl);
    TMatr.sumP1(bQQp2i,bQQ0,dI*Math.pow(pl,2));
    TMatr.sumP1(bQQp3i,bQQ0,dI*Math.pow(pl,3));

    final int ixH=n-1,d=2;
    final double [] bbasis=M.EV.getEV(n,bQQi,true),bpsi=EVXData.getPsiFromBasis(ixH,bbasis,n);
    final double mi0=bbasis[ixH],mi1=TMatr.getAMatrA(bpsi,bQQpi,n),mi2=TMatr.getAMatrA(bpsi,bQQp2i,n),mi3=TMatr.getAMatrA(bpsi,bQQp3i,n),
	mt0=TMatr.mulVV(bpsi,bpsi),mt1=TMatr.getAMatrA(bpsi,bQQpt,n),mt2=TMatr.getAMatrA(bpsi,bQQp2t,n),mt3=TMatr.getAMatrA(bpsi,bQQp3t,n);
    final double [] MI=new double []{mi0,mi1,mi1,mi2},MT=new double []{mt0,mt1,mt1,mt2},
	evP=M.EV.getGEV_Generalized(d,MI,MT,true),
	phi_imin=EVXData.getPsiFromBasis(0 /* min over phi */,evP,d),
	phi_imax=EVXData.getPsiFromBasis(d-1 /* max over phi */,evP,d);
    final double IHf=mi0,p_imin=-phi_imin[0]/phi_imin[1],p_imax=-phi_imax[0]/phi_imax[1],
	w_imin=(IHf-evP[d-1])/(evP[0]-evP[d-1]),w_imax=1-w_imin;
    if(eps>0)
    {
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi,n),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi,bQQi,n),"<psi|If|psi>",bbasis[ixH],"bbasis[ixH]",eps);
	UnitTests.checkDoubleRel(mi0/mt0,"mi0/mt0",IHf,"IHf",eps);
	final double [] psie=TMatr.sumaAbB(Math.sqrt(w_imin),phi_imin,Math.sqrt(w_imax),phi_imax),
	    psiee=new double []{1,0};
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psie,MI,d),"IHf",IHf,"IHf",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psie,MT,d),"1",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psiee,MI,d)/TMatr.getAMatrA(psiee,MT,d),"IHf",IHf,"IHf",eps);
	final double Pe=w_imax*p_imin+w_imin*p_imax,Pt=mt1/mt0;
	UnitTests.checkDoubleRel(Pt,"Pt",Pe,"Pe",eps);
	UnitTests.checkDoubleRel(w_imin*evP[0]+w_imax*evP[d-1],"IHf",IHf,"IHf",eps);
	UnitTests.checkDoubleRel(w_imin,"w_imin",Math.pow(TMatr.getAMatrB(phi_imin,psiee,MT,d),2)/TMatr.getAMatrA(psiee,MT,d),"<phi_imin|psiee>^2",eps);
	UnitTests.checkDoubleRel(w_imax,"w_imax",Math.pow(TMatr.getAMatrB(phi_imax,psiee,MT,d),2)/TMatr.getAMatrA(psiee,MT,d),"<phi_imax|psiee>^2",eps);
	System.err.println("PnLdIE2: n="+n+" evP[0]="+evP[0]+" evP[1]="+evP[1]+" IHf="+IHf+" add_dI="+add_dI+" eps="+eps);
    }

    return new PnLdIE2(I,p_offset,p_last__with_p_offset_included,I_0_feq,
		       TMatr.getAMatrA(bpsi,bQQpi,n)/bbasis[ixH],TMatr.getAMatrA(bpsi,bQQpt,n),
		       p_imin,w_imin,p_imax,w_imax,add_dI,
		       TMatr.getAMatrA(I.psiH,QQpi,n)/I.sH,
		       Skewness.getSkewnessFromMonomialMoments(mi0,mi1,mi2,mi3),
		       Skewness.getSkewnessFromMonomialMoments(mt0,mt1,mt2,mt3),
		       eps);
}


static void matchPnLdIDSk(final PnLdIE2 a,final PnLdIDSk b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    UnitTests.checkDoubleRel(a.I_0_feq,"a.I_0_feq",b.I_0_feq,"b.I_0_feq",eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    Skewness.matchSkewness(a.SKi.getSameSkewnessXShifted(a.p_offset),b.SKs[a.I.neq-1].sk.getSkewnessAt(b.p_last__with_p_offset_included).getSameSkewnessXShifted(b.p_offset),eps);

    System.err.println("PnLdIE2.matchPnLdIDSk test OK n="+a.I.neq+" eps="+eps);
}

static void matchPnLdIE(final PnLdIE2 a,final PnLdIE b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    //UnitTests.checkDoubleRel(a.I_0_feq,"a.I_0_feq",b.I_0_feq,"b.I_0_feq",eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IHf,"a.p_IHf",b.it.Pi+p_offset_shift,"b.it.Pi+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.pt_IHf,"a.pt_IHf",b.it.Pt+p_offset_shift,"b.it.Pt+p_offset_shift",eps);
    Skewness.matchSkewness(a.SKi.getSameSkewnessXShifted(a.p_offset),b.SKi.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SKt.getSameSkewnessXShifted(a.p_offset),b.SKt.getSameSkewnessXShifted(b.p_offset),eps);

    System.err.println("PnLdIE2.matchPnLdIE test OK n="+a.I.neq+" eps="+eps);
}

}
