package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Skewness;

/**
 * Skewness is back.
 *  @version $Id: PnLdIP.java,v 1.16 2022/08/05 11:25:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdIP {


public final EVXData I;
public final double p_offset,p_last__with_p_offset_included,p_IH,pt_IH,I_0_feqF;
public final Skewness IHf_S,IHf_St;
final double I_p1,I_p2;
PnLdIP(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,final double p_IH,final double pt_IH,
       final Skewness IHf_S,final double I_p1,final double I_p2,
       final Skewness IHf_St,
       final double [] bbasis,
       final boolean add_dI,
       final double eps)
{
    this.I=I;
    this.p_offset=p_offset;
    this.p_last__with_p_offset_included=p_last__with_p_offset_included;
    this.p_IH=p_IH;
    this.pt_IH=pt_IH;
    final int n=I.neq;
    this.I_0_feqF=bbasis[n-1];
    this.IHf_S=IHf_S;
    this.I_p1=I_p1;
    this.I_p2=I_p2;
    this.IHf_St=IHf_St;
}



/**
 * Solve {@literal <psi|I^f|psi>/<psi|psi>  -> max}
 * extremum problem, where I^{f} if execution flow
 * operator (with or without impact from the future).
 * Then consider skewness and other characteristics.
 */
public static PnLdIP get_PnLdIP(final MomentsData mm,
				final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				final double [] QQp1t,final double [] QQp2t,final double [] QQp3t,
				final boolean add_dI,
				final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double I_0=I.s0,I_0_feq=I.sH,dI=I_0_feq-I_0,
	p_last__with_p_offset_included=mm.p_last;

    final double [] bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp2i,I.ev),
	bQQp3i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp3i,I.ev),
	bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o)),
	bQQpt=EVXData.convertMatrFromQtoB_Basis(n,QQp1t,I.ev),
	bQQp2t=EVXData.convertMatrFromQtoB_Basis(n,QQp2t,I.ev),
	bQQp3t=EVXData.convertMatrFromQtoB_Basis(n,QQp3t,I.ev);


    // Create operators with an impact from the future.
    if(add_dI)
    {
	final double pl=p_last__with_p_offset_included;
	final double [] bQQ0=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o));
	TMatr.sumP1(bQQi,bQQ0,1);
	TMatr.sumP1(bQQpi,bQQ0,pl);
	TMatr.sumP1(bQQp2i,bQQ0,Math.pow(pl,2));
	TMatr.sumP1(bQQp3i,bQQ0,Math.pow(pl,3));
	// bQQpt does not need to add
    }
    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdIP.get_PnLdIP:I.ev",n,I.ev,mm.QQi,mm.QQt,eps); // old basis
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi0,bpsi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi,n),"<psi0|I|psi0>",add_dI?I_0_feq:I_0,"I_0_feq",eps);
    }

    final double [] bbasis=M.EV.getEV(n,bQQi,true);
    final int ixH=n-1;
    final double [] psi=EVXData.getPsiFromBasis(ixH,bbasis,n);
    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdIP.get_PnLdIP:bbasis",n,bbasis,bQQi,TMatr.getUnitMatrix(n),eps); // new basis
	UnitTests.checkDoubleRel(bbasis[ixH],"bbasis[ixH]",TMatr.getAMatrA(psi,bQQi,n),"<psi|bQQi|psi>",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(psi,psi),"<psi|psi>",1,"1",eps);
    }
    final double mi=TMatr.getAMatrA(psi,bQQi,n),
	mpi=TMatr.getAMatrA(psi,bQQpi,n),
	mp2i=TMatr.getAMatrA(psi,bQQp2i,n),
	mp3i=TMatr.getAMatrA(psi,bQQp3i,n),
	mt=1,
	mpt=TMatr.getAMatrA(psi,bQQpt,n),
	mp2t=TMatr.getAMatrA(psi,bQQp2t,n),
	mp3t=TMatr.getAMatrA(psi,bQQp3t,n);

    final double [] mI=new double []{mi,mpi,mpi,mp2i},mpI=new double []{mpi,mp2i,mp2i,mp3i},mT=new double []{mt,mpt,mpt,mp2t},
       evP=M.EV.getGEV_Generalized(2,mpI,mI,true),
       evI=M.EV.getGEV_Generalized(2,mI,mT,true);

    final Skewness IHf_S=Skewness.getSkewnessFromMonomialMoments(mi,mpi,mp2i,mp3i);
    final double I_p1=1/TMatr.getAMatrA(2,evP,2+2*0,mT,2),I_p2=1/TMatr.getAMatrA(2,evP,2+2*1,mT,2);
    if(eps>0)
    {
	UnitTests.checkDoubleRel(IHf_S.x1,"IHf_S.x1",evP[0],"evP[0]",eps);
	UnitTests.checkDoubleRel(IHf_S.x2,"IHf_S.x2",evP[1],"evP[1]",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(2,evP,2+2*0,mI,2),"<psiP|I|psiP>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(2,evP,2+2*1,mI,2),"<psiP|I|psiP>",1,"1",eps);
	System.err.println("PnLdIP: selftest OK n="+n+" eps="+eps);
    }

    return new PnLdIP(I,mm.p_offset,p_last__with_p_offset_included,I.getFPsi(n-1,mm.QQpi)/I.sH,I.getFPsi(n-1,QQp1t),
		      IHf_S,I_p1,I_p2,
		      Skewness.getSkewnessFromMonomialMoments(mt,mpt,mp2t,mp3t),
		      bbasis,add_dI,eps);
}

}
