package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.Skewness;

/**
 *  @version $Id: PnLdIS.java,v 1.28 2022/08/07 15:44:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdIS {

    public final EVXData I;
    public final double p_offset,p_IH,I_0_feq,P_movaver,P_movaver_spur,
	dp_orig_IH,dp_orig_0,w_IHf_0,p_IHf_nodI;
    public final Skewness SP,SdP,SPPL;
    PnLdIS(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,
	   final double P_movaver,final double P_movaver_spur,final double p_IH,final double I_0_feq,
	   final double dp_orig_IH,final double dp_orig_0,
	   final double [] psik_psi0,final double [] psik_I_psik,final double [] psik_pI_psik,final double [] psik_dP_psik,final double [] bbasis)
    {
	this.I=I;
	this.p_offset=p_offset;
	this.P_movaver=P_movaver;
	this.P_movaver_spur=TMatr.sumElements(psik_pI_psik)/TMatr.sumElements(psik_I_psik);
	this.p_IH=p_IH;
	this.I_0_feq=I_0_feq;
	this.dp_orig_IH=dp_orig_IH;
	this.dp_orig_0=dp_orig_0;
	final double dI=I_0_feq-I.s0;
	final int n=I.neq,ixH=n-1;
	this.p_IHf_nodI=psik_pI_psik[ixH]/psik_I_psik[ixH];
	this.w_IHf_0=Math.pow(psik_psi0[ixH],2);
	double m0=0,mp1=0,mp2=0,mp3=0,mdp1=0,mdp2=0,mdp3=0,mppl1=0,mppl2=0,mppl3=0;
	for(int k=0;k<n;k++)
	{
	    final double wk=Math.pow(psik_psi0[k],2),pk=psik_pI_psik[k]/psik_I_psik[k],dpk=psik_dP_psik[k],
		pplk=(psik_pI_psik[k]+p_last__with_p_offset_included*dI*wk)/bbasis[k];
	    m0+=wk;
	    mp1+=wk*pk;
	    mp2+=wk*Math.pow(pk,2);
	    mp3+=wk*Math.pow(pk,3);
	    mdp1+=wk*dpk;
	    mdp2+=wk*Math.pow(dpk,2);
	    mdp3+=wk*Math.pow(dpk,3);
	    mppl1+=wk*pplk;
	    mppl2+=wk*Math.pow(pplk,2);
	    mppl3+=wk*Math.pow(pplk,3);
	}
	this.SP=Skewness.getSkewnessFromMonomialMoments(m0,mp1,mp2,mp3);
	this.SdP=Skewness.getSkewnessFromMonomialMoments(m0,mdp1,mdp2,mdp3);
	this.SPPL=Skewness.getSkewnessFromMonomialMoments(m0,mppl1,mppl2,mppl3);
    }

/**
 * Solve <psi|I|psi>/<psi|psi>=lambda extremum problem, find eigenstates & eigenvalues.
 * find psi^{IH} and lambda^{IH}=max lambda, used as an estimator of
 * the level of "future" execution.
 * Then dI=lambda^{IH}-<psi0|I|psi0> (contribution from the future, of not yet executed, trades).
 * Then operator I^{f} (future I) is: I^{f}= I + |psi0>dI<psi0|
 * Find the eigenstates of this new I^{f} matrix,
 * in dynamic impact approximation its eigenstates should be also the P^{f} operator eigenstates.
 * because <psik|I^{f}|psik>= <psik|I|psik> + <psik|psi0>^2 dI
 * for |psik> the |I^{f}|psik>=lambda |psik> solutions the contribition of not yet executed trades
 * is proportional to <psik|psi0>^2 dI. Averaging with <psik|psi0>^2 give "eff price",
 * then calculate the skewness for future direction.
 */
public static PnLdIS get_PnLdIS(final MomentsData mm,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				  final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final int ixH=n-1; // the state with maximal I
    final double I_0=I.s0,I_0_feq=I.ev[ixH],dI=I_0_feq-I_0;

    // Create bQQi_feq (operator of "future" I), with the |bpsi0>dI<bpsi0| contribution from the future:
    // bQQi_feq = bQQi + |bpsi0>dI<bpsi0|
    final double [] bQQi_feq=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o)),bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));
    for(int k=0;k<n;k++) bQQi_feq[k+n*k]+=I.ev[k];

    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQdp=EVXData.convertMatrFromQtoB_Basis(n,mm.QQdp,I.ev);

    // Diagonalize "future" I:
    final double [] bbasis=M.EV.getEV(n,bQQi_feq,true),
	psik_psi0=new double [n],psik_I_psik=new double[n],psik_pI_psik=new double[n],psik_dP_psik=new double[n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n);
	psik_psi0[k]=TMatr.mulVV(psik,bpsi0);
	psik_I_psik[k]=TMatr.mulV2V(psik,I.getEValues());
	psik_pI_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n);
	psik_dP_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQdp,n);
    }

    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdIS.get_PnLdIS:I.ev",n,I.ev,mm.QQi,mm.QQt,eps); // old basis
	EVXData.test_orthogonality("PnLdIS.get_PnLdIS:bbasis",n,bbasis,bQQi_feq,TMatr.getUnitMatrix(n),eps); // new basis
	// Check spur invariance.
	final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,mm.QQt);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQpi),"Spur(IQQt,QQpi)",TMatr.sumElements(psik_pI_psik),"sum(psik_pI_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQpi),"Spur(bQQpi)",TMatr.sumElements(psik_pI_psik),"sum(psik_pI_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQi),"Spur(IQQt,QQi)",TMatr.sumElements(psik_I_psik),"sum(psik_I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues()),"sum(I.ev)",TMatr.sumElements(psik_I_psik),"sum(psik_I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues())+dI,"sum(I.ev)+dI",TMatr.sumElements(java.util.Arrays.copyOf(bbasis,n)),"sum(ev bbasis)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQi_feq),"Spur(bQQi_feq)",TMatr.sumElements(java.util.Arrays.copyOf(bbasis,n)),"sum(ev bbasis)",eps);

	// Check some matrix elements to match known values.
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psi0),"sum(psik_psi0[k]^2)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(psik_psi0,java.util.Arrays.copyOf(bbasis,n)),"sum(psik_psi0[k]^2 If_k)",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_feq,n),"<psi0|I^f|psi0>",I_0_feq,"I_0_feq",eps);
	// Check basis invariance.
	for(int k=0;k<n;k++)
	{
	    final double [] psik=EVXData.convertPsiFromBtoQ_Basis(n,EVXData.getPsiFromBasis(k,bbasis,n),I.ev);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psik,mm.QQi,n),"<psik|I|psik>",psik_I_psik[k],"psik_I_psik[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psik,mm.QQpi,n),"<psik|pI|psik>",psik_pI_psik[k],"psik_pI_psik[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psik,mm.QQt,n),"<psik|psik>",1,"1",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psik,I.psi0,mm.QQt,n),"<psik|psi0>",psik_psi0[k],"psik_psi0[k]",eps);
	}

	System.err.println("PnLdIS.get_PnLdIS selftest OK n="+n+" eps="+eps);
    }

    return new PnLdIS(I,mm.p_offset,mm.p_last,
		      mm.QQpi[0]/mm.QQi[0],TMatr.getSpurM(n,bQQpi)/TMatr.sumElements(I.getEValues()),
		      bQQpi[ixH+n*ixH]/I.ev[ixH],I_0_feq,bQQdp[ixH+n*ixH],TMatr.getAMatrA(bpsi0,bQQdp,n),
		      psik_psi0,psik_I_psik,psik_pI_psik,psik_dP_psik,bbasis);
}
}
