package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.Skewness;

/**
 * Skewness is back. See also our old skewness approaches in RkiQVM,DynSki and QkQVM,EAssym.
 *  @version $Id: PnLdISk.java,v 1.55 2022/08/07 15:44:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdISk {

    public final EVXData I;
    public final double p_offset,p_IH,I_0_feq,I_movaver_spur,P_movaver_spur,
	w_IHf_0,w_IHf;
    public final Skewness SK_IHf,SK_IHf__nodI,SK_IH,SK_movaver,SK_IH_V,SK_IHf_V,SK_IHf_W0,SK_IHf_WS,SK_IHf_Pfeq;
    PnLdISk(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,
	    final double p_IH,final double I_0_feq,
	    final double [] psik_psi0,
	    final double [] psik_I_psik,final double [] psik_pI_psik,
	    final double [] psik_p2I_psik,final double [] psik_p3I_psik,final double [] bbasis,
	    final Skewness SK_IH,
	    final Skewness SK_movaver,
	    final Skewness SK_IH_V,final Skewness SK_IHf_V,
	    final Skewness SK_IHf_Pfeq,
	    final double eps)
    {
	this.I=I;
	this.p_offset=p_offset;
	this.I_movaver_spur=TMatr.sumElements(psik_I_psik)/psik_I_psik.length;
	this.P_movaver_spur=TMatr.sumElements(psik_pI_psik)/TMatr.sumElements(psik_I_psik);
	this.p_IH=p_IH;
	this.I_0_feq=I_0_feq;
	this.SK_IH=SK_IH;
	this.SK_movaver=SK_movaver;
	final int n=I.neq,ixH=n-1;
	final double dI=(I_0_feq-I.s0);
	this.w_IHf_0=Math.pow(psik_psi0[ixH],2);
	this.w_IHf=this.w_IHf_0*dI;
	this.SK_IHf__nodI=Skewness.getSkewnessFromMonomialMoments(psik_I_psik[ixH],psik_pI_psik[ixH],psik_p2I_psik[ixH],psik_p3I_psik[ixH]);
	this.SK_IHf=Skewness.getSkewnessFromMonomialMoments(psik_I_psik[ixH]+w_IHf,psik_pI_psik[ixH]+w_IHf*p_last__with_p_offset_included,psik_p2I_psik[ixH]+w_IHf*Math.pow(p_last__with_p_offset_included,2),psik_p3I_psik[ixH]+w_IHf*Math.pow(p_last__with_p_offset_included,3));
	this.SK_IH_V=SK_IH_V;
	this.SK_IHf_V=SK_IHf_V;
	final double [] w0=TMatr.squareArrayElements(psik_psi0);
	final double w0w0=TMatr.mulVV(w0,w0);
	this.SK_IHf_W0=Skewness.getSkewnessFromMonomialMoments(
	    TMatr.mulVV(w0,psik_I_psik)+w0w0*dI,
	    TMatr.mulVV(w0,psik_pI_psik)+w0w0*dI*p_last__with_p_offset_included,
	    TMatr.mulVV(w0,psik_p2I_psik)+w0w0*dI*Math.pow(p_last__with_p_offset_included,2),
	    TMatr.mulVV(w0,psik_p3I_psik)+w0w0*dI*Math.pow(p_last__with_p_offset_included,3));
	this.SK_IHf_WS=Skewness.getSkewnessFromMonomialMoments(
	    TMatr.sumElements(psik_I_psik)+dI,
	    TMatr.sumElements(psik_pI_psik)+dI*p_last__with_p_offset_included,
	    TMatr.sumElements(psik_p2I_psik)+dI*Math.pow(p_last__with_p_offset_included,2),
	    TMatr.sumElements(psik_p3I_psik)+dI*Math.pow(p_last__with_p_offset_included,3));
	this.SK_IHf_Pfeq=SK_IHf_Pfeq;
	if(eps>0)
	{
	    // match weights
	    UnitTests.checkDoubleRel(bbasis[ixH],"bbasis[ixH]",SK_IHf.w1+SK_IHf.w2,"w_IHf+SK_IHf.w1+SK_IHf.w2",eps);
	    UnitTests.checkDoubleRel(bbasis[ixH],"bbasis[ixH]",w_IHf+SK_IHf__nodI.w1+SK_IHf__nodI.w2,"w_IHf+SK_IHf__nodI.w1+SK_IHf__nodI.w2",eps);
	    UnitTests.checkDoubleRel(psik_p3I_psik[ixH]+dI*Math.pow(psik_psi0[ixH],2)*Math.pow(p_last__with_p_offset_included,3),"m3",SK_IHf.w1*Math.pow(SK_IHf.x1,3)+SK_IHf.w2*Math.pow(SK_IHf.x2,3),"SK_IHf.w1*x1^3+SK_IHf.w2*x2^3",eps);

	    System.err.println("PnLdISk.PnLdISk selftest2 OK n="+n+" eps="+eps);
	}

    }

/**
 * Solve <psi|I|psi>/<psi|psi>=lambda extremum problem, find eigenstates & eigenvalues.
 * find psi^{IH} and lambda^{IH}=max lambda, used as an estimator of
 * the level of "future" execution.
 * Then dI=lambda^{IH}-<psi0|I|psi0> (contribution from the future, of not yet executed, trades).
 * Then operator I^{f} (future I) is: I^{f}= I + |psi0>dI<psi0|
 * Find the eigenstates of this new I^{f} matrix,
 * in dynamic impact approximation its eigenstates should be also the P^{f} operator eigenstates.
 * because <psik|I^{f}|psik>= <psik|I|psik> + <psik|psi0>^2 dI
 * for |psik> the |I^{f}|psik>=lambda |psik> solutions the contribition of not yet executed trades
 * is proportional to <psik|psi0>^2 dI.
 * For the moments of |psik> state calculate the price, manipulating with the skewness.
 */
public static PnLdISk get_PnLdISk(final MomentsData mm,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				  final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final int ixH=n-1; // the state with maximal I
    final double I_0=I.s0,I_0_feq=I.ev[ixH],dI=I_0_feq-I_0;

    // Create bQQi_feq (operator of "future" I), with the |bpsi0>dI<bpsi0| contribution from the future:
    // bQQi_feq = bQQi + |bpsi0>dI<bpsi0|
    final double [] bQQi_feq=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o)),bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));
    for(int k=0;k<n;k++) bQQi_feq[k+n*k]+=I.ev[k];

    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp2i,I.ev),
	bQQp3i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp3i,I.ev);

    // Diagonalize "future" I:
    final double [] bbasis=M.EV.getEV(n,bQQi_feq,true),
	psik_psi0=new double [n],psik_I_psik=new double[n],psik_pI_psik=new double[n],
	psik_p2I_psik=new double[n],psik_p3I_psik=new double[n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n);
	psik_psi0[k]=TMatr.mulVV(psik,bpsi0);
	psik_I_psik[k]=TMatr.mulV2V(psik,I.getEValues());
	psik_pI_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n);
	psik_p2I_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQp2i,n);
	psik_p3I_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQp3i,n);
    }

    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdISk.get_PnLdISk:I.ev",n,I.ev,mm.QQi,mm.QQt,eps); // old basis
	EVXData.test_orthogonality("PnLdISk.get_PnLdISk:bbasis",n,bbasis,bQQi_feq,TMatr.getUnitMatrix(n),eps); // new basis
	// Check spur invariance.
	final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,mm.QQt);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQpi),"Spur(IQQt,QQpi)",TMatr.sumElements(psik_pI_psik),"sum(psik_pI_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQpi),"Spur(bQQpi)",TMatr.sumElements(psik_pI_psik),"sum(psik_pI_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQp2i),"Spur(IQQt,QQp2i)",TMatr.sumElements(psik_p2I_psik),"sum(psik_p2I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQp2i),"Spur(bQQp2i)",TMatr.sumElements(psik_p2I_psik),"sum(psik_p2I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQp3i),"Spur(IQQt,QQp3i)",TMatr.sumElements(psik_p3I_psik),"sum(psik_p3I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQp3i),"Spur(bQQp3i)",TMatr.sumElements(psik_p3I_psik),"sum(psik_p3I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQi),"Spur(IQQt,QQi)",TMatr.sumElements(psik_I_psik),"sum(psik_I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues()),"sum(I.ev)",TMatr.sumElements(psik_I_psik),"sum(psik_I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues())+dI,"sum(I.ev)+dI",TMatr.sumElements(java.util.Arrays.copyOf(bbasis,n)),"sum(ev bbasis)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQi_feq),"Spur(bQQi_feq)",TMatr.sumElements(java.util.Arrays.copyOf(bbasis,n)),"sum(ev bbasis)",eps);

	// Check some matrix elements to match known values.
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psi0),"sum(psik_psi0[k]^2)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(psik_psi0,java.util.Arrays.copyOf(bbasis,n)),"sum(psik_psi0[k]^2 If_k)",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_feq,n),"<psi0|I^f|psi0>",I_0_feq,"I_0_feq",eps);
	// Check basis invariance.
	for(int k=0;k<n;k++)
	{
	    final double [] psik=EVXData.convertPsiFromBtoQ_Basis(n,EVXData.getPsiFromBasis(k,bbasis,n),I.ev);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psik,mm.QQi,n),"<psik|I|psik>",psik_I_psik[k],"psik_I_psik[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psik,mm.QQpi,n),"<psik|pI|psik>",psik_pI_psik[k],"psik_pI_psik[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psik,mm.QQp2i,n),"<psik|p2I|psik>",psik_p2I_psik[k],"psik_p2I_psik[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psik,mm.QQp3i,n),"<psik|p3I|psik>",psik_p3I_psik[k],"psik_p3I_psik[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psik,mm.QQt,n),"<psik|psik>",1,"1",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psik,I.psi0,mm.QQt,n),"<psik|psi0>",psik_psi0[k],"psik_psi0[k]",eps);
	    new Skewness0(psik_I_psik[k],psik_pI_psik[k],psik_p2I_psik[k],psik_p3I_psik[k],dI*Math.pow(psik_psi0[k],2),eps);
	    new SkewnessMinMax(psik_I_psik[k],psik_pI_psik[k],psik_p2I_psik[k],psik_p3I_psik[k],dI*Math.pow(psik_psi0[k],2),eps);
	}
	Skewness.matchSkewness(DynSki.get_fromTR_DynSki(n,mm.QQi,mm.QQt,mm.QQpi,mm.QQp2i,mm.QQp3i),new DynSki(psik_I_psik,psik_pI_psik,psik_p2I_psik,psik_p3I_psik).sk,eps);

	for(int k=0;k<n;k++)
	{
	    final Skewness s=Skewness.getSkewnessFromMonomialMoments(psik_I_psik[k],psik_pI_psik[k],psik_p2I_psik[k],psik_p3I_psik[k]);
	    UnitTests.checkDoubleRel(psik_I_psik[k],"psik_I_psik[k]",s.w1+s.w2,"s.w1+s.w2",eps);
	    UnitTests.checkDoubleRel(psik_I_psik[k]+dI*Math.pow(psik_psi0[k],2),"psik_I_psik[k]+dI*<psik|psi0>^2",bbasis[k],"bbasis[k]",eps);
	    UnitTests.checkDoubleRel(psik_pI_psik[k]/psik_I_psik[k],"xa",s.xa,"s.xa",eps);
	    if(s.x2<s.x1) throw new RuntimeException("s.x2="+s.x2+" s.x1="+s.x1);
	}
	System.err.println("PnLdISk.get_PnLdISk selftest OK n="+n+" eps="+eps);
    }

    final Skewness SK_IH_V,SK_IHf_V;
    if(true)
    {
	final double [] bQQV=EVXData.convertMatrFromQtoB_Basis(n,mm.QQV,I.ev),
	    bQQpV=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpV,I.ev),
	    bQQp2V=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp2V,I.ev),
	    bQQp3V=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp3V,I.ev);

	SK_IH_V=Skewness.getSkewnessFromMonomialMoments(-bQQV[ixH+n*ixH],-bQQpV[ixH+n*ixH],-bQQp2V[ixH+n*ixH],-bQQp3V[ixH+n*ixH]);
	final double [] bpsiIHf=EVXData.getPsiFromBasis(ixH,bbasis,n);
	final double dv=dI/I.o_o,pl=mm.p_last;
	SK_IHf_V=Skewness.getSkewnessFromMonomialMoments(
	    -TMatr.getAMatrA(bpsiIHf,bQQV,n)+dv,
	    -TMatr.getAMatrA(bpsiIHf,bQQpV,n)+dv*pl,
	    -TMatr.getAMatrA(bpsiIHf,bQQp2V,n)+dv*Math.pow(pl,2),
	    -TMatr.getAMatrA(bpsiIHf,bQQp3V,n)+dv*Math.pow(pl,3));
	if(eps>0)
	{
	    // exponential moving average
	    UnitTests.checkDoubleRel(mm.QQi[0],"mm.QQi[0]",-mm.QQV[0],"-mm.QQV[0]",eps);
	    UnitTests.checkDoubleRel(mm.QQpi[0],"mm.QQpi[0]",-mm.QQpV[0],"-mm.QQpV[0]",eps);
	    UnitTests.checkDoubleRel(mm.QQp2i[0],"mm.QQp2i[0]",-mm.QQp2V[0],"-mm.QQp2V[0]",eps);
	    UnitTests.checkDoubleRel(mm.QQp3i[0],"mm.QQp3i[0]",-mm.QQp3V[0],"-mm.QQp3V[0]",eps);
	    System.err.println("PnLdISk.get_PnLdISk selftestV OK n="+n+" eps="+eps);
	}
    }

    return new PnLdISk(I,mm.p_offset,mm.p_last,
		       bQQpi[ixH+n*ixH]/I.ev[ixH],I_0_feq,
		       psik_psi0,psik_I_psik,psik_pI_psik,psik_p2I_psik,psik_p3I_psik,bbasis,
		       Skewness.getSkewnessFromMonomialMoments(I.ev[ixH],bQQpi[ixH+n*ixH],bQQp2i[ixH+n*ixH],bQQp3i[ixH+n*ixH]),
		       Skewness.getSkewnessFromMonomialMoments(mm.QQi[0]/mm.QQt[0],mm.QQpi[0]/mm.QQt[0],mm.QQp2i[0]/mm.QQt[0],mm.QQp3i[0]/mm.QQt[0]),
		       SK_IH_V,SK_IHf_V,
		       Skewness.getSkewnessFromMonomialMoments(
			   bbasis[ixH],
			   getPmIEq_PmFuture(psik_pI_psik[ixH],TMatr.getAMatrB(n,bbasis,n+ixH*n,bpsi0,0,bQQpi,n),psik_psi0[ixH]),
			   getPmIEq_PmFuture(psik_p2I_psik[ixH],TMatr.getAMatrB(n,bbasis,n+ixH*n,bpsi0,0,bQQp2i,n),psik_psi0[ixH]),
			   getPmIEq_PmFuture(psik_p3I_psik[ixH],TMatr.getAMatrB(n,bbasis,n+ixH*n,bpsi0,0,bQQp3i,n),psik_psi0[ixH])),
		       eps);
}

static double getPmIEq_PmFuture(final double psik_pmI_psik,
				final double psik_pmI_psi0,
				final double psik_psi0)
{
    return (psik_pmI_psik-psik_psi0*psik_pmI_psi0)/(1-psik_psi0*psik_psi0);
}


static void matchPnLdIVSk(final PnLdISk a,final PnLdIVSk b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.I_0_feq,"a.I_0_feq",b.I_0_feq,"b.I_0_feq",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    Skewness.matchSkewness(a.SK_IHf.getSameSkewnessXShifted(a.p_offset),b.SK_IHf.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SK_IHf_V.getSameSkewnessXShifted(a.p_offset),b.SK_IHf_V.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SK_IH.getSameSkewnessXShifted(a.p_offset),b.SK_IH.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SK_IH_V.getSameSkewnessXShifted(a.p_offset),b.SK_IH_V.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SK_movaver.getSameSkewnessXShifted(a.p_offset),b.SK_movaver.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SK_IHf_W0.getSameSkewnessXShifted(a.p_offset),b.SK_IHf_W0.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SK_IHf_WS.getSameSkewnessXShifted(a.p_offset),b.SK_IHf_WS.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SK_IHf_Pfeq.getSameSkewnessXShifted(a.p_offset),b.SK_IHf_Pfeq.getSameSkewnessXShifted(b.p_offset),eps);

    System.err.println("PnLdISk.matchPnLdIVSk test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}

static void matchPnLdIV4(final PnLdISk a,final PnLdIV4 bf,final PnLdIV4 bp,final double eps)
{
    EVXData.matchEVXData(a.I,bf.I,false,eps);
    EVXData.matchEVXData(a.I,bp.I,false,eps);
    Skewness.matchSkewness(a.SK_IHf.getSameSkewnessXShifted(a.p_offset),bf.SK_init.getSameSkewnessXShifted(bf.p_offset),eps);
    Skewness.matchSkewness(a.SK_IH.getSameSkewnessXShifted(a.p_offset),bp.SK_init.getSameSkewnessXShifted(bp.p_offset),eps);
    Skewness.matchSkewness(a.SK_IHf_WS.getSameSkewnessXShifted(a.p_offset),bf.SK_spur.getSameSkewnessXShifted(bf.p_offset),eps);
    System.err.println("PnLdISk.matchPnLdIV4 test OK n="+a.I.neq+" eps="+eps);
}

static void matchPnLdISkewness(final PnLdISk a,final PnLdISkewness b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    Skewness.matchSkewness(a.SK_IHf.getSameSkewnessXShifted(a.p_offset),b.IHf_S.getSameSkewnessXShifted(b.p_offset),eps);
    // no dI in a.SK_movaver!
    //Skewness.matchSkewness(a.SK_movaver.getSameSkewnessXShifted(a.p_offset),b.movaver_S.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SK_IHf_W0.getSameSkewnessXShifted(a.p_offset),b.W0pi_S.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SK_IHf_WS.getSameSkewnessXShifted(a.p_offset),b.spur_S.getSameSkewnessXShifted(b.p_offset),eps);

    System.err.println("PnLdISk.matchPnLdISkewness test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}

}
