package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.EVSolver;

/**
 * Skewness is back.
 *  @version $Id: PnLdISkewness.java,v 1.67 2022/08/07 15:44:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdISkewness {


public final EVXData I;
    public final double p_offset,I_0_feq,p_IH,I_0_feqF;
    public final Skewness IHf_S,W0truepi_S,W0pi_S,W0pt_S,movaver_S,spur_S,SK_bbasis[];
PnLdISkewness(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,
	      final double I_0_feq,final double p_IH,
	      final Skewness [] SK_bbasis,final double [] bbasis,final double [] psik_psi0,
	      final Skewness W0truepi_S,
	      final Skewness movaver_S,
	      final Skewness spur_S,
	      final boolean add_dI,
	      final double eps)
{
    this.I=I;
    this.p_IH=p_IH;
    this.I_0_feq=I_0_feq;
    this.SK_bbasis=SK_bbasis;
    final int n=I.neq;
    this.p_offset=p_offset;
    this.I_0_feqF=bbasis[n-1];
    this.IHf_S=SK_bbasis[n-1];
    this.W0truepi_S=W0truepi_S;
    this.movaver_S=movaver_S;
    this.spur_S=spur_S;

    double m0pi0=0,m1pi0=0,m2pi0=0,m3pi0=0,m0pt0=0,m1pt0=0,m2pt0=0,m3pt0=0;
    for(int k=0;k<n;k++)
    {
	final Skewness s=SK_bbasis[k];
	final double w=Math.pow(psik_psi0[k],2),ik=s.w1+s.w2,
	    pi0=(s.w1+s.w2),pi1=(s.x1*s.w1+s.x2*s.w2),pi2=(Math.pow(s.x1,2)*s.w1+Math.pow(s.x2,2)*s.w2),
	    pi3=(Math.pow(s.x1,3)*s.w1+Math.pow(s.x2,3)*s.w2);
	m0pi0+=w*pi0;m1pi0+=w*pi1;m2pi0+=w*pi2;m3pi0+=w*pi3;
	m0pt0+=w*(pi0/ik);m1pt0+=w*(pi1/ik);m2pt0+=w*(pi2/ik);m3pt0+=w*(pi3/ik);
    }
    // For unit test, to match the PnLdISk.SK_IHf_W0 and the PnLdIE.{it,ti}.
    this.W0pi_S=Skewness.getSkewnessFromMonomialMoments(m0pi0,m1pi0,m2pi0,m3pi0);
    this.W0pt_S=Skewness.getSkewnessFromMonomialMoments(m0pt0*I_0_feq,m1pt0*I_0_feq,m2pt0*I_0_feq,m3pt0*I_0_feq);
    if(eps>0)
    { // unit test
	for(int k=0;k<n;k++) UnitTests.checkDoubleRel(SK_bbasis[k].w1+SK_bbasis[k].w2,"SK_bbasis[k].w1+SK_bbasis[k].w2",bbasis[k],"bbasis[k]",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(I_0_feq,"I_0_feq",TMatr.mulV2V(psik_psi0,java.util.Arrays.copyOf(bbasis,n)),"I_0_feq",eps);
	UnitTests.checkDoubleRel(W0pi_S.w1+W0pi_S.w2,"w1+w2",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(W0pt_S.w1+W0pt_S.w2,"w1+w2",I_0_feq,"I_0_feq",eps);
	System.err.println("PnLdISkewness: test OK n="+I.neq+" eps="+eps);
    }
}



/**
 * Solve <psi|I^f|psi>/<psi|psi>  -> max
 * extremum problem, where I^{f} if execution flow
 * operator (with or without impact from the future).
 * Then consider skewness (calculated from quadratures built) on:
 *  <psi|p^m I|psi> moments (price--space regular skewness).
 */
public static PnLdISkewness get_PnLdISkewness(final MomentsData mm,
					      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
					      final boolean add_dI,
					      final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double I_0=I.s0,I_0_feq=add_dI?I.sH:I.s0,dI=I_0_feq-I_0,
	p_last__with_p_offset_included=mm.p_last;

    final double [] bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp2i,I.ev),
	bQQp3i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp3i,I.ev),
	bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));

    // Create operators with an impact from the future.
    final double pl=p_last__with_p_offset_included;
    final double [] bQQ0=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),1/(I.o_o));
    TMatr.sumP1(bQQi,bQQ0,dI);
    TMatr.sumP1(bQQpi,bQQ0,dI*pl);
    TMatr.sumP1(bQQp2i,bQQ0,dI*Math.pow(pl,2));
    TMatr.sumP1(bQQp3i,bQQ0,dI*Math.pow(pl,3));

    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdISkewness.get_PnLdISkewness:I.ev",n,I.ev,mm.QQi,mm.QQt,eps); // old basis
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi0,bpsi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi,n),"<psi0|I|psi0>",I_0_feq,"I_0_feq",eps);
    }

    final double [] bbasis=M.EV.getEV(n,bQQi,true),
	psiC=EVXData.convertPsiFromQtoB_Basis(n,java.util.Arrays.copyOf(new double []{1/Math.sqrt(mm.QQt[0])},n),I.ev,mm.QQt);
    if(eps>0)
    {
	final int ixH=n-1;
	final double [] psi=EVXData.getPsiFromBasis(ixH,bbasis,n);
	EVXData.test_orthogonality("PnLdISkewness.get_PnLdISkewness:bbasis",n,bbasis,bQQi,TMatr.getUnitMatrix(n),eps); // new basis
	UnitTests.checkDoubleRel(bbasis[ixH],"bbasis[ixH]",TMatr.getAMatrA(psi,bQQi,n),"<psi|bQQi|psi>",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(psi,psi),"<psi|psi>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(psiC,psiC),"<psiC|psiC>",1,"1",eps);
	if(!add_dI)
	{
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiC,bQQi,n),"<psiC|I|psiC>",mm.QQi[0]/mm.QQt[0],"mm.QQi[0]/mm.QQt[0]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiC,bQQpi,n),"<psiC|pI|psiC>",mm.QQpi[0]/mm.QQt[0],"mm.QQpi[0]/mm.QQt[0]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiC,bQQp2i,n),"<psiC|p^2I|psiC>",mm.QQp2i[0]/mm.QQt[0],"mm.QQp2i[0]/mm.QQt[0]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psiC,bQQp3i,n),"<psiC|p^3I|psiC>",mm.QQp3i[0]/mm.QQt[0],"mm.QQp3i[0]/mm.QQt[0]",eps);
	}
    }
    final Skewness [] SK_bbasis=new Skewness [n];
    final double [] psik_psi0=new double [n];
    for(int k=0;k<n;k++)
    {
	SK_bbasis[k]=Skewness.getSkewnessFromMonomialMoments(
	    bbasis[k],
	    TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n),
	    TMatr.getAMatrA(n,bbasis,n+k*n,bQQp2i,n),
	    TMatr.getAMatrA(n,bbasis,n+k*n,bQQp3i,n));
	psik_psi0[k]=DeprecatedHelperTMatr.mulVVo(n,bbasis,n+k*n,bpsi0,0);
    }

    return new PnLdISkewness(I,mm.p_offset,p_last__with_p_offset_included,
			     I_0_feq,I.getFPsi(n-1,mm.QQpi)/I.sH,
			     SK_bbasis,bbasis,psik_psi0,
			     Skewness.getSkewnessFromMonomialMoments(TMatr.getAMatrA(bpsi0,bQQi,n),
								     TMatr.getAMatrA(bpsi0,bQQpi,n),
								     TMatr.getAMatrA(bpsi0,bQQp2i,n),
								     TMatr.getAMatrA(bpsi0,bQQp3i,n)),
			     Skewness.getSkewnessFromMonomialMoments(
				 TMatr.getAMatrA(psiC,bQQi,n),
				 TMatr.getAMatrA(psiC,bQQpi,n),
				 TMatr.getAMatrA(psiC,bQQp2i,n),
				 TMatr.getAMatrA(psiC,bQQp3i,n)),
			     Skewness.getSkewnessFromMonomialMoments(
				 TMatr.getSpurM(n,bQQi),
				 TMatr.getSpurM(n,bQQpi),
				 TMatr.getSpurM(n,bQQp2i),
				 TMatr.getSpurM(n,bQQp3i)),
			     add_dI,
			     eps);
}

static void matchPnLdISkewness(final PnLdISkewness a,final PnLdISkewness b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    Skewness.matchSkewness(a.IHf_S.getSameSkewnessXShifted(a.p_offset),b.IHf_S.getSameSkewnessXShifted(b.p_offset),eps);
    for(int k=0;k<a.I.neq;k++)Skewness.matchSkewness(a.SK_bbasis[k].getSameSkewnessXShifted(a.p_offset),b.SK_bbasis[k].getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.W0pi_S.getSameSkewnessXShifted(a.p_offset),b.W0pi_S.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.W0pt_S.getSameSkewnessXShifted(a.p_offset),b.W0pt_S.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.movaver_S.getSameSkewnessXShifted(a.p_offset),b.movaver_S.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.spur_S.getSameSkewnessXShifted(a.p_offset),b.spur_S.getSameSkewnessXShifted(b.p_offset),eps);

    System.err.println("PnLdISkewness.matchPnLdISkewness test OK n="+a.I.neq+" eps="+eps);
}

static void matchPnLdIDSk(final PnLdISkewness a,final PnLdIDSk b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    for(int k=0;k<a.I.neq;k++){
	Skewness.matchSkewness(a.SK_bbasis[k].getSameSkewnessXShifted(a.p_offset),b.SKs[k].sk.getSkewnessAt(b.p_last__with_p_offset_included).getSameSkewnessXShifted(b.p_offset),eps);
    }
    Skewness.matchSkewness(a.IHf_S.getSameSkewnessXShifted(a.p_offset),b.SKs[b.I.neq-1].sk.getSkewnessAt(b.p_last__with_p_offset_included).getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.spur_S.getSameSkewnessXShifted(a.p_offset),b.SKs_spur.sk.getSkewnessAt(b.p_last__with_p_offset_included).getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.W0pi_S.getSameSkewnessXShifted(a.p_offset),b.SKs_W0pi.sk.getSkewnessAt(b.p_last__with_p_offset_included).getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.W0pt_S.getSameSkewnessXShifted(a.p_offset),b.SKs_W0pt.sk.getSkewnessAt(b.p_last__with_p_offset_included).getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.W0truepi_S.getSameSkewnessXShifted(a.p_offset),b.SKs_W0truepi.sk.getSkewnessAt(b.p_last__with_p_offset_included).getSameSkewnessXShifted(b.p_offset),eps);
    System.err.println("PnLdISkewness.matchPnLdIDSk test OK n="+a.I.neq+" eps="+eps);
}

static void matchPnLdIDSk__nodI(final PnLdISkewness a,final PnLdIDSk b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
     for(int k=0;k<a.I.neq;k++){
	 // basis if different for dI=0. do not match
//	 Skewness.matchSkewness(a.SK_bbasis[k].getSameSkewnessXShifted(a.p_offset),b.SKs[k].sk.sknodi.getSameSkewnessXShifted(b.p_offset),eps);
     }
     Skewness.matchSkewness(a.movaver_S.getSameSkewnessXShifted(a.p_offset),b.SK_movaver__nodI.getSameSkewnessXShifted(b.p_offset),eps);
     Skewness.matchSkewness(a.spur_S.getSameSkewnessXShifted(a.p_offset),b.SK_spur__nodI.getSameSkewnessXShifted(b.p_offset),eps);
     Skewness.matchSkewness(a.spur_S.getSameSkewnessXShifted(a.p_offset),b.SKs_spur.sk.sknodi.getSameSkewnessXShifted(b.p_offset),eps);
     System.err.println("PnLdISkewness.matchPnLdIDSk__nodI test OK n="+a.I.neq+" eps="+eps);
}

static void matchPnLdIdI(final PnLdISkewness a,final PnLdIdI b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.I_0_feqF,"a.I_0_feqF",b.I_0_feqF,"b.I_0_feqF",eps);
    UnitTests.checkDoubleRel(a.I.sH,"a.I.sH",b.I_0_feq,"b.I_0_feq",eps);
    final int n=a.I.neq;
    for(int k=0;k<n;k++)
    {
	//Skewness.matchSkewness(a.spur_S.getSameSkewnessXShifted(a.p_offset),b.SKs[k].sknodi.getSameSkewnessXShifted(b.p_offset),eps);
    }
    System.err.println("PnLdISkewness.matchPnLdIdI test OK n="+a.I.neq+" eps="+eps);
}

static void matchDynVar2DISkewness(final PnLdISkewness a,final DynVar2DISkewness b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    Skewness.matchSkewness(a.IHf_S.getSameSkewnessXShifted(a.p_offset),b.IHf_S.getSameSkewnessXShifted(b.p_offset),eps);
    System.err.println("PnLdISkewness.DynVar2DISkewness test OK n="+a.I.neq+" eps="+eps);
}


static void matchDynVar2DIDP(final PnLdISkewness a,final DynVar2DIDP b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.I_0_feq,"a.I_0_feq",b.I_0_feq,"b.I_0_feq",eps);
    UnitTests.checkDoubleRel(a.spur_S.xa,"a.spur_S.xa",b.Pf_SPUR+p_offset_shift,"b.Pf_SPUR+p_offset_shift",eps);
    System.err.println("PnLdISkewness.matchDynVar2DIDP test OK n="+a.I.neq+" eps="+eps);
}
}
