package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.DiffSkewness;
import com.polytechnik.utils.SortArrays;
import com.polytechnik.utils.Skewness;

/**
 *  @version $Id: PnLdISpur.java,v 1.29 2022/08/07 15:44:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdISpur {




public final EVXData I;
public final double p_offset,p_last__with_p_offset_included,p_IH,I_0_feq,
    degt,Pft_from_f0,deg,Pf_from_f0,
    we_xx,wp_xx,Pf_from_xx;
public final DiffSkewness PnLsk;
final Skewness W0_SK,W0w_SK;
PnLdISpur(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,
	  final double p_IH,final double I_0_feq,
	  final double [] psik_psi0,
	  final double [] psik_I_psik,final double [] psik_pI_psik,
	  final double [] psik_p2I_psik,final double [] psik_p3I_psik,
	  final double [] bbasis,final double [] bQQpi,
	  final double psi0_pI_psi0,
	  final double eps)
{
    this.I=I;
    this.p_offset=p_offset;
    this.p_last__with_p_offset_included=p_last__with_p_offset_included;
    this.p_IH=p_IH;
    this.I_0_feq=I_0_feq;
    final int n=I.neq;
    final double dI=I_0_feq-I.s0,wPnLk[]=new double [n],w0[]=new double [n],w0r[]=new double [n],w0ir[]=new double [n];
    for(int k=0;k<n;k++)
    {
	w0[k]=Math.pow(psik_psi0[k],2);
	w0r[k]=w0[k]*(I_0_feq/bbasis[k]);
	wPnLk[k]=Math.min(1,(dI/psik_I_psik[k])*w0[k]);
	w0ir[k]=w0[k]/psik_I_psik[k];
	if(eps>0)
	{
	    UnitTests.checkDoubleRel(bbasis[k],"bbasis[k]",dI*w0[k]+psik_I_psik[k],"dI*w0[k]+psik_I_psik[k]",eps);
	}
    }
    this.PnLsk=DiffSkewness.getDiffSkewness_FromMonomials(
	TMatr.mulVV(wPnLk,psik_I_psik),
	TMatr.mulVV(wPnLk,psik_pI_psik),
	TMatr.mulVV(wPnLk,psik_p2I_psik),
	TMatr.mulVV(wPnLk,psik_p3I_psik),
	TMatr.mulVV(wPnLk,psik_I_psik));
    if(eps>0)
    {
	if(PnLsk.di/dI>1+eps){throw new RuntimeException("PnLsk.di="+PnLsk.di+" dI="+dI);}
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psi0),"sum(<psik|psi0>^2)",1,"1",eps);
	UnitTests.checkDoubleRel(PnLsk.di,"PnLsk.di",PnLsk.sknodi.w1+PnLsk.sknodi.w2,"PnLsk.sknodi.w1+PnLsk.sknodi.w2",eps);
	double sw=0,spw=0;
	for(int k=0;k<n;k++)
	{
	    final double wik=Math.min(dI*w0[k],psik_I_psik[k]);
	    sw+=wik;
	    spw+=(psik_pI_psik[k]/psik_I_psik[k])*wik;
	}
	UnitTests.checkDoubleRel(PnLsk.sknodi.xa,"PnLsk.sknodi.xa",spw/sw,"spw/sw",eps);
	UnitTests.checkDoubleRel(sw,"sw",PnLsk.sknodi.w1+PnLsk.sknodi.w2,"PnLsk.sknodi.w1+PnLsk.sknodi.w2",eps);
    }
    this.degt=1-TMatr.mulVV(w0,w0r);
    this.Pft_from_f0=(-psi0_pI_psi0+TMatr.mulVV(w0r,psik_pI_psik))/(degt*dI);
    this.deg=1-TMatr.mulVV(w0,w0);
    this.Pf_from_f0=(-psi0_pI_psi0+TMatr.mulVV(w0,psik_pI_psik))/(deg*dI);

    final int [] ind=SortArrays.sortArrayIndexes(w0);
    final int i0_p=ind[n-2],i0_e=ind[n-1]; // top 2 max psik(x0)^2
    this.we_xx=w0[i0_e];
    this.wp_xx=w0[i0_p];
    this.Pf_from_xx=-TMatr.getAMatrB(n,bbasis,n+i0_p*n,bbasis,n+i0_e*n,bQQpi,n)/((psik_psi0[i0_p]*psik_psi0[i0_e])*dI);

    this.W0_SK=Skewness.getSkewnessFromMonomialMoments(
	TMatr.mulVV(w0ir,psik_I_psik),
	TMatr.mulVV(w0ir,psik_pI_psik),
	TMatr.mulVV(w0ir,psik_p2I_psik),
	TMatr.mulVV(w0ir,psik_p3I_psik));
    this.W0w_SK=Skewness.getSkewnessFromMonomialMoments(
	TMatr.mulVVV(w0ir,w0,psik_I_psik),
	TMatr.mulVVV(w0ir,w0,psik_pI_psik),
	TMatr.mulVVV(w0ir,w0,psik_p2I_psik),
	TMatr.mulVVV(w0ir,w0,psik_p3I_psik));
    if(eps>0)
    {
	UnitTests.checkDoubleRel(W0_SK.w1+W0_SK.w2,"1",1,"1",eps);
	UnitTests.checkDoubleRel(W0w_SK.w1+W0w_SK.w2,"<w>",TMatr.mulVV(w0,w0),"<w>",eps);
    }
    if(eps>0)
    {
	final double [] bQQi=TMatr.getDiagMatrix(I.getEValues());
	UnitTests.checkDoubleRel(-TMatr.getAMatrB(n,bbasis,n+i0_p*n,bbasis,n+i0_e*n,bQQi,n)/((psik_psi0[i0_p]*psik_psi0[i0_e])*dI),"1",1,"1",eps);
	System.err.println("PnLdISpur.PnLdISpur selftest2 OK n="+n+" deg="+deg+" degt="+degt+" w0="+java.util.Arrays.toString(w0)+" inds=("+i0_p+","+i0_e+") sum_xx="+(w0[i0_p]+w0[i0_e])+" eps="+eps);
    }
}

/**
 * Solve <psi|I|psi>/<psi|psi>=lambda extremum problem, find eigenstates & eigenvalues.
 * find psi^{IH} and lambda^{IH}=max lambda, used as an estimator of
 * the level of "future" execution.
 * Then dI=lambda^{IH}-<psi0|I|psi0> (contribution from the future, of not yet executed, trades).
 * Then operator I^{f} (future I) is: I^{f}= I + |psi0>dI<psi0|
 * Find the eigenstates of this new I^{f} matrix,
 * in dynamic impact approximation its eigenstates should be also the P^{f} operator eigenstates
 * (for the states of hight I).
 * Then consider P&L = sum  (P^f - <psik|pI|psik>/<psik|pI|psik>)*min(<psik|I|psik>,dI*<psik|psi0>^2)
 * and its assymetry.
 *
 */
public static PnLdISpur get_PnLdISpur(final MomentsData mm,
				      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				      final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final int ixH=n-1; // the state with maximal I
    final double I_0=I.s0,I_0_feq=I.ev[ixH],dI=I_0_feq-I_0;

    // Create bQQi_feq (operator of "future" I), with the |bpsi0>dI<bpsi0| contribution from the future:
    // bQQi_feq = bQQi + |bpsi0>dI<bpsi0|
    final double [] bQQi_feq=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o)),bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));
    for(int k=0;k<n;k++) bQQi_feq[k+n*k]+=I.ev[k];

    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp2i,I.ev),
	bQQp3i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp3i,I.ev);

    // Diagonalize "future" I:
    final double [] bbasis=M.EV.getEV(n,bQQi_feq,true),
	psik_psi0=new double [n],psik_I_psik=new double[n],
	psik_pI_psik=new double[n],psik_p2I_psik=new double[n],psik_p3I_psik=new double[n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n);

	psik_psi0[k]=TMatr.mulVV(psik,bpsi0);
	psik_I_psik[k]=TMatr.mulV2V(psik,I.getEValues());
	psik_pI_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n);
	psik_p2I_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQp2i,n);
	psik_p3I_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQp3i,n);
    }

    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdISpur.get_PnLdISpur:I.ev",n,I.ev,mm.QQi,mm.QQt,eps); // old basis
	EVXData.test_orthogonality("PnLdISpur.get_PnLdISpur:bbasis",n,bbasis,bQQi_feq,TMatr.getUnitMatrix(n),eps); // new basis
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQpi),"Spur(bQQpi)",TMatr.sumElements(psik_pI_psik),"sum(psik_pI_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQp2i),"Spur(bQQp2i)",TMatr.sumElements(psik_p2I_psik),"sum(psik_p2I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQp3i),"Spur(bQQp3i)",TMatr.sumElements(psik_p3I_psik),"sum(psik_p3I_psik)",eps);

	System.err.println("PnLdISpur.get_PnLdISpur selftest OK n="+n+" eps="+eps);
    }
    return new PnLdISpur(I,mm.p_offset,mm.p_last,
			 bQQpi[ixH+n*ixH]/I.ev[ixH],I_0_feq,
			 psik_psi0,
			 psik_I_psik,psik_pI_psik,
			 psik_p2I_psik,psik_p3I_psik,bbasis,bQQpi,
			 TMatr.getAMatrA(bpsi0,bQQpi,n),
			 eps);


}
}
