package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.EVSolver;

/**
 * Skewness is back. See also our old skewness approaches in RkiQVM,DynSki and QkQVM,EAssym.
 *  @version $Id: PnLdIV4.java,v 1.30 2022/08/07 15:44:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdIV4 {

    public final EVXData I;
    public final double p_offset;
    public final Skewness SK_init,SK_iter,SK_spur;
    PnLdIV4(final EVXData I,final double p_offset,
	    final Skewness SK_init,
	    final Skewness SK_iter,
	    final Skewness SK_spur)
    {
	this.I=I;
	this.p_offset=p_offset;
	this.SK_init=SK_init;
	this.SK_iter=SK_iter;
	this.SK_spur=SK_spur;
    }


public static PnLdIV4 get_PnLdIV4_QQ(final int n,
				     final double p_offset,
				     final double [] QQt,
				     final double [] QQi,
				     final double [] QQpi,
				     final double [] QQp2i,
				     final double [] QQp3i,
				     final double [] QQp4i,
				     final boolean add_dI,
				     final double p_last__with_p_offset_included,
				     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				     final double eps)
{
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double I_0=I.s0,I_0_feq=I.sH,dI=I_0_feq-I_0;
    final double [] bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,QQp2i,I.ev),
	bQQp3i=EVXData.convertMatrFromQtoB_Basis(n,QQp3i,I.ev),
	bQQp4i=EVXData.convertMatrFromQtoB_Basis(n,QQp4i,I.ev);

    // Create operators with an impact from the future.
    if(add_dI)
    {
	final double pl=p_last__with_p_offset_included;
	final double [] bQQ0=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o));
	TMatr.sumP1(bQQi,bQQ0,1);
	TMatr.sumP1(bQQpi,bQQ0,pl);
	TMatr.sumP1(bQQp2i,bQQ0,Math.pow(pl,2));
	TMatr.sumP1(bQQp3i,bQQ0,Math.pow(pl,3));
	TMatr.sumP1(bQQp4i,bQQ0,Math.pow(pl,4));
    }
    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdIV4.get_PnLdIV4:I.ev",n,I.ev,QQi,QQt,eps); // old basis
    }
    return get_PnLdIV4(I,p_offset,new double [][]{bQQi,bQQpi,bQQp2i,bQQp3i,bQQp4i},M.EV,eps);
}

/**
 * Solve min_{c1,c0} max_{psi} <psi|(p^2+c1*p+c0)^2 I|psi>/<psi|psi>
 * extremum problem, find eigenstates & eigenvalues.
 * Then consider skewness for quadratures built on <psi|p^m I|psi> moments.
 * The approach is similar to the one of DynYp.java,
 * but now the quadrature is of two nodes.
 */
public static PnLdIV4 get_PnLdIV4(final EVXData I,
				  final double p_offset,
				  final double [][] bQQpmi_matrices,
				  final EVSolver MEV,
				  final double eps)
{
    final int n=I.neq,N_Iterations=15;
    if(n<2||bQQpmi_matrices.length!=5) throw new RuntimeException("n="+n+"<2");
    final int ixH=n-1; // the state with maximal I
    final double [] bQQ=new double [n*n],m=new double [bQQpmi_matrices.length],km=new double [bQQpmi_matrices.length];
    km[0]=1; // bQQi is initial matrix
    Skewness SK_init=null,SK_iter=null;
    for(int iter=1;iter<=N_Iterations;iter++)
    {
	for(int k=0;k<bQQ.length;k++)
	{
	    double s=0;for(int j=km.length;--j>=0;){s+=km[j]*bQQpmi_matrices[j][k];}bQQ[k]=s;
	}
	final double [] ev=MEV.getEV(n,bQQ,true);
	for(int j=m.length;--j>=0;){m[j]=TMatr.getAMatrA(n,ev,n+ixH*n,bQQpmi_matrices[j],n);}
	final Skewness sk=Skewness.getSkewnessFromMonomialMoments(m[0],m[1],m[2],m[3]);
	final double c1=-(sk.x1+sk.x2),c0=sk.x1*sk.x2;
	km[0]=c0*c0;km[1]=2*c1*c0;km[2]=(2*c0+c1*c1);km[3]=2*c1;km[4]=1;
	if(eps>0)
	{
	    System.err.println("iter="+iter+" ev[ixH]="+ev[ixH]+" min_{p1,p2}="+TMatr.mulVV(km,m)+" m[0]="+m[0]);
	    EVXData.test_orthogonality("PnLdIV4.get_PnLdIV4:ev",n,ev,bQQ,TMatr.getUnitMatrix(n),eps); // new basis
	    UnitTests.checkDoubleRel(m[0],"m[0]",sk.w1+sk.w2,"sk.w1+sk.w2",eps);
	    final com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis MON=
		new com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis();
	    final double [] p=MON.getSingleOrthogonalPolynomial(2,m);
	    if(p[2]<0) TMatr.scaleVectorInPlace(p,-1);
	    UnitTests.checkDoubleArrayRel(new double []{c0,c1,1},"{c0,c1,1}",p,"p",eps);
	    UnitTests.checkDoubleArrayRel(km,"km",MON.B.mult2Pol(p,p),"p*p",eps);
	}
	if(iter==1){SK_init=sk;}
	SK_iter=sk;
    }
    return new PnLdIV4(I,p_offset,
		       SK_init,SK_iter,
		       Skewness.getSkewnessFromMonomialMoments(
			   TMatr.getSpurM(n,bQQpmi_matrices[0]),
			   TMatr.getSpurM(n,bQQpmi_matrices[1]),
			   TMatr.getSpurM(n,bQQpmi_matrices[2]),
			   TMatr.getSpurM(n,bQQpmi_matrices[3]))
		       );
}

static void matchPnLdICorr(final PnLdIV4 a,final PnLdICorr b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    Skewness.matchSkewness(a.SK_init.getSameSkewnessXShifted(a.p_offset),b.IHf_cc.SK_pv.getSameSkewnessXShifted(b.p_offset),eps);
    Skewness.matchSkewness(a.SK_spur.getSameSkewnessXShifted(a.p_offset),b.spur_cc.SK_pv.getSameSkewnessXShifted(b.p_offset),eps);

    System.err.println("PnLdIV4.matchPnLdICorr test OK n="+a.I.neq+" eps="+eps);
}

}
