package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2017
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.Skewness;

/**
 * Skewness is back. See also PnLdISk.java.
 *  @version $Id: PnLdIVSk.java,v 1.23 2022/07/28 21:03:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdIVSk {

    public final EVXData I;
    public final double p_offset,p_IH,I_0_feq;
    public final Skewness SK_IHf,SK_IHf_V,SK_IH,SK_IH_V,SK_movaver,SK_IHf_W0,SK_IHf_WS,SK_IHf_Pfeq;
    PnLdIVSk(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,
	     final double p_IH,final double I_0_feq,
	     final Skewness SK_IHf,
	     final Skewness SK_IHf_V,
	     final Skewness SK_IH,
	     final Skewness SK_IH_V,
	     final Skewness SK_movaver,
	     final Skewness SK_IHf_W0,
	     final Skewness SK_IHf_WS,
	     final Skewness SK_IHf_Pfeq)
    {
	this.I=I;
	this.p_offset=p_offset;
	this.p_IH=p_IH;
	this.I_0_feq=I_0_feq;
	this.SK_IHf=SK_IHf;
	this.SK_IHf_V=SK_IHf_V;
	this.SK_IH=SK_IH;
	this.SK_IH_V=SK_IH_V;
	this.SK_movaver=SK_movaver;
	this.SK_IHf_W0=SK_IHf_W0;
	this.SK_IHf_WS=SK_IHf_WS;
	this.SK_IHf_Pfeq=SK_IHf_Pfeq;
    }

/**
 * Solve <psi|I|psi>/<psi|psi>=lambda extremum problem, find eigenstates & eigenvalues.
 * find psi^{IH} and lambda^{IH}=max lambda, used as an estimator of
 * the level of "future" execution.
 * Then dI=lambda^{IH}-<psi0|I|psi0> (contribution from the future, of not yet executed, trades).
 * Then operator I^{f} (future I) is: I^{f}= I + |psi0>dI<psi0|
 * Find the eigenstates of this new I^{f} matrix.
 * Then consider Volume and Capital skewness for the state of max I.
 * The class is used to unit--test PnLdISk.
 */
public static PnLdIVSk get_PnLdIVSk(final int n,
				    final double [] tmom,
				    final double [] imom,
				    final double [] pimom,
				    final double [] p2imom,
				    final double [] p3imom,
				    final double p_offset,final double p_last__with_p_offset_included,
				    final java.util.function.Function<double [],double []> intg,
				    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				    final double eps)
{
    final double [] QQt=M.B.getQQMatr(n,tmom),QQi=M.B.getQQMatr(n,imom);
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double I_0=I.s0,I_0_feq=I.sH,dI=I_0_feq-I_0;
    final double [] QQ0=TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),QQi_feq=TMatr.sumAcB(QQi,dI/I.o_o,QQ0);
    final EVXData If=EVXData.getEVXDataFromQQ(n,QQi_feq,QQt,M);
    final double dv=dI/I.o_o,pl=p_last__with_p_offset_included;
    final double [] psi2fw_intdt=intg.apply(If.psiH),psi2f=M.B.mult2Pol(If.psiH,If.psiH),
	psif_psi0=M.B.mult2Pol(If.psiH,If.psi0);
    final int nmoms=psi2f.length;
    if(nmoms!=psi2fw_intdt.length||nmoms!=psi2f.length) throw new RuntimeException();
    final double [] o=M.B.getX0Moments(nmoms,I.x0),
	imomf=TMatr.sumAcB(java.util.Arrays.copyOf(imom,nmoms),dv,o),
	pimomf=TMatr.sumAcB(java.util.Arrays.copyOf(pimom,nmoms),dv*pl,o),
	p2imomf=TMatr.sumAcB(java.util.Arrays.copyOf(p2imom,nmoms),dv*Math.pow(pl,2),o),
	p3imomf=TMatr.sumAcB(java.util.Arrays.copyOf(p3imom,nmoms),dv*Math.pow(pl,3),o);
    final double psif_psi0_w=TMatr.mulVVn1(psif_psi0,tmom),wf0=psif_psi0_w*psif_psi0_w,
	dI_p0eq=(TMatr.mulVVn1(psi2f,imom)-TMatr.mulVVn1(psif_psi0,imom)/psif_psi0_w)/(1-wf0),
	dI_p1eq=(TMatr.mulVVn1(psi2f,pimom)-TMatr.mulVVn1(psif_psi0,pimom)/psif_psi0_w)/(1-wf0),
	dI_p2eq=(TMatr.mulVVn1(psi2f,p2imom)-TMatr.mulVVn1(psif_psi0,p2imom)/psif_psi0_w)/(1-wf0),
	dI_p3eq=(TMatr.mulVVn1(psi2f,p3imom)-TMatr.mulVVn1(psif_psi0,p3imom)/psif_psi0_w)/(1-wf0);
    final double []
	imomF=TMatr.sumAcB(java.util.Arrays.copyOf(imom,nmoms),dI_p0eq/If.o_o,o),
	pimomF=TMatr.sumAcB(java.util.Arrays.copyOf(pimom,nmoms),dI_p1eq/If.o_o,o),
	p2imomF=TMatr.sumAcB(java.util.Arrays.copyOf(p2imom,nmoms),dI_p2eq/If.o_o,o),
	p3imomF=TMatr.sumAcB(java.util.Arrays.copyOf(p3imom,nmoms),dI_p3eq/If.o_o,o);


    double mOw0=0,mOw1=0,mOw2=0,mOw3=0,mSw0=0,mSw1=0,mSw2=0,mSw3=0;
    for(int k=0;k<n;k++)
    {
	final double [] psik=If.getPsi(k),psi2k=M.B.mult2Pol(psik,psik);
	final double w0=Math.pow(TMatr.getAMatrB(psik,If.psi0,QQt,n),2),m0=TMatr.mulVVn1(psi2k,imomf),m1=TMatr.mulVVn1(psi2k,pimomf),m2=TMatr.mulVVn1(psi2k,p2imomf),m3=TMatr.mulVVn1(psi2k,p3imomf);
	mOw0+=w0*m0;
	mOw1+=w0*m1;
	mOw2+=w0*m2;
	mOw3+=w0*m3;
	mSw0+=m0;
	mSw1+=m1;
	mSw2+=m2;
	mSw3+=m3;
    }
    final Skewness SK_IHf_W0=Skewness.getSkewnessFromMonomialMoments(mOw0,mOw1,mOw2,mOw3),
	SK_IHf_WS=Skewness.getSkewnessFromMonomialMoments(mSw0,mSw1,mSw2,mSw3);

    final double [] psi2w_intdt=intg.apply(I.psiH),psi2=M.B.mult2Pol(I.psiH,I.psiH);
    if(nmoms!=psi2w_intdt.length||nmoms!=psi2.length) throw new RuntimeException();

    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdIVSk.get_PnLdIVSk:I.ev",n,I.ev,QQi,QQt,eps); // old basis
	EVXData.test_orthogonality("PnLdIVSk.get_PnLdIVSk:If.ev",n,If.ev,QQi_feq,QQt,eps); // new basis

	UnitTests.checkDoubleRel(TMatr.mulVV(psi2fw_intdt,o),"<psiHf|psiHf>",1,"1",eps);
	UnitTests.checkDoubleRel(M.B.getD0(psi2fw_intdt,If.x0),"psi2fw_intdt(x0)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulVVn1(psi2f,imomf),"<psiHf|If|psiHf>",If.sH,"If.sH",eps);
	UnitTests.checkDoubleRel(TMatr.mulVVn1(M.B.mult2Pol(I.psi0,I.psi0),imomf),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);

	UnitTests.checkDoubleRel(TMatr.mulVVn1(psi2f,imomF),"<psiHf|If|psiHf>",If.sH,"If.sH",eps);
	UnitTests.checkDoubleRel(TMatr.mulVVn1(M.B.mult2Pol(I.psi0,I.psi0),imomF),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.mulVVn1(psif_psi0,imomF),"<psiHf|If|psi0>",If.sH*psif_psi0_w,"If.sH*psif_psi0_w",eps);
	UnitTests.checkDoubleRel(TMatr.mulVVn1(psif_psi0,pimomF),"<psiHf|pIf|psi0>",TMatr.mulVVn1(psi2f,pimomF)*psif_psi0_w,"<psiHf|pIf|psiHf>*psif_psi0_w",eps);
	UnitTests.checkDoubleRel(TMatr.mulVVn1(psif_psi0,p2imomF),"<psiHf|p^2If|psi0>",TMatr.mulVVn1(psi2f,p2imomF)*psif_psi0_w,"<psiHf|p^2If|psiHf>*psif_psi0_w",eps);
	UnitTests.checkDoubleRel(TMatr.mulVVn1(psif_psi0,p3imomF),"<psiHf|p^3If|psi0>",TMatr.mulVVn1(psi2f,p3imomF)*psif_psi0_w,"<psiHf|p^3If|psiHf>*psif_psi0_w",eps);

	UnitTests.checkDoubleRel(TMatr.mulVV(psi2w_intdt,o),"<psiH|psiH>",1,"1",eps);
	UnitTests.checkDoubleRel(M.B.getD0(psi2w_intdt,I.x0),"psi2w_intdt(x0)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulVVn1(psi2,imom),"<psiH|I|psiH>",I.sH,"I.sH",eps);
	UnitTests.checkDoubleRel(TMatr.mulVVn1(M.B.mult2Pol(I.psi0,I.psi0),imom),"<psi0|I|psi0>",I.s0,"I.s0",eps);

	System.err.println("PnLdISVk.get_PnLdISVk selftest OK n="+n+" eps="+eps);
    }
    final double [] psi2a=new double []{1/tmom[0]};

    return new PnLdIVSk(I,p_offset,p_last__with_p_offset_included,TMatr.mulVVn1(psi2,pimom)/I.sH,I_0_feq,
			Skewness.getSkewnessFromMonomialMoments(TMatr.mulVVn1(psi2f,imomf),TMatr.mulVVn1(psi2f,pimomf),TMatr.mulVVn1(psi2f,p2imomf),TMatr.mulVVn1(psi2f,p3imomf)),
			Skewness.getSkewnessFromMonomialMoments(TMatr.mulVVn1(psi2fw_intdt,imomf),TMatr.mulVVn1(psi2fw_intdt,pimomf),TMatr.mulVVn1(psi2fw_intdt,p2imomf),TMatr.mulVVn1(psi2fw_intdt,p3imomf)),
			Skewness.getSkewnessFromMonomialMoments(TMatr.mulVVn1(psi2,imom),TMatr.mulVVn1(psi2,pimom),TMatr.mulVVn1(psi2,p2imom),TMatr.mulVVn1(psi2,p3imom)),
			Skewness.getSkewnessFromMonomialMoments(TMatr.mulVVn1(psi2w_intdt,imom),TMatr.mulVVn1(psi2w_intdt,pimom),TMatr.mulVVn1(psi2w_intdt,p2imom),TMatr.mulVVn1(psi2w_intdt,p3imom)),
			Skewness.getSkewnessFromMonomialMoments(TMatr.mulVVn1(psi2a,imom),TMatr.mulVVn1(psi2a,pimom),TMatr.mulVVn1(psi2a,p2imom),TMatr.mulVVn1(psi2a,p3imom)),
			SK_IHf_W0,SK_IHf_WS,
			Skewness.getSkewnessFromMonomialMoments(TMatr.mulVVn1(psi2f,imomF),TMatr.mulVVn1(psi2f,pimomF),TMatr.mulVVn1(psi2f,p2imomF),TMatr.mulVVn1(psi2f,p3imomF)));
}
}
