package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.DiffSkewness;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.DerivativesCalculateable;

/**
 * Parameter--dependent orthogonal system is back.
 * @version $Id: PnLdIdI.java,v 1.29 2022/08/05 11:25:33 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdIdI {


public final EVXData I;
public final double p_offset,p_last__with_p_offset_included,p_IH,I_0_feq,I_0_feqF,sDir,p_MUSE;
public final DiffSkewness SKs[];
PnLdIdI(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,
	final double p_IH,final double I_0_feq,final double I_0_feqF,
	final DiffSkewness [] SKs,final double [] psik_psi0,final double [] bfbasis,
	final double eps)
{
    final int n=I.neq;
    this.I=I;
    this.SKs=SKs;
    this.p_offset=p_offset;
    this.p_last__with_p_offset_included=p_last__with_p_offset_included;
    this.p_IH=p_IH;
    this.I_0_feq=I_0_feq;
    this.I_0_feqF=I_0_feqF;

    double sDir=0,smuse=0;
    for(DiffSkewness sk:SKs)
    {
	final DerivativesCalculateable _pp[]=sk.getX1X2(),P1=_pp[0],P2=_pp[1];
	final double om1=P1.getD2F(sk.sknodi.x2), // d^2p1/dp^2|_{p2}
	    om2=P2.getD2F(sk.sknodi.x1); // d^2p2/dp^2|_{p1}
	if(eps>0)
	{
	    final Skewness st=sk.sknodi;
	    UnitTests.checkDoubleRel(P1.getF(st.x1),"P1(x1)",st.x1,"st.x1",eps);
	    UnitTests.checkDoubleRel(P1.getF(st.x2),"P1(x2)",st.x1,"st.x1",eps);
	    UnitTests.checkDoubleRel(P2.getF(st.x1),"P2(x1)",st.x2,"st.x2",eps);
	    UnitTests.checkDoubleRel(P2.getF(st.x2),"P2(x2)",st.x2,"st.x2",eps);
	    UnitTests.checkDoubleRel(P1.getDF(st.x2),"dp1/dp2",0,"0",eps);
	    UnitTests.checkDoubleRel(P2.getDF(st.x1),"dp2/dp1",0,"0",eps);
	    if(om1<0) throw new RuntimeException("om1="+om1);
	    if(om2>0) throw new RuntimeException("om2="+om2);
	}
	final double f=om1+om2;
	sDir+=f;
	smuse+=sk.di*(sk.sknodi.x1+sk.sknodi.x2)/2;
    }
    this.sDir=sDir;
    this.p_MUSE=smuse/(I_0_feqF-I_0_feq);
    if(eps>0)
    { // unit test
	double sdIF=0;
	for(int k=0;k<n;k++)
	{
	    final DiffSkewness sk=SKs[k];
	    sdIF+=sk.di;
	    UnitTests.checkDoubleRel(sk.sknodi.w1+sk.sknodi.w2+sk.di,"sk.sknodi.w1+sk.sknodi.w2+sk.di",bfbasis[k],"bfbasis[k]",eps);
	}
	UnitTests.checkDoubleRel(sdIF,"sdIF",I_0_feqF-I_0_feq,"I_0_feqF-I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(psik_psi0,java.util.Arrays.copyOf(bfbasis,n)),"<psi0|I^ff|psi0>",I_0_feqF,"I_0_feqF",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psi0),"<psi0|psi0>",1,"1",eps);
	System.err.println("PnLdIdI: test OK n="+n+" dI="+(I_0_feq-I.s0)+" dIF="+(I_0_feqF-I_0_feq)+" eps="+eps);
    }
}



/**
 * Consider
 * ||p^mI^{ff}||= ||p^mI|| + {@literal |psi0>}(P^last)^m dI{@literal <psi0|} +{@literal |psi0>}(P^f)^m dI'{@literal <psi0|}
 * first find dI, then dI'.
 * The dI=I_0_feq-I_0, where I_0_feq is estimated as max ||I||.
 * The dI'=I_0_feqF-I_0_feq, where where I_0_feqF is estimated as max ||I^{f}||=||I||+{@literal |psi0>dI<psi0|}.
 * (Note that dI'=0 if {@literal |psi0>} is ||I^{f}|| eigenfunction).
 * After that consifer parameter--dependent (P^f) skewness charactersistics.
 */
public static PnLdIdI get_PnLdIdI(final MomentsData mm,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				  final double eps)
{
    final int n=mm.n,ixH=n-1; // the state with maximal I
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double I_0=I.s0,I_0_feq=I.sH,dI=I_0_feq-I_0,
	p_last__with_p_offset_included=mm.p_last;

    final double [] bQQi=TMatr.getDiagMatrix(I.getEValues()),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp2i,I.ev),
	bQQp3i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp3i,I.ev),
	bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));

    // Create operators with an impact from the future.
    final double pl=p_last__with_p_offset_included,p_IH=bQQpi[ixH+n*ixH]/I.ev[ixH];
    final double [] bQQ0=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),1/(I.o_o));
    TMatr.sumP1(bQQi,bQQ0,dI);
    TMatr.sumP1(bQQpi,bQQ0,dI*pl);
    TMatr.sumP1(bQQp2i,bQQ0,dI*Math.pow(pl,2));
    TMatr.sumP1(bQQp3i,bQQ0,dI*Math.pow(pl,3));
    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdIdI.get_PnLdIdI:I.ev",n,I.ev,mm.QQi,mm.QQt,eps); // old basis
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi0,bpsi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi,n),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);
    }

    final double I_0_feqF=M.EV.getEV(n,bQQi,false)[ixH],dIF=I_0_feqF-I_0_feq;
    final double [] bQQi_feq=TMatr.sumAcB(bQQi,dIF,bQQ0),bfbasis=M.EV.getEV(n,bQQi_feq,true),psik_psi0=new double [n];
    final DiffSkewness [] SKs=new DiffSkewness [n];
    for(int k=0;k<n;k++)
    {
	psik_psi0[k]=DeprecatedHelperTMatr.mulVVo(n,bfbasis,n+k*n,bpsi0,0);
	SKs[k]=DiffSkewness.getDiffSkewness_FromMonomials(
	    TMatr.getAMatrA(n,bfbasis,n+k*n,bQQi,n),
	    TMatr.getAMatrA(n,bfbasis,n+k*n,bQQpi,n),
	    TMatr.getAMatrA(n,bfbasis,n+k*n,bQQp2i,n),
	    TMatr.getAMatrA(n,bfbasis,n+k*n,bQQp3i,n),
	    dIF*Math.pow(psik_psi0[k],2));
    }
    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdIdI.get_PnLdIdI:bfbasis",n,bfbasis,bQQi_feq,TMatr.getUnitMatrix(n),eps); // new basis
	UnitTests.checkDoubleRel(TMatr.mulV2V(bpsi0,I.getEValues()),"<psi0|I|psi0>",I.s0,"I.s0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi,n),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_feq,n),"<psi0|Iff|psi0>",I_0_feqF,"I_0_feqF",eps);
    }

    return new PnLdIdI(I,mm.p_offset,p_last__with_p_offset_included,
		       p_IH,I_0_feq,I_0_feqF,
		       SKs,psik_psi0,bfbasis,eps);
}




}
