package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: PnLdIdP.java,v 1.53 2022/08/07 15:44:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdIdP {

    public final EVXData I;
    public final double p_offset,p_IH,p_IHf,p_IHf_nodI,dp_IH,dp_IHf,dp_0,I_0_feq,P_movaver,P_movaver_spur,
	dP_movaver,dP_movaver_spur,dP_w0,dP_wIH,dC_w0,
	dPf_0,dP_IH_IHf,pp_IHf;
    PnLdIdP(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,final double P_movaver,final double dP_movaver,
	    final double p_IH,final double dp_IH,final double dp_0,final double I_0_feq,
	    final double [] psik_psi0,final double [] psik_I_psik,final double [] psik_pI_psik,final double [] psik_dP_psik,final double [] bbasis,final double [] psik_psiIH,final double [] psik_pI_psiIH)
    {
	this.I=I;
	this.p_offset=p_offset;
	this.P_movaver=P_movaver;
	this.P_movaver_spur=TMatr.sumElements(psik_pI_psik)/TMatr.sumElements(psik_I_psik);
	this.dP_movaver=dP_movaver;
	this.dP_movaver_spur=TMatr.sumElements(psik_dP_psik)/psik_dP_psik.length;
	this.p_IH=p_IH;
	this.dp_IH=dp_IH;
	this.dp_0=dp_0;
	this.I_0_feq=I_0_feq;
	final double dI=I_0_feq-I.s0;
	final int n=I.neq,ixH=n-1;
	final double [] w0=TMatr.squareArrayElements(psik_psi0),wIH=TMatr.squareArrayElements(psik_psiIH),
	    wdI=TMatr.sumAcB(java.util.Arrays.copyOf(bbasis,n),-1,psik_I_psik);
	this.dP_w0=TMatr.mulVV(w0,psik_dP_psik);
	this.dP_wIH=TMatr.mulVV(wIH,psik_dP_psik);
	this.dC_w0=TMatr.mulVV(wdI,psik_dP_psik); // to match dP_w0*dI
	//System.err.println("sum(wdI)="+TMatr.sumElements(wdI)+" dI="+dI+" dP_w0="+dP_w0+" dC_wdI/dI="+(dC_wdI/dI));
	this.dPf_0=-dP_w0/TMatr.mulVV(w0,w0);
	this.dp_IHf=psik_dP_psik[ixH];
	this.dP_IH_IHf=(dp_IHf-dp_IH)/w0[ixH];
	//System.err.println("PnLdIdP: wHf="+w0[ixH]+" wH="+Math.pow(I.wH,2));if(w0[ixH]<Math.pow(I.wH,2)) throw new RuntimeException();
	pp_IHf=psik_pI_psiIH[ixH]/(I.sH*psik_psiIH[ixH]);
	p_IHf=(psik_pI_psik[ixH]+p_last__with_p_offset_included*dI*w0[ixH])/bbasis[ixH];
	p_IHf_nodI=psik_pI_psik[ixH]/psik_I_psik[ixH];
	//System.err.println("PnLdIdP: bbasis[ixH]="+bbasis[ixH]+" <psik|I|psik>+di*w0="+(psik_I_psik[ixH]+dI*w0[ixH]));
    }

/**
 * Solve {@literal <psi|I|psi>/<psi|psi>}=lambda extremum problem, find eigenstates &amp; eigenvalues.
 * find psi^{IH} and lambda^{IH}=max lambda, used as an estimator of
 * the level of "future" execution.
 * Then dI=lambda^{IH}-{@literal <psi0|I|psi0>} (contribution from the future, of not yet executed, trades).
 * Then operator I^{f} (future I) is: I^{f}= I + {@literal |psi0>dI<psi0|}
 * Find the eigenstates of this new I^{f} matrix,
 * in dynamic impact approximation its eigenstates should be also the P^{f} operator eigenstates.
 * From this follow that dP^{f} operator should be zero on them.
 */
public static PnLdIdP get_PnLdIdP(final MomentsData mm,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				  final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final int ixH=n-1; // the state with maximal I
    final double I_0=I.s0,I_0_feq=I.ev[ixH],dI=I_0_feq-I_0;

    // Create bQQi_feq (operator of "future" I), with the |bpsi0>dI<bpsi0| contribution from the future:
    // bQQi_feq = bQQi + |bpsi0>dI<bpsi0|
    final double [] bQQi_feq=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o)),bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));
    for(int k=0;k<n;k++) bQQi_feq[k+n*k]+=I.ev[k];

    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQdp=EVXData.convertMatrFromQtoB_Basis(n,mm.QQdp,I.ev);

    // Diagonalize "future" I:
    final double [] bbasis=M.EV.getEV(n,bQQi_feq,true),
	psik_psi0=new double [n],psik_I_psik=new double[n],psik_pI_psik=new double[n],psik_dP_psik=new double[n],psik_psiIH=new double[n],psik_pI_psiIH=new double[n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n);
	psik_psi0[k]=TMatr.mulVV(psik,bpsi0);
	psik_I_psik[k]=TMatr.mulV2V(psik,I.getEValues());
	psik_pI_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n);
	psik_dP_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQdp,n);
	psik_psiIH[k]=bbasis[n+k*n+ixH];
	psik_pI_psiIH[k]=DeprecatedHelperTMatr.mulVVo(n,bbasis,n+k*n,bQQpi,ixH*n);
    }

    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdIdP.get_PnLdIdP:I.ev",n,I.ev,mm.QQi,mm.QQt,eps); // old basis
	EVXData.test_orthogonality("PnLdIdP.get_PnLdIdP:bbasis",n,bbasis,bQQi_feq,TMatr.getUnitMatrix(n),eps); // new basis
	// Check spur invariance.
	final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,mm.QQt);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQpi),"Spur(IQQt,QQpi)",TMatr.sumElements(psik_pI_psik),"sum(psik_pI_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQpi),"Spur(bQQpi)",TMatr.sumElements(psik_pI_psik),"sum(psik_pI_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQi),"Spur(IQQt,QQi)",TMatr.sumElements(psik_I_psik),"sum(psik_I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues()),"sum(I.ev)",TMatr.sumElements(psik_I_psik),"sum(psik_I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQdp),"Spur(IQQt,QQdp)",TMatr.sumElements(psik_dP_psik),"sum(psik_dP_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQdp),"Spur(bQQdp)",TMatr.sumElements(psik_dP_psik),"sum(psik_dP_psik)",eps);

	// Check some matrix elements to match known values.
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psi0),"sum(psik_psi0[k]^2)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psiIH,psik_psiIH),"sum(psik_psiIH[k]^2)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psiIH),"sum(psik_psiIH[k]*psik_psi0[k])",I.wH,"I.wH",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(psik_psi0,java.util.Arrays.copyOf(bbasis,n)),"sum(psik_psi0[k]^2 If_k)",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(psik_psiIH,java.util.Arrays.copyOf(bbasis,n)),"sum(psik_psiIH[k]^2 If_k)",I.sH+dI*Math.pow(I.wH,2),"I.sH+dI*Math.pow(I.wH,2)",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_feq,n),"<psi0|I^f|psi0>",I_0_feq,"I_0_feq",eps);

	// Check basis invariance.
	for(int k=0;k<n;k++)
	{
	    final double [] psik=EVXData.convertPsiFromBtoQ_Basis(n,EVXData.getPsiFromBasis(k,bbasis,n),I.ev);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psik,mm.QQi,n),"<psik|I|psik>",psik_I_psik[k],"psik_I_psik[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psik,mm.QQpi,n),"<psik|pI|psik>",psik_pI_psik[k],"psik_pI_psik[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(psik,mm.QQdp,n),"<psik|dP|psik>",psik_dP_psik[k],"psik_dP_psik[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psik,I.psi0,mm.QQt,n),"<psik|psi0>",psik_psi0[k],"psik_psi0[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psik,I.psiH,mm.QQt,n),"<psik|psiIH>",psik_psiIH[k],"psik_psiIH[k]",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(psik,I.psiH,mm.QQpi,n),"<psik|pI|psiIH>",psik_pI_psiIH[k],"psik_pI_psiIH[k]",eps);
	}

	System.err.println("PnLdIdP.get_PnLdIdP selftest OK n="+n+" eps="+eps);
    }

    return new PnLdIdP(I,mm.p_offset,mm.p_last,
		       mm.QQpi[0]/mm.QQi[0],mm.QQdp[0]/mm.QQt[0],bQQpi[ixH+n*ixH]/I.ev[ixH],bQQdp[ixH+n*ixH],
		       TMatr.getAMatrA(bpsi0,bQQdp,n),I_0_feq,
		       psik_psi0,psik_I_psik,psik_pI_psik,psik_dP_psik,bbasis,psik_psiIH,psik_pI_psiIH);
}


static void matchPnLdIdPRQ(final PnLdIdP a,final PnLdIdPRQ b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.I_0_feq,"a.I_0_feq",b.I_0_feq,"b.I_0_feq",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_IHf,"a.p_IHf",b.p_IHf+p_offset_shift,"b.p_IHf+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P_movaver,"a.P_movaver",b.P_movaver+p_offset_shift,"b.P_movaver+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P_movaver_spur,"a.P_movaver_spur",b.P_movaver_spur+p_offset_shift,"b.P_movaver_spur+p_offset_shift",eps);

    System.err.println("PnLdIdP.matchPnLdIdPRQ test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}

static void matchPnLdISk(final PnLdIdP a,final PnLdISk b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.I_0_feq,"a.I_0_feq",b.I_0_feq,"b.I_0_feq",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_IHf,"a.p_IHf",b.SK_IHf.xa+p_offset_shift,"b.SK_IHf.xa+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_IHf_nodI,"a.p_IHf_nodI",b.SK_IHf__nodI.xa+p_offset_shift,"b.SK_IHf__nodI.xa+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P_movaver,"a.P_movaver",b.SK_movaver.xa+p_offset_shift,"b.SK_movaver.xa+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P_movaver_spur,"a.P_movaver_spur",b.P_movaver_spur+p_offset_shift,"b.P_movaver_spur+p_offset_shift",eps);

    System.err.println("PnLdIdP.matchPnLdISk test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}

}
