package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.VolMatrix;

/**
 *  @version $Id: PnLdIf0.java,v 1.70 2022/08/07 15:44:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdIf0 {

    public final EVXData I;
    public final double p_offset,p_IH,p_IHf_nodI,I_0_feq,P_movaver,P_movaver_spur,dP_movaver,dP_movaver_spur,
	A_PfdI,A_Pf,A_Pb,A_PbI,Pa,
	p_IHf,p_0f,pd_f,P_IH_IHf,P1b,p_E_IHf,p_E_0,P1_IHf,P2_IHf,P1_IH,P2_IH,PfadI,Pfa,PfdI,Pf,P0f,P0lf,dC0lf,dP0f,P0ff,P0ffi,dPf_0,dP_w0,dC_w0;
    PnLdIf0(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,final double P_movaver,final double P_movaver_spur,
	    final double p_IH,final double dP_movaver,final double I_0_feq,
	    final double [] psik_psi0,final double [] psik_I_psi0,final double [] psik_pI_psi0,
	    final double [] psik_I_psik,final double [] psik_pI_psik,final double [] psik_dp_psik,
	    final double [] If,final double [] bbQQpi,
	    final double P_IH_IHf,final double P1b,final double p_E_IHf,final double p_E_0,final double pI_0,
	    final double P1_IHf,final double P2_IHf,final double P1_IH,final double P2_IH)
    {
	this.I=I;
	this.p_offset=p_offset;
	this.P_movaver=P_movaver;
	this.P_movaver_spur=P_movaver_spur;
	this.p_IH=p_IH;
	this.I_0_feq=I_0_feq;
	this.P_IH_IHf=P_IH_IHf;
	this.P1b=P1b;
	this.p_E_IHf=p_E_IHf;
	this.p_E_0=p_E_0;
	this.P1_IHf=P1_IHf;
	this.P2_IHf=P2_IHf;
	this.P1_IH=P1_IH;
	this.P2_IH=P2_IH;
	final double dI=I_0_feq-I.s0;
	final int n=I.neq;
	final double [] PfkdI=new double [n];
	double sPfdI=0,sPb=0,sPbI=0,sbI=0;
	for(int k=0;k<n;k++)
	{
	    final double psik_psi0_sq=Math.pow(psik_psi0[k],2);
	    PfkdI[k]=(psik_pI_psik[k]-(psik_pI_psi0[k]/psik_psi0[k]))/(1-psik_psi0_sq);
	    sPfdI+=(psik_pI_psik[k]*psik_psi0_sq-(psik_pI_psi0[k]*psik_psi0[k]))/(1-psik_psi0_sq);
	    final double pk=psik_pI_psik[k]/psik_I_psik[k];
	    sPb+=pk*psik_psi0_sq;
	    sPbI+=pk*(psik_psi0_sq/psik_I_psik[k]);
	    sbI+=(psik_psi0_sq/psik_I_psik[k]);
	}
	double sn=0,sd=0;
	for(int k=0;k<n;k++) for(int l=0;l<n;l++)
	if(l==k) continue;
	else
	{
	    final double w=psik_psi0[k]*psik_psi0[l];
	    sn+=bbQQpi[k+n*l]*w;
	    sd+=Math.pow(w,2);
	}
	this.PfadI=-sn/sd;
	this.Pfa=PfadI/dI;
	this.A_PfdI=sPfdI;
	this.A_Pf=sPfdI/dI;
	this.A_Pb=sPb;
	this.A_PbI=sPbI/sbI;
	this.Pa=TMatr.mulV2V(psik_psi0,psik_pI_psik)/TMatr.mulV2V(psik_psi0,psik_I_psik);
	final int ixH=n-1;
	this.match_Pf_psi0_dI_PnLfIHI0=PfkdI[ixH];
	final double pI_IHf=psik_pI_psik[ixH]+PfkdI[ixH]*Math.pow(psik_psi0[ixH],2),
	    pI_0f=TMatr.mulVV(psik_pI_psi0,psik_psi0)+PfkdI[ixH];
	this.p_IHf=pI_IHf/If[ixH];
	this.p_IHf_nodI=psik_pI_psik[ixH]/psik_I_psik[ixH];
        this.p_0f=pI_0f/I_0_feq;
	this.pd_f=(pI_IHf-pI_0f)/(If[ixH]-I_0_feq);

	sn=sd=0;
	for(int k=0,l=ixH;k<n;k++)
	if(l==k) continue;
	else
	{
	    final double w=psik_psi0[k]*psik_psi0[l];
	    sn+=bbQQpi[k+n*l]*w;
	    sd+=Math.pow(w,2);
	}
	this.PfdI=-sn/sd;
	this.Pf=PfdI/dI;

	double s4=0,si=0,spi=0,sp=0,sdp=0;
	for(int k=0;k<n;k++)
	{
	    final double w=Math.pow(psik_psi0[k],2),w2=w*w;
	    s4+=w2;
	    si+=psik_I_psik[k]*w;
	    spi+=psik_pI_psik[k]*w;
	    sp+=(psik_pI_psik[k]/psik_I_psik[k])*w;
	    sdp+=psik_dp_psik[k]*w;
	}
	// P0f actually match Pfa!
	this.P0f=((spi-pI_0)/(1-s4))/dI;
	this.P0lf=((spi-I.s0*p_last__with_p_offset_included)/(1-s4))/dI;
	this.dC0lf=(spi-I.s0*p_last__with_p_offset_included)/(1-s4)-p_last__with_p_offset_included*dI;
	this.dP0f=((sp-pI_0/I.s0)/(1-s4));
	this.P0ff=sp;
	this.P0ffi=spi/si;
	this.dPf_0=-sdp/s4;
	this.dP_w0=sdp;
	this.dC_w0=dP_w0*dI;
	this.dP_movaver=dP_movaver;
	this.dP_movaver_spur=TMatr.sumElements(psik_dp_psik)/psik_dp_psik.length;
	//System.err.println("psik_dp_psik="+java.util.Arrays.toString(psik_dp_psik)+"psik_psi0="+java.util.Arrays.toString(psik_psi0));
	//System.err.println("PnLfIHI0:.PnLfIHI0:(si-I.s0)/((1-s4)*dI)="+(((si-I.s0)/(1-s4))/dI)+" must be =1");
    }
    final double match_Pf_psi0_dI_PnLfIHI0;

/**
 * Solve {@literal <psi|I|psi>/<psi|psi>}=lambda extremum problem, find eigenstates &amp; eigenvalues.
 * find psi^{IH} and lambda^{IH}=max lambda, used as an estimator of
 * the level of "future" execution.
 * Then dI=lambda^{IH}-{@literal <psi0|I|psi0>} (contribution from the future, of not yet executed, trades).
 * Then operator I^{f} (future I) is: I^{f}= I + {@literal |psi0>dI<psi0|}
 * Find the eigenstates of this new I^{f} matrix, the {@literal |psif>} is the one, corresponding to
 * the state with some condition on {@literal |psif>} (such as max {@literal <psi0|psif>}^2 or possible some other).
 * Then variate pI^{f} (future pI)= pI+{@literal |psi0>Pf*dI<psi0|} on this {@literal |psif>} eigenstate.
 * Find Pf from the condition of pI^{f} variation equal to 0 on {@literal |psif>} for some {@literal |dpsi>}
 * There are two questions to be carefuly considered:
 * 1. The selection of {@literal |psif>} from n eigenstates available (typically from max {@literal <psi0|psif>}^2)
 *    or, another option, try to average all eigenstates with {@literal <psi0|psif>}^2 weights.
 * 2. The selection of |dpsi> to require 0 variation of pI^{f} on {@literal |psif>} state (typically use {@literal |psi0>}).
 */
public static PnLdIf0 get_PnLdIf0(final MomentsData mm,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				  final double eps)
{
    // option flag_reset_nondiag=true is used only for PnLdIfdyn unit test.
    return get_PnLdIf0(mm,M,eps,PnLdIDyn.IMPACTAPPROXIMATION_NONE);
}
public static PnLdIf0 get_PnLdIf0(final MomentsData mm,
			   final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			   final double eps,
			   final int flag_IMPACTAPPROXIMATION)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final double I_0=I.s0,I_0_feq=I.sH,dI=I_0_feq-I_0;
    final int ixH=n-1; // the state with maximal I

    // Create bQQi_feq (operator of "future" I), with the |bpsi0>dI<bpsi0| contribution from the future:
    // bQQi_feq = bQQi + |bpsi0>dI<bpsi0|
    final double [] bQQi_feq=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o)),bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));
    for(int k=0;k<n;k++) bQQi_feq[k+n*k]+=I.ev[k];

    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQdp=EVXData.convertMatrFromQtoB_Basis(n,mm.QQdp,I.ev);
    if(flag_IMPACTAPPROXIMATION==PnLdIDyn.IMPACTAPPROXIMATION_NONE){}
    else if(flag_IMPACTAPPROXIMATION==PnLdIDyn.IMPACTAPPROXIMATION_DYNAMIC_PURE)
    {
	for(int k=0;k<n;k++) for(int l=0;l<n;l++) if(k!=l) bQQpi[k+n*l]=0;
    }
    else if(flag_IMPACTAPPROXIMATION==PnLdIDyn.IMPACTAPPROXIMATION_DYNAMIC_IH)
    {
	for(int k=ixH,l=0;l<n;l++) if(k!=l) bQQpi[k+n*l]=bQQpi[l+n*k]=0;
    }
    else throw new RuntimeException("flag_IMPACTAPPROXIMATION="+flag_IMPACTAPPROXIMATION+" is not known");
    // Diagonalize "future" I:
    final double [] bbasis=M.EV.getEV(n,bQQi_feq,true),
	bbQQpi=EVXData.convertMatrFromQtoB_Basis(n,bQQpi,bbasis),
	psik_psi0=new double [n],psik_I_psi0=new double [n],psik_pI_psi0=new double [n],
	psik_I_psik=new double[n],psik_pI_psik=new double[n],psik_dp_psik=new double[n];

    for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n);

	psik_psi0[k]=TMatr.mulVV(psik,bpsi0);
	psik_I_psi0[k]=TMatr.mulVVV(psik,I.getEValues(),bpsi0);
	psik_pI_psi0[k]=TMatr.getAMatrB(n,bbasis,n+k*n,bpsi0,0,bQQpi,n);
	psik_I_psik[k]=TMatr.mulV2V(psik,I.getEValues());
	psik_pI_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n);
	psik_dp_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQdp,n);
    }
    final double IHf=bbasis[ixH];
    final double [] bpsi_IHf=EVXData.getPsiFromBasis(ixH,bbasis,n),
	bpsi_IHf_projO0=TMatr.sumAcB(bpsi_IHf,-TMatr.mulVV(bpsi_IHf,bpsi0),bpsi0),
	bpsi_IH=new double [n];
    bpsi_IH[ixH]=1;
    final double [] bpsi_IH_projO0=TMatr.sumAcB(bpsi_IH,-TMatr.mulVV(bpsi_IH,bpsi0),bpsi0);

    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdIf0.get_PnLdIf0:I.ev",n,I.ev,mm.QQi,mm.QQt,eps);
	// Check spur invariance
	final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,mm.QQt);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQpi),"Spur(IQQt,QQpi)",TMatr.sumElements(psik_pI_psik),"sum(psik_pI_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurM(n,bQQpi),"Spur(bQQpi)",TMatr.sumElements(psik_pI_psik),"sum(psik_pI_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.getSpurMM(n,IQQt,mm.QQi),"Spur(IQQt,QQi)",TMatr.sumElements(psik_I_psik),"sum(psik_I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(I.getEValues()),"sum(I.ev)",TMatr.sumElements(psik_I_psik),"sum(psik_I_psik)",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psi0),"sum(psik_psi0[k]^2)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(psik_psi0,java.util.Arrays.copyOf(bbasis,n)),"sum(psik_psi0[k]^2 If_k)",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_I_psi0,psik_psi0),"sum(<psik|I|psi0><psik|psi0>)",I.s0,"I.s0",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_I_psi0,psik_I_psi0),"sum(<psi0|I|psik><psik|I|psi0>)",TMatr.getAMatrA(I.psi0,TMatr.mulMatrices(n,new double [][]{mm.QQi,IQQt,mm.QQi}),n),"<psi0|I|I|psi0>",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_feq,n),"<psi0|I^f|psi0>",I_0_feq,"I_0_feq",eps);
	if(flag_IMPACTAPPROXIMATION==PnLdIDyn.IMPACTAPPROXIMATION_NONE)
	{
	    UnitTests.checkDoubleRel(TMatr.mulVV(psik_pI_psi0,psik_psi0),"sum(<psik|pI|psi0><psik|psi0>)",TMatr.getAMatrA(I.psi0,mm.QQpi,n),"<psi0|pI|psi0>",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQpi,n),"<bpsi0|bQQpi|bpsi0>",TMatr.getAMatrA(I.psi0,mm.QQpi,n),"<psi0|pI|psi0>",eps);
	    UnitTests.checkDoubleRel(TMatr.mulVV(psik_pI_psi0,psik_psi0),"sum(<psi0|psik><psik|pI|psi0>)",TMatr.getAMatrA(I.psi0,mm.QQpi,n),"<psi0|pI|psi0>",eps);
	    UnitTests.checkDoubleRel(TMatr.mulVV(psik_pI_psi0,psik_pI_psi0),"sum(<psi0|pI|psik><psik|pI|psi0>)",TMatr.getAMatrA(I.psi0,TMatr.mulMatrices(n,new double [][]{mm.QQpi,IQQt,mm.QQpi}),n),"<psi0|pI|pI|psi0>",eps);
	}

	System.err.println("PnLdIf0.get_PnLdIf0 selftest OK n="+n+" eps="+eps);
    }

    final double P_IH_IHf;
    if(true)
    {
	final double psi_IH_If_psi_IH=TMatr.getAMatrA(bpsi_IH,bQQi_feq,n),
	    psi_IHf_psi_IH=TMatr.mulVV(bpsi_IHf,bpsi_IH),
	    psi0_psi_IH=TMatr.mulVV(bpsi0,bpsi_IH),
	    psi0_psi_IHf=TMatr.mulVV(bpsi0,bpsi_IHf),
	    PfdI_IHf=(psik_pI_psik[ixH]-TMatr.getAMatrB(bpsi_IHf,bpsi_IH,bQQpi,n)/psi_IHf_psi_IH)/(psi0_psi_IH*psi0_psi_IHf/psi_IHf_psi_IH-Math.pow(psi0_psi_IHf,2));
	final double [] bQQpi_feq=TMatr.sumAcB(bQQpi,PfdI_IHf,TMatr.getQQaa(bpsi0)),O=new double [n*n];

	for(int k=0;k<n;k++) O[k+n*k]=1;
	final double []
	    rqi=RayleighQuotient.getRQVariationValues(bpsi_IHf,bpsi_IH,null,bQQi_feq,O,n),
	    rqpi=RayleighQuotient.getRQVariationValues(bpsi_IHf,bpsi_IH,null,bQQpi_feq,O,n);

	final double psi_IHf_pIf_psi_IHf=TMatr.getAMatrA(bpsi_IHf,bQQpi_feq,n),
	    psi_IH_pIf_psi_IH=TMatr.getAMatrA(bpsi_IH,bQQpi_feq,n);
	P_IH_IHf=rqpi[2]/rqi[2];

	if(eps>0)
	{
	    UnitTests.checkDoubleRel(rqi[0],"rqi[0]",IHf,"IHf",eps);
	    UnitTests.checkDoubleRel(rqi[1],"rqi[1]",0,"0",eps);
	    UnitTests.checkDoubleRel(0.5*rqi[2],"0.5*rqi[2]",psi_IH_If_psi_IH-IHf,"I_0_feq-IHf",eps);
	    UnitTests.checkDoubleRel(rqpi[0],"rqpi[0]",psi_IHf_pIf_psi_IHf,"psi_IHf_pIf_psi_IHf",eps);
	    UnitTests.checkDoubleRel(rqpi[1],"rqpi[1]",0,"0",eps);
	    UnitTests.checkDoubleRel(0.5*rqpi[2],"0.5*rqpi[2]",psi_IH_pIf_psi_IH-psi_IHf_pIf_psi_IHf,"psi_IH_pIf_psi_IH-psi_IHf_pIf_psi_IHf",eps);
	    System.err.println("PnLdIf0.get_PnLdIf0 selftest2 OK n="+n+" eps="+eps);
	}

    }

    final double pI_0=TMatr.getAMatrA(bpsi0,bQQpi,n),
	P1b=TMatr.getAMatrB(bpsi_IHf,bpsi_IH,bQQpi,n)/TMatr.mulVV(bpsi_IHf,bpsi_IH)/I.ev[ixH],
	p_E_IHf=(TMatr.getAMatrA(bpsi_IHf,bQQpi,n)-TMatr.getAMatrA(bpsi_IH,bQQpi,n))/
	(TMatr.mulV2V(bpsi_IHf,I.getEValues())-I.ev[ixH]/*TMatr.mulV2V(bpsi_IH,I.getEValues())*/),
	p_E_0=(pI_0-TMatr.getAMatrA(bpsi_IH,bQQpi,n))/
	(TMatr.mulV2V(bpsi0,I.getEValues())-I.ev[ixH]/*TMatr.mulV2V(bpsi_IH,I.getEValues())*/);

    final double [] I_ev_n=I.getEValues();
    final PnLdIf0 r=
	new PnLdIf0(I,mm.p_offset,mm.p_last,
		    mm.QQpi[0]/mm.QQi[0],TMatr.getSpurM(n,bQQpi)/TMatr.sumElements(I.getEValues()),
		    bQQpi[ixH+n*ixH]/I.ev[ixH],mm.QQdp[0]/mm.QQt[0],I_0_feq,
		    psik_psi0,psik_I_psi0,psik_pI_psi0,psik_I_psik,psik_pI_psik,psik_dp_psik,
		    bbasis,bbQQpi,
		    P_IH_IHf,P1b,p_E_IHf,p_E_0,pI_0,
		    TMatr.getAMatrB(bpsi_IHf_projO0,bpsi_IHf,bQQpi,n)/
		    TMatr.mulVVV(bpsi_IHf_projO0,bpsi_IHf,I_ev_n),
		    TMatr.getAMatrB(bpsi_IHf_projO0,bpsi_IHf_projO0,bQQpi,n)/
		    TMatr.mulVVV(bpsi_IHf_projO0,bpsi_IHf_projO0,I_ev_n),
		    TMatr.getAMatrB(bpsi_IH_projO0,bpsi_IH,bQQpi,n)/
		    TMatr.mulVVV(bpsi_IH_projO0,bpsi_IH,I_ev_n),
		    TMatr.getAMatrB(bpsi_IH_projO0,bpsi_IH_projO0,bQQpi,n)/
		    TMatr.mulVVV(bpsi_IH_projO0,bpsi_IH_projO0,I_ev_n)
		    );
    if(eps>0)
    {
	final double PfdI_IHf=r.match_Pf_psi0_dI_PnLfIHI0;
	final double [] bQQpi_feq=TMatr.sumAcB(bQQpi,PfdI_IHf,TMatr.getQQaa(bpsi0)),O=new double [n*n];
	for(int k=0;k<n;k++) O[k+n*k]=1;
	final double []
	    rqi=RayleighQuotient.getRQVariationValues(bpsi_IHf,bpsi0,null,bQQi_feq,O,n),
	    rqpi=RayleighQuotient.getRQVariationValues(bpsi_IHf,bpsi0,null,bQQpi_feq,O,n);
	UnitTests.checkDoubleRel(rqi[0],"rqi[0]",IHf,"IHf",eps);
	UnitTests.checkDoubleRel(rqi[1],"rqi[1]",0,"0",eps);
	UnitTests.checkDoubleRel(0.5*rqi[2],"0.5*rqi[2]",I_0_feq-IHf,"I_0_feq-IHf",eps);
	final double pI_IHf=TMatr.getAMatrA(bpsi_IHf,bQQpi_feq,n),pI_0f=TMatr.getAMatrA(bpsi0,bQQpi_feq,n);
	UnitTests.checkDoubleRel(rqpi[0],"rqpi[0]",pI_IHf,"pI_IHf",eps);
	UnitTests.checkDoubleRel(rqpi[1],"rqpi[1]",0,"0",eps);
	UnitTests.checkDoubleRel(0.5*rqpi[2],"0.5*rqpi[2]",pI_0f-pI_IHf,"pI_0f-pI_IHf",eps);
	UnitTests.checkDoubleRel(r.p_IHf,"r.p_IHf",rqpi[0]/rqi[0],"rqpi[0]/rqi[0]",eps);
	UnitTests.checkDoubleRel(r.p_0f,"r.p_0f",pI_0f/I_0_feq,"pI_0f/I_0_feq",eps);
	UnitTests.checkDoubleRel(r.pd_f,"r.pd_f",rqpi[2]/rqi[2],"rqpi[2]/rqi[2]",eps);

	System.err.println("PnLdIf0.get_PnLdIf0 selftest3 OK n="+n+" eps="+eps);

    }
    if(eps>0)
    {
	double sna=0,sda=0,sn=0,sd=0;
	for(int k=0;k<n;k++)
	{
	    for(int l=0;l<n;l++)
	    {
		if(k!=l)
		{
		    final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n),
			psil=EVXData.getPsiFromBasis(l,bbasis,n);
		    final double w=TMatr.mulVV(psik,bpsi0)*TMatr.mulVV(psil,bpsi0),
			tsn=w*TMatr.getAMatrB(n,bbasis,n+k*n,bbasis,n+l*n,bQQpi,n),tsd=w*w;
		    sna+=tsn;
		    sda+=tsd;
		    if(k==ixH||l==ixH)
		    {
			sn+=tsn;
			sd+=tsd;
		    }
		    final double in=TMatr.mulVVV(psik,psil,I.getEValues())/dI;
		    UnitTests.checkDoubleRel(-in,"-in",w,"w",eps);
		}
	    }
	}
	final double PfadI=-sna/sda,PfdI=-sn/sd;
	UnitTests.checkDoubleRel(r.PfadI,"r.PfadI",PfadI,"PfadI",eps);
	UnitTests.checkDoubleRel(r.PfdI,"r.PfdI",PfdI,"PfdI",eps);
	// These actually match: Pfa & P0f!!!!
	UnitTests.checkDoubleRel(r.Pfa,"r.Pfa",r.P0f,"r.P0f",eps);
	System.err.println("PnLdIf0.get_PnLdIf0 selftest4 OK n="+n+" eps="+eps);
    }


    return r;
}

static void matchPnLdIfdyn(final PnLdIf0 a,final PnLdIfdyn b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.I_0_feq,"a.I_0_feq",b.I_0_feq,"b.I_0_feq",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.A_Pf,"a.A_Pf",b.Pf+p_offset_shift,"b.Pf+p_offset_shift",eps);

    System.err.println("PnLdIf0.matchPnLdIfdyn test OK n="+a.I.neq+" eps="+eps);
}

static void matchPnLdIDyn(final PnLdIf0 a,final PnLdIDyn b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.I_0_feq,"a.I_0_feq",b.I_0_feq,"b.I_0_feq",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P_movaver,"a.P_movaver",b.P_movaver+p_offset_shift,"b.P_movaver+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P_movaver_spur,"a.P_movaver_spur",b.P_movaver_spur+p_offset_shift,"b.P_movaver_spur+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_E_IHf,"a.p_E_IHf",b.p_E_IHf+p_offset_shift,"b.p_E_IHf+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_E_0,"a.p_E_0",b.p_E_0+p_offset_shift,"b.p_E_0+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P1_IHf,"a.P1_IHf",b.P1_IHf+p_offset_shift,"b.P1_IHf+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P2_IHf,"a.P2_IHf",b.P2_IHf+p_offset_shift,"b.P2_IHf+p_offset_shift",eps);

    System.err.println("PnLdIf0.matchPnLdIDyn test OK n="+a.I.neq+" eps="+eps);
}


static void matchPnLdIdP(final PnLdIf0 a,final PnLdIdP b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.I_0_feq,"a.I_0_feq",b.I_0_feq,"b.I_0_feq",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dPf_0,"a.dPf_0",b.dPf_0,"b.dPf_0",eps);
    UnitTests.checkDoubleRel(a.dP_w0,"a.dP_w0",b.dP_w0,"b.dP_w0",eps);
    UnitTests.checkDoubleRel(a.dC_w0,"a.dC_w0",b.dC_w0,"b.dC_w0",eps);
    UnitTests.checkDoubleRel(a.dP_movaver,"a.dP_movaver",b.dP_movaver,"b.dP_movaver",eps);
    UnitTests.checkDoubleRel(a.dP_movaver_spur,"a.dP_movaver_spur",b.dP_movaver_spur,"b.dP_movaver_spur",eps);

    System.err.println("PnLdIf0.matchPnLdIdP test OK n="+a.I.neq+" eps="+eps);
}

static void matchPnLdIS(final PnLdIf0 a,final PnLdIS b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);

    final double p_offset_shift=b.p_offset-a.p_offset;
    UnitTests.checkDoubleRel(a.I_0_feq,"a.I_0_feq",b.I_0_feq,"b.I_0_feq",eps);
    UnitTests.checkDoubleRel(a.p_IH,"a.p_IH",b.p_IH+p_offset_shift,"b.p_IH+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.A_Pb,"a.A_Pb",b.SP.xa+p_offset_shift,"b.SP.xa+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.dP_w0,"a.dP_w0",b.SdP.xa,"b.SdP.xa",eps);
    UnitTests.checkDoubleRel(a.P_movaver,"a.P_movaver",b.P_movaver+p_offset_shift,"b.P_movaver+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.P_movaver_spur,"a.P_movaver_spur",b.P_movaver_spur+p_offset_shift,"b.P_movaver_spur+p_offset_shift",eps);
    UnitTests.checkDoubleRel(a.p_IHf_nodI,"a.p_IHf_nodI",b.p_IHf_nodI+p_offset_shift,"b.p_IHf_nodI+p_offset_shift",eps);

    System.err.println("PnLdIf0.matchPnLdIS test OK n="+a.I.neq+" p_offset_shift="+p_offset_shift+" eps="+eps);
}

}
