package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.DerivativesCalculateable;
import com.polytechnik.utils.SortArrays;

/**
 *  @version $Id: PnLdIpx.java,v 1.35 2022/08/05 11:25:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PnLdIpx {




public final EVXData I;
public final double p_offset,p_last__with_p_offset_included,p_IH,I_0_feq,
    Pf_from_dyn,deg_from_dyn,Pf_from_f0,degt_from_dyn,Pft_from_f0,delta_Pdyn_from_dyn,delta_P_IHf,
    P_IHf,P_IHf_dyn,deg_BIH,P_BIH,
    P_B0,P_B0d,deg2_Hf,Pf2_Hf,Pf2o_Hf,Pf2os_Hf,Pf_from_xx;
PnLdIpx(final EVXData I,final double p_offset,final double p_last__with_p_offset_included,
	final double p_IH,final double I_0_feq,
	final double  Pf_from_dyn,final double deg_from_dyn,final double Pf_from_f0,
	final double degt_from_dyn,final double Pft_from_f0,final double delta_Pdyn_from_dyn,
	final double delta_P_IHf,
	final double P_IHf,final double P_IHf_dyn,final double deg_BIH,final double P_BIH,
	final double P_B0,final double P_B0d,
	final double deg2_Hf,final double Pf2_Hf,final double Pf2o_Hf,final double Pf2os_Hf,final double Pf_from_xx,
	final double eps)
{
    this.I=I;
    this.p_offset=p_offset;
    this.p_last__with_p_offset_included=p_last__with_p_offset_included;
    this.p_IH=p_IH;
    this.I_0_feq=I_0_feq;
    this.Pf_from_dyn=Pf_from_dyn;
    this.deg_from_dyn=deg_from_dyn;
    this.Pf_from_f0=Pf_from_f0;
    this.degt_from_dyn=degt_from_dyn;
    this.Pft_from_f0=Pft_from_f0;
    this.delta_Pdyn_from_dyn=delta_Pdyn_from_dyn;
    this.delta_P_IHf=delta_P_IHf;
    this.P_IHf=P_IHf;
    this.P_IHf_dyn=P_IHf_dyn;
    this.deg_BIH=deg_BIH;
    this.P_BIH=P_BIH;
    this.P_B0=P_B0;
    this.P_B0d=P_B0d;
    this.deg2_Hf=deg2_Hf;
    this.Pf2_Hf=Pf2_Hf;
    this.Pf2o_Hf=Pf2o_Hf;
    this.Pf2os_Hf=Pf2os_Hf;
    this.Pf_from_xx=Pf_from_xx;
}

/**
 * Solve {@literal <psi|I|psi>/<psi|psi>}=lambda extremum problem, find eigenstates &amp; eigenvalues.
 * find psi^{IH} and lambda^{IH}=max lambda, used as an estimator of
 * the level of "future" execution.
 * Then dI=lambda^{IH}-{@literal <psi0|I|psi0>} (contribution from the future, of not yet executed, trades).
 * Then operator I^{f} (future I) is: I^{f}= I + {@literal |psi0>dI<psi0|}
 * Find the eigenstates of this new I^{f} matrix,
 * in dynamic impact approximation its eigenstates should be also the P^{f} operator eigenstates.
 *
 */
public static PnLdIpx get_PnLdIpx(final MomentsData mm,
				  final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
				  final double eps)
{
    final int n=mm.n;
    if(n<2) throw new RuntimeException("n="+n+"<2");
    final EVXData I=EVXData.getEVXDataFromQQ(n,mm.QQi,mm.QQt,M);
    final int ixH=n-1; // the state with maximal I
    final double I_0=I.s0,I_0_feq=I.ev[ixH],dI=I_0_feq-I_0;

    // Create bQQi_feq (operator of "future" I), with the |bpsi0>dI<bpsi0| contribution from the future:
    // bQQi_feq = bQQi + |bpsi0>dI<bpsi0|
    final double [] bQQi_feq=TMatr.scaleVectorInPlace(TMatr.getQQaa(I.psik_at_x0),dI/(I.o_o)),bpsi0=TMatr.getVectorScaled(I.psik_at_x0,1/Math.sqrt(I.o_o));
    for(int k=0;k<n;k++) bQQi_feq[k+n*k]+=I.ev[k];

    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,mm.QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp2i,I.ev),
	bQQp3i=EVXData.convertMatrFromQtoB_Basis(n,mm.QQp3i,I.ev),
	bQQdp=EVXData.convertMatrFromQtoB_Basis(n,mm.QQdp,I.ev);

    // Diagonalize "future" I:
    final double [] bbasis=M.EV.getEV(n,bQQi_feq,true),
	bpsiHf=EVXData.getPsiFromBasis(ixH,bbasis,n),
	wHF=TMatr.squareArrayElements(bpsiHf),
	w0B=TMatr.scaleVectorInPlace(TMatr.squareArrayElements(I.psik_at_x0),1/I.o_o),
	psik_psi0=new double [n],psik_psiB=new double [n],
	psik_I_psik=new double[n],psik_pI_psik=new double[n],psik_dp_psik=new double[n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,bbasis,n);

	psik_psi0[k]=TMatr.mulVV(psik,bpsi0);
	psik_psiB[k]=bbasis[n+k*n+ixH];
	psik_I_psik[k]=TMatr.mulV2V(psik,I.getEValues());
	psik_pI_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQpi,n);
	psik_dp_psik[k]=TMatr.getAMatrA(n,bbasis,n+k*n,bQQdp,n);
    }
    final double w0F[]=TMatr.squareArrayElements(psik_psi0),
	w0Fr[]=TMatr.scaleVectorInPlace(DeprecatedHelperTMatr.getRatio(n,w0F,bbasis),I_0_feq),
	wBF[]=TMatr.squareArrayElements(psik_psiB),
	pI_0=TMatr.getAMatrA(bpsi0,bQQpi,n),
	deg_from_dyn=1-TMatr.mulVV(w0F,w0F),
	Pf_from_f0=(TMatr.mulVV(w0F,psik_pI_psik)-pI_0)/(dI*deg_from_dyn),
	degt_from_dyn=1-TMatr.mulVV(w0F,w0Fr),
	Pft_from_f0=(TMatr.mulVV(w0Fr,psik_pI_psik)-pI_0)/(dI*degt_from_dyn),
	Pf_from_dyn=(TMatr.mulVV(w0F,psik_pI_psik)-TMatr.mulVV(w0B,TMatr.getMatrixDiagElements(n,bQQpi)))/(dI*deg_from_dyn),
	delta_Pdyn_from_dyn=(TMatr.mulVV(w0F,psik_dp_psik)-TMatr.getAMatrA(bpsi0,bQQdp,n))/(deg_from_dyn*I.o_o),
	delta_P_IHf=-psik_dp_psik[ixH]/w0F[ixH]/I.o_o,
	I_IHf=TMatr.mulV2V(bpsiHf,I.getEValues()),
	PI_IHf=TMatr.getAMatrA(bpsiHf,bQQpi,n),
	PI_IHf_dyn=TMatr.mulVV(wHF,TMatr.getMatrixDiagElements(n,bQQpi)),
	deg_BIH=TMatr.mulVV(w0F,wBF)-Math.pow(bpsi0[ixH],2),
	P_BIH=(bQQpi[n*ixH+ixH]-TMatr.mulVV(wBF,psik_pI_psik))/(dI*deg_BIH);

    final double a_psiHf_psi0=TMatr.mulVV(bpsi0,bpsiHf),
	bpsi0_ort_bpsiHf []=TMatr.sumAcB(bpsi0,-a_psiHf_psi0,bpsiHf);
    TMatr.scaleVectorInPlace(bpsi0_ort_bpsiHf,1/Math.sqrt(TMatr.mulVV(bpsi0_ort_bpsiHf,bpsi0_ort_bpsiHf)));
    final double a_ort_psi0=TMatr.mulVV(bpsi0,bpsi0_ort_bpsiHf),
	deg2_Hf=1-(Math.pow(a_psiHf_psi0*TMatr.mulVV(bpsi0,bpsiHf),2)+Math.pow(a_ort_psi0*TMatr.mulVV(bpsi0,bpsi0_ort_bpsiHf),2)),
	PdIf2_Hf=Math.pow(a_psiHf_psi0,2)*TMatr.getAMatrA(bpsiHf,bQQpi,n)+
	Math.pow(a_ort_psi0,2)*TMatr.getAMatrA(bpsi0_ort_bpsiHf,bQQpi,n)-
	TMatr.getAMatrA(bpsi0,bQQpi,n),
	Pf2_Hf=PdIf2_Hf/(deg2_Hf*dI),
	Pf2o_Hf=-TMatr.getAMatrB(bpsiHf,bpsi0_ort_bpsiHf,bQQpi,n)/(dI*a_psiHf_psi0*a_ort_psi0);
    double spos=0,sos=0;
    for(int k=0;k<n;k++)
    {
	if(k==ixH) continue;
	sos+=a_psiHf_psi0*Math.pow(psik_psi0[k],2);
	spos+=TMatr.getAMatrB(n,bpsiHf,0,bbasis,n+k*n,bQQpi,n)*psik_psi0[k];
    }
    final double Pf2os_Hf=-spos/(dI*sos);
    final int inds_w0F[]=SortArrays.sortArrayIndexes(w0F),i0_e=inds_w0F[n-1],i0_p=inds_w0F[n-2];
    final double Pf_from_xx=-TMatr.getAMatrB(n,bbasis,n+i0_e*n,bbasis,n+i0_p*n,bQQpi,n)/(psik_psi0[i0_e]*psik_psi0[i0_p]*dI);
    if(eps>0)
    {
	// |psi0>= aHf_0* |psiHf> + |bpsi0_ort_bpsiHf>
	final double [] bQQi=TMatr.getDiagMatrix(I.getEValues());
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi0_ort_bpsiHf,bpsiHf),"<bpsi0_ort_bpsiHf|psiHf>",0,"0",eps);
	UnitTests.checkDoubleRel(psik_psi0[ixH],"psik_psi0[ixH]",a_psiHf_psi0,"a_psiHf_psi0",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi0_ort_bpsiHf,bpsi0_ort_bpsiHf),"<bpsi0_ort_bpsiHf|bpsi0_ort_bpsiHf>",1,"1",eps);
	UnitTests.checkDoubleArrayRel1(TMatr.sumaAbB(a_psiHf_psi0,bpsiHf,a_ort_psi0,bpsi0_ort_bpsiHf),"bpsi0",bpsi0,"bpsi0",eps);
	UnitTests.checkDoubleRel(Math.pow(a_psiHf_psi0,2)*TMatr.getAMatrA(bpsiHf,bQQi,n)+
				 Math.pow(a_ort_psi0,2)*TMatr.getAMatrA(bpsi0_ort_bpsiHf,bQQi,n)-
				 TMatr.getAMatrA(bpsi0,bQQi,n),"deg2_Hf*dI",deg2_Hf*dI,"deg2_Hf*dI",eps);
	UnitTests.checkDoubleRel(-TMatr.getAMatrB(bpsiHf,bpsi0_ort_bpsiHf,bQQi,n)/(dI*a_psiHf_psi0*a_ort_psi0),"1",1,"1",eps);
	double uspos=0;
	for(int k=0;k<n;k++)
	{
	    if(k==ixH) continue;
	    uspos+=TMatr.getAMatrB(n,bpsiHf,0,bbasis,n+k*n,bQQi,n)*psik_psi0[k];
	}
	UnitTests.checkDoubleRel(-uspos/(dI*sos),"1",1,"1",eps);
    }
    if(eps>0)
    {
	EVXData.test_orthogonality("PnLdIpx.get_PnLdIpx:I.ev",n,I.ev,mm.QQi,mm.QQt,eps); // old basis
	EVXData.test_orthogonality("PnLdIpx.get_PnLdIpx:bbasis",n,bbasis,bQQi_feq,TMatr.getUnitMatrix(n),eps); // new basis
	final double [] bQQi=TMatr.getDiagMatrix(I.getEValues());
	// check some matrix elements
	UnitTests.checkDoubleRel(TMatr.mulVV(psik_psi0,psik_psi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsi0,bQQi_feq,n),"<psi0|I^f|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(psik_psi0,java.util.Arrays.copyOf(bbasis,n)),"<psi0|If|psi0>",I_0_feq,"I_0_feq",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(bpsi0,bpsi0),"<psi0|psi0>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(wHF),"sum(wHF)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(w0B),"sum(w0B)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.sumElements(w0F),"sum(w0F)",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(bpsi0,I.getEValues()),"<psi0|I|psi0>",I.s0,"I.s0",eps);
	UnitTests.checkDoubleRel(TMatr.mulV2V(bpsiHf,I.getEValues()),"<psiHf|I|psiHf>",TMatr.mulVVn1(wHF,I.ev),"sum(wHF*I.ev)",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(bpsiHf,bQQi,n),"<psiHf|I|psiHf>",TMatr.mulVVn1(wHF,I.ev),"sum(wHF*I.ev)",eps);
	UnitTests.checkDoubleRel(TMatr.mulVV(wHF,TMatr.getMatrixDiagElements(n,bQQi)),"<psiHf|I|psiHf>",TMatr.mulVVn1(wHF,I.ev),"sum(wHF*I.ev)",eps);
	UnitTests.checkDoubleRel(I_IHf,"I_IHf",TMatr.mulVVn1(wHF,I.ev),"sum(wHF*I.ev)",eps);

	UnitTests.checkDoubleRel(I_0_feq,"I_0_feq",TMatr.mulVV(wBF,psik_I_psik)+deg_BIH*dI,"TMatr.mulVV(wHF,I.ev)+deg_BIH*dI",eps);
	UnitTests.checkDoubleRel(I.wH,"I.wH",TMatr.mulVV(psik_psiB,psik_psi0),"<psiB|psi0>",eps);
	UnitTests.checkDoubleRel(bpsi0[ixH],"bpsi0[ixH]",TMatr.mulVV(psik_psiB,psik_psi0),"<psiB|psi0>",eps);
	for(int k=0;k<n;k++)
	{
	    final double [] psik=EVXData.convertPsiFromBtoQ_Basis(n,EVXData.getPsiFromBasis(k,bbasis,n),I.ev);
	    UnitTests.checkDoubleRel(psik_psiB[k],"psik_psiB[k]",TMatr.getAMatrB(psik,I.psiH,mm.QQt,n),"<psik|psiB>",eps);
	    UnitTests.checkDoubleRel(psik_psi0[k],"psik_psi0[k]",TMatr.getAMatrB(psik,I.psi0,mm.QQt,n),"<psik|psi0>",eps);
	}
	UnitTests.checkDoubleRel((bQQi[n*ixH+ixH]-TMatr.mulVV(wBF,psik_I_psik))/(dI*deg_BIH),"1",1,"1",eps);
	UnitTests.checkDoubleRel((bQQi[ixH+n*ixH]-TMatr.mulVV(w0F,psik_I_psik))/(dI*TMatr.mulVV(w0F,w0F)),"1",1,"1",eps);


	System.err.println("PnLdIpx.get_PnLdIpx selftest deg_BIH="+deg_BIH+" OK n="+n+" eps="+eps);
    }
    return new PnLdIpx(I,mm.p_offset,mm.p_last,
		       bQQpi[ixH+n*ixH]/I.ev[ixH],I_0_feq,
		       Pf_from_dyn,deg_from_dyn,Pf_from_f0,
		       degt_from_dyn,Pft_from_f0,delta_Pdyn_from_dyn,delta_P_IHf,
		       PI_IHf/I_IHf,PI_IHf_dyn/I_IHf,deg_BIH,P_BIH,
		       (bQQpi[ixH+n*ixH]-TMatr.getAMatrA(bpsi0,bQQpi,n))/(I.ev[ixH]-I.s0),
		       (bQQpi[ixH+n*ixH]-TMatr.mulVV(w0F,psik_pI_psik))/(dI*TMatr.mulVV(w0F,w0F)),
		       deg2_Hf,Pf2_Hf,Pf2o_Hf,Pf2os_Hf,Pf_from_xx,
		       eps);
}

}
