package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.utils.ReadWriteUtils;

/** Prepare filestem & cost for C5 calculation.
 *  @version $Id: PrepareFilestems.java,v 1.133 2022/07/28 21:03:03 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PrepareFilestems
{

    static final int N_PREEDICTORS=6;


    private static final boolean PRINT_PRG_DEBUG=true;




   static class ConsequentStat {
       final long tau_discretization,tau_look_ahead;
       final String filestem;
       final String aname;
       final String refattrib;
       final double level;
       final int n_points;
       final int n_points_out_of_level;
       final String [] antecedents_to_ignore,antecedent_tags_to_ignore;
       ConsequentStat(final String filestem,
		      final String aname,
		      final String refattrib,
		      final double level,
		      final double aver_mod,
		      final int n_points,
		      final int n_points_out_of_level,
		      final long tau_discretization,
		      final long tau_look_ahead,
		      final String [] antecedents_to_ignore,
		      final String [] antecedent_tags_to_ignore)
	{
	    this.filestem=filestem;
	    this.aname=aname;
	    this.refattrib=refattrib;
	    this.level=level;
	    this.n_points=n_points;
	    this.n_points_out_of_level=n_points_out_of_level;
	    this.tau_discretization=tau_discretization;
	    this.tau_look_ahead=tau_look_ahead;
	    this.antecedents_to_ignore=antecedents_to_ignore;
	    this.antecedent_tags_to_ignore=antecedent_tags_to_ignore;
	}
       final String getDesctiption()
       {
	   return this.filestem+" level="+this.level+
	       " n_points="+this.n_points+
	       " n_points_out_of_level="+this.n_points_out_of_level+
	       " tau_discretization="+this.tau_discretization+
	       " tau_look_ahead="+this.tau_look_ahead;
       }


	static ConsequentStat readConsequentStat(final String filename)
	    throws java.io.IOException
	{
	    final java.util.Map<String,String> m=
		com.polytechnik.outils.EncodeDecode.decodeURLStringSingleValue(
		    ReadWriteUtils.getFileContent(filename));
	    return new ConsequentStat(m.get("filestem"),
				      m.get("aname"),
				      m.get("refattrib"),
				      Double.parseDouble(m.get("level")),
				      Double.NaN,
				      Integer.parseInt(m.get("n_points")),
				      Integer.parseInt(m.get("n_points_out_of_level")),
				      Long.parseLong(m.get("tau_discretization")),
				      Long.parseLong(m.get("tau_look_ahead")),
				      com.polytechnik.outils.EncodeDecode.decodeStringArray(m.get("antecedents_to_ignore")),
				      com.polytechnik.outils.EncodeDecode.decodeStringArray(m.get("antecedents_tags_to_ignore"))
				      );
	}

	String getConsequentStatAsString() throws java.io.UnsupportedEncodingException
	{
	    final StringBuilder buf=new StringBuilder(200);
	    com.polytechnik.outils.EncodeDecode.appendField(buf,"filestem",this.filestem);
	    com.polytechnik.outils.EncodeDecode.appendField(buf,"aname",this.aname);
	    com.polytechnik.outils.EncodeDecode.appendField(buf,"refattrib",this.refattrib);
	    com.polytechnik.outils.EncodeDecode.appendField(buf,"level",String.valueOf(this.level));
	    com.polytechnik.outils.EncodeDecode.appendField(buf,"n_points",String.valueOf(this.n_points));
	    com.polytechnik.outils.EncodeDecode.appendField(buf,"n_points_out_of_level",String.valueOf(this.n_points_out_of_level));
	    com.polytechnik.outils.EncodeDecode.appendField(buf,"tau_discretization",String.valueOf(this.tau_discretization));
	    com.polytechnik.outils.EncodeDecode.appendField(buf,"tau_look_ahead",String.valueOf(this.tau_look_ahead));
	    com.polytechnik.outils.EncodeDecode.appendField(buf,"antecedents_to_ignore",this.antecedents_to_ignore);
	    com.polytechnik.outils.EncodeDecode.appendField(buf,"antecedent_tags_to_ignore",this.antecedent_tags_to_ignore);
	    return buf.toString();
	}
    }





    private static class CalculateLevelsTau implements
						java.util.function.Predicate<String []>
    {
	private final FilestemData.NamesFileData nd;
	private final String namesprefix;
	private final ModelParameters.ModelParametersData mparametersdata;
	final int [] ptr_min,ptr_max,ptr_med_buy,ptr_med_sell,ptr_dtau_buy,ptr_dtau_sell;
	private final double [][] cnsq_min_points,cnsq_max_points;
	private final int [] n_cnsq_min_points,n_cnsq_max_points;
	private final double [][] cnsq_med_buy_points,cnsq_med_sell_points;
	private final int [] n_cnsq_med_buy_points,n_cnsq_med_sell_points;
	private final double [][] cnsq_dtau_buy_points,cnsq_dtau_sell_points;
	private final int [] n_cnsq_dtau_buy_points,n_cnsq_dtau_sell_points;
	CalculateLevelsTau(final FilestemData.NamesFileData nd,
			   final String namesprefix,
			   final ModelParameters.ModelParametersData mparametersdata)
	{
	    this.nd=nd;
	    this.namesprefix=namesprefix;
	    this.mparametersdata=mparametersdata;
	    final int nt=nd.tau_consequent.length;
	    if(CalculateConsequents.ATTRIB_MINMAX)
	    {
		this.ptr_min=new int [nt];
		this.ptr_max=new int [nt];
	    }
	    else
	    {
		this.ptr_min=this.ptr_max=null;
	    }

	    if(CalculateConsequents.ATTRIB_MEDIAN)
	    {
		this.ptr_med_buy=new int [nt];
		this.ptr_med_sell=new int [nt];
	    }
	    else
	    {
		this.ptr_med_buy=this.ptr_med_sell=null;
	    }

	    if(CalculateConsequents.ATTRIB_EXACTT)
	    {
		this.ptr_dtau_buy=new int [nt];
		this.ptr_dtau_sell=new int [nt];
	    }
	    else
	    {
		this.ptr_dtau_buy=this.ptr_dtau_sell=null;
	    }

	    if(PRINT_PRG_DEBUG)
	    {
		for(int j=0;j<nd.all_attribs.length;j++)
		{
		    System.err.println(nd.all_attribs[j]);
		}
	    }
	    final StringBuilder buf=new StringBuilder();
	    for(int i=0;i<nt;i++)
	    {
		// Use prefixes before t string to match reference consequent attribute.
		// we potentially can create several attributes from a single reference.
		// Attribute name is created by adding time to these prefixes:
		final String t=nd.tau_consequent[i];
		if(CalculateConsequents.ATTRIB_MINMAX)
		{
		    final String f=FilestemData.escape_fs(CalculateConsequents.PMIN_PREFIX,buf)+t;
		    if((this.ptr_min[i]=ReadWriteUtils.elementIndex(nd.all_attribs,f))<0)
		    {
			throw new RuntimeException(
			    "No field "+f+
			    " found in "+nd.filename+
			    " fields="+java.util.Arrays.toString(nd.all_attribs));
		    }
		}
		if(CalculateConsequents.ATTRIB_MEDIAN)
		{
		    final String f=FilestemData.escape_fs(CalculateConsequents.PMED_BUY_PREFIX,buf)+t;
		    if((this.ptr_med_buy[i]=ReadWriteUtils.elementIndex(nd.all_attribs,f))<0)
		    {
			throw new RuntimeException(
			    "No field "+f+
			    " found in "+nd.filename+
			    " fields="+java.util.Arrays.toString(nd.all_attribs));
		    }
		}
		if(CalculateConsequents.ATTRIB_MINMAX)
		{
		    final String f=FilestemData.escape_fs(CalculateConsequents.PMAX_PREFIX,buf)+t;
		    if((this.ptr_max[i]=ReadWriteUtils.elementIndex(nd.all_attribs,f))<0)
		    {
			throw new RuntimeException(
			    "No field "+f+
			    " found in "+nd.filename+
			    " fields="+java.util.Arrays.toString(nd.all_attribs));
		    }
		}
		if(CalculateConsequents.ATTRIB_MEDIAN)
		{
		    final String f=FilestemData.escape_fs(CalculateConsequents.PMED_SELL_PREFIX,buf)+t;
		    if((this.ptr_med_sell[i]=ReadWriteUtils.elementIndex(nd.all_attribs,f))<0)
		    {
			throw new RuntimeException(
			    "No field "+f+
			    " found in "+nd.filename+
			    " fields="+java.util.Arrays.toString(nd.all_attribs));
		    }
		}
		if(CalculateConsequents.ATTRIB_EXACTT)
		{
		    final String f=FilestemData.escape_fs(CalculateConsequents.PTAU_BUY_PREFIX,buf)+t;
		    if((this.ptr_dtau_buy[i]=ReadWriteUtils.elementIndex(nd.all_attribs,f))<0)
		    {
			throw new RuntimeException(
			    "No field "+f+
			    " found in "+nd.filename+
			    " fields="+java.util.Arrays.toString(nd.all_attribs));
		    }
		}
		if(CalculateConsequents.ATTRIB_EXACTT)
		{
		    final String f=FilestemData.escape_fs(CalculateConsequents.PTAU_SELL_PREFIX,buf)+t;
		    if((this.ptr_dtau_sell[i]=ReadWriteUtils.elementIndex(nd.all_attribs,f))<0)
		    {
			throw new RuntimeException(
			    "No field "+f+
			    " found in "+nd.filename+
			    " fields="+java.util.Arrays.toString(nd.all_attribs));
		    }
		}
	    }
	    // we use only data file for discretization
	    final int nd_lines=this.nd.getFLines(FilestemData.P_DATA);
	    if(CalculateConsequents.ATTRIB_MINMAX)
	    {
		this.cnsq_min_points=new double [nt][nd_lines];
		this.cnsq_max_points=new double [nt][nd_lines];
		this.n_cnsq_min_points=new int [nt];
		this.n_cnsq_max_points=new int [nt];
	    }
	    else
	    {
		this.cnsq_min_points=this.cnsq_max_points=null;
		this.n_cnsq_min_points=this.n_cnsq_max_points=null;
	    }
	    if(CalculateConsequents.ATTRIB_MEDIAN)
	    {
		this.cnsq_med_buy_points=new double [nt][nd_lines];
		this.cnsq_med_sell_points=new double [nt][nd_lines];
		this.n_cnsq_med_buy_points=new int [nt];
		this.n_cnsq_med_sell_points=new int [nt];
	    }
	    else
	    {
		this.cnsq_med_buy_points=this.cnsq_med_sell_points=null;
		this.n_cnsq_med_buy_points=this.n_cnsq_med_sell_points=null;
	    }
	    if(CalculateConsequents.ATTRIB_EXACTT)
	    {
		this.cnsq_dtau_buy_points=new double [nt][nd_lines];
		this.cnsq_dtau_sell_points=new double [nt][nd_lines];
		this.n_cnsq_dtau_buy_points=new int [nt];
		this.n_cnsq_dtau_sell_points=new int [nt];
	    }
	    else
	    {
		this.cnsq_dtau_buy_points=this.cnsq_dtau_sell_points=null;
		this.n_cnsq_dtau_buy_points=this.n_cnsq_dtau_sell_points=null;
	    }
	}
	public boolean test(final String [] b)
	{
	    for(int i=this.nd.tau_consequent.length;i-->0;)
	    {
		if(CalculateConsequents.ATTRIB_MINMAX)
		{
		    final String s=b[this.ptr_min[i]];
		    if(!s.equals("?"))
		    {
			this.cnsq_min_points[i][this.n_cnsq_min_points[i]++]=Double.parseDouble(s);
		    }
		}
		if(CalculateConsequents.ATTRIB_MEDIAN)
		{
		    final String s=b[this.ptr_med_sell[i]];
		    if(!s.equals("?"))
		    {
			this.cnsq_med_sell_points[i][this.n_cnsq_med_sell_points[i]++]=Double.parseDouble(s);
		    }
		}
		if(CalculateConsequents.ATTRIB_MINMAX)
		{
		    final String s=b[this.ptr_max[i]];
		    if(!s.equals("?"))
		    {
			this.cnsq_max_points[i][this.n_cnsq_max_points[i]++]=Double.parseDouble(s);
		    }
		}
		if(CalculateConsequents.ATTRIB_MEDIAN)
		{
		    final String s=b[this.ptr_med_buy[i]];
		    if(!s.equals("?"))
		    {
			this.cnsq_med_buy_points[i][this.n_cnsq_med_buy_points[i]++]=Double.parseDouble(s);
		    }
		}
		if(CalculateConsequents.ATTRIB_EXACTT)
		{
		    final String s=b[this.ptr_dtau_buy[i]];
		    if(!s.equals("?"))
		    {
			this.cnsq_dtau_buy_points[i][this.n_cnsq_dtau_buy_points[i]++]=Double.parseDouble(s);
		    }
		}
		if(CalculateConsequents.ATTRIB_EXACTT)
		{
		    final String s=b[this.ptr_dtau_sell[i]];
		    if(!s.equals("?"))
		    {
			this.cnsq_dtau_sell_points[i][this.n_cnsq_dtau_sell_points[i]++]=Double.parseDouble(s);
		    }
		}

	    }
	    return true;
	}

	private ConsequentStat getConsequentBelow(final String txt,
						  final String aname,
						  final String ra,
						  final double [] min,
						  final int n_min,
						  final double fixedlevel,
						  final int fracdiv,
						  final long tau_discretization,
						  final long tau_look_ahead)
	{
	    int n_op_min_below=0;
	    double minop;
	    if(n_min>0)
	    {
		com.polytechnik.outils.TMPQSortAlgorithm.doubleQuickSort(min,0,n_min-1);
		if(fracdiv>0)
		{
		    minop=min[n_min/fracdiv];
		    if(!(minop<=fixedlevel))
		    {
			minop=fixedlevel;
		    }
		}
		else
		{
		    minop=fixedlevel;
		}
		for(int j=0;j<n_min;j++)
		{
		    // compare with thresholds
		    final double v_min=min[j];
		    if(v_min<minop)
		    {
			n_op_min_below++;
		    }
		    if(j>0 && min[j-1]>min[j])
		    {
			throw new RuntimeException("Unsorted min_points");
		    }
		}
		if(fracdiv>0 && n_op_min_below>n_min/fracdiv)
		{
		    throw new RuntimeException("Error in discretization n_op_min_below="+n_op_min_below+" n_min="+n_min+" fracdiv="+fracdiv+" minop="+minop+" fixedlevel="+fixedlevel);
		}
	    }
	    else
	    {
		minop=fixedlevel;
	    }
	    return new ConsequentStat(txt,
				      aname,
				      ra,
				      minop,
				      Double.NaN,
				      n_min,
				      n_op_min_below,
				      tau_discretization,
				      tau_look_ahead,
				      getAntecedentsToIgnore(),
				      getAntecedentTagsToIgnore_Down());
	}

	private ConsequentStat getConsequentAbove(final String txt,
						  final String aname,
						  final String ra,
						  final double [] max,
						  final int n_max,
						  final double fixedlevel,
						  final int fracdiv,
						  final long tau_discretization,
						  final long tau_look_ahead)
	{
	    int n_op_max_above=0;
	    double maxop;
	    if(n_max>0)
	    {
		com.polytechnik.outils.TMPQSortAlgorithm.doubleQuickSort(max,0,n_max-1);
		if(fracdiv>0)
		{
		    maxop=max[n_max-1-n_max/fracdiv];
		    if(!(maxop>=fixedlevel))
		    {
			maxop=fixedlevel;
		    }
		}
		else
		{
		    maxop=fixedlevel;
		}
		for(int j=n_max;j-- >0;)
		{
		    // compare with thresholds
		    final double v_max=max[j];
		    if(v_max>maxop)
		    {
			n_op_max_above++;
		    }
		    if(j>0 && max[j-1]>max[j])
		    {
			throw new RuntimeException("Unsorted max_points");
		    }
		}
		if(fracdiv>0 && n_op_max_above>n_max/fracdiv)
		{
		    throw new RuntimeException("Error in discretization n_op_max_above="+n_op_max_above+" n_max="+n_max+" fracdiv="+fracdiv+" maxop="+maxop+" fixedlevel="+fixedlevel);
		}
	    }
	    else
	    {
		maxop=fixedlevel;
	    }
	    return new ConsequentStat(txt,
				      aname,
				      ra,
				      maxop,
				      Double.NaN,
				      n_max,
				      n_op_max_above,
				      tau_discretization,
				      tau_look_ahead,
				      getAntecedentsToIgnore(),
				      getAntecedentTagsToIgnore_Up()
						   );
	}


	String [] getAntecedentTagsToIgnore_Down()
	{
	    final StringBuilder buf=new StringBuilder();
	    return new String []{FilestemData.escape_fs(CalculateAntecedents.OPT_SELL,buf)};
	}

	String [] getAntecedentTagsToIgnore_Up()
	{
	    final StringBuilder buf=new StringBuilder();
	    return new String []{FilestemData.escape_fs(CalculateAntecedents.OPT_BUY,buf)};
	}

	String [] getAntecedentsToIgnore()
	{
	    return new String [0];
	}



	ConsequentStat [][] getConsequentStat()
	{
	    final int nt=this.nd.tau_consequent.length;
	    final StringBuilder buf=new StringBuilder();

	    final ConsequentStat [][] c=new ConsequentStat [nt][];
	    for(int i=0;i<nt;i++)
	    {
		final long tau_discretization=this.mparametersdata.tau_training_quant;
		final long tau_look_ahead=Long.parseLong(this.nd.tau_consequent[i])*ModelParameters.DISTANCE_TIME_SCALE;

		final ConsequentStat [] ci=c[i]=new ConsequentStat [PrepareFilestems.N_PREEDICTORS];
		final String fpr=this.namesprefix+"_F_";
		final String t=this.nd.tau_consequent[i];

		if(CalculateConsequents.ATTRIB_MINMAX)
		{
		    final String s=this.nd.all_attribs[this.ptr_min[i]];
		    ci[0]=getConsequentBelow(fpr+FilestemData.escape_fs(CalculateConsequents.PMIN_PREFIX,buf)+t,
					     s+"__discr_min_sell",
					     s,
					     this.cnsq_min_points[i],
					     this.n_cnsq_min_points[i],
					     1.0/this.mparametersdata.MINMAX_LEVEL,
					     this.mparametersdata.FRAC_MINMAX_DIV,
					     tau_discretization,
					     tau_look_ahead);
		    this.cnsq_min_points[i]=null; // poisoned, do not use
		}

		if(CalculateConsequents.ATTRIB_MEDIAN)
		{
		    final String s=this.nd.all_attribs[this.ptr_med_sell[i]];
		    ci[1]=getConsequentBelow(fpr+FilestemData.escape_fs(CalculateConsequents.PMED_SELL_PREFIX,buf)+t,
					     s+"__discr_med_sell",
					     s,
					     this.cnsq_med_sell_points[i],
					     this.n_cnsq_med_sell_points[i],
					     1.0/this.mparametersdata.MEDIAN_LEVEL,
					     this.mparametersdata.FRAC_MEDIAN_DIV,
					     tau_discretization,
					     tau_look_ahead);
		    this.cnsq_med_sell_points[i]=null; // poisoned, do not use
		}

		if(CalculateConsequents.ATTRIB_MINMAX)
		{
		    final String s=this.nd.all_attribs[this.ptr_max[i]];
		    ci[2]=getConsequentAbove(fpr+FilestemData.escape_fs(CalculateConsequents.PMAX_PREFIX,buf)+t,
					     s+"__discr_max_buy",
					     s,
					     this.cnsq_max_points[i],
					     this.n_cnsq_max_points[i],
					     this.mparametersdata.MINMAX_LEVEL,
					     this.mparametersdata.FRAC_MINMAX_DIV,
					     tau_discretization,
					     tau_look_ahead);
		    this.cnsq_max_points[i]=null; // poisoned, do not use
		}

		if(CalculateConsequents.ATTRIB_MEDIAN)
		{
		    final String s=this.nd.all_attribs[this.ptr_med_buy[i]];
		    ci[3]=getConsequentAbove(fpr+FilestemData.escape_fs(CalculateConsequents.PMED_BUY_PREFIX,buf)+t,
					     s+"__discr_med_buy",
					     s,
					     this.cnsq_med_buy_points[i],
					     this.n_cnsq_med_buy_points[i],
					     this.mparametersdata.MEDIAN_LEVEL,
					     this.mparametersdata.FRAC_MEDIAN_DIV,
					     tau_discretization,
					     tau_look_ahead);
		    this.cnsq_med_buy_points[i]=null; // poisoned, do not use
		}

		if(CalculateConsequents.ATTRIB_EXACTT)
		{
		    final String s=this.nd.all_attribs[this.ptr_dtau_buy[i]];
		    ci[4]=getConsequentAbove(fpr+FilestemData.escape_fs(CalculateConsequents.PTAU_BUY_PREFIX,buf)+t,
					     s+"__discr_dtau_buy",
					     s,
					     this.cnsq_dtau_buy_points[i],
					     this.n_cnsq_dtau_buy_points[i],
					     this.mparametersdata.DTAU_LEVEL,
					     this.mparametersdata.FRAC_DTAU_DIV,
					     tau_discretization,
					     tau_look_ahead);
		    this.cnsq_dtau_buy_points[i]=null; // poisoned, do not use
		}

		if(CalculateConsequents.ATTRIB_EXACTT)
		{
		    final String s=this.nd.all_attribs[this.ptr_dtau_sell[i]];
		    ci[5]=getConsequentBelow(fpr+FilestemData.escape_fs(CalculateConsequents.PTAU_SELL_PREFIX,buf)+t,
					     s+"__discr_dtau_sell",
					     s,
					     this.cnsq_dtau_sell_points[i],
					     this.n_cnsq_dtau_sell_points[i],
					     1.0/this.mparametersdata.DTAU_LEVEL,
					     this.mparametersdata.FRAC_DTAU_DIV,
					     tau_discretization,
					     tau_look_ahead);
		    this.cnsq_dtau_sell_points[i]=null; // poisoned, do not use
		}

	    }
	    return c;
	}
    }





    private static class PrintConsequentDataFile implements
					     java.util.function.Predicate<String []>
    {
	private final FilestemData.NamesFileData nd;
	private final ConsequentStat [][] cs;
	private final java.io.PrintWriter out;
	private final int [] ptr_min,ptr_max,ptr_med_sell,ptr_med_buy,ptr_dtau_sell,ptr_dtau_buy;
	private final int []
	    n_op_min_below,n_min_total,
	    n_med_sell_down,n_med_sell_down_total,
	    n_op_max_above,n_max_total,
	    n_med_buy_up,n_med_buy_up_total,
	    n_dtau_buy_up,n_dtau_buy_up_total,
	    n_dtau_sell_down,n_dtau_sell_down_total;

	PrintConsequentDataFile(final FilestemData.NamesFileData nd,
				final ConsequentStat [][] cs,
				final java.io.PrintWriter out,
				final int [] ptr_min,
				final int [] ptr_med_sell,
				final int [] ptr_max,
				final int [] ptr_med_buy,
				final int [] ptr_dtau_buy,
				final int [] ptr_dtau_sell)
	{
	    this.nd=nd;
	    this.cs=cs;
	    final int nt=nd.tau_consequent.length;
	    this.out=out;
	    if(CalculateConsequents.ATTRIB_MINMAX)
	    {
		this.ptr_min=ptr_min;
		this.ptr_max=ptr_max;
		this.n_op_min_below=new int [nt];
		this.n_min_total=new int [nt];
		this.n_op_max_above=new int [nt];
		this.n_max_total=new int [nt];
	    }
	    else
	    {
		if(ptr_min!=null||ptr_max!=null)
		{
		    throw new IllegalArgumentException();
		}
		this.ptr_min=
		this.ptr_max=
		this.n_op_min_below=
		this.n_min_total=
		this.n_op_max_above=
		this.n_max_total=null;
	    }

	    if(CalculateConsequents.ATTRIB_MEDIAN)
	    {
		this.ptr_med_sell=ptr_med_sell;
		this.ptr_med_buy=ptr_med_buy;
		this.n_med_sell_down=new int [nt];
		this.n_med_sell_down_total=new int [nt];
		this.n_med_buy_up=new int [nt];
		this.n_med_buy_up_total=new int [nt];
	    }
	    else
	    {
		if(ptr_med_sell!=null||ptr_med_buy!=null)
		{
		    throw new IllegalArgumentException();
		}
		this.ptr_med_sell=
		this.ptr_med_buy=
		this.n_med_sell_down=
		this.n_med_sell_down_total=
		this.n_med_buy_up=
		this.n_med_buy_up_total=null;
	    }

	    if(CalculateConsequents.ATTRIB_EXACTT)
	    {
		this.ptr_dtau_sell=ptr_dtau_sell;
		this.ptr_dtau_buy=ptr_dtau_buy;
	        this.n_dtau_buy_up=new int [nt];
		this.n_dtau_buy_up_total=new int [nt];
		this.n_dtau_sell_down=new int [nt];
		this.n_dtau_sell_down_total=new int [nt];
	    }
	    else
	    {
		if(ptr_dtau_sell!=null||ptr_dtau_buy!=null)
		{
		    throw new IllegalArgumentException();
		}
		this.ptr_dtau_sell=
		this.ptr_dtau_buy=
	        this.n_dtau_buy_up=
		this.n_dtau_buy_up_total=
		this.n_dtau_sell_down=
		this.n_dtau_sell_down_total=null;
	    }

	    if(cs.length!=nt)
	    {
		throw new RuntimeException();
	    }
	}

	private static void printDAttrUP(final String [] data,
					 final int i,
					 final java.io.PrintWriter out,
					 final int [] ptr,
					 final int [] n_lev,
					 final int [] n_total,
					 final ConsequentStat op)
	{

	    out.print(",");
	    final String s=data[ptr[i]];
	    final double v=!s.equals("?")?Double.parseDouble(s):Double.NaN;
	    if(!Double.isNaN(v))
	    {
		if(v>op.level)
		{
		    n_lev[i]++;
		    out.print("1");
		}
		else
		{
		    out.print("0");
		}
		n_total[i]++;
	    }
	    else
	    {
		out.print("?");
	    }
	}

	private static void printDAttrDOWN(final String [] data,
					   final int i,
					   final java.io.PrintWriter out,
					   final int [] ptr,
					   final int [] n_lev,
					   final int [] n_total,
					   final ConsequentStat op)
	{

	    out.print(",");
	    final String s=data[ptr[i]];
	    final double v=!s.equals("?")?Double.parseDouble(s):Double.NaN;
	    if(!Double.isNaN(v))
	    {
		if(v<op.level)
		{
		    n_lev[i]++;
		    out.print("1");
		}
		else
		{
		    out.print("0");
		}
		n_total[i]++;
	    }
	    else
	    {
		out.print("?");
	    }
	}

	public boolean test(final String [] b)
	{
	    for(int j=0;j<b.length;j++)
	    {
		if(j>0)
		{
		    this.out.print(",");
		}
		this.out.print(b[j]);
	    }
	    final ConsequentStat [][] ci=this.cs;
	    for(int i=0,nt=ci.length;i<nt;i++)
	    {
		final ConsequentStat [] c=ci[i];
		if(c.length!=PrepareFilestems.N_PREEDICTORS)
		{
		    throw new IllegalStateException();
		}
		if(CalculateConsequents.ATTRIB_MINMAX)
		{
		    printDAttrDOWN(b,
				   i,
				   this.out,
				   this.ptr_min,
				   this.n_op_min_below,
				   this.n_min_total,
				   c[0]);
		}

		if(CalculateConsequents.ATTRIB_MEDIAN)
		{
		    printDAttrDOWN(b,
				   i,
				   this.out,
				   this.ptr_med_sell,
				   this.n_med_sell_down,
				   this.n_med_sell_down_total,
				   c[1]);
		}

		if(CalculateConsequents.ATTRIB_MINMAX)
		{
		    printDAttrUP(b,
				 i,
				 this.out,
				 this.ptr_max,
				 this.n_op_max_above,
				 this.n_max_total,
				 c[2]);
		}

		if(CalculateConsequents.ATTRIB_MEDIAN)
		{
		    printDAttrUP(b,
				 i,
				 this.out,
				 this.ptr_med_buy,
				 this.n_med_buy_up,
				 this.n_med_buy_up_total,
				 c[3]);
		}

		if(CalculateConsequents.ATTRIB_EXACTT)
		{
		    printDAttrUP(b,
				 i,
				 this.out,
				 this.ptr_dtau_buy,
				 this.n_dtau_buy_up,
				 this.n_dtau_buy_up_total,
				 c[4]);
		}

		if(CalculateConsequents.ATTRIB_EXACTT)
		{
		    printDAttrDOWN(b,
				   i,
				   this.out,
				   this.ptr_dtau_sell,
				   this.n_dtau_sell_down,
				   this.n_dtau_sell_down_total,
				   c[5]);
		}
	    }
	    this.out.println(",EOL");
	    return true;
	}
	void makeHeader()
	{
	    this.out.print("|");
	    for(int i=0;i<this.nd.all_attribs.length;i++)
	    {
		if(i>0)
		{
		    this.out.print(",");
		}
		this.out.print(this.nd.all_attribs[i]);
	    }
	    for(int i=0,nt=this.nd.tau_consequent.length;i<nt;i++)
	    {
		final ConsequentStat [] ci=cs[i];
		if(CalculateConsequents.ATTRIB_MINMAX)
		{
		    this.out.print(",");
		    this.out.print(ci[0].aname);
		}
		if(CalculateConsequents.ATTRIB_MEDIAN)
		{
		    this.out.print(",");
		    this.out.print(ci[1].aname);
		}
		if(CalculateConsequents.ATTRIB_MINMAX)
		{
		    this.out.print(",");
		    this.out.print(ci[2].aname);
		}
		if(CalculateConsequents.ATTRIB_MEDIAN)
		{
		    this.out.print(",");
		    this.out.print(ci[3].aname);
		}
		if(CalculateConsequents.ATTRIB_EXACTT)
		{
		    this.out.print(",");
		    this.out.print(ci[4].aname);
		}
		if(CalculateConsequents.ATTRIB_EXACTT)
		{
		    this.out.print(",");
		    this.out.print(ci[5].aname);
		}
	    }
	    this.out.println(",__Lattrib2__");
	}

	void checkStats()
	{
	    final ConsequentStat [][] ci=this.cs;
	    for(int i=0,nt=this.cs.length;i<nt;i++)
	    {
		final ConsequentStat [] c=ci[i];
		if(c.length!=N_PREEDICTORS)
		{
		    throw new IllegalStateException();
		}
		if((CalculateConsequents.ATTRIB_MINMAX && (
		       n_op_min_below[i]!=c[0].n_points_out_of_level||
		       n_min_total[i]!=c[0].n_points||
		       n_op_max_above[i]!=c[2].n_points_out_of_level||
		       n_max_total[i]!=c[2].n_points))||
		   (CalculateConsequents.ATTRIB_MEDIAN && (
		       n_med_sell_down[i]!=c[1].n_points_out_of_level||
		       n_med_sell_down_total[i]!=c[1].n_points||
		       n_med_buy_up[i]!=c[3].n_points_out_of_level||
		       n_med_buy_up_total[i]!=c[3].n_points))||
		   (CalculateConsequents.ATTRIB_EXACTT && (
		       n_dtau_buy_up[i]!=c[4].n_points_out_of_level||
		       n_dtau_buy_up_total[i]!=c[4].n_points||
		       n_dtau_sell_down[i]!=c[5].n_points_out_of_level||
		       n_dtau_sell_down_total[i]!=c[5].n_points))
		   )
		{
		    throw new RuntimeException("Stat mismatch for one of "+
					       (c[0]!=null?c[0].filestem+": "+n_op_min_below[i]+" "+c[0].n_points_out_of_level+"; "+n_min_total[i]+" "+c[0].n_points+"; ":"N/A")+";"+
					       (c[1]!=null?c[1].filestem+": "+n_med_sell_down[i]+" "+c[1].n_points_out_of_level+"; "+n_med_sell_down_total[i]+" "+c[1].n_points+"; ":"N/A")+";"+
					       (c[2]!=null?c[2].filestem+": "+n_op_max_above[i]+" "+c[2].n_points_out_of_level+"; "+n_max_total[i]+" "+c[2].n_points+"; ":"N/A")+";"+
					       (c[3]!=null?c[3].filestem+": "+n_med_buy_up[i]+" "+c[3].n_points_out_of_level+"; "+n_med_buy_up_total[i]+" "+c[3].n_points:"N/A")+";"+
					       (c[4]!=null?c[4].filestem+": "+n_dtau_buy_up[i]+" "+c[4].n_points_out_of_level+"; "+n_dtau_buy_up_total[i]+" "+c[4].n_points:"N/A")+";"+
					       (c[5]!=null?c[5].filestem+": "+n_dtau_sell_down[i]+" "+c[5].n_points_out_of_level+"; "+n_dtau_sell_down_total[i]+" "+c[5].n_points:"N/A")+";"
					       );
		}
	    }
	}
    }


    private void convertDataFile(final String outfile,
				 final FilestemData.NamesFileData nd,
				 final ConsequentStat [][] cs,
				 final int [] ptr_min,
				 final int [] ptr_med_sell,
				 final int [] ptr_max,
				 final int [] ptr_med_buy,
				 final int [] ptr_dtau_buy,
				 final int [] ptr_dtau_sell,
				 final boolean chek_stats,
				 final int data_or_test_type)
	throws java.io.IOException
    {
	final String infile=nd.getFData(data_or_test_type);
	if(PRINT_PRG_DEBUG)
	{
	    System.err.print("convertDataFile:: "+infile+" to "+outfile);
	}
	final java.io.PrintWriter out=new java.io.PrintWriter(new java.io.FileWriter(outfile));
	final PrintConsequentDataFile p=new PrintConsequentDataFile(nd,
								    cs,
								    out,
								    ptr_min,
								    ptr_med_sell,
								    ptr_max,
								    ptr_med_buy,
								    ptr_dtau_buy,
								    ptr_dtau_sell);
	p.makeHeader();
	// add this comment, so no possible call of FilestemData.processDataFile for this file
	out.println("|_infile="+infile);
	FilestemData.processDataFile(nd,p,data_or_test_type);
	out.close();
	if(chek_stats)
	{
	    p.checkStats();
	    if(PRINT_PRG_DEBUG)
	    {
		System.err.print(" stats OK;");
	    }
	}
	else
	{
	    if(PRINT_PRG_DEBUG)
	    {
		System.err.print(" no need to check stats;");
	    }
	}
	if(PRINT_PRG_DEBUG)
	{
	    System.err.println(" Processed OK.");
	}
    }


    private void convertNamesFiles(final String infile,
				   final FilestemData.NamesFileData nd,
				   final ConsequentStat [][] cs)
	throws java.io.IOException
    {
	if(PRINT_PRG_DEBUG)
	{
	    System.err.println("convertNamesFiles:: "+infile);
	}
	final String names=com.polytechnik.utils.ReadWriteUtils.getFileContent(infile);
	for(int i=0;i<cs.length;i++)
	{
	    // create .names file
	    final ConsequentStat [] ci=cs[i];
	    for(int it=0;it<ci.length;it++)
	    {
		final ConsequentStat c=ci[it];
		if(c==null)
		{
		    continue;
		}
		if(PRINT_PRG_DEBUG)
		{
		    System.err.println("\t saving to "+c.filestem+".names"+
				       " attrib "+c.aname+" level="+c.level);
		}
		final java.io.PrintWriter out=new java.io.PrintWriter(new java.io.FileWriter(c.filestem+".names"));
		out.println(c.aname+".");
		out.println("|_level="+c.level);
		out.println("|_refattrib="+c.refattrib);
		out.println("|_infile="+infile);
		out.print(names);
		for(int j=0;j<cs.length;j++)
		{
		    final ConsequentStat [] cj=cs[j];
		    for(int jt=0;jt<cj.length;jt++)
		    {
			if(cj[jt]==null)
			{
			    continue;
			}

			out.print(cj[jt].aname);
			if(i==j && it==jt)
			{
			    out.print(":0,1");
			}
			else
			{
			    out.print(":ignore");
			}
			out.println();
		    }
		}
		out.println("__Lattrib2__:xEOL,EOL");

		// Skip some antecedent attributes from infile according to
		// marks in ConsequentStat
		if(c.antecedents_to_ignore.length>0||
		   c.antecedent_tags_to_ignore.length>0)
		{
		    out.print("attributes excluded:");
		    int na=0;
		    for(String a : c.antecedents_to_ignore)
		    {
			if(na>0)
			{
			    out.print(",");
			}
			out.print(a);
			na++;
		    }
		    for(String a : nd.all_attribs)
		    {
			for(String p : c.antecedent_tags_to_ignore)
			{
			    if(a.startsWith(p))
			    {
				if(na>0)
				{
				    out.print(",");
				}
				out.print(a);
				na++;
				break;
			    }
			}
		    }
		    if(na>0)
		    {
			out.print(".");
		    }
		    out.println();
		}
		out.close();
	    }
	}
    }

    private void saveConsequentStat(final FilestemData.NamesFileData nd,
				    final ConsequentStat [][] cs)
	throws java.io.IOException
    {
	if(PRINT_PRG_DEBUG)
	{
	    System.err.print("saveConsequentStat::");
	}
	for(int i=0;i<cs.length;i++)
	{
	    for(int it=0;it<cs[i].length;it++)
	    {
		final ConsequentStat c=cs[i][it];
		if(c==null)
		{
		    continue;
		}
		// write consequent stat to a file
		if(PRINT_PRG_DEBUG)
		{
		    System.err.println("\t saving to "+c.filestem+".consequent_dicr");
		}
		final java.io.PrintWriter out=new java.io.PrintWriter(new java.io.FileWriter(c.filestem+".consequent_dicr"));
		out.print(c.getConsequentStatAsString());
		out.close();
	    }
	}
    }


    private void createCostsFiles(final FilestemData.NamesFileData nd,
				  final ConsequentStat [][] cs)
	throws java.io.IOException
    {
	for(int i=0;i<cs.length;i++)
	{
	    final ConsequentStat [] ci=cs[i];
	    for(int it=0;it<ci.length;it++)
	    {
		final ConsequentStat c=ci[it];
		if(c==null)
		{
		    continue;
		}
		// create .costs file
		if(PRINT_PRG_DEBUG)
		{
		    System.err.println("createCostsFiles:: "+c.filestem+".costs");
		}
		final java.io.PrintWriter out=new java.io.PrintWriter(new java.io.FileWriter(c.filestem+".costs"));
		out.println("0,0:0");
		if(c.n_points_out_of_level<=0||c.n_points_out_of_level>=c.n_points)
		{
		    out.println("1,0:1");
		    out.println("0,1:1");
		}
		else
		{
		    final double cx=1.0/c.n_points_out_of_level;
		    out.println("1,0:"+cx);
		    final double cxx=1.0/(c.n_points-c.n_points_out_of_level);
		    out.println("0,1:"+cxx);
		}
		out.println("1,1:0");
		out.close();
	    }
	}
    }

    private void createCommandsFiles(final java.io.PrintWriter out,
				     final String namesprefix,
				     final FilestemData.NamesFileData nd,
				     final ConsequentStat [][] cs,
				     final int n_processes_to_start_simultaneously,
				     final boolean test_exist)
	throws java.io.IOException
    {
	if(PRINT_PRG_DEBUG)
	{
	    System.err.println("createCommandsFiles:: printing commands");
	}
	out.println();
	out.println();
	out.println();
	out.println("# =======================================================================");
	out.println("echo \"DATA converted from filename="+nd.filename+" source="+nd.source+"\"");
	for(int i=0;i<cs.length;i++)
	{
	    for(int it=0;it<cs[i].length;it++)
	    {

		final ConsequentStat c=cs[i][it];
		if(c==null)
		{
		    continue;
		}
		// create commands file
		out.println();
		out.println("# !!!!!!! FILESTEM="+c.filestem);
		out.println("[ -a '"+c.filestem+".data' ] && unlink '"+c.filestem+".data'");
		out.println("ln -s '"+namesprefix+".common.data' '"+c.filestem+".data'");
		out.println("[ -a '"+c.filestem+".test' ] && unlink '"+c.filestem+".test'");
		if(test_exist)
		{
		    out.println("ln -s '"+namesprefix+".common.test' '"+c.filestem+".test'");
		}
		out.print(ClassificationPredictorC5.getCMD(c));
		if(n_processes_to_start_simultaneously>1)
		{
		    out.print(" & ");
		}
		out.println();
		if(n_processes_to_start_simultaneously>1)
		{
		    out.println("declare -a aprocesses=(`jobs -rp`) ;  [ \"${#aprocesses[@]}\" -ge "+
				n_processes_to_start_simultaneously+" ] && wait");
		}
		out.println("# !!!!!!! END OF FILESTEM="+c.filestem);
	    }
	}
	out.println("echo \"END DATA converted from filename="+nd.filename+" source="+nd.source+"\"");
	out.println("# =======================================================================");
    }


    void prepareData(final String names,
		     final int n_processes_to_start_simultaneously,
		     final java.io.PrintWriter outcmd,
		     final boolean testfile_exists,
		     final ModelParameters mparameters) throws java.io.IOException
    {
	final FilestemData.NamesFileData nd=FilestemData.NamesFileData.readNamesFileData(names);
	final String namesprefix=nd.getPrefix();
	// calculate consequent discretization levels (later saved to .consequent_dicr files)
	// for the original datafile
	final CalculateLevelsTau ctl=new CalculateLevelsTau(nd,
							    namesprefix,
							    mparameters.getParameters(nd.ticker));
	FilestemData.processDataFile(nd,ctl,FilestemData.P_DATA);
	final ConsequentStat [][] cs=ctl.getConsequentStat();

	// create data file where we compare with levels
	convertDataFile(namesprefix+".common.data",nd,cs,ctl.ptr_min,ctl.ptr_med_sell,ctl.ptr_max,ctl.ptr_med_buy,ctl.ptr_dtau_buy,ctl.ptr_dtau_sell,true,FilestemData.P_DATA);
	// names file
	convertNamesFiles(namesprefix+".names",nd,cs);

	// save ConsequentStat array to .consequent_dicr files
	saveConsequentStat(nd,cs);

	// create test file if necessary
	if(testfile_exists)
	{
	    convertDataFile(namesprefix+".common.test",nd,cs,ctl.ptr_min,ctl.ptr_med_sell,ctl.ptr_max,ctl.ptr_med_buy,ctl.ptr_dtau_buy,ctl.ptr_dtau_sell,false,FilestemData.P_TEST);
	}

	createCostsFiles(nd,cs);

	if(outcmd!=null)
	{
	    createCommandsFiles(outcmd,namesprefix,nd,cs,n_processes_to_start_simultaneously,testfile_exists);
	}

    }


    public static void main(final String [] args)
	throws java.io.IOException
    {

	for(String s : args)
	{
	    final FilestemData.NamesFileData nd=FilestemData.NamesFileData.readNamesFileData(s);
	    final int ns=nd.filename.lastIndexOf("/");
	    final String f=ns>0?nd.filename.substring(0,ns+1):"";
	    final ModelParameters mp=new ModelParameters(f,f,
							 new String []{nd.ticker});
	    new PrepareFilestems().prepareData(s,1,null,false,mp);
	}
    }



}
