package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */


/** A class to perform postconversion after processing.
 *  The data is read from a file.
 *  @version $Id: PrepareFilestemsPostConversion.java,v 1.13 2020/02/06 10:55:18 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class PrepareFilestemsPostConversion implements java.util.function.BiConsumer<String,FilestemData>
{

    final int N_PROCESSES_TO_START;
    java.io.PrintWriter outcmd;
    final ModelParameters mparameters;
    PrepareFilestemsPostConversion(final int N_PROCESSES_TO_START,
				   final ModelParameters mparameters)
    {
	this.N_PROCESSES_TO_START=N_PROCESSES_TO_START;
	this.outcmd=null;
	this.mparameters=mparameters;
    }


    void startPostProcess() throws java.io.IOException
    {
	if(outcmd!=null || this.perform_discretization)
	{
	    throw new IllegalStateException();
	}
	outcmd=new java.io.PrintWriter(new java.io.FileWriter(this.mparameters.out_dir+"learn.cmd.sh"));
	outcmd.println("#!/bin/bash -x");
	outcmd.println("# Commands to execute C5 learning.");
	outcmd.println("# Created on "+new java.util.Date());
	outcmd.println("# in the directory="+this.mparameters.out_dir);
	outcmd.println("#");
	outcmd.println("t_start=\"`date '+%s'`\"");
	outcmd.println("t_started=\"`date`\"");
	outcmd.println("echo \"Started $t_started\"");
	outcmd.flush();
    }

    void finishPostProcess() throws java.io.IOException
    {
	if(!this.perform_discretization)
	{
	    throw new IllegalStateException();
	}
	outcmd.println("wait");
	outcmd.println("t_end=\"`date '+%s'`\"");
	outcmd.println("echo \"Processing done in $[ $t_end - $t_start ] seconds\"");
	outcmd.println("echo \"DONE:`date` started $t_started\"");
	outcmd.close();
	outcmd=null;
    }

    private static final int P_DISCR=1+FilestemData.P_TEST,N_P=1+P_DISCR;

    private boolean perform_discretization=false;
    void setToDoDescritization()
    {
	if(this.perform_discretization)
	{
	    throw new IllegalStateException();
	}
	this.perform_discretization=true;
    }

    private final java.util.Map<String,int []> m=
	new java.util.HashMap<String,int []>();

    public void accept(final String v,final FilestemData fd)
    {
	int [] vc=m.get(v);
	if(vc==null)
	{
	    vc=new int [N_P];
	    m.put(v,vc);
	}
	final int t=fd.data_or_test_type;
	final int n_processed_before=vc[t];
	try {
	    vc[t]=fd.makedatafile(v,vc[FilestemData.P_DATA],vc[FilestemData.P_TEST]);
	    if(this.perform_discretization)
	    {
		if(vc[P_DISCR]!=0)
		{
		    // discretization can be performed only once
		    throw new IllegalStateException();
		}
		vc[P_DISCR]++;
		new PrepareFilestems().prepareData(v+".names",N_PROCESSES_TO_START,outcmd,vc[FilestemData.P_TEST]!=0,this.mparameters);
		outcmd.flush();
	    }
	} catch(final java.io.IOException e) {
	    e.printStackTrace(System.err);
	    throw new RuntimeException();
	}
    }

}
