package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.realtimebook.*;

/** Perturbation theory.
 *  @version $Id: PsiAverPertr.java,v 1.18 2020/09/27 20:33:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class PsiAverPertr {

final double fXx,mu,pertpsi,pertpsiC,pertp,fact,Pt,Pi,sigma2_Pt,sigma2_Pi,Ipsi;
final double [] psi;

PsiAverPertr(final int n,final double lam,final double sign,
	     final double [] imom,final double [] tmom,
	     final double [] pimom,final double [] ptmom,
	     final double [] p2imom,final double [] p2tmom,
	     final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final boolean flag_edgeatright,final double x0)
{
    final double [] mumom=new double [imom.length],pmumom=new double [imom.length];
    for(int k=imom.length;--k>=0;)
    {
	mumom[k]=sign*(imom[k]-lam*tmom[k]);
	pmumom[k]=sign*(pimom[k]-lam*ptmom[k]);
    }
    final double [] ev=M.EV.getGEV_Generalized(n,M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(mumom)),M.B.getQQMatr(n,mumom),true);
    final double [] Xx=new double [n],w=new double [n],pp=new double [n];
    for(int k=0;k<n;k++)
    {
	Xx[k]=ev[k];
	System.arraycopy(ev,n+k*n,pp,0,n);
	w[k]=1/Math.pow(M.B.getD0(pp,Xx[k]),2);
    }
    final int indX=DjQVM.getIndX0_NonSortedX(Xx,x0,flag_edgeatright);
    this.psi=new double [n];
    if(indX>=0)
    {
	this.fXx=Xx[indX];
	this.mu=w[indX];
	System.arraycopy(ev,n+indX*n,psi,0,n);
    }
    else
    {
	this.fXx=this.mu=Double.NaN;
	if(n>1)	java.util.Arrays.fill(psi,Double.NaN);
	else {
	    psi[0]=1;
	    psi[0]=1/Math.sqrt(mumom[0]*M.B.getD0(psi,x0));
	}
    }
    final double [] pertmom=M.B.getXQkMomentsFromQkMoments(tmom);
    for(int k=0;k<pertmom.length;k++)
    {
	pertmom[k]=-(pertmom[k]-this.fXx*tmom[k]);
    }
    final double [] QQpert=M.B.getQQMatr(n,pertmom),QQpmu=M.B.getQQMatr(n,pmumom);
    double psi_p_vpsi=0,psi_Cp_vpsi=0;
    for(int k=0;k<n;k++)
    {
	if(k!=indX)
	{
	    final double f=TMatr.getAMatrB(psi.length,psi,0,n,ev,n+k*n,QQpert,n)*
		TMatr.getAMatrB(psi.length,psi,0,n,ev,n+k*n,QQpmu,n)/(this.fXx-ev[k]);
	    psi_p_vpsi+=f;
	    if((flag_edgeatright && ev[k]<this.fXx)||
	       (!flag_edgeatright && ev[k]>this.fXx)) {psi_Cp_vpsi+=f;}
	}
    }
    this.pertpsi=lam*2*psi_p_vpsi;
    this.pertpsiC=lam*2*psi_Cp_vpsi;
    final double nt=TMatr.getAMatrA(psi.length,psi,M.B.getQQMatr(n,tmom),n),
	ni=TMatr.getAMatrA(psi.length,psi,M.B.getQQMatr(n,imom),n);
    this.fact=lam*nt*ni;
    this.Ipsi=ni/nt;
    this.Pt=TMatr.getAMatrA(psi.length,psi,M.B.getQQMatr(n,ptmom),n)/nt;
    this.sigma2_Pt=p2tmom==null?Double.NaN:(TMatr.getAMatrA(psi.length,psi,M.B.getQQMatr(n,p2tmom),n)/nt-this.Pt*this.Pt);
    this.Pi=TMatr.getAMatrA(psi.length,psi,M.B.getQQMatr(n,pimom),n)/ni;
    this.sigma2_Pi=p2imom==null?Double.NaN:(TMatr.getAMatrA(psi.length,psi,M.B.getQQMatr(n,p2imom),n)/ni-this.Pi*this.Pi);

    this.pertp=(this.Pi-this.Pt)*this.fact;
}

} // END OF class PsiAverPertr
