package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.freemoney.FreeMoneyForAll;
import com.polytechnik.freemoney.MatricesFromPI;
import com.polytechnik.freemoney.CommonBasisOperations;
/**
 * Since spike approach is back.
 * @see com.polytechnik.freemoney.FreeMoneyForAll
 * @version $Id: PsiMSelection.java,v 1.5 2022/09/19 22:44:27 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
*/
public class PsiMSelection {
    public final EVXData I;
    final double [] psiM;


    public final double I_M,Tv_M,T_M,V_M,pv_M,pt_M,VdivT_M,Tv_spurM,pv_spurM,pt_spurM,dpdt_M,dpdtPI_M;
    public final double
	dpdt_spurM,Idpdt_spurM,
	dpdtPI_spurM,IdpdtPI_spurM
	;

    public final double Tv_dpMin,pv_dpMin,dp_dpMin,Tv_dpMax,pv_dpMax,dp_dpMax;

    PsiMSelection(final int n,
		      final double p_offset,
		      final double p_last,

		      // input moment matrices
		      final double [] QQt,
		      final double [] QQi,
		      final double [] QQpi,
		      final double [] QQTi,
		      final double [] QQTpi,

		      final double [] QQdp,
		      final double [] QQpt,

		      final double [] QQT,
		      final double [] QQV,
		      final double [] QQTdp,
		      // basis operations
		      final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		      final java.util.function.Function<double [],double []> getPolynomialWIntegratedDt,
		      final java.util.function.Function<double [],double []> getEDPsi,

		      // for unit test
		      final double eps){

	this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	final double [] UNIT=TMatr.getUnitMatrix(n),
	    // Gram matrix inverse can be obtained without solving a linear system.
	    IQQt=EVXData.convertMatrFromBtoQ_Basis(n,UNIT,I.ev,UNIT);
	final double []
	    bQQi=TMatr.getDiagMatrix(I.getEValues()),
	    	    bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,I.ev),
	    bQQTi=EVXData.convertMatrFromQtoB_Basis(n,QQTi,I.ev),
	    bQQTpi=EVXData.convertMatrFromQtoB_Basis(n,QQTpi,I.ev),
	    bQQV=EVXData.convertMatrFromQtoB_Basis(n,QQV,I.ev),
	    bQQT=EVXData.convertMatrFromQtoB_Basis(n,QQT,I.ev),
	    // p from original sample
	    bQQpt=EVXData.convertMatrFromQtoB_Basis(n,QQpt,I.ev),
	    // dp/dt from original sample
	    bQQdp=EVXData.convertMatrFromQtoB_Basis(n,QQdp,I.ev)
	    ;

	final double []
	    bQQdpPI=MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(0,QQpi,I,getEDPsi,QQi,p_last),
	    bQQIdpPI=MatricesFromPI.getbQQIpowdpdtFromQQpi_withLastP(1,QQpi,I,getEDPsi,QQi,p_last);


	final double [] toMaximize=new double [n];
	for(int k=0;k<n;k++) {
	    final int diagk=k*n+k;
	    toMaximize[k]=bQQTi[diagk]/bQQi[diagk];
	}
	// Sort according to toMaximize.
	final int [] ind=com.polytechnik.utils.SortArrays.sortArrayIndexes(toMaximize);
	int ind_M=-1,ind_dpMin=-1,ind_dpMax=-1;
	for(int kc=0;kc<n;kc++){
	    final int k=ind[kc];
	    final int diagk=k*n+k;
	    if(bQQTi[diagk]<=bQQV[diagk]||bQQi[diagk]<=I.s0) continue;
	    if(ind_M<0) ind_M=k; // take the first one, then ignore
	    if(ind_dpMin<0||bQQdp[ind_dpMin*n+ind_dpMin]<=bQQdp[diagk]){
		ind_dpMin=k;
	    }
	    if(ind_dpMax<0||bQQdp[ind_dpMax*n+ind_dpMax]>=bQQdp[diagk]){
		ind_dpMax=k;
	    }
	    //break;
	}
	if(ind_M<0) ind_M=n-1;
	if(ind_dpMin<0) ind_dpMin=n-1;
	if(ind_dpMax<0) ind_dpMax=n-1;
	psiM=I.getPsi(ind_M);

	Tv_dpMin=bQQTi[ind_dpMin*n+ind_dpMin]/I.ev[ind_dpMin];
	pv_dpMin=p_offset+bQQpi[ind_dpMin*n+ind_dpMin]/I.ev[ind_dpMin];
	dp_dpMin=bQQdp[ind_dpMin*n+ind_dpMin];
	Tv_dpMax=bQQTi[ind_dpMax*n+ind_dpMax]/I.ev[ind_dpMax];
	pv_dpMax=p_offset+bQQpi[ind_dpMax*n+ind_dpMax]/I.ev[ind_dpMax];
	dp_dpMax=bQQdp[ind_dpMax*n+ind_dpMax];

	final double []
	    DpsiM=getEDPsi.apply(psiM),
	    bpsiM=EVXData.convertPsiFromQtoB_Basis(n,psiM,I.ev,QQt),
	    bDpsiM=EVXData.convertPsiFromQtoB_Basis(n,DpsiM,I.ev,QQt)
	    ;
	I_M=TMatr.getAMatrA(bpsiM,bQQi,n);
	Tv_M=TMatr.getAMatrA(bpsiM,bQQTi,n)/I_M;
	T_M=TMatr.getAMatrA(bpsiM,bQQT,n);
	V_M=TMatr.getAMatrA(bpsiM,bQQV,n);
	pt_M=p_offset+TMatr.getAMatrA(bpsiM,bQQpt,n);
	pv_M=p_offset+TMatr.getAMatrA(bpsiM,bQQpi,n)/I_M;
	dpdt_M=TMatr.getAMatrA(bpsiM,bQQdp,n);
	dpdtPI_M=TMatr.getAMatrA(bpsiM,bQQdpPI,n);
	VdivT_M=V_M/T_M;


	// A mixed state with |QQDensityMatrix| density matrix obtained as "an interval since |psiM>".
	final double []
	    // As the calculations require multiplication operator -- perfrom them in Q-basis,
	    // where multiplication is readily available.
	    //
	    // Integrate psiM^2 omega(x) dt: time since psiM spike
	    // dmu=pol_QQDensityMatrix(x(t))*omega(t)dt
	    pol_QQDensityMatrix=getPolynomialWIntegratedDt.apply(M.B.mult2Pol(psiM,psiM)),
	    // Construct the measure corresponding to pol_QQDensityMatrix polynomial
	    // using Theorem 3 of https://arxiv.org/abs/1807.06007 : find the measure generating given polynomial,
	    // Find measure moments, then create density matrix corresponding to this measure.
	    QQDensityMatrix=M.B.getQQMatr(n,M.B.getMomentsOfMeasureProducingPolynomialInKK_MQQM(n,pol_QQDensityMatrix,IQQt)),
	    // Convert it to b-basis to avoid extra IQQt in spur to simplify calculations.
	    bQQDensityMatrix=EVXData.convertMatrFromQtoB_Basis(n,QQDensityMatrix,I.ev)
	    ;

	Tv_spurM=TMatr.getSpurMM(n,bQQTi,bQQDensityMatrix)/V_M;
	pt_spurM=p_offset+TMatr.getSpurMM(n,bQQpt,bQQDensityMatrix)/T_M;
	pv_spurM=p_offset+TMatr.getSpurMM(n,bQQpi,bQQDensityMatrix)/V_M;
	dpdt_spurM=TMatr.getSpurMM(n,bQQdp,bQQDensityMatrix);
	Idpdt_spurM=TMatr.getSpurMMM(n,bQQdp,bQQi,bQQDensityMatrix);

	dpdtPI_spurM=TMatr.getSpurMM(n,bQQdpPI,bQQDensityMatrix);
	IdpdtPI_spurM=TMatr.getSpurMM(n,bQQIdpPI,bQQDensityMatrix);


	if(eps>0){
	    matchIntervalSincePsiM(this,new IntervalSincePsiM(
		      n,
		      p_offset,
		      p_last,
		      psiM,

		      // input moment matrices
		      QQt,
		      QQi,
		      QQpi,
		      QQTi,
		      QQTpi,

		      QQdp,
		      QQpt,

		      QQT,
		      QQV,
		      QQTdp,
		      // basis operations
		      M,
		      getPolynomialWIntegratedDt,
		      getEDPsi,

		      // for unit test
		      null,
		      null,
		      eps*0),eps);
	}

    }


    /** Find the {@literal |psi>} state among all states of {@code I}
     * that has the maximal value {@code dV/dt -V/T}.
     * The result is typically the state of maximal {@code dV/dt}.
     */
    static double [] getPsi_MaxImVdivT(final EVXData I,final double [] bQQV,final double [] bQQT, final double eps){
	final int n=I.neq;
	final double [] toMaximize=new double [n];
	for(int k=0;k<n;k++) {
	    final int diagk=k*n+k;
	    toMaximize[k]=I.ev[k]-bQQV[diagk]/bQQT[diagk];  // try this?
	    //toMaximize[k]=bQQV[diagk]/bQQT[diagk];
	}
	// Sort according to toMaximize.
	final int [] ind=com.polytechnik.utils.SortArrays.sortArrayIndexes(toMaximize);
	if(eps>0){
	    System.err.println("getPsi_MaxImVdivT: I.ev="+java.util.Arrays.toString(I.getEValues())+" toMaximize="+java.util.Arrays.toString(toMaximize)+" indexes="+java.util.Arrays.toString(ind)+" ind_selected="+ind[n-1]);
	}
	// return the state of maximal I-V/T
	return I.getPsi(ind[n-1]);
    }



static void matchIntervalSincePsiM(final PsiMSelection a,final IntervalSincePsiM b,final double eps)
{
    EVXData.matchEVXData(a.I,b.I,false,eps);
    // in this version output prices (public fields) are without an offset

    UnitTests.checkDoubleRel(a.I_M,"a.I_M",b.I_M,"b.I_M",eps);
    UnitTests.checkDoubleRel(a.pv_M,"a.pv_M",b.pv_M,"b.pv_M",eps);
    UnitTests.checkDoubleRel(a.pt_M,"a.pt_M",b.pt_M,"b.pt_M",eps);
    UnitTests.checkDoubleRel(a.Tv_M,"a.Tv_M",b.Tv_M,"b.Tv_M",eps);
    UnitTests.checkDoubleRel(a.T_M,"a.T_M",b.T_M,"b.T_M",eps);
    UnitTests.checkDoubleRel(a.V_M,"a.V_M",b.V_M,"b.V_M",eps);
    UnitTests.checkDoubleRel(a.VdivT_M,"a.VdivT_M",b.VdivT_M,"b.VdivT_M",eps);


    UnitTests.checkDoubleRel(a.Tv_spurM,"a.Tv_spurM",b.Tv_spurM,"b.Tv_spurM",eps);
    UnitTests.checkDoubleRel(a.pv_spurM,"a.pv_spurM",b.pv_spurM,"b.pv_spurM",eps);
    UnitTests.checkDoubleRel(a.pt_spurM,"a.pt_spurM",b.pt_spurM,"b.pt_spurM",eps);
    UnitTests.checkDoubleRel(a.dpdt_spurM,"a.dpdt_spurM",b.dpdt_spurM,"b.dpdt_spurM",eps);
    UnitTests.checkDoubleRel(a.Idpdt_spurM,"a.Idpdt_spurM",b.Idpdt_spurM,"b.Idpdt_spurM",eps);

    UnitTests.checkDoubleRel(a.dpdtPI_spurM,"a.dpdtPI_spurM",b.dpdtPI_spurM,"b.dpdtPI_spurM",eps);
    UnitTests.checkDoubleRel(a.IdpdtPI_spurM,"a.IdpdtPI_spurM",b.IdpdtPI_spurM,"b.IdpdtPI_spurM",eps);

    System.err.println("PsiMSelection test OK n="+a.I.neq+" eps="+eps);
}


}
