package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.LogPolynomialsRatio;
import com.polytechnik.realtimebook.*;
import com.polytechnik.outils.*;

public class QQVMLaguerre extends QQVM<QVMDataL> {

    static final Laguerre B=new Laguerre();

QQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static QQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final QQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new QQVMLaguerre(neq,o);
}

DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
TwoCloseRootsFinder.RootsData getTR(final double [] kv,final double [] kt)
{
    if(kv.length!=kt.length) throw new IllegalArgumentException("kv.length="+kv.length+"!=kt.length="+kt.length);
    return TwoCloseRootsFinder.findRootsData(getX0(),2.1,3000,new LogPolynomialsRatio(kv,kt,getABasis().B));
}


static class QQVMLaguerreTest extends QQVMTest<QVMDataL> {
QQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6873L),1e-5);
}
long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
QQVMLaguerre getQQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
ZQVMLaguerre getZQVM(){return ZQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
ZVQVMLaguerre getZVQVM(){return ZVQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
BQVMLaguerre getBQVM(){return BQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
boolean isDescending(){return false;}
QVMData getPData(final int nmoms)
{
    final QVMData qdata=QVMData.calculateQVMData(1,nmoms,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
    return qdata;
}
double [][] getMonomialMoments(final QVMDataL odata)
{
    final int nmoms=2*neq-1;
    final QVMData qd=getPData(nmoms);
    return new double [][]{
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomT(nmoms),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomRTS(nmoms,qd),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomRVS(nmoms,qd),
    };
}


} // END of class QQVMLaguerreTest

public static void main(String [] args)
{
    new QQVMLaguerreTest("QQVMLaguerre").runTests(5);
} // END main



} // END OF class QQVMLaguerre
