package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.LogPolynomialsRatio;
import com.polytechnik.realtimebook.*;
import com.polytechnik.outils.*;

public class QQVMLegendreShifted extends QQVM<QVMDataP> {

    static final LegendreShifted B=new LegendreShifted();

QQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static QQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final QQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new QQVMLegendreShifted(neq,o);
}

DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
double getX0(){return 1;}
double getPOffset(){return odata.p_offset;}
TwoCloseRootsFinder.RootsData getTR(final double [] kv,final double [] kt)
{
    if(kv.length!=kt.length) throw new IllegalArgumentException("kv.length="+kv.length+"!=kt.length="+kt.length);
    return TwoCloseRootsFinder.findRootsData(getX0(),0.5,3000,new LogPolynomialsRatio(kv,kt,getABasis().B));
}

static class QQVMLegendreShiftedTest extends QQVMTest<QVMDataP> {
QQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-5);
}
long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/2)/(9*5*7*8)*(9*5*7*8);
}
QQVMLegendreShifted getQQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
ZQVMLegendreShifted getZQVM(){return ZQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
ZVQVMLegendreShifted getZVQVM(){return ZVQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
BQVMLegendreShifted getBQVM(){return BQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return B.convertFunctionTo_Monomials_from_Basis(p);}
boolean isDescending(){return true;}
QVMData [] getPData(final int nmoms)
{
    final QVMData [] qdata=new QVMData [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}
double [][] getMonomialMoments(final QVMDataP odata)
{
    final int nmoms=2*neq-1;
    final QVMData [] qd=getPData(nmoms);
    return new double [][]{
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomT(nmoms),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomRTS(nmoms,qd),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomRVS(nmoms,qd),
    };
}


} // END of class QQVMLegendreShiftedTest

public static void main(String [] args)
{
    new QQVMLegendreShiftedTest("QQVMLegendreShifted").runTests(5);
} // END main



} // END OF class QQVMLegendreShifted
