package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/** Calculate <Qj|dI/dt|Qk> matrix with various boundary conditions.
 * @version $Id: QQdidtMatrix.java,v 1.13 2020/09/27 20:33:20 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class QQdidtMatrix {
    public final double [] QQdidt;
    public final double I_0_feq;
    public static final int BOUNDARY_IeqI0=671,BOUNDARY_IeqIH=672,BOUNDARY_DIeq0=673,BOUNDARY_Ieq0=674,BOUNDARY_NOMATRIXTRANSFORM=675;
    public QQdidtMatrix(final int boundary_condition_type,
			final EVXData I,
			final double [] QQi,
			final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
			final VolMatrix VOLATILITY_EDPsi)
    {
	final int n=I.neq;
	if(boundary_condition_type==BOUNDARY_NOMATRIXTRANSFORM){
	    I_0_feq=Double.NaN;
	    this.QQdidt=java.util.Arrays.copyOf(QQi,n*n);
	} else {
	    final double [] _QQdi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi);
	    if(boundary_condition_type==BOUNDARY_IeqI0){
		I_0_feq=I.s0;
	    } else if(boundary_condition_type==BOUNDARY_IeqIH){
		I_0_feq=I.sH;
	    } else if(boundary_condition_type==BOUNDARY_DIeq0){
		I_0_feq=TMatr.getAMatrA(I.psi0,_QQdi,n)/I.o_o;
	    } else if(boundary_condition_type==BOUNDARY_Ieq0){
		I_0_feq=0;
	    } else {
		throw new IllegalArgumentException("boundary_condition_type="+boundary_condition_type);
	    }
	    if(false){
		this.QQdidt=TMatr.sumaAbB(I_0_feq,TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),-1,_QQdi);
	    } else {
		// optimization
		final double [] q0=M.B.getX0Moments(n,I.x0);
		for(int j=0;j<n;j++) for(int k=0;k<n;k++) _QQdi[j*n+k]=I_0_feq*q0[j]*q0[k] - _QQdi[j*n+k];
		this.QQdidt=_QQdi;
	    }
	}
    }


    /** A constructor without EVXData object, the boundary_condition_type==BOUNDARY_IeqIH is not supported.
     */
/*
    QQdidtMatrix(final int boundary_condition_type,
		 final double [] QQi,
		 final VolMatrix VOLATILITY_EDPsi,
		 final double [] psi0,final double [] Qatx0)
    {
	final int n=Qatx0.length;
	if(psi0.length!=n||QQi.length!=n*n) throw new IllegalArgumentException();
	if(boundary_condition_type==BOUNDARY_NOMATRIXTRANSFORM){
	    I_0_feq=Double.NaN;
	    this.QQdidt=java.util.Arrays.copyOf(QQi,n*n);
	} else {
	    final double [] _QQdi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi);
	    if(boundary_condition_type==BOUNDARY_IeqI0){
		I_0_feq=TMatr.getAMatrA(psi0,QQi,n);
	    } else if(boundary_condition_type==BOUNDARY_IeqIH){
		throw new IllegalArgumentException("boundary_condition_type="+boundary_condition_type+" is not supported in this constructor.");
	    } else if(boundary_condition_type==BOUNDARY_DIeq0){
		I_0_feq=TMatr.getAMatrA(psi0,_QQdi,n)/Math.pow(TMatr.mulVV(psi0,Qatx0),2);
	    } else if(boundary_condition_type==BOUNDARY_Ieq0){
		I_0_feq=0;
	    } else {
		throw new IllegalArgumentException("boundary_condition_type="+boundary_condition_type);
	    }
	    for(int j=0;j<n;j++) for(int k=0;k<n;k++) _QQdi[j*n+k]=I_0_feq*Qatx0[j]*Qatx0[k] - _QQdi[j*n+k];
	    this.QQdidt=_QQdi;
	}
    }
*/
}
