package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.realtimebook.*;

/** Class for QVM testing.
 *  @version $Id: QTest.java,v 1.6 2022/06/27 13:32:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class QTest {


abstract long getTau();
abstract double [] convertToMonomials(final double [] coefs);

final String name;
final java.util.Random r;
final double eps;
QTest(final String name,final java.util.Random r,final double eps)
{
    this.name=name;
    this.r=r;
    this.eps=eps;
}

int neq;
long currenttime,tau;
Trade [] tb;
void setData(final int tsize,final int neq)
{
    this.neq=neq;
    this.tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
    this.currenttime=tb[tb.length-1].time+330;
    this.tau=getTau();
    //System.err.println("tau="+tau);
}

double [] getMonomialMomentsConvertedFromBasis(final double [] DTYPE_moments)
{
    final int n=DTYPE_moments.length;
    final double [] m=new double [n*n];
    for(int k=0;k<n;k++)
    {
	final double [] mo=new double [k+1];
	mo[k]=1;
	System.arraycopy(convertToMonomials(mo),0,m,k*n,k+1);
    }
    final double [] mom=new double [n];
    System.arraycopy(DTYPE_moments,0,mom,0,n);
    final int [] ipvt=new int [n];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(m,n,n,ipvt);
    if(pinfo!=0) throw new RuntimeException("pinfo="+pinfo);
    com.polytechnik.utils.Linsystems.dgesl_m1(m,n,n,ipvt,mom,1);
    return mom;
}
double [] getBasisMomentsConvertedFromMonomials(final double [] monomial_moments)
{
    final int n=monomial_moments.length;
    final double [] basis_moments=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] mo=new double [k+1];
	mo[k]=1;
	basis_moments[k]=TMatr.mulVVn1(convertToMonomials(mo),monomial_moments);
    }
    return basis_moments;
}

static long findInitRandom_NoFail(final long start,final long end,
				  final java.util.function.Consumer<Long> trier)
{
for(long initrandom=start;initrandom<=end;initrandom++)
{
    System.err.println("QTest.findInitRandom_NoFail:try initrandom="+initrandom);
    try {trier.accept(initrandom);}
    catch(final RuntimeException e) {
	e.printStackTrace(System.err);
	System.err.println("QTest.findInitRandom_NoFail:initrandom="+initrandom+" FAILED");
	continue;
    }
    System.err.println("QTest.findInitRandom_NoFail:initrandom="+initrandom+" FOUND OK");
    return initrandom;
}
System.err.println("QTest.findInitRandom_NoFail:Working initrandom ; start="+start+" end="+end+" was not found");
return 0;
}

} // END OF class QTest
