package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;

/** Simplified QVMLL & QVMPP version.
 *  @version $Id: QVMD.java,v 1.16 2019/01/06 23:33:47 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class QVMD {

private final DeprecatedHelperBasisInterface basis;
final double p_eq,sigma2_eq,osp,pi_eq,sigma2pi_eq,di,dip,io,scale_io,scale_io0,scale_dip0,dpi_eq;

QVMD(final DeprecatedHelperBasisInterface basis)
{
    this.basis=basis;
    final int neq_t=basis.get_ntm();
    final double [] dp=new double [neq_t],i=new double [neq_t],o=new double [neq_t];
    basis.setI(i);
    basis.setDPn(o,0);
    basis.setDPn(dp,1);

    final double p_offset=basis.getPOffset(),o_o=basis.get_o_o(),
	dp_o=basis.get_f_o(dp),
	i_i=basis.get_f_g(i,i),i_o=basis.get_f_o(i);
    final double fdp_o=dp_o/o_o,fi_o=i_o/o_o;
    this.p_eq=p_offset+fdp_o;
    TMatr.sumP1(i,o,-fi_o);
    TMatr.sumP1(dp,o,-fdp_o);
    final double x_dp_dp=basis.get_f_g(dp,dp),x_i_dp=basis.get_f_g(i,dp),x_i_i=basis.get_f_g(i,i);
    this.sigma2_eq=x_dp_dp/o_o;
    this.osp=x_i_dp/Math.sqrt(x_i_i*x_dp_dp);
    final double dpi=(x_i_dp/x_i_i)*fi_o;
    this.pi_eq=p_eq-dpi;
    this.sigma2pi_eq=this.sigma2_eq-(x_i_dp*x_i_dp/x_i_i)/o_o;
    final double dplast=basis.getPLast()-p_offset;
    final double fi_q1=x_i_dp/x_dp_dp,x_sigma2_ii=x_i_i-x_i_dp*x_i_dp/x_dp_dp;
    this.di=(-fi_q1*(dplast-fdp_o)-fi_o)*o_o;
    this.dip=-fi_q1;
    this.io=i_o;
    this.scale_io=Math.sqrt(o_o*x_sigma2_ii);
    this.scale_io0=Math.sqrt(o_o*x_i_i);
    this.scale_dip0=Math.sqrt(x_i_i/x_dp_dp);
    this.dpi_eq=(osp*osp)*(dplast-fdp_o)+dpi;
}



static abstract class QVMDTest {
final String name;
final double eps,epssum;
QVMDTest(final String name,final double eps,final double epssum)
{
    this.name=name;
    this.eps=eps;
    this.epssum=epssum;
}

abstract QVMD getQVMD(final int tsize,final int ntm,final int npm,final java.util.Random r);

private void checkSPIMin(final int tsize,final java.util.Random r,final int neq_t)
{
    System.err.println(name+":QVMDTest:checkSPIMin(): neq_t="+neq_t);
    final QVMD q=getQVMD(tsize,neq_t,2,r);

    final double plast=q.basis.getPLast(),p_offset=q.basis.getPOffset(),dplast=plast-p_offset;
    if(true)
    {
	final SPIMin sm=new SPIMin(neq_t,1,q.basis);
	final double [] ps=sm.getP0I();
	UnitTests.checkDoubleRel(q.p_eq,"q.p_eq",ps[0],"ps[0]",eps);
	UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",ps[1],"ps[1]",eps);
    }
    if(true)
    {
	final SPIMin sm=new SPIMin(neq_t,2,q.basis);
	final double [] ps=sm.getP0I();
	UnitTests.checkDoubleRel(q.pi_eq,"q.p_eq",ps[0],"ps[0]",eps);
	UnitTests.checkDoubleRel(q.sigma2pi_eq,"q.sigma2_eq",ps[1],"ps[1]",eps);
    }
    if(true)
    {
	final SPIMin sm=new SPIMin(neq_t,2,q.basis);
	final SPIMin.SPIMinData xx=sm.getI0P();
	final double o_o=q.basis.get_o_o();
	UnitTests.checkDoubleRel(q.di,"q.di",Polynomials.getD0(xx.inter,dplast),"xx.inter(p)",eps);
	UnitTests.checkDoubleRel(q.dip,"q.dip",Polynomials.getD1(xx.inter,dplast)/o_o,"d xx.inter(p)/dp/o_o",eps);
	UnitTests.checkDoubleRel(-q.osp,"-q.osp",q.dip*(o_o*Math.sqrt(q.sigma2_eq)/q.scale_io0),"q.dip*scale",eps);
	UnitTests.checkDoubleRel(q.scale_io,"q.scale_io",xx.scale_io,"xx.scale_io",eps);
	UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",xx.scale_io0,"xx.scale_io0",eps);
	UnitTests.checkDoubleRel(q.p_eq,"q.p_eq",xx.p_eq,"xx.p_eq",eps);
	UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",xx.sigma2_eq,"xx.sigma2_eq",eps);
	final double dpi_eq=Polynomials.getD0(xx.inter,dplast)*(q.osp*(-1)*Math.sqrt(q.sigma2_eq)/xx.scale_io0);
	UnitTests.checkDoubleRel(q.dpi_eq,"q.dpi_eq",dpi_eq,"dpi_eq",eps);
    }
    System.err.println(name+":QVMDTest:checkSPIMin(): TEST OK neq_t="+neq_t);
}
void checkExternal(final int tsize,final java.util.Random r,final int neq_t){}
void checkCalculations(final int tsize,final java.util.Random r,final int neq_t,final int neq_p)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":QVMDTest:checkCalculations(): test with N points="+tsize+
		       " neq_t="+neq_t+" neq_p="+neq_p+" eps="+eps+" epssum="+epssum);
    checkSPIMin(tsize,r,neq_t);
    checkExternal(tsize,r,neq_t);
    System.err.println(name+":QVMDTest:checkCalculations(): test OK with N points="+tsize+
		       " neq_t="+neq_t+" neq_p="+neq_p+" eps="+eps+" epssum="+epssum);
}
void runTests()
{
    System.err.println(name+":QVMDTest: starting tests; ver=$Id: QVMD.java,v 1.16 2019/01/06 23:33:47 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79091);
    for(int neq_t=1;neq_t<=9;neq_t++)
    {
	for(int neq_p=2;neq_p<=Math.min(neq_t-4,5);neq_p++)
	{
	    for(int i=0;i<118;i++)
	    {
		checkCalculations(210+r.nextInt(210),r,neq_t,neq_p);
	    }
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":QVMDTest: tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: QVMD.java,v 1.16 2019/01/06 23:33:47 mal Exp $");
}
} // END OF class QVMDTest

} // END OF class QVMD
