package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.utils.UnitTests;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: QVMDLaguerre.java,v 1.6 2019/01/06 23:50:31 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class QVMDLaguerre extends QVMD {

private final DeprecatedHelperLaguerreBasis odata;
QVMDLaguerre(final DeprecatedHelperLaguerreBasis basis)
{
    super(basis);
    this.odata=basis;
}

public static QVMDLaguerre calculate(final int neq_t,
				     final long currenttime,
				     final long tau,
				     final Trade [] tb,
				     final int tsize,
				     final QVMDLaguerre ref)
{
    final DeprecatedHelperLaguerreBasis o=
	DeprecatedHelperLaguerreBasis.calculateBasis(neq_t,
				     3,
				     currenttime,
				     tau,
				     tb,
				     tsize,
				     ref!=null?ref.odata:null);
    return new QVMDLaguerre(o);
}
public static QVMDLaguerre calculateBS(final int neq_t,
				       final long currenttime,
				       final long tau,
				       final Trade [] tb,
				       final int tsize,
				       final QVMDLaguerre ref)
{
    final DeprecatedHelperLaguerreBasis o=
	DeprecatedHelperLaguerreBasis.calculateBasis(neq_t,
				     3,
				     currenttime,
				     tau,
				     tb,
				     tsize,
				     ref!=null?ref.odata:null);
    return new QVMDLaguerre(getBS(o));
}
private static DeprecatedHelperLaguerreBasis getBS(final DeprecatedHelperLaguerreBasis o)
{
    return new DeprecatedHelperLaguerreBasis(o.odata){
	public void setI(final double [] i){super.setIBS(i);}
	public void setIPn(final double [] i,final int n){if(n>0) throw new RuntimeException();this.setI(i);}};
}

public static void main(String [] args)
{
  for(int t=0;t<2;t++)
  {
      final int f=t;
      new QVMDTest("QVMDLaguerre:"+(f==0?"V":"BS"),1e-6,1e-3){
	QVMDLaguerre getQVMD(final int tsize,final int ntm,final int npm,final java.util.Random r)
	{
	    final Trade [] tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
	    final long currenttime=tb[tb.length-1].time+1000;
	    final long tau=(tb[tb.length-1].time-tb[0].time)/24;
	    if(f<=0) return QVMDLaguerre.calculate(ntm,currenttime,tau,tb,tsize,null);
	    else return QVMDLaguerre.calculateBS(ntm,currenttime,tau,tb,tsize,null);
	}
      }.runTests();
  }
}

}


