package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;

/** Moments data in monomials basis, use {@link QVMDataDirectAccess} to extract.
 *  @see QVMDataDirectAccess
 *  @see com.polytechnik.freemoney.CommonlyUsedMomentsMonomials
 *  @version $Id: QVMData.java,v 1.185 2022/09/14 14:46:35 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QVMData {

    final long currenttime,tau,vol,svol;
    final double p_offset;

    final double pt_sum,t_sum,v_sum,v_scale;
    final int nvm,ntm,npm;
    final double [] // here offset=<p>_t and array dims are: nvm,ntm,npm.
	tmoments_vol_x_tau_x_p,   // [k][l][m]=sum (vol-v_cur)^k (t-t_cur)^l (p-offset)^m dt_i exp()
	vmoments_vol_x_tau_x_p,   // [k][l][m]=sum (vol-v_cur)^k (t-t_cur)^l (p-offset)^m dv_i exp()
	Tvmoments_vol_x_tau_x_p,   // [k][l][m]=sum ((t-t_cur)/tau)*(vol-v_cur)^k (t-t_cur)^l (p-offset)^m dv_i exp()
	VdPmoments_vol_x_tau_x_p,
	Dp_tmoments_vol_x_tau_x_p,
	Dp_vmoments_vol_x_tau_x_p,
	Da_tmoments_vol_x_tau_x_p,
	Da_vmoments_vol_x_tau_x_p,
	vbuymoments_vol_x_tau_x_p,
	vsellmoments_vol_x_tau_x_p,
	pmoments_vol_x_tau_x_p,
	Tpmoments_vol_x_tau_x_p,
	pluspmoments_vol_x_tau_x_p,
	minuspmoments_vol_x_tau_x_p,
	amoments_vol_x_tau_x_p,
	admoments_vol_x_tau_x_p,
	tsmoments_vol_x_tau_x_p,
	vsmoments_vol_x_tau_x_p,
	rtsmoments_vol_x_tau_x_p,
	rvsmoments_vol_x_tau_x_p,
	tauvbuymoments_vol_x_tau_x_p,
	tauvsellmoments_vol_x_tau_x_p
	;

    // references to previos calculations
    final Trade trade_last;
    final Trade dp_diff_last;

    QVMData(final long currenttime,
	    final long tau,
	    final long vol,
	    final long svol,
	    final double p_offset,

	    final double pt_sum,
	    final double t_sum,
	    final double v_sum,
	    final double v_scale,

	    final double [] tmoments_vol_x_tau_x_p,
	    final double [] vmoments_vol_x_tau_x_p,
	    final double [] Tvmoments_vol_x_tau_x_p,
	    final double [] VdPmoments_vol_x_tau_x_p,
	    final double [] Dp_tmoments_vol_x_tau_x_p,
	    final double [] Dp_vmoments_vol_x_tau_x_p,
	    final double [] Da_tmoments_vol_x_tau_x_p,
	    final double [] Da_vmoments_vol_x_tau_x_p,

	    final double [] vbuymoments_vol_x_tau_x_p,
	    final double [] vsellmoments_vol_x_tau_x_p,
	    final double [] pmoments_vol_x_tau_x_p,
	    final double [] Tpmoments_vol_x_tau_x_p,
	    final double [] pluspmoments_vol_x_tau_x_p,
	    final double [] minuspmoments_vol_x_tau_x_p,
	    final double [] amoments_vol_x_tau_x_p,
	    final double [] admoments_vol_x_tau_x_p,
	    final double [] tsmoments_vol_x_tau_x_p,
	    final double [] vsmoments_vol_x_tau_x_p,
	    final double [] rtsmoments_vol_x_tau_x_p,
	    final double [] rvsmoments_vol_x_tau_x_p,
	    final double [] tauvbuymoments_vol_x_tau_x_p,
	    final double [] tauvsellmoments_vol_x_tau_x_p,

	    final int nvm,
	    final int ntm,
	    final int npm,

	    // references
	    final Trade trade_last,
	    final Trade dp_diff_last)
    {
	this.currenttime=currenttime;
	this.tau=tau;
	this.vol=vol;
	this.svol=svol;
	this.p_offset=p_offset;

	this.pt_sum=pt_sum;
	this.t_sum=t_sum;
	this.v_sum=v_sum;
	this.v_scale=v_scale;

	this.tmoments_vol_x_tau_x_p=tmoments_vol_x_tau_x_p;
	this.vmoments_vol_x_tau_x_p=vmoments_vol_x_tau_x_p;
	this.Tvmoments_vol_x_tau_x_p=Tvmoments_vol_x_tau_x_p;
	this.VdPmoments_vol_x_tau_x_p=VdPmoments_vol_x_tau_x_p;
	this.Dp_tmoments_vol_x_tau_x_p=Dp_tmoments_vol_x_tau_x_p;
	this.Dp_vmoments_vol_x_tau_x_p=Dp_vmoments_vol_x_tau_x_p;
	this.Da_tmoments_vol_x_tau_x_p=Da_tmoments_vol_x_tau_x_p;
	this.Da_vmoments_vol_x_tau_x_p=Da_vmoments_vol_x_tau_x_p;
	this.vbuymoments_vol_x_tau_x_p=vbuymoments_vol_x_tau_x_p;
	this.vsellmoments_vol_x_tau_x_p=vsellmoments_vol_x_tau_x_p;
	this.pmoments_vol_x_tau_x_p=pmoments_vol_x_tau_x_p;
	this.Tpmoments_vol_x_tau_x_p=Tpmoments_vol_x_tau_x_p;
	this.pluspmoments_vol_x_tau_x_p=pluspmoments_vol_x_tau_x_p;
	this.minuspmoments_vol_x_tau_x_p=minuspmoments_vol_x_tau_x_p;
	this.amoments_vol_x_tau_x_p=amoments_vol_x_tau_x_p;
	this.admoments_vol_x_tau_x_p=admoments_vol_x_tau_x_p;
	this.tsmoments_vol_x_tau_x_p=tsmoments_vol_x_tau_x_p;
	this.vsmoments_vol_x_tau_x_p=vsmoments_vol_x_tau_x_p;
	this.rtsmoments_vol_x_tau_x_p=rtsmoments_vol_x_tau_x_p;
	this.rvsmoments_vol_x_tau_x_p=rvsmoments_vol_x_tau_x_p;
	this.tauvbuymoments_vol_x_tau_x_p=tauvbuymoments_vol_x_tau_x_p;
	this.tauvsellmoments_vol_x_tau_x_p=tauvsellmoments_vol_x_tau_x_p;

	this.nvm=nvm;
	this.ntm=ntm;
	this.npm=npm;

	this.trade_last=trade_last;
	this.dp_diff_last=dp_diff_last;
    }

    private static void checkMatching(final QVMData r,final QVMData ref,final double eps,final double epssum)
    {
	if(r.currenttime!=ref.currenttime)
	{
	    throw new RuntimeException("r.currenttime="+r.currenttime+
				       " ref.currenttime="+ref.currenttime);
	}
	if(r.tau!=ref.tau)
	{
	    throw new RuntimeException("r.tau="+r.tau+" ref.tau="+ref.tau);
	}
	if(r.vol!=ref.vol)
	{
	    throw new RuntimeException("r.vol="+r.vol+" ref.vol="+ref.vol);
	}
	if(r.svol!=ref.svol)
	{
	    throw new RuntimeException("r.svol="+r.svol+" ref.svol="+ref.svol);
	}

	UnitTests.checkDoubleRel(r.pt_sum,"r.pt_sum",ref.pt_sum,"ref.pt_sum",eps);
	if(r.npm>1)
	UnitTests.checkDoubleRel(r.pt_sum,"r.pt_sum",r.tmoments_vol_x_tau_x_p[1]+r.p_offset*r.tmoments_vol_x_tau_x_p[0],"r.tmoments_vol_x_tau_x_p[1]+r.p_offset*r.tmoments_vol_x_tau_x_p[0]",epssum);
	UnitTests.checkDoubleRel(r.t_sum,"r.t_sum",ref.t_sum,"ref.t_sum",eps);
	UnitTests.checkDoubleRel(r.t_sum,"r.t_sum",r.tmoments_vol_x_tau_x_p[0],"r.tmoments_vol_x_tau_x_p[0]",eps);
	UnitTests.checkDoubleRel(r.v_sum,"r.v_sum",ref.v_sum,"ref.v_sum",eps);
	UnitTests.checkDoubleRel(r.v_sum,"r.v_sum",r.vmoments_vol_x_tau_x_p[0],"r.vmoments_vol_x_tau_x_p[0]",eps);
	UnitTests.checkDoubleRel(r.v_sum,"r.v_sum",r.v_scale,"r.v_scale",eps);

	UnitTests.checkDoubleRel(r.p_offset,"r.p_offset",ref.p_offset,"ref.p_offset",eps);
	UnitTests.checkDoubleRel(r.v_scale,"r.v_scale",ref.v_scale,"ref.v_scale",eps);

	UnitTests.checkDoubleArrayRel1(r.tmoments_vol_x_tau_x_p,"r.tmoments_vol_x_tau_x_p",
				       ref.tmoments_vol_x_tau_x_p,"ref.tmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.vmoments_vol_x_tau_x_p,"r.vmoments_vol_x_tau_x_p",
				      ref.vmoments_vol_x_tau_x_p,"ref.vmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.Tvmoments_vol_x_tau_x_p,"r.Tvmoments_vol_x_tau_x_p",
				      ref.Tvmoments_vol_x_tau_x_p,"ref.Tvmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.VdPmoments_vol_x_tau_x_p,"r.VdPmoments_vol_x_tau_x_p",
				      ref.VdPmoments_vol_x_tau_x_p,"ref.VdPmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel1(r.Dp_tmoments_vol_x_tau_x_p,"r.Dp_tmoments_vol_x_tau_x_p",
				       ref.Dp_tmoments_vol_x_tau_x_p,"ref.Dp_tmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.Dp_vmoments_vol_x_tau_x_p,"r.Dp_vmoments_vol_x_tau_x_p",
				      ref.Dp_vmoments_vol_x_tau_x_p,"ref.Dp_vmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel1(r.Da_tmoments_vol_x_tau_x_p,"r.Da_tmoments_vol_x_tau_x_p",
				       ref.Da_tmoments_vol_x_tau_x_p,"ref.Da_tmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.Da_vmoments_vol_x_tau_x_p,"r.Da_vmoments_vol_x_tau_x_p",
				      ref.Da_vmoments_vol_x_tau_x_p,"ref.Da_vmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.vbuymoments_vol_x_tau_x_p,"r.vbuymoments_vol_x_tau_x_p",
				      ref.vbuymoments_vol_x_tau_x_p,"ref.vbuymoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.vsellmoments_vol_x_tau_x_p,"r.vsellmoments_vol_x_tau_x_p",
				      ref.vsellmoments_vol_x_tau_x_p,"ref.vsellmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.pmoments_vol_x_tau_x_p,"r.pmoments_vol_x_tau_x_p",
				      ref.pmoments_vol_x_tau_x_p,"ref.pmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.Tpmoments_vol_x_tau_x_p,"r.Tpmoments_vol_x_tau_x_p",
				      ref.Tpmoments_vol_x_tau_x_p,"ref.Tpmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.pluspmoments_vol_x_tau_x_p,"r.pluspmoments_vol_x_tau_x_p",
				      ref.pluspmoments_vol_x_tau_x_p,"ref.pluspmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.minuspmoments_vol_x_tau_x_p,"r.minuspmoments_vol_x_tau_x_p",
				      ref.minuspmoments_vol_x_tau_x_p,"ref.minuspmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.amoments_vol_x_tau_x_p,"r.amoments_vol_x_tau_x_p",
				      ref.amoments_vol_x_tau_x_p,"ref.amoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.admoments_vol_x_tau_x_p,"r.admoments_vol_x_tau_x_p",
				      ref.admoments_vol_x_tau_x_p,"ref.admoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.tsmoments_vol_x_tau_x_p,"r.tsmoments_vol_x_tau_x_p",
				      ref.tsmoments_vol_x_tau_x_p,"ref.tsmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.vsmoments_vol_x_tau_x_p,"r.vsmoments_vol_x_tau_x_p",
				      ref.vsmoments_vol_x_tau_x_p,"ref.vsmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.rtsmoments_vol_x_tau_x_p,"r.rtsmoments_vol_x_tau_x_p",
				      ref.rtsmoments_vol_x_tau_x_p,"ref.rtsmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.rvsmoments_vol_x_tau_x_p,"r.rvsmoments_vol_x_tau_x_p",
				      ref.rvsmoments_vol_x_tau_x_p,"ref.rvsmoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.tauvbuymoments_vol_x_tau_x_p,"r.tauvbuymoments_vol_x_tau_x_p",
				      ref.tauvbuymoments_vol_x_tau_x_p,"ref.tauvbuymoments_vol_x_tau_x_p",eps);
	UnitTests.checkDoubleArrayRel(r.tauvsellmoments_vol_x_tau_x_p,"r.tauvsellmoments_vol_x_tau_x_p",
				      ref.tauvsellmoments_vol_x_tau_x_p,"ref.tauvsellmoments_vol_x_tau_x_p",eps);
	final double [] psum=TMatr.sumAcB(r.pluspmoments_vol_x_tau_x_p,1,r.minuspmoments_vol_x_tau_x_p);
	UnitTests.checkDoubleArrayRel1(r.pmoments_vol_x_tau_x_p,"r.pmoments_vol_x_tau_x_p",psum,"psum",/*epssum*/1e-3);
	final double [] vsum=TMatr.sumAcB(r.vbuymoments_vol_x_tau_x_p,1,r.vsellmoments_vol_x_tau_x_p);
	UnitTests.checkDoubleArrayRel(r.vmoments_vol_x_tau_x_p,"r.vmoments_vol_x_tau_x_p",vsum,"vsum",epssum);

	// Match r.VdPmoments_vol_x_tau_x_p used in various data source
	// with QVMData - specific volume-powered.
	for(int l=1;l<r.nvm;l++){
	    final double [][] Voldp=get3ArrayFromArrayForTesting(r.pmoments_vol_x_tau_x_p,r.nvm,r.ntm,r.npm)[l],
		VdP=get3ArrayFromArrayForTesting(r.VdPmoments_vol_x_tau_x_p,r.nvm,r.ntm,r.npm)[l-1];
	    for(int k=0;k<r.ntm;k++){
		UnitTests.checkDoubleArrayRel1(VdP[k],"VdP",TMatr.getVectorScaled(Voldp[k],r.v_scale),"Voldp[k]*r.v_scale",epssum);
	    }
	}

	System.err.println("QVMData:checkMatching(): matching OK nvm="+r.nvm+" ntm="+r.ntm+" npm="+r.npm+" eps="+eps+" epssum="+epssum);
    }

static class ScanHelper {
    final Trade b,be;final int ind_b,ind_be;
    final boolean flag_edge_reached;
    ScanHelper(final Trade b,final int ind_b,final Trade be,final int ind_be,final boolean flag_edge_reached){this.b=b;this.ind_b=ind_b;this.be=be;this.ind_be=ind_be;this.flag_edge_reached=flag_edge_reached;}

/** Scan back to find the trade of different price.
 * Keep for unit tests, the concept of scanning price-different ticks
 * is not working and is conceptually wrong.
 */
static ScanHelper findTradeScanBackDifferentPrice(final int istart,final Trade[] tb,final Trade bref,final Trade dp_diff_last_ref)
{
    Trade b=tb[istart],be=null;
    // scan back to find the first trade giving dp!=0
    int ind_be=istart;
    boolean flag_edge_reached=false;
    for(;--ind_be>=-1;)
    {
	Trade bx=null;
	if(ind_be<0||(bx=tb[ind_be])==bref){
	    flag_edge_reached=true;
	    if(bref!=null && isPriceNotTheSame(bref.price,b.price)){be=bref;break;}
	    if(dp_diff_last_ref!=null) {be=dp_diff_last_ref;break;}
	    be=bx;break;
	}
	be=bx;
	if(isPriceNotTheSame(bx.price,b.price)){be=bx;break;}
    }
    if(be==null) {be=b;}
    return new ScanHelper(b,istart,be,ind_be,flag_edge_reached);
}
static boolean isPriceNotTheSame(final double p1,final double p2)
{
    return !(p1==p2) ; // p1!=p2; // think of NaN
}
}

public static QVMData calculateQVMData(final int nvm,
				       final int ntm,
				       final int npm,
				       final long currenttime,
				       final long tau,
				       final Trade [] tb,
				       final int tsize,
				       final QVMData ref)
{
    final double atau=tau;
    final Trade bref,dp_diff_last_ref;
    if(ref!=null)
    {
	bref=ref.trade_last;
	dp_diff_last_ref=ref.dp_diff_last;
	if(ref.tau!=tau||ref.currenttime>currenttime||
	   ref.nvm!=nvm||ref.ntm!=ntm||ref.npm!=npm)
	{
	    throw new RuntimeException("Invalid ref; ref.tau="+ref.tau+" tau="+tau+
				       " ref.nvm="+ref.nvm+" nvm="+nvm+
				       " ref.ntm="+ref.ntm+" ntm="+ntm+
				       " ref.npm="+ref.npm+" npm="+npm+
				       " ref.currenttime="+ref.currenttime+" currenttime="+currenttime);

	}
    }
    else
    {
	bref=dp_diff_last_ref=null;
    }
    double pt_sum=0,t_sum=0,v_sum=0;
    long prevt=currenttime,tinterval=currenttime,vol=0,sumdt=0;
    for(int i=tsize;--i>=0;)
    {
	final Trade b=tb[i];
	if(b.time>prevt)
	{
	    throw new RuntimeException("The data is not sorted:"+prevt+" and "+b.time);
	}
	if(b==bref)
	{ // optimization
	    break;
	}
	prevt=b.time;
	final long dt=tinterval-prevt;
	vol+=b.shares;
	sumdt+=dt;
	final double ex=Math.exp((prevt-currenttime)/atau),twmw=ex*(dt/atau);
	pt_sum+=b.price*twmw;
	t_sum+=twmw;
	v_sum+=ex*b.shares;

	tinterval=prevt;
    }
    final double ref_delay,exp_ref_delay,bref_delay,expdt_bref_delay;
    if(bref!=null)
    { // optimization
	// add missed half slice
	sumdt+=tinterval-ref.currenttime;
	if(sumdt!=(currenttime-ref.currenttime))
	{
	    throw new RuntimeException("sumdt="+sumdt+" currenttime-ref.currenttime="+(currenttime-ref.currenttime));
	}
	bref_delay=(bref.time-currenttime)/atau;
	expdt_bref_delay=Math.exp(bref_delay)*((tinterval-ref.currenttime)/atau);
	pt_sum+=expdt_bref_delay*bref.price;
	t_sum+=expdt_bref_delay;

	// previous interval
	ref_delay=(ref.currenttime-currenttime)/atau;
	exp_ref_delay=Math.exp(ref_delay);
	pt_sum+=exp_ref_delay*ref.pt_sum;
	t_sum+=exp_ref_delay*ref.t_sum;
	v_sum+=exp_ref_delay*ref.v_sum;
	vol+=ref.vol;
    }
    else
    {
	ref_delay=exp_ref_delay=bref_delay=expdt_bref_delay=0;
    }
    final double p_offset,v_scale=v_sum>0?v_sum:1;
    final boolean offset_eq_average;
    if(t_sum>0)
    {
	p_offset=pt_sum/t_sum;
	offset_eq_average=false;//true;
    }
    else
    {
	p_offset=ref!=null?ref.p_offset:(tsize>0?tb[tsize-1].price:0);
	offset_eq_average=false;
    }


    final double []  // all 0 by default
	tmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	vmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	Tvmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	VdPmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	Dp_tmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	Dp_vmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	Da_tmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	Da_vmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	vbuymoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	vsellmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	pmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	Tpmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	pluspmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	minuspmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	amoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	admoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	tsmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	vsmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	rtsmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	rvsmoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	tauvbuymoments_vol_x_tau_x_p=new double [nvm*ntm*npm],
	tauvsellmoments_vol_x_tau_x_p=new double [nvm*ntm*npm];

    long vx=0,svol=0;
    prevt=tinterval=currenttime;
    Trade dp_diff_last_next=dp_diff_last_ref,bref_next=bref;
    QVMData.ScanHelper beh=new QVMData.ScanHelper(null,tsize,null,tsize,true);
    for(int i=tsize;--i>=0;)
    {
	final Trade b=tb[i];
	if(b==bref)
	{ // optimization
	    break;
	}
	prevt=b.time;
	vx-=b.shares;
	final double etau=(prevt-currenttime)/atau,ex=Math.exp(etau),evol=vx/v_scale,
	    dp=b.price-p_offset,twmw=ex*((tinterval-prevt)/atau),bwmw=ex*b.shares,dpprev;
	final Trade bprev;
	if(i>0) {bprev=tb[i-1];}
	else if(bref!=null) {bprev=bref;}
	else {bprev=b;}
	dpprev=bprev.price-p_offset;

	if(i<=beh.ind_be) beh=QVMData.ScanHelper.findTradeScanBackDifferentPrice(i,tb,bref,dp_diff_last_ref);
	final double dp_nonzero=b.price-beh.be.price,da_nonzero=Math.abs(dp_nonzero);
	if(i==(tsize-1)){bref_next=b;dp_diff_last_next=beh.be;}

	final int fplus=(b.price>bprev.price?1:(b.price<bprev.price?-1:0));
	final double awmw=ex*Math.abs(b.price-bprev.price),adwmw=ex*(ScanHelper.isPriceNotTheSame(b.price,bprev.price)?1:0),
	    tauorig=b.origtime>0 && b.time>b.origtime ?(b.time-b.origtime)/atau:0;

	if(b.order!=null)
	{
	    if(b.buy_sell_indicator!=b.order.buy_sell_indicator) throw new RuntimeException("b="+b.buy_sell_indicator+" b.order="+b.order.buy_sell_indicator);
	    if(b.order.order!=null && (b.order.buy_sell_indicator==b.order.order.buy_sell_indicator))
		throw new RuntimeException("b.order="+b.order.buy_sell_indicator+" b.order.order.buy_sell_indicator|="+b.order.order.buy_sell_indicator);
	}

	final double Sv;
	if(b.buy_sell_indicator==BookItem.BOOK_BUY){
	    Sv=b.best_sell!=null?Math.abs(b.best_sell.price-b.price):0;
	    svol-=b.shares;
	}
	else if(b.buy_sell_indicator==BookItem.BOOK_SELL){
	    Sv=b.best_buy!=null?Math.abs(b.price-b.best_buy.price):0;
	    svol+=b.shares;
	} else Sv=0;

	double wve=1;
	for(int k=0,j=0;k<nvm;k++,wve*=evol)
	{
	    double wte=wve;
	    for(int l=0;l<ntm;l++,wte*=etau)
	    {
		double wpe=wte,wpreve=wpe;
		for(int m=0;m<npm;m++,j++,wpe*=dp,wpreve*=dpprev)
		{
		    tmoments_vol_x_tau_x_p[j]+=wpe*twmw;
		    vmoments_vol_x_tau_x_p[j]+=wpe*bwmw;
		    Tvmoments_vol_x_tau_x_p[j]+=etau*wpe*bwmw;
		    VdPmoments_vol_x_tau_x_p[j]+=(vx*(wpe-wpreve))*ex;
		    Dp_tmoments_vol_x_tau_x_p[j]+=dp_nonzero*wpe*twmw;
		    Dp_vmoments_vol_x_tau_x_p[j]+=dp_nonzero*wpe*bwmw;
		    Da_tmoments_vol_x_tau_x_p[j]+=da_nonzero*wpe*twmw;
		    Da_vmoments_vol_x_tau_x_p[j]+=da_nonzero*wpe*bwmw;
		    final double wdp=(wpe-wpreve)*ex;
		    pmoments_vol_x_tau_x_p[j]+=wdp;
		    Tpmoments_vol_x_tau_x_p[j]+=etau*wdp;
		    amoments_vol_x_tau_x_p[j]+=wpe*awmw;
		    admoments_vol_x_tau_x_p[j]+=wpe*adwmw;
		    if(fplus==1) pluspmoments_vol_x_tau_x_p[j]+=wdp;
		    else if(fplus==-1) minuspmoments_vol_x_tau_x_p[j]+=wdp;
		    if(b.buy_sell_indicator==BookItem.BOOK_BUY)
		    {
			vbuymoments_vol_x_tau_x_p[j]+=wpe*bwmw;
			tauvbuymoments_vol_x_tau_x_p[j]+=wpe*(tauorig*bwmw);
			tsmoments_vol_x_tau_x_p[j]-=wpe*(twmw);
			vsmoments_vol_x_tau_x_p[j]-=wpe*(bwmw);
			rtsmoments_vol_x_tau_x_p[j]-=wpe*(Sv*twmw);
			rvsmoments_vol_x_tau_x_p[j]-=wpe*(Sv*bwmw);
		    }
		    else if(b.buy_sell_indicator==BookItem.BOOK_SELL)
		    {
			vsellmoments_vol_x_tau_x_p[j]+=wpe*bwmw;
			tauvsellmoments_vol_x_tau_x_p[j]+=wpe*(tauorig*bwmw);
			tsmoments_vol_x_tau_x_p[j]+=wpe*(twmw);
			vsmoments_vol_x_tau_x_p[j]+=wpe*(bwmw);
			rtsmoments_vol_x_tau_x_p[j]+=wpe*(Sv*twmw);
			rvsmoments_vol_x_tau_x_p[j]+=wpe*(Sv*bwmw);
		    }
		}
	    }
	}
	tinterval=prevt;
    }


    if(bref!=null)
    { // optimization

	// previous interval & half slice
	svol+=ref.svol;
	final double bref_dp=bref.price-p_offset,bref_evol=(vx-bref.shares)/v_scale,
	    dpa=ref.p_offset-p_offset,dvol=(ref.vol-vol)/ref.v_scale,dscale=ref.v_scale/v_scale,
	    dp_nonzero=dp_diff_last_ref==null?0:bref.price-dp_diff_last_ref.price,da_nonzero=Math.abs(dp_nonzero);
	double wvest=1,scalesk=1;
	for(int k=0,j=0;k<nvm;k++,wvest*=bref_evol,scalesk*=dscale)
	{
	    double wtest=wvest;
	    for(int l=0;l<ntm;l++,wtest*=bref_delay)
	    {
		double wpest=wtest;
		for(int m=0;m<npm;m++,j++,wpest*=bref_dp)
		{
		    // add missed half slice
		    tmoments_vol_x_tau_x_p[j]+=wpest*expdt_bref_delay;
		    Dp_tmoments_vol_x_tau_x_p[j]+=dp_nonzero*wpest*expdt_bref_delay;
		    Da_tmoments_vol_x_tau_x_p[j]+=da_nonzero*wpest*expdt_bref_delay;
		    if(bref.buy_sell_indicator==BookItem.BOOK_BUY){
			final double Sv=bref.best_sell!=null?Math.abs(bref.best_sell.price-bref.price):0;
			tsmoments_vol_x_tau_x_p[j]-=wpest*expdt_bref_delay;
			rtsmoments_vol_x_tau_x_p[j]-=(Sv)*wpest*expdt_bref_delay;
		    }
		    else if(bref.buy_sell_indicator==BookItem.BOOK_SELL){
			final double Sv=bref.best_buy!=null?Math.abs(bref.price-bref.best_buy.price):0;
			tsmoments_vol_x_tau_x_p[j]+=wpest*expdt_bref_delay;
			rtsmoments_vol_x_tau_x_p[j]+=(Sv)*wpest*expdt_bref_delay;
		    }

		    // Convert old interval to new (v_cur,t_cur,p_offset) basis,
		    // D3 polynomial group transform.
		    // The expansion would long overflow if any of {k,l,m}>=62.
		    double skt=0,skv=0,skTv=0,skVdP=0,Dp_skt=0,Dp_skv=0,Da_skt=0,Da_skv=0,skvbuy=0,skvsell=0,skp=0,skTp=0,skpplus=0,skpminus=0,ska=0,skad=0,skts=0,skvs=0,skrts=0,skrvs=0,sktauvbuy=0,sktauvsell=0;long fk=1;
		    for(int ks=0;ks<=k;fk=(fk*(k-ks))/(++ks))
		    {
			double slt=0,slv=0,slTv=0,slVdP=0,Dp_slt=0,Dp_slv=0,Da_slt=0,Da_slv=0,slvbuy=0,slvsell=0,slp=0,slTp=0,slpplus=0,slpminus=0,sla=0,slad=0,slts=0,slvs=0,slrts=0,slrvs=0,sltauvbuy=0,sltauvsell=0;long fl=1;
			for(int ls=0;ls<=l;fl=(fl*(l-ls))/(++ls))
			{
			    double smt=0,smv=0,smTv=0,smVdP=0,Dp_smt=0,Dp_smv=0,Da_smt=0,Da_smv=0,smvbuy=0,smvsell=0,smp=0,smTp=0,smpplus=0,smpminus=0,sma=0,smad=0,smts=0,smvs=0,smrts=0,smrvs=0,smtauvbuy=0,smtauvsell=0;long fm=1;
			    for(int ms=m,o=(ks*ntm+ls)*npm;ms>=0;fm=(fm*ms)/(m-(--ms)),++o)
			    {
				smt=dpa*smt+fm*ref.tmoments_vol_x_tau_x_p[o];
				smv=dpa*smv+fm*ref.vmoments_vol_x_tau_x_p[o];
				smTv=dpa*smTv+fm*(ref.Tvmoments_vol_x_tau_x_p[o]+ref_delay*ref.vmoments_vol_x_tau_x_p[o]);
				smVdP=dpa*smVdP+fm*ref.VdPmoments_vol_x_tau_x_p[o];
				Dp_smt=dpa*Dp_smt+fm*ref.Dp_tmoments_vol_x_tau_x_p[o];
				Dp_smv=dpa*Dp_smv+fm*ref.Dp_vmoments_vol_x_tau_x_p[o];
				Da_smt=dpa*Da_smt+fm*ref.Da_tmoments_vol_x_tau_x_p[o];
				Da_smv=dpa*Da_smv+fm*ref.Da_vmoments_vol_x_tau_x_p[o];
				smvbuy=dpa*smvbuy+fm*ref.vbuymoments_vol_x_tau_x_p[o];
				smvsell=dpa*smvsell+fm*ref.vsellmoments_vol_x_tau_x_p[o];
				smp=dpa*smp+fm*ref.pmoments_vol_x_tau_x_p[o];
				smTp=dpa*smTp+fm*(ref.Tpmoments_vol_x_tau_x_p[o]+ref_delay*ref.pmoments_vol_x_tau_x_p[o]);
				smpplus=dpa*smpplus+fm*ref.pluspmoments_vol_x_tau_x_p[o];
				smpminus=dpa*smpminus+fm*ref.minuspmoments_vol_x_tau_x_p[o];
				sma=dpa*sma+fm*ref.amoments_vol_x_tau_x_p[o];
				smad=dpa*smad+fm*ref.admoments_vol_x_tau_x_p[o];
				smts=dpa*smts+fm*ref.tsmoments_vol_x_tau_x_p[o];
				smvs=dpa*smvs+fm*ref.vsmoments_vol_x_tau_x_p[o];
				smrts=dpa*smrts+fm*ref.rtsmoments_vol_x_tau_x_p[o];
				smrvs=dpa*smrvs+fm*ref.rvsmoments_vol_x_tau_x_p[o];
				smtauvbuy=dpa*smtauvbuy+fm*ref.tauvbuymoments_vol_x_tau_x_p[o];
				smtauvsell=dpa*smtauvsell+fm*ref.tauvsellmoments_vol_x_tau_x_p[o];
			    }
			    slt=ref_delay*slt+fl*smt;
			    slv=ref_delay*slv+fl*smv;
			    slTv=ref_delay*slTv+fl*smTv;
			    slVdP=ref_delay*slVdP+fl*smVdP;
			    Dp_slt=ref_delay*Dp_slt+fl*Dp_smt;
			    Dp_slv=ref_delay*Dp_slv+fl*Dp_smv;
			    Da_slt=ref_delay*Da_slt+fl*Da_smt;
			    Da_slv=ref_delay*Da_slv+fl*Da_smv;
			    slvbuy=ref_delay*slvbuy+fl*smvbuy;
			    slvsell=ref_delay*slvsell+fl*smvsell;
			    slp=ref_delay*slp+fl*smp;
			    slTp=ref_delay*slTp+fl*smTp;
			    slpplus=ref_delay*slpplus+fl*smpplus;
			    slpminus=ref_delay*slpminus+fl*smpminus;
			    sla=ref_delay*sla+fl*sma;
			    slad=ref_delay*slad+fl*smad;
			    slts=ref_delay*slts+fl*smts;
			    slvs=ref_delay*slvs+fl*smvs;
			    slrts=ref_delay*slrts+fl*smrts;
			    slrvs=ref_delay*slrvs+fl*smrvs;
			    sltauvbuy=ref_delay*sltauvbuy+fl*smtauvbuy;
			    sltauvsell=ref_delay*sltauvsell+fl*smtauvsell;
			}
			skt=dvol*skt+fk*slt;
			skv=dvol*skv+fk*slv;
			skTv=dvol*skTv+fk*slTv;
			skVdP=dvol*skVdP+fk*slVdP;
			Dp_skt=dvol*Dp_skt+fk*Dp_slt;
			Dp_skv=dvol*Dp_skv+fk*Dp_slv;
			Da_skt=dvol*Da_skt+fk*Da_slt;
			Da_skv=dvol*Da_skv+fk*Da_slv;
			skvbuy=dvol*skvbuy+fk*slvbuy;
			skvsell=dvol*skvsell+fk*slvsell;
			skp=dvol*skp+fk*slp;
			skTp=dvol*skTp+fk*slTp;
			skpplus=dvol*skpplus+fk*slpplus;
			skpminus=dvol*skpminus+fk*slpminus;
			ska=dvol*ska+fk*sla;
			skad=dvol*skad+fk*slad;
			skts=dvol*skts+fk*slts;
			skvs=dvol*skvs+fk*slvs;
			skrts=dvol*skrts+fk*slrts;
			skrvs=dvol*skrvs+fk*slrvs;
			sktauvbuy=dvol*sktauvbuy+fk*sltauvbuy;
			sktauvsell=dvol*sktauvsell+fk*sltauvsell;
		    }
		    tmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skt);
		    vmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skv);
		    Tvmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skTv);
		    VdPmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*(skVdP+(ref.vol-vol)*skp));
		    Dp_tmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*Dp_skt);
		    Dp_vmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*Dp_skv);
		    Da_tmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*Da_skt);
		    Da_vmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*Da_skv);
		    vbuymoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skvbuy);
		    vsellmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skvsell);
		    pmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skp);
		    Tpmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skTp);
		    pluspmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skpplus);
		    minuspmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skpminus);
		    amoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*ska);
		    admoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skad);
		    tsmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skts);
		    vsmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skvs);
		    rtsmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skrts);
		    rvsmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*skrvs);
		    tauvbuymoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*sktauvbuy);
		    tauvsellmoments_vol_x_tau_x_p[j]+=exp_ref_delay*(scalesk*sktauvsell);
		}
	    }
	}
    } // end optimization
    if(offset_eq_average && bref!=null && npm>1)
    {
	// reduce rounding errors for known values
	// <p-<p>>=0
	tmoments_vol_x_tau_x_p[(0*ntm+0)*npm+1]=0;
    }

    return new QVMData(currenttime,
		       tau,
		       vol,
		       svol,
		       p_offset,
		       pt_sum,
		       t_sum,
		       v_sum,
		       v_scale,
		       tmoments_vol_x_tau_x_p,
		       vmoments_vol_x_tau_x_p,
		       Tvmoments_vol_x_tau_x_p,
		       VdPmoments_vol_x_tau_x_p,
		       Dp_tmoments_vol_x_tau_x_p,
		       Dp_vmoments_vol_x_tau_x_p,
		       Da_tmoments_vol_x_tau_x_p,
		       Da_vmoments_vol_x_tau_x_p,
		       vbuymoments_vol_x_tau_x_p,
		       vsellmoments_vol_x_tau_x_p,
		       pmoments_vol_x_tau_x_p,
		       Tpmoments_vol_x_tau_x_p,
		       pluspmoments_vol_x_tau_x_p,
		       minuspmoments_vol_x_tau_x_p,
		       amoments_vol_x_tau_x_p,
		       admoments_vol_x_tau_x_p,
		       tsmoments_vol_x_tau_x_p,
		       vsmoments_vol_x_tau_x_p,
		       rtsmoments_vol_x_tau_x_p,
		       rvsmoments_vol_x_tau_x_p,
		       tauvbuymoments_vol_x_tau_x_p,
		       tauvsellmoments_vol_x_tau_x_p,
		       nvm,
		       ntm,
		       npm,
		       bref_next,
		       dp_diff_last_next);
}


    private static void matchWithTest(final String description,
				      final QVMData r,
				      final Trade [] tb,
				      final int tsize,
				      final double eps)
    {
	System.err.println("QVMData:matchWithTest():======================"+description);

	final CalculateKendall.TradeIPVMData rt=
	    CalculateKendall.TradeIPVMData.calculateTestTradeIPVMData(r.nvm,
								      r.ntm,
								      r.npm,
								      r.currenttime,
								      r.tau,
								      tb,
								      tsize);
	checkMatchingTradeIPVMData(r,rt,eps);

	System.err.println("QVMData:matchWithTest(): OK eps="+eps);
    }


    private static void checkMatchingTradeIPVMData(final QVMData r,
						   final CalculateKendall.TradeIPVMData ref,
						   final double eps)
    {
	if(r.currenttime!=ref.currenttime)
	{
	    throw new RuntimeException("r.currenttime="+r.currenttime+
				       " ref.currenttime="+ref.currenttime);
	}
	if(r.tau!=ref.tau)
	{
	    throw new RuntimeException("r.tau="+r.tau+" ref.tau="+ref.tau);
	}
	if(r.vol!=ref.vol)
	{
	    throw new RuntimeException("r.vol="+r.vol+" ref.vol="+ref.vol);
	}
	if(r.nvm!=ref.nvm||r.ntm!=ref.ntm||r.npm!=ref.np)
	{
	    throw new RuntimeException("r dims=("+r.nvm+":"+r.ntm+":"+r.npm+")\n"+
				       "ref dims=("+ref.nvm+":"+ref.ntm+":"+ref.np+")");
	}

	UnitTests.checkDoubleRel(r.pt_sum,"r.pt_sum",ref.pt_sum,"ref.pt_sum",eps);
	UnitTests.checkDoubleRel(r.t_sum,"r.t_sum",ref.t_sum,"ref.t_sum",eps);
	UnitTests.checkDoubleRel(r.v_sum,"r.v_sum",ref.v_sum,"ref.v_sum",eps);
	UnitTests.checkDoubleRel(r.p_offset,"r.p_offset",ref.p_offset,"ref.p_offset",eps);
	UnitTests.checkDoubleRel(r.v_scale,"r.v_scale",ref.v_scale,"ref.v_scale",eps);
	if(r.npm>1)
	System.err.println("r.tmoments_vol_x_tau_x_p[1]="+r.tmoments_vol_x_tau_x_p[1]+
			   "\tref.moments_vol_x_tau_x_p[0][0][1]="+ref.moments_vol_x_tau_x_p[0][0][1]);

	for(int k=0,j=0;k<r.nvm;k++)
	{
	    for(int l=0;l<r.ntm;l++)
	    {
		for(int m=0;m<r.npm;m++,j++)
		{
		    final double mom;
		    if(k==0 && l==0 && m==1 && Math.abs(r.tmoments_vol_x_tau_x_p[j])<eps) mom=0;
		    else mom=r.tmoments_vol_x_tau_x_p[j];
		    if(UnitTests.checkDoubleRelNoFail(mom,ref.moments_vol_x_tau_x_p[k][l][m],eps))
		    {
			throw new RuntimeException("Mismatch:r.tmoments_vol_x_tau_x_p["+j+"]="+r.tmoments_vol_x_tau_x_p[j]+"\n"+
						   "ref.moments_vol_x_tau_x_p["+k+"]["+l+"]["+m+"]="+ref.moments_vol_x_tau_x_p[k][l][m]+"\neps="+eps);
		    }
		}
	    }
	}

	final double [][][] abc=get3ArrayFromArrayForTesting(r.tmoments_vol_x_tau_x_p,r.nvm,r.ntm,r.npm);
	for(int k=0;k<r.nvm;k++)
	for(int l=0;l<r.ntm;l++)
	{
	    UnitTests.checkDoubleArrayRel1(abc[k][l],"ro.odata.tmoments_vol_x_tau_x_p[k][l][*]",
						 ref.moments_vol_x_tau_x_p[k][l],"ref.moments_vol_x_tau_x_p[k][l]",eps);
	}
	System.err.println("QVMData:checkMatchingTradeIPVMData(): OK eps="+eps);
    }

    private static void checkVTProjections(final QVMData o,final double eps)
    {
	final double [][][] abc=get3ArrayFromArrayForTesting(o.tmoments_vol_x_tau_x_p,o.nvm,o.ntm,o.npm);
	final double pt0=abc[0][0][0],
	    vs=abc.length<=1?Double.NaN:abc[1][0][0]/pt0,
	    ts=abc[0].length<=1?Double.NaN:abc[0][1][0]/pt0;

	for(int pi=1;pi<=6;pi++)
	{
	    if(pi>o.nvm || pi>o.ntm)
		continue;
	    final double [] p=o.projecToVT(pi),pr=new double [pi*o.npm];
	    for(int m=0;m<o.npm;m++)
	    {
		if(pi>=1)
		pr[0*o.npm+m]=abc[0][0][m];

		if(pi>=2)
		pr[1*o.npm+m]=abc[1][0][m]/vs-abc[0][1][m]/ts;

		if(pi>=3)
		pr[2*o.npm+m]=  abc[2][0][m]/vs/vs
		             -2*abc[1][1][m]/vs/ts
			     +  abc[0][2][m]/ts/ts;

		if(pi>=4)
		pr[3*o.npm+m]=  abc[3][0][m]/vs/vs/vs
		             -3*abc[2][1][m]/vs/vs/ts
		             +3*abc[1][2][m]/vs/ts/ts
			     -  abc[0][3][m]/ts/ts/ts;

		if(pi>=5)
		pr[4*o.npm+m]=  abc[4][0][m]/vs/vs/vs/vs
		             -4*abc[3][1][m]/vs/vs/vs/ts
		             +6*abc[2][2][m]/vs/vs/ts/ts
		             -4*abc[1][3][m]/vs/ts/ts/ts
			     +  abc[0][4][m]/ts/ts/ts/ts;
		if(pi>=6)
		pr[5*o.npm+m]=  abc[5][0][m]/vs/vs/vs/vs/vs
		             -5*abc[4][1][m]/vs/vs/vs/vs/ts
		            +10*abc[3][2][m]/vs/vs/vs/ts/ts
		            -10*abc[2][3][m]/vs/vs/ts/ts/ts
		             +5*abc[1][4][m]/vs/ts/ts/ts/ts
			     -  abc[0][5][m]/ts/ts/ts/ts/ts;
	    }
	    UnitTests.checkDoubleArrayRel1(p,"p",pr,"pr",eps);
	    for(int k=0;k<pi;k++)
	    {
		System.err.println("QVMData:pr["+k+"][0]="+pr[k*o.npm+0]);
	    }
	    System.err.println("QVMData:checkVTProjections(): OK d="+pi);

	}
	System.err.println("QVMData:checkVTProjections(): ALL OK");
    }


    private static void checkPOffset(final QVMData o,final java.util.Random r,final double eps)
    {
	final double dp_offset=r.nextDouble();
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_dV),"vm",shiftPOffsetOnly(o.vmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:vm",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_TdV),"vm",shiftPOffsetOnly(o.Tvmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:Tvm",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_VdPdt),"vm",shiftPOffsetOnly(o.VdPmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:VdP",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_dT),"tm",shiftPOffsetOnly(o.tmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:tm",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_DPV),"Dp_vm",shiftPOffsetOnly(o.Dp_vmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:Dp_vm",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_DPT),"Dp_tm",shiftPOffsetOnly(o.Dp_tmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:Dp_tm",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_DAV),"Da_vm",shiftPOffsetOnly(o.Da_vmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:Da_vm",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_DAT),"Da_tm",shiftPOffsetOnly(o.Da_tmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:Da_tm",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_VBUY),"vbuym",shiftPOffsetOnly(o.vbuymoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:vbuym",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_VSELL),"vsellm",shiftPOffsetOnly(o.vsellmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:vsellm",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_DP),"dpm",shiftPOffsetOnly(o.pmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:dpm",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_TDP),"Tdpm",shiftPOffsetOnly(o.Tpmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:Tdpm",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_DPPLUS),"dpmplus",shiftPOffsetOnly(o.pluspmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:dpmplus",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_DPMINUS),"dpmminus",shiftPOffsetOnly(o.minuspmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:dpmminus",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_A),"a",shiftPOffsetOnly(o.amoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:a",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_AD),"ad",shiftPOffsetOnly(o.admoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:ad",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_TS),"ts",shiftPOffsetOnly(o.tsmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:ts",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_VS),"vs",shiftPOffsetOnly(o.vsmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:vs",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_RTS),"rts",shiftPOffsetOnly(o.rtsmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:rts",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_RVS),"rvs",shiftPOffsetOnly(o.rvsmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:rvs",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_TAUIBUY),"tauvbuy",shiftPOffsetOnly(o.tauvbuymoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:tauvbuy",eps);
	UnitTests.checkDoubleArrayRel(o.shiftOffset(0,0,dp_offset,MOM_TAUISELL),"tauvsell",shiftPOffsetOnly(o.tauvsellmoments_vol_x_tau_x_p,o.ntm*o.nvm,o.npm,dp_offset),"po:tauvsell",eps);

	System.err.println("QVMData:checkPOffset(): matched OK nvm="+o.nvm+" ntm="+o.ntm+" npm="+o.npm);
    }

private static void checkCalculations(final int tsize,
				      final java.util.Random r,
				      final int nvm,
				      final int ntm,
				      final int npm,
				      final int n_price_quants,
				      final double eps)
{
    final Trade [] tb=CalculateKendall.TradeEMoments.getRandomTradesQuantumPrice(tsize,r,n_price_quants);
    final long tau=10000L*tb.length/20,currenttime=tb[tb.length-1].time+20000;
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println("QVMData:checkCalculations(): test with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+" nvm="+nvm+" ntm="+ntm+" npm="+npm+
		       " n_price_quants="+n_price_quants+" eps="+eps);

    final QVMData ro=QVMData.calculateQVMData(nvm,ntm,npm,currenttime,tau,tb,tsize,null);
    matchWithTest("INITIAL CALCULATION",ro,tb,tsize,eps);

    // recurrent test
    final int n_split=8*(tb.length/10);
    final QVMData em_ref=QVMData.calculateQVMData(nvm,ntm,npm,(tb[n_split-1].time+tb[n_split].time)/2,tau,tb,n_split,null);
    matchWithTest("GET REFERENCE FOR RECURRENT",em_ref,tb,n_split,eps);
    if(false){
	final var rRef=CommonlyUsedMoments_TestFromSample.fromSeqRecurrently(
	    IvsVT_CalculateMoments_Monomials::new,ntm,em_ref.currenttime,tau,java.util.Arrays.copyOf(tb,n_split));
	rRef.updateBasisToNewTimeReference(em_ref.currenttime,em_ref.p_offset);
	System.err.println("Orig rRef.dv="+java.util.Arrays.toString(rRef.getdVMoments()));
	System.err.println("Orig rRef.VdP="+java.util.Arrays.toString(rRef.getVdPMoments()));
	System.err.println("Orig rRef.dP="+java.util.Arrays.toString(rRef.getdpMoments()));
	rRef.updateBasisToNewTimeReference(currenttime,ro.p_offset);
	System.err.println("sh rRef.dv="+java.util.Arrays.toString(rRef.getdVMoments()));
	System.err.println("sh rRef.VdP="+java.util.Arrays.toString(rRef.getVdPMoments()));
	System.err.println("sh rRef.dP="+java.util.Arrays.toString(rRef.getdpMoments()));
    }

    final QVMData em_fromref=QVMData.calculateQVMData(nvm,ntm,npm,currenttime,tau,tb,tsize,em_ref);
    checkMatching(em_fromref,ro,eps,eps);
    matchWithTest("RECURRENT CALCULATIONS",em_fromref,tb,tsize,eps);

    // check with data cut
    final int ncut=tb.length-n_split;
    System.arraycopy(tb,n_split,tb,0,ncut);
    final QVMData emc_cut=QVMData.calculateQVMData(nvm,ntm,npm,currenttime,tau,tb,ncut,em_ref);
    checkMatching(ro,emc_cut,eps,eps);
    checkMatching(em_fromref,emc_cut,eps*0,eps); // have to match exactly

    checkVTProjections(ro,eps);
    checkPOffset(ro,r,eps);

    // dump values to diag
    for(int k=0;k<ro.nvm;k++)
    {
	System.err.println("V00:tmoments_vol_x_tau_x_p["+k+"][0][0]="+ro.tmoments_vol_x_tau_x_p[k*ro.ntm*ro.npm]+
			   " ; should be "+(k==0?"1":"??"));
    }
    for(int l=0;l<ro.ntm;l++)
    {
	System.err.println("T00:tmoments_vol_x_tau_x_p[0]["+l+"][0]="+ro.tmoments_vol_x_tau_x_p[l*ro.npm]+
			   " ; should be = "+(((l/2)*2==l?1:-1)*Polynomials.fact(l)));
    }
    for(int m=0;m<ro.npm;m++)
    {
	System.err.println("P00:tmoments_vol_x_tau_x_p[0][0]["+m+"]="+ro.tmoments_vol_x_tau_x_p[m]
			   +" ; should be "+(m==0?"1":(m==1?"0":"??")));
    }

    System.err.println("OrdersAndTrades.USE_ORIGTIME="+OrdersAndTrades.USE_ORIGTIME+
		       "\nOrdersAndTrades.SAVE_NONDISPLAYABLE_TRADES="+OrdersAndTrades.SAVE_NONDISPLAYABLE_TRADES+
		       "\nOrdersAndTrades.SAVE_EDGE_ORDERS="+OrdersAndTrades.SAVE_EDGE_ORDERS+
		       "\nOrdersAndTrades.FLAG_PSEUDOTRADES="+OrdersAndTrades.FLAG_PSEUDOTRADES);
    System.err.println("QVMData:checkCalculations(): OK with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+" nvm="+nvm+" ntm="+ntm+" npm="+npm+
		       " n_price_quants="+n_price_quants+" eps="+eps);
}

/** Used in old code of {@link QVM.QVMAverage}.
 */
    double [] projecToVT(final int nd)
    {
	final int ntm=this.ntm,npm=this.npm;
	if(nd>this.nvm||nd>ntm||nd<1)
	{
	    throw new IllegalArgumentException("nd="+nd);
	}
	final double [] abc=this.tmoments_vol_x_tau_x_p,ad=new double [nd*npm];
	final double pt0=abc[0],
	    vs=this.nvm<=1?(nd<=1?1:Double.NaN):abc[1*ntm*npm]/pt0,
	    ts=ntm<=1?Double.NaN:abc[1*npm]/pt0;
	for(int id=0,j=0;id<nd;id++)
	{
	    for(int m=0;m<npm;m++,j++)
	    {
		double s=0,w=1;long fk=1;
		for(int ids=0;ids<=id;fk*=(ids-id),fk/=(++ids))
		{
		    s=s/vs+fk*(abc[(ntm*(id-ids)+ids)*npm+m]/w);
		    w*=ts;
		}
		ad[j]=s;
	    }
	}
	return ad;
    }

/** Possible moment {@code whatmom} for {@link #shiftOffset}.
 * Contrary to {@link DataInterfaceDirectTradesHistorical}
 * these IDs are internal and {@link QVMData} class specific.
 */
static final int MOM_dV=67,MOM_dT=68,MOM_TdV=168,MOM_VdPdt=169,MOM_VBUY=69,MOM_VSELL=70,MOM_DP=71,MOM_TDP=171,MOM_DPPLUS=72,MOM_DPMINUS=73,MOM_A=74,MOM_AD=75,MOM_TS=76,MOM_VS=77,MOM_RTS=78,MOM_RVS=79,MOM_TAUIBUY=80,MOM_TAUISELL=81,MOM_DPV=82,MOM_DPT=83,MOM_DAV=84,MOM_DAT=85;

/** Used for unit tests of {@link QVMDataL} and {@link QVMDataP}.
 */
double [] shiftOffset(final double dv,final double dt,final double dp,final int whatmom)
    {
	final int ntm=this.ntm,npm=this.npm;
	final double [] abc;
	if(whatmom==MOM_dT) abc=this.tmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_dV) abc=this.vmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_TdV) abc=this.Tvmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_VdPdt) abc=this.VdPmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_DPT) abc=this.Dp_tmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_DPV) abc=this.Dp_vmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_DAT) abc=this.Da_tmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_DAV) abc=this.Da_vmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_VBUY) abc=this.vbuymoments_vol_x_tau_x_p;
	else if(whatmom==MOM_VSELL) abc=this.vsellmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_DP) abc=this.pmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_TDP) abc=this.Tpmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_DPPLUS) abc=this.pluspmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_DPMINUS) abc=this.minuspmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_A) abc=this.amoments_vol_x_tau_x_p;
	else if(whatmom==MOM_AD) abc=this.admoments_vol_x_tau_x_p;
	else if(whatmom==MOM_TS) abc=this.tsmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_VS) abc=this.vsmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_RTS) abc=this.rtsmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_RVS) abc=this.rvsmoments_vol_x_tau_x_p;
	else if(whatmom==MOM_TAUIBUY) abc=this.tauvbuymoments_vol_x_tau_x_p;
	else if(whatmom==MOM_TAUISELL) abc=this.tauvsellmoments_vol_x_tau_x_p;
	else throw new IllegalArgumentException("whatmom="+whatmom);
	return shiftOffset(nvm,ntm,npm,abc,dv,dt,dp);
    }

private static double [] shiftOffset(final int nvm,final int ntm,final int npm,final double [] in,
				     final double dv,final double dt,final double dp)
{
    final double [] out=new double [nvm*ntm*npm];
    for(int k=0,j=0;k<nvm;k++)
    {
	for(int l=0;l<ntm;l++)
	{
	    for(int m=0;m<npm;m++,j++)
	    {
		double sk=0;long fk=1;
		for(int ks=0;ks<=k;fk=(fk*(k-ks))/(++ks))
		{
		    double sl=0;long fl=1;
		    for(int ls=0;ls<=l;fl=(fl*(l-ls))/(++ls))
		    {
			double sm=0;long fm=1;
			for(int ms=m,o=(ks*ntm+ls)*npm;ms>=0;fm=(fm*ms)/(m-(--ms)),++o)
			{
			    sm=dp*sm+fm*in[o];
			}
			sl=dt*sl+fl*sm;
		    }
		    sk=dv*sk+fk*sl;
		}
		out[j]=sk;
	    }
	}
    }
    return out;
}



/** Used for unit tests of {@link QVM.QVMCoefs}.
 */
static double [] getArrayFrom3ArrayForTesting(final double [][][] abc,
					      final int na,final int nb,final int nc)
    {
	final double [] mom=new double [na*nb*nc];
	for(int k=0,j=0;k<na;k++)
	{
	    for(int l=0;l<nb;l++)
	    {
		for(int m=0;m<nc;m++,j++)
		{
		    mom[j]=abc[k][l][m];
		}
	    }
	}
	return mom;
    }

/** Used for unit tests of {@link QVMDataL} and {@link QVMDataP}.
 * @see #get4ArrayFromArrayForTesting
 */
static double [][][] get3ArrayFromArrayForTesting(final double [] mom,
						  final int na,final int nb,final int nc)
    {
	return get4ArrayFromArrayForTesting(mom,1,na,nb,nc)[0];
    }

/**  Used for unit tests of {@link QVM.QVMDataMU}.
 */
static double [][][][] get4ArrayFromArrayForTesting(final double [] mom,
						    final int na,final int nb,final int nc,final int nd)
    {
	final double [][][][] abcd=new double [na][nb][nc][nd];
	for(int k=0,j=0;k<na;k++)
	{
	    for(int l=0;l<nb;l++)
	    {
		for(int m=0;m<nc;m++)
		{
		    for(int n=0;n<nd;n++,j++)
		    {
			abcd[k][l][m][n]=mom[j];
		    }
		}
	    }
	}
	return abcd;
    }

private static double [] shiftPOffsetOnly(final double [] moments,int nxm,int npm,final double dp)
    {
	final double [] cmoments=new double [nxm*npm];
	for(int k=0,j=0;k<nxm;k++)
	{
	    for(int m=0;m<npm;m++,j++)
	    {
		double sm=0;long fm=1;
		for(int ms=m,o=k*npm;ms>=0;fm=(fm*ms)/(m-(--ms)),++o)
		{
		    sm=dp*sm+fm*moments[o];
		}
		cmoments[j]=sm;
	    }
	}
	return cmoments;
    }

static void test_Self(final long initrandom)
{
    final java.util.Random r=new java.util.Random(initrandom);
    final double eps=1e-4;

    for(int nvm :new int []{2,3,1,2,3,4,5})
    {
	for(int ntm :new int []{2,1,2,3,4,5})
	{
	    for(int npm :new int []{2,1,3,4,5})
	    {
		for(int i=0;i<5;i++)
		for(int n_price_quants:new int[]{-1,2,3,4,5,10})
		{
		    checkCalculations(100+r.nextInt(100),r,nvm,ntm,npm,n_price_quants,eps);
		    checkCalculations(20010+r.nextInt(100010),r,nvm,ntm,npm,n_price_quants,eps);
		}
	    }
	}
    }
}


public static void main(String [] args)
{
    System.err.println("QVMData::main(): starting tests; ver=$Id: QVMData.java,v 1.185 2022/09/14 14:46:35 mal Exp $");

    final long t0=System.currentTimeMillis();
    if(false){QTest.findInitRandom_NoFail(1,100,(final Long irandom) -> test_Self(irandom));}
    else {test_Self(1);}

    final long t1=System.currentTimeMillis();
    System.err.println("QVMData::main(): tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: QVMData.java,v 1.185 2022/09/14 14:46:35 mal Exp $");
}


} // END OF class QVMData
