package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.WIntegratorMonomials;

/** Extract the moments in Monomials basis from {@link QVMData}.
 *  @see QVMData
 *  @see com.polytechnik.freemoney.CommonlyUsedMomentsMonomials
 *  @version $Id: QVMDataDirectAccess.java,v 1.29 2022/09/14 08:19:26 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QVMDataDirectAccess implements DataInterfaceToMoments<QVMData> {

public void setM(final double [] m,final int n,final int mom,final QVMData odata,final int nmoms)
{
    if(n>=odata.npm||nmoms>odata.ntm||odata.nvm<=0) throw new IllegalArgumentException("n="+n+" odata.npm="+odata.npm+" nmoms="+nmoms+" odata.ntm="+odata.ntm+" odata.nvm="+odata.nvm);


    final double [] abc;
    if(mom==M_I) abc=odata.vmoments_vol_x_tau_x_p;
    else if(mom==M_TI) abc=odata.Tvmoments_vol_x_tau_x_p;
    else if(mom==M_ISELL) abc=odata.vsellmoments_vol_x_tau_x_p;
    else if(mom==M_IBUY) abc=odata.vbuymoments_vol_x_tau_x_p;
    else if(mom==M_DP) abc=odata.pmoments_vol_x_tau_x_p;
    else if(mom==M_TDP) abc=odata.Tpmoments_vol_x_tau_x_p;
    else if(mom==M_VdPdt) abc=odata.VdPmoments_vol_x_tau_x_p;
    else if(mom==M_DPPLUS) abc=odata.pluspmoments_vol_x_tau_x_p;
    else if(mom==M_DPMINUS) abc=odata.minuspmoments_vol_x_tau_x_p;
    else if(mom==M_T) abc=odata.tmoments_vol_x_tau_x_p;
    else if(mom==M_A) abc=odata.amoments_vol_x_tau_x_p;
    else if(mom==M_AD) abc=odata.admoments_vol_x_tau_x_p;
    else if(mom==M_TS) abc=odata.tsmoments_vol_x_tau_x_p;
    else if(mom==M_VS) abc=odata.vsmoments_vol_x_tau_x_p;
    else if(mom==M_RTS) abc=odata.rtsmoments_vol_x_tau_x_p;
    else if(mom==M_RVS) abc=odata.rvsmoments_vol_x_tau_x_p;
    else if(mom==M_TAUIBUY) abc=odata.tauvbuymoments_vol_x_tau_x_p;
    else if(mom==M_TAUISELL) abc=odata.tauvsellmoments_vol_x_tau_x_p;
    else if(mom==M_DPwV) abc=odata.Dp_vmoments_vol_x_tau_x_p;
    else if(mom==M_DPwT) abc=odata.Dp_tmoments_vol_x_tau_x_p;
    else if(mom==M_DAwV) abc=odata.Da_vmoments_vol_x_tau_x_p;
    else if(mom==M_DAwT) abc=odata.Da_tmoments_vol_x_tau_x_p;
    else throw new RuntimeException("mom="+mom+" is not supported by QVMData");
    for(int k=0,o=n;k<nmoms;k++,o+=odata.npm){m[k]=abc[o];}
}

/** Sets \int_{-\infty}^0 t^k exp(t) dt.
 */
public void setMT(final double [] m,final int nmoms){double f=1;for(int k=0;k<nmoms;++k,f*=(-k))m[k]=f;}


public int setFMoments(final double [] Vmoments,final double [] dVmoments,final int n)
{
    for(int k=n;--k>=0;)
    {
	final double [] a=new double [k+1];
	a[k]= -1;
	/*
	  final double [] f=in.clone();
	  final double [] df=Polynomials.getPolynomialDifferentiated(f);
	  TMatr.sumP1(f,df,1);
	  System.err.println("f="+java.util.Arrays.toString(f));
	*/
	Vmoments[k]=TMatr.mulVVn1(WIntegratorMonomials.getPolynomialWIntegratedDt(a),dVmoments);
    }
    return n;
}
public int setDFMoments(final double [] dFmoments,final double [] Fmoments,final int n)
{
    for(int k=0;k<n;k++){dFmoments[k]=((k<=0?0:k*Fmoments[k-1])+Fmoments[k]);}
    return n;
}

public double getPOffset(final QVMData odata){return odata.p_offset;}
public double getPLast(final QVMData odata){return (odata.trade_last!=null?odata.trade_last.price:Double.NaN)-getPOffset(odata);}
public long getTLast(final QVMData odata){return odata.currenttime;}
public long getTau(final QVMData odata){return odata.tau;}
public long getTotalVolume(final QVMData odata){return odata.vol;}

final java.util.function.Function<double [],double []> getFMomentsFromDFMomentsMatchRadau=(dFmoms)->{
    final double [] Fmoms=new double [dFmoms.length-1];
    setFMoments(Fmoms,dFmoms,Fmoms.length);
    TMatr.scaleVectorInPlace(Fmoms,-1); // to match sign
    return Fmoms;
};
}
