package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.WIntegratorLaguerre;
import com.polytechnik.freemoney.WIntegratorMonomials;

/** Moments data in Laguerre basis, use {@link QVMDataLDirectAccess} to extract.
 *  @see QVMDataLDirectAccess
 *  @see com.polytechnik.freemoney.CommonlyUsedMomentsLaguerre
 *  @version $Id: QVMDataL.java,v 1.185 2022/09/14 08:19:26 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QVMDataL {

final long currenttime,tau,vol,svol;
final double p_offset;

final int ntm,npm;

final double [] // here offset=p_offset and array dims are: ntm,npm.
    tmoments_Ltau_x_p, // [k][l]=sum L_k(t_cur-t) (p-offset)^l dt_i exp()
    vmoments_Ltau_x_p, // [k][l]=sum L_k(t_cur-t) (p-offset)^l dv_i exp()
    Tvmoments_Ltau_x_p, // [k][l]=sum ((t-t_cur)/tau)*L_k(t_cur-t) (p-offset)^l dv_i exp()
    VdPmoments_Ltau_x_p,
    Dp_tmoments_Ltau_x_p,
    Dp_vmoments_Ltau_x_p,
    Da_tmoments_Ltau_x_p,
    Da_vmoments_Ltau_x_p,
    vbuymoments_Ltau_x_p,
    vsellmoments_Ltau_x_p,
    pmoments_Ltau_x_p,
    Tpmoments_Ltau_x_p,
    pluspmoments_Ltau_x_p,
    minuspmoments_Ltau_x_p,
    amoments_Ltau_x_p,
    admoments_Ltau_x_p,
    tsmoments_Ltau_x_p,
    vsmoments_Ltau_x_p,
    rtsmoments_Ltau_x_p,
    rvsmoments_Ltau_x_p,
    tauvbuymoments_Ltau_x_p,
    tauvsellmoments_Ltau_x_p
    ;

// references to previos calculations
final Trade trade_last;
final Trade dp_diff_last;

private QVMDataL(final long currenttime,
		 final long tau,
		 final long vol,
		 final long svol,
		 final double p_offset,

		 final double [] tmoments_Ltau_x_p,
		 final double [] vmoments_Ltau_x_p,
		 final double [] Tvmoments_Ltau_x_p,
		 final double [] VdPmoments_Ltau_x_p,
		 final double [] Dp_tmoments_Ltau_x_p,
		 final double [] Dp_vmoments_Ltau_x_p,
		 final double [] Da_tmoments_Ltau_x_p,
		 final double [] Da_vmoments_Ltau_x_p,
		 final double [] pmoments_Ltau_x_p,
		 final double [] Tpmoments_Ltau_x_p,
		 final double [] pluspmoments_Ltau_x_p,
		 final double [] minuspmoments_Ltau_x_p,
		 final double [] vbuymoments_Ltau_x_p,
		 final double [] vsellmoments_Ltau_x_p,
		 final double [] amoments_Ltau_x_p,
		 final double [] admoments_Ltau_x_p,
		 final double [] tsmoments_Ltau_x_p,
		 final double [] vsmoments_Ltau_x_p,
		 final double [] rtsmoments_Ltau_x_p,
		 final double [] rvsmoments_Ltau_x_p,
		 final double [] tauvbuymoments_Ltau_x_p,
		 final double [] tauvsellmoments_Ltau_x_p,
		 final int ntm,
		 final int npm,

		 // references
		 final Trade trade_last,
		 final Trade dp_diff_last)
{
    this.currenttime=currenttime;
    this.tau=tau;
    this.vol=vol;
    this.svol=svol;
    this.p_offset=p_offset;
    this.tmoments_Ltau_x_p=tmoments_Ltau_x_p;
    this.vmoments_Ltau_x_p=vmoments_Ltau_x_p;
    this.Tvmoments_Ltau_x_p=Tvmoments_Ltau_x_p;
    this.VdPmoments_Ltau_x_p=VdPmoments_Ltau_x_p;
    this.Dp_tmoments_Ltau_x_p=Dp_tmoments_Ltau_x_p;
    this.Dp_vmoments_Ltau_x_p=Dp_vmoments_Ltau_x_p;
    this.Da_tmoments_Ltau_x_p=Da_tmoments_Ltau_x_p;
    this.Da_vmoments_Ltau_x_p=Da_vmoments_Ltau_x_p;
    this.pmoments_Ltau_x_p=pmoments_Ltau_x_p;
    this.Tpmoments_Ltau_x_p=Tpmoments_Ltau_x_p;
    this.pluspmoments_Ltau_x_p=pluspmoments_Ltau_x_p;
    this.minuspmoments_Ltau_x_p=minuspmoments_Ltau_x_p;
    this.vbuymoments_Ltau_x_p=vbuymoments_Ltau_x_p;
    this.vsellmoments_Ltau_x_p=vsellmoments_Ltau_x_p;
    this.amoments_Ltau_x_p=amoments_Ltau_x_p;
    this.admoments_Ltau_x_p=admoments_Ltau_x_p;
    this.tsmoments_Ltau_x_p=tsmoments_Ltau_x_p;
    this.vsmoments_Ltau_x_p=vsmoments_Ltau_x_p;
    this.rtsmoments_Ltau_x_p=rtsmoments_Ltau_x_p;
    this.rvsmoments_Ltau_x_p=rvsmoments_Ltau_x_p;
    this.tauvbuymoments_Ltau_x_p=tauvbuymoments_Ltau_x_p;
    this.tauvsellmoments_Ltau_x_p=tauvsellmoments_Ltau_x_p;
    this.ntm=ntm;
    this.npm=npm;
    this.trade_last=trade_last;
    this.dp_diff_last=dp_diff_last;
}


public static QVMDataL calculateQVMDataL(final int ntm,
					 final int npm,
					 final long currenttime,
					 final long tau,
					 final Trade [] tb,
					 final int tsize,
					 final QVMDataL ref)
{
    final double atau=tau;
    final double p_offset=(tsize>0?tb[tsize-1].price:(ref!=null?ref.p_offset:1.0));

    final Trade bref,dp_diff_last_ref;
    if(ref!=null)
    {
	bref=ref.trade_last;
	dp_diff_last_ref=ref.dp_diff_last;
	if(ref.tau!=tau||ref.currenttime>currenttime||
	   ref.ntm!=ntm||ref.npm!=npm)
	{
	    throw new RuntimeException("Invalid ref; ref.tau="+ref.tau+" tau="+tau+
				       " ref.ntm="+ref.ntm+" ntm="+ntm+
				       " ref.npm="+ref.npm+" npm="+npm+
				       " ref.currenttime="+ref.currenttime+" currenttime="+currenttime);

	}
    }
    else
    {
	bref=dp_diff_last_ref=null;
    }

    final double []  // all 0 by default
	tmoments_Ltau_x_p=new double [ntm*npm],
	vmoments_Ltau_x_p=new double [ntm*npm],
	Tvmoments_Ltau_x_p=new double [ntm*npm],
	VdPmoments_Ltau_x_p=new double [ntm*npm],
	Dp_tmoments_Ltau_x_p=new double [ntm*npm],
	Dp_vmoments_Ltau_x_p=new double [ntm*npm],
	Da_tmoments_Ltau_x_p=new double [ntm*npm],
	Da_vmoments_Ltau_x_p=new double [ntm*npm],
	pmoments_Ltau_x_p=new double [ntm*npm],
	Tpmoments_Ltau_x_p=new double [ntm*npm],
	pluspmoments_Ltau_x_p=new double [ntm*npm],
	minuspmoments_Ltau_x_p=new double [ntm*npm],
	vbuymoments_Ltau_x_p=new double [ntm*npm],
	vsellmoments_Ltau_x_p=new double [ntm*npm],
	amoments_Ltau_x_p=new double [ntm*npm],
	admoments_Ltau_x_p=new double [ntm*npm],
	tsmoments_Ltau_x_p=new double [ntm*npm],
	vsmoments_Ltau_x_p=new double [ntm*npm],
	rtsmoments_Ltau_x_p=new double [ntm*npm],
	rvsmoments_Ltau_x_p=new double [ntm*npm],
	tauvbuymoments_Ltau_x_p=new double [ntm*npm],
	tauvsellmoments_Ltau_x_p=new double [ntm*npm];

    long vol=0,svol=0,prevt=currenttime,tinterval=currenttime;
    Trade dp_diff_last_next=dp_diff_last_ref,bref_next=bref;
    QVMData.ScanHelper beh=new QVMData.ScanHelper(null,tsize,null,tsize,true);
    for(int i=tsize;--i>=0;)
    {
	final Trade b=tb[i];
	if(b.time>prevt)
	{
	    throw new RuntimeException("The data is not sorted:"+prevt+" and "+b.time);
	}
	if(b==bref)
	{ // optimization
	    break;
	}
	prevt=b.time;
	vol+=b.shares;
	final double etau=(currenttime-prevt)/atau,ex=Math.exp(-etau),
	    dp=b.price-p_offset,bwmw=ex*b.shares,twmw=ex*((tinterval-prevt)/atau),dpprev;
	final Trade bprev;
	if(i>0) {bprev=tb[i-1];}
	else if(bref!=null) {bprev=bref;}
	else {bprev=b;}
	dpprev=bprev.price-p_offset;

	if(i<=beh.ind_be) beh=QVMData.ScanHelper.findTradeScanBackDifferentPrice(i,tb,bref,dp_diff_last_ref);
	final double dp_nonzero=b.price-beh.be.price,da_nonzero=Math.abs(dp_nonzero);
	if(i==(tsize-1)){bref_next=b;dp_diff_last_next=beh.be;}

	final int fplus=(b.price>bprev.price?1:(b.price<bprev.price?-1:0));
	final double awmw=ex*Math.abs(b.price-bprev.price),adwmw=ex*(QVMData.ScanHelper.isPriceNotTheSame(b.price,bprev.price)?1:0),
	    tauorig=b.origtime>0 && b.time>b.origtime ?(b.time-b.origtime)/atau:0;


	if(b.order!=null)
	{
	    if(b.buy_sell_indicator!=b.order.buy_sell_indicator) throw new RuntimeException("b="+b.buy_sell_indicator+" b.order="+b.order.buy_sell_indicator);
	    if(b.order.order!=null && (b.order.buy_sell_indicator==b.order.order.buy_sell_indicator))
		throw new RuntimeException("b.order="+b.order.buy_sell_indicator+" b.order.order.buy_sell_indicator|="+b.order.order.buy_sell_indicator);
	}

	final double Sv;
	if(b.buy_sell_indicator==BookItem.BOOK_BUY){
	    Sv=b.best_sell!=null?Math.abs(b.best_sell.price-b.price):0;
	    svol-=b.shares;
	}
	else if(b.buy_sell_indicator==BookItem.BOOK_SELL){
	    Sv=b.best_buy!=null?Math.abs(b.price-b.best_buy.price):0;
	    svol+=b.shares;
	}
	else {Sv=0;}

	double bte=0,ste=1;
	for(int k=0,j=0;k<ntm;++k,ste=((1-k)*bte+((2*k-1)-etau)*(bte=ste))/k) // Laguerre moments
	{
	    double dpe=1,dppreve=1;
	    for(int l=0;l<npm;l++,j++,dpe*=dp,dppreve*=dpprev) // Polynomial moments
	    {
		final double wdp=(dpe-dppreve)*(ex*ste),wpeb=(dpe*ste)*bwmw,wpet=(dpe*ste)*twmw;
		tmoments_Ltau_x_p[j]+=wpet;
		vmoments_Ltau_x_p[j]+=wpeb;
		Tvmoments_Ltau_x_p[j]+= -etau*wpeb;
		VdPmoments_Ltau_x_p[j]-=(vol*(dpe-dppreve))*(ex*ste);
		Dp_tmoments_Ltau_x_p[j]+=dp_nonzero*wpet;
		Dp_vmoments_Ltau_x_p[j]+=dp_nonzero*wpeb;
		Da_tmoments_Ltau_x_p[j]+=da_nonzero*wpet;
		Da_vmoments_Ltau_x_p[j]+=da_nonzero*wpeb;
		pmoments_Ltau_x_p[j]+=wdp;
		Tpmoments_Ltau_x_p[j]+= -etau*wdp;
		amoments_Ltau_x_p[j]+=(dpe*ste)*awmw;
		admoments_Ltau_x_p[j]+=(dpe*ste)*adwmw;
		if(false && (fplus==0)!=(adwmw==0)) throw new RuntimeException();
		if(fplus==1) pluspmoments_Ltau_x_p[j]+=wdp;
		else if(fplus==-1) minuspmoments_Ltau_x_p[j]+=wdp;
		if(b.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    vbuymoments_Ltau_x_p[j]+=wpeb;
		    tauvbuymoments_Ltau_x_p[j]+=wpeb*tauorig;
		    tsmoments_Ltau_x_p[j]-=wpet;
		    vsmoments_Ltau_x_p[j]-=wpeb;
		    rtsmoments_Ltau_x_p[j]-=Sv*wpet;
		    rvsmoments_Ltau_x_p[j]-=Sv*wpeb;
		}
		else if(b.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    vsellmoments_Ltau_x_p[j]+=wpeb;
		    tauvsellmoments_Ltau_x_p[j]+=wpeb*tauorig;
		    tsmoments_Ltau_x_p[j]+=wpet;
		    vsmoments_Ltau_x_p[j]+=wpeb;
		    rtsmoments_Ltau_x_p[j]+=Sv*wpet;
		    rvsmoments_Ltau_x_p[j]+=Sv*wpeb;
		}
	    }
	}
	tinterval=prevt;
    }

    if(bref!=null)
    { // optimization

	// add previous interval
	vol+=ref.vol;
	svol+=ref.svol;
	final double ref_delay=(currenttime-ref.currenttime)/atau,exp_ref_delay=Math.exp(-ref_delay),
	    dpa=ref.p_offset-p_offset,
	    bref_dp=bref.price-p_offset,bref_delay=(currenttime-bref.time)/atau,
	    expdt_bref_delay=Math.exp(-bref_delay)*((tinterval-ref.currenttime)/atau),
	    dp_nonzero=dp_diff_last_ref==null?0:bref.price-dp_diff_last_ref.price,da_nonzero=Math.abs(dp_nonzero);
	double bwtest=0,swtest=1;
	for(int k=0,j=0;k<ntm;++k,swtest=((1-k)*bwtest+((2*k-1)-bref_delay)*(bwtest=swtest))/k)
	{
	    double wpest=swtest;
	    for(int l=0;l<npm;l++,j++,wpest*=bref_dp)
	    {
		// add missed half slice
		final double wpet=wpest*expdt_bref_delay;
		tmoments_Ltau_x_p[j]+=wpet;
		Dp_tmoments_Ltau_x_p[j]+=dp_nonzero*wpet;
		Da_tmoments_Ltau_x_p[j]+=da_nonzero*wpet;
		if(bref.buy_sell_indicator==BookItem.BOOK_BUY){
		    final double Sv=bref.best_sell!=null?Math.abs(bref.best_sell.price-bref.price):0;
		    tsmoments_Ltau_x_p[j]-=wpet;
		    rtsmoments_Ltau_x_p[j]-=(Sv)*wpet;
		}
		else if(bref.buy_sell_indicator==BookItem.BOOK_SELL){
		    final double Sv=bref.best_buy!=null?Math.abs(bref.price-bref.best_buy.price):0;
		    tsmoments_Ltau_x_p[j]+=wpet;
		    rtsmoments_Ltau_x_p[j]+=(Sv)*wpet;
		}

		// Convert old interval to new (t_cur,p_offset) basis.
		// Laguerre x P1 polynomial group transform
		double st=0,sv=0,sTv=0,sVdP=0,Dp_st=0,Dp_sv=0,Da_st=0,Da_sv=0,sp=0,sTp=0,sa=0,sad=0,sts=0,svs=0,srts=0,srvs=0,stauvbuy=0,stauvsell=0,spplus=0,spminus=0,svbuy=0,svsell=0,d=1,bkste=0,skste=1;
		// bkst & skst : three term Laguerre recurrence.
		// d : Express L_n through L^1_n and L^1_{n-1}.
		for(int ks=k,oks=1;ks>=0;d=((oks-1)*(skste-bkste)-ref_delay*skste)/oks,skste=((1-oks)*bkste+((2*oks-1)-ref_delay)*(bkste=skste))/oks,--ks,++oks)
		{
		    double slt=0,slv=0,slTv=0,slVdP=0,Dp_slt=0,Dp_slv=0,Da_slt=0,Da_slv=0,slp=0,slTp=0,sla=0,slad=0,slts=0,slvs=0,slrts=0,slrvs=0,sltauvbuy=0,sltauvsell=0,slpplus=0,slpminus=0,slvbuy=0,slvsell=0;long fl=1;
		    // Newton binomial & Horner rule.
		    // The expansion would long overflow if l>=62.
		    for(int ls=l,o=ks*npm;ls>=0;fl=(fl*ls)/(l-(--ls)),++o)
		    {
			slt=dpa*slt+fl*ref.tmoments_Ltau_x_p[o];
			slv=dpa*slv+fl*ref.vmoments_Ltau_x_p[o];
			slTv=dpa*slTv+fl*(ref.Tvmoments_Ltau_x_p[o]-ref_delay*ref.vmoments_Ltau_x_p[o]);
			slVdP=dpa*slVdP+fl*ref.VdPmoments_Ltau_x_p[o];
			Dp_slt=dpa*Dp_slt+fl*ref.Dp_tmoments_Ltau_x_p[o];
			Dp_slv=dpa*Dp_slv+fl*ref.Dp_vmoments_Ltau_x_p[o];
			Da_slt=dpa*Da_slt+fl*ref.Da_tmoments_Ltau_x_p[o];
			Da_slv=dpa*Da_slv+fl*ref.Da_vmoments_Ltau_x_p[o];
			slp=dpa*slp+fl*ref.pmoments_Ltau_x_p[o];
			slTp=dpa*slTp+fl*(ref.Tpmoments_Ltau_x_p[o]-ref_delay*ref.pmoments_Ltau_x_p[o]);
			sla=dpa*sla+fl*ref.amoments_Ltau_x_p[o];
			slad=dpa*slad+fl*ref.admoments_Ltau_x_p[o];
			slts=dpa*slts+fl*ref.tsmoments_Ltau_x_p[o];
			slvs=dpa*slvs+fl*ref.vsmoments_Ltau_x_p[o];
			slrts=dpa*slrts+fl*ref.rtsmoments_Ltau_x_p[o];
			slrvs=dpa*slrvs+fl*ref.rvsmoments_Ltau_x_p[o];
			sltauvbuy=dpa*sltauvbuy+fl*ref.tauvbuymoments_Ltau_x_p[o];
			sltauvsell=dpa*sltauvsell+fl*ref.tauvsellmoments_Ltau_x_p[o];
			slpplus=dpa*slpplus+fl*ref.pluspmoments_Ltau_x_p[o];
			slpminus=dpa*slpminus+fl*ref.minuspmoments_Ltau_x_p[o];
			slvbuy=dpa*slvbuy+fl*ref.vbuymoments_Ltau_x_p[o];
			slvsell=dpa*slvsell+fl*ref.vsellmoments_Ltau_x_p[o];
		    }
		    st+=slt*d;
		    sv+=slv*d;
		    sTv+=slTv*d;
		    sVdP+=slVdP*d;
		    Dp_st+=Dp_slt*d;
		    Dp_sv+=Dp_slv*d;
		    Da_st+=Da_slt*d;
		    Da_sv+=Da_slv*d;
		    sp+=slp*d;
		    sTp+=slTp*d;
		    sa+=sla*d;
		    sad+=slad*d;
		    sts+=slts*d;
		    svs+=slvs*d;
		    srts+=slrts*d;
		    srvs+=slrvs*d;
		    stauvbuy+=sltauvbuy*d;
		    stauvsell+=sltauvsell*d;
		    spplus+=slpplus*d;
		    spminus+=slpminus*d;
		    svbuy+=slvbuy*d;
		    svsell+=slvsell*d;
		}
		tmoments_Ltau_x_p[j]+=exp_ref_delay*st;
		vmoments_Ltau_x_p[j]+=exp_ref_delay*sv;
		Tvmoments_Ltau_x_p[j]+=exp_ref_delay*sTv;
		VdPmoments_Ltau_x_p[j]+=exp_ref_delay*(sVdP+(ref.vol-vol)*sp);
		Dp_tmoments_Ltau_x_p[j]+=exp_ref_delay*Dp_st;
		Dp_vmoments_Ltau_x_p[j]+=exp_ref_delay*Dp_sv;
		Da_tmoments_Ltau_x_p[j]+=exp_ref_delay*Da_st;
		Da_vmoments_Ltau_x_p[j]+=exp_ref_delay*Da_sv;
		pmoments_Ltau_x_p[j]+=exp_ref_delay*sp;
		Tpmoments_Ltau_x_p[j]+=exp_ref_delay*sTp;
		amoments_Ltau_x_p[j]+=exp_ref_delay*sa;
		admoments_Ltau_x_p[j]+=exp_ref_delay*sad;
		tsmoments_Ltau_x_p[j]+=exp_ref_delay*sts;
		vsmoments_Ltau_x_p[j]+=exp_ref_delay*svs;
		rtsmoments_Ltau_x_p[j]+=exp_ref_delay*srts;
		rvsmoments_Ltau_x_p[j]+=exp_ref_delay*srvs;
		tauvbuymoments_Ltau_x_p[j]+=exp_ref_delay*stauvbuy;
		tauvsellmoments_Ltau_x_p[j]+=exp_ref_delay*stauvsell;
		pluspmoments_Ltau_x_p[j]+=exp_ref_delay*spplus;
		minuspmoments_Ltau_x_p[j]+=exp_ref_delay*spminus;
		vbuymoments_Ltau_x_p[j]+=exp_ref_delay*svbuy;
		vsellmoments_Ltau_x_p[j]+=exp_ref_delay*svsell;
	    }
	}
    } // end optimization

    return new QVMDataL(currenttime,
			tau,
			vol,
			svol,
			p_offset,
			tmoments_Ltau_x_p,
			vmoments_Ltau_x_p,
			Tvmoments_Ltau_x_p,
			VdPmoments_Ltau_x_p,
			Dp_tmoments_Ltau_x_p,
			Dp_vmoments_Ltau_x_p,
			Da_tmoments_Ltau_x_p,
			Da_vmoments_Ltau_x_p,
			pmoments_Ltau_x_p,
			Tpmoments_Ltau_x_p,
			pluspmoments_Ltau_x_p,
			minuspmoments_Ltau_x_p,
			vbuymoments_Ltau_x_p,
			vsellmoments_Ltau_x_p,
			amoments_Ltau_x_p,
			admoments_Ltau_x_p,
			tsmoments_Ltau_x_p,
			vsmoments_Ltau_x_p,
			rtsmoments_Ltau_x_p,
			rvsmoments_Ltau_x_p,
			tauvbuymoments_Ltau_x_p,
			tauvsellmoments_Ltau_x_p,
			ntm,
			npm,
			bref_next,
			dp_diff_last_next);
}


private static void checkMatching(final QVMDataL r,final QVMDataL ref,final double eps,final double epssum)
{
    if(r.currenttime!=ref.currenttime)
    {
	throw new RuntimeException("r.currenttime="+r.currenttime+
				   " ref.currenttime="+ref.currenttime);
    }
    if(r.tau!=ref.tau)
    {
	throw new RuntimeException("r.tau="+r.tau+" ref.tau="+ref.tau);
    }
    if(r.vol!=ref.vol)
    {
	throw new RuntimeException("r.vol="+r.vol+" ref.vol="+ref.vol);
    }
    if(r.svol!=ref.svol)
    {
	throw new RuntimeException("r.svol="+r.svol+" ref.svol="+ref.svol);
    }

    UnitTests.checkDoubleRel(r.p_offset,"r.p_offset",ref.p_offset,"ref.p_offset",eps);

    UnitTests.checkDoubleArrayRel(r.tmoments_Ltau_x_p,"r.tmoments_Ltau_x_p",
				  ref.tmoments_Ltau_x_p,"ref.tmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.vmoments_Ltau_x_p,"r.vmoments_Ltau_x_p",
				  ref.vmoments_Ltau_x_p,"ref.vmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Tvmoments_Ltau_x_p,"r.Tvmoments_Ltau_x_p",
				  ref.Tvmoments_Ltau_x_p,"ref.Tvmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.VdPmoments_Ltau_x_p,"r.VdPmoments_Ltau_x_p",
				  ref.VdPmoments_Ltau_x_p,"ref.VdPmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Dp_tmoments_Ltau_x_p,"r.Dp_tmoments_Ltau_x_p",
				  ref.Dp_tmoments_Ltau_x_p,"ref.Dp_tmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Dp_vmoments_Ltau_x_p,"r.Dp_vmoments_Ltau_x_p",
				  ref.Dp_vmoments_Ltau_x_p,"ref.Dp_vmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Da_tmoments_Ltau_x_p,"r.Da_tmoments_Ltau_x_p",
				  ref.Da_tmoments_Ltau_x_p,"ref.Da_tmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Da_vmoments_Ltau_x_p,"r.Da_vmoments_Ltau_x_p",
				  ref.Da_vmoments_Ltau_x_p,"ref.Da_vmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.pmoments_Ltau_x_p,"r.pmoments_Ltau_x_p",
				  ref.pmoments_Ltau_x_p,"ref.pmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Tpmoments_Ltau_x_p,"r.Tpmoments_Ltau_x_p",
				  ref.Tpmoments_Ltau_x_p,"ref.Tpmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.amoments_Ltau_x_p,"r.amoments_Ltau_x_p",
				  ref.amoments_Ltau_x_p,"ref.amoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.admoments_Ltau_x_p,"r.admoments_Ltau_x_p",
				  ref.admoments_Ltau_x_p,"ref.admoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.tsmoments_Ltau_x_p,"r.tsmoments_Ltau_x_p",
				  ref.tsmoments_Ltau_x_p,"ref.tsmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.vsmoments_Ltau_x_p,"r.vsmoments_Ltau_x_p",
				  ref.vsmoments_Ltau_x_p,"ref.vsmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.rtsmoments_Ltau_x_p,"r.rtsmoments_Ltau_x_p",
				  ref.rtsmoments_Ltau_x_p,"ref.rtsmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.rvsmoments_Ltau_x_p,"r.rvsmoments_Ltau_x_p",
				  ref.rvsmoments_Ltau_x_p,"ref.rvsmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.tauvbuymoments_Ltau_x_p,"r.tauvbuymoments_Ltau_x_p",
				  ref.tauvbuymoments_Ltau_x_p,"ref.tauvbuymoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.tauvsellmoments_Ltau_x_p,"r.tauvsellmoments_Ltau_x_p",
				  ref.tauvsellmoments_Ltau_x_p,"ref.tauvsellmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.pluspmoments_Ltau_x_p,"r.pluspmoments_Ltau_x_p",
				  ref.pluspmoments_Ltau_x_p,"ref.pluspmoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.minuspmoments_Ltau_x_p,"r.minuspmoments_Ltau_x_p",
				  ref.minuspmoments_Ltau_x_p,"ref.minuspmoments_Ltau_x_p",eps);

    final double [] pmom=TMatr.sumAcB(r.pluspmoments_Ltau_x_p,1,r.minuspmoments_Ltau_x_p);
    UnitTests.checkDoubleArrayRel1(r.pmoments_Ltau_x_p,"r.pmoments_Ltau_x_p",pmom,"pmom",epssum);

    UnitTests.checkDoubleArrayRel(r.vbuymoments_Ltau_x_p,"r.vbuymoments_Ltau_x_p",
				  ref.vbuymoments_Ltau_x_p,"ref.vbuymoments_Ltau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.vsellmoments_Ltau_x_p,"r.vsellmoments_Ltau_x_p",
				  ref.vsellmoments_Ltau_x_p,"ref.vsellmoments_Ltau_x_p",eps);
    final double [] vmom=TMatr.sumAcB(r.vbuymoments_Ltau_x_p,1,r.vsellmoments_Ltau_x_p);
    UnitTests.checkDoubleArrayRel(r.vmoments_Ltau_x_p,"r.vmoments_Ltau_x_p",vmom,"vmom",epssum);

    for(int j=1;j<Math.min(2,r.npm);j++)
    {
	final double [] sa=new double [r.ntm],spm=new double [r.ntm];
	for(int k=0,o=0;k<r.ntm;k++,o+=r.npm)
	{
	    spm[k]=r.pluspmoments_Ltau_x_p[o+j]-r.minuspmoments_Ltau_x_p[o+j];
	    sa[k]=r.amoments_Ltau_x_p[o+j-1];
	}
	UnitTests.checkDoubleArrayRel(spm,"spm",sa,"sa",epssum);
    }

    System.err.println("QVMDataL:checkMatching(): matching OK ntm="+r.ntm+" npm="+r.npm+" eps="+eps+" epssum="+epssum);
}


private static void matchWithTest(final String description,
				  final QVMDataL r,
				  final Trade [] tb,
				  final int tsize,
				  final double eps)
{
    System.err.println("QVMDataL:matchWithTest():======================"+description);

    final QVMData rt=QVMData.calculateQVMData(1,
					      r.ntm,
					      r.npm,
					      r.currenttime,
					      r.tau,
					      tb,
					      tsize,
					      null);

    checkMatchingQVMData(r,rt,eps);

    System.err.println("QVMDataL:matchWithTest(): OK eps="+eps);
}


private static void checkMomQVMData(final String txt,final QVMDataL r,final double [] r_mom,
				    final QVMData ref,final int QVMData_MOM,final double eps)
{
    if(r.currenttime!=ref.currenttime)
    {
	throw new RuntimeException(txt+":r.currenttime="+r.currenttime+
				   " ref.currenttime="+ref.currenttime);
    }
    if(r.tau!=ref.tau)
    {
	throw new RuntimeException(txt+":r.tau="+r.tau+" ref.tau="+ref.tau);
    }
    if(r.vol!=ref.vol)
    {
	throw new RuntimeException(txt+":r.vol="+r.vol+" ref.vol="+ref.vol);
    }
    if(r.svol!=ref.svol)
    {
	throw new RuntimeException(txt+":r.svol="+r.svol+" ref.svol="+ref.svol);
    }
    if(r.ntm!=ref.ntm||r.npm!=ref.npm)
    {
	throw new RuntimeException(txt+":r dims=("+r.ntm+":"+r.npm+")\n"+
				   "ref dims=("+ref.nvm+":"+ref.ntm+":"+ref.npm+")");
    }

    final double [][] abc_ref=QVMData.get3ArrayFromArrayForTesting(ref.shiftOffset(0,0,ref.p_offset-r.p_offset,QVMData_MOM),ref.nvm,ref.ntm,ref.npm)[0];
    final double [][] abc=QVMData.get3ArrayFromArrayForTesting(r_mom,1,r.ntm,r.npm)[0];

    for(int l=0;l<r.npm;l++)
    {
	double [] ra=null,rb=null;
	for(int k=0;k<r.ntm;k++)
	{
	    final double [] x=WIntegratorLaguerre.B.getNext(k,ra,rb,WIntegratorMonomials.B);rb=ra;ra=x;
	    double s=0;
	    for(int i=0,sign=1;i<=k;i++,sign=-sign)
	    {
		s+=(abc_ref[i][l]*sign)*x[i];
	    }
	    if(UnitTests.checkDoubleRelNoFail(s,abc[k][l],eps))
	    {
		throw new RuntimeException(txt+":Mismatch: abc_ref.s="+s+" abc["+k+"]["+l+"]="+abc[k][l]);
	    }
	}
    }
    System.err.println(txt+":QVMDataL:checkMomQVMData(): OK ntm="+r.ntm+" npm="+r.npm+" eps="+eps);
}

private static void checkMatchingQVMData(final QVMDataL r,final QVMData ref,final double eps)
{
    checkMomQVMData("tmoments_Ltau_x_p",r,r.tmoments_Ltau_x_p,ref,QVMData.MOM_dT,eps);
    checkMomQVMData("vmoments_Ltau_x_p",r,r.vmoments_Ltau_x_p,ref,QVMData.MOM_dV,eps);
    checkMomQVMData("Tvmoments_Ltau_x_p",r,r.Tvmoments_Ltau_x_p,ref,QVMData.MOM_TdV,eps);
    checkMomQVMData("VdPmoments_Ltau_x_p",r,r.VdPmoments_Ltau_x_p,ref,QVMData.MOM_VdPdt,eps);
    checkMomQVMData("Dp_tmoments_Ltau_x_p",r,r.Dp_tmoments_Ltau_x_p,ref,QVMData.MOM_DPT,eps);
    checkMomQVMData("Dp_vmoments_Ltau_x_p",r,r.Dp_vmoments_Ltau_x_p,ref,QVMData.MOM_DPV,eps);
    checkMomQVMData("Da_tmoments_Ltau_x_p",r,r.Da_tmoments_Ltau_x_p,ref,QVMData.MOM_DAT,eps);
    checkMomQVMData("Da_vmoments_Ltau_x_p",r,r.Da_vmoments_Ltau_x_p,ref,QVMData.MOM_DAV,eps);
    checkMomQVMData("vbuymoments_Ltau_x_p",r,r.vbuymoments_Ltau_x_p,ref,QVMData.MOM_VBUY,eps);
    checkMomQVMData("vsellmoments_Ltau_x_p",r,r.vsellmoments_Ltau_x_p,ref,QVMData.MOM_VSELL,eps);
    checkMomQVMData("pmoments_Ltau_x_p",r,r.pmoments_Ltau_x_p,ref,QVMData.MOM_DP,eps);
    checkMomQVMData("Tpmoments_Ltau_x_p",r,r.Tpmoments_Ltau_x_p,ref,QVMData.MOM_TDP,eps);
    checkMomQVMData("pluspmoments_Ltau_x_p",r,r.pluspmoments_Ltau_x_p,ref,QVMData.MOM_DPPLUS,eps);
    checkMomQVMData("minuspmoments_Ltau_x_p",r,r.minuspmoments_Ltau_x_p,ref,QVMData.MOM_DPMINUS,eps);
    checkMomQVMData("amoments_Ltau_x_p",r,r.amoments_Ltau_x_p,ref,QVMData.MOM_A,eps);
    checkMomQVMData("admoments_Ltau_x_p",r,r.admoments_Ltau_x_p,ref,QVMData.MOM_AD,eps);
    checkMomQVMData("tsmoments_Ltau_x_p",r,r.tsmoments_Ltau_x_p,ref,QVMData.MOM_TS,eps);
    checkMomQVMData("vsmoments_Ltau_x_p",r,r.vsmoments_Ltau_x_p,ref,QVMData.MOM_VS,eps);
    checkMomQVMData("rtsmoments_Ltau_x_p",r,r.rtsmoments_Ltau_x_p,ref,QVMData.MOM_RTS,eps);
    checkMomQVMData("rvsmoments_Ltau_x_p",r,r.rvsmoments_Ltau_x_p,ref,QVMData.MOM_RVS,eps);
    checkMomQVMData("tauvbuymoments_Ltau_x_p",r,r.tauvbuymoments_Ltau_x_p,ref,QVMData.MOM_TAUIBUY,eps);
    checkMomQVMData("tauvsellmoments_Ltau_x_p",r,r.tauvsellmoments_Ltau_x_p,ref,QVMData.MOM_TAUISELL,eps);

    System.err.println("QVMDataL:checkMatchingQVMData(): OK ntm="+r.ntm+" npm="+r.npm+" eps="+eps);
}



private static void checkCalculations(final int tsize,
				      final java.util.Random r,
				      final int ntm,
				      final int npm,
				      final int n_price_quants,
				      final double eps)
{
    final Trade [] tb=CalculateKendall.TradeEMoments.getRandomTradesQuantumPrice(tsize,r,n_price_quants);
    final long tau=10000L*tb.length/20,currenttime=tb[tb.length-1].time+20000;
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println("QVMDataL:checkCalculations(): test with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+" ntm="+ntm+" npm="+npm+
		       " n_price_quants="+n_price_quants+" eps="+eps);

    final QVMDataL ro=QVMDataL.calculateQVMDataL(ntm,npm,currenttime,tau,tb,tsize,null);
    matchWithTest("INITIAL CALCULATION",ro,tb,tsize,eps);

    // recurrent test
    final int n_split=8*(tb.length/10);
    final QVMDataL em_ref=QVMDataL.calculateQVMDataL(ntm,npm,(tb[n_split-1].time+tb[n_split].time)/2,tau,tb,n_split,null);
    matchWithTest("GET REFERENCE FOR RECURRENT",em_ref,tb,n_split,eps);

    final QVMDataL em_fromref=QVMDataL.calculateQVMDataL(ntm,npm,currenttime,tau,tb,tsize,em_ref);
    checkMatching(em_fromref,ro,eps,eps);
    matchWithTest("RECURRENT CALCULATIONS",em_fromref,tb,tsize,eps);

    // check with data cut
    final int ncut=tb.length-n_split;
    System.arraycopy(tb,n_split,tb,0,ncut);
    final QVMDataL emc_cut=QVMDataL.calculateQVMDataL(ntm,npm,currenttime,tau,tb,ncut,em_ref);
    checkMatching(ro,emc_cut,eps,eps);
    checkMatching(em_fromref,emc_cut,eps*0,eps); // have to match exactly
    System.err.println("QVMDataL:checkCalculations() data cut OK");

    System.err.println("OrdersAndTrades.USE_ORIGTIME="+OrdersAndTrades.USE_ORIGTIME+
		       "\nOrdersAndTrades.SAVE_NONDISPLAYABLE_TRADES="+OrdersAndTrades.SAVE_NONDISPLAYABLE_TRADES+
		       "\nOrdersAndTrades.SAVE_EDGE_ORDERS="+OrdersAndTrades.SAVE_EDGE_ORDERS+
		       "\nOrdersAndTrades.FLAG_PSEUDOTRADES="+OrdersAndTrades.FLAG_PSEUDOTRADES);
    System.err.println("QVMDataL:checkCalculations(): OK with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+" ntm="+ntm+" npm="+npm+
		       " n_price_quants="+n_price_quants+" eps="+eps);
}

static void test_Self(final long initrandom)
{
    final java.util.Random r=new java.util.Random(initrandom);
    final double eps=1e-5;

    for(int ntm=1;ntm<=7;ntm++)
    for(int npm=1;npm<=7;npm++)
    {
	for(int i=0;i<17;i++)
	for(int n_price_quants:new int[]{-1,2,3,4,5,10})
	{
	    checkCalculations(100+r.nextInt(100),r,ntm,npm,n_price_quants,eps);
	    checkCalculations(10010+r.nextInt(100010),r,ntm,npm,n_price_quants,eps);
	}
    }
}

public static void main(String [] args)
{
    System.err.println("QVMDataL::main(): starting tests; ver=$Id: QVMDataL.java,v 1.185 2022/09/14 08:19:26 mal Exp $");

    final long t0=System.currentTimeMillis();

    if(false){QTest.findInitRandom_NoFail(1,100,(final Long irandom) -> test_Self(irandom));}
    else {test_Self(5);}

    final long t1=System.currentTimeMillis();
    System.err.println("QVMDataL::main(): tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: QVMDataL.java,v 1.185 2022/09/14 08:19:26 mal Exp $");
}

static final double [][] Laguerre_POLS=new double [64][];
static {
    for(int k=0;k<Laguerre_POLS.length;k++)
    {
	Laguerre_POLS[k]=WIntegratorLaguerre.B.getNext(k,k<1?null:Laguerre_POLS[k-1],k<2?null:Laguerre_POLS[k-2],WIntegratorMonomials.B);
    }
}

} // END of class QVMDataL
