package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.WIntegratorLaguerre;

/** Extract the moments in Laguerre basis from {@link QVMDataL}.
 *  @see QVMDataL
 *  @see com.polytechnik.freemoney.CommonlyUsedMomentsLaguerre
 *  @version $Id: QVMDataLDirectAccess.java,v 1.55 2022/09/14 08:19:26 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QVMDataLDirectAccess implements DataInterfaceToMoments<QVMDataL> {

public void setM(final double [] m,final int n,final int mom,final QVMDataL odata,final int nmoms)
{
    if(n>=odata.npm||nmoms>odata.ntm) throw new IllegalArgumentException("n="+n+" odata.npm="+odata.npm+" nmoms="+nmoms+" odata.ntm="+odata.ntm);

    if(mom==M_O)
    {
	for(int k=0;k<nmoms;k++){m[k]=1;}
	return;
    }
    final double [] abc;
    if(mom==M_I) abc=odata.vmoments_Ltau_x_p;
    else if(mom==M_TI) abc=odata.Tvmoments_Ltau_x_p;
    else if(mom==M_ISELL) abc=odata.vsellmoments_Ltau_x_p;
    else if(mom==M_IBUY) abc=odata.vbuymoments_Ltau_x_p;
    else if(mom==M_DPPLUS) abc=odata.pluspmoments_Ltau_x_p;
    else if(mom==M_DPMINUS) abc=odata.minuspmoments_Ltau_x_p;
    else if(mom==M_DP) abc=odata.pmoments_Ltau_x_p;
    else if(mom==M_TDP) abc=odata.Tpmoments_Ltau_x_p;
    else if(mom==M_VdPdt) abc=odata.VdPmoments_Ltau_x_p;
    else if(mom==M_T) abc=odata.tmoments_Ltau_x_p;
    else if(mom==M_A) abc=odata.amoments_Ltau_x_p;
    else if(mom==M_AD) abc=odata.admoments_Ltau_x_p;
    else if(mom==M_TS) abc=odata.tsmoments_Ltau_x_p;
    else if(mom==M_VS) abc=odata.vsmoments_Ltau_x_p;
    else if(mom==M_RTS) abc=odata.rtsmoments_Ltau_x_p;
    else if(mom==M_RVS) abc=odata.rvsmoments_Ltau_x_p;
    else if(mom==M_TAUIBUY) abc=odata.tauvbuymoments_Ltau_x_p;
    else if(mom==M_TAUISELL) abc=odata.tauvsellmoments_Ltau_x_p;
    else if(mom==M_DPwV) abc=odata.Dp_vmoments_Ltau_x_p;
    else if(mom==M_DPwT) abc=odata.Dp_tmoments_Ltau_x_p;
    else if(mom==M_DAwV) abc=odata.Da_vmoments_Ltau_x_p;
    else if(mom==M_DAwT) abc=odata.Da_tmoments_Ltau_x_p;
    else throw new RuntimeException("mom="+mom);
    for(int k=0,o=n;k<nmoms;k++,o+=odata.npm){m[k]=abc[o];}
}

public void setMT(final double [] m,final int nmoms){java.util.Arrays.fill(m,0,nmoms,0);m[0]=1;}

public int setFMoments(final double [] Vmoments,final double [] dVmoments,final int n)
{
    for(int k=n;--k>=0;)
    {
	final double [] a=new double [k+1];
	// "-" from integration by parts is compensated
	// by set_getLaguerreExpFIntegrated definition.
	// one extra "-" because time is inverted: "now" is at x=0, past at x=infinity
	a[k]=-1;
	Vmoments[k]=TMatr.mulVVn1(WIntegratorLaguerre.getPolynomialWIntegratedDt(a),dVmoments);
    }
    return n;
}
public int setDFMoments(final double [] dFmoments,final double [] Fmoments,final int n)
{
    final double [] dF=WIntegratorLaguerre.B.getL1FromL0(java.util.Arrays.copyOf(Fmoments,n));
    System.arraycopy(dF,0,dFmoments,0,n);
    return n;
}

public double getPOffset(final QVMDataL odata){return odata.p_offset;}
public double getPLast(final QVMDataL odata){return (odata.trade_last!=null?odata.trade_last.price:Double.NaN)-getPOffset(odata);}
public long getTLast(final QVMDataL odata){return odata.currenttime;}
public long getTau(final QVMDataL odata){return odata.tau;}
public long getTotalVolume(final QVMDataL odata){return odata.vol;}

final java.util.function.Function<double [],double []> getFMomentsFromDFMomentsMatchRadau=(dFmoms)->{
	final double [] Fmoms=new double [dFmoms.length-1];
	setFMoments(Fmoms,dFmoms,Fmoms.length);
	TMatr.scaleVectorInPlace(Fmoms,-1); // to match sign
	return Fmoms;
    };
}
