package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.WIntegratorLegendreShifted;
import com.polytechnik.freemoney.WIntegratorMonomials;

/** Moments data in shifted to [0:1] Legendre basis, use {@link QVMDataPDirectAccess} to extract.
 *  @see QVMDataPDirectAccess
 *  @see com.polytechnik.freemoney.CommonlyUsedMomentsLegendreShifted
 *  @version $Id: QVMDataP.java,v 1.234 2022/09/14 08:19:26 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QVMDataP {

final long currenttime,tau,vol,svol;
final double p_offset;

final int ntm,npm;

final double [] // here offset=p_offset and array dims are: ntm,npm. P_k - shifted to [0,1] Legendge polynomials.
    tmoments_Ptau_x_p, // [k][l]=sum P_k(exp(-(t_cur-t)/tau)) (p-offset)^l exp(-(t_cur-t)/tau) dt_i
    vmoments_Ptau_x_p, // [k][l]=sum P_k(exp(-(t_cur-t)/tau)) (p-offset)^l exp(-(t_cur-t)/tau) dv_i
    Tvmoments_Ptau_x_p, // [k][l]=sum ((t-t_cur)/tau)* P_k(exp(-(t_cur-t)/tau)) (p-offset)^l exp(-(t_cur-t)/tau) dv_i
    VdPmoments_Ptau_x_p,
    Dp_tmoments_Ptau_x_p,
    Dp_vmoments_Ptau_x_p,
    Da_tmoments_Ptau_x_p,
    Da_vmoments_Ptau_x_p,
    vbuymoments_Ptau_x_p,
    vsellmoments_Ptau_x_p,
    pmoments_Ptau_x_p,
    Tpmoments_Ptau_x_p,
    pluspmoments_Ptau_x_p,
    minuspmoments_Ptau_x_p,
    amoments_Ptau_x_p,
    admoments_Ptau_x_p,
    tsmoments_Ptau_x_p,
    vsmoments_Ptau_x_p,
    rtsmoments_Ptau_x_p,
    rvsmoments_Ptau_x_p,
    tauvbuymoments_Ptau_x_p,
    tauvsellmoments_Ptau_x_p
    ;

// references to previos calculations
final Trade trade_last;
final Trade dp_diff_last;


private QVMDataP(final long currenttime,
		 final long tau,
		 final long vol,
		 final long svol,
		 final double p_offset,

		 final double [] tmoments_Ptau_x_p,
		 final double [] vmoments_Ptau_x_p,
		 final double [] Tvmoments_Ptau_x_p,
		 final double [] VdPmoments_Ptau_x_p,
		 final double [] Dp_tmoments_Ptau_x_p,
		 final double [] Dp_vmoments_Ptau_x_p,
		 final double [] Da_tmoments_Ptau_x_p,
		 final double [] Da_vmoments_Ptau_x_p,
		 final double [] pmoments_Ptau_x_p,
		 final double [] Tpmoments_Ptau_x_p,
		 final double [] pluspmoments_Ptau_x_p,
		 final double [] minuspmoments_Ptau_x_p,
		 final double [] vbuymoments_Ptau_x_p,
		 final double [] vsellmoments_Ptau_x_p,
		 final double [] amoments_Ptau_x_p,
		 final double [] admoments_Ptau_x_p,
		 final double [] tsmoments_Ptau_x_p,
		 final double [] vsmoments_Ptau_x_p,
		 final double [] rtsmoments_Ptau_x_p,
		 final double [] rvsmoments_Ptau_x_p,
		 final double [] tauvbuymoments_Ptau_x_p,
		 final double [] tauvsellmoments_Ptau_x_p,
		 final int ntm,
		 final int npm,

		 // references
		 final Trade trade_last,
		 final Trade dp_diff_last)
{
    this.currenttime=currenttime;
    this.tau=tau;
    this.vol=vol;
    this.svol=svol;
    this.p_offset=p_offset;
    this.tmoments_Ptau_x_p=tmoments_Ptau_x_p;
    this.vmoments_Ptau_x_p=vmoments_Ptau_x_p;
    this.Tvmoments_Ptau_x_p=Tvmoments_Ptau_x_p;
    this.VdPmoments_Ptau_x_p=VdPmoments_Ptau_x_p;
    this.Dp_tmoments_Ptau_x_p=Dp_tmoments_Ptau_x_p;
    this.Dp_vmoments_Ptau_x_p=Dp_vmoments_Ptau_x_p;
    this.Da_tmoments_Ptau_x_p=Da_tmoments_Ptau_x_p;
    this.Da_vmoments_Ptau_x_p=Da_vmoments_Ptau_x_p;
    this.pmoments_Ptau_x_p=pmoments_Ptau_x_p;
    this.Tpmoments_Ptau_x_p=Tpmoments_Ptau_x_p;
    this.pluspmoments_Ptau_x_p=pluspmoments_Ptau_x_p;
    this.minuspmoments_Ptau_x_p=minuspmoments_Ptau_x_p;
    this.vbuymoments_Ptau_x_p=vbuymoments_Ptau_x_p;
    this.vsellmoments_Ptau_x_p=vsellmoments_Ptau_x_p;
    this.amoments_Ptau_x_p=amoments_Ptau_x_p;
    this.admoments_Ptau_x_p=admoments_Ptau_x_p;
    this.tsmoments_Ptau_x_p=tsmoments_Ptau_x_p;
    this.vsmoments_Ptau_x_p=vsmoments_Ptau_x_p;
    this.rtsmoments_Ptau_x_p=rtsmoments_Ptau_x_p;
    this.rvsmoments_Ptau_x_p=rvsmoments_Ptau_x_p;
    this.tauvbuymoments_Ptau_x_p=tauvbuymoments_Ptau_x_p;
    this.tauvsellmoments_Ptau_x_p=tauvsellmoments_Ptau_x_p;
    this.ntm=ntm;
    this.npm=npm;
    this.trade_last=trade_last;
    this.dp_diff_last=dp_diff_last;
}


public static QVMDataP calculateQVMDataP(final int ntm,
					 final int npm,
					 final long currenttime,
					 final long tau,
					 final Trade [] tb,
					 final int tsize,
					 final QVMDataP ref)
{
    final double atau=tau;
    final double p_offset=(tsize>0?tb[tsize-1].price:(ref!=null?ref.p_offset:1.0));

    final Trade bref,dp_diff_last_ref;
    if(ref!=null)
    {
	bref=ref.trade_last;
	dp_diff_last_ref=ref.dp_diff_last;
	if(ref.tau!=tau||ref.currenttime>currenttime||
	   ref.ntm!=ntm||ref.npm!=npm)
	{
	    throw new RuntimeException("Invalid ref; ref.tau="+ref.tau+" tau="+tau+
				       " ref.ntm="+ref.ntm+" ntm="+ntm+
				       " ref.npm="+ref.npm+" npm="+npm+
				       " ref.currenttime="+ref.currenttime+" currenttime="+currenttime);

	}
    }
    else
    {
	bref=dp_diff_last_ref=null;
    }

    final double []  // all 0 by default
	tmoments_Ptau_x_p=new double [ntm*npm],
	vmoments_Ptau_x_p=new double [ntm*npm],
	Tvmoments_Ptau_x_p=new double [ntm*npm],
	VdPmoments_Ptau_x_p=new double [ntm*npm],
	Dp_tmoments_Ptau_x_p=new double [ntm*npm],
	Dp_vmoments_Ptau_x_p=new double [ntm*npm],
	Da_tmoments_Ptau_x_p=new double [ntm*npm],
	Da_vmoments_Ptau_x_p=new double [ntm*npm],
	pmoments_Ptau_x_p=new double [ntm*npm],
	Tpmoments_Ptau_x_p=new double [ntm*npm],
	pluspmoments_Ptau_x_p=new double [ntm*npm],
	minuspmoments_Ptau_x_p=new double [ntm*npm],
	vbuymoments_Ptau_x_p=new double [ntm*npm],
	vsellmoments_Ptau_x_p=new double [ntm*npm],
	amoments_Ptau_x_p=new double [ntm*npm],
	admoments_Ptau_x_p=new double [ntm*npm],
	tsmoments_Ptau_x_p=new double [ntm*npm],
	vsmoments_Ptau_x_p=new double [ntm*npm],
	rtsmoments_Ptau_x_p=new double [ntm*npm],
	rvsmoments_Ptau_x_p=new double [ntm*npm],
	tauvbuymoments_Ptau_x_p=new double [ntm*npm],
	tauvsellmoments_Ptau_x_p=new double [ntm*npm];

    long vol=0,svol=0,prevt=currenttime,tinterval=currenttime;
    Trade dp_diff_last_next=dp_diff_last_ref,bref_next=bref;
    QVMData.ScanHelper beh=new QVMData.ScanHelper(null,tsize,null,tsize,true);
    for(int i=tsize;--i>=0;)
    {
	final Trade b=tb[i];
	if(b.time>prevt)
	{
	    throw new RuntimeException("The data is not sorted:"+prevt+" and "+b.time);
	}
	if(b==bref)
	{ // optimization
	    break;
	}
	prevt=b.time;
	vol+=b.shares;
	final double Td= -(currenttime-prevt)/atau,ex=Math.exp(Td),xxex=2*ex-1,
	    dp=b.price-p_offset,bwmw=ex*b.shares,twmw=ex*((tinterval-prevt)/atau),dpprev;
	final Trade bprev;
	if(i>0) {bprev=tb[i-1];}
	else if(bref!=null) {bprev=bref;}
	else {bprev=b;}
	dpprev=bprev.price-p_offset;

	if(i<=beh.ind_be) beh=QVMData.ScanHelper.findTradeScanBackDifferentPrice(i,tb,bref,dp_diff_last_ref);
	final double dp_nonzero=b.price-beh.be.price,da_nonzero=Math.abs(dp_nonzero);
	if(i==(tsize-1)){bref_next=b;dp_diff_last_next=beh.be;}

	final int fplus=(b.price>bprev.price?1:(b.price<bprev.price?-1:0));
	final double awmw=ex*Math.abs(b.price-bprev.price),adwmw=ex*(QVMData.ScanHelper.isPriceNotTheSame(b.price,bprev.price)?1:0),
	    tauorig=b.origtime>0 && b.time>b.origtime ?(b.time-b.origtime)/atau:0;

	if(b.order!=null)
	{
	    if(b.buy_sell_indicator!=b.order.buy_sell_indicator) throw new RuntimeException("b="+b.buy_sell_indicator+" b.order="+b.order.buy_sell_indicator);
	    if(b.order.order!=null && (b.order.buy_sell_indicator==b.order.order.buy_sell_indicator))
		throw new RuntimeException("b.order="+b.order.buy_sell_indicator+" b.order.order.buy_sell_indicator!="+b.order.order.buy_sell_indicator);
	}
	final double Sv;
	if(b.buy_sell_indicator==BookItem.BOOK_BUY){
	    Sv=b.best_sell!=null?Math.abs(b.best_sell.price-b.price):0;
	    svol-=b.shares;
	}
	else if(b.buy_sell_indicator==BookItem.BOOK_SELL){
	    Sv=b.best_buy!=null?Math.abs(b.price-b.best_buy.price):0;
	    svol+=b.shares;
	}
	else {Sv=0;}

	double bte=0,ste=1;
	for(int k=0,j=0;k<ntm;++k,ste=((1-k)*bte+(2*k-1)*(bte=ste)*xxex)/k) // Legendre moments
	{
	    //System.err.println("ste="+ste+" legendreCalculate("+k+","+ex+")="+LegendreShifted.legendreCalculate(k,ex));
	    double dpe=1,dppreve=1;
	    for(int l=0;l<npm;l++,j++,dpe*=dp,dppreve*=dpprev) // Polynomial moments
	    {
		final double wdp=(dpe-dppreve)*(ex*ste),wpeb=(dpe*ste)*bwmw,wpet=(dpe*ste)*twmw;
		tmoments_Ptau_x_p[j]+=wpet;
		vmoments_Ptau_x_p[j]+=wpeb;
		Tvmoments_Ptau_x_p[j]+=Td*wpeb;
		VdPmoments_Ptau_x_p[j]-=(vol*(dpe-dppreve))*(ex*ste);
		Dp_tmoments_Ptau_x_p[j]+=dp_nonzero*wpet;
		Dp_vmoments_Ptau_x_p[j]+=dp_nonzero*wpeb;
		Da_tmoments_Ptau_x_p[j]+=da_nonzero*wpet;
		Da_vmoments_Ptau_x_p[j]+=da_nonzero*wpeb;
		pmoments_Ptau_x_p[j]+=wdp;
		Tpmoments_Ptau_x_p[j]+=Td*wdp;
		amoments_Ptau_x_p[j]+=(dpe*ste)*awmw;
		admoments_Ptau_x_p[j]+=(dpe*ste)*adwmw;
		if(fplus==1) pluspmoments_Ptau_x_p[j]+=wdp;
		else if(fplus==-1) minuspmoments_Ptau_x_p[j]+=wdp;
		if(b.buy_sell_indicator==BookItem.BOOK_BUY)
		{
		    vbuymoments_Ptau_x_p[j]+=wpeb;
		    tauvbuymoments_Ptau_x_p[j]+=tauorig*wpeb;
		    tsmoments_Ptau_x_p[j]-=wpet;
		    vsmoments_Ptau_x_p[j]-=wpeb;
		    rtsmoments_Ptau_x_p[j]-=Sv*wpet;
		    rvsmoments_Ptau_x_p[j]-=Sv*wpeb;
		}
		else if(b.buy_sell_indicator==BookItem.BOOK_SELL)
		{
		    vsellmoments_Ptau_x_p[j]+=wpeb;
		    tauvsellmoments_Ptau_x_p[j]+=tauorig*wpeb;
		    tsmoments_Ptau_x_p[j]+=wpet;
		    vsmoments_Ptau_x_p[j]+=wpeb;
		    rtsmoments_Ptau_x_p[j]+=Sv*wpet;
		    rvsmoments_Ptau_x_p[j]+=Sv*wpeb;
		}
	    }
	}
	tinterval=prevt;
    }

    if(bref!=null)
    { // optimization
	// temporary arrays
	final double [] b=new double [ntm],quot=new double [ntm];


	// add previous interval
	vol+=ref.vol;
	svol+=ref.svol;
	final double Td= -(currenttime-ref.currenttime)/atau,exp_ref_delay=Math.exp(Td),
	    dpa=ref.p_offset-p_offset,
	    bref_dp=bref.price-p_offset,exp_bref_delay=Math.exp(-(currenttime-bref.time)/atau),bref_xxex=2*exp_bref_delay-1,
	    dp_nonzero=dp_diff_last_ref==null?0:bref.price-dp_diff_last_ref.price,da_nonzero=Math.abs(dp_nonzero);
	double b_brefP=0,s_brefP=1;
	for(int k=0,j=0;k<ntm;++k,s_brefP=((1-k)*b_brefP+(2*k-1)*(b_brefP=s_brefP)*bref_xxex)/k)
	{
	    if(false) // Calculate conversion coefficients to new t_cur and save them to b.
	    {
		if(ntm>LegendreShifted_POLS.length) throw new IllegalArgumentException("ntm="+ntm+" max="+LegendreShifted_POLS.length);

		// I have no time to program exact transition coefficients b_k,
		// for shifted [0,1] Legendre polynomials P_n,
		// where b_k (the polynomials of a) are defined as: P_n(ax)=sum_{k=0}^{k=n}b_k(a) P_k(x)
		// see Prudnikov 2.7.15 formulae 1.
		// Do it here directly, using synthetic division,
		// but this approach can be numerically unstable.
		// Can one obtain 3 term recurrence for b_k using 3 term recurrence for P_k?
		final double [] pk=LegendreShifted_POLS[k]; // The polynomial matrix is cached for speed.
		double mul=1;
		for(int kk=0;kk<=k;kk++,mul*=exp_ref_delay){b[kk]=pk[kk]*mul;}
		System.arraycopy(TMatr.expandOverLowerDiagBasis(java.util.Arrays.copyOf(b,k+1),LegendreShifted_POLS),0,b,0,k+1);
		//System.err.println("QVMDataP: b expansion");
	    }
	    else if(false)
	    { // Alternative recurrence. Still may be unstable.
		double mul=1;
		for(int kk=0;kk<=k;kk++,mul*=exp_ref_delay){b[kk]=mul;}
		for(int kk=k;kk>=0;kk--)
		{
		    double s=0;mul=1;
		    for(int l=kk;++l<=k;){s+=b[l]*(mul*=(l+kk)/((double)(kk-l)));}
		    b[kk]=b[kk]*mul-s;
		}
	    }
	    else if(true)
	    { // 3 term recurrence applied directly to arrays
		WIntegratorLegendreShifted.B.setNewtonBinomialLikeCoefs(k,b,quot,exp_ref_delay,0);
	    }
	    else if(true)
	    { // Trying to get stable recurrence.
		for(int kk=0;kk<=k;kk++) {b[kk]=quot[kk]=0;}
		quot[k]=1; // start with P_k
		for(int kk=0;kk<=k;kk++)
		{
		    double s=0;final double fmul=(exp_ref_delay-1)/(kk+1);
		    for(int l=k;l>=0;--l)
		    {
			final double x;
			b[l]+=(x=quot[l]);
			quot[l]=(s*(2*l+1)-(kk-l)*x)*fmul;
			s+=x;
		    }
		}
	    }

	    double wdp_bref=s_brefP;
	    for(int l=0;l<npm;l++,j++,wdp_bref*=bref_dp)
	    {
		// add missed half slice
		final double wpet=wdp_bref*exp_bref_delay*((tinterval-ref.currenttime)/atau);
		tmoments_Ptau_x_p[j]+=wpet;
		Dp_tmoments_Ptau_x_p[j]+=dp_nonzero*wpet;
		Da_tmoments_Ptau_x_p[j]+=da_nonzero*wpet;
		if(bref.buy_sell_indicator==BookItem.BOOK_BUY){
		    final double Sv=bref.best_sell!=null?Math.abs(bref.best_sell.price-bref.price):0;
		    tsmoments_Ptau_x_p[j]-=wpet;
		    rtsmoments_Ptau_x_p[j]-=(Sv)*wpet;
		}
		else if(bref.buy_sell_indicator==BookItem.BOOK_SELL){
		    final double Sv=bref.best_buy!=null?Math.abs(bref.price-bref.best_buy.price):0;
		    tsmoments_Ptau_x_p[j]+=wpet;
		    rtsmoments_Ptau_x_p[j]+=(Sv)*wpet;
		}

		// Convert old interval to new (t_cur,p_offset) basis.
		// Legendre x P1 polynomial group transform
		double st=0,sv=0,sTv=0,sVdP=0,Dp_st=0,Dp_sv=0,Da_st=0,Da_sv=0,sp=0,sTp=0,sa=0,sad=0,sts=0,svs=0,srts=0,srvs=0,stauvbuy=0,stauvsell=0,spplus=0,spminus=0,svbuy=0,svsell=0;
		for(int ks=k;ks>=0;--ks)
		{
		    double slt=0,slv=0,slTv=0,slVdP=0,Dp_slt=0,Dp_slv=0,Da_slt=0,Da_slv=0,slp=0,slTp=0,sla=0,slad=0,slts=0,slvs=0,slrts=0,slrvs=0,sltauvbuy=0,sltauvsell=0,slpplus=0,slpminus=0,slvbuy=0,slvsell=0;long fl=1;
		    // Newton binomial & Horner rule.
		    // The expansion would long overflow if l>=62.
		    for(int ls=l,o=ks*npm;ls>=0;fl=(fl*ls)/(l-(--ls)),++o)
		    {
			slt=dpa*slt+fl*ref.tmoments_Ptau_x_p[o];
			slv=dpa*slv+fl*ref.vmoments_Ptau_x_p[o];
			slTv=dpa*slTv+fl*(ref.Tvmoments_Ptau_x_p[o]+Td*ref.vmoments_Ptau_x_p[o]);
			slVdP=dpa*slVdP+fl*ref.VdPmoments_Ptau_x_p[o];
			Dp_slt=dpa*Dp_slt+fl*ref.Dp_tmoments_Ptau_x_p[o];
			Dp_slv=dpa*Dp_slv+fl*ref.Dp_vmoments_Ptau_x_p[o];
			Da_slt=dpa*Da_slt+fl*ref.Da_tmoments_Ptau_x_p[o];
			Da_slv=dpa*Da_slv+fl*ref.Da_vmoments_Ptau_x_p[o];
			slp=dpa*slp+fl*ref.pmoments_Ptau_x_p[o];
			slTp=dpa*slTp+fl*(ref.Tpmoments_Ptau_x_p[o]+Td*ref.pmoments_Ptau_x_p[o]);
			sla=dpa*sla+fl*ref.amoments_Ptau_x_p[o];
			slad=dpa*slad+fl*ref.admoments_Ptau_x_p[o];
			slts=dpa*slts+fl*ref.tsmoments_Ptau_x_p[o];
			slvs=dpa*slvs+fl*ref.vsmoments_Ptau_x_p[o];
			slrts=dpa*slrts+fl*ref.rtsmoments_Ptau_x_p[o];
			slrvs=dpa*slrvs+fl*ref.rvsmoments_Ptau_x_p[o];
			sltauvbuy=dpa*sltauvbuy+fl*ref.tauvbuymoments_Ptau_x_p[o];
			sltauvsell=dpa*sltauvsell+fl*ref.tauvsellmoments_Ptau_x_p[o];
			slpplus=dpa*slpplus+fl*ref.pluspmoments_Ptau_x_p[o];
			slpminus=dpa*slpminus+fl*ref.minuspmoments_Ptau_x_p[o];
			slvbuy=dpa*slvbuy+fl*ref.vbuymoments_Ptau_x_p[o];
			slvsell=dpa*slvsell+fl*ref.vsellmoments_Ptau_x_p[o];
		    }
		    final double d=b[ks];
		    st+=slt*d;
		    sv+=slv*d;
		    sTv+=slTv*d;
		    sVdP+=slVdP*d;
		    Dp_st+=Dp_slt*d;
		    Dp_sv+=Dp_slv*d;
		    Da_st+=Da_slt*d;
		    Da_sv+=Da_slv*d;
		    sp+=slp*d;
		    sTp+=slTp*d;
		    sa+=sla*d;
		    sad+=slad*d;
		    sts+=slts*d;
		    svs+=slvs*d;
		    srts+=slrts*d;
		    srvs+=slrvs*d;
		    stauvbuy+=sltauvbuy*d;
		    stauvsell+=sltauvsell*d;
		    spplus+=slpplus*d;
		    spminus+=slpminus*d;
		    svbuy+=slvbuy*d;
		    svsell+=slvsell*d;
		}
		tmoments_Ptau_x_p[j]+=exp_ref_delay*st;
		vmoments_Ptau_x_p[j]+=exp_ref_delay*sv;
		Tvmoments_Ptau_x_p[j]+=exp_ref_delay*sTv;
		VdPmoments_Ptau_x_p[j]+=exp_ref_delay*(sVdP+(ref.vol-vol)*sp);
		Dp_tmoments_Ptau_x_p[j]+=exp_ref_delay*Dp_st;
		Dp_vmoments_Ptau_x_p[j]+=exp_ref_delay*Dp_sv;
		Da_tmoments_Ptau_x_p[j]+=exp_ref_delay*Da_st;
		Da_vmoments_Ptau_x_p[j]+=exp_ref_delay*Da_sv;
		pmoments_Ptau_x_p[j]+=exp_ref_delay*sp;
		Tpmoments_Ptau_x_p[j]+=exp_ref_delay*sTp;
		amoments_Ptau_x_p[j]+=exp_ref_delay*sa;
		admoments_Ptau_x_p[j]+=exp_ref_delay*sad;
		tsmoments_Ptau_x_p[j]+=exp_ref_delay*sts;
		vsmoments_Ptau_x_p[j]+=exp_ref_delay*svs;
		rtsmoments_Ptau_x_p[j]+=exp_ref_delay*srts;
		rvsmoments_Ptau_x_p[j]+=exp_ref_delay*srvs;
		tauvbuymoments_Ptau_x_p[j]+=exp_ref_delay*stauvbuy;
		tauvsellmoments_Ptau_x_p[j]+=exp_ref_delay*stauvsell;
		pluspmoments_Ptau_x_p[j]+=exp_ref_delay*spplus;
		minuspmoments_Ptau_x_p[j]+=exp_ref_delay*spminus;
		vbuymoments_Ptau_x_p[j]+=exp_ref_delay*svbuy;
		vsellmoments_Ptau_x_p[j]+=exp_ref_delay*svsell;
	    }
	}
    } // end optimization

    return new QVMDataP(currenttime,
			tau,
			vol,
			svol,
			p_offset,
			tmoments_Ptau_x_p,
			vmoments_Ptau_x_p,
			Tvmoments_Ptau_x_p,
			VdPmoments_Ptau_x_p,
			Dp_tmoments_Ptau_x_p,
			Dp_vmoments_Ptau_x_p,
			Da_tmoments_Ptau_x_p,
			Da_vmoments_Ptau_x_p,
			pmoments_Ptau_x_p,
			Tpmoments_Ptau_x_p,
			pluspmoments_Ptau_x_p,
			minuspmoments_Ptau_x_p,
			vbuymoments_Ptau_x_p,
			vsellmoments_Ptau_x_p,
			amoments_Ptau_x_p,
			admoments_Ptau_x_p,
			tsmoments_Ptau_x_p,
			vsmoments_Ptau_x_p,
			rtsmoments_Ptau_x_p,
			rvsmoments_Ptau_x_p,
			tauvbuymoments_Ptau_x_p,
			tauvsellmoments_Ptau_x_p,
			ntm,
			npm,
			bref_next,
			dp_diff_last_next);
}



private static void checkMatching(final QVMDataP r,final QVMDataP ref,final double eps,final double epssum)
{
    if(r.currenttime!=ref.currenttime)
    {
	throw new RuntimeException("r.currenttime="+r.currenttime+
				   " ref.currenttime="+ref.currenttime);
    }
    if(r.tau!=ref.tau)
    {
	throw new RuntimeException("r.tau="+r.tau+" ref.tau="+ref.tau);
    }
    if(r.vol!=ref.vol)
    {
	throw new RuntimeException("r.vol="+r.vol+" ref.vol="+ref.vol);
    }
    if(r.svol!=ref.svol)
    {
	throw new RuntimeException("r.svol="+r.svol+" ref.svol="+ref.svol);
    }

    UnitTests.checkDoubleRel(r.p_offset,"r.p_offset",ref.p_offset,"ref.p_offset",eps);

    UnitTests.checkDoubleArrayRel(r.tmoments_Ptau_x_p,"r.tmoments_Ptau_x_p",
				  ref.tmoments_Ptau_x_p,"ref.tmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.vmoments_Ptau_x_p,"r.vmoments_Ptau_x_p",
				  ref.vmoments_Ptau_x_p,"ref.vmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Tvmoments_Ptau_x_p,"r.Tvmoments_Ptau_x_p",
				  ref.Tvmoments_Ptau_x_p,"ref.Tvmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.VdPmoments_Ptau_x_p,"r.VdPmoments_Ptau_x_p",
				  ref.VdPmoments_Ptau_x_p,"ref.VdPmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Dp_tmoments_Ptau_x_p,"r.Dp_tmoments_Ptau_x_p",
				  ref.Dp_tmoments_Ptau_x_p,"ref.Dp_tmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Dp_vmoments_Ptau_x_p,"r.Dp_vmoments_Ptau_x_p",
				  ref.Dp_vmoments_Ptau_x_p,"ref.Dp_vmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Da_tmoments_Ptau_x_p,"r.Da_tmoments_Ptau_x_p",
				  ref.Da_tmoments_Ptau_x_p,"ref.Da_tmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Da_vmoments_Ptau_x_p,"r.Da_vmoments_Ptau_x_p",
				  ref.Da_vmoments_Ptau_x_p,"ref.Da_vmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.pmoments_Ptau_x_p,"r.pmoments_Ptau_x_p",
				  ref.pmoments_Ptau_x_p,"ref.pmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.Tpmoments_Ptau_x_p,"r.Tpmoments_Ptau_x_p",
				  ref.Tpmoments_Ptau_x_p,"ref.Tpmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.amoments_Ptau_x_p,"r.amoments_Ptau_x_p",
				  ref.amoments_Ptau_x_p,"ref.amoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.admoments_Ptau_x_p,"r.admoments_Ptau_x_p",
				  ref.admoments_Ptau_x_p,"ref.admoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.tsmoments_Ptau_x_p,"r.tsmoments_Ptau_x_p",
				  ref.tsmoments_Ptau_x_p,"ref.tsmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.vsmoments_Ptau_x_p,"r.vsmoments_Ptau_x_p",
				  ref.vsmoments_Ptau_x_p,"ref.vsmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.rtsmoments_Ptau_x_p,"r.rtsmoments_Ptau_x_p",
				  ref.rtsmoments_Ptau_x_p,"ref.rtsmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.rvsmoments_Ptau_x_p,"r.rvsmoments_Ptau_x_p",
				  ref.rvsmoments_Ptau_x_p,"ref.rvsmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.tauvbuymoments_Ptau_x_p,"r.tauvbuymoments_Ptau_x_p",
				  ref.tauvbuymoments_Ptau_x_p,"ref.tauvbuymoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.tauvsellmoments_Ptau_x_p,"r.tauvsellmoments_Ptau_x_p",
				  ref.tauvsellmoments_Ptau_x_p,"ref.tauvsellmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.pluspmoments_Ptau_x_p,"r.pluspmoments_Ptau_x_p",
				  ref.pluspmoments_Ptau_x_p,"ref.pluspmoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.minuspmoments_Ptau_x_p,"r.minuspmoments_Ptau_x_p",
				  ref.minuspmoments_Ptau_x_p,"ref.minuspmoments_Ptau_x_p",eps);
    final double [] pmom=TMatr.sumAcB(r.pluspmoments_Ptau_x_p,1,r.minuspmoments_Ptau_x_p);
    UnitTests.checkDoubleArrayRel1(r.pmoments_Ptau_x_p,"r.pmoments_Ptau_x_p",pmom,"pmom",epssum);

    UnitTests.checkDoubleArrayRel(r.vbuymoments_Ptau_x_p,"r.vbuymoments_Ptau_x_p",
				  ref.vbuymoments_Ptau_x_p,"ref.vbuymoments_Ptau_x_p",eps);
    UnitTests.checkDoubleArrayRel(r.vsellmoments_Ptau_x_p,"r.vsellmoments_Ptau_x_p",
				  ref.vsellmoments_Ptau_x_p,"ref.vsellmoments_Ptau_x_p",eps);
    final double [] vmom=TMatr.sumAcB(r.vbuymoments_Ptau_x_p,1,r.vsellmoments_Ptau_x_p);
    UnitTests.checkDoubleArrayRel(r.vmoments_Ptau_x_p,"r.vmoments_Ptau_x_p",vmom,"vmom",epssum);

    for(int j=1;j<Math.min(2,r.npm);j++)
    {
	final double [] sa=new double [r.ntm],spm=new double [r.ntm];
	for(int k=0,o=0;k<r.ntm;k++,o+=r.npm)
	{
	    spm[k]=r.pluspmoments_Ptau_x_p[o+j]-r.minuspmoments_Ptau_x_p[o+j];
	    sa[k]=r.amoments_Ptau_x_p[o+j-1];
	}
	UnitTests.checkDoubleArrayRel(spm,"spm",sa,"sa",epssum);
    }

    System.err.println("QVMDataP:checkMatching(): matching OK ntm="+r.ntm+" npm="+r.npm+" eps="+eps+" epssum="+epssum);
}


private static void matchWithTest(final String description,
				  final QVMDataP r,
				  final Trade [] tb,
				  final int tsize,
				  final double eps)
{
    System.err.println("QVMDataP:matchWithTest():======================"+description);

    final QVMData [] rt=new QVMData [r.ntm];
    for(int i=0;i<r.ntm;i++)
    {
	if(r.tau!=(r.tau/(i+1))*(i+1))
	    throw new RuntimeException("i="+i+" r.tau="+r.tau+" r.tau/(i+1))*(i+1)="+((r.tau/(i+1))*(i+1)));
	rt[i]=QVMData.calculateQVMData(1,
				       1,
				       r.npm,
				       r.currenttime,
				       r.tau/(i+1),
				       tb,
				       tsize,
				       null);
    }
    checkMatchingQVMData(r,rt,eps);

    System.err.println("QVMDataP:matchWithTest(): OK eps="+eps);
}


private static void checkMomQVMData(final String txt,final QVMDataP r,final double [] r_mom,
				    final QVMData [] rt,final int QVMData_MOM,final boolean flag_tscale,
				    final double eps)
{
    final double [][] abc=QVMData.get3ArrayFromArrayForTesting(r_mom,1,r.ntm,r.npm)[0];
    final double [][] abc_ref=new double [r.ntm][];
    for(int i=0;i<rt.length;i++)
    {
	final QVMData ref=rt[i];
	if(r.currenttime!=ref.currenttime)
	{
	    throw new RuntimeException(txt+":r.currenttime="+r.currenttime+
				       " ref.currenttime="+ref.currenttime);
	}
	if(r.tau!=ref.tau*(i+1))
	{
	    throw new RuntimeException(txt+":r.tau="+r.tau+" ref.tau="+ref.tau);
	}
	if(r.vol!=ref.vol)
	{
	    throw new RuntimeException(txt+":r.vol="+r.vol+" ref.vol="+ref.vol);
	}
	if(r.svol!=ref.svol)
	{
	    throw new RuntimeException(txt+":r.svol="+r.svol+" ref.svol="+ref.svol);
	}
	if(1!=ref.ntm||r.npm!=ref.npm)
	{
	    throw new RuntimeException(txt+":r dims=("+r.ntm+":"+r.npm+")\n"+
				       "ref dims=("+ref.nvm+":"+ref.ntm+":"+ref.npm+")");
	}
	abc_ref[i]=QVMData.get3ArrayFromArrayForTesting(ref.shiftOffset(0,0,ref.p_offset-r.p_offset,QVMData_MOM),ref.nvm,ref.ntm,ref.npm)[0][0];

	for(int k=0;flag_tscale && k<abc_ref[i].length;k++){ // different tau-scale
	    abc_ref[i][k]/=(r.tau/ref.tau);
	}
    }

    for(int l=0;l<r.npm;l++)
    {
	double [] ra=null,rb=null;
	for(int k=0;k<r.ntm;k++)
	{
	    final double [] x=WIntegratorLegendreShifted.B.getNext(k,ra,rb,WIntegratorMonomials.B);rb=ra;ra=x;
	    double s=0;
	    for(int i=0;i<=k;i++)
	    {
		s+=abc_ref[i][l]*x[i];
	    }
	    if(UnitTests.checkDoubleRelNoFail(s,abc[k][l],eps))
	    {
		throw new RuntimeException(txt+":Mismatch: abc_ref.s="+s+" abc["+k+"]["+l+"]="+abc[k][l]);
	    }
	}
    }
    System.err.println(txt+":QVMDataP:checkMomQVMData(): OK ntm="+r.ntm+" npm="+r.npm+" eps="+eps);
}

private static void checkMatchingQVMData(final QVMDataP r,final QVMData [] rt,final double eps)
{

    checkMomQVMData("tmoments_Ptau_x_p",r,r.tmoments_Ptau_x_p,rt,QVMData.MOM_dT,true,eps);
    checkMomQVMData("vmoments_Ptau_x_p",r,r.vmoments_Ptau_x_p,rt,QVMData.MOM_dV,false,eps);
    checkMomQVMData("Tvmoments_Ptau_x_p",r,r.Tvmoments_Ptau_x_p,rt,QVMData.MOM_TdV,true,eps);
    checkMomQVMData("VdPmoments_Ptau_x_p",r,r.VdPmoments_Ptau_x_p,rt,QVMData.MOM_VdPdt,false,eps);
    checkMomQVMData("Dp_tmoments_Ptau_x_p",r,r.Dp_tmoments_Ptau_x_p,rt,QVMData.MOM_DPT,true,eps);
    checkMomQVMData("Dp_vmoments_Ptau_x_p",r,r.Dp_vmoments_Ptau_x_p,rt,QVMData.MOM_DPV,false,eps);
    checkMomQVMData("Da_tmoments_Ptau_x_p",r,r.Da_tmoments_Ptau_x_p,rt,QVMData.MOM_DAT,true,eps);
    checkMomQVMData("Da_vmoments_Ptau_x_p",r,r.Da_vmoments_Ptau_x_p,rt,QVMData.MOM_DAV,false,eps);
    checkMomQVMData("vbuymoments_Ptau_x_p",r,r.vbuymoments_Ptau_x_p,rt,QVMData.MOM_VBUY,false,eps);
    checkMomQVMData("vsellmoments_Ptau_x_p",r,r.vsellmoments_Ptau_x_p,rt,QVMData.MOM_VSELL,false,eps);
    checkMomQVMData("pmoments_Ptau_x_p",r,r.pmoments_Ptau_x_p,rt,QVMData.MOM_DP,false,eps);
    checkMomQVMData("Tpmoments_Ptau_x_p",r,r.Tpmoments_Ptau_x_p,rt,QVMData.MOM_TDP,true,eps);
    checkMomQVMData("pluspmoments_Ptau_x_p",r,r.pluspmoments_Ptau_x_p,rt,QVMData.MOM_DPPLUS,false,eps);
    checkMomQVMData("minuspmoments_Ptau_x_p",r,r.minuspmoments_Ptau_x_p,rt,QVMData.MOM_DPMINUS,false,eps);
    checkMomQVMData("amoments_Ptau_x_p",r,r.amoments_Ptau_x_p,rt,QVMData.MOM_A,false,eps);
    checkMomQVMData("admoments_Ptau_x_p",r,r.admoments_Ptau_x_p,rt,QVMData.MOM_AD,false,eps);
    checkMomQVMData("tsmoments_Ptau_x_p",r,r.tsmoments_Ptau_x_p,rt,QVMData.MOM_TS,true,eps);
    checkMomQVMData("vsmoments_Ptau_x_p",r,r.vsmoments_Ptau_x_p,rt,QVMData.MOM_VS,false,eps);
    checkMomQVMData("rtsmoments_Ptau_x_p",r,r.rtsmoments_Ptau_x_p,rt,QVMData.MOM_RTS,true,eps);
    checkMomQVMData("rvsmoments_Ptau_x_p",r,r.rvsmoments_Ptau_x_p,rt,QVMData.MOM_RVS,false,eps);
    checkMomQVMData("tauvbuymoments_Ptau_x_p",r,r.tauvbuymoments_Ptau_x_p,rt,QVMData.MOM_TAUIBUY,true,eps);
    checkMomQVMData("tauvsellmoments_Ptau_x_p",r,r.tauvsellmoments_Ptau_x_p,rt,QVMData.MOM_TAUISELL,true,eps);

    System.err.println("QVMDataP:checkMatchingQVMData(): OK ntm="+r.ntm+" npm="+r.npm+" eps="+eps);
}


private static void checkCalculations(final int tsize,
				      final java.util.Random r,
				      final int ntm,
				      final int npm,
				      final int n_price_quants,
				      final double eps)
{
    final Trade [] tb=CalculateKendall.TradeEMoments.getRandomTradesQuantumPrice(tsize,r,n_price_quants);
    final long tau=(10000L*tb.length/3)/(3*5*7*8)*(3*5*7*8),currenttime=tb[tb.length-1].time+20000;
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println("QVMDataP:checkCalculations(): test with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+" ntm="+ntm+" npm="+npm+
		       " n_price_quants="+n_price_quants+" eps="+eps);

    final QVMDataP ro=QVMDataP.calculateQVMDataP(ntm,npm,currenttime,tau,tb,tsize,null);
    matchWithTest("INITIAL CALCULATION",ro,tb,tsize,eps);

    if(false){
	final var rU=CommonlyUsedMoments_TestFromSample.fromSeqRecurrently(
	    IvsVT_CalculateMoments_LegendreShifted::new,ntm,currenttime,tau,java.util.Arrays.copyOf(tb,tsize));
	rU.updateBasisToNewTimeReference(currenttime,ro.p_offset);
	System.err.println("dv="+java.util.Arrays.toString(rU.getdVMoments()));
	System.err.println("VdP="+java.util.Arrays.toString(rU.getVdPMoments()));
	System.err.println("dP="+java.util.Arrays.toString(rU.getdpMoments()));
    }

    // recurrent test
    final int n_split=8*(tb.length/10);
    final QVMDataP em_ref=QVMDataP.calculateQVMDataP(ntm,npm,(tb[n_split-1].time+tb[n_split].time)/2,tau,tb,n_split,null);
    matchWithTest("GET REFERENCE FOR RECURRENT",em_ref,tb,n_split,eps);

    if(false){
	final var rRef=CommonlyUsedMoments_TestFromSample.fromSeqRecurrently(
	    IvsVT_CalculateMoments_LegendreShifted::new,ntm,em_ref.currenttime,tau,java.util.Arrays.copyOf(tb,n_split));
	rRef.updateBasisToNewTimeReference(em_ref.currenttime,em_ref.p_offset);
	System.err.println("Orig rRef.dv="+java.util.Arrays.toString(rRef.getdVMoments()));
	System.err.println("Orig rRef.VdP="+java.util.Arrays.toString(rRef.getVdPMoments()));
	rRef.updateBasisToNewTimeReference(currenttime,ro.p_offset);
	System.err.println("sh rRef.dv="+java.util.Arrays.toString(rRef.getdVMoments()));
	System.err.println("sh rRef.VdP="+java.util.Arrays.toString(rRef.getVdPMoments()));
    }

    final QVMDataP em_fromref=QVMDataP.calculateQVMDataP(ntm,npm,currenttime,tau,tb,tsize,em_ref);
    checkMatching(em_fromref,ro,eps,eps);
    matchWithTest("RECURRENT CALCULATIONS",em_fromref,tb,tsize,eps);

    // check with data cut
    final int ncut=tb.length-n_split;
    System.arraycopy(tb,n_split,tb,0,ncut);
    final QVMDataP emc_cut=QVMDataP.calculateQVMDataP(ntm,npm,currenttime,tau,tb,ncut,em_ref);
    checkMatching(ro,emc_cut,eps,eps);
    checkMatching(em_fromref,emc_cut,eps*0,eps); // have to match exactly
    System.err.println("QVMDataP:checkCalculations() data cut OK");

    System.err.println("OrdersAndTrades.USE_ORIGTIME="+OrdersAndTrades.USE_ORIGTIME+
		       "\nOrdersAndTrades.SAVE_NONDISPLAYABLE_TRADES="+OrdersAndTrades.SAVE_NONDISPLAYABLE_TRADES+
		       "\nOrdersAndTrades.SAVE_EDGE_ORDERS="+OrdersAndTrades.SAVE_EDGE_ORDERS+
		       "\nOrdersAndTrades.FLAG_PSEUDOTRADES="+OrdersAndTrades.FLAG_PSEUDOTRADES);
    System.err.println("QVMDataP:checkCalculations(): OK with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+" ntm="+ntm+" npm="+npm+
		       " n_price_quants="+n_price_quants+" eps="+eps);
}


static void test_Self(final long initrandom)
{
    final java.util.Random r=new java.util.Random(initrandom);
    final double eps=1e-5;

    for(int ntm=1;ntm<=6;ntm++)
    for(int npm=1;npm<=6;npm++)
    {
	for(int i=0;i<11;i++)
	for(int n_price_quants:new int []{-1,2,3,4,5,10})
	{
	    checkCalculations(100+r.nextInt(100),r,ntm,npm,n_price_quants,eps);
	    checkCalculations(10010+r.nextInt(100010),r,ntm,npm,n_price_quants,eps);
	}
    }
}

public static void main(String [] args)
{
    System.err.println("QVMDataP::main(): starting tests; ver=$Id: QVMDataP.java,v 1.234 2022/09/14 08:19:26 mal Exp $");

    final long t0=System.currentTimeMillis();

    if(false){QTest.findInitRandom_NoFail(1,100,(final Long irandom) -> test_Self(irandom));}
    else {test_Self(1);}

    final long t1=System.currentTimeMillis();
    System.err.println("QVMDataP::main(): tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: QVMDataP.java,v 1.234 2022/09/14 08:19:26 mal Exp $");
}


static final double [][] LegendreShifted_POLS=new double [64][];
static {
    for(int k=0;k<LegendreShifted_POLS.length;k++)
    {
	LegendreShifted_POLS[k]=WIntegratorLegendreShifted.B.getNext(k,k<1?null:LegendreShifted_POLS[k-1],k<2?null:LegendreShifted_POLS[k-2],WIntegratorMonomials.B);
    }
}
} // END of class QVMDataP
