package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.WIntegratorLegendreShifted;

/** Extract the moments in shifted to [0:1] Legendre basis from {@link QVMDataP}.
 *  @see QVMDataP
 *  @see com.polytechnik.freemoney.CommonlyUsedMomentsLegendreShifted
 *  @version $Id: QVMDataPDirectAccess.java,v 1.55 2022/09/14 08:19:26 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QVMDataPDirectAccess implements DataInterfaceToMoments<QVMDataP> {

public void setM(final double [] m,final int n,final int mom,final QVMDataP odata,final int nmoms)
{
    if(n>=odata.npm||nmoms>odata.ntm) throw new IllegalArgumentException("n="+n+" odata.npm="+odata.npm+" nmoms="+nmoms+" odata.ntm="+odata.ntm);

    if(mom==M_O)
    {
	for(int k=0;k<nmoms;k++){m[k]=2*k+1;}
	return;
    }
    final double [] abc;
    if(mom==M_I) abc=odata.vmoments_Ptau_x_p;
    else if(mom==M_TI) abc=odata.Tvmoments_Ptau_x_p;
    else if(mom==M_ISELL) abc=odata.vsellmoments_Ptau_x_p;
    else if(mom==M_IBUY) abc=odata.vbuymoments_Ptau_x_p;
    else if(mom==M_DPPLUS) abc=odata.pluspmoments_Ptau_x_p;
    else if(mom==M_DPMINUS) abc=odata.minuspmoments_Ptau_x_p;
    else if(mom==M_DP) abc=odata.pmoments_Ptau_x_p;
    else if(mom==M_TDP) abc=odata.Tpmoments_Ptau_x_p;
    else if(mom==M_VdPdt) abc=odata.VdPmoments_Ptau_x_p;
    else if(mom==M_T) abc=odata.tmoments_Ptau_x_p;
    else if(mom==M_A) abc=odata.amoments_Ptau_x_p;
    else if(mom==M_AD) abc=odata.admoments_Ptau_x_p;
    else if(mom==M_TS) abc=odata.tsmoments_Ptau_x_p;
    else if(mom==M_VS) abc=odata.vsmoments_Ptau_x_p;
    else if(mom==M_RTS) abc=odata.rtsmoments_Ptau_x_p;
    else if(mom==M_RVS) abc=odata.rvsmoments_Ptau_x_p;
    else if(mom==M_TAUIBUY) abc=odata.tauvbuymoments_Ptau_x_p;
    else if(mom==M_TAUISELL) abc=odata.tauvsellmoments_Ptau_x_p;
    else if(mom==M_DPwV) abc=odata.Dp_vmoments_Ptau_x_p;
    else if(mom==M_DPwT) abc=odata.Dp_tmoments_Ptau_x_p;
    else if(mom==M_DAwV) abc=odata.Da_vmoments_Ptau_x_p;
    else if(mom==M_DAwT) abc=odata.Da_tmoments_Ptau_x_p;
    else throw new RuntimeException("mom="+mom);
    for(int k=0,o=n;k<nmoms;k++,o+=odata.npm){m[k]=abc[o];}
}

public void setMT(final double [] m,final int nmoms){java.util.Arrays.fill(m,0,nmoms,0);m[0]=1;}


public int setFMoments(final double [] Vmoments,final double [] dVmoments,final int n)
{
    for(int k=n;--k>=0;)
    {
	final double [] a=new double [k+1];
	a[k]=-1; // "-" sign integration by parts
	Vmoments[k]=TMatr.mulVVn1(WIntegratorLegendreShifted.getPolynomialWIntegratedDt(a),dVmoments);
    }
    return n;
}
public int setDFMoments(final double [] dFmoments,final double [] Fmoments,final int n)
{
    final double [] dF=WIntegratorLegendreShifted.B.getXPnprimeplusPnMomentsFromPnMoments(java.util.Arrays.copyOf(Fmoments,n));
    System.arraycopy(dF,0,dFmoments,0,n);
    return n;
}

public double getPOffset(final QVMDataP odata){return odata.p_offset;}
public double getPLast(final QVMDataP odata){return (odata.trade_last!=null?odata.trade_last.price:Double.NaN)-getPOffset(odata);}
public long getTLast(final QVMDataP odata){return odata.currenttime;}
public long getTau(final QVMDataP odata){return odata.tau;}
public long getTotalVolume(final QVMDataP odata){return odata.vol;}

final java.util.function.Function<double [],double []> getFMomentsFromDFMomentsMatchRadau=(dFmoms)->{
	final double [] XdFmoms=WIntegratorLegendreShifted.B.getXQkMomentsFromQkMoments(dFmoms),
	XFmoms=new double [XdFmoms.length];
	setFMoments(XFmoms,XdFmoms,XFmoms.length);
	TMatr.scaleVectorInPlace(XFmoms,-1); // to match sign
	return XFmoms;
    };
}
