package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.utils.TMatr;

/** QVML version of QVMCoefs.
 *  @version $Id: QVML.java,v 1.31 2022/07/28 22:30:50 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class QVML {

    static final Laguerre B=new Laguerre();

final double sigma,p_average;
final double [] icoefs,iPcoefs,tcoefst,tPcoefs,ptpcoefst,pdIcoefs,dIcoefs,Fcoefs,ibuycoefs,isellcoefs;
final int neq_t;
final QVMData odata;

final double pt,pt_f;

final double I,I_f,pI,pI_f,p_f,p_df,p_dfl,Ibuy,Isell,IMax,IbuyMax,IsellMax;

private QVML(final com.polytechnik.utils.PolynomialRootsFinder rf,
	     final int neq_t,
	     final QVMData odata)
{
    this.odata=odata;
    this.neq_t=neq_t;
    if(odata.npm<2||odata.ntm<neq_t) throw new IllegalArgumentException();

    final double [] abc=odata.vmoments_vol_x_tau_x_p,abc_t=odata.tmoments_vol_x_tau_x_p;
    final double pt0_t=abc_t[0],pt0=abc[0],pa=odata.npm<2?Double.NaN:abc[1]/pt0;
    this.p_average=pa+odata.p_offset;
    this.sigma=odata.npm<3?Double.NaN:Math.sqrt((abc[2]/pt0)-(pa*pa));
    final double dp_last=this.odata.trade_last==null?Double.NaN:(this.odata.trade_last.price-odata.p_offset);

    this.icoefs=new double [neq_t];
    this.iPcoefs=new double [neq_t];
    this.tcoefst=new double [neq_t];
    this.tPcoefs=new double [neq_t];
    this.ptpcoefst=new double [neq_t];
    this.pdIcoefs=new double [neq_t];
    this.dIcoefs=new double [neq_t];
    this.Fcoefs=new double [neq_t];
    this.ibuycoefs=new double [neq_t];
    this.isellcoefs=new double [neq_t];
    double [] ra=null,rb=null;
    double stf=0,stpf=0,st=0,stp=0;
    double sI=0,sI_f=0,spI=0,spI_f=0,sIbuy=0,sIsell=0;
    for(int n=0,n1=1;n<neq_t;++n,++n1)
    {
	final double [] l=B.getNext(n,ra,rb,WIntegratorMonomials.B);rb=ra;ra=l;
	final double [] L=new double [n+1];
	WIntegratorMonomials.inverseOddCoefs(n+1,L,l); // inverse time
	final double [] _ldl=Polynomials.getPolynomialDifferentiated(L),LdL=new double [n+1];
	System.arraycopy(_ldl,0,LdL,0,_ldl.length);LdL[n]=0;DeprecatedHelperTMatr.sumP1(LdL,L,n1,1); // create LdL
	DeprecatedHelperTMatr.sumP1(this.icoefs,L,n1,QVM.getAverage(L,n1,abc,0,odata.npm,pt0));
	DeprecatedHelperTMatr.sumP1(this.ibuycoefs,L,n1,QVM.getAverage(L,n1,odata.vbuymoments_vol_x_tau_x_p,0,odata.npm,pt0));
	DeprecatedHelperTMatr.sumP1(this.isellcoefs,L,n1,QVM.getAverage(L,n1,odata.vsellmoments_vol_x_tau_x_p,0,odata.npm,pt0));
	DeprecatedHelperTMatr.sumP1(this.iPcoefs,L,n1,QVM.getAverage(L,n1,abc,1,odata.npm,pt0));
	DeprecatedHelperTMatr.sumP1(this.tPcoefs,L,n1,QVM.getAverage(L,n1,abc_t,1,odata.npm,pt0_t));
	DeprecatedHelperTMatr.sumP1(this.tcoefst,L,n1,QVM.getAverage(L,n1,abc_t,0,odata.npm,pt0_t));

	//DeprecatedHelperTMatr.sumP1(this.ptpcoefst,L,n1,dp_last*Polynomials.getD0(n1,L,0)-QVM.getAverage(LdL,n1,abc_t,1,odata.npm,pt0_t));
	DeprecatedHelperTMatr.sumP1(this.ptpcoefst,L,n1,dp_last*QVM.getAverage(LdL,n1,abc_t,0,odata.npm,pt0_t)-QVM.getAverage(LdL,n1,abc_t,1,odata.npm,pt0_t));

	DeprecatedHelperTMatr.sumP1(this.pdIcoefs,L,n1,QVM.getAverage(LdL,n1,abc,1,odata.npm,pt0));
	DeprecatedHelperTMatr.sumP1(this.dIcoefs,L,n1,QVM.getAverage(LdL,n1,abc,0,odata.npm,pt0));

	final double lk0=Polynomials.getD0(L,0);
	DeprecatedHelperTMatr.sumP1(this.Fcoefs,L,n1,lk0);

	st+=lk0*QVM.getAverage(L,n1,abc_t,0,odata.npm,pt0_t);
	stp+=lk0*QVM.getAverage(L,n1,abc_t,1,odata.npm,pt0_t);
	stf+=lk0*QVM.getAverage(LdL,n1,abc_t,0,odata.npm,pt0_t);
	stpf+=lk0*QVM.getAverage(LdL,n1,abc_t,1,odata.npm,pt0_t);

	sI_f+=lk0*QVM.getAverage(LdL,n1,abc,0,odata.npm,pt0);
	spI_f+=lk0*QVM.getAverage(LdL,n1,abc,1,odata.npm,pt0);
	sI+=lk0*QVM.getAverage(L,n1,abc,0,odata.npm,pt0);
	spI+=lk0*QVM.getAverage(L,n1,abc,1,odata.npm,pt0);
	sIbuy+=lk0*QVM.getAverage(L,n1,odata.vbuymoments_vol_x_tau_x_p,0,odata.npm,pt0);
	sIsell+=lk0*QVM.getAverage(L,n1,odata.vsellmoments_vol_x_tau_x_p,0,odata.npm,pt0);
    }

    this.IMax=Polynomials.getD0(this.icoefs,getMaxClose(this.icoefs,rf));
    this.IbuyMax=Polynomials.getD0(this.ibuycoefs,getMaxClose(this.ibuycoefs,rf));
    this.IsellMax=Polynomials.getD0(this.isellcoefs,getMaxClose(this.isellcoefs,rf));

    this.I_f=sI_f/neq_t;
    this.I=sI;
    this.Ibuy=sIbuy;
    this.Isell=sIsell;
    this.pI=spI;
    this.pI_f=spI_f/neq_t;
    this.p_f=odata.p_offset+this.pI_f/this.I_f;
    this.p_df=odata.p_offset+(this.pI-this.pI_f)/(this.I-this.I_f);
    this.p_dfl=odata.p_offset+(dp_last*this.I-this.pI_f)/(this.I-this.I_f);


    //this.pt=odata.p_offset+stp/st;
    this.pt=odata.p_offset+stp;
    //this.pt_f=odata.p_offset+stpf/stf;
    this.pt_f=odata.p_offset+stpf/neq_t;


}

public static QVML calculateQVML(final int neq_t,
				 final long currenttime,
				 final long tau,
				 final Trade [] tb,
				 final int tsize,
				 final com.polytechnik.utils.PolynomialRootsFinder rf,
				 final QVML ref)
{
    final QVMData o=QVMData.calculateQVMData(1,
					     neq_t,
					     2,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new QVML(rf,neq_t,o);
}


private static double getMaxClose(final double [] coefs,
				  final com.polytechnik.utils.PolynomialRootsFinder rf)
{
    final double [] diff=Polynomials.getPolynomialDifferentiated(coefs);
    final int n=diff.length-1; // max order

    final PolynomialRootsFinder.PRoots u;
    if(n<=0)
    {
	u=new PolynomialRootsFinder.PRoots(0,diff,new double [0],new double [0]);
    }
    else if(n>4){
	if(rf==null) {
	    u=new PolynomialRootsFinder.PRoots(0,diff,new double [n],new double [n]);
	} else {
	    u=rf.findRoots(diff);
	}
    } else {
	u=com.polytechnik.utils.PolynomialRootsElementary.ROOTSFINDER.findRoots(diff);
    }
    final double [] root_re=u.re_root,root_im=u.im_root;

    // insertion sort by real part with extra load
    com.polytechnik.utils.SortArrays.sortArrayWithLoad(u.nroots,root_re,root_im);
    final int n_reroots=CalculateKendall.selectRealRoots(root_re,u.nroots,root_re,root_im,CalculateKendall.EPS_IM);
    int ind_el=-1;
    for(int k=n_reroots;--k>=0;)
    {
	final double t=root_re[k];
	if(!(t<=0)) {}
	else if(Polynomials.getD1(diff,t)<=0)
	{ // max
	    ind_el=k;
	    break;
	}
    }
    //System.err.println("diff1="+java.util.Arrays.toString(diff));
    return ind_el<0?Double.NaN:root_re[ind_el];
}



private static void checkMatchingLaguerre(final int neq_t,
					  final java.util.Random r,
					  final QVMData odata,
					  final double eps,
					  final com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG rf)
{
  final double [][] abc3v=QVMData.get3ArrayFromArrayForTesting(odata.vmoments_vol_x_tau_x_p,odata.nvm,odata.ntm,odata.npm)[0],
	abc3t=QVMData.get3ArrayFromArrayForTesting(odata.tmoments_vol_x_tau_x_p,odata.nvm,odata.ntm,odata.npm)[0];
  final double [] tm=new double [odata.ntm],vm=new double [odata.ntm],
      dvm=new double [odata.ntm],dpvm=new double [odata.ntm],
      ptm=new double [odata.ntm],pvm=new double [odata.ntm],one=new double [odata.ntm];
  one[0]=1;
  final double fft=6.1,ffv=1.78;
  for(int k=0;k<odata.ntm;k++)
  {
      tm[k]=abc3t[k][0]*fft;
      ptm[k]=abc3t[k][1]*fft;
      vm[k]=abc3v[k][0]*ffv;
      dvm[k]=(abc3v[k][0]+(k>0?k*abc3v[k-1][0]:0))*ffv;
      pvm[k]=abc3v[k][1]*ffv;
      dpvm[k]=(abc3v[k][1]+(k>0?k*abc3v[k-1][1]:0))*ffv;
  }

  final double [] lins=DeprecatedHelperEVXData.getLinsysSol(neq_t,tm,0,vm,0),plins=DeprecatedHelperEVXData.getLinsysSol(neq_t,tm,0,pvm,0),
      dlins=DeprecatedHelperEVXData.getLinsysSol(neq_t,tm,0,dvm,0),dplins=DeprecatedHelperEVXData.getLinsysSol(neq_t,tm,0,dpvm,0),
      ones=DeprecatedHelperEVXData.getLinsysSol(neq_t,tm,0,one,0);
  for(int k=0;k<neq_t;k++) {final double f=tm[0]/vm[0];lins[k]*=f;plins[k]*=f;dlins[k]*=f;dplins[k]*=f;ones[k]*=fft;}
  final QVML q=new QVML(rf,neq_t,odata);
  UnitTests.checkDoubleArrayRel1(q.icoefs,"q.icoefs",lins,"lins",eps);
  UnitTests.checkDoubleArrayRel1(q.iPcoefs,"q.iPcoefs",plins,"plins",eps);
  UnitTests.checkDoubleArrayRel1(q.tPcoefs,"q.tPcoefs",DeprecatedHelperEVXData.getLinsysSol(neq_t,tm,0,ptm,0),"tPcoefs LIN",eps);
  UnitTests.checkDoubleArrayRel1(q.dIcoefs,"q.dIcoefs",dlins,"dlins",eps);
  UnitTests.checkDoubleArrayRel1(q.pdIcoefs,"q.pdIcoefs",dplins,"dplins",eps);
  UnitTests.checkDoubleArrayRel1(q.Fcoefs,"q.Fcoefs",ones,"ones",eps);
  //System.err.println("O="+java.util.Arrays.toString(ones));

  final double [] eqt=new double [neq_t];eqt[0]=1;
  UnitTests.checkDoubleArrayRel1(q.tcoefst,"q.tcoefst",eqt,"eqt",eps);
  UnitTests.checkDoubleArrayRel1(q.tcoefst,"q.tcoefst",DeprecatedHelperEVXData.getLinsysSol(neq_t,tm,0,tm,0),"tcoefst LIN",eps);


  System.err.println("QVML:checkMatchingLaguerre: tests OK  neq_t="+neq_t+" eps="+eps);
}

private static void checkMatchingPolynomialP(final int neq_t,
					     final java.util.Random r,
					     final double eps,
					     final com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG rf,
					     final int tsize,
					     final long tau)
{
    if(false && neq_t<=1)
    {
	System.err.println("QVML:checkMatchingPolynomialP: neq_t="+neq_t+" is too low, skipping test. Also note that trades with 'all prices the same' possibly make new version of QVMData{L,P} extremely slow!");
	return;
    }

final double [] pcoefs_o=new double [neq_t],pcoefs_o_TradeVX=new double [neq_t];
    for(int k=0;k<neq_t;k++)
    {
	// set random coefficients. also make sure high order powers are relatively low
	pcoefs_o[k]=r.nextDouble()/(2+Math.exp(k)+r.nextDouble());
	// different normalizing & signs for odd moments is used by TradeVX.getRandomTradesPolynomialP
	pcoefs_o_TradeVX[k]=pcoefs_o[k]*((k/2*2==k?1:-1)*Polynomials.fact(k));
    }

    final Trade [] tb=new Trade [tsize];
    final long currenttime=CalculateKendall.TradeVX.getRandomTradesPolynomialP(tb,r,tau,pcoefs_o_TradeVX);
    final QVMData odata=QVMData.calculateQVMData(1+r.nextInt(2),
						 neq_t+r.nextInt(2),
						 1+1+r.nextInt(3),
						 currenttime,
						 tau,
						 tb,
						 tsize,
						 null);

    final QVML q=new QVML(rf,neq_t,odata);
    System.err.println("D="+java.util.Arrays.toString(q.ptpcoefst)+"\nDD="+java.util.Arrays.toString(Polynomials.getPolynomialDifferentiated(pcoefs_o)));
    UnitTests.checkDoubleArrayRel1(java.util.Arrays.copyOf(q.ptpcoefst,q.ptpcoefst.length-1),"q.ptpcoefst",Polynomials.getPolynomialDifferentiated(pcoefs_o),"d/dt pcoefs_o",eps);
    final double [] pcoefs=q.tPcoefs.clone();pcoefs[0]+=q.odata.p_offset;
    UnitTests.checkDoubleArrayRel1(pcoefs,"pcoefs",pcoefs_o,"pcoefs_o",eps);

    System.err.println("QVML:checkMatchingPolynomialP: tests OK  neq_t="+neq_t+" eps="+eps);
}

private static void checkMatchingValues(final int neq_t,
					final java.util.Random r,
					final QVMData odata,
					final double eps,
					final double epssum,
					final com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG rf)
{
    final QVML q=new QVML(rf,neq_t,odata);

    // check pt & pt_f
    UnitTests.checkDoubleRel(q.pt,"q.pt",odata.p_offset+Polynomials.getD0(q.tPcoefs,0)/Polynomials.getD0(q.tcoefst,0),"pt from pols",epssum);
    UnitTests.checkDoubleRel(q.pt_f,"q.pt_f",q.odata.trade_last.price-Polynomials.getD0(q.ptpcoefst,0)/q.neq_t,"pt_f from ptpcoefst",epssum);

    UnitTests.checkDoubleRel(q.I,"q.I",Polynomials.getD0(q.icoefs,0),"q.icoefs(0)",eps);
    UnitTests.checkDoubleRel(q.I_f,"q.I_f",Polynomials.getD0(q.dIcoefs,0)/q.neq_t,"q.dIcoefs(0)/neq_t",eps);

    UnitTests.checkDoubleRel(q.pI,"q.pI",Polynomials.getD0(q.iPcoefs,0),"q.iPcoefs(0)",eps);
    UnitTests.checkDoubleRel(q.pI_f,"q.pI_f",Polynomials.getD0(q.pdIcoefs,0)/q.neq_t,"q.pdIcoefs(0)/neq_t",eps);

    UnitTests.checkDoubleRel(q.I,"q.I",q.Ibuy+q.Isell,"q.Ibuy+q.Isell",eps);


    UnitTests.checkDoubleRel(q.p_f,"q.p_f",q.odata.p_offset+Polynomials.getD0(q.pdIcoefs,0)/Polynomials.getD0(q.dIcoefs,0),"p_f from pdIcoefs & dIcoefs",eps);
    UnitTests.checkDoubleRel(q.p_df,"q.p_df",q.odata.p_offset+(Polynomials.getD0(q.iPcoefs,0)*q.neq_t-Polynomials.getD0(q.pdIcoefs,0))/(Polynomials.getD0(q.icoefs,0)*q.neq_t-Polynomials.getD0(q.dIcoefs,0)),"p_df from coefs",eps);
    UnitTests.checkDoubleRel(q.p_dfl,"q.p_dfl",q.odata.p_offset+((odata.trade_last.price-odata.p_offset)*Polynomials.getD0(q.icoefs,0)*q.neq_t-Polynomials.getD0(q.pdIcoefs,0))/(Polynomials.getD0(q.icoefs,0)*q.neq_t-Polynomials.getD0(q.dIcoefs,0)),"p_dfl from coefs",eps);


    System.err.println("QVML:checkMatchingValues: tests OK  neq_t="+neq_t+" eps="+eps+" epssum="+epssum);
}


private static void matchLaguerre(final int tsize,final java.util.Random r,final double eps)
{
    final Trade [] tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
    final long currenttime=tb[tb.length-1].time+1000;
    final long tau=(tb[tb.length-1].time-tb[0].time)/50;
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println("QVML:matchLaguerre(): test with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau);
 for(int n_max=0;n_max<=5;n_max++)
 {
    final QVMData odata=QVMData.calculateQVMData(1+r.nextInt(2),
						 1+2*n_max+r.nextInt(2),
						 1+r.nextInt(2),
						 currenttime,
						 tau,
						 tb,
						 tsize,
						 null);

    double [] ra=null,rb=null;
    for(double [] abc:new double [][]{odata.vmoments_vol_x_tau_x_p,odata.tmoments_vol_x_tau_x_p})
    for(int n=0;n<n_max;n++)
    {
	final double [] x=B.getNext(n,ra,rb,WIntegratorMonomials.B);rb=ra;ra=x;

	final double [][][] abc3=QVMData.get3ArrayFromArrayForTesting(abc,odata.nvm,odata.ntm,odata.npm);
	final double [] mom=new double [odata.ntm];
	for(int k=odata.ntm;--k>=0;)
	{
	    mom[k]=abc3[0][k][0];
	}

	final double [] f=QVM.QVMORT.getFLast(mom,mom.length,n)[0];
	final double [] fl=new double [n+1];
	WIntegratorMonomials.inverseOddCoefs(n+1,fl,x); // inverse time
	UnitTests.checkDoubleArrayRel(f,"f",fl,"fl",eps);

	final double [] fa=new double [n];
	for(int k=n;--k>=0;fa[k]=-fl[k]/fl[n]);
	UnitTests.checkDoubleArrayRel(DeprecatedHelperEVXData.getLinsysSol(n,mom,0,mom,n),"LS",fa,"fa",eps);
    }
 }
 System.err.println("QVML:matchLaguerre(): test OK with N points="+tsize+
		    " currenttime="+currenttime+" tau="+tau+" eps="+eps);

}

private static void checkCalculations(final int tsize,final java.util.Random r,final int neq_t,final double eps)
{
    final Trade [] tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
    final long tau=10000L*tb.length/50,currenttime=tb[tb.length-1].time+1000;
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println("QVML:checkCalculations(): test with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+
		       " neq_t="+neq_t);

    final com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG rf=
	new com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG();
    final QVMData odata=QVMData.calculateQVMData(1+r.nextInt(2),
						 1+2*neq_t+r.nextInt(2),
						 1+1+r.nextInt(3),
						 currenttime,
						 tau,
						 tb,
						 tsize,
						 null);
    // Laguerre
    checkMatchingLaguerre(neq_t,r,odata,eps,rf);
    // check Polynomials
    checkMatchingPolynomialP(neq_t,r,eps,rf,tsize,tau);
    // check values
    checkMatchingValues(neq_t,r,odata,eps*1e-6,eps,rf);

    System.err.println("QVML:checkCalculations(): OK with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+
		       " neq_t="+neq_t+" eps="+eps);
}


static double [] getOddcoefsInversed(final double [] l)
{
    final double [] res=new double [l.length];
    // helper function
    WIntegratorMonomials.inverseOddCoefs(l.length,res,l);
    return res;
}


public static void main(String [] args)
{
    System.err.println("QVML::main(): starting tests; ver=$Id: QVML.java,v 1.31 2022/07/28 22:30:50 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79095);
    final double eps=5e-3,epssum=3e-2;
    for(int neq_t=1;neq_t<=5;neq_t++)
    {
	for(int i=0;i<7;i++)
	{
	    // Match Laguerre directly
	    matchLaguerre(1000010+r.nextInt(1000010),r,epssum);
	    // regular checks
	    checkCalculations(100010+r.nextInt(100010),r,neq_t,eps);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println("QVML::main(): tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: QVML.java,v 1.31 2022/07/28 22:30:50 mal Exp $");
}

} // END of class QVML
