package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.PolynomialRootsFinder;

/** Like QVML but uses Laguerre basis.
 *  @version $Id: QVMLL.java,v 1.290 2022/07/28 22:30:50 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class QVMLL {

static final Laguerre B=new Laguerre();

final double sigma,p_average;
final double [] icoefs,ibuycoefs,isellcoefs,iPcoefs,tPcoefs,dPcoefs,dP2coefs;
final int neq_t;
final QVMDataL odata;

final double I,paverage_eq,p_eq,sigma2_eq,osp,ospp,pi_eq,sigma2pi_eq,scale_io,scale_io0,di_1,dip_1,scale_io1,sigma2_ii0,sigma2_ii1,dpi_eq,di_2,dip_2,scale_io2,sigma2_ii2,Dosp,scale_dip0,scale_dip0a,io,scale_ix,dp_eq,scaleo_dp_eq,scaleo_io,dip_1o;
/*private*/ QVMLL(final int neq_t,final QVMDataL odata,final PolynomialRootsFinder rf)
{
    this.odata=odata;
    this.neq_t=neq_t;
    if(odata.npm<3||odata.ntm<neq_t) throw new IllegalArgumentException();

    final double [] abc=odata.vmoments_Ltau_x_p,abc_t=odata.tmoments_Ltau_x_p;
    final double pt0=abc[0],pa=odata.npm<2?Double.NaN:abc[1]/pt0;
    this.p_average=pa+odata.p_offset;
    this.sigma=odata.npm<3?Double.NaN:Math.sqrt((abc[2]/pt0)-(pa*pa));

    this.icoefs=new double [neq_t];
    this.ibuycoefs=new double [neq_t];
    this.isellcoefs=new double [neq_t];
    this.iPcoefs=new double [neq_t];
    this.tPcoefs=new double [neq_t];
    this.dPcoefs=new double [neq_t];
    this.dP2coefs=new double [neq_t];

    final double dplast=(odata.trade_last!=null?odata.trade_last.price:Double.NaN)-odata.p_offset,dplast2=dplast*dplast;
    for(int n=0,o=0;n<neq_t;++n,o+=odata.npm)
    {
	this.icoefs[n]=abc[o]/pt0;
	this.ibuycoefs[n]=odata.vbuymoments_Ltau_x_p[o]/pt0;
	this.isellcoefs[n]=odata.vsellmoments_Ltau_x_p[o]/pt0;
	this.iPcoefs[n]=abc[1+o]/pt0;
	this.tPcoefs[n]=abc_t[1+o]/abc_t[0];
	this.dPcoefs[n]=dplast-odata.pmoments_Ltau_x_p[1+o];
	this.dP2coefs[n]=dplast2-odata.pmoments_Ltau_x_p[2+o];
    }
    this.I=get_f_o(neq_t,this.icoefs);
    this.paverage_eq=this.dPcoefs[0]+odata.p_offset;
    if(true)
    {
	final double [] xcoefs=getADiss(neq_t,odata);
	this.p_eq=xcoefs[0];
	this.sigma2_eq=xcoefs[1];
	this.osp=xcoefs[2];
	this.ospp=xcoefs[3];
	this.pi_eq=xcoefs[4];
	this.sigma2pi_eq=xcoefs[5];
	this.scale_io0=xcoefs[6];
	this.scale_io=xcoefs[7];
	this.di_1=xcoefs[8];
	this.dip_1=xcoefs[9];
	this.scale_io1=xcoefs[10];
	this.sigma2_ii0=xcoefs[11];
	this.sigma2_ii1=xcoefs[12];
	this.dpi_eq=xcoefs[13];
	this.di_2=xcoefs[14];
	this.dip_2=xcoefs[15];
	this.scale_io2=xcoefs[16];
	this.sigma2_ii2=xcoefs[17];
	this.Dosp=xcoefs[18];
	this.scale_dip0=xcoefs[19];
	this.io=xcoefs[20];
	this.scale_ix=xcoefs[21];
	this.scale_dip0a=xcoefs[22];
	this.dp_eq=xcoefs[23];
	this.scaleo_dp_eq=xcoefs[24];
	this.scaleo_io=xcoefs[25];
	this.dip_1o=xcoefs[26];
    }
}

public static QVMLL calculateQVMLL(final int neq_t,
				   final long currenttime,
				   final long tau,
				   final Trade [] tb,
				   final int tsize,
				   final QVMLL ref,
				   final PolynomialRootsFinder rf)
{
    final QVMDataL o=QVMDataL.calculateQVMDataL(neq_t,
						3,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new QVMLL(neq_t,o,rf);
}

private static double [] getADiss(final int neq_t,final QVMDataL odata)
{
    final double [] dp=new double [neq_t],dp2=new double [neq_t],i=new double [neq_t];
    final double dplast=(odata.trade_last!=null?odata.trade_last.price:Double.NaN)-odata.p_offset,dplast2=dplast*dplast;
    for(int k=0,ok=0;k<neq_t;k++,ok+=odata.npm)
    {
	dp[k]=dplast-odata.pmoments_Ltau_x_p[1+ok];
	dp2[k]=dplast2-odata.pmoments_Ltau_x_p[2+ok];
	i[k]=odata.vmoments_Ltau_x_p[ok]/odata.vmoments_Ltau_x_p[0];
    }

    final double o_o=get_o_o(neq_t),dp_o=get_f_o(neq_t,dp),dp_dp=get_f_g(neq_t,dp,dp),
	i_i=get_f_g(neq_t,i,i),i_o=get_f_o(neq_t,i);
    final double fdp_o=dp_o/o_o,fdp2_o=get_f_o(neq_t,dp2)/o_o,fi_o=i_o/o_o;
    final double p_eq=odata.p_offset+fdp_o;
    for(int k=0;k<neq_t;k++){dp[k]-=fdp_o;dp2[k]-=fdp2_o;i[k]-=fi_o;}
    final double x_i_dp=get_f_g(neq_t,i,dp),x_dp_dp=get_f_g(neq_t,dp,dp),x_i_i=get_f_g(neq_t,i,i),
	fi_q1=x_i_dp/x_dp_dp,fdp2_q1=get_f_g(neq_t,dp2,dp)/x_dp_dp;
    final double sigma2_eq=x_dp_dp/o_o,osp=x_i_dp/Math.sqrt(x_i_i*x_dp_dp),
	dpi=(x_i_dp/x_i_i)*fi_o,pi_eq=p_eq-dpi,
	sigma2pi_eq=(x_dp_dp-x_i_dp*x_i_dp/x_i_i)/o_o;
    final double [] dissinter1=new double []{-fi_o+fi_q1*fdp_o,-fi_q1};
    for(int k=dissinter1.length;--k>=0;) dissinter1[k]*=o_o;

    final double sigma2_ii1=x_i_i-x_i_dp*fi_q1,
	di_1=Polynomials.getD0(dissinter1,dplast),
	dip_1=Polynomials.getD1(dissinter1,dplast)/o_o,
	scale_io1=Math.sqrt(o_o*sigma2_ii1),
	dpl_dp_eq=dplast-fdp_o,
	dpi_eq=(osp*osp)*dpl_dp_eq+dpi;

    for(int k=0;k<neq_t;k++){dp2[k]-=dp[k]*fdp2_q1;i[k]-=dp[k]*fi_q1;}
    final double xx_i_dp2=get_f_g(neq_t,i,dp2),xx_dp2_dp2=get_f_g(neq_t,dp2,dp2),xx_i_i=get_f_g(neq_t,i,i),
	ospp=xx_i_dp2/Math.sqrt(xx_dp2_dp2*xx_i_i),fi_q2=xx_i_dp2/xx_dp2_dp2;
    final double [] dissinter2=new double []{-fi_o+fi_q1*fdp_o+fi_q2*(fdp2_o-fdp2_q1*fdp_o),-fi_q1+fi_q2*fdp2_q1,-fi_q2};
    for(int k=dissinter2.length;--k>=0;) dissinter2[k]*=o_o;

    final double sigma2_ii2=sigma2_ii1-xx_i_dp2*xx_i_dp2/xx_dp2_dp2,
	di_2=Polynomials.getD0(dissinter2,dplast),
	dip_2=Polynomials.getD1(dissinter2,dplast)/o_o;

    final double scale_io0=Math.sqrt(o_o*x_i_i),
	dip_1o=(i_o*dpl_dp_eq-x_i_dp)/(o_o*dpl_dp_eq*dpl_dp_eq+x_dp_dp),
	Dosp=(dip_1o+fi_q1)*Math.sqrt(x_dp_dp/x_i_i);

    return new double []{p_eq,
			 sigma2_eq,
			 osp,
			 ospp,
			 pi_eq,
			 sigma2pi_eq,
			 scale_io0,
			 Math.sqrt(o_o*i_i),
			 di_1,
			 dip_1,
			 scale_io1,
			 x_i_i,
			 sigma2_ii1,
			 dpi_eq,
			 di_2,
			 dip_2,
			 Math.sqrt(o_o*sigma2_ii2),
			 sigma2_ii2,
			 Dosp,Math.sqrt(x_i_i/x_dp_dp),i_o,
			 Math.sqrt(sigma2_ii1*o_o+i_o*i_o),
			 Math.sqrt(x_i_i/(x_dp_dp+dp_o*fdp_o)),
			 dpl_dp_eq*o_o,
			 Math.sqrt(o_o*(dpl_dp_eq*dpl_dp_eq*o_o+x_dp_dp)),
			 Math.sqrt(i_o*i_o+o_o*x_i_i),
			 dip_1o};
}

static double get_f_g(final int neq_t,final double [] f,final double [] g)
{
    if(neq_t!=f.length) throw new RuntimeException();
    return B.get2LaguerreAveraged(f,g);
}
static double get_f_o(final int neq_t,final double [] f)
{
    if(f.length!=neq_t) throw new RuntimeException();
    return B.getD0(f,0);
}
static double get_o_o(final int neq_t){return neq_t;}
static double [] get_dmoments0(final int neq_t)
{
    final double [] g=new double [neq_t];g[0]=1;
    return B.getL1FromL0(g);
}

private static void checkMatchingOValues(final int neq_t,final java.util.Random r,final long currenttime,final long tau,final Trade [] tb,final int tsize,final double eps,final double epssum)
{
    if(neq_t<4)
    {
	System.err.println("QVMLL:checkMatchingOValues: neq_t="+neq_t+" is too low, skipping test");
	return;
    }

    final double [] o=get_dmoments0(neq_t);
    if(true)
    {
	final Trade [] tbt=tb.clone();
	final QVMLL q=QVMLL.calculateQVMLL(neq_t,currenttime,tau,tbt,tsize-1,null,null);
	final Trade tl=new Trade(CalculateKendall.TradeEMoments.getRandomPrice(r),
				 100,
				 null,
				 BookItem.BOOK_BUY,
				 currenttime,
				 0,
				 0,
				 currenttime-r.nextInt(100000),
				 null,
				 Trade.TRADE_TYPE_REGULAR,
				 null,
				 null,
				 0);
	tbt[tsize-1]=tl;
	final double [] il=TMatr.getVectorScaled(q.icoefs,q.odata.vmoments_Ltau_x_p[0]);
	TMatr.sumP1(il,o,tl.shares);
	final QVMLL ql=QVMLL.calculateQVMLL(neq_t,currenttime,tau,tbt,tsize,null,null);
	UnitTests.checkDoubleArrayRel(TMatr.getVectorScaled(ql.icoefs,ql.odata.vmoments_Ltau_x_p[0]),"ql.icoefs*scale",il,"il",eps);

	final double [] dpf=q.dPcoefs.clone(); // offset is compensated, no polynomial shift is required
	TMatr.sumP1(dpf,o,-(ql.odata.trade_last.price-q.odata.trade_last.price));
	UnitTests.checkDoubleArrayRel(ql.dPcoefs,"ql.dPcoefs",dpf,"dpf",eps);
	for(int k=0,ok=0;k<neq_t;++k,ok+=q.odata.npm)
	{
	    final double dm=ql.odata.pmoments_Ltau_x_p[1+ok]-q.odata.pmoments_Ltau_x_p[1+ok],
		dmx=ql.odata.trade_last.price-q.odata.trade_last.price;
	    UnitTests.checkDoubleRel(dm,"dm",dmx,"dmx",eps);
	}
    }

    System.err.println("QVMLL:checkMatchingOValues: tests OK  neq_t="+neq_t+" eps="+eps+" epssum="+epssum);
}

private static SPIMin getSPI(final int neq_t,final int neq_p,final QVMDataL odata)
{
    return new SPIMinLaguerre(neq_t,neq_p,odata);
}
private static void checkMatchingADissValues(final int neq_t,final java.util.Random r,final long currenttime,final long tau,final Trade [] tb,final int tsize,final double eps,final double epssum)
{
    if(neq_t<5)
    {
	System.err.println("QVMLL:checkMatchingADissValues: neq_t="+neq_t+" is too low, skipping test");
	return;
    }

    final QVMLL q=QVMLL.calculateQVMLL(neq_t,currenttime,tau,tb,tsize,null,null);
    final double plast=q.odata.trade_last.price,dplast=plast-q.odata.p_offset;
    if(true)
    {
	final SPIMin sa=getSPI(neq_t,1,q.odata);
	if(true)
	{
	    final double [] ps=sa.getP0I();
	    UnitTests.checkDoubleRel(q.p_eq,"q.p_eq",ps[0],"ps[0]",eps);
	    UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",ps[1],"ps[1]",eps);
	}
	final SPIMin.SPIMinData xx=sa.getI0P();
	UnitTests.checkDoubleRel(xx.inter[0],"xx.inter[0]",-xx.io,"-xx.io",eps);
	UnitTests.checkDoubleRel(xx.scale_io,"xx.scale_io",xx.scale_io0,"scale.io0",eps);
    }
    final double o_o=get_o_o(neq_t);
    if(true)
    {
	final SPIMin sa=getSPI(neq_t,2,q.odata);
	if(true)
	{
	    final double [] ps=sa.getP0I();
	    UnitTests.checkDoubleRel(q.pi_eq,"q.pi_eq",ps[0],"ps[0]",eps);
	    UnitTests.checkDoubleRel(q.sigma2pi_eq,"q.sigma2pi_eq",ps[1],"ps[1]",eps);
	    UnitTests.checkDoubleRel(plast-q.pi_eq-(1-q.osp*q.osp)*(plast-q.p_eq),"dpi_eq",q.dpi_eq,"q.dpi_eq",eps);
	}
	final SPIMin.SPIMinData xx=sa.getI0P();
	UnitTests.checkDoubleRel(q.di_1,"q.di_1",Polynomials.getD0(xx.inter,dplast),"xx.inter(p)",eps);
	UnitTests.checkDoubleRel(q.dip_1,"q.dip_1",Polynomials.getD1(xx.inter,dplast)/o_o,"d xx.inter(p)/dp/o_o",eps);
	UnitTests.checkDoubleRel(-q.osp,"-q.osp",q.dip_1*(o_o*Math.sqrt(q.sigma2_eq)/q.scale_io0),"q.dip_1*scale",eps);
	UnitTests.checkDoubleRel(q.scale_io1,"q.scale_io1",xx.scale_io,"xx.scale_io",eps);
	UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",xx.scale_io0,"xx.scale_io0",eps);
	UnitTests.checkDoubleRel(q.sigma2_ii1,"q.sigma2_ii1",xx.sigma2_ii,"xx.sigma2_ii",eps);
	UnitTests.checkDoubleRel(q.p_eq,"q.p_eq",xx.p_eq,"xx.p_eq",eps);
	UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",xx.sigma2_eq,"xx.sigma2_eq",eps);
	UnitTests.checkDoubleRel(plast-q.p_eq,"p_last-p_eq",(xx.io+Polynomials.getD0(xx.inter,dplast))/Polynomials.getD1(xx.inter,dplast),"i/i'",eps);
	UnitTests.checkDoubleRel(q.Dosp,"q.Dosp",((dplast*xx.io-get_f_g(neq_t,q.icoefs,q.dPcoefs))/Polynomials.getD0(xx.dissp,dplast)-Polynomials.getD1(xx.inter,dplast)/o_o)*(o_o*Math.sqrt(xx.sigma2_eq)/xx.scale_io0),"Dosp:inter",eps);
    }
    if(true)
    {
	final SPIMin.SPIMinData xx=getSPI(neq_t,3,q.odata).getI0P();
	UnitTests.checkDoubleRel(q.di_2,"q.di_2",Polynomials.getD0(xx.inter,dplast),"xx.inter(p)",eps);
	UnitTests.checkDoubleRel(q.dip_2,"q.dip_2",Polynomials.getD1(xx.inter,dplast)/o_o,"d xx.inter(p)/dp/o_o",eps);
	UnitTests.checkDoubleRel(q.scale_io2,"q.scale_io2",xx.scale_io,"xx.scale_io",eps);
	UnitTests.checkDoubleRel(q.scale_io0,"q.scale_io0",xx.scale_io0,"xx.scale_io0",eps);
	UnitTests.checkDoubleRel(q.sigma2_ii2,"q.sigma2_ii2",xx.sigma2_ii,"xx.sigma2_ii",eps);
	UnitTests.checkDoubleRel(q.p_eq,"q.p_eq",xx.p_eq,"xx.p_eq",eps);
	UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",xx.sigma2_eq,"xx.sigma2_eq",eps);
	System.err.println("d=3;xx.inter="+java.util.Arrays.toString(xx.inter));
    }

    System.err.println("QVMLL:checkMatchingADissValues: tests OK  neq_t="+neq_t+" eps="+eps+" epssum="+epssum);
}

private static void checkMatchingLinsystemValues(final int neq_t,final java.util.Random r,final long currenttime,final long tau,final Trade [] tb,final int tsize,final double eps,final double epssum)
{
    final QVMLL q=new QVMLL(neq_t,QVMDataL.calculateQVMDataL(2*neq_t-1+r.nextInt(3),
							     3+r.nextInt(3),
							     currenttime,
							     tau,
							     tb,
							     tsize,
							     null),null);

    final double scale=q.odata.tmoments_Ltau_x_p[0]/q.odata.vmoments_Ltau_x_p[0];
    System.err.println("QVMLL:checkMatchingLinsystemValues: neq_t="+neq_t+" q.odata.ntm="+q.odata.ntm+" q.odata.npm="+q.odata.npm);
    UnitTests.checkDoubleArrayRel1(DeprecatedHelperEVXData.getLinsysSolPBasis(neq_t,q.odata.tmoments_Ltau_x_p,0,q.odata.npm,q.odata.tmoments_Ltau_x_p,1/* 1 offset for P */,q.odata.npm,QVMDataL.Laguerre_POLS),"tPcoefs lin",q.tPcoefs,"q.tPcoefs",epssum);
    UnitTests.checkDoubleArrayRel1(DeprecatedHelperEVXData.getLinsysSolPBasis(neq_t,q.odata.tmoments_Ltau_x_p,0,q.odata.npm,TMatr.getVectorScaled(q.odata.vmoments_Ltau_x_p,scale),0,q.odata.npm,QVMDataL.Laguerre_POLS),"icoefs lin",q.icoefs,"q.icoefs",epssum);
    UnitTests.checkDoubleArrayRel1(DeprecatedHelperEVXData.getLinsysSolPBasis(neq_t,q.odata.tmoments_Ltau_x_p,0,q.odata.npm,TMatr.getVectorScaled(q.odata.vbuymoments_Ltau_x_p,scale),0,q.odata.npm,QVMDataL.Laguerre_POLS),"ibuycoefs lin",q.ibuycoefs,"q.ibuycoefs",epssum);
    UnitTests.checkDoubleArrayRel1(DeprecatedHelperEVXData.getLinsysSolPBasis(neq_t,q.odata.tmoments_Ltau_x_p,0,q.odata.npm,TMatr.getVectorScaled(q.odata.vsellmoments_Ltau_x_p,scale),0,q.odata.npm,QVMDataL.Laguerre_POLS),"isellcoefs lin",q.isellcoefs,"q.isellcoefs",epssum);
    UnitTests.checkDoubleArrayRel1(DeprecatedHelperEVXData.getLinsysSolPBasis(neq_t,q.odata.tmoments_Ltau_x_p,0,q.odata.npm,TMatr.getVectorScaled(q.odata.vmoments_Ltau_x_p,scale),1 /* 1 offset for P */,q.odata.npm,QVMDataL.Laguerre_POLS),"iPcoefs lin",q.iPcoefs,"q.iPcoefs",epssum);

    System.err.println("QVMLL:checkMatchingLinsystemValues: tests OK  neq_t="+neq_t+" eps="+eps+" epssum="+epssum);

}

private static void checkMatchingPolynomialP(final int neq_t,
					     final java.util.Random r,
					     final double eps,
					     final int tsize,
					     final long tau)
{
    if(false && neq_t<=1)
    {
	System.err.println("QVMLL:checkMatchingPolynomialP: neq_t="+neq_t+" is too low, skipping test. Also note that trades with 'all prices the same' possibly make new version of QVMData{L,P} extremely slow!");
	return;
    }
    final double [] pcoefsa=new double [neq_t];
    for(int k=0;k<neq_t;k++)
    {
	// set random coefficients.
	pcoefsa[k]=0.3*(2*r.nextDouble()-1);
    }
    final double [] pcoefs=B.convertFunctionTo_Monomials_from_Basis(pcoefsa),pcoefs_o_TradeVX=new double [neq_t];
    for(int k=0;k<neq_t;k++)
    {
	// different normalizing is used by TradeVX.getRandomTradesPolynomialP
	pcoefs_o_TradeVX[k]=pcoefs[k]*Polynomials.fact(k);
    }

    final Trade [] tb=new Trade [tsize];
    final long currenttime=CalculateKendall.TradeVX.getRandomTradesPolynomialP(tb,r,tau,pcoefs_o_TradeVX);

    final QVMLL q=calculateQVMLL(neq_t,currenttime,tau,tb,tsize,null,null);

    // q.tPcoefs has an offset qt.odata.p_offset
    final double [] res_pcoefs=q.tPcoefs.clone();res_pcoefs[0]+=q.odata.p_offset;

    UnitTests.checkDoubleArrayRel1(B.convertFunctionTo_Monomials_from_Basis(res_pcoefs),"res_pcoefs",pcoefs,"pcoefs",eps);

    final double [] pcoefs_laguerre=B.convertFunctionTo_Basis_from_Monomials(pcoefs);
    pcoefs_laguerre[0]-=q.odata.p_offset;
    UnitTests.checkDoubleArrayRel1(q.tPcoefs,"q.tPcoefs",pcoefs_laguerre,"pcoefs_laguerre",eps);

    final double [] x_dpcoefs=TMatr.getVectorScaled(B.getDifferentiated(pcoefs_laguerre),-1),o0=new double [neq_t],dpmom=new double [neq_t];o0[0]=1;
    final double [] o=B.getL1FromL0(o0);
    TMatr.sumP1(dpmom,o,q.odata.trade_last.price-q.odata.p_offset);
    TMatr.sumP1(dpmom,q.dPcoefs,-1);
    System.err.println("DD="+java.util.Arrays.toString(x_dpcoefs)+
		       "\nDP="+java.util.Arrays.toString(dpmom));
    UnitTests.checkDoubleArrayRel1(x_dpcoefs,"x_dpcoefs",java.util.Arrays.copyOf(dpmom,neq_t-1),"dpmom",eps);
    final double [] dpmom2=B.getL1FromL0(q.tPcoefs);
    UnitTests.checkDoubleArrayRel1(q.dPcoefs,"q.dPcoefs",dpmom2,"dpmom2",eps);


    System.err.println("QVMLL:checkMatchingPolynomialP: tests OK  neq_t="+neq_t+" eps="+eps);
}

private static void checkMatchingQVML(final int neq_t,
				      final java.util.Random r,
				      final long currenttime,
				      final long tau,
				      final Trade [] tb,
				      final int tsize,
				      final double eps,
				      final double epssum,
				      final com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG rf)
{

  final QVML ref=QVML.calculateQVML(neq_t,currenttime,tau,tb,tsize,rf,null);
  final QVMDataL ox=QVMDataL.calculateQVMDataL(neq_t+r.nextInt(3),
					       3+r.nextInt(3),
					       currenttime,tau,tb,tsize,null);
  for(QVMLL rll: new QVMLL[]{QVMLL.calculateQVMLL(neq_t,currenttime,tau,tb,tsize,null,null),new QVMLL(neq_t,ox,null)})
  {
    final double doff=rll.odata.p_offset-ref.odata.p_offset;
    UnitTests.checkDoubleRel(rll.p_average,"rll.p_average",ref.p_average,"ref.p_average",eps);
    if(rll.odata.npm>2 == ref.odata.npm>2)
	UnitTests.checkDoubleRel(rll.sigma,"rll.sigma",ref.sigma,"ref.sigma",eps);
    UnitTests.checkDoubleRel(rll.neq_t,"rll.neq_t",ref.neq_t,"ref.neq_t",eps);

    final double [] ref_icoefs=QVML.getOddcoefsInversed(B.convertFunctionTo_Monomials_from_Basis(rll.icoefs)),
	ref_ibuycoefs=QVML.getOddcoefsInversed(B.convertFunctionTo_Monomials_from_Basis(rll.ibuycoefs)),
	ref_isellcoefs=QVML.getOddcoefsInversed(B.convertFunctionTo_Monomials_from_Basis(rll.isellcoefs));
    UnitTests.checkDoubleArrayRel1(ref_icoefs,"ref_icoefs",ref.icoefs,"ref.icoefs",eps);
    UnitTests.checkDoubleArrayRel1(ref_ibuycoefs,"ref_ibuycoefs",ref.ibuycoefs,"ref.ibuycoefs",eps);
    UnitTests.checkDoubleArrayRel1(ref_isellcoefs,"ref_isellcoefs",ref.isellcoefs,"ref.isellcoefs",eps);
    UnitTests.checkDoubleArrayRel1(B.convertFunctionTo_Basis_from_Monomials(QVML.getOddcoefsInversed(ref_icoefs)),"ref_icoefs",rll.icoefs,"rll.icoefs",eps);

    final double [] rll_iPcoefs_offs2=new double [rll.neq_t],rll_tPcoefs_offs2=new double [rll.neq_t];
    for(int k=0;k<rll.neq_t;k++)
    {
	rll_iPcoefs_offs2[k]=rll.iPcoefs[k]+doff*rll.icoefs[k];
	rll_tPcoefs_offs2[k]=rll.tPcoefs[k]+doff*rll.odata.tmoments_Ltau_x_p[k*rll.odata.npm]/rll.odata.tmoments_Ltau_x_p[0];
    }
    final double [] ref_iPcoefs=QVML.getOddcoefsInversed(B.convertFunctionTo_Monomials_from_Basis(rll_iPcoefs_offs2)),
	ref_tPcoefs=QVML.getOddcoefsInversed(B.convertFunctionTo_Monomials_from_Basis(rll_tPcoefs_offs2));
    UnitTests.checkDoubleArrayRel1(ref_iPcoefs,"ref_iPcoefs",ref.iPcoefs,"ref.iPcoefs",eps);
    UnitTests.checkDoubleArrayRel1(ref_tPcoefs,"ref_tPcoefs",ref.tPcoefs,"ref.tPcoefs",eps);
    UnitTests.checkDoubleArrayRel1(B.convertFunctionTo_Basis_from_Monomials(QVML.getOddcoefsInversed(ref_tPcoefs)),"ref_tPcoefs",rll_tPcoefs_offs2,"rll_tPcoefs_offs2",eps);
    UnitTests.checkDoubleArrayRel1(B.convertFunctionTo_Basis_from_Monomials(QVML.getOddcoefsInversed(ref_iPcoefs)),"ref_iPcoefs",rll_iPcoefs_offs2,"rll_iPcoefs_offs2",eps);

    UnitTests.checkDoubleRel(rll.I,"rll.I",ref.I,"ref.I",eps);

    for(int j=0;j<100;j++)
    {
	final double x=r.nextDouble();
	UnitTests.checkDoubleRel(B.getD0(rll.icoefs,x),"rll.icoefs",Polynomials.getD0(ref.icoefs,-x),"ref.icoefs",eps);
	UnitTests.checkDoubleRel(rll.odata.p_offset+B.getD0(rll.iPcoefs,x)/B.getD0(rll.icoefs,x),"rll.iPcoefs/rll.icoefs",ref.odata.p_offset+Polynomials.getD0(ref.iPcoefs,-x)/Polynomials.getD0(ref.icoefs,-x),"ref.iPcoefs/ref.icoefs",eps);
    }
  }

  System.err.println("QVMLL:checkMatchingQVML: tests OK  neq_t="+neq_t+" eps="+eps+" epssum="+epssum);
}


private static void checkCalculations(final int tsize,final java.util.Random r,final int neq_t,final double eps,final double epssum)
{
    final Trade [] tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
    final long currenttime=tb[tb.length-1].time+1000;
    final long tau=(tb[tb.length-1].time-tb[0].time)/24;
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println("QVMLL:checkCalculations(): test with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+
		       " neq_t="+neq_t);

    final com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG rf=
	new com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG();

    // match to QVML
    checkMatchingQVML(neq_t,r,currenttime,tau,tb,tsize,eps,epssum,rf);
    checkMatchingOValues(neq_t,r,currenttime,tau,tb,tsize,eps,epssum);
    checkMatchingADissValues(neq_t,r,currenttime,tau,tb,tsize,eps,epssum);
    checkMatchingPolynomialP(neq_t>4?4:neq_t,r,epssum,tsize,tau);
    checkMatchingLinsystemValues(neq_t>5?5:neq_t,r,currenttime,tau,tb,tsize,eps,epssum);


    System.err.println("QVMLL:checkCalculations(): OK with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+
		       " neq_t="+neq_t);
}

public static void main(String [] args)
{
    System.err.println("QVMLL::main(): starting tests; ver=$Id: QVMLL.java,v 1.290 2022/07/28 22:30:50 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79091);
    final double eps=1e-6,epssum=1e-3;
    for(int neq_t=1;neq_t<=11;neq_t++)
    {
	for(int i=0;i<11;i++)
	{
	    //checkCalculations(100+r.nextInt(100),r,neq_t,eps,epssum);
	    checkCalculations(100010+r.nextInt(100010),r,neq_t,eps,epssum);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println("QVMLL::main(): tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: QVMLL.java,v 1.290 2022/07/28 22:30:50 mal Exp $");
}

} // END of class QVMLL
