package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Laguerre;

/** Like QVMLL but uses indexes.
 *  @version $Id: QVMLMV.java,v 1.10 2022/06/29 06:23:40 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QVMLMV extends QVMR<QVMDataL>{
static final Laguerre B=new Laguerre();
private QVMLMV(final int neq_t,final int neq_i,
	       final QVMDataL odata,final QVMDataL [] idata)
{
    super(neq_t,neq_i,odata,idata);
}

double getOffset(final QVMDataL d){return d.p_offset;}
double get_o_o(){return neq_t;}
double get_f_o(final double [] f){
    if(f.length!=neq_t) throw new RuntimeException();
    return B.getD0(f,0);
}
double get_f_g(final double [] f,final double [] g){
    if(neq_t!=f.length) throw new RuntimeException();
    return B.get2LaguerreAveraged(f,g);
}
void setI(final double [] i,final QVMDataL d)
{
    if(odata.npm<2||odata.ntm<neq_t||(neq_i+1)>=neq_t||d.npm!=odata.npm||d.ntm!=odata.ntm) throw new IllegalArgumentException();

    final double [] abc=d.vmoments_Ltau_x_p;
    for(int k=0,o=0;k<neq_t;k++,o+=d.npm)
    {
	i[k]=(abc[o]/abc[0]);
    }
}
void setDP(final double [] dp,final QVMDataL d)
{
    if(odata.npm<2||odata.ntm<neq_t||(neq_i+1)>=neq_t||d.npm!=odata.npm||d.ntm!=odata.ntm) throw new IllegalArgumentException();
    for(int k=0,o=1;k<neq_t;k++,o+=d.npm){dp[k]=d.pmoments_Ltau_x_p[o];}
}
void rem0(final double [] f)
{
    final double delta=get_f_o(f)/get_o_o();
    for(int k=0;k<neq_t;k++){f[k]-=delta;}
}


public static QVMLMV calculateQVMLMV(final int neq_t,
				     final int neq_i,
				     final long currenttime,
				     final long tau,
				     final Trade [] tb,
				     final int tsize,
				     final Trade [][] itb,
				     final int [] itbsize,
				     final QVMLMV ref)
{
    final int npm=3;
    final QVMDataL odata=QVMDataL.calculateQVMDataL(neq_t,
						    npm,
						    currenttime,
						    tau,
						    tb,
						    tsize,
						    ref!=null?ref.odata:null);
    final QVMDataL [] idata=new QVMDataL [neq_i];
    for(int k=0;k<neq_i;k++)
    {
	idata[k]=QVMDataL.calculateQVMDataL(neq_t,
					    npm,
					    currenttime,
					    tau,
					    itb[k],
					    itbsize[k],
					    ref!=null?ref.idata[k]:null);
    }
    return new QVMLMV(neq_t,neq_i,odata,idata);
}


private static void checkMatchingQVM(final int neq_t,final int neq_i,final java.util.Random r,final int tsize,final double eps,final double epssum)
{
    if(!(neq_t>neq_i+2))
    {
	System.err.println("QVMLMV:checkMatchingQVM(): skipping neq_t="+neq_t+"<=neq_i+2="+(neq_i+2));
	return;
    }
    final Trade [] o_tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
    final long tau=(o_tb[o_tb.length-1].time-o_tb[0].time)/10;
    long currenttime=o_tb[o_tb.length-1].time+1000;
    final Trade [][] i_tb=new Trade [neq_i][];
    final int [] i_tb_size=new int [neq_i];
    for(int i=0;i<neq_i;i++)
    {
	final Trade [] tb=i_tb[i]=CalculateKendall.TradeEMoments.getRandomTrades(tsize+r.nextInt(tsize+1),r);
	i_tb_size[i]=tb.length;
	if(currenttime<tb[tb.length-1].time)
	{
	    currenttime=tb[tb.length-1].time;
	}
    }

    QVMLMV q=calculateQVMLMV(neq_t,
			     neq_i,
			     currenttime,
			     tau,
			     o_tb,
			     tsize,
			     i_tb,
			     i_tb_size,
			     null);

    final QVMLMV qll=calculateQVMLMV(neq_t,
				     1,
				     currenttime,
				     tau,
				     o_tb,
				     tsize,
				     new Trade[][]{o_tb},
				     new int []{tsize},
				     null);

    UnitTests.checkDoubleRel(q.p_eq,"q.p_eq",qll.p_eq,"qll.p_eq",eps);
    UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",qll.sigma2_eq,"qll.sigma2_eq",eps);


    if(neq_i>0)
    {
	i_tb[neq_i-1]=o_tb;
	i_tb_size[neq_i-1]=tsize;
	q=calculateQVMLMV(neq_t,
			  neq_i,
			  currenttime,
			  tau,
			  o_tb,
			  tsize,
			  i_tb,
			  i_tb_size,
			  null);
	UnitTests.checkDoubleRel(q.p_eq,"q.p_eq",qll.p_eq,"qll.p_eq",eps);
	UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",qll.sigma2_eq,"qll.sigma2_eq",eps);
	UnitTests.checkDoubleRel(q.pf_eq,"q.pf_eq",qll.pf_eq,"qll.pf_eq",eps);
    }
    else
    {
	UnitTests.checkDoubleRel(q.p_eq,"q.p_eq",qll.p_eq,"qll.p_eq",eps);
	UnitTests.checkDoubleRel(q.sigma2_eq,"q.sigma2_eq",qll.sigma2_eq,"qll.sigma2_eq",eps);
	UnitTests.checkDoubleRel(q.pf_eq,"q.pf_eq",qll.p_eq,"qll.p_eq",eps);
    }

    System.err.println("QVMLMV:checkMatchingQVM: tests OK  neq_t="+neq_t+" neq_i="+neq_i+" eps="+eps+" epssum="+epssum);
}

private static void checkCalculations(final int tsize,final java.util.Random r,final int neq_t,final int neq_i,final double eps,final double epssum)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println("QVMLMV:checkCalculations(): test with N points="+tsize+" neq_t="+neq_t+" neq_i="+neq_i);

    // match dissi directly
    checkMatchingQVM(neq_t,neq_i,r,tsize,eps,epssum);

    System.err.println("QVMLMV:checkCalculations(): OK with N points="+tsize+
		       " neq_t="+neq_t+" neq_i="+neq_i);
}

public static void main(String [] args)
{
    System.err.println("QVMLMV:main(): starting tests; ver=$Id: QVMLMV.java,v 1.10 2022/06/29 06:23:40 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79091);
    final double eps=1e-6,epssum=1e-3;
    for(int neq_t=1;neq_t<=11;neq_t++)
    {
	for(int neq_i=0;neq_i<=5;neq_i++)
	{
	    for(int i=0;i<11;i++)
	    {
		checkCalculations(200+r.nextInt(200),r,neq_t,neq_i,eps,epssum);
		checkCalculations(100010+r.nextInt(100010),r,neq_t,neq_i,eps,epssum);
	    }
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println("QVMLMV::main(): tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: QVMLMV.java,v 1.10 2022/06/29 06:23:40 mal Exp $");
}

} // END of QVMLMV
