package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.utils.TMatr;

/** The results extend this class.
 *  @version $Id: QVMR.java,v 1.3 2022/06/27 13:32:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
abstract class QVMR<DTYPE> {

final int neq_t,neq_i;
final DTYPE odata;
final DTYPE [] idata;

public final double p_eq,sigma2_eq,pf_eq;
private final double virial_factor;
QVMR(final int neq_t,final int neq_i,
    final DTYPE odata,final DTYPE [] idata)
{
    this.odata=odata;
    this.idata=idata;
    this.neq_t=neq_t;
    this.neq_i=neq_i;

    this.virial_factor=SM.VIRIAL_FACTOR;
    if(true)
    {
	final double p_offset=getOffset(odata);
	final double [][] xx=getDiss(odata,idata);
	if(true)
	{
	    final double [] dissp=xx[0];
	    if(true)
	    {
		final double da=2*dissp[2],dp=-dissp[1]/da;
		this.p_eq=p_offset+dp;
		this.sigma2_eq=2*dissp[0]/da-dp*dp;
		final double [] ips=xx[1];
		final double dpf=-(dissp[1]-this.virial_factor*ips[0])/da;
		this.pf_eq=p_offset+dpf;
	    }
	}
    }
}
abstract double getOffset(final DTYPE d);
abstract double get_o_o();
abstract double get_f_o(final double [] f);
abstract double get_f_g(final double [] f,final double [] g);
abstract void setDP(final double [] dp,final DTYPE d);
abstract void setI(final double [] i,final DTYPE d);
abstract void rem0(final double [] f);
double [][] getDiss(final DTYPE d,final DTYPE [] idata)
{
    final double [] dp=new double [neq_t];
    setDP(dp,odata);
    final double dp_dp=get_f_g(dp,dp),dp_o=get_f_o(dp),o_o=get_o_o();
    final double [] dissp=new double []{dp_dp,-2*dp_o,o_o};

    final int neq=neq_i;
    rem0(dp);
    final double [] ma=new double [neq_t],mb=new double [neq_t],i=new double [neq_t];
    final double [] G=new double [neq*neq];
    final double [] rpart=new double [neq],ipsf=new double [neq];
    for(int k=0;k<neq;k++)
    {
	setI(i,idata[k]);rem0(i);
	setDP(ma,idata[k]);rem0(ma);
	ipsf[k]=-(get_f_g(i,ma)/Math.sqrt(o_o*get_f_g(i,i))); // using ATYPE_FMIN model
	for(int l=0;l<=k;l++)
	{
	    setDP(mb,idata[l]);rem0(mb);
	    G[k*neq+l]=G[l*neq+k]=get_f_g(ma,mb);
	}
	rpart[k]=get_f_g(ma,dp);
    }
    final double [] ips=new double [1];
    final int [] ipvt=new int [neq];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(G,neq,neq,ipvt); // G is clobbered now
    if(pinfo!=0)
    {
	java.util.Arrays.fill(ips,Double.NaN);
	return new double [][]{dissp,ips};
    }
    com.polytechnik.utils.Linsystems.dgesl_m1(G,neq,neq,ipvt,rpart,1);
    ips[0]=o_o*TMatr.mulVVn1(rpart,ipsf);
    return new double [][]{dissp,ips};
}

} // END of class QVMR<DTYPE>
