package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;

/** Like QVMLL but in price polynomial basis.
 *  @version $Id: QVMpi.java,v 1.88 2019/01/07 19:53:45 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QVMpi {

final int neq_p;
final double sigma,p_average;
private final double [] gcoefs,dissp,dvissp,kcoefs,kvcoefs;
final double sigma2tot_ii,di,dip,sigma2tot_x_ii,inorm,pnorm,dpi_eq_scaled,dip_scaled;
final double Ik,dIk,dIkL,dIt,dIv;

private final QVMDataL odata;

private QVMpi(final int neq_p,final QVMDataL odata,
	      final com.polytechnik.utils.PolynomialRootsFinder rf)
{
    this.odata=odata;
    this.neq_p=neq_p;
    //if(odata.ntm!=1) throw new IllegalArgumentException();
    if(odata.npm<2*neq_p+1) throw new IllegalArgumentException();

    final double [] abc=odata.vmoments_Ltau_x_p,abc_t=odata.tmoments_Ltau_x_p;
    final double pt0=abc[0],dpa=odata.npm<2?Double.NaN:abc[1]/pt0;
    this.p_average=dpa+odata.p_offset;
    this.sigma=odata.npm<3?Double.NaN:Math.sqrt((abc[2]/pt0)-(dpa*dpa));

    final double [] ri=new double [2*neq_p+1];
    final double norm=abc_t[0]/pt0;
    for(int k=0;k<neq_p;++k){ri[k]=abc[k]*norm;} // normalize
    // Calculate directly, orthogonal polynomials calculation is in the unit test.
    final double [][] xs=QVM.QVMORT.getFLast(abc_t,0,odata.npm,neq_p,ri,0),
	xv=QVM.QVMORT.getFLast(abc,0,odata.npm,neq_p,null,-1);
    this.kcoefs=xs[1];
    this.kvcoefs=xv[1];
    this.dissp=xs[2];
    this.dvissp=xv[2];
    final double [] icoefs=xs[3];
    this.gcoefs=QVM.getPolynomialAIntegrated(icoefs,abc_t,abc_t[0]);
    this.sigma2tot_ii=QVM.getAverage(Polynomials.mult2Pol(icoefs,icoefs),odata.tmoments_Ltau_x_p,odata.tmoments_Ltau_x_p[0]);

    final double dplast=(odata.trade_last!=null?odata.trade_last.price:Double.NaN)-odata.p_offset;
    if(true)
    {
	// for now
	final double F=Polynomials.getD0(dissp,dplast),G=Polynomials.getD0(gcoefs,dplast),
	    K=Polynomials.getD0(kcoefs,dplast),
	    dF=Polynomials.getD1(dissp,dplast),dG=Polynomials.getD1(gcoefs,dplast);

	final double x_i_dp=(G-dG*(dF/(2*K))),
	    x_dp_dp=(F-dF*(dF/(4*K))),
	    x_i_i=this.sigma2tot_ii-dG*(dG/K);
	sigma2tot_x_ii=x_i_i;
	dip=x_i_dp/x_dp_dp;
	di=dip*(dF/2)-dG;

	dpi_eq_scaled=((dF/2)-(x_i_dp/x_i_i)*dG)/Math.sqrt(K*x_dp_dp);
	dip_scaled=x_i_dp/Math.sqrt(x_dp_dp*x_i_i);
	this.pnorm=Math.sqrt(F/K);
	this.inorm=Math.sqrt(K*sigma2tot_x_ii);
    }
    if(true)
    {
	final double Kt=Polynomials.getD0(kcoefs,dplast),
	    Kv=Polynomials.getD0(kvcoefs,dplast),
	    dKt=Polynomials.getD1(kcoefs,dplast),
	    dKv=Polynomials.getD1(kvcoefs,dplast);

	this.Ik=Kt/Kv;
	this.dIk=dKt/Kv-(Ik)*(dKv/Kv);
	dIt=dKt/Kt;
	dIv=dKv/Kv;
	this.dIkL=dIt-dIv;
    }
}


public static QVMpi calculateQVMpi(final int neq_p,
				   final long currenttime,
				   final long tau,
				   final Trade [] tb,
				   final int tsize,
				   final com.polytechnik.utils.PolynomialRootsFinder rf,
				   final QVMpi ref)
{
    final QVMDataL o=QVMDataL.calculateQVMDataL(1,
						2*neq_p+1,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new QVMpi(neq_p,o,rf);
}


private static void checkMatchingQATradeIP(final int neq_p,
					   final java.util.Random r,
					   final long currenttime,
					   final long tau,
					   final Trade [] tb,
					   final int tsize,
					   final double eps,
					   final double epssum,
					   final com.polytechnik.utils.PolynomialRootsFinder rf)
{

  if(neq_p<2)
  {
      System.err.println("QVMpi:checkMatchingQATradeIP: neq_p="+neq_p+"<2; skipping test");
      return;
  }
  final CalculateKendall.QATradeIP ref=CalculateKendall.QATradeIP.calculateQATradeIP(neq_p-1,currenttime,tau,tb,tsize,rf,null), // number of equations is now defined differently (-1 to neq_p)
      refv=CalculateKendall.QATradeIP.calculateQATradeIP_V(neq_p-1,currenttime,tau,tb,tsize,rf,null);

  final QVMDataL ox=QVMDataL.calculateQVMDataL(1+r.nextInt(4),Math.max(2*neq_p+1,3)+r.nextInt(4),currenttime,tau,tb,tsize,null);

  for(QVMpi q: new QVMpi[]{QVMpi.calculateQVMpi(neq_p,currenttime,tau,tb,tsize,rf,null),new QVMpi(neq_p,ox,rf)})
  {

      final double plast=q.odata.trade_last.price,dplast=plast-q.odata.p_offset;

      UnitTests.checkDoubleRel(Polynomials.getD1(q.gcoefs,dplast),"d gcoefs/dp",1+ref.getAI0(plast),"1+I:QATradeIP",epssum);

      final double [] ref_pvcoefs=Polynomials.getPolynomialShifted(ref.pcoefs,q.odata.p_offset-ref.odata.p_offset);
      ref_pvcoefs[0]+=1; // add average;
      UnitTests.checkDoubleArrayRel(ref_pvcoefs,"ref_pvcoefs",Polynomials.getPolynomialDifferentiated(q.gcoefs),"d gcoefs/dp",eps);
      UnitTests.checkDoubleRel(QVM.getAverage(q.gcoefs,q.odata.tmoments_Ltau_x_p,q.odata.tmoments_Ltau_x_p[0]),"<gcoefs>",0,"0",eps);
      final double [] ref_kcoefs=Polynomials.getPolynomialShifted(ref.didv,q.odata.p_offset-ref.odata.p_offset),
	  refv_kcoefs=Polynomials.getPolynomialShifted(refv.didv,q.odata.p_offset-refv.odata.p_offset);
      ref_kcoefs[0]+=1;
      refv_kcoefs[0]+=1;
      UnitTests.checkDoubleArrayRel(ref_kcoefs,"ref_kcoefs",q.kcoefs,"q.kcoefs",eps);
      UnitTests.checkDoubleArrayRel(refv_kcoefs,"refv_kcoefs",q.kvcoefs,"q.kvcoefs",eps);

      // match to QVM.getPDPCoefs
      final double [][] pdp0=QVM.getPDPCoefs(q.odata.tmoments_Ltau_x_p,neq_p-1);
      UnitTests.checkDoubleArrayRel(q.kcoefs,"q.kcoefs",pdp0[0],"pdp0[0]",eps);
      UnitTests.checkDoubleArrayRel(q.dissp,"q.dissp",pdp0[1],"pdp0[1]",eps);
      final double [][] pdp0v=QVM.getPDPCoefs(q.odata.vmoments_Ltau_x_p,neq_p-1);
      UnitTests.checkDoubleArrayRel(q.kvcoefs,"q.kvcoefs",pdp0v[0],"pdp0v[0]",eps);
      UnitTests.checkDoubleArrayRel(q.dvissp,"q.dvissp",pdp0v[1],"pdp0v[1]",eps);
  }

  System.err.println("QVMpi:checkMatchingQATradeIP: tests OK  neq_p="+neq_p+" eps="+eps+" epssum="+epssum);
}

private static void checkMatchingLinsystemValues(final int neq_p,final java.util.Random r,final long currenttime,final long tau,final Trade [] tb,final int tsize,final double eps,final double epssum,final com.polytechnik.utils.PolynomialRootsFinder rf)
{
    final QVMpi q=QVMpi.calculateQVMpi(neq_p,currenttime,tau,tb,tsize,rf,null);
    System.err.println("QVMpi:checkMatchingLinsystemValues: neq_p="+neq_p+" q.odata.ntm="+q.odata.ntm+" q.odata.npm="+q.odata.npm);

    final double [] lsys=DeprecatedHelperEVXData.getLinsysSol(q.neq_p,q.odata.tmoments_Ltau_x_p,0,TMatr.getVectorScaled(q.odata.vmoments_Ltau_x_p,q.odata.tmoments_Ltau_x_p[0]/q.odata.vmoments_Ltau_x_p[0]),0);
    UnitTests.checkDoubleArrayRel(lsys,"LINSYS icoefs",Polynomials.getPolynomialDifferentiated(q.gcoefs),"d q.gcoefs/dp",eps);

    System.err.println("QVMpi:checkMatchingLinsystemValues: tests OK  neq_p="+neq_p+" eps="+eps+" epssum="+epssum);
}

private static void checkMatchingOrthogonalPolynomials(final int neq_p,final java.util.Random r,final long currenttime,final long tau,final Trade [] tb,final int tsize,final double eps,final double epssum,final com.polytechnik.utils.PolynomialRootsFinder rf)
{
  final QVMpi q=QVMpi.calculateQVMpi(neq_p,currenttime,tau,tb,tsize,rf,null);

  final double [] abc=q.odata.vmoments_Ltau_x_p,abc_t=q.odata.tmoments_Ltau_x_p;
  final double pt0=abc[0],pt0t=abc_t[0];

  if(true)
  {
    // calculate using orthogonal polynomials basis.
    final double [] gcoefs=new double [neq_p+1],dissp=new double [2*neq_p+1],
	dp=new double [2*neq_p],kcoefs=new double [2*neq_p-1];
    double sigma2tot_ii=0;
    for(int k=0;k<q.neq_p;++k)
    {
	final double [] pi=QVM.QVMORT.getFLast(abc_t,q.odata.npm,k)[0],
	                int_pi=QVM.getPolynomialAIntegrated(pi,k+1,abc_t,pt0t);
	UnitTests.checkDoubleRel(QVM.getAverage(int_pi,abc_t,pt0t),"<int_pi>",0,"0",eps);
	UnitTests.checkDoubleRel(QVM.getAverage(Polynomials.mult2Pol(pi,pi),abc_t,pt0t),"<pi^2>",1,"1",eps);
	final double pia=QVM.getAverage(pi,abc,pt0);
	sigma2tot_ii+=pia*pia;
	TMatr.sumP1(gcoefs,int_pi,pia);
	DeprecatedHelperTMatr.sumP2(kcoefs,pi);
	DeprecatedHelperTMatr.sumP2(dissp,int_pi);
	TMatr.sumP1(dp,Polynomials.mult2Pol(int_pi,pi),1);
    }
    UnitTests.checkDoubleRel(sigma2tot_ii,"sigma2tot_ii",q.sigma2tot_ii,"q.sigma2tot_ii",eps);
    UnitTests.checkDoubleArrayRel(gcoefs,"gcoefs",q.gcoefs,"q.gcoefs",eps);
    UnitTests.checkDoubleArrayRel(kcoefs,"kcoefs",q.kcoefs,"q.kcoefs",eps);
    UnitTests.checkDoubleArrayRel(dissp,"dissp",q.dissp,"q.dissp",eps);
    UnitTests.checkDoubleArrayRel(dp,"dp",TMatr.getVectorScaled(Polynomials.getPolynomialDifferentiated(q.dissp),0.5),"0.5 d/dp q.dissp",eps);
  }

  if(true)
  {
    final double [][] pdp0=QVM.getPDPCoefs(abc_t,neq_p-1);
    UnitTests.checkDoubleArrayRel(q.kcoefs,"q.kcoefs",pdp0[0],"pdp0[0]",eps);
    UnitTests.checkDoubleArrayRel(q.dissp,"q.dissp",pdp0[1],"pdp0[1]",eps);
    final double [][] pdp0v=QVM.getPDPCoefs(abc,neq_p-1);
    UnitTests.checkDoubleArrayRel(q.kvcoefs,"q.kvcoefs",pdp0v[0],"pdp0v[0]",eps);
    UnitTests.checkDoubleArrayRel(q.dvissp,"q.dvissp",pdp0v[1],"pdp0v[1]",eps);
  }

  System.err.println("QVMpi:checkMatchingOrthogonalPolynomials: tests OK  neq_p="+neq_p+" eps="+eps+" epssum="+epssum);
}



private static void checkCalculations(final int tsize,final java.util.Random r,final int neq_p,final double eps,final double epssum)
{
    final Trade [] tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
    final long tau=10000L*tb.length/30,currenttime=tb[tb.length-1].time+1000;
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println("QVMpi:checkCalculations(): test with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+
		       " neq_p="+neq_p);

    final com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG rf=
	new com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG();

    checkMatchingQATradeIP(neq_p,r,currenttime,tau,tb,tsize,eps,epssum,rf);
    checkMatchingLinsystemValues(neq_p,r,currenttime,tau,tb,tsize,eps,epssum,rf);
    checkMatchingOrthogonalPolynomials(neq_p,r,currenttime,tau,tb,tsize,eps,epssum,rf);

    System.err.println("QVMpi:checkCalculations(): OK with N points="+tsize+
		       " currenttime="+currenttime+" tau="+tau+
		       " neq_p="+neq_p);
}

public static void main(String [] args)
{
    System.err.println("QVMpi::main(): starting tests; ver=$Id: QVMpi.java,v 1.88 2019/01/07 19:53:45 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79091);
    final double eps=1e-7,epssum=1e-5;
    for(int neq_p=1;neq_p<=5;neq_p++)
    {
	for(int i=0;i<118;i++)
	{
	    checkCalculations(10010+r.nextInt(10010),r,neq_p,eps,epssum);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println("QVMpi::main(): tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: QVMpi.java,v 1.88 2019/01/07 19:53:45 mal Exp $");
}


} // END of class QVMpi
