package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.ProbabilityCorrelation;

/** Polynomial basis space version.
 *  @version $Id: QkPnLpi_PBasis.java,v 1.25 2018/07/27 01:08:54 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QkPnLpi_PBasis {

private final OrthogonalPolynomialsMonomialsBasis M;
private final QVMData odata;
private final VolMatrix DP_VolMatrix=new VolMatrix(){
	public double [] getDPsi(final double [] psi){return getDP(psi);}
    };

public final ImpactIStatesExample ist;
public final PnLDiagProj pnldiag;
public final PnLVarVolatilitypi pnlvarvol;
public final PnLdISkewness pnldiskewness_nodI,pnldiskewness;
public final PnLVar3Mupi pnlvar3;
public final PnLVar3Mu pnlvar3mu;
public final PnLVarMupi pnlvarmupi;
public final ProbabilityCorrelation PCORR_average;
public final PnLSimpleSkewness pnlss;

private QkPnLpi_PBasis(final int n,final QVMData odata)
{
    this.odata=odata;
    this.M=getABasis(getPLast_NOOFFSET()-odata.p_offset);
    final MomentsData mm=MomentsData.get_PriceMonomials_MomentsData(n,odata,this.M);
    if(true) { // trivial probability correlation (with fixed tau).
	 final MomentsData mm2=MomentsData.get_PriceMonomials_MomentsData(2,odata,this.M);
	 this.pnlss=new PnLSimpleSkewness(mm2,M);
    }
    this.ist=ImpactIStatesExample.get_ImpactIStatesExample(mm,this.M,DP_VolMatrix,0);
    this.pnldiag=PnLDiagProj.get_PnLDiagProj(mm,this.M,0);
    this.pnlvarvol=PnLVarVolatilitypi.get_PnLVarVolatilitypi(mm,this.M,DP_VolMatrix,0);
    this.pnldiskewness_nodI=PnLdISkewness.get_PnLdISkewness(mm,M,false,0);
    this.pnldiskewness=PnLdISkewness.get_PnLdISkewness(mm,M,true,0);
    this.pnlvar3=PnLVar3Mupi.get_PnLVar3Mupi(mm,this.M,DP_VolMatrix,0);
    this.pnlvar3mu=PnLVar3Mu.get_PnLVar3Mu(mm,M,0);
    final double [] rm[]=QmQVMpi.get_ti_Moms(2*n+1,odata),tmoms=rm[0],imoms=rm[1];
    pnlvarmupi=PnLVarMupi.get_PnLVarMupi(n,odata.p_offset,tmoms,imoms,M,0);
    PCORR_average=new ProbabilityCorrelation(imoms[0],imoms[1],imoms[2],
					     imoms[1],imoms[2],imoms[3],
					     tmoms[0],tmoms[1],tmoms[2],true);
}
double getPLast_NOOFFSET(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
private OrthogonalPolynomialsMonomialsBasis getABasis(final double pl){return new OrthogonalPolynomialsMonomialsBasisX(1,-pl);}
private double [] getDP(final double [] psi) {return Polynomials.getPolynomialDifferentiated(psi);}


public static QkPnLpi_PBasis calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final QkPnLpi_PBasis ref)
{
    final int nvm=2,ntm=1,npm=2*neq+2;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     ntm,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new QkPnLpi_PBasis(neq,o);
}



static class QkPnLpi_PBasisTest extends QTest {

QkPnLpi_PBasis getQkPnLpi_PBasis(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
QkQVMpi getQkQVMpi(){return QkQVMpi.calculate(neq,currenttime,tau,tb,tb.length,null);}
ImpactQVMLaguerre getImpactQVMLaguerre(){return ImpactQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

QkPnLpi_PBasisTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

void test_QkQVMpi()
{
    final QkPnLpi_PBasis q=getQkPnLpi_PBasis();
    final int n=q.ist.eI.neq;
    final MomentsData mm=MomentsData.get_PriceMonomials_MomentsData(n,q.odata,q.M);
    final ImpactIStatesExample tist=ImpactIStatesExample.get_ImpactIStatesExample(mm,q.M,q.DP_VolMatrix,eps);
    final QkQVMpi s=getQkQVMpi();
    ImpactIStatesExample.matchImpactIStatesExample(q.ist,s.ist,eps);
    ImpactIStatesExample.matchImpactIStatesExample(q.ist,tist,eps);
    ImpactIStatesExample.matchPnLDiagProj(q.ist,q.pnldiag,eps);
    ImpactIStatesExample.matchPnLDiagProj(s.ist,q.pnldiag,eps);
    ImpactIStatesExample.matchPnLVarVolatilitypi(q.ist,q.pnlvarvol,eps);
    ImpactIStatesExample.matchPnLVarVolatilitypi(s.ist,q.pnlvarvol,eps);
    ImpactIStatesExample.matchPnLVarVolatilitypi(s.ist,PnLVarVolatilitypi.get_PnLVarVolatilitypi(mm,q.M,q.DP_VolMatrix,eps),eps);
    PnLdISkewness.matchPnLdISkewness(q.pnldiskewness_nodI,s.pnldiskewness_nodI,eps);
    PnLdISkewness.matchPnLdISkewness(q.pnldiskewness,s.pnldiskewness,eps);

    ImpactIStatesExample.matchPnLVar3Mupi(s.ist,q.pnlvar3,eps);
    ImpactIStatesExample.matchPnLVar3Mupi(q.ist,PnLVar3Mupi.get_PnLVar3Mupi(mm,q.M,q.DP_VolMatrix,eps),eps);

    PnLVarMupi.matchPnLVarMupi(q.pnlvarmupi,s.pnlvarmupi,eps);
    PnLSimpleSkewness.matchPnLSimpleSkewness(q.pnlss,s.pnlss,eps);
    System.err.println(name+":QkPnLpi_PBasisTest:test_QkQVMpi(): test OK neq="+neq+" eps="+eps);
}

void test_Common()
{
    final int n=neq;
    final QkPnLpi_PBasis q=getQkPnLpi_PBasis();
    final MomentsData mm=MomentsData.get_PriceMonomials_MomentsData(n,q.odata,q.M);
    ImpactIStatesExample.matchPnLDiagProj(q.ist,PnLDiagProj.get_PnLDiagProj(mm,q.M,eps),eps);

    if(true)
    {
	// test some moments
	final ImpactQVMLaguerre ql=getImpactQVMLaguerre();
	final MomentsData mmql=MomentsData.get_MomentsData(n,ql.M,ql.odata,ql.getAccess());
	System.err.println("mm.QQV[0]="+mm.QQV[0]+" mmql.QQV[0]="+mmql.QQV[0]+" have to match approximately");
	UnitTests.checkDoubleRel(mm.QQV[0],"mm.QQV[0]",mmql.QQV[0],"mmql.QQV[0]",1e-2/* eps */);

	// test mm.p_last has an offset included
	UnitTests.checkDoubleRel(q.M.getRadauRoot(),"q.M.getRadauRoot()",mm.p_last,"mm.p_last",eps);
    }

    final double [][] rm=QmQVMpi.get_ti_Moms(2*n+1,q.odata);
    PnLVarMupi.get_PnLVarMupi(n,q.odata.p_offset,rm[0],rm[1],q.M,eps);

    System.err.println(name+":QkPnLpi_PBasisTest:test_Common(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":QkPnLpi_PBasisTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_QkQVMpi();

    System.err.println(name+":QkPnLpi_PBasisTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":QkPnLpi_PBasisTest: starting tests; ver=$Id: QkPnLpi_PBasis.java,v 1.25 2018/07/27 01:08:54 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":QkPnLpi_PBasisTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: QkPnLpi_PBasis.java,v 1.25 2018/07/27 01:08:54 mal Exp $");
}
} // END OF class QkPnLpi_PBasisTest

public static void main(String [] args)
{
    new QkPnLpi_PBasisTest("QkPnLpi_PBasis",new java.util.Random(3222),1e-4).runTests(6);
} // END main




} // END of class QkPnLpi_PBasis
