package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.realtimebook.*;

/** New assymetry test.
 *  @version $Id: QkQVM.java,v 1.21 2022/07/28 12:27:19 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class QkQVM<DTYPE> {

final DTYPE odata;
final DataInterfaceToMoments<DTYPE> access;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;

final double I_0,I_L,I_H;
final EAssym aL,aH,aC,a0,aVl,aVh,aIb,aFutureI,aFutureIpl;
final EVXData Vt;


QkQVM(final int n,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();
    this.M=getABasis();

    final int nmoms=2*n-1;
    final double [] imom=new double [nmoms],tmom=new double [nmoms],p1imom=new double [nmoms],p2imom=new double [nmoms],p3imom=new double [nmoms],p4imom=new double [nmoms];
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(p1imom,1,access.M_I,odata,nmoms);
    access.setM(p2imom,2,access.M_I,odata,nmoms);
    access.setM(p3imom,3,access.M_I,odata,nmoms);
    access.setM(p4imom,4,access.M_I,odata,nmoms);
    access.setMT(tmom,nmoms);

    final double [] basis=getBasis(n,imom,tmom,M);
    final int i0=0,iL=1,iH=n-1;
    this.I_0=basis[i0];
    this.I_L=basis[iL];
    this.I_H=basis[iH];

    this.aL=new EAssym(EVXData.getPsiFromBasis(iL,basis,n),imom,p1imom,p2imom,p3imom,p4imom,M);
    this.aH=new EAssym(EVXData.getPsiFromBasis(iH,basis,n),imom,p1imom,p2imom,p3imom,p4imom,M);
    this.aC=new EAssym(new double []{1},imom,p1imom,p2imom,p3imom,p4imom,M);
    this.a0=new EAssym(EVXData.getPsiFromBasis(i0,basis,n),imom,p1imom,p2imom,p3imom,p4imom,M);

    final double [] Vmom=getFmom(imom),p1Vmom=getFmom(p1imom),p2Vmom=getFmom(p2imom),p3Vmom=getFmom(p3imom),p4Vmom=getFmom(p4imom);
    this.Vt=EVXData.getEVXDataFromQQ(n,M.B.getQQMatr(n,Vmom),M.B.getQQMatr(n,tmom),M);
    this.aVl=new EAssym(this.Vt.getPsi(0),Vmom,p1Vmom,p2Vmom,p3Vmom,p4Vmom,M);
    this.aVh=new EAssym(this.Vt.getPsi(n-1),Vmom,p1Vmom,p2Vmom,p3Vmom,p4Vmom,M);
    final EVXData [] aIf=getPsiIFuture(n,imom,tmom,M);
    final EVXData If=aIf[0],Ib=aIf[1];
    this.aIb=new EAssym(Ib.psiH,imom,p1imom,p2imom,p3imom,p4imom,M);
    this.aFutureI=new EAssym(If.psiH,imom,p1imom,p2imom,p3imom,p4imom,M);
    final double difx=(If.s0-Ib.s0)/If.o_o,pl=access.getPLast(odata);
    final double [] m0=M.B.getX0Moments(nmoms,If.x0);
    //System.err.println("If.sH="+If.sH+" pl="+pl);
    this.aFutureIpl=new EAssym(If.psiH,
			       TMatr.sumAcB(imom,difx,m0),
			       TMatr.sumAcB(p1imom,pl*difx,m0),
			       TMatr.sumAcB(p2imom,pl*pl*difx,m0),
			       TMatr.sumAcB(p3imom,Math.pow(pl,3)*difx,m0),
			       TMatr.sumAcB(p4imom,Math.pow(pl,4)*difx,m0),M);


}

double [] getFmom(final double [] moms)
{
    final double [] fmoms=new double [moms.length];
    access.setFMoments(fmoms,moms,fmoms.length);
    TMatr.scaleVectorInPlace(fmoms,-1);
    return fmoms;
}

abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();

double [] getBasis(final int n,final double [] imom,final double [] tmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    return EdComm.getIRadauBasisPsi0(n,imom,tmom,M);
}

/** psi, corresponding to future I.
 */
EVXData [] getPsiIFuture(final int n,final double [] imom,final double [] tmom,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] QQt=M.B.getQQMatr(n,tmom),QQi=M.B.getQQMatr(n,imom);
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double I_0=I.s0,I_0_feq=I.sH,dI=I_0_feq-I_0;
    final double [] QQ0=TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),QQi_feq=TMatr.sumAcB(QQi,dI/I.o_o,QQ0);
    final EVXData If=EVXData.getEVXDataFromQQ(n,QQi_feq,QQt,M);
    return new EVXData[]{If,I};
}

static abstract class QkQVMTest<DTYPE> extends QTest {

abstract QkQVM<DTYPE> getQkQVM();


QkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_Common()
{
    final QkQVM<DTYPE> q=getQkQVM();
    final int n=q.Vt.neq;
    q.aL.test_Self(eps);
    q.aH.test_Self(eps);
    q.aC.test_Self(eps);
    q.a0.test_Self(eps);
    q.aVl.test_Self(eps);
    UnitTests.checkDoubleRel(q.aVl.I,"q.aVl.I",q.Vt.ev[0],"q.Vt.ev[0]",eps);
    q.aVh.test_Self(eps);
    UnitTests.checkDoubleRel(q.aVh.I,"q.aVh.I",q.Vt.ev[n-1],"q.Vt.ev[n-1]",eps);
    q.aFutureI.test_Self(eps);

    System.err.println(name+":QkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}


void test_Extra(){}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":QkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Extra();

    System.err.println(name+":QkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":QkQVMTest: starting tests; ver=$Id: QkQVM.java,v 1.21 2022/07/28 12:27:19 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":QkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: QkQVM.java,v 1.21 2022/07/28 12:27:19 mal Exp $");
}
} // END OF class QkQVMTest




}// END OF class QkQVM
