package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: QkQVMMonomials.java,v 1.4 2015/03/02 20:58:41 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QkQVMMonomials extends QkQVM<QVMData> {
QkQVMMonomials(final int neq,final QVMData odata){super(neq,odata);}


public static QkQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final QkQVMMonomials ref)
{
    final int nvm=1,npm=5;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new QkQVMMonomials(neq,o);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}

static class QkQVMMonomialsTest extends QkQVMTest<QVMData> {
QkQVMMonomialsTest(final long initrandom)
{
    super("QkQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
QkQVMMonomials getQkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

@Override
void test_Common()
{
    super.test_Common();
    test_Laguerre();
}
void test_Laguerre()
{
    final QkQVMMonomials q=getQkQVM();
    final QkQVMLaguerre s=QkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);
    q.aL.match_EAssym(s.aL,eps,false);
    q.aH.match_EAssym(s.aH,eps,false);
    q.aC.match_EAssym(s.aC,eps,false);
    q.a0.match_EAssym(s.a0,eps,false);
    q.aVl.match_EAssym(s.aVl,eps,false);
    q.aVh.match_EAssym(s.aVh,eps,false);
    System.err.println(name+":QkQVMTest:test_Laguerre(): test OK neq="+neq+" eps="+eps);
}


} // END of class QkQVMMonomialsTest

public static void main(String [] args)
{
    new QkQVMMonomialsTest(1117L).runTests(5);
} // END main



} // END OF class QkQVMMonomials
