package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.PolynomialsRatio;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.Quadrature;

/** Polynomial basis space version.
 *  @version $Id: QkQVMpi.java,v 1.57 2022/11/20 12:41:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QkQVMpi {


private static final QVMDataLDirectAccess access=new QVMDataLDirectAccess();
private final OrthogonalPolynomialsMonomialsBasis M;

final Qmu pmL,pmH,pmHf;
final EVXData I,If;
final double varI0p,varI1p,varI2p;
final Skewness SKf_pv;

final double [] all_p_t,all_p_v;
final double i0,Dpi0;
private final QVMDataL odata;

final ImpactIStatesExample ist;
public final PnLdISkewness pnldiskewness_nodI,pnldiskewness;
public final PnLVar3Mupi pnlvar3;
public final PnLVarMupi pnlvarmupi;
public final PnLSimpleSkewness pnlss;
private final VolMatrix DP_VolMatrix=new VolMatrix(){
	public double [] getDPsi(final double [] psi){return getDP(psi);}
    };

private QkQVMpi(final int n,final QVMDataL odata)
{
    this.odata=odata;
    this.M=getABasis(access.getPLast(odata)); // getPLast does have p_offset included
    final MomentsData mm=MomentsData.get_PriceMonomials_MomentsData(n,odata,access,this.M);
    this.ist=ImpactIStatesExample.get_ImpactIStatesExample(mm,this.M,DP_VolMatrix,0);
     if(true) { // trivial probability correlation (with fixed tau).
	 final MomentsData mm2=MomentsData.get_PriceMonomials_MomentsData(2,odata,access,this.M);
	 this.pnlss=new PnLSimpleSkewness(mm2,M);
    }
    this.pnldiskewness_nodI=PnLdISkewness.get_PnLdISkewness(mm,M,false,0);
    this.pnldiskewness=PnLdISkewness.get_PnLdISkewness(mm,M,true,0);
    this.pnlvar3=PnLVar3Mupi.get_PnLVar3Mupi(mm,this.M,DP_VolMatrix,0);

    // code below is for unit test only
    final double [][] moms=getMoms(2*n+2,access,odata);
    final double [] mom_t=moms[0],mom_v=moms[1];
    pnlvarmupi=PnLVarMupi.get_PnLVarMupi(n,access.getPOffset(odata),mom_t,mom_v,M,0);
    final double [] QQv=M.B.getQQMatr(n,mom_v),QQt=M.B.getQQMatr(n,mom_t);
    this.I=EVXData.getEVXDataFromQQ(n,QQv,QQt,M);
    final double [] psi=this.I.psiH,Dpsi=getDP(psi),D2psi=getDP(Dpsi);
    final double [] rq=RayleighQuotient.getRQVariationValues(psi,Dpsi,D2psi,QQv,QQt,n);
    varI0p=rq[0];
    varI1p=rq[1];
    varI2p=rq[2];




    final double [] mom_pv=M.B.getXQkMomentsFromQkMoments(mom_v),mom_p2v=M.B.getXQkMomentsFromQkMoments(mom_pv),mom_p3v=M.B.getXQkMomentsFromQkMoments(mom_p2v),
	pQQv=M.B.getQQMatr(n,mom_pv),p2QQv=M.B.getQQMatr(n,mom_p2v),p3QQv=M.B.getQQMatr(n,mom_p3v),
	QQ0=TMatr.getQQaa(M.B.getX0Moments(n,I.x0)),
	mom_pt=M.B.getXQkMomentsFromQkMoments(mom_t),pQQt=M.B.getQQMatr(n,mom_pt),p2QQt=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(mom_pt));
    this.all_p_t=M.EV.getGEV_Generalized(n,pQQt,QQt,false);
    this.all_p_v=M.EV.getGEV_Generalized(n,pQQv,QQv,false);
    final double dI=(I.sH-I.s0)/I.o_o;
    final double [] QQfv=TMatr.sumAcB(QQv,dI,QQ0),pQQfv=TMatr.sumAcB(pQQv,I.x0*dI,QQ0),
	p2QQfv=TMatr.sumAcB(p2QQv,Math.pow(I.x0,2)*dI,QQ0),p3QQfv=TMatr.sumAcB(p3QQv,Math.pow(I.x0,3)*dI,QQ0);
    this.If=EVXData.getEVXDataFromQQ(n,QQfv,QQt,M);
    this.pmHf=new Qmu(If.psiH,If.sH,true,pQQfv,QQfv,p2QQfv,pQQt,QQt,p2QQt);
    this.SKf_pv=Skewness.getSkewnessFromMonomialMoments(
				 TMatr.getAMatrA(If.psiH,QQfv,n),
				 TMatr.getAMatrA(If.psiH,pQQfv,n),
				 TMatr.getAMatrA(If.psiH,p2QQfv,n),
				 TMatr.getAMatrA(If.psiH,p3QQfv,n));

    this.pmL=new Qmu(I.psiL,I.sL,false,pQQv,QQv,p2QQv,pQQt,QQt,p2QQt);
    this.pmH=new Qmu(I.psiH,I.sH,true,pQQv,QQv,p2QQv,pQQt,QQt,p2QQt);
    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	QQn=TMatr.mulMatrices(n,new double [][]{IQQt,QQv,IQQt});
    final double [] kn=M.B.getKK(n,QQn),kd=M.B.getKK(n,IQQt);
    final double kn0=M.B.getD0(kn,mm.p_last),kd0=M.B.getD0(kd,mm.p_last),Dkn0=M.B.getD1(kn,mm.p_last),Dkd0=M.B.getD1(kd,mm.p_last);
    this.i0=kn0/kd0;
    this.Dpi0=Dkn0/kn0-Dkd0/kd0;

    smom_t=mom_t;
    smom_v=mom_v;
    sQQv=QQv;
    sQQt=QQt;
    spQQv=pQQv;
    spQQt=pQQt;

}
final double [] sQQv,sQQt,spQQv,spQQt,smom_t,smom_v;
double getPOffset(){return access.getPOffset(odata);}
private OrthogonalPolynomialsMonomialsBasis getABasis(final double pl){return new OrthogonalPolynomialsMonomialsBasisX(1,-pl);}
private double [] getDP(final double [] psi) {return Polynomials.getPolynomialDifferentiated(psi);}



class Qmu {
    final double p_t,p_v,lam,lamp,stdev2_p_t,stdev2_p_v;
    final double dIdp;
    final double [] px,Ix,mu0,mu3;
    final boolean flag_neg;
    Qmu(final double [] psi,final double lam,final boolean flag_neg,
	final double [] pQQv,final double [] QQv,final double [] p2QQv,
	final double [] pQQt,final double [] QQt,final double [] p2QQt)
    {
	final int n=psi.length;
	this.lam=lam;
	this.flag_neg=flag_neg;
	final double wnt=TMatr.getAMatrA(psi,QQt,n),wnv=TMatr.getAMatrA(psi,QQv,n),
	    pwnt=TMatr.getAMatrA(psi,pQQt,n),pwnv=TMatr.getAMatrA(psi,pQQv,n);
	this.lamp=wnv/wnt;
	this.p_t=pwnt/wnt;
	this.stdev2_p_t=(TMatr.getAMatrA(psi,p2QQt,n)+p_t*(-2*pwnt+p_t*wnt))/wnt;
	this.p_v=pwnv/wnv;
	this.stdev2_p_v=(TMatr.getAMatrA(psi,p2QQv,n)+p_v*(-2*pwnv+p_v*wnv))/wnv;
	this.px=M.getXNodes(psi).re_root;
	this.Ix=new double [this.px.length];
	this.mu0=new double [px.length];
	this.mu3=new double [px.length];
	for(int k=0;k<this.px.length;k++)
	{
	    final double pk=this.px[k];
	    final double [] r=M.B.sdiv1(psi,pk).q;
	    TMatr.scaleVectorInPlace(r,1/M.B.getD0(r,pk));
	    mu0[k]=(TMatr.getAMatrA(r,QQv,n)-lam*TMatr.getAMatrA(r,QQt,n));
	    this.Ix[k]=(flag_neg?-1:1)*mu0[k];
	    mu3[k]=(pwnv-lam*pwnt)/Math.pow(M.B.getD1(psi,pk),2);
	}
	this.dIdp=this.px.length<=0?Double.NaN:((Ix[this.px.length-1]-Ix[0])/(px[this.px.length-1]-px[0]));
    }
    void test_Self(final double eps,final double [] vmom,final double [] tmom)
    {
	UnitTests.checkDoubleRel(lam,"lam",lamp,"lamp",eps);
	UnitTests.checkDoubleRel(i0,"i0",I.s0,"I.s0",eps);
	final int n=px.length;
	final double [] mu=new double [2*n];
	for(int k=0;k<mu.length;k++)
	{
	    mu[k]=(flag_neg?-1:1)*(vmom[k]-lam*tmom[k]);
	}
	final Quadrature Q=M.getGaussQuadratureFromMoments(n,mu);
	final double [] qp=Q.x,qw=Q.w;
	System.err.println("px="+java.util.Arrays.toString(px));
	System.err.println("Ix="+java.util.Arrays.toString(Ix));
	UnitTests.checkDoubleArrayRel(px,"px",qp,"qp",eps);
	UnitTests.checkDoubleArrayRel(Ix,"Ix",qw,"qw",eps);

	System.err.println("QkQVMpi:Qmu:test_Self: test OK n="+n+" eps="+eps);
    }
} // END OF class Qmu


static <T> double [][] getMoms(final int nmoms,final DataInterfaceToMoments<T> access,final T odata)
{
    final double [] r=new double [1],R=new double [1];
    final double [] tmom=new double [nmoms],vmom=new double [nmoms],Vmom=new double [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	access.setM(r,k,access.M_T,odata,1);
	tmom[k]=r[0];
	access.setM(r,k,access.M_I,odata,1);
	vmom[k]=r[0];
	access.setFMoments(R,r,1);
	Vmom[k]=R[0];
    }
    return new double [][]{tmom,vmom,Vmom};
}


public static QkQVMpi calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final QkQVMpi ref)
{
    final int ntm=1,npm=2*neq+2;
    final QVMDataL o=QVMDataL.calculateQVMDataL(ntm,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new QkQVMpi(neq,o);
}



static class QkQVMpiTest extends QTest {

QkQVMpi getQkQVMpi(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
QVM.QVMVT getQVM_QVMVT(){
    return QVM.QVMVT.calculateQVMVT(neq,currenttime,tau,tb,tb.length,
				     new com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG(),
				     null);
}

QkQVMpiTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

void test_ImpactIStatesExample()
{
    final QkQVMpi q=getQkQVMpi();
    final int n=q.I.neq;
    EVXData.matchEVXData(q.I,q.ist.eI,eps);
    UnitTests.checkDoubleRel(q.pmH.p_v,"q.pmH.p_v",q.ist.p_IH,"q.ist.p_IH",eps);
    UnitTests.checkDoubleRel(q.i0,"q.i0",q.ist.eI.s0,"q.ist.eI.s0",eps);
    UnitTests.checkDoubleRel(q.varI0p,"q.varI0p",q.ist.eI.ev[n-1],"q.ist.eI.ev[n-1]",eps);
    UnitTests.checkDoubleRel(q.varI2p,"q.varI2p",q.ist.var2_I[n-1],"q.ist.var2_I[n-1]",eps);
    ImpactIStatesExample.matchPnLVar3Mupi(q.ist,q.pnlvar3,eps);
    final MomentsData mm=MomentsData.get_PriceMonomials_MomentsData(n,q.odata,q.access,q.M);
    ImpactIStatesExample.get_ImpactIStatesExample(mm,q.M,q.DP_VolMatrix,eps);
    UnitTests.checkDoubleRel(q.pmH.stdev2_p_v*q.pmH.lamp,"stdev2 H",q.ist.p2volI_I[n-1],"q.ist.p2volI_I[n-1]",eps);
    UnitTests.checkDoubleRel(q.pmL.stdev2_p_v*q.pmL.lamp,"stdev2 L",q.ist.p2volI_I[0],"q.ist.p2volI_I[0]",eps);
    ImpactIStatesExample.matchPnLVar3Mupi(q.ist,q.pnlvar3,eps);

    System.err.println(name+":QkQVMpiTest.test_ImpactIStatesExample(): test OK neq="+neq+" eps="+eps);
}

void test_IPnLdISkewness()
{
    final QkQVMpi q=getQkQVMpi();
    final int n=q.I.neq;
    EVXData.matchEVXData(q.I,q.pnldiskewness_nodI.I,eps);
    UnitTests.checkDoubleRel(q.If.s0,"q.If.s0",q.I.sH,"q.I.sH",eps);

    //EVXData.matchEVXData(q.If,q.pnldiskewness.I,eps);
    UnitTests.checkDoubleRel(q.pmH.p_v,"q.pmH.p_v",q.pnldiskewness_nodI.IHf_S.xa,"q.pnldiskewness_nodI.IHf_S.xa",eps);
    UnitTests.checkDoubleRel(q.pmHf.lamp,"q.pmHf.lamp",q.pnldiskewness.IHf_S.w1+q.pnldiskewness.IHf_S.w2,"q.pnldiskewness.IHf_S.w1+q.pnldiskewness.IHf_S.w2",eps);
    UnitTests.checkDoubleRel(q.pmHf.p_v,"q.pmHf.p_v",q.pnldiskewness.IHf_S.xa,"q.pnldiskewness.IHf_S.xa",eps);
    Skewness.matchSkewness(q.SKf_pv,q.pnldiskewness.IHf_S,eps);

    System.err.println(name+":QkQVMpiTest.test_IPnLdISkewness(): test OK neq="+neq+" eps="+eps);
}

void test_QVM_QVMVT()
{
    final QkQVMpi q=getQkQVMpi();
    final int n=q.I.neq;
    final QVM.QVMVT s=getQVM_QVMVT();
    {
	final double dof=s.vm.odata.p_offset-q.getPOffset();
	final double [] pp=s.vm.p_real_roots.clone();
	for(int k=0;k<pp.length;k++){pp[k]+=dof;}
	UnitTests.checkDoubleArrayRel(q.all_p_v,"q.all_p_v",pp,"V:pp",eps);
    }
    {
	final double dof=s.tm.odata.p_offset-q.getPOffset();
	final double [] pp=s.tm.p_real_roots.clone();
	for(int k=0;k<pp.length;k++){pp[k]+=dof;}
	UnitTests.checkDoubleArrayRel(q.all_p_t,"q.all_p_t",pp,"T:pp",eps);
    }
    System.err.println(name+":QkQVMpiTest:test_QVM_QVMVT(): test OK neq="+neq+" eps="+eps);
}

void test_Common()
{
    final int n=neq;
    final QkQVMpi q=getQkQVMpi();

    q.pmL.test_Self(eps,q.smom_v,q.smom_t);
    q.pmH.test_Self(eps,q.smom_v,q.smom_t);
    PnLVarMupi.get_PnLVarMupi(n,q.access.getPOffset(q.odata),q.smom_t,q.smom_v,q.M,eps);
    UnitTests.checkDoubleArrayRel1(q.pmL.mu0,"q.pmL.mu0",q.pnlvarmupi.mu0,"q.pnlvarmupi.mu0",eps);
    UnitTests.checkDoubleArrayRel1(q.pmL.mu3,"q.pmL.mu3",q.pnlvarmupi.mu3,"q.pnlvarmupi.mu3",eps);

    System.err.println(name+":QkQVMpiTest:test_Common(): test OK neq="+neq+" eps="+eps);
}

void test_Vars()
{
    final QkQVMpi q=getQkQVMpi();
    final QVM.QVMVT s=getQVM_QVMVT();
    final int n=neq;
    final double [] QQt=s.tm.QQ,QQi=s.vm.QQ;
    final double [] basis=q.M.EV.getGEV_Generalized(n,QQi,QQt,true);
    UnitTests.checkDoubleArrayRel(java.util.Arrays.copyOf(basis,n),"I from QVM.QVMVT",q.I.getEValues(),"I from QkQVMpi",eps);
    final double off_s=s.odata.p_offset,off_q=q.odata.p_offset;
    System.err.println("off_s="+off_s+" off_q="+off_q);
    for(int k=0;k<n;k++)
    {
	final double [] spsi=EVXData.getPsiFromBasis(k,basis,n),qpsi=q.I.getPsi(k);
	final double [] spsi_offq=Polynomials.getPolynomialShifted(spsi,off_q-off_s);
	UnitTests.checkDoubleArrayRel1(EVXData.getMSign(qpsi),"qpsi",EVXData.getMSign(spsi_offq),"spsi_offq",eps);
    }

    UnitTests.checkDoubleRel(q.varI0p,"q.varI0p",q.I.sH,"q.I.sH",eps);
    UnitTests.checkDoubleRel(q.varI1p,"q.varI1p",0,"0",eps);

    final int iX=n-1;
    final double [] psi=EVXData.getPsiFromBasis(iX,basis,n),psi2=Polynomials.mult2Pol(psi,psi),D_psi2=Polynomials.getPolynomialDifferentiated(psi2),D2_psi2=Polynomials.getPolynomialDifferentiated(D_psi2),kt=new double [0],kf=new double [0];

    final PolynomialsRatio pr=new PolynomialsRatio(kt,kf,new com.polytechnik.utils.Monomials(){
	    public double getD0(final double [] m,final double x){return TMatr.mulVVn1(psi2,m==kt?s.odata.tmoments_vol_x_tau_x_p:s.odata.vmoments_vol_x_tau_x_p);}
	    public double getD1(final double [] m,final double x){return TMatr.mulVVn1(D_psi2,m==kt?s.odata.tmoments_vol_x_tau_x_p:s.odata.vmoments_vol_x_tau_x_p);}
	    public double getD2(final double [] m,final double x){return TMatr.mulVVn1(D2_psi2,m==kt?s.odata.tmoments_vol_x_tau_x_p:s.odata.vmoments_vol_x_tau_x_p);}
	});
    UnitTests.checkDoubleRel(q.varI2p,"q.varI2p",pr.getD2F(Double.NaN),"pr.getD2F",eps);
    UnitTests.checkDoubleRel(q.varI1p,"q.varI1p",0,"0",eps);
    UnitTests.checkDoubleRel(q.varI0p,"q.varI0p",pr.getF(Double.NaN),"pr.getF",eps);
    if(q.varI2p>0) throw new RuntimeException("q.varI2p="+q.varI2p+">0");


    System.err.println(name+":QkQVMpiTest:test_Vars(): test OK neq="+neq+" eps="+eps);
}



void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":QkQVMpiTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_QVM_QVMVT();
    test_Vars();
    test_ImpactIStatesExample();
    test_IPnLdISkewness();

    System.err.println(name+":QkQVMpiTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":QkQVMpiTest: starting tests; ver=$Id: QkQVMpi.java,v 1.57 2022/11/20 12:41:33 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":QkQVMpiTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: QkQVMpi.java,v 1.57 2022/11/20 12:41:33 mal Exp $");
}
} // END OF class QkQVMpiTest

public static void main(String [] args)
{
    new QkQVMpiTest("QkQVMpi",new java.util.Random(3222),1e-4).runTests(6);
} // END main




} // END of class QkQVMpi
