package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.RayleighQuotient;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.SortArrays;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.PolynomialRootsFinder;

/** Polynomial basis space version.
 *  @version $Id: QmQVMpi.java,v 1.32 2022/07/28 12:27:19 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class QmQVMpi {
private static final boolean FLAG_DEBUG=false;
private double eps=FLAG_DEBUG?1e-5:Double.NaN;

private final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;

final EVXData I;
private final QVMData odata;
final double p_offset,vartI,varvI,varV,p_v_IH,impD,impU;
final double i_max,p_i_max,pa_i,w_max,p_w_max,pa_w,vp_IH,v2p_IH;
final double [] p,w,i;
private QmQVMpi(final int n,final QVMData odata)
{
    this.odata=odata;
    this.p_offset=getPOffset();
    final double offset_to_pl=getPLast()-p_offset;
    this.M=getABasis(offset_to_pl);
    final double [][] it_moms=get_ti_Moms(2*n+1,odata),V_moms=getMoms(2*n+1,odata);
    final double [] mom_t=it_moms[0],mom_v=it_moms[1],mom_tV=V_moms[0],mom_vV=V_moms[1];
    final double [] QQv=M.B.getQQMatr(n,mom_v),QQt=M.B.getQQMatr(n,mom_t),
	QQtV=M.B.getQQMatr(n,mom_tV),QQvV=M.B.getQQMatr(n,mom_vV);
    this.I=EVXData.getEVXDataFromQQ(n,QQv,QQt,M);
    final double [] psi=I.psiH,Dpsi=M.B.getDifferentiated(psi),dpsi=getDpsiOrthoginal(psi,QQt,n);
    this.vp_IH=2*TMatr.getAMatrB(psi,Dpsi,QQt,n);
    this.v2p_IH=2*(TMatr.getAMatrA(Dpsi,QQv,n)-I.sH*TMatr.getAMatrA(Dpsi,QQt,n));
    this.vartI=2*TMatr.getAMatrB(psi,dpsi,QQtV,n);
    this.varvI=2*TMatr.getAMatrB(psi,dpsi,QQvV,n);
    this.varV=TMatr.getAMatrA(psi,QQvV,n)-I.sH*TMatr.getAMatrA(psi,QQtV,n);
    this.p_v_IH=TMatr.mulVVn1(M.B.mult2Pol(M.getRadauMultiplied(psi),psi),mom_v)/I.sH;
    if(FLAG_DEBUG)
    {
	System.err.println("<psi|dpsi>="+TMatr.getAMatrB(psi,dpsi,QQt,n)+" <psi|I|dpsi>="+TMatr.getAMatrB(psi,dpsi,QQv,n));
	final double vart1=RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQtV,QQt,n)[1],
	    varv1=RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQvV,QQt,n)[1];
	final double [] rqv=RayleighQuotient.getRQVariationValues(psi,Dpsi,null,QQv,QQt,n);
	System.err.println("vartI="+vartI+" var1t="+vart1+" varvI="+varvI+" varv1="+varv1+" I.sH="+I.sH+"=rqv[0]="+rqv[0]+" rqv[1]="+rqv[1]+" must be =0"+" rqv[2]="+rqv[2]);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,dpsi,QQt,n),"<psi|dpsi>",0,"0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(psi,dpsi,QQv,n),"<psi|I|dpsi>",0,"0",eps);
	UnitTests.checkDoubleRel(vartI,"vartI",vart1,"vart1",eps);
	UnitTests.checkDoubleRel(varvI,"varvI",varv1,"varv1",eps);
	UnitTests.checkDoubleRel(I.sH,"I.sH",rqv[0],"rqv[0]",eps);
	UnitTests.checkDoubleRel(0,"0",rqv[1],"rqv[1]",eps);
	UnitTests.checkDoubleRel(0,"0",TMatr.getAMatrA(psi,QQv,n)-I.sH*TMatr.getAMatrA(psi,QQt,n),"<psi|I|psi>-I.sH(<psi|psi>",eps);
	System.err.println("QmQVMpi: test OK n="+n+" eps="+eps+" FLAG_DEBUG="+FLAG_DEBUG);
    }

/*    final double [] QQDip=getDIPMath(I,QQv,QQt);
    final double [] evDip=M.EV.getEV(n,QQDip,n,true);
    this.impD=evDip[0];
    this.impU=evDip[n-1];
    if(FLAG_DEBUG)
    {
	final double [] QQDtp=getDIPMath(I,QQt,QQt);
	System.err.println("QQDtp="+java.util.Arrays.toString(QQDtp));
	System.err.println("QQDip="+java.util.Arrays.toString(QQDip));
    }
*/
    this.impD=this.impU=Double.NaN;

    final double [][] rr=getRWIFromPis(I.psiL,M,QQv,I.sL,QQt);
    final int nd=n-1;
    this.p=rr[0];this.w=rr[1];this.i=rr[2];
    if(p.length!=nd) throw new RuntimeException();
    final int [] ind_w=SortArrays.sortArrayIndexes(w),ind_i=SortArrays.sortArrayIndexes(i);
    i_max=i[ind_i[nd-1]];
    p_i_max=p[ind_i[nd-1]]-offset_to_pl;
    pa_i=TMatr.mulVV(p,i)/TMatr.sumElements(i)-offset_to_pl;
    w_max=w[ind_w[nd-1]];
    p_w_max=p[ind_w[nd-1]]-offset_to_pl;
    pa_w=TMatr.mulVV(p,w)/TMatr.sumElements(w)-offset_to_pl;

    if(FLAG_DEBUG)
    {
	final double mu0=QQv[0]-I.sL*QQt[0];
	System.err.println("QQv[0]-I.sL*QQt[0]="+mu0+" sum(w)="+TMatr.sumElements(w));
	UnitTests.checkDoubleRel(mu0,"mu0",TMatr.sumElements(w),"sum(w)",eps);
	System.err.println("p="+java.util.Arrays.toString(p)+
			   " w_max="+w_max+
			   " w="+java.util.Arrays.toString(w)+
			   " ind_w="+java.util.Arrays.toString(ind_w)+
			   " i_max="+i_max+
			   " i="+java.util.Arrays.toString(i)+
			   " ind_i="+java.util.Arrays.toString(ind_i));
    }

}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
private OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis(final double pl){return new OrthogonalPolynomialsMonomialsBasisX(1,-pl);}

double [] getDpsiOrthoginal(final double [] psi,final double [] QQt,final int n)
{
    final double [] Dpsi=M.B.getDifferentiated(psi),dpsi=new double [psi.length];
    System.arraycopy(Dpsi,0,dpsi,0,Dpsi.length);
    TMatr.sumP1(dpsi,psi,-TMatr.getAMatrB(psi,Dpsi,QQt,n));
    return dpsi;
}
double [] getDIPMath(final EVXData I,final double [] QQi,final double [] QQt)
{
    final int n=I.neq;
    final double [] QQDip=new double [n*n];
    for(int k=0;k<n;k++)
    {
	final double [] psik=I.getPsi(k),dpsik=getDpsiOrthoginal(psik,QQt,n);
	for(int l=0;l<=k;l++)
	{
	    final double [] psil=I.getPsi(l),dpsil=getDpsiOrthoginal(psil,QQt,n);
	    QQDip[k*n+l]=QQDip[l*n+k]=TMatr.getAMatrB(psil,dpsik,QQi,n)+TMatr.getAMatrB(dpsil,psik,QQi,n);
	}
    }
    return QQDip;
}

static double [][] getRWIFromPis(final double [] psi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final double [] QQi,final double lam,final double [] QQt)
{
    final int n=psi.length,nd=n-1;
    final PolynomialRootsFinder.PRoots u=
	M.getPolynomialRootsFinderInBasis().findRoots(psi);
    final double [] re_root=u.re_root;
    if(u.nroots!=nd) {java.util.Arrays.fill(re_root,Double.NaN);}

    final double [] w=new double[nd],i=new double [nd];
    final double [] QQmu=TMatr.sumAcB(QQi,-lam,QQt);
    for(int k=0;k<nd;k++)
    {
	final double xk=re_root[k];
	final double [] l=M.B.sdiv1(psi,xk).q;
	TMatr.scaleVectorInPlace(l,1/M.B.getD0(l,xk));
	final double mk=TMatr.getAMatrA(l,QQmu,n),vk=TMatr.getAMatrA(l,QQi,n),tk=TMatr.getAMatrA(l,QQt,n);
	w[k]=mk;
	i[k]=vk/tk-lam;
    }
    return new double [][]{re_root,w,i};
}

static double [][] get_ti_Moms(final int nmoms,final QVMData odata)
{
    final double [] tmom=new double [nmoms],vmom=new double [nmoms];
    final int nvm=odata.nvm,ntm=odata.ntm,npm=odata.npm;
    if(nmoms>npm || nvm<1 || ntm<1) throw new IllegalArgumentException();

    for(int k=0;k<nmoms;k++)
    {
	tmom[k]=odata.tmoments_vol_x_tau_x_p[0*ntm*npm+0*npm+k];
	vmom[k]=odata.vmoments_vol_x_tau_x_p[0*ntm*npm+0*npm+k];
    }
    return new double [][]{tmom,vmom};
}

static double [][] getMoms(final int nmoms,final QVMData odata)
{
    final double [] Vtmom=new double [nmoms],Vvmom=new double [nmoms],V2tmom=new double [nmoms];
    final int nvm=odata.nvm,ntm=odata.ntm,npm=odata.npm;
    if(nmoms>npm || nvm<3 || ntm<1) throw new IllegalArgumentException();

    for(int k=0;k<nmoms;k++)
    {
	Vtmom[k]=odata.tmoments_vol_x_tau_x_p[1*ntm*npm+0*npm+k]*odata.v_scale;
	Vvmom[k]=odata.vmoments_vol_x_tau_x_p[1*ntm*npm+0*npm+k]*odata.v_scale;
	V2tmom[k]=odata.tmoments_vol_x_tau_x_p[2*ntm*npm+0*npm+k]*(odata.v_scale*odata.v_scale);
    }
    return new double [][]{Vtmom,Vvmom,V2tmom};
}


public static QmQVMpi calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final QmQVMpi ref)
{
    final int nvm=3,ntm=1,npm=2*neq+1;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     ntm,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new QmQVMpi(neq,o);
}



static class QmQVMpiTest extends QTest {

QmQVMpi getQmQVMpi(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
QkQVMpi getQkQVMpi(){return QkQVMpi.calculate(neq,currenttime,tau,tb,tb.length,null);}


QmQVMpiTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

void test_QkQVMpi()
{
    final QmQVMpi q=getQmQVMpi();
    final QkQVMpi s=getQkQVMpi();
    EVXData.matchEVXData(q.I,s.I,false,true,eps);
    UnitTests.checkDoubleRel(q.p_v_IH,"q.p_v_IH",s.pmH.p_v,"s.pmH.p_v",eps);
    // here offset is set to p_last
    UnitTests.checkDoubleRel(q.p_v_IH,"q.p_v_IH",s.pnlvar3.p_IH,"s.pnlvar3.p_IH",eps);
    UnitTests.checkDoubleRel(q.vp_IH,"q.vp_IH",s.pnlvar3.vp_IH,"s.pnlvar3.vp_IH",eps);
    UnitTests.checkDoubleRel(q.v2p_IH,"q.v2p_IH",s.pnlvar3.v2p_IH,"s.pnlvar3.v2p_IH",eps);

    final int [] ind_qp=SortArrays.sortArrayIndexes(q.p),ind_sp=SortArrays.sortArrayIndexes(s.pnlvar3.p);
    for(int k=0;k<ind_qp.length;k++)
    {
	UnitTests.checkDoubleRel(q.w[ind_qp[k]],"q.w",s.pnlvar3.w[ind_sp[k]],"s.pnlvar3.w",eps);
	UnitTests.checkDoubleRel(q.i[ind_qp[k]],"q.i",s.pnlvar3.i[ind_sp[k]],"s.pnlvar3.i",eps);
	UnitTests.checkDoubleRel(q.p[ind_qp[k]],"q.p",s.pnlvar3.p[ind_sp[k]]-(q.p_offset-s.pnlvar3.p_offset),"s.pnlvar3.p",eps);
    }
    System.err.println("s.varI2p="+s.varI2p);

    System.err.println(name+":QmQVMpiTest:test_QkQVMpi(): test OK neq="+neq+" eps="+eps);
}

void test_Common()
{
    final QmQVMpi q=getQmQVMpi();
    if(true)
    { // test moments
	final double [][] it_moms=q.get_ti_Moms(1,q.odata),V_moms=q.getMoms(1,q.odata);
	final double [] mom_t=it_moms[0],mom_v=it_moms[1],mom_tV=V_moms[0],mom_vV=V_moms[1],mom_tV2=V_moms[2];
	System.err.println("mom_v[0]="+mom_v[0]+" mom_tV[0]="+mom_tV[0]+" must be different in sign");
	System.err.println("mom_vV[0]="+mom_vV[0]+" mom_tV2[0]/2="+(mom_tV2[0]/2)+" must be different in sign");
    }

    System.err.println(name+":QmQVMpiTest:test_Common(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":QmQVMpiTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_QkQVMpi();

    System.err.println(name+":QmQVMpiTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":QmQVMpiTest: starting tests; ver=$Id: QmQVMpi.java,v 1.32 2022/07/28 12:27:19 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":QmQVMpiTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: QmQVMpi.java,v 1.32 2022/07/28 12:27:19 mal Exp $");
}
} // END OF class QmQVMpiTest

public static void main(String [] args)
{
    new QmQVMpiTest("QmQVMpi",new java.util.Random(3222),1e-4).runTests(6);
} // END main




} // END of class QmQVMpi
