package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
/**
 * Helper class, store moments (not matrices) like {I,J}MomentsData,
 * to obtain the matrices use the method asQQMatrices().
 * @version $Id: RMomentsData.java,v 1.19 2022/06/30 08:18:24 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class RMomentsData {
    public final int n;
    public final double p_offset,p_last;
    public final double [] tmom,ptmom,p2tmom,p3tmom,imom,pimom,p2imom,p3imom,amom,pamom,p2amom,p3amom,dpmom,Tmom,Vmom,Dpwv,Dpwt,
	pVmom,p2Vmom,p3Vmom;
    public final boolean flagQQ;
    public RMomentsData(final int n,final double p_offset,final double p_last,
			final double [] tmom,
			final double [] ptmom,
			final double [] p2tmom,
			final double [] p3tmom,
			final double [] imom,
			final double [] pimom,
			final double [] p2imom,
			final double [] p3imom,
			final double [] amom,
			final double [] pamom,
			final double [] p2amom,
			final double [] p3amom,
			final double [] dpmom,
			final double [] Tmom,
			final double [] Vmom,
			final double [] pVmom,
			final double [] p2Vmom,
			final double [] p3Vmom,
			final double [] Dpwv,
			final double [] Dpwt,
			final boolean flagQQ)
    {
	this.n=n;this.p_offset=p_offset;this.p_last=p_last;
	this.tmom=tmom;
	this.ptmom=ptmom;
	this.p2tmom=p2tmom;
	this.p3tmom=p3tmom;
	this.imom=imom;
	this.pimom=pimom;
	this.p2imom=p2imom;
	this.p3imom=p3imom;
	this.amom=amom;
	this.pamom=pamom;
	this.p2amom=p2amom;
	this.p3amom=p3amom;
	this.dpmom=dpmom;
	this.Tmom=Tmom;
	this.Vmom=Vmom;
	this.pVmom=pVmom;
	this.p2Vmom=p2Vmom;
	this.p3Vmom=p3Vmom;
	this.Dpwv=Dpwv;
	this.Dpwt=Dpwt;
	this.flagQQ=flagQQ;
    }
    public RMomentsData asQQMatrices(final int ndim,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M){
	if(flagQQ) throw new IllegalArgumentException();
	final int n=ndim;
	return new RMomentsData(n,p_offset,p_last,
				M.B.getQQMatr(n,tmom),
				M.B.getQQMatr(n,ptmom),
				M.B.getQQMatr(n,p2tmom),
				M.B.getQQMatr(n,p3tmom),
				M.B.getQQMatr(n,imom),
				M.B.getQQMatr(n,pimom),
				M.B.getQQMatr(n,p2imom),
				M.B.getQQMatr(n,p3imom),
				M.B.getQQMatr(n,amom),
				M.B.getQQMatr(n,pamom),
				M.B.getQQMatr(n,p2amom),
				M.B.getQQMatr(n,p3amom),
				M.B.getQQMatr(n,dpmom),
				M.B.getQQMatr(n,Tmom),
				M.B.getQQMatr(n,Vmom),
				M.B.getQQMatr(n,pVmom),
				M.B.getQQMatr(n,p2Vmom),
				M.B.getQQMatr(n,p3Vmom),
				M.B.getQQMatr(n,Dpwv),
				M.B.getQQMatr(n,Dpwt),
				true);
    }
public static <DTYPE> RMomentsData get_MomentsData(final int nmoms,final DTYPE odata,final DataInterfaceToMoments<DTYPE> access)
{
    final double [] tmom=new double [nmoms];access.setMT(tmom,nmoms);
    final double [] Tmom=new double [nmoms];access.setFMoments(Tmom,tmom,nmoms);
    final double [] imom=new double [nmoms];access.setM(imom,0,access.M_I,odata,nmoms);
    final double [] Vmom=new double [nmoms];access.setFMoments(Vmom,imom,nmoms);
    final double [] Dpwv=new double [nmoms];access.setM(Dpwv,0,access.M_DPwV,odata,nmoms);
    final double [] Dpwt=new double [nmoms];access.setM(Dpwt,0,access.M_DPwT,odata,nmoms);
    final double [] pimom=new double [nmoms];access.setM(pimom,1,access.M_I,odata,nmoms);
    final double [] pVmom=new double [nmoms];access.setFMoments(pVmom,pimom,nmoms);
    final double [] p2imom=new double [nmoms];access.setM(p2imom,2,access.M_I,odata,nmoms);
    final double [] p2Vmom=new double [nmoms];access.setFMoments(p2Vmom,p2imom,nmoms);
    final double [] p3imom=new double [nmoms];access.setM(p3imom,3,access.M_I,odata,nmoms);
    final double [] p3Vmom=new double [nmoms];access.setFMoments(p3Vmom,p3imom,nmoms);
    final double [] amom=new double [nmoms];access.setM(amom,0,access.M_A,odata,nmoms);
    final double [] pamom=new double [nmoms];access.setM(pamom,1,access.M_A,odata,nmoms);
    final double [] p2amom=new double [nmoms];access.setM(p2amom,2,access.M_A,odata,nmoms);
    final double [] p3amom=new double [nmoms];access.setM(p3amom,3,access.M_A,odata,nmoms);
    final double pl_m_poffset=access.getPLast(odata);// p_offset is already included in access.getPLast(DTYPE odata).
    final double [] dp0mom=new double [nmoms],dpmom=new double [nmoms],dp2mom=new double [nmoms],dp3mom=new double [nmoms];
    access.setM(dp0mom,0,access.M_DP,odata,nmoms);
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setM(dp2mom,2,access.M_DP,odata,nmoms);
    access.setM(dp3mom,3,access.M_DP,odata,nmoms);
    // shift to p_offset=p_last
    Polynomials.shiftPOffsetInPlace(nmoms,dp0mom,dpmom,dp2mom,dp3mom,pl_m_poffset);
    final double [] ptmom=new double [nmoms],p2tmom=new double [nmoms],p3tmom=new double [nmoms];
    access.setFMoments(ptmom,dpmom,nmoms);
    access.setFMoments(p2tmom,dp2mom,nmoms);
    access.setFMoments(p3tmom,dp3mom,nmoms);
    // shift price offset back to commnon value. (integration by parts give last price as the offset)
    Polynomials.shiftPOffsetInPlace(nmoms,tmom,ptmom,p2tmom,p3tmom,-pl_m_poffset);
    return new RMomentsData(nmoms,access.getPOffset(odata),pl_m_poffset,
			    tmom,ptmom,p2tmom,p3tmom,imom,pimom,p2imom,p3imom,amom,pamom,p2amom,p3amom,dpmom,Tmom,Vmom,pVmom,p2Vmom,p3Vmom,Dpwv,Dpwt,false);
}

}
