package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.realtimebook.*;

/** Impact QVM calculations.
 *  @version $Id: RQVM.java,v 1.72 2022/07/28 10:18:50 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class RQVM<DTYPE> {

final double [] rnodes,wnodes,xnodes,px,p2x;
final double dp00;
final double [] wnodes2,dpnodes,dpnodes2,dipnodes,dipnodes2;
final double wl,dpl,wl2,dpl2,wil,dipl,wil2,dipl2;

final double [] pnodes,p2nodes,pnodes2,p2nodes2;

final double [] xplus,wplus,xminus,wminus;

final DTYPE odata;
final DataInterfaceToMoments<DTYPE> access;
RQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final int nmoms=2*neq+1;
    final double [] imom=new double [nmoms],e=new double [nmoms];
    access.setM(imom,0,access.M_I,odata,nmoms);
    final double v0=imom[0];
    for(int k=nmoms;--k>=0;) imom[k]/=v0;

    final double [] ep=new double [neq+1];
    access.setM(ep,1,access.M_I,odata,neq+1);
    access.setM(e,2,access.M_I,odata,neq+1);
    for(int k=neq+1;--k>=0;) {ep[k]/=v0;e[k]/=v0;}
    final double [][] xx=getNodes(neq,imom,ep,e);
    rnodes=xx[0];
    final double [] ptr=xx[1],p2tr=xx[2];
    xnodes=getXNodes(rnodes);
    wnodes=getWeights(xnodes,rnodes,imom);
    wnodes2=getWeights2(xnodes,rnodes,imom); // match to wnodes identically
    px=new double [xnodes.length];
    p2x=new double [xnodes.length];
    for(int k=xnodes.length;--k>=0;)
    {
	final double dpf=fx(neq,ptr,xnodes[k]),dp2f=fx(neq,p2tr,xnodes[k]);
	px[k]=dpf+getPOffset();
	p2x[k]=dp2f-dpf*dpf;
    }

    if(true)
    {
	access.setM(imom,0,access.M_ISELL,odata,nmoms);
	access.setM(e,0,access.M_IBUY,odata,nmoms);
	final double d0=imom[0]+e[0];
	for(int k=nmoms;--k>=0;) {imom[k]/=d0;e[k]/=d0;}
	final double [] rnplus=getNodes(neq,imom);
	xplus=getXNodes(rnplus);
	wplus=getWeights(xplus,rnplus,imom);
	final double [] rnminus=getNodes(neq,e);
	xminus=getXNodes(rnminus);
	wminus=getWeights(xminus,rnminus,e);
    }

    if(true){
	access.setM(imom,1,access.M_DPPLUS,odata,1);
	access.setM(e,1,access.M_DPMINUS,odata,1);
	final double d0=imom[0]-e[0];
	this.dp00=d0;
	access.setM(imom,1,access.M_DP,odata,nmoms);
	for(int k=nmoms;--k>=0;) imom[k]/=d0;
	dpnodes=getWeights(xnodes,rnodes,imom);
	dpnodes2=getWeights2(xnodes,rnodes,imom);
	double [] l=getLdpw(dpnodes,wnodes);
	wl=l[0];dpl=l[1];
	l=getLdpw(dpnodes2,wnodes);
	wl2=l[0];dpl2=l[1];
    }
    if(true){
	access.setM(imom,0,access.M_ISELL,odata,nmoms);
	access.setM(e,0,access.M_IBUY,odata,nmoms);
	final double d0=imom[0]+e[0];
	for(int k=nmoms;--k>=0;) imom[k]=(imom[k]-e[k])/d0;
	dipnodes=getWeights(xnodes,rnodes,imom);
	dipnodes2=getWeights2(xnodes,rnodes,imom);
	double [] l=getLdpw(dipnodes,wnodes);
	wil=l[0];dipl=l[1];
	l=getLdpw(dipnodes2,wnodes);
	wil2=l[0];dipl2=l[1];
    }

    if(true)
    {
	access.setM(imom,1,access.M_I,odata,nmoms);
	for(int k=nmoms;--k>=0;) imom[k]/=v0;
	pnodes=getWeights(xnodes,rnodes,imom);
	pnodes2=getWeights2(xnodes,rnodes,imom);

	access.setM(imom,2,access.M_I,odata,nmoms);
	for(int k=nmoms;--k>=0;) imom[k]/=v0;
	p2nodes=getWeights(xnodes,rnodes,imom);
	p2nodes2=getWeights2(xnodes,rnodes,imom);

	for(int k=wnodes.length;--k>=0;)
	{
	    final double w=wnodes[k],dp=pnodes[k]/w,dp2=p2nodes[k]/w;
	    pnodes[k]=dp+getPOffset();
	    p2nodes[k]=dp2-dp*dp;
	    final double sdp=pnodes2[k]/w,sdp2=p2nodes2[k]/w;
	    pnodes2[k]=sdp+getPOffset();
	    p2nodes2[k]=sdp2-sdp*sdp;
	}
    }
}
double [][] getNodes(final int neq,final double [] imom,final double [] pmom,final double [] p2mom)
{
    final double [] rnodes=new double [neq+1],e=new double [2*neq+1],rp=new double [neq],rp2=new double [neq];
    rnodes[neq]=1;
    final double [] matr=new double [neq*neq];
    for(int k=0;k<neq;k++)
    {
	setQQCoefs(neq,k,e);
	rnodes[k]=-TMatr.mulVVn(neq+k+2,imom,e);
	java.util.Arrays.fill(e,0,k,0);e[k]=1;setRCoefs(k,e,e);
	rp[k]=TMatr.mulVVn(k+2,pmom,e);
	rp2[k]=TMatr.mulVVn(k+2,p2mom,e);
	for(int l=0;l<=k;l++)
	{
	    setQQCoefs(l,k,e);
	    matr[k+l*neq]=matr[l+k*neq]=TMatr.mulVVn(l+k+2,e,imom);
	}
    }
    final int [] ipvt=new int[neq];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,neq,neq,ipvt);
    if(pinfo!=0)
    {
	java.util.Arrays.fill(rnodes,Double.NaN);
	java.util.Arrays.fill(rp,Double.NaN);
	java.util.Arrays.fill(rp2,Double.NaN);
    }
    else
    {
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq,neq,ipvt,rnodes,1);
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq,neq,ipvt,rp,1);
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq,neq,ipvt,rp2,1);
    }
    return new double [][]{rnodes,rp,rp2};
}
double [] getNodes(final int neq,final double [] imom)
{
    final double [] rnodes=new double [neq+1],e=new double [2*neq+1];
    rnodes[neq]=1;
    final double [] matr=new double [neq*neq];
    for(int k=0;k<neq;k++)
    {
	setQQCoefs(neq,k,e);
	rnodes[k]=-TMatr.mulVVn(neq+k+2,imom,e);
	for(int l=0;l<=k;l++)
	{
	    setQQCoefs(l,k,e);
	    matr[k+l*neq]=matr[l+k*neq]=TMatr.mulVVn(l+k+2,e,imom);
	}
    }
    final int [] ipvt=new int[neq];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,neq,neq,ipvt);
    if(pinfo!=0) java.util.Arrays.fill(rnodes,Double.NaN);
    else com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq,neq,ipvt,rnodes,1);
    return rnodes;
}

static double [] getLdpw(final double [] dp,final double [] w)
{
    double swl=0,sdpl=0;
    for(int k=0,sign=0;k<dp.length;k++)
    {
	final double d=dp[k];
	final int s=d>0?1:-1;
	if(k<=0){sign=s;}
	if(s==sign)
	{
	    swl+=w[k];
	    sdpl+=d;
	} else break;
    }
    return new double []{swl,sdpl};
}

abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract double getPOffset();
abstract void setQQCoefs(final int l,final int m,final double [] coefs);
abstract void setRCoefs(final int m,final double [] res,final double [] coefs);
abstract double [] getXNodes(final double [] nodespolynomial);
abstract double [] getWeights(final double [] nodes,final double [] nodespolynomial,final double [] moms);
abstract double [] getWeights2(final double [] nodes,final double [] nodespolynomial,final double [] moms);
abstract double fx(final int nm,final double [] m,final double x);

static abstract class RQVMTest<DTYPE> {

int neq;
long currenttime,tau;
Trade [] tb;
java.util.Random r;
abstract void setData(final int tsize,final java.util.Random r,final int neq);

abstract RQVM<DTYPE> getRQVM();
abstract void matchMoments();
abstract double [] getRadauNodesFromNodes(final double [] nodes);

final String name;
final double eps,epssum;
RQVMTest(final String name,final double eps,final double epssum)
{
    this.name=name;
    this.eps=eps;
    this.epssum=epssum;
}

void matchCommon()
{
    final RQVM<DTYPE> q=getRQVM();
    UnitTests.checkDoubleArrayRel(q.wnodes,"q.wnodes",q.wnodes2,"q.wnodes2",eps);

    // there is a deep reason why they match on weights nodes,
    // see Christoffel formulae.
    UnitTests.checkDoubleArrayRel(DeprecatedHelperTMatr.remove0Element(q.px),"q.px",DeprecatedHelperTMatr.remove0Element(q.pnodes),"q.pnodes",eps);
    UnitTests.checkDoubleArrayRel(DeprecatedHelperTMatr.remove0Element(q.p2x),"q.p2x",DeprecatedHelperTMatr.remove0Element(q.p2nodes),"q.p2nodes",eps);


    if(true)
    {
	final int nmoms=2*neq+1;
	final double [] imom=new double [nmoms];
	q.access.setM(imom,0,q.access.M_I,q.odata,nmoms);
	final double v0=imom[0];
	for(int k=nmoms;--k>=0;) imom[k]/=v0;
	UnitTests.checkDoubleArrayRel(q.rnodes,"q.rnodes",q.getNodes(neq,imom),"q.getNodes(neq,imom)",eps);
    }

    // match only in unit test
    UnitTests.checkDoubleArrayRel(q.dpnodes,"q.dpnodes",q.dipnodes,"q.dipnodes",eps);
    UnitTests.checkDoubleArrayRel(q.dpnodes2,"q.dpnodes2",q.dipnodes2,"q.dipnodes2",eps);

    System.err.println(name+":RQVMTest:matchCommon(): test OK"+
		       " neq="+neq+" eps="+eps+" epssum="+epssum);
}


void matchRadauPt(final RQVM<DTYPE> q,final double [] mi,final double [] radauImom,
		  final double [] radauPmom,final double [] radauP2mom,
		  final boolean flag_desc,final double p_offset)
{
    System.err.println(name+":RQVM:matchRadauPt(): neq="+neq);
    final double [] mqcoefs=SQVM.SQVMTest.getNodesPolynomialBasis(neq,radauImom);
    final double [][] nq=SQVM.SQVMTest.checkNodesPolynomialBasis(mqcoefs,radauImom,2*neq-1,flag_desc,eps,r);
    final double [] xt=nq[0];
    UnitTests.checkDoubleArrayRel(DeprecatedHelperTMatr.remove0Element(q.xnodes),"DeprecatedHelperTMatr.remove0Element(q.xnodes)",xt,"xt",eps);
    final double [] ptr=DeprecatedHelperEVXData.getLinsysSol(neq,radauImom,0,radauPmom,0),
	p2tr=DeprecatedHelperEVXData.getLinsysSol(neq,radauImom,0,radauP2mom,0);

    final double [] px=new double [xt.length],p2x=new double [xt.length];
    for(int k=0;k<px.length;k++)
    {
	final double dpf=Polynomials.getD0(ptr,xt[k]),dp2f=Polynomials.getD0(p2tr,xt[k]);
	px[k]=dpf+p_offset;
	p2x[k]=dp2f-dpf*dpf;
    }
    UnitTests.checkDoubleArrayRel(DeprecatedHelperTMatr.remove0Element(q.px),"DeprecatedHelperTMatr.remove0Element(q.px)",px,"px",eps);
    UnitTests.checkDoubleArrayRel(DeprecatedHelperTMatr.remove0Element(q.p2x),"DeprecatedHelperTMatr.remove0Element(q.p2x)",p2x,"p2x",eps);

    final double [][] nxt=SQVM.SQVMTest.checkNodesPolynomialBasis(getRadauNodesFromNodes(mqcoefs),mi,2*neq,flag_desc,eps,r);
    final double [] xnodes=nxt[0],wnodes=TMatr.getVectorScaled(nxt[1],1/mi[0]);
    UnitTests.checkDoubleArrayRel(q.xnodes,"q.xnodes",xnodes,"xnodes",eps);
    UnitTests.checkDoubleArrayRel(q.wnodes,"q.wnodes",wnodes,"wnodes",eps);

    System.err.println(name+":RQVMTest:matchRadauPt(): test OK with neq="+neq);
}


void checkXW(final RQVM<DTYPE> q,final double [] nodes,final double [] mi,final double [] ms,final double [] mb,final boolean flag_desc)
{
    final double [][] nxt=SQVM.SQVMTest.checkNodesPolynomialBasis(nodes,mi,2*neq,flag_desc,eps,r);
    final double [][] nxs=SQVM.SQVMTest.checkNodesPolynomialBasis(nodes,ms,neq,flag_desc,eps,r);
    final double [][] nxb=SQVM.SQVMTest.checkNodesPolynomialBasis(nodes,mb,neq,flag_desc,eps,r);
    final double [] xt=nxt[0],wt=nxt[1],wt2=nxt[2],xs=nxs[0],ws=nxs[1],ws2=nxs[2],xb=nxb[0],wb=nxb[1],wb2=nxb[2];
    UnitTests.checkDoubleArrayRel(xt,"xt",xs,"xs",eps);
    UnitTests.checkDoubleArrayRel(xt,"xt",xb,"xb",eps);
    UnitTests.checkDoubleArrayRel(xt,"xt",q.xnodes,"q.xnodes",eps);
    UnitTests.checkDoubleArrayRel(wt,"wt",wt2,"wt2",eps);

    final double [] ww=new double [wt.length];
    final double ndp=ms[0]+mb[0],nw=mi[0];
    TMatr.sumP1(ww,ws,1/ndp);
    TMatr.sumP1(ww,wb,-1/ndp);
    UnitTests.checkDoubleArrayRel1(q.dpnodes,"q.dpnodes",ww,"ww",eps);
    java.util.Arrays.fill(ww,0);
    TMatr.sumP1(ww,ws2,1/ndp);
    TMatr.sumP1(ww,wb2,-1/ndp);
    UnitTests.checkDoubleArrayRel1(q.dpnodes2,"q.dpnodes2",ww,"ww",eps);
    java.util.Arrays.fill(ww,0);
    TMatr.sumP1(ww,wt,1/nw);
    UnitTests.checkDoubleArrayRel(q.wnodes,"q.wnodes",ww,"ww",eps);
    java.util.Arrays.fill(ww,0);
    TMatr.sumP1(ww,ws,1/ndp);
    TMatr.sumP1(ww,wb,1/ndp);
    UnitTests.checkDoubleArrayRel(q.wnodes,"q.wnodes",ww,"ww",eps);

    UnitTests.checkDoubleRel(TMatr.sumElements(q.wnodes),"sum(q.wnodes)",1,"1",eps);
    System.err.println(name+":RQVMTest:checkXW(): test OK with neq="+neq);
}

void checkOneXW(final double [] nodes,final double [] moms,final double [] xmatch,final double [] wmatch,final boolean flag_desc,final double scale)
{
    final double [][] nx=SQVM.SQVMTest.checkNodesPolynomialBasis(nodes,moms,2*neq,flag_desc,eps,r);
    final double [] xs=nx[0],ws=TMatr.getVectorScaled(nx[1],scale);
    UnitTests.checkDoubleArrayRel(xs,"xs",xmatch,"xmatch",eps);
    UnitTests.checkDoubleArrayRel(ws,"ws",wmatch,"wmatch",eps);
    System.err.println(name+":RQVMTest:checkOneXW(): test OK with neq="+neq);
}

static <DTYPE> boolean setMTest(final DataInterfaceToMoments<DTYPE> i,final double [] m,final int n,final int mom,final DTYPE odata,final int neq){
    final double ff=4.67127233; // to catch wrongh scale, if any
    if(mom==i.M_DPPLUS) {
	if(n!=1) throw new RuntimeException();
	i.setM(m,0,i.M_ISELL,odata,neq);
	for(int k=neq;--k>=0;) m[k]=ff*m[k];
    } else if(mom==i.M_DPMINUS) {
	if(n!=1) throw new RuntimeException();
	i.setM(m,0,i.M_IBUY,odata,neq);
	for(int k=neq;--k>=0;) m[k]=-ff*m[k];
    } else if(mom==i.M_DP) {
	i.setM(m,n,i.M_DPPLUS,odata,neq);
	final double [] tmp=new double [neq];
	i.setM(tmp,n,i.M_DPMINUS,odata,neq);
	for(int k=neq;--k>=0;) m[k]+=tmp[k];
    } else return true;
    return false;
}

void checkCalculations(final int tsize,final java.util.Random r,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":RQVMTest:checkCalculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps+" epssum="+epssum);

    setData(tsize,r,neq);
    matchMoments();
    matchCommon();

    System.err.println(name+":RQVMTest:checkCalculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps+" epssum="+epssum);
}
void runTests(final int neq_max)
{
    System.err.println(name+":RQVMTest: starting tests; ver=$Id: RQVM.java,v 1.72 2022/07/28 10:18:50 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79091);
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<118;i++)
	{
	    checkCalculations(2210+r.nextInt(2210),r,neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":RQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: RQVM.java,v 1.72 2022/07/28 10:18:50 mal Exp $");
}
} // END OF class RQVMTest


}// END OF class RQVM
