package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.realtimebook.*;

/** QVM calculations.
 *  @version $Id: RQVMLegendreShifted.java,v 1.72 2022/07/28 22:30:50 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class RQVMLegendreShifted extends RQVM<QVMDataP> {

    static final LegendreShifted B=new LegendreShifted();

RQVMLegendreShifted(final int neq,final QVMDataP odata)
{
    super(neq,odata);
}
QVMDataPDirectAccess getAccess(){return new QVMDataPDirectAccess();}
double getPOffset(){return odata.p_offset;}
public static RQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final RQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new RQVMLegendreShifted(neq,o);
}

void setRCoefs(final int m,final double [] res,final double [] coefs)
{
    final double [] a=B.get1mxP(java.util.Arrays.copyOf(coefs,m+1));
    System.arraycopy(a,0,res,0,a.length);
}
void setQQCoefs(final int l,final int m,final double [] coefs)
{
    B.setQlQmExpansionCoefs(l,m,coefs);
    setRCoefs(l+m,coefs,coefs);
}
double fx(final int nm,final double [] m,final double x)
{
    if(m.length!=nm) throw new RuntimeException();
    return B.getD0(m,x);
}
/** Calculate all nodes. The Radau is added explicitely,
 *  it does not present in the nodespolynomial.
 */
double [] getXNodes(final double [] nodespolynomial)
{
    final double [] x=new double [nodespolynomial.length];
    final int n=nodespolynomial.length-1;
    //final double [] npol=LegendreShifted.convertToPolynomialFromLegendre_Test(n,nodespolynomial);
    //final int nroots=new com.polytechnik.utils.PolynomialRootJenkinsTraubORIG(n).findRoots(n,npol,x,npol);
    final PolynomialRootsFinder.PRoots u=
	new com.polytechnik.outils.LegendreShiftedPolynomialRootsFinder().findRoots(nodespolynomial);
    System.arraycopy(u.re_root,0,x,0,u.nroots);
    if(u.nroots!=n) java.util.Arrays.fill(x,Double.NaN);
    else x[n]=1;
    com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(x.length,x,null);
    return x;
}
double [] getWeights(final double [] x,final double [] nodespolynomial,final double [] bmoms)
{
    final double [] w=new double [x.length];
    final int nn=nodespolynomial.length,n=nn-1;
    final double [] xn=B.get1mxP(nodespolynomial);
    for(int k=x.length;--k>=0;)
    {
	final double xk=x[k];
	double [] tmp=new double [nn];
	if(Double.compare(xk,1.0)==0)
	{ // Radau
	    w[k]=TMatr.mulVVn1(nodespolynomial,bmoms)/fx(nn,nodespolynomial,xk);
	}
	else
	{
	    tmp=B.sdiv1(xn,xk).q;
	    w[k]=TMatr.mulVVn1(tmp,bmoms)/fx(nn,tmp,xk);
	}
    }
    return w;
}
double [] getWeights2(final double [] x,final double [] nodespolynomial,final double [] bmoms)
{
    final double [] w=new double [x.length];
    final int n=nodespolynomial.length-1;
    final double [] xn=B.get1mxP(nodespolynomial);
    for(int k=x.length;--k>=0;)
    {
	double [] tmp=new double [nodespolynomial.length];
	final double xk=x[k];
	if(Double.compare(xk,1.0)==0) System.arraycopy(nodespolynomial,0,tmp,0,nodespolynomial.length); // Radau
	else tmp=B.sdiv1(xn,xk).q;
	final double t0=fx(tmp.length,tmp,xk);
	for(int m=tmp.length;--m>=0;) tmp[m]/=t0;
	final double [] mul2=B.mult2Pol(tmp,tmp);
	w[k]=TMatr.mulVVn1(mul2,bmoms);
    }
    return w;
}

public static void main(String [] args)
{
    new RQVMTest<QVMDataP>("RQVMLegendreShifted",1e-6,1e-3){
	void setData(final int tsize,final java.util.Random r,final int neq)
	{
	    this.neq=neq;
	    this.r=r;
	    this.tb=CalculateKendall.TradeEMoments.getRandomTrades(tsize,r);
	    this.currenttime=tb[tb.length-1].time+1000;
	    this.tau=((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8*11*13)*(9*5*7*8*11*13);
	}
	RQVMLegendreShifted getRQVM()
	{
	    final RQVMLegendreShifted q=calculate(neq,currenttime,tau,tb,tb.length,null);
	    return new RQVMLegendreShifted(neq,q.odata){
		QVMDataPDirectAccess getAccess(){return new QVMDataPDirectAccess(){
			public void setM(final double [] m,final int n,final int mom,final QVMDataP odata,final int neq){if(setMTest(this,m,n,mom,odata,neq))super.setM(m,n,mom,odata,neq);}};}};
	}

	QVMData [] getPData(final RQVMLegendreShifted q)
	{
	    final QVMData [] qdata=new QVMData [q.odata.ntm];
	    for(int k=0;k<q.odata.ntm;k++)
	    {
		final long t=tau/(1+k);
		if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
		qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
	    }
	    return qdata;
	}
	double [] getRadauFromData(final double [] mom)
	{
	    final double [] rmom=new double [mom.length-1];
	    for(int k=0;k<rmom.length;k++) rmom[k]=mom[k]-mom[k+1];
	    return rmom;
	}
	double [] getRadauNodesFromNodes(final double [] nodes)
	{
	    return Polynomials.get1mxP(nodes);
	}
	void matchMoments()
	{
	    System.err.println(name+":RQVMLegendreShiftedTest:matchMoments(): neq="+neq);

	    final RQVMLegendreShifted q=getRQVM();
	    UnitTests.checkDoubleRel(q.xnodes[0],"Radau q.xnodes[0]",1,"1",eps);

	    final QVMData [] s=getPData(q);

	    matchRadauPt(q,MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(q.odata.ntm,s,0,QVMData.MOM_dV,false,q.odata.p_offset),
			 getRadauFromData(MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(q.odata.ntm,s,0,QVMData.MOM_dV,false,q.odata.p_offset)),
			 getRadauFromData(MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(q.odata.ntm,s,1,QVMData.MOM_dV,false,q.odata.p_offset)),
			 getRadauFromData(MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(q.odata.ntm,s,2,QVMData.MOM_dV,false,q.odata.p_offset)),
			 true,q.odata.p_offset);
	    System.err.println(name+":RQVMLegendreShiftedTest:matchMoments(): ALL TESTS OK, neq="+neq+" eps="+eps);

	    matchMomentsPP(); // to remove later
	}

	void matchMomentsPP()
	{
	    System.err.println(name+":RQVMLegendreShiftedTest:matchMomentsPP(): neq="+neq);
	    final RQVMLegendreShifted q=getRQVM();
	    final QVMData [] s=getPData(q);
	    final double [] mi=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(s.length,s,0,QVMData.MOM_dV,false,q.odata.p_offset),
		ms=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(s.length,s,0,QVMData.MOM_VSELL,false,q.odata.p_offset),
		mb=MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(s.length,s,0,QVMData.MOM_VBUY,false,q.odata.p_offset);

	    final double [] mqcoefs=SQVM.SQVMTest.getNodesPolynomialBasis(neq,getRadauFromData(mi));
	    final double [] lqcoefs=B.convertFunctionTo_Basis_from_Monomials(mqcoefs);
	    final double f=q.rnodes[q.rnodes.length-1]/lqcoefs[lqcoefs.length-1];
	    for(int k=0;k<lqcoefs.length;k++) lqcoefs[k]*=f;
	    UnitTests.checkDoubleArrayRel1(q.rnodes,"q.rnodes",lqcoefs,"lqcoefs",eps);

	    final double [] nodes=getRadauNodesFromNodes(mqcoefs);
	    checkXW(q,nodes,mi,ms,mb,true);

	    checkOneXW(nodes,mi,q.xnodes,q.wnodes,true,1/mi[0]);

	    checkOneXW(getRadauNodesFromNodes(SQVM.SQVMTest.getNodesPolynomialBasis(neq,getRadauFromData(ms))),ms,q.xplus,q.wplus,true,1/mi[0]);
	    checkOneXW(getRadauNodesFromNodes(SQVM.SQVMTest.getNodesPolynomialBasis(neq,getRadauFromData(mb))),mb,q.xminus,q.wminus,true,1/mi[0]);

	    System.err.println(name+":RQVMLegendreShiftedTest:matchMomentsPP(): ALL TESTS OK, neq="+neq+" eps="+eps);
	}
    }.runTests(6);
}


} // END OF class RQVMLegendreShifted
