package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.RayleighQuotient;

/** Variation of the <psi|I|psi>/<psi|psi>
 * on psi equalt to P eigenvector.
 *  @version $Id: RVarIpsi.java,v 1.8 2022/10/30 21:29:54 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class RVarIpsi {
    final double f_lev0,I_lev0,I_lev1,dI_lev0,dI_lev0_,Iq_lev0;
    final double [] psi0,g,psi1;
    RVarIpsi(final int ind_lev,final EVXData j,final double [] QQi,final double [] QQt,final int ldQQ)
    {
	final int n=j.neq;
	final double [] basis=getBasisCond(ind_lev,j);
	this.psi0=new double [n];
	final int nd=basis.length/n;
	System.arraycopy(j.ev,n+ind_lev*n,psi0,0,n);
	this.I_lev0=TMatr.getAMatrA(psi0,QQi,n)/TMatr.getAMatrA(psi0,QQt,n);
	this.f_lev0=j.ev[ind_lev];

	final RayleighQuotient.RQVariationInMatrixForm
	    r=new RayleighQuotient.RQVariationInMatrixForm(psi0,n,QQi,QQt,n);
	this.sr=r;
	final double [] D1=new double [nd],D2=new double [nd*nd],tmp=new double [n*nd];
	// Convert d1 & d2 to new basis.
	// D2=basis d2 basis^T
	TMatr.multiply_MA_x_MBTrans(tmp,n,r.d2,n,basis,nd); // tmp: n x nd
	TMatr.multiply_MA_x_MB(D2,nd,basis,n,tmp,nd); // D2: nd x nd
	// D1=d1 basis^T
	TMatr.multiply_MA_x_MBTrans(D1,1,r.d1,n,basis,nd); // D1: 1 x nd

	// find G=-D2^{-1} D1 and dI_lev0=-0.5<G|D2|G>=0.5*<G|D1>
	{
	    final double [] DD2=tmp,G=new double [nd];
	    System.arraycopy(D2,0,DD2,0,D2.length);
	    final int [] ipvt=new int [nd];
	    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(DD2,nd,nd,ipvt);
	    if(pinfo!=0) {
		java.util.Arrays.fill(G,Double.NaN);
	    } else {
		for(int k=D1.length;--k>=0;){G[k]=-D1[k];}
		com.polytechnik.utils.Linsystems.dgesl_m1(DD2,nd,nd,ipvt,G,1);
	    }
	this.dI_lev0=-0.5*TMatr.getAMatrA(G,D2,nd);
	this.dI_lev0_=0.5*TMatr.mulVV(G,D1);
	this.g=new double [n];
	// g=G basis
	TMatr.multiply_MA_x_MB(g,1,G,nd,basis,n);
	Iq_lev0=r.d0+TMatr.mulVV(g,r.d1)+0.5*TMatr.getAMatrA(g,r.d2,n);
	}
	this.psi1=new double [n];
	for(int k=n;--k>=0;){this.psi1[k]=this.psi0[k]+this.g[k];}
	this.I_lev1=TMatr.getAMatrA(this.psi1,QQi,n)/TMatr.getAMatrA(this.psi1,QQt,n);
    }
    final RayleighQuotient.RQVariationInMatrixForm sr;

void test_Self(final double eps)
{
    final int n=g.length;

    UnitTests.checkDoubleRel(I_lev0,"I_lev0",sr.d0,"sr.d0",eps);
    UnitTests.checkDoubleRel(dI_lev0,"dI_lev0",dI_lev0_,"dI_lev0_",eps);
    UnitTests.checkDoubleRel(dI_lev0,"dI_lev0",Iq_lev0-sr.d0,"Iq_lev0-sr.d0",eps);
    System.err.println("dI_lev0="+dI_lev0+" (I_lev1-I_lev0)="+(I_lev1-I_lev0));
    UnitTests.checkDoubleRel(TMatr.mulVV(g,sr.d1),"TMatr.mulVV(g,sr.d1)",-TMatr.getAMatrA(g,sr.d2,n),"-TMatr.getAMatrA(g,sr.d2,n)",eps);
    if(Math.signum(I_lev1-I_lev0)!=Math.signum(dI_lev0))
	throw new RuntimeException("I_lev1="+I_lev1+" I_lev0="+I_lev0+" dI_lev0="+dI_lev0);

    System.err.println("EkQVM:RVarIpsi:test_Self: test OK n="+n+" eps="+eps);
}

static double [] getBasisCond(final int ind_lev,final EVXData j)
{
    final int n=j.neq;
    if(ind_lev<0||ind_lev>=n) throw new RuntimeException("ind_lev="+ind_lev+" j.neq="+j.neq);
    final int nd=n-1;
    final double [] basis=new double [nd*n];
    for(int k=0,nc=0;k<n;k++)
    {
	if(k!=ind_lev)
	{
	    System.arraycopy(j.ev,n+k*n,basis,nc*n,n);
	    nc++;
	}
    }
    return basis;
}

} // END of class RVarIpsi

