package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.Quadrature;
import com.polytechnik.realtimebook.*;

/** QVM calculations.
 *  @version $Id: RadauLQVMLegendreShifted.java,v 1.34 2019/10/09 09:17:31 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class RadauLQVMLegendreShifted extends LQVMLegendreShifted {
static final LegendreShifted B=new LegendreShifted();
RadauLQVMLegendreShifted(final int neq,final QVMDataP odata)
{
    super(neq,odata);
}
double [][] getTNodes(final int n)
{
    final double [] x=B.getRadauRoots(n);
    return new double [][]{x,B.getRadauWeights(x),B.getRadauNodesPolynomial(n)};
}
double [][] getINodes(final double [][] p)
{
    final Quadrature.GaussQuadratureWithNodesPolynomialData q=basis.getRadauQuadratureFromPolynomials(p.length-2,p);
    return scaleWInd1(new double [][]{q.x,q.w,q.nodespolynomial});
}

public static RadauLQVMLegendreShifted calculate(final int neq,
						 final long currenttime,
						 final long tau,
						 final Trade [] tb,
						 final int tsize,
						 final RadauLQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new RadauLQVMLegendreShifted(neq,o);
}


static class RadauLQVMLegendreShiftedTest extends LQVMLegendreShiftedTest {
RadauLQVMLegendreShiftedTest(final String name){super(name);}
RadauLQVMLegendreShifted getLQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
double [][] getNI(final int n,final double [] mom)
{
    final Quadrature.GaussQuadratureWithNodesPolynomialData q=M.getRadauQuadratureFromPolynomials(n-1,M.getOrthogonalPolynomialsFirstKind(n+1,mom));
    return scaleWInd1(new double [][]{q.x,q.w,q.nodespolynomial});
}

void test_OLD_QVM()
{
    final RadauLQVMLegendreShifted q=getLQVM();
    final QVMPP s=QVMPP.calculateQVMPP(q.xtnodes.length,currenttime,tau,tb,tb.length,null,null);
    System.err.println("q.xinodes[0]="+q.xinodes[0]+" q.wttnodes[0]="+q.wttnodes[0]+" s.neq_t="+s.neq_t+" neq="+neq+" q.xtnodes.length="+q.xtnodes.length);

    final double [] o=new double [q.xtnodes.length];o[0]=1;
    s.set_dmoments(o.length,o,o);
    final double [] mul=q.basis.getRadauMultiplied(o);

    final double [] dp=new double [o.length];dp[0]=1;
    s.set_dmoments(dp.length,dp,dp);
    for(int k=0;k<o.length;k++) dp[k]=((s.odata.trade_last.price-s.odata.p_offset)*dp[k]-s.dPcoefs[k]);
    final double [] dptnodes=new double [o.length];
    for(int k=0;k<o.length;k++)
    {
	UnitTests.checkDoubleRel(q.basis.B.getD0(mul,q.xtnodes[k]),"mul(x)",0,"0",eps);
	final double [] b=q.basis.B.sdiv1(mul,q.xtnodes[k]).q;
	final double f0=q.basis.B.getD0(b,q.xtnodes[k]);
	for(int j=b.length;--j>=0;) b[j]/=f0;
	dptnodes[k]=s.get_f_g(s.neq_t,b,dp);
	UnitTests.checkDoubleRel(q.wttnodes[k],"q.wttnodes[k]",b[0],"b[0]",eps);
    }
    UnitTests.checkDoubleArrayRel(q.dptnodes,"q.dptnodes",dptnodes,"dptnodes",eps);
    UnitTests.checkDoubleRel(q.dptnodes[0]/q.wttnodes[0],"q.dptnodes[0]/q.wttnodes[0]",s.dp_eq,"s.dp_eq=<dp|o>_t",eps);
    UnitTests.checkDoubleRel(q.dstnodes[0]/q.wttnodes[0],"q.dstnodes[0]/q.wttnodes[0]",s.get_f_o(s.neq_t,s.isellcoefs)-s.get_f_o(s.neq_t,s.ibuycoefs),"<is-ib|o>_t",eps);
    UnitTests.checkDoubleRel(q.dp2t,"q.dp2t",s.get_f_g(s.neq_t,dp,dp),"<dp|dp>_t",eps);
    UnitTests.checkDoubleRel(q.ds2t,"q.ds2t",s.get_f_g(s.neq_t,s.isellcoefs,s.isellcoefs)-2*s.get_f_g(s.neq_t,s.ibuycoefs,s.isellcoefs)+s.get_f_g(s.neq_t,s.ibuycoefs,s.ibuycoefs),"<ds|ds>_t",eps);

    System.err.println(name+":test_OLD_QVM(): Test OK, neq="+neq+" eps="+eps+" class="+q.getClass().getName());
    test_dp2i(true);
}
void test_SQVM()
{
    final RadauLQVMLegendreShifted q=getLQVM();
    final RQVMLegendreShifted s=RQVMLegendreShifted.calculate(neq-1,currenttime,tau,tb,tb.length,null);
    UnitTests.checkDoubleArrayRel(q.xinodes,"q.xinodes",s.xnodes,"s.xnodes",eps);
    UnitTests.checkDoubleArrayRel(q.wiinodes,"q.wiinodes",s.wnodes,"s.wnodes",eps);
    UnitTests.checkDoubleArrayRel(q.dpinodes,"q.dpinodes",TMatr.getVectorScaled(s.dpnodes,s.dp00),"s.dpnodes*s.dp00",eps);
    UnitTests.checkDoubleArrayRel(q.dsinodes,"q.dsinodes",s.dipnodes,"s.dipnodes",eps);
    //UnitTests.checkDoubleArrayRel(q.pinodes,"q.pinodes",s.pnodes,"s.pnodes",eps);

    System.err.println(name+":test_SQVM(): Test RQVM OK, neq="+neq+" eps="+eps+" class="+q.getClass().getName());
}
} // END of class RadauLQVMLegendreShiftedTest

public static void main(String [] args)
{
    new RadauLQVMLegendreShiftedTest("RadauLQVMLegendreShifted").runTests(5);
} // END main


} // END OF class RadauLQVMLegendreShifted
