package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: RkQVMLaguerre.java,v 1.6 2022/07/10 10:41:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class RkQVMLaguerre extends RkQVM<QVMDataL> {
RkQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}


public static RkQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final RkQVMLaguerre ref)
{
    final int npm=4;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*(neq-1)+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new RkQVMLaguerre(neq,o);
}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}


static class RkQVMLaguerreTest extends RkQVMTest<QVMDataL> {
RkQVMLaguerreTest(final long initrandom)
{
    super("RkQVMLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
RkQVMLaguerre getRkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class RkQVMLaguerreTest

public static void main(String [] args)
{
    new RkQVMLaguerreTest(1951117L).runTests(5);
} // END main



} // END OF class RkQVMLaguerre
