package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.realtimebook.*;

/** Skewness is back.
 *  @version $Id: RkiQVM.java,v 1.23 2022/07/15 00:01:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class RkiQVM<DTYPE> {

final VolMatrix VOLATILITY_EDPsi=this::getEDPsi;

static class State {
}
final State sst;

final DTYPE odata;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
final DynRki Rk0,RkR,Rk,RkV0,RkVR;
final DynSki Sk0,SkR,Sk;
final DynVarCD pixC,pixP,piVP,piDP;
final DynVar2I v2I,v2Io,v2I0,v2I0o;

RkiQVM(final int n,final DTYPE odata,final State rst)
{
    this.odata=odata;
    this.M=getABasis();

    {
    final int nmoms=2*n-1;
    final double [][] mm=getMoments(nmoms);
    final double [] QQt=M.B.getQQMatr(n,mm[0]),QQi=M.B.getQQMatr(n,mm[1]),
	QQpi=M.B.getQQMatr(n,mm[2]),QQp2i=M.B.getQQMatr(n,mm[3]),QQp3i=M.B.getQQMatr(n,mm[4]);
    this.Rk0=DynRki.get0_DynRki(n,QQi,QQt,QQi,QQpi,QQp2i,QQp3i,M);
    this.RkR=DynRki.getR_DynRki(n,QQi,QQt,QQi,QQpi,QQp2i,QQp3i,M);
    this.Rk=DynRki.get_DynRki(n,QQi,QQt,QQi,QQpi,QQp2i,QQp3i,M);
    final double [] QQV=M.B.getQQMatr(n,mm[5]),QQpV=M.B.getQQMatr(n,mm[6]),QQp2V=M.B.getQQMatr(n,mm[7]),QQp3V=M.B.getQQMatr(n,mm[8]);

    RkV0=DynRki.get0_DynRki(n,QQi,QQt,QQV,QQpV,QQp2V,QQp3V,M);
    RkVR=DynRki.getR_DynRki(n,QQi,QQt,QQV,QQpV,QQp2V,QQp3V,M);

    this.Sk0=DynSki.get0_DynSki(n,QQi,QQt,QQpi,QQp2i,QQp3i,M);
    this.SkR=DynSki.getR_DynSki(n,QQi,QQt,QQpi,QQp2i,QQp3i,M);
    this.Sk=DynSki.get_DynSki(n,QQi,QQt,QQpi,QQp2i,QQp3i,M);
    }
    {
    final int nmoms=2*n-1+2;
    final double [][] mm=getXXMoments(nmoms);
    final double [] QQt=M.B.getQQMatr(n,mm[0]),QQi=M.B.getQQMatr(n,mm[1]),
	QQpi=M.B.getQQMatr(n,mm[2]),QQp2i=M.B.getQQMatr(n,mm[3]),
	QQV=M.B.getQQMatr(n,mm[4]),
	Ximom=M.getRadauMomentsFromMoments(mm[1]),QQXi=M.B.getQQMatr(n,Ximom),
	QQX2i=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(Ximom)),
	QQpXi=M.B.getQQMatr(n,M.getRadauMomentsFromMoments(mm[2]));
    this.pixC=DynVarCD.get_DynVarCD(n,QQi,QQt,QQpi,QQp2i,QQXi,QQX2i,QQpXi,getPOffset(),M);
    this.pixP=DynVarCD.getXp_DynVarCD(n,QQi,QQt,QQpi,QQXi,M);
    this.piVP=DynVarCD.getXp_DynVarCD(n,QQi,QQt,QQpi,QQV,M);
    this.piDP=DynVarCD.getDp_DynVarCD(n,QQi,QQt,QQpi,true,M,VOLATILITY_EDPsi);
    v2I=DynVar2I.get_DynVar2I(n,QQi,QQt,QQpi,Double.NaN,false,M,VOLATILITY_EDPsi);
    v2Io=DynVar2I.get_DynVar2I(n,QQi,QQt,QQpi,getPLast()-getPOffset(),true,M,VOLATILITY_EDPsi);
    v2I0=DynVar2I.get0_DynVar2I(n,QQi,QQt,QQpi,Double.NaN,false,M,VOLATILITY_EDPsi);
    v2I0o=DynVar2I.get0_DynVar2I(n,QQi,QQt,QQpi,getPLast()-getPOffset(),true,M,VOLATILITY_EDPsi);
    }

    this.sst=null;
}

double [][] getMoments(final int nmoms)
{
    final DataInterfaceToMoments<DTYPE> access=getAccess();
    final double [] tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms],
	p2imom=new double [nmoms],p3imom=new double [nmoms];

    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    access.setM(p2imom,2,access.M_I,odata,nmoms);
    access.setM(p3imom,3,access.M_I,odata,nmoms);
    Polynomials.shiftPOffsetInPlace(nmoms,imom,pimom,p2imom,p3imom,getPLast()-getPOffset());

    final double [] Vmom=new double [nmoms],pVmom=new double [nmoms],
	p2Vmom=new double [nmoms],p3Vmom=new double [nmoms];
    access.setFMoments(Vmom,imom,nmoms);
    access.setFMoments(pVmom,pimom,nmoms);
    access.setFMoments(p2Vmom,p2imom,nmoms);
    access.setFMoments(p3Vmom,p3imom,nmoms);

    return new double [][]{tmom,imom,pimom,p2imom,p3imom,TMatr.scaleVectorInPlace(Vmom,-1),TMatr.scaleVectorInPlace(pVmom,-1),TMatr.scaleVectorInPlace(p2Vmom,-1),TMatr.scaleVectorInPlace(p3Vmom,-1)};
}

double [][] getXXMoments(final int nmoms)
{
    final DataInterfaceToMoments<DTYPE> access=getAccess();
    final double [] tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms],
	p2imom=new double [nmoms],Vmom=new double [nmoms];

    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    access.setM(p2imom,2,access.M_I,odata,nmoms);
    access.setFMoments(Vmom,imom,nmoms);
    TMatr.scaleVectorInPlace(Vmom,-1);

    return new double [][]{tmom,imom,pimom,p2imom,Vmom};
}

abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double [] getEDPsi(final double [] psi);
abstract double getPOffset();
abstract double getPLast();



static abstract class RkiQVMTest<DTYPE> extends QTest {

abstract RkiQVM<DTYPE> getRkiQVM();
abstract YpnlQVM<DTYPE> getYpnlQVM();


RkiQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}

void test_Aver()
{
    final int n=neq;
    final RkiQVM<DTYPE> q=getRkiQVM();
    final YpnlQVM<DTYPE> s=getYpnlQVM();

    UnitTests.checkDoubleRel(q.Rk0.sk.xa,"q.Rk0.sk.xa",s.xpi.p_IH,"s.xpi.p_IH",eps);
    UnitTests.checkDoubleRel(q.Rk0.p_IH,"q.Rk0.p_IH",s.xpi.p_IH,"s.xpi.p_IH",eps);
    UnitTests.checkDoubleRel(q.Rk0.I_0,"q.Rk0.I_0",s.xpi.I_0,"s.xpi.I_0",eps);
    UnitTests.checkDoubleRel(q.Rk0.I_L,"q.Rk0.I_L",s.xpi.I_L,"s.xpi.I_L",eps);
    UnitTests.checkDoubleRel(q.Rk0.I_H,"q.Rk0.I_H",s.xpi.I_H,"s.xpi.I_H",eps);

    UnitTests.checkDoubleRel(q.RkR.sk.xa,"q.RkR.sk.xa",s.rpi.p_IH,"s.rpi.p_IH",eps);
    UnitTests.checkDoubleRel(q.RkR.p_IH,"q.RkR.p_IH",s.rpi.p_IH,"s.rpi.p_IH",eps);
    //UnitTests.checkDoubleRel(q.RkR.I_0,"q.RkR.I_0",s.rpi.I_0,"s.rpi.I_0",eps);
    UnitTests.checkDoubleRel(q.RkR.I_L,"q.RkR.I_L",s.rpi.I_L,"s.rpi.I_L",eps);
    UnitTests.checkDoubleRel(q.RkR.I_H,"q.RkR.I_H",s.rpi.I_H,"s.rpi.I_H",eps);

    UnitTests.checkDoubleRel(q.Rk.sk.xa,"q.Rk.sk.xa",s.piVH.p_IH,"s.piVH.p_IH",eps);
    UnitTests.checkDoubleRel(q.Rk.p_IH,"q.Rk.p_IH",s.piVH.p_IH,"s.piVH.p_IH",eps);
    UnitTests.checkDoubleRel(q.Rk.I_0,"q.Rk.I_0",s.piVH.I.s0,"s.piVH.I.s0",eps);
    UnitTests.checkDoubleRel(q.Rk.I_L,"q.Rk.I_L",s.piVH.I.sL,"s.piVH.I.sL",eps);
    UnitTests.checkDoubleRel(q.Rk.I_H,"q.Rk.I_H",s.piVH.I.sH,"s.piVH.I.sH",eps);

    UnitTests.checkDoubleRel(q.RkV0.sk.xa,"q.RkV0.sk.xa",s.xpV.p_IH,"s.xpV.p_IH",eps);
    UnitTests.checkDoubleRel(q.RkV0.p_IH,"q.RkV0.p_IH",s.xpV.p_IH,"s.xpV.p_IH",eps);
    UnitTests.checkDoubleRel(q.RkV0.I_0,"q.RkV0.I_0",s.xpV.I_0,"s.xpV.I_0",eps);
    UnitTests.checkDoubleRel(q.RkV0.I_L,"q.RkV0.I_L",s.xpV.I_L,"s.xpV.I_L",eps);
    UnitTests.checkDoubleRel(q.RkV0.I_H,"q.RkV0.I_H",s.xpV.I_H,"s.xpV.I_H",eps);

    // match some moments
    UnitTests.checkDoubleRel(q.Rk0.p_IH,"q.Rk0.p_IH",q.Sk0.p_IH,"s.Sk0.p_IH",eps);
    UnitTests.checkDoubleRel(q.Rk0.I_H,"q.Rk0.I_H",q.Sk0.I_IH,"s.Sk0.I_IH",eps);
    UnitTests.checkDoubleRel(q.RkR.p_IH,"q.RkR.p_IH",q.SkR.p_IH,"s.SkR.p_IH",eps);
    UnitTests.checkDoubleRel(q.RkR.I_H,"q.RkR.I_H",q.SkR.I_IH,"s.SkR.I_IH",eps);
    UnitTests.checkDoubleRel(q.Rk.p_IH,"q.Rk.p_IH",q.Sk.p_IH,"s.Sk.p_IH",eps);
    UnitTests.checkDoubleRel(q.Rk.I_H,"q.Rk.I_H",q.Sk.I_IH,"s.Sk.I_IH",eps);

    // match DynVHPnL
    //UnitTests.checkDoubleRel(q.pixP.I_IX,"q.pixP.I_IX",s.piXiVH.I_IH,"s.piXiVH.I_IH",eps);
    UnitTests.checkDoubleRel(q.pixP.p_IX,"q.pixP.p_IX",s.piXiVH.p_IH+(s.getPLast()-s.getPOffset()),"s.piXiVH.p_IH",eps);
    UnitTests.checkDoubleRel(q.pixP.a.x_a_b,"q.pixP.a.x_a_b",s.piXiVH.var_p_IH,"s.piXiVH.var_p_IH",eps);
    UnitTests.checkDoubleRel(q.pixP.a.osp,"q.pixP.a.osp",s.piXiVH.sw_p_IH,"s.piXiVH.sw_p_IH",eps);
    UnitTests.checkDoubleRel(q.piDP.p_IX,"q.piDP.p_IX",s.piDiVH.p_IH+(s.getPLast()-s.getPOffset()),"s.piDiVH.p_IH",eps);
    UnitTests.checkDoubleRel(q.piDP.a.x_a_b,"q.piDP.a.x_a_b",s.piDiVH.var_p_IH,"s.piDiVH.var_p_IH",eps);
    UnitTests.checkDoubleRel(q.piDP.a.osp,"q.piDP.a.osp",s.piDiVH.sw_p_IH,"s.piDiVH.sw_p_IH",eps);
    UnitTests.checkDoubleRel(q.piVP.p_IX,"q.piVP.p_IX",s.piVVH.p_IH+(s.getPLast()-s.getPOffset()),"s.piVVH.p_IH",eps);
    UnitTests.checkDoubleRel(q.piVP.a.x_a_b,"q.piVP.a.x_a_b",s.piVVH.var_p_IH,"s.piVVH.var_p_IH",eps);
    UnitTests.checkDoubleRel(q.piVP.a.osp,"q.piVP.a.osp",s.piVVH.sw_p_IH,"s.piVVH.sw_p_IH",eps);

    System.err.println(name+":RkiQVMTest:test_Aver(): test OK n="+n+" eps="+eps);
}

void test_Spur()
{
    final int n=neq,nmoms=2*n-1;
    final RkiQVM<DTYPE> q=getRkiQVM();
    final double [][] mm=q.getMoments(nmoms);
    final double [] QQt=q.M.B.getQQMatr(n,mm[0]),QQi=q.M.B.getQQMatr(n,mm[1]),
	QQpi=q.M.B.getQQMatr(n,mm[2]),QQp2i=q.M.B.getQQMatr(n,mm[3]),QQp3i=q.M.B.getQQMatr(n,mm[4]);
    Skewness.matchSkewness(q.Sk.sk,DynSki.get_fromTR_DynSki(n,QQi,QQt,QQpi,QQp2i,QQp3i),eps);
    System.err.println(name+":RkiQVMTest:test_Spur(): test OK n="+n+" eps="+eps);
}


void test_Extra(){}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":RkiQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Aver();
    test_Spur();
    test_Extra();

    System.err.println(name+":RkiQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":RkiQVMTest: starting tests; ver=$Id: RkiQVM.java,v 1.23 2022/07/15 00:01:20 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":RkiQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: RkiQVM.java,v 1.23 2022/07/15 00:01:20 mal Exp $");
}
} // END OF class RkiQVMTest




}// END OF class RkiQVM
