package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: RkiQVMLaguerre.java,v 1.9 2022/07/10 10:41:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class RkiQVMLaguerre extends RkiQVM<QVMDataL> {
RkiQVMLaguerre(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}


public static RkiQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final RkiQVMLaguerre ref)
{
    final int npm=4;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*(neq-1)+1+2,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new RkiQVMLaguerre(neq,o,ref!=null?ref.sst:null);
}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}
double getPOffset(){return odata.p_offset;}

static class RkiQVMLaguerreTest extends RkiQVMTest<QVMDataL> {
RkiQVMLaguerreTest(final long initrandom)
{
    super("RkiQVMLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
RkiQVMLaguerre getRkiQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
YpnlQVMLaguerre getYpnlQVM(){return YpnlQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class RkiQVMLaguerreTest

public static void main(String [] args)
{
    new RkiQVMLaguerreTest(-1517L).runTests(6);
} // END main



} // END OF class RkiQVMLaguerre
