package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: RkiQVMMonomials.java,v 1.9 2022/07/10 10:41:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class RkiQVMMonomials extends RkiQVM<QVMData> {
RkiQVMMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static RkiQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final RkiQVMMonomials ref)
{
    final int nvm=1,npm=4;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1+2,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new RkiQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}

static class RkiQVMMonomialsTest extends RkiQVMTest<QVMData> {
RkiQVMMonomialsTest(final long initrandom)
{
    super("RkiQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

@Override
void test_Extra(){
    final RkiQVMMonomials q=getRkiQVM();
    final RkiQVMLaguerre s=getRkiQVMLaguerre();

    DynRki.matchDynRki(q.Rk,s.Rk,eps);
    DynRki.matchDynRki(q.Rk0,s.Rk0,eps);
    DynRki.matchDynRki(q.RkR,s.RkR,eps);
    DynRki.matchDynRki(q.RkV0,s.RkV0,eps);
    DynRki.matchDynRki(q.RkVR,s.RkVR,eps);

    DynSki.matchDynSki(q.Sk,s.Sk,eps);
    DynSki.matchDynSki(q.Sk0,s.Sk0,eps);
    DynSki.matchDynSki(q.SkR,s.SkR,eps);

    final double oq=q.getPOffset(),os=s.getPOffset();
    UnitTests.checkDoubleRel(q.v2I.p_IX+oq,"q.v2I.p_IX",s.v2I.p_IX+os,"s.v2I.p_IX",eps);
    UnitTests.checkDoubleRel(q.v2I.p_DIL+oq,"q.v2I.p_DIL",s.v2I.p_DIL+os,"s.v2I.p_DIL",eps);
    UnitTests.checkDoubleRel(q.v2I.p_DIH+oq,"q.v2I.p_DIH",s.v2I.p_DIH+os,"s.v2I.p_DIH",eps);
    UnitTests.checkDoubleRel(q.v2Io.p_IX+oq,"q.v2Io.p_IX",s.v2Io.p_IX+os,"s.v2Io.p_IX",eps);
    UnitTests.checkDoubleRel(q.v2Io.p_DIL+oq,"q.v2Io.p_DIL",s.v2Io.p_DIL+os,"s.v2Io.p_DIL",eps);
    UnitTests.checkDoubleRel(q.v2Io.p_DIH+oq,"q.v2Io.p_DIH",s.v2Io.p_DIH+os,"s.v2Io.p_DIH",eps);
    UnitTests.checkDoubleRel(q.v2I0.p_IX+oq,"q.v2I0.p_IX",s.v2I0.p_IX+os,"s.v2I0.p_IX",eps);
    UnitTests.checkDoubleRel(q.v2I0.p_DIL+oq,"q.v2I0.p_DIL",s.v2I0.p_DIL+os,"s.v2I0.p_DIL",eps);
    UnitTests.checkDoubleRel(q.v2I0.p_DIH+oq,"q.v2I0.p_DIH",s.v2I0.p_DIH+os,"s.v2I0.p_DIH",eps);
    UnitTests.checkDoubleRel(q.v2I0o.p_IX+oq,"q.v2I0o.p_IX",s.v2I0o.p_IX+os,"s.v2I0o.p_IX",eps);
    UnitTests.checkDoubleRel(q.v2I0o.p_DIL+oq,"q.v2I0o.p_DIL",s.v2I0o.p_DIL+os,"s.v2I0o.p_DIL",eps);
    UnitTests.checkDoubleRel(q.v2I0o.p_DIH+oq,"q.v2I0o.p_DIH",s.v2I0o.p_DIH+os,"s.v2I0o.p_DIH",eps);

    System.err.println(name+":RkiQVMMonomialsTest:test_Extra(): test RkiQVMLaguerre OK neq="+neq+" eps="+eps);
}


RkiQVMLaguerre getRkiQVMLaguerre(){return RkiQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
RkiQVMMonomials getRkiQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
YpnlQVMMonomials getYpnlQVM(){return YpnlQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class RkiQVMMonomialsTest

public static void main(String [] args)
{
    new RkiQVMMonomialsTest(17L).runTests(5);
} // END main



} // END OF class RkiQVMMonomials
