package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */

/** Save state.
 *  @version $Id: SMMStateSaver.java,v 1.5 2014/07/29 17:50:40 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class SMMStateSaver {
    public static class SMM {
	public final double Ic,Pc,Fc;
	public final long Tc,Uc,Vc;
	double PcMin,PcMax;
	SMM(final double Ic,final double Pc,final long Tc,final double Fc,final long Uc,final long Vc)
	{this.Ic=Ic;this.Pc=Pc;this.Tc=Tc;this.Fc=Fc;this.Uc=Uc;this.Vc=Vc;this.PcMin=this.PcMax=Pc;}

	void updatePcMM(final double p){if(!(p>this.PcMin))this.PcMin=p;if(!(p<this.PcMax))this.PcMax=p;}

	static final SMM EMPTY=new SMM(Double.NaN,Double.NaN,0,Double.NaN,0,0);
    }

    public final SMM L_before,H,L_after;
    private final int from;
    double hPcMin=Double.NaN,hPcMax=Double.NaN;

    private static final int F_LOW=2891,F_HIGH=2892,F_NONE=2893;

    SMMStateSaver(final SMM L_before,final SMM H,final SMM L_after,final int from)
    {
	this.L_before=L_before;
	this.H=H;
	this.L_after=L_after;
	this.from=from;
    }

    public long getVb(){
	if(L_before!=SMM.EMPTY && H!=SMM.EMPTY) return H.Vc-L_before.Vc;
	return 0;
    }
    public long getVf(){
	if(from==F_LOW && H!=SMM.EMPTY && L_after!=SMM.EMPTY) return L_after.Vc-H.Vc;
	return 0;
    }

    public static SMMStateSaver getHighUpdate(final double I,final double p,final long t,final double F,final long U,final long V,final SMMStateSaver rr)
    {
	final SMMStateSaver r=getNothingUpdate(p,rr);
	if(r.from!=F_HIGH || (r.from==F_HIGH && !(I<r.H.Ic)))
	{
	    final SMM sm=r.L_after!=SMM.EMPTY?r.L_after:r.L_before;
	    final SMMStateSaver s=new SMMStateSaver(sm,new SMM(I,p,t,F,U,V),SMM.EMPTY,F_HIGH);
	    s.hPcMin=sm.PcMin;
	    s.hPcMax=sm.PcMax;
	    return s;
	}
	return r;
    }

    public static SMMStateSaver getLowUpdate(final double I,final double p,final long t,final double F,final long U,final long V,final SMMStateSaver rr)
    {
	final SMMStateSaver r=getNothingUpdate(p,rr);
	if(r.from!=F_LOW || (r.from==F_LOW && !(I>r.L_after.Ic)))
	    return new SMMStateSaver(r.L_before,r.H,new SMM(I,p,t,F,U,V),F_LOW); // also resents min/max for L_after
	return r;
    }

    public static SMMStateSaver getNothingUpdate(final double p,final SMMStateSaver rr)
    {
	final SMMStateSaver r=rr!=null?rr:new SMMStateSaver(SMM.EMPTY,SMM.EMPTY,SMM.EMPTY,F_NONE);
	r.L_before.updatePcMM(p);
	r.H.updatePcMM(p);
	r.L_after.updatePcMM(p);
	return r;
    }
}
