package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
/**
 * A helper class to store the moments <fQ_k>;
 * to obtain the matrices $<Q_j|f|Q_k> use the method asQQMatrices().
 * @version $Id: SMomentsData.java,v 1.30 2022/09/14 08:19:26 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class SMomentsData {
    final long currenttime,totalvolume;
    /** The dimension. If flagQQ=false, then n is the number of moments stored,
     * if flagQQ=true, then n is matrix dimension, arrays size is n*n.
     */
    public final int n;
    public final double p_offset,p_last;
    public final double [] tmom,p0tmom,p1tmom,p2tmom,p3tmom,imom,pimom,p2imom,p3imom,
	Timom,Tpimom,Tp2imom,Tp3imom,
	Tmom,p0Tmom,p1Tmom,p2Tmom,p3Tmom,Vmom,pVmom,p2Vmom,p3Vmom,dpmom,Tdpmom,amom,pamom,
	VdPdtmom;
    /** An indication of what is stored in the class.
     * If false then the .*mom are <Q_m|f> moments , m=0..n-1.
     * If true then the .*mom are <Q_j|f|Q_k> matrices of n x n dimension.
     */
    public final boolean flagQQ;
    public SMomentsData(final int n,final long currenttime,final long totalvolume,final double p_offset,final double p_last,
			final double [] tmom, // exact moments
			final double [] p0tmom, // the same or almost the same as tmom
			final double [] p1tmom, // prefix p1 is only for t (historical reasons, exact & from sample)
			final double [] p2tmom,
			final double [] p3tmom,
			final double [] imom, // no prefix for i
			final double [] pimom, // prefix p
			final double [] p2imom,
			final double [] p3imom,
			final double [] Timom,
			final double [] Tpimom,
			final double [] Tp2imom,
			final double [] Tp3imom,
			final double [] Tmom, // exact
			final double [] p0Tmom,
			final double [] p1Tmom,
			final double [] p2Tmom,
			final double [] p3Tmom,
			final double [] Vmom,
			final double [] pVmom,
			final double [] p2Vmom,
			final double [] p3Vmom,
			final double [] dpmom,
			final double [] Tdpmom,
			final double [] amom,
			final double [] pamom,
			final double [] VdPdtmom,
			final boolean flagQQ)
    {
	this.n=n;this.currenttime=currenttime;this.totalvolume=totalvolume;this.p_offset=p_offset;this.p_last=p_last;
	this.tmom=tmom;
	this.p0tmom=p0tmom;
	this.p1tmom=p1tmom;
	this.p2tmom=p2tmom;
	this.p3tmom=p3tmom;
	this.imom=imom;
	this.pimom=pimom;
	this.p2imom=p2imom;
	this.p3imom=p3imom;
	this.Timom=Timom;
	this.Tpimom=Tpimom;
	this.Tp2imom=Tp2imom;
	this.Tp3imom=Tp3imom;
	this.Tmom=Tmom;
	this.p0Tmom=p0Tmom;
	this.p1Tmom=p1Tmom;
	this.p2Tmom=p2Tmom;
	this.p3Tmom=p3Tmom;
	this.Vmom=Vmom;
	this.pVmom=pVmom;
	this.p2Vmom=p2Vmom;
	this.p3Vmom=p3Vmom;
	this.dpmom=dpmom;
	this.Tdpmom=Tdpmom;
	this.amom=amom;
	this.pamom=pamom;
	this.VdPdtmom=VdPdtmom;
	this.flagQQ=flagQQ;
    }
    /** Convert <fQ_k> internal vectors to <Q_j|f|Q_k> matrices,
     * the names are the same, the arrays size to become ndim*ndim.
     * @param ndmim Matrices dimension, the returned arrays to be the ndim*ndim size,
     * the original object to contain at least 2*ndim-1 moments.
     * @param M basis operator.
     * @return An object of SMomentsData type, with n=ndim ; all .*mom are now <Q_j|f|Q_k> matrices of ndim x ndim dimension, the original object has .*mom as  <Q_m|f> , 0..n-1. Typical use case:
     * final SMomentsData mm=get_MomentsData(2*d-1,odata,false).asQQMatrices(d,M);
     */
    public SMomentsData asQQMatrices(final int ndim,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M){
	if(flagQQ) throw new IllegalArgumentException();
	final int n=ndim;
	return new SMomentsData(n,currenttime,totalvolume,p_offset,p_last,
				M.B.getQQMatr(n,tmom),
				M.B.getQQMatr(n,p0tmom),
				M.B.getQQMatr(n,p1tmom),
				M.B.getQQMatr(n,p2tmom),
				M.B.getQQMatr(n,p3tmom),
				M.B.getQQMatr(n,imom),
				M.B.getQQMatr(n,pimom),
				M.B.getQQMatr(n,p2imom),
				M.B.getQQMatr(n,p3imom),
				M.B.getQQMatr(n,Timom),
				M.B.getQQMatr(n,Tpimom),
				M.B.getQQMatr(n,Tp2imom),
				M.B.getQQMatr(n,Tp3imom),
				M.B.getQQMatr(n,Tmom),
				M.B.getQQMatr(n,p0Tmom),
				M.B.getQQMatr(n,p1Tmom),
				M.B.getQQMatr(n,p2Tmom),
				M.B.getQQMatr(n,p3Tmom),
				M.B.getQQMatr(n,Vmom),
				M.B.getQQMatr(n,pVmom),
				M.B.getQQMatr(n,p2Vmom),
				M.B.getQQMatr(n,p3Vmom),
				M.B.getQQMatr(n,dpmom),
				M.B.getQQMatr(n,Tdpmom),
				M.B.getQQMatr(n,amom),
				M.B.getQQMatr(n,pamom),
				M.B.getQQMatr(n,VdPdtmom),
				true);
    }

    /** Extract moments data from moments storage DTYPE.
     * @param nmoms The number of moments to extract.
     * @param odata Moments storage.
     * @param access Moments access function.
     * @param use_sampled_data_for_dt If true then sampled dt is used for <Q_kp^m> moments,
     * otherwise we use <Q_k dp^m/dt> and then integrate by parts.
     * @return Extracted moments.
     */
    public static <DTYPE> SMomentsData get_MomentsData(final int nmoms,final DTYPE odata,final DataInterfaceToMoments<DTYPE> access,final boolean use_sampled_data_for_dt)
{
    final double [] tmom=new double [nmoms];access.setMT(tmom,nmoms);
    final double [] p0tmom=new double [nmoms],p1tmom=new double [nmoms],p2tmom=new double [nmoms],p3tmom=new double [nmoms];
    if(use_sampled_data_for_dt){
	// direct sample
	access.setM(p0tmom,0,access.M_T,odata,nmoms);
	access.setM(p1tmom,1,access.M_T,odata,nmoms);
	access.setM(p2tmom,2,access.M_T,odata,nmoms);
	access.setM(p3tmom,3,access.M_T,odata,nmoms);
    } else {
	// integration by parts
	final double pl_m_poffset=access.getPLast(odata);// p_offset is already included in access.getPLast(DTYPE odata).
	final double [] dp0mom=new double [nmoms],dp1mom=new double [nmoms],dp2mom=new double [nmoms],dp3mom=new double [nmoms];
	access.setM(dp0mom,0,access.M_DP,odata,nmoms);
	access.setM(dp1mom,1,access.M_DP,odata,nmoms);
	access.setM(dp2mom,2,access.M_DP,odata,nmoms);
	access.setM(dp3mom,3,access.M_DP,odata,nmoms);
	// shift to p_offset=p_last
	com.polytechnik.utils.Polynomials.shiftPOffsetInPlace(nmoms,dp0mom,dp1mom,dp2mom,dp3mom,pl_m_poffset);
	// integrate by parts
	access.setMT(p0tmom,nmoms);
	access.setFMoments(p1tmom,dp1mom,nmoms);
	access.setFMoments(p2tmom,dp2mom,nmoms);
	access.setFMoments(p3tmom,dp3mom,nmoms);
	// shift price offset back to common value. (integration by parts give last price as the offset)
	com.polytechnik.utils.Polynomials.shiftPOffsetInPlace(nmoms,p0tmom,p1tmom,p2tmom,p3tmom,-pl_m_poffset);
    }
    final double [] Timom=new double [nmoms];access.setM(Timom,0,access.M_TI,odata,nmoms);
    final double [] Tpimom=new double [nmoms];access.setM(Tpimom,1,access.M_TI,odata,nmoms);
    final double [] Tp2imom=new double [nmoms];access.setM(Tp2imom,2,access.M_TI,odata,nmoms);
    final double [] Tp3imom=new double [nmoms];access.setM(Tp3imom,3,access.M_TI,odata,nmoms);
    final double [] Tmom=new double [nmoms];access.setFMoments(Tmom,tmom,nmoms);
    final double [] p0Tmom=new double [nmoms];access.setFMoments(p0Tmom,p0tmom,nmoms);
    final double [] p1Tmom=new double [nmoms];access.setFMoments(p1Tmom,p1tmom,nmoms);
    final double [] p2Tmom=new double [nmoms];access.setFMoments(p2Tmom,p2tmom,nmoms);
    final double [] p3Tmom=new double [nmoms];access.setFMoments(p3Tmom,p3tmom,nmoms);

    final double [] imom=new double [nmoms];access.setM(imom,0,access.M_I,odata,nmoms);
    final double [] pimom=new double [nmoms];access.setM(pimom,1,access.M_I,odata,nmoms);
    final double [] p2imom=new double [nmoms];access.setM(p2imom,2,access.M_I,odata,nmoms);
    final double [] p3imom=new double [nmoms];access.setM(p3imom,3,access.M_I,odata,nmoms);
    final double [] Vmom=new double [nmoms];access.setFMoments(Vmom,imom,nmoms);
    final double [] pVmom=new double [nmoms];access.setFMoments(pVmom,pimom,nmoms);
    final double [] p2Vmom=new double [nmoms];access.setFMoments(p2Vmom,p2imom,nmoms);
    final double [] p3Vmom=new double [nmoms];access.setFMoments(p3Vmom,p3imom,nmoms);

    // dp/dt moments
    final double [] dpmom=new double [nmoms];access.setM(dpmom,1,access.M_DP,odata,nmoms);
    // T*dp/dt moments
    final double [] Tdpmom=new double [nmoms];access.setM(Tdpmom,1,access.M_TDP,odata,nmoms);
    // absolute value of dp/dt, "a surrogate of imom".
    final double [] amom=new double [nmoms];access.setM(amom,0,access.M_A,odata,nmoms);
    final double [] pamom=new double [nmoms];access.setM(pamom,1,access.M_A,odata,nmoms);
    // d(PV)/dt
    final double [] VdPdtmom=new double [nmoms];access.setM(VdPdtmom,1,access.M_VdPdt,odata,nmoms);

    return new SMomentsData(nmoms,access.getTLast(odata),access.getTotalVolume(odata),access.getPOffset(odata),access.getPLast(odata),tmom,
			    p0tmom,p1tmom,p2tmom,p3tmom,
			    imom,pimom,p2imom,p3imom,
			    Timom,Tpimom,Tp2imom,Tp3imom,
			    Tmom,p0Tmom,p1Tmom,p2Tmom,p3Tmom,
			    Vmom,pVmom,p2Vmom,p3Vmom,
			    dpmom,Tdpmom,amom,pamom,VdPdtmom,false);
}

}
