package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;

/** A code for action minimization. I basis.
 *  @version $Id: SPIMin.java,v 1.43 2022/06/27 13:32:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class SPIMin {

private final int neq_t,neq_p;
private final DeprecatedHelperBasisInterface basis;

SPIMin(final int neq_t,final int neq_p,final DeprecatedHelperBasisInterface basis)
{
    this.neq_t=neq_t;
    this.neq_p=neq_p;
    this.basis=basis;
    if(neq_t!=basis.get_ntm()) throw new IllegalArgumentException(" neq_t="+neq_t+" basis.get_ntm()="+basis.get_ntm());
    if(neq_p>basis.get_npm()||basis.get_npm()<2)
	throw new IllegalArgumentException(" neq_p="+neq_p+" basis.get_npm()="+basis.get_npm());

}

static class SPIMinData {
    final double [] dissp,inter;
    final double io,sigma2_ii,scale_io,scale_io0,p_eq,sigma2_eq,dissp_scaletoP2;
    SPIMinData(final double [] dissp,
	       final double [] inter,
	       final double io,
	       final double sigma2_ii,
	       final double scale_io,
	       final double scale_io0,
	       final double p_eq,
	       final double sigma2_eq,
	       final double dissp_scaletoP2)
    {
	this.dissp=dissp;
	this.inter=inter;
	this.io=io;
	this.sigma2_ii=sigma2_ii;
	this.scale_io=scale_io;
	this.scale_io0=scale_io0;
	this.p_eq=p_eq;
	this.sigma2_eq=sigma2_eq;
	this.dissp_scaletoP2=dissp_scaletoP2;
    }
}

SPIMinData getI0P()
{
    final int neq_t=this.neq_t;
    final double [] i=new double [neq_t];
    basis.setI(i);
    final double []  dpk=new double [neq_t],dpl=new double [neq_t];
    final int np=this.neq_p;
    double dp_dp=Double.NaN,dp_o=Double.NaN;
    final double [] matr=new double [np*np],rpart=new double [np],i_dp=new double [np];
    for(int k=0;k<np;++k)
    {
	basis.setDPn(dpk,k);
	if(k==1)
	{
	    dp_dp=basis.get_f_g(dpk,dpk);
	    dp_o=basis.get_f_o(dpk);
	}
	for(int l=0;l<=k;++l)
	{
	    basis.setDPn(dpl,l);
	    matr[k+np*l]=matr[l+np*k]=basis.get_f_g(dpk,dpl);
	}
	i_dp[k]=rpart[k]=basis.get_f_g(i,dpk);
    }
    //System.err.println("matr="+java.util.Arrays.toString(matr));
    final int [] ipvt=new int [np];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,np,np,ipvt); // matr is clobbered now
    if(pinfo!=0)
    {
	java.util.Arrays.fill(rpart,Double.NaN);
    }
    else
    {
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,np,np,ipvt,rpart,1);
    }
    final double i_o=basis.get_f_o(i),i_i=basis.get_f_g(i,i),o_o=basis.get_o_o(),
	x_i_i=i_i-i_o*(i_o/o_o),sigma2_ii=i_i-TMatr.mulVVn1(rpart,i_dp);
    for(int k=0;k<np;k++)
    { // need to inverse sign when using our definition of dpk.
	rpart[k]=-o_o*rpart[k];
    }

    final double dp_eq=dp_o/o_o;
    return new SPIMinData(new double []{dp_dp,-2*dp_o,o_o},
			  rpart,
			  i_o,
			  sigma2_ii,
			  Math.sqrt(o_o*sigma2_ii),
			  Math.sqrt(o_o*x_i_i),
			  basis.getPOffset()+dp_eq,
			  dp_dp/o_o-dp_eq*dp_eq,
			  o_o);
}

double [] getDissp()
{
    final double [] dp=new double [neq_t];
    basis.setDPn(dp,1);
    return new double []{basis.get_f_g(dp,dp),-2*basis.get_f_o(dp),basis.get_o_o()};
}


double [] getP0I()
{
    final double [] dp=new double [neq_t];
    basis.setDPn(dp,1);
    final double [] jk=new double [neq_t],jl=new double [neq_t];
    final int np=neq_p;
    final double [] matr=new double [np*np],rpart=new double [np],dp_j=new double [np];
    for(int k=0;k<np;++k)
    {
	if(k<=0) basis.setDPn(jk,0);
	else basis.setIPn(jk,k-1);
	for(int l=0;l<=k;++l)
	{
	    if(l<=0) basis.setDPn(jl,0);
	    else basis.setIPn(jl,l-1);
	    matr[k+np*l]=matr[l+np*k]=basis.get_f_g(jk,jl);
	}
	dp_j[k]=rpart[k]=basis.get_f_g(dp,jk);
    }
    //System.err.println("matr="+java.util.Arrays.toString(matr));
    final int [] ipvt=new int [np];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,np,np,ipvt); // matr is clobbered now
    if(pinfo!=0)
    {
	return new double []{Double.NaN,Double.NaN};
    }
    com.polytechnik.utils.Linsystems.dgesl_m1(matr,np,np,ipvt,rpart,1);
    //System.err.println("getP0I():rpart="+java.util.Arrays.toString(rpart)+" np="+np+" offset="+basis.getPOffset());
    final double p_eq=rpart[0],sigma2_eq=(basis.get_f_g(dp,dp)-TMatr.mulVVn1(rpart,dp_j))/basis.get_o_o();
    return new double []{basis.getPOffset()+p_eq,sigma2_eq};
}

static abstract class SPIMinTest<DTYPE> {
final String name;
final double eps,epssum;
SPIMinTest(final String name,final double eps,final double epssum)
{
    this.name=name;
    this.eps=eps;
    this.epssum=epssum;
}

abstract DTYPE getData(final int tsize,final int ntm,final int npm,final java.util.Random r);
abstract SPIMin getSPIMin(final int neq_t,final int neq_p,final DTYPE o);

private void checkN2(final int tsize,final java.util.Random r,final int neq_t)
{
    System.err.println(name+":SPIMinTest:checkN2(): neq_t="+neq_t);
    final SPIMin sm=getSPIMin(neq_t,2,getData(tsize,neq_t,2,r));
    final double [] i=new double [neq_t],dp=new double [neq_t],o=new double [neq_t];
    sm.basis.setIPn(i,0);
    sm.basis.setDPn(o,0);
    sm.basis.setDPn(dp,1);
    final double i_o=sm.basis.get_f_g(i,o),dp_o=sm.basis.get_f_g(dp,o),o_o=sm.basis.get_f_g(o,o);
    TMatr.sumP1(i,o,-i_o/o_o);
    TMatr.sumP1(dp,o,-dp_o/o_o);
    final double x_i_dp=sm.basis.get_f_g(i,dp),x_dp_dp=sm.basis.get_f_g(dp,dp),x_i_i=sm.basis.get_f_g(i,i);
    final double [] ps=new double []{sm.basis.getPOffset()+dp_o/o_o-(x_i_dp/x_i_i)*(i_o/o_o),(x_dp_dp-x_i_dp*x_i_dp/x_i_i)/o_o};
    UnitTests.checkDoubleArrayRel(ps,"ps",sm.getP0I(),"sm.getP0I()",eps);
    System.err.println(name+":SPIMinTest:checkN2(): neq_t="+neq_t+" TEST OK");
}
private void checkN3(final int tsize,final java.util.Random r,final int neq_t)
{
    System.err.println(name+":SPIMinTest:checkN3(): neq_t="+neq_t);
    final SPIMin sm=getSPIMin(neq_t,3,getData(tsize,neq_t,3,r));
    final double [] dp=new double [neq_t],o=new double [neq_t],i=new double [neq_t],j=new double [neq_t];
    sm.basis.setDPn(dp,1);
    sm.basis.setDPn(o,0);
    sm.basis.setIPn(i,0);
    sm.basis.setIPn(j,1);
    final double [] interdir;
    if(true)
    {
	final double [][] v=new double [][]{o,i,j};
	interdir=new double [v.length];
	final double [] matr=new double [v.length*v.length];
	for(int k=0;k<interdir.length;k++)
	{
	    interdir[k]=sm.basis.get_f_g(v[k],dp);
	    for(int l=0;l<=k;l++)
	    {
		matr[k+l*v.length]=matr[l+k*v.length]=sm.basis.get_f_g(v[k],v[l]);
	    }
	}
	final int [] ipvt=new int [v.length];
	final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,v.length,v.length,ipvt);
	if(pinfo!=0) throw new RuntimeException("pinfo="+pinfo);
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,v.length,v.length,ipvt,interdir,1);
    }
    final double dp_dp=sm.basis.get_f_g(dp,dp),o_o=sm.basis.get_f_g(o,o),
	fi_o=sm.basis.get_f_g(i,o)/o_o,fj_o=sm.basis.get_f_g(j,o)/o_o,
	dp_o=sm.basis.get_f_g(dp,o),fdp_o=dp_o/o_o,dp_i=sm.basis.get_f_g(dp,i),dp_j=sm.basis.get_f_g(dp,j);
    TMatr.sumP1(dp,o,-fdp_o);
    TMatr.sumP1(i,o,-fi_o);
    TMatr.sumP1(j,o,-fj_o);
    final double x_i_dp=sm.basis.get_f_g(i,dp),x_dp_dp=sm.basis.get_f_g(dp,dp),x_i_i=sm.basis.get_f_g(i,i),
	fdp_qi=x_i_dp/x_i_i,fj_qi=sm.basis.get_f_g(j,i)/x_i_i;
    TMatr.sumP1(j,i,-fj_qi);
    TMatr.sumP1(dp,i,-fdp_qi);
    final double xx_dp_j=sm.basis.get_f_g(dp,j),xx_j_j=sm.basis.get_f_g(j,j),xx_dp_dp=sm.basis.get_f_g(dp,dp),
	fdp_qj=xx_dp_j/xx_j_j;
    final double [] inter=new double []{fdp_o-fdp_qi*fi_o-fdp_qj*(fj_o-fj_qi*fi_o),fdp_qi-fdp_qj*fj_qi,fdp_qj};
    System.err.println("checkN3:inter="+java.util.Arrays.toString(inter));
    UnitTests.checkDoubleRel(sm.basis.getPOffset()+inter[0],"offset+inter[0]",sm.getP0I()[0],"sm.getP0I()[0]",eps);
    UnitTests.checkDoubleArrayRel(inter,"inter",interdir,"interdir",eps);
    final double sigma2_eq=(dp_dp-dp_o*dp_o/o_o-x_i_dp*x_i_dp/x_i_i-xx_dp_j*xx_dp_j/xx_j_j)/o_o;
    UnitTests.checkDoubleRel(sigma2_eq,"sigma2_eq",sm.getP0I()[1],"sm.getP0I()[1]",eps);
    final double asigma2_eq=(dp_dp-TMatr.mulVV(inter,new double []{dp_o,dp_i,dp_j}))/o_o;
    UnitTests.checkDoubleRel(sigma2_eq,"sigma2_eq",asigma2_eq,"asigma2_eq",eps);

    if(true)
    {
	final double [] ri=inter.clone();
	for(int k=0;k<ri.length;k++) ri[k]=-ri[k];
	double s=0;
	final double [] jk=new double [neq_t];
	for(int k=1;k<ri.length;k++)
	{
	    sm.basis.setIPn(jk,k-1);
	    s+=ri[k]*sm.basis.get_f_o(jk);
	}
	sm.basis.setDPn(jk,1);
	s+=sm.basis.get_f_o(jk);
	s/=sm.basis.get_o_o();
	// match to p_eq
	UnitTests.checkDoubleRel(s,"s",inter[0],"inter[0]",eps);
    }

    System.err.println(name+":SPIMinTest:checkN3(): neq_t="+neq_t+" TEST OK");
}
private void i_checkN2(final int tsize,final java.util.Random r,final int neq_t)
{
    System.err.println(name+":SPIMinTest:i_checkN2(): neq_t="+neq_t);
    final SPIMin sm=getSPIMin(neq_t,2,getData(tsize,neq_t,2,r));
    final double [] i=new double [neq_t],dp=new double [neq_t],o=new double [neq_t];
    sm.basis.setI(i);
    sm.basis.setDPn(o,0);
    sm.basis.setDPn(dp,1);
    final double i_o=sm.basis.get_f_g(i,o),dp_o=sm.basis.get_f_g(dp,o),o_o=sm.basis.get_f_g(o,o),
	i_i=sm.basis.get_f_g(i,i);
    TMatr.sumP1(i,o,-i_o/o_o);
    TMatr.sumP1(dp,o,-dp_o/o_o);
    final double x_i_dp=sm.basis.get_f_g(i,dp),x_dp_dp=sm.basis.get_f_g(dp,dp);
    final double [] inter=new double[]{-i_o/o_o+(dp_o/o_o)*x_i_dp/x_dp_dp,-x_i_dp/x_dp_dp};
    for(int k=inter.length;--k>=0;) inter[k]*=o_o;
    UnitTests.checkDoubleArrayRel(inter,"inter",sm.getI0P().inter,"sm.getI0P().inter",eps);
    final double sigma2_ii=i_i-i_o*i_o/o_o-x_i_dp*x_i_dp/x_dp_dp;
    UnitTests.checkDoubleRel(sigma2_ii,"sigma2_ii",sm.getI0P().sigma2_ii,"sm.getI0P().sigma2_ii",eps);
    System.err.println(name+":SPIMinTest:i_checkN2(): neq_t="+neq_t+" TEST OK");
}
private void i_checkN3(final int tsize,final java.util.Random r,final int neq_t)
{
    System.err.println(name+":SPIMinTest:i_checkN3(): neq_t="+neq_t);
    final SPIMin sm=getSPIMin(neq_t,3,getData(tsize,neq_t,3,r));
    final double [] i=new double [neq_t],dp=new double [neq_t],dp2=new double [neq_t],o=new double [neq_t];
    sm.basis.setI(i);
    sm.basis.setDPn(o,0);
    sm.basis.setDPn(dp,1);
    sm.basis.setDPn(dp2,2);
    final double o_o=sm.basis.get_f_g(o,o),fdp_o=sm.basis.get_f_g(dp,o)/o_o,fdp2_o=sm.basis.get_f_g(dp2,o)/o_o,fi_o=sm.basis.get_f_g(i,o)/o_o;
    final double i_o=sm.basis.get_f_g(i,o),i_i=sm.basis.get_f_g(i,i);
    TMatr.sumP1(i,o,-fi_o);
    TMatr.sumP1(dp,o,-fdp_o);
    TMatr.sumP1(dp2,o,-fdp2_o);
    final double x_i_dp=sm.basis.get_f_g(i,dp),x_dp_dp=sm.basis.get_f_g(dp,dp),
	x_i_i=sm.basis.get_f_g(i,i),fi_q1=x_i_dp/x_dp_dp,fdp2_q1=sm.basis.get_f_g(dp2,dp)/x_dp_dp;
    TMatr.sumP1(dp2,dp,-fdp2_q1);
    TMatr.sumP1(i,dp,-fi_q1);
    final double xx_i_dp2=sm.basis.get_f_g(i,dp2),xx_dp2_dp2=sm.basis.get_f_g(dp2,dp2),xx_i_i=sm.basis.get_f_g(i,i),
	fi_q2=xx_i_dp2/xx_dp2_dp2;
    final double [] inter=new double []{-fi_o+fi_q1*fdp_o+fi_q2*(fdp2_o-fdp2_q1*fdp_o),-fi_q1+fi_q2*fdp2_q1,-fi_q2};
    for(int k=inter.length;--k>=0;) inter[k]*=o_o;
    UnitTests.checkDoubleArrayRel(inter,"inter",sm.getI0P().inter,"sm.getI0P().inter",eps);

    final double sigma2_ii=i_i-i_o*i_o/o_o-x_i_dp*x_i_dp/x_dp_dp-xx_i_dp2*xx_i_dp2/xx_dp2_dp2;
    UnitTests.checkDoubleRel(sigma2_ii,"sigma2_ii",sm.getI0P().sigma2_ii,"sm.getI0P().sigma2_ii",eps);
    System.err.println(name+":SPIMinTest:i_checkN3(): neq_t="+neq_t+" TEST OK");
}
private void i_check0Coef(final int tsize,final java.util.Random r,final int neq_t,final int neq_p)
{
    System.err.println(name+":SPIMinTest:i_check0Coef(): neq_t="+neq_t+" neq_p="+neq_p);
    final SPIMin sm=getSPIMin(neq_t,neq_p,getData(tsize,neq_t,neq_p,r));
    final SPIMin.SPIMinData xx=sm.getI0P();
    UnitTests.checkDoubleArrayRel(xx.dissp,"xx.dissp",sm.getDissp(),"sm.getDissp()",eps);
    final double [] i0p=xx.inter;
    final double [] dpk=new double [neq_t];
    final double o_o=sm.basis.get_o_o();
    double s=0;
    for(int k=1;k<neq_p;k++)
    {
	sm.basis.setDPn(dpk,k);
	s-=i0p[k]*(sm.basis.get_f_o(dpk)/o_o);
    }
    sm.basis.setI(dpk);
    final double i_o=sm.basis.get_f_o(dpk);
    s-=i_o;
    // i0p[0]=-<i|o>-sum_{k=1}^{k=neq_p-1} i0p[k]*(|dp^k>/o_o)
    UnitTests.checkDoubleRel(s,"s",i0p[0],"i0p[0]",eps);
    UnitTests.checkDoubleRel(i_o,"i_o",xx.io,"xx.io",eps);
    System.err.println(name+":SPIMinTest:i_check0Coef(): neq_t="+neq_t+" neq_p="+neq_p+" 0Coef test OK");
}
void checkCalculations(final int tsize,final java.util.Random r,final int neq_t,final int neq_p)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":SPIMinTest:checkCalculations(): test with N points="+tsize+
		       " neq_t="+neq_t+" neq_p="+neq_p);
    //checkI2(tsize,r,neq_t,neq_p);
    checkN2(tsize,r,neq_t);
    checkN3(tsize,r,neq_t);
    i_checkN2(tsize,r,neq_t);
    i_checkN3(tsize,r,neq_t);
    i_check0Coef(tsize,r,neq_t,neq_p);
    System.err.println(name+":SPIMinTest:checkCalculations(): test OK with N points="+tsize+
		       " neq_t="+neq_t+" neq_p="+neq_p+" eps="+eps+" epssum="+epssum);
}
void runTests()
{
    System.err.println(name+":SPIMinTest: starting tests; ver=$Id: SPIMin.java,v 1.43 2022/06/27 13:32:46 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79091);
    for(int neq_t=1;neq_t<=9;neq_t++)
    {
	for(int neq_p=2;neq_p<=Math.min(neq_t-4,5);neq_p++)
	{
	    for(int i=0;i<118;i++)
	    {
		checkCalculations(210+r.nextInt(210),r,neq_t,neq_p);
	    }
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":SPIMinTest: tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: SPIMin.java,v 1.43 2022/06/27 13:32:46 mal Exp $");
}
} // END OF class SPIMinTest


}


