package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.realtimebook.*;

/** Impact QVM calculations.
 *  @version $Id: SQVM.java,v 1.45 2022/06/27 13:32:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class SQVM<DTYPE> {

final double [] rnodes,wnodes,xnodes,dpnodes,dipnodes,px,p2x;

final double [] pnodes,p2nodes,pnodes2,p2nodes2;
final double dp00;

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
SQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final int nmoms=2*neq;
    final double [] imom=new double [nmoms];
    access.setM(imom,0,access.M_I,odata,nmoms);
    final double v0=imom[0];
    for(int k=nmoms;--k>=0;) imom[k]/=v0;

    final double [] ep=new double [neq],ep2=new double [neq];
    access.setM(ep,1,access.M_I,odata,neq);
    access.setM(ep2,2,access.M_I,odata,neq);
    for(int k=neq;--k>=0;) {ep[k]/=v0;ep2[k]/=v0;}
    final double [][] xx=getNodes(neq,imom,ep,ep2);
    rnodes=xx[0];
    final double [] ptr=xx[1],p2tr=xx[2];
    xnodes=getXNodes(rnodes);
    wnodes=getWeights(xnodes,rnodes,imom);
    px=new double [xnodes.length];
    p2x=new double [xnodes.length];
    for(int k=xnodes.length;--k>=0;)
    {
	final double dpf=fx(neq,ptr,xnodes[k]),dp2f=fx(neq,p2tr,xnodes[k]);
	px[k]=dpf+getPOffset();
	p2x[k]=dp2f-dpf*dpf;
    }


    access.setM(imom,1,access.M_I,odata,nmoms);
    for(int k=nmoms;--k>=0;) imom[k]/=v0;
    pnodes=getWeights(xnodes,rnodes,imom);
    pnodes2=getWeights2(xnodes,rnodes,imom);

    access.setM(imom,2,access.M_I,odata,nmoms);
    for(int k=nmoms;--k>=0;) imom[k]/=v0;
    p2nodes=getWeights(xnodes,rnodes,imom);
    p2nodes2=getWeights2(xnodes,rnodes,imom);
    for(int k=0;k<wnodes.length;k++)
    {
	//System.err.println("w="+wnodes[k]+" pnodes2[k]="+pnodes2[k]+" p2nodes2[k]="+p2nodes2[k]);
	final double w=wnodes[k],dp=pnodes[k]/w,dp2=p2nodes[k]/w;
	pnodes[k]=dp+getPOffset();
	p2nodes[k]=dp2-dp*dp;
	final double sdp=pnodes2[k]/w,sdp2=p2nodes2[k]/w;
	pnodes2[k]=sdp+getPOffset();
	p2nodes2[k]=sdp2-sdp*sdp;
    }

    if(true)
    {
	final double [] mom=new double [neq];
	access.setM(mom,1,access.M_DPPLUS,odata,1);
	final double dpp0=mom[0];
	access.setM(mom,1,access.M_DPMINUS,odata,1);
	final double d0=dpp0-mom[0];
	this.dp00=d0;
	access.setM(mom,1,access.M_DP,odata,mom.length);
	for(int k=mom.length;--k>=0;) mom[k]/=d0;
	dpnodes=getWeights(xnodes,rnodes,mom);

	final double [] e=new double [neq];
	access.setM(mom,0,access.M_ISELL,odata,e.length);
	access.setM(e,0,access.M_IBUY,odata,e.length);
	final double vv0=mom[0]+e[0];
	for(int k=e.length;--k>=0;) {e[k]=(mom[k]-e[k])/vv0;}
	dipnodes=getWeights(xnodes,rnodes,e);
    }
}

double [][] getNodes(final int neq,final double [] imom,final double [] pmom,final double [] p2mom)
{
    final double [] rnodes=new double [neq+1],e=new double [2*neq],rp=new double [neq],rp2=new double [neq];
    rnodes[neq]=1;
    final double [] matr=new double [neq*neq];
    for(int k=0;k<neq;k++)
    {
	final int naa=access.setQQCoefs(neq,k,e);
	rnodes[k]=-TMatr.mulVVn(naa,imom,e);
	rp[k]=pmom[k];
	rp2[k]=p2mom[k];
	for(int l=0;l<=k;l++)
	{
	    final int na=access.setQQCoefs(l,k,e);
	    matr[k+l*neq]=matr[l+k*neq]=TMatr.mulVVn(na,e,imom);
	}
    }
    final int [] ipvt=new int[neq];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,neq,neq,ipvt);
    if(pinfo!=0)
    {
	java.util.Arrays.fill(rnodes,Double.NaN);
	java.util.Arrays.fill(rp,Double.NaN);
	java.util.Arrays.fill(rp2,Double.NaN);
    }
    else
    {
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq,neq,ipvt,rnodes,1);
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq,neq,ipvt,rp,1);
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq,neq,ipvt,rp2,1);
    }
    return new double [][]{rnodes,rp,rp2};
}
double [] getNodes(final int neq,final double [] imom)
{
    final double [] rnodes=new double [neq+1],e=new double [2*neq];
    rnodes[neq]=1;
    final double [] matr=new double [neq*neq];
    for(int k=0;k<neq;k++)
    {
	final int naa=access.setQQCoefs(neq,k,e);
	rnodes[k]=-TMatr.mulVVn(naa,imom,e);
	for(int l=0;l<=k;l++)
	{
	    final int na=access.setQQCoefs(l,k,e);
	    matr[k+l*neq]=matr[l+k*neq]=TMatr.mulVVn(na,e,imom);
	}
    }
    final int [] ipvt=new int[neq];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,neq,neq,ipvt);
    if(pinfo!=0) java.util.Arrays.fill(rnodes,Double.NaN);
    else com.polytechnik.utils.Linsystems.dgesl_m1(matr,neq,neq,ipvt,rnodes,1);
    return rnodes;
}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double getPOffset();
abstract double [] getXNodes(final double [] nodespolynomial);
abstract double [] getWeights(final double [] x,final double [] nodespolynomial,final double [] moms);
abstract double [] getWeights2(final double [] x,final double [] nodespolynomial,final double [] moms);
abstract double fx(final int nm,final double [] m,final double x);

static abstract class SQVMTest<DTYPE> {

int neq;
long currenttime,tau;
Trade [] tb;
java.util.Random r;
abstract void setData(final int tsize,final java.util.Random r,final int neq);

abstract SQVM<DTYPE> getSQVM();
abstract void matchMoments();

final String name;
final double eps,epssum;
SQVMTest(final String name,final double eps,final double epssum)
{
    this.name=name;
    this.eps=eps;
    this.epssum=epssum;
}

void matchCommon()
{
    final SQVM<DTYPE> q=getSQVM();
    // there is a deep reason why they match on weights nodes,
    // see Christoffel formulae.
    UnitTests.checkDoubleArrayRel(q.px,"q.px",q.pnodes,"q.pnodes",eps);
    UnitTests.checkDoubleArrayRel(q.p2x,"q.p2x",q.p2nodes,"q.p2nodes",eps);

    if(true)
    {
	final int nmoms=2*neq;
	final double [] imom=new double [nmoms];
	q.access.setM(imom,0,q.access.M_I,q.odata,nmoms);
	final double v0=imom[0];
	for(int k=nmoms;--k>=0;) imom[k]/=v0;
	UnitTests.checkDoubleArrayRel(q.rnodes,"q.rnodes",q.getNodes(neq,imom),"q.getNodes(neq,imom)",eps);
    }

    // FIXME

    System.err.println(name+":SQVMTest:matchCommon(): test OK"+
		       " neq="+neq+" eps="+eps+" epssum="+epssum);
}

static double [] getNodesPolynomialBasis(final int neq,final double [] imom)
{
    final double [] rp=new double [neq+1];
    for(int k=neq;--k>=0;){rp[k]=-imom[neq+k];}
    final double [] qcoefs=DeprecatedHelperEVXData.getLinsysSol(neq,imom,0,rp,0);
    System.arraycopy(qcoefs,0,rp,0,qcoefs.length);
    rp[qcoefs.length]=1;
    return rp;
}

static double [][] checkNodesPolynomialBasis(final double [] nodes,final double [] mom,final int maxdegree,final boolean flag_desc,final double eps,final java.util.Random r)
{
    final int n=nodes.length-1;
    final PolynomialRootsFinder.PRoots u=new com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG().findRoots(nodes);
    final double [] xs=u.re_root,root_im=u.im_root;
    if(u.nroots!=n) throw new RuntimeException("n="+n+" u.nroots="+u.nroots);
    if(flag_desc) com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(xs.length,xs,null);
    else com.polytechnik.utils.SortArrays.sortArrayWithLoad(xs.length,xs,null);

    final double [] ws=new double [n],ws2=new double [n];
    for(int k=0;k<n;k++)
    {
	UnitTests.checkDoubleRel(root_im[k],"root_im[k]",0,"0",eps);
	final double [] tmp=Polynomials.sdiv1(nodes,xs[k]).q;
	final double t0=Polynomials.getD0(tmp,xs[k]);
	for(int m=tmp.length;--m>=0;) tmp[m]/=t0;
	ws[k]=TMatr.mulVVn1(tmp,mom);
	ws2[k]=TMatr.mulVVn1(Polynomials.mult2Pol(tmp,tmp),mom);
    }
    UnitTests.checkDoubleRel(TMatr.sumElements(ws),"sum(ws)",mom[0],"mom[0]",eps);

    final double [] pt=new double [maxdegree+1];
    for(int k=pt.length;--k>=0;) pt[k]=r.nextDouble();
    pt[0]+=0.6+r.nextDouble();
    double i=0;
    for(int k=0;k<xs.length;k++)
    {
	i+=ws[k]*Polynomials.getD0(pt,xs[k]);
    }
    final double ia=TMatr.mulVVn1(pt,mom);
    UnitTests.checkDoubleRel(i,"i",ia,"ia",eps);

    System.err.println("SQVMTest:checkNodesPolynomialBasis(): test OK with n="+n+" maxdegree="+maxdegree+
		       " neq="+n+" eps="+eps);

    return new double [][]{xs,ws,ws2};
}

void matchPt(final SQVM<DTYPE> q,final double [] mi,final double [] mp,final double [] mp2,
	     final boolean flag_desc,final double p_offset)
{
    System.err.println(name+":RQVM:matchPt(): neq="+neq);
    final double [] mqcoefs=SQVM.SQVMTest.getNodesPolynomialBasis(neq,mi);
    final double [][] nq=SQVM.SQVMTest.checkNodesPolynomialBasis(mqcoefs,mi,2*neq-1,flag_desc,eps,r);
    final double [] xnodes=nq[0],wnodes=TMatr.getVectorScaled(nq[1],1/mi[0]);
    UnitTests.checkDoubleArrayRel(q.xnodes,"q.xnodes",xnodes,"xnodes",eps);
    UnitTests.checkDoubleArrayRel(q.wnodes,"q.wnodes",wnodes,"wnodes",eps);
    final double [] ptr=DeprecatedHelperEVXData.getLinsysSol(neq,mi,0,mp,0),
	p2tr=DeprecatedHelperEVXData.getLinsysSol(neq,mi,0,mp2,0);

    final double [] px=new double [xnodes.length],p2x=new double [xnodes.length];
    for(int k=0;k<px.length;k++)
    {
	final double dpf=Polynomials.getD0(ptr,xnodes[k]),dp2f=Polynomials.getD0(p2tr,xnodes[k]);
	px[k]=dpf+p_offset;
	p2x[k]=dp2f-dpf*dpf;
    }
    UnitTests.checkDoubleArrayRel(q.px,"q.px",px,"px",eps);
    UnitTests.checkDoubleArrayRel(q.p2x,"q.p2x",p2x,"p2x",eps);

    System.err.println(name+":SQVMTest:matchPt(): test OK with neq="+neq);
}

void checkXW(final SQVM<DTYPE> q,final double [] rnodes,final double [] mi,final double [] mp,final double [] mp2,final double p_offset,final boolean flag_desc)
{
    final double [][] nxi=checkNodesPolynomialBasis(rnodes,mi,2*neq-1,flag_desc,eps,r);
    final double [][] nxp=checkNodesPolynomialBasis(rnodes,mp,neq-1,flag_desc,eps,r);
    final double [][] nxp2=checkNodesPolynomialBasis(rnodes,mp2,neq-1,flag_desc,eps,r);
    final double [] xi=nxi[0],wi=nxi[1],wia=nxi[2],xp=nxp[0],wp=nxp[1],wpa=nxp[2],xp2=nxp2[0],wp2=nxp2[1],wp2a=nxp2[2];
    UnitTests.checkDoubleArrayRel(xi,"xi",xp,"xp",eps);
    UnitTests.checkDoubleArrayRel(xi,"xi",xp2,"xp2",eps);
    UnitTests.checkDoubleArrayRel(xi,"xi",q.xnodes,"q.xnodes",eps);
    UnitTests.checkDoubleArrayRel(wi,"wi",wia,"wia",eps);
    final double [] w=new double [xi.length],p=new double [xi.length],p2=new double [xi.length],
	pa=new double [xi.length],p2a=new double [xi.length];
    for(int k=wi.length;--k>=0;)
    {
	{
	w[k]=wi[k]/mi[0];
	final double sw=wi[k],dp=wp[k]/sw,dp2=wp2[k]/sw;
	p[k]=dp+p_offset;
	p2[k]=dp2-dp*dp;
	//if(p2[k]<0) throw new RuntimeException("p2["+k+"]="+p2[k]);
	}
	{
	final double sw=wia[k],dp=wpa[k]/sw,dp2=wp2a[k]/sw;
	pa[k]=dp+p_offset;
	p2a[k]=dp2-dp*dp;
	}
    }
    UnitTests.checkDoubleArrayRel(q.wnodes,"q.wnodes",w,"w",eps);
    UnitTests.checkDoubleArrayRel(q.pnodes,"q.pnodes",p,"p",eps);
    UnitTests.checkDoubleArrayRel(q.p2nodes,"q.p2nodes",p2,"p2",eps);
    UnitTests.checkDoubleArrayRel(q.pnodes2,"q.pnodes2",pa,"pa",eps);
    UnitTests.checkDoubleArrayRel(q.p2nodes2,"q.p2nodes2",p2a,"p2a",eps);

    UnitTests.checkDoubleRel(TMatr.sumElements(q.wnodes),"sum(q.wnodes)",1,"1",eps);
    System.err.println(name+":SQVMTest:checkXW(): test OK with neq="+neq);
}

void checkCalculations(final int tsize,final java.util.Random r,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":SQVMTest:checkCalculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps+" epssum="+epssum);

    setData(tsize,r,neq);
    matchMoments();
    matchCommon();

    System.err.println(name+":SQVMTest:checkCalculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps+" epssum="+epssum);
}
void runTests(final int neq_max)
{
    System.err.println(name+":SQVMTest: starting tests; ver=$Id: SQVM.java,v 1.45 2022/06/27 13:32:46 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79091);
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<118;i++)
	{
	    checkCalculations(2210+r.nextInt(2210),r,neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":SQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: SQVM.java,v 1.45 2022/06/27 13:32:46 mal Exp $");
}
} // END OF class SQVMTest


}// END OF class SQVM
