package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: ScalpedMaxIProjectionLaguerre.java,v 1.26 2022/07/28 07:29:40 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ScalpedMaxIProjectionLaguerre extends ScalpedMaxIProjection<QVMDataL> {
ScalpedMaxIProjectionLaguerre(final int neq,final QVMDataL odata,final ScalpedMaxIProjectionStateSaver rst){super(neq,odata,rst);}


public static ScalpedMaxIProjectionLaguerre calculate(final int neq,
					  final long currenttime,
					  final long tau,
					  final Trade [] tb,
					  final int tsize,
					  final ScalpedMaxIProjectionLaguerre ref)
{

    final int npm=4,nmoms=2*neq-1;
    final QVMDataL o=QVMDataL.calculateQVMDataL(nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new ScalpedMaxIProjectionLaguerre(neq,o,ref!=null?ref.sst:null);
}
@Override QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
@Override OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
@Override double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}
@Override double [] getPsi2WIntegratedDt(final double [] psi){return WIntegratorLaguerre.getPsi2WIntegratedDt(psi);}
@Override WIntegratorLaguerre getEmptyWIntegrator(){return new WIntegratorLaguerre(odata.ntm,odata.tau,5);}

static class ScalpedMaxIProjectionLaguerreTest extends ScalpedMaxIProjectionTest<QVMDataL,ScalpedMaxIProjectionLaguerre> {
ScalpedMaxIProjectionLaguerreTest(final long initrandom)
{
    super("ScalpedMaxIProjectionLaguerre",new java.util.Random(initrandom),1e-5);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
ScalpedMaxIProjectionLaguerre getScalpedMaxIProjectionWithThisRef(final int tsize_to,final long currenttime,final ScalpedMaxIProjectionLaguerre ref){return calculate(neq,currenttime,tau,tb,tsize_to,ref);}

} // END of class ScalpedMaxIProjectionLaguerreTest

public static void main(String [] args)
{
    new ScalpedMaxIProjectionLaguerreTest(11127L).runTests(2,5);
} // END main


}
