package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: ScalpedMaxIProjectionMonomials.java,v 1.33 2022/07/28 07:29:40 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ScalpedMaxIProjectionMonomials extends ScalpedMaxIProjection<QVMData> {
    ScalpedMaxIProjectionMonomials(final int neq,final QVMData odata,final ScalpedMaxIProjectionStateSaver rst){super(neq,odata,rst);}


public static ScalpedMaxIProjectionMonomials calculate(final int neq,
					  final long currenttime,
					  final long tau,
					  final Trade [] tb,
					  final int tsize,
					  final ScalpedMaxIProjectionMonomials ref)
{

    final int nvm=1,npm=4,nmoms=2*neq-1;
    final QVMData o=QVMData.calculateQVMData(nvm,nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
					     ref!=null?ref.odata:null);

    return new ScalpedMaxIProjectionMonomials(neq,o,ref!=null?ref.sst:null);
}
@Override QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
@Override OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
@Override double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}
@Override double [] getPsi2WIntegratedDt(final double [] psi){return WIntegratorMonomials.getPsi2WIntegratedDt(psi);}
@Override WIntegratorMonomials getEmptyWIntegrator(){return new WIntegratorMonomials(odata.ntm,odata.tau,5);}


static class ScalpedMaxIProjectionMonomialsTest extends ScalpedMaxIProjectionTest<QVMData,ScalpedMaxIProjectionMonomials> {
ScalpedMaxIProjectionMonomialsTest(final long initrandom)
{
    super("ScalpedMaxIProjectionMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}



@Override
void test_Extra(){
    // These to be identical
    final ScalpedMaxIProjectionMonomials qm=getScalpedMaxIProjection();
    final ScalpedMaxIProjectionLaguerre ql=getScalpedMaxIProjectionLaguerre();

    ScalpedMaxIProjection.matchScalpedMaxIProjection(qm,ql,eps);

    System.err.println(name+": ScalpedMaxIProjectionMonomials:test_Extra(): test ScalpedMaxIProjectionLaguerre OK neq="+neq+" eps="+eps);
}


ScalpedMaxIProjectionMonomials getScalpedMaxIProjectionWithThisRef(final int tsize_to,final long currenttime,final ScalpedMaxIProjectionMonomials ref){return calculate(neq,currenttime,tau,tb,tsize_to,ref);}
ScalpedMaxIProjectionLaguerre getScalpedMaxIProjectionLaguerre(){
    final ScalpedMaxIProjectionLaguerre.ScalpedMaxIProjectionLaguerreTest l=
	new ScalpedMaxIProjectionLaguerre.ScalpedMaxIProjectionLaguerreTest(-1);
    l.neq=neq;
    l.currenttime=currenttime;
    l.tau=tau;
    l.tb=tb;
    return l.getScalpedMaxIProjection();
}

} // END of class ScalpedMaxIProjectionMonomialsTest

public static void main(String [] args)
{
    new ScalpedMaxIProjectionMonomialsTest(4134L).runTests(2,4);
} // END main

}
