package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.freemoney.WIntegrator;

/** Save the state. The class used in ScalpedMaxIProjection.java.
 *  @version $Id: ScalpedMaxIProjectionStateSaver.java,v 1.13 2022/07/10 10:59:04 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ScalpedMaxIProjectionStateSaver {
    public final long seq,time,allshares;
    public final WIntegrator wi;
    public final double P,p_IH,IH,scalp;
    ScalpedMaxIProjectionStateSaver(final ScalpedMaxIProjectionStateSaver prev,final long time,final double P,final long dV,final double p_IH,final double IH,final double scalp,final java.util.function.Supplier<WIntegrator> getemptywintegrator)
    {
	if(prev==null){
	    this.seq=0;
	    this.allshares=dV;
	    this.wi=getemptywintegrator.get();if(wi.nobservables!=5/* five time sequences to save */) throw new IllegalArgumentException();
	} else {
	    this.seq=1+prev.seq;
	    this.allshares=dV+prev.allshares;
	    this.wi=prev.wi;
	}
	this.time=time;this.P=P;
	this.p_IH=p_IH;this.IH=IH;this.scalp=scalp;
    }
    void updateState(long currenttime,double dt,double dp,double dv,double dvorig){
	final long t=currenttime;
	// five arguments, the order: dt:[0], dp:[1], dv:[2], abs dp:[3], dv original [4]
	wi.updateSavedMomentsToNewTimeReference(currenttime);
	wi.addSingleObservation(currenttime,t,new double []{dt/*[0]*/,dp/*[1]*/,dv/*[2]*/,Math.abs(dp)/*[3]*/,dvorig/*[4]*/});
    }
    public double getSumFdt(){return wi.sum[1/* dp */];}
    public double getSumdVScalped(){return wi.sum[2/* dv */];}
    double getSumdVorig(){return wi.sum[4/* dvorig */];}
    double [] getdTScalpedMoments(){return wi.integration_moments[0/* dt */];}
    double [] getFMoments(){return wi.integration_moments[1/* dp */];}
    double [] getdVScalpedMoments(){return wi.integration_moments[2/* dv */];}
    double [] getAbsFMoments(){return wi.integration_moments[3/* abs dp */];}
    /** For unit tests, to match regular imom.
     */
    double [] getdVOriginalMoments(){return wi.integration_moments[4/* dvorig */];}
}
