package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVSolver;
import com.polytechnik.utils.EVXData;

/** A skewness like characteristics (eigenvalues projection).
 *  @version $Id: SepSkewness.java,v 1.3 2022/06/27 10:50:04 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class SepSkewness {
    public final double f,w_plus,w_minus;
    public final double [] ev_f,w;
    public SepSkewness(final double [] psi,final double [] QQf,final double [] QQ,
		       final EVSolver EV,
		       final double eps)
    {
	final int n=psi.length;
	// QQ=null is assumed to be uint matrix
	this.f=TMatr.getAMatrA(psi,QQf,n)/(QQ!=null?TMatr.getAMatrA(psi,QQ,n):TMatr.mulVV(psi,psi));
	final double [] ev=QQ!=null?
	    EV.getGEV_Generalized(n,QQf,QQ,true)// generalized eignevlaues
	    :
	    EV.getEV(n,QQf,true);
	this.ev_f=new double [n];
	this.w=new double [n];
	double w_plus=0,w_minus=0;
	for(int k=0;k<n;k++){
	    final double wk=w[k]=Math.pow(QQ!=null?TMatr.getAMatrB(n,psi,0,ev,n+k*n,QQ,n):TMatr.mulVV(psi,EVXData.getPsiFromBasis(k,ev,n)),2),fk=ev_f[k]=ev[k];
	    if(fk>=f){w_plus+=wk;}
	    else{w_minus+=wk;}
	}
	this.w_plus=w_plus;
	this.w_minus=w_minus;
	if(eps>0){
	    UnitTests.checkDoubleRel(f,"f",TMatr.mulVVn1(w,ev_f),"sum(w*ev_f)",eps);
	    UnitTests.checkDoubleRel(w_plus+w_minus,"w_plus+w_minus",1,"1",eps);
	}
	if(eps>0){
	    System.err.println("SepSkewness: test OK n="+n+" w_plus="+w_plus+" w_minus="+w_minus+" eps="+eps);
	}
    }
    static void matchSepSkewness(final SepSkewness a,final SepSkewness b,final double eps){
	final int n=a.w.length;
	UnitTests.checkDoubleArrayRel(a.w,"a.w",b.w,"b.w",eps);
	UnitTests.checkDoubleArrayRel(a.ev_f,"a.ev_f",b.ev_f,"b.ev_f",eps);
	UnitTests.checkDoubleRel(a.f,"a.f",b.f,"b.f",eps);
	UnitTests.checkDoubleRel(a.w_plus,"a.w_plus",b.w_plus,"b.w_plus",eps);
	UnitTests.checkDoubleRel(a.w_minus,"a.w_minus",b.w_minus,"b.w_minus",eps);
	System.err.println("SepSkewness: matchSepSkewness OK n="+n+" eps="+eps);
    }
}
