package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Skewness;
import com.polytechnik.utils.PolynomialRootsElementary;

/** Find the p, corresponding to 0 skewness.
 *  @version $Id: Skewness0.java,v 1.7 2019/10/04 08:44:57 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class Skewness0 {
    public final double [] re_roots=new double [3],im_roots=new double [3];
    final Skewness SKinit,SK_with_dm_at_average;

    /** Given the measure with
     * (m0,m1,m2,m3) moments find the p,
     * such the measure with the moments
     * (m0+dm,m1+dm*x,m2+dm*x^2,m3+dm*x^3) (initial measure plus delta function with dm)
     * have zero skewness.
     * if eps>0, then unit test with this eps.
     */
    Skewness0(final double m0,final double m1,final double m2,final double m3,
	      final double dm,final double eps)
    {
	this.SKinit=Skewness.getSkewnessFromMonomialMoments(m0,m1,m2,m3);
	final double m=m0+dm,b=dm/m,average=m1/m0,
	    a=m1/m,  // aver(p)=a+b*p
	    a2=m2/m, // aver(p^2)=a2+b*p^2
	    a3=m3/m; // aver(p^3)=a3+b*p^3
	this.SK_with_dm_at_average=Skewness.getSkewnessFromMonomialMoments(m0+dm,m1+dm*average,m2+dm*Math.pow(average,2),m3+dm*Math.pow(average,3));
	// Find the coefficients of d3*p^3+d2*p^2+d1*p+d0=0 equation
	// obtained from the aver((p-aver(p))^3)=0.
	final double d3=b*(1+b*(-3+2*b)),a_sq=a*a,
	    d2=3*a*b*(-1+2*b),
	    d1=3*b*(-a2+2*a_sq),
	    d0=(a3+a*(-3*a2+2*a_sq));
	PolynomialRootsElementary.root3(new double []{d0,d1,d2,d3},re_roots,im_roots);

	if(eps>0)
	{
	    for(int k=0;k<re_roots.length;k++)
	    {
		if(Math.abs(im_roots[k])<eps)
		{
		    final double p=re_roots[k];
		    final double [] mms=new double []{m0+dm,m1+dm*p,m2+dm*Math.pow(p,2),m3+dm*Math.pow(p,3),1};
		    /*
		    System.err.println("Skewness="+com.polytechnik.algorithms.Skewness.getRegularSkewness(mms)+
				       " MSkewness="+com.polytechnik.algorithms.Skewness.getMSkewness(mms)+
				       " p="+p+" im="+im_roots[k]);
		    */
		    final Skewness s=Skewness.getSkewnessFromMonomialMoments(mms[0],mms[1],mms[2],mms[3]);
		    UnitTests.checkDoubleRel(s.sk,"s.sk",0,"0",eps);
		    System.err.println("Skewness0: test p["+k+"]="+p+" s.sk="+s.sk+"=0 passed eps="+eps);
		}
	    }

	    System.err.println("Skewness0: re_roots="+java.util.Arrays.toString(re_roots)+" im_roots="+java.util.Arrays.toString(im_roots)+" SKinit.x1="+SKinit.x1+" SKinit.x2="+SKinit.x2);
	}

    }
}
