package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */

/** Save state.
 *  @version $Id: StateSaver.java,v 1.17 2014/07/27 19:18:53 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class StateSaver {
    public static class S {
	public final double Ic,Pc,Fc;
	public final long Tc,Uc,Vc;
	S(final double Ic,final double Pc,final long Tc,final double Fc,final long Uc,final long Vc)
	{this.Ic=Ic;this.Pc=Pc;this.Tc=Tc;this.Fc=Fc;this.Uc=Uc;this.Vc=Vc;}
	static final S EMPTY=new S(Double.NaN,Double.NaN,0,Double.NaN,0,0);
    }

    public final S L_before,H,L_after;
    private final int from;

    private static final int F_LOW=891,F_HIGH=892,F_NONE=893;

    StateSaver(final S L_before,final S H,final S L_after,final int from)
    {
	this.L_before=L_before;
	this.H=H;
	this.L_after=L_after;
	this.from=from;
    }

    public long getUb(){
	if(L_before!=S.EMPTY && H!=S.EMPTY) return H.Uc-L_before.Uc;
	return 0;
    }
    public long getVb(){
	if(L_before!=S.EMPTY && H!=S.EMPTY) return H.Vc-L_before.Vc;
	return 0;
    }
    public long getVf(){
	if(from==F_LOW && H!=S.EMPTY && L_after!=S.EMPTY) return L_after.Vc-H.Vc;
	return 0;
    }

    public static StateSaver getHighUpdate(final double I,final double p,final long t,final double F,final long U,final long V,final StateSaver rr)
    {
	final StateSaver r=getNothingUpdate(rr);
	if(r.from!=F_HIGH || (r.from==F_HIGH && !(I<r.H.Ic)))
	    return new StateSaver(r.L_after!=S.EMPTY?r.L_after:r.L_before,new S(I,p,t,F,U,V),S.EMPTY,F_HIGH);
	return r;
    }

    public static StateSaver getLowUpdate(final double I,final double p,final long t,final double F,final long U,final long V,final StateSaver rr)
    {
	final StateSaver r=getNothingUpdate(rr);
	if(r.from!=F_LOW || (r.from==F_LOW && !(I>r.L_after.Ic)))
	    return new StateSaver(r.L_before,r.H,new S(I,p,t,F,U,V),F_LOW);
	return r;
    }

    public static StateSaver getNothingUpdate(final StateSaver rr)
    {
	if(rr!=null) return rr;
	final S s=S.EMPTY;
	return new StateSaver(s,s,s,F_NONE);
    }
}
