package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2016
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.utils.TMatr;

class StateWithHedge {
    final double posA,pA_last;
    final double [] pH_last,posH;
    StateWithHedge(final double posA,final double [] posH,
	  final double pA_last,final double [] pH_last)
    {
	this.posA=posA;
	this.posH=posH;
	this.pA_last=pA_last;
	this.pH_last=pH_last;
    }

    StateWithHedge(final double I,final double slope,final double [] IpHedges,
	  final double pA_last,final double [] pH_last)
    {
	if(IpHedges.length!=pH_last.length) throw new IllegalArgumentException("dimension mismatch");
	final double pp=pA_last*I+slope*TMatr.mulVV(pH_last,IpHedges);
	if(pp>0)
	{
	    this.posA=1;
	    this.posH=TMatr.getVectorScaled(IpHedges,slope/I);
	}
	else if(pp<0)
	{
	    this.posA=-1;
	    this.posH=TMatr.getVectorScaled(IpHedges,-slope/I);
	}
	else
	{
	    this.posA=0;
	    this.posH=new double [IpHedges.length];
	}
	this.pA_last=pA_last;
	this.pH_last=pH_last;
    }
    double getPosCOrig()
    {
	return getPosC(pA_last,pH_last);
    }
    double getPosC(final double cur_pA_last,final double [] cur_pH_last)
    {
	return posA*cur_pA_last+TMatr.mulVV(posH,cur_pH_last);
    }
    double getPnL(final double exit_pA_last,final double [] exit_pH_last)
    {
	return getPosC(exit_pA_last,exit_pH_last)-getPosCOrig();
    }
    double getPnL(final StateWithHedge exitPrices)
    {
	return getPnL(exitPrices.pA_last,exitPrices.pH_last);
    }
    StateWithHedge getStateWithGivenPrices(final double pA_last,final double [] pH_last)
    {
	return new StateWithHedge(posA,posH,pA_last,pH_last);
    }
}
