package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.realtimebook.OrdersAndTrades;
import com.polytechnik.realtimebook.DumpTradingDataAnalyzable;
import com.polytechnik.broker.TraderContainer;
import com.polytechnik.broker.MoneyManager;

/** A class to store per stock calculations.
 *  @version $Id: StatusData.java,v 1.100 2016/08/19 10:37:42 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class StatusData extends TraderContainer
{

    private static final boolean FLAG_DIRECT_CALCULATION=true;

    private long nexttime_check=0L;
    private boolean force_check=false;
    private final Trader trader;
    final ModelParameters.ModelParametersData mparametersdata;
    volatile boolean processstarted=false;
    long time_processstarted=0;

    final TradingAlgorithmByEvents PREDICTOR;
    final DumpTradingDataAnalyzable<CalculationDataRef> DIAG;

    final CalculationDataList pastcalculations=new CalculationDataList();
   /** Antecedent count. Consequents count. */
    final AttributesCount antecedents,consequents;
    final OrdersAndTrades ot;

    StatusData(final String ticker,
	       final boolean flag_realtime,
	       final MoneyManager moneymanager,
	       final Trader trader,
	       final ModelParameters.ModelParametersData mparametersdata,
	       final CalculateAntecedents calc_antecedents,
	       final CalculateConsequents calc_consequents,
	       final OrdersAndTrades ot,
	       final boolean flag_training)
    {
	super(ticker,flag_realtime,moneymanager);
	this.trader=trader;
	this.mparametersdata=mparametersdata;
	if(FLAG_DIRECT_CALCULATION && (!flag_training || !mparametersdata.flag_active))
	{
	    this.antecedents=null;
	    this.consequents=null;
	}
	else
	{
	    this.antecedents=calc_antecedents.setAttributesNames(ticker,mparametersdata);
	    this.consequents=calc_consequents.setAttributesNames(ticker,mparametersdata);
	}
	this.ot=ot;
	this.flag_statusOK=this.trader.DEFAULT_ALLOW_TO_TRADE;
	this.allow_trading=this.trader.DEFAULT_ALLOW_TO_TRADE;

	if(mparametersdata.flag_active && !flag_training)
	{
	    final TradingAlgorithmByEvents te=
		new TradingAlgorithmByEvents(mparametersdata,this,trader);
	    this.PREDICTOR=te;
	    this.DIAG=te;
	}
	else
	{
	    this.PREDICTOR=null;
	    this.DIAG=null;
	}
    }


    boolean shouldCheck(final long currenttime)
    {
	if(this.force_check)
	{
	    this.force_check=false;
	    return true;
	}
	if(this.mparametersdata.flag_active)
	{
	    return currenttime>=this.nexttime_check;
	}
	return false;
    }

    public void notifyTrader()
    {
	this.force_check=true;
    }

    void setNextCheck(final long nextcheck)
    {
	/* // remove constrains, when taking orders time may decrease
	if(nextcheck<this.nexttime_check)
	{
	    throw new IllegalArgumentException(" nextcheck="+nextcheck+"<"+
					       " prev nexttime_check="+this.nexttime_check);
	}
	*/
	this.nexttime_check=nextcheck;
    }



    private boolean flag_statusOK;
    void setStatus(final boolean flag_statusOK,
		   final int round_lot_shares,
		   final boolean round_lot_only,
		   final String market)
    {
	this.flag_statusOK=flag_statusOK;
	this.moneymanager.setRountLotSizes(round_lot_shares,
					   round_lot_only,
					   market);
    }

    private boolean allow_trading;
    void setAllowTrading(final boolean on_off)
    {
	this.allow_trading=on_off;
    }

    boolean isAllowToTrade()
    {
	return
	    this.allow_trading &&
	    this.flag_statusOK &&
	    this.trader.allow_trading;
    }


    private long tradertime=-1L;
    void setTraderCurrentTime(final long tradertime)
    {
	this.tradertime=tradertime;
    }
    public long getTraderCurrentTime()
    {
	return this.tradertime;
    }



    CalculateKendall.TradeEMoments [] recurrent_speedup_CalculateLiquidityDeficit13=null;
    CalculateLiquidityDeficit15.VXM [] recurrent_speedup_CalculateLiquidityDeficit15=null;
    CalculateLiquidityDeficit16.VQ  [] recurrent_speedup_CalculateLiquidityDeficit16=null;
    CalculateLiquidityDeficit17.VQ  [] recurrent_speedup_CalculateLiquidityDeficit17=null;
    CalculateLiquidityDeficit18.VQ  [] recurrent_speedup_CalculateLiquidityDeficit18=null;
    CalculateLiquidityDeficit19.VQ  [] recurrent_speedup_CalculateLiquidityDeficit19=null;
    CalculateLiquidityDeficit20.VQ  [] recurrent_speedup_CalculateLiquidityDeficit20=null;
    CalculateLiquidityDeficit21.VQ  [] recurrent_speedup_CalculateLiquidityDeficit21=null;
}
