package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: SupportPriceMonomials.java,v 1.16 2022/07/28 07:29:40 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class SupportPriceMonomials extends SupportPrice<QVMData> {
    SupportPriceMonomials(final int neq,final QVMData odata,final SupportPriceStateSaver rst){super(neq,odata,rst);}


public static SupportPriceMonomials calculate(final int neq,
					  final long currenttime,
					  final long tau,
					  final Trade [] tb,
					  final int tsize,
					  final SupportPriceMonomials ref)
{

    final int nvm=1,npm=4,nmoms=2*neq-1;
    final QVMData o=QVMData.calculateQVMData(nvm,nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
					     ref!=null?ref.odata:null);

    return new SupportPriceMonomials(neq,o,ref!=null?ref.sst:null);
}
@Override QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
@Override OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
@Override double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}
@Override double [] getPsi2WIntegratedDt(final double [] psi){return WIntegratorMonomials.getPsi2WIntegratedDt(psi);}
@Override WIntegratorMonomials getEmptyWIntegrator(){return new WIntegratorMonomials(odata.ntm,odata.tau,5);}


static class SupportPriceMonomialsTest extends SupportPriceTest<QVMData,SupportPriceMonomials> {
SupportPriceMonomialsTest(final long initrandom)
{
    super("SupportPriceMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}



@Override
void test_Extra(){
    // These to be identical
    final SupportPriceMonomials qm=getSupportPrice();
    final SupportPriceLaguerre ql=getSupportPriceLaguerre();

    SupportPrice.matchSupportPrice(qm,ql,eps);

    System.err.println(name+": SupportPriceMonomials:test_Extra(): test SupportPriceLaguerre OK neq="+neq+" eps="+eps);
}


SupportPriceMonomials getSupportPriceWithThisRef(final int tsize_to,final long currenttime,final SupportPriceMonomials ref){return calculate(neq,currenttime,tau,tb,tsize_to,ref);}
SupportPriceLaguerre getSupportPriceLaguerre(){
    final SupportPriceLaguerre.SupportPriceLaguerreTest l=
	new SupportPriceLaguerre.SupportPriceLaguerreTest(-1);
    l.neq=neq;
    l.currenttime=currenttime;
    l.tau=tau;
    l.tb=tb;
    return l.getSupportPrice();
}

} // END of class SupportPriceMonomialsTest

public static void main(String [] args)
{
    new SupportPriceMonomialsTest(434L).runTests(2,4);
} // END main

}
