package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */

import com.polytechnik.realtimebook.*;

/** A class to perform actual trading.
 *  The data is read from a file.
 *  @version $Id: TestTradeReal.java,v 1.28 2020/02/08 10:42:27 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class TestTradeReal {

    private final com.polytechnik.itch.ITCHParser parser;
    private final Trader trader;
    TestTradeReal(final com.polytechnik.broker.BrokerInterface BROKER,
		  final ModelParameters modelparameters,
		  final String filename)
    {
	this.parser=new com.polytechnik.itch.ITCHParser(
	    // when BROKER!=null && EXTERNALANALYZIS==null
	    // predictor data will be read from the files saved.
	    // the directory used is sd.mparametersdata.knowledgedir
	    this.trader=new Trader(null,
				   DumpTradingDataAnalyzable.DUMP_TICKS|
				   DumpTradingDataAnalyzable.DUMP_END_POSTIONS|
				   DumpTradingDataAnalyzable.DUMP_POSTIONS_TIME_END
				   ,
				   modelparameters,
				   BROKER,
				   true,
				   false,
				   filename)
							 );
    }


    void runParser(final String file) throws java.io.IOException
    {
	new com.polytechnik.itch.FileFeeder(parser,file).processStream();
    }

    public static void main(String [] args)
	throws Exception
    {
	if(args.length<2)
	{
	    System.err.println("java -server -Xmx6500000K [-Dtr.OUTPUT_DIR=/out/directory/] com/polytechnik/trading/TestTradeReal config_file itch_file_1 itch_file_2 ... ");
	}
	else
	{
	    final String config_file=args[0];
	    final long t0=System.currentTimeMillis();
	    System.err.println("Version");
	    System.err.println("Started="+new java.util.Date(t0));
	    System.getProperties().list(System.err);
	    System.err.println("END Version");

	    // trade on each file
	    int n=0;
	    for(int i=1;i<args.length;i++)
	    {
		final String f=args[i];
		System.err.println("Processing "+f+" by $Id: TestTradeReal.java,v 1.28 2020/02/08 10:42:27 mal Exp $");
		final ModelParameters m=new ModelParameters(config_file);

		final TestTradeReal tr=new TestTradeReal(new com.polytechnik.broker.DumpDataBroker(),m,f);
		tr.runParser(f);
		// close all tickers
		tr.parser.overAll(new java.util.function.Predicate<OrdersAndTrades>(){
			public boolean test(final OrdersAndTrades b)
			{
			    try {
				final StatusData sd=b.getData();
				if(sd.DIAG!=null)
				{
				    sd.DIAG.closeTicker();
				}
			    }catch(Exception e) {
				throw new RuntimeException(e);
			    }
			    return true;
			}
		    });
		System.err.println("TestTradeReal: DONE processing "+f);
		System.err.flush();

		n++;
	    }

	    final long t1=System.currentTimeMillis();
	    System.err.println("Finished="+new java.util.Date(t1)+" Started="+new java.util.Date(t0));
	    System.err.println("Processed "+n+" files in "+
			       ((t1-t0)/1000L)+
			       " seconds. by $Id: TestTradeReal.java,v 1.28 2020/02/08 10:42:27 mal Exp $");
	}
    }
}
