package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.realtimebook.*;

/** Test John Pierpont Morgan D+ version.
 *  @version $Id: TkQVM.java,v 1.80 2022/11/19 11:08:58 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class TkQVM<DTYPE> {

final DTYPE odata;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
final double I_0,I_L,I_H,I_A,F_0,F_L,F_H,f_0,f_L,f_H,PnL_0,iPnL_0,DiPnL_0,vFromH,tFromH,Svs0;
final EVXData Th,STh,PnL,iPnL,Ji,Jt;
final double sTL,sTH,sT0,sT0b,SsTL,SsTH,SsT0,SsT0b;
final double w_IHSL,w_IHSH,I_SL,I_SH,_w_IHSL,_w_IHSH;
final SState sst;

static class SState {
    final StateSaver r;
    final FStateSaver rf;
    final SMMStateSaver rq;
    final SMMXStateSaver rxq;
    final MinMaxStateSaver rmm;
    final MaxStateSaver rlm,rhm;
    SState(final StateSaver r,
	   final FStateSaver rf,
	   final SMMStateSaver rq,
	   final SMMXStateSaver rxq,
	   final MinMaxStateSaver rmm,
	   final MaxStateSaver rlm,
	   final MaxStateSaver rhm)
    {
	this.r=r;
	this.rf=rf;
	this.rq=rq;
	this.rxq=rxq;
	this.rmm=rmm;
	this.rlm=rlm;
	this.rhm=rhm;
    }
}

TkQVM(final int n,final DTYPE odata,final SState rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.odata=odata;
    this.M=getABasis();

    final int nmoms=2*n-1;

    final double [][] mm=getMoments(nmoms);
    final double [] basis=getBasis(n,mm[1],mm[0]);

    final int i0=0,iL=1,iH=n-1;
    this.I_0=basis[i0];
    this.I_L=basis[iL];
    this.I_H=basis[iH];
    final double [] QQt=M.B.getQQMatr(n,mm[0]),QQf=M.B.getQQMatr(n,mm[2]),QQF=M.B.getQQMatr(n,mm[3]),QQi=M.B.getQQMatr(n,mm[1]);
    {
    final int nd=n-1;
    final double [] _QQfn=EVXData.convertMatrFromQtoB_Basis(n,QQf,basis,nd,n+n),QQtn=TMatr.getUnitMatrix(nd);
    this.Th=EVXData.getEVXDataFromQQfDiagQQa(nd,_QQfn,QQtn,M); //|psi0> is broken, M is incorrect for this basis
    final double [] ee=DeprecatedHelperEVXData.getSymEV(nd,_QQfn);
    this._w_IHSL=ee[nd+0*nd+(nd-1)];
    this._w_IHSH=ee[nd+(nd-1)*nd+(nd-1)];


    // spur calculation
    double sa=0;for(int k=iL;k<=iH;k++) sa+=basis[k];
    this.I_A=sa/nd;
    final double [] QQpnl=M.B.getQQMatr(n,mm[4]),QQfn_pnl=EVXData.convertMatrFromQtoB_Basis(n,QQpnl,basis,nd,n+n);
    this.PnL=EVXData.getEVXDataFromQQfDiagQQa(nd,QQfn_pnl,QQtn,M); //|psi0> is broken, M is incorrect for this basis
    this.PnL_0=TMatr.getAMatrA(EVXData.getPsiFromBasis(i0,basis,n),QQpnl,n);

    final double [] QQSvs=M.B.getQQMatr(n,mm[8]),QQfn_Svs=EVXData.convertMatrFromQtoB_Basis(n,QQSvs,basis,nd,n+n);
    this.STh=EVXData.getEVXDataFromQQfDiagQQa(nd,QQfn_Svs,QQtn,M);
    final double [] bQQSvs=EVXData.convertMatrFromQtoB_Basis(n,QQSvs,basis);
    this.Svs0=bQQSvs[i0+n*i0];

    }

    {
	final double [] sbasis=getBasis(n,mm[2],mm[0]);
	final double [] psiIH=EVXData.getPsiFromBasis(iH,basis,n),
	    psiSL=EVXData.getPsiFromBasis(iL,sbasis,n),
	    psiSH=EVXData.getPsiFromBasis(iH,sbasis,n);
	this.w_IHSL=TMatr.getAMatrB(psiIH,psiSL,QQt,n);
	this.w_IHSH=TMatr.getAMatrB(psiIH,psiSH,QQt,n);
	this.I_SL=TMatr.getAMatrA(psiSL,QQi,n);
	this.I_SH=TMatr.getAMatrA(psiSH,QQi,n);
    }


    {
	{
	final double [] QQipnl=M.B.getQQMatr(n,mm[5]),QQDipnl=M.B.getQQMatr(n,mm[6]);
	//System.err.println("test iPnL_0="+TMatr.getAMatrA(EVXData.getPsiFromBasis(i0,basis,n),QQipnl,n));
	//System.err.println("test2 iPnL_0="+EVXData.getEVXDataFromQQ(n,QQipnl,QQt,M).s0);
	this.iPnL=EVXData.getEVXDataFromQQ(n,QQipnl,QQt,M);
	//System.err.println("this.iPnL.s0="+this.iPnL.s0);
	final double [] bQQipnl=EVXData.convertMatrFromQtoB_Basis(n,QQipnl,basis),
	    bQQDipnl=EVXData.convertMatrFromQtoB_Basis(n,QQDipnl,basis);
	this.DiPnL_0=bQQDipnl[i0+n*i0];
	this.iPnL_0=bQQipnl[i0+n*i0];
	//System.err.println("iPnL_0="+iPnL_0);
	}

	{
	final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	    QQdp=M.B.getQQMatr(n,mm[7]),QQI=M.B.getQQMatr(n,mm[10]),
	    QQj=TMatr.mulMatrices(n,new double [][]{QQdp,IQQt,QQI});
	TMatr.selectHermitianPartInPlace(n,QQj);
	this.Ji=EVXData.getEVXDataFromQQ(n,QQj,QQt,M);
	this.Jt=EVXData.getEVXDataFromQQ(n,QQdp,QQt,M);
	}
    }

    final double [] bQQf=EVXData.convertMatrFromQtoB_Basis(n,QQf,basis),
	bQQF=EVXData.convertMatrFromQtoB_Basis(n,QQF,basis);

    this.f_0=bQQf[i0+n*i0];
    this.f_L=bQQf[iL+n*iL];
    this.f_H=bQQf[iH+n*iH];
    this.F_0=bQQF[i0+n*i0];
    this.F_L=bQQF[iL+n*iL];
    this.F_H=bQQF[iH+n*iH];


    final StateSaver s;
    if(I_0>I_H){
	s=StateSaver.getHighUpdate(I_0,getPLast(),getTLast(),f_0,getSVol(),getVol(),rst!=null?rst.r:null);
    } else if(I_0<I_L) {
	s=StateSaver.getLowUpdate(I_0,getPLast(),getTLast(),f_0,getSVol(),getVol(),rst!=null?rst.r:null);
    } else {
	s=StateSaver.getNothingUpdate(rst!=null?rst.r:null);
    }
    this.vFromH=getVol()-s.H.Vc;
    this.tFromH=getTLast()-s.H.Tc;

    final FStateSaver sf;
    if(f_0>Th.sH){
	sf=FStateSaver.getMaxUpdate(f_0,getPLast(),getTLast(),I_0,getSVol(),getVol(),rst!=null?rst.rf:null);
    } else if(f_0<Th.sL) {
	sf=FStateSaver.getMinUpdate(f_0,getPLast(),getTLast(),I_0,getSVol(),getVol(),rst!=null?rst.rf:null);
    } else if(n-1>1 && (
		  (f_0<Th.ev[(n-1-1) -1] && (rst!=null && rst.rf.from==rst.rf.F_MAX))
	       || (f_0>Th.ev[0       +1] && (rst!=null && rst.rf.from==rst.rf.F_MIN))
		       )){
	sf=FStateSaver.getTaintedUpdate(rst!=null?rst.rf:null);
    } else {
	sf=FStateSaver.getNothingUpdate(rst!=null?rst.rf:null);
    }

    final SMMStateSaver sq;
    if(I_0>I_H){
	sq=SMMStateSaver.getHighUpdate(I_0,getPLast(),getTLast(),f_0,getSVol(),getVol(),rst!=null?rst.rq:null);
    } else if(I_0<I_L) {
	sq=SMMStateSaver.getLowUpdate(I_0,getPLast(),getTLast(),f_0,getSVol(),getVol(),rst!=null?rst.rq:null);
    } else {
	sq=SMMStateSaver.getNothingUpdate(getPLast(),rst!=null?rst.rq:null);
    }

    final SMMXStateSaver sxq;
    if(I_0>I_H){
	sxq=SMMXStateSaver.getHighUpdate(I_0,getPLast(),getTLast(),f_0,getSVol(),getVol(),rst!=null?rst.rxq:null);
    } else if(I_0<I_L) {
	sxq=SMMXStateSaver.getLowUpdate(I_0,getPLast(),getTLast(),f_0,getSVol(),getVol(),rst!=null?rst.rxq:null);
    } else {
	sxq=SMMXStateSaver.getNothingUpdate(getPLast(),rst!=null?rst.rxq:null);
    }

    final MinMaxStateSaver smm;
    if(f_0>Th.sH){
	smm=MinMaxStateSaver.getMaxUpdate(I_0,getPLast(),getTLast(),f_0,rst!=null?rst.rmm:null);
    } else if(f_0<Th.sL) {
	smm=MinMaxStateSaver.getMinUpdate(I_0,getPLast(),getTLast(),f_0,rst!=null?rst.rmm:null);
    } else {
	smm=MinMaxStateSaver.getNothingUpdate(rst!=null?rst.rmm:null);
    }


    final MaxStateSaver slm;
    if(I_0<I_L){
	slm=MaxStateSaver.getTaintedUpdate(rst!=null?rst.rlm:null);
    } else if(-f_0>-Th.sL){
	slm=MaxStateSaver.getMaxUpdate(-f_0,getPLast(),getTLast(),I_0,getSVol(),getVol(),rst!=null?rst.rlm:null);
    } else {
	slm=MaxStateSaver.getNothingUpdate(rst!=null?rst.rlm:null);
    }
    final MaxStateSaver shm;
    if(I_0<I_L){
	shm=MaxStateSaver.getTaintedUpdate(rst!=null?rst.rhm:null);
    } else if(f_0>Th.sH){
	shm=MaxStateSaver.getMaxUpdate(f_0,getPLast(),getTLast(),I_0,getSVol(),getVol(),rst!=null?rst.rhm:null);
    } else {
	shm=MaxStateSaver.getNothingUpdate(rst!=null?rst.rhm:null);
    }



    {

	final double [] psi0=EVXData.getPsiFromBasis(i0,basis,n),psi0_2=M.B.mult2Pol(psi0,psi0);
	final double Svs=TMatr.mulVVn1(psi0_2,mm[8]),vs=TMatr.mulVVn1(psi0_2,mm[9]),i=TMatr.mulVVn1(psi0_2,mm[10]);
	final double [] Sb=getBasis(n,mm[8],mm[10]);
	this.SsTL=Sb[iL];
	this.SsTH=Sb[iH];
	this.SsT0=Svs/i;
	this.SsT0b=Sb[i0];

	final double [] b=getBasis(n,mm[9],mm[10]);
	this.sTL=b[iL];
	this.sTH=b[iH];
	this.sT0=vs/i;
	this.sT0b=b[i0];
    }

    this.sst=new SState(s,sf,sq,sxq,smm,slm,shm);


}

double [][] getMoments(final int nmoms)
{
    final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access=getAccess();
    final double [] tmom=new double [nmoms],imom=new double [nmoms],vsmom=new double [nmoms],VSmom=new double [nmoms],pnvsmom=new double [nmoms],ipnlmom=new double [nmoms],dpmom=new double [nmoms];
    if(false) access.setM(vsmom,0,access.M_RVS,odata,nmoms);
    else if(false) access.setM(vsmom,1,access.M_DP,odata,nmoms);
    else if(false) access.setM(vsmom,0,access.M_VS,odata,nmoms);
    else { // convert to 0 offset
	access.setM(vsmom,1,access.M_RVS,odata,nmoms);
	access.setM(tmom,0,access.M_RVS,odata,nmoms);
	TMatr.sumP1(vsmom,tmom,getPOffset());
	// convert to offset equal to p last.
	access.setM(pnvsmom,1,access.M_RVS,odata,nmoms);
	TMatr.sumP1(pnvsmom,tmom,getPOffset()-getPLast());
    }
    access.setM(imom,1,access.M_I,odata,nmoms);
    access.setM(tmom,0,access.M_I,odata,nmoms);
    TMatr.sumP1(imom,tmom,getPOffset());
    access.setM(ipnlmom,1,access.M_I,odata,nmoms);
    TMatr.sumP1(ipnlmom,tmom,getPOffset()-getPLast());
    final double [] Dipnlmom=new double [nmoms];
    access.setDFMoments(Dipnlmom,ipnlmom,nmoms);

    access.setMT(tmom,nmoms);
    access.setFMoments(VSmom,vsmom,nmoms);
    access.setM(dpmom,1,access.M_DP,odata,nmoms);

    final double [] Svsmom=new double [nmoms],_vsmom=new double [nmoms],_imom=new double [nmoms];
    access.setM(Svsmom,0,access.M_VS,odata,nmoms);
    access.setM(_vsmom,0,access.M_RVS,odata,nmoms);
    access.setM(_imom,0,access.M_I,odata,nmoms);

    return new double [][]{tmom,imom,vsmom,VSmom,pnvsmom,ipnlmom,Dipnlmom,dpmom,Svsmom,_vsmom,_imom};
}

double [] getBasis(final int n,final double [] imom,final double [] tmom)
{
    return EdComm.getIRadauBasisPsi0(n,imom,tmom,this.M);
}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getPLast();
abstract double getPOffset();
abstract long getTLast();
abstract long getSVol();
abstract long getVol();

static abstract class TkQVMTest<DTYPE> extends QTest {

abstract TkQVM<DTYPE> getTkQVM();


TkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}




void test_Common()
{
    final TkQVM<DTYPE> q=getTkQVM();
    final int n=neq,nmoms=2*n-1;
    final double [][] mm=q.getMoments(nmoms);
    final EVXData bq=BQVM.getEVXData0Removed(n,mm[2],mm[0],q.getAccess(),q.Th.x0);
    EVXData.matchEVXData(q.Th,bq,false,false,eps);
    final EVXData pq=BQVM.getEVXData0Removed(n,mm[4],mm[0],q.getAccess(),q.PnL.x0);
    EVXData.matchEVXData(q.PnL,pq,false,false,eps);
    final EVXData qiPnL=DeprecatedHelperEVXData.getEVXData(n,mm[5],mm[0],q.getAccess(),q.iPnL.x0);
    EVXData.matchEVXData(q.iPnL,qiPnL,false,true,eps);
    final EVXData Sbq=BQVM.getEVXData0Removed(n,mm[8],mm[0],q.getAccess(),q.STh.x0);
    EVXData.matchEVXData(q.STh,Sbq,false,false,eps);
    final double [] SvsB=q.getBasis(n,mm[8],mm[0]);
    UnitTests.checkDoubleRel(q.Svs0,"q.Svs0",SvsB[0],"<psi0|SVs|psi0>",eps);
    UnitTests.checkDoubleRel(q.STh.sL,"q.STh.sL",SvsB[1],"<psiL|SVs|psiL>",eps);
    UnitTests.checkDoubleRel(q.STh.sH,"q.STh.sH",SvsB[n-1],"<psiH|SVs|psiH>",eps);

    final EVXData sT=BQVM.getEVXData0Removed(n,mm[9],mm[10],q.getAccess(),q.STh.x0);
    UnitTests.checkDoubleRel(sT.sL,"sT.sL",q.sTL,"q.sTL",eps);
    UnitTests.checkDoubleRel(sT.sH,"sT.sH",q.sTH,"q.sTH",eps);
    UnitTests.checkDoubleRel(DeprecatedHelperEVXData.getEVXData(n,mm[9],mm[10],q.getAccess(),q.STh.x0).s0,"s0b",q.sT0b,"q.sT0b",eps);
    UnitTests.checkDoubleRel(DeprecatedHelperEVXData.getEVXData(n,mm[9],mm[0],q.getAccess(),q.STh.x0).s0/DeprecatedHelperEVXData.getEVXData(n,mm[10],mm[0],q.getAccess(),q.STh.x0).s0,"s0",q.sT0,"q.sT0",eps);
    final EVXData SsT=BQVM.getEVXData0Removed(n,mm[8],mm[10],q.getAccess(),q.STh.x0);
    UnitTests.checkDoubleRel(SsT.sL,"SsT.sL",q.SsTL,"q.SsTL",eps);
    UnitTests.checkDoubleRel(SsT.sH,"SsT.sH",q.SsTH,"q.SsTH",eps);
    UnitTests.checkDoubleRel(DeprecatedHelperEVXData.getEVXData(n,mm[8],mm[10],q.getAccess(),q.STh.x0).s0,"Ss0b",q.SsT0b,"q.SsT0b",eps);
    UnitTests.checkDoubleRel(DeprecatedHelperEVXData.getEVXData(n,mm[8],mm[0],q.getAccess(),q.STh.x0).s0/DeprecatedHelperEVXData.getEVXData(n,mm[10],mm[0],q.getAccess(),q.STh.x0).s0,"s0",q.SsT0,"q.SsT0",eps);

    final double [] QQt=q.M.B.getQQMatr(n,mm[0]),QQi=q.M.B.getQQMatr(n,mm[1]),QQdp=q.M.B.getQQMatr(n,mm[7]);
    final EVXData qJt=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQdp,QQt,q.getAccess(),q.Jt.x0);
    EVXData.matchEVXData(q.Jt,qJt,false,true,eps);
    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	QQI=q.M.B.getQQMatr(n,mm[10]),
	QQj=TMatr.scaleVectorInPlace(DeprecatedHelperTMatr.sumAB(n*n,TMatr.mulMatrices(neq,new double [][]{QQdp,IQQt,QQI}),TMatr.mulMatrices(neq,new double [][]{QQI,IQQt,QQdp})),0.5);
    final EVXData qJi=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQj,QQt,q.getAccess(),q.Ji.x0);
    EVXData.matchEVXData(q.Ji,qJi,false,true,eps);

    final double [] QQSpnl=q.M.B.getQQMatr(n,mm[4]),QQiPnL=q.M.B.getQQMatr(n,mm[5]),QQDiPnL=q.M.B.getQQMatr(n,mm[6]),psi0=qJt.psi0;

    UnitTests.checkDoubleRel(q.PnL_0,"q.PnL_0",TMatr.getAMatrA(psi0,QQSpnl,n),"<psi0|QQSpnl|psi0>",eps);
    UnitTests.checkDoubleRel(q.PnL_0,"q.PnL_0",DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQSpnl,QQt,q.getAccess(),q.PnL.x0).s0,".s0",eps);
    UnitTests.checkDoubleRel(q.DiPnL_0,"q.DiPnL_0",TMatr.getAMatrA(psi0,QQDiPnL,n),"<psi0|QQDiPnL|psi0>",eps);
    UnitTests.checkDoubleRel(q.DiPnL_0,"q.DiPnL_0",DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQDiPnL,QQt,q.getAccess(),q.PnL.x0).s0,".s0",eps);
    System.err.println("q.iPnL_0="+q.iPnL_0+" .s0="+DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQiPnL,QQt,q.getAccess(),q.PnL.x0).s0);
    UnitTests.checkDoubleRel(q.iPnL.s0,"q.iPnL.s0",DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQiPnL,QQt,q.getAccess(),q.PnL.x0).s0,".s0",eps);
    UnitTests.checkDoubleRel(q.iPnL_0,"q.iPnL_0",q.iPnL.s0,"q.iPnL.s0",eps);


    UnitTests.checkDoubleRel(Math.abs(q.w_IHSL),"q.w_IHSL",Math.abs(q._w_IHSL),"q._w_IHSL",eps);
    UnitTests.checkDoubleRel(Math.abs(q.w_IHSH),"q.w_IHSH",Math.abs(q._w_IHSH),"q._w_IHSH",eps);


    System.err.println(name+":TkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}


void test_Extra(){}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":TkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Extra();

    System.err.println(name+":TkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":TkQVMTest: starting tests; ver=$Id: TkQVM.java,v 1.80 2022/11/19 11:08:58 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":TkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: TkQVM.java,v 1.80 2022/11/19 11:08:58 mal Exp $");
}
} // END OF class TkQVMTest




}// END OF class TkQVM
