package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: TkQVMMonomials.java,v 1.14 2019/01/06 23:39:13 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class TkQVMMonomials extends TkQVM<QVMData> {
TkQVMMonomials(final int neq,final QVMData odata,final SState rst){super(neq,odata,rst);}


public static TkQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final TkQVMMonomials ref)
{
    final int nvm=1,npm=4;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new TkQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
DeprecatedHelperQVMDataDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataDirectAccessWithBasis();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double getPOffset(){return odata.p_offset;}
long getTLast(){return odata.currenttime;}
long getSVol(){return odata.svol;}
long getVol(){return odata.vol;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}

static class TkQVMMonomialsTest extends TkQVMTest<QVMData> {
TkQVMMonomialsTest(final long initrandom)
{
    super("TkQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
TkQVMMonomials getTkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

TkQVMLaguerre getTkQVMLaguerre(){return TkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

@Override
void test_Extra(){
    final TkQVMMonomials q=getTkQVM();
    final TkQVMLaguerre s=getTkQVMLaguerre();

    UnitTests.checkDoubleRel(q.I_0,"q.I_0",s.I_0,"s.I_0",eps);
    UnitTests.checkDoubleRel(q.I_L,"q.I_L",s.I_L,"s.I_L",eps);
    UnitTests.checkDoubleRel(q.I_H,"q.I_H",s.I_H,"s.I_H",eps);
    UnitTests.checkDoubleRel(q.I_A,"q.I_A",s.I_A,"s.I_A",eps);
    UnitTests.checkDoubleRel(q.F_0,"q.F_0",s.F_0,"s.F_0",eps);
    UnitTests.checkDoubleRel(q.F_L,"q.F_L",s.F_L,"s.F_L",eps);
    UnitTests.checkDoubleRel(q.F_H,"q.F_H",s.F_H,"s.F_H",eps);
    UnitTests.checkDoubleRel(q.f_0,"q.f_0",s.f_0,"s.f_0",eps);
    UnitTests.checkDoubleRel(q.f_L,"q.f_L",s.f_L,"s.f_L",eps);
    UnitTests.checkDoubleRel(q.f_H,"q.f_H",s.f_H,"s.f_H",eps);
    UnitTests.checkDoubleRel(q.PnL_0,"q.PnL_0",s.PnL_0,"s.PnL_0",eps);
    UnitTests.checkDoubleRel(q.DiPnL_0,"q.DiPnL_0",s.DiPnL_0,"s.DiPnL_0",eps);
    EVXData.matchEVXData(q.Th,s.Th,false,false,eps);
    EVXData.matchEVXData(q.PnL,s.PnL,false,false,eps);
    EVXData.matchEVXData(q.iPnL,s.iPnL,false,false,eps);

    System.err.println(name+":TkQVMMonomialsTest:test_Extra(): test OK neq="+neq+" eps="+eps);
}

} // END of class TkQVMMonomialsTest

public static void main(String [] args)
{
    new TkQVMMonomialsTest(1117L).runTests(5);
} // END main



} // END OF class TkQVMMonomials
