package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;

/** A program to find a trade opportunity.
 *  @version $Id: TradeOpportunityChecker.java,v 1.514 2020/02/08 10:42:27 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class TradeOpportunityChecker  {


    private static final boolean
	DEBUG_PRINTPRICES=false,
	DEBUG_PRINTCALCULATIONS=false;


    private final Distribution<BookItem>
	buy=new Distribution<BookItem>(),
	sell=new Distribution<BookItem>(),
	tmpordersbuf=new Distribution<BookItem>();

    private final Distribution<Trade>
	tmptradesbuf=new Distribution<Trade>();
    final Distribution<Trade>
	savedtrades=new Distribution<Trade>();
    final Distribution<Trade>
	savedpseudotrades=new Distribution<Trade>();


    long trades_cleared_before=0L;

    final CalculateConsequentData
	consequentstats=new CalculateConsequentData(this.savedtrades,this.tmptradesbuf),
	consequentrealtime=new CalculateConsequentData(this.savedtrades,this.tmptradesbuf);

    final CalculateTRDynamics tr_dynamics=
	new CalculateTRDynamics(this.buy,this.sell,this.savedtrades,this.tmpordersbuf,this.tmptradesbuf);

    final CalculateTradeVolume tradevolumecalculator=
	new CalculateTradeVolume(this.savedtrades,this.tmptradesbuf);
    final CalculateTradeHContrib tradehcontrib=
	new CalculateTradeHContrib(this.savedtrades,this.tmptradesbuf);

    final CalculateBookVolume bookvolumecalculator=
	new CalculateBookVolume(this.buy,this.sell,this.tmpordersbuf);

    final CalculateLiquidityDeficit
	liquidity_buy01=new CalculateLiquidityDeficit(this.buy,
						    this.tmpordersbuf,
						    this.savedtrades,
						    true,
						    this.tmptradesbuf),
	liquidity_sell01=new CalculateLiquidityDeficit(this.sell,
						     this.tmpordersbuf,
						     this.savedtrades,
						     false,
						     this.tmptradesbuf);
    final CalculateLiquidityDeficit2
	liquiditydeficit2=new CalculateLiquidityDeficit2(this.buy,
							 this.sell,
							 this.savedtrades,
							 this.tmpordersbuf,
							 this.tmptradesbuf);

    final CalculateLiquidityDeficit3
	book_trades_liquiditydeficit3=new CalculateLiquidityDeficit3(this.buy,
								 this.sell,
								 this.savedtrades,
								 this.tmpordersbuf,
								 this.tmptradesbuf);

    final CalculateLiquidityDeficit4
	book_trades_liquiditydeficit4=new CalculateLiquidityDeficit4(this.buy,
								     this.sell,
								     this.savedtrades,
								     this.tmptradesbuf);

    final CalculateLiquidityDeficit5
	book_trades_liquiditydeficit5_down=new CalculateLiquidityDeficit5(this.buy,
						     this.savedtrades,
						     true,
						     this.tmpordersbuf,
						     this.tmptradesbuf),
	book_trades_liquiditydeficit5_up=new CalculateLiquidityDeficit5(this.sell,
						   this.savedtrades,
						   false,
						   this.tmpordersbuf,
						   this.tmptradesbuf);


   final CalculateLiquidityDeficit6
	book_trades_liquiditydeficit6=new CalculateLiquidityDeficit6(this.buy,
								     this.sell,
								     this.savedtrades,
								     this.tmpordersbuf,
								     this.tmptradesbuf);

   final CalculateLiquidityDeficit7
	book_trades_liquiditydeficit7=new CalculateLiquidityDeficit7(this.buy,
								     this.sell,
								     this.savedtrades,
								     this.tmpordersbuf,
								     this.tmptradesbuf);

   final CalculateLiquidityDeficit8
	book_trades_liquiditydeficit8=new CalculateLiquidityDeficit8(this.buy,
								     this.sell,
								     this.savedtrades,
								     this.tmpordersbuf,
								     this.tmptradesbuf);

    final CalculateLiquidityDeficit9
	book_trades_liquiditydeficit9=new CalculateLiquidityDeficit9(this.buy,
								     this.sell,
								     this.savedtrades,
								     this.tmpordersbuf,
								     this.tmptradesbuf);

    final CalculateLiquidityDeficit9T
	tradesT_liquiditydeficit9T=new CalculateLiquidityDeficit9T(this.savedtrades,
								  this.tmptradesbuf);

    final CalculateLiquidityDeficit10
	book_trades_liquiditydeficit10=new CalculateLiquidityDeficit10(this.buy,
								      this.sell,
								      this.savedtrades,
								      this.tmpordersbuf,
								      this.tmptradesbuf);

    final CalculateLiquidityDeficit11
	book_trades_liquiditydeficit11=new CalculateLiquidityDeficit11(this.buy,
								       this.sell,
								       this.savedtrades,
								       this.tmpordersbuf,
								       this.tmptradesbuf);

    final CalculateLiquidityDeficit12
	book_trades_liquiditydeficit12=new CalculateLiquidityDeficit12(this.buy,
								       this.sell,
								       this.savedtrades,
								       this.tmpordersbuf,
								       this.tmptradesbuf);

    final CalculateLiquidityDeficit13
	book_trades_liquiditydeficit13=new CalculateLiquidityDeficit13(this.buy,
								       this.sell,
								       this.savedtrades,
								       this.tmpordersbuf,
								       this.tmptradesbuf);

    final CalculateLiquidityDeficit15
	book_trades_liquiditydeficit15=new CalculateLiquidityDeficit15(this.savedtrades,this.tmptradesbuf);
    final CalculateLiquidityDeficit16
	book_trades_liquiditydeficit16=new CalculateLiquidityDeficit16(this.savedtrades);
    final CalculateLiquidityDeficit17
	book_trades_liquiditydeficit17=new CalculateLiquidityDeficit17(this.savedtrades);
    final CalculateLiquidityDeficit18
	book_trades_liquiditydeficit18=new CalculateLiquidityDeficit18(this.savedtrades,this.buy,this.sell);
    final CalculateLiquidityDeficit19
	book_trades_liquiditydeficit19=new CalculateLiquidityDeficit19(this.savedtrades,this.buy,this.sell);
    final CalculateLiquidityDeficit20
	book_trades_liquiditydeficit20=new CalculateLiquidityDeficit20(this.savedtrades,this.buy,this.sell);
    final CalculateLiquidityDeficit21
	book_trades_liquiditydeficit21=new CalculateLiquidityDeficit21(this.savedtrades,this.buy,this.sell);

    final CalculateBookVolumePriceThreshold
	calculate_buy_thresholds=new CalculateBookVolumePriceThreshold(true,this.buy),
	calculate_sell_thresholds=new CalculateBookVolumePriceThreshold(false,this.sell);
    final CalculateBookVolumePriceFixedThreshold
	calculate_buy_fixedthresholds=new CalculateBookVolumePriceFixedThreshold(true,this.buy),
	calculate_sell_fixedthresholds=new CalculateBookVolumePriceFixedThreshold(false,this.sell);
    final CalculateBookCriticalExponent
	calculate_buy_crit=new CalculateBookCriticalExponent(true,this.buy),
	calculate_sell_crit=new CalculateBookCriticalExponent(false,this.sell);


    final CalculateBookVirtualShifts calculate_virtual_shifts=
	new CalculateBookVirtualShifts(this.buy,this.sell,this.savedtrades,this.tmpordersbuf,this.tmptradesbuf);
    final CalculateBookVirtualShifts2 calculate_virtual_shifts2=
	new CalculateBookVirtualShifts2(this.buy,this.sell,this.savedtrades,this.tmpordersbuf,this.tmptradesbuf);
    final CalculateBookVirtualShiftsTau calculate_virtual_shifts_tau=
	new CalculateBookVirtualShiftsTau(this.buy,this.sell);

    final CalculateBookLiquidityDeficit book_liquiditydeficit02=
	new CalculateBookLiquidityDeficit(this.buy,this.sell,this.savedtrades);

    final CalculateAntecedents attribscalculator=new CalculateAntecedents();
    final CalculateConsequents consequentscalculator=new CalculateConsequents();


    final DataAnalyzable externalanalyzis;
    TradeOpportunityChecker(final DataAnalyzable externalanalyzis)
    {
	this.externalanalyzis=externalanalyzis;
    }


    StatusData tinfo=null;
    long currenttime=0L;
    private long id=-1;
    long getMagicID()
    {
	return this.id;
    }
    /** The number of shared we already have in this position.
     *  Depending on it we use different strategy.
     */
    public void setData(final long currenttime,
			final StatusData tinfo,
			final long id)
    {
	if(this.currenttime!=0L||
	   this.buy.getSize()>0||
	   this.sell.getSize()>0||
	   this.tmpordersbuf.getSize()>0||
	   this.tmptradesbuf.getSize()>0||
	   this.savedtrades.getSize()>0||
	   this.savedpseudotrades.getSize()>0||
	   this.trades_cleared_before!=0L||
	   this.tinfo!=null||
	   this.id>=0||
	   id<0)
	{
	    throw new IllegalStateException("An attempt to copy to a non-clean object.");
	}
	/* final long t0=System.nanoTime();
	 */
	tinfo.ot.orderbook.copyOrdersSeparateBuySell(this.buy,this.sell);

	// copy trades
	tinfo.ot.tradebook.copyTrades(this.savedtrades);
	/*
	System.err.println("T="+tinfo.ot.ticker+" this.savedtrades.getSize="+this.savedtrades.getSize()+
			   " cap="+this.savedtrades.getCapacity());
	*/
	// copy pseudotrades
	tinfo.ot.pseudotrades.copyTrades(this.savedpseudotrades);

	// the rest of the parameters
	this.trades_cleared_before=tinfo.ot.tradebook.getLastClearedBefore();
	this.currenttime=currenttime;
	this.tinfo=tinfo;
	this.id=id;
	/*
	final long dt=System.nanoTime()-t0;
	t_save_data+=dt;
	if(currenttime/1000*1000==currenttime)
	{
	    System.err.println("currenttime="+(currenttime/1e9)+" t_save_data="+(t_save_data/1e9));
	}
	*/
    }
    //private static long t_save_data=0;



    double p_buy=0.0,p_sell=0.0,p_last=0.0;
    long vol_p_buy=0,vol_p_sell=0;
    private void calculatePrices()
    {
	final StatusData sd=this.tinfo;
	// p_last is set in prepareBook().

	final long currenttime=sd.getTraderCurrentTime();
	// Set current buy/sell prices, read from book edges.
	final BookItem ib=sd.ot.orderbook.getBestLevel(currenttime,BookItem.BOOK_BUY),
	    is=sd.ot.orderbook.getBestLevel(currenttime,BookItem.BOOK_SELL);

	if(ib!=null){this.p_buy=ib.price;this.vol_p_buy=ib.shares;}
	else {this.p_buy=Double.NaN;this.vol_p_buy=0;}
	if(is!=null){this.p_sell=is.price;this.vol_p_sell=is.shares;}
	else {this.p_sell=Double.NaN;this.vol_p_sell=0;}

	if(this.p_buy>this.p_sell)
	{
	    System.err.println(
		"Negative sell-buy spread for ticker="+this.tinfo.ticker+
		" buy="+(ib!=null?BookItem.getOrderDescription(ib):"null")+
		" sell="+(is!=null?BookItem.getOrderDescription(is):"null")+
		" at time="+BookItem.getTimeDiag(this.currenttime));
	}

	if(DEBUG_PRINTPRICES)
	{
	    System.err.print(this.tinfo.ticker);
	    System.err.print("\t");
	    System.err.print(this.currenttime);
	    System.err.print("\t");
	    System.err.print(this.p_buy);
	    System.err.print("\t");
	    System.err.print(this.p_sell);
	    System.err.print("\t");
	    System.err.print(this.p_last);
	    System.err.println();
	}
    }


    CalculationDataRef calculateRef()
    {
	final CalculationDataRef cr=
	    new CalculationDataRef(this.id,
				   this.currenttime,
				   this.p_buy,
				   this.p_sell,
				   this.p_last,
				   this.vol_p_buy,
				   this.vol_p_sell);
	if(this.externalanalyzis!=null)
	{
	    final StatusData sd=this.tinfo;
	    if(sd.pastcalculations.training_last_data_processed!=null)
	    {
		sd.pastcalculations.training_last_data_processed_offset++;
	    }
	    sd.pastcalculations.add(cr);
	}
	return cr;
    }


    CalculationData calculateNothing()
    {
	final StatusData sd=this.tinfo;
	return new CalculationData(this.id,
				   this.currenttime,
				   this.p_buy,
				   this.p_sell,
				   this.p_last,
				   this.vol_p_buy,
				   this.vol_p_sell,
				   this.trades_cleared_before,
				   this,
				   sd.mparametersdata,
				   sd.mparametersdata.tau_embedding);
    }


    CalculationData calculateEverything()
    {
	// do calculations
	final StatusData sd=this.tinfo;

	if(CalculateAntecedents.ATTRIB_TR_DYNAMICS)
	{
	    this.tr_dynamics.calculateProjectedFlow(sd.mparametersdata.tau_embedding,
						    this.currenttime,
						    this.p_buy,
						    this.p_sell,
						    this.trades_cleared_before);
	}



	if(CalculateAntecedents.ATTRIB_BOOK_PRESETLEVELS)
	{
	    // volume with price thresholds
	    this.calculate_buy_thresholds.calculateThresholds(
		sd.mparametersdata.getBuyLevels(this.p_buy),
		this.currenttime,
		0.5,
		this.p_buy,
		this.p_sell);
	    this.calculate_sell_thresholds.calculateThresholds(
		sd.mparametersdata.getSellLevels(this.p_sell),
		this.currenttime,
		0.5,
		this.p_buy,
		this.p_sell);
	}

	if(CalculateAntecedents.ATTRIB_CRITICAL_EXPONENT_S)
	{
	    // critical exponent
	    this.calculate_buy_crit.calculateCrit(
		this.currenttime,
		this.p_buy,
		this.p_sell,
		sd.mparametersdata.good_price_n_points_max,
		this.p_buy/sd.mparametersdata.good_price_book_range_filter_antecedent,
		this.p_sell*sd.mparametersdata.good_price_book_range_filter_antecedent);

	    this.calculate_sell_crit.calculateCrit(
		this.currenttime,
		this.p_buy,
		this.p_sell,
		sd.mparametersdata.good_price_n_points_max,
		this.p_buy/sd.mparametersdata.good_price_book_range_filter_antecedent,
		this.p_sell*sd.mparametersdata.good_price_book_range_filter_antecedent);
	}

	if(CalculateAntecedents.ATTRIB_TRADE_TAU_CHANGE)
	{
	    // delayed trade volume
	    this.tradevolumecalculator.calculateTradeVolume(
		sd.mparametersdata.tau_embedding,
		this.currenttime,
		0.5,
		0.92,
		0.5,
		this.p_buy,
		this.p_sell,
		this.trades_cleared_before);
	}

	if(CalculateAntecedents.ATTRIB_HCONTRIB)
	{
	    this.tradehcontrib.calculateContribs(
		sd.mparametersdata.tau_embedding,
		this.currenttime,
		this.p_buy,
		this.p_sell,
		this.trades_cleared_before);
	    if(CalculateAntecedents.ATTRIB_TRADE_TAU_CHANGE)
	    {
		// check constrain if available
		for(int i=0;i<sd.mparametersdata.tau_embedding.length;i++)
		{
		    if(this.tradehcontrib.vol_all[i]!=this.tradevolumecalculator.v_tot_alltrades[i] ||
		       this.tradehcontrib.vol_hidden[i]!=this.tradevolumecalculator.v_tot_hiddentrades[i] ||
		       Double.isNaN(this.tradehcontrib.minprice[i])!=Double.isNaN(this.tradevolumecalculator.tot_minprice[i]) ||
		       (!Double.isNaN(this.tradehcontrib.minprice[i]) && this.tradehcontrib.minprice[i]!=this.tradevolumecalculator.tot_minprice[i]) ||
		       Double.isNaN(this.tradehcontrib.maxprice[i])!=Double.isNaN(this.tradevolumecalculator.tot_maxprice[i]) ||
		       (!Double.isNaN(this.tradehcontrib.maxprice[i]) && this.tradehcontrib.maxprice[i]!=this.tradevolumecalculator.tot_maxprice[i]))
		    {
			throw new RuntimeException(
			    "Data mismatch:"+
			    " vol_all["+i+"]="+this.tradehcontrib.vol_all[i]+
			    " tradevolumecalculator.v_tot_alltrades["+i+"]="+this.tradevolumecalculator.v_tot_alltrades[i]+
			    " vol_hidden["+i+"]="+this.tradehcontrib.vol_hidden[i]+
			    " tradevolumecalculator.v_tot_hiddentrades["+i+"]="+this.tradevolumecalculator.v_tot_hiddentrades[i]+
			    " minprice["+i+"]="+this.tradehcontrib.minprice[i]+
			    " tradevolumecalculator.tot_minprice["+i+"]="+this.tradevolumecalculator.tot_minprice[i]+
			    " maxprice["+i+"]="+this.tradehcontrib.maxprice[i]+
			    " tradevolumecalculator.tot_maxprice["+i+"]="+this.tradevolumecalculator.tot_maxprice[i]+
			    " delay="+sd.mparametersdata.tau_embedding[i]+
			    " currenttime="+currenttime);
		    }
		}
	    }
	}

	if(CalculateAntecedents.ATTRIB_BOOK_THRESHOLDS_FROM_TRADES &&
	   CalculateAntecedents.ATTRIB_TRADE_TAU_CHANGE)
	{
	    this.calculate_buy_fixedthresholds.calculateThresholds(
		this.currenttime,
		this.p_buy,
		this.p_sell,
		this.tradevolumecalculator.cc_minprice_buy);

	    this.calculate_sell_fixedthresholds.calculateThresholds(
		this.currenttime,
		this.p_buy,
		this.p_sell,
		this.tradevolumecalculator.cc_maxprice_sell);
	}

	if(CalculateAntecedents.ATTRIB_LIQUIDITYDEFICIT01)
	{
	    this.liquidity_buy01.calculateLiquidityDeficit(
		sd.mparametersdata.getBuyLevels(this.p_buy),
		this.currenttime,
		this.p_buy,
		this.p_sell,
		this.trades_cleared_before);
	    this.liquidity_sell01.calculateLiquidityDeficit(
		sd.mparametersdata.getSellLevels(this.p_sell),
		this.currenttime,
		this.p_buy,
		this.p_sell,
		this.trades_cleared_before);
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT4)
	{
	    this.book_trades_liquiditydeficit4.calculateLiquidityDeficit(
		sd.mparametersdata.getBuyLevels(this.p_buy),
		sd.mparametersdata.getSellLevels(this.p_sell),
		this.currenttime,
		this.p_buy,
		this.p_sell,
		this.trades_cleared_before);
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT5)
	{
	    this.book_trades_liquiditydeficit5_down.calculateLiquidityDeficit(
		sd.mparametersdata.getBuyLevels(this.p_buy),
		this.currenttime,
		this.p_buy,
		this.p_sell,
		this.trades_cleared_before);
	    this.book_trades_liquiditydeficit5_up.calculateLiquidityDeficit(
		sd.mparametersdata.getSellLevels(this.p_sell),
		this.currenttime,
		this.p_buy,
		this.p_sell,
		this.trades_cleared_before);
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT12)
	{
	    this.book_trades_liquiditydeficit12.calculateLiquidityDeficit(
		this.currenttime,
		sd.mparametersdata.N_TLEV,
		this.trades_cleared_before);
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT13)
	{
	    if(false)
	    {
		this.book_trades_liquiditydeficit13.calculateLiquidityDeficit(
		    sd.mparametersdata.tau_embedding,
		    this.currenttime,
		    this.trades_cleared_before);
	    }
	    else
	    { // recurrent speedup
		this.book_trades_liquiditydeficit13.calculateLiquidityDeficitRecurrent(
		    sd.mparametersdata.tau_embedding,
		    this.currenttime,
		    this.trades_cleared_before,
		    sd.recurrent_speedup_CalculateLiquidityDeficit13);
		sd.recurrent_speedup_CalculateLiquidityDeficit13=this.book_trades_liquiditydeficit13.tdata;
	    }
	}


	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT15)
	{
	    if(false)
	    {
		/*
		this.book_trades_liquiditydeficit15.calculateLiquidityDeficit(
		    sd.mparametersdata.tau_embedding,
		    this.currenttime,
		    this.trades_cleared_before);
		*/
		throw new RuntimeException("DISABLED");
	    }
	    else
	    { // recurrent speedup
		this.book_trades_liquiditydeficit15.calculateLiquidityDeficitRecurrent(
		    sd.mparametersdata.tau_embedding,
		    this.currenttime,
		    this.trades_cleared_before,
		    sd.recurrent_speedup_CalculateLiquidityDeficit15);
		sd.recurrent_speedup_CalculateLiquidityDeficit15=this.book_trades_liquiditydeficit15.tdata;
	    }
	}


	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT16)
	{
	    // recurrent speedup
	    this.book_trades_liquiditydeficit16.calculateLiquidityDeficitRecurrent(
		    sd.mparametersdata.tau_embedding,
		    this.currenttime,
		    this.trades_cleared_before,
		    sd.recurrent_speedup_CalculateLiquidityDeficit16);
	    sd.recurrent_speedup_CalculateLiquidityDeficit16=this.book_trades_liquiditydeficit16.tdata;
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT17)
	{
	    // recurrent speedup
	    this.book_trades_liquiditydeficit17.calculateLiquidityDeficitRecurrent(
		    sd.mparametersdata.neq_t,
		    sd.mparametersdata.tau_embedding,
		    this.currenttime,
		    this.trades_cleared_before,
		    sd.recurrent_speedup_CalculateLiquidityDeficit17);
	    sd.recurrent_speedup_CalculateLiquidityDeficit17=this.book_trades_liquiditydeficit17.tdata;
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT18)
	{
	    // recurrent speedup
	    this.book_trades_liquiditydeficit18.calculateLiquidityDeficitRecurrent(
		    sd.mparametersdata.neq_t,
		    sd.mparametersdata.tau_embedding,
		    this.currenttime,
		    this.trades_cleared_before,
		    sd.recurrent_speedup_CalculateLiquidityDeficit18);
	    sd.recurrent_speedup_CalculateLiquidityDeficit18=this.book_trades_liquiditydeficit18.tdata;
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT19)
	{
	    // recurrent speedup
	    this.book_trades_liquiditydeficit19.calculateLiquidityDeficitRecurrent(
		    sd.mparametersdata.neq_t,
		    sd.mparametersdata.tau_embedding,
		    this.currenttime,
		    this.trades_cleared_before,
		    sd.recurrent_speedup_CalculateLiquidityDeficit19);
	    sd.recurrent_speedup_CalculateLiquidityDeficit19=this.book_trades_liquiditydeficit19.tdata;
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT20)
	{
	    // recurrent speedup
	    this.book_trades_liquiditydeficit20.calculateLiquidityDeficitRecurrent(
		    sd.mparametersdata.neq_t,
		    sd.mparametersdata.tau_embedding,
		    this.currenttime,
		    this.trades_cleared_before,
		    sd.recurrent_speedup_CalculateLiquidityDeficit20);
	    sd.recurrent_speedup_CalculateLiquidityDeficit20=this.book_trades_liquiditydeficit20.tdata;
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT21)
	{
	    // recurrent speedup
	    this.book_trades_liquiditydeficit21.calculateLiquidityDeficitRecurrent(
		    sd.mparametersdata.neq_t,
		    sd.mparametersdata.tau_embedding,
		    this.currenttime,
		    this.trades_cleared_before,
		    sd.recurrent_speedup_CalculateLiquidityDeficit21);
	    sd.recurrent_speedup_CalculateLiquidityDeficit21=this.book_trades_liquiditydeficit21.tdata;
	}


	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT11)
	{
	    this.book_trades_liquiditydeficit11.calculateLiquidityDeficit(
		sd.mparametersdata.getBuyLevels(this.p_buy),
		sd.mparametersdata.getSellLevels(this.p_sell),
		this.currenttime,
		this.p_buy,
		this.p_sell,
		this.trades_cleared_before,
		this.vol_p_buy+this.vol_p_sell);
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT10)
	{
	    this.book_trades_liquiditydeficit10.calculateLiquidityDeficit(
		sd.mparametersdata.tau_embedding,
		this.currenttime,
		this.p_buy,
		this.p_sell,
		this.trades_cleared_before);
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT9)
	{
	    this.book_trades_liquiditydeficit9.calculateLiquidityDeficit(
		sd.mparametersdata.getBuyLevels(this.p_buy),
		sd.mparametersdata.getSellLevels(this.p_sell),
		this.currenttime,
		this.p_buy,
		this.p_sell,
		this.trades_cleared_before,
		sd.mparametersdata.VOLUME_FACTOR,
		this.vol_p_buy+this.vol_p_sell);
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT9T)
	{
	    this.tradesT_liquiditydeficit9T.calculateLiquidityDeficit(
	      	sd.mparametersdata.tau_embedding,
		this.currenttime,
		this.trades_cleared_before,
		this.p_buy,
		this.p_sell,
		this.vol_p_buy+this.vol_p_sell);
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT8)
	{
	    final double [][] r=
		sd.mparametersdata.getDeltaOKAndFailure(this.p_buy,this.p_sell);
	    this.book_trades_liquiditydeficit8.calculateLiquidityDeficit(
		this.currenttime,
		r[0],
		r[1],
		this.p_buy,
		this.p_sell,
		this.trades_cleared_before);
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT7)
	{
	    this.book_trades_liquiditydeficit7.calculateLiquidityDeficit(
		this.currenttime,
		this.p_buy,
		this.p_sell,
		sd.mparametersdata.N_TLEV,
		this.trades_cleared_before);
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT6)
	{
	    this.book_trades_liquiditydeficit6.calculateLiquidityDeficit(
		this.currenttime,
		this.p_buy,
		this.p_sell,
		sd.mparametersdata.N_TLEV,
		this.trades_cleared_before);
	}

	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT3)
	{
	    this.book_trades_liquiditydeficit3.calculateLiquidityDeficit(
		this.currenttime,
		this.p_buy,
		this.p_sell,
		sd.mparametersdata.N_TLEV,
		this.trades_cleared_before);
	}

	if(CalculateAntecedents.ATTRIB_LIQUIDITYDEFICIT2)
	{
	    final double [] shiftlevels=sd.mparametersdata.getSellLevels(0.5*(this.p_buy+this.p_sell));
	    final double medp=0.5*(this.p_buy+this.p_sell);
	    for(int i=shiftlevels.length;--i>=0;)
	    {
		shiftlevels[i]-=medp;
	    }
	    this.liquiditydeficit2.calculateLiquidityDeficit(shiftlevels,
							     currenttime,
							     this.p_buy,
							     this.p_sell,
							     this.trades_cleared_before);
	}

	if(CalculateAntecedents.ATTRIB_BOOK_LIQUIDITYDEFICIT02)
	{
	    this.book_liquiditydeficit02.calculateLiquidityDeficit(
		this.currenttime,
		30,// FIXME final int n_levels_book,
		50,// FIXME n_levels_trades
		this.p_buy,
		this.p_sell);
	}

	if(CalculateAntecedents.ATTRIB_VIRTUAL_SHIFTS_TAU)
	{
	    this.calculate_virtual_shifts_tau.calculateVirtualShifts(
		this.currenttime,
		sd.mparametersdata.tau_embedding,
		this.p_buy,
		this.p_sell,
		0.5*(this.p_buy+this.p_sell)*
		(sd.mparametersdata.good_price_book_range_filter_antecedent-1));
	}

	if(CalculateAntecedents.ATTRIB_VIRTUAL_SHIFTS)
	{
	    this.calculate_virtual_shifts.calculateVirtualShifts(
		this.currenttime,
		this.p_buy,
		this.p_sell,
		/* // max shift. simplistic model as a fixed cut.
		0.5*(this.p_buy+this.p_sell)*
		  (sd.mparametersdata.good_price_book_range_filter_antecedent-1)
		*/
		this.tradehcontrib.maxprice[sd.mparametersdata.tau_embedding.length-1]-this.tradehcontrib.minprice[sd.mparametersdata.tau_embedding.length-1]
		,
		this.trades_cleared_before);
	}

	if(CalculateAntecedents.ATTRIB_VIRTUAL_SHIFTS2)
	{
	    this.calculate_virtual_shifts2.calculateVirtualShifts(
		sd.mparametersdata.tau_embedding,
		this.currenttime,
		this.p_buy,
		this.p_sell,
		0.5*(this.p_buy+this.p_sell)*
		(sd.mparametersdata.good_price_book_range_filter_antecedent-1),
		this.trades_cleared_before);
	}

	if(CalculateAntecedents.ATTRIB_BOOK_TAU_CHANGE)
	{
	    // book volume with time thresholds
	    this.bookvolumecalculator.calculateBookVolume(
		sd.mparametersdata.tau_embedding,
		this.currenttime,
		0.5,
		this.p_buy,
		this.p_sell,
		this.p_buy/sd.mparametersdata.good_price_book_range_filter_antecedent,
		this.p_sell*sd.mparametersdata.good_price_book_range_filter_antecedent);

	    if(this.calculate_buy_thresholds.vtotalorders!=this.bookvolumecalculator.vtotal_buy||
	       this.calculate_sell_thresholds.vtotalorders!=this.bookvolumecalculator.vtotal_sell)
	    {
		throw new RuntimeException("TOTAL VOLUME MISMATCH: vc_total_buy="+
					   this.calculate_buy_thresholds.vtotalorders+
					   " vtotal_buy="+this.bookvolumecalculator.vtotal_buy+
					   " vc_total_sell="+this.calculate_sell_thresholds.vtotalorders+
					   " vtotal_sell="+this.bookvolumecalculator.vtotal_sell);
	    }
	}


	if(DEBUG_PRINTCALCULATIONS)
	{
	    System.err.println("calculateEverything::TPC="+this.currenttime+" ticker="+sd.ticker+
			       " p_buy="+this.p_buy+" p_sell="+this.p_sell);
	}
	// return all we calculated
	final CalculationData cd=
	    new CalculationData(this.id,
				this.currenttime,
				this.p_buy,
				this.p_sell,
				this.p_last,
				this.vol_p_buy,
				this.vol_p_sell,
				this.trades_cleared_before,
				this,

				sd.mparametersdata,
				sd.mparametersdata.tau_embedding);

	if(this.externalanalyzis!=null)
	{
	    this.attribscalculator.setAntecedentAttribs(
			sd.ticker,
			sd.mparametersdata,
			cd,
			sd.antecedents);
	    if(ClassificationPredictorNewTrade.DEBUGGING_OUT_RULESCALCULATION)
	    {
		throw new RuntimeException("FIXME");
		/*
		((FilestemData)this.externalanalyzis).__DUMPprintAntecedents(
		    cd.id,
		    sd.ticker,
		    this.attribscalculator,
		    sd.PREDICTOR.getPredictionsForTesting(this.attribscalculator,cd));
		*/
	    }
	    else
	    {
		this.externalanalyzis.printAntecedents(cd.id,
						       sd.ticker,
						       this.attribscalculator);
	    }
	}
	return cd;


    }


    private void preparePricesNoBook()
    {

	calculatePrices();

	// set buffers to invalid state to catch their misusage
	this.buy.setInvalidState();
	this.sell.setInvalidState();
	this.tmpordersbuf.setInvalidState();
    }



    void prepareBook()
    {

	if(true)
	{
	    final int n=this.savedtrades.getSize();
	    if(n>0)
	    {
		this.p_last=this.savedtrades.getBuffer()[n-1].price;
	    }
	    else
	    {
		this.p_last=Double.NaN;
	    }
	}

	if(this.tinfo.mparametersdata.FLAG_NO_BOOK)
	{
	    // no order data available, just best prices
	    preparePricesNoBook();
	    return;
	}

	if(this.tmpordersbuf.getCapacity()<this.buy.getCapacity())
	{ // temporary buffer may be increased independently and have the size large, than buy size
	    this.tmpordersbuf.replaceBuffer(new BookItem [this.buy.getCapacity()]);
	}


	// check temporary trade buffer size
	if(this.tmptradesbuf.getCapacity()<this.savedtrades.getCapacity())
	{
	    this.tmptradesbuf.replaceBuffer(new Trade [this.savedtrades.getCapacity()]);
	}

	// get current buy/sell prices
	calculatePrices();


	// TradeList is already sorted by trade date.
	// oldest trade has 0-th index.
	// no sort is necessary here.


	// check constrains
	if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    final BookItem [] bb=this.buy.getBuffer();
	    for(int i=0,n=this.buy.getSize();i<n;i++)
	    {
		if(i>0)
		{
		    if(bb[i-1].price<bb[i].price)
		    {
			throw new RuntimeException(
			    "================!!!!! "+this.tinfo.ticker+" UNSORTED BUY PRICES i1="+
			    i+" p1="+bb[i].price+" i2="+(i-1)+" p2="+
			    bb[i-1].price);
		    }
		    else
		    {
			if(bb[i-1].price==bb[i].price)
			{
			    if(bb[i-1].time>bb[i].time)
			    {
				throw new RuntimeException(
				    "================!!!!! "+this.tinfo.ticker+" UNSORTED BUY TIME i1="+
				    i+" t1="+bb[i].time+
				    " i2="+(i-1)+" t2="+bb[i-1].time);
			    }
			    else if(bb[i-1].time==bb[i].time && (bb[i-1].orderreferencenumber>bb[i].orderreferencenumber))
			    {
				throw new RuntimeException(
				    "================!!!!! "+this.tinfo.ticker+" UNSORTED BUY ORDERREFERENCENUMBEE i1="+
				    i+" t1="+bb[i].orderreferencenumber+
				    " i2="+(i-1)+" t2="+bb[i-1].orderreferencenumber);
			    }
			}
		    }
		}
		if(bb[i].buy_sell_indicator!=BookItem.BOOK_BUY)
		{
		    throw new RuntimeException("Invalid order type="+bb[i].buy_sell_indicator);
		}
	    }
	}

	if(OrdersAndTrades.CONSTRAINS_EXPENSIVE_CHECKS)
	{
	    final BookItem [] bb=this.sell.getBuffer();
	    for(int i=0,n=this.sell.getSize();i<n;i++)
	    {
		if(i>0)
		{
		    if(bb[i-1].price>bb[i].price)
		    {
			throw new RuntimeException(
			    "================!!!!! "+this.tinfo.ticker+" UNSORTED SELL PRICES"+
			    i+" p1="+bb[i].price+
			    " i2="+(i-1)+" p2="+bb[i-1].price);
		    }
		    else
		    {
			if(bb[i-1].price==bb[i].price)
			{
			    if(bb[i-1].time>bb[i].time)
			    {
				throw new RuntimeException(
				    "================!!!!! "+this.tinfo.ticker+" UNSORTED BUY TIME i1="+
				    i+" t1="+bb[i].time+
				    " i2="+(i-1)+" t2="+bb[i-1].time);
			    }
			    else if(bb[i-1].time==bb[i].time && (bb[i-1].orderreferencenumber>bb[i].orderreferencenumber))
			    {
				throw new RuntimeException(
				    "================!!!!! "+this.tinfo.ticker+" UNSORTED BUY ORDERREFERENCENUMBEE i1="+
				    i+" orderreferencenumber1="+bb[i].orderreferencenumber+
				    " i2="+(i-1)+" orderreferencenumber2="+bb[i-1].orderreferencenumber);
			    }
			}
		    }
		}
		if(bb[i].buy_sell_indicator!=BookItem.BOOK_SELL)
		{
		    throw new RuntimeException("Invalid order type="+bb[i].buy_sell_indicator);
		}
	    }
	}

	if(DEBUG_PRINTPRICES)
	{
	    // dump ordered boot
	    final int bs=this.buy.getSize(),ss=this.sell.getSize(),m=Math.max(bs,ss);
	    System.err.println("================BOOK "+this.tinfo.ticker+" N_BUY="+bs+" N_SELL="+ss);
	    final BookItem [] bb=this.buy.getBuffer(),sb=this.sell.getBuffer();
	    for(int i=0;i<m;i++)
	    {
		System.err.print(i+"\t");
		if(i<bs)
		{
		    System.err.print(bb[i].price+"\t("+bb[i].shares+")"+"\t");
		}
		else
		{
		    System.err.print("\t\t");
		}
		if(i<ss)
		{
		    System.err.print(sb[i].price+"\t("+sb[i].shares+")"+"\t");
		}
		else
		{
		    System.err.print("\t\t");
		}
		System.err.println();
	    }

	}

    }


    void training_CalculateConsequent(final CalculationDataRef cd)
    {
	final StatusData sd=this.tinfo;
	final CalculationDataList pastcalculations=sd.pastcalculations;

	if(pastcalculations.training_last_data_processed==null)
	{
	    // looks not prety, but we do this just once
	    pastcalculations.training_last_data_processed_offset=pastcalculations.getSize();
	    pastcalculations.forwardForEachEntry(
		new java.util.function.Predicate<CalculationDataRef>(){
		    	public final boolean test(final CalculationDataRef b)
			{
			    pastcalculations.training_last_data_processed=b;
			    return false;
			}
		},pastcalculations.training_last_data_processed_offset);
	    if(pastcalculations.training_last_data_processed==null)
	    {
		throw new RuntimeException("This should never happen training_last_processed="+
					   pastcalculations.training_last_data_processed);
	    }
	    // We got training_last_processed as the first element to reference.
	    // We miss this first element in data processing
	    // (because this is a reference), this is not a big deal
	    // because without previous data it is very much useless.
	}
	// get reference
	final CalculationDataRef training_last_processed=pastcalculations.training_last_data_processed;
	final long tmax=sd.mparametersdata.tau_embedding_consequent[sd.mparametersdata.tau_embedding_consequent.length-1];
	pastcalculations.forwardForEachEntry(
	    new java.util.function.Predicate<CalculationDataRef>(){
		private boolean flag_first=true;
		public final boolean test(final CalculationDataRef b)
		{
		    if(this.flag_first)
		    {
			// the first one is the element already processed
			if(b!=training_last_processed)
			{
			    throw new RuntimeException("This should never hapen b="+b+
						       " this.training_last_processed="+
						       training_last_processed+
						       " offset="+
						       pastcalculations.training_last_data_processed_offset+
						       " size="+pastcalculations.getSize());
			}
			this.flag_first=false;
			return true;
		    }
		    final long bt=b.getTime(),cdt=cd.getTime();
		    if(bt<pastcalculations.training_last_data_processed.getTime()||cdt<bt)
		    {
			throw new RuntimeException("Something is terrible wrong with sorting");
		    }
		    if(bt+tmax>=cdt && CalculateConsequents.CALCULATE_LOOK_FORWARD)
		    {
			// too little time left to determine consequents.
			// process next time when the data become available
			return false;
		    }
		    pastcalculations.training_last_data_processed_offset--;
		    pastcalculations.training_last_data_processed=b;
		    // do training for b


		    if(CalculateConsequents.CALCULATE_LOOK_FORWARD)
		    {
			consequentstats.calculateStats(cdt,
						       bt,
						       0.5,
						       0.6,
						       b.p_buy,
						       b.p_sell,
						       sd.mparametersdata.tau_embedding_consequent,
						       trades_cleared_before);
		    }

		    consequentscalculator.setConsequentsAttribs(
			sd.ticker,
			sd.mparametersdata,
			cdt,
			consequentstats,
			b,
			sd.consequents);

		    // now combine all the calculated data together
		    externalanalyzis.printConsequents(b.id,sd.ticker,consequentscalculator);

		    // clear data because we may be called second time in forEach() loop.
		    consequentstats.clearData();
		    consequentscalculator.clearData();
		    return true;
		}

	    },pastcalculations.training_last_data_processed_offset);

    }



    public long clear()
    {
	final long old_id=this.id;

	if(CalculateAntecedents.ATTRIB_TR_DYNAMICS)
	{
	    this.tr_dynamics.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOK_PRESETLEVELS)
	{
	    this.calculate_buy_thresholds.setData(-1);
	    this.calculate_sell_thresholds.setData(-1);
	}
	if(CalculateAntecedents.ATTRIB_CRITICAL_EXPONENT_S)
	{
	    this.calculate_buy_crit.clearData();
	    this.calculate_sell_crit.clearData();
	}
	if(CalculateAntecedents.ATTRIB_TRADE_TAU_CHANGE)
	{
	    this.tradevolumecalculator.setData(-1);
	}
	if(CalculateAntecedents.ATTRIB_HCONTRIB)
	{
	    this.tradehcontrib.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOK_THRESHOLDS_FROM_TRADES)
	{
	    this.calculate_buy_fixedthresholds.clearData();
	    this.calculate_sell_fixedthresholds.clearData();
	}
	if(CalculateAntecedents.ATTRIB_LIQUIDITYDEFICIT01)
	{
	    this.liquidity_buy01.clear();
	    this.liquidity_sell01.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT4)
	{
	    this.book_trades_liquiditydeficit4.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT5)
	{
	    this.book_trades_liquiditydeficit5_down.clear();
	    this.book_trades_liquiditydeficit5_up.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT12)
	{
	    this.book_trades_liquiditydeficit12.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT13)
	{
	    this.book_trades_liquiditydeficit13.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT15)
	{
	    this.book_trades_liquiditydeficit15.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT16)
	{
	    this.book_trades_liquiditydeficit16.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT17)
	{
	    this.book_trades_liquiditydeficit17.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT18)
	{
	    this.book_trades_liquiditydeficit18.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT19)
	{
	    this.book_trades_liquiditydeficit19.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT20)
	{
	    this.book_trades_liquiditydeficit20.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT21)
	{
	    this.book_trades_liquiditydeficit21.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT11)
	{
	    this.book_trades_liquiditydeficit11.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT10)
	{
	    this.book_trades_liquiditydeficit10.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT9)
	{
	    this.book_trades_liquiditydeficit9.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT9T)
	{
	    this.tradesT_liquiditydeficit9T.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT8)
	{
	    this.book_trades_liquiditydeficit8.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT7)
	{
	    this.book_trades_liquiditydeficit7.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT6)
	{
	    this.book_trades_liquiditydeficit6.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT3)
	{
	    this.book_trades_liquiditydeficit3.clear();
	}
	if(CalculateAntecedents.ATTRIB_LIQUIDITYDEFICIT2)
	{
	    this.liquiditydeficit2.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOK_LIQUIDITYDEFICIT02)
	{
	    this.book_liquiditydeficit02.clear();
	}
	if(CalculateAntecedents.ATTRIB_VIRTUAL_SHIFTS_TAU)
	{
	    this.calculate_virtual_shifts_tau.clear();
	}
	if(CalculateAntecedents.ATTRIB_VIRTUAL_SHIFTS)
	{
	    this.calculate_virtual_shifts.clear();
	}
	if(CalculateAntecedents.ATTRIB_VIRTUAL_SHIFTS2)
	{
	    this.calculate_virtual_shifts2.clear();
	}
	if(CalculateAntecedents.ATTRIB_BOOK_TAU_CHANGE)
	{
	    this.bookvolumecalculator.setData(-1);
	}
	this.attribscalculator.clearData();
	this.consequentscalculator.clearData();
	this.consequentstats.clearData();
	this.consequentrealtime.clearData();


	this.buy.clearAll();
	this.sell.clearAll();
	this.tmpordersbuf.clearAll();
	this.savedtrades.clearAll();
	this.savedpseudotrades.clearAll();
	this.tmptradesbuf.clearAll();

	this.trades_cleared_before=this.currenttime=0L;
	this.tinfo=null;
	this.p_buy=this.p_sell=this.p_last=0.0;
	this.id=-1;

	return old_id;
    }

}
