package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.broker.*;
import com.polytechnik.realtimebook.*;

/** A first version of trading algorith,
 *  @version $Id: TradingAlgorithmByEvents.java,v 1.822 2020/02/06 01:28:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class TradingAlgorithmByEvents extends TradingAlgorithm<TradingAlgorithmByEvents.TradingEventsFound>  implements DumpTradingDataAnalyzable<CalculationDataRef> {

/** An event we use. */
class TradingEventsFound extends TradingAlgorithm.TradingEvent {
    /** Direction: true-up, false - down. */
    final boolean flag_up;

    private final double max_diff_cancel_off_price_order;
    private final long time_to_die;
    final String comments;
    private boolean flag_wasEnter_up,flag_wasEnter_down;

    TradingEventsFound(final boolean flag_up,
		       final long time,
		       final double price,
		       final long time_to_die,
		       final double max_diff_cancel_off_price_order,
		       final boolean flag_close_postion,
		       final String comments,
		       final boolean flag_wasEnter_up,
		       final boolean flag_wasEnter_down)
    {
	super(flag_up?BOrder.SIDE_BUY:BOrder.SIDE_SELL_SHORT,time,price,flag_close_postion);
	this.time_to_die=time_to_die;
	this.comments=comments;
	this.max_diff_cancel_off_price_order=max_diff_cancel_off_price_order;
	this.flag_up=flag_up;
	this.flag_wasEnter_up=flag_wasEnter_up;
	this.flag_wasEnter_down=flag_wasEnter_down;
    }
    /** For exit. */
    TradingEventsFound(final boolean flag_up,
		       final long time,
		       final double price,
		       final double max_diff_cancel_off_price_order,
		       final String comments,
		       final boolean flag_wasEnter_up,
		       final boolean flag_wasEnter_down)
    {
	this(flag_up,time,price,sdv.moneymanager.getTimeOrderToDie_ClosePostion(time),max_diff_cancel_off_price_order,true,comments,flag_wasEnter_up,flag_wasEnter_down);
    }


    private boolean cancelled=false;
    void setOrdersCancelled()
    {
	this.cancelled=true;
    }

    public boolean shouldOrderBeAlive(final AOrder o,final long t,final double p_buy,final double p_sell)
    {
	if(this.time_to_die<0)
	{
	    throw new RuntimeException("default time_to_die is disabled");
	}
	if(this.cancelled)
	{
	    return false; // manually cancelled
	}
	if(this.time_to_die<t)
	{
	    return false; // timeout
	}
	if(isOrderOffPrice(o,p_buy,p_sell))
	{
	    return false; // price slipped
	}
	return true; // order OK
    }
    private boolean isOrderOffPrice(final AOrder o,final double p_buy,final double p_sell)
    {
	if(this.max_diff_cancel_off_price_order>=0)
	{
	    final boolean op=
		o.getSharesSign()>0?o.price<p_buy-this.max_diff_cancel_off_price_order-OrdersFlow.EPS:
		o.price>(p_sell+this.max_diff_cancel_off_price_order+OrdersFlow.EPS);
	    if(PRINT_DEBUG && op)
	    {
		System.err.println(getClass().getName()+":isOrderOffPrice:OFF PRICE:"+
				   " o.price="+o.price+
				   " p_buy="+p_buy+
				   " p_sell="+p_sell+
				   " shares sign="+o.getSharesSign()+
				   " this.max_diff_cancel_off_price_order="+this.max_diff_cancel_off_price_order);
	    }
	    return op;
	}
	// order is not off price
	return false;
    }

    public String getDescription()
    {
	return
	    this.comments+":"+
	    sdv.ticker+
	    " price="+this.price+
	    " flag_up="+this.flag_up+
	    " t="+BookItem.getTimeDiag(this.time)+
	    " flag_close_postion="+flag_close_postion+
	    " time_to_die="+BookItem.getTimeDiag(this.time_to_die)+
	    " max_diff_cancel_off_price_order="+this.max_diff_cancel_off_price_order+
	    " cancelled="+this.cancelled;
    }
}
// END OF TradingEventsFound


    final double MINIMAL_SPREAD;
    private final StatusData sdv;


    /** Global options. Should we read them per stock basis? */
    final int
	STYLE_ENTER_PRICING=PSTYLE_AGGRESSIVE,//PSTYLE_EPSPLAST,//PSTYLE_AGGRESSIVE,//PSTYLE_PLAST,//PSTYLE_CONSERVATIVE,
	STYLE_EXIT_PRICING=PSTYLE_AGGRESSIVE,
	STYLE_EMERGENCY_EXIT_PRICING=PSTYLE_CONSERVATIVE;

    final Trader trader;

    private final double THR_ENTER,THR_EXIT,THRW_ENTER,THRW_EXIT;
    private final long [] TAU;
    private final int NEQ_t;
    private final String DIR_DIAG;
    TradingAlgorithmByEvents(final ModelParameters.ModelParametersData mparametersdata,
			     final StatusData sd,
			     final Trader trader)
    {
	super(sd);
	this.sdv=sd;
	this.trader=trader;
	MINIMAL_SPREAD=sdv.mparametersdata.MINIMAL_SPREAD;
	THR_ENTER=sdv.mparametersdata.thr_enter;
	THR_EXIT=sdv.mparametersdata.thr_exit;
	THRW_ENTER=sdv.mparametersdata.thrw_enter;
	THRW_EXIT=sdv.mparametersdata.thrw_exit;
	TAU=new long []{sdv.mparametersdata.tau};
	NEQ_t=this.sdv.mparametersdata.neq_t;
	DIR_DIAG=sdv.mparametersdata.write_knowledgedir;
	if(PRINT_DEBUG)
	{
	    System.err.println(getClass().getName()+":-----------------------------------------");
	    System.err.println("TICKER="+sdv.ticker);
	    System.err.println("THR_ENTER="+THR_ENTER);
	    System.err.println("THR_EXIT="+THR_EXIT);
	    System.err.println("THRW_ENTER="+THRW_ENTER);
	    System.err.println("THRW_EXIT="+THRW_EXIT);
	    System.err.println("MINIMAL_SPREAD="+MINIMAL_SPREAD);
	    System.err.println("TAU="+java.util.Arrays.toString(TAU));
	    System.err.println("NEQ_t="+NEQ_t);
	    System.err.println("DIR_DIAG="+DIR_DIAG);
	    System.err.flush();
	}
    }

    private NkQVMLegendreShifted op_data=null;
    private CalculationData cdr=null;
    private final CalculateLiquidityDeficit20.VQ [] eprev=new CalculateLiquidityDeficit20.VQ [1];

    void updateInternalState(final CalculationData c)
    {
	if(this.op_data!=null || this.cdr!=null)
	{
	    throw new IllegalStateException("Race is somewhere");
	}
	this.cdr=c;
	c.topp.book_trades_liquiditydeficit20.calculateLiquidityDeficitRecurrent(
	    NEQ_t,
	    TAU,
	    c.getTime(),
	    c.t_trades_cleared_before,
	    this.eprev);
	this.eprev[0]=c.topp.book_trades_liquiditydeficit20.tdata[0];
	this.op_data=this.eprev[0].nkqvmpp;
	if(PRINT_DEBUG)
	{
	   System.err.print("DIAG:TICKER="+sdv.ticker+
			    " time="+BookItem.getTimeDiag(c.getTime())+" ("+(c.getTime()/(3600.0*TraderInterface.TIME_SCALE))+") "+
			    " getDS()="+getDS()+
			    " getDWenter()="+getDWenter()+" getDWexit()="+getDWexit()+
			    " p_last="+c.p_last);
	   final TradingEventsFound e=getEventOntherunDiag();
	   if(e!=null)
	   {
	       System.err.print(" EOR: flag_up="+e.flag_up+" flag_close_postion="+e.flag_close_postion+
				" price="+e.price+" time="+BookItem.getTimeDiag(e.time)+" ("+(e.time/(3600.0*TraderInterface.TIME_SCALE))+") "+
				" flag_wasEnter_up="+e.flag_wasEnter_up+" flag_wasEnter_down="+e.flag_wasEnter_down+
				" comments="+e.comments);

	   }
	   System.err.println();
	}
    }
   void clear()
   {
       this.op_data=null;
       this.cdr=null;
       this.dir_opportunity=this.dir_exit=0;
   }

   private double getDS()
   {
       //return this.op_data.bo_ds1_a;
       //return Double.NaN; // FIXME
       //return this.op_data.getDS();
       return Double.NaN;
   }
   private double getDWenter()
   {
       return Math.pow(this.op_data.I.wL,2);
   }
   private double getDWexit()
   {
       return Math.pow(this.op_data.I.wH,2);
   }

   public TradingEventsFound getEmergencyClose(final TradingEventsFound event_ontherun,final Positions.PositionsInfo pinfo)
    {
	final int shares_signed=pinfo.trade_info.current_shares;
	final boolean flag_dir=!(shares_signed>0);
	if(event_ontherun!=null) event_ontherun.setOrdersCancelled();

	return new TradingEventsFound(
	    flag_dir,
	    pinfo.time,
	    getPrice(flag_dir,pinfo.p_buy,pinfo.p_sell,this.cdr.p_last,STYLE_EMERGENCY_EXIT_PRICING),
	    getOrderOffPriceCancel(flag_dir,pinfo.p_buy,pinfo.p_sell,OPC_CLOSE_OTHER),
	    "FORCE_EXIT",
	    false,false);
    }

    public TradingEventsFound getOpportunityEvents(final TradingEventsFound event_ontherun,
						   final Positions.PositionsInfo pinfo)
    {
	if(event_ontherun!=null)
	{
	    throw new RuntimeException("event_ontherun="+event_ontherun.getDescription());
	}
	//if(!(getIO()>0)) return null;
	//if(!(Math.abs(getDIPScaled())>THR_DIP)) return null;

	//if(!(getDWenter()>THRW_ENTER)) return null;

	final double dS=getDS();
	final boolean flag_dir;
	if(dS>THR_ENTER) flag_dir=false;
	else if(dS<-THR_ENTER) flag_dir=true;
	else return null;

	return new TradingEventsFound(
	    flag_dir,
	    pinfo.time,
	    getPrice(flag_dir,pinfo.p_buy,pinfo.p_sell,this.cdr.p_last,STYLE_ENTER_PRICING),
	    sdv.moneymanager.getTimeOrderToDie_Opportunity(pinfo.time), // use default
	    Double.NaN, // no off price cancel for enter
	    false,
	    "OPPORTUNITY:"+getTradeKStatsTxt(),
	    flag_dir,!flag_dir);
    }
    public TradingEventsFound getExitEvents(final TradingEventsFound event_ontherun,
					    final Positions.PositionsInfo pinfo)
    {
	if(event_ontherun==null)
	{
	    throw new RuntimeException("event_ontherun="+null);
	}

	final double dS=getDS();
	if(dS>THR_ENTER){event_ontherun.flag_wasEnter_down=true;}
	if(dS<-THR_ENTER){event_ontherun.flag_wasEnter_up=true;}


	final boolean flag_dir;
	if(event_ontherun.flag_close_postion)
	{ // replace exit order, same direction
	    flag_dir=event_ontherun.flag_up;
	}
	else
	{ // exit from opportunity, opposite direction
	    flag_dir=!event_ontherun.flag_up;
	}

	int exit_type=0;
	if(!(flag_dir==true) || !event_ontherun.flag_wasEnter_down){exit_type=0;}
	else if(dS<-THR_ENTER){exit_type=-2;}
	else if(dS<-THR_EXIT){exit_type=-1;}
	if(exit_type!=0)
	{
	    event_ontherun.flag_wasEnter_down=false;
	    event_ontherun.setOrdersCancelled();
	    return new TradingEventsFound(
		flag_dir, // true
		pinfo.time,
		getPrice(flag_dir,pinfo.p_buy,pinfo.p_sell,this.cdr.p_last,STYLE_EXIT_PRICING),
		getOrderOffPriceCancel(flag_dir,pinfo.p_buy,pinfo.p_sell,OPC_CLOSE_HOLDED),
		"BUY TO COVER:"+getTradeKStatsTxt()+" exit_type="+exit_type+
		" EOR.flag_up="+event_ontherun.flag_up+" EOR.flag_close_postion="+event_ontherun.flag_close_postion,
		event_ontherun.flag_wasEnter_up,event_ontherun.flag_wasEnter_down);
	}

	exit_type=0;
	if(!(flag_dir==false) || !event_ontherun.flag_wasEnter_up){exit_type=0;}
	else if(dS>THR_ENTER){exit_type=2;}
	else if(dS>THR_EXIT){exit_type=1;}
	if(exit_type!=0)
	{
	    event_ontherun.flag_wasEnter_up=false;
	    event_ontherun.setOrdersCancelled();
	    return new TradingEventsFound(
		flag_dir, // false
		pinfo.time,
		getPrice(flag_dir,pinfo.p_buy,pinfo.p_sell,this.cdr.p_last,STYLE_EXIT_PRICING),
		getOrderOffPriceCancel(flag_dir,pinfo.p_buy,pinfo.p_sell,OPC_CLOSE_HOLDED),
		"SELL POSITION:"+getTradeKStatsTxt()+" exit_type="+exit_type+
		" EOR.flag_up="+event_ontherun.flag_up+" EOR.flag_close_postion="+event_ontherun.flag_close_postion,
		event_ontherun.flag_wasEnter_up,event_ontherun.flag_wasEnter_down);
	}
	return null;
    }

    String getTradeKStatsTxt()
    {
	final double p_last=this.cdr.p_last;
	return
	    " getDS()="+getDS()+
	    " getDWenter()="+getDWenter()+" getDWexit()="+getDWexit()+
	    " tau="+eprev[0].tau+" currenttime="+BookItem.getTimeDiag(eprev[0].currenttime);
    }



    static final int OPC_CLOSE_HOLDED=97,OPC_CLOSE_OTHER=98,OPC_OPPORTUNITY=99;
    /** A method to return price difference at which limit order should be cancelled.
     */
    double getOrderOffPriceCancel(final boolean flag_up,
				  final double p_buy,
				  final double p_sell,
				  final int twhat)
    {
	return Double.NaN; // turned off for now
    }


    static final int
    PSTYLE_PREVLEVEL=866, // for buy: best buy-eps, for sell: best sell+eps
    PSTYLE_SUPERAGGRESSIVE=867, // for buy: best buy, for sell: best sell
    PSTYLE_AGGRESSIVE=868, // for buy: buy+eps, for sell: sell-eps
    PSTYLE_MIDPOINT=869, // for buy: buy+d/2, for sell: sell-d/2
    PSTYLE_CONSERVATIVEATTEMPT=870, // for buy: sell-eps, for sell: buy+eps
    PSTYLE_CONSERVATIVE=871, // for buy: sell, for sell: buy
    PSTYLE_PLAST=872,
    PSTYLE_EPSPLAST=873;
double getPrice(final boolean flag_buy,
		final double p_buy,
		final double p_sell,
		final double p_last,
		final int pstyle)
{
    if(pstyle==PSTYLE_PLAST)
    {
	return p_last;
    }
    if(pstyle==PSTYLE_EPSPLAST)
    {
	return flag_buy?p_last+MINIMAL_SPREAD:p_last-MINIMAL_SPREAD;
    }

    if(pstyle==PSTYLE_PREVLEVEL)
    {
	return flag_buy?p_buy-MINIMAL_SPREAD:p_sell+MINIMAL_SPREAD;
    }

    if(pstyle==PSTYLE_SUPERAGGRESSIVE)
    {
	return flag_buy?p_buy:p_sell;
    }

    if(pstyle==PSTYLE_AGGRESSIVE)
    {
	if(flag_buy)
	{
	    final double px=p_buy+MINIMAL_SPREAD;
	    return px<p_sell?px:p_buy;
	}
	else
	{
	    final double px=p_sell-MINIMAL_SPREAD;
	    return px>p_buy?px:p_sell;
	}
    }

    if(pstyle==PSTYLE_MIDPOINT)
    {
	final double s=Math.round((p_sell-p_buy)/MINIMAL_SPREAD)*MINIMAL_SPREAD;
	if(s<=0)
	{
	    return flag_buy?p_buy:p_sell;
	}
	if(flag_buy)
	{
	    final double px=p_buy+s;
	    return px<p_sell?px:p_buy;
	}
	else
	{
	    final double px=p_sell-s;
	    return px>p_buy?px:p_sell;
	}
    }

    if(pstyle==PSTYLE_CONSERVATIVEATTEMPT)
    {
	if(flag_buy)
	{
	    final double px=p_sell-MINIMAL_SPREAD;
	    return px>p_buy?px:p_buy;
	}
	else
	{
	    final double px=p_buy+MINIMAL_SPREAD;
	    return px<p_sell?px:p_sell;
	}
    }

    if(pstyle==PSTYLE_CONSERVATIVE)
    {
	return flag_buy?p_sell:p_buy;
    }

    throw new IllegalArgumentException("pstyle="+pstyle);
}




// =========================== DUMP REPORTS & DIAGNOSTICS =========================

    private int dir_opportunity=0,dir_exit=0;
    private long order_id;
    /** A hook to dump order posting events to diagnostics file. */
    @Override
    protected void testing_reportPostedOrders(final AOrder o)
    {
	dir_opportunity=dir_exit=0;
	order_id=0;
	if(o==null) return;
	order_id=o.getID();
	if(o.side==AOrder.SIDE_BUY||o.side==AOrder.SIDE_SELL_SHORT)
	{
	    dir_opportunity=-o.getSharesSign();
	}
	else if(o.side==AOrder.SIDE_BUY_COVER||o.side==AOrder.SIDE_SELL)
	{
	    dir_exit=-o.getSharesSign();
	}
    }


    private java.io.PrintWriter ticks_dump_trading_diag_data=null;
    public void setTicker()
    {
	String fname=null;
	try{
	    fname=DIR_DIAG+
		com.polytechnik.outils.EncodeDecode.urlencode(sdv.ticker)+
		".diag.trading";
	    this.ticks_dump_trading_diag_data=new java.io.PrintWriter(fname);
	} catch(final java.io.IOException e) {
	    throw new RuntimeException(e);
	}
	final StringBuilder buf=new StringBuilder();
	final String h=
	    "#ticker\tt00_data\tp_buy\tp_sell\tp_last\tvol_p_buy\tvol_p_sell\t"+
	    "getDS\tgetDWenter\tgetDWexit\t"+
	    //"sa0\tsaL\tsaH\t"+
	    //"I0\tIE\t"+
	    "tau\tdir_opportunity\tdir_exit\torder_id\t";

	// header
	final String [] hs=com.polytechnik.utils.ReadWriteUtils.extractnames(h,"\t");
	System.err.println("column\tfield for "+this.sdv.ticker+
			   " in diagfile="+fname);
	int n=1;
	for(String s:hs)
	{
	    System.err.print(n);
	    System.err.print("\t");
	    System.err.println(s);
	    this.ticks_dump_trading_diag_data.print(s);
	    this.ticks_dump_trading_diag_data.print("("+n+")\t");
	    n++;
	}
	this.ticks_dump_trading_diag_data.println();
    }


/* To match k.txt:

perl -w /home/polytechnik/new_way/com/polytechnik/scripts/plot_chart.pl /pdata/res/test_ca_tau_tdb/S080610/sp_3na/AAPL.data \|aTicker t00_data  P_buy P_sell P_last LF15_tip_{p_offset,diffw_dirv,dirv,sigma,ddirp,dp,dirp,p_close}_16000
Converting /pdata/res/test_ca_tau_tdb/S080610/sp_3na/AAPL.data to /tmp/k.dat
column	fieldname
1	|aTicker
2	t00_data
3	P_buy
4	P_sell
5	P_last
6	LF15_tip_p_offset_16000
7	LF15_tip_diffw_dirv_16000
8	LF15_tip_dirv_16000
9	LF15_tip_sigma_16000
10	LF15_tip_ddirp_16000
11	LF15_tip_dp_16000
12	LF15_tip_dirp_16000
13	LF15_tip_p_close_16000
conversion to /tmp/k.dat DONE 13 fields of 1934 read 22076 lines OK

*/
    public void dumpTickData(final CalculationDataRef c)
    {
      if(this.cdr!=c)
      {
	  throw new IllegalStateException("Race is somewhere");
      }

      if(0!=(this.trader.dump_trading_diagnostics & DumpTradingDataAnalyzable.DUMP_TICKS))
      {
	  this.ticks_dump_trading_diag_data.println(
	      sdv.ticker+"\t"+
	      c.getTime()+"\t"+
	      c.p_buy+"\t"+
	      c.p_sell+"\t"+
	      c.p_last+"\t"+
	      c.vol_p_buy+"\t"+
	      c.vol_p_sell+"\t"+
	      getDS()+"\t"+
	      getDWenter()+"\t"+
	      getDWexit()+"\t"+
	      //op_data.sa0+"\t"+
	      //op_data.saL+"\t"+
	      //op_data.saH+"\t"+
	      //op_data.I0+"\t"+
	      //op_data.IE+"\t"+
	      eprev[0].tau+"\t"+

	      // posted orders information
	      (this.dir_opportunity*0.5)+"\t"+
	      (this.dir_exit*0.3)+"\t"+
	      (order_id)+"\t");
      }
    }


    private void saveOrdersAndTrades(final String txt,final long t,final double p_buy,final double p_sell) throws java.io.IOException
    {
	synchronized(this.sdv.positions)
        {
	    final String fprefix=DIR_DIAG+
		com.polytechnik.outils.EncodeDecode.urlencode(sdv.ticker);
	    final String fname_trades=fprefix+".trades.csv",
		fname_orders=fprefix+".orders.csv";
	    final java.io.PrintWriter otrades=new java.io.PrintWriter(fname_trades);
	    otrades.println("DESCRIPTION\t"+txt);
	    otrades.println("VERSION\t"+"$Id: TradingAlgorithmByEvents.java,v 1.822 2020/02/06 01:28:24 mal Exp $");
	    otrades.println("SOURCE\t"+this.trader.description);
	    this.sdv.positions.dumpTradesCSV(otrades,t,p_buy,p_sell);
	    otrades.println("VERSION\t"+"$Id: TradingAlgorithmByEvents.java,v 1.822 2020/02/06 01:28:24 mal Exp $");
	    otrades.close();
	    final java.io.PrintWriter oorders=new java.io.PrintWriter(fname_orders);
	    oorders.println("DESCRIPTION\t"+txt);
	    oorders.println("VERSION\t"+"$Id: TradingAlgorithmByEvents.java,v 1.822 2020/02/06 01:28:24 mal Exp $");
	    oorders.println("SOURCE\t"+this.trader.description);
	    this.sdv.positions.dumpOrdersCSV(oorders,t,p_buy,p_sell);
	    oorders.println("VERSION\t"+"$Id: TradingAlgorithmByEvents.java,v 1.822 2020/02/06 01:28:24 mal Exp $");
	    oorders.close();
	}
    }


    @Override
    protected void testing_saveOrdersAndTrades(final String txt,final Positions.PositionsInfo pinfo)
    {
	try {
	    if(0!=(this.trader.dump_trading_diagnostics & DumpTradingDataAnalyzable.DUMP_POSTIONS_TIME_END))
	    {
		saveOrdersAndTrades(txt,pinfo.time,pinfo.p_buy,pinfo.p_sell);
	    }
	} catch(final java.io.IOException e) {
	    throw new RuntimeException(e);
	}
    }


    public void closeTicker()
    {
      try {
        if(0!=(this.trader.dump_trading_diagnostics & DumpTradingDataAnalyzable.DUMP_TICKS))
	{
	    this.ticks_dump_trading_diag_data.close();
	    this.ticks_dump_trading_diag_data=null;
	}
	if(0!=(this.trader.dump_trading_diagnostics & DumpTradingDataAnalyzable.DUMP_END_POSTIONS))
	{
	    saveOrdersAndTrades("CLOSE FINAL DUMP",this.sdv.getTraderCurrentTime(),Double.NaN,Double.NaN);
	}
      } catch(final java.io.IOException e) {
	  throw new RuntimeException(e);
      }
    }
// =========================== END DUMP REPORTS & DIAGNOSTICS =========================


}


