package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2009
 * This file is under GPL version 3.
 *
 */

/** A simple final state machine to follow a trend.
 *  @version $Id: TrendFollower.java,v 1.13 2009/12/08 21:24:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
abstract class TrendFollower {
    double prev_min=Double.POSITIVE_INFINITY,prev_max=Double.NEGATIVE_INFINITY,
	min=Double.POSITIVE_INFINITY,max=Double.NEGATIVE_INFINITY,last=0;
    long t_switch=-1,t_min=-1,t_max=-1,t_last=-1;

    static final int STATE_UP=665,STATE_DOWN=666;
    private int state=STATE_DOWN;
    String description=null;

    final double NOICE_LEVEL;
    TrendFollower(final double NOICE_LEVEL)
    {
	this.NOICE_LEVEL=NOICE_LEVEL;
    }


    void processEvent(final double current,final long current_time)
    {
	final int new_state;
	if(current<-NOICE_LEVEL)
	{
	    new_state=STATE_DOWN;
	}
	else if(current>NOICE_LEVEL)
	{
	    new_state=STATE_UP;
	}
	else
	{
	    new_state=state;
	}


	if(new_state!=this.state)
	{
	    stateChanged(new_state,current,current_time);
	    this.prev_min=this.min;
	    this.prev_max=this.max;
	    this.min=this.max=current;
	    this.t_switch=current_time;
	}
	else
	{
	    sameState(new_state,current,current_time);
	}

	final String s1="LAST STATE: state="+this.state+
	    " min="+this.min+" max="+this.max;
	final String s2=(this.state!=new_state?"NEW STATE=":"OLD STATE=")+
	    new_state;

	this.state=new_state;
	this.last=current;
	this.t_last=current_time;
	if(current<=this.min)
	{
	    this.min=current;
	    this.t_min=current_time;
	}
	if(current>=this.max)
	{
	    this.max=current;
	    this.t_max=current_time;
	}
	this.description=s2+" current="+current+" "+s1;
    }



    abstract void stateChanged(final int new_state,
			       final double current,
			       final long current_time);

    abstract void sameState(final int state,
			    final double current,
			    final long current_time);


    int getTrendSign()
    {
	if(state==STATE_UP)
	{
	    return 1;
	}
	else if(state==STATE_DOWN)
	{
	    return -1;
	}
	else
	{
	    throw new IllegalStateException();
	}
    }
}

