package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: TstPnLILowIHighMonomials.java,v 1.12 2022/07/28 07:29:40 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class TstPnLILowIHighMonomials extends TstPnLILowIHigh<QVMData> {
    TstPnLILowIHighMonomials(final int neq,final QVMData odata,final TstPnLILowIHighStateSaverWithWI rst){super(neq,odata,rst);}


public static TstPnLILowIHighMonomials calculate(final int neq,
					  final long currenttime,
					  final long tau,
					  final Trade [] tb,
					  final int tsize,
					  final TstPnLILowIHighMonomials ref)
{

    final int nvm=1,npm=4,nmoms=2*neq-1;
    final QVMData o=QVMData.calculateQVMData(nvm,nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
					     ref!=null?ref.odata:null);

    return new TstPnLILowIHighMonomials(neq,o,ref!=null?ref.sst:null);
}
@Override QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
@Override OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
@Override double [] getPsi2WIntegratedDt(final double [] psi){return WIntegratorMonomials.getPsi2WIntegratedDt(psi);}
@Override WIntegratorMonomials getEmptyWIntegrator(int dim){return new WIntegratorMonomials(odata.ntm,odata.tau,dim);}
@Override double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}


static class TstPnLILowIHighMonomialsTest extends PnLILowIHighTest<QVMData,TstPnLILowIHighMonomials> {
TstPnLILowIHighMonomialsTest(final long initrandom)
{
    super("TstPnLILowIHighMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}



@Override
void test_Extra(){
    // These to be identical
    final TstPnLILowIHighMonomials qm=getTstPnLILowIHigh();
    final TstPnLILowIHighLaguerre ql=getTstPnLILowIHighLaguerre();

    TstPnLILowIHigh.matchTstPnLILowIHigh(qm,ql,eps);
    TstPnLILowIHigh.matchTPV(ql,getTPV(),eps);

    System.err.println(name+": TstPnLILowIHighMonomials:test_Extra(): test TstPnLILowIHighLaguerre OK neq="+neq+" eps="+eps);
}


TstPnLILowIHighMonomials getTstPnLILowIHighWithThisRef(final int tsize_to,final long currenttime,final TstPnLILowIHighMonomials ref){return calculate(neq,currenttime,tau,tb,tsize_to,ref);}

PnLILowIHighTPVFactoryMonomials getTPVFactory(){return new PnLILowIHighTPVFactoryMonomials(neq,tau);}

TstPnLILowIHighLaguerre getTstPnLILowIHighLaguerre(){
    final TstPnLILowIHighLaguerre.TstPnLILowIHighLaguerreTest l=
	new TstPnLILowIHighLaguerre.TstPnLILowIHighLaguerreTest(-1);
    l.neq=neq;
    l.currenttime=currenttime;
    l.tau=tau;
    l.tb=tb;
    return l.getTstPnLILowIHigh();
}

} // END of class TstPnLILowIHighMonomialsTest

public static void main(String [] args)
{
    new TstPnLILowIHighMonomialsTest(434L).runTests(2,4);
} // END main

}
