package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2020
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.EVXData;
import com.polytechnik.freemoney.WIntegrator;

/** Save the state, WIntegrator is added for convenience of having the state of
 * {@link TstPnLILowIHigh TstPnLILowIHigh} class stored in a single class,
 * not split on Low/High I monitoring in
 * {@link TstPnLILowIHighStateSaver TstPnLILowIHighStateSaver}
 * and recurrent integrator {@link WIntegrator WIntegrator}.
 * @version $Id: TstPnLILowIHighStateSaverWithWI.java,v 1.23 2022/07/10 10:59:04 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class TstPnLILowIHighStateSaverWithWI extends EnterILowExitIHighStateSaver {


    /** Recurrent integrator.
     */
    final WIntegrator wi;

    /** Initial state.
     */
    public TstPnLILowIHighStateSaverWithWI(final WIntegrator wi){
	super();
	this.wi=wi;if(wi.nobservables!=5/* the number of time sequences to save */) throw new IllegalArgumentException();
    }

    /** Recurrently updated state.
     */
    public TstPnLILowIHighStateSaverWithWI(final TstPnLILowIHighStateSaverWithWI prev,
					final long time,final double P,final long dV,
					final EVXData I)
    {
	super(prev,time,P,dV,I);
	this.wi=prev.wi;
    }


    void addObservation(final long currenttime,/*final long t,*/final double dt,final double dv,final double PriceChangePnL,final double AbsPriceChangePnL,final double dIHpnl){
	final long t=currenttime;
	// arguments, the order: dt:[0], dv:[1], dIHpnl[2], PriceChangePnL*dIHpnl[3], Abs(PriceChangePnL)*dIHpnl[4]
	wi.addSingleObservation(currenttime,t,new double []{dt/*[0]*/,dv/*[1]*/,dIHpnl/*[2]*/,PriceChangePnL/*[3]*/,AbsPriceChangePnL/*[4]*/});
    }


    public double getSumPcDIH(){return wi.sum[3/* PriceChangePnL*dIHpnl */];}
    public double getSumDv(){return wi.sum[1/* dv */];}

    double [] getDtMoments(){return wi.integration_moments[0/* dt */];}
    double [] getDvMoments(){return wi.integration_moments[1/* dv */];}
    double [] getDIHpnlMoments(){return wi.integration_moments[2/* dIHpnl */];}
    double [] getPcDIHpnlMoments(){return wi.integration_moments[3/* PriceChangePnL*dIHpnl */];}
    double [] getAbsPcDIHpnlMoments(){return wi.integration_moments[4/* Abs(PriceChangePnL)*dIHpnl */];}

} // END OF class TstPnLILowIHighStateSaverWithWI
