package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */


import com.polytechnik.outils.BissectionRootFinder;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.outils.*;

/** Uses Two Roots finder.
 *  @version $Id: TwoRootsIFD.java,v 1.8 2020/02/06 10:45:31 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
abstract class TwoRootsIFD {
    final DeprecatedHelperDataDirectAccessWithBasis<?> access;
    final double [] kkt,kkf;
    final TwoCloseRootsFinder.RootsData r;
    final double xD_1,xD_2,fD_1,fD_2;

    TwoRootsIFD(final double [] kkt,final double [] kkf,final DeprecatedHelperDataDirectAccessWithBasis<?> access)
    {
	this.kkt=kkt;
	this.kkf=kkf;
	this.access=access;
	this.r=getTRD0(kkt,kkf);
	this.xD_2=(r.x2_type==r.TYPE_UNKNOWN||r.x1_type==r.TYPE_UNKNOWN)?Double.NaN:new IFDSample().doBissection(r.x1,r.x2);
	this.fD_2=r.D.getDF(xD_2);

	this.xD_1=(r.x1_type==r.TYPE_UNKNOWN)?Double.NaN:(Math.signum(r.d2F0)==Math.signum(r.d2F1)?r.x0:new IFDSample().doBissection(r.x0,r.x1));
	this.fD_1=r.D.getDF(xD_1);

    }

    abstract TwoCloseRootsFinder.RootsData getTRD0(final double [] kkt,final double [] kkf);

    static void test_F(final TwoRootsIFD q,final double eps)
    {
	if(q.r.x1_type!=q.r.TYPE_UNKNOWN)
	{
	    UnitTests.checkDoubleRel(q.access.fx(q.kkf,q.r.x1)/q.access.fx(q.kkt,q.r.x1),"kkf/kkt",0,"0",eps);
	    if(q.xD_1!=q.r.x0) UnitTests.checkDoubleRel(q.r.D.getD2F(q.xD_1),"D2F(xD_1)",0,"0",eps);
	}
	if(q.r.x2_type!=q.r.TYPE_UNKNOWN)
	{
	    UnitTests.checkDoubleRel(q.access.fx(q.kkf,q.r.x2)/q.access.fx(q.kkt,q.r.x2),"kkf/kkt",0,"0",eps);
	    UnitTests.checkDoubleRel(q.r.D.getD2F(q.xD_2),"D2F(xD_2)",0,"0",eps);
	}
	System.err.println("TwoRootsIFD:test_F: x test OK x0="+q.r.x0+" x1="+q.r.x1+" xD_1="+q.xD_1+
			   " x2="+q.r.x2+" xD_2="+q.xD_2+" fD_1="+q.fD_1+" fD_2="+q.fD_2+", eps="+eps);
    }

    class IFDSample extends BissectionRootFinder {
	public double getF(final double x)
	{
	    return r.D.getD2F(x);
	}
    }
}
