package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */


import com.polytechnik.utils.UnitTests;
import com.polytechnik.outils.*;

/** Uses Two Roots finder.
 *  @version $Id: TwoRootsIFX.java,v 1.7 2020/02/06 00:09:23 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
abstract class TwoRootsIFX {
    final DeprecatedHelperDataDirectAccessWithBasis<?> access;
    final double [] kkt,kkf;
    final TwoCloseRootsFinder.RootsData r;
    final double It_0,It_1,It_2,If_0,If_1,If_2;
    final IFXSample s1,s2;
    static final int NSAMPLE=1000;

    TwoRootsIFX(final double [] kkt,final double [] kkf,final DeprecatedHelperDataDirectAccessWithBasis<?> access)
    {
	this.kkt=kkt;
	this.kkf=kkf;
	this.access=access;
	final double [] It=integrateP(kkt),If=integrateP(kkf);
	this.r=getTRD0(kkt,kkf);
	this.It_0=access.fx(It,r.x0);
	this.It_1=access.fx(It,r.x1);
	this.It_2=access.fx(It,r.x2);
	this.If_0=access.fx(If,r.x0);
	this.If_1=access.fx(If,r.x1);
	this.If_2=access.fx(If,r.x2);

	this.s1=new IFXSample(NSAMPLE,r.x1,r.x0);
	this.s2=new IFXSample(NSAMPLE,r.x2,r.x1);
    }

    abstract TwoCloseRootsFinder.RootsData getTRD0(final double [] kkt,final double [] kkf);
    abstract double [] integrateP(final double [] x);

    static void test_F(final TwoRootsIFX q,final double eps)
    {
	if(q.r.x1_type!=q.r.TYPE_UNKNOWN)
	{
	    UnitTests.checkDoubleRel(q.access.fx(q.kkf,q.r.x1)/q.access.fx(q.kkt,q.r.x1),"kkf/kkt",0,"0",eps);
	    UnitTests.checkDoubleRel(q.s1.getF(q.r.x1),"kkf/kkt",0,"0",eps);
	}
	if(q.r.x2_type!=q.r.TYPE_UNKNOWN)
	{
	    UnitTests.checkDoubleRel(q.access.fx(q.kkf,q.r.x2)/q.access.fx(q.kkt,q.r.x2),"kkf/kkt",0,"0",eps);
	    UnitTests.checkDoubleRel(q.s2.getF(q.r.x2),"kkf/kkt",0,"0",eps);
	}
	System.err.println("TwoRootsIFX:test_F: x test OK x1="+q.r.x1+" x2="+q.r.x2+", eps="+eps);
    }

    class IFXSample extends SamplingMinMax {
	IFXSample(final int nt,final double xl,final double xr)
	{
	    super(nt,xl,xr);
	}
	public double getF(final double x)
	{
	    return access.fx(kkf,x)/access.fx(kkt,x);
	}
    }
}
