package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.EVXData;
/**
 * Two top levels of dV/dt.
 * @version $Id: TwoTopI.java,v 1.1 2022/09/12 22:37:01 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
*/
public class TwoTopI {
    public final EVXData I;

    public final double I_IlH,pv_IlH,Tv_IlH,V_IlH,dp_IlH;
    public final double I_IlL,pv_IlL,Tv_IlL,V_IlL,dp_IlL;
    public final double pv_sel,Tv_sel,V_sel,dp_sel;

    TwoTopI(final EVXData I,
	    final double p_offset,
	    final double p_last,
	    final double [] QQTi,
	    final double [] QQpi,
	    final double [] QQV,
	    final double [] QQdp
	    ){
	this.I=I;
	final int n=I.neq;
	final int ilH=n-1,ilL=n>=2?n-2:0;
	I_IlH=I.ev[ilH];
	pv_IlH=p_offset+I.getFPsi(ilH,QQpi)/I_IlH;
	Tv_IlH=I.getFPsi(ilH,QQTi)/I_IlH;
	V_IlH=I.getFPsi(ilH,QQV);
	dp_IlH=I.getFPsi(ilH,QQdp);

	I_IlL=I.ev[ilL];
	pv_IlL=p_offset+I.getFPsi(ilL,QQpi)/I_IlL;
	Tv_IlL=I.getFPsi(ilL,QQTi)/I_IlL;
	V_IlL=I.getFPsi(ilL,QQV);
	dp_IlL=I.getFPsi(ilL,QQdp);

	if(Tv_IlH<Tv_IlL){
	    pv_sel=pv_IlH;
	    Tv_sel=Tv_IlH;
	    V_sel=V_IlH;
	    dp_sel=dp_IlH;
	} else {
	    pv_sel=pv_IlL;
	    Tv_sel=Tv_IlL;
	    V_sel=V_IlL;
	    dp_sel=dp_IlL;
	}
    }
}
