package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.realtimebook.*;

/** Liquidity deficit with hedging.
 *  Not working. Need directional.
 *  @version $Id: UMlQVM.java,v 1.29 2020/09/27 20:33:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class UMlQVM<DTYPE> {
public static final String VERSION="$Id: UMlQVM.java,v 1.29 2020/09/27 20:33:21 mal Exp $";
final DTYPE [] idata;
final DTYPE odata;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=getABasis();;
final DataInterfaceToMoments<DTYPE> access=getAccess();

final double lam_0,lam_L,lam_H,Ir_H,si_L,si_H;
final StateWithHedge st;

UMlQVM(final int n,final DTYPE odata,final DTYPE [] idata)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.idata=idata;
    this.odata=odata;

    final QQHedges QQH=getQQHedgeMatrs(n);
    final double [][] QQ=getQQMatrs(n);
    final double [] QQc=QQ[0],QQl=QQ[1],QQi=QQ[2],QQt=QQ[3];

    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQc,QQH.QQc,M);
    final int i0=0,iL=1,iH=n-1;
    this.lam_0=basis[i0];
    this.lam_L=basis[iL];
    this.lam_H=basis[iH];

    final double [] psiH=EVXData.getPsiFromBasis(iH,basis,n),
	psiL=EVXData.getPsiFromBasis(iL,basis,n);
    si_L=TMatr.getAMatrA(psiL,QQl,n)-lam_L*TMatr.getAMatrA(psiL,QQH.QQl,n);
    si_H=TMatr.getAMatrA(psiH,QQl,n)-lam_H*TMatr.getAMatrA(psiH,QQH.QQl,n);
    //System.err.println("si_L="+si_L+" si_H="+si_H);

    this.Ir_H=TMatr.getAMatrA(psiH,QQi,n);
    this.st=new StateWithHedge(Ir_H,-lam_H,QQH.getIHedges(psiH),getPLast(odata),getLastPHedges());
    //System.err.println("st.getPosCOrig()="+st.getPosCOrig()+" si_H/Ir_H="+(si_H/Ir_H)+" have to match withing +-  sign");
    //System.err.println("lam_0="+lam_0+" lam_L="+lam_L+" lam_H="+lam_H);
}

double [] getLastPHedges()
{
    final double [] pH_last=new double [idata.length];
    for(int k=0;k<idata.length;k++)
    {
	pH_last[k]=getPLast(idata[k]);
    }
    return pH_last;
}

static class QQHedges {
    final double [] QQc,QQl;
    final double [][] QQiHedges;
    final int dim;

    QQHedges(final double [] QQc,final double [] QQl,
	     final double [][] QQiHedges,final int dim)
    {
	this.QQc=QQc;
	this.QQl=QQl;
	this.QQiHedges=QQiHedges;
	this.dim=dim;
    }
    double [] getIHedges(final double [] psi)
    {
	final double [] i=new double [QQiHedges.length];
	for(int k=0;k<i.length;k++)
	{
	    i[k]=TMatr.getAMatrA(psi,QQiHedges[k],dim);
	}
	return i;
    }
}

/** Calculates C-moments sum pI and Last-moments sum (p-p_l)I.
 */
QQHedges getQQHedgeMatrs(final int n)
{
    final int nmoms=2*n-1;
    final double [] momc=new double [nmoms],moml=new double [nmoms],
	mom0=new double [nmoms],mom1=new double [nmoms];
    final double [][] QQiHedges=new double [idata.length][];

    for(int k=0;k<idata.length;k++)
    {
	final double po=getPOffset(idata[k]),pl=getPLast(idata[k]),dpo=po-pl;
	access.setM(mom0,0,access.M_I,idata[k],nmoms);
	access.setM(mom1,1,access.M_I,idata[k],nmoms);
	QQiHedges[k]=M.B.getQQMatr(n,mom0);
	for(int l=0;l<nmoms;l++)
	{
	    momc[l]+=mom1[l]+po*mom0[l];
	    moml[l]+=mom1[l]+dpo*mom0[l];
	}
	//System.err.println("QQH["+k+"]: mom0[0]="+mom0[0]);
    }
    return new QQHedges(M.B.getQQMatr(n,momc),M.B.getQQMatr(n,moml),QQiHedges,n);
}

/** Calculates C-moments pI, Last-moments (p-p_l)I and I.
 */
double [][] getQQMatrs(final int n)
{
    final int nmoms=2*n-1;
    final double [] momc=new double [nmoms],moml=new double [nmoms],
	mom0=new double [nmoms],mom1=new double [nmoms];
    final double po=getPOffset(odata),pl=getPLast(odata),dpo=po-pl;
    access.setM(mom0,0,access.M_I,odata,nmoms);
    access.setM(mom1,1,access.M_I,odata,nmoms);
    for(int l=0;l<nmoms;l++)
    {
	momc[l]+=mom1[l]+po*mom0[l];
	moml[l]+=mom1[l]+dpo*mom0[l];
    }
    //System.err.println("QQ: momc[0]="+momc[0]+" mom0[0]="+mom0[0]);
    access.setMT(mom1,nmoms);
    return new double [][]{M.B.getQQMatr(n,momc),M.B.getQQMatr(n,moml),M.B.getQQMatr(n,mom0),M.B.getQQMatr(n,mom1)};
}


abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getPLast(final DTYPE o);
abstract double getPOffset(final DTYPE o);

static abstract class UMlQVMTest<DTYPE> extends QTest {

abstract UMlQVM<DTYPE> getUMlQVM();


UMlQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}




void test_Common()
{
    final UMlQVM<DTYPE> q=getUMlQVM();
    UnitTests.checkDoubleRel(Math.abs(q.si_H/q.Ir_H),"|q.si_H/q.Ir_H|",q.st.getPosCOrig(),"|q.st.getPosCOrig()|",eps);
    UnitTests.checkDoubleRel(q.st.posA,"q.st.posA",-Math.signum(q.si_H),"-sign(q.si_H)",eps);
    System.err.println(name+":UMlQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}

void test_addSame()
{
    System.err.println("ORIG:tdata.length="+tdata.length);
    final UMlQVM<DTYPE> q=getUMlQVM();
    for(int j=1;j<=3;j++)
    {
    // !!!! THIS CALL CLOBBER THE DATA !!!!
    addTbToData();
    final double fact=j;
    System.err.println("AFTER addTbToData:tdata.length="+tdata.length+" fact="+fact);
    final UMlQVM<DTYPE> s=getUMlQVM();
    System.err.println("s.lam_L="+s.lam_L+" q.lam_L="+q.lam_L);
    System.err.println("s.si_L="+s.si_L+" q.si_L="+q.si_L);
    System.err.println("s.lam_H="+s.lam_H+" q.lam_H="+q.lam_H);
    System.err.println("s.si_H="+s.si_H+" q.si_H="+q.si_H);
    UnitTests.checkDoubleRel(s.si_L,"s.si_L",q.si_L/Math.pow(1+fact*q.lam_L,2),"q.si_L/(1+fact*q.lam_L)^2",eps);
    UnitTests.checkDoubleRel(s.si_H,"s.si_H",q.si_H/Math.pow(1+fact*q.lam_H,2),"q.si_H/(1+fact*q.lam_H)^2",eps);
    UnitTests.checkDoubleRel(s.lam_L,"s.lam_L",q.lam_L/(1+fact*q.lam_L),"q.lam_L/(1+fact*q.lam_L)",eps);
    UnitTests.checkDoubleRel(s.lam_H,"s.lam_H",q.lam_H/(1+fact*q.lam_H),"q.lam_H/(1+fact*q.lam_H)",eps);
    }

    System.err.println(name+":UMlQVMTest:test_addSame(): test OK neq="+neq+" eps="+eps);
}


void test_Extra(){}

Trade [][] tdata;
int [] tdata_size;
void setDataS(final int nstocks,final int tsize,final int neq)
{
    this.tdata=new Trade [nstocks][];
    long ct=Long.MIN_VALUE;
    for(int i=0;i<nstocks;i++)
    {
	setData(tsize+r.nextInt(tsize)/7,neq); // sets tau as a sideeffect
	this.tdata[i]=tb;
	final long t=tb[tb.length-1].time;
	//System.err.println("t="+t);
	if(t>=ct){ct=t;}
	tb=null;
    }
    setData(tsize,neq);
    final long t=tb[tb.length-1].time;
    if(t>=ct){ct=t;}

    currenttime=ct+330; // maximal last time plus some small delay
    // tau has the value of latest generated tb
    tdata_size=new int [tdata.length];
    for(int k=0;k<tdata.length;k++)
    {
	tdata_size[k]=tdata[k].length;
    }
}
void addTbToData()
{
    final Trade [][] tdata1=new Trade [1+tdata.length][];
    System.arraycopy(tdata,0,tdata1,0,tdata.length);
    tdata1[tdata1.length-1]=tb;
    final int [] tdata_size1=new int [1+tdata_size.length];
    System.arraycopy(tdata_size,0,tdata_size1,0,tdata_size.length);
    tdata_size1[tdata_size1.length-1]=tb.length;
    tdata=tdata1;
    tdata_size=tdata_size1;
}


void test_Calculations(final int nstocks,final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":UMlQVMTest:test_Calculations(): test with N points="+tsize+
		       " nstocks="+nstocks+" neq="+neq+" eps="+eps);

    setDataS(nstocks,tsize,neq);
    test_Common();
    test_Extra();

    // the last one, clobber the testing data
    test_addSame();

    System.err.println(name+":UMlQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":UMlQVMTest: starting tests; ver=$Id: UMlQVM.java,v 1.29 2020/09/27 20:33:21 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int nstocks=1;nstocks<=5;nstocks++)
	{
	    for(int i=0;i<119;i++)
	    {
		test_Calculations(nstocks,1950+r.nextInt(1850),neq);
	    }
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":UMlQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: UMlQVM.java,v 1.29 2020/09/27 20:33:21 mal Exp $");
}
} // END OF class UMlQVMTest




}// END OF class UMlQVM
