package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: UMlQVMLaguerre.java,v 1.7 2022/07/10 10:41:23 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class UMlQVMLaguerre extends UMlQVM<QVMDataL> {
    UMlQVMLaguerre(final int neq,final QVMDataL odata,final QVMDataL [] idata){super(neq,odata,idata);}

public static UMlQVMLaguerre calculate(final int neq,
					      final long currenttime,
					      final long tau,
					      final Trade [] tb,
					      final int tsize,
					      final Trade [][] itb,
					      final int [] itbsize,
					      final UMlQVMLaguerre ref)
{
    final int npm=2,nmoms=2*neq-1;
    final QVMDataL o=QVMDataL.calculateQVMDataL(nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    final QVMDataL [] i=new QVMDataL [itb.length];
    for(int k=0;k<i.length;k++)
    {
	i[k]=QVMDataL.calculateQVMDataL(nmoms,
					npm,
					currenttime,
					tau,
					itb[k],
					itbsize[k],
					ref!=null?ref.idata[k]:null);
    }

    return new UMlQVMLaguerre(neq,o,i);
}


QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getPOffset(final QVMDataL odata){return odata.p_offset;}
double getPLast(final QVMDataL odata){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}

static class UMlQVMLaguerreTest extends UMlQVMTest<QVMDataL> {
UMlQVMLaguerreTest(final long initrandom)
{
    super("UMlQVMLaguerre",new java.util.Random(initrandom),1e-5);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

UMlQVMLaguerre getUMlQVM(){return calculate(neq,currenttime,tau,tb,tb.length,tdata,tdata_size,null);}

} // END of class UMlQVMLaguerreTest

public static void main(String [] args)
{
    new UMlQVMLaguerreTest(6221287L).runTests(5);
} // END main



} // END OF class UMlQVMLaguerre
