package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: UMlQVMMonomials.java,v 1.4 2015/12/24 14:02:48 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class UMlQVMMonomials extends UMlQVM<QVMData> {
    UMlQVMMonomials(final int neq,final QVMData odata,final QVMData [] idata){super(neq,odata,idata);}

public static UMlQVMMonomials calculate(final int neq,
					      final long currenttime,
					      final long tau,
					      final Trade [] tb,
					      final int tsize,
					      final Trade [][] itb,
					      final int [] itbsize,
					      final UMlQVMMonomials ref)
{
    final int nvm=1,npm=2,nmoms=2*neq-1;
    final QVMData o=QVMData.calculateQVMData(nvm,nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    final QVMData [] i=new QVMData [itb.length];
    for(int k=0;k<i.length;k++)
    {
	i[k]=QVMData.calculateQVMData(nvm,nmoms,
					npm,
					currenttime,
					tau,
					itb[k],
					itbsize[k],
					ref!=null?ref.idata[k]:null);
    }

    return new UMlQVMMonomials(neq,o,i);
}


QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis();}
double getPOffset(final QVMData odata){return odata.p_offset;}
double getPLast(final QVMData odata){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}

static class UMlQVMMonomialsTest extends UMlQVMTest<QVMData> {
UMlQVMMonomialsTest(final long initrandom)
{
    super("UMlQVMMonomials",new java.util.Random(initrandom),1e-5);
}
double [] convertToMonomials(final double [] p){return p;}
long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

@Override
void test_Extra(){
    final UMlQVMMonomials q=getUMlQVM();
    final UMlQVMLaguerre s=getUMlQVMLaguerre();
    UnitTests.checkDoubleRel(q.si_L,"s.siL",q.si_L,"s.si_L",eps);
    UnitTests.checkDoubleRel(q.si_H,"s.si_H",q.si_H,"s.si_H",eps);
    System.err.println(name+":UMlQVMMonomialsTest:test_Extra(): test UMnQVMLaguerre OK neq="+neq+" eps="+eps);
}

UMlQVMMonomials getUMlQVM(){return calculate(neq,currenttime,tau,tb,tb.length,tdata,tdata_size,null);}
UMlQVMLaguerre getUMlQVMLaguerre(){return UMlQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,tdata,tdata_size,null);}

} // END of class UMlQVMMonomialsTest

public static void main(String [] args)
{
    new UMlQVMMonomialsTest(6221287L).runTests(5);
} // END main



} // END OF class UMlQVMMonomials
