package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: UMnQVMMonomials.java,v 1.6 2015/03/02 21:24:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class UMnQVMMonomials extends UMnQVM<QVMData> {
UMnQVMMonomials(final int neq,final QVMData [] idata,final State rst){super(neq,idata,rst);}


public static UMnQVMMonomials calculate(final int neq,
					final long currenttime,
					final long tau,
					final Trade [][] itb,
					final int [] itbsize,
					final UMnQVMMonomials ref)
{
    final int nvm=1,npm=2,nmoms=2*neq-1;
    final QVMData [] o=new QVMData [itb.length];
    for(int k=0;k<itb.length;k++)
    {
	o[k]=QVMData.calculateQVMData(nvm,
				      nmoms,
				      npm,
				      currenttime,
				      tau,
				      itb[k],
				      itbsize[k],
				      ref!=null?ref.idata[k]:null);
    }
    return new UMnQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
public static UMnQVMMonomials calculate1(final int neq,
					 final long currenttime,
					 final long tau,
					 final Trade [] tb,
					 final int tsize,
					 final UMnQVMMonomials ref)
{
    return calculate(neq,currenttime,tau,new Trade [][]{tb},new int []{tsize},ref);
}

QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double getPOffset(final QVMData odata){return odata.p_offset;}
double getPLast(final QVMData odata){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
long getVolume(final QVMData odata){return odata.vol;}
long getTEnd(final QVMData odata){return odata.currenttime;}


static class UMnQVMMonomialsTest extends UMnQVMTest<QVMData> {
UMnQVMMonomialsTest(final long initrandom)
{
    super("UMnQVMMonomials",new java.util.Random(initrandom),1e-5);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

UMnQVMMonomials getUMnQVM(){return calculate(neq,currenttime,tau,tdata,tdata_size,null);}
NkQVMMonomials getNkQVM(){return NkQVMMonomials.calculate(neq,currenttime,tau,tdata,tdata_size,null);}
UMnQVMLaguerre getUMnQVMLaguerre(){return UMnQVMLaguerre.calculate(neq,currenttime,tau,tdata,tdata_size,null);}

@Override
void test_Extra(){
    final UMnQVMMonomials q=getUMnQVM();
    final UMnQVMLaguerre s=getUMnQVMLaguerre();

    UnitTests.checkDoubleRel(q.I_0,"q.I_0",s.I_0,"s.I_0",eps);
    UnitTests.checkDoubleRel(q.I_L,"q.I_L",s.I_L,"s.I_L",eps);
    UnitTests.checkDoubleRel(q.I_H,"q.I_H",s.I_H,"s.I_H",eps);
    UnitTests.checkDoubleRel(q.p_I_H,"q.p_I_H",q.p_I_H,"q.p_I_H",eps);
    UnitTests.checkDoubleRel(Math.abs(q.w_piL_IH),"q.w_piL_IH",Math.abs(s.w_piL_IH),"s.w_piL_IH",eps);
    UnitTests.checkDoubleRel(Math.abs(q.w_piH_IH),"q.w_piH_IH",Math.abs(s.w_piH_IH),"s.w_piH_IH",eps);
    UnitTests.checkDoubleRel(q.r_piL_IH,"q.r_piL_IH",s.r_piL_IH,"s.r_piL_IH",eps);
    UnitTests.checkDoubleRel(q.r_piH_IH,"q.r_piH_IH",s.r_piH_IH,"s.r_piH_IH",eps);

    System.err.println(name+":UMnQVMMonomialsTest:test_Extra(): test UMnQVMLaguerre OK neq="+neq+" eps="+eps);
}



} // END of class UMnQVMMonomialsTest

public static void main(String [] args)
{
    new UMnQVMMonomialsTest(9082900L).runTests(5);
} // END main



} // END OF class UMnQVMMonomials
