package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.realtimebook.*;

/** Multi stock version of UnQVM with individual scale..
 *  @version $Id: UMsQVM.java,v 1.27 2022/06/27 07:19:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class UMsQVM<DTYPE> {

final DTYPE [] idata;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
final State sst;
final double cap_IH,pnl_IH,r,jcap_0,jcap_H,jvarcap_H;

static class State {
    final MMaxStateSaver [] rimF;
    final double [] p_last;
    State(final MMaxStateSaver [] rimF,final double [] p_last)
    {
	this.rimF=rimF;
	this.p_last=p_last;
	if(rimF.length!=p_last.length) throw new IllegalArgumentException("dimension mismatch");
    }
}

UMsQVM(final int n,final DTYPE [] idata,final State rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.idata=idata;
    this.M=getABasis();
    final DataInterfaceToMoments<DTYPE> access=getAccess();

    final int nmoms=2*n-1;
    final double [] tmoms=new double [nmoms],imoms=new double [nmoms],pimoms=new double [nmoms],tmpmoms=new double [nmoms];
    access.setMT(tmoms,nmoms);
    final double [] QQt=M.B.getQQMatr(n,tmoms);

    final int i0=0,iL=1,iH=n-1;
    final MMaxStateSaver [] simF=new MMaxStateSaver [idata.length];
    final double [] p_last=new double [simF.length];
    long tends=0;double cap=0,pnl=0,cap0=0,capH=0,varcapH=0;
    for(int k=0;k<simF.length;k++)
    {
	final DTYPE o=idata[k];
	final long tend=getTEnd(o);
	if(k!=0 && tend!=tends) throw new IllegalArgumentException("tend["+k+"]="+tend+" mistmatch tends="+tends);
	tends=tend;

	p_last[k]=getPLast(o);
	access.setM(imoms,0,access.M_I,o,nmoms);
	access.setM(pimoms,1,access.M_I,o,nmoms);TMatr.sumP1(pimoms,imoms,getPOffset(o)-p_last[k]);

	final double [] QQi=M.B.getQQMatr(n,imoms),QQpi=M.B.getQQMatr(n,pimoms);
	final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
	final double [] psi0=EVXData.getPsiFromBasis(i0,basis,n),
	    psiH=EVXData.getPsiFromBasis(iH,basis,n),varpsiH=getEDPsi(psiH);
	TMatr.scaleVectorInPlace(varpsiH,1/Math.sqrt(TMatr.getAMatrA(varpsiH,QQt,n)));
	//TMatr.sumP1(varpsiH,psi0,-TMatr.getAMatrB(psi0,varpsiH,QQt,n));
	final double I_0=basis[i0],I_L=basis[iL],I_H=basis[iH],
	    pnl_I_H=TMatr.getAMatrA(psiH,QQpi,n),cap_I_H=pnl_I_H+p_last[k]*I_H;
	cap0+=TMatr.getAMatrA(psi0,QQpi,n)+p_last[k]*I_0;
	capH+=cap_I_H;
	varcapH+=TMatr.getAMatrB(psiH,varpsiH,QQpi,n)-pnl_I_H*TMatr.getAMatrB(psiH,varpsiH,QQt,n);
	System.err.println("<psiH|varpsiH>="+TMatr.getAMatrB(psiH,varpsiH,QQt,n)+" must be =0\t <varpsiH|varpsiH>="+TMatr.getAMatrA(varpsiH,QQt,n)+" must be =1 <psiH|QQi|varpsiH>="+TMatr.getAMatrB(psiH,varpsiH,QQi,n)+" <psi0|varpsiH>="+TMatr.getAMatrB(psi0,varpsiH,QQt,n));

	if(true)
	{ // test some novel idea
	    final int nd=n-1;
	    final double [] Rev=M.EV.getGEV_Generalized(nd,M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(imoms)),M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(tmoms)),true);
	    final double R_I_H=Rev[nd-1];
	    final double [] R_psiH=EVXData.getPsiFromBasis(nd-1,Rev,nd),RM_psiH=M.getRadauMultiplied(R_psiH),
		varRM_psiH=getEDPsi(RM_psiH);
	    System.err.println("Test Radau: R_I_H="+R_I_H+" <R_psiH|I|RM_psiH>="+TMatr.getAMatrB(R_psiH,RM_psiH,QQi,n)+" <R_psiH|RM_psiH>="+TMatr.getAMatrB(R_psiH,RM_psiH,QQt,n)+" <varRM_psiH|RM_psiH>="+TMatr.getAMatrB(varRM_psiH,RM_psiH,QQt,n)+" <varRM_psiH|I|RM_psiH>="+TMatr.getAMatrB(varRM_psiH,RM_psiH,QQi,n));
	}

	if(true)
	{
	    access.setDFMoments(tmpmoms,imoms,nmoms);
	    final double [] QQdi=M.B.getQQMatr(n,tmpmoms);
	    final double [] dibasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQdi,QQt,M);
	    if(true)
	    {
		final double eps=1e-4;
		for(double [] b:new double [][]{dibasis,basis})
		for(int j=0;j<n;j++)
		{
		    final double [] psi=EVXData.getPsiFromBasis(j,b,n);
		    final double lt=2*TMatr.getAMatrB(psi,getEDPsi(psi),QQi,n),
			ld=TMatr.getAMatrA(psi,QQdi,n);
		    System.err.println("ld["+j+"]="+ld+" test="+lt);
		    UnitTests.checkDouble(ld,"ld",lt,"lt",eps);
		}
	    }
	}

	if(I_0<I_L){
	    simF[k]=MMaxStateSaver.getTaintedUpdate(rst!=null?rst.rimF[k]:null);
	} else if(I_0>I_H){
	    simF[k]=MMaxStateSaver.getMaxUpdate(I_0,p_last[k],tend,I_0,I_L,I_H,getVolume(o),cap_I_H,pnl_I_H,rst!=null?rst.rimF[k]:null);
	} else {
	    simF[k]=MMaxStateSaver.getNothingUpdate(rst!=null?rst.rimF[k]:null);
	}
	cap+=simF[k].Max.a;
	pnl+=simF[k].Max.b;
    }

    this.cap_IH=cap;
    this.pnl_IH=pnl;
    this.r=pnl/cap;
    this.jcap_0=cap0;
    this.jcap_H=capH;
    this.jvarcap_H=varcapH;

    this.sst=new State(simF,p_last);

}


abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getPLast(final DTYPE o);
abstract double getPOffset(final DTYPE o);
abstract long getTEnd(final DTYPE o);
abstract long getVolume(final DTYPE o);
abstract double [] getEDPsi(final double [] psi);

static abstract class UMsQVMTest<DTYPE> extends QTest {

abstract UMsQVM<DTYPE> getUMsQVM();


UMsQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}




void test_Common()
{
    System.err.println(name+":UMsQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}


void test_Extra(){}
abstract UMnQVM<?> getUMnQVM();
void test_UMnQVM()
{
    final UMsQVM<DTYPE> q=getUMsQVM();
    final UMnQVM<?> s=getUMnQVM();

    UnitTests.checkDoubleRel(q.jcap_0,"q.jcap_0",s.I_0,"s.I_0",eps);

    System.err.println(name+":UMsQVMTest:test_UMnQVM(): test OK neq="+neq+" eps="+eps);
}

abstract UnQVM<?> getUnQVM(final int ind_k);
void test_UnQVM()
{
    final UMsQVM<DTYPE> q=getUMsQVM();
    double sa=0,sb=0;
    for(int k=0;k<tdata.length;k++)
    {
	final UnQVM<?> s=getUnQVM(k);
	sa+=s.sst.rimF.Max.a;
	sb+=s.sst.rimF.Max.b;
    }

    UnitTests.checkDoubleRel(q.cap_IH,"q.cap_IH",sa,"sa",eps);
    UnitTests.checkDoubleRel(q.pnl_IH,"q.pnl_IH",sb,"sb",eps);
    System.err.println(name+":UMsQVMTest:test_UnQVM(): sa="+sa+" sb="+sb);
    System.err.println(name+":UMsQVMTest:test_UnQVM(): test OK neq="+neq+" eps="+eps);
}


Trade [][] tdata;
int [] tdata_size;
void setDataS(final int nstocks,final int tsize,final int neq)
{
    this.tdata=new Trade [nstocks][];
    long ct=Long.MIN_VALUE;
    for(int i=0;i<nstocks;i++)
    {
	setData(tsize+r.nextInt(tsize),neq); // sets tau as a sideeffect
	this.tdata[i]=tb;
	final long t=tb[tb.length-1].time;
	if(t>=ct){ct=t;}
	tb=null;
    }
    currenttime=ct+330; // maximal last time plus some small delay
    // tau has the value of latest generated tb
    tdata_size=new int [tdata.length];
    for(int k=0;k<tdata.length;k++)
    {
	tdata_size[k]=tdata[k].length;
    }
}


void test_Calculations(final int nstocks,final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":UMsQVMTest:test_Calculations(): test with N points="+tsize+
		       " nstocks="+nstocks+" neq="+neq+" eps="+eps);

    setDataS(nstocks,tsize,neq);
    test_Common();
    test_Extra();
    test_UMnQVM();
    test_UnQVM();

    System.err.println(name+":UMsQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":UMsQVMTest: starting tests; ver=$Id: UMsQVM.java,v 1.27 2022/06/27 07:19:16 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int nstocks=1;nstocks<=5;nstocks++)
	{
	    for(int i=0;i<119;i++)
	    {
		test_Calculations(nstocks,1950+r.nextInt(1850),neq);
	    }
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":UMsQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: UMsQVM.java,v 1.27 2022/06/27 07:19:16 mal Exp $");
}
} // END OF class UMsQVMTest




}// END OF class UMsQVM
