package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: UMsQVMLaguerre.java,v 1.10 2022/07/10 10:41:23 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class UMsQVMLaguerre extends UMsQVM<QVMDataL> {
UMsQVMLaguerre(final int neq,final QVMDataL [] idata,final State rst){super(neq,idata,rst);}


public static UMsQVMLaguerre calculate(final int neq,
				       final long currenttime,
				       final long tau,
				       final Trade [][] itb,
				       final int [] itbsize,
				       final UMsQVMLaguerre ref)
{
    final int npm=2,nmoms=2*neq-1;
    final QVMDataL [] o=new QVMDataL [itb.length];
    for(int k=0;k<itb.length;k++)
    {
	o[k]=QVMDataL.calculateQVMDataL(nmoms,
					npm,
					currenttime,
					tau,
					itb[k],
					itbsize[k],
					ref!=null?ref.idata[k]:null);
    }
    return new UMsQVMLaguerre(neq,o,ref!=null?ref.sst:null);
}
public static UMsQVMLaguerre calculate1(final int neq,
					final long currenttime,
					final long tau,
					final Trade [] tb,
					final int tsize,
					final UMsQVMLaguerre ref)
{
    return calculate(neq,currenttime,tau,new Trade [][]{tb},new int []{tsize},ref);
}


QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getPOffset(final QVMDataL odata){return odata.p_offset;}
double getPLast(final QVMDataL odata){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
long getVolume(final QVMDataL odata){return odata.vol;}
long getTEnd(final QVMDataL odata){return odata.currenttime;}
double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}

static class UMsQVMLaguerreTest extends UMsQVMTest<QVMDataL> {
UMsQVMLaguerreTest(final long initrandom)
{
    super("UMsQVMLaguerre",new java.util.Random(initrandom),1e-5);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
UMsQVMLaguerre getUMsQVM(){return calculate(neq,currenttime,tau,tdata,tdata_size,null);}
UMnQVMLaguerre getUMnQVM(){return UMnQVMLaguerre.calculate(neq,currenttime,tau,tdata,tdata_size,null);}
UnQVMLaguerre getUnQVM(final int ind_k){return UnQVMLaguerre.calculate(neq,currenttime,tau,tdata[ind_k],tdata_size[ind_k],null);}

} // END of class UMsQVMLaguerreTest

public static void main(String [] args)
{
    new UMsQVMLaguerreTest(817790L).runTests(5);
} // END main



} // END OF class UMsQVMLaguerre
