package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: UMsQVMMonomials.java,v 1.9 2022/07/10 10:41:23 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class UMsQVMMonomials extends UMsQVM<QVMData> {
UMsQVMMonomials(final int neq,final QVMData [] idata,final State rst){super(neq,idata,rst);}


public static UMsQVMMonomials calculate(final int neq,
					final long currenttime,
					final long tau,
					final Trade [][] itb,
					final int [] itbsize,
					final UMsQVMMonomials ref)
{
    final int nvm=1,npm=2,nmoms=2*neq-1;
    final QVMData [] o=new QVMData [itb.length];
    for(int k=0;k<itb.length;k++)
    {
	o[k]=QVMData.calculateQVMData(nvm,
				      nmoms,
				      npm,
				      currenttime,
				      tau,
				      itb[k],
				      itbsize[k],
				      ref!=null?ref.idata[k]:null);
    }
    return new UMsQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
public static UMsQVMMonomials calculate1(final int neq,
					 final long currenttime,
					 final long tau,
					 final Trade [] tb,
					 final int tsize,
					 final UMsQVMMonomials ref)
{
    return calculate(neq,currenttime,tau,new Trade [][]{tb},new int []{tsize},ref);
}

QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double getPOffset(final QVMData odata){return odata.p_offset;}
double getPLast(final QVMData odata){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
long getVolume(final QVMData odata){return odata.vol;}
long getTEnd(final QVMData odata){return odata.currenttime;}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}

static class UMsQVMMonomialsTest extends UMsQVMTest<QVMData> {
UMsQVMMonomialsTest(final long initrandom)
{
    super("UMsQVMMonomials",new java.util.Random(initrandom),1e-5);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

UMsQVMMonomials getUMsQVM(){return calculate(neq,currenttime,tau,tdata,tdata_size,null);}
UMnQVMMonomials getUMnQVM(){return UMnQVMMonomials.calculate(neq,currenttime,tau,tdata,tdata_size,null);}
UMsQVMLaguerre getUMsQVMLaguerre(){return UMsQVMLaguerre.calculate(neq,currenttime,tau,tdata,tdata_size,null);}
UnQVMMonomials getUnQVM(final int ind_k){return UnQVMMonomials.calculate(neq,currenttime,tau,tdata[ind_k],tdata_size[ind_k],null);}
@Override
void test_Extra(){
    final UMsQVMMonomials q=getUMsQVM();
    final UMsQVMLaguerre s=getUMsQVMLaguerre();

    UnitTests.checkDoubleRel(q.cap_IH,"q.cap_IH",s.cap_IH,"s.cap_IH",eps);
    UnitTests.checkDoubleRel(q.pnl_IH,"q.pnl_IH",s.pnl_IH,"s.pnl_IH",eps);
    UnitTests.checkDoubleRel(q.jcap_0,"q.jcap_0",s.jcap_0,"s.jcap_0",eps);
    UnitTests.checkDoubleRel(q.jcap_H,"q.jcap_H",s.jcap_H,"s.jcap_H",eps);
    UnitTests.checkDoubleRel(q.jvarcap_H,"q.jvarcap_H",s.jvarcap_H,"s.jvarcap_H",eps);

    System.err.println(name+":UMsQVMMonomialsTest:test_Extra(): test UMsQVMLaguerre OK neq="+neq+" eps="+eps);
}



} // END of class UMsQVMMonomialsTest

public static void main(String [] args)
{
    new UMsQVMMonomialsTest(9082900L).runTests(5);
} // END main



} // END OF class UMsQVMMonomials
