package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.realtimebook.*;

/** UQVM and a set of measures.
 *  @version $Id: UQVM.java,v 1.109 2022/08/08 20:10:47 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class UQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final double w00;
final GEVData pv,pt,pvd,pa;
final BEVData bv,bt;
final WQVM.REVData rurik;

final double [] psi0,psiok;
final double I0,Iok,dpI0,dpIok,peq;

final double [] okv,okt,okd;
final double K00v,K00t,K00d,peqv,stdev2_peqv,peqt,stdev2_peqt,peqd,stdev2_peqd;

UQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final int nmoms=2*neq-1;
    final double [] mom=new double [nmoms],pmom=new double [nmoms],p2mom=new double [nmoms];
    access.setM(mom,0,access.M_I,odata,nmoms);
    w00=mom[0];
    access.setM(pmom,1,access.M_I,odata,nmoms);
    access.setM(p2mom,2,access.M_I,odata,nmoms);
    final double [] dpmom=new double [nmoms],dsmom=new double [nmoms];
    access.setM(dsmom,0,access.M_ISELL,odata,nmoms);
    access.setM(dpmom,0,access.M_IBUY,odata,nmoms);
    for(int k=nmoms;--k>=0;) dsmom[k]=dsmom[k]-dpmom[k];
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    final double [] tmom=new double [nmoms],ptmom=new double [nmoms],p2tmom=new double [nmoms];
    access.setM(tmom,0,access.M_T,odata,nmoms);
    access.setM(ptmom,1,access.M_T,odata,nmoms);
    access.setM(p2tmom,2,access.M_T,odata,nmoms);
    final double [] momt=new double [nmoms];momt[0]=w00;
    final double [] amom=new double [nmoms],pamom=new double [nmoms],p2amom=new double [nmoms];
    access.setM(amom,0,access.M_A,odata,nmoms);
    access.setM(pamom,1,access.M_A,odata,nmoms);
    access.setM(p2amom,2,access.M_A,odata,nmoms);

  final double [][] QQ=access.getQQ_Matrs(neq,w00,new double [][]{mom,pmom,p2mom,dpmom,dsmom,tmom,ptmom,p2tmom,momt});
  {
  final double [] QQv=QQ[0],qqdp=QQ[3],QQdp=new double [neq*neq];
  for(int k=qqdp.length;--k>=0;) QQdp[k]=w00*qqdp[k];
  this.rurik=getREVData(neq,QQv,QQdp);
  }

  {
  final double [] QQv=QQ[0],QQpv=QQ[1],QQp2v=QQ[2],QQpt=QQ[6],QQp2t=QQ[7],QQt=QQ[8];
  this.bv=getBEVData(neq,getETH(neq,QQv),getETH_OK(neq),QQt,false,QQpv);
  this.bt=getBEVData(neq,getEVH(neq,QQv),access.getOK(neq,QQv,getX0()),QQt,true,QQpv);
  }
  {
   access.setMT(momt,nmoms);
   final double [][] QQf=access.getQQ_Matrs(neq,1,new double [][]{mom,pmom,p2mom,momt,ptmom,p2tmom,tmom,amom,pamom,p2amom});
   //System.err.println("momt="+java.util.Arrays.toString(momt));
   //System.err.println("momd="+java.util.Arrays.toString(tmom));
   final double [] QQv=QQf[0],QQpv=QQf[1],QQp2v=QQf[2],QQt=QQf[3],QQpt=QQf[4],QQp2t=QQf[5],QQd=QQf[6];
   for(int k=QQv.length;--k>=0;) {QQv[k]/=w00;QQpv[k]/=w00;QQp2v[k]/=w00;}
   pv=getGEVData(neq,getETH(neq,QQv),getETH_OK(neq),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,getX0());
   pt=getGEVData(neq,getEVH(neq,QQv),access.getOK(neq,QQv,getX0()),QQt,QQpt,QQp2t,QQv,QQpv,QQp2v,getX0());
   final double [] evd=solveGeneralizedEV(neq,QQv,QQd);inverseOrderOfEV(neq,evd);
   pvd=getGEVData(neq,evd,access.getOK(neq,QQd,getX0()),QQv,QQpv,QQp2v,QQd,QQpt,QQp2t,getX0());
   final double [] QQa=QQf[7],QQpa=QQf[8],QQp2a=QQf[9];
   pa=getGEVData(neq,solveGeneralizedEV(neq,QQa,QQv),access.getOK(neq,QQv,getX0()),QQa,QQpa,QQp2a,QQv,QQpv,QQp2v,getX0());

   okv=access.getOK(neq,QQv,getX0());
   okt=getETH_OK(neq);
   okd=access.getOK(neq,QQd,getX0());
   K00v=access.fx(okv,getX0());
   K00t=access.fx(okt,getX0());
   K00d=access.fx(okd,getX0());
   final double dpv=TMatr.getAMatrA(neq,okv,QQpv,neq)/K00v,dp2v=TMatr.getAMatrA(neq,okv,QQp2v,neq)/K00v;
   peqv=getPOffset()+dpv;
   stdev2_peqv=dp2v-dpv*dpv;
   final double dpt=TMatr.getAMatrA(neq,okt,QQpt,neq)/K00t,dp2t=TMatr.getAMatrA(neq,okt,QQp2t,neq)/K00t;
   peqt=getPOffset()+dpt;
   stdev2_peqt=dp2t-dpt*dpt;
   final double dpd=TMatr.getAMatrA(neq,okd,QQpt,neq)/K00d,dp2d=TMatr.getAMatrA(neq,okd,QQp2t,neq)/K00d;
   peqd=getPOffset()+dpd;
   stdev2_peqd=dp2d-dpd*dpd;
  }


  {
  final double [] QQv=QQ[0],QQpv=QQ[1],QQp2v=QQ[2];
  final double [] ev=getETH(neq,QQv);
  this.psi0=new double [neq];
  System.arraycopy(ev,(1+0)*neq,this.psi0,0,neq);
  this.I0=ev[0];
  this.psiok=getETH_OK(neq);
  final double K00t=access.fx(psiok,getX0());
  final double norm=Math.sqrt(K00t);
  for(int k=neq;--k>=0;) psiok[k]/=norm; // normalize as Nevai operator
  this.Iok=TMatr.getAMatrA(neq,psiok,QQv,neq);
  final double dp0=TMatr.getAMatrA(neq,psi0,QQpv,neq),dpok=TMatr.getAMatrA(neq,psiok,QQpv,neq);
  this.dpI0=dp0;
  this.dpIok=dpok;
  this.peq=getPOffset()+(dpIok-dpI0)/(Iok-I0);
  }
}


abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double getX0();
abstract double getPOffset();

static class BEVData {
    final double [] psi0,psiok,ev,psiok_over_psi;
    final double K00,I0,IpI,Iok,Iok_0,Iok_o,dP_Iok,dP_I0,dP_Iok_0,dP_Iok_o;
    BEVData(final double [] psi0,
	    final double [] psiok,final double K00,final double [] ev,
	    final double [] psiok_over_psi,
	    final double [] QQpvScal)
    {
	this.psi0=psi0;
	this.psiok=psiok;
	this.K00=K00;
	this.ev=ev;
	this.psiok_over_psi=psiok_over_psi;
	this.I0=ev[0];
	final int n=psiok_over_psi.length;
	double sIok_0=0,sIok_o=0;
	for(int k=n;--k>=0;)
	{
	    final double f=ev[k]*Math.pow(psiok_over_psi[k],2);
	    if(k<=0) sIok_0+=f;
	    else sIok_o+=f;
	}
	this.Iok=(sIok_0+sIok_o);
	this.Iok_0=sIok_0;
	this.Iok_o=sIok_o;
	this.dP_Iok=TMatr.getAMatrB(n,psiok,0,psiok,0,QQpvScal,n);
	this.dP_I0=TMatr.getAMatrB(n,psi0,0,psi0,0,QQpvScal,n);
	this.IpI=psiok_over_psi[0]*(TMatr.getAMatrB(n,psiok,0,psi0,0,QQpvScal,n)-psiok_over_psi[0]*this.dP_I0);
	this.dP_Iok_0=TMatr.getAMatrB(n,psi0,0,psi0,0,QQpvScal,n)*Math.pow(psiok_over_psi[0],2);
	this.dP_Iok_o=this.dP_Iok+this.dP_Iok_0-2*psiok_over_psi[0]*TMatr.getAMatrB(n,psiok,0,psi0,0,QQpvScal,n);
    }
}




static class GEVData {
    final double [] psiok,ev,psiok_over_psi;
    final double [] lam,pL,pR,stdev2_pL,stdev2_pR;
    final double Koo,Iok,pLok,stdev2_pLok,pRok,stdev2_pRok;
    GEVData(final double Koo,
	    final double [] psiok,final double [] ev,
	    final double [] QQl,final double [] QQpl,final double [] QQp2l,
	    final double [] QQr,final double [] QQpr,final double [] QQp2r,
	    final double p_offset)
{
    this.psiok=psiok;
    this.ev=ev;

    final int n=psiok.length;
    this.Koo=Koo;
    this.Iok=TMatr.getAMatrA(n,psiok,QQl,n);
    final double dpLok=TMatr.getAMatrA(n,psiok,QQpl,n)/this.Iok,
	dp2Lok=TMatr.getAMatrA(n,psiok,QQp2l,n)/this.Iok;
    this.pLok=dpLok+p_offset;
    this.stdev2_pLok=dp2Lok-dpLok*dpLok;
    final double dpRok=TMatr.getAMatrA(n,psiok,QQpr,n),
	dp2Rok=TMatr.getAMatrA(n,psiok,QQp2r,n);
    this.pRok=dpRok+p_offset;
    this.stdev2_pRok=dp2Rok-dpRok*dpRok;

    psiok_over_psi=new double [n];
    lam=new double [n];
    pL=new double [n];
    stdev2_pL=new double [n];
    pR=new double [n];
    stdev2_pR=new double [n];
    for(int k=n;--k>=0;)
    {
	psiok_over_psi[k]=TMatr.getAMatrB(n,psiok,0,ev,(k+1)*n,QQr,n);

	final double l=lam[k]=ev[k];
	final double dpl=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQpl,n)/l,
	    dp2l=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQp2l,n)/l;
	pL[k]=dpl+p_offset;
	stdev2_pL[k]=dp2l-dpl*dpl;
	final double dpr=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQpr,n),
	    dp2r=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQp2r,n);
	pR[k]=dpr+p_offset;
	stdev2_pR[k]=dp2r-dpr*dpr;
    }
}
}

static double [] solveGeneralizedEV(final int n,final double [] A,final double [] B)
{
    return DeprecatedHelperEVXData.solveGeneralizedEV(n,A,B);
}

GEVData getGEVData(final int n,final double [] ev,final double [] ok,
		   final double [] QQl,final double [] QQpl,final double [] QQp2l,
		   final double [] QQr,final double [] QQpr,final double [] QQp2r,
		   final double x0)
{

    final double [] psiok=ok.clone();
    final double Koo=access.fx(psiok,x0),norm=Math.sqrt(Koo);
    for(int k=n;--k>=0;) psiok[k]/=norm; // normalize as Nevai

    return new GEVData(Koo,psiok,ev,
	    QQl,QQpl,QQp2l,QQr,QQpr,QQp2r,getPOffset());
}

BEVData getBEVData(final int n,final double [] ev,final double [] ok,
		   final double [] QQscal,final boolean norm_eqev,
		   final double [] QQpv
		   /*,
		   final double [] QQv,final double [] QQpv,final double [] QQp2v*/)
{

    final double I0,p0,stdev2_p0,Iok_bv,pok_bv,stdev2_pok_bv;
    final double [] psi0=new double [n];
    System.arraycopy(ev,(0+1)*n,psi0,0,n);
    /*
    final double dp0=TMatr.getAMatrA(neq,psi0,QQpv,neq)/I0,dp20=TMatr.getAMatrA(neq,psi0,QQp2v,neq)/I0;
    final double I0=ev[0],p0=getPOffset()+dp0,stdev2_p0=dp20-dp0*dp0;
    */

    final double [] psiok=ok;
    final double K00=access.fx(psiok,getX0()),norm=Math.sqrt(K00);
    for(int k=n;--k>=0;) psiok[k]/=norm; // normalize as Nevai operator
    /*
    final double Iok=TMatr.getAMatrA(neq,psiok,QQv,neq),
	dpok=TMatr.getAMatrA(neq,psiok,QQpv,neq)/Iok,dp2ok=TMatr.getAMatrA(neq,psiok,QQp2v,neq)/Iok;
    */

    final double [] expand_psiok_over_psi=new double [n];
    for(int k=0;k<n;k++)
    {
	double f=TMatr.getAMatrB(n,psiok,0,ev,(k+1)*n,QQscal,n);
	if(norm_eqev) f/=ev[k];
	expand_psiok_over_psi[k]=f;
    }

    return new BEVData(psi0,psiok,K00,ev,expand_psiok_over_psi,QQpv);
}


WQVM.REVData getREVData(final int n,final double [] QQv,final double [] QQdp)
{
    return new WQVM.REVData(n,getEVH(n,QQv),QQdp);
}


/** Inverse order low/high of eigenvectors & eigenvalues.
 *  First n elements are eigenvalues, then n eigenvectors follow.
 */
static void inverseOrderOfEV(final int n,final double [] ev)
{
    final int n2=n/2;
    for(int k=0,kk=n-1;k<n2;k++,kk--)
    {
	double t=ev[k];ev[k]=ev[kk];ev[kk]=t;
	for(int l=0,o=n*(k+1),ok=n*(kk+1);l<n;l++,o++,ok++)
	{
	    t=ev[o];ev[o]=ev[ok];ev[ok]=t;
	}
    }
}
abstract double [] getEVH(final int n,final double [] QQv);
abstract double [] getETH(final int n,final double [] QQv);
abstract double [] getETH_OK(final int n);


static abstract class UQVMTest<DTYPE> extends QTest {


abstract double [][] getMonomialMoments(DTYPE o);
abstract UQVM<DTYPE> getUQVM();
abstract RQVM<DTYPE> getRQVM();

UQVMTest(final String name,
	 final java.util.Random r,final double eps)
{
    super(name,r,eps);
}
void test_Common()
{
    final UQVM<DTYPE> q=getUQVM();
    System.err.println(name+":UQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}


GEVData pol_getGEVData(final int n,
		       final double [] QQl,final double [] QQpl,final double [] QQp2l,
		       final double [] QQr,final double [] QQpr,final double [] QQp2r,
		       final double x0,final double p_offset,final boolean flag_order_max)
{
    final double [] ev=DeprecatedHelperEVXData.solveGeneralizedEV(n,QQl,QQr);
    final double [] QQd=new double [n*n],tmp=new double [n];
    for(int k=n;--k>=0;)
    {
	tmp[k]=1;final double fk=Polynomials.getD0(tmp,x0);tmp[k]=0;
	for(int l=n;--l>=0;)
	{
	    tmp[l]=1;final double fl=Polynomials.getD0(tmp,x0);tmp[l]=0;
	    QQd[k*n+l]=fk*fl;
	}
    }
    final double [] evd=DeprecatedHelperEVXData.solveGeneralizedEV(n,QQd,QQr);
    final double Koo=evd[n-1];
    final double [] psiok=new double [n];
    System.arraycopy(evd,(n-1+1)*n,psiok,0,n);

    double lam;
    if(flag_order_max)
    {
	inverseOrderOfEV(neq,ev);
	lam=Double.POSITIVE_INFINITY;
    }
    else
    {
	lam=Double.NEGATIVE_INFINITY;
    }
    for(int k=0;k<n;k++)
    {
	if(flag_order_max)
	{
	    if(!(lam>=ev[k])) throw new RuntimeException("Eigenvalues are not sorted");
	}
	else
	{
	    if(!(lam<=ev[k])) throw new RuntimeException("Eigenvalues are not sorted");
	}
	lam=ev[k];
	UnitTests.checkDoubleRel(TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQr,n),"<q|R|q>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQl,n),"<q|L|q>",lam,"lam",eps);
    }

    final GEVData g=new GEVData(Koo,psiok,ev,QQl,QQpl,QQp2l,QQr,QQpr,QQp2r,p_offset);
    final double [] dyson=new double [n],es=new double [n];
    double ndyson=0;
    for(int k=0;k<n;k++)
    {
	System.arraycopy(ev,(k+1)*n,es,0,n);
	final double nd=Polynomials.getD0(es,x0);
	TMatr.sumP1(dyson,es,nd);
	ndyson+=nd*nd;
	final double il=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQl,n),
	    dpl=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQpl,n)/il,
	    dp2l=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQp2l,n)/il;
	UnitTests.checkDoubleRel(g.pL[k],"g.pL[k]",p_offset+dpl,"p_offset+dpl",eps);
	UnitTests.checkDoubleRel(g.stdev2_pL[k],"g.stdev2_pL[k]",dp2l-dpl*dpl,"dp2l-dpl*dpl",eps);
	final double ir=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQr,n),
	    dpr=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQpr,n)/ir,
	    dp2r=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQp2r,n)/ir;
	UnitTests.checkDoubleRel(g.pR[k],"g.pR[k]",p_offset+dpr,"p_offset+dpr",eps);
	UnitTests.checkDoubleRel(g.stdev2_pR[k],"g.stdev2_pR[k]",dp2r-dpr*dpr,"dp2r-dpr*dpr",eps);
    }
    UnitTests.checkDoubleRel(TMatr.getAMatrA(n,g.psiok,QQr,n),"<psiok|R|psiok>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(n,g.psiok,QQl,n),"<psiok|L|psiok>",g.Iok,"Iok",eps);
    ndyson=Math.sqrt(ndyson);
    for(int k=0;k<n;k++) dyson[k]/=ndyson;
    if(Math.signum(dyson[0])!=Math.signum(g.psiok[0])) for(int k=0;k<n;k++) dyson[k]=-dyson[k];
    UnitTests.checkDoubleArrayRel(g.psiok,"g.psiok",dyson,"dyson",eps);

    return g;
}

void matchGEVData(final GEVData q,final GEVData s)
{
    final int n=q.lam.length;

    UnitTests.checkDoubleRel(q.Koo,"q.Koo",s.Koo,"s.Koo",eps);
    UnitTests.checkDoubleRel(q.Iok,"q.Iok",s.Iok,"s.Iok",eps);
    UnitTests.checkDoubleRel(q.pLok,"q.pLok",s.pLok,"s.pLok",eps);
    UnitTests.checkDoubleRel(q.stdev2_pLok,"q.stdev2_pLok",s.stdev2_pLok,"s.stdev2_pLok",eps);
    UnitTests.checkDoubleRel(q.pRok,"q.pRok",s.pRok,"s.pRok",eps);
    UnitTests.checkDoubleRel(q.stdev2_pRok,"q.stdev2_pRok",s.stdev2_pRok,"s.stdev2_pRok",eps);

    int m=0;
    for(double [][] d:new double [][][]{
	    {q.lam,s.lam},
	    {q.pL,s.pL},
	    {q.stdev2_pL,s.stdev2_pL},
	    {q.pR,s.pR},
	    {q.stdev2_pR,s.stdev2_pR}})
    {
	UnitTests.checkDoubleArrayRel(d[0],"q."+m+".",d[1],"s."+m+".",eps);
	m++;
    }

    UnitTests.checkDoubleArrayRel(java.util.Arrays.copyOf(q.ev,n),"q.ev",java.util.Arrays.copyOf(s.ev,n),"s.ev",eps);
    final double [] sumpsiok=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] es=new double [n];
	System.arraycopy(s.ev,(k+1)*n,es,0,n);
	final double [] eq=new double [n];
	System.arraycopy(q.ev,(k+1)*n,eq,0,n);
	TMatr.sumP1(sumpsiok,eq,q.psiok_over_psi[k]);
	double [] ev_frombasis=convertToMonomials(eq);
	if(Math.signum(ev_frombasis[0])!=Math.signum(es[0]))
	    ev_frombasis=TMatr.getVectorScaled(ev_frombasis,-1);
	UnitTests.checkDoubleArrayRel(ev_frombasis,"ev_frombasis",es,"es",eps);
	UnitTests.checkDoubleRel(Math.abs(q.psiok_over_psi[k]),"|q.psiok_over_psi[k]",Math.abs(s.psiok_over_psi[k]),"|s.psiok_over_psi[k]|",eps);
    }

    UnitTests.checkDoubleArrayRel(q.psiok,"q.psiok",sumpsiok,"sumpsiok",eps);
    double [] pol_psiok=convertToMonomials(q.psiok);
    if(Math.signum(pol_psiok[0])!=Math.signum(s.psiok[0]))
	pol_psiok=TMatr.getVectorScaled(pol_psiok,-1);
    UnitTests.checkDoubleArrayRel(pol_psiok,"pol_psiok",s.psiok,"s.psiok",eps);

    System.err.println(name+":UQVMTest:matchGEVData(): test OK neq="+neq+" eps="+eps);
}

void test_Monomials_BasisExpansion()
{
    final UQVM<DTYPE> q=getUQVM();
    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] mom_v=amoms[0],mom_pv=amoms[1],mom_p2v=amoms[2],
	mom_pt=amoms[4],mom_p2t=amoms[5],mom_t=amoms[8];
    final double x0=q.getX0(),p_offset=q.getPOffset();

    final double [] QQv=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_v,mom_v[0]),
	QQpv=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_pv,mom_v[0]),
	QQp2v=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_p2v,mom_v[0]),
	QQt=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_t,1),
	QQpt=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_pt,1),
	QQp2t=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_p2t,1),
	QQtd=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,amoms[3],1),
	QQa=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,amoms[9],1),
	QQpa=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,amoms[10],1),
	QQp2a=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,amoms[11],1);

    matchGEVData(q.pv,pol_getGEVData(neq,QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,x0,p_offset,true));
    matchGEVData(q.pvd,pol_getGEVData(neq,QQv,QQpv,QQp2v,QQtd,QQpt,QQp2t,x0,p_offset,true));
    matchGEVData(q.pt,pol_getGEVData(neq,QQt,QQpt,QQp2t,QQv,QQpv,QQp2v,x0,p_offset,false));
    matchGEVData(q.pa,pol_getGEVData(neq,QQa,QQpa,QQp2a,QQv,QQpv,QQp2v,x0,p_offset,false));

    UnitTests.checkDoubleRel(q.K00v,"q.K00v",q.pt.Koo,"q.pt.Koo",eps);
    UnitTests.checkDoubleRel(q.peqv,"q.peqv",q.pt.pRok,"q.pt.pRok",eps);
    UnitTests.checkDoubleRel(q.stdev2_peqv,"q.stdev2_peqv",q.pt.stdev2_pRok,"q.pt.stdev2_pRok",eps);

    UnitTests.checkDoubleRel(q.K00t,"q.K00t",q.pv.Koo,"q.pv.Koo",eps);
    UnitTests.checkDoubleRel(q.peqt,"q.peqt",q.pv.pRok,"q.pv.pRok",eps);
    UnitTests.checkDoubleRel(q.stdev2_peqt,"q.stdev2_peqt",q.pv.stdev2_pRok,"q.pv.stdev2_pRok",eps);

    UnitTests.checkDoubleRel(q.K00d,"q.K00d",q.pvd.Koo,"q.pvd.Koo",eps);
    UnitTests.checkDoubleRel(q.peqd,"q.peqd",q.pvd.pRok,"q.pvd.pRok",eps);
    UnitTests.checkDoubleRel(q.stdev2_peqd,"q.stdev2_peqd",q.pvd.stdev2_pRok,"q.pvd.stdev2_pRok",eps);

    System.err.println("q.K00v="+q.K00v+" q.K00t="+q.K00t+" q.K00d="+q.K00d);
    System.err.println("okv="+java.util.Arrays.toString(q.okv));
    System.err.println("okt="+java.util.Arrays.toString(q.okt));
    System.err.println("okd="+java.util.Arrays.toString(q.okd));

    System.err.println(name+":UQVMTest:test_Monomials_BasisExpansion(): test OK neq="+neq+" eps="+eps);
}

void test_Monomials_VT(final String txt,final double x0,final double p_offset,
		       final BEVData b,
		       final double [] QQt,final double [] QQv,final double [] QQpv,final double [] QQp2v,
		       final boolean flag_order_max)
{
    final double [] ev_pol=DeprecatedHelperEVXData.solveGeneralizedEV(neq,QQt,QQv);
    double lam;
    if(flag_order_max)
    {
	inverseOrderOfEV(neq,ev_pol);
	lam=Double.POSITIVE_INFINITY;
    }
    else
    {
	lam=Double.NEGATIVE_INFINITY;
    }
    UnitTests.checkDoubleArrayRel(java.util.Arrays.copyOf(ev_pol,neq),"ev_pol",java.util.Arrays.copyOf(b.ev,neq),"b.ev",eps);

    final double [] fo=new double [neq];
    for(int k=0;k<neq;k++)
    {
	if(flag_order_max)
	{
	    if(!(lam>=ev_pol[k])) throw new RuntimeException("Eigenvalues are not sorted");
	}
	else
	{
	    if(!(lam<=ev_pol[k])) throw new RuntimeException("Eigenvalues are not sorted");
	}
	lam=ev_pol[k];
	// match eigenvalue
	UnitTests.checkDoubleRel(b.ev[k],"b.ev[k]",lam,"lam",eps);
	// match eigenvector
	final double [] es=new double [neq];
	System.arraycopy(ev_pol,(k+1)*neq,es,0,neq);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,es,QQv,neq),"<q|B|q>",1,"1",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,es,QQt,neq),"<q|A|q>",lam,"lam",eps);

	final double [] eq=new double [neq];
	System.arraycopy(b.ev,(k+1)*neq,eq,0,neq);
	double [] ev_frombasis=convertToMonomials(eq);
	if(Math.signum(ev_frombasis[0])!=Math.signum(es[0]))
	    ev_frombasis=TMatr.getVectorScaled(ev_frombasis,-1);
	UnitTests.checkDoubleArrayRel(ev_frombasis,"conv(eq)",es,"es",eps);

	final double [] tmp=new double [neq];tmp[k]=1;
	fo[k]=Polynomials.getD0(tmp,x0);

	if(true)
	{
	    final double [] esx=new double [neq];
	    for(int j=0;j<neq;j++)
	    {
		final double sa,sb;
		if(j==k) {sa=ev_pol[k];sb=1;}
		else {sa=sb=0;}
		System.arraycopy(ev_pol,(j+1)*neq,esx,0,neq);
		UnitTests.checkDoubleRel(TMatr.getAMatrB(neq,es,esx,QQv,neq),"<q|B|s>",sb,String.valueOf(sb),eps);
		UnitTests.checkDoubleRel(TMatr.getAMatrB(neq,es,esx,QQt,neq),"<q|A|s>",sa,String.valueOf(sa),eps);
	    }
	}
    }

    // minimal eigenvalue psi0
    final double [] pol_psi0=new double [neq];
    System.arraycopy(ev_pol,(0+1)*neq,pol_psi0,0,neq);
    double [] conv_psi0=convertToMonomials(b.psi0);
    if(Math.signum(conv_psi0[0])!=Math.signum(pol_psi0[0]))
	conv_psi0=TMatr.getVectorScaled(conv_psi0,-1);
    UnitTests.checkDoubleArrayRel(conv_psi0,"conv_psi0",pol_psi0,"pol_psi0",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,pol_psi0,QQv,neq),"<psi0|B|psi0>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,pol_psi0,QQt,neq),"<psi0|A|psi0>",b.ev[0],"b.ev[0]",eps);
    UnitTests.checkDoubleRel(ev_pol[0],"ev_pol[0]",b.I0,"b.I0",eps);
    //final double dp0=TMatr.getAMatrA(neq,pol_psi0,QQpv,neq),dp20=TMatr.getAMatrA(neq,pol_psi0,QQp2v,neq);
    //UnitTests.checkDoubleRel(dp0+p_offset,"p0",b.p0,"b.p0",eps);
    //UnitTests.checkDoubleRel(dp20-dp0*dp0,"dp20-dp0*dp0",b.stdev2_p0,"b.stdev2_p0",eps);

    // psiok
    final double [] QQd=new double [neq*neq];
    for(int k=0;k<neq;k++) for(int l=0;l<neq;l++) QQd[k*neq+l]=fo[k]*fo[l];
    final double [] ev_d=DeprecatedHelperEVXData.solveGeneralizedEV(neq,QQd,QQv);
    final double [] pol_psiok=new double [neq];
    System.arraycopy(ev_d,(neq-1+1)*neq,pol_psiok,0,neq);
    double [] conv_psiok=convertToMonomials(b.psiok);
    if(Math.signum(conv_psiok[0])!=Math.signum(pol_psiok[0]))
	conv_psiok=TMatr.getVectorScaled(conv_psiok,-1);
    UnitTests.checkDoubleArrayRel(conv_psiok,"conv_psiok",pol_psiok,"pol_psiok",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,pol_psiok,QQv,neq),"<psiok|B|psiok>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,pol_psiok,QQd,neq),"<psiok|A|psiok>",b.K00,"b.K00",eps);
    UnitTests.checkDoubleRel(ev_d[neq-1],"ev_d[neq-1]",b.K00,"b.K00",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,pol_psiok,QQt,neq),"<psiok|A_t|psipk>",b.Iok,"b.Iok",eps);

    //final double dpok=TMatr.getAMatrA(neq,pol_psiok,QQpv,neq),dp2ok=TMatr.getAMatrA(neq,pol_psiok,QQp2v,neq);
    //UnitTests.checkDoubleRel(dpok+p_offset,"pok",b.pok,"b.pok",eps);
    //UnitTests.checkDoubleRel(dp2ok-dpok*dpok,"dp2ok-dpok*dpok",b.stdev2_pok,"b.stdev2_pok",eps);


    double [] psiok_sum=new double [neq],psiok_pol_sum=new double [neq];
    double lok=0,Iok_0=0,Iok_o=0;
    final double [] QQP=new double [neq*neq],fexp=new double [neq];
    for(int k=neq;--k>=0;)
    {
	final double f=TMatr.getAMatrB(neq,ev_pol,(k+1)*neq,pol_psiok,0,QQv,neq);
	UnitTests.checkDoubleRel(Math.abs(f),"|f|",Math.abs(b.psiok_over_psi[k]),"|b.psiok_over_psi|",eps);
	fexp[k]=f;
	lok+=(f*f)*ev_pol[k];
	final double [] eq=new double [neq];
	System.arraycopy(b.ev,(k+1)*neq,eq,0,neq);
	TMatr.sumP1(psiok_sum,eq,b.psiok_over_psi[k]);
	final double [] es=new double [neq];
	System.arraycopy(ev_pol,(k+1)*neq,es,0,neq);
	if(k==0)
	{
	    Iok_o+=TMatr.getAMatrA(neq,psiok_pol_sum,QQt,neq);
	    Iok_0+=TMatr.getAMatrA(neq,es,QQt,neq)*(f*f);
	}
	TMatr.sumP1(psiok_pol_sum,es,f);
	for(int j=0;j<neq;j++)
	{
	   final double [] esj=new double [neq];
	   System.arraycopy(ev_pol,(j+1)*neq,esj,0,neq);
	   QQP[k*neq+j]=TMatr.getAMatrB(neq,es,0,esj,0,QQpv,neq);
	}
    }
    UnitTests.checkDoubleArrayRel(b.psiok,"b.psiok",psiok_sum,"psiok_sum",eps);
    UnitTests.checkDoubleArrayRel(pol_psiok,"pol_psiok",psiok_pol_sum,"psiok_pol_sum",eps);
    UnitTests.checkDoubleRel(b.Iok,"b.Iok",lok,"lok",eps);
    final double [] f1=new double [neq],f2=new double [neq];
    System.arraycopy(fexp,0,f1,0,neq);
    f1[0]=0;f2[0]=fexp[0];
    UnitTests.checkDoubleRel(b.IpI,"b.IpI",TMatr.getAMatrB(neq,f1,0,f2,0,QQP,neq),"<f1|QQP|f2>",eps);
    UnitTests.checkDoubleRel(b.Iok_0,"b.Iok_0",Iok_0,"Iok_0",eps);
    UnitTests.checkDoubleRel(b.Iok_o,"b.Iok_o",Iok_o,"Iok_o",eps);
    UnitTests.checkDoubleRel(b.dP_Iok_0,"b.dP_Iok_0",TMatr.getAMatrB(neq,f2,0,f2,0,QQP,neq),"<f2|QQP|f2>",eps);
    UnitTests.checkDoubleRel(b.dP_Iok_o,"b.dP_Iok_o",TMatr.getAMatrB(neq,f1,0,f1,0,QQP,neq),"<f1|QQP|f1>",eps);
    UnitTests.checkDoubleRel(b.dP_Iok,"b.dP_Iok",TMatr.getAMatrB(neq,fexp,0,fexp,0,QQP,neq),"<fexp|QQP|fexp>",eps);
    UnitTests.checkDoubleRel(b.dP_Iok_0+b.dP_Iok_o+2*b.IpI,"b.dP_Iok_0+b.dP_Iok_o+2*b.IpI",b.dP_Iok,"b.dP_Iok",eps);

    System.err.println(name+":UQVMTest:test_Monomials_VT():"+txt+": test OK neq="+neq+" eps="+eps);
}


void test_Monomials()
{
    final UQVM<DTYPE> q=getUQVM();
    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] mom_v=amoms[0],mom_pv=amoms[1],mom_p2v=amoms[2],
	mom_pt=amoms[4],mom_p2t=amoms[5],mom_t=amoms[8];
    System.err.println("mom_t="+java.util.Arrays.toString(mom_t)+"\n"+"amoms[3]="+java.util.Arrays.toString(amoms[3]));
    final double [] QQ_v=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_v,mom_v[0]),
	QQ_pv=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_pv,mom_v[0]),
	QQ_p2v=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_p2v,mom_v[0]),
	QQ_t=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_t,1);

    test_Monomials_VT("T",q.getX0(),q.getPOffset(),q.bt,QQ_t,QQ_v,QQ_pv,null,false);
    test_Monomials_VT("V",q.getX0(),q.getPOffset(),q.bv,QQ_v,QQ_t,QQ_pv,null,true);

    System.err.println(name+":UQVMTest:test_Monomials(): test OK neq="+neq+" eps="+eps);
}

void test_Psi0_PsiOK()
{

    final UQVM<DTYPE> q=getUQVM();
    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] mom_v=amoms[0],mom_pv=amoms[1],mom_p2v=amoms[2],mom_t=amoms[8];
    final double [] QQ_v=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_v,mom_v[0]),
	QQ_pv=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_pv,mom_v[0]),
	QQ_p2v=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_p2v,mom_v[0]),
	QQ_t=NQVM.NQVMTest.pol_getMM_Matr_scal(neq,mom_t,1);


    UnitTests.checkDoubleRel(q.I0,"q.I0",q.bv.ev[0],"q.bv.ev[0]",eps);
    final double [] psi0_test=Math.signum(q.psi0[0])==Math.signum(q.bv.psi0[0])?q.bv.psi0:TMatr.getVectorScaled(q.bv.psi0,-1);
    UnitTests.checkDoubleArrayRel(q.psi0,"q.psi0",psi0_test,"psi0_test",eps);

    UnitTests.checkDoubleRel(q.K00t,"q.K00t",q.bv.K00,"q.bv.K00",eps);
    final double [] psiok_test=Math.signum(q.psiok[0])==Math.signum(q.bv.psiok[0])?q.bv.psiok:TMatr.getVectorScaled(q.bv.psiok,-1);
    UnitTests.checkDoubleArrayRel(q.psiok,"q.psiok",psiok_test,"psiok_test",eps);
    final double [] pol_psi0=convertToMonomials(q.psi0);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,pol_psi0,QQ_t,neq),"<q|B|q>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,pol_psi0,QQ_v,neq),"<q|A|q>",q.I0,"q.I0",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,pol_psi0,QQ_pv,neq),"<q|pA|q>",q.dpI0,"q.dpI0",eps);
    final double [] pol_psiok=convertToMonomials(q.psiok);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,pol_psiok,QQ_t,neq),"<q|B|q>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,pol_psiok,QQ_v,neq),"<q|A|q>",q.Iok,"q.Iok",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(neq,pol_psiok,QQ_pv,neq),"<q|pA|q>",q.dpIok,"q.dpIok",eps);

    System.err.println("q.I0="+q.I0+" q.Iok="+q.Iok);



    System.err.println(name+":UQVMTest:test_Psi0_PsiOK(): test OK neq="+neq+" eps="+eps);
}

void test_RQVM()
{
    final UQVM<DTYPE> q=getUQVM();
    final RQVM<DTYPE> s=getRQVM();
    System.err.println("s.xnodes[0]="+s.xnodes[0]+" s.xnodes.length="+s.xnodes.length);

     UnitTests.checkDoubleRel(1/q.bt.K00,"1/q.bt.K00",s.wnodes[0],"s.wnodes[0]",eps);
     // Broken for some reason. Need to think more about Nevai operator and squared Legendre Interpolatory polynomial.
     /*if(false)
    {
    // match Nevai operator to squared Legendre interpolary polynomial
    UnitTests.checkDoubleRel(q.bv.pok,"q.bv.pok",s.p2nodes[0],"s.p2nodes[0]",eps);
    UnitTests.checkDoubleRel(q.bv.stdev2_pok,"q.bv.stdev2_pok",s.p2nodes2[0],"s.p2nodes2[0]",eps);
    }*/

    System.err.println(name+":UQVMTest:test_RQVM(): test OK neq="+neq+" eps="+eps);
}




static WQVM.REVData pol_REVData(final int n,final double [] mom_v,final double [] mom_t,final double [] mom_dp,final double eps)
{
    final double []
	QQv=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_v,mom_v[0]),
	QQdp=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_dp,1),
	QQt=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_t,1);

    final double [] ev=DeprecatedHelperEVXData.solveGeneralizedEV(n,QQt,QQv);

    double lam=Double.NEGATIVE_INFINITY;
    for(int k=0;k<n;k++)
    {
	if(!(lam<=ev[k])) throw new RuntimeException("Eigenvalues are not sorted");
	lam=ev[k];
	for(int l=0;k<n;k++)
	{
	    final double normx,lamx;
	    if(k==l){normx=1;lamx=lam;}
	    else {normx=lamx=0;}
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(n,ev,(k+1)*n,ev,(l+1)*n,QQv,n),"<q|QQv|l>",normx,"normx",eps);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(n,ev,(k+1)*n,ev,(l+1)*n,QQt,n),"<q|QQt|l>",lamx,"lamx",eps);
	}
    }

    final WQVM.REVData g=new WQVM.REVData(n,ev,QQdp);
    final double [] QQdp_x_IQQv_x_QQdp=TMatr.mulMMM(n,QQdp,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQv),QQdp);
    for(int k=0;k<n;k++)
    {
	final double lamk=ev[k],nrm=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQv,n);
	final double jk=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQdp,n)/nrm,
	    sigma2_jk=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQdp_x_IQQv_x_QQdp,n)/nrm-jk*jk;
	UnitTests.checkDoubleRel(g.lam[k],"g.lam[k]",lamk,"ev[k]",eps);
	UnitTests.checkDoubleRel(g.j[k],"g.j[k]",jk,"jk",eps);
	UnitTests.checkDoubleRel(g.sigma2_j[k],"g.sigma2_j[k]",sigma2_jk,"sigma2_jk",eps);
    }

    return g;
}

void matchRurik(final WQVM.REVData q,final WQVM.REVData s)
{
    final int n=q.lam.length;

    UnitTests.checkDoubleArrayRel(q.lam,"q.lam",s.lam,"s.lam",eps);
    UnitTests.checkDoubleArrayRel(q.j,"q.j",s.j,"s.j",eps);
    UnitTests.checkDoubleArrayRel(q.sigma2_j,"q.sigma2_j",s.sigma2_j,"s.sigma2_j",eps);
    UnitTests.checkDoubleArrayRel(java.util.Arrays.copyOf(q.ev,n),"q.ev",java.util.Arrays.copyOf(s.ev,n),"s.ev",eps);

    final double [] eq=new double [n],es=new double [n];
    for(int k=0;k<n;k++)
    {
	System.arraycopy(s.ev,(k+1)*n,es,0,n);
	System.arraycopy(q.ev,(k+1)*n,eq,0,n);
	double [] ev_frombasis=convertToMonomials(eq);
	if(Math.signum(ev_frombasis[0])!=Math.signum(es[0]))
	    ev_frombasis=TMatr.getVectorScaled(ev_frombasis,-1);
	UnitTests.checkDoubleArrayRel(ev_frombasis,"ev_frombasis",es,"es",eps);
    }

    System.err.println(name+":UQVMTest:matchRurik(): test OK neq="+neq+" eps="+eps);
}


void test_Rurik()
{

    final UQVM<DTYPE> q=getUQVM();
    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] mom_v=amoms[0],mom_dp=amoms[6],mom_t=amoms[8];
    matchRurik(q.rurik,pol_REVData(neq,mom_v,mom_t,mom_dp,eps));

    System.err.println(name+":UQVMTest:test_Rurik(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":UQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Monomials_BasisExpansion();
    test_Monomials();
    test_RQVM();
    test_Psi0_PsiOK();
    test_Rurik();

    System.err.println(name+":UQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":UQVMTest: starting tests; ver=$Id: UQVM.java,v 1.109 2022/08/08 20:10:47 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<118;i++)
	{
	    test_Calculations(530+r.nextInt(4350),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":UQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: UQVM.java,v 1.109 2022/08/08 20:10:47 mal Exp $");
}
} // END OF class UQVMTest



}// END OF class UQVM
