package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class UQVMLaguerre extends UQVM<QVMDataL> {

UQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static UQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final UQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new UQVMLaguerre(neq,o);
}

DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
double getX0(){return 0;}
double getPOffset(){return odata.p_offset;}
double [] getEVH(final int n,final double [] QQv)
{
    final double [] ev=DeprecatedHelperEVXData.getSymEV(n,QQv);
    // convert back to our basis
    for(int k=0;k<n;k++)
    {
	final double lam=ev[k],sf=Math.sqrt(lam);
	ev[k]=1/lam;
	for(int l=0,o=(k+1)*n;l<n;l++,o++) ev[o]=ev[o]/sf;
    }
    // now after conversion highest lamda eff are on top. Inverse order.
    inverseOrderOfEV(n,ev);
    return ev;
}
double [] getETH(final int n,final double [] QQv)
{
    final double [] ev=DeprecatedHelperEVXData.getSymEV(n,QQv);
    // now make highest lamda on top. Inverse order.
    inverseOrderOfEV(n,ev);
    return ev;
}
double [] getETH_OK(final int n)
{
    final double [] ok=new double [n]; ok[0]=1;
    return WIntegratorLaguerre.B.getL1FromL0(ok);
}

static class UQVMLaguerreTest extends UQVMTest<QVMDataL> {
UQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-5);
}
long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
UQVMLaguerre getUQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
RQVMLaguerre getRQVM(){return RQVMLaguerre.calculate(neq-1,currenttime,tau,tb,tb.length,null);}

double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData getPData(final int nmoms)
{
    final QVMData qdata=QVMData.calculateQVMData(1,nmoms,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
    return qdata;
}
double [][] getMonomialMoments(final QVMDataL odata)
{
    final int nmoms=2*neq-1;
    final QVMData qd=getPData(nmoms);
    return new double [][]{
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_dV,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_dV,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dT,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_dT,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_dT,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomDP(nmoms,qd),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomDS(nmoms,qd),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomT(nmoms),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_A,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_A,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_A,odata.p_offset)
    };
}


} // END of class UQVMLaguerreTest

public static void main(String [] args)
{
    new UQVMLaguerreTest("UQVMLaguerre").runTests(5);
} // END main



} // END OF class UQVMLaguerre
