package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class UQVMLegendreShifted extends UQVM<QVMDataP> {

UQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static UQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final UQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new UQVMLegendreShifted(neq,o);
}

DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
double getX0(){return 1;}
double getPOffset(){return odata.p_offset;}
double [] getEVH(final int n,final double [] QQv)
{
    final int oqq=n*n;
    final double [] qq=new double [oqq],nrm=new double [n];
    // adjust scale to LegendreShifted basis <Q_n|Q_n>_t=1/(2*n+1)
    for(int k=0;k<n;k++)
    {
	final double s=nrm[k]=Math.sqrt(2*k+1);
	for(int l=0,ok=k*n;l<=k;l++)
	{
	    qq[ok+l]=qq[l*n+k]=QQv[ok+l]*(s*nrm[l]);
	}
    }
    final double [] ev=DeprecatedHelperEVXData.getSymEV(n,qq);
    // convert back to our basis
    for(int k=0;k<n;k++)
    {
	final double lam=ev[k],sf=Math.sqrt(lam);
	ev[k]=1/lam;
	for(int l=0,o=(k+1)*n;l<n;l++,o++) ev[o]=(nrm[l]/sf)*ev[o];
    }
    // now after conversion highest lamda eff are on top. Inverse order.
    inverseOrderOfEV(n,ev);
    return ev;
}
double [] getETH(final int n,final double [] QQv)
{
    final int oqq=n*n;
    final double [] qq=new double [oqq],nrm=new double [n];
    // adjust scale to LegendreShifted basis <Q_n|Q_n>_t=1/(2*n+1)
    for(int k=0;k<n;k++)
    {
	final double s=nrm[k]=Math.sqrt(2*k+1);
	for(int l=0,ok=k*n;l<=k;l++)
	{
	    qq[ok+l]=qq[l*n+k]=QQv[ok+l]*(s*nrm[l]);
	}
    }
    final double [] ev=DeprecatedHelperEVXData.getSymEV(n,qq);
    // convert back to our basis
    for(int k=0;k<n;k++)
    {
	for(int l=0,o=(k+1)*n;l<n;l++,o++) ev[o]=(nrm[l])*ev[o];
    }
    // now make highest lamda on top. Inverse order.
    inverseOrderOfEV(n,ev);
    return ev;
}
double [] getETH_OK(final int n)
{
    final double [] ok0=new double [n]; ok0[0]=1;
    return DeprecatedHelperEVXData.getXPnprimeplusPnMoments(ok0);
}

static class UQVMLegendreShiftedTest extends UQVMTest<QVMDataP> {
UQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/2)/(9*5*7*8)*(9*5*7*8);
}
UQVMLegendreShifted getUQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
RQVMLegendreShifted getRQVM(){return RQVMLegendreShifted.calculate(neq-1,currenttime,tau,tb,tb.length,null);}

double [] convertToMonomials(final double [] p){return WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData [] getPData(final int nmoms)
{
    final QVMData [] qdata=new QVMData [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}
double [][] getMonomialMoments(final QVMDataP odata)
{
    final int nmoms=2*neq-1;
    final QVMData [] qd=getPData(nmoms);
    return new double [][]{
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dT,true,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_dT,true,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_dT,true,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomDP(nmoms,qd),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomDS(nmoms,qd),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomT(nmoms),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_A,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_A,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_A,false,odata.p_offset)
    };
}


} // END of class UQVMLegendreShiftedTest

public static void main(String [] args)
{
    new UQVMLegendreShiftedTest("UQVMLegendreShifted").runTests(5);
} // END main



} // END OF class UQVMLegendreShifted
