package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: UkQVM.java,v 1.105 2022/11/20 12:41:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class UkQVM<DTYPE> {

final DTYPE odata;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=getABasis();

final double I_0,I_L,I_H,f_0,f_L,f_H,F_0,F_L,F_H,theta_f_LH,theta_F_LH,PA_L,PA_H,Idp_0,Idp_L,Idp_H,dp_0,dp_L,dp_H;
final double Isell_0,Isell_L,Isell_H,Ibuy_0,Ibuy_L,Ibuy_H;
final State sst;
final double p_IH,var_p_IH,var_p_Ispur,var_p_IH_Proj;
final EVXData VS,Th,IT,ITo,PnL_IH;
final double var_pI_ITo_IH,p_ITo_IH;
final double _I_0,_I_L,_I_H,_pI_0,_pI_L,_pI_H,_pI_pL,_I_pL,_I_pH,_pI_pH,_p_I_H,_p_I_L,_w_pnl_L_I0,_w_pnl_H_I0,
    _J_0,_J_L,_J_H,_SJ_0,_SJ_L,_SJ_H,
    _rI_L,_rI_H,_p_rI_H,_rpI_L,_rpI_H,_w_rpIL_rIH,_w_rpIH_rIH,_w_rpIL_rIL,_w_rpIH_rIL,_rI_pIL,_rI_pIH;
final double _w_pIL_IH,_w_pIH_IH,_w_pIL_IL,_w_pIH_IL,theta_pIL_pIH,theta_pIL_IH,theta_pIH_IH;

static class State {
    final MaxStateSaver rim,rlm,rhm,risellm,ribuym;
    final MMaxStateSaver rimF;
    State(final MaxStateSaver rim,
	  final MaxStateSaver rlm,
	  final MaxStateSaver rhm,
	  final MaxStateSaver risellm,
	  final MaxStateSaver ribuym,
	  final MMaxStateSaver rimF)
    {
	this.rim=rim;
	this.rlm=rlm;
	this.rhm=rhm;
	this.risellm=risellm;
	this.ribuym=ribuym;
	this.rimF=rimF;
    }
}

UkQVM(final int n,final DTYPE odata,final State rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.odata=odata;

    final int nmoms=2*n-1;

    final double [][] mm=getMoments(nmoms);
    final double [] basis=getBasis(n,mm[1],mm[0]),sbasis=getBasis(n,mm[2],mm[0]),Sbasis=getBasis(n,mm[3],mm[0]),PAbasis=getBasis(n,mm[5],mm[4]);
    final double [] QQt=M.B.getQQMatr(n,mm[0]),QQf=M.B.getQQMatr(n,mm[2]),QQVS=M.B.getQQMatr(n,mm[3]);
    this.VS=EVXData.getEVXDataFromQQ(n,QQVS,QQt,M);
    this.Th=EVXData.getEVXDataFromQQ(n,QQf,QQt,M);

    final int i0=0,iL=1,iH=n-1;
    this.I_0=basis[i0];
    this.I_L=basis[iL];
    this.I_H=basis[iH];
    this.f_0=sbasis[i0];
    this.f_L=sbasis[iL];
    this.f_H=sbasis[iH];
    this.theta_f_LH=getDistance2(EVXData.getPsiFromBasis(iL,sbasis,n),EVXData.getPsiFromBasis(iH,sbasis,n));
    this.F_0=Sbasis[i0];
    this.F_L=Sbasis[iL];
    this.F_H=Sbasis[iH];
    this.theta_F_LH=getDistance2(EVXData.getPsiFromBasis(iL,Sbasis,n),EVXData.getPsiFromBasis(iH,Sbasis,n));
    this.PA_L=PAbasis[iL];
    this.PA_H=PAbasis[iH];

    final MaxStateSaver slm;
    if(I_0<I_L){
	slm=MaxStateSaver.getTaintedUpdate(rst!=null?rst.rlm:null);
    } else if(-f_0>-f_L){
	slm=MaxStateSaver.getMaxUpdate(-f_0,getPLast(),getTLast(),I_0,getSVol(),getVol(),rst!=null?rst.rlm:null);
    } else {
	slm=MaxStateSaver.getNothingUpdate(rst!=null?rst.rlm:null);
    }
    final MaxStateSaver shm;
    if(I_0<I_L){
	shm=MaxStateSaver.getTaintedUpdate(rst!=null?rst.rhm:null);
    } else if(f_0>f_H){
	shm=MaxStateSaver.getMaxUpdate(f_0,getPLast(),getTLast(),I_0,getSVol(),getVol(),rst!=null?rst.rhm:null);
    } else {
	shm=MaxStateSaver.getNothingUpdate(rst!=null?rst.rhm:null);
    }
    final MaxStateSaver sim;
    if(I_0<I_L){
	sim=MaxStateSaver.getTaintedUpdate(rst!=null?rst.rim:null);
    } else if(I_0>I_H){
	sim=MaxStateSaver.getMaxUpdate(I_0,getPLast(),getTLast(),f_0,getSVol(),getVol(),rst!=null?rst.rim:null);
    } else {
	sim=MaxStateSaver.getNothingUpdate(rst!=null?rst.rim:null);
    }

    final MMaxStateSaver simF;
    if(I_0<I_L){
	simF=MMaxStateSaver.getTaintedUpdate(rst!=null?rst.rimF:null);
    } else if(I_0>I_H){
	simF=MMaxStateSaver.getMaxUpdate(I_0,getPLast(),getTLast(),f_0,f_L,f_H,getVol(),0,0,rst!=null?rst.rimF:null);
    } else {
	simF=MMaxStateSaver.getNothingUpdate(rst!=null?rst.rimF:null);
    }


    final double [] basisSell=getBasis(n,mm[6],mm[0]),basisBuy=getBasis(n,mm[7],mm[0]);
    this.Isell_0=basisSell[i0];
    this.Isell_L=basisSell[iL];
    this.Isell_H=basisSell[iH];
    this.Ibuy_0=basisBuy[i0];
    this.Ibuy_L=basisBuy[iL];
    this.Ibuy_H=basisBuy[iH];

    final MaxStateSaver sisellm;
    if(Isell_0<Isell_L){
	sisellm=MaxStateSaver.getTaintedUpdate(rst!=null?rst.risellm:null);
    } else if(Isell_0>Isell_H){
	sisellm=MaxStateSaver.getMaxUpdate(Isell_0,getPLast(),getTLast(),f_0,getSVol(),getVol(),rst!=null?rst.risellm:null);
    } else {
	sisellm=MaxStateSaver.getNothingUpdate(rst!=null?rst.risellm:null);
    }
    final MaxStateSaver sibuym;
    if(Ibuy_0<Ibuy_L){
	sibuym=MaxStateSaver.getTaintedUpdate(rst!=null?rst.ribuym:null);
    } else if(Ibuy_0>Ibuy_H){
	sibuym=MaxStateSaver.getMaxUpdate(Ibuy_0,getPLast(),getTLast(),f_0,getSVol(),getVol(),rst!=null?rst.ribuym:null);
    } else {
	sibuym=MaxStateSaver.getNothingUpdate(rst!=null?rst.ribuym:null);
    }

    {
	final double [] QQi=M.B.getQQMatr(n,mm[8]),QQpi=M.B.getQQMatr(n,mm[9]),QQdp=M.B.getQQMatr(n,mm[10]),
	    IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	    QQj=TMatr.mulMatrices(n,new double [][]{QQdp,IQQt,QQi});
	TMatr.selectHermitianPartInPlace(n,QQj);
	final double [] basisIdp=EdComm.getIRadauBasisPsi0FromQQ(n,QQj,QQt,M);
	Idp_0=basisIdp[i0];
	Idp_L=basisIdp[iL];
	Idp_H=basisIdp[iH];
	final double [] basisdp=EdComm.getIRadauBasisPsi0FromQQ(n,QQdp,QQt,M);
	dp_0=basisdp[i0];
	dp_L=basisdp[iL];
	dp_H=basisdp[iH];

	final double [] _basis=getBasis(n,mm[8],mm[0]),_vsbasis=getBasis(n,mm[11],mm[0]),_Svsbasis=getBasis(n,mm[12],mm[0]);
	_I_0=_basis[i0];
	_I_L=_basis[iL];
	_I_H=_basis[iH];
	_J_0=_vsbasis[i0];
	_J_L=_vsbasis[iL];
	_J_H=_vsbasis[iH];
	_SJ_0=_Svsbasis[i0];
	_SJ_L=_Svsbasis[iL];
	_SJ_H=_Svsbasis[iH];
	final double [] psi_I_H=EVXData.getPsiFromBasis(iH,_basis,n),
	    psi_I_L=EVXData.getPsiFromBasis(iL,_basis,n),
	    psi_0=EVXData.getPsiFromBasis(i0,_basis,n);
	this._p_I_H=TMatr.getAMatrA(psi_I_H,QQpi,n)/_I_H;
	this._p_I_L=TMatr.getAMatrA(psi_I_L,QQpi,n)/_I_L;
	final double [] rpnl=M.EV.getGEV_Generalized(n,TMatr.sumAcB(QQpi,-this._p_I_H,QQi),QQt,true);
	this._w_pnl_L_I0=TMatr.getAMatrB(EVXData.getPsiFromBasis(0,rpnl,n),psi_0,QQt,n);
	this._w_pnl_H_I0=TMatr.getAMatrB(EVXData.getPsiFromBasis(n-1,rpnl,n),psi_0,QQt,n);

	final double [] _pI_basis=getBasis(n,mm[9],mm[0]);
	_pI_0=_pI_basis[i0];
	_pI_L=_pI_basis[iL];
	_pI_H=_pI_basis[iH];
	final double [] psi_pI_H=EVXData.getPsiFromBasis(iH,_pI_basis,n),
	    psi_pI_L=EVXData.getPsiFromBasis(iL,_pI_basis,n);
	this._w_pIL_IH=TMatr.getAMatrB(psi_I_H,psi_pI_L,QQt,n);
	this._w_pIH_IH=TMatr.getAMatrB(psi_I_H,psi_pI_H,QQt,n);
	this._w_pIL_IL=TMatr.getAMatrB(psi_I_L,psi_pI_L,QQt,n);
	this._w_pIH_IL=TMatr.getAMatrB(psi_I_L,psi_pI_H,QQt,n);
	this._I_pL=TMatr.getAMatrA(psi_pI_L,QQi,n);
	this._I_pH=TMatr.getAMatrA(psi_pI_H,QQi,n);
	this._pI_pL=_pI_L/_I_pL;
	this._pI_pH=_pI_H/_I_pH;
	//System.err.println("_pI_basis="+java.util.Arrays.toString(_pI_basis)+"\n_I_pL="+_I_pL+" _I_pH="+_I_pH+" _I_L="+_I_L+" _I_H="+_I_H);


	this.theta_pIL_pIH=getDistance2(psi_pI_L,psi_pI_H);
	this.theta_pIL_IH=getDistance2(psi_pI_L,psi_I_H);
	this.theta_pIH_IH=getDistance2(psi_pI_H,psi_I_H);

	final double [] tmoms1=new double [nmoms+1];
	getAccess().setMT(tmoms1,nmoms+1);

	final int nd=n-1;
	final double [] RQQt=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(mm[0])),
	    RQQi=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(mm[8])),
	    RQQpi=M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(mm[9]));
	this.rIt=EVXData.getEVXDataFromQQ(nd,RQQi,RQQt,M);
	{
	final double [] pert_Rpsi_IH=DeprecatedHelperEVXData.getPsiPertr(rIt,nd-1,RQQpi),Rpsi_IH=rIt.getPsi(nd-1),
	    p2=M.getRadauMultiplied(M.B.mult2Pol(Rpsi_IH,Rpsi_IH)),
	    pert_p2=M.getRadauMultiplied(M.B.mult2Pol(Rpsi_IH,pert_Rpsi_IH));
	this.dist2_RIp=getDistanceNo2(p2,pert_p2);
	this.distX_RIp=TMatr.mulVVn1(M.getRadauMultiplied(pert_p2),mm[0]);
	//System.err.println("<p2>="+TMatr.mulVVn1(p2,mm[0])+" must be =1");
	//System.err.println("<pert_p2>="+TMatr.mulVVn1(pert_p2,mm[0])+" must be =0");
	}

	final double [] revI=M.EV.getGEV_Generalized(nd,RQQi,RQQt,true),
	    revpI=M.EV.getGEV_Generalized(nd,RQQpi,RQQt,true),
	    rpsi_IH=EVXData.getPsiFromBasis(nd-1,revI,nd),
	    rpsi_IL=EVXData.getPsiFromBasis(0,revI,nd),
	    rpsi_pIL=EVXData.getPsiFromBasis(0,revpI,nd),
	    rpsi_pIH=EVXData.getPsiFromBasis(nd-1,revpI,nd);

	_rI_L=revI[0];
	_rI_H=revI[nd-1];
	_p_rI_H=TMatr.getAMatrA(rpsi_IH,RQQpi,nd)/_rI_H;
	_rpI_L=revpI[0];
	_rpI_H=revpI[nd-1];
	_rI_pIL=TMatr.getAMatrA(rpsi_pIL,RQQi,nd);
	_rI_pIH=TMatr.getAMatrA(rpsi_pIH,RQQi,nd);
	_w_rpIL_rIH=TMatr.getAMatrB(rpsi_IH,rpsi_pIL,RQQt,nd);
	_w_rpIH_rIH=TMatr.getAMatrB(rpsi_IH,rpsi_pIH,RQQt,nd);
	_w_rpIL_rIL=TMatr.getAMatrB(rpsi_IL,rpsi_pIL,RQQt,nd);
	_w_rpIH_rIL=TMatr.getAMatrB(rpsi_IL,rpsi_pIH,RQQt,nd);


	{
	this.IT=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
	final double [] IT_psiH=IT.getPsi(n-1),pert_IT_psiH=DeprecatedHelperEVXData.getPsiPertr(IT,n-1,QQpi),
	    pp2=M.B.mult2Pol(IT_psiH,IT_psiH),pert_pp2=M.B.mult2Pol(pert_IT_psiH,IT_psiH),
	    D_IT_psiH=getEDPsi(IT_psiH),D_IT_psi0=getEDPsi(IT.psi0);
	this.dist2_ITp=getDistanceNo2(pp2,pert_pp2);
	this.distX_ITp=TMatr.mulVVn1(M.getRadauMultiplied(pert_pp2),tmoms1);
	this.distDX_ITp=TMatr.getAMatrB(pert_IT_psiH,D_IT_psiH,QQt,n);
	this.Px_ITp=2*TMatr.getAMatrB(IT_psiH,D_IT_psiH,QQpi,n)/(IT.sH*Math.pow(M.B.getD0(IT_psiH,IT.x0),2));
	this.p_IH=TMatr.getAMatrA(IT.psiH,QQpi,n)/IT.sH;
	this.var_p_IH=TMatr.getAMatrB(IT_psiH,D_IT_psiH,QQpi,n)-TMatr.getAMatrB(IT_psiH,IT_psiH,QQpi,n)*TMatr.getAMatrB(IT_psiH,D_IT_psiH,QQt,n);
	double spurVar=0;
	for(int k=0;k<n;k++) spurVar+=getVarDPiOrt(n,IT.getPsi(k),QQt,QQpi,VOLATILITY_EDPsi);
	this.var_p_Ispur=spurVar;
	this.var_p_IH_Proj=getVarPiBasisProj(n,IT.psiH,_basis,QQt,QQpi,VOLATILITY_EDPsi);
	this.var_pI_I0=TMatr.getAMatrB(IT.psi0,TMatr.sumAcB(D_IT_psi0,-TMatr.getAMatrB(D_IT_psi0,IT.psi0,QQt,n),IT.psi0),QQpi,n);
	this.var_I_I0=TMatr.getAMatrB(IT.psi0,TMatr.sumAcB(D_IT_psi0,-TMatr.getAMatrB(D_IT_psi0,IT.psi0,QQt,n),IT.psi0),QQi,n);
	{
	    final double [] IT_psiH_no0=TMatr.sumAcB(IT.psiH,-TMatr.getAMatrB(IT.psiH,IT.psi0,QQt,n),IT.psi0);
	    this.var_pI_I0IH=TMatr.getAMatrB(IT.psi0,IT_psiH_no0,QQpi,n)*TMatr.getAMatrB(IT_psiH_no0,D_IT_psi0,QQt,n);
	    this.var_I_I0IH=TMatr.getAMatrB(IT.psi0,IT_psiH_no0,QQi,n)*TMatr.getAMatrB(IT_psiH_no0,D_IT_psi0,QQt,n);
	}


	final double [] reH=getvPsifPsi(n,IT,n-1,QQt,QQpi),reL=getvPsifPsi(n,IT,0,QQt,QQpi);
	this.Is_p_IH=reH[0];
	this.IL_p_IH=reH[1];
	this.IM_p_IH=reH[2];
	this.Iae_p_IH=reH[3];
	this.Ida0_p_IH=reH[4];
	this.Iie0_p_IH=reH[5];
	this.II0_p_IH=reH[6];
	this.Io0_p_IH=reH[7];
	this.Is_p_IL=reL[0];
	this.IL_p_IL=reL[1];
	this.IM_p_IL=reL[2];
	this.Iae_p_IL=reL[3];
	this.Ida0_p_IL=reL[4];
	this.Iie0_p_IL=reL[5];
	this.II0_p_IL=reL[6];
	this.Io0_p_IL=reL[7];
	this.IT_IT0_p_IH0_IH1=getVarPi(n,false,false,QQt,QQi,QQpi,M,VOLATILITY_EDPsi);
	this.IT_IT0_p_IL0_IL1=getVarPi(n,true,true,QQt,QQi,QQpi,M,VOLATILITY_EDPsi);
	this.IT_IT0_p_IL0_IH1=getVarPi(n,true,false,QQt,QQi,QQpi,M,VOLATILITY_EDPsi);
	this.IT_IT0_p_IH0_IL1=getVarPi(n,false,true,QQt,QQi,QQpi,M,VOLATILITY_EDPsi);


	final double [] var_Psi_dI=new double [n];
	for(int k=0;k<n;k++)
	{
	    if(k==(n-1)) continue;
	    final double [] psik=IT.getPsi(k);
	    final double vt_kh=(M.B.getD0(IT_psiH,IT.x0)*M.B.getD0(psik,IT.x0)*IT.sH
		-TMatr.getAMatrB(IT_psiH,getEDPsi(psik),QQi,n)
		-TMatr.getAMatrB(D_IT_psiH,psik,QQi,n))/
		(IT.ev[n-1]-IT.ev[k]);
	    TMatr.sumP1(var_Psi_dI,psik,vt_kh);
	    final double v_kh=TMatr.getAMatrB(D_IT_psiH,psik,QQt,n);
	    //System.err.println("IT:k="+k+"\tv_kh="+v_kh+"\tvt_kh="+vt_kh);
	}
	if(false)
	{
	System.err.println("IT:<pert_IT_psiH|IT_psiH>="+TMatr.getAMatrB(pert_IT_psiH,IT_psiH,QQt,n)+" must be =0");
	System.err.println("IT:2<psi|I|Dpsi>/(Ih*psi(x0)^2)="+(2*TMatr.getAMatrB(IT_psiH,D_IT_psiH,QQi,n)/(IT.sH*Math.pow(M.B.getD0(IT_psiH,IT.x0),2)))+" must be =1");
	System.err.println("IT:var_I_IH="+(TMatr.getAMatrB(IT_psiH,D_IT_psiH,QQi,n)-TMatr.getAMatrB(IT_psiH,IT_psiH,QQi,n)*TMatr.getAMatrB(IT_psiH,D_IT_psiH,QQt,n))+" must be =0");
	System.err.println("IT:var_p_IH="+var_p_IH+" test_var_p_IH="+TMatr.getAMatrB(IT_psiH,var_Psi_dI,QQpi,n));
	}

	final double [] pert_IT_psiH_nd=getPsiPert(n,IT.ev,n-1,QQpi,false);
	this.dist2nd_ITp=-2*getK2(IT_psiH,pert_IT_psiH_nd,IT_psiH,IT_psiH);
	this.distXnd_ITp=TMatr.mulVVn1(M.getRadauMultiplied(M.B.mult2Pol(pert_IT_psiH_nd,IT_psiH)),tmoms1);

	if(false)
	{
	System.err.println("IT:<pp2>="+TMatr.mulVVn1(pp2,tmoms1)+" must be =1");
	System.err.println("IT:<pert_pp2>="+TMatr.mulVVn1(pert_pp2,tmoms1)+" must be =0");
	System.err.println("IT:dist2_ITp="+dist2_ITp+" -2getK2(IT_psiH,pert_IT_psiH,IT_psiH,IT_psiH)="+(-2*getK2(IT_psiH,pert_IT_psiH,IT_psiH,IT_psiH))+" must be equal");
	}
	final double [] QQo=getQQoMatr(n,IT.x0,M);
	final double psi2x0=Math.pow(M.B.getD0(IT.psi0,IT.x0),2);
	final double [] QQio=TMatr.sumAcB(QQi,-IT.s0/psi2x0,QQo),
	    QQpio=TMatr.sumAcB(QQpi,-0*IT.s0/psi2x0,QQo); // p offset=0
	this.ITo=EVXData.getEVXDataFromQQ(n,QQio,QQt,M);
	this.var_pI_ITo_IH=getVarDPiOrt(n,ITo.psiH,QQt,QQpio,VOLATILITY_EDPsi);
	this.p_ITo_IH=TMatr.getAMatrA(ITo.psiH,QQpio,n)/ITo.sH;
	if(false)
	{
	    System.err.println("ITo:s0="+ITo.s0+" must be =0");
	    System.err.println("ITo:var_pI_ITo_IH="+var_pI_ITo_IH+" test="+getVarDPiOrt(n,ITo.psiH,QQt,TMatr.sumAcB(M.B.getQQMatr(n,mm[1]),-getPLast()*IT.s0/psi2x0,QQo),VOLATILITY_EDPsi));
	}
	this.Comm_pIo_Io_0=getCommutatorDpsiABmBApsi(n,IQQt,QQpio,QQio,IT.psi0,VOLATILITY_EDPsi);
	this.Comm_pI_I_0=getCommutatorDpsiABmBApsi(n,IQQt,QQpi,QQi,IT.psi0,VOLATILITY_EDPsi);
	this.Comm_pI_I_H=getCommutatorDpsiABmBApsi(n,IQQt,QQpi,QQi,IT.psiH,VOLATILITY_EDPsi);
	if(false)
	{
	    final double [][] ps=ACorr.getCommutatorVarPsiX0(n,mm[9],mm[8],mm[0],M,IT.x0);
	    System.err.println("Comm:Comm_pI_I_0="+Comm_pI_I_0+" from ACorr="+TMatr.getAMatrB(ps[1],getEDPsi(ps[0]),QQt,n)+" <psi|varpsi>="+TMatr.getAMatrB(ps[1],ps[0],QQt,n)+" must be =0");
	}
	}

	this.PnL_IH=EVXData.getEVXDataFromQQ(n,TMatr.sumAcB(QQpi,-this.p_IH,QQi),QQt,M);
	//System.err.println("PnL and psiH="+TMatr.getAMatrA(IT.psiH,TMatr.sumAcB(QQpi,-this.p_IH,QQi),n)/IT.sH);

	this.psipert=getPsiPert0Skipped(n,_basis,iH,QQpi);
	this.dist1_Ip=getDistanceNo2(psi_I_H,psipert);
	{
	final double [] pp2=M.B.mult2Pol(psi_I_H,psi_I_H),pert_pp2=M.B.mult2Pol(psi_I_H,psipert);
	this.dist2_Ip=getDistanceNo2(pp2,pert_pp2);
	this.distX_Ip=TMatr.mulVVn1(M.getRadauMultiplied(pert_pp2),tmoms1);
	this.distDX_Ip=TMatr.getAMatrB(psipert,getEDPsi(psi_I_H),QQt,n);
	this.distDX_IL0_IH1_Ip=getVarPi(n,psi_I_L,psi_I_H,QQt,QQpi,VOLATILITY_EDPsi);
	if(false)
	{
	    System.err.println("<pp2>="+TMatr.mulVVn1(pp2,tmoms1)+" must be =1");
	    System.err.println("<pert_pp2>="+TMatr.mulVVn1(pert_pp2,tmoms1)+" must be =0");
	    System.err.println("distDX_Ip="+distDX_Ip+"\t<Dpsipert|psi_I_H>="+TMatr.getAMatrB(getEDPsi(psipert),psi_I_H,QQt,n)+" must be opposite sign");
	    final double [] pH=EVXData.getPsiFromBasis(iH,_basis,n),DpH=getEDPsi(pH);
	    double s=0;
	    for(int k=1;k<n;k++)
	    {
		if(k==iH) continue;
		final double [] psik=EVXData.getPsiFromBasis(k,_basis,n);
		s+=TMatr.getAMatrB(pH,psik,QQpi,n)*TMatr.getAMatrB(psik,DpH,QQt,n)/(_basis[iH]-_basis[k]);
	    }
	    System.err.println("distDX_Ip="+distDX_Ip+" s="+s+" must be equal");
	}


	final double [] psipertI=getPsiPert0Skipped(n,_basis,iH,QQpi,false);
	this.distDX_IIp=TMatr.getAMatrB(psipertI,getEDPsi(psi_I_H),QQt,n);
	this.distX_IIp=TMatr.mulVVn1(M.getRadauMultiplied(M.B.mult2Pol(psi_I_H,psipertI)),tmoms1);
	//System.err.println("<psipertI|psi_I_H>="+TMatr.getAMatrB(psipertI,psi_I_H,QQt,n)+" must be =0");
	}

    }


    this.sst=new State(sim,slm,shm,sisellm,sibuym,simF);


}
final double [] psipert;
final double dist1_Ip,dist2_Ip,distX_Ip,distDX_Ip,distDX_IL0_IH1_Ip,distX_IIp,distDX_IIp;
final EVXData rIt;
final double dist2_RIp,distX_RIp;
final double dist2_ITp,distX_ITp,distDX_ITp,dist2nd_ITp,distXnd_ITp,Px_ITp;
final double Is_p_IH,IL_p_IH,IM_p_IH,Iae_p_IH,Ida0_p_IH,Iie0_p_IH,II0_p_IH,Io0_p_IH,
             Is_p_IL,IL_p_IL,IM_p_IL,Iae_p_IL,Ida0_p_IL,Iie0_p_IL,II0_p_IL,Io0_p_IL,
    IT_IT0_p_IH0_IH1,IT_IT0_p_IL0_IL1,IT_IT0_p_IL0_IH1,IT_IT0_p_IH0_IL1;
final double var_pI_I0,var_I_I0,var_pI_I0IH,var_I_I0IH,
    Comm_pIo_Io_0,Comm_pI_I_0,Comm_pI_I_H;

final VolMatrix VOLATILITY_EDPsi=this::getEDPsi;

static double [] getQQoMatr(final int n,final double x0,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] x0mom=M.B.getX0Moments(n,x0),QQo=new double [n*n];
    for(int k=0;k<n;k++) for(int l=0;l<n;l++) QQo[k*n+l]=x0mom[k]*x0mom[l];
    return QQo;
}

double [][] getMoments(final int nmoms)
{
    final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access=getAccess();
    final double [] tmom=new double [nmoms],imom=new double [nmoms],vsmom=new double [nmoms],VSmom=new double [nmoms];
    // convert to 0 offset
    access.setM(vsmom,1,access.M_RVS,odata,nmoms);
    access.setM(tmom,0,access.M_RVS,odata,nmoms);
    TMatr.sumP1(vsmom,tmom,getPOffset());

    access.setM(imom,1,access.M_I,odata,nmoms);
    access.setM(tmom,0,access.M_I,odata,nmoms);
    TMatr.sumP1(imom,tmom,getPOffset());

    access.setMT(tmom,nmoms);
    access.setFMoments(VSmom,vsmom,nmoms);

    final double [] Imom=new double [nmoms],pImom=new double [nmoms],PVmom=new double [nmoms],Vmom=new double [nmoms],Svsmom=new double [nmoms],_vsmom=new double [nmoms];
    access.setM(Imom,0,access.M_I,odata,nmoms);
    access.setM(Svsmom,0,access.M_VS,odata,nmoms);
    access.setM(_vsmom,0,access.M_RVS,odata,nmoms);
    access.setM(pImom,1,access.M_I,odata,nmoms);
    TMatr.sumP1(pImom,Imom,getPOffset()-getPLast());
    access.setFMoments(PVmom,pImom,nmoms);TMatr.scaleVectorInPlace(PVmom,-1);
    access.setFMoments(Vmom,Imom,nmoms);TMatr.scaleVectorInPlace(Vmom,-1);

    final double [] Isell=new double [nmoms],Ibuy=new double [nmoms],dpmom=new double [nmoms];
    access.setM(Isell,0,access.M_ISELL,odata,nmoms);
    access.setM(Ibuy,0,access.M_IBUY,odata,nmoms);

    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    return new double [][]{tmom,imom,vsmom,VSmom,Vmom,PVmom,Isell,Ibuy,Imom,pImom,dpmom,_vsmom,Svsmom};
}

/** Note! the 0-th ev is skipped.
 */
static double [] getPsiPert0Skipped(final int n,final double [] basis,final int ind_pert,final double [] QQpert,final boolean flag_div_EE)
{
    final double [] pertpsi=new double [n];
    for(int k=1;k<n;k++)
    {
	if(k==ind_pert) continue;
	DeprecatedHelperTMatr.sumP1(pertpsi,basis,n+k*n,n,TMatr.getAMatrB(n,basis,n+k*n,basis,n+ind_pert*n,QQpert,n)/
		    (flag_div_EE?(basis[ind_pert]-basis[k]):1));
    }
    return pertpsi;
}
static double [] getPsiPert0Skipped(final int n,final double [] basis,final int ind_pert,final double [] QQpert)
{
    return getPsiPert0Skipped(n,basis,ind_pert,QQpert,true);
}

static double [] getPsiPert(final int n,final double [] basis,final int ind_pert,final double [] QQpert,final boolean flag_div_EE)
{
    final double [] pertpsi=new double [n];
    for(int k=0;k<n;k++)
    {
	if(k==ind_pert) continue;
	DeprecatedHelperTMatr.sumP1(pertpsi,basis,n+k*n,n,TMatr.getAMatrB(n,basis,n+k*n,basis,n+ind_pert*n,QQpert,n)/
		    (flag_div_EE?(basis[ind_pert]-basis[k]):1));
    }
    return pertpsi;
}

double [] getvPsifPsi(final int n,final EVXData e,final int ind_pert,final double [] QQt,final double [] QQpi)
{
    double s_k_da=0,s_dk_a=0,s_0_0=0,s_ae=0,s_k_da0=0,sie0=0,sI0=0,so0=0;
    final double [] psia=e.getPsi(ind_pert),Dpsia=getEDPsi(psia);
    final double Ift=e.s0;
    for(int k=0;k<n;k++)
    {
	if(k==ind_pert) continue;
	final double [] psik=EVXData.getPsiFromBasis(k,e.ev,n),Dpsik=getEDPsi(psik);
	final double p=TMatr.getAMatrB(psik,psia,QQpi,n);
	final double kDa=TMatr.getAMatrB(psik,Dpsia,QQt,n),aDk=TMatr.getAMatrB(Dpsik,psia,QQt,n),k0a0=M.B.getD0(psia,e.x0)*M.B.getD0(psik,e.x0),pertdenom=(e.ev[ind_pert]-e.ev[k]);
	s_k_da+=p*kDa;
	s_dk_a-=p*aDk;
	s_0_0+=p*k0a0;
	s_ae+=p*(kDa-aDk);
	s_k_da0+=p*M.B.getD1(psia,e.x0)*M.B.getD0(psik,e.x0);
	// assume I=0
	sie0+=p*(k0a0*0-e.ev[k]*kDa-e.ev[ind_pert]*aDk)/pertdenom;
	sI0+=p*(k0a0*(Ift-e.ev[ind_pert])/pertdenom+kDa);
	so0+=p*k0a0*(e.ev[ind_pert]/pertdenom);
	if(false)
	{
	System.err.println("kDa="+kDa+" testkDa="+((k0a0*e.ev[ind_pert]-(e.ev[k]*kDa+e.ev[ind_pert]*aDk))/pertdenom));
	System.err.println("sI0a="+(k0a0*(Ift-e.ev[ind_pert])/pertdenom+kDa)+" testsI0a="+((k0a0*Ift-e.ev[k]*kDa-e.ev[ind_pert]*aDk)/pertdenom));
	}
    }
    return new double []{s_k_da,s_dk_a,s_0_0,s_ae,s_k_da0,sie0,sI0,so0};
}

static double getVarPi(final int n,final boolean flag0_ILow,final boolean flag1_ILow,final double [] QQt,final double [] QQi,final double [] QQpi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix CAL_Dpsi)
{
    final double [] basis0=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M),
	basis1=M.EV.getGEV_Generalized(n,QQi,QQt,true);
    final double [] psi_basis0=EVXData.getPsiFromBasis(flag0_ILow?1:n-1,basis0,n), // 0-th element in basis0 is |psi0>, not |psiL>
	psi_basis1=EVXData.getPsiFromBasis(flag1_ILow?0:n-1,basis1,n), // 0-th elements in basis1 is |psiL>
	var_psi_basis1=TMatr.sumAcB(psi_basis0,-TMatr.getAMatrB(psi_basis0,psi_basis1,QQt,n),psi_basis1);
    if(false)
    {
    System.err.println("<psi_basis1|var_psi_basis1>="+TMatr.getAMatrB(psi_basis1,var_psi_basis1,QQt,n)+" must be =0");
    System.err.println("<psi_basis1|I|var_psi_basis1>="+TMatr.getAMatrB(psi_basis1,var_psi_basis1,QQi,n)+" must be =0");
    System.err.println("<psi_basis0|var_psi_basis1>="+TMatr.getAMatrB(psi_basis0,var_psi_basis1,QQt,n)+" must be =1-<psi_basis0|psi_basis1>^2="+(1-Math.pow(TMatr.getAMatrB(psi_basis0,psi_basis1,QQt,n),2)));
    System.err.println("<psi_basis1|D|psi_basis0>="+TMatr.getAMatrB(psi_basis1,CAL_Dpsi.getDPsi(psi_basis0),QQt,n)+" must be equal to -<psi_basis0|D|psi_basis1>="+(-TMatr.getAMatrB(psi_basis0,CAL_Dpsi.getDPsi(psi_basis1),QQt,n)));
    }
    // <psi_basis1|pI|psi_basis0-<psi_basis0|psi_basis1>psi_basis1>*<psi_basis1|D|psi_basis0>
    return TMatr.getAMatrB(psi_basis1,var_psi_basis1,QQpi,n)
	*TMatr.getAMatrB(psi_basis0,CAL_Dpsi.getDPsi(psi_basis1),QQt,n);
}
static double getVarPi(final int n,final double [] psi0,final double [] psi1,final double [] QQt,final double [] QQpi,final VolMatrix CAL_Dpsi)
{
    return (TMatr.getAMatrB(psi0,psi1,QQpi,n)-TMatr.getAMatrB(psi1,psi1,QQpi,n)*TMatr.getAMatrB(psi0,psi1,QQt,n))*TMatr.getAMatrB(CAL_Dpsi.getDPsi(psi1),psi0,QQt,n);
}
static double getVarPiOrt(final int n,final double [] psi0,final double [] psi1,final double [] QQt,final double [] QQpi,final VolMatrix CAL_Dpsi)
{
    final double [] psi1_no_psi0=TMatr.sumAcB(psi1,-TMatr.getAMatrB(psi1,psi0,QQt,n),psi0);
    return TMatr.getAMatrB(psi0,psi1_no_psi0,QQpi,n)*TMatr.getAMatrB(CAL_Dpsi.getDPsi(psi0),psi1_no_psi0,QQt,n);
}
static double getVarDPiOrt(final int n,final double [] psi,final double [] QQt,final double [] QQpi,final VolMatrix CAL_Dpsi)
{
    final double [] Dpsi=CAL_Dpsi.getDPsi(psi),
	 Dpsi_no_psi0=TMatr.sumAcB(Dpsi,-TMatr.getAMatrB(psi,Dpsi,QQt,n),psi);
    return TMatr.getAMatrB(psi,Dpsi_no_psi0,QQpi,n);
}
static double getVarPiBasisProj(final int n,final double [] psiA,final double [] basis,final double [] QQt,final double [] QQpi,final VolMatrix CAL_Dpsi)
{
    final double [] DpsiA=CAL_Dpsi.getDPsi(psiA);
    double s=0;
    for(int k=0+1;k<n;k++)
    {
	final double [] psiBk=EVXData.getPsiFromBasis(k,basis,n);
	s+=TMatr.getAMatrB(psiA,TMatr.sumAcB(psiBk,-TMatr.getAMatrB(psiBk,psiA,QQt,n),psiA),QQpi,n)*TMatr.getAMatrB(psiBk,DpsiA,QQt,n);
    }
    return s;
}
static double getCommutatorDpsiABmBApsi(final int n,final double [] IQQt,final double [] QQa,final double [] QQb,final double [] psi,final VolMatrix CAL_Dpsi)
{
    final double [] matr=TMatr.sumAcB(
		    TMatr.mulMatrices(n,new double [][] {QQa,IQQt,QQb}),
		    -1,
		    TMatr.mulMatrices(n,new double [][] {QQb,IQQt,QQa})),
	Dpsi=CAL_Dpsi.getDPsi(psi);
    return TMatr.getAMatrB(Dpsi,psi,matr,n);
}

double [] getBasis(final int n,final double [] imom,final double [] tmom)
{
    return EdComm.getIRadauBasisPsi0(n,imom,tmom,this.M);
}
double getDistance2(final double [] a,final double [] b){return getK2(a,a,b,b)-getK2(b,b,a,a);}

final double [] ONE=M.B.getONE();
double getDistanceNo2(final double [] a,final double [] b){return getK2(a,ONE,b,ONE)-getK2(b,ONE,a,ONE);}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getPLast();
abstract double getPOffset();
abstract long getTLast();
abstract long getSVol();
abstract long getVol();
abstract double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb);
abstract double [] getEDPsi(final double [] psi);

static abstract class UkQVMTest<DTYPE> extends QTest {

abstract UkQVM<DTYPE> getUkQVM();


UkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_getvPsifPsi()
{
    final UkQVM<DTYPE> q=getUkQVM();
    UnitTests.checkDoubleRel(q.var_p_IH,"q.var_p_IH",q.Is_p_IH,"q.Is_p_IH",eps);
    UnitTests.checkDoubleRel(q.IM_p_IH+q.IL_p_IH,"q.IM_p_IH+q.IL_p_IH",q.Is_p_IH,"q.Is_p_IH",eps);
    UnitTests.checkDoubleRel(q.IM_p_IL+q.IL_p_IL,"q.IM_p_IL+q.IL_p_IL",q.Is_p_IL,"q.Is_p_IL",eps);

    final int n=neq,nmoms=2*n-1;
    final double [][] mm=q.getMoments(nmoms);
    final double [] QQt=q.M.B.getQQMatr(n,mm[0]),QQpi=q.M.B.getQQMatr(n,mm[9]),QQi=q.M.B.getQQMatr(n,mm[8]);

    final double [] QQD=q.VOLATILITY_EDPsi.getDiffMatrix(n,QQpi);
    UnitTests.checkDoubleRel(q.var_p_IH,"q.var_p_IH",0.5*(TMatr.getAMatrA(q.IT.psiH,QQD,n)-TMatr.getAMatrA(q.IT.psiH,QQpi,n)*Math.pow(q.M.B.getD0(q.IT.psiH,q.IT.x0),2)),"(<psiH|QQD|psiH>-<psiH|QQpi|psiH>*psiH(x0)^2)/2",eps);
    UnitTests.checkDoubleRel(q.Is_p_IL,"q.Is_p_IL",0.5*(TMatr.getAMatrA(q.IT.psiL,QQD,n)-TMatr.getAMatrA(q.IT.psiL,QQpi,n)*Math.pow(q.M.B.getD0(q.IT.psiL,q.IT.x0),2)),"(<psiL|QQD|psiL>-<psiL|QQpi|psiL>*psiL(x0)^2)/2",eps);
    final DynVHPnL d=DynVHPnL.get_DynVHPnL(n,QQi,QQt,QQpi,q.M,q.VOLATILITY_EDPsi);
    UnitTests.checkDoubleRel(q.var_p_IH,"q.var_p_IH",d.var_p_IH,"d.var_p_IH",eps);

    final double [] QQtD=TMatr.mulMatrices(n,new double [][]{QQt,q.VOLATILITY_EDPsi.getDMatrix(n)});
    TMatr.scaleVectorInPlace(TMatr.selectAntiHermitianPartInPlace(n,QQtD),2);
    final double [] QQpiD=TMatr.mulMatrices(n,new double [][]{QQpi,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),QQtD});
    //System.err.println("QQtD="+java.util.Arrays.toString(QQtD));
    UnitTests.checkDoubleRel(q.Iae_p_IH,"q.Iae_p_IH",TMatr.getAMatrA(q.IT.psiH,QQpiD,n),"<psiH|QQpi QQtd |psiH>",eps);
    UnitTests.checkDoubleRel(q.Iae_p_IL,"q.Iae_p_IL",TMatr.getAMatrA(q.IT.psiL,QQpiD,n),"<psiH|QQpi QQtd |psiH>",eps);

    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,q.M);
    final double [] psi0L=EVXData.getPsiFromBasis(1,basis,n),psi0H=EVXData.getPsiFromBasis(n-1,basis,n);
    UnitTests.checkDoubleRel(q.IT_IT0_p_IH0_IH1,"q.IT_IT0_p_IH0_IH1",getVarPi(n,psi0H,q.IT.psiH,QQt,QQpi,q.VOLATILITY_EDPsi),"getVarPsi",eps);
    UnitTests.checkDoubleRel(q.IT_IT0_p_IL0_IH1,"q.IT_IT0_p_IL0_IH1",getVarPi(n,psi0L,q.IT.psiH,QQt,QQpi,q.VOLATILITY_EDPsi),"getVarPsi",eps);
    UnitTests.checkDoubleRel(q.IT_IT0_p_IH0_IL1,"q.IT_IT0_p_IH0_IL1",getVarPi(n,psi0H,q.IT.psiL,QQt,QQpi,q.VOLATILITY_EDPsi),"getVarPsi",eps);
    UnitTests.checkDoubleRel(q.IT_IT0_p_IL0_IL1,"q.IT_IT0_p_IL0_IL1",getVarPi(n,psi0L,q.IT.psiL,QQt,QQpi,q.VOLATILITY_EDPsi),"getVarPsi",eps);

    UnitTests.checkDoubleRel(q.var_p_IH,"q.var_p_IH",getVarDPiOrt(n,q.IT.psiH,QQt,QQpi,q.VOLATILITY_EDPsi),"test",eps);

    double s=0;
    for(int k=1;k<n;k++)
    {
	s+=getVarPi(n,EVXData.getPsiFromBasis(k,basis,n),q.IT.psiH,QQt,QQpi,q.VOLATILITY_EDPsi);
    }
    UnitTests.checkDoubleRel(q.var_p_IH_Proj,"q.var_p_IH_Proj",s,"from getVarPsi",eps);


    UnitTests.checkDoubleRel(q.var_pI_I0,"q.var_pI_I0",0.5*TMatr.getAMatrB(getPsiPert0Skipped(n,basis,0,QQpi,false),EVXData.getPsiFromBasis(0,basis,n),QQtD,n),"test var_pI_I0",eps);
    UnitTests.checkDoubleRel(q.var_I_I0,"q.var_I_I0",0.5*TMatr.getAMatrB(getPsiPert0Skipped(n,basis,0,QQi,false),EVXData.getPsiFromBasis(0,basis,n),QQtD,n),"test var_I_I0",eps);
    UnitTests.checkDoubleRel(q.var_pI_I0IH,"q.var_pI_I0IH",getVarPiOrt(n,q.IT.psi0,q.IT.psiH,QQt,QQpi,q.VOLATILITY_EDPsi),"test var_pI_I0IH",eps);
    UnitTests.checkDoubleRel(q.var_I_I0IH,"q.var_I_I0IH",getVarPiOrt(n,q.IT.psi0,q.IT.psiH,QQt,QQi,q.VOLATILITY_EDPsi),"test var_I_I0IH",eps);



    System.err.println(name+":UkQVMTest:test_getvPsifPsi(): test OK neq="+neq+" eps="+eps);
}

void test_Common()
{
    final int n=neq,nmoms=2*n-1;
    final UkQVM<DTYPE> q=getUkQVM();
    UnitTests.checkDoubleRel(q.F_0,"q.F_0",q.VS.s0,"q.VS.s0",eps);

    final double [][] mm=q.getMoments(nmoms);
    final EVXData PA=BQVM.getEVXData0Removed(n,mm[5],mm[4],q.getAccess(),q.VS.x0);
    UnitTests.checkDoubleRel(q.PA_L,"q.PA_L",PA.sL,"PA.sL",eps);
    UnitTests.checkDoubleRel(q.PA_H,"q.PA_H",PA.sH,"PA.sH",eps);

    System.err.println(name+":UkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}


void test_Extra(){}

abstract TkQVM<?> getTkQVM();
void test_TkQVM()
{
    final TkQVM<?> s=getTkQVM();
    final UkQVM<DTYPE> q=getUkQVM();

    final int n=neq,nmoms=2*n-1;
    UnitTests.checkDoubleRel(q.Idp_0,"q.Idp_0",s.Ji.s0,"s.Ji.s0",eps);


    if(true)
    {
    final double [][] mm=q.getMoments(nmoms);
    final double [] QQt=q.M.B.getQQMatr(n,mm[0]),
	QQi=q.M.B.getQQMatr(n,mm[8]),QQdp=q.M.B.getQQMatr(n,mm[10]),
	IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	QQj=TMatr.mulMatrices(n,new double [][]{QQdp,IQQt,QQi});
    TMatr.selectHermitianPartInPlace(n,QQj);
    final double [][] QQ=BQVM.getQQ0RemovedA(n,new double [][]{QQj,QQt},q.getAccess(),q.Th.x0);
    final EVXData e=EVXData.getEVXDataFromQQ(n-1,QQ[0],QQ[1],q.M);
    UnitTests.checkDoubleRel(q.Idp_L,"q.Idp_L",e.sL,"e.sL",eps);
    UnitTests.checkDoubleRel(q.Idp_H,"q.Idp_H",e.sH,"e.sH",eps);
    }

    System.err.println(name+":UkQVMTest:test_TkQVM(): test OK neq="+neq+" eps="+eps);
}

void test_PsiPert()
{
    final UkQVM<DTYPE> q=getUkQVM();
    final int n=neq,nmoms=2*n-1,nd=n-1;
    final double [][] mm=q.getMoments(nmoms);

    final double [] QQt=q.M.B.getQQMatr(nd,q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[0]))),
	QQi=q.M.B.getQQMatr(nd,q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[8]))),
	QQpi=q.M.B.getQQMatr(nd,q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[9])));

    final EVXData e=EVXData.getEVXDataFromQQ(nd,QQi,QQt,q.M);
    final double [] pert=q.M.getRadauMultiplied(DeprecatedHelperEVXData.getPsiPertr(e,nd-1,QQpi));
    //System.err.println("pert="+java.util.Arrays.toString(pert)+"\tq.psipert="+java.util.Arrays.toString(q.psipert));
    UnitTests.checkDoubleArrayRel1(EVXData.getMSign(pert),"pert",EVXData.getMSign(q.psipert),"q.psipert",eps);

    System.err.println(name+":UkQVMTest:test_PsiPert(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":UkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_TkQVM();
    test_PsiPert();
    test_getvPsifPsi();
    test_Extra();

    System.err.println(name+":UkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":UkQVMTest: starting tests; ver=$Id: UkQVM.java,v 1.105 2022/11/20 12:41:33 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":UkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: UkQVM.java,v 1.105 2022/11/20 12:41:33 mal Exp $");
}
} // END OF class UkQVMTest




}// END OF class UkQVM
