package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: UkQVMLaguerre.java,v 1.10 2022/07/10 10:41:23 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class UkQVMLaguerre extends UkQVM<QVMDataL> {
UkQVMLaguerre(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}


public static UkQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final UkQVMLaguerre ref)
{
    final int npm=4;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*(neq-1)+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new UkQVMLaguerre(neq,o,ref!=null?ref.sst:null);
}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
long getTLast(){return odata.currenttime;}
long getSVol(){return odata.svol;}
long getVol(){return odata.vol;}
double getPOffset(){return odata.p_offset;}
double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return KkQVMLaguerre._getK2(a,aa,b,bb);}
double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}

static class UkQVMLaguerreTest extends UkQVMTest<QVMDataL> {
UkQVMLaguerreTest(final long initrandom)
{
    super("UkQVMLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
UkQVMLaguerre getUkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
TkQVMLaguerre getTkQVM(){return TkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class UkQVMLaguerreTest

public static void main(String [] args)
{
    new UkQVMLaguerreTest(1951117L).runTests(5);
} // END main



} // END OF class UkQVMLaguerre
