package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: UkQVMMonomials.java,v 1.19 2022/07/10 10:41:23 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class UkQVMMonomials extends UkQVM<QVMData> {
UkQVMMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static UkQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final UkQVMMonomials ref)
{
    final int nvm=1,npm=2;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new UkQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
DeprecatedHelperQVMDataDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataDirectAccessWithBasis();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double getPOffset(){return odata.p_offset;}
long getTLast(){return odata.currenttime;}
long getSVol(){return odata.svol;}
long getVol(){return odata.vol;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getK2(final double [] a,final double [] aa,final double [] b,final double [] bb){return KkQVMMonomials._getK2(a,aa,b,bb);}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}

static class UkQVMMonomialsTest extends UkQVMTest<QVMData> {
UkQVMMonomialsTest(final long initrandom)
{
    super("UkQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
UkQVMMonomials getUkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
TkQVMMonomials getTkQVM(){return TkQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}

UkQVMLaguerre getUkQVMLaguerre(){return UkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

@Override
void test_Extra(){
    final UkQVMMonomials q=getUkQVM();
    final UkQVMLaguerre s=getUkQVMLaguerre();

    UnitTests.checkDoubleRel(q.I_0,"q.I_0",s.I_0,"s.I_0",eps);
    UnitTests.checkDoubleRel(q.I_L,"q.I_L",s.I_L,"s.I_L",eps);
    UnitTests.checkDoubleRel(q.I_H,"q.I_H",s.I_H,"s.I_H",eps);
    UnitTests.checkDoubleRel(q.f_0,"q.f_0",s.f_0,"s.f_0",eps);
    UnitTests.checkDoubleRel(q.f_L,"q.f_L",s.f_L,"s.f_L",eps);
    UnitTests.checkDoubleRel(q.f_H,"q.f_H",s.f_H,"s.f_H",eps);
    UnitTests.checkDoubleRel(q.theta_f_LH,"q.theta_f_LH",s.theta_f_LH,"s.theta_f_LH",eps);
    UnitTests.checkDoubleRel(q.theta_F_LH,"q.theta_F_LH",s.theta_F_LH,"s.theta_F_LH",eps);
    EVXData.matchEVXData(q.VS,s.VS,false,true,eps);

    UnitTests.checkDoubleRel(q.PA_L,"q.PA_L",s.PA_L,"s.PA_L",eps);
    UnitTests.checkDoubleRel(q.PA_H,"q.PA_H",s.PA_H,"s.PA_H",eps);

    UnitTests.checkDoubleRel(q.Idp_0,"q.Idp_0",s.Idp_0,"s.Idp_0",eps);
    UnitTests.checkDoubleRel(q.Idp_L,"q.Idp_L",s.Idp_L,"s.Idp_L",eps);
    UnitTests.checkDoubleRel(q.Idp_H,"q.Idp_H",s.Idp_H,"s.Idp_H",eps);

    UnitTests.checkDoubleRel(q._I_0,"q._I_0",s._I_0,"s._I_0",eps);
    UnitTests.checkDoubleRel(q._I_L,"q._I_L",s._I_L,"s._I_L",eps);
    UnitTests.checkDoubleRel(q._I_H,"q._I_H",s._I_H,"s._I_H",eps);
    UnitTests.checkDoubleRel(q._p_I_H,"q._p_I_H",s._p_I_H,"s._p_I_H",eps);
    UnitTests.checkDoubleRel(q._pI_0,"q._pI_0",s._pI_0,"s._pI_0",eps);
    UnitTests.checkDoubleRel(q._pI_L,"q._pI_L",s._pI_L,"s._pI_L",eps);
    UnitTests.checkDoubleRel(q._pI_H,"q._pI_H",s._pI_H,"s._pI_H",eps);
    UnitTests.checkDoubleRel(Math.abs(q._w_pIL_IH),"q._w_pIL_IH",Math.abs(s._w_pIL_IH),"s._w_pIL_IH",eps);
    UnitTests.checkDoubleRel(Math.abs(q._w_pIH_IH),"q._w_pIH_IH",Math.abs(s._w_pIH_IH),"s._w_pIH_IH",eps);
    UnitTests.checkDoubleRel(Math.abs(q._w_pIL_IL),"q._w_pIL_IL",Math.abs(s._w_pIL_IL),"s._w_pIL_IL",eps);
    UnitTests.checkDoubleRel(Math.abs(q._w_pIH_IL),"q._w_pIH_IL",Math.abs(s._w_pIH_IL),"s._w_pIH_IL",eps);
    UnitTests.checkDoubleRel(q.theta_pIL_pIH,"q.theta_pIL_pIH",s.theta_pIL_pIH,"s.theta_pIL_pIH",eps);


    UnitTests.checkDoubleRel(q._rI_L,"q._rI_L",s._rI_L,"s._rI_L",eps);
    UnitTests.checkDoubleRel(q._rI_H,"q._rI_H",s._rI_H,"s._rI_H",eps);
    UnitTests.checkDoubleRel(q._rI_pIL,"q._rI_pIL",s._rI_pIL,"s._rI_pIL",eps);
    UnitTests.checkDoubleRel(q._rI_pIH,"q._rI_pIH",s._rI_pIH,"s._rI_pIH",eps);
    UnitTests.checkDoubleRel(q._p_rI_H,"q._p_rI_H",s._p_rI_H,"s._p_rI_H",eps);
    UnitTests.checkDoubleRel(q._rpI_L,"q._rpI_L",s._rpI_L,"s._rpI_L",eps);
    UnitTests.checkDoubleRel(Math.abs(q._w_rpIL_rIH),"q._w_rpIL_rIH",Math.abs(s._w_rpIL_rIH),"s._w_rpIL_rIH",eps);
    UnitTests.checkDoubleRel(Math.abs(q._w_rpIH_rIH),"q._w_rpIH_rIH",Math.abs(s._w_rpIH_rIH),"s._w_rpIH_rIH",eps);
    UnitTests.checkDoubleRel(Math.abs(q._w_rpIL_rIL),"q._w_rpIL_rIL",Math.abs(s._w_rpIL_rIL),"s._w_rpIL_rIL",eps);
    UnitTests.checkDoubleRel(Math.abs(q._w_rpIH_rIL),"q._w_rpIH_rIL",Math.abs(s._w_rpIH_rIL),"s._w_rpIH_rIL",eps);

    UnitTests.checkDoubleRel(q.IT_IT0_p_IH0_IH1,"q.IT_IT0_p_IH0_IH1",s.IT_IT0_p_IH0_IH1,"s.IT_IT0_p_IH0_IH1",eps);
    UnitTests.checkDoubleRel(q.IT_IT0_p_IL0_IH1,"q.IT_IT0_p_IL0_IH1",s.IT_IT0_p_IL0_IH1,"s.IT_IT0_p_IL0_IH1",eps);
    UnitTests.checkDoubleRel(q.IT_IT0_p_IH0_IL1,"q.IT_IT0_p_IH0_IL1",s.IT_IT0_p_IH0_IL1,"s.IT_IT0_p_IH0_IL1",eps);
    UnitTests.checkDoubleRel(q.IT_IT0_p_IL0_IL1,"q.IT_IT0_p_IL0_IL1",s.IT_IT0_p_IL0_IL1,"s.IT_IT0_p_IL0_IL1",eps);
    UnitTests.checkDoubleRel(q.var_p_IH_Proj,"q.var_p_IH_Proj",s.var_p_IH_Proj,"s.var_p_IH_Proj",eps);

    System.err.println(name+":UkQVMMonomialsTest:test_Extra(): test UkQVMLaguerre OK neq="+neq+" eps="+eps);
}

} // END of class UkQVMMonomialsTest

public static void main(String [] args)
{
    new UkQVMMonomialsTest(1117L).runTests(5);
} // END main



} // END OF class UkQVMMonomials
