package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.EVXData;

/** Book in price space.
 *  @version $Id: UkbQVMBMP.java,v 1.7 2019/10/09 09:17:32 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class UkbQVMBMP extends UkbQVM<QVMDataBMP> {

UkbQVMBMP(final int neq,final QVMDataBMP odata_buy,final QVMDataBMP odata_sell){super(neq,odata_buy,odata_sell);}

double getPOffset(final QVMDataBMP odata){return odata.p_offset;}
double getPEdge(final QVMDataBMP odata){return odata.edge!=null?odata.edge.price:Double.NaN;}
QVMDataBMPDirectAccessBook getAccess(){return new QVMDataBMPDirectAccessBook();}

public static UkbQVMBMP calculate(final int neq,
				  final long currenttime,
				  final long tau,
				  final BookItem [] buy,
				  final int buysize,
				  final BookItem [] sell,
				  final int sellsize,
				  final double delta_p_cut)
{
    final int npm=2*neq-1;
    final QVMDataBMP obuy=QVMDataBMP.calculateQVMDataBMP(npm,
							 currenttime,
							 tau,
							 buy,
							 buysize,
							 buysize>0?buy[0].price-delta_p_cut:Double.NaN),
	osell=QVMDataBMP.calculateQVMDataBMP(npm,
					     currenttime,
					     tau,
					     sell,
					     sellsize,
					     sellsize>0?sell[0].price+delta_p_cut:Double.NaN);
    return new UkbQVMBMP(neq,obuy,osell);
}


static class UkbQVMBMPTest extends UkbQVMTest<QVMDataBMP> {
UkbQVMBMPTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
long getTau()
{
    final long a=(tbuy[tbuy.length-1].time-tbuy[0].time),
	b=(tsell[tsell.length-1].time-tsell[0].time);
    return Math.max(a,b)/24;
}
double getPTauEstimate(final Trade [] data,final double p_edge,final int mom)
{
    final int neq_t=2,neq_p=neq,nmoms=2*neq_p-1;
    final double [] vmom=new double [nmoms],tvmom=new double [nmoms];
    final double doffset;
    if(mom==QVMDataBMPDirectAccessBook.M_V)
    {
	final double [] t=new double [neq_t];
	final QVMDataL o=QVMDataL.calculateQVMDataL(neq_t,
						    nmoms,
						    currenttime,
						    tau,
						    data,
						    data.length,
						    null);
	final DeprecatedHelperQVMDataLDirectAccessWithBasis access=new DeprecatedHelperQVMDataLDirectAccessWithBasis();
	for(int k=0;k<nmoms;k++)
	{
	    access.setM(t,k,access.M_I,o,neq_t);
	    vmom[k]=t[0];
	    tvmom[k]=-(t[1]-t[0]); // x=-(L_1(x)-L_0(x))
	}
	doffset=(p_edge-o.p_offset);
    } else if(mom==QVMDataBMPDirectAccessBook.M_EV){
	final double [] t=new double [neq_t];
	final QVMDataP o=QVMDataP.calculateQVMDataP(neq_t,
						    nmoms,
						    currenttime,
						    tau,
						    data,
						    data.length,
						    null);
	final DeprecatedHelperQVMDataPDirectAccessWithBasis access=new DeprecatedHelperQVMDataPDirectAccessWithBasis();
	for(int k=0;k<nmoms;k++)
	{
	    access.setM(t,k,access.M_I,o,neq_t);
	    vmom[k]=t[0];
	    tvmom[k]=-(t[1]-t[0])/2; // (1-x)=-(P_1(x)-P_0(x))/2
	}
	doffset=(p_edge-o.p_offset);
    }
    else throw new IllegalArgumentException("mom="+mom);
    final OrthogonalPolynomialsMonomialsBasisX M=new OrthogonalPolynomialsMonomialsBasisX(1,-doffset);
    final EVXData e=EVXData.getEVXDataFromQQ(neq_p,M.B.getQQMatr(neq_p,tvmom),M.B.getQQMatr(neq_p,vmom),M);
    //System.err.println("e.x0="+e.x0+"\tp_edge-o.p_offset="+(p_edge-o.p_offset)+"\tp_edge="+p_edge);
    return e.s0;
}

void test_Extra(){
    final UkbQVMBMP q=getUkbQVM();

    final double tau_buy=getPTauEstimate(tbuy,q.p_buy,q.MOM_TYPE);
    UnitTests.checkDoubleRel(tau_buy,"tau_buy",q.tau_buy,"q.tau_buy",eps);
    final double tau_sell=getPTauEstimate(tsell,q.p_sell,q.MOM_TYPE);
    UnitTests.checkDoubleRel(tau_sell,"tau_sell",q.tau_sell,"q.tau_sell",eps);

    System.err.println(name+":UkbQVMBMBTest:test_Extra(): test OK n="+neq+" eps="+eps);
}
UkbQVMBMP getUkbQVM(){return calculate(neq,currenttime,tau,tbuy_byp,tbuy_byp.length,tsell_byp,tsell_byp.length,Double.NaN);}

} // END of class UkbQVMBMPTest

public static void main(String [] args)
{
    new UkbQVMBMPTest("UkbQVMBMP").runTests(5);
} // END main



} // END of class UkbQVMBMP
