package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: UnQVMMonomials.java,v 1.9 2022/07/10 10:41:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class UnQVMMonomials extends UnQVM<QVMData> {
UnQVMMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static UnQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final UnQVMMonomials ref)
{
    final int nvm=1,npm=2;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new UnQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
DeprecatedHelperQVMDataDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataDirectAccessWithBasis();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getPOffset(){return odata.p_offset;}
long getTLast(){return odata.currenttime;}
long getVolume(){return odata.vol;}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}

static class UnQVMMonomialsTest extends UnQVMTest<QVMData> {
UnQVMMonomialsTest(final long initrandom)
{
    super("UnQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
UnQVMMonomials getUnQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
UkQVMMonomials getUkQVM(){return UkQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}

UnQVMLaguerre getUnQVMLaguerre(){return UnQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

@Override
void test_Extra(){
    final UnQVMMonomials q=getUnQVM();
    final UnQVMLaguerre s=getUnQVMLaguerre();

    UnitTests.checkDoubleRel(q.I_0,"q.I_0",s.I_0,"s.I_0",eps);
    UnitTests.checkDoubleRel(q.I_L,"q.I_L",s.I_L,"s.I_L",eps);
    UnitTests.checkDoubleRel(q.I_H,"q.I_H",s.I_H,"s.I_H",eps);
    UnitTests.checkDoubleRel(q.p_I_H,"q.p_I_H",q.p_I_H,"q.p_I_H",eps);
    UnitTests.checkDoubleRel(Math.abs(q.w_piL_IH),"q.w_piL_IH",Math.abs(s.w_piL_IH),"s.w_piL_IH",eps);
    UnitTests.checkDoubleRel(Math.abs(q.w_piH_IH),"q.w_piH_IH",Math.abs(s.w_piH_IH),"s.w_piH_IH",eps);

    System.err.println(name+":UnQVMMonomialsTest:test_Extra(): test UnQVMLaguerre OK neq="+neq+" eps="+eps);
}

} // END of class UnQVMMonomialsTest

public static void main(String [] args)
{
    new UnQVMMonomialsTest(1117L).runTests(5);
} // END main



} // END OF class UnQVMMonomials
