package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.PolynomialsRatio;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.realtimebook.*;
import com.polytechnik.outils.*;

/** VQVM and a set of measures.
 *  @version $Id: VQVM.java,v 1.118 2022/08/08 20:10:47 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class VQVM<DTYPE> {
private static final boolean FLAG_exact_QQt=true,FLAG_direct_QQt=true,FLAG_QQa=true;

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final double w00;
final TwoCloseRootsFinder.RootsData rv,rvd,rp,rpd,ra;
final XEVData rvd0_pv,rvd1_pv,rvd2_pv;
final XEVData rv0_pv,rv1_pv,rv2_pv;
final XEVData rpd0_pv,rpd1_pv,rpd2_pv;
final XEVData rp0_pv,rp1_pv,rp2_pv;
final XEVData ra0,ra1,ra2;

/** Rurik version. Started on Jul, 7 2012. */
final double [] kkv,kkdpv,kkdp2v,kkav,kka2v;
final double Kv00,j0eq,sigma2_j0eq,joeq,sigma2_joeq,a0eq,sigma2_a0eq,aoeq,sigma2_aoeq,
    js0eq,sigma2_js0eq,jt0eq,sigma2_jt0eq,

    jsoeq,sigma2_jsoeq,jtoeq,sigma2_jtoeq;

final double [] kkpv,kkp2v,rnodes,kkt,kkpt,kkp2t;
final double pv0eq,sigma2_pv0eq,pt0eq,sigma2_pt0eq,pvoeq,sigma2_pvoeq;
final double [] xvnodes,pvnodes,sigma2_pvnodes,pp,ptnodes,sigma2_ptnodes;
final double [] kke,kkpe,kkp2e,enodes,xenodes,penodes,sigma2_penodes;
final double pe0eq,sigma2_pe0eq;

final double odp0,sigma2_odp0,ods0,sigma2_ods0,odts0,sigma2_odts0,odvs0,sigma2_odvs0,oK00;
final double ndp0,sigma2_ndp0,nds0,sigma2_nds0,ndts0,sigma2_ndts0,ndvs0,sigma2_ndvs0,nK00;
final double rdts0,sigma2_rdts0,rdvs0,sigma2_rdvs0;
final double [] kka,kkdvse,dvse;
final double dvse0,min_dvse,max_dvse;
final double dp0_psi,sigma2_dp0_psi,i_psi,sigma2_i_psi,vs_psi,sigma2_vs_psi;

VQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final int nmoms=2*neq-1;

    final double [] vmom=new double [nmoms],pvmom=new double [nmoms],p2vmom=new double [nmoms],imom=new double [nmoms];
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(vmom,0,access.M_I,odata,nmoms);
    w00=vmom[0];
    access.setM(pvmom,1,access.M_I,odata,nmoms);
    access.setM(p2vmom,2,access.M_I,odata,nmoms);
    for(int k=nmoms;--k>=0;){vmom[k]/=w00;pvmom[k]/=w00;p2vmom[k]/=w00;}
    final double [] tmom=new double [nmoms],ptmom=new double [nmoms],p2tmom=new double [nmoms],temom=new double [nmoms];
    access.setM(tmom,0,access.M_T,odata,nmoms);
    access.setM(ptmom,1,access.M_T,odata,nmoms);
    access.setM(p2tmom,2,access.M_T,odata,nmoms);
    final double [] dpmom=new double [nmoms];
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setMT(temom,nmoms);
    final double [] amom=new double [nmoms],pamom=new double [nmoms],p2amom=new double [nmoms];
    access.setM(amom,0,access.M_A,odata,nmoms);
    access.setM(pamom,1,access.M_A,odata,nmoms);
    access.setM(p2amom,2,access.M_A,odata,nmoms);

    final double [] dsmom=new double [nmoms],dtsmom=new double [nmoms],dvsmom=new double [nmoms];
    access.setM(dsmom,0,access.M_ISELL,odata,nmoms);
    access.setM(dtsmom,0,access.M_IBUY,odata,nmoms);
    for(int k=0;k<nmoms;k++) dsmom[k]=(dsmom[k]-dtsmom[k])/w00;
    access.setM(dtsmom,0,access.M_TS,odata,nmoms);
    access.setM(dvsmom,0,access.M_VS,odata,nmoms);

    final double [][] QQ=access.getQQ_Matrs(neq,1,new double [][]{vmom,pvmom,p2vmom,tmom,ptmom,p2tmom,temom,dpmom,amom,pamom,p2amom,dsmom,dtsmom,dvsmom});
    final double [] QQv=QQ[0],QQpv=QQ[1],QQp2v=QQ[2],QQpt=QQ[4],QQp2t=QQ[5],QQa=QQ[8],QQpa=QQ[9],QQp2a=QQ[10];

    if(true)
    {
	final double [] res=getdPAndSigma_PSI(neq,getX0(),vmom,1,new double [][]{dpmom,dsmom,dtsmom,dvsmom});
	odp0=res[0];
	sigma2_odp0=res[1];
	ods0=res[2];
	sigma2_ods0=res[3];
	odts0=res[4];
	sigma2_odts0=res[5];
	odvs0=res[6];
	sigma2_odvs0=res[7];
	oK00=res[8];
    }
    if(true)
    {
	final double [] res=getdPAndSigma_PSI2(neq,getX0(),vmom,1,new double [][]{dpmom,dsmom,dtsmom,dvsmom});
	ndp0=res[0];
	sigma2_ndp0=res[1];
	nds0=res[2];
	sigma2_nds0=res[3];
	ndts0=res[4];
	sigma2_ndts0=res[5];
	ndvs0=res[6];
	sigma2_ndvs0=res[7];
	nK00=res[8];
    }
    if(true)
    {
	final double [] res=getdPAndSigma_PSI(neq,getX0(),temom,1,new double [][]{dtsmom,dvsmom});
	rdts0=res[0];
	sigma2_rdts0=res[1];
	rdvs0=res[2];
	sigma2_rdvs0=res[3];
    }
    if(true)
    {
	final double [] res=getdPAndSigma_PSI2(neq,getX0(),temom,1,new double [][]{dpmom,imom,dvsmom});
	dp0_psi=res[0];
	sigma2_dp0_psi=res[1]+dp0_psi*dp0_psi;
	i_psi=res[2];
	sigma2_i_psi=res[3]+i_psi*i_psi;
	vs_psi=res[4];
	sigma2_vs_psi=res[5]+vs_psi*vs_psi;
    }


    if(FLAG_QQa)
    {
    final double []
	m_kkdenom=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQv),
	m_pkkdenom=TMatr.mulMMM(neq,m_kkdenom,QQpv,m_kkdenom),
	m_kknom=TMatr.mulMMM(neq,m_kkdenom,QQa,m_kkdenom),
	m_pkknom=TMatr.mulMMM(neq,m_kkdenom,QQpa,m_kkdenom);

    final double [][] kks=access.getKK(neq,new double [][] {m_kkdenom,m_kknom,m_pkkdenom,m_pkknom});
    this.ra=getTR(kks[0],kks[1]);
    ra0=getXEVData(neq,ra.x0,getPsi(neq,m_kkdenom,ra.x0),QQa,QQpa,QQp2a,QQv,QQpv,QQp2v,kks[0],kks[1],kks[2],kks[3]);
    ra1=getXEVData(neq,ra.x1,getPsi(neq,m_kkdenom,ra.x1),QQa,QQpa,QQp2a,QQv,QQpv,QQp2v,kks[0],kks[1],kks[2],kks[3]);
    ra2=getXEVData(neq,ra.x2,getPsi(neq,m_kkdenom,ra.x2),QQa,QQpa,QQp2a,QQv,QQpv,QQp2v,kks[0],kks[1],kks[2],kks[3]);
    }
    else
    {
	this.ra=null;ra0=ra1=ra2=null;
    }

    if(FLAG_direct_QQt){
    final double [] QQt=QQ[3];
    final double [] IQQt_denom=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQt),
	IQQt_nom=TMatr.mulMMM(neq,IQQt_denom,QQv,IQQt_denom),
	IQQt_ptnom=TMatr.mulMMM(neq,IQQt_denom,QQpt,IQQt_denom),
	IQQt_pvnom=TMatr.mulMMM(neq,IQQt_denom,QQpv,IQQt_denom);
    final double [][] kks=access.getKK(neq,new double [][] {IQQt_denom,IQQt_nom,IQQt_ptnom,IQQt_pvnom});
    this.rvd=getTR(kks[0],kks[1]);
    rvd0_pv=getXEVData(neq,rvd.x0,getPsi(neq,IQQt_denom,rvd.x0),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);
    rvd1_pv=getXEVData(neq,rvd.x1,getPsi(neq,IQQt_denom,rvd.x1),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);
    rvd2_pv=getXEVData(neq,rvd.x2,getPsi(neq,IQQt_denom,rvd.x2),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);

    this.rpd=getTRP(kks[0],kks[1],kks[2],kks[3]);
    rpd0_pv=getXEVData(neq,rpd.x0,getPsi(neq,IQQt_denom,rpd.x0),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);
    rpd1_pv=getXEVData(neq,rpd.x1,getPsi(neq,IQQt_denom,rpd.x1),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);
    rpd2_pv=getXEVData(neq,rpd.x2,getPsi(neq,IQQt_denom,rpd.x2),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);
    } else {
	this.rvd=null;rvd0_pv=rvd1_pv=rvd2_pv=null;
	this.rpd=null;rpd0_pv=rpd1_pv=rpd2_pv=null;
    }
    if(FLAG_exact_QQt){
	final double [] QQt=QQ[6],IQQt_denom,IQQt_nom,IQQt_ptnom,IQQt_pvnom;
    if(false) {
	IQQt_denom=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQt);
	IQQt_nom=TMatr.mulMMM(neq,IQQt_denom,QQv,IQQt_denom);
	IQQt_ptnom=TMatr.mulMMM(neq,IQQt_denom,QQpt,IQQt_denom);
	IQQt_pvnom=TMatr.mulMMM(neq,IQQt_denom,QQpv,IQQt_denom);
    } else {
	IQQt_denom=getDenomQQtDiag(neq,QQt);
	IQQt_nom=getNomQQtDiag(neq,IQQt_denom,QQv);
	IQQt_ptnom=getNomQQtDiag(neq,IQQt_denom,QQpt);
	IQQt_pvnom=getNomQQtDiag(neq,IQQt_denom,QQpv);
    }
    final double [][] kks=access.getKK(neq,new double [][] {IQQt_denom,IQQt_nom,IQQt_ptnom,IQQt_pvnom});
    this.rv=getTR(kks[0],kks[1]);
    rv0_pv=getXEVData(neq,rv.x0,getPsi(neq,IQQt_denom,rv.x0),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);
    rv1_pv=getXEVData(neq,rv.x1,getPsi(neq,IQQt_denom,rv.x1),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);
    rv2_pv=getXEVData(neq,rv.x2,getPsi(neq,IQQt_denom,rv.x2),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);

    this.rp=getTRP(kks[0],kks[1],kks[2],kks[3]);
    rp0_pv=getXEVData(neq,rp.x0,getPsi(neq,IQQt_denom,rp.x0),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);
    rp1_pv=getXEVData(neq,rp.x1,getPsi(neq,IQQt_denom,rp.x1),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);
    rp2_pv=getXEVData(neq,rp.x2,getPsi(neq,IQQt_denom,rp.x2),QQv,QQpv,QQp2v,QQt,QQpt,QQp2t,kks[0],kks[1],kks[2],kks[3]);
    } else {
	this.rv=null;rv0_pv=rv1_pv=rv2_pv=null;
	this.rp=null;rp0_pv=rp1_pv=rp2_pv=null;
    }


    if(true)
    {
	final double [] QQdp=QQ[7],QQds=QQ[11],QQdts=QQ[12];
	final double [] IQQv=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQv),
	    IQQv_x_QQdp=TMatr.mulMM(neq,IQQv,QQdp),
	    IQQv_x_QQdp_x_IQQv=TMatr.mulMM(neq,IQQv_x_QQdp,IQQv),
	    IQQv_x_QQdp_IQQv_x_QQdp_x_IQQv=TMatr.mulMM(neq,IQQv_x_QQdp,IQQv_x_QQdp_x_IQQv),
	    IQQv_x_QQa=TMatr.mulMM(neq,IQQv,QQa),
	    IQQv_x_QQa_x_IQQv=TMatr.mulMM(neq,IQQv_x_QQa,IQQv),
	    IQQv_x_QQa_IQQv_x_QQa_x_IQQv=TMatr.mulMM(neq,IQQv_x_QQa,IQQv_x_QQa_x_IQQv);

	final double []  IQQv_x_QQpv_x_IQQv=TMatr.mulMMM(neq,IQQv,QQpv,IQQv),
	    IQQv_x_QQp2v_x_IQQv=TMatr.mulMMM(neq,IQQv,QQp2v,IQQv),
	    QQt=QQ[6/*3*/],
	    IQQv_x_QQt_x_IQQv=TMatr.mulMMM(neq,IQQv,QQt,IQQv),
	    IQQv_x_QQpt_x_IQQv=TMatr.mulMMM(neq,IQQv,QQpt,IQQv),
	    IQQv_x_QQp2t_x_IQQv=TMatr.mulMMM(neq,IQQv,QQp2t,IQQv);

	final double [] IQQv_x_QQds=TMatr.mulMM(neq,IQQv,QQds),
	    IQQv_x_QQds_x_IQQv=TMatr.mulMM(neq,IQQv_x_QQds,IQQv),
	    IQQv_x_QQds_IQQv_x_QQds_x_IQQv=TMatr.mulMM(neq,IQQv_x_QQds,IQQv_x_QQds_x_IQQv),
	    IQQv_x_QQdt=TMatr.mulMM(neq,IQQv,QQdts),
	    IQQv_x_QQdt_x_IQQv=TMatr.mulMM(neq,IQQv_x_QQdt,IQQv),
	    IQQv_x_QQdt_IQQv_x_QQdt_x_IQQv=TMatr.mulMM(neq,IQQv_x_QQdt,IQQv_x_QQdt_x_IQQv);

	final double [][] kks=access.getKK(neq,new double [][] {IQQv,IQQv_x_QQdp_x_IQQv,IQQv_x_QQdp_IQQv_x_QQdp_x_IQQv,IQQv_x_QQa_x_IQQv,IQQv_x_QQa_IQQv_x_QQa_x_IQQv,IQQv_x_QQpv_x_IQQv,IQQv_x_QQp2v_x_IQQv,IQQv_x_QQt_x_IQQv,IQQv_x_QQpt_x_IQQv,IQQv_x_QQp2t_x_IQQv,IQQv_x_QQds_x_IQQv,IQQv_x_QQds_IQQv_x_QQds_x_IQQv,IQQv_x_QQdt_x_IQQv,IQQv_x_QQdt_IQQv_x_QQdt_x_IQQv});
	kkv=kks[0];
	kkdpv=kks[1];
	kkdp2v=kks[2];
	kkav=kks[3];
	kka2v=kks[4];
	final double x0=getX0();
	Kv00=access.fx(kkv,x0);
	j0eq=access.fx(kkdpv,x0)/Kv00;
	sigma2_j0eq=access.fx(kkdp2v,x0)/Kv00-j0eq*j0eq;
	final double [] kkdsv=kks[10],kkds2v=kks[11],kkdtsv=kks[12],kkdts2v=kks[13];
	js0eq=access.fx(kkdsv,x0)/Kv00;
	sigma2_js0eq=access.fx(kkds2v,x0)/Kv00-js0eq*js0eq;
	jt0eq=access.fx(kkdtsv,x0)/Kv00;
	sigma2_jt0eq=access.fx(kkdts2v,x0)/Kv00-jt0eq*jt0eq;

	final double KKo=kkv[kkv.length-1];
	joeq=kkdpv[kkdpv.length-1]/KKo;
	sigma2_joeq=kkdp2v[kkdp2v.length-1]/KKo-joeq*joeq;
	jsoeq=kkdsv[kkdsv.length-1]/KKo;
	sigma2_jsoeq=kkds2v[kkds2v.length-1]/KKo-jsoeq*jsoeq;
	jtoeq=kkdtsv[kkdtsv.length-1]/KKo;
	sigma2_jtoeq=kkdts2v[kkdts2v.length-1]/KKo-jtoeq*jtoeq;


	a0eq=access.fx(kkav,x0)/Kv00;
	sigma2_a0eq=access.fx(kka2v,x0)/Kv00-a0eq*a0eq;
	aoeq=kkav[kkav.length-1]/KKo;
	sigma2_aoeq=kka2v[kka2v.length-1]/KKo-aoeq*aoeq;

	final double p_offset=getPOffset();
	kkpv=kks[5];
	kkp2v=kks[6];
	kkt=kks[7];
	kkpt=kks[8];
	kkp2t=kks[9];
	{
	final double dpv0=access.fx(kkpv,x0)/Kv00,dpvo=kkpv[kkpv.length-1]/kkv[kkv.length-1];
	pv0eq=p_offset+dpv0;
	sigma2_pv0eq=access.fx(kkp2v,x0)/Kv00-dpv0*dpv0;
	pvoeq=p_offset+dpvo;
	sigma2_pvoeq=kkp2v[kkp2v.length-1]/kkv[kkv.length-1]-dpvo*dpvo;
	}
	{
	final double Kt00=access.fx(kkt,x0),dpt0=access.fx(kkpt,x0)/Kt00;
	pt0eq=p_offset+dpt0;
	sigma2_pt0eq=access.fx(kkp2t,x0)/Kt00-dpt0*dpt0;
	}

	rnodes=getPsi(neq,IQQv,x0);
	xvnodes=getXNodes(rnodes);
	pvnodes=new double [xvnodes.length];
	sigma2_pvnodes=new double [xvnodes.length];
	ptnodes=new double [xvnodes.length];
	sigma2_ptnodes=new double [xvnodes.length];
	pp=new double [xvnodes.length];

	for(int k=0;k<xvnodes.length;k++)
	{
	    final double x=xvnodes[k];
	    {
		final double K=access.fx(kkv,x),dpx=access.fx(kkpv,x)/K,sigma2=access.fx(kkp2v,x)/K-dpx*dpx;
		pvnodes[k]=p_offset+dpx;
		sigma2_pvnodes[k]=sigma2;
		pp[k]=TMatr.getAMatrB(rnodes,getPsi(neq,IQQv,x),QQpv,neq);
	    }
	    {
		final double K=access.fx(kkt,x),dpx=access.fx(kkpt,x)/K,sigma2=access.fx(kkp2t,x)/K-dpx*dpx;
		ptnodes[k]=p_offset+dpx;
		sigma2_ptnodes[k]=sigma2;
	    }
	}


	final double [] QQdvs=QQ[13];
	final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQt),
	    IQQt_x_QQv_x_IQQt=TMatr.mulMMM(neq,IQQt,QQv,IQQt),
	    IQQt_x_QQpv_x_IQQt=TMatr.mulMMM(neq,IQQt,QQpv,IQQt),
	    IQQt_x_QQp2v_x_IQQt=TMatr.mulMMM(neq,IQQt,QQp2v,IQQt),
	    IQQt_x_QQdvs_x_IQQt=TMatr.mulMMM(neq,IQQt,QQdvs,IQQt);

	final double [][] kkqe=access.getKK(neq,new double [][] {IQQt,IQQt_x_QQv_x_IQQt,IQQt_x_QQpv_x_IQQt,IQQt_x_QQp2v_x_IQQt,IQQt_x_QQdvs_x_IQQt});
	enodes=getPsi(neq,IQQt,x0);
	kka=kkqe[0];
	kke=kkqe[1];
	kkpe=kkqe[2];
	kkp2e=kkqe[3];
	kkdvse=kkqe[4];
	{
	    final double K=access.fx(kke,x0),dpe0=access.fx(kkpe,x0)/K;
	    pe0eq=p_offset+dpe0;
	    sigma2_pe0eq=access.fx(kkp2e,x0)/K-dpe0*dpe0;
	    dvse0=access.fx(kkdvse,x0)/access.fx(kka,x0);
	}
	xenodes=getXNodes(enodes);
	penodes=new double [xenodes.length];
	sigma2_penodes=new double [xenodes.length];
	dvse=new double [xenodes.length];
	double min_dd=Double.POSITIVE_INFINITY,max_dd=Double.NEGATIVE_INFINITY;
	for(int k=0;k<xenodes.length;k++)
	{
	    final double x=xenodes[k];
	    final double K=access.fx(kke,x),dpe=access.fx(kkpe,x)/K,sigma2=access.fx(kkp2e,x)/K-dpe*dpe;
	    penodes[k]=p_offset+dpe;
	    sigma2_penodes[k]=sigma2;
	    final double dd=access.fx(kkdvse,x)/access.fx(kka,x);
	    dvse[k]=dd;
	    if(dd<=min_dd) min_dd=dd;
	    if(dd>=max_dd) max_dd=dd;
	}
	min_dvse=min_dd;
	max_dvse=max_dd;
    }

}


double [] getPsi(final int n,final double [] qq,final double x0)
{
    final double [] psi=new double [n],r=new double [n];
    for(int k=0;k<n;k++)
    {
	r[k]=1;psi[k]=access.fx(r,x0);r[k]=0;
    }
    for(int l=0;l<n;l++)
    {
	double s=0;
	for(int m=0,o=l*n;m<n;m++,o++) s+=qq[o]*psi[m];
	r[l]=s;
    }
    final double s=Math.sqrt(access.fx(r,x0));
    for(int l=0;l<n;l++)
    {
	r[l]/=s;
    }
    return r;
}

double [] getPnodes(final int n, final double [] moms)
{
    final double [] rnodes=new double [n],e=new double [2*n-1];
    rnodes[n-1]=1;
    final double [] matr=new double [(n-1)*(n-1)];
    for(int k=0;k<(n-1);k++)
    {
	final int naa=access.setQQCoefs(n-1,k,e);
	rnodes[k]=-TMatr.mulVVn(naa,moms,e);
	for(int l=0;l<=k;l++)
	{
	    final int na=access.setQQCoefs(l,k,e);
	    matr[k+l*(n-1)]=matr[l+k*(n-1)]=TMatr.mulVVn(na,e,moms);
	}
    }
    final int [] ipvt=new int[n-1];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,n-1,n-1,ipvt);
    if(pinfo!=0)
    {
	java.util.Arrays.fill(rnodes,Double.NaN);
    }
    else
    {
	com.polytechnik.utils.Linsystems.dgesl_m1(matr,n-1,n-1,ipvt,rnodes,1);
    }
    return rnodes;
}

/** A shortcut in case of diagonal QQt. */
double [] getDenomQQtDiag(final int n,final double [] QQt)
{
    final double [] inv=new double [n*n];
    for(int k=0;k<n;k++) {final int o=k*n+k;inv[o]=1/QQt[o];}
    return inv;
}
/** A shortcut in case of diagonal QQt. */
double [] getNomQQtDiag(final int n,final double [] invDiagQQt,final double [] QQv)
{
    final double [] nom=new double [n*n];
    for(int k=0;k<n;k++)
    {
	final double nrm=invDiagQQt[k*n+k];
	for(int l=0,ol=k*n;l<n;l++,ol++)
	{
	    nom[ol]=QQv[ol]*(nrm*invDiagQQt[l*n+l]);
	}
    }
    return nom;
}



abstract double [] getXNodes(final double [] nodespolynomial);
abstract double getX0();
abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getPOffset();
abstract TwoCloseRootsFinder.RootsData getTR(final double [] kkdenom,final double [] kknom);
abstract TwoCloseRootsFinder.RootsData getTRP(final double [] kkdenom,final double [] kknom,final double [] pkkdenom,final double [] pkknom);

static class XEVData {
    final double x,Iok,pLok,stdev2_pLok,pRok,stdev2_pRok;
    final double [] psiok;
    final double [] kkdenom,kknom,pkkdenom,pkknom;
    XEVData(final double x,final double [] psiok,
	    final double [] QQl,final double [] QQpl,final double [] QQp2l,
	    final double [] QQr,final double [] QQpr,final double [] QQp2r,
	    final double p_offset,
	    final double [] kkdenom,
	    final double [] kknom,
	    final double [] pkkdenom,
	    final double [] pkknom)
{
    this.x=x;
    this.psiok=psiok;
    this.kkdenom=kkdenom;
    this.kknom=kknom;
    this.pkkdenom=pkkdenom;
    this.pkknom=pkknom;
    final int n=psiok.length;
    this.Iok=TMatr.getAMatrA(psiok,QQl,n);
    final double dpLok=TMatr.getAMatrA(psiok,QQpl,n)/this.Iok,
	dp2Lok=TMatr.getAMatrA(psiok,QQp2l,n)/this.Iok;
    this.pLok=dpLok+p_offset;
    this.stdev2_pLok=dp2Lok-dpLok*dpLok;
    final double dpRok=TMatr.getAMatrA(psiok,QQpr,n),
	dp2Rok=TMatr.getAMatrA(psiok,QQp2r,n);
    this.pRok=dpRok+p_offset;
    this.stdev2_pRok=dp2Rok-dpRok*dpRok;
}
}


XEVData getXEVData(final int n,final double x,final double [] psi,
		   final double [] QQl,final double [] QQpl,final double [] QQp2l,
		   final double [] QQr,final double [] QQpr,final double [] QQp2r,
		   final double [] kkdenom,final double [] kknom,
		   final double [] pkkdenom,final double [] pkknom)
{
    return new XEVData(x,psi,QQl,QQpl,QQp2l,QQr,QQpr,QQp2r,getPOffset(),kkdenom,kknom,pkkdenom,pkknom);
}


double [] getdPAndSigma_PSI(final int n,final double x0,final double [] momv,final double vscale,final double [][] momdps)
{
    final double [] QQ=access.getQQ_Matrs(n,vscale,new double [][]{momv})[0];
    final double [] psi=new double [n],r=new double [n];
    for(int k=0;k<n;k++)
    {
	r[k]=1;psi[k]=access.fx(r,x0);r[k]=0;
    }
    final int [] ipvt=new int [n];
    final double [] resdpsigma=new double [2*momdps.length+1];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(QQ,n,n,ipvt);
    if(pinfo!=0) {
	java.util.Arrays.fill(resdpsigma,Double.NaN);
	return resdpsigma;
    }
    com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,n,ipvt,psi,1);
    final double K00=access.fx(psi,x0),sqrtK00=Math.sqrt(K00);
    int o=0;
    for(int k=0;k<momdps.length;k++)
    {
	final double [] momdp=momdps[k];
	final double d=TMatr.mulVVn1(psi,momdp);
	System.arraycopy(momdp,0,r,0,n);
	com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,n,ipvt,r,1);
	final double dp0=d/sqrtK00,sigma2_dp0=TMatr.mulVVn1(r,momdp)-dp0*dp0;
	resdpsigma[o++]=dp0;
	resdpsigma[o++]=sigma2_dp0;
    }
    resdpsigma[o++]=K00;
    return resdpsigma;
}

double [] getdPAndSigma_PSI2(final int n,final double x0,final double [] momv,final double vscale,final double [][] momdps)
{
    final double [][] qqmom=new double [1+momdps.length][];
    qqmom[0]=momv;
    System.arraycopy(momdps,0,qqmom,1,momdps.length);
    final double [][] qq=access.getQQ_Matrs(n,1,qqmom);
    final double [] QQ=qq[0];
    for(int k=QQ.length;--k>=0;) QQ[k]/=vscale;

    double [] psi=new double [n],r=new double [n];
    for(int k=0;k<n;k++)
    {
	r[k]=1;psi[k]=access.fx(r,x0);r[k]=0;
    }
    final int [] ipvt=new int [n];
    final double [] resdpsigma=new double [2*momdps.length+1];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(QQ,n,n,ipvt);
    if(pinfo!=0) {
	java.util.Arrays.fill(resdpsigma,Double.NaN);
	return resdpsigma;
    }
    com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,n,ipvt,psi,1);
    final double K00=access.fx(psi,x0);
    int o=0;
    for(int k=0;k<momdps.length;k++)
    {
	final double [] QQdp=qq[k+1];
	r=TMatr.getMatr_x_A(n,QQdp,psi);
	final double d=TMatr.mulVVn1(psi,r);
	com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,n,ipvt,r,1);
	double [] f=TMatr.getMatr_x_A(n,QQdp,r);
	final double dp0=d/K00,sigma2_dp0=TMatr.mulVVn1(psi,f)/K00-dp0*dp0;
	resdpsigma[o++]=dp0;
	resdpsigma[o++]=sigma2_dp0;
    }
    resdpsigma[o++]=K00;
    return resdpsigma;
}



static abstract class VQVMTest<DTYPE> extends QTest {

abstract double [][] getMonomialMoments(DTYPE o);
abstract VQVM<DTYPE> getVQVM();
abstract UQVM<DTYPE> getUQVM();
abstract LQVM<DTYPE> getLQVM();
abstract boolean isRadauRight();


VQVMTest(final String name,
	 final java.util.Random r,final double eps)
{
    super(name,r,eps);
}
void test_Common()
{
    final VQVM<DTYPE> q=getVQVM();
    System.err.println(name+":VQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}

XEVData pol_getXEVData(final PolStoreQQtest m,final double x,final double p_offset)
{
    final int n=m.n;
    final double [] r=new double [n],tmp=new double [n];
    for(int k=0;k<n;k++){
	tmp[k]=1;r[k]=Polynomials.getD0(tmp,x);tmp[k]=0;
    }
    final double [] psi=TMatr.getMatr_x_A(n,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,m.QQr),r);
    final double nrm=Math.sqrt(Polynomials.getD0(psi,x));
    for(int k=0;k<n;k++) psi[k]/=nrm;
    final XEVData s=new XEVData(x,psi,m.QQl,m.QQpl,m.QQp2l,m.QQr,m.QQpr,m.QQp2r,p_offset,m.kkdenom,m.kknom,m.pkkdenom,m.pkknom);

    UnitTests.checkDoubleRel(TMatr.getAMatrA(s.psiok,m.QQr,n),"<q|B|q>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(s.psiok,m.QQl,n),"<q|A|q>",s.Iok,"s.Iok",eps);
    UnitTests.checkDoubleRel(Polynomials.getD0(s.kknom,s.x)/Polynomials.getD0(s.kkdenom,s.x),"kknom(x)/kkdenom(x)",s.Iok,"s.Iok",eps);
    UnitTests.checkDoubleRel(Polynomials.getD0(s.kkdenom,s.x)/Math.pow(Polynomials.getD0(s.psiok,s.x),2),"kkdenom(x)/psi(0)^2",1,"1",eps);
    UnitTests.checkDoubleRel(Polynomials.getD0(s.kknom,s.x)/Math.pow(Polynomials.getD0(s.psiok,s.x),2),"kknom(x)/psi(0)^2",s.Iok,"s.Iok",eps);

    final double kdenom=Polynomials.getD0(m.kkdenom,s.x),knom=Polynomials.getD0(m.kknom,s.x),
	pkdenom=Polynomials.getD0(m.pkkdenom,s.x),pknom=Polynomials.getD0(m.pkknom,s.x),
	p2kdenom=Polynomials.getD0(m.p2kkdenom,s.x),p2knom=Polynomials.getD0(m.p2kknom,s.x);
    UnitTests.checkDoubleRel(p_offset+pknom/knom,"pkknom(x)/kknom(x)",s.pLok,"s.pLok",eps);
    UnitTests.checkDoubleRel(p_offset+pkdenom/kdenom,"pkkdenom(x)/kkdenom(x)",s.pRok,"s.pRok",eps);
    UnitTests.checkDoubleRel(p2knom/knom-Math.pow(pknom/knom,2),"stdev2",s.stdev2_pLok,"s.stdev2_pLok",eps);
    UnitTests.checkDoubleRel(p2kdenom/kdenom-Math.pow(pkdenom/kdenom,2),"stdev2",s.stdev2_pRok,"s.stdev2_pRok",eps);

    for(int t=0;t<100;t++)
    {
	for(int k=0;k<n;k++) r[k]=1-2*this.r.nextDouble();
	UnitTests.checkDoubleRel(Polynomials.getD0(s.psiok,s.x)*TMatr.getAMatrB(s.psiok,r,m.QQr,n),"psi(x0)<psi|r>",Polynomials.getD0(r,s.x),"r(x0)",eps);
    }

    System.err.println(name+":pol_getXEVData(): test OK x="+x+" neq="+neq+" eps="+eps);
    return s;
}

void matchXEVData(final XEVData q,final XEVData s)
{
    UnitTests.checkDoubleRel(q.Iok,"q.Iok",s.Iok,"s.Iok",eps);
    UnitTests.checkDoubleRel(q.pLok,"q.pLok",s.pLok,"s.pLok",eps);
    UnitTests.checkDoubleRel(q.stdev2_pLok,"q.stdev2_pLok",s.stdev2_pLok,"s.stdev2_pLok",eps);
    UnitTests.checkDoubleRel(q.pRok,"q.pRok",s.pRok,"s.pRok",eps);
    UnitTests.checkDoubleRel(q.stdev2_pRok,"q.stdev2_pRok",s.stdev2_pRok,"s.stdev2_pRok",eps);
    double [] conv_psiok=convertToMonomials(q.psiok);
    if(Math.signum(conv_psiok[0])!=Math.signum(s.psiok[0])) conv_psiok=TMatr.getVectorScaled(conv_psiok,-1);
    UnitTests.checkDoubleArrayRel(conv_psiok,"conv(q.psiok)",s.psiok,"s.psiok",eps);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.kkdenom),"conv(q.kkdenom)",s.kkdenom,"s.kkdenom",eps);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.kknom),"conv(q.kknom)",s.kknom,"s.kknom",eps);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.pkkdenom),"conv(q.pkkdenom)",s.pkkdenom,"s.pkkdenom",eps);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.pkknom),"conv(q.pkknom)",s.pkknom,"s.pkknom",eps);

    System.err.println(name+":VQVMTest:matchGEVData(): test OK neq="+neq+" eps="+eps);
}

static class PolStoreQQtest {
final int n;
final double [] QQl,QQpl,QQp2l,QQr,QQpr,QQp2r;
    final double []  kkdenom,kknom,pkkdenom,pkknom,p2kkdenom,p2kknom;
PolStoreQQtest(final int n,final double [] mom_l,final double [] mom_pl,final double [] mom_p2l,final double scale_l,final double [] mom_r,final double [] mom_pr,final double [] mom_p2r,final double scale_r)
{
    this.n=n;
    QQl=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_l,scale_l);
    QQpl=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_pl,scale_l);
    QQp2l=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_p2l,scale_l);
    QQr=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_r,scale_r);
    QQpr=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_pr,scale_r);
    QQp2r=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_p2r,scale_r);

    final double [] In=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQr);
    kkdenom=OQVM.OQVMTest.getKK_fromQQ(n,In);
    kknom=OQVM.OQVMTest.getKK_fromQQ(n,TMatr.mulMMM(n,In,QQl,In));
    pkkdenom=OQVM.OQVMTest.getKK_fromQQ(n,TMatr.mulMMM(n,In,QQpr,In));
    pkknom=OQVM.OQVMTest.getKK_fromQQ(n,TMatr.mulMMM(n,In,QQpl,In));
    p2kkdenom=OQVM.OQVMTest.getKK_fromQQ(n,TMatr.mulMMM(n,In,QQp2r,In));
    p2kknom=OQVM.OQVMTest.getKK_fromQQ(n,TMatr.mulMMM(n,In,QQp2l,In));
}
}


void test_to_Monomials(final int n,
		       final TwoCloseRootsFinder.RootsData r,
		       final VQVM<DTYPE> q,
		       final XEVData [] xe,
		       final PolStoreQQtest m)
{
    int nj=-1;
    for(XEVData qr:xe)
    {
	nj++;
	final double x=qr.x;
	if(Double.isNaN(x)) break;

	final double d0q=Math.exp(nj==0?r.F0:(nj==1?r.F1:r.F2));
	UnitTests.checkDoubleRel(qr.Iok,"Iok",d0q,"d0q",eps);

	final double d1=(Polynomials.getD1(m.kknom,x)/Polynomials.getD0(m.kknom,x)-
			Polynomials.getD1(m.kkdenom,x)/Polynomials.getD0(m.kkdenom,x));
	final double d1q=nj==0?r.dF0:0;
	UnitTests.checkDoubleRel(d1,"d1."+nj,d1q,"d1q",eps);

	final double d2=Polynomials.getD2(m.kknom,x)/Polynomials.getD0(m.kknom,x)-Polynomials.getD2(m.kkdenom,x)/Polynomials.getD0(m.kkdenom,x)-Math.pow(Polynomials.getD1(m.kknom,x)/Polynomials.getD0(m.kknom,x),2)+Math.pow(Polynomials.getD1(m.kkdenom,x)/Polynomials.getD0(m.kkdenom,x),2);
	final double d2q=nj==0?r.d2F0:(nj==1?r.d2F1:r.d2F2);
	UnitTests.checkDoubleRel(d2,"d2",d2q,"d2q",eps);


	final XEVData s=pol_getXEVData(m,x,q.getPOffset());
	matchXEVData(qr,s);
    }
    System.err.println(name+":VQVMTest:test_Monomials_BasisExpansion(): test OK neq="+neq+" nj="+nj+" eps="+eps);
}

void test_to_PMonomials(final int n,
		       final TwoCloseRootsFinder.RootsData r,
		       final VQVM<DTYPE> q,
		       final XEVData [] xe,
		       final PolStoreQQtest m)
{
    int nj=-1;
    for(XEVData qr:xe)
    {
	nj++;
	final double x=qr.x;
	if(Double.isNaN(x)) break;

	final double d0q=(nj==0?r.F0:(nj==1?r.F1:r.F2));
	UnitTests.checkDoubleRel(qr.pLok-qr.pRok,"qr.pLok-qr.pRok",d0q,"d0q",eps);

	final double kn=Polynomials.getD0(m.kknom,x),kd=Polynomials.getD0(m.kkdenom,x),
	d1=(Polynomials.getD1(m.pkknom,x)/kn-(Polynomials.getD0(m.pkknom,x)/kn)*Polynomials.getD1(m.kknom,x)/kn)-
	    (Polynomials.getD1(m.pkkdenom,x)/kd-(Polynomials.getD0(m.pkkdenom,x)/kd)*Polynomials.getD1(m.kkdenom,x)/kd);
	final double d1q=nj==0?r.dF0:0;
	UnitTests.checkDoubleRel(d1,"d1."+nj,d1q,"d1q",eps);

	/*
	final double d2=Polynomials.getD2(m.kknom,x)/Polynomials.getD0(m.kknom,x)-Polynomials.getD2(m.kkdenom,x)/Polynomials.getD0(m.kkdenom,x)-Math.pow(Polynomials.getD1(m.kknom,x)/Polynomials.getD0(m.kknom,x),2)+Math.pow(Polynomials.getD1(m.kkdenom,x)/Polynomials.getD0(m.kkdenom,x),2);
	final double d2q=nj==0?r.d2F0:(nj==1?r.d2F1:r.d2F2);
	UnitTests.checkDoubleRel(d2,"d2",d2q,"d2q",eps);
	*/


	final XEVData s=pol_getXEVData(m,x,q.getPOffset());
	matchXEVData(qr,s);
    }
    System.err.println(name+":VQVMTest:test_PMonomials_BasisExpansion(): test OK neq="+neq+" nj="+nj+" eps="+eps);
}



void test_Monomials_BasisExpansion()
{

    final VQVM<DTYPE> q=getVQVM();
    final double [][] amoms=getMonomialMoments(q.odata);
    final int n=neq;
    if(q.rvd!=null){
    final PolStoreQQtest m=new PolStoreQQtest(n,amoms[0],amoms[1],amoms[2],amoms[0][0],amoms[3],amoms[4],amoms[5],1);
    test_to_Monomials(n,q.rvd,q,new XEVData []{q.rvd0_pv,q.rvd1_pv,q.rvd2_pv},m);
    test_to_PMonomials(n,q.rpd,q,new XEVData []{q.rpd0_pv,q.rpd1_pv,q.rpd2_pv},m);
    }
    if(q.rv!=null){
    final PolStoreQQtest m=new PolStoreQQtest(n,amoms[0],amoms[1],amoms[2],amoms[0][0],amoms[6],amoms[4],amoms[5],1);
    test_to_Monomials(n,q.rv,q,new XEVData []{q.rv0_pv,q.rv1_pv,q.rv2_pv},m);
    test_to_PMonomials(n,q.rp,q,new XEVData []{q.rp0_pv,q.rp1_pv,q.rp2_pv},m);
    }
    if(q.ra!=null){
    final PolStoreQQtest m=new PolStoreQQtest(n,amoms[8],amoms[9],amoms[10],1,amoms[0],amoms[1],amoms[2],amoms[0][0]);
    test_to_Monomials(n,q.ra,q,new XEVData []{q.ra0,q.ra1,q.ra2},m);
    UnitTests.checkDoubleRel(q.a0eq,"q.a0eq",q.ra0.Iok,"q.ra0.Iok",eps);
    }

    System.err.println(name+":VQVMTest:test_Monomials_BasisExpansion(): test OK neq="+neq+" eps="+eps);
}


void matchXEVData_GEVData(final XEVData q,final UQVM.GEVData s)
{
    UnitTests.checkDoubleArrayRel(q.psiok,"q.psiok",s.psiok,"s.psiok",eps);
    UnitTests.checkDoubleRel(q.Iok,"q.Iok",s.Iok,"s.Iok",eps);
    UnitTests.checkDoubleRel(q.pLok,"q.pLok",s.pLok,"s.pLok",eps);
    UnitTests.checkDoubleRel(q.stdev2_pLok,"q.stdev2_pLok",s.stdev2_pLok,"s.stdev2_pLok",eps);
    UnitTests.checkDoubleRel(q.pRok,"q.pRok",s.pRok,"s.pRok",eps);
    UnitTests.checkDoubleRel(q.stdev2_pRok,"q.stdev2_pRok",s.stdev2_pRok,"s.stdev2_pRok",eps);

    System.err.println(name+":VQVMTest:matchXEVData_GEVData(): match OK neq="+neq+" eps="+eps);
}
void test_UQVM()
{
    final VQVM<DTYPE> q=getVQVM();
    final UQVM<DTYPE> s=getUQVM();

    if(q.rv!=null) matchXEVData_GEVData(q.rv0_pv,s.pv);
    if(q.rvd!=null) matchXEVData_GEVData(q.rvd0_pv,s.pvd);

    UnitTests.checkDoubleRel(q.pv0eq,"q.pv0eq",s.pt.pRok,"s.pt.pRok",eps);
    UnitTests.checkDoubleRel(q.sigma2_pv0eq,"q.sigma2_pv0eq",s.pt.stdev2_pRok,"s.pt.stdev2_pRok",eps);

    UnitTests.checkDoubleRel(q.pt0eq,"q.pt0eq",s.pt.pLok,"s.pt.pLok",eps);
    UnitTests.checkDoubleRel(q.sigma2_pt0eq,"q.sigma2_pt0eq",s.pt.stdev2_pLok,"s.pt.stdev2_pLok",eps);

    UnitTests.checkDoubleRel(q.pe0eq,"q.pe0eq",s.pv.pLok,"s.pv.pLok",eps);
    UnitTests.checkDoubleRel(q.sigma2_pe0eq,"q.sigma2_pe0eq",s.pv.stdev2_pLok,"s.pv.stdev2_pLok",eps);

    System.err.println(name+":VQVMTest:test_UQVM(): test OK neq="+neq+" eps="+eps);
}
void test_LQVM()
{
    final VQVM<DTYPE> q=getVQVM();
    final LQVM<DTYPE> s=getLQVM();

    UnitTests.checkDoubleRel(q.oK00,"q.oK00",1/s.wiinodes[0],"1/s.wiinodes[0]",eps);
    UnitTests.checkDoubleRel(q.odp0,"q.odp0",s.dpi0,"s.dpi0",eps);
    UnitTests.checkDoubleRel(q.sigma2_odp0,"q.sigma2_odp0",s.sigma2_dpi0,"s.sigma2_dpi0",eps);
    UnitTests.checkDoubleRel(q.ods0,"q.ods0",s.dsi0,"s.dsi0",eps);
    UnitTests.checkDoubleRel(q.sigma2_ods0,"q.sigma2_ods0",s.sigma2_dsi0,"s.sigma2_dsi0",eps);
    UnitTests.checkDoubleRel(q.odts0,"q.odts0",s.dti0,"s.dti0",eps);
    UnitTests.checkDoubleRel(q.sigma2_odts0,"q.sigma2_odts0",s.sigma2_dti0,"s.sigma2_dti0",eps);

    System.err.println(name+":VQVMTest:test_LQVM(): test OK neq="+neq+" eps="+eps);
}

void test_dpdsdts()
{
    final VQVM<DTYPE> q=getVQVM();

    UnitTests.checkDoubleRel(q.oK00,"q.oK00",q.Kv00,"q.Kv00",eps);
    UnitTests.checkDoubleRel(q.ndp0,"q.ndp0",q.j0eq,"q.j0eq",eps);
    if(neq>2) UnitTests.checkDoubleRel(q.sigma2_ndp0,"q.sigma2_ndp0",q.sigma2_j0eq,"q.sigma2_j0eq",eps);
    UnitTests.checkDoubleRel(q.nds0,"q.nds0",q.js0eq,"q.js0eq",eps);
    if(neq>2) UnitTests.checkDoubleRel(q.sigma2_nds0,"q.sigma2_nds0",q.sigma2_js0eq,"q.sigma2_js0eq",eps);
    UnitTests.checkDoubleRel(q.ndts0,"q.ndts0",q.jt0eq,"q.jt0eq",eps);
    if(neq>2) UnitTests.checkDoubleRel(q.sigma2_ndts0,"q.sigma2_ndts0",q.sigma2_jt0eq,"q.sigma2_jt0eq",eps);

    System.err.println(name+":VQVMTest:test_dpdsdts(): test OK neq="+neq+" eps="+eps);
}

void test_Lastpoint()
{
    final VQVM<DTYPE> q=getVQVM();
    final Trade [] tb=new Trade [this.tb.length+1];
    System.arraycopy(this.tb,0,tb,0,this.tb.length);
    final double pl=130.0+2*r.nextDouble();
    tb[this.tb.length]=new Trade(pl,
				 10000,
				 null,
				 r.nextInt(2)==1?BookItem.BOOK_BUY:BookItem.BOOK_SELL,
				 currenttime,
				 -1,-1,-1,
				 null,Trade.TRADE_TYPE_REGULAR,
				 Trade.getDummyTrade(pl-10*r.nextDouble(),100,BookItem.BOOK_BUY,currenttime),
				 Trade.getDummyTrade(pl+10*r.nextDouble(),100,BookItem.BOOK_SELL,currenttime),
				 -1);
    this.tb=tb;
    final VQVM<DTYPE> s=getVQVM();
    final double [] s_rnodes=new double [neq];
    for(int k=s_rnodes.length;--k>=0;) s_rnodes[k]=s.rnodes[k]*(q.rnodes[q.rnodes.length-1]/s.rnodes[s.rnodes.length-1]);
    UnitTests.checkDoubleArrayRel(q.rnodes,"q.rnodes",s_rnodes,"s_rnodes",eps);

    final double f=q.w00/s.w00,g=f*q.nK00/s.nK00;
    UnitTests.checkDoubleRel(q.sigma2_odp0,"q.sigma2_odp0",s.sigma2_odp0*f,"s.sigma2_odp0*f",eps);
    UnitTests.checkDoubleRel(q.sigma2_ods0,"q.sigma2_ods0",s.sigma2_ods0/f,"s.sigma2_ods0/f",eps);
    UnitTests.checkDoubleRel(q.sigma2_odts0,"q.sigma2_odts0",s.sigma2_odts0*f,"s.sigma2_odts0*f",eps);
    System.err.println("q.sigma2_ndp0="+q.sigma2_ndp0+" s.sigma2_ndp0="+s.sigma2_ndp0+" f="+f+" q.nK00="+q.nK00+" s.nK00="+s.nK00);
    UnitTests.checkDoubleRel(q.sigma2_ndp0,"q.sigma2_ndp0",s.sigma2_ndp0*g,"s.sigma2_ndp0*g",eps);
    UnitTests.checkDoubleRel(q.sigma2_nds0,"q.sigma2_nds0",s.sigma2_nds0*g/f/f,"s.sigma2_nds0*g/f/f",eps);
    UnitTests.checkDoubleRel(q.sigma2_ndts0,"q.sigma2_ndts0",s.sigma2_ndts0*g,"s.sigma2_ndts0*g",eps);


    System.err.println(name+":VQVMTest:test_Lastpoint(): test OK neq="+neq+" eps="+eps);
}


abstract void test_enodes();

void test_PTwoRoots()
{
    final VQVM<DTYPE> q=getVQVM();

    final double epsD=1e-3,dx=2e-6;

    final PTwoRoots p=new PTwoRoots(q.rv0_pv.kkdenom,q.rv0_pv.kknom,q.rv0_pv.pkkdenom,q.rv0_pv.pkknom,q.access);
    final PolynomialsRatio fv=new PolynomialsRatio(q.rv0_pv.kknom,q.rv0_pv.pkknom,q.getABasis().B),
	ft=new PolynomialsRatio(q.rv0_pv.kkdenom,q.rv0_pv.pkkdenom,q.getABasis().B);

    for(int t=0;t<900;t++)
    {
	final double x=1.1*r.nextDouble();
	final double f=p.getF(x),df=p.getDF(x),d2f=p.getD2F(x);
	if(false)
	{
	    final double tdf=(p.getF(x+dx/2)-p.getF(x-dx/2))/dx;
	    final double td2f=(p.getDF(x+dx/2)-p.getDF(x-dx/2))/dx;
	    UnitTests.checkDoubleRel(df,"df",tdf,"tdf",epsD);
	    UnitTests.checkDoubleRel(d2f,"d2f",td2f,"td2f",epsD);
	}
	UnitTests.checkDoubleRel(p.getF(x),"p.getF(x)",fv.getF(x)-ft.getF(x),"fv.getF(x)-ft.getF(x)",eps);
	UnitTests.checkDoubleRel(p.getDF(x),"p.getDF(x)",fv.getDF(x)-ft.getDF(x),"fv.getDF(x)-ft.getDF(x)",eps);
	UnitTests.checkDoubleRel(p.getD2F(x),"p.getD2F(x)",fv.getD2F(x)-ft.getD2F(x),"fv.getD2F(x)-ft.getD2F(x)",eps);
    }

    System.err.println(name+":VQVMTest:test_PTwoRoots(): test OK neq="+neq+" epsD="+epsD);
}


void test_NevaiQQ(final int n,final double x,final double [] IQQv,final double [] QQv,final double [] QQdp,final double [] QQa,final double K0,final double j0,final double sigma2_j0,final double a0,final double sigma2_a0)
{

    final double [] r=new double [n],tmp=new double [n];
    for(int k=0;k<n;k++){
	tmp[k]=1;r[k]=Polynomials.getD0(tmp,x);tmp[k]=0;
    }
    final double [] psi=TMatr.getMatr_x_A(n,IQQv,r);
    final double nrm=Math.sqrt(Polynomials.getD0(psi,x));
    for(int k=0;k<n;k++) psi[k]/=nrm;
    final double j0eq=TMatr.getAMatrA(psi,QQdp,n);
    UnitTests.checkDoubleRel(K0,"K0",Math.pow(Polynomials.getD0(psi,x),2),"psi(x0)^2",eps);
    UnitTests.checkDoubleRel(j0,"j0",j0eq,"<psi|QQdp|psi>",eps);
    final double a0eq=TMatr.getAMatrA(psi,QQa,n);
    UnitTests.checkDoubleRel(a0,"a0",a0eq,"<psi|QQa|psi>",eps);
    if(n>2)
    {
	UnitTests.checkDoubleRel(sigma2_j0,"sigma2_j0",TMatr.getAMatrA(psi,TMatr.mulMMM(n,QQdp,IQQv,QQdp),n)-j0eq*j0eq,"<psi|QQdp G^{-1} QQdp|psi>-j0eq^2",eps);
	UnitTests.checkDoubleRel(sigma2_a0,"sigma2_a0",TMatr.getAMatrA(psi,TMatr.mulMMM(n,QQa,IQQv,QQa),n)-a0eq*a0eq,"<psi|QQa G^{-1} QQa|psi>-a0eq^2",eps);
    }

}

double [] pol_getPsi(final int n,final double [] qq,final double x0)
{
    final double [] psi=new double [n],r=new double [n];
    for(int k=0;k<n;k++)
    {
	r[k]=1;psi[k]=Polynomials.getD0(r,x0);r[k]=0;
    }
    for(int l=0;l<n;l++)
    {
	double s=0;
	for(int m=0,o=l*n;m<n;m++,o++) s+=qq[o]*psi[m];
	r[l]=s;
    }
    final double s=Math.sqrt(Polynomials.getD0(r,x0));
    for(int l=0;l<n;l++)
    {
	r[l]/=s;
    }
    return r;
}

void test_Nodes(final int n,final double [] rnodes,final double [] qnodesr,final double [] qxvnodes,
		final double [] qpvnodes,final double [] qsigma2_pvnodes,final double [] qpp,
		final double p_offset,
		final double x0,final double [] QQv,final double [] QQpv,final double [] QQp2v)
{
    if(rnodes.length!=n) throw new IllegalArgumentException();
    final double [] qnodes=convertToMonomials(qnodesr);
    final double f=qnodes[qnodes.length-1]/rnodes[rnodes.length-1];
    for(int k=0;k<qnodes.length;k++) qnodes[k]/=f;
    UnitTests.checkDoubleArrayRel(qnodes,"qnodes",rnodes,"rnodes",eps);
    System.err.println("n-1="+(n-1)+" rnodes="+java.util.Arrays.toString(rnodes));
    final PolynomialRootsFinder.PRoots u=new com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG().findRoots(rnodes);
    final double [] xnodes=u.re_root;
    if(u.nroots!=n-1) throw new RuntimeException("n-1="+(n-1)+" u.nroots="+u.nroots);
    if(!isRadauRight()) com.polytechnik.utils.SortArrays.sortArrayWithLoad(xnodes.length,xnodes,null);
    else com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(xnodes.length,xnodes,null);
    UnitTests.checkDoubleArrayRel(qxvnodes,"qxvnodes",xnodes,"xnodes",eps);

    final double [] IQQv=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQv);

    final double [] psi0=pol_getPsi(n,IQQv,x0);
    for(int k=-1;k<xnodes.length;k++)
    {
	final double xk=k<0?x0:xnodes[k];
	final double [] psik=pol_getPsi(n,IQQv,xk);
	for(int l=-1;l<xnodes.length;l++)
	{
	    final double xl=l<0?x0:xnodes[l];
	    final double [] psil=pol_getPsi(n,IQQv,xl);
	    final double pp=TMatr.getAMatrB(psik,psil,QQv,n);
	    final double nrm=k==l?1:0;
	    UnitTests.checkDoubleRel(nrm,"nrm",pp,"pp",eps);
	}
	if(k>0)
	{
	    UnitTests.checkDoubleRel(Math.abs(qpp[k]),"|qpp[k]|",Math.abs(TMatr.getAMatrB(psik,psi0,QQpv,n)),"|pp|",eps);
	    final double pvk=TMatr.getAMatrA(psik,QQpv,n);
	    UnitTests.checkDoubleRel(qpvnodes[k],"qpvnodes[k]",p_offset+pvk,"p_offset+<psik|QQpv|psik>",eps);
	    UnitTests.checkDoubleRel(qsigma2_pvnodes[k],"qsigma2_pvnodes[k]",TMatr.getAMatrA(psik,QQp2v,n)-pvk*pvk,"stdev2",eps);
	}
    }

    System.err.println(name+":VQVMTest:test_Nodes(): test OK neq="+neq+" eps="+eps);
}

void test_MatchRadau()
{
    final int n=neq;
    neq=n+1;
    final double [][] amoms=getMonomialMoments(getVQVM().odata); // kludge for Radau moments
    neq=n;
    final VQVM<DTYPE> q=getVQVM();
    final com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis M=
	new com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis()
	{
	    @Override
	    public double [] getRadauMultiplied(final double [] nodes0)
	    {
		final double [] m=isRadauRight()?new double []{q.getX0(),-1}:new double []{-q.getX0(),1};
		return Polynomials.mult2Pol(nodes0,m);
	    }
	};
    final double [] momv=amoms[0],mompv=amoms[1],momp2v=amoms[2];
    final double [][] R=M.getRadauPolynomials(M.getOrthogonalPolynomialsFirstKind(n+1,momv));

    final double []
	QQv=NQVM.NQVMTest.pol_getMM_Matr_scal(n,momv,1),
	QQpv=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mompv,1),
	QQp2v=NQVM.NQVMTest.pol_getMM_Matr_scal(n,momp2v,1);
    test_Nodes(n,R[R.length-1],q.rnodes,q.xvnodes,q.pvnodes,q.sigma2_pvnodes,q.pp,q.getPOffset(),q.getX0(),QQv,QQpv,QQp2v);

    System.err.println(name+":VQVMTest:test_MatchRadau(): test OK neq="+neq+" eps="+eps);
}


void test_MatchPol()
{
    final int n=neq;
    neq=n+1;
    final double [][] amoms=getMonomialMoments(getVQVM().odata); // kludge for Radau moments
    neq=n;
    final VQVM<DTYPE> q=getVQVM();

    final com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis M=
	new com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis();
    final double [] momv=amoms[0];
    final double [][] R=M.getOrthogonalPolynomialsFirstKind(n+1,momv);

    //test_Nodes(n,R[R.length-1],q.pnodes);

    System.err.println(name+":VQVMTest:test_MatchPol(): test OK neq="+neq+" eps="+eps);
}


void test_Monomials_DPBasisExpansion_Rurik_version()
{

    final VQVM<DTYPE> q=getVQVM();
    final double [][] amoms=getMonomialMoments(q.odata);
    final int n=neq;
    final double [] QQv=NQVM.NQVMTest.pol_getMM_Matr_scal(n,amoms[0],amoms[0][0]),
	QQdp=NQVM.NQVMTest.pol_getMM_Matr_scal(n,amoms[7],1),
	QQa=NQVM.NQVMTest.pol_getMM_Matr_scal(n,amoms[8],1);
    final double [] IQQv=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQv),
	QQdp_x_IQQv=TMatr.mulMM(n,QQdp,IQQv),
	IQQv_x_QQdp_x_IQQv=TMatr.mulMM(n,IQQv,QQdp_x_IQQv),
	IQQv_x_QQdp_x_IQQv_x_QQdp_x_IQQv=TMatr.mulMM(n,IQQv_x_QQdp_x_IQQv,QQdp_x_IQQv),
	QQa_x_IQQv=TMatr.mulMM(n,QQa,IQQv),
	IQQv_x_QQa_x_IQQv=TMatr.mulMM(n,IQQv,QQa_x_IQQv),
	IQQv_x_QQa_x_IQQv_x_QQa_x_IQQv=TMatr.mulMM(n,IQQv_x_QQa_x_IQQv,QQa_x_IQQv);

    final double []
	kkv=OQVM.OQVMTest.getKK_fromQQ(n,IQQv),
	kkdpv=OQVM.OQVMTest.getKK_fromQQ(n,IQQv_x_QQdp_x_IQQv),
	kkdp2v=OQVM.OQVMTest.getKK_fromQQ(n,IQQv_x_QQdp_x_IQQv_x_QQdp_x_IQQv),
	kkav=OQVM.OQVMTest.getKK_fromQQ(n,IQQv_x_QQa_x_IQQv),
	kka2v=OQVM.OQVMTest.getKK_fromQQ(n,IQQv_x_QQa_x_IQQv_x_QQa_x_IQQv);

    UnitTests.checkDoubleArrayRel(convertToMonomials(q.kkv),"conv(q.kkv)",kkv,"kkv",eps);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.kkdpv),"conv(q.kkdpv)",kkdpv,"kkdpv",eps);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.kkdp2v),"conv(q.kkdp2v)",kkdp2v,"kkdp2v",eps);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.kkav),"conv(q.kkav)",kkav,"kkav",eps);
    UnitTests.checkDoubleArrayRel(convertToMonomials(q.kka2v),"conv(q.kka2v)",kka2v,"kka2v",eps);


    for(int k=0;k<99;k++)
    {
	final double x,K0,j0eq,sigma2_j0eq,a0eq,sigma2_a0eq;
	if(k<=0)
	{
	    x=q.getX0();
	    K0=q.Kv00;
	    j0eq=q.j0eq;
	    sigma2_j0eq=q.sigma2_j0eq;
	    a0eq=q.a0eq;
	    sigma2_a0eq=q.sigma2_a0eq;
	}
	else
	{
	    x=r.nextDouble();
	    if(k<10)
	    {
		K0=q.access.fx(q.kkv,x);
		j0eq=q.access.fx(q.kkdpv,x)/K0;
		sigma2_j0eq=q.access.fx(q.kkdp2v,x)/K0-j0eq*j0eq;
		a0eq=q.access.fx(q.kkav,x)/K0;
		sigma2_a0eq=q.access.fx(q.kka2v,x)/K0-a0eq*a0eq;
	    }
	    else
	    {
		K0=Polynomials.getD0(kkv,x);
		j0eq=Polynomials.getD0(kkdpv,x)/K0;
		sigma2_j0eq=Polynomials.getD0(kkdp2v,x)/K0-j0eq*j0eq;
		a0eq=Polynomials.getD0(kkav,x)/K0;
		sigma2_a0eq=Polynomials.getD0(kka2v,x)/K0-a0eq*a0eq;
	    }
	}
	test_NevaiQQ(n,x,IQQv,QQv,QQdp,QQa,K0,j0eq,sigma2_j0eq,a0eq,sigma2_a0eq);
    }


    final double joeq=kkdpv[kkdpv.length-1]/kkv[kkv.length-1];
    UnitTests.checkDoubleRel(q.joeq,"q.joeq",joeq,"joeq",eps);
    final double sigma2_joeq=kkdp2v[kkdp2v.length-1]/kkv[kkv.length-1]-joeq*joeq;
    UnitTests.checkDoubleRel(q.sigma2_joeq,"q.sigma2_joeq",sigma2_joeq,"sigma2_joeq",eps);
    final double aoeq=kkav[kkav.length-1]/kkv[kkv.length-1];
    UnitTests.checkDoubleRel(q.aoeq,"q.aoeq",aoeq,"aoeq",eps);
    final double sigma2_aoeq=kka2v[kka2v.length-1]/kkv[kkv.length-1]-aoeq*aoeq;
    UnitTests.checkDoubleRel(q.sigma2_aoeq,"q.sigma2_aoeq",sigma2_aoeq,"sigma2_aoeq",eps);

    System.err.println(name+":VQVMTest:test_Monomials_DPBasisExpansion_Rurik_version(): test OK neq="+neq+" eps="+eps);
}



void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":VQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Monomials_BasisExpansion();
    test_UQVM();
    test_PTwoRoots();
    test_Monomials_DPBasisExpansion_Rurik_version();
    test_MatchRadau();
    //test_MatchPol();
    test_enodes();
    test_LQVM();
    test_dpdsdts();
    if(neq>1) test_Lastpoint();

    System.err.println(name+":VQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":VQVMTest: starting tests; ver=$Id: VQVM.java,v 1.118 2022/08/08 20:10:47 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":VQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: VQVM.java,v 1.118 2022/08/08 20:10:47 mal Exp $");
}
} // END OF class VQVMTest



}// END OF class VQVM
