package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.LogPolynomialsRatio;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;
import com.polytechnik.outils.*;

public class VQVMLegendreShifted extends VQVM<QVMDataP> {

VQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static VQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final VQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new VQVMLegendreShifted(neq,o);
}

double [] getXNodes(final double [] nodespolynomial)
{
    final int n=nodespolynomial.length-1;
    //final double [] npol=LegendreShifted.convertToPolynomialFromLegendre_Test(n,nodespolynomial);
    //final int nroots=new com.polytechnik.utils.PolynomialRootJenkinsTraubORIG(n).findRoots(n,npol,x,npol);
    final PolynomialRootsFinder.PRoots u=new com.polytechnik.outils.LegendreShiftedPolynomialRootsFinder().findRoots(nodespolynomial);
    final double [] x=u.re_root;
    if(u.nroots!=n) java.util.Arrays.fill(x,Double.NaN);
    com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(x.length,x,null);
    return x;
}
double getX0(){return 1;}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
double getPOffset(){return odata.p_offset;}
TwoCloseRootsFinder.RootsData getTR(final double [] kv,final double [] kt)
{
    final double x0=1;
    if(kv.length!=kt.length) throw new IllegalArgumentException("kv.length="+kv.length+"!=kt.length="+kt.length);
    return TwoCloseRootsFinder.findRootsData(x0,0.5,3000,new LogPolynomialsRatio(kv,kt,getABasis().B));
}
TwoCloseRootsFinder.RootsData getTRP(final double [] kkdenom,final double [] kknom,final double [] pkkdenom,final double [] pkknom)
{
    final double x0=1;
    return TwoCloseRootsFinder.findRootsData(x0,0.5,3000,new PTwoRoots(kkdenom,kknom,pkkdenom,pkknom,access));
}

static class VQVMLegendreShiftedTest extends VQVMTest<QVMDataP> {
VQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(6873L),1e-4);
}
long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8*11)*(9*5*7*8*11);
}
VQVMLegendreShifted getVQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
UQVMLegendreShifted getUQVM(){return UQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
RadauLQVMLegendreShifted getLQVM(){return RadauLQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}

boolean isRadauRight(){return true;}
double [] convertToMonomials(final double [] p){return WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData [] getPData(final int nmoms)
{
    final QVMData [] qdata=new QVMData [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}
double [][] getMonomialMoments(final QVMDataP odata)
{
    final int nmoms=2*neq-1;
    final QVMData [] qd=getPData(nmoms);
    return new double [][]{
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dT,true,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_dT,true,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_dT,true,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomT(nmoms),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomDP(nmoms,qd),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_A,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,1,QVMData.MOM_A,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,2,QVMData.MOM_A,false,odata.p_offset)
    };
}

void test_enodes()
{
    final VQVMLegendreShifted q=getVQVM();
    final double [] xr=WIntegratorLegendreShifted.B.getRadauRoots(neq);
    com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(xr.length,xr,null);
    UnitTests.checkDoubleArrayRel(q.xenodes,"q.xenodes",DeprecatedHelperTMatr.remove0Element(xr),"xr",eps);
    final double [] en0=WIntegratorLegendreShifted.B.getRadauNodesPolynomial(neq);
    final double [] en=WIntegratorLegendreShifted.B.sdiv1(en0,q.getX0()).q;
    final double f=q.enodes[q.enodes.length-1]/en[en.length-1];
    for(int k=0;k<en.length;k++) en[k]*=f;
    UnitTests.checkDoubleArrayRel(q.enodes,"q.enodes",en,"en",eps);

    System.err.println(name+":VQVMTest:test_enodes(): test OK neq="+neq+" eps="+eps);
}


} // END of class VQVMLegendreShiftedTest

public static void main(String [] args)
{
    new VQVMLegendreShiftedTest("VQVMLegendreShifted").runTests(5);
} // END main



} // END OF class VQVMLegendreShifted
