package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;

/*
 * @version $Id: Var2I.java,v 1.12 2020/09/27 20:33:22 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
class Var2I {
    final double norm,norm_,f_l,f_a,f_aa,p,pd,i,var2i;

    Var2I(final int ind_ev,final EVXData I,final double [] QQi,final double [] QQt,final double [] QQpi,final double pL,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi)
    {
	final int n=I.neq;
	final double [] psi=I.getPsi(ind_ev),Dpsi=VOLATILITY_EDPsi.getDPsi(psi),DDpsi=VOLATILITY_EDPsi.getDPsi(Dpsi);
	final double i=I.ev[ind_ev],ipp=i*M.B.getD0(psi,I.x0)*M.B.getD0(Dpsi,I.x0),
	    dd=TMatr.getAMatrB(Dpsi,Dpsi,QQt,n),Pi=TMatr.getAMatrB(psi,psi,QQpi,n),Pddi=TMatr.getAMatrB(Dpsi,Dpsi,QQpi,n),ddi=TMatr.getAMatrB(Dpsi,Dpsi,QQi,n);
	norm=2*(TMatr.getAMatrB(psi,DDpsi,QQi,n)+TMatr.getAMatrB(Dpsi,Dpsi,QQi,n)-ipp);
	norm_=2*(ddi-i*dd);
	f_l=2*(TMatr.getAMatrB(psi,DDpsi,QQpi,n)+TMatr.getAMatrB(Dpsi,Dpsi,QQpi,n)-pL*ipp)/norm;
	f_a=2*(Pddi-dd*Pi)/norm;
	p=Pi/i;
	pd=Pddi/ddi;
	this.i=i;
	this.var2i=ddi/dd-i;
	TMatr.sumP1(Dpsi,psi,-TMatr.getAMatrB(Dpsi,psi,QQt,n));
	//System.err.println("<Dpsi|psi>="+TMatr.getAMatrB(Dpsi,psi,QQt,n));
	final double dd0=TMatr.getAMatrB(Dpsi,Dpsi,QQt,n),ddi0=TMatr.getAMatrB(Dpsi,Dpsi,QQi,n),Pddi0=TMatr.getAMatrB(Dpsi,Dpsi,QQpi,n),Pi0=TMatr.getAMatrB(psi,psi,QQpi,n),i0=TMatr.getAMatrB(psi,psi,QQi,n);
	this.f_aa=(Pddi0-dd0*Pi0)/(ddi0-i0*dd0);

    }

    static void test_Var2I(final int n,final double [] QQi,final double [] QQt,final double [] QQpi,final double pL,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix VOLATILITY_EDPsi,final double eps)
{
    final EVXData I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] DQQtD=VOLATILITY_EDPsi.getVolMatrix(n,QQt),
	DQQiD=VOLATILITY_EDPsi.getVolMatrix(n,QQi),
	DQQpiD=VOLATILITY_EDPsi.getVolMatrix(n,QQpi),
	DDd2pi=VOLATILITY_EDPsi.getDiff2Matrix(n,QQpi);
    for(int k=0;k<n && n>2;k++)
    {
	final Var2I q=new Var2I(k,I,QQi,QQt,QQpi,pL,M,VOLATILITY_EDPsi);
	UnitTests.checkDoubleRel(q.norm,"q.norm",q.norm_,"q.norm_",eps);
	final double [] psi=I.getPsi(k);
	final double norm=2*(TMatr.getAMatrA(psi,DQQiD,n)-TMatr.getAMatrA(psi,DQQtD,n)*I.ev[k]);
	UnitTests.checkDoubleRel(q.norm,"q.norm",norm,"norm",eps);
	final double f_a=2*(TMatr.getAMatrA(psi,DQQpiD,n)-TMatr.getAMatrA(psi,DQQtD,n)*TMatr.getAMatrA(psi,QQpi,n))/norm;
	UnitTests.checkDoubleRel(q.f_a,"q.f_a",f_a,"f_a",eps);
	final double [] Dpsi=VOLATILITY_EDPsi.getDPsi(psi);
	final double f_l=(2*-pL*I.ev[k]*M.B.getD0(psi,I.x0)*M.B.getD0(Dpsi,I.x0)+TMatr.getAMatrA(psi,DDd2pi,n))/norm;
	UnitTests.checkDoubleRel(q.f_l,"q.f_l",f_l,"f_l",eps);

	final double var2i=(TMatr.getAMatrA(psi,DQQiD,n)/TMatr.getAMatrA(psi,DQQtD,n)-TMatr.getAMatrA(psi,QQi,n)),
	    f_a_=(TMatr.getAMatrA(psi,DQQpiD,n)/TMatr.getAMatrA(psi,DQQtD,n)-TMatr.getAMatrA(psi,QQpi,n))/var2i;
	UnitTests.checkDoubleRel(q.var2i,"q.var2i",var2i,"var2i",eps);
	UnitTests.checkDoubleRel(q.f_a,"q.f_a",f_a_,"f_a_",eps);
    }
    System.err.println("Var2I:test_Var2I(): test OK n="+n+" eps="+eps);
}

}

