package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: VkQVM.java,v 1.89 2022/11/19 11:08:58 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class VkQVM<DTYPE> {

final DTYPE odata;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
final double I_0,dI_Di0,dI_DiL,dI_DiH,dp_Di0,dp_DiL,dp_DiH,I_DiL,I_DiH,c_DiL,c_DiH,p_DiL,p_DiH,r_DiL,r_DiH;

static class State {}
final State sst;
VkQVM(final int n,final DTYPE odata,final State rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.odata=odata;
    this.M=getABasis();

    final int nmoms=2*n-1;
    final double [][] mm=getMoments(nmoms);
    final double [] QQt=M.B.getQQMatr(n,mm[0]),QQi=M.B.getQQMatr(n,mm[1]),QQc=M.B.getQQMatr(n,mm[3]),
	QQdp=M.B.getQQMatr(n,mm[4]),QQdi=M.B.getQQMatr(n,mm[5]),QQpi=M.B.getQQMatr(n,mm[2]),QQdip=M.B.getQQMatr(n,mm[7]);

    final double [] dibasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQdi,QQt,M);
    final int i0=0,iL=1,iH=n-1;

    // Note: Because of integration by parts sign the "_DiL" values correpond to max dI/dt and "_DiH" to min dI/dt.

    final double [] psi_0=EVXData.getPsiFromBasis(i0,dibasis,n),
	psi_DiL=EVXData.getPsiFromBasis(iL,dibasis,n),psi_DiH=EVXData.getPsiFromBasis(iH,dibasis,n);

    this.I_0=TMatr.getAMatrA(psi_0,QQi,n);

    this.dI_Di0=dibasis[i0];
    this.dI_DiL=dibasis[iL];
    this.dI_DiH=dibasis[iH];

    this.dp_Di0=TMatr.getAMatrA(psi_0,QQdp,n);
    this.dp_DiL=TMatr.getAMatrA(psi_DiL,QQdp,n);
    this.dp_DiH=TMatr.getAMatrA(psi_DiH,QQdp,n);

    this.I_DiL=TMatr.getAMatrA(psi_DiL,QQi,n);
    this.I_DiH=TMatr.getAMatrA(psi_DiH,QQi,n);

    this.c_DiL=TMatr.getAMatrA(psi_DiL,QQc,n);
    this.c_DiH=TMatr.getAMatrA(psi_DiH,QQc,n);

    this.p_DiL=TMatr.getAMatrA(psi_DiL,QQdip,n)/dI_DiL;
    this.p_DiH=TMatr.getAMatrA(psi_DiH,QQdip,n)/dI_DiH;

    this.r_DiL=dp_DiL*I_DiL/c_DiL;
    this.r_DiH=dp_DiH*I_DiH/c_DiH;

    if(true)
    {
	final double [] QQd2p=M.B.getQQMatr(n,mm[6]);
	final double [] ibasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M),
	    psi_iH=EVXData.getPsiFromBasis(iH,ibasis,n),
	    psi_iL=EVXData.getPsiFromBasis(iL,ibasis,n);
	I_iL=ibasis[iL];
	I_iH=ibasis[iH];
	this.dp_iL=TMatr.getAMatrA(psi_iL,QQdp,n);
	this.dp_iH=TMatr.getAMatrA(psi_iH,QQdp,n);
	this.d2p_iL=TMatr.getAMatrA(psi_iL,QQd2p,n);
	this.d2p_iH=TMatr.getAMatrA(psi_iH,QQd2p,n);
	final double [] QQi_dp=TMatr.mulMatrices(n,new double [][]{QQi,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),QQdp});
	TMatr.selectHermitianPartInPlace(n,QQi_dp);
	this.aiH_I_dp=TMatr.getAMatrA(psi_iH,QQi_dp,n);
	final double [] idp_basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi_dp,QQt,M);
	Idp_IdpL=idp_basis[iL];
	Idp_IdpH=idp_basis[iH];
	w_IdpL_iH=TMatr.getAMatrB(psi_iH,EVXData.getPsiFromBasis(iL,idp_basis,n),QQt,n);
	w_IdpH_iH=TMatr.getAMatrB(psi_iH,EVXData.getPsiFromBasis(iH,idp_basis,n),QQt,n);

	this.varPIDt=TMatr.getAMatrB(psi_iH,UkQVM.getPsiPert0Skipped(n,ibasis,iH,QQdi),QQpi,n);

	final double [] dpbasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQdp,QQt,M),
	    psi_dpL=EVXData.getPsiFromBasis(iL,dpbasis,n),
	    psi_dpH=EVXData.getPsiFromBasis(iH,dpbasis,n);
	this.dp_dpL=dpbasis[iL];
	this.dp_dpH=dpbasis[iH];
	this.I_dpL=TMatr.getAMatrA(psi_dpL,QQi,n);
	this.I_dpH=TMatr.getAMatrA(psi_dpH,QQi,n);
	this.dI_dpL=TMatr.getAMatrA(psi_dpL,QQdi,n);
	this.dI_dpH=TMatr.getAMatrA(psi_dpH,QQdi,n);

	this.d2p_DiL=TMatr.getAMatrA(psi_DiL,QQd2p,n);
	this.d2p_DiH=TMatr.getAMatrA(psi_DiH,QQd2p,n);
	double sp_d2I_d2p=0;
	for(int k=1;k<n;k++)
	{
	    sp_d2I_d2p+=dibasis[k]*TMatr.getAMatrA(EVXData.getPsiFromBasis(k,dibasis,n),QQd2p,n);
	}
	this.a_d2I_d2p=sp_d2I_d2p;

	/*
	final double x0=M.getRadauRoot();
	final double [] d2tmoms=new double [nmoms];
	getAccess().setDFMoments(d2tmoms,M.B.getX0Moments(nmoms,x0),nmoms);
	System.err.println("psi(x0)^2="+Math.pow(M.B.getD0(psi_0,x0),2)+"\t<psi0|QQd0|psi0>="+TMatr.getAMatrA(psi_0,M.B.getQQMatr(n,d2tmoms),n)+" n="+n);
	*/
    }

    this.sst=null;
}
final double dp_iL,dp_iH,d2p_iL,d2p_iH,I_iL,I_iH,dp_dpL,dp_dpH,d2p_DiL,d2p_DiH,a_d2I_d2p,I_dpL,I_dpH,dI_dpL,dI_dpH,aiH_I_dp,Idp_IdpL,Idp_IdpH,w_IdpL_iH,w_IdpH_iH,varPIDt;


double [][] getMoments(final int nmoms)
{
    final double [] tmoms=new double [nmoms],imoms=new double [nmoms],pimoms=new double [nmoms],dpmoms=new double [nmoms],dimoms=new double [nmoms],d2pmoms=new double [nmoms],dpimoms=new double [nmoms];

    final DataInterfaceToMoments<DTYPE> access=getAccess();

    access.setMT(tmoms,nmoms);
    access.setM(imoms,0,access.M_I,odata,nmoms);
    access.setM(pimoms,1,access.M_I,odata,nmoms);
    final double [] cmoms=TMatr.sumAcB(pimoms,getPOffset(),imoms);
    TMatr.sumP1(pimoms,imoms,getPOffset()-getPLast());
    access.setM(dpmoms,1,access.M_DP,odata,nmoms);
    access.setDFMoments(dimoms,imoms,nmoms);
    access.setDFMoments(d2pmoms,dpmoms,nmoms);
    access.setDFMoments(dpimoms,pimoms,nmoms);

    return new double [][]{tmoms,imoms,pimoms,cmoms,dpmoms,dimoms,d2pmoms,dpimoms};
}


abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getPLast();
abstract double getPOffset();


static abstract class VkQVMTest<DTYPE> extends QTest {

abstract VkQVM<DTYPE> getVkQVM();
abstract double [] getEDPsi(final double [] psi);

final VolMatrix VOLATILITY_EDPsi=this::getEDPsi;

VkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}

void test_QQpsi(final int n,final double [] tmoms,final double [] imoms,final double [] dimoms,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] QQt=M.B.getQQMatr(n,tmoms),QQi=M.B.getQQMatr(n,imoms),QQdi=M.B.getQQMatr(n,dimoms);
    final double [] dibasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQdi,QQt,M);
    for(int k=1;k<n;k++)
    {
	final double [] psi=EVXData.getPsiFromBasis(k,dibasis,n),Dpsi=getEDPsi(psi);
	final double psi_Dpsi=TMatr.getAMatrB(psi,Dpsi,QQt,n),psi_I_Dpsi=TMatr.getAMatrB(psi,Dpsi,QQi,n),psi_DI_Dpsi=TMatr.getAMatrB(psi,Dpsi,QQdi,n);
	System.err.println("<psi|Dpsi>="+psi_Dpsi+"\t2<psi|I|Dpsi>="+(2*psi_I_Dpsi)+" ev["+k+"]="+dibasis[k]+"\t<psi|DI|Dpsi>="+psi_DI_Dpsi);
	UnitTests.checkDoubleRel(psi_Dpsi,"psi_Dpsi",0,"0",eps);
	UnitTests.checkDoubleRel(2*psi_I_Dpsi,"2*psi_I_Dpsi",dibasis[k],"dibasis[k]",eps);
	//UnitTests.checkDoubleRel(psi_DI_Dpsi,"psi_DI_Dpsi",0,"0",eps);
    }

    System.err.println(name+":VkQVMTest:test_QQpsi(): test OK n="+n+" eps="+eps);
}

static double [][] get_QQ_QQD_D2QQ_DQQD(final int n,final double [] imoms,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix DpsiCalculator)
{
    final double [] QQ=new double [n*n],QQD=new double [n*n],DQQD=new double [n*n],D2QQ=new double [n*n];

    for(int k=0;k<n;k++)
    {
	final double [] r=new double [k+1];r[k]=1;
	final double [] dr=DpsiCalculator.getDPsi(r),dr2=DpsiCalculator.getDPsi(dr);
	for(int l=0;l<n;l++)
	{
	    final double [] s=new double [l+1];s[l]=1;
	    final double [] ds=DpsiCalculator.getDPsi(s),ds2=DpsiCalculator.getDPsi(ds);
	    QQ[k*n+l]=TMatr.mulVVn1(M.B.mult2Pol(r,s),imoms);
	    QQD[k*n+l]=TMatr.mulVVn1(M.B.mult2Pol(r,ds),imoms)+TMatr.mulVVn1(M.B.mult2Pol(s,dr),imoms);
	    DQQD[k*n+l]=TMatr.mulVVn1(M.B.mult2Pol(dr,ds),imoms);
	    D2QQ[k*n+l]=2*TMatr.mulVVn1(M.B.mult2Pol(dr,ds),imoms)+
		TMatr.mulVVn1(M.B.mult2Pol(dr2,s),imoms)+TMatr.mulVVn1(M.B.mult2Pol(r,ds2),imoms);
	}
    }
    return new double [][] {QQ,QQD,D2QQ,DQQD};
}

void test_varPIDt(final int n,final double [] tmoms,final double [] imoms,final double [] dimoms,final double [] pimoms,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final double varPIDt)
{
    final int nd=n-1;
    // First test.
   final EVXData e=EVXData.getEVXDataFromQQ(nd,M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(M.getRadauMomentsFromMoments(imoms))),M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(M.getRadauMomentsFromMoments(tmoms))),M);
   final double [] QQi=M.B.getQQMatr(n,imoms),QQDi=VOLATILITY_EDPsi.getDiffMatrix(n,QQi);
   final int indH=nd-1;
   final double [] QQpert=new double [nd*nd];
   for(int k=0;k<nd;k++)
   {
       final double [] _psik=new double [k+1];_psik[k]=1;
       final double [] psik=M.getRadauMultiplied(_psik);
       for(int l=0;l<nd;l++)
       {
	   final double [] _psil=new double [l+1];_psil[l]=1;
	   QQpert[nd*k+l]=TMatr.getAMatrB(psik,M.getRadauMultiplied(_psil),QQDi,n);
       }
   }
   final double _varPIDt=TMatr.getAMatrB(DeprecatedHelperEVXData.getPsiPertr(e,indH,QQpert),e.getPsi(indH),M.B.getQQMatr(nd,M.getRadauMomentsFromMoments(M.getRadauMomentsFromMoments(pimoms))),nd);
   UnitTests.checkDoubleRel(varPIDt,"varPIDt",_varPIDt,"_varPIDt",eps);

   // Second test.
   final double [] QQt=M.B.getQQMatr(n,tmoms),QQpi=M.B.getQQMatr(n,pimoms);
   final int iH=n-1;
   final double [] ibasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);
   final double [] bQQDi=EVXData.convertMatrFromQtoB_Basis(n,QQDi,ibasis),
      bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,ibasis);
   double s_varPIDt=0,s2_varPIDt=0,s3_varPIDt=0;
   for(int k=0+1;k<n;k++)
   {
       if(k==iH) continue;
       final double [] psi=EVXData.getPsiFromBasis(iH,ibasis,n),psik=EVXData.getPsiFromBasis(k,ibasis,n);
       s_varPIDt+=((TMatr.getAMatrB(getEDPsi(psik),psi,QQi,n)+TMatr.getAMatrB(psik,getEDPsi(psi),QQi,n))/(ibasis[iH]-ibasis[k]))*TMatr.getAMatrB(psik,psi,QQpi,n);
       s2_varPIDt+=(TMatr.getAMatrB(psik,psi,QQDi,n)/(ibasis[iH]-ibasis[k]))*TMatr.getAMatrB(psik,psi,QQpi,n);
       s3_varPIDt+=(bQQDi[iH*n+k]/(ibasis[iH]-ibasis[k]))*bQQpi[iH*n+k];
   }
   UnitTests.checkDoubleRel(varPIDt,"varPIDt",s_varPIDt,"s_varPIDt",eps);
   UnitTests.checkDoubleRel(varPIDt,"varPIDt",s2_varPIDt,"s2_varPIDt",eps);
   UnitTests.checkDoubleRel(varPIDt,"varPIDt",s3_varPIDt,"s3_varPIDt",eps);
   System.err.println("varPIDt="+varPIDt+"\t_varPIDt="+_varPIDt+"\ts_varPIDt="+s_varPIDt+"\ts2_varPIDt="+s2_varPIDt+"\ts3_varPIDt="+s3_varPIDt);

   System.err.println(name+":VkQVMTest:test_varPIDt(): test OK n="+n+" eps="+eps);
}

void test_QQdi(final int n,final double [] imoms,final double [] dimoms,final double [] d2imoms,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M)
{
    final double [] QQi=M.B.getQQMatr(n,imoms),QQdi=M.B.getQQMatr(n,dimoms),QQd2i=M.B.getQQMatr(n,d2imoms);
    final double [][] ddQQ=get_QQ_QQD_D2QQ_DQQD(n,imoms,M,VOLATILITY_EDPsi);
    final double [] QQ=ddQQ[0],QQD=ddQQ[1],D2QQ=ddQQ[2],DQQD=ddQQ[3];

    //System.err.println("QQdi="+java.util.Arrays.toString(QQdi));
    //System.err.println(" QQD="+java.util.Arrays.toString(QQD));
    UnitTests.checkDoubleArrayRel(QQdi,"QQdi",QQD,"QQD",eps);
    UnitTests.checkDoubleArrayRel(QQdi,"QQdi",VOLATILITY_EDPsi.getDiffMatrix(n,QQi),"VOLATILITY_EDPsi.getDiffMatrix",eps);

    UnitTests.checkDoubleArrayRel(QQd2i,"QQd2i",D2QQ,"D2QQ",eps);
    UnitTests.checkDoubleArrayRel(QQd2i,"QQd2i",VOLATILITY_EDPsi.getDiff2Matrix(n,QQi),"VOLATILITY_EDPsi.getDiff2Matrix",eps);
    final double [] D=VOLATILITY_EDPsi.getDMatrix(n),
	MD2=DeprecatedHelperTMatr.sumAB(n*n,TMatr.mulMatrices(n,new double [][] {TMatr.getMatrTrans(n,D),QQi,D}),TMatr.mulMatrices(n,new double [][] {QQi,D,D}));
    TMatr.scaleVectorInPlace(TMatr.selectHermitianPartInPlace(n,MD2),2);
    UnitTests.checkDoubleArrayRel(QQd2i,"QQd2i",MD2,"MD2",eps);
    UnitTests.checkDoubleArrayRel(QQdi,"QQdi",VolMatrix.getHAB(n,QQi,D),"getHAB(QQi,D)",eps);


    UnitTests.checkDoubleArrayRel(DQQD,"DQQD",VOLATILITY_EDPsi.getVolMatrix(n,QQi),"VOLATILITY_EDPsi.getVolMatrix",eps);
    UnitTests.checkDoubleArrayRel(QQD,"QQD",VOLATILITY_EDPsi.getDiffMatrix(n,QQi),"VOLATILITY_EDPsi.getDiffMatrix",eps);
    UnitTests.checkDoubleArrayRel(D2QQ,"D2QQ",VOLATILITY_EDPsi.getDiff2Matrix(n,QQi),"VOLATILITY_EDPsi.getDiff2Matrix",eps);
    UnitTests.checkDoubleArrayRel(TMatr.sumAcB(D2QQ,-2,DQQD),"D2QQ-2DQQD",VOLATILITY_EDPsi.getDiff2PartMatrix(n,QQi),"VOLATILITY_EDPsi.getDiff2PartMatrix",eps);
    UnitTests.checkDoubleArrayRel(QQ,"QQ",QQi,"QQ",eps);

    final int nd=n-1;
    final double [] Rimoms=M.getRadauMomentsFromMoments(imoms),DRimoms=M.getRadauMomentsFromMoments(dimoms),
	R2imoms=M.getRadauMomentsFromMoments(Rimoms),DR2imoms=M.getRadauMomentsFromMoments(DRimoms);
    UnitTests.checkDoubleArrayRel(M.B.getQQMatr(nd,Rimoms),"RQQi",VolMatrix.getRadauMatrix(n,QQi,M),"RQQi",eps);
    UnitTests.checkDoubleArrayRel(M.B.getQQMatr(nd,DRimoms),"RQQDi",VOLATILITY_EDPsi.getDRadauMatrix(n,QQi,M),"RQQDi",eps);
    UnitTests.checkDoubleArrayRel(VolMatrix.getRadauMatrix(n,VOLATILITY_EDPsi.getDiffMatrix(n,QQi),M),"Radau(Diff(QQi))",VOLATILITY_EDPsi.getDRadauMatrix(n,QQi,M),"RQQDi",eps);
    UnitTests.checkDoubleArrayRel(M.B.getQQMatr(nd,R2imoms),"R2QQi",VolMatrix.getRadau2Matrix(n,QQi,M),"R2QQi",eps);
    UnitTests.checkDoubleArrayRel(M.B.getQQMatr(nd,DR2imoms),"R2QQDi",VOLATILITY_EDPsi.getDRadau2Matrix(n,QQi,M),"R2QQDi",eps);
    UnitTests.checkDoubleArrayRel(VolMatrix.getRadau2Matrix(n,VOLATILITY_EDPsi.getDiffMatrix(n,QQi),M),"Radau2(Diff(QQi))",VOLATILITY_EDPsi.getDRadau2Matrix(n,QQi,M),"R2QQDi",eps);

    System.err.println(name+":VkQVMTest:test_QQdi(): test OK n="+n+" eps="+eps);
}

static double [] getBasis_DQQi_QQt(final int n,final double [] tmoms,final double [] imoms,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final VolMatrix DpsiCalculator)
{
    final int nd=n-1;
    final double [] QQt=new double [nd*nd],QQDi=new double [nd*nd];

    for(int k=0;k<nd;k++)
    {
	final double [] pk=new double [k+1];pk[k]=1;
	final double [] rpk=M.getRadauMultiplied(pk),Drpk=DpsiCalculator.getDPsi(rpk);
	for(int l=0;l<nd;l++)
	{
	    final double [] pl=new double [l+1];pl[l]=1;
	    final double [] rpl=M.getRadauMultiplied(pl),Drpl=DpsiCalculator.getDPsi(rpl);
	    QQt[k*nd+l]=TMatr.mulVVn1(M.B.mult2Pol(rpk,rpl),tmoms);
	    QQDi[k*nd+l]=TMatr.mulVVn1(M.B.mult2Pol(rpk,Drpl),imoms)+TMatr.mulVVn1(M.B.mult2Pol(rpl,Drpk),imoms);
	}
    }
    final double [] ev=M.EV.getGEV_Generalized(nd,QQDi,QQt,true);
    final double [] eev=new double [n+n*n];
    for(int k=0;k<nd;k++)
    {
	eev[k+1]=ev[k];
	System.arraycopy(M.getRadauMultiplied(EVXData.getPsiFromBasis(k,ev,nd)),0,eev,n+(k+1)*n,n);
    }

    // psi0
    final double [] aQQt=M.B.getQQMatr(n,tmoms);final int [] ipvt=new int [n];
    final double x0=M.getRadauRoot();
    final double [] psi0=M.B.getX0Moments(n,x0);
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(aQQt,n,n,ipvt);
    if(pinfo!=0){java.util.Arrays.fill(psi0,Double.NaN);}
    else {com.polytechnik.utils.Linsystems.dgesl_m1(aQQt,n,n,ipvt,psi0,1);}
    TMatr.scaleVectorInPlace(psi0,1/Math.sqrt(M.B.getD0(psi0,x0)));
    System.arraycopy(psi0,0,eev,n+0,n);
    eev[0]=2*TMatr.mulVVn1(M.B.mult2Pol(psi0,DpsiCalculator.getDPsi(psi0)),imoms);

    return eev;
}


void test_Basis(final VkQVM<DTYPE> q)
{
    final int n=neq,nmoms=2*n-1;
    final double [][] mm=q.getMoments(nmoms);
    final double [] basis=getBasis_DQQi_QQt(n,mm[0],mm[1],q.M,VOLATILITY_EDPsi);

    UnitTests.checkDoubleRel(basis[0],"basis[0]",q.dI_Di0,"q.dI_Di0",eps);
    UnitTests.checkDoubleRel(basis[1],"basis[1]",q.dI_DiL,"q.dI_DiL",eps);
    UnitTests.checkDoubleRel(basis[n-1],"basis[0]",q.dI_DiH,"q.dI_DiH",eps);

    System.err.println(name+":VkQVMTest:test_Basis(): test OK n="+n+" eps="+eps);
}



void test_Common()
{
    final int n=neq,nmoms=2*n-1;
    final VkQVM<DTYPE> q=getVkQVM();

    final double [][] mm=q.getMoments(nmoms);
    final double [] d2imoms=new double [nmoms];
    q.getAccess().setDFMoments(d2imoms,mm[5],nmoms);
    test_QQdi(n,mm[1],mm[5],d2imoms,q.M);
    test_varPIDt(n,mm[0],mm[1],mm[5],mm[2],q.M,q.varPIDt);
    test_QQpsi(n,mm[0],mm[1],mm[5],q.M);
    test_Basis(q);

    if(true)
    {
	final ACorr ac=ACorr.getACorr_SPUR(n-1,q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[5])),q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[6])),q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[0])),q.M);
	System.err.println("q.a_d2I_d2p="+q.a_d2I_d2p+" ac.x_a_b="+ac.x_a_b);
	UnitTests.checkDoubleRel(q.a_d2I_d2p,"q.a_d2I_d2p",ac.x_a_b,"ac.x_a_b",eps);

	final double [] psi_iH=EVXData.getPsiFromBasis(n-1,EdComm.getIRadauBasisPsi0(n,mm[1],mm[0],q.M),n);
	final ACorr aiH=ACorr.getACorr_NotModifiedPsi(psi_iH,mm[1],mm[4],mm[0],q.M);
	UnitTests.checkDoubleRel(q.aiH_I_dp,"q.aiH_I_dp",aiH.x_a_b,"aiH.x_a_b",eps);
    }

    System.err.println(name+":VkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}


void test_Extra(){}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":VkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Extra();

    System.err.println(name+":VkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":VkQVMTest: starting tests; ver=$Id: VkQVM.java,v 1.89 2022/11/19 11:08:58 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":VkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: VkQVM.java,v 1.89 2022/11/19 11:08:58 mal Exp $");
}
} // END OF class VkQVMTest




}// END OF class VkQVM
