package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: VkQVMMonomials.java,v 1.8 2022/07/10 10:41:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class VkQVMMonomials extends VkQVM<QVMData> {
VkQVMMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static VkQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final VkQVMMonomials ref)
{
    final int nvm=1,npm=2;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new VkQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}

static class VkQVMMonomialsTest extends VkQVMTest<QVMData> {
VkQVMMonomialsTest(final long initrandom)
{
    super("VkQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}


@Override
void test_Extra(){
    final VkQVMMonomials q=getVkQVM();
    final VkQVMLaguerre s=getVkQVMLaguerre();

    UnitTests.checkDoubleRel(q.I_0,"q.I_0",s.I_0,"s.I_0",eps);
    UnitTests.checkDoubleRel(q.dI_Di0,"q.dI_Di0",s.dI_Di0,"s.dI_Di0",eps);
    UnitTests.checkDoubleRel(q.dI_DiL,"q.dI_DiL",s.dI_DiL,"s.dI_DiL",eps);
    UnitTests.checkDoubleRel(q.dI_DiH,"q.dI_DiH",s.dI_DiH,"s.dI_DiH",eps);
    UnitTests.checkDoubleRel(q.I_DiL,"q.I_DiL",s.I_DiL,"s.I_DiL",eps);
    UnitTests.checkDoubleRel(q.I_DiH,"q.I_DiH",s.I_DiH,"s.I_DiH",eps);
    UnitTests.checkDoubleRel(q.dp_DiL,"q.dp_DiL",s.dp_DiL,"s.dp_DiL",eps);
    UnitTests.checkDoubleRel(q.dp_DiH,"q.dp_DiH",s.dp_DiH,"s.dp_DiH",eps);
    UnitTests.checkDoubleRel(q.c_DiL,"q.c_DiL",s.c_DiL,"s.c_DiL",eps);
    UnitTests.checkDoubleRel(q.c_DiH,"q.c_DiH",s.c_DiH,"s.c_DiH",eps);

    UnitTests.checkDoubleRel(q.I_dpL,"q.I_dpL",s.I_dpL,"s.I_dpL",eps);
    UnitTests.checkDoubleRel(q.I_dpH,"q.I_dpH",s.I_dpH,"s.I_dpH",eps);
    UnitTests.checkDoubleRel(q.dI_dpL,"q.dI_dpL",s.dI_dpL,"s.dI_dpL",eps);
    UnitTests.checkDoubleRel(q.dI_dpH,"q.dI_dpH",s.dI_dpH,"s.dI_dpH",eps);

    System.err.println(name+":VkQVMMonomialsTest:test_Extra(): test VkQVMLaguerre OK neq="+neq+" eps="+eps);
}

VkQVMLaguerre getVkQVMLaguerre(){return VkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
VkQVMMonomials getVkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}

} // END of class VkQVMMonomialsTest

public static void main(String [] args)
{
    new VkQVMMonomialsTest(1117L).runTests(5);
} // END main



} // END OF class VkQVMMonomials
