package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.realtimebook.*;

/** WQVM and Rurik type of measures.
 *  @version $Id: WQVM.java,v 1.26 2022/08/08 20:10:47 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class WQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final double w00;
final REVData rdp2,radp2,radp,rv;

WQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final int nmoms=2*neq-1;
    final double [] vmom=new double [nmoms],dpmom=new double [nmoms],dpamom=new double [nmoms],tmom=new double [nmoms];
    access.setM(vmom,0,access.M_I,odata,nmoms);
    w00=vmom[0];
    access.setM(dpamom,1,access.M_DPPLUS,odata,nmoms);
    access.setM(dpmom,1,access.M_DPMINUS,odata,nmoms);
    for(int k=nmoms;--k>=0;){vmom[k]/=w00;dpamom[k]-=dpmom[k];}
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setMT(tmom,nmoms);
    final double [][] QQ=access.getQQ_Matrs(neq,1,new double [][]{vmom,dpmom,dpamom,tmom});
    final double [] QQv=QQ[0],QQdp=QQ[1],QQadp=QQ[2],QQt=QQ[3];
    this.rdp2=getREVData(neq,getM_QQtm1diag_M(neq,QQdp,QQt),QQv,QQdp);
    this.radp2=getREVData(neq,getM_QQtm1diag_M(neq,QQadp,QQt),QQv,QQdp);
    this.radp=getREVData(neq,QQadp,QQv,QQdp);
    this.rv=getREVData(neq,QQt,QQv,QQdp);
}


double [] getM_QQtm1diag_M(final int n,final double [] M,final double [] QQtdiag)
{
    final double [] r=new double [n*n];
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<n;l++)
	{
	    double s=0;
	    for(int m=0;m<n;m++)
	    {
		s+=M[k*n+m]*M[m*n+l]/QQtdiag[n*m+m];
	    }
	    r[k*n+l]=s;
	}
    }
    return r;
}


REVData getREVData(final int n,final double [] QQl,final double [] QQr,final double [] QQdp)
{
    return new REVData(n,DeprecatedHelperEVXData.solveGeneralizedEV(n,QQl,QQr),QQdp);
}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();


/** Rurik EV Data. */
static class REVData {
    final double [] ev;
    /** j - dp/dv values,sigma2_j - standard deviatiom.
     */
    final double [] lam,j,sigma2_j;
    final int n;

    REVData(final int n,
	    final double [] ev,
	    final double [] QQdp)
    {
	this.n=n;
	this.ev=ev;
	this.lam=new double [n];
	this.j=new double [n];
	this.sigma2_j=new double [n];

	for(int k=0;k<n;k++)
	{
	    lam[k]=ev[k];
	    j[k]=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQdp,n);
	    double s2=0;
	    for(int s=0;s<n;s++)
	    { // sum over all other eigenvector basis.
		if(s!=k){
		    s2+=Math.pow(TMatr.getAMatrB(n,ev,(k+1)*n,ev,(s+1)*n,QQdp,n),2);
		}
	    }
	    sigma2_j[k]=s2;
	}
    }
}


static abstract class WQVMTest<DTYPE> extends QTest {

abstract double [][] getMonomialMoments(DTYPE o);
abstract WQVM<DTYPE> getWQVM();
abstract UQVM<DTYPE> getUQVM();

WQVMTest(final String name,
	 final java.util.Random r,final double eps)
{
    super(name,r,eps);
}
void test_Common()
{
    final WQVM<DTYPE> q=getWQVM();
    System.err.println(name+":WQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}



REVData pol_REVData_Rurik_version(final int n,final double [] mom_v,final double [] mom_t,final double [] mom_dp,final double [] mom_dpbasis)
{
    final double [] QQv=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_v,mom_v[0]),
	QQdp=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_dp,1),
	QQdpbasis=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_dpbasis,1),
	QQt=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_t,1);

    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),
	QQdpbasis_x_IQQt_x_QQdpbasis=TMatr.mulMMM(n,QQdpbasis,IQQt,QQdpbasis),
	QQdp_x_IQQv_x_QQdp=TMatr.mulMMM(n,QQdp,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQv),QQdp);

    return pol_REVData_Rurik_version_FromQQ(n,QQdpbasis_x_IQQt_x_QQdpbasis,QQv,QQdp,QQdp_x_IQQv_x_QQdp);
}
REVData pol_REVData_Rurik_version_Vol(final int n,final double [] mom_v,final double [] mom_t,final double [] mom_dp)
{
    final double [] QQv=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_v,mom_v[0]),
	QQdp=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_dp,1),
	QQt=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom_t,1);

    final double [] IQQv=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQv),
	QQdp_x_IQQv_x_QQdp=TMatr.mulMMM(n,QQdp,IQQv,QQdp);

    return pol_REVData_Rurik_version_FromQQ(n,QQt,QQv,QQdp,QQdp_x_IQQv_x_QQdp);
}

REVData pol_REVData_Rurik_version_FromQQ(final int n,final double [] QQl,final double [] QQr,final double [] QQdp,final double [] QQdp2test)
{
    final double [] ev=DeprecatedHelperEVXData.solveGeneralizedEV(n,QQl,QQr);
    double lam=Double.NEGATIVE_INFINITY;
    System.err.println("ev="+java.util.Arrays.toString(ev));
    for(int k=0;k<n;k++)
    {
	if(!(lam<=ev[k])) throw new RuntimeException("Eigenvalues are not sorted");
	lam=ev[k];
	for(int l=0;k<n;k++)
	{
	    final double normx,lamx;
	    if(k==l){normx=1;lamx=lam;}
	    else {normx=lamx=0;}
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(n,ev,(k+1)*n,ev,(l+1)*n,QQr,n),"<q|QQr|l>",normx,"normx",eps);
	    if(l!=0) // test may fail for small lambda
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(n,ev,(k+1)*n,ev,(l+1)*n,QQl,n),"<q|QQl|l>",lamx,"lamx",eps);
	}
    }

    final REVData g=new REVData(n,ev,QQdp);
    for(int k=0;k<n;k++)
    {
	final double lamk=ev[k],nrm=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQr,n);
	UnitTests.checkDoubleRel(nrm,"nrm",1,"1",eps);
	final double jk=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQdp,n)/nrm,
	    sigma2_jk=TMatr.getAMatrB(n,ev,(k+1)*n,ev,(k+1)*n,QQdp2test,n)/nrm-jk*jk;
	UnitTests.checkDoubleRel(g.lam[k],"g.lam[k]",lamk,"ev[k]",eps);
	UnitTests.checkDoubleRel(g.j[k],"g.j[k]",jk,"jk",eps);
	if(n>2&&(k>0||lamk>eps)) // kludge to pass unit test
	UnitTests.checkDoubleRel(g.sigma2_j[k],"g.sigma2_j[k]",sigma2_jk,"sigma2_jk",eps);
    }

    return g;
}
void matchRurik(final String txt,final REVData q,final REVData s)
{
    final int n=q.lam.length;

    System.err.println(txt+"="+java.util.Arrays.toString(q.lam));
    UnitTests.checkDoubleArrayRel1(q.lam,"q.lam",s.lam,"s.lam",eps);
    UnitTests.checkDoubleArrayRel1(q.j,"q.j",s.j,"s.j",eps);
    UnitTests.checkDoubleArrayRel1(q.sigma2_j,"q.sigma2_j",s.sigma2_j,"s.sigma2_j",eps);
    UnitTests.checkDoubleArrayRel1(java.util.Arrays.copyOf(q.ev,n),"q.ev",java.util.Arrays.copyOf(s.ev,n),"s.ev",eps);

    final double [] eq=new double [n],es=new double [n];
    for(int k=0;k<n;k++)
    {
	System.arraycopy(s.ev,(k+1)*n,es,0,n);
	System.arraycopy(q.ev,(k+1)*n,eq,0,n);
	double [] ev_frombasis=convertToMonomials(eq);
	if(Math.signum(ev_frombasis[0])!=Math.signum(es[0]))
	    ev_frombasis=TMatr.getVectorScaled(ev_frombasis,-1);
	UnitTests.checkDoubleArrayRel1(ev_frombasis,"ev_frombasis",es,"es",eps);
    }

    System.err.println(name+":WQVMTest:matchRurik(): test OK neq="+neq+" eps="+eps);
}

void test_Rurik()
{
    final int n=neq;
    final WQVM<DTYPE> q=getWQVM();
    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] mom_v=amoms[0],mom_dp=amoms[1],mom_adp=amoms[2],mom_t=amoms[3];
    matchRurik("rdp2",q.rdp2,pol_REVData_Rurik_version(n,mom_v,mom_t,mom_dp,mom_dp));
    matchRurik("radp2",q.radp2,pol_REVData_Rurik_version(n,mom_v,mom_t,mom_dp,mom_adp));
    matchRurik("radp",q.radp,pol_REVData_Rurik_version_Vol(n,mom_v,mom_adp,mom_dp));
    matchRurik("rv",q.rv,pol_REVData_Rurik_version_Vol(n,mom_v,mom_t,mom_dp));

    System.err.println(name+":WQVMTest:test_Rurik(): test OK neq="+neq+" eps="+eps);
}

void test_UQVM()
{
    final WQVM<DTYPE> q=getWQVM();
    final UQVM<DTYPE> s=getUQVM();

    UnitTests.checkDoubleArrayRel(q.rv.lam,"q.rv.lam",s.rurik.lam,"s.rurik.lam",eps);
    UnitTests.checkDoubleArrayRel(q.rv.j,"q.rv.j",s.rurik.j,"s.rurik.j",eps);
    UnitTests.checkDoubleArrayRel(q.rv.sigma2_j,"q.rv.sigma2_j",s.rurik.sigma2_j,"s.rurik.sigma2_j",eps);

    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] mom_v=amoms[0],mom_dp=amoms[1],mom_t=amoms[3];
    matchRurik("UQVM.rv",q.rv,UQVM.UQVMTest.pol_REVData(neq,mom_v,mom_t,mom_dp,eps));

    System.err.println(name+":WQVMTest:test_UQVM(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":WQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Rurik();
    test_UQVM();

    System.err.println(name+":WQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":WQVMTest: starting tests; ver=$Id: WQVM.java,v 1.26 2022/08/08 20:10:47 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<118;i++)
	{
	    test_Calculations(610+r.nextInt(750),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":WQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: WQVM.java,v 1.26 2022/08/08 20:10:47 mal Exp $");
}
} // END OF class WQVMTest



}// END OF class WQVM
