package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class WQVMLaguerre extends WQVM<QVMDataL> {

WQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static WQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final WQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new WQVMLaguerre(neq,o);
}

DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}

static class WQVMLaguerreTest extends WQVMTest<QVMDataL> {
WQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
WQVMLaguerre getWQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
UQVMLaguerre getUQVM(){return UQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData getPData(final int nmoms)
{
    final QVMData qdata=QVMData.calculateQVMData(1,nmoms,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
    return qdata;
}
double [][] getMonomialMoments(final QVMDataL odata)
{
    final int nmoms=2*neq-1;
    final QVMData qd=getPData(nmoms);
    final double [][] momc=new NQVMLaguerre.NQVMLaguerreTest(name).getMonomialMomentsConvertedFromBasis_dv_dvP_dvM_dpP_dpM(nmoms,new NQVMLaguerre(neq,odata));
    final double [] momadp=new double [nmoms];
    for(int k=nmoms;--k>=0;) momadp[k]=momc[3][k]+momc[4][k];
    return new double [][]{
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomDP(nmoms,qd),
	momadp,
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomT(nmoms)
    };
}


} // END of class WQVMLaguerreTest

public static void main(String [] args)
{
    new WQVMLaguerreTest("WQVMLaguerre").runTests(5);
} // END main



} // END OF class WQVMLaguerre
