package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class WQVMLegendreShifted extends WQVM<QVMDataP> {

WQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static WQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final WQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new WQVMLegendreShifted(neq,o);
}

DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}

static class WQVMLegendreShiftedTest extends WQVMTest<QVMDataP> {
WQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(61572L),1e-4);
}
long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/1)/(9*5*7*8)*(9*5*7*8);
}
WQVMLegendreShifted getWQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
UQVMLegendreShifted getUQVM(){return UQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
double [] convertToMonomials(final double [] p){return WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData [] getPData(final int nmoms)
{
    final QVMData [] qdata=new QVMData [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}
double [][] getMonomialMoments(final QVMDataP odata)
{
    final int nmoms=2*neq-1;
    final QVMData [] qd=getPData(nmoms);
    final double [][] momc=new NQVMLegendreShifted.NQVMLegendreShiftedTest(name).getMonomialMomentsConvertedFromBasis_dv_dvP_dvM_dpP_dpM(nmoms,new NQVMLegendreShifted(neq,odata));
    final double [] momadp=new double [nmoms];
    for(int k=nmoms;--k>=0;) momadp[k]=momc[3][k]+momc[4][k];
    return new double [][]{
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomDP(nmoms,qd),
	momadp,
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomT(nmoms)
    };
}


} // END of class WQVMLegendreShiftedTest

public static void main(String [] args)
{
    // limit to n=5, prone to degeneracy
    new WQVMLegendreShiftedTest("WQVMLegendreShifted").runTests(4);
} // END main



} // END OF class WQVMLegendreShifted
