package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: WkQVM.java,v 1.12 2020/09/27 20:33:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class WkQVM<DTYPE> {

final DTYPE odata;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=getABasis();

final EVXData IT;
final double distDX_psiH;
final State sst;

static class State {
}

WkQVM(final int n,final DTYPE odata,final State rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.odata=odata;

    final int nmoms=2*n-1;

    final double [][] mm=getMoments(nmoms);
    final double [] QQt=M.B.getQQMatr(n,mm[0]),QQi=M.B.getQQMatr(n,mm[1]),QQpi=M.B.getQQMatr(n,mm[2]);
    this.IT=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] IT_psiH=IT.getPsi(n-1),pert_IT_psiH=DeprecatedHelperEVXData.getPsiPertr(IT,n-1,QQpi);
    this.distDX_psiH=TMatr.getAMatrB(pert_IT_psiH,getEDPsi(IT_psiH),QQt,n);
    //System.err.println("IT:<pert_IT_psiH|IT_psiH>="+TMatr.getAMatrB(pert_IT_psiH,IT_psiH,QQt,n)+" must be =0");


    this.sst=null;


}


double [][] getMoments(final int nmoms)
{
    final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access=getAccess();
    final double [] tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms];

    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    TMatr.sumP1(pimom,imom,getPOffset()-getPLast());
    access.setMT(tmom,nmoms);

    return new double [][]{tmom,imom,pimom};
}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getPLast();
abstract double getPOffset();
abstract double [] getEDPsi(final double [] psi);

static abstract class WkQVMTest<DTYPE> extends QTest {

abstract WkQVM<DTYPE> getWkQVM();


WkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_PrintXDir()
{
    final int n=neq-1,nmoms=2*n-1+1;
    final WkQVM<DTYPE> q=getWkQVM();
    final double [][] mm=q.getMoments(nmoms);
    final double [] QQiX=q.M.B.getQQMatr(n,q.M.getRadauMomentsFromMoments(mm[1])),
	QQi=q.M.B.getQQMatr(n,mm[1]),QQt=q.M.B.getQQMatr(n,mm[0]);
    final EVXData IT=EVXData.getEVXDataFromQQ(n,QQi,QQt,q.M);
    final double [] IT_psiH=IT.getPsi(n-1),pert_IT_psiH=DeprecatedHelperEVXData.getPsiPertr(IT,n-1,QQiX);
    final double distDX_psiH=TMatr.getAMatrB(pert_IT_psiH,q.getEDPsi(IT_psiH),QQt,n);
    System.err.println("test_PrintXDir:<pert_IT_psiH|IT_psiH>="+TMatr.getAMatrB(pert_IT_psiH,IT_psiH,QQt,n)+" must be =0");
    System.err.println("test_PrintXDir:<pert_IT_psiH|d/dt|IT_psiH>="+distDX_psiH);
    if(distDX_psiH>0){
	//throw new RuntimeException
	System.err.println("!!!NOTE:n="+n+"\t"+
			   "distDX_psiH="+distDX_psiH+" must be negative");
    }

    System.err.println(name+":WkQVMTest:test_Common(): test OK n="+n+" eps="+eps);
}

void test_Common()
{
    final int n=neq,nmoms=2*n-1;
    final WkQVM<DTYPE> q=getWkQVM();

    System.err.println(name+":WkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}


void test_Extra(){}

abstract UkQVM<?> getUkQVM();
void test_UkQVM()
{
    final UkQVM<?> s=getUkQVM();
    final WkQVM<DTYPE> q=getWkQVM();

    EVXData.matchEVXData(q.IT,s.IT,true,true,eps);
    UnitTests.checkDoubleRel(q.distDX_psiH,"q.distDX_psiH",s.distDX_ITp,"s.distDX_ITp",eps);

    System.err.println(name+":WkQVMTest:test_UkQVM(): test OK neq="+neq+" eps="+eps);
}




void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":WkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_PrintXDir();
    test_UkQVM();
    test_Extra();

    System.err.println(name+":WkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":WkQVMTest: starting tests; ver=$Id: WkQVM.java,v 1.12 2020/09/27 20:33:22 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":WkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: WkQVM.java,v 1.12 2020/09/27 20:33:22 mal Exp $");
}
} // END OF class WkQVMTest




}// END OF class WkQVM
