package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: WkQVMMonomials.java,v 1.6 2022/07/10 10:41:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class WkQVMMonomials extends WkQVM<QVMData> {
WkQVMMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static WkQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final WkQVMMonomials ref)
{
    final int nvm=1,npm=2;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new WkQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
DeprecatedHelperQVMDataDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataDirectAccessWithBasis();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}

static class WkQVMMonomialsTest extends WkQVMTest<QVMData> {
WkQVMMonomialsTest(final long initrandom)
{
    super("WkQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}


@Override
void test_Extra(){
    final WkQVMMonomials q=getWkQVM();
    final WkQVMLaguerre s=getWkQVMLaguerre();

    EVXData.matchEVXData(q.IT,s.IT,false,true,eps);
    UnitTests.checkDoubleRel(q.distDX_psiH,"q.distDX_psiH",s.distDX_psiH,"s.distDX_psiH",eps);

    System.err.println(name+":WkQVMMonomialsTest:test_Extra(): test WkQVMLaguerre OK neq="+neq+" eps="+eps);
}

WkQVMLaguerre getWkQVMLaguerre(){return WkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
WkQVMMonomials getWkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
UkQVMMonomials getUkQVM(){return UkQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class WkQVMMonomialsTest

public static void main(String [] args)
{
    new WkQVMMonomialsTest(1117L).runTests(5);
} // END main



} // END OF class WkQVMMonomials
