package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.realtimebook.*;

/** XQVM and a set of measures.
 *  @version $Id: XQVM.java,v 1.70 2022/08/08 20:10:47 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class XQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final double w00;

/** Rurik evolution. */
final double ovdts0,sigma2_ovdts0,ovdvs0,sigma2_ovdvs0,ovK00,
    otdts0,sigma2_otdts0,otdvs0,sigma2_otdvs0,otK00,
    ovdtso,sigma2_ovdtso,otdvso,sigma2_otdvso,
    nvdts0,sigma2_nvdts0,nvdvs0,sigma2_nvdvs0,nvK00,nvK00a,nvdts0a,sigma2_nvdts0a,nvdtsoa,sigma2_nvdtsoa,
    ntdts0,sigma2_ntdts0,ntdvs0,sigma2_ntdvs0,sigma2A_otdvs0,ntK00,ntK00a,ntdvs0a,sigma2_ntdvs0a,ntdvsoa,sigma2_ntdvsoa;
final double sigma2T_nvdts0a,sigma2T_ntdvs0a,sigma2A_ovdts0a,sigma2A_otdvs0a,
    ntrvs0a,sigma2T_ntrvs0a,sigma2A_otrvs0a;
final double ovrdts0,sigma2_ovrdts0,otrdvs0,sigma2_otrdvs0;
final double tsL;
final double rvs0,sigma2A_rvs0;
final double nvrts0,ovrts0,
    sigma2A_ovrts0;


XQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final int nmoms=2*neq-1;

    final double [] vmom=new double [nmoms];
    access.setM(vmom,0,access.M_I,odata,nmoms);
    w00=vmom[0];
    final double [] dtsmom=new double [nmoms],dvsmom=new double [nmoms],rdtsmom=new double [nmoms],rdvsmom=new double [nmoms],tmom=new double [nmoms];
    access.setM(dvsmom,0,access.M_VS,odata,nmoms);
    access.setM(dtsmom,0,access.M_TS,odata,nmoms);
    access.setM(rdvsmom,0,access.M_RVS,odata,nmoms);
    access.setM(rdtsmom,0,access.M_RTS,odata,nmoms);
    access.setMT(tmom,nmoms);
    if(true)
    {
	final double [] res=getdPAndSigma_PSI(neq,getX0(),vmom,1,new double [][]{dtsmom,dvsmom,rdtsmom,rdvsmom});
	ovdts0=res[0];
	sigma2_ovdts0=res[1];
	ovdvs0=res[2];
	sigma2_ovdvs0=res[3];
	ovrdts0=res[4];
	sigma2_ovrdts0=res[5];
	ovK00=res[8];

	final double [] resp=getdPAndSigma_PSIPol(neq,getX0(),vmom,1,new double [][]{dtsmom});
	ovdtso=resp[0];
	sigma2_ovdtso=resp[1];
    }
    if(true)
    {
	final double [] res=getdPAndSigma_PSI(neq,getX0(),tmom,1,new double [][]{dtsmom,dvsmom,rdtsmom,rdvsmom});
	otdts0=res[0];
	sigma2_otdts0=res[1];
	otdvs0=res[2];
	sigma2_otdvs0=res[3];
	otrdvs0=res[6];
	sigma2_otrdvs0=res[7];
	otK00=res[8];

	final double [] resp=getdPAndSigma_PSIPol(neq,getX0(),tmom,1,new double [][]{dvsmom});
	otdvso=resp[0];
	sigma2_otdvso=resp[1];
    }

    final double [][] QQ=access.getQQ_Matrs(neq,1,new double [][]{vmom,tmom,dtsmom,dvsmom,rdtsmom,rdvsmom});
    final double [] QQdts=QQ[2],QQdvs=QQ[3],QQrdvs=QQ[5];
    if(true)
    {
	final double x0=getX0();
	final double [] res=getdPAndSigma_PSI2(neq,x0,vmom,1,new double [][]{dtsmom,dvsmom,rdtsmom});
	nvdts0=res[0];
	sigma2_nvdts0=res[1];
	nvdvs0=res[4];
	sigma2_nvdvs0=res[5];
	nvrts0=res[8];
	ovrts0=res[10];
	sigma2A_ovrts0=res[11];

	nvK00=res[12];

	final double [] QQi=QQ[0],m_kk=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQi),QQd=QQdts,
	    m_kk_x_QQd=TMatr.mulMM(neq,m_kk,QQd),
	    m_dkk=TMatr.mulMM(neq,m_kk_x_QQd,m_kk),
	    m_d2kk=TMatr.mulMM(neq,m_kk_x_QQd,m_dkk),
	    m_sigma2=TMatr.mulMM(neq,m_dkk,QQd);

	final double [][] kks=access.getKK(neq,new double [][] {m_kk,m_dkk,m_d2kk});
	final double [] kk=kks[0],kkd=kks[1],kkd2=kks[2];
	nvK00a=access.fx(kk,x0);
	nvdts0a=access.fx(kkd,x0)/nvK00a;
	sigma2_nvdts0a=access.fx(kkd2,x0)/nvK00a-nvdts0a*nvdts0a;
	sigma2T_nvdts0a=TMatr.getSpurM(neq,m_sigma2)-nvdts0a*nvdts0a;
	sigma2A_ovdts0a=(TMatr.getAMatrA(neq,vmom,m_d2kk,neq)-ovdts0*ovdts0);

	final double KKo=kk[kk.length-1];
	nvdtsoa=kkd[kkd.length-1]/KKo;
	sigma2_nvdtsoa=kkd2[kkd2.length-1]/KKo-nvdtsoa*nvdtsoa;
    }

    if(true)
    {
	final double x0=getX0();
	final double [] res=getdPAndSigma_PSI2(neq,x0,tmom,1,new double [][]{dtsmom,dvsmom,rdvsmom});
	ntdts0=res[0];
	sigma2_ntdts0=res[1];
	ntdvs0=res[4];
	sigma2_ntdvs0=res[5];
	sigma2A_otdvs0=res[7];
	rvs0=res[8];
	sigma2A_rvs0=res[11];
	ntK00=res[12];

	final double [] QQi=QQ[1], m_kk=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQi),QQd=QQdvs,QQr=QQrdvs,
	    m_kk_x_QQd=TMatr.mulMM(neq,m_kk,QQd),
	    m_dkk=TMatr.mulMM(neq,m_kk_x_QQd,m_kk),
	    m_d2kk=TMatr.mulMM(neq,m_kk_x_QQd,m_dkk),
	    m_dsigma2=TMatr.mulMM(neq,m_dkk,QQd),
	    m_kk_x_QQr=TMatr.mulMM(neq,m_kk,QQr),
	    m_rkk=TMatr.mulMM(neq,m_kk_x_QQr,m_kk),
	    m_r2kk=TMatr.mulMM(neq,m_kk_x_QQr,m_rkk),
	    m_rsigma2=TMatr.mulMM(neq,m_rkk,QQr);

	final double [][] kks=access.getKK(neq,new double [][] {m_kk,m_dkk,m_d2kk,m_rkk});
	final double [] kk=kks[0],kkd=kks[1],kkd2=kks[2],kkr=kks[3];
	ntK00a=access.fx(kk,x0);
	ntdvs0a=access.fx(kkd,x0)/ntK00a;
	sigma2_ntdvs0a=access.fx(kkd2,x0)/ntK00a-ntdvs0a*ntdvs0a;
	sigma2T_ntdvs0a=TMatr.getSpurM(neq,m_dsigma2)-ntdvs0a*ntdvs0a;
	sigma2A_otdvs0a=(TMatr.getAMatrA(neq,tmom,m_d2kk,neq)-otdvs0*otdvs0);

	final double KKo=kk[kk.length-1];
	ntdvsoa=kkd[kkd.length-1]/KKo;
	sigma2_ntdvsoa=kkd2[kkd2.length-1]/KKo-ntdvsoa*ntdvsoa;

	ntrvs0a=access.fx(kkr,x0)/ntK00a;
	sigma2T_ntrvs0a=TMatr.getSpurM(neq,m_rsigma2)-ntrvs0a*ntrvs0a;
	sigma2A_otrvs0a=(TMatr.getAMatrA(neq,tmom,m_r2kk,neq)-otrdvs0*otrdvs0);
    }

    this.tsL=getTSL();
}
abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double getX0();
abstract double getTSL();


static double getTSL(final BookItem b,final BookItem b_buy,final BookItem b_sell)
{
    final double sS;
    if(b==null) sS=Double.NaN;
    else if(b.buy_sell_indicator==BookItem.BOOK_BUY){sS=b_sell!=null?(b.price-b_sell.price):0;}
    else if(b.buy_sell_indicator==BookItem.BOOK_SELL){sS=b_buy!=null?(b.price-b_buy.price):0;}
    else sS=0;
    return sS;
}


double [] getdPAndSigma_PSI(final int n,final double x0,final double [] momv,final double vscale,final double [][] momdps)
{
    final double [] QQ=access.getQQ_Matrs(n,vscale,new double [][]{momv})[0];
    final double [] psi=new double [n],r=new double [n];
    for(int k=0;k<n;k++)
    {
	r[k]=1;psi[k]=access.fx(r,x0);r[k]=0;
    }
    final int [] ipvt=new int [n];
    final double [] resdpsigma=new double [2*momdps.length+1];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(QQ,n,n,ipvt);
    if(pinfo!=0) {
	java.util.Arrays.fill(resdpsigma,Double.NaN);
	return resdpsigma;
    }
    com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,n,ipvt,psi,1);
    final double K00=access.fx(psi,x0),sqrtK00=Math.sqrt(K00);
    int o=0;
    for(int k=0;k<momdps.length;k++)
    {
	final double [] momdp=momdps[k];
	final double d=TMatr.mulVVn1(psi,momdp);
	System.arraycopy(momdp,0,r,0,n);
	com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,n,ipvt,r,1);
	final double dp0=d/sqrtK00,sigma2_dp0=TMatr.mulVVn1(r,momdp)-dp0*dp0;
	resdpsigma[o++]=dp0;
	resdpsigma[o++]=sigma2_dp0;
    }
    resdpsigma[o++]=K00;
    return resdpsigma;
}

double [] getdPAndSigma_PSIPol(final int nn,final double x0_forsign,final double [] momv,final double vscale,final double [][] momdps)
{
    final int n=nn-1;
    final double [] QQ=access.getQQ_Matrs(nn,vscale,new double [][]{momv})[0],QQs=QQ.clone();
    final double [] psi=new double [nn];
    psi[n]=1;
    for(int k=0;k<n;k++)
    {
	psi[k]=-QQ[n*nn+k];
    }
    final int [] ipvt=new int [n];
    final double [] resdpsigma=new double [2*momdps.length];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(QQ,n,nn,ipvt);
    if(pinfo!=0) {
	java.util.Arrays.fill(resdpsigma,Double.NaN);
	return resdpsigma;
    }
    com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,nn,ipvt,psi,1);
    double norm=Math.sqrt(TMatr.getAMatrA(nn,psi,QQs,nn));
    if(!(access.fx(psi,x0_forsign)>0)) norm=-norm;
    for(int k=0;k<psi.length;k++) psi[k]/=norm;

    final double [] r=new double [n];
    int o=0;
    for(int k=0;k<momdps.length;k++)
    {
	final double [] momdp=momdps[k];
	final double d=TMatr.mulVVn1(psi,momdp);
	System.arraycopy(momdp,0,r,0,n);
	com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,nn,ipvt,r,1);
	final double sigma2_dp0=TMatr.mulVVn1(r,momdp);
	resdpsigma[o++]=d;
	resdpsigma[o++]=sigma2_dp0;
    }
    return resdpsigma;
}

double [] getdPAndSigma_PSI2(final int n,final double x0,final double [] momv,final double vscale,final double [][] momdps)
{
    final double [][] qqmom=new double [1+momdps.length][];
    qqmom[0]=momv;
    System.arraycopy(momdps,0,qqmom,1,momdps.length);
    final double [][] qq=access.getQQ_Matrs(n,1,qqmom);
    final double [] QQ=qq[0];
    for(int k=QQ.length;--k>=0;) QQ[k]/=vscale;

    double [] psi=new double [n],r=new double [n];
    for(int k=0;k<n;k++)
    {
	r[k]=1;psi[k]=access.fx(r,x0);r[k]=0;
    }
    final int [] ipvt=new int [n];
    final double [] resdpsigma=new double [4*momdps.length+1];
    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(QQ,n,n,ipvt);
    if(pinfo!=0) {
	java.util.Arrays.fill(resdpsigma,Double.NaN);
	return resdpsigma;
    }
    com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,n,ipvt,psi,1);
    final double K00=access.fx(psi,x0);
    int o=0;
    for(int k=0;k<momdps.length;k++)
    {
	final double [] QQdp=qq[k+1];
	r=TMatr.getMatr_x_A(n,QQdp,psi);
	final double d=TMatr.mulVV(psi,r);
	com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,n,ipvt,r,1);
	double [] f=TMatr.getMatr_x_A(n,QQdp,r);
	final double dp0=d/K00,sigma2_dp0=TMatr.mulVV(f,psi)/K00-dp0*dp0;
	resdpsigma[o++]=dp0;
	resdpsigma[o++]=sigma2_dp0;
	System.arraycopy(momdps[k],0,r,0,n);
	com.polytechnik.utils.Linsystems.dgesl_m1(QQ,n,n,ipvt,r,1);
	final double fo=access.fx(r,x0);
	final double sigma2=TMatr.mulVVn1(r,momdps[k]);
	resdpsigma[o++]=fo/Math.sqrt(K00);
	resdpsigma[o++]=(sigma2-fo*fo/K00);
    }
    resdpsigma[o++]=K00;
    return resdpsigma;
}



static abstract class XQVMTest<DTYPE> extends QTest {

abstract double [][] getMonomialMoments(DTYPE o);
abstract XQVM<DTYPE> getXQVM();
abstract VQVM<DTYPE> getVQVM();
abstract boolean isRadauRight();


XQVMTest(final String name,
	 final java.util.Random r,final double eps)
{
    super(name,r,eps);
}
void test_Common()
{
    final XQVM<DTYPE> q=getXQVM();
    UnitTests.checkDoubleRel(q.nvK00,"q.nvK00",q.ovK00,"q.ovK00",eps);
    UnitTests.checkDoubleRel(q.nvK00,"q.nvK00",q.nvK00a,"q.nvK00a",eps);
    UnitTests.checkDoubleRel(q.nvdts0,"q.nvdts0",q.nvdts0a,"q.nvdts0a",eps);
    if(neq>1) UnitTests.checkDoubleRel(q.sigma2_nvdts0,"q.sigma2_nvdts0",q.sigma2_nvdts0a,"q.sigma2_nvdts0a",eps);

    UnitTests.checkDoubleRel(q.ntK00,"q.ntK00",q.otK00,"q.otK00",eps);
    UnitTests.checkDoubleRel(q.ntK00,"q.ntK00",q.ntK00a,"q.ntK00a",eps);
    UnitTests.checkDoubleRel(q.ntdvs0,"q.ntdvs0",q.ntdvs0a,"q.ntdvs0a",eps);
    if(neq>1) UnitTests.checkDoubleRel(q.sigma2_ntdvs0,"q.sigma2_ntdvs0",q.sigma2_ntdvs0a,"q.sigma2_ntdvs0a",eps);

    System.err.println(name+":XQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}
void test_VQVM()
{
    final XQVM<DTYPE> q=getXQVM();
    final VQVM<DTYPE> s=getVQVM();

    final double w0=q.w00;

    UnitTests.checkDoubleRel(q.ovK00*w0,"q.ovK00*w0",s.oK00,"s.oK00",eps);
    UnitTests.checkDoubleRel(q.ovdts0*Math.sqrt(w0),"q.ovdts0*sqrt(w0)",s.odts0,"s.odts0",eps);
    if(neq>1) UnitTests.checkDoubleRel(q.sigma2_ovdts0*w0,"q.sigma2_ovdts0*w0",s.sigma2_odts0,"s.sigma2_odts0",eps);
    UnitTests.checkDoubleRel(q.ovdvs0*Math.sqrt(w0),"q.ovdvs0*sqrt(w0)",s.odvs0,"s.odvs0",eps);
    if(neq>1) UnitTests.checkDoubleRel(q.sigma2_ovdvs0*w0,"q.sigma2_ovdvs0*w0",s.sigma2_odvs0,"s.sigma2_odvs0",eps);
    UnitTests.checkDoubleRel(q.nvdts0*w0,"q.nvdts0*w0",s.ndts0,"s.ndts0",eps);
    if(neq>1) UnitTests.checkDoubleRel(q.sigma2_nvdts0*w0*w0,"q.sigma2_nvdts0*w0*w0",s.sigma2_ndts0,"s.sigma2_ndts0",eps);
    UnitTests.checkDoubleRel(q.nvdvs0*w0,"q.nvdvs0*w0",s.ndvs0,"s.ndvs0",eps);
    if(neq>1) UnitTests.checkDoubleRel(q.sigma2_nvdvs0*w0*w0,"q.sigma2_nvdvs0*w0*w0",s.sigma2_ndvs0,"s.sigma2_ndvs0",eps);

    UnitTests.checkDoubleRel(q.otdts0,"q.otdts0",s.rdts0,"s.rdts0",eps);
    if(neq>1) UnitTests.checkDoubleRel(q.sigma2_otdts0,"q.sigma2_otdts0",s.sigma2_rdts0,"s.sigma2_rdts0",eps);
    UnitTests.checkDoubleRel(q.otdvs0,"q.otdvs0",s.rdvs0,"s.rdvs0",eps);
    if(neq>1) UnitTests.checkDoubleRel(q.sigma2_otdvs0,"q.sigma2_otdvs0",s.sigma2_rdvs0,"s.sigma2_rdvs0",eps);

    UnitTests.checkDoubleRel(q.ntdvs0,"q.ntdvs0",s.dvse0,"s.dvse0",eps);

    System.err.println(name+":XQVMTest:test_VQVM(): test OK neq="+neq+" eps="+eps);
}

static class PolStoreQQtestMonomials {
final int n;
final double x0,p0,sigma2_p0,K0;
final double [] p,kk,kkd,kkd2;
PolStoreQQtestMonomials(final int n,final double x0,final double [] mom,final double scale_mom,final double [] momd,final double scale_momd)
{
    this.n=n;
    this.x0=x0;
    final double [] QQ=NQVM.NQVMTest.pol_getMM_Matr_scal(n,mom,scale_mom),
	QQd=NQVM.NQVMTest.pol_getMM_Matr_scal(n,momd,scale_momd);

    final double [] In=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQ);
    this.p=TMatr.getMatr_x_A(n,In,java.util.Arrays.copyOf(momd,n));
    kk=OQVM.OQVMTest.getKK_fromQQ(n,In);
    this.K0=Polynomials.getD0(this.kk,x0);
    final double dp=Polynomials.getD0(this.p,x0);
    this.sigma2_p0=TMatr.getAMatrA(n,momd,In,n)-(dp*dp)/K0;
    this.p0=dp/Math.sqrt(K0);
    kkd=OQVM.OQVMTest.getKK_fromQQ(n,TMatr.mulMMM(n,In,QQd,In));
    kkd2=OQVM.OQVMTest.getKK_fromQQ(n,TMatr.mulMM(n,TMatr.mulMM(n,In,QQd),TMatr.mulMM(n,TMatr.mulMM(n,In,QQd),In)));

}

void matchK(final double K0,final double od0,final double sigma2_od0,final double nd0,final double sigma2_nd0,final double ndo,final double sigma2_ndo,final double eps)
{
    UnitTests.checkDoubleRel(K0,"K0",this.K0,"this.K0",eps);

    UnitTests.checkDoubleRel(od0,"od0",this.p0,"this.p0",eps);
    if(n>1) UnitTests.checkDoubleRel(sigma2_od0,"sigma2_od0",this.sigma2_p0,"this.sigma2_p0",eps);
    final double nd0c=Polynomials.getD0(this.kkd,x0)/this.K0;
    UnitTests.checkDoubleRel(nd0,"nd0",nd0c,"kkd(x0)/kk(x0)",eps);
    if(n>1) UnitTests.checkDoubleRel(sigma2_nd0,"sigma2_nd0",Polynomials.getD0(this.kkd2,x0)/K0-nd0c*nd0c,"sigma2_nd0c",eps);
    final double Ko=this.kk[kk.length-1],ndoc=this.kkd[this.kkd.length-1]/Ko,sigma2_ndoc=this.kkd2[this.kkd2.length-1]/Ko-ndoc*ndoc;
    UnitTests.checkDoubleRel(ndo,"ndo",ndoc,"ndoc",eps);
    if(n>1) UnitTests.checkDoubleRel(sigma2_ndo,"sigma2_ndo",sigma2_ndoc,"sigma2_ndoc",eps);

    System.err.println("XQVMTest:PolStoreQQtestMonomials:matchK(): test OK n="+n+" eps="+eps);
}
} //  END OF class PolStoreQQtestMonomials

void test_Monomials()
{

    final XQVM<DTYPE> q=getXQVM();
    final double [][] amoms=getMonomialMoments(q.odata);
    final double [] momv=amoms[0],momt=amoms[1],momdvs=amoms[2],momdts=amoms[3];

    new PolStoreQQtestMonomials(neq,q.getX0(),momv,1,momdts,1).matchK(q.ovK00,q.ovdts0,q.sigma2_ovdts0,q.nvdts0,q.sigma2_nvdts0,q.nvdtsoa,q.sigma2_nvdtsoa,eps);
    new PolStoreQQtestMonomials(neq,q.getX0(),momt,1,momdvs,1).matchK(q.otK00,q.otdvs0,q.sigma2_otdvs0,q.ntdvs0,q.sigma2_ntdvs0,q.ntdvsoa,q.sigma2_ntdvsoa,eps);

    System.err.println(name+":XQVMTest:test_Monomials(): test OK neq="+neq+" eps="+eps);
}

/** Unit test answer in basis.
    ds=<P_n^2 f>.
    sigma2_ds=\sum_{k=0}^{n-1} <P_n P_k f>^2
*/
void test_Basis(final int n,final double [][] basis,final double [] QQd,final double o_ds,final double o_sigma2_ds,final double n_ds,final double n_sigma2_ds)
{
    final double [] r=new double [n],rn=basis[n-1];
    double ods=0,nds=0,sigma2_ods=0,sigma2_nds=0;
    for(int k=n;--k>=0;)
    {
	java.util.Arrays.fill(r,0);
	System.arraycopy(basis[k],0,r,0,basis[k].length);
	final double mo=TMatr.mulVVn1(r,QQd),mn=TMatr.getAMatrB(n,r,rn,QQd,n);
	if(k==(n-1)) {ods=mo;nds=mn;} // last element is the value
	else {sigma2_ods+=mo*mo;sigma2_nds+=mn*mn;}
    }
    System.err.println("ods="+ods+" nds="+nds+" sigma2_ods="+sigma2_ods+" sigma2_nds="+sigma2_nds+" sigma2_ods+ods*ods="+(sigma2_ods+ods*ods));
    UnitTests.checkDoubleRel(ods,"ods",o_ds,"o_ds",eps);
    UnitTests.checkDoubleRel(sigma2_ods,"sigma2_ods",o_sigma2_ds,"o_sigma2_ds",eps);
    UnitTests.checkDoubleRel(nds,"nds",n_ds,"n_ds",eps);
    UnitTests.checkDoubleRel(sigma2_nds,"sigma2_nds",n_sigma2_ds,"n_sigma2_ds",eps);

    System.err.println(name+":XQVMTest:test_Basis(): test OK n="+n+" eps="+eps);
}
/** Generate Legendre Interpolatory Polynomials with an edge at x0 basis.
 */
static double [][][] getRadauBasis(final int n,final double [] rnodes,final double x0,final double [] QQ)
{
    if(n!=rnodes.length) throw new RuntimeException();
    final PolynomialRootsFinder.PRoots u=
	new com.polytechnik.utils.PolynomialRootsJenkinsTraubORIG().findRoots(rnodes);
    final double [] xnodes=u.re_root;
    if(u.nroots!=n-1) throw new RuntimeException("n-1="+(n-1)+" u.nroots="+u.nroots);
    final double [][] basis=new double [n][];
    final double [] xr=new double [n],wr=new double [n];
    for(int k=0;k<n;k++)
    {
	final double xx;
	final double [] b;
	if(k<(n-1))
	{
	    xx=xnodes[k];
	    final double [] p=Polynomials.sdiv1(rnodes,xx).q;
	    b=Polynomials.mult2Pol(p,new double[]{-x0,1});
	}
	else
	{
	    xx=x0;
	    b=rnodes.clone();
	}
	final double norm=Math.sqrt(TMatr.getAMatrA(b.length,b,QQ,n));
	for(int l=0;l<b.length;l++) b[l]/=norm;
	basis[k]=b;
	xr[k]=xx;
	wr[k]=1/Math.pow(Polynomials.getD0(b,xx),2);
    }
    System.err.println("wr="+java.util.Arrays.toString(wr));
    return new double [][][]{getBasisFx0positive(x0,basis),new double [][]{xr,wr}};
}

static double [][] getBasisFx0positive(final double x0,final double [][] basis)
{
    final double [][] b=new double [basis.length][];
    for(int k=0;k<basis.length;k++)
    {
	if(Polynomials.getD0(basis[k],x0)>0) b[k]=basis[k];
	else b[k]=TMatr.getVectorScaled(basis[k],-1);
    }
    return b;
}

void test_sigma2(final int n,final double [][] basis,final double [] QQ,final double [] QQd,final double [] xr,final double [] wr,final double oK00,final double o_ds,final double n_ds,final double sigma2_T,final double sigma2_A)
{
    if(basis.length!=n) throw new RuntimeException();
    final double [] a=new double [n],b=new double [n],F=new double [n*n],aw=new double [n],aws=new double [n];
    double ods=0,nds=0,sigma2_ods=0,sigma2_nds=0,sigma2=0,dof=0,K00=0;
    for(int k=0;k<n;k++)
    {
	java.util.Arrays.fill(a,0);
	System.arraycopy(basis[k],0,a,0,basis[k].length);
	final double m=TMatr.mulVVn1(a,QQd);
	sigma2+=m*m;
	aws[k]=Math.sqrt(wr[k]);
	if(Polynomials.getD0(a,xr[k])<=0) aws[k]=-aws[k];
	UnitTests.checkDoubleRel(TMatr.mulVVn1(a,QQ),"<psi>",aws[k],"sqrt(w[k])",eps);
	TMatr.sumP1(aw,a,aws[k]);

	for(int l=0;l<n;l++)
	{
	    java.util.Arrays.fill(b,0);
	    System.arraycopy(basis[l],0,b,0,basis[l].length);
	    F[k*n+l]=TMatr.getAMatrB(n,a,b,QQd,n);
	}
	if(k==(n-1)) {dof=m;K00=1/wr[k];}
    }
    final double ds=F[(n-1)*n+(n-1)];
    UnitTests.checkDoubleRel(ds,"ds",n_ds,"n_ds",eps);
    final double [] F2=TMatr.mulMM(n,F,F);
    final double sigma2_t=TMatr.getSpurM(n,F2)-ds*ds;
    UnitTests.checkDoubleRel(sigma2_T,"sigma2_T",sigma2_t,"sigma2_t",eps);

    final double c=TMatr.getAMatrA(n,aw,QQ,n);
    System.err.println("c="+c+" QQ[0]="+QQ[0]);
    UnitTests.checkDoubleRel(c,"c",QQ[0],"QQ[0]",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(n,aws,F,n),"<f>",QQd[0],"QQd[0]",eps);

    UnitTests.checkDoubleRel(oK00,"oK00",K00,"K00",eps);
    UnitTests.checkDoubleRel(o_ds,"o_ds",dof,"dof",eps);
    final double s2=TMatr.getAMatrA(n,aws,F2,n);
    UnitTests.checkDoubleRel(sigma2,"sigma2",s2,"s2",eps);
    final double sigma2_a=(s2-dof*dof);
    UnitTests.checkDoubleRel(sigma2_A,"sigma2_A",sigma2_a,"sigma2_a",eps);

    System.err.println(name+":XQVMTest:test_sigma2(): test OK neq="+neq+" eps="+eps);
}

void test_VariousBasises()
{
    final int n=neq;
    neq=n+1; // kludge for Radau moments
    final double [][] amoms=getMonomialMoments(getXQVM().odata);
    neq=n;
    final XQVM<DTYPE> q=getXQVM();
    final com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis M=
	new com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis()
	{
	    @Override
	    public double [] getRadauMultiplied(final double [] nodes0)
	    {
		final double [] m=isRadauRight()?new double []{q.getX0(),-1}:new double []{-q.getX0(),1};
		return Polynomials.mult2Pol(nodes0,m);
	    }
	};
    final double [] momv=amoms[0],momt=amoms[1],momdvs=amoms[2],momdts=amoms[3],momrvs=amoms[4];
    {
    final double [] QQd=NQVM.NQVMTest.pol_getMM_Matr_scal(n,momdts,1);
    final double [][] R=M.getOrthogonalPolynomialsFirstKind(n+1,momv);
    if(n>1) test_Basis(n,getBasisFx0positive(q.getX0(),R),QQd,q.ovdtso,q.sigma2_ovdtso,q.nvdtsoa,q.sigma2_nvdtsoa);
    final double [] QQ=NQVM.NQVMTest.pol_getMM_Matr_scal(n,momv,1);
    final double [][][] rr=getRadauBasis(n,M.getRadauPolynomials(R)[n-1],q.getX0(),QQ);
    final double [][] rbasis=rr[0];final double [] xr=rr[1][0],wr=rr[1][1];
    if(n>1) test_Basis(n,rbasis,QQd,q.ovdts0,q.sigma2_ovdts0,q.nvdts0a,q.sigma2_nvdts0a);
    if(n>1) test_sigma2(n,rbasis,QQ,QQd,xr,wr,q.ovK00,q.ovdts0,q.nvdts0a,q.sigma2T_nvdts0a,q.sigma2A_ovdts0a);
    }
    {
    final double [] QQd=NQVM.NQVMTest.pol_getMM_Matr_scal(n,momdvs,1),QQr=NQVM.NQVMTest.pol_getMM_Matr_scal(n,momrvs,1);
    final double [][] R=M.getOrthogonalPolynomialsFirstKind(n+1,momt);
    if(n>1) test_Basis(n,getBasisFx0positive(q.getX0(),R),QQd,q.otdvso,q.sigma2_otdvso,q.ntdvsoa,q.sigma2_ntdvsoa);
    final double [] QQ=NQVM.NQVMTest.pol_getMM_Matr_scal(n,momt,1);
    final double [][][] rr=getRadauBasis(n,M.getRadauPolynomials(R)[n-1],q.getX0(),QQ);
    final double [][] rbasis=rr[0];final double [] xr=rr[1][0],wr=rr[1][1];
    if(n>1) test_Basis(n,rbasis,QQd,q.otdvs0,q.sigma2_otdvs0,q.ntdvs0a,q.sigma2_ntdvs0a);
    if(n>1) test_sigma2(n,rbasis,QQ,QQd,xr,wr,q.otK00,q.otdvs0,q.ntdvs0a,q.sigma2T_ntdvs0a,q.sigma2A_otdvs0a);
    if(n>1) test_sigma2(n,rbasis,QQ,QQr,xr,wr,q.otK00,q.otrdvs0,q.ntrvs0a,q.sigma2T_ntrvs0a,q.sigma2A_otrvs0a);
    }

    System.err.println(name+":XQVMTest:test_VariousBasises(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":XQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_VQVM();
    test_Monomials();
    test_VariousBasises();


    System.err.println(name+":XQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":XQVMTest: starting tests; ver=$Id: XQVM.java,v 1.70 2022/08/08 20:10:47 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":XQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: XQVM.java,v 1.70 2022/08/08 20:10:47 mal Exp $");
}
} // END OF class XQVMTest






}// END OF class XQVM
