package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.Laguerre;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class XQVMLaguerre extends XQVM<QVMDataL> {

XQVMLaguerre(final int neq,final QVMDataL odata){super(neq,odata);}

public static XQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final XQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new XQVMLaguerre(neq,o);
}

double getX0(){return 0;}
DeprecatedHelperQVMDataLDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataLDirectAccessWithBasis();}
double getTSL(){return getTSL(odata.trade_last,odata.trade_last!=null?odata.trade_last.best_buy:null,odata.trade_last!=null?odata.trade_last.best_sell:null);}

static class XQVMLaguerreTest extends XQVMTest<QVMDataL> {
XQVMLaguerreTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
XQVMLaguerre getXQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
VQVMLaguerre getVQVM(){return VQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
boolean isRadauRight(){return false;}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}
QVMData getPData(final int nmoms)
{
    final QVMData qdata=QVMData.calculateQVMData(1,nmoms,4+r.nextInt(3),currenttime,tau,tb,tb.length,null);
    return qdata;
}
double [][] getMonomialMoments(final QVMDataL odata)
{
    final int nmoms=2*neq-1;
    final QVMData qd=getPData(nmoms);
    return new double [][]{
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,odata.p_offset),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomT(nmoms),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomVS(nmoms,qd),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomTS(nmoms,qd),
	MQVMLaguerre.MQVMLaguerreTest.getMonomialMomRVS(nmoms,qd)
    };
}

} // END of class XQVMLaguerreTest

public static void main(String [] args)
{
    new XQVMLaguerreTest("XQVMLaguerre").runTests(5);
} // END main



} // END OF class XQVMLaguerre
