package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.LegendreShifted;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

public class XQVMLegendreShifted extends XQVM<QVMDataP> {

XQVMLegendreShifted(final int neq,final QVMDataP odata){super(neq,odata);}

public static XQVMLegendreShifted calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final XQVMLegendreShifted ref)
{
    final int npm=3;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*neq-1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new XQVMLegendreShifted(neq,o);
}

double getX0(){return 1;}
DeprecatedHelperQVMDataPDirectAccessWithBasis getAccess(){return new DeprecatedHelperQVMDataPDirectAccessWithBasis();}
double getTSL(){return getTSL(odata.trade_last,odata.trade_last!=null?odata.trade_last.best_buy:null,odata.trade_last!=null?odata.trade_last.best_sell:null);}

static class XQVMLegendreShiftedTest extends XQVMTest<QVMDataP> {
XQVMLegendreShiftedTest(final String name)
{
    super(name,new java.util.Random(6871L),1e-4);
}
long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8*11)*(9*5*7*8*11);
}
XQVMLegendreShifted getXQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
VQVMLegendreShifted getVQVM(){return VQVMLegendreShifted.calculate(neq,currenttime,tau,tb,tb.length,null);}
boolean isRadauRight(){return true;}
double [] convertToMonomials(final double [] p){return WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(p);}


QVMData [] getPData(final int nmoms)
{
    final QVMData [] qdata=new QVMData [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final long t=tau/(1+k);
	if(t*(1+k)!=tau) throw new RuntimeException("t*(1+k)="+(t*(1+k))+" tau="+tau+" (1+k)="+(1+k));
	qdata[k]=QVMData.calculateQVMData(1,1,4+r.nextInt(3),currenttime,t,tb,tb.length,null);
    }
    return qdata;
}
double [][] getMonomialMoments(final QVMDataP odata)
{
    final int nmoms=2*neq-1;
    final QVMData [] qd=getPData(nmoms);
    return new double [][]{
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMoments(nmoms,qd,0,QVMData.MOM_dV,false,odata.p_offset),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomT(nmoms),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomVS(nmoms,qd),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomTS(nmoms,qd),
	MQVMLegendreShifted.MQVMLegendreShiftedTest.getMonomialMomRVS(nmoms,qd)
    };
}

} // END of class XQVMLegendreShiftedTest

public static void main(String [] args)
{
    new XQVMLegendreShiftedTest("XQVMLegendreShifted").runTests(5);
} // END main



} // END OF class XQVMLegendreShifted
