package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: XkQVM.java,v 1.28 2022/11/19 11:08:58 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class XkQVM<DTYPE> {

final DTYPE odata;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=getABasis();
final VolMatrix VOLATILITY_EDPsi=this::getEDPsi;

final State sst;

static class State {
}

XkQVM(final int n,final DTYPE odata,final State rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.odata=odata;

    final int nmoms=2*n-1,nd=n-1;

    final double [][] mm=getMoments(nmoms);
    final double [] QQt=M.B.getQQMatr(n,mm[0]),QQi=M.B.getQQMatr(n,mm[1]),QQpi=M.B.getQQMatr(n,mm[2]),D=VOLATILITY_EDPsi.getDMatrix(n);
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);

    final double [] QdQ=QQi;
    TMatr.multiply_MA_x_MB(QdQ,n,QQt,n,D,n);
    final double [] bQQpi=EVXData.convertMatrFromQtoB_Basis(n,QQpi,basis,nd,n+n+0),
	bQdQ=EVXData.convertMatrFromQtoB_Basis(n,QdQ,basis,nd,n+n+0);
    this.SbQQpi=bQQpi;
    this.SbQdQ=bQdQ;
    this.SbMatrDpi=new double [nd*nd];
    // we can set diagonal elements of bQdQ to 0 to reduce rounding errors
    TMatr.scaleVectorInPlace(TMatr.selectAntiHermitianPartInPlace(nd,bQdQ),2); // factor of 2 is here
    TMatr.multiply_MA_x_MB(this.SbMatrDpi,nd,bQQpi,nd,bQdQ,nd);
    final double [] He=this.SbMatrDpi.clone();
    TMatr.scaleVectorInPlace(TMatr.selectHermitianPartInPlace(nd,He),2); // factor of 2 is here
    final double [] evHe=M.EV.getEV(nd,He,true);

    if(true)
    {
    System.err.println("XkQVM:bQQpi="+java.util.Arrays.toString(bQQpi));
    System.err.println("XkQVM:bQdQ="+java.util.Arrays.toString(bQdQ));
    System.err.println("XkQVM:2*SbMatrDpi="+java.util.Arrays.toString(TMatr.getVectorScaled(SbMatrDpi,2)));
    System.err.println("XkQVM:He="+java.util.Arrays.toString(He));
    System.err.println("ev(He)="+java.util.Arrays.toString(java.util.Arrays.copyOf(evHe,nd)));
    System.err.println("I_{low}:He(0,0)="+He[0*nd+0]);
    System.err.println("I_{high}:He(nd-1,nd-1)="+He[nd*(nd-1)+(nd-1)]);
    System.err.println("He_{low}="+evHe[0]);
    System.err.println("He_{high}="+evHe[nd-1]);
    System.err.println("<psi_{He{low}}|psi_{I_{high}}>="+evHe[nd+0*nd+nd-1]);
    System.err.println("<psi_{He{high}}|psi_{I_{high}}>="+evHe[nd+(nd-1)*nd+nd-1]);
    }



    this.sst=null;


}
final double [] SbMatrDpi,SbQQpi,SbQdQ;

double [][] getMoments(final int nmoms)
{
    final DataInterfaceToMoments<DTYPE> access=getAccess();
    final double [] tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms];

    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    TMatr.sumP1(pimom,imom,getPOffset()-getPLast());
    access.setMT(tmom,nmoms);

    return new double [][]{tmom,imom,pimom};
}

abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getPLast();
abstract double getPOffset();
abstract double [] getEDPsi(final double [] psi);

static abstract class XkQVMTest<DTYPE> extends QTest {

abstract XkQVM<DTYPE> getXkQVM();


XkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


abstract UkQVM<?> getUkQVM();
void test_UkQVM()
{
    final UkQVM<?> s=getUkQVM();
    final XkQVM<DTYPE> q=getXkQVM();
    final int n=neq,nd=n-1;

    UnitTests.checkDoubleRel(q.SbMatrDpi[nd*(nd-1)+(nd-1)],"q.SbMatrDpi at nd-1",2*s.distDX_IIp,"2*s.distDX_IIp",eps);

    System.err.println(name+":XkQVMTest:test_UkQVM(): test OK neq="+neq+" eps="+eps);
}

void test_Matrices()
{
    final int n=neq,nmoms=2*n-1,nd=n-1;

    final XkQVM<DTYPE> q=getXkQVM();
    final double [][] mm=q.getMoments(nmoms);
    final double [] QQt=q.M.B.getQQMatr(n,mm[0]),QQi=q.M.B.getQQMatr(n,mm[1]),QQpi=q.M.B.getQQMatr(n,mm[2]);
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,q.M);


    final double [] bQQpi=new double [nd*nd],bQdQ=new double [nd*nd],bMatrDpi=new double [nd*nd],bD2QQi=new double [nd*nd];
    for(int k=0+1;k<n;k++)
    {
	final double [] psik=EVXData.getPsiFromBasis(k,basis,n);
	for(int l=0+1;l<n;l++)
	{
	    final double [] psil=EVXData.getPsiFromBasis(l,basis,n),Dpsil=q.getEDPsi(psil);
	    bQQpi[(k-1)*nd+(l-1)]=TMatr.getAMatrB(psik,psil,QQpi,n);
	    bQdQ[(k-1)*nd+(l-1)]=2*TMatr.getAMatrB(psik,Dpsil,QQt,n);
	    bD2QQi[(k-1)*nd+(l-1)]=2*TMatr.getAMatrB(q.getEDPsi(psik),Dpsil,QQi,n)+
		TMatr.getAMatrB(q.getEDPsi(q.getEDPsi(psik)),psil,QQi,n)+TMatr.getAMatrB(q.getEDPsi(Dpsil),psik,QQi,n);
	}
	UnitTests.checkDoubleRel(TMatr.getAMatrB(psik,q.getEDPsi(psik),QQt,n),"<psik|Dpsik>",0,"0",eps);
    }
    UnitTests.checkDoubleArrayRel(bQQpi,"bQQpi",q.SbQQpi,"q.SbQQpi",eps);
    UnitTests.checkDoubleArrayRel1(bQdQ,"bQdQ",q.SbQdQ,"q.SbQdQ",eps);
    final double [] SbD2QQi=EVXData.convertMatrFromQtoB_Basis(n,q.VOLATILITY_EDPsi.getDiff2Matrix(n,QQi),basis,nd,n+n);
    UnitTests.checkDoubleArrayRel1(bD2QQi,"bD2QQi",SbD2QQi,"SbD2QQi",eps);
    for(int k=0;k<nd;k++)
    {
	for(int j=0;j<nd;j++)
	{
	    double s=0;
	    for(int l=0;l<nd;l++)
	    {
		s+=bQQpi[k*nd+l]*bQdQ[l*nd+j];
	    }
	    bMatrDpi[k*nd+j]=s;
	    if(k==j) UnitTests.checkDoubleRel(bQdQ[k*nd+j],"bQdQ[k*nd+k]",0,"0",eps);
	    else UnitTests.checkDoubleRel(bQdQ[k*nd+j],"bQdQ[k*nd+j]",-bQdQ[j*nd+k],"-bQdQ[j*nd+k]",eps);
	}
    }
    UnitTests.checkDoubleArrayRel(bMatrDpi,"bMatrDpi",q.SbMatrDpi,"q.SbMatrDpi",eps);

    System.err.println(name+":XkQVMTest:test_Matrices(): test OK n="+n+" eps="+eps);
}



void test_Extra(){}



void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":XkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Matrices();
    test_UkQVM();
    test_Extra();

    System.err.println(name+":XkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":XkQVMTest: starting tests; ver=$Id: XkQVM.java,v 1.28 2022/11/19 11:08:58 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":XkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: XkQVM.java,v 1.28 2022/11/19 11:08:58 mal Exp $");
}
} // END OF class XkQVMTest




}// END OF class XkQVM
