package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: XkQVMLaguerre.java,v 1.9 2022/07/10 10:41:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class XkQVMLaguerre extends XkQVM<QVMDataL> {
XkQVMLaguerre(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}


public static XkQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final XkQVMLaguerre ref)
{
    final int npm=2;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*(neq-1)+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new XkQVMLaguerre(neq,o,ref!=null?ref.sst:null);
}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getPOffset(){return odata.p_offset;}
double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}

static class XkQVMLaguerreTest extends XkQVMTest<QVMDataL> {
XkQVMLaguerreTest(final long initrandom)
{
    super("XkQVMLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
XkQVMLaguerre getXkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
UkQVMLaguerre getUkQVM(){return UkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class XkQVMLaguerreTest

public static void main(String [] args)
{
    new XkQVMLaguerreTest(1951117L).runTests(6);
} // END main



} // END OF class XkQVMLaguerre
