package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.realtimebook.*;

/** YQVM and a set of measures.
 *  @version $Id: YQVM.java,v 1.11 2019/01/06 23:33:47 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class YQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataInterface<DTYPE> access;

/** Rurik evolution. */
final double
    otdvs0,sigma2_otdvs0,otK00,otrdvs0,sigma2_otrdvs0;
final double tsL;

YQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess(neq);

    final int nmoms=neq;

    final double [] dvsmom=new double [nmoms];
    access.setM(dvsmom,0,access.M_VS,odata);

    final double df_o=access.get_f_o(dvsmom),o_o=access.get_o_o(),df_df=access.get_f_g(dvsmom,dvsmom);
    otdvs0=df_o/Math.sqrt(o_o);
    sigma2_otdvs0=df_df-otdvs0*otdvs0;
    otK00=o_o;

    access.setM(dvsmom,0,access.M_RVS,odata);
    final double rdf_o=access.get_f_o(dvsmom),rdf_rdf=access.get_f_g(dvsmom,dvsmom);
    otrdvs0=rdf_o/Math.sqrt(o_o);
    sigma2_otrdvs0=rdf_rdf-otrdvs0*otrdvs0;

    this.tsL=getTSL();
}

abstract DeprecatedHelperDataInterface<DTYPE> getAccess(final int n);
abstract double getTSL();


static abstract class YQVMTest<DTYPE> {

int neq;
long currenttime,tau;
Trade [] tb;

abstract void setData(final int tsize,final int neq);
abstract YQVM<DTYPE> getYQVM();
abstract XQVM<DTYPE> getXQVM();

final String name;
final java.util.Random r;
final double eps;
YQVMTest(final String name,
	 final java.util.Random r,final double eps)
{
    this.name=name;
    this.r=r;
    this.eps=eps;
}
void test_Common()
{
    final YQVM<DTYPE> q=getYQVM();

    System.err.println(name+":YQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}
void test_XQVM()
{
    final YQVM<DTYPE> q=getYQVM();
    final XQVM<DTYPE> s=getXQVM();

    UnitTests.checkDoubleRel(q.tsL,"q.tsL",s.tsL,"s.tsL",eps);
    UnitTests.checkDoubleRel(q.otK00,"q.otK00",s.otK00,"s.otK00",eps);
    UnitTests.checkDoubleRel(q.otdvs0,"q.otdvs0",s.otdvs0,"s.otdvs0",eps);
    UnitTests.checkDoubleRel(q.sigma2_otdvs0,"q.sigma2_otdvs0",s.sigma2_otdvs0,"s.sigma2_otdvs0",eps);
    UnitTests.checkDoubleRel(q.otrdvs0,"q.otrdvs0",s.otrdvs0,"s.otrdvs0",eps);
    UnitTests.checkDoubleRel(q.sigma2_otrdvs0,"q.sigma2_otrdvs0",s.sigma2_otrdvs0,"s.sigma2_otrdvs0",eps);

    System.err.println(name+":YQVMTest:test_XQVM(): test OK neq="+neq+" eps="+eps);
}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":YQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_XQVM();

    System.err.println(name+":YQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":YQVMTest: starting tests; ver=$Id: YQVM.java,v 1.11 2019/01/06 23:33:47 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":YQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: YQVM.java,v 1.11 2019/01/06 23:33:47 mal Exp $");
}
} // END OF class YQVMTest






}// END OF class YQVM
