package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: YkDpI.java,v 1.8 2020/09/27 20:33:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class YkDpI<DTYPE> {

static class State {
}
final State sst;

final DTYPE odata;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=getABasis();

final DynDpDi i0_pt,i0_pi,iR_pt,iR_pi;
final DynSIV pSIV;

YkDpI(final int n,final DTYPE odata,final State rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.odata=odata;

    final int nmoms=2*n-1;

    final double [][] mm=getMoments(2*nmoms-1);
    final double [] tmoms=mm[0],imoms=mm[1],pimoms=mm[3],dpmoms=mm[4];
    final DataInterfaceToMoments<DTYPE> access=getAccess();
    i0_pt=DynDpDi.get0_DynDpDi(n,imoms,tmoms,dpmoms,tmoms,M,access);
    i0_pi=DynDpDi.get0_DynDpDi(n,imoms,tmoms,dpmoms,imoms,M,access);
    iR_pt=DynDpDi.getR_DynDpDi(n,imoms,tmoms,dpmoms,tmoms,M,access);
    iR_pi=DynDpDi.getR_DynDpDi(n,imoms,tmoms,dpmoms,imoms,M,access);

    pSIV=DynSIV.get_DynSIV(n,imoms,tmoms,pimoms,M);

    this.sst=null;


}

double [][] getMoments(final int nmoms)
{
    final DataInterfaceToMoments<DTYPE> access=getAccess();
    final double [] tmom=new double [nmoms],imom=new double [nmoms],ptmom=new double [nmoms],pimom=new double [nmoms],dpmom=new double [nmoms];

    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    TMatr.sumP1(pimom,imom,getPOffset()-getPLast());
    access.setMT(tmom,nmoms);
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setFMoments(ptmom,dpmom,nmoms);

    return new double [][]{tmom,imom,ptmom,pimom,dpmom};
}

abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getPLast();
abstract double getPOffset();


static abstract class YkDpITest<DTYPE> extends QTest {

abstract YkDpI<DTYPE> getYkDpI();


YkDpITest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}


void test_Extra(){}

void test_pH()
{
    final int n=neq,nmoms=2*n-1;
    final YkDpI<DTYPE> q=getYkDpI();
    final double [][] mm=q.getMoments(nmoms);
    final double [] QQt=q.M.B.getQQMatr(n,mm[0]),QQi=q.M.B.getQQMatr(n,mm[1]),QQpt=q.M.B.getQQMatr(n,mm[2]),QQpi=q.M.B.getQQMatr(n,mm[3]);
    final DynMUPnL m0=new DynMUPnL(n,QQi,QQt,QQpi,QQpt,q.M,true),
	mR=new DynMUPnL(n-1,
			VolMatrix.getRadauMatrix(n,QQi,q.M),
			VolMatrix.getRadauMatrix(n,QQt,q.M),
			VolMatrix.getRadauMatrix(n,QQpi,q.M),
			VolMatrix.getRadauMatrix(n,QQpt,q.M),
			q.M,false);
    for(int j=0;j<2;j++)
    {
	final DynMUPnL m;final DynDpDi s;
	if(j<=0){m=m0;s=q.i0_pt;}
	else{m=mR;s=q.iR_pt;}
	System.err.println("j="+j);
	UnitTests.checkDoubleRel(s.I_L,"s.I_L",m.I_L,"m.I_L",eps);
	UnitTests.checkDoubleRel(s.I_H,"s.I_H",m.I_H,"m.I_H",eps);
	UnitTests.checkDoubleRel(s.I_0,"s.I_0",m.I_0,"m.I_0",eps);
	UnitTests.checkDoubleRel(s.p_IH,"s.p_IH",m.pt_IH,"m.pt_IH",eps);
    }

    System.err.println(name+":YkDpITest:test_pH(): test OK n="+n+" eps="+eps);

}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":YkDpITest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Extra();
    test_pH();

    System.err.println(name+":YkDpITest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":YkDpITest: starting tests; ver=$Id: YkDpI.java,v 1.8 2020/09/27 20:33:22 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":YkDpITest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: YkDpI.java,v 1.8 2020/09/27 20:33:22 mal Exp $");
}
} // END OF class YkDpITest




}// END OF class YkDpI
