package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: YkDpILaguerre.java,v 1.5 2022/07/10 10:41:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class YkDpILaguerre extends YkDpI<QVMDataL> {
YkDpILaguerre(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}


public static YkDpILaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final YkDpILaguerre ref)
{
    final int npm=2,nmoms=2*(2*(neq-1)+1)-1;
    final QVMDataL o=QVMDataL.calculateQVMDataL(nmoms,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new YkDpILaguerre(neq,o,ref!=null?ref.sst:null);
}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double getPOffset(){return odata.p_offset;}

static class YkDpILaguerreTest extends YkDpITest<QVMDataL> {
YkDpILaguerreTest(final long initrandom)
{
    super("YkDpILaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
YkDpILaguerre getYkDpI(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class YkDpILaguerreTest

public static void main(String [] args)
{
    new YkDpILaguerreTest(-157L).runTests(6);
} // END main



} // END OF class YkDpILaguerre
