package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: YkDpIMonomials.java,v 1.3 2015/07/08 01:49:59 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class YkDpIMonomials extends YkDpI<QVMData> {
YkDpIMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static YkDpIMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final YkDpIMonomials ref)
{
    final int nvm=1,npm=2,nmoms=2*(2*(neq-1)+1)-1;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     nmoms,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new YkDpIMonomials(neq,o,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}

static class YkDpIMonomialsTest extends YkDpITest<QVMData> {
YkDpIMonomialsTest(final long initrandom)
{
    super("YkDpIMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

YkDpILaguerre getYkDpILaguerre(){return YkDpILaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
YkDpIMonomials getYkDpI(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

@Override
void test_Extra(){
    final YkDpIMonomials q=getYkDpI();
    final YkDpILaguerre s=getYkDpILaguerre();
    DynDpDi.matchDynDpDi(q.i0_pt,s.i0_pt,eps);
    DynDpDi.matchDynDpDi(q.i0_pi,s.i0_pi,eps);
    DynDpDi.matchDynDpDi(q.iR_pt,s.iR_pt,eps);
    DynDpDi.matchDynDpDi(q.iR_pi,s.iR_pi,eps);

    DynSIV.matchDynSIV(q.pSIV,s.pSIV,eps);

    System.err.println(name+":YkDpIMonomialsTest:test_Extra(): test YkDpILaguerre OK neq="+neq+" eps="+eps);
}

} // END of class YkDpIMonomialsTest

public static void main(String [] args)
{
    new YkDpIMonomialsTest(7L).runTests(5);
} // END main



} // END OF class YkDpIMonomials
