package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.realtimebook.*;

/**
 *  Two more or less working variants:
 *  1. dir=Ipnl_psi_I_H.wH^2-Ipnl_psi_I_H.wL^2 (from DynHPnL.dir).
 *  2. dir=<pisH|pI|DpsiH> - <pisH|DpsiH><pisH|pI|psiH>
 *  (from DynVHPnL.var_p_IH and DynVar2C.var_pI_IX ).
 *  All other are much worse.
 *  @version $Id: YkQVM.java,v 1.61 2022/07/28 12:27:19 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class YkQVM<DTYPE> {

final DTYPE odata;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=getABasis();
final K2Distance DISTANCE2=getK2Distance();
final EVXData I,Ipnl_psi_I_H;
final double Ipnl_psi_I_H_wLH,Ipnl_psi_I_H_wHH,Ixen_wLH,Ixen_wHH,
    Ipnl_pnl_0,Ipnl_pnl_L,Ipnl_pnl_H;
final double di_dIL,di_dIH,p_dIL,p_dIH,pe_dIL,pe_dIH;
final double I_x2en_I_H,p_x2en_I_H;
final State sst;
final DynHPnL pi;

static class State {
}

final double I_0,I_L,I_H,p_IH,pt_IH,pm_IH,plh_IH,phh1_IH,
    p_Ispur,pat_p_IH,p_pIL,I_pIL,p_pIH,I_pIH,aI_assym,p_IPnL;
YkQVM(final int n,final DTYPE odata,final State rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.odata=odata;

    final int nmoms=2*n-1;

    final double [][] mm=getMoments(nmoms);
    final double [] QQt=M.B.getQQMatr(n,mm[0]),QQi=M.B.getQQMatr(n,mm[1]),QQpi=M.B.getQQMatr(n,mm[2]),QQp2i=M.B.getQQMatr(n,mm[3]),QQdp=M.B.getQQMatr(n,mm[4]),QQpt=M.B.getQQMatr(n,mm[5]),QQV=M.B.getQQMatr(n,mm[7]),QQpV=M.B.getQQMatr(n,mm[8]);
    final double [] basis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,M);

    final int i0=0,iL=1,iH=n-1;
    final double [] psi_I_H=EVXData.getPsiFromBasis(iH,basis,n),
	psi_I_L=EVXData.getPsiFromBasis(iL,basis,n);
    this.I_0=basis[i0];
    this.I_L=basis[iL];
    this.I_H=basis[iH];
    this.p_IH=TMatr.getAMatrA(psi_I_H,QQpi,n)/I_H;
    this.pt_IH=TMatr.getAMatrA(psi_I_H,QQpt,n);
    this.pm_IH=(TMatr.getAMatrA(psi_I_H,QQpi,n)-I_L*pt_IH)/(I_H-I_L);

    this.pi=new DynHPnL(n,QQi,QQt,QQpi,QQi,QQV,QQpV,M,DISTANCE2);

    this.plh_IH=(TMatr.getAMatrA(psi_I_H,QQpi,n)-TMatr.getAMatrA(psi_I_L,QQpi,n))/(I_H-I_L);
    this.phh1_IH=n<=2?Double.NaN:(TMatr.getAMatrA(psi_I_H,QQpi,n)-TMatr.getAMatrA(EVXData.getPsiFromBasis(iH-1,basis,n),QQpi,n))/(I_H-basis[iH-1]);
    {
    final double [] x2enbasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQp2i,QQt,M),
	psi_x2en_I_H=EVXData.getPsiFromBasis(iH,x2enbasis,n);
    this.I_x2en_I_H=TMatr.getAMatrA(psi_x2en_I_H,QQi,n);
    this.p_x2en_I_H=TMatr.getAMatrA(psi_x2en_I_H,QQpi,n)/this.I_x2en_I_H;

    }
    {
    double spi=0,si=0;for(int k=1;k<n;k++){spi+=TMatr.getAMatrA(n,basis,n+k*n,QQpi,n);si+=TMatr.getAMatrA(n,basis,n+k*n,QQi,n);}
    this.p_Ispur=spi/si;
    final double [] dimom=new double [nmoms],dpimom=new double [nmoms];
    getAccess().setDFMoments(dimom,mm[1],nmoms);
    getAccess().setDFMoments(dpimom,mm[2],nmoms);
    final double [] QQdi=M.B.getQQMatr(n,dimom),QQdpi=M.B.getQQMatr(n,dpimom);
    final double [] dbasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQdi,QQt,M);
    this.di_dIL=dbasis[iL];
    this.di_dIH=dbasis[iH];
    this.p_dIL=TMatr.getAMatrA(n,dbasis,n+iL*n,QQdpi,n)/di_dIL;
    this.p_dIH=TMatr.getAMatrA(n,dbasis,n+iH*n,QQdpi,n)/di_dIH;
    this.pe_dIL=TMatr.getAMatrA(n,dbasis,n+iL*n,QQpi,n)/TMatr.getAMatrA(n,dbasis,n+iL*n,QQi,n);
    this.pe_dIH=TMatr.getAMatrA(n,dbasis,n+iH*n,QQpi,n)/TMatr.getAMatrA(n,dbasis,n+iH*n,QQi,n);
    }



    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    final double [] QQat_p_IH=TMatr.sumAcB(QQpi,-p_IH,QQi);
    this.Ipnl_psi_I_H=EVXData.getEVXDataFromQQ(n,QQat_p_IH,QQt,M);
    this.Ipnl_psi_I_H_wLH=TMatr.getAMatrB(I.psiH,Ipnl_psi_I_H.psiL,QQt,n);
    this.Ipnl_psi_I_H_wHH=TMatr.getAMatrB(I.psiH,Ipnl_psi_I_H.psiH,QQt,n);
    {
    final double [] basis_at=EdComm.getIRadauBasisPsi0FromQQ(n,QQat_p_IH,QQt,M),
	psi_at_L=EVXData.getPsiFromBasis(iL,basis_at,n),
	psi_at_H=EVXData.getPsiFromBasis(iH,basis_at,n);
    this.pat_p_IH=(basis_at[iL]+basis_at[iH])/(TMatr.getAMatrA(psi_at_L,QQi,n)+TMatr.getAMatrA(psi_at_H,QQi,n))-(p_IH-getPLast());
    this.Ipnl_pnl_0=basis_at[i0];
    this.Ipnl_pnl_L=basis_at[iL];
    this.Ipnl_pnl_H=basis_at[iH];
    if(false)
    {
	System.err.println("<psiH|QQat_p_IH|psiH>="+TMatr.getAMatrA(psi_I_H,QQat_p_IH,n)+" must be =0");
	System.err.println("basis_at[iL]="+basis_at[iL]+"\tbasis_at[iH]="+basis_at[iH]);
    }
    }



    final double [] pibasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQpi,QQt,M);
    //System.err.println("pibasis="+java.util.Arrays.toString(pibasis));
    final double [] psi_pIL=EVXData.getPsiFromBasis(iL,pibasis,n),
	psi_pIH=EVXData.getPsiFromBasis(iH,pibasis,n);
    I_pIL=TMatr.getAMatrA(psi_pIL,QQi,n);
    I_pIH=TMatr.getAMatrA(psi_pIH,QQi,n);
    //System.err.println("I_pIL="+I_pIL+" I_pIH="+I_pIH+" I_L="+I_L+" I_H="+I_H);
    this.p_pIL=pibasis[iL]/I_pIL;
    this.p_pIH=pibasis[iH]/I_pIH;
    this.aI_assym=(I_pIL-I_pIH)/(I_H-I_L);
    this.p_IPnL=(pibasis[iL]+pibasis[iH])/(I_pIL+I_pIH);
    this.Ixen_wLH=TMatr.getAMatrB(psi_I_H,psi_pIL,QQt,n);
    this.Ixen_wHH=TMatr.getAMatrB(psi_I_H,psi_pIH,QQt,n);

    this.dpnl_I0=new DynPnL(n,basis,n,new int[]{i0},QQpi,QQi,QQt,0,M);
    this.dpnl_I0IH=new DynPnL(n,basis,n,new int[]{i0,iH},QQpi,QQi,QQt,p_IH,M);
    this.dpnl_IH=new DynPnL(n,M.EV.getGEV_Generalized(n,QQi,QQt,true),n,new int[]{iH},QQpi,QQi,QQt,M);
    this.dpnl_I0H=new DynPnL(n,basis,n,new int[]{iH},QQpi,QQi,QQt,M);
    this.vdpnl_IH=new DynVolPnL(n,iH,QQt,QQi,QQpi,QQp2i,M);

    this.sst=null;


}

final DynPnL dpnl_I0,dpnl_I0IH,dpnl_IH,dpnl_I0H;
final DynVolPnL vdpnl_IH;

double [][] getMoments(final int nmoms)
{
    final DataInterfaceToMoments<DTYPE> access=getAccess();
    final double [] tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms],p2imom=new double [nmoms],dpmom=new double [nmoms],ptmom=new double [nmoms];

    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    access.setM(p2imom,2,access.M_I,odata,nmoms);
    Polynomials.shiftPOffsetInPlace(nmoms,imom,pimom,p2imom,null,getPLast()-getPOffset());
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setFMoments(ptmom,dpmom,nmoms);
    access.setMT(tmom,nmoms);

    final double [] Tmom=new double [nmoms],Vmom=new double [nmoms],pVmom=new double [nmoms];
    access.setFMoments(Vmom,imom,nmoms);
    access.setFMoments(Tmom,tmom,nmoms);
    access.setFMoments(pVmom,pimom,nmoms);
    TMatr.scaleVectorInPlace(Tmom,-1);
    TMatr.scaleVectorInPlace(Vmom,-1);
    TMatr.scaleVectorInPlace(pVmom,-1);

    return new double [][]{tmom,imom,pimom,p2imom,dpmom,ptmom,Tmom,Vmom,pVmom};
}

abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract K2Distance getK2Distance();
abstract double getPLast();
double getPOffset(){return getAccess().getPOffset(odata);}


static abstract class YkQVMTest<DTYPE> extends QTest {

abstract YkQVM<DTYPE> getYkQVM();


YkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}

void test_DynPnL()
{
    final YkQVM<DTYPE> q=getYkQVM();
    final int n=neq;
    UnitTests.checkDoubleRel(q.p_IPnL,"q.p_IPnL",q.dpnl_I0.pEQ,"q.dpnl_I0.pEQ",eps);
    UnitTests.checkDoubleRel(q.p_IH,"q.p_IH",q.dpnl_I0H.pEB,"q.dpnl_I0H.pEB",eps);
    EVXData.matchEVXData(q.pi.Ipnl,q.Ipnl_psi_I_H,eps);
    UnitTests.checkDoubleRel(q.p_IH,"q.p_IH",q.pi.p_IH,"q.pi.p_IH",eps);
    UnitTests.checkDoubleRel(Math.pow(q.Ipnl_psi_I_H.wH,2)-Math.pow(q.Ipnl_psi_I_H.wL,2),"Math.pow(q.Ipnl_psi_I_H.wH,2)-Math.pow(q.Ipnl_psi_I_H.wL,2)",q.pi.dir,"q.pi.dir",eps);


    final double [][] mm=q.getMoments(2*n-1);
    final double [] QQt=q.M.B.getQQMatr(n,mm[0]),QQi=q.M.B.getQQMatr(n,mm[1]),QQpi=q.M.B.getQQMatr(n,mm[2]),QQV=q.M.B.getQQMatr(n,mm[7]),QQpV=q.M.B.getQQMatr(n,mm[8]);
    for(int j=0;j<3;j++)
    {
	final double [] basis;
	final int nd;
	final double pb;
	final DynPnL p;
	if(j<=0)
	{
	    if(n<=2) continue;
	    final double [] ibasis=EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,q.M);
	    pb=TMatr.getAMatrA(EVXData.getPsiFromBasis(n-1,ibasis,n),QQpi,n)/ibasis[n-1];
	    basis=DeprecatedHelperEVXData.getBasisWithVectorsRemoved(n-1,DeprecatedHelperEVXData.getBasisOneVectorRemoved(n,ibasis,n,0),n,0,new int []{n-2});
	    nd=n-2;
	    p=q.dpnl_I0IH;
	}
	else if(j==1)
	{
	    pb=q.p_IH;
	    basis=DeprecatedHelperEVXData.getBasisOneVectorRemoved(n,EdComm.getIRadauBasisPsi0FromQQ(n,QQi,QQt,q.M),n,n-1);
	    nd=n-1;
	    p=q.dpnl_I0H;
	}
	else
	{
	    final EVXData i=EVXData.getEVXDataFromQQ(n,QQi,QQt,q.M);
	    pb=TMatr.getAMatrA(i.psiH,QQpi,n)/i.sH;
	    basis=DeprecatedHelperEVXData.getBasisOneVectorRemoved(n,i.ev,n,n-1);
	    nd=n-1;
	    p=q.dpnl_IH;
	}
	final double [] BQQpi=new double [nd*nd],BQQi=new double [nd*nd];
	for(int k=0;k<nd;k++)
	{
	    for(int l=0;l<nd;l++)
	    {
		BQQi[k*nd+l]=TMatr.getAMatrB(n,basis,k*n,n,basis,l*n,QQi,n);
		BQQpi[k*nd+l]=TMatr.getAMatrB(n,basis,k*n,n,basis,l*n,QQpi,n)-pb*BQQi[k*nd+l];
	    }
	}
	final double [] ev=q.M.EV.getEV(nd,BQQpi,true);
	final double pe=(ev[0]+ev[nd-1])/(TMatr.getAMatrA(nd,ev,nd+0*nd,BQQi,nd)+TMatr.getAMatrA(nd,ev,nd+(nd-1)*nd,BQQi,nd))+pb;
	UnitTests.checkDoubleRel(pe,"pe",p.pEQ,"p.pEQ",eps);
	UnitTests.checkDoubleRel(pb,"pb",p.pEB,"p.pEB",eps);
	UnitTests.checkDoubleRel(ev[0],"ev[0]",p.pnlL,"p.pnlL",eps);
	UnitTests.checkDoubleRel(ev[nd-1],"ev[nd-1]",p.pnlH,"p.pnlH",eps);
    }

    if(true)
    {
	final ACorr a=ACorr.getACorr_SPUR(
	    n-1,
	    q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[2])),
	    q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[0])),
	    q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[0])),
	    q.M),
	    b=ACorr.getACorr_SPUR(
	    n-1,
	    q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[1])),
	    q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[0])),
	    q.M.getRadauMomentsFromMoments(q.M.getRadauMomentsFromMoments(mm[0])),
	    q.M);
	UnitTests.checkDoubleRel(q.p_Ispur,"q.p_Ispur",a.x_a_b/b.x_a_b,"a.x_a_b/b.x_a_b",eps);
    }

    DynHPnL.test_DynHPnL(n,QQi,QQt,QQpi,QQi,QQV,QQpV,q.M,q.DISTANCE2,eps);

    System.err.println(name+":YkQVMTest:test_DynPnL(): test OK neq="+neq+" eps="+eps);
}

abstract UkQVM<?> getUkQVM();
void test_UkQVM()
{
    final UkQVM<?> s=getUkQVM();
    final YkQVM<DTYPE> q=getYkQVM();
    final int n=neq;

    UnitTests.checkDoubleRel(q.aI_assym,"q.aI_assym",(s._I_pL-s._I_pH)/(s._I_H-s._I_L),"(s._I_pL-s._I_pH)/(s._I_H-s._I_L)",eps);
    UnitTests.checkDoubleRel(q.p_pIL,"q.p_pIL",s._pI_pL,"s._pI_pL",eps);
    UnitTests.checkDoubleRel(q.p_pIH,"q.p_pIH",s._pI_pH,"s._pI_pH",eps);

    System.err.println(name+":YkQVMTest:test_UkQVM(): test OK neq="+neq+" eps="+eps);
}


abstract VkQVM<?> getVkQVM();
void test_VkQVM()
{
    final VkQVM<?> s=getVkQVM();
    final YkQVM<DTYPE> q=getYkQVM();
    final int n=neq;

    UnitTests.checkDoubleRel(q.di_dIL,"q.di_dIL",s.dI_DiL,"s.dI_DiL",eps);
    UnitTests.checkDoubleRel(q.di_dIH,"q.di_dIH",s.dI_DiH,"s.dI_DiH",eps);
    UnitTests.checkDoubleRel(q.p_dIL,"q.p_dIL",s.p_DiL,"s.p_DiL",eps);
    UnitTests.checkDoubleRel(q.p_dIH,"q.p_dIH",s.p_DiH,"s.p_DiH",eps);

    System.err.println(name+":YkQVMTest:test_VkQVM(): test OK neq="+neq+" eps="+eps);
}




void test_Extra(){}



void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":YkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_UkQVM();
    test_VkQVM();
    test_DynPnL();
    test_Extra();

    System.err.println(name+":YkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":YkQVMTest: starting tests; ver=$Id: YkQVM.java,v 1.61 2022/07/28 12:27:19 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":YkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: YkQVM.java,v 1.61 2022/07/28 12:27:19 mal Exp $");
}
} // END OF class YkQVMTest




}// END OF class YkQVM
