package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: YkQVMLaguerre.java,v 1.11 2022/07/10 10:41:24 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class YkQVMLaguerre extends YkQVM<QVMDataL> {
YkQVMLaguerre(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}


public static YkQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final YkQVMLaguerre ref)
{
    final int npm=3;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*(neq-1)+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new YkQVMLaguerre(neq,o,ref!=null?ref.sst:null);
}
QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
K2Distance getK2Distance(){return KkQVMLaguerre.K2DISTANCE;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}

static class YkQVMLaguerreTest extends YkQVMTest<QVMDataL> {
YkQVMLaguerreTest(final long initrandom)
{
    super("YkQVMLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
YkQVMLaguerre getYkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
UkQVMLaguerre getUkQVM(){return UkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
VkQVMLaguerre getVkQVM(){return VkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class YkQVMLaguerreTest

public static void main(String [] args)
{
    new YkQVMLaguerreTest(1951117L).runTests(6);
} // END main



} // END OF class YkQVMLaguerre
