package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: YkQVMMonomials.java,v 1.10 2016/08/24 17:08:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class YkQVMMonomials extends YkQVM<QVMData> {
YkQVMMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static YkQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final YkQVMMonomials ref)
{
    final int nvm=1,npm=3;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new YkQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
K2Distance getK2Distance(){return KkQVMMonomials.K2DISTANCE;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}

static class YkQVMMonomialsTest extends YkQVMTest<QVMData> {
YkQVMMonomialsTest(final long initrandom)
{
    super("YkQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}


@Override
void test_Extra(){
    final YkQVMMonomials q=getYkQVM();
    final YkQVMLaguerre s=getYkQVMLaguerre();

     UnitTests.checkDoubleRel(q.aI_assym,"q.aI_assym",s.aI_assym,"s.aI_assym",eps);
     UnitTests.checkDoubleRel(q.p_IPnL,"q.p_IPnL",s.p_IPnL,"s.p_IPnL",eps);

     UnitTests.checkDoubleRel(q.I_x2en_I_H,"q.I_x2en_I_H",s.I_x2en_I_H,"s.I_x2en_I_H",eps);
     UnitTests.checkDoubleRel(q.p_x2en_I_H,"q.p_x2en_I_H",s.p_x2en_I_H,"s.p_x2en_I_H",eps);
     UnitTests.checkDoubleRel(q.p_IH,"q.p_IH",s.p_IH,"s.p_IH",eps);
     UnitTests.checkDoubleRel(q.pt_IH,"q.pt_IH",s.pt_IH,"s.pt_IH",eps);


    System.err.println(name+":YkQVMMonomialsTest:test_Extra(): test YkQVMLaguerre OK neq="+neq+" eps="+eps);
}

YkQVMLaguerre getYkQVMLaguerre(){return YkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
YkQVMMonomials getYkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
UkQVMMonomials getUkQVM(){return UkQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}
VkQVMMonomials getVkQVM(){return VkQVMMonomials.calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class YkQVMMonomialsTest

public static void main(String [] args)
{
    new YkQVMMonomialsTest(1117L).runTests(5);
} // END main



} // END OF class YkQVMMonomials
