package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: YpkQVM.java,v 1.30 2022/07/15 00:01:20 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class YpkQVM<DTYPE> {

final DTYPE odata;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M=getABasis();
final EVXData pI,dpI;
final State sst;
final DynY dY;
final DynYp dYp;
final DynYp1 dYp1;
final DynYpm dYpmL,dYpmH,dYpi;
final DynYpNoIter dYpNi;
final VolMatrix VOLATILITY_EDPsi=this::getEDPsi;

static class State {
}

YpkQVM(final int n,final DTYPE odata,final State rst)
{
    if(n<=1) throw new RuntimeException("n="+n+" must be >1");

    this.odata=odata;

    final int nd=n-1,nmoms=2*nd-1;

    final double [][] mm=get_PV_PT_Moments_shift_offset_to_PLast(nmoms,odata,getAccess());
    final double [] QQt=M.B.getQQMatr(nd,mm[0]),QQpt=M.B.getQQMatr(nd,mm[1]),QQp2t=M.B.getQQMatr(nd,mm[2]),QQp3t=M.B.getQQMatr(nd,mm[3]),
	QQi=M.B.getQQMatr(nd,mm[4]),QQpi=M.B.getQQMatr(nd,mm[5]),QQp2i=M.B.getQQMatr(nd,mm[6]),QQp3i=M.B.getQQMatr(nd,mm[7]);

    final double [] QQa=PEVXData.getQQMatrInBasisFromQQ(n,QQi,QQpi,QQp2i),
	QQb=PEVXData.getQQMatrInBasisFromQQ(n,QQt,QQpt,QQp2t);

    this.pI=PEVXData.getPEVXDataFromQQ(n,QQa,QQb,M);
    final double [] dpQQa=getDMatr(n,QQi,QQpi),dpQQb=getDMatr(n,QQt,QQpt),
	QQpa=PEVXData.getQQMatrInBasisFromQQ(n,QQpi,QQp2i,QQp3i);

    this.varP_pI_psiH=TMatr.getAMatrA(pI.psiH,dpQQa,n)-pI.sH*TMatr.getAMatrA(pI.psiH,dpQQb,n);
    this.P_pI_psiH=TMatr.getAMatrA(pI.psiH,QQpa,n)/pI.sH;

    final int i0=0,iL=1,iH=n-1;
    final double [] basis=PEVXData.getIRadauBasisPsi0FromQQInBasis(n,QQa,QQb,M),
	rpsi_H=EVXData.getPsiFromBasis(iH,basis,n);
    this.I_0=basis[i0];
    this.I_L=basis[iL];
    this.I_H=basis[iH];
    this.varP_I_rpsiH=TMatr.getAMatrA(rpsi_H,dpQQa,n)-basis[iH]*TMatr.getAMatrA(rpsi_H,dpQQb,n);
    this.P_I_rpsiH=TMatr.getAMatrA(rpsi_H,QQpa,n)/basis[iH];


    this.dpI=PEVXData.getPEVXDataFromQQ(n,dpQQa,QQb,M);
    final double [] dpIbasis=PEVXData.getIRadauBasisPsi0FromQQInBasis(n,dpQQa,QQb,M);
    this.rdpI_0=dpIbasis[i0];
    this.rdpI_L=dpIbasis[iL];
    this.rdpI_H=dpIbasis[iH];

    dY=DynY.get_DynY(nd,QQi,QQt,QQpi,QQp2i,VOLATILITY_EDPsi,M);
    dYp=DynYp.get_DynYp(nd,QQi,QQt,QQpi,QQt,QQpt,QQp2t,QQi,false,VOLATILITY_EDPsi,M);
    dYp1=DynYp1.get_DynYp1(nd,QQi,QQt,QQpi,QQt,QQpt,QQp2t,VOLATILITY_EDPsi,M);
    dYpmH=DynYpm.get_DynYpm(nd,QQi,QQt,QQpi,QQp2i,true,VOLATILITY_EDPsi,M);
    dYpmL=DynYpm.get_DynYpm(nd,QQi,QQt,QQpi,QQp2i,false,VOLATILITY_EDPsi,M);
    dYpi=DynYpm.get_DynYpi(nd,QQi,QQt,QQpi,QQp2i,VOLATILITY_EDPsi,M);
    dYpNi=DynYpNoIter.get_DynYpNoIter(nd,QQi,QQt,QQpi,QQp2i,VOLATILITY_EDPsi,M,0);

    this.sst=null;


}
final double I_0,I_L,I_H,rdpI_0,rdpI_L,rdpI_H,P_pI_psiH,varP_pI_psiH,P_I_rpsiH,varP_I_rpsiH;


static double [] getDMatr(final int n,final double [] QQf,final double [] QQpf)
{
    final int nd=n-1;
    return PEVXData.getQQMatrInBasisFromQQ(n,new double [nd*nd],QQf,TMatr.getVectorScaled(QQpf,2));
}

static <DTYPE> double [][] get_PV_PT_Moments_shift_offset_to_PLast(final int nmoms,DTYPE odata,final DataInterfaceToMoments<DTYPE> access)
{
    final double [][] mm=get_PV_PT_Moments(nmoms,odata,access);
    final double pl_m_poffset=access.getPLast(odata);// p_offset is already included in access.getPLast(DTYPE odata). Note it is not icluded in YpkQVM.getPLast()!
    Polynomials.shiftPOffsetInPlace(nmoms,mm[4],mm[5],mm[6],mm[7],pl_m_poffset);
    Polynomials.shiftPOffsetInPlace(nmoms,mm[0],mm[1],mm[2],mm[3],pl_m_poffset);
    Polynomials.shiftPOffsetInPlace(nmoms,mm[8],mm[9],mm[10],mm[11],pl_m_poffset);
    return mm;
}

public static <DTYPE> double [][] get_PV_PT_Moments(final int nmoms,DTYPE odata,final DataInterfaceToMoments<DTYPE> access)
{
    final double [] imom=new double [nmoms],pimom=new double [nmoms],p2imom=new double [nmoms],p3imom=new double [nmoms];

    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(pimom,1,access.M_I,odata,nmoms);
    access.setM(p2imom,2,access.M_I,odata,nmoms);
    access.setM(p3imom,3,access.M_I,odata,nmoms);

    final double [] _tmom=new double [nmoms],_ptmom=new double [nmoms],_p2tmom=new double [nmoms],_p3tmom=new double [nmoms];
    access.setM(_tmom,0,access.M_T,odata,nmoms);
    access.setM(_ptmom,1,access.M_T,odata,nmoms);
    access.setM(_p2tmom,2,access.M_T,odata,nmoms);
    access.setM(_p3tmom,3,access.M_T,odata,nmoms);

    final double [] dp0mom=new double [nmoms],dp1mom=new double [nmoms],dp2mom=new double [nmoms],dp3mom=new double [nmoms];
    access.setM(dp0mom,0,access.M_DP,odata,nmoms);
    access.setM(dp1mom,1,access.M_DP,odata,nmoms);
    access.setM(dp2mom,2,access.M_DP,odata,nmoms);
    access.setM(dp3mom,3,access.M_DP,odata,nmoms);
    final double pl_m_poffset=access.getPLast(odata);// p_offset is already included in access.getPLast(DTYPE odata). Note it is not icluded in YpkQVM.getPLast()!
    // convert to p_offset=p_last to have out of integral term to be equal to 0
    Polynomials.shiftPOffsetInPlace(nmoms,dp0mom,dp1mom,dp2mom,dp3mom,pl_m_poffset);

    final double [] tmom=new double [nmoms],ptmom=new double [nmoms],p2tmom=new double [nmoms],p3tmom=new double [nmoms];
    access.setFMoments(ptmom,dp1mom,nmoms);
    access.setFMoments(p2tmom,dp2mom,nmoms);
    access.setFMoments(p3tmom,dp3mom,nmoms);
    access.setMT(tmom,nmoms);
    // convert back to original p_offset.
    Polynomials.shiftPOffsetInPlace(nmoms,tmom,ptmom,p2tmom,p3tmom,-pl_m_poffset);

    return new double [][]{tmom,ptmom,p2tmom,p3tmom,imom,pimom,p2imom,p3imom,
			   _tmom,_ptmom,_p2tmom,_p3tmom};
}

abstract DataInterfaceToMoments<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getPLast();
abstract double getPOffset();
abstract double [] getEDPsi(final double [] psi);

static abstract class YpkQVMTest<DTYPE> extends QTest {

abstract YpkQVM<DTYPE> getYpkQVM();

void test_Common()
{
    final YpkQVM<DTYPE> q=getYpkQVM();
    final int n=neq;
    final double [][] mm=q.get_PV_PT_Moments_shift_offset_to_PLast(2*(n-1)+1,q.odata,q.getAccess());

    PEVXData.test_E0_and_Norm(n,mm[0],mm[1],mm[2],mm[4],mm[5],mm[6],q.M,eps);

    System.err.println(name+":YpkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}
void test_DynYp()
{
    final YpkQVM<DTYPE> q=getYpkQVM();
    final int nd=neq-1;
    final double [][] mm=q.get_PV_PT_Moments_shift_offset_to_PLast(2*nd-1,q.odata,q.getAccess());
    final double [] QQt=q.M.B.getQQMatr(nd,mm[0]),
	QQi=q.M.B.getQQMatr(nd,mm[4]),QQpi=q.M.B.getQQMatr(nd,mm[5]),QQp2i=q.M.B.getQQMatr(nd,mm[6]),
	QQpt=q.M.B.getQQMatr(nd,mm[1]),QQp2t=q.M.B.getQQMatr(nd,mm[2]);
    DynYp.match_DynYp(q.dYp,DynYp.test_get_DynYp(nd,QQi,QQt,QQpi,QQt,QQpt,QQp2t,QQi,false,q.VOLATILITY_EDPsi,q.M),eps);
    DynYp1.match_DynYp(q.dYp1,q.dYp,eps);
    DynYpm.match_DynYpm(q.dYpmH,DynYpm.test_get_DynYpm(nd,QQi,QQt,QQpi,QQp2i,true,q.VOLATILITY_EDPsi,q.M),eps);
    DynYpm.match_DynYpm(q.dYpmL,DynYpm.test_get_DynYpm(nd,QQi,QQt,QQpi,QQp2i,false,q.VOLATILITY_EDPsi,q.M),eps);
    DynYpm.match_DynYpm(q.dYpi,DynYpm.test_get_DynYpi(nd,QQi,QQt,QQpi,QQp2i,q.VOLATILITY_EDPsi,q.M),eps);
    final DynYpNoIter dypni=DynYpNoIter.get_DynYpNoIter(nd,QQi,QQt,QQpi,QQp2i,q.VOLATILITY_EDPsi,q.M,eps);
    final DynYp dyp=DynYp.get_DynYp(nd,QQi,QQt,QQpi,QQi,QQpi,QQp2i,QQt,true,q.VOLATILITY_EDPsi,q.M);
    DynYpNoIter.match_DynYp(dypni,dyp,eps);

    System.err.println(name+":YpkQVMTest:test_DynYp(): test OK neq="+neq+" eps="+eps);
}
YpkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}






void test_Extra(){}



void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":YpkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_DynYp();
    test_Extra();

    System.err.println(name+":YpkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":YpkQVMTest: starting tests; ver=$Id: YpkQVM.java,v 1.30 2022/07/15 00:01:20 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=3;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(1850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":YpkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: YpkQVM.java,v 1.30 2022/07/15 00:01:20 mal Exp $");
}
} // END OF class YpkQVMTest




}// END OF class YpkQVM
