package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: YpkQVMMonomials.java,v 1.8 2022/07/10 10:41:25 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class YpkQVMMonomials extends YpkQVM<QVMData> {
YpkQVMMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static YpkQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final YpkQVMMonomials ref)
{
    final int nvm=1,npm=4;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new YpkQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
double getPOffset(){return odata.p_offset;}
double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}

static class YpkQVMMonomialsTest extends YpkQVMTest<QVMData> {
YpkQVMMonomialsTest(final long initrandom)
{
    super("YpkQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}


@Override
void test_Extra(){
    final YpkQVMMonomials q=getYpkQVM();
    final YpkQVMLaguerre s=getYpkQVMLaguerre();

    EVXData.matchEVXData(q.pI,s.pI,false,true,eps);
    EVXData.matchEVXData(q.dpI,s.dpI,false,true,eps);

    UnitTests.checkDoubleRel(q.P_pI_psiH,"q.P_pI_psiH",s.P_pI_psiH,"s.P_pI_psiH",eps);
    UnitTests.checkDoubleRel(q.P_I_rpsiH,"q.P_I_rpsiH",s.P_I_rpsiH,"s.P_I_rpsiH",eps);
    if(neq>2)
    {
	UnitTests.checkDoubleRel(q.varP_pI_psiH,"q.varP_pI_psiH",s.varP_pI_psiH,"s.varP_pI_psiH",eps);
	UnitTests.checkDoubleRel(q.varP_I_rpsiH,"q.varP_I_rpsiH",s.varP_I_rpsiH,"s.varP_I_rpsiH",eps);
    }

 System.err.println(name+":YpkQVMMonomialsTest:test_Extra(): test YpkQVMLaguerre OK neq="+neq+" eps="+eps);
}

YpkQVMLaguerre getYpkQVMLaguerre(){return YpkQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
YpkQVMMonomials getYpkQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

} // END of class YpkQVMMonomialsTest

public static void main(String [] args)
{
    new YpkQVMMonomialsTest(1117L).runTests(5);
} // END main



} // END OF class YpkQVMMonomials
