package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLaguerreBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: YpnlQVMLaguerre.java,v 1.29 2022/07/29 19:35:39 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class YpnlQVMLaguerre extends YpnlQVM<QVMDataL> {
YpnlQVMLaguerre(final int neq,final QVMDataL odata,final State rst){super(neq,odata,rst);}


public static YpnlQVMLaguerre calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final YpnlQVMLaguerre ref)
{
    final int npm=2;
    final QVMDataL o=QVMDataL.calculateQVMDataL(2*(neq-1)+1+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new YpnlQVMLaguerre(neq,o,ref!=null?ref.sst:null);
}
@Override QVMDataLDirectAccess getAccess(){return new QVMDataLDirectAccess();}
@Override OrthogonalPolynomialsLaguerreBasis getABasis(){return new OrthogonalPolynomialsLaguerreBasis();}
@Override PsiIntegratedByInterval getINTRV(){return PsiIntegratedByInterval.INTR_QVMDataL;}
@Override double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
@Override double [] getEDPsi(final double [] psi){return WIntegratorLaguerre.getEDPsi(psi);}
@Override double [] getEIntPsi(final double [] psi){return KkQVMLaguerre._getEIntPsi_test(psi);}
@Override K3Distance getK3Distance(final double [] a,final double [] b,final double [] c)
{
    return new K3DistanceLaguerre(a,b,c);
}
@Override double [] getPsi2WIntegrated(final double [] psi)
{
    return WIntegratorLaguerre.getPsi2WIntegratedDt(psi);
}
@Override K2Distance getK2Distance(){return KkQVMLaguerre.K2DISTANCE;}

@Override java.util.function.Function<double [],double []> getFFromDFMatchRadau() {return getAccess().getFMomentsFromDFMomentsMatchRadau;}


static class YpnlQVMLaguerreTest extends YpnlQVMTest<QVMDataL> {
YpnlQVMLaguerreTest(final long initrandom)
{
    super("YpnlQVMLaguerre",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return WIntegratorLaguerre.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}
YpnlQVMLaguerre getYpnlQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}
boolean testK3(final int n){return n<4;}

void test_Extra(){
    final YpnlQVMLaguerre q=getYpnlQVM();
    if(true)
    {
	UnitTests.checkDoubleRel(q.vrDit.tr_IH,"q.vrDit.tr_IH",1,"1",eps);
	System.err.println(name+":test_Extra():DynRXPnL: tr_IH test OK n="+neq+" eps="+eps);
    }
}
} // END of class YpnlQVMLaguerreTest

public static void main(String [] args)
{
    new YpnlQVMLaguerreTest(-15711112L).runTests(6);
} // END main



} // END OF class YpnlQVMLaguerre
