package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.OrthogonalPolynomialsLegendreShiftedBasis;
import com.polytechnik.utils.TMatr;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: YpnlQVMLegendreShifted.java,v 1.35 2022/07/29 19:35:39 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class YpnlQVMLegendreShifted extends YpnlQVM<QVMDataP> {

YpnlQVMLegendreShifted(final int neq,final QVMDataP odata,final State rst){super(neq,odata,rst);}

public static YpnlQVMLegendreShifted calculate(final int neq,
					     final long currenttime,
					     final long tau,
					     final Trade [] tb,
					     final int tsize,
					     final YpnlQVMLegendreShifted ref)
{
    final int npm=2;
    final QVMDataP o=QVMDataP.calculateQVMDataP(2*(neq-1)+1+1,
						npm,
						currenttime,
						tau,
						tb,
						tsize,
						ref!=null?ref.odata:null);
    return new YpnlQVMLegendreShifted(neq,o,ref!=null?ref.sst:null);
}
@Override QVMDataPDirectAccess getAccess(){return new QVMDataPDirectAccess();}
@Override OrthogonalPolynomialsLegendreShiftedBasis getABasis(){return new OrthogonalPolynomialsLegendreShiftedBasis();}
@Override PsiIntegratedByInterval getINTRV(){return PsiIntegratedByInterval.INTR_QVMDataP;}
@Override double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
@Override double [] getEDPsi(final double [] psi){return WIntegratorLegendreShifted.getEDPsi(psi);}
@Override double [] getEIntPsi(final double [] psi){return KkQVMLegendreShifted._getEIntPsi_test(psi);}
@Override K3Distance getK3Distance(final double [] a,final double [] b,final double [] c)
{
    return new K3DistanceLegendreShifted(a,b,c);
}
@Override double [] getPsi2WIntegrated(final double [] psi){return  WIntegratorLegendreShifted.getPsi2WIntegratedDt(psi);}
@Override K2Distance getK2Distance(){return KkQVMLegendreShifted.K2DISTANCE;}

@Override java.util.function.Function<double [],double []> getFFromDFMatchRadau() {return getAccess().getFMomentsFromDFMomentsMatchRadau;}

static class YpnlQVMLegendreShiftedTest extends YpnlQVMTest<QVMDataP> {
YpnlQVMLegendreShiftedTest(final long initrandom)
{
    super("YpnlQVMLegendreShifted",new java.util.Random(initrandom),1e-5);
}
double [] convertToMonomials(final double [] p){return WIntegratorLegendreShifted.B.convertFunctionTo_Monomials_from_Basis(p);}

long getTau()
{
    return ((tb[tb.length-1].time-tb[0].time)/3)/(9*5*7*8)*(9*5*7*8);
}
YpnlQVMLegendreShifted getYpnlQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

void test_Extra(){
    final YpnlQVMLegendreShifted q=getYpnlQVM();
    final int n=neq,nmoms=2*n-1;
    final double [][] mm=q.getMoments(nmoms);
    final double [] tmom=mm[0],imom=mm[1],pimom=mm[2];
    if(true)
    {
	final double dtr_IH=TMatr.mulVVn1(q.M.B.mult2Pol(q.vrDit.ev.psiH,q.vrDit.ev.psiH),q.M.B.getXQkMomentsFromQkMoments(tmom));
	UnitTests.checkDoubleRel(q.vrDit.tr_IH,"q.vrDit.tr_IH",dtr_IH,"dtr_IH",eps);
	UnitTests.checkDoubleRel(q.vrit.tr_IH,"q.vrit.tr_IH",1-dtr_IH,"1-dtr_IH",eps);
	System.err.println(name+":test_Extra():DynRXPnL: tr_IH test OK n="+n+" eps="+eps);
    }
}

} // END of class YpnlQVMLegendreShiftedTest

public static void main(String [] args)
{
    new YpnlQVMLegendreShiftedTest(1116611L).runTests(7);
} // END main



} // END OF class YpnlQVMLegendreShifted
