package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2013
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.freemoney.*;
import com.polytechnik.realtimebook.*;

/**
 *  @version $Id: YpnlQVMMonomials.java,v 1.33 2022/07/29 19:35:39 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class YpnlQVMMonomials extends YpnlQVM<QVMData> {
YpnlQVMMonomials(final int neq,final QVMData odata,final State rst){super(neq,odata,rst);}


public static YpnlQVMMonomials calculate(final int neq,
					    final long currenttime,
					    final long tau,
					    final Trade [] tb,
					    final int tsize,
					    final YpnlQVMMonomials ref)
{
    final int nvm=1,npm=2;
    final QVMData o=QVMData.calculateQVMData(nvm,
					     2*(neq-1)+1+1,
					     npm,
					     currenttime,
					     tau,
					     tb,
					     tsize,
					     ref!=null?ref.odata:null);
    return new YpnlQVMMonomials(neq,o,ref!=null?ref.sst:null);
}
@Override QVMDataDirectAccess getAccess(){return new QVMDataDirectAccess();}
@Override OrthogonalPolynomialsMonomialsBasis getABasis(){return new OrthogonalPolynomialsMonomialsBasis0();}
@Override PsiIntegratedByInterval getINTRV(){return PsiIntegratedByInterval.INTR_QVMData;}
@Override double getPLast(){return odata.trade_last!=null?odata.trade_last.price:Double.NaN;}
@Override double [] getEDPsi(final double [] psi){return WIntegratorMonomials.getEDPsi(psi);}
@Override double [] getEIntPsi(final double [] psi){return KkQVMMonomials._getEIntPsi_test(psi);}
@Override K3Distance getK3Distance(final double [] a,final double [] b,final double [] c)
{
    return new K3DistanceMonomials(a,b,c);
}
@Override double [] getPsi2WIntegrated(final double [] psi)
{
    return WIntegratorMonomials.getPsi2WIntegratedDt(psi);
}
@Override K2Distance getK2Distance(){return KkQVMMonomials.K2DISTANCE;}

@Override java.util.function.Function<double [],double []> getFFromDFMatchRadau() {return getAccess().getFMomentsFromDFMomentsMatchRadau;}


static class YpnlQVMMonomialsTest extends YpnlQVMTest<QVMData> {
YpnlQVMMonomialsTest(final long initrandom)
{
    super("YpnlQVMMonomials",new java.util.Random(initrandom),1e-4);
}
double [] convertToMonomials(final double [] p){return p;}

long getTau()
{
    return (tb[tb.length-1].time-tb[0].time)/24;
}

void test_Offset(){
    final int n=neq,nmoms=2*n-1;
    final YpnlQVMMonomials q=getYpnlQVM();
    final YpnlQVMLaguerre s=getYpnlQVMLaguerre();
    final DataInterfaceToMoments<QVMData> access=q.getAccess();
    final double [] tmom=new double [nmoms],imom=new double [nmoms],pimom=new double [nmoms];

    access.setM(imom,0,access.M_I,q.odata,nmoms);
    access.setM(pimom,1,access.M_I,q.odata,nmoms);
    access.setMT(tmom,nmoms);
    final double [] QQt=q.M.B.getQQMatr(n,tmom),QQi=q.M.B.getQQMatr(n,imom),QQpi=q.M.B.getQQMatr(n,pimom);
    final double o=q.getPLast()-q.getPOffset();
    for(DynVarC a:new DynVarC []{
	    DynVarC.get0_PnL_DynVarC(n,QQi,QQt,QQpi,q.M,o),
	    DynVarC.test_get0_PnL_DynVarC(n,QQi,QQt,QQpi,q.M,o),
	    DynVarC.test2_get0_PnL_DynVarC(n,QQi,QQt,QQpi,q.M,o)})
    for(DynVarC b:new DynVarC []{q.vc0_PnL_IH,s.vc0_PnL_IH})
    {
	UnitTests.checkDoubleRel(a.I_0,"a.I_0",b.I_0,"b.I_0",eps);
	UnitTests.checkDoubleRel(a.I_IL,"a.I_IL",b.I_IL,"b.I_IL",eps);
	UnitTests.checkDoubleRel(a.I_IH,"a.I_IH",b.I_IH,"b.I_IH",eps);
	UnitTests.checkDoubleRel(a.I_IX,"a.I_IX",b.I_IX,"b.I_IX",eps);
	UnitTests.checkDoubleRel(a.p_IX-o,"a.p_IX-o",b.p_IX,"b.p_IX",eps);
	UnitTests.checkDoubleRel(a.Id_cL,"a.Id_cL",b.Id_cL,"b.Id_cL",eps);
	UnitTests.checkDoubleRel(a.p_cL,"a.p_cL",b.p_cL,"b.p_cL",eps);
	UnitTests.checkDoubleRel(a.Id_cH,"a.Id_cH",b.Id_cH,"b.Id_cH",eps);
	UnitTests.checkDoubleRel(a.p_cH,"a.p_cH",b.p_cH,"b.p_cH",eps);
    }
    System.err.println(name+":YpnlQVMMonomialsTest:test_Offset(): test OK neq="+neq+" eps="+eps);
}

@Override
void test_Extra(){
    final YpnlQVMMonomials q=getYpnlQVM();
    final YpnlQVMLaguerre s=getYpnlQVMLaguerre();

    UnitTests.checkDoubleRel(Math.abs(q.w_pnl_L_Id_H),"q.w_pnl_L_Id_H",Math.abs(s.w_pnl_L_Id_H),"s.w_pnl_L_Id_H",eps);
    UnitTests.checkDoubleRel(Math.abs(q.w_pnl_H_Id_H),"q.w_pnl_H_Id_H",Math.abs(s.w_pnl_H_Id_H),"s.w_pnl_H_Id_H",eps);
    UnitTests.checkDoubleRel(Math.abs(q.w_dp_L_Id_H),"q.w_dp_L_Id_H",Math.abs(s.w_dp_L_Id_H),"s.w_dp_L_Id_H",eps);
    UnitTests.checkDoubleRel(Math.abs(q.w_dp_H_Id_H),"q.w_dp_H_Id_H",Math.abs(s.w_dp_H_Id_H),"s.w_dp_H_Id_H",eps);

    UnitTests.checkDoubleRel(q.dp_I_psiH,"q.dp_I_psiH",s.dp_I_psiH,"s.dp_I_psiH",eps);
    UnitTests.checkDoubleRel(q.px_I_psiL,"q.px_I_psiL",s.px_I_psiL,"s.px_I_psiL",eps);
    UnitTests.checkDoubleRel(q.px_I_psiH,"q.px_I_psiH",s.px_I_psiH,"s.px_I_psiH",eps);

    EVX2Data.matchEVX2Data(q.dI_dp,s.dI_dp,true,false,eps);
    EVX2Data.matchEVX2Data(q.dI_d2p,s.dI_d2p,true,false,eps);
    EVX2Data.matchEVX2Data(q.I_dp,s.I_dp,true,false,eps);
    EVX2Data.matchEVX2Data(q.I_d2p,s.I_d2p,true,false,eps);
    UnitTests.checkDoubleRel(q.sp0.dirC,"q.sp0.dirC",s.sp0.dirC,"s.sp0.dirC",eps);
    UnitTests.checkDoubleRel(q.sp0.dirP,"q.sp0.dirP",s.sp0.dirP,"s.sp0.dirP",eps);
    UnitTests.checkDoubleRel(q.sp0.dirPC,"q.sp0.dirPC",s.sp0.dirPC,"s.sp0.dirPC",eps);

    if(true)
    {
	DynRXPnL.matchDynRXPnL(q.vrDit,s.vrDit,eps);
	DynRXPnL.matchDynRXPnL(q.vrit,s.vrit,eps);
	UnitTests.checkDoubleRel(q.vrDit.tr_IH,"q.vrDit.tr_IH",1,"1",eps);
	System.err.println(name+":test_Extra():DynRXPnL: tr_IH test OK n="+neq+" eps="+eps);
    }

    DynpIV.matchDynpIV(q.pIV0,s.pIV0,eps);
    DynpIV.matchDynpIV(q.pIVR,s.pIVR,eps);
    if(!(neq<3)) DynVarC.matchDynVarC(q.VC_PnL_IH0,s.VC_PnL_IH0,eps);
    if(!(neq<3)) DynVarFiniteC.matchDynVarFiniteC(q.finC_pV,s.finC_pV,eps);
    if(!(neq<3)) DynVarFiniteC.matchDynVarFiniteC(q.finC_pI,s.finC_pI,eps);

    System.err.println(name+":YpnlQVMMonomialsTest:test_Extra(): test YpnlQVMLaguerre OK neq="+neq+" eps="+eps);
    test_Offset();
}


YpnlQVMLaguerre getYpnlQVMLaguerre(){return YpnlQVMLaguerre.calculate(neq,currenttime,tau,tb,tb.length,null);}
YpnlQVMMonomials getYpnlQVM(){return calculate(neq,currenttime,tau,tb,tb.length,null);}

boolean testK3(final int n){return n<5;}

} // END of class YpnlQVMMonomialsTest

public static void main(String [] args)
{
    new YpnlQVMMonomialsTest(7L).runTests(5);
} // END main



} // END OF class YpnlQVMMonomials
