package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.TMatr;
import com.polytechnik.realtimebook.*;

/** ZQVM and a set of measures.
 *  @version $Id: ZQVM.java,v 1.52 2022/08/08 20:10:47 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class ZQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;

/** Rurik evolution. */
final double K00,
    ntrvs0,sigma2T_ntrvs0,otrvs0,sigma2A_otrvs0,
    ntsvs0,sigma2T_ntsvs0,otsvs0,sigma2A_otsvs0,
    ntrts0,sigma2T_ntrts0,otrts0,sigma2A_otrts0,
    nA0,sigma2T_nA0,oA0,sigma2A_oA0,nIdp0;

final double [] kk,kkr,kks,kkrt,kka;

ZQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();

    final int nmoms=2*neq-1;

    final double [] rvsmom=new double [nmoms],vsmom=new double [nmoms],rtsmom=new double [nmoms],vmom=new double [nmoms],tmom=new double [nmoms],dpmom=new double [nmoms];
    access.setM(rvsmom,0,access.M_RVS,odata,nmoms);
    access.setM(vsmom,0,access.M_VS,odata,nmoms);
    access.setM(rtsmom,0,access.M_RTS,odata,nmoms);
    access.setM(vmom,0,access.M_I,odata,nmoms);
    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setMT(tmom,nmoms);
    final double [][] QQ=access.getQQ_Matrs(neq,1,new double [][]{rvsmom,vsmom,rtsmom,vmom,tmom,dpmom});
    final double [] QQrvs=QQ[0],QQsvs=QQ[1],QQrts=QQ[2],QQv=QQ[3],QQt=QQ[4],QQdp=QQ[5],
	tmp=new double [neq*neq],
	IQQ=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQt),
	QQr=getIQQ_F_IQQ(neq,IQQ,QQrvs,tmp),
	QQa=getIQQ_F_IQQ(neq,IQQ,QQv,tmp),
	QQs=getIQQ_F_IQQ(neq,IQQ,QQsvs,tmp),QQrt=getIQQ_F_IQQ(neq,IQQ,QQrts,tmp),
	KQQdp=getIQQ_F_IQQ(neq,IQQ,TMatr.mulMatrices(neq,new double [][]{QQdp,IQQ,QQv}),tmp);

    final double [][] kkk=access.getKK(neq,new double [][] {IQQ,QQr,QQs,QQrt,QQa,KQQdp});
    kk=kkk[0];
    kkr=kkk[1];
    kks=kkk[2];
    kkrt=kkk[3];
    kka=kkk[4];
    final double [] kkIdp=kkk[5];
    final double x0=getX0();
    K00=access.fx(kk,x0);
    final double sqrtK00=Math.sqrt(K00);

    ntrvs0=access.fx(kkr,x0)/K00;
    sigma2T_ntrvs0=TMatr.getSpurMM(neq,QQrvs,QQr)-ntrvs0*ntrvs0;
    double [] r=TMatr.getMatr_x_A(neq,IQQ,java.util.Arrays.copyOf(rvsmom,neq));
    otrvs0=access.fx(r,x0)/sqrtK00;
    sigma2A_otrvs0=TMatr.mulVVn1(r,rvsmom)-otrvs0*otrvs0;

    ntsvs0=access.fx(kks,x0)/K00;
    sigma2T_ntsvs0=TMatr.getSpurMM(neq,QQsvs,QQs)-ntsvs0*ntsvs0;
    r=TMatr.getMatr_x_A(neq,IQQ,java.util.Arrays.copyOf(vsmom,neq));
    otsvs0=access.fx(r,x0)/sqrtK00;
    sigma2A_otsvs0=TMatr.mulVVn1(r,vsmom)-otsvs0*otsvs0;

    ntrts0=access.fx(kkrt,x0)/K00;
    sigma2T_ntrts0=TMatr.getSpurMM(neq,QQrts,QQrt)-ntrts0*ntrts0;
    r=TMatr.getMatr_x_A(neq,IQQ,java.util.Arrays.copyOf(rtsmom,neq));
    otrts0=access.fx(r,x0)/sqrtK00;
    sigma2A_otrts0=TMatr.mulVVn1(r,rtsmom)-otrts0*otrts0;

    nA0=access.fx(kka,x0)/K00;
    sigma2T_nA0=TMatr.getSpurMM(neq,QQa,QQv)-nA0*nA0;
    r=TMatr.getMatr_x_A(neq,IQQ,java.util.Arrays.copyOf(vmom,neq));
    oA0=access.fx(r,x0)/sqrtK00;
    sigma2A_oA0=TMatr.mulVVn1(r,vmom)-oA0*oA0;

    nIdp0=access.fx(kkIdp,x0)/K00;
}

static double [] getIQQ_F_IQQ(final int n,final double [] IQQ,final double [] F,final double [] tmp)
{
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,F,n,IQQ,n);
    return TMatr.mulMM(n,IQQ,tmp);
}



abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract double getX0();

static abstract class ZQVMTest<DTYPE> {

int neq;
long currenttime,tau;
Trade [] tb;

abstract void setData(final int tsize,final int neq);
abstract ZQVM<DTYPE> getZQVM();
abstract VQVM<DTYPE> getVQVM();
abstract XQVM<DTYPE> getXQVM();
abstract YQVM<DTYPE> getYQVM();

final String name;
final java.util.Random r;
final double eps;
ZQVMTest(final String name,
	 final java.util.Random r,final double eps)
{
    this.name=name;
    this.r=r;
    this.eps=eps;
}
void test_Common()
{
    final ZQVM<DTYPE> q=getZQVM();

    System.err.println(name+":ZQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}
void test_VQVM()
{
    final ZQVM<DTYPE> q=getZQVM();
    final VQVM<DTYPE> s=getVQVM();

    UnitTests.checkDoubleArrayRel(q.kk,"q.kk",s.kka,"s.kka",eps);
    UnitTests.checkDoubleArrayRel(q.kks,"q.kks",s.kkdvse,"s.kkdvse",eps);
    UnitTests.checkDoubleRel(q.otsvs0,"q.otsvs0",s.rdvs0,"s.rdvs0",eps);
    UnitTests.checkDoubleRel(q.sigma2A_otsvs0,"q.sigma2A_otsvs0",s.sigma2_rdvs0,"s.sigma2_rdvs0",eps);
    UnitTests.checkDoubleRel(q.ntsvs0,"q.ntsvs0",s.dvse0,"s.dvse0",eps);

    System.err.println(name+":ZQVMTest:test_VQVM(): test OK neq="+neq+" eps="+eps);
}
void test_YQVM()
{
    final ZQVM<DTYPE> q=getZQVM();
    final YQVM<DTYPE> s=getYQVM();

    UnitTests.checkDoubleRel(q.K00,"q.K00",s.otK00,"s.otK00",eps);

    UnitTests.checkDoubleRel(q.otrvs0,"S2 r q",s.otrdvs0,"S2 r s",eps);
    UnitTests.checkDoubleRel(q.sigma2A_otrvs0,"S2 r q",s.sigma2_otrdvs0,"S2 r s",eps);
    UnitTests.checkDoubleRel(q.otsvs0,"S2 s q",s.otdvs0,"S2 s s",eps);
    UnitTests.checkDoubleRel(q.sigma2A_otsvs0,"S2 s q",s.sigma2_otdvs0,"S2 s s",eps);

    System.err.println(name+":ZQVMTest:test_YQVM(): test OK neq="+neq+" eps="+eps);
}
void test_XQVM()
{
    final ZQVM<DTYPE> q=getZQVM();
    final XQVM<DTYPE> s=getXQVM();

    UnitTests.checkDoubleRel(q.ntsvs0,"q.ntsvs0",s.ntdvs0a,"s.ntdvs0a",eps);
    UnitTests.checkDoubleRel(q.sigma2T_ntsvs0,"q.sigma2T_ntsvs0",s.sigma2T_ntdvs0a,"s.sigma2T_ntdvs0a",eps);
    UnitTests.checkDoubleRel(q.sigma2A_otsvs0,"q.sigma2A_otsvs0",s.sigma2A_otdvs0a,"s.sigma2A_otdvs0a",eps);

    UnitTests.checkDoubleRel(q.ntrvs0,"q.ntrvs0",s.ntrvs0a,"s.ntrvs0a",eps);
    UnitTests.checkDoubleRel(q.sigma2T_ntrvs0,"q.sigma2T_ntrvs0",s.sigma2T_ntrvs0a,"s.sigma2T_ntrvs0a",eps);
    UnitTests.checkDoubleRel(q.sigma2A_otrvs0,"q.sigma2A_otrvs0",s.sigma2A_otrvs0a,"s.sigma2A_otrvs0a",eps);

    UnitTests.checkDoubleRel(q.K00,"q.K00",s.ntK00,"s.ntK00",eps);
    UnitTests.checkDoubleRel(q.ntsvs0,"q.ntsvs0",s.ntdvs0,"s.ntdvs0",eps);
    UnitTests.checkDoubleRel(q.otsvs0,"q.otsvs0",s.otdvs0,"s.otdvs0",eps);
    UnitTests.checkDoubleRel(q.sigma2A_otsvs0,"q.sigma2A_otsvs0",s.sigma2_otdvs0,"s.sigma2_otdvs0",eps);
    UnitTests.checkDoubleRel(q.sigma2A_otsvs0,"q.sigma2A_otsvs0",s.sigma2A_otdvs0,"s.sigma2A_otdvs0",eps);
    UnitTests.checkDoubleRel(q.ntrvs0,"q.ntrvs0",s.rvs0,"s.rvs0",eps);
    UnitTests.checkDoubleRel(q.otrvs0,"q.otrvs0",s.otrdvs0,"s.otrdvs0",eps);
    UnitTests.checkDoubleRel(q.sigma2A_otrvs0,"q.sigma2A_otrvs0",s.sigma2_otrdvs0,"s.sigma2_otrdvs0",eps);
    UnitTests.checkDoubleRel(q.sigma2A_otrvs0,"q.sigma2A_otrvs0",s.sigma2A_rvs0,"s.sigma2A_rvs0",eps);

    System.err.println(name+":ZQVMTest:test_XQVM(): test OK neq="+neq+" eps="+eps);
}

void test_Lastpoint()
{
    final ZQVM<DTYPE> q=getZQVM();
    final Trade [] tb=new Trade [this.tb.length+1];
    System.arraycopy(this.tb,0,tb,0,this.tb.length);
    final double pl=20.0+2*r.nextDouble();
    tb[this.tb.length]=new Trade(pl,
				 10000,
				 null,
				 r.nextInt(2)==1?BookItem.BOOK_BUY:BookItem.BOOK_SELL,
				 currenttime,
				 -1,-1,-1,
				 null,Trade.TRADE_TYPE_REGULAR,
				 Trade.getDummyTrade(pl-3*r.nextDouble(),100,BookItem.BOOK_BUY,currenttime),
				 Trade.getDummyTrade(pl+3*r.nextDouble(),100,BookItem.BOOK_SELL,currenttime),
				 -1);
    this.tb=tb;
    final ZQVM<DTYPE> s=getZQVM();

    UnitTests.checkDoubleRel(q.K00,"q.K00",s.K00,"s.K00",eps);
    UnitTests.checkDoubleRel(q.sigma2T_ntsvs0,"q.sigma2T_ntsvs0",s.sigma2T_ntsvs0,"s.sigma2T_ntsvs0",eps);
    UnitTests.checkDoubleRel(q.sigma2A_otsvs0,"q.sigma2A_otsvs0",s.sigma2A_otsvs0,"s.sigma2A_otsvs0",eps);
    UnitTests.checkDoubleRel(q.sigma2T_ntrvs0,"q.sigma2T_ntrvs0",s.sigma2T_ntrvs0,"s.sigma2T_ntrvs0",eps);
    UnitTests.checkDoubleRel(q.sigma2A_otrvs0,"q.sigma2A_otrvs0",s.sigma2A_otrvs0,"s.sigma2A_otrvs0",eps);
;   UnitTests.checkDoubleRel(q.sigma2T_nA0,"q.sigma2T_nA0",s.sigma2T_nA0,"s.sigma2T_nA0",eps);
    UnitTests.checkDoubleRel(q.sigma2A_oA0,"q.sigma2A_oA0",s.sigma2A_oA0,"s.sigma2A_oA0",eps);

    System.err.println(name+":ZQVMTest:test_Lastpoint(): test OK neq="+neq+" eps="+eps);
}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":ZQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_VQVM();
    test_XQVM();
    test_YQVM();
    if(neq>1) test_Lastpoint();

    System.err.println(name+":ZQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":ZQVMTest: starting tests; ver=$Id: ZQVM.java,v 1.52 2022/08/08 20:10:47 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(950+r.nextInt(859),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":ZQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: ZQVM.java,v 1.52 2022/08/08 20:10:47 mal Exp $");
}
} // END OF class ZQVMTest






}// END OF class ZQVM
