package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/** An interface with a single method  to convert attributes to attributes products not higher than
 * a given multiindex.
 * @version $Id: AttributesProductsMultiIndexed.java,v 1.16 2022/11/01 22:43:57 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public interface AttributesProductsMultiIndexed {

/** Create attributes products of a multi--index not greater than d.
 * @param d Maximal multi-index.
 * @param orig_sample Original sample.
 * @return A newly constructed sample with "product" attributes.
 */
DataReadObservationVectorXF getAttribsProductsMultiIndexed(
    final int d,
    final DataReadObservationVectorXF orig_sample);

    /** A helper class to scan recursivery.
     */
    static class MIScanner {
	final java.util.ArrayList<int []> allmultiindex=new java.util.ArrayList<>();
	final int d,mi[];
	MIScanner(final int d){
	    this.d=d;
	    this.mi=new int [d];
	}

	void scanmultiindex(int this_mi,int ind_end){
	    for(mi[this_mi]=0;mi[this_mi]<=ind_end;mi[this_mi]++){
		if(this_mi<this.d-1){
		    scanmultiindex(this_mi+1,mi[this_mi]);
		} else {
		    allmultiindex.add(mi.clone());
		}
	    }
	}
	/** A helper class.
	 */
	static class IndPower {
	    final int ind,order;
	    IndPower(final int ind,final int order){
		this.ind=ind;
		this.order=order;
	    }
	} // END of class IndPower
	String [] Xhmi;
	IndPower [][] ip;
	/** Sets Xhmi and ip.
	 */
	void prepareProductAttributesMultiIndexes(){
	    final int nXmi=allmultiindex.size();
	    this.ip=new IndPower[nXmi][];
	    this.Xhmi=new String [nXmi];
	    final java.util.ArrayList<IndPower> ipk=new java.util.ArrayList<>();
	    for(int k=0;k<nXmi;k++){
		final int [] mi=allmultiindex.get(k);
		// convert repeated attributes to their polynomial power
		//System.err.println("MIScanner:multiindex="+java.util.Arrays.toString(mi));
		final int [] mc=mi.clone();
		ipk.clear();

		for(int m=0;m<mc.length;m++){
		    if(mc[m]>=0){
			int order=0;
			// scan forward, find the same indexes, count them
			for(int mo=m;mo<mc.length;mo++){
			    if(mc[mo]>=0 && mi[m]==mc[mo]){
				order++;
				mc[mo]=-1;
			    }
			}
			ipk.add(new IndPower(mi[m],order));
		    }
		}
		ip[k]=ipk.toArray(IndPower[]::new);
		// the header
		Xhmi[k]=java.util.Arrays.stream(ip[k]).map(o->"["+o.ind+"]^"+o.order).collect(java.util.stream.Collectors.joining("*"));
		// diag
		if(true || RadonNikodymSpectralModel.EPS_selftest>0){
		    System.err.println("MIScanner:multiindex="+java.util.Arrays.toString(mi)+" ip="+Xhmi[k]);
		}
	    }
	}
    } // END OF class MIScanner

@FunctionalInterface
public interface BasisFunction1D {
    double apply(int order,double x);
}


/** A helper method.
 * @param d Maximal multi-index.
 * @param orig_sample Original sample.
 * @param convertxFromOriginalToX basis conversion, product attributes
 * are constructed in X basis (not in original).
 * @return A newly constructed sample with "product" attributes.
 */
public static DataReadObservationVectorXF getAttribsProductsMultiIndexed(
    final int d,
    final DataReadObservationVectorXF orig_sample,
    final java.util.function.Function<double [],double []> convertxFromOriginalToX){
    return getAttribsProductsMultiIndexed(d,orig_sample,convertxFromOriginalToX,(order,x)->Math.pow(x,order));
}

/** A helper method.
 * @param d Maximal multi-index.
 * @param orig_sample Original sample.
 * @param convertxFromOriginalToX basis conversion, product attributes
 * are constructed in X basis (not in original).
 * @param basisfunX Basis function calculation (Monomials or Chebyshev).
 * @return A newly constructed sample with "product" attributes.
 */
static DataReadObservationVectorXF getAttribsProductsMultiIndexed(
    final int d,
    final DataReadObservationVectorXF orig_sample,
    final java.util.function.Function<double [],double []> convertxFromOriginalToX,
    final BasisFunction1D basisfunX){

    final int noriginal=orig_sample.xHeader.size(),nX=convertxFromOriginalToX.apply(new double [noriginal]).length;

    final MIScanner s=new MIScanner(d);
    s.scanmultiindex(0,nX-1);
    s.prepareProductAttributesMultiIndexes();
    final java.util.ArrayList<ObservationVectorXF> data=new java.util.ArrayList<>();

    if(true || RadonNikodymSpectralModel.EPS_selftest>0){
	System.err.println("AttributesProductsMultiIndexed.getAttribsProductsMultiIndexed:input noriginal="+noriginal+" nX="+nX+" sample size="+orig_sample.data.size()+" sum(w)="+orig_sample.data.stream().mapToDouble(b->b.weight).sum()+" max_multiindex=d="+d+" terms N("+nX+","+d+")="+s.ip.length+" C_{nX+d-1}^d=C_"+(nX+d-1)+"^"+d+"="+Polynomials.C(nX+d-1,d));
    }


    for(ObservationVectorXF o:orig_sample.data)
    {
	final double [] X=convertxFromOriginalToX.apply(o.x),xnmi=new double [s.ip.length];
	//System.err.println("x="+java.util.Arrays.toString(o.x)+" X="+java.util.Arrays.toString(X)+" w="+o.weight);
	for(int k=0;k<s.ip.length;k++){
	    double p=1;
	    for(AttributesProductsMultiIndexed.MIScanner.IndPower i:s.ip[k]) {
		p*=basisfunX.apply(i.order,X[i.ind]);
	    }
	    xnmi[k]=p;
	}
	data.add(new ObservationVectorXF(o.label,xnmi,o.f,o.weight));
    }
    return new DataReadObservationVectorXF(data,java.util.Arrays.asList(s.Xhmi));
}

}


