package com.polytechnik.utils;

/** Get confederate matrix.
 *
 * @version $Id: ConfederateMatrixCalculatable.java,v 1.5 2019/10/04 07:45:41 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
@FunctionalInterface
public interface ConfederateMatrixCalculatable {

    /** Calculates confederate matrix to solve
     * polynomial equations in basis directly.
     * The leading dimension of returned matrix is coefs.length-1,
     * the number of elements (coefs.length-1)^2.
     * Eigenvalues of confederate matrix are the roots.
     * See:
     * Tom Bella, Y.Eidelman, I.Gohberg, V.Olshevsky, E.Tyrtyshnikov,
     * "Fast inversion of Hessenberg-quasiseparable-Vandermonde matrices
     * and resulting recurrence relations and characterizations", March 2007.
     * <a href="http://www.math.uconn.edu/~olshevsky/papers/traubqs2_tb36.pdf">http://www.math.uconn.edu/~olshevsky/papers/traubqs2_tb36.pdf</a>
     * Use the class {@link PolynomialRootsConfederateMatrix PolynomialRootsConfederateMatrix}
     * to calculate the roots.
     */
    double [] getConfederateMatrix(final double [] coefs);
}
