package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/** A class to store read sample data.
  * @version $Id: DataReadObservationVectorXF.java,v 1.117 2024/05/07 11:30:33 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
 */
public class DataReadObservationVectorXF {
    public final java.util.List<ObservationVectorXF> data;
    public final java.util.List<String> xHeader;
    public final DataStat datastat;

    public DataReadObservationVectorXF(final java.util.List<ObservationVectorXF> data,final java.util.List<String> xHeader)
    {
	this.data=data;
	this.xHeader=xHeader;
	this.datastat=new DataStat(this);
    }

    public void printData()
    {
	for(ObservationVectorXF o:data)
	{
	    System.err.println("x="+java.util.Arrays.toString(o.x)+"\tf="+o.f+"\tweight="+o.weight);
	}
    }

    /** Create a dataset with observations replaced.
     */
    public DataReadObservationVectorXF getReplacedData(
	final java.util.function.Function<ObservationVectorXF,ObservationVectorXF> replacement,
	final java.util.function.Function<java.util.List<String>,java.util.List<String>> getheader){
	final java.util.ArrayList<ObservationVectorXF> newdata=new java.util.ArrayList<>();
	for(ObservationVectorXF o:data){
	    newdata.add(replacement.apply(o));
	}
	return new DataReadObservationVectorXF(newdata,getheader.apply(xHeader));
    }


    /**  Regularization parameter.
     */
    static final double reg_eps_rel=1e-5,reg_eps_abs=1e-6;

    class AttribsPreparation {
	final double [] norm,matrnormalized,fmatrnorm;
	final int [] inds;
	AttribsPreparation(final int n,double [] XfX,double [] XX){
	    final double [] covar=TMatr.getVectorScaled(XX,1/datastat.totalweight),fcovarnoscale=XfX;
	    int ngood=0,indstmp[]=new int [n];
	    this.norm=new double [n];
	    for(int k=0;k<n;k++){
		norm[k]=Math.sqrt(covar[k+n*k]);
		if(norm[k]>reg_eps_abs){
		    // an attribute with non--zero standard deviation is found
		    indstmp[ngood++]=k;
		}
	    }
	    this.inds=java.util.Arrays.copyOf(indstmp,ngood);

	    //System.err.println("noriginal="+n+" inds.length="+inds.length+" covar="+java.util.Arrays.toString(covar)+" fcovarnoscale="+java.util.Arrays.toString(fcovarnoscale));
	    //System.err.println("ev covar="+java.util.Arrays.toString(Skewness.MB.EV.getEV(n,covar,false)));

	    this.matrnormalized=new double [ngood*ngood];
	    this.fmatrnorm=new double [n*n];
	    for(int kl=0;kl<ngood;kl++){
		final int k=inds[kl];
		for(int jl=0;jl<ngood;jl++){
		    final int j=inds[jl];
		    matrnormalized[kl*ngood+jl]=covar[k*n+j]/norm[k]/norm[j];
		    fmatrnorm[k*n+j]=fcovarnoscale[k*n+j]/norm[k]/norm[j];
		}
	    }
	}
    } // END OF class AttribsPreparation

    /** Add attribute x=const to basis.
     */
    XXMatrs addConstantToBasis(final int n,final int ngoodx,final double [] fmatrnorm,final double [] basisfun,final double [] fXaverage){
	final double [] XfX=EVXData.convertMatrFromQtoB_Basis(n,fmatrnorm,basisfun,ngoodx,0);
	// add the constant to the basis
	final int nX=ngoodx+1;
	final double [] aXfX=new double [nX*nX];
	for(int k=0;k<ngoodx;k++){
	    System.arraycopy(XfX,ngoodx*k,aXfX,nX*k,ngoodx);
	    aXfX[nX*(nX-1)+k]=aXfX[nX*k+(nX-1)]=fXaverage[k];
	}
	// const is the last attribute (nX-1) index.
	aXfX[(nX-1)*nX+(nX-1)]=datastat.faverage;
	return new XXMatrs(nX,nX,aXfX,TMatr.getUnitMatrix(nX));
    }

    String createAttributeTextualRepresentation(final double [] psiorigbasis,final int [] indexes,final double [] norm,final double [] xorig_average){
	// create attribute textual description
	final StringBuilder s=new StringBuilder();
	for(int jl=0;jl<indexes.length;jl++){
	    final int j=indexes[jl];
	    if(jl>0) s.append(" + ");
	    final double jcoef=psiorigbasis[j]/norm[j];
	    String jcoefdesc=String.valueOf(jcoef);
	    if(jcoefdesc.startsWith("-")) jcoefdesc="("+jcoefdesc+")";
	    s.append("(x["+j+"]- "+String.valueOf(xorig_average[j])+")*"+jcoefdesc);
	}
	return s.toString();
    }

    /** Remove duplicated data from x attributes.
     */
    public RegularizedVectorXF getDataRegularized_EV(){
	final int n=xHeader.size();
	final XXMatrs cv=getCovarianceMatrix();
	final AttribsPreparation ap=new AttribsPreparation(n,cv.XfX,cv.XX);

	final double [] ev_matrnormalized=Skewness.MB.EV.getEV(ap.inds.length,ap.matrnormalized,true);
	//System.err.println("matrnormalized="+java.util.Arrays.toString(matrnormalized)+" ev="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_matrnormalized,ap.inds.length)));
        double [] basisfun=new double [ap.inds.length*n];
	int ngoodx=0; // the number of good linear combinations of the input attributes found
	final double sqrt_totalweight=Math.sqrt(datastat.totalweight);
	final double [] fXaverage=new double [ap.inds.length];
	final java.util.ArrayList<String> attribsdescription=new java.util.ArrayList<>();
	for(int kl=0;kl<ap.inds.length;kl++){
	    // should we use Math.sqrt(ev_matrnormalized[kl])?
	    if(ev_matrnormalized[kl]>reg_eps_rel){
		// a good eigenvector is found
		final double [] psi=TMatr.scaleVectorInPlace(EVXData.getPsiFromBasis(kl,ev_matrnormalized,ap.inds.length),Math.sqrt(1/ev_matrnormalized[kl])/sqrt_totalweight),psiorigbasis=new double [n];
		double fa=0;
		for(int jl=0;jl<ap.inds.length;jl++){
		    final int ojl=ap.inds[jl];
		    fa+=(psiorigbasis[ojl]=psi[jl])*(datastat.fxmxorig_average[ojl]/ap.norm[ojl]);
		}
		System.arraycopy(psiorigbasis,0,basisfun,ngoodx*n,n);
		fXaverage[ngoodx]=fa*sqrt_totalweight;
		attribsdescription.add(createAttributeTextualRepresentation(psiorigbasis,ap.inds,ap.norm,datastat.xorig_average));
		ngoodx++;
	    }
	}
	basisfun=java.util.Arrays.copyOf(basisfun,ngoodx*n);
	if(RadonNikodymSpectralModel.EPS_selftest>0){
	    System.err.println("getDataRegularized_EV(): noriginal="+n+" ngoodx="+ngoodx+" ap.inds.length="+ap.inds.length+" spectrum="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_matrnormalized,ap.inds.length))+" after a regularization eigenvectors with lower that "+reg_eps_rel+" eigenvalues will be removed; sum w="+datastat.totalweight);
	}

	// add constant to basis
	attribsdescription.add(String.valueOf(1/sqrt_totalweight));
	final XXMatrs amm=addConstantToBasis(n,ngoodx,ap.fmatrnorm,basisfun,fXaverage);

	if(RadonNikodymSpectralModel.EPS_selftest>0){
	    // amm.dim=1+ngoodx (constant added).
	    System.err.println("getDataRegularized_EV(): data regularized noriginal="+n+" amm.dim1="+amm.dim1+" amm.dim2="+amm.dim2);
	}
	return new RegularizedVectorXF(amm.dim2,n,amm.XfX,amm.XX,attribsdescription.toArray(String[]::new),datastat.xorig_average,ap.norm,ap.inds,basisfun,datastat.totalweight,RadonNikodymSpectralModel.EPS_selftest);
    }

    /** Remove duplicated data from x attributes,
     * similar to {@link #getDataRegularized_EV getDataRegularized_EV}
     * but a regular Gaussian elimination with pivoting is
     * used instead of the eigenproblem.
     * Similar to  {@link OrthogonalPolynomialsBasisFunctionsCalculatable#getGramSchmidtOrthogonalized getGramSchmidtOrthogonalized} with pivoting.
     */
    public RegularizedVectorXF getDataRegularized_LIN(){
	final int n=xHeader.size();
	final XXMatrs cv=getCovarianceMatrix();
	final AttribsPreparation ap=new AttribsPreparation(n,cv.XfX,cv.XX);

	final double sqrt_totalweight=Math.sqrt(datastat.totalweight);
	final double [] fXaverage=new double [ap.inds.length];
	final java.util.ArrayList<String> attribsdescription=new java.util.ArrayList<>();

	final double [][] basisfunS=new double [ap.inds.length][ap.inds.length];
	// the basis of ap.matrnormalized
	for(int kl=0;kl<ap.inds.length;kl++) {basisfunS[kl][kl]=1;}
	// non--degenerated basis
	double [] basisfun=new double [ap.inds.length*n],norm2basisfunS=new double [ap.inds.length];
	double basisfunScur[]={},norm2cur=1;
	int ngoodx=0; // the number of good linear combinations of the input attributes found
	for(int kl=0;kl<ap.inds.length;kl++) {
	    //System.err.println("kl="+kl+" of "+ap.inds.length+" total");
	    double norm2j_max=0;int indnorm2j_max=-1;
	    for(int jl=kl;jl<ap.inds.length;jl++){
		TMatr.sumP1(basisfunS[jl],basisfunScur,-TMatr.getAMatrB(basisfunS[jl],basisfunScur,ap.matrnormalized,ap.inds.length)/norm2cur);
		final double norm2j=TMatr.getAMatrA(basisfunS[jl],ap.matrnormalized,ap.inds.length);
		if(norm2j>=norm2j_max){
		    norm2j_max=norm2j;
		    indnorm2j_max=jl;
		}
	    }
	    // should we use Math.sqrt(norm2j_max)?
	    if(indnorm2j_max>=0 && norm2j_max>reg_eps_rel){
		// a good vector is found
		basisfunScur=basisfunS[indnorm2j_max];
		basisfunS[indnorm2j_max]=basisfunS[kl];
		basisfunS[kl]=basisfunScur;
		norm2cur=norm2basisfunS[kl]=norm2j_max;

		final double scaleCur=Math.sqrt(norm2cur)*sqrt_totalweight;
		final double [] psiorigbasis=new double [n];
		double fa=0;
		for(int jl=0;jl<ap.inds.length;jl++){
		    final int ojl=ap.inds[jl];
		    fa+=(psiorigbasis[ojl]=(basisfunScur[jl]/scaleCur))*(datastat.fxmxorig_average[ojl]/ap.norm[ojl]);
		}
		System.arraycopy(psiorigbasis,0,basisfun,ngoodx*n,n);
		fXaverage[ngoodx]=fa*sqrt_totalweight;
		attribsdescription.add(createAttributeTextualRepresentation(psiorigbasis,ap.inds,ap.norm,datastat.xorig_average));
		ngoodx++;
	    } else {
		break;
	    }
	}
	basisfun=java.util.Arrays.copyOf(basisfun,ngoodx*n);

	if(RadonNikodymSpectralModel.EPS_selftest>0){
	    System.err.println("getDataRegularized_LIN(): noriginal="+n+" ngoodx="+ngoodx+" ap.inds.length="+ap.inds.length+" variations="+java.util.Arrays.toString(norm2basisfunS)+" after a regularization vectors with lower that "+reg_eps_rel+" norm will be removed; sum w="+datastat.totalweight);
	}

	// add constant to basis
	attribsdescription.add(String.valueOf(1/sqrt_totalweight));
	final XXMatrs amm=addConstantToBasis(n,ngoodx,ap.fmatrnorm,basisfun,fXaverage);

	if(RadonNikodymSpectralModel.EPS_selftest>0){
	    // amm.dim=1+ngoodx (constant added).
	    System.err.println("getDataRegularized_LIN(): data regularized noriginal="+n+" amm.dim1="+amm.dim1+" amm.dim2="+amm.dim2);
	}
	return new RegularizedVectorXF(amm.dim2,n,amm.XfX,amm.XX,attribsdescription.toArray(String[]::new),datastat.xorig_average,ap.norm,ap.inds,basisfun,datastat.totalweight,RadonNikodymSpectralModel.EPS_selftest);
    }


    /** Covariance matrix (without normalizing to total weight).
    */
    XXMatrs getCovarianceMatrix(){
	final var t0=System.currentTimeMillis();
	final int n=xHeader.size();
	final double [] covar=new double [n*n],fcovar=new double [n*n];
	for(ObservationVectorXF o:data)
	{
	    for(int k=0;k<n;k++){
		double xkd=(o.x[k]-datastat.xorig_average[k]);
		for(int j=0;j<n;j++){
		    final double s=xkd*(o.x[j]-datastat.xorig_average[j])*o.weight;
		    covar[k*n+j]+=s;
		    fcovar[k*n+j]+=s*o.f;
		}
	    }
	    //System.err.println("o="+o.f+" n="+n);
	}
	final var t1=System.currentTimeMillis();System.err.println("getCovarianceMatrix() done in dt="+(t1-t0));
	return new XXMatrs(n,n,fcovar,covar);
    }

    /** Calculate &lt;observationToFun1(o)[j]|observationToFun2(o)[k]&gt;
     * and &lt;observationToFun1(o)[j]|f|observationToFun2(o)[k]&gt;
averages.
     * This is a more general version than {@link #getXXandXfX getXXandXfX}
     * that works only with symmetric matrices.
     * @param observationToFun1 a function to obtain first arg to average.
     * @param observationToFun2 a function to obtain second arg to average.
     * @return Two matrices matrix[j][k] of Fun1.length rows and Fun2.length columns
     * of product averages with and without f.
     */
    XXMatrs getFun1Fun2Averaged(
	final java.util.function.Function<ObservationVectorXF,double []> observationToFun1,
	final java.util.function.Function<ObservationVectorXF,double []> observationToFun2){

	final ObservationVectorXF ot=new ObservationVectorXF(null,new double [xHeader.size()],0,1);
	final int nFun1Values=observationToFun1.apply(ot).length,
	    nFun2Values=observationToFun2.apply(ot).length;
	final double [] scalprods=new double [nFun1Values*nFun2Values],
	    scalprodsF=new double [nFun1Values*nFun2Values];
	for(ObservationVectorXF o:data)
	{
	    final double [] x=observationToFun1.apply(o);
	    if(x.length!=nFun1Values) throw new IllegalArgumentException();
	    final double [] y=observationToFun2.apply(o);
	    if(y.length!=nFun2Values) throw new IllegalArgumentException();
	    for(int j=0;j<x.length;j++){
		final double fact=x[j]*o.weight,factF=o.f*fact;
		for(int k=0,of=j*nFun2Values;k<y.length;k++,of++){
		    scalprods[of]+=y[k]*fact;
		    scalprodsF[of]+=y[k]*factF;
		}
	    }
	}
	return new XXMatrs(nFun1Values,nFun2Values,scalprodsF,scalprods);
    }

    XXMatrs getXXandXfX(final java.util.function.Function<double [],double []> convertxFromOriginalToX){
	return getXXandXfX(convertxFromOriginalToX,(X)->1);
    }
    /** For unit test.
     */
    XXMatrs getXXandXfX(final java.util.function.Function<double [],double []> convertxFromOriginalToX,final java.util.function.ToDoubleFunction<double []> getFactorToDivide){
	final int nX=convertxFromOriginalToX.apply(new double [xHeader.size()]).length;
	final double [] XX=new double [nX*nX],XfX=new double [nX*nX];
	for(ObservationVectorXF o:data)
	{
	    final double [] X=convertxFromOriginalToX.apply(o.x);
	    //System.err.println("X.length="+X.length+" nX="+nX+" X="+java.util.Arrays.toString(X));
	    if(X.length!=nX) throw new IllegalArgumentException();
	    final double wf=getFactorToDivide.applyAsDouble(X);
	    for(int k=0;k<nX;k++){
		for(int j=0;j<nX;j++){
		    final double s=X[k]*X[j]*o.weight/wf;
		    XX[k*nX+j]+=s;
		    XfX[k*nX+j]+=s*o.f;
		}
	    }
	}
	return new XXMatrs(nX,nX,XfX,XX);
    }

    /** For unit tests. Use with a big care.
     * The method 1) may be unstable for redundant data 2) if there is no const
     * among x_k linear combinations the result is incorrect.
     */
    public RadonNikodymSpectralModel getRadonNikodymSpectralModel_WithoutRegularization(){
	final int n=xHeader.size();
	final XXMatrs rsxx=getXXandXfX((x)->x);
	return new RadonNikodymSpectralModel(
	    n,n,
	    rsxx.XfX,rsxx.XX,Skewness.MB.EV.getGEV_Generalized(n,rsxx.XfX,rsxx.XX,true),
	    (x)->x,
	    (psiX)->datastat.totalweight*TMatr.mulVV(datastat.xorig_average,psiX));
    }
    /** For unit tests. Use with a big care.
     * The method 1) may be unstabele for redundant data 2) if there is no const
     * among x_k linear combinations the result is incorrect.
     */
    public DataReadObservationVectorXF getAttribsProductsMultiIndexed_WithoutRegularization(final int d){
	return AttributesProductsMultiIndexed.getAttribsProductsMultiIndexed(d,this,(x)->x);
    }
    public static class DataStat {
	public final double totalweight,faverage;
	final double [] xorig_average,fxmxorig_average;
	DataStat(DataReadObservationVectorXF data)
	{
	    double wtotal=0,fa=0;
	    final int n=data.xHeader.size();
	    final double [] xa=new double [n];
	    for(ObservationVectorXF o:data.data)
	    {
		if(o.x.length!=n) throw new IllegalArgumentException();
		TMatr.sumP1(xa,o.x,o.weight);
		fa+=o.f*o.weight;
		wtotal+=o.weight;
	    }
	    this.faverage=fa/wtotal;
	    this.xorig_average=TMatr.getVectorScaled(xa,1/wtotal);
	    this.totalweight=wtotal;
	    if(false){
		this.fxmxorig_average=TMatr.getVectorScaled(data.getFun1Fun2Averaged(o -> new double []{1},o->TMatr.sumAcB(o.x,-1,xorig_average)).XfX,1/wtotal);
	    } else {
		// directly
		final double [] fxmxa=new double [n];
		for(ObservationVectorXF o:data.data)
		{
		    for(int c=0;c<n;c++){
			fxmxa[c]+=(o.x[c]-xorig_average[c])*o.f*o.weight;
		    }
		}
		this.fxmxorig_average=TMatr.getVectorScaled(fxmxa,1/wtotal);
	    }
	}
    }

/** Fields separator.
 */
public static final String SEPARATOR=",";
/** Read observations sequence from a file.
 * The lines starting with "|" are considered as comments.
 * If the first line starts with "|#", then it considered as attributes names.
 */
public static DataReadObservationVectorXF getObservations(final String filename,final int columns_number,final int [] indcol_x,final int indcol_f,final int indcol_weight,final int indcol_label)
	throws java.io.IOException
    {
	final java.util.ArrayList<ObservationVectorXF> data=
	    new java.util.ArrayList<>();
	final java.util.function.Function<String,String> postprocess=(str)->str.trim();
	System.err.print("Opening file="+filename);
	final java.io.BufferedReader br=
            new java.io.BufferedReader(new java.io.FileReader(filename));
        final java.io.StreamTokenizer tk=ReadWriteUtils.getTokenizer(SEPARATOR,br);
        tk.quoteChar('\"');

	if(indcol_f>=columns_number) throw new IllegalArgumentException("indcol_f="+indcol_f+" is invalid, must be >=0 and <"+columns_number);
	if(/*indcol_weight<0||*/indcol_weight>=columns_number) throw new IllegalArgumentException("indcol_weight="+indcol_weight+" is invalid, must <"+columns_number);
	final java.util.ArrayList<String> xHeader=new java.util.ArrayList<>(indcol_x.length);
	// default header
	for(int c=0;c<indcol_x.length;c++){
	    xHeader.add("x["+c+"]:ind="+indcol_x[c]);
	    if(indcol_x[c]<0||indcol_x[c]>=columns_number) throw new IllegalArgumentException("indcol_x["+c+"]="+indcol_x[c]+" is invalid, must be >=0 and <"+columns_number);
	}

	int n=0;
	for(;;)
	{
	    final String [] s=ReadWriteUtils.readOneLine(tk);
	    if(s==null) break; // end of file
	    // trim spaces
	    for(int c=0;c<s.length;c++) s[c]=postprocess.apply(s[c]);
	    n++;
	    if(n==1){
		// First line is possibly the header
		if(s.length>0 && s[0].startsWith("|#")){
		    if(s.length!=columns_number) throw new RuntimeException("File="+filename+" line="+n+" records number is not "+columns_number+" from string="+java.util.Arrays.toString(s));
		    s[0]=s[0].substring(2); // remove |#
		    xHeader.clear();
		    // replace by provided header
		    for(int c=0;c<indcol_x.length;c++){
			xHeader.add("x["+c+"]"+s[indcol_x[c]]+":ind="+indcol_x[c]);
		    }
		    continue;
		}
	    }
	    //System.err.println("n="+n+" s.length="+s.length+" s="+java.util.Arrays.toString(s));
	    if(s.length<=0||s[0].startsWith("|")) continue; // comments
	    if(s.length!=columns_number) throw new RuntimeException("File="+filename+" line="+n+" records number="+s.length+" is not "+columns_number+" from string="+java.util.Arrays.toString(s));
	    final double [] x=new double [indcol_x.length];
	    for(int c=0;c<indcol_x.length;c++){
		x[c]=Double.valueOf(s[indcol_x[c]]);
	    }
	    data.add(new ObservationVectorXF(indcol_label<0?null:s[indcol_label],
					     x,
					     indcol_f<0?Double.NaN:Double.valueOf(s[indcol_f]),
					     indcol_weight<0?1:Double.valueOf(s[indcol_weight])));
	}
	br.close();
	System.err.println(" read "+n+" lines , nrecords="+data.size());
	return new DataReadObservationVectorXF(data,xHeader);
    }

} // END OF class DataReadObservationVectorXF
