package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Eigenvectors problem.
 * This is an interface to <a href="http://www.netlib.org/lapack/lapacke.html">lapack</a>.
 * There are three version <ul>
 * <li> Java Native {@link com.polytechnik.lapack.Eigenvalues com.polytechnik.lapack.Eigenvalues}. Converted by {@code F2J} then manually adjusted.
 * <li> Java Native {@link com.polytechnik.lapack310.Eigenvalues com.polytechnik.lapack310.Eigenvalues}. Converted by {@code F2J} then manually adjusted.
 * <li> JNI version {@link com.polytechnik.lapack.Eigenvalues_JNI_lapacke  com.polytechnik.lapack.Eigenvalues_JNI_lapacke}
 * </ul>
 * This version calls the {@link com.polytechnik.lapack.Eigenvalues com.polytechnik.lapack.Eigenvalues}.
 * @version $Id: EVSolver.java,v 1.24 2022/11/09 16:07:54 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public interface  EVSolver {

    /** A method with default implementation; Edit and recomplie
     * if different eigenvalue solver is required.
     */
    static EVSolver getDefaultEVSolver(){
	//return new EVSolver_JNI_lapacke();
	//return new EVSolver_LapackOLD();
	return new EVSolver_Lapack310();
    }

/** Solve eigenvalues problem.
 */
double [] getEV(final int n,final double [] QQ,final boolean flag_ev);

/** Solve thridiagonal eigenvalues problem. */
double [] getEV_Thridiagonal(final int n,final double [] diag,final int diag_off,final double [] offdiag,final int offdiag_off,final boolean flag_ev);

/** Solve generalized eigenvalues problem.
 */
double [] getGEV_Generalized(final int n,final double [] A,final double [] B,final boolean flag_ev);


int getNonSymEV(final int n,final double [] matr,final double [] wr,final double [] wi);


    /** A result for SVD problem.
     */
    public static class SVDResult {
	public final int m,n;
	public final double [] U,VT,S;
	public final boolean flagFailure;
	public SVDResult(final int m,
			 final int n,
			 final double [] U,
			 final double [] S,
			 final double [] VT,
			 final boolean flagFailure){
	    this.m=m;
	    this.n=n;
	    this.U=U;
	    this.S=S;
	    this.VT=VT;
	    if(U.length!=m*m) throw new IllegalArgumentException();
	    if(VT.length!=n*n) throw new IllegalArgumentException();
	    if(S.length!=Math.min(m,n)) throw new IllegalArgumentException();
	    this.flagFailure=flagFailure;
	}
    }

    /** SVD decomposition.
     */
    SVDResult getSVD(int m,final int n,double [] matr);
}
