package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Eigenvectors problem.
 * Uses JNI calls to original lapack via
 * {@link com.polytechnik.lapack.Eigenvalues_JNI_lapacke  com.polytechnik.lapack.Eigenvalues_JNI_lapacke}.
 *  @version $Id: EVSolver_JNI_lapacke.java,v 1.10 2022/11/09 16:07:54 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class EVSolver_JNI_lapacke implements EVSolver {

@Override public double [] getEV(final int n,final double [] QQ,final boolean flag_ev)
{
    final int lda_QQ=n;
    if(QQ.length!=n*n) throw new IllegalArgumentException();
    return com.polytechnik.lapack.Eigenvalues_JNI_lapacke.getEV_dsyevx(n,QQ,lda_QQ,flag_ev);
}

@Override public double [] getEV_Thridiagonal(final int n,final double [] diag,final int diag_off,final double [] offdiag,final int offdiag_off,final boolean flag_ev)
{
    if(n+diag_off>diag.length||n+offdiag_off-1>offdiag.length) throw new IllegalArgumentException();
    return com.polytechnik.lapack.Eigenvalues_JNI_lapacke.getEV3_dstemr(n,diag,diag_off,offdiag,offdiag_off,flag_ev);
}

@Override public double [] getGEV_Generalized(final int n,final double [] A,final double [] B,final boolean flag_ev)
{
    final int lda_A=n,lda_B=n;
    if(A.length!=n*n||B.length!=A.length) throw new IllegalArgumentException();
    return com.polytechnik.lapack.Eigenvalues_JNI_lapacke.getGEV_dsygvx(n,A,lda_A,B,lda_B,flag_ev);
}

@Override public int getNonSymEV(final int n,final double [] matr,final double [] wr,final double [] wi)
{
    final int lda_matr=n;
    if(matr.length!=n*n) throw new IllegalArgumentException();
    //System.err.println("getNonSymEV called");
    return com.polytechnik.lapack.Eigenvalues_JNI_lapacke.getNonSymEV_dgees(n,matr,lda_matr,wr,wi);
}


@Override public SVDResult getSVD(int m,final int n,double [] matr){
    throw new RuntimeException("FIXME");
}

}
