package com.polytechnik.utils;
import com.polytechnik.lapack.Eigenvalues;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Eigenvectors problem.
 * This is an interface to <a href="http://www.netlib.org/lapack/lapacke.html">lapack</a>.
 * @version $Id: EVSolver_LapackOLD.java,v 1.24 2022/11/09 16:07:55 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
class EVSolver_LapackOLD implements EVSolver  {

private static final boolean USE_dsyevr=false;

@Override public double [] getEV(final int n,final double [] QQ,final boolean flag_ev)
{
    final int lda_QQ=n;
    if(QQ.length!=n*n) throw new IllegalArgumentException();
    if(USE_dsyevr)
    {
	return Eigenvalues.getEV_dsyevr(n,QQ,lda_QQ,flag_ev);
    }
    else
    {
	return Eigenvalues.getEV_dsyevx(n,QQ,lda_QQ,flag_ev);
    }
}

@Override public double [] getEV_Thridiagonal(final int n,final double [] diag,final int diag_off,final double [] offdiag,final int offdiag_off,final boolean flag_ev)
{
    if(n+diag_off>diag.length||n+offdiag_off-1>offdiag.length) throw new IllegalArgumentException();
    if(!flag_ev)
    {
	return Eigenvalues.getEV3_dstebz(n,diag,diag_off,offdiag,offdiag_off,flag_ev);
    }
    return Eigenvalues.getEV3_dstemr(n,diag,diag_off,offdiag,offdiag_off,flag_ev);
}

@Override public double [] getGEV_Generalized(final int n,final double [] A,final double [] B,final boolean flag_ev)
{
    final int lda_A=n,lda_B=n;
    if(A.length!=n*n||B.length!=A.length) throw new IllegalArgumentException();
    if(USE_dsyevr)
    {
	return Eigenvalues.getGEV_dsygvx_dsyevr(n,A,lda_A,B,lda_B,flag_ev);
    }
    else
    {
	return Eigenvalues.getGEV_dsygvx(n,A,lda_A,B,lda_B,flag_ev);
    }
    // the dsygvx may still be broken. June, 13, 2015

    // Our conversion is still some kind broken. Use original jlapack call below.
    //return com.polytechnik.algorithms.EigenvaluesSymmetric.getGEV(n,A,n,B,n,true);
    // something is fixed in Dec, 19, 2012. but still not sure.
}

@Override public int getNonSymEV(final int n,final double [] matr,final double [] wr,final double [] wi)
{
    final int lda_matr=n;
    if(matr.length!=n*n) throw new IllegalArgumentException();
    return Eigenvalues.getNonSymEV_dgees(n,matr,lda_matr,wr,wi);
}

@Override public SVDResult getSVD(int m,final int n,double [] matr){
    throw new RuntimeException("FIXME");
}


}
