package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/** An interface to data to calculate Christoffel function
 * and its moments out of sampled data.
 * A single interface method just replaces f by the Christoffel function.
 * @version $Id: FtoChristoffelFunctionTransformable.java,v 1.7 2020/10/17 08:32:58 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public interface FtoChristoffelFunctionTransformable {

    /** Create a model with sampled f is replaced by the Christoffel function value, the <b>x</b> are left intact.
     * Useful for unsupervised learning and clustering a vector--valued f.
     * @param orig_sample Original data the model is built from.
     * @return The {@link RadonNikodymSpectralModel RadonNikodymSpectralModel}
     * with the Christoffel function instead of f.
     */
    RadonNikodymSpectralModel getRadonNikodymSpectralModelWithChristoffelFunctionAsF(final DataReadObservationVectorXF orig_sample);

    /** Create a model with f-matrix assumed diagonal in the Christoffel function basis.
     * Useful for a vector--valued f.
     */
    RadonNikodymSpectralModel getRadonNikodymSpectralModelWithFassumedDiagonalInChristoffelFunctionBasis(final DataReadObservationVectorXF orig_sample);

}
