% $Id: LebesgueQuadratures_TestHelpers.m,v 1.26 2020/02/11 09:49:24 mal Exp $

%/**
% * (C) Vladislav Malyshkin 2018
% * This file is under GPL version 3.
% * @version $Id: LebesgueQuadratures_TestHelpers.m,v 1.26 2020/02/11 09:49:24 mal Exp $
% * @author Vladislav Malyshkin mal@gromco.com
% */

%HELPER CLASS for LebesgueQuadratureWithEVData.m USAGE DEMONSTRATION.
%
classdef LebesgueQuadratures_TestHelpers
  methods (Static = true)

% function example to obtain Lebesgue quadrature in octave, without "polynomials_abasis.jar" java library.
  function ocorr_fg=printLebesgueOctaveNatively(x,f,g,w,n,xrn)
  % octave bug http://savannah.gnu.org/bugs/?46659 workaround
  % basisfun=@LebesgueQuadratureWithEVData.basisfun_chebyshev
  basisfun=@(num,x) (LebesgueQuadratureWithEVData.basisfun_chebyshev(num,x));
  sumW=sum(w); % total weight
  % obtain matrices, e.g. <Qj|f|Qk> by sum(w(m)f(m)Qj(x(m))Qk(x(m))), m=1:Nsample ; j,k=0:n-1
  QQ = LebesgueQuadratureWithEVData.getMatrixBySampleSum (basisfun,w,x,n);
  QQf = LebesgueQuadratureWithEVData.getMatrixBySampleSum (basisfun,w .* f,x,n);
  QQg = LebesgueQuadratureWithEVData.getMatrixBySampleSum (basisfun,w .* g ,x,n);
  % calculate two Lebesgue quadratures (for f and g) from calculated matrices:
  qlf=LebesgueQuadratureWithEVData(QQf,QQ,basisfun);
  disp(["Octave:Lebesgue: F=" mat2str(qlf.f) " W=" mat2str(qlf.w) " sum=" mat2str([sum(qlf.w),sumW])]);
  qlg=LebesgueQuadratureWithEVData(QQg,QQ,basisfun);
  disp(["Octave:Lebesgue: G=" mat2str(qlg.f) " W=" mat2str(qlg.w) " sum=" mat2str([sum(qlg.w),sumW])]);

  % Value and Probability correlations
  ocorr_fg=LebesgueQuadratureJointDistribution(qlf,qlg);
  Val_fg=ocorr_fg.getValueCorrelation();
  Prob_fg=ocorr_fg.getProbabilityCorrelation();

  disp(["Octave:Value Correlation(f,g) sum=" mat2str([sum(sum(Val_fg)), sumW])]);
  Val_fg
  disp(["Octave:Probability Correlation(f,g) sum=" mat2str([sum(sum(Prob_fg)), n])]);
  Prob_fg

  % Print Radon-Nikodym and Least Squares f(x) estimation using getRNatX0 and getLSatX0 for a list of pre-set (in xrn array) values:
  for x0=xrn
    % RN(x0)=qlf.getRNatX0(x0); LS(x0)=qlf.getLSatX0(x0);
    disp(["Octave:from (EV): at x0=" mat2str(x0) " RN f=" mat2str(qlf.getRNatX0(x0)) " LS f=" mat2str(qlf.getLSatX0(x0))] );
  endfor
% instead of localized moments Qk(x0) obtained by getX0Moments(n,basisfun,x0) a Qk distribution can be used.
% Note that A_{RN}(x0) of https://arxiv.org/abs/1611.07386 is equal to:
% A_{RN}(x0)=qlf.getRNat(qlf.getX0Moments(x0)) where qlf is Lebesgue Quadrature built on f.
% a shortcut is qlf.getRNatX0(x0)
%
endfunction


% Generate (x,w,f,g) sample of Nsample size used in unit tests.
function [x,w,f,g,sumW,Nsample,xrn]=getSampleWeUseForTests(flag_resetrandom)
  % create sample
  Nsample=1000
  if(flag_resetrandom)
    rand ("state", [0;1;2;3] ); % to get reproducible random numbers
  endif
  % random x from [-1:1]
  x = 1-2*rand(1, Nsample);
  % a function on x to Lebesgue--integrate, an example with sin(x)
  f=sin(x);
  % random weights [0:1]
  w=rand(1,Nsample);
  % sum to match
  sumW=sum(w);
  % a test for (f,g) correlelation
  g=x .* x;

  % for RN & LS operator test
  xrn=[0,0.1,0.15,0.19];
endfunction

% Compare function interpolation using Radon-Nikodym & Least Squares
function [xorig,forig,fRN,fLS,ql]=getFunRNandLSinterpolated(n,fun,a,b,omegafun,nPointsIntegration,nPointsOutput)
  basisfun=@(num,x) (LebesgueQuadratureWithEVData.basisfun_chebyshev(num,x));
  x=zeros(nPointsIntegration,1);
  w=zeros(nPointsIntegration,1);
  f=zeros(nPointsIntegration,1);
  for k=1:nPointsIntegration
    % midle interval
    x(k)=-1+2*(k-0.5)/nPointsIntegration;
    % original x
    xarg=a+(b-a)*(x(k) - (-1))/2;
    % function value
    f(k)=fun(xarg);
    % w(k) omega(x) *dx , dx is the same=(b-a)/nPointsIntegration
    w(k)=omegafun(xarg)*(b-a)/nPointsIntegration;
  endfor
  QQ = LebesgueQuadratureWithEVData.getMatrixBySampleSum (basisfun,w,x,n);
  QQf = LebesgueQuadratureWithEVData.getMatrixBySampleSum (basisfun,w .* f,x,n);
  % Lebesgue quadrature
  ql=LebesgueQuadratureWithEVData(QQf,QQ,basisfun);
  % Make Radon Nikodyn and Least squares interpolation
  xorig=zeros(nPointsOutput,1);
  forig=zeros(nPointsOutput,1);
  fRN=zeros(nPointsOutput,1);
  fLS=zeros(nPointsOutput,1);
  for k=1:nPointsOutput
    xscaled=-1+2*(k-0.5)/nPointsOutput;
    xarg=a+(b-a)*(xscaled - (-1))/2;
    xorig(k)=xarg;
    forig(k)=fun(xarg);
    fRN(k)=ql.getRNatX0(xscaled);
    fLS(k)=ql.getLSatX0(xscaled);
  endfor
endfunction


% ========== A function to match java and octave implementations.
% jcorr_fg is java object, ocorr_fg is octave object
function testJavaAndOctaveJointDistributionsMatch(javaclassname,jcorr_fg,ocorr_fg,xrn,xxeps,flag_fail_on_error)
  % called from LebesgueQuadratures_TestHelpers.m
 disp(["Java class:" javaclassname " flag_fail_on_error=" mat2str(flag_fail_on_error) " The Value--correlation and Probability correlation for these x,f,g to match the ones produced by LebesgueQuadrature.m xxeps=",mat2str(xxeps)]);
 LebesgueQuadratureWithEVData.matchVectors(jcorr_fg.getValueCorrelation(),"jcorr_fg.getValueCorrelation()",ocorr_fg.getValueCorrelation()'(:),"ocorr_fg.getValueCorrelation()",xxeps,flag_fail_on_error);
 LebesgueQuadratureWithEVData.matchVectors(jcorr_fg.getProbabilityCorrelation(),"jcorr_fg.getProbabilityCorrelation()",ocorr_fg.getProbabilityCorrelation()'(:),"ocorr_fg.getProbabilityCorrelation()",xxeps,flag_fail_on_error);
 LebesgueQuadratureWithEVData.matchVectors(jcorr_fg.ff.f,"jcorr_fg.ff.f",ocorr_fg.ff.f,"ocorr_fg.ff.f",xxeps,flag_fail_on_error);
 LebesgueQuadratureWithEVData.matchVectors(jcorr_fg.ff.w,"jcorr_fg.ff.w",ocorr_fg.ff.w,"ocorr_fg.ff.w",xxeps,flag_fail_on_error);
 LebesgueQuadratureWithEVData.matchVectors(jcorr_fg.ff.getMatrixDiagonalsInPsiStates(jcorr_fg.ff.QQf),"<psi|QQf|psi>",jcorr_fg.ff.f,"jcorr_fg.ff.f",xxeps,flag_fail_on_error);
 LebesgueQuadratureWithEVData.matchVectors(ocorr_fg.ff.getMatrixDiagonalsInPsiStates(ocorr_fg.ff.QQf),"<psi|QQf|psi>",ocorr_fg.ff.f,"ocorr_fg.ff.f",xxeps,flag_fail_on_error);
 LebesgueQuadratureWithEVData.matchVectors(jcorr_fg.gg.f,"jcorr_fg.gg.f",ocorr_fg.gg.f,"ocorr_fg.gg.f",xxeps,flag_fail_on_error);
 LebesgueQuadratureWithEVData.matchVectors(jcorr_fg.gg.w,"jcorr_fg.gg.w",ocorr_fg.gg.w,"ocorr_fg.gg.w",xxeps,flag_fail_on_error);
 for qqk=1:2 % f,g
   if(qqk==1)
     oq=ocorr_fg.ff;
     jq=jcorr_fg.ff;
   else
      oq=ocorr_fg.gg;
      jq=jcorr_fg.gg;
   endif
   % try several x0
   for x0=xrn
     % psi to match within +-1 sign
     LebesgueQuadratureWithEVData.matchVectors(abs(jq.getPSIatX0(x0)),"jq.getPSIatX0",abs(oq.getPSIatX0(x0)),"oq.getPSIatX0(x0)",xxeps,flag_fail_on_error);
     % RN & LS to match
     LebesgueQuadratureWithEVData.matchVectors(jq.getEvaluationAtX0(x0).getRN(),"jq.getEvaluationAtX0(x0).getRN()",oq.getRNatX0(x0),"oq.getRNatX0(x0)",xxeps,flag_fail_on_error);
     LebesgueQuadratureWithEVData.matchVectors(jq.getEvaluationAtX0(x0).getLS(),"jq.getEvaluationAtX0(x0).getLS()",oq.getLSatX0(x0),"oq.getLSatX0(x0)",xxeps,flag_fail_on_error);
     LebesgueQuadratureWithEVData.matchVectors(jq.getEvaluationAtX0(x0).getChristoffelOatX(),"jq.getEvaluationAtX0(x0).getChristoffelOatX()",oq.getChristoffelOatX0(x0),"oq.getChristoffelOatX0(x0)",xxeps,flag_fail_on_error);
   endfor
 endfor % END f,g
 disp(["Test " javaclassname " is DONE xxeps=",mat2str(xxeps)]);
endfunction % testJavaAndOctaveJointDistributionsMatch
% ========== END A function to match java and octave implementations.


% ========== printChristoffelChart
function printChristoffelChart(ql,xmin,xstep,xmax,xselected)
% plot Christoffel function
  xa = xmin:xstep:xmax;
  ya=xa;
  for k=1:length(xa)
    ya(k)=1/ql.getEvaluationAtX0(xa(k)).getChristoffelOatX();
  endfor
  xvert=zeros(2,length(xselected));
  yvert=zeros(2,length(xselected));
  xselected_weights=zeros(length(xselected),1);
  for k=1:length(xselected)
    xvert(1,k)=xvert(2,k)=xselected(k);
    yvert(1,k)=0;yvert(2,k)=xselected_weights(k)=1/ql.getEvaluationAtX0(xselected(k)).getChristoffelOatX();
  endfor
  plot(xa, ya, "linewidth", 5,xvert,yvert,"linewidth",2,"color","g") ; title ("K(x)");
  disp(["xselected=" mat2str(xselected) " xselected_weights=" mat2str(xselected_weights) " sum=" mat2str(sum(xselected_weights))]);

endfunction
% ========== END printChristoffelChart

endmethods

endclassdef
