package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */

/** Two Roots finder.
 *  Calculates ln(kt/kv) , (ln(kt/kv))' , (ln(kt/kv))''.
 *  @version $Id: LogPolynomialsRatio.java,v 1.5 2019/10/15 15:27:49 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class LogPolynomialsRatio implements DerivativesCalculateable {
    final BasisFunctionsCalculatable B;
    final double [] kt,kv;
    public LogPolynomialsRatio(final double [] kv,final double [] kt,final BasisFunctionsCalculatable B)
    {
	this.B=B;
	this.kv=kv;
	this.kt=kt;
    }
    public double getF(final double x)
    {
	return Math.log(B.getD0(kt,x)/B.getD0(kv,x));
    }
    public double getDF(final double x)
    {
	return B.getD1(kt,x)/B.getD0(kt,x)-B.getD1(kv,x)/B.getD0(kv,x);
    }
    public double getD2F(final double x)
    {
	final double Ktx=B.getD0(kt,x),Kvx=B.getD0(kv,x),
	    DKtx=B.getD1(kt,x),DKvx=B.getD1(kv,x),
	    D2Ktx=B.getD2(kt,x),D2Kvx=B.getD2(kv,x);
	return (D2Ktx/Ktx-Math.pow(DKtx/Ktx,2))-(D2Kvx/Kvx-Math.pow(DKvx/Kvx,2));
    }
}
