package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */

/**
  * Find {@literal \( \left|\psi\right> \) } state such that
  * {@literal \[
  P\&L=\frac{\left<\psi|(p-\mathtt{pa})^2 I| \psi\right>}{\left<\psi|\psi\right> }
  \xrightarrow[\psi]{\quad }\max
  \] }
  * subject to:
{@literal \[
\left<\psi|(p-\mathtt{pa}) I|\psi\right>=0
\] }
  * The constant
{@literal \( \mathtt{pa}=\left<\psi|p I|\psi\right>/\left<\psi|I|\psi\right> \)}
  * is average price in {@literal \( \left|\psi\right> \)} state.
  * and the functional itself has the meaning of {@literal \( P\&L \)}.
  * Contrary to pure iteration type approach of {@link com.polytechnik.trading.DIminP2maxI}
  * this implementation uses Lagrange multipliers technique
  * to improve the convergence.
The functional {@literal \[
\left<\psi|(p-\mathtt{pa})^2 I| \psi\right>
+\mu\left<\psi|(p-\mathtt{pa}) I|\psi\right>
- \lambda\left[\left<\psi|\psi\right> -1\right]
\]}
is constructed. Variate it by {@literal \( \left|\psi\right> \) }, obtain the equation:
{@literal \[
\left|(p-\mathtt{pa})^2 I| \psi\right>
+\mu\left|(p-\mathtt{pa}) I|\psi\right>=
\lambda\left|\psi\right>
\]}
* An iteration process is:
* <ul>
<li> For an initial {@literal \( \left|\psi\right> \) }
calculate average price \( \mathtt{pa} \) and Lagrange multiplier \( \mu \):
{@literal
\begin{align}
\mathtt{pa}&=\frac{\left<\psi|p I|\psi\right>}{\left<\psi|I|\psi\right>}  \\
\left|b\right>&=\left|(p-\mathtt{pa}) I|\psi\right>
\\
\mu &= - \, \frac{\left<b|(p-\mathtt{pa})^2 I|\psi\right>}{\left<b|b\right>}
\end{align}
}
<li>Construct the matrix {@literal \[
\left|{\cal PL}\right|=
\left|(p-\mathtt{pa})^2 I\right|
+\mu\left|(p-\mathtt{pa}) I\right|
\]}
<li>Solve eigenvalue problem {@literal \[
\left|{\cal PL}|\psi^{[i]}\right>=
\lambda^{[i]} \left|\psi^{[i]}\right>
\] }
and take the state {@literal \( \left|\psi^{[i]}\right> \)}
corresponding an eigenvalue \( \lambda^{[i]} \)
as next iteration {@literal \( \left|\psi\right> \) }.
Often (but not always!) this is the state corresponding to
maximal eigenvalue. In general case see {@link EVSelection} that
selects the state of maximal {@literal \( P\&L \)}.
<li> Take the matrix
\( \left|{\cal PL}\right| \)
with updated \( \mathtt{pa} \) and  \( \mu \) and repeat the process of above
for this new {@literal \( \left|\psi\right> \) }.
If a solution exists -- iteration process
converges quickly (typically 5-7 iterations), unless price \( |p| \) and execution flow \( |I| \)
operators have several eigenvectors in common;
for example when \( \left|(p-\mathtt{pa})^2 I\right| \) and \( \left|(p-\mathtt{pa})I\right| \)
operators have the same eigenvectors the \( \left|{\cal PL}\right| \)
eigenvectors are always the same and iteration process fails.
On the first iteration
as {@literal \( \left|\psi\right> \) }
select an eigenvectors of the problem
{@literal \( \left|I|\psi\right>=\lambda\left|\psi\right> \) },
a few eigenvectors corresponding to large execution flow is a good try.
Typically the {@link StandardDeviationForPsi} is used to calculate the {@literal \( P\&L \) }
for all these eigenvectors and sorting them by it.
<li> The result of this iteration process is the state of maximal {@literal \( P\&L \) }, it is equal to the value of maximal eigenvalue of {@literal
\( \left|{\cal PL}|\psi^{[i]}\right>=
\lambda^{[i]} \left|\psi^{[i]}\right>
\)} problem. Position change \( dS \)
corresponding to it is: {@literal
\begin{align}
dS&=-(p-\mathtt{pa})\psi^2(x(t))\omega(t) dV \\
P\&L&= -\int p dS  = \left<\psi|(p-\mathtt{pa})^2I|\psi\right> = \lambda^{[\max]}\\
0&=\int dS = - \left<\psi|(p-\mathtt{pa})I|\psi\right>
\end{align}
}
Position change is a subject to \(0=\int dS\) constraint,
{@literal \( P\&L \) } is a covariance (with minus sign)
between price and postion change.
The {@literal \( P\&L \) } can be written in dual form by applying integration by parts
to express it not via postion change \( dS \) but via postion \( S\) itself:
{@literal
\begin{align}
S(t_{now})&=\int_{t=-\infty}^{t=t_{now}} dS \\
P\&L&= \int S dp \\
0&=S(-\infty)=S(t_{now})
\end{align}
}
Directional information can be obtained from considering \( S(t) \)
behavior, localized form of \( \psi^2(x(t)) \)
makes it often sufficient to consider
{@literal \( \left<S\right> \) }:
whether "on average" we have to be long or short.
In some \( x, \omega(x) \) bases {@literal \( \left<S\right> \) }
can be plain reduced to {@literal \(
\left<\psi|(p-\mathtt{pa})Ix|\psi\right>
\)} form or similar; in such a case normalized solution to market direction
can be e.g. a correlation
between  \( (p-\mathtt{pa})I \) and \( x \) in the found state
{@literal \( \left|\psi\right> \) },
see an example in unit test {@link com.polytechnik.trading.PnLInPsiHstateLegendreShifted.PnLInPsiHstateLegendreShiftedTest#test_Extra()}. In general, however, such a correlation-style approach is typically unsatisfactory.
</ul>
* This class takes four matrices {@literal \(
\left<Q_j Q_k\right>,
\left<Q_j I Q_k\right>,
\left<Q_j pI Q_k\right>,
\left<Q_j p^2I Q_k\right>
\)} as input, runs the algorithm of above
and obtains the state {@link MaximizeStandardDeviation#psi_PnLMax}
corresponding to maximal {@literal \( P\&L \)}.
  * @see IstatesConditionalLocalized
  * @see IstatesConditionalV2
  * @version $Id: MaximizeStandardDeviation.java,v 1.91 2022/07/19 08:36:32 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  */
public class MaximizeStandardDeviation {
    private static final int N_iterations=10;

    public final double
	p_IH,
	pA_PnLMax,
	PnLMax,I_PnLMax;

    /** In case of good convergence matches {@link #PnLMax} exactly.
     */
    public final double lambda_PnLMax;

    /** Lagrange multiplier (has a dimension of price change).
     */
    public final double mu_PnLMax;

    /** The state maximaizing P&amp;L functional.
     */
    public final double [] psi_PnLMax;
    /** The {@literal \( \left|I|\psi\right>=\lambda \left|\psi\right> \)} solution.
     */
    public final EVXData I;

    public MaximizeStandardDeviation(
	final int n,
	final double [] QQt,
	final double [] QQi,
	final double [] QQpi,
	final double [] QQp2i,
	final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
	final double eps)
{
    // solve generalized eignenvalue problem, find I->max state
    this.I=EVXData.getEVXDataFromQQ(n,QQi,QQt,M);
    p_IH=TMatr.getAMatrA(I.psiH,QQpi,n)/I.sH;

    // convert the matrices to |psik> basis to simplify algebra
    final double []
	bQQi=TMatr.getDiagMatrix(I.neq,I.ev),
	bQQpi=EVXData.convertMatrFromQtoB_Basis(I.neq,QQpi,I.ev),
	bQQp2i=EVXData.convertMatrFromQtoB_Basis(I.neq,QQp2i,I.ev);

    double pa_saved=0,Icur_saved=0,PnLcur_saved=0,mu_saved=0,lambda_PnLcur_saved=0,psi_saved[]=null;int indPsiIev_saved=-1,iTry_saved=-1;
    // temporary arrays
    final double [] mbQQppI=new double [n*n],mbQQpp2I=new double [n*n];
    // Try several initial values. As we have all |I|psi>=lambda|psi> eigenvectors, try them.
    // As we use the |I|psi>=lambda|psi> basis, the eigenvectors are:
    final java.util.List<double []> bpsiIev=java.util.stream.IntStream.range(-1,n)
	.mapToObj(k->{
		final var d=new double [n];
		// an eigenstate or all projections the same init.
		if(k<0){java.util.Arrays.fill(d,1/Math.sqrt(n));}else d[k]=1;
		return d;
	    }).toList();
    // single psiIH init
    //final java.util.List<StandardDeviationForPsi> psiInitSortedByPnL=java.util.List.of(StandardDeviationForPsi.getStatesSortedByPnL(n,bpsiIev,bQQi,bQQpi,bQQp2i,false).get(n-1));
    // various psi init
    final java.util.List<StandardDeviationForPsi> psiInitSortedByPnL=StandardDeviationForPsi.getStatesSelectedByPnL(n,bpsiIev,bQQi,bQQpi,bQQp2i);

    for(int iTry=0,nTry=psiInitSortedByPnL.size();iTry<nTry;iTry++){
	final StandardDeviationForPsi init=psiInitSortedByPnL.get(iTry);
	// initial psi
	double [] psi=init.psi;
	// iterations
	double pa=Double.NaN,mu=0,Icur=0,PnLcur=0,lambda_PnLcur=0;
	int ind_lambda_pnlCur=-1;
	for(int it=0;it<N_iterations;it++){
	    Icur=TMatr.getAMatrA(psi,bQQi,n);
	    // average price
	    pa=TMatr.getAMatrA(psi,bQQpi,n)/Icur;

	    // calculate Lagrange multiplier
	    for(int k=0;k<mbQQppI.length;k++) mbQQppI[k]=bQQpi[k]-pa*bQQi[k];
	    final double [] m=TMatr.getMatr_x_A(n,mbQQppI,psi);
	    for(int k=0;k<mbQQpp2I.length;k++) mbQQpp2I[k]=(bQQp2i[k]-pa*bQQpi[k])-pa*mbQQppI[k];
	    final double [] mC=TMatr.getMatr_x_A(n,mbQQpp2I,psi);
	    PnLcur=TMatr.mulVV(psi,mC); // <psi|mbQQpp2I|psi>
	    final double nr=TMatr.mulVV(m,m);
	    if(nr>0) mu= -TMatr.mulVV(m,mC)/nr; else mu=0;
	    if(!Double.isFinite(mu)) mu=0;

	    // save new matrix to mbQQpp2I=mbQQpp2I+mu*mbQQppI
	    TMatr.sumP1(mbQQpp2I,mbQQppI,mu);
	    if(eps>0){
		// initial values test
		UnitTests.checkDouble(TMatr.mulVV(psi,psi),"<psi|psi>",1,"1",eps);
		if(it==0) UnitTests.checkDoubleRel(init.PnL,"init.PnL",PnLcur,"PnLcur",eps);
		// calculated values test
		UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,mbQQppI,n),"<psi|(p-pa)I|psi>",0,"0",eps);
		UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,TMatr.sumAcBdE(bQQp2i,-2*pa,bQQpi,pa*pa,bQQi),n),"<psi|(p-pa)^2I|psi>",PnLcur,"PnLcur",eps);
		// may fail on initial ineration if mu is very large
		//UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,mbQQpp2I,n),"<psi|(p-pa)^2I|psi>",PnLcur,"PnLcur",eps);
		// possible non-monotonic behavior
		// convergence diag
		System.err.println("MaximizeStandardDeviation convergence: it="+it+" p_IH="+p_IH+" IH="+I.sH+" pa="+pa+" Icur="+Icur+" PnLcur="+PnLcur+" lambda_PnLcur="+lambda_PnLcur+" ind_lambda_pnlCur="+ind_lambda_pnlCur+" mu="+mu+" iTry="+iTry+" init.origIndex="+init.origIndex);
	    }
	    if(it>=N_iterations-1) break;

	    // select next iteration |psi>
	    final EVSelection s=new EVSelection(n,mbQQpp2I,bQQi,bQQpi,bQQp2i,M,eps);
	    psi=s.psi_pnlMax;
	    lambda_PnLcur=s.lambda_pnlMax;
	    ind_lambda_pnlCur=s.ind_lambda_pnlMax;

	    if(eps>0){
		System.err.println("MaximizeStandardDeviation:lambda selected:with: iTry="+iTry+" init.origIndex="+init.origIndex+" it="+it+" old PnLcur="+PnLcur+" s.pnlMax="+s.pnlMax+" s.lambda_pnlMax="+s.lambda_pnlMax+" s.ind_lambda_pnlMax="+s.ind_lambda_pnlMax+" lambdas="+java.util.Arrays.toString(java.util.Arrays.copyOf(s.ev_bQQPnLmatrix,n)));
	    }
	}
	if(eps>0){
	    // convergence diag
	     System.err.println("MaximizeStandardDeviation: iTry="+iTry+" result: with:ind_lambda_pnlCur="+ind_lambda_pnlCur+" init.origIndex="+init.origIndex+" p_IH="+p_IH+" IH="+I.sH+" pa="+pa+" Icur="+Icur+" PnLcur="+PnLcur+" lambda_PnLcur="+lambda_PnLcur+" changePnL="+(PnLcur-init.PnL)+" mu="+mu);
	}
	if(Double.isFinite(PnLcur) && (psi_saved==null || PnLcur>=PnLcur_saved)){
	    // save this try
	    pa_saved=pa;
	    Icur_saved=Icur;
	    PnLcur_saved=PnLcur;
	    mu_saved=mu;
	    lambda_PnLcur_saved=lambda_PnLcur;
	    psi_saved=psi;
	    indPsiIev_saved=init.origIndex;
	    iTry_saved=iTry;
	}
    }
    pA_PnLMax=pa_saved;
    PnLMax=PnLcur_saved;
    I_PnLMax=Icur_saved;
    lambda_PnLMax=lambda_PnLcur_saved;
    mu_PnLMax=mu_saved;
    // convert found |psi> back to regular basis.
    psi_PnLMax=EVXData.convertPsiFromBtoQ_Basis(n,psi_saved,I.ev);
    // test in original basis
    if(eps>0){
	System.err.println("MaximizeStandardDeviation result: with:indPsiIev_saved="+indPsiIev_saved+" iTry_saved="+iTry_saved+" PnLMax="+PnLMax+" lambda_PnLMax="+lambda_PnLMax+" pA_PnLMax="+pA_PnLMax+" I_PnLMax="+I_PnLMax+" mu_PnLMax="+mu_PnLMax+" n="+n);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi_PnLMax,QQt,n),"<psi_PnLMax|psi_PnLMax>",1,"1",eps);
	UnitTests.checkDouble(TMatr.getAMatrA(psi_PnLMax,QQi,n),"<psi_PnLMax|I|psi_PnLMax>",I_PnLMax,"I_PnLMax",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi_PnLMax,QQpi,n)/I_PnLMax,"pI/I",pA_PnLMax,"pA_PnLMax",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi_PnLMax,TMatr.sumAcBdE(QQp2i,-2*pA_PnLMax,QQpi,pA_PnLMax*pA_PnLMax,QQi),n),"PnL",PnLMax,"PnLMax",eps);
    }

}



/**
 * A helper class: calculate standard deviation for a
 * set of {@literal \( \left|\psi\right> \) } states.
 * @version $Id: MaximizeStandardDeviation.java,v 1.91 2022/07/19 08:36:32 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
private static class StandardDeviationForPsi {
    final double [] psi;
    final double PnL;
    final int origIndex;
    StandardDeviationForPsi(final double [] psi,final double PnL,final int origIndex){
	this.psi=psi;
	this.PnL=PnL;
	this.origIndex=origIndex;
    }
    /** Calculate {@literal \( P\&L \)} for all {@literal \( \left|\psi\right> \) },
     * sort the result by {@literal \( P\&L \)} (highest in the beginning).
     */
    static java.util.List<StandardDeviationForPsi> getStatesSortedByPnL(
	final int n,
	final java.util.List<double []> psiSeq,
	final double [] bQQi,
	final double [] bQQpi,
	final double [] bQQp2i){

	final java.util.ArrayList<StandardDeviationForPsi> r=new java.util.ArrayList<>();
	for(int k=0,sz=psiSeq.size();k<sz;k++){
	    final double [] psiS=psiSeq.get(k);
	    final double i=TMatr.getAMatrA(psiS,bQQi,n),
		pi=TMatr.getAMatrA(psiS,bQQpi,n),
		p2i=TMatr.getAMatrA(psiS,bQQp2i,n),
		paS=pi/i,pnlS=(p2i-paS*pi)-paS*(pi-paS*i);
	    if(Double.isFinite(pnlS)){
		r.add(new StandardDeviationForPsi(psiS,pnlS,k));
	    }
	}
	// sort the data, highest P&L at index 0:
	r.sort((p1,p2)->Double.compare​(p2.PnL,p1.PnL));
	return r;
    }
    /** Compared to the problem solved in {@link IstatesConditionalV2}
     * the problem in question
     * is much more likely to stick into local maximum, take top 10 init function.
     */
    static java.util.List<StandardDeviationForPsi> getStatesSelectedByPnL(
	final int n,
	final java.util.List<double []> psiSeq,
	final double [] bQQi,
	final double [] bQQpi,
	final double [] bQQp2i){

	//final int nInitApprox=psiSeq.size(); // or even all available?
	final int nInitApprox=10;

	final var inSorted=getStatesSortedByPnL(n,psiSeq,bQQi,bQQpi,bQQp2i);

	// as input init functions are typically orthogonal -- this is equivalent to taking top nInitApprox.
	// for now
	return IstatesConditionalV2.selectSomeNotVerySimilarInitFunctions(n,nInitApprox,inSorted,x->x.psi);
    }
}


/**
 * As the state corresponding to maximal {@literal \( P\&L \)}
 * is not necessary the maximal eigenvalue state due to average price offset --
 * check all the eigenstates. A helper class.
 * @version $Id: MaximizeStandardDeviation.java,v 1.91 2022/07/19 08:36:32 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
private static class EVSelection {
    final double [] ev_bQQPnLmatrix;
    final double pnlMax,lambda_pnlMax,pa,I_pnlMax;
    final int ind_lambda_pnlMax;
    final double [] psi_pnlMax;
    EVSelection(final int n,
		final double [] bQQPnLmatrix,
		final double [] bQQi,
		final double [] bQQpi,
		final double [] bQQp2i,
		final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,
		final double eps){
	// solve eigenvalue problem
	ev_bQQPnLmatrix=M.EV.getEV(n,bQQPnLmatrix,true);

	// iteration: update the |psi> state
	// select the eigenvector to continue with
	// continue with selected psi.
	// Select the one corresponding to maximal P&L, this may not be the maximal lambda!!!
	double pnlSmax=0,iSmax=0,paSmax=0,lambdaSmax=0,psi[]=null;
	int indLam=-1;
	for(int s=0;s<n;s++){
	    final double [] psiS=EVXData.getPsiFromBasis(s,ev_bQQPnLmatrix,n);
	    final double i=TMatr.getAMatrA(psiS,bQQi,n),
		pi=TMatr.getAMatrA(psiS,bQQpi,n),
		p2i=TMatr.getAMatrA(psiS,bQQp2i,n),
		paS=pi/i,pnlS=(p2i-paS*pi)-paS*(pi-paS*i);
	    if(Double.isFinite(pnlS) && (indLam<0 || pnlS>=pnlSmax)){
		indLam=s;
		psi=psiS;
		pnlSmax=pnlS;
		iSmax=i;
		paSmax=paS;
		lambdaSmax=ev_bQQPnLmatrix[s];
	    }
	    if(eps>0){
		// unit test
		UnitTests.checkDoubleRel(TMatr.getAMatrA(psiS,TMatr.sumAcB(bQQpi,-paS,bQQi),n),"<psiS|(p-paS)I|psiS>",0,"0",eps);
		UnitTests.checkDoubleRel(TMatr.getAMatrA(psiS,TMatr.sumAcBdE(bQQp2i,-2*paS,bQQpi,paS*paS,bQQi),n),"<psiS|(p-paS)^2I|psiS>",pnlS,"pnlS",eps);
	    }
	}
	psi_pnlMax=psi;
	pnlMax=pnlSmax;
	lambda_pnlMax=lambdaSmax;
	ind_lambda_pnlMax=indLam;
	I_pnlMax=iSmax;
	pa=paSmax;
	if(eps>0){
	    System.err.println("EVSelection:lambda selection: pnlMax="+pnlMax+" lambda_pnlMax="+lambda_pnlMax+" pa="+pa+" lambdas="+java.util.Arrays.toString(java.util.Arrays.copyOf(ev_bQQPnLmatrix,n)));
	}
    }

}



static void printDiag(MaximizeStandardDeviation m){
    // print some diag
    System.err.println("MaximizeStandardDeviation.main:I EV="+java.util.Arrays.toString(m.I.getEValues()));
    System.err.println("MaximizeStandardDeviation.main:p_IH="+m.p_IH+" pA_PnLMax="+m.pA_PnLMax+" I_PnLMax="+m.I_PnLMax+" PnLMax="+m.PnLMax+" lambda_PnLMax="+m.lambda_PnLMax+" psi_PnLMax="+java.util.Arrays.toString(m.psi_PnLMax)+" n="+m.I.neq);
}


/** Unit test. */
public static void main(String [] args){
    TestOptimizationPsi.runMultiTest("MaximizeStandardDeviation",15,600,
		 new java.util.Random(18301L),
		 1e-5,
		 testC-> {final var m=new MaximizeStandardDeviation(
		                testC.n,
				testC.QQt,
				testC.QQi,
				testC.QQpi,
				testC.QQp2i,
				testC.M,
				testC.eps);
			  printDiag(m);
			 }
    );
}

}
