package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Manupulate orthogonal polynomials using Chebyshev polynomials as basis.
 *  @version $Id: OrthogonalPolynomialsChebyshevBasis.java,v 1.23 2022/08/02 07:06:04 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class OrthogonalPolynomialsChebyshevBasis extends OrthogonalPolynomialsBasisFunctionsCalculatable<Chebyshev> {

public OrthogonalPolynomialsChebyshevBasis(){super(new Chebyshev());}

/** Multiply by (1-x).
 */
public double [] getRadauMultiplied(final double [] nodes0)
{
    return B.getaxbP(nodes0,-1,1);
}


static class TestChebyshevInMonomials extends TestOrthogonalPolynomialsBasisFunctionsCalculatable {
    TestChebyshevInMonomials()
    {
	super("BasisChebyshev",
	      new OrthogonalPolynomialsChebyshevBasis(),
	      1e-6,
	      new java.util.Random(18302L),
	      -1,1);
    }
@Override void test_RadauMoments_directly_Bbasis(final double [] bnodes,final double [] rbnodes)
	{
	    final double [] res=O.B.getXQkMomentsFromQkMoments(bnodes);
	    for(int k=res.length;--k>=0;) {res[k]=bnodes[k]-res[k];}
	    UnitTests.checkDoubleArrayRel(res,"res",rbnodes,"rbnodes",eps);
	    System.err.println(name+":test_RadauMoments_directly_Bbasis test OK n="+bnodes.length+" eps="+eps);
	}
} // END OF class TestChebyshevInMonomials

/** A unit test.
 */
public static void main(String [] args)
{
    new TestChebyshevInMonomials().runTests(7);
}

}
