package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Manupulate orthogonal polynomials using Hermite polynomials as basis.
 *  @version $Id: OrthogonalPolynomialsHermiteEBasis.java,v 1.27 2021/11/27 00:50:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class OrthogonalPolynomialsHermiteEBasis extends OrthogonalPolynomialsBasisFunctionsCalculatable<HermiteE> {

public OrthogonalPolynomialsHermiteEBasis(){super(new HermiteE());}

/** Multiply by (1-x).
 * Radau is not applicable to Hermite (-inf, inf) interval.
 */
public double [] getRadauMultiplied(final double [] nodes0)
{
    return B.getaxbP(nodes0,-1,1);
}


private static class TestHermiteEInMonomials extends TestOrthogonalPolynomialsBasisFunctionsCalculatable {
    TestHermiteEInMonomials()
    {
	super("BasisHermiteE",
	      new OrthogonalPolynomialsHermiteEBasis(),
	      1e-5,
	      new java.util.Random(131265L),
	      -1,1);
    }
    @Override void test_RadauMoments_directly_Bbasis(final double [] bnodes,final double [] rbnodes)
    {
	final double [] res=O.B.getXQkMomentsFromQkMoments(bnodes);
	for(int k=res.length;--k>=0;) {res[k]=bnodes[k]-res[k];}
	UnitTests.checkDoubleArrayRel(res,"res",rbnodes,"rbnodes",eps);
	System.err.println(name+":test_RadauMoments_directly_Bbasis test OK n="+bnodes.length+" eps="+eps);
    }
} // END OF class TestHermiteEInMonomials

/** A unit test.
 */
public static void main(String [] args)
{
    new TestHermiteEInMonomials().runTests(4);
}

}
