package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Manupulate orthogonal polynomials using Laguerre polynomials as basis.
 *  @version $Id: OrthogonalPolynomialsLaguerreBasis.java,v 1.47 2020/09/27 20:14:06 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class OrthogonalPolynomialsLaguerreBasis extends OrthogonalPolynomialsBasisFunctionsCalculatable<Laguerre> {
public OrthogonalPolynomialsLaguerreBasis(){super(new Laguerre());}

/** To match sign to monomials basis. */
public double getSignToMatch(final int k){return (k%2==0)?1:-1;}

public double [] getRadauMultiplied(final double [] nodes0)
{
    // multiply by x
    return B.getaxbP(nodes0,1,0);
}


/** A unit test.
 */
public static void main(String [] args)
{
    new TestOrthogonalPolynomialsBasisFunctionsCalculatable("BasisLaguerre",
		      new OrthogonalPolynomialsLaguerreBasis(),
		      1e-5,
		      new java.util.Random(115L),
		      0,11)
    {
	@Override void test_RadauMoments_directly_Bbasis(final double [] bnodes,final double [] rbnodes)
	{
	    final double [] res=O.B.getXQkMomentsFromQkMoments(bnodes);
	    UnitTests.checkDoubleArrayRel(res,"res",rbnodes,"rbnodes",eps);
	    System.err.println(name+":test_RadauMoments_directly_Bbasis test OK n="+bnodes.length+" eps="+eps);
	}
    }.runTests(6);
}

}
