package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Manupulate orthogonal polynomials using Legendre polynomials as basis.
 *  @version $Id: OrthogonalPolynomialsLegendreBasis.java,v 1.19 2020/09/27 20:14:06 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class OrthogonalPolynomialsLegendreBasis extends OrthogonalPolynomialsBasisFunctionsCalculatable<Legendre> {

public OrthogonalPolynomialsLegendreBasis(){super(new Legendre());}

/** Multiply by (1-x).
 */
public double [] getRadauMultiplied(final double [] nodes0)
{
    // multiply by 1-x
    return B.getaxbP(nodes0,-1,1);
}


/** A unit test.
 */
public static void main(String [] args)
{
    new TestOrthogonalPolynomialsBasisFunctionsCalculatable("BasisLegendre",
		      new OrthogonalPolynomialsLegendreBasis(),
		      1e-6,
		      new java.util.Random(18831L),
		      -1,1)
    {
	@Override void test_RadauMoments_directly_Bbasis(final double [] bnodes,final double [] rbnodes)
	{
	    final double [] res=O.B.getXQkMomentsFromQkMoments(bnodes);
	    for(int k=res.length;--k>=0;) {res[k]=bnodes[k]-res[k];}
	    UnitTests.checkDoubleArrayRel(res,"res",rbnodes,"rbnodes",eps);
	    System.err.println(name+":test_RadauMoments_directly_Bbasis test OK n="+bnodes.length+" eps="+eps);
	}
    }.runTests(7);
}

}
