package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/** Construct orthogonal polynomials (typically from sampled data) in
 * the basis determined by given three term recurrence coefficients.
 * Note: the number of recurrence coefficients for basis should be large enough
 * to accomodate quadrature/orthogonal polynomials/recurrence coefficients/etc.
 * constructed from sampled data. The number of basis functions required is
 * equal to the number of moments required.
 *
 * @version $Id: OrthogonalPolynomialsRecurrenceABBasis.java,v 1.24 2022/07/10 23:08:38 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class OrthogonalPolynomialsRecurrenceABBasis extends OrthogonalPolynomialsBasisFunctionsCalculatable<RecurrenceAB> {
    /**
     * @param B The basis in which to construct orthogonal polynomials. We construct some other orthogonal polynomials in the basis {@code B}.
     */
    public OrthogonalPolynomialsRecurrenceABBasis(final RecurrenceAB B){super(B);}



static TestOrthogonalPolynomialsBasisFunctionsCalculatable getTestOrthogonalPolynomialsBasisFunctionsCalculatable(){
    final double eps=1e-5;

    // create random sample moments
    final OrthogonalPolynomialsChebyshevBasis.TestChebyshevInMonomials TM=new OrthogonalPolynomialsChebyshevBasis.TestChebyshevInMonomials();

    final int neq_max=50/* For Kronrod & multiple orthogonality tests we need a large number of basis functions. */,
	n_sample=100000;

    // extract ab
    final RecurrenceAB ab=TM.getRecurrenceABFromBasisRandomMoments(neq_max,n_sample);
    System.err.println("AB:Max polynomial order="+neq_max+" ab.numAk()="+ab.numAk()+" the number should be at least 3*neq for Kronrod");
    OrthogonalPolynomialsBasisFunctionsCalculatable.printAB("Working basis: it's three term recurrence coefficients:",ab);

    // create basis
    final OrthogonalPolynomialsRecurrenceABBasis
	 basis=new OrthogonalPolynomialsRecurrenceABBasis(ab){
		 /** Multiply by (1-xmax).
		  */
		 public double [] getRadauMultiplied(final double [] nodes0)
		 {
		     return B.getaxbP(nodes0,-1,TM.x_max);
		 }};
    // regular test
    return new TestOrthogonalPolynomialsBasisFunctionsCalculatable("BasisRecurrenceAB",
			     basis,
			     eps,
			     TM.r,
			     TM.x_min,TM.x_max);
}


/** Unit test orthogonal polynomials in a general 3 term recurrence basis.
 */
public static void main(String [] args)
{
    getTestOrthogonalPolynomialsBasisFunctionsCalculatable().runTests(7);
}

}
