package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */

/**
 *  Calculates f/kt , (f/kt)' , (f/kt)''.
 *  @version $Id: PolynomialsRatio.java,v 1.6 2019/10/15 15:27:50 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PolynomialsRatio implements DerivativesCalculateable {
    final BasisFunctionsCalculatable B;
    final double [] f,kt;
    public PolynomialsRatio(final double [] kt,final double [] f,final BasisFunctionsCalculatable B)
    {
	this.B=B;
	this.kt=kt;
	this.f=f;
    }
    public double getF(final double x)
    {
	return (B.getD0(f,x)/B.getD0(kt,x));
    }
    public double getDF(final double x)
    {
	final double Ktx=B.getD0(kt,x);
	return (B.getD1(f,x)/Ktx-(B.getD0(f,x)/Ktx)*(B.getD1(kt,x)/Ktx));
    }
    public double getD2F(final double x)
    {
	final double Ktx=B.getD0(kt,x),Kfx=B.getD0(f,x),
	    DKtx=B.getD1(kt,x),DKfx=B.getD1(f,x),
	    D2Ktx=B.getD2(kt,x),D2Kfx=B.getD2(f,x),rt=DKtx/Ktx;
	final double t=(D2Kfx/Ktx)-2*(DKfx/Ktx)*rt-(Kfx/Ktx)*(D2Ktx/Ktx-2*rt*rt);
	return t;
    }
}
