package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */

/**
 *  Calculates sqrt(SNOM)/DENOM , (sqrt(SNOM)/DENOM)' , (sqrt(SNOM)/DENOM)''.
 *  @version $Id: PolynomialsSQDRatio.java,v 1.5 2019/10/15 15:27:50 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PolynomialsSQDRatio implements DerivativesCalculateable {
    final BasisFunctionsCalculatable B;
    final double [] SNOM,DENOM;
    public PolynomialsSQDRatio(final double [] DENOM,final double [] SNOM,final BasisFunctionsCalculatable B)
    {
	this.B=B;
	this.DENOM=DENOM;
	this.SNOM=SNOM;
    }
    public double getF(final double x)
    {
	return Math.sqrt(B.getD0(SNOM,x))/B.getD0(DENOM,x);
    }
    public double getDF(final double x)
    {
	final double a=B.getD0(SNOM,x),
	    sqa=Math.sqrt(a),
	    da=B.getD1(SNOM,x),
	    b=B.getD0(DENOM,x),
	    db=B.getD1(DENOM,x),
	    db_divb=db/b,
	    F=(sqa/b);
	return (0.5*da/sqa)/b-F*db_divb;
    }
    public double getD2F(final double x)
    {
	final double a=B.getD0(SNOM,x),
	    sqa=Math.sqrt(a),
	    da=B.getD1(SNOM,x),
	    d2a=B.getD2(SNOM,x),
	    b=B.getD0(DENOM,x),
	    db=B.getD1(DENOM,x),
	    d2b=B.getD2(DENOM,x),
	    db_divb=db/b,
	    da_diva=da/a,
	    F=(sqa/b),
	    DF=(0.5*da/sqa)/b-F*db_divb;
	return DF*(0.5*da_diva-db_divb)
	    +F*(0.5*(d2a/a-Math.pow(da_diva,2))-(d2b/b-Math.pow(db_divb,2)));
    }
}
