package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */

/**
 *  Calculates NOM/sqrt(SDENOM) , (NOM/sqrt(SDENOM))' , (NOM/sqrt(SDENOM))''.
 *  @version $Id: PolynomialsSQRatio.java,v 1.7 2019/10/15 15:27:50 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PolynomialsSQRatio implements DerivativesCalculateable {
    final BasisFunctionsCalculatable B;
    final double [] SDENOM,NOM;
    public PolynomialsSQRatio(final double [] SDENOM,final double [] NOM,final BasisFunctionsCalculatable B)
    {
	this.B=B;
	this.SDENOM=SDENOM;
	this.NOM=NOM;
    }
    public double getF(final double x)
    {
	return (B.getD0(NOM,x)/Math.sqrt(B.getD0(SDENOM,x)));
    }
    public double getDF(final double x)
    {
	final double a=B.getD0(NOM,x),
	    da=B.getD1(NOM,x),
	    b=B.getD0(SDENOM,x),
	    sqb=Math.sqrt(b),
	    db=B.getD1(SDENOM,x),
	    db_divb=db/b,
	    F=(a/sqb);
	return da/sqb-0.5*F*db_divb;
    }
    public double getD2F(final double x)
    {
	final double a=B.getD0(NOM,x),
	    da=B.getD1(NOM,x),
	    d2a=B.getD2(NOM,x),
	    b=B.getD0(SDENOM,x),
	    sqb=Math.sqrt(b),
	    db=B.getD1(SDENOM,x),
	    d2b=B.getD2(SDENOM,x),
	    db_divb=db/b,
	    F=(a/sqb),
	    DF=da/sqb-0.5*F*db_divb;
	return d2a/sqb-(2*0.5)*db_divb*DF
	    -0.5*F*(d2b/b-0.5*Math.pow(db_divb,2));
    }
}
