package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2018
 * This file is under GPL version 3.
 *
 */

/** Store Gauss quadrature.
 *  @version $Id: Quadrature.java,v 1.18 2019/09/26 07:27:11 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class Quadrature {
    public final double [] x,w;
    public Quadrature(final double [] x,final double [] w){
	this.x=x;this.w=w;
	if(x.length!=w.length) throw new IllegalArgumentException();
    }

/** Gaussian quadrature with nodes polynomail availavle.
 */
public static class GaussQuadratureWithNodesPolynomialData extends Quadrature {
    public final double [] nodespolynomial;
    public GaussQuadratureWithNodesPolynomialData(final double [] x,final double [] w,final double [] nodespolynomial)
    {
	super(x,w);
	this.nodespolynomial=nodespolynomial;
    }
} // END OF public static class GaussQuadratureWithNodesPolynomialData

    public static class GaussQuadratureWithEVData extends Quadrature {
	public final double [] ev,QQx,QQ;
	public GaussQuadratureWithEVData(final double [] x,final double [] w,final double [] ev,final double [] QQx,final double [] QQ)
	{
	    super(x,w);
	    this.ev=ev;
	    this.QQx=QQx;
	    this.QQ=QQ;
	}
    }

/** Kronrod quadrature. Two kind of nodes and weights.
 */
public static class KronrodQuadrature {
    public final double [] xg,wg,imxg,xk,wk,imxk;
    public KronrodQuadrature(final double [] xg,final double [] wg,final double [] imxg,final double [] xk,final double [] wk,final double [] imxk)
	{
	    this.xg=xg;
	    this.wg=wg;
	    this.imxg=imxg;
	    this.xk=xk;
	    this.wk=wk;
	    this.imxk=imxk;
	    if(xg.length!=wg.length||xg.length!=imxg.length) throw new IllegalArgumentException();
	    if(xk.length!=wk.length||xk.length!=imxk.length) throw new IllegalArgumentException();
	}
    /** Combine two kinds of nodes and weights
     * to a single quadrature.
     */
    public Quadrature getGKCombined(){
	final double [] x=new double [xg.length+xk.length],w=new double [x.length];
	System.arraycopy(xg,0,x,0,xg.length);
	System.arraycopy(xk,0,x,xg.length,xk.length);
	System.arraycopy(wg,0,w,0,wg.length);
	System.arraycopy(wk,0,w,wg.length,wk.length);
	return new Quadrature(x,w);
    }
} // END OF public static class KronrodQuadrature


/** Multiple Orthogonality quadtarures.
 * Nodes, weights, and nodes polynomials.
 */
public static class MultipleOrthogonalityQuadrature {
    public final double [] x,imx,nodespolynomial;
    public final double [][] w;
    public MultipleOrthogonalityQuadrature(final double [] x,final double [] imx,
					   final double [] nodespolynomial,final double [][] w)
    {
	this.x=x;
	this.imx=imx;
	this.nodespolynomial=nodespolynomial;
	this.w=w;
    }
} // END OF public static class MultipleOrthogonalityQuadrature

} // END OF public class Quadrature
