package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/** Command line options for {@link RN RN},
 * a class to read config.
 * @version $Id: RNConfig.java,v 1.15 2024/04/05 17:10:31 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class RNConfig {
    public final int indcol_x_start,indcol_x_end,indcol_f,indcol_weight,indcol_label,columns_number,flag_print_verbosity;
    public final boolean flag_replace_f_by_christoffel_function,flag_assume_f_is_diagonal_in_christoffel_function_basis;
    public final int clusters_number,max_multiindex;
    public final String output_files_prefix,data_file_to_build_model_from,data_file_evaluation[];
    public final RegularizationMethod regularization_method;
    private final String regularization_method_Str;

    protected final java.util.List<String> opts_remained_unprocessed=new java.util.ArrayList<String>();
    public RNConfig(final String [] args)
    {
	int indcol_x_start=-1,indcol_x_end=-1,indcol_f=-1,indcol_weight=-1,indcol_label=-1,columns_number=-1,max_multiindex=-1;
	String data_file_to_build_model_from=null,output_files_prefix="";
	String regularization_method_Str=RegularizationMethod.EV.name();
	int flag_print_verbosity=2; // print psik by default
	boolean flag_replace_f_by_christoffel_function=false,
	    flag_assume_f_is_diagonal_in_christoffel_function_basis=false;
	int clusters_number=-1;
	final java.util.ArrayList<String> data_file_evaluation=new java.util.ArrayList<>();
	for(String a:args)
	{
	    String v=null;
	    if(null!=(v=getVal(a,"--data_file_to_build_model_from="))) data_file_to_build_model_from=v;
	    else if(null!=(v=getVal(a,"--data_file_evaluation="))) data_file_evaluation.add(v);
	    else if(null!=(v=getVal(a,"--output_files_prefix="))) output_files_prefix=v;
	    else if(null!=(v=getVal(a,"--flag_print_verbosity="))) flag_print_verbosity=Integer.valueOf(v);
	    else if(null!=(v=getVal(a,"--flag_replace_f_by_christoffel_function="))) flag_replace_f_by_christoffel_function=Boolean.valueOf(v);
	    else if(null!=(v=getVal(a,"--flag_assume_f_is_diagonal_in_christoffel_function_basis="))) flag_assume_f_is_diagonal_in_christoffel_function_basis=Boolean.valueOf(v);
	    else if(null!=(v=getVal(a,"--clusters_number="))) clusters_number=Integer.valueOf(v);
	    else if(null!=(v=getVal(a,"--max_multiindex="))) max_multiindex=Integer.valueOf(v);
	    else if(null!=(v=getVal(a,"--regularization_method="))) regularization_method_Str=v;
	    else if(null!=(v=getVal(a,"--data_cols="))){
		final String [] k=com.polytechnik.utils.ReadWriteUtils.extractnames(v,":");
		if(k.length!=5) throw new RuntimeException("(numcols:xstart,xend:f:w:label) colums index="+v+" is invalid. The format is like 15:0,3:4:5:0");
		columns_number=Integer.valueOf(k[0]);
		final String [] kx=com.polytechnik.utils.ReadWriteUtils.extractnames(k[1],",");
		indcol_f=Integer.valueOf(k[2]);
		indcol_weight=Integer.valueOf(k[3]);
		indcol_label=Integer.valueOf(k[4]);
		if(kx.length!=2) throw new RuntimeException("(numcols:xstart,xend:f:w:label) colums index="+v+" is invalid. The format is like 15:0,3:4:5:0");
		indcol_x_start=Integer.valueOf(kx[0]);
		indcol_x_end=Integer.valueOf(kx[1]);
	    }
	    else{
		opts_remained_unprocessed.add(a);
	    }
	}
	this.columns_number=columns_number;
	this.max_multiindex=max_multiindex;
	this.indcol_x_start=indcol_x_start;
	this.indcol_x_end=indcol_x_end;
	this.indcol_f=indcol_f;
	this.indcol_weight=indcol_weight;
	this.indcol_label=indcol_label;
	this.data_file_to_build_model_from=data_file_to_build_model_from;
	this.data_file_evaluation=data_file_evaluation.toArray(String[]::new);
	this.output_files_prefix=output_files_prefix;
	this.flag_print_verbosity=flag_print_verbosity;
	this.flag_replace_f_by_christoffel_function=flag_replace_f_by_christoffel_function;
	this.flag_assume_f_is_diagonal_in_christoffel_function_basis=flag_assume_f_is_diagonal_in_christoffel_function_basis;
	this.clusters_number=clusters_number;
	this.regularization_method_Str=regularization_method_Str;
	this.regularization_method=RegularizationMethod.opt(regularization_method_Str);
    }
    public java.util.List<String> validateOptions()
    {
	final java.util.List<String> errs=new java.util.ArrayList<String>();
	if(data_file_to_build_model_from==null) errs.add("no --data_file_to_build_model_from= specified");
	if(columns_number<=0||
	   indcol_x_start<0||indcol_x_end<0||
	   indcol_x_start>indcol_x_end||indcol_x_end>=columns_number||
	   indcol_f<0||indcol_f>=columns_number||
	   indcol_weight>=columns_number ||
	   indcol_label>=columns_number ) errs.add("no --data_cols=numcols:xstart,xend:f:w:label columns numbers specified for the file --data_file_to_build_model_from=");
	if(regularization_method==null)
	    errs.add("Unknown regularization method "+regularization_method_Str);
	if(opts_remained_unprocessed.size()>0)
	    errs.add("There are unknown options found:"+opts_remained_unprocessed.toString());
	return errs;
    }
    public int [] getXIndexesSequence(){
	final int [] inX=new int [indcol_x_end-indcol_x_start+1];
	for(int k=indcol_x_start,ind=0;k<=indcol_x_end;k++){
	    inX[ind++]=k;
	}
	return inX;
    }
    public String getDescription()
    {
	return "#BEGIN{RNConfig}\n"+
	    "data_file_to_build_model_from="+data_file_to_build_model_from+" # input file name\n"+
	    "data_file_evaluation="+java.util.Arrays.toString(data_file_evaluation)+" # data file(s) to evaluate the model\n"+
	    "output_files_prefix="+output_files_prefix+"\n"+
	    "data_cols="+columns_number+":"+indcol_x_start+","+indcol_x_end+":"+indcol_f+":"+indcol_weight+":"+indcol_label+" # (numcols:xstart,xend:f:w:label) the file="+data_file_to_build_model_from+" columns index (base 0)\n"+
	    "flag_print_verbosity="+flag_print_verbosity+" # (2 print psik, 3 print xorigtopsik projections in output)\n"+
	    "flag_replace_f_by_christoffel_function="+flag_replace_f_by_christoffel_function+" # (replace f by the Christoffel function)\n"+
	    "flag_assume_f_is_diagonal_in_christoffel_function_basis="+flag_assume_f_is_diagonal_in_christoffel_function_basis+" # (take f in the Christoffel function basis as diagonal)\n"+
	    "clusters_number="+clusters_number+" # if >0, select this number of best superpositions of attributes\n"+
	    "regularization_method="+regularization_method+" # available "+java.util.Arrays.toString(RegularizationMethod.values())+"\n"+
	    "max_multiindex="+max_multiindex+" # if set then a polynomial of this multi-index is constructed\n"+
	    "#END{RNConfig}\n";
    }


    public static String getVal(final String a,final String key)
    {
	if(a.startsWith(key))
	{
	    return a.substring(key.length());
	}
	return null;
    }

} // END OF class RNConfig
