package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/** The same as {@link RecurrenceAB RecurrenceAB}, but the multiplication {@link BasisFunctionsMultipliable#setQlQmExpansionCoefs setQlQmExpansionCoefs} is
 * cached using {@link BasisFunctionsMultiplicationCache BasisFunctionsMultiplicationCache}.
 *
 * @version $Id: RecurrenceABWithMultiplicationCached.java,v 1.21 2022/08/09 15:13:34 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class RecurrenceABWithMultiplicationCached extends RecurrenceAB {

    private final BasisFunctionsMultiplicationCache bmulcache;
    private final double [][] dPol;

    public RecurrenceABWithMultiplicationCached(final double [] a,final double [] b){
	super(a,b);
	this.bmulcache=new BasisFunctionsMultiplicationCache(super::setQlQmExpansionCoefs);
	this.dPol=super.getBasisFunctionsDifferentiated(numAk());
    }

    @Override protected double [][] getBasisFunctionsDifferentiated(final int numQl){
	if(numQl>dPol.length) throw new IllegalArgumentException("too large numQl="+numQl+" numAk()="+numAk());
	return this.dPol;
    }

    @Override public void setQlQmExpansionCoefs(final int l,final int m,final double [] s){
	bmulcache.setQlQmExpansionCoefs(l,m,s);
    }

    @Override public double [] getPQkMomentsFromQkMoments(final double [] P,final double [] moments){
	return bmulcache.getPQkMomentsFromQkMoments(P,moments);
    }

    @Override public double [] mult2Pol(final double [] p1,final double [] p2){
	return bmulcache.mult2Pol(p1,p2);
    }


/** Unit test a general 3 term recurrence basis with cached multiplication.
 */
public static void main(String [] args)
{
       final double eps=1e-6;
    // create random sample moments
    final OrthogonalPolynomialsChebyshevBasis.TestChebyshevInMonomials TM=new OrthogonalPolynomialsChebyshevBasis.TestChebyshevInMonomials();

    final int neq_max=50/* a large number of basis functions. */,
	n_sample=100000;
    // from sample
    final RecurrenceAB _ab=TM.getRecurrenceABFromBasisRandomMoments(neq_max,n_sample),
	ab_multiplicationcached=new RecurrenceABWithMultiplicationCached(_ab.getAllAk(),_ab.getAllBk());

    // Use the ab as a basis.
    // Create a unit test from recurrence ab and run it:
    new TestBasisPolynomials("RecurrenceABWithMultiplicationCached",ab_multiplicationcached,new java.util.Random(-79099),eps){
	@Override double [] getMonomialsMoments(final int n){return null;}
	@Override OrthogonalPolynomialsRecurrenceABBasis getOrtPol(){return new OrthogonalPolynomialsRecurrenceABBasis(ab_multiplicationcached){
		@Override public double [] getRadauMultiplied(final double [] nodes0)
		{
		    return B.getaxbP(nodes0,-1,TM.x_max);
		}
	    };}
	@Override double [][] getExact(){return null;}
    }.runTests("$Id: RecurrenceABWithMultiplicationCached.java,v 1.21 2022/08/09 15:13:34 mal Exp $");

}


}
