package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */

/** A class to store regularized data.
 * Similar to {@link RegularizedVectorXF}, but scalar {@code f} is removed;
 * some code is not re-implemented and temporarily calls {@link RegularizedVectorXF}
 * with dummy {@code f} internally.
 * @version $Id: RegularizedVectorX.java,v 1.14 2024/04/04 20:51:48 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public class RegularizedVectorX {
    public final int nX,noriginal;
    public final double totalweight;

    /** The index of const in basis. */
    public final int indexConst;
    public final double normalizedconstbasisfunction;

    private final int [] inds_xorig;
    private final String [] Xdescription;
    private final double [] xorig_average,xorig_norm,xorig_basisfunctions;

    private final double eps;

    RegularizedVectorX(
	final int nX,
	final int noriginal,
	final int [] inds_xorig,
	final double [] xorig_average,
	final double [] xorig_norm,
	final double [] xorig_basisfunctions,
	final String [] Xdescription,

	final double totalweight,
	final double eps){

	this.nX=nX;
	this.noriginal=noriginal;
	this.inds_xorig=inds_xorig;
	this.xorig_average=xorig_average;
	this.xorig_norm=xorig_norm;
	this.xorig_basisfunctions=xorig_basisfunctions;
	this.Xdescription=Xdescription;
	this.indexConst=nX-1;

	if(noriginal+1<nX||Xdescription.length!=nX||
	   xorig_average.length!=noriginal||
	   xorig_norm.length!=noriginal||
	   xorig_basisfunctions.length!=noriginal*(indexConst/* const is added to the basis */)) throw new IllegalArgumentException("noriginal="+noriginal+" nX="+nX+" xorig_basisfunctions.length="+xorig_basisfunctions.length);

	this.totalweight=totalweight;
	this.eps=eps;
	this.normalizedconstbasisfunction=1/Math.sqrt(totalweight);
    }
    /** A transform from original x to regularized X basis.
     */
    public double [] getRegularizedBasisXFromOriginalX(final double [] xorig){
	if(xorig.length!=noriginal) throw new IllegalArgumentException("xorig.length="+xorig.length+" noriginal="+noriginal);
	final double [] xs=new double [noriginal];
	for(int k:inds_xorig) xs[k]=(xorig[k]-xorig_average[k])/xorig_norm[k];
	final double [] res=new double [nX];
	TMatr.multiply_MA_x_MB(res,indexConst,xorig_basisfunctions,noriginal,xs,1);
	res[indexConst]=normalizedconstbasisfunction; // last element in basis is the const
	return res;
    }
    public double [] getBasisFunctionEqual1(){
	final double [] res=new double [nX];
	// First 0..nX-2 are (x-x_average)/sigma, and the last (indexConst) is the constant.
	res[indexConst]=1/normalizedconstbasisfunction; // last element in basis is the const
	return res;
    }


    /** Regularized basis function description.
     * @param i Basis function index.
     * @return The description.
     */
    public String getRegulaizedBasisXDescription(final int i){
	if(i<0||i>=Xdescription.length) throw new IllegalArgumentException("i="+i+" Xdescription.length="+Xdescription.length);
	return Xdescription[i];
    }


} // END OF class RegularizedVectorX
