package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/** A class to store regularized data.
  * @version $Id: RegularizedVectorXF.java,v 1.25 2022/11/02 20:37:40 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  */
public class RegularizedVectorXF implements AttributesProductsMultiIndexed,RNPointEvaluatable,FtoChristoffelFunctionTransformable {
    public final int nX,noriginal;
    /** The index of const in basis. */
    public final int indexConst;
    final int [] inds_xorig;
    final double [] XfX,XX;
    final double [] xorig_average,xorig_norm,xorig_basisfunctions;
    public final double totalweight;
    final String [] Xdescription;
    final double normalizedconstbasisfunction;
    private final double eps;
    RegularizedVectorXF(final int nX,final int noriginal,final double [] XfX,final double [] XX,final String [] Xdescription,final double [] xorig_average,final double [] xorig_norm,final int [] inds_xorig,final double [] xorig_basisfunctions,double totalweight,final double eps){
	this.noriginal=noriginal;
	this.Xdescription=Xdescription;
	this.nX=nX;
	this.inds_xorig=inds_xorig;
	this.indexConst=nX-1;
	this.XfX=XfX;
	this.XX=XX;
	this.xorig_average=xorig_average;
	this.xorig_norm=xorig_norm;
	this.xorig_basisfunctions=xorig_basisfunctions;
	this.totalweight=totalweight;
	this.eps=eps;
	//System.err.println("xorig_basisfunctions.length="+xorig_basisfunctions.length+" nX="+nX+" noriginal="+noriginal);
	if(noriginal+1<nX||XX.length!=nX*nX||XfX.length!=XX.length||Xdescription.length!=nX||
	   xorig_average.length!=noriginal||
	   xorig_norm.length!=noriginal||
	   xorig_basisfunctions.length!=noriginal*(indexConst/* const is added to the basis */)) throw new IllegalArgumentException("noriginal="+noriginal+" nX="+nX+" xorig_basisfunctions.length="+xorig_basisfunctions.length);
	this.normalizedconstbasisfunction=Math.sqrt(XX[(indexConst)*nX+indexConst]/totalweight);
    }
    /** A transform from original x to regularized X basis.
     */
    public double [] getRegularizedBasisXFromOriginalX(final double [] xorig){
	if(xorig.length!=noriginal) throw new IllegalArgumentException();
	final double [] xs=new double [noriginal];
	for(int k:inds_xorig) xs[k]=(xorig[k]-xorig_average[k])/xorig_norm[k];
	final double [] res=new double [nX];
	TMatr.multiply_MA_x_MB(res,indexConst,xorig_basisfunctions,noriginal,xs,1);
	res[indexConst]=normalizedconstbasisfunction; // last element in basis is the const
	return res;
    }
    double [] getBasisFunctionEqual1(){
	final double [] res=new double [nX];
	// First 0..nX-2 are (x-x_average)/sigma, and the last (indexConst) is the constant.
	res[indexConst]=1/normalizedconstbasisfunction; // last element in basis is the const
	return res;
    }
    /* Not possible to calculate uniquely, the problem can be degenerated.
       double [] getOriginalXFromRegulaizedBasisX(final double [] X){
       return null;
       }
    */
    /** Calculate Christoffel function (.XX) from original sample.
     * The second pass, Christoffel matrix (in X basis).
     */
    XXMatrs getChristoffelFunctionDensityMatrixInXBasis(final DataReadObservationVectorXF orig_sample){
	if(eps>0){
	    if(!java.util.Arrays.equals(XX,TMatr.getUnitMatrix(nX))) throw new RuntimeException("XX is not a unit matrx="+java.util.Arrays.toString(XX)+" created by getDataRegularized_{EV,LIN}()");
	}
	return orig_sample.getXXandXfX(
	    this::getRegularizedBasisXFromOriginalX,
	    (X)->TMatr.mulVV(X,X) // Assume getDataRegularized_{EV,LIN}() created XX as unit matrix.
				       );
    }

    /** A compaitibility layer to use to new vector-to vector code;
     * skip {@code f} from the basis. Used in unit tests.
     */
    RegularizedVectorX getDataWithoutF(){
	if(eps>0){
	    if(!java.util.Arrays.equals(XX,TMatr.getUnitMatrix(nX))) throw new RuntimeException("XX is not a unit matrx="+java.util.Arrays.toString(XX)+" created by getDataRegularized_{EV,LIN}()");
	}
	return new RegularizedVectorX(
	    nX,
	    noriginal,
	    inds_xorig,
	    xorig_average,
	    xorig_norm,
	    xorig_basisfunctions,
	    Xdescription,

	    totalweight,
	    eps);
    }

    /** Regularized basis function description.
     * @param i Basis function index.
     * @return The description.
     */
    public String getRegulaizedBasisXDescription(final int i){
	if(i<0||i>=Xdescription.length) throw new IllegalArgumentException("i="+i+" Xdescription.length="+Xdescription.length);
	return Xdescription[i];
    }
    /** Create Radon--Nikodym model from the data.
     */
    public RadonNikodymSpectralModel getRadonNikodymSpectralModel(){
	return new RadonNikodymSpectralModel(
	    noriginal,nX,
	    XfX,XX,Skewness.MB.EV.getGEV_Generalized(nX,XfX,XX,true),
	    this::getRegularizedBasisXFromOriginalX,(psiX)->TMatr.getAMatrB(psiX,getBasisFunctionEqual1(),XX,nX));
    }

    public RadonNikodymSpectralModel getRadonNikodymSpectralModelWithChristoffelFunctionAsF(final DataReadObservationVectorXF orig_sample){
	final double [] XKX=getChristoffelFunctionDensityMatrixInXBasis(orig_sample).XX;
	return new RadonNikodymSpectralModel(
	    noriginal,nX,
	    XKX,XX,Skewness.MB.EV.getGEV_Generalized(nX,XKX,XX,true),
	    this::getRegularizedBasisXFromOriginalX,(psiX)->TMatr.getAMatrB(psiX,getBasisFunctionEqual1(),XX,nX));
    }
    public RadonNikodymSpectralModel getRadonNikodymSpectralModelWithFassumedDiagonalInChristoffelFunctionBasis(final DataReadObservationVectorXF orig_sample){
	final double [] XKX=getChristoffelFunctionDensityMatrixInXBasis(orig_sample).XX,
	    evK=Skewness.MB.EV.getGEV_Generalized(nX,XKX,XX,true),
	    diagFels=TMatr.getMatrixDiagElementsInBasis(nX,evK,XfX);
	// assume diag elements are now the eigenvalues
	System.arraycopy(diagFels,0,evK,0,nX);
	final double [] XfXdiagK=EVXData.convertMatrFromBtoQ_Basis(nX,TMatr.getDiagMatrix(diagFels),evK,XX);
	return new RadonNikodymSpectralModel(
	    noriginal,nX,
	    XfXdiagK,XX,evK,
	    this::getRegularizedBasisXFromOriginalX,(psiX)->TMatr.getAMatrB(psiX,getBasisFunctionEqual1(),XX,nX));
    }

    /** Create attributes products of a multi--index not greater than d.
     * This model attributes are "producted" and new data model
     * of {@link DataReadObservationVectorXF DataReadObservationVectorXF}
     * type is created.
     * @param d max product index, d=1 make the model unchanged.
     * @param orig_sample Original data the model is built from.
     * @return The data as it were read from a sample.
     */
    public DataReadObservationVectorXF getAttribsProductsMultiIndexed(
	final int d,
	final DataReadObservationVectorXF orig_sample){
	return AttributesProductsMultiIndexed.getAttribsProductsMultiIndexed(d,orig_sample,this::getRegularizedBasisXFromOriginalX);
    }

    /** Calculates a RN point at given xorig,
     * for a localized point we do not need a spectral
     * model {@link RadonNikodymSpectralModel RadonNikodymSpectralModel}.
     * @param xorig Original x.
     * @return RN point evaluated.
     */
    public RNPointAccess getRNatXoriginal(final double [] xorig){
	return new RNPointAccess(){
	    private final double
		Ochristoffel, // the denominator is X (XX)^{-1} X = 1/K(X)
		fnom,// the nominator is X (XX)^{-1} XfX XX^{-1} X
		ls;
	    {
		final double [] X=getRegularizedBasisXFromOriginalX(xorig),fXa=TMatr.getMatr_x_A(nX,XfX,getBasisFunctionEqual1());
		if(false){
		    final double [] IXX=Linsystems.getInvertedMatrix(nX,nX,XX);
		    this.Ochristoffel=TMatr.getAMatrA(X,IXX,nX);
		    this.fnom=TMatr.getAMatrA(X,TMatr.mulMMM(nX,IXX,XfX,IXX),nX);
		    this.ls=TMatr.getAMatrB(X,fXa,IXX,nX);
		} else {
		    // Optimized version.
		    // The getDataRegularized_EV() and getDataRegularized_LIN() creates the class with XX equals to the unit matrix.
		    if(eps>0){
			if(!java.util.Arrays.equals(XX,TMatr.getUnitMatrix(nX))) throw new RuntimeException("XX is not a unit matrx="+java.util.Arrays.toString(XX)+" created by getDataRegularized_{EV,LIN}()");
		    }
		    this.Ochristoffel=TMatr.mulVV(X,X);
		    this.fnom=TMatr.getAMatrA(X,XfX,nX);
		    this.ls=TMatr.mulVV(X,fXa);
		}
	    }
	    /** An evaluation of Radon-Nikodym derivative.
	     */
	    public double getRN(){return this.fnom/this.Ochristoffel;}
	    /** 1/K(x).
	     */
	    public double getChristoffelOatX(){return this.Ochristoffel;}
	    /** Least squares.
	     */
	    public double getLS(){return this.ls;}
	};
    }
} // END OF class RegularizedVectorXF
