package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/** A class to store regularized data.
  * @version $Id: RegularizedVectorXVectorF.java,v 1.18 2024/04/02 01:45:23 mal Exp $
  * @author Vladislav Malyshkin mal@gromco.com
  */
public class RegularizedVectorXVectorF {
    public final RegularizedVectorX X;
    public final RegularizedVectorX F;

    RegularizedVectorXVectorF(
        final RegularizedVectorX X,
	final RegularizedVectorX F){
	this.X=X;
	this.F=F;
    }


    public double [] getDyadicProductMomentsInFXBasis(final DataReadObservationVectorXVectorF orig_sample,final java.util.function.ToDoubleFunction<ObservationVectorXVectorF>observationCommonFactor){
	return orig_sample.getFun1Fun2Averaged(
	    o->TMatr.dyadicMatrix(this.F.getRegularizedBasisXFromOriginalX(o.f),this.X.getRegularizedBasisXFromOriginalX(o.x)),
	    o->TMatr.dyadicMatrix(this.F.getRegularizedBasisXFromOriginalX(o.f),this.X.getRegularizedBasisXFromOriginalX(o.x)),
	    observationCommonFactor);
    }

    /** Calculate {X,F}-Christoffel functions product moments from original sample.
     * The second pass, Christoffel matrix (in X,F basis).
     */
    public double [] getChristoffelFunctionsProductMomentsInFXBasis(final DataReadObservationVectorXVectorF orig_sample){
	return getDyadicProductMomentsInFXBasis(orig_sample, o->{
		final double [] rF=this.F.getRegularizedBasisXFromOriginalX(o.f),
		    rX=this.X.getRegularizedBasisXFromOriginalX(o.x);
		return 1/(TMatr.mulVV(rF,rF)*TMatr.mulVV(rX,rX));
	    }
	    );
    }

    /** Calculate {X,F}-Christoffel functions product moments from original sample,
     * the Christoffel(x) is calculated in a subspace of X.
     * The second pass, Christoffel matrix (in X,F basis).
     */
    public double [] getChristoffelFunctionsProductMomentsInFXBasis_nCdegreesOfFreedom(final DataReadObservationVectorXVectorF orig_sample,final double [] CX){
	return getDyadicProductMomentsInFXBasis(orig_sample,o->{
		final double [] rF=this.F.getRegularizedBasisXFromOriginalX(o.f),
		    rX=this.X.getRegularizedBasisXFromOriginalX(o.x),
		    cX=TMatr.mul_MA_x_MB(CX.length/rX.length,CX,rX.length,rX,1);
		return 1/(TMatr.mulVV(rF,rF)*TMatr.mulVV(cX,cX));
	    }
	    );
    }

    /** For unit test.
     */
    public double [] getFXmoments(final DataReadObservationVectorXVectorF orig_sample){
	return orig_sample.getFun1Fun2Averaged(
	    o->this.F.getRegularizedBasisXFromOriginalX(o.f),
	    o->this.X.getRegularizedBasisXFromOriginalX(o.x),
	    o->1);
    }

    public double [] getXChristoffelFunctionsMoments(final DataReadObservationVectorXVectorF orig_sample){
	return orig_sample.getFun1Fun2Averaged(
	    o->this.X.getRegularizedBasisXFromOriginalX(o.x),
	    o->this.X.getRegularizedBasisXFromOriginalX(o.x),
	    o->{
		final double [] rX=this.X.getRegularizedBasisXFromOriginalX(o.x);
		return 1/TMatr.mulVV(rX,rX);
	    });
    }
    public double [] getFChristoffelFunctionFMoments(final DataReadObservationVectorXVectorF orig_sample){
	return orig_sample.getFun1Fun2Averaged(
	    o->this.F.getRegularizedBasisXFromOriginalX(o.f),
	    o->this.F.getRegularizedBasisXFromOriginalX(o.f),
	    o->{
		final double [] rF=this.F.getRegularizedBasisXFromOriginalX(o.f);
		return 1/TMatr.mulVV(rF,rF);
	    });
    }
    public double [] getFChristoffelFunctionXMoments(final DataReadObservationVectorXVectorF orig_sample){
	return orig_sample.getFun1Fun2Averaged(
	    o->this.X.getRegularizedBasisXFromOriginalX(o.x),
	    o->this.X.getRegularizedBasisXFromOriginalX(o.x),
	    o->{
		final double [] rF=this.F.getRegularizedBasisXFromOriginalX(o.f);
		return 1/TMatr.mulVV(rF,rF);
	    });
    }


    /** Create attributes products of a multi--index not greater than xD for {@link #X}
     * and fD for {@link #X}.
     * This model attributes are "producted" and new data model
     * of {@link RegularizedVectorXVectorF} type is created.
     * @param xD max product index for {@link #X}, xD=1 make the model unchanged.
     * @param fD max product index for {@link #F}, fD=1 make the model unchanged.
     * @param orig_sample Original data the model is built from.
     * @return The data as it were read from a sample.
     */
    public DataReadObservationVectorXVectorF getAttribsProductsMultiIndexed(
	final int xD,
	final int fD,
	final DataReadObservationVectorXVectorF orig_sample){
	System.err.println("xD="+xD+" fD="+fD);

	// For now. Call old "scalar" code with unused f, re-implement it later.
	return DataReadObservationVectorXVectorF.getVectorXVectorFromScalars(
	    xD<=0? o-> getTheSame(true,o) :
	    o->AttributesProductsMultiIndexed.getAttribsProductsMultiIndexed(
		xD,
		orig_sample.getXBasisSample_OLDSCALARCODE(),
		this.X::getRegularizedBasisXFromOriginalX),
	    fD<=0? o-> getTheSame(false,o) :
	    o->AttributesProductsMultiIndexed.getAttribsProductsMultiIndexed(
		fD,
		orig_sample.getFBasisSample_OLDSCALARCODE(),
		this.F::getRegularizedBasisXFromOriginalX),
	    orig_sample);
    }


    /** An adapter to "skip" attributes production.
     */
    private static DataReadObservationVectorXF getTheSame(final boolean flagX,final DataReadObservationVectorXVectorF orig_sample){
	final var data=new java.util.ArrayList<ObservationVectorXF>(orig_sample.data.size()+10);
	int l=0;
	for(ObservationVectorXVectorF o:orig_sample.data)
	{
	    data.add(new ObservationVectorXF(o.label,flagX?o.x:o.f,++l,o.weight));
	}
	return new DataReadObservationVectorXF(data,flagX?orig_sample.xHeader:orig_sample.fHeader);
    }

}
