package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/** Unit test for a BasisFunctionsCalculatable object.
 *  @version $Id: TestBasisFunctionsCalculatable.java,v 1.127 2022/07/19 08:36:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class TestBasisFunctionsCalculatable {

final BasisFunctionsCalculatable B;
final java.util.Random r;
final double eps;
final String txt;

static final BasisPolynomials [] PBASIS_CONVERSION_TEST=new BasisPolynomials []{
    new Monomials(),
    new OrthogonalPolynomialsChebyshevBasis.TestChebyshevInMonomials().getRecurrenceABFromBasisRandomMoments(50,200000),
    new Chebyshev(),
    new Legendre(),
    new LegendreShifted(),
    new Laguerre(),
    new HermiteE()
};
static final BasisPolynomials [] PBASIS_CONVERSION_TEST_WITHOUT_Laguerre_AND_HermiteE=java.util.Arrays.copyOf(PBASIS_CONVERSION_TEST,PBASIS_CONVERSION_TEST.length-2);

TestBasisFunctionsCalculatable(final String txt,final BasisFunctionsCalculatable B,final java.util.Random r,final double eps)
{
    this.txt=txt;
    this.r=r;
    this.eps=eps;
    this.B=B;
}


void test_setQlQmExpansionCoefs(final int n_tests,final int nmax)
{
    final double [][] P=new double [nmax+1][];
    for(int k=0;k<P.length;k++)
    {
	final double [] Tcoefs=new double [k+1];Tcoefs[k]=1;
	P[k]=B.convertFunctionTo_Monomials_from_Basis(Tcoefs);
    }

    for(int t=0;t<n_tests;t++)
    {

    for(int l=0;l<=nmax;l++)
    for(int m=0;m<=nmax;m++)
    {
	// a random vector to check clear
	final double [] res=TMatr.getRandomVector(l+m+1,r);
	B.setQlQmExpansionCoefs(l,m,res);
	final double [] Pplpm=B.convertFunctionTo_Basis_from_Monomials(Polynomials.mult2Pol(P[l],P[m]));
	UnitTests.checkDoubleArrayRel1(Pplpm,"direct TlTm conv",res,"setQlQmExpansionCoefs",eps);
    }

    }

    System.err.println(txt+":test_setQlQmExpansionCoefs(): test OK with nmax="+nmax+" eps="+eps);
}



void printPolynomials(final int nmax)
{
    System.err.println("========="+txt+": START print basis polynomials");
    for(int n=0;n<=2*nmax;n++)
    {
	final double [] Tcoefs=new double [n+1];Tcoefs[n]=1;
	final double [] rn=B.convertFunctionTo_Monomials_from_Basis(Tcoefs);
	System.err.print("n="+n);
	for(int j=0;j<=n;j++)
	{
	    System.err.print("\t"+rn[j]);
	}
	System.err.println();
    }
    System.err.println("========="+txt+": END print basis polynomials");
}



void test_simpleSum(final int n_tests,final int nmax)
{
  for(int t=0;t<n_tests;t++)
  {
    for(int nn=0;nn<=nmax;nn++)
    {
	// a random polynomial in basis B
	double [] coefs=TMatr.getRandomVector(nn+1,r);
	double [] ru=null,res=new double [nn+1];
	for(int n=0;n<=nn;n++)
	{
	    final double [] Tcoefs=new double [n+1];Tcoefs[n]=1;
	    ru=B.convertFunctionTo_Monomials_from_Basis(Tcoefs);
	    TMatr.sumP1(res,ru,coefs[n]);
	}
	final double [] sum_test=B.convertFunctionTo_Monomials_from_Basis(coefs);
	UnitTests.checkDoubleArrayRel1(sum_test,"sum_test",res,"res",eps);
	UnitTests.checkDoubleArrayRel1(coefs,"coefs",B.convertFunctionTo_Basis_from_Monomials(sum_test),"sum_test",eps);


	// differentiation
	UnitTests.checkDoubleArrayRel1(B.convertFunctionTo_Monomials_from_Basis(B.getDifferentiated(coefs)),"DBasis",
				      Polynomials.getPolynomialDifferentiated(B.convertFunctionTo_Monomials_from_Basis(coefs)),"DP",eps);
	UnitTests.checkDoubleArrayRel1(B.getDifferentiated(coefs),"DBasis",
				      B.convertFunctionTo_Basis_from_Monomials(Polynomials.getPolynomialDifferentiated(B.convertFunctionTo_Monomials_from_Basis(coefs))),"DP",eps);

	// integration
	UnitTests.checkDoubleArrayRel(B.convertFunctionTo_Monomials_from_Basis(B.getIntegrated(coefs)),"IBasis",
				      Polynomials.getPolynomialIntegrated(B.convertFunctionTo_Monomials_from_Basis(coefs)),"IP",eps);
	UnitTests.checkDoubleArrayRel1(B.getIntegrated(coefs),"IBasis",
				      B.convertFunctionTo_Basis_from_Monomials(Polynomials.getPolynomialIntegrated(B.convertFunctionTo_Monomials_from_Basis(coefs))),"IBasis",eps);


	final double FE=1e-3;
	for(double x:TMatr.getRandomVector(50,r,1,0))
	{
	    UnitTests.checkDoubleRel(B.getD0(B.getONE(),x),"B.getONE",1,"1",eps);

	    if(!(Math.abs(Polynomials.getD0(ru,x))<=FE))
		UnitTests.checkDoubleRel(Polynomials.getD0(ru,x),"Ln",B.calculate(nn,x),"calculate",eps);
	    if(!(Math.abs(Polynomials.getD0(res,x))<=FE))
		UnitTests.checkDoubleRel(Polynomials.getD0(res,x),"D0sum",B.getD0(coefs,x),"getD0",eps);
	    if(!(Math.abs(Polynomials.getD1(res,x))<=FE))
		UnitTests.checkDoubleRel(Polynomials.getD1(res,x),"D1sum L",B.getD1(coefs,x),"getD1",eps);
	    if(!(Math.abs(Polynomials.getD2(res,x))<=FE))
		UnitTests.checkDoubleRel(Polynomials.getD2(res,x),"D2sum L",B.getD2(coefs,x),"getD2",eps);
	}
    }
  }
  System.err.println(txt+":test_simpleSum(): test OK with n_tests="+n_tests+" nmax="+nmax+" eps="+eps);
}
void test_simpleSum_PBASIS(final int n_tests,final int nmax)
{
  for(int t=0;t<n_tests;t++)
  {
    for(int nn=0;nn<=nmax;nn++)
    {
	// Create a random polynomial in basis B.
	// select random vector for unit test stability.
	final double [] coefs=B.convertFunctionTo_Basis_from_PBASIS(TMatr.getRandomVector(nn+1,r,-2,1.1), new Monomials() );
	for(BasisFunctionsCalculatable PBASIS:PBASIS_CONVERSION_TEST)
	{
	    double [] ru=null,res=new double [nn+1];
	    for(int n=0;n<=nn;n++)
	    {
		final double [] Tcoefs=new double [n+1];Tcoefs[n]=1;
		ru=B.convertFunctionTo_PBASIS_from_Basis(Tcoefs,PBASIS);
		TMatr.sumP1(res,ru,coefs[n]);
	    }
	    final double [] sum_test=B.convertFunctionTo_PBASIS_from_Basis(coefs,PBASIS);
	    UnitTests.checkDoubleArrayRel1(sum_test,"sum_test",res,"res",eps);
	    UnitTests.checkDoubleArrayRel1(coefs,"coefs",B.convertFunctionTo_Basis_from_PBASIS(sum_test,PBASIS),"sum_test",eps);

	    // coefs: basis B
	    // res: basis PBASIS
	    // differentiation
	    UnitTests.checkDoubleArrayRel1(B.convertFunctionTo_PBASIS_from_Basis(B.getDifferentiated(coefs),PBASIS),"DBasis",
					  PBASIS.getDifferentiated(B.convertFunctionTo_PBASIS_from_Basis(coefs,PBASIS)),"DP",eps);
	    UnitTests.checkDoubleArrayRel1(B.getDifferentiated(coefs),"DBasis",
					  B.convertFunctionTo_Basis_from_PBASIS(PBASIS.getDifferentiated(B.convertFunctionTo_PBASIS_from_Basis(coefs,PBASIS)),PBASIS),"DP",eps);

	    // integration
	    UnitTests.checkDoubleArrayRel1(B.convertFunctionTo_PBASIS_from_Basis(B.getIntegrated(coefs),PBASIS),"IBasis",
					  PBASIS.getIntegrated(B.convertFunctionTo_PBASIS_from_Basis(coefs,PBASIS)),"IP",eps);
	    UnitTests.checkDoubleArrayRel1(B.getIntegrated(coefs),"IBasis",
					   B.convertFunctionTo_Basis_from_PBASIS(PBASIS.getIntegrated(B.convertFunctionTo_PBASIS_from_Basis(coefs,PBASIS)),PBASIS),"IBasis",eps);

	    final double FE=1e-3;
	    for(double x:TMatr.getRandomVector(50,r,1,0))
	    {
		UnitTests.checkDoubleRel(B.getD0(B.getONE(),x),"B.getONE",1,"1",eps);

		if(!(Math.abs(PBASIS.getD0(ru,x))<=FE))
		    UnitTests.checkDoubleRel(PBASIS.getD0(ru,x),"Ln",B.calculate(nn,x),"calculate",eps);
		if(!(Math.abs(PBASIS.getD0(res,x))<=FE))
		    UnitTests.checkDoubleRel(PBASIS.getD0(res,x),"D0sum",B.getD0(coefs,x),"getD0",eps);
		if(!(Math.abs(PBASIS.getD1(res,x))<=FE))
		    UnitTests.checkDoubleRel(PBASIS.getD1(res,x),"D1sum L",B.getD1(coefs,x),"getD1",eps);
		if(!(Math.abs(PBASIS.getD2(res,x))<=FE))
		    UnitTests.checkDoubleRel(PBASIS.getD2(res,x),"D2sum L",B.getD2(coefs,x),"getD2",eps);
	    }
	}
    }
  }
  System.err.println(txt+":test_simpleSum_PBASIS(): test OK with n_tests="+n_tests+" nmax="+nmax+" eps="+eps);
}



/** Check p==d*q+r.
 */
void matchmul(final double [] p,final double [] d,final double [] q,final double [] r)
{
    final double [] mul=B.mult2Pol(d,q);
    TMatr.sumP1(mul,r,r.length,1);
    UnitTests.checkDoubleArrayRel(p,"p",mul,"q x d +r",eps);
}
void test_getaxbP(final int n_tests,final int nmax)
{
    if(nmax<=0){
	System.err.println(txt+":test_getaxbP(): skipping nmax="+nmax+" eps="+eps);
	return;
    }
    for(int t=0;t<n_tests;t++)
    {
	final double ma=1-2*r.nextDouble(),mb=1-2*r.nextDouble();
	final double [] maxmb=B.convertFunctionTo_Basis_from_Monomials(new double []{mb,ma});
	final double [] p=TMatr.getRandomVector(1+r.nextInt(Math.min(1+2*nmax,nmax+2)),r);
	if(true)
	{
	    matchmul(B.getaxbP(p,ma,mb),p,maxmb,new double [] {0});
	    final double [] cpres=B.convertFunctionTo_Basis_from_Monomials(Polynomials.mult2Pol(B.convertFunctionTo_Monomials_from_Basis(p),new double []{mb,ma}));
	    UnitTests.checkDoubleArrayRel(B.getaxbP(p,ma,mb),"B.getaxbP(p,ma,mb)",cpres,"cpres",eps);
	}

	final double [] moms=TMatr.getRandomVector(p.length,r);
	final double [] Txmom=new double [p.length-1],Tmaxmbmoms=new double [p.length-1];
	for(int k=0;k<Txmom.length;k++)
	{
	    final double [] m=new double [k+1];m[k]=1;
	    Txmom[k]=TMatr.mulVVn1(B.getaxbP(m,1,0),moms);
	    Tmaxmbmoms[k]=TMatr.mulVVn1(B.getaxbP(m,ma,mb),moms);
	}
	UnitTests.checkDoubleArrayRel(B.getXQkMomentsFromQkMoments(moms),"X*moms",Txmom,"Txmom",eps);
	UnitTests.checkDoubleArrayRel(B.getPQkMomentsFromQkMoments(maxmb,moms),"(ma*x+mb)*moms",Tmaxmbmoms,"Tmaxmbmoms",eps);
    }
    System.err.println(txt+":test_getaxbP(): test OK with nmax="+nmax+" eps="+eps);
}

void test_mult2Pol(final int n_tests,final int nmax)
{
    final int nc=4;
    for(int t=0;t<n_tests;t++)
    {
	final double [] p1=new double [1+r.nextInt(nmax+1)],p2=new double [1+r.nextInt(nmax+1)];
	for(int i=p1.length;--i>=0;) p1[i]=r.nextDouble()/(i<nc?1:Polynomials.fact(i-nc));
	for(int i=p2.length;--i>=0;) p2[i]=r.nextDouble()/(i<nc?1:Polynomials.fact(i-nc));
	final double [] res=B.mult2Pol(p1,p2);

	final double [] mtest=B.convertFunctionTo_Basis_from_Monomials(Polynomials.mult2Pol(B.convertFunctionTo_Monomials_from_Basis(p1),B.convertFunctionTo_Monomials_from_Basis(p2)));
	UnitTests.checkDoubleArrayRel1(res,"res",mtest,"mtest",eps);

	final double [] P,moms;if(p1.length<=p2.length){P=p1;moms=p2;}else{P=p2;moms=p1;}
	final double [] PQmoms=B.getPQkMomentsFromQkMoments(P,moms),pq=new double [moms.length-(P.length-1)];
	for(int k=0;k<pq.length;k++){
	    final double [] tmp=new double [k+1];tmp[k]=1;
	    pq[k]=TMatr.mulVVn1(B.mult2Pol(tmp,P),moms);
	}
	UnitTests.checkDoubleArrayRel(pq,"pq",PQmoms,"PQmoms",eps);

	final double [][] Qlp=B.getBasisFunctionsMultipliedByPolynomial(p1.length,p1);
	for(int k=0;k<Qlp.length;k++){
	    final double [] ck=new double [k+1];ck[k]=1;
	    UnitTests.checkDoubleArrayRel(Qlp[k],"Qlp[k]",B.mult2Pol(p1,ck),"B.mult2Pol(p1,ck)",eps);
	}
    }
    System.err.println(txt+":test_mult2Pol: OK with n_tests="+n_tests+" nmax="+nmax+" eps="+eps);
}


void test_sdiv1(final int n_tests,final int nmax)
{
    if(nmax<=0){
	System.err.println(txt+":test_sdiv1(): skipping nmax="+nmax+" eps="+eps);
	return;
    }

    for(int t=0;t<n_tests;t++)
    {
	final double [] p=TMatr.getRandomVector(1+r.nextInt(Math.min(1+2*nmax,nmax+2)),r,1.9,0.1);
	p[p.length-1]=(0.5+r.nextDouble())*(r.nextInt(2)<1?1:-1);
	p[0]+=0.1+r.nextDouble();
	if(true)
	{
	    final double x0=1-2*r.nextDouble();
	    final SDivRes sdr=B.sdiv1(p,x0);
	    if(sdr.q.length!=p.length-1 || sdr.r.length!=1) throw new RuntimeException();
	    final double [] d=B.convertFunctionTo_Basis_from_Monomials(new double []{-x0,1});
	    matchmul(p,d,sdr.q,sdr.r);
	    matchmul(p,sdr.q,d,sdr.r);
	    final SDivRes sdrg=B.sdiv(p,d);
	    UnitTests.checkDoubleArrayRel(sdr.q,"sdr.q",sdrg.q,"sdrg.q",eps);
	    UnitTests.checkDoubleArrayRel(sdr.r,"sdr.r",sdrg.r,"sdrg.r",eps);

	    final SDivRes pol_sdr=Polynomials.sdiv1(B.convertFunctionTo_Monomials_from_Basis(p),x0);
	    UnitTests.checkDoubleArrayRel(sdr.q,"sdr.q",B.convertFunctionTo_Basis_from_Monomials(pol_sdr.q),"B.convertFunctionTo_Basis_from_Monomials(pol_sdr.q)",eps);
	    UnitTests.checkDoubleArrayRel(sdr.r,"sdr.r",B.convertFunctionTo_Basis_from_Monomials(pol_sdr.r),"B.convertFunctionTo_Basis_from_Monomials(pol_sdr.r)",eps);

	    final SDivRes pol_sdrg=Polynomials.sdiv(B.convertFunctionTo_Monomials_from_Basis(p),B.convertFunctionTo_Monomials_from_Basis(d));
	    UnitTests.checkDoubleArrayRel(sdr.q,"sdr.q",B.convertFunctionTo_Basis_from_Monomials(pol_sdrg.q),"B.convertFunctionTo_Basis_from_Monomials(pol_sdrg.q)",eps);
	    UnitTests.checkDoubleArrayRel(sdr.r,"sdr.r",B.convertFunctionTo_Basis_from_Monomials(pol_sdrg.r),"B.convertFunctionTo_Basis_from_Monomials(pol_sdrg.r)",eps);
	}
    }
    System.err.println(txt+":test_sdiv1(): test OK with nmax="+nmax+" eps="+eps);
}

void test_sdiv(final int n_tests,final int nmax)
{
    for(int t=0;t<n_tests;t++)
    {
	final double [] p=TMatr.getRandomVector(1+r.nextInt(Math.min(1+2*nmax,nmax+2)),r,1.9,0.1),
	    d=TMatr.getRandomVector(1+this.r.nextInt(p.length),r);
	if(false)
	{
	    System.err.println("d="+java.util.Arrays.toString(d));
	    System.err.println("p="+java.util.Arrays.toString(p));
	}
	// make the leading coefficient >1, to avoid instability in tests.
	d[d.length-1]=2+r.nextDouble();

	final SDivRes sdr=B.sdiv(p,d);
	if(sdr.q.length!=p.length-(d.length-1)||sdr.r.length!=d.length-1) throw new RuntimeException();
	matchmul(p,d,sdr.q,sdr.r);
	matchmul(p,sdr.q,d,sdr.r);

	final SDivRes pol_sdr=Polynomials.sdiv_stable(B.convertFunctionTo_Monomials_from_Basis(p),B.convertFunctionTo_Monomials_from_Basis(d));
	UnitTests.checkDoubleArrayRel(sdr.q,"sdr.q",B.convertFunctionTo_Basis_from_Monomials(pol_sdr.q),"B.convertFunctionTo_Basis_from_Monomials(pol_sdr.q)",eps);
	UnitTests.checkDoubleArrayRel(sdr.r,"sdr.r",B.convertFunctionTo_Basis_from_Monomials(pol_sdr.r),"B.convertFunctionTo_Basis_from_Monomials(pol_sdr.r)",eps);
    }
    System.err.println(txt+":test_sdiv(): test OK with nmax="+nmax+" eps="+eps);
}


void test_ConfederateMatrix(final int n_tests,final int n)
{
    for(int t=0;t<n_tests;t++)
    {
	final double [] pol=new double [n+1];
	for(int k=pol.length;--k>=0;) pol[k]=(1-2*r.nextDouble())/(k+1);
	pol[n]+=3.2;

	final PolynomialRootsFinder.PRoots u=new PolynomialRootsJenkinsTraubORIG().findRoots(pol);
	if(u.nroots!=n) throw new RuntimeException("u.nroots="+u.nroots+" n="+n);
	SortArrays.sortArrayWith2Load(u.nroots,TMatr.sumAB(u.nroots,u.re_root,u.im_root),u.re_root,u.im_root);

	final PolynomialRootsFinder.PRoots ub=new PolynomialRootsConfederateMatrix(B).findRoots(B.convertFunctionTo_Basis_from_Monomials(pol));
	if(ub.nroots!=n) throw new RuntimeException("ub.nroots="+ub.nroots+" n="+n+" bpol="+java.util.Arrays.toString(ub.Pcoefs));
	SortArrays.sortArrayWith2Load(ub.nroots,TMatr.sumAB(ub.nroots,ub.re_root,ub.im_root),ub.re_root,ub.im_root);
	UnitTests.checkDoubleArrayRel1(ub.re_root,"ub.re_root",u.re_root,"u.re_root",eps);
	UnitTests.checkDoubleArrayRel1(ub.im_root,"ub.im_root",u.im_root,"u.im_root",eps);
    }
    System.err.println(txt+":test_ConfederateMatrix: test OK with n="+n+" eps="+eps);
}

void test_MomentsCalculation(final int n_tests,final int sample_max,final int n)
{
    for(int t=0;t<n_tests;t++)
    {
	final double [] x=TMatr.getRandomVector(12+r.nextInt(sample_max),r),
	    w=TMatr.getRandomVector(x.length,r,1,0);
	final int nelsinsample=x.length-r.nextInt(4);
	final double [] m=B.getMomentsFromSample(n,x,w,nelsinsample),mm=new double [m.length];
	if(m.length!=n) throw new RuntimeException();
	for(int j=0;j<m.length;j++)
	{
	    for(int k=0;k<nelsinsample;k++)
	    {
		mm[j]+=w[k]*B.calculate(j,x[k]);
	    }
	}
	UnitTests.checkDoubleArrayRel(m,"m",mm,"mm",eps);
    }
    System.err.println(txt+":test_MomentsCalculation: test OK with n="+n+" eps="+eps);
}

void test_MomentsCalculation2(final int n_tests,final int n)
{
    for(int t=0;t<n_tests;t++)
    {
	final double [] xk=TMatr.getRandomVector(n+1+10*r.nextInt(20),r,1,0),
	    wk=TMatr.getRandomVector(xk.length,r,3,0);
	final double [] mw=B.getMomentsFromSample(n,xk,wk),
	    m1=B.getMomentsFromSample(n,xk,null);
	if(mw.length!=n || m1.length!=n) throw new RuntimeException();
	final double [] test_mw=new double [n],test_m1=new double [n];
	for(int k=0;k<xk.length;k++){
	    for(int i=0;i<n;i++)
	    {
		final double f=B.calculate(i,xk[k]);
		test_mw[i]+=f*wk[k];
		test_m1[i]+=f;
	    }
	}
	UnitTests.checkDoubleArrayRel(mw,"mw",test_mw,"test_mw",eps);
	UnitTests.checkDoubleArrayRel(m1,"m1",test_m1,"test_m1",eps);
    }
    System.err.println(txt+":test_MomentsCalculation2: test OK n_tests="+n_tests+" eps="+eps);
}


void runSingleTestInSerie(final int n_tests,final int nmax){
    test_simpleSum(n_tests,nmax); // must match almost identically
    test_simpleSum_PBASIS(11+n_tests/10,Math.min(nmax,6));
    test_getaxbP(n_tests,nmax);
    test_setQlQmExpansionCoefs(n_tests,nmax);
    test_sdiv1(n_tests,nmax);
    test_sdiv(n_tests,nmax);
    test_mult2Pol(n_tests,nmax);
    test_MomentsCalculation(n_tests,200,nmax);
    test_MomentsCalculation2(n_tests,nmax);
    test_ConfederateMatrix(n_tests,Math.min(nmax,7));
}

void runSingleTest(final int nmax){
    printPolynomials(nmax);
}

void runTestsOneN(final String id,int n_test,int nmax)
{
    runSingleTest(nmax);
    for(int i=0;i<111;i++)
    {
	runSingleTestInSerie(n_test,nmax);
    }
}

void runTests(final String id,int n_test,int nmaxtotest,int nmaxtoprint)
{
    System.err.println(txt+"runTests(): starting tests; id="+id+" ver=$Id: TestBasisFunctionsCalculatable.java,v 1.127 2022/07/19 08:36:33 mal Exp $");
    final long t0=System.currentTimeMillis();
    runSingleTest(nmaxtoprint);

    for(int nmax=0;nmax<=nmaxtotest;nmax++)
    {
	runTestsOneN(id,n_test,nmax);
    }
    final long t1=System.currentTimeMillis();
    System.err.println("TestBasisFunctionsCalculatable::main(): tests OK done in "+(t1-t0)+" milliseconds. id="+id+" ver=$Id: TestBasisFunctionsCalculatable.java,v 1.127 2022/07/19 08:36:33 mal Exp $");
}

}
