package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2022
 * This file is under GPL version 3.
 *
 */

public class TestEigenvalues {

    static final double EPS_ORT=1e-6;

@FunctionalInterface public interface EVSolverOneMatr {
    double [] getEV(int n,double [] A,int ldA,boolean flagEV);
}
public static void test_getEV(final EVSolverOneMatr evsolver,
			      final int n_tests,
			      final int n,
			      final java.util.Random r,
			      final double eps)
{
 final double [] matr=new double [n*n];
 for(int t=0;t<n_tests;t++)
 {
     final double scale;
     final int is=r.nextInt(10);
     if(is==0) scale=1e200;
     else if(is==1) scale=1e-200;
     else scale=1;

    for(int k=0;k<n;k++)
    {
	for(int l=0;l<=k;l++) matr[k*n+l]=matr[l*n+k]=scale*r.nextDouble();
    }
    final double [] ev=evsolver.getEV(n,matr,n,true);
    final double [] lv=new double [n],mv=new double [n];
    final int o=n;
    for(int k=n;k<n;k++)
    {
	final double lambda=ev[k];
	double nrm=0;
	for(int l=0;l<n;l++)
	{
	    lv[l]=lambda*ev[o+k*n+l];
	    nrm+=ev[o+k*n+l]*ev[o+k*n+l];
	    double s=0;
	    for(int j=0;j<n;j++)
	    {
		s+=matr[l*n+j]*ev[o+k*n+j];
	    }
	    mv[l]=s;
	}
	UnitTests.checkDoubleArrayRel(lv,"lv",mv,"mv",eps);
	UnitTests.checkDoubleRel(nrm,"nrm",1,"1",eps);
	EVXData.test_orthogonality(null,n,ev,matr,TMatr.getUnitMatrix(n),EPS_ORT);
    }
 }
 System.err.println("TestEigenvalues:test_getEV(): n="+n+" n_tests="+n_tests+" eps="+eps+" OK");
}


@FunctionalInterface public interface EVSolverTwoMatr {
    double [] getEV(int n,double [] A,int ldA,double [] B,int ldB,boolean flagEV);
}

public static void test_getGEV(final EVSolverTwoMatr gevsolver,
			       final int n_tests,
			       final int n,
			       final java.util.Random r,
			       final double eps)
{
 final double [] A=new double [n*n],B=new double [n*n];
 for(int t=0;t<n_tests;t++)
 {
     final double scale;
     final int is=r.nextInt(10);
     if(is==0) scale=1e200;
     else if(is==1) scale=1e-200;
     else scale=1;

    for(int k=0;k<n;k++)
    {
	for(int l=0;l<=k;l++){
	    A[k*n+l]=A[l*n+k]=scale*(1-2*r.nextDouble());
	    B[k*n+l]=B[l*n+k]=scale*(r.nextDouble());
	}
    }
    final double [] ev=gevsolver.getEV(n,A,n,B,n,true);
    final double [] lv=new double [n],mv=new double [n];
    final int o=n;
    for(int k=n;k<n;k++)
    {
	final double lambda=ev[k];
	for(int l=0;l<n;l++)
	{
	    double sa=0,sb=0;
	    for(int j=0;j<n;j++)
	    {
		sa+=A[l*n+j]*ev[o+k*n+j];
		sb+=B[l*n+j]*ev[o+k*n+j];
	    }
	    mv[l]=sa;
	    lv[l]=lambda*sb;
	}
	UnitTests.checkDoubleArrayRel(lv,"lv",mv,"mv",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(n,ev,o+k*n,ev,o+k*n,B,n),"<psi|B|psi>",1,"1",eps);
	EVXData.test_orthogonality(null,n,ev,A,B,EPS_ORT);

    }
 }
 System.err.println("TestEigenvalues:test_getGEV(): n="+n+" n_tests="+n_tests+" eps="+eps+" OK");
}

public static void test_getEV3(final EVSolverTwoMatr evsolver,
			       final int n_tests,
			       final int nmax,
			       final java.util.Random r,
			       final double eps)
{
    for(int t=0;t<n_tests;t++)
    {
	final int n=1+r.nextInt(nmax+1);
	final double [] d=new double [n],e=new double [n-1];
	for(int k=0;k<n;k++)
	{
	    d[k]=r.nextDouble();
	    if(k<n-1) e[k]=r.nextDouble();
	}
	final double [] res=evsolver.getEV(n,d,0,e,0,true);
	UnitTests.checkDoubleRel(TMatr.sumElements(java.util.Arrays.copyOf(res,n)),"sum(res)",TMatr.sumElements(d),"sum(d)",eps);
	for(int i=0;i<n;i++){
	    final double [] v=EVXData.getPsiFromBasis(i,res,n),
		mv=new double [n],matr=new double [n*n];
	    for(int k=0;k<n;k++){
		mv[k]=(k<=0?0:e[k-1]*v[k-1])
		    +d[k]*v[k]
		    + (k>=n-1?0:e[k]*v[k+1]);
		matr[k*n+k]=d[k];
		if(k>0) matr[k*n+k-1]=e[k-1];
		if(k<n-1) matr[k*n+k+1]=e[k];
	    }
	    UnitTests.checkDoubleArrayRel1(mv,"mv",TMatr.getVectorScaled(v,res[i]),"lambda*v",eps);
	    EVXData.test_orthogonality(null,n,res,matr,TMatr.getUnitMatrix(n),EPS_ORT);
	}
	for(int k=0;k<n;k++)
	{
	    boolean flag=false;
	    final double [] dt=d.clone();
	    for(int j=0;j<n;j++)
	    {
		dt[j]-=res[k];
	    }
	    final double [] res1=evsolver.getEV(n,dt,0,e,0,false);
	    for(int j=0;j<n;j++)
	    {
		if(Math.abs(res1[j])<=eps)
		{
		    flag=true;
		}
	    }
	    //System.err.println("n="+n+"; res1="+java.util.Arrays.toString(res1));
	    if(!flag) throw new RuntimeException("n="+n+"; res1="+java.util.Arrays.toString(res1));
	}
	//System.err.println("n="+n+"; res1="+java.util.Arrays.toString(res1));
    }
    System.err.println("TestEigenvalues:test_getEV3(): nmax="+nmax+" n_tests="+n_tests+" tests OK eps="+eps);
}

@FunctionalInterface public interface EVSolverSVD {
    EVSolver.SVDResult getSVD(int m,final int n,double [] matr);
}

public static void test_getSVD(final EVSolverSVD svdsolver,
			       final int n_tests,
			       final int N,
			       final java.util.Random r,
			       final double eps)
{
    int nt=0;
    for(int t=0;t<n_tests;t++)
    {
    for(int m=1;m<=N;m++){
	for(int n=1;n<=N;n++,nt++){
	    final double [] matr=new double [m*n];

	    for(int k=0;k<matr.length;k++)
	    {
		matr[k]=(1-2*r.nextDouble());
	    }

	    final var svd=svdsolver.getSVD(m,n,matr);
	    final int s=Math.min(m,n);
	    final double [] S=new double [m*n];
	    for(int k=0;k<s;k++){
		S[k*n+k]=svd.S[k];
	    }
	    //System.err.println("m="+m+" n="+n+" U.length="+svd.U.length+" S.length="+S.length+" VT.length="+svd.VT.length);
	    final double [] US=TMatr.mul_MA_x_MB(m,svd.U,m,S,n),
		USVT=TMatr.mul_MA_x_MB(m,US,n,svd.VT,n),
		_USVT=new double[m*n];
	    for(int j=0;j<m;j++){
		for(int k=0;k<n;k++){
		    for(int l=0;l<Math.min(m,n);l++){
			_USVT[j*n+k]+=svd.U[j*m+l]*svd.S[l]*svd.VT[l*n+k];
		    }
		}
	    }
	    //System.err.println("m="+m+" n="+n+" S="+java.util.Arrays.toString(svd.S));
	    UnitTests.checkDoubleArrayRel1(USVT,"USVT",matr,"matr",eps);
	    UnitTests.checkDoubleArrayRel1(_USVT,"_USVT",matr,"matr",eps);
	    UnitTests.checkDoubleArrayRel1(TMatr.mulMM(m,svd.U,TMatr.getMatrTrans(m,svd.U)),"U*U^T",TMatr.getUnitMatrix(m),"1",eps);
	    UnitTests.checkDoubleArrayRel1(TMatr.mulMM(n,svd.VT,TMatr.getMatrTrans(n,svd.VT)),"VT*VT^T",TMatr.getUnitMatrix(n),"1",eps);
	}
    }
    }
    System.err.println("TestEigenvalues:test_getSVD(): N="+N+" n_tests="+n_tests+" total test="+nt+" eps="+eps+" OK");
}

}
