package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2021
 * This file is under GPL version 3.
 *
 */

/** A class with the data for unit test of vector to scalar RN model.
 * @see TestRN
 * @version $Id: TestObservationSequenceFromSampleVectorXF.java,v 1.17 2024/04/15 06:15:34 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
class TestObservationSequenceFromSampleVectorXF {
    private final DataReadObservationVectorXF ro;
    private TestObservationSequenceFromSampleVectorXF(RandomSample [] in,RandomSample f,RandomSample weight){
	final java.util.ArrayList<ObservationVectorXF> data=new java.util.ArrayList<>();
	final java.util.ArrayList<String> xHeader=new java.util.ArrayList<>(in.length);
	for(int m=0;m<in.length;m++){
	    xHeader.add("x"+m);
	    if(in[m].sample.length!=f.sample.length||f.sample.length!=weight.sample.length) throw new IllegalArgumentException();
	    }
	for(int k=0;k<f.sample.length;k++){
	    final double [] x=new double [in.length];
	    for(int m=0;m<in.length;m++){
		x[m]=in[m].sample[k];
	    }
	    //System.err.println("x="+java.util.Arrays.toString(x)+" weight.sample["+k+"]="+weight.sample[k]);
	    data.add(new ObservationVectorXF(String.valueOf(k),x,f.sample[k],weight.sample[k]));
	}
	this.ro=new DataReadObservationVectorXF(data,xHeader);
    }

    static XXMatrs getPsiAverageFromSample(final java.util.function.Function<double [],double []> convertxFromOriginalToX,final double [] ev,final DataReadObservationVectorXF ro){
	final int nX=convertxFromOriginalToX.apply(new double [ro.xHeader.size()]).length;
	final double [] psi_average=new double [nX],fpsi_average=new double [nX];
	for(ObservationVectorXF o:ro.data)
	{
	    final double [] X=convertxFromOriginalToX.apply(o.x);
	    //System.err.println("X.length="+X.length+" nX="+nX+" X="+java.util.Arrays.toString(X));
	    if(X.length!=nX) throw new IllegalArgumentException();
	    for(int k=0;k<nX;k++){
		final double psiAtX=TMatr.mulVV(nX,ev,nX+k*nX,X,0);
		psi_average[k]+=psiAtX*o.weight;
		fpsi_average[k]+=psiAtX*o.weight*o.f;
	    }
	}
	return new XXMatrs(1,nX,fpsi_average,psi_average);
    }
    /** Calculate f^0,f^1,f^2.
     */
    static double [] getfMoments(final DataReadObservationVectorXF ro){
	double f0=0,f1=0,f2=0;
	for(ObservationVectorXF o:ro.data)
	{
	    f0+=o.weight;
	    f1+=o.f*o.weight;
	    f2+=o.f*o.f*o.weight;
	}
	return new double []{f0,f1,f2};
    }
    /** Calculate {@literal <(f-sum_k lambda_k <psik> psik(x))^s> s=0,1,2}.
     */
    static double [] getStdevExpansion(RadonNikodymSpectralModel rn,final DataReadObservationVectorXF ro){
	double f0=0,f1=0,f2=0;
	for(ObservationVectorXF o:ro.data)
	{
	    final double df=o.f-TMatr.mulVVV(rn.f,rn.psiaverage,rn.getRNatXoriginal(o.x).getPsikAtX());
	    f0+=o.weight;
	    f1+=df*o.weight;
	    f2+=df*df*o.weight;
	}
	return new double []{f0,f1,f2};
    }


    static DataReadObservationVectorXF getNoNDegenerated(final java.util.Random r,final int size){
	// default dimension 17
	return getNoNDegenerated(17,r,size);
    }
    static DataReadObservationVectorXF getNoNDegenerated(
	final int n,
	final java.util.Random r,
	final int size){
	final RandomSample [] in=new RandomSample[n];
	for(int m=0;m<in.length;m++){
	    if(m<=0) in[m]=new RandomSample(1,1,size,r); // const
	    else in[m]=new RandomSample(-3*r.nextDouble(),3*r.nextDouble(),size,r);
	}
	return new TestObservationSequenceFromSampleVectorXF(in,new RandomSample(7,size,r),new RandomSample(0.01,3,size,r)).ro;
    }


static DataReadObservationVectorXF makeSampleDegenerate(
    final DataReadObservationVectorXF ro,
    final int nToAddX,
    final int nSumX,
    final java.util.Random r){

    final int [][] xInd=new int [nToAddX][nSumX];
    final double [][] xCoef=new double [nToAddX][nSumX];
    final java.util.ArrayList<String> xHeader=new java.util.ArrayList<>();
    xHeader.addAll(ro.xHeader);
    final int nX=ro.xHeader.size();
    for(int j=0;j<xInd.length;j++){
	String s="";
	for(int k=0;k<nSumX;k++){
	    xInd[j][k]=r.nextInt(nX);
	    xCoef[j][k]=5.2*(1-2*r.nextDouble());
	    s+=(j>0?"+":"")+"x["+xInd[j][k]+"]*"+xCoef[j][k];
	}
	xHeader.add(s);
    }
    final var data=new java.util.ArrayList<ObservationVectorXF>(10+ro.data.size());
    for(var o:ro.data){
	final double [] x=new double [nX+xInd.length];
	System.arraycopy(o.x,0,x,0,nX);
	for(int ns=nX,j=0;j<xInd.length;j++,ns++){
	    for(int k=0;k<nSumX;k++){
		x[ns]+=x[xInd[j][k]]*xCoef[j][k];
	    }
	}
	data.add(new ObservationVectorXF(o.label,x,o.f,o.weight));
    }
    return new DataReadObservationVectorXF(data,xHeader);
}

    static DegAndNDeg<DataReadObservationVectorXF>
	getDegenerated_VectorX(final int n,final java.util.Random r,final int size){
	final DataReadObservationVectorXF ro=getNoNDegenerated(n,r,size),
	    ro_deg=makeSampleDegenerate(ro,6,12,r);
	return new DegAndNDeg<DataReadObservationVectorXF>(ro,ro_deg);
    }

} // END OF class TestObservationSequenceFromSampleVectorXF
