package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2019
 * This file is under GPL version 3.
 *
 */

/** A unit test for optimization problems such as
  {@link IstatesConditionalV2},
  {@link IstatesConditionalLocalized}
  {@link MaximizeStandardDeviation}
  {@link IstatesConditional}.
 *  @version $Id: TestOptimizationPsi.java,v 1.8 2022/07/19 08:36:33 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class TestOptimizationPsi {
    final int n,sampleSize;
    final double eps;
    final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
    final double [] QQt,QQi,QQpi,QQp2i,QQpt,QQp2t;
    TestOptimizationPsi(final int n,
			final int sampleSize,
			final java.util.function.Consumer<TestOptimizationPsi> runTest,
			final java.util.Random r,
			final double eps){
	this.n=n;
	this.sampleSize=sampleSize;
	this.M=new OrthogonalPolynomialsChebyshevBasis();
	this.eps=eps;

	// observations sample
	final double []
	    x=TMatr.getRandomVector(sampleSize,r),
	    p=TMatr.getRandomVector(sampleSize,r),
	    wI=TMatr.getRandomVector(sampleSize,r,5,5),
	    wt=TMatr.getRandomVector(sampleSize,r,1,1.0001);

	// sample moments
	final double []
	    bt=M.B.getMomentsFromSample(2*n-1,x,wt),
	    bpt=M.B.getMomentsFromSample(2*n-1,x,TMatr.getVAB(wt,p)),
	    bp2t=M.B.getMomentsFromSample(2*n-1,x,TMatr.getVAB(wt,TMatr.squareArrayElements(p))),
	    bI=M.B.getMomentsFromSample(2*n-1,x,wI),
	    bpI=M.B.getMomentsFromSample(2*n-1,x,TMatr.getVAB(wI,p)),
	    bp2I=M.B.getMomentsFromSample(2*n-1,x,TMatr.getVAB(wI,TMatr.squareArrayElements(p)));
	System.err.println("TestOptimizationPsi: average p="+(bpI[0]/bI[0])+" stdev2 p="+(bp2I[0]/bI[0]-Math.pow(bpI[0]/bI[0],2)));

	// matrices
	this.QQt=M.B.getQQMatr(n,bt);
	this.QQpt=M.B.getQQMatr(n,bpt);
	this.QQp2t=M.B.getQQMatr(n,bp2t);
	this.QQi=M.B.getQQMatr(n,bI);
	this.QQpi=M.B.getQQMatr(n,bpI);
	this.QQp2i=M.B.getQQMatr(n,bp2I);
    }


    static void runMultiTest(final String name,
			     final int maxN,
			     final int sampleInitSize,
			     final java.util.Random r,
			     final double eps,
			     final java.util.function.Consumer<TestOptimizationPsi> runTest){
	System.err.println(name+":TestOptimizationPsi:starting tests; ver=$Id: TestOptimizationPsi.java,v 1.8 2022/07/19 08:36:33 mal Exp $");
	final long t0=System.currentTimeMillis();
	for(int n=1;n<=maxN;n++){
	    for(int tNum=0;tNum<500;tNum++){
		runTest.accept(new TestOptimizationPsi(n,sampleInitSize+r.nextInt(100),runTest,r,eps));
	    }
	}
	final long t1=System.currentTimeMillis();
	System.err.println(name+":TestOptimizationPsi: tests OK done in "+(t1-t0)+" milliseconds. maxN="+maxN+" ver=$Id: TestOptimizationPsi.java,v 1.8 2022/07/19 08:36:33 mal Exp $");
    }

}




