package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/* Test Polynomial roots.
 *
 *  @version $Id: TestPolynomialRoots.java,v 1.13 2022/07/19 09:01:28 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */

public class TestPolynomialRoots  {

    private final PolynomialRootsFinder rootfinder;
    private final double eps;
    public TestPolynomialRoots(final PolynomialRootsFinder rootfinder,
			       final double eps)
    {
	this.rootfinder=rootfinder;
	this.eps=eps;
    }

    public void testPolynomialFromFile(final String fname) throws java.io.IOException
    {
	System.err.println("Reading polynomial from "+fname);
	final double [] p=readPolynomialFromFile(fname);
	final int n=p.length-1;
	final PolynomialRootsFinder.PRoots r=testPolynom(p);
	System.err.println("N="+n+" read");
	for(int i=p.length;--i>=0;)
	{
	    System.err.println("pcoefs("+i+")="+p[i]+" read");
	}
	System.err.println("ROOTS:");
	for(int i=0;i<n;i++)
	{
	    System.err.println(r.re_root[i]+"\t"+r.im_root[i]);
	}
    }

    public double [] readPolynomialFromFile(final String fname) throws java.io.IOException
    {
	final java.io.LineNumberReader fr=new java.io.LineNumberReader(new java.io.FileReader(fname));
	final int n=Integer.valueOf(fr.readLine());
	final double [] p=new double [n+1];
	for(int i=p.length;--i>=0;)
	{
	    p[i]=Double.valueOf(fr.readLine());
	}
	fr.close();
	return p;
    }

    public PolynomialRootsFinder.PRoots testPolynom(final double [] p)
    {
	final PolynomialRootsFinder.PRoots r=rootfinder.findRoots(p);
	final int n=r.Pcoefs.length-1;
	if(r.nroots!=n)
	{
	    System.err.println("Some roots were not found :"+PolynomialRootsElementary.getPolinomTXT(p)+"\nr.nroots="+r.nroots+" n="+n+"\t"+
			       "Roots:\n"+PolynomialRootsElementary.getRootsTXT(r.nroots,r.re_root,r.im_root));
	    throw new RuntimeException(PolynomialRootsElementary.getPolinomTXT(p)+"\nr.nroots="+r.nroots+" n="+n);
	}

	for(int j=0;j<r.nroots;j++)
	{
	    //System.err.println("j="+j);
	    PolynomialRootsElementary.checkValues(p,n,r.re_root[j],r.im_root[j],eps,"test"/* " t="+i+" roots:"+PolynomialRootsElementary.getRootsTXT(r.nroots,r.re_root,r.im_root)*/);
	}
	return r;
    }

    public void testRootsRandomPolynomial(final int n,
					  final int n_tests,
					  final java.util.Random rn)
    {
	System.err.println(rootfinder.getClass().getSimpleName()+":TestPolynomialRoots:starting tests for n="+n+"; ver=$Id: TestPolynomialRoots.java,v 1.13 2022/07/19 09:01:28 mal Exp $");
	final long t0=System.currentTimeMillis();

	final double [] p=new double [n+1];
	for(int i=0;i<n_tests;i++)
	{
	  for(int dg=n;dg-->-1;)
	  {
	    for(int dr=3;dr-->0;)
	    {
	      PolynomialRootsElementary.setRandomP(p,n,rn);
	      for(int j=0;j<=dg;j++)
	      {
		  p[j]=0;
	      }
	      if(dr==0)
	      {
		  p[0]=-1+2.0*rn.nextDouble();
	      }
	      else if(dr==1)
	      {
		  p[0]=p[1]=0;
	      }

	      testPolynom(p);
	    }
	  }
	}

	final long t1=System.currentTimeMillis();
	System.err.println(rootfinder.getClass().getSimpleName()+":TestPolynomialRoots:"+n_tests+" tests OK done in "+(t1-t0)+" milliseconds. n="+n+" ver=$Id: TestPolynomialRoots.java,v 1.13 2022/07/19 09:01:28 mal Exp $");
    }
}
