package com.polytechnik.utils;

/** Three term recurrence coefficients.
 * The definitions as in
 * Walter Gautschi, "Orthogonal Polynomials:Computation and approximation", page
 12.
 * Here a[k]=sqrt(beta_k) and b[k]=alfa_k;
 * the recurrence is the same as in {@link RecurrenceAB#getConfederateMatrix RecurrenceAB#getConfederateMatrix}:
 *
\[    x*p_{k}=a_{k+1}p_{k+1}+b_{k}p_{k}+a_{k}p_{k-1} \]
 *
 * @version $Id: ThreeTermRecurrenceCalculatable.java,v 1.6 2021/11/27 12:02:56 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public interface ThreeTermRecurrenceCalculatable {

    double getAk(int k);
    double getBk(int k);
    /** The number of {@code a_k} available, there is one less {@code b_k} available.
     */
    int numAk();
}
