package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Unit test utils.
 *  @version $Id: UnitTests.java,v 1.57 2024/05/12 23:41:55 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class UnitTests {

    /** A helper interface for array checking.
     */
    @FunctionalInterface private interface TwoValuesCheckMatching {
	boolean checkMatching(double v1,double v2,double eps);
    }


    public static void checkDouble(final double v1,final String s1,
				   final double v2,final String s2)
    {
	if(Double.isNaN(v1)!=Double.isNaN(v2)||
	   (!Double.isNaN(v1) && v2!=v1))
	{
	    throw new RuntimeException("Mismatch: "+s1+"="+v1+" "+s2+"="+v2);
	}
    }

    public static boolean checkDoubleNoFail(final double v1,final double v2,final double eps)
    {
	if(Double.isNaN(v1)!=Double.isNaN(v2)||
	   (!Double.isNaN(v1) && Math.abs(v2-v1)>eps))
	{
	    return true;
	}
	return false;
    }

    public static void checkDouble(final double v1,final String s1,
				   final double v2,final String s2,
				   final double eps)
    {
	if(checkDoubleNoFail(v1,v2,eps))
	{
	    throw new RuntimeException("Mismatch: "+s1+"="+v1+" "+s2+"="+v2+" eps="+eps);
	}
    }

    public static void checkDoubleRel(final double v1,final String s1,
				      final double v2,final String s2,
				      final double eps)
    {
	if(checkDoubleRelNoFail(v1,v2,eps))
	{
	    throw new RuntimeException("Mismatch: "+s1+"="+v1+" "+s2+"="+v2+" eps="+eps);
	}
    }

    public static boolean checkDoubleRelNoFail(final double v1,final double v2,final double eps)
    {
	if(v1==0 || v2==0 /* ||Math.abs(v1)<1 */)
	{
	    return checkDoubleNoFail(v1,v2,eps);
	}
	else if(Double.isNaN(v1)!=Double.isNaN(v2)||
		(!Double.isNaN(v1) && Math.abs((v2-v1)/Math.max(Math.abs(v1),Math.abs(v2)))>eps))
	{
	    return true;
	}
	return false;
    }

    public static void checkDoubleRel1(final double v1,final String s1,
				       final double v2,final String s2,
				       final double eps)
    {
	if(checkDoubleRel1NoFail(v1,v2,eps))
	{
	    throw new RuntimeException("Mismatch: "+s1+"="+v1+" "+s2+"="+v2+" eps="+eps);
	}
    }

    /** If absolute value of {@code v1} is under {@code 1}
     * use {@link #checkDoubleNoFail} otherwise
     * use {@link #checkDoubleRelNoFail}.
     */
    public static boolean checkDoubleRel1NoFail(final double v1,final double v2,final double eps) {
	if(Double.isNaN(v1)!=Double.isNaN(v2))
	{
	    return true;
	}
	else if(v1==0 || v2==0 || Math.abs(v1)<1 )
	{
	    return checkDoubleNoFail(v1,v2,eps);
	}
	else
	{
	    return checkDoubleRelNoFail(v1,v2,eps);
	}
    }

private static void checkDoubleArrays(final String descr,
				      final TwoValuesCheckMatching check,
				      final double [] av1,final String s1,
				      final double [] av2,final String s2,
				      final double eps)
{
    if(av1.length!=av2.length)
    {
	throw new RuntimeException("Mismatch: "+s1+".length="+av1.length+" "+s2+".length="+av2.length);
    }

    final int len=av1.length;
    boolean f=false;
    final StringBuilder buf=new StringBuilder();
    for(int k=0;k<len;k++)
    {
	if(check.checkMatching(av1[k],av2[k],eps))
	{
	    f=true;
	    buf.append("mismatch:ind="+k+"\t"+s1+"["+k+"]="+av1[k]+"\t"+s2+"["+k+"]="+av2[k]+"\n");
	}
    }
    if(f)
    {
	final StringBuilder s=new StringBuilder();
	s.append(buf.toString());
	s.append("\n");
	for(int i=0;i<len;i++)
	{
	    s.append(s1+"["+i+"]="+av1[i]+"\t"+s2+"["+i+"]="+av2[i]+"\n");
	}
	throw new RuntimeException("Mismatch("+descr+"): array values mismatch, len="+len+" eps="+eps+"\n"+s.toString());
    }
}

    public static void checkDoubleArrayRel(final double [] av1,final String s1,
					   final double [] av2,final String s2,
					   final double eps){
	checkDoubleArrays("Rel",UnitTests::checkDoubleRelNoFail,av1,s1,av2,s2,eps);
    }

    public static void checkDoubleArrayRel1(final double [] av1,final String s1,
					    final double [] av2,final String s2,
					    final double eps)
    {
	checkDoubleArrays("Rel1",UnitTests::checkDoubleRel1NoFail,av1,s1,av2,s2,eps);
    }


public static void main(String [] args)
{
 System.err.println("UnitTests::main(): starting ALL tests; ver=$Id: UnitTests.java,v 1.57 2024/05/12 23:41:55 mal Exp $");
 final long t0=System.currentTimeMillis();
 System.err.println("Version");
 System.err.println("Started="+new java.util.Date(t0));
 System.getProperties().list(System.err);
 System.err.println("END Version");

 CircularList.main(args);

 // test the bases
 Laguerre.main(args);
 Chebyshev.main(args);
 Legendre.main(args);
 HermiteE.main(args);
 LegendreShifted.main(args);
 RecurrenceAB.main(args);
 RecurrenceABWithMultiplicationCached.main(args);
 Monomials.main(args);

 // test utilites
 Linsystems.main(args);
 LongHashMap.main(args);
 LongTreeValueMapTest.main(args);
 PolynomialRootsElementary.main(args);
 PolynomialRootsJenkinsTraubORIG.main(new String [0]);
 Polynomials.main(args);
 Progonka3.main(args);
 Progonka5.main(args);
 TMatr.main(args);
 TestRN.main(args);
 SortArrays.main(args);
 QUANC8.main(args);

 // orthogonal polynomials in a variety of bases
 OrthogonalPolynomialsLegendreShiftedBasis.main(args);
 OrthogonalPolynomialsLaguerreBasis.main(args);
 OrthogonalPolynomialsMonomialsBasis.main(args);
 OrthogonalPolynomialsChebyshevBasis.main(args);
 OrthogonalPolynomialsLegendreBasis.main(args);
 OrthogonalPolynomialsHermiteEBasis.main(args);
 OrthogonalPolynomialsRecurrenceABBasis.main(args);

 // optimization
 EliminateLinearConstraints_HomegrownLUFactorization.main(args);
 MaximizeStandardDeviation.main(args);
 IstatesConditionalSubspaceLinearConstraints.main(args); // best version with an extra linear constraint
 IstatesConditionalV2.main(args); // new version
 IstatesConditional.main(args); // old version
 IstatesConditionalLocalized.main(args);
 IstatesConditionalSameAverage.main(args);

 // misc
 RayleighQuotient.main(args);
 PolynomialRootsConfederateMatrix.main(args);


 final long t1=System.currentTimeMillis();
 System.err.println("Version");
 System.err.println("Finished="+new java.util.Date(t1)+" Started="+new java.util.Date(t0));
 System.getProperties().list(System.err);
 System.err.println("END Version");
 System.err.println("UnitTests::main(): ALL tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: UnitTests.java,v 1.57 2024/05/12 23:41:55 mal Exp $");
}


}
