package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2021
 * This file is under GPL version 3.
 *
 */

/** An interface to access a single point of a vector  X to Vector F model.
 * @version $Id: VectorXVectorFPointAccess.java,v 1.21 2022/11/06 17:22:57 mal Exp $
 * @author Vladislav Malyshkin mal@gromco.com
 */
public interface VectorXVectorFPointAccess {

    /** An evaluation of conditional probability.
     */
    double getFProbabilityAtX(final double [] forig);

    /** Like a 1/K(x).
     */
    double getChristoffelOatX();

    /** A regular 1/K(F).
     */
    double getChristoffelOatF(final double [] forig);

    /** The maximal value of probability, corresponds to the only non--zero eigenvalue.
     */
    double getMaximalProbability();

    /** The state psi(F) giving maximal probability.
     * @return psi(F) in regularized basis.
     */
    double [] getPsiFNonNormalized();


    /** Least squares approximation of {@code forig} (for unit tests).
     * @return The value in original basis (forig).
     */
    double [] getFLeastSquaresX();


    /** The value of {@code forig} corresponding to the maximal probability.
     * @return The value in original basis (forig).
     */
    double [] getFofMaximalProbability();

}
