/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlaqr5.f
 *  f2java version: 0.8.1
 *
 */

package com.polytechnik.lapack310;


public class Dlaqr5 {

// *> \brief \b DLAQR5 performs a single small-bulge multi-shift QR sweep.

// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at
// *            http://www.netlib.org/lapack/explore-html/
// *
// *> \htmlonly
// *> Download DLAQR5 + dependencies
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a>
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAQR5( WANTT, WANTZ, KACC22, N, KTOP, KBOT, NSHFTS,

// *                          SR, SI, H, LDH, ILOZ, IHIZ, Z, LDZ, V, LDV, U
// *                          LDU, NV, WV, LDWV, NH, WH, LDWH )
// *
// *       .. Scalar Arguments ..
// *       INTEGER            IHIZ, ILOZ, KACC22, KBOT, KTOP, LDH, LDU, LDV
// *      $                   LDWH, LDWV, LDZ, N, NH, NSHFTS, NV
// *       LOGICAL            WANTT, WANTZ
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   H( LDH, * ), SI( * ), SR( * ), U( LDU, * ),
// *      $                   V( LDV, * ), WH( LDWH, * ), WV( LDWV, * ),
// *      $                   Z( LDZ, * )
// *       ..
// *
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *>    DLAQR5, called by DLAQR0, performs a
// *>    single small-bulge multi-shift QR sweep.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] WANTT
// *> \verbatim
// *>          WANTT is LOGICAL
// *>             WANTT = .true. if the quasi-triangular Schur factor
// *>             is being computed.  WANTT is set to .false. otherwise.
// *> \endverbatim
// *>
// *> \param[in] WANTZ
// *> \verbatim
// *>          WANTZ is LOGICAL
// *>             WANTZ = .true. if the orthogonal Schur factor is being
// *>             computed.  WANTZ is set to .false. otherwise.
// *> \endverbatim
// *>
// *> \param[in] KACC22
// *> \verbatim
// *>          KACC22 is INTEGER with value 0, 1, or 2.
// *>             Specifies the computation mode of far-from-diagonal
// *>             orthogonal updates.
// *>        = 0: DLAQR5 does not accumulate reflections and does not
// *>             use matrix-matrix multiply to update far-from-diagonal
// *>             matrix entries.
// *>        = 1: DLAQR5 accumulates reflections and uses matrix-matrix
// *>             multiply to update the far-from-diagonal matrix entries.

// *>        = 2: Same as KACC22 = 1. This option used to enable exploiting
// *>             the 2-by-2 structure during matrix multiplications, but
// *>             this is no longer supported.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>             N is the order of the Hessenberg matrix H upon which this
// *>             subroutine operates.
// *> \endverbatim
// *>
// *> \param[in] KTOP
// *> \verbatim
// *>          KTOP is INTEGER
// *> \endverbatim
// *>
// *> \param[in] KBOT
// *> \verbatim
// *>          KBOT is INTEGER
// *>             These are the first and last rows and columns of an
// *>             isolated diagonal block upon which the QR sweep is to be

// *>             applied. It is assumed without a check that
// *>                       either KTOP = 1  or   H(KTOP,KTOP-1) = 0
// *>             and
// *>                       either KBOT = N  or   H(KBOT+1,KBOT) = 0.
// *> \endverbatim
// *>
// *> \param[in] NSHFTS
// *> \verbatim
// *>          NSHFTS is INTEGER
// *>             NSHFTS gives the number of simultaneous shifts.  NSHFTS
// *>             must be positive and even.
// *> \endverbatim
// *>
// *> \param[in,out] SR
// *> \verbatim
// *>          SR is DOUBLE PRECISION array, dimension (NSHFTS)
// *> \endverbatim
// *>
// *> \param[in,out] SI
// *> \verbatim
// *>          SI is DOUBLE PRECISION array, dimension (NSHFTS)
// *>             SR contains the real parts and SI contains the imaginary

// *>             parts of the NSHFTS shifts of origin that define the
// *>             multi-shift QR sweep.  On output SR and SI may be
// *>             reordered.
// *> \endverbatim
// *>
// *> \param[in,out] H
// *> \verbatim
// *>          H is DOUBLE PRECISION array, dimension (LDH,N)
// *>             On input H contains a Hessenberg matrix.  On output a
// *>             multi-shift QR sweep with shifts SR(J)+i*SI(J) is applied
// *>             to the isolated diagonal block in rows and columns KTOP
// *>             through KBOT.
// *> \endverbatim
// *>
// *> \param[in] LDH
// *> \verbatim
// *>          LDH is INTEGER
// *>             LDH is the leading dimension of H just as declared in the
// *>             calling procedure.  LDH >= MAX(1,N).
// *> \endverbatim
// *>
// *> \param[in] ILOZ
// *> \verbatim
// *>          ILOZ is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IHIZ
// *> \verbatim
// *>          IHIZ is INTEGER
// *>             Specify the rows of Z to which transformations must be
// *>             applied if WANTZ is .TRUE.. 1 <= ILOZ <= IHIZ <= N
// *> \endverbatim
// *>
// *> \param[in,out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (LDZ,IHIZ)
// *>             If WANTZ = .TRUE., then the QR Sweep orthogonal
// *>             similarity transformation is accumulated into
// *>             Z(ILOZ:IHIZ,ILOZ:IHIZ) from the right.
// *>             If WANTZ = .FALSE., then Z is unreferenced.
// *> \endverbatim
// *>
// *> \param[in] LDZ
// *> \verbatim
// *>          LDZ is INTEGER
// *>             LDA is the leading dimension of Z just as declared in
// *>             the calling procedure. LDZ >= N.
// *> \endverbatim
// *>
// *> \param[out] V
// *> \verbatim
// *>          V is DOUBLE PRECISION array, dimension (LDV,NSHFTS/2)
// *> \endverbatim
// *>
// *> \param[in] LDV
// *> \verbatim
// *>          LDV is INTEGER
// *>             LDV is the leading dimension of V as declared in the
// *>             calling procedure.  LDV >= 3.
// *> \endverbatim
// *>
// *> \param[out] U
// *> \verbatim
// *>          U is DOUBLE PRECISION array, dimension (LDU,2*NSHFTS)
// *> \endverbatim
// *>
// *> \param[in] LDU
// *> \verbatim
// *>          LDU is INTEGER
// *>             LDU is the leading dimension of U just as declared in the
// *>             in the calling subroutine.  LDU >= 2*NSHFTS.
// *> \endverbatim
// *>
// *> \param[in] NV
// *> \verbatim
// *>          NV is INTEGER
// *>             NV is the number of rows in WV agailable for workspace.
// *>             NV >= 1.
// *> \endverbatim
// *>
// *> \param[out] WV
// *> \verbatim
// *>          WV is DOUBLE PRECISION array, dimension (LDWV,2*NSHFTS)
// *> \endverbatim
// *>
// *> \param[in] LDWV
// *> \verbatim
// *>          LDWV is INTEGER
// *>             LDWV is the leading dimension of WV as declared in the
// *>             in the calling subroutine.  LDWV >= NV.
// *> \endverbatim
// *
// *> \param[in] NH
// *> \verbatim
// *>          NH is INTEGER
// *>             NH is the number of columns in array WH available for
// *>             workspace. NH >= 1.
// *> \endverbatim
// *>
// *> \param[out] WH
// *> \verbatim
// *>          WH is DOUBLE PRECISION array, dimension (LDWH,NH)
// *> \endverbatim
// *>
// *> \param[in] LDWH
// *> \verbatim
// *>          LDWH is INTEGER
// *>             Leading dimension of WH just as declared in the
// *>             calling procedure.  LDWH >= 2*NSHFTS.
// *> \endverbatim
// *>
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee
// *> \author Univ. of California Berkeley
// *> \author Univ. of Colorado Denver
// *> \author NAG Ltd.
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *> \par Contributors:
// *  ==================
// *>
// *>       Karen Braman and Ralph Byers, Department of Mathematics,
// *>       University of Kansas, USA
// *>
// *>       Lars Karlsson, Daniel Kressner, and Bruno Lang
// *>
// *>       Thijs Steel, Department of Computer science,
// *>       KU Leuven, Belgium
// *
// *> \par References:
// *  ================
// *>
// *>       K. Braman, R. Byers and R. Mathias, The Multi-Shift QR
// *>       Algorithm Part I: Maintaining Well Focused Shifts, and Level 3

// *>       Performance, SIAM Journal of Matrix Analysis, volume 23, pages

// *>       929--947, 2002.
// *>
// *>       Lars Karlsson, Daniel Kressner, and Bruno Lang, Optimally packe
// *>       chains of bulges in multishift QR algorithms.
// *>       ACM Trans. Math. Softw. 40, 2, Article 12 (February 2014).
// *>
// *  =====================================================================
/***
*
*  -- LAPACK auxiliary routine --
*  -- LAPACK is a software package provided by Univ. of Tennessee,    --
*  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
*
*     .. Scalar Arguments ..
***/
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  ================================================================
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
*
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     ==== If there are no shifts, then there is nothing to do. ====
*
***/
/***
*
*     ==== If the active block is empty or 1-by-1, then there
*     .    is nothing to do. ====
*
***/

public static void dlaqr5 (boolean wantt,
boolean wantz,
int kacc22,
int n,
int ktop,
int kbot,
int nshfts,
double [] sr, int _sr_offset,
double [] si, int _si_offset,
double [] h, int _h_offset,
int ldh,
int iloz,
int ihiz,
double [] z, int _z_offset,
int ldz,
double [] v, int _v_offset,
int ldv,
double [] u, int _u_offset,
int ldu,
int nv,
double [] wv, int _wv_offset,
int ldwv,
int nh,
double [] wh, int _wh_offset,
int ldwh)  {

label_999999:{

doubleW alpha= new doubleW(0.0d);
doubleW beta= new doubleW(0.0d);
double h11= 0.0d;
double h12= 0.0d;
double h21= 0.0d;
double h22= 0.0d;
double refsum= 0.0d;
doubleW safmax= new doubleW(0.0d);
doubleW safmin= new doubleW(0.0d);
double scl= 0.0d;
double smlnum= 0.0d;
double swap= 0.0d;
double tst1= 0.0d;
double tst2= 0.0d;
double ulp= 0.0d;
int i= 0;
int i2= 0;
int i4= 0;
int incol= 0;
int j= 0;
int jbot= 0;
int jcol= 0;
int jlen= 0;
int jrow= 0;
int jtop= 0;
int k= 0;
int k1= 0;
int kdu= 0;
int kms= 0;
int krcol= 0;
int m= 0;
int m22= 0;
int mbot= 0;
int mtop= 0;
int nbmps= 0;
int ndcol= 0;
int ns= 0;
int nu= 0;
boolean accum= false;
boolean bmp22= false;
double [] vt= new double[(3)];
if ((nshfts < 2)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlaqr5",999999);
}
    /***
*
*     ==== Shuffle shifts into pairs of real shifts and pairs
*     .    of complex conjugate shifts assuming complex
*     .    conjugate shifts are already adjacent to one
*     .    another. ====
*
***/
if ((ktop >= kbot)) {
    break label_999999;//Dummy.go_to("com/polytechnik/lapack310/Dlaqr5",999999);
}
    {
int _i_inc = 2;
for (i = 1; i <= (nshfts-2); i += _i_inc) {
// *
if ((si[(i-(1))+ _si_offset] != (-(si[((i+1)-(1))+ _si_offset]))))  {
    swap = sr[(i-(1))+ _sr_offset];
sr[(i-(1))+ _sr_offset] = sr[((i+1)-(1))+ _sr_offset];
sr[((i+1)-(1))+ _sr_offset] = sr[((i+2)-(1))+ _sr_offset];
// *
sr[((i+2)-(1))+ _sr_offset] = swap;
swap = si[(i-(1))+ _si_offset];
si[(i-(1))+ _si_offset] = si[((i+1)-(1))+ _si_offset];
si[((i+1)-(1))+ _si_offset] = si[((i+2)-(1))+ _si_offset];
si[((i+2)-(1))+ _si_offset] = swap;
}
/***
*
*     ==== NSHFTS is supposed to be even, but if it is odd,
*     .    then simply reduce it by one.  The shuffle above
*     .    ensures that the dropped shift is real and that
*     .    the remaining shifts are paired. ====
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",10);
}              //  Close for() loop.
}
/***
*
*     ==== Machine constants for deflation ====
*
***/
ns = (nshfts-(nshfts)%(2));
safmin.val = com.polytechnik.lapack310.Dlamch.dlamch("SAFE MINIMUM");
safmax.val = (1.0/safmin.val);
com.polytechnik.lapack310.Dlabad.dlabad(safmin,safmax);
ulp = com.polytechnik.lapack310.Dlamch.dlamch("PRECISION");
/***
*
*     ==== Use accumulated reflections to update far-from-diagonal
*     .    entries ? ====
*
***/
smlnum = (safmin.val*(((double)(n)/ulp)));
/***
*
*     ==== clear trash ====
*
***/
accum = (((kacc22 == 1)) || ((kacc22 == 2)));
/***
*
*     ==== NBMPS = number of 2-shift bulges in the chain ====
*
***/
if (((ktop+2) <= kbot)) {
    h[((ktop+2)-(1))+(ktop-(1)) * (ldh)+ _h_offset] = 0.0;
}
    /***
*
*     ==== KDU = width of slab ====
*
***/
nbmps = (ns/2);
/***
*
*     ==== Create and chase chains of NBMPS bulges ====
*
***/
kdu = (4*nbmps);
/***
*
*        JTOP = Index from which updates from the right start.
*
***/
{
int _incol_inc = (2*nbmps);
for (incol = ((ktop-(2*nbmps))+1); (_incol_inc < 0) ? incol >= (kbot-2) : incol <= (kbot-2); incol += _incol_inc) {
if (accum)  {
    jtop = Math.max(ktop, incol);
}
else if (wantt)  {
    jtop = 1;
}              // Close else if()
else  {
  jtop = ktop;
// *
}              //  Close else.
ndcol = (incol+kdu);
/***
*
*        ==== Near-the-diagonal bulge chase.  The following loop
*        .    performs the near-the-diagonal part of a small bulge
*        .    multi-shift QR sweep.  Each 4*NBMPS column diagonal
*        .    chunk extends from column INCOL to column NDCOL
*        .    (including both column INCOL and column NDCOL). The
*        .    following loop chases a 2*NBMPS+1 column long chain of
*        .    NBMPS bulges 2*NBMPS columns to the right.  (INCOL
*        .    may be less than KTOP and and NDCOL may be greater than
*        .    KBOT indicating phantom columns from which to chase
*        .    bulges before they are actually introduced or to which
*        .    to chase bulges beyond column KBOT.)  ====
*
***/
if (accum) {
    com.polytechnik.lapack310.Dlaset.dlaset("ALL",kdu,kdu,0.0,1.0,u,_u_offset,ldu);
}
    /***
*
*           ==== Bulges number MTOP to MBOT are active double implicit
*           .    shift bulges.  There may or may not also be small
*           .    2-by-2 bulge, if there is room.  The inactive bulges
*           .    (if any) must wait until the active bulges have moved
*           .    down the diagonal to make room.  The phantom matrix
*           .    paradigm described above helps keep track.  ====
*
***/
{
for (krcol = incol; krcol <= Math.min(((incol+(2*nbmps))-1), (kbot-2)); krcol++) {
mtop = Math.max(1, ((((ktop-krcol))/2)+1));
mbot = Math.min(nbmps, ((((kbot-krcol)-1))/2));
m22 = (mbot+1);
/***
*
*           ==== Generate reflections to chase the chain right
*           .    one column.  (The minimum value of K is KTOP-1.) ====
*
***/
bmp22 = (((mbot < nbmps)) && (((krcol+(2*((m22-1))))) == ((kbot-2))));
/***
*
*              ==== Special case: 2-by-2 reflection at bottom treated
*              .    separately ====
*
***/
if (bmp22)  {
    k = (krcol+(2*((m22-1))));
if ((k == (ktop-1)))  {
    com.polytechnik.lapack310.Dlaqr1.dlaqr1(2,h,((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset,ldh,sr[(((2*m22)-1)-(1))+ _sr_offset],si[(((2*m22)-1)-(1))+ _si_offset],sr[((2*m22)-(1))+ _sr_offset],si[((2*m22)-(1))+ _si_offset],v,(1-(1))+(m22-(1)) * (ldv)+ _v_offset);
beta.val = v[(1-(1))+(m22-(1)) * (ldv)+ _v_offset];
dlarfg_adapter(2,beta,v,(2-(1))+(m22-(1)) * (ldv)+ _v_offset,1,v,(1-(1))+(m22-(1)) * (ldv)+ _v_offset);
}
else  {
  beta.val = h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset];
v[(2-(1))+(m22-(1)) * (ldv)+ _v_offset] = h[((k+2)-(1))+(k-(1)) * (ldh)+ _h_offset];
dlarfg_adapter(2,beta,v,(2-(1))+(m22-(1)) * (ldv)+ _v_offset,1,v,(1-(1))+(m22-(1)) * (ldv)+ _v_offset);
h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset] = beta.val;
h[((k+2)-(1))+(k-(1)) * (ldh)+ _h_offset] = 0.0;
/***

*
*              ==== Perform update from right within
*              .    computational window. ====
*
***/
}              //  Close else.
{
for (j = jtop; j <= Math.min(kbot, (k+3)); j++) {
refsum = (v[(1-(1))+(m22-(1)) * (ldv)+ _v_offset]*((h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset]+(v[(2-(1))+(m22-(1)) * (ldv)+ _v_offset]*h[(j-(1))+((k+2)-(1)) * (ldh)+ _h_offset]))));
h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset] = (h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset]-refsum);
h[(j-(1))+((k+2)-(1)) * (ldh)+ _h_offset] = (h[(j-(1))+((k+2)-(1)) * (ldh)+ _h_offset]-(refsum*v[(2-(1))+(m22-(1)) * (ldv)+ _v_offset]));
/***
*
*              ==== Perform update from left within
*              .    computational window. ====
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",30);
}              //  Close for() loop.
}
if (accum)  {
    jbot = Math.min(ndcol, kbot);
}
else if (wantt)  {
    jbot = n;
}              // Close else if()
else  {
  jbot = kbot;
}              //  Close else.
{
for (j = (k+1); j <= jbot; j++) {
refsum = (v[(1-(1))+(m22-(1)) * (ldv)+ _v_offset]*((h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset]+(v[(2-(1))+(m22-(1)) * (ldv)+ _v_offset]*h[((k+2)-(1))+(j-(1)) * (ldh)+ _h_offset]))));
h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset] = (h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset]-refsum);
h[((k+2)-(1))+(j-(1)) * (ldh)+ _h_offset] = (h[((k+2)-(1))+(j-(1)) * (ldh)+ _h_offset]-(refsum*v[(2-(1))+(m22-(1)) * (ldv)+ _v_offset]));
/***
*
*              ==== The following convergence test requires that
*              .    the tradition small-compared-to-nearby-diagonals
*              .    criterion and the Ahues & Tisseur (LAWN 122, 1997)
*              .    criteria both be satisfied.  The latter improves
*              .    accuracy in some examples. Falling back on an
*              .    alternate convergence criterion when TST1 or TST2
*              .    is zero (as done here) is traditional but probably
*              .    unnecessary. ====
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",40);
}              //  Close for() loop.
}
if ((k >= ktop))  {
    if ((h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset] != 0.0))  {
    tst1 = (Math.abs(h[(k-(1))+(k-(1)) * (ldh)+ _h_offset])+Math.abs(h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
if ((tst1 == 0.0))  {
    if ((k >= (ktop+1))) {
    tst1 = (tst1+Math.abs(h[(k-(1))+((k-1)-(1)) * (ldh)+ _h_offset]));
}
    if ((k >= (ktop+2))) {
    tst1 = (tst1+Math.abs(h[(k-(1))+((k-2)-(1)) * (ldh)+ _h_offset]));
}
    if ((k >= (ktop+3))) {
    tst1 = (tst1+Math.abs(h[(k-(1))+((k-3)-(1)) * (ldh)+ _h_offset]));
}
    if ((k <= (kbot-2))) {
    tst1 = (tst1+Math.abs(h[((k+2)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
}
    if ((k <= (kbot-3))) {
    tst1 = (tst1+Math.abs(h[((k+3)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
}
    if ((k <= (kbot-4))) {
    tst1 = (tst1+Math.abs(h[((k+4)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
}
    }
if ((Math.abs(h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset]) <= Math.max(smlnum, (ulp*tst1))))  {
    h12 = Math.max(Math.abs(h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset]), Math.abs(h[(k-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
h21 = Math.min(Math.abs(h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset]), Math.abs(h[(k-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
h11 = Math.max(Math.abs(h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]), Math.abs((h[(k-(1))+(k-(1)) * (ldh)+ _h_offset]-h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset])));
h22 = Math.min(Math.abs(h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]), Math.abs((h[(k-(1))+(k-(1)) * (ldh)+ _h_offset]-h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset])));
scl = (h11+h12);
// *
tst2 = (h22*((h11/scl)));
if (((tst2 == 0.0) || ((h21*((h12/scl))) <= Math.max(smlnum, (ulp*tst2)))))  {
    h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset] = 0.0;
}
}
}
/***
*
*              ==== Accumulate orthogonal transformations. ====
*
***/
}
if (accum)  {
    kms = (k-incol);
{
for (j = Math.max(1, (ktop-incol)); j <= kdu; j++) {
refsum = (v[(1-(1))+(m22-(1)) * (ldv)+ _v_offset]*((u[(j-(1))+((kms+1)-(1)) * (ldu)+ _u_offset]+(v[(2-(1))+(m22-(1)) * (ldv)+ _v_offset]*u[(j-(1))+((kms+2)-(1)) * (ldu)+ _u_offset]))));
u[(j-(1))+((kms+1)-(1)) * (ldu)+ _u_offset] = (u[(j-(1))+((kms+1)-(1)) * (ldu)+ _u_offset]-refsum);
u[(j-(1))+((kms+2)-(1)) * (ldu)+ _u_offset] = (u[(j-(1))+((kms+2)-(1)) * (ldu)+ _u_offset]-(refsum*v[(2-(1))+(m22-(1)) * (ldv)+ _v_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",50);
}              //  Close for() loop.
}
}
else if (wantz)  {
    {
for (j = iloz; j <= ihiz; j++) {
refsum = (v[(1-(1))+(m22-(1)) * (ldv)+ _v_offset]*((z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset]+(v[(2-(1))+(m22-(1)) * (ldv)+ _v_offset]*z[(j-(1))+((k+2)-(1)) * (ldz)+ _z_offset]))));
z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset] = (z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset]-refsum);
z[(j-(1))+((k+2)-(1)) * (ldz)+ _z_offset] = (z[(j-(1))+((k+2)-(1)) * (ldz)+ _z_offset]-(refsum*v[(2-(1))+(m22-(1)) * (ldv)+ _v_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",60);
}              //  Close for() loop.
}
}              // Close else if()
/***
*
*           ==== Normal case: Chain of 3-by-3 reflections ====
*
***/
}
{
int _m_inc = -1;
label80:for (m = mbot; m >= mtop; m += _m_inc) {
k = (krcol+(2*((m-1))));
if ((k == (ktop-1)))  {
    com.polytechnik.lapack310.Dlaqr1.dlaqr1(3,h,(ktop-(1))+(ktop-(1)) * (ldh)+ _h_offset,ldh,sr[(((2*m)-1)-(1))+ _sr_offset],si[(((2*m)-1)-(1))+ _si_offset],sr[((2*m)-(1))+ _sr_offset],si[((2*m)-(1))+ _si_offset],v,(1-(1))+(m-(1)) * (ldv)+ _v_offset);
alpha.val = v[(1-(1))+(m-(1)) * (ldv)+ _v_offset];
dlarfg_adapter(3,alpha,v,(2-(1))+(m-(1)) * (ldv)+ _v_offset,1,v,(1-(1))+(m-(1)) * (ldv)+ _v_offset);
/***
*
*                 ==== Perform delayed transformation of row below
*                 .    Mth bulge. Exploit fact that first two elements
*                 .    of row are actually zero. ====
*
***/
}
else  {
  refsum = ((v[(1-(1))+(m-(1)) * (ldv)+ _v_offset]*v[(3-(1))+(m-(1)) * (ldv)+ _v_offset])*h[((k+3)-(1))+((k+2)-(1)) * (ldh)+ _h_offset]);
h[((k+3)-(1))+(k-(1)) * (ldh)+ _h_offset] = (-(refsum));
h[((k+3)-(1))+((k+1)-(1)) * (ldh)+ _h_offset] = (-((refsum*v[(2-(1))+(m-(1)) * (ldv)+ _v_offset])));
/***
*
*                 ==== Calculate reflection to move
*                 .    Mth bulge one step. ====
*
***/
h[((k+3)-(1))+((k+2)-(1)) * (ldh)+ _h_offset] = (h[((k+3)-(1))+((k+2)-(1)) * (ldh)+ _h_offset]-(refsum*v[(3-(1))+(m-(1)) * (ldv)+ _v_offset]));
beta.val = h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset];
v[(2-(1))+(m-(1)) * (ldv)+ _v_offset] = h[((k+2)-(1))+(k-(1)) * (ldh)+ _h_offset];
v[(3-(1))+(m-(1)) * (ldv)+ _v_offset] = h[((k+3)-(1))+(k-(1)) * (ldh)+ _h_offset];
/***
*
*                 ==== A Bulge may collapse because of vigilant
*                 .    deflation or destructive underflow.  In the
*                 .    underflow case, try the two-small-subdiagonals
*                 .    trick to try to reinflate the bulge.  ====
*
***/
dlarfg_adapter(3,beta,v,(2-(1))+(m-(1)) * (ldv)+ _v_offset,1,v,(1-(1))+(m-(1)) * (ldv)+ _v_offset);
/***
*
*                    ==== Typical case: not collapsed (yet). ====
*
***/
if ((((h[((k+3)-(1))+(k-(1)) * (ldh)+ _h_offset] != 0.0) || (h[((k+3)-(1))+((k+1)-(1)) * (ldh)+ _h_offset] != 0.0)) || (h[((k+3)-(1))+((k+2)-(1)) * (ldh)+ _h_offset] == 0.0)))  {
    h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset] = beta.val;
h[((k+2)-(1))+(k-(1)) * (ldh)+ _h_offset] = 0.0;
h[((k+3)-(1))+(k-(1)) * (ldh)+ _h_offset] = 0.0;
/***
*
*                    ==== Atypical case: collapsed.  Attempt to
*                    .    reintroduce ignoring H(K+1,K) and H(K+2,K).
*                    .    If the fill resulting from the new
*                    .    reflector is too large, then abandon it.
*                    .    Otherwise, use the new one. ====
*
***/
}
else  {
  com.polytechnik.lapack310.Dlaqr1.dlaqr1(3,h,((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset,ldh,sr[(((2*m)-1)-(1))+ _sr_offset],si[(((2*m)-1)-(1))+ _si_offset],sr[((2*m)-(1))+ _sr_offset],si[((2*m)-(1))+ _si_offset],vt,0);
alpha.val = vt[(1-(1))];
dlarfg_adapter(3,alpha,vt,(2-(1)),1,vt,(1-(1)));
// *
refsum = (vt[(1-(1))]*((h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset]+(vt[(2-(1))]*h[((k+2)-(1))+(k-(1)) * (ldh)+ _h_offset]))));
/***
*
*                       ==== Starting a new bulge here would
*                       .    create non-negligible fill.  Use
*                       .    the old one with trepidation. ====
*
***/
if (((Math.abs((h[((k+2)-(1))+(k-(1)) * (ldh)+ _h_offset]-(refsum*vt[(2-(1))])))+Math.abs((refsum*vt[(3-(1))]))) > (ulp*(((Math.abs(h[(k-(1))+(k-(1)) * (ldh)+ _h_offset])+Math.abs(h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]))+Math.abs(h[((k+2)-(1))+((k+2)-(1)) * (ldh)+ _h_offset]))))))  {
    h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset] = beta.val;
h[((k+2)-(1))+(k-(1)) * (ldh)+ _h_offset] = 0.0;
h[((k+3)-(1))+(k-(1)) * (ldh)+ _h_offset] = 0.0;
/***
*
*                       ==== Starting a new bulge here would
*                       .    create only negligible fill.
*                       .    Replace the old reflector with
*                       .    the new one. ====
*
***/
}
else  {
  h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset] = (h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset]-refsum);
h[((k+2)-(1))+(k-(1)) * (ldh)+ _h_offset] = 0.0;
h[((k+3)-(1))+(k-(1)) * (ldh)+ _h_offset] = 0.0;
v[(1-(1))+(m-(1)) * (ldv)+ _v_offset] = vt[(1-(1))];
v[(2-(1))+(m-(1)) * (ldv)+ _v_offset] = vt[(2-(1))];
v[(3-(1))+(m-(1)) * (ldv)+ _v_offset] = vt[(3-(1))];
}              //  Close else.
}              //  Close else.
/***
*
*              ====  Apply reflection from the right and
*              .     the first column of update from the left.
*              .     These updates are required for the vigilant
*              .     deflation check. We still delay most of the
*              .     updates from the left for efficiency. ====
*
***/
}              //  Close else.
{
for (j = jtop; j <= Math.min(kbot, (k+3)); j++) {
refsum = (v[(1-(1))+(m-(1)) * (ldv)+ _v_offset]*(((h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset]+(v[(2-(1))+(m-(1)) * (ldv)+ _v_offset]*h[(j-(1))+((k+2)-(1)) * (ldh)+ _h_offset]))+(v[(3-(1))+(m-(1)) * (ldv)+ _v_offset]*h[(j-(1))+((k+3)-(1)) * (ldh)+ _h_offset]))));
h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset] = (h[(j-(1))+((k+1)-(1)) * (ldh)+ _h_offset]-refsum);
h[(j-(1))+((k+2)-(1)) * (ldh)+ _h_offset] = (h[(j-(1))+((k+2)-(1)) * (ldh)+ _h_offset]-(refsum*v[(2-(1))+(m-(1)) * (ldv)+ _v_offset]));
h[(j-(1))+((k+3)-(1)) * (ldh)+ _h_offset] = (h[(j-(1))+((k+3)-(1)) * (ldh)+ _h_offset]-(refsum*v[(3-(1))+(m-(1)) * (ldv)+ _v_offset]));
/***
*
*              ==== Perform update from left for subsequent
*              .    column. ====
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",70);
}              //  Close for() loop.
}
refsum = (v[(1-(1))+(m-(1)) * (ldv)+ _v_offset]*(((h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]+(v[(2-(1))+(m-(1)) * (ldv)+ _v_offset]*h[((k+2)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]))+(v[(3-(1))+(m-(1)) * (ldv)+ _v_offset]*h[((k+3)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]))));
h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset] = (h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]-refsum);
h[((k+2)-(1))+((k+1)-(1)) * (ldh)+ _h_offset] = (h[((k+2)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]-(refsum*v[(2-(1))+(m-(1)) * (ldv)+ _v_offset]));
/***
*
*              ==== The following convergence test requires that
*              .    the tradition small-compared-to-nearby-diagonals
*              .    criterion and the Ahues & Tisseur (LAWN 122, 1997)
*              .    criteria both be satisfied.  The latter improves
*              .    accuracy in some examples. Falling back on an
*              .    alternate convergence criterion when TST1 or TST2
*              .    is zero (as done here) is traditional but probably
*              .    unnecessary. ====
*
***/
h[((k+3)-(1))+((k+1)-(1)) * (ldh)+ _h_offset] = (h[((k+3)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]-(refsum*v[(3-(1))+(m-(1)) * (ldv)+ _v_offset]));
if ((k < ktop)) {
    continue label80;//Dummy.go_to("com/polytechnik/lapack310/Dlaqr5",80);
}
    if ((h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset] != 0.0))  {
    tst1 = (Math.abs(h[(k-(1))+(k-(1)) * (ldh)+ _h_offset])+Math.abs(h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
if ((tst1 == 0.0))  {
    if ((k >= (ktop+1))) {
    tst1 = (tst1+Math.abs(h[(k-(1))+((k-1)-(1)) * (ldh)+ _h_offset]));
}
    if ((k >= (ktop+2))) {
    tst1 = (tst1+Math.abs(h[(k-(1))+((k-2)-(1)) * (ldh)+ _h_offset]));
}
    if ((k >= (ktop+3))) {
    tst1 = (tst1+Math.abs(h[(k-(1))+((k-3)-(1)) * (ldh)+ _h_offset]));
}
    if ((k <= (kbot-2))) {
    tst1 = (tst1+Math.abs(h[((k+2)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
}
    if ((k <= (kbot-3))) {
    tst1 = (tst1+Math.abs(h[((k+3)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
}
    if ((k <= (kbot-4))) {
    tst1 = (tst1+Math.abs(h[((k+4)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
}
    }
if ((Math.abs(h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset]) <= Math.max(smlnum, (ulp*tst1))))  {
    h12 = Math.max(Math.abs(h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset]), Math.abs(h[(k-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
h21 = Math.min(Math.abs(h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset]), Math.abs(h[(k-(1))+((k+1)-(1)) * (ldh)+ _h_offset]));
h11 = Math.max(Math.abs(h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]), Math.abs((h[(k-(1))+(k-(1)) * (ldh)+ _h_offset]-h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset])));
h22 = Math.min(Math.abs(h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset]), Math.abs((h[(k-(1))+(k-(1)) * (ldh)+ _h_offset]-h[((k+1)-(1))+((k+1)-(1)) * (ldh)+ _h_offset])));
scl = (h11+h12);
// *
tst2 = (h22*((h11/scl)));
if (((tst2 == 0.0) || ((h21*((h12/scl))) <= Math.max(smlnum, (ulp*tst2)))))  {
    h[((k+1)-(1))+(k-(1)) * (ldh)+ _h_offset] = 0.0;
}
}
}
/***
*
*           ==== Multiply H by reflections from the left ====
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",80);
}              //  Close for() loop.
}
if (accum)  {
    jbot = Math.min(ndcol, kbot);
}
else if (wantt)  {
    jbot = n;
}              // Close else if()
else  {
  jbot = kbot;
// *
}              //  Close else.
{
int _m_inc = -1;
for (m = mbot; m >= mtop; m += _m_inc) {
k = (krcol+(2*((m-1))));
{
for (j = Math.max(ktop, (krcol+(2*m))); j <= jbot; j++) {
refsum = (v[(1-(1))+(m-(1)) * (ldv)+ _v_offset]*(((h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset]+(v[(2-(1))+(m-(1)) * (ldv)+ _v_offset]*h[((k+2)-(1))+(j-(1)) * (ldh)+ _h_offset]))+(v[(3-(1))+(m-(1)) * (ldv)+ _v_offset]*h[((k+3)-(1))+(j-(1)) * (ldh)+ _h_offset]))));
h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset] = (h[((k+1)-(1))+(j-(1)) * (ldh)+ _h_offset]-refsum);
h[((k+2)-(1))+(j-(1)) * (ldh)+ _h_offset] = (h[((k+2)-(1))+(j-(1)) * (ldh)+ _h_offset]-(refsum*v[(2-(1))+(m-(1)) * (ldv)+ _v_offset]));
h[((k+3)-(1))+(j-(1)) * (ldh)+ _h_offset] = (h[((k+3)-(1))+(j-(1)) * (ldh)+ _h_offset]-(refsum*v[(3-(1))+(m-(1)) * (ldv)+ _v_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",90);
}              //  Close for() loop.
}
/***
*
*           ==== Accumulate orthogonal transformations. ====
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",100);
}              //  Close for() loop.
}
/***
*
*              ==== Accumulate U. (If needed, update Z later
*              .    with an efficient matrix-matrix
*              .    multiply.) ====
*
***/
if (accum)  {
    {
int _m_inc = -1;
for (m = mbot; m >= mtop; m += _m_inc) {
k = (krcol+(2*((m-1))));
kms = (k-incol);
i2 = Math.max(1, (ktop-incol));
i2 = Math.max(i2, ((kms-((krcol-incol)))+1));
i4 = Math.min(kdu, (((krcol+(2*((mbot-1))))-incol)+5));
{
for (j = i2; j <= i4; j++) {
refsum = (v[(1-(1))+(m-(1)) * (ldv)+ _v_offset]*(((u[(j-(1))+((kms+1)-(1)) * (ldu)+ _u_offset]+(v[(2-(1))+(m-(1)) * (ldv)+ _v_offset]*u[(j-(1))+((kms+2)-(1)) * (ldu)+ _u_offset]))+(v[(3-(1))+(m-(1)) * (ldv)+ _v_offset]*u[(j-(1))+((kms+3)-(1)) * (ldu)+ _u_offset]))));
u[(j-(1))+((kms+1)-(1)) * (ldu)+ _u_offset] = (u[(j-(1))+((kms+1)-(1)) * (ldu)+ _u_offset]-refsum);
u[(j-(1))+((kms+2)-(1)) * (ldu)+ _u_offset] = (u[(j-(1))+((kms+2)-(1)) * (ldu)+ _u_offset]-(refsum*v[(2-(1))+(m-(1)) * (ldv)+ _v_offset]));
u[(j-(1))+((kms+3)-(1)) * (ldu)+ _u_offset] = (u[(j-(1))+((kms+3)-(1)) * (ldu)+ _u_offset]-(refsum*v[(3-(1))+(m-(1)) * (ldv)+ _v_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",110);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",120);
}              //  Close for() loop.
}
/***
*
*              ==== U is not accumulated, so update Z
*              .    now by multiplying by reflections
*              .    from the right. ====
*
***/
}
else if (wantz)  {
    {
int _m_inc = -1;
for (m = mbot; m >= mtop; m += _m_inc) {
k = (krcol+(2*((m-1))));
{
for (j = iloz; j <= ihiz; j++) {
refsum = (v[(1-(1))+(m-(1)) * (ldv)+ _v_offset]*(((z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset]+(v[(2-(1))+(m-(1)) * (ldv)+ _v_offset]*z[(j-(1))+((k+2)-(1)) * (ldz)+ _z_offset]))+(v[(3-(1))+(m-(1)) * (ldv)+ _v_offset]*z[(j-(1))+((k+3)-(1)) * (ldz)+ _z_offset]))));
z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset] = (z[(j-(1))+((k+1)-(1)) * (ldz)+ _z_offset]-refsum);
z[(j-(1))+((k+2)-(1)) * (ldz)+ _z_offset] = (z[(j-(1))+((k+2)-(1)) * (ldz)+ _z_offset]-(refsum*v[(2-(1))+(m-(1)) * (ldv)+ _v_offset]));
z[(j-(1))+((k+3)-(1)) * (ldz)+ _z_offset] = (z[(j-(1))+((k+3)-(1)) * (ldz)+ _z_offset]-(refsum*v[(3-(1))+(m-(1)) * (ldv)+ _v_offset]));
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",130);
}              //  Close for() loop.
}
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",140);
}              //  Close for() loop.
}
/***
*
*           ==== End of near-the-diagonal bulge chase. ====
*
***/
}              // Close else if()
/***
*
*        ==== Use U (if accumulated) to update far-from-diagonal
*        .    entries in H.  If required, use U to update Z as
*        .    well. ====
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",145);
}              //  Close for() loop.
}
if (accum)  {
    if (wantt)  {
    jtop = 1;
jbot = n;
}
else  {
  jtop = ktop;
jbot = kbot;
}              //  Close else.
k1 = Math.max(1, (ktop-incol));
/***
*
*           ==== Horizontal Multiply ====
*
***/
nu = ((((kdu-Math.max(0, (ndcol-kbot))))-k1)+1);
{
int _jcol_inc = nh;
for (jcol = (Math.min(ndcol, kbot)+1); (_jcol_inc < 0) ? jcol >= jbot : jcol <= jbot; jcol += _jcol_inc) {
jlen = Math.min(nh, ((jbot-jcol)+1));
com.polytechnik.lapack310.Dgemm.dgemm("C","N",nu,jlen,nu,1.0,u,(k1-(1))+(k1-(1)) * (ldu)+ _u_offset,ldu,h,((incol+k1)-(1))+(jcol-(1)) * (ldh)+ _h_offset,ldh,0.0,wh,_wh_offset,ldwh);
com.polytechnik.lapack310.Dlacpy.dlacpy("ALL",nu,jlen,wh,_wh_offset,ldwh,h,((incol+k1)-(1))+(jcol-(1)) * (ldh)+ _h_offset,ldh);
/***
*
*           ==== Vertical multiply ====
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",150);
}              //  Close for() loop.
}
{
int _jrow_inc = nv;
for (jrow = jtop; (_jrow_inc < 0) ? jrow >= (Math.max(ktop, incol)-1) : jrow <= (Math.max(ktop, incol)-1); jrow += _jrow_inc) {
jlen = Math.min(nv, (Math.max(ktop, incol)-jrow));
com.polytechnik.lapack310.Dgemm.dgemm("N","N",jlen,nu,nu,1.0,h,(jrow-(1))+((incol+k1)-(1)) * (ldh)+ _h_offset,ldh,u,(k1-(1))+(k1-(1)) * (ldu)+ _u_offset,ldu,0.0,wv,_wv_offset,ldwv);
com.polytechnik.lapack310.Dlacpy.dlacpy("ALL",jlen,nu,wv,_wv_offset,ldwv,h,(jrow-(1))+((incol+k1)-(1)) * (ldh)+ _h_offset,ldh);
/***
*
*           ==== Z multiply (also vertical) ====
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",160);
}              //  Close for() loop.
}
if (wantz)  {
    {
int _jrow_inc = nv;
for (jrow = iloz; (_jrow_inc < 0) ? jrow >= ihiz : jrow <= ihiz; jrow += _jrow_inc) {
jlen = Math.min(nv, ((ihiz-jrow)+1));
com.polytechnik.lapack310.Dgemm.dgemm("N","N",jlen,nu,nu,1.0,z,(jrow-(1))+((incol+k1)-(1)) * (ldz)+ _z_offset,ldz,u,(k1-(1))+(k1-(1)) * (ldu)+ _u_offset,ldu,0.0,wv,_wv_offset,ldwv);
com.polytechnik.lapack310.Dlacpy.dlacpy("ALL",jlen,nu,wv,_wv_offset,ldwv,z,(jrow-(1))+((incol+k1)-(1)) * (ldz)+ _z_offset,ldz);
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",170);
}              //  Close for() loop.
}
}
}
/***
*
*     ==== End of DLAQR5 ====
*
***/
//Dummy.label("com/polytechnik/lapack310/Dlaqr5",180);
}              //  Close for() loop.
}
// dummy stmt
break label_999999;//Dummy.label("com/polytechnik/lapack310/Dlaqr5",999999);
}
return;
   }
// adapter for dlarfg(ILorg/netlib/util/doubleW;[DIILorg/netlib/util/doubleW;)V
private static void dlarfg_adapter(int arg0 ,doubleW arg1 ,double [] arg2 , int arg2_offset ,int arg3 ,double [] arg4 , int arg4_offset )
{
doubleW _f2j_tmp4 = new doubleW(arg4[arg4_offset]);

com.polytechnik.lapack310.Dlarfg.dlarfg(arg0,arg1,arg2, arg2_offset,arg3,_f2j_tmp4);

arg4[arg4_offset] = _f2j_tmp4.val;
}

} // End class.
