package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.realtimebook.*;
import com.polytechnik.utils.EVXData;
import com.polytechnik.outils.*;

/** A class to calculate actual attributes values for ML program.
 *  @version $Id: CalculateAntecedents.java,v 1.3626 2020/02/06 00:09:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class CalculateAntecedents {

    static final boolean ATTRIB_TEST_ALL=false;//true;//false;

    static final String OPT_BUY="Opt.buy_",OPT_SELL="Opt.sell_";



    private static final boolean
	ATTRIB_BM_DIFFERENCES_AUTOGENERATED=false || ATTRIB_TEST_ALL,
	ATTRIB_BM_SUM_AUTOGENERATED=true || ATTRIB_TEST_ALL,
	ATTRIB_VOLUME_DIFFERENCES_AUTOGENERATED=false || ATTRIB_TEST_ALL,
	ATTRIB_VOLUMES=true || ATTRIB_TEST_ALL,// think later
	ATTRIB_VOLUME_EO=true || ATTRIB_TEST_ALL,
	ATTRIB_FRAC_VOLUME_ABOVE_BELOW=false || ATTRIB_TEST_ALL,
	ATTRIB_HIDDEN=false || ATTRIB_TEST_ALL, // comment out as unstable
	ATTRIB_MINMAX=false || ATTRIB_TEST_ALL, // think later
	ATTRIB_TMM=true || ATTRIB_TEST_ALL,
	ATTRIB_TMM2=true || ATTRIB_TEST_ALL,
	ATTRIB_TMC=true || ATTRIB_TEST_ALL,
	ATTRIB_T_DISTANCE=false || ATTRIB_TEST_ALL,
	ATTRIB_BOOK_MEDIAN=true || ATTRIB_TEST_ALL,
	ATTRIB_BOOK_N_ORDERS=false || ATTRIB_TEST_ALL,
	ATTRIB_BOOK_PC_PRESETLEVELS=false || ATTRIB_TEST_ALL,
	ATTRIB_BOOK_D2_PRESETLEVELS=false || ATTRIB_TEST_ALL,
	ATTRIB_BOOK_PC_PRESETLEVELS_SUM=false || ATTRIB_TEST_ALL,
	ATTRIB_BOOK_PC_DIS_PRESETLEVELS=false || ATTRIB_TEST_ALL,
	ATTRIB_ADDORDER_V_RATIO=true || ATTRIB_TEST_ALL,
	ATTRIB_ADDORDER_MEDIAN_RATIO=true || ATTRIB_TEST_ALL
	;


    /** Non private because
     *  we switch off some outside code (for speed) when
     *  the value is false.
     */
    static final boolean
    // calculations in CalculateLiquidityDeficit21
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT21=true || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit20
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT20=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit19
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT19=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit18
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT18=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit17
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT17=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit16
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT16=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit15
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT15=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit13
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT13=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit12
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT12=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit11
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT11=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit10
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT10=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit9T
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT9T=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit9
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT9=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit8
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT8=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit7
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT7=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit6
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT6=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit5
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT5=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit4
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT4=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit3
	ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT3=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit2
	ATTRIB_LIQUIDITYDEFICIT2=false || ATTRIB_TEST_ALL,
    // calculations in CalculateLiquidityDeficit
	ATTRIB_LIQUIDITYDEFICIT01=false || ATTRIB_TEST_ALL,
    // calculations in CalculateBookLiquidityDeficit
	ATTRIB_BOOK_LIQUIDITYDEFICIT02=false || ATTRIB_TEST_ALL,
    // calculations in CalculateBookVirtualShiftsTau
	ATTRIB_VIRTUAL_SHIFTS_TAU=false || ATTRIB_TEST_ALL,
    // calculations in CalculateBookVirtualShifts
	ATTRIB_VIRTUAL_SHIFTS=false || ATTRIB_TEST_ALL,
    // calculations in CalculateBookVirtualShifts2
	ATTRIB_VIRTUAL_SHIFTS2=false || ATTRIB_TEST_ALL,
    // hidden trades contribution calculated independently on slow CalculateTradeVolume
	ATTRIB_HCONTRIB=false || ATTRIB_TEST_ALL,
    // calculation in CalculateDynamics
	ATTRIB_DYNAMICS=false || ATTRIB_TEST_ALL,
    // calculation in CalculateTRDynamics
	ATTRIB_TR_DYNAMICS=false || ATTRIB_TEST_ALL,
    // calculations in CalculateBookVolumePriceThreshold
	ATTRIB_BOOK_PRESETLEVELS=false || ATTRIB_TEST_ALL,
    // calculations in CalculateTradeVolume
	ATTRIB_TRADE_TAU_CHANGE=false || ATTRIB_TEST_ALL,
    // calculations in CalculateBookVolumePriceFixedThreshold
	ATTRIB_BOOK_THRESHOLDS_FROM_TRADES=(true && ATTRIB_TRADE_TAU_CHANGE) || ATTRIB_TEST_ALL,
    // calculations in CalculateBookVolume
	ATTRIB_BOOK_TAU_CHANGE=(false && ATTRIB_BOOK_PRESETLEVELS) || ATTRIB_TEST_ALL,
    // calculations in CalculateBookCriticalExpoinent
	ATTRIB_CRITICAL_EXPONENT_S=false || ATTRIB_TEST_ALL

	;



    final StringAttributesWithDescription sattribs_names=new StringAttributesWithDescription("ignore");
    final StringAttributes sattribs=new StringAttributes();

    final StringAttributesWithDescription dattribs_names=new StringAttributesWithDescription("continuous");
    final DoubleAttributes dattribs=new DoubleAttributes();

    final StringAttributesWithDescription nattribs_names=new StringAttributesWithDescription("0,1");
    final NominalAttributes nattribs=new NominalAttributes();

    void clearData()
    {
	this.dattribs_names.clearData();
	this.dattribs.clearData();
	this.sattribs_names.clearData();
	this.sattribs.clearData();
	this.nattribs_names.clearData();
	this.nattribs.clearData();
    }

    private int setAttrInstrInfo(final boolean flag_processheader,
				 final String ticker,
				 final long datatime)
    {
	if(flag_processheader)
	{
	    this.sattribs_names.addValue("aTicker");
	    this.sattribs_names.addValue("t00.data");
	}
	else
	{
	    this.sattribs.addValue(ticker);
	    this.sattribs.addValue(String.valueOf(datatime));
	}
	return 2;
    }


    private int setAttrCriticalExp(final boolean flag_processheader,
				   final String ticker,
				   final CalculationData c)
    {
	int n=0;
	n+=2;
	if(flag_processheader)
	{
	    this.dattribs_names.addValue("Gce.buy");
	    this.dattribs_names.addValue("Gce.sell");
	}
	else
	{
	    final CalculateBookCriticalExponent bc=c.topp.calculate_buy_crit,sc=c.topp.calculate_sell_crit;
	    this.dattribs.addValue(bc.beta_crit);
	    this.dattribs.addValue(sc.beta_crit);
	}
	return n;
    }



    private int setAttrCi(final boolean flag_processheader,
			  final String ticker,
			  final CalculationData c)
    {
	int n=0;
	n+=6;
	if(flag_processheader)
	{
	    this.dattribs_names.addValue("spread.sb","","ignore");
	    this.dattribs_names.addValue("P.buy","","ignore");
	    this.dattribs_names.addValue("P.sell","","ignore");
	    this.dattribs_names.addValue("P.last","","ignore");
	    this.dattribs_names.addValue("P.vol_p_buy","","ignore");
	    this.dattribs_names.addValue("P.vol_p_sell","","ignore");
	}
	else
	{
	    // 2*(sell-buy)/(sell+buy) spread
	    this.dattribs.addValue((2.0*(c.p_sell-c.p_buy))/(c.p_sell+c.p_buy));
	    this.dattribs.addValue(c.p_buy);
	    this.dattribs.addValue(c.p_sell);
	    this.dattribs.addValue(c.p_last);
	    this.dattribs.addValue(c.vol_p_buy);
	    this.dattribs.addValue(c.vol_p_sell);
	}
	return n;
    }


    private int printQTradeIPNames(final String prefix,final String s)
    {
	int n=0;

	n+=2;
	this.dattribs_names.addValue(prefix+".n_p_up_",s,null);
	this.dattribs_names.addValue(prefix+".n_p_down_",s,null);
	n+=2;
	this.dattribs_names.addValue(prefix+".n_dp_up_",s,null);
	this.dattribs_names.addValue(prefix+".n_dp_down_",s,null);

	for(int r=0;r<2;r++)
	{
	    n+=2;
	    this.dattribs_names.addValue(prefix+".p_up_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p_down_"+r+"_",s,null);
	    n+=2;
	    this.dattribs_names.addValue(prefix+".dp_up_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".dp_down_"+r+"_",s,null);

	    n+=4;
	    this.dattribs_names.addValue(prefix+".p_i_eq_imax_low_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p_i_eq_imax_high_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p_i_eq_imin_low_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p_i_eq_imin_high_"+r+"_",s,null);
	    n+=2;
	    this.dattribs_names.addValue(prefix+".i_imin_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".i_imax_"+r+"_",s,null);
	}

	n+=6;
	this.dattribs_names.addValue(prefix+".p_offset_",s,null);
	this.dattribs_names.addValue(prefix+".sigma_",s,null);
	this.dattribs_names.addValue(prefix+".ia0_",s,null);
	this.dattribs_names.addValue(prefix+".dia0_",s,null);
	this.dattribs_names.addValue(prefix+".getDIDP_",s,null);
	this.dattribs_names.addValue(prefix+".getAI0_",s,null);

	n+=9;
	this.dattribs_names.addValue(prefix+".getTID_",s,null);
	this.dattribs_names.addValue(prefix+".pm_",s,null);
	this.dattribs_names.addValue(prefix+".pm_l_",s,null);
	this.dattribs_names.addValue(prefix+".pm_h_",s,null);
	this.dattribs_names.addValue(prefix+".pe_",s,null);
	this.dattribs_names.addValue(prefix+".pe_l_",s,null);
	this.dattribs_names.addValue(prefix+".pe_h_",s,null);
	this.dattribs_names.addValue(prefix+".pmm_",s,null);
	this.dattribs_names.addValue(prefix+".pmo_",s,null);

	n+=4;
	this.dattribs_names.addValue(prefix+".enter_buy_",s,null);
	this.dattribs_names.addValue(prefix+".exit_buy_",s,null);
	this.dattribs_names.addValue(prefix+".enter_sell_",s,null);
	this.dattribs_names.addValue(prefix+".exit_sell_",s,null);

	return n;
    }

    private int printQTradeIPValues(final CalculateKendall.QTradeIP x,final CalculationData c)
    {
	int n=0;

	n+=2;
	this.dattribs.addValue(x.p_up.length);
	this.dattribs.addValue(x.p_down.length);
	n+=2;
	this.dattribs.addValue(x.dp_up.length);
	this.dattribs.addValue(x.dp_down.length);

	for(int r=0;r<2;r++)
	{
	    n+=2;
	    this.dattribs.addValue(r<x.p_up.length?x.p_up[r]:Double.NaN);
	    this.dattribs.addValue(r<x.p_down.length?x.p_down[r]:Double.NaN);
	    n+=2;
	    this.dattribs.addValue(r<x.dp_up.length?x.dp_up[r]:Double.NaN);
	    this.dattribs.addValue(r<x.dp_down.length?x.dp_down[r]:Double.NaN);

	    n+=4;
	    this.dattribs.addValue(r<x.ptopology.p_i_eq_imax_low.length?x.ptopology.p_i_eq_imax_low[r]:Double.NaN);
	    this.dattribs.addValue(r<x.ptopology.p_i_eq_imax_high.length?x.ptopology.p_i_eq_imax_high[r]:Double.NaN);
	    this.dattribs.addValue(r<x.ptopology.p_i_eq_imin_low.length?x.ptopology.p_i_eq_imin_low[r]:Double.NaN);
	    this.dattribs.addValue(r<x.ptopology.p_i_eq_imin_high.length?x.ptopology.p_i_eq_imin_high[r]:Double.NaN);

	    n+=2;
	    this.dattribs.addValue(r<x.ptopology.i_imin.length?x.ptopology.i_imin[r]:Double.NaN);
	    this.dattribs.addValue(r<x.ptopology.i_imax.length?x.ptopology.i_imax[r]:Double.NaN);

	}

	n+=6;
	this.dattribs.addValue(x.p_offset);
	this.dattribs.addValue(x.sigma);
	this.dattribs.addValue(x.ia0);
	this.dattribs.addValue(x.dia0);
	this.dattribs.addValue(x.getDIDP(c.p_last));
	this.dattribs.addValue(x.getAI0(c.p_last));

	n+=9;
	this.dattribs.addValue(x.ptopology.getTID());
	this.dattribs.addValue(x.ptopology.pm);
	this.dattribs.addValue(x.ptopology.pm_l);
	this.dattribs.addValue(x.ptopology.pm_h);
	this.dattribs.addValue(x.ptopology.pe);
	this.dattribs.addValue(x.ptopology.pe_l);
	this.dattribs.addValue(x.ptopology.pe_h);
	this.dattribs.addValue(x.ptopology.pmm);
	this.dattribs.addValue(x.ptopology.pmo);

	n+=4;
	this.dattribs.addValue(x.ptopology.checkEnterBuy(c.p_last)?1:0);
	this.dattribs.addValue(x.ptopology.checkExitBuy(c.p_last)?1:0);
	this.dattribs.addValue(x.ptopology.checkEnterSell(c.p_last)?1:0);
	this.dattribs.addValue(x.ptopology.checkExitSell(c.p_last)?1:0);

	return n;
    }



    private int printQATradeIPNames(final String prefix,final String s)
    {

	int n=0;

	n+=2;
	this.dattribs_names.addValue(prefix+".n_p_imin_",s,null);
	this.dattribs_names.addValue(prefix+".n_p_imax_",s,null);

	for(int r=0;r<CalculateLiquidityDeficit16.NP-1;r++)
	{
	    n+=2;
	    this.dattribs_names.addValue(prefix+".p_imin_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p_imax_"+r+"_",s,null);
	}

	n+=10;
	this.dattribs_names.addValue(prefix+".p_offset_",s,null);
	this.dattribs_names.addValue(prefix+".sigma_",s,null);
	this.dattribs_names.addValue(prefix+".getDIDPDT_TOT_",s,null);
	this.dattribs_names.addValue(prefix+".getDIDT_",s,null);
	this.dattribs_names.addValue(prefix+".getPGMIN_",s,null);
	this.dattribs_names.addValue(prefix+".getPLMIN_",s,null);
	this.dattribs_names.addValue(prefix+".getIF_",s,null);
	this.dattribs_names.addValue(prefix+".getIDP_",s,null);
	this.dattribs_names.addValue(prefix+".getAI0_",s,null);
	this.dattribs_names.addValue(prefix+".getDIDP_",s,null);

	return n;
    }


    /** Think later about unification. */
    private int printQATradeIPValues(final CalculateKendall.QATradeIP x,final CalculationData c)
    {
	int n=0;

	n+=2;
	this.dattribs.addValue(x.p_imin.length);
	this.dattribs.addValue(x.p_imax.length);

	for(int r=0;r<CalculateLiquidityDeficit16.NP-1;r++)
	{
	    n+=2;
	    this.dattribs.addValue(r<x.p_imin.length?x.p_imin[r]:Double.NaN);
	    this.dattribs.addValue(r<x.p_imax.length?x.p_imax[r]:Double.NaN);
	}

	n+=10;
	this.dattribs.addValue(x.p_offset);
	this.dattribs.addValue(x.sigma);
	this.dattribs.addValue(x.getDIDPDT_TOT(c.p_last));
	this.dattribs.addValue(x.getDIDT(c.p_last));
	this.dattribs.addValue(x.getPGMIN(c.p_last));
	this.dattribs.addValue(x.getPLMIN(c.p_last));
	this.dattribs.addValue(x.getIF(c.p_last));
	this.dattribs.addValue(x.getIDP(c.p_last));
	this.dattribs.addValue(x.getAI0(c.p_last));
	this.dattribs.addValue(x.getDIDP(c.p_last));

	return n;
    }



    private int printQVMNames(final String prefix,final String s)
    {
	int n=0;

	n+=2;
	this.dattribs_names.addValue(prefix+".n_p_imin_",s,null);
	this.dattribs_names.addValue(prefix+".n_p_imax_",s,null);

	for(int r=0;r<CalculateLiquidityDeficit16.N_EM;r++)
	{
	    n+=2;
	    this.dattribs_names.addValue(prefix+".p_imin_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p_imax_"+r+"_",s,null);
	}

	n+=8;
	this.dattribs_names.addValue(prefix+".p_offset_",s,null);
	this.dattribs_names.addValue(prefix+".sigma_",s,null);
	this.dattribs_names.addValue(prefix+".getDIDPDT_TOT_",s,null);
	this.dattribs_names.addValue(prefix+".getDIDT_",s,null);
	this.dattribs_names.addValue(prefix+".getPGMIN_",s,null);
	this.dattribs_names.addValue(prefix+".getPLMIN_",s,null);
	this.dattribs_names.addValue(prefix+".getIF_",s,null);
	this.dattribs_names.addValue(prefix+".getIDP_",s,null);

	return n;
    }

    private int printQVMValues(final QVM.QVMCoefs x,final CalculationData c)
    {
	int n=0;

	n+=2;
	this.dattribs.addValue(x.p_imin.length);
	this.dattribs.addValue(x.p_imax.length);

	for(int r=0;r<CalculateLiquidityDeficit16.N_EM;r++)
	{
	    n+=2;
	    this.dattribs.addValue(r<x.p_imin.length?x.p_imin[r]:Double.NaN);
	    this.dattribs.addValue(r<x.p_imax.length?x.p_imax[r]:Double.NaN);
	}

	n+=8;
	this.dattribs.addValue(x.pa_offset);
	this.dattribs.addValue(x.sigma);
	this.dattribs.addValue(x.getDIDPDT_TOT(c.p_last));
	this.dattribs.addValue(x.getDIDT(c.p_last));
	this.dattribs.addValue(x.getPGMIN(c.p_last));
	this.dattribs.addValue(x.getPLMIN(c.p_last));
	this.dattribs.addValue(x.getIF(c.p_last));
	this.dattribs.addValue(x.getIDP(c.p_last));

	return n;
    }

    private int printQVMORTNames(final String prefix,final String s)
    {
	int n=0;

	n+=1;
	this.dattribs_names.addValue(prefix+".n_p_imin_",s,null);
	for(int r=0;r<CalculateLiquidityDeficit16.N_EM;r++)
	{
	    n+=2;
	    this.dattribs_names.addValue(prefix+".p_imin_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".icoefs_"+r+"_",s,null);
	    n+=1;
	    this.dattribs_names.addValue(prefix+".dtx_"+r+"_",s,null);
	}

	n+=2;
	this.dattribs_names.addValue(prefix+".p_average_",s,null);
	this.dattribs_names.addValue(prefix+".sigma_",s,null);

	n+=2;
	this.dattribs_names.addValue(prefix+".t_pl_",s,null);
	this.dattribs_names.addValue(prefix+".dtdp_pl_",s,null);

	n+=1;
	this.dattribs_names.addValue(prefix+".p_close_",s,null);

	n+=5;
	this.dattribs_names.addValue(prefix+".mu_pl_",s,null);
	this.dattribs_names.addValue(prefix+".dmudp_pl_",s,null);
	this.dattribs_names.addValue(prefix+".vl_pl_",s,null);
	this.dattribs_names.addValue(prefix+".dvldp_pl_",s,null);
	this.dattribs_names.addValue(prefix+".deltaS_",s,null);

	return n;
    }
    private int printQVMORTValues(final QVM.QVMORT x,final CalculationData c)
    {
	int n=0;

	n+=1;
	this.dattribs.addValue(x.p_imin.length);
	for(int r=0;r<CalculateLiquidityDeficit16.N_EM;r++)
	{
	    n+=2;
	    this.dattribs.addValue(r<x.p_imin.length?x.p_imin[r]:Double.NaN);
	    this.dattribs.addValue(r<x.icoefs.length?x.icoefs[r]:Double.NaN);
	    n+=1;
	    this.dattribs.addValue(r<x.dtx.length?x.dtx[r]:Double.NaN); // FIXME to remove
	}

	n+=2;
	this.dattribs.addValue(x.p_average);
	this.dattribs.addValue(x.sigma);

	n+=2;
	this.dattribs.addValue(x.t_pl);
	this.dattribs.addValue(x.dtdp_pl);

	n+=1;
	this.dattribs.addValue(x.ind_close<0?Double.NaN:x.p_imin[x.ind_close]);

	n+=5;
	this.dattribs.addValue(x.mu_pl);
	this.dattribs.addValue(x.dmudp_pl);
	this.dattribs.addValue(x.vl_pl);
	this.dattribs.addValue(x.dvldp_pl);
	this.dattribs.addValue(x.deltaS);

	return n;
    }

    private int printQVMVTNames(final String prefix,final String s)
    {
	int n=0;
	n+=printQVMORTNames(prefix+".vm",s);
	n+=printQVMORTNames(prefix+".tm",s);
	n+=6;
	this.dattribs_names.addValue(prefix+".pvya_low_",s,null);
	this.dattribs_names.addValue(prefix+".ptya_low_",s,null);
	this.dattribs_names.addValue(prefix+".pvya_high_",s,null);
	this.dattribs_names.addValue(prefix+".ptya_high_",s,null);
	this.dattribs_names.addValue(prefix+".pvya_",s,null);
	this.dattribs_names.addValue(prefix+".ptya_",s,null);

	n+=2;
	this.dattribs_names.addValue(prefix+".ptiv_close_",s,null);
	this.dattribs_names.addValue(prefix+".pvit_close_",s,null);
	n+=2;
	this.dattribs_names.addValue(prefix+".p2v_close_",s,null);
	this.dattribs_names.addValue(prefix+".p2tiv_close_",s,null);

	n+=4;
	this.dattribs_names.addValue(prefix+".pvpa_low_",s,null);
	this.dattribs_names.addValue(prefix+".ptpa_low_",s,null);
	this.dattribs_names.addValue(prefix+".pvpa_high_",s,null);
	this.dattribs_names.addValue(prefix+".ptpa_high_",s,null);


	n+=4;
	this.dattribs_names.addValue(prefix+".sd_low_",s,null);
	this.dattribs_names.addValue(prefix+".sd_high_",s,null);
	this.dattribs_names.addValue(prefix+".xsd_low_",s,null);
	this.dattribs_names.addValue(prefix+".xsd_high_",s,null);

	n+=4;
	this.dattribs_names.addValue(prefix+".pv_low_",s,null);
	this.dattribs_names.addValue(prefix+".ptiv_low_",s,null);
	this.dattribs_names.addValue(prefix+".wv_low_",s,null);
	this.dattribs_names.addValue(prefix+".wtiv_low_",s,null);
	n+=4;
	this.dattribs_names.addValue(prefix+".pv_high_",s,null);
	this.dattribs_names.addValue(prefix+".ptiv_high_",s,null);
	this.dattribs_names.addValue(prefix+".wv_high_",s,null);
	this.dattribs_names.addValue(prefix+".wtiv_high_",s,null);


	n+=4;
	this.dattribs_names.addValue(prefix+".pav_low_",s,null);
	this.dattribs_names.addValue(prefix+".pav_high_",s,null);
	this.dattribs_names.addValue(prefix+".pat_low_",s,null);
	this.dattribs_names.addValue(prefix+".pat_high_",s,null);
	n+=4;
	this.dattribs_names.addValue(prefix+".wav_low_",s,null);
	this.dattribs_names.addValue(prefix+".wav_high_",s,null);
	this.dattribs_names.addValue(prefix+".wat_low_",s,null);
	this.dattribs_names.addValue(prefix+".wat_high_",s,null);

	n+=4;
	this.dattribs_names.addValue(prefix+".enter_buy_",s,null);
	this.dattribs_names.addValue(prefix+".exit_buy_",s,null);
	this.dattribs_names.addValue(prefix+".enter_sell_",s,null);
	this.dattribs_names.addValue(prefix+".exit_sell_",s,null);

	return n;
    }
    private int printQVMVTValues(final QVM.QVMVT x,final CalculationData c)
    {
	int n=0;
	n+=printQVMORTValues(x.vm,c);
	n+=printQVMORTValues(x.tm,c);
	n+=6;
	this.dattribs.addValue(x.pvya_low);
	this.dattribs.addValue(x.ptya_low);
	this.dattribs.addValue(x.pvya_high);
	this.dattribs.addValue(x.ptya_high);
	this.dattribs.addValue(x.pvya);
	this.dattribs.addValue(x.ptya);

	n+=2;
	this.dattribs.addValue(x.ptiv_close);
	this.dattribs.addValue(x.pvit_close);
	n+=2;
	this.dattribs.addValue(x.p2v_close);
	this.dattribs.addValue(x.p2tiv_close);

	n+=4;
	this.dattribs.addValue(x.pvpa_low);
	this.dattribs.addValue(x.ptpa_low);
	this.dattribs.addValue(x.pvpa_high);
	this.dattribs.addValue(x.ptpa_high);


	n+=4;
	this.dattribs.addValue(x.sd_low);
	this.dattribs.addValue(x.sd_high);
	this.dattribs.addValue(x.xsd_low);
	this.dattribs.addValue(x.xsd_high);

	n+=4;
	this.dattribs.addValue(x.pv_low);
	this.dattribs.addValue(x.ptiv_low);
	this.dattribs.addValue(x.wv_low);
	this.dattribs.addValue(x.wtiv_low);
	n+=4;
	this.dattribs.addValue(x.pv_high);
	this.dattribs.addValue(x.ptiv_high);
	this.dattribs.addValue(x.wv_high);
	this.dattribs.addValue(x.wtiv_high);

	n+=4;
	this.dattribs.addValue(x.pav_low);
	this.dattribs.addValue(x.pav_high);
	this.dattribs.addValue(x.pat_low);
	this.dattribs.addValue(x.pat_high);
	n+=4;
	this.dattribs.addValue(x.wav_low);
	this.dattribs.addValue(x.wav_high);
	this.dattribs.addValue(x.wat_low);
	this.dattribs.addValue(x.wat_high);

	n+=4;
	this.dattribs.addValue(x.enter_buy);
	this.dattribs.addValue(x.exit_buy);
	this.dattribs.addValue(x.enter_sell);
	this.dattribs.addValue(x.exit_sell);

	return n;
    }


private int printQVMMUNames(final String prefix,final String s)
{
    int n=0;

    n+=1;
    this.dattribs_names.addValue(prefix+".n_p_imin_",s,null);
    for(int r=0;r<CalculateLiquidityDeficit16.N_EM;r++)
    {
	n+=2;
	this.dattribs_names.addValue(prefix+".p_imin_"+r+"_",s,null);
	this.dattribs_names.addValue(prefix+".icoefs_"+r+"_",s,null);
	n+=2;
	this.dattribs_names.addValue(prefix+".f_"+r+"_",s,null);
	this.dattribs_names.addValue(prefix+".dfdp_"+r+"_",s,null);
    }
    n+=2;
    this.dattribs_names.addValue(prefix+".f_pl_",s,null);
    this.dattribs_names.addValue(prefix+".dfdp_pl_",s,null);
    return n;
}
private int printQVMMUValues(final QVM.QVMMU x,final CalculationData c)
{
    int n=0;

    n+=1;
    this.dattribs.addValue(x.p_imin.length);
    for(int r=0;r<CalculateLiquidityDeficit16.N_EM;r++)
    {
	n+=2;
	this.dattribs.addValue(r<x.p_imin.length?x.p_imin[r]:Double.NaN);
	this.dattribs.addValue(r<x.icoefs.length?x.icoefs[r]:Double.NaN);
	n+=2;
	this.dattribs.addValue(r<x.f.length?x.f[r]:Double.NaN);
	this.dattribs.addValue(r<x.dfdp.length?x.dfdp[r]:Double.NaN);
    }
    n+=2;
    this.dattribs.addValue(x.f_pl);
    this.dattribs.addValue(x.dfdp_pl);
    return n;
}


    private int printQVMVTMUNames(final String prefix,final String s)
    {
	int n=0;
	n+=printQVMMUNames(prefix+".muvm",s);
	n+=printQVMMUNames(prefix+".mutm",s);
	return n;
    }
    private int printQVMVTMUValues(final QVM.QVMVTMU x,final CalculationData c)
    {
	int n=0;
	n+=printQVMMUValues(x.vm,c);
	n+=printQVMMUValues(x.tm,c);
	return n;
    }


private int printQVMLNames(final String prefix,final String s)
{
    int n=0;

    n+=2;
    this.dattribs_names.addValue(prefix+".p_average_",s,null);
    this.dattribs_names.addValue(prefix+".sigma_",s,null);

    n+=2;
    this.dattribs_names.addValue(prefix+".pt_",s,null);
    this.dattribs_names.addValue(prefix+".pt_f_",s,null);

    n+=12;
    this.dattribs_names.addValue(prefix+".I_",s,null);
    this.dattribs_names.addValue(prefix+".Ibuy_",s,null);
    this.dattribs_names.addValue(prefix+".Isell_",s,null);
    this.dattribs_names.addValue(prefix+".IMax_",s,null);
    this.dattribs_names.addValue(prefix+".IbuyMax_",s,null);
    this.dattribs_names.addValue(prefix+".IsellMax_",s,null);
    this.dattribs_names.addValue(prefix+".I_f_",s,null);
    this.dattribs_names.addValue(prefix+".pI_",s,null);
    this.dattribs_names.addValue(prefix+".pI_f_",s,null);
    this.dattribs_names.addValue(prefix+".p_f_",s,null);
    this.dattribs_names.addValue(prefix+".p_df_",s,null);
    this.dattribs_names.addValue(prefix+".p_dfl_",s,null);

    return n;
}
private int printQVMLValues(final QVML x,final CalculationData c)
{
    int n=0;

    n+=2;
    this.dattribs.addValue(x.p_average);
    this.dattribs.addValue(x.sigma);

    n+=2;
    this.dattribs.addValue(x.pt);
    this.dattribs.addValue(x.pt_f);

    n+=12;
    this.dattribs.addValue(x.I);
    this.dattribs.addValue(x.Ibuy);
    this.dattribs.addValue(x.Isell);
    this.dattribs.addValue(x.IMax);
    this.dattribs.addValue(x.IbuyMax);
    this.dattribs.addValue(x.IsellMax);
    this.dattribs.addValue(x.I_f);
    this.dattribs.addValue(x.pI);
    this.dattribs.addValue(x.pI_f);
    this.dattribs.addValue(x.p_f);
    this.dattribs.addValue(x.p_df);
    this.dattribs.addValue(x.p_dfl);

    return n;
}


private int printQVMDNames(final String prefix,final String s)
{
    int n=0;

    n+=11;
    this.dattribs_names.addValue(prefix+".p_eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_eq_",s,null);
    this.dattribs_names.addValue(prefix+".osp_",s,null);
    this.dattribs_names.addValue(prefix+".pi_eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2pi_eq_",s,null);
    this.dattribs_names.addValue(prefix+".dpi_eq_",s,null);
    this.dattribs_names.addValue(prefix+".di_",s,null);
    this.dattribs_names.addValue(prefix+".dip_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io0_",s,null);
    this.dattribs_names.addValue(prefix+".scale_dip0_",s,null);

    return n;
}
private int printQVMDValues(final QVMD x,final CalculationData c)
{
    int n=0;

    n+=11;
    this.dattribs.addValue(x.p_eq);
    this.dattribs.addValue(x.sigma2_eq);
    this.dattribs.addValue(x.osp);
    this.dattribs.addValue(x.pi_eq);
    this.dattribs.addValue(x.sigma2pi_eq);
    this.dattribs.addValue(x.dpi_eq);
    this.dattribs.addValue(x.di);
    this.dattribs.addValue(x.dip);
    this.dattribs.addValue(x.scale_io);
    this.dattribs.addValue(x.scale_io0);
    this.dattribs.addValue(x.scale_dip0);

    return n;
}
private int printAQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".p_eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_eq_",s,null);
    this.dattribs_names.addValue(prefix+".osp_",s,null);
    this.dattribs_names.addValue(prefix+".pi_eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2pi_eq_",s,null);
    this.dattribs_names.addValue(prefix+".dpi_eq_",s,null);

    n+=8;
    this.dattribs_names.addValue(prefix+".scaleo_io_",s,null);
    this.dattribs_names.addValue(prefix+".di_",s,null);
    this.dattribs_names.addValue(prefix+".dip_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io0_",s,null);
    this.dattribs_names.addValue(prefix+".dj_",s,null);
    this.dattribs_names.addValue(prefix+".scale_dj_",s,null);
    this.dattribs_names.addValue(prefix+".scale_dj0_",s,null);

    return n;
}
private int printAQVMValues(final AQVM x,final CalculationData c)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.p_eq);
    this.dattribs.addValue(x.sigma2_eq);
    this.dattribs.addValue(x.osp);
    this.dattribs.addValue(x.pi_eq);
    this.dattribs.addValue(x.sigma2pi_eq);
    this.dattribs.addValue(x.dpi_eq);

    n+=8;
    this.dattribs.addValue(x.scaleo_io);
    this.dattribs.addValue(x.di);
    this.dattribs.addValue(x.dip);
    this.dattribs.addValue(x.scale_io);
    this.dattribs.addValue(x.scale_io0);
    this.dattribs.addValue(x.dj);
    this.dattribs.addValue(x.scale_dj);
    this.dattribs.addValue(x.scale_dj0);

    return n;
}

private int printQVMLLNames(final String prefix,final String s)
{
    int n=0;

    n+=2;
    this.dattribs_names.addValue(prefix+".p_average_",s,null);
    this.dattribs_names.addValue(prefix+".sigma_",s,null);

    n+=1;
    this.dattribs_names.addValue(prefix+".I_",s,null);

    n+=21;
    this.dattribs_names.addValue(prefix+".paverage_eq_",s,null);
    this.dattribs_names.addValue(prefix+".p_eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_eq_",s,null);
    this.dattribs_names.addValue(prefix+".osp_",s,null);
    this.dattribs_names.addValue(prefix+".ospp_",s,null);
    this.dattribs_names.addValue(prefix+".pi_eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2pi_eq_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io0_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io_",s,null);
    this.dattribs_names.addValue(prefix+".di_1_",s,null);
    this.dattribs_names.addValue(prefix+".dip_1_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io1_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ii1_",s,null);
    this.dattribs_names.addValue(prefix+".di_2_",s,null);
    this.dattribs_names.addValue(prefix+".dip_2_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io2_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ii2_",s,null);
    this.dattribs_names.addValue(prefix+".Dosp_",s,null);
    this.dattribs_names.addValue(prefix+".scale_dip0_",s,null);
    this.dattribs_names.addValue(prefix+".scale_dip0a_",s,null);
    this.dattribs_names.addValue(prefix+".scale_ix_",s,null);

    return n;
}
private int printQVMLLValues(final QVMLL x,final CalculationData c)
{
    int n=0;

    n+=2;
    this.dattribs.addValue(x.p_average);
    this.dattribs.addValue(x.sigma);

    n+=1;
    this.dattribs.addValue(x.I);

    n+=21;
    this.dattribs.addValue(x.paverage_eq);
    this.dattribs.addValue(x.p_eq);
    this.dattribs.addValue(x.sigma2_eq);
    this.dattribs.addValue(x.osp);
    this.dattribs.addValue(x.ospp);
    this.dattribs.addValue(x.pi_eq);
    this.dattribs.addValue(x.sigma2pi_eq);
    this.dattribs.addValue(x.scale_io0);
    this.dattribs.addValue(x.scale_io);
    this.dattribs.addValue(x.di_1);
    this.dattribs.addValue(x.dip_1);
    this.dattribs.addValue(x.scale_io1);
    this.dattribs.addValue(x.sigma2_ii1);
    this.dattribs.addValue(x.di_2);
    this.dattribs.addValue(x.dip_2);
    this.dattribs.addValue(x.scale_io2);
    this.dattribs.addValue(x.sigma2_ii2);
    this.dattribs.addValue(x.Dosp);
    this.dattribs.addValue(x.scale_dip0);
    this.dattribs.addValue(x.scale_dip0a);
    this.dattribs.addValue(x.scale_ix);

    return n;
}



private int printQVMPPNames(final String prefix,final String s)
{
    int n=0;

    n+=2;
    this.dattribs_names.addValue(prefix+".p_average_",s,null);
    this.dattribs_names.addValue(prefix+".sigma_",s,null);

    n+=1;
    this.dattribs_names.addValue(prefix+".I_",s,null);

    n+=21;
    this.dattribs_names.addValue(prefix+".paverage_eq_",s,null);
    this.dattribs_names.addValue(prefix+".p_eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_eq_",s,null);
    this.dattribs_names.addValue(prefix+".osp_",s,null);
    this.dattribs_names.addValue(prefix+".ospp_",s,null);
    this.dattribs_names.addValue(prefix+".pi_eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2pi_eq_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io0_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io_",s,null);
    this.dattribs_names.addValue(prefix+".di_1_",s,null);
    this.dattribs_names.addValue(prefix+".dip_1_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io1_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ii1_",s,null);
    this.dattribs_names.addValue(prefix+".di_2_",s,null);
    this.dattribs_names.addValue(prefix+".dip_2_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io2_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ii2_",s,null);
    this.dattribs_names.addValue(prefix+".Dosp_",s,null);
    this.dattribs_names.addValue(prefix+".scale_dip0_",s,null);
    this.dattribs_names.addValue(prefix+".scale_dip0a_",s,null);
    this.dattribs_names.addValue(prefix+".scale_ix_",s,null);

    return n;
}
private int printQVMPPValues(final QVMPP x,final CalculationData c)
{
    int n=0;

    n+=2;
    this.dattribs.addValue(x.p_average);
    this.dattribs.addValue(x.sigma);

    n+=1;
    this.dattribs.addValue(x.I);

    n+=21;
    this.dattribs.addValue(x.paverage_eq);
    this.dattribs.addValue(x.p_eq);
    this.dattribs.addValue(x.sigma2_eq);
    this.dattribs.addValue(x.osp);
    this.dattribs.addValue(x.ospp);
    this.dattribs.addValue(x.pi_eq);
    this.dattribs.addValue(x.sigma2pi_eq);
    this.dattribs.addValue(x.scale_io0);
    this.dattribs.addValue(x.scale_io);
    this.dattribs.addValue(x.di_1);
    this.dattribs.addValue(x.dip_1);
    this.dattribs.addValue(x.scale_io1);
    this.dattribs.addValue(x.sigma2_ii1);
    this.dattribs.addValue(x.di_2);
    this.dattribs.addValue(x.dip_2);
    this.dattribs.addValue(x.scale_io2);
    this.dattribs.addValue(x.sigma2_ii2);
    this.dattribs.addValue(x.Dosp);
    this.dattribs.addValue(x.scale_dip0);
    this.dattribs.addValue(x.scale_dip0a);
    this.dattribs.addValue(x.scale_ix);

    return n;
}




private int printQVMRNames(final String prefix,final String s)
{
    int n=0;

    n+=3;
    this.dattribs_names.addValue(prefix+".p_eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigmat_eq_",s,null);
    this.dattribs_names.addValue(prefix+".pf_eq_",s,null);

    return n;
}
private <T> int printQVMRValues(final QVMR<T> x,final CalculationData c)
{
    int n=0;

    n+=3;
    this.dattribs.addValue(x.p_eq);
    this.dattribs.addValue(x.sigma2_eq);
    this.dattribs.addValue(x.pf_eq);

    return n;
}

private int printEQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=25;
    this.dattribs_names.addValue(prefix+".scale_ix_",s,null);
    this.dattribs_names.addValue(prefix+".io_",s,null);
    this.dattribs_names.addValue(prefix+".di_",s,null);
    this.dattribs_names.addValue(prefix+".dip_",s,null);
    this.dattribs_names.addValue(prefix+".dip_o_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dip_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dip_o_",s,null);
    this.dattribs_names.addValue(prefix+".dpii_",s,null);
    this.dattribs_names.addValue(prefix+".dpii_o_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dpii_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dpii_o_",s,null);
    this.dattribs_names.addValue(prefix+".scale_dip0_",s,null);
    this.dattribs_names.addValue(prefix+".scale_dip0a_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io0_",s,null);
    this.dattribs_names.addValue(prefix+".scale_io_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_eq_",s,null);
    this.dattribs_names.addValue(prefix+".osp_",s,null);
    this.dattribs_names.addValue(prefix+".tosp_",s,null);
    this.dattribs_names.addValue(prefix+".dp_eq_",s,null);
    this.dattribs_names.addValue(prefix+".Si_",s,null);
    this.dattribs_names.addValue(prefix+".ii_",s,null);
    this.dattribs_names.addValue(prefix+".scaleo_io_",s,null);
    this.dattribs_names.addValue(prefix+".dpi_eq_scaled_",s,null);
    this.dattribs_names.addValue(prefix+".scaleo_io_",s,null);
    this.dattribs_names.addValue(prefix+".scaleo_dp_eq_",s,null);
    n+=16;
    this.dattribs_names.addValue(prefix+".dj_",s,null);
    this.dattribs_names.addValue(prefix+".djp_",s,null);
    this.dattribs_names.addValue(prefix+".dpj_",s,null);
    this.dattribs_names.addValue(prefix+".dsi_eq_",s,null);
    this.dattribs_names.addValue(prefix+".dsj_",s,null);
    this.dattribs_names.addValue(prefix+".pm_osp_",s,null);
    this.dattribs_names.addValue(prefix+".scale_jo_",s,null);
    this.dattribs_names.addValue(prefix+".djsdjd_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_djsdjd_",s,null);
    this.dattribs_names.addValue(prefix+".djsdjd_o_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_djsdjd_o_",s,null);
    this.dattribs_names.addValue(prefix+".jsosp_",s,null);
    this.dattribs_names.addValue(prefix+".scale_jsd_",s,null);
    this.dattribs_names.addValue(prefix+".jsd_",s,null);
    this.dattribs_names.addValue(prefix+".djsdjd_x_",s,null);
    this.dattribs_names.addValue(prefix+".jsosp_x_",s,null);
    n+=20;
    this.dattribs_names.addValue(prefix+".js_",s,null);
    this.dattribs_names.addValue(prefix+".jdjdp_",s,null);
    this.dattribs_names.addValue(prefix+".jdp_eq_",s,null);
    this.dattribs_names.addValue(prefix+".jbs_o_",s,null);
    this.dattribs_names.addValue(prefix+".jbs_plus_o_",s,null);
    this.dattribs_names.addValue(prefix+".jbs_minus_o_",s,null);
    this.dattribs_names.addValue(prefix+".jdjp_plus_",s,null);
    this.dattribs_names.addValue(prefix+".jdjp_minus_",s,null);
    this.dattribs_names.addValue(prefix+".jdjp_",s,null);
    this.dattribs_names.addValue(prefix+".jdjp_d_",s,null);
    this.dattribs_names.addValue(prefix+".jdjp_dd0_",s,null);
    this.dattribs_names.addValue(prefix+".djp_sd0_",s,null);
    this.dattribs_names.addValue(prefix+".dpj_sd0_",s,null);
    this.dattribs_names.addValue(prefix+".dpo_sd0_",s,null);
    this.dattribs_names.addValue(prefix+".io_sd0_",s,null);
    this.dattribs_names.addValue(prefix+".si_sd0_",s,null);
    this.dattribs_names.addValue(prefix+".ss_sd0_",s,null);
    this.dattribs_names.addValue(prefix+".ii_sd0_",s,null);
    this.dattribs_names.addValue(prefix+".scale_jbs_",s,null);
    this.dattribs_names.addValue(prefix+".scale_jbs0_",s,null);
    n+=23;
    this.dattribs_names.addValue(prefix+".iplus_",s,null);
    this.dattribs_names.addValue(prefix+".iminus_",s,null);
    this.dattribs_names.addValue(prefix+".dix_",s,null);
    this.dattribs_names.addValue(prefix+".dixp_",s,null);
    this.dattribs_names.addValue(prefix+".dixpm_",s,null);
    this.dattribs_names.addValue(prefix+".scale_dix_",s,null);
    this.dattribs_names.addValue(prefix+".di_plus_",s,null);
    this.dattribs_names.addValue(prefix+".di_minus_",s,null);
    this.dattribs_names.addValue(prefix+".dip_plus_",s,null);
    this.dattribs_names.addValue(prefix+".dip_minus_",s,null);
    this.dattribs_names.addValue(prefix+".dpplus_o_",s,null);
    this.dattribs_names.addValue(prefix+".dpminus_o_",s,null);
    this.dattribs_names.addValue(prefix+".scale_di_plus_",s,null);
    this.dattribs_names.addValue(prefix+".scale_di_minus_",s,null);
    this.dattribs_names.addValue(prefix+".mosp_plus_",s,null);
    this.dattribs_names.addValue(prefix+".mosp_minus_",s,null);
    this.dattribs_names.addValue(prefix+".tmosp_plus_",s,null);
    this.dattribs_names.addValue(prefix+".tmosp_minus_",s,null);
    this.dattribs_names.addValue(prefix+".jo_",s,null);
    this.dattribs_names.addValue(prefix+".scale_j0_",s,null);
    this.dattribs_names.addValue(prefix+".jop_",s,null);
    this.dattribs_names.addValue(prefix+".djimp_",s,null);
    this.dattribs_names.addValue(prefix+".djdmdp_",s,null);


    n+=14;
    this.dattribs_names.addValue(prefix+".ca_re_pi_o_",s,null);
    this.dattribs_names.addValue(prefix+".ca_im_pi_o_",s,null);
    this.dattribs_names.addValue(prefix+".ca_imo_pi_o_",s,null);
    this.dattribs_names.addValue(prefix+".ca_sigma2_pi_o_",s,null);
    this.dattribs_names.addValue(prefix+".ca_re_pi_",s,null);
    this.dattribs_names.addValue(prefix+".ca_im_pi_",s,null);
    this.dattribs_names.addValue(prefix+".ca_sigma2_pi_",s,null);
    this.dattribs_names.addValue(prefix+".cb_re_pi_o_",s,null);
    this.dattribs_names.addValue(prefix+".cb_im_pi_o_",s,null);
    this.dattribs_names.addValue(prefix+".cb_imo_pi_o_",s,null);
    this.dattribs_names.addValue(prefix+".cb_sigma2_pi_o_",s,null);
    this.dattribs_names.addValue(prefix+".cb_re_pi_",s,null);
    this.dattribs_names.addValue(prefix+".cb_im_pi_",s,null);
    this.dattribs_names.addValue(prefix+".cb_sigma2_pi_",s,null);

    return n;
}
private <T> int printEQVMValues(final EQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=25;
    this.dattribs.addValue(x.scale_ix);
    this.dattribs.addValue(x.io);
    this.dattribs.addValue(x.di);
    this.dattribs.addValue(x.dip);
    this.dattribs.addValue(x.dip_o);
    this.dattribs.addValue(x.sigma2_dip);
    this.dattribs.addValue(x.sigma2_dip_o);
    this.dattribs.addValue(x.dpii);
    this.dattribs.addValue(x.dpii_o);
    this.dattribs.addValue(x.sigma2_dpii);
    this.dattribs.addValue(x.sigma2_dpii_o);
    this.dattribs.addValue(x.scale_dip0);
    this.dattribs.addValue(x.scale_dip0a);
    this.dattribs.addValue(x.scale_io0);
    this.dattribs.addValue(x.scale_io);
    this.dattribs.addValue(x.sigma2_eq);
    this.dattribs.addValue(x.osp);
    this.dattribs.addValue(x.tosp);
    this.dattribs.addValue(x.dp_eq);
    this.dattribs.addValue(x.Si);
    this.dattribs.addValue(x.ii);
    this.dattribs.addValue(x.scaleo_io);
    this.dattribs.addValue(x.dpi_eq_scaled);
    this.dattribs.addValue(x.scaleo_io);
    this.dattribs.addValue(x.scaleo_dp_eq);
    n+=16;
    this.dattribs.addValue(x.dj);
    this.dattribs.addValue(x.djp);
    this.dattribs.addValue(x.dpj);
    this.dattribs.addValue(x.dsi_eq);
    this.dattribs.addValue(x.dsj);
    this.dattribs.addValue(x.pm_osp);
    this.dattribs.addValue(x.scale_jo);
    this.dattribs.addValue(x.djsdjd);
    this.dattribs.addValue(x.sigma2_djsdjd);
    this.dattribs.addValue(x.djsdjd_o);
    this.dattribs.addValue(x.sigma2_djsdjd_o);
    this.dattribs.addValue(x.jsosp);
    this.dattribs.addValue(x.scale_jsd);
    this.dattribs.addValue(x.jsd);
    this.dattribs.addValue(x.djsdjd_x);
    this.dattribs.addValue(x.jsosp_x);
    n+=20;
    this.dattribs.addValue(x.js);
    this.dattribs.addValue(x.jdjdp);
    this.dattribs.addValue(x.jdp_eq);
    this.dattribs.addValue(x.jbs_o);
    this.dattribs.addValue(x.jbs_plus_o);
    this.dattribs.addValue(x.jbs_minus_o);
    this.dattribs.addValue(x.jdjp_plus);
    this.dattribs.addValue(x.jdjp_minus);
    this.dattribs.addValue(x.jdjp);
    this.dattribs.addValue(x.jdjp_d);
    this.dattribs.addValue(x.jdjp_dd0);
    this.dattribs.addValue(x.djp_sd0);
    this.dattribs.addValue(x.dpj_sd0);
    this.dattribs.addValue(x.dpo_sd0);
    this.dattribs.addValue(x.io_sd0);
    this.dattribs.addValue(x.si_sd0);
    this.dattribs.addValue(x.ss_sd0);
    this.dattribs.addValue(x.ii_sd0);
    this.dattribs.addValue(x.scale_jbs);
    this.dattribs.addValue(x.scale_jbs0);
    n+=23;
    this.dattribs.addValue(x.iplus);
    this.dattribs.addValue(x.iminus);
    this.dattribs.addValue(x.dix);
    this.dattribs.addValue(x.dixp);
    this.dattribs.addValue(x.dixpm);
    this.dattribs.addValue(x.scale_dix);
    this.dattribs.addValue(x.di_plus);
    this.dattribs.addValue(x.di_minus);
    this.dattribs.addValue(x.dip_plus);
    this.dattribs.addValue(x.dip_minus);
    this.dattribs.addValue(x.dpplus_o);
    this.dattribs.addValue(x.dpminus_o);
    this.dattribs.addValue(x.scale_di_plus);
    this.dattribs.addValue(x.scale_di_minus);
    this.dattribs.addValue(x.mosp_plus);
    this.dattribs.addValue(x.mosp_minus);
    this.dattribs.addValue(x.tmosp_plus);
    this.dattribs.addValue(x.tmosp_minus);
    this.dattribs.addValue(x.jo);
    this.dattribs.addValue(x.scale_j0);
    this.dattribs.addValue(x.jop);
    this.dattribs.addValue(x.djimp);
    this.dattribs.addValue(x.djdmdp);

    n+=14;
    this.dattribs.addValue(x.ca_re_pi_o);
    this.dattribs.addValue(x.ca_im_pi_o);
    this.dattribs.addValue(x.ca_imo_pi_o);
    this.dattribs.addValue(x.ca_sigma2_pi_o);
    this.dattribs.addValue(x.ca_re_pi);
    this.dattribs.addValue(x.ca_im_pi);
    this.dattribs.addValue(x.ca_sigma2_pi);
    this.dattribs.addValue(x.cb_re_pi_o);
    this.dattribs.addValue(x.cb_im_pi_o);
    this.dattribs.addValue(x.cb_imo_pi_o);
    this.dattribs.addValue(x.cb_sigma2_pi_o);
    this.dattribs.addValue(x.cb_re_pi);
    this.dattribs.addValue(x.cb_im_pi);
    this.dattribs.addValue(x.cb_sigma2_pi);

    return n;
}


private int printIQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".io_",s,null);
    this.dattribs_names.addValue(prefix+".jo_",s,null);
    this.dattribs_names.addValue(prefix+".qf_",s,null);
    this.dattribs_names.addValue(prefix+".q_eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_eq_",s,null);
    this.dattribs_names.addValue(prefix+".i_i_",s,null);
    this.dattribs_names.addValue(prefix+".x_q_eq_",s,null);
    this.dattribs_names.addValue(prefix+".x_sigma2_eq_",s,null);
    this.dattribs_names.addValue(prefix+".x_i_i_",s,null);

    return n;
}
private <T> int printIQVMValues(final IQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.io);
    this.dattribs.addValue(x.jo);
    this.dattribs.addValue(x.qf);
    this.dattribs.addValue(x.q_eq);
    this.dattribs.addValue(x.sigma2_eq);
    this.dattribs.addValue(x.i_i);
    this.dattribs.addValue(x.x_q_eq);
    this.dattribs.addValue(x.x_sigma2_eq);
    this.dattribs.addValue(x.x_i_i);

    return n;
}


private int printRQVMNames(final String prefix,final String s,final int na)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".wl_",s,null);
    this.dattribs_names.addValue(prefix+".dpl_",s,null);
    this.dattribs_names.addValue(prefix+".wl2_",s,null);
    this.dattribs_names.addValue(prefix+".dpl2_",s,null);
    this.dattribs_names.addValue(prefix+".wil_",s,null);
    this.dattribs_names.addValue(prefix+".dipl_",s,null);
    this.dattribs_names.addValue(prefix+".wil2_",s,null);
    this.dattribs_names.addValue(prefix+".dipl2_",s,null);
    if(na>0)
    {
	for(int j=0;j<na;j++)
	{
	    n+=10;
	    this.dattribs_names.addValue(prefix+".xnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".wnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".dpnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".dpnodes2_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".dipnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".dipnodes2_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".pnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p2nodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".pnodes2_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p2nodes2_"+j+"_",s,null);

	    n+=4;
	    this.dattribs_names.addValue(prefix+".xplus_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".wplus_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".xminus_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".wminus_"+j+"_",s,null);
	}
    }

    return n;
}
private <T> int printRQVMValues(final RQVM<T> x,final CalculationData c,final int na)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.wl);
    this.dattribs.addValue(x.dpl);
    this.dattribs.addValue(x.wl2);
    this.dattribs.addValue(x.dpl2);
    this.dattribs.addValue(x.wil);
    this.dattribs.addValue(x.dipl);
    this.dattribs.addValue(x.wil2);
    this.dattribs.addValue(x.dipl2);
    if(na>0)
    {
	for(int j=0;j<na;j++)
	{
	    n+=10;
	    this.dattribs.addValue(x.xnodes[j]);
	    this.dattribs.addValue(x.wnodes[j]);
	    this.dattribs.addValue(x.dpnodes[j]);
	    this.dattribs.addValue(x.dpnodes2[j]);
	    this.dattribs.addValue(x.dipnodes[j]);
	    this.dattribs.addValue(x.dipnodes2[j]);
	    this.dattribs.addValue(x.pnodes[j]);
	    this.dattribs.addValue(x.p2nodes[j]);
	    this.dattribs.addValue(x.pnodes2[j]);
	    this.dattribs.addValue(x.p2nodes2[j]);

	    n+=4;
	    this.dattribs.addValue(x.xplus[j]);
	    this.dattribs.addValue(x.wplus[j]);
	    this.dattribs.addValue(x.xminus[j]);
	    this.dattribs.addValue(x.wminus[j]);
	}
    }


    return n;
}


private int printSQVMNames(final String prefix,final String s,final int na)
{
    int n=0;

    if(na>0)
    {
	for(int j=0;j<na;j++)
	{
	    n+=12;
	    this.dattribs_names.addValue(prefix+".xnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".wnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".dpnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".dipnodes_"+j+"_",s,null);

	    this.dattribs_names.addValue(prefix+".pnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p2nodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".pnodes2_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p2nodes2_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".pnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p2nodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".pnodes2_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p2nodes2_"+j+"_",s,null);
	}
    }

    return n;
}
private <T> int printSQVMValues(final SQVM<T> x,final CalculationData c,final int na)
{
    int n=0;

    if(na>0)
    {
	for(int j=0;j<na;j++)
	{
	    n+=12;
	    this.dattribs.addValue(x.xnodes[j]);
	    this.dattribs.addValue(x.wnodes[j]);
	    this.dattribs.addValue(x.dpnodes[j]);
	    this.dattribs.addValue(x.dipnodes[j]);

	    this.dattribs.addValue(x.pnodes[j]);
	    this.dattribs.addValue(x.p2nodes[j]);
	    this.dattribs.addValue(x.pnodes2[j]);
	    this.dattribs.addValue(x.p2nodes2[j]);
	    this.dattribs.addValue(x.pnodes[j]);
	    this.dattribs.addValue(x.p2nodes[j]);
	    this.dattribs.addValue(x.pnodes2[j]);
	    this.dattribs.addValue(x.p2nodes2[j]);
	}
    }

    return n;
}


private int printKQVMNames(final String prefix,final String s,final int na)
{
    int n=0;
    n+=6;
    this.dattribs_names.addValue(prefix+".df_",s,null);
    this.dattribs_names.addValue(prefix+".dp_ll_",s,null);
    this.dattribs_names.addValue(prefix+".dp_ld_",s,null);
    this.dattribs_names.addValue(prefix+".dp_le_",s,null);
    this.dattribs_names.addValue(prefix+".dp_nle_",s,null);
    this.dattribs_names.addValue(prefix+".dp_nld_",s,null);

    if(na>0)
    {
	for(int j=0;j<na;j++)
	{
	    n+=3;
	    this.dattribs_names.addValue(prefix+".x_lexcess_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".w_lexcess_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".dp_lexcess_"+j+"_",s,null);
	    if(j<na-1)
	    {
		n+=3;
		this.dattribs_names.addValue(prefix+".x_ldeficit_"+j+"_",s,null);
		this.dattribs_names.addValue(prefix+".w_ldeficit_"+j+"_",s,null);
		this.dattribs_names.addValue(prefix+".dp_ldeficit_"+j+"_",s,null);
		n+=1;
		this.dattribs_names.addValue(prefix+".apnodes_"+j+"_",s,null);
	    }



	    n+=10;
	    this.dattribs_names.addValue(prefix+".xr_ldeficit_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".wr_ldeficit_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".xnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".wnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".dpnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".dipnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".sxnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".swnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".sdpnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".sdipnodes_"+j+"_",s,null);

	    if(j<na-1)
	    {
	    n+=8;
	    this.dattribs_names.addValue(prefix+".axnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".awnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".adpnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".adipnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".saxnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".sawnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".sadpnodes_"+j+"_",s,null);
	    this.dattribs_names.addValue(prefix+".sadipnodes_"+j+"_",s,null);
	    }
	}
    }

    return n;
}
private <T> int printKQVMValues(final KQVM<T> x,final CalculationData c,final int na)
{
    int n=0;
    n+=6;
    this.dattribs.addValue(x.df);
    this.dattribs.addValue(x.dp_ll);
    this.dattribs.addValue(x.dp_ld);
    this.dattribs.addValue(x.dp_le);
    this.dattribs.addValue(x.dp_nle);
    this.dattribs.addValue(x.dp_nld);

    if(na>0)
    {
	for(int j=0;j<na;j++)
	{
	    n+=3;
	    this.dattribs.addValue(x.x_lexcess[j]);
	    this.dattribs.addValue(x.w_lexcess[j]);
	    this.dattribs.addValue(x.dp_lexcess[j]);
	    if(j<na-1)
	    {
		n+=3;
		this.dattribs.addValue(x.x_ldeficit[j]);
		this.dattribs.addValue(x.w_ldeficit[j]);
		this.dattribs.addValue(x.dp_ldeficit[j]);
		n+=1;
		this.dattribs.addValue(x.apnodes[j]);
	    }

	    n+=10;
	    this.dattribs.addValue(x.xr_ldeficit[j]);
	    this.dattribs.addValue(x.wr_ldeficit[j]);
	    this.dattribs.addValue(x.xnodes[j]);
	    this.dattribs.addValue(x.wnodes[j]);
	    this.dattribs.addValue(x.dpnodes[j]);
	    this.dattribs.addValue(x.dipnodes[j]);
	    this.dattribs.addValue(x.sxnodes[j]);
	    this.dattribs.addValue(x.swnodes[j]);
	    this.dattribs.addValue(x.sdpnodes[j]);
	    this.dattribs.addValue(x.sdipnodes[j]);

	    if(j<na-1)
	    {
	    n+=8;
	    this.dattribs.addValue(x.axnodes[j]);
	    this.dattribs.addValue(x.awnodes[j]);
	    this.dattribs.addValue(x.adpnodes[j]);
	    this.dattribs.addValue(x.adipnodes[j]);
	    this.dattribs.addValue(x.saxnodes[j]);
	    this.dattribs.addValue(x.sawnodes[j]);
	    this.dattribs.addValue(x.sadpnodes[j]);
	    this.dattribs.addValue(x.sadipnodes[j]);
	    }
	}
    }

    return n;
}

private int printLQVMNames(final String prefix,final String s,final int na)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".dp2t_",s,null);
    this.dattribs_names.addValue(prefix+".dp2i_",s,null);
    this.dattribs_names.addValue(prefix+".ds2i_",s,null);
    this.dattribs_names.addValue(prefix+".dw2t_",s,null);
    this.dattribs_names.addValue(prefix+".dw2i_",s,null);
    this.dattribs_names.addValue(prefix+".dpd2t_",s,null);
    this.dattribs_names.addValue(prefix+".dpd2i_",s,null);
    this.dattribs_names.addValue(prefix+".dp00_",s,null);
    this.dattribs_names.addValue(prefix+".v00_",s,null);

    n+=8;
    this.dattribs_names.addValue(prefix+".sigma2_dpi0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dpt0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dsi0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dst0_",s,null);
    this.dattribs_names.addValue(prefix+".dpi0_",s,null);
    this.dattribs_names.addValue(prefix+".dpt0_",s,null);
    this.dattribs_names.addValue(prefix+".dsi0_",s,null);
    this.dattribs_names.addValue(prefix+".dst0_",s,null);

    for(int j=0;j<na;j++)
    {
	n+=10;
	this.dattribs_names.addValue(prefix+".xinodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".wiinodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".wtinodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".dpinodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".dsinodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".xtnodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".wttnodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".witnodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".dptnodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".dstnodes_"+j+"_",s,null);
    }
    return n;
}
private <T> int printLQVMValues(final LQVM<T> x,final CalculationData c,final int na)
{
    int n=0;
    n+=9;
    this.dattribs.addValue(x.dp2t);
    this.dattribs.addValue(x.dp2i);
    this.dattribs.addValue(x.ds2i);
    this.dattribs.addValue(x.dw2t);
    this.dattribs.addValue(x.dw2i);
    this.dattribs.addValue(x.dpd2t);
    this.dattribs.addValue(x.dpd2i);
    this.dattribs.addValue(x.dp00);
    this.dattribs.addValue(x.v00);

    n+=8;
    this.dattribs.addValue(x.sigma2_dpi0);
    this.dattribs.addValue(x.sigma2_dpt0);
    this.dattribs.addValue(x.sigma2_dsi0);
    this.dattribs.addValue(x.sigma2_dst0);
    this.dattribs.addValue(x.dpi0);
    this.dattribs.addValue(x.dpt0);
    this.dattribs.addValue(x.dsi0);
    this.dattribs.addValue(x.dst0);

    for(int j=0;j<na;j++)
    {
	n+=10;
	this.dattribs.addValue(x.xinodes[j]);
	this.dattribs.addValue(x.wiinodes[j]);
	this.dattribs.addValue(x.wtinodes[j]);
	this.dattribs.addValue(x.dpinodes[j]);
	this.dattribs.addValue(x.dsinodes[j]);
	this.dattribs.addValue(x.xtnodes[j]);
	this.dattribs.addValue(x.wttnodes[j]);
	this.dattribs.addValue(x.witnodes[j]);
	this.dattribs.addValue(x.dptnodes[j]);
	this.dattribs.addValue(x.dstnodes[j]);
    }
    return n;
}


private int printMQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=10;
    this.dattribs_names.addValue(prefix+".dp0_",s,null);
    this.dattribs_names.addValue(prefix+".ds0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dp0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ds0_",s,null);
    this.dattribs_names.addValue(prefix+".dp00_",s,null);
    this.dattribs_names.addValue(prefix+".v00_",s,null);
    this.dattribs_names.addValue(prefix+".dp0v_",s,null);
    this.dattribs_names.addValue(prefix+".ds0v_",s,null);
    this.dattribs_names.addValue(prefix+".dp0i_",s,null);
    this.dattribs_names.addValue(prefix+".ds0i_",s,null);

    return n;
}
private <T> int printMQVMValues(final MQVM<T> x,final CalculationData c)
{
    int n=0;
    n+=10;
    this.dattribs.addValue(x.dp0);
    this.dattribs.addValue(x.ds0);
    this.dattribs.addValue(x.sigma2_dp0);
    this.dattribs.addValue(x.sigma2_ds0);
    this.dattribs.addValue(x.dp00);
    this.dattribs.addValue(x.v00);
    this.dattribs.addValue(x.dp0v);
    this.dattribs.addValue(x.ds0v);
    this.dattribs.addValue(x.dp0i);
    this.dattribs.addValue(x.ds0i);

    return n;
}



private int printNQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=18;
    this.dattribs_names.addValue(prefix+".dpexcess0r_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dpexcess0r_",s,null);
    this.dattribs_names.addValue(prefix+".dpdeficit0r_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dpdeficit0r_",s,null);
    this.dattribs_names.addValue(prefix+".dsexcess0r_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dsexcess0r_",s,null);
    this.dattribs_names.addValue(prefix+".dsdeficit0r_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dsdeficit0r_",s,null);
    this.dattribs_names.addValue(prefix+".wexcess0r_",s,null);
    this.dattribs_names.addValue(prefix+".wdeficit0r_",s,null);
    this.dattribs_names.addValue(prefix+".dpi0r_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dpi0r_",s,null);
    this.dattribs_names.addValue(prefix+".dsi0r_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dsi0r_",s,null);
    this.dattribs_names.addValue(prefix+".wii0r_",s,null);
    this.dattribs_names.addValue(prefix+".dtdv0r_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dtdv0r_",s,null);
    this.dattribs_names.addValue(prefix+".dp00_",s,null);

    n+=9;
    this.dattribs_names.addValue(prefix+".wii0S_",s,null);
    this.dattribs_names.addValue(prefix+".dpi0S_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dpi0S_",s,null);
    this.dattribs_names.addValue(prefix+".dapi0S_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dapi0S_",s,null);
    this.dattribs_names.addValue(prefix+".dki0S_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dki0S_",s,null);
    this.dattribs_names.addValue(prefix+".dS0S_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dS0S_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".vplus00_",s,null);
    this.dattribs_names.addValue(prefix+".vminus00_",s,null);
    this.dattribs_names.addValue(prefix+".wiplus0r_",s,null);
    this.dattribs_names.addValue(prefix+".wiminus0r_",s,null);

    n+=10;
    this.dattribs_names.addValue(prefix+".dpplus0_",s,null);
    this.dattribs_names.addValue(prefix+".dpminus0_",s,null);
    this.dattribs_names.addValue(prefix+".dvplus0_",s,null);
    this.dattribs_names.addValue(prefix+".dvminus0_",s,null);
    this.dattribs_names.addValue(prefix+".w0_",s,null);
    this.dattribs_names.addValue(prefix+".dpplus00_",s,null);
    this.dattribs_names.addValue(prefix+".dpminus00_",s,null);
    this.dattribs_names.addValue(prefix+".dvplus00_",s,null);
    this.dattribs_names.addValue(prefix+".dvminus00_",s,null);
    this.dattribs_names.addValue(prefix+".w00_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".dpdvdiff_sigma2_",s,null);
    this.dattribs_names.addValue(prefix+".dSw_",s,null);
    this.dattribs_names.addValue(prefix+".dSd_",s,null);
    this.dattribs_names.addValue(prefix+".dSa_",s,null);

    return n;
}
private <T> int printNQVMValues(final NQVM<T> x,final CalculationData c)
{
    int n=0;
    n+=18;
    this.dattribs.addValue(x.dpexcess0r);
    this.dattribs.addValue(x.sigma2_dpexcess0r);
    this.dattribs.addValue(x.dpdeficit0r);
    this.dattribs.addValue(x.sigma2_dpdeficit0r);
    this.dattribs.addValue(x.dsexcess0r);
    this.dattribs.addValue(x.sigma2_dsexcess0r);
    this.dattribs.addValue(x.dsdeficit0r);
    this.dattribs.addValue(x.sigma2_dsdeficit0r);
    this.dattribs.addValue(x.wexcess0r);
    this.dattribs.addValue(x.wdeficit0r);
    this.dattribs.addValue(x.dpi0r);
    this.dattribs.addValue(x.sigma2_dpi0r);
    this.dattribs.addValue(x.dsi0r);
    this.dattribs.addValue(x.sigma2_dsi0r);
    this.dattribs.addValue(x.wii0r);
    this.dattribs.addValue(x.dtdv0r);
    this.dattribs.addValue(x.sigma2_dtdv0r);
    this.dattribs.addValue(x.dp00);

    n+=9;
    this.dattribs.addValue(x.wii0S);
    this.dattribs.addValue(x.dpi0S);
    this.dattribs.addValue(x.sigma2_dpi0S);
    this.dattribs.addValue(x.dapi0S);
    this.dattribs.addValue(x.sigma2_dapi0S);
    this.dattribs.addValue(x.dki0S);
    this.dattribs.addValue(x.sigma2_dki0S);
    this.dattribs.addValue(x.dS0S);
    this.dattribs.addValue(x.sigma2_dS0S);

    n+=4;
    this.dattribs.addValue(x.vplus00);
    this.dattribs.addValue(x.vminus00);
    this.dattribs.addValue(x.wiplus0r);
    this.dattribs.addValue(x.wiminus0r);

    n+=10;
    this.dattribs.addValue(x.dpplus0);
    this.dattribs.addValue(x.dpminus0);
    this.dattribs.addValue(x.dvplus0);
    this.dattribs.addValue(x.dvminus0);
    this.dattribs.addValue(x.w0);
    this.dattribs.addValue(x.dpplus00);
    this.dattribs.addValue(x.dpminus00);
    this.dattribs.addValue(x.dvplus00);
    this.dattribs.addValue(x.dvminus00);
    this.dattribs.addValue(x.w00);

    n+=4;
    this.dattribs.addValue(x.dpdvdiff_sigma2);
    this.dattribs.addValue(x.dSw);
    this.dattribs.addValue(x.dSd);
    this.dattribs.addValue(x.dSa);


    return n;
}




private int printOQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=11;
    this.dattribs_names.addValue(prefix+".w00_",s,null);
    this.dattribs_names.addValue(prefix+".wa_",s,null);
    this.dattribs_names.addValue(prefix+".xeq_",s,null);
    this.dattribs_names.addValue(prefix+".peq_",s,null);
    this.dattribs_names.addValue(prefix+".p2eq_",s,null);
    this.dattribs_names.addValue(prefix+".dLp_",s,null);
    this.dattribs_names.addValue(prefix+".dpL_",s,null);
    this.dattribs_names.addValue(prefix+".dpLE_",s,null);
    this.dattribs_names.addValue(prefix+".dpLE_sigma2_",s,null);
    this.dattribs_names.addValue(prefix+".dpLE0_sigma2_",s,null);
    this.dattribs_names.addValue(prefix+".dpxeq_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".dLs_",s,null);
    this.dattribs_names.addValue(prefix+".dsL_",s,null);
    this.dattribs_names.addValue(prefix+".dsLE_",s,null);
    this.dattribs_names.addValue(prefix+".dsLE_sigma2_",s,null);
    this.dattribs_names.addValue(prefix+".dsLE0_sigma2_",s,null);
    this.dattribs_names.addValue(prefix+".dsxeq_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".kka_",s,null);
    this.dattribs_names.addValue(prefix+".kkasell_",s,null);
    this.dattribs_names.addValue(prefix+".kkabuy_",s,null);
    this.dattribs_names.addValue(prefix+".kka0_",s,null);
    this.dattribs_names.addValue(prefix+".kkasell0_",s,null);
    this.dattribs_names.addValue(prefix+".kkabuy0_",s,null);

    n+=7;
    this.dattribs_names.addValue(prefix+".xeqdVdT_",s,null);
    this.dattribs_names.addValue(prefix+".xoutdVdT_",s,null);
    this.dattribs_names.addValue(prefix+".dIxout_",s,null);
    this.dattribs_names.addValue(prefix+".dIx0_",s,null);
    this.dattribs_names.addValue(prefix+".dIxend_",s,null);
    this.dattribs_names.addValue(prefix+".xeqsell_",s,null);
    this.dattribs_names.addValue(prefix+".xeqbuy_",s,null);

    n+=5;
    this.dattribs_names.addValue(prefix+".Ik_",s,null);
    this.dattribs_names.addValue(prefix+".dIkL_",s,null);
    this.dattribs_names.addValue(prefix+".dIt_",s,null);
    this.dattribs_names.addValue(prefix+".dIv_",s,null);
    this.dattribs_names.addValue(prefix+".dIa_",s,null);

    n+=10;
    this.dattribs_names.addValue(prefix+".dbsIk_",s,null);
    this.dattribs_names.addValue(prefix+".dbsIkL_",s,null);
    this.dattribs_names.addValue(prefix+".dbsIksL_",s,null);
    this.dattribs_names.addValue(prefix+".dbsIkbL_",s,null);
    this.dattribs_names.addValue(prefix+".dbsa_",s,null);
    this.dattribs_names.addValue(prefix+".dpmIk_",s,null);
    this.dattribs_names.addValue(prefix+".dpmIkL_",s,null);
    this.dattribs_names.addValue(prefix+".dpmIksL_",s,null);
    this.dattribs_names.addValue(prefix+".dpmIkbL_",s,null);
    this.dattribs_names.addValue(prefix+".dpma_",s,null);

    return n;
}
private <T> int printOQVMValues(final OQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=11;
    this.dattribs.addValue(x.w00);
    this.dattribs.addValue(x.wa);
    this.dattribs.addValue(x.xeq);
    this.dattribs.addValue(x.peq);
    this.dattribs.addValue(x.p2eq);
    this.dattribs.addValue(x.dLp);
    this.dattribs.addValue(x.dpL);
    this.dattribs.addValue(x.dpLE);
    this.dattribs.addValue(x.dpLE_sigma2);
    this.dattribs.addValue(x.dpLE0_sigma2);
    this.dattribs.addValue(x.dpxeq);

    n+=6;
    this.dattribs.addValue(x.dLs);
    this.dattribs.addValue(x.dsL);
    this.dattribs.addValue(x.dsLE);
    this.dattribs.addValue(x.dsLE_sigma2);
    this.dattribs.addValue(x.dsLE0_sigma2);
    this.dattribs.addValue(x.dsxeq);

    n+=6;
    this.dattribs.addValue(x.kka);
    this.dattribs.addValue(x.kkasell);
    this.dattribs.addValue(x.kkabuy);
    this.dattribs.addValue(x.kka0);
    this.dattribs.addValue(x.kkasell0);
    this.dattribs.addValue(x.kkabuy0);

    n+=7;
    this.dattribs.addValue(x.xeqdVdT);
    this.dattribs.addValue(x.xoutdVdT);
    this.dattribs.addValue(x.dIxout);
    this.dattribs.addValue(x.dIx0);
    this.dattribs.addValue(x.dIxend);
    this.dattribs.addValue(x.xeqsell);
    this.dattribs.addValue(x.xeqbuy);

    n+=5;
    this.dattribs.addValue(x.Ik);
    this.dattribs.addValue(x.dIkL);
    this.dattribs.addValue(x.dIt);
    this.dattribs.addValue(x.dIv);
    this.dattribs.addValue(x.dIa);

    n+=10;
    this.dattribs.addValue(x.dbsIk);
    this.dattribs.addValue(x.dbsIkL);
    this.dattribs.addValue(x.dbsIksL);
    this.dattribs.addValue(x.dbsIkbL);
    this.dattribs.addValue(x.dbsa);
    this.dattribs.addValue(x.dpmIk);
    this.dattribs.addValue(x.dpmIkL);
    this.dattribs.addValue(x.dpmIksL);
    this.dattribs.addValue(x.dpmIkbL);
    this.dattribs.addValue(x.dpma);

    return n;
}


private int TwoRootsDataNames(final String prefix,final String s)
{
    int n=0;
    n+=14;
    this.dattribs_names.addValue(prefix+".x0_",s,null);
    this.dattribs_names.addValue(prefix+".x1_",s,null);
    this.dattribs_names.addValue(prefix+".x2_",s,null);
    this.dattribs_names.addValue(prefix+".F0_",s,null);
    this.dattribs_names.addValue(prefix+".F1_",s,null);
    this.dattribs_names.addValue(prefix+".F2_",s,null);
    this.dattribs_names.addValue(prefix+".dF0_",s,null);
    this.dattribs_names.addValue(prefix+".dF1_",s,null);
    this.dattribs_names.addValue(prefix+".dF2_",s,null);
    this.dattribs_names.addValue(prefix+".d2F0_",s,null);
    this.dattribs_names.addValue(prefix+".d2F1_",s,null);
    this.dattribs_names.addValue(prefix+".d2F2_",s,null);
    this.dattribs_names.addValue(prefix+".x1_type_",s,null);
    this.dattribs_names.addValue(prefix+".x2_type_",s,null);
    return n;
}
private int TwoRootsDataValues(final TwoCloseRootsFinder.RootsData x)
{
    int n=0;
    n+=14;
    this.dattribs.addValue(x.x0);
    this.dattribs.addValue(x.x1);
    this.dattribs.addValue(x.x2);
    this.dattribs.addValue(x.F0);
    this.dattribs.addValue(x.F1);
    this.dattribs.addValue(x.F2);
    this.dattribs.addValue(x.dF0);
    this.dattribs.addValue(x.dF1);
    this.dattribs.addValue(x.dF2);
    this.dattribs.addValue(x.d2F0);
    this.dattribs.addValue(x.d2F1);
    this.dattribs.addValue(x.d2F2);
    this.dattribs.addValue(x.x1_type);
    this.dattribs.addValue(x.x2_type);
    return n;
}

private int printQQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=13;
    this.dattribs_names.addValue(prefix+".xeq_",s,null);
    this.dattribs_names.addValue(prefix+".K00t_",s,null);
    this.dattribs_names.addValue(prefix+".K00v_",s,null);
    this.dattribs_names.addValue(prefix+".A00v_",s,null);
    this.dattribs_names.addValue(prefix+".KEQt_",s,null);
    this.dattribs_names.addValue(prefix+".KEQv_",s,null);
    this.dattribs_names.addValue(prefix+".AEQv_",s,null);
    this.dattribs_names.addValue(prefix+".I0_",s,null);
    this.dattribs_names.addValue(prefix+".Ia0eq_",s,null);
    this.dattribs_names.addValue(prefix+".Ieq_",s,null);
    this.dattribs_names.addValue(prefix+".Iaeq_",s,null);
    this.dattribs_names.addValue(prefix+".SI0_",s,null);
    this.dattribs_names.addValue(prefix+".SIeq_",s,null);

    n+=TwoRootsDataNames(prefix+".d",s);

    n+=8;
    this.dattribs_names.addValue(prefix+".ptE_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ptE_",s,null);
    this.dattribs_names.addValue(prefix+".pt0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pt0_",s,null);
    this.dattribs_names.addValue(prefix+".pvE_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pvE_",s,null);
    this.dattribs_names.addValue(prefix+".pv0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pv0_",s,null);

    return n;

}
private <T> int printQQVMValues(final QQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=13;
    this.dattribs.addValue(x.xeq);
    this.dattribs.addValue(x.K00t);
    this.dattribs.addValue(x.K00v);
    this.dattribs.addValue(x.A00v);
    this.dattribs.addValue(x.KEQt);
    this.dattribs.addValue(x.KEQv);
    this.dattribs.addValue(x.AEQv);
    this.dattribs.addValue(x.I0);
    this.dattribs.addValue(x.Ia0eq);
    this.dattribs.addValue(x.Ieq);
    this.dattribs.addValue(x.Iaeq);
    this.dattribs.addValue(x.SI0);
    this.dattribs.addValue(x.SIeq);

    n+=TwoRootsDataValues(x.d);

    n+=8;
    this.dattribs.addValue(x.ptE);
    this.dattribs.addValue(x.sigma2_ptE);
    this.dattribs.addValue(x.pt0);
    this.dattribs.addValue(x.sigma2_pt0);
    this.dattribs.addValue(x.pvE);
    this.dattribs.addValue(x.sigma2_pvE);
    this.dattribs.addValue(x.pv0);
    this.dattribs.addValue(x.sigma2_pv0);

    return n;

}


private int EVXData_Names(final String prefix,final String s)
{
    int n=0;
    n+=6;
    this.dattribs_names.addValue(prefix+".wL_",s,null);
    this.dattribs_names.addValue(prefix+".sL_",s,null);
    this.dattribs_names.addValue(prefix+".wH_",s,null);
    this.dattribs_names.addValue(prefix+".sH_",s,null);
    this.dattribs_names.addValue(prefix+".s0_",s,null);
    this.dattribs_names.addValue(prefix+".o_o_",s,null);

    return n;
}
private int EVXData_Values(final EVXData x)
{
    int n=0;
    n+=6;
    this.dattribs.addValue(x.wL);
    this.dattribs.addValue(x.sL);
    this.dattribs.addValue(x.wH);
    this.dattribs.addValue(x.sH);
    this.dattribs.addValue(x.s0);
    this.dattribs.addValue(x.o_o);

    return n;
}


private int EVX2Data_Names(final String prefix,final String s)
{
    int n=0;
    n+=8;
    this.dattribs_names.addValue(prefix+".sIL_",s,null);
    this.dattribs_names.addValue(prefix+".sIH_",s,null);
    this.dattribs_names.addValue(prefix+".sfL_",s,null);
    this.dattribs_names.addValue(prefix+".sfH_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_fL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_fH_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_fL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_fH_",s,null);

    n+=printACorrNames(prefix+".a",s);

    return n;
}
private int EVX2Data_Values(final EVX2Data x)
{
    int n=0;
    n+=8;
    this.dattribs.addValue(x.sIL);
    this.dattribs.addValue(x.sIH);
    this.dattribs.addValue(x.sfL);
    this.dattribs.addValue(x.sfH);
    this.dattribs.addValue(x.w_IL_fL);
    this.dattribs.addValue(x.w_IL_fH);
    this.dattribs.addValue(x.w_IH_fL);
    this.dattribs.addValue(x.w_IH_fH);

    n+=printACorrValues(x.a);

    return n;
}

private int SMMStateSaver_Names(final String prefix,final String s)
{
    int n=0;

    n+=SMMStateSaver_SMM_Names(prefix+".L_before",s);
    n+=SMMStateSaver_SMM_Names(prefix+".H",s);
    n+=SMMStateSaver_SMM_Names(prefix+".L_after",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".hPcMin_",s,null);
    this.dattribs_names.addValue(prefix+".hPcMax_",s,null);
    this.dattribs_names.addValue(prefix+".getVb_",s,null);
    this.dattribs_names.addValue(prefix+".getVf_",s,null);

    return n;
}
private int SMMStateSaver_Values(final SMMStateSaver x)
{
    int n=0;
    n+=SMMStateSaver_SMM_Values(x.L_before);
    n+=SMMStateSaver_SMM_Values(x.H);
    n+=SMMStateSaver_SMM_Values(x.L_after);

    n+=4;
    this.dattribs.addValue(x.hPcMin);
    this.dattribs.addValue(x.hPcMax);
    this.dattribs.addValue(x.getVb());
    this.dattribs.addValue(x.getVf());

    return n;
}


private int SMMXStateSaver_Names(final String prefix,final String s)
{
    int n=0;

    n+=SMMStateSaver_SMM_Names(prefix+".L",s);
    n+=SMMStateSaver_SMM_Names(prefix+".H",s);

    return n;
}
private int SMMXStateSaver_Values(final SMMXStateSaver x)
{
    int n=0;
    n+=SMMStateSaver_SMM_Values(x.L);
    n+=SMMStateSaver_SMM_Values(x.H);


    return n;
}



private int StateSaver_Names(final String prefix,final String s)
{
    int n=0;

    n+=SStateSaver_Names(prefix+".L_before",s);
    n+=SStateSaver_Names(prefix+".H",s);
    n+=SStateSaver_Names(prefix+".L_after",s);

    n+=3;
    this.dattribs_names.addValue(prefix+".getUb_",s,null);
    this.dattribs_names.addValue(prefix+".getVb_",s,null);
    this.dattribs_names.addValue(prefix+".getVf_",s,null);

    return n;
}
private int StateSaver_Values(final StateSaver x)
{
    int n=0;
    n+=SStateSaver_Values(x.L_before);
    n+=SStateSaver_Values(x.H);
    n+=SStateSaver_Values(x.L_after);

    n+=3;
    this.dattribs.addValue(x.getUb());
    this.dattribs.addValue(x.getVb());
    this.dattribs.addValue(x.getVf());

    return n;
}

private int FStateSaver_Names(final String prefix,final String s)
{
    int n=0;

    n+=SStateSaver_Names(prefix+".J",s);

    n+=2;
    this.dattribs_names.addValue(prefix+".getPMin_",s,null);
    this.dattribs_names.addValue(prefix+".getPMax_",s,null);

    return n;
}
private int FStateSaver_Values(final FStateSaver x)
{
    int n=0;
    n+=SStateSaver_Values(x.J);

    n+=2;
    this.dattribs.addValue(x.getPMin());
    this.dattribs.addValue(x.getPMax());

    return n;
}


private int MinMaxStateSaver_Names(final String prefix,final String s)
{
    int n=0;
    n+=SStateSaver_Names(prefix+".Min",s);
    n+=SStateSaver_Names(prefix+".Max",s);
    return n;
}
private int MinMaxStateSaver_Values(final MinMaxStateSaver x)
{
    int n=0;
    n+=SStateSaver_Values(x.Min);
    n+=SStateSaver_Values(x.Max);
    return n;
}


private int MaxStateSaver_Names(final String prefix,final String s)
{
    int n=0;
    n+=SStateSaver_Names(prefix+".Max",s);
    return n;
}
private int MaxStateSaver_Values(final MaxStateSaver x)
{
    int n=0;
    n+=SStateSaver_Values(x.Max);
    return n;
}


private int MMaxStateSaver_Names(final String prefix,final String s)
{
    int n=0;
    n+=MMaxStateSaver_S_Names(prefix+".Max",s);
    n+=1;
    this.dattribs_names.addValue(prefix+".isTainted_",s,null);
    return n;
}
private int MMaxStateSaver_Values(final MMaxStateSaver x)
{
    int n=0;
    n+=MMaxStateSaver_S_Values(x.Max);
    n+=1;
    this.dattribs.addValue(x.isTainted()?1:0);
    return n;
}




private int SStateSaver_Names(final String prefix,final String s)
{
    int n=0;
    n+=6;
    this.dattribs_names.addValue(prefix+".Ic_",s,null);
    this.dattribs_names.addValue(prefix+".Pc_",s,null);
    this.dattribs_names.addValue(prefix+".Tc_",s,null);
    this.dattribs_names.addValue(prefix+".Fc_",s,null);
    this.dattribs_names.addValue(prefix+".Uc_",s,null);
    this.dattribs_names.addValue(prefix+".Vc_",s,null);

    return n;
}
private int SStateSaver_Values(final StateSaver.S x)
{
    int n=0;
    n+=6;
    this.dattribs.addValue(x.Ic);
    this.dattribs.addValue(x.Pc);
    this.dattribs.addValue(x.Tc);
    this.dattribs.addValue(x.Fc);
    this.dattribs.addValue(x.Uc);
    this.dattribs.addValue(x.Vc);

    return n;
}


private int MMaxStateSaver_S_Names(final String prefix,final String s)
{
    int n=0;
    n+=9;
    this.dattribs_names.addValue(prefix+".Ic_",s,null);
    this.dattribs_names.addValue(prefix+".Pc_",s,null);
    this.dattribs_names.addValue(prefix+".Tc_",s,null);
    this.dattribs_names.addValue(prefix+".Fc_",s,null);
    this.dattribs_names.addValue(prefix+".FL_",s,null);
    this.dattribs_names.addValue(prefix+".FH_",s,null);
    this.dattribs_names.addValue(prefix+".volume_",s,null);
    this.dattribs_names.addValue(prefix+".a_",s,null);
    this.dattribs_names.addValue(prefix+".b_",s,null);

    return n;
}
private int MMaxStateSaver_S_Values(final MMaxStateSaver.S x)
{
    int n=0;
    n+=9;
    this.dattribs.addValue(x.Ic);
    this.dattribs.addValue(x.Pc);
    this.dattribs.addValue(x.Tc);
    this.dattribs.addValue(x.Fc);
    this.dattribs.addValue(x.FL);
    this.dattribs.addValue(x.FH);
    this.dattribs.addValue(x.volume);
    this.dattribs.addValue(x.a);
    this.dattribs.addValue(x.b);

    return n;
}


private int SMMStateSaver_SMM_Names(final String prefix,final String s)
{
    int n=0;
    n+=8;
    this.dattribs_names.addValue(prefix+".Ic_",s,null);
    this.dattribs_names.addValue(prefix+".Pc_",s,null);
    this.dattribs_names.addValue(prefix+".Tc_",s,null);
    this.dattribs_names.addValue(prefix+".Fc_",s,null);
    this.dattribs_names.addValue(prefix+".Uc_",s,null);
    this.dattribs_names.addValue(prefix+".Vc_",s,null);
    this.dattribs_names.addValue(prefix+".PcMin_",s,null);
    this.dattribs_names.addValue(prefix+".PcMax_",s,null);

    return n;
}
private int SMMStateSaver_SMM_Values(final SMMStateSaver.SMM x)
{
    int n=0;
    n+=8;
    this.dattribs.addValue(x.Ic);
    this.dattribs.addValue(x.Pc);
    this.dattribs.addValue(x.Tc);
    this.dattribs.addValue(x.Fc);
    this.dattribs.addValue(x.Uc);
    this.dattribs.addValue(x.Vc);
    this.dattribs.addValue(x.PcMin);
    this.dattribs.addValue(x.PcMax);

    return n;
}


private int EVXSpectrum_Names(final String prefix,final String s)
{
    int n=0;
    n+=2;
    this.dattribs_names.addValue(prefix+".sL_",s,null);
    this.dattribs_names.addValue(prefix+".sH_",s,null);
    for(int j=0;j<8;j++)
    {
	n+=1;
	this.dattribs_names.addValue(prefix+".E_"+j+"_",s,null);
    }
    return n;
}
private int EVXSpectrum_Values(final EVXSpectrum x)
{
    int n=0;
    n+=2;
    this.dattribs.addValue(x.sL);
    this.dattribs.addValue(x.sH);
    for(int j=0;j<8;j++)
    {
	n+=1;
	this.dattribs.addValue(j<x.neq?x.getE(j):Double.NaN);
    }
    return n;
}


private int Skewness_Names(final String prefix,final String s)
{
    int n=0;
    n+=7;
    this.dattribs_names.addValue(prefix+".xa_",s,null);
    this.dattribs_names.addValue(prefix+".xm_",s,null);
    this.dattribs_names.addValue(prefix+".sk_",s,null);
    this.dattribs_names.addValue(prefix+".x1_",s,null);
    this.dattribs_names.addValue(prefix+".w1_",s,null);
    this.dattribs_names.addValue(prefix+".x2_",s,null);
    this.dattribs_names.addValue(prefix+".w2_",s,null);
    return n;
}
private int Skewness_Values(final com.polytechnik.utils.Skewness x)
{
    int n=0;
    n+=7;
    this.dattribs.addValue(x.xa);
    this.dattribs.addValue(x.xm);
    this.dattribs.addValue(x.sk);
    this.dattribs.addValue(x.x1);
    this.dattribs.addValue(x.w1);
    this.dattribs.addValue(x.x2);
    this.dattribs.addValue(x.w2);
    return n;
}

private int SepSkewness_Names(final String prefix,final String s)
{
    int n=0;
    n+=3;
    this.dattribs_names.addValue(prefix+".f_",s,null);
    this.dattribs_names.addValue(prefix+".w_plus_",s,null);
    this.dattribs_names.addValue(prefix+".w_minus_",s,null);
    return n;
}
private int SepSkewness_Values(final SepSkewness x)
{
    int n=0;
    n+=3;
    this.dattribs.addValue(x.f);
    this.dattribs.addValue(x.w_plus);
    this.dattribs.addValue(x.w_minus);
    return n;
}


private int DPDVm_Names(final String prefix,final String s)
{
    int n=0;
    n+=4;
    this.dattribs_names.addValue(prefix+".lambda_",s,null);
    this.dattribs_names.addValue(prefix+".lambdamin_",s,null);
    this.dattribs_names.addValue(prefix+".dp_",s,null);
    this.nattribs_names.addValue(prefix+".flag_max_",s,null);

    return n;
}
private int DPDVm_Values(final BiQVM.DPDVm x)
{
    int n=0;
    n+=4;
    this.dattribs.addValue(x.lambda);
    this.dattribs.addValue(x.lambdamin);
    this.dattribs.addValue(x.dp);
    this.nattribs.addValue(x.flag_max?1:0);

    return n;
}

private int Mu2_Names(final String prefix,final String s)
{
    int n=0;
    n+=8;
    this.dattribs_names.addValue(prefix+".IL_",s,null);
    this.dattribs_names.addValue(prefix+".IH_",s,null);
    this.dattribs_names.addValue(prefix+".root_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".root_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".pa_",s,null);
    this.dattribs_names.addValue(prefix+".pm_",s,null);
    this.dattribs_names.addValue(prefix+".pc_weight_",s,null);
    this.dattribs_names.addValue(prefix+".qsk_",s,null);

    n+=ProbabilityCorrelation_Names(prefix+".pc",s);

    return n;
}
private int Mu2_Values(final Mu2 x)
{
    int n=0;
    n+=8;
    this.dattribs.addValue(x.IL);
    this.dattribs.addValue(x.IH);
    this.dattribs.addValue(x.root_psiL);
    this.dattribs.addValue(x.root_psiH);
    this.dattribs.addValue(x.pa);
    this.dattribs.addValue(x.pm);
    this.dattribs.addValue(x.pc_weight);
    this.dattribs.addValue(x.qsk);

    n+=ProbabilityCorrelation_Values(x.pc);

    return n;
}

private int InDIState_Names(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=Mu2_Names(prefix+".mu2dIL",s);
    n+=Mu2_Names(prefix+".mu2dIH",s);
    n+=2;
    this.dattribs_names.addValue(prefix+".dIL_",s,null);
    this.dattribs_names.addValue(prefix+".dIH_",s,null);
    return n;
}
private int InDIState_Values(final InDIState x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=Mu2_Values(x.mu2dIL);
    n+=Mu2_Values(x.mu2dIH);
    n+=2;
    this.dattribs.addValue(x.dIL);
    this.dattribs.addValue(x.dIH);
    return n;
}
private int InIStateV_Names(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".V",s);
    n+=Skewness_Names(prefix+".sk_V",s);
    n+=Skewness_Names(prefix+".sk_VV",s);
    n+=Skewness_Names(prefix+".sk_VIH",s);
    n+=37;
    this.dattribs_names.addValue(prefix+".dV_shares_",s,null);
    this.dattribs_names.addValue(prefix+".Dpw_dir_",s,null);
    this.dattribs_names.addValue(prefix+".Dp_dir_",s,null);
    this.dattribs_names.addValue(prefix+".dpw_dir_",s,null);
    this.dattribs_names.addValue(prefix+".dp_dir_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IX_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".ind_ev_",s,null);
    this.dattribs_names.addValue(prefix+".Dpw_VV_",s,null);
    this.dattribs_names.addValue(prefix+".Dp_VV_",s,null);
    this.dattribs_names.addValue(prefix+".dpw_V1_",s,null);
    this.dattribs_names.addValue(prefix+".dp_V1_",s,null);
    this.dattribs_names.addValue(prefix+".pi_V1_",s,null);
    this.dattribs_names.addValue(prefix+".V_V1_",s,null);
    this.dattribs_names.addValue(prefix+".dpw_V2_",s,null);
    this.dattribs_names.addValue(prefix+".dp_V2_",s,null);
    this.dattribs_names.addValue(prefix+".pi_V2_",s,null);
    this.dattribs_names.addValue(prefix+".V_V2_",s,null);
    this.dattribs_names.addValue(prefix+".Vs_",s,null);
    this.dattribs_names.addValue(prefix+".dVs_",s,null);
    this.dattribs_names.addValue(prefix+".V_IH_",s,null);
    this.dattribs_names.addValue(prefix+".V_VIH_",s,null);
    this.dattribs_names.addValue(prefix+".p_VIH_",s,null);
    this.dattribs_names.addValue(prefix+".Dp_VIH_",s,null);
    this.dattribs_names.addValue(prefix+".Dpw_VIH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".picorr_VIH_",s,null);
    this.dattribs_names.addValue(prefix+".pVcorr_VIH_",s,null);
    this.dattribs_names.addValue(prefix+".degeneracyVdiag_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHVdiag_",s,null);
    this.dattribs_names.addValue(prefix+".pp_IHVdiag_",s,null);
    this.dattribs_names.addValue(prefix+".dirpicorr_",s,null);
    this.dattribs_names.addValue(prefix+".cvLcorr_",s,null);
    this.dattribs_names.addValue(prefix+".cvIcorr_",s,null);
    this.dattribs_names.addValue(prefix+".pi_Imaxv_",s,null);
    this.dattribs_names.addValue(prefix+".pV_Imaxv_",s,null);
    return n;
}
private int InIStateV_Values(final InIStateV x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.V);
    n+=Skewness_Values(x.sk_V);
    n+=Skewness_Values(x.sk_VV);
    n+=Skewness_Values(x.sk_VIH);
    n+=37;
    this.dattribs.addValue(x.dV_shares);
    this.dattribs.addValue(x.Dpw_dir);
    this.dattribs.addValue(x.Dp_dir);
    this.dattribs.addValue(x.dpw_dir);
    this.dattribs.addValue(x.dp_dir);
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.pt_IX);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.ind_ev);
    this.dattribs.addValue(x.Dpw_VV);
    this.dattribs.addValue(x.Dp_VV);
    this.dattribs.addValue(x.dpw_V1);
    this.dattribs.addValue(x.dp_V1);
    this.dattribs.addValue(x.pi_V1);
    this.dattribs.addValue(x.V_V1);
    this.dattribs.addValue(x.dpw_V2);
    this.dattribs.addValue(x.dp_V2);
    this.dattribs.addValue(x.pi_V2);
    this.dattribs.addValue(x.V_V2);
    this.dattribs.addValue(x.Vs);
    this.dattribs.addValue(x.dVs);
    this.dattribs.addValue(x.V_IH);
    this.dattribs.addValue(x.V_VIH);
    this.dattribs.addValue(x.p_VIH);
    this.dattribs.addValue(x.Dp_VIH);
    this.dattribs.addValue(x.Dpw_VIH);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.picorr_VIH);
    this.dattribs.addValue(x.pVcorr_VIH);
    this.dattribs.addValue(x.degeneracyVdiag);
    this.dattribs.addValue(x.p_IHVdiag);
    this.dattribs.addValue(x.pp_IHVdiag);
    this.dattribs.addValue(x.dirpicorr);
    this.dattribs.addValue(x.cvLcorr);
    this.dattribs.addValue(x.cvIcorr);
    this.dattribs.addValue(x.pi_Imaxv);
    this.dattribs.addValue(x.pV_Imaxv);
    return n;
}

private int MXData_Names(final String prefix,final String s)
{
    int n=0;
    n+=4;
    this.dattribs_names.addValue(prefix+".lam_",s,null);
    this.dattribs_names.addValue(prefix+".p0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_p0_",s,null);
    this.dattribs_names.addValue(prefix+".lamE_",s,null);

    return n;
}
private int MXData_Values(final BQVM.MXData x)
{
    int n=0;
    n+=4;
    this.dattribs.addValue(x.lam);
    this.dattribs.addValue(x.p0);
    this.dattribs.addValue(x.sigma2_p0);
    this.dattribs.addValue(x.lamE);

    return n;
}

private int M2Data_Names(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".evx",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".lam0_",s,null);
    this.dattribs_names.addValue(prefix+".lam1_",s,null);
    this.dattribs_names.addValue(prefix+".dp_",s,null);
    this.dattribs_names.addValue(prefix+".D_",s,null);

    return n;
}
private int M2Data_Values(final BQVM.M2Data x)
{
    int n=0;
    n+=EVXData_Values(x.evx);
    n+=4;
    this.dattribs.addValue(x.lam0);
    this.dattribs.addValue(x.lam1);
    this.dattribs.addValue(x.dp);
    this.dattribs.addValue(x.D);

    return n;
}


private int TRIFX_Names(final String prefix,final String s)
{
    int n=0;
    n+=TwoRootsDataNames(prefix+".r",s);

    n+=6;
    this.dattribs_names.addValue(prefix+".It_0_",s,null);
    this.dattribs_names.addValue(prefix+".It_1_",s,null);
    this.dattribs_names.addValue(prefix+".It_2_",s,null);
    this.dattribs_names.addValue(prefix+".If_0_",s,null);
    this.dattribs_names.addValue(prefix+".If_1_",s,null);
    this.dattribs_names.addValue(prefix+".If_2_",s,null);


    n+=4;
    this.dattribs_names.addValue(prefix+".s1.x_min_",s,null);
    this.dattribs_names.addValue(prefix+".s1.f_min_",s,null);
    this.dattribs_names.addValue(prefix+".s1.x_max_",s,null);
    this.dattribs_names.addValue(prefix+".s1.f_max_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".s2.x_min_",s,null);
    this.dattribs_names.addValue(prefix+".s2.f_min_",s,null);
    this.dattribs_names.addValue(prefix+".s2.x_max_",s,null);
    this.dattribs_names.addValue(prefix+".s2.f_max_",s,null);

    return n;
}
private int TRIFX_Values(final TwoRootsIFX x)
{
    int n=0;
    n+=TwoRootsDataValues(x.r);

    n+=6;
    this.dattribs.addValue(x.It_0);
    this.dattribs.addValue(x.It_1);
    this.dattribs.addValue(x.It_2);
    this.dattribs.addValue(x.If_0);
    this.dattribs.addValue(x.If_1);
    this.dattribs.addValue(x.If_2);

    n+=4;
    this.dattribs.addValue(x.s1.x_min);
    this.dattribs.addValue(x.s1.f_min);
    this.dattribs.addValue(x.s1.x_max);
    this.dattribs.addValue(x.s1.f_max);

    n+=4;
    this.dattribs.addValue(x.s2.x_min);
    this.dattribs.addValue(x.s2.f_min);
    this.dattribs.addValue(x.s2.x_max);
    this.dattribs.addValue(x.s2.f_max);

    return n;
}



private int TRIFD_Names(final String prefix,final String s)
{
    int n=0;
    n+=TwoRootsDataNames(prefix+".r",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".xD_1_",s,null);
    this.dattribs_names.addValue(prefix+".fD_1_",s,null);
    this.dattribs_names.addValue(prefix+".xD_2_",s,null);
    this.dattribs_names.addValue(prefix+".fD_2_",s,null);

    return n;
}
private int TRIFD_Values(final TwoRootsIFD x)
{
    int n=0;
    n+=TwoRootsDataValues(x.r);

    n+=4;
    this.dattribs.addValue(x.xD_1);
    this.dattribs.addValue(x.fD_1);
    this.dattribs.addValue(x.xD_2);
    this.dattribs.addValue(x.fD_2);

    return n;
}






private int printBQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=TRIFX_Names(prefix+".rm0_kkdp_kkt",s);
    n+=TRIFX_Names(prefix+".rm0_kkvdp_kkt",s);
    n+=TRIFX_Names(prefix+".rm0_kkdp_kkiv",s);
    n+=TRIFX_Names(prefix+".rm0_kkdp_kka",s);
    n+=1;
    this.dattribs_names.addValue(prefix+".c_dp_i_x0_",s,null);
    n+=TRIFD_Names(prefix+".rms_kkvdp_kkt",s);
    n+=TRIFD_Names(prefix+".rms_kkdp_kka",s);


    n+=24;
    this.dattribs_names.addValue(prefix+".K00t_",s,null);
    this.dattribs_names.addValue(prefix+".I0_",s,null);
    this.dattribs_names.addValue(prefix+".I0sam_",s,null);
    this.dattribs_names.addValue(prefix+".A0_",s,null);
    this.dattribs_names.addValue(prefix+".SA0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2T_SA0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2T_A0_",s,null);
    this.dattribs_names.addValue(prefix+".K11t_",s,null);
    this.dattribs_names.addValue(prefix+".A1_",s,null);
    this.dattribs_names.addValue(prefix+".SA1_",s,null);
    this.dattribs_names.addValue(prefix+".dS_",s,null);
    this.dattribs_names.addValue(prefix+".dSe_",s,null);
    this.dattribs_names.addValue(prefix+".dSf_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2T_dS_",s,null);
    this.dattribs_names.addValue(prefix+".XM_",s,null);
    this.dattribs_names.addValue(prefix+".AM_",s,null);
    this.dattribs_names.addValue(prefix+".SAM_",s,null);
    this.dattribs_names.addValue(prefix+".AE_",s,null);
    this.dattribs_names.addValue(prefix+".wIE_",s,null);
    this.dattribs_names.addValue(prefix+".SAE_",s,null);
    this.dattribs_names.addValue(prefix+".SBE_",s,null);
    this.dattribs_names.addValue(prefix+".AR_",s,null);
    this.dattribs_names.addValue(prefix+".SAR_",s,null);
    this.dattribs_names.addValue(prefix+".SBR_",s,null);
    n+=6;
    this.dattribs_names.addValue(prefix+".AEl_",s,null);
    this.dattribs_names.addValue(prefix+".SAEl_",s,null);
    this.dattribs_names.addValue(prefix+".SBEl_",s,null);
    this.dattribs_names.addValue(prefix+".ARl_",s,null);
    this.dattribs_names.addValue(prefix+".SARl_",s,null);
    this.dattribs_names.addValue(prefix+".SBRl_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".SArvsL_",s,null);
    this.dattribs_names.addValue(prefix+".wL_",s,null);
    this.dattribs_names.addValue(prefix+".SArvsH_",s,null);
    this.dattribs_names.addValue(prefix+".wH_",s,null);


    n+=4;
    this.dattribs_names.addValue(prefix+".sPL_",s,null);
    this.dattribs_names.addValue(prefix+".wPL_",s,null);
    this.dattribs_names.addValue(prefix+".sPH_",s,null);
    this.dattribs_names.addValue(prefix+".wPH_",s,null);

    n+=2;
    this.dattribs_names.addValue(prefix+".SARrvsL_",s,null);
    this.dattribs_names.addValue(prefix+".SARrvsH_",s,null);

    n+=8;
    this.dattribs_names.addValue(prefix+".ptE_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ptE_",s,null);
    this.dattribs_names.addValue(prefix+".pt0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pt0_",s,null);
    this.dattribs_names.addValue(prefix+".pvE_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pvE_",s,null);
    this.dattribs_names.addValue(prefix+".pv0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pv0_",s,null);
    n+=3;
    this.dattribs_names.addValue(prefix+".pf0_",s,null);
    this.dattribs_names.addValue(prefix+".pfL_",s,null);
    this.dattribs_names.addValue(prefix+".pfLt_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".pH0_",s,null);
    this.dattribs_names.addValue(prefix+".sH0_",s,null);
    this.dattribs_names.addValue(prefix+".ppv0_",s,null);
    this.dattribs_names.addValue(prefix+".ppt0_",s,null);
    this.dattribs_names.addValue(prefix+".ppvH_",s,null);
    this.dattribs_names.addValue(prefix+".pptH_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".peamL_",s,null);
    this.dattribs_names.addValue(prefix+".peamH_",s,null);
    this.dattribs_names.addValue(prefix+".pev2oamL_",s,null);
    this.dattribs_names.addValue(prefix+".pev2oamH_",s,null);
    this.dattribs_names.addValue(prefix+".pev2oDamL_",s,null);
    this.dattribs_names.addValue(prefix+".pev2oDamH_",s,null);

    n+=2;
    this.dattribs_names.addValue(prefix+".DPt0_f0_",s,null);
    this.dattribs_names.addValue(prefix+".D2Pt0_f0_",s,null);

    n+=EVXData_Names(prefix+".RVS",s);
    n+=EVXData_Names(prefix+".RVSsam",s);
    n+=EVXData_Names(prefix+".RTS",s);
    n+=EVXData_Names(prefix+".DPS",s);
    n+=EVXData_Names(prefix+".RVSV",s);
    n+=EVXData_Names(prefix+".RTST",s);
    n+=EVXData_Names(prefix+".RTSTsam",s);
    n+=EVXData_Names(prefix+".IT",s);
    n+=EVXData_Names(prefix+".ITinv",s);
    n+=EVXData_Names(prefix+".ITshifted",s);
    n+=EVXData_Names(prefix+".IT0R",s);
    n+=EVXData_Names(prefix+".ITsam",s);
    n+=EVXData_Names(prefix+".DS",s);
    n+=EVXData_Names(prefix+".DSsam",s);
    n+=EVXData_Names(prefix+".DSsamV",s);
    n+=EVXData_Names(prefix+".DSva",s);
    n+=EVXData_Names(prefix+".DSta",s);
    n+=EVXData_Names(prefix+".DStaI",s);
    n+=EVXData_Names(prefix+".DStaI0",s);
    n+=EVXData_Names(prefix+".DSva_sam",s);
    n+=EVXData_Names(prefix+".DSta_sam",s);

    n+=EVXData_Names(prefix+".DSI",s);
    n+=EVXData_Names(prefix+".DSImI",s);
    n+=EVXData_Names(prefix+".DSIpI",s);

    n+=EVXData_Names(prefix+".DPt0",s);
    n+=EVXData_Names(prefix+".D2Pt0",s);

    n+=EVXData_Names(prefix+".VSV",s);
    n+=EVXData_Names(prefix+".TSV",s);
    n+=EVXData_Names(prefix+".TS",s);
    n+=EVXData_Names(prefix+".TSsam",s);
    n+=EVXData_Names(prefix+".VS",s);
    n+=EVXData_Names(prefix+".VSsam",s);


    n+=EVXData_Names(prefix+".DRVSta",s);
    n+=EVXData_Names(prefix+".DVSta",s);


    n+=EVXData_Names(prefix+".DSRmu_sam",s);
    n+=EVXData_Names(prefix+".DSmu_sam",s);
    n+=EVXData_Names(prefix+".DSRmu",s);
    n+=EVXData_Names(prefix+".DSmu",s);
    n+=EVXData_Names(prefix+".P_sam",s);

    n+=EVXData_Names(prefix+".PVOL",s);
    n+=EVXData_Names(prefix+".PX2VOL",s);
    n+=EVXData_Names(prefix+".PXVOL",s);
    n+=EVXData_Names(prefix+".EIsell",s);
    n+=EVXData_Names(prefix+".EIbuy",s);
    n+=EVXData_Names(prefix+".EAM",s);
    n+=EVXData_Names(prefix+".EV2oAM",s);
    n+=EVXData_Names(prefix+".EV2oDAM",s);

    n+=M2Data_Names(prefix+".mdp",s);
    n+=M2Data_Names(prefix+".mS",s);

    n+=MXData_Names(prefix+".mpv",s);
    n+=MXData_Names(prefix+".mDpv",s);

    return n;

}
private <T> int printBQVMValues(final BQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=TRIFX_Values(x.rm0_kkdp_kkt);
    n+=TRIFX_Values(x.rm0_kkvdp_kkt);
    n+=TRIFX_Values(x.rm0_kkdp_kkiv);
    n+=TRIFX_Values(x.rm0_kkdp_kka);
    n+=1;
    this.dattribs.addValue(x.c_dp_i_x0);

    n+=TRIFD_Values(x.rms_kkvdp_kkt);
    n+=TRIFD_Values(x.rms_kkdp_kka);

    n+=24;
    this.dattribs.addValue(x.K00t);
    this.dattribs.addValue(x.I0);
    this.dattribs.addValue(x.I0sam);
    this.dattribs.addValue(x.A0);
    this.dattribs.addValue(x.SA0);
    this.dattribs.addValue(x.sigma2T_SA0);
    this.dattribs.addValue(x.sigma2T_A0);
    this.dattribs.addValue(x.K11t);
    this.dattribs.addValue(x.A1);
    this.dattribs.addValue(x.SA1);
    this.dattribs.addValue(x.dS);
    this.dattribs.addValue(x.dSe);
    this.dattribs.addValue(x.dSf);
    this.dattribs.addValue(x.sigma2T_dS);
    this.dattribs.addValue(x.XM);
    this.dattribs.addValue(x.AM);
    this.dattribs.addValue(x.SAM);
    this.dattribs.addValue(x.AE);
    this.dattribs.addValue(x.wIE);
    this.dattribs.addValue(x.SAE);
    this.dattribs.addValue(x.SBE);
    this.dattribs.addValue(x.AR);
    this.dattribs.addValue(x.SAR);
    this.dattribs.addValue(x.SBR);
    n+=6;
    this.dattribs.addValue(x.AEl);
    this.dattribs.addValue(x.SAEl);
    this.dattribs.addValue(x.SBEl);
    this.dattribs.addValue(x.ARl);
    this.dattribs.addValue(x.SARl);
    this.dattribs.addValue(x.SBRl);

    n+=4;
    this.dattribs.addValue(x.SArvsL);
    this.dattribs.addValue(x.wL);
    this.dattribs.addValue(x.SArvsH);
    this.dattribs.addValue(x.wH);

    n+=4;
    this.dattribs.addValue(x.sPL);
    this.dattribs.addValue(x.wPL);
    this.dattribs.addValue(x.sPH);
    this.dattribs.addValue(x.wPH);

    n+=2;
    this.dattribs.addValue(x.SARrvsL);
    this.dattribs.addValue(x.SARrvsH);

    n+=8;
    this.dattribs.addValue(x.ptE);
    this.dattribs.addValue(x.sigma2_ptE);
    this.dattribs.addValue(x.pt0);
    this.dattribs.addValue(x.sigma2_pt0);
    this.dattribs.addValue(x.pvE);
    this.dattribs.addValue(x.sigma2_pvE);
    this.dattribs.addValue(x.pv0);
    this.dattribs.addValue(x.sigma2_pv0);
    n+=3;
    this.dattribs.addValue(x.pf0);
    this.dattribs.addValue(x.pfL);
    this.dattribs.addValue(x.pfLt);

    n+=6;
    this.dattribs.addValue(x.pH0);
    this.dattribs.addValue(x.sH0);
    this.dattribs.addValue(x.ppv0);
    this.dattribs.addValue(x.ppt0);
    this.dattribs.addValue(x.ppvH);
    this.dattribs.addValue(x.pptH);

    n+=6;
    this.dattribs.addValue(x.peamL);
    this.dattribs.addValue(x.peamH);
    this.dattribs.addValue(x.pev2oamL);
    this.dattribs.addValue(x.pev2oamH);
    this.dattribs.addValue(x.pev2oDamL);
    this.dattribs.addValue(x.pev2oDamH);


    n+=2;
    this.dattribs.addValue(x.DPt0_f0);
    this.dattribs.addValue(x.D2Pt0_f0);

    n+=EVXData_Values(x.RVS);
    n+=EVXData_Values(x.RVSsam);
    n+=EVXData_Values(x.RTS);
    n+=EVXData_Values(x.DPS);
    n+=EVXData_Values(x.RVSV);
    n+=EVXData_Values(x.RTST);
    n+=EVXData_Values(x.RTSTsam);
    n+=EVXData_Values(x.IT);
    n+=EVXData_Values(x.ITinv);
    n+=EVXData_Values(x.ITshifted);
    n+=EVXData_Values(x.IT0R);
    n+=EVXData_Values(x.ITsam);
    n+=EVXData_Values(x.DS);
    n+=EVXData_Values(x.DSsam);
    n+=EVXData_Values(x.DSsamV);
    n+=EVXData_Values(x.DSva);
    n+=EVXData_Values(x.DSta);
    n+=EVXData_Values(x.DStaI);
    n+=EVXData_Values(x.DStaI0);
    n+=EVXData_Values(x.DSva_sam);
    n+=EVXData_Values(x.DSta_sam);

    n+=EVXData_Values(x.DSI);
    n+=EVXData_Values(x.DSImI);
    n+=EVXData_Values(x.DSIpI);

    n+=EVXData_Values(x.DPt0);
    n+=EVXData_Values(x.D2Pt0);

    n+=EVXData_Values(x.VSV);
    n+=EVXData_Values(x.TSV);
    n+=EVXData_Values(x.TS);
    n+=EVXData_Values(x.TSsam);
    n+=EVXData_Values(x.VS);
    n+=EVXData_Values(x.VSsam);

    n+=EVXData_Values(x.DRVSta);
    n+=EVXData_Values(x.DVSta);

    n+=EVXData_Values(x.DSRmu_sam);
    n+=EVXData_Values(x.DSmu_sam);
    n+=EVXData_Values(x.DSRmu);
    n+=EVXData_Values(x.DSmu);
    n+=EVXData_Values(x.P_sam);

    n+=EVXData_Values(x.PVOL);
    n+=EVXData_Values(x.PX2VOL);
    n+=EVXData_Values(x.PXVOL);
    n+=EVXData_Values(x.EIsell);
    n+=EVXData_Values(x.EIbuy);
    n+=EVXData_Values(x.EAM);
    n+=EVXData_Values(x.EV2oAM);
    n+=EVXData_Values(x.EV2oDAM);

    n+=M2Data_Values(x.mdp);
    n+=M2Data_Values(x.mS);

    n+=MXData_Values(x.mpv);
    n+=MXData_Values(x.mDpv);

    return n;

}




private int printBiQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".TI",s);
    n+=EVXData_Names(prefix+".pI",s);
    n+=EVXData_Names(prefix+".Vt",s);
    n+=EVXData_Names(prefix+".Pt",s);
    n+=EVXData_Names(prefix+".pt",s);
    n+=EVXData_Names(prefix+".P2t",s);
    n+=EVXData_Names(prefix+".P2i",s);
    n+=EVXData_Names(prefix+".p2t",s);
    n+=EVXData_Names(prefix+".PV",s);
    n+=EVXData_Names(prefix+".pV",s);
    n+=EVXData_Names(prefix+".P2V",s);
    n+=EVXData_Names(prefix+".p2V",s);
    n+=EVXData_Names(prefix+".Ct",s);
    n+=EVXData_Names(prefix+".C2t",s);
    n+=EVXData_Names(prefix+".dp2I",s);

    n+=EVXData_Names(prefix+".Cx",s);
    n+=EVXData_Names(prefix+".C2x",s);

    n+=EVXData_Names(prefix+".dAt",s);
    n+=EVXData_Names(prefix+".dAv",s);
    n+=EVXData_Names(prefix+".dRTSa",s);
    n+=EVXData_Names(prefix+".dRTSt",s);
    n+=EVXData_Names(prefix+".dRTSv",s);
    n+=EVXData_Names(prefix+".dPa",s);
    n+=EVXData_Names(prefix+".dPt",s);
    n+=EVXData_Names(prefix+".dPv",s);

    n+=EVXData_Names(prefix+".Rx",s);
    n+=EVXData_Names(prefix+".Px",s);

    n+=EVXData_Names(prefix+".DdPt.",s);
    n+=EVXData_Names(prefix+".DIt.",s);
    n+=EVXData_Names(prefix+".dxt.",s);
    n+=EVXData_Names(prefix+".Ddxt.",s);


    n+=DPDVm_Names(prefix+".d",s);
    n+=DPDVm_Names(prefix+".D",s);
    n+=DPDVm_Names(prefix+".dm",s);
    n+=DPDVm_Names(prefix+".Dm",s);
    n+=DPDVm_Names(prefix+".C",s);
    n+=DPDVm_Names(prefix+".Cm",s);

    n+=2;
    this.dattribs_names.addValue(prefix+".dst_i_",s,null);
    this.dattribs_names.addValue(prefix+".dsi_t_",s,null);

    n+=EVXData_Names(prefix+".J.IT",s);
    n+=EVXData_Names(prefix+".J.DIT",s);
    n+=11;
    this.dattribs_names.addValue(prefix+".J.I0_",s,null);
    this.dattribs_names.addValue(prefix+".J.o_o_",s,null);
    this.dattribs_names.addValue(prefix+".J.dp_dp0_",s,null);
    this.dattribs_names.addValue(prefix+".J.dp_di0_",s,null);
    this.dattribs_names.addValue(prefix+".J.di_di0_",s,null);
    this.dattribs_names.addValue(prefix+".J.P0_",s,null);
    this.dattribs_names.addValue(prefix+".J.osp0_",s,null);
    this.dattribs_names.addValue(prefix+".J.tdp_dp_",s,null);
    this.dattribs_names.addValue(prefix+".J.tdp_di_",s,null);
    this.dattribs_names.addValue(prefix+".J.tdi_di_",s,null);
    this.dattribs_names.addValue(prefix+".J.tP_",s,null);

    n+=EVXData_Names(prefix+".dVSt",s);
    n+=7;
    this.dattribs_names.addValue(prefix+".psiH_vs_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".psiL_vs_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".psi0_vs_psi0_",s,null);
    this.dattribs_names.addValue(prefix+".psi0_vs_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".psi0_vs_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".psi0_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".psi0_psiL_",s,null);

    n+=3;
    this.dattribs_names.addValue(prefix+".psiH_rvs_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".psiL_rvs_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".psi0_rvs_psi0_",s,null);

    n+=TwoRootsDataNames(prefix+".rv",s);
    n+=24;
    this.dattribs_names.addValue(prefix+".rv_vs_x0_",s,null);
    this.dattribs_names.addValue(prefix+".rv_vs_x1_",s,null);
    this.dattribs_names.addValue(prefix+".rv_vs_x2_",s,null);
    this.dattribs_names.addValue(prefix+".rv_Pt_x0_",s,null);
    this.dattribs_names.addValue(prefix+".rv_Pt_x1_",s,null);
    this.dattribs_names.addValue(prefix+".rv_Pt_x2_",s,null);
    this.dattribs_names.addValue(prefix+".rv_pt_x0_",s,null);
    this.dattribs_names.addValue(prefix+".rv_pt_x1_",s,null);
    this.dattribs_names.addValue(prefix+".rv_pt_x2_",s,null);
    this.dattribs_names.addValue(prefix+".rv_Dpv_x0_",s,null);
    this.dattribs_names.addValue(prefix+".rv_Dpv_x1_",s,null);
    this.dattribs_names.addValue(prefix+".rv_Dpv_x2_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DDpI_x0_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DDpI_x1_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DDpI_x2_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DSI_x0_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DSI_x1_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DSI_x2_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DdpI_x0_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DdpI_x1_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DdpI_x2_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DvsI_x0_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DvsI_x1_",s,null);
    this.dattribs_names.addValue(prefix+".rv_DvsI_x2_",s,null);

    n+=12;
    this.dattribs_names.addValue(prefix+".pt0_",s,null);
    this.dattribs_names.addValue(prefix+".ptL_",s,null);
    this.dattribs_names.addValue(prefix+".ptH_",s,null);
    this.dattribs_names.addValue(prefix+".Pt0_",s,null);
    this.dattribs_names.addValue(prefix+".PtL_",s,null);
    this.dattribs_names.addValue(prefix+".PtH_",s,null);
    this.dattribs_names.addValue(prefix+".ptQ0_",s,null);
    this.dattribs_names.addValue(prefix+".ptQL_",s,null);
    this.dattribs_names.addValue(prefix+".ptQH_",s,null);
    this.dattribs_names.addValue(prefix+".PtQ0_",s,null);
    this.dattribs_names.addValue(prefix+".PtQL_",s,null);
    this.dattribs_names.addValue(prefix+".PtQH_",s,null);
    n+=6;
    this.dattribs_names.addValue(prefix+".DptQ0_",s,null);
    this.dattribs_names.addValue(prefix+".DptQL_",s,null);
    this.dattribs_names.addValue(prefix+".DptQH_",s,null);
    this.dattribs_names.addValue(prefix+".DPtQ0_",s,null);
    this.dattribs_names.addValue(prefix+".DPtQL_",s,null);
    this.dattribs_names.addValue(prefix+".DPtQH_",s,null);
    n+=3;
    this.dattribs_names.addValue(prefix+".dp_i_QL_",s,null);
    this.dattribs_names.addValue(prefix+".dp_dp_QL_",s,null);
    this.dattribs_names.addValue(prefix+".i_i_QL_",s,null);

    n+=3;
    this.dattribs_names.addValue(prefix+".vsQL_",s,null);
    this.dattribs_names.addValue(prefix+".rvsQL_",s,null);
    this.dattribs_names.addValue(prefix+".arvsQL_",s,null);

    n+=3;
    this.dattribs_names.addValue(prefix+".dpP2QL_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dpP2QL_",s,null);
    this.dattribs_names.addValue(prefix+".adpP2QL_",s,null);

    n+=2;
    this.dattribs_names.addValue(prefix+".pvL_dp2I_",s,null);
    this.dattribs_names.addValue(prefix+".DP_P2t_psiL_",s,null);
    n+=4;
    this.dattribs_names.addValue(prefix+".DP2_P2i_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".DP1_P2i_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".DP0_P2i_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".jm_",s,null);


    n+=8;
    this.dattribs_names.addValue(prefix+".c_Dpt_i_QL_",s,null);
    this.dattribs_names.addValue(prefix+".c_vs_i_QL_",s,null);
    this.dattribs_names.addValue(prefix+".c_Dpt_c_QL_",s,null);
    this.dattribs_names.addValue(prefix+".c_vs_c_QL_",s,null);
    this.dattribs_names.addValue(prefix+".cv_Dpt_i_QL_",s,null);
    this.dattribs_names.addValue(prefix+".cv_vs_i_QL_",s,null);
    this.dattribs_names.addValue(prefix+".cv_Dpt_c_QL_",s,null);
    this.dattribs_names.addValue(prefix+".cv_vs_c_QL_",s,null);

    n+=3;
    this.dattribs_names.addValue(prefix+".pv0_",s,null);
    this.dattribs_names.addValue(prefix+".pvL_",s,null);
    this.dattribs_names.addValue(prefix+".pvH_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".VSH_",s,null);
    this.dattribs_names.addValue(prefix+".VSL_",s,null);
    this.dattribs_names.addValue(prefix+".VS0_",s,null);
    this.dattribs_names.addValue(prefix+".RVSH_",s,null);
    this.dattribs_names.addValue(prefix+".RVSL_",s,null);
    this.dattribs_names.addValue(prefix+".RVS0_",s,null);

    n+=7;
    this.dattribs_names.addValue(prefix+".psiH_C1_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".psiH_VS_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".psiH_RVS_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".psiH_P_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".psiH_pt_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".psiH_pv_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".psiH_dI_psiH_",s,null);
    n+=18;
    this.dattribs_names.addValue(prefix+".idp_0_",s,null);
    this.dattribs_names.addValue(prefix+".idp_L_",s,null);
    this.dattribs_names.addValue(prefix+".idp_H_",s,null);
    this.dattribs_names.addValue(prefix+".ivs_0_",s,null);
    this.dattribs_names.addValue(prefix+".ivs_L_",s,null);
    this.dattribs_names.addValue(prefix+".ivs_H_",s,null);
    this.dattribs_names.addValue(prefix+".Didp_0_",s,null);
    this.dattribs_names.addValue(prefix+".Didp_L_",s,null);
    this.dattribs_names.addValue(prefix+".Didp_H_",s,null);
    this.dattribs_names.addValue(prefix+".Divs_0_",s,null);
    this.dattribs_names.addValue(prefix+".Divs_L_",s,null);
    this.dattribs_names.addValue(prefix+".Divs_H_",s,null);
    this.dattribs_names.addValue(prefix+".DdpI_0_",s,null);
    this.dattribs_names.addValue(prefix+".DdpI_L_",s,null);
    this.dattribs_names.addValue(prefix+".DdpI_H_",s,null);
    this.dattribs_names.addValue(prefix+".DvsI_0_",s,null);
    this.dattribs_names.addValue(prefix+".DvsI_L_",s,null);
    this.dattribs_names.addValue(prefix+".DvsI_H_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".DpsiH_dp_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".DpsiL_dp_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".Dpsi0_dp_psi0_",s,null);
    this.dattribs_names.addValue(prefix+".DpsiH_vs_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".DpsiL_vs_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".Dpsi0_vs_psi0_",s,null);

    return n;

}
private <T> int printBiQVMValues(final BiQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.TI);
    n+=EVXData_Values(x.pI);
    n+=EVXData_Values(x.Vt);
    n+=EVXData_Values(x.Pt);
    n+=EVXData_Values(x.pt);
    n+=EVXData_Values(x.P2t);
    n+=EVXData_Values(x.P2i);
    n+=EVXData_Values(x.p2t);
    n+=EVXData_Values(x.PV);
    n+=EVXData_Values(x.pV);
    n+=EVXData_Values(x.P2V);
    n+=EVXData_Values(x.p2V);
    n+=EVXData_Values(x.Ct);
    n+=EVXData_Values(x.C2t);
    n+=EVXData_Values(x.dp2I);

    n+=EVXData_Values(x.Cx);
    n+=EVXData_Values(x.C2x);

    n+=EVXData_Values(x.dAt);
    n+=EVXData_Values(x.dAv);
    n+=EVXData_Values(x.dRTSa);
    n+=EVXData_Values(x.dRTSt);
    n+=EVXData_Values(x.dRTSv);
    n+=EVXData_Values(x.dPa);
    n+=EVXData_Values(x.dPt);
    n+=EVXData_Values(x.dPv);

    n+=EVXData_Values(x.Rx);
    n+=EVXData_Values(x.Px);


    n+=EVXData_Values(x.DdPt);
    n+=EVXData_Values(x.DIt);
    n+=EVXData_Values(x.dxt);
    n+=EVXData_Values(x.Ddxt);

    n+=DPDVm_Values(x.d);
    n+=DPDVm_Values(x.D);
    n+=DPDVm_Values(x.dm);
    n+=DPDVm_Values(x.Dm);
    n+=DPDVm_Values(x.C);
    n+=DPDVm_Values(x.Cm);

    n+=2;
    this.dattribs.addValue(x.dst_i);
    this.dattribs.addValue(x.dsi_t);

    n+=EVXData_Values(x.J.IT);
    n+=EVXData_Values(x.J.DIT);
    n+=11;
    this.dattribs.addValue(x.J.I0);
    this.dattribs.addValue(x.J.o_o);
    this.dattribs.addValue(x.J.dp_dp0);
    this.dattribs.addValue(x.J.dp_di0);
    this.dattribs.addValue(x.J.di_di0);
    this.dattribs.addValue(x.J.P0);
    this.dattribs.addValue(x.J.osp0);
    this.dattribs.addValue(x.J.tdp_dp);
    this.dattribs.addValue(x.J.tdp_di);
    this.dattribs.addValue(x.J.tdi_di);
    this.dattribs.addValue(x.J.tP);

    n+=EVXData_Values(x.dVSt);
    n+=7;
    this.dattribs.addValue(x.psiH_vs_psiH);
    this.dattribs.addValue(x.psiL_vs_psiL);
    this.dattribs.addValue(x.psi0_vs_psi0);
    this.dattribs.addValue(x.psi0_vs_psiH);
    this.dattribs.addValue(x.psi0_vs_psiL);
    this.dattribs.addValue(x.psi0_psiH);
    this.dattribs.addValue(x.psi0_psiL);

    n+=3;
    this.dattribs.addValue(x.psiH_rvs_psiH);
    this.dattribs.addValue(x.psiL_rvs_psiL);
    this.dattribs.addValue(x.psi0_rvs_psi0);

    n+=TwoRootsDataValues(x.rv);
    n+=24;
    this.dattribs.addValue(x.rv_vs_x0);
    this.dattribs.addValue(x.rv_vs_x1);
    this.dattribs.addValue(x.rv_vs_x2);
    this.dattribs.addValue(x.rv_Pt_x0);
    this.dattribs.addValue(x.rv_Pt_x1);
    this.dattribs.addValue(x.rv_Pt_x2);
    this.dattribs.addValue(x.rv_pt_x0);
    this.dattribs.addValue(x.rv_pt_x1);
    this.dattribs.addValue(x.rv_pt_x2);
    this.dattribs.addValue(x.rv_Dpv_x0);
    this.dattribs.addValue(x.rv_Dpv_x1);
    this.dattribs.addValue(x.rv_Dpv_x2);
    this.dattribs.addValue(x.rv_DDpI_x0);
    this.dattribs.addValue(x.rv_DDpI_x1);
    this.dattribs.addValue(x.rv_DDpI_x2);
    this.dattribs.addValue(x.rv_DSI_x0);
    this.dattribs.addValue(x.rv_DSI_x1);
    this.dattribs.addValue(x.rv_DSI_x2);
    this.dattribs.addValue(x.rv_DdpI_x0);
    this.dattribs.addValue(x.rv_DdpI_x1);
    this.dattribs.addValue(x.rv_DdpI_x2);
    this.dattribs.addValue(x.rv_DvsI_x0);
    this.dattribs.addValue(x.rv_DvsI_x1);
    this.dattribs.addValue(x.rv_DvsI_x2);

    n+=12;
    this.dattribs.addValue(x.pt0);
    this.dattribs.addValue(x.ptL);
    this.dattribs.addValue(x.ptH);
    this.dattribs.addValue(x.Pt0);
    this.dattribs.addValue(x.PtL);
    this.dattribs.addValue(x.PtH);
    this.dattribs.addValue(x.ptQ0);
    this.dattribs.addValue(x.ptQL);
    this.dattribs.addValue(x.ptQH);
    this.dattribs.addValue(x.PtQ0);
    this.dattribs.addValue(x.PtQL);
    this.dattribs.addValue(x.PtQH);
    n+=6;
    this.dattribs.addValue(x.DptQ0);
    this.dattribs.addValue(x.DptQL);
    this.dattribs.addValue(x.DptQH);
    this.dattribs.addValue(x.DPtQ0);
    this.dattribs.addValue(x.DPtQL);
    this.dattribs.addValue(x.DPtQH);
    n+=3;
    this.dattribs.addValue(x.dp_i_QL);
    this.dattribs.addValue(x.dp_dp_QL);
    this.dattribs.addValue(x.i_i_QL);


    n+=3;
    this.dattribs.addValue(x.vsQL);
    this.dattribs.addValue(x.rvsQL);
    this.dattribs.addValue(x.arvsQL);

    n+=3;
    this.dattribs.addValue(x.dpP2QL);
    this.dattribs.addValue(x.sigma2_dpP2QL);
    this.dattribs.addValue(x.adpP2QL);

    n+=2;
    this.dattribs.addValue(x.pvL_dp2I);
    this.dattribs.addValue(x.DP_P2t_psiL);
    n+=4;
    this.dattribs.addValue(x.DP2_P2i_psiL);
    this.dattribs.addValue(x.DP1_P2i_psiL);
    this.dattribs.addValue(x.DP0_P2i_psiL);
    this.dattribs.addValue(x.jm);

    n+=8;
    this.dattribs.addValue(x.c_Dpt_i_QL);
    this.dattribs.addValue(x.c_vs_i_QL);
    this.dattribs.addValue(x.c_Dpt_c_QL);
    this.dattribs.addValue(x.c_vs_c_QL);
    this.dattribs.addValue(x.cv_Dpt_i_QL);
    this.dattribs.addValue(x.cv_vs_i_QL);
    this.dattribs.addValue(x.cv_Dpt_c_QL);
    this.dattribs.addValue(x.cv_vs_c_QL);

    n+=3;
    this.dattribs.addValue(x.pv0);
    this.dattribs.addValue(x.pvL);
    this.dattribs.addValue(x.pvH);

    n+=6;
    this.dattribs.addValue(x.VSH);
    this.dattribs.addValue(x.VSL);
    this.dattribs.addValue(x.VS0);
    this.dattribs.addValue(x.RVSH);
    this.dattribs.addValue(x.RVSL);
    this.dattribs.addValue(x.RVS0);


    n+=7;
    this.dattribs.addValue(x.psiH_C1_psiH);
    this.dattribs.addValue(x.psiH_VS_psiH);
    this.dattribs.addValue(x.psiH_RVS_psiH);
    this.dattribs.addValue(x.psiH_P_psiH);
    this.dattribs.addValue(x.psiH_pt_psiH);
    this.dattribs.addValue(x.psiH_pv_psiH);
    this.dattribs.addValue(x.psiH_dI_psiH);
    n+=18;
    this.dattribs.addValue(x.idp_0);
    this.dattribs.addValue(x.idp_L);
    this.dattribs.addValue(x.idp_H);
    this.dattribs.addValue(x.ivs_0);
    this.dattribs.addValue(x.ivs_L);
    this.dattribs.addValue(x.ivs_H);
    this.dattribs.addValue(x.Didp_0);
    this.dattribs.addValue(x.Didp_L);
    this.dattribs.addValue(x.Didp_H);
    this.dattribs.addValue(x.Divs_0);
    this.dattribs.addValue(x.Divs_L);
    this.dattribs.addValue(x.Divs_H);
    this.dattribs.addValue(x.DdpI_0);
    this.dattribs.addValue(x.DdpI_L);
    this.dattribs.addValue(x.DdpI_H);
    this.dattribs.addValue(x.DvsI_0);
    this.dattribs.addValue(x.DvsI_L);
    this.dattribs.addValue(x.DvsI_H);


    n+=6;
    this.dattribs.addValue(x.DpsiH_dp_psiH);
    this.dattribs.addValue(x.DpsiL_dp_psiL);
    this.dattribs.addValue(x.Dpsi0_dp_psi0);
    this.dattribs.addValue(x.DpsiH_vs_psiH);
    this.dattribs.addValue(x.DpsiL_vs_psiL);
    this.dattribs.addValue(x.Dpsi0_vs_psi0);


    return n;

}






private int printDiQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=TwoRootsDataNames(prefix+".r_mdp0",s);
    n+=TwoRootsDataNames(prefix+".r_mvdp0",s);

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".RI",s);
    n+=EVXData_Names(prefix+".RItau",s);
    n+=EVXData_Names(prefix+".dRI",s);
    n+=EVXData_Names(prefix+".Rdp",s);
    n+=EVXData_Names(prefix+".dRdp",s);
    n+=EVXData_Names(prefix+".Rvs",s);
    n+=EVXData_Names(prefix+".dRvs",s);
    n+=EVXData_Names(prefix+".dRdpv",s);
    n+=EVXData_Names(prefix+".dRvsv",s);


    n+=12;
    this.dattribs_names.addValue(prefix+".c_RI_Rdp_L_",s,null);
    this.dattribs_names.addValue(prefix+".c_RI_Rdp_H_",s,null);
    this.dattribs_names.addValue(prefix+".c_RI_Rdp_0_",s,null);
    this.dattribs_names.addValue(prefix+".c_RI_Rvs_L_",s,null);
    this.dattribs_names.addValue(prefix+".c_RI_Rvs_H_",s,null);
    this.dattribs_names.addValue(prefix+".c_RI_Rvs_0_",s,null);
    this.dattribs_names.addValue(prefix+".c_RI_dRdp_L_",s,null);
    this.dattribs_names.addValue(prefix+".c_RI_dRdp_H_",s,null);
    this.dattribs_names.addValue(prefix+".c_RI_dRdp_0_",s,null);
    this.dattribs_names.addValue(prefix+".c_RI_dRvs_L_",s,null);
    this.dattribs_names.addValue(prefix+".c_RI_dRvs_H_",s,null);
    this.dattribs_names.addValue(prefix+".c_RI_dRvs_0_",s,null);

    n+=10;
    this.dattribs_names.addValue(prefix+".RI_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".dRI_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".Rdp_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".dRdp_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".Rvs_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".dRvs_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".cx_RI_Rdp_H_",s,null);
    this.dattribs_names.addValue(prefix+".cx_RI_dRdp_H_",s,null);
    this.dattribs_names.addValue(prefix+".cx_RI_Rvs_H_",s,null);
    this.dattribs_names.addValue(prefix+".cx_RI_dRvs_H_",s,null);

    n+=3;
    this.dattribs_names.addValue(prefix+".Rdx_H_",s,null);
    this.dattribs_names.addValue(prefix+".dRdx_H_",s,null);
    this.dattribs_names.addValue(prefix+".cX_RI_dRdp_H_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".pt0_",s,null);
    this.dattribs_names.addValue(prefix+".ptL_",s,null);
    this.dattribs_names.addValue(prefix+".ptH_",s,null);
    this.dattribs_names.addValue(prefix+".pv0_",s,null);
    this.dattribs_names.addValue(prefix+".pvL_",s,null);
    this.dattribs_names.addValue(prefix+".pvH_",s,null);

    return n;

}
private <T> int printDiQVMValues(final DiQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=TwoRootsDataValues(x.r_mdp0);
    n+=TwoRootsDataValues(x.r_mvdp0);

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.RI);
    n+=EVXData_Values(x.RItau);
    n+=EVXData_Values(x.dRI);
    n+=EVXData_Values(x.Rdp);
    n+=EVXData_Values(x.dRdp);
    n+=EVXData_Values(x.Rvs);
    n+=EVXData_Values(x.dRvs);
    n+=EVXData_Values(x.dRdpv);
    n+=EVXData_Values(x.dRvsv);

    n+=12;
    this.dattribs.addValue(x.c_RI_Rdp_L);
    this.dattribs.addValue(x.c_RI_Rdp_H);
    this.dattribs.addValue(x.c_RI_Rdp_0);
    this.dattribs.addValue(x.c_RI_Rvs_L);
    this.dattribs.addValue(x.c_RI_Rvs_H);
    this.dattribs.addValue(x.c_RI_Rvs_0);
    this.dattribs.addValue(x.c_RI_dRdp_L);
    this.dattribs.addValue(x.c_RI_dRdp_H);
    this.dattribs.addValue(x.c_RI_dRdp_0);
    this.dattribs.addValue(x.c_RI_dRvs_L);
    this.dattribs.addValue(x.c_RI_dRvs_H);
    this.dattribs.addValue(x.c_RI_dRvs_0);

    n+=10;
    this.dattribs.addValue(x.RI_psiH);
    this.dattribs.addValue(x.dRI_psiH);
    this.dattribs.addValue(x.Rdp_psiH);
    this.dattribs.addValue(x.dRdp_psiH);
    this.dattribs.addValue(x.Rvs_psiH);
    this.dattribs.addValue(x.dRvs_psiH);
    this.dattribs.addValue(x.cx_RI_Rdp_H);
    this.dattribs.addValue(x.cx_RI_dRdp_H);
    this.dattribs.addValue(x.cx_RI_Rvs_H);
    this.dattribs.addValue(x.cx_RI_dRvs_H);

    n+=3;
    this.dattribs.addValue(x.Rdx_H);
    this.dattribs.addValue(x.dRdx_H);
    this.dattribs.addValue(x.cX_RI_dRdp_H);

    n+=6;
    this.dattribs.addValue(x.pt0);
    this.dattribs.addValue(x.ptL);
    this.dattribs.addValue(x.ptH);
    this.dattribs.addValue(x.pv0);
    this.dattribs.addValue(x.pvL);
    this.dattribs.addValue(x.pvH);


    return n;

}




private int printBjQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".A",s);
    n+=EVXData_Names(prefix+".plusDpt",s);
    n+=EVXData_Names(prefix+".minusDpt",s);
    n+=EVXData_Names(prefix+".Isell",s);
    n+=EVXData_Names(prefix+".Ibuy",s);
    n+=EVXData_Names(prefix+".RTi",s);;
    n+=EVXData_Names(prefix+".RTt",s);
    n+=EVXData_Names(prefix+".RplusDpt",s);
    n+=EVXData_Names(prefix+".RminusDpt",s);
    n+=EVXData_Names(prefix+".RIsell",s);
    n+=EVXData_Names(prefix+".RIbuy",s);
    n+=EVXData_Names(prefix+".Vt",s);
    n+=EVXData_Names(prefix+".Ti",s);
    n+=EVXData_Names(prefix+".Tt",s);
    n+=EVXData_Names(prefix+".Ta",s);
    n+=EVXData_Names(prefix+".RtI",s);
    n+=EVXData_Names(prefix+".RI",s);
    n+=EVXData_Names(prefix+".RItau",s);
    n+=EVXData_Names(prefix+".P2t",s);
    n+=EVXData_Names(prefix+".P2Ri",s);
    n+=4;
    this.dattribs_names.addValue(prefix+".RtI_dp_I_",s,null);
    this.dattribs_names.addValue(prefix+".RtI_vs_I_",s,null);
    this.dattribs_names.addValue(prefix+".RtI_x_",s,null);
    this.dattribs_names.addValue(prefix+".P2i_x_",s,null);

    n+=2;
    this.dattribs_names.addValue(prefix+".Rv_dp_I_",s,null);
    this.dattribs_names.addValue(prefix+".Rv_vs_I_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".x_Isell_",s,null);
    this.dattribs_names.addValue(prefix+".w_Isell_",s,null);
    this.dattribs_names.addValue(prefix+".x_Ibuy_",s,null);
    this.dattribs_names.addValue(prefix+".w_Ibuy_",s,null);

    n+=16;
    this.dattribs_names.addValue(prefix+".v_dp_I_",s,null);
    this.dattribs_names.addValue(prefix+".v_dp_dp_",s,null);
    this.dattribs_names.addValue(prefix+".v_I_I_",s,null);
    this.dattribs_names.addValue(prefix+".cv_dp_I_",s,null);
    this.dattribs_names.addValue(prefix+".ckv_dp_I_",s,null);
    this.dattribs_names.addValue(prefix+".cv_vs_I_",s,null);
    this.dattribs_names.addValue(prefix+".ckv_vs_I_",s,null);
    this.dattribs_names.addValue(prefix+".v_vs_I_",s,null);
    this.dattribs_names.addValue(prefix+".v_rvs_I_",s,null);
    this.dattribs_names.addValue(prefix+".x_I_",s,null);
    this.dattribs_names.addValue(prefix+".x_Tt_",s,null);
    this.dattribs_names.addValue(prefix+".dp_T_",s,null);
    this.dattribs_names.addValue(prefix+".dp_Tt_",s,null);
    this.dattribs_names.addValue(prefix+".ca_dp_a_",s,null);
    this.dattribs_names.addValue(prefix+".c_a_",s,null);
    this.dattribs_names.addValue(prefix+".x_Ta_",s,null);

    return n;

}
private <T> int printBjQVMValues(final BjQVM<T> x,final CalculationData c)
{
    int n=0;


    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.A);
    n+=EVXData_Values(x.plusDpt);
    n+=EVXData_Values(x.minusDpt);
    n+=EVXData_Values(x.Isell);
    n+=EVXData_Values(x.Ibuy);
    n+=EVXData_Values(x.RTi);
    n+=EVXData_Values(x.RTt);
    n+=EVXData_Values(x.RplusDpt);
    n+=EVXData_Values(x.RminusDpt);
    n+=EVXData_Values(x.RIsell);
    n+=EVXData_Values(x.RIbuy);
    n+=EVXData_Values(x.Vt);
    n+=EVXData_Values(x.Ti);
    n+=EVXData_Values(x.Tt);
    n+=EVXData_Values(x.Ta);
    n+=EVXData_Values(x.RtI);
    n+=EVXData_Values(x.RI);
    n+=EVXData_Values(x.RItau);
    n+=EVXData_Values(x.P2t);
    n+=EVXData_Values(x.P2Ri);

    n+=4;
    this.dattribs.addValue(x.RtI_dp_I);
    this.dattribs.addValue(x.RtI_vs_I);
    this.dattribs.addValue(x.RtI_x);
    this.dattribs.addValue(x.P2i_x);

    n+=2;
    this.dattribs.addValue(x.Rv_dp_I);
    this.dattribs.addValue(x.Rv_vs_I);

    n+=4;
    this.dattribs.addValue(x.x_Isell);
    this.dattribs.addValue(x.w_Isell);
    this.dattribs.addValue(x.x_Ibuy);
    this.dattribs.addValue(x.w_Ibuy);


    n+=16;
    this.dattribs.addValue(x.v_dp_I);
    this.dattribs.addValue(x.v_dp_dp);
    this.dattribs.addValue(x.v_I_I);
    this.dattribs.addValue(x.cv_dp_I);
    this.dattribs.addValue(x.ckv_dp_I);
    this.dattribs.addValue(x.cv_vs_I);
    this.dattribs.addValue(x.ckv_vs_I);
    this.dattribs.addValue(x.v_vs_I);
    this.dattribs.addValue(x.v_rvs_I);
    this.dattribs.addValue(x.x_I);
    this.dattribs.addValue(x.x_Tt);
    this.dattribs.addValue(x.dp_T);
    this.dattribs.addValue(x.dp_Tt);
    this.dattribs.addValue(x.ca_dp_a);
    this.dattribs.addValue(x.c_a);
    this.dattribs.addValue(x.x_Ta);

    return n;

}



private int printDjQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".RI",s);
    n+=EVXData_Names(prefix+".P2Ri",s);
    n+=EVXData_Names(prefix+".P2Rt",s);
    n+=EVXData_Names(prefix+".P2V",s);
    n+=EVXData_Names(prefix+".VRt",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".fWdp_",s,null);
    this.dattribs_names.addValue(prefix+".fWvs_",s,null);
    this.dattribs_names.addValue(prefix+".fWx_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".qfXx_",s,null);
    this.dattribs_names.addValue(prefix+".qfWdp_",s,null);
    this.dattribs_names.addValue(prefix+".qfWvs_",s,null);
    this.dattribs_names.addValue(prefix+".qfWx_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".rfXx_",s,null);
    this.dattribs_names.addValue(prefix+".rfWdp_",s,null);
    this.dattribs_names.addValue(prefix+".rfWvs_",s,null);
    this.dattribs_names.addValue(prefix+".rfWx_",s,null);

    n+=5;
    this.dattribs_names.addValue(prefix+".fP1_0_",s,null);
    this.dattribs_names.addValue(prefix+".fP1_x_",s,null);
    this.dattribs_names.addValue(prefix+".fP1_xn_",s,null);
    this.dattribs_names.addValue(prefix+".fP1_xFIL_",s,null);
    this.dattribs_names.addValue(prefix+".fP1_xFIH_",s,null);

    n+=10;
    this.dattribs_names.addValue(prefix+".fi0_",s,null);
    this.dattribs_names.addValue(prefix+".fdp0_",s,null);
    this.dattribs_names.addValue(prefix+".fIdp0_",s,null);
    this.dattribs_names.addValue(prefix+".fvs0_",s,null);
    this.dattribs_names.addValue(prefix+".frvs0_",s,null);
    this.dattribs_names.addValue(prefix+".fix_",s,null);
    this.dattribs_names.addValue(prefix+".fdpx_",s,null);
    this.dattribs_names.addValue(prefix+".fIdpx_",s,null);
    this.dattribs_names.addValue(prefix+".fvsx_",s,null);
    this.dattribs_names.addValue(prefix+".frvsx_",s,null);


    n+=10;
    this.dattribs_names.addValue(prefix+".fRi0_",s,null);
    this.dattribs_names.addValue(prefix+".fRdp0_",s,null);
    this.dattribs_names.addValue(prefix+".fRIdp0_",s,null);
    this.dattribs_names.addValue(prefix+".fRvs0_",s,null);
    this.dattribs_names.addValue(prefix+".fRrvs0_",s,null);
    this.dattribs_names.addValue(prefix+".fRix_",s,null);
    this.dattribs_names.addValue(prefix+".fRdpx_",s,null);
    this.dattribs_names.addValue(prefix+".fRIdpx_",s,null);
    this.dattribs_names.addValue(prefix+".fRvsx_",s,null);
    this.dattribs_names.addValue(prefix+".fRrvsx_",s,null);


    n+=8;
    this.dattribs_names.addValue(prefix+".xIL_",s,null);
    this.dattribs_names.addValue(prefix+".xIH_",s,null);
    this.dattribs_names.addValue(prefix+".xFIL_",s,null);
    this.dattribs_names.addValue(prefix+".xFILn_",s,null);
    this.dattribs_names.addValue(prefix+".xFIH_",s,null);
    this.dattribs_names.addValue(prefix+".xFIHn_",s,null);
    this.dattribs_names.addValue(prefix+".xRFIL_",s,null);
    this.dattribs_names.addValue(prefix+".xRFIH_",s,null);

    n+=11;
    this.dattribs_names.addValue(prefix+".xL_P2Rt_",s,null);
    this.dattribs_names.addValue(prefix+".xH_P2Rt_",s,null);
    this.dattribs_names.addValue(prefix+".xL_P2V_",s,null);
    this.dattribs_names.addValue(prefix+".xH_P2V_",s,null);
    this.dattribs_names.addValue(prefix+".dpL_P2V_",s,null);
    this.dattribs_names.addValue(prefix+".dpH_P2V_",s,null);
    this.dattribs_names.addValue(prefix+".xL_VRt_",s,null);
    this.dattribs_names.addValue(prefix+".xH_VRt_",s,null);
    this.dattribs_names.addValue(prefix+".VRt_dp_i_",s,null);
    this.dattribs_names.addValue(prefix+".VRt_vs_",s,null);
    this.dattribs_names.addValue(prefix+".VRt_rvs_",s,null);

    n+=8;
    this.dattribs_names.addValue(prefix+".P2Ri_i_dp_i_",s,null);
    this.dattribs_names.addValue(prefix+".P2Ri_dp_i_",s,null);
    this.dattribs_names.addValue(prefix+".P2Ri_vs_",s,null);
    this.dattribs_names.addValue(prefix+".P2Ri_rvs_",s,null);
    this.dattribs_names.addValue(prefix+".P2Rt_dp_i_",s,null);
    this.dattribs_names.addValue(prefix+".P2Rt_dp_",s,null);
    this.dattribs_names.addValue(prefix+".P2Rt_vs_",s,null);
    this.dattribs_names.addValue(prefix+".P2Rt_rvs_",s,null);
    n+=3;
    this.dattribs_names.addValue(prefix+".x_P2Ri_i_dp_i_",s,null);
    this.dattribs_names.addValue(prefix+".x_P2Ri_dp_i_dp_",s,null);
    this.dattribs_names.addValue(prefix+".x_P2Ri_i_i_i_",s,null);



    return n;

}
private <T> int printDjQVMValues(final DjQVM<T> x,final CalculationData c)
{
    int n=0;


    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.RI);
    n+=EVXData_Values(x.P2Ri);
    n+=EVXData_Values(x.P2Rt);
    n+=EVXData_Values(x.P2V);
    n+=EVXData_Values(x.VRt);

   n+=4;
   this.dattribs.addValue(x.fXx);
   this.dattribs.addValue(x.fWdp);
   this.dattribs.addValue(x.fWvs);
   this.dattribs.addValue(x.fWx);

   n+=4;
   this.dattribs.addValue(x.qfXx);
   this.dattribs.addValue(x.qfWdp);
   this.dattribs.addValue(x.qfWvs);
   this.dattribs.addValue(x.qfWx);

   n+=4;
   this.dattribs.addValue(x.rfXx);
   this.dattribs.addValue(x.rfWdp);
   this.dattribs.addValue(x.rfWvs);
   this.dattribs.addValue(x.rfWx);

    n+=5;
    this.dattribs.addValue(x.fP1_0);
    this.dattribs.addValue(x.fP1_x);
    this.dattribs.addValue(x.fP1_xn);
    this.dattribs.addValue(x.fP1_xFIL);
    this.dattribs.addValue(x.fP1_xFIH);

    n+=10;
    this.dattribs.addValue(x.fi0);
    this.dattribs.addValue(x.fdp0);
    this.dattribs.addValue(x.fIdp0);
    this.dattribs.addValue(x.fvs0);
    this.dattribs.addValue(x.frvs0);
    this.dattribs.addValue(x.fix);
    this.dattribs.addValue(x.fdpx);
    this.dattribs.addValue(x.fIdpx);
    this.dattribs.addValue(x.fvsx);
    this.dattribs.addValue(x.frvsx);


    n+=10;
    this.dattribs.addValue(x.fRi0);
    this.dattribs.addValue(x.fRdp0);
    this.dattribs.addValue(x.fRIdp0);
    this.dattribs.addValue(x.fRvs0);
    this.dattribs.addValue(x.fRrvs0);
    this.dattribs.addValue(x.fRix);
    this.dattribs.addValue(x.fRdpx);
    this.dattribs.addValue(x.fRIdpx);
    this.dattribs.addValue(x.fRvsx);
    this.dattribs.addValue(x.fRrvsx);

    n+=8;
    this.dattribs.addValue(x.xIL);
    this.dattribs.addValue(x.xIH);
    this.dattribs.addValue(x.xFIL);
    this.dattribs.addValue(x.xFILn);
    this.dattribs.addValue(x.xFIH);
    this.dattribs.addValue(x.xFIHn);
    this.dattribs.addValue(x.xRFIL);
    this.dattribs.addValue(x.xRFIH);

    n+=11;
    this.dattribs.addValue(x.xL_P2Rt);
    this.dattribs.addValue(x.xH_P2Rt);
    this.dattribs.addValue(x.xL_P2V);
    this.dattribs.addValue(x.xH_P2V);
    this.dattribs.addValue(x.dpL_P2V);
    this.dattribs.addValue(x.dpH_P2V);
    this.dattribs.addValue(x.xL_VRt);
    this.dattribs.addValue(x.xH_VRt);
    this.dattribs.addValue(x.VRt_dp_i);
    this.dattribs.addValue(x.VRt_vs);
    this.dattribs.addValue(x.VRt_rvs);

    n+=8;
    this.dattribs.addValue(x.P2Ri_i_dp_i);
    this.dattribs.addValue(x.P2Ri_dp_i);
    this.dattribs.addValue(x.P2Ri_vs);
    this.dattribs.addValue(x.P2Ri_rvs);
    this.dattribs.addValue(x.P2Rt_dp_i);
    this.dattribs.addValue(x.P2Rt_dp);
    this.dattribs.addValue(x.P2Rt_vs);
    this.dattribs.addValue(x.P2Rt_rvs);
    n+=3;
    this.dattribs.addValue(x.x_P2Ri_i_dp_i);
    this.dattribs.addValue(x.x_P2Ri_dp_i_dp);
    this.dattribs.addValue(x.x_P2Ri_i_i_i);


    return n;

}




private int printREVDataNames(final String prefix,final String s,final int na)
{
    int n=0;
    for(int j=0;j<na;j++)
    {
	n+=3;
	this.dattribs_names.addValue(prefix+".lam_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".j_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".sigma2_j_"+j+"_",s,null);
    }
    return n;
}
private int printREVDataValues(final WQVM.REVData x,final int na)
{
    int n=0;
    for(int j=0;j<na;j++)
    {
	n+=3;
	this.dattribs.addValue(x.lam[j]);
	this.dattribs.addValue(x.j[j]);
	this.dattribs.addValue(x.sigma2_j[j]);
    }
    return n;
}

private int printGEVDataNames(final String prefix,final String s,final int na)
{
    int n=0;
    n+=6;
    this.dattribs_names.addValue(prefix+".Koo_",s,null);
    this.dattribs_names.addValue(prefix+".Iok_",s,null);
    this.dattribs_names.addValue(prefix+".pLok_",s,null);
    this.dattribs_names.addValue(prefix+".stdev2_pLok_",s,null);
    this.dattribs_names.addValue(prefix+".pRok_",s,null);
    this.dattribs_names.addValue(prefix+".stdev2_pRok_",s,null);
    for(int j=0;j<na;j++)
    {
	n+=5;
	this.dattribs_names.addValue(prefix+".lam_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".pL_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".stdev2_pL_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".pR_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".stdev2_pR_"+j+"_",s,null);
    }
    return n;
}
private int printGEVDataValues(final UQVM.GEVData x,final int na)
{
    int n=0;
    n+=6;
    this.dattribs.addValue(x.Koo);
    this.dattribs.addValue(x.Iok);
    this.dattribs.addValue(x.pLok);
    this.dattribs.addValue(x.stdev2_pLok);
    this.dattribs.addValue(x.pRok);
    this.dattribs.addValue(x.stdev2_pRok);

    for(int j=0;j<na;j++)
    {

	n+=5;
	this.dattribs.addValue(x.lam[j]);
	this.dattribs.addValue(x.pL[j]);
	this.dattribs.addValue(x.stdev2_pL[j]);
	this.dattribs.addValue(x.pR[j]);
	this.dattribs.addValue(x.stdev2_pR[j]);
    }
    return n;
}
private int printBEVDataNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".K00_",s,null);
    this.dattribs_names.addValue(prefix+".I0_",s,null);
    this.dattribs_names.addValue(prefix+".IpI_",s,null);
    this.dattribs_names.addValue(prefix+".Iok_",s,null);
    this.dattribs_names.addValue(prefix+".Iok_0_",s,null);
    this.dattribs_names.addValue(prefix+".Iok_o_",s,null);
    this.dattribs_names.addValue(prefix+".dP_Iok_",s,null);
    this.dattribs_names.addValue(prefix+".dP_Iok_0_",s,null);
    this.dattribs_names.addValue(prefix+".dP_Iok_o_",s,null);

    return n;
}
private int printBEVDataValues(final UQVM.BEVData x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.K00);
    this.dattribs.addValue(x.I0);
    this.dattribs.addValue(x.IpI);
    this.dattribs.addValue(x.Iok);
    this.dattribs.addValue(x.Iok_0);
    this.dattribs.addValue(x.Iok_o);
    this.dattribs.addValue(x.dP_Iok);
    this.dattribs.addValue(x.dP_Iok_0);
    this.dattribs.addValue(x.dP_Iok_o);

    return n;
}




private int printDQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".IT",s);

    n+=5;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".ndp_x0_",s,null);
    this.dattribs_names.addValue(prefix+".K_x0_",s,null);
    this.dattribs_names.addValue(prefix+".ndp_fXx_",s,null);
    this.dattribs_names.addValue(prefix+".K_fXx_",s,null);

    n+=2;
    this.dattribs_names.addValue(prefix+".jXx_",s,null);
    this.dattribs_names.addValue(prefix+".jWa0_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".K00_",s,null);
    this.dattribs_names.addValue(prefix+".odp0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_odp0_",s,null);
    this.dattribs_names.addValue(prefix+".odp0a_",s,null);
    this.dattribs_names.addValue(prefix+".ndp0_",s,null);
    this.dattribs_names.addValue(prefix+".neqe_",s,null);

    return n;

}
private <T> int printDQVMValues(final DQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=EVXData_Values(x.IT);

    n+=5;
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.ndp_x0);
    this.dattribs.addValue(x.K_x0);
    this.dattribs.addValue(x.ndp_fXx);
    this.dattribs.addValue(x.K_fXx);

    n+=2;
    this.dattribs.addValue(x.jXx);
    this.dattribs.addValue(x.jWa0);

    n+=6;
    this.dattribs.addValue(x.K00);
    this.dattribs.addValue(x.odp0);
    this.dattribs.addValue(x.sigma2_odp0);
    this.dattribs.addValue(x.odp0a);
    this.dattribs.addValue(x.ndp0);
    this.dattribs.addValue(x.neqe);

    return n;

}


private int printDkQVMWpPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".Px",s);

    n+=6;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".PfXx_",s,null);
    this.dattribs_names.addValue(prefix+".D2pxWfXx_",s,null);
    this.dattribs_names.addValue(prefix+".PmXx_",s,null);
    this.dattribs_names.addValue(prefix+".indX_",s,null);
    this.dattribs_names.addValue(prefix+".indW_",s,null);

    for(int j=0;j<10;j++)
    {
	n+=10;
	this.dattribs_names.addValue(prefix+".lamsP_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".XP_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".WP_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".Pm_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".Im_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".dIm_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".dpIm_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".DpIm_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".D2pxW_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".dmW_"+j+"_",s,null);
    }

    return n;
}

private <T> int printDkQVMWpPsiValues(final DkQVM<T>.WpPsi x,final CalculationData c)
{
    int n=0;

    n+=EVXData_Values(x.Px);

    n+=6;
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.PfXx);
    this.dattribs.addValue(x.D2pxWfXx);
    this.dattribs.addValue(x.PmXx);
    this.dattribs.addValue(x.indX);
    this.dattribs.addValue(x.indW);

    for(int j=0;j<10;j++)
    {
	n+=10;
	this.dattribs.addValue(j<x.lamsP.length?x.lamsP[j]:Double.NaN);
	this.dattribs.addValue(j<x.XP.length?x.XP[j]:Double.NaN);
	this.dattribs.addValue(j<x.WP.length?x.WP[j]:Double.NaN);
	this.dattribs.addValue(j<x.Pm.length?x.Pm[j]:Double.NaN);
	this.dattribs.addValue(j<x.Im.length?x.Im[j]:Double.NaN);
	this.dattribs.addValue(j<x.dIm.length?x.dIm[j]:Double.NaN);
	this.dattribs.addValue(j<x.dpIm.length?x.dpIm[j]:Double.NaN);
	this.dattribs.addValue(j<x.DpIm.length?x.DpIm[j]:Double.NaN);
	this.dattribs.addValue(j<x.D2pxW.length?x.D2pxW[j]:Double.NaN);
	this.dattribs.addValue(j<x.dmW.length?x.dmW[j]:Double.NaN);
    }

    return n;
}



private int printDkQVMWpVPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".P2x",s);

    for(int j=0;j<10;j++)
    {
	n+=5;
	this.dattribs_names.addValue(prefix+".lamsP_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".XP_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".X2P_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".varP_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".vargP_"+j+"_",s,null);
    }

    return n;
}

private <T> int printDkQVMWpVPsiValues(final DkQVM<T>.WpVPsi x,final CalculationData c)
{
    int n=0;

    n+=EVXData_Values(x.P2x);

    for(int j=0;j<10;j++)
    {
	n+=5;
	this.dattribs.addValue(j<x.lamsP.length?x.lamsP[j]:Double.NaN);
	this.dattribs.addValue(j<x.XP.length?x.XP[j]:Double.NaN);
	this.dattribs.addValue(j<x.X2P.length?x.X2P[j]:Double.NaN);
	this.dattribs.addValue(j<x.varP.length?x.varP[j]:Double.NaN);
	this.dattribs.addValue(j<x.vargP.length?x.vargP[j]:Double.NaN);
    }

    return n;
}


private int printDkQVMPVTatXNames(final String prefix,final String s)
{
    int n=0;
    n+=5;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".P_v_",s,null);
    this.dattribs_names.addValue(prefix+".P_t_",s,null);
    this.dattribs_names.addValue(prefix+".P_v0_",s,null);
    this.dattribs_names.addValue(prefix+".P_t0_",s,null);

    return n;
}

private <T> int printDkQVMPVTatXValues(final DkQVM<T>.PVTatX x,final CalculationData c)
{
    int n=0;

    n+=5;
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.P_v);
    this.dattribs.addValue(x.P_t);
    this.dattribs.addValue(x.P_v0);
    this.dattribs.addValue(x.P_t0);

    return n;
}


private int printDkQVMWVTPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".VT",s);
    n+=10;
    this.dattribs_names.addValue(prefix+".pT_L_",s,null);
    this.dattribs_names.addValue(prefix+".pV_L_",s,null);
    this.dattribs_names.addValue(prefix+".xT_L_",s,null);
    this.dattribs_names.addValue(prefix+".xV_L_",s,null);
    this.dattribs_names.addValue(prefix+".fXx_L_",s,null);
    this.dattribs_names.addValue(prefix+".pT_H_",s,null);
    this.dattribs_names.addValue(prefix+".pV_H_",s,null);
    this.dattribs_names.addValue(prefix+".xT_H_",s,null);
    this.dattribs_names.addValue(prefix+".xV_H_",s,null);
    this.dattribs_names.addValue(prefix+".fXx_H_",s,null);
    n+=4;
    this.dattribs_names.addValue(prefix+".DpV_L_",s,null);
    this.dattribs_names.addValue(prefix+".DpV_H_",s,null);
    this.dattribs_names.addValue(prefix+".DpT_L_",s,null);
    this.dattribs_names.addValue(prefix+".DpT_H_",s,null);

    return n;
}

private <T> int printDkQVMWVTPsiValues(final DkQVM<T>.WVTPsi x,final CalculationData c)
{
    int n=0;

    n+=EVXData_Values(x.VT);

    n+=10;
    this.dattribs.addValue(x.pT_L);
    this.dattribs.addValue(x.pV_L);
    this.dattribs.addValue(x.xT_L);
    this.dattribs.addValue(x.xV_L);
    this.dattribs.addValue(x.fXx_L);
    this.dattribs.addValue(x.pT_H);
    this.dattribs.addValue(x.pV_H);
    this.dattribs.addValue(x.xT_H);
    this.dattribs.addValue(x.xV_H);
    this.dattribs.addValue(x.fXx_H);
    n+=4;
    this.dattribs.addValue(x.DpV_L);
    this.dattribs.addValue(x.DpV_H);
    this.dattribs.addValue(x.DpT_L);
    this.dattribs.addValue(x.DpT_H);

    return n;
}


private int printDkQVMWVTDPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".VT",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".J_",s,null);
    this.dattribs_names.addValue(prefix+".C_",s,null);
    this.dattribs_names.addValue(prefix+".DC_",s,null);

    return n;
}

private <T> int printDkQVMWVTDPsiValues(final DkQVM<T>.WVTDPsi x,final CalculationData c)
{
    int n=0;

    n+=EVXData_Values(x.VT);

    n+=4;
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.J);
    this.dattribs.addValue(x.C);
    this.dattribs.addValue(x.DC);

    return n;
}




private int printDkQVMWqPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);

    n+=6;
    this.dattribs_names.addValue(prefix+".dP2_psi_",s,null);
    this.dattribs_names.addValue(prefix+".lamP_psi_",s,null);
    this.dattribs_names.addValue(prefix+".ind_psi_",s,null);
    this.dattribs_names.addValue(prefix+".dpsi_mux_dpsi_",s,null);
    this.dattribs_names.addValue(prefix+".dPpsiL_",s,null);
    this.dattribs_names.addValue(prefix+".dPpsiH_",s,null);

    return n;
}

private <T> int printDkQVMWqPsiValues(final DkQVM<T>.WqPsi x,final CalculationData c)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=6;
    this.dattribs.addValue(x.dP2_psi);
    this.dattribs.addValue(x.lamP_psi);
    this.dattribs.addValue(x.ind_psi);
    this.dattribs.addValue(x.dpsi_mux_dpsi);
    this.dattribs.addValue(x.dPpsiL);
    this.dattribs.addValue(x.dPpsiH);

    return n;
}


private int printDkQVMWoPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);

    n+=5;
    this.dattribs_names.addValue(prefix+".lamI_skip_",s,null);
    this.dattribs_names.addValue(prefix+".dP2_psi_",s,null);
    this.dattribs_names.addValue(prefix+".lamP_psi_",s,null);
    this.dattribs_names.addValue(prefix+".ind_psi_",s,null);
    this.dattribs_names.addValue(prefix+".dpsi_mux_dpsi_",s,null);

    return n;
}

private <T> int printDkQVMWoPsiValues(final DkQVM<T>.WoPsi x,final CalculationData c)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=5;
    this.dattribs.addValue(x.lamI_skip);
    this.dattribs.addValue(x.dP2_psi);
    this.dattribs.addValue(x.lamP_psi);
    this.dattribs.addValue(x.ind_psi);
    this.dattribs.addValue(x.dpsi_mux_dpsi);

    return n;
}



private int printDkQVMWABpsiNames(final String prefix,final String s)
{
    int n=0;
    n+=11;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".fAx_",s,null);
    this.dattribs_names.addValue(prefix+".fBx_",s,null);
    this.dattribs_names.addValue(prefix+".flamx_",s,null);
    this.dattribs_names.addValue(prefix+".fXxp_",s,null);
    this.dattribs_names.addValue(prefix+".fAxp_",s,null);
    this.dattribs_names.addValue(prefix+".fBxp_",s,null);
    this.dattribs_names.addValue(prefix+".flamxp_",s,null);
    this.dattribs_names.addValue(prefix+".lam_",s,null);
    this.dattribs_names.addValue(prefix+".fXxpamax_",s,null);
    this.dattribs_names.addValue(prefix+".muX_",s,null);

    return n;
}

private <T> int printDkQVMWABpsiValues(final DkQVM<T>.WABpsi x,final CalculationData c)
{
    int n=0;
    n+=11;
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.fAx);
    this.dattribs.addValue(x.fBx);
    this.dattribs.addValue(x.flamx);
    this.dattribs.addValue(x.fXxp);
    this.dattribs.addValue(x.fAxp);
    this.dattribs.addValue(x.fBxp);
    this.dattribs.addValue(x.flamxp);
    this.dattribs.addValue(x.lam);
    this.dattribs.addValue(x.fXxpamax);
    this.dattribs.addValue(x.muX);

    return n;
}




private int printDkQVMWdPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=7;
    this.dattribs_names.addValue(prefix+".lamm_",s,null);
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".jX_",s,null);
    this.dattribs_names.addValue(prefix+".dp2IX_",s,null);
    this.dattribs_names.addValue(prefix+".delta0_",s,null);
    this.dattribs_names.addValue(prefix+".f0min_",s,null);
    this.dattribs_names.addValue(prefix+".delta0rel_",s,null);

    return n;
}

private <T> int printDkQVMWdPsiValues(final DkQVM<T>.WdPsi x,final CalculationData c)
{
    int n=0;
    n+=7;
    this.dattribs.addValue(x.lamm);
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.jX);
    this.dattribs.addValue(x.dp2IX);
    this.dattribs.addValue(x.delta0);
    this.dattribs.addValue(x.f0min);
    this.dattribs.addValue(x.delta0rel);

    return n;
}



private int printDkQVMWfPsiNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".mu0_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".mu0_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".mu1_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".sigmma2_mu1_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".mu1_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_mu1_psiH_",s,null);

    return n;
}

private <T> int printDkQVMWfPsiValues(final DkQVM<T>.WfPsi x,final CalculationData c)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.mu0_psiL);
    this.dattribs.addValue(x.mu0_psiH);
    this.dattribs.addValue(x.mu1_psiL);
    this.dattribs.addValue(x.sigma2_mu1_psiL);
    this.dattribs.addValue(x.mu1_psiH);
    this.dattribs.addValue(x.sigma2_mu1_psiH);

    return n;
}


private int printDkQVMWmuPsiNames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".mu_Max_",s,null);
    this.dattribs_names.addValue(prefix+".fXx_Max_",s,null);
    this.dattribs_names.addValue(prefix+".pq_x_Max_",s,null);
    this.dattribs_names.addValue(prefix+".p_x_Max_",s,null);
    this.dattribs_names.addValue(prefix+".Mu_Max_",s,null);
    this.dattribs_names.addValue(prefix+".P_X_Max_",s,null);
    this.dattribs_names.addValue(prefix+".Mu_A_",s,null);
    this.dattribs_names.addValue(prefix+".P_X_A_",s,null);

    n+=40;
    this.dattribs_names.addValue(prefix+".g_t_",s,null);
    this.dattribs_names.addValue(prefix+".Fg_",s,null);
    this.dattribs_names.addValue(prefix+".P_X_",s,null);
    this.dattribs_names.addValue(prefix+".ldP_",s,null);
    this.dattribs_names.addValue(prefix+".Mu_",s,null);
    this.dattribs_names.addValue(prefix+".drPd_",s,null);
    this.dattribs_names.addValue(prefix+".p_tmd_",s,null);
    this.dattribs_names.addValue(prefix+".p_td_",s,null);
    this.dattribs_names.addValue(prefix+".dr_",s,null);
    this.dattribs_names.addValue(prefix+".drP_",s,null);
    this.dattribs_names.addValue(prefix+".txD1a_",s,null);
    this.dattribs_names.addValue(prefix+".txXD1a_",s,null);
    this.dattribs_names.addValue(prefix+".txD1b_",s,null);
    this.dattribs_names.addValue(prefix+".txXD1b_",s,null);
    this.dattribs_names.addValue(prefix+".xXD0_",s,null);
    this.dattribs_names.addValue(prefix+".xXD1_",s,null);
    this.dattribs_names.addValue(prefix+".xXD2_",s,null);
    this.dattribs_names.addValue(prefix+".xD0_",s,null);
    this.dattribs_names.addValue(prefix+".xD1_",s,null);
    this.dattribs_names.addValue(prefix+".xD2_",s,null);
    this.dattribs_names.addValue(prefix+".pq_x_",s,null);
    this.dattribs_names.addValue(prefix+".p_x_",s,null);
    this.dattribs_names.addValue(prefix+".p_x_p_",s,null);
    this.dattribs_names.addValue(prefix+".p_v_p_",s,null);
    this.dattribs_names.addValue(prefix+".p_t_p_",s,null);
    this.dattribs_names.addValue(prefix+".p_v_",s,null);
    this.dattribs_names.addValue(prefix+".p_t_",s,null);
    this.dattribs_names.addValue(prefix+".mf_",s,null);
    this.dattribs_names.addValue(prefix+".mu_",s,null);
    this.dattribs_names.addValue(prefix+".g_",s,null);
    this.dattribs_names.addValue(prefix+".dp_",s,null);
    this.dattribs_names.addValue(prefix+".g_dp_",s,null);
    this.dattribs_names.addValue(prefix+".lg_",s,null);
    this.dattribs_names.addValue(prefix+".ldp_",s,null);
    this.dattribs_names.addValue(prefix+".lg_ldp_",s,null);
    this.dattribs_names.addValue(prefix+".b_lg_ldp_",s,null);
    this.dattribs_names.addValue(prefix+".c_lg_ldp_",s,null);
    this.dattribs_names.addValue(prefix+".p_lg_ldp_",s,null);
    this.dattribs_names.addValue(prefix+".P_v_",s,null);
    this.dattribs_names.addValue(prefix+".P_t_",s,null);

    n+=printDkQVMWABpsiNames(prefix+".jL",s);

    return n;
}

private <T> int printDkQVMWmuPsiValues(final DkQVM<T>.WmuPsi x,final CalculationData c)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.mu_Max);
    this.dattribs.addValue(x.fXx_Max);
    this.dattribs.addValue(x.pq_x_Max);
    this.dattribs.addValue(x.p_x_Max);
    this.dattribs.addValue(x.Mu_Max);
    this.dattribs.addValue(x.P_X_Max);
    this.dattribs.addValue(x.Mu_A);
    this.dattribs.addValue(x.P_X_A);

    n+=40;
    this.dattribs.addValue(x.g_t);
    this.dattribs.addValue(x.Fg);
    this.dattribs.addValue(x.P_X);
    this.dattribs.addValue(x.ldP);
    this.dattribs.addValue(x.Mu);
    this.dattribs.addValue(x.drPd);
    this.dattribs.addValue(x.p_tmd);
    this.dattribs.addValue(x.p_td);
    this.dattribs.addValue(x.dr);
    this.dattribs.addValue(x.drP);
    this.dattribs.addValue(x.txD1a);
    this.dattribs.addValue(x.txXD1a);
    this.dattribs.addValue(x.txD1b);
    this.dattribs.addValue(x.txXD1b);
    this.dattribs.addValue(x.xXD0);
    this.dattribs.addValue(x.xXD1);
    this.dattribs.addValue(x.xXD2);
    this.dattribs.addValue(x.xD0);
    this.dattribs.addValue(x.xD1);
    this.dattribs.addValue(x.xD2);
    this.dattribs.addValue(x.pq_x);
    this.dattribs.addValue(x.p_x);
    this.dattribs.addValue(x.p_x_p);
    this.dattribs.addValue(x.p_v_p);
    this.dattribs.addValue(x.p_t_p);
    this.dattribs.addValue(x.p_v);
    this.dattribs.addValue(x.p_t);
    this.dattribs.addValue(x.mf);
    this.dattribs.addValue(x.mu);
    this.dattribs.addValue(x.g);
    this.dattribs.addValue(x.dp);
    this.dattribs.addValue(x.g_dp);
    this.dattribs.addValue(x.lg);
    this.dattribs.addValue(x.ldp);
    this.dattribs.addValue(x.lg_ldp);
    this.dattribs.addValue(x.b_lg_ldp);
    this.dattribs.addValue(x.c_lg_ldp);
    this.dattribs.addValue(x.p_lg_ldp);
    this.dattribs.addValue(x.P_v);
    this.dattribs.addValue(x.P_t);

    n+=printDkQVMWABpsiValues(x.jL,c);

    return n;
}





private int printDkQVMQuadEVNames(final String prefix,final String s)
{
    int n=0;

    n+=2;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".mu_",s,null);
    return n;
}

private <T> int printDkQVMQuadEVValues(final DkQVM<T>.QuadEV x)
{
    int n=0;

    n+=2;
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.mu);

    return n;
}


private int printDkQVMWDmuPsiNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=printDkQVMQuadEVNames(prefix+".Q",s);
    n+=printDkQVMQuadEVNames(prefix+".QD",s);

    n+=12;
    this.dattribs_names.addValue(prefix+".p_x_",s,null);
    this.dattribs_names.addValue(prefix+".p_Dx_",s,null);
    this.dattribs_names.addValue(prefix+".Dp_x_",s,null);
    this.dattribs_names.addValue(prefix+".Dmu_",s,null);
    this.dattribs_names.addValue(prefix+".Q_getDP_NoMu_",s,null);
    this.dattribs_names.addValue(prefix+".Q_getDP_NoDpsi_",s,null);
    this.dattribs_names.addValue(prefix+".Q_getXiSens_",s,null);
    this.dattribs_names.addValue(prefix+".Q_getXiSensAver_",s,null);
    this.dattribs_names.addValue(prefix+".Q_p_x_getFAverFromQQ_",s,null);
    this.dattribs_names.addValue(prefix+".Q0_getDP_NoMu_",s,null);
    this.dattribs_names.addValue(prefix+".Q0_getDP_NoDpsi_",s,null);
    this.dattribs_names.addValue(prefix+".Q0_getXiSens_",s,null);
    return n;
}

private <T> int printDkQVMWDmuPsiValues(final DkQVM<T>.WDmuPsi x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=printDkQVMQuadEVValues(x.Q);
    n+=printDkQVMQuadEVValues(x.QD);

    n+=12;
    this.dattribs.addValue(x.p_x);
    this.dattribs.addValue(x.p_Dx);
    this.dattribs.addValue(x.Dp_x);
    this.dattribs.addValue(x.Dmu);
    this.dattribs.addValue(x.Q_getDP_NoMu);
    this.dattribs.addValue(x.Q_getDP_NoDpsi);
    this.dattribs.addValue(x.Q_getXiSens);
    this.dattribs.addValue(x.Q_getXiSensAver);
    this.dattribs.addValue(x.Q_p_x_getFAverFromQQ);
    this.dattribs.addValue(x.Q0_getDP_NoMu);
    this.dattribs.addValue(x.Q0_getDP_NoDpsi);
    this.dattribs.addValue(x.Q0_getXiSens);

    return n;
}





private int printDkQVMWDDelaysBecauseOfSapsanNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".m.I",s);

    n+=2;
    this.dattribs_names.addValue(prefix+".xiSens_",s,null);
    this.dattribs_names.addValue(prefix+".m.fXx_",s,null);

    return n;
}

private <T> int printDkQVMWDDelaysBecauseOfSapsanValues(final DkQVM<T>.DDelaysBecauseOfSapsan x)
{
    int n=0;

    n+=EVXData_Values(x.m.I);

    n+=2;
    this.dattribs.addValue(x.xiSens);
    this.dattribs.addValue(x.m.fXx);

    return n;
}



private int printDkQVMWWkkEVNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=5;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    this.dattribs_names.addValue(prefix+".dPf_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2b_Pf_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_g_",s,null);

    for(int j=0;j<10;j++)
    {
	n+=2;
	this.dattribs_names.addValue(prefix+".Il_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".Pl_"+j+"_",s,null);
    }

    return n;
}

private <T> int printDkQVMWWkkEVValues(final DkQVM<T>.WkkEV x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=5;
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.Pf);
    this.dattribs.addValue(x.dPf);
    this.dattribs.addValue(x.sigma2b_Pf);
    this.dattribs.addValue(x.sigma2_g);

    for(int j=0;j<10;j++)
    {
	n+=2;
	this.dattribs.addValue(j<x.Il.length?x.Il[j]:Double.NaN);
	this.dattribs.addValue(j<x.Pl.length?x.Pl[j]:Double.NaN);
    }


    return n;
}


private int printDkQVMWWkkBoundaryNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=9;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    this.dattribs_names.addValue(prefix+".dPf_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2b_Pf_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_g_",s,null);
    this.dattribs_names.addValue(prefix+".Paf_",s,null);
    this.dattribs_names.addValue(prefix+".Ia_",s,null);
    this.dattribs_names.addValue(prefix+".Pbf_",s,null);
    this.dattribs_names.addValue(prefix+".Ib_",s,null);
    return n;
}

private <T> int printDkQVMWWkkBoundaryValues(final DkQVM<T>.WkkBoundary x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=9;
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.Pf);
    this.dattribs.addValue(x.dPf);
    this.dattribs.addValue(x.sigma2b_Pf);
    this.dattribs.addValue(x.sigma2_g);
    this.dattribs.addValue(x.Paf);
    this.dattribs.addValue(x.Ia);
    this.dattribs.addValue(x.Pbf);
    this.dattribs.addValue(x.Ib);

    return n;
}



private int printDkQVMWWkkVolBoundaryNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=1;
    this.dattribs_names.addValue(prefix+".dPf_",s,null);
    return n;
}

private <T> int printDkQVMWWkkVolBoundaryValues(final DkQVM<T>.WkkVolBoundary x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=1;
    this.dattribs.addValue(x.dPf);

    return n;
}







private int printACorrNames(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".x_a_b_",s,null);
    this.dattribs_names.addValue(prefix+".x_a_a_",s,null);
    this.dattribs_names.addValue(prefix+".x_b_b_",s,null);
    this.dattribs_names.addValue(prefix+".osp_",s,null);
    return n;
}

private int printACorrValues(final ACorr x)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.x_a_b);
    this.dattribs.addValue(x.x_a_a);
    this.dattribs.addValue(x.x_b_b);
    this.dattribs.addValue(x.osp);

    return n;
}


private int printECommNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=10;
    this.dattribs_names.addValue(prefix+".spDsens_",s,null);
    this.dattribs_names.addValue(prefix+".spDsens_rho_",s,null);
    this.dattribs_names.addValue(prefix+".xDsens_",s,null);
    this.dattribs_names.addValue(prefix+".IDsens_",s,null);
    this.dattribs_names.addValue(prefix+".Dsens_",s,null);
    this.dattribs_names.addValue(prefix+".Dsens_rho_",s,null);
    this.dattribs_names.addValue(prefix+".Dsens_nrm_",s,null);
    this.dattribs_names.addValue(prefix+".xxDsens_",s,null);
    this.dattribs_names.addValue(prefix+".xxDsens_rho_",s,null);
    this.dattribs_names.addValue(prefix+".xxDsens_Rrho_",s,null);

    return n;
}

private int printECommValues(final EComm x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=10;
    this.dattribs.addValue(x.spDsens);
    this.dattribs.addValue(x.spDsens_rho);
    this.dattribs.addValue(x.xDsens);
    this.dattribs.addValue(x.IDsens);
    this.dattribs.addValue(x.Dsens);
    this.dattribs.addValue(x.Dsens_rho);
    this.dattribs.addValue(x.Dsens_nrm);
    this.dattribs.addValue(x.xxDsens);
    this.dattribs.addValue(x.xxDsens_rho);
    this.dattribs.addValue(x.xxDsens_Rrho);

    return n;
}



private int printEiCommNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=3;
    this.dattribs_names.addValue(prefix+".Dsens_",s,null);
    this.dattribs_names.addValue(prefix+".Dsens_Rrho_",s,null);
    this.dattribs_names.addValue(prefix+".Dsens_rho_",s,null);

    return n;
}

private int printEiCommValues(final EiComm x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=3;
    this.dattribs.addValue(x.Dsens);
    this.dattribs.addValue(x.Dsens_Rrho);
    this.dattribs.addValue(x.Dsens_rho);

    return n;
}


private int printEpiCommNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=2;
    this.dattribs_names.addValue(prefix+".sensL_",s,null);
    this.dattribs_names.addValue(prefix+".sensH_",s,null);

    return n;
}

private int printEpiCommValues(final EpiComm x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=2;
    this.dattribs.addValue(x.sensL);
    this.dattribs.addValue(x.sensH);

    return n;
}




private int printEdCommNames(final String prefix,final String s)
{
    int n=0;

    n+=1;
    this.dattribs_names.addValue(prefix+".dir_",s,null);

    for(int j=0;j<8;j++)
    {
	n+=10;
	this.dattribs_names.addValue(prefix+".aPdir_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".axPdir_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".Pdir_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".Pdir_rho_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".Pdir_Rrho_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".Px_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".di_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".di_Rrho_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".df_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".df_Rrho_"+j+"_",s,null);
    }

    return n;
}

private int printEdCommValues(final EdComm x)
{
    int n=0;

    n+=1;
    this.dattribs.addValue(x.dir);

    final int d=x.Px.length;
    for(int j=0;j<8;j++)
    {
	n+=10;
	this.dattribs.addValue(j<d?x.aPdir[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.axPdir[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.Pdir[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.Pdir_rho[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.Pdir_Rrho[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.Px[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.di[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.di_Rrho[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.df[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.df_Rrho[j]:Double.NaN);
    }

    return n;
}


private int printEbCommNames(final String prefix,final String s)
{
    int n=0;

    n+=12;
    this.dattribs_names.addValue(prefix+".Pdir_L0_",s,null);
    this.dattribs_names.addValue(prefix+".Pdir_H0_",s,null);
    this.dattribs_names.addValue(prefix+".Pdir_LH_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".f_L0_",s,null);
    this.dattribs_names.addValue(prefix+".f_H0_",s,null);
    this.dattribs_names.addValue(prefix+".f_LH_",s,null);
    this.dattribs_names.addValue(prefix+".f_0_",s,null);
    this.dattribs_names.addValue(prefix+".f_L_",s,null);
    this.dattribs_names.addValue(prefix+".f_H_",s,null);

    return n;
}

private int printEbCommValues(final EbComm x)
{
    int n=0;

    n+=12;
    this.dattribs.addValue(x.Pdir_L0);
    this.dattribs.addValue(x.Pdir_H0);
    this.dattribs.addValue(x.Pdir_LH);
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.f_L0);
    this.dattribs.addValue(x.f_H0);
    this.dattribs.addValue(x.f_LH);
    this.dattribs.addValue(x.f_0);
    this.dattribs.addValue(x.f_L);
    this.dattribs.addValue(x.f_H);

    return n;
}


private int printEAssymNames(final String prefix,final String s)
{
    int n=0;

    n+=10;
    this.dattribs_names.addValue(prefix+".I_",s,null);
    this.dattribs_names.addValue(prefix+".pe_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pe_",s,null);
    this.dattribs_names.addValue(prefix+".pm_",s,null);
    this.dattribs_names.addValue(prefix+".sigma4_pm_",s,null);
    this.dattribs_names.addValue(prefix+".xi3_pe_",s,null);
    this.dattribs_names.addValue(prefix+".xi3_",s,null);
    this.dattribs_names.addValue(prefix+".sk_",s,null);
    this.dattribs_names.addValue(prefix+".plow_",s,null);
    this.dattribs_names.addValue(prefix+".phigh_",s,null);

    return n;
}

private int printEAssymValues(final EAssym x)
{
    int n=0;

    n+=10;
    this.dattribs.addValue(x.I);
    this.dattribs.addValue(x.pe);
    this.dattribs.addValue(x.sigma2_pe);
    this.dattribs.addValue(x.pm);
    this.dattribs.addValue(x.sigma4_pm);
    this.dattribs.addValue(x.xi3_pe);
    this.dattribs.addValue(x.xi3);
    this.dattribs.addValue(x.sk);
    this.dattribs.addValue(x.plow);
    this.dattribs.addValue(x.phigh);

    return n;
}

private int printEcCommNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".DsensL_",s,null);
    this.dattribs_names.addValue(prefix+".DsensL_rho_",s,null);
    this.dattribs_names.addValue(prefix+".DsensH_",s,null);
    this.dattribs_names.addValue(prefix+".DsensH_rho_",s,null);
    this.dattribs_names.addValue(prefix+".Dsens0_",s,null);
    this.dattribs_names.addValue(prefix+".Dsens0_rho_",s,null);

    return n;
}

private int printEcCommValues(final EcComm x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.DsensL);
    this.dattribs.addValue(x.DsensL_rho);
    this.dattribs.addValue(x.DsensH);
    this.dattribs.addValue(x.DsensH_rho);
    this.dattribs.addValue(x.Dsens0);
    this.dattribs.addValue(x.Dsens0_rho);

    return n;
}


private int printEpCommNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".pE1_",s,null);
    this.dattribs_names.addValue(prefix+".pE0_",s,null);
    this.dattribs_names.addValue(prefix+".pEm1_",s,null);
    this.dattribs_names.addValue(prefix+".pEm2_",s,null);
    this.dattribs_names.addValue(prefix+".pL0_",s,null);
    this.dattribs_names.addValue(prefix+".pH0_",s,null);

    return n;
}

private int printEpCommValues(final EpComm x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.pE1);
    this.dattribs.addValue(x.pE0);
    this.dattribs.addValue(x.pEm1);
    this.dattribs.addValue(x.pEm2);
    this.dattribs.addValue(x.pL0);
    this.dattribs.addValue(x.pH0);

    return n;
}


private int printEjpCommNames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".I_FL_",s,null);
    this.dattribs_names.addValue(prefix+".I_FH_",s,null);
    this.dattribs_names.addValue(prefix+".F_0_",s,null);
    this.dattribs_names.addValue(prefix+".ft_",s,null);
    this.dattribs_names.addValue(prefix+".fi_",s,null);
    n+=EVXData_Names(prefix+".F",s);
    n+=printACorrNames(prefix+".acorr",s);
    n+=printACorrNames(prefix+".bcorr",s);
    n+=printACorrNames(prefix+".ccorr",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".w_IH_FL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_FH_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_FL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_FH_",s,null);

    return n;
}

private int printEjpCommValues(final EjpComm x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.I_FL);
    this.dattribs.addValue(x.I_FH);
    this.dattribs.addValue(x.F_0);
    this.dattribs.addValue(x.ft);
    this.dattribs.addValue(x.fi);
    n+=EVXData_Values(x.F);
    n+=printACorrValues(x.acorr);
    n+=printACorrValues(x.bcorr);
    n+=printACorrValues(x.ccorr);

    n+=4;
    this.dattribs.addValue(x.w_IH_FL);
    this.dattribs.addValue(x.w_IH_FH);
    this.dattribs.addValue(x.w_IL_FL);
    this.dattribs.addValue(x.w_IL_FH);

    return n;
}



private int printEjaCommNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=5;
    this.dattribs_names.addValue(prefix+".p_t_",s,null);
    this.dattribs_names.addValue(prefix+".p_i_",s,null);
    this.dattribs_names.addValue(prefix+".p_nt_",s,null);
    this.dattribs_names.addValue(prefix+".p_ni_",s,null);
    this.dattribs_names.addValue(prefix+".spuri_",s,null);

    return n;
}

private int printEjaCommValues(final EjaComm x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=5;
    this.dattribs.addValue(x.p_t);
    this.dattribs.addValue(x.p_i);
    this.dattribs.addValue(x.p_nt);
    this.dattribs.addValue(x.p_ni);
    this.dattribs.addValue(x.spuri);

    return n;
}





private int printTSTwoLevelsNames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".I0a_",s,null);
    this.dattribs_names.addValue(prefix+".I0b_",s,null);
    this.dattribs_names.addValue(prefix+".f0a_",s,null);
    this.dattribs_names.addValue(prefix+".f0b_",s,null);
    this.dattribs_names.addValue(prefix+".IL_",s,null);
    this.dattribs_names.addValue(prefix+".IH_",s,null);
    this.dattribs_names.addValue(prefix+".fL_",s,null);
    this.dattribs_names.addValue(prefix+".fH_",s,null);

    return n;
}

private int printTSTwoLevelsValues(final TSTwoLevels x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.I0a);
    this.dattribs.addValue(x.I0b);
    this.dattribs.addValue(x.f0a);
    this.dattribs.addValue(x.f0b);
    this.dattribs.addValue(x.IL);
    this.dattribs.addValue(x.IH);
    this.dattribs.addValue(x.fL);
    this.dattribs.addValue(x.fH);

    return n;
}



private int printEfiCommNames(final String prefix,final String s)
{
    int n=0;

    n+=18;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".f0_",s,null);
    this.dattribs_names.addValue(prefix+".fL_",s,null);
    this.dattribs_names.addValue(prefix+".fH_",s,null);
    this.dattribs_names.addValue(prefix+".df0_",s,null);
    this.dattribs_names.addValue(prefix+".dfL_",s,null);
    this.dattribs_names.addValue(prefix+".dfH_",s,null);
    this.dattribs_names.addValue(prefix+".fi0_",s,null);
    this.dattribs_names.addValue(prefix+".fiL_",s,null);
    this.dattribs_names.addValue(prefix+".fiH_",s,null);
    this.dattribs_names.addValue(prefix+".dC0_",s,null);
    this.dattribs_names.addValue(prefix+".dC0_rho_",s,null);
    this.dattribs_names.addValue(prefix+".dCf0_",s,null);
    this.dattribs_names.addValue(prefix+".dCf0_rho_",s,null);
    this.dattribs_names.addValue(prefix+".fv_",s,null);
    this.dattribs_names.addValue(prefix+".fvn_",s,null);

    n+=printTSTwoLevelsNames(prefix+".L0",s);
    n+=printTSTwoLevelsNames(prefix+".H0",s);
    n+=printTSTwoLevelsNames(prefix+".DL",s);
    n+=printTSTwoLevelsNames(prefix+".DH",s);


    return n;
}

private int printEfiCommValues(final EfiComm x)
{
    int n=0;

    n+=18;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.f0);
    this.dattribs.addValue(x.fL);
    this.dattribs.addValue(x.fH);
    this.dattribs.addValue(x.df0);
    this.dattribs.addValue(x.dfL);
    this.dattribs.addValue(x.dfH);
    this.dattribs.addValue(x.fi0);
    this.dattribs.addValue(x.fiL);
    this.dattribs.addValue(x.fiH);
    this.dattribs.addValue(x.dC0);
    this.dattribs.addValue(x.dC0_rho);
    this.dattribs.addValue(x.dCf0);
    this.dattribs.addValue(x.dCf0_rho);
    this.dattribs.addValue(x.fv);
    this.dattribs.addValue(x.fvn);

    n+=printTSTwoLevelsValues(x.L0);
    n+=printTSTwoLevelsValues(x.H0);
    n+=printTSTwoLevelsValues(x.DL);
    n+=printTSTwoLevelsValues(x.DH);

    return n;
}



private int printEbiCommNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".ft_L0_",s,null);
    this.dattribs_names.addValue(prefix+".fi_L0_",s,null);
    this.dattribs_names.addValue(prefix+".ft_H0_",s,null);
    this.dattribs_names.addValue(prefix+".fi_H0_",s,null);
    this.dattribs_names.addValue(prefix+".ft_LH_",s,null);
    this.dattribs_names.addValue(prefix+".fi_LH_",s,null);

    return n;
}

private int printEbiCommValues(final EbiComm x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.ft_L0);
    this.dattribs.addValue(x.fi_L0);
    this.dattribs.addValue(x.ft_H0);
    this.dattribs.addValue(x.fi_H0);
    this.dattribs.addValue(x.ft_LH);
    this.dattribs.addValue(x.fi_LH);

    return n;
}



private int printEaCommNames(final String prefix,final String s)
{
    int n=0;

    n+=7;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".Istar_",s,null);
    this.dattribs_names.addValue(prefix+".f0_",s,null);
    this.dattribs_names.addValue(prefix+".f1_",s,null);
    this.dattribs_names.addValue(prefix+".f2_",s,null);

    return n;
}

private int printEaCommValues(final EaComm x)
{
    int n=0;

    n+=7;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.Istar);
    this.dattribs.addValue(x.f0);
    this.dattribs.addValue(x.f1);
    this.dattribs.addValue(x.f2);

    return n;
}





private int printDkQVMWkPsiNames(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".p_t_",s,null);
    this.dattribs_names.addValue(prefix+".p_i_",s,null);
    this.dattribs_names.addValue(prefix+".p_td_",s,null);
    n+=13;
    this.dattribs_names.addValue(prefix+".fDP_",s,null);
    this.dattribs_names.addValue(prefix+".fPD_",s,null);
    this.dattribs_names.addValue(prefix+".lam_",s,null);
    this.dattribs_names.addValue(prefix+".pf_",s,null);
    this.dattribs_names.addValue(prefix+".f_",s,null);
    this.dattribs_names.addValue(prefix+".ft_",s,null);
    this.dattribs_names.addValue(prefix+".fx0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_f_",s,null);
    this.dattribs_names.addValue(prefix+".Xd_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_Xd_",s,null);
    this.dattribs_names.addValue(prefix+".Pd_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_Pd_",s,null);
    this.dattribs_names.addValue(prefix+".Id_",s,null);

    n+=3;
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    this.dattribs_names.addValue(prefix+".Pfx_",s,null);
    this.dattribs_names.addValue(prefix+".Pfd_",s,null);

    return n;
}

private <T> int printDkQVMWkPsiValues(final DkQVM<T>.WkPsi x,final CalculationData c)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.p_t);
    this.dattribs.addValue(x.p_i);
    this.dattribs.addValue(x.p_td);
    n+=13;
    this.dattribs.addValue(x.fDP);
    this.dattribs.addValue(x.fPD);
    this.dattribs.addValue(x.lam);
    this.dattribs.addValue(x.pf);
    this.dattribs.addValue(x.f);
    this.dattribs.addValue(x.ft);
    this.dattribs.addValue(x.fx0);
    this.dattribs.addValue(x.sigma2_f);
    this.dattribs.addValue(x.Xd);
    this.dattribs.addValue(x.sigma2_Xd);
    this.dattribs.addValue(x.Pd);
    this.dattribs.addValue(x.sigma2_Pd);
    this.dattribs.addValue(x.Id);

    n+=3;
    this.dattribs.addValue(x.Pf);
    this.dattribs.addValue(x.Pfx);
    this.dattribs.addValue(x.Pfd);

    return n;
}



private int printDkQVMWkePsiNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".DPf_",s,null);
    this.dattribs_names.addValue(prefix+".dPf_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_Pf_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2b_Pf_",s,null);
    this.dattribs_names.addValue(prefix+".Pfd_",s,null);
    this.dattribs_names.addValue(prefix+".g_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_g_",s,null);
    this.dattribs_names.addValue(prefix+".rho_",s,null);

    return n;
}

private <T> int printDkQVMWkePsiValues(final DkQVM<T>.WkePsi x,final CalculationData c)
{
    int n=0;


    n+=9;
    this.dattribs.addValue(x.DPf);
    this.dattribs.addValue(x.dPf);
    this.dattribs.addValue(x.Pf);
    this.dattribs.addValue(x.sigma2_Pf);
    this.dattribs.addValue(x.sigma2b_Pf);
    this.dattribs.addValue(x.Pfd);
    this.dattribs.addValue(x.g);
    this.dattribs.addValue(x.sigma2_g);
    this.dattribs.addValue(x.rho);

    return n;
}


private int printDkQVMWkkPsiNames(final String prefix,final String s)
{
    int n=0;

    n+=12;
    this.dattribs_names.addValue(prefix+".I2_",s,null);
    this.dattribs_names.addValue(prefix+".PI2_",s,null);
    this.dattribs_names.addValue(prefix+".I2c_",s,null);
    this.dattribs_names.addValue(prefix+".PI2c_",s,null);
    this.dattribs_names.addValue(prefix+".dPI2c_",s,null);
    this.dattribs_names.addValue(prefix+".DPf_",s,null);
    this.dattribs_names.addValue(prefix+".dPf_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2b_Pf_",s,null);
    this.dattribs_names.addValue(prefix+".g_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_g_",s,null);
    this.dattribs_names.addValue(prefix+".fXx_",s,null);

    return n;
}

private <T> int printDkQVMWkkPsiValues(final DkQVM<T>.WkkPsi x,final CalculationData c)
{
    int n=0;


    n+=12;
    this.dattribs.addValue(x.I2);
    this.dattribs.addValue(x.PI2);
    this.dattribs.addValue(x.I2c);
    this.dattribs.addValue(x.PI2c);
    this.dattribs.addValue(x.dPI2c);
    this.dattribs.addValue(x.DPf);
    this.dattribs.addValue(x.dPf);
    this.dattribs.addValue(x.Pf);
    this.dattribs.addValue(x.sigma2b_Pf);
    this.dattribs.addValue(x.g);
    this.dattribs.addValue(x.sigma2_g);
    this.dattribs.addValue(x.fXx);

    return n;
}



private int printDkQVMWkfPsiNames(final String prefix,final String s)
{
    int n=0;

    n+=10;
    this.dattribs_names.addValue(prefix+".fXx_L_",s,null);
    this.dattribs_names.addValue(prefix+".dFL_",s,null);
    this.dattribs_names.addValue(prefix+".PtL_",s,null);
    this.dattribs_names.addValue(prefix+".PiL_",s,null);
    this.dattribs_names.addValue(prefix+".dPtL_",s,null);
    this.dattribs_names.addValue(prefix+".fXx_H_",s,null);
    this.dattribs_names.addValue(prefix+".dFH_",s,null);
    this.dattribs_names.addValue(prefix+".PtH_",s,null);
    this.dattribs_names.addValue(prefix+".PiH_",s,null);
    this.dattribs_names.addValue(prefix+".dPtH_",s,null);

    n+=EVXData_Names(prefix+".ID",s);

    return n;
}

private <T> int printDkQVMWkfPsiValues(final DkQVM<T>.WkfPsi x,final CalculationData c)
{
    int n=0;

    n+=10;
    this.dattribs.addValue(x.fXx_L);
    this.dattribs.addValue(x.dFL);
    this.dattribs.addValue(x.PtL);
    this.dattribs.addValue(x.PiL);
    this.dattribs.addValue(x.dPtL);
    this.dattribs.addValue(x.fXx_H);
    this.dattribs.addValue(x.dFH);
    this.dattribs.addValue(x.PtH);
    this.dattribs.addValue(x.PiH);
    this.dattribs.addValue(x.dPtH);

    n+=EVXData_Values(x.ID);

    return n;
}



private int printDkQVMWlPsiNames(final String prefix,final String s)
{
    int n=0;

    n+=5;
    this.dattribs_names.addValue(prefix+".lam_",s,null);
    this.dattribs_names.addValue(prefix+".f_",s,null);
    this.dattribs_names.addValue(prefix+".g_",s,null);
    this.dattribs_names.addValue(prefix+".fg_",s,null);
    this.dattribs_names.addValue(prefix+".mu_",s,null);

    return n;
}

private <T> int printDkQVMWlPsiValues(final DkQVM<T>.WlPsi x,final CalculationData c)
{
    int n=0;

    n+=5;
    this.dattribs.addValue(x.lam);
    this.dattribs.addValue(x.f);
    this.dattribs.addValue(x.g);
    this.dattribs.addValue(x.fg);
    this.dattribs.addValue(x.mu);

    return n;
}




private int printDkQVMWjPsiNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".w_IH_JL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_JH_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_JL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_JH_",s,null);
    this.dattribs_names.addValue(prefix+".I_JL_",s,null);
    this.dattribs_names.addValue(prefix+".I_JH_",s,null);
    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".J",s);
    n+=EVXData_Names(prefix+".K",s);
    n+=4;
    this.dattribs_names.addValue(prefix+".x_JH_",s,null);
    this.dattribs_names.addValue(prefix+".x_JL_",s,null);
    this.dattribs_names.addValue(prefix+".x_KH_",s,null);
    this.dattribs_names.addValue(prefix+".x_KL_",s,null);

    n+=5;
    this.dattribs_names.addValue(prefix+".x_dp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".x_IH_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_x_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dp_IH_",s,null);

    n+=5;
    this.dattribs_names.addValue(prefix+".x_dp_IL_",s,null);
    this.dattribs_names.addValue(prefix+".x_IL_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_x_IL_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IL_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_dp_IL_",s,null);


    return n;
}

private <T> int printDkQVMWjPsiValues(final DkQVM<T>.WjPsi x,final CalculationData c)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.w_IH_JL);
    this.dattribs.addValue(x.w_IH_JH);
    this.dattribs.addValue(x.w_IL_JL);
    this.dattribs.addValue(x.w_IL_JH);
    this.dattribs.addValue(x.I_JL);
    this.dattribs.addValue(x.I_JH);
    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.J);
    n+=EVXData_Values(x.K);
    n+=4;
    this.dattribs.addValue(x.x_JH);
    this.dattribs.addValue(x.x_JL);
    this.dattribs.addValue(x.x_KH);
    this.dattribs.addValue(x.x_KL);

    n+=5;
    this.dattribs.addValue(x.x_dp_IH);
    this.dattribs.addValue(x.x_IH);
    this.dattribs.addValue(x.sigma2_x_IH);
    this.dattribs.addValue(x.dp_IH);
    this.dattribs.addValue(x.sigma2_dp_IH);

    n+=5;
    this.dattribs.addValue(x.x_dp_IL);
    this.dattribs.addValue(x.x_IL);
    this.dattribs.addValue(x.sigma2_x_IL);
    this.dattribs.addValue(x.dp_IL);
    this.dattribs.addValue(x.sigma2_dp_IL);

    return n;
}




private int printDkQVMWXpsiNames(final String prefix,final String s)
{
    int n=0;
    n+=9;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".fW_",s,null);
    this.dattribs_names.addValue(prefix+".fWdp_",s,null);
    this.dattribs_names.addValue(prefix+".fWms_",s,null);
    this.dattribs_names.addValue(prefix+".fPx_",s,null);
    this.dattribs_names.addValue(prefix+".lam_",s,null);
    this.dattribs_names.addValue(prefix+".fW_x0_",s,null);
    this.dattribs_names.addValue(prefix+".fWdp_x0_",s,null);
    this.dattribs_names.addValue(prefix+".fWms_x0_",s,null);
    return n;
}

private <T> int printDkQVMWXpsiValues(final DkQVM<T>.WXpsi x,final CalculationData c)
{
    int n=0;
    n+=9;
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.fW);
    this.dattribs.addValue(x.fWdp);
    this.dattribs.addValue(x.fWms);
    this.dattribs.addValue(x.fPx);
    this.dattribs.addValue(x.lam);
    this.dattribs.addValue(x.fW_x0);
    this.dattribs.addValue(x.fWdp_x0);
    this.dattribs.addValue(x.fWms_x0);
    return n;
}




private int printDkQVMWLHpsiNames(final String prefix,final String s)
{
    int n=0;
    n+=4;
    this.dattribs_names.addValue(prefix+".xplus_",s,null);
    this.dattribs_names.addValue(prefix+".xminus_",s,null);
    this.dattribs_names.addValue(prefix+".quality_",s,null);
    this.dattribs_names.addValue(prefix+".dir_",s,null);

    n+=printDkQVMWABpsiNames(prefix+".jL",s);
    n+=EVXData_Names(prefix+".PM",s);

    return n;
}

private <T> int printDkQVMWLHpsiValues(final DkQVM<T>.WLHpsi x,final CalculationData c)
{
    int n=0;
    n+=4;
    this.dattribs.addValue(x.xplus);
    this.dattribs.addValue(x.xminus);
    this.dattribs.addValue(x.quality);
    this.dattribs.addValue(x.dir);

    n+=printDkQVMWABpsiValues(x.jL,c);
    n+=EVXData_Values(x.PM);

    return n;
}

private int printDkQVMWLHTpsiNames(final String prefix,final String s)
{
    int n=0;
    n+=6;
    this.dattribs_names.addValue(prefix+".mu_",s,null);
    this.dattribs_names.addValue(prefix+".muplus_",s,null);
    this.dattribs_names.addValue(prefix+".muminus_",s,null);
    this.dattribs_names.addValue(prefix+".xmu_",s,null);
    this.dattribs_names.addValue(prefix+".xmuplus_",s,null);
    this.dattribs_names.addValue(prefix+".xmuminus_",s,null);

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".Iplus",s);
    n+=EVXData_Names(prefix+".Iminus",s);
    n+=printDkQVMWABpsiNames(prefix+".jL",s);
    n+=printDkQVMWABpsiNames(prefix+".jLplus",s);
    n+=printDkQVMWABpsiNames(prefix+".jLminus",s);

    return n;
}

private <T> int printDkQVMWLHTpsiValues(final DkQVM<T>.WLHTpsi x,final CalculationData c)
{
    int n=0;
    n+=6;
    this.dattribs.addValue(x.mu);
    this.dattribs.addValue(x.muplus);
    this.dattribs.addValue(x.muminus);
    this.dattribs.addValue(x.xmu);
    this.dattribs.addValue(x.xmuplus);
    this.dattribs.addValue(x.xmuminus);

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.Iplus);
    n+=EVXData_Values(x.Iminus);
    n+=printDkQVMWABpsiValues(x.jL,c);
    n+=printDkQVMWABpsiValues(x.jLplus,c);
    n+=printDkQVMWABpsiValues(x.jLminus,c);

    return n;
}




private int printDkQVMWXpsipsi0Names(final String prefix,final String s)
{
    int n=0;
    n+=10;
    this.dattribs_names.addValue(prefix+".indM_",s,null);
    this.dattribs_names.addValue(prefix+".lam_",s,null);
    this.dattribs_names.addValue(prefix+".x0_",s,null);
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".flamx_",s,null);
    this.dattribs_names.addValue(prefix+".flamxvv_",s,null);
    this.dattribs_names.addValue(prefix+".fWx_",s,null);
    this.dattribs_names.addValue(prefix+".flamx0_",s,null);
    this.dattribs_names.addValue(prefix+".fwMw0_",s,null);
    this.dattribs_names.addValue(prefix+".K00_",s,null);
    return n;
}

private <T> int printDkQVMWXpsipsi0Values(final DkQVM<T>.WXpsipsi0 x,final CalculationData c)
{
    int n=0;
    n+=10;
    this.dattribs.addValue(x.indM);
    this.dattribs.addValue(x.lam);
    this.dattribs.addValue(x.x0);
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.flamx);
    this.dattribs.addValue(x.flamxvv);
    this.dattribs.addValue(x.fWx);
    this.dattribs.addValue(x.flamx0);
    this.dattribs.addValue(x.fwMw0);
    this.dattribs.addValue(x.K00);
    return n;
}





private int printDkQVMNames(final String prefix,final String s)
{
    int n=0;
    n+=printDkQVMWlPsiNames(prefix+".p_lamL_psiH",s);
    n+=printDkQVMWlPsiNames(prefix+".p_lamH_psiL",s);
    n+=printDkQVMWlPsiNames(prefix+".dp_lamL_psiH",s);
    n+=printDkQVMWlPsiNames(prefix+".dp_lamH_psiL",s);
    n+=printDkQVMWlPsiNames(prefix+".p_lam0_psiH",s);
    n+=printDkQVMWlPsiNames(prefix+".p_i0_psiH",s);
    n+=printDkQVMWlPsiNames(prefix+".p_mi0_psiH",s);
    n+=printDkQVMWlPsiNames(prefix+".p_lamn_psiH",s);


    n+=printDkQVMWkfPsiNames(prefix+".pik_sam",s);
    n+=printDkQVMWkfPsiNames(prefix+".pik",s);
    n+=printDkQVMWkfPsiNames(prefix+".ptk_sam",s);
    n+=printDkQVMWkfPsiNames(prefix+".ptk",s);

    n+=printDkQVMWkePsiNames(prefix+".epit_psiIL",s);
    n+=printDkQVMWkePsiNames(prefix+".epit_psiIH",s);
    n+=printDkQVMWkePsiNames(prefix+".eptt_psiIL",s);
    n+=printDkQVMWkePsiNames(prefix+".eptt_psiIH",s);
    n+=printDkQVMWkePsiNames(prefix+".epii_psiIL",s);
    n+=printDkQVMWkePsiNames(prefix+".epii_psiIH",s);
    n+=printDkQVMWkePsiNames(prefix+".epti_psiIL",s);
    n+=printDkQVMWkePsiNames(prefix+".epti_psiIH",s);

    n+=printDkQVMWkePsiNames(prefix+".edpt_psiIL",s);
    n+=printDkQVMWkePsiNames(prefix+".edpt_psiIH",s);
    n+=printDkQVMWkePsiNames(prefix+".edpi_psiIL",s);
    n+=printDkQVMWkePsiNames(prefix+".edpi_psiIH",s);

    n+=printDkQVMWkePsiNames(prefix+".edp_psiIDL",s);
    n+=printDkQVMWkePsiNames(prefix+".edp_psiIDH",s);


    n+=printDkQVMWkePsiNames(prefix+".evst_psiIL",s);
    n+=printDkQVMWkePsiNames(prefix+".evst_psiIH",s);
    n+=printDkQVMWkePsiNames(prefix+".evsi_psiIL",s);
    n+=printDkQVMWkePsiNames(prefix+".evsi_psiIH",s);
    n+=printDkQVMWkePsiNames(prefix+".etst_psiIL",s);
    n+=printDkQVMWkePsiNames(prefix+".etst_psiIH",s);
    n+=printDkQVMWkePsiNames(prefix+".etsi_psiIL",s);
    n+=printDkQVMWkePsiNames(prefix+".etsi_psiIH",s);


    n+=printDkQVMWkkPsiNames(prefix+".Ekpit_psiIL",s);
    n+=printDkQVMWkkPsiNames(prefix+".Rekpit_psiIL",s);
    n+=printDkQVMWkkPsiNames(prefix+".ekpit_psiIL",s);
    n+=printDkQVMWkkPsiNames(prefix+".ekpit_psiIH",s);
    n+=printDkQVMWkkPsiNames(prefix+".ekptt_psiIL",s);
    n+=printDkQVMWkkPsiNames(prefix+".ekptt_psiIH",s);
    n+=printDkQVMWkkPsiNames(prefix+".ekpii_psiIL",s);
    n+=printDkQVMWkkPsiNames(prefix+".ekpii_psiIH",s);
    n+=printDkQVMWkkPsiNames(prefix+".ekpti_psiIL",s);
    n+=printDkQVMWkkPsiNames(prefix+".ekpti_psiIH",s);


    n+=printDkQVMWkPsiNames(prefix+".dp_psiIL",s);
    n+=printDkQVMWkPsiNames(prefix+".dp_psiIH",s);
    n+=printDkQVMWkPsiNames(prefix+".pt_psiIL",s);
    n+=printDkQVMWkPsiNames(prefix+".pt_psiIH",s);

    n+=printDkQVMWkPsiNames(prefix+".p_psiIL",s);
    n+=printDkQVMWkPsiNames(prefix+".p_psiIH",s);
    n+=printDkQVMWkPsiNames(prefix+".j_psiIL",s);
    n+=printDkQVMWkPsiNames(prefix+".j_psiIH",s);

    n+=printDkQVMWkPsiNames(prefix+".pe_psiIL",s);
    n+=printDkQVMWkPsiNames(prefix+".pe_psiIH",s);
    n+=printDkQVMWkPsiNames(prefix+".je_psiIL",s);
    n+=printDkQVMWkPsiNames(prefix+".je_psiIH",s);

    n+=printDkQVMWoPsiNames(prefix+".wo0p",s);
    n+=printDkQVMWoPsiNames(prefix+".wo1p",s);
    n+=printDkQVMWoPsiNames(prefix+".wo2p",s);
    n+=printDkQVMWoPsiNames(prefix+".wo0s",s);
    n+=printDkQVMWoPsiNames(prefix+".wo1s",s);
    n+=printDkQVMWoPsiNames(prefix+".wo2s",s);

    n+=printDkQVMWqPsiNames(prefix+".wqmuP",s);
    n+=printDkQVMWqPsiNames(prefix+".wqmuPp",s);
    n+=printDkQVMWqPsiNames(prefix+".wqmuOp",s);
    n+=printDkQVMWqPsiNames(prefix+".wqmuPs",s);
    n+=printDkQVMWqPsiNames(prefix+".wqmuOs",s);

    n+=printDkQVMWpPsiNames(prefix+".Pmu",s);
    n+=printDkQVMWpPsiNames(prefix+".RPmu",s);
    n+=printDkQVMWpPsiNames(prefix+".Pv",s);
    n+=printDkQVMWpPsiNames(prefix+".Pt",s);
    n+=printDkQVMWpPsiNames(prefix+".Vt",s);
    n+=printDkQVMWpPsiNames(prefix+".Vty",s);
    n+=printDkQVMWpPsiNames(prefix+".P2t",s);
    n+=printDkQVMWpPsiNames(prefix+".P2v",s);

    n+=printDkQVMWpVPsiNames(prefix+".P2it_E",s);
    n+=printDkQVMWpVPsiNames(prefix+".P2it_XX",s);
    n+=printDkQVMWpVPsiNames(prefix+".P2tt_E",s);
    n+=printDkQVMWpVPsiNames(prefix+".P2tt_XX",s);
    n+=printDkQVMWpVPsiNames(prefix+".P2ti_E",s);
    n+=printDkQVMWpVPsiNames(prefix+".P2ti_XX",s);
    n+=printDkQVMWpVPsiNames(prefix+".Pd2ti_E",s);
    n+=printDkQVMWpVPsiNames(prefix+".Pd2ti_XX",s);
    n+=printDkQVMWpVPsiNames(prefix+".Pd2tV_E",s);
    n+=printDkQVMWpVPsiNames(prefix+".Pd2tV_XX",s);
    n+=printDkQVMWpVPsiNames(prefix+".Pd2tT_E",s);
    n+=printDkQVMWpVPsiNames(prefix+".Pd1tV_E",s);
    n+=printDkQVMWpVPsiNames(prefix+".Pd1tT_E",s);
    n+=printDkQVMWpVPsiNames(prefix+".Ad1tV_E",s);
    n+=printDkQVMWpVPsiNames(prefix+".Ad1ti_E",s);

    n+=printDkQVMWVTPsiNames(prefix+".vt",s);
    n+=printDkQVMWVTPsiNames(prefix+".vt_sam",s);
    n+=printDkQVMWVTDPsiNames(prefix+".vtD",s);


    n+=printDkQVMWmuPsiNames(prefix+".Rwmup",s);
    n+=printDkQVMWmuPsiNames(prefix+".Rwmup_sam",s);
    n+=printDkQVMWmuPsiNames(prefix+".wmupVT_sam",s);
    n+=printDkQVMWmuPsiNames(prefix+".wmusVT_sam",s);
    n+=printDkQVMWmuPsiNames(prefix+".wmup",s);
    n+=printDkQVMWmuPsiNames(prefix+".wmus_sam",s);
    n+=printDkQVMWmuPsiNames(prefix+".wmup_sam",s);
    n+=printDkQVMWmuPsiNames(prefix+".wmuv",s);
    n+=printDkQVMWmuPsiNames(prefix+".wmuv_sam",s);
    n+=printDkQVMWmuPsiNames(prefix+".wmudp",s);
    n+=printDkQVMWmuPsiNames(prefix+".wmudpT",s);
    n+=printDkQVMWmuPsiNames(prefix+".wmuvs",s);
    n+=printDkQVMWmuPsiNames(prefix+".wmurvs",s);


    n+=printDkQVMWjPsiNames(prefix+".jvSt",s);
    n+=printDkQVMWjPsiNames(prefix+".jdpt",s);
    n+=printDkQVMWjPsiNames(prefix+".jvStT",s);
    n+=printDkQVMWjPsiNames(prefix+".jdptT",s);

    n+=printDkQVMWfPsiNames(prefix+".rpmf",s);
    n+=printDkQVMWfPsiNames(prefix+".rvst",s);
    n+=printDkQVMWfPsiNames(prefix+".vst",s);
    n+=printDkQVMWfPsiNames(prefix+".dpt",s);

    n+=printDkQVMWdPsiNames(prefix+".dp2I",s);
    n+=printDkQVMWdPsiNames(prefix+".dp2Im",s);

    n+=EVXData_Names(prefix+".IT",s);
    n+=EVXData_Names(prefix+".ID",s);
    n+=EVXData_Names(prefix+".aI",s);
    n+=EVXData_Names(prefix+".p2I",s);
    n+=EVXData_Names(prefix+".pp2I",s);
    n+=EVXData_Names(prefix+".IrsellT",s);
    n+=EVXData_Names(prefix+".IrbuyT",s);

    n+=printDkQVMWXpsiNames(prefix+".L",s);
    n+=printDkQVMWXpsiNames(prefix+".H",s);
    n+=printDkQVMWXpsiNames(prefix+".dpL",s);
    n+=printDkQVMWXpsiNames(prefix+".dpH",s);

    n+=printDkQVMWXpsipsi0Names(prefix+".iL",s);
    n+=printDkQVMWXpsipsi0Names(prefix+".iH",s);

    n+=EVXData_Names(prefix+".JTVS",s);
    n+=EVXData_Names(prefix+".IBS",s);
    n+=printDkQVMWABpsiNames(prefix+".JbsL",s);
    n+=printDkQVMWABpsiNames(prefix+".JbsH",s);
    n+=EVXData_Names(prefix+".JDP",s);
    n+=printDkQVMWABpsiNames(prefix+".JdpL",s);
    n+=printDkQVMWABpsiNames(prefix+".JdpH",s);
    n+=printDkQVMWABpsiNames(prefix+".jL",s);
    n+=printDkQVMWABpsiNames(prefix+".jH",s);

    n+=printDkQVMWABpsiNames(prefix+".aL",s);
    n+=printDkQVMWABpsiNames(prefix+".aH",s);
    n+=printDkQVMWABpsiNames(prefix+".p2L",s);
    n+=1;
    this.dattribs_names.addValue(prefix+".p2L_jX_",s,null);
    n+=printDkQVMWABpsiNames(prefix+".p2H",s);
    n+=printDkQVMWABpsiNames(prefix+".pp2L",s);
    n+=printDkQVMWABpsiNames(prefix+".pp2H",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".sXi_",s,null);
    this.dattribs_names.addValue(prefix+".sXdp_",s,null);
    this.dattribs_names.addValue(prefix+".sXdpd_",s,null);
    this.dattribs_names.addValue(prefix+".sXbs_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".xVsc_",s,null);
    this.dattribs_names.addValue(prefix+".iVsc_",s,null);
    this.dattribs_names.addValue(prefix+".xVbc_",s,null);
    this.dattribs_names.addValue(prefix+".iVbc_",s,null);
    this.dattribs_names.addValue(prefix+".xVc_",s,null);
    this.dattribs_names.addValue(prefix+".iVc_",s,null);

    n+=EVXData_Names(prefix+".IRBS",s);
    n+=printDkQVMWABpsiNames(prefix+".IRBSL",s);
    n+=printDkQVMWABpsiNames(prefix+".IRBSH",s);
    n+=printDkQVMWABpsiNames(prefix+".dpLv",s);
    n+=printDkQVMWABpsiNames(prefix+".dpHv",s);

    n+=EVXData_Names(prefix+".tsI",s);
    n+=printDkQVMWABpsiNames(prefix+".tsLv",s);
    n+=printDkQVMWABpsiNames(prefix+".tsHv",s);

    n+=printDkQVMWABpsiNames(prefix+".jrsellT",s);
    n+=printDkQVMWABpsiNames(prefix+".jrbuyT",s);
    n+=printDkQVMWABpsiNames(prefix+".jTL",s);
    n+=printDkQVMWABpsiNames(prefix+".jTH",s);

    n+=6;
    this.dattribs_names.addValue(prefix+".jL_VS_",s,null);
    this.dattribs_names.addValue(prefix+".jL_VSp_",s,null);
    this.dattribs_names.addValue(prefix+".jL_V_",s,null);
    this.dattribs_names.addValue(prefix+".jL_Vp_",s,null);
    this.dattribs_names.addValue(prefix+".jL_vsx_",s,null);
    this.dattribs_names.addValue(prefix+".jL_vx_",s,null);

    n+=15;
    this.dattribs_names.addValue(prefix+".jL_re_vs_",s,null);
    this.dattribs_names.addValue(prefix+".jL_re_sigma2_vs_",s,null);
    this.dattribs_names.addValue(prefix+".jL_rex_vs_",s,null);
    this.dattribs_names.addValue(prefix+".jL_rex_sigma2_vs_",s,null);
    this.dattribs_names.addValue(prefix+".jL_re_rvs_",s,null);
    this.dattribs_names.addValue(prefix+".jL_re_sigma2_rvs_",s,null);
    this.dattribs_names.addValue(prefix+".jL_rex_rvs_",s,null);
    this.dattribs_names.addValue(prefix+".jL_rex_sigma2_rvs_",s,null);
    this.dattribs_names.addValue(prefix+".jL_re_dp_",s,null);
    this.dattribs_names.addValue(prefix+".jL_re_sigma2_dp_",s,null);
    this.dattribs_names.addValue(prefix+".jL_rex_dp_",s,null);
    this.dattribs_names.addValue(prefix+".jL_rex_sigma2_dp_",s,null);
    this.dattribs_names.addValue(prefix+".jL_Wxx_",s,null);
    this.dattribs_names.addValue(prefix+".jL_dTdMu_",s,null);
    this.dattribs_names.addValue(prefix+".jL_dVdMu_",s,null);

    n+=printDkQVMWABpsiNames(prefix+".jL_sam",s);
    n+=4;
    this.dattribs_names.addValue(prefix+".jL_sam_XS_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_XSp_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_X_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_Xp_",s,null);
    n+=4;
    this.dattribs_names.addValue(prefix+".jL_sam_qXS_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_qXSp_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_qX_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_qXp_",s,null);
    n+=2;
    this.dattribs_names.addValue(prefix+".jL_qP_",s,null);
    this.dattribs_names.addValue(prefix+".jL_qPp_",s,null);
    n+=5;
    this.dattribs_names.addValue(prefix+".jL_qPk_",s,null);
    this.dattribs_names.addValue(prefix+".jL_qPpk_",s,null);
    this.dattribs_names.addValue(prefix+".jL_qP0k_",s,null);
    this.dattribs_names.addValue(prefix+".xjL_qP_jH_",s,null);
    this.dattribs_names.addValue(prefix+".jL_qP_Hk_",s,null);


    n+=12;
    this.dattribs_names.addValue(prefix+".jL_dpm_",s,null);
    this.dattribs_names.addValue(prefix+".jL_dp_",s,null);
    this.dattribs_names.addValue(prefix+".jL_dp2_",s,null);
    this.dattribs_names.addValue(prefix+".jL_dp2e_",s,null);
    this.dattribs_names.addValue(prefix+".jL_vsm_",s,null);
    this.dattribs_names.addValue(prefix+".jL_vs_",s,null);
    this.dattribs_names.addValue(prefix+".jL_vs2_",s,null);
    this.dattribs_names.addValue(prefix+".jL_vs2e_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_xsm_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_xs_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_xs2_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_xs2e_",s,null);
    n+=4;
    this.dattribs_names.addValue(prefix+".jL_sam_xpm_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_xp_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_xp2_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_xp2e_",s,null);
    n+=4;
    this.dattribs_names.addValue(prefix+".jL_sam_pt_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_pv_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_D_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_sigma2_D_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".jL_pt_",s,null);
    this.dattribs_names.addValue(prefix+".jL_pv_",s,null);
    this.dattribs_names.addValue(prefix+".jL_D_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sigma2_D_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".jL_sam_St_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_Sv_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_SD_",s,null);
    this.dattribs_names.addValue(prefix+".jL_sam_sigma2_SD_",s,null);

    n+=printDkQVMWLHpsiNames(prefix+".jdp_pm",s);
    n+=printDkQVMWLHpsiNames(prefix+".jdp_T",s);
    n+=printDkQVMWLHpsiNames(prefix+".jvs_pm",s);
    n+=printDkQVMWLHpsiNames(prefix+".jvs_T",s);
    n+=printDkQVMWLHpsiNames(prefix+".jrvs_pm",s);
    n+=printDkQVMWLHpsiNames(prefix+".jrvs_T",s);

    n+=printDkQVMWLHTpsiNames(prefix+".jTrvs",s);
    n+=printDkQVMWLHTpsiNames(prefix+".jTvs",s);
    n+=printDkQVMWLHTpsiNames(prefix+".jTadp",s);

    n+=8;
    this.dattribs_names.addValue(prefix+".jL_dpplus_x_",s,null);
    this.dattribs_names.addValue(prefix+".jL_dpminus_x_",s,null);
    this.dattribs_names.addValue(prefix+".jL_dpplus_x2_",s,null);
    this.dattribs_names.addValue(prefix+".jL_dpminus_x2_",s,null);
    this.dattribs_names.addValue(prefix+".jL_vSplus_x_",s,null);
    this.dattribs_names.addValue(prefix+".jL_vSminus_x_",s,null);
    this.dattribs_names.addValue(prefix+".jL_vSplus_x2_",s,null);
    this.dattribs_names.addValue(prefix+".jL_vSminus_x2_",s,null);

    n+=10;
    this.dattribs_names.addValue(prefix+".jL_x_dpplus_",s,null);
    this.dattribs_names.addValue(prefix+".jL_x_dpminus_",s,null);
    this.dattribs_names.addValue(prefix+".jL_x_vSplus_",s,null);
    this.dattribs_names.addValue(prefix+".jL_x_vSminus_",s,null);
    this.dattribs_names.addValue(prefix+".jL_x_rvSplus_",s,null);
    this.dattribs_names.addValue(prefix+".jL_x_rvSminus_",s,null);
    this.dattribs_names.addValue(prefix+".jL_x_dpt_low_",s,null);
    this.dattribs_names.addValue(prefix+".jL_x_dpt_high_",s,null);
    this.dattribs_names.addValue(prefix+".jL_x_vSt_low_",s,null);
    this.dattribs_names.addValue(prefix+".jL_x_vSt_high_",s,null);

    n+=printDkQVMPVTatXNames(prefix+".Pvte_i",s);
    n+=printDkQVMPVTatXNames(prefix+".Pvt_i",s);
    n+=printDkQVMPVTatXNames(prefix+".Pvt_v",s);
    n+=printDkQVMPVTatXNames(prefix+".pvt_i",s);
    n+=printDkQVMPVTatXNames(prefix+".pvt_v",s);

    n+=printDkQVMWDmuPsiNames(prefix+".wmuDrs_sam",s);
    n+=printDkQVMWDmuPsiNames(prefix+".wmuDs_sam",s);
    n+=printDkQVMWDmuPsiNames(prefix+".wmuDp_sam",s);
    n+=printDkQVMWDmuPsiNames(prefix+".wmuDrsW_sam",s);
    n+=printDkQVMWDmuPsiNames(prefix+".wmuDsW_sam",s);
    n+=printDkQVMWDmuPsiNames(prefix+".wmuDpW_sam",s);

    n+=printACorrNames(prefix+".acorr_dp_t",s);
    n+=printACorrNames(prefix+".acorr_dp_i",s);
    n+=printACorrNames(prefix+".acorr_dp_di",s);
    n+=printACorrNames(prefix+".acorr_dp_mu",s);

    n+=printDkQVMWDDelaysBecauseOfSapsanNames(prefix+".mL_vtpv",s);
    n+=printDkQVMWDDelaysBecauseOfSapsanNames(prefix+".mL_vtpt",s);
    n+=printDkQVMWDDelaysBecauseOfSapsanNames(prefix+".mL_vtSvs",s);
    n+=printDkQVMWDDelaysBecauseOfSapsanNames(prefix+".mL_Svtvs",s);
    n+=printDkQVMWDDelaysBecauseOfSapsanNames(prefix+".mL_vtdp",s);
    n+=printDkQVMWDDelaysBecauseOfSapsanNames(prefix+".mH_vtpv",s);
    n+=printDkQVMWDDelaysBecauseOfSapsanNames(prefix+".mH_vtpt",s);
    n+=printDkQVMWDDelaysBecauseOfSapsanNames(prefix+".mH_vtSvs",s);
    n+=printDkQVMWDDelaysBecauseOfSapsanNames(prefix+".mH_Svtvs",s);
    n+=printDkQVMWDDelaysBecauseOfSapsanNames(prefix+".mH_vtdp",s);

    n+=printDkQVMWWkkEVNames(prefix+".sdeit_pi_L",s);
    n+=printDkQVMWWkkEVNames(prefix+".eit_pi_L",s);
    n+=printDkQVMWWkkEVNames(prefix+".eit_pi_H",s);
    n+=printDkQVMWWkkEVNames(prefix+".eit_pt_L",s);
    n+=printDkQVMWWkkEVNames(prefix+".eit_pt_H",s);

    n+=printDkQVMWWkkBoundaryNames(prefix+".sdbit_pi_L",s);
    n+=printDkQVMWWkkBoundaryNames(prefix+".bit_pi_L",s);
    n+=printDkQVMWWkkBoundaryNames(prefix+".bit_pi_H",s);
    n+=printDkQVMWWkkBoundaryNames(prefix+".bit_pt_L",s);
    n+=printDkQVMWWkkBoundaryNames(prefix+".bit_pt_H",s);
    n+=printDkQVMWWkkBoundaryNames(prefix+".biti_pi_L",s);
    n+=printDkQVMWWkkBoundaryNames(prefix+".biti_pi_H",s);
    n+=printDkQVMWWkkBoundaryNames(prefix+".biti_pt_L",s);
    n+=printDkQVMWWkkBoundaryNames(prefix+".biti_pt_H",s);

    n+=printDkQVMWWkkVolBoundaryNames(prefix+".cit_pi_L",s);
    n+=printDkQVMWWkkVolBoundaryNames(prefix+".cit_pi_H",s);

    return n;

}
private <T> int printDkQVMValues(final DkQVM<T> x,final CalculationData c)
{
    int n=0;
    n+=printDkQVMWlPsiValues(x.p_lamL_psiH,c);
    n+=printDkQVMWlPsiValues(x.p_lamH_psiL,c);
    n+=printDkQVMWlPsiValues(x.dp_lamL_psiH,c);
    n+=printDkQVMWlPsiValues(x.dp_lamH_psiL,c);
    n+=printDkQVMWlPsiValues(x.p_lam0_psiH,c);
    n+=printDkQVMWlPsiValues(x.p_i0_psiH,c);
    n+=printDkQVMWlPsiValues(x.p_mi0_psiH,c);
    n+=printDkQVMWlPsiValues(x.p_lamn_psiH,c);

    n+=printDkQVMWkfPsiValues(x.pik_sam,c);
    n+=printDkQVMWkfPsiValues(x.pik,c);
    n+=printDkQVMWkfPsiValues(x.ptk_sam,c);
    n+=printDkQVMWkfPsiValues(x.ptk,c);

    n+=printDkQVMWkePsiValues(x.epit_psiIL,c);
    n+=printDkQVMWkePsiValues(x.epit_psiIH,c);
    n+=printDkQVMWkePsiValues(x.eptt_psiIL,c);
    n+=printDkQVMWkePsiValues(x.eptt_psiIH,c);
    n+=printDkQVMWkePsiValues(x.epii_psiIL,c);
    n+=printDkQVMWkePsiValues(x.epii_psiIH,c);
    n+=printDkQVMWkePsiValues(x.epti_psiIL,c);
    n+=printDkQVMWkePsiValues(x.epti_psiIH,c);

    n+=printDkQVMWkePsiValues(x.edpt_psiIL,c);
    n+=printDkQVMWkePsiValues(x.edpt_psiIH,c);
    n+=printDkQVMWkePsiValues(x.edpi_psiIL,c);
    n+=printDkQVMWkePsiValues(x.edpi_psiIH,c);

    n+=printDkQVMWkePsiValues(x.edp_psiIDL,c);
    n+=printDkQVMWkePsiValues(x.edp_psiIDH,c);

    n+=printDkQVMWkePsiValues(x.evst_psiIL,c);
    n+=printDkQVMWkePsiValues(x.evst_psiIH,c);
    n+=printDkQVMWkePsiValues(x.evsi_psiIL,c);
    n+=printDkQVMWkePsiValues(x.evsi_psiIH,c);
    n+=printDkQVMWkePsiValues(x.etst_psiIL,c);
    n+=printDkQVMWkePsiValues(x.etst_psiIH,c);
    n+=printDkQVMWkePsiValues(x.etsi_psiIL,c);
    n+=printDkQVMWkePsiValues(x.etsi_psiIH,c);

    n+=printDkQVMWkkPsiValues(x.Ekpit_psiIL,c);
    n+=printDkQVMWkkPsiValues(x.Rekpit_psiIL,c);
    n+=printDkQVMWkkPsiValues(x.ekpit_psiIL,c);
    n+=printDkQVMWkkPsiValues(x.ekpit_psiIH,c);
    n+=printDkQVMWkkPsiValues(x.ekptt_psiIL,c);
    n+=printDkQVMWkkPsiValues(x.ekptt_psiIH,c);
    n+=printDkQVMWkkPsiValues(x.ekpii_psiIL,c);
    n+=printDkQVMWkkPsiValues(x.ekpii_psiIH,c);
    n+=printDkQVMWkkPsiValues(x.ekpti_psiIL,c);
    n+=printDkQVMWkkPsiValues(x.ekpti_psiIH,c);

    n+=printDkQVMWkPsiValues(x.dp_psiIL,c);
    n+=printDkQVMWkPsiValues(x.dp_psiIH,c);
    n+=printDkQVMWkPsiValues(x.pt_psiIL,c);
    n+=printDkQVMWkPsiValues(x.pt_psiIH,c);

    n+=printDkQVMWkPsiValues(x.p_psiIL,c);
    n+=printDkQVMWkPsiValues(x.p_psiIH,c);
    n+=printDkQVMWkPsiValues(x.j_psiIL,c);
    n+=printDkQVMWkPsiValues(x.j_psiIH,c);

    n+=printDkQVMWkPsiValues(x.pe_psiIL,c);
    n+=printDkQVMWkPsiValues(x.pe_psiIH,c);
    n+=printDkQVMWkPsiValues(x.je_psiIL,c);
    n+=printDkQVMWkPsiValues(x.je_psiIH,c);

    n+=printDkQVMWoPsiValues(x.wo0p,c);
    n+=printDkQVMWoPsiValues(x.wo1p,c);
    n+=printDkQVMWoPsiValues(x.wo2p,c);
    n+=printDkQVMWoPsiValues(x.wo0s,c);
    n+=printDkQVMWoPsiValues(x.wo1s,c);
    n+=printDkQVMWoPsiValues(x.wo2s,c);

    n+=printDkQVMWqPsiValues(x.wqmuP,c);
    n+=printDkQVMWqPsiValues(x.wqmuPp,c);
    n+=printDkQVMWqPsiValues(x.wqmuOp,c);
    n+=printDkQVMWqPsiValues(x.wqmuPs,c);
    n+=printDkQVMWqPsiValues(x.wqmuOs,c);

    n+=printDkQVMWpPsiValues(x.Pmu,c);
    n+=printDkQVMWpPsiValues(x.RPmu,c);
    n+=printDkQVMWpPsiValues(x.Pv,c);
    n+=printDkQVMWpPsiValues(x.Pt,c);
    n+=printDkQVMWpPsiValues(x.Vt,c);
    n+=printDkQVMWpPsiValues(x.Vty,c);
    n+=printDkQVMWpPsiValues(x.P2t,c);
    n+=printDkQVMWpPsiValues(x.P2v,c);

    n+=printDkQVMWpVPsiValues(x.P2it_E,c);
    n+=printDkQVMWpVPsiValues(x.P2it_XX,c);
    n+=printDkQVMWpVPsiValues(x.P2tt_E,c);
    n+=printDkQVMWpVPsiValues(x.P2tt_XX,c);
    n+=printDkQVMWpVPsiValues(x.P2ti_E,c);
    n+=printDkQVMWpVPsiValues(x.P2ti_XX,c);
    n+=printDkQVMWpVPsiValues(x.Pd2ti_E,c);
    n+=printDkQVMWpVPsiValues(x.Pd2ti_XX,c);
    n+=printDkQVMWpVPsiValues(x.Pd2tV_E,c);
    n+=printDkQVMWpVPsiValues(x.Pd2tV_XX,c);
    n+=printDkQVMWpVPsiValues(x.Pd2tT_E,c);
    n+=printDkQVMWpVPsiValues(x.Pd1tV_E,c);
    n+=printDkQVMWpVPsiValues(x.Pd1tT_E,c);
    n+=printDkQVMWpVPsiValues(x.Ad1tV_E,c);
    n+=printDkQVMWpVPsiValues(x.Ad1ti_E,c);

    n+=printDkQVMWVTPsiValues(x.vt,c);
    n+=printDkQVMWVTPsiValues(x.vt_sam,c);
    n+=printDkQVMWVTDPsiValues(x.vtD,c);

    n+=printDkQVMWmuPsiValues(x.Rwmup,c);
    n+=printDkQVMWmuPsiValues(x.Rwmup_sam,c);
    n+=printDkQVMWmuPsiValues(x.wmupVT_sam,c);
    n+=printDkQVMWmuPsiValues(x.wmusVT_sam,c);
    n+=printDkQVMWmuPsiValues(x.wmup,c);
    n+=printDkQVMWmuPsiValues(x.wmus_sam,c);
    n+=printDkQVMWmuPsiValues(x.wmup_sam,c);
    n+=printDkQVMWmuPsiValues(x.wmuv,c);
    n+=printDkQVMWmuPsiValues(x.wmuv_sam,c);
    n+=printDkQVMWmuPsiValues(x.wmudp,c);
    n+=printDkQVMWmuPsiValues(x.wmudpT,c);
    n+=printDkQVMWmuPsiValues(x.wmuvs,c);
    n+=printDkQVMWmuPsiValues(x.wmurvs,c);

    n+=printDkQVMWjPsiValues(x.jvSt,c);
    n+=printDkQVMWjPsiValues(x.jdpt,c);
    n+=printDkQVMWjPsiValues(x.jvStT,c);
    n+=printDkQVMWjPsiValues(x.jdptT,c);

    n+=printDkQVMWfPsiValues(x.rpmf,c);
    n+=printDkQVMWfPsiValues(x.rvst,c);
    n+=printDkQVMWfPsiValues(x.vst,c);
    n+=printDkQVMWfPsiValues(x.dpt,c);

    n+=printDkQVMWdPsiValues(x.dp2I,c);
    n+=printDkQVMWdPsiValues(x.dp2Im,c);

    n+=EVXData_Values(x.IT);
    n+=EVXData_Values(x.ID);
    n+=EVXData_Values(x.aI);
    n+=EVXData_Values(x.p2I);
    n+=EVXData_Values(x.pp2I);
    n+=EVXData_Values(x.IrsellT);
    n+=EVXData_Values(x.IrbuyT);

    n+=printDkQVMWXpsiValues(x.L,c);
    n+=printDkQVMWXpsiValues(x.H,c);
    n+=printDkQVMWXpsiValues(x.dpL,c);
    n+=printDkQVMWXpsiValues(x.dpH,c);

    n+=printDkQVMWXpsipsi0Values(x.iL,c);
    n+=printDkQVMWXpsipsi0Values(x.iH,c);

    n+=EVXData_Values(x.JTVS);
    n+=EVXData_Values(x.IBS);
    n+=printDkQVMWABpsiValues(x.JbsL,c);
    n+=printDkQVMWABpsiValues(x.JbsH,c);
    n+=EVXData_Values(x.JDP);
    n+=printDkQVMWABpsiValues(x.JdpL,c);
    n+=printDkQVMWABpsiValues(x.JdpH,c);
    n+=printDkQVMWABpsiValues(x.jL,c);
    n+=printDkQVMWABpsiValues(x.jH,c);

    n+=printDkQVMWABpsiValues(x.aL,c);
    n+=printDkQVMWABpsiValues(x.aH,c);
    n+=printDkQVMWABpsiValues(x.p2L,c);
    n+=1;
    this.dattribs.addValue(x.p2L_jX);
    n+=printDkQVMWABpsiValues(x.p2H,c);
    n+=printDkQVMWABpsiValues(x.pp2L,c);
    n+=printDkQVMWABpsiValues(x.pp2H,c);

    n+=4;
    this.dattribs.addValue(x.sXi);
    this.dattribs.addValue(x.sXdp);
    this.dattribs.addValue(x.sXdpd);
    this.dattribs.addValue(x.sXbs);

    n+=6;
    this.dattribs.addValue(x.xVsc);
    this.dattribs.addValue(x.iVsc);
    this.dattribs.addValue(x.xVbc);
    this.dattribs.addValue(x.iVbc);
    this.dattribs.addValue(x.xVc);
    this.dattribs.addValue(x.iVc);

    n+=EVXData_Values(x.IRBS);
    n+=printDkQVMWABpsiValues(x.IRBSL,c);
    n+=printDkQVMWABpsiValues(x.IRBSH,c);

    n+=printDkQVMWABpsiValues(x.dpLv,c);
    n+=printDkQVMWABpsiValues(x.dpHv,c);


    n+=EVXData_Values(x.tsI);
    n+=printDkQVMWABpsiValues(x.tsLv,c);
    n+=printDkQVMWABpsiValues(x.tsHv,c);

    n+=printDkQVMWABpsiValues(x.jrsellT,c);
    n+=printDkQVMWABpsiValues(x.jrbuyT,c);
    n+=printDkQVMWABpsiValues(x.jTL,c);
    n+=printDkQVMWABpsiValues(x.jTH,c);

    n+=6;
    this.dattribs.addValue(x.jL_VS);
    this.dattribs.addValue(x.jL_VSp);
    this.dattribs.addValue(x.jL_V);
    this.dattribs.addValue(x.jL_Vp);
    this.dattribs.addValue(x.jL_vsx);
    this.dattribs.addValue(x.jL_vx);

    n+=15;
    this.dattribs.addValue(x.jL_re_vs);
    this.dattribs.addValue(x.jL_re_sigma2_vs);
    this.dattribs.addValue(x.jL_rex_vs);
    this.dattribs.addValue(x.jL_rex_sigma2_vs);
    this.dattribs.addValue(x.jL_re_rvs);
    this.dattribs.addValue(x.jL_re_sigma2_rvs);
    this.dattribs.addValue(x.jL_rex_rvs);
    this.dattribs.addValue(x.jL_rex_sigma2_rvs);
    this.dattribs.addValue(x.jL_re_dp);
    this.dattribs.addValue(x.jL_re_sigma2_dp);
    this.dattribs.addValue(x.jL_rex_dp);
    this.dattribs.addValue(x.jL_rex_sigma2_dp);
    this.dattribs.addValue(x.jL_Wxx);
    this.dattribs.addValue(x.jL_dTdMu);
    this.dattribs.addValue(x.jL_dVdMu);

    n+=printDkQVMWABpsiValues(x.jL_sam,c);
    n+=4;
    this.dattribs.addValue(x.jL_sam_XS);
    this.dattribs.addValue(x.jL_sam_XSp);
    this.dattribs.addValue(x.jL_sam_X);
    this.dattribs.addValue(x.jL_sam_Xp);
    n+=4;
    this.dattribs.addValue(x.jL_sam_qXS);
    this.dattribs.addValue(x.jL_sam_qXSp);
    this.dattribs.addValue(x.jL_sam_qX);
    this.dattribs.addValue(x.jL_sam_qXp);
    n+=2;
    this.dattribs.addValue(x.jL_qP);
    this.dattribs.addValue(x.jL_qPp);

    n+=5;
    this.dattribs.addValue(x.jL_qPk);
    this.dattribs.addValue(x.jL_qPpk);
    this.dattribs.addValue(x.jL_qP0k);
    this.dattribs.addValue(x.xjL_qP_jH);
    this.dattribs.addValue(x.jL_qP_Hk);


    n+=12;
    this.dattribs.addValue(x.jL_dpm);
    this.dattribs.addValue(x.jL_dp);
    this.dattribs.addValue(x.jL_dp2);
    this.dattribs.addValue(x.jL_dp2e);
    this.dattribs.addValue(x.jL_vsm);
    this.dattribs.addValue(x.jL_vs);
    this.dattribs.addValue(x.jL_vs2);
    this.dattribs.addValue(x.jL_vs2e);
    this.dattribs.addValue(x.jL_sam_xsm);
    this.dattribs.addValue(x.jL_sam_xs);
    this.dattribs.addValue(x.jL_sam_xs2);
    this.dattribs.addValue(x.jL_sam_xs2e);

    n+=4;
    this.dattribs.addValue(x.jL_sam_xpm);
    this.dattribs.addValue(x.jL_sam_xp);
    this.dattribs.addValue(x.jL_sam_xp2);
    this.dattribs.addValue(x.jL_sam_xp2e);
    n+=4;
    this.dattribs.addValue(x.jL_sam_pt);
    this.dattribs.addValue(x.jL_sam_pv);
    this.dattribs.addValue(x.jL_sam_D);
    this.dattribs.addValue(x.jL_sam_sigma2_D);

    n+=4;
    this.dattribs.addValue(x.jL_pt);
    this.dattribs.addValue(x.jL_pv);
    this.dattribs.addValue(x.jL_D);
    this.dattribs.addValue(x.jL_sigma2_D);

    n+=4;
    this.dattribs.addValue(x.jL_sam_St);
    this.dattribs.addValue(x.jL_sam_Sv);
    this.dattribs.addValue(x.jL_sam_SD);
    this.dattribs.addValue(x.jL_sam_sigma2_SD);

    n+=printDkQVMWLHpsiValues(x.jdp_pm,c);
    n+=printDkQVMWLHpsiValues(x.jdp_T,c);
    n+=printDkQVMWLHpsiValues(x.jvs_pm,c);
    n+=printDkQVMWLHpsiValues(x.jvs_T,c);
    n+=printDkQVMWLHpsiValues(x.jrvs_pm,c);
    n+=printDkQVMWLHpsiValues(x.jrvs_T,c);

    n+=printDkQVMWLHTpsiValues(x.jTrvs,c);
    n+=printDkQVMWLHTpsiValues(x.jTvs,c);
    n+=printDkQVMWLHTpsiValues(x.jTadp,c);


    n+=8;
    this.dattribs.addValue(x.jL_dpplus_x);
    this.dattribs.addValue(x.jL_dpminus_x);
    this.dattribs.addValue(x.jL_dpplus_x2);
    this.dattribs.addValue(x.jL_dpminus_x2);
    this.dattribs.addValue(x.jL_vSplus_x);
    this.dattribs.addValue(x.jL_vSminus_x);
    this.dattribs.addValue(x.jL_vSplus_x2);
    this.dattribs.addValue(x.jL_vSminus_x2);


    n+=10;
    this.dattribs.addValue(x.jL_x_dpplus);
    this.dattribs.addValue(x.jL_x_dpminus);
    this.dattribs.addValue(x.jL_x_vSplus);
    this.dattribs.addValue(x.jL_x_vSminus);
    this.dattribs.addValue(x.jL_x_rvSplus);
    this.dattribs.addValue(x.jL_x_rvSminus);
    this.dattribs.addValue(x.jL_x_dpt_low);
    this.dattribs.addValue(x.jL_x_dpt_high);
    this.dattribs.addValue(x.jL_x_vSt_low);
    this.dattribs.addValue(x.jL_x_vSt_high);

    n+=printDkQVMPVTatXValues(x.Pvte_i,c);
    n+=printDkQVMPVTatXValues(x.Pvt_i,c);
    n+=printDkQVMPVTatXValues(x.Pvt_v,c);
    n+=printDkQVMPVTatXValues(x.pvt_i,c);
    n+=printDkQVMPVTatXValues(x.pvt_v,c);

    n+=printDkQVMWDmuPsiValues(x.wmuDrs_sam);
    n+=printDkQVMWDmuPsiValues(x.wmuDs_sam);
    n+=printDkQVMWDmuPsiValues(x.wmuDp_sam);
    n+=printDkQVMWDmuPsiValues(x.wmuDrsW_sam);
    n+=printDkQVMWDmuPsiValues(x.wmuDsW_sam);
    n+=printDkQVMWDmuPsiValues(x.wmuDpW_sam);

    n+=printACorrValues(x.acorr_dp_t);
    n+=printACorrValues(x.acorr_dp_i);
    n+=printACorrValues(x.acorr_dp_di);
    n+=printACorrValues(x.acorr_dp_mu);

    n+=printDkQVMWDDelaysBecauseOfSapsanValues(x.mL_vtpv);
    n+=printDkQVMWDDelaysBecauseOfSapsanValues(x.mL_vtpt);
    n+=printDkQVMWDDelaysBecauseOfSapsanValues(x.mL_vtSvs);
    n+=printDkQVMWDDelaysBecauseOfSapsanValues(x.mL_Svtvs);
    n+=printDkQVMWDDelaysBecauseOfSapsanValues(x.mL_vtdp);
    n+=printDkQVMWDDelaysBecauseOfSapsanValues(x.mH_vtpv);
    n+=printDkQVMWDDelaysBecauseOfSapsanValues(x.mH_vtpt);
    n+=printDkQVMWDDelaysBecauseOfSapsanValues(x.mH_vtSvs);
    n+=printDkQVMWDDelaysBecauseOfSapsanValues(x.mH_Svtvs);
    n+=printDkQVMWDDelaysBecauseOfSapsanValues(x.mH_vtdp);

    n+=printDkQVMWWkkEVValues(x.sdeit_pi_L);
    n+=printDkQVMWWkkEVValues(x.eit_pi_L);
    n+=printDkQVMWWkkEVValues(x.eit_pi_H);
    n+=printDkQVMWWkkEVValues(x.eit_pt_L);
    n+=printDkQVMWWkkEVValues(x.eit_pt_H);

    n+=printDkQVMWWkkBoundaryValues(x.sdbit_pi_L);
    n+=printDkQVMWWkkBoundaryValues(x.bit_pi_L);
    n+=printDkQVMWWkkBoundaryValues(x.bit_pi_H);
    n+=printDkQVMWWkkBoundaryValues(x.bit_pt_L);
    n+=printDkQVMWWkkBoundaryValues(x.bit_pt_H);
    n+=printDkQVMWWkkBoundaryValues(x.biti_pi_L);
    n+=printDkQVMWWkkBoundaryValues(x.biti_pi_H);
    n+=printDkQVMWWkkBoundaryValues(x.biti_pt_L);
    n+=printDkQVMWWkkBoundaryValues(x.biti_pt_H);

    n+=printDkQVMWWkkVolBoundaryValues(x.cit_pi_L);
    n+=printDkQVMWWkkVolBoundaryValues(x.cit_pi_H);

    return n;

}

private int printQVMFNames(final String prefix,final String s)
{
    int n=0;

    n+=10;
    this.dattribs_names.addValue(prefix+".pbuy_",s,null);
    this.dattribs_names.addValue(prefix+".k0buy_pbuy_",s,null);
    this.dattribs_names.addValue(prefix+".k0buy_psell_",s,null);
    this.dattribs_names.addValue(prefix+".k0buy_peq",s,null);
    this.dattribs_names.addValue(prefix+".psell_",s,null);
    this.dattribs_names.addValue(prefix+".k0sell_psell_",s,null);
    this.dattribs_names.addValue(prefix+".k0sell_pbuy_",s,null);
    this.dattribs_names.addValue(prefix+".k0sell_peq",s,null);
    this.dattribs_names.addValue(prefix+".peq_",s,null);
    this.dattribs_names.addValue(prefix+".peqx_",s,null);


    return n;

}
private <T> int printQVMFValues(final QVMF<T> x,final CalculationData c)
{
    int n=0;

    n+=10;
    this.dattribs.addValue(x.pbuy);
    this.dattribs.addValue(x.k0buy_pbuy);
    this.dattribs.addValue(x.k0buy_psell);
    this.dattribs.addValue(x.k0buy_peq);
    this.dattribs.addValue(x.psell);
    this.dattribs.addValue(x.k0sell_psell);
    this.dattribs.addValue(x.k0sell_pbuy);
    this.dattribs.addValue(x.k0sell_peq);
    this.dattribs.addValue(x.peq);
    this.dattribs.addValue(x.peqx);

    return n;

}


private int printHaarQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".pbuy_",s,null);
    this.dattribs_names.addValue(prefix+".k0buy_pbuy_",s,null);
    this.dattribs_names.addValue(prefix+".k0buy_psell_",s,null);
    this.dattribs_names.addValue(prefix+".k0buy_peq",s,null);
    this.dattribs_names.addValue(prefix+".psell_",s,null);
    this.dattribs_names.addValue(prefix+".k0sell_psell_",s,null);
    this.dattribs_names.addValue(prefix+".k0sell_pbuy_",s,null);
    this.dattribs_names.addValue(prefix+".k0sell_peq",s,null);
    this.dattribs_names.addValue(prefix+".peq_",s,null);


    return n;

}
private int printHaarQVMValues(final HaarQVM x,final CalculationData c)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.pbuy);
    this.dattribs.addValue(x.k0buy_pbuy);
    this.dattribs.addValue(x.k0buy_psell);
    this.dattribs.addValue(x.k0buy_peq);
    this.dattribs.addValue(x.psell);
    this.dattribs.addValue(x.k0sell_psell);
    this.dattribs.addValue(x.k0sell_pbuy);
    this.dattribs.addValue(x.k0sell_peq);
    this.dattribs.addValue(x.peq);

    return n;

}





private int printUQVMNames(final String prefix,final String s,final int na)
{
    int n=0;
    n+=5;
    this.dattribs_names.addValue(prefix+".I0_",s,null);
    this.dattribs_names.addValue(prefix+".Iok_",s,null);
    this.dattribs_names.addValue(prefix+".dpI0_",s,null);
    this.dattribs_names.addValue(prefix+".dpIok_",s,null);
    this.dattribs_names.addValue(prefix+".peq_",s,null);

    n+=9;
    this.dattribs_names.addValue(prefix+".K00v_",s,null);
    this.dattribs_names.addValue(prefix+".peqv_",s,null);
    this.dattribs_names.addValue(prefix+".stdev2_peqv_",s,null);
    this.dattribs_names.addValue(prefix+".K00t_",s,null);
    this.dattribs_names.addValue(prefix+".peqt_",s,null);
    this.dattribs_names.addValue(prefix+".stdev2_peqt_",s,null);
    this.dattribs_names.addValue(prefix+".K00d_",s,null);
    this.dattribs_names.addValue(prefix+".peqd_",s,null);
    this.dattribs_names.addValue(prefix+".stdev2_peqd_",s,null);

    n+=printBEVDataNames(prefix+".bv",s);
    n+=printBEVDataNames(prefix+".bt",s);


    n+=printGEVDataNames(prefix+".pv",s,na);
    n+=printGEVDataNames(prefix+".pt",s,na);
    n+=printGEVDataNames(prefix+".pvd",s,na);
    n+=printGEVDataNames(prefix+".pa",s,na);

    return n;
}
private <T> int printUQVMValues(final UQVM<T> x,final CalculationData c,final int na)
{
    int n=0;

    n+=5;
    this.dattribs.addValue(x.I0);
    this.dattribs.addValue(x.Iok);
    this.dattribs.addValue(x.dpI0);
    this.dattribs.addValue(x.dpIok);
    this.dattribs.addValue(x.peq);

    n+=9;
    this.dattribs.addValue(x.K00v);
    this.dattribs.addValue(x.peqv);
    this.dattribs.addValue(x.stdev2_peqv);
    this.dattribs.addValue(x.K00t);
    this.dattribs.addValue(x.peqt);
    this.dattribs.addValue(x.stdev2_peqt);
    this.dattribs.addValue(x.K00d);
    this.dattribs.addValue(x.peqd);
    this.dattribs.addValue(x.stdev2_peqd);

    n+=printBEVDataValues(x.bv);
    n+=printBEVDataValues(x.bt);

    n+=printGEVDataValues(x.pv,na);
    n+=printGEVDataValues(x.pt,na);
    n+=printGEVDataValues(x.pvd,na);
    n+=printGEVDataValues(x.pa,na);

    return n;
}

private int printWQVMNames(final String prefix,final String s,final int na)
{
    int n=0;

    n+=printREVDataNames(prefix+".rdp2",s,na);
    n+=printREVDataNames(prefix+".radp2",s,na);
    n+=printREVDataNames(prefix+".radp",s,na);
    n+=printREVDataNames(prefix+".rv",s,na);

    return n;
}
private <T> int printWQVMValues(final WQVM<T> x,final CalculationData c,final int na)
{
    int n=0;

    n+=printREVDataValues(x.rdp2,na);
    n+=printREVDataValues(x.radp2,na);
    n+=printREVDataValues(x.radp,na);
    n+=printREVDataValues(x.rv,na);

    return n;
}



private int printXEVDataNames(final String prefix,final String s)
{
    int n=0;
    n+=6;

    this.dattribs_names.addValue(prefix+".x_",s,null);
    this.dattribs_names.addValue(prefix+".Iok_",s,null);
    this.dattribs_names.addValue(prefix+".pLok_",s,null);
    this.dattribs_names.addValue(prefix+".stdev2_pLok_",s,null);
    this.dattribs_names.addValue(prefix+".pRok_",s,null);
    this.dattribs_names.addValue(prefix+".stdev2_pRok_",s,null);

    return n;
}
private int printXEVDataValues(final VQVM.XEVData x)
{
    int n=0;
    n+=6;

    this.dattribs.addValue(x.x);
    this.dattribs.addValue(x.Iok);
    this.dattribs.addValue(x.pLok);
    this.dattribs.addValue(x.stdev2_pLok);
    this.dattribs.addValue(x.pRok);
    this.dattribs.addValue(x.stdev2_pRok);

    return n;
}

private int printVQVMNames(final String prefix,final String s,final int na)
{
    int n=0;

    n+=3;
    this.dattribs_names.addValue(prefix+".dvse0_",s,null);
    this.dattribs_names.addValue(prefix+".min_dvse_",s,null);
    this.dattribs_names.addValue(prefix+".max_dvse_",s,null);

    n+=9;
    this.dattribs_names.addValue(prefix+".odp0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_odp0_",s,null);
    this.dattribs_names.addValue(prefix+".ods0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ods0_",s,null);
    this.dattribs_names.addValue(prefix+".odts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_odts0_",s,null);
    this.dattribs_names.addValue(prefix+".odvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_odvs0_",s,null);
    this.dattribs_names.addValue(prefix+".oK00_",s,null);

    n+=9;
    this.dattribs_names.addValue(prefix+".ndp0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ndp0_",s,null);
    this.dattribs_names.addValue(prefix+".nds0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_nds0_",s,null);
    this.dattribs_names.addValue(prefix+".ndts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ndts0_",s,null);
    this.dattribs_names.addValue(prefix+".ndvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ndvs0_",s,null);
    this.dattribs_names.addValue(prefix+".nK00_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".rdts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_rdts0_",s,null);
    this.dattribs_names.addValue(prefix+".rdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_rdvs0_",s,null);

    n+=TwoRootsDataNames(prefix+".rvd",s);
    n+=printXEVDataNames(prefix+".rvd0_pv",s);
    n+=printXEVDataNames(prefix+".rvd1_pv",s);
    n+=printXEVDataNames(prefix+".rvd2_pv",s);

    n+=TwoRootsDataNames(prefix+".rv",s);
    n+=printXEVDataNames(prefix+".rv0_pv",s);
    n+=printXEVDataNames(prefix+".rv1_pv",s);
    n+=printXEVDataNames(prefix+".rv2_pv",s);


    n+=TwoRootsDataNames(prefix+".rpd",s);
    n+=printXEVDataNames(prefix+".rpd0_pv",s);
    n+=printXEVDataNames(prefix+".rpd1_pv",s);
    n+=printXEVDataNames(prefix+".rpd2_pv",s);

    n+=TwoRootsDataNames(prefix+".rp",s);
    n+=printXEVDataNames(prefix+".rp0_pv",s);
    n+=printXEVDataNames(prefix+".rp1_pv",s);
    n+=printXEVDataNames(prefix+".rp2_pv",s);

    n+=TwoRootsDataNames(prefix+".ra",s);
    n+=printXEVDataNames(prefix+".ra0",s);
    n+=printXEVDataNames(prefix+".ra1",s);
    n+=printXEVDataNames(prefix+".ra2",s);

    n+=7;
    this.dattribs_names.addValue(prefix+".Kv00_",s,null);
    this.dattribs_names.addValue(prefix+".j0eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_j0eq_",s,null);
    this.dattribs_names.addValue(prefix+".js0eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_js0eq_",s,null);
    this.dattribs_names.addValue(prefix+".jt0eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_jt0eq_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".joeq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_joeq_",s,null);
    this.dattribs_names.addValue(prefix+".jsoeq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_jsoeq_",s,null);
    this.dattribs_names.addValue(prefix+".jtoeq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_jtoeq_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".a0eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_a0eq_",s,null);
    this.dattribs_names.addValue(prefix+".aoeq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_aoeq_",s,null);

    n+=8;
    this.dattribs_names.addValue(prefix+".pv0eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pv0eq_",s,null);
    this.dattribs_names.addValue(prefix+".pt0eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pt0eq_",s,null);
    this.dattribs_names.addValue(prefix+".pe0eq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pe0eq_",s,null);
    this.dattribs_names.addValue(prefix+".pvoeq_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pvoeq_",s,null);


    for(int j=0;j<na;j++)
    {
	n+=8;
	this.dattribs_names.addValue(prefix+".xvnodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".pvnodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".sigma2_pvnodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".ptnodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".sigma2_ptnodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".penodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".sigma2_penodes_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".pp_"+j+"_",s,null);
	n+=1;
	this.dattribs_names.addValue(prefix+".dvse_"+j+"_",s,null);
    }

    return n;

}
private <T> int printVQVMValues(final VQVM<T> x,final CalculationData c,final int na)
{
    int n=0;

    n+=3;
    this.dattribs.addValue(x.dvse0);
    this.dattribs.addValue(x.min_dvse);
    this.dattribs.addValue(x.max_dvse);

    n+=9;
    this.dattribs.addValue(x.odp0);
    this.dattribs.addValue(x.sigma2_odp0);
    this.dattribs.addValue(x.ods0);
    this.dattribs.addValue(x.sigma2_ods0);
    this.dattribs.addValue(x.odts0);
    this.dattribs.addValue(x.sigma2_odts0);
    this.dattribs.addValue(x.odvs0);
    this.dattribs.addValue(x.sigma2_odvs0);
    this.dattribs.addValue(x.oK00);

    n+=9;
    this.dattribs.addValue(x.ndp0);
    this.dattribs.addValue(x.sigma2_ndp0);
    this.dattribs.addValue(x.nds0);
    this.dattribs.addValue(x.sigma2_nds0);
    this.dattribs.addValue(x.ndts0);
    this.dattribs.addValue(x.sigma2_ndts0);
    this.dattribs.addValue(x.ndvs0);
    this.dattribs.addValue(x.sigma2_ndvs0);
    this.dattribs.addValue(x.nK00);

    n+=4;
    this.dattribs.addValue(x.rdts0);
    this.dattribs.addValue(x.sigma2_rdts0);
    this.dattribs.addValue(x.rdvs0);
    this.dattribs.addValue(x.sigma2_rdvs0);

    n+=TwoRootsDataValues(x.rvd);
    n+=printXEVDataValues(x.rvd0_pv);
    n+=printXEVDataValues(x.rvd1_pv);
    n+=printXEVDataValues(x.rvd2_pv);


    n+=TwoRootsDataValues(x.rv);
    n+=printXEVDataValues(x.rv0_pv);
    n+=printXEVDataValues(x.rv1_pv);
    n+=printXEVDataValues(x.rv2_pv);

    n+=TwoRootsDataValues(x.rpd);
    n+=printXEVDataValues(x.rpd0_pv);
    n+=printXEVDataValues(x.rpd1_pv);
    n+=printXEVDataValues(x.rpd2_pv);


    n+=TwoRootsDataValues(x.rp);
    n+=printXEVDataValues(x.rp0_pv);
    n+=printXEVDataValues(x.rp1_pv);
    n+=printXEVDataValues(x.rp2_pv);

    n+=TwoRootsDataValues(x.ra);
    n+=printXEVDataValues(x.ra0);
    n+=printXEVDataValues(x.ra1);
    n+=printXEVDataValues(x.ra2);

    n+=7;
    this.dattribs.addValue(x.Kv00);
    this.dattribs.addValue(x.j0eq);
    this.dattribs.addValue(x.sigma2_j0eq);
    this.dattribs.addValue(x.js0eq);
    this.dattribs.addValue(x.sigma2_js0eq);
    this.dattribs.addValue(x.jt0eq);
    this.dattribs.addValue(x.sigma2_jt0eq);

    n+=6;
    this.dattribs.addValue(x.joeq);
    this.dattribs.addValue(x.sigma2_joeq);
    this.dattribs.addValue(x.jsoeq);
    this.dattribs.addValue(x.sigma2_jsoeq);
    this.dattribs.addValue(x.jtoeq);
    this.dattribs.addValue(x.sigma2_jtoeq);

    n+=4;
    this.dattribs.addValue(x.a0eq);
    this.dattribs.addValue(x.sigma2_a0eq);
    this.dattribs.addValue(x.aoeq);
    this.dattribs.addValue(x.sigma2_aoeq);

    n+=8;
    this.dattribs.addValue(x.pv0eq);
    this.dattribs.addValue(x.sigma2_pv0eq);
    this.dattribs.addValue(x.pt0eq);
    this.dattribs.addValue(x.sigma2_pt0eq);
    this.dattribs.addValue(x.pe0eq);
    this.dattribs.addValue(x.sigma2_pe0eq);
    this.dattribs.addValue(x.pvoeq);
    this.dattribs.addValue(x.sigma2_pvoeq);

    for(int j=0;j<na;j++)
    {
	n+=8;
	this.dattribs.addValue(x.xvnodes[j]);
	this.dattribs.addValue(x.pvnodes[j]);
	this.dattribs.addValue(x.sigma2_pvnodes[j]);
	this.dattribs.addValue(x.ptnodes[j]);
	this.dattribs.addValue(x.sigma2_ptnodes[j]);
	this.dattribs.addValue(x.penodes[j]);
	this.dattribs.addValue(x.sigma2_penodes[j]);
	this.dattribs.addValue(x.pp[j]);
	n+=1;
	this.dattribs.addValue(x.dvse[j]);
    }

    return n;

}




private int printXQVMNames(final String prefix,final String s)
{
    int n=0;


    n+=1;
    this.dattribs_names.addValue(prefix+".tsL_",s,null);

    n+=9;
    this.dattribs_names.addValue(prefix+".ovdts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ovdts0_",s,null);
    this.dattribs_names.addValue(prefix+".ovdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ovdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".ovK00_",s,null);
    this.dattribs_names.addValue(prefix+".ovdtso_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ovdtso_",s,null);
    this.dattribs_names.addValue(prefix+".ovrdts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ovrdts0_",s,null);

    n+=9;
    this.dattribs_names.addValue(prefix+".otdts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_otdts0_",s,null);
    this.dattribs_names.addValue(prefix+".otdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_otdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".otK00_",s,null);
    this.dattribs_names.addValue(prefix+".otdvso_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_otdvso_",s,null);
    this.dattribs_names.addValue(prefix+".otrdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_otrdvs0_",s,null);

    n+=10;
    this.dattribs_names.addValue(prefix+".nvdts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_nvdts0_",s,null);
    this.dattribs_names.addValue(prefix+".nvdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_nvdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".nvK00_",s,null);
    this.dattribs_names.addValue(prefix+".nvK00a_",s,null);
    this.dattribs_names.addValue(prefix+".nvdts0a_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_nvdts0a_",s,null);
    this.dattribs_names.addValue(prefix+".nvdtsoa_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_nvdtsoa_",s,null);

    n+=10;
    this.dattribs_names.addValue(prefix+".ntdts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ntdts0_",s,null);
    this.dattribs_names.addValue(prefix+".ntdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ntdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".ntK00_",s,null);
    this.dattribs_names.addValue(prefix+".ntK00a_",s,null);
    this.dattribs_names.addValue(prefix+".ntdvs0a_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ntdvs0a_",s,null);
    this.dattribs_names.addValue(prefix+".ntdvsoa_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_ntdvsoa_",s,null);

    return n;

}
private <T> int printXQVMValues(final XQVM<T> x,final CalculationData c)
{
    int n=0;


    n+=1;
    this.dattribs.addValue(x.tsL);

    n+=9;
    this.dattribs.addValue(x.ovdts0);
    this.dattribs.addValue(x.sigma2_ovdts0);
    this.dattribs.addValue(x.ovdvs0);
    this.dattribs.addValue(x.sigma2_ovdvs0);
    this.dattribs.addValue(x.ovK00);
    this.dattribs.addValue(x.ovdtso);
    this.dattribs.addValue(x.sigma2_ovdtso);
    this.dattribs.addValue(x.ovrdts0);
    this.dattribs.addValue(x.sigma2_ovrdts0);

    n+=9;
    this.dattribs.addValue(x.otdts0);
    this.dattribs.addValue(x.sigma2_otdts0);
    this.dattribs.addValue(x.otdvs0);
    this.dattribs.addValue(x.sigma2_otdvs0);
    this.dattribs.addValue(x.otK00);
    this.dattribs.addValue(x.otdvso);
    this.dattribs.addValue(x.sigma2_otdvso);
    this.dattribs.addValue(x.otrdvs0);
    this.dattribs.addValue(x.sigma2_otrdvs0);

    n+=10;
    this.dattribs.addValue(x.nvdts0);
    this.dattribs.addValue(x.sigma2_nvdts0);
    this.dattribs.addValue(x.nvdvs0);
    this.dattribs.addValue(x.sigma2_nvdvs0);
    this.dattribs.addValue(x.nvK00);
    this.dattribs.addValue(x.nvK00a);
    this.dattribs.addValue(x.nvdts0a);
    this.dattribs.addValue(x.sigma2_nvdts0a);
    this.dattribs.addValue(x.nvdtsoa);
    this.dattribs.addValue(x.sigma2_nvdtsoa);

    n+=10;
    this.dattribs.addValue(x.ntdts0);
    this.dattribs.addValue(x.sigma2_ntdts0);
    this.dattribs.addValue(x.ntdvs0);
    this.dattribs.addValue(x.sigma2_ntdvs0);
    this.dattribs.addValue(x.ntK00);
    this.dattribs.addValue(x.ntK00a);
    this.dattribs.addValue(x.ntdvs0a);
    this.dattribs.addValue(x.sigma2_ntdvs0a);
    this.dattribs.addValue(x.ntdvsoa);
    this.dattribs.addValue(x.sigma2_ntdvsoa);

    return n;

}


private int printYQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".tsL_",s,null);
    this.dattribs_names.addValue(prefix+".otdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_otdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".otK00_",s,null);
    this.dattribs_names.addValue(prefix+".otrdvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_otrdvs0_",s,null);

    return n;

}
private <T> int printYQVMValues(final YQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.tsL);
    this.dattribs.addValue(x.otdvs0);
    this.dattribs.addValue(x.sigma2_otdvs0);
    this.dattribs.addValue(x.otK00);
    this.dattribs.addValue(x.otrdvs0);
    this.dattribs.addValue(x.sigma2_otrdvs0);

    return n;
}

private int printZQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=1;
    this.dattribs_names.addValue(prefix+".K00_",s,null);

    n+=8;

    this.dattribs_names.addValue(prefix+".ntrvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2T_ntrvs0_",s,null);
    this.dattribs_names.addValue(prefix+".otrvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2A_otrvs0_",s,null);

    this.dattribs_names.addValue(prefix+".ntsvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2T_ntsvs0_",s,null);
    this.dattribs_names.addValue(prefix+".otsvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2A_otsvs0_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".ntrts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2T_ntrts0_",s,null);
    this.dattribs_names.addValue(prefix+".otrts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2A_otrts0_",s,null);


    return n;
}
private <T> int printZQVMValues(final ZQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=1;
    this.dattribs.addValue(x.K00);

    n+=8;

    this.dattribs.addValue(x.ntrvs0);
    this.dattribs.addValue(x.sigma2T_ntrvs0);
    this.dattribs.addValue(x.otrvs0);
    this.dattribs.addValue(x.sigma2A_otrvs0);

    this.dattribs.addValue(x.ntsvs0);
    this.dattribs.addValue(x.sigma2T_ntsvs0);
    this.dattribs.addValue(x.otsvs0);
    this.dattribs.addValue(x.sigma2A_otsvs0);

    n+=4;
    this.dattribs.addValue(x.ntrts0);
    this.dattribs.addValue(x.sigma2T_ntrts0);
    this.dattribs.addValue(x.otrts0);
    this.dattribs.addValue(x.sigma2A_otrts0);

    return n;
}


private int printZVQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=1;
    this.dattribs_names.addValue(prefix+".K00_",s,null);

    n+=8;

    this.dattribs_names.addValue(prefix+".nvrts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2T_nvrts0_",s,null);
    this.dattribs_names.addValue(prefix+".ovrts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2A_ovrts0_",s,null);

    this.dattribs_names.addValue(prefix+".nvsts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2T_nvsts0_",s,null);
    this.dattribs_names.addValue(prefix+".ovsts0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2A_ovsts0_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".nvrvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2T_nvrvs0_",s,null);
    this.dattribs_names.addValue(prefix+".ovrvs0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2A_ovrvs0_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".nvdp0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2T_nvdp0_",s,null);
    this.dattribs_names.addValue(prefix+".ovdp0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2A_ovdp0_",s,null);

    return n;
}
private <T> int printZVQVMValues(final ZVQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=1;
    this.dattribs.addValue(x.K00);

    n+=8;

    this.dattribs.addValue(x.nvrts0);
    this.dattribs.addValue(x.sigma2T_nvrts0);
    this.dattribs.addValue(x.ovrts0);
    this.dattribs.addValue(x.sigma2A_ovrts0);

    this.dattribs.addValue(x.nvsts0);
    this.dattribs.addValue(x.sigma2T_nvsts0);
    this.dattribs.addValue(x.ovsts0);
    this.dattribs.addValue(x.sigma2A_ovsts0);

    n+=4;
    this.dattribs.addValue(x.nvrvs0);
    this.dattribs.addValue(x.sigma2T_nvrvs0);
    this.dattribs.addValue(x.ovrvs0);
    this.dattribs.addValue(x.sigma2A_ovrvs0);

    n+=4;
    this.dattribs.addValue(x.nvdp0);
    this.dattribs.addValue(x.sigma2T_nvdp0);
    this.dattribs.addValue(x.ovdp0);
    this.dattribs.addValue(x.sigma2A_ovdp0);

    return n;
}




private int printQVMpiNames(final String prefix,final String s)
{
    int n=0;

    /*
    n+=2;
    this.dattribs_names.addValue(prefix+".p_average_",s,null);
    this.dattribs_names.addValue(prefix+".sigma_",s,null);
    */
    n+=5;
    this.dattribs_names.addValue(prefix+".Ik_",s,null);
    this.dattribs_names.addValue(prefix+".dIk_",s,null);
    this.dattribs_names.addValue(prefix+".dIkL_",s,null);
    this.dattribs_names.addValue(prefix+".dIt_",s,null);
    this.dattribs_names.addValue(prefix+".dIv_",s,null);

    n+=8;
    this.dattribs_names.addValue(prefix+".sigma_",s,null);
    this.dattribs_names.addValue(prefix+".p_average_",s,null);
    this.dattribs_names.addValue(prefix+".di_",s,null);
    this.dattribs_names.addValue(prefix+".dip_",s,null);
    this.dattribs_names.addValue(prefix+".inorm_",s,null);
    this.dattribs_names.addValue(prefix+".pnorm_",s,null);
    this.dattribs_names.addValue(prefix+".dpi_eq_scaled_",s,null);
    this.dattribs_names.addValue(prefix+".dip_scaled_",s,null);

    return n;
}
private int printQVMpiValues(final QVMpi x,final CalculationData c)
{
    int n=0;

    /*
    n+=2;
    this.dattribs.addValue(x.p_average);
    this.dattribs.addValue(x.sigma);
    */

    n+=5;
    this.dattribs.addValue(x.Ik);
    this.dattribs.addValue(x.dIk);
    this.dattribs.addValue(x.dIkL);
    this.dattribs.addValue(x.dIt);
    this.dattribs.addValue(x.dIv);

    n+=8;
    this.dattribs.addValue(x.sigma);
    this.dattribs.addValue(x.p_average);
    this.dattribs.addValue(x.di);
    this.dattribs.addValue(x.dip);
    this.dattribs.addValue(x.inorm);
    this.dattribs.addValue(x.pnorm);
    this.dattribs.addValue(x.dpi_eq_scaled);
    this.dattribs.addValue(x.dip_scaled);

    return n;
}


private int printQVMLPVNames(final String prefix,final String s)
{
    int n=0;

    n+=2;
    this.dattribs_names.addValue(prefix+".p_average_",s,null);
    this.dattribs_names.addValue(prefix+".sigma_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".I_",s,null);
    this.dattribs_names.addValue(prefix+".dIdT_",s,null);
    this.dattribs_names.addValue(prefix+".dIdP_",s,null);
    this.dattribs_names.addValue(prefix+".dpt_",s,null);

    return n;
}
private int printQVMLPVValues(final QVM.QVMLPV x,final CalculationData c)
{
    int n=0;

    n+=2;
    this.dattribs.addValue(x.p_average);
    this.dattribs.addValue(x.sigma);

    n+=4;
    this.dattribs.addValue(x.I);
    this.dattribs.addValue(x.dIdT);
    this.dattribs.addValue(x.dIdP);
    this.dattribs.addValue(x.dpt);

    return n;
}


    private int setAttrLiquidity16(final boolean flag_processheader,
				   final String ticker,
				   final long [] delays,
				   final CalculationData c)
    {
	int n=0;

	for(int j=0;j<delays.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=String.valueOf(delays[j]/ModelParameters.DISTANCE_TIME_SCALE);
		n+=printQVMNames("LF16.atip",s);
		n+=printQATradeIPNames("LF16.qatip",s);
		n+=printQVMVTNames("LF16.vt",s);
		n+=printQVMVTMUNames("LF16.mmu",s);
		n+=printQVMLNames("LF16.qvml",s);
		n+=printQVMLLNames("LF16.qvmll",s);
		n+=printQVMPPNames("LF16.qvmpp",s);
		n+=printQVMpiNames("LF16.qvmpi",s);
		n+=printQVMRNames("LF16.qvmlmv",s);
		n+=printQVMRNames("LF16.qvmpmv",s);

		n+=printQVMDNames("LF16.qvmdll",s);
		n+=printQVMDNames("LF16.qvmdllbs",s);
		n+=printQVMDNames("LF16.qvmdpp",s);
		n+=printQVMDNames("LF16.qvmdppbs",s);

		n+=printAQVMNames("LF16.aqvmll",s);
		n+=printAQVMNames("LF16.aqvmpp",s);

		n+=printEQVMNames("LF16.eqvmll",s);
		n+=printEQVMNames("LF16.eqvmpp",s);

		n+=printIQVMNames("LF16.iqvmll",s);
		n+=printIQVMNames("LF16.iqvmpp",s);

		n+=printRQVMNames("LF16.rqvmll",s,CalculateLiquidityDeficit16.NT+1);
		n+=printRQVMNames("LF16.rqvmpp",s,CalculateLiquidityDeficit16.NT+1);
		n+=printSQVMNames("LF16.sqvmll",s,CalculateLiquidityDeficit16.NT);
		n+=printSQVMNames("LF16.sqvmpp",s,CalculateLiquidityDeficit16.NT);
		n+=printKQVMNames("LF16.kqvmll",s,CalculateLiquidityDeficit16.NT+1);
		n+=printKQVMNames("LF16.kqvmpp",s,CalculateLiquidityDeficit16.NT+1);

		n+=printLQVMNames("LF16.lqvml",s,CalculateLiquidityDeficit16.NT);
		n+=printLQVMNames("LF16.lqvmp",s,CalculateLiquidityDeficit16.NT);
		n+=printLQVMNames("LF16.rlqvml",s,CalculateLiquidityDeficit16.NT);
		n+=printLQVMNames("LF16.rlqvmp",s,CalculateLiquidityDeficit16.NT);

		n+=printMQVMNames("LF16.mqvmll",s);
		n+=printMQVMNames("LF16.mqvmpp",s);

		n+=printNQVMNames("LF16.nqvmll",s);
		n+=printNQVMNames("LF16.nqvmpp",s);

		n+=printOQVMNames("LF16.oqvmll",s);
		n+=printOQVMNames("LF16.oqvmpp",s);

		n+=printQQVMNames("LF16.qqvmll",s);
		n+=printQQVMNames("LF16.qqvmpp",s);

		n+=printUQVMNames("LF16.uqvmll",s,CalculateLiquidityDeficit16.NT);
		n+=printUQVMNames("LF16.uqvmpp",s,CalculateLiquidityDeficit16.NT);

		n+=printVQVMNames("LF16.vqvmll",s,CalculateLiquidityDeficit16.NT-1);
		n+=printVQVMNames("LF16.vqvmpp",s,CalculateLiquidityDeficit16.NT-1);

		n+=printWQVMNames("LF16.wqvmll",s,CalculateLiquidityDeficit16.NT);
		n+=printWQVMNames("LF16.wqvmpp",s,CalculateLiquidityDeficit16.NT);

		n+=printXQVMNames("LF16.xqvmll",s);
		n+=printXQVMNames("LF16.xqvmpp",s);
	    }
	    else
	    {
		final CalculateLiquidityDeficit16 l=c.topp.book_trades_liquiditydeficit16;
		final CalculateLiquidityDeficit16.VQ vx=l.tdata[j];
		n+=printQVMValues(vx.atip,c);
		n+=printQATradeIPValues(vx.qatip,c);
		n+=printQVMVTValues(vx.vt,c);
		n+=printQVMVTMUValues(vx.mmu,c);
		n+=printQVMLValues(vx.qvml,c);
		n+=printQVMLLValues(vx.qvmll,c);
		n+=printQVMPPValues(vx.qvmpp,c);
		n+=printQVMpiValues(vx.qvmpi,c);
		n+=printQVMRValues(vx.qvmlmv,c);
		n+=printQVMRValues(vx.qvmpmv,c);

		n+=printQVMDValues(vx.qvmdll,c);
		n+=printQVMDValues(vx.qvmdllbs,c);
		n+=printQVMDValues(vx.qvmdpp,c);
		n+=printQVMDValues(vx.qvmdppbs,c);

		n+=printAQVMValues(vx.aqvmll,c);
		n+=printAQVMValues(vx.aqvmpp,c);

		n+=printEQVMValues(vx.eqvmll,c);
		n+=printEQVMValues(vx.eqvmpp,c);

		n+=printIQVMValues(vx.iqvmll,c);
		n+=printIQVMValues(vx.iqvmpp,c);

		n+=printRQVMValues(vx.rqvmll,c,CalculateLiquidityDeficit16.NT+1);
		n+=printRQVMValues(vx.rqvmpp,c,CalculateLiquidityDeficit16.NT+1);
		n+=printSQVMValues(vx.sqvmll,c,CalculateLiquidityDeficit16.NT);
		n+=printSQVMValues(vx.sqvmpp,c,CalculateLiquidityDeficit16.NT);
		n+=printKQVMValues(vx.kqvmll,c,CalculateLiquidityDeficit16.NT+1);
		n+=printKQVMValues(vx.kqvmpp,c,CalculateLiquidityDeficit16.NT+1);

		n+=printLQVMValues(vx.lqvml,c,CalculateLiquidityDeficit16.NT);
		n+=printLQVMValues(vx.lqvmp,c,CalculateLiquidityDeficit16.NT);
		n+=printLQVMValues(vx.rlqvml,c,CalculateLiquidityDeficit16.NT);
		n+=printLQVMValues(vx.rlqvmp,c,CalculateLiquidityDeficit16.NT);

		n+=printMQVMValues(vx.mqvmll,c);
		n+=printMQVMValues(vx.mqvmpp,c);

		n+=printNQVMValues(vx.nqvmll,c);
		n+=printNQVMValues(vx.nqvmpp,c);

		n+=printOQVMValues(vx.oqvmll,c);
		n+=printOQVMValues(vx.oqvmpp,c);

		n+=printQQVMValues(vx.qqvmll,c);
		n+=printQQVMValues(vx.qqvmpp,c);

		n+=printUQVMValues(vx.uqvmll,c,CalculateLiquidityDeficit16.NT);
		n+=printUQVMValues(vx.uqvmpp,c,CalculateLiquidityDeficit16.NT);

		n+=printVQVMValues(vx.vqvmll,c,CalculateLiquidityDeficit16.NT-1);
		n+=printVQVMValues(vx.vqvmpp,c,CalculateLiquidityDeficit16.NT-1);

		n+=printWQVMValues(vx.wqvmll,c,CalculateLiquidityDeficit16.NT);
		n+=printWQVMValues(vx.wqvmpp,c,CalculateLiquidityDeficit16.NT);

		n+=printXQVMValues(vx.xqvmll,c);
		n+=printXQVMValues(vx.xqvmpp,c);
	    }
	}

	return n;
    }


 private int setAttrLiquidity17(final boolean flag_processheader,
				final String ticker,
				final long [] delays,
				final CalculationData c)
    {
	int n=0;

	for(int j=0;j<delays.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=String.valueOf(delays[j]/ModelParameters.DISTANCE_TIME_SCALE);

		n+=printDjQVMNames("LF17.djqvmll",s);
		n+=printDjQVMNames("LF17.djqvmpp",s);

		n+=printBjQVMNames("LF17.bjqvmll",s);
		n+=printBjQVMNames("LF17.bjqvmpp",s);
		n+=printDiQVMNames("LF17.diqvmll",s);
		n+=printDiQVMNames("LF17.diqvmpp",s);
		n+=printBiQVMNames("LF17.biqvmll",s);
		n+=printBiQVMNames("LF17.biqvmpp",s);
		n+=printBQVMNames("LF17.bqvmll",s);
		n+=printBQVMNames("LF17.bqvmpp",s);
		n+=printQQVMNames("LF17.qqvmll",s);
		n+=printQQVMNames("LF17.qqvmpp",s);
		n+=printZQVMNames("LF17.zqvmll",s);
		n+=printZQVMNames("LF17.zqvmpp",s);
		n+=printZVQVMNames("LF17.zvqvmll",s);
		n+=printZVQVMNames("LF17.zvqvmpp",s);
	    }
	    else
	    {
		final CalculateLiquidityDeficit17 l=c.topp.book_trades_liquiditydeficit17;
		final CalculateLiquidityDeficit17.VQ vx=l.tdata[j];

		n+=printDjQVMValues(vx.djqvmll,c);
		n+=printDjQVMValues(vx.djqvmpp,c);

		n+=printBjQVMValues(vx.bjqvmll,c);
		n+=printBjQVMValues(vx.bjqvmpp,c);
		n+=printDiQVMValues(vx.diqvmll,c);
		n+=printDiQVMValues(vx.diqvmpp,c);
		n+=printBiQVMValues(vx.biqvmll,c);
		n+=printBiQVMValues(vx.biqvmpp,c);
		n+=printBQVMValues(vx.bqvmll,c);
		n+=printBQVMValues(vx.bqvmpp,c);
		n+=printQQVMValues(vx.qqvmll,c);
		n+=printQQVMValues(vx.qqvmpp,c);
		n+=printZQVMValues(vx.zqvmll,c);
		n+=printZQVMValues(vx.zqvmpp,c);
		n+=printZVQVMValues(vx.zvqvmll,c);
		n+=printZVQVMValues(vx.zvqvmpp,c);
	    }
	}

	return n;
    }


 private int setAttrLiquidity18(final boolean flag_processheader,
				final String ticker,
				final long [] delays,
				final CalculationData c)
    {
	int n=0;

	for(int j=0;j<delays.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=String.valueOf(delays[j]/ModelParameters.DISTANCE_TIME_SCALE);

		n+=printDQVMNames("LF18.dqvmll",s);
		n+=printDQVMNames("LF18.dqvmpp",s);
		n+=printQVMFNames("LF18.b",s);
		n+=EVXData_Names("LF18.doqvmll.buy",s);
		n+=EVXData_Names("LF18.doqvmll.sell",s);
		n+=EVXData_Names("LF18.doqvmpp.buy",s);
		n+=EVXData_Names("LF18.doqvmpp.sell",s);
		n+=printHaarQVMNames("LF18.haar",s);
		n+=printDkQVMNames("LF18.dkqvmll",s);
		n+=printDkQVMNames("LF18.dkqvmpp",s);
	    }
	    else
	    {
		final CalculateLiquidityDeficit18 l=c.topp.book_trades_liquiditydeficit18;
		final CalculateLiquidityDeficit18.VQ vx=l.tdata[j];

		n+=printDQVMValues(vx.dqvmll,c);
		n+=printDQVMValues(vx.dqvmpp,c);
		n+=printQVMFValues(vx.b,c);
		n+=EVXData_Values(vx.doqvmll.buy);
		n+=EVXData_Values(vx.doqvmll.sell);
		n+=EVXData_Values(vx.doqvmpp.buy);
		n+=EVXData_Values(vx.doqvmpp.sell);
		n+=printHaarQVMValues(vx.haarqvm,c);
		n+=printDkQVMValues(vx.dkqvmll,c);
		n+=printDkQVMValues(vx.dkqvmpp,c);
	    }
	}

	return n;
    }


private int printEkQVMWEpvtNames(final String prefix,final String s)
{
    int n=0;


    n+=printPsiAverPertrNames(prefix+".pert",s);

    n+=19;
    this.dattribs_names.addValue(prefix+".Mu_",s,null);
    this.dattribs_names.addValue(prefix+".pMu_",s,null);
    this.dattribs_names.addValue(prefix+".xpMu_",s,null);
    this.dattribs_names.addValue(prefix+".l0_",s,null);
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".g_",s,null);
    this.dattribs_names.addValue(prefix+".getGScaled_",s,null);
    this.dattribs_names.addValue(prefix+".Fg_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_g_",s,null);
    this.dattribs_names.addValue(prefix+".pmu_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_pmu_",s,null);
    this.dattribs_names.addValue(prefix+".d0f_",s,null);
    this.dattribs_names.addValue(prefix+".d1f_",s,null);
    this.dattribs_names.addValue(prefix+".d2f_",s,null);
    this.dattribs_names.addValue(prefix+".d2fc_",s,null);
    this.dattribs_names.addValue(prefix+".d0i_",s,null);
    this.dattribs_names.addValue(prefix+".d1i_",s,null);
    this.dattribs_names.addValue(prefix+".d2i_",s,null);
    this.dattribs_names.addValue(prefix+".d2ic_",s,null);
    n+=17;
    this.dattribs_names.addValue(prefix+".dPtb_",s,null);
    this.dattribs_names.addValue(prefix+".dPto_",s,null);
    this.dattribs_names.addValue(prefix+".dPtd_",s,null);
    this.dattribs_names.addValue(prefix+".Ptb_",s,null);
    this.dattribs_names.addValue(prefix+".Pto_",s,null);
    this.dattribs_names.addValue(prefix+".Ptd_",s,null);
    this.dattribs_names.addValue(prefix+".Pib_",s,null);
    this.dattribs_names.addValue(prefix+".Pio_",s,null);
    this.dattribs_names.addValue(prefix+".Pid_",s,null);
    this.dattribs_names.addValue(prefix+".g_PL_",s,null);
    this.dattribs_names.addValue(prefix+".dg_PL_",s,null);
    this.dattribs_names.addValue(prefix+".dP_PL_",s,null);
    this.dattribs_names.addValue(prefix+".ipnl_PL_",s,null);
    this.dattribs_names.addValue(prefix+".g_PH_",s,null);
    this.dattribs_names.addValue(prefix+".dg_PH_",s,null);
    this.dattribs_names.addValue(prefix+".dP_PH_",s,null);
    this.dattribs_names.addValue(prefix+".ipnl_PH_",s,null);

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".PmuT",s);

    return n;
}

private <T> int printEkQVMWEpvtValues(final EkQVM<T>.WEpvt x)
{
    int n=0;

    n+=printPsiAverPertrValues(x.pert);

    n+=19;
    this.dattribs.addValue(x.Mu);
    this.dattribs.addValue(x.pMu);
    this.dattribs.addValue(x.xpMu);
    this.dattribs.addValue(x.l0);
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.g);
    this.dattribs.addValue(x.getGScaled());
    this.dattribs.addValue(x.Fg);
    this.dattribs.addValue(x.sigma2_g);
    this.dattribs.addValue(x.pmu);
    this.dattribs.addValue(x.sigma2_pmu);
    this.dattribs.addValue(x.d0f);
    this.dattribs.addValue(x.d1f);
    this.dattribs.addValue(x.d2f);
    this.dattribs.addValue(x.d2fc);
    this.dattribs.addValue(x.d0i);
    this.dattribs.addValue(x.d1i);
    this.dattribs.addValue(x.d2i);
    this.dattribs.addValue(x.d2ic);
    n+=17;
    this.dattribs.addValue(x.dPtb);
    this.dattribs.addValue(x.dPto);
    this.dattribs.addValue(x.dPtd);
    this.dattribs.addValue(x.Ptb);
    this.dattribs.addValue(x.Pto);
    this.dattribs.addValue(x.Ptd);
    this.dattribs.addValue(x.Pib);
    this.dattribs.addValue(x.Pio);
    this.dattribs.addValue(x.Pid);
    this.dattribs.addValue(x.g_PL);
    this.dattribs.addValue(x.dg_PL);
    this.dattribs.addValue(x.dP_PL);
    this.dattribs.addValue(x.ipnl_PL);
    this.dattribs.addValue(x.g_PH);
    this.dattribs.addValue(x.dg_PH);
    this.dattribs.addValue(x.dP_PH);
    this.dattribs.addValue(x.ipnl_PH);

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.PmuT);

    return n;
}




private int printEkQVMWProjNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".pmu_iL_",s,null);
    this.dattribs_names.addValue(prefix+".pmu_iH_",s,null);
    this.dattribs_names.addValue(prefix+".P_iL_PL_",s,null);
    this.dattribs_names.addValue(prefix+".P_iL_PH_",s,null);
    this.dattribs_names.addValue(prefix+".P_iH_PL_",s,null);
    this.dattribs_names.addValue(prefix+".P_iH_PH_",s,null);

    n+=8;
    this.dattribs_names.addValue(prefix+".I_iL_PL_",s,null);
    this.dattribs_names.addValue(prefix+".I_iL_PH_",s,null);
    this.dattribs_names.addValue(prefix+".I_iH_PL_",s,null);
    this.dattribs_names.addValue(prefix+".I_iH_PH_",s,null);
    this.dattribs_names.addValue(prefix+".g_iL_PLprev_",s,null);
    this.dattribs_names.addValue(prefix+".g_iL_PHprev_",s,null);
    this.dattribs_names.addValue(prefix+".g_iH_PLprev_",s,null);
    this.dattribs_names.addValue(prefix+".g_iH_PHprev_",s,null);

    n+=8;
    this.dattribs_names.addValue(prefix+".w_iL_PL_",s,null);
    this.dattribs_names.addValue(prefix+".w_iL_PH_",s,null);
    this.dattribs_names.addValue(prefix+".in_iL_PL_",s,null);
    this.dattribs_names.addValue(prefix+".in_iL_PH_",s,null);
    this.dattribs_names.addValue(prefix+".w_iH_PL_",s,null);
    this.dattribs_names.addValue(prefix+".w_iH_PH_",s,null);
    this.dattribs_names.addValue(prefix+".in_iH_PL_",s,null);
    this.dattribs_names.addValue(prefix+".in_iH_PH_",s,null);

    n+=12;
    this.dattribs_names.addValue(prefix+".g_iL_PL_",s,null);
    this.dattribs_names.addValue(prefix+".g_iL_PH_",s,null);
    this.dattribs_names.addValue(prefix+".g_iH_PL_",s,null);
    this.dattribs_names.addValue(prefix+".g_iH_PH_",s,null);
    this.dattribs_names.addValue(prefix+".go_iL_PL_",s,null);
    this.dattribs_names.addValue(prefix+".go_iL_PH_",s,null);
    this.dattribs_names.addValue(prefix+".go_iH_PL_",s,null);
    this.dattribs_names.addValue(prefix+".go_iH_PH_",s,null);
    this.dattribs_names.addValue(prefix+".g0_iL_PL_",s,null);
    this.dattribs_names.addValue(prefix+".g0_iL_PH_",s,null);
    this.dattribs_names.addValue(prefix+".g0_iH_PL_",s,null);
    this.dattribs_names.addValue(prefix+".g0_iH_PH_",s,null);


    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private <T> int printEkQVMWProjValues(final EkQVM<T>.WProj x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.pmu_iL);
    this.dattribs.addValue(x.pmu_iH);
    this.dattribs.addValue(x.P_iL_PL);
    this.dattribs.addValue(x.P_iL_PH);
    this.dattribs.addValue(x.P_iH_PL);
    this.dattribs.addValue(x.P_iH_PH);

    n+=8;
    this.dattribs.addValue(x.I_iL_PL);
    this.dattribs.addValue(x.I_iL_PH);
    this.dattribs.addValue(x.I_iH_PL);
    this.dattribs.addValue(x.I_iH_PH);
    this.dattribs.addValue(x.g_iL_PLprev);
    this.dattribs.addValue(x.g_iL_PHprev);
    this.dattribs.addValue(x.g_iH_PLprev);
    this.dattribs.addValue(x.g_iH_PHprev);

    n+=8;
    this.dattribs.addValue(x.w_iL_PL);
    this.dattribs.addValue(x.w_iL_PH);
    this.dattribs.addValue(x.in_iL_PL);
    this.dattribs.addValue(x.in_iL_PH);
    this.dattribs.addValue(x.w_iH_PL);
    this.dattribs.addValue(x.w_iH_PH);
    this.dattribs.addValue(x.in_iH_PL);
    this.dattribs.addValue(x.in_iH_PH);

    n+=12;
    this.dattribs.addValue(x.g_iL_PL);
    this.dattribs.addValue(x.g_iL_PH);
    this.dattribs.addValue(x.g_iH_PL);
    this.dattribs.addValue(x.g_iH_PH);
    this.dattribs.addValue(x.go_iL_PL);
    this.dattribs.addValue(x.go_iL_PH);
    this.dattribs.addValue(x.go_iH_PL);
    this.dattribs.addValue(x.go_iH_PH);
    this.dattribs.addValue(x.g0_iL_PL);
    this.dattribs.addValue(x.g0_iL_PH);
    this.dattribs.addValue(x.g0_iH_PL);
    this.dattribs.addValue(x.g0_iH_PH);

    n+=EVXData_Values(x.I);

    return n;
}



private int printEkQVMVarItPNames(final String prefix,final String s)
{
    int n=0;

    n+=12;
    this.dattribs_names.addValue(prefix+".Pt_",s,null);
    this.dattribs_names.addValue(prefix+".Pv_",s,null);
    this.dattribs_names.addValue(prefix+".I_PL_",s,null);
    this.dattribs_names.addValue(prefix+".varI_PL_",s,null);
    this.dattribs_names.addValue(prefix+".I_PH_",s,null);
    this.dattribs_names.addValue(prefix+".varI_PH_",s,null);
    this.dattribs_names.addValue(prefix+".VT_PL_",s,null);
    this.dattribs_names.addValue(prefix+".varVT_PL_",s,null);
    this.dattribs_names.addValue(prefix+".VT_PH_",s,null);
    this.dattribs_names.addValue(prefix+".varVT_PH_",s,null);
    this.dattribs_names.addValue(prefix+".varVP_",s,null);
    this.dattribs_names.addValue(prefix+".avarVP_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".PT0_L_",s,null);
    this.dattribs_names.addValue(prefix+".PV0_L_",s,null);
    this.dattribs_names.addValue(prefix+".PT0_H_",s,null);
    this.dattribs_names.addValue(prefix+".PV0_H_",s,null);

    n+=EVXData_Names(prefix+".P1t",s);

    return n;
}

private <T> int printEkQVMVarItPValues(final EkQVM<T>.VarItP x)
{
    int n=0;

    n+=12;
    this.dattribs.addValue(x.Pt);
    this.dattribs.addValue(x.Pv);
    this.dattribs.addValue(x.I_PL);
    this.dattribs.addValue(x.varI_PL);
    this.dattribs.addValue(x.I_PH);
    this.dattribs.addValue(x.varI_PH);
    this.dattribs.addValue(x.VT_PL);
    this.dattribs.addValue(x.varVT_PL);
    this.dattribs.addValue(x.VT_PH);
    this.dattribs.addValue(x.varVT_PH);
    this.dattribs.addValue(x.varVP);
    this.dattribs.addValue(x.avarVP);

    n+=4;
    this.dattribs.addValue(x.PT0_L);
    this.dattribs.addValue(x.PV0_L);
    this.dattribs.addValue(x.PT0_H);
    this.dattribs.addValue(x.PV0_H);

    n+=EVXData_Values(x.P1t);

    return n;
}




private int printEkQVMPNLvarNames(final String prefix,final String s)
{
    int n=0;

    n+=3;
    this.dattribs_names.addValue(prefix+".dpnl_",s,null);
    this.dattribs_names.addValue(prefix+".sd0_",s,null);
    this.dattribs_names.addValue(prefix+".ind_lev_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private <T> int printEkQVMPNLvarValues(final EkQVM<T>.PNLvar x)
{
    int n=0;

    n+=3;
    this.dattribs.addValue(x.dpnl);
    this.dattribs.addValue(x.sd0);
    this.dattribs.addValue(x.ind_lev);

    n+=EVXData_Values(x.I);

    return n;
}


private int printEkQVMUnrelPNLvarNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".Il_",s,null);
    this.dattribs_names.addValue(prefix+".Isell_",s,null);
    this.dattribs_names.addValue(prefix+".Ibuy_",s,null);
    this.dattribs_names.addValue(prefix+".pIsell_",s,null);
    this.dattribs_names.addValue(prefix+".pIbuy_",s,null);
    this.dattribs_names.addValue(prefix+".PNL_",s,null);
    this.dattribs_names.addValue(prefix+".PNLl_",s,null);
    this.dattribs_names.addValue(prefix+".dI_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private <T> int printEkQVMUnrelPNLvarValues(final EkQVM<T>.UnrelPNLvar x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.Il);
    this.dattribs.addValue(x.Isell);
    this.dattribs.addValue(x.Ibuy);
    this.dattribs.addValue(x.pIsell);
    this.dattribs.addValue(x.pIbuy);
    this.dattribs.addValue(x.PNL);
    this.dattribs.addValue(x.PNLl);
    this.dattribs.addValue(x.dI);

    n+=EVXData_Values(x.I);

    return n;
}




private int printEkQVMPNLGvarNames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".ptd_",s,null);
    this.dattribs_names.addValue(prefix+".ptdo_",s,null);
    this.dattribs_names.addValue(prefix+".pt_",s,null);
    this.dattribs_names.addValue(prefix+".pto_",s,null);
    this.dattribs_names.addValue(prefix+".pnlt_",s,null);
    this.dattribs_names.addValue(prefix+".pi_",s,null);
    this.dattribs_names.addValue(prefix+".pio_",s,null);
    this.dattribs_names.addValue(prefix+".pnli_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private <T> int printEkQVMPNLGvarValues(final EkQVM<T>.PNLGvar x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.ptd);
    this.dattribs.addValue(x.ptdo);
    this.dattribs.addValue(x.pt);
    this.dattribs.addValue(x.pto);
    this.dattribs.addValue(x.pnlt);
    this.dattribs.addValue(x.pi);
    this.dattribs.addValue(x.pio);
    this.dattribs.addValue(x.pnli);

    n+=EVXData_Values(x.I);

    return n;
}


private int printEkQVMPNLMvarNames(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".pa_buy_L_",s,null);
    this.dattribs_names.addValue(prefix+".pa_buy_H_",s,null);
    this.dattribs_names.addValue(prefix+".pa_sell_L_",s,null);
    this.dattribs_names.addValue(prefix+".pa_sell_H_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private <T> int printEkQVMPNLMvarValues(final EkQVM<T>.PNLMvar x)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.pa_buy_L);
    this.dattribs.addValue(x.pa_buy_H);
    this.dattribs.addValue(x.pa_sell_L);
    this.dattribs.addValue(x.pa_sell_H);

    n+=EVXData_Values(x.I);

    return n;
}


private int printEkQVMPNLIvarNames(final String prefix,final String s)
{
    int n=0;

    n+=2;
    this.dattribs_names.addValue(prefix+".dvol_",s,null);
    this.dattribs_names.addValue(prefix+".varpdvol_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private <T> int printEkQVMPNLIvarValues(final EkQVM<T>.PNLIvar x)
{
    int n=0;

    n+=2;
    this.dattribs.addValue(x.dvol);
    this.dattribs.addValue(x.varpdvol);

    n+=EVXData_Values(x.I);

    return n;
}



private int printRVarIpsiNames(final String prefix,final String s)
{
    int n=0;

    n+=5;
    this.dattribs_names.addValue(prefix+".f_lev0_",s,null);
    this.dattribs_names.addValue(prefix+".I_lev0_",s,null);
    this.dattribs_names.addValue(prefix+".I_lev1_",s,null);
    this.dattribs_names.addValue(prefix+".dI_lev0_",s,null);
    this.dattribs_names.addValue(prefix+".Iq_lev0_",s,null);

    return n;
}

private int printRVarIpsiValues(final RVarIpsi x)
{
    int n=0;

    n+=5;
    this.dattribs.addValue(x.f_lev0);
    this.dattribs.addValue(x.I_lev0);
    this.dattribs.addValue(x.I_lev1);
    this.dattribs.addValue(x.dI_lev0);
    this.dattribs.addValue(x.Iq_lev0);

    return n;
}







private int printEkQVMW2ProjNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".w_psi_PL_",s,null);
    this.dattribs_names.addValue(prefix+".w_psi_PH_",s,null);
    this.dattribs_names.addValue(prefix+".w_psi0_PL_",s,null);
    this.dattribs_names.addValue(prefix+".w_psi0_PH_",s,null);
    this.dattribs_names.addValue(prefix+".w_sum_",s,null);
    this.dattribs_names.addValue(prefix+".w_diff2_",s,null);

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".PNL",s);

    return n;
}

private <T> int printEkQVMW2ProjValues(final EkQVM<T>.W2Proj x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.w_psi_PL);
    this.dattribs.addValue(x.w_psi_PH);
    this.dattribs.addValue(x.w_psi0_PL);
    this.dattribs.addValue(x.w_psi0_PH);
    this.dattribs.addValue(x.w_sum);
    this.dattribs.addValue(x.w_diff2);

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.PNL);

    return n;
}





private int printEkQVMVarPsiNames(final String prefix,final String s)
{
    int n=0;

    n+=5;
    this.dattribs_names.addValue(prefix+".P_psi_",s,null);
    this.dattribs_names.addValue(prefix+".P_vpsi_",s,null);
    this.dattribs_names.addValue(prefix+".dP_psi_",s,null);
    this.dattribs_names.addValue(prefix+".dP_vpsi_",s,null);
    this.dattribs_names.addValue(prefix+".PA_pp_",s,null);
    return n;
}

private int printEkQVMVarPsiValues(final EkQVM.VarPsi x)
{
    int n=0;


    n+=5;
    this.dattribs.addValue(x.P_psi);
    this.dattribs.addValue(x.P_vpsi);
    this.dattribs.addValue(x.dP_psi);
    this.dattribs.addValue(x.dP_vpsi);
    this.dattribs.addValue(x.PA_pp);

    return n;
}





private int printPsiAverPertrNames(final String prefix,final String s)
{
    int n=0;

    n+=11;

    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    this.dattribs_names.addValue(prefix+".mu_",s,null);
    this.dattribs_names.addValue(prefix+".Ipsi_",s,null);
    this.dattribs_names.addValue(prefix+".pertpsi_",s,null);
    this.dattribs_names.addValue(prefix+".pertpsiC_",s,null);
    this.dattribs_names.addValue(prefix+".pertp_",s,null);
    this.dattribs_names.addValue(prefix+".fact_",s,null);
    this.dattribs_names.addValue(prefix+".Pt_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_Pt_",s,null);
    this.dattribs_names.addValue(prefix+".Pi_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_Pi_",s,null);

    return n;
}

private int printPsiAverPertrValues(final PsiAverPertr x)
{
    int n=0;


    n+=11;

    this.dattribs.addValue(x.fXx);
    this.dattribs.addValue(x.mu);
    this.dattribs.addValue(x.Ipsi);
    this.dattribs.addValue(x.pertpsi);
    this.dattribs.addValue(x.pertpsiC);
    this.dattribs.addValue(x.pertp);
    this.dattribs.addValue(x.fact);
    this.dattribs.addValue(x.Pt);
    this.dattribs.addValue(x.sigma2_Pt);
    this.dattribs.addValue(x.Pi);
    this.dattribs.addValue(x.sigma2_Pi);

    return n;
}



private int printPNLVolatilityNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".D",s);

    n+=8;
    this.dattribs_names.addValue(prefix+".Pt_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".Pi_L_",s,null);
    this.dattribs_names.addValue(prefix+".dP_L_",s,null);

    this.dattribs_names.addValue(prefix+".Pt_H_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".Pi_H_",s,null);
    this.dattribs_names.addValue(prefix+".dP_H_",s,null);

    return n;
}

private int printPNLVolatilityValues(final EkQVM.PNLVolatility x)
{
    int n=0;

    n+=EVXData_Values(x.D);

    n+=8;
    this.dattribs.addValue(x.Pt_L);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.Pi_L);
    this.dattribs.addValue(x.dP_L);

    this.dattribs.addValue(x.Pt_H);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.Pi_H);
    this.dattribs.addValue(x.dP_H);

    return n;
}







private int printEkQVMWvarPsiNames(final String prefix,final String s)
{
    int n=0;

    n+=1;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);

    n+=EVXData_Names(prefix+".I",s);
    n+=printEkQVMVarPsiNames(prefix+".PL_H",s);
    n+=printEkQVMVarPsiNames(prefix+".PH_L",s);
    n+=printEkQVMVarPsiNames(prefix+".PL_dL0",s);
    n+=printEkQVMVarPsiNames(prefix+".PL_dL",s);

    return n;
}

private <T> int printEkQVMWvarPsiValues(final EkQVM<T>.WvarPsi x)
{
    int n=0;

    n+=1;
    this.dattribs.addValue(x.fXx);

    n+=EVXData_Values(x.I);
    n+=printEkQVMVarPsiValues(x.PL_H);
    n+=printEkQVMVarPsiValues(x.PH_L);
    n+=printEkQVMVarPsiValues(x.PL_dL0);
    n+=printEkQVMVarPsiValues(x.PL_dL);

    return n;
}




private int printEkQVMNames(final String prefix,final String s)
{
    int n=0;
    n+=printPsiAverPertrNames(prefix+".pertIT",s);
    n+=printPNLVolatilityNames(prefix+".pnlvol",s);
    n+=printPNLVolatilityNames(prefix+".pnlvolT",s);

    n+=1;
    this.dattribs_names.addValue(prefix+".getVolume_",s,null);
    n+=printEkQVMW2ProjNames(prefix+".wpnlL_sam",s);
    n+=printEkQVMW2ProjNames(prefix+".wpnlH_sam",s);
    n+=printEkQVMW2ProjNames(prefix+".wpnlL",s);
    n+=printEkQVMW2ProjNames(prefix+".wpnlH",s);
    n+=printEkQVMW2ProjNames(prefix+".wpnlVSL_sam",s);
    n+=printEkQVMW2ProjNames(prefix+".wpnlVSH_sam",s);
    n+=printEkQVMW2ProjNames(prefix+".wpnlvsL_sam",s);
    n+=printEkQVMW2ProjNames(prefix+".wpnlvsH_sam",s);

    n+=EVXData_Names(prefix+".IT",s);
    n+=TwoRootsDataNames(prefix+".pVT.trd",s);
    n+=TwoRootsDataNames(prefix+".pvt.trd",s);

    n+=printEkQVMUnrelPNLvarNames(prefix+".up",s);
    n+=printEkQVMUnrelPNLvarNames(prefix+".upH",s);
    n+=printEkQVMUnrelPNLvarNames(prefix+".upP",s);

    n+=printEkQVMWProjNames(prefix+".WPpvt",s);
    n+=printEkQVMPNLvarNames(prefix+".pvar_L",s);
    n+=printEkQVMPNLvarNames(prefix+".pvar_H",s);
    n+=printEkQVMPNLvarNames(prefix+".pivar_L",s);
    n+=printEkQVMPNLvarNames(prefix+".pivar_H",s);
    n+=printEkQVMPNLvarNames(prefix+".svar_L",s);
    n+=printEkQVMPNLvarNames(prefix+".svar_H",s);
    n+=printEkQVMPNLvarNames(prefix+".Svar_L",s);
    n+=printEkQVMPNLvarNames(prefix+".Svar_H",s);

    n+=printEkQVMPNLMvarNames(prefix+".it_p",s);
    n+=printEkQVMPNLMvarNames(prefix+".bs_p",s);

    n+=printEkQVMPNLGvarNames(prefix+".gvar",s);
    n+=printEkQVMPNLGvarNames(prefix+".gvar_sam",s);
    n+=printEkQVMPNLIvarNames(prefix+".vpivar_L",s);
    n+=printEkQVMPNLIvarNames(prefix+".vpivar_H",s);

    n+=printEkQVMWEpvtNames(prefix+".eapvaL",s);
    n+=printEkQVMWEpvtNames(prefix+".eapvaH",s);
    n+=printEkQVMWEpvtNames(prefix+".eapvt",s);
    n+=printEkQVMWEpvtNames(prefix+".eapvt_sam",s);
    n+=printEkQVMWEpvtNames(prefix+".eapvtH_sam",s);
    n+=printEkQVMWEpvtNames(prefix+".epvta",s);
    n+=printEkQVMWEpvtNames(prefix+".epvtA",s);
    n+=printEkQVMWEpvtNames(prefix+".epvt",s);
    n+=printEkQVMWEpvtNames(prefix+".epvt_sam",s);
    n+=printEkQVMWEpvtNames(prefix+".epvtH_sam",s);
    n+=printEkQVMWEpvtNames(prefix+".epvtaD",s);
    n+=printEkQVMWEpvtNames(prefix+".epvtD_sam",s);
    n+=printEkQVMWEpvtNames(prefix+".eRVSvt_sam",s);
    n+=printEkQVMWEpvtNames(prefix+".eVSvt_sam",s);
    n+=printEkQVMWEpvtNames(prefix+".evsvt_sam",s);
    n+=printEkQVMWEpvtNames(prefix+".epvtaDH",s);
    n+=printEkQVMWEpvtNames(prefix+".epvtDH_sam",s);

    n+=printEkQVMWvarPsiNames(prefix+".wv",s);
    n+=printEkQVMVarItPNames(prefix+".vitp",s);

    n+=printRVarIpsiNames(prefix+".rpL",s);
    n+=printRVarIpsiNames(prefix+".rpH",s);

    return n;
}
private <T> int printEkQVMValues(final EkQVM<T> x,final CalculationData c)
{
    int n=0;
    n+=printPsiAverPertrValues(x.pertIT);
    n+=printPNLVolatilityValues(x.pnlvol);
    n+=printPNLVolatilityValues(x.pnlvolT);

    n+=1;
    this.dattribs.addValue(x.getVolume());
    n+=printEkQVMW2ProjValues(x.wpnlL_sam);
    n+=printEkQVMW2ProjValues(x.wpnlH_sam);
    n+=printEkQVMW2ProjValues(x.wpnlL);
    n+=printEkQVMW2ProjValues(x.wpnlH);
    n+=printEkQVMW2ProjValues(x.wpnlVSL_sam);
    n+=printEkQVMW2ProjValues(x.wpnlVSH_sam);
    n+=printEkQVMW2ProjValues(x.wpnlvsL_sam);
    n+=printEkQVMW2ProjValues(x.wpnlvsH_sam);

    n+=EVXData_Values(x.IT);
    n+=TwoRootsDataValues(x.pVT.trd);
    n+=TwoRootsDataValues(x.pvt.trd);

    n+=printEkQVMUnrelPNLvarValues(x.up);
    n+=printEkQVMUnrelPNLvarValues(x.upH);
    n+=printEkQVMUnrelPNLvarValues(x.upP);

    n+=printEkQVMWProjValues(x.WPpvt);
    n+=printEkQVMPNLvarValues(x.pvar_L);
    n+=printEkQVMPNLvarValues(x.pvar_H);
    n+=printEkQVMPNLvarValues(x.pivar_L);
    n+=printEkQVMPNLvarValues(x.pivar_H);
    n+=printEkQVMPNLvarValues(x.svar_L);
    n+=printEkQVMPNLvarValues(x.svar_H);
    n+=printEkQVMPNLvarValues(x.Svar_L);
    n+=printEkQVMPNLvarValues(x.Svar_H);

    n+=printEkQVMPNLMvarValues(x.it_p);
    n+=printEkQVMPNLMvarValues(x.bs_p);

    n+=printEkQVMPNLGvarValues(x.gvar);
    n+=printEkQVMPNLGvarValues(x.gvar_sam);
    n+=printEkQVMPNLIvarValues(x.vpivar_L);
    n+=printEkQVMPNLIvarValues(x.vpivar_H);

    n+=printEkQVMWEpvtValues(x.eapvaL);
    n+=printEkQVMWEpvtValues(x.eapvaH);
    n+=printEkQVMWEpvtValues(x.eapvt);
    n+=printEkQVMWEpvtValues(x.eapvt_sam);
    n+=printEkQVMWEpvtValues(x.eapvtH_sam);
    n+=printEkQVMWEpvtValues(x.epvta);
    n+=printEkQVMWEpvtValues(x.epvtA);
    n+=printEkQVMWEpvtValues(x.epvt);
    n+=printEkQVMWEpvtValues(x.epvt_sam);
    n+=printEkQVMWEpvtValues(x.epvtH_sam);
    n+=printEkQVMWEpvtValues(x.epvtaD);
    n+=printEkQVMWEpvtValues(x.epvtD_sam);
    n+=printEkQVMWEpvtValues(x.eRVSvt_sam);
    n+=printEkQVMWEpvtValues(x.eVSvt_sam);
    n+=printEkQVMWEpvtValues(x.evsvt_sam);
    n+=printEkQVMWEpvtValues(x.epvtaDH);
    n+=printEkQVMWEpvtValues(x.epvtDH_sam);

    n+=printEkQVMWvarPsiValues(x.wv);
    n+=printEkQVMVarItPValues(x.vitp);

    n+=printRVarIpsiValues(x.rpL);
    n+=printRVarIpsiValues(x.rpH);

    return n;
}



private int printGGPDataNames(final String prefix,final String s)
{
    int n=0;
    n+=8;
    this.dattribs_names.addValue(prefix+".Pt_x_",s,null);
    this.dattribs_names.addValue(prefix+".Pt_y_",s,null);
    this.dattribs_names.addValue(prefix+".C_y_gt_x_",s,null);
    this.dattribs_names.addValue(prefix+".wC_y_gt_x_",s,null);
    this.dattribs_names.addValue(prefix+".C_y_lt_x_",s,null);
    this.dattribs_names.addValue(prefix+".wC_y_lt_x_",s,null);
    this.dattribs_names.addValue(prefix+".Cdiff_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_Cdiff_",s,null);

    return n;
}

private int printGGPDataValues(final IkQVM.GGPData x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.Pt_x);
    this.dattribs.addValue(x.Pt_y);
    this.dattribs.addValue(x.C_y_gt_x);
    this.dattribs.addValue(x.wC_y_gt_x);
    this.dattribs.addValue(x.C_y_lt_x);
    this.dattribs.addValue(x.wC_y_lt_x);
    this.dattribs.addValue(x.Cdiff);
    this.dattribs.addValue(x.sigma2_Cdiff);
    return n;
}


private int printIkQVMNames(final String prefix,final String s)
{
    int n=0;


    n+=printGGPDataNames(prefix+".g",s);
    n+=printGGPDataNames(prefix+".gP",s);
    n+=printGGPDataNames(prefix+".giPL",s);
    n+=printGGPDataNames(prefix+".giPH",s);

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".P1t",s);

    n+=2;
    this.dattribs_names.addValue(prefix+".I_P1t_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_P1t_H_",s,null);

    return n;
}
private <T> int printIkQVMValues(final IkQVM<T> x,final CalculationData c)
{
    int n=0;


    n+=printGGPDataValues(x.g);
    n+=printGGPDataValues(x.gP);
    n+=printGGPDataValues(x.giPL);
    n+=printGGPDataValues(x.giPH);

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.P1t);

    n+=2;
    this.dattribs.addValue(x.I_P1t_L);
    this.dattribs.addValue(x.I_P1t_H);

    return n;
}



private int printJkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".psi_dp_",s,null);
    this.dattribs_names.addValue(prefix+".psi_X_",s,null);
    this.dattribs_names.addValue(prefix+".fP0_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_fP0_",s,null);
    this.dattribs_names.addValue(prefix+".fP1_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_fP1_",s,null);
    this.dattribs_names.addValue(prefix+".fP2_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_fP2_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".P1t_Il_",s,null);
    this.dattribs_names.addValue(prefix+".P1t_Ih_",s,null);
    this.dattribs_names.addValue(prefix+".Imax_",s,null);
    this.dattribs_names.addValue(prefix+".Imax1_",s,null);
    this.dattribs_names.addValue(prefix+".P1t_PImax_",s,null);
    this.dattribs_names.addValue(prefix+".P1t_PImax1_",s,null);
    n+=EVXData_Names(prefix+".P1t",s);

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".P2t",s);

    return n;
}
private <T> int printJkQVMValues(final JkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.psi_dp);
    this.dattribs.addValue(x.psi_X);
    this.dattribs.addValue(x.fP0);
    this.dattribs.addValue(x.sigma2_fP0);
    this.dattribs.addValue(x.fP1);
    this.dattribs.addValue(x.sigma2_fP1);
    this.dattribs.addValue(x.fP2);
    this.dattribs.addValue(x.sigma2_fP2);

    n+=6;
    this.dattribs.addValue(x.P1t_Il);
    this.dattribs.addValue(x.P1t_Ih);
    this.dattribs.addValue(x.Imax);
    this.dattribs.addValue(x.Imax1);
    this.dattribs.addValue(x.P1t_PImax);
    this.dattribs.addValue(x.P1t_PImax1);
    n+=EVXData_Values(x.P1t);

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.P2t);

    return n;
}




private int printKkQVMPnLeNames(final String prefix,final String s)
{
    int n=0;

    n+=11;
    this.dattribs_names.addValue(prefix+".PpsiH_",s,null);
    this.dattribs_names.addValue(prefix+".pL_",s,null);
    this.dattribs_names.addValue(prefix+".pH_",s,null);
    this.dattribs_names.addValue(prefix+".iL_",s,null);
    this.dattribs_names.addValue(prefix+".iH_",s,null);
    this.dattribs_names.addValue(prefix+".w_pnlL_gt_pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".w_pnlL_lt_pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".w_pnlL_gt_I_",s,null);
    this.dattribs_names.addValue(prefix+".w_pnlL_lt_I_",s,null);
    this.dattribs_names.addValue(prefix+".w_pnlH_gt_I_",s,null);
    this.dattribs_names.addValue(prefix+".w_pnlH_lt_I_",s,null);


    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".PnL",s);

    return n;
}

private <T> int printKkQVMPnLeValues(final KkQVM<T>.PnLe x)
{
    int n=0;

    n+=11;
    this.dattribs.addValue(x.PpsiH);
    this.dattribs.addValue(x.pL);
    this.dattribs.addValue(x.pH);
    this.dattribs.addValue(x.iL);
    this.dattribs.addValue(x.iH);
    this.dattribs.addValue(x.w_pnlL_gt_pnlH);
    this.dattribs.addValue(x.w_pnlL_lt_pnlH);
    this.dattribs.addValue(x.w_pnlL_gt_I);
    this.dattribs.addValue(x.w_pnlL_lt_I);
    this.dattribs.addValue(x.w_pnlH_gt_I);
    this.dattribs.addValue(x.w_pnlH_lt_I);

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.PnL);

    return n;
}


private int printKkQVMWDistanceNames(final String prefix,final String s)
{
    int n=0;

    for(int j=0;j<8;j++)
    {
	n+=5;
	this.dattribs_names.addValue(prefix+".t_order_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".e_distancesorted_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".pi_distancesorted_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".I_distancesorted_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".sens_distancesorted_"+j+"_",s,null);
    }

    return n;
}

private <T> int printKkQVMWDistanceValues(final KkQVM<T>.WDistance x)
{
    int n=0;


    final int d=x.t_order.length;
    for(int j=0;j<8;j++)
    {
	n+=5;
	this.dattribs.addValue(j<d?x.t_order[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.e_distancesorted[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.pi_distancesorted[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.I_distancesorted[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.sens_distancesorted[j]:Double.NaN);
    }

    return n;
}



private int printKkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=printKkQVMWDistanceNames(prefix+".distanceP1t",s);
    n+=printKkQVMWDistanceNames(prefix+".distanceI",s);
    n+=printKkQVMPnLeNames(prefix+".pnl",s);

    n+=12;
    this.dattribs_names.addValue(prefix+".Paver_I_",s,null);
    this.dattribs_names.addValue(prefix+".I_P1t_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_P1t_H_",s,null);
    this.dattribs_names.addValue(prefix+".Imax_",s,null);
    this.dattribs_names.addValue(prefix+".P_Imax_",s,null);
    this.dattribs_names.addValue(prefix+".Imax1_",s,null);
    this.dattribs_names.addValue(prefix+".P_Imax1_",s,null);
    this.dattribs_names.addValue(prefix+".Imin_",s,null);
    this.dattribs_names.addValue(prefix+".P_Imin_",s,null);
    this.dattribs_names.addValue(prefix+".Imin1_",s,null);
    this.dattribs_names.addValue(prefix+".P_Imin1_",s,null);
    this.dattribs_names.addValue(prefix+".getDS_",s,null);


    n+=EVXData_Names(prefix+".pVt",s);
    n+=EVXData_Names(prefix+".pVV",s);
    n+=EVXData_Names(prefix+".P1t",s);
    n+=EVXData_Names(prefix+".I",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".I_P1t_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_P1t_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_PH_gt_PL_",s,null);
    this.dattribs_names.addValue(prefix+".w_PH_lt_PL_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".w_PL_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_PH_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_PL_IH_",s,null);
    this.dattribs_names.addValue(prefix+".w_PH_IH_",s,null);

    n+=14;
    this.dattribs_names.addValue(prefix+".w_pVtH_gt_pVtL_",s,null);
    this.dattribs_names.addValue(prefix+".w_pVtH_lt_pVtL_",s,null);
    this.dattribs_names.addValue(prefix+".pVt_PL_",s,null);
    this.dattribs_names.addValue(prefix+".pVt_PH_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_gt_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_lt_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_PH_gt_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_PH_lt_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_PL_gt_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_PL_lt_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_PH_gt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".w_PH_lt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".w_PL_gt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".w_PL_lt_IH_",s,null);

    n+=8;
    this.dattribs_names.addValue(prefix+".w_PH_IH_PL_",s,null);
    this.dattribs_names.addValue(prefix+".w_PL_IH_PH_",s,null);
    this.dattribs_names.addValue(prefix+".w_PH_PL_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_PL_PH_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_PH_PL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_PL_PH_",s,null);
    this.dattribs_names.addValue(prefix+".w_IHPL_ILPH_",s,null);
    this.dattribs_names.addValue(prefix+".w_IHPH_ILPL_",s,null);
    n+=5;
    this.dattribs_names.addValue(prefix+".w_PL_PH_PM_",s,null);
    this.dattribs_names.addValue(prefix+".w_PH_PL_PM_",s,null);
    this.dattribs_names.addValue(prefix+".w_PL_I_PH_",s,null);
    this.dattribs_names.addValue(prefix+".pi_PL_",s,null);
    this.dattribs_names.addValue(prefix+".pi_PH_",s,null);
    n+=4;
    this.dattribs_names.addValue(prefix+".w_IHPL_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_IHPL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IHPH_IL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_IHPH_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".sens_PL_pertI_",s,null);
    this.dattribs_names.addValue(prefix+".sens_PH_pertI_",s,null);
    this.dattribs_names.addValue(prefix+".sens_IL_pertPt_",s,null);
    this.dattribs_names.addValue(prefix+".sens_IH_pertPt_",s,null);
    this.dattribs_names.addValue(prefix+".sens_IL_pertPnL_",s,null);
    this.dattribs_names.addValue(prefix+".sens_IH_pertPnL_",s,null);

    n+=printRVarIpsiNames(prefix+".rpL",s);
    n+=printRVarIpsiNames(prefix+".rpH",s);
    n+=4;
    this.dattribs_names.addValue(prefix+".wrpL_J0_gt_J1_",s,null);
    this.dattribs_names.addValue(prefix+".wrpL_J0_lt_J1_",s,null);
    this.dattribs_names.addValue(prefix+".wrpH_J0_gt_J1_",s,null);
    this.dattribs_names.addValue(prefix+".wrpH_J0_lt_J1_",s,null);

    return n;
}
private <T> int printKkQVMValues(final KkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=printKkQVMWDistanceValues(x.distanceP1t);
    n+=printKkQVMWDistanceValues(x.distanceI);
    n+=printKkQVMPnLeValues(x.pnl);

    n+=12;
    this.dattribs.addValue(x.Paver_I);
    this.dattribs.addValue(x.I_P1t_L);
    this.dattribs.addValue(x.I_P1t_H);
    this.dattribs.addValue(x.Imax);
    this.dattribs.addValue(x.P_Imax);
    this.dattribs.addValue(x.Imax1);
    this.dattribs.addValue(x.P_Imax1);
    this.dattribs.addValue(x.Imin);
    this.dattribs.addValue(x.P_Imin);
    this.dattribs.addValue(x.Imin1);
    this.dattribs.addValue(x.P_Imin1);
    this.dattribs.addValue(x.getDS());


    n+=EVXData_Values(x.pVt);
    n+=EVXData_Values(x.pVV);
    n+=EVXData_Values(x.P1t);
    n+=EVXData_Values(x.I);

    n+=4;
    this.dattribs.addValue(x.I_P1t_L);
    this.dattribs.addValue(x.I_P1t_H);
    this.dattribs.addValue(x.w_PH_gt_PL);
    this.dattribs.addValue(x.w_PH_lt_PL);

    n+=4;
    this.dattribs.addValue(x.w_PL_IL);
    this.dattribs.addValue(x.w_PH_IL);
    this.dattribs.addValue(x.w_PL_IH);
    this.dattribs.addValue(x.w_PH_IH);

    n+=14;
    this.dattribs.addValue(x.w_pVtH_gt_pVtL);
    this.dattribs.addValue(x.w_pVtH_lt_pVtL);
    this.dattribs.addValue(x.pVt_PL);
    this.dattribs.addValue(x.pVt_PH);
    this.dattribs.addValue(x.w_IH_gt_IL);
    this.dattribs.addValue(x.w_IH_lt_IL);
    this.dattribs.addValue(x.w_PH_gt_IL);
    this.dattribs.addValue(x.w_PH_lt_IL);
    this.dattribs.addValue(x.w_PL_gt_IL);
    this.dattribs.addValue(x.w_PL_lt_IL);
    this.dattribs.addValue(x.w_PH_gt_IH);
    this.dattribs.addValue(x.w_PH_lt_IH);
    this.dattribs.addValue(x.w_PL_gt_IH);
    this.dattribs.addValue(x.w_PL_lt_IH);

    n+=8;
    this.dattribs.addValue(x.w_PH_IH_PL);
    this.dattribs.addValue(x.w_PL_IH_PH);
    this.dattribs.addValue(x.w_PH_PL_IL);
    this.dattribs.addValue(x.w_PL_PH_IL);
    this.dattribs.addValue(x.w_IL_PH_PL);
    this.dattribs.addValue(x.w_IL_PL_PH);
    this.dattribs.addValue(x.w_IHPL_ILPH);
    this.dattribs.addValue(x.w_IHPH_ILPL);
    n+=5;
    this.dattribs.addValue(x.w_PL_PH_PM);
    this.dattribs.addValue(x.w_PH_PL_PM);
    this.dattribs.addValue(x.w_PL_I_PH);
    this.dattribs.addValue(x.pi_PL);
    this.dattribs.addValue(x.pi_PH);

    n+=4;
    this.dattribs.addValue(x.w_IHPL_IL);
    this.dattribs.addValue(x.w_IL_IHPL);
    this.dattribs.addValue(x.w_IHPH_IL);
    this.dattribs.addValue(x.w_IL_IHPH);

    n+=6;
    this.dattribs.addValue(x.sens_PL_pertI);
    this.dattribs.addValue(x.sens_PH_pertI);
    this.dattribs.addValue(x.sens_IL_pertPt);
    this.dattribs.addValue(x.sens_IH_pertPt);
    this.dattribs.addValue(x.sens_IL_pertPnL);
    this.dattribs.addValue(x.sens_IH_pertPnL);

    n+=printRVarIpsiValues(x.rpL);
    n+=printRVarIpsiValues(x.rpH);
    n+=4;
    this.dattribs.addValue(x.wrpL_J0_gt_J1);
    this.dattribs.addValue(x.wrpL_J0_lt_J1);
    this.dattribs.addValue(x.wrpH_J0_gt_J1);
    this.dattribs.addValue(x.wrpH_J0_lt_J1);

    return n;
}


private int printLkQVMAveragePEDataNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".dpA_LT_",s,null);
    this.dattribs_names.addValue(prefix+".wA_LT_",s,null);
    this.dattribs_names.addValue(prefix+".dpA_GT_",s,null);
    this.dattribs_names.addValue(prefix+".wA_GT_",s,null);
    this.dattribs_names.addValue(prefix+".dpPsi_LT_",s,null);
    this.dattribs_names.addValue(prefix+".dpPsi_GT_",s,null);

    return n;
}

private <T> int printLkQVMAveragePEDataValues(final LkQVM<T>.AveragePEData x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.dpA_LT);
    this.dattribs.addValue(x.wA_LT);
    this.dattribs.addValue(x.dpA_GT);
    this.dattribs.addValue(x.wA_GT);
    this.dattribs.addValue(x.dpPsi_LT);
    this.dattribs.addValue(x.dpPsi_GT);

    return n;
}


private int printLkQVMPDataNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".J",s);

    n+=2;
    this.dattribs_names.addValue(prefix+".w_PL_GT_PH_",s,null);
    this.dattribs_names.addValue(prefix+".w_PH_GT_PL_",s,null);

    return n;
}

private <T> int printLkQVMPDataValues(final LkQVM<T>.PData x)
{
    int n=0;

    n+=EVXData_Values(x.J);

    n+=2;
    this.dattribs.addValue(x.w_PL_GT_PH);
    this.dattribs.addValue(x.w_PH_GT_PL);

    return n;
}


private int printLkQVMXRDataNames(final String prefix,final String s)
{
    int n=0;
    n+=printLkQVMPnLDataNames(prefix+".pnl",s);
    n+=printLkQVMPDataNames(prefix+".bp",s);
    n+=printLkQVMPDataNames(prefix+".fp",s);
    n+=1;
    this.dattribs_names.addValue(prefix+".fXx_",s,null);
    return n;
}

private <T> int printLkQVMXRDataValues(final LkQVM<T>.XRData x)
{
    int n=0;

    n+=printLkQVMPnLDataValues(x.pnl);
    n+=printLkQVMPDataValues(x.bp);
    n+=printLkQVMPDataValues(x.fp);
    n+=1;
    this.dattribs.addValue(x.fXx);
    return n;
}




private int printLkQVMPnLDataNames(final String prefix,final String s)
{
    int n=0;

    n+=13;
    this.dattribs_names.addValue(prefix+".pPsiS_",s,null);
    this.dattribs_names.addValue(prefix+".gtPnL_",s,null);
    this.dattribs_names.addValue(prefix+".ltPnL_",s,null);
    this.dattribs_names.addValue(prefix+".gtI_",s,null);
    this.dattribs_names.addValue(prefix+".gtpI_",s,null);
    this.dattribs_names.addValue(prefix+".gtPi_",s,null);
    this.dattribs_names.addValue(prefix+".gtPt_",s,null);
    this.dattribs_names.addValue(prefix+".gtDp_",s,null);
    this.dattribs_names.addValue(prefix+".ltI_",s,null);
    this.dattribs_names.addValue(prefix+".ltpI_",s,null);
    this.dattribs_names.addValue(prefix+".ltPi_",s,null);
    this.dattribs_names.addValue(prefix+".ltPt_",s,null);
    this.dattribs_names.addValue(prefix+".ltDp_",s,null);

    return n;
}

private <T> int printLkQVMPnLDataValues(final LkQVM<T>.PnLData x)
{
    int n=0;

    n+=13;
    this.dattribs.addValue(x.pPsiS);
    this.dattribs.addValue(x.gtPnL);
    this.dattribs.addValue(x.ltPnL);
    this.dattribs.addValue(x.gtI);
    this.dattribs.addValue(x.gtpI);
    this.dattribs.addValue(x.gtPi);
    this.dattribs.addValue(x.gtPt);
    this.dattribs.addValue(x.gtDp);
    this.dattribs.addValue(x.ltI);
    this.dattribs.addValue(x.ltpI);
    this.dattribs.addValue(x.ltPi);
    this.dattribs.addValue(x.ltPt);
    this.dattribs.addValue(x.ltDp);

    return n;
}


private int printLkQVMPEDataNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".J",s);

    for(int j=0;j<8;j++)
    {
	n+=10;
	this.dattribs_names.addValue(prefix+".sensft_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".senspsi_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".sensf_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".varf_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".prob_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".f_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".fprob_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".fi_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".i_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".iprob_"+j+"_",s,null);
    }
    return n;
}

private <T> int printLkQVMPEDataValues(final LkQVM<T>.PEData x)
{
    int n=0;
    n+=EVXData_Values(x.J);

    final int d=x.J.neq;
    for(int j=0;j<8;j++)
    {
	n+=10;
	this.dattribs.addValue(j<d?x.sensft[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.senspsi[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.sensf[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.varf[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.prob[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.f[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.fprob[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.fi[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.i[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.iprob[j]:Double.NaN);
    }
    return n;
}



private int printLkQVMProbVarDataNames(final String prefix,final String s)
{
    int n=0;
    n+=1;
    this.dattribs_names.addValue(prefix+".d1G_",s,null);
    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".J",s);

    for(int j=0;j<8;j++)
    {
	n+=3;
	this.dattribs_names.addValue(prefix+".probpert_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".i_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".fi_"+j+"_",s,null);
    }
    return n;
}

private <T> int printLkQVMProbVarDataValues(final LkQVM<T>.ProbVarData x)
{
    int n=0;
    n+=1;
    this.dattribs.addValue(x.d1G);
    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.J);

    final int d=x.J.neq;
    for(int j=0;j<8;j++)
    {
	n+=3;
	this.dattribs.addValue(j<d?x.probpert[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.i[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.fi[j]:Double.NaN);
    }
    return n;
}


private int printLkQVMProbIVarDataNames(final String prefix,final String s)
{
    int n=0;
    n+=1;
    this.dattribs_names.addValue(prefix+".d1G_",s,null);
    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".J",s);

    for(int j=0;j<8;j++)
    {
	n+=4;
	this.dattribs_names.addValue(prefix+".probpert_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".i_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".mu_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".fmu_"+j+"_",s,null);
    }
    return n;
}

private <T> int printLkQVMProbIVarDataValues(final LkQVM<T>.ProbIVarData x)
{
    int n=0;
    n+=1;
    this.dattribs.addValue(x.d1G);
    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.J);

    final int d=x.J.neq;
    for(int j=0;j<8;j++)
    {
	n+=4;
	this.dattribs.addValue(j<d?x.probpert[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.i[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.mu[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.fmu[j]:Double.NaN);
    }
    return n;
}




private int printLkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".P1t",s);
    n+=printLkQVMProbIVarDataNames(prefix+".itpnl_IL",s);
    n+=printLkQVMProbIVarDataNames(prefix+".itpnl_IH",s);
    n+=printLkQVMProbVarDataNames(prefix+".itpi_IL",s);
    n+=printLkQVMProbVarDataNames(prefix+".itpi_IH",s);
    n+=printLkQVMProbVarDataNames(prefix+".itpt_IL",s);
    n+=printLkQVMProbVarDataNames(prefix+".itpt_IH",s);
    n+=printLkQVMPEDataNames(prefix+".pedt_IL",s);
    n+=printLkQVMPEDataNames(prefix+".pedt_IH",s);
    n+=printLkQVMPEDataNames(prefix+".pedi_IL",s);
    n+=printLkQVMPEDataNames(prefix+".pedi_IH",s);
    n+=printLkQVMPDataNames(prefix+".bpiL",s);
    n+=printLkQVMPDataNames(prefix+".fpiL",s);
    n+=printLkQVMPDataNames(prefix+".bpiH",s);
    n+=printLkQVMPDataNames(prefix+".fpiH",s);
    n+=printLkQVMPDataNames(prefix+".bipL",s);
    n+=printLkQVMPDataNames(prefix+".fipL",s);
    n+=printLkQVMPDataNames(prefix+".bipH",s);
    n+=printLkQVMPDataNames(prefix+".fipH",s);

    n+=printLkQVMPnLDataNames(prefix+".pnlIL",s);
    n+=printLkQVMPnLDataNames(prefix+".pnlIH",s);

    n+=printLkQVMXRDataNames(prefix+".xIL",s);
    n+=printLkQVMXRDataNames(prefix+".xIH",s);
    n+=printLkQVMXRDataNames(prefix+".xNIL",s);
    n+=printLkQVMXRDataNames(prefix+".xNIH",s);

    n+=printLkQVMAveragePEDataNames(prefix+".dpt_IL",s);
    n+=printLkQVMAveragePEDataNames(prefix+".dpt_IH",s);
    n+=printLkQVMAveragePEDataNames(prefix+".vs_IL",s);
    n+=printLkQVMAveragePEDataNames(prefix+".vs_IH",s);
    n+=printLkQVMAveragePEDataNames(prefix+".Svs_IL",s);
    n+=printLkQVMAveragePEDataNames(prefix+".Svs_IH",s);

    return n;
}
private <T> int printLkQVMValues(final LkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.P1t);
    n+=printLkQVMProbIVarDataValues(x.itpnl_IL);
    n+=printLkQVMProbIVarDataValues(x.itpnl_IH);
    n+=printLkQVMProbVarDataValues(x.itpi_IL);
    n+=printLkQVMProbVarDataValues(x.itpi_IH);
    n+=printLkQVMProbVarDataValues(x.itpt_IL);
    n+=printLkQVMProbVarDataValues(x.itpt_IH);
    n+=printLkQVMPEDataValues(x.pedt_IL);
    n+=printLkQVMPEDataValues(x.pedt_IH);
    n+=printLkQVMPEDataValues(x.pedi_IL);
    n+=printLkQVMPEDataValues(x.pedi_IH);
    n+=printLkQVMPDataValues(x.bpiL);
    n+=printLkQVMPDataValues(x.fpiL);
    n+=printLkQVMPDataValues(x.bpiH);
    n+=printLkQVMPDataValues(x.fpiH);
    n+=printLkQVMPDataValues(x.bipL);
    n+=printLkQVMPDataValues(x.fipL);
    n+=printLkQVMPDataValues(x.bipH);
    n+=printLkQVMPDataValues(x.fipH);

    n+=printLkQVMPnLDataValues(x.pnlIL);
    n+=printLkQVMPnLDataValues(x.pnlIH);

    n+=printLkQVMXRDataValues(x.xIL);
    n+=printLkQVMXRDataValues(x.xIH);
    n+=printLkQVMXRDataValues(x.xNIL);
    n+=printLkQVMXRDataValues(x.xNIH);

    n+=printLkQVMAveragePEDataValues(x.dpt_IL);
    n+=printLkQVMAveragePEDataValues(x.dpt_IH);
    n+=printLkQVMAveragePEDataValues(x.vs_IL);
    n+=printLkQVMAveragePEDataValues(x.vs_IH);
    n+=printLkQVMAveragePEDataValues(x.Svs_IL);
    n+=printLkQVMAveragePEDataValues(x.Svs_IH);

    return n;
}




private int printMkQVMMDistanceNames(final String prefix,final String s)
{
    int n=0;

    for(int j=0;j<8;j++)
    {
	n+=6;
	this.dattribs_names.addValue(prefix+".t_order_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".I_distancesorted_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".p_distancesorted_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".varPt_distancesorted_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".varPi_distancesorted_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".dp_distancesorted_"+j+"_",s,null);
    }

    return n;
}

private <T> int printMkQVMMDistanceValues(final MkQVM<T>.MDistance x)
{
    int n=0;

    final int d=x.t_order.length;
    for(int j=0;j<8;j++)
    {
	n+=6;
	this.dattribs.addValue(j<d?x.t_order[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.I_distancesorted[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.p_distancesorted[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.varPt_distancesorted[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.varPi_distancesorted[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.dp_distancesorted[j]:Double.NaN);
    }

    return n;
}



private int printMkQVMPertPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".J",s);

    for(int j=0;j<8;j++)
    {
	n+=2;
	this.dattribs_names.addValue(prefix+".fpert_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".senspert_"+j+"_",s,null);
    }

    return n;
}

private <T> int printMkQVMPertPsiValues(final MkQVM<T>.PertPsi x)
{
    int n=0;
    n+=EVXData_Values(x.J);

    final int d=x.J.neq;
    for(int j=0;j<8;j++)
    {
	n+=2;
	this.dattribs.addValue(j<d?x.fpert[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.senspert[j]:Double.NaN);
    }

    return n;
}

private int printMkQVMFPertPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".J",s);

    for(int j=0;j<8;j++)
    {
	n+=5;
	this.dattribs_names.addValue(prefix+".senspsi_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".f_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".varf_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".varFn_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".var2j_"+j+"_",s,null);
    }

    return n;
}

private <T> int printMkQVMFPertPsiValues(final MkQVM<T>.FPertPsi x)
{
    int n=0;
    n+=EVXData_Values(x.J);

    final int d=x.J.neq;
    for(int j=0;j<8;j++)
    {
	n+=5;
	this.dattribs.addValue(j<d?x.senspsi[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.f[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.varf[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.varFn[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.var2j[j]:Double.NaN);
    }

    return n;
}



private int printMkQVMLPertPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".J",s);

    for(int j=0;j<8;j++)
    {
	n+=5;
	this.dattribs_names.addValue(prefix+".X_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".Xf_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".senspsi_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".f_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".varf_"+j+"_",s,null);
    }

    return n;
}

private <T> int printMkQVMLPertPsiValues(final MkQVM<T>.LPertPsi x)
{
    int n=0;
    n+=EVXData_Values(x.J);

    final int d=x.J.neq;
    for(int j=0;j<8;j++)
    {
	n+=5;
	this.dattribs.addValue(j<d?x.X[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.Xf[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.senspsi[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.f[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.varf[j]:Double.NaN);
    }

    return n;
}


private int printMkQVMRadauPertNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".J",s);
    n+=1;
    this.dattribs_names.addValue(prefix+".w0_",s,null);

    for(int j=0;j<8;j++)
    {
	n+=6;
	this.dattribs_names.addValue(prefix+".X_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".sigma2_X_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".f_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".sigma2_f_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".Xf_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".corr_"+j+"_",s,null);
    }
    return n;
}

private <T> int printMkQVMRadauPertValues(final MkQVM<T>.RadauPert x)
{
    int n=0;
    n+=EVXData_Values(x.J);
    n+=1;
    this.dattribs.addValue(x.w0);

    final int d=x.J.neq;
    for(int j=0;j<8;j++)
    {
	n+=6;
	this.dattribs.addValue(j<d?x.X[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.sigma2_X[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.f[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.sigma2_f[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.Xf[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.corr[j]:Double.NaN);
    }

    return n;
}




private int printMkQVMX0PsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".J",s);

    n+=14;
    this.dattribs_names.addValue(prefix+".mAL_",s,null);
    this.dattribs_names.addValue(prefix+".mAH_",s,null);
    this.dattribs_names.addValue(prefix+".mA0_",s,null);
    this.dattribs_names.addValue(prefix+".mA_",s,null);
    this.dattribs_names.addValue(prefix+".mA_rho_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_f_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_I_",s,null);
    this.dattribs_names.addValue(prefix+".mAI_",s,null);
    this.dattribs_names.addValue(prefix+".mAI_rho_",s,null);
    this.dattribs_names.addValue(prefix+".mAId_rho_",s,null);
    this.dattribs_names.addValue(prefix+".mL0_f_",s,null);
    this.dattribs_names.addValue(prefix+".mL0_I_",s,null);
    this.dattribs_names.addValue(prefix+".mH0_f_",s,null);
    this.dattribs_names.addValue(prefix+".mH0_I_",s,null);

    n+=12;
    this.dattribs_names.addValue(prefix+".sens_f_",s,null);
    this.dattribs_names.addValue(prefix+".sensI_f_",s,null);
    this.dattribs_names.addValue(prefix+".sensL_f_",s,null);
    this.dattribs_names.addValue(prefix+".sensH_f_",s,null);
    this.dattribs_names.addValue(prefix+".fI_L_",s,null);
    this.dattribs_names.addValue(prefix+".sens_fJ_L_",s,null);
    this.dattribs_names.addValue(prefix+".fJ_L_",s,null);
    this.dattribs_names.addValue(prefix+".w_L_",s,null);
    this.dattribs_names.addValue(prefix+".fI_H_",s,null);
    this.dattribs_names.addValue(prefix+".sens_fJ_H_",s,null);
    this.dattribs_names.addValue(prefix+".fJ_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_H_",s,null);

    return n;
}

private <T> int printMkQVMX0PsiValues(final MkQVM<T>.X0Psi x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.J);

    n+=14;
    this.dattribs.addValue(x.mAL);
    this.dattribs.addValue(x.mAH);
    this.dattribs.addValue(x.mA0);
    this.dattribs.addValue(x.mA);
    this.dattribs.addValue(x.mA_rho);
    this.dattribs.addValue(x.sigma2_f);
    this.dattribs.addValue(x.sigma2_I);
    this.dattribs.addValue(x.mAI);
    this.dattribs.addValue(x.mAI_rho);
    this.dattribs.addValue(x.mAId_rho);
    this.dattribs.addValue(x.mL0_f);
    this.dattribs.addValue(x.mL0_I);
    this.dattribs.addValue(x.mH0_f);
    this.dattribs.addValue(x.mH0_I);

    n+=12;
    this.dattribs.addValue(x.sens_f);
    this.dattribs.addValue(x.sensI_f);
    this.dattribs.addValue(x.sensL_f);
    this.dattribs.addValue(x.sensH_f);
    this.dattribs.addValue(x.fI_L);
    this.dattribs.addValue(x.sens_fJ_L);
    this.dattribs.addValue(x.fJ_L);
    this.dattribs.addValue(x.w_L);
    this.dattribs.addValue(x.fI_H);
    this.dattribs.addValue(x.sens_fJ_H);
    this.dattribs.addValue(x.fJ_H);
    this.dattribs.addValue(x.w_H);

    return n;
}



private int printMkQVMPIPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".P",s);

    n+=7;
    this.dattribs_names.addValue(prefix+".mAE_",s,null);
    this.dattribs_names.addValue(prefix+".mA0_",s,null);
    this.dattribs_names.addValue(prefix+".mAd0_",s,null);
    this.dattribs_names.addValue(prefix+".mL_",s,null);
    this.dattribs_names.addValue(prefix+".mH_",s,null);
    this.dattribs_names.addValue(prefix+".mA_",s,null);
    this.dattribs_names.addValue(prefix+".mA_rho_",s,null);

    return n;
}

private <T> int printMkQVMPIPsiValues(final MkQVM<T>.PIPsi x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.P);

    n+=7;
    this.dattribs.addValue(x.mAE);
    this.dattribs.addValue(x.mA0);
    this.dattribs.addValue(x.mAd0);
    this.dattribs.addValue(x.mL);
    this.dattribs.addValue(x.mH);
    this.dattribs.addValue(x.mA);
    this.dattribs.addValue(x.mA_rho);

    return n;
}


private int printMkQVMDPIPsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);

    n+=printACorrNames(prefix+".SPcorr_f_i",s);
    n+=printACorrNames(prefix+".SPcorr_f_di",s);
    n+=printACorrNames(prefix+".SPcorr_f_odi",s);
    n+=printACorrNames(prefix+".SPrcorr_f_i",s);
    n+=printACorrNames(prefix+".SPrcorr_f_di",s);
    n+=printACorrNames(prefix+".SPrrcorr_f_i",s);
    n+=printACorrNames(prefix+".SPrrcorr_f_di",s);


    n+=printACorrNames(prefix+".ccorr_f_i",s);
    n+=printACorrNames(prefix+".ccorr_f_di",s);
    n+=printACorrNames(prefix+".ccorr_i_di",s);
    n+=printACorrNames(prefix+".ccorr_t_i",s);
    n+=printACorrNames(prefix+".ccorr_t_di",s);
    n+=printACorrNames(prefix+".ccorr_f_t",s);

    n+=printACorrNames(prefix+".bcorr_f_i",s);
    n+=printACorrNames(prefix+".bcorr_f_di",s);

    n+=printACorrNames(prefix+".acorr_f_i",s);
    n+=printACorrNames(prefix+".acorr_f_di",s);

    n+=printACorrNames(prefix+".Acorr_f_i",s);
    n+=printACorrNames(prefix+".Acorr_f_di",s);
    n+=printACorrNames(prefix+".Acorr_t_di",s);

    n+=15;
    this.dattribs_names.addValue(prefix+".mA_ospd_",s,null);
    this.dattribs_names.addValue(prefix+".mA_ospd_rho_",s,null);
    this.dattribs_names.addValue(prefix+".I0d_",s,null);
    this.dattribs_names.addValue(prefix+".mAverd_",s,null);
    this.dattribs_names.addValue(prefix+".mAverd_rho_",s,null);
    this.dattribs_names.addValue(prefix+".mAver_oDid_",s,null);
    this.dattribs_names.addValue(prefix+".mAver_oDi_",s,null);
    this.dattribs_names.addValue(prefix+".mAver_oDiof_",s,null);
    this.dattribs_names.addValue(prefix+".mAver_oDi0_",s,null);
    this.dattribs_names.addValue(prefix+".mAver_aa_",s,null);
    this.dattribs_names.addValue(prefix+".mAver_a_",s,null);
    this.dattribs_names.addValue(prefix+".mAver_",s,null);
    this.dattribs_names.addValue(prefix+".mAver_rho_",s,null);
    this.dattribs_names.addValue(prefix+".mAI0_",s,null);
    this.dattribs_names.addValue(prefix+".mAc_",s,null);

    return n;
}

private <T> int printMkQVMDPIPsiValues(final MkQVM<T>.DPIPsi x)
{
    int n=0;
    n+=EVXData_Values(x.I);

    n+=printACorrValues(x.SPcorr_f_i);
    n+=printACorrValues(x.SPcorr_f_di);
    n+=printACorrValues(x.SPcorr_f_odi);
    n+=printACorrValues(x.SPrcorr_f_i);
    n+=printACorrValues(x.SPrcorr_f_di);
    n+=printACorrValues(x.SPrrcorr_f_i);
    n+=printACorrValues(x.SPrrcorr_f_di);

    n+=printACorrValues(x.ccorr_f_i);
    n+=printACorrValues(x.ccorr_f_di);
    n+=printACorrValues(x.ccorr_i_di);
    n+=printACorrValues(x.ccorr_t_i);
    n+=printACorrValues(x.ccorr_t_di);
    n+=printACorrValues(x.ccorr_f_t);

    n+=printACorrValues(x.bcorr_f_i);
    n+=printACorrValues(x.bcorr_f_di);

    n+=printACorrValues(x.acorr_f_i);
    n+=printACorrValues(x.acorr_f_di);

    n+=printACorrValues(x.Acorr_f_i);
    n+=printACorrValues(x.Acorr_f_di);
    n+=printACorrValues(x.Acorr_t_di);

    n+=15;
    this.dattribs.addValue(x.mA_ospd);
    this.dattribs.addValue(x.mA_ospd_rho);
    this.dattribs.addValue(x.I0d);
    this.dattribs.addValue(x.mAverd);
    this.dattribs.addValue(x.mAverd_rho);
    this.dattribs.addValue(x.mAver_oDid);
    this.dattribs.addValue(x.mAver_oDi);
    this.dattribs.addValue(x.mAver_oDiof);
    this.dattribs.addValue(x.mAver_oDi0);
    this.dattribs.addValue(x.mAver_aa);
    this.dattribs.addValue(x.mAver_a);
    this.dattribs.addValue(x.mAver);
    this.dattribs.addValue(x.mAver_rho);
    this.dattribs.addValue(x.mAI0);
    this.dattribs.addValue(x.mAc);

    return n;
}





private int printMkQVMRX0PsiNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".J",s);

    n+=2;
    this.dattribs_names.addValue(prefix+".sens_fJ_L_",s,null);
    this.dattribs_names.addValue(prefix+".sens_fJ_H_",s,null);

    return n;
}

private <T> int printMkQVMRX0PsiValues(final MkQVM<T>.RX0Psi x)
{
    int n=0;
    n+=EVXData_Values(x.J);

    n+=2;
    this.dattribs.addValue(x.sens_fJ_L);
    this.dattribs.addValue(x.sens_fJ_H);

    return n;
}





private int printMkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=printEiCommNames(prefix+".eiVsit",s);
    n+=printEiCommNames(prefix+".eipVsit",s);

    n+=printEdCommNames(prefix+".edVsit",s);
    n+=printEdCommNames(prefix+".edpVsit",s);
    n+=printEdCommNames(prefix+".edpit",s);
    n+=printEdCommNames(prefix+".edpiit",s);
    n+=printEdCommNames(prefix+".eddpit",s);

    n+=printACorrNames(prefix+".xxC",s);
    n+=printACorrNames(prefix+".xxCp",s);
    n+=printACorrNames(prefix+".xxCd",s);
    n+=printACorrNames(prefix+".xxCpd",s);
    n+=printECommNames(prefix+".eePiIt",s);
    n+=printECommNames(prefix+".eePit",s);
    n+=printECommNames(prefix+".eeVsit",s);
    n+=printECommNames(prefix+".eeVsitL",s);
    n+=printECommNames(prefix+".eeVsitH",s);

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".P1t",s);
    n+=printMkQVMMDistanceNames(prefix+".distance",s);

    n+=printMkQVMDPIPsiNames(prefix+".dpipsi",s);
    n+=printMkQVMDPIPsiNames(prefix+".vsipsi",s);

    n+=printMkQVMPIPsiNames(prefix+".pipsi",s);

    n+=printMkQVMRX0PsiNames(prefix+".Ritpi",s);
    n+=printMkQVMRX0PsiNames(prefix+".Ritpt",s);

    n+=printMkQVMX0PsiNames(prefix+".itdp",s);
    n+=printMkQVMX0PsiNames(prefix+".itpi",s);
    n+=printMkQVMX0PsiNames(prefix+".itpt",s);
    n+=printMkQVMX0PsiNames(prefix+".itVs",s);
    n+=printMkQVMX0PsiNames(prefix+".itSVs",s);
    n+=printMkQVMX0PsiNames(prefix+".PtI",s);

    n+=printMkQVMRadauPertNames(prefix+".Rip",s);
    n+=printMkQVMFPertPsiNames(prefix+".Itip",s);
    n+=printMkQVMLPertPsiNames(prefix+".aip",s);
    n+=printMkQVMLPertPsiNames(prefix+".itPvL",s);
    n+=printMkQVMLPertPsiNames(prefix+".tip",s);
    n+=printMkQVMLPertPsiNames(prefix+".itp",s);
    n+=printMkQVMFPertPsiNames(prefix+".Ai",s);
    n+=printMkQVMFPertPsiNames(prefix+".ai",s);
    n+=printMkQVMFPertPsiNames(prefix+".AV",s);
    n+=printMkQVMFPertPsiNames(prefix+".aV",s);
    n+=printMkQVMFPertPsiNames(prefix+".itPi",s);
    n+=printMkQVMFPertPsiNames(prefix+".itPt",s);
    n+=printMkQVMFPertPsiNames(prefix+".itPm",s);
    n+=printMkQVMFPertPsiNames(prefix+".itPm_sam",s);
    n+=printMkQVMFPertPsiNames(prefix+".itPvF",s);

    n+=printMkQVMPertPsiNames(prefix+".peVS",s);
    n+=printMkQVMPertPsiNames(prefix+".peVB",s);
    n+=printMkQVMPertPsiNames(prefix+".PeVS",s);
    n+=printMkQVMPertPsiNames(prefix+".PeVB",s);
    n+=printMkQVMPertPsiNames(prefix+".PVeVS",s);
    n+=printMkQVMPertPsiNames(prefix+".PVeVB",s);
    n+=printMkQVMPertPsiNames(prefix+".pePLUS",s);
    n+=printMkQVMPertPsiNames(prefix+".peMINUS",s);
    n+=printMkQVMPertPsiNames(prefix+".PePLUS",s);
    n+=printMkQVMPertPsiNames(prefix+".PeMINUS",s);
    n+=printMkQVMPertPsiNames(prefix+".Pe",s);


    n+=7;
    this.dattribs_names.addValue(prefix+".d1G_IL_",s,null);
    this.dattribs_names.addValue(prefix+".d1G_IH_",s,null);
    this.dattribs_names.addValue(prefix+".d1G_",s,null);
    this.dattribs_names.addValue(prefix+".d2G_",s,null);
    this.dattribs_names.addValue(prefix+".varPi_",s,null);
    this.dattribs_names.addValue(prefix+".DpI_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".varpI_psiH_",s,null);

    return n;
}
private <T> int printMkQVMValues(final MkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=printEiCommValues(x.eiVsit);
    n+=printEiCommValues(x.eipVsit);

    n+=printEdCommValues(x.edVsit);
    n+=printEdCommValues(x.edpVsit);
    n+=printEdCommValues(x.edpit);
    n+=printEdCommValues(x.edpiit);
    n+=printEdCommValues(x.eddpit);

    n+=printACorrValues(x.xxC);
    n+=printACorrValues(x.xxCp);
    n+=printACorrValues(x.xxCd);
    n+=printACorrValues(x.xxCpd);
    n+=printECommValues(x.eePiIt);
    n+=printECommValues(x.eePit);
    n+=printECommValues(x.eeVsit);
    n+=printECommValues(x.eeVsitL);
    n+=printECommValues(x.eeVsitH);

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.P1t);
    n+=printMkQVMMDistanceValues(x.distance);

    n+=printMkQVMDPIPsiValues(x.dpipsi);
    n+=printMkQVMDPIPsiValues(x.vsipsi);

    n+=printMkQVMPIPsiValues(x.pipsi);

    n+=printMkQVMRX0PsiValues(x.Ritpi);
    n+=printMkQVMRX0PsiValues(x.Ritpt);

    n+=printMkQVMX0PsiValues(x.itdp);
    n+=printMkQVMX0PsiValues(x.itpi);
    n+=printMkQVMX0PsiValues(x.itpt);
    n+=printMkQVMX0PsiValues(x.itVs);
    n+=printMkQVMX0PsiValues(x.itSVs);
    n+=printMkQVMX0PsiValues(x.PtI);

    n+=printMkQVMRadauPertValues(x.Rip);
    n+=printMkQVMFPertPsiValues(x.Itip);
    n+=printMkQVMLPertPsiValues(x.aip);
    n+=printMkQVMLPertPsiValues(x.itPvL);
    n+=printMkQVMLPertPsiValues(x.tip);
    n+=printMkQVMLPertPsiValues(x.itp);
    n+=printMkQVMFPertPsiValues(x.Ai);
    n+=printMkQVMFPertPsiValues(x.ai);
    n+=printMkQVMFPertPsiValues(x.AV);
    n+=printMkQVMFPertPsiValues(x.aV);
    n+=printMkQVMFPertPsiValues(x.itPi);
    n+=printMkQVMFPertPsiValues(x.itPt);
    n+=printMkQVMFPertPsiValues(x.itPm);
    n+=printMkQVMFPertPsiValues(x.itPm_sam);
    n+=printMkQVMFPertPsiValues(x.itPvF);

    n+=printMkQVMPertPsiValues(x.peVS);
    n+=printMkQVMPertPsiValues(x.peVB);
    n+=printMkQVMPertPsiValues(x.PeVS);
    n+=printMkQVMPertPsiValues(x.PeVB);
    n+=printMkQVMPertPsiValues(x.PVeVS);
    n+=printMkQVMPertPsiValues(x.PVeVB);
    n+=printMkQVMPertPsiValues(x.pePLUS);
    n+=printMkQVMPertPsiValues(x.peMINUS);
    n+=printMkQVMPertPsiValues(x.PePLUS);
    n+=printMkQVMPertPsiValues(x.PeMINUS);
    n+=printMkQVMPertPsiValues(x.Pe);


    n+=7;
    this.dattribs.addValue(x.d1G_IL);
    this.dattribs.addValue(x.d1G_IH);
    this.dattribs.addValue(x.d1G);
    this.dattribs.addValue(x.d2G);
    this.dattribs.addValue(x.varPi);
    this.dattribs.addValue(x.DpI_psiH);
    this.dattribs.addValue(x.varpI_psiH);

    return n;
}



private int printOkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=printEdCommNames(prefix+".Ed_Pt_I",s);
    n+=printEdCommNames(prefix+".Ed_Pt_x0",s);
    n+=printEdCommNames(prefix+".Ed_Vs_I",s);
    n+=printEdCommNames(prefix+".Ed_Vs_x0",s);
    n+=printEdCommNames(prefix+".Ed_SVs_I",s);
    n+=printEdCommNames(prefix+".Ed_Pi_I",s);
    n+=printEdCommNames(prefix+".Ed_Pi_x0",s);
    n+=printEdCommNames(prefix+".Ed_I_x0",s);

    n+=printEiCommNames(prefix+".Ei_Pt_I",s);
    n+=printEiCommNames(prefix+".Ei_Pt_x0",s);
    n+=printEiCommNames(prefix+".Ei_Vs_I",s);
    n+=printEiCommNames(prefix+".Ei_Vs_x0",s);
    n+=printEiCommNames(prefix+".Ei_SVs_I",s);
    n+=printEiCommNames(prefix+".Ei_Pi_I",s);
    n+=printEiCommNames(prefix+".Ei_Pi_x0",s);
    n+=printEiCommNames(prefix+".Ei_I_x0",s);

    n+=printEpiCommNames(prefix+".E_cd_Pt",s);
    n+=printEpiCommNames(prefix+".E_cd_Pi",s);
    n+=printEpiCommNames(prefix+".E_cd_Vs",s);

    n+=printEbCommNames(prefix+".Eb_Pt",s);
    n+=printEbCommNames(prefix+".Eb_Pi",s);
    n+=printEbCommNames(prefix+".Eb_Vs",s);

    n+=printEcCommNames(prefix+".Ec_Pt_I",s);
    n+=printEcCommNames(prefix+".Ec_Pt_V",s);
    n+=printEcCommNames(prefix+".Ec_dp_I",s);
    n+=printEcCommNames(prefix+".Ec_Pi_I",s);
    n+=printEcCommNames(prefix+".Ec_Vs_I",s);
    n+=printEcCommNames(prefix+".Ec_Vs_V",s);
    n+=printEcCommNames(prefix+".Ec_vs_I",s);

    n+=printEbiCommNames(prefix+".Ebi_P",s);

    n+=printEpCommNames(prefix+".Ep_itdp",s);
    n+=printEpCommNames(prefix+".Ep_itpt",s);
    n+=printEpCommNames(prefix+".Ep_itVs",s);

    n+=printEfiCommNames(prefix+".Efi_itpt",s);
    n+=printEfiCommNames(prefix+".Efi_itpi",s);
    n+=printEfiCommNames(prefix+".Efi_itVs",s);
    n+=printEfiCommNames(prefix+".Efi_itdp",s);
    n+=printEfiCommNames(prefix+".Efi_itddp",s);
    n+=printEfiCommNames(prefix+".Efi_itvs",s);

    n+=printEaCommNames(prefix+".Ea_itpi",s);
    n+=printEaCommNames(prefix+".Ea_itdp",s);
    n+=printEaCommNames(prefix+".Ea_itvs",s);
    n+=printEaCommNames(prefix+".Ea_dptPt",s);
    n+=printEaCommNames(prefix+".Ea_dpiPt",s);

    return n;
}
private <T> int printOkQVMValues(final OkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=printEdCommValues(x.Ed_Pt_I);
    n+=printEdCommValues(x.Ed_Pt_x0);
    n+=printEdCommValues(x.Ed_Vs_I);
    n+=printEdCommValues(x.Ed_Vs_x0);
    n+=printEdCommValues(x.Ed_SVs_I);
    n+=printEdCommValues(x.Ed_Pi_I);
    n+=printEdCommValues(x.Ed_Pi_x0);
    n+=printEdCommValues(x.Ed_I_x0);

    n+=printEiCommValues(x.Ei_Pt_I);
    n+=printEiCommValues(x.Ei_Pt_x0);
    n+=printEiCommValues(x.Ei_Vs_I);
    n+=printEiCommValues(x.Ei_Vs_x0);
    n+=printEiCommValues(x.Ei_SVs_I);
    n+=printEiCommValues(x.Ei_Pi_I);
    n+=printEiCommValues(x.Ei_Pi_x0);
    n+=printEiCommValues(x.Ei_I_x0);

    n+=printEpiCommValues(x.E_cd_Pt);
    n+=printEpiCommValues(x.E_cd_Pi);
    n+=printEpiCommValues(x.E_cd_Vs);

    n+=printEbCommValues(x.Eb_Pt);
    n+=printEbCommValues(x.Eb_Pi);
    n+=printEbCommValues(x.Eb_Vs);

    n+=printEcCommValues(x.Ec_Pt_I);
    n+=printEcCommValues(x.Ec_Pt_V);
    n+=printEcCommValues(x.Ec_dp_I);
    n+=printEcCommValues(x.Ec_Pi_I);
    n+=printEcCommValues(x.Ec_Vs_I);
    n+=printEcCommValues(x.Ec_Vs_V);
    n+=printEcCommValues(x.Ec_vs_I);

    n+=printEbiCommValues(x.Ebi_P);

    n+=printEpCommValues(x.Ep_itdp);
    n+=printEpCommValues(x.Ep_itpt);
    n+=printEpCommValues(x.Ep_itVs);

    n+=printEfiCommValues(x.Efi_itpt);
    n+=printEfiCommValues(x.Efi_itpi);
    n+=printEfiCommValues(x.Efi_itVs);
    n+=printEfiCommValues(x.Efi_itdp);
    n+=printEfiCommValues(x.Efi_itddp);
    n+=printEfiCommValues(x.Efi_itvs);

    n+=printEaCommValues(x.Ea_itpi);
    n+=printEaCommValues(x.Ea_itdp);
    n+=printEaCommValues(x.Ea_itvs);
    n+=printEaCommValues(x.Ea_dptPt);
    n+=printEaCommValues(x.Ea_dpiPt);

    return n;
}



private int printQkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=3;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);

    n+=printEAssymNames(prefix+".aL",s);
    n+=printEAssymNames(prefix+".aH",s);
    n+=printEAssymNames(prefix+".aC",s);
    n+=printEAssymNames(prefix+".a0",s);
    n+=printEAssymNames(prefix+".aVl",s);
    n+=printEAssymNames(prefix+".aVh",s);

    return n;
}
private <T> int printQkQVMValues(final QkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=3;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);

    n+=printEAssymValues(x.aL);
    n+=printEAssymValues(x.aH);
    n+=printEAssymValues(x.aC);
    n+=printEAssymValues(x.a0);
    n+=printEAssymValues(x.aVl);
    n+=printEAssymValues(x.aVh);

    return n;
}



private int printQkQVMpi_Qmu_Names(final String prefix,final String s)
{
    int n=0;

    n+=3;
    this.dattribs_names.addValue(prefix+".p_t_",s,null);
    this.dattribs_names.addValue(prefix+".p_v_",s,null);
    this.dattribs_names.addValue(prefix+".dIdp_",s,null);

    for(int j=0;j<8;j++)
    {
	n+=2;
	this.dattribs_names.addValue(prefix+".px_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".Ix_"+j+"_",s,null);
    }

    return n;
}
private int printQkQVMpi_Qmu_Values(final QkQVMpi.Qmu x)
{
    int n=0;

    n+=3;
    this.dattribs.addValue(x.p_t);
    this.dattribs.addValue(x.p_v);
    this.dattribs.addValue(x.dIdp);

    final int d=x.px.length;
    for(int j=0;j<8;j++)
    {
	n+=2;
	this.dattribs.addValue(j<d?x.px[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.Ix[j]:Double.NaN);
    }

    return n;
}


private int printPnLVar3MupiNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=Skewness_Names(prefix+".skmu",s);
    n+=Skewness_Names(prefix+".iskmu",s);
    //n+=ProbabilityCorrelation_Names(prefix+".pcorr",s);

    n+=6;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".vp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".v2p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".vp_spur_",s,null);
    this.dattribs_names.addValue(prefix+".vpI_spur_",s,null);

    return n;
}
private int printPnLVar3MupiValues(final PnLVar3Mupi x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=Skewness_Values(x.skmu);
    n+=Skewness_Values(x.iskmu);
    //n+=ProbabilityCorrelation_Values(x.pcorr);

    n+=6;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.vp_IH);
    this.dattribs.addValue(x.v2p_IH);
    this.dattribs.addValue(x.vp_spur);
    this.dattribs.addValue(x.vpI_spur);

    return n;
}

private int printPnLFutureSkNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=9;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".P0_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".dI_",s,null);
    this.dattribs_names.addValue(prefix+".GammaPast_",s,null);
    this.dattribs_names.addValue(prefix+".GammaFuture_",s,null);
    this.dattribs_names.addValue(prefix+".degeneracy_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_SameGammaFuture_",s,null);

    return n;
}
private int printPnLFutureSkValues(final PnLFutureSk x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=9;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.P0);
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.dI);
    this.dattribs.addValue(x.GammaPast);
    this.dattribs.addValue(x.GammaFuture);
    this.dattribs.addValue(x.degeneracy);
    this.dattribs.addValue(x.Pf_SameGammaFuture);

    return n;
}
private int printPnLSensitivityNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=16;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".P0_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".p_exit_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_pnl_L_",s,null);
    this.dattribs_names.addValue(prefix+".P_pnl_L_",s,null);
    this.dattribs_names.addValue(prefix+".T_pnl_L_",s,null);
    this.dattribs_names.addValue(prefix+".V_pnl_L_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_H_",s,null);
    this.dattribs_names.addValue(prefix+".I_pnl_H_",s,null);
    this.dattribs_names.addValue(prefix+".P_pnl_H_",s,null);
    this.dattribs_names.addValue(prefix+".T_pnl_H_",s,null);
    this.dattribs_names.addValue(prefix+".V_pnl_H_",s,null);

    return n;
}
private int printPnLSensitivityValues(final PnLSensitivity x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=16;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.P0);
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.p_exit);
    this.dattribs.addValue(x.pnl_L);
    this.dattribs.addValue(x.I_pnl_L);
    this.dattribs.addValue(x.P_pnl_L);
    this.dattribs.addValue(x.T_pnl_L);
    this.dattribs.addValue(x.V_pnl_L);
    this.dattribs.addValue(x.pnl_H);
    this.dattribs.addValue(x.I_pnl_H);
    this.dattribs.addValue(x.P_pnl_H);
    this.dattribs.addValue(x.T_pnl_H);
    this.dattribs.addValue(x.V_pnl_H);

    return n;
}

private int printDIselPnLSensitivityNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=17;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pprev_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pprevimin_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pimax_IH0_",s,null);
    this.dattribs_names.addValue(prefix+".pimin_IH0_",s,null);
    this.dattribs_names.addValue(prefix+".pimaxa_IH0_",s,null);
    this.dattribs_names.addValue(prefix+".pimina_IH0_",s,null);
    this.dattribs_names.addValue(prefix+".distance_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_dist_plus_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_dist_plus_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_dist_plus_",s,null);
    this.dattribs_names.addValue(prefix+".dist_IH_dist_plus_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_dist_minus_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_dist_minus_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_dist_minus_",s,null);
    this.dattribs_names.addValue(prefix+".dist_IH_dist_minus_",s,null);

    return n;
}
private int printDIselPnLSensitivityValues(final DIselPnLSensitivity x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=17;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pprev_IH);
    this.dattribs.addValue(x.pprevimin_IH);
    this.dattribs.addValue(x.pimax_IH0);
    this.dattribs.addValue(x.pimin_IH0);
    this.dattribs.addValue(x.pimaxa_IH0);
    this.dattribs.addValue(x.pimina_IH0);
    this.dattribs.addValue(x.distance_IH);
    this.dattribs.addValue(x.p_IH_dist_plus);
    this.dattribs.addValue(x.p_IL_dist_plus);
    this.dattribs.addValue(x.I_IH_dist_plus);
    this.dattribs.addValue(x.dist_IH_dist_plus);
    this.dattribs.addValue(x.p_IH_dist_minus);
    this.dattribs.addValue(x.p_IL_dist_minus);
    this.dattribs.addValue(x.I_IH_dist_minus);
    this.dattribs.addValue(x.dist_IH_dist_minus);

    return n;
}


private int printDIselPdivINames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=8;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".sk_PmulI.w_plus_",s,null);
    this.dattribs_names.addValue(prefix+".sk_PmulI.w_minus_",s,null);
    this.dattribs_names.addValue(prefix+".sk_P.w_plus_",s,null);
    this.dattribs_names.addValue(prefix+".sk_P.w_minus_",s,null);
    this.dattribs_names.addValue(prefix+".sk_PdivI.w_plus_",s,null);
    this.dattribs_names.addValue(prefix+".sk_PdivI.w_minus_",s,null);

    return n;
}
private int printDIselPdivIValues(final DIselPdivI x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=8;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.sk_PmulI.w_plus);
    this.dattribs.addValue(x.sk_PmulI.w_minus);
    this.dattribs.addValue(x.sk_P.w_plus);
    this.dattribs.addValue(x.sk_P.w_minus);
    this.dattribs.addValue(x.sk_PdivI.w_plus);
    this.dattribs.addValue(x.sk_PdivI.w_minus);

    return n;
}

private int printDIselVTNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".VT",s);
    n+=ProbabilityCorrelation_Names(prefix+".pc_IHv",s);
    n+=ProbabilityCorrelation_Names(prefix+".pct_IHv",s);
    n+=Skewness_Names(prefix+".sk_IHv",s);

    n+=11;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pi_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pV_IH_",s,null);
    this.dattribs_names.addValue(prefix+".T_IH_",s,null);
    this.dattribs_names.addValue(prefix+".VT_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdt_IH_orig_",s,null);
    this.dattribs_names.addValue(prefix+".dPdT_IH_orig_",s,null);
    this.dattribs_names.addValue(prefix+".dPdTw_IH_orig_",s,null);
    this.dattribs_names.addValue(prefix+".dPdTwI_IH_orig_",s,null);
    n+=8;
    this.dattribs_names.addValue(prefix+".p_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".pi_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".pV_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".dpdt_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".r_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IHv_orig_",s,null);

    return n;
}
private int printDIselVTValues(final DIselVT x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.VT);
    n+=ProbabilityCorrelation_Values(x.pc_IHv);
    n+=ProbabilityCorrelation_Values(x.pct_IHv);
    n+=Skewness_Values(x.sk_IHv);

    n+=11;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pi_IH);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.pV_IH);
    this.dattribs.addValue(x.T_IH);
    this.dattribs.addValue(x.VT_IH);
    this.dattribs.addValue(x.dpdt_IH);
    this.dattribs.addValue(x.dpdt_IH_orig);
    this.dattribs.addValue(x.dPdT_IH_orig);
    this.dattribs.addValue(x.dPdTw_IH_orig);
    this.dattribs.addValue(x.dPdTwI_IH_orig);
    n+=8;
    this.dattribs.addValue(x.p_IHv);
    this.dattribs.addValue(x.pi_IHv);
    this.dattribs.addValue(x.pt_IHv);
    this.dattribs.addValue(x.pV_IHv);
    this.dattribs.addValue(x.dpdt_IHv);
    this.dattribs.addValue(x.r_IHv);
    this.dattribs.addValue(x.dp_IHv);
    this.dattribs.addValue(x.dp_IHv_orig);

    return n;
}

private int printDIselVTsimpleNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);

    n+=8;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pV_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".pV_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".dpdt_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".r_IHv_",s,null);

    return n;
}
private int printDIselVTsimpleValues(final DIselVTsimple x)
{
    int n=0;
    n+=EVXData_Values(x.I);

    n+=8;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pV_IH);
    this.dattribs.addValue(x.dpdt_IH);
    this.dattribs.addValue(x.p_IHv);
    this.dattribs.addValue(x.pV_IHv);
    this.dattribs.addValue(x.dpdt_IHv);
    this.dattribs.addValue(x.dp_IHv);
    this.dattribs.addValue(x.r_IHv);

    return n;
}
private int printDIselPIVNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".PIV",s);

    n+=6;
    this.dattribs_names.addValue(prefix+".p_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_H_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_HIV_",s,null);
    this.dattribs_names.addValue(prefix+".p_HIT_",s,null);

    return n;
}
private int printDIselPIVValues(final DIselPIV x)
{
    int n=0;
    n+=EVXData_Values(x.PIV);

    n+=6;
    this.dattribs.addValue(x.p_L);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.p_H);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_HIV);
    this.dattribs.addValue(x.p_HIT);

    return n;
}

private int printDIstatesVTNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".VT",s);

    n+=9;
    this.dattribs_names.addValue(prefix+".pV_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".pi_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".pT_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".pvar_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".r_IHv_",s,null);
    this.dattribs_names.addValue(prefix+".var1pv_",s,null);
    this.dattribs_names.addValue(prefix+".var1pi_",s,null);

    return n;
}
private int printDIstatesVTValues(final DIstatesVT x)
{
    int n=0;
    n+=EVXData_Values(x.VT);

    n+=9;
    this.dattribs.addValue(x.pV_IHv);
    this.dattribs.addValue(x.pi_IHv);
    this.dattribs.addValue(x.pT_IHv);
    this.dattribs.addValue(x.pt_IHv);
    this.dattribs.addValue(x.dp_IHv);
    this.dattribs.addValue(x.pvar_IHv);
    this.dattribs.addValue(x.r_IHv);
    this.dattribs.addValue(x.var1pv);
    this.dattribs.addValue(x.var1pi);

    return n;
}

private int printInIstatesPVNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".PV",s);

    n+=5;
    this.dattribs_names.addValue(prefix+".Imax_",s,null);
    this.dattribs_names.addValue(prefix+".pv_Imax_",s,null);
    this.dattribs_names.addValue(prefix+".pi_Imax_",s,null);
    this.dattribs_names.addValue(prefix+".var2pV_",s,null);
    this.dattribs_names.addValue(prefix+".pv_distmin_",s,null);

    return n;
}
private int printInIstatesPVValues(final InIstatesPV x)
{
    int n=0;
    n+=EVXData_Values(x.PV);

    n+=5;
    this.dattribs.addValue(x.Imax);
    this.dattribs.addValue(x.pv_Imax);
    this.dattribs.addValue(x.pi_Imax);
    this.dattribs.addValue(x.var2pV);
    this.dattribs.addValue(x.pv_distmin);

    return n;
}

private int printIstatesPselectionNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".P",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".Imax_",s,null);
    this.dattribs_names.addValue(prefix+".pm_Imax_",s,null);
    this.dattribs_names.addValue(prefix+".pi_Imax_",s,null);
    this.dattribs_names.addValue(prefix+".var2pm_",s,null);

    return n;
}
private int printIstatesPselectionValues(final IstatesPselection x)
{
    int n=0;
    n+=EVXData_Values(x.P);

    n+=4;
    this.dattribs.addValue(x.Imax);
    this.dattribs.addValue(x.pm_Imax);
    this.dattribs.addValue(x.pi_Imax);
    this.dattribs.addValue(x.var2pm);

    return n;
}

private int printMu2PXinILNames(final String prefix,final String s)
{
    int n=0;

    n+=11;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".lam_",s,null);
    this.dattribs_names.addValue(prefix+".pcmax_",s,null);
    this.dattribs_names.addValue(prefix+".imax_",s,null);
    this.dattribs_names.addValue(prefix+".ximax_",s,null);
    this.dattribs_names.addValue(prefix+".pc1_",s,null);
    this.dattribs_names.addValue(prefix+".i1_",s,null);
    this.dattribs_names.addValue(prefix+".x1_",s,null);
    this.dattribs_names.addValue(prefix+".pc2_",s,null);
    this.dattribs_names.addValue(prefix+".i2_",s,null);
    this.dattribs_names.addValue(prefix+".x2_",s,null);

    return n;
}
private int printMu2PXinILValues(final Mu2PXinIL x)
{
    int n=0;

    n+=11;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.lam);
    this.dattribs.addValue(x.pcmax);
    this.dattribs.addValue(x.imax);
    this.dattribs.addValue(x.ximax);
    this.dattribs.addValue(x.pc1);
    this.dattribs.addValue(x.i1);
    this.dattribs.addValue(x.x1);
    this.dattribs.addValue(x.pc2);
    this.dattribs.addValue(x.i2);
    this.dattribs.addValue(x.x2);

    return n;
}


private int printDIminP2maxINames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=Skewness_Names(prefix+".sk_IH",s);
    n+=Skewness_Names(prefix+".sk_iter",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_",s,null);
    this.dattribs_names.addValue(prefix+".varpvart_",s,null);

    return n;
}
private int printDIminP2maxIValues(final DIminP2maxI x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=Skewness_Values(x.sk_IH);
    n+=Skewness_Values(x.sk_iter);

    n+=4;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_iter);
    this.dattribs.addValue(x.varpvart);

    return n;
}


private int printDIminP2VmaxINames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);

    n+=5;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_i_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_i_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_V_",s,null);

    return n;
}
private int printDIminP2VmaxIValues(final DIminP2VmaxI x)
{
    int n=0;
    n+=EVXData_Values(x.I);

    n+=5;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_init_i);
    this.dattribs.addValue(x.p_init_V);
    this.dattribs.addValue(x.p_iter_i);
    this.dattribs.addValue(x.p_iter_V);

    return n;
}
private int printMaxPP2VINames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=Skewness_Names(prefix+".sk",s);

    n+=5;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_i_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_i_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_V_",s,null);

    return n;
}
private int printMaxPP2VIValues(final MaxPP2VI x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=Skewness_Values(x.sk);

    n+=5;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_init_i);
    this.dattribs.addValue(x.p_init_V);
    this.dattribs.addValue(x.p_iter_i);
    this.dattribs.addValue(x.p_iter_V);

    return n;
}

private int printMaxPnLratioNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);

    n+=11;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_t_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_t_",s,null);
    this.dattribs_names.addValue(prefix+".M_iter_",s,null);
    this.dattribs_names.addValue(prefix+".MI_iter_",s,null);
    this.dattribs_names.addValue(prefix+".p_root_Mmin_",s,null);
    this.dattribs_names.addValue(prefix+".p_root_Mmax_",s,null);
    this.dattribs_names.addValue(prefix+".varp_",s,null);
    this.dattribs_names.addValue(prefix+".DIR_",s,null);

    return n;
}
private int printMaxPnLratioValues(final MaxPnLratio x)
{
    int n=0;
    n+=EVXData_Values(x.I);

    n+=11;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_init_V);
    this.dattribs.addValue(x.p_init_t);
    this.dattribs.addValue(x.p_iter_V);
    this.dattribs.addValue(x.p_iter_t);
    this.dattribs.addValue(x.M_iter);
    this.dattribs.addValue(x.MI_iter);
    this.dattribs.addValue(x.p_root_Mmin);
    this.dattribs.addValue(x.p_root_Mmax);
    this.dattribs.addValue(x.varp);
    this.dattribs.addValue(x.DIR);

    return n;
}

private int printMaxPPl2INames(final String prefix,final String s)
{
    int n=0;

    n+=10;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".pt_",s,null);
    this.dattribs_names.addValue(prefix+".pi_",s,null);
    this.dattribs_names.addValue(prefix+".pV_",s,null);
    this.dattribs_names.addValue(prefix+".V_",s,null);
    this.dattribs_names.addValue(prefix+".MI_",s,null);
    this.dattribs_names.addValue(prefix+".p_root_Mmin_",s,null);
    this.dattribs_names.addValue(prefix+".p_root_Mmax_",s,null);
    this.dattribs_names.addValue(prefix+".varp_",s,null);
    this.dattribs_names.addValue(prefix+".DIR_",s,null);

    return n;
}
private int printMaxPPl2IValues(final MaxPPl2I x)
{
    int n=0;

    n+=10;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.pt);
    this.dattribs.addValue(x.pi);
    this.dattribs.addValue(x.pV);
    this.dattribs.addValue(x.V);
    this.dattribs.addValue(x.MI);
    this.dattribs.addValue(x.p_root_Mmin);
    this.dattribs.addValue(x.p_root_Mmax);
    this.dattribs.addValue(x.varp);
    this.dattribs.addValue(x.DIR);

    return n;
}


private int printMinIPlPVNames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_i_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_i_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_V_",s,null);
    this.dattribs_names.addValue(prefix+".MI_",s,null);
    this.dattribs_names.addValue(prefix+".IM_",s,null);
    this.dattribs_names.addValue(prefix+".p2neg_",s,null);

    return n;
}
private int printMinIPlPVValues(final MinIPlPV x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_init_i);
    this.dattribs.addValue(x.p_init_V);
    this.dattribs.addValue(x.p_iter_i);
    this.dattribs.addValue(x.p_iter_V);
    this.dattribs.addValue(x.MI);
    this.dattribs.addValue(x.IM);
    this.dattribs.addValue(x.p2neg);

    return n;
}


private int printMaxPPl2IinbasisNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);

    n+=15;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".pi_",s,null);
    this.dattribs_names.addValue(prefix+".pt_",s,null);
    this.dattribs_names.addValue(prefix+".pV_",s,null);
    this.dattribs_names.addValue(prefix+".pT_",s,null);
    this.dattribs_names.addValue(prefix+".V_",s,null);
    this.dattribs_names.addValue(prefix+".T_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pV_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pj_",s,null);
    this.dattribs_names.addValue(prefix+".pVj_",s,null);
    this.dattribs_names.addValue(prefix+".pTj_",s,null);
    this.dattribs_names.addValue(prefix+".Vj_",s,null);
    this.dattribs_names.addValue(prefix+".Tj_",s,null);
    this.dattribs_names.addValue(prefix+".pVt_",s,null);

    return n;
}
private int printMaxPPl2IinbasisValues(final MaxPPl2Iinbasis x)
{
    int n=0;
    n+=EVXData_Values(x.I);

    n+=15;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.pi);
    this.dattribs.addValue(x.pt);
    this.dattribs.addValue(x.pV);
    this.dattribs.addValue(x.pT);
    this.dattribs.addValue(x.V);
    this.dattribs.addValue(x.T);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pV_IH);
    this.dattribs.addValue(x.pj);
    this.dattribs.addValue(x.pVj);
    this.dattribs.addValue(x.pTj);
    this.dattribs.addValue(x.Vj);
    this.dattribs.addValue(x.Tj);
    this.dattribs.addValue(x.pVt);

    return n;
}

private int printMaxIP2ratioNames(final String prefix,final String s)
{
    int n=0;

    n+=7;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_t_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_t_",s,null);
    this.dattribs_names.addValue(prefix+".M_iter_",s,null);
    this.dattribs_names.addValue(prefix+".MI_iter_",s,null);

    return n;
}
private int printMaxIP2ratioValues(final MaxIP2ratio x)
{
    int n=0;

    n+=7;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_init_V);
    this.dattribs.addValue(x.p_init_t);
    this.dattribs.addValue(x.p_iter_V);
    this.dattribs.addValue(x.p_iter_t);
    this.dattribs.addValue(x.M_iter);
    this.dattribs.addValue(x.MI_iter);

    return n;
}


private int printDIminP2tmaxINames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=Skewness_Names(prefix+".sk_iter",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_",s,null);
    this.dattribs_names.addValue(prefix+".varpvart_",s,null);

    return n;
}
private int printDIminP2tmaxIValues(final DIminP2tmaxI x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=Skewness_Values(x.sk_iter);

    n+=4;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.p_iter);
    this.dattribs.addValue(x.varpvart);

    return n;
}

private int printMinMaxPnLratioNormNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);

    n+=10;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_init_t_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_iter_t_",s,null);
    this.dattribs_names.addValue(prefix+".M_iter_",s,null);
    this.dattribs_names.addValue(prefix+".p_root_Mmin_",s,null);
    this.dattribs_names.addValue(prefix+".p_root_Mmax_",s,null);
    this.dattribs_names.addValue(prefix+".DIR_",s,null);
    this.dattribs_names.addValue(prefix+".varpvart_",s,null);

    return n;
}
private int printMinMaxPnLratioNormValues(final MinMaxPnLratioNorm x)
{
    int n=0;
    n+=EVXData_Values(x.I);

    n+=10;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_init_V);
    this.dattribs.addValue(x.p_init_t);
    this.dattribs.addValue(x.p_iter_V);
    this.dattribs.addValue(x.p_iter_t);
    this.dattribs.addValue(x.M_iter);
    this.dattribs.addValue(x.p_root_Mmin);
    this.dattribs.addValue(x.p_root_Mmax);
    this.dattribs.addValue(x.DIR);
    this.dattribs.addValue(x.varpvart);

    return n;
}

private int printDIselP2divINames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=ProbabilityCorrelation_Names(prefix+".pc",s);

    n+=8;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_ppl2divI_min_",s,null);
    this.dattribs_names.addValue(prefix+".var0_pi_",s,null);
    this.dattribs_names.addValue(prefix+".var1_pi_",s,null);
    this.dattribs_names.addValue(prefix+".pi_",s,null);
    this.dattribs_names.addValue(prefix+".pV_",s,null);
    this.dattribs_names.addValue(prefix+".V_",s,null);

    return n;
}
private int printDIselP2divIValues(final DIselP2divI x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=ProbabilityCorrelation_Values(x.pc);

    n+=8;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_ppl2divI_min);
    this.dattribs.addValue(x.var0_pi);
    this.dattribs.addValue(x.var1_pi);
    this.dattribs.addValue(x.pi);
    this.dattribs.addValue(x.pV);
    this.dattribs.addValue(x.V);

    return n;
}


private int printIstatesSelectionNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=ProbabilityCorrelation_Names(prefix+".pc_offdiagI",s);
    n+=ProbabilityCorrelation_Names(prefix+".pc_offdiagInoIH",s);
    n+=21;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var2p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".distance_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pimax_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var2_pimax_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pimin_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var2_pdimax_0_",s,null);
    this.dattribs_names.addValue(prefix+".var2_pdimax_",s,null);
    this.dattribs_names.addValue(prefix+".p_diVmax_",s,null);
    this.dattribs_names.addValue(prefix+".var2p_diVmax_",s,null);
    this.dattribs_names.addValue(prefix+".p_iVmax_",s,null);
    this.dattribs_names.addValue(prefix+".var2p_iVmax_",s,null);
    this.dattribs_names.addValue(prefix+".p_ditmax_",s,null);
    this.dattribs_names.addValue(prefix+".var2p_ditmax_",s,null);
    this.dattribs_names.addValue(prefix+".p_d2Im_",s,null);
    this.dattribs_names.addValue(prefix+".v2p_d2Im_",s,null);
    this.dattribs_names.addValue(prefix+".dp0_offdiagI_",s,null);
    this.dattribs_names.addValue(prefix+".var1p_offdiagI_",s,null);
    this.dattribs_names.addValue(prefix+".var1p_offdiagInoIH_",s,null);
    this.dattribs_names.addValue(prefix+".pc_wh_",s,null);
    n+=3;
    this.dattribs_names.addValue(prefix+".p_IHwi_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHws_",s,null);
    this.dattribs_names.addValue(prefix+".dpH_",s,null);

    return n;
}
private int printIstatesSelectionValues(final IstatesSelection x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=ProbabilityCorrelation_Values(x.pc_offdiagI);
    n+=ProbabilityCorrelation_Values(x.pc_offdiagInoIH);
    n+=21;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var2p_IH);
    this.dattribs.addValue(x.distance_IH);
    this.dattribs.addValue(x.pimax_IH);
    this.dattribs.addValue(x.var2_pimax_IH);
    this.dattribs.addValue(x.pimin_IH);
    this.dattribs.addValue(x.var2_pdimax_0);
    this.dattribs.addValue(x.var2_pdimax);
    this.dattribs.addValue(x.p_diVmax);
    this.dattribs.addValue(x.var2p_diVmax);
    this.dattribs.addValue(x.p_iVmax);
    this.dattribs.addValue(x.var2p_iVmax);
    this.dattribs.addValue(x.p_ditmax);
    this.dattribs.addValue(x.var2p_ditmax);
    this.dattribs.addValue(x.p_d2Im);
    this.dattribs.addValue(x.v2p_d2Im);
    this.dattribs.addValue(x.dp0_offdiagI);
    this.dattribs.addValue(x.var1p_offdiagI);
    this.dattribs.addValue(x.var1p_offdiagInoIH);
    this.dattribs.addValue(x.pc_wh);
    n+=3;
    this.dattribs.addValue(x.p_IHwi);
    this.dattribs.addValue(x.p_IHws);
    this.dattribs.addValue(x.dpH);

    return n;
}


private int printIstatesLocalizedNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=Skewness_Names(prefix+".sk_Mweighted",s);
    n+=4;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".sk_Iweighted_",s,null);
    this.dattribs_names.addValue(prefix+".number_priceMatches_",s,null);

    n+=Skewness_Names(prefix+".pm.sk",s);
    n+=3;
    this.dattribs_names.addValue(prefix+".pm.wr0_",s,null);
    this.dattribs_names.addValue(prefix+".pm.x_",s,null);
    this.dattribs_names.addValue(prefix+".pm.i_",s,null);
    return n;
}
private int printIstatesLocalizedValues(final IstatesLocalized x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=Skewness_Values(x.sk_Mweighted);
    n+=4;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.sk_Iweighted);
    this.dattribs.addValue(x.priceMatches.length);

    final IstatesLocalized.PriceMatchStorage pm=x.getPriceMatchMaxI();
    if(pm!=null){
	n+=Skewness_Values(pm.sk);
	n+=3;
	this.dattribs.addValue(pm.wr0);
	this.dattribs.addValue(pm.x);
	this.dattribs.addValue(pm.i);
    } else {
	// will be NaN
	n+=Skewness_Values(com.polytechnik.utils.Skewness.getSkewnessFromMonomialMoments(0,0,0,0));
	n+=3;
	this.dattribs.addValue(Double.NaN);
	this.dattribs.addValue(Double.NaN);
	this.dattribs.addValue(0.0);
    }

    return n;
}

private int printIstatesConditionalNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=5;
    this.dattribs_names.addValue(prefix+".i_M_",s,null);
    this.dattribs_names.addValue(prefix+".LagrangeMultiplier_M_",s,null);
    this.dattribs_names.addValue(prefix+".wr0_M_",s,null);
    this.dattribs_names.addValue(prefix+".orig_constraint_",s,null);
    this.nattribs_names.addValue(prefix+".flag_solution_exists_",s,null);
    return n;
}
private int printIstatesConditionalValues(final com.polytechnik.utils.IstatesConditional x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=5;
    this.dattribs.addValue(x.i_M);
    this.dattribs.addValue(x.LagrangeMultiplier_M);
    this.dattribs.addValue(x.wr0_M);
    this.dattribs.addValue(x.orig_constraint);
    this.nattribs.addValue(x.flag_solution_exists?1:0);
    return n;
}

private int printDIselPIProbabilityCorrelationNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=ProbabilityCorrelation_Names(prefix+".pc",s);
    n+=3;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_dim_",s,null);

    return n;
}
private int printDIselPIProbabilityCorrelationValues(final DIselPIProbabilityCorrelation x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=ProbabilityCorrelation_Values(x.pc);
    n+=3;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_dim);

    return n;
}

private int printPnLVar3MuNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=9;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dir_PCORR_",s,null);
    this.dattribs_names.addValue(prefix+".dir_PCORR_PnL_",s,null);
    this.dattribs_names.addValue(prefix+".dP_PCORR_",s,null);
    this.dattribs_names.addValue(prefix+".dir_gSK_I_",s,null);
    this.dattribs_names.addValue(prefix+".dir_gSK_P_",s,null);
    this.dattribs_names.addValue(prefix+".dir_gSK_P_PnL_",s,null);
    this.dattribs_names.addValue(prefix+".dP_gSK_P_",s,null);

    return n;
}
private int printPnLVar3MuValues(final PnLVar3Mu x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=9;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.dir_PCORR);
    this.dattribs.addValue(x.dir_PCORR_PnL);
    this.dattribs.addValue(x.dP_PCORR);
    this.dattribs.addValue(x.dir_gSK_I);
    this.dattribs.addValue(x.dir_gSK_P);
    this.dattribs.addValue(x.dir_gSK_P_PnL);
    this.dattribs.addValue(x.dP_gSK_P);

    return n;
}



private int printQkPnLpi_PBasisNames(final String prefix,final String s)
{
    int n=0;

    n+=printImpactIStatesExampleNames(prefix+".ist",s);
    n+=printPnLDiagProjNames(prefix+".pnldiag",s);
    n+=printPnLVarVolatilitypiNames(prefix+".pnlvarvol",s);
    n+=printPnLVar3MupiNames(prefix+".pnlvar3",s);
    n+=printPnLVar3MuNames(prefix+".pnlvar3mu",s);

return n;
}

private int printQkPnLpi_PBasisValues(final QkPnLpi_PBasis x,final CalculationData c)
{
    int n=0;

    n+=printImpactIStatesExampleValues(x.ist);
    n+=printPnLDiagProjValues(x.pnldiag);
    n+=printPnLVarVolatilitypiValues(x.pnlvarvol);
    n+=printPnLVar3MupiValues(x.pnlvar3);
    n+=printPnLVar3MuValues(x.pnlvar3mu);

    return n;
}


private int printQkQVMpiNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=printQkQVMpi_Qmu_Names(prefix+".pmL",s);
    n+=printQkQVMpi_Qmu_Names(prefix+".pmH",s);
    n+=printImpactIStatesExampleNames(prefix+".ist",s);
    n+=printPnLdISkewnessNames(prefix+".pnldiskewness_nodI",s);
    n+=printPnLdISkewnessNames(prefix+".pnldiskewness",s);
    n+=PnLSimpleSkewness_Names(prefix+".pnlss",s);

    n+=5;
    this.dattribs_names.addValue(prefix+".i0_",s,null);
    this.dattribs_names.addValue(prefix+".Dpi0_",s,null);
    this.dattribs_names.addValue(prefix+".varI0p_",s,null);
    this.dattribs_names.addValue(prefix+".varI1p_",s,null);
    this.dattribs_names.addValue(prefix+".varI2p_",s,null);

    for(int j=0;j<8;j++)
    {
	n+=2;
	this.dattribs_names.addValue(prefix+".all_p_t_"+j+"_",s,null);
	this.dattribs_names.addValue(prefix+".all_p_v_"+j+"_",s,null);
    }

    return n;
}
private int printQkQVMpiValues(final QkQVMpi x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=printQkQVMpi_Qmu_Values(x.pmL);
    n+=printQkQVMpi_Qmu_Values(x.pmH);
    n+=printImpactIStatesExampleValues(x.ist);
    n+=printPnLdISkewnessValues(x.pnldiskewness_nodI);
    n+=printPnLdISkewnessValues(x.pnldiskewness);
    n+=PnLSimpleSkewness_Values(x.pnlss);

    n+=5;
    this.dattribs.addValue(x.i0);
    this.dattribs.addValue(x.Dpi0);
    this.dattribs.addValue(x.varI0p);
    this.dattribs.addValue(x.varI1p);
    this.dattribs.addValue(x.varI2p);

    final int d=x.I.neq;
    for(int j=0;j<8;j++)
    {
	n+=2;
	this.dattribs.addValue(j<d?x.all_p_t[j]:Double.NaN);
	this.dattribs.addValue(j<d?x.all_p_v[j]:Double.NaN);
    }

    return n;
}


private int printQmQVMpiNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=6;
    this.dattribs_names.addValue(prefix+".p_v_IH_",s,null);
    this.dattribs_names.addValue(prefix+".varV_",s,null);
    this.dattribs_names.addValue(prefix+".vartI_",s,null);
    this.dattribs_names.addValue(prefix+".varvI_",s,null);
    this.dattribs_names.addValue(prefix+".impD_",s,null);
    this.dattribs_names.addValue(prefix+".impU_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".i_max_",s,null);
    this.dattribs_names.addValue(prefix+".p_i_max_",s,null);
    this.dattribs_names.addValue(prefix+".pa_i_",s,null);
    this.dattribs_names.addValue(prefix+".w_max_",s,null);
    this.dattribs_names.addValue(prefix+".p_w_max_",s,null);
    this.dattribs_names.addValue(prefix+".pa_w_",s,null);

    return n;
}
private int printQmQVMpiValues(final QmQVMpi x)
{
    int n=0;

    n+=EVXData_Values(x.I);

    n+=6;
    this.dattribs.addValue(x.p_v_IH);
    this.dattribs.addValue(x.varV);
    this.dattribs.addValue(x.vartI);
    this.dattribs.addValue(x.varvI);
    this.dattribs.addValue(x.impD);
    this.dattribs.addValue(x.impU);

    n+=6;
    this.dattribs.addValue(x.i_max);
    this.dattribs.addValue(x.p_i_max);
    this.dattribs.addValue(x.pa_i);
    this.dattribs.addValue(x.w_max);
    this.dattribs.addValue(x.p_w_max);
    this.dattribs.addValue(x.pa_w);

    return n;
}




private int printNkQVMNames(final String prefix,final String s)
{
    int n=0;


    n+=printEdCommNames(prefix+".E",s);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private <T> int printNkQVMValues(final NkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=printEdCommValues(x.E);
    n+=EVXData_Values(x.I);


    return n;
}


private int printImpactQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=printImpactExampleNames(prefix+".iex",s);
    n+=printImpactIStatesExampleNames(prefix+".ist",s);
    n+=printPnLDiagProjNames(prefix+".pnldiag",s);
    n+=printPnLMuNames(prefix+".pnlmuIL",s);
    n+=printPnLMuNames(prefix+".pnlmuIH",s);
    n+=printPnLVHNames(prefix+".pnlVH",s);
    n+=printDynVar2DiagProjNames(prefix+".dvardiagproj",s);
    n+=printPnLI0Names(prefix+".pnlI0",s);
    n+=printPnLI0Names(prefix+".bpnlI0",s);
    n+=printPnL2I0Names(prefix+".pnl2bI0",s);
    n+=printPnL2I0Names(prefix+".pnl2iterI0",s);
    n+=printPnLbI0Names(prefix+".pnlbI0",s);
    n+=printPnLdIf0Names(prefix+".pnldif0",s);
    n+=printPnLdIf0Names(prefix+".pnldif0_dyn_IH",s);
    n+=printPnLdIf0Names(prefix+".pnldif0_dyn_PURE",s);
    n+=printPnLdIfdynNames(prefix+".pnldifdyn",s);
    n+=printPnLI0eNames(prefix+".pnli0e",s);
    n+=printPnLI0eNames(prefix+".pnli0e_lI",s);
    n+=printPnLI0IHNames(prefix+".pnli0ih",s);
    n+=printPnLI0BNames(prefix+".pnli0b",s);
    n+=printPnLdIdPNames(prefix+".pnldidp",s);
    n+=printPnLdIDNames(prefix+".pnldid",s);
    n+=printPnLdISNames(prefix+".pnldis",s);
    n+=printPnLdISkNames(prefix+".pnldisk",s);
    n+=printPnLdIV4Names(prefix+".pnldiv4_nodI",s);
    n+=printPnLdIV4Names(prefix+".pnldiv4",s);
    n+=printPnLdICorrNames(prefix+".pnldicorr_nodI",s);
    n+=printPnLdICorrNames(prefix+".pnldicorr_nodI_t",s);
    n+=printPnLdICorrNames(prefix+".pnldicorr",s);
    n+=printPnLdICorrNames(prefix+".pnldicorr_t",s);
    n+=printPnLdISkewnessNames(prefix+".pnldiskewness_nodI",s);
    n+=printPnLdISkewnessNames(prefix+".pnldiskewness",s);
    n+=printPnLdIP(prefix+".pnldip_nodI",s);
    n+=printPnLdIP(prefix+".pnldip",s);
    n+=printPnLdIDSkNames(prefix+".pnldidsk",s);
    n+=printPnLdIdINames(prefix+".pnldidi",s);
    n+=printDynVar2DISkewnessNames(prefix+".dv2diskewness",s);
    n+=printDynVar2RDISkewnessNames(prefix+".dv2rdiskewness",s);
    n+=printDynVar2RDISkewnessNames(prefix+".dv2rdiiskewness",s);
    n+=printDynVar2DIDPNames(prefix+".dv2didp_nodI",s);
    n+=printDynVar2DIDPNames(prefix+".dv2didp",s);
    n+=printDynEVdINames(prefix+".dynevdi",s);
    n+=printDynEVdI0Names(prefix+".dynevdi0",s);
    n+=printPnLdIpxNames(prefix+".pnldipx",s);
    n+=printPnLdISpurNames(prefix+".pnldispur",s);
    n+=printMicroPnLdINames(prefix+".mpnlP",s);
    n+=printMicroPnLdINames(prefix+".mpnlDP",s);
    n+=printMicroPnLdINames(prefix+".mpnlDPDT",s);
    n+=printPnLdIENames(prefix+".pnldie",s);
    n+=printPnLdIENames(prefix+".pnldie_dyn",s);
    n+=printPnLdIENames(prefix+".pnldie_dyn__nodI",s);
    n+=printPnLdIE2Names(prefix+".pnldie2",s);
    n+=printPnLdIE2Names(prefix+".pnldie2__nodI",s);
    n+=printPnLVar3MuNames(prefix+".pnlvar3mu",s);
    n+=PnLSimpleSkewness_Names(prefix+".pnlss",s);
    n+=printPnLFutureSkNames(prefix+".pnlfutureSk",s);
    n+=printPnLSensitivityNames(prefix+".pnlsens_pl",s);
    n+=printPnLSensitivityNames(prefix+".pnlsens_p0",s);
    n+=printPnLSensitivityNames(prefix+".pnlsens_pih",s);

    n+=26;
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdt_inDPstate_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdt_inIstate_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdtI_inIstate_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdt_inDIstate_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdt_inDMstate_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.p_IHdm_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdt_spur_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdtI_spur_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdtdI_spur_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdtdIdm_spur_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.DpDI_spur_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.DpfDI_spur_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.DpfDI_spur_HERM_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.DpDI0_corr_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.DpDI0_spur2_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdi_spur2DMIH_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.DpDI0_jointS_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.DpDI0_jointSS_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.DpDI0_jointS2_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdi0dm_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdi0dmdm_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dpdt_dI_DI0_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dp_dI_DI0_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dp_LOWtoHIGH_",s,null);
    this.dattribs_names.addValue(prefix+".inistateihdensity.dp_HIGHtoLOW_",s,null);

    return n;
}
private <T> int printImpactQVMValues(final ImpactQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=printImpactExampleValues(x.iex);
    n+=printImpactIStatesExampleValues(x.ist);
    n+=printPnLDiagProjValues(x.pnldiag);
    n+=printPnLMuValues(x.pnlmuIL);
    n+=printPnLMuValues(x.pnlmuIH);
    n+=printPnLVHValues(x.pnlVH);
    n+=printDynVar2DiagProjValues(x.dvardiagproj);
    n+=printPnLI0Values(x.pnlI0);
    n+=printPnLI0Values(x.bpnlI0);
    n+=printPnL2I0Values(x.pnl2bI0);
    n+=printPnL2I0Values(x.pnl2iterI0);
    n+=printPnLbI0Values(x.pnlbI0);
    n+=printPnLdIf0Values(x.pnldif0);
    n+=printPnLdIf0Values(x.pnldif0_dyn_IH);
    n+=printPnLdIf0Values(x.pnldif0_dyn_PURE);
    n+=printPnLdIfdynValues(x.pnldifdyn);
    n+=printPnLI0eValues(x.pnli0e);
    n+=printPnLI0eValues(x.pnli0e_lI);
    n+=printPnLI0IHValues(x.pnli0ih);
    n+=printPnLI0BValues(x.pnli0b);
    n+=printPnLdIdPValues(x.pnldidp);
    n+=printPnLdIDValues(x.pnldid);
    n+=printPnLdISValues(x.pnldis);
    n+=printPnLdISkValues(x.pnldisk);
    n+=printPnLdIV4Values(x.pnldiv4_nodI);
    n+=printPnLdIV4Values(x.pnldiv4);
    n+=printPnLdICorrValues(x.pnldicorr_nodI);
    n+=printPnLdICorrValues(x.pnldicorr_nodI_t);
    n+=printPnLdICorrValues(x.pnldicorr);
    n+=printPnLdICorrValues(x.pnldicorr_t);
    n+=printPnLdISkewnessValues(x.pnldiskewness_nodI);
    n+=printPnLdISkewnessValues(x.pnldiskewness);
    n+=printPnLdIP(x.pnldip_nodI);
    n+=printPnLdIP(x.pnldip);
    n+=printPnLdIDSkValues(x.pnldidsk);
    n+=printPnLdIdIValues(x.pnldidi);
    n+=printDynVar2DISkewnessValues(x.dv2diskewness);
    n+=printDynVar2RDISkewnessValues(x.dv2rdiskewness);
    n+=printDynVar2RDISkewnessValues(x.dv2rdiiskewness);
    n+=printDynVar2DIDPValues(x.dv2didp_nodI);
    n+=printDynVar2DIDPValues(x.dv2didp);
    n+=printDynEVdIValues(x.dynevdi);
    n+=printDynEVdI0Values(x.dynevdi0);
    n+=printPnLdIpxValues(x.pnldipx);
    n+=printPnLdISpurValues(x.pnldispur);
    n+=printMicroPnLdIValues(x.mpnlP);
    n+=printMicroPnLdIValues(x.mpnlDP);
    n+=printMicroPnLdIValues(x.mpnlDPDT);
    n+=printPnLdIEValues(x.pnldie);
    n+=printPnLdIEValues(x.pnldie_dyn);
    n+=printPnLdIEValues(x.pnldie_dyn__nodI);
    n+=printPnLdIE2Values(x.pnldie2);
    n+=printPnLdIE2Values(x.pnldie2__nodI);
    n+=printPnLVar3MuValues(x.pnlvar3mu);
    n+=PnLSimpleSkewness_Values(x.pnlss);
    n+=printPnLFutureSkValues(x.pnlfutureSk);
    n+=printPnLSensitivityValues(x.pnlsens_pl);
    n+=printPnLSensitivityValues(x.pnlsens_p0);
    n+=printPnLSensitivityValues(x.pnlsens_pih);

    n+=26;
    this.dattribs.addValue(x.inistateihdensity.dpdt_inDPstate);
    this.dattribs.addValue(x.inistateihdensity.dpdt_inIstate);
    this.dattribs.addValue(x.inistateihdensity.dpdtI_inIstate);
    this.dattribs.addValue(x.inistateihdensity.dpdt_inDIstate);
    this.dattribs.addValue(x.inistateihdensity.dpdt_inDMstate);
    this.dattribs.addValue(x.inistateihdensity.p_IH);
    this.dattribs.addValue(x.inistateihdensity.p_IHdm);
    this.dattribs.addValue(x.inistateihdensity.dpdt_spur);
    this.dattribs.addValue(x.inistateihdensity.dpdtI_spur);
    this.dattribs.addValue(x.inistateihdensity.dpdtdI_spur);
    this.dattribs.addValue(x.inistateihdensity.dpdtdIdm_spur);
    this.dattribs.addValue(x.inistateihdensity.DpDI_spur);
    this.dattribs.addValue(x.inistateihdensity.DpfDI_spur);
    this.dattribs.addValue(x.inistateihdensity.DpfDI_spur_HERM);
    this.dattribs.addValue(x.inistateihdensity.DpDI0_corr);
    this.dattribs.addValue(x.inistateihdensity.DpDI0_spur2);
    this.dattribs.addValue(x.inistateihdensity.dpdi_spur2DMIH);
    this.dattribs.addValue(x.inistateihdensity.DpDI0_jointS);
    this.dattribs.addValue(x.inistateihdensity.DpDI0_jointSS);
    this.dattribs.addValue(x.inistateihdensity.DpDI0_jointS2);
    this.dattribs.addValue(x.inistateihdensity.dpdi0dm);
    this.dattribs.addValue(x.inistateihdensity.dpdi0dmdm);
    this.dattribs.addValue(x.inistateihdensity.dpdt_dI_DI0);
    this.dattribs.addValue(x.inistateihdensity.dp_dI_DI0);
    this.dattribs.addValue(x.inistateihdensity.dp_LOWtoHIGH);
    this.dattribs.addValue(x.inistateihdensity.dp_HIGHtoLOW);

    return n;
}
private int PnLSimpleSkewness_Names(final String prefix,final String s)
{
    int n=0;
    n+=5;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".p_B_",s,null);
    this.dattribs_names.addValue(prefix+".P0_",s,null);

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".P",s);
    n+=Skewness_Names(prefix+".SK_P_average",s);
    n+=Skewness_Names(prefix+".gSK_P_average",s);
    n+=Skewness_Names(prefix+".gSK_I_average",s);

    n+=ProbabilityCorrelation_Names(prefix+".PCORR_average",s);
    n+=ProbabilityCorrelation_Names(prefix+".PCORR_dpI",s);

    return n;
}
private int PnLSimpleSkewness_Values(final PnLSimpleSkewness x)
{
    int n=0;
    n+=5;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.p_B);
    this.dattribs.addValue(x.P0);

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.P);
    n+=Skewness_Values(x.SK_P_average);
    n+=Skewness_Values(x.gSK_P_average);
    n+=Skewness_Values(x.gSK_I_average);

    n+=ProbabilityCorrelation_Values(x.PCORR_average);
    n+=ProbabilityCorrelation_Values(x.PCORR_dpI);

    return n;
}
private int printImpactQVMMuseNames(final String prefix,final String s)
{
    int n=0;

    //n+=Skewness_Names(prefix+".SK_average",s);

    return n;
}
private <T> int printImpactQVMMuseValues(final ImpactQVMMuse<T> x,final CalculationData c)
{
    int n=0;

    //n+=Skewness_Values(x.SK_average);

    return n;
}


private int Mu2P2Asymmetry_MuRQ_Names(final String prefix,final String s)
{
    int n=0;
    n+=8;
    this.dattribs_names.addValue(prefix+".wmu_",s,null);
    this.dattribs_names.addValue(prefix+".D_",s,null);
    this.dattribs_names.addValue(prefix+".beta_",s,null);
    this.dattribs_names.addValue(prefix+".lam_",s,null);
    this.dattribs_names.addValue(prefix+".lamconst_",s,null);
    this.dattribs_names.addValue(prefix+".p_psiroot_",s,null);
    this.dattribs_names.addValue(prefix+".pt_",s,null);
    this.dattribs_names.addValue(prefix+".pi_",s,null);

    return n;
}
private int Mu2P2Asymmetry_MuRQ_Values(final Mu2P2Asymmetry.MuRQ x)
{
    int n=0;
    n+=8;
    this.dattribs.addValue(x.wmu);
    this.dattribs.addValue(x.D);
    this.dattribs.addValue(x.beta);
    this.dattribs.addValue(x.lam);
    this.dattribs.addValue(x.lamconst);
    this.dattribs.addValue(x.p_psiroot);
    this.dattribs.addValue(x.pt);
    this.dattribs.addValue(x.pi);

    return n;
}

private int Mu2P2Asymmetry_Names(final String prefix,final String s)
{
    int n=0;
    n+=Mu2P2Asymmetry_MuRQ_Names(prefix+".mL",s);
    n+=Mu2P2Asymmetry_MuRQ_Names(prefix+".mH",s);
    n+=ProbabilityCorrelation_Names(prefix+".pc",s);
    n+=ProbabilityCorrelation_Names(prefix+".pci",s);
    n+=ProbabilityCorrelation_Names(prefix+".pcA",s);
    n+=ProbabilityCorrelation_Names(prefix+".pcDpA",s);
    n+=ProbabilityCorrelation_Names(prefix+".pcDA",s);
    n+=2;
    this.dattribs_names.addValue(prefix+".betaS_",s,null);
    this.dattribs_names.addValue(prefix+".IbetaS_",s,null);
    return n;
}
private int Mu2P2Asymmetry_Values(final Mu2P2Asymmetry x)
{
    int n=0;
    n+=Mu2P2Asymmetry_MuRQ_Values(x.mL);
    n+=Mu2P2Asymmetry_MuRQ_Values(x.mH);
    n+=ProbabilityCorrelation_Values(x.pc);
    n+=ProbabilityCorrelation_Values(x.pci);
    n+=ProbabilityCorrelation_Values(x.pcA);
    n+=ProbabilityCorrelation_Values(x.pcDpA);
    n+=ProbabilityCorrelation_Values(x.pcDA);
    n+=2;
    this.dattribs.addValue(x.betaS);
    this.dattribs.addValue(x.IbetaS);
    return n;
}
private int printDynImpactAsymmetryNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=Mu2P2Asymmetry_Names(prefix+".dia_average",s);
    n+=Mu2P2Asymmetry_Names(prefix+".dia_spur",s);
    n+=Mu2P2Asymmetry_Names(prefix+".dia_IH",s);

    n+=5;
    this.dattribs_names.addValue(prefix+".dia_mu.corr_weighted_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Pi.sum_impact_dP_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Pi.sum_impact_dPdt_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Pi.dP_eq_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Pi.Pi_eq_",s,null);
    n+=6;
    this.dattribs_names.addValue(prefix+".ifa_P0i.Pf_from_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_P0i.deg_from_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_P0i.dir_Dpf_I_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_P0i.dir_Dp_pf_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_P0i.dir_Dp_I_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_P0i.dir_iDp_pf_",s,null);
    n+=4;
    this.dattribs_names.addValue(prefix+".ifa_Pfi.p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Pfi.Pf_from_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Pfi.deg_from_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Pfi.dir_scaled_",s,null);
    n+=13;
    this.dattribs_names.addValue(prefix+".ifa_Ci.p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.Pf_from_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.deg_from_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.dir_scaled_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.var_f_IH_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.var_pmu_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.var_pmu_IHfa_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.var_pmu_IHfm_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.var_pmu_IHf_qsk_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.var_pmu_IH_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.var_pmu_IHa_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.var_pmu_IHm_",s,null);
    this.dattribs_names.addValue(prefix+".ifa_Ci.var_pmu_IH_qsk_",s,null);

    n+=InDIState_Names(prefix+".Rindistate",s);
    n+=InDIState_Names(prefix+".Rindistate2",s);
    n+=InDIState_Names(prefix+".RindistateF",s);
    n+=10;
    this.dattribs_names.addValue(prefix+".indistateI0f.dpP_",s,null);
    this.dattribs_names.addValue(prefix+".indistateI0f.dpaP_",s,null);
    this.dattribs_names.addValue(prefix+".indistateI0f.dpM_",s,null);
    this.dattribs_names.addValue(prefix+".indistateI0f.dpaM_",s,null);
    this.dattribs_names.addValue(prefix+".indistateI0f.dV_shares_",s,null);
    this.dattribs_names.addValue(prefix+".indistateI0.dpP_",s,null);
    this.dattribs_names.addValue(prefix+".indistateI0.dpaP_",s,null);
    this.dattribs_names.addValue(prefix+".indistateI0.dpM_",s,null);
    this.dattribs_names.addValue(prefix+".indistateI0.dpaM_",s,null);
    this.dattribs_names.addValue(prefix+".indistateI0.dV_shares_",s,null);

    n+=5;
    this.dattribs_names.addValue(prefix+".inistateVT.p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".inistateVT.p_Vmax_",s,null);
    this.dattribs_names.addValue(prefix+".inistateVT.p_Tmax_",s,null);
    this.dattribs_names.addValue(prefix+".inistateVT.d_pIH_",s,null);
    this.dattribs_names.addValue(prefix+".inistateVT.d_pItau_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".indistatemean.p_average_",s,null);
    this.dattribs_names.addValue(prefix+".indistatemean.dp_withDIaverage_",s,null);
    this.dattribs_names.addValue(prefix+".indistatemean.dpdt_withDIaverage_",s,null);
    this.dattribs_names.addValue(prefix+".indistatemean.dpdta_withDIaverage_",s,null);
    return n;
}
private <T> int printDynImpactAsymmetryValues(final DynImpactAsymmetry<T> x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=Mu2P2Asymmetry_Values(x.dia_average);
    n+=Mu2P2Asymmetry_Values(x.dia_spur);
    n+=Mu2P2Asymmetry_Values(x.dia_IH);

    n+=5;
    this.dattribs.addValue(x.dia_mu.corr_weighted);
    this.dattribs.addValue(x.ifa_Pi.sum_impact_dP);
    this.dattribs.addValue(x.ifa_Pi.sum_impact_dPdt);
    this.dattribs.addValue(x.ifa_Pi.dP_eq);
    this.dattribs.addValue(x.ifa_Pi.Pi_eq);
    n+=6;
    this.dattribs.addValue(x.ifa_P0i.Pf_from_dyn);
    this.dattribs.addValue(x.ifa_P0i.deg_from_dyn);
    this.dattribs.addValue(x.ifa_P0i.dir_Dpf_I);
    this.dattribs.addValue(x.ifa_P0i.dir_Dp_pf);
    this.dattribs.addValue(x.ifa_P0i.dir_Dp_I);
    this.dattribs.addValue(x.ifa_P0i.dir_iDp_pf);
    n+=4;
    this.dattribs.addValue(x.ifa_Pfi.p_IH);
    this.dattribs.addValue(x.ifa_Pfi.Pf_from_dyn);
    this.dattribs.addValue(x.ifa_Pfi.deg_from_dyn);
    this.dattribs.addValue(x.ifa_Pfi.dir_scaled);
    n+=13;
    this.dattribs.addValue(x.ifa_Ci.p_IH);
    this.dattribs.addValue(x.ifa_Ci.Pf_from_dyn);
    this.dattribs.addValue(x.ifa_Ci.deg_from_dyn);
    this.dattribs.addValue(x.ifa_Ci.dir_scaled);
    this.dattribs.addValue(x.ifa_Ci.var_f_IH);
    this.dattribs.addValue(x.ifa_Ci.var_pmu_IHf);
    this.dattribs.addValue(x.ifa_Ci.var_pmu_IHfa);
    this.dattribs.addValue(x.ifa_Ci.var_pmu_IHfm);
    this.dattribs.addValue(x.ifa_Ci.var_pmu_IHf_qsk);
    this.dattribs.addValue(x.ifa_Ci.var_pmu_IH);
    this.dattribs.addValue(x.ifa_Ci.var_pmu_IHa);
    this.dattribs.addValue(x.ifa_Ci.var_pmu_IHm);
    this.dattribs.addValue(x.ifa_Ci.var_pmu_IH_qsk);

    n+=InDIState_Values(x.Rindistate);
    n+=InDIState_Values(x.Rindistate2);
    n+=InDIState_Values(x.RindistateF);
    n+=10;
    this.dattribs.addValue(x.indistateI0f.dpP);
    this.dattribs.addValue(x.indistateI0f.dpaP);
    this.dattribs.addValue(x.indistateI0f.dpM);
    this.dattribs.addValue(x.indistateI0f.dpaM);
    this.dattribs.addValue(x.indistateI0f.dV_shares);
    this.dattribs.addValue(x.indistateI0.dpP);
    this.dattribs.addValue(x.indistateI0.dpaP);
    this.dattribs.addValue(x.indistateI0.dpM);
    this.dattribs.addValue(x.indistateI0.dpaM);
    this.dattribs.addValue(x.indistateI0.dV_shares);

    n+=5;
    this.dattribs.addValue(x.inistateVT.p_IH);
    this.dattribs.addValue(x.inistateVT.p_Vmax);
    this.dattribs.addValue(x.inistateVT.p_Tmax);
    this.dattribs.addValue(x.inistateVT.d_pIH);
    this.dattribs.addValue(x.inistateVT.d_pItau);

    n+=4;
    this.dattribs.addValue(x.indistatemean.p_average);
    this.dattribs.addValue(x.indistatemean.dp_withDIaverage);
    this.dattribs.addValue(x.indistatemean.dpdt_withDIaverage);
    this.dattribs.addValue(x.indistatemean.dpdta_withDIaverage);

    return n;
}


private int Mu2F_Names(final String prefix,final String s)
{
    int n=0;
    n+=ProbabilityCorrelation_Names(prefix+".pc",s);
    n+=4;
    this.dattribs_names.addValue(prefix+".pt_",s,null);
    this.dattribs_names.addValue(prefix+".pi_",s,null);
    this.dattribs_names.addValue(prefix+".root_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".root_psiH_",s,null);

    return n;
}
private int Mu2F_Values(final Mu2F x)
{
    int n=0;
    n+=ProbabilityCorrelation_Values(x.pc);
    n+=4;
    this.dattribs.addValue(x.pt);
    this.dattribs.addValue(x.pi);
    this.dattribs.addValue(x.root_psiL);
    this.dattribs.addValue(x.root_psiH);

    return n;
}


private int printIAImpactNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=Skewness_Names(prefix+".sk_I",s);
    n+=10;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Fdp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Fad_IH_",s,null);
    this.dattribs_names.addValue(prefix+".FDpwv_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Fv_IH_",s,null);
    this.dattribs_names.addValue(prefix+".FDpwt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Ft_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpw_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pa_IH_",s,null);
    n+=13;
    this.dattribs_names.addValue(prefix+".varA1_",s,null);
    this.dattribs_names.addValue(prefix+".varp1_IAH_",s,null);
    this.dattribs_names.addValue(prefix+".pi_IAH_",s,null);
    this.dattribs_names.addValue(prefix+".pa_IAH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IAH_",s,null);
    this.dattribs_names.addValue(prefix+".dm_IAH_",s,null);
    this.dattribs_names.addValue(prefix+".wv_IAH_",s,null);
    this.dattribs_names.addValue(prefix+".wt_IAH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IAL_",s,null);
    this.dattribs_names.addValue(prefix+".iar.dp_IA_",s,null);
    this.dattribs_names.addValue(prefix+".iar.p_I_",s,null);
    this.dattribs_names.addValue(prefix+".iar.p_A_",s,null);
    this.dattribs_names.addValue(prefix+".iar.dir_",s,null);
    n+=EVXData_Names(prefix+".A",s);
    n+=EVXData_Names(prefix+".IdpAA",s);
    n+=EVXData_Names(prefix+".PnL",s);
    n+=Skewness_Names(prefix+".sk_PnL",s);
    n+=EVXData_Names(prefix+".IA",s);
    n+=Skewness_Names(prefix+".sk_IAH",s);
    n+=EVXData_Names(prefix+".PnLa",s);
    n+=Skewness_Names(prefix+".sk_PnLa",s);
    n+=EVXData_Names(prefix+".PnLad",s);
    n+=Skewness_Names(prefix+".sk_PnLad",s);
    n+=EVXData_Names(prefix+".Vmax",s);
    n+=Skewness_Names(prefix+".sk_Vmax",s);
    n+=EVXData_Names(prefix+".Ia",s);
    n+=Skewness_Names(prefix+".sk_Ia",s);
    n+=6;
    this.dattribs_names.addValue(prefix+".dpn_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpi_IH_",s,null);
    this.dattribs_names.addValue(prefix+".PnL_H_dp_",s,null);
    this.dattribs_names.addValue(prefix+".PnL_H_dir_",s,null);
    this.dattribs_names.addValue(prefix+".PnLa_H_dir_",s,null);
    this.dattribs_names.addValue(prefix+".PnLad_H_dir_",s,null);

    n+=Mu2F_Names(prefix+".pIH",s);
    n+=Mu2F_Names(prefix+".pAH",s);
    n+=Mu2F_Names(prefix+".pIAH",s);

    n+=5;
    this.dattribs_names.addValue(prefix+".iad.dir_",s,null);
    this.dattribs_names.addValue(prefix+".mu2ia.p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".mu2ia.dir.dir_",s,null);
    this.dattribs_names.addValue(prefix+".mu2ia.dir.root_ipsiL_",s,null);
    this.dattribs_names.addValue(prefix+".mu2ia.dir.root_apsiL_",s,null);

    return n;
}
private <T> int printIAImpactValues(final IAImpact<T> x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=Skewness_Values(x.sk_I);
    n+=10;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.Fdp_IH);
    this.dattribs.addValue(x.Fad_IH);
    this.dattribs.addValue(x.FDpwv_IH);
    this.dattribs.addValue(x.Fv_IH);
    this.dattribs.addValue(x.FDpwt_IH);
    this.dattribs.addValue(x.Ft_IH);
    this.dattribs.addValue(x.dpw_IH);
    this.dattribs.addValue(x.pa_IH);
    n+=13;
    this.dattribs.addValue(x.varA1);
    this.dattribs.addValue(x.varp1_IAH);
    this.dattribs.addValue(x.pi_IAH);
    this.dattribs.addValue(x.pa_IAH);
    this.dattribs.addValue(x.dp_IAH);
    this.dattribs.addValue(x.dm_IAH);
    this.dattribs.addValue(x.wv_IAH);
    this.dattribs.addValue(x.wt_IAH);
    this.dattribs.addValue(x.dp_IAL);
    this.dattribs.addValue(x.iar.dp_IA);
    this.dattribs.addValue(x.iar.p_I);
    this.dattribs.addValue(x.iar.p_A);
    this.dattribs.addValue(x.iar.dir);
    n+=EVXData_Values(x.A);
    n+=EVXData_Values(x.IdpAA);
    n+=EVXData_Values(x.PnL);
    n+=Skewness_Values(x.sk_PnL);
    n+=EVXData_Values(x.IA);
    n+=Skewness_Values(x.sk_IAH);
    n+=EVXData_Values(x.PnLa);
    n+=Skewness_Values(x.sk_PnLa);
    n+=EVXData_Values(x.PnLad);
    n+=Skewness_Values(x.sk_PnLad);
    n+=EVXData_Values(x.Vmax);
    n+=Skewness_Values(x.sk_Vmax);
    n+=EVXData_Values(x.Ia);
    n+=Skewness_Values(x.sk_Ia);
    n+=6;
    this.dattribs.addValue(x.dpn_IH);
    this.dattribs.addValue(x.dpi_IH);
    this.dattribs.addValue(x.PnL_H_dp);
    this.dattribs.addValue(x.PnL_H_dir);
    this.dattribs.addValue(x.PnLa_H_dir);
    this.dattribs.addValue(x.PnLad_H_dir);

    n+=Mu2F_Values(x.pIH);
    n+=Mu2F_Values(x.pAH);
    n+=Mu2F_Values(x.pIAH);

    n+=5;
    this.dattribs.addValue(x.iad.dir);
    this.dattribs.addValue(x.mu2ia.p_offset);
    this.dattribs.addValue(x.mu2ia.dir.dir);
    this.dattribs.addValue(x.mu2ia.dir.root_ipsiL);
    this.dattribs.addValue(x.mu2ia.dir.root_apsiL);

    return n;
}

private int printIStateNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);

    n+=2;
    this.dattribs_names.addValue(prefix+".sst.s_IL_before_IH.p_",s,null);
    this.dattribs_names.addValue(prefix+".sst.s_IH.p_",s,null);

    return n;
}
private <T> int printIStateValues(final IState<T> x)
{
    int n=0;
    n+=EVXData_Values(x.I);

    n+=2;
    this.dattribs.addValue(x.sst.s_IL_before_IH!=null?x.sst.s_IL_before_IH.p:Double.NaN);
    this.dattribs.addValue(x.sst.s_IH!=null?x.sst.s_IH.p:Double.NaN);

    return n;
}
private int printDM_HTNames(final String prefix,final String s)
{
   int n=0;
   n+=EVXData_Names(prefix+".I",s);
   n+=EVXData_Names(prefix+".V",s);
   n+=10;
   this.dattribs_names.addValue(prefix+".p_IH_",s,null);
   this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
   this.dattribs_names.addValue(prefix+".dpdt_IH_",s,null);
   this.dattribs_names.addValue(prefix+".T_IH_",s,null);
   this.dattribs_names.addValue(prefix+".V_IH_",s,null);
   this.dattribs_names.addValue(prefix+".p_VH_",s,null);
   this.dattribs_names.addValue(prefix+".I_IL_BeforeIH_",s,null);
   this.dattribs_names.addValue(prefix+".p_IL_BeforeIH_",s,null);
   this.dattribs_names.addValue(prefix+".T_IL_BeforeIH_",s,null);
   this.dattribs_names.addValue(prefix+".V_IL_BeforeIH_",s,null);
   return n;
}
private int printDM_HTValues(final DM_HT x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.V);
    n+=10;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.dpdt_IH);
    this.dattribs.addValue(x.T_IH);
    this.dattribs.addValue(x.V_IH);
    this.dattribs.addValue(x.p_VH);
    this.dattribs.addValue(x.I_IL_BeforeIH);
    this.dattribs.addValue(x.p_IL_BeforeIH);
    this.dattribs.addValue(x.T_IL_BeforeIH);
    this.dattribs.addValue(x.V_IL_BeforeIH);
    return n;
}
private int printIHStateNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".Ibi",s);
    n+=ProbabilityCorrelation_Names(prefix+".pib",s);
    n+=ProbabilityCorrelation_Names(prefix+".pif",s);

    n+=10;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IAH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHb_",s,null);
    this.dattribs_names.addValue(prefix+".i_IHb_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IHb_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".i_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IHf_",s,null);
    n+=7;
    this.dattribs_names.addValue(prefix+".pab_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpa_",s,null);
    this.dattribs_names.addValue(prefix+".dpab_",s,null);
    this.dattribs_names.addValue(prefix+".dpaf_",s,null);
    this.dattribs_names.addValue(prefix+".dpIbi_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHbi_",s,null);
    this.dattribs_names.addValue(prefix+".p_ILbt_",s,null);

    return n;
}
private <T> int printIHStateValues(final IHState<T> x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.Ibi);
    n+=ProbabilityCorrelation_Values(x.pib);
    n+=ProbabilityCorrelation_Values(x.pif);

    n+=10;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.dp_IH);
    this.dattribs.addValue(x.dp_IAH);
    this.dattribs.addValue(x.p_IHb);
    this.dattribs.addValue(x.i_IHb);
    this.dattribs.addValue(x.dp_IHb);
    this.dattribs.addValue(x.p_IHf);
    this.dattribs.addValue(x.i_IHf);
    this.dattribs.addValue(x.dp_IHf);
    n+=7;
    this.dattribs.addValue(x.pab_IH);
    this.dattribs.addValue(x.dpa);
    this.dattribs.addValue(x.dpab);
    this.dattribs.addValue(x.dpaf);
    this.dattribs.addValue(x.dpIbi);
    this.dattribs.addValue(x.p_IHbi);
    this.dattribs.addValue(x.p_ILbt);


    return n;
}

private int printDM_DINames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".DM",s);

    n+=20;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".V_IH_",s,null);
    this.dattribs_names.addValue(prefix+".T_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_spurDMIH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_wI_spurDMIH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_winvI_spurDMIH_",s,null);
    this.dattribs_names.addValue(prefix+".di_spurDMIH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdi_spur2DMIH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdp_spur2DMIH_",s,null);
    this.dattribs_names.addValue(prefix+".didi_spur2DMIH_",s,null);
    this.dattribs_names.addValue(prefix+".dpI_spurDMIH_",s,null);
    this.dattribs_names.addValue(prefix+".dpI_spur2DMIH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdi_spur2DMIH_LOWtoHIGH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdi_spur2DMIH_HIGHtoLOW_",s,null);
    this.dattribs_names.addValue(prefix+".dpdi_spurDMIH_LOWtoHIGH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdi_spurDMIH_HIGHtoLOW_",s,null);
    this.dattribs_names.addValue(prefix+".dpdi_LOWtoHIGH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdi_HIGHtoLOW_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".dpi_LOWtoHIGH_rho_",s,null);
    this.dattribs_names.addValue(prefix+".dpi_HIGHtoLOW_rho_",s,null);
    this.dattribs_names.addValue(prefix+".dpi_LOWtoHIGH_",s,null);
    this.dattribs_names.addValue(prefix+".dpi_HIGHtoLOW_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".pit_fplus_",s,null);
    this.dattribs_names.addValue(prefix+".pit_fminus_",s,null);
    this.dattribs_names.addValue(prefix+".pitDM_fplus_",s,null);
    this.dattribs_names.addValue(prefix+".pitDM_fminus_",s,null);

    return n;
}
private int printDM_DIValues(final DM_DI x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.DM);

    n+=20;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.dpdt_IH);
    this.dattribs.addValue(x.V_IH);
    this.dattribs.addValue(x.T_IH);
    this.dattribs.addValue(x.dp_spurDMIH);
    this.dattribs.addValue(x.dp_wI_spurDMIH);
    this.dattribs.addValue(x.dp_winvI_spurDMIH);
    this.dattribs.addValue(x.di_spurDMIH);
    this.dattribs.addValue(x.dpdi_spur2DMIH);
    this.dattribs.addValue(x.dpdp_spur2DMIH);
    this.dattribs.addValue(x.didi_spur2DMIH);
    this.dattribs.addValue(x.dpI_spurDMIH);
    this.dattribs.addValue(x.dpI_spur2DMIH);
    this.dattribs.addValue(x.dpdi_spur2DMIH_LOWtoHIGH);
    this.dattribs.addValue(x.dpdi_spur2DMIH_HIGHtoLOW);
    this.dattribs.addValue(x.dpdi_spurDMIH_LOWtoHIGH);
    this.dattribs.addValue(x.dpdi_spurDMIH_HIGHtoLOW);
    this.dattribs.addValue(x.dpdi_LOWtoHIGH);
    this.dattribs.addValue(x.dpdi_HIGHtoLOW);

    n+=4;
    this.dattribs.addValue(x.dpi_LOWtoHIGH_rho);
    this.dattribs.addValue(x.dpi_HIGHtoLOW_rho);
    this.dattribs.addValue(x.dpi_LOWtoHIGH);
    this.dattribs.addValue(x.dpi_HIGHtoLOW);

    n+=4;
    this.dattribs.addValue(x.pit_fplus);
    this.dattribs.addValue(x.pit_fminus);
    this.dattribs.addValue(x.pitDM_fplus);
    this.dattribs.addValue(x.pitDM_fminus);

    return n;
}


private int printDM_DIfNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);

    n+=7;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".T_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_spurDMIH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_wIf_",s,null);
    this.dattribs_names.addValue(prefix+".dp_wI_",s,null);

    return n;
}
private int printDM_DIfValues(final DM_DIf x)
{
    int n=0;
    n+=EVXData_Values(x.I);

    n+=7;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.dpdt_IH);
    this.dattribs.addValue(x.T_IH);
    this.dattribs.addValue(x.dp_spurDMIH);
    this.dattribs.addValue(x.dp_wIf);
    this.dattribs.addValue(x.dp_wI);

    return n;
}

private int DMstateAnswer_Names(final String prefix,final String s)
{
    int n=0;
    n+=5;
    this.dattribs_names.addValue(prefix+".wt_",s,null);
    this.dattribs_names.addValue(prefix+".wi_",s,null);
    this.dattribs_names.addValue(prefix+".pi_",s,null);
    this.dattribs_names.addValue(prefix+".dpdt_",s,null);
    this.dattribs_names.addValue(prefix+".wpsiH_projection_",s,null);

    return n;
}
private int DMstateAnswer_Values(final DMstateAnswer x)
{
    int n=0;
    n+=5;
    this.dattribs.addValue(x.wt);
    this.dattribs.addValue(x.wi);
    this.dattribs.addValue(x.pi);
    this.dattribs.addValue(x.dpdt);
    this.dattribs.addValue(x.wpsiH_projection);

    return n;
}



private int printDIselIHNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=5;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dEVseparation_IH_",s,null);
    this.dattribs_names.addValue(prefix+".wpsiH_projection_simple_plus_",s,null);
    this.dattribs_names.addValue(prefix+".wpsiH_projection_simple_minus_",s,null);

    n+=DMstateAnswer_Names(prefix+".sIH",s);
    n+=DMstateAnswer_Names(prefix+".sIH_plus",s);
    n+=DMstateAnswer_Names(prefix+".sIH_minus",s);

    return n;
}
private int printDIselIHValues(final DIselIH x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=5;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.dpdt_IH);
    this.dattribs.addValue(x.dEVseparation_IH);
    this.dattribs.addValue(x.wpsiH_projection_simple_plus);
    this.dattribs.addValue(x.wpsiH_projection_simple_minus);

    n+=DMstateAnswer_Values(x.sIH);
    n+=DMstateAnswer_Values(x.sIH_plus);
    n+=DMstateAnswer_Values(x.sIH_minus);

    return n;
}

private int printDIselCorrPINames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=6;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".corr_",s,null);
    this.dattribs_names.addValue(prefix+".w_pp_",s,null);
    this.dattribs_names.addValue(prefix+".w_pm_",s,null);
    this.dattribs_names.addValue(prefix+".w_mp_",s,null);
    this.dattribs_names.addValue(prefix+".w_mm_",s,null);

    return n;
}
private int printDIselCorrPIValues(final DIselCorrPI x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=6;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.corr);
    this.dattribs.addValue(x.w_pp);
    this.dattribs.addValue(x.w_pm);
    this.dattribs.addValue(x.w_mp);
    this.dattribs.addValue(x.w_mm);

    return n;
}


private int printDIselIHskewnessNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=6;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var2p_dim_",s,null);
    this.dattribs_names.addValue(prefix+".sk_dI_plus_",s,null);
    this.dattribs_names.addValue(prefix+".sk_dI_minus_",s,null);
    this.dattribs_names.addValue(prefix+".pI_plus_",s,null);
    this.dattribs_names.addValue(prefix+".pI_minus_",s,null);

    n+=Skewness_Names(prefix+".skIH",s);
    n+=Skewness_Names(prefix+".sk_dim",s);
    n+=Skewness_Names(prefix+".skIH_plus",s);
    n+=Skewness_Names(prefix+".skIH_minus",s);

    return n;
}
private int printDIselIHskewnessValues(final DIselIHskewness x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=6;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var2p_dim);
    this.dattribs.addValue(x.sk_dI_plus);
    this.dattribs.addValue(x.sk_dI_minus);
    this.dattribs.addValue(x.pI_plus);
    this.dattribs.addValue(x.pI_minus);

    n+=Skewness_Values(x.skIH);
    n+=Skewness_Values(x.sk_dim);
    n+=Skewness_Values(x.skIH_plus);
    n+=Skewness_Values(x.skIH_minus);

    return n;
}


private int printDIselIH2piNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=9;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pf_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dEVseparation_IH_",s,null);
    this.dattribs_names.addValue(prefix+".wpsiHP_projection_simple_plus_",s,null);
    this.dattribs_names.addValue(prefix+".wpsiHP_projection_simple_minus_",s,null);
    this.dattribs_names.addValue(prefix+".GammaP_",s,null);
    this.dattribs_names.addValue(prefix+".wpsiHPI_projection_simple_plus_",s,null);
    this.dattribs_names.addValue(prefix+".wpsiHPI_projection_simple_minus_",s,null);
    this.dattribs_names.addValue(prefix+".GammaPI_",s,null);

    return n;
}
private int printDIselIH2piValues(final DIselIH2pi x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=9;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pf_IH);
    this.dattribs.addValue(x.dEVseparation_IH);
    this.dattribs.addValue(x.wpsiHP_projection_simple_plus);
    this.dattribs.addValue(x.wpsiHP_projection_simple_minus);
    this.dattribs.addValue(x.GammaP);
    this.dattribs.addValue(x.wpsiHPI_projection_simple_plus);
    this.dattribs.addValue(x.wpsiHPI_projection_simple_minus);
    this.dattribs.addValue(x.GammaPI);

    return n;
}
private int printDIselIH2PositiondINames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=2;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    n+=ProbabilityCorrelation_Names(prefix+".corr_Position_dI",s);

    return n;
}
private int printDIselIH2PositiondIValues(final DIselIH2PositiondI x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=2;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_offset);

    n+=ProbabilityCorrelation_Values(x.corr_Position_dI);

    return n;
}
private int printDIselIHPositiondINames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=12;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".dm_spur_",s,null);
    this.dattribs_names.addValue(prefix+".position_eq_",s,null);
    this.dattribs_names.addValue(prefix+".dI_in_dS_plus_",s,null);
    this.dattribs_names.addValue(prefix+".dI_in_dS_minus_",s,null);
    this.dattribs_names.addValue(prefix+".dS_in_dI_plus_",s,null);
    this.dattribs_names.addValue(prefix+".dS_in_dI_minus_",s,null);
    this.dattribs_names.addValue(prefix+".w_pp_",s,null);
    this.dattribs_names.addValue(prefix+".w_pm_",s,null);
    this.dattribs_names.addValue(prefix+".w_mp_",s,null);
    this.dattribs_names.addValue(prefix+".w_mm_",s,null);

    return n;
}
private int printDIselIHPositiondIValues(final DIselIHPositiondI x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=12;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.dm_spur);
    this.dattribs.addValue(x.position_eq);
    this.dattribs.addValue(x.dI_in_dS_plus);
    this.dattribs.addValue(x.dI_in_dS_minus);
    this.dattribs.addValue(x.dS_in_dI_plus);
    this.dattribs.addValue(x.dS_in_dI_minus);
    this.dattribs.addValue(x.w_pp);
    this.dattribs.addValue(x.w_pm);
    this.dattribs.addValue(x.w_mp);
    this.dattribs.addValue(x.w_mm);

    return n;
}

private int printDIselPsi0Names(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=6;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_0f_",s,null);
    this.dattribs_names.addValue(prefix+".dEVseparation_0_",s,null);
    this.dattribs_names.addValue(prefix+".wpsi0f_projection_simple_plus_",s,null);
    this.dattribs_names.addValue(prefix+".wpsi0f_projection_simple_minus_",s,null);
    this.dattribs_names.addValue(prefix+".GammaPI_",s,null);

    return n;
}
private int printDIselPsi0Values(final DIselPsi0 x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=6;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_0f);
    this.dattribs.addValue(x.dEVseparation_0);
    this.dattribs.addValue(x.wpsi0f_projection_simple_plus);
    this.dattribs.addValue(x.wpsi0f_projection_simple_minus);
    this.dattribs.addValue(x.GammaPI);

    return n;
}

private int printDIselD2PNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=6;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var2p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dirpI_plus_",s,null);
    this.dattribs_names.addValue(prefix+".dirpV_plus_",s,null);
    this.dattribs_names.addValue(prefix+".dirpI_minus_",s,null);
    this.dattribs_names.addValue(prefix+".dirpV_minus_",s,null);

    /*
    n+=6;
    this.dattribs_names.addValue(prefix+".w_0_",s,null);
    this.dattribs_names.addValue(prefix+".w_plus_",s,null);
    this.dattribs_names.addValue(prefix+".lw_plus_",s,null);
    this.dattribs_names.addValue(prefix+".w_minus_",s,null);
    this.dattribs_names.addValue(prefix+".lw_minus_",s,null);
    this.dattribs_names.addValue(prefix+".sumdpmu_",s,null);
    */
    return n;
}
private int printDIselD2PValues(final DIselD2P x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=6;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var2p_IH);
    this.dattribs.addValue(x.dirpI_plus);
    this.dattribs.addValue(x.dirpV_plus);
    this.dattribs.addValue(x.dirpI_minus);
    this.dattribs.addValue(x.dirpV_minus);

    /*
    n+=6;
    this.dattribs.addValue(x.w_0);
    this.dattribs.addValue(x.w_plus);
    this.dattribs.addValue(x.lw_plus);
    this.dattribs.addValue(x.w_minus);
    this.dattribs.addValue(x.lw_minus);
    this.dattribs.addValue(x.sumdpmu);
    */
    return n;
}

private int printDIselEVpNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".P",s);
    n+=8;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var2p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_dIH_",s,null);
    this.dattribs_names.addValue(prefix+".var2p_dIH_",s,null);
    this.dattribs_names.addValue(prefix+".p_dIHn_",s,null);
    this.dattribs_names.addValue(prefix+".var2p_dIHn_",s,null);
    this.dattribs_names.addValue(prefix+".p_dIHV_",s,null);
    this.dattribs_names.addValue(prefix+".var2p_dIHV_",s,null);
    n+=ProbabilityCorrelation_Names(prefix+".pc_dIHn",s);

    n+=8;
    this.dattribs_names.addValue(prefix+".dirpI_plus_",s,null);
    this.dattribs_names.addValue(prefix+".dirpV_plus_",s,null);
    this.dattribs_names.addValue(prefix+".dirpI_minus_",s,null);
    this.dattribs_names.addValue(prefix+".dirpV_minus_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_above_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_above_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_below_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_below_",s,null);

    return n;
}
private int printDIselEVpValues(final DIselEVp x)
{
    int n=0;
    n+=EVXData_Values(x.P);
    n+=8;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var2p_IH);
    this.dattribs.addValue(x.p_dIH);
    this.dattribs.addValue(x.var2p_dIH);
    this.dattribs.addValue(x.p_dIHn);
    this.dattribs.addValue(x.var2p_dIHn);
    this.dattribs.addValue(x.p_dIHV);
    this.dattribs.addValue(x.var2p_dIHV);
    n+=ProbabilityCorrelation_Values(x.pc_dIHn);

    n+=8;
    this.dattribs.addValue(x.dirpI_plus);
    this.dattribs.addValue(x.dirpV_plus);
    this.dattribs.addValue(x.dirpI_minus);
    this.dattribs.addValue(x.dirpV_minus);
    this.dattribs.addValue(x.p_IH_above);
    this.dattribs.addValue(x.I_IH_above);
    this.dattribs.addValue(x.p_IH_below);
    this.dattribs.addValue(x.I_IH_below);

    return n;
}

private int printDIselDMNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=DMstateAnswer_Names(prefix+".sIH",s);
    n+=DMstateAnswer_Names(prefix+".sDM_diplus",s);
    n+=DMstateAnswer_Names(prefix+".sDM_diminus",s);
    n+=DMstateAnswer_Names(prefix+".sDIH_diplus",s);
    n+=DMstateAnswer_Names(prefix+".sDIH_diminus",s);

    n+=2;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdtvari_IH_",s,null);

    return n;
}
private int printDIselDMValues(final DIselDM x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=DMstateAnswer_Values(x.sIH);
    n+=DMstateAnswer_Values(x.sDM_diplus);
    n+=DMstateAnswer_Values(x.sDM_diminus);
    n+=DMstateAnswer_Values(x.sDIH_diplus);
    n+=DMstateAnswer_Values(x.sDIH_diminus);

    n+=2;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.dpdtvari_IH);

    return n;
}



private int printIHStateDMNames(final String prefix,final String s)
{
    int n=0;
    n+=printDM_DINames(prefix+".dmdi",s);
    n+=printDM_DIfNames(prefix+".dmdif",s);
    n+=printDM_HTNames(prefix+".dmht",s);
    n+=printDIselDMNames(prefix+".diselDM",s);

    n+=printDIselIHNames(prefix+".diselIHsepdpdt",s);
    n+=printDIselIHNames(prefix+".diselIHsepd2pdt2",s);
    n+=printDIselIHNames(prefix+".diselIHsepdDpwvdt",s);
    n+=printDIselIHNames(prefix+".diselIHsepdIdt",s);
    n+=printDIselIHNames(prefix+".diselIHsepPIH",s);
    n+=printDIselIHNames(prefix+".diselIHplast",s);
    n+=printDIselIHNames(prefix+".diselIHdpI",s);

    n+=printDIselIHNames(prefix+".diselIH2sepdpdt",s);
    n+=printDIselIHNames(prefix+".diselIH2sepd2pdt2",s);
    n+=printDIselIHNames(prefix+".diselIH2sepdDpwvdt",s);
    n+=printDIselIHNames(prefix+".diselIH2sepdIdt",s);
    n+=printDIselIHNames(prefix+".diselIH2sepPIH",s);
    n+=printDIselIHNames(prefix+".diselIH2plast",s);

    n+=printDIselIH2piNames(prefix+".diselPSkewness",s);
    n+=printDIselIH2piNames(prefix+".diselPSkewnessF",s);
    n+=printDIselIH2piNames(prefix+".diselPSkewness0",s);
    n+=printDIselIH2piNames(prefix+".diselPSkewness0F",s);

    n+=printDIselPsi0Names(prefix+".diselPsi0",s);
    n+=printDIselPsi0Names(prefix+".diselPsi0F",s);

    n+=printDIselIH2PositiondINames(prefix+".disel2PositiondI",s);
    n+=printDIselIHPositiondINames(prefix+".diselPositiondI_inPsiH",s);
    n+=printDIselIHPositiondINames(prefix+".diselPositionPdI_inPsiH",s);
    n+=printDIselIHPositiondINames(prefix+".diselPositiondI_inPsiH_dynamicimpact",s);
    n+=printDIselIHPositiondINames(prefix+".diselPositiondI_Dp_afterPsiH",s);
    n+=printDIselIHPositiondINames(prefix+".diselPositiondI_Dp_afterPsiH_p_last",s);
    n+=printDIselIHPositiondINames(prefix+".diselPositiondI_DpWW_afterPsiH",s);
    n+=printDIselIHPositiondINames(prefix+".diselPositiondI_DpWW_afterPsiH_p_last",s);
    n+=printDIselIHPositiondINames(prefix+".diselPositiondI_Dpv_afterPsiH",s);


    n+=3;
    this.dattribs_names.addValue(prefix+".istatesppl.p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".istatesppl.PPl_IH_",s,null);
    this.dattribs_names.addValue(prefix+".istatesppl.PPl_diag_",s,null);

    n+=printIstatesConditionalNames(prefix+".istatesconditionalIcV",s);
    n+=printIstatesConditionalNames(prefix+".istatesconditionalIcdI",s);
    n+=printIstatesConditionalNames(prefix+".istatesconditionalIcdP",s);
    n+=printIstatesConditionalNames(prefix+".istatesconditionalIcd2P",s);
    n+=printIstatesConditionalNames(prefix+".istatesconditionalIcI",s);
    n+=10;
    this.dattribs_names.addValue(prefix+".istatesconditionalIcV_var1_",s,null);
    this.dattribs_names.addValue(prefix+".istatesconditionalIcV_px_",s,null);
    this.dattribs_names.addValue(prefix+".istatesconditionalIcdI_var1_",s,null);
    this.dattribs_names.addValue(prefix+".istatesconditionalIcdI_px_",s,null);
    this.dattribs_names.addValue(prefix+".istatesconditionalIcdP_var1_",s,null);
    this.dattribs_names.addValue(prefix+".istatesconditionalIcdP_px_",s,null);
    this.dattribs_names.addValue(prefix+".istatesconditionalIcd2P_var1_",s,null);
    this.dattribs_names.addValue(prefix+".istatesconditionalIcd2P_px_",s,null);
    this.dattribs_names.addValue(prefix+".istatesconditionalIcI_var1_",s,null);
    this.dattribs_names.addValue(prefix+".istatesconditionalIcI_px_",s,null);

    n+=printIstatesConditionalNames(prefix+".istatesconditionalVcI",s);
    n+=1;this.dattribs_names.addValue(prefix+".istatesconditional_skpc.var_ppi_",s,null);
    n+=1;this.dattribs_names.addValue(prefix+".istatesconditional_skpc.p_m_",s,null);
    n+=Skewness_Names(prefix+".istatesconditional_skpc.sk",s);
    n+=ProbabilityCorrelation_Names(prefix+".istatesconditional_skpc.pc",s);
    n+=printIstatesConditionalNames(prefix+".istatesconditionalV",s);
    n+=1;this.dattribs_names.addValue(prefix+".istatesconditionalV_skpc.var_ppi_",s,null);
    n+=1;this.dattribs_names.addValue(prefix+".istatesconditionalV_skpc.p_m_",s,null);
    n+=Skewness_Names(prefix+".istatesconditionalV_skpc.sk",s);
    n+=ProbabilityCorrelation_Names(prefix+".istatesconditionalV_skpc.pc",s);

    n+=printIstatesSelectionNames(prefix+".istatesselection",s);
    n+=printIstatesLocalizedNames(prefix+".istateslocalized",s);
    n+=printDIselPIProbabilityCorrelationNames(prefix+".diselpiprobabilitycorrPi",s);
    n+=printDIselPIProbabilityCorrelationNames(prefix+".diselpiprobabilitycorrPt",s);
    n+=printDIselPnLSensitivityNames(prefix+".diselpnlsens",s);
    n+=SepSkewness_Names(prefix+".disel_corr_dS_I_inPsiH.pt_ss",s);
    n+=SepSkewness_Names(prefix+".disel_corr_dS_I_inPsiH.dpdt_ss",s);
    n+=10;
    this.dattribs_names.addValue(prefix+".disel_corr_dS_I_inPsiH.reg_corr_Position_I_",s,null);
    this.dattribs_names.addValue(prefix+".disel_corr_dS_I_inPsiH.reg_corr_dp_dI_",s,null);
    this.dattribs_names.addValue(prefix+".disel_corr_dS_I_inPsiH.reg_corr_d2p_I_",s,null);
    this.dattribs_names.addValue(prefix+".disel_corr_dS_I_inPsiH.dp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".disel_corr_dS_I_inPsiH.dp_ort_IH_",s,null);
    this.dattribs_names.addValue(prefix+".disel_corr_dS_I_afterPsiH.reg_corr_Position_I_",s,null);
    this.dattribs_names.addValue(prefix+".disel_corr_dS_I_afterPsiH.reg_corr_dp_dI_",s,null);
    this.dattribs_names.addValue(prefix+".disel_corr_dS_I_afterPsiH.reg_corr_d2p_I_",s,null);
    this.dattribs_names.addValue(prefix+".disel_corr_dS_I_afterPsiH.dp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".disel_corr_dS_I_afterPsiH.dp_ort_IH_",s,null);

    n+=printMu2PXinILNames(prefix+".mu2pxinil",s);
    n+=printDIselD2PNames(prefix+".diseld2pt",s);
    n+=printDIselD2PNames(prefix+".diseld2pi",s);
    n+=printDIselEVpNames(prefix+".diselevp",s);
    n+=printDIselIHskewnessNames(prefix+".diselsk",s);
    n+=printDIselCorrPINames(prefix+".diselcorrpi",s);
    n+=printDIselPdivINames(prefix+".diselpdivI",s);
    n+=printDIselP2divINames(prefix+".diselp2divI",s);
    n+=printIstatesPselectionNames(prefix+".ipselV",s);
    n+=printIstatesPselectionNames(prefix+".ipselI",s);
    n+=printInIstatesPVNames(prefix+".inistatesPV",s);
    n+=printDIstatesVTNames(prefix+".distatesVT",s);
    n+=printDIstatesVTNames(prefix+".distatesVTmin",s);
    n+=printDIselVTNames(prefix+".diselVT",s);
    n+=printDIselVTsimpleNames(prefix+".diselVTsimple",s);
    n+=printDIselPIVNames(prefix+".diselPIV",s);
    n+=printDIminP2tmaxINames(prefix+".minp2tmaxI",s);
    n+=printMaxPnLratioNames(prefix+".maxpnlratioI",s);
    n+=printMaxPnLratioNames(prefix+".maxpnlratioV",s);
    n+=printMinIPlPVNames(prefix+".minIPlPV",s);
    n+=printMinIPlPVNames(prefix+".minVPlPI",s);
    n+=printMaxPPl2IinbasisNames(prefix+".maxppl2iinbasis",s);
    n+=printMaxPPl2INames(prefix+".maxppl2",s);
    n+=printMaxPPl2INames(prefix+".maxppl2div",s);
    n+=printMaxPPl2INames(prefix+".maxppl2I",s);
    n+=printMaxPPl2INames(prefix+".maxppl2V",s);
    n+=printMaxPP2VINames(prefix+".maxpp2vi",s);
    n+=printMaxIP2ratioNames(prefix+".maxip2I",s);
    n+=printMaxIP2ratioNames(prefix+".maxip2V",s);
    n+=printDIminP2VmaxINames(prefix+".diminp2vmaxI",s);
    n+=printDIminP2VmaxINames(prefix+".diminp2vmaxVT",s);
    n+=printDIminP2maxINames(prefix+".diminP2maxI",s);
    n+=printDIminP2maxINames(prefix+".diminP2maxVT",s);
    n+=printDIminP2maxINames(prefix+".diminP2maxIT",s);
    n+=printDIminP2maxINames(prefix+".diminP2maxIV",s);
    n+=printMinMaxPnLratioNormNames(prefix+".minpnlratioNI",s);
    n+=printMinMaxPnLratioNormNames(prefix+".maxpnlratioNI",s);
    n+=printMinMaxPnLratioNormNames(prefix+".minpnlratioNV",s);
    n+=printMinMaxPnLratioNormNames(prefix+".maxpnlratioNV",s);

    return n;
}
private <T> int printIHStateDMValues(final IHStateDM<T> x)
{
    int n=0;
    n+=printDM_DIValues(x.dmdi);
    n+=printDM_DIfValues(x.dmdif);
    n+=printDM_HTValues(x.dmht);
    n+=printDIselDMValues(x.diselDM);

    n+=printDIselIHValues(x.diselIHsepdpdt);
    n+=printDIselIHValues(x.diselIHsepd2pdt2);
    n+=printDIselIHValues(x.diselIHsepdDpwvdt);
    n+=printDIselIHValues(x.diselIHsepdIdt);
    n+=printDIselIHValues(x.diselIHsepPIH);
    n+=printDIselIHValues(x.diselIHplast);
    n+=printDIselIHValues(x.diselIHdpI);

    n+=printDIselIHValues(x.diselIH2sepdpdt);
    n+=printDIselIHValues(x.diselIH2sepd2pdt2);
    n+=printDIselIHValues(x.diselIH2sepdDpwvdt);
    n+=printDIselIHValues(x.diselIH2sepdIdt);
    n+=printDIselIHValues(x.diselIH2sepPIH);
    n+=printDIselIHValues(x.diselIH2plast);

    n+=printDIselIH2piValues(x.diselPSkewness);
    n+=printDIselIH2piValues(x.diselPSkewnessF);
    n+=printDIselIH2piValues(x.diselPSkewness0);
    n+=printDIselIH2piValues(x.diselPSkewness0F);

    n+=printDIselPsi0Values(x.diselPsi0);
    n+=printDIselPsi0Values(x.diselPsi0F);

    n+=printDIselIH2PositiondIValues(x.disel2PositiondI);
    n+=printDIselIHPositiondIValues(x.diselPositiondI_inPsiH);
    n+=printDIselIHPositiondIValues(x.diselPositionPdI_inPsiH);
    n+=printDIselIHPositiondIValues(x.diselPositiondI_inPsiH_dynamicimpact);
    n+=printDIselIHPositiondIValues(x.diselPositiondI_Dp_afterPsiH);
    n+=printDIselIHPositiondIValues(x.diselPositiondI_Dp_afterPsiH_p_last);
    n+=printDIselIHPositiondIValues(x.diselPositiondI_DpWW_afterPsiH);
    n+=printDIselIHPositiondIValues(x.diselPositiondI_DpWW_afterPsiH_p_last);
    n+=printDIselIHPositiondIValues(x.diselPositiondI_Dpv_afterPsiH);


    n+=3;
    this.dattribs.addValue(x.istatesppl.p_IH);
    this.dattribs.addValue(x.istatesppl.PPl_IH);
    this.dattribs.addValue(x.istatesppl.PPl_diag);

    n+=printIstatesConditionalValues(x.istatesconditionalIcV);
    n+=printIstatesConditionalValues(x.istatesconditionalIcdI);
    n+=printIstatesConditionalValues(x.istatesconditionalIcdP);
    n+=printIstatesConditionalValues(x.istatesconditionalIcd2P);
    n+=printIstatesConditionalValues(x.istatesconditionalIcI);
    n+=10;
    this.dattribs.addValue(x.istatesconditionalIcV_var1);
    this.dattribs.addValue(x.istatesconditionalIcV_px);
    this.dattribs.addValue(x.istatesconditionalIcdI_var1);
    this.dattribs.addValue(x.istatesconditionalIcdI_px);
    this.dattribs.addValue(x.istatesconditionalIcdP_var1);
    this.dattribs.addValue(x.istatesconditionalIcdP_px);
    this.dattribs.addValue(x.istatesconditionalIcd2P_var1);
    this.dattribs.addValue(x.istatesconditionalIcd2P_px);
    this.dattribs.addValue(x.istatesconditionalIcI_var1);
    this.dattribs.addValue(x.istatesconditionalIcI_px);

    n+=printIstatesConditionalValues(x.istatesconditionalVcI);
    n+=1;this.dattribs.addValue(x.istatesconditional_skpc.var_ppi);
    n+=1;this.dattribs.addValue(x.istatesconditional_skpc.p_m);
    n+=Skewness_Values(x.istatesconditional_skpc.sk);
    n+=ProbabilityCorrelation_Values(x.istatesconditional_skpc.pc);
    n+=printIstatesConditionalValues(x.istatesconditionalV);
    n+=1;this.dattribs.addValue(x.istatesconditionalV_skpc.var_ppi);
    n+=1;this.dattribs.addValue(x.istatesconditionalV_skpc.p_m);
    n+=Skewness_Values(x.istatesconditionalV_skpc.sk);
    n+=ProbabilityCorrelation_Values(x.istatesconditionalV_skpc.pc);

    n+=printIstatesSelectionValues(x.istatesselection);
    n+=printIstatesLocalizedValues(x.istateslocalized);
    n+=printDIselPIProbabilityCorrelationValues(x.diselpiprobabilitycorrPi);
    n+=printDIselPIProbabilityCorrelationValues(x.diselpiprobabilitycorrPt);
    n+=printDIselPnLSensitivityValues(x.diselpnlsens);
    n+=SepSkewness_Values(x.disel_corr_dS_I_inPsiH.pt_ss);
    n+=SepSkewness_Values(x.disel_corr_dS_I_inPsiH.dpdt_ss);
    n+=10;
    this.dattribs.addValue(x.disel_corr_dS_I_inPsiH.reg_corr_Position_I);
    this.dattribs.addValue(x.disel_corr_dS_I_inPsiH.reg_corr_dp_dI);
    this.dattribs.addValue(x.disel_corr_dS_I_inPsiH.reg_corr_d2p_I);
    this.dattribs.addValue(x.disel_corr_dS_I_inPsiH.dp_IH);
    this.dattribs.addValue(x.disel_corr_dS_I_inPsiH.dp_ort_IH);
    this.dattribs.addValue(x.disel_corr_dS_I_afterPsiH.reg_corr_Position_I);
    this.dattribs.addValue(x.disel_corr_dS_I_afterPsiH.reg_corr_dp_dI);
    this.dattribs.addValue(x.disel_corr_dS_I_afterPsiH.reg_corr_d2p_I);
    this.dattribs.addValue(x.disel_corr_dS_I_afterPsiH.dp_IH);
    this.dattribs.addValue(x.disel_corr_dS_I_afterPsiH.dp_ort_IH);

    n+=printMu2PXinILValues(x.mu2pxinil);
    n+=printDIselD2PValues(x.diseld2pt);
    n+=printDIselD2PValues(x.diseld2pi);
    n+=printDIselEVpValues(x.diselevp);
    n+=printDIselIHskewnessValues(x.diselsk);
    n+=printDIselCorrPIValues(x.diselcorrpi);
    n+=printDIselPdivIValues(x.diselpdivI);
    n+=printDIselP2divIValues(x.diselp2divI);
    n+=printIstatesPselectionValues(x.ipselV);
    n+=printIstatesPselectionValues(x.ipselI);
    n+=printInIstatesPVValues(x.inistatesPV);
    n+=printDIstatesVTValues(x.distatesVT);
    n+=printDIstatesVTValues(x.distatesVTmin);
    n+=printDIselVTValues(x.diselVT);
    n+=printDIselVTsimpleValues(x.diselVTsimple);
    n+=printDIselPIVValues(x.diselPIV);
    n+=printDIminP2tmaxIValues(x.minp2tmaxI);
    n+=printMaxPnLratioValues(x.maxpnlratioI);
    n+=printMaxPnLratioValues(x.maxpnlratioV);
    n+=printMinIPlPVValues(x.minIPlPV);
    n+=printMinIPlPVValues(x.minVPlPI);
    n+=printMaxPPl2IinbasisValues(x.maxppl2iinbasis);
    n+=printMaxPPl2IValues(x.maxppl2);
    n+=printMaxPPl2IValues(x.maxppl2div);
    n+=printMaxPPl2IValues(x.maxppl2I);
    n+=printMaxPPl2IValues(x.maxppl2V);
    n+=printMaxPP2VIValues(x.maxpp2vi);
    n+=printMaxIP2ratioValues(x.maxip2I);
    n+=printMaxIP2ratioValues(x.maxip2V);
    n+=printDIminP2VmaxIValues(x.diminp2vmaxI);
    n+=printDIminP2VmaxIValues(x.diminp2vmaxVT);
    n+=printDIminP2maxIValues(x.diminP2maxI);
    n+=printDIminP2maxIValues(x.diminP2maxVT);
    n+=printDIminP2maxIValues(x.diminP2maxIT);
    n+=printDIminP2maxIValues(x.diminP2maxIV);
    n+=printMinMaxPnLratioNormValues(x.minpnlratioNI);
    n+=printMinMaxPnLratioNormValues(x.maxpnlratioNI);
    n+=printMinMaxPnLratioNormValues(x.minpnlratioNV);
    n+=printMinMaxPnLratioNormValues(x.maxpnlratioNV);

    return n;
}


private int printScalpedMaxIProjectionNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);

    n+=10;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".pt_average_",s,null);
    this.dattribs_names.addValue(prefix+".pi_average_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpdt_0_",s,null);
    this.dattribs_names.addValue(prefix+".p_0_",s,null);
    this.dattribs_names.addValue(prefix+".Fdt_",s,null);
    this.dattribs_names.addValue(prefix+".DIR_",s,null);
    this.dattribs_names.addValue(prefix+".DIRa_",s,null);

    return n;
}
private <T> int printScalpedMaxIProjectionValues(final ScalpedMaxIProjection<T> x)
{
    int n=0;
    n+=EVXData_Values(x.I);

    n+=10;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.pt_average);
    this.dattribs.addValue(x.pi_average);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.dpdt_0);
    this.dattribs.addValue(x.p_0);
    this.dattribs.addValue(x.Fdt);
    this.dattribs.addValue(x.DIR);
    this.dattribs.addValue(x.DIRa);

    return n;
}

private int printDynMicroImpactNames(final boolean useV,final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".dmiIH.I",s);
    n+=Skewness_Names(prefix+".dmiIH.SK_P_movaver",s);
    n+=Skewness_Names(prefix+".dmiIH.SK_P_IH",s);

    n+=14;
    this.dattribs_names.addValue(prefix+".dmiIH.p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.dpdt_orig_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.dp_wv_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.dp_wt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.DpV_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.DpT_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.sk_dp_wv_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.I_dPIL_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.p_dPIL_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.dPI_dPIL_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.I_dPIH_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.p_dPIH_",s,null);
    this.dattribs_names.addValue(prefix+".dmiIH.dPI_dPIH_",s,null);

    n+=11;
    this.dattribs_names.addValue(prefix+".mudmi.dp_mu_",s,null);
    this.dattribs_names.addValue(prefix+".mudmi.dpaver_mu_",s,null);
    this.dattribs_names.addValue(prefix+".mudmi.p_mu_",s,null);
    this.dattribs_names.addValue(prefix+".mudmi.paver_mu_",s,null);
    this.dattribs_names.addValue(prefix+".mudmi.mdp_mu_",s,null);
    this.dattribs_names.addValue(prefix+".mudmi.mdpaver_mu_",s,null);
    this.dattribs_names.addValue(prefix+".mudmi.mp_mu_",s,null);
    this.dattribs_names.addValue(prefix+".mudmi.mpaver_mu_",s,null);
    this.dattribs_names.addValue(prefix+".mudmi.dp_it_mu_",s,null);
    this.dattribs_names.addValue(prefix+".mudmi.ddp_it_mu_",s,null);
    this.dattribs_names.addValue(prefix+".mudmi.pcorr_mu_",s,null);

    n+=7;
    this.dattribs_names.addValue(prefix+".pdmi.p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".pdmi.p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pdmi.dpdt_orig_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pdmi.imL_",s,null);
    this.dattribs_names.addValue(prefix+".pdmi.p_imL_",s,null);
    this.dattribs_names.addValue(prefix+".pdmi.imH_",s,null);
    this.dattribs_names.addValue(prefix+".pdmi.p_imH_",s,null);

    n+=7;
    this.dattribs_names.addValue(prefix+".mu2dmi.p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".mu2dmi.p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".mu2dmi.pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".mu2dmi.pnl_ILtoIH_",s,null);
    this.dattribs_names.addValue(prefix+".mu2dmi.vol_pnl_",s,null);
    this.dattribs_names.addValue(prefix+".mu2dmi.muL.p_Mu_",s,null);
    this.dattribs_names.addValue(prefix+".mu2dmi.muH.p_Mu_",s,null);


    //n+=Skewness_Names(prefix+".mu2px.skmus",s);
    n+=2;
    this.dattribs_names.addValue(prefix+".mu2px.p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".mu2px.pt_IH_",s,null);
    n+=10;
    this.dattribs_names.addValue(prefix+".mu2pinil.betaW_",s,null);
    this.dattribs_names.addValue(prefix+".mu2pinil.betaA_",s,null);
    this.dattribs_names.addValue(prefix+".mu2pinil.betaM_",s,null);
    this.dattribs_names.addValue(prefix+".mu2pinil.qsk_",s,null);
    this.dattribs_names.addValue(prefix+".mu2pinil.sL_cpx_",s,null);
    this.dattribs_names.addValue(prefix+".mu2pinil.sH_cpx_",s,null);
    this.dattribs_names.addValue(prefix+".mu2pinil.dirLLt_",s,null);
    this.dattribs_names.addValue(prefix+".mu2pinil.dirLLi_",s,null);
    this.dattribs_names.addValue(prefix+".mu2pinil.dirLHt_",s,null);
    this.dattribs_names.addValue(prefix+".mu2pinil.dirLHi_",s,null);

    n+=14;
    this.dattribs_names.addValue(prefix+".inistatePI.p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.p1_IH_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.pd_IH_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.dp_a_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.dp_Ia_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.dir_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.pdir_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.dirPnL_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.dirPnLsk_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.dirsk2_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.dirHmu_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.dirHsk2_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.dirHlsk2_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePI.dirHsk2_l_",s,null);
    n+=Skewness_Names(prefix+".inistatePI.PnLsk",s);
    n+=Skewness_Names(prefix+".inistatePI.PnLskP",s);
    n+=Skewness_Names(prefix+".inistatePI.PnLskf",s);
    n+=Skewness_Names(prefix+".inistatePI.PnLskL",s);
    n+=Skewness_Names(prefix+".inistatePI.PnLskLP",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".inistatePIfF.dirp_",s,null);
    n+=Skewness_Names(prefix+".inistatePIfF.PnLsk",s);
    this.dattribs_names.addValue(prefix+".inistatePIfB.dirp_",s,null);
    n+=Skewness_Names(prefix+".inistatePIfB.PnLsk",s);
    this.dattribs_names.addValue(prefix+".inistatePIf2.dirp_",s,null);
    this.dattribs_names.addValue(prefix+".inistatePIf2.diri_",s,null);
    n+=Skewness_Names(prefix+".inistatePIf2.PnLsk",s);

    if(useV)
    {
	 n+=EVXData_Names(prefix+".dmvt.VT",s);
	 n+=7;
	 this.dattribs_names.addValue(prefix+".dmvt.p_offset_",s,null);
	 this.dattribs_names.addValue(prefix+".dmvt.pi_VTH_",s,null);
	 this.dattribs_names.addValue(prefix+".dmvt.DpV_VTH_",s,null);
	 this.dattribs_names.addValue(prefix+".dmvt.nDpV_VTH_",s,null);
	 this.dattribs_names.addValue(prefix+".dmvt.nDpT_VTH_",s,null);
	 this.dattribs_names.addValue(prefix+".dmvt.DpV_H_",s,null);
	 this.dattribs_names.addValue(prefix+".dmvt.DpV_L_",s,null);
	 n+=InIStateV_Names(prefix+".inistateV",s);
    }

    return n;
}
private <T> int printDynMicroImpactValues(final boolean useV,final DynMicroImpact<T> x)
{
    int n=0;
    n+=EVXData_Values(x.dmiIH.I);
    n+=Skewness_Values(x.dmiIH.SK_P_movaver);
    n+=Skewness_Values(x.dmiIH.SK_P_IH);

    n+=14;
    this.dattribs.addValue(x.dmiIH.p_offset);
    this.dattribs.addValue(x.dmiIH.pt_IH);
    this.dattribs.addValue(x.dmiIH.dpdt_orig_IH);
    this.dattribs.addValue(x.dmiIH.dp_wv_IH);
    this.dattribs.addValue(x.dmiIH.dp_wt_IH);
    this.dattribs.addValue(x.dmiIH.DpV_IH);
    this.dattribs.addValue(x.dmiIH.DpT_IH);
    this.dattribs.addValue(x.dmiIH.sk_dp_wv_IH);
    this.dattribs.addValue(x.dmiIH.I_dPIL);
    this.dattribs.addValue(x.dmiIH.p_dPIL);
    this.dattribs.addValue(x.dmiIH.dPI_dPIL);
    this.dattribs.addValue(x.dmiIH.I_dPIH);
    this.dattribs.addValue(x.dmiIH.p_dPIH);
    this.dattribs.addValue(x.dmiIH.dPI_dPIH);

    n+=11;
    this.dattribs.addValue(x.mudmi.dp_mu);
    this.dattribs.addValue(x.mudmi.dpaver_mu);
    this.dattribs.addValue(x.mudmi.p_mu);
    this.dattribs.addValue(x.mudmi.paver_mu);
    this.dattribs.addValue(x.mudmi.mdp_mu);
    this.dattribs.addValue(x.mudmi.mdpaver_mu);
    this.dattribs.addValue(x.mudmi.mp_mu);
    this.dattribs.addValue(x.mudmi.mpaver_mu);
    this.dattribs.addValue(x.mudmi.dp_it_mu);
    this.dattribs.addValue(x.mudmi.ddp_it_mu);
    this.dattribs.addValue(x.mudmi.pcorr_mu);

    n+=7;
    this.dattribs.addValue(x.pdmi.p_offset);
    this.dattribs.addValue(x.pdmi.p_IH);
    this.dattribs.addValue(x.pdmi.dpdt_orig_IH);
    this.dattribs.addValue(x.pdmi.imL);
    this.dattribs.addValue(x.pdmi.p_imL);
    this.dattribs.addValue(x.pdmi.imH);
    this.dattribs.addValue(x.pdmi.p_imH);

    n+=7;
    this.dattribs.addValue(x.mu2dmi.p_offset);
    this.dattribs.addValue(x.mu2dmi.p_IH);
    this.dattribs.addValue(x.mu2dmi.pt_IH);
    this.dattribs.addValue(x.mu2dmi.pnl_ILtoIH);
    this.dattribs.addValue(x.mu2dmi.vol_pnl);
    this.dattribs.addValue(x.mu2dmi.muL.p_Mu);
    this.dattribs.addValue(x.mu2dmi.muH.p_Mu);


    //n+=Skewness_Values(x.mu2px.skmus);
    n+=2;
    this.dattribs.addValue(x.mu2px.p_IH);
    this.dattribs.addValue(x.mu2px.pt_IH);
    n+=10;
    this.dattribs.addValue(x.mu2pinil.betaW);
    this.dattribs.addValue(x.mu2pinil.betaA);
    this.dattribs.addValue(x.mu2pinil.betaM);
    this.dattribs.addValue(x.mu2pinil.qsk);
    this.dattribs.addValue(x.mu2pinil.sL_cpx);
    this.dattribs.addValue(x.mu2pinil.sH_cpx);
    this.dattribs.addValue(x.mu2pinil.dirLLt);
    this.dattribs.addValue(x.mu2pinil.dirLLi);
    this.dattribs.addValue(x.mu2pinil.dirLHt);
    this.dattribs.addValue(x.mu2pinil.dirLHi);

    n+=14;
    this.dattribs.addValue(x.inistatePI.p_IH);
    this.dattribs.addValue(x.inistatePI.p1_IH);
    this.dattribs.addValue(x.inistatePI.pd_IH);
    this.dattribs.addValue(x.inistatePI.dp_a);
    this.dattribs.addValue(x.inistatePI.dp_Ia);
    this.dattribs.addValue(x.inistatePI.dir);
    this.dattribs.addValue(x.inistatePI.pdir);
    this.dattribs.addValue(x.inistatePI.dirPnL);
    this.dattribs.addValue(x.inistatePI.dirPnLsk);
    this.dattribs.addValue(x.inistatePI.dirsk2);
    this.dattribs.addValue(x.inistatePI.dirHmu);
    this.dattribs.addValue(x.inistatePI.dirHsk2);
    this.dattribs.addValue(x.inistatePI.dirHlsk2);
    this.dattribs.addValue(x.inistatePI.dirHsk2_l);
    n+=Skewness_Values(x.inistatePI.PnLsk);
    n+=Skewness_Values(x.inistatePI.PnLskP);
    n+=Skewness_Values(x.inistatePI.PnLskf);
    n+=Skewness_Values(x.inistatePI.PnLskL);
    n+=Skewness_Values(x.inistatePI.PnLskLP);

    n+=4;
    this.dattribs.addValue(x.inistatePIfF.dirp);
    n+=Skewness_Values(x.inistatePIfF.PnLsk);
    this.dattribs.addValue(x.inistatePIfB.dirp);
    n+=Skewness_Values(x.inistatePIfB.PnLsk);
    this.dattribs.addValue(x.inistatePIf2.dirp);
    this.dattribs.addValue(x.inistatePIf2.diri);
    n+=Skewness_Values(x.inistatePIf2.PnLsk);

    if(useV)
    {
	n+=EVXData_Values(x.dmvt.VT);
	n+=7;
	this.dattribs.addValue(x.dmvt.p_offset);
	this.dattribs.addValue(x.dmvt.pi_VTH);
	this.dattribs.addValue(x.dmvt.DpV_VTH);
	this.dattribs.addValue(x.dmvt.nDpV_VTH);
	this.dattribs.addValue(x.dmvt.nDpT_VTH);
	this.dattribs.addValue(x.dmvt.DpV_L);
	this.dattribs.addValue(x.dmvt.DpV_H);
	n+=InIStateV_Values(x.inistateV);
    }

    return n;
}
private int printPkxVolNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".P2",s);

    n+=2;
    this.dattribs_names.addValue(prefix+".x_",s,null);
    this.dattribs_names.addValue(prefix+".varX_",s,null);

    return n;
}
private int printPkxVolValues(final PkxVol x,final CalculationData c)
{
    int n=0;

    n+=EVXData_Values(x.P2);

    n+=2;
    this.dattribs.addValue(x.x);
    this.dattribs.addValue(x.varX);

    return n;
}


private int printPkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".P2t",s);

    n+=4;
    this.dattribs_names.addValue(prefix+".x_",s,null);
    this.dattribs_names.addValue(prefix+".Vs_",s,null);
    this.dattribs_names.addValue(prefix+".V_",s,null);
    this.dattribs_names.addValue(prefix+".sens_",s,null);

    n+=printPkxVolNames(prefix+".PVt",s);
    n+=printPkxVolNames(prefix+".PVi",s);


    return n;
}
private <T> int printPkQVMValues(final PkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.P2t);

    n+=4;
    this.dattribs.addValue(x.x);
    this.dattribs.addValue(x.Vs);
    this.dattribs.addValue(x.V);
    this.dattribs.addValue(x.sens);

    n+=printPkxVolValues(x.PVt,c);
    n+=printPkxVolValues(x.PVi,c);

    return n;
}




private int printRkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXSpectrum_Names(prefix+".PA",s);
    n+=Skewness_Names(prefix+".sk",s);
    n+=3;
    this.dattribs_names.addValue(prefix+".plo_",s,null);
    this.dattribs_names.addValue(prefix+".Pi_",s,null);
    this.dattribs_names.addValue(prefix+".Pt_",s,null);
    return n;
}
private <T> int printRkQVMValues(final RkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=EVXSpectrum_Values(x.PA);
    n+=Skewness_Values(x.sk);
    n+=3;
    this.dattribs.addValue(x.plo);
    this.dattribs.addValue(x.Pi);
    this.dattribs.addValue(x.Pt);
    return n;
}

private int printDynRkiNames(final String prefix,final String s)
{
    int n=0;

    n+=7;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".ps_IH_",s,null);
    this.dattribs_names.addValue(prefix+".vola_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pm_IH_",s,null);
    n+=Skewness_Names(prefix+".sk",s);

    return n;
}

private int printDynRkiValues(final DynRki x)
{
    int n=0;

    n+=7;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.ps_IH);
    this.dattribs.addValue(x.vola_IH);
    this.dattribs.addValue(x.pm_IH);
    n+=Skewness_Values(x.sk);

    return n;
}

private int printDynSkiNames(final String prefix,final String s)
{
    int n=0;

    n+=2;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    n+=Skewness_Names(prefix+".sk",s);

    return n;
}

private int printDynSkiValues(final DynSki x)
{
    int n=0;

    n+=2;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_IH);
    n+=Skewness_Values(x.sk);

    return n;
}

private int printDynVarCDNames(final String prefix,final String s)
{
    int n=0;

    n+=3;
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".X_IX_",s,null);
    n+=EVXData_Names(prefix+".I",s);
    n+=printACorrNames(prefix+".a",s);

    return n;
}

private int printDynVarCDValues(final DynVarCD x)
{
    int n=0;

    n+=3;
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.X_IX);
    n+=EVXData_Values(x.I);
    n+=printACorrValues(x.a);

    return n;
}
private int printDynVarMuNames(final String prefix,final String s)
{
    int n=0;

    n+=10;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".p_ia_",s,null);
    this.dattribs_names.addValue(prefix+".p_va_",s,null);
    this.dattribs_names.addValue(prefix+".i_max_",s,null);
    this.dattribs_names.addValue(prefix+".p_i_max_",s,null);
    this.dattribs_names.addValue(prefix+".x_i_max_",s,null);
    this.dattribs_names.addValue(prefix+".i_c_",s,null);
    this.dattribs_names.addValue(prefix+".p_i_c_",s,null);
    this.dattribs_names.addValue(prefix+".x_i_c_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printDynVarMuValues(final DynVarMu x)
{
    int n=0;

    n+=10;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.p_ia);
    this.dattribs.addValue(x.p_va);
    this.dattribs.addValue(x.i_max);
    this.dattribs.addValue(x.p_i_max);
    this.dattribs.addValue(x.x_i_max);
    this.dattribs.addValue(x.i_c);
    this.dattribs.addValue(x.p_i_c);
    this.dattribs.addValue(x.x_i_c);
    n+=EVXData_Values(x.I);

    return n;
}
private int printDynVarMuXNames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".xenter_",s,null);
    this.dattribs_names.addValue(prefix+".penter_",s,null);
    this.dattribs_names.addValue(prefix+".ienter_",s,null);
    this.dattribs_names.addValue(prefix+".xexit_",s,null);
    this.dattribs_names.addValue(prefix+".pexit_",s,null);
    this.dattribs_names.addValue(prefix+".iexit_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printDynVarMuXValues(final DynVarMuX x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.xenter);
    this.dattribs.addValue(x.penter);
    this.dattribs.addValue(x.ienter);
    this.dattribs.addValue(x.xexit);
    this.dattribs.addValue(x.pexit);
    this.dattribs.addValue(x.iexit);
    n+=EVXData_Values(x.I);

    return n;
}
private int printDynYNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pd_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Id_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_JL_",s,null);
    this.dattribs_names.addValue(prefix+".r_JL_",s,null);
    this.dattribs_names.addValue(prefix+".p_JH_",s,null);
    this.dattribs_names.addValue(prefix+".r_JH_",s,null);
    this.dattribs_names.addValue(prefix+".p_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".p_DIH_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printDynYValues(final DynY x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pd_IH);
    this.dattribs.addValue(x.Id_IH);
    this.dattribs.addValue(x.p_JL);
    this.dattribs.addValue(x.r_JL);
    this.dattribs.addValue(x.p_JH);
    this.dattribs.addValue(x.r_JH);
    this.dattribs.addValue(x.p_DIL);
    this.dattribs.addValue(x.p_DIH);
    n+=EVXData_Values(x.I);

    return n;
}
private int printDynYpNames(final String prefix,final String s)
{
    int n=0;

    n+=7;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pd_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IJp_",s,null);
    this.dattribs_names.addValue(prefix+".pd_IJp_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_pt_IJp_",s,null);
    this.dattribs_names.addValue(prefix+".var_pt2_IJp_",s,null);
    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".Ip",s);

    return n;
}
private int printDynYpValues(final DynYp x)
{
    int n=0;

    n+=7;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pd_IH);
    this.dattribs.addValue(x.p_IJp);
    this.dattribs.addValue(x.pd_IJp);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.var_pt_IJp);
    this.dattribs.addValue(x.var_pt2_IJp);
    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.Ip);

    return n;
}

private int printDynYpNoIterNames(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_pp_IpH_",s,null);
    this.dattribs_names.addValue(prefix+".varp_pp_IpH_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printDynYpNoIterValues(final DynYpNoIter x)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.var_pp_IpH);
    this.dattribs.addValue(x.varp_pp_IpH);
    n+=EVXData_Values(x.I);

    return n;
}

private int printDynVH2PnLNames(final String prefix,final String s)
{
    int n=0;

    n+=11;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var2tt_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".var2tt_pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".var2tt0_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".var2tt0_pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".var2pt_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".var2pt_pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".pnl0_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printDynVH2PnLValues(final DynVH2PnL x)
{
    int n=0;

    n+=11;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.var2tt_pnlL);
    this.dattribs.addValue(x.var2tt_pnlH);
    this.dattribs.addValue(x.var2tt0_pnlL);
    this.dattribs.addValue(x.var2tt0_pnlH);
    this.dattribs.addValue(x.var2pt_pnlL);
    this.dattribs.addValue(x.var2pt_pnlH);
    this.dattribs.addValue(x.pnlL);
    this.dattribs.addValue(x.pnlH);
    this.dattribs.addValue(x.pnl0);
    n+=EVXData_Values(x.I);

    return n;
}

private int printDynVarPnLSpurNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".sumvar2PnL_",s,null);
    this.dattribs_names.addValue(prefix+".sumPnL_",s,null);
    this.dattribs_names.addValue(prefix+".sumPnLProjIH_",s,null);
    this.dattribs_names.addValue(prefix+".sumv2PnLProjIH_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printDynVarPnLSpurValues(final DynVarPnLSpur x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.sumvar2PnL);
    this.dattribs.addValue(x.sumPnL);
    this.dattribs.addValue(x.sumPnLProjIH);
    this.dattribs.addValue(x.sumv2PnLProjIH);

    n+=EVXData_Values(x.I);

    return n;
}

private int printDynVar2DiagProjNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pI_dI_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_dI_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var2_pI_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_var2_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_var2_A_",s,null);
    this.dattribs_names.addValue(prefix+".p_A_I0_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_A_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printDynVar2DiagProjValues(final DynVar2DiagProj x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var_pI_IH);
    this.dattribs.addValue(x.pI_dI_IH);
    this.dattribs.addValue(x.p_dI_IH);
    this.dattribs.addValue(x.var2_pI_IH);
    this.dattribs.addValue(x.p_var2_IH);
    this.dattribs.addValue(x.p_var2_A);
    this.dattribs.addValue(x.p_A_I0);
    this.dattribs.addValue(x.var_pI_A);

    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLMuNames(final String prefix,final String s)
{
    int n=0;

    n+=2;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".p_wmu_",s,null);
    this.dattribs_names.addValue(prefix+".p_imu_",s,null);
    this.dattribs_names.addValue(prefix+".p_vm_",s,null);
    this.dattribs_names.addValue(prefix+".mpnlmin_",s,null);
    this.dattribs_names.addValue(prefix+".mpnlmax_",s,null);
    this.dattribs_names.addValue(prefix+".mpnlsum_",s,null);
    n+=4;
    this.dattribs_names.addValue(prefix+".pnlmin_",s,null);
    this.dattribs_names.addValue(prefix+".pnlmax_",s,null);
    this.dattribs_names.addValue(prefix+".x_pnlmin_",s,null);
    this.dattribs_names.addValue(prefix+".x_pnlmax_",s,null);
/*
    this.dattribs_names.addValue(prefix+".mingipnl_",s,null);
    this.dattribs_names.addValue(prefix+".maxgipnl_",s,null);
    this.dattribs_names.addValue(prefix+".mingwpnl_",s,null);
    this.dattribs_names.addValue(prefix+".maxgwpnl_",s,null);
    this.dattribs_names.addValue(prefix+".minpgipnl_",s,null);
    this.dattribs_names.addValue(prefix+".maxpgipnl_",s,null);
    this.dattribs_names.addValue(prefix+".minpgwpnl_",s,null);
    this.dattribs_names.addValue(prefix+".maxpgwpnl_",s,null);
    */

    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLMuValues(final PnLMu x)
{
    int n=0;

    n+=2;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_IL);

    n+=6;
    this.dattribs.addValue(x.p_wmu);
    this.dattribs.addValue(x.p_imu);
    this.dattribs.addValue(x.p_vm);
    this.dattribs.addValue(x.mpnlmin);
    this.dattribs.addValue(x.mpnlmax);
    this.dattribs.addValue(x.mpnlsum);
    n+=4;
    this.dattribs.addValue(x.pnlmin);
    this.dattribs.addValue(x.pnlmax);
    this.dattribs.addValue(x.x_pnlmin);
    this.dattribs.addValue(x.x_pnlmax);
    /*
    this.dattribs.addValue(x.mingipnl);
    this.dattribs.addValue(x.maxgipnl);
    this.dattribs.addValue(x.mingwpnl);
    this.dattribs.addValue(x.maxgwpnl);
    this.dattribs.addValue(x.minpgipnl);
    this.dattribs.addValue(x.maxpgipnl);
    this.dattribs.addValue(x.minpgwpnl);
    this.dattribs.addValue(x.maxpgwpnl);
    */

    n+=EVXData_Values(x.I);

    return n;
}



private int printPnLVHNames(final String prefix,final String s)
{
    int n=0;

    n+=15;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".varpI0_",s,null);
    this.dattribs_names.addValue(prefix+".p_iSel_",s,null);
    this.dattribs_names.addValue(prefix+".p_LR_aver_",s,null);
    this.dattribs_names.addValue(prefix+".pV_iSel_",s,null);
    this.dattribs_names.addValue(prefix+".V_iSel_",s,null);
    this.dattribs_names.addValue(prefix+".p_left_",s,null);
    this.dattribs_names.addValue(prefix+".pV_left_",s,null);
    this.dattribs_names.addValue(prefix+".V_left_",s,null);
    this.dattribs_names.addValue(prefix+".p_right_",s,null);
    this.dattribs_names.addValue(prefix+".pV_right_",s,null);
    this.dattribs_names.addValue(prefix+".V_right_",s,null);
    this.dattribs_names.addValue(prefix+".PnLv_",s,null);
    this.dattribs_names.addValue(prefix+".PnL3_",s,null);
    this.dattribs_names.addValue(prefix+".pP_",s,null);

    n+=EVXData_Names(prefix+".I",s);
    n+=printACorrNames(prefix+".PnL_I",s);

    return n;
}
private int printPnLVHValues(final PnLVH x)
{
    int n=0;

    n+=15;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.varpI0);
    this.dattribs.addValue(x.p_iSel);
    this.dattribs.addValue(x.p_LR_aver);
    this.dattribs.addValue(x.pV_iSel);
    this.dattribs.addValue(x.V_iSel);
    this.dattribs.addValue(x.p_left);
    this.dattribs.addValue(x.pV_left);
    this.dattribs.addValue(x.V_left);
    this.dattribs.addValue(x.p_right);
    this.dattribs.addValue(x.pV_right);
    this.dattribs.addValue(x.V_right);
    this.dattribs.addValue(x.PnLv);
    this.dattribs.addValue(x.PnL3);
    this.dattribs.addValue(x.pP);

    n+=EVXData_Values(x.I);
    n+=printACorrValues(x.PnL_I);

    return n;
}

private int printPnLI0Names(final String prefix,final String s)
{
    int n=0;

    n+=13;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_eq_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_eq_",s,null);
    this.dattribs_names.addValue(prefix+".rf_",s,null);
    this.dattribs_names.addValue(prefix+".P_eq_",s,null);
    this.dattribs_names.addValue(prefix+".P_phi0_",s,null);
    this.dattribs_names.addValue(prefix+".P1_eq_",s,null);
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".Peqq_",s,null);
    this.dattribs_names.addValue(prefix+".PeqqdI_",s,null);

    return n;
}
private int printPnLI0Values(final PnLI0 x)
{
    int n=0;

    n+=13;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.I_0_eq);
    this.dattribs.addValue(x.I_H_eq);
    this.dattribs.addValue(x.rf);
    this.dattribs.addValue(x.P_eq);
    this.dattribs.addValue(x.P_phi0);
    this.dattribs.addValue(x.P1_eq);
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.Peqq);
    this.dattribs.addValue(x.PeqqdI);

    return n;
}

    private int printPnLI0BNames(final String prefix,final String s)
{
    int n=0;

    n+=5;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pb_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pb_IHb_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_IH_",s,null);
    this.dattribs_names.addValue(prefix+".P0f_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLI0BValues(final PnLI0B x)
{
    int n=0;

    n+=5;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pb_IH);
    this.dattribs.addValue(x.pb_IHb);
    this.dattribs.addValue(x.Pf_IH);
    this.dattribs.addValue(x.P0f);
    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLI0eNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".PeqqdI_",s,null);
    this.dattribs_names.addValue(prefix+".Peqq_",s,null);
    this.dattribs_names.addValue(prefix+".PfdI_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLI0eValues(final PnLI0e x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.PeqqdI);
    this.dattribs.addValue(x.Peqq);
    this.dattribs.addValue(x.PfdI);
    this.dattribs.addValue(x.Pf);
    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLI0IHNames(final String prefix,final String s)
{
    int n=0;

    n+=5;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".PfdI_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    this.dattribs_names.addValue(prefix+".Pfi_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLI0IHValues(final PnLI0IH x)
{
    int n=0;

    n+=5;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.PfdI);
    this.dattribs.addValue(x.Pf);
    this.dattribs.addValue(x.Pfi);
    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLbI0Names(final String prefix,final String s)
{
    int n=0;

    n+=17;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".P_eq_IHI0_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    this.dattribs_names.addValue(prefix+".Pfdynimp_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_psi0_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_psiIH_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_eq_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_eq_",s,null);
    this.dattribs_names.addValue(prefix+".P_eq_",s,null);
    this.dattribs_names.addValue(prefix+".Pw_eq_",s,null);
    this.dattribs_names.addValue(prefix+".Pe_eq_",s,null);
    this.dattribs_names.addValue(prefix+".P_phi0_",s,null);
    this.dattribs_names.addValue(prefix+".P1_eq_",s,null);
    this.dattribs_names.addValue(prefix+".fact_dI_",s,null);
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".C_enter_",s,null);
    this.dattribs_names.addValue(prefix+".C_exit_",s,null);

    return n;
}
private int printPnLbI0Values(final PnLbI0 x)
{
    int n=0;

    n+=17;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.P_eq_IHI0);
    this.dattribs.addValue(x.Pf);
    this.dattribs.addValue(x.Pfdynimp);
    this.dattribs.addValue(x.Pf_psi0);
    this.dattribs.addValue(x.Pf_psiIH);
    this.dattribs.addValue(x.I_0_eq);
    this.dattribs.addValue(x.I_H_eq);
    this.dattribs.addValue(x.P_eq);
    this.dattribs.addValue(x.Pw_eq);
    this.dattribs.addValue(x.Pe_eq);
    this.dattribs.addValue(x.P_phi0);
    this.dattribs.addValue(x.P1_eq);
    this.dattribs.addValue(x.fact_dI);
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.C_enter);
    this.dattribs.addValue(x.C_exit);

    return n;
}

private int printPnLdIf0Names(final String prefix,final String s)
{
    int n=0;

    n+=35;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".P_movaver_",s,null);
    this.dattribs_names.addValue(prefix+".P_movaver_spur_",s,null);
    this.dattribs_names.addValue(prefix+".dP_movaver_",s,null);
    this.dattribs_names.addValue(prefix+".dP_movaver_spur_",s,null);
    this.dattribs_names.addValue(prefix+".Pa_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".A_PfdI_",s,null);
    this.dattribs_names.addValue(prefix+".A_Pf_",s,null);
    this.dattribs_names.addValue(prefix+".A_Pb_",s,null);
    this.dattribs_names.addValue(prefix+".A_PbI_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".p_0f_",s,null);
    this.dattribs_names.addValue(prefix+".pd_f_",s,null);
    this.dattribs_names.addValue(prefix+".P_IH_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".P1b_",s,null);
    this.dattribs_names.addValue(prefix+".p_E_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".p_E_0_",s,null);
    this.dattribs_names.addValue(prefix+".P1_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".P2_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".P1_IH_",s,null);
    this.dattribs_names.addValue(prefix+".P2_IH_",s,null);
    this.dattribs_names.addValue(prefix+".PfdI_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    this.dattribs_names.addValue(prefix+".PfadI_",s,null);
    this.dattribs_names.addValue(prefix+".Pfa_",s,null);
    this.dattribs_names.addValue(prefix+".P0f_",s,null);
    this.dattribs_names.addValue(prefix+".P0lf_",s,null);
    this.dattribs_names.addValue(prefix+".dC0lf_",s,null);
    this.dattribs_names.addValue(prefix+".dP0f_",s,null);
    this.dattribs_names.addValue(prefix+".P0ff_",s,null);
    this.dattribs_names.addValue(prefix+".P0ffi_",s,null);
    this.dattribs_names.addValue(prefix+".dPf_0_",s,null);
    this.dattribs_names.addValue(prefix+".dP_w0_",s,null);
    this.dattribs_names.addValue(prefix+".dC_w0_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLdIf0Values(final PnLdIf0 x)
{
    int n=0;

    n+=35;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.P_movaver);
    this.dattribs.addValue(x.P_movaver_spur);
    this.dattribs.addValue(x.dP_movaver);
    this.dattribs.addValue(x.dP_movaver_spur);
    this.dattribs.addValue(x.Pa);
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.A_PfdI);
    this.dattribs.addValue(x.A_Pf);
    this.dattribs.addValue(x.A_Pb);
    this.dattribs.addValue(x.A_PbI);
    this.dattribs.addValue(x.p_IHf);
    this.dattribs.addValue(x.p_0f);
    this.dattribs.addValue(x.pd_f);
    this.dattribs.addValue(x.P_IH_IHf);
    this.dattribs.addValue(x.P1b);
    this.dattribs.addValue(x.p_E_IHf);
    this.dattribs.addValue(x.p_E_0);
    this.dattribs.addValue(x.P1_IHf);
    this.dattribs.addValue(x.P2_IHf);
    this.dattribs.addValue(x.P1_IH);
    this.dattribs.addValue(x.P2_IH);
    this.dattribs.addValue(x.PfdI);
    this.dattribs.addValue(x.Pf);
    this.dattribs.addValue(x.PfadI);
    this.dattribs.addValue(x.Pfa);
    this.dattribs.addValue(x.P0f);
    this.dattribs.addValue(x.P0lf);
    this.dattribs.addValue(x.dC0lf);
    this.dattribs.addValue(x.dP0f);
    this.dattribs.addValue(x.P0ff);
    this.dattribs.addValue(x.P0ffi);
    this.dattribs.addValue(x.dPf_0);
    this.dattribs.addValue(x.dP_w0);
    this.dattribs.addValue(x.dC_w0);

    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLdIdPNames(final String prefix,final String s)
{
    int n=0;

    n+=16;
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".dp_0_",s,null);
    this.dattribs_names.addValue(prefix+".P_movaver_",s,null);
    this.dattribs_names.addValue(prefix+".P_movaver_spur_",s,null);
    this.dattribs_names.addValue(prefix+".dP_movaver_",s,null);
    this.dattribs_names.addValue(prefix+".dP_movaver_spur_",s,null);
    this.dattribs_names.addValue(prefix+".dP_wIH_",s,null);
    this.dattribs_names.addValue(prefix+".dP_w0_",s,null);
    this.dattribs_names.addValue(prefix+".dC_w0_",s,null);
    this.dattribs_names.addValue(prefix+".dPf_0_",s,null);
    this.dattribs_names.addValue(prefix+".dP_IH_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".pp_IHf_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLdIdPValues(final PnLdIdP x)
{
    int n=0;

    n+=16;
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_IHf);
    this.dattribs.addValue(x.dp_IH);
    this.dattribs.addValue(x.dp_IHf);
    this.dattribs.addValue(x.dp_0);
    this.dattribs.addValue(x.P_movaver);
    this.dattribs.addValue(x.P_movaver_spur);
    this.dattribs.addValue(x.dP_movaver);
    this.dattribs.addValue(x.dP_movaver_spur);
    this.dattribs.addValue(x.dP_wIH);
    this.dattribs.addValue(x.dP_w0);
    this.dattribs.addValue(x.dC_w0);
    this.dattribs.addValue(x.dPf_0);
    this.dattribs.addValue(x.dP_IH_IHf);
    this.dattribs.addValue(x.pp_IHf);

    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLdISNames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".P_movaver_",s,null);
    this.dattribs_names.addValue(prefix+".P_movaver_spur_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHf_nodI_",s,null);
    this.dattribs_names.addValue(prefix+".w_IHf_0_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_0_",s,null);

    n+=Skewness_Names(prefix+".SP",s);
    n+=Skewness_Names(prefix+".SdP",s);
    n+=Skewness_Names(prefix+".SPPL",s);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLdISValues(final PnLdIS x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.P_movaver);
    this.dattribs.addValue(x.P_movaver_spur);
    this.dattribs.addValue(x.p_IHf_nodI);
    this.dattribs.addValue(x.w_IHf_0);
    this.dattribs.addValue(x.dp_orig_IH);
    this.dattribs.addValue(x.dp_orig_0);

    n+=Skewness_Values(x.SP);
    n+=Skewness_Values(x.SdP);
    n+=Skewness_Values(x.SPPL);
    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLdISkNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_movaver_spur_",s,null);
    this.dattribs_names.addValue(prefix+".P_movaver_spur_",s,null);
    this.dattribs_names.addValue(prefix+".w_IHf_0_",s,null);
    this.dattribs_names.addValue(prefix+".w_IHf_",s,null);

    n+=Skewness_Names(prefix+".SK_IH",s);
    n+=Skewness_Names(prefix+".SK_IHf",s);
    n+=Skewness_Names(prefix+".SK_IH_V",s);
    n+=Skewness_Names(prefix+".SK_IHf_V",s);
    n+=Skewness_Names(prefix+".SK_IHf__nodI",s);
    n+=Skewness_Names(prefix+".SK_movaver",s);
    n+=Skewness_Names(prefix+".SK_IHf_W0",s);
    n+=Skewness_Names(prefix+".SK_IHf_WS",s);
    n+=Skewness_Names(prefix+".SK_IHf_Pfeq",s);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLdISkValues(final PnLdISk x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_movaver_spur);
    this.dattribs.addValue(x.P_movaver_spur);
    this.dattribs.addValue(x.w_IHf_0);
    this.dattribs.addValue(x.w_IHf);

    n+=Skewness_Values(x.SK_IH);
    n+=Skewness_Values(x.SK_IHf);
    n+=Skewness_Values(x.SK_IH_V);
    n+=Skewness_Values(x.SK_IHf_V);
    n+=Skewness_Values(x.SK_IHf__nodI);
    n+=Skewness_Values(x.SK_movaver);
    n+=Skewness_Values(x.SK_IHf_W0);
    n+=Skewness_Values(x.SK_IHf_WS);
    n+=Skewness_Values(x.SK_IHf_Pfeq);
    n+=EVXData_Values(x.I);

    return n;
}


private int printPnLdIV4Names(final String prefix,final String s)
{
    int n=0;

    n+=1;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);

    n+=Skewness_Names(prefix+".SK_init",s);
    n+=Skewness_Names(prefix+".SK_iter",s);
    n+=Skewness_Names(prefix+".SK_spur",s);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLdIV4Values(final PnLdIV4 x)
{
    int n=0;

    n+=1;
    this.dattribs.addValue(x.p_offset);

    n+=Skewness_Values(x.SK_init);
    n+=Skewness_Values(x.SK_iter);
    n+=Skewness_Values(x.SK_spur);

    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLdISkewnessNames(final String prefix,final String s)
{
    int n=0;

    n+=2;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);

    n+=Skewness_Names(prefix+".IHf_S",s);
    n+=Skewness_Names(prefix+".W0pi_S",s);
    n+=Skewness_Names(prefix+".W0pt_S",s);
    n+=Skewness_Names(prefix+".movaver_S",s);
    n+=Skewness_Names(prefix+".spur_S",s);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printPnLdISkewnessValues(final PnLdISkewness x)
{
    int n=0;

    n+=2;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);

    n+=Skewness_Values(x.IHf_S);
    n+=Skewness_Values(x.W0pi_S);
    n+=Skewness_Values(x.W0pt_S);
    n+=Skewness_Values(x.movaver_S);
    n+=Skewness_Values(x.spur_S);
    n+=EVXData_Values(x.I);

    return n;
}


private int printPnLdIP(final String prefix,final String s)
{
    int n=0;

    n+=5;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_p1_",s,null);
    this.dattribs_names.addValue(prefix+".I_p2_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feqF_",s,null);

    n+=Skewness_Names(prefix+".IHf_S",s);
    n+=Skewness_Names(prefix+".IHf_St",s);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printPnLdIP(final PnLdIP x)
{
    int n=0;

    n+=5;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_p1);
    this.dattribs.addValue(x.I_p2);
    this.dattribs.addValue(x.I_0_feqF);

    n+=Skewness_Values(x.IHf_S);
    n+=Skewness_Values(x.IHf_St);
    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLdIpxNames(final String prefix,final String s)
{
    int n=0;

    n+=19;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_from_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_from_f0_",s,null);
    this.dattribs_names.addValue(prefix+".deg_from_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".Pft_from_f0_",s,null);
    this.dattribs_names.addValue(prefix+".degt_from_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".delta_Pdyn_from_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".delta_P_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".P_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".P_IHf_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".deg_BIH_",s,null);
    this.dattribs_names.addValue(prefix+".P_BIH_",s,null);
    this.dattribs_names.addValue(prefix+".P_B0_",s,null);
    this.dattribs_names.addValue(prefix+".P_B0d_",s,null);
    this.dattribs_names.addValue(prefix+".deg2_Hf_",s,null);
    this.dattribs_names.addValue(prefix+".Pf2_Hf_",s,null);
    this.dattribs_names.addValue(prefix+".Pf2o_Hf_",s,null);
    this.dattribs_names.addValue(prefix+".Pf2os_Hf_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printPnLdIpxValues(final PnLdIpx x)
{
    int n=0;

    n+=19;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.Pf_from_dyn);
    this.dattribs.addValue(x.Pf_from_f0);
    this.dattribs.addValue(x.deg_from_dyn);
    this.dattribs.addValue(x.Pft_from_f0);
    this.dattribs.addValue(x.degt_from_dyn);
    this.dattribs.addValue(x.delta_Pdyn_from_dyn);
    this.dattribs.addValue(x.delta_P_IHf);
    this.dattribs.addValue(x.P_IHf);
    this.dattribs.addValue(x.P_IHf_dyn);
    this.dattribs.addValue(x.deg_BIH);
    this.dattribs.addValue(x.P_BIH);
    this.dattribs.addValue(x.P_B0);
    this.dattribs.addValue(x.P_B0d);
    this.dattribs.addValue(x.deg2_Hf);
    this.dattribs.addValue(x.Pf2_Hf);
    this.dattribs.addValue(x.Pf2o_Hf);
    this.dattribs.addValue(x.Pf2os_Hf);

    n+=EVXData_Values(x.I);

    return n;
}


private int printPnLdISpurNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".degt_",s,null);
    this.dattribs_names.addValue(prefix+".Pft_from_f0_",s,null);
    this.dattribs_names.addValue(prefix+".deg_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_from_f0_",s,null);
    this.dattribs_names.addValue(prefix+".we_xx_",s,null);
    this.dattribs_names.addValue(prefix+".wp_xx_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_from_xx_",s,null);

    n+=Skewness_Names(prefix+".PnLsk.sknodi",s);
    n+=Skewness_Names(prefix+".W0_SK",s);
    n+=Skewness_Names(prefix+".W0w_SK",s);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printPnLdISpurValues(final PnLdISpur x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.degt);
    this.dattribs.addValue(x.Pft_from_f0);
    this.dattribs.addValue(x.deg);
    this.dattribs.addValue(x.Pf_from_f0);
    this.dattribs.addValue(x.we_xx);
    this.dattribs.addValue(x.wp_xx);
    this.dattribs.addValue(x.Pf_from_xx);

    n+=Skewness_Values(x.PnLsk.sknodi);
    n+=Skewness_Values(x.W0_SK);
    n+=Skewness_Values(x.W0w_SK);
    n+=EVXData_Values(x.I);

    return n;
}


private int printMicroPnLdINames(final String prefix,final String s)
{
    int n=0;

    n+=7;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".g_IH_",s,null);
    this.dattribs_names.addValue(prefix+".g_0_",s,null);
    this.dattribs_names.addValue(prefix+".deg_from_pt_true_pi_",s,null);
    this.dattribs_names.addValue(prefix+".gf_from_pt_true_pi_",s,null);
    this.dattribs_names.addValue(prefix+".deg_from_dyn_",s,null);
    this.dattribs_names.addValue(prefix+".gf_from_f0_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printMicroPnLdIValues(final MicroPnLdI x)
{
    int n=0;

    n+=7;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.g_IH);
    this.dattribs.addValue(x.g_0);
    this.dattribs.addValue(x.deg_from_pt_true_pi);
    this.dattribs.addValue(x.gf_from_pt_true_pi);
    this.dattribs.addValue(x.deg_from_dyn);
    this.dattribs.addValue(x.gf_from_f0);

    n+=EVXData_Values(x.I);

    return n;
}


private int printPnLdIDSkNames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dI_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_from_ptpi_",s,null);
    this.dattribs_names.addValue(prefix+".deg_from_pt_true_pi_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_from_pt_true_pi_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_from_f0_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_from_dyn_",s,null);

    n+=EVXData_Names(prefix+".I",s);
    n+=Skewness_Names(prefix+".SK_movaver__nodI",s);
    n+=Skewness_Names(prefix+".SK_spur__nodI",s);

    return n;
}

private int printPnLdIDSkValues(final PnLdIDSk x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.dI);
    this.dattribs.addValue(x.Pf_from_ptpi);
    this.dattribs.addValue(x.deg_from_pt_true_pi);
    this.dattribs.addValue(x.Pf_from_pt_true_pi);
    this.dattribs.addValue(x.Pf_from_f0);
    this.dattribs.addValue(x.Pf_from_dyn);

    n+=EVXData_Values(x.I);
    n+=Skewness_Values(x.SK_movaver__nodI);
    n+=Skewness_Values(x.SK_spur__nodI);

    return n;
}

private int printPnLdIdINames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dI_",s,null);
    this.dattribs_names.addValue(prefix+".dIF_",s,null);
    this.dattribs_names.addValue(prefix+".sDir_",s,null);
    this.dattribs_names.addValue(prefix+".p_MUSE_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printPnLdIdIValues(final PnLdIdI x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_0_feq-x.I.s0);
    this.dattribs.addValue(x.I_0_feqF-x.I_0_feq);
    this.dattribs.addValue(x.sDir);
    this.dattribs.addValue(x.p_MUSE);

    n+=EVXData_Values(x.I);

    return n;
}

private int printDynVar2DISkewnessNames(final String prefix,final String s)
{
    int n=0;

    n+=15;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dI_",s,null);
    this.dattribs_names.addValue(prefix+".IHf_w_",s,null);
    this.dattribs_names.addValue(prefix+".ILf_w_",s,null);
    this.dattribs_names.addValue(prefix+".DIHf_w_",s,null);
    this.dattribs_names.addValue(prefix+".DIhaf_w_",s,null);
    this.dattribs_names.addValue(prefix+".DILf_w_",s,null);
    this.dattribs_names.addValue(prefix+".p_DI_",s,null);
    this.dattribs_names.addValue(prefix+".sk_DI_",s,null);
    this.dattribs_names.addValue(prefix+".pd_DI_",s,null);
    this.dattribs_names.addValue(prefix+".pd_DII_",s,null);
    this.dattribs_names.addValue(prefix+".pd_DI0_",s,null);
    this.dattribs_names.addValue(prefix+".pd_DIDp_",s,null);
    this.dattribs_names.addValue(prefix+".dpd_DI_",s,null);

    n+=EVXData_Names(prefix+".I",s);
    n+=Skewness_Names(prefix+".IHf_S",s);
    n+=Skewness_Names(prefix+".IHf_S__nodI",s);
    n+=Skewness_Names(prefix+".ILf_S",s);
    n+=Skewness_Names(prefix+".ILf_S__nodI",s);
    n+=Skewness_Names(prefix+".DIHf_S",s);
    n+=Skewness_Names(prefix+".DIHf_S__nodI",s);
    n+=Skewness_Names(prefix+".DIhaf_S",s);
    n+=Skewness_Names(prefix+".DIhaf_S__nodI",s);
    n+=Skewness_Names(prefix+".DILf_S",s);
    n+=Skewness_Names(prefix+".DILf_S__nodI",s);
    n+=Skewness_Names(prefix+".DIIH_S",s);

    return n;
}

private int printDynVar2DISkewnessValues(final DynVar2DISkewness x)
{
    int n=0;

    n+=15;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_0_feq-x.I.s0);
    this.dattribs.addValue(x.IHf_w);
    this.dattribs.addValue(x.ILf_w);
    this.dattribs.addValue(x.DIHf_w);
    this.dattribs.addValue(x.DIhaf_w);
    this.dattribs.addValue(x.DILf_w);
    this.dattribs.addValue(x.p_DI);
    this.dattribs.addValue(x.sk_DI);
    this.dattribs.addValue(x.pd_DI);
    this.dattribs.addValue(x.pd_DII);
    this.dattribs.addValue(x.pd_DI0);
    this.dattribs.addValue(x.pd_DIDp);
    this.dattribs.addValue(x.dpd_DI);

    n+=EVXData_Values(x.I);
    n+=Skewness_Values(x.IHf_S);
    n+=Skewness_Values(x.IHf_S__nodI);
    n+=Skewness_Values(x.ILf_S);
    n+=Skewness_Values(x.ILf_S__nodI);
    n+=Skewness_Values(x.DIHf_S);
    n+=Skewness_Values(x.DIHf_S__nodI);
    n+=Skewness_Values(x.DIhaf_S);
    n+=Skewness_Values(x.DIhaf_S__nodI);
    n+=Skewness_Values(x.DILf_S);
    n+=Skewness_Values(x.DILf_S__nodI);
    n+=Skewness_Values(x.DIIH_S);

    return n;
}



private int printDynVar2DIDPNames(final String prefix,final String s)
{
    int n=0;

    n+=13;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_dIeq0_",s,null);
    this.dattribs_names.addValue(prefix+".sumdp_",s,null);
    this.dattribs_names.addValue(prefix+".sumdi_",s,null);
    this.dattribs_names.addValue(prefix+".sumdpdi_",s,null);
    this.dattribs_names.addValue(prefix+".delta_P_",s,null);
    this.dattribs_names.addValue(prefix+".delta_Pdyn_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_from_dI_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_from_ptpi_",s,null);
    this.dattribs_names.addValue(prefix+".sumdpifmik_",s,null);
    this.dattribs_names.addValue(prefix+".sumdpifmik_ee_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printDynVar2DIDPValues(final DynVar2DIDP x)
{
    int n=0;

    n+=13;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.I_0_dIeq0);
    this.dattribs.addValue(x.sumdp);
    this.dattribs.addValue(x.sumdi);
    this.dattribs.addValue(x.sumdpdi);
    this.dattribs.addValue(x.delta_P);
    this.dattribs.addValue(x.delta_Pdyn);
    this.dattribs.addValue(x.Pf_from_dI);
    this.dattribs.addValue(x.Pf_from_ptpi);
    this.dattribs.addValue(x.sumdpifmik);
    this.dattribs.addValue(x.sumdpifmik_ee);

    n+=EVXData_Values(x.I);

    return n;
}


private int printDynEVdINames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".dI_",s,null);
    this.dattribs_names.addValue(prefix+".dW_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    this.dattribs_names.addValue(prefix+".sigma2_Pf_",s,null);
    this.dattribs_names.addValue(prefix+".PfH_",s,null);
    this.dattribs_names.addValue(prefix+".PfH0_",s,null);

    n+=Skewness_Names(prefix+".SK",s);
    n+=Skewness_Names(prefix+".SKf",s);
    n+=Skewness_Names(prefix+".SK0",s);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printDynEVdIValues(final DynEVdI x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.dI);
    this.dattribs.addValue(x.dW);
    this.dattribs.addValue(x.Pf);
    this.dattribs.addValue(x.sigma2_Pf);
    this.dattribs.addValue(x.PfH);
    this.dattribs.addValue(x.PfH0);

    n+=Skewness_Values(x.SK);
    n+=Skewness_Values(x.SKf);
    n+=Skewness_Values(x.SK0);
    n+=EVXData_Values(x.I);

    return n;
}

private int printDynEVdI0Names(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".dI_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    this.dattribs_names.addValue(prefix+".PfdI_",s,null);
    this.dattribs_names.addValue(prefix+".PfdI0_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_IX_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_spur_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printDynEVdI0Values(final DynEVdI0 x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.dI);
    this.dattribs.addValue(x.Pf);
    this.dattribs.addValue(x.PfdI);
    this.dattribs.addValue(x.PfdI0);
    this.dattribs.addValue(x.var_pI_IX);
    this.dattribs.addValue(x.var_pI_spur);

    n+=EVXData_Values(x.I);

    return n;
}

private int printDynVar2RDISkewnessNames(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dI_",s,null);
    this.dattribs_names.addValue(prefix+".sD_",s,null);

    n+=EVXData_Names(prefix+".I",s);
    n+=Skewness_Names(prefix+".RDIHf_S",s);
    n+=Skewness_Names(prefix+".RDILf_S",s);

    return n;
}

private int printDynVar2RDISkewnessValues(final DynVar2RDISkewness x)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_0_feq-x.I.s0);
    this.dattribs.addValue(x.sD);

    n+=EVXData_Values(x.I);
    n+=Skewness_Values(x.RDIHf_S);
    n+=Skewness_Values(x.RDILf_S);

    return n;
}


private int printPnLdICorr_CrossCorr_Names(final String prefix,final String s)
{
    int n=0;

    n+=Skewness_Names(prefix+".SK_pv",s);
    n+=Skewness_Names(prefix+".SK_pt",s);
    n+=printPnLdICorr_RateRel_Names(prefix+".ipv",s);
    n+=Skewness_Names(prefix+".SK_rv",s);
    n+=Skewness_Names(prefix+".SK_rt",s);

    n+=5;
    this.dattribs_names.addValue(prefix+".corr_v_",s,null);
    this.dattribs_names.addValue(prefix+".crosscorr_v.p1r1_",s,null);
    this.dattribs_names.addValue(prefix+".crosscorr_v.p1r2_",s,null);
    this.dattribs_names.addValue(prefix+".crosscorr_v.p2r1_",s,null);
    this.dattribs_names.addValue(prefix+".crosscorr_v.p2r2_",s,null);
    return n;
}

private int printPnLdICorr_CrossCorr_Values(final PnLdICorr.CrossCorr x)
{
    int n=0;

    n+=Skewness_Values(x.SK_pv);
    n+=Skewness_Values(x.SK_pt);
    n+=printPnLdICorr_RateRel_Values(x.ipv);
    n+=Skewness_Values(x.SK_rv);
    n+=Skewness_Values(x.SK_rt);

    n+=5;
    this.dattribs.addValue(x.corr_v);
    this.dattribs.addValue(x.crosscorr_v[0]);
    this.dattribs.addValue(x.crosscorr_v[1]);
    this.dattribs.addValue(x.crosscorr_v[2]);
    this.dattribs.addValue(x.crosscorr_v[3]);
    return n;
}

private int printPnLdICorr_RateRel_Names(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".r1_",s,null);
    this.dattribs_names.addValue(prefix+".r2_",s,null);
    this.dattribs_names.addValue(prefix+".r1_div_",s,null);
    this.dattribs_names.addValue(prefix+".r2_div_",s,null);
    return n;
}

private int printPnLdICorr_RateRel_Values(final PnLdICorr.RateRel x)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.r1);
    this.dattribs.addValue(x.r2);
    this.dattribs.addValue(x.r1_div);
    this.dattribs.addValue(x.r2_div);
    return n;
}




private int printPnLdICorrNames(final String prefix,final String s)
{
    int n=0;

    n+=1;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    n+=printPnLdICorr_CrossCorr_Names(prefix+".IHf_cc",s);
    n+=printPnLdICorr_CrossCorr_Names(prefix+".movaver_cc",s);
    n+=printPnLdICorr_CrossCorr_Names(prefix+".spur_cc",s);

    return n;
}
private int printPnLdICorrValues(final PnLdICorr x)
{
    int n=0;

    n+=1;
    this.dattribs.addValue(x.p_offset);

    n+=EVXData_Values(x.I);

    n+=printPnLdICorr_CrossCorr_Values(x.IHf_cc);
    n+=printPnLdICorr_CrossCorr_Values(x.movaver_cc);
    n+=printPnLdICorr_CrossCorr_Values(x.spur_cc);

    return n;
}

private int printPnLdIDNames(final String prefix,final String s)
{
    int n=0;

    n+=13;
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".P_movaver_",s,null);
    this.dattribs_names.addValue(prefix+".P_movaver_spur_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_0_",s,null);
    this.dattribs_names.addValue(prefix+".P0_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".P0_IHf_dp0_",s,null);
    this.dattribs_names.addValue(prefix+".w_IHf_0_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLdIDValues(final PnLdID x)
{
    int n=0;

    n+=13;
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_IHf);
    this.dattribs.addValue(x.P_movaver);
    this.dattribs.addValue(x.P_movaver_spur);
    this.dattribs.addValue(x.var_pI_IH);
    this.dattribs.addValue(x.dp_IH);
    this.dattribs.addValue(x.dp_orig_IH);
    this.dattribs.addValue(x.dp_orig_IHf);
    this.dattribs.addValue(x.dp_orig_0);
    this.dattribs.addValue(x.P0_IHf);
    this.dattribs.addValue(x.P0_IHf_dp0);
    this.dattribs.addValue(x.w_IHf_0);

    n+=EVXData_Values(x.I);

    return n;
}

private int ProbabilityCorrelation_Names(final String prefix,final String s)
{
    int n=0;
    n+=10;
    this.dattribs_names.addValue(prefix+".pL_",s,null);
    this.dattribs_names.addValue(prefix+".pH_",s,null);
    this.dattribs_names.addValue(prefix+".iL_",s,null);
    this.dattribs_names.addValue(prefix+".iH_",s,null);
    this.dattribs_names.addValue(prefix+".w_pL_iL_",s,null);
    this.dattribs_names.addValue(prefix+".w_pL_iH_",s,null);
    this.dattribs_names.addValue(prefix+".w_pH_iL_",s,null);
    this.dattribs_names.addValue(prefix+".w_pH_iH_",s,null);
    this.dattribs_names.addValue(prefix+".W_",s,null);
    this.dattribs_names.addValue(prefix+".corr_",s,null);

    return n;
}
private int ProbabilityCorrelation_Values(final com.polytechnik.utils.ProbabilityCorrelation x)
{
    int n=0;
    n+=10;
    this.dattribs.addValue(x.pL);
    this.dattribs.addValue(x.pH);
    this.dattribs.addValue(x.iL);
    this.dattribs.addValue(x.iH);
    this.dattribs.addValue(x.w_pL_iL);
    this.dattribs.addValue(x.w_pL_iH);
    this.dattribs.addValue(x.w_pH_iL);
    this.dattribs.addValue(x.w_pH_iH);
    this.dattribs.addValue(x.W);
    this.dattribs.addValue(x.corr);

    return n;
}

private int PnLdIE_CorrPI_Names(final String prefix,final String s)
{
    int n=0;
    n+=ProbabilityCorrelation_Names(prefix,s);
    n+=2;
    this.dattribs_names.addValue(prefix+".Pt_",s,null);
    this.dattribs_names.addValue(prefix+".Pi_",s,null);

    return n;
}
private int PnLdIE_CorrPI_Values(final PnLdIE.CorrPI x)
{
    int n=0;
    n+=ProbabilityCorrelation_Values(x);
    n+=2;
    this.dattribs.addValue(x.Pt);
    this.dattribs.addValue(x.Pi);

    return n;
}

private int printPnLdIENames(final String prefix,final String s)
{
    int n=0;

    n+=2;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);

    n+=PnLdIE_CorrPI_Names(prefix+".it",s);
    n+=PnLdIE_CorrPI_Names(prefix+".ti",s);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLdIEValues(final PnLdIE x)
{
    int n=0;

    n+=2;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.I_0_feq);

    n+=PnLdIE_CorrPI_Values(x.it);
    n+=PnLdIE_CorrPI_Values(x.ti);
    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLdIE2Names(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".p_imin_",s,null);
    this.dattribs_names.addValue(prefix+".w_imin_",s,null);
    this.dattribs_names.addValue(prefix+".p_imax_",s,null);
    this.dattribs_names.addValue(prefix+".w_imax_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLdIE2Values(final PnLdIE2 x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_IHf);
    this.dattribs.addValue(x.pt_IHf);
    this.dattribs.addValue(x.p_imin);
    this.dattribs.addValue(x.w_imin);
    this.dattribs.addValue(x.p_imax);
    this.dattribs.addValue(x.w_imax);

    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLdIfdynNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IHf_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_feq_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_",s,null);
    this.dattribs_names.addValue(prefix+".A_Pb_",s,null);
    this.dattribs_names.addValue(prefix+".Pa_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLdIfdynValues(final PnLdIfdyn x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_IHf);
    this.dattribs.addValue(x.I_0_feq);
    this.dattribs.addValue(x.Pf);
    this.dattribs.addValue(x.A_Pb);
    this.dattribs.addValue(x.Pa);

    n+=EVXData_Values(x.I);

    return n;
}

private int printPnL2I0Names(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_eq_",s,null);
    this.dattribs_names.addValue(prefix+".dI_0_eq_",s,null);
    this.dattribs_names.addValue(prefix+".phi_fact_",s,null);
    this.dattribs_names.addValue(prefix+".P_eq_",s,null);
    this.dattribs_names.addValue(prefix+".Pf_eq_",s,null);
    this.dattribs_names.addValue(prefix+".P1_eq_",s,null);
    this.dattribs_names.addValue(prefix+".P_phi0_",s,null);
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnL2I0Values(final PnL2I0 x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_0_eq);
    this.dattribs.addValue(x.dI_0_eq);
    this.dattribs.addValue(x.phi_fact);
    this.dattribs.addValue(x.P_eq);
    this.dattribs.addValue(x.Pf_eq);
    this.dattribs.addValue(x.P1_eq);
    this.dattribs.addValue(x.P_phi0);
    this.dattribs.addValue(x.p_offset);
    n+=EVXData_Values(x.I);

    return n;
}

private int printPnLVarVolatilitypiNames(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".varVol_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_A_",s,null);
    this.dattribs_names.addValue(prefix+".varVol_A_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printPnLVarVolatilitypiValues(final PnLVarVolatilitypi x)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.varVol_IH);
    this.dattribs.addValue(x.p_A);
    this.dattribs.addValue(x.varVol_A);

    n+=EVXData_Values(x.I);

    return n;
}

private int printDynVar2DynImpactDINames(final String prefix,final String s)
{
    int n=0;

    n+=15;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_dIL_",s,null);
    this.dattribs_names.addValue(prefix+".dp_dIH_",s,null);
    this.dattribs_names.addValue(prefix+".dI_dIL_",s,null);
    this.dattribs_names.addValue(prefix+".dI_dIH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_plus_",s,null);
    this.dattribs_names.addValue(prefix+".dp_minus_",s,null);
    this.dattribs_names.addValue(prefix+".dpI_plus_",s,null);
    this.dattribs_names.addValue(prefix+".dpI_minus_",s,null);
    this.dattribs_names.addValue(prefix+".dpdI_plus_",s,null);
    this.dattribs_names.addValue(prefix+".dpdI_minus_",s,null);
    n+=printACorrNames(prefix+".a_dpdI",s);
    n+=printACorrNames(prefix+".a_dpdII",s);

    return n;
}
private int printDynVar2DynImpactDIValues(final DynVar2DynImpactDI x)
{
    int n=0;

    n+=15;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var_pI_IH);
    this.dattribs.addValue(x.dp_dIL);
    this.dattribs.addValue(x.dp_dIH);
    this.dattribs.addValue(x.dI_dIL);
    this.dattribs.addValue(x.dI_dIH);
    this.dattribs.addValue(x.dp_plus);
    this.dattribs.addValue(x.dp_minus);
    this.dattribs.addValue(x.dpI_plus);
    this.dattribs.addValue(x.dpI_minus);
    this.dattribs.addValue(x.dpdI_plus);
    this.dattribs.addValue(x.dpdI_minus);
    n+=printACorrValues(x.a_dpdI);
    n+=printACorrValues(x.a_dpdII);

    return n;
}

private int printImpactExampleNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".I_bc_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_bc_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_bc_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_bc_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_bc_IH_",s,null);

    n+=EVXData_Names(prefix+".edI",s);
    n+=3;
    this.dattribs_names.addValue(prefix+".p_dIH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_dIH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_dIH_",s,null);

    n+=EVXData_Names(prefix+".eI",s);
    n+=12;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_vIeq_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_eIplus_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_eIminus_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_eI_v2I_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_eI_A_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_eI_Aa_",s,null);

    n+=3;
    this.dattribs_names.addValue(prefix+".p_0_",s,null);
    this.dattribs_names.addValue(prefix+".dp_0_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_0_",s,null);

    n+=EVXData_Names(prefix+".eIdp",s);
    n+=6;
    this.dattribs_names.addValue(prefix+".I_eIdp_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_eIdp_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_eIdp_L_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_eIdp_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_eIdp_L_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_eIdp_H_",s,null);

    return n;
}
private int printImpactExampleValues(final ImpactExample x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.I_bc_0);
    this.dattribs.addValue(x.I_bc_IL);
    this.dattribs.addValue(x.I_bc_IH);
    this.dattribs.addValue(x.p_bc_IH);
    this.dattribs.addValue(x.dp_orig_bc_IH);

    n+=EVXData_Values(x.edI);
    n+=3;
    this.dattribs.addValue(x.p_dIH);
    this.dattribs.addValue(x.dp_dIH);
    this.dattribs.addValue(x.dp_orig_dIH);

    n+=EVXData_Values(x.eI);
    n+=12;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.var_pI_IH);
    this.dattribs.addValue(x.p_vIeq);
    this.dattribs.addValue(x.dp_IH);
    this.dattribs.addValue(x.dpt_IH);
    this.dattribs.addValue(x.dp_orig_IH);
    this.dattribs.addValue(x.dp_orig_eIplus);
    this.dattribs.addValue(x.dp_orig_eIminus);
    this.dattribs.addValue(x.dp_orig_eI_v2I);
    this.dattribs.addValue(x.dp_orig_eI_A);
    this.dattribs.addValue(x.dp_orig_eI_Aa);

    n+=3;
    this.dattribs.addValue(x.p_0);
    this.dattribs.addValue(x.dp_0);
    this.dattribs.addValue(x.dp_orig_0);

    n+=EVXData_Values(x.eIdp);
    n+=6;
    this.dattribs.addValue(x.I_eIdp_L);
    this.dattribs.addValue(x.I_eIdp_H);
    this.dattribs.addValue(x.w_IH_eIdp_L);
    this.dattribs.addValue(x.w_IH_eIdp_H);
    this.dattribs.addValue(x.w_IL_eIdp_L);
    this.dattribs.addValue(x.w_IL_eIdp_H);

    return n;
}

private int printImpactIStatesExampleNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".eI",s);
    n+=19;
    this.dattribs_names.addValue(prefix+".p_offset_",s,null);
    this.dattribs_names.addValue(prefix+".eIdp_L_",s,null);
    this.dattribs_names.addValue(prefix+".eIdp_H_",s,null);
    this.dattribs_names.addValue(prefix+".I_eIdp_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_eIdp_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_eIdp_L_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_eIdp_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_eIdp_L_",s,null);
    this.dattribs_names.addValue(prefix+".w_IL_eIdp_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".vp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".v2p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".vp_spur_",s,null);
    this.dattribs_names.addValue(prefix+".vpI_spur_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dpI_orig_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_eI_v2I_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_eI_A_",s,null);
    this.dattribs_names.addValue(prefix+".dp_orig_eI_Aa_",s,null);

    return n;
}
private int printImpactIStatesExampleValues(final ImpactIStatesExample x)
{
    int n=0;

    n+=EVXData_Values(x.eI);
    n+=19;
    this.dattribs.addValue(x.p_offset);
    this.dattribs.addValue(x.eIdp[0]);
    this.dattribs.addValue(x.eIdp[x.eI.neq-1]);
    this.dattribs.addValue(x.I_eIdp_L);
    this.dattribs.addValue(x.I_eIdp_H);
    this.dattribs.addValue(x.w_IH_eIdp_L);
    this.dattribs.addValue(x.w_IH_eIdp_H);
    this.dattribs.addValue(x.w_IL_eIdp_L);
    this.dattribs.addValue(x.w_IL_eIdp_H);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.vp_IH);
    this.dattribs.addValue(x.v2p_IH);
    this.dattribs.addValue(x.vp_spur);
    this.dattribs.addValue(x.vpI_spur);
    this.dattribs.addValue(x.dp_orig_IH);
    this.dattribs.addValue(x.dpI_orig_IH);
    this.dattribs.addValue(x.dp_orig_eI_v2I);
    this.dattribs.addValue(x.dp_orig_eI_A);
    this.dattribs.addValue(x.dp_orig_eI_Aa);



    return n;
}

private int printPnLDiagProjNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=24;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p1_",s,null);
    this.dattribs_names.addValue(prefix+".p2_",s,null);
    this.dattribs_names.addValue(prefix+".maxPnL_",s,null);
    this.dattribs_names.addValue(prefix+".I1_",s,null);
    this.dattribs_names.addValue(prefix+".I2_",s,null);
    this.dattribs_names.addValue(prefix+".V1_",s,null);
    this.dattribs_names.addValue(prefix+".V2_",s,null);
    this.dattribs_names.addValue(prefix+".d1_",s,null);
    this.dattribs_names.addValue(prefix+".d2_",s,null);
    this.dattribs_names.addValue(prefix+".psix0_1_",s,null);
    this.dattribs_names.addValue(prefix+".psix0_2_",s,null);
    this.nattribs_names.addValue(prefix+".e1_",s,null);
    this.nattribs_names.addValue(prefix+".e2_",s,null);
    this.dattribs_names.addValue(prefix+".Ia_",s,null);
    this.dattribs_names.addValue(prefix+".signedIa0_",s,null);
    this.dattribs_names.addValue(prefix+".signedI_p_last_",s,null);
    this.dattribs_names.addValue(prefix+".signedI0_p_last_",s,null);
    this.dattribs_names.addValue(prefix+".PnL_L_",s,null);
    this.dattribs_names.addValue(prefix+".PnL_H_",s,null);
    this.dattribs_names.addValue(prefix+".PnL_pl_L_",s,null);
    this.dattribs_names.addValue(prefix+".PnL_pl_H_",s,null);
    this.dattribs_names.addValue(prefix+".PnL_IH_L_",s,null);
    this.dattribs_names.addValue(prefix+".PnL_IH_H_",s,null);
    n+=6;
    this.dattribs_names.addValue(prefix+".i0_I1_",s,null);
    this.dattribs_names.addValue(prefix+".i0_I2_",s,null);
    this.dattribs_names.addValue(prefix+".i0_p1_",s,null);
    this.dattribs_names.addValue(prefix+".i0_p2_",s,null);
    this.dattribs_names.addValue(prefix+".i0_psi1_",s,null);
    this.dattribs_names.addValue(prefix+".i0_psi2_",s,null);
    n+=6;
    this.dattribs_names.addValue(prefix+".ii0_I1_",s,null);
    this.dattribs_names.addValue(prefix+".ii0_I2_",s,null);
    this.dattribs_names.addValue(prefix+".ii0_p1_",s,null);
    this.dattribs_names.addValue(prefix+".ii0_p2_",s,null);
    this.dattribs_names.addValue(prefix+".ii0_psi1_",s,null);
    this.dattribs_names.addValue(prefix+".ii0_psi2_",s,null);
    return n;
}
private int printPnLDiagProjValues(final PnLDiagProj x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=24;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p1);
    this.dattribs.addValue(x.p2);
    this.dattribs.addValue(x.maxPnL);
    this.dattribs.addValue(x.I1);
    this.dattribs.addValue(x.I2);
    this.dattribs.addValue(x.V1);
    this.dattribs.addValue(x.V2);
    this.dattribs.addValue(x.d1);
    this.dattribs.addValue(x.d2);
    this.dattribs.addValue(x.psix0_1);
    this.dattribs.addValue(x.psix0_2);
    this.nattribs.addValue(x.e1);
    this.nattribs.addValue(x.e2);
    this.dattribs.addValue(x.Ia);
    this.dattribs.addValue(x.signedIa0);
    this.dattribs.addValue(x.signedI_p_last);
    this.dattribs.addValue(x.signedI0_p_last);
    this.dattribs.addValue(x.PnL_L);
    this.dattribs.addValue(x.PnL_H);
    this.dattribs.addValue(x.PnL_pl_L);
    this.dattribs.addValue(x.PnL_pl_H);
    this.dattribs.addValue(x.PnL_IH_L);
    this.dattribs.addValue(x.PnL_IH_H);

    n+=6;
    this.dattribs.addValue(x.i0_I1);
    this.dattribs.addValue(x.i0_I2);
    this.dattribs.addValue(x.i0_p1);
    this.dattribs.addValue(x.i0_p2);
    this.dattribs.addValue(x.i0_psi1);
    this.dattribs.addValue(x.i0_psi2);
    n+=6;
    this.dattribs.addValue(x.ii0_I1);
    this.dattribs.addValue(x.ii0_I2);
    this.dattribs.addValue(x.ii0_p1);
    this.dattribs.addValue(x.ii0_p2);
    this.dattribs.addValue(x.ii0_psi1);
    this.dattribs.addValue(x.ii0_psi2);

    return n;
}

private int printDynVarJPMorganNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_MP_",s,null);
    this.dattribs_names.addValue(prefix+".P_MP_",s,null);
    this.dattribs_names.addValue(prefix+".v2_P_MP_",s,null);
    this.dattribs_names.addValue(prefix+".var2_p_IH_PROJ_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printDynVarJPMorganValues(final DynVarJPMorgan x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.I_MP);
    this.dattribs.addValue(x.P_MP);
    this.dattribs.addValue(x.v2_P_MP);
    this.dattribs.addValue(x.var2_p_IH_PROJ);
    n+=EVXData_Values(x.I);

    return n;
}

private int printDynYpmNames(final String prefix,final String s)
{
    int n=0;

    n+=7;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".sigma_p_",s,null);
    this.dattribs_names.addValue(prefix+".pe_VOL_",s,null);
    this.dattribs_names.addValue(prefix+".I_VOL_",s,null);
    this.dattribs_names.addValue(prefix+".p_VOL_",s,null);
    this.dattribs_names.addValue(prefix+".var_pe_VOL_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printDynYpmValues(final DynYpm x)
{
    int n=0;

    n+=7;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.sigma_p);
    this.dattribs.addValue(x.pe_VOL);
    this.dattribs.addValue(x.I_VOL);
    this.dattribs.addValue(x.p_VOL);
    this.dattribs.addValue(x.var_pe_VOL);
    n+=EVXData_Values(x.I);

    return n;
}

private int printDynVarYNames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_vI_min_",s,null);
    this.dattribs_names.addValue(prefix+".p_vI_max_",s,null);
    this.dattribs_names.addValue(prefix+".p_vI_A_",s,null);
    this.dattribs_names.addValue(prefix+".p_vI_AA_",s,null);

    return n;
}
private int printDynVarYValues(final DynVarY x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_vI_min);
    this.dattribs.addValue(x.p_vI_max);
    this.dattribs.addValue(x.p_vI_A);
    this.dattribs.addValue(x.p_vI_AA);

    return n;
}
private int printDynVarSP0Names(final String prefix,final String s)
{
    int n=0;

    n+=5;
    this.dattribs_names.addValue(prefix+".I0_",s,null);
    this.dattribs_names.addValue(prefix+".dirC_",s,null);
    this.dattribs_names.addValue(prefix+".dirPC_",s,null);
    this.dattribs_names.addValue(prefix+".dirP_",s,null);
    this.dattribs_names.addValue(prefix+".Pi0_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printDynVarSP0Values(final DynVarSP0 x)
{
    int n=0;

    n+=5;
    this.dattribs.addValue(x.I0);
    this.dattribs.addValue(x.dirC);
    this.dattribs.addValue(x.dirPC);
    this.dattribs.addValue(x.dirP);
    this.dattribs.addValue(x.Pi0);
    n+=EVXData_Values(x.I);

    return n;
}
private int printDynRVarSP0Names(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".v0_",s,null);
    this.dattribs_names.addValue(prefix+".p0_",s,null);
    this.dattribs_names.addValue(prefix+".pe0_",s,null);
    this.dattribs_names.addValue(prefix+".pdI_",s,null);

    return n;
}

private int printDynRVarSP0Values(final DynRVarSP0 x)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.v0);
    this.dattribs.addValue(x.p0);
    this.dattribs.addValue(x.pe0);
    this.dattribs.addValue(x.pdI);

    return n;
}
private int printDynVar2INames(final String prefix,final String s)
{
    int n=0;

    n+=10;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".pe_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".Di_DiL_",s,null);
    this.dattribs_names.addValue(prefix+".p_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".pe_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".Di_DiH_",s,null);

    return n;
}
private int printDynVar2IValues(final DynVar2I x)
{
    int n=0;

    n+=10;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.p_DIL);
    this.dattribs.addValue(x.pe_DIL);
    this.dattribs.addValue(x.Di_DIL);
    this.dattribs.addValue(x.p_DIH);
    this.dattribs.addValue(x.pe_DIH);
    this.dattribs.addValue(x.Di_DIH);

    return n;
}
private int printDynDINames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=5;
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".dI_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".P_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".dI_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".P_DIH_",s,null);

    return n;
}
private int printDynDIValues(final DynDI x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=5;
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.dI_DIL);
    this.dattribs.addValue(x.P_DIL);
    this.dattribs.addValue(x.dI_DIH);
    this.dattribs.addValue(x.P_DIH);

    return n;
}
private int printDynVNames(final String prefix,final String s)
{
    int n=0;
    n+=8;
    this.dattribs_names.addValue(prefix+".diraver_",s,null);
    this.dattribs_names.addValue(prefix+".pa_I_",s,null);
    this.dattribs_names.addValue(prefix+".pa_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_I_",s,null);
    this.dattribs_names.addValue(prefix+".p_V_",s,null);
    this.dattribs_names.addValue(prefix+".I_",s,null);
    this.dattribs_names.addValue(prefix+".V_",s,null);
    this.dattribs_names.addValue(prefix+".p2x0_",s,null);

    return n;
}
private int printDynVValues(final DynV x)
{
    int n=0;
    n+=8;
    this.dattribs.addValue(x.diraver);
    this.dattribs.addValue(x.pa_I);
    this.dattribs.addValue(x.pa_V);
    this.dattribs.addValue(x.p_I);
    this.dattribs.addValue(x.p_V);
    this.dattribs.addValue(x.I);
    this.dattribs.addValue(x.V);
    this.dattribs.addValue(x.p2x0);

    return n;
}
private int printDynIndPnLNames(final String prefix,final String s)
{
    int n=0;
    n+=EVXData_Names(prefix+".I",s);
    n+=7;
    this.dattribs_names.addValue(prefix+".ind_t_",s,null);
    this.dattribs_names.addValue(prefix+".ind_i_",s,null);
    this.dattribs_names.addValue(prefix+".P_IH_",s,null);
    this.dattribs_names.addValue(prefix+".P_ind_t_",s,null);
    this.dattribs_names.addValue(prefix+".Pd_ind_t_",s,null);
    this.dattribs_names.addValue(prefix+".P_ind_i_",s,null);
    this.dattribs_names.addValue(prefix+".Pd_ind_i_",s,null);

    return n;
}
private int printDynIndPnLValues(final DynIndPnL x)
{
    int n=0;
    n+=EVXData_Values(x.I);
    n+=7;
    this.dattribs.addValue(x.ind_t);
    this.dattribs.addValue(x.ind_i);
    this.dattribs.addValue(x.P_IH);
    this.dattribs.addValue(x.P_ind_t);
    this.dattribs.addValue(x.Pd_ind_t);
    this.dattribs.addValue(x.P_ind_i);
    this.dattribs.addValue(x.Pd_ind_i);

    return n;
}
private int printDynVarCDVNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".pm_IX_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    n+=printACorrNames(prefix+".a",s);

    return n;
}

private int printDynVarCDVValues(final DynVarCDV x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.pm_IX);
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.I_IH);
    n+=printACorrValues(x.a);

    return n;
}


private int printDynVarFiniteD1Names(final String prefix,final String s)
{
    int n=0;

    n+=12;
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX2_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX2_",s,null);
    this.dattribs_names.addValue(prefix+".dist2_",s,null);
    this.dattribs_names.addValue(prefix+".pB_",s,null);
    this.dattribs_names.addValue(prefix+".pF_",s,null);
    this.dattribs_names.addValue(prefix+".dist2m_",s,null);
    this.dattribs_names.addValue(prefix+".pBm_",s,null);
    this.dattribs_names.addValue(prefix+".pFm_",s,null);
    this.dattribs_names.addValue(prefix+".p1m_",s,null);
    this.dattribs_names.addValue(prefix+".p2m_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printDynVarFiniteD1Values(final DynVarFiniteD1 x)
{
    int n=0;

    n+=12;
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.p_IX2);
    this.dattribs.addValue(x.I_IX2);
    this.dattribs.addValue(x.dist2);
    this.dattribs.addValue(x.pB);
    this.dattribs.addValue(x.pF);
    this.dattribs.addValue(x.dist2m);
    this.dattribs.addValue(x.pBm);
    this.dattribs.addValue(x.pFm);
    this.dattribs.addValue(x.p1m);
    this.dattribs.addValue(x.p2m);
    n+=EVXData_Values(x.I);

    return n;
}

private int printDynVarFiniteFNames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".pm_jL_",s,null);
    this.dattribs_names.addValue(prefix+".pm_jH_",s,null);
    this.dattribs_names.addValue(prefix+".pi_jL_",s,null);
    this.dattribs_names.addValue(prefix+".pi_jH_",s,null);
    this.dattribs_names.addValue(prefix+".pv_jL_",s,null);
    this.dattribs_names.addValue(prefix+".pv_jH_",s,null);
    this.dattribs_names.addValue(prefix+".var_pCm_jL_",s,null);
    this.dattribs_names.addValue(prefix+".var_pCm_jH_",s,null);
    n+=EVXData_Names(prefix+".J",s);

    return n;
}

private int printDynVarFiniteFValues(final DynVarFiniteF x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.pm_jL);
    this.dattribs.addValue(x.pm_jH);
    this.dattribs.addValue(x.pi_jL);
    this.dattribs.addValue(x.pi_jH);
    this.dattribs.addValue(x.pv_jL);
    this.dattribs.addValue(x.pv_jH);
    this.dattribs.addValue(x.var_pCm_jL);
    this.dattribs.addValue(x.var_pCm_jH);
    n+=EVXData_Values(x.J);

    return n;
}

private int printDynVarFiniteFVNames(final String prefix,final String s)
{
    int n=0;

    n+=12;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".V_IH_",s,null);
    this.dattribs_names.addValue(prefix+".J_IX_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".V_IX_",s,null);
    this.dattribs_names.addValue(prefix+".tau_IX_",s,null);
    this.dattribs_names.addValue(prefix+".pj_IX_",s,null);
    this.dattribs_names.addValue(prefix+".Dpj_IX_",s,null);

    return n;
}

private int printDynVarFiniteFVValues(final DynVarFiniteFV x)
{
    int n=0;

    n+=12;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.V_IH);
    this.dattribs.addValue(x.J_IX);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.V_IX);
    this.dattribs.addValue(x.tau_IX);
    this.dattribs.addValue(x.pj_IX);
    this.dattribs.addValue(x.Dpj_IX);

    return n;
}

private int printDynVarFiniteENames(final String prefix,final String s)
{
    int n=0;

    n+=13;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_IE_",s,null);
    this.dattribs_names.addValue(prefix+".I_IE_",s,null);
    this.dattribs_names.addValue(prefix+".w_IX_IE_",s,null);
    this.dattribs_names.addValue(prefix+".pix_IX_IE_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_IX_IE_",s,null);
    this.dattribs_names.addValue(prefix+".pix_0_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_0_",s,null);
    this.dattribs_names.addValue(prefix+".distance2_IX_IE_",s,null);

    return n;
}

private int printDynVarFiniteEValues(final DynVarFiniteE x)
{
    int n=0;

    n+=13;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.p_IE);
    this.dattribs.addValue(x.I_IE);
    this.dattribs.addValue(x.w_IX_IE);
    this.dattribs.addValue(x.pix_IX_IE);
    this.dattribs.addValue(x.pnl_IX_IE);
    this.dattribs.addValue(x.pix_0);
    this.dattribs.addValue(x.pnl_0);
    this.dattribs.addValue(x.distance2_IX_IE);

    return n;
}

private int printRkiQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=printDynSkiNames(prefix+".Sk0",s);
    n+=printDynSkiNames(prefix+".SkR",s);
    n+=printDynSkiNames(prefix+".Sk",s);
    n+=printDynRkiNames(prefix+".Rk0",s);
    n+=printDynRkiNames(prefix+".RkR",s);
    n+=printDynRkiNames(prefix+".Rk",s);
    n+=printDynRkiNames(prefix+".RkV0",s);
    n+=printDynRkiNames(prefix+".RkVR",s);
    n+=printDynVarCDNames(prefix+".pixC",s);
    n+=printDynVarCDNames(prefix+".pixP",s);
    n+=printDynVarCDNames(prefix+".piVP",s);
    n+=printDynVarCDNames(prefix+".piDP",s);

    return n;
}
private <T> int printRkiQVMValues(final RkiQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=printDynSkiValues(x.Sk0);
    n+=printDynSkiValues(x.SkR);
    n+=printDynSkiValues(x.Sk);
    n+=printDynRkiValues(x.Rk0);
    n+=printDynRkiValues(x.RkR);
    n+=printDynRkiValues(x.Rk);
    n+=printDynRkiValues(x.RkV0);
    n+=printDynRkiValues(x.RkVR);
    n+=printDynVarCDValues(x.pixC);
    n+=printDynVarCDValues(x.pixP);
    n+=printDynVarCDValues(x.piVP);
    n+=printDynVarCDValues(x.piDP);

    return n;
}



private int printSkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=printEjpCommNames(prefix+".dptI",s);
    n+=printEjpCommNames(prefix+".vstI",s);
    n+=printEjpCommNames(prefix+".DdptI",s);
    n+=printEjpCommNames(prefix+".DvstI",s);

    n+=printEjaCommNames(prefix+".ja_sam",s);
    n+=printEjaCommNames(prefix+".Rja_sam",s);
    n+=printEjaCommNames(prefix+".RRja_sam",s);

    return n;
}
private <T> int printSkQVMValues(final SkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=printEjpCommValues(x.dptI);
    n+=printEjpCommValues(x.vstI);
    n+=printEjpCommValues(x.DdptI);
    n+=printEjpCommValues(x.DvstI);


    n+=printEjaCommValues(x.ja_sam);
    n+=printEjaCommValues(x.Rja_sam);
    n+=printEjaCommValues(x.RRja_sam);

    return n;
}


private int printTkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=StateSaver_Names(prefix+".sst.r",s);
    n+=FStateSaver_Names(prefix+".sst.rf",s);
    n+=SMMStateSaver_Names(prefix+".sst.rq",s);
    n+=SMMXStateSaver_Names(prefix+".sst.rxq",s);
    n+=MinMaxStateSaver_Names(prefix+".sst.rmm",s);
    n+=MaxStateSaver_Names(prefix+".sst.rlm",s);
    n+=MaxStateSaver_Names(prefix+".sst.rhm",s);
    n+=EVXData_Names(prefix+".Th",s);
    n+=EVXData_Names(prefix+".STh",s);
    n+=EVXData_Names(prefix+".PnL",s);
    n+=EVXData_Names(prefix+".iPnL",s);
    n+=EVXData_Names(prefix+".Ji",s);
    n+=EVXData_Names(prefix+".Jt",s);
    n+=18;
    this.dattribs_names.addValue(prefix+".Svs0_",s,null);
    this.dattribs_names.addValue(prefix+".vFromH_",s,null);
    this.dattribs_names.addValue(prefix+".tFromH_",s,null);
    this.dattribs_names.addValue(prefix+".DiPnL_0_",s,null);
    this.dattribs_names.addValue(prefix+".iPnL_0_",s,null);
    this.dattribs_names.addValue(prefix+".PnL_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".I_A_",s,null);
    this.dattribs_names.addValue(prefix+".F_0_",s,null);
    this.dattribs_names.addValue(prefix+".F_L_",s,null);
    this.dattribs_names.addValue(prefix+".F_H_",s,null);
    this.dattribs_names.addValue(prefix+".f_0_",s,null);
    this.dattribs_names.addValue(prefix+".f_L_",s,null);
    this.dattribs_names.addValue(prefix+".f_H_",s,null);
    this.dattribs_names.addValue(prefix+".getSVol_",s,null);
    this.dattribs_names.addValue(prefix+".getVol_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".sTL_",s,null);
    this.dattribs_names.addValue(prefix+".sTH_",s,null);
    this.dattribs_names.addValue(prefix+".sT0_",s,null);
    this.dattribs_names.addValue(prefix+".sT0b_",s,null);
    n+=4;
    this.dattribs_names.addValue(prefix+".SsTL_",s,null);
    this.dattribs_names.addValue(prefix+".SsTH_",s,null);
    this.dattribs_names.addValue(prefix+".SsT0_",s,null);
    this.dattribs_names.addValue(prefix+".SsT0b_",s,null);
    n+=4;
    this.dattribs_names.addValue(prefix+".w_IHSL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IHSH_",s,null);
    this.dattribs_names.addValue(prefix+".I_SL_",s,null);
    this.dattribs_names.addValue(prefix+".I_SH_",s,null);

    return n;
}
private <T> int printTkQVMValues(final TkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=StateSaver_Values(x.sst.r);
    n+=FStateSaver_Values(x.sst.rf);
    n+=SMMStateSaver_Values(x.sst.rq);
    n+=SMMXStateSaver_Values(x.sst.rxq);
    n+=MinMaxStateSaver_Values(x.sst.rmm);
    n+=MaxStateSaver_Values(x.sst.rlm);
    n+=MaxStateSaver_Values(x.sst.rhm);
    n+=EVXData_Values(x.Th);
    n+=EVXData_Values(x.STh);
    n+=EVXData_Values(x.PnL);
    n+=EVXData_Values(x.iPnL);
    n+=EVXData_Values(x.Ji);
    n+=EVXData_Values(x.Jt);
    n+=18;
    this.dattribs.addValue(x.Svs0);
    this.dattribs.addValue(x.vFromH);
    this.dattribs.addValue(x.tFromH);
    this.dattribs.addValue(x.DiPnL_0);
    this.dattribs.addValue(x.iPnL_0);
    this.dattribs.addValue(x.PnL_0);
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.I_A);
    this.dattribs.addValue(x.F_0);
    this.dattribs.addValue(x.F_L);
    this.dattribs.addValue(x.F_H);
    this.dattribs.addValue(x.f_0);
    this.dattribs.addValue(x.f_L);
    this.dattribs.addValue(x.f_H);
    this.dattribs.addValue(x.getSVol());
    this.dattribs.addValue(x.getVol());

    n+=4;
    this.dattribs.addValue(x.sTL);
    this.dattribs.addValue(x.sTH);
    this.dattribs.addValue(x.sT0);
    this.dattribs.addValue(x.sT0b);
    n+=4;
    this.dattribs.addValue(x.SsTL);
    this.dattribs.addValue(x.SsTH);
    this.dattribs.addValue(x.SsT0);
    this.dattribs.addValue(x.SsT0b);
    n+=4;
    this.dattribs.addValue(x.w_IHSL);
    this.dattribs.addValue(x.w_IHSH);
    this.dattribs.addValue(x.I_SL);
    this.dattribs.addValue(x.I_SH);

    return n;
}


private int printUkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=MaxStateSaver_Names(prefix+".sst.rim",s);
    n+=MaxStateSaver_Names(prefix+".sst.rlm",s);
    n+=MaxStateSaver_Names(prefix+".sst.rhm",s);
    n+=MaxStateSaver_Names(prefix+".sst.risellm",s);
    n+=MaxStateSaver_Names(prefix+".sst.ribuym",s);
    n+=EVXData_Names(prefix+".VS",s);
    n+=EVXData_Names(prefix+".Th",s);
    n+=EVXData_Names(prefix+".IT",s);
    n+=EVXData_Names(prefix+".ITo",s);
    n+=EVXData_Names(prefix+".PnL_IH",s);
    n+=MMaxStateSaver_Names(prefix+".sst.rimF",s);

    n+=3;
    this.dattribs_names.addValue(prefix+".Comm_pIo_Io_0_",s,null);
    this.dattribs_names.addValue(prefix+".Comm_pI_I_0_",s,null);
    this.dattribs_names.addValue(prefix+".Comm_pI_I_H_",s,null);

    n+=27;
    this.dattribs_names.addValue(prefix+".var_pI_ITo_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_ITo_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_I0IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_I_I0IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_I0_",s,null);
    this.dattribs_names.addValue(prefix+".var_I_I0_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_Ispur_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_Proj_",s,null);
    this.dattribs_names.addValue(prefix+".theta_f_LH_",s,null);
    this.dattribs_names.addValue(prefix+".theta_F_LH_",s,null);
    this.dattribs_names.addValue(prefix+".dist1_Ip_",s,null);
    this.dattribs_names.addValue(prefix+".dist2_Ip_",s,null);
    this.dattribs_names.addValue(prefix+".distX_Ip_",s,null);
    this.dattribs_names.addValue(prefix+".distDX_Ip_",s,null);
    this.dattribs_names.addValue(prefix+".distDX_IL0_IH1_Ip_",s,null);
    this.dattribs_names.addValue(prefix+".distX_IIp_",s,null);
    this.dattribs_names.addValue(prefix+".distDX_IIp_",s,null);
    this.dattribs_names.addValue(prefix+".dist2_RIp_",s,null);
    this.dattribs_names.addValue(prefix+".distX_RIp_",s,null);
    this.dattribs_names.addValue(prefix+".dist2_ITp_",s,null);
    this.dattribs_names.addValue(prefix+".distX_ITp_",s,null);
    this.dattribs_names.addValue(prefix+".distDX_ITp_",s,null);
    this.dattribs_names.addValue(prefix+".Px_ITp_",s,null);
    this.dattribs_names.addValue(prefix+".dist2nd_ITp_",s,null);
    this.dattribs_names.addValue(prefix+".distXnd_ITp_",s,null);
    n+=20;
    this.dattribs_names.addValue(prefix+".Is_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".IL_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".IM_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Iae_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Ida0_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Iie0_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".II0_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Io0_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Is_p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".IL_p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".IM_p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".Iae_p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".Ida0_p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".Iie0_p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".II0_p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".Io0_p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".IT_IT0_p_IH0_IH1_",s,null);
    this.dattribs_names.addValue(prefix+".IT_IT0_p_IL0_IL1_",s,null);
    this.dattribs_names.addValue(prefix+".IT_IT0_p_IL0_IH1_",s,null);
    this.dattribs_names.addValue(prefix+".IT_IT0_p_IH0_IL1_",s,null);

    n+=16;
    this.dattribs_names.addValue(prefix+"._w_pnl_L_I0_",s,null);
    this.dattribs_names.addValue(prefix+"._w_pnl_H_I0_",s,null);
    this.dattribs_names.addValue(prefix+"._pI_0_",s,null);
    this.dattribs_names.addValue(prefix+"._pI_L_",s,null);
    this.dattribs_names.addValue(prefix+"._pI_H_",s,null);
    this.dattribs_names.addValue(prefix+"._I_pL_",s,null);
    this.dattribs_names.addValue(prefix+"._I_pH_",s,null);
    this.dattribs_names.addValue(prefix+"._pI_pL_",s,null);
    this.dattribs_names.addValue(prefix+"._pI_pH_",s,null);
    this.dattribs_names.addValue(prefix+"._w_pIL_IH_",s,null);
    this.dattribs_names.addValue(prefix+"._w_pIH_IH_",s,null);
    this.dattribs_names.addValue(prefix+"._w_pIL_IL_",s,null);
    this.dattribs_names.addValue(prefix+"._w_pIH_IL_",s,null);
    this.dattribs_names.addValue(prefix+".theta_pIL_pIH_",s,null);
    this.dattribs_names.addValue(prefix+".theta_pIL_IH_",s,null);
    this.dattribs_names.addValue(prefix+".theta_pIH_IH_",s,null);

    n+=11;
    this.dattribs_names.addValue(prefix+"._rI_L_",s,null);
    this.dattribs_names.addValue(prefix+"._rI_H_",s,null);
    this.dattribs_names.addValue(prefix+"._p_rI_H_",s,null);
    this.dattribs_names.addValue(prefix+"._rpI_L_",s,null);
    this.dattribs_names.addValue(prefix+"._rpI_H_",s,null);
    this.dattribs_names.addValue(prefix+"._rI_pIL_",s,null);
    this.dattribs_names.addValue(prefix+"._rI_pIH_",s,null);
    this.dattribs_names.addValue(prefix+"._w_rpIL_rIH_",s,null);
    this.dattribs_names.addValue(prefix+"._w_rpIH_rIH_",s,null);
    this.dattribs_names.addValue(prefix+"._w_rpIL_rIL_",s,null);
    this.dattribs_names.addValue(prefix+"._w_rpIH_rIL_",s,null);

    n+=28;
    this.dattribs_names.addValue(prefix+"._I_0_",s,null);
    this.dattribs_names.addValue(prefix+"._I_L_",s,null);
    this.dattribs_names.addValue(prefix+"._I_H_",s,null);
    this.dattribs_names.addValue(prefix+"._p_I_H_",s,null);
    this.dattribs_names.addValue(prefix+"._p_I_L_",s,null);
    this.dattribs_names.addValue(prefix+"._J_0_",s,null);
    this.dattribs_names.addValue(prefix+"._J_L_",s,null);
    this.dattribs_names.addValue(prefix+"._J_H_",s,null);
    this.dattribs_names.addValue(prefix+"._SJ_0_",s,null);
    this.dattribs_names.addValue(prefix+"._SJ_L_",s,null);
    this.dattribs_names.addValue(prefix+"._SJ_H_",s,null);
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".f_0_",s,null);
    this.dattribs_names.addValue(prefix+".f_L_",s,null);
    this.dattribs_names.addValue(prefix+".f_H_",s,null);
    this.dattribs_names.addValue(prefix+".F_0_",s,null);
    this.dattribs_names.addValue(prefix+".F_L_",s,null);
    this.dattribs_names.addValue(prefix+".F_H_",s,null);
    this.dattribs_names.addValue(prefix+".PA_L_",s,null);
    this.dattribs_names.addValue(prefix+".PA_H_",s,null);
    this.dattribs_names.addValue(prefix+".Idp_0_",s,null);
    this.dattribs_names.addValue(prefix+".Idp_L_",s,null);
    this.dattribs_names.addValue(prefix+".Idp_H_",s,null);
    this.dattribs_names.addValue(prefix+".dp_0_",s,null);
    this.dattribs_names.addValue(prefix+".dp_L_",s,null);
    this.dattribs_names.addValue(prefix+".dp_H_",s,null);

    return n;
}
private <T> int printUkQVMValues(final UkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=MaxStateSaver_Values(x.sst.rim);
    n+=MaxStateSaver_Values(x.sst.rlm);
    n+=MaxStateSaver_Values(x.sst.rhm);
    n+=MaxStateSaver_Values(x.sst.risellm);
    n+=MaxStateSaver_Values(x.sst.ribuym);
    n+=EVXData_Values(x.VS);
    n+=EVXData_Values(x.Th);
    n+=EVXData_Values(x.IT);
    n+=EVXData_Values(x.ITo);
    n+=EVXData_Values(x.PnL_IH);
    n+=MMaxStateSaver_Values(x.sst.rimF);

    n+=3;
    this.dattribs.addValue(x.Comm_pIo_Io_0);
    this.dattribs.addValue(x.Comm_pI_I_0);
    this.dattribs.addValue(x.Comm_pI_I_H);

    n+=27;
    this.dattribs.addValue(x.var_pI_ITo_IH);
    this.dattribs.addValue(x.p_ITo_IH);
    this.dattribs.addValue(x.var_pI_I0IH);
    this.dattribs.addValue(x.var_I_I0IH);
    this.dattribs.addValue(x.var_pI_I0);
    this.dattribs.addValue(x.var_I_I0);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.var_p_Ispur);
    this.dattribs.addValue(x.var_p_IH_Proj);
    this.dattribs.addValue(x.theta_f_LH);
    this.dattribs.addValue(x.theta_F_LH);
    this.dattribs.addValue(x.dist1_Ip);
    this.dattribs.addValue(x.dist2_Ip);
    this.dattribs.addValue(x.distX_Ip);
    this.dattribs.addValue(x.distDX_Ip);
    this.dattribs.addValue(x.distDX_IL0_IH1_Ip);
    this.dattribs.addValue(x.distX_IIp);
    this.dattribs.addValue(x.distDX_IIp);
    this.dattribs.addValue(x.dist2_RIp);
    this.dattribs.addValue(x.distX_RIp);
    this.dattribs.addValue(x.dist2_ITp);
    this.dattribs.addValue(x.distX_ITp);
    this.dattribs.addValue(x.distDX_ITp);
    this.dattribs.addValue(x.Px_ITp);
    this.dattribs.addValue(x.dist2nd_ITp);
    this.dattribs.addValue(x.distXnd_ITp);

    n+=20;
    this.dattribs.addValue(x.Is_p_IH);
    this.dattribs.addValue(x.IL_p_IH);
    this.dattribs.addValue(x.IM_p_IH);
    this.dattribs.addValue(x.Iae_p_IH);
    this.dattribs.addValue(x.Ida0_p_IH);
    this.dattribs.addValue(x.Iie0_p_IH);
    this.dattribs.addValue(x.II0_p_IH);
    this.dattribs.addValue(x.Io0_p_IH);
    this.dattribs.addValue(x.Is_p_IL);
    this.dattribs.addValue(x.IL_p_IL);
    this.dattribs.addValue(x.IM_p_IL);
    this.dattribs.addValue(x.Iae_p_IL);
    this.dattribs.addValue(x.Ida0_p_IL);
    this.dattribs.addValue(x.Iie0_p_IL);
    this.dattribs.addValue(x.II0_p_IL);
    this.dattribs.addValue(x.Io0_p_IL);
    this.dattribs.addValue(x.IT_IT0_p_IH0_IH1);
    this.dattribs.addValue(x.IT_IT0_p_IL0_IL1);
    this.dattribs.addValue(x.IT_IT0_p_IL0_IH1);
    this.dattribs.addValue(x.IT_IT0_p_IH0_IL1);

    n+=16;
    this.dattribs.addValue(x._w_pnl_L_I0);
    this.dattribs.addValue(x._w_pnl_H_I0);
    this.dattribs.addValue(x._pI_0);
    this.dattribs.addValue(x._pI_L);
    this.dattribs.addValue(x._pI_H);
    this.dattribs.addValue(x._I_pL);
    this.dattribs.addValue(x._I_pH);
    this.dattribs.addValue(x._pI_pL);
    this.dattribs.addValue(x._pI_pH);
    this.dattribs.addValue(x._w_pIL_IH);
    this.dattribs.addValue(x._w_pIH_IH);
    this.dattribs.addValue(x._w_pIL_IL);
    this.dattribs.addValue(x._w_pIH_IL);
    this.dattribs.addValue(x.theta_pIL_pIH);
    this.dattribs.addValue(x.theta_pIL_IH);
    this.dattribs.addValue(x.theta_pIH_IH);

    n+=11;
    this.dattribs.addValue(x._rI_L);
    this.dattribs.addValue(x._rI_H);
    this.dattribs.addValue(x._p_rI_H);
    this.dattribs.addValue(x._rpI_L);
    this.dattribs.addValue(x._rpI_H);
    this.dattribs.addValue(x._rI_pIL);
    this.dattribs.addValue(x._rI_pIH);
    this.dattribs.addValue(x._w_rpIL_rIH);
    this.dattribs.addValue(x._w_rpIH_rIH);
    this.dattribs.addValue(x._w_rpIL_rIL);
    this.dattribs.addValue(x._w_rpIH_rIL);

    n+=28;
    this.dattribs.addValue(x._I_0);
    this.dattribs.addValue(x._I_L);
    this.dattribs.addValue(x._I_H);
    this.dattribs.addValue(x._p_I_H);
    this.dattribs.addValue(x._p_I_L);
    this.dattribs.addValue(x._J_0);
    this.dattribs.addValue(x._J_L);
    this.dattribs.addValue(x._J_H);
    this.dattribs.addValue(x._SJ_0);
    this.dattribs.addValue(x._SJ_L);
    this.dattribs.addValue(x._SJ_H);
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.f_0);
    this.dattribs.addValue(x.f_L);
    this.dattribs.addValue(x.f_H);
    this.dattribs.addValue(x.F_0);
    this.dattribs.addValue(x.F_L);
    this.dattribs.addValue(x.F_H);
    this.dattribs.addValue(x.PA_L);
    this.dattribs.addValue(x.PA_H);
    this.dattribs.addValue(x.Idp_0);
    this.dattribs.addValue(x.Idp_L);
    this.dattribs.addValue(x.Idp_H);
    this.dattribs.addValue(x.dp_0);
    this.dattribs.addValue(x.dp_L);
    this.dattribs.addValue(x.dp_H);

    return n;
}



private int printUnQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=MMaxStateSaver_Names(prefix+".sst.rimF",s);

    n+=6;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_I_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_piL_IH_",s,null);
    this.dattribs_names.addValue(prefix+".w_piH_IH_",s,null);

    return n;
}
private <T> int printUnQVMValues(final UnQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=MMaxStateSaver_Values(x.sst.rimF);

    n+=6;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_I_H);
    this.dattribs.addValue(x.w_piL_IH);
    this.dattribs.addValue(x.w_piH_IH);

    return n;
}


private int printVkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=12;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".dI_Di0_",s,null);
    this.dattribs_names.addValue(prefix+".dI_DiL_",s,null);
    this.dattribs_names.addValue(prefix+".dI_DiH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_Di0_",s,null);
    this.dattribs_names.addValue(prefix+".dp_DiL_",s,null);
    this.dattribs_names.addValue(prefix+".dp_DiH_",s,null);
    this.dattribs_names.addValue(prefix+".I_DiL_",s,null);
    this.dattribs_names.addValue(prefix+".I_DiH_",s,null);
    this.dattribs_names.addValue(prefix+".c_DiL_",s,null);
    this.dattribs_names.addValue(prefix+".c_DiH_",s,null);
    this.dattribs_names.addValue(prefix+".varPIDt_",s,null);

    n+=16;
    this.dattribs_names.addValue(prefix+".I_iL_",s,null);
    this.dattribs_names.addValue(prefix+".I_iH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_iL_",s,null);
    this.dattribs_names.addValue(prefix+".dp_iH_",s,null);
    this.dattribs_names.addValue(prefix+".d2p_iL_",s,null);
    this.dattribs_names.addValue(prefix+".d2p_iH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_dpL_",s,null);
    this.dattribs_names.addValue(prefix+".dp_dpH_",s,null);
    this.dattribs_names.addValue(prefix+".I_dpL_",s,null);
    this.dattribs_names.addValue(prefix+".I_dpH_",s,null);
    this.dattribs_names.addValue(prefix+".dI_dpL_",s,null);
    this.dattribs_names.addValue(prefix+".dI_dpH_",s,null);
    this.dattribs_names.addValue(prefix+".d2p_DiL_",s,null);
    this.dattribs_names.addValue(prefix+".d2p_DiH_",s,null);
    this.dattribs_names.addValue(prefix+".a_d2I_d2p_",s,null);
    this.dattribs_names.addValue(prefix+".aiH_I_dp_",s,null);
    n+=4;
    this.dattribs_names.addValue(prefix+".Idp_IdpL_",s,null);
    this.dattribs_names.addValue(prefix+".Idp_IdpH_",s,null);
    this.dattribs_names.addValue(prefix+".w_IdpL_iH_",s,null);
    this.dattribs_names.addValue(prefix+".w_IdpH_iH_",s,null);


    return n;
}
private <T> int printVkQVMValues(final VkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=12;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.dI_Di0);
    this.dattribs.addValue(x.dI_DiL);
    this.dattribs.addValue(x.dI_DiH);
    this.dattribs.addValue(x.dp_Di0);
    this.dattribs.addValue(x.dp_DiL);
    this.dattribs.addValue(x.dp_DiH);
    this.dattribs.addValue(x.I_DiL);
    this.dattribs.addValue(x.I_DiH);
    this.dattribs.addValue(x.c_DiL);
    this.dattribs.addValue(x.c_DiH);
    this.dattribs.addValue(x.varPIDt);

    n+=16;
    this.dattribs.addValue(x.I_iL);
    this.dattribs.addValue(x.I_iH);
    this.dattribs.addValue(x.dp_iL);
    this.dattribs.addValue(x.dp_iH);
    this.dattribs.addValue(x.d2p_iL);
    this.dattribs.addValue(x.d2p_iH);
    this.dattribs.addValue(x.dp_dpL);
    this.dattribs.addValue(x.dp_dpH);
    this.dattribs.addValue(x.I_dpL);
    this.dattribs.addValue(x.I_dpH);
    this.dattribs.addValue(x.dI_dpL);
    this.dattribs.addValue(x.dI_dpH);
    this.dattribs.addValue(x.d2p_DiL);
    this.dattribs.addValue(x.d2p_DiH);
    this.dattribs.addValue(x.a_d2I_d2p);
    this.dattribs.addValue(x.aiH_I_dp);

    n+=4;
    this.dattribs.addValue(x.Idp_IdpL);
    this.dattribs.addValue(x.Idp_IdpH);
    this.dattribs.addValue(x.w_IdpL_iH);
    this.dattribs.addValue(x.w_IdpH_iH);


    return n;
}



private int printDynPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".pEQ_",s,null);
    this.dattribs_names.addValue(prefix+".pEB_",s,null);
    this.dattribs_names.addValue(prefix+".pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".I_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".I_pnlH_",s,null);

    return n;
}

private int printDynPnLValues(final DynPnL x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.pEQ);
    this.dattribs.addValue(x.pEB);
    this.dattribs.addValue(x.pnlL);
    this.dattribs.addValue(x.pnlH);
    this.dattribs.addValue(x.I_pnlL);
    this.dattribs.addValue(x.I_pnlH);

    return n;
}


private int printVar2INames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".norm_",s,null);
    this.dattribs_names.addValue(prefix+".f_l_",s,null);
    this.dattribs_names.addValue(prefix+".f_a_",s,null);
    this.dattribs_names.addValue(prefix+".f_aa_",s,null);
    this.dattribs_names.addValue(prefix+".p_",s,null);
    this.dattribs_names.addValue(prefix+".pd_",s,null);
    this.dattribs_names.addValue(prefix+".i_",s,null);
    this.dattribs_names.addValue(prefix+".var2i_",s,null);

    return n;
}

private int printVar2IValues(final Var2I x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.norm);
    this.dattribs.addValue(x.f_l);
    this.dattribs.addValue(x.f_a);
    this.dattribs.addValue(x.f_aa);
    this.dattribs.addValue(x.p);
    this.dattribs.addValue(x.pd);
    this.dattribs.addValue(x.i);
    this.dattribs.addValue(x.var2i);

    return n;
}


private int printDynVarPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=12;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".pdo_IL_",s,null);
    this.dattribs_names.addValue(prefix+".pd_IL_",s,null);
    this.dattribs_names.addValue(prefix+".dif_IL_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pdo_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pd_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dif_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pDiff_",s,null);

    return n;
}

private int printDynVarPnLValues(final DynVarPnL x)
{
    int n=0;

    n+=12;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.pdo_IL);
    this.dattribs.addValue(x.pd_IL);
    this.dattribs.addValue(x.dif_IL);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pdo_IH);
    this.dattribs.addValue(x.pd_IH);
    this.dattribs.addValue(x.dif_IH);
    this.dattribs.addValue(x.pDiff);

    return n;
}

private int printDynVarXkNames(final String prefix,final String s)
{
    int n=0;

    n+=16;
    this.dattribs_names.addValue(prefix+".varpIk_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_EQ_",s,null);
    this.dattribs_names.addValue(prefix+".pa_EQ_",s,null);
    this.dattribs_names.addValue(prefix+".pat_EQ_",s,null);
    this.dattribs_names.addValue(prefix+".pm_EQ_",s,null);
    this.dattribs_names.addValue(prefix+".pmv_EQ_",s,null);
    this.dattribs_names.addValue(prefix+".xpm_EQ_",s,null);
    this.dattribs_names.addValue(prefix+".pax_MU_",s,null);
    this.dattribs_names.addValue(prefix+".pd_EQ_",s,null);
    this.dattribs_names.addValue(prefix+".pad_EQ_",s,null);
    this.dattribs_names.addValue(prefix+".pdm_EQ_",s,null);
    this.dattribs_names.addValue(prefix+".x_EQ_",s,null);
    this.dattribs_names.addValue(prefix+".xa_EQ_",s,null);
    this.dattribs_names.addValue(prefix+".xm_EQ_",s,null);

    return n;
}

private int printDynVarXkValues(final DynVarXk x)
{
    int n=0;

    n+=16;
    this.dattribs.addValue(x.varpIk);
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.p_EQ);
    this.dattribs.addValue(x.pa_EQ);
    this.dattribs.addValue(x.pat_EQ);
    this.dattribs.addValue(x.pm_EQ);
    this.dattribs.addValue(x.pmv_EQ);
    this.dattribs.addValue(x.xpm_EQ);
    this.dattribs.addValue(x.pax_MU);
    this.dattribs.addValue(x.pd_EQ);
    this.dattribs.addValue(x.pad_EQ);
    this.dattribs.addValue(x.pdm_EQ);
    this.dattribs.addValue(x.x_EQ);
    this.dattribs.addValue(x.xa_EQ);
    this.dattribs.addValue(x.xm_EQ);

    return n;
}

private int printDynVarXiNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".x_IX_",s,null);
    this.dattribs_names.addValue(prefix+".sPnlMuB_",s,null);
    this.dattribs_names.addValue(prefix+".sPnlMuA_",s,null);
    this.dattribs_names.addValue(prefix+".sPnlIB_",s,null);
    this.dattribs_names.addValue(prefix+".sPnlIA_",s,null);
    this.dattribs_names.addValue(prefix+".sMuB_",s,null);
    this.dattribs_names.addValue(prefix+".sMuA_",s,null);

    return n;
}

private int printDynVarXiValues(final DynVarXi x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.x_IX);
    this.dattribs.addValue(x.sPnlMuB);
    this.dattribs.addValue(x.sPnlMuA);
    this.dattribs.addValue(x.sPnlIB);
    this.dattribs.addValue(x.sPnlIA);
    this.dattribs.addValue(x.sMuB);
    this.dattribs.addValue(x.sMuA);

    return n;
}

private int printDynVarXNames(final String prefix,final String s)
{
    int n=0;

    n+=13;
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".x_IX_",s,null);
    this.dattribs_names.addValue(prefix+".px_IX_",s,null);
    this.dattribs_names.addValue(prefix+".pv_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX2_",s,null);
    this.dattribs_names.addValue(prefix+".pv_IX2_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX2_",s,null);
    this.dattribs_names.addValue(prefix+".x_IX2_",s,null);
    this.dattribs_names.addValue(prefix+".pax_MU_",s,null);
    this.dattribs_names.addValue(prefix+".pB_",s,null);
    this.dattribs_names.addValue(prefix+".pF_",s,null);
    this.dattribs_names.addValue(prefix+".Pjtau_",s,null);

    n+=printACorrNames(prefix+".px_mu",s);
    n+=printACorrNames(prefix+".px_i",s);

    return n;
}

private int printDynVarXValues(final DynVarX x)
{
    int n=0;

    n+=13;
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.x_IX);
    this.dattribs.addValue(x.px_IX);
    this.dattribs.addValue(x.pv_IX);
    this.dattribs.addValue(x.p_IX2);
    this.dattribs.addValue(x.pv_IX2);
    this.dattribs.addValue(x.I_IX2);
    this.dattribs.addValue(x.x_IX2);
    this.dattribs.addValue(x.pax_MU);
    this.dattribs.addValue(x.pB);
    this.dattribs.addValue(x.pF);
    this.dattribs.addValue(x.Pjtau);

    n+=printACorrValues(x.px_mu);
    n+=printACorrValues(x.px_i);

    return n;
}

private int printDynVarCNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".Id_cL_",s,null);
    this.dattribs_names.addValue(prefix+".p_cL_",s,null);
    this.dattribs_names.addValue(prefix+".Id_cH_",s,null);
    this.dattribs_names.addValue(prefix+".p_cH_",s,null);

    return n;
}

private int printDynVarCValues(final DynVarC x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.Id_cL);
    this.dattribs.addValue(x.p_cL);
    this.dattribs.addValue(x.Id_cH);
    this.dattribs.addValue(x.p_cH);

    return n;
}

private int printDynVarFiniteCNames(final String prefix,final String s)
{
    int n=0;

    n+=14;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".var_pn_IX_",s,null);
    this.dattribs_names.addValue(prefix+".var2_pn_IX_",s,null);
    this.dattribs_names.addValue(prefix+".Id_cL_",s,null);
    this.dattribs_names.addValue(prefix+".pd_cL_",s,null);
    this.dattribs_names.addValue(prefix+".Id_cH_",s,null);
    this.dattribs_names.addValue(prefix+".pd_cH_",s,null);
    this.dattribs_names.addValue(prefix+".distance2_cL_cH_",s,null);
    this.dattribs_names.addValue(prefix+".distance2_IH_cH_",s,null);
    this.dattribs_names.addValue(prefix+".distance2_IH_cL_",s,null);

    return n;
}

private int printDynVarFiniteCValues(final DynVarFiniteC x)
{
    int n=0;

    n+=14;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.var_pn_IX);
    this.dattribs.addValue(x.var2_pn_IX);
    this.dattribs.addValue(x.Id_cL);
    this.dattribs.addValue(x.pd_cL);
    this.dattribs.addValue(x.Id_cH);
    this.dattribs.addValue(x.pd_cH);
    this.dattribs.addValue(x.distance2_cL_cH);
    this.dattribs.addValue(x.distance2_IH_cH);
    this.dattribs.addValue(x.distance2_IH_cL);

    return n;
}


private int printDynVarFiniteDNames(final String prefix,final String s)
{
    int n=0;

    n+=12;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".Id_cL_",s,null);
    this.dattribs_names.addValue(prefix+".pd_cL_",s,null);
    this.dattribs_names.addValue(prefix+".Id_cH_",s,null);
    this.dattribs_names.addValue(prefix+".pd_cH_",s,null);
    this.dattribs_names.addValue(prefix+".distance2_cL_cH_",s,null);
    this.dattribs_names.addValue(prefix+".distance2_IH_cH_",s,null);
    this.dattribs_names.addValue(prefix+".distance2_IH_cL_",s,null);

    return n;
}

private int printDynVarFiniteDValues(final DynVarFiniteD x)
{
    int n=0;

    n+=12;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.Id_cL);
    this.dattribs.addValue(x.pd_cL);
    this.dattribs.addValue(x.Id_cH);
    this.dattribs.addValue(x.pd_cH);
    this.dattribs.addValue(x.distance2_cL_cH);
    this.dattribs.addValue(x.distance2_IH_cH);
    this.dattribs.addValue(x.distance2_IH_cL);

    return n;
}

private int printDynCPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_IX_",s,null);
    this.dattribs_names.addValue(prefix+".var_pn_IX_",s,null);
    this.dattribs_names.addValue(prefix+".var2_pn_IX_",s,null);
    this.dattribs_names.addValue(prefix+".test_var2_pn_IX_",s,null);

    return n;
}

private int printDynCPnLValues(final DynCPnL x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.var_pI_IX);
    this.dattribs.addValue(x.var_pn_IX);
    this.dattribs.addValue(x.var2_pn_IX);
    this.dattribs.addValue(x.test_var2_pn_IX);

    return n;
}


private int printDynVar2CNames(final String prefix,final String s)
{
    int n=0;

    n+=19;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".pi_sum_",s,null);
    this.dattribs_names.addValue(prefix+".pid_sum_",s,null);
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".pd_IX_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_IX_",s,null);
    this.dattribs_names.addValue(prefix+".var2I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_var2I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_sum_",s,null);
    this.dattribs_names.addValue(prefix+".var_pI_sumA_",s,null);
    this.dattribs_names.addValue(prefix+".dp_var_pI_sumA_",s,null);
    this.dattribs_names.addValue(prefix+".p2_sum_",s,null);
    this.dattribs_names.addValue(prefix+".pp_sum_",s,null);
    this.dattribs_names.addValue(prefix+".pa_sum_",s,null);
    this.dattribs_names.addValue(prefix+".pe_sum_",s,null);
    this.dattribs_names.addValue(prefix+".pae_sum_",s,null);

    return n;
}

private int printDynVar2CValues(final DynVar2C x)
{
    int n=0;

    n+=19;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.pi_sum);
    this.dattribs.addValue(x.pid_sum);
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.pd_IX);
    this.dattribs.addValue(x.var_pI_IX);
    this.dattribs.addValue(x.var2I_IX);
    this.dattribs.addValue(x.p_var2I_IX);
    this.dattribs.addValue(x.var_pI_sum);
    this.dattribs.addValue(x.var_pI_sumA);
    this.dattribs.addValue(x.dp_var_pI_sumA);
    this.dattribs.addValue(x.p2_sum);
    this.dattribs.addValue(x.pp_sum);
    this.dattribs.addValue(x.pa_sum);
    this.dattribs.addValue(x.pe_sum);
    this.dattribs.addValue(x.pae_sum);

    return n;
}


private int printDynVar2CaNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=4;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var1pI_",s,null);
    this.dattribs_names.addValue(prefix+".dp2It_",s,null);
    this.dattribs_names.addValue(prefix+".dp1P_",s,null);
    return n;
}

private int printDynVar2CaValues(final DynVar2Ca x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=4;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var1pI);
    this.dattribs.addValue(x.dp2It);
    this.dattribs.addValue(x.dp1P);
    return n;
}

private int printDynVarVNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".V",s);
    n+=2;
    this.dattribs_names.addValue(prefix+".pi_sum_",s,null);
    this.dattribs_names.addValue(prefix+".pv_sum_",s,null);
    return n;
}

private int printDynVarVValues(final DynVarV x)
{
    int n=0;

    n+=EVXData_Values(x.V);
    n+=2;
    this.dattribs.addValue(x.pi_sum);
    this.dattribs.addValue(x.pv_sum);
    return n;
}
private int printDynVar2PiPtNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=8;
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".varPit_",s,null);
    this.dattribs_names.addValue(prefix+".varAPit_",s,null);
    this.dattribs_names.addValue(prefix+".varHPit_",s,null);
    this.dattribs_names.addValue(prefix+".varPi_",s,null);
    this.dattribs_names.addValue(prefix+".varAPi_",s,null);
    this.dattribs_names.addValue(prefix+".varHPi_",s,null);
    return n;
}

private int printDynVar2PiPtValues(final DynVar2PiPt x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=8;
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.varPit);
    this.dattribs.addValue(x.varAPit);
    this.dattribs.addValue(x.varHPit);
    this.dattribs.addValue(x.varPi);
    this.dattribs.addValue(x.varAPi);
    this.dattribs.addValue(x.varHPi);
    return n;
}

private int printDynVar2ProjNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=3;
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_EQ_",s,null);
    return n;
}

private int printDynVar2ProjValues(final DynVar2Proj x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=3;
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_EQ);
    return n;
}


private int printDynVar2DynImpactNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=4;
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pa_sum_",s,null);
    this.dattribs_names.addValue(prefix+".dp_EQ_",s,null);
    n+=printACorrNames(prefix+".a",s);
    n+=printACorrNames(prefix+".aI0",s);
    n+=printACorrNames(prefix+".ap0",s);
    n+=printACorrNames(prefix+".am",s);
    n+=printACorrNames(prefix+".am0",s);

    return n;
}

private int printDynVar2DynImpactValues(final DynVar2DynImpact x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=4;
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pa_sum);
    this.dattribs.addValue(x.dp_EQ);
    n+=printACorrValues(x.a);
    n+=printACorrValues(x.aI0);
    n+=printACorrValues(x.ap0);
    n+=printACorrValues(x.am);
    n+=printACorrValues(x.am0);

    return n;
}

private int printDynVar2DIDynImpactNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=6;
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pa_sum_",s,null);
    this.dattribs_names.addValue(prefix+".var2_ppIH_",s,null);
    this.dattribs_names.addValue(prefix+".peq_I0_",s,null);
    this.dattribs_names.addValue(prefix+".peq_oI0_",s,null);
    n+=printACorrNames(prefix+".a",s);
    n+=printACorrNames(prefix+".aa",s);

    return n;
}

private int printDynVar2DIDynImpactValues(final DynVar2DIDynImpact x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=6;
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pa_sum);
    this.dattribs.addValue(x.var2_ppIH);
    this.dattribs.addValue(x.peq_I0);
    this.dattribs.addValue(x.peq_oI0);
    n+=printACorrValues(x.a);
    n+=printACorrValues(x.aa);

    return n;
}


private int printDynVarICNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".C",s);
    n+=3;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".vp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".rvp_IH_",s,null);
    return n;
}

private int printDynVarICValues(final DynVarIC x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.C);
    n+=3;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.vp_IH);
    this.dattribs.addValue(x.rvp_IH);
    return n;
}

private int printDynVarIC2Names(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);
    n+=2;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".vp_IH_",s,null);
    return n;
}

private int printDynVarIC2Values(final DynVarIC2 x)
{
    int n=0;

    n+=EVXData_Values(x.I);
    n+=2;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.vp_IH);
    return n;
}

private int printDynVarIPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".p_A_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".pd_IL_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".t_IL_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pd_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".t_IH_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printDynVarIPnLValues(final DynVarIPnL x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.p_A);
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.pd_IL);
    this.dattribs.addValue(x.var_p_IL);
    this.dattribs.addValue(x.t_IL);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pd_IH);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.t_IH);

    n+=EVXData_Values(x.I);

    return n;
}


private int printDynpIVNames(final String prefix,final String s)
{
    int n=0;

    n+=10;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".V_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_V_",s,null);
    this.dattribs_names.addValue(prefix+".V_V_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH2_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH2_",s,null);

    return n;
}

private int printDynpIVValues(final DynpIV x)
{
    int n=0;

    n+=10;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.V_IH);
    this.dattribs.addValue(x.I_V);
    this.dattribs.addValue(x.p_V);
    this.dattribs.addValue(x.V_V);
    this.dattribs.addValue(x.I_IH2);
    this.dattribs.addValue(x.p_IH2);

    return n;
}


private int printTryD2Names(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".p_",s,null);
    this.dattribs_names.addValue(prefix+".px_",s,null);
    this.dattribs_names.addValue(prefix+".I.F0_",s,null);
    this.dattribs_names.addValue(prefix+".I.F2_",s,null);
    this.dattribs_names.addValue(prefix+".I.F4_",s,null);
    this.dattribs_names.addValue(prefix+".pI.F0_",s,null);
    this.dattribs_names.addValue(prefix+".pI.F2_",s,null);
    this.dattribs_names.addValue(prefix+".pI.F4_",s,null);

    return n;
}

private int printTryD2Values(final TryD2 x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.p);
    this.dattribs.addValue(x.px);
    this.dattribs.addValue(x.I.F0);
    this.dattribs.addValue(x.I.F2);
    this.dattribs.addValue(x.I.F4);
    this.dattribs.addValue(x.pI.F0);
    this.dattribs.addValue(x.pI.F2);
    this.dattribs.addValue(x.pI.F4);

    return n;
}

private int printDyn3PnLNames(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".p_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".I_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".p_pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".I_pnlH_",s,null);

    return n;
}

private int printDyn3PnLValues(final Dyn3PnL x)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.p_pnlL);
    this.dattribs.addValue(x.I_pnlL);
    this.dattribs.addValue(x.p_pnlH);
    this.dattribs.addValue(x.I_pnlH);

    return n;
}

private int printDynDVarPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=15;
    this.dattribs_names.addValue(prefix+".i_I0_",s,null);
    this.dattribs_names.addValue(prefix+".i_IL_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".i_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);

    this.dattribs_names.addValue(prefix+".dII_L.ind_",s,null);
    this.dattribs_names.addValue(prefix+".dII_L.rate_",s,null);
    this.dattribs_names.addValue(prefix+".dII_L.p_",s,null);
    this.dattribs_names.addValue(prefix+".dII_L.pd_",s,null);
    this.dattribs_names.addValue(prefix+".dII_L.dif_",s,null);

    this.dattribs_names.addValue(prefix+".dII_H.ind_",s,null);
    this.dattribs_names.addValue(prefix+".dII_H.rate_",s,null);
    this.dattribs_names.addValue(prefix+".dII_H.p_",s,null);
    this.dattribs_names.addValue(prefix+".dII_H.pd_",s,null);
    this.dattribs_names.addValue(prefix+".dII_H.dif_",s,null);

    return n;
}

private int printDynDVarPnLValues(final DynDVarPnL x)
{
    int n=0;

    n+=15;
    this.dattribs.addValue(x.i_I0);
    this.dattribs.addValue(x.i_IL);
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.i_IH);
    this.dattribs.addValue(x.p_IH);

    this.dattribs.addValue(x.dII_L.ind);
    this.dattribs.addValue(x.dII_L.rate);
    this.dattribs.addValue(x.dII_L.p);
    this.dattribs.addValue(x.dII_L.pd);
    this.dattribs.addValue(x.dII_L.dif);

    this.dattribs.addValue(x.dII_H.ind);
    this.dattribs.addValue(x.dII_H.rate);
    this.dattribs.addValue(x.dII_H.p);
    this.dattribs.addValue(x.dII_H.pd);
    this.dattribs.addValue(x.dII_H.dif);

    return n;
}



private int printDynEHPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=17;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pd_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dir0_",s,null);
    this.dattribs_names.addValue(prefix+".dirL_",s,null);
    this.dattribs_names.addValue(prefix+".dirH_",s,null);
    this.dattribs_names.addValue(prefix+".dirH0_",s,null);
    this.dattribs_names.addValue(prefix+".dirS_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_pL_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_pH_",s,null);
    this.dattribs_names.addValue(prefix+".pi_pL_",s,null);
    this.dattribs_names.addValue(prefix+".pi_pH_",s,null);
    this.dattribs_names.addValue(prefix+".i_pL_",s,null);
    this.dattribs_names.addValue(prefix+".i_pH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH2_",s,null);
    this.dattribs_names.addValue(prefix+".i_IH2_",s,null);
    this.dattribs_names.addValue(prefix+".dirH2_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printDynEHPnLValues(final DynEHPnL x)
{
    int n=0;

    n+=17;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pd_IH);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.dir0);
    this.dattribs.addValue(x.dirL);
    this.dattribs.addValue(x.dirH);
    this.dattribs.addValue(x.dirH0);
    this.dattribs.addValue(x.dirS);
    this.dattribs.addValue(x.pnl_pL);
    this.dattribs.addValue(x.pnl_pH);
    this.dattribs.addValue(x.pi_pL);
    this.dattribs.addValue(x.pi_pH);
    this.dattribs.addValue(x.i_pL);
    this.dattribs.addValue(x.i_pH);
    this.dattribs.addValue(x.p_IH2);
    this.dattribs.addValue(x.i_IH2);
    this.dattribs.addValue(x.dirH2);


    n+=EVXData_Values(x.I);

    return n;
}


private int printDynVHPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=10;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dir_a_",s,null);
    this.dattribs_names.addValue(prefix+".dp_a_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".sw_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pb_IH_",s,null);
    this.dattribs_names.addValue(prefix+".ib_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pd_IH_",s,null);
    this.dattribs_names.addValue(prefix+".id_IH_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printDynVHPnLValues(final DynVHPnL x)
{
    int n=0;

    n+=10;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.dir_a);
    this.dattribs.addValue(x.dp_a);
    this.dattribs.addValue(x.dp_IH);
    this.dattribs.addValue(x.sw_p_IH);
    this.dattribs.addValue(x.pb_IH);
    this.dattribs.addValue(x.ib_IH);
    this.dattribs.addValue(x.pd_IH);
    this.dattribs.addValue(x.id_IH);

    n+=EVXData_Values(x.I);

    return n;
}

private int printDynDVHPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_ID_",s,null);
    this.dattribs_names.addValue(prefix+".I_ID_",s,null);

    return n;
}

private int printDynDVHPnLValues(final DynDVHPnL x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.dp_IH);
    this.dattribs.addValue(x.p_ID);
    this.dattribs.addValue(x.I_ID);

    return n;
}

private int printDynHPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=12;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_H1_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH1_",s,null);
    this.dattribs_names.addValue(prefix+".dir_",s,null);
    this.dattribs_names.addValue(prefix+".PfI_0_",s,null);
    this.dattribs_names.addValue(prefix+".PfI_L_",s,null);
    this.dattribs_names.addValue(prefix+".PfI_H_",s,null);
    this.dattribs_names.addValue(prefix+".distance2_IH_cL_",s,null);
    this.dattribs_names.addValue(prefix+".distance2_IH_cH_",s,null);

    n+=EVXData_Names(prefix+".Ipnl",s);

    return n;
}

private int printDynHPnLValues(final DynHPnL x)
{
    int n=0;

    n+=12;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.I_H1);
    this.dattribs.addValue(x.p_IH1);
    this.dattribs.addValue(x.dir);
    this.dattribs.addValue(x.PfI_0);
    this.dattribs.addValue(x.PfI_L);
    this.dattribs.addValue(x.PfI_H);
    this.dattribs.addValue(x.distance2_IH_cL);
    this.dattribs.addValue(x.distance2_IH_cH);

    n+=EVXData_Values(x.Ipnl);

    return n;
}

private int printDynVTPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=9;
    this.dattribs_names.addValue(prefix+".pnl_I_",s,null);
    this.dattribs_names.addValue(prefix+".p_I_",s,null);
    this.dattribs_names.addValue(prefix+".I_I_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_V_",s,null);
    this.dattribs_names.addValue(prefix+".p_V_",s,null);
    this.dattribs_names.addValue(prefix+".I_V_",s,null);
    this.dattribs_names.addValue(prefix+".V_",s,null);
    this.dattribs_names.addValue(prefix+".varPnL_",s,null);
    this.dattribs_names.addValue(prefix+".varT_",s,null);

    n+=EVXData_Names(prefix+".VT",s);

    return n;
}

private int printDynVTPnLValues(final DynVTPnL x)
{
    int n=0;

    n+=9;
    this.dattribs.addValue(x.pnl_I);
    this.dattribs.addValue(x.p_I);
    this.dattribs.addValue(x.I_I);
    this.dattribs.addValue(x.pnl_V);
    this.dattribs.addValue(x.p_V);
    this.dattribs.addValue(x.I_V);
    this.dattribs.addValue(x.V);
    this.dattribs.addValue(x.varPnL);
    this.dattribs.addValue(x.varT);

    n+=EVXData_Values(x.VT);

    return n;
}

private int printDynIVTNames(final String prefix,final String s)
{
    int n=0;

    n+=11;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_IL_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".Ia_IL_",s,null);
    this.dattribs_names.addValue(prefix+".pa_IL_",s,null);
    this.dattribs_names.addValue(prefix+".peq_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Ia_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pa_IH_",s,null);
    this.dattribs_names.addValue(prefix+".peq_IH_",s,null);

    return n;
}

private int printDynIVTValues(final DynIVT x)
{
    int n=0;

    n+=11;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_IL);
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.Ia_IL);
    this.dattribs.addValue(x.pa_IL);
    this.dattribs.addValue(x.peq_IL);
    this.dattribs.addValue(x.I_IH);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.Ia_IH);
    this.dattribs.addValue(x.pa_IH);
    this.dattribs.addValue(x.peq_IH);

    return n;
}


private int printDREVXDataNames(final String prefix,final String s)
{
    int n=0;

    n+=12;
    this.dattribs_names.addValue(prefix+".p_0_",s,null);
    this.dattribs_names.addValue(prefix+".i_0_",s,null);
    this.dattribs_names.addValue(prefix+".var_0_",s,null);
    this.dattribs_names.addValue(prefix+".p_L_",s,null);
    this.dattribs_names.addValue(prefix+".i_L_",s,null);
    this.dattribs_names.addValue(prefix+".var_L_",s,null);
    this.dattribs_names.addValue(prefix+".f_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_H_",s,null);
    this.dattribs_names.addValue(prefix+".i_H_",s,null);
    this.dattribs_names.addValue(prefix+".var_H_",s,null);
    this.dattribs_names.addValue(prefix+".f_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_m_",s,null);

    n+=EVXData_Names(prefix+".ev",s);

    return n;
}

private int printDREVXDataValues(final DREVXData x)
{
    int n=0;

    n+=12;
    this.dattribs.addValue(x.p_0);
    this.dattribs.addValue(x.i_0);
    this.dattribs.addValue(x.var_0);
    this.dattribs.addValue(x.p_L);
    this.dattribs.addValue(x.i_L);
    this.dattribs.addValue(x.var_L);
    this.dattribs.addValue(x.f_L);
    this.dattribs.addValue(x.p_H);
    this.dattribs.addValue(x.i_H);
    this.dattribs.addValue(x.var_H);
    this.dattribs.addValue(x.f_H);
    this.dattribs.addValue(x.p_m);

    n+=EVXData_Values(x.ev);

    return n;
}


private int printDynRPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=14;
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_p0_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var_I_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Im_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pm_IH_",s,null);
    this.dattribs_names.addValue(prefix+".rm_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Dpsi2x0_",s,null);
    this.dattribs_names.addValue(prefix+".rateT_IH_",s,null);
    this.dattribs_names.addValue(prefix+".rateI_IH_",s,null);
    this.dattribs_names.addValue(prefix+".f_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Df_IH_",s,null);

    return n;
}

private int printDynRPnLValues(final DynRPnL x)
{
    int n=0;

    n+=14;
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.var_p_IH);
    this.dattribs.addValue(x.var_p0_IH);
    this.dattribs.addValue(x.var_I_IH);
    this.dattribs.addValue(x.Im_IH);
    this.dattribs.addValue(x.pm_IH);
    this.dattribs.addValue(x.rm_IH);
    this.dattribs.addValue(x.Dpsi2x0);
    this.dattribs.addValue(x.rateT_IH);
    this.dattribs.addValue(x.rateI_IH);
    this.dattribs.addValue(x.f_IH);
    this.dattribs.addValue(x.Df_IH);

    return n;
}


private int printDynRXPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pr_IH_",s,null);
    this.dattribs_names.addValue(prefix+".ir_IH_",s,null);
    this.dattribs_names.addValue(prefix+".tr_IH_",s,null);

    n+=EVXData_Names(prefix+".ev",s);

    return n;
}

private int printDynRXPnLValues(final DynRXPnL x)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pr_IH);
    this.dattribs.addValue(x.ir_IH);
    this.dattribs.addValue(x.tr_IH);

    n+=EVXData_Values(x.ev);

    return n;
}


private int printDynRPnLINames(final String prefix,final String s)
{
    int n=0;

    n+=8;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".px_",s,null);
    this.dattribs_names.addValue(prefix+".pas_",s,null);
    this.dattribs_names.addValue(prefix+".Ix_",s,null);

    return n;
}

private int printDynRPnLIValues(final DynRPnLI x)
{
    int n=0;

    n+=8;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.px);
    this.dattribs.addValue(x.pas);
    this.dattribs.addValue(x.Ix);

    return n;
}


private int printDynMUPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=7;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".pi_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pmu_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pas_IH_",s,null);

    return n;
}

private int printDynMUPnLValues(final DynMUPnL x)
{
    int n=0;

    n+=7;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.pi_IH);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.pmu_IH);
    this.dattribs.addValue(x.pas_IH);

    return n;
}

private int printDynSPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=11;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_IH_",s,null);
    this.dattribs_names.addValue(prefix+".I_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".p_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".I_pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".p_pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_pnlH_",s,null);

    return n;
}

private int printDynSPnLValues(final DynSPnL x)
{
    int n=0;

    n+=11;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pnl_IH);
    this.dattribs.addValue(x.I_pnlL);
    this.dattribs.addValue(x.p_pnlL);
    this.dattribs.addValue(x.w_IH_pnlL);
    this.dattribs.addValue(x.I_pnlH);
    this.dattribs.addValue(x.p_pnlH);
    this.dattribs.addValue(x.w_IH_pnlH);

    return n;
}

private int printDynSDPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".I",s);

    n+=17;
    this.dattribs_names.addValue(prefix+".p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".pd_IX_",s,null);
    this.dattribs_names.addValue(prefix+".I_IX_",s,null);
    this.dattribs_names.addValue(prefix+".DI_IX_",s,null);
    this.dattribs_names.addValue(prefix+".avar_p_IX_",s,null);
    this.dattribs_names.addValue(prefix+".p_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".pd_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".I_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".DI_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".avar_p_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".p_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".pd_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".I_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".DI_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".avar_p_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".dist2_IX_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".dist2_IX_DIH_",s,null);

    return n;
}

private int printDynSDPnLValues(final DynSDPnL x)
{
    int n=0;
    n+=EVXData_Values(x.I);

    n+=17;
    this.dattribs.addValue(x.p_IX);
    this.dattribs.addValue(x.pd_IX);
    this.dattribs.addValue(x.I_IX);
    this.dattribs.addValue(x.DI_IX);
    this.dattribs.addValue(x.avar_p_IX);
    this.dattribs.addValue(x.p_DIL);
    this.dattribs.addValue(x.pd_DIL);
    this.dattribs.addValue(x.I_DIL);
    this.dattribs.addValue(x.DI_DIL);
    this.dattribs.addValue(x.avar_p_DIL);
    this.dattribs.addValue(x.p_DIH);
    this.dattribs.addValue(x.pd_DIH);
    this.dattribs.addValue(x.I_DIH);
    this.dattribs.addValue(x.DI_DIH);
    this.dattribs.addValue(x.avar_p_DIH);
    this.dattribs.addValue(x.dist2_IX_DIL);
    this.dattribs.addValue(x.dist2_IX_DIH);

    return n;
}

private int printDynSD2PnLNames(final String prefix,final String s)
{
    int n=0;

    n+=EVXData_Names(prefix+".ID",s);

    n+=11;
    this.dattribs_names.addValue(prefix+".p_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".pd_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".I_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".DI_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".avar_p_DIL_",s,null);
    this.dattribs_names.addValue(prefix+".p_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".pd_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".I_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".DI_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".avar_p_DIH_",s,null);
    this.dattribs_names.addValue(prefix+".dist2_DIL_DIH_",s,null);

    return n;
}

private int printDynSD2PnLValues(final DynSD2PnL x)
{
    int n=0;
    n+=EVXData_Values(x.ID);

    n+=11;
    this.dattribs.addValue(x.p_DIL);
    this.dattribs.addValue(x.pd_DIL);
    this.dattribs.addValue(x.I_DIL);
    this.dattribs.addValue(x.DI_DIL);
    this.dattribs.addValue(x.avar_p_DIL);
    this.dattribs.addValue(x.p_DIH);
    this.dattribs.addValue(x.pd_DIH);
    this.dattribs.addValue(x.I_DIH);
    this.dattribs.addValue(x.DI_DIH);
    this.dattribs.addValue(x.avar_p_DIH);
    this.dattribs.addValue(x.dist2_DIL_DIH);

    return n;
}


private int printDynRPnLlastNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".I_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".p_pnlL_",s,null);
    this.dattribs_names.addValue(prefix+".I_pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_pnlH_",s,null);
    this.dattribs_names.addValue(prefix+".p_pnlH_",s,null);

    return n;
}

private int printDynRPnLlastValues(final DynRPnLlast x)
{
    int n=0;

    n+=6;

    this.dattribs.addValue(x.I_pnlL);
    this.dattribs.addValue(x.pnl_pnlL);
    this.dattribs.addValue(x.p_pnlL);
    this.dattribs.addValue(x.I_pnlH);
    this.dattribs.addValue(x.pnl_pnlH);
    this.dattribs.addValue(x.p_pnlH);

    return n;
}

private int printDynTIPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=3;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_I0_",s,null);
    this.dattribs_names.addValue(prefix+".var_p_IHI0_",s,null);

    return n;
}

private int printDynTIPnLValues(final DynTIPnL x)
{
    int n=0;

    n+=3;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.p_I0);
    this.dattribs.addValue(x.var_p_IHI0);

    return n;
}


private int printDynMPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".avar_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pb_IH_",s,null);
    this.dattribs_names.addValue(prefix+".ib_IH_",s,null);

    n+=EVXData_Names(prefix+".I",s);

    return n;
}

private int printDynMPnLValues(final DynMPnL x)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.avar_p_IH);
    this.dattribs.addValue(x.pb_IH);
    this.dattribs.addValue(x.ib_IH);

    n+=EVXData_Values(x.I);

    return n;
}

private int printDynM0PnLNames(final String prefix,final String s)
{
    int n=0;

    n+=4;
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".avar_p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pb_IH_",s,null);
    this.dattribs_names.addValue(prefix+".ib_IH_",s,null);

    return n;
}

private int printDynM0PnLValues(final DynM0PnL x)
{
    int n=0;

    n+=4;
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.avar_p_IH);
    this.dattribs.addValue(x.pb_IH);
    this.dattribs.addValue(x.ib_IH);

    return n;
}



private int printK3DistanceNames(final String prefix,final String s)
{
    int n=0;

    n+=6;
    this.dattribs_names.addValue(prefix+".w_LEH_",s,null);
    this.dattribs_names.addValue(prefix+".w_HEL_",s,null);
    this.dattribs_names.addValue(prefix+".w_LHE_",s,null);
    this.dattribs_names.addValue(prefix+".w_HLE_",s,null);
    this.dattribs_names.addValue(prefix+".w_ELH_",s,null);
    this.dattribs_names.addValue(prefix+".w_EHL_",s,null);

    return n;
}

private int printK3DistanceValues(final K3Distance x)
{
    int n=0;

    n+=6;
    this.dattribs.addValue(x.w_LEH);
    this.dattribs.addValue(x.w_HEL);
    this.dattribs.addValue(x.w_LHE);
    this.dattribs.addValue(x.w_HLE);
    this.dattribs.addValue(x.w_ELH);
    this.dattribs.addValue(x.w_EHL);

    return n;
}



private int printDynVar1PnLNames(final String prefix,final String s)
{
    int n=0;

    n+=12;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_IL_",s,null);
    this.dattribs_names.addValue(prefix+".dif_IL_",s,null);
    this.dattribs_names.addValue(prefix+".var1_pI_L_",s,null);
    this.dattribs_names.addValue(prefix+".var2_pI_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dif_IH_",s,null);
    this.dattribs_names.addValue(prefix+".var1_pI_H_",s,null);
    this.dattribs_names.addValue(prefix+".var2_pI_H_",s,null);
    this.dattribs_names.addValue(prefix+".pDiff_",s,null);

    return n;
}

private int printDynVar1PnLValues(final DynVar1PnL x)
{
    int n=0;

    n+=12;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_IL);
    this.dattribs.addValue(x.dif_IL);
    this.dattribs.addValue(x.var1_pI_L);
    this.dattribs.addValue(x.var2_pI_L);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.dif_IH);
    this.dattribs.addValue(x.var1_pI_H);
    this.dattribs.addValue(x.var2_pI_H);
    this.dattribs.addValue(x.pDiff);

    return n;
}



private int printDynVolPnLNames(final String prefix,final String s)
{
    int n=0;

    n+=3;
    this.dattribs_names.addValue(prefix+".pEQ_",s,null);
    this.dattribs_names.addValue(prefix+".pEB_",s,null);
    this.dattribs_names.addValue(prefix+".IEQ_",s,null);

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".Ipnl",s);

    return n;
}

private int printDynVolPnLValues(final DynVolPnL x)
{
    int n=0;

    n+=3;
    this.dattribs.addValue(x.pEQ);
    this.dattribs.addValue(x.pEB);
    this.dattribs.addValue(x.IEQ);

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.Ipnl);

    return n;
}


private int printYkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=30;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pm_IH_",s,null);
    this.dattribs_names.addValue(prefix+".plh_IH_",s,null);
    this.dattribs_names.addValue(prefix+".phh1_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_Ispur_",s,null);
    this.dattribs_names.addValue(prefix+".I_x2en_I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_x2en_I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_pIL_",s,null);
    this.dattribs_names.addValue(prefix+".I_pIL_",s,null);
    this.dattribs_names.addValue(prefix+".p_pIH_",s,null);
    this.dattribs_names.addValue(prefix+".I_pIH_",s,null);
    this.dattribs_names.addValue(prefix+".aI_assym_",s,null);
    this.dattribs_names.addValue(prefix+".p_IPnL_",s,null);
    this.dattribs_names.addValue(prefix+".Ipnl_psi_I_H_wLH_",s,null);
    this.dattribs_names.addValue(prefix+".Ipnl_psi_I_H_wHH_",s,null);
    this.dattribs_names.addValue(prefix+".Ipnl_pnl_0_",s,null);
    this.dattribs_names.addValue(prefix+".Ipnl_pnl_L_",s,null);
    this.dattribs_names.addValue(prefix+".Ipnl_pnl_H_",s,null);
    this.dattribs_names.addValue(prefix+".Ixen_wLH_",s,null);
    this.dattribs_names.addValue(prefix+".Ixen_wHH_",s,null);
    this.dattribs_names.addValue(prefix+".di_dIL_",s,null);
    this.dattribs_names.addValue(prefix+".di_dIH_",s,null);
    this.dattribs_names.addValue(prefix+".p_dIL_",s,null);
    this.dattribs_names.addValue(prefix+".p_dIH_",s,null);
    this.dattribs_names.addValue(prefix+".pe_dIL_",s,null);
    this.dattribs_names.addValue(prefix+".pe_dIH_",s,null);

    n+=EVXData_Names(prefix+".Ipnl_psi_I_H",s);
    n+=printDynPnLNames(prefix+".dpnl_I0",s);
    n+=printDynPnLNames(prefix+".dpnl_I0IH",s);
    n+=printDynPnLNames(prefix+".dpnl_IH",s);
    n+=printDynPnLNames(prefix+".dpnl_I0H",s);
    n+=printDynVolPnLNames(prefix+".vdpnl_IH",s);
    n+=printDynHPnLNames(prefix+".pi",s);

    return n;
}
private <T> int printYkQVMValues(final YkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=30;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.pm_IH);
    this.dattribs.addValue(x.plh_IH);
    this.dattribs.addValue(x.phh1_IH);
    this.dattribs.addValue(x.p_Ispur);
    this.dattribs.addValue(x.I_x2en_I_H);
    this.dattribs.addValue(x.p_x2en_I_H);
    this.dattribs.addValue(x.p_pIL);
    this.dattribs.addValue(x.I_pIL);
    this.dattribs.addValue(x.p_pIH);
    this.dattribs.addValue(x.I_pIH);
    this.dattribs.addValue(x.aI_assym);
    this.dattribs.addValue(x.p_IPnL);
    this.dattribs.addValue(x.Ipnl_psi_I_H_wLH);
    this.dattribs.addValue(x.Ipnl_psi_I_H_wHH);
    this.dattribs.addValue(x.Ipnl_pnl_0);
    this.dattribs.addValue(x.Ipnl_pnl_L);
    this.dattribs.addValue(x.Ipnl_pnl_H);
    this.dattribs.addValue(x.Ixen_wLH);
    this.dattribs.addValue(x.Ixen_wHH);
    this.dattribs.addValue(x.di_dIL);
    this.dattribs.addValue(x.di_dIH);
    this.dattribs.addValue(x.p_dIL);
    this.dattribs.addValue(x.p_dIH);
    this.dattribs.addValue(x.pe_dIL);
    this.dattribs.addValue(x.pe_dIH);

    n+=EVXData_Values(x.Ipnl_psi_I_H);
    n+=printDynPnLValues(x.dpnl_I0);
    n+=printDynPnLValues(x.dpnl_I0IH);
    n+=printDynPnLValues(x.dpnl_IH);
    n+=printDynPnLValues(x.dpnl_I0H);
    n+=printDynVolPnLValues(x.vdpnl_IH);
    n+=printDynHPnLValues(x.pi);

    return n;
}



private int printYpkQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=10;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".rdpI_0_",s,null);
    this.dattribs_names.addValue(prefix+".rdpI_L_",s,null);
    this.dattribs_names.addValue(prefix+".rdpI_H_",s,null);
    this.dattribs_names.addValue(prefix+".P_pI_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".varP_pI_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".P_I_rpsiH_",s,null);
    this.dattribs_names.addValue(prefix+".varP_I_rpsiH_",s,null);

    n+=EVXData_Names(prefix+".pI",s);
    n+=EVXData_Names(prefix+".dpI",s);
    n+=printDynYNames(prefix+".dY",s);
    n+=printDynYpNames(prefix+".dYp",s);
    n+=printDynYpNoIterNames(prefix+".dYpNi",s);
    n+=printDynYpmNames(prefix+".dYpmL",s);
    n+=printDynYpmNames(prefix+".dYpmH",s);
    n+=printDynYpmNames(prefix+".dYpi",s);

    return n;
}
private <T> int printYpkQVMValues(final YpkQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=10;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.rdpI_0);
    this.dattribs.addValue(x.rdpI_L);
    this.dattribs.addValue(x.rdpI_H);
    this.dattribs.addValue(x.P_pI_psiH);
    this.dattribs.addValue(x.varP_pI_psiH);
    this.dattribs.addValue(x.P_I_rpsiH);
    this.dattribs.addValue(x.varP_I_rpsiH);

    n+=EVXData_Values(x.pI);
    n+=EVXData_Values(x.dpI);
    n+=printDynYValues(x.dY);
    n+=printDynYpValues(x.dYp);
    n+=printDynYpNoIterValues(x.dYpNi);
    n+=printDynYpmValues(x.dYpmL);
    n+=printDynYpmValues(x.dYpmH);
    n+=printDynYpmValues(x.dYpi);

    return n;
}


private int printYpnlQVMNames(final String prefix,final String s)
{
    int n=0;

    n+=46;
    this.dattribs_names.addValue(prefix+".Pt_average_",s,null);
    this.dattribs_names.addValue(prefix+".Pi_average_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pV_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pt_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_IH_",s,null);
    this.dattribs_names.addValue(prefix+".d2p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".p_d2iPart_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_d2iPart_H_",s,null);
    this.dattribs_names.addValue(prefix+".pd_d2iPart_L_",s,null);
    this.dattribs_names.addValue(prefix+".pd_d2iPart_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_d2iPartD_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_d2iPartD_H_",s,null);
    this.dattribs_names.addValue(prefix+".pd_d2iPartD_L_",s,null);
    this.dattribs_names.addValue(prefix+".pd_d2iPartD_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_iVol_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_iVol_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_DiDt_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_DiDt_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_d2I_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_d2I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_d2ID_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_d2ID_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_dIdH_",s,null);
    this.dattribs_names.addValue(prefix+".pd_dIdH_",s,null);
    this.dattribs_names.addValue(prefix+".p_dIdL_",s,null);
    this.dattribs_names.addValue(prefix+".pd_dIdL_",s,null);
    this.dattribs_names.addValue(prefix+".pD2_IH_",s,null);
    this.dattribs_names.addValue(prefix+".ptD2_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pD2a_IH_",s,null);
    this.dattribs_names.addValue(prefix+".ptD2a_IH_",s,null);
    this.dattribs_names.addValue(prefix+".dp_I_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".px_I_psiL_",s,null);
    this.dattribs_names.addValue(prefix+".px_I_psiH_",s,null);
    this.dattribs_names.addValue(prefix+".p_Di_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_Di_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_mH_L_",s,null);
    this.dattribs_names.addValue(prefix+".p_mH_H_",s,null);
    this.dattribs_names.addValue(prefix+".pId_IL_",s,null);
    this.dattribs_names.addValue(prefix+".pId_IH_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_L_",s,null);
    this.dattribs_names.addValue(prefix+".pnl_H_",s,null);
    this.dattribs_names.addValue(prefix+".dp_L_",s,null);
    this.dattribs_names.addValue(prefix+".dp_H_",s,null);
    this.dattribs_names.addValue(prefix+".Id_L_",s,null);
    this.dattribs_names.addValue(prefix+".Id_H_",s,null);

    n+=4;
    this.dattribs_names.addValue(prefix+".d2p_L_",s,null);
    this.dattribs_names.addValue(prefix+".d2p_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_d2pL_",s,null);
    this.dattribs_names.addValue(prefix+".w_IH_d2pH_",s,null);

    n+=6;
    this.dattribs_names.addValue(prefix+".w_pnl_L_Id_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_pnl_H_Id_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_dp_L_Id_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_dp_H_Id_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_IDpi_L_I_H_",s,null);
    this.dattribs_names.addValue(prefix+".w_IDpi_H_I_H_",s,null);

    n+=EVXData_Names(prefix+".I",s);
    n+=EVXData_Names(prefix+".Iadp",s);
    n+=EVXData_Names(prefix+".IDpi",s);

    n+=EVX2Data_Names(prefix+".dI_dp",s);
    n+=EVX2Data_Names(prefix+".dI_d2p",s);
    n+=EVX2Data_Names(prefix+".I_dp",s);
    n+=EVX2Data_Names(prefix+".I_d2p",s);

     n+=printVar2INames(prefix+".pv_IL",s);
     n+=printVar2INames(prefix+".pv_IH",s);

     n+=printDynVarPnLNames(prefix+".xpi",s);
     n+=printDynVarPnLNames(prefix+".xpV",s);
     n+=printDynVarPnLNames(prefix+".xditDpi",s);
     n+=printDynVarPnLNames(prefix+".xdiiDpi",s);
     n+=printDynVarPnLNames(prefix+".xid2tpi",s);
     n+=printDynVarPnLNames(prefix+".xdid2tDpi",s);

     n+=printDynVarPnLNames(prefix+".xpt",s);
     n+=printDynVarPnLNames(prefix+".xvS",s);
     n+=printDynVarPnLNames(prefix+".xstaui",s);
     n+=printDynVarPnLNames(prefix+".xSTaui",s);
     n+=printDynVarPnLNames(prefix+".xdiipi",s);
     n+=printDynVarPnLNames(prefix+".xditpi",s);

     n+=printDynVarXkNames(prefix+".piXk_IL",s);
     n+=printDynVarXkNames(prefix+".RpiXk_IL",s);
     n+=printDynVarXkNames(prefix+".piXk_IH",s);
     n+=printDynVarXkNames(prefix+".RpiXk_IH",s);

     n+=printDynVarXiNames(prefix+".pXiMu",s);
     n+=printDynVarXiNames(prefix+".pXiI",s);
     n+=printDynVarXiNames(prefix+".RpXiMu",s);
     n+=printDynVarXiNames(prefix+".RpXiI",s);

     n+=printDynVarXNames(prefix+".piMu",s);
     n+=printDynVarXNames(prefix+".piI",s);
     n+=printDynVarXNames(prefix+".piMut",s);
     n+=printDynVarXNames(prefix+".piIt",s);

     n+=printDynVar2CNames(prefix+".var2pIH",s);
     n+=printDynVar2CNames(prefix+".var2pIS",s);
     n+=printDynVar2CaNames(prefix+".var2ca",s);
     n+=printDynVarICNames(prefix+".varic",s);
     n+=printDynVarIC2Names(prefix+".varic2",s);
     n+=printDynVarIC2Names(prefix+".varic2_0",s);
     n+=printDynCPnLNames(prefix+".cpnl0",s);
     n+=printDynCPnLNames(prefix+".cpnlR",s);
     n+=printDynCPnLNames(prefix+".cpnl",s);
     n+=printDynVarFiniteD1Names(prefix+".finD1_pI",s);
     n+=printDynVarFiniteDNames(prefix+".finD_0Is",s);
     n+=printDynVarFiniteDNames(prefix+".finD_0Im",s);
     n+=printDynVarFiniteDNames(prefix+".finD_0Ip",s);
     n+=printDynVarFiniteDNames(prefix+".finD_0Ip1",s);
     n+=printDynVarFiniteDNames(prefix+".finD_0Iw",s);
     n+=printDynVarFiniteDNames(prefix+".finD_0Vs",s);
     n+=printDynVarFiniteDNames(prefix+".finD_0Vm",s);
     n+=printDynVarFiniteDNames(prefix+".finD_0Vp",s);
     n+=printDynVarFiniteDNames(prefix+".finD_0Vp1",s);
     n+=printDynVarFiniteDNames(prefix+".finD_0Vw",s);
     n+=printDynVarFiniteDNames(prefix+".finD_hIs",s);
     n+=printDynVarFiniteDNames(prefix+".finD_hIm",s);
     n+=printDynVarFiniteDNames(prefix+".finD_hIp",s);
     n+=printDynVarFiniteDNames(prefix+".finD_hIp1",s);
     n+=printDynVarFiniteDNames(prefix+".finD_hIw",s);
     n+=printDynVarFiniteDNames(prefix+".finD_hVs",s);
     n+=printDynVarFiniteDNames(prefix+".finD_hVm",s);
     n+=printDynVarFiniteDNames(prefix+".finD_hVp",s);
     n+=printDynVarFiniteDNames(prefix+".finD_hVp1",s);
     n+=printDynVarFiniteDNames(prefix+".finD_hVw",s);
     n+=printDynVarFiniteCNames(prefix+".finC_pV",s);
     n+=printDynVarFiniteCNames(prefix+".finC_pI",s);
     n+=printDynVarFiniteCNames(prefix+".finC_RpI",s);
     n+=printDynVarFiniteFNames(prefix+".finFJ",s);
     n+=printDynVarFiniteFVNames(prefix+".finFJV_TOT",s);
     n+=printDynVarFiniteFVNames(prefix+".finFJV_PLUS",s);
     n+=printDynVarFiniteFVNames(prefix+".finFJV_MINUS",s);
     n+=printDynVarFiniteFVNames(prefix+".finFJV_REL_IH",s);
     n+=printDynVarFiniteFVNames(prefix+".finFJV_REL_IL",s);
     n+=printDynVarFiniteFVNames(prefix+".finFJV_rI_IH",s);
     n+=printDynVarFiniteENames(prefix+".finEi",s);
     n+=printDynVarFiniteENames(prefix+".finEV",s);
     n+=printDynVarFiniteENames(prefix+".RfinEi",s);
     n+=printDynVarFiniteENames(prefix+".RfinEV",s);
     n+=printDynVarFiniteENames(prefix+".fin0Ei",s);
     n+=printDynVarFiniteENames(prefix+".finD0Ei",s);
     n+=printDynVarFiniteENames(prefix+".fin0EV",s);
     n+=printDynVarFiniteENames(prefix+".finD0EV",s);
     n+=printDynVarCNames(prefix+".vc0_PnL_IH",s);
     n+=printDynVarCNames(prefix+".vc0_PnLX_IH",s);
     n+=printDynVarCNames(prefix+".vc_PnLX_IH",s);
     n+=printDynVarCNames(prefix+".VC_PnL_IH0",s);
     n+=printDynVarCNames(prefix+".vc_IL",s);
     n+=printDynVarCNames(prefix+".Rvc_IL",s);
     n+=printDynVarCNames(prefix+".vc_IH",s);
     n+=printDynVarCNames(prefix+".Rvc_IH",s);
     n+=printDynVarCNames(prefix+".vcpnl_IL",s);
     n+=printDynVarCNames(prefix+".Rvcpnl_IL",s);
     n+=printDynVarCNames(prefix+".vcpnl_IH",s);
     n+=printDynVarCNames(prefix+".Rvcpnl_IH",s);

     n+=printDynVarIPnLNames(prefix+".varpiED",s);
     n+=printDynVarIPnLNames(prefix+".varpiEI",s);
     n+=printDynVarIPnLNames(prefix+".fvarpi.var",s);

     n+=printDynpIVNames(prefix+".pIV0",s);
     n+=printDynpIVNames(prefix+".pIVR",s);

     n+=printDynVar1PnLNames(prefix+".it_EE",s);
     n+=printDynVar1PnLNames(prefix+".it_NEE",s);
     n+=printDynVar1PnLNames(prefix+".id2t_EE",s);
     n+=printDynVar1PnLNames(prefix+".id2t_NEE",s);

     n+=printDynEHPnLNames(prefix+".piEH",s);
     n+=printK3DistanceNames(prefix+".k3",s);
     n+=printDynDVHPnLNames(prefix+".piId",s);
     n+=printDynVHPnLNames(prefix+".piVH",s);
     n+=printDynVHPnLNames(prefix+".piXVH",s);
     n+=printDynVHPnLNames(prefix+".pvVH",s);
     n+=printDynVHPnLNames(prefix+".pVVH",s);
     n+=printDynMPnLNames(prefix+".piMD",s);
     n+=printDynM0PnLNames(prefix+".piM0",s);

     n+=printDynTIPnLNames(prefix+".piTI",s);
     n+=printDynTIPnLNames(prefix+".piTI0",s);
     n+=printDynTIPnLNames(prefix+".piTIH",s);
     n+=printDynTIPnLNames(prefix+".piTIH0",s);
     n+=printDynTIPnLNames(prefix+".piTIDH",s);
     n+=printDynTIPnLNames(prefix+".piTID0",s);

     n+=printDynDVarPnLNames(prefix+".dII",s);
     n+=printTryD2Names(prefix+".d2_psi_IH",s);
     n+=printTryD2Names(prefix+".Rd2_psi_IH",s);

     n+=printDyn3PnLNames(prefix+".Rpnl3",s);
     n+=printDynSPnLNames(prefix+".DSpnl",s);
     n+=printDynSPnLNames(prefix+".DSOpnl",s);
     n+=printDynSPnLNames(prefix+".DPpnl",s);
     n+=printDynIVTNames(prefix+".ivt0",s);
     n+=printDynIVTNames(prefix+".ivt",s);
     n+=printDynVTPnLNames(prefix+".vt",s);
     n+=printDynRPnLNames(prefix+".rpi",s);
     n+=printDynRPnLlastNames(prefix+".rpiR",s);
     n+=printDynRPnLINames(prefix+".rpnli",s);
     n+=printDynRPnLINames(prefix+".rpnli0",s);
     n+=printDynRPnLINames(prefix+".rpnl",s);
     n+=printDynRPnLINames(prefix+".rpnl0",s);
     n+=printDREVXDataNames(prefix+".rdit",s);
     n+=printDREVXDataNames(prefix+".rdit0",s);
     n+=printDREVXDataNames(prefix+".rdii",s);
     n+=printDynRXPnLNames(prefix+".vrit",s);
     n+=printDynRXPnLNames(prefix+".vrDit",s);
     n+=printDynVarCDNames(prefix+".piCDipsi",s);
     n+=printDynVarCDNames(prefix+".piCDtpsi",s);
     n+=printDynVarCDNames(prefix+".piCDVpsi",s);
     n+=printDynVarCDNames(prefix+".pCCDVpsi",s);
     n+=printDynVarCDVNames(prefix+".piCDV0",s);
     n+=printDynVarCDVNames(prefix+".piCDV",s);
     n+=printDynVarCDVNames(prefix+".piDt0",s);
     n+=printDynVarCDVNames(prefix+".piDi0",s);
     n+=printDynVarCDVNames(prefix+".v_V_C",s);
     n+=printDynVarCDVNames(prefix+".v_V_pI",s);
     n+=printDynVarCDVNames(prefix+".v_V_DpI",s);
     n+=printDynVarCDVNames(prefix+".v_DI_C",s);
     n+=printDynVarCDVNames(prefix+".v_DI_pI",s);
     n+=printDynVarCDVNames(prefix+".v_DI_DpI",s);
     n+=printDynSDPnLNames(prefix+".sd",s);
     n+=printDynSD2PnLNames(prefix+".sd2",s);
     n+=printDynVar2INames(prefix+".v2DI",s);
     n+=printDynVar2INames(prefix+".v2I",s);
     n+=printDynVar2INames(prefix+".v2Io",s);
     n+=printDynVar2INames(prefix+".v2I0",s);
     n+=printDynVar2INames(prefix+".v2I0o",s);
     n+=printDynDINames(prefix+".dI",s);
     n+=printDynVNames(prefix+".edV",s);
     n+=printDynVNames(prefix+".edI",s);
     n+=printDynIndPnLNames(prefix+".indPnL",s);
     n+=printDynRVarSP0Names(prefix+".Rsp0",s);
     n+=printDynVarSP0Names(prefix+".sp0",s);
     n+=printDynVarSP0Names(prefix+".spl",s);

     n+=printDynMUPnLNames(prefix+".pnlmuR",s);
     n+=printDynMUPnLNames(prefix+".pnlmu0",s);
     n+=EVXSpectrum_Names(prefix+".Pt",s);
     n+=EVXSpectrum_Names(prefix+".Pi",s);
     n+=printDynVarYNames(prefix+".vY",s);
     n+=printDynVarMuNames(prefix+".dvmuIL",s);
     n+=printDynVarMuNames(prefix+".dvmuIH",s);
     n+=printDynVarMuXNames(prefix+".dvmuXIH",s);
     n+=printDynVarMuXNames(prefix+".dvmuXX",s);
     n+=printDynVarMuXNames(prefix+".dvmuXW",s);
     n+=printDynVarMuXNames(prefix+".dvmuXW_SUM",s);
     n+=printDynVarVNames(prefix+".dvV",s);
     n+=printDynVar2PiPtNames(prefix+".vPiPt",s);
     n+=printDynVar2ProjNames(prefix+".v2Proj",s);
     n+=printDynVar2DynImpactNames(prefix+".dimpact",s);
     n+=printDynVar2DIDynImpactNames(prefix+".DIdimpact",s);
     n+=printDynVH2PnLNames(prefix+".dvh2pnl",s);
     n+=printDynVarPnLSpurNames(prefix+".dvh2pnlSpur",s);
     n+=printDynVarJPMorganNames(prefix+".dmorgan",s);
     n+=printDynVar2DiagProjNames(prefix+".dvardiagproj",s);
     n+=printDynVar2DynImpactDINames(prefix+".dynimpactDI",s);
     n+=printImpactExampleNames(prefix+".iex",s);

    return n;
}
private <T> int printYpnlQVMValues(final YpnlQVM<T> x,final CalculationData c)
{
    int n=0;

    n+=46;
    this.dattribs.addValue(x.Pt_average);
    this.dattribs.addValue(x.Pi_average);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.pV_IH);
    this.dattribs.addValue(x.pt_IH);
    this.dattribs.addValue(x.dp_IH);
    this.dattribs.addValue(x.d2p_IH);
    this.dattribs.addValue(x.p_d2iPart_L);
    this.dattribs.addValue(x.p_d2iPart_H);
    this.dattribs.addValue(x.pd_d2iPart_L);
    this.dattribs.addValue(x.pd_d2iPart_H);
    this.dattribs.addValue(x.p_d2iPartD_L);
    this.dattribs.addValue(x.p_d2iPartD_H);
    this.dattribs.addValue(x.pd_d2iPartD_L);
    this.dattribs.addValue(x.pd_d2iPartD_H);
    this.dattribs.addValue(x.p_iVol_L);
    this.dattribs.addValue(x.p_iVol_H);
    this.dattribs.addValue(x.p_DiDt_L);
    this.dattribs.addValue(x.p_DiDt_H);
    this.dattribs.addValue(x.p_d2I_L);
    this.dattribs.addValue(x.p_d2I_H);
    this.dattribs.addValue(x.p_d2ID_L);
    this.dattribs.addValue(x.p_d2ID_H);
    this.dattribs.addValue(x.p_dIdH);
    this.dattribs.addValue(x.pd_dIdH);
    this.dattribs.addValue(x.p_dIdL);
    this.dattribs.addValue(x.pd_dIdL);
    this.dattribs.addValue(x.pD2_IH);
    this.dattribs.addValue(x.ptD2_IH);
    this.dattribs.addValue(x.pD2a_IH);
    this.dattribs.addValue(x.ptD2a_IH);
    this.dattribs.addValue(x.dp_I_psiH);
    this.dattribs.addValue(x.px_I_psiL);
    this.dattribs.addValue(x.px_I_psiH);
    this.dattribs.addValue(x.p_Di_L);
    this.dattribs.addValue(x.p_Di_H);
    this.dattribs.addValue(x.p_mH_L);
    this.dattribs.addValue(x.p_mH_H);
    this.dattribs.addValue(x.pId_IL);
    this.dattribs.addValue(x.pId_IH);
    this.dattribs.addValue(x.pnl_L);
    this.dattribs.addValue(x.pnl_H);
    this.dattribs.addValue(x.dp_L);
    this.dattribs.addValue(x.dp_H);
    this.dattribs.addValue(x.Id_L);
    this.dattribs.addValue(x.Id_H);

    n+=4;
    this.dattribs.addValue(x.d2p_L);
    this.dattribs.addValue(x.d2p_H);
    this.dattribs.addValue(x.w_IH_d2pL);
    this.dattribs.addValue(x.w_IH_d2pH);

    n+=6;
    this.dattribs.addValue(x.w_pnl_L_Id_H);
    this.dattribs.addValue(x.w_pnl_H_Id_H);
    this.dattribs.addValue(x.w_dp_L_Id_H);
    this.dattribs.addValue(x.w_dp_H_Id_H);
    this.dattribs.addValue(x.w_IDpi_L_I_H);
    this.dattribs.addValue(x.w_IDpi_H_I_H);

    n+=EVXData_Values(x.I);
    n+=EVXData_Values(x.Iadp);
    n+=EVXData_Values(x.IDpi);

    n+=EVX2Data_Values(x.dI_dp);
    n+=EVX2Data_Values(x.dI_d2p);
    n+=EVX2Data_Values(x.I_dp);
    n+=EVX2Data_Values(x.I_d2p);


    n+=printVar2IValues(x.pv_IL);
    n+=printVar2IValues(x.pv_IH);

    n+=printDynVarPnLValues(x.xpi);
    n+=printDynVarPnLValues(x.xpV);
    n+=printDynVarPnLValues(x.xditDpi);
    n+=printDynVarPnLValues(x.xdiiDpi);
    n+=printDynVarPnLValues(x.xid2tpi);
    n+=printDynVarPnLValues(x.xdid2tDpi);

    n+=printDynVarPnLValues(x.xpt);
    n+=printDynVarPnLValues(x.xvS);
    n+=printDynVarPnLValues(x.xstaui);
    n+=printDynVarPnLValues(x.xSTaui);
    n+=printDynVarPnLValues(x.xdiipi);
    n+=printDynVarPnLValues(x.xditpi);

    n+=printDynVarXkValues(x.piXk_IL);
    n+=printDynVarXkValues(x.RpiXk_IL);
    n+=printDynVarXkValues(x.piXk_IH);
    n+=printDynVarXkValues(x.RpiXk_IH);

    n+=printDynVarXiValues(x.pXiMu);
    n+=printDynVarXiValues(x.pXiI);
    n+=printDynVarXiValues(x.RpXiMu);
    n+=printDynVarXiValues(x.RpXiI);

    n+=printDynVarXValues(x.piMu);
    n+=printDynVarXValues(x.piI);
    n+=printDynVarXValues(x.piMut);
    n+=printDynVarXValues(x.piIt);

    n+=printDynVar2CValues(x.var2pIH);
    n+=printDynVar2CValues(x.var2pIS);
    n+=printDynVar2CaValues(x.var2ca);
    n+=printDynVarICValues(x.varic);
    n+=printDynVarIC2Values(x.varic2);
    n+=printDynVarIC2Values(x.varic2_0);
    n+=printDynCPnLValues(x.cpnl0);
    n+=printDynCPnLValues(x.cpnlR);
    n+=printDynCPnLValues(x.cpnl);
    n+=printDynVarFiniteD1Values(x.finD1_pI);
    n+=printDynVarFiniteDValues(x.finD_0Is);
    n+=printDynVarFiniteDValues(x.finD_0Im);
    n+=printDynVarFiniteDValues(x.finD_0Ip);
    n+=printDynVarFiniteDValues(x.finD_0Ip1);
    n+=printDynVarFiniteDValues(x.finD_0Iw);
    n+=printDynVarFiniteDValues(x.finD_0Vs);
    n+=printDynVarFiniteDValues(x.finD_0Vm);
    n+=printDynVarFiniteDValues(x.finD_0Vp);
    n+=printDynVarFiniteDValues(x.finD_0Vp1);
    n+=printDynVarFiniteDValues(x.finD_0Vw);
    n+=printDynVarFiniteDValues(x.finD_hIs);
    n+=printDynVarFiniteDValues(x.finD_hIm);
    n+=printDynVarFiniteDValues(x.finD_hIp);
    n+=printDynVarFiniteDValues(x.finD_hIp1);
    n+=printDynVarFiniteDValues(x.finD_hIw);
    n+=printDynVarFiniteDValues(x.finD_hVs);
    n+=printDynVarFiniteDValues(x.finD_hVm);
    n+=printDynVarFiniteDValues(x.finD_hVp);
    n+=printDynVarFiniteDValues(x.finD_hVp1);
    n+=printDynVarFiniteDValues(x.finD_hVw);
    n+=printDynVarFiniteCValues(x.finC_pV);
    n+=printDynVarFiniteCValues(x.finC_pI);
    n+=printDynVarFiniteCValues(x.finC_RpI);
    n+=printDynVarFiniteFValues(x.finFJ);
    n+=printDynVarFiniteFVValues(x.finFJV_TOT);
    n+=printDynVarFiniteFVValues(x.finFJV_PLUS);
    n+=printDynVarFiniteFVValues(x.finFJV_MINUS);
    n+=printDynVarFiniteFVValues(x.finFJV_REL_IH);
    n+=printDynVarFiniteFVValues(x.finFJV_REL_IL);
    n+=printDynVarFiniteFVValues(x.finFJV_rI_IH);
    n+=printDynVarFiniteEValues(x.finEi);
    n+=printDynVarFiniteEValues(x.finEV);
    n+=printDynVarFiniteEValues(x.RfinEi);
    n+=printDynVarFiniteEValues(x.RfinEV);
    n+=printDynVarFiniteEValues(x.fin0Ei);
    n+=printDynVarFiniteEValues(x.finD0Ei);
    n+=printDynVarFiniteEValues(x.fin0EV);
    n+=printDynVarFiniteEValues(x.finD0EV);
    n+=printDynVarCValues(x.vc0_PnL_IH);
    n+=printDynVarCValues(x.vc0_PnLX_IH);
    n+=printDynVarCValues(x.vc_PnLX_IH);
    n+=printDynVarCValues(x.VC_PnL_IH0);
    n+=printDynVarCValues(x.vc_IL);
    n+=printDynVarCValues(x.Rvc_IL);
    n+=printDynVarCValues(x.vc_IH);
    n+=printDynVarCValues(x.Rvc_IH);
    n+=printDynVarCValues(x.vcpnl_IL);
    n+=printDynVarCValues(x.Rvcpnl_IL);
    n+=printDynVarCValues(x.vcpnl_IH);
    n+=printDynVarCValues(x.Rvcpnl_IH);

    n+=printDynVarIPnLValues(x.varpiED);
    n+=printDynVarIPnLValues(x.varpiEI);
    n+=printDynVarIPnLValues(x.fvarpi.var);

    n+=printDynpIVValues(x.pIV0);
    n+=printDynpIVValues(x.pIVR);

    n+=printDynVar1PnLValues(x.it_EE);
    n+=printDynVar1PnLValues(x.it_NEE);
    n+=printDynVar1PnLValues(x.id2t_EE);
    n+=printDynVar1PnLValues(x.id2t_NEE);

    n+=printDynEHPnLValues(x.piEH);
    n+=printK3DistanceValues(x.k3);
    n+=printDynDVHPnLValues(x.piId);
    n+=printDynVHPnLValues(x.piVH);
    n+=printDynVHPnLValues(x.piXVH);
    n+=printDynVHPnLValues(x.pvVH);
    n+=printDynVHPnLValues(x.pVVH);
    n+=printDynMPnLValues(x.piMD);
    n+=printDynM0PnLValues(x.piM0);

    n+=printDynTIPnLValues(x.piTI);
    n+=printDynTIPnLValues(x.piTI0);
    n+=printDynTIPnLValues(x.piTIH);
    n+=printDynTIPnLValues(x.piTIH0);
    n+=printDynTIPnLValues(x.piTIDH);
    n+=printDynTIPnLValues(x.piTID0);

    n+=printDynDVarPnLValues(x.dII);
    n+=printTryD2Values(x.d2_psi_IH);
    n+=printTryD2Values(x.Rd2_psi_IH);

    n+=printDyn3PnLValues(x.Rpnl3);
    n+=printDynSPnLValues(x.DSpnl);
    n+=printDynSPnLValues(x.DSOpnl);
    n+=printDynSPnLValues(x.DPpnl);
    n+=printDynIVTValues(x.ivt0);
    n+=printDynIVTValues(x.ivt);
    n+=printDynVTPnLValues(x.vt);
    n+=printDynRPnLValues(x.rpi);
    n+=printDynRPnLlastValues(x.rpiR);
    n+=printDynRPnLIValues(x.rpnli);
    n+=printDynRPnLIValues(x.rpnli0);
    n+=printDynRPnLIValues(x.rpnl);
    n+=printDynRPnLIValues(x.rpnl0);
    n+=printDREVXDataValues(x.rdit);
    n+=printDREVXDataValues(x.rdit0);
    n+=printDREVXDataValues(x.rdii);
    n+=printDynRXPnLValues(x.vrit);
    n+=printDynRXPnLValues(x.vrDit);
    n+=printDynVarCDValues(x.piCDipsi);
    n+=printDynVarCDValues(x.piCDtpsi);
    n+=printDynVarCDValues(x.piCDVpsi);
    n+=printDynVarCDValues(x.pCCDVpsi);
    n+=printDynVarCDVValues(x.piCDV0);
    n+=printDynVarCDVValues(x.piCDV);
    n+=printDynVarCDVValues(x.piDt0);
    n+=printDynVarCDVValues(x.piDi0);
    n+=printDynVarCDVValues(x.v_V_C);
    n+=printDynVarCDVValues(x.v_V_pI);
    n+=printDynVarCDVValues(x.v_V_DpI);
    n+=printDynVarCDVValues(x.v_DI_C);
    n+=printDynVarCDVValues(x.v_DI_pI);
    n+=printDynVarCDVValues(x.v_DI_DpI);
    n+=printDynSDPnLValues(x.sd);
    n+=printDynSD2PnLValues(x.sd2);
    n+=printDynVar2IValues(x.v2DI);
    n+=printDynVar2IValues(x.v2I);
    n+=printDynVar2IValues(x.v2Io);
    n+=printDynVar2IValues(x.v2I0);
    n+=printDynVar2IValues(x.v2I0o);
    n+=printDynDIValues(x.dI);
    n+=printDynVValues(x.edV);
    n+=printDynVValues(x.edI);
    n+=printDynIndPnLValues(x.indPnL);
    n+=printDynRVarSP0Values(x.Rsp0);
    n+=printDynVarSP0Values(x.sp0);
    n+=printDynVarSP0Values(x.spl);

    n+=printDynMUPnLValues(x.pnlmuR);
    n+=printDynMUPnLValues(x.pnlmu0);
    n+=EVXSpectrum_Values(x.Pt);
    n+=EVXSpectrum_Values(x.Pi);
    n+=printDynVarYValues(x.vY);
    n+=printDynVarMuValues(x.dvmuIL);
    n+=printDynVarMuValues(x.dvmuIH);
    n+=printDynVarMuXValues(x.dvmuXIH);
    n+=printDynVarMuXValues(x.dvmuXX);
    n+=printDynVarMuXValues(x.dvmuXW);
    n+=printDynVarMuXValues(x.dvmuXW_SUM);
    n+=printDynVarVValues(x.dvV);
    n+=printDynVar2PiPtValues(x.vPiPt);
    n+=printDynVar2ProjValues(x.v2Proj);
    n+=printDynVar2DynImpactValues(x.dimpact);
    n+=printDynVar2DIDynImpactValues(x.DIdimpact);
    n+=printDynVH2PnLValues(x.dvh2pnl);
    n+=printDynVarPnLSpurValues(x.dvh2pnlSpur);
    n+=printDynVarJPMorganValues(x.dmorgan);
    n+=printDynVar2DiagProjValues(x.dvardiagproj);
    n+=printDynVar2DynImpactDIValues(x.dynimpactDI);
    n+=printImpactExampleValues(x.iex);

    return n;
}




private int printDynDpDiNames(final String prefix,final String s)
{
    int n=0;


    n+=5;
    this.dattribs_names.addValue(prefix+".I_0_",s,null);
    this.dattribs_names.addValue(prefix+".I_L_",s,null);
    this.dattribs_names.addValue(prefix+".I_H_",s,null);
    this.dattribs_names.addValue(prefix+".p_IH_",s,null);
    this.dattribs_names.addValue(prefix+".Dp_",s,null);

    return n;
}
private int printDynDpDiValues(final DynDpDi x)
{
    int n=0;

    n+=5;
    this.dattribs.addValue(x.I_0);
    this.dattribs.addValue(x.I_L);
    this.dattribs.addValue(x.I_H);
    this.dattribs.addValue(x.p_IH);
    this.dattribs.addValue(x.Dp);

    return n;
}

private int printDynSIVNames(final String prefix,final String s)
{
    int n=0;


    n+=5;
    this.dattribs_names.addValue(prefix+".I1_",s,null);
    this.dattribs_names.addValue(prefix+".p1_",s,null);
    this.dattribs_names.addValue(prefix+".I2_",s,null);
    this.dattribs_names.addValue(prefix+".p2_",s,null);
    this.dattribs_names.addValue(prefix+".tau_",s,null);
    n+=EVXData_Names(prefix+".I",s);

    return n;
}
private int printDynSIVValues(final DynSIV x)
{
    int n=0;

    n+=5;
    this.dattribs.addValue(x.I1);
    this.dattribs.addValue(x.p1);
    this.dattribs.addValue(x.I2);
    this.dattribs.addValue(x.p2);
    this.dattribs.addValue(x.tau);
    n+=EVXData_Values(x.I);

    return n;
}

private int printYkDpINames(final String prefix,final String s)
{
    int n=0;

    n+=printDynDpDiNames(prefix+".i0_pt",s);
    n+=printDynDpDiNames(prefix+".i0_pi",s);
    n+=printDynDpDiNames(prefix+".iR_pt",s);
    n+=printDynDpDiNames(prefix+".iR_pi",s);
    n+=printDynSIVNames(prefix+".pSIV",s);

    return n;
}
private <T> int printYkDpIValues(final YkDpI<T> x,final CalculationData c)
{
    int n=0;

    n+=printDynDpDiValues(x.i0_pt);
    n+=printDynDpDiValues(x.i0_pi);
    n+=printDynDpDiValues(x.iR_pt);
    n+=printDynDpDiValues(x.iR_pi);
    n+=printDynSIVValues(x.pSIV);

    return n;
}



private int printUkbQVMNames(final String prefix,final String s)
{
    int n=0;


    n+=4;
    this.dattribs_names.addValue(prefix+".p_buy_",s,null);
    this.dattribs_names.addValue(prefix+".tau_buy_",s,null);
    this.dattribs_names.addValue(prefix+".p_sell_",s,null);
    this.dattribs_names.addValue(prefix+".tau_sell_",s,null);

    return n;
}
private <T> int printUkbQVMValues(final UkbQVM<T> x,final CalculationData c)
{
    int n=0;


    n+=4;
    this.dattribs.addValue(x.p_buy);
    this.dattribs.addValue(x.tau_buy);
    this.dattribs.addValue(x.p_sell);
    this.dattribs.addValue(x.tau_sell);

    return n;
}






private int setAttrLiquidity19(final boolean flag_processheader,
				final String ticker,
				final long [] delays,
				final CalculationData c)
    {
	int n=0;

	for(int j=0;j<delays.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=String.valueOf(delays[j]/ModelParameters.DISTANCE_TIME_SCALE);

		n+=printEkQVMNames("LF19.ekqvmll",s);
		n+=printEkQVMNames("LF19.ekqvmpp",s);
		n+=printIkQVMNames("LF19.ikqvmll",s);
		n+=printIkQVMNames("LF19.ikqvmpp",s);
		n+=printJkQVMNames("LF19.jkqvmll",s);
		n+=printJkQVMNames("LF19.jkqvmpp",s);
		n+=printKkQVMNames("LF19.kkqvmll",s);
		n+=printKkQVMNames("LF19.kkqvmpp",s);
		n+=printLkQVMNames("LF19.lkqvmll",s);
		n+=printLkQVMNames("LF19.lkqvmpp",s);
		n+=printMkQVMNames("LF19.mkqvmll",s);
		n+=printMkQVMNames("LF19.mkqvmpp",s);
	    }
	    else
	    {
		final CalculateLiquidityDeficit19 l=c.topp.book_trades_liquiditydeficit19;
		final CalculateLiquidityDeficit19.VQ vx=l.tdata[j];

		n+=printEkQVMValues(vx.ekqvmll,c);
		n+=printEkQVMValues(vx.ekqvmpp,c);
		n+=printIkQVMValues(vx.ikqvmll,c);
		n+=printIkQVMValues(vx.ikqvmpp,c);
		n+=printJkQVMValues(vx.jkqvmll,c);
		n+=printJkQVMValues(vx.jkqvmpp,c);
		n+=printKkQVMValues(vx.kkqvmll,c);
		n+=printKkQVMValues(vx.kkqvmpp,c);
		n+=printLkQVMValues(vx.lkqvmll,c);
		n+=printLkQVMValues(vx.lkqvmpp,c);
		n+=printMkQVMValues(vx.mkqvmll,c);
		n+=printMkQVMValues(vx.mkqvmpp,c);
	    }
	}

	return n;
    }


private int setAttrLiquidity20(final boolean flag_processheader,
				final String ticker,
				final long [] delays,
				final CalculationData c)
    {
	int n=0;

	for(int j=0;j<delays.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=String.valueOf(delays[j]/ModelParameters.DISTANCE_TIME_SCALE);

		n+=printQkQVMpiNames("LF20.qkqvmpi",s);
		n+=printQmQVMpiNames("LF20.qmqvmpi",s);
		n+=printNkQVMNames("LF20.nkqvmll",s);
		n+=printNkQVMNames("LF20.nkqvmpp",s);
		n+=printOkQVMNames("LF20.okqvmll",s);
		n+=printOkQVMNames("LF20.okqvmpp",s);
		n+=printQkQVMNames("LF20.qkqvmll",s);
		n+=printQkQVMNames("LF20.qkqvmpp",s);
		n+=printPkQVMNames("LF20.pkqvmll",s);
		n+=printPkQVMNames("LF20.pkqvmpp",s);
		n+=printRkQVMNames("LF20.rkqvmll",s);
		n+=printRkQVMNames("LF20.rkqvmpp",s);
		n+=printRkiQVMNames("LF20.rkiqvmll",s);
		n+=printRkiQVMNames("LF20.rkiqvmpp",s);
		n+=printSkQVMNames("LF20.skqvmll",s);
		n+=printSkQVMNames("LF20.skqvmpp",s);
		n+=printTkQVMNames("LF20.tkqvmll",s);
		n+=printTkQVMNames("LF20.tkqvmpp",s);
		n+=printUkQVMNames("LF20.ukqvmll",s);
		n+=printUkQVMNames("LF20.ukqvmpp",s);
		n+=printUkbQVMNames("LF20.ukbqvmbmp",s);
		n+=printUnQVMNames("LF20.unqvmll",s);
		n+=printUnQVMNames("LF20.unqvmpp",s);
		n+=printVkQVMNames("LF20.vkqvmll",s);
		n+=printVkQVMNames("LF20.vkqvmpp",s);
		n+=printYkQVMNames("LF20.ykqvmll",s);
		n+=printYkQVMNames("LF20.ykqvmpp",s);
		n+=printYpkQVMNames("LF20.ypkqvmll",s);
		n+=printYpkQVMNames("LF20.ypkqvmpp",s);
		n+=printYpnlQVMNames("LF20.ypnlqvmll",s);
		n+=printYpnlQVMNames("LF20.ypnlqvmpp",s);
		n+=printYkDpINames("LF20.ykdpill",s);
		n+=printYkDpINames("LF20.ykdpipp",s);
	    }
	    else
	    {
		final CalculateLiquidityDeficit20 l=c.topp.book_trades_liquiditydeficit20;
		final CalculateLiquidityDeficit20.VQ vx=l.tdata[j];

		n+=printQkQVMpiValues(vx.qkqvmpi);
		n+=printQmQVMpiValues(vx.qmqvmpi);
		n+=printNkQVMValues(vx.nkqvmll,c);
		n+=printNkQVMValues(vx.nkqvmpp,c);
		n+=printOkQVMValues(vx.okqvmll,c);
		n+=printOkQVMValues(vx.okqvmpp,c);
		n+=printQkQVMValues(vx.qkqvmll,c);
		n+=printQkQVMValues(vx.qkqvmpp,c);
		n+=printPkQVMValues(vx.pkqvmll,c);
		n+=printPkQVMValues(vx.pkqvmpp,c);
		n+=printRkQVMValues(vx.rkqvmll,c);
		n+=printRkQVMValues(vx.rkqvmpp,c);
		n+=printRkiQVMValues(vx.rkiqvmll,c);
		n+=printRkiQVMValues(vx.rkiqvmpp,c);
		n+=printSkQVMValues(vx.skqvmll,c);
		n+=printSkQVMValues(vx.skqvmpp,c);
		n+=printTkQVMValues(vx.tkqvmll,c);
		n+=printTkQVMValues(vx.tkqvmpp,c);
		n+=printUkQVMValues(vx.ukqvmll,c);
		n+=printUkQVMValues(vx.ukqvmpp,c);
		n+=printUkbQVMValues(vx.ukbqvmbmp,c);
		n+=printUnQVMValues(vx.unqvmll,c);
		n+=printUnQVMValues(vx.unqvmpp,c);
		n+=printVkQVMValues(vx.vkqvmll,c);
		n+=printVkQVMValues(vx.vkqvmpp,c);
		n+=printYkQVMValues(vx.ykqvmll,c);
		n+=printYkQVMValues(vx.ykqvmpp,c);
		n+=printYpkQVMValues(vx.ypkqvmll,c);
		n+=printYpkQVMValues(vx.ypkqvmpp,c);
		n+=printYpnlQVMValues(vx.ypnlqvmll,c);
		n+=printYpnlQVMValues(vx.ypnlqvmpp,c);
		n+=printYkDpIValues(vx.ykdpill,c);
		n+=printYkDpIValues(vx.ykdpipp,c);
	    }
	}

	return n;
    }


private int setAttrLiquidity21(final boolean flag_processheader,
				final String ticker,
				final long [] delays,
				final CalculationData c)
    {
	int n=0;

	for(int j=0;j<delays.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=String.valueOf(delays[j]/ModelParameters.DISTANCE_TIME_SCALE);
		n+=printImpactQVMNames("LF21.iqvmll",s);
		n+=printImpactQVMNames("LF21.iqvmpp",s);
		n+=printQkQVMpiNames("LF21.qkqvmpi",s);
		n+=printQkPnLpi_PBasisNames("LF21.qkpnlpbasis",s);
		n+=printImpactQVMMuseNames("LF21.iqvmmll",s);
		n+=printImpactQVMMuseNames("LF21.iqvmmpp",s);
		n+=printDynMicroImpactNames(true,"LF21.dmill",s);
		n+=printDynMicroImpactNames(true,"LF21.dmipp",s);
		n+=printDynMicroImpactNames(false,"LF21.dmipi",s);
		n+=1;this.dattribs_names.addValue("LF21.dmipi.iffa_pi.sum_impact_",s,null);
		n+=printDynImpactAsymmetryNames("LF21.diall",s);
		n+=printDynImpactAsymmetryNames("LF21.diapp",s);
		n+=printDynImpactAsymmetryNames("LF21.diapi",s);
		n+=printIAImpactNames("LF21.iall",s);
		n+=printIAImpactNames("LF21.iapp",s);
		n+=printIAImpactNames("LF21.iapi",s);
		n+=printIStateNames("LF21.isll",s);
		n+=printIStateNames("LF21.ispp",s);
		n+=printIStateNames("LF21.ispi",s);
		n+=printIHStateNames("LF21.ihll",s);
		n+=printIHStateNames("LF21.ihpp",s);
		n+=printIHStateDMNames("LF21.ihdmll",s);
		n+=printIHStateDMNames("LF21.ihdmpp",s);
		n+=printScalpedMaxIProjectionNames("LF21.cmill",s);
		n+=printScalpedMaxIProjectionNames("LF21.cmipp",s);
	    }
	    else
	    {
		final CalculateLiquidityDeficit21 l=c.topp.book_trades_liquiditydeficit21;
		final CalculateLiquidityDeficit21.VQ vx=l.tdata[j];

		n+=printImpactQVMValues(vx.iqvmll,c);
		n+=printImpactQVMValues(vx.iqvmpp,c);
		n+=printQkQVMpiValues(vx.qkqvmpi);
		n+=printQkPnLpi_PBasisValues(vx.qkpnlpbasis,c);
		n+=printImpactQVMMuseValues(vx.iqvmmll,c);
		n+=printImpactQVMMuseValues(vx.iqvmmpp,c);
		n+=printDynMicroImpactValues(true,vx.dmill);
		n+=printDynMicroImpactValues(true,vx.dmipp);
		n+=printDynMicroImpactValues(false,vx.dmipi);
		n+=1;this.dattribs.addValue(vx.dmipi.iffa_pi.sum_impact);
		n+=printDynImpactAsymmetryValues(vx.diall);
		n+=printDynImpactAsymmetryValues(vx.diapp);
		n+=printDynImpactAsymmetryValues(vx.diapi);
		n+=printIAImpactValues(vx.iall);
		n+=printIAImpactValues(vx.iapp);
		n+=printIAImpactValues(vx.iapi);
		n+=printIStateValues(vx.isll);
		n+=printIStateValues(vx.ispp);
		n+=printIStateValues(vx.ispi);
		n+=printIHStateValues(vx.ihll);
		n+=printIHStateValues(vx.ihpp);
		n+=printIHStateDMValues(vx.ihdmll);
		n+=printIHStateDMValues(vx.ihdmpp);
		n+=printScalpedMaxIProjectionValues(vx.cmill);
		n+=printScalpedMaxIProjectionValues(vx.cmipp);
	    }
	}

	return n;
    }




    private int printPTradeIPNames(final String prefix,final String s)
    {
	int n=0;
	n+=4;
	this.dattribs_names.addValue(prefix+".I_p_last_",s,null);
	this.dattribs_names.addValue(prefix+".DI_p_last_",s,null);
	this.dattribs_names.addValue(prefix+".I_p_buy_",s,null);
	this.dattribs_names.addValue(prefix+".I_p_sell_",s,null);

	n+=4;
	this.dattribs_names.addValue(prefix+".n_vattractors_",s,null);
	this.dattribs_names.addValue(prefix+".n_vrepellers_",s,null);
	this.dattribs_names.addValue(prefix+".n_p_up_",s,null);
	this.dattribs_names.addValue(prefix+".n_p_down_",s,null);

	for(int r=0;r<2;r++)
	{
	    n+=4;
	    this.dattribs_names.addValue(prefix+".vattractors_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".vrepellers_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p_up_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".p_down_"+r+"_",s,null);
	}
	n+=2;
	this.dattribs_names.addValue(prefix+".p_down_close_",s,null);
	this.dattribs_names.addValue(prefix+".p_offset_",s,null);

	n+=4;
	this.dattribs_names.addValue(prefix+".min_A_",s,null);
	this.dattribs_names.addValue(prefix+".max_A_",s,null);
	this.dattribs_names.addValue(prefix+".min_R_",s,null);
	this.dattribs_names.addValue(prefix+".max_R_",s,null);

	return n;
    }

    private int printDPTradeIPNames(final String prefix,final String s)
    {
	int n=printPTradeIPNames(prefix,s);
	n+=4;

	this.dattribs_names.addValue(prefix+".pa_Ihigh_",s,null);
	this.dattribs_names.addValue(prefix+".pa_Ilow_",s,null);
	this.dattribs_names.addValue(prefix+".pa_m_",s,null);
	this.dattribs_names.addValue(prefix+".pr_m_",s,null);

	n+=2;
	this.dattribs_names.addValue(prefix+".I1_p_buy_",s,null);
	this.dattribs_names.addValue(prefix+".I1_p_sell_",s,null);

	n+=6;
	this.dattribs_names.addValue(prefix+".min_IA_",s,null);
	this.dattribs_names.addValue(prefix+".max_IA_",s,null);
	this.dattribs_names.addValue(prefix+".ex_IA_",s,null);
	this.dattribs_names.addValue(prefix+".min_IR_",s,null);
	this.dattribs_names.addValue(prefix+".max_IR_",s,null);
	this.dattribs_names.addValue(prefix+".ex_IR_",s,null);

	for(int r=0;r<2;r++)
	{
	    n+=2;
	    this.dattribs_names.addValue(prefix+".I1_p_up_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".I1_p_down_"+r+"_",s,null);

	    n+=4;
	    this.dattribs_names.addValue(prefix+".Iplow_p_up_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".Iphigh_p_up_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".Iplow_p_down_"+r+"_",s,null);
	    this.dattribs_names.addValue(prefix+".Iphigh_p_down_"+r+"_",s,null);
	}

	n+=4;
	this.dattribs_names.addValue(prefix+".getTID_",s,null);
	this.dattribs_names.addValue(prefix+".pm_",s,null);
	this.dattribs_names.addValue(prefix+".pm_l_",s,null);
	this.dattribs_names.addValue(prefix+".pm_h_",s,null);

	n+=4;
	this.dattribs_names.addValue(prefix+".enter_buy_",s,null);
	this.dattribs_names.addValue(prefix+".exit_buy_",s,null);
	this.dattribs_names.addValue(prefix+".enter_sell_",s,null);
	this.dattribs_names.addValue(prefix+".exit_sell_",s,null);

	return n;
    }

    private int printPTradeIPValues(final CalculateKendall.PTradeIP x,final CalculationData c)
    {
	int n=0;
	n+=4;
	this.dattribs.addValue(x.getD0T(c.p_last));
	this.dattribs.addValue(x.getD1T(c.p_last));
	this.dattribs.addValue(x.getD0T(c.p_buy));
	this.dattribs.addValue(x.getD0T(c.p_sell));

	n+=4;
	this.dattribs.addValue(x.vattractors.length);
	this.dattribs.addValue(x.vrepellers.length);
	this.dattribs.addValue(x.p_up.length);
	this.dattribs.addValue(x.p_down.length);
	for(int r=0;r<2;r++)
	{
	    n+=4;
	    this.dattribs.addValue(r<x.vattractors.length?x.vattractors[r]:Double.NaN);
	    this.dattribs.addValue(r<x.vrepellers.length?x.vrepellers[r]:Double.NaN);
	    this.dattribs.addValue(r<x.p_up.length?x.p_up[r]:Double.NaN);
	    this.dattribs.addValue(r<x.p_down.length?x.p_down[r]:Double.NaN);
	}
	n+=2;
	this.dattribs.addValue(x.getPDownClose(c.p_last));
	this.dattribs.addValue(x.p_offset);

	n+=4;
	this.dattribs.addValue(x.min_A);
	this.dattribs.addValue(x.max_A);
	this.dattribs.addValue(x.min_R);
	this.dattribs.addValue(x.max_R);

	return n;
    }

    private int printDPTradeIPValues(final CalculateKendall.DPTradeIP x,final CalculationData c)
    {
	int n=printPTradeIPValues(x,c);
	n+=4;

	this.dattribs.addValue(x.pa_Ihigh);
	this.dattribs.addValue(x.pa_Ilow);
	this.dattribs.addValue(x.pa_m);
	this.dattribs.addValue(x.pr_m);

	n+=2;
	this.dattribs.addValue(x.getI1(c.p_buy));
	this.dattribs.addValue(x.getI1(c.p_sell));

	n+=6;
	this.dattribs.addValue(x.min_IA);
	this.dattribs.addValue(x.max_IA);
	this.dattribs.addValue(x.ex_IA);
	this.dattribs.addValue(x.min_IR);
	this.dattribs.addValue(x.max_IR);
	this.dattribs.addValue(x.ex_IR);

	for(int r=0;r<2;r++)
	{
	    n+=2;
	    this.dattribs.addValue(x.getI1(r<x.p_up.length?x.p_up[r]:Double.NaN));
	    this.dattribs.addValue(x.getI1(r<x.p_down.length?x.p_down[r]:Double.NaN));

	    n+=4;
	    this.dattribs.addValue(r<x.p_up.length?x.Iplow_p_up[r]:Double.NaN);
	    this.dattribs.addValue(r<x.p_up.length?x.Iphigh_p_up[r]:Double.NaN);
	    this.dattribs.addValue(r<x.p_down.length?x.Iplow_p_down[r]:Double.NaN);
	    this.dattribs.addValue(r<x.p_down.length?x.Iphigh_p_down[r]:Double.NaN);
	}


	n+=4;
	this.dattribs.addValue(x.ptopology.getTID());
	this.dattribs.addValue(x.ptopology.pm);
	this.dattribs.addValue(x.ptopology.pm_l);
	this.dattribs.addValue(x.ptopology.pm_h);

	n+=4;
	this.dattribs.addValue(x.ptopology.checkEnterBuy(c.p_last)?1:0);
	this.dattribs.addValue(x.ptopology.checkExitBuy(c.p_last)?1:0);
	this.dattribs.addValue(x.ptopology.checkEnterSell(c.p_last)?1:0);
	this.dattribs.addValue(x.ptopology.checkExitSell(c.p_last)?1:0);

	return n;
    }

   private int setAttrLiquidity15(final boolean flag_processheader,
				  final String ticker,
				  final long [] delays,
				  final CalculationData c)
    {
	int n=0;
	for(int j=0;j<delays.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=String.valueOf(delays[j]/ModelParameters.DISTANCE_TIME_SCALE);

		if(true)
		{
		    for(String x:new String []{"dbtip.buy","dbtip.sell","dbtip.total","dbtip.total2","dbtip.diff","dbtip.difft","dtbstip.diff","dtbstip.diff2","dtbstip.buy","dtbstip.sell","sdtip.diff","sdtip.diff2","sdtip.buy","sdtip.sell"})
		    {
			n+=printPTradeIPNames("LF15.dtip."+x,s);
		    }
		    for(String x:new String []{"dpt.diff","dpt.kdiff","dpt.diff2","dpt.total","dbstip.diff","dbstip.kdiff","dbstip.diff2","dbstip.total","kdt.kdiff","kdt.diff","kdt.diff3","kdt.buy","kdt.sell"})
		    {
			n+=printDPTradeIPNames("LF15.dd."+x,s);
		    }
		}

		n+=4;
		this.dattribs_names.addValue("LF15.tip.p_offset_",s,null);
		this.dattribs_names.addValue("LF15.tip.sigma_",s,null);
		this.dattribs_names.addValue("LF15.tip.skewness_",s,null);
		this.dattribs_names.addValue("LF15.tip.kurtosis_",s,null);

		n+=10;
		this.dattribs_names.addValue("LF15.tip.aIPlast_",s,null);
		this.dattribs_names.addValue("LF15.tip.aIPaver_",s,null);
		this.dattribs_names.addValue("LF15.tip.DILast_buy_",s,null);
		this.dattribs_names.addValue("LF15.tip.DILast_sell_",s,null);
		this.dattribs_names.addValue("LF15.tip.DILast_sum_",s,null);
		this.dattribs_names.addValue("LF15.tip.DILast_diff_",s,null);
		this.dattribs_names.addValue("LF15.tip.ILast_buy_",s,null);
		this.dattribs_names.addValue("LF15.tip.ILast_sell_",s,null);
		this.dattribs_names.addValue("LF15.tip.ILast_sum_",s,null);
		this.dattribs_names.addValue("LF15.tip.ILast_diff_",s,null);

		n+=13;
		this.dattribs_names.addValue("LF15.tip.p_forward_",s,null);
		this.dattribs_names.addValue("LF15.tip.p_forward_sd_",s,null);
		this.dattribs_names.addValue("LF15.tip.p_backward_",s,null);
		this.dattribs_names.addValue("LF15.tip.pw_close_",s,null);
		this.dattribs_names.addValue("LF15.tip.pw_stable_",s,null);
		this.dattribs_names.addValue("LF15.tip.I0_",s,null);
		this.dattribs_names.addValue("LF15.tip.Iplast_",s,null);
		this.dattribs_names.addValue("LF15.tip.dirv_",s,null);
		this.dattribs_names.addValue("LF15.tip.diff_dirv_",s,null);
		this.dattribs_names.addValue("LF15.tip.diffw_dirv_",s,null);
		this.dattribs_names.addValue("LF15.tip.diffw_dirv_dp_",s,null);
		this.dattribs_names.addValue("LF15.tip.dirv_buy_",s,null);
		this.dattribs_names.addValue("LF15.tip.dirv_sell_",s,null);
		if(true)
		{
		    n+=4;
		    this.dattribs_names.addValue("LF15.tip.n_roots_",s,null);
		    this.dattribs_names.addValue("LF15.tip.n_vroots_",s,null);
		    this.dattribs_names.addValue("LF15.tip.n_vattractors_",s,null);
		    this.dattribs_names.addValue("LF15.tip.n_vrepellers_",s,null);
		    n+=4;
		    this.dattribs_names.addValue("LF15.tip.min_A_",s,null);
		    this.dattribs_names.addValue("LF15.tip.max_A_",s,null);
		    this.dattribs_names.addValue("LF15.tip.ex_A_",s,null);
		    this.dattribs_names.addValue("LF15.tip.i0_A_",s,null);
		    n+=4;
		    this.dattribs_names.addValue("LF15.tip.min_R_",s,null);
		    this.dattribs_names.addValue("LF15.tip.max_R_",s,null);
		    this.dattribs_names.addValue("LF15.tip.ex_R_",s,null);
		    this.dattribs_names.addValue("LF15.tip.i0_R_",s,null);
		    for(int r=0;r<2;r++)
		    {
			n+=2;
			this.dattribs_names.addValue("LF15.tip.attractor_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.repeller_"+r+"_",s,null);
			n+=2;
			this.dattribs_names.addValue("LF15.tip.vattractors_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.vrepellers_"+r+"_",s,null);
			n+=2;
			this.dattribs_names.addValue("LF15.tip.Ivattractor_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.Ivrepeller_"+r+"_",s,null);
			n+=2;
			this.dattribs_names.addValue("LF15.tip.interval_min_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.interval_max_"+r+"_",s,null);
			n+=8;
			this.dattribs_names.addValue("LF15.tip.DIrpos_buy_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.DIrneg_buy_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.DIrpos_sell_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.DIrneg_sell_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.DIrpos_sum_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.DIrneg_sum_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.DIrpos_diff_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.DIrneg_diff_"+r+"_",s,null);
		    }
		    n+=4;
		    this.dattribs_names.addValue("LF15.tip.Emin_P_buy_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emax_P_buy_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emin_N_buy_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emax_N_buy_",s,null);
		    n+=4;
		    this.dattribs_names.addValue("LF15.tip.Emin_P_sell_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emax_P_sell_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emin_N_sell_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emax_N_sell_",s,null);
		    n+=4;
		    this.dattribs_names.addValue("LF15.tip.Emin_P_sum_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emax_P_sum_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emin_N_sum_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emax_N_sum_",s,null);
		    n+=4;
		    this.dattribs_names.addValue("LF15.tip.Emin_P_diff_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emax_P_diff_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emin_N_diff_",s,null);
		    this.dattribs_names.addValue("LF15.tip.Emax_N_diff_",s,null);

		}

		n+=7;
		this.dattribs_names.addValue("LF15.tip.dirp0_",s,null);
		this.dattribs_names.addValue("LF15.tip.dirv0_",s,null);
		this.dattribs_names.addValue("LF15.tip.ddirp_",s,null);
		this.dattribs_names.addValue("LF15.tip.ddirpbuy_",s,null);
		this.dattribs_names.addValue("LF15.tip.ddirpsell_",s,null);
		this.dattribs_names.addValue("LF15.tip.dddirp_",s,null);
		this.dattribs_names.addValue("LF15.tip.dp_",s,null);
		if(true)
		{
		    n+=2;
		    this.dattribs_names.addValue("LF15.tip.n_p_up_",s,null);
		    this.dattribs_names.addValue("LF15.tip.n_p_down_",s,null);
		    for(int r=0;r<2;r++)
		    {
			n+=2;
			this.dattribs_names.addValue("LF15.tip.p_up_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.tip.p_down_"+r+"_",s,null);
		    }

		    n+=8;
		    this.dattribs_names.addValue("LF15.tip.fA.up_enter_",s,null);
		    this.dattribs_names.addValue("LF15.tip.fA.up_exit_",s,null);
		    this.dattribs_names.addValue("LF15.tip.fA.down_enter_",s,null);
		    this.dattribs_names.addValue("LF15.tip.fA.down_exit_",s,null);
		    this.dattribs_names.addValue("LF15.tip.fB.up_enter_",s,null);
		    this.dattribs_names.addValue("LF15.tip.fB.up_exit_",s,null);
		    this.dattribs_names.addValue("LF15.tip.fB.down_enter_",s,null);
		    this.dattribs_names.addValue("LF15.tip.fB.down_exit_",s,null);
		}



		n+=4;
		this.dattribs_names.addValue("LF15.otip.p_offset_",s,null);
		this.dattribs_names.addValue("LF15.otip.sigma_",s,null);
		this.dattribs_names.addValue("LF15.otip.skewness_",s,null);
		this.dattribs_names.addValue("LF15.otip.kurtosis_",s,null);
		if(true)
		{
		    n+=3;
		    this.dattribs_names.addValue("LF15.otip.oIPlast_",s,null);
		    this.dattribs_names.addValue("LF15.otip.n_p_up_",s,null);
		    this.dattribs_names.addValue("LF15.otip.n_p_down_",s,null);
		    for(int r=0;r<2;r++)
		    {
			n+=2;
			this.dattribs_names.addValue("LF15.otip.p_up_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.otip.p_down_"+r+"_",s,null);
		    }

		    n+=3;
		    this.dattribs_names.addValue("LF15.otip.n_vroots_",s,null);
		    this.dattribs_names.addValue("LF15.otip.n_vattractors_",s,null);
		    this.dattribs_names.addValue("LF15.otip.n_vrepellers_",s,null);
		    n+=4;
		    this.dattribs_names.addValue("LF15.otip.min_A_",s,null);
		    this.dattribs_names.addValue("LF15.otip.max_A_",s,null);
		    this.dattribs_names.addValue("LF15.otip.ex_A_",s,null);
		    this.dattribs_names.addValue("LF15.otip.i0_A_",s,null);
		    n+=4;
		    this.dattribs_names.addValue("LF15.otip.min_R_",s,null);
		    this.dattribs_names.addValue("LF15.otip.max_R_",s,null);
		    this.dattribs_names.addValue("LF15.otip.ex_R_",s,null);
		    this.dattribs_names.addValue("LF15.otip.i0_R_",s,null);
		    for(int r=0;r<2;r++)
		    {
			n+=2;
			this.dattribs_names.addValue("LF15.otip.vattractors_"+r+"_",s,null);
			this.dattribs_names.addValue("LF15.otip.vrepellers_"+r+"_",s,null);
		    }
		}
	    }
	    else
	    {
		final CalculateLiquidityDeficit15 l=c.topp.book_trades_liquiditydeficit15;
		final CalculateLiquidityDeficit15.VXM vx=l.tdata[j];
		if(true)
		{
		    for(CalculateKendall.PTradeIP x:new CalculateKendall.PTradeIP[]{vx.dbtip.buy,vx.dbtip.sell,vx.dbtip.total,vx.dbtip.total2,vx.dbtip.diff,vx.dbtip.difft,vx.dtbstip.diff,vx.dtbstip.diff2,vx.dtbstip.buy,vx.dtbstip.sell,vx.sdtip.diff,vx.sdtip.diff2,vx.sdtip.buy,vx.sdtip.sell})
		    {
			n+=printPTradeIPValues(x,c);
		    }
		    for(CalculateKendall.DPTradeIP x:new CalculateKendall.DPTradeIP[]{vx.dpt.diff,vx.dpt.kdiff,vx.dpt.diff2,vx.dpt.total,vx.dbstip.diff,vx.dbstip.kdiff,vx.dbstip.diff2,vx.dbstip.total,vx.kdt.kdiff,vx.kdt.diff,vx.kdt.diff3,vx.kdt.buy,vx.kdt.sell})
		    {
			n+=printDPTradeIPValues(x,c);
		    }
		}

		n+=4;
		this.dattribs.addValue(vx.tip.tipdata.p_offset);
		this.dattribs.addValue(vx.tip.sigma);
		this.dattribs.addValue(vx.tip.skewness);
		this.dattribs.addValue(vx.tip.kurtosis);

		n+=10;
		this.dattribs.addValue(vx.tip.aIPlast);
		this.dattribs.addValue(vx.tip.aIPaver);
		this.dattribs.addValue(vx.tip.getD1T(c.p_last,CalculateKendall.TradeIP.D_BUY));
		this.dattribs.addValue(vx.tip.getD1T(c.p_last,CalculateKendall.TradeIP.D_SELL));
		this.dattribs.addValue(vx.tip.getD1T(c.p_last,CalculateKendall.TradeIP.D_SUMM));
		this.dattribs.addValue(vx.tip.getD1T(c.p_last,CalculateKendall.TradeIP.D_DIFF));
		this.dattribs.addValue(vx.tip.getD0T(c.p_last,CalculateKendall.TradeIP.D_BUY));
		this.dattribs.addValue(vx.tip.getD0T(c.p_last,CalculateKendall.TradeIP.D_SELL));
		this.dattribs.addValue(vx.tip.getD0T(c.p_last,CalculateKendall.TradeIP.D_SUMM));
		this.dattribs.addValue(vx.tip.getD0T(c.p_last,CalculateKendall.TradeIP.D_DIFF));

		n+=13;
		this.dattribs.addValue(vx.tip.p_forward);
		this.dattribs.addValue(vx.tip.p_forward_sd);
		this.dattribs.addValue(vx.tip.p_backward);
		this.dattribs.addValue(vx.tip.pw_close);
		this.dattribs.addValue(vx.tip.pw_stable);
		this.dattribs.addValue(vx.tip.I0);
		this.dattribs.addValue(vx.tip.Iplast);
		this.dattribs.addValue(vx.tip.dirv);
		this.dattribs.addValue(vx.tip.diff_dirv);
		this.dattribs.addValue(vx.tip.diffw_dirv);
		this.dattribs.addValue(vx.tip.diffw_dirv_dp);
		this.dattribs.addValue(vx.tip.dirv_buy);
		this.dattribs.addValue(vx.tip.dirv_sell);

		if(true)
		{
		    n+=4;
		    this.dattribs.addValue(vx.tip.attractors.length+vx.tip.repellers.length);
		    this.dattribs.addValue(vx.tip.vattractors.length+vx.tip.vrepellers.length);
		    this.dattribs.addValue(vx.tip.vattractors.length);
		    this.dattribs.addValue(vx.tip.vrepellers.length);
		    n+=4;
		    this.dattribs.addValue(vx.tip.min_A);
		    this.dattribs.addValue(vx.tip.max_A);
		    this.dattribs.addValue(vx.tip.ex_A);
		    this.dattribs.addValue(vx.tip.i0_A);
		    n+=4;
		    this.dattribs.addValue(vx.tip.min_R);
		    this.dattribs.addValue(vx.tip.max_R);
		    this.dattribs.addValue(vx.tip.ex_R);
		    this.dattribs.addValue(vx.tip.i0_R);
		    for(int r=0;r<2;r++)
		    {
			n+=2;
			this.dattribs.addValue(r<vx.tip.attractors.length?vx.tip.attractors[r]:Double.NaN);
			this.dattribs.addValue(r<vx.tip.repellers.length?vx.tip.repellers[r]:Double.NaN);

			n+=2;
			this.dattribs.addValue(r<vx.tip.vattractors.length?vx.tip.vattractors[r]:Double.NaN);
			this.dattribs.addValue(r<vx.tip.vrepellers.length?vx.tip.vrepellers[r]:Double.NaN);

			n+=2;
			this.dattribs.addValue(vx.tip.getIV(r<vx.tip.vattractors.length?vx.tip.vattractors[r]:Double.NaN));
			this.dattribs.addValue(vx.tip.getIV(r<vx.tip.vrepellers.length?vx.tip.vrepellers[r]:Double.NaN));

			n+=2;
			this.dattribs.addValue(r<vx.tip.interval_min.length?vx.tip.interval_min[r]:Double.NaN);
			this.dattribs.addValue(r<vx.tip.interval_max.length?vx.tip.interval_max[r]:Double.NaN);

			n+=8;
			this.dattribs.addValue(r<vx.tip.DIrpos_buy.length?vx.tip.DIrpos_buy[r]:Double.NaN);
			this.dattribs.addValue(r<vx.tip.DIrneg_buy.length?vx.tip.DIrneg_buy[r]:Double.NaN);
			this.dattribs.addValue(r<vx.tip.DIrpos_sell.length?vx.tip.DIrpos_sell[r]:Double.NaN);
			this.dattribs.addValue(r<vx.tip.DIrneg_sell.length?vx.tip.DIrneg_sell[r]:Double.NaN);
			this.dattribs.addValue(r<vx.tip.DIrpos_sum.length?vx.tip.DIrpos_sum[r]:Double.NaN);
			this.dattribs.addValue(r<vx.tip.DIrneg_sum.length?vx.tip.DIrneg_sum[r]:Double.NaN);
			this.dattribs.addValue(r<vx.tip.DIrpos_diff.length?vx.tip.DIrpos_diff[r]:Double.NaN);
			this.dattribs.addValue(r<vx.tip.DIrneg_diff.length?vx.tip.DIrneg_diff[r]:Double.NaN);
		    }
		    n+=4;
		    this.dattribs.addValue(vx.tip.Emin_P_buy);
		    this.dattribs.addValue(vx.tip.Emax_P_buy);
		    this.dattribs.addValue(vx.tip.Emin_N_buy);
		    this.dattribs.addValue(vx.tip.Emax_N_buy);
		    n+=4;
		    this.dattribs.addValue(vx.tip.Emin_P_sell);
		    this.dattribs.addValue(vx.tip.Emax_P_sell);
		    this.dattribs.addValue(vx.tip.Emin_N_sell);
		    this.dattribs.addValue(vx.tip.Emax_N_sell);
		    n+=4;
		    this.dattribs.addValue(vx.tip.Emin_P_sum);
		    this.dattribs.addValue(vx.tip.Emax_P_sum);
		    this.dattribs.addValue(vx.tip.Emin_N_sum);
		    this.dattribs.addValue(vx.tip.Emax_N_sum);
		    n+=4;
		    this.dattribs.addValue(vx.tip.Emin_P_diff);
		    this.dattribs.addValue(vx.tip.Emax_P_diff);
		    this.dattribs.addValue(vx.tip.Emin_N_diff);
		    this.dattribs.addValue(vx.tip.Emax_N_diff);
		}

		n+=7;
		this.dattribs.addValue(vx.tip.dirp0);
		this.dattribs.addValue(vx.tip.dirv0);
		this.dattribs.addValue(vx.tip.ddirp);
		this.dattribs.addValue(vx.tip.getDDirP(c.p_buy));
		this.dattribs.addValue(vx.tip.getDDirP(c.p_sell));
		this.dattribs.addValue(vx.tip.dddirp);
		this.dattribs.addValue(vx.tip.dp);
		if(true)
		{
		    n+=2;
		    this.dattribs.addValue(vx.tip.p_up.length);
		    this.dattribs.addValue(vx.tip.p_down.length);
		    for(int r=0;r<2;r++)
		    {
			n+=2;
			this.dattribs.addValue(r<vx.tip.p_up.length?vx.tip.p_up[r]:Double.NaN);
			this.dattribs.addValue(r<vx.tip.p_down.length?vx.tip.p_down[r]:Double.NaN);
		    }

		    n+=8;
		    final double [] u=vx.tip.p_up,d=vx.tip.p_down;
		    final int nu=u.length,nd=d.length;
		    this.dattribs.addValue(nu<=0 || nd<=0 ?Double.NaN:(d[0]<u[0]?d[0]:Double.NaN));
		    this.dattribs.addValue(nu<=0 || nd<=0 ?Double.NaN:(d[0]<u[0]?u[0]:u[nu-1]));
		    this.dattribs.addValue(nu<=0 || nd<=0 ?Double.NaN:(d[0]<u[0]?u[nu-1]:Double.NaN));
		    this.dattribs.addValue(nu<=0 || nd<=0 ?Double.NaN:(d[0]<u[0]?d[nd-1]:d[0]));

		    this.dattribs.addValue(nu<=0 || nd<=0 ?Double.NaN:(u[0]<d[0]?u[0]:Double.NaN));
		    this.dattribs.addValue(nu<=0 || nd<=0 ?Double.NaN:(u[0]<d[0]?d[0]:d[nd-1]));
		    this.dattribs.addValue(nu<=0 || nd<=0 ?Double.NaN:(u[0]<d[0]?d[nd-1]:Double.NaN));
		    this.dattribs.addValue(nu<=0 || nd<=0 ?Double.NaN:(u[0]<d[0]?u[nu-1]:u[0]));
		}


		n+=4;
		this.dattribs.addValue(vx.otip.tipdata.p_offset);
		this.dattribs.addValue(vx.otip.sigma);
		this.dattribs.addValue(vx.otip.skewness);
		this.dattribs.addValue(vx.otip.kurtosis);
		if(true)
		{
		    n+=3;
		    this.dattribs.addValue(vx.otip.getD0T(c.p_last));
		    this.dattribs.addValue(vx.otip.p_up.length);
		    this.dattribs.addValue(vx.otip.p_down.length);
		    for(int r=0;r<2;r++)
		    {
			n+=2;
			this.dattribs.addValue(r<vx.otip.p_up.length?vx.otip.p_up[r]:Double.NaN);
			this.dattribs.addValue(r<vx.otip.p_down.length?vx.otip.p_down[r]:Double.NaN);
		    }


		    n+=3;
		    this.dattribs.addValue(vx.otip.vattractors.length+vx.otip.vrepellers.length);
		    this.dattribs.addValue(vx.otip.vattractors.length);
		    this.dattribs.addValue(vx.otip.vrepellers.length);
		    n+=4;
		    this.dattribs.addValue(vx.otip.min_A);
		    this.dattribs.addValue(vx.otip.max_A);
		    this.dattribs.addValue(vx.otip.ex_A);
		    this.dattribs.addValue(vx.otip.i0_A);
		    n+=4;
		    this.dattribs.addValue(vx.otip.min_R);
		    this.dattribs.addValue(vx.otip.max_R);
		    this.dattribs.addValue(vx.otip.ex_R);
		    this.dattribs.addValue(vx.otip.i0_R);
		    for(int r=0;r<2;r++)
		    {
			n+=2;
			this.dattribs.addValue(r<vx.otip.vattractors.length?vx.otip.vattractors[r]:Double.NaN);
			this.dattribs.addValue(r<vx.otip.vrepellers.length?vx.otip.vrepellers[r]:Double.NaN);
		    }
		}
	    }
	}
	return n;
    }




    private int setAttrLiquidity13(final boolean flag_processheader,
				   final String ticker,
				   final long [] delays,
				   final CalculationData c)
    {
	int n=0;
	for(int j=0;j<delays.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=String.valueOf(delays[j]/ModelParameters.DISTANCE_TIME_SCALE);
		n+=37;
		this.sattribs_names.addValue("LF13.te.tau_",s,null);
		this.dattribs_names.addValue("LF13.te.p_last_",s,null);
		this.dattribs_names.addValue("LF13.te.d1Q_",s,null);

		this.dattribs_names.addValue("LF13.te.d1_b1_",s,null);
		this.dattribs_names.addValue("LF13.te.d1_b2_",s,null);
		this.dattribs_names.addValue("LF13.te.d1_b3_",s,null);
		this.dattribs_names.addValue("LF13.te.d1_b4_",s,null);
		this.dattribs_names.addValue("LF13.te.d1_b5_",s,null);

		this.dattribs_names.addValue("LF13.te.d0fa_",s,null);
		this.dattribs_names.addValue("LF13.te.dp0fa_",s,null);

		this.dattribs_names.addValue("LF13.te.ddpt0a_",s,null);
		this.dattribs_names.addValue("LF13.te.ddpv0a_",s,null);
		this.dattribs_names.addValue("LF13.te.dvsignedt0a_",s,null);
		this.dattribs_names.addValue("LF13.te.dvsignedv0a_",s,null);
		this.dattribs_names.addValue("LF13.te.dv0a_",s,null);
		this.dattribs_names.addValue("LF13.te.dv1e_",s,null);
		this.dattribs_names.addValue("LF13.te.dv2e_",s,null);
		this.dattribs_names.addValue("LF13.te.dv3e_",s,null);
		this.dattribs_names.addValue("LF13.te.dp0a_",s,null);
		this.dattribs_names.addValue("LF13.te.dp1e_",s,null);
		this.dattribs_names.addValue("LF13.te.dp2e_",s,null);
		this.dattribs_names.addValue("LF13.te.dp3e_",s,null);
		this.dattribs_names.addValue("LF13.te.dp1f_",s,null);

		this.dattribs_names.addValue("LF13.te.d1e_",s,null);
		this.dattribs_names.addValue("LF13.te.d2e_",s,null);
		this.dattribs_names.addValue("LF13.te.d3e_",s,null);
		this.dattribs_names.addValue("LF13.te.d1f_",s,null);
		this.dattribs_names.addValue("LF13.te.d2f_",s,null);
		this.dattribs_names.addValue("LF13.te.d3f_",s,null);
		this.dattribs_names.addValue("LF13.te.d0a_",s,null);
		this.dattribs_names.addValue("LF13.te.d1a_",s,null);
		this.dattribs_names.addValue("LF13.te.d2a_",s,null);
		this.dattribs_names.addValue("LF13.te.d3a_",s,null);
		this.dattribs_names.addValue("LF13.te.d0aa_",s,null);
		this.dattribs_names.addValue("LF13.te.d1aa_",s,null);
		this.dattribs_names.addValue("LF13.te.d2aa_",s,null);
		this.dattribs_names.addValue("LF13.te.d3aa_",s,null);
		for(int m=0;m<CalculateKendall.N_EMOMENTS;m++)
		{
		    n+=7;
		    this.dattribs_names.addValue("LF13.te.em"+m+"_",s,null);
		    this.dattribs_names.addValue("LF13.te.empt"+m+"_",s,null);
		    this.dattribs_names.addValue("LF13.te.emvt"+m+"_",s,null);
		    this.dattribs_names.addValue("LF13.te.sum_v"+m+"_",s,null);
		    this.dattribs_names.addValue("LF13.te.sum_vsigned"+m+"_",s,null);
		    this.dattribs_names.addValue("LF13.te.sum_vtau"+m+"_",s,null);
		    this.dattribs_names.addValue("LF13.te.iv"+m+"_",s,null);
		}
		n+=7;
		this.dattribs_names.addValue("LF13.te.d1e_iv_",s,null);
		this.dattribs_names.addValue("LF13.te.d1a_iv_",s,null);
		this.dattribs_names.addValue("LF13.te.d1e_v_",s,null);
		this.dattribs_names.addValue("LF13.te.d1a_v_",s,null);
		this.dattribs_names.addValue("LF13.te.d1a_vsigned_",s,null);
		this.dattribs_names.addValue("LF13.te.d0a_vsigned_",s,null);
		this.dattribs_names.addValue("LF13.te.d0a_v_",s,null);



		/*
		n+=8;
		this.dattribs_names.addValue("LF13.ptm.apt0_",s,null);
		this.dattribs_names.addValue("LF13.ptm.ap2t0_",s,null);
		this.dattribs_names.addValue("LF13.ptm.ap3t0_",s,null);
		this.dattribs_names.addValue("LF13.ptm.ap5t0_",s,null);
		this.dattribs_names.addValue("LF13.ptm.d0a_",s,null);
		this.dattribs_names.addValue("LF13.ptm.d1e_",s,null);
		this.dattribs_names.addValue("LF13.ptm.d2e_",s,null);
		this.dattribs_names.addValue("LF13.ptm.d3e_",s,null);
		for(int t=0;t<CalculateKendall.N_PTMOMENTS_T;t++)
		{
		    n+=3;
		    this.dattribs_names.addValue("LF13.ptm"+t+".pm2_",s,null);
		    this.dattribs_names.addValue("LF13.ptm"+t+".pm3_",s,null);
		    this.dattribs_names.addValue("LF13.ptm"+t+".pm5_",s,null);
		}
		*/
	    }
	    else
	    {
		final CalculateLiquidityDeficit13 l=c.topp.book_trades_liquiditydeficit13;
		n+=37;
		final CalculateKendall.TradeEMoments em=l.tdata[j];
		this.sattribs.addValue(String.valueOf(em.tau));
		this.dattribs.addValue(em.p_last);
		this.dattribs.addValue(em.d1Q);

		this.dattribs.addValue(em.d1_b1);
		this.dattribs.addValue(em.d1_b2);
		this.dattribs.addValue(em.d1_b3);
		this.dattribs.addValue(em.d1_b4);
		this.dattribs.addValue(em.d1_b5);

		this.dattribs.addValue(em.d0fa);
		this.dattribs.addValue(em.dp0fa);

		this.dattribs.addValue(em.ddpt0a);
		this.dattribs.addValue(em.ddpv0a);
		this.dattribs.addValue(em.dvsignedt0a);
		this.dattribs.addValue(em.dvsignedv0a);
		this.dattribs.addValue(em.dv0a);
		this.dattribs.addValue(em.dv1e);
		this.dattribs.addValue(em.dv2e);
		this.dattribs.addValue(em.dv3e);
		this.dattribs.addValue(em.dp0a);
		this.dattribs.addValue(em.dp1e);
		this.dattribs.addValue(em.dp2e);
		this.dattribs.addValue(em.dp3e);
		this.dattribs.addValue(em.dp1f);

		this.dattribs.addValue(em.d1e);
		this.dattribs.addValue(em.d2e);
		this.dattribs.addValue(em.d3e);
		this.dattribs.addValue(em.d1f);
		this.dattribs.addValue(em.d2f);
		this.dattribs.addValue(em.d3f);
		this.dattribs.addValue(em.d0a);
		this.dattribs.addValue(em.d1a);
		this.dattribs.addValue(em.d2a);
		this.dattribs.addValue(em.d3a);
		this.dattribs.addValue(em.d0aa);
		this.dattribs.addValue(em.d1aa);
		this.dattribs.addValue(em.d2aa);
		this.dattribs.addValue(em.d3aa);
		for(int m=0;m<CalculateKendall.N_EMOMENTS;m++)
		{
		    n+=7;
		    this.dattribs.addValue(em.emoments[m]);
		    this.dattribs.addValue(em.emoments_pt[m]);
		    this.dattribs.addValue(em.emoments_vt[m]);
		    this.dattribs.addValue(em.sum_v_emoments[m]);
		    this.dattribs.addValue(em.sum_vsigned_emoments[m]);
		    this.dattribs.addValue(em.sum_vtau[m]);
		    this.dattribs.addValue(em.iv[m]);
		}

		n+=7;
		this.dattribs.addValue(em.d1e_iv);
		this.dattribs.addValue(em.d1a_iv);
		this.dattribs.addValue(em.d1e_v);
		this.dattribs.addValue(em.d1a_v);
		this.dattribs.addValue(em.d1a_vsigned);
		this.dattribs.addValue(em.d0a_vsigned);
		this.dattribs.addValue(em.d0a_v);

		/*
		final CalculateKendall.TradePTMoments ptm=l.tpmdata[j];
		n+=8;
		this.dattribs.addValue(ptm.apt0);
		this.dattribs.addValue(ptm.ap2t0);
		this.dattribs.addValue(ptm.ap3t0);
		this.dattribs.addValue(ptm.ap5t0);
		this.dattribs.addValue(ptm.d0a);
		this.dattribs.addValue(ptm.d1e);
		this.dattribs.addValue(ptm.d2e);
		this.dattribs.addValue(ptm.d3e);
		for(int t=0;t<CalculateKendall.N_PTMOMENTS_T;t++)
		{
		    n+=3;
		    this.dattribs.addValue(ptm.ap2[t]);
		    this.dattribs.addValue(ptm.ap3[t]);
		    this.dattribs.addValue(ptm.ap5[t]);
		}
		*/
	    }
	}
	return n;
    }





private int setAttrLiquidity12(final boolean flag_processheader,
			       final String ticker,
			       final int nt,
			       final CalculationData c)
    {
	int n=0;
	if(flag_processheader)
	{
	    n+=4;
	    this.dattribs_names.addValue("LF12.buy.liq_","",null);
	    this.dattribs_names.addValue("LF12.buy.liq_aver_","",null);
	    this.dattribs_names.addValue("LF12.sell.liq_","",null);
	    this.dattribs_names.addValue("LF12.sell.liq_aver_","",null);
	}
	else
	{
	    n+=4;
	    final CalculateLiquidityDeficit12 l=c.topp.book_trades_liquiditydeficit12;
	    final CalculateKendall.BookTradesBKStats lbuy=l.getBookBuyLiquidityLast();
	    if(lbuy!=null)
	    {
		this.dattribs.addValue(lbuy.p_stop);
		this.dattribs.addValue(lbuy.pbook_average);
	    }
	    else
	    {
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
	    }
	    final CalculateKendall.BookTradesBKStats lsell=l.getBookSellLiquidityLast();
	    if(lsell!=null)
	    {
		this.dattribs.addValue(lsell.p_stop);
		this.dattribs.addValue(lsell.pbook_average);
	    }
	    else
	    {
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
	    }
	}


	for(int j=0;j<nt;j++)
	{
	    if(flag_processheader)
	    {
		n+=16+14+16+14+7+7+14+12;
		final String s=String.valueOf(j);
		this.dattribs_names.addValue("LF12.buy.vbook_",s,null);
		this.dattribs_names.addValue("LF12.buy.tau_book_",s,null);
		this.dattribs_names.addValue("LF12.buy.p0_",s,null);
		this.dattribs_names.addValue("LF12.buy.p_stop_",s,null);
		this.dattribs_names.addValue("LF12.buy.pbook_average_",s,null);
		this.dattribs_names.addValue("LF12.buy.pbook_average_var2_",s,null);
		this.dattribs_names.addValue("LF12.buy.pbook_average_k2_",s,null);
		this.dattribs_names.addValue("LF12.buy.pbook_average_k3_",s,null);
		this.dattribs_names.addValue("LF12.buy.pbook_x_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.buy.normaK1_",s,null);
		this.dattribs_names.addValue("LF12.buy.normaK2_",s,null);
		this.dattribs_names.addValue("LF12.buy.normaK3	_",s,null);
		this.dattribs_names.addValue("LF12.buy.k1_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.buy.k1_ptrades_po_var2_",s,null);
		this.dattribs_names.addValue("LF12.buy.k2_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.buy.k3_ptrades_po_",s,null);

		this.dattribs_names.addValue("LF12.buy.pr.vbook_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.tau_book_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.p0_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.p_stop_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.pbook_average_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.pbook_average_var2_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.pbook_x_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.normaK1_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.normaK2_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.normaK3	_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.k1_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.k1_ptrades_po_var2_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.k2_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.buy.pr.k3_ptrades_po_",s,null);


		this.dattribs_names.addValue("LF12.sell.vbook_",s,null);
		this.dattribs_names.addValue("LF12.sell.tau_book_",s,null);
		this.dattribs_names.addValue("LF12.sell.p0_",s,null);
		this.dattribs_names.addValue("LF12.sell.p_stop_",s,null);
		this.dattribs_names.addValue("LF12.sell.pbook_average_",s,null);
		this.dattribs_names.addValue("LF12.sell.pbook_average_var2_",s,null);
		this.dattribs_names.addValue("LF12.sell.pbook_average_k2_",s,null);
		this.dattribs_names.addValue("LF12.sell.pbook_average_k3_",s,null);
		this.dattribs_names.addValue("LF12.sell.pbook_x_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.sell.normaK1_",s,null);
		this.dattribs_names.addValue("LF12.sell.normaK2_",s,null);
		this.dattribs_names.addValue("LF12.sell.normaK3	_",s,null);
		this.dattribs_names.addValue("LF12.sell.k1_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.sell.k1_ptrades_po_var2_",s,null);
		this.dattribs_names.addValue("LF12.sell.k2_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.sell.k3_ptrades_po_",s,null);

		this.dattribs_names.addValue("LF12.sell.pr.vbook_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.tau_book_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.p0_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.p_stop_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.pbook_average_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.pbook_average_var2_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.pbook_x_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.normaK1_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.normaK2_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.normaK3_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.k1_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.k1_ptrades_po_var2_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.k2_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF12.sell.pr.k3_ptrades_po_",s,null);

		this.dattribs_names.addValue("LF12.buy.dt.pbook_average_",s,null);
		this.dattribs_names.addValue("LF12.buy.dt.pbook_average_var2_",s,null);
		this.dattribs_names.addValue("LF12.buy.dt.d1_",s,null);
		this.dattribs_names.addValue("LF12.buy.dt.d2_",s,null);
		this.dattribs_names.addValue("LF12.buy.dt.d3_",s,null);
		this.dattribs_names.addValue("LF12.buy.dt.dk2_",s,null);
		this.dattribs_names.addValue("LF12.buy.dt.norma_",s,null);
		this.dattribs_names.addValue("LF12.buy.dt.k1v_",s,null);
		this.dattribs_names.addValue("LF12.buy.dt.k2v_",s,null);
		this.dattribs_names.addValue("LF12.buy.dt.k3v_",s,null);

		this.dattribs_names.addValue("LF12.sell.dt.pbook_average_",s,null);
		this.dattribs_names.addValue("LF12.sell.dt.pbook_average_var2_",s,null);
		this.dattribs_names.addValue("LF12.sell.dt.d1_",s,null);
		this.dattribs_names.addValue("LF12.sell.dt.d2_",s,null);
		this.dattribs_names.addValue("LF12.sell.dt.d3_",s,null);
		this.dattribs_names.addValue("LF12.sell.dt.dk2_",s,null);
		this.dattribs_names.addValue("LF12.sell.dt.norma_",s,null);
		this.dattribs_names.addValue("LF12.sell.dt.k1v_",s,null);
		this.dattribs_names.addValue("LF12.sell.dt.k2v_",s,null);
		this.dattribs_names.addValue("LF12.sell.dt.k3v_",s,null);

		this.dattribs_names.addValue("LF12.buy.tau_",s,null);
		this.dattribs_names.addValue("LF12.buy.tau.k1_",s,null);
		this.dattribs_names.addValue("LF12.buy.tau.k2_",s,null);
		this.dattribs_names.addValue("LF12.buy.tau.k3_",s,null);
		this.dattribs_names.addValue("LF12.buy.tau.normaK1_",s,null);
		this.dattribs_names.addValue("LF12.buy.tau.normaK2_",s,null);
		this.dattribs_names.addValue("LF12.buy.tau.normaK3_",s,null);
		this.dattribs_names.addValue("LF12.buy.tau.k1v_",s,null);
		this.dattribs_names.addValue("LF12.buy.tau.k2v_",s,null);
		this.dattribs_names.addValue("LF12.buy.tau.k3v_",s,null);

		this.dattribs_names.addValue("LF12.sell.tau_",s,null);
		this.dattribs_names.addValue("LF12.sell.tau.k1_",s,null);
		this.dattribs_names.addValue("LF12.sell.tau.k2_",s,null);
		this.dattribs_names.addValue("LF12.sell.tau.k3_",s,null);
		this.dattribs_names.addValue("LF12.sell.tau.normaK1_",s,null);
		this.dattribs_names.addValue("LF12.sell.tau.normaK2_",s,null);
		this.dattribs_names.addValue("LF12.sell.tau.normaK3_",s,null);
		this.dattribs_names.addValue("LF12.sell.tau.k1v_",s,null);
		this.dattribs_names.addValue("LF12.sell.tau.k2v_",s,null);
		this.dattribs_names.addValue("LF12.sell.tau.k3v_",s,null);

	    }
	    else
	    {
		n+=16+14+16+14+7+7+14+12;
		final CalculateLiquidityDeficit12 l=c.topp.book_trades_liquiditydeficit12;
		final CalculateKendall.BookTradesBKStats sbuy=l.book_buy[j];
		this.dattribs.addValue(sbuy.vbook);
		this.dattribs.addValue(tt(sbuy.tau_book));
		this.dattribs.addValue(sbuy.p0);
		this.dattribs.addValue(sbuy.p_stop);
		this.dattribs.addValue(sbuy.pbook_average);
		this.dattribs.addValue(sbuy.pbook_average_var2);
		this.dattribs.addValue(sbuy.pbook_average_k2);
		this.dattribs.addValue(sbuy.pbook_average_k3);
		this.dattribs.addValue(sbuy.pbook_x_ptrades_po);
		this.dattribs.addValue(sbuy.normaK1);
		this.dattribs.addValue(sbuy.normaK2);
		this.dattribs.addValue(sbuy.normaK3);
		this.dattribs.addValue(sbuy.k1_ptrades_po);
		this.dattribs.addValue(sbuy.k1_ptrades_po_var2);
		this.dattribs.addValue(sbuy.k2_ptrades_po);
		this.dattribs.addValue(sbuy.k3_ptrades_po);
		final CalculateKendall.BookTradesBKStats sbuy_pr=l.book_buy_pr[j];
		this.dattribs.addValue(sbuy_pr.vbook);
		this.dattribs.addValue(tt(sbuy_pr.tau_book));
		this.dattribs.addValue(sbuy_pr.p0);
		this.dattribs.addValue(sbuy_pr.p_stop);
		this.dattribs.addValue(sbuy_pr.pbook_average);
		this.dattribs.addValue(sbuy_pr.pbook_average_var2);
		this.dattribs.addValue(sbuy_pr.pbook_x_ptrades_po);
		this.dattribs.addValue(sbuy_pr.normaK1);
		this.dattribs.addValue(sbuy_pr.normaK2);
		this.dattribs.addValue(sbuy_pr.normaK3);
		this.dattribs.addValue(sbuy_pr.k1_ptrades_po);
		this.dattribs.addValue(sbuy_pr.k1_ptrades_po_var2);
		this.dattribs.addValue(sbuy_pr.k2_ptrades_po);
		this.dattribs.addValue(sbuy_pr.k3_ptrades_po);


		final CalculateKendall.BookTradesBKStats ssell=l.book_sell[j];
		this.dattribs.addValue(ssell.vbook);
		this.dattribs.addValue(tt(ssell.tau_book));
		this.dattribs.addValue(ssell.p0);
		this.dattribs.addValue(ssell.p_stop);
		this.dattribs.addValue(ssell.pbook_average);
		this.dattribs.addValue(ssell.pbook_average_var2);
		this.dattribs.addValue(ssell.pbook_average_k2);
		this.dattribs.addValue(ssell.pbook_average_k3);
		this.dattribs.addValue(ssell.pbook_x_ptrades_po);
		this.dattribs.addValue(ssell.normaK1);
		this.dattribs.addValue(ssell.normaK2);
		this.dattribs.addValue(ssell.normaK3);
		this.dattribs.addValue(ssell.k1_ptrades_po);
		this.dattribs.addValue(ssell.k1_ptrades_po_var2);
		this.dattribs.addValue(ssell.k2_ptrades_po);
		this.dattribs.addValue(ssell.k3_ptrades_po);
		final CalculateKendall.BookTradesBKStats ssell_pr=l.book_sell_pr[j];
		this.dattribs.addValue(ssell_pr.vbook);
		this.dattribs.addValue(tt(ssell_pr.tau_book));
		this.dattribs.addValue(ssell_pr.p0);
		this.dattribs.addValue(ssell_pr.p_stop);
		this.dattribs.addValue(ssell_pr.pbook_average);
		this.dattribs.addValue(ssell_pr.pbook_average_var2);
		this.dattribs.addValue(ssell_pr.pbook_x_ptrades_po);
		this.dattribs.addValue(ssell_pr.normaK1);
		this.dattribs.addValue(ssell_pr.normaK2);
		this.dattribs.addValue(ssell_pr.normaK3);
		this.dattribs.addValue(ssell_pr.k1_ptrades_po);
		this.dattribs.addValue(ssell_pr.k1_ptrades_po_var2);
		this.dattribs.addValue(ssell_pr.k2_ptrades_po);
		this.dattribs.addValue(ssell_pr.k3_ptrades_po);

		final CalculateKendall.BookTradesK2Stats sbuy_dt=l.book_buy_dt[j];
		this.dattribs.addValue(sbuy_dt.pbook_average);
		this.dattribs.addValue(sbuy_dt.pbook_average_var2);
		this.dattribs.addValue(sbuy_dt.d1);
		this.dattribs.addValue(sbuy_dt.d2);
		this.dattribs.addValue(sbuy_dt.d3);
		this.dattribs.addValue(sbuy_dt.dk2);
		this.dattribs.addValue(sbuy_dt.norma);
		this.dattribs.addValue(sbuy_dt.k1v);
		this.dattribs.addValue(sbuy_dt.k2v);
		this.dattribs.addValue(sbuy_dt.k3v);

		final CalculateKendall.BookTradesK2Stats ssell_dt=l.book_sell_dt[j];
		this.dattribs.addValue(ssell_dt.pbook_average);
		this.dattribs.addValue(ssell_dt.pbook_average_var2);
		this.dattribs.addValue(ssell_dt.d1);
		this.dattribs.addValue(ssell_dt.d2);
		this.dattribs.addValue(ssell_dt.d3);
		this.dattribs.addValue(ssell_dt.dk2);
		this.dattribs.addValue(ssell_dt.norma);
		this.dattribs.addValue(ssell_dt.k1v);
		this.dattribs.addValue(ssell_dt.k2v);
		this.dattribs.addValue(ssell_dt.k3v);


		final CalculateKendall.TradesK1K2K3 sbuy_tau=l.book_buy_tau[j];
		this.dattribs.addValue(tdiff(sbuy_tau.currenttime,sbuy_tau.t_range_cut));
		this.dattribs.addValue(sbuy_tau.k1);
		this.dattribs.addValue(sbuy_tau.k2);
		this.dattribs.addValue(sbuy_tau.k3);
		this.dattribs.addValue(sbuy_tau.normaK1);
		this.dattribs.addValue(sbuy_tau.normaK2);
		this.dattribs.addValue(sbuy_tau.normaK3);
		this.dattribs.addValue(sbuy_tau.k1v);
		this.dattribs.addValue(sbuy_tau.k2v);
		this.dattribs.addValue(sbuy_tau.k3v);

		final CalculateKendall.TradesK1K2K3 ssell_tau=l.book_sell_tau[j];
		this.dattribs.addValue(tdiff(ssell_tau.currenttime,ssell_tau.t_range_cut));
		this.dattribs.addValue(ssell_tau.k1);
		this.dattribs.addValue(ssell_tau.k2);
		this.dattribs.addValue(ssell_tau.k3);
		this.dattribs.addValue(ssell_tau.normaK1);
		this.dattribs.addValue(ssell_tau.normaK2);
		this.dattribs.addValue(ssell_tau.normaK3);
		this.dattribs.addValue(ssell_tau.k1v);
		this.dattribs.addValue(ssell_tau.k2v);
		this.dattribs.addValue(ssell_tau.k3v);
	    }
	}
	return n;
    }



 private int setAttrLiquidity11(final boolean flag_processheader,
				final String ticker,
				final String [] levels_names,
				final CalculationData c)
    {
	int n=0;
	for(int j=0;j<levels_names.length;j++)
	{
	    if(flag_processheader)
	    {
		n+=16+16+5+4+4+8+4;
		final String s=levels_names[j];
		this.dattribs_names.addValue("LF11.buy.vbook_",s,null);
		this.dattribs_names.addValue("LF11.buy.tau_book_",s,null);
		this.dattribs_names.addValue("LF11.buy.p0_",s,null);
		this.dattribs_names.addValue("LF11.buy.p_stop_",s,null);
		this.dattribs_names.addValue("LF11.buy.pbook_average_",s,null);
		this.dattribs_names.addValue("LF11.buy.pbook_average_var2_",s,null);
		this.dattribs_names.addValue("LF11.buy.pbook_average_k2_",s,null);
		this.dattribs_names.addValue("LF11.buy.pbook_average_k3_",s,null);
		this.dattribs_names.addValue("LF11.buy.pbook_x_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF11.buy.normaK1_",s,null);
		this.dattribs_names.addValue("LF11.buy.normaK2_",s,null);
		this.dattribs_names.addValue("LF11.buy.normaK3	_",s,null);
		this.dattribs_names.addValue("LF11.buy.k1_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF11.buy.k1_ptrades_po_var2_",s,null);
		this.dattribs_names.addValue("LF11.buy.k2_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF11.buy.k3_ptrades_po_",s,null);


		this.dattribs_names.addValue("LF11.sell.vbook_",s,null);
		this.dattribs_names.addValue("LF11.sell.tau_book_",s,null);
		this.dattribs_names.addValue("LF11.sell.p0_",s,null);
		this.dattribs_names.addValue("LF11.sell.p_stop_",s,null);
		this.dattribs_names.addValue("LF11.sell.pbook_average_",s,null);
		this.dattribs_names.addValue("LF11.sell.pbook_average_var2_",s,null);
		this.dattribs_names.addValue("LF11.sell.pbook_average_k2_",s,null);
		this.dattribs_names.addValue("LF11.sell.pbook_average_k3_",s,null);
		this.dattribs_names.addValue("LF11.sell.pbook_x_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF11.sell.normaK1_",s,null);
		this.dattribs_names.addValue("LF11.sell.normaK2_",s,null);
		this.dattribs_names.addValue("LF11.sell.normaK3	_",s,null);
		this.dattribs_names.addValue("LF11.sell.k1_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF11.sell.k1_ptrades_po_var2_",s,null);
		this.dattribs_names.addValue("LF11.sell.k2_ptrades_po_",s,null);
		this.dattribs_names.addValue("LF11.sell.k3_ptrades_po_",s,null);


		this.dattribs_names.addValue("LF11.bd.k1_",s,null);
		this.dattribs_names.addValue("LF11.bd.k1_var2_",s,null);
		this.dattribs_names.addValue("LF11.bd.k2_",s,null);
		this.dattribs_names.addValue("LF11.bd.k3_",s,null);
		this.dattribs_names.addValue("LF11.bd.fact_med_",s,null);

		this.dattribs_names.addValue("LF11.td.k1_",s,null);
		this.dattribs_names.addValue("LF11.td.k1_var2_",s,null);
		this.dattribs_names.addValue("LF11.td.k2_",s,null);
		this.dattribs_names.addValue("LF11.td.k3_",s,null);

		this.dattribs_names.addValue("LF11.td.k1_dp_",s,null);
		this.dattribs_names.addValue("LF11.td.k1_dp_var2_",s,null);
		this.dattribs_names.addValue("LF11.td.k2_dp_",s,null);
		this.dattribs_names.addValue("LF11.td.k3_dp_",s,null);
		this.dattribs_names.addValue("LF11.td.k1_dp_buy_",s,null);
		this.dattribs_names.addValue("LF11.td.k1_dp_buy_var2_",s,null);
		this.dattribs_names.addValue("LF11.td.k2_dp_buy_",s,null);
		this.dattribs_names.addValue("LF11.td.k3_dp_buy_",s,null);
		this.dattribs_names.addValue("LF11.td.k1_dp_sell_",s,null);
		this.dattribs_names.addValue("LF11.td.k1_dp_sell_var2_",s,null);
		this.dattribs_names.addValue("LF11.td.k2_dp_sell_",s,null);
		this.dattribs_names.addValue("LF11.td.k3_dp_sell_",s,null);
		this.dattribs_names.addValue("LF11.td.pt_x_dp_buy_",s,null);
		this.dattribs_names.addValue("LF11.td.pt_x_dp_sell_",s,null);
		this.dattribs_names.addValue("LF11.td.pt_aver_",s,null);
		this.dattribs_names.addValue("LF11.td.pt_aver_var2_",s,null);
	    }
	    else
	    {
		n+=16+16+5+4+4+8+4;
		final CalculateLiquidityDeficit11 l=c.topp.book_trades_liquiditydeficit11;
		final CalculateKendall.BookTradesBKStats sbuy=l.book_buy[j];
		this.dattribs.addValue(sbuy.vbook);
		this.dattribs.addValue(tt(sbuy.tau_book));
		this.dattribs.addValue(sbuy.p0);
		this.dattribs.addValue(sbuy.p_stop);
		this.dattribs.addValue(sbuy.pbook_average);
		this.dattribs.addValue(sbuy.pbook_average_var2);
		this.dattribs.addValue(sbuy.pbook_average_k2);
		this.dattribs.addValue(sbuy.pbook_average_k3);
		this.dattribs.addValue(sbuy.pbook_x_ptrades_po);
		this.dattribs.addValue(sbuy.normaK1);
		this.dattribs.addValue(sbuy.normaK2);
		this.dattribs.addValue(sbuy.normaK3);
		this.dattribs.addValue(sbuy.k1_ptrades_po);
		this.dattribs.addValue(sbuy.k1_ptrades_po_var2);
		this.dattribs.addValue(sbuy.k2_ptrades_po);
		this.dattribs.addValue(sbuy.k3_ptrades_po);


		final CalculateKendall.BookTradesBKStats ssell=l.book_sell[j];
		this.dattribs.addValue(ssell.vbook);
		this.dattribs.addValue(tt(ssell.tau_book));
		this.dattribs.addValue(ssell.p0);
		this.dattribs.addValue(ssell.p_stop);
		this.dattribs.addValue(ssell.pbook_average);
		this.dattribs.addValue(ssell.pbook_average_var2);
		this.dattribs.addValue(ssell.pbook_average_k2);
		this.dattribs.addValue(ssell.pbook_average_k3);
		this.dattribs.addValue(ssell.pbook_x_ptrades_po);
		this.dattribs.addValue(ssell.normaK1);
		this.dattribs.addValue(ssell.normaK2);
		this.dattribs.addValue(ssell.normaK3);
		this.dattribs.addValue(ssell.k1_ptrades_po);
		this.dattribs.addValue(ssell.k1_ptrades_po_var2);
		this.dattribs.addValue(ssell.k2_ptrades_po);
		this.dattribs.addValue(ssell.k3_ptrades_po);

		final CalculateKendall.BookKStats bd=l.bdata[j];
		this.dattribs.addValue(bd.k1);
		this.dattribs.addValue(bd.k1_var2);
		this.dattribs.addValue(bd.k2);
		this.dattribs.addValue(bd.k3);
		this.dattribs.addValue(bd.fact_med);

		final CalculateKendall.TradeKStats td=l.tdata[j];
		this.dattribs.addValue(td.k1);
		this.dattribs.addValue(td.k1_var2);
		this.dattribs.addValue(td.k2);
		this.dattribs.addValue(td.k3);
		this.dattribs.addValue(td.k1_dp);
		this.dattribs.addValue(td.k1_dp_var2);
		this.dattribs.addValue(td.k2_dp);
		this.dattribs.addValue(td.k3_dp);
		this.dattribs.addValue(td.k1_dp_buy);
		this.dattribs.addValue(td.k1_dp_buy_var2);
		this.dattribs.addValue(td.k2_dp_buy);
		this.dattribs.addValue(td.k3_dp_buy);
		this.dattribs.addValue(td.k1_dp_sell);
		this.dattribs.addValue(td.k1_dp_sell_var2);
		this.dattribs.addValue(td.k2_dp_sell);
		this.dattribs.addValue(td.k3_dp_sell);
		this.dattribs.addValue(td.pt_x_dp_buy);
		this.dattribs.addValue(td.pt_x_dp_sell);
		this.dattribs.addValue(td.pt_aver);
		this.dattribs.addValue(td.pt_aver_var2);
	    }
	}
	return n;
    }




   private int setAttrLiquidity10(final boolean flag_processheader,
				  final String ticker,
				  final long [] delays,
				  final CalculationData c)
    {
	final int FM=CalculateKendall.FM_ORDER;
	int n=0;

	for(int j=0;j<delays.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=String.valueOf(delays[j]/ModelParameters.DISTANCE_TIME_SCALE);
		n+=28+9+3+3+9+4+4;

		this.dattribs_names.addValue("LF10.td.k1_",s,null);
		this.dattribs_names.addValue("LF10.td.k1_var2_",s,null);
		this.dattribs_names.addValue("LF10.td.k2_",s,null);
		this.dattribs_names.addValue("LF10.td.k3_",s,null);
		this.dattribs_names.addValue("LF10.td.k1_dp_",s,null);
		this.dattribs_names.addValue("LF10.td.k1_dp_var2_",s,null);
		this.dattribs_names.addValue("LF10.td.k2_dp_",s,null);
		this.dattribs_names.addValue("LF10.td.k3_dp_",s,null);
		this.dattribs_names.addValue("LF10.td.k1_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.td.k1_dp_buy_var2_",s,null);
		this.dattribs_names.addValue("LF10.td.k2_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.td.k3_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.td.k3d2_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.td.k3d1_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.td.k1_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.td.k1_dp_sell_var2_",s,null);
		this.dattribs_names.addValue("LF10.td.k2_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.td.k3_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.td.k3d2_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.td.k3d1_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.td.k1_dpa_",s,null);
		this.dattribs_names.addValue("LF10.td.k1_dpa_var2_",s,null);
		this.dattribs_names.addValue("LF10.td.k2_dpa_",s,null);
		this.dattribs_names.addValue("LF10.td.k3_dpa_",s,null);
		this.dattribs_names.addValue("LF10.td.normaCoupled_",s,null);
		this.dattribs_names.addValue("LF10.td.dpt_coupled_",s,null);
		this.dattribs_names.addValue("LF10.td.pt_x_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.td.pt_x_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.td.pt_aver_",s,null);
		this.dattribs_names.addValue("LF10.td.pt_aver_var2_",s,null);
		this.dattribs_names.addValue("LF10.td.pt_aver_k2_",s,null);
		this.dattribs_names.addValue("LF10.td.pt_aver_k3_",s,null);
		this.dattribs_names.addValue("LF10.td.pt_aver_k3d2_",s,null);
		this.dattribs_names.addValue("LF10.td.pt_aver_k3d1_",s,null);
		this.dattribs_names.addValue("LF10.td.vt_buy_",s,null);
		this.dattribs_names.addValue("LF10.td.vt_sell_",s,null);

		this.dattribs_names.addValue("LF10.td.lk1_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.td.lk2_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.td.lk3_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.td.lk1_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.td.lk2_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.td.lk3_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.td.pt_lk1_",s,null);
		this.dattribs_names.addValue("LF10.td.pt_lk2_",s,null);
		this.dattribs_names.addValue("LF10.td.pt_lk3_",s,null);

		this.dattribs_names.addValue("LF10.td.k1v_",s,null);
		this.dattribs_names.addValue("LF10.td.k2v_",s,null);
		this.dattribs_names.addValue("LF10.td.k3v_",s,null);
		this.dattribs_names.addValue("LF10.td.k1tau_",s,null);
		this.dattribs_names.addValue("LF10.td.k2tau_",s,null);
		this.dattribs_names.addValue("LF10.td.k3tau_",s,null);
		this.dattribs_names.addValue("LF10.td.k3d2tau_",s,null);
		this.dattribs_names.addValue("LF10.td.k3d1tau_",s,null);


		this.dattribs_names.addValue("LF10.td.tau_",s,null);

		this.dattribs_names.addValue("LF10.td.pmin_",s,null);
		this.dattribs_names.addValue("LF10.td.paver_after_min_",s,null);
		this.dattribs_names.addValue("LF10.td.vafter_min_",s,null);
		this.dattribs_names.addValue("LF10.td.pmax_",s,null);
		this.dattribs_names.addValue("LF10.td.paver_after_max_",s,null);
		this.dattribs_names.addValue("LF10.td.vafter_max_",s,null);



		n+=24;
		this.dattribs_names.addValue("LF10.kt.k1_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.kt.k1_dp_buy_var2_",s,null);
		this.dattribs_names.addValue("LF10.kt.k2_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.kt.k3_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.kt.k1_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.kt.k1_dp_sell_var2_",s,null);
		this.dattribs_names.addValue("LF10.kt.k2_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.kt.k3_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.kt.pt_x_dp_buy_",s,null);
		this.dattribs_names.addValue("LF10.kt.pt_x_dp_sell_",s,null);
		this.dattribs_names.addValue("LF10.kt.pt_aver_",s,null);
		this.dattribs_names.addValue("LF10.kt.pt_aver_var2_",s,null);
		this.dattribs_names.addValue("LF10.kt.pt_aver_k2_",s,null);
		this.dattribs_names.addValue("LF10.kt.pt_aver_k3_",s,null);
		this.dattribs_names.addValue("LF10.kt.vt_buy_",s,null);
		this.dattribs_names.addValue("LF10.kt.vt_sell_",s,null);
		this.dattribs_names.addValue("LF10.kt.pmin_",s,null);
		this.dattribs_names.addValue("LF10.kt.paver_after_min_",s,null);
		this.dattribs_names.addValue("LF10.kt.vafter_min_",s,null);
		this.dattribs_names.addValue("LF10.kt.tau_pmin_",s,null);
		this.dattribs_names.addValue("LF10.kt.pmax_",s,null);
		this.dattribs_names.addValue("LF10.kt.paver_after_max_",s,null);
		this.dattribs_names.addValue("LF10.kt.vafter_max_",s,null);
		this.dattribs_names.addValue("LF10.kt.tau_pmax_",s,null);



		n+=18+4;
		this.dattribs_names.addValue("LF10.fm.tau_",s,null);
		this.dattribs_names.addValue("LF10.fm.vi_",s,null);
		this.dattribs_names.addValue("LF10.fm.p_last_",s,null);
		this.dattribs_names.addValue("LF10.fm.pt_aver_",s,null);
		this.dattribs_names.addValue("LF10.fm.pt_aver_var2_",s,null);
		this.dattribs_names.addValue("LF10.fm.d1m_",s,null);
		this.dattribs_names.addValue("LF10.fm.d1e_",s,null);
		this.dattribs_names.addValue("LF10.fm.d1es_",s,null);
		this.dattribs_names.addValue("LF10.fm.d1f_",s,null);
		this.dattribs_names.addValue("LF10.fm.d2m_",s,null);
		this.dattribs_names.addValue("LF10.fm.d2e_",s,null);
		this.dattribs_names.addValue("LF10.fm.d2es_",s,null);
		this.dattribs_names.addValue("LF10.fm.d2f_",s,null);
		this.dattribs_names.addValue("LF10.fm.d3m_",s,null);
		this.dattribs_names.addValue("LF10.fm.d3e_",s,null);
		this.dattribs_names.addValue("LF10.fm.d3es_",s,null);
		this.dattribs_names.addValue("LF10.fm.d3f_",s,null);
		this.dattribs_names.addValue("LF10.fm.f0_",s,null);
		this.dattribs_names.addValue("LF10.fm.d1v_e_",s,null);
		this.dattribs_names.addValue("LF10.fm.d2v_e_",s,null);
		this.dattribs_names.addValue("LF10.fm.d3v_e_",s,null);
		this.sattribs_names.addValue("LF10.fm.t_vol_median_",s,null);
		for(int jf=1;jf<=FM;jf++)
		{
		    final String fn="LF10.fm"+jf+".f";
		    for(int jjf=1;jjf<=jf;jjf++)
		    {
			n++;
			this.dattribs_names.addValue(fn+jjf+"_",s,null);
		    }
		}

                /*
		n+=10;
		this.dattribs_names.addValue("LF10.tc.k1c_buy_",s,null);
		this.dattribs_names.addValue("LF10.tc.k2c_buy_",s,null);
		this.dattribs_names.addValue("LF10.tc.n1_buy_",s,null);
		this.dattribs_names.addValue("LF10.tc.n2_buy_",s,null);
		this.dattribs_names.addValue("LF10.tc.k1c_sell_",s,null);
		this.dattribs_names.addValue("LF10.tc.k2c_sell_",s,null);
		this.dattribs_names.addValue("LF10.tc.n1_sell_",s,null);
		this.dattribs_names.addValue("LF10.tc.n2_sell_",s,null);
		this.dattribs_names.addValue("LF10.tc.k1c_",s,null);
		this.dattribs_names.addValue("LF10.tc.k2c_",s,null);
		*/


		n+=12;
		this.dattribs_names.addValue("LF10.tds.tau_",s,null);
		this.dattribs_names.addValue("LF10.tds.vi_",s,null);
		this.dattribs_names.addValue("LF10.tds.p_last_",s,null);
		this.dattribs_names.addValue("LF10.tds.pt_aver_",s,null);
		this.dattribs_names.addValue("LF10.tds.pt_aver_var2_",s,null);
		this.dattribs_names.addValue("LF10.tds.d1pbe_",s,null);
		this.dattribs_names.addValue("LF10.tds.d1pb_",s,null);
		this.dattribs_names.addValue("LF10.tds.d2pb_",s,null);
		this.dattribs_names.addValue("LF10.tds.d3pb_",s,null);
		this.dattribs_names.addValue("LF10.tds.dps_sum_",s,null);
		this.dattribs_names.addValue("LF10.tds.dpsa_",s,null);
		this.dattribs_names.addValue("LF10.tds.dpsdv_",s,null);

	    }
	    else
	    {
		final CalculateLiquidityDeficit10 l=c.topp.book_trades_liquiditydeficit10;
		n+=28+9+3+3+9+4+4;

		final CalculateKendall.TradeKStats td=l.tdata[j];
		this.dattribs.addValue(td.k1);
		this.dattribs.addValue(td.k1_var2);
		this.dattribs.addValue(td.k2);
		this.dattribs.addValue(td.k3);
		this.dattribs.addValue(td.k1_dp);
		this.dattribs.addValue(td.k1_dp_var2);
		this.dattribs.addValue(td.k2_dp);
		this.dattribs.addValue(td.k3_dp);
		this.dattribs.addValue(td.k1_dp_buy);
		this.dattribs.addValue(td.k1_dp_buy_var2);
		this.dattribs.addValue(td.k2_dp_buy);
		this.dattribs.addValue(td.k3_dp_buy);
		this.dattribs.addValue(td.k3d2_dp_buy);
		this.dattribs.addValue(td.k3d1_dp_buy);
		this.dattribs.addValue(td.k1_dp_sell);
		this.dattribs.addValue(td.k1_dp_sell_var2);
		this.dattribs.addValue(td.k2_dp_sell);
		this.dattribs.addValue(td.k3_dp_sell);
		this.dattribs.addValue(td.k3d2_dp_sell);
		this.dattribs.addValue(td.k3d1_dp_sell);
		this.dattribs.addValue(td.k1_dpa);
		this.dattribs.addValue(td.k1_dpa_var2);
		this.dattribs.addValue(td.k2_dpa);
		this.dattribs.addValue(td.k3_dpa);
		this.dattribs.addValue(td.normaCoupled);
		this.dattribs.addValue(td.dpt_coupled);
		this.dattribs.addValue(td.pt_x_dp_buy);
		this.dattribs.addValue(td.pt_x_dp_sell);
		this.dattribs.addValue(td.pt_aver);
		this.dattribs.addValue(td.pt_aver_var2);
		this.dattribs.addValue(td.pt_aver_k2);
		this.dattribs.addValue(td.pt_aver_k3);
		this.dattribs.addValue(td.pt_aver_k3d2);
		this.dattribs.addValue(td.pt_aver_k3d1);
		this.dattribs.addValue(td.vt_buy);
		this.dattribs.addValue(td.vt_sell);

		this.dattribs.addValue(td.lk1_dp_buy);
		this.dattribs.addValue(td.lk2_dp_buy);
		this.dattribs.addValue(td.lk3_dp_buy);
		this.dattribs.addValue(td.lk1_dp_sell);
		this.dattribs.addValue(td.lk2_dp_sell);
		this.dattribs.addValue(td.lk3_dp_sell);
		this.dattribs.addValue(td.pt_lk1);
		this.dattribs.addValue(td.pt_lk2);
		this.dattribs.addValue(td.pt_lk3);

		this.dattribs.addValue(td.k1v);
		this.dattribs.addValue(td.k2v);
		this.dattribs.addValue(td.k3v);
		this.dattribs.addValue(td.k1tau);
		this.dattribs.addValue(td.k2tau);
		this.dattribs.addValue(td.k3tau);
		this.dattribs.addValue(td.k3d2tau);
		this.dattribs.addValue(td.k3d1tau);

		this.dattribs.addValue(tdiff(td.currenttime,td.t_range_cut));

		this.dattribs.addValue(td.pmin);
		this.dattribs.addValue(td.paver_after_min);
		this.dattribs.addValue(td.vafter_min);
		this.dattribs.addValue(td.pmax);
		this.dattribs.addValue(td.paver_after_max);
		this.dattribs.addValue(td.vafter_max);

		n+=24;
		final CalculateKendall.TradeKT2Stats kt=l.ktdata[j];
		this.dattribs.addValue(kt.k1_dp_buy);
		this.dattribs.addValue(kt.k1_dp_buy_var2);
		this.dattribs.addValue(kt.k2_dp_buy);
		this.dattribs.addValue(kt.k3_dp_buy);
		this.dattribs.addValue(kt.k1_dp_sell);
		this.dattribs.addValue(kt.k1_dp_sell_var2);
		this.dattribs.addValue(kt.k2_dp_sell);
		this.dattribs.addValue(kt.k3_dp_sell);
		this.dattribs.addValue(kt.pt_x_dp_buy);
		this.dattribs.addValue(kt.pt_x_dp_sell);
		this.dattribs.addValue(kt.pt_aver);
		this.dattribs.addValue(kt.pt_aver_var2);
		this.dattribs.addValue(kt.pt_aver_k2);
		this.dattribs.addValue(kt.pt_aver_k3);
		this.dattribs.addValue(kt.vt_buy);
		this.dattribs.addValue(kt.vt_sell);
		this.dattribs.addValue(kt.pmin);
		this.dattribs.addValue(kt.paver_after_min);
		this.dattribs.addValue(kt.vafter_min);
		this.dattribs.addValue(tdiff(kt.currenttime,kt.t_pmin));
		this.dattribs.addValue(kt.pmax);
		this.dattribs.addValue(kt.paver_after_max);
		this.dattribs.addValue(kt.vafter_max);
		this.dattribs.addValue(tdiff(kt.currenttime,kt.t_pmax));


		n+=18+4;
		final CalculateKendall.TradeDStats fm=l.tfmdata[j];
		this.dattribs.addValue(tdiff(fm.currenttime,fm.t_range_cut));
		this.dattribs.addValue(fm.vi);
		this.dattribs.addValue(fm.p_last);
		this.dattribs.addValue(fm.pt_aver);
		this.dattribs.addValue(fm.pt_aver_var2);
		this.dattribs.addValue(fm.d1m);
		this.dattribs.addValue(fm.d1e);
		this.dattribs.addValue(fm.d1es);
		this.dattribs.addValue(fm.d1f);
		this.dattribs.addValue(fm.d2m);
		this.dattribs.addValue(fm.d2e);
		this.dattribs.addValue(fm.d2es);
		this.dattribs.addValue(fm.d2f);
		this.dattribs.addValue(fm.d3m);
		this.dattribs.addValue(fm.d3e);
		this.dattribs.addValue(fm.d3es);
		this.dattribs.addValue(fm.d3f);
		this.dattribs.addValue(fm.f0);
		this.dattribs.addValue(fm.d1v_e);
		this.dattribs.addValue(fm.d2v_e);
		this.dattribs.addValue(fm.d3v_e);
		this.sattribs.addValue(String.valueOf(l.t_vol_median[j]));
		for(int jf=1;jf<=FM;jf++)
		{
		    for(int jjf=1;jjf<=jf;jjf++)
		    {
			n++;
			this.dattribs.addValue(fm.all_fmoments[jf][jjf]);
		    }
		}

		/*
		n+=10;
		final CalculateKendall.TradeCoupledStats tc=l.tcdata[j];
		this.dattribs.addValue(tc.k1c_buy);
		this.dattribs.addValue(tc.k2c_buy);
		this.dattribs.addValue(tc.n1_buy);
		this.dattribs.addValue(tc.n2_buy);
		this.dattribs.addValue(tc.k1c_sell);
		this.dattribs.addValue(tc.k2c_sell);
		this.dattribs.addValue(tc.n1_sell);
		this.dattribs.addValue(tc.n2_sell);
		this.dattribs.addValue(tc.k1c);
		this.dattribs.addValue(tc.k2c);
		*/

		n+=12;
		final CalculateKendall.TradeDSStats tds=l.tdsdata[j];
		this.dattribs.addValue(tdiff(tds.currenttime,tds.t_range_cut));
		this.dattribs.addValue(tds.vi);
		this.dattribs.addValue(tds.p_last);
		this.dattribs.addValue(tds.pt_aver);
		this.dattribs.addValue(tds.pt_aver_var2);
		this.dattribs.addValue(tds.d1pbe);
		this.dattribs.addValue(tds.d1pb);
		this.dattribs.addValue(tds.d2pb);
		this.dattribs.addValue(tds.d3pb);
		this.dattribs.addValue(tds.dps_sum);
		this.dattribs.addValue(tds.dpsa);
		this.dattribs.addValue(tds.dpsdv);
	    }
	}
	return n;
    }





   private int setAttrLiquidity9(final boolean flag_processheader,
				  final String ticker,
				  final String [] levels_names,
				  final CalculationData c)
    {
	int n=0;
	for(int j=0;j<levels_names.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=levels_names[j];
		n+=121;
		this.dattribs_names.addValue("LF9.pbook_buy_",s,null);
		this.dattribs_names.addValue("LF9.pbook_sell_",s,null);
		this.nattribs_names.addValue("LF9.n_levels_buy_",s,null);
		this.nattribs_names.addValue("LF9.n_levels_sell_",s,null);

		this.dattribs_names.addValue("LF9.trades_krank_forward_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_forward_p_getValA_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_forward_pd_getValA_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_forward_p_buy_getValA_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_forward_p_sell_getValA_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_forward_p_underestimate_getValA_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_forward_p_overestimate_getValA_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_forward_p_end_getValA_",s,null);

		this.dattribs_names.addValue("LF9.trades_krank_VS_ttype_dir_average_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_ttype_dir_average2_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_ttype_dir_average2f_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_dttype_dir_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_tau_average_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_tau_average2_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_tau_average2f_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_dtau_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_dp_average_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_dp_average2_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_dp_average2f_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_ddp_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_dpv_average_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_dpv_average2_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_dpv_average2f_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_VS_ddpv_",s,null);

		this.dattribs_names.addValue("LF9.trades_ko_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_ko_getRankA_",s,null);
		this.dattribs_names.addValue("LF9.trades_ko_orig_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_ko_orig_getRankA_",s,null);

		this.dattribs_names.addValue("LF9.trades_p_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_p_getRankA_",s,null);
		this.dattribs_names.addValue("LF9.trades_p_getRankAStdev_",s,null);
		this.dattribs_names.addValue("LF9.trades_p_getValA_",s,null);
		this.dattribs_names.addValue("LF9.trades_p_getValAA_",s,null);
		this.dattribs_names.addValue("LF9.trades_p_getValStdev_",s,null);
		this.dattribs_names.addValue("LF9.trades_p_getTauRel_",s,null);
		this.dattribs_names.addValue("LF9.trades_p_getTau_",s,null);
		this.dattribs_names.addValue("LF9.trades_p.p0_direction_",s,null);
		this.dattribs_names.addValue("LF9.trades_time._getRank_",s,null);

		this.dattribs_names.addValue("LF9.trades_trades_Dcoupled_getRank_",s,null);
		this.dattribs_names.addValue("LF9.book_trades_Dcoupled_getRank_",s,null);
		this.dattribs_names.addValue("LF9.book_trades_buy_dir_Dcoupled_getRank_",s,null);
		this.dattribs_names.addValue("LF9.book_trades_sell_dir_Dcoupled_getRank_",s,null);

		this.dattribs_names.addValue("LF9.trades_D2sum_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_0krank_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_0krank_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_0krank_sell_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_0krank_dir_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_dkrank_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_dkrank_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_dkrank_sell_getRank_",s,null);

		this.dattribs_names.addValue("LF9.trades_book_pkrank_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_book_pkrank_sell_getRank_",s,null);

		this.dattribs_names.addValue("LF9.trades0_dir_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades3_dir_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades3_ddir_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_dbs_getRank_",s,null);

		this.dattribs_names.addValue("LF9.book_side_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF9.book_side_sell_getRank_",s,null);

		this.dattribs_names.addValue("LF9.trades_D3_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_D2_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_D2_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_D2_sell_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_Dcoupled_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_Dcoupled_sell_getRank_",s,null);

		this.dattribs_names.addValue("LF9.ntrades0_dir_getRank_",s,null);
		this.dattribs_names.addValue("LF9.ntrades2_dir_getRank_",s,null);
		this.dattribs_names.addValue("LF9.ntrades3_dir_getRank_",s,null);

		this.dattribs_names.addValue("LF9.trades_krank_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_sell_getRank_",s,null);

		this.dattribs_names.addValue("LF9.trades_krank_bs_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_tau_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_cd_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_XY_getRank_",s,null);
		this.dattribs_names.addValue("LF9.trades_krank_self_getRank_",s,null);
		this.dattribs_names.addValue("LF9.book_krank_XY_getRank_",s,null);
		this.dattribs_names.addValue("LF9.book_krank_cumm_getRank_",s,null);
		this.dattribs_names.addValue("LF9.tmed_getTauMedRel_",s,null);
		this.dattribs_names.addValue("LF9.tmed_getTauMedRelS_",s,null);
		this.dattribs_names.addValue("LF9.tmed_getVRel_",s,null);
		this.dattribs_names.addValue("LF9.TD_",s,null);
		this.dattribs_names.addValue("LF9.getTauR_Buy_",s,null);
		this.dattribs_names.addValue("LF9.getTauR_Sell_",s,null);
		this.dattribs_names.addValue("LF9.getTauRTotal_Buy_",s,null);
		this.dattribs_names.addValue("LF9.getTauRTotal_Sell_",s,null);

		this.dattribs_names.addValue("LF9.BOOK_getTauR_buy_",s,null);
		this.dattribs_names.addValue("LF9.BOOK_getTauR_sell_",s,null);

		this.dattribs_names.addValue("LF9.BOOK_getTauTotal_buy_",s,null);
		this.dattribs_names.addValue("LF9.BOOK_getTauTotal_sell_",s,null);

		this.dattribs_names.addValue("LF9.BOOK_getTauDistToTrades_buy_",s,null);
		this.dattribs_names.addValue("LF9.BOOK_getTauDistToTrades_sell_",s,null);

		this.dattribs_names.addValue("LF9.BOOK_getTauDistToTradesMAX_buy_",s,null);
		this.dattribs_names.addValue("LF9.BOOK_getTauDistToTradesMAX_sell_",s,null);
		this.dattribs_names.addValue("LF9.BOOK_getTauDistToTradesRelTotal_buy_",s,null);
		this.dattribs_names.addValue("LF9.BOOK_getTauDistToTradesRelTotal_sell_",s,null);

		this.dattribs_names.addValue("LF9.BOOK_buy_getTauTrades_Buy_",s,null);
		this.dattribs_names.addValue("LF9.BOOK_buy_getTauTrades_Sell_",s,null);
		this.dattribs_names.addValue("LF9.BOOK_sell_getTauTrades_Buy_",s,null);
		this.dattribs_names.addValue("LF9.BOOK_sell_getTauTrades_Sell_",s,null);

		this.dattribs_names.addValue("LF9.BOOKRelT_buy_getTauTrades_Buy_",s,null);
		this.dattribs_names.addValue("LF9.BOOKRelT_buy_getTauTrades_Sell_",s,null);
		this.dattribs_names.addValue("LF9.BOOKRelT_sell_getTauTrades_Buy_",s,null);
		this.dattribs_names.addValue("LF9.BOOKRelT_sell_getTauTrades_Sell_",s,null);

		this.dattribs_names.addValue("LF9.getETauInDiff_",s,null);
		this.dattribs_names.addValue("LF9.getEDITauInDiff_",s,null);
		this.dattribs_names.addValue("LF9.getESDITauInDiff_",s,null);
		this.dattribs_names.addValue("LF9.getTauTDiff_",s,null);
		this.dattribs_names.addValue("LF9.getTauInTESDiff_",s,null);
		this.dattribs_names.addValue("LF9.getEVInDiff_",s,null);
		this.dattribs_names.addValue("LF9.getAESDITauInDiff_",s,null);

		this.dattribs_names.addValue("LF9.getTauR_Diff_buy_",s,null);
		this.dattribs_names.addValue("LF9.getTauR_Diff_sell_",s,null);
		this.dattribs_names.addValue("LF9.getTauREDiff_",s,null);
		this.dattribs_names.addValue("LF9.getTauRDiff_",s,null);
		this.dattribs_names.addValue("LF9.BOOK_getTauTotalDiff_",s,null);
		this.dattribs_names.addValue("LF9.paver_",s,null);
		this.dattribs_names.addValue("LF9.vfrac_",s,null);
	    }
	    else
	    {
		final CalculateLiquidityDeficit9 l=c.topp.book_trades_liquiditydeficit9;
		n+=121;
		this.dattribs.addValue(Math.log(l.price_book_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.price_book_sell[j]/c.p_sell));
		this.nattribs.addValue(l.n_levels_buy[j]);
		this.nattribs.addValue(l.n_levels_sell[j]);

		final CalculateKendall.MSAverageKendall kf=l.trades_krank_forward[j];
		this.dattribs.addValue(kf.trades_krank_forward.getRank());
		this.dattribs.addValue(kf.trades_krank_forward.getValA());
		this.dattribs.addValue(kf.dp_delayed_average);
		this.dattribs.addValue(kf.dp_buy);
		this.dattribs.addValue(kf.dp_sell);
		this.dattribs.addValue(kf.dp_underestimate);
		this.dattribs.addValue(kf.dp_overestimate);
		this.dattribs.addValue(kf.dp_end);

		final CalculateKendall.DAvergeCalculateKendall kw=l.trades_krank_VS[j];
		this.dattribs.addValue(kw.ttype_dir_average);
		this.dattribs.addValue(kw.ttype_dir_average2);
		this.dattribs.addValue(kw.ttype_dir_average2f);
		this.dattribs.addValue(kw.dttype_dir);
		this.dattribs.addValue(kw.tau_average);
		this.dattribs.addValue(kw.tau_average2);
		this.dattribs.addValue(kw.tau_average2f);
		this.dattribs.addValue(kw.dtau);
		this.dattribs.addValue(kw.dp_average);
		this.dattribs.addValue(kw.dp_average2);
		this.dattribs.addValue(kw.dp_average2f);
		this.dattribs.addValue(kw.ddp);
		this.dattribs.addValue(kw.dpv_average);
		this.dattribs.addValue(kw.dpv_average2);
		this.dattribs.addValue(kw.dpv_average2f);
		this.dattribs.addValue(kw.ddpv);

		this.dattribs.addValue(l.trades_ko[j].getRank());
		this.dattribs.addValue(l.trades_ko[j].getRankA());
		this.dattribs.addValue(l.trades_ko_orig[j].getRank());
		this.dattribs.addValue(l.trades_ko_orig[j].getRankA());

		final CalculateKendall.KendallMStats tpj=l.trades_p[j];
		this.dattribs.addValue(tpj.trades_price.getRank());
		this.dattribs.addValue(tpj.trades_price.getRankA());
		this.dattribs.addValue(tpj.trades_price.getRankAStdev());
		this.dattribs.addValue(tpj.trades_price.getValA());
		this.dattribs.addValue(tpj.trades_price.getValAA());
		this.dattribs.addValue(tpj.trades_price.getValAStdev());
		this.dattribs.addValue(tpj.getTauRel());
		this.dattribs.addValue(tpj.trades_time.getValA());
		this.dattribs.addValue(tpj.p0_direction);
		this.dattribs.addValue(tpj.trades_time.getRank());

		this.dattribs.addValue(l.trades_trades_Dcoupled[j].krank_coupling.getRank());
		this.dattribs.addValue(l.book_trades_Dcoupled[j].getRank());
		this.dattribs.addValue(l.trades_book_buy_dir[j].krank_coupling.getRank());
		this.dattribs.addValue(l.trades_book_sell_dir[j].krank_coupling.getRank());

		this.dattribs.addValue(l.trades_D2sum[j].getRank());
		this.dattribs.addValue(l.trades_0krank[j].getRank());
		this.dattribs.addValue(l.trades_0krank_buy[j].getRank());
		this.dattribs.addValue(l.trades_0krank_sell[j].getRank());
		this.dattribs.addValue(l.trades_0krank_dir[j].getRank());
		this.dattribs.addValue(l.trades_dkrank[j].getRank());
		this.dattribs.addValue(l.trades_dkrank_buy[j].getRank());
		this.dattribs.addValue(l.trades_dkrank_sell[j].getRank());

		this.dattribs.addValue(l.trades_book_pkrank_buy[j].getRank());
		this.dattribs.addValue(l.trades_book_pkrank_sell[j].getRank());

		this.dattribs.addValue(l.trades0_dir[j].getRank());
		this.dattribs.addValue(l.trades3_dir[j].getRank());
		this.dattribs.addValue(l.trades3_ddir[j].getRank());
		this.dattribs.addValue(l.trades_dbs[j].getRank());

		this.dattribs.addValue(l.book_side_buy[j].getRank());
		this.dattribs.addValue(l.book_side_sell[j].getRank());

		this.dattribs.addValue(l.trades_D3[j].getRank());
		this.dattribs.addValue(l.trades_D2[j].getRank());
		this.dattribs.addValue(l.trades_D2_buy[j].getRank());
		this.dattribs.addValue(l.trades_D2_sell[j].getRank());
		this.dattribs.addValue(l.trades_Dcoupled_buy[j].getRank());
		this.dattribs.addValue(l.trades_Dcoupled_sell[j].getRank());

		this.dattribs.addValue(l.ntrades0_dir[j].getRank());
		this.dattribs.addValue(l.ntrades2_dir[j].getRank());
		this.dattribs.addValue(l.ntrades3_dir[j].getRank());

		this.dattribs.addValue(l.trades_krank[j].getRank());
		this.dattribs.addValue(l.trades_krank_buy[j].getRank());
		this.dattribs.addValue(l.trades_krank_sell[j].getRank());

		this.dattribs.addValue(l.trades_krank_bs[j].getRank());
		this.dattribs.addValue(l.trades_krank_tau[j].getRank());
		this.dattribs.addValue(l.trades_krank_cd[j].getRank());
		this.dattribs.addValue(l.trades_krank_XY[j].getRank());
		this.dattribs.addValue(l.trades_krank_self[j].getRank());
		this.dattribs.addValue(l.book_krank_XY[j].getRank());
		this.dattribs.addValue(l.book_krank_cumm[j].getRank());
		this.dattribs.addValue(l.tmed[j].getTauMedRel());
		this.dattribs.addValue(l.tmed[j].getTauMedRelS());
		this.dattribs.addValue(l.tmed[j].getVRel());
		final double td=l.tdiff[j].t_range_cut<0?Double.NaN:c.getTime()-l.tdiff[j].t_range_cut;
		this.dattribs.addValue(td);
		this.dattribs.addValue(l.tdiff[j].getTauR_Buy()/td);
		this.dattribs.addValue(l.tdiff[j].getTauR_Sell()/td);
		this.dattribs.addValue(l.tdiff[j].getTauRTotal_Buy()/td);
		this.dattribs.addValue(l.tdiff[j].getTauRTotal_Sell()/td);

		this.dattribs.addValue(l.book_buy[j].getTauR()/td);
		this.dattribs.addValue(l.book_sell[j].getTauR()/td);

		this.dattribs.addValue(l.book_buy[j].getTauTotal()/td);
		this.dattribs.addValue(l.book_sell[j].getTauTotal()/td);

		this.dattribs.addValue(l.book_buy[j].getTauDistToTrades()/td);
		this.dattribs.addValue(l.book_sell[j].getTauDistToTrades()/td);

		this.dattribs.addValue(l.book_buy[j].getTauDistToTradesMAX()/td);
		this.dattribs.addValue(l.book_sell[j].getTauDistToTradesMAX()/td);
		this.dattribs.addValue(
		    (l.book_buy[j].getTauTotal()<0?Double.NaN:l.book_buy[j].getTauTotal())/
		    (l.book_buy[j].getTauDistToTrades()<0?Double.NaN:l.book_buy[j].getTauDistToTrades())
				       );
		this.dattribs.addValue(
		    (l.book_sell[j].getTauTotal()<0?Double.NaN:l.book_sell[j].getTauTotal())/
		    (l.book_sell[j].getTauDistToTrades()<0?Double.NaN:l.book_sell[j].getTauDistToTrades())
				       );
		this.dattribs.addValue(l.book_buy[j].getTauTrades_Buy()/td);
		this.dattribs.addValue(l.book_buy[j].getTauTrades_Sell()/td);
		this.dattribs.addValue(l.book_sell[j].getTauTrades_Buy()/td);
		this.dattribs.addValue(l.book_sell[j].getTauTrades_Sell()/td);

		final double ttr_buy=l.book_buy[j].getTauDistToTrades()<0?Double.NaN:l.book_buy[j].getTauDistToTrades();
		final double ttr_sell=l.book_sell[j].getTauDistToTrades()<0?Double.NaN:l.book_sell[j].getTauDistToTrades();
		this.dattribs.addValue(l.book_buy[j].getTauTrades_Buy()/ttr_buy);
		this.dattribs.addValue(l.book_buy[j].getTauTrades_Sell()/ttr_buy);
		this.dattribs.addValue(l.book_sell[j].getTauTrades_Buy()/ttr_sell);
		this.dattribs.addValue(l.book_sell[j].getTauTrades_Sell()/ttr_sell);

		this.dattribs.addValue(l.tdiff[j].getETauInDiff());
		this.dattribs.addValue(l.tdiff[j].getEDITauInDiff());
		this.dattribs.addValue(l.tdiff[j].getESDITauInDiff());
		this.dattribs.addValue(l.tdiff[j].getTauTDiff());
		this.dattribs.addValue(l.tdiff[j].getTauInTESDiff());
		this.dattribs.addValue(l.tdiff[j].getEVInDiff());
		this.dattribs.addValue(l.tdiff[j].getAESDITauInDiff());

		this.dattribs.addValue(l.book_buy[j].getTauR_Diff(l.tdiff[j]));
		this.dattribs.addValue(l.book_sell[j].getTauR_Diff(l.tdiff[j]));
		this.dattribs.addValue(l.book_buy[j].getTauREDiff(l.book_sell[j]));
		this.dattribs.addValue(l.tdiff[j].getTauRDiff());
		this.dattribs.addValue(l.book_buy[j].getTauTotalDiff(l.book_sell[j]));
		this.dattribs.addValue(((0.5*(c.p_buy+c.p_sell))/l.tdiff[j].paver)-1);
		final long vt=l.tdiff[j].getV();
		this.dattribs.addValue(vt>0?((double)(l.book_buy[j].getV()+l.book_sell[j].getV()))/vt:Double.NaN);
	    }
	}
	return n;
    }



   private int setAttrLiquidity9T(final boolean flag_processheader,
				  final String ticker,
				  final long [] delays,
				  final CalculationData c)
    {
	final int nt=delays.length;
	int n=0;
	for(int j=0;j<nt;j++)
	{
	    if(flag_processheader)
	    {
		final String s=String.valueOf(delays[j]/ModelParameters.DISTANCE_TIME_SCALE);
		n+=114;

		this.dattribs_names.addValue("LF9T.trades_tau_krank1_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_tau_krank2_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_tau_krank2taubuy_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_tau_krank2tausell_getRank_",s,null);

		this.dattribs_names.addValue("LF9T.trades_krank2_o_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2_o_getRankA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2_o_paver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2_o_delayed_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2_o_delayed_getRankA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2_o_delayed_halfT_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2_o_delayed_halfT_getRankA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2_o_doublecouple_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2_o_doublecouple_getRankA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_dinterval_getRank_",s,null);

		this.dattribs_names.addValue("LF9T.trades_krank1_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2taubuy_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2tausell_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank2cd_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank3_getRank_",s,null);

		this.dattribs_names.addValue("LF9T.trades_krank_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_sell_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_bs_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_tau_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_cd_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_cd_delayed_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_XY_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_self_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.tmed_getTauMedRel_",s,null);
		this.dattribs_names.addValue("LF9T.tmed_getTauMedRelS_",s,null);
		this.dattribs_names.addValue("LF9T.tmed_getVRel_",s,null);
		this.dattribs_names.addValue("LF9T.TD_",s,null);
		this.dattribs_names.addValue("LF9T.getTauR_Buy_",s,null);
		this.dattribs_names.addValue("LF9T.getTauR_Sell_",s,null);
		this.dattribs_names.addValue("LF9T.getTauRTotal_Buy_",s,null);
		this.dattribs_names.addValue("LF9T.getTauRTotal_Sell_",s,null);

		this.dattribs_names.addValue("LF9T.getETauInDiff_",s,null);
		this.dattribs_names.addValue("LF9T.getEDITauInDiff_",s,null);
		this.dattribs_names.addValue("LF9T.getESDITauInDiff_",s,null);
		this.dattribs_names.addValue("LF9T.getTauTDiff_",s,null);
		this.dattribs_names.addValue("LF9T.getTauInTESDiff_",s,null);
		this.dattribs_names.addValue("LF9T.getEVInDiff_",s,null);
		this.dattribs_names.addValue("LF9T.getAESDITauInDiff_",s,null);
		this.dattribs_names.addValue("LF9T.getTauRDiff_",s,null);

		this.dattribs_names.addValue("LF9T.trades_p_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_p_getRankA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_p_getRankAStdev_",s,null);
		this.dattribs_names.addValue("LF9T.trades_p_getValA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_p_getValAA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_p_getValStdev_",s,null);
		this.dattribs_names.addValue("LF9T.trades_p_getTauRel_",s,null);
		this.dattribs_names.addValue("LF9T.trades_p_getTau_",s,null);
		this.dattribs_names.addValue("LF9T.trades.p_p0_direction_",s,null);
		this.dattribs_names.addValue("LF9T.trades_time_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_volume_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_volume_getRankA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_V_getRank_",s,null);


		this.dattribs_names.addValue("LF9T.trades_krank_VS_ttype_dir_average_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_ttype_dir_average2_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_ttype_dir_average2f_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dttype_dir_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_tau_average_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_tau_average2_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_tau_average2f_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dtau_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dp_average_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dp_average2_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dp_average2f_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_ddp_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dpv_average_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dpv_average2_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dpv_average2f_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_ddpv_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_time_to_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_time_to_aver_2_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_time_to_aver_2f_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_ptauaver_normp_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dpdtau_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dpdatau_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dpdt_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dtaudt_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dvdt_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dpdv_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dtaudv_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dttypedt_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dttypedp_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_dpdvsigned_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_v_aver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_paver2_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_taver2_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_tauaver2_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_vaver2_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_ttype2_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_VS_vsignedaver2_",s,null);


		this.dattribs_names.addValue("LF9T.trades_krank_dd_dpdv_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_dd_dpdt_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_dd_dvdtp_",s,null);

		this.dattribs_names.addValue("LF9T.trades_krank_pt_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_pt_ptaver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_pt_sptaver_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_pt_tauaver_",s,null);

		this.dattribs_names.addValue("LF9T.trades_krank_forward_getRank_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_forward_p_getValA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_forward_pd_getValA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_forward_p_buy_getValA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_forward_p_sell_getValA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_forward_p_underestimate_getValA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_forward_p_overestimate_getValA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_forward_p_end_getValA_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_forward_tau_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_forward_tau_cut_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_forward_dtau_",s,null);
		this.dattribs_names.addValue("LF9T.trades_krank_forward_dtau_cut_",s,null);
	    }
	    else
	    {
		final CalculateLiquidityDeficit9T l=c.topp.tradesT_liquiditydeficit9T;
		final CalculateKendall.KendallMultipleStats kj=l.ms_trades_krank[j],kjt=l.ms_trades_krank_tau[j];
		n+=114;

		this.dattribs.addValue(kjt.krank1.getRank());
		this.dattribs.addValue(kjt.krank2.getRank());
		this.dattribs.addValue(kjt.krank2taubuy.getRank());
		this.dattribs.addValue(kjt.krank2tausell.getRank());

		this.dattribs.addValue(l.trades_krank_o[j].krank2.getRank());
		this.dattribs.addValue(l.trades_krank_o[j].krank2.getRankA());
		this.dattribs.addValue((0.5*(c.p_buy+c.p_sell))/l.trades_krank_o[j].p_aver-1);
		this.dattribs.addValue(l.trades_krank_o_delayed[j].krank2.getRank());
		this.dattribs.addValue(l.trades_krank_o_delayed[j].krank2.getRankA());
		this.dattribs.addValue(l.trades_krank_o_delayed_halfT[j].krank2.getRank());
		this.dattribs.addValue(l.trades_krank_o_delayed_halfT[j].krank2.getRankA());
		this.dattribs.addValue(l.trades_krank_o_doublecouple[j].getRank());
		this.dattribs.addValue(l.trades_krank_o_doublecouple[j].getRankA());
		this.dattribs.addValue(l.trades_krank_dinterval[j].getRank());

		this.dattribs.addValue(kj.krank1.getRank());
		this.dattribs.addValue(kj.krank2.getRank());
		this.dattribs.addValue(kj.krank2taubuy.getRank());
		this.dattribs.addValue(kj.krank2tausell.getRank());
		this.dattribs.addValue(kj.krank2cd.getRank());
		this.dattribs.addValue(kj.krank3.getRank());

		this.dattribs.addValue(l.trades_krank[j].getRank());
		this.dattribs.addValue(l.trades_krank_buy[j].getRank());
		this.dattribs.addValue(l.trades_krank_sell[j].getRank());
		this.dattribs.addValue(l.trades_krank_bs[j].getRank());
		this.dattribs.addValue(l.trades_krank_tau[j].getRank());
		this.dattribs.addValue(l.trades_krank_cd[j].getRank());
		this.dattribs.addValue(l.trades_krank_cd_delayed[j].getRank());
		this.dattribs.addValue(l.trades_krank_XY[j].getRank());
		this.dattribs.addValue(l.trades_krank_self[j].getRank());
		this.dattribs.addValue(l.tmed[j].getTauMedRel());
		this.dattribs.addValue(l.tmed[j].getTauMedRelS());
		this.dattribs.addValue(l.tmed[j].getVRel());
		final double td=l.tdiff[j].t_range_cut<0?Double.NaN:c.getTime()-l.tdiff[j].t_range_cut;
		this.dattribs.addValue(td);
		this.dattribs.addValue(l.tdiff[j].getTauR_Buy()/td);
		this.dattribs.addValue(l.tdiff[j].getTauR_Sell()/td);
		this.dattribs.addValue(l.tdiff[j].getTauRTotal_Buy()/td);
		this.dattribs.addValue(l.tdiff[j].getTauRTotal_Sell()/td);

		this.dattribs.addValue(l.tdiff[j].getETauInDiff());
		this.dattribs.addValue(l.tdiff[j].getEDITauInDiff());
		this.dattribs.addValue(l.tdiff[j].getESDITauInDiff());
		this.dattribs.addValue(l.tdiff[j].getTauTDiff());
		this.dattribs.addValue(l.tdiff[j].getTauInTESDiff());
		this.dattribs.addValue(l.tdiff[j].getEVInDiff());
		this.dattribs.addValue(l.tdiff[j].getAESDITauInDiff());
		this.dattribs.addValue(l.tdiff[j].getTauRDiff());

		final CalculateKendall.KendallMStats tpj=l.trades_p[j];
		this.dattribs.addValue(tpj.trades_price.getRank());
		this.dattribs.addValue(tpj.trades_price.getRankA());
		this.dattribs.addValue(tpj.trades_price.getRankAStdev());
		this.dattribs.addValue(tpj.trades_price.getValA());
		this.dattribs.addValue(tpj.trades_price.getValAA());
		this.dattribs.addValue(tpj.trades_price.getValAStdev());
		this.dattribs.addValue(tpj.getTauRel());
		this.dattribs.addValue(tpj.trades_time.getValA());
		this.dattribs.addValue(tpj.p0_direction);
		this.dattribs.addValue(tpj.trades_time.getRank());
		this.dattribs.addValue(tpj.volume_growth.getRank());
		this.dattribs.addValue(tpj.volume_growth.getRankA());
		this.dattribs.addValue(l.trades_krank_V[j].getRank());

		final CalculateKendall.DAvergeCalculateKendall kw=l.trades_krank_VS[j];
		this.dattribs.addValue(kw.ttype_dir_average);
		this.dattribs.addValue(kw.ttype_dir_average2);
		this.dattribs.addValue(kw.ttype_dir_average2f);
		this.dattribs.addValue(kw.dttype_dir);
		this.dattribs.addValue(kw.tau_average);
		this.dattribs.addValue(kw.tau_average2);
		this.dattribs.addValue(kw.tau_average2f);
		this.dattribs.addValue(kw.dtau);
		this.dattribs.addValue(kw.dp_average);
		this.dattribs.addValue(kw.dp_average2);
		this.dattribs.addValue(kw.dp_average2f);
		this.dattribs.addValue(kw.ddp);
		this.dattribs.addValue(kw.dpv_average);
		this.dattribs.addValue(kw.dpv_average2);
		this.dattribs.addValue(kw.dpv_average2f);
		this.dattribs.addValue(kw.ddpv);
		this.dattribs.addValue(kw.time_to_aver);
		this.dattribs.addValue(kw.time_to_aver_2);
		this.dattribs.addValue(kw.time_to_aver_2f);
		this.dattribs.addValue(kw.ptauaver_normp);
		this.dattribs.addValue(kw.dpdtau_aver);
		this.dattribs.addValue(kw.dpdatau_aver);
		this.dattribs.addValue(kw.dpdt_aver);
		this.dattribs.addValue(kw.dtaudt_aver);
		this.dattribs.addValue(kw.dvdt_aver);
		this.dattribs.addValue(kw.dpdv_aver);
		this.dattribs.addValue(kw.dtaudv_aver);
		this.dattribs.addValue(kw.dttypedt_aver);
		this.dattribs.addValue(kw.dttypedp_aver);
		this.dattribs.addValue(kw.dpdvsigned_aver);
		this.dattribs.addValue(kw.v_aver);
		this.dattribs.addValue(kw.paver2);
		this.dattribs.addValue(kw.taver2);
		this.dattribs.addValue(kw.tauaver2);
		this.dattribs.addValue(kw.vaver2);
		this.dattribs.addValue(kw.ttype2);
		this.dattribs.addValue(kw.vsignedaver2);

		final CalculateKendall.DDiffCalculateKendall kdd=l.trades_krank_dd[j];
		this.dattribs.addValue(kdd.dpdv);
		this.dattribs.addValue(kdd.dpdt);
		this.dattribs.addValue(kdd.dvdtp);

		final CalculateKendall.DPTCalculateKendall kpt=l.trades_pt[j];
		this.dattribs.addValue(kpt.pt.getRank());
		this.dattribs.addValue(kpt.ptaver);
		this.dattribs.addValue(kpt.sptaver);
		this.dattribs.addValue(kpt.tau_aver);

		final CalculateKendall.MSAverageKendall kf=l.trades_krank_forward[j];
		this.dattribs.addValue(kf.trades_krank_forward.getRank());
		this.dattribs.addValue(kf.trades_krank_forward.getValA());
		this.dattribs.addValue(kf.dp_delayed_average);
		this.dattribs.addValue(kf.dp_buy);
		this.dattribs.addValue(kf.dp_sell);
		this.dattribs.addValue(kf.dp_underestimate);
		this.dattribs.addValue(kf.dp_overestimate);
		this.dattribs.addValue(kf.dp_end);
		this.dattribs.addValue(kf.dt_tau);
		this.dattribs.addValue(kf.dt_tau_cut);
		this.dattribs.addValue(kf.dtd_tau);
		this.dattribs.addValue(kf.dtd_tau_cut);
	    }
	}
	return n;
    }



    private int setAttrLiquidity8(final boolean flag_processheader,
				  final String ticker,
				  final String [] levels_names,
				  final CalculationData c)
    {
	int n=0;
	for(int j=0;j<levels_names.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=levels_names[j];
		n+=66;
		this.dattribs_names.addValue("LF8.pinc_buy_",s,null);
		this.dattribs_names.addValue("LF8.pinc_sell_",s,null);

		this.dattribs_names.addValue("LF8.of_book_st_buy_",s,null);
		this.dattribs_names.addValue("LF8.taud_book_st_buy_",s,null);
		this.dattribs_names.addValue("LF8.taurel_book_st_buy_",s,null);
		this.dattribs_names.addValue("LF8.taurelA_book_st_buy_",s,null);
		this.dattribs_names.addValue("LF8.of_book_no_sell_buy_",s,null);
		this.dattribs_names.addValue("LF8.taurel_book_no_sell_buy_",s,null);
		this.dattribs_names.addValue("LF8.taurelA_book_no_sell_buy_",s,null);
		this.dattribs_names.addValue("LF8.tt_tcut_book_no_sell_buy_",s,null);
		this.dattribs_names.addValue("LF8.of_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.of_trades_unsatisfied_buy_",s,null);
		this.dattribs_names.addValue("LF8.of_trades_in_unsatisfied_buy_",s,null);
		this.dattribs_names.addValue("LF8.ofi_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.taurel_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.taureli_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.tauin_diff_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.tauin_diffL_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.tauin_diffE_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.tauin_diffEDI_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.v_diffE_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.sq_diffE_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.tauin_diff_bt_buy_",s,null);
		this.dattribs_names.addValue("LF8.es_diff_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.esdi_diff_trades_buy_",s,null);
		this.dattribs_names.addValue("LF8.les_diff_trades_buy_",s,null);

		this.dattribs_names.addValue("LF8.of_book_st_sell_",s,null);
		this.dattribs_names.addValue("LF8.taud_book_st_sell_",s,null);
		this.dattribs_names.addValue("LF8.taurel_book_st_sell_",s,null);
		this.dattribs_names.addValue("LF8.taurelA_book_st_sell_",s,null);
		this.dattribs_names.addValue("LF8.of_book_no_buy_sell_",s,null);
		this.dattribs_names.addValue("LF8.taurel_book_no_buy_sell_",s,null);
		this.dattribs_names.addValue("LF8.taurelA_book_no_buy_sell_",s,null);
		this.dattribs_names.addValue("LF8.tt_tcut_book_no_buy_sell_",s,null);
		this.dattribs_names.addValue("LF8.of_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.of_trades_unsatisfied_sell_",s,null);
		this.dattribs_names.addValue("LF8.of_trades_in_unsatisfied_sell_",s,null);
		this.dattribs_names.addValue("LF8.ofi_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.taurel_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.taureli_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.tauin_diff_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.tauin_diffL_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.tauin_diffE_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.tauin_diffEDI_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.v_diffE_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.sq_diffE_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.tauin_diff_bt_sell_",s,null);
		this.dattribs_names.addValue("LF8.es_diff_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.esdi_diff_trades_sell_",s,null);
		this.dattribs_names.addValue("LF8.les_diff_trades_sell_",s,null);


		this.dattribs_names.addValue("LF8.buy_trades_krank_getRank_",s,null);
		this.dattribs_names.addValue("LF8.buy_trades_krank_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF8.buy_trades_krank_sell_getRank_",s,null);
		this.dattribs_names.addValue("LF8.buy_trades_krank_tau_getRank_",s,null);
		this.dattribs_names.addValue("LF8.sell_trades_krank_getRank_",s,null);
		this.dattribs_names.addValue("LF8.sell_trades_krank_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF8.sell_trades_krank_sell_getRank_",s,null);
		this.dattribs_names.addValue("LF8.sell_trades_krank_tau_getRank_",s,null);

		this.dattribs_names.addValue("LF8.trades0_dir_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF8.trades2_dir_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF8.trades3_dir_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF8.trades3_ddir_buy_getRank_",s,null);
		this.dattribs_names.addValue("LF8.trades0_dir_sell_getRank_",s,null);
		this.dattribs_names.addValue("LF8.trades2_dir_sell_getRank_",s,null);
		this.dattribs_names.addValue("LF8.trades3_dir_sell_getRank_",s,null);
		this.dattribs_names.addValue("LF8.trades3_ddir_sell_getRank_",s,null);
	    }
	    else
	    {
		final CalculateLiquidityDeficit8 ls=c.topp.book_trades_liquiditydeficit8;
		if(levels_names.length!=ls.vlev.length)
		{
		    throw new IllegalArgumentException(
			"L1="+levels_names.length+
			" L2="+ls.vlev.length);
		}
		final CalculateLiquidityDeficit8.LDData l=ls.vlev[j];
		n+=66;
		this.dattribs.addValue(Math.log(l.pinc_buy/c.p_buy));
		this.dattribs.addValue(Math.log(l.pinc_sell/c.p_sell));

		this.dattribs.addValue(l.book_buy_st.getOldFraction());
		this.dattribs.addValue(l.book_buy_st.getTauRel());
		this.dattribs.addValue(l.book_buy_st.getTauInDiff(l.book_no_sell));
		this.dattribs.addValue(l.book_buy_st.getTauRelA());
		this.dattribs.addValue(l.book_no_sell.getOldFraction());
		this.dattribs.addValue(l.book_no_sell.getTauRel());
		this.dattribs.addValue(l.book_no_sell.getTauRelA());
		this.dattribs.addValue(l.book_no_sell.getTauTradesRel());
		this.dattribs.addValue(l.tr_buy.getOldFraction());
		this.dattribs.addValue(l.tr_buy.getVUnsatisfiedRel(l.book_no_sell));
		this.dattribs.addValue(l.tr_buy.getVUnsatisfiedInRel(l.book_no_sell));
		this.dattribs.addValue(l.tr_buy.getiOldFraction());
		this.dattribs.addValue(l.tr_buy.getTauRel());
		this.dattribs.addValue(l.tr_buy.getiTauRel());
		this.dattribs.addValue(l.tr_buy.getTauInDiff());
		this.dattribs.addValue(l.tr_buy.getTauInLDiff());
		this.dattribs.addValue(l.tr_buy.getETauInDiff());
		this.dattribs.addValue(l.tr_buy.getEDITauInDiff());
		this.dattribs.addValue(l.tr_buy.getEVInDiff());
		this.dattribs.addValue(l.tr_buy.getSQInDiff());
		this.dattribs.addValue(l.tr_buy.getTauInDiff(l.book_buy_st,l.book_no_sell));
		this.dattribs.addValue(l.tr_buy.getESTauInDiff());
		this.dattribs.addValue(l.tr_buy.getESDITauInDiff());
		this.dattribs.addValue(l.tr_buy.getTauInESLDiff());

		this.dattribs.addValue(l.book_sell_st.getOldFraction());
		this.dattribs.addValue(l.book_sell_st.getTauRel());
		this.dattribs.addValue(l.book_sell_st.getTauInDiff(l.book_no_buy));
		this.dattribs.addValue(l.book_sell_st.getTauRelA());
		this.dattribs.addValue(l.book_no_buy.getOldFraction());
		this.dattribs.addValue(l.book_no_buy.getTauRel());
		this.dattribs.addValue(l.book_no_buy.getTauRelA());
		this.dattribs.addValue(l.book_no_buy.getTauTradesRel());
		this.dattribs.addValue(l.tr_sell.getOldFraction());
		this.dattribs.addValue(l.tr_sell.getVUnsatisfiedRel(l.book_no_buy));
		this.dattribs.addValue(l.tr_sell.getVUnsatisfiedInRel(l.book_no_buy));
		this.dattribs.addValue(l.tr_sell.getiOldFraction());
		this.dattribs.addValue(l.tr_sell.getTauRel());
		this.dattribs.addValue(l.tr_sell.getiTauRel());
		this.dattribs.addValue(l.tr_sell.getTauInDiff());
		this.dattribs.addValue(l.tr_sell.getTauInLDiff());
		this.dattribs.addValue(l.tr_sell.getETauInDiff());
		this.dattribs.addValue(l.tr_sell.getEDITauInDiff());
		this.dattribs.addValue(l.tr_sell.getEVInDiff());
		this.dattribs.addValue(l.tr_sell.getSQInDiff());
		this.dattribs.addValue(l.tr_sell.getTauInDiff(l.book_no_buy,l.book_sell_st));
		this.dattribs.addValue(l.tr_sell.getESTauInDiff());
		this.dattribs.addValue(l.tr_sell.getESDITauInDiff());
		this.dattribs.addValue(l.tr_sell.getTauInESLDiff());


		this.dattribs.addValue(l.buy_trades_krank.getRank());
		this.dattribs.addValue(l.buy_trades_krank_buy.getRank());
		this.dattribs.addValue(l.buy_trades_krank_sell.getRank());
		this.dattribs.addValue(l.buy_trades_krank_tau.getRank());
		this.dattribs.addValue(l.sell_trades_krank.getRank());
		this.dattribs.addValue(l.sell_trades_krank_buy.getRank());
		this.dattribs.addValue(l.sell_trades_krank_sell.getRank());
		this.dattribs.addValue(l.sell_trades_krank_tau.getRank());

		this.dattribs.addValue(l.trades0_dir_buy.getRank());
	        this.dattribs.addValue(l.trades2_dir_buy.getRank());
		this.dattribs.addValue(l.trades3_dir_buy.getRank());
	        this.dattribs.addValue(l.trades3_ddir_buy.getRank());
		this.dattribs.addValue(l.trades0_dir_sell.getRank());
		this.dattribs.addValue(l.trades2_dir_sell.getRank());
	        this.dattribs.addValue(l.trades3_dir_sell.getRank());
	        this.dattribs.addValue(l.trades3_ddir_sell.getRank());

	    }
	}
	return n;
    }




private int setAttrLiquidity7(final boolean flag_processheader,
			      final String ticker,
			      final int nt,
			      final CalculationData c)
    {
	int n=0;
	for(int j=0;j<nt;j++)
	{
	    if(flag_processheader)
	    {
		n+=10;
		final String s=String.valueOf(j);
		this.dattribs_names.addValue("LF7.pl_buy_",s,null);
		this.dattribs_names.addValue("LF7.pl_sell_",s,null);
		this.dattribs_names.addValue("LF7.of_trades_buy_",s,null);
		this.dattribs_names.addValue("LF7.of_trades_sell_",s,null);
		this.dattribs_names.addValue("LF7.t_trades_buy_",s,null);
		this.dattribs_names.addValue("LF7.t_trades_sell_",s,null);
		this.dattribs_names.addValue("LF7.of_book_buy_",s,null);
		this.dattribs_names.addValue("LF7.of_book_sell_",s,null);
		this.dattribs_names.addValue("LF7.t_book_buy_",s,null);
		this.dattribs_names.addValue("LF7.t_book_sell_",s,null);

	    }
	    else
	    {
		n+=10;
		final CalculateLiquidityDeficit7 l=c.topp.book_trades_liquiditydeficit7;
		this.dattribs.addValue(l.price_book_buy[j]/c.p_buy-1.0);
		this.dattribs.addValue(l.price_book_sell[j]/c.p_sell-1.0);
		this.dattribs.addValue(l.trades_buy[j].getOldFraction());
		this.dattribs.addValue(l.trades_sell[j].getOldFraction());
		this.dattribs.addValue(tt(l.trades_buy[j].getTauOld()));
		this.dattribs.addValue(tt(l.trades_sell[j].getTauOld()));
		this.dattribs.addValue(l.book_buy[j].getOldFraction());
		this.dattribs.addValue(l.book_sell[j].getOldFraction());
		this.dattribs.addValue(tt(l.book_buy[j].getTauOut()));
		this.dattribs.addValue(tt(l.book_sell[j].getTauOut()));

	    }
	}
	return n;
    }

 private int setAttrLiquidity6(final boolean flag_processheader,
				  final String ticker,
				  final int nt,
				  final CalculationData c)
    {
	int n=0;
	if(flag_processheader)
	{
	    n+=18;
	    this.dattribs_names.addValue("LF6.lev_iE_buy","",null);
	    this.dattribs_names.addValue("LF6.lev_iE_sell","",null);
	    this.dattribs_names.addValue("LF6.lev_iE3_buy","",null);
	    this.dattribs_names.addValue("LF6.lev_iE3_sell","",null);
	    this.dattribs_names.addValue("LF6.t_cont_iE_buy","",null);
	    this.dattribs_names.addValue("LF6.t_cont_iE_sell","",null);
	    this.dattribs_names.addValue("LF6.t_cont_iE3_buy","",null);
	    this.dattribs_names.addValue("LF6.t_cont_iE3_sell","",null);
	    this.dattribs_names.addValue("LF6.t_max_cont_iE3_buy","",null);
	    this.dattribs_names.addValue("LF6.t_max_cont_iE3_sell","",null);

	    this.dattribs_names.addValue("LF6.tbuy0_max","",null);
	    this.dattribs_names.addValue("LF6.tbuy0_min","",null);
	    this.dattribs_names.addValue("LF6.pi0_tmax_buy","",null);
	    this.dattribs_names.addValue("LF6.pi0_tmin_buy","",null);

	    this.dattribs_names.addValue("LF6.tsell0_max","",null);
	    this.dattribs_names.addValue("LF6.tsell0_min","",null);
	    this.dattribs_names.addValue("LF6.pi0_tmax_sell","",null);
	    this.dattribs_names.addValue("LF6.pi0_tmin_sell","",null);
	}
	else
	{
	    n+=18;
	    final CalculateLiquidityDeficit6 l=c.topp.book_trades_liquiditydeficit6;
	    this.dattribs.addValue(Math.log(l.lev_iE_buy/c.p_buy));
	    this.dattribs.addValue(Math.log(l.lev_iE_sell/c.p_sell));
	    this.dattribs.addValue(Math.log(l.lev_iE3_buy/c.p_buy));
	    this.dattribs.addValue(Math.log(l.lev_iE3_sell/c.p_sell));
	    this.dattribs.addValue(tt(l.t_cont_iE_buy));
	    this.dattribs.addValue(tt(l.t_cont_iE_sell));
	    this.dattribs.addValue(tt(l.t_cont_iE3_buy));
	    this.dattribs.addValue(tt(l.t_cont_iE3_sell));
	    this.dattribs.addValue(tt(l.t_max_cont_iE3_buy));
	    this.dattribs.addValue(tt(l.t_max_cont_iE3_sell));

	    this.dattribs.addValue(tt(l.tbuy0_max));
	    this.dattribs.addValue(tt(l.tbuy0_min));
	    this.dattribs.addValue(Math.log(l.pi0_tmax_buy/l.price_book_level_buy[0]));
	    this.dattribs.addValue(Math.log(l.pi0_tmin_buy/l.price_book_level_buy[0]));

	    this.dattribs.addValue(tt(l.tsell0_max));
	    this.dattribs.addValue(tt(l.tsell0_min));
	    this.dattribs.addValue(Math.log(l.pi0_tmax_sell/l.price_book_level_sell[0]));
	    this.dattribs.addValue(Math.log(l.pi0_tmin_sell/l.price_book_level_sell[0]));
	}

	for(int j=0;j<nt;j++)
	{
	    if(flag_processheader)
	    {
		n+=10;
		final String s=String.valueOf(j);
		this.dattribs_names.addValue("LF6.pi_book_buy_",s,null);
		this.dattribs_names.addValue("LF6.pi_book_sell_",s,null);

		this.dattribs_names.addValue("LF6.v_trades_buy_",s,null);
		this.dattribs_names.addValue("LF6.v_trades_sell_",s,null);
		this.dattribs_names.addValue("LF6.tte_trades_buy_",s,null);
		this.dattribs_names.addValue("LF6.tte_trades_sell_",s,null);

		this.dattribs_names.addValue("LF6.tte_book_acc_buy_",s,null);
		this.dattribs_names.addValue("LF6.tte_book_acc_sell_",s,null);

		this.dattribs_names.addValue("LF6.price_inv_ttrades_buy_",s,null);
		this.dattribs_names.addValue("LF6.price_inv_ttrades_sell_",s,null);
	    }
	    else
	    {
		n+=10;
		final CalculateLiquidityDeficit6 l=c.topp.book_trades_liquiditydeficit6;
		this.dattribs.addValue(Math.log(l.price_book_split_buy[j]/l.price_book_level_buy[j]));
		this.dattribs.addValue(Math.log(l.price_book_split_sell[j]/l.price_book_level_sell[j]));

		this.dattribs.addValue(gv(l.v_trades_buy[j]));
		this.dattribs.addValue(gv(l.v_trades_sell[j]));
		this.dattribs.addValue(tt(l.tau_trades_buy[j]));
		this.dattribs.addValue(tt(l.tau_trades_sell[j]));

		this.dattribs.addValue(tt(l.tau_book_acc_buy[j]));
		this.dattribs.addValue(tt(l.tau_book_acc_sell[j]));

		this.dattribs.addValue(Math.log(l.pinv_ttrades_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.pinv_ttrades_sell[j]/c.p_sell));
	    }
	}

	return n;

    }



    private int setAttrLiquidity3(final boolean flag_processheader,
				  final String ticker,
				  final int nt,
				  final CalculationData c)
    {
	int n=0;
	for(int j=0;j<nt;j++)
	{
	    if(flag_processheader)
	    {
		n+=34;
		final String s=String.valueOf(j);
		this.dattribs_names.addValue("LF3.pi_book_buy_",s,null);
		this.dattribs_names.addValue("LF3.pi_book_sell_",s,null);

		this.dattribs_names.addValue("LF3.v_trades_buy_",s,null);
		this.dattribs_names.addValue("LF3.v_trades_sell_",s,null);
		this.dattribs_names.addValue("LF3.tte_trades_buy_",s,null);
		this.dattribs_names.addValue("LF3.tte_trades_sell_",s,null);
		this.dattribs_names.addValue("LF3.p_aver_trades_tte_buy_",s,null);
		this.dattribs_names.addValue("LF3.p_aver_trades_tte_sell_",s,null);
		this.dattribs_names.addValue("LF3.p_max_trades_tte_buy_",s,null);
		this.dattribs_names.addValue("LF3.p_min_trades_tte_sell_",s,null);


		this.dattribs_names.addValue("LF3.v_book_acc_buy_",s,null);
		this.dattribs_names.addValue("LF3.v_book_acc_sell_",s,null);
		this.dattribs_names.addValue("LF3.tte_book_acc_buy_",s,null);
		this.dattribs_names.addValue("LF3.tte_book_acc_sell_",s,null);
		this.dattribs_names.addValue("LF3.p_aver_book_acc_tte_buy_",s,null);
		this.dattribs_names.addValue("LF3.p_aver_book_acc_tte_sell_",s,null);
		this.dattribs_names.addValue("LF3.p_max_book_acc_tte_buy_",s,null);
		this.dattribs_names.addValue("LF3.p_min_book_acc_tte_sell_",s,null);


		this.dattribs_names.addValue("LF3.price_inv_ttrades_buy_",s,null);
		this.dattribs_names.addValue("LF3.price_inv_ttrades_sell_",s,null);
		this.dattribs_names.addValue("LF3.price_inv_tbook_buy_",s,null);
		this.dattribs_names.addValue("LF3.price_inv_tbook_sell_",s,null);
		this.dattribs_names.addValue("LF3.n_levs_inv_ttrades_buy_",s,null);
		this.dattribs_names.addValue("LF3.n_levs_inv_ttrades_sell_",s,null);

		this.dattribs_names.addValue("LF3.tte_trades_buy_max_",s,null);
		this.dattribs_names.addValue("LF3.tte_trades_sell_max_",s,null);
		this.dattribs_names.addValue("LF3.v_trades_buy_to_tte_max_",s,null);
		this.dattribs_names.addValue("LF3.v_trades_sell_to_tte_max_",s,null);

		this.dattribs_names.addValue("LF3.price_min_t_book_acc_tte_buy_",s,null);
		this.dattribs_names.addValue("LF3.price_max_t_book_acc_tte_buy_",s,null);
		this.dattribs_names.addValue("LF3.price_min_t_book_acc_tte_sell_",s,null);
		this.dattribs_names.addValue("LF3.price_max_t_book_acc_tte_sell_",s,null);

		this.dattribs_names.addValue("LF3.vol_trades_t_book_acc_tte_buy_",s,null);
		this.dattribs_names.addValue("LF3.vol_trades_t_book_acc_tte_sell_",s,null);
	    }
	    else
	    {
		n+=34;
		final CalculateLiquidityDeficit3 l=c.topp.book_trades_liquiditydeficit3;
		this.dattribs.addValue(Math.log(l.price_book_split_buy[j]/l.price_book_level_buy[j]));
		this.dattribs.addValue(Math.log(l.price_book_split_sell[j]/l.price_book_level_sell[j]));

		this.dattribs.addValue(gv(l.v_trades_exe_buy[j]));
		this.dattribs.addValue(gv(l.v_trades_exe_sell[j]));
		this.dattribs.addValue(tt(l.tte_trades_buy[j]));
		this.dattribs.addValue(tt(l.tte_trades_sell[j]));
		this.dattribs.addValue(Math.log(l.price_aver_trades_tte_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.price_aver_trades_tte_sell[j]/c.p_sell));
		this.dattribs.addValue(Math.log(l.price_max_trades_tte_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.price_min_trades_tte_sell[j]/c.p_sell));


		this.dattribs.addValue(gv(l.v_book_acc_buy[j]));
		this.dattribs.addValue(gv(l.v_book_acc_sell[j]));
		this.dattribs.addValue(tt(l.tte_book_acc_buy[j]));
		this.dattribs.addValue(tt(l.tte_book_acc_sell[j]));
		this.dattribs.addValue(Math.log(l.price_aver_book_acc_tte_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.price_aver_book_acc_tte_sell[j]/c.p_sell));
		this.dattribs.addValue(Math.log(l.price_max_book_acc_tte_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.price_min_book_acc_tte_sell[j]/c.p_sell));

		this.dattribs.addValue(Math.log(l.price_inv_ttrades_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.price_inv_ttrades_sell[j]/c.p_sell));
		this.dattribs.addValue(Math.log(l.price_inv_tbook_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.price_inv_tbook_sell[j]/c.p_sell));
		this.dattribs.addValue(l.n_levs_inv_ttrades_buy[j]);
		this.dattribs.addValue(l.n_levs_inv_ttrades_sell[j]);

		this.dattribs.addValue(tt(l.tte_trades_buy_max[j]));
		this.dattribs.addValue(tt(l.tte_trades_sell_max[j]));
		this.dattribs.addValue(gv(l.v_trades_buy_to_tte_max[j]));
		this.dattribs.addValue(gv(l.v_trades_sell_to_tte_max[j]));

		this.dattribs.addValue(Math.log(c.p_buy/l.price_min_t_book_acc_tte_buy[j]));
		this.dattribs.addValue(Math.log(c.p_buy/l.price_max_t_book_acc_tte_buy[j]));
		this.dattribs.addValue(Math.log(c.p_sell/l.price_min_t_book_acc_tte_sell[j]));
		this.dattribs.addValue(Math.log(c.p_sell/l.price_max_t_book_acc_tte_sell[j]));

		this.dattribs.addValue(gv(l.vol_trades_t_book_acc_tte_buy[j]));
		this.dattribs.addValue(gv(l.vol_trades_t_book_acc_tte_sell[j]));
	    }
	}

	return n;

    }

    private int setAttrLiquidity5(final boolean flag_processheader,
				  final String ticker,
				  final String [] levels_names,
				  final CalculationData c)
    {
	int n=0;
	for(int j=0;j<levels_names.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=levels_names[j];
		n+=4;
		this.dattribs_names.addValue("LF5.pbook_buy_",s,null);
		this.dattribs_names.addValue("LF5.ptrades_buy_",s,null);
		this.dattribs_names.addValue("LF5.pbook_sell_",s,null);
		this.dattribs_names.addValue("LF5.ptrades_sell_",s,null);
	    }
	    else
	    {
		final CalculateLiquidityDeficit5 gb=c.topp.book_trades_liquiditydeficit5_down,gs=c.topp.book_trades_liquiditydeficit5_up;
		n+=4;
		this.dattribs.addValue(Math.log(gb.price_aver_t_orders[j]/c.p_buy));
		this.dattribs.addValue(Math.log(gb.price_aver_t_trades[j]/c.p_buy));
		this.dattribs.addValue(Math.log(gs.price_aver_t_orders[j]/c.p_sell));
		this.dattribs.addValue(Math.log(gs.price_aver_t_trades[j]/c.p_sell));
	    }
	}
	return n;
    }

    private int setAttrLiquidity4(final boolean flag_processheader,
				  final String ticker,
				  final String [] levels_names,
				  final CalculationData c)
    {
	int n=0;

	if(flag_processheader)
	{
	    n+=8;
	    this.dattribs_names.addValue("LF4.lev_E_buy","",null);
	    this.dattribs_names.addValue("LF4.lev_E_sell","",null);
	    this.dattribs_names.addValue("LF4.lev_iE_buy","",null);
	    this.dattribs_names.addValue("LF4.lev_iE_sell","",null);
	    this.dattribs_names.addValue("LF4.lev_iE2_buy","",null);
	    this.dattribs_names.addValue("LF4.lev_iE2_sell","",null);
	    this.dattribs_names.addValue("LF4.lev_iE3_buy","",null);
	    this.dattribs_names.addValue("LF4.lev_iE3_sell","",null);
	}
	else
	{
	    n+=8;
	    final CalculateLiquidityDeficit4 l=c.topp.book_trades_liquiditydeficit4;
	    this.dattribs.addValue(Math.log(l.lev_E_buy/c.p_buy));
	    this.dattribs.addValue(Math.log(l.lev_E_sell/c.p_sell));
	    this.dattribs.addValue(Math.log(l.lev_iE_buy/c.p_buy));
	    this.dattribs.addValue(Math.log(l.lev_iE_sell/c.p_sell));
	    this.dattribs.addValue(Math.log(l.lev_iE2_buy/c.p_buy));
	    this.dattribs.addValue(Math.log(l.lev_iE2_sell/c.p_sell));
	    this.dattribs.addValue(Math.log(l.lev_iE3_buy/c.p_buy));
	    this.dattribs.addValue(Math.log(l.lev_iE3_sell/c.p_sell));
	}
	for(int j=0;j<levels_names.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=levels_names[j];
		n+=30;
		this.dattribs_names.addValue("LF4.pld_buy_",s,null);
		this.dattribs_names.addValue("LF4.pld_sell_",s,null);
		this.dattribs_names.addValue("LF4.rpld_buy_",s,null);
		this.dattribs_names.addValue("LF4.rpld_sell_",s,null);
		this.dattribs_names.addValue("LF4.spld_buy_",s,null);
		this.dattribs_names.addValue("LF4.spld_sell_",s,null);

		this.dattribs_names.addValue("LF4.lev_Econv_buy_",s,null);
		this.dattribs_names.addValue("LF4.lev_Econv_sell_",s,null);

		this.dattribs_names.addValue("LF4.pinv_tau_book_buy_",s,null);
		this.dattribs_names.addValue("LF4.pinv_tau_book_sell_",s,null);
		this.dattribs_names.addValue("LF4.pdinv_tau_book_buy_",s,null);
		this.dattribs_names.addValue("LF4.pdinv_tau_book_sell_",s,null);

		this.dattribs_names.addValue("LF4.pmax_tau_book_buy_",s,null);
		this.dattribs_names.addValue("LF4.pmin_tau_book_buy_",s,null);
		this.dattribs_names.addValue("LF4.paver_tau_book_buy_",s,null);
		this.dattribs_names.addValue("LF4.pmax_tau_book_sell_",s,null);
		this.dattribs_names.addValue("LF4.pmin_tau_book_sell_",s,null);
		this.dattribs_names.addValue("LF4.paver_tau_book_sell_",s,null);

		this.dattribs_names.addValue("LF4.pmin_Tw_buy_",s,null);
		this.dattribs_names.addValue("LF4.pmax_Tw_sell_",s,null);

		this.dattribs_names.addValue("LF4.tau_book_buy_",s,null);
		this.dattribs_names.addValue("LF4.tau_book_sell_",s,null);

		this.dattribs_names.addValue("LF4.pext_buy_",s,null);
		this.dattribs_names.addValue("LF4.pext_sell_",s,null);
		this.dattribs_names.addValue("LF4.lf_buy_",s,null);
		this.dattribs_names.addValue("LF4.lf_sell_",s,null);

		this.dattribs_names.addValue("LF4.buy_frac_",s,null);
		this.dattribs_names.addValue("LF4.sell_frac_",s,null);
		this.dattribs_names.addValue("LF4.buy_Nfrac_",s,null);
		this.dattribs_names.addValue("LF4.sell_Nfrac_",s,null);
	    }
	    else
	    {
		n+=30;
		final CalculateLiquidityDeficit4 l=c.topp.book_trades_liquiditydeficit4;

		final double dbuy=(c.p_buy-l.price_lev_cut_buy[j]),
		    dsell=(l.price_lev_cut_sell[j]-c.p_sell);
		final double dbuyi=(c.p_sell-l.price_lev_cut_buy[j]),
		    dselli=(l.price_lev_cut_sell[j]-c.p_buy);
		this.dattribs.addValue(l.inversion_length_buy[j]/dbuyi);
		this.dattribs.addValue(l.inversion_length_sell[j]/dselli);
		this.dattribs.addValue(l.rinversion_length_buy[j]/dbuy);
		this.dattribs.addValue(l.rinversion_length_sell[j]/dsell);
		this.dattribs.addValue(l.sinversion_length_buy[j]/dbuy);
		this.dattribs.addValue(l.sinversion_length_sell[j]/dsell);

		this.dattribs.addValue(Math.log(l.lev_Econv_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.lev_Econv_sell[j]/c.p_sell));

		this.dattribs.addValue(Math.log(l.price_inv_tau_book_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.price_inv_tau_book_sell[j]/c.p_sell));
		this.dattribs.addValue(Math.log(l.price_dinv_tau_book_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.price_dinv_tau_book_sell[j]/c.p_sell));

		this.dattribs.addValue(Math.log(c.p_buy/l.price_trades_max_back_tau_book_buy[j]));
		this.dattribs.addValue(Math.log(c.p_buy/l.price_trades_min_back_tau_book_buy[j]));
		this.dattribs.addValue(Math.log(c.p_buy/l.price_trades_aver_back_tau_book_buy[j]));
		this.dattribs.addValue(Math.log(c.p_sell/l.price_trades_max_back_tau_book_sell[j]));
		this.dattribs.addValue(Math.log(c.p_sell/l.price_trades_min_back_tau_book_sell[j]));
		this.dattribs.addValue(Math.log(c.p_sell/l.price_trades_aver_back_tau_book_sell[j]));

		this.dattribs.addValue(Math.log(l.price_trades_min_back_bweighted_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.price_trades_max_back_bweighted_sell[j]/c.p_sell));

		this.dattribs.addValue(tt(l.tau_book_buy[j]));
		this.dattribs.addValue(tt(l.tau_book_sell[j]));

		this.dattribs.addValue(Math.log(l.pe_lev_buy[j]/c.p_buy));
		this.dattribs.addValue(Math.log(l.pe_lev_sell[j]/c.p_sell));
		this.dattribs.addValue(Math.log(l.pe_book_buy[j]/l.pe_trades_buy[j]));
		this.dattribs.addValue(Math.log(l.pe_book_sell[j]/l.pe_trades_sell[j]));

		this.dattribs.addValue(Math.log(l.price_trades_min_back_tau_book_buy[j]/l.price_trades_min_back_bweighted_buy[j]));
		this.dattribs.addValue(Math.log(l.price_trades_max_back_tau_book_sell[j]/l.price_trades_max_back_bweighted_sell[j]));
		this.dattribs.addValue((l.price_trades_min_back_tau_book_buy[j]-c.p_buy)/(l.price_trades_min_back_bweighted_buy[j]-c.p_buy));
		this.dattribs.addValue((l.price_trades_max_back_tau_book_sell[j]-c.p_sell)/(l.price_trades_max_back_bweighted_sell[j]-c.p_sell));
	    }
	}
	return n;
    }

    private int setAttrLiquidity2(final boolean flag_processheader,
				  final String ticker,
				  final String [] levels_names,
				  final CalculationData c)
    {
	int n=0;
	for(int j=0;j<levels_names.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=levels_names[j];
		n+=48;
		this.dattribs_names.addValue("LF2.dtau_",s,null);
		this.dattribs_names.addValue("LF2.tdtau_",s,null);
		this.dattribs_names.addValue("LF2.otdtau_buy_",s,null);
		this.dattribs_names.addValue("LF2.otdtau_sell_",s,null);
		this.dattribs_names.addValue("LF2.obuy_tsell_",s,null);
		this.dattribs_names.addValue("LF2.osell_tbuy_",s,null);

		this.dattribs_names.addValue("LF2.tau_obuy_",s,null);
		this.dattribs_names.addValue("LF2.tau_osell_",s,null);
		this.dattribs_names.addValue("LF2.tau_tbuy_",s,null);
		this.dattribs_names.addValue("LF2.tau_tsell_",s,null);

		this.dattribs_names.addValue("LF2.taut_limit_buy_",s,null);
		this.dattribs_names.addValue("LF2.taut_market_sell_",s,null);
		this.dattribs_names.addValue("LF2.taut_end_buy_",s,null);
		this.dattribs_names.addValue("LF2.taut_plev_buy_",s,null);
		this.dattribs_names.addValue("LF2.taut_limit_sell_",s,null);
		this.dattribs_names.addValue("LF2.taut_market_buy_",s,null);
		this.dattribs_names.addValue("LF2.taut_end_sell_",s,null);
		this.dattribs_names.addValue("LF2.taut_plev_sell_",s,null);
		this.dattribs_names.addValue("LF2.pa_taut_buy_",s,null);
		this.dattribs_names.addValue("LF2.pa_tauFt_buy_",s,null);
		this.dattribs_names.addValue("LF2.pa_taut_sell_",s,null);
		this.dattribs_names.addValue("LF2.pa_tauFt_sell_",s,null);
		this.dattribs_names.addValue("LF2.vr_taut_buy_",s,null);
		this.dattribs_names.addValue("LF2.vr_taut_sell_",s,null);

		this.dattribs_names.addValue("LF2.pt_pa_buy_",s,null);
		this.dattribs_names.addValue("LF2.pt_pa_rel_buy_",s,null);
		this.dattribs_names.addValue("LF2.pt_pe_buy_",s,null);
		this.dattribs_names.addValue("LF2.pt_pe_rel_buy_",s,null);

		this.dattribs_names.addValue("LF2.pt_pa_sell_",s,null);
		this.dattribs_names.addValue("LF2.pt_pa_rel_sell_",s,null);
		this.dattribs_names.addValue("LF2.pt_pe_sell_",s,null);
		this.dattribs_names.addValue("LF2.pt_pe_rel_sell_",s,null);

		this.dattribs_names.addValue("LF2.psqt_pa_buy_",s,null);
		this.dattribs_names.addValue("LF2.psqt_pa_rel_buy_",s,null);
		this.dattribs_names.addValue("LF2.psqt_pe_buy_",s,null);
		this.dattribs_names.addValue("LF2.psqt_pe_rel_buy_",s,null);

		this.dattribs_names.addValue("LF2.psqt_pa_sell_",s,null);
		this.dattribs_names.addValue("LF2.psqt_pa_rel_sell_",s,null);
		this.dattribs_names.addValue("LF2.psqt_pe_sell_",s,null);
		this.dattribs_names.addValue("LF2.psqt_pe_rel_sell_",s,null);

		this.dattribs_names.addValue("LF2.delta_",s,null);
		this.dattribs_names.addValue("LF2.eflow_",s,null);
		this.dattribs_names.addValue("LF2.prel_",s,null);
		this.dattribs_names.addValue("LF2.aver.prel_",s,null);
		this.dattribs_names.addValue("LF2.rprel_",s,null);
		this.dattribs_names.addValue("LF2.aver.rprel_",s,null);

		this.dattribs_names.addValue("LF2.k.buy_",s,null);
		this.dattribs_names.addValue("LF2.k.sell_",s,null);
	    }
	    else
	    {
		n+=48;
		final CalculateLiquidityDeficit2 l=c.topp.liquiditydeficit2;
		final MatchOrdersVariousShifts.MatchWithShift m=l.orders_mref[j];
		long vbuy_flow=l.v_trades_flow_exe_buy[j],
		    vsell_flow=l.v_trades_flow_exe_sell[j],
		    tau_orders_buy=l.tau_orders_buy[j],
		    tau_orders_sell=l.tau_orders_sell[j],
		    tau_trades_buy=l.tau_trades_buy[j],
		    tau_trades_sell=l.tau_trades_sell[j];
		final double
		    pbuy=l.price_trades_tau_buy[j],
		    psell=l.price_trades_tau_sell[j],
		    pabuy=l.price_trades_aver_tau_buy[j],
		    pasell=l.price_trades_aver_tau_sell[j],
		    pmed=0.5*(c.p_buy+c.p_sell);

		this.dattribs.addValue(tdiff(tau_orders_buy,tau_orders_sell));
		this.dattribs.addValue(tdiff(tau_trades_buy,tau_trades_sell));
		this.dattribs.addValue(tdiff(tau_orders_buy,tau_trades_buy));
		this.dattribs.addValue(tdiff(tau_orders_sell,tau_trades_sell));
		this.dattribs.addValue(tdiff(tau_orders_buy,tau_trades_sell));
		this.dattribs.addValue(tdiff(tau_orders_sell,tau_trades_buy));

		this.dattribs.addValue(tt(tau_orders_buy));
		this.dattribs.addValue(tt(tau_orders_sell));
		this.dattribs.addValue(tt(tau_trades_buy));
		this.dattribs.addValue(tt(tau_trades_sell));

		this.dattribs.addValue(tt(l.tau_trades_limit_buy[j]));
		this.dattribs.addValue(tt(l.tau_trades_market_sell[j]));
		this.dattribs.addValue(tt(l.tau_trades_ending_buy[j]));
		this.dattribs.addValue(tt(l.tau_trades_plev_buy[j]));
		this.dattribs.addValue(tt(l.tau_trades_limit_sell[j]));
		this.dattribs.addValue(tt(l.tau_trades_market_buy[j]));
		this.dattribs.addValue(tt(l.tau_trades_ending_sell[j]));
		this.dattribs.addValue(tt(l.tau_trades_plev_sell[j]));
		this.dattribs.addValue(l.price_trades_aver_tau_trades_buy[j]/pmed);
		this.dattribs.addValue(l.price_trades_aver_tauF_trades_buy[j]/pmed);
		this.dattribs.addValue(l.price_trades_aver_tau_trades_sell[j]/pmed);
		this.dattribs.addValue(l.price_trades_aver_tauF_trades_sell[j]/pmed);
		this.dattribs.addValue(Math.log(l.v_trades_tau_trades_buy[j]/((double)l.v_trades_buy_exe[j])));
		this.dattribs.addValue(Math.log(l.v_trades_tau_trades_sell[j]/((double)l.v_trades_sell_exe[j])));

		if(true)
		{
		    final double p=Math.log(l.price_orders_taver_buy[j]/l.price_orders_aver_buy[j]);
		    final double p1=Math.log(l.price_orders_taver_buy[j]/c.p_buy);
		    final double norm=Math.log(c.p_buy/l.price_orders_levels_buy[j]);

		    this.dattribs.addValue(p);
		    this.dattribs.addValue(p/norm);
		    this.dattribs.addValue(p1);
		    this.dattribs.addValue(p1/norm);
		}
		if(true)
		{
		    final double p=Math.log(l.price_orders_taver_sell[j]/l.price_orders_aver_sell[j]);
		    final double p1=Math.log(l.price_orders_taver_sell[j]/c.p_sell);
		    final double norm=Math.log(c.p_sell/l.price_orders_levels_sell[j]);

		    this.dattribs.addValue(p);
		    this.dattribs.addValue(p/norm);
		    this.dattribs.addValue(p1);
		    this.dattribs.addValue(p1/norm);
		}

		if(true)
		{
		    final double p=Math.log(l.price_orders_sqtaver_buy[j]/l.price_orders_aver_buy[j]);
		    final double p1=Math.log(l.price_orders_sqtaver_buy[j]/c.p_buy);
		    final double norm=Math.log(c.p_buy/l.price_orders_levels_buy[j]);

		    this.dattribs.addValue(p);
		    this.dattribs.addValue(p/norm);
		    this.dattribs.addValue(p1);
		    this.dattribs.addValue(p1/norm);
		}
		if(true)
		{
		    final double p=Math.log(l.price_orders_sqtaver_sell[j]/l.price_orders_aver_sell[j]);
		    final double p1=Math.log(l.price_orders_sqtaver_sell[j]/c.p_sell);
		    final double norm=Math.log(c.p_sell/l.price_orders_levels_sell[j]);

		    this.dattribs.addValue(p);
		    this.dattribs.addValue(p/norm);
		    this.dattribs.addValue(p1);
		    this.dattribs.addValue(p1/norm);
		}


		this.dattribs.addValue(m!=null?m.delta/pmed:Double.NaN);
		this.dattribs.addValue(Math.log((vbuy_flow<=0?1.0:(double)vbuy_flow)/(vsell_flow<=0?1.0:(double)vsell_flow)));
		this.dattribs.addValue(Math.log(psell/pbuy));
		this.dattribs.addValue(Math.log(pasell/pabuy));
		this.dattribs.addValue(Math.log((psell*pbuy)/(pmed*pmed)));
		this.dattribs.addValue(Math.log((pasell*pabuy)/(pmed*pmed)));

		this.dattribs.addValue(l.kendall_buy[j]);
		this.dattribs.addValue(l.kendall_sell[j]);
	    }
	}
	return n;
    }

    private static double tdiff(final long t1,final long t2)
    {
	if(t1<0||t2<0)
	{
	    return Double.NaN;
	}
	return ((double)(t1-t2))/ModelParameters.DISTANCE_TIME_SCALE;
    }
    private static double tt(final long t)
    {
	if(t<0)
	{
	    return Double.NaN;
	}
	return ((double)t)/ModelParameters.DISTANCE_TIME_SCALE;
    }


    private int setAttrLiquidityDeficit01(final boolean flag_processheader,
				   final String ticker,
				   final String [] levels_names,
				   final CalculationData c)
    {
	int n=0;
	for(int j=0;j<levels_names.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=levels_names[j];
		n+=15;
		this.dattribs_names.addValue("LF.flow_buy_",s,null);
		this.dattribs_names.addValue("LF.flow_sell_",s,null);
		this.dattribs_names.addValue("LF.flow_",s,null);
		this.dattribs_names.addValue("LF.eflow_",s,null);
		this.dattribs_names.addValue("LF.eotflow_",s,null);
		this.dattribs_names.addValue("LF.bflow_",s,null);
		this.dattribs_names.addValue("LF.dtau_",s,null);
		this.dattribs_names.addValue("LF.bttau_buy_",s,null);
		this.dattribs_names.addValue("LF.bttau_sell_",s,null);
		this.dattribs_names.addValue("LF.ptaumin_buy_",s,null);
		this.dattribs_names.addValue("LF.ptaumin_sell_",s,null);
		this.dattribs_names.addValue("LF.ptaubs_",s,null);
		this.dattribs_names.addValue("LF.pottaubs_",s,null);
		this.dattribs_names.addValue("LF.aver_ptaubs_",s,null);
		this.dattribs_names.addValue("LF.aver_pottaubs_",s,null);
	    }
	    else
	    {
		final CalculateLiquidityDeficit lf_buy=c.topp.liquidity_buy01,lf_sell=c.topp.liquidity_sell01;
		long vbuy_book=lf_buy.v_orders[j],
		    vbuy_exe=lf_buy.v_flow_exe[j],
		    vbuy_ot_exe=lf_buy.v_flow_ot_exe[j],
		    tau_buy=lf_buy.tau_orders[j],
		    tau_trades_buy=lf_buy.tau_trades[j],
		    vsell_book=lf_sell.v_orders[j],
		    vsell_exe=lf_sell.v_flow_exe[j],
		    vsell_ot_exe=lf_sell.v_flow_ot_exe[j],
		    tau_sell=lf_sell.tau_orders[j],
		    tau_trades_sell=lf_sell.tau_trades[j];
		final double fbuy;
		if(vbuy_book<=0 || vbuy_exe<=0)
		{
		    fbuy=Double.NaN;
		}
		else
		{
		    fbuy=Math.log((vbuy_book<=0?1.0:(double)vbuy_book)
				  /(vbuy_exe<=0?1.0:(double)vbuy_exe));
		}
		final double fsell;
		if(vsell_book<=0 || vsell_exe<=0)
		{
		    fsell=Double.NaN;
		}
		else
		{
		    fsell=Math.log((vsell_book<=0?1.0:(double)vsell_book)
				   /(vsell_exe<=0?1.0:(double)vsell_exe));
		}
		n+=15;
		this.dattribs.addValue(fbuy);
		this.dattribs.addValue(fsell);
		this.dattribs.addValue(fbuy-fsell);
		this.dattribs.addValue(Math.log((vbuy_exe<=0?1.0:(double)vbuy_exe)/(vsell_exe<=0?1.0:(double)vsell_exe)));
		this.dattribs.addValue(Math.log((vbuy_ot_exe<=0?1.0:(double)vbuy_ot_exe)/(vsell_ot_exe<=0?1.0:(double)vsell_ot_exe)));
		this.dattribs.addValue(Math.log((vbuy_book<=0?1.0:(double)vbuy_book)/(vsell_book<=0?1.0:(double)vsell_book)));
		this.dattribs.addValue(tau_buy>0&&tau_sell>0?(tau_buy-tau_sell)/ModelParameters.DISTANCE_TIME_SCALE:Double.NaN);
		this.dattribs.addValue(tau_buy>0&&tau_trades_buy>0?(tau_buy-tau_trades_buy)/ModelParameters.DISTANCE_TIME_SCALE:Double.NaN);
		this.dattribs.addValue(tau_sell>0&&tau_trades_sell>0?(tau_sell-tau_trades_sell)/ModelParameters.DISTANCE_TIME_SCALE:Double.NaN);
		this.dattribs.addValue(Math.log(lf_buy.price_taumin[j]/c.p_buy));
		this.dattribs.addValue(Math.log(lf_sell.price_taumin[j]/c.p_sell));
		this.dattribs.addValue(Math.log(lf_buy.price_tau[j]/lf_sell.price_tau[j]));
		this.dattribs.addValue(Math.log(lf_buy.price_ot_tau[j]/lf_sell.price_ot_tau[j]));
		this.dattribs.addValue(Math.log(lf_buy.price_aver_tau[j]/lf_sell.price_aver_tau[j]));
		this.dattribs.addValue(Math.log(lf_buy.price_ot_aver_tau[j]/lf_sell.price_ot_aver_tau[j]));
	    }
	}
	return n;
    }


    private int setAttrCurrent(final boolean flag_processheader,
			       final String ticker,
			       final String [] levels_names,
			       final CalculationData c)
    {
	int n=0;
	for(int j=0;j<levels_names.length;j++)
	{
	    if(flag_processheader)
	    {
		final String s=levels_names[j];
		if(ATTRIB_VOLUMES)
		{
		    n+=2;
		    this.dattribs_names.addValue("Vb.buy_",s,null);
		    this.dattribs_names.addValue("Vb.sell_",s,null);
		}
		if(ATTRIB_VOLUME_DIFFERENCES_AUTOGENERATED)
                {
                    n++;
                    this.dattribs_names.addValue("Vb.buy_sell_",s,null);
                }
		if(ATTRIB_BOOK_MEDIAN)
		{
		    n+=2;
		    this.dattribs_names.addValue("Bm.buy_",s,null);
		    this.dattribs_names.addValue("Bm.sell_",s,null);
		}
		if(ATTRIB_BOOK_N_ORDERS)
		{
		    n+=2;
		    this.dattribs_names.addValue("Bno.buy_",s,null);
		    this.dattribs_names.addValue("Bno.sell_",s,null);
		}
		if(ATTRIB_BOOK_PC_PRESETLEVELS)
		{
		    n+=2;
		    this.dattribs_names.addValue("Bce.buy_",s,null);
		    this.dattribs_names.addValue("Bce.sell_",s,null);
		}
		if(ATTRIB_BOOK_D2_PRESETLEVELS)
		{
		    n+=2;
		    this.dattribs_names.addValue("Bd2.buy_",s,null);
		    this.dattribs_names.addValue("Bd2.sell_",s,null);
		}
		if(ATTRIB_BOOK_PC_PRESETLEVELS_SUM)
		{
		    n+=1;
		    this.dattribs_names.addValue("Bce.s_",s,null);
		}
		if(ATTRIB_BOOK_PC_DIS_PRESETLEVELS)
		{
		    n+=2;
		    this.nattribs_names.addValue("Bce.L.buy_",s,"0,1");
		    this.nattribs_names.addValue("Bce.L.sell_",s,"0,1");
		}
		if(ATTRIB_BM_DIFFERENCES_AUTOGENERATED)
                {
                    n+=2;
		    this.dattribs_names.addValue("Bm.p.buy_sell_",s,null);
		    this.dattribs_names.addValue("Bm.m.buy_sell_",s,null);
		}
		if(ATTRIB_BM_SUM_AUTOGENERATED)
		{
		    n++;
		    this.dattribs_names.addValue("Bm.s.buy_sell_",s,null);
		}
	    }
	    else
	    {
		final CalculateBookVolumePriceThreshold bbuy=c.topp.calculate_buy_thresholds,bsell=c.topp.calculate_sell_thresholds;
		if(ATTRIB_VOLUMES)
		{
		    n+=2;
		    // book volume (calculated at j-th fraction of best buy/sell).
		    this.dattribs.addValue(gv(bbuy.out_shares[j]));
		    this.dattribs.addValue(gv(bsell.out_shares[j]));
		}
		if(ATTRIB_VOLUME_DIFFERENCES_AUTOGENERATED)
                {
                    n++;
		    this.dattribs.addValue(gv(bbuy.out_shares[j])-gv(bsell.out_shares[j]));
		}
		final double bm=bbuy.book_median[j]/c.p_buy,sm=bsell.book_median[j]/c.p_sell;
		if(ATTRIB_BOOK_MEDIAN)
		{
		    n+=2;
		    this.dattribs.addValue(bm);
		    this.dattribs.addValue(sm);
		}
		if(ATTRIB_BOOK_N_ORDERS)
		{
		    n+=2;
		    this.dattribs.addValue(bbuy.n_orders[j]);
		    this.dattribs.addValue(bsell.n_orders[j]);
		}
		if(ATTRIB_BOOK_PC_PRESETLEVELS)
		{
		    n+=2;
		    this.dattribs.addValue(bbuy.pc_r[j]/c.p_buy);
		    this.dattribs.addValue(bsell.pc_r[j]/c.p_sell);
		}
		if(ATTRIB_BOOK_D2_PRESETLEVELS)
		{
		    n+=2;
		    this.dattribs.addValue(bbuy.book_d2[j]);
		    this.dattribs.addValue(bsell.book_d2[j]);
		}
		if(ATTRIB_BOOK_PC_PRESETLEVELS_SUM)
		{
		    n+=1;
		    this.dattribs.addValue((bbuy.pc_r[j]+bsell.pc_r[j])/(c.p_buy+c.p_sell));
		}
		if(ATTRIB_BOOK_PC_DIS_PRESETLEVELS)
		{
		    n+=2;
		    this.nattribs.addValue(bbuy.pc_r[j]>=c.p_buy?1:0);
		    this.nattribs.addValue(bsell.pc_r[j]<=c.p_sell?1:0);
		}
		if(ATTRIB_BM_DIFFERENCES_AUTOGENERATED)
                {
		    n+=2;
		    this.dattribs.addValue(1.0/bm-sm);
		    this.dattribs.addValue(bm-sm);
		}
		if(ATTRIB_BM_SUM_AUTOGENERATED)
		{
		    n++;
		    this.dattribs.addValue(bm+sm);
		}
	    }
	}
	return n;
    }


    private int setTRDynamicsAttrTauChange(final boolean flag_processheader,
					   final String ticker,
					   final long [] delays,
					   final CalculationData c)
    {
	final int nt=delays.length;
	int n=0;
	for(int i=0;i<nt;i++)
	{
	    if(flag_processheader)
	    {
		final String t=String.valueOf(delays[i]/ModelParameters.DISTANCE_TIME_SCALE);

		n+=13;
		this.dattribs_names.addValue("Dtr.vmatched_D1_",t,null);
		this.dattribs_names.addValue("Dtr.vmatched_D2_",t,null);
		this.dattribs_names.addValue("Dtr.modelled_D1_",t,null);
		this.dattribs_names.addValue("Dtr.modelled_D2_",t,null);
		this.dattribs_names.addValue("Dtr.sA_",t,null);
		this.dattribs_names.addValue("Dtr.sAm_",t,null);
		this.dattribs_names.addValue("Dtr.dmdir_",t,null);
		this.dattribs_names.addValue("Dtr.ddmdir_",t,null);
		this.dattribs_names.addValue("Dtr.flow_price_",t,null);
		this.dattribs_names.addValue("Dtr.nlmatched_D1_",t,null);
		this.dattribs_names.addValue("Dtr.nlmatched_D2_",t,null);
		this.dattribs_names.addValue("Dtr.shift_",t,null);
		this.dattribs_names.addValue("Dtr.shift_w_",t,null);
	    }
	    else
	    {
		n+=13;
		final CalculateTRDynamics dtr=c.topp.tr_dynamics;
		final double cur=0.5*(c.p_sell+c.p_buy),
		    pmodelledD1=dtr.price_bookD1_modelled[i],
		    pmodelledD2=dtr.price_bookD2_modelled[i],
		    pflow=dtr.price_book_flow_tau[i];
		final long w1=dtr.v_effD1_matched_tau[i],w2=dtr.v_effD2_matched_tau[i];
		this.dattribs.addValue(gv(w1));
		this.dattribs.addValue(gv(w2));
		this.dattribs.addValue(pmodelledD1/cur);
		this.dattribs.addValue(pmodelledD2/cur);
		this.dattribs.addValue((pmodelledD1+pmodelledD2)/cur);
		this.dattribs.addValue((pmodelledD1-pmodelledD2)/cur);
		this.dattribs.addValue(Math.log((cur*cur)/(pmodelledD1*pmodelledD2)));
		this.dattribs.addValue(Math.log((cur*cur)/(pmodelledD1*pmodelledD2))/Math.log(pmodelledD2/pmodelledD1));
		this.dattribs.addValue(pflow/cur);
		this.dattribs.addValue(dtr.minlevelsD1[i]);
		this.dattribs.addValue(dtr.minlevelsD2[i]);
		this.dattribs.addValue(dtr.price_mmshift_tau[i]);
		this.dattribs.addValue(dtr.price_mmshift_w_tau[i]);
	    }
	}
	return n;
    }

    private int setVirtualShiftsTau(final boolean flag_processheader,
				    final String ticker,
				    final long [] delays,
				    final CalculationData c)
    {
	final int nt=delays.length;
	int n=0;
	for(int i=0;i<nt;i++)
	{
	    if(flag_processheader)
	    {
		n+=8;
		final String t=String.valueOf(delays[i]/ModelParameters.DISTANCE_TIME_SCALE);

		this.dattribs_names.addValue("AT.buy_",t,null);
		this.dattribs_names.addValue("AT.sell_",t,null);
		this.dattribs_names.addValue("AT.diff_",t,null);
		this.dattribs_names.addValue("AT_",t,null);

		this.dattribs_names.addValue("AT.shift_",t,null);
		this.dattribs_names.addValue("AT.delta_",t,null);
		this.dattribs_names.addValue("AT.deltarel_",t,null);
		this.dattribs_names.addValue("AT.vol_",t,null);
	    }
	    else
	    {
		n+=8;
		final CalculateBookVirtualShiftsTau vs=c.topp.calculate_virtual_shifts_tau;
		final double ds=vs.dspread;
		final double cur=0.5*(c.p_sell+c.p_buy);

		this.dattribs.addValue(vs.a_tau_buy[i]<0?Double.NaN:vs.a_tau_buy[i]);
		this.dattribs.addValue(vs.a_tau_sell[i]<0?Double.NaN:vs.a_tau_sell[i]);
		this.dattribs.addValue(vs.a_tau_diff[i]); // may be negative
		this.dattribs.addValue(vs.a_tau[i]<0?Double.NaN:vs.a_tau[i]);

		final MatchOrdersVariousShifts.MatchWithShift ei=vs.match_results_tau[i];
		if(ei!=null)
		{
		    this.dattribs.addValue((ds+ei.shift)/cur);
		    this.dattribs.addValue(ei.delta/cur);
		    this.dattribs.addValue(ei.delta/(ds+ei.shift));
		    this.dattribs.addValue(gv(ei.vmatch));
		}
		else
		{
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		}
	    }
	}
	return n;
    }


    private int setVirtualShifts2(final boolean flag_processheader,
				  final String ticker,
				  final long [] delays,
				  final CalculationData c)
    {
	final int nt=delays.length;
	int n=0;
	for(int i=0;i<nt;i++)
	{
	    if(flag_processheader)
	    {
		n+=19;
		final String t=String.valueOf(delays[i]/ModelParameters.DISTANCE_TIME_SCALE);

		this.dattribs_names.addValue("DvS2.min_",t,null);
		this.dattribs_names.addValue("DvS2.max_",t,null);
		this.dattribs_names.addValue("DvS2.med_",t,null);

		this.dattribs_names.addValue("DvS2.conv.maxval_",t,null);
		this.dattribs_names.addValue("DvS2.hc_",t,null);
		this.dattribs_names.addValue("DvS2.tm_",t,null);
		this.dattribs_names.addValue("DvS2.tmm_",t,null);

		this.dattribs_names.addValue("DvS2.end.shift_",t,null);
		this.dattribs_names.addValue("DvS2.end.delta_",t,null);
		this.dattribs_names.addValue("DvS2.end.deltarel_",t,null);
		this.dattribs_names.addValue("DvS2.end.vol_",t,null);

		this.dattribs_names.addValue("DvS2.m.shift_",t,null);
		this.dattribs_names.addValue("DvS2.m.delta_",t,null);
		this.dattribs_names.addValue("DvS2.m.deltarel_",t,null);
		this.dattribs_names.addValue("DvS2.m.vol_",t,null);

		this.dattribs_names.addValue("DvS2.m2.shift_",t,null);
		this.dattribs_names.addValue("DvS2.m2.delta_",t,null);
		this.dattribs_names.addValue("DvS2.m2.deltarel_",t,null);
		this.dattribs_names.addValue("DvS2.m2.vol_",t,null);
	    }
	    else
	    {
		n+=19;
		final CalculateBookVirtualShifts2 vs=c.topp.calculate_virtual_shifts2;
		final MatchOrdersVariousShifts.MatchWithShift
		    m=vs.convexity_interv[i],
		    m2=vs.convexity_interv2[i],
		    ei=vs.end_of_interval[i];
		final double ds=vs.dspread;
		final double
		    cur=0.5*(c.p_sell+c.p_buy),
		    min=vs.tot_minprice[i],
		    max=vs.tot_maxprice[i],
		    med=0.5*(vs.trade_tot_median_low_tau[i]+vs.trade_tot_median_high_tau[i]);

		this.dattribs.addValue(vs.tot_minprice[i]/cur);
		this.dattribs.addValue(vs.tot_maxprice[i]/cur);
		this.dattribs.addValue(med/cur);

		this.dattribs.addValue(vs.max_convexity[i]);
		this.dattribs.addValue(gv(vs.v_tot_hiddentrades[i])-gv(vs.v_tot_alltrades[i]));
		this.dattribs.addValue(Math.log(med/cur));
		this.dattribs.addValue(Math.log((max*min)/(med*med)));

		if(ei!=null)
		{
		    this.dattribs.addValue((ds+ei.shift)/cur);
		    this.dattribs.addValue(ei.delta/cur);
		    this.dattribs.addValue(ei.delta/(ds+ei.shift));
		    this.dattribs.addValue(gv(ei.vmatch));
		}
		else
		{
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		}

		if(m!=null)
		{
		    this.dattribs.addValue((ds+m.shift)/cur);
		    this.dattribs.addValue(m.delta/cur);
		    this.dattribs.addValue(m.delta/(ds+m.shift));
		    this.dattribs.addValue(gv(m.vmatch));
		}
		else
		{
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		}

		if(m2!=null)
		{
		    this.dattribs.addValue((ds+m2.shift)/cur);
		    this.dattribs.addValue(m2.delta/cur);
		    this.dattribs.addValue(m2.delta/(ds+m2.shift));
		    this.dattribs.addValue(gv(m2.vmatch));
		}
		else
		{
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		    this.dattribs.addValue(Double.NaN);
		}
	    }
	}
	return n;
    }



    private int setVirtualShifts(final boolean flag_processheader,
				 final String ticker,
				 final CalculationData c)
    {
	int n=0;
	if(flag_processheader)
	{
	    final String t="";
	    n+=20;
	    this.dattribs_names.addValue("DvS.conv.maxval",t,null);

	    this.dattribs_names.addValue("DvS.cut.shift",t,null);
	    this.dattribs_names.addValue("DvS.cut.delta",t,null);
	    this.dattribs_names.addValue("DvS.cut.deltarel",t,null);
	    this.dattribs_names.addValue("DvS.cut.vol",t,null);

	    this.dattribs_names.addValue("DvS.cut2.shift",t,null);
	    this.dattribs_names.addValue("DvS.cut2.delta",t,null);
	    this.dattribs_names.addValue("DvS.cut2.deltarel",t,null);
	    this.dattribs_names.addValue("DvS.cut2.vol",t,null);

	    this.dattribs_names.addValue("DvS.eq.convexity",t,null);
	    this.dattribs_names.addValue("DvS.eq.mm",t,null);
	    this.dattribs_names.addValue("DvS.eq.tau",t,null);
	    this.dattribs_names.addValue("DvS.eq.shift",t,null);
	    this.dattribs_names.addValue("DvS.eq.delta",t,null);
	    this.dattribs_names.addValue("DvS.eq.deltarel",t,null);
	    this.dattribs_names.addValue("DvS.eq.vol",t,null);

	    this.dattribs_names.addValue("DvS.end.shift",t,null);
	    this.dattribs_names.addValue("DvS.end.delta",t,null);
	    this.dattribs_names.addValue("DvS.end.deltarel",t,null);
	    this.dattribs_names.addValue("DvS.end.vol",t,null);


	}
	else
	{
	    final CalculateBookVirtualShifts vs=c.topp.calculate_virtual_shifts;
	    n+=20;
	    final MatchOrdersVariousShifts.MatchWithShift
		ea=vs.end_of_active,
		ea2=vs.end_of_active2,
		eq=vs.equilibrium_with_trades,
		ei=vs.end_of_interval;
	    final double ds=vs.dspread;
	    final double cur=0.5*(c.p_sell+c.p_buy);

	    this.dattribs.addValue(vs.max_convexity);
	    if(ea!=null)
	    {
		this.dattribs.addValue((ds+ea.shift)/cur);
		this.dattribs.addValue(ea.delta/cur);
		this.dattribs.addValue(ea.delta/(ds+ea.shift));
		this.dattribs.addValue(gv(ea.vmatch));
	    }
	    else
	    {
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
	    }
	    if(ea2!=null)
	    {
		this.dattribs.addValue((ds+ea2.shift)/cur);
		this.dattribs.addValue(ea2.delta/cur);
		this.dattribs.addValue(ea2.delta/(ds+ea2.shift));
		this.dattribs.addValue(gv(ea2.vmatch));
	    }
	    else
	    {
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
	    }
	    this.dattribs.addValue(vs.equilibrium_convexity);
	    this.dattribs.addValue(vs.equilibrium_trades_max_min/cur);
	    this.dattribs.addValue(vs.time_equilibrium);
	    if(eq!=null)
	    {
		this.dattribs.addValue((ds+eq.shift)/cur);
		this.dattribs.addValue(eq.delta/cur);
		this.dattribs.addValue(eq.delta/(ds+eq.shift));
		this.dattribs.addValue(gv(eq.vmatch));
	    }
	    else
	    {
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
	    }
	    if(ei!=null)
	    {
		this.dattribs.addValue((ds+ei.shift)/cur);
		this.dattribs.addValue(ei.delta/cur);
		this.dattribs.addValue(ei.delta/(ds+ei.shift));
		this.dattribs.addValue(gv(ei.vmatch));
	    }
	    else
	    {
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
		this.dattribs.addValue(Double.NaN);
	    }
	}
	return n;
    }


    private int setTradeAttrTauChange(final boolean flag_processheader,
				      final String ticker,
				      final long [] delays,
				      final CalculationData c)
    {
	final int nt=delays.length;
	int n=0;
	for(int i=0;i<nt;i++)
	{
	    if(flag_processheader)
	    {
		final String t=String.valueOf(delays[i]/ModelParameters.DISTANCE_TIME_SCALE);
		if(ATTRIB_VOLUMES)
		{
		    n+=2;
		    this.dattribs_names.addValue("TV.buy_",t,null);
		    this.dattribs_names.addValue("TV.sell_",t,null);
		}
		if(ATTRIB_VOLUME_EO)
		{
		   n+=4;
		   this.dattribs_names.addValue("TVeo.buy_",t,null);
		   this.dattribs_names.addValue("TVeo.sell_",t,null);
		   this.dattribs_names.addValue("Ttte.buy_",t,null);
		   this.dattribs_names.addValue("Ttte.sell_",t,null);
		}
		if(ATTRIB_VOLUME_DIFFERENCES_AUTOGENERATED)
                {
                    n++;
                    this.dattribs_names.addValue("TV.buy_sell_",t,null);
                }
		if(ATTRIB_HIDDEN && ATTRIB_VOLUMES)
		{
		    n+=2;
		    this.dattribs_names.addValue("TVh.buy_",t,null);
		    this.dattribs_names.addValue("TVh.sell_",t,null);
		}
		if(ATTRIB_HIDDEN && ATTRIB_VOLUME_DIFFERENCES_AUTOGENERATED)
                {
                    n++;
                    this.dattribs_names.addValue("TVh.buy_sell_",t,null);
                }
		if(ATTRIB_FRAC_VOLUME_ABOVE_BELOW)
		{
		    n+=2;
		    this.dattribs_names.addValue("TVbuy.fabove_",t,null);
		    this.dattribs_names.addValue("TVsell.fabove_",t,null);
		}
		if(ATTRIB_T_DISTANCE)
		{
		    n+=3;
		    this.dattribs_names.addValue("T.dist_buy_",t,null);
		    this.dattribs_names.addValue("T.dist_sell_",t,null);
		    this.dattribs_names.addValue("T.dist_",t,null);
		    if(ATTRIB_HIDDEN)
		    {
			n+=3;
			this.dattribs_names.addValue("Th.dist_buy_",t,null);
			this.dattribs_names.addValue("Th.dist_sell_",t,null);
			this.dattribs_names.addValue("Th.dist_",t,null);
		    }
		}
		if(ATTRIB_MINMAX)
		{
		    if(ATTRIB_T_DISTANCE)
		    {
			n+=2;
			this.dattribs_names.addValue("T.dist_min_",t,null);
			this.dattribs_names.addValue("T.dist_max_",t,null);
		    }
		    n+=2;
		    this.dattribs_names.addValue("TPmin.",t,null);
		    this.dattribs_names.addValue("TPmax.",t,null);

		    if(true)
		    {
			n+=4;
			this.dattribs_names.addValue("TPf.buy.",t,null);
			this.dattribs_names.addValue("TPf.sell.",t,null);
			this.dattribs_names.addValue("TP0.buy.",t,null);
			this.dattribs_names.addValue("TP0.sell.",t,null);
		    }
		}
		if(ATTRIB_TMM)
		{
		    n+=2;
		    this.dattribs_names.addValue("Tm.buy_",t,null);
		    this.dattribs_names.addValue("Tm.sell_",t,null);
		}
		if(ATTRIB_TMM2)
		{
		    n+=2;
		    this.dattribs_names.addValue("Tmf2.buy_",t,null);
		    this.dattribs_names.addValue("Tmf2.sell_",t,null);
		}

		if(ATTRIB_TMC)
		{
		    n+=2;
		    this.dattribs_names.addValue("Tmc.buy_",t,null);
		    this.dattribs_names.addValue("Tmc.sell_",t,null);
		}
		if(true)
		{
		    n+=10;
		    this.dattribs_names.addValue(OPT_BUY,t,null);
		    this.dattribs_names.addValue(OPT_SELL,t,null);
		    this.dattribs_names.addValue("Opt_",t,null);
		    this.dattribs_names.addValue("Opta.buy_",t,null);
		    this.dattribs_names.addValue("Opta.sell_",t,null);
		    this.dattribs_names.addValue("Opta_",t,null);
		    this.dattribs_names.addValue("ON.buy_",t,null);
		    this.dattribs_names.addValue("ON.sell_",t,null);
		    this.dattribs_names.addValue("OT.buy_",t,null);
		    this.dattribs_names.addValue("OT.sell_",t,null);
		}
		if(true)
	        {
		    n+=2;
		    this.dattribs_names.addValue("Ttem.buy_",t,null);
		    this.dattribs_names.addValue("Ttem.sell_",t,null);
		}
		if(ATTRIB_BOOK_THRESHOLDS_FROM_TRADES)
		{
		    n+=5;
		    this.dattribs_names.addValue("Btt.buy_",t,null);
		    this.dattribs_names.addValue("Btt.sell_",t,null);
		    this.dattribs_names.addValue("BT_",t,null);
		    this.dattribs_names.addValue("Tva.buy_",t,null);
		    this.dattribs_names.addValue("Tva.sell_",t,null);
		}
	    }
	    else
	    {
		final CalculateTradeVolume tvc=c.topp.tradevolumecalculator;
		if(ATTRIB_VOLUMES)
		{
		    n+=2;
		    this.dattribs.addValue(gv(tvc.v_alltrades_buy_tau[i]));
		    this.dattribs.addValue(gv(tvc.v_alltrades_sell_tau[i]));
		}
		if(ATTRIB_VOLUME_EO)
		{
		   n+=4;
		   this.dattribs.addValue(gv(tvc.v_eo_buy_tau[i]));
		   this.dattribs.addValue(gv(tvc.v_eo_sell_tau[i]));
		   this.dattribs.addValue(tt(tvc.time_to_execute_aver_buy[i]));
		   this.dattribs.addValue(tt(tvc.time_to_execute_aver_sell[i]));
		}
		if(ATTRIB_VOLUME_DIFFERENCES_AUTOGENERATED)
                {
                    n++;
                    this.dattribs.addValue(gv(tvc.v_alltrades_buy_tau[i])-
					   gv(tvc.v_alltrades_sell_tau[i]));
                }
		if(ATTRIB_HIDDEN && ATTRIB_VOLUMES)
		{
		    n+=2;
		    this.dattribs.addValue(gv(tvc.v_hiddentrades_buy_tau[i]));
		    this.dattribs.addValue(gv(tvc.v_hiddentrades_sell_tau[i]));
		}
		if(ATTRIB_HIDDEN && ATTRIB_VOLUME_DIFFERENCES_AUTOGENERATED)
                {
                    n++;
                    this.dattribs.addValue(gv(tvc.v_hiddentrades_buy_tau[i])-
					   gv(tvc.v_hiddentrades_sell_tau[i]));
                }
		if(ATTRIB_FRAC_VOLUME_ABOVE_BELOW)
		{
		    n+=2;
		    this.dattribs.addValue(gv(tvc.v_abovecurrent_buy_tau[i]));
		    this.dattribs.addValue(gv(tvc.v_belowcurrent_sell_tau[i]));
		}
		if(ATTRIB_T_DISTANCE)
		{
		    n+=3;
		    this.dattribs.addValue(tvc.disttofrac_all_buy[i]);
		    this.dattribs.addValue(tvc.disttofrac_all_sell[i]);
		    this.dattribs.addValue(tvc.disttofrac_all[i]);

		    if(ATTRIB_HIDDEN)
		    {
			n+=3;
			this.dattribs.addValue(tvc.disttofrac_hidden_buy[i]);
			this.dattribs.addValue(tvc.disttofrac_hidden_sell[i]);
			this.dattribs.addValue(tvc.disttofrac_hidden[i]);
		    }
		}

		if(ATTRIB_MINMAX)
		{
		    if(ATTRIB_T_DISTANCE)
		    {
			n+=2;
			this.dattribs.addValue(tvc.disttomin[i]);
			this.dattribs.addValue(tvc.disttomax[i]);
		    }
		    n+=2;
		    this.dattribs.addValue(tvc.minprice[i]/c.p_sell);
		    this.dattribs.addValue(tvc.maxprice[i]/c.p_buy);

		    if(true)
		    {
			n+=4;
			this.dattribs.addValue(tvc.trade_tm_buy_tau[i]/c.p_buy);
			this.dattribs.addValue(tvc.trade_tm_sell_tau[i]/c.p_sell);
			this.dattribs.addValue(tvc.trade_t0_buy_tau[i]/c.p_buy);
			this.dattribs.addValue(tvc.trade_t0_sell_tau[i]/c.p_sell);
		    }
		}

		if(ATTRIB_TMM)
		{
		    n+=2;
		    this.dattribs.addValue(tvc.trade_median_buy_tau[i]/tvc.maxprice[i]);
		    this.dattribs.addValue(tvc.trade_median_sell_tau[i]/tvc.minprice[i]);
		}
		if(ATTRIB_TMM2)
		{
		    n+=2;
		    this.dattribs.addValue((tvc.trade_median_f2_buy_tau[i]*tvc.maxprice[i])/(tvc.trade_median_buy_tau[i]*tvc.trade_median_buy_tau[i]));
		    this.dattribs.addValue((tvc.trade_median_f2_sell_tau[i]*tvc.minprice[i])/(tvc.trade_median_sell_tau[i]*tvc.trade_median_sell_tau[i]));
		}
		if(ATTRIB_TMC)
		{
		    n+=2;
		    this.dattribs.addValue(tvc.trade_median_buy_tau[i]/c.p_buy);
		    this.dattribs.addValue(tvc.trade_median_sell_tau[i]/c.p_sell);
		}
		if(true)
		{
		    n+=10;
		    this.dattribs.addValue(((double)tvc.n_orders_buy_tau[i])/((double)tvc.n_trades_buy_tau[i]));
		    this.dattribs.addValue(((double)tvc.n_orders_sell_tau[i])/((double)tvc.n_trades_sell_tau[i]));
		    this.dattribs.addValue(((double)(tvc.n_orders_buy_tau[i]+tvc.n_orders_sell_tau[i]))
					   /
					   ((double)(tvc.n_trades_buy_tau[i]+tvc.n_trades_sell_tau[i])));
		    this.dattribs.addValue(tvc.opt_buy_aver[i]);
		    this.dattribs.addValue(tvc.opt_sell_aver[i]);
		    this.dattribs.addValue(tvc.opt_aver[i]);
		    this.dattribs.addValue(gv(tvc.n_orders_buy_tau[i]));
		    this.dattribs.addValue(gv(tvc.n_orders_sell_tau[i]));
		    this.dattribs.addValue(gv(tvc.n_trades_buy_tau[i]));
		    this.dattribs.addValue(gv(tvc.n_trades_sell_tau[i]));
		}
		if(true)
	        {
		    n+=2;
		    final int tte_b=tvc.time_to_execute_buy[i];
		    this.dattribs.addValue(tte_b<0?Double.NaN:(double)tte_b);
		    final int tte_s=tvc.time_to_execute_sell[i];
		    this.dattribs.addValue(tte_s<0?Double.NaN:(double)tte_s);
		}
		if(ATTRIB_BOOK_THRESHOLDS_FROM_TRADES)
		{
		    n+=5;
		    final CalculateBookVolumePriceFixedThreshold
			bth=c.topp.calculate_buy_fixedthresholds,sth=c.topp.calculate_sell_fixedthresholds;
		    this.dattribs.addValue(gv(bth.vol_threshold_vcalc[i]));
		    this.dattribs.addValue(gv(sth.vol_threshold_vcalc[i]));
		    this.dattribs.addValue(gv(bth.vol_threshold_vcalc[i]+sth.vol_threshold_vcalc[i]));
		    this.dattribs.addValue(gv(tvc.v_abovecurrent_buy_tau[i]));
		    this.dattribs.addValue(gv(tvc.v_belowcurrent_sell_tau[i]));
		}
	    }
	}
	return n;
    }


    private int setAttrBookLiquidityDeficit02(final boolean flag_processheader,
				   final String ticker,
				   final CalculationData c)
    {
	int n=0;
	n+=14;
	if(flag_processheader)
	{
	    this.dattribs_names.addValue("LD.tau.deficit");
	    this.dattribs_names.addValue("LD.p.expected");
	    this.dattribs_names.addValue("LD.lnum.buy");
	    this.dattribs_names.addValue("LD.lnum.sell");
	    this.dattribs_names.addValue("LD.p.expected.buy");
	    this.dattribs_names.addValue("LD.p.expected.sell");
	    this.dattribs_names.addValue("LD.tau.deficit.buy");
	    this.dattribs_names.addValue("LD.tau.deficit.sell");
	    this.dattribs_names.addValue("LD.tau.tc.buy");
	    this.dattribs_names.addValue("LD.tau.tc.sell");
	    this.dattribs_names.addValue("LD.tau.book_tmin.buy");
	    this.dattribs_names.addValue("LD.tau.book_tmin.sell");
	    this.dattribs_names.addValue("LD.tau.btrades_tmin.buy");
	    this.dattribs_names.addValue("LD.tau.btrades_tmin.sell");
	}
	else
	{
	    final CalculateBookLiquidityDeficit l=c.topp.book_liquiditydeficit02;
	    this.dattribs.addValue(l.deficit);
	    this.dattribs.addValue(l.expected_price_change);
	    this.dattribs.addValue(l.ind_lev_tmin_book_buy);
	    this.dattribs.addValue(l.ind_lev_tmin_book_sell);
	    this.dattribs.addValue(l.expected_buy);
	    this.dattribs.addValue(l.expected_sell);
	    this.dattribs.addValue(l.deficit_buy);
	    this.dattribs.addValue(l.deficit_sell);
	    this.dattribs.addValue(l.t_crit_buy);
	    this.dattribs.addValue(l.t_crit_sell);
	    this.dattribs.addValue(l.t_tmin_book_buy);
	    this.dattribs.addValue(l.t_tmin_book_sell);
	    this.dattribs.addValue(l.t_tmin_btrades_buy);
	    this.dattribs.addValue(l.t_tmin_btrades_sell);
	}
	return n;
    }


    private int setTradeHContrib(final boolean flag_processheader,
				      final String ticker,
				      final long [] delays,
				      final CalculationData c)
    {
	final int nt=delays.length;
	int n=0;
	for(int i=0;i<nt;i++)
	{
	    if(flag_processheader)
	    {
		final String t=String.valueOf(delays[i]/ModelParameters.DISTANCE_TIME_SCALE);

		n+=2;
		this.dattribs_names.addValue("THc.all_",t,null);
		this.dattribs_names.addValue("THc.contrib_",t,null);
	    }
	    else
	    {
		final CalculateTradeHContrib hc=c.topp.tradehcontrib;
		n+=2;
		this.dattribs.addValue(gv(hc.vol_all[i]));
		this.dattribs.addValue(gv(hc.vol_hidden[i])-gv(hc.vol_all[i]));
	    }

	}
	return n;
    }


    private int setBookAttrTauChange(final boolean flag_processheader,
				     final String ticker,
				     final long [] delays,
				     final CalculationData c)
    {
	final int nt=delays.length;
	int n=0;
	for(int i=0;i<nt;i++)
	{
	    if(flag_processheader)
	    {
		final String t=String.valueOf(delays[i]/ModelParameters.DISTANCE_TIME_SCALE);
		if(ATTRIB_VOLUMES)
		{
		    n+=2;
		    this.dattribs_names.addValue("BV.buy_",t,null);
		    this.dattribs_names.addValue("BV.sell_",t,null);
		}
		if(ATTRIB_T_DISTANCE)
		{
		    n+=3;
		    this.dattribs_names.addValue("B.dist_buy_",t,null);
		    this.dattribs_names.addValue("B.dist_sell_",t,null);
		    this.dattribs_names.addValue("B.dist_",t,null);
		}
		n+=2;
		this.dattribs_names.addValue("Bmt.buy_",t,null);
		this.dattribs_names.addValue("Bmt.sell_",t,null);
	    }
	    else
	    {
		final CalculateBookVolume bvc=c.topp.bookvolumecalculator;
		if(ATTRIB_VOLUMES)
		{
		    n+=2;
		    this.dattribs.addValue(gv(bvc.v_buy_tau[i]));
		    this.dattribs.addValue(gv(bvc.v_sell_tau[i]));
		}
		if(ATTRIB_T_DISTANCE)
		{
		    n+=3;
		    this.dattribs.addValue(bvc.disttofrac_all_buy[i]);
		    this.dattribs.addValue(bvc.disttofrac_all_sell[i]);
		    this.dattribs.addValue(bvc.disttofrac_all[i]);
		}
		n+=2;
		this.dattribs.addValue(bvc.book_median_buy_tau[i]/c.p_buy);
		this.dattribs.addValue(bvc.book_median_sell_tau[i]/c.p_sell);
	    }
	}
	return n;
    }



    int setAntecedentAttribs(final String ticker,
			     final ModelParameters.ModelParametersData mparametersdata,
			     final CalculationData c,
			     final AttributesCount check_signatures)
    {
	int n=0;
	n+=setAttrInstrInfo(false,ticker,c.getTime());
	n+=setAttrCi(false,ticker,c);
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT21)
	{
	    n+=setAttrLiquidity21(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT20)
	{
	    n+=setAttrLiquidity20(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT19)
	{
	    n+=setAttrLiquidity19(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT18)
	{
	    n+=setAttrLiquidity18(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT17)
	{
	    n+=setAttrLiquidity17(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT16)
	{
	    n+=setAttrLiquidity16(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT15)
	{
	    n+=setAttrLiquidity15(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT13)
	{
	    n+=setAttrLiquidity13(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT12)
	{
	    n+=setAttrLiquidity12(false,ticker,mparametersdata.N_TLEV,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT11)
	{
	    n+=setAttrLiquidity11(false,ticker,mparametersdata.getLevelsIDs(),c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT10)
	{
	    n+=setAttrLiquidity10(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT9)
	{
	    n+=setAttrLiquidity9(false,ticker,mparametersdata.getLevelsIDs(),c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT9T)
	{
	    n+=setAttrLiquidity9T(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT8)
	{
	    n+=setAttrLiquidity8(false,ticker,mparametersdata.getLevelsIDs(),c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT7)
	{
	    n+=setAttrLiquidity7(false,ticker,mparametersdata.N_TLEV,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT6)
	{
	    n+=setAttrLiquidity6(false,ticker,mparametersdata.N_TLEV,c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT5)
	{
	    n+=setAttrLiquidity5(false,ticker,mparametersdata.getLevelsIDs(),c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT4)
	{
	    n+=setAttrLiquidity4(false,ticker,mparametersdata.getLevelsIDs(),c);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT3)
	{
	    n+=setAttrLiquidity3(false,ticker,mparametersdata.N_TLEV,c);
	}
	if(ATTRIB_LIQUIDITYDEFICIT2)
	{
	    n+=setAttrLiquidity2(false,ticker,mparametersdata.getLevelsIDs(),c);
	}
	if(ATTRIB_LIQUIDITYDEFICIT01)
	{
	    n+=setAttrLiquidityDeficit01(false,ticker,mparametersdata.getLevelsIDs(),c);
	}
	if(ATTRIB_BOOK_LIQUIDITYDEFICIT02)
	{
	    n+=setAttrBookLiquidityDeficit02(false,ticker,c);
	}
	if(ATTRIB_VIRTUAL_SHIFTS_TAU)
	{
	    n+=setVirtualShiftsTau(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_VIRTUAL_SHIFTS)
	{
	    n+=setVirtualShifts(false,ticker,c);
	}
	if(ATTRIB_VIRTUAL_SHIFTS2)
	{
	    n+=setVirtualShifts2(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_TR_DYNAMICS)
	{
	     n+=setTRDynamicsAttrTauChange(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_CRITICAL_EXPONENT_S)
	{
	    n+=setAttrCriticalExp(false,ticker,c);
	}
	if(ATTRIB_BOOK_PRESETLEVELS)
	{
	    n+=setAttrCurrent(false,ticker,mparametersdata.getLevelsIDs(),c);
	}
	if(ATTRIB_TRADE_TAU_CHANGE)
	{
	    n+=setTradeAttrTauChange(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_HCONTRIB)
	{
	    n+=setTradeHContrib(false,ticker,mparametersdata.tau_embedding,c);
	}
	if(ATTRIB_BOOK_TAU_CHANGE)
	{
	    n+=setBookAttrTauChange(false,ticker,mparametersdata.tau_embedding,c);
	}

	final int ns=this.sattribs.getNValues(),
	    nd=this.dattribs.getNValues(),
	    nn=this.nattribs.getNValues();
	if(n!=nd+nn+ns||
	   nd!=check_signatures.n_doubles||
	   nn!=check_signatures.n_nominals||
	   ns!=check_signatures.n_strings)
	{
	    throw new RuntimeException("This should never happen nd="+nd+
				       " check_signatures.n_doubles="+check_signatures.n_doubles+
				       " nn="+nn+" check_signatures.n_nominals="+
				       check_signatures.n_nominals+" ns="+
				       ns+" check_signatures.n_strings="+check_signatures.n_strings);
	}
	return n;
    }


    AttributesCount setAttributesNames(final String ticker,
				       final ModelParameters.ModelParametersData mparametersdata)
    {
	int n=0;
	n+=setAttrInstrInfo(true,ticker,0L);
	n+=setAttrCi(true,ticker,null);

	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT21)
	{
	    n+=setAttrLiquidity21(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT20)
	{
	    n+=setAttrLiquidity20(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT19)
	{
	    n+=setAttrLiquidity19(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT18)
	{
	    n+=setAttrLiquidity18(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT17)
	{
	    n+=setAttrLiquidity17(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT16)
	{
	    n+=setAttrLiquidity16(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT15)
	{
	    n+=setAttrLiquidity15(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT13)
	{
	    n+=setAttrLiquidity13(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT12)
	{
	    n+=setAttrLiquidity12(true,ticker,mparametersdata.N_TLEV,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT11)
	{
	    n+=setAttrLiquidity11(true,ticker,mparametersdata.getLevelsIDs(),null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT10)
	{
	    n+=setAttrLiquidity10(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT9)
	{
	    n+=setAttrLiquidity9(true,ticker,mparametersdata.getLevelsIDs(),null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT9T)
	{
	    n+=setAttrLiquidity9T(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT8)
	{
	    n+=setAttrLiquidity8(true,ticker,mparametersdata.getLevelsIDs(),null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT7)
	{
	    n+=setAttrLiquidity7(true,ticker,mparametersdata.N_TLEV,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT6)
	{
	    n+=setAttrLiquidity6(true,ticker,mparametersdata.N_TLEV,null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT5)
	{
	    n+=setAttrLiquidity5(true,ticker,mparametersdata.getLevelsIDs(),null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT4)
	{
	    n+=setAttrLiquidity4(true,ticker,mparametersdata.getLevelsIDs(),null);
	}
	if(ATTRIB_BOOKTRADES_LIQUIDITYDEFICIT3)
	{
	    n+=setAttrLiquidity3(true,ticker,mparametersdata.N_TLEV,null);
	}
	if(ATTRIB_LIQUIDITYDEFICIT2)
	{
	    n+=setAttrLiquidity2(true,ticker,mparametersdata.getLevelsIDs(),null);
	}
	if(ATTRIB_LIQUIDITYDEFICIT01)
	{
	    n+=setAttrLiquidityDeficit01(true,ticker,mparametersdata.getLevelsIDs(),null);
	}
	if(ATTRIB_BOOK_LIQUIDITYDEFICIT02)
	{
	    n+=setAttrBookLiquidityDeficit02(true,ticker,null);
	}
	if(ATTRIB_VIRTUAL_SHIFTS_TAU)
	{
	    n+=setVirtualShiftsTau(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_VIRTUAL_SHIFTS)
	{
	    n+=setVirtualShifts(true,ticker,null);
	}
	if(ATTRIB_VIRTUAL_SHIFTS2)
	{
	    n+=setVirtualShifts2(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_TR_DYNAMICS)
	{
	    n+=setTRDynamicsAttrTauChange(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_CRITICAL_EXPONENT_S)
	{
	    n+=setAttrCriticalExp(true,ticker,null);
	}
	if(ATTRIB_BOOK_PRESETLEVELS)
	{
	    n+=setAttrCurrent(true,ticker,mparametersdata.getLevelsIDs(),null);
	}
	if(ATTRIB_TRADE_TAU_CHANGE)
	{
	    n+=setTradeAttrTauChange(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_HCONTRIB)
	{
	    n+=setTradeHContrib(true,ticker,mparametersdata.tau_embedding,null);
	}
	if(ATTRIB_BOOK_TAU_CHANGE)
	{
	    n+=setBookAttrTauChange(true,ticker,mparametersdata.tau_embedding,null);
	}

	// should we add names to AttributesCount ?
	return new AttributesCount(this.dattribs_names.getNValues(),
				   this.sattribs_names.getNValues(),
				   this.nattribs_names.getNValues(),
				   n);
    }



    static double gv(final long v)
    {
	if(v<0)
	{
	    throw new IllegalArgumentException();
	}
	if(v<=0)
	{
	    return 0;
	}
	return Math.log(v);
    }

}
