package com.polytechnik.trading;
/*
 * (C) Vladislav Malyshkin 2012
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.UnitTests;
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.EVXData;
import com.polytechnik.utils.OrthogonalPolynomialsBasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.BasisFunctionsCalculatable;
import com.polytechnik.utils.PolynomialsRatio;
import com.polytechnik.utils.Polynomials;
import com.polytechnik.utils.OrthogonalPolynomialsMonomialsBasis;
import com.polytechnik.utils.VolMatrix;
import com.polytechnik.utils.SortArrays;
import com.polytechnik.utils.PolynomialRootsFinder;
import com.polytechnik.utils.Quadrature;
import com.polytechnik.realtimebook.*;

/** Rurik measure, spectral property and pseudotrades.
 *  @version $Id: DkQVM.java,v 1.677 2022/11/19 11:08:57 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class DkQVM<DTYPE> {

final DTYPE odata;
final DeprecatedHelperDataDirectAccessWithBasis<DTYPE> access;
final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M;
final EVXData VT,IT,RIT,ID,dpT,dpI,IBS,JTVS,JDP,IRBS,tsI,IT_sam,aI,p2I,pp2I,IrsellT,IrbuyT;
final WXpsi L,H,dpL,dpH,ssL,ssH;
final WXpsipsi0 iL,iH;
final WABpsi aL,aH,jL_sam,jL,jH,JbsL,JbsH,JdpL,JdpH,IRBSL,IRBSH,dpLv,dpHv,tsLv,tsHv,p2L,p2H,pp2L,pp2H;
final double sXi,sXdp,sXdpd,sXbs;
final double xVsc,iVsc,xVbc,iVbc,xVc,iVc;
final double jL_vsx,jL_vx,jL_VS,jL_VSp,jL_V,jL_Vp;
final double jL_VSd,jL_Vd,jL_VSdp,jL_Vdp;
final double jL_sam_XS,jL_sam_XSp,jL_sam_X,jL_sam_Xp,jL_sam_qXS,jL_sam_qXSp,jL_sam_qX,jL_sam_qXp;
final double jL_qP,jL_qPp;
final double jL_qPk,jL_qPpk,jL_qP0k,xjL_qP_jH,jL_qP_Hk;
final double jL_dpm,jL_dp,jL_dp2,jL_dp2e,jL_vsm,jL_vs,jL_vs2,jL_vs2e,
jL_sam_xsm,jL_sam_xs,jL_sam_xs2,jL_sam_xs2e,
jL_sam_xpm,jL_sam_xp,jL_sam_xp2,jL_sam_xp2e;
final double jL_sam_pt,jL_sam_pv,jL_sam_D,jL_sam_sigma2_D,
    jL_pt,jL_pv,jL_D,jL_sigma2_D,
    jL_sam_St,jL_sam_Sv,jL_sam_SD,jL_sam_sigma2_SD;
final WABpsi jrsellT,jrbuyT,jTL,jTH;
final WLHpsi jdp_pm,jdp_T,jvs_pm,jvs_T,jrvs_pm,jrvs_T;
final WLHTpsi jTrvs,jTvs,jTadp;
final double jL_dpplus_x,jL_dpminus_x,jL_dpplus_x2,jL_dpminus_x2,
    jL_vSplus_x,jL_vSminus_x,jL_vSplus_x2,jL_vSminus_x2;
final double jL_x_dpplus,jL_x_dpminus,jL_x_vSplus,jL_x_vSminus,jL_x_rvSplus,jL_x_rvSminus,p2L_jX,jL_x_dpt_low,jL_x_dpt_high,jL_x_i_low,jL_x_i_high,jL_x_vSt_low,jL_x_vSt_high;
final WdPsi dp2I,dp2Im;
final double jL_re_vs,jL_re_rvs,jL_re_dp,jL_rex_vs,jL_rex_rvs,jL_rex_dp,
    jL_re_sigma2_vs,jL_re_sigma2_rvs,jL_re_sigma2_dp,jL_rex_sigma2_vs,jL_rex_sigma2_rvs,jL_rex_sigma2_dp,
    jL_Wxx,jL_dTdMu,jL_dVdMu;
final WfPsi rpmf,rvst,vst,dpt,x_rpmf,x_rvst,x_vst,x_dpt;
final WjPsi jvSt,jdpt,jvStT,jdptT;
final WmuPsi wmudp,wmudpT,wmuvs,wmurvs,wmup,Rwmup,wmup_sam,wmus_sam,Rwmup_sam,wmuv,wmuv_sam,wmupVT_sam,wmusVT_sam,wmuPi;
final WDmuPsi wmuDp_sam,wmuDpW_sam,wmuDs_sam,wmuDsW_sam,wmuDrs_sam,wmuDrsW_sam;
final WpVPsi P2it_E,P2it_XX,P2tt_E,P2tt_XX,_P2tt_XX,P2ti_E,P2ti_XX,Pd2ti_E,Pd2ti_XX,Pd2tV_E,Pd2tV_XX,
    Pd2tT_E,Pd1tV_E,Pd1tT_E,Ad1tV_E,Ad1ti_E,pptt_E;
final WpPsi Pmu,RPmu,Pv,Pt,Vt,Vty,P2v,P2t,P1t,P1i,pVt,pVV,P1t_pV,P1t_pT;
final WqPsi wqmuPp,wqmuOp,wqmuPs,wqmuOs,wqmuP;
final WoPsi wo0p,wo1p,wo2p,wo0s,wo1s,wo2s;
final WkPsi p_psiIL,p_psiIH,j_psiIL,j_psiIH,pe_psiIL,pe_psiIH,je_psiIL,je_psiIH,p_IL_psiIH,dp_psiIL,dp_psiIH,pt_psiIL,pt_psiIH;
final WlPsi p_lamL_psiH,p_lamH_psiL,dp_lamL_psiH,dp_lamH_psiL,
    p_lam0_psiH,p_i0_psiH,p_mi0_psiH,p_lamn_psiH;
final WkePsi epit_psiIL,epit_psiIH,
    eptt_psiIL,eptt_psiIH,
    epii_psiIL,epii_psiIH,
    epti_psiIL,epti_psiIH,
    edpt_psiIL,edpt_psiIH,
    edpi_psiIL,edpi_psiIH,
    evst_psiIL,evst_psiIH,
    evsi_psiIL,evsi_psiIH,
    etst_psiIL,etst_psiIH,
    etsi_psiIL,etsi_psiIH,
    edp_psiIDL,edp_psiIDH;

WkfPsi pik_sam,pik,ptk_sam,ptk;

final WkkPsi Ekpit_psiIL,ekpit_psiIL,Rekpit_psiIL,ekpit_psiIH,
    ekptt_psiIL,ekptt_psiIH,
    ekpii_psiIL,ekpii_psiIH,
    ekpti_psiIL,ekpti_psiIH;
final WkkEV sdeit_pi_L,eit_pi_L,eit_pi_H,eit_pt_L,eit_pt_H;
final WkkBoundary sdbit_pi_L,bit_pi_L,bit_pi_H,bit_pt_L,bit_pt_H,biti_pi_L,biti_pi_H,biti_pt_L,biti_pt_H;
final WkkVolBoundary cit_pi_L,cit_pi_H;

final WVTPsi vt,vt_sam;
final WVTDPsi vtD;
final WVTmuPsi vtmu;
final PVTatX Pvt_i,Pvt_v,pvt_i,pvt_v,Pvte_i;
final ACorr acorr_dp_t,acorr_dp_i,acorr_dp_mu,acorr_dp_di;
final DDelaysBecauseOfSapsan mL_vtpv,mL_vtSvs,mL_Svtvs,mL_vtdp,mL_vtpt,
    mH_vtpv,mH_vtSvs,mH_Svtvs,mH_vtdp,mH_vtpt,mL_PL,mL_PH;

DkQVM(final int neq,final DTYPE odata)
{
    this.odata=odata;
    this.access=getAccess();
    this.M=getABasis();

    final int nmoms=2*neq-1+1+1+1;

    final double [] tmom=new double [nmoms],imom=new double [nmoms],dpmom=new double [nmoms],dp2mom=new double [nmoms],vsmom=new double [nmoms],tsmom=new double [nmoms],rvsmom=new double [nmoms],rtsmom=new double [nmoms],amom=new double [nmoms],
	tmom_sam=new double [nmoms],
	VSmom=new double [nmoms],TSmom=new double [nmoms];

    access.setM(dpmom,1,access.M_DP,odata,nmoms);
    access.setM(dp2mom,2,access.M_DP,odata,nmoms);
    access.setMT(tmom,nmoms);
    access.setM(imom,0,access.M_I,odata,nmoms);
    access.setM(amom,0,access.M_A,odata,nmoms);
    access.setM(tmom_sam,0,access.M_T,odata,nmoms);
    access.setM(vsmom,0,access.M_VS,odata,nmoms);
    access.setM(tsmom,0,access.M_TS,odata,nmoms);
    access.setM(rvsmom,0,access.M_RVS,odata,nmoms);
    access.setM(rtsmom,0,access.M_RTS,odata,nmoms);
    access.setFMoments(VSmom,vsmom,nmoms);
    access.setFMoments(TSmom,tsmom,nmoms);
    final double [] Pmom=new double [nmoms],P2mom=new double [nmoms],
	P1vmom=new double [nmoms],P1tmom=new double [nmoms],
	Vmom=new double [nmoms],pVmom=new double [nmoms];
    access.setFMoments(Vmom,imom,nmoms);
    final double [] QQV=M.B.getQQMatr(neq,Vmom),QQVS=M.B.getQQMatr(neq,VSmom);
    access.setFMoments(Pmom,dpmom,nmoms);

    access.setFMoments(P2mom,dp2mom,nmoms);
    access.setM(P1vmom,1,access.M_I,odata,nmoms);
    access.setM(P1tmom,1,access.M_T,odata,nmoms);
    access.setFMoments(pVmom,P1vmom,nmoms);
    final double [] mvsellmom=new double [nmoms],mvbuymom=new double [nmoms],
	momdpplus=new double [nmoms],momdpminus=new double [nmoms];
    access.setM(mvbuymom,0,access.M_TAUIBUY,odata,nmoms);
    access.setM(mvsellmom,0,access.M_TAUISELL,odata,nmoms);
    //System.err.println("mvbuymom="+java.util.Arrays.toString(mvbuymom)+" mvsellmom="+java.util.Arrays.toString(mvsellmom));
    access.setM(momdpplus,1,access.M_DPPLUS,odata,nmoms);
    access.setM(momdpminus,1,access.M_DPMINUS,odata,nmoms);
    for(int k=nmoms;--k>=0;){
	momdpminus[k]=-momdpminus[k];
    }
    final double [] QQmvsell=M.B.getQQMatr(neq,mvsellmom),QQmvbuy=M.B.getQQMatr(neq,mvbuymom),
	QQdpplus=M.B.getQQMatr(neq,momdpplus),QQdpminus=M.B.getQQMatr(neq,momdpminus);
    final double [] momrsell=new double [nmoms],momrbuy=new double [nmoms];
    access.setM(momrsell,0,access.M_ISELL,odata,nmoms);
    access.setM(momrbuy,0,access.M_IBUY,odata,nmoms);
    final double [] QQrsell=M.B.getQQMatr(neq,momrsell),QQrbuy=M.B.getQQMatr(neq,momrbuy);
    final double x0=getX0();

    final double [] QQt=M.B.getQQMatr(neq,tmom),QQi=M.B.getQQMatr(neq,imom),
	IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQt),
	QQa=M.B.getQQMatr(neq,amom),
	QQt_sam=M.B.getQQMatr(neq,tmom_sam),
	QQdp=M.B.getQQMatr(neq,dpmom),
	QQts=M.B.getQQMatr(neq,tsmom),
	QQvs=M.B.getQQMatr(neq,vsmom);
    this.IT=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQi,QQt,access,x0);
    this.VT=DeprecatedHelperEVXData.getEVXData(neq,getFmom(imom),getFmom(tmom),access,x0);
    this.RIT=DeprecatedHelperEVXData.getEVXData(neq,M.getRadauMomentsFromMoments(imom),M.getRadauMomentsFromMoments(tmom),access,x0);
    this.ID=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQi,getVolMatrix2(neq,QQt),access,x0);
    this.dpT=DeprecatedHelperEVXData.getEVXData(neq,dpmom,tmom,access,x0);
    this.aI=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQa,QQi,access,x0);

    this.L=new WXpsi(IT.psiL,IT.sL,imom,tmom,1,dpmom,vsmom,tsmom,P1vmom,P1tmom);
    this.H=new WXpsi(IT.psiH,IT.sH,imom,tmom,-1,dpmom,vsmom,tsmom,P1vmom,P1tmom);

    this.dpL=new WXpsi(dpT.psiL,dpT.sL,dpmom,tmom,1,dpmom,vsmom,tsmom,P1vmom,P1tmom);
    this.dpH=new WXpsi(dpT.psiH,dpT.sH,dpmom,tmom,-1,dpmom,vsmom,tsmom,P1vmom,P1tmom);


    this.iL=new WXpsipsi0(IT.psiL,IT.sL,imom,tmom,1);
    this.iH=new WXpsipsi0(IT.psiH,IT.sH,imom,tmom,-1);

    this.jL=new WABpsi(IT.psiL,IT.sL,QQi,QQt,neq);
    this.jL_vsx=jL.getD(QQvs,neq);
    this.jL_vx=jL.getD(QQi,neq);
    {
    final double [] j=jL.getDelta(vsmom,false);
    this.jL_re_vs=j[0];
    this.jL_re_sigma2_vs=j[1];
    }
    {
    final double [] j=jL.getDelta(vsmom,true);
    this.jL_rex_vs=j[0];
    this.jL_rex_sigma2_vs=j[1];
    }
    {
    final double [] j=jL.getDelta(rvsmom,false);
    this.jL_re_rvs=j[0];
    this.jL_re_sigma2_rvs=j[1];
    }
    {
    final double [] j=jL.getDelta(rvsmom,true);
    this.jL_rex_rvs=j[0];
    this.jL_rex_sigma2_rvs=j[1];
    }
    {
    final double [] j=jL.getDelta(dpmom,false);
    this.jL_re_dp=j[0];
    this.jL_re_sigma2_dp=j[1];
    }
    {
    final double [] j=jL.getDelta(dpmom,true);
    this.jL_rex_dp=j[0];
    this.jL_rex_sigma2_dp=j[1];
    }
    this.jL_Wxx=jL.getWX();
    jL_dTdMu=jL.getdTdMu();
    jL_dVdMu=jL.getdVdMu();

    this.jL_VS=TMatr.getAMatrA(this.jL.psiInd,QQVS,neq)/TMatr.getAMatrA(this.jL.psiInd,QQt,neq);
    this.jL_VSp=TMatr.getAMatrA(this.jL.psiIndp,QQVS,neq)/TMatr.getAMatrA(this.jL.psiIndp,QQt,neq);
    this.jL_V=TMatr.getAMatrA(this.jL.psiInd,QQV,neq)/TMatr.getAMatrA(this.jL.psiInd,QQt,neq);
    this.jL_Vp=TMatr.getAMatrA(this.jL.psiIndp,QQV,neq)/TMatr.getAMatrA(this.jL.psiIndp,QQt,neq);

    {
	final double [] j=jL.getVFXX(dpmom,WABpsi.MT_B);
	jL_dpm=j[0];
	jL_dp=j[1];
	jL_dp2=j[2];
	jL_dp2e=j[3];
    }
    {
	final double [] j=jL.getVFXX(vsmom,WABpsi.MT_B);
	jL_vsm=j[0];
	jL_vs=j[1];
	jL_vs2=j[2];
	jL_vs2e=j[3];
    }


    {
     final double [] j=jL.getLast2(jL.getF(vsmom));
     this.jL_VSd=j[0];
     this.jL_VSdp=j[1];
    }
    {
     final double [] j=jL.getLast2(jL.getF(imom));
     this.jL_Vd=j[0];
     this.jL_Vdp=j[1];
    }




    this.jH=new WABpsi(IT.psiH,IT.sH,QQi,QQt,neq);


    this.IBS=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQmvsell,QQmvbuy,access,x0);
    //System.err.println("IBS.psiH="+java.util.Arrays.toString(IBS.psiH)+" QQmvbuy="+java.util.Arrays.toString(QQmvbuy));

    this.ssL=new WXpsi(IBS.psiL,IBS.sL,mvsellmom,mvbuymom,1,dpmom,vsmom,tsmom,P1vmom,P1tmom);
    this.ssH=new WXpsi(IBS.psiH,IBS.sH,mvsellmom,mvbuymom,-1,dpmom,vsmom,tsmom,P1vmom,P1tmom);

    this.JbsL=new WABpsi(IBS.psiL,IBS.sL,QQmvsell,QQmvbuy,neq);
    this.JbsH=new WABpsi(IBS.psiH,IBS.sH,QQmvsell,QQmvbuy,neq);

    {
    IrsellT=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQrsell,QQt,access,x0);
    jrsellT=new WABpsi(IrsellT.psiL,IrsellT.sL,QQrsell,QQt,neq);
    }
    {
    IrbuyT=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQrbuy,QQt,access,x0);
    jrbuyT=new WABpsi(IrbuyT.psiL,IrbuyT.sL,QQrbuy,QQt,neq);
    }
    {
    JTVS=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQvs,QQt,access,x0);
    jTL=new WABpsi(JTVS.psiL,JTVS.sL,QQvs,QQt,neq);
    jTH=new WABpsi(JTVS.psiH,JTVS.sH,QQvs,QQt,neq);
    }

    this.JDP=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQdpplus,QQdpminus,access,x0);

    this.JdpL=new WABpsi(JDP.psiL,JDP.sL,QQdpplus,QQdpminus,neq);
    this.JdpH=new WABpsi(JDP.psiH,JDP.sH,QQdpplus,QQdpminus,neq);


    this.IRBS=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQrsell,QQrbuy,access,x0);
    this.IRBSL=new WABpsi(IRBS.psiL,IRBS.sL,QQrsell,QQrbuy,neq);
    this.IRBSH=new WABpsi(IRBS.psiH,1/IRBS.sH,QQrbuy,QQrsell,neq);

    this.dpI=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQdp,QQi,access,x0);
    this.dpLv=new WABpsi(dpI.psiL,dpI.sL,QQdp,QQi,neq);
    this.dpHv=new WABpsi(dpI.psiH,dpI.sH,QQdp,QQi,neq);

    this.tsI=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQts,QQi,access,x0);
    this.tsLv=new WABpsi(tsI.psiL,tsI.sL,QQts,QQi,neq);
    this.tsHv=new WABpsi(tsI.psiH,tsI.sH,QQts,QQi,neq);

    this.sXi=this.jL.fXx;
    this.sXdp=getDx(this.sXi,this.JdpH.Xx,this.JdpL.Xx,isEdgeAtRight());
    this.sXbs=getDx(this.sXi,this.JbsH.Xx,this.JbsL.Xx,isEdgeAtRight());
    this.sXdpd=getDx(this.sXi,this.dpH.Xx,this.dpL.Xx,isEdgeAtRight());

    if(true)
    {
	final Quadrature q=M.getGaussQuadratureFromMoments(neq,mvsellmom);
	sortArray(q.x,q.w);
	this.xVsc=q.x[0];
	this.iVsc=q.w[0];
    }
    if(true)
    {
	final Quadrature q=M.getGaussQuadratureFromMoments(neq,mvbuymom);
	sortArray(q.x,q.w);
	this.xVbc=q.x[0];
	this.iVbc=q.w[0];
    }
    if(true)
    {
	final Quadrature q=M.getGaussQuadratureFromMoments(neq,imom);
	sortArray(q.x,q.w);
	this.xVc=q.x[0];
	this.iVc=q.w[0];
    }

    this.IT_sam=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQi,QQt_sam,access,x0);
    this.jL_sam=new WABpsi(IT_sam.psiL,IT_sam.sL,QQi,QQt_sam,neq);
    final double [] xmom=new double [nmoms],xsmom=new double [nmoms];
    for(int k=nmoms;--k>=0;)
    {
	xmom[k]=imom[k]-this.jL_sam.lam*tmom_sam[k];
	xsmom[k]=vsmom[k]-this.jL_sam.lam*tsmom[k];
    }
    this.sxmom=xmom;
    this.sxsmom=xsmom;
    this.Stmom=tmom;
    this.Simom=imom;
    this.Spvmom=P1vmom;

    {
	final double [] j=jL_sam.getVFXX(xsmom,WABpsi.MT_MU);
	jL_sam_xsm=j[0];
	jL_sam_xs=j[1];
	jL_sam_xs2=j[2];
	jL_sam_xs2e=j[3];
    }
    {
     final double [] j=jL_sam.getLast2(jL_sam.getF(xsmom));
     this.jL_sam_XS=j[0];
     this.jL_sam_XSp=j[1];
    }
    {
     final double [] j=jL_sam.getLast2(jL_sam.getF(xmom));
     this.jL_sam_X=j[0];
     this.jL_sam_Xp=j[1];
    }
    {
     final double [] j=jL_sam.getLast2(jL_sam.getFq(xsmom));
     this.jL_sam_qXS=j[0];
     this.jL_sam_qXSp=j[1];
    }
    {
     final double [] j=jL_sam.getLast2(jL_sam.getFq(xmom));
     this.jL_sam_qX=j[0];
     this.jL_sam_qXp=j[1];
    }
    {
     final double [] j=jL.getLast2(jL.getFq(dpmom));
     this.jL_qP=j[0];
     this.jL_qPp=j[1];
    }
    final double [] ptmom=new double [nmoms],pvmom=new double [nmoms],
	p2tmom=new double [nmoms],p2vmom=new double [nmoms];
    access.setM(ptmom,1,access.M_T,odata,nmoms);
    access.setM(pvmom,1,access.M_I,odata,nmoms);
    access.setM(p2tmom,2,access.M_T,odata,nmoms);
    access.setM(p2vmom,2,access.M_I,odata,nmoms);
    {
	final double [] xpmom=new double [nmoms];
	for(int k=nmoms;--k>=0;)
	{
	    xpmom[k]=pvmom[k]-jL_sam.lam*ptmom[k];
	}
	final double [] j=jL_sam.getVFXX(xpmom,WABpsi.MT_MU);
	jL_sam_xpm=j[0];
	jL_sam_xp=j[1];
	jL_sam_xp2=j[2];
	jL_sam_xp2e=j[3];

	{
	final double dpt=jL_sam.getLast(jL_sam.getF(ptmom,false,WABpsi.MT_B)),
	    dpv=jL_sam.getLast(jL_sam.getF(pvmom,false,WABpsi.MT_A));
	jL_sam_pt=getPOffset()+dpt;
	jL_sam_pv=getPOffset()+dpv;
	final double dp2t=jL_sam.getLast(jL_sam.getF(p2tmom,false,WABpsi.MT_B)),
	    dp2v=jL_sam.getLast(jL_sam.getF(p2vmom,false,WABpsi.MT_A));
	jL_sam_D=dpv-dpt;
	jL_sam_sigma2_D=dp2v+dp2t-2*dpt*dpv-jL_sam_D*jL_sam_D;
	}
	{
	final double dpt=jL_sam.getLast(jL_sam.getF(tsmom,false,WABpsi.MT_B)),
	    dpv=jL_sam.getLast(jL_sam.getF(vsmom,false,WABpsi.MT_A));
	jL_sam_St=dpt;
	jL_sam_Sv=dpv;
	final double dp2t=Double.NaN,dp2v=Double.NaN;
	jL_sam_SD=dpv-dpt;
	jL_sam_sigma2_SD=dp2v+dp2t-2*dpt*dpv-jL_sam_SD*jL_sam_SD;
	}

	{
	final double dpt=jL.getLast(jL.getF(dpmom,true,WABpsi.MT_B)),
	    dpv=jL.getLast(jL.getF(pvmom,false,WABpsi.MT_A));
	jL_pt=getPOffset()+dpt;
	jL_pv=getPOffset()+dpv;
	final double dp2t=jL.getLast(jL.getF(dp2mom,true,WABpsi.MT_B)),
	    dp2v=jL.getLast(jL.getF(p2vmom,false,WABpsi.MT_A));
	jL_D=dpv-dpt;
	jL_sigma2_D=dp2v+dp2t-2*dpt*dpv-jL_D*jL_D;
	}
    }

    {
    final double [][] kkk=jL.getFqKK(dpmom);
    final double [] kkF=kkk[0],kkmu=kkk[1];
    this.jL_qPk=M.B.getD0(kkF,jL.fXx)/M.B.getD0(kkmu,jL.fXx);
    this.jL_qPpk=M.B.getD0(kkF,jL.fXxp)/M.B.getD0(kkmu,jL.fXxp);
    this.jL_qP0k=M.B.getD0(kkF,x0)/M.B.getD0(kkmu,x0);
    final int indH=DjQVM.getIndX0(jH.Xx,jL.fXx,isEdgeAtRight());
    if(indH>=0)
    {
	this.xjL_qP_jH=jH.Xx[indH];
	this.jL_qP_Hk=M.B.getD0(kkF,this.xjL_qP_jH)/M.B.getD0(kkmu,this.xjL_qP_jH);
    }
    else{xjL_qP_jH=jL_qP_Hk=Double.NaN;}

    }

    jdp_pm=new WLHpsi(this.jL,momdpplus,momdpminus);
    jdp_T=new WLHpsi(this.jL,dpmom,tmom);
    jvs_pm=new WLHpsi(this.jL,mvsellmom,mvbuymom);
    jvs_T=new WLHpsi(this.jL,vsmom,tmom);
    jrvs_pm=new WLHpsi(this.jL,momrsell,momrbuy);
    jrvs_T=new WLHpsi(this.jL,rvsmom,tmom);

    final double [] momSv=new double [nmoms];
    for(int k=0;k<nmoms;k++){momSv[k]=mvsellmom[k]+mvbuymom[k];}

    jTrvs=new WLHTpsi(neq,imom,tmom,momrsell,momrbuy);
    {
    jTvs=new WLHTpsi(neq,momSv,tmom,mvsellmom,mvbuymom);
    }
    {
    final double [] moma=new double [nmoms];
    for(int k=0;k<nmoms;k++){moma[k]=momdpplus[k]+momdpminus[k];}
    jTadp=new WLHTpsi(neq,moma,tmom,momdpplus,momdpminus);
    }

    {
    final double [][] j=jL.getFxFx2(momdpplus);
    jL_dpplus_x=jL.getLast(j[0]);
    jL_dpplus_x2=jL.getLast(j[1]);
    }
    {
    final double [][] j=jL.getFxFx2(momdpminus);
    jL_dpminus_x=jL.getLast(j[0]);
    jL_dpminus_x2=jL.getLast(j[1]);
    }


    {
    final double [][] j=jL.getFxFx2(mvsellmom);
    jL_vSplus_x=jL.getLast(j[0]);
    jL_vSplus_x2=jL.getLast(j[1]);
    }
    {
    final double [][] j=jL.getFxFx2(mvbuymom);
    jL_vSminus_x=jL.getLast(j[0]);
    jL_vSminus_x2=jL.getLast(j[1]);
    }
    {
    final double [] j=jL.getXMM(mvsellmom,mvbuymom);
    jL_x_vSplus=j[0];
    jL_x_vSminus=j[1];
    }
    {
    final double [] j=jL.getXMM(momrsell,momrbuy);
    jL_x_rvSplus=j[0];
    jL_x_rvSminus=j[1];
    }
    {
    final double [] j=jL.getXMM(momdpplus,momdpminus);
    jL_x_dpplus=j[0];
    jL_x_dpminus=j[1];
    }
    {
    final double [] j=jL.getXMM(dpmom,tmom);
    jL_x_dpt_low=j[0];
    jL_x_dpt_high=j[1];
    }
    {
    final double [] j=jL.getXMM(imom,tmom);
    jL_x_i_low=j[0];
    jL_x_i_high=j[1];
    }
    {
    final double [] j=jL.getXMM(vsmom,tmom);
    jL_x_vSt_low=j[0];
    jL_x_vSt_high=j[1];
    }

    this.aH=new WABpsi(aI.psiH,aI.sH,QQa,QQi,neq);
    this.aL=new WABpsi(aI.psiL,aI.sL,QQa,QQi,neq);

    {
    final double [] QQp2=TMatr.mulMMM(neq,QQdp,IQQt,QQdp);
    this.p2I=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQp2,QQi,access,x0);
    this.p2L=new WABpsi(p2I.psiL,p2I.sL,QQp2,QQi,neq);
    this.p2L_jX=p2L.getLast(p2L.getF(dpmom,false,WABpsi.MT_B));
    this.p2H=new WABpsi(p2I.psiH,p2I.sH,QQp2,QQi,neq);
    }
    {
    final double [] QQp2=TMatr.mulMatrices(neq,new double [][]{QQdp,IQQt,QQi,IQQt,QQdp});
    this.pp2I=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQp2,QQt,access,x0);
    this.pp2L=new WABpsi(pp2I.psiL,pp2I.sL,QQp2,QQt,neq);
    this.pp2H=new WABpsi(pp2I.psiH,pp2I.sH,QQp2,QQt,neq);
    }

    this.dp2I=new WdPsi(neq,dpmom,imom,tmom,false);
    this.dp2Im=new WdPsi(neq,dpmom,imom,tmom,true);
    this.rpmf=new WfPsi(neq,momrsell,momrbuy,tmom);
    this.rvst=new WfPsi(neq,rvsmom,tmom,tmom);
    this.vst=new WfPsi(neq,vsmom,tmom,tmom);
    this.dpt=new WfPsi(neq,dpmom,tmom,tmom);

    this.x_rpmf=new WfPsi(neq,momrsell,momrbuy,tmom,this.access,x0);
    this.x_rvst=new WfPsi(neq,rvsmom,tmom,tmom,this.access,x0);
    this.x_vst=new WfPsi(neq,vsmom,tmom,tmom,this.access,x0);
    this.x_dpt=new WfPsi(neq,dpmom,tmom,tmom,this.access,x0);

    this.jvSt=new WjPsi(neq,vsmom,imom,tmom);
    this.jdpt=new WjPsi(neq,dpmom,imom,tmom);
    this.jvStT=new WjPsi(neq,M.getRadauMomentsFromMoments(vsmom),M.getRadauMomentsFromMoments(imom),M.getRadauMomentsFromMoments(tmom));
    this.jdptT=new WjPsi(neq,M.getRadauMomentsFromMoments(dpmom),M.getRadauMomentsFromMoments(imom),M.getRadauMomentsFromMoments(tmom));

    this.wmudp=new WmuPsi(neq,imom,tmom,dpmom,null);
    this.wmuPi=new WmuPsi(neq,imom,tmom,pvmom,null);
    this.wmudpT=new WmuPsi(neq,M.getRadauMomentsFromMoments(imom),M.getRadauMomentsFromMoments(tmom),M.getRadauMomentsFromMoments(dpmom),null);
    this.wmuvs=new WmuPsi(neq,imom,tmom,vsmom,null);
    this.wmurvs=new WmuPsi(neq,imom,tmom,rvsmom,null);



    final double [] pxmom=new double [nmoms];
    for(int k=nmoms;--k>=0;){pxmom[k]=pvmom[k]-this.jL_sam.lam*ptmom[k];}
    // OK, we now have xmom,pxmom
    this.Pmu=new WpPsi(neq-1,pxmom,xmom,pvmom,imom);
    {
	final double [] Rpxmom=new double [nmoms],Rxmom=new double [nmoms];
	for(int k=nmoms;--k>=0;){
	    Rpxmom[k]=pvmom[k]-this.RIT.sL*ptmom[k];
	    Rxmom[k]=imom[k]-this.RIT.sL*tmom[k];
	}
	this.RPmu=new WpPsi(neq-1,M.getRadauMomentsFromMoments(Rpxmom),M.getRadauMomentsFromMoments(Rxmom),M.getRadauMomentsFromMoments(pvmom),M.getRadauMomentsFromMoments(imom));
    }
    this.Pv=new WpPsi(neq-1,pvmom,imom,ptmom,tmom);
    this.Pt=new WpPsi(neq-1,ptmom,tmom,pvmom,imom);

    this.P1t_pV=new WpPsi(neq,Pmom,tmom,getFmom(pvmom),getFmom(imom));
    this.P1t_pT=new WpPsi(neq,Pmom,tmom,getFmom(Pmom),getFmom(tmom));


    for(int k=Vmom.length;--k>=0;){Vmom[k]=-Vmom[k];} // compatibility with BiQVM
    this.pVV=new WpPsi(neq,TMatr.getVectorScaled(pVmom,-1),Vmom,Pmom,tmom);
    this.pVt=new WpPsi(neq,pVmom,tmom,Pmom,tmom);
    this.Vt=new WpPsi(neq,Vmom,tmom,Pmom,tmom);
    this.Vty=new WpPsi(neq,Vmom,tmom,ptmom,tmom);
    this.P2v=new WpPsi(neq,p2vmom,tmom,ptmom,tmom);
    this.P2t=new WpPsi(neq,p2tmom,tmom,ptmom,tmom);
    this.P1t=new WpPsi(neq,Pmom,tmom,pvmom,imom);
    this.P1i=new WpPsi(neq,pvmom,imom,ptmom,tmom);

    this.P2it_E=new WpVPsi_EDPsi(neq,p2vmom,pvmom,imom,tmom);
    this.P2it_XX=new WpVPsi_XX(neq,p2vmom,pvmom,imom,tmom);
    this.P2tt_E=new WpVPsi_EDPsi(neq,p2tmom,ptmom,tmom,tmom);
    this.P2tt_XX=new WpVPsi_XX(neq,p2tmom,ptmom,tmom,tmom);
    this._P2tt_XX=new WpVPsi_XX(neq,P2mom,Pmom,tmom,tmom);
    this.P2ti_E=new WpVPsi_EDPsi(neq,p2tmom,ptmom,tmom,imom);
    this.P2ti_XX=new WpVPsi_XX(neq,p2tmom,ptmom,tmom,imom);
    this.Pd2ti_E=new WpVPsi_EDPsi(neq,P2mom,Pmom,tmom,imom);
    this.Pd2ti_XX=new WpVPsi_XX(neq,P2mom,Pmom,tmom,imom);
    this.Pd2tV_E=new WpVPsi_EDPsi(neq,P2mom,Pmom,tmom,Vmom);
    this.Pd2tV_XX=new WpVPsi_XX(neq,P2mom,Pmom,tmom,Vmom);
    final double [] Tmom=new double [nmoms],Amom=new double [nmoms];
    access.setFMoments(Tmom,tmom,nmoms);
    access.setFMoments(Amom,amom,nmoms);
    for(int k=Tmom.length;--k>=0;){Tmom[k]=-Tmom[k];Amom[k]=-Amom[k];}
    Pd2tT_E=new WpVPsi_EDPsi(neq,P2mom,Pmom,tmom,Tmom);
    Pd1tV_E=new WpVPsi_EDPsi(neq,Pmom,Pmom,tmom,Vmom);
    Pd1tT_E=new WpVPsi_EDPsi(neq,Pmom,Pmom,tmom,Tmom);
    Ad1tV_E=new WpVPsi_EDPsi(neq,Amom,Pmom,tmom,Vmom);
    Ad1ti_E=new WpVPsi_EDPsi(neq,Amom,Pmom,tmom,imom);
    pptt_E=new WpVPsi_EDPsi(neq,Vmom,Pmom,tmom,Tmom);


    vt=new WVTPsi(neq,pvmom,imom,Pmom,tmom);
    vt_sam=new WVTPsi(neq,pvmom,imom,ptmom,tmom_sam);
    vtD=new WVTDPsi(neq,pvmom,imom,tmom);
    vtmu=new WVTmuPsi(neq,pvmom,imom,ptmom,tmom_sam);


    this.wqmuPp=new WqPsi(neq,pvmom,imom,ptmom,tmom_sam,WqPsi.BASIS_P);
    this.wqmuOp=new WqPsi(neq,pvmom,imom,ptmom,tmom_sam,WqPsi.BASIS_O);
    this.wqmuP=new WqPsi(neq,pvmom,imom,ptmom,tmom,WqPsi.BASIS_P);

    this.wqmuPs=new WqPsi(neq,VSmom,imom,TSmom,tmom_sam,WqPsi.BASIS_P);
    this.wqmuOs=new WqPsi(neq,VSmom,imom,TSmom,tmom_sam,WqPsi.BASIS_O);

    this.Rwmup=new WmuPsi(neq,M.getRadauMomentsFromMoments(imom),M.getRadauMomentsFromMoments(tmom),M.getRadauMomentsFromMoments(pvmom),M.getRadauMomentsFromMoments(ptmom));
    this.wmup=new WmuPsi(neq,imom,tmom,pvmom,ptmom);
    this.wmup_sam=new WmuPsi(neq,imom,tmom_sam,pvmom,ptmom);
    this.wmus_sam=new WmuPsi(neq,imom,tmom_sam,vsmom,tsmom);
    this.wmupVT_sam=new WmuPsi(neq,getFmom(imom),getFmom(tmom_sam),getFmom(pvmom),getFmom(ptmom));
    this.wmusVT_sam=new WmuPsi(neq,getFmom(imom),getFmom(tmom_sam),getFmom(vsmom),getFmom(tsmom));
    this.Rwmup_sam=new WmuPsi(neq,M.getRadauMomentsFromMoments(imom),M.getRadauMomentsFromMoments(tmom_sam),M.getRadauMomentsFromMoments(pvmom),M.getRadauMomentsFromMoments(ptmom));
    this.wmuv=new WmuPsi(neq,imom,tmom,VSmom,TSmom);
    this.wmuv_sam=new WmuPsi(neq,imom,tmom_sam,VSmom,TSmom);

    this.wmuDp_sam=new WDmuPsi(neq,imom,tmom_sam,pvmom,ptmom,QuadEV.IND_CLOSE);
    this.wmuDs_sam=new WDmuPsi(neq,imom,tmom_sam,vsmom,tsmom,QuadEV.IND_CLOSE);
    this.wmuDrs_sam=new WDmuPsi(neq,imom,tmom_sam,rvsmom,rtsmom,QuadEV.IND_CLOSE);
    this.wmuDpW_sam=new WDmuPsi(neq,imom,tmom_sam,pvmom,ptmom,QuadEV.IND_WMAX);
    this.wmuDsW_sam=new WDmuPsi(neq,imom,tmom_sam,vsmom,tsmom,QuadEV.IND_WMAX);
    this.wmuDrsW_sam=new WDmuPsi(neq,imom,tmom_sam,rvsmom,rtsmom,QuadEV.IND_WMAX);
    this.acorr_dp_t=ACorr.getACorr_F(neq,dpmom,tmom,tmom,M,x0);
    this.acorr_dp_i=ACorr.getACorr_F(neq,dpmom,imom,tmom,M,x0);
    final double [] Dimom=new double [nmoms];access.setDFMoments(Dimom,imom,nmoms);TMatr.scaleVectorInPlace(Dimom,-1);
    this.acorr_dp_di=ACorr.getACorr_F(neq,dpmom,Dimom,tmom,M,x0);
    final double [] xmom_nosam=new double [imom.length];
    for(int k=xmom_nosam.length;--k>=0;){
	xmom_nosam[k]=imom[k]-this.jL.lam*tmom[k];
    }
    this.acorr_dp_mu=ACorr.getACorr_F(neq,dpmom,xmom_nosam,tmom,M,x0);


    this.wo0p=new WoPsi(neq,0,pvmom,imom,ptmom,tmom_sam);
    this.wo1p=new WoPsi(neq,1,pvmom,imom,ptmom,tmom_sam);
    this.wo2p=new WoPsi(neq,neq>2?2:1,pvmom,imom,ptmom,tmom_sam);
    this.wo0s=new WoPsi(neq,0,VSmom,imom,TSmom,tmom_sam);
    this.wo1s=new WoPsi(neq,1,VSmom,imom,TSmom,tmom_sam);
    this.wo2s=new WoPsi(neq,neq>2?2:1,VSmom,imom,TSmom,tmom_sam);



    this.dp_psiIL=new WkPsi(IT.psiL,IT.sL,dpmom,dpmom,imom,null,null,tmom);
    this.dp_psiIH=new WkPsi(IT.psiH,IT.sH,dpmom,dpmom,imom,null,null,tmom);
    this.pt_psiIL=new WkPsi(IT.psiL,IT.sL,ptmom,ptmom,imom,null,null,tmom);
    this.pt_psiIH=new WkPsi(IT.psiH,IT.sH,ptmom,ptmom,imom,null,null,tmom);


    this.p_psiIL=new WkPsi(IT_sam.psiL,IT_sam.sL,p2vmom,pvmom,imom,p2tmom,ptmom,tmom_sam);
    this.p_psiIH=new WkPsi(IT_sam.psiH,IT_sam.sH,p2vmom,pvmom,imom,p2tmom,ptmom,tmom_sam);
    this.j_psiIL=new WkPsi(IT_sam.psiL,IT_sam.sL,p2vmom,VSmom,imom,p2tmom,TSmom,tmom_sam);
    this.j_psiIH=new WkPsi(IT_sam.psiH,IT_sam.sH,p2vmom,VSmom,imom,p2tmom,TSmom,tmom_sam);
    this.pe_psiIL=new WkPsi(IT.psiL,IT.sL,p2vmom,pvmom,imom,p2tmom,ptmom,tmom);
    this.pe_psiIH=new WkPsi(IT.psiH,IT.sH,p2vmom,pvmom,imom,p2tmom,ptmom,tmom);
    this.je_psiIL=new WkPsi(IT.psiL,IT.sL,p2vmom,VSmom,imom,p2tmom,TSmom,tmom);
    this.je_psiIH=new WkPsi(IT.psiH,IT.sH,p2vmom,VSmom,imom,p2tmom,TSmom,tmom);

    this.p_IL_psiIH=new WkPsi(IT_sam.psiH,IT_sam.sL,p2vmom,pvmom,imom,p2tmom,ptmom,tmom_sam);


    this.p_lamL_psiH=new WlPsi(IT_sam.psiH,IT_sam.sL,pvmom,imom,ptmom,tmom_sam);
    this.p_lamH_psiL=new WlPsi(IT_sam.psiL,IT_sam.sH,pvmom,imom,ptmom,tmom_sam);
    this.dp_lamL_psiH=new WlPsi(IT_sam.psiH,IT_sam.sL,dpmom,imom,null,tmom_sam);
    this.dp_lamH_psiL=new WlPsi(IT_sam.psiL,IT_sam.sH,dpmom,imom,null,tmom_sam);

    p_lam0_psiH=new WlPsi(IT_sam.psiH,0,pvmom,imom,ptmom,tmom_sam);
    p_i0_psiH=new WlPsi(IT_sam.psiH,IT_sam.s0,pvmom,imom,ptmom,tmom_sam);
    p_mi0_psiH=new WlPsi(IT_sam.psiH,IT_sam.sH-IT_sam.s0,pvmom,imom,ptmom,tmom_sam);
    p_lamn_psiH=new WlPsi(IT_sam.psiH,IT_sam.ev[IT_sam.neq>=2?IT_sam.neq-2:0],pvmom,imom,ptmom,tmom_sam);

    this.epit_psiIL=new WkePsi(IT.psiL,tmom,imom,pvmom,p2vmom);
    this.epit_psiIH=new WkePsi(IT.psiH,tmom,imom,pvmom,p2vmom);
    this.eptt_psiIL=new WkePsi(IT.psiL,tmom_sam,tmom_sam,ptmom,p2tmom);
    this.eptt_psiIH=new WkePsi(IT.psiH,tmom_sam,tmom_sam,ptmom,p2tmom);
    this.epii_psiIL=new WkePsi(IT.psiL,imom,imom,pvmom,p2vmom);
    this.epii_psiIH=new WkePsi(IT.psiH,imom,imom,pvmom,p2vmom);
    this.epti_psiIL=new WkePsi(IT_sam.psiL,imom,tmom_sam,ptmom,p2tmom);
    this.epti_psiIH=new WkePsi(IT_sam.psiH,imom,tmom_sam,ptmom,p2tmom);

    this.edpt_psiIL=new WkePsi(IT.psiL,tmom,tmom,dpmom,null);
    this.edpt_psiIH=new WkePsi(IT.psiH,tmom,tmom,dpmom,null);
    this.edpi_psiIL=new WkePsi(IT.psiL,imom,imom,dpmom,null);
    this.edpi_psiIH=new WkePsi(IT.psiH,imom,imom,dpmom,null);

    evst_psiIL=new WkePsi(IT.psiL,tmom,imom,vsmom,null);
    evst_psiIH=new WkePsi(IT.psiH,tmom,imom,vsmom,null);
    evsi_psiIL=new WkePsi(IT.psiL,imom,imom,vsmom,null);
    evsi_psiIH=new WkePsi(IT.psiH,imom,imom,vsmom,null);
    etst_psiIL=new WkePsi(IT.psiL,tmom,imom,tsmom,null);
    etst_psiIH=new WkePsi(IT.psiH,tmom,imom,tsmom,null);
    etsi_psiIL=new WkePsi(IT.psiL,imom,imom,tsmom,null);
    etsi_psiIH=new WkePsi(IT.psiH,imom,imom,tsmom,null);

    this.Ekpit_psiIL=new WkkPsi(VT.psiL,getFmom(tmom),getFmom(imom),getFmom(pvmom),getFmom(p2vmom));
    this.Rekpit_psiIL=new WkkPsi(RIT.psiL,M.getRadauMomentsFromMoments(tmom),M.getRadauMomentsFromMoments(imom),M.getRadauMomentsFromMoments(pvmom),M.getRadauMomentsFromMoments(p2vmom));
    this.ekpit_psiIL=new WkkPsi(IT.psiL,tmom,imom,pvmom,p2vmom);
    this.ekpit_psiIH=new WkkPsi(IT.psiH,tmom,imom,pvmom,p2vmom);
    this.ekptt_psiIL=new WkkPsi(IT.psiL,tmom_sam,tmom_sam,ptmom,p2tmom);
    this.ekptt_psiIH=new WkkPsi(IT.psiH,tmom_sam,tmom_sam,ptmom,p2tmom);
    this.ekpii_psiIL=new WkkPsi(IT.psiL,imom,imom,pvmom,p2vmom);
    this.ekpii_psiIH=new WkkPsi(IT.psiH,imom,imom,pvmom,p2vmom);
    this.ekpti_psiIL=new WkkPsi(IT_sam.psiL,imom,tmom_sam,ptmom,p2tmom);
    this.ekpti_psiIH=new WkkPsi(IT_sam.psiH,imom,tmom_sam,ptmom,p2tmom);

    sdeit_pi_L=new WkkEV(neq,imom,tmom,imom,pvmom,p2vmom,DelaysBecauseOfSapsanEV.LH_LOW,WkkEV.PERTRt,VOLATILITY_EDPsiX);
    eit_pi_L=new WkkEV(neq,imom,tmom,imom,pvmom,p2vmom,DelaysBecauseOfSapsanEV.LH_LOW,WkkEV.PERTRt,VOLATILITY_EDPsi);
    eit_pi_H=new WkkEV(neq,imom,tmom,imom,pvmom,p2vmom,DelaysBecauseOfSapsanEV.LH_HIGH,WkkEV.PERTRt,VOLATILITY_EDPsi);
    eit_pt_L=new WkkEV(neq,imom,tmom_sam,tmom_sam,ptmom,p2tmom,DelaysBecauseOfSapsanEV.LH_LOW,WkkEV.PERTRt,VOLATILITY_EDPsi);
    eit_pt_H=new WkkEV(neq,imom,tmom_sam,tmom_sam,ptmom,p2tmom,DelaysBecauseOfSapsanEV.LH_HIGH,WkkEV.PERTRt,VOLATILITY_EDPsi);
    sdbit_pi_L=new WkkBoundary(neq,imom,tmom,imom,pvmom,p2vmom,DelaysBecauseOfSapsanEV.LH_LOW,WkkEV.PERTRt,VOLATILITY_EDPsiX);
    bit_pi_L=new WkkBoundary(neq,imom,tmom,imom,pvmom,p2vmom,DelaysBecauseOfSapsanEV.LH_LOW,WkkEV.PERTRt,VOLATILITY_EDPsi);
    bit_pi_H=new WkkBoundary(neq,imom,tmom,imom,pvmom,p2vmom,DelaysBecauseOfSapsanEV.LH_HIGH,WkkEV.PERTRt,VOLATILITY_EDPsi);
    bit_pt_L=new WkkBoundary(neq,imom,tmom_sam,tmom_sam,ptmom,p2tmom,DelaysBecauseOfSapsanEV.LH_LOW,WkkEV.PERTRt,VOLATILITY_EDPsi);
    bit_pt_H=new WkkBoundary(neq,imom,tmom_sam,tmom_sam,ptmom,p2tmom,DelaysBecauseOfSapsanEV.LH_HIGH,WkkEV.PERTRt,VOLATILITY_EDPsi);
    biti_pi_L=new WkkBoundary(neq,imom,tmom,imom,pvmom,p2vmom,DelaysBecauseOfSapsanEV.LH_LOW,WkkEV.PERTRi,VOLATILITY_EDPsi);
    biti_pi_H=new WkkBoundary(neq,imom,tmom,imom,pvmom,p2vmom,DelaysBecauseOfSapsanEV.LH_HIGH,WkkEV.PERTRi,VOLATILITY_EDPsi);
    biti_pt_L=new WkkBoundary(neq,imom,tmom_sam,tmom_sam,ptmom,p2tmom,DelaysBecauseOfSapsanEV.LH_LOW,WkkEV.PERTRi,VOLATILITY_EDPsi);
    biti_pt_H=new WkkBoundary(neq,imom,tmom_sam,tmom_sam,ptmom,p2tmom,DelaysBecauseOfSapsanEV.LH_HIGH,WkkEV.PERTRi,VOLATILITY_EDPsi);

    cit_pi_L=new WkkVolBoundary(neq,imom,tmom,imom,pvmom,p2vmom,DelaysBecauseOfSapsanEV.LH_LOW);
    cit_pi_H=new WkkVolBoundary(neq,imom,tmom,imom,pvmom,p2vmom,DelaysBecauseOfSapsanEV.LH_HIGH);


    this.edp_psiIDL=new WkePsi(ID.psiL,tmom,tmom,dpmom,null);
    this.edp_psiIDH=new WkePsi(ID.psiH,tmom,tmom,dpmom,null);

    this.pik_sam=new WkfPsi(neq,imom,tmom_sam,pvmom,ptmom,dpmom);
    this.pik=new WkfPsi(neq,imom,tmom,pvmom,ptmom,dpmom);
    this.ptk_sam=new WkfPsi(neq,tmom_sam,imom,ptmom,pvmom,dpmom);
    this.ptk=new WkfPsi(neq,tmom,imom,ptmom,pvmom,dpmom);

    this.Pvt_i=new PVTatX(neq,this.wmup_sam.jL.fXx,tmom_sam,
			  getFmom(pvmom),getFmom(imom),
			  getFmom(ptmom),getFmom(tmom_sam));
    this.Pvte_i=new PVTatX(neq,this.wmup.jL.fXx,tmom,
			   getFmom(pvmom),getFmom(imom),
			   getFmom(ptmom),getFmom(tmom_sam));
    this.Pvt_v=new PVTatX(neq,this.vtmu.fXx,tmom_sam,
			  getFmom(pvmom),getFmom(imom),
			  getFmom(ptmom),getFmom(tmom_sam));
    this.pvt_i=new PVTatX(neq,this.wmup_sam.jL.fXx,tmom_sam,pvmom,imom,ptmom,tmom_sam);
    this.pvt_v=new PVTatX(neq,this.vtmu.fXx,tmom_sam,pvmom,imom,ptmom,tmom_sam);


    mL_PL=new DDelaysBecauseOfSapsan(neq,Pmom,tmom,imom,DelaysBecauseOfSapsanEV.LH_LOW);
    mL_PH=new DDelaysBecauseOfSapsan(neq,Pmom,tmom,imom,DelaysBecauseOfSapsanEV.LH_HIGH);

    mL_vtpv=new DDelaysBecauseOfSapsan(neq,imom,tmom,pvmom,DelaysBecauseOfSapsanEV.LH_LOW);
    mL_vtpt=new DDelaysBecauseOfSapsan(neq,imom,tmom_sam,ptmom,DelaysBecauseOfSapsanEV.LH_LOW);
    mL_vtSvs=new DDelaysBecauseOfSapsan(neq,imom,tmom,vsmom,DelaysBecauseOfSapsanEV.LH_LOW);
    mL_Svtvs=new DDelaysBecauseOfSapsan(neq,momSv,tmom,rvsmom,DelaysBecauseOfSapsanEV.LH_LOW);
    mL_vtdp=new DDelaysBecauseOfSapsan(neq,imom,tmom,dpmom,DelaysBecauseOfSapsanEV.LH_LOW);
    mH_vtpv=new DDelaysBecauseOfSapsan(neq,imom,tmom,pvmom,DelaysBecauseOfSapsanEV.LH_HIGH);
    mH_vtpt=new DDelaysBecauseOfSapsan(neq,imom,tmom_sam,ptmom,DelaysBecauseOfSapsanEV.LH_HIGH);
    mH_vtSvs=new DDelaysBecauseOfSapsan(neq,imom,tmom,vsmom,DelaysBecauseOfSapsanEV.LH_HIGH);
    mH_Svtvs=new DDelaysBecauseOfSapsan(neq,momSv,tmom,rvsmom,DelaysBecauseOfSapsanEV.LH_HIGH);
    mH_vtdp=new DDelaysBecauseOfSapsan(neq,imom,tmom,dpmom,DelaysBecauseOfSapsanEV.LH_HIGH);
}
final double [] sxmom,sxsmom,Stmom,Simom,Spvmom;

boolean isDiagQQt(){return true;}
double getDS()
{
    return Double.NaN; // FIXME
}
/** Calculates DT QQ D + 1/2(DT QQ + QQ D)+ 1/4 QQ
 */
double [] getVolMatr(final int n,final double [] QQ)
{
    final double [] D=getDMatr(n);
    final double [] M=new double [n*n],tmp=new double [n*n];
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,QQ,n,0,D,n,0);
    DeprecatedHelperTMatr.multiply_MatrTrans_x_Matr(n,M,D,n,0,tmp,n,0);

    for(int k=0;k<n;k++)
    {
	for(int l=0;l<n;l++)
	{
	    M[k*n+l]+=(0.5*(tmp[k*n+l]+tmp[l*n+k])+0.25*QQ[k*n+l]);
	}
    }
    return M;
}
/** To test getVolMatr.
 */
double [] getVolMatrix2(final int n,final double [] QQ)
{
    return getDTxAxD(n,getED(n),QQ,new double [n*n]);
}
/** To test d/dt.
 */
double [] getDiffMatrix2(final int n,final double [] QQ)
{
    final double [] M=new double [n*n];
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,M,QQ,n,0,getED(n),n,0);
    TMatr.scaleVectorInPlace(TMatr.selectHermitianPartInPlace(n,M),2); // factor of 2 is here
    return M;
}

/** Calculate d/dt matrix elements.
 */
double [] getED(final int n)
{
    final double [] D=new double [n*n];
    for(int k=0;k<n;k++)
    {
	final double [] p=new double [k+1];
	p[k]=1;
	final double [] d=getEDPsi(p);
	for(int j=d.length;--j>=0;)
	{
	    D[j*n+k]=d[j];
	}
    }
    return D;
}
/** Calculate (x-x0) matrix elements.
 *  Returns [n+1 x n] matrix
 *  transforming |psi> (d=n) to (x-x0) |psi> (d=n+1).
 *  The leading dimension of returned matrix is n.
 */
double [] getMulX(final int n,final double x0)
{
    final double [] D=new double [n*(n+1)];
    final BasisFunctionsCalculatable B=getBasisPolynomials();
    for(int k=0;k<n;k++)
    {
	final double [] p=new double [k+1];p[k]=1;
	final double [] xp=B.getaxbP(p,1,-x0);
	for(int j=k+1;j>=0;j--)
	{
	    D[j*(n)+k]=xp[j];
	}
    }
    return D;
}

/** Calculate 1/(x-x0) matrix elements asumin the remainder is=0
 *  Returns [n-1 x n] matrix
 *  transforming |psi> (d=n) to 1/(x-x0) |psi> (d=n-1).
 *  The leading dimension of returned matrix is n.
 */
double [] getSDivX(final int n,final double x0)
{
    final double [] D=new double [n*(n-1)];
    final BasisFunctionsCalculatable B=getBasisPolynomials();
    for(int k=0;k<n;k++)
    {
	final double [] p=new double [k+1];
	p[k]=1;
	final double [] xp=B.sdiv1(p,x0).q;
	for(int j=k-1;j>=0;j--)
	{
	    D[j*(n)+k]=xp[j];
	}
    }
    return D;
}



/** Calculates |Dpsi>=getEDPsi(|psi>), then
 * return |Dpsi> - <psi|Dpsi>/<psi|psi> |psi>.
 */
double [] getEPsiOrt(final double [] psi,final double [] QQx,final int ldQQ)
{
    final double [] Dpsi=getEDPsi(psi);
    final double g=TMatr.getAMatrB(Dpsi,psi,QQx,ldQQ)/
	TMatr.getAMatrB(psi,psi,QQx,ldQQ);
    if(Dpsi.length>=psi.length)
    {
	for(int k=psi.length;--k>=0;){Dpsi[k]-=g*psi[k];}
	return Dpsi;
    }
    final double [] r=new double [psi.length];
    for(int k=psi.length;--k>=0;){r[k]=(k<Dpsi.length?Dpsi[k]:0)-g*psi[k];}
    return r;
}

abstract DeprecatedHelperDataDirectAccessWithBasis<DTYPE> getAccess();
abstract OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> getABasis();
abstract double getX0();
abstract double getPOffset();
abstract double [] getDMatr(final int n);
abstract double [] getDPsi(final double [] psi);
abstract double [] getEDPsi(final double [] psi);
abstract double [] getEDPsiX(final double [] psi,final double x);

abstract double getIntegratedToEnd(final double x,final double [] psi);
abstract double getXTestEnd(final int nd);
abstract double getXTestEndUnitTest();
abstract BasisFunctionsCalculatable getBasisPolynomials();

abstract boolean isEdgeAtRight();
void sortArray(double [] re,double [] im)
{
    if(isEdgeAtRight()) {com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(re.length,re,im);}
    else {com.polytechnik.utils.SortArrays.sortArrayWithLoad(re.length,re,im);}
}

static double [][] getRoots(final double [] psi,final OrthogonalPolynomialsBasisFunctionsCalculatable<? extends BasisFunctionsCalculatable> M,final boolean flag_edgeatright)
{
    final int n=psi.length-1;
    if(n<=0) return null;
    final PolynomialRootsFinder.PRoots u=M.getPolynomialRootsFinderInBasis().findRoots(psi);
    final double [] re_root=u.re_root,im_root=u.im_root;
    if(u.nroots!=n) return null;
    if(flag_edgeatright){com.polytechnik.utils.SortArrays.sortArrayWithLoadDesc(re_root.length,re_root,im_root);}
    else {com.polytechnik.utils.SortArrays.sortArrayWithLoad(re_root.length,re_root,im_root);}
    return new double [][]{re_root,im_root};
}


static double getXb(final double x,final double [] xs,final boolean flag_right)
{
    for(int k=0;k<xs.length;k++)
    {
	if(flag_right?xs[k]<x:xs[k]>x)
	{
	    return xs[k];
	}
    }
    return Double.NaN;
}
static double getDx(final double x,final double [] xL,final double [] xH,final boolean flag_right)
{
    if(xL==null || xH==null) return Double.NaN;
    final double xbL=getXb(x,xL,flag_right),xbH=getXb(x,xH,flag_right);
    if(flag_right)
    {
	if(xbH>xbL)
	{ // xbL,xbH,x
	    return x-xbH;
	}
	else
	{ // xbH,xbL,x
	    return xbL-x;
	}
    }
    else
    {
	if(xbH<xbL)
	{ // x,xbH,xbL
	    return xbH-x;
	}
	else
	{ // x,xbL,xbH
	    return x-xbL;
	}
    }
}

class WVTPsi {
final EVXData VT;
    final double pT_L,pV_L,xT_L,xV_L,fXx_L,pi_L,pT_H,pV_H,xT_H,xV_H,fXx_H,pi_H;
    final double DpV_L,DpV_H,DpT_L,DpT_H;
WVTPsi(final int n,final double [] pimom,final double [] imom,
       final double [] ptmom,final double [] tmom)
{
    final double [] Vmom=new double [imom.length],Tmom=new double [tmom.length];
    access.setFMoments(Vmom,imom,Vmom.length);
    for(int k=Vmom.length;--k>=0;){Vmom[k]=-Vmom[k];}
    access.setFMoments(Tmom,tmom,Tmom.length);
    for(int k=Tmom.length;--k>=0;){Tmom[k]=-Tmom[k];}
    final double [] QQV=M.B.getQQMatr(n,Vmom),QQT=M.B.getQQMatr(n,Tmom);
    final double x0=getX0();
    this.VT=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQV,QQT,access,x0);

    final double [] PVmom=new double [pimom.length],PTmom=new double [ptmom.length];
    access.setFMoments(PVmom,pimom,PVmom.length);
    for(int k=PVmom.length;--k>=0;){PVmom[k]=-PVmom[k];}
    access.setFMoments(PTmom,ptmom,PTmom.length);
    for(int k=PTmom.length;--k>=0;){PTmom[k]=-PTmom[k];}

    final double [] QQPV=M.B.getQQMatr(n,PVmom),QQPT=M.B.getQQMatr(n,PTmom);
    pT_L=TMatr.getAMatrA(n,VT.psiL,QQPT,n);
    pV_L=TMatr.getAMatrA(n,VT.psiL,QQPV,n)/VT.sL;
    pT_H=TMatr.getAMatrA(n,VT.psiH,QQPT,n);
    pV_H=TMatr.getAMatrA(n,VT.psiH,QQPV,n)/VT.sH;

    final double [] xQQT=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(Tmom)),
	xQQV=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(Vmom));
    xT_L=TMatr.getAMatrA(n,VT.psiL,xQQT,n);
    xV_L=TMatr.getAMatrA(n,VT.psiL,xQQV,n)/VT.sL;
    xT_H=TMatr.getAMatrA(n,VT.psiH,xQQT,n);
    xV_H=TMatr.getAMatrA(n,VT.psiH,xQQV,n)/VT.sH;

    {
    final double [] pp=this.VT.psiL;
    final double [][] r=getRoots(pp,M,isEdgeAtRight());
    final double [] Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0(Xx,x0,isEdgeAtRight());
    this.fXx_L=indX>=0?Xx[indX]:Double.NaN;
    final double [] Dpp=getDpsi(pp,this.fXx_L);
    {
    final double nt=TMatr.getAMatrA(pp.length,pp,QQT,n),
	pi=TMatr.getAMatrA(pp.length,pp,QQPV,n)/nt,
	gi=TMatr.getAMatrB(pp,Dpp,QQT,n)/nt;
    this.DpV_L=TMatr.getAMatrB(pp,Dpp,QQPV,n)/nt-gi*pi;
    this.pi_L=pi;
    }
    {
    final double nt=TMatr.getAMatrA(pp.length,pp,QQV,n),
	pi=TMatr.getAMatrA(pp,QQPT,n)/nt,
	gi=TMatr.getAMatrB(pp,Dpp,QQV,n)/nt;
    this.DpT_L=TMatr.getAMatrB(pp,Dpp,QQPT,n)/nt-gi*pi;
    }
    }
    {
    final double [] pp=this.VT.psiH;
    final double [][] r=getRoots(pp,M,isEdgeAtRight());
    final double [] Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0(Xx,x0,isEdgeAtRight());
    this.fXx_H=indX>=0?Xx[indX]:Double.NaN;
    final double [] Dpp=getDpsi(pp,this.fXx_H);
    {
    final double nt=TMatr.getAMatrA(pp.length,pp,QQT,n),
	pi=TMatr.getAMatrA(pp,QQPV,n)/nt,
	gi=TMatr.getAMatrB(pp,Dpp,QQT,n)/nt;
    this.DpV_H=TMatr.getAMatrB(pp,Dpp,QQPV,n)/nt-gi*pi;
    this.pi_H=pi;
    }
    {
    final double nt=TMatr.getAMatrA(pp.length,pp,QQV,n),
	pi=TMatr.getAMatrA(pp,QQPT,n)/nt,
	gi=TMatr.getAMatrB(pp,Dpp,QQV,n)/nt;
    this.DpT_H=TMatr.getAMatrB(pp,Dpp,QQPT,n)/nt-gi*pi;
    }
    }

    this.spimom=pimom;
    this.simom=imom;
    this.sptmom=ptmom;
    this.stmom=tmom;
}
double [] getDpsi(final double [] psi,final double x)
{
    return getEDPsi(psi);
}
final double [] spimom,simom,sptmom,stmom;
void test_Self(final double eps)
{
    final double [] ppi=spimom.clone(),ppt=sptmom.clone();
    final double POV=4.61,POT=-2.46;
    TMatr.sumP1(ppi,simom,POV);
    TMatr.sumP1(ppt,stmom,POT);
    WVTPsi q=new WVTPsi(VT.neq,ppi,simom,ppt,stmom);
    EVXData.matchEVXData(q.VT,VT,eps);

    UnitTests.checkDoubleRel(q.pT_L,"q.pT_L",pT_L+POT,"pT_L+POT",eps);
    UnitTests.checkDoubleRel(q.pT_H,"q.pT_H",pT_H+POT,"pT_H+POT",eps);
    UnitTests.checkDoubleRel(q.pV_L,"q.pV_L",pV_L+POV,"pV_L+POV",eps);
    UnitTests.checkDoubleRel(q.pV_H,"q.pV_H",pV_H+POV,"pV_H+POV",eps);
    UnitTests.checkDoubleRel(q.DpV_L,"q.DpV_L",DpV_L,"dpV_L",eps);
    UnitTests.checkDoubleRel(q.DpV_H,"q.DpV_H",DpV_H,"dpV_H",eps);

    q=new WVTPsi(VT.neq,TMatr.getVectorScaled(simom,11.7373),simom,TMatr.getVectorScaled(stmom,19.7373*Math.abs(simom[0])),stmom);
    UnitTests.checkDoubleRel(q.DpV_L,"q.DpV_L",0,"0",eps);
    UnitTests.checkDoubleRel(q.DpV_H,"q.DpV_H",0,"0",eps);
    UnitTests.checkDoubleRel(q.DpT_L,"q.DpT_L",0,"0",eps);
    UnitTests.checkDoubleRel(q.DpT_H,"q.DpT_H",0,"0",eps);

    q=new WVTPsi(VT.neq,TMatr.getVectorScaled(stmom,19.7373*Math.abs(simom[0])),simom,TMatr.getVectorScaled(simom,11.7373),stmom);
    UnitTests.checkDoubleRel(q.DpV_L,"q.DpV_L",0,"0",eps);
    UnitTests.checkDoubleRel(q.DpV_H,"q.DpV_H",0,"0",eps);
    UnitTests.checkDoubleRel(q.DpT_L,"q.DpT_L",0,"0",eps);
    UnitTests.checkDoubleRel(q.DpT_H,"q.DpT_H",0,"0",eps);

    System.err.println("DkQVM:WVTPsi:test_Self: test OK neq="+VT.neq+" eps="+eps);
}
} // END OF class WVTPsi


class WVTDPsi {
final EVXData VT;
final double fXx,C,DC,J;
WVTDPsi(final int n,final double [] pimom,final double [] imom,
	final double [] tmom)
{
    final double [] Vmom=new double [imom.length],Tmom=new double [tmom.length];
    access.setFMoments(Vmom,imom,Vmom.length);
    for(int k=Vmom.length;--k>=0;){Vmom[k]=-Vmom[k];}
    access.setFMoments(Tmom,tmom,Tmom.length);
    for(int k=Tmom.length;--k>=0;){Tmom[k]=-Tmom[k];}
    final double [] QQV=M.B.getQQMatr(n,Vmom),QQT=M.B.getQQMatr(n,Tmom);
    final double x0=getX0();
    this.VT=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQV,QQT,access,x0);

    // psi & Dpsi
    final double [] psi=this.VT.psiL;
    final double [][] r=getRoots(psi,M,isEdgeAtRight());
    final double [] Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0(Xx,x0,isEdgeAtRight());
    this.fXx=indX>=0?Xx[indX]:Double.NaN;
    final double [] Dpsi=getDpsi(psi,this.fXx);

    final double [] PVmom=new double [pimom.length];
    access.setFMoments(PVmom,pimom,PVmom.length);
    for(int k=PVmom.length;--k>=0;){PVmom[k]=-PVmom[k];}
    final double [] QQPV=M.B.getQQMatr(n,PVmom);

    final double nt=1;
    this.J=this.VT.sL;
    this.C=TMatr.getAMatrA(psi.length,psi,QQPV,n)/nt;
    final double g=TMatr.getAMatrB(psi,Dpsi,QQT,n)/nt;
    this.DC=TMatr.getAMatrB(psi,Dpsi,QQPV,n)/nt-g*this.C;

    this.spimom=pimom;
    this.simom=imom;
    this.stmom=tmom;
}
double [] getDpsi(final double [] psi,final double x)
{
    return M.B.sdiv1(psi,x).q;
}
final double [] spimom,simom,stmom;
void test_Self(final double eps)
{
    WVTDPsi q=new WVTDPsi(this.VT.neq,TMatr.getVectorScaled(simom,11.7373),simom,stmom);
    UnitTests.checkDoubleRel(q.DC,"q.DC",0,"0",eps);

    final double [] ppi=spimom.clone();
    final double POV=4.61;
    TMatr.sumP1(ppi,simom,POV);
    q=new WVTDPsi(this.VT.neq,ppi,simom,stmom);
    UnitTests.checkDoubleRel(DC,"DC",q.DC,"q.DC",eps);

    WVTPsi s=new WVTPsi(VT.neq,spimom,simom,spimom,stmom);
    UnitTests.checkDoubleRel(J,"J",s.VT.sL,"s.VT.sL",eps);
    UnitTests.checkDoubleRel(C,"C",s.pi_L,"s.pi_L",eps);
    EVXData.matchEVXData(q.VT,s.VT,eps);

    final double [] Vmom=new double [simom.length],Tmom=new double [stmom.length];
    access.setFMoments(Vmom,simom,Vmom.length);
    for(int k=Vmom.length;--k>=0;){Vmom[k]=-Vmom[k];}
    access.setFMoments(Tmom,stmom,Tmom.length);
    for(int k=Tmom.length;--k>=0;){Tmom[k]=-Tmom[k];}
    final double [] PVmom=new double [spimom.length];
    access.setFMoments(PVmom,spimom,PVmom.length);
    for(int k=PVmom.length;--k>=0;){PVmom[k]=-PVmom[k];}

    final WkkPsi t=new WkkPsi(VT.psiL,Tmom,Vmom,PVmom,PVmom/*2*/);
    UnitTests.checkDoubleRel(J,"J",t.Ipsi,"t.Ipsi",eps);
    UnitTests.checkDoubleRel(C,"C",t.Pf,"t.Pf",eps);
    UnitTests.checkDoubleRel(DC,"DC",t.dPf,"t.dPf",eps);
    UnitTests.checkDoubleRel(fXx,"fXx",t.fXx,"t.fXx",eps);

    System.err.println("DkQVM:WVTDPsi:test_Self: test OK neq="+VT.neq+" eps="+eps);
}
} // END OF class WVTDPsi



double [] getFmom(final double [] moms)
{
    final double [] fmoms=new double [moms.length];
    access.setFMoments(fmoms,moms,fmoms.length);
    for(int k=fmoms.length;--k>=0;){fmoms[k]=-fmoms[k];}
    return fmoms;
}


class PVTatX {
    final double fXx,P_t,P_v,P_t0,P_v0;
    final int n;
    PVTatX(final int n,final double fXx,final double [] mmom,
	   final double [] PVmom,final double [] Vmom,
	   final double [] PTmom,final double [] Tmom)
    {
	this.fXx=fXx;
	this.n=n;
	final double [][] QQ=access.getQQ_Matrs(n,1,new double [][] {mmom,PVmom,Vmom,PTmom,Tmom});
	final double [] QQm=QQ[0],QQPV=QQ[1],QQV=QQ[2],QQPT=QQ[3],QQT=QQ[4];
	{
	final double [] ok=access.getOK(n,QQm,this.fXx);
	this.P_v=TMatr.getAMatrA(ok.length,ok,QQPV,n)/TMatr.getAMatrA(ok.length,ok,QQV,n);
	this.P_t=TMatr.getAMatrA(ok.length,ok,QQPT,n)/TMatr.getAMatrA(ok.length,ok,QQT,n);
	}
	{
	final double [] ok=access.getOK(n,QQm,getX0());
	this.P_v0=TMatr.getAMatrA(ok.length,ok,QQPV,n)/TMatr.getAMatrA(ok.length,ok,QQV,n);
	this.P_t0=TMatr.getAMatrA(ok.length,ok,QQPT,n)/TMatr.getAMatrA(ok.length,ok,QQT,n);
	}

    }
}


class WVTmuPsi {
final EVXData VT;
final double fXx,lam,mu,Pv,Pt,Px;
final double x0P,x1P,x2P;
WVTmuPsi(final int n,final double [] pimom,final double [] imom,
	 final double [] ptmom,final double [] tmom)
{
    final double [] Vmom=getFmom(imom),Tmom=getFmom(tmom);
    final double [] QQV=M.B.getQQMatr(n,Vmom),QQT=M.B.getQQMatr(n,Tmom);
    final double x0=getX0();
    this.VT=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQV,QQT,access,x0);
    this.lam=this.VT.sL;

    // psi & Dpsi
    final double [] psi=this.VT.psiL;
    final double [][] r=getRoots(psi,M,isEdgeAtRight());
    final double [] Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0(Xx,x0,isEdgeAtRight());
    this.fXx=indX>=0?Xx[indX]:Double.NaN;
    final double [] Dpsi=getDpsi(psi,this.fXx);

    final double [] PVmom=getFmom(pimom),PTmom=getFmom(ptmom);
    final double [] QQPV=M.B.getQQMatr(n,PVmom),QQPT=M.B.getQQMatr(n,PTmom);

    final double mV=TMatr.getAMatrA(Dpsi.length,Dpsi,QQV,n),
	mT=TMatr.getAMatrA(Dpsi.length,Dpsi,QQT,n),
	mPV=TMatr.getAMatrA(Dpsi.length,Dpsi,QQPV,n),
	mPT=TMatr.getAMatrA(Dpsi.length,Dpsi,QQPT,n);
    final double fx=M.B.getD0(Dpsi,this.fXx),mus=(mV-this.lam*mT);
    this.mu=mus/(fx*fx);
    this.Px=(mPV-this.lam*mPT)/mus;
    this.Pt=mPT/mT;
    this.Pv=mPV/mV;

    final double [] pxmom=new double [imom.length],xmom=new double [imom.length];
    for(int k=pimom.length;--k>=0;){
	pxmom[k]=pimom[k]-this.lam*ptmom[k];
	xmom[k]=imom[k]-this.lam*tmom[k];
    }
    final double [] PXmom=getFmom(pxmom),xPXmom=M.B.getXQkMomentsFromQkMoments(PXmom),x2PXmom=M.B.getXQkMomentsFromQkMoments(xPXmom);
    final double [] QQX=M.B.getQQMatr(n,getFmom(xmom)),QQPX=M.B.getQQMatr(n,PXmom),QQxPX=M.B.getQQMatr(n,xPXmom),QQx2PX=M.B.getQQMatr(n,x2PXmom);
    this.x0P=TMatr.getAMatrA(Dpsi.length,Dpsi,QQPX,n)/mus;
    this.x1P=TMatr.getAMatrA(Dpsi.length,Dpsi,QQxPX,n)/mus-this.fXx*this.x0P;
    this.x2P=TMatr.getAMatrA(Dpsi.length,Dpsi,QQx2PX,n)/mus+this.fXx*(-2*this.x1P-this.x0P*this.fXx);

    this.spimom=pimom;
    this.sptmom=ptmom;
    this.simom=imom;
    this.stmom=tmom;
}

double [] getDpsi(final double [] psi,final double x)
{
    return M.B.sdiv1(psi,x).q;
}
final double [] spimom,sptmom,simom,stmom;
void test_Self(final double eps)
{
    final int n=this.VT.neq;
    UnitTests.checkDoubleRel(Px,"Px",x0P,"x0P",eps);

    {
    WVTDPsi q=new WVTDPsi(n,spimom,simom,stmom);
    EVXData.matchEVXData(this.VT,q.VT,eps);
    }
    {
	final double ff=11.72;
	WVTmuPsi q=new WVTmuPsi(n,TMatr.getVectorScaled(simom,ff),simom,TMatr.getVectorScaled(stmom,ff),stmom);
	if(!Double.isNaN(q.mu) && !Double.isNaN(q.x1P))
	{
	    UnitTests.checkDoubleRel(q.x0P,"q.x0P",ff,"ff",eps);
	    UnitTests.checkDoubleRel(q.x1P,"q.x1P",0,"0",eps);
	    UnitTests.checkDoubleRel(q.x2P,"q.x2P",0,"0",eps);
	}
    }

    {
    WmuPsi q=new WmuPsi(n,getFmom(simom),getFmom(stmom),getFmom(spimom),getFmom(sptmom));
    EVXData.matchEVXData(this.VT,q.I,eps);
    if(!Double.isNaN(q.jL.muX))
    {
    UnitTests.checkDoubleRel(fXx,"fXx",q.jL.fXx,"q.jL.fXx",eps);
    UnitTests.checkDoubleRel(lam,"lam",q.jL.lam,"q.jL.lam",eps);
    UnitTests.checkDoubleRel(mu,"mu",q.jL.muX,"q.jL.muX",eps);
    UnitTests.checkDoubleRel(Pt,"Pt",q.p_t,"q.p_t",eps);
    UnitTests.checkDoubleRel(Pv,"Pv",q.p_v,"q.p_v",eps);
    UnitTests.checkDoubleRel(Px,"Px",q.p_x,"q.p_x",eps);

    UnitTests.checkDoubleRel(x0P,"x0P",q.xD0,"q.xD0",eps);
    UnitTests.checkDoubleRel(x1P,"x1P",q.xXD0,"q.xXD0",eps);
    }
    }

    System.err.println("DkQVM:WVTmuPsi:test_Self: test OK neq="+VT.neq+" eps="+eps);
}
} // END OF class WVTmuPsi





class WqPsi {
static final int BASIS_P=711,BASIS_O=712;
final EVXData I;
final int b_type;
final double [] psiP,basis;
final double [] evP;
final int ind_psi;
final double dP_psi,dP2_psi,lamP_psi,dpsi_mux_dpsi;
final double [] psiBest;
final double P_psiL,sigma2_P_psiL,g_psiL,sigma2_g_psiL,dPpsiL,dPpsiH,g_psiH,sigma2_g_psiH,P_psiH,sigma2_P_psiH;

WqPsi(final int neq,final double [] pimom,final double [] imom,final double [] ptmom,final double [] tmom,final int b_type)
{
    this.b_type=b_type;
    final double x0=getX0();
    final double [] QQi=M.B.getQQMatr(neq,imom),QQt=M.B.getQQMatr(neq,tmom);
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQi,QQt,access,x0);
    final double [] QQx=new double [QQt.length],pxmom=new double [tmom.length];
    for(int k=pxmom.length;--k>=0;){pxmom[k]=pimom[k]-this.I.sL*ptmom[k];}
    for(int k=QQx.length;--k>=0;){QQx[k]=QQi[k]-this.I.sL*QQt[k];}
    final double [] QQpx=M.B.getQQMatr(neq,pxmom);
    this.basis=getXBasis(b_type); //  basis: n x neq matrix.
    final int n=neq-1,basis_offset=0;
    final double [] QQmx=new double [n*n],QQmpx=new double [n*n],tmp=new double [neq*n];
    DeprecatedHelperTMatr.multiply_MA_x_MBTrans(tmp,neq,QQx,neq,0,neq,basis,n,neq,basis_offset); // tmp: neq x n
    DeprecatedHelperTMatr.multiply_MA_x_MB(QQmx,n,basis,neq,basis_offset,neq,tmp,n,n,0); // QQmx: n x n

    DeprecatedHelperTMatr.multiply_MA_x_MBTrans(tmp,neq,QQpx,neq,0,neq,basis,n,neq,basis_offset); // tmp: neq x n
    DeprecatedHelperTMatr.multiply_MA_x_MB(QQmpx,n,basis,neq,basis_offset,neq,tmp,n,n,0); // QQmpx: n x n

    this.evP=DeprecatedHelperEVXData.solveGeneralizedEV(n,QQmpx,QQmx);

    this.psiP=new double [n*neq]; //  n x neq
    DeprecatedHelperTMatr.multiply_MA_x_MB(this.psiP,n,this.evP,n,0+n,n,basis,neq,neq,basis_offset); // psiP: n x neq matrix.

    this.ind_psi=getIndBestPsi();
    this.psiBest=new double [neq];
    if(this.ind_psi>=0){
	System.arraycopy(this.psiP,this.ind_psi*neq,this.psiBest,0,neq);
	this.lamP_psi=this.evP[this.ind_psi];
    } else {
	java.util.Arrays.fill(this.psiBest,Double.NaN);
	this.lamP_psi=Double.NaN;
    }
    final double [] dpsiBest,dpsiBest_in=getBasisPolynomials().getDifferentiated(this.psiBest);
    if(b_type==BASIS_O)
    { // Remove off-basis elements. Not really good operation, but what else to do?
	// If dpsiBest have off-basis elements, then the condition
	// of dp/dt =0 will not hold because of extra terms in scalar product.
	dpsiBest=new double [neq];
	for(int k=0;k<n;k++)
	{
	    final double b=TMatr.getAMatrB(dpsiBest_in.length,dpsiBest_in,0,neq,this.psiP,k*neq,QQx,neq);
	    DeprecatedHelperTMatr.sumP1(dpsiBest,this.psiP,k*neq,neq,b);
	}
	//System.err.println("dpsiBest="+TMatr.getAMatrA(dpsiBest.length,dpsiBest,QQx,neq)+" els="+java.util.Arrays.toString(dpsiBest));
	//System.err.println("dpsiBest_in="+TMatr.getAMatrA(dpsiBest_in.length,dpsiBest_in,QQx,neq)+" els="+java.util.Arrays.toString(dpsiBest_in));
	//System.err.println("psiP="+java.util.Arrays.toString(psiP));
    }
    else
    { // basis already OK
	dpsiBest=dpsiBest_in;
    }
    this.dpsi_mux_dpsi=TMatr.getAMatrA(dpsiBest.length,dpsiBest,QQx,neq);
    this.dP2_psi=TMatr.getAMatrA(dpsiBest.length,dpsiBest,QQpx,neq)-this.lamP_psi*this.dpsi_mux_dpsi;
    final double g=TMatr.getAMatrB(psiBest,dpsiBest,QQx,neq);
    this.dP_psi=TMatr.getAMatrB(psiBest,dpsiBest,QQpx,neq)-this.lamP_psi*g;
    //System.err.println("this.dP_psi="+this.dP_psi);


    final double [] QQpi=M.B.getQQMatr(neq,pimom),piQQpi=getDTxAxD(neq,QQpi,com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQt),new double [neq*neq]);
    {
    final double [] pp=this.I.psiL,Dpp=getEDPsi(pp);
    final double nt=TMatr.getAMatrA(pp.length,pp,QQt,neq),pi=TMatr.getAMatrA(pp.length,pp,QQpi,neq)/nt,
	gi=TMatr.getAMatrB(pp,Dpp,QQt,neq)/nt;
    this.dPpsiL=TMatr.getAMatrB(pp,Dpp,QQpi,neq)/nt-gi*pi;
    this.g_psiL=gi;
    this.sigma2_g_psiL=TMatr.getAMatrA(Dpp,QQt,neq)/nt-gi*gi;
    this.P_psiL=pi;
    this.sigma2_P_psiL=TMatr.getAMatrA(pp,piQQpi,neq)/nt-pi*pi;
    }
    {
    final double [] pp=this.I.psiH,Dpp=getEDPsi(pp);
    final double nt=TMatr.getAMatrA(pp,QQt,neq),pi=TMatr.getAMatrA(pp,QQpi,neq)/nt,
	gi=TMatr.getAMatrB(pp,Dpp,QQt,neq)/nt;
    this.dPpsiH=TMatr.getAMatrB(pp,Dpp,QQpi,neq)/nt-gi*pi;
    this.g_psiH=gi;
    this.sigma2_g_psiH=TMatr.getAMatrA(Dpp,QQt,neq)/nt-gi*gi;
    this.P_psiH=pi;
    this.sigma2_P_psiH=TMatr.getAMatrA(pp,piQQpi,neq)/nt-pi*pi;
    }



    // for unit test
    this.sQQx=QQx;
    this.sQQpx=QQpx;
    this.sQQmx=QQmx;
    this.sQQmpx=QQmpx;

}
final double [] sQQx,sQQpx,sQQmx,sQQmpx;
double [] getXBasis(final int b_type)
{
    final int neq=this.I.neq,n=neq-1;
    final double [] basis=new double [n*neq];
    if(b_type==BASIS_P)
    {
	for(int k=0;k<n;k++){basis[k*neq+k]=1;}
    }
    else if(b_type==BASIS_O)
    {
	final int off_k=1;
	for(int k=0;k<n;k++)
	{
	    System.arraycopy(this.I.ev,neq+(k+off_k)*neq,basis,k*neq,neq);
	}
    } else throw new RuntimeException("b_type="+b_type);
    return basis;
}

int getIndBestPsi()
{
    final int neq=this.I.neq,n=neq-1;
    final double x0=getX0();
    int ind=-1;
    double apsiM=0;
    final double [] psi=new double [neq];
    for(int k=0;k<n;k++)
    {
	System.arraycopy(this.psiP,k*neq,psi,0,neq);
	final double apsi=Math.abs(M.B.getD0(psi,x0));
	if(apsi>=apsiM)
	{
	    apsiM=apsi;
	    ind=k;
	}
    }
    return ind;
}

void test_Self(final double eps)
{
    UnitTests.checkDoubleRel(dP_psi,"dP_psi",0,"0",eps);

    final int neq=this.I.neq,n=neq-1;
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<n;l++)
	{
	    final double qqmx=TMatr.getAMatrB(neq,basis,neq*k,basis,neq*l,sQQx,neq);
	    final double fm=(b_type==WqPsi.BASIS_O&&l!=k)?0:1;
	    UnitTests.checkDoubleRel(qqmx,"qqmx",sQQmx[k*n+l]*fm,"sQQmx[k*n+l]",eps);

	    final double qqmpx=TMatr.getAMatrB(neq,basis,neq*k,basis,neq*l,sQQpx,neq);
	    UnitTests.checkDoubleRel(qqmpx,"qqmpx",sQQmpx[k*n+l],"sQQmpx[k*n+l]",eps);

	    final double qqx_psiP=TMatr.getAMatrB(neq,psiP,neq*k,psiP,neq*l,sQQx,neq),
		qqmx_psiP=TMatr.getAMatrB(n,evP,n+n*k,evP,n+n*l,sQQmx,n),
		qqpx_psiP=TMatr.getAMatrB(neq,psiP,neq*k,psiP,neq*l,sQQpx,neq),
		qqmpx_psiP=TMatr.getAMatrB(n,evP,n+n*k,evP,n+n*l,sQQmpx,n);
	    if(k==l){
		UnitTests.checkDoubleRel(qqmx_psiP,"qqmx_psiP",1,"1",eps);
		UnitTests.checkDoubleRel(qqmpx_psiP,"qqmpx_psiP",evP[k],"evP[k]",eps);

		UnitTests.checkDoubleRel(qqx_psiP,"qqx_psiP",1,"1",eps);
		UnitTests.checkDoubleRel(qqpx_psiP,"qqpx_psiP",evP[k],"evP[k]",eps);
	    } else {
		UnitTests.checkDoubleRel(qqmx_psiP,"qqmx_psiP",0,"0",eps);
		UnitTests.checkDoubleRel(qqmpx_psiP,"qqmpx_psiP",0,"0",eps);

		UnitTests.checkDoubleRel(qqx_psiP,"qqx_psiP",0,"0",eps);
		UnitTests.checkDoubleRel(qqpx_psiP,"qqpx_psiP",0,"0",eps);
	    }

	    if(b_type==WqPsi.BASIS_O)
	    {
		if(k==l){
		    UnitTests.checkDoubleRel(sQQmx[k*n+l],"sQQmx[k*n+l]",this.I.ev[k+1]-this.I.sL,"I.ev[k+1]-I.ev[0]",eps);
		}else{
		    UnitTests.checkDoubleRel(sQQmx[k*n+l],"sQQmx[k*n+l]",0,"0",eps);
		}
	    }
	}
    }
    System.err.println("DkQVM:WqPsi:test_Self: test OK neq="+neq+" eps="+eps);
}

} // END OF class WqPsi



class WoPsi {
final EVXData I;
final double [] psiP;
final double [] evP;
final int ind_psi;
final double dP_psi,dP2_psi,lamP_psi,lamI_skip,dpsi_mux_dpsi;
final double [] psiBest;
final int nskip;

WoPsi(final int neq,final int nskip,final double [] pimom,final double [] imom,final double [] ptmom,final double [] tmom)
{
    if(!(0<=nskip && nskip<=neq-1)) throw new RuntimeException("0<=nskip="+nskip+" <="+(neq-1));
    this.nskip=nskip;
    final double x0=getX0();
    final double [] QQi=M.B.getQQMatr(neq,imom),QQt=M.B.getQQMatr(neq,tmom);
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQi,QQt,access,x0);
    this.lamI_skip=nskip<=0?0:this.I.ev[nskip-1];
    final double [] QQx=new double [QQt.length],pxmom=new double [tmom.length];
    for(int k=pxmom.length;--k>=0;){pxmom[k]=pimom[k]-this.lamI_skip*ptmom[k];}
    for(int k=QQx.length;--k>=0;){QQx[k]=QQi[k]-this.lamI_skip*QQt[k];}
    final double [] QQpx=M.B.getQQMatr(neq,pxmom);


    final int n=neq-nskip,basis_offset=neq+nskip*neq;
    final double [] basis=this.I.ev;

    final double [] QQmx=new double [n*n],QQmpx=new double [n*n],tmp=new double [neq*n];
    DeprecatedHelperTMatr.multiply_MA_x_MBTrans(tmp,neq,QQx,neq,0,neq,basis,n,neq,basis_offset); // tmp: neq x n
    DeprecatedHelperTMatr.multiply_MA_x_MB(QQmx,n,basis,neq,basis_offset,neq,tmp,n,n,0); // QQmx: n x n

    DeprecatedHelperTMatr.multiply_MA_x_MBTrans(tmp,neq,QQpx,neq,0,neq,basis,n,neq,basis_offset); // tmp: neq x n
    DeprecatedHelperTMatr.multiply_MA_x_MB(QQmpx,n,basis,neq,basis_offset,neq,tmp,n,n,0); // QQmpx: n x n

    this.evP=DeprecatedHelperEVXData.solveGeneralizedEV(n,QQmpx,QQmx);

    this.psiP=new double [n*neq]; //  n x neq
    DeprecatedHelperTMatr.multiply_MA_x_MB(this.psiP,n,this.evP,n,0+n,n,basis,neq,neq,basis_offset); // psiP: n x neq matrix.

    int ind_b=-1;double apsiM=0;
    this.psiBest=new double [neq];
    for(int k=0;k<n;k++)
    {
	System.arraycopy(this.psiP,k*neq,this.psiBest,0,neq);
	final double apsi=Math.abs(M.B.getD0(this.psiBest,x0));
	if(Math.abs(apsi)>=apsiM)
	{
	    apsiM=apsi;
	    ind_b=k;
	}
    }
    this.ind_psi=ind_b;
    if(this.ind_psi>=0){
	System.arraycopy(this.psiP,this.ind_psi*neq,this.psiBest,0,neq);
	this.lamP_psi=this.evP[this.ind_psi];
    } else {
	java.util.Arrays.fill(this.psiBest,Double.NaN);
	this.lamP_psi=Double.NaN;
    }

    final double [] dpsiBest,d=getBasisPolynomials().getDifferentiated(this.psiBest);
    if(nskip>0)
    { // Remove off-basis elements. Not really good operation, but what else to do?
	// If dpsiBest have off-basis elements, then the condition
	// of dp/dt =0 will not hold because of extra terms in scalar product.
	// The norm is preserved.
	dpsiBest=new double [neq];
	for(int k=0;k<n;k++)
	{
	    final double b=TMatr.getAMatrB(d.length,d,0,neq,this.psiP,k*neq,QQx,neq);
	    DeprecatedHelperTMatr.sumP1(dpsiBest,this.psiP,k*neq,neq,b);
	}
    }
    else
    { // basis already OK
	dpsiBest=d;
    }
    this.dpsi_mux_dpsi=TMatr.getAMatrA(dpsiBest,QQx,neq);
    this.dP2_psi=TMatr.getAMatrA(dpsiBest,QQpx,neq)-this.lamP_psi*this.dpsi_mux_dpsi;
    final double g=TMatr.getAMatrB(psiBest,dpsiBest,QQx,neq);
    this.dP_psi=TMatr.getAMatrB(psiBest,dpsiBest,QQpx,neq)-this.lamP_psi*g;

    // for unit test
    this.sQQx=QQx;
    this.sQQpx=QQpx;
    this.sQQmx=QQmx;
    this.sQQmpx=QQmpx;
}
final double [] sQQx,sQQpx,sQQmx,sQQmpx;

void test_Self(final double eps)
{
    UnitTests.checkDoubleRel(dP_psi,"dP_psi",0,"0",eps);

    final int neq=this.I.neq,n=neq-nskip;
    for(int k=0;k<n;k++)
    {
	for(int l=0;l<n;l++)
	{
	    final double qqx_psiP=TMatr.getAMatrB(neq,psiP,neq*k,psiP,neq*l,sQQx,neq),
		qqmx_psiP=TMatr.getAMatrB(n,evP,n+n*k,evP,n+n*l,sQQmx,n),
		qqpx_psiP=TMatr.getAMatrB(neq,psiP,neq*k,psiP,neq*l,sQQpx,neq),
		qqmpx_psiP=TMatr.getAMatrB(n,evP,n+n*k,evP,n+n*l,sQQmpx,n);
	    if(k==l){
		UnitTests.checkDoubleRel(qqmx_psiP,"qqmx_psiP",1,"1",eps);
		UnitTests.checkDoubleRel(qqmpx_psiP,"qqmpx_psiP",evP[k],"evP[k]",eps);

		UnitTests.checkDoubleRel(qqx_psiP,"qqx_psiP",1,"1",eps);
		UnitTests.checkDoubleRel(qqpx_psiP,"qqpx_psiP",evP[k],"evP[k]",eps);

		UnitTests.checkDoubleRel(sQQmx[k*n+l],"sQQmx[k*n+l]",this.I.ev[k+nskip]-this.lamI_skip,"I.ev[k+nskip]-lamI_skip",eps);

	    } else {
		UnitTests.checkDoubleRel(qqmx_psiP,"qqmx_psiP",0,"0",eps);
		UnitTests.checkDoubleRel(qqmpx_psiP,"qqmpx_psiP",0,"0",eps);

		UnitTests.checkDoubleRel(qqx_psiP,"qqx_psiP",0,"0",eps);
		UnitTests.checkDoubleRel(qqpx_psiP,"qqpx_psiP",0,"0",eps);

		UnitTests.checkDoubleRel(sQQmx[k*n+l],"sQQmx[k*n+l]",0,"0",eps);
	    }
	}
    }
    System.err.println("DkQVM:WoPsi:test_Self: test OK neq="+neq+" eps="+eps);
}

} // END OF class WoPsi



class WpPsi {
    final EVXData Px;
    final double [] lamsP,D2pxW,XP,WP,dmW,Pm,Im,dpIm,dIm,DpIm;
    final double fXx,PfXx,WXx,D2pxWfXx,PmXx,PpsiL,PpsiH;
    final int indX,indW;
    WpPsi(final int n,final double [] pxmom,final double [] xmom,double [] pMmom,final double [] Mmom)
    {
	final double x0=getX0();
	this.lamsP=new double [n];
	this.Pm=new double [n];
	this.Im=new double [n];
	this.dpIm=new double [n];
	this.DpIm=new double [n];
	this.dIm=new double [n];
	this.D2pxW=new double [n];
	this.XP=new double [n];
	this.WP=new double [n];
	this.dmW=new double [n];

	final double [] QQx=M.B.getQQMatr(n,xmom),QQpx=M.B.getQQMatr(n,pxmom),xQQx=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(xmom)),
	    QQm=M.B.getQQMatr(n,Mmom),QQpm=M.B.getQQMatr(n,pMmom);
	this.Px=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQpx,QQx,access,x0);

	final double sqrtK00=Math.sqrt(Px.o_o);
	final BasisFunctionsCalculatable B=getBasisPolynomials();
	final double [] pp=new double [n];
	for(int k=0;k<n;k++)
	{
	    System.arraycopy(Px.ev,n+k*n,pp,0,n);
	    this.lamsP[k]=Px.ev[k];
	    final double nm=TMatr.getAMatrA(pp,QQm,n);
	    this.Pm[k]=TMatr.getAMatrA(pp,QQpm,n)/nm;
	    this.XP[k]=TMatr.getAMatrA(pp,xQQx,n);
	    this.WP[k]=M.B.getD0(pp,x0)/sqrtK00;
	    this.Im[k]=nm;

	    final double [] DppOrt=getEPsiOrt(pp,QQx,n);
	    final double nrd=TMatr.getAMatrA(DppOrt,QQx,n);
	    this.dmW[k]=nrd;
	    this.D2pxW[k]=TMatr.getAMatrA(DppOrt,QQpx,n)/nrd-this.lamsP[k];
	    this.dpIm[k]=TMatr.getAMatrB(pp,DppOrt,QQpm,n);
	    this.dIm[k]=TMatr.getAMatrB(pp,DppOrt,QQm,n);
	    this.DpIm[k]=this.dpIm[k]/nm-this.Pm[k]*(this.dIm[k]/nm);
	}
	PpsiL=Pm[0];
	PpsiH=Pm[Pm.length-1];

	this.indW=DjQVM.getIndX0_NonSortedX(this.WP,1.5,true);
	final int indX=this.indX=DjQVM.getIndX0_NonSortedX(this.XP,x0,isEdgeAtRight());
	if(indX>=0)
	{
	    this.fXx=this.XP[indX];
	    this.PfXx=this.lamsP[indX];
	    this.PmXx=this.Pm[indX];
	    this.WXx=this.WP[indX];
	    this.D2pxWfXx=this.D2pxW[indX];
	}
	else
	{
	    this.fXx=this.PfXx=this.PmXx=this.WXx=this.D2pxWfXx=Double.NaN;
	}

	this.sxmom=xmom;
	this.spxmom=pxmom;
    }
final double [] sxmom,spxmom;
void test_Self(final double eps)
{
    final int n=this.Px.neq;
    final double [] QQx=M.B.getQQMatr(n,sxmom),QQpx=M.B.getQQMatr(n,spxmom),
	QQPM=getVolMatrix2(n,QQpx),QQM=getVolMatrix2(n,QQx);
    final double [] pp=new double [n];
    for(int k=0;k<n;k++)
    {
	System.arraycopy(Px.ev,n+k*n,pp,0,n);
	final double d1=getDFShift(pp,QQpx,QQx,n);
	UnitTests.checkDoubleRel(d1,"d1",0,"0",eps);

	UnitTests.checkDoubleRel(lamsP[k],"lamsP[k]",TMatr.getAMatrA(pp,QQpx,n),"<psi|QQp|psi>",eps);
	UnitTests.checkDoubleRel(1,"1",TMatr.getAMatrA(pp,QQx,n),"<psi|QQ|psi>",eps);
	if(Math.abs(WP[k])>1+eps)
	{
	    throw new RuntimeException("WP["+k+"]="+WP[k]);
	}

	{
	final double dw=TMatr.getAMatrA(pp,QQPM,n)-TMatr.getAMatrA(pp,QQM,n)*lamsP[k];
	if(dmW[k]!=0)
	UnitTests.checkDoubleRel(dw,"dw",D2pxW[k]*dmW[k],"D2pxW[k]*dmW",eps);
	}

	{
	final double [] D=getED(n);
	final double dmw=TMatr.getAMatrA(pp,QQM,n)-Math.pow(TMatr.getAMatrA(pp,TMatr.mulMatrices(n,new double [][]{QQx,D}),n),2);
	UnitTests.checkDoubleRel(dmw,"dmw",dmW[k],"dmW[k]",eps);
	}

	{
	final double [] DppOrt=getEPsiOrt(pp,QQx,n);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(pp,DppOrt,QQx,n),"<pp|DppOrt>",0,"0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(pp,DppOrt,QQpx,n),"<pp|QQp|DppOrt>",0,"0",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(DppOrt,QQx,n),"<DppOrt|DppOrt>",dmW[k],"dmW[k]",eps);
	}

	System.err.println("dmW["+k+"]="+dmW[k]+" D2pxW["+k+"]*dmW="+(D2pxW[k]*dmW[k])+" getD2FShift(pp)="+(getD2FShift(pp,QQpx,QQx,n)));
    }
    final WpPsi s=new WpPsi(n,spxmom,sxmom,spxmom,sxmom);
    UnitTests.checkDoubleArrayRel(this.lamsP,"this.lamsP",s.Pm,"s.Pm",eps);

    System.err.println("DkQVM:WpPsi:test_Self: test OK n="+n+" eps="+eps);
}
} // END OF class WpPsi




abstract class WpVPsi {
    final EVXData P2x;
    final double [] lamsP,XP,X2P,varP,vargP;
    WpVPsi(final int n,final double [] p2imom,final double [] pimom,double [] imom,final double [] tmom)
    {
	final double x0=getX0();
	this.lamsP=new double [n];
	this.XP=new double [n];
	this.X2P=new double [n];
	this.varP=new double [n];
	this.vargP=new double [n];

	final double [] QQp2i=M.B.getQQMatr(n,p2imom),QQt=M.B.getQQMatr(n,tmom);
	this.P2x=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQp2i,QQt,access,x0);
	final double [] xtmom=M.B.getXQkMomentsFromQkMoments(tmom),
	    xQQt=M.B.getQQMatr(n,xtmom),
	    x2QQt=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(xtmom)),
	    QQpi=M.B.getQQMatr(n,pimom);
	this.sQQp2i=QQp2i;
	this.sQQpi=QQpi;
	this.sQQt=QQt;
	this.sxQQt=xQQt;
	final double [] pp=new double [n];
	for(int k=0;k<n;k++)
	{
	    System.arraycopy(P2x.ev,n+k*n,pp,0,n);
	    this.lamsP[k]=P2x.ev[k];
	    this.XP[k]=TMatr.getAMatrA(pp,xQQt,n);
	    this.X2P[k]=TMatr.getAMatrA(pp,x2QQt,n)-this.XP[k]*this.XP[k];
	    final double [] varpp=getVarPsi(pp,this.XP[k]);
	    this.vargP[k]=TMatr.getAMatrB(pp,pp,QQpi,n);
	    this.varP[k]=TMatr.getAMatrB(pp,varpp,QQpi,n)-TMatr.getAMatrB(pp,varpp,QQt,n)*this.vargP[k];
	}
	this.stmom=tmom;
    }
final double [] sQQp2i,sQQpi,sQQt,sxQQt;
final double [] stmom;
abstract double [] getVarPsi(final double [] psi,final double x);
void test_Self2(final double eps){}
void test_Self(final double eps)
{
    final int n=this.P2x.neq;
    final double [] pp=new double [n];
    for(int k=0;k<n;k++)
    {
	System.arraycopy(P2x.ev,n+k*n,pp,0,n);

	UnitTests.checkDoubleRel(lamsP[k],"lamsP[k]",TMatr.getAMatrA(pp,sQQp2i,n),"<psi|QQp2i|psi>",eps);
	UnitTests.checkDoubleRel(1,"1",TMatr.getAMatrA(pp,sQQt,n),"<psi|QQt|psi>",eps);
	UnitTests.checkDoubleRel(XP[k],"XP[k]",TMatr.getAMatrA(pp,sxQQt,n),"<psi|xQQt|psi>",eps);
	UnitTests.checkDoubleRel(vargP[k],"vargP[k]",TMatr.getAMatrA(pp,sQQpi,n),"<psi|QQpi|psi>",eps);

	final double [] varpp=getVarPsi(pp,this.XP[k]);
	final double var2=TMatr.getAMatrB(pp,varpp,sQQp2i,n)-TMatr.getAMatrB(pp,varpp,sQQt,n)*TMatr.getAMatrB(pp,pp,sQQp2i,n);
	UnitTests.checkDoubleRel(0,"0",var2,"var2",eps);
    }

    System.err.println("DkQVM:WpVPsi:test_Self: test OK n="+n+" eps="+eps);
    test_Self2(eps);
}
} // END OF class WpVPsi

class WpVPsi_EDPsi extends WpVPsi {
   WpVPsi_EDPsi(final int n,final double [] p2imom,final double [] pimom,double [] imom,final double [] tmom)
    {
	super(n,p2imom,pimom,imom,tmom);
    }
    double [] getVarPsi(final double [] psi,final double x)
    {
	return getEDPsi(psi);
    }
}
class WpVPsi_XX extends WpVPsi {
   WpVPsi_XX(final int n,final double [] p2imom,final double [] pimom,double [] imom,final double [] tmom)
    {
	super(n,p2imom,pimom,imom,tmom);
    }
    double [] getVarPsi(final double [] psi,final double x)
    {
	final BasisFunctionsCalculatable B=getBasisPolynomials();
	final double [] dpsi=dx_getDop(psi,B);
	final double [] res=B.getaxbP(dpsi,1,-x);
	//System.err.println("dpsi.length="+dpsi.length);
	return res;
    }
    void test_Self2(final double eps)
    {
	final int n=this.P2x.neq;
	final double [] xtmom=M.B.getXQkMomentsFromQkMoments(stmom);
	final double [] pp=new double [n];
	for(int k=0;k<n;k++)
	{
	    System.arraycopy(P2x.ev,n+k*n,pp,0,n);

	    final double [] varpp=getVarPsi(pp,this.XP[k]);
	    final double vara=TMatr.mulVVn1(varpp,stmom);

	    final double [] dpp=dx_getDop(pp,getBasisPolynomials());
	    final double varb=TMatr.mulVVn1(dpp,xtmom)-XP[k]*TMatr.mulVVn1(dpp,stmom);
	    UnitTests.checkDoubleRel(vara,"vara",varb,"varb",eps);
	}
	System.err.println("DkQVM:WpVPsi:test_Self2: test OK n="+n+" eps="+eps);
    }
}


double getDFShift(final double [] psi,final double [] QQfx,final double [] QQx,final int ldQQ)
{
    final int n=psi.length;
    final double mu=TMatr.getAMatrA(psi,QQx,ldQQ),sf=TMatr.getAMatrA(psi,QQfx,ldQQ);
    final BasisFunctionsCalculatable B=getBasisPolynomials();
    final double [] dpsi=B.getDifferentiated(psi);
    final double f=sf/mu,g=TMatr.getAMatrB(psi,dpsi,QQx,ldQQ)/mu;
    final double fg=TMatr.getAMatrB(psi,dpsi,QQfx,ldQQ)/mu;
    return fg-f*g;
}
double getD2FShift(final double [] psi,final double [] QQfx,final double [] QQx,final int ldQQ)
{
    final int n=psi.length;
    final double mu=TMatr.getAMatrA(psi,QQx,ldQQ),sf=TMatr.getAMatrA(psi,QQfx,ldQQ);
    final BasisFunctionsCalculatable B=getBasisPolynomials();
    final double [] dpsi=B.getDifferentiated(psi);
    final double f=sf/mu,g=TMatr.getAMatrB(dpsi,dpsi,QQx,ldQQ)/mu;
    final double fg=TMatr.getAMatrB(dpsi,dpsi,QQfx,ldQQ)/mu;
    return fg-f*g;
}


class WdPsi {

final double [] psi,psiP;
final double [] Xx;
final int indX;
final double lamm,fXx,jX,dp2IX;
final double delta0,f0min,delta0rel;

private final double a,b,c,at,bt,ct;

WdPsi(final int neq, final double [] fmom,final double [] imom,final double [] tmom,final boolean flag_max)
{
    final double [] QQi=M.B.getQQMatr(neq,imom),QQf=M.B.getQQMatr(neq,fmom),QQt=M.B.getQQMatr(neq,tmom),
	IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQt),
	tmp=new double [neq*neq],
	QQfIQQtQQf=ZQVM.getIQQ_F_IQQ(neq,QQf,IQQt,tmp);

    final double [] evpI=M.EV.getGEV_Generalized(neq,QQfIQQtQQf,QQi,true);
    // psi corresponds to minimal eigenvalue.
    this.psi=new double [neq];
    final int inde=flag_max?neq-1:0;
    System.arraycopy(evpI,neq+inde*neq,psi,0,neq);
    this.lamm=evpI[inde];
    this.Xx=M.getXNodes(psi).re_root;
    sortArray(this.Xx,null);
    this.indX=DjQVM.getIndX0(Xx,getX0(),isEdgeAtRight());
    final double xk=indX>=0?Xx[indX]:Double.NaN;
    this.fXx=xk;
    final double [] rxk=new double [neq],rtmp=new double [neq],KKxk=new double [neq*neq];
    for(int k=0;k<neq;k++)
    {
	rtmp[k]=1;
	rxk[k]=M.B.getD0(rtmp,xk);
	rtmp[k]=0;
    }
    for(int k=0;k<neq;k++)
    {
	for(int l=0;l<neq;l++)
	{
	    KKxk[k+neq*l]=rxk[k]*rxk[l];
	}
    }
    final double [] K_K=TMatr.mulMMM(neq,KKxk,IQQt,KKxk),
	K_Df=TMatr.mulMMM(neq,KKxk,IQQt,QQf),
	Df_Df=TMatr.mulMMM(neq,QQf,IQQt,QQf);

    this.psiP=M.B.sdiv1(psi,xk).q;
    final double fp=M.B.getD0(psiP,xk);
    //for(int l=0;l<psiP.length;l++){psiP[l]/=fp;}
    if(Double.isNaN(xk))
    {
	java.util.Arrays.fill(psiP,Double.NaN);
    }
    final double wk=TMatr.getAMatrA(psiP,QQi,neq);
    this.dp2IX=TMatr.getAMatrA(psiP,QQfIQQtQQf,neq)/wk;
    this.jX=TMatr.getAMatrA(psiP,QQf,neq)/wk;
    //System.err.println("fXx="+fXx+" xk="+xk+" wk="+wk+" psiP="+java.util.Arrays.toString(psiP)+" QQi="+java.util.Arrays.toString(QQi)+" psi="+java.util.Arrays.toString(psi)+" fp="+fp+" Xx="+java.util.Arrays.toString(Xx));

    a=TMatr.getAMatrA(psiP,K_K,neq)/wk;
    b=2*TMatr.getAMatrA(psiP,K_Df,neq)/wk;
    c=TMatr.getAMatrA(psiP,Df_Df,neq)/wk;

    this.delta0=-b/(2*a);
    this.f0min=c-a*Math.pow(this.delta0,2);
    this.delta0rel=this.delta0*Math.sqrt(a/f0min);

    final double ff=M.B.getD0(psiP,xk);
    at=(ff*ff)*TMatr.getAMatrA(rxk,IQQt,neq)/wk;
    bt=2*ff*TMatr.getAMatrB(psiP,rxk,TMatr.mulMM(neq,QQf,IQQt),neq)/wk;
    ct=this.dp2IX;
}

} // END OF class WdPsi


class WfPsi {

final EVXData PM;
final double mu0_psiL,mu0_psiH,
    mu1_psiL,sigma2_mu1_psiL,mu1_psiH,sigma2_mu1_psiH;

WfPsi(final int n, final double [] fplus,final double [] fminus,final double [] tmom)
{
    final double [] fQQplus=M.B.getQQMatr(n,fplus),fxQQplus=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(fplus)),
	fQQminus=M.B.getQQMatr(n,fminus),fxQQminus=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(fminus));
    this.PM=DeprecatedHelperEVXData.getEVXDataFromQQ(n,fQQplus,fQQminus,access,getX0());

    final double m0plus_psiH=TMatr.getAMatrA(this.PM.psiH,fQQplus,n),m1plus_psiH=TMatr.getAMatrA(this.PM.psiH,fxQQplus,n);

    final double m0plus_psiL=TMatr.getAMatrA(this.PM.psiL,fQQplus,n),m1plus_psiL=TMatr.getAMatrA(this.PM.psiL,fxQQplus,n);

    final double m0minus_psiH=TMatr.getAMatrA(this.PM.psiH,fQQminus,n),m1minus_psiH=TMatr.getAMatrA(this.PM.psiH,fxQQminus,n);

    final double m0minus_psiL=TMatr.getAMatrA(this.PM.psiL,fQQminus,n),m1minus_psiL=TMatr.getAMatrA(this.PM.psiL,fxQQminus,n);


    final double [] QQt=M.B.getQQMatr(n,tmom),IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
    final double m0t_psiL=TMatr.getAMatrA(this.PM.psiL,QQt,n),m0t_psiH=TMatr.getAMatrA(this.PM.psiH,QQt,n);
    final double [] fxQQplus2=TMatr.mulMMM(n,fxQQplus,IQQt,fxQQplus),
	fxQQminus2=TMatr.mulMMM(n,fxQQminus,IQQt,fxQQminus),
	fxQminusQplus=TMatr.mulMMM(n,fxQQminus,IQQt,fxQQplus);

    this.mu0_psiL=(m0plus_psiL-this.PM.sL*m0minus_psiL)/m0t_psiL;
    this.mu0_psiH=(m0plus_psiH-this.PM.sH*m0minus_psiH)/m0t_psiH;

    this.mu1_psiL=(m1plus_psiL-this.PM.sL*m1minus_psiL)/m0t_psiL;
    this.sigma2_mu1_psiL=(TMatr.getAMatrA(this.PM.psiL,fxQQplus2,n)-2*this.PM.sL*TMatr.getAMatrA(this.PM.psiL,fxQminusQplus,n)+this.PM.sL*this.PM.sL*TMatr.getAMatrA(this.PM.psiL,fxQQminus2,n))/m0t_psiL-Math.pow(this.mu1_psiL,2);

    this.mu1_psiH=(m1plus_psiH-this.PM.sH*m1minus_psiH)/m0t_psiH;
    this.sigma2_mu1_psiH=(TMatr.getAMatrA(this.PM.psiH,fxQQplus2,n)-2*this.PM.sH*TMatr.getAMatrA(this.PM.psiH,fxQminusQplus,n)+this.PM.sH*this.PM.sH*TMatr.getAMatrA(this.PM.psiH,fxQQminus2,n))/m0t_psiH-Math.pow(this.mu1_psiH,2);

}

WfPsi(final int n, final double [] fplus,final double [] fminus,final double [] tmom,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double x0)
{
    final double [] fQQplus=M.B.getQQMatr(n,fplus),fQQminus=M.B.getQQMatr(n,fminus);
    this.PM=DeprecatedHelperEVXData.getEVXDataFromQQ(n,fQQplus,fQQminus,access,x0);
    final double [] Lmom=new double [fminus.length],Hmom=new double [Lmom.length];
    for(int k=Lmom.length;--k>=0;)
    {
	Lmom[k]=fplus[k]-this.PM.sL*fminus[k];
	Hmom[k]=fplus[k]-this.PM.sH*fminus[k];
    }
    final double [] QQxL=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(Lmom)),QQxH=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(Hmom)),QQt=M.B.getQQMatr(n,tmom),IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
    final double m0t_psiL=TMatr.getAMatrA(this.PM.psiL,QQt,n),m0t_psiH=TMatr.getAMatrA(this.PM.psiH,QQt,n);
    this.mu0_psiL=this.mu0_psiH=0;
    this.mu1_psiL=TMatr.getAMatrA(this.PM.psiL,QQxL,n)/m0t_psiL;
    this.sigma2_mu1_psiL=TMatr.getAMatrA(this.PM.psiL,TMatr.mulMMM(n,QQxL,IQQt,QQxL),n)/m0t_psiL-Math.pow(this.mu1_psiL,2);
    this.mu1_psiH=TMatr.getAMatrA(this.PM.psiH,QQxH,n)/m0t_psiH;
    this.sigma2_mu1_psiH=TMatr.getAMatrA(this.PM.psiH,TMatr.mulMMM(n,QQxH,IQQt,QQxH),n)/m0t_psiH-Math.pow(this.mu1_psiH,2);
}

void test_matchWfPsi(final WfPsi s,final double eps)
{
    UnitTests.checkDoubleRel(mu0_psiL,"mu0_psiL",s.mu0_psiL,"s.mu0_psiL",eps);
    UnitTests.checkDoubleRel(mu0_psiH,"mu0_psiL",s.mu0_psiH,"s.mu0_psiL",eps);
    UnitTests.checkDoubleRel(mu1_psiL,"mu1_psiL",s.mu1_psiL,"s.mu1_psiL",eps);
    UnitTests.checkDoubleRel(mu1_psiH,"mu1_psiH",s.mu1_psiH,"s.mu1_psiH",eps);
    UnitTests.checkDoubleRel(sigma2_mu1_psiL,"sigma2_mu1_psiL",s.sigma2_mu1_psiL,"s.sigma2_mu1_psiL",eps);
    UnitTests.checkDoubleRel(sigma2_mu1_psiH,"sigma2_mu1_psiH",s.sigma2_mu1_psiH,"s.sigma2_mu1_psiH",eps);
    System.err.println("DkQVM:WfPsi:WABpsi:test_matchWfPsi(): test OK n="+this.PM.neq+" eps="+eps);
}

}// END OF class WfPsi


class WkfPsi {
    final EVXData ID,DT;
    final double dFH,PtH,PiH,dFL,PtL,PiL,dPtL,dPtH,
    fXx_L,fXx_H;
WkfPsi(final int n,final double [] imom,final double [] tmom,
       final double [] pimom,final double [] ptmom,final double [] dpmom)
{
    final double [] QQi=M.B.getQQMatr(n,imom),QQt=M.B.getQQMatr(n,tmom),DQQt=getVolMatr(n,QQt);
    this.ID=DeprecatedHelperEVXData.getEVXDataFromQQ(n,DQQt,QQi,access,getX0());
    {
	final double [] Xx=M.getXNodes(this.ID.psiL).re_root;
	final int indX=DjQVM.getIndX0_NonSortedX(Xx,getX0(),isEdgeAtRight());
	this.fXx_L=indX>=0?Xx[indX]:Double.NaN;
    }
    {
	final double [] Xx=M.getXNodes(this.ID.psiH).re_root;
	final int indX=DjQVM.getIndX0_NonSortedX(Xx,getX0(),isEdgeAtRight());
	this.fXx_H=indX>=0?Xx[indX]:Double.NaN;
    }
    this.DT=DeprecatedHelperEVXData.getEVXDataFromQQ(n,DQQt,QQt,access,getX0());

    final double [] PQQi=M.B.getQQMatr(n,pimom),PDQQt=getVolMatr(n,M.B.getQQMatr(n,ptmom));
    //System.err.println("DiP="+java.util.Arrays.toString(DiP));
    this.dFH=TMatr.getAMatrA(this.ID.psiH,PDQQt,n)-this.ID.sH*TMatr.getAMatrA(this.ID.psiH,PQQi,n);
    this.PtH=TMatr.getAMatrA(this.ID.psiH,PDQQt,n)/TMatr.getAMatrA(this.ID.psiH,DQQt,n);
    this.PiH=TMatr.getAMatrA(this.ID.psiH,PQQi,n)/TMatr.getAMatrA(this.ID.psiH,QQi,n);

    this.dFL=TMatr.getAMatrA(this.ID.psiL,PDQQt,n)-this.ID.sL*TMatr.getAMatrA(this.ID.psiL,PQQi,n);
    this.PtL=TMatr.getAMatrA(this.ID.psiL,PDQQt,n)/TMatr.getAMatrA(this.ID.psiL,DQQt,n);
    this.PiL=TMatr.getAMatrA(this.ID.psiL,PQQi,n)/TMatr.getAMatrA(this.ID.psiL,QQi,n);

    final double [] QQdp=M.B.getQQMatr(n,dpmom),D=getED(n),tmp=new double [n*n];
    {
    final double [] pp=this.ID.psiL;
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,QQt,n,0,D,n,0);
    final double nt=TMatr.getAMatrA(pp,QQt,n),
	g=TMatr.getAMatrA(pp,tmp,n)/nt,
	pt=TMatr.getAMatrA(pp,QQdp,n)/nt;
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,QQdp,n,0,D,n,0);
    this.dPtL=TMatr.getAMatrA(pp,tmp,n)/nt-pt*g;
    }
    {
    final double [] pp=this.ID.psiH;
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,QQt,n,0,D,n,0);
    final double nt=TMatr.getAMatrA(pp,QQt,n),
	g=TMatr.getAMatrA(pp,tmp,n)/nt,
	pt=TMatr.getAMatrA(pp,QQdp,n)/nt;
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,QQdp,n,0,D,n,0);
    this.dPtH=TMatr.getAMatrA(pp,tmp,n)/nt-pt*g;
    }

    this.sPQQt=M.B.getQQMatr(n,ptmom);
    this.sPDQQt=PDQQt;
    this.sDQQt=DQQt;
    this.sQQt=QQt;
    this.sQQi=QQi;
    this.sdpmom=dpmom;
    this.stmom=tmom;

}
    final double [] sPQQt,sPDQQt,sDQQt,sQQt,sQQi,sdpmom,stmom;
void test_Self(final double eps)
{
    final int n=ID.neq;
    System.err.println("ID.sL="+ID.sL+"\tID.psiL="+java.util.Arrays.toString(ID.psiL)+"\nEV="+java.util.Arrays.toString(ID.getEValues()));
    System.err.println("dFL="+dFL+" ID.sL="+ID.sL);
    UnitTests.checkDoubleRel(dFL,"dFL",ID.sL*(PtL-PiL),"ID.sL*(PtL-PiL)",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(this.ID.psiL,sDQQt,n)-this.ID.sL*TMatr.getAMatrA(this.ID.psiL,sQQi,n),"<psiL|v|psiL>",0,"0",eps);

    System.err.println("ID.sH="+ID.sH+"\tID.psiH="+java.util.Arrays.toString(ID.psiH)+"\nEV="+java.util.Arrays.toString(ID.getEValues()));
    System.err.println("dFH="+dFH+" ID.sH="+ID.sH);
    UnitTests.checkDoubleRel(dFH,"dFH",ID.sH*(PtH-PiH),"ID.sH*(PtH-PiH)",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(this.ID.psiH,sDQQt,n)-this.ID.sH*TMatr.getAMatrA(this.ID.psiH,sQQi,n),"<psiH|v|psiH>",0,"0",eps);

    final EVXData b=DeprecatedHelperEVXData.getEVXDataFromQQ(n,sDQQt,sQQt,access,getX0());
    System.err.println("DQQt=EV*QQt:EV="+java.util.Arrays.toString(b.getEValues()));
    if(false) System.err.println("DQQt="+java.util.Arrays.toString(sDQQt)+"\nQQt="+java.util.Arrays.toString(sQQt));

    final double [] p1=new double [n],p2=new double [n];
    for(int k=0;k<n;k++)
    {
	System.arraycopy(ID.ev,n+k*n,p1,0,n);
	final double [] d1=getDPsi(p1);
	for(int l=0;l<n;l++)
	{
	    System.arraycopy(ID.ev,n+l*n,p2,0,n);
	    final double [] d2=getDPsi(p2);
	    UnitTests.checkDoubleRel(TMatr.getAMatrB(d1,d2,sPQQt,n)+
				     +0.5*TMatr.getAMatrB(d1,p2,sPQQt,n)+
				     +0.5*TMatr.getAMatrB(p1,d2,sPQQt,n)+
				     +0.25*TMatr.getAMatrB(p1,p2,sPQQt,n),
				     "<f|PQQt|g>",TMatr.getAMatrB(p1,p2,sPDQQt,n),"<p1|PDQQt|p2>",eps);
	}
    }

    UnitTests.checkDoubleArrayRel(getVolMatrix2(n,sPQQt),"getVolMatrix2(n,sPQQt)",getVolMatr(n,sPQQt),"getVolMatr(n,sPQQt)",eps);
    UnitTests.checkDoubleArrayRel(getVolMatrix2(n,sPQQt),"getVolMatrix2(n,sPQQt)",sPDQQt,"sPDQQt",eps);

    // test VolMatrix
    UnitTests.checkDoubleArrayRel(getVolMatrix2(n,sQQi),"getVolMatrix2(n,sQQi)",VOLATILITY_EDPsi.getVolatility(Double.NaN).getVolMatrix(n,sQQi),"VOLATILITY_EDPsi.getVolMatrix",eps);
    UnitTests.checkDoubleArrayRel(getDiffMatrix2(n,sQQi),"getDiffMatrix2(n,sQQi)",VOLATILITY_EDPsi.getVolatility(Double.NaN).getDiffMatrix(n,sQQi),"VOLATILITY_EDPsi.getDiffMatrix",eps);


    System.err.println("DkQVM:WkfPsi:test_Self(): test OK n="+n+" eps="+eps);
}
} // END OF class WkfPsi

static double [] getDTxAxD(final int n,final double [] D,final double [] A,final double [] tmp)
{
    final double [] M=new double [n*n];
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,A,n,0,D,n,0);
    DeprecatedHelperTMatr.multiply_MatrTrans_x_Matr(n,M,D,n,0,tmp,n,0);
    return M;
}


static double [] dx_getDop(final double [] p,final BasisFunctionsCalculatable B)
{
    return B.getDifferentiated(p);
}
/** Calculate d/dx matrix. */
static double [] getDop(final int n,final BasisFunctionsCalculatable B)
{
    final double [] D=new double [n*n];
    for(int k=0;k<n;k++)
    {
	final double [] p=new double [k+1];
	p[k]=1;
	final double [] d=dx_getDop(p,B);
	for(int j=d.length;--j>=0;)
	{
	    D[j*n+k]=d[j];
	}
	//System.arraycopy(d,0,D,k*n,d.length);
    }
    //System.err.println("D="+java.util.Arrays.toString(D));
    return D;
}

static double [] dx_getXDop(final double [] p,final BasisFunctionsCalculatable B)
{
    double [] d=B.getDifferentiated(p),xd=new double [d.length+1];
    if(d.length>0) xd=B.getaxbP(d,1,0);
    else java.util.Arrays.fill(xd,0);
    return xd;
}
/** Calculate x d/dx matrix. */
static double [] getXDop(final int n,final BasisFunctionsCalculatable B)
{
    final double [] D=new double [n*n];
    for(int k=0;k<n;k++)
    {
	final double [] p=new double [k+1];
	p[k]=1;
	final double [] d=dx_getXDop(p,B);
	for(int j=d.length;--j>=0;)
	{
	    D[j*n+k]=d[j];
	}
	//System.arraycopy(d,0,D,k*n,d.length);
    }
    //System.err.println("D="+java.util.Arrays.toString(D));
    return D;
}


class WkePsi {

final double [] psi;
final double Pf,Pfd,dPf,DPf,g,sigma2_g,sigma2_Pf,sigma2b_Pf,rho,Ipsi;

WkePsi(final double [] psi,final double [] tmom,final double [] imom,
       final double [] pimom,final double [] p2imom)
{
    this.psi=psi;
    final int n=psi.length;
    final double [] QQt=M.B.getQQMatr(n,tmom),PQQi=M.B.getQQMatr(n,pimom),
	D=getDMatr(n),tmp=new double [n*n],
	dQQt=getDTxAxD(n,D,QQt,tmp),dPQQi=getDTxAxD(n,D,PQQi,tmp);
    final double nd=TMatr.getAMatrA(psi,dQQt,n),
	nt=TMatr.getAMatrA(psi,QQt,n);
    this.Pf=TMatr.getAMatrA(psi,PQQi,n)/nt;
    this.Pfd=TMatr.getAMatrA(psi,dPQQi,n)/nd;
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,QQt,n,0,D,n,0);
    this.g=TMatr.getAMatrA(psi,tmp,n)/nt;
    this.sigma2_g=nd/nt-this.g*this.g;
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,PQQi,n,0,D,n,0);
    this.DPf=TMatr.getAMatrA(psi,tmp,n)/nt;
    this.dPf=this.DPf-this.g*this.Pf;
    final double [] IQQt=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt);
    final double [] piQQpi=getDTxAxD(n,PQQi,IQQt,tmp);
    this.sigma2_Pf=TMatr.getAMatrA(psi,piQQpi,n)/nt-this.Pf*this.Pf;
    final double [] QQi=M.B.getQQMatr(n,imom);
    this.Ipsi=TMatr.getAMatrA(psi,QQi,n)/nt;
    final double sigma2_P;
    if(p2imom!=null)
    {
	final double [] P2QQi=M.B.getQQMatr(n,p2imom);
	final double [] M=TMatr.mulMMM(n,P2QQi,IQQt,QQi);
	this.sigma2b_Pf=TMatr.getAMatrA(psi,M,n)/nt-this.Pf*this.Pf;
	sigma2_P=this.sigma2b_Pf;
    }
    else
    {
	this.sigma2b_Pf=Double.NaN;
	sigma2_P=this.sigma2_Pf;
    }
    this.rho=this.dPf/Math.sqrt(this.sigma2_g*sigma2_P);


    this.stmom=tmom;
    this.simom=imom;
    this.spimom=pimom;
    this.sp2imom=p2imom;
}
double [] getDMatr(final int n)
{
    return getED(n);
}
final double [] stmom,spimom,simom,sp2imom;
void test_Self(final double eps)
{
    final int n=psi.length;
    // check psi is the eigenfunction
    WkePsi b=new WkePsi(psi,stmom,simom,simom,sp2imom);
    UnitTests.checkDoubleRel(b.dPf,"b.dPf",0,"0",eps);
    UnitTests.checkDoubleRel(b.sigma2_Pf/Math.abs(b.Ipsi),"b.sigma2_Pf",0,"0",eps);
    //UnitTests.checkDoubleRel(b.sigma2_g,"b.sigma2_g",0,"0",eps);

    // match WlPsi
    final WlPsi l=new WlPsi(psi,0,spimom,stmom,null,null);
    UnitTests.checkDoubleRel(dPf,"dPf",l.fgD,"l.fgD",eps);
    System.err.println("DkQVM:WkePsi:test_Self:Pf="+Pf+" Pfd="+Pfd+" dPf="+dPf+" g="+g);

    // Check invariance, which follows from the fact that QQi psi=lambda QQt psi.
    final double [] pimom=new double [spimom.length],p2imom=new double [spimom.length];
    for(double PD:new double []{6.41233,-8.2,1.345,-2.1434})
    {
    for(int k=0;k<pimom.length;k++)
    {
	pimom[k]=spimom[k]+PD*simom[k];
	p2imom[k]=sp2imom[k]+PD*(2*spimom[k]+PD*simom[k]);
    }
    b=new WkePsi(psi,stmom,simom,pimom,p2imom);
    UnitTests.checkDoubleRel(Ipsi,"Ipsi",b.Ipsi,"b.Ipsi",eps);
    UnitTests.checkDoubleRel(dPf,"dPf",b.dPf,"b.dPf",eps);
    UnitTests.checkDoubleRel(sigma2_Pf,"sigma2_Pf",b.sigma2_Pf,"b.sigma2_Pf",eps);
    UnitTests.checkDoubleRel(sigma2_g,"sigma2_g",b.sigma2_g,"b.sigma2_g",eps);
    UnitTests.checkDoubleRel(sigma2b_Pf,"sigma2b_Pf",b.sigma2b_Pf,"b.sigma2b_Pf",eps);
    }
    b=new WkePsi(psi,stmom,simom,sp2imom,sp2imom);
    UnitTests.checkDoubleRel(sigma2b_Pf,"sigma2b_Pf",b.Ipsi*b.Pf-Math.pow(this.Pf,2),"sp2imom direct",eps);

    System.err.println("DkQVM:WkePsi:test_Self: test OK neq="+n+" eps="+eps);
}
} // END OF class WkePsi

class WkkPsi {
final double [] psi;
final double fXx,Ipsi;
final double Pf,dPf,DPf,g,sigma2_g,sigma2b_Pf,I2c,PI2c,dPI2c,I2,PI2;

WkkPsi(final double [] psi,final double [] tmom,final double [] imom,
       final double [] pimom,final double [] p2imom)
{
    this.psi=psi;
    final int n=psi.length;
    final double x0=getX0();
    final double [][] r=getRoots(psi,M,isEdgeAtRight());
    final double [] Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0_NonSortedX(Xx,x0,isEdgeAtRight());
    this.fXx=indX>=0?Xx[indX]:Double.NaN;
    final double [] Dpsi=getDpsi(psi,this.fXx);

    final double [] QQt=M.B.getQQMatr(n,tmom),QQi=M.B.getQQMatr(n,imom),
	QQpi=M.B.getQQMatr(n,pimom),QQp2i=M.B.getQQMatr(n,p2imom);
    final double nt=TMatr.getAMatrA(psi,QQt,n),
	dgg=TMatr.getAMatrA(Dpsi,QQt,n)/nt;
    this.Ipsi=TMatr.getAMatrA(psi,QQi,n)/nt;
    this.g=TMatr.getAMatrB(psi,Dpsi,QQt,n)/nt;
    this.sigma2_g=dgg-this.g*this.g;
    this.Pf=TMatr.getAMatrA(psi,QQpi,n)/nt;
    this.sigma2b_Pf=(TMatr.getAMatrA(psi,QQp2i,n)/nt)*Ipsi-this.Pf*this.Pf;
    this.DPf=TMatr.getAMatrB(psi,Dpsi,QQpi,n)/nt;
    this.dPf=this.DPf-this.Pf*this.g;
    this.I2c=2*(TMatr.getAMatrA(Dpsi,QQi,n)/nt-this.Ipsi*dgg);
    this.PI2c=2*(TMatr.getAMatrA(Dpsi,QQpi,n)/nt-this.Pf*dgg)/this.I2c;
    this.dPI2c=(-2)*((-2)*this.g*(-2)*this.dPf)/this.I2c;


    if(false||Dpsi.length>=n)
    {
	final double [] mi=getfDsx(psi,Dpsi,QQi,QQt,n),mpi=getfDsx(psi,Dpsi,QQpi,QQt,n);
	this.I2=mi[2];
	this.PI2=mpi[2]/mi[2];
    }
    else
    {
	final BasisFunctionsCalculatable B=getBasisPolynomials();
	this.I2=getfD2x(Dpsi,this.fXx,QQi,QQt,n,B);
	this.PI2=getfD2x(Dpsi,this.fXx,QQpi,QQt,n,B)/I2;
    }

    this.stmom=tmom;
    this.simom=imom;
    this.spimom=pimom;
    this.sp2imom=p2imom;
}

double [] getDpsi(final double [] psi,final double xk)
{
    return M.B.sdiv1(psi,xk).q;
}
final double [] stmom,spimom,simom,sp2imom;
void test_Self(final java.util.Random r,final double eps)
{
    final int n=psi.length;
    // check psi is the eigenfunction
    WkkPsi b=new WkkPsi(psi,stmom,simom,simom,sp2imom);
    UnitTests.checkDoubleRel(b.dPf,"b.dPf",0,"0",eps);
    System.err.println("DkQVM:WkkPsi:test_Self: I2="+I2+" I2c="+I2c+" n="+n);
    if(!Double.isNaN(b.fXx)&&b.I2c!=0&&b.I2!=0)
    {
	UnitTests.checkDoubleRel(PI2,"PI2",PI2c+dPI2c,"PI2c+dPI2c",eps);
	UnitTests.checkDoubleRel(I2c,"I2c",I2,"I2",eps);
	UnitTests.checkDoubleRel(b.I2c,"b.I2c",b.I2,"b.I2",eps);
	UnitTests.checkDoubleRel(b.PI2c,"b.PI2c",1,"1",eps);
	UnitTests.checkDoubleRel(b.PI2,"b.PI2",1,"1",eps);
	System.err.println("DkQVM:WkkPsi:test_Self: I2/I2c match");
    }


    // Check invariance, which follows from the fact that QQi psi=lambda QQt psi.
    final double [] pimom=new double [spimom.length],p2imom=new double [spimom.length];
    for(double PD:new double []{6.41233,-8.2,1.345,-2.1434})
    {
	for(int k=0;k<pimom.length;k++)
	{
	    pimom[k]=spimom[k]+PD*simom[k];
	    p2imom[k]=sp2imom[k]+PD*(2*spimom[k]+PD*simom[k]);
	}
	b=new WkkPsi(psi,stmom,simom,pimom,p2imom);
	UnitTests.checkDoubleRel(Ipsi,"Ipsi",b.Ipsi,"b.Ipsi",eps);
	UnitTests.checkDoubleRel(dPf,"dPf",b.dPf,"b.dPf",eps);
	UnitTests.checkDoubleRel(sigma2_g,"sigma2_g",b.sigma2_g,"b.sigma2_g",eps);
	UnitTests.checkDoubleRel(sigma2b_Pf,"sigma2b_Pf",b.sigma2b_Pf,"b.sigma2b_Pf",eps);
	UnitTests.checkDoubleRel(PI2,"PI2",b.PI2-PD,"b.PI2-PD",eps);
    }
    b=new WkkPsi(psi,stmom,simom,sp2imom,sp2imom);
    UnitTests.checkDoubleRel(sigma2b_Pf,"sigma2b_Pf",b.Ipsi*b.Pf-Math.pow(this.Pf,2),"sp2imom direct",eps);

    final WABpsi m=new WABpsi(psi,Ipsi,M.B.getQQMatr(n,simom),M.B.getQQMatr(n,stmom),n);
    UnitTests.checkDoubleRel(m.lam,"m.lam",Ipsi,"Ipsi",eps);
    UnitTests.checkDoubleRel(m.fXx,"m.fXx",fXx,"fXx",eps);

    final double nrm=Math.pow(M.B.getD0(getDpsi(psi,this.fXx),this.fXx),2)/TMatr.getAMatrA(psi,M.B.getQQMatr(n,stmom),n)*(isEdgeAtRight()?1:-1),
	DPF=(m.getVFXX(spimom,WABpsi.MT_MU)[1])*nrm;
    if(!Double.isNaN(DPF))
	UnitTests.checkDoubleRel(DPF,"DPF",this.DPf,"DPf",eps);
    else
	System.err.println("DPF="+DPF+" DPf="+DPf);

    if(!Double.isNaN(m.muX))
    {
	final double [] fnom=getX0X1X2(m.psiInd,spimom),pnom=new double []{fnom[2],-2*fnom[1],fnom[0]},
	    fdnom=getX0X1X2(m.psiInd,stmom),pdnom=new double []{fdnom[2],-2*fdnom[1],fdnom[0]};
	final double mPf=Polynomials.getD0(pnom,fXx)/Polynomials.getD0(pdnom,fXx),
		mg=Polynomials.getD1(pdnom,fXx)/Polynomials.getD0(pdnom,fXx),
		mdPf=Polynomials.getD1(pnom,fXx)/Polynomials.getD0(pdnom,fXx)-mPf*mg;
	UnitTests.checkDoubleRel(Pf,"Pf",mPf,"mPf",eps);
	UnitTests.checkDoubleRel(g,"g",-0.5*mg,"-mg/2",eps);
	UnitTests.checkDoubleRel(dPf,"dPf",-0.5*mdPf,"-mdPf/2",eps);
    }


    final WkePsi e=new WkePsi(psi,stmom,simom,spimom,sp2imom){
	    double [] getDMatr(final int n)
	    {
		final double [] D=new double [n*n];
		for(int k=0;k<n;k++)
		{
		    final double [] p=new double [k+1];
		    p[k]=1;
		    final double [] d=getDpsi(p,fXx);
		    for(int j=d.length;--j>=0;)
		    {
			D[j*n+k]=d[j];
		    }
		}
		return D;
	    }
	};
    UnitTests.checkDoubleRel(Ipsi,"Ipsi",e.Ipsi,"e.Ipsi",eps);
    UnitTests.checkDoubleRel(dPf,"dPf",e.dPf,"e.dPf",eps);
    UnitTests.checkDoubleRel(sigma2_g,"sigma2_g",e.sigma2_g,"e.sigma2_g",eps);
    UnitTests.checkDoubleRel(sigma2b_Pf,"sigma2b_Pf",e.sigma2b_Pf,"e.sigma2b_Pf",eps);



    System.err.println("DkQVM:WkkPsi:test_Self: test OK neq="+n+" eps="+eps);

    test_D2(r,eps);
}

void test_D2(final java.util.Random r,final double eps)
{

    final double [] Dpsi=getDpsi(psi,this.fXx);
    final int ldQQ=1+Dpsi.length;
    final int ntests=10;
    final BasisFunctionsCalculatable B=getBasisPolynomials();
    for(int t=0;t<ntests;t++)
    {
	final double [] tmom,imom,pimom,p2imom;
	if(t<=0)
	{
	    tmom=stmom;
	    imom=simom;
	    pimom=spimom;
	    p2imom=sp2imom;
	}
	else
	{
	    final int nmom=2*ldQQ-1,nsum=8000;
	    tmom=getRandomPositivelyDefined(nmom,nsum,r);
	    imom=getRandomPositivelyDefined(nmom,nsum,r);
	    pimom=getRandomPositivelyDefined(nmom,nsum,r);
	    p2imom=getRandomPositivelyDefined(nmom,nsum,r);
	}

	final double [] QQt=M.B.getQQMatr(ldQQ,tmom),QQi=M.B.getQQMatr(ldQQ,imom),
	    QQpi=M.B.getQQMatr(ldQQ,pimom),QQp2i=M.B.getQQMatr(ldQQ,p2imom);

	final PolynomialsRatio j2=testfD(Dpsi,QQi,QQt,ldQQ,B),pj2=testfD(Dpsi,QQpi,QQt,ldQQ,B);
	if(t<=0 && !Double.isNaN(this.fXx))
	{ // check eigenvector
	    UnitTests.checkDoubleRel(0,"0",j2.getDF(this.fXx),"j2.getDF(q.fXx)",eps);
	    UnitTests.checkDoubleRel(0,"0",getfD1x(Dpsi,this.fXx,QQi,QQt,ldQQ,B),"getfD1x(Dpsi,this.fXx,QQi,QQt,ldQQ,B)",eps);
	}
	final WkkPsi q=new WkkPsi(psi,tmom,imom,pimom,p2imom);
	if(!Double.isNaN(j2.getD2F(q.fXx)))
	{
	    if(q.I2!=0 && q.I2c!=0) UnitTests.checkDoubleRel(q.PI2,"q.PI2",(q.PI2c+q.dPI2c)*(q.I2c/q.I2),"(q.PI2c+q.dPI2c)*(q.I2c/q.I2)",eps);
	    UnitTests.checkDoubleRel(q.I2,"q.I2",j2.getD2F(q.fXx),"j2.getD2F(q.fXx)",eps);
	    UnitTests.checkDoubleRel(q.PI2,"q.PI2",pj2.getD2F(q.fXx)/j2.getD2F(q.fXx),"pj2.getD2F(q.fXx)/j2.getD2F(q.fXx)",eps);
	}
	for(int xt=0;xt<10;xt++)
	{
	    final double xtest=r.nextDouble();
	    UnitTests.checkDoubleRel(getfD1x(Dpsi,xtest,QQi,QQt,ldQQ,B),"getfD1x(Dpsi,xtest,QQi,QQt,ldQQ,B)",j2.getDF(xtest),"j2.getDF(xtest)",eps);
	    UnitTests.checkDoubleRel(getfD2x(Dpsi,xtest,QQi,QQt,ldQQ,B),"getfD2x(Dpsi,xtest,QQi,QQt,ldQQ,B)",j2.getD2F(xtest),"j2.getD2F(xtest)",eps);

	    final double [] ipsi=M.getxP(Dpsi);for(int k=0;k<Dpsi.length;k++){ipsi[k]-=xtest*Dpsi[k];}
	    UnitTests.checkDoubleRel(M.B.getD0(ipsi,xtest),"ipsi(xtest)",0,"0",eps);

	    final double [] dd=getDpsi(ipsi,xtest);
	    final double [] m=getfDsx(ipsi,dd,QQi,QQt,ldQQ);
	    UnitTests.checkDoubleRel(m[0],"m[0]",j2.getF(xtest),"j2.getF(xtest)",eps);
	    UnitTests.checkDoubleRel(m[1],"m[1]",j2.getDF(xtest),"j2.getDF(xtest)",eps);
	    UnitTests.checkDoubleRel(m[2],"m[2]",j2.getD2F(xtest),"j2.getD2F(xtest)",eps);
	    final double [] ddp=new double [ipsi.length];
	    for(int k=0;k<ddp.length;k++) {ddp[k]=2.11*ipsi[k]+(k<dd.length?dd[k]:0);}
	    final double [] mp=getfDsx(ipsi,ddp,QQi,QQt,ldQQ);
	    UnitTests.checkDoubleRel(m[1],"m[1]",mp[1],"mp[1]",eps);
	    //UnitTests.checkDoubleRel(m[2],"m[2]",mp[2],"mp[2]",eps);

	    final double [] mm=M.B.mult2Pol(ipsi,ipsi);
	    final double [] dmm=B.getDifferentiated(mm),d2mm=B.getDifferentiated(dmm);
	    final PolynomialsRatio jd=new PolynomialsRatio(
		new double []{TMatr.mulVVn1(mm,tmom),-TMatr.mulVVn1(dmm,tmom),0.5*TMatr.mulVVn1(d2mm,tmom)},
		new double []{TMatr.mulVVn1(mm,imom),-TMatr.mulVVn1(dmm,imom),0.5*TMatr.mulVVn1(d2mm,imom)},new OrthogonalPolynomialsMonomialsBasis().B);

	    final double [] aDpsi=B.getDifferentiated(ipsi),
		aD2psi=B.getDifferentiated(aDpsi);
	    final double [] md=getfDDx(ipsi,aDpsi,aD2psi,QQi,QQt,ldQQ);
	    UnitTests.checkDoubleRel(md[0],"md[0]",jd.getF(0),"jd.getF(0)",eps);
	    UnitTests.checkDoubleRel(md[1],"md[1]",jd.getDF(0),"jd.getDF(0)",eps);
	    UnitTests.checkDoubleRel(md[2],"md[2]",jd.getD2F(0),"jd.getD2F(0)",eps);
	}
    }
    System.err.println("DkQVM:WkkPsi:test_D2: test OK neq="+psi.length+" eps="+eps);
}
} // END OF class WkkPsi



class WkPsi {

final double [] psi;
final double lam;

final double pf,f,fx0,sigma2_f,fI,XfI,Xd,sigma2_Xd,Pd,sigma2_Pd,Id,ft,mu;
final double Pfd,Pfx,Pf;
final double fDP,fPD;
final double fXx,p_t,p_i,p_td,n1;

WkPsi(final double [] psi,final double lam,
      final double [] p2imom,final double [] pimom,final double [] imom,
      final double [] p2tmom,final double [] ptmom,final double [] tmom)
{
    this.psi=psi;
    this.lam=lam;
    final int n=psi.length;
    final double [] momx=new double [imom.length],pmomx=new double [imom.length],p2momx=new double [imom.length];
    for(int k=momx.length;--k>=0;)
    {
	momx[k]=imom[k]-this.lam*(tmom!=null?tmom[k]:0);
	pmomx[k]=pimom[k]-this.lam*(ptmom!=null?ptmom[k]:0);
	p2momx[k]=p2imom[k]-this.lam*(p2tmom!=null?p2tmom[k]:0);
    }
    final BasisFunctionsCalculatable B=getBasisPolynomials();
    final double [] QQx=M.B.getQQMatr(n,momx),pQQx=M.B.getQQMatr(n,pmomx);
    final double [] dpsi=B.getDifferentiated(psi);

    final double x0=getX0(),fk=M.B.getD0(psi,x0),dfk=M.B.getD1(psi,x0);
    this.Id=TMatr.getAMatrA(dpsi,QQx,n);
    this.f=TMatr.getAMatrB(psi,dpsi,pQQx,n);
    this.pf=TMatr.getAMatrA(psi,pQQx,n);
    this.mu=TMatr.getAMatrA(psi,QQx,n);
    this.fx0=(this.f-(dfk/fk)*this.pf);

    final double [] Xmomx=M.B.getXQkMomentsFromQkMoments(momx),XQQx=M.B.getQQMatr(n,Xmomx),
	X2momx=M.B.getXQkMomentsFromQkMoments(Xmomx),X2QQx=M.B.getQQMatr(n,X2momx),
	p2QQx=M.B.getQQMatr(n,p2momx);
    this.XfI=TMatr.getAMatrB(psi,dpsi,XQQx,n);
    this.fI=TMatr.getAMatrB(psi,dpsi,QQx,n);

    this.Xd=TMatr.getAMatrA(dpsi,XQQx,n)/this.Id;
    this.sigma2_Xd=TMatr.getAMatrA(dpsi,X2QQx,n)/this.Id-Math.pow(this.Xd,2);
    this.Pd=TMatr.getAMatrA(dpsi,pQQx,n)/this.Id;
    this.sigma2_Pd=TMatr.getAMatrA(dpsi,p2QQx,n)/this.Id-Math.pow(this.Pd,2);
    this.sigma2_f=this.sigma2_Pd*this.sigma2_Xd;

    final double [] D=getDop(n,B),DP=TMatr.mulMM(n,D,pQQx),PD=TMatr.mulMM(n,pQQx,D);
    this.fDP=TMatr.getAMatrA(psi,DP,n);
    this.fPD=TMatr.getAMatrA(psi,PD,n);
    // dp/dmu is equal to fDP-fPD

    final double [] Di=TMatr.mulMM(n,D,QQx),iD=TMatr.mulMM(n,QQx,D),
	XDi=TMatr.mulMM(n,D,XQQx),XiD=TMatr.mulMM(n,XQQx,D);
    this.fDi=TMatr.getAMatrA(psi,Di,n);
    this.fiD=TMatr.getAMatrA(psi,iD,n);
    this.fXDi=TMatr.getAMatrA(psi,XDi,n);
    this.fXiD=TMatr.getAMatrA(psi,XiD,n);

    final double [] QQt=M.B.getQQMatr(n,tmom);
    final double nr=TMatr.getAMatrA(psi,QQt,n),
	gt=TMatr.getAMatrB(psi,dpsi,QQt,n)/nr,
	dpt=TMatr.getAMatrA(psi,pQQx,n)/nr;
    this.ft=TMatr.getAMatrB(psi,dpsi,pQQx,n)/nr-gt*dpt;

    final double [] psid=getDpsiX(psi,x0,B);
    //System.err.println("wkpsi:psi="+java.util.Arrays.toString(psi));
    //System.err.println("wkpsi:pQQx="+java.util.Arrays.toString(pQQx));
    Pf=TMatr.getAMatrA(psi,pQQx,n)/TMatr.getAMatrA(psi,QQt,n);
    Pfx=TMatr.getAMatrA(psid,pQQx,n)/TMatr.getAMatrA(psid,QQt,n);
    Pfd=TMatr.getAMatrA(dpsi,pQQx,n)/TMatr.getAMatrA(dpsi,QQt,n);



    final double [] QQi=M.B.getQQMatr(n,imom),
	QQpi=M.B.getQQMatr(n,pimom),QQpt=ptmom!=null?M.B.getQQMatr(n,ptmom):QQpi;
    final double t_d=TMatr.getAMatrA(psi,QQt,n),p_t_d=TMatr.getAMatrA(psi,QQpt,n);
    this.p_t=p_t_d/t_d;
    this.p_i=TMatr.getAMatrA(psi,QQpi,n)/TMatr.getAMatrA(psi,QQi,n);
    final double [][] r=getRoots(psi,M,isEdgeAtRight());
    final double [] Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0(Xx,x0,isEdgeAtRight());
    this.fXx=indX>=0?Xx[indX]:Double.NaN;
    final double ot,dpl=0; // set dpl=0 asuming offset=last
    {
	final double [] pp=B.mult2Pol(psi,psi);
	ot=getIntegratedToEnd(this.fXx,pp);
	this.n1=t_d/getIntegratedToEnd(getXTestEndUnitTest(),pp);
    }
    this.p_td=(p_t_d-dpl*ot)/(t_d-ot);


    this.smomx=momx;
    this.spmomx=pmomx;
    this.sp2momx=p2momx;
    this.stmom=tmom;
    this.simom=imom;
}
    final double [] smomx,spmomx,sp2momx,stmom,simom;
final double fDi,fiD,fXDi,fXiD;
void test_Self(final double eps)
{
    final int n=psi.length;
    UnitTests.checkDoubleRel(fI,"fI",0,"0",eps);
    UnitTests.checkDoubleRel(XfI,"XfI",0,"0",eps);
    System.err.println("Id="+Id+" fI="+fI+" XfI="+XfI+" fDP="+fDP+" fPD="+fPD);
    final BasisFunctionsCalculatable B=getBasisPolynomials();
    final double [] dpsi=B.getDifferentiated(psi);
    final double [] D=getDop(n,B);
    final double [] Dpsi=TMatr.getMatr_x_A(n,D,psi);
    UnitTests.checkDoubleArrayRel(java.util.Arrays.copyOf(dpsi,n),"dpsi",Dpsi,"Dpsi",eps);
    UnitTests.checkDoubleRel(Dpsi[Dpsi.length-1],"Dpsi[Dpsi.length-1]",0,"0",eps);
    final double [] XD=getXDop(n,B);
    final double [] XDpsi=TMatr.getMatr_x_A(n,XD,psi),Xdpsi=M.getxP(dpsi);
    UnitTests.checkDoubleArrayRel(Xdpsi,"Xdpsi",XDpsi,"XDpsi",eps);
    //System.err.println("XDpsi="+java.util.Arrays.toString(XDpsi));

    UnitTests.checkDoubleRel(f,"f",fPD,"fPD",eps);
    final double [] Ke=getD2KEFromPsi(psi,getX0(),B);
    UnitTests.checkDoubleRel(fx0,"fx0",TMatr.mulVVn1(Ke,spmomx),"<Ke|spmomx>",eps);
    UnitTests.checkDoubleRel(M.B.getD0(Ke,getX0()),"Ke(x0)",0,"0",eps);

    UnitTests.checkDoubleRel(fDi,"fDi",0,"0",eps);
    UnitTests.checkDoubleRel(fiD,"fiD",0,"0",eps);
    //UnitTests.checkDoubleRel(fXDi,"fXDi",0,"0",eps);
    UnitTests.checkDoubleRel(fXiD,"fXiD",0,"0",eps);

    final double [] mm=getmufg(psi,M.B.getQQMatr(n,stmom),M.B.getQQMatr(n,spmomx),n,B);
    UnitTests.checkDoubleRel(ft,"ft",mm[3]/mm[0],"mm[3]/mm[0]",eps);
    final double ffta=(this.f-(mm[2]/mm[0])*this.pf);
    UnitTests.checkDoubleRel(ft,"ft",ffta,"ffta",eps);

    System.err.println("DkQVM:WkPsi:test_Self: test OK neq="+n+" eps="+eps);
}
void test_DSelf(final double eps,final java.util.Random r)
{
    final int n=psi.length;
    final double [] QQt=M.B.getQQMatr(n,stmom);
    final BasisFunctionsCalculatable B=getBasisPolynomials();
    final double [] p1=new double [n],p2=new double [n];
    final int n_tests=100;
    for(int t=0;t<n_tests;t++)
    {
	for(int k=0;k<n;k++){p1[k]=1-2*r.nextDouble();p2[k]=1-2*r.nextDouble();}
	final double a=TMatr.getAMatrB(p1,p2,QQt,n);
	final double [] pp=B.mult2Pol(p1,p2);
	final double b=getIntegratedToEnd(getXTestEndUnitTest(),pp);
	UnitTests.checkDoubleRel(a,"a",b,"b",eps);
    }
    UnitTests.checkDoubleRel(n1,"n1",1,"1",eps);
    System.err.println("DkQVM:WkPsi:test_DSelf: test OK neq="+n+" eps="+eps);
}
} // END OF class WkPsi


class WlPsi {

final double [] psi;
final double lam;

final double fg,f,g,mu,fgD,gD;

WlPsi(final double [] psi,final double lam,
      final double [] pimom,final double [] imom,
      final double [] ptmom,final double [] tmom)
{
    this.psi=psi;
    this.lam=lam;
    final int n=psi.length;
    final double [] momx=new double [imom.length],pmomx=new double [imom.length];
    for(int k=momx.length;--k>=0;)
    {
	momx[k]=imom[k]-(tmom!=null?this.lam*tmom[k]:0);
	pmomx[k]=pimom[k]-(ptmom!=null?this.lam*ptmom[k]:0);
    }
    final BasisFunctionsCalculatable B=getBasisPolynomials();
    final double [] QQx=M.B.getQQMatr(n,momx),pQQx=M.B.getQQMatr(n,pmomx);
    final double [] dpsi=B.getDifferentiated(psi);

    this.mu=TMatr.getAMatrA(psi,QQx,n);
    this.f=TMatr.getAMatrA(psi,pQQx,n)/this.mu;
    this.g=TMatr.getAMatrB(psi,dpsi,QQx,n)/this.mu;
    this.fg=TMatr.getAMatrB(psi,dpsi,pQQx,n)/this.mu-this.f*this.g;

    final double [] Dpsi=getDPsi(psi);
    this.gD=TMatr.getAMatrB(psi,Dpsi,QQx,n)/this.mu;
    this.fgD=TMatr.getAMatrB(psi,Dpsi,pQQx,n)/this.mu-this.f*this.gD;

    this.sQQx=QQx;
    this.spQQx=pQQx;
}
final double [] sQQx,spQQx;
void test_Self(final double eps)
{
    final int n=psi.length;
    UnitTests.checkDoubleRel(fg,"fg",getDFShift(psi,spQQx,sQQx,n),"var p",eps);

    System.err.println("DkQVM:WlPsi:test_Self: test OK neq="+n+" eps="+eps);
}
} // END OF class WlPsi




class WjPsi {

final EVXData I,J,K;
final double w_IH_JL,w_IH_JH,w_IL_JL,w_IL_JH,I_JL,I_JH;
final double x_IH,x_IL,x_JH,x_JL,x_KH,x_KL;
final double x_dp_IL,x_dp_IH,sigma2_dp_IL,sigma2_dp_IH,sigma2_x_IL,sigma2_x_IH;
final double dp_IL,dp_IH;


WjPsi(final int n, final double [] dpmom,final double [] imom,final double [] tmom)
{
    final double x0=getX0();
    final double [] QQdp=M.B.getQQMatr(n,dpmom),QQi=M.B.getQQMatr(n,imom),QQt=M.B.getQQMatr(n,tmom);
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);
    this.J=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQdp,QQt,access,x0);

    this.w_IH_JL=TMatr.getAMatrB(this.I.psiH,this.J.psiL,QQt,n);
    this.w_IH_JH=TMatr.getAMatrB(this.I.psiH,this.J.psiH,QQt,n);
    this.w_IL_JL=TMatr.getAMatrB(this.I.psiL,this.J.psiL,QQt,n);
    this.w_IL_JH=TMatr.getAMatrB(this.I.psiL,this.J.psiH,QQt,n);
    this.I_JL=TMatr.getAMatrA(this.J.psiL,QQi,n);
    this.I_JH=TMatr.getAMatrA(this.J.psiH,QQi,n);
    final double [] xtmom=M.B.getXQkMomentsFromQkMoments(tmom),
	xQQt=M.B.getQQMatr(n,xtmom),
	xQQdp=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(dpmom)),
	x2QQt=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(xtmom));
    this.x_IH=TMatr.getAMatrA(this.I.psiH,xQQt,n);
    this.sigma2_x_IH=TMatr.getAMatrA(this.I.psiH,x2QQt,n)-Math.pow(this.x_IH,2);
    this.x_IL=TMatr.getAMatrA(this.I.psiL,xQQt,n);
    this.sigma2_x_IL=TMatr.getAMatrA(this.I.psiL,x2QQt,n)-Math.pow(this.x_IL,2);

    this.x_JH=TMatr.getAMatrA(this.J.psiH,xQQt,n);
    this.x_JL=TMatr.getAMatrA(this.J.psiL,xQQt,n);

    final double sg=isEdgeAtRight()?1:-1;
    final double [] QQdp2=TMatr.mulMMM(n,QQdp,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),QQdp);

    this.dp_IH=TMatr.getAMatrA(this.I.psiH,QQdp,n);
    this.x_dp_IH=sg*(TMatr.getAMatrA(this.I.psiH,xQQdp,n)-this.x_IH*this.dp_IH);
    this.sigma2_dp_IH=TMatr.getAMatrA(this.I.psiH,QQdp2,n)-Math.pow(this.dp_IH,2);
    this.dp_IL=TMatr.getAMatrA(this.I.psiL,QQdp,n);
    this.x_dp_IL=sg*(TMatr.getAMatrA(this.I.psiL,xQQdp,n)-this.x_IL*this.dp_IL);
    this.sigma2_dp_IL=TMatr.getAMatrA(this.I.psiL,QQdp2,n)-Math.pow(this.dp_IL,2);

    final double [] QQdp0=new double [n*n],Q0=new double [n];
    final double fp0=TMatr.getAMatrA(this.I.psi0,QQdp,n)/Math.pow(M.B.getD0(this.I.psi0,x0),2);
    for(int k=0;k<n;k++)
    {
	final double [] tmp=new double [k+1];
	tmp[k]=1;
	Q0[k]=M.B.getD0(tmp,x0);
	for(int l=0;l<=k;l++)
	{
	    QQdp0[k*n+l]=QQdp0[l*n+k]=QQdp[k*n+l]-fp0*Q0[k]*Q0[l];
	}
    }
    this.K=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQdp0,QQt,access,x0);
    this.x_KH=TMatr.getAMatrA(this.K.psiH,xQQt,n);
    this.x_KL=TMatr.getAMatrA(this.K.psiL,xQQt,n);


    s_QQdp0=QQdp0;
    s_dpmom=dpmom;
    s_imom=imom;
    s_tmom=tmom;
}
    final double [] s_dpmom,s_imom,s_tmom,s_QQdp0;

void test_Self(final double eps)
{
    final int n=this.I.neq;
    UnitTests.checkDoubleRel(TMatr.getAMatrA(this.I.psi0,s_QQdp0,n),"<psi0|QQdp0|psi0>",0,"0",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(this.J.psi0,s_QQdp0,n),"<psi0|QQdp0|psi0>",0,"0",eps);
    UnitTests.checkDoubleRel(this.K.s0,"<psi0|QQdp0|psi0>",0,"0",eps);

    final double [] momdpH=M.B.getXQkMomentsFromQkMoments(s_dpmom);
    DeprecatedHelperTMatr.sumP1(momdpH,s_dpmom,momdpH.length,-this.x_IH);
    final double sg=isEdgeAtRight()?1:-1;
    final double x_dp_iH=sg*TMatr.getAMatrA(this.I.psiH,M.B.getQQMatr(n,momdpH),n);
    UnitTests.checkDoubleRel(x_dp_iH,"x_dp_iH",this.x_dp_IH,"this.x_dp_IH",eps);


    final double [] momdpL=M.B.getXQkMomentsFromQkMoments(s_dpmom);
    DeprecatedHelperTMatr.sumP1(momdpL,s_dpmom,momdpL.length,-this.x_IL);
    final double x_dp_iL=sg*TMatr.getAMatrA(this.I.psiL,M.B.getQQMatr(n,momdpL),n);
    UnitTests.checkDoubleRel(x_dp_iL,"x_dp_iL",this.x_dp_IL,"this.x_dp_IL",eps);

    final double [] QQt=M.B.getQQMatr(n,s_tmom),xtmom=M.B.getXQkMomentsFromQkMoments(s_tmom),
	xQQt=M.B.getQQMatr(n,xtmom),x2QQt=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(xtmom)),
	QQdp=M.B.getQQMatr(n,s_dpmom),xQQdp=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(s_dpmom)),
	QQdp2=TMatr.mulMMM(n,QQdp,com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQt),QQdp);
    for(int k=0;k<2;k++)
    {
	final double [] psi=k<=0?this.I.psiL:this.I.psiH;
	UnitTests.checkDoubleRel(TMatr.getAMatrA(psi,QQt,n),"<psi|QQt|psi>",1,"1",eps);
	final double dp=TMatr.getAMatrA(psi,QQdp,n),x=TMatr.getAMatrA(psi,xQQt,n),
	    xdp=sg*(TMatr.getAMatrA(psi,xQQdp,n)-x*dp),
	    sigma2_dp=TMatr.getAMatrA(psi,QQdp2,n)-dp*dp,
	    sigma2_x=TMatr.getAMatrA(psi,x2QQt,n)-x*x;
	final double q_x,q_xdp,q_dp,q_sigma2_dp,q_sigma2_x;
	if(k==0)
	{
	    q_x=this.x_IL;
	    q_xdp=this.x_dp_IL;
	    q_dp=this.dp_IL;
	    q_sigma2_dp=this.sigma2_dp_IL;
	    q_sigma2_x=this.sigma2_x_IL;
	}
	else
	{
	    q_x=this.x_IH;
	    q_xdp=this.x_dp_IH;
	    q_dp=this.dp_IH;
	    q_sigma2_dp=this.sigma2_dp_IH;
	    q_sigma2_x=this.sigma2_x_IH;
	}
	UnitTests.checkDoubleRel(x,"x",q_x,"q_x",eps);
	UnitTests.checkDoubleRel(xdp,"xdp",q_xdp,"q_xdp",eps);
	UnitTests.checkDoubleRel(dp,"dp",q_dp,"q_dp",eps);
	UnitTests.checkDoubleRel(sigma2_dp,"sigma2_dp",q_sigma2_dp,"q_sigma2_dp",eps);
	UnitTests.checkDoubleRel(sigma2_x,"sigma2_x",q_sigma2_x,"q_sigma2_x",eps);
    }


    if(this.sigma2_dp_IH<0 || this.sigma2_dp_IL<0)
    {
	throw new RuntimeException("this.sigma2_dp_IH="+this.sigma2_dp_IH+" this.sigma2_dp_IL="+this.sigma2_dp_IL);
    }
    for(int k=0;k<this.I.neq;k++)
    {
	final double [] pp=new double [this.I.neq];
	System.arraycopy(this.I.ev,this.I.neq+k*this.I.neq,pp,0,this.I.neq);
	final WkePsi b=new WkePsi(pp,s_tmom,s_imom,s_imom,s_imom);
	UnitTests.checkDoubleRel(b.dPf,"b.dPf",0,"0",eps);
    }

    System.err.println("DkQVM:WjPsi:test_Self: test OK n="+this.I.neq+" eps="+eps);
}
} // END OF class WjPsi


/** Perturbation to p by I.
 */
class WDmuPsi {
    final double D=0.95;
    final EVXData I;
    final QuadEV Q,QD;
    final double p_x,p_Dx,Dp_x,Dmu;
    final double Q_getDP_NoMu,Q_getDP_NoDpsi,Q_getXiSens,Q_getXiSensAver,Q0_getDP_NoMu,Q0_getDP_NoDpsi,Q0_getXiSens,Q_p_x_getFAverFromQQ;
    final int ind_type;
    WDmuPsi(final int neq,final double [] imom,final double [] tmom,
	    final double [] pimom,final double [] ptmom,final int ind_type)
    {
	final double x0=getX0();
	this.ind_type=ind_type;
	final double [] QQi=M.B.getQQMatr(neq,imom),QQt=M.B.getQQMatr(neq,tmom);
	this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQi,QQt,access,x0);
	final double lam=I.sL;
	final int n=neq-1;
	final double [] momx=new double [2*n],momDx=new double [2*n],
	    pmomx=new double [2*n],pmomDx=new double [2*n];
	for(int k=0;k<momx.length;k++)
	{
	    momx[k]=imom[k]-lam*tmom[k];
	    momDx[k]=imom[k]-D*lam*tmom[k];
	    pmomx[k]=pimom[k]-lam*ptmom[k];
	    pmomDx[k]=pimom[k]-D*lam*ptmom[k];
	}
    this.Q=new QuadEV(n,momx,ind_type);
    this.QD=new QuadEV(n,momDx,ind_type);
    final double [] QQpx=M.B.getQQMatr(n,pmomx),QQpDx=M.B.getQQMatr(n,pmomDx);
    //final double [] QQx=M.B.getQQMatr(n,momx),QQDx=M.B.getQQMatr(n,momDx);
    //System.err.println("nrm="+TMatr.getAMatrA(Q.psi,QQx,n));
    //System.err.println("nrmD="+TMatr.getAMatrA(QD.psi,QQDx,n));
    this.p_x=TMatr.getAMatrA(Q.psi,QQpx,n);
    this.p_Dx=TMatr.getAMatrA(QD.psi,QQpDx,n);
    this.Dp_x=(p_x-p_Dx)/(1-D);
    this.Dmu=(Q.mu-QD.mu)/(1-D);

    this.Q_getDP_NoMu=lam*this.Q.getDP_NoMu(pmomx,tmom);
    this.Q_getDP_NoDpsi=lam*this.Q.getDP_NoDpsi(pmomx,ptmom,tmom);
    this.Q_getXiSens=this.Q.getXiSens(pmomx);
    this.Q_getXiSensAver=this.Q.getXiSensAver(pmomx);
    this.Q_p_x_getFAverFromQQ=this.Q.getFAverFromQQ(QQpx,n);

    final double [] v0mom=new double [tmom.length],pv0mom=new double [tmom.length];
    for(int k=0;k<v0mom.length;k++)
    {
	final double [] tn=new double [k+1];
	tn[k]=1;
	v0mom[k]=M.B.getD0(tn,x0);
	pv0mom[k]=(0-0)*v0mom[k]; // assume p_offset=p_last
    }
    this.Q0_getDP_NoMu=lam*this.Q.getDP_NoMu(pmomx,v0mom);
    this.Q0_getDP_NoDpsi=lam*this.Q.getDP_NoDpsi(pmomx,pv0mom,v0mom);
    this.Q0_getXiSens=this.Q.getXiSens(pv0mom);

    simom=imom;
    stmom=tmom;
    spimom=pimom;
    sptmom=ptmom;
}
final double [] simom,stmom,spimom,sptmom;
void test_Self(final double eps)
{
    if(!(this.Q.mu!=0)||Double.isNaN(this.Q.mu)) return;

    WmuPsi q=new WmuPsi(this.I.neq,simom,stmom,spimom,sptmom);
    final double [] pmumoms=new double [spimom.length],mumoms=new double [pmumoms.length];
    for(int k=pmumoms.length;--k>=0;){
	pmumoms[k]=spimom[k]-this.I.sL*sptmom[k];
	mumoms[k]=simom[k]-this.I.sL*stmom[k];
    }
    final double [] pxmu=q.jL.getF(pmumoms,false,WABpsi.MT_MU);
    UnitTests.checkDoubleRel(TMatr.mulVVn1(pxmu,q.jL.Mu)/TMatr.sumElements(q.jL.Mu),"<px>_mu",Q_p_x_getFAverFromQQ,"Q_p_x_getFAverFromQQ",eps);

    final double pt_px=this.Q.getDP_NoDpsi(pmumoms,sptmom,stmom);
    if(ind_type==QuadEV.IND_CLOSE)
    {
	UnitTests.checkDoubleRel(q.jL.fXx,"q.jL.fXx",Q.fXx,"Q.fXx",eps);
	UnitTests.checkDoubleRel(q.mu,"q.mu",Q.mu,"Q.mu",eps);
	UnitTests.checkDoubleRel(q.p_x,"q.p_x",p_x,"p_x",eps);
	UnitTests.checkDoubleRel(pt_px,"pt_px",(q.p_t-q.p_x)*(q.jL.fBx/q.jL.muX),"(q.p_t-q.p_x)*(q.jL.fBx/q.jL.muX)",eps);
	UnitTests.checkDoubleRel(q.xXD0,"q.xXD0",Q_getXiSens,"Q_getXiSens",eps);

	final PsiAverPertr p=new PsiAverPertr(this.I.neq-1,this.I.sL,1,
					      simom,stmom,spimom,sptmom,null,null,M,isEdgeAtRight(),getX0());
	UnitTests.checkDoubleRel(p.fXx,"p.fXx",Q.fXx,"Q.fXx",eps);
	UnitTests.checkDoubleRel(p.mu,"p.mu",Q.mu,"Q.mu",eps);
	final double mm=-1;
	UnitTests.checkDoubleRel(p.pertpsi*mm,"p.pertpsi*mm",Q_getDP_NoMu,"Q_getDP_NoMu",eps);
	UnitTests.checkDoubleRel(p.pertp*mm,"p.pertp*mm",Q_getDP_NoDpsi,"Q_getDP_NoDpsi",eps);
    }
    else if(ind_type==QuadEV.IND_WMAX)
    {
	UnitTests.checkDoubleRel(q.fXx_Max,"q.fXx_Max",Q.fXx,"Q.fXx",eps);
	UnitTests.checkDoubleRel(q.mu_Max,"q.mu_Max",Q.mu,"Q.mu",eps);
	UnitTests.checkDoubleRel(q.p_x_Max,"q.p_x_Max",p_x,"p_x",eps);
	UnitTests.checkDoubleRel(pt_px,"pt_px",(q.p_t_Max-q.p_x_Max)*(q.Bt_Max/q.mu_Max),"(q.p_t_Max-q.p_x_Max)*(q.Bt_Max/q.mu_Max)",eps);
    }


    final double dX=this.Q.getDP_NoMu(M.B.getXQkMomentsFromQkMoments(mumoms),stmom);
    UnitTests.checkDoubleRel(dX,"dX",0,"0",eps);

    final double pp=TMatr.getAMatrB(this.Q.n,this.Q.getDPsiOrder1(simom),this.Q.psi,this.Q.sQQ,this.Q.n);
    UnitTests.checkDoubleRel(pp,"pp",0,"0",eps);
    double [] dpsi=this.Q.getDPsiOrder1(mumoms);
    UnitTests.checkDoubleArrayRel(dpsi,"dpsi",new double [dpsi.length],"0",eps);
    dpsi=this.Q.getDPsiOrder1(M.B.getXQkMomentsFromQkMoments(mumoms));
    UnitTests.checkDoubleArrayRel(dpsi,"dpsi",new double [dpsi.length],"0",eps);


    final double ppD=TMatr.getAMatrB(this.QD.n,this.QD.getDPsiOrder1(simom),this.QD.psi,this.QD.sQQ,this.QD.n);
    UnitTests.checkDoubleRel(ppD,"ppD",0,"0",eps);

    double s=0,sw=0;
    for(int k=0;k<Q.n;k++)
    {
	final int kk=k;
	final double f=new QuadEV(Q.n,mumoms,-1){
		int getIndX(final double [] x,final double [] w,final int ind_type){return kk;}
	    }.getXiSens(pmumoms);
	s+=f*Q.w[k];
	sw+=Q.w[k];
    }
    final double sens=s/sw;
    UnitTests.checkDoubleRel(sens,"sens",Q.getXiSensAver(pmumoms),"Q.getXiSensAver(pmumoms)",eps);

    UnitTests.checkDoubleRel(1,"1",Q.getFAverFromQQ(M.B.getQQMatr(Q.n,mumoms),Q.n),"Q.getFAverFromQQ(mu)",eps);
    // assume the first element in basis in 1.
    UnitTests.checkDoubleRel(mumoms[0],"mumoms[0]",TMatr.sumElements(Q.w),"sum(Q.w)",eps);


    System.err.println("DkQVM:WDmuPsi:test_Self: test OK neq="+this.I.neq+" eps="+eps);
}
}// END OF class WDmuPsi

class QuadEV {
final double fXx,mu;
final int n,indX;
final double [] ev;
final double [] psi,x,w;
static final int IND_CLOSE=34,IND_WMAX=35;
QuadEV(final int n,final double [] moments,final int ind_type)
{
    this.n=n;
    final double [] QQ=M.B.getQQMatr(n,moments),XQQ=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(moments));
    this.ev=M.EV.getGEV_Generalized(n,XQQ,QQ,true);
    this.x=new double [n];
    this.w=new double [n];
    this.psi=new double [n];
    for(int k=0;k<n;k++)
    {
	this.x[k]=this.ev[k];
	System.arraycopy(this.ev,n+k*n,this.psi,0,n);
	this.w[k]=1/Math.pow(M.B.getD0(this.psi,this.x[k]),2);
    }
    this.indX=getIndX(this.x,this.w,ind_type);
    if(indX>=0)
    {
	this.fXx=this.x[indX];
	this.mu=this.w[indX];
	System.arraycopy(this.ev,n+indX*n,this.psi,0,n);
    }
    else
    {
	this.fXx=this.mu=Double.NaN;
	java.util.Arrays.fill(this.psi,Double.NaN);
    }
    this.sQQ=QQ;
}
final double [] sQQ;
int getIndX(final double [] x,final double [] w,final int ind_type)
{
    if(ind_type==IND_CLOSE)
    {
	return DjQVM.getIndX0_NonSortedX(x,getX0(),isEdgeAtRight());
    }
    else if(ind_type==IND_WMAX)
    {
	return DjQVM.getIndMax(w);
    }
    else throw new RuntimeException("ind_type="+ind_type);
}
/** First order perturbation theory.
 */
double [] getQQPertr(final double [] V)
{
    final double [] mm=M.B.getXQkMomentsFromQkMoments(V);
    final double lamX=this.indX<0?Double.NaN:this.ev[this.indX];
    for(int k=mm.length;--k>=0;)
    {
	mm[k]=mm[k]-lamX*V[k];
    }
    return M.B.getQQMatr(n,mm);
}
double [] getDPsiOrder1(final double [] V)
{
    return getDPsiOrder1FromQQ(getQQPertr(V),n);
}
double [] getDPsiOrder1FromQQ(final double [] QQ,final int ldQQ)
{
    final double [] dpsi=new double [n];
    if(this.indX<0)
    {
	java.util.Arrays.fill(dpsi,Double.NaN);
	return dpsi;
    }

    for(int k=0;k<n;k++)
    {
	if(k!=this.indX)
	{
	    final double s=TMatr.getAMatrB(n,this.ev,n+this.indX*n,n,this.ev,n+k*n,QQ,ldQQ)/(this.ev[this.indX]-this.ev[k]);
	    DeprecatedHelperTMatr.sumP1(dpsi,this.ev,n+k*n,n,s);
	}
    }
    return dpsi;
}
double getXiSens(final double [] pmumoms)
{
    final double [] QQ=getQQPertr(pmumoms);
    return TMatr.getAMatrB(n,this.ev,n+this.indX*n,n,this.ev,n+this.indX*n,QQ,n);
}
double getXiSensAver(final double [] pmumoms)
{
    final double [] QQ=M.B.getQQMatr(n,pmumoms),XQQ=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(pmumoms));
    double s=0,sw=0;
    for(int k=0;k<n;k++)
    {
	final double f=TMatr.getAMatrB(n,this.ev,n+k*n,n,this.ev,n+k*n,XQQ,n)-
	    this.x[k]*TMatr.getAMatrB(n,this.ev,n+k*n,n,this.ev,n+k*n,QQ,n);
	s+=f*w[k];
	sw+=w[k];
    }
    return s/sw;
}
double getFAverFromQQ(final double [] fmumomsQQ,final int ldQQ)
{
    double s=0,sw=0;
    for(int k=0;k<n;k++)
    {
	final double f=TMatr.getAMatrB(n,this.ev,n+k*n,n,this.ev,n+k*n,fmumomsQQ,ldQQ);
	s+=f*w[k];
	sw+=w[k];
    }
    return s/sw;
}


double getDP_NoMu(final double [] pmumoms,final double [] dmumoms)
{
    final double [] PQQ=M.B.getQQMatr(n,pmumoms),dpsi=getDPsiOrder1(dmumoms);
    final double px=TMatr.getAMatrB(this.psi,this.psi,PQQ,n);
    return 2*(
	TMatr.getAMatrB(this.psi,dpsi,PQQ,n)-
	px*TMatr.getAMatrB(this.psi,dpsi,sQQ,n)
			   );

}
double getDP_NoDpsi(final double [] pmumoms,final double [] pdmumoms,final double [] dmumoms)
{
    final double [] PQQ=M.B.getQQMatr(n,pmumoms),PQQd=M.B.getQQMatr(n,pdmumoms),QQd=M.B.getQQMatr(n,dmumoms);
    final double px=TMatr.getAMatrB(this.psi,this.psi,PQQ,n);
    return TMatr.getAMatrB(this.psi,this.psi,PQQd,n)-
	px*TMatr.getAMatrB(this.psi,this.psi,QQd,n);
}


} // END OF class QuadEV



final GetVolMatrix VOLATILITY_EDPsi=new GetVolMatrix(){
	public VolMatrix getVolatility(final double x){return new VolMatrix(){
		@Override
		public double [] getDPsi(final double [] psi){return getEDPsi(psi);}
	    };
	}},
    VOLATILITY_EDPsiX=new GetVolMatrix(){
	public VolMatrix getVolatility(final double x){
	    return new VolMatrix(){
		@Override
		public double [] getDPsi(final double [] psi){return getEDPsiX(psi,x);}
	    };
	}};


/** Try petrurbation theory.
 */
class WkkEV {
    static final int PERTRi=74,PERTRt=75;
    final EVXData I;
    final double fXx,Pf,dPf,sigma2b_Pf,sigma2_g;
    final int flag_LH,ind_lev,pertr_type;
    final double [] Pl,Il;
    final VolMatrix volatility;
    WkkEV(final int n,final double [] imom,final double [] tmom,final double [] fmom0,final double [] fmom1,final double [] fmom2,final int flag_LH,final int pertr_type,final GetVolMatrix getvolatility)
    {
	final double [] QQi=M.B.getQQMatr(n,imom),QQt=M.B.getQQMatr(n,tmom);
	final double x0=getX0();
	this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,x0);
	this.pertr_type=pertr_type;
	final double [] QQpertr;
	if(pertr_type==WkkEV.PERTRi) QQpertr=QQi;
	else if(pertr_type==WkkEV.PERTRt) QQpertr=QQt;
	else throw new RuntimeException("pertr_type="+pertr_type);

	this.flag_LH=flag_LH;
	if(flag_LH==DelaysBecauseOfSapsanEV.LH_LOW){ind_lev=0;}
	else if(flag_LH==DelaysBecauseOfSapsanEV.LH_HIGH){ind_lev=n-1;}
	else throw new RuntimeException("flag_LH="+flag_LH);

	final double [] psiind=new double [n];
	System.arraycopy(this.I.ev,n+ind_lev*n,psiind,0,n);
	final double [] Xx=M.getXNodes(psiind).re_root;
	final int indX=DjQVM.getIndX0_NonSortedX(Xx,x0,isEdgeAtRight());
	this.fXx=indX>=0?Xx[indX]:Double.NaN;

	this.volatility=getvolatility.getVolatility(this.fXx);
	final double [] DQQD=volatility.getVolMatrix(n,QQpertr),QQf1=M.B.getQQMatr(n,fmom1);

	this.Pf=this.I.getFPsi(ind_lev,QQf1);
	final double [] psipert=DeprecatedHelperEVXData.getPsiPertr(this.I,ind_lev,DQQD);
	this.dPf=this.I.getFPsiPsi(ind_lev,psipert,QQf1);
	this.sigma2_g=TMatr.getAMatrA(psipert,QQt,n);
	final double [] QQf0=M.B.getQQMatr(n,fmom0),QQf2=M.B.getQQMatr(n,fmom2);
	this.sigma2b_Pf=this.I.getFPsi(ind_lev,QQf0)*this.I.getFPsi(ind_lev,QQf2)-this.Pf*this.Pf;
	this.Pl=new double [n];
	this.Il=new double [n];
	for(int k=0;k<n;k++)
	{
	    this.Il[k]=this.I.getFPsi(k,QQi);
	    this.Pl[k]=this.I.getFPsi(k,QQf1)/this.I.getFPsi(k,QQf0);
	}

	this.simom=imom;
	this.stmom=tmom;
	this.sf0mom=fmom0;
	this.sf1mom=fmom1;
	this.sf2mom=fmom2;
    }
final double [] simom,stmom,sf0mom,sf1mom,sf2mom;


void test_Self(final double eps)
{
    final int n=this.I.neq;
    final double [] psi;
    if(flag_LH==DelaysBecauseOfSapsanEV.LH_LOW)
    {
	psi=I.psiL;
    }
    else if(flag_LH==DelaysBecauseOfSapsanEV.LH_HIGH)
    {
	psi=I.psiH;
    }
    else throw new RuntimeException("flag_LH="+flag_LH);

    for(int jj=0;jj<2;jj++) {
	final double [] spertr=(pertr_type==WkkEV.PERTRi)?simom:stmom;
	final double [] VV;
	if(jj<=0) {VV=volatility.getVolMatrix(n,M.B.getQQMatr(n,spertr));}
	else{
	    VV=new double [n*n];
	    final double [] pp=new double [n];
	    final BasisFunctionsCalculatable B=getBasisPolynomials();
	    for(int k=0;k<n;k++)
	    {
		pp[k]=1;final double [] dpsik=volatility.getDPsi(pp);pp[k]=0;
		for(int l=0;l<n;l++)
		{
		    pp[l]=1;final double [] dpsil=volatility.getDPsi(pp);pp[l]=0;
		    final double [] m=B.mult2Pol(dpsik,dpsil);
		    VV[k*n+l]=TMatr.mulVVn1(m,spertr);
		}
	    }
	}
	//System.err.println("VV="+java.util.Arrays.toString(VV));
	final WkkPsi q=new WkkPsi(psi,stmom,simom,sf1mom,sf2mom){
		double [] getDpsi(final double [] apsi,final double xk)
		{
		    return new DelaysBecauseOfSapsanEV(n,WkkEV.this.simom,WkkEV.this.stmom,flag_LH).getDPsiOrder1FromQQ(VV,n);
		}
	    };
	UnitTests.checkDoubleRel(Pf,"Pf",q.Pf,"q.Pf_"+jj,eps);
	UnitTests.checkDoubleRel(dPf,"dPf",q.dPf,"q.dPf_"+jj,eps);
	UnitTests.checkDoubleRel(sigma2_g,"sigma2_g",q.sigma2_g,"q.sigma2_g_"+jj,eps);
	if(simom==sf0mom) {
	    UnitTests.checkDoubleRel(sigma2b_Pf,"sigma2b_Pf",q.sigma2b_Pf,"q.sigma2b_Pf_"+jj,eps);
	}
    }

    System.err.println("DkQVM:WkkEV:test_Self: test OK n="+n+" eps="+eps);
}
} // END OF class WkkEV


double [] getRandomPositivelyDefined(final int nmom,final int nsum,final java.util.Random r)
{
    return getRandomPositivelyDefined(nmom,nsum,r,access,getX0(),getXTestEnd(nmom/2+1));
}

static double [] getRandomPositivelyDefined(final int nmom,final int nsum,final java.util.Random r,final DeprecatedHelperDataDirectAccessWithBasis<?> access,final double xa,final double xb)
{
    final double [] moms=new double [nmom],x=new double [nmom];
    for(int t=0;t<nsum;t++)
    {
	final double xt=xa+(xb-xa)*r.nextDouble(),w=r.nextDouble();
	//System.err.println("xt="+xt+" w="+w);
	for(int k=0;k<nmom;k++)
	{
	    x[k]=1;
	    moms[k]+=w*access.fx(x,xt);
	    x[k]=0;
	}
    }
    return moms;
}

static double [] getRandomNonDegeneratedMatrix(final int n,final java.util.Random r)
{
    final double [] basis=new double [n*n];
    for(int k=0;k<n;k++){
	for(int l=0;l<n;l++){
	    basis[k*n+l]=1-2*r.nextDouble();
	}
	basis[k*n+k]+=3+6*r.nextDouble();
    }
    return basis;
}


double [] getBasisCondFromEDPsi(final int n)
{
    final int nd=n-1;
    final double [] basis=new double [n*nd],pp=new double [n];
    final double x0=getX0();
    double dq0=Double.NaN;
    for(int k=0,kk=0;k<n;k++)
    {
	pp[k]=1;final double [] dpsik=getEDPsi(pp);pp[k]=0;
	final double dpp=M.B.getD0(dpsik,x0);
	if(k<=0){dq0=dpp;}
	else
	{
	    basis[n*kk+0]=-dpp/dq0;
	    basis[n*kk+k]=1;
	    kk++;
	}
    }
    return basis;
}
/** Return a basis satisfying boundary conditions.
 */
abstract double [] getBasisFuncsBoundaryCond(final int n);
/** Try petrurbation theory.
 */


class WkkBoundary {
    final EVXData I;
    final double fXx,Pf,dPf,sigma2_g,sigma2b_Pf,Paf,Ia,Pbf,Ib;
    final int flag_LH,ind_lev,pertr_type,n;
    final double [] psiind;
    final GetVolMatrix getvolatility;
    WkkBoundary(final int n,final double [] imom,final double [] tmom,final double [] fmom0,final double [] fmom1,final double [] fmom2,final int flag_LH,final int pertr_type,final GetVolMatrix getvolatility)
    {
	this.getvolatility=getvolatility;
	final double [] basis=getBasisCond(n); // basis: nd x n matrix
	final int nd=basis.length/n;
	this.n=n;
	this.pertr_type=pertr_type;
	final double [] QQi=new double [nd*nd],QQt=new double [nd*nd],tmp=new double [n*nd];
	// convert to the basis with proper boundary conditions
	for(double [][] mm:new double [][][]{
		{QQi,M.B.getQQMatr(n,imom)},{QQt,M.B.getQQMatr(n,tmom)}
	    })
	{
	    final double [] m=mm[1],mr=mm[0];
	    // m: n x n matrix
	    TMatr.multiply_MA_x_MBTrans(tmp,n,m,n,basis,nd); // tmp: n x nd
	    TMatr.multiply_MA_x_MB(mr,nd,basis,n,tmp,nd); // m: nd x nd
	    // The result is saved back to m, which is now nd x nd matrix in new basis.
	}
	final double x0=getX0();
	this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(nd,QQi,QQt,access,x0);
	this.flag_LH=flag_LH;
	if(flag_LH==DelaysBecauseOfSapsanEV.LH_LOW){ind_lev=0;}
	else if(flag_LH==DelaysBecauseOfSapsanEV.LH_HIGH){ind_lev=nd-1;}
	else throw new RuntimeException("flag_LH="+flag_LH);

	this.psiind=new double [n];
	for(int k=0;k<nd;k++)
	{
	    DeprecatedHelperTMatr.sumP1(psiind,basis,k*n,n,this.I.ev[nd+ind_lev*nd+k]);
	}
	final double [] Xx=M.getXNodes(psiind).re_root;
	final int indX=DjQVM.getIndX0_NonSortedX(Xx,x0,isEdgeAtRight());
	this.fXx=indX>=0?Xx[indX]:Double.NaN;

	final double [] pertrmom;
	if(pertr_type==WkkEV.PERTRi) pertrmom=imom;
	else if(pertr_type==WkkEV.PERTRt) pertrmom=tmom;
	else throw new RuntimeException("pertr_type="+pertr_type);
	final VolMatrix volatility=getvolatility.getVolatility(this.fXx);

	final double [] DQQD=new double [nd*nd],QQf0=new double [nd*nd],QQf1=new double [nd*nd],QQf2=new double [nd*nd];
	// convert to the basis with proper boundary conditions
	for(double [][] mm:new double [][][]{
		{DQQD,volatility.getVolMatrix(n,M.B.getQQMatr(n,pertrmom))},
		{QQf0,M.B.getQQMatr(n,fmom0)},
		{QQf1,M.B.getQQMatr(n,fmom1)},
		{QQf2,M.B.getQQMatr(n,fmom2)}
	    })
	{
	    final double [] m=mm[1],mr=mm[0];
	    // m: n x n matrix
	    TMatr.multiply_MA_x_MBTrans(tmp,n,m,n,basis,nd); // tmp: n x nd
	    TMatr.multiply_MA_x_MB(mr,nd,basis,n,tmp,nd); // m: nd x nd
	}

	this.Pf=this.I.getFPsi(ind_lev,QQf1);
	final double [] psipert=DeprecatedHelperEVXData.getPsiPertr(this.I,ind_lev,DQQD);
	this.dPf=this.I.getFPsiPsi(ind_lev,psipert,QQf1);
	this.sigma2_g=TMatr.getAMatrA(psipert,QQt,nd);
	this.sigma2b_Pf=this.I.getFPsi(ind_lev,QQf0)*this.I.getFPsi(ind_lev,QQf2)-this.Pf*this.Pf;


	Paf=this.I.getFPsi(ind_lev,QQf1)/this.I.getFPsi(ind_lev,QQf0);
	Ia=this.I.getFPsi(ind_lev,QQi);
	Pbf=TMatr.getAMatrA(psipert,QQf1,nd)/TMatr.getAMatrA(psipert,QQf0,nd);
	Ib=TMatr.getAMatrA(psipert,QQi,nd)/TMatr.getAMatrA(psipert,QQt,nd);

	// for unit tests
	this.simom=imom;
	this.stmom=tmom;
	this.sf0mom=fmom0;
	this.sf1mom=fmom1;
	this.sf2mom=fmom2;
	sQQt=QQt;
	sQQi=QQi;
	sDQQD=DQQD;
	sQQf0=QQf0;
	sQQf1=QQf1;
	sQQf2=QQf2;
    }

    /** A simple version of basis: basik_k=Q_k - cQ_0
     *  such that fx(getEDPsi(basis_k),x0)=0.
     */
    double [] getBasisCond(final int n)
    {
	return getBasisCondFromEDPsi(n);
	//return getBasisFuncsBoundaryCond(n);
    }
    final double [] simom,stmom,sf0mom,sf1mom,sf2mom,sQQt,sQQi,sDQQD,sQQf0,sQQf1,sQQf2;

    void match_WkkBoundary(final WkkBoundary q,final double eps)
    {
	EVXData.matchEVXData(this.I,q.I,false,false,eps);
	UnitTests.checkDoubleRel(Pf,"Pf",q.Pf,"q.Pf",eps);
	UnitTests.checkDoubleRel(dPf,"dPf",q.dPf,"q.dPf",eps);
	UnitTests.checkDoubleRel(sigma2_g,"sigma2_g",q.sigma2_g,"q.sigma2_g",eps);
	UnitTests.checkDoubleRel(sigma2b_Pf,"sigma2b_Pf",q.sigma2b_Pf,"q.sigma2b_Pf",eps);
	System.err.println("DkQVM:WkkBoundary:match_WkkBoundary: test OK n="+n+" eps="+eps);
    }

void test_Self(final java.util.Random r,final double eps)
{
    // Results in different basises match.
    if(true){
	WkkBoundary q=new WkkBoundary(n,simom,stmom,sf0mom,sf1mom,sf2mom,flag_LH,pertr_type,getvolatility)
	{
	    @Override
	    double [] getBasisCond(final int n){return getBasisFuncsBoundaryCond(n);}
	};
	this.match_WkkBoundary(q,eps);
	q=new WkkBoundary(n,simom,stmom,sf0mom,sf1mom,sf2mom,flag_LH,pertr_type,getvolatility)
	{
	    @Override
	    double [] getBasisCond(final int n){
		final double [] basis=super.getBasisCond(n);
		final int nd=basis.length/n;
		final double [] m=getRandomNonDegeneratedMatrix(nd,r);
		final double [] re=new double [nd*n];
		TMatr.multiply_MA_x_MB(re,nd,m,nd,basis,n);
		return re;
	    }
	};
	this.match_WkkBoundary(q,eps);
    }

    // Same dimension basis must match WkkEV
    if(true){
	final WkkBoundary q=new WkkBoundary(n,simom,stmom,sf0mom,sf1mom,sf2mom,flag_LH,pertr_type,getvolatility)
	{
	    /** Try different basis. */
	    @Override
	    double [] getBasisCond(final int n){return getRandomNonDegeneratedMatrix(n,r);}
	};
	final WkkEV s=new WkkEV(n,simom,stmom,sf0mom,sf1mom,sf2mom,flag_LH,pertr_type,getvolatility);
    EVXData.matchEVXData(s.I,q.I,false,false,eps);
    UnitTests.checkDoubleRel(s.Pf,"s.Pf",q.Pf,"q.Pf",eps);
    UnitTests.checkDoubleRel(s.dPf,"s.dPf",q.dPf,"q.dPf",eps);
    UnitTests.checkDoubleRel(s.sigma2_g,"s.sigma2_g",q.sigma2_g,"q.sigma2_g",eps);
    UnitTests.checkDoubleRel(s.sigma2b_Pf,"s.sigma2b_Pf",q.sigma2b_Pf,"q.sigma2b_Pf",eps);
    }

    // Test QQ matrices in new basis
    if(true)
    {
	// volatility matrix
	final double [] basis=getBasisCond(n);
	final int nd=basis.length/n;
	final double [] pp=new double [n];
	final double x0=getX0();
	final BasisFunctionsCalculatable B=getBasisPolynomials();
	final double [] spertr=(pertr_type==WkkEV.PERTRi)?simom:stmom;
	final VolMatrix volatility=getvolatility.getVolatility(this.fXx);
	for(int k=0;k<nd;k++){
	    System.arraycopy(basis,k*n,pp,0,n);
	    final double [] dpsik=volatility.getDPsi(pp);
	    // check boundary condition
	    UnitTests.checkDoubleRel(M.B.getD0(getEDPsi(pp),x0),"getEDPsi(pp)(x0)",0,"0",eps);
	    UnitTests.checkDoubleRel(M.B.getD0(getEDPsi(psiind),x0),"getEDPsi(psiind)(x0)",0,"0",eps);
	    if(!Double.isNaN(fXx)) UnitTests.checkDoubleRel(M.B.getD0(psiind,fXx),"psiind(fXx)",0,"0",eps);
	    for(int l=0;l<nd;l++)
	    {
		System.arraycopy(basis,l*n,pp,0,n);
		final double [] dpsil=volatility.getDPsi(pp);
		final double [] m=B.mult2Pol(dpsik,dpsil);
		final double qq=TMatr.mulVVn1(m,spertr);
		// check volatility matrix element match
		UnitTests.checkDoubleRel(sDQQD[k*nd+l],"sDQQD[k*nd+l]",qq,"qq",eps);
	    }
	}
	// other matrices
	final double [][] sQQ=new double [][]{sQQt,sQQi,sQQf0,sQQf1,sQQf2};
	final double [][] moms=new double [][]{stmom,simom,sf0mom,sf1mom,sf2mom};
	final double [] pk=new double [n],pl=new double [n];
	for(int t=0;t<moms.length;t++)
	{
	    final double [] QQ=sQQ[t],mom=moms[t];
	    for(int k=0;k<nd;k++)
	    {
		System.arraycopy(basis,k*n,pk,0,n);
		for(int l=0;l<nd;l++)
		{
		    System.arraycopy(basis,l*n,pl,0,n);
		    final double [] m=B.mult2Pol(pk,pl);
		    final double qq=TMatr.mulVVn1(m,mom);
		    // check matrix element match
		    UnitTests.checkDoubleRel(QQ[k*nd+l],"QQ[k*nd+l]",qq,"qq",eps);
		}
	    }
	}
    }


    System.err.println("DkQVM:WkkBoundary:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class WkkBoundary





class WkkVolBoundary {
    final EVXData I;
    final double dPf;
    final int flag_LH,ind_lev,n;
    WkkVolBoundary(final int n,final double [] imom,final double [] tmom,final double [] fmom0,final double [] fmom1,final double [] fmom2,final int flag_LH)
    {
	final double [] basis=getBasisCond(n); // basis: nd x n matrix
	final int nd=basis.length/n;
	this.n=n;
	final double [] _QQi=M.B.getQQMatr(n,imom),_QQf1=M.B.getQQMatr(n,fmom1),tmp=new double [n*nd],
	    QQi=new double [nd*nd],QQt=new double [nd*nd],DQQD=new double [nd*nd],QQf0=new double [nd*nd],QQf1=new double [nd*nd],QQf2=new double [nd*nd],DQQf1D=new double [nd*nd];
	// convert to the basis with proper boundary conditions
	for(double [][] mm:new double [][][]{
		{QQi,_QQi},
		{QQt,M.B.getQQMatr(n,tmom)},
		{DQQD,getVolMatrix2(n,_QQi)},
		{QQf0,M.B.getQQMatr(n,fmom0)},
		{QQf1,_QQf1},
		{QQf2,M.B.getQQMatr(n,fmom2)},
		{DQQf1D,getVolMatrix2(n,_QQf1)}
	    })
	{
	    final double [] m=mm[1],mr=mm[0];
	    // m: n x n matrix
	    TMatr.multiply_MA_x_MBTrans(tmp,n,m,n,basis,nd); // tmp: n x nd
	    TMatr.multiply_MA_x_MB(mr,nd,basis,n,tmp,nd); // m: nd x nd
	    // The result is saved back to m, which is now nd x nd matrix in new basis.
	}
	this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(nd,DQQD,QQt,access,getX0());

	this.flag_LH=flag_LH;
	if(flag_LH==DelaysBecauseOfSapsanEV.LH_LOW){ind_lev=0;}
	else if(flag_LH==DelaysBecauseOfSapsanEV.LH_HIGH){ind_lev=nd-1;}
	else throw new RuntimeException("flag_LH="+flag_LH);

	final double [] psipert=DeprecatedHelperEVXData.getPsiPertr(this.I,ind_lev,QQi);
	this.dPf=this.I.getFPsiPsi(ind_lev,psipert,DQQf1D);

	// for unit tests
	this.simom=imom;
	this.stmom=tmom;
	this.sf0mom=fmom0;
	this.sf1mom=fmom1;
	this.sf2mom=fmom2;

    }
    /** A simple version of basis: basik_k=Q_k - cQ_0
     *  such that fx(getEDPsi(basis_k),x0)=0.
     */
    double [] getBasisCond(final int n)
    {
	return getBasisCondFromEDPsi(n);
    }
    final double [] simom,stmom,sf0mom,sf1mom,sf2mom;

    void match_WkkVolBoundary(final WkkVolBoundary q,final double eps)
    {
	EVXData.matchEVXData(this.I,q.I,false,false,eps);
	UnitTests.checkDoubleRel(dPf,"dPf",q.dPf,"q.dPf",eps);
	System.err.println("DkQVM:WkkVolBoundary:match_WkkVolBoundary: test OK n="+n+" eps="+eps);
    }

void test_Self(final java.util.Random r,final double eps)
{
    // Results in different basises match.
    if(true){
	WkkVolBoundary q=new WkkVolBoundary(n,simom,stmom,sf0mom,sf1mom,sf2mom,flag_LH)
	{
	    @Override
	    double [] getBasisCond(final int n){return getBasisFuncsBoundaryCond(n);}
	};
	this.match_WkkVolBoundary(q,eps);
	q=new WkkVolBoundary(n,simom,stmom,sf0mom,sf1mom,sf2mom,flag_LH)
	{
	    @Override
	    double [] getBasisCond(final int n){
		final double [] basis=super.getBasisCond(n);
		final int nd=basis.length/n;
		final double [] m=getRandomNonDegeneratedMatrix(nd,r);
		final double [] re=new double [nd*n];
		TMatr.multiply_MA_x_MB(re,nd,m,nd,basis,n);
		return re;
	    }
	};
	this.match_WkkVolBoundary(q,eps);

	q=new WkkVolBoundary(n,simom,stmom,sf0mom,simom,sf2mom,flag_LH);
	UnitTests.checkDoubleRel(0,"0",q.dPf,"q.dPf",eps);
    }


    System.err.println("DkQVM:WkkVolBoundary:test_Self: test OK n="+n+" eps="+eps);
}

} // END OF class WkkVolBoundary








class DDelaysBecauseOfSapsan {
    final double xiSens;
    final DelaysBecauseOfSapsanEV m;
    final int flag_LH;
    final double [] dpsi;
    DDelaysBecauseOfSapsan(final int n,final double [] imom,final double [] tmom,final double [] fmom,final int flag_LH)
    {
	this.m=new DelaysBecauseOfSapsanEV(n,imom,tmom,flag_LH);
	this.xiSens=this.m.getXiSens(fmom);
	this.flag_LH=flag_LH;
	this.dpsi=this.m.getDpsiNormalized(fmom,true);

	this.simom=imom;
	this.stmom=tmom;
	this.sfmom=fmom;
    }

final double [] simom,stmom,sfmom;
void test_Self(final double eps)
{
    final int n=this.m.I.neq;
    final WABpsi q;
    final double [] psi;
    if(flag_LH==DelaysBecauseOfSapsanEV.LH_LOW)
    {
	final WmuPsi qq=new WmuPsi(n,simom,stmom,sfmom,sfmom);
	EVXData.matchEVXData(qq.I,this.m.I,eps);
	q=qq.jL;
	psi=m.I.psiL;
    }
    else if(flag_LH==DelaysBecauseOfSapsanEV.LH_HIGH)
    {
	q=new WABpsi(m.I.psiH,m.I.sH,m.sQQi,m.sQQt,n);
	psi=m.I.psiH;
    }
    else throw new RuntimeException("flag_LH="+flag_LH);
    if(!(q.muX!=0)||Double.isNaN(q.muX))
    {
	UnitTests.checkDoubleRel(q.fXx,"q.fXx",m.fXx,"m.fXx",eps);
    }

    UnitTests.checkDoubleArrayRel(m.getDPsiOrder1FromQQ(M.B.getQQMatr(n,simom),n),"order1(imom)",new double[n],"0",eps);
    UnitTests.checkDoubleArrayRel(m.getDPsiOrder1FromQQ(M.B.getQQMatr(n,stmom),n),"order1(tmom)",new double[n],"0",eps);
    double xi=m.getXiSens(stmom);
    if(!Double.isNaN(xi)) UnitTests.checkDoubleRel(xi,"m.getXiSens(stmom)",0,"0",eps);
    xi=m.getXiSens(simom);
    if(!Double.isNaN(xi)) UnitTests.checkDoubleRel(xi,"m.getXiSens(simom)",0,"0",eps);

    final double [] QQf=M.B.getQQMatr(n,sfmom);
    final double [] ddpsi=m.getDPsiOrder1FromQQ(QQf,n);
    UnitTests.checkDoubleArrayRel(ddpsi,"ddpsi",DeprecatedHelperEVXData.getPsiPertr(m.I,m.ind_psiskip,QQf),"m.I.getPsiPertr",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(ddpsi,psi,m.sQQi,n),"<ddpsi|QQi|psi>",0,"0i",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(ddpsi,psi,m.sQQt,n),"<ddpsi|QQt|psi>",0,"0t",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(dpsi,psi,m.sQQi,n),"<dpsi|QQi|psi>",0,"0i",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(dpsi,psi,m.sQQt,n),"<dpsi|QQt|psi>",0,"0t",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrB(dpsi,dpsi,m.sQQt,n),"<dpsi|QQt|dpsi>",1,"1t",eps);

    System.err.println("DkQVM:DDelaysBecauseOfSapsan:test_Self: test OK n="+n+" eps="+eps);
}
} // END OF class DDelaysBecauseOfSapsan

/**
 *  Version form Jul 3-Jul 4, 2013 by
 *  @version $Id: DkQVM.java,v 1.677 2022/11/19 11:08:57 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
class DelaysBecauseOfSapsanEV {
static final int LH_LOW=862,LH_HIGH=863;
final double [] Xx;
final double fXx;
final int indX,ind_psiskip;
final EVXData I;
DelaysBecauseOfSapsanEV(final int n,final double [] imom,final double [] tmom,final int flag_LH)
{
    final double [] QQi=M.B.getQQMatr(n,imom),QQt=M.B.getQQMatr(n,tmom);
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(n,QQi,QQt,access,getX0());
    final double [] psi;
    if(flag_LH==LH_LOW){psi=this.I.psiL;ind_psiskip=0;}
    else if(flag_LH==LH_HIGH){psi=this.I.psiH;ind_psiskip=n-1;}
    else throw new RuntimeException("flag_LH="+flag_LH);
    final double [][] r=getRoots(psi,M,isEdgeAtRight());
    this.Xx=r!=null?r[0]:null;
    this.indX=getIndX(this.Xx);
    if(this.indX>=0)
    {
	this.fXx=this.Xx[this.indX];
    }
    else
    {
	this.fXx=Double.NaN;
    }
    this.sQQt=QQt;
    this.sQQi=QQi;
}
final double [] sQQt,sQQi;
int getIndX(final double [] x)
{
    return DjQVM.getIndX0_NonSortedX(x,getX0(),isEdgeAtRight());
}
/** First order perturbation theory.
 */
double [] getDPsiOrder1FromQQ(final double [] QQ,final int ldQQ)
{
    final int n=this.I.neq;
    final double [] dpsi=new double [n];

    for(int k=0;k<n;k++)
    {
	if(k!=ind_psiskip) // skip lowest eigenvalue.
	{
	    final double s=TMatr.getAMatrB(n,this.I.ev,n+ind_psiskip*n,n,this.I.ev,n+k*n,QQ,ldQQ)/(this.I.ev[ind_psiskip]-this.I.ev[k]);
	    DeprecatedHelperTMatr.sumP1(dpsi,this.I.ev,n+k*n,n,s);
	}
    }
    return dpsi;
}

double [] getDpsiNormalized(final double [] pertrmoms,final boolean flag_normalize)
{
    final int n=this.I.neq;
    final double [] QQpertr=M.B.getQQMatr(n,pertrmoms);
    final double [] dpsi=getDPsiOrder1FromQQ(QQpertr,n);
    if(flag_normalize)
    {
	// Think later about proper normalizing of dpsi.
	final double ndpsi=Math.sqrt(TMatr.getAMatrA(dpsi,sQQt,n));
	for(int k=0;k<n;k++) {dpsi[k]/=ndpsi;}
    }
    return dpsi;
}

/** Calculate first order perturbation to the indX root
 *  caused by pertrmoms moments.
 */
double getXiSens(final double [] pertrmoms)
{
    final int n=this.I.neq;
    final double [] psi=new double [n],dpsi=getDpsiNormalized(pertrmoms,false);
    System.arraycopy(this.I.ev,n+ind_psiskip*n,psi,0,n);
    // Newton iteration to estimate perturbation to the root.
    final double sens=-M.B.getD0(dpsi,this.fXx)/M.B.getD1(psi,this.fXx);
    return sens;
}
} // END OF class DelaysBecauseOfSapsanEV


class WmuPsi {
    final EVXData I;
    final WABpsi jL,jLpi;
    final double mu,g,dp,g_dp;
    final double lg,ldp,lg_ldp,lg_lx,lmu,a_lg_ldp,b_lg_ldp,c_lg_ldp,a_lg_lg,a_lg_x_lg,a_lg_x2_lg,p_lg_ldp;
    final double Pf,Pfd,sigma2_Pf;
    final double p_v,p_t,p_td,p_tmd,mf,p_v_p,p_t_p,p_x,p_x_p,dr,drP,drPd,g_t,sigma2_g_t,g_i,Fg;
    final double xD0,xD1,xD2,xXD0,xXD1,xXD2;
    final double Mu,P_X,ldP,Mu_A,P_X_A;

    final double P_v,P_t;

    final double mu_Max,fXx_Max,p_x_Max,p_t_Max,Bt_Max,Mu_Max,P_X_Max;
    final double pq_x,pq_x_Max;

WmuPsi(final int neq,final double [] imom,final double [] tmom,
       final double [] pimom,final double [] ptmom)
{
    final double x0=getX0();
    final double [] QQi=M.B.getQQMatr(neq,imom),QQt=M.B.getQQMatr(neq,tmom);
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQi,QQt,access,x0);
    this.jL=new WABpsi(this.I.psiL,this.I.sL,QQi,QQt,neq);
    this.jLpi=new WABpsi(this.I.psiL,this.I.sL,M.B.getQQMatr(neq,pimom),QQt,neq);
    final double [] dpmom=new double [pimom.length];
    for(int k=dpmom.length;--k>=0;) {dpmom[k]=pimom[k]-(ptmom!=null?this.I.sL*ptmom[k]:0);}

    final BasisFunctionsCalculatable B=getBasisPolynomials();
    final double [] psi=this.jL.psiInd,dpsi=B.getDifferentiated(psi);
    final int n=psi.length;
    final double [] QQdp=M.B.getQQMatr(n,dpmom),QQx=new double [n*n];
    for(int k=0;k<n;k++)for(int l=0;l<n;l++){QQx[k*n+l]=this.jL.sQQa[k*this.jL.ldQQ+l]-this.jL.lam*this.jL.sQQb[k*this.jL.ldQQ+l];}
    final double [] rd=getmufg(psi,QQx,QQdp,n,B);
    this.mu=rd[0];
    this.dp=rd[1];
    this.g=rd[2];
    this.g_dp=rd[3];

    final double l=M.B.getD0(psi,this.jL.fXx),dl=M.B.getD1(psi,this.jL.fXx);
    //System.err.println("l="+l+" dl="+dl);
    this.ldp=TMatr.getAMatrB(psi,psi,QQdp,n)/(l*l);
    this.lmu=TMatr.getAMatrB(psi,psi,QQx ,n)/(l*l);
    this.lg=dl/l;
    this.lg_ldp=TMatr.getAMatrB(psi,dpsi,QQdp,n)/(l*l)-this.ldp*this.lg;
    this.lg_lx =TMatr.getAMatrB(psi,dpsi,QQx ,n)/(l*l)-this.lmu*this.lg;

    final double [] DKe=getD2KE(this.jL.psi,this.jL.fXx,B);
    this.a_lg_ldp=TMatr.mulVVn1(DKe,dpmom);
    final double [] DKeb=B.sdiv1(DKe,this.jL.fXx).q;
    this.b_lg_ldp=TMatr.mulVVn1(DKeb,dpmom);
    final double [] psi2=B.mult2Pol(psi,psi),psi2sdx=B.sdiv1(psi2,this.jL.fXx).q;
    // Quotent. The remainder give Markov function for dp measure at x=this.jL.fXx
    this.c_lg_ldp=TMatr.mulVVn1(psi2sdx,dpmom)/(l*l);

    final double [] xmom=new double [tmom.length];
    for(int k=xmom.length;--k>=0;){xmom[k]=imom[k]-this.I.sL*tmom[k];}
    //System.err.println("<DKeb|momx>="+TMatr.mulVVn1(DKeb,xmom));
    this.a_lg_lg=TMatr.mulVVn1(DKe,xmom);
    final double [] Xxmom=M.B.getXQkMomentsFromQkMoments(xmom);
    this.a_lg_x_lg=TMatr.mulVVn1(DKe,Xxmom);
    this.a_lg_x2_lg=TMatr.mulVVn1(DKe,M.B.getXQkMomentsFromQkMoments(Xxmom));

    final double [] pDKe=getD2KE(this.jL.psi,this.jL.fXxp,B);
    this.p_lg_ldp=TMatr.mulVVn1(pDKe,dpmom);

    final double [] QQpi=M.B.getQQMatr(neq,pimom),QQpt=ptmom!=null?M.B.getQQMatr(neq,ptmom):QQpi;
    final double bv=TMatr.getAMatrA(psi,QQi,neq),bt=TMatr.getAMatrA(psi,QQt,neq);
    this.p_v=TMatr.getAMatrA(psi,QQpi,neq)/bv;
    final double n_p_t=TMatr.getAMatrA(psi,QQpt,neq);
    this.p_t=n_p_t/bt;
    final double ot,dpl=0; // set dpl=0 asuming offset=last
    {
	final double [] pp=B.mult2Pol(psi,psi);
	ot=getIntegratedToEnd(this.jL.fXx,pp);
    }

    this.p_td=(n_p_t-dpl*ot)/(bt-ot);
    this.p_tmd=(n_p_t+dpl*ot)/(bt+ot);
    this.dr=bv*(this.jL.lam*bt)/(lmu*lmu);
    this.drP=(bv*(this.jL.lam*bt)/Math.pow(bv-this.jL.lam*bt,2))*(this.p_v-this.p_t);
    this.drPd=(bv*(this.jL.lam*bt)/Math.pow(bv-this.jL.lam*bt,2))*(this.p_v-this.p_td);
    this.mf=bv*this.jL.lam*bt/Math.pow(bv-this.jL.lam*bt,2);
    this.p_v_p=TMatr.getAMatrA(this.jL.psiIndp,QQpi,neq)/TMatr.getAMatrA(this.jL.psiIndp,QQi,neq);
    this.p_t_p=TMatr.getAMatrA(this.jL.psiIndp,QQpt,neq)/TMatr.getAMatrA(this.jL.psiIndp,QQt,neq);
    this.p_x=TMatr.getAMatrA(psi,QQdp,n)/TMatr.getAMatrA(psi,QQx,n);
    this.p_x_p=TMatr.getAMatrA(this.jL.psiIndp,QQdp,n)/TMatr.getAMatrA(this.jL.psiIndp,QQx,n);

    final double [] psiI=this.jL.psi,dpsiI=B.getDifferentiated(psiI),QQf=M.B.getQQMatr(neq,dpmom);
    //System.err.println("qqmu:psiI="+java.util.Arrays.toString(psiI));
    this.Pf=TMatr.getAMatrA(psiI,QQf,neq)/TMatr.getAMatrA(psiI,QQt,neq);
    this.Pfd=TMatr.getAMatrA(dpsiI,QQf,neq)/TMatr.getAMatrA(dpsiI,QQt,neq);
    pq_x=TMatr.mulVVn1(psi,dpmom)/(this.mu);
    final double [] SDpsi=M.B.sdiv1(psiI,this.jL.fXx).q;
    this.g_t=TMatr.getAMatrB(psiI,SDpsi,QQt,neq)/TMatr.getAMatrA(psiI,QQt,neq);
    this.sigma2_g_t=TMatr.getAMatrB(SDpsi,SDpsi,QQt,neq)/TMatr.getAMatrA(psiI,QQt,neq)-this.g_t*this.g_t;
    this.g_i=TMatr.getAMatrB(psiI,SDpsi,QQi,neq)/TMatr.getAMatrA(psiI,QQi,neq);
    this.Fg=TMatr.getAMatrB(psiI,SDpsi,QQf,neq)-this.g_t*this.Pf;
    final double [] QQPf2=getDTxAxD(neq,QQf,com.polytechnik.utils.Linsystems.getInvertedMatrix(neq,neq,QQt),new double [neq*neq]);
    this.sigma2_Pf=TMatr.getAMatrA(psiI,QQPf2,neq)-this.Pf*this.Pf;

    final double [] IQQx=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQx);
    {
    final double [] QQn=TMatr.mulMMM(n,IQQx,QQdp,IQQx);
    final double [][] kkk=access.getKK(n,new double [][] {IQQx,QQn});
    final PolynomialsRatio tr=new PolynomialsRatio(kkk[0],kkk[1],M.B);
    this.xD0=tr.getF(this.jL.fXx);
    this.xD1=tr.getDF(this.jL.fXx);
    this.xD2=tr.getD2F(this.jL.fXx);
    }
    final double [] dpxmom=M.B.getXQkMomentsFromQkMoments(dpmom);
    for(int k=0;k<dpxmom.length;k++){dpxmom[k]=(dpxmom[k]-this.jL.fXx*dpmom[k]);}
    final double [] QQXdp=M.B.getQQMatr(n,dpxmom);
    {
    final double [] QQn=TMatr.mulMMM(n,IQQx,QQXdp,IQQx);
    final double [][] kkk=access.getKK(n,new double [][] {IQQx,QQn});
    final PolynomialsRatio tr=new PolynomialsRatio(kkk[0],kkk[1],M.B);
    this.xXD0=tr.getF(this.jL.fXx);
    this.xXD1=tr.getDF(this.jL.fXx);
    this.xXD2=tr.getD2F(this.jL.fXx);
    }

    final double [] D=getDop(n,B),qq=new double [n],Dqq=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] q=new double [k+1];
	q[k]=1;
	qq[k]=M.B.getD0(q,this.jL.fXx);
	Dqq[k]=M.B.getD1(q,this.jL.fXx);
    }
    final double nm=TMatr.getAMatrA(qq,IQQx,n);
    { final double f=TMatr.getAMatrA(qq,TMatr.mulMMM(n,IQQx,QQdp,IQQx),n)/nm;
      this.tf=f;
      this.txD1a=2*(TMatr.getAMatrA(qq,TMatr.mulMatrices(n,new double [][]{D,IQQx,QQdp,IQQx}),n)/nm-f*TMatr.getAMatrA(qq,TMatr.mulMatrices(n,new double [][]{D,IQQx}),n)/nm);
      //2*(TMatr.getAMatrB(qq,Dqq,TMatr.mulMatrices(n,new double [][]{IQQx,QQdp,IQQx}),n)/nm-f*TMatr.getAMatrB(n,qq,0,Dqq,0,IQQx,n)/nm);
      this.txD1b=TMatr.getAMatrA(n,qq,TMatr.mulMatrices(n,new double [][]{IQQx,QQdp,D,IQQx}),n)/nm-f*TMatr.getAMatrA(n,qq,TMatr.mulMatrices(n,new double [][]{D,IQQx}),n)/nm;
    }
    { final double f=TMatr.getAMatrA(n,qq,TMatr.mulMatrices(n,new double [][]{IQQx,QQXdp,IQQx}),n)/nm;
      this.txXD1a=2*(TMatr.getAMatrA(n,qq,TMatr.mulMatrices(n,new double [][]{D,IQQx,QQXdp,IQQx}),n)/nm-f*TMatr.getAMatrA(n,qq,TMatr.mulMatrices(n,new double [][]{D,IQQx}),n)/nm);
      //2*(TMatr.getAMatrB(n,qq,0,Dqq,0,TMatr.mulMatrices(n,new double [][]{IQQx,QQXdp,IQQx}),n)/nm-f*TMatr.getAMatrB(qq,Dqq,IQQx,n)/nm);
      this.txXD1b=TMatr.getAMatrA(qq,TMatr.mulMatrices(n,new double [][]{IQQx,QQXdp,D,IQQx}),n)/nm-f*TMatr.getAMatrA(qq,TMatr.mulMatrices(n,new double [][]{D,IQQx}),n)/nm;
    }

    {
	final double [] QQT=M.B.getQQMatr(neq,getFmom(tmom)),QQV=M.B.getQQMatr(neq,getFmom(imom)),QQPV=M.B.getQQMatr(neq,getFmom(pimom)),QQPT=ptmom!=null?M.B.getQQMatr(neq,getFmom(ptmom)):QQPV;
	this.P_v=TMatr.getAMatrA(psi,QQPV,neq)/TMatr.getAMatrA(psi,QQV,neq);
	this.P_t=TMatr.getAMatrA(psi,QQPT,neq)/TMatr.getAMatrA(psi,QQT,neq);
    }

    final double [] QQX=M.B.getQQMatr(n,getFmom(xmom)),QQPX=M.B.getQQMatr(n,getFmom(dpmom));
    final double mmu=TMatr.getAMatrB(psi,psi,QQX ,n);
    this.Mu=mmu/(l*l);
    final double Pmmu=TMatr.getAMatrB(psi,psi,QQPX,n);
    this.ldP=Pmmu/(l*l);
    this.P_X=Pmmu/mmu;

    final int indmax=DjQVM.getIndMax(this.jL.Mu);
    if(indmax>=0){
	mu_Max=this.jL.Mu[indmax];
	fXx_Max=this.jL.Xx!=null?this.jL.Xx[indmax]:Double.NaN;
	final double [] psim=B.sdiv1(this.jL.psi,fXx_Max).q;
	final double ff=M.B.getD0(psim,fXx_Max);
	p_x_Max=TMatr.getAMatrA(psim,QQdp,n)/TMatr.getAMatrA(psim,QQx,n);

	pq_x_Max=(TMatr.mulVVn1(psim,dpmom)/ff)/(mu_Max);

	final double mmuMax=TMatr.getAMatrA(psim,QQX,n);
	this.Mu_Max=mmuMax/(ff*ff);
	this.P_X_Max=TMatr.getAMatrA(psim,QQPX,n)/mmuMax;
	final double bb=TMatr.getAMatrA(psim,QQt,neq);
	p_t_Max=TMatr.getAMatrA(psim,QQpt,neq)/bb;
	Bt_Max=bb/(ff*ff);
    }
    else
    {
	mu_Max=fXx_Max=p_x_Max=Mu_Max=P_X_Max=pq_x_Max=p_t_Max=Bt_Max=Double.NaN;
    }
    final double [] psia=new double [psi.length];
    psia[0]=1;
    final double mmuA=TMatr.getAMatrA(psia,QQX,n);
    this.Mu_A=mmuA;
    this.P_X_A=TMatr.getAMatrA(psia,QQPX,n)/mmuA;

    simom=imom;
    stmom=tmom;
    spimom=pimom;
    sptmom=ptmom;
    this.sDke=DKe;
    this.sDkeb=DKeb;
}
    final double tf,txD1a,txD1b,txXD1a,txXD1b;
    final double [] sDke,sDkeb;
    final double [] simom,stmom,spimom,sptmom;
void test_Self(final double eps)
{
    if(!(this.mu!=0)||Double.isNaN(this.mu)) return;
    final BasisFunctionsCalculatable B=getBasisPolynomials();

    final int neq=this.I.neq;
    WmuPsi q=new WmuPsi(neq,simom,stmom,simom,stmom);
    UnitTests.checkDoubleRel(q.g_dp,"q.g_dp",0,"0",eps);
    UnitTests.checkDoubleRel(q.P_t,"q.P_t",1,"1",eps);
    UnitTests.checkDoubleRel(q.P_v,"q.P_v",1,"1",eps);
    q=new WmuPsi(neq,simom,stmom,M.B.getXQkMomentsFromQkMoments(simom),M.B.getXQkMomentsFromQkMoments(stmom));
    UnitTests.checkDoubleRel(q.g_dp,"q.g_dp",0,"0",eps);

    UnitTests.checkDoubleRel(mu,"mu",this.jL.getWX(),"this.jL.getWX()",eps);
    UnitTests.checkDoubleRel(mu,"mu",lmu,"lmu",eps);
    UnitTests.checkDoubleRel(dp,"dp",ldp/mu,"ldp/mu",eps);
    UnitTests.checkDoubleRel(g,"g",lg,"lg",eps);

    UnitTests.checkDoubleRel(ldp/lmu,"ldp/lmu",p_x,"p_x",eps);

    // have to match identically, because we build on roots.
    UnitTests.checkDoubleRel(g_t,"g_t",g_i,"g_i",eps);
    UnitTests.checkDoubleRel(g_dp,"g_dp",lg_ldp/lmu,"lg_ldp/lmu",eps);
    UnitTests.checkDoubleRel(g_dp,"g_dp",a_lg_ldp/lmu,"a_lg_ldp/lmu",eps);

    UnitTests.checkDoubleRel(a_lg_lg,"a_lg_lg",0,"0",eps);
    UnitTests.checkDoubleRel(a_lg_x_lg,"a_lg_x_lg",0,"0",eps);
    UnitTests.checkDoubleRel(lg_lx,"lg_lx",0,"0",eps);
    // normalizing=0
    UnitTests.checkDoubleRel(a_lg_x2_lg,"a_lg_x2_lg",0,"0",eps);
    UnitTests.checkDoubleRel(M.B.getD0(sDke,this.jL.fXx),"Dke(xk)",0,"0",eps);
    if(sDke.length>=2)
    {
	final double [] tDke=B.getaxbP(sDkeb,1,-this.jL.fXx);
	UnitTests.checkDoubleArrayRel(tDke,"tDke",sDke,"sDke",eps);
    }
    UnitTests.checkDoubleRel(mf,"mf",this.jL.fAx*this.jL.fBx*this.jL.lam/Math.pow(this.jL.fAx-this.jL.fBx*this.jL.lam,2),"mt from jL",eps);

    UnitTests.checkDoubleRel(ldp/lmu,"ldp/lmu",xD0,"xD0",eps);
    UnitTests.checkDoubleRel(ldp/lmu,"ldp/lmu",tf,"tf",eps);
    UnitTests.checkDoubleRel(xD1,"xD1",txD1a,"txD1a",eps);
    UnitTests.checkDoubleRel(lg_ldp/lmu,"lg_ldp/lmu",txD1b,"txD1b",eps);
    UnitTests.checkDoubleRel(xXD1,"xXD1",txXD1a,"txXD1a",eps);

    {
      final WmuPsi b=new WmuPsi(this.I.neq,simom,stmom,simom,stmom);
      UnitTests.checkDoubleRel(b.xD1,"b.xD1",0,"0",eps);
      UnitTests.checkDoubleRel(b.xD2,"b.xD2",0,"0",eps);
      UnitTests.checkDoubleRel(b.xXD0,"b.xXD0",0,"0",eps);
      UnitTests.checkDoubleRel(b.xXD1,"b.xXD1",0,"0",eps);
      final double [] xsimom=M.B.getXQkMomentsFromQkMoments(simom),xstmom=M.B.getXQkMomentsFromQkMoments(stmom);
      final WmuPsi c=new WmuPsi(this.I.neq,simom,stmom,xsimom,xstmom);
      UnitTests.checkDoubleRel(c.xD1,"c.xD1",0,"0",eps);
      //UnitTests.checkDoubleRel(c.xD2,"c.xD2",0,"0",eps);
      UnitTests.checkDoubleRel(c.xXD0,"c.xXD0",0,"0",eps);
      UnitTests.checkDoubleRel(c.xXD1,"c.xXD1",0,"0",eps);
      final WmuPsi d=new WmuPsi(this.I.neq,simom,stmom,M.B.getXQkMomentsFromQkMoments(xsimom),M.B.getXQkMomentsFromQkMoments(xstmom));
      UnitTests.checkDoubleRel(d.xD1,"d.xD1",0,"0",eps);
      //UnitTests.checkDoubleRel(d.xXD0,"d.xXD0",0,"0",eps);
      //UnitTests.checkDoubleRel(d.xXD1,"d.xXD1",0,"0",eps);
    }
    if(sptmom!=null){
    WlPsi s=new WlPsi(this.jL.psiInd,this.jL.lam,spimom,simom,sptmom,stmom);
    UnitTests.checkDoubleRel(p_x,"p_x",s.f,"s.f",eps);
    UnitTests.checkDoubleRel(mu,"mu",s.mu,"s.mu",eps);
    s=new WlPsi(this.jL.psiInd,this.jL.lam,getFmom(spimom),getFmom(simom),getFmom(sptmom),getFmom(stmom));
    UnitTests.checkDoubleRel(P_X,"P_X",s.f,"s.f",eps);
    UnitTests.checkDoubleRel(Mu,"Mu",s.mu,"s.mu",eps);
    }

    if(this.jL.fXx==fXx_Max)
    {
	System.err.println("this.jL.fXx="+this.jL.fXx+" fXx_Max="+fXx_Max);
	UnitTests.checkDoubleRel(mu,"mu",mu_Max,"mu_Max",eps);
	UnitTests.checkDoubleRel(p_x,"p_x",p_x_Max,"p_x_Max",eps);
	UnitTests.checkDoubleRel(Mu,"Mu",Mu_Max,"Mu_Max",eps);
	UnitTests.checkDoubleRel(P_X,"P_X",P_X_Max,"P_x_Max",eps);
    }
    System.err.println("DkQVM:WmuPsi:mu="+mu+" Mu="+Mu+" mu_Max="+mu_Max+" Mu_Max="+Mu_Max);

    System.err.println("DkQVM:WmuPsi:test_Self: test OK neq="+neq+" eps="+eps);

}
void match_DynVarMu(final DynVarMu q,final double eps)
{
    EVXData.matchEVXData(q.I,I,eps);
    UnitTests.checkDoubleRel(I.sL,"I.sL",jL.Wa_psi,"jL.Wa_psi",eps);
    UnitTests.checkDoubleRel(q.p_IL*I.sL,"q.p_IL*I.sL",jLpi.Wa_psi,"jLpi.Wa_psi",eps);
    UnitTests.checkDoubleRel(1,"1",jL.Wb_psi,"jL.Wb_psi",eps);
    UnitTests.checkDoubleRel(1,"1",jLpi.Wb_psi,"jLpi.Wb_psi",eps);
    UnitTests.checkDoubleArrayRel(jL.Xx,"jL.Xx",jLpi.Xx,"jLpi.Xx",eps*0);
    UnitTests.checkDoubleArrayRel(jL.Wb,"jL.Wb",jLpi.Wb,"jLpi.Wb",eps*0);
    // test root
    UnitTests.checkDoubleRel(jL.fXx,"jL.fXx",q.x_i_c,"q.x_i_c",eps);
    UnitTests.checkDoubleRel(jL.flamx-jL.lam,"jL.flamx-jL.lam",q.i_c,"q.i_c",eps);
    UnitTests.checkDoubleRel(jL.fXxpamax,"jL.fXxpamax",q.x_i_max,"q.x_i_max",eps);
    // test quadrature on dmu=dv-lam*dt measure
    UnitTests.checkDoubleRel(simom[0]-I.sL*stmom[0],"simom[0]-I.sL*stmom[0]",TMatr.mulVV(q.i,q.tau),"i*tau",eps);
    // check all nodes of this quadrature
    final double [] x=jL.Xx,lt=jL.Wb,li=jL.Wa,lpi=jLpi.Wa;
    UnitTests.checkDoubleArrayRel(q.x,"q.x",x,"x",eps);
    for(int k=0;k<q.x.length;k++)
    {
	final double ik=li[k]/lt[k]-jL.Wa_psi/jL.Wb_psi;
	UnitTests.checkDoubleRel(q.i[k],"q.i[k]",ik,"ik",eps);
	final double cik=lpi[k]/lt[k]-jLpi.Wa_psi/jL.Wb_psi;
	UnitTests.checkDoubleRel(q.ci[k],"q.ci[k]",cik,"cik",eps);
    }
    System.err.println("DkQVM:WmuPsi:match_DynVarMu: test OK n="+I.neq+" eps="+eps);
}

} // END OF class WmuPsi
static double [] getmufg(final double [] psi,final double [] QQx,final double [] QQfx,final int ldQQ,final BasisFunctionsCalculatable B)
{
   final double [] dpsi=dx_getDop(psi,B);
   final double mu=TMatr.getAMatrB(psi,psi,QQx,ldQQ),
       f=TMatr.getAMatrB(psi,psi,QQfx,ldQQ)/mu,
       g=TMatr.getAMatrB(psi,dpsi,QQx,ldQQ)/mu,
       fg=TMatr.getAMatrB(psi,dpsi,QQfx,ldQQ)/mu-f*g;
   return new double []{mu,f,g,fg};
}

/** D2 Kernel.
 *  Calculate psi(x)=psiI(x)/(x-xk),
 *  then return (psi(x)*psi'(x)- (psi'(xk)/psi(xk))*psi(x)*psi(x))/psi(xk)^2.
 */
static double [] getD2KE(final double [] psiI,final double xk,final BasisFunctionsCalculatable B)
{
    final int n=psiI.length-1;
    final double [] psi=B.sdiv1(psiI,xk).q;
    final double [] DKe=getD2KEFromPsi(psi,xk,B);
    final double fk=B.getD0(psi,xk);
    final double ff=fk*fk;
    for(int j=DKe.length;--j>=0;){DKe[j]/=ff;}
    return DKe;
}
/** D2 Kernel.
 *  Calculate (psi(x)*psi'(x)- (psi'(xk)/psi(xk))*psi(x)*psi(x)
 */
static double [] getD2KEFromPsi(final double [] psi,final double xk,final BasisFunctionsCalculatable B)
{
    final int n=psi.length;
    final double [] dpsi=B.getDifferentiated(psi),
	DKe=B.mult2Pol(psi,dpsi),Ke=B.mult2Pol(psi,psi);
    final double fk=B.getD0(psi,xk),dfk=B.getD1(psi,xk);
    final double [] DKe1=new double [DKe.length+1];System.arraycopy(DKe,0,DKe1,0,DKe.length);
    TMatr.sumP1(DKe1,Ke,-(dfk/fk));
    return DKe1;
}


/**
 *  Calculate d/dxk <psil|(x-xk)^2f|psil>/<psil|(x-xk)^2|psil>=
 *               -2*(<psil|(x-xk)f|psil>/<psil|(x-xk)^2|psil>-
 *               <psil|f(x-xk)^2|psil>/<psil|(x-xk)^2|psil>*
 *               <psil|(x-xk)|psil>/<psil|(x-xk)^2|psil>)
 */
static double getfD1x(final double [] psil,final double xk,final double [] QQf,final double [] QQt,final int ldQQ,final BasisFunctionsCalculatable B)
{
    double [] psi=new double [psil.length+1];
    if(psil.length>0) psi=B.getaxbP(psil,1,-xk);
    else java.util.Arrays.fill(psi,0);
    final double dn=TMatr.getAMatrA(psi,QQt,ldQQ),
	f=TMatr.getAMatrA(psi,QQf,ldQQ)/dn,
	df=-2*TMatr.getAMatrB(psi,psil,QQf,ldQQ)/dn,
	dg=-2*TMatr.getAMatrB(psi,psil,QQt,ldQQ)/dn;
    return df-f*dg;
}

/**
 *  Calculate d^2/dxk^2 <psil|(x-xk)^2f|psil>/<psil|(x-xk)^2|psil>
 */
static double getfD2x(final double [] psil,final double xk,final double [] QQf,final double [] QQt,final int ldQQ,final BasisFunctionsCalculatable B)
{
    double [] psi=new double [psil.length+1];
    if(psil.length>0) psi=B.getaxbP(psil,1,-xk);
    else java.util.Arrays.fill(psi,0);
    //System.err.println("ldQQ="+ldQQ+" psi.length="+psi.length);
    final double dn=TMatr.getAMatrA(psi,QQt,ldQQ),
	f=TMatr.getAMatrA(psi,QQf,ldQQ)/dn,
	df=-2*TMatr.getAMatrB(psi,psil,QQf,ldQQ)/dn,
	dg=-2*TMatr.getAMatrB(psi,psil,QQt,ldQQ)/dn,
	ddf=2*TMatr.getAMatrA(psil,QQf,ldQQ)/dn,
	ddg=2*TMatr.getAMatrA(psil,QQt,ldQQ)/dn;
    return (ddf-f*ddg)-2*dg*(df-f*dg);
}

static PolynomialsRatio testfD(final double [] psil,final double [] QQf,final double [] QQt,final int ldQQ,final BasisFunctionsCalculatable B)
{
  double [] psi=new double [psil.length+1];
  if(psil.length>0) psi=B.getaxbP(psil,1,0);
  else java.util.Arrays.fill(psi,0);
  return new PolynomialsRatio(new double [] {
	  TMatr.getAMatrA(psi,QQt,ldQQ),
	  -2*TMatr.getAMatrB(psi,psil,QQt,ldQQ),
	  TMatr.getAMatrA(psil,QQt,ldQQ)
      },
      new double [] {
	  TMatr.getAMatrA(psi,QQf,ldQQ),
	  -2*TMatr.getAMatrB(psi,psil,QQf,ldQQ),
	  TMatr.getAMatrA(psil,QQf,ldQQ)
      },
      new OrthogonalPolynomialsMonomialsBasis().B);
}

/**
 *  Calculate d^m/dxk^m <Dpsi|(x-xk)^2f|Dpsi>/<Dpsi|(x-xk)^2|Dpsi>
 *  for m=0,1,2.
 *  @param psi=(x-xk)Dpsi.
 *  @param Dpsi.
 *  @param QQf The <Q_i f Q_j> matrix.
 *  @param QQt The <Q_i Q_j> matrix.
 *  @param ldQQ The leading dimension of QQ matrices.
 *  @return An array containing D0 D1 D2 derivatives.
 */
static double [] getfDsx(final double [] psi,final double [] Dpsi,final double [] QQf,final double [] QQt,final int ldQQ)
{
    return getfDDx(psi,Dpsi,null,QQf,QQt,ldQQ);
}


/**
 *  Calculate d^m/ds^m <psi(x-s)|f|psi(x-s)>/<psi(x-s)|psi(x-s)>
 *  for m=0,1,2. Average over x and differentiaated over s.
 *  @param psi psi.
 *  @param Dpsi d psi/dx.
 *  @param D2psi d^2 psi/dx^2.
 *  @param QQf The <Q_i f Q_j> matrix.
 *  @param QQt The <Q_i Q_j> matrix.
 *  @param ldQQ The leading dimension of QQ matrices.
 *  @return An array containing D0 D1 D2 derivatives.
 */
static double [] getfDDx(final double [] psi,final double [] Dpsi,final double [] D2psi,final double [] QQf,final double [] QQt,final int ldQQ)
{
    final double [] dpsi=new double [Dpsi.length];
    for(int k=Dpsi.length;--k>=0;){dpsi[k]=-Dpsi[k];}
    return com.polytechnik.utils.RayleighQuotient.getRQVariationValues(psi,dpsi,D2psi,QQf,QQt,ldQQ);
}



/** Calculates psi'*(x-x0)
 */
static double [] getDpsiX(final double [] psi,final double x0,final BasisFunctionsCalculatable B)
{
    final double [] dpsi=B.getDifferentiated(psi);
    return B.getaxbP(dpsi,1,-x0);
}


class WLHTpsi {
    final EVXData I,Iplus,Iminus;
    final WABpsi jL,jLplus,jLminus;
    final double mu,muplus,muminus,xmu,xmuplus,xmuminus;

WLHTpsi(final int neq,final double [] imom,final double [] tmom,
	final double [] fplus,final double [] fminus)
{
    final double x0=getX0();
    final double [] QQi=M.B.getQQMatr(neq,imom),QQt=M.B.getQQMatr(neq,tmom),
	QQfplus=M.B.getQQMatr(neq,fplus),QQfminus=M.B.getQQMatr(neq,fminus);
    this.I=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQi,QQt,access,x0);
    this.jL=new WABpsi(this.I.psiL,this.I.sL,QQi,QQt,neq);

    this.Iplus=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQfplus,QQt,access,x0);
    this.jLplus=new WABpsi(this.Iplus.psiL,this.Iplus.sL,QQfplus,QQt,neq);

    this.Iminus=DeprecatedHelperEVXData.getEVXDataFromQQ(neq,QQfminus,QQt,access,x0);
    this.jLminus=new WABpsi(this.Iminus.psiL,this.Iminus.sL,QQfminus,QQt,neq);

    this.mu=this.jL.getFMu(this.jL.psiInd);
    this.muplus=this.jLplus.getFMu(this.jL.psiInd);
    this.muminus=this.jLminus.getFMu(this.jL.psiInd);
    final int n=this.jL.psiInd.length;
    final double [] XQQi=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(imom)),
	XQQt=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(tmom)),
	XQQfplus=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(fplus)),
	XQQfminus=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(fminus));
    final double [] tmp=new double [n*n];
    for(int k=0;k<tmp.length;k++){tmp[k]=XQQi[k]-this.jL.lam*XQQt[k];}
    this.xmu=TMatr.getAMatrA(this.jL.psiInd,tmp,n)/this.mu;
    for(int k=0;k<tmp.length;k++){tmp[k]=XQQfplus[k]-this.jLplus.lam*XQQt[k];}
    this.xmuplus=TMatr.getAMatrA(this.jL.psiInd,tmp,n)/this.muplus;
    for(int k=0;k<tmp.length;k++){tmp[k]=XQQfminus[k]-this.jLminus.lam*XQQt[k];}
    this.xmuminus=TMatr.getAMatrA(this.jL.psiInd,tmp,n)/this.muminus;

}
} // END OF class WLHTpsi

class WLHpsi {
    final WABpsi jL;
    EVXData PM;
    final double xplus,xminus,quality;
    final double [] Xxplus,Xxminus;
    final int dir;

WLHpsi(final WABpsi jL,
       final double [] fplus,final double [] fminus)
{
    this.jL=jL;
    this.PM=DeprecatedHelperEVXData.getEVXData(this.jL.psi.length,fplus,fminus,access,getX0());
    this.Xxplus=M.getXNodes(PM.psiL).re_root;
    this.Xxminus=M.getXNodes(PM.psiH).re_root;
    final int indplus=DjQVM.getIndXClose(Xxplus,this.jL.fXx,getX0(),isEdgeAtRight()),
	indminus=DjQVM.getIndXClose(Xxminus,this.jL.fXx,getX0(),isEdgeAtRight());
    this.xplus=indplus>=0?Xxplus[indplus]:Double.NaN;
    this.xminus=indminus>=0?Xxminus[indminus]:Double.NaN;
    final double dplus=Math.abs(xplus-this.jL.fXx),dminus=Math.abs(xminus-this.jL.fXx);
    if(dplus<dminus)
    {
	quality=dplus/dminus;
	dir=1;
    }
    else
    {
	quality=dminus/dplus;
	dir=-1;
    }
}
} // END OF class WLHTpsi


class WABpsi {
    final double [] psi;
    final double lam,Wa_psi,Wb_psi;
    final double [] Xx,Wa,Wb,Mu;
    final double fXx,fAx,fBx,flamx,muX;
    final double fXxp,fXxpamax,fAxp,fBxp,flamxp;
    final double [] psiInd,psiIndp;
    final int indX,ldQQ;
final double [] sQQa,sQQb;
WABpsi(final double [] psi,final double lam,
       final double [] QQa,final double [] QQb,final int ldQQ)
{
    this.psi=psi;
    this.lam=lam;
    this.sQQa=QQa;
    this.sQQb=QQb;
    this.ldQQ=ldQQ;
    this.Wa_psi=TMatr.getAMatrA(psi,QQa,ldQQ);
    this.Wb_psi=TMatr.getAMatrA(psi,QQb,ldQQ);
    final double x0=getX0();
    final double [][] r=getRoots(psi,M,isEdgeAtRight());
    this.Xx=r!=null?r[0]:null;
    this.indX=DjQVM.getIndX0(Xx,x0,isEdgeAtRight());

    final int n=psi.length-1;
    this.psiInd=new double [n];
    this.psiIndp=new double [n];
    java.util.Arrays.fill(psiInd,Double.NaN);
    this.Wa=new double [n];
    this.Wb=new double [n];
    this.Mu=new double [n];
    final int nx=(Xx!=null?Xx.length:0);
    double aw=0;
    int indxfmax=-1;
    for(int k=0;k<nx;k++)
    {
	final double xk=Xx[k];
	final double [] p=M.B.sdiv1(psi,xk).q;
	final double f0=M.B.getD0(p,xk);
	if(k==indX)
	{
	    for(int l=0;l<psiInd.length;l++){psiInd[l]=p[l]/f0;}
	}
	if(indX>=0 && k==(indX+1))
	{
	    for(int l=0;l<psiIndp.length;l++){psiIndp[l]=p[l]/f0;}
	}
	Wa[k]=TMatr.getAMatrA(n,p,QQa,ldQQ)/(f0*f0);
	Wb[k]=TMatr.getAMatrA(n,p,QQb,ldQQ)/(f0*f0);
	Mu[k]=Wa[k]-lam*Wb[k];
	final double ww=Math.abs(Wa[k]/Wb[k]);
	if(ww>=aw)
	{
	    aw=ww;
	    indxfmax=k;
	}
    }
    fXxpamax=indxfmax>=0?Xx[indxfmax]:Double.NaN;
    if(indX>=0)
    {
	this.fXx=Xx[indX];
	this.fAx=Wa[indX];
	this.fBx=Wb[indX];
	this.flamx=this.fAx/this.fBx;
	this.muX=this.fAx-this.lam*this.fBx;
	if(indX<Xx.length-1)
	{
	    this.fXxp=Xx[indX+1];
	    this.fAxp=Wa[indX+1];
	    this.fBxp=Wb[indX+1];
	    this.flamxp=this.fAxp/this.fBxp;
	}
	else
	{
	    this.fXxp=this.fAxp=this.fBxp=this.flamxp=Double.NaN;
	}
    }
    else
    {
	this.fXx=this.fAx=this.fBx=this.flamx=this.muX=Double.NaN;
	this.fXxp=this.fAxp=this.fBxp=this.flamxp=Double.NaN;
    }
}
double getWX()
{
    return getLast(Wa)-this.lam*getLast(Wb);
}
double getdTdMu()
{
    return getLast(Wb)/getWX();
}
double getdVdMu()
{
    return getLast(Wa)/getWX();
}
double getD(final double [] QQf,final int ldQQ)
{
    return TMatr.getAMatrA(this.psiInd.length,this.psiInd,QQf,this.ldQQ)/getFMu(this.psiInd);
}
final double [] getLast2(final double [] F)
{
    final double [] r=new double []{Double.NaN,Double.NaN};
    if(indX>=0) r[0]=F[indX];
    if(indX>=0 && (indX<F.length-1)) r[1]=F[indX+1];
    return r;
}
final double getLast(final double [] F)
{
    if(indX>=0) return F[indX];
    return Double.NaN;
}
final double getFMu(final double [] p)
{
    return TMatr.getAMatrA(p.length,p,this.sQQa,this.ldQQ)-lam*TMatr.getAMatrA(p.length,p,this.sQQb,this.ldQQ);
}

double [] getF(final double [] dfdtmom)
{
    return getF(dfdtmom,true,MT_B);
}
double [] getF(final double [] dfdtmom,final boolean flag_integrate,final int m_AB)
{
    final double [] Ftmom=new double [dfdtmom.length];
    if(flag_integrate)
    {
	access.setFMoments(Ftmom,dfdtmom,dfdtmom.length);
    }
    else
    {
	System.arraycopy(dfdtmom,0,Ftmom,0,dfdtmom.length);
    }
    final int n=psi.length-1;
    final double [] QQF=M.B.getQQMatr(n,Ftmom);
    final double [] F=new double [n];
    java.util.Arrays.fill(F,Double.NaN);
    final int nx=(Xx!=null?Xx.length:0);
    for(int k=0;k<nx;k++)
    {
	final double xk=Xx[k];
	final double [] p=M.B.sdiv1(psi,xk).q;
	final double nr;
	if(m_AB==MT_A)
	{
	    nr=TMatr.getAMatrA(n,p,sQQa,ldQQ);
	}
	else if(m_AB==MT_B)
	{
	    nr=TMatr.getAMatrA(n,p,sQQb,ldQQ);
	}
	else if(m_AB==MT_MU)
	{
	    nr=TMatr.getAMatrA(n,p,sQQa,ldQQ)-lam*TMatr.getAMatrA(n,p,sQQb,ldQQ);
	}
	else throw new RuntimeException("m_AB="+m_AB);
	F[k]=TMatr.getAMatrA(n,p,QQF,n)/nr;
    }
    return F;
}
double [] getFq(final double [] dfdtmom)
{
    final double [] Ftmom=new double [dfdtmom.length];
    access.setFMoments(Ftmom,dfdtmom,dfdtmom.length);
    final int n=psi.length-1;
    final double [] QQF=M.B.getQQMatr(n,Ftmom);
    final double [] F=new double [n];
    java.util.Arrays.fill(F,Double.NaN);
    final double [] IQQb=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,ldQQ,sQQb);
    final double [] QQx=new double [sQQb.length];
    for(int k=QQx.length;--k>=0;){QQx[k]=sQQa[k]-this.lam*sQQb[k];}
    final double [] QQ1=new double [n*n],QQFm=new double [n*n];
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,QQ1,IQQb,n,QQx,ldQQ);
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,QQFm,QQF,n,QQ1,n);
    final int nx=(Xx!=null?Xx.length:0);
    for(int k=0;k<nx;k++)
    {
	final double xk=Xx[k];
	final double [] p=M.B.sdiv1(psi,xk).q;
	F[k]=TMatr.getAMatrA(n,p,QQFm,n)/TMatr.getAMatrA(n,p,QQx,ldQQ);
    }
    return F;
}
double [][] getFqKK(final double [] dfdtmom)
{
    final int n=psi.length-1;
    final double [] QQx=new double [sQQb.length];
    for(int k=QQx.length;--k>=0;){QQx[k]=sQQa[k]-this.lam*sQQb[k];}
    final double [] IQQb=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,ldQQ,sQQb),
	IQQx=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,ldQQ,QQx);
    final double [] Ftmom=new double [dfdtmom.length];
    access.setFMoments(Ftmom,dfdtmom,dfdtmom.length);
    final double [] QQF=M.B.getQQMatr(n,Ftmom);
    final double [] kkmu=M.B.getKK(n,IQQx);
    final double [] QQtmp=new double [n*n];
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,QQtmp,QQF,n,IQQb,n);
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,QQF,IQQx,n,QQtmp,n);
    final double [] kkF=M.B.getKK(n,QQF);
    return new double [][]{kkF,kkmu};
}

void test_getFq(final double [] dfdtmom,final double eps)
{
    final double [] F=getFq(dfdtmom);
    final double [][] kkk=getFqKK(dfdtmom);
    final double [] kkF=kkk[0],kkmu=kkk[1];
    for(int k=0;k<Xx.length;k++)
    {
	final double xk=Xx[k];
	if(Math.abs(xk)<20) // kludge to pass unit test
	{
	final double f=M.B.getD0(kkF,xk)/M.B.getD0(kkmu,xk);
	UnitTests.checkDoubleRel(F[k],"F[("+xk+"),k="+k+"]",f,"f; X="+java.util.Arrays.toString(Xx),eps);
	}
    }
    System.err.println("DkQVMTest:WABpsi:test_getFq(): test OK n="+psi.length+" eps="+eps);
}

double [] getVFXX(final double [] fmom,final int mt)
{
    final double [][] r=getFXX(fmom,mt);
    final double [] v;
    if(indX>=0){v=new double []{r[0][indX],r[1][indX],r[2][indX],r[3][indX]};}
    else{v=new double []{Double.NaN,Double.NaN,Double.NaN,Double.NaN};}
    return v;
}

double [][] getFXX(final double [] fmom,final int mt)
{
    return getFXX(fmom,mt,true);
}
double [][] getFXX(final double [] fmom,final int mt,final boolean flag_xx)
{
    final int n=psi.length-1;
    final double [] QQx=new double [sQQb.length];
    for(int k=QQx.length;--k>=0;){QQx[k]=sQQa[k]-this.lam*sQQb[k];}
    final double [] QQc=new double [n*n];
    if(mt==MT_B)
    {
	final double [] IQQ=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,ldQQ,sQQb);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,QQc,IQQ,n,QQx,ldQQ);
    }
    else if(mt==MT_A)
    {
	final double [] IQQ=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,ldQQ,sQQa);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,QQc,IQQ,n,QQx,ldQQ);
    }
    else if(mt==MT_MU)
    {
	for(int k=0;k<n;k++) {QQc[k+n*k]=1;}
    }
    else throw new RuntimeException("mt="+mt);

    final double [] IQQx=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,ldQQ,QQx);
    final double [] Fxx=new double [n],F2xx=new double [n],F2xxf=new double [n],wxx=new double [n];
    final int nx=(Xx!=null?Xx.length:0);
    final double [] xfmom=M.B.getXQkMomentsFromQkMoments(fmom),xxfmom=new double [xfmom.length];
    final double [] QQmu=new double [n*n],tmp=new double [n*n];
    for(int k=0;k<nx;k++)
    {
	final double xk=Xx[k];
	final double [] p=M.B.sdiv1(psi,xk).q;
	final double fxk=M.B.getD0(p,xk);
	for(int m=0;m<p.length;m++){p[m]/=fxk;}
	wxx[k]=TMatr.getAMatrA(n,p,QQx,ldQQ);
	if(flag_xx)
	{
	    for(int m=0;m<xxfmom.length;m++){xxfmom[m]=(xfmom[m]-xk*fmom[m])*(isEdgeAtRight()?1:-1);}
	}
	else
	{
	    System.arraycopy(fmom,0,xxfmom,0,xxfmom.length);
	}
	final double [] QQxxf=M.B.getQQMatr(n,xxfmom);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,QQmu,QQxxf,n,QQc,n);
	Fxx[k]=TMatr.getAMatrA(n,p,QQmu,n);
	DeprecatedHelperTMatr.multiply_Matr_x_MatrTrans(n,QQxxf,IQQx,n,0,QQmu,n,0);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,QQmu,n,QQxxf,n);
	F2xx[k]=TMatr.getAMatrA(n,p,tmp,n);
	DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,QQxxf,IQQx,n,0,QQmu,n,0);
	DeprecatedHelperTMatr.multiply_MatrTrans_x_Matr(n,tmp,QQmu,n,0,QQxxf,n,0);
	F2xxf[k]=TMatr.getAMatrA(n,p,tmp,n);
    }
    return new double [][]{wxx,Fxx,F2xx,F2xxf};
}


double [][] getFXXtest(final double [] fmom,final int mt)
{
    final int n=psi.length-1;
    final double [] QQxf=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(fmom)),QQf=M.B.getQQMatr(n,fmom);
    final double [] QQx=new double [n*n];
    for(int k=0;k<n;k++)for(int l=0;l<n;l++){QQx[k*n+l]=sQQa[k*ldQQ+l]-this.lam*sQQb[k*ldQQ+l];}
    final double [] IQQx=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQx);
    final double [] IQQd;
    if(mt==MT_B)
    {
	IQQd=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,ldQQ,sQQb);
    }
    else if(mt==MT_A)
    {
	IQQd=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,ldQQ,sQQa);
    }
    else if(mt==MT_MU)
    {
	IQQd=IQQx;
    }
    else throw new RuntimeException("mt="+mt);

    final double [] tmp=new double [n*n];
    final double [] Fxx=new double [n],F2xx=new double [n],F2xxf=new double [n],wxx=new double [n];
    final int nx=(Xx!=null?Xx.length:0);
    for(int k=0;k<nx;k++)
    {
	final double xk=Xx[k];
	final double [] p=M.B.sdiv1(psi,xk).q;
	final double fxk=M.B.getD0(p,xk);
	for(int m=0;m<p.length;m++){p[m]/=fxk;}
	wxx[k]=TMatr.getAMatrA(n,p,QQx,n);
	for(int m=0;m<QQxf.length;m++) {tmp[m]=(QQxf[m]-xk*QQf[m])*(isEdgeAtRight()?1:-1);}
	final double [] q1=TMatr.mulMatrices(n,new double [][]{tmp,IQQd,QQx});
	Fxx[k]=TMatr.getAMatrA(n,p,q1,n);
	final double [] q2=TMatr.mulMatrices(n,new double [][]{tmp,IQQd,QQx,IQQx,QQx,IQQd,tmp});
	F2xx[k]=TMatr.getAMatrA(n,p,q2,n);
	final double [] q2f=TMatr.mulMatrices(n,new double [][]{QQx,IQQd,tmp,IQQx,tmp,IQQd,QQx});
	F2xxf[k]=TMatr.getAMatrA(n,p,q2f,n);
    }
    return new double [][]{wxx,Fxx,F2xx,F2xxf};
}
void test_getFXX(final int ntests,final java.util.Random r,final double eps)
{
    final double [] fmom=new double [2*psi.length-1+1];
    for(int t=0;t<ntests;t++)
    {
	for(int m=0;m<fmom.length;m++) {fmom[m]=2*r.nextDouble()-1;}
	for(int mt:new int []{MT_MU,MT_B,MT_A})
	{
	    final double [][] qt=getFXX(fmom,mt),st=getFXXtest(fmom,mt);
	    for(int j=0;j<qt.length;j++)
	    {
		UnitTests.checkDoubleArrayRel1(qt[j],"mt="+mt+" qt["+j+"]",st[j],"st["+j+"]",eps);
	    }
	    final double [] w=qt[0];
	    for(int k=0;k<w.length;k++)
	    {
		//System.err.println("w["+k+"]="+w[k]+" Wa[k]-lam*Wb[k]="+(Wa[k]-lam*Wb[k]));
		UnitTests.checkDoubleRel(w[k],"w[k]",Wa[k]-lam*Wb[k],"Wa-lam*Wb",eps);
	    }
	}
    }
    System.err.println("DkQVMTest:WABpsi:test_getFXX: test OK n="+psi.length+" ntests="+ntests+" eps="+eps);
}
static final int MT_MU=50,MT_B=51,MT_A=52;



double [][] getFabXX(final double [] fAmom,final double [] fBmom)
{
    final int n=psi.length-1;
    final double [] fQQa=M.B.getQQMatr(n,fAmom),fQQb=M.B.getQQMatr(n,fBmom);
    final double [] Fa=new double [n],Fb=new double [n];
    final int nx=(Xx!=null?Xx.length:0);
    for(int k=0;k<nx;k++)
    {
	final double xk=Xx[k];
	final double [] p=M.B.sdiv1(psi,xk).q;
	final double fxk=M.B.getD0(p,xk);
	for(int m=0;m<p.length;m++){p[m]/=fxk;}
	Fa[k]=TMatr.getAMatrA(n,p,fQQa,n)/TMatr.getAMatrA(n,p,sQQa,ldQQ);
	Fb[k]=TMatr.getAMatrA(n,p,fQQb,n)/TMatr.getAMatrA(n,p,sQQb,ldQQ);
    }
    return new double [][]{Fa,Fb};
}

void test_getFabXX(final int ntests,final java.util.Random r,final double eps)
{
    final double [] fmom=new double [2*psi.length-1+1];
    for(int t=0;t<ntests;t++)
    {
	for(int m=0;m<fmom.length;m++) {fmom[m]=2*r.nextDouble()-1;}
	final double [][] qt=getFabXX(fmom,fmom);
	final double [] fa=getF(fmom,false,WABpsi.MT_A),fb=getF(fmom,false,WABpsi.MT_B);
	UnitTests.checkDoubleArrayRel(qt[0],"qt[0]",fa,"fa",eps);
	UnitTests.checkDoubleArrayRel(qt[1],"qt[1]",fb,"fb",eps);
    }
    System.err.println("DkQVMTest:WABpsi:test_getFabXX: test OK n="+psi.length+" ntests="+ntests+" eps="+eps);
}

double [][] getFxFx2(final double [] fmom)
{
    final double [][] m=get012Moms(fmom);
    final double [] Fx=new double [m[0].length],Fx2=new double [Fx.length];
    for(int k=0;k<Fx.length;k++)
    {
	Fx[k]=m[1][k]/m[0][k];
	Fx2[k]=m[2][k]/m[0][k]-Math.pow(Fx[k],2);
    }
    return new double [][]{Fx,Fx2};
}
void test_getFxFx2(final int ntests,final java.util.Random r,final double eps)
{
    final int n=psi.length-1;
    final double [] fmom=new double [2*psi.length-1+1];
    final double [] xfmom=new double [fmom.length-2],x2fmom=new double [fmom.length-2];
    final double [] xF=new double [n],x2F=new double [n];
    for(int t=0;t<ntests;t++)
    {
	for(int m=0;m<fmom.length;m++) {fmom[m]=2*r.nextDouble()-1;}
	for(int m=0;m<xfmom.length;m++)
	{
	    final double [] tmp=new double [m+1];tmp[m]=1;
	    final double [] xtmp=M.getxP(tmp),x2tmp=M.getxP(xtmp);
	    xfmom[m]=TMatr.mulVVn1(xtmp,fmom);
	    x2fmom[m]=TMatr.mulVVn1(x2tmp,fmom);
	}
	final double [] QQ=M.B.getQQMatr(n,fmom),QQx=M.B.getQQMatr(n,xfmom),QQx2=M.B.getQQMatr(n,x2fmom);
	for(int k=0;k<n;k++)
	{
	    final double xk=Xx[k];
	    final double [] p=M.B.sdiv1(psi,xk).q;
	    final double qq=TMatr.getAMatrA(n,p,QQ,n);
	    xF[k]=TMatr.getAMatrA(n,p,QQx,n)/qq;
	    x2F[k]=TMatr.getAMatrA(n,p,QQx2,n)/qq-xF[k]*xF[k];
	}

	final double [][] qt=getFxFx2(fmom);
	UnitTests.checkDoubleArrayRel(qt[0],"qt[0]",xF,"xF",eps);
	UnitTests.checkDoubleArrayRel(qt[1],"qt[1]",x2F,"x2F",eps);
    }
    System.err.println("DkQVMTest:WABpsi:test_getFxFx2: test OK n="+psi.length+" ntests="+ntests+" eps="+eps);
}


double [][] get012Moms(final double [] fmom)
{
    final int n=psi.length-1;
    final double [] fxmom=M.B.getXQkMomentsFromQkMoments(fmom),fx2mom=M.B.getXQkMomentsFromQkMoments(fxmom);
    final double [] fQQ=M.B.getQQMatr(n,fmom),fxQQ=M.B.getQQMatr(n,fxmom),fx2QQ=M.B.getQQMatr(n,fx2mom);
    final double [] m0=new double [n],m1=new double [n],m2=new double [n];
    java.util.Arrays.fill(m0,Double.NaN);
    java.util.Arrays.fill(m1,Double.NaN);
    java.util.Arrays.fill(m2,Double.NaN);
    final int nx=(Xx!=null?Xx.length:0);
    for(int k=0;k<nx;k++)
    {
	final double xk=Xx[k];
	final double [] p=M.B.sdiv1(psi,xk).q;
	m0[k]=TMatr.getAMatrA(n,p,fQQ,n);
	m1[k]=TMatr.getAMatrA(n,p,fxQQ,n);
	m2[k]=TMatr.getAMatrA(n,p,fx2QQ,n);
    }
    return new double [][]{m0,m1,m2};
}
/** First element xplus, second- xminus.
 */
double [] getXMM(final double [] fplus,final double [] fminus)
{
    final double [][] mplus=get012Moms(fplus),mminus=get012Moms(fminus);
    final double m0plus=getLast(mplus[0]),m1plus=getLast(mplus[1]),m2plus=getLast(mplus[2]),
	m0minus=getLast(mminus[0]),m1minus=getLast(mminus[1]),m2minus=getLast(mminus[2]);
    //System.err.println("getXMM:m0plus="+m0plus+" m1plus="+m1plus+" m2plus="+m2plus);
    //System.err.println("getXMM:m0minus="+m0minus+" m1minus="+m1minus+" m2minus="+m2minus);
    final double [] p=new double []{
	m1plus*m2minus-m1minus*m2plus,
	m2plus*m0minus-m2minus*m0plus,
	m0plus*m1minus-m0minus*m1plus
    };
    final PolynomialRootsFinder.PRoots u=com.polytechnik.utils.PolynomialRootsElementary.ROOTSFINDER.findRoots(p);
    final double [] re=u.re_root,im=u.im_root;
    if(u.nroots!=2)
    {
	java.util.Arrays.fill(re,Double.NaN);
	return re;
    }

    final double x0=re[0],x1=re[1],
	lam0=(m2plus+x0*(-2*m1plus+x0*m0plus))/(m2minus+x0*(-2*m1minus+x0*m0minus)),
	lam1=(m2plus+x1*(-2*m1plus+x1*m0plus))/(m2minus+x1*(-2*m1minus+x1*m0minus));
    if(lam0<=lam1)
    {
    }
    else
    {
	final double xx=re[0];
	re[0]=re[1];
	re[1]=xx;
    }
    return re;
}
double [] getXMMTest(final double [] fplus,final double [] fminus)
{
    final int n=psiInd.length;
    final double [] Xfplus=M.B.getXQkMomentsFromQkMoments(fplus),X2fplus=M.B.getXQkMomentsFromQkMoments(Xfplus),
	Xfminus=M.B.getXQkMomentsFromQkMoments(fminus),X2fminus=M.B.getXQkMomentsFromQkMoments(Xfminus);
    final double m0plus=TMatr.getAMatrA(n,psiInd,M.B.getQQMatr(n,fplus),n),
	m1plus=TMatr.getAMatrA(n,psiInd,M.B.getQQMatr(n,Xfplus),n),
	m2plus=TMatr.getAMatrA(n,psiInd,M.B.getQQMatr(n,X2fplus),n),
	m0minus=TMatr.getAMatrA(n,psiInd,M.B.getQQMatr(n,fminus),n),
	m1minus=TMatr.getAMatrA(n,psiInd,M.B.getQQMatr(n,Xfminus),n),
	m2minus=TMatr.getAMatrA(n,psiInd,M.B.getQQMatr(n,X2fminus),n);
    //System.err.println("m0plus="+m0plus+" m1plus="+m1plus+" m2plus="+m2plus);
    //System.err.println("m0minus="+m0minus+" m1minus="+m1minus+" m2minus="+m2minus);
    final int no=2;
    final double [] ev=M.EV.getGEV_Generalized(no,new double []{m0plus,m1plus,m1plus,m2plus},
					    new double []{m0minus,m1minus,m1minus,m2minus},true);
    //System.err.println("ev="+java.util.Arrays.toString(ev)+"\npsiInd="+java.util.Arrays.toString(psiInd));
    return new double []{-ev[no+0*no+0]/ev[no+0*no+1],-ev[no+1*no+0]/ev[no+1*no+1]};
}
double [] getXMMTest2(final double [] fplus,final double [] fminus)
{
    final int n=psiInd.length;
    final BasisFunctionsCalculatable B=getBasisPolynomials();
    final double [] b0=new double []{1},b1=new double []{0,1};
    final double [] pp=B.mult2Pol(psiInd,psiInd);
    double [] temp=B.mult2Pol(pp,b0);
    final double [] p00=B.mult2Pol(temp,b0),p01=B.mult2Pol(temp,b1);
    temp=B.mult2Pol(pp,b1);
    final double [] p11=B.mult2Pol(temp,b1);

    final int no=2;
    final double [] ev=M.EV.getGEV_Generalized(no,new double []{
	    TMatr.mulVVn1(p00,fplus),
	    TMatr.mulVVn1(p01,fplus),
	    TMatr.mulVVn1(p01,fplus),
	    TMatr.mulVVn1(p11,fplus)},
	new double []{
	    TMatr.mulVVn1(p00,fminus),
	    TMatr.mulVVn1(p01,fminus),
	    TMatr.mulVVn1(p01,fminus),
	    TMatr.mulVVn1(p11,fminus)},
	true);
    //System.err.println("ev="+java.util.Arrays.toString(ev)+"\npsiInd="+java.util.Arrays.toString(psiInd));
    final double [] p=new double [no];
    System.arraycopy(ev,no+0*no,p,0,no);
    final double xl=-M.B.getD0(p,0)/M.B.getD1(p,0);
    System.arraycopy(ev,no+1*no,p,0,no);
    final double xh=-M.B.getD0(p,0)/M.B.getD1(p,0);
    return new double []{xl,xh};
}



void test_getXMM(final int ntests,final java.util.Random r,final double eps)
{
    final int n=psi.length-1;
    for(int t=0;t<ntests;t++)
    {
	final double [] fplus=getRandomPositivelyDefined(2*psi.length-1+1,1000,r),
	    fminus=getRandomPositivelyDefined(2*psi.length-1+1,1000,r);
	//System.err.println("fplus="+java.util.Arrays.toString(fplus));
	//System.err.println("fminus="+java.util.Arrays.toString(fminus));
	final double [] q=getXMM(fplus,fminus),s=getXMMTest(fplus,fminus),s2=getXMMTest2(fplus,fminus);

	if(n==1 && false)
	{
	    final WLHpsi qw=new WLHpsi(this,fplus,fminus);
	    UnitTests.checkDoubleRel(s[0],"s[0]",qw.Xxplus[0],"qw.Xxplus[0]",eps);
	    UnitTests.checkDoubleRel(s[1],"s[1]",qw.Xxminus[0],"qw.Xxminus[0]",eps);
	    //System.err.println("qw.Xxplus[0]="+qw.Xxplus[0]+" qw.Xxminus[0]="+qw.Xxminus[0]);
	    //System.err.println("qw.PM.sL="+qw.PM.sL+" qw.PM.sH="+qw.PM.sH);
	}
	UnitTests.checkDoubleArrayRel(q,"q",s,"s",eps);
	UnitTests.checkDoubleArrayRel(s2,"s2",s,"s",eps);

    }
    System.err.println("DkQVMTest:WABpsi:test_getXMM: test OK n="+psi.length+" ntests="+ntests+" eps="+eps);
}


double [][] getFXabc(final double [] fmom,final boolean flag_xf)
{
    final int n=psi.length-1;
    final double [] QQf_=M.B.getQQMatr(n,fmom),QQfX_=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(fmom));
    final double [] QQx=new double [n*n];
    for(int k=0;k<n;k++)for(int l=0;l<n;l++){QQx[k*n+l]=sQQa[k*ldQQ+l]-this.lam*sQQb[k*ldQQ+l];}
    final double [] IQQx=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQx);

    final double [] rxk=new double [n],rtmp=new double [n];
    final int nx=(Xx!=null?Xx.length:0);
    final double [] a=new double [n],b=new double [n],c=new double [n];
    final double [] tmp=new double [n*n];
    for(int k=0;k<nx;k++)
    {
	final double xk=Xx[k];
	final double [] QQf;
	if(flag_xf)
	{
	    for(int m=0;m<QQf_.length;m++) {tmp[m]=(QQfX_[m]-xk*QQf_[m])*(isEdgeAtRight()?1:-1);}
	    QQf=tmp;
	}
	else
	{
	    QQf=QQf_;
	}
	final double [] QQfIQQxQQf=TMatr.mulMatrices(n,new double [][]{QQf,IQQx,QQf}),
	    IQQxf=TMatr.mulMM(n,IQQx,QQf);
	for(int j=0;j<n;j++)
	{
	    rtmp[j]=1;
	    rxk[j]=M.B.getD0(rtmp,xk);
	    rtmp[j]=0;
	}

	final double [] p=M.B.sdiv1(psi,xk).q;
	final double ff=M.B.getD0(p,xk);
	final double wk=TMatr.getAMatrA(n,p,QQx,n);
	a[k]=(ff*ff)*TMatr.getAMatrA(n,rxk,IQQx,n)/wk;
	b[k]=2*ff*TMatr.getAMatrB(n,rxk,0,n,p,0,IQQxf,n)/wk;
	c[k]=TMatr.getAMatrA(n,p,QQfIQQxQQf,n)/wk;
    }
    return new double [][]{c,b,a};
}


double [][] getFXabcTest(final double [] fmom,final boolean flag_xf)
{
    final int n=psi.length-1;
    final double [] QQf_=M.B.getQQMatr(n,fmom),QQfX_=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(fmom));
    final double [] QQx=new double [n*n],tmp=new double [n*n];
    for(int k=0;k<n;k++)for(int l=0;l<n;l++){QQx[k*n+l]=sQQa[k*ldQQ+l]-this.lam*sQQb[k*ldQQ+l];}
    final double [] IQQx=com.polytechnik.utils.Linsystems.getInvertedMatrix(n,n,QQx);

    final int nx=(Xx!=null?Xx.length:0);
    final double [] rtmp=new double [n],rxk=new double [n];
    final double [] a=new double [n],b=new double [n],c=new double [n];
    for(int k=0;k<nx;k++)
    {
	final double xk=Xx[k];
	final double [] QQf;
	if(flag_xf)
	{
	    for(int m=0;m<QQf_.length;m++) {tmp[m]=(QQfX_[m]-xk*QQf_[m])*(isEdgeAtRight()?1:-1);}
	    QQf=tmp;
	}
	else
	{
	    QQf=QQf_;
	}
	final double []
	    IQQxQQfIQQx=TMatr.mulMatrices(n,new double [][]{IQQx,QQf,IQQx}),
	    IQQxQQfIQQxQQfIQQx=TMatr.mulMatrices(n,new double [][]{IQQxQQfIQQx,QQf,IQQx});

	for(int j=0;j<n;j++)
	{
	    rtmp[j]=1;
	    rxk[j]=M.B.getD0(rtmp,xk);
	    rtmp[j]=0;
	}
	final double K=TMatr.getAMatrA(n,rxk,IQQx,n);
	a[k]=K*K;
	b[k]=2*TMatr.getAMatrA(n,rxk,IQQxQQfIQQx,n);
	c[k]=TMatr.getAMatrA(n,rxk,IQQxQQfIQQxQQfIQQx,n)/K;
    }
    return new double [][]{c,b,a};
}

void test_getFXabc(final int ntests,final java.util.Random r,final double eps)
{
    final double [] fmom=new double [2*psi.length-1+1];
    for(int t=0;t<ntests;t++)
    {
	for(int m=0;m<fmom.length;m++) {fmom[m]=2*r.nextDouble()-1;}
	for(boolean f:new boolean []{false,true})
	{
	    final double [][] abc=getFXabc(fmom,f),abctest=getFXabcTest(fmom,f);
	    UnitTests.checkDoubleArrayRel1(abc[0],"abc[0]",abctest[0],"abctest[0]",eps);
	    UnitTests.checkDoubleArrayRel1(abc[1],"abc[1]",abctest[1],"abctest[1]",eps);
	    UnitTests.checkDoubleArrayRel1(abc[2],"abc[2]",abctest[2],"abctest[2]",eps);
	    final double [][]  ff=getFXX(fmom,WABpsi.MT_MU,f);
	    final double [] wx=ff[0],f1=ff[1],f2=ff[2];
	    final double [] at=new double [wx.length],bt=new double [wx.length],ct=new double [wx.length];
	    for(int k=0;k<wx.length;k++)
	    {
		at[k]=Math.pow(1/wx[k],2);
		bt[k]=2*f1[k]/wx[k]/wx[k];
		ct[k]=f2[k]/wx[k];
	    }
	    UnitTests.checkDoubleArrayRel(abc[2],"abc[2]",at,"at",eps);
	    UnitTests.checkDoubleArrayRel(abc[1],"abc[1]",bt,"bt",eps);
	    UnitTests.checkDoubleArrayRel(abc[0],"abc[0]",ct,"ct",eps);

	    final double KK=getLast(abc[2]);
	    UnitTests.checkDoubleRel(getWX(),"getWX()",1/Math.sqrt(KK),"1/sqrt(KK)",eps);
	    UnitTests.checkDoubleRel(getDelta(fmom,f)[0],"getDelta(fmom,f)[0]",getRelDf(fmom,f),"getRelDf(fmom,f)",eps);
	}
    }
    System.err.println("DkQVMTest:WABpsi:test_getFXabc: test OK n="+psi.length+" ntests="+ntests+" eps="+eps);
}

double getRelDf(final double [] fmom,final boolean flag_xf)
{
    final double [][] abc=getFXabc(fmom,flag_xf);
    final double a=getLast(abc[2]),b=getLast(abc[1]),c=getLast(abc[0]);

    final double delta0=-b/(2*a);
    final double delta0rel;
    if(false)
    { // version with fmin
	final double f0min=c-a*Math.pow(delta0,2);
	delta0rel=-delta0*Math.sqrt(a/f0min);
    }
    else
    {
	delta0rel=-delta0;
    }
    return delta0rel;
}

double [] getDelta(final double [] fmom,final boolean flag_xf)
{
    final double [][] abc=getFXabc(fmom,flag_xf);
    final double [] a=abc[2],b=abc[1],c=abc[0];
    final int n=psi.length-1;
    double s=0,Ki=Double.NaN,deltai=Double.NaN;
    for(int k=0;k<n;k++)
    {
	final double K=Math.sqrt(a[k]),delta=-b[k]/(2*a[k]);
	if(k==indX)
	{
	    Ki=K;
	    deltai=-delta;
	}
	else
	{
	    s+=Math.pow(delta,2)*K;
	}
    }
    s=s/Ki;
    return new double []{deltai,s};
}

} // END OF class WABpsi


double [] getX0X1X2(final double [] p,final double [] fmom)
{
    final int n=p.length;
    final double [] fxmom=M.B.getXQkMomentsFromQkMoments(fmom),fx2mom=M.B.getXQkMomentsFromQkMoments(fxmom);
    final double [] fQQ=M.B.getQQMatr(n,fmom),fxQQ=M.B.getQQMatr(n,fxmom),fx2QQ=M.B.getQQMatr(n,fx2mom);
    final double
	x0=TMatr.getAMatrA(n,p,fQQ,n),
	x1=TMatr.getAMatrA(n,p,fxQQ,n),
	x2=TMatr.getAMatrA(n,p,fx2QQ,n);
    return new double []{x0,x1,x2};
}


class WXpsi {
    final double [] psi;
    final double lam;
    final double [] Xe,Xx,W,Ww0,Wdp,Wms,Px;
    final double fXx,fW,fWdp,fWms,fPx;
    final double [] kkx,kkdp,kkms/*,kkP1*/;
    final double fW_x0,fWdp_x0,fWms_x0;
    final int sign;
    final double [] sQQx; // for unit test
WXpsi(final double [] psi,final double lam,
      final double [] imom,final double [] tmom,final int sign,
      final double [] dpmom,
      final double [] vsmom,final double [] tsmom,
      final double [] P1vmom,final double [] P1tmom)
{
    this.psi=psi;
    final int n=psi.length;
    this.lam=lam;
    this.sign=sign;
    final double x0=getX0();
    final double [][] r=getRoots(psi,M,isEdgeAtRight());
    this.Xx=r!=null?r[0]:null;
    final int indX=DjQVM.getIndX0(Xx,x0,isEdgeAtRight());
    this.fXx=indX>=0?Xx[indX]:Double.NaN;
    final int nmoms=tmom.length;
    final double [] xmom=new double [nmoms],msmom=new double [nmoms],P1xmom=new double [nmoms];
    for(int k=nmoms;--k>=0;) {
	xmom[k]=sign*(imom[k]-lam*tmom[k]);
	msmom[k]=sign*(vsmom[k]-lam*tsmom[k]);
	P1xmom[k]=sign*(P1vmom[k]-lam*P1tmom[k]);
    }

    final double [] QQx=M.B.getQQMatr(n,xmom),QQdp=M.B.getQQMatr(n,dpmom),
	QQms=M.B.getQQMatr(n,msmom),QQP1x=M.B.getQQMatr(n,P1xmom),
	QQXe=M.B.getQQMatr(n,M.B.getXQkMomentsFromQkMoments(xmom));
    this.sQQx=QQx;
    final double [][] Wres=DjQVM.getIntegralEstimation(Xx,psi,new double [][]{QQdp,QQms,QQP1x,QQXe},QQx,n,M);
    Wdp=Wres[0];
    Wms=Wres[1];
    Px=Wres[2];
    Xe=Wres[3];
    W=Wres[4];

    if(indX>=0)
    {
	fWdp=Wdp[indX];
	fWms=Wms[indX];
	fPx=getPOffset()+Px[indX];
	fW=W[indX];
    }
    else
    {
	fWdp=fWms=fPx=fW=Double.NaN;
    }

    final int nl=n-1;
    final double [] IQQ=com.polytechnik.utils.Linsystems.getInvertedMatrix(nl,n,QQx);
    kkx=M.B.getKK(nl,IQQ);
    kkdp=M.B.getKK(nl,getMNom(nl,IQQ,nl,QQdp,n));
    kkms=M.B.getKK(nl,getMNom(nl,IQQ,nl,QQms,n));
    this.Ww0=new double [nl];
    final double [] r0=new double [nl];
    for(int k=0;k<nl;k++)
    {
	Ww0[k]=1;
	r0[k]=M.B.getD0(Ww0,x0);
	Ww0[k]=0;
    }
    final double [] fr0=TMatr.getMatr_x_A(nl,IQQ,r0);
    for(int k=0;k<nl;k++)
    {
	final double xk=Xx!=null?Xx[k]:Double.NaN;
	Ww0[k]=M.B.getD0(fr0,xk)/Math.sqrt(M.B.getD0(kkx,x0)*M.B.getD0(kkx,xk));
    }

    fW_x0=1/M.B.getD0(kkx,x0);
    fWdp_x0=M.B.getD0(kkdp,x0)*fW_x0;
    fWms_x0=M.B.getD0(kkms,x0)*fW_x0;
    stmoms=tmom;
    simoms=imom;
}
double [] getMNom(final int n,final double [] IQQ,final int nIQQ,final double [] fQQ,final int nfQQ)
{
    final double [] tmp=new double [n*n],tmp2=new double [n*n];
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp,fQQ,nfQQ,IQQ,nIQQ);
    DeprecatedHelperTMatr.multiply_Matr_x_Matr(n,tmp2,IQQ,nIQQ,tmp,n);
    return tmp2;
}
final double [] stmoms,simoms;
void test_WXpsi(final double eps)
{
    UnitTests.checkDoubleArrayRel(Xx,"Xx",Xe,"Xe",eps);
    final double [] QQt=M.B.getQQMatr(psi.length,stmoms),QQi=M.B.getQQMatr(psi.length,simoms);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(psi.length,psi,QQt,psi.length),"<psi|QQt|psi>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(psi.length,psi,QQi,psi.length),"<psi|QQi|psi>",lam,"lam",eps);

    final double afW=1/M.B.getD0(kkx,fXx);
    UnitTests.checkDoubleRel(fW,"fW",afW,"afW",eps);
    final double afWdp=afW*M.B.getD0(kkdp,fXx),afWms=afW*M.B.getD0(kkms,fXx);
    UnitTests.checkDoubleRel(fWdp,"fXx="+fXx+" fWdp",afWdp,"afWdp",eps);
    UnitTests.checkDoubleRel(fWms,"fXx="+fXx+" fWms",afWms,"afWms",eps);



    System.err.println("DkQVMTest:WXpsi:test_WXpsi(): test OK n="+psi.length+" eps="+eps);
}

void test_WXpsipsi0(final WXpsipsi0 q,final double eps)
{
    UnitTests.checkDoubleArrayRel(W,"W",q.W,"q.W",eps);
    UnitTests.checkDoubleArrayRel(Xx,"Xx",q.Xx,"q.Xx",eps);
    final double [] ww=new double [Xx.length],qww=new double [q.Xx.length];
    for(int k=0;k<Xx.length;k++)
    {
	ww[k]=Math.abs(Ww0[k]);
	qww[k]=Math.abs(q.Ww0[k]);
    }
    UnitTests.checkDoubleArrayRel(ww,"ww",qww,"qww",eps);

    System.err.println("DkQVMTest:WXpsi:test_WXpsipsi0(): test OK n="+psi.length+" eps="+eps);
}
void test_WABpsi(final WABpsi q,final double eps)
{
    UnitTests.checkDoubleRel(q.lam,"q.lam",this.lam,"this.lam",eps);
    UnitTests.checkDoubleArrayRel(EVXData.getMSign(q.psi,this.psi[this.psi.length-1]),"q.psi",this.psi,"this.psi",eps);
    UnitTests.checkDoubleArrayRel(Xx,"Xx",q.Xx,"q.Xx",eps);
    final int n=q.psi.length;
    UnitTests.checkDoubleRel(TMatr.getAMatrA(n,q.psi,M.B.getQQMatr(n,stmoms),n),"<psi|QQb|psi>",1,"1",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(n,q.psi,M.B.getQQMatr(n,simoms),n),"<psi|QQa|psi>",q.lam,"q.lam",eps);

    final double [] w=new double [Xx.length];
    for(int k=w.length;--k>=0;)
    {
	w[k]=(q.Wa[k]-q.lam*q.Wb[k])*this.sign;
	if(w[k]<=0) throw new RuntimeException("k="+k+" w="+java.util.Arrays.toString(w));
    }
    UnitTests.checkDoubleArrayRel(w,"w",this.W,"this.W",eps);
    UnitTests.checkDoubleRel(TMatr.getAMatrA(q.psi.length,q.psi,this.sQQx,q.psi.length),"<psi|QQx|psi>",0,"0",eps);
    System.err.println("DkQVMTest:WXpsi:test_WABpsi(): test OK n="+psi.length+" eps="+eps);
}


} // END OF class WXpsi


class WXpsipsi0 {
final double [] Xx,W,Ww0,Wot,Wov;
final double [] psi;
final int indM;
final double lam,x0,fXx,flamx,flamxvv,fWx,flamx0,fwMw0,K00,w0;
final int sign;

final double [] sQQx,sQQt,sQQi;

WXpsipsi0(final double [] psi,final double lam,
	  final double [] imom,final double [] tmom,final int sign)
{
    this.psi=psi;
    this.sign=sign;
    final int n=psi.length-1;
    this.lam=lam;
    this.x0=getX0();
    final double [] xmom=new double [tmom.length];
    for(int k=xmom.length;--k>=0;)
    {
	xmom[k]=sign*(imom[k]-lam*tmom[k]);
    }
    final double [] QQx=M.B.getQQMatr(n,xmom),QQt=M.B.getQQMatr(n,tmom),QQi=M.B.getQQMatr(n,imom);
    this.sQQx=QQx;
    this.sQQt=QQt;
    this.sQQi=QQi;
    final double [] ok=access.getOK(n,QQx,x0);
    this.K00=M.B.getD0(ok,x0);
    this.w0=1/this.K00;
    this.flamx0=TMatr.getAMatrA(n,ok,QQx,n)/TMatr.getAMatrA(n,ok,QQt,n);

    final double [][] r=getRoots(psi,M,isEdgeAtRight());
    this.Xx=r!=null?r[0]:null;
    this.W=new double [n];
    this.Ww0=new double [n];
    this.Wot=new double [n];
    this.Wov=new double [n];
    int ind=-1;
    double wm=Double.NEGATIVE_INFINITY;
    for(int k=0;k<n;k++)
    {
	final double xk=Xx!=null?Xx[k]:Double.NaN;
	final double [] p=M.B.sdiv1(psi,xk).q;
	final double fk=M.B.getD0(p,xk),wn=TMatr.getAMatrA(n,p,QQx,n),wk=W[k]=wn/(fk*fk),
	    tn=TMatr.getAMatrA(n,p,QQt,n);
	this.Wot[k]=wn/tn;
	this.Wov[k]=wn/TMatr.getAMatrA(n,p,QQi,n);
	Ww0[k]=(M.B.getD0(p,x0)/fk)/Math.sqrt(wk*K00);
	if(Math.abs(Ww0[k])>wm)
	{
	    wm=Math.abs(Ww0[k]);
	    ind=k;
	}
    }
    this.indM=DjQVM.getIndX0(Xx,x0,isEdgeAtRight());
    if(indM>=0)
    {
	fXx=Xx[indM];
	fWx=W[indM];
	flamx=Wot[indM];
	flamxvv=Wov[indM];
	fwMw0=Ww0[indM];
    }
    else
    {
	fXx=fWx=flamx=flamxvv=fwMw0=Double.NaN;
    }
}

void testRoots(final double eps)
{
    final int n=psi.length-1;
    for(int k=0;k<Xx.length;k++)
    {
	final double [] ok=access.getOK(n,sQQx,Xx[k]);
	final double w=1/M.B.getD0(ok,Xx[k]);
	UnitTests.checkDoubleRel(w,"w",W[k],"W[k]",eps);
	final double wot=TMatr.getAMatrA(n,ok,sQQx,n)/TMatr.getAMatrA(n,ok,sQQt,n);
	UnitTests.checkDoubleRel(wot,"wot",Wot[k],"Wot[k]",eps);
	final double wov=TMatr.getAMatrA(n,ok,sQQx,n)/TMatr.getAMatrA(n,ok,sQQi,n);
	UnitTests.checkDoubleRel(wov,"wov",Wov[k],"Wov[k]",eps);
    }
    System.err.println("DkQVMTest:WXpsipsi0::testRoots(): test OK n="+psi.length+" eps="+eps);
}

void test_WABpsi(final WABpsi s,final double eps)
{
    UnitTests.checkDoubleRel(lam,"lam",s.lam,"s.lam",eps);
    UnitTests.checkDoubleArrayRel(Xx,"Xx",s.Xx,"s.Xx",eps);
    final double [] wot=new double [Xx.length],wov=new double [Xx.length];
    for(int k=0;k<wot.length;k++)
    {
	wot[k]=this.sign*(s.Wa[k]/s.Wb[k]-s.lam);
	wov[k]=this.sign*(1-s.lam*s.Wb[k]/s.Wa[k]);
    }
    UnitTests.checkDoubleArrayRel(Wot,"Wot",wot,"wot",eps);
    UnitTests.checkDoubleArrayRel(Wov,"Wov",wov,"wov",eps);
    UnitTests.checkDoubleRel(fXx,"fXx",s.fXx,"s.fXx",eps);
    UnitTests.checkDoubleRel(flamx,"flamx",sign*(s.flamx-s.lam),"sign*(s.flamx-s.lam)",eps);
    //UnitTests.checkDoubleRel(flamx0,"flamx0",s.flamx0-s.lam,"s.flamx0-s.lam0",eps);
    UnitTests.checkDoubleRel(fWx,"fWx",sign*(s.fAx-s.lam*s.fBx),"sign*(s.fAx-s.lam*s.fBx)",eps);
    UnitTests.checkDoubleRel(TMatr.mulVVn1(Ww0,Ww0),"Ww0^2",1,"1",eps);
    System.err.println("DkQVMTest:WXpsipsi0::test_WABpsi(): test OK n="+psi.length+" eps="+eps);
}

}

void test_ED(final int n,final java.util.Random r,final double eps)
{
    final int n_tests=10000;
    final BasisFunctionsCalculatable B=getBasisPolynomials();
    final double [] D=getED(n);
    final double [] p=new double [n],xpM=new double [n+1],
	xdM=new double [n-1];
    for(int t=0;t<n_tests;t++)
    {
	final double x0=r.nextDouble();
	final double [] MUL=getMulX(n,x0),SD=getSDivX(n,x0);
	for(int k=0;k<p.length;k++) {p[k]=1-2*r.nextDouble();}
	UnitTests.checkDoubleArrayRel(getEDPsi(p),"getEDPsi(p)",TMatr.getMatr_x_A(n,D,p),"Dp",eps);
	final double [] xp=B.getaxbP(p,1,-x0);
	TMatr.multiply_MA_x_MB(xpM,n+1,MUL,n,p,1);
	UnitTests.checkDoubleArrayRel(xp,"xp",xpM,"xpM",eps);
	TMatr.multiply_MA_x_MBTrans(xpM,n+1,MUL,n,p,1);
	UnitTests.checkDoubleArrayRel(xp,"xp",xpM,"xpM",eps);
	final double [] xd=B.sdiv1(p,x0).q;
	TMatr.multiply_MA_x_MB(xdM,n-1,SD,n,p,1);
	UnitTests.checkDoubleArrayRel(xd,"xd",xdM,"xdM",eps);

	final double [] edpsi=getEDPsi(p),edpsix=getEDPsiX(p,x0);
	UnitTests.checkDoubleRel(M.B.getD0(edpsi,x0),"edpsi(x0)",M.B.getD0(edpsix,x0),"edpsix(x0)",eps);

    }
    System.err.println("DkQVM:test_ED(): test OK n="+n+" eps="+eps);
}


static abstract class DkQVMTest<DTYPE> extends QTest {

abstract DkQVM<DTYPE> getDkQVM();
abstract DjQVM<?> getDjQVM();
abstract BiQVM<?> getBiQVM();
abstract EQVM<?> getEQVM();


DkQVMTest(final String name,
	  final java.util.Random r,final double eps)
{
    super(name,r,eps);
}




void test_Common()
{
    final DkQVM<DTYPE> q=getDkQVM();

    q.L.test_WXpsi(eps);
    q.H.test_WXpsi(eps);
    q.dpL.test_WXpsi(eps);
    q.dpH.test_WXpsi(eps);

    q.L.test_WXpsipsi0(q.iL,eps);
    q.H.test_WXpsipsi0(q.iH,eps);

    q.iL.testRoots(eps);
    q.iH.testRoots(eps);
    q.iL.test_WABpsi(q.jL,eps);
    q.iH.test_WABpsi(q.jH,eps);


    q.L.test_WABpsi(q.jL,eps);
    q.H.test_WABpsi(q.jH,eps);

    //q.ssL.test_WXpsi(eps);
    //q.ssH.test_WXpsi(eps);
    q.ssL.test_WABpsi(q.JbsL,eps);
    q.ssH.test_WABpsi(q.JbsH,eps);
    EVXData.matchEVXData(q.IRBS,q.rpmf.PM,true,true,eps);
    UnitTests.checkDoubleRel(q.rpmf.mu0_psiL,"q.rpmf.mu0_psiL",0,"0",eps);
    UnitTests.checkDoubleRel(q.rpmf.mu0_psiH,"q.rpmf.mu0_psiH",0,"0",eps);
    EVXData.matchEVXData(q.dpT,q.jdpt.J,true,true,eps);
    EVXData.matchEVXData(q.JTVS,q.jvSt.J,true,true,eps);
    EVXData.matchEVXData(q.IT,q.jdpt.I,true,true,eps);
    q.jvSt.test_Self(eps);
    q.jdpt.test_Self(eps);

    q.rpmf.test_matchWfPsi(q.x_rpmf,eps);
    q.rvst.test_matchWfPsi(q.x_rvst,eps);
    q.vst.test_matchWfPsi(q.x_vst,eps);
    q.dpt.test_matchWfPsi(q.x_dpt,eps);

    /*
    // test interlacing
    final boolean fl=q.L.Xx[0]<q.H.Xx[0];
    for(int k=0;k<q.L.Xx.length;k++)
    {
	boolean fb=false;
	if(fl)
	{
	    if(!(q.L.Xx[k]<q.H.Xx[k])) fb=true;
	}
	else
	{
	    if(!(q.L.Xx[k]>q.H.Xx[k])) fb=true;
	}
	if(fb) throw new RuntimeException("interlacing is broken:\nq.L.Xx="+java.util.Arrays.toString(q.L.Xx)+"\nq.H.Xx="+java.util.Arrays.toString(q.H.Xx));
    }
    */

    UnitTests.checkDoubleRel(q.sXi,"q.sXi",q.L.fXx,"q.L.fXx",eps);
    UnitTests.checkDoubleRel(q.jL_vx,"q.jL_vx",q.jL.fAx/(q.jL.fAx-q.jL.lam*q.jL.fBx),"f(jL)",eps);

    UnitTests.checkDoubleRel(q.jL_VS,"q.jL_VS",q.jL_VSd,"q.jL_VSd",eps);
    UnitTests.checkDoubleRel(q.jL_VSp,"q.jL_VSp",q.jL_VSdp,"q.jL_VSdp",eps);
    UnitTests.checkDoubleRel(q.jL_V,"q.jL_V",q.jL_Vd,"q.jL_Vd",eps);
    UnitTests.checkDoubleRel(q.jL_Vp,"q.jL_Vp",q.jL_Vdp,"q.jL_Vdp",eps);

    UnitTests.checkDoubleRel(q.jL_qP,"q.jL_qP",q.jL_qPk,"q.jL_qPk",eps);
    UnitTests.checkDoubleRel(q.jL_qPp,"q.jL_qPp",q.jL_qPpk,"q.jL_qPpk",eps);


    UnitTests.checkDoubleRel(q.p2L.lam,"q.p2L.lam",q.dp2I.lamm,"q.dp2I.lam",eps);
    UnitTests.checkDoubleArrayRel(q.p2L.Xx,"q.p2L.Xx",q.dp2I.Xx,"q.dp2I.Xx",eps);
    UnitTests.checkDoubleArrayRel(q.p2L.psi,"q.p2L.psi",EVXData.getMSign(q.dp2I.psi,q.p2L.psi[q.p2L.psi.length-1]),"q.dp2I.psi",eps);
    UnitTests.checkDoubleRel(q.p2L.fXx,"q.p2L.fXx",q.dp2I.fXx,"q.dp2I.fXx",eps);
    UnitTests.checkDoubleRel(q.p2L_jX,"q.p2L_jX",q.dp2I.jX,"q.dp2I.jX",eps);
    if(Math.abs(q.dp2I.a)>1e-10 && (q.isEdgeAtRight()?(q.getXTestEnd(neq)<=q.dp2I.fXx && q.dp2I.fXx<=q.getX0()):(q.getXTestEnd(neq)>=q.dp2I.fXx && q.dp2I.fXx>=q.getX0())))
    {
	System.err.println("q.dp2I.fXx="+q.dp2I.fXx+" a="+q.dp2I.a+" b="+q.dp2I.b+" c="+q.dp2I.c);
	UnitTests.checkDoubleRel(q.dp2I.a,"q.dp2I.a",q.dp2I.at,"q.dp2I.at",eps);
	UnitTests.checkDoubleRel(q.dp2I.b,"q.dp2I.b",q.dp2I.bt,"q.dp2I.bt",eps);
	UnitTests.checkDoubleRel(q.dp2I.c,"q.dp2I.c",q.dp2I.ct,"q.dp2I.ct",eps);
    }
    UnitTests.checkDoubleRel(q.jL_x_i_low,"q.jL_x_i_low",q.jL.fXx,"q.jL.fXx",eps);
    UnitTests.checkDoubleRel(q.jL.getWX(),"q.jL.getWX()",q.jTrvs.mu,"q.jTrvs.mu",eps);
    UnitTests.checkDoubleRel(q.jL.fXx,"q.jL.fXx",q.jTrvs.xmu,"q.jTrvs.xmu",eps);

    UnitTests.checkDoubleRel(q.wmudp.mu,"q.wmudp.mu",q.jL.getWX(),"q.jL.getWX()",eps);
    UnitTests.checkDoubleRel(q.wmudp.dp*q.wmudp.mu,"q.wmudp.dp*q.wmudp.mu",q.jL_re_dp,"q.jL_re_dp",eps);
    UnitTests.checkDoubleRel(q.wmudp.lmu,"q.wmudp.lmu",q.jL.getWX(),"q.jL.getWX()",eps);

    q.wqmuOp.test_Self(eps);

    q.wqmuPp.test_Self(eps);
    //UnitTests.checkDoubleRel(q.wqmuPp.dP2_psi,"q.wqmuPp.dP2_psi",q.Pmu.dP2_psi,"q.Pmu.dP2_psi",eps);
    //UnitTests.checkDoubleRel(q.wqmuPp.lamP_psi,"q.wqmuPp.lamP_psi",q.Pmu.lamP_psi,"q.Pmu.lamP_psi",eps);
    //UnitTests.checkDoubleRel(q.wqmuPp.dpsi_mux_dpsi,"q.wqmuPp.dpsi_mux_dpsi",q.Pmu.dpsi_mux_dpsi,"q.Pmu.dpsi_mux_dpsi",eps);

    q.wqmuOs.test_Self(eps);
    q.wqmuPs.test_Self(eps);

    q.Pmu.test_Self(eps);
    q.RPmu.test_Self(eps);
    q.Pv.test_Self(eps);
    q.Pt.test_Self(eps);

    q.wmudp.test_Self(eps);
    q.wmup.test_Self(eps);
    q.wmup_sam.test_Self(eps);
    q.wmuDp_sam.test_Self(eps);
    q.wmuDs_sam.test_Self(eps);
    q.wmuDrs_sam.test_Self(eps);
    q.wmuDpW_sam.test_Self(eps);
    q.wmuDsW_sam.test_Self(eps);
    q.wmuDrsW_sam.test_Self(eps);

    //UnitTests.checkDoubleRel(q.pe_psiIL.p_td,"q.pe_psiIL.p_td",q.wmup.p_td,"q.wmup.p_td",eps);
    UnitTests.checkDoubleRel(q.pe_psiIL.fXx,"q.pe_psiIL.fXx",q.wmup.jL.fXx,"q.wmup.jL.fXx",eps);

    UnitTests.checkDoubleRel(q.wqmuOp.dP2_psi,"q.wqmuOp.dP2_psi",q.wo1p.dP2_psi,"q.wo1p.dP2_psi",eps);
    UnitTests.checkDoubleRel(q.wqmuOp.dpsi_mux_dpsi,"q.wqmuOp.dpsi_mux_dpsi",q.wo1p.dpsi_mux_dpsi,"q.wo1p.dpsi_mux_dpsi",eps);

    q.wo0p.test_Self(eps);
    q.wo1p.test_Self(eps);
    q.wo2p.test_Self(eps);
    q.wo0s.test_Self(eps);
    q.wo1s.test_Self(eps);
    q.wo2s.test_Self(eps);


    q.epit_psiIL.test_Self(eps);
    {
     UnitTests.checkDoubleRel(q.epit_psiIL.g,"q.epit_psiIL.g",Math.pow(q.M.B.getD0(q.IT.psiL,q.getX0()),2)/2,"psiL(x0)^2/2",eps);
     UnitTests.checkDoubleRel(q.epit_psiIL.dPf,"q.epit_psiIL.dPf",q.wqmuP.dPpsiL,"q.wqmuP.dPpsiL",eps);
     UnitTests.checkDoubleRel(q.epit_psiIL.Pf,"q.epit_psiIL.Pf",q.wqmuP.P_psiL,"q.wqmuP.P_psiL",eps);
     UnitTests.checkDoubleRel(q.epit_psiIL.sigma2_Pf,"q.epit_psiIL.sigma2_Pf",q.wqmuP.sigma2_P_psiL,"q.wqmuP.sigma2_P_psiL",eps);
     UnitTests.checkDoubleRel(q.epit_psiIL.g,"q.epit_psiIL.g",q.wqmuP.g_psiL,"q.wqmuP.g_psiL",eps);
     UnitTests.checkDoubleRel(q.epit_psiIL.sigma2_g,"q.epit_psiIL.sigma2_g",q.wqmuP.sigma2_g_psiL,"q.wqmuP.sigma2_g_psiL",eps);
    }
    q.epit_psiIH.test_Self(eps);
    {
     UnitTests.checkDoubleRel(q.epit_psiIH.g,"q.epit_psiIH.g",Math.pow(q.M.B.getD0(q.IT.psiH,q.getX0()),2)/2,"psiH(x0)^2/2",eps);
     UnitTests.checkDoubleRel(q.epit_psiIH.dPf,"q.epit_psiIH.dPf",q.wqmuP.dPpsiH,"q.wqmuP.dPpsiH",eps);
     UnitTests.checkDoubleRel(q.epit_psiIH.Pf,"q.epit_psiIH.Pf",q.wqmuP.P_psiH,"q.wqmuP.P_psiH",eps);
     UnitTests.checkDoubleRel(q.epit_psiIH.sigma2_Pf,"q.epit_psiIH.sigma2_Pf",q.wqmuP.sigma2_P_psiH,"q.wqmuP.sigma2_P_psiH",eps);
     UnitTests.checkDoubleRel(q.epit_psiIH.g,"q.epit_psiIH.g",q.wqmuP.g_psiH,"q.wqmuP.g_psiH",eps);
     UnitTests.checkDoubleRel(q.epit_psiIH.sigma2_g,"q.epit_psiIH.sigma2_g",q.wqmuP.sigma2_g_psiH,"q.wqmuP.sigma2_g_psiH",eps);
    }

    q.eptt_psiIL.test_Self(eps);
    q.eptt_psiIH.test_Self(eps);
    q.epii_psiIL.test_Self(eps);
    q.epii_psiIH.test_Self(eps);
    q.epti_psiIL.test_Self(eps);
    q.epti_psiIH.test_Self(eps);

    q.Ekpit_psiIL.test_Self(r,eps);
    q.Rekpit_psiIL.test_Self(r,eps);
    q.ekpit_psiIL.test_Self(r,eps);
    {
	UnitTests.checkDoubleRel(q.ekpit_psiIL.fXx,"q.ekpit_psiIL.fXx",q.jL.fXx,"q.jL.fXx",eps);
    }
    q.ekpit_psiIH.test_Self(r,eps);
    q.ekptt_psiIL.test_Self(r,eps);
    q.ekptt_psiIH.test_Self(r,eps);
    q.ekpii_psiIL.test_Self(r,eps);
    q.ekpii_psiIH.test_Self(r,eps);
    q.ekpti_psiIL.test_Self(r,eps);
    q.ekpti_psiIH.test_Self(r,eps);

    q.sdeit_pi_L.test_Self(eps);
    q.eit_pi_L.test_Self(eps);
    q.eit_pi_H.test_Self(eps);
    q.eit_pt_L.test_Self(eps);
    q.eit_pt_H.test_Self(eps);

    q.sdbit_pi_L.test_Self(r,eps);
    q.bit_pi_L.test_Self(r,eps);
    q.bit_pi_H.test_Self(r,eps);
    q.bit_pt_L.test_Self(r,eps);
    q.bit_pt_H.test_Self(r,eps);
    q.biti_pi_L.test_Self(r,eps);
    q.biti_pi_H.test_Self(r,eps);
    q.biti_pt_L.test_Self(r,eps);
    q.biti_pt_H.test_Self(r,eps);

    q.cit_pi_L.test_Self(r,eps);
    q.cit_pi_H.test_Self(r,eps);

    q.pt_psiIL.test_Self(eps);
    {
	final DkQVM<DTYPE>.WmuPsi qp=q.new WmuPsi(q.pt_psiIL.psi.length,q.pt_psiIL.simom,q.pt_psiIL.stmom,q.pt_psiIL.spmomx,null);
	UnitTests.checkDoubleRel(q.pt_psiIL.Pf,"q.pt_psiIL.Pf",qp.Pf,"qp.Pf",eps);
	UnitTests.checkDoubleRel(q.pt_psiIL.Pfd,"q.pt_psiIL.Pfd",qp.Pfd,"qp.Pfd",eps);
    }
    q.pt_psiIH.test_Self(eps);

    q.dp_psiIL.test_Self(eps);
    q.dp_psiIH.test_Self(eps);

    q.p_psiIL.test_Self(eps);
    q.p_psiIH.test_Self(eps);
    q.j_psiIL.test_Self(eps);
    q.j_psiIH.test_Self(eps);

    q.pe_psiIL.test_Self(eps);
    q.pe_psiIL.test_DSelf(eps,r);
    q.pe_psiIH.test_Self(eps);
    q.pe_psiIH.test_DSelf(eps,r);
    q.je_psiIL.test_Self(eps);
    q.je_psiIH.test_Self(eps);

    //q.p_IL_psiIH.test_Self(eps);


    q.p_lamL_psiH.test_Self(eps);
    q.p_lamH_psiL.test_Self(eps);
    q.dp_lamL_psiH.test_Self(eps);
    q.dp_lamH_psiL.test_Self(eps);
    q.p_mi0_psiH.test_Self(eps);

    q.pik_sam.test_Self(eps);
    q.pik.test_Self(eps);
    q.ptk_sam.test_Self(eps);
    q.ptk.test_Self(eps);

    UnitTests.checkDoubleRel(q.pik.dPtL,"q.pik.dPtL",q.edp_psiIDH.dPf,"q.edp_psiIDH.dPf",eps);
    UnitTests.checkDoubleRel(q.pik.dPtH,"q.pik.dPtH",q.edp_psiIDL.dPf,"q.edp_psiIDL.dPf",eps);

    q.P2it_E.test_Self(eps);
    q.P2it_XX.test_Self(eps);
    //UnitTests.checkDoubleArrayRel(q.P2it_E.varP,"q.P2it_E.varP",q.P2it_XX.varP,"q.P2it_XX.varP",eps);

    q.P2tt_E.test_Self(eps);
    q.P2tt_XX.test_Self(eps);
    //UnitTests.checkDoubleArrayRel(q.P2tt_E.varP,"q.P2tt_E.varP",q.P2tt_XX.varP,"q.P2tt_XX.varP",eps);

    q.P2ti_E.test_Self(eps);
    q.P2ti_XX.test_Self(eps);
    //UnitTests.checkDoubleArrayRel(q.P2ti_E.varP,"q.P2ti_E.varP",q.P2ti_XX.varP,"q.P2ti_XX.varP",eps);

    q.Pd2ti_E.test_Self(eps);
    q.Pd2ti_XX.test_Self(eps);
    //UnitTests.checkDoubleArrayRel(q.Pd2ti_E.varP,"q.Pd2ti_E.varP",q.Pd2ti_XX.varP,"q.Pd2ti_XX.varP",eps);

    EVXData.matchEVXData(q.pptt_E.P2x,q.vt.VT,eps);
    q.vt.test_Self(eps);
    q.vt_sam.test_Self(eps);
    q.vtD.test_Self(eps);
    q.vtmu.test_Self(eps);

    q.mL_vtpv.test_Self(eps);
    q.mL_vtpt.test_Self(eps);
    q.mL_vtSvs.test_Self(eps);
    q.mL_Svtvs.test_Self(eps);
    q.mL_vtdp.test_Self(eps);
    q.mH_vtpv.test_Self(eps);
    q.mH_vtpt.test_Self(eps);
    q.mH_vtSvs.test_Self(eps);
    q.mH_Svtvs.test_Self(eps);
    q.mH_vtdp.test_Self(eps);

    UnitTests.checkDoubleArrayRel(q.getED(neq),"getED(neq)",q.VOLATILITY_EDPsi.getVolatility(Double.NaN).getDMatrix(neq),"getDMatrix",eps);


    if(!Double.isNaN(q.wmupVT_sam.xD0)) UnitTests.checkDoubleRel(q.vtmu.x0P,"q.vtmu.x0P",q.wmupVT_sam.xD0,"q.wmupVT_sam.xD0",eps);
    if(!Double.isNaN(q.wmupVT_sam.xXD0)) UnitTests.checkDoubleRel(q.vtmu.x1P,"q.vtmu.x1P",q.wmupVT_sam.xXD0,"q.wmupVT_sam.xXD0",eps);

    System.err.println(name+":DkQVMTest:test_Common(): test OK neq="+neq+" eps="+eps);
}

void test_Fq()
{
    final DkQVM<DTYPE> q=getDkQVM();

    q.jL_sam.test_getFq(q.sxmom,eps);
    q.jL_sam.test_getFq(q.sxsmom,eps);

    final int ntests=1000;
    q.jL_sam.test_getFXX(ntests,r,eps);
    q.jL_sam.test_getFabXX(ntests,r,eps);
    q.jL_sam.test_getFxFx2(ntests,r,eps);
    q.jL_sam.test_getXMM(100,r,eps);
    q.jL_sam.test_getFXabc(ntests,r,eps);

    final double [][] f=q.jL_sam.getFXX(q.sxmom,DkQVM.WABpsi.MT_MU,false);
    UnitTests.checkDoubleArrayRel(f[0],"f[0]",f[1],"f[1]",eps);
    final double [][] f0=q.jL_sam.getFXX(q.sxmom,DkQVM.WABpsi.MT_MU,true);
    UnitTests.checkDoubleArrayRel(f0[0],"f0[0]",f[0],"f[0]",eps);
    UnitTests.checkDoubleArrayRel(f0[1],"f0[1]",new double [f0[1].length],"0",eps);
    UnitTests.checkDoubleArrayRel(f0[2],"f0[2]",new double [f0[2].length],"0",eps);
    UnitTests.checkDoubleArrayRel(f0[3],"f0[3]",new double [f0[3].length],"0",eps);


    q.jL.test_getFXX(ntests,r,eps);
    q.jH.test_getFXX(ntests,r,eps);
    q.jL.test_getFabXX(ntests,r,eps);
    q.jH.test_getFabXX(ntests,r,eps);

    System.err.println(name+":DkQVMTest:test_Fq(): test OK neq="+neq+" eps="+eps);
}

void test_DjQVM()
{
    final DjQVM<?> s=getDjQVM();
    final DkQVM<DTYPE> q=getDkQVM();

    EVXData.matchEVXData(q.IT,s.I,true,true,eps);
    EVXData.matchEVXData(q.tsI,s.tsI,eps);
    UnitTests.checkDoubleArrayRel(q.L.Xx,"q.L.Xx",s.Xx,"s.Xx",eps);
    UnitTests.checkDoubleArrayRel(q.L.W,"q.L.W",s.Wx,"s.Wx",eps);
    UnitTests.checkDoubleArrayRel(q.L.Wms,"q.L.Wms",s.Wms,"s.Wms",eps);
    UnitTests.checkDoubleArrayRel(q.L.Wdp,"q.L.Wdp",s.Wdp,"s.Wdp",eps);

    EVXData.matchEVXData(q.RIT,s.RI,eps);
    UnitTests.checkDoubleRel(q.L.fXx,"q.L.fXx",s.fXx,"s.fXx",eps);
    UnitTests.checkDoubleRel(q.Rwmup.jL.fXx,"q.Rwmup.jL.fXx",s.xRFIL,"s.xRFIL",eps);

    q.test_ED(neq,r,eps);

    System.err.println(name+":DkQVMTest:test_DjQVM(): test OK neq="+neq+" eps="+eps);
}


void test_DynVarMu()
{
    final DkQVM<DTYPE> q=getDkQVM();
    final int n=neq;
    final double [] QQt=q.M.B.getQQMatr(n,q.wmuPi.stmom),QQi=q.M.B.getQQMatr(n,q.wmuPi.simom),QQpi=q.M.B.getQQMatr(n,q.wmuPi.spimom);
    final DynVarMu s=DynVarMu.get_DynVarMu(n,QQi,QQt,QQpi,q.M,true);
    q.wmuPi.match_DynVarMu(s,eps);
    System.err.println(name+":DkQVMTest:test_DynVarMu(): test OK n="+n+" eps="+eps);
}

void test_BiQVM()
{
    final BiQVM<?> s=getBiQVM();
    final DkQVM<DTYPE> q=getDkQVM();

    EVXData.matchEVXData(q.IT,s.I,true,true,eps);
    EVXData.matchEVXData(q.p2I,s.dp2I,eps);
    UnitTests.checkDoubleRel(q.evst_psiIL.Pf,"q.evst_psiIL.Pf",s.psiL_vs_psiL,"s.psiL_vs_psiL",eps);
    UnitTests.checkDoubleRel(q.evst_psiIH.Pf,"q.evst_psiIH.Pf",s.psiH_vs_psiH,"s.psiH_vs_psiH",eps);
    UnitTests.checkDoubleRel(q.pe_psiIL.p_t,"q.pe_psiIL.p_t",s.ptL-s.getPOffset(),"s.ptL",eps);
    UnitTests.checkDoubleRel(q.pe_psiIL.p_i,"q.pe_psiIL.p_i",s.pvL-s.getPOffset(),"s.pvL",eps);
    UnitTests.checkDoubleRel(q.pe_psiIH.p_t,"q.pe_psiIH.p_t",s.ptH-s.getPOffset(),"s.ptH",eps);
    UnitTests.checkDoubleRel(q.pe_psiIH.p_i,"q.pe_psiIH.p_i",s.pvH-s.getPOffset(),"s.pvH",eps);

    EVXData.matchEVXData(q.Vt.Px,s.Vt,eps);
    EVXData.matchEVXData(q.Vty.Px,s.Vt,eps);
    EVXData.matchEVXData(q.P2t.Px,s.p2t,eps);
    EVXData.matchEVXData(q.P2v.Px,s.p2v,eps);
    EVXData.matchEVXData(q.P2ti_XX.P2x,s.p2ti,eps);
    EVXData.matchEVXData(q.Pd2ti_XX.P2x,s.P2i,eps);
    EVXData.matchEVXData(q.Pd2tV_XX.P2x,s.P2V,eps);
    EVXData.matchEVXData(q.vt.VT,s.VT,eps);

    UnitTests.checkDoubleRel(q.Pvte_i.P_v0,"q.Pvte_i.P_v0",s.Ct.s0/s.Vt.s0,"s.Ct.s0/s.Vt.s0",eps);

    System.err.println(name+":DkQVMTest:test_BiQVM(): test OK neq="+neq+" eps="+eps);
}


void test_EQVM()
{
    final EQVM<?> s=getEQVM();
    final DkQVM<DTYPE> q=getDkQVM();

    UnitTests.checkDoubleRel(q.acorr_dp_i.osp,"q.acorr_dp_i.osp",-s.osp,"-s.osp",eps);

    System.err.println(name+":DkQVMTest:test_EQVM(): test OK neq="+neq+" eps="+eps);
}


void test_FMoments(final int n){}


void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":DkQVMTest:test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,neq);
    test_Common();
    test_Fq();
    test_DjQVM();
    test_BiQVM();
    test_EQVM();
    test_FMoments(neq);
    test_DynVarMu();

    System.err.println(name+":DkQVMTest:test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":DkQVMTest: starting tests; ver=$Id: DkQVM.java,v 1.677 2022/11/19 11:08:57 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=2;neq<=neq_max;neq++)
    {
	for(int i=0;i<119;i++)
	{
	    test_Calculations(1950+r.nextInt(850),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":DkQVMTest: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: DkQVM.java,v 1.677 2022/11/19 11:08:57 mal Exp $");
}
} // END OF class DkQVMTest

}// END OF class DkQVM
